#include <sys/types.h>  
#include <sys/stat.h>  
#include <stdio.h>  
#define __USE_BSD
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>  
#include <time.h>
#include "prefix.h"
#include "pline.h"
#include "main.h"
#include "ioparser.h"
#include "fortrantypes.h"
#include "tparser.h"
#include "tools.h"
#include "helpbp.h"
#include "merror.h"
#include "allocread.h"


#define PS_BLACK 0
#define PS_RED     1
#define PS_GREEN   2
#define PS_BLUE    3
#define PS_ORANGE  4
#define PS_MAGENTA 5
#define PS_GRAY    6
#define PS_turquoise    7
#define PS_WHITE   8


static const double  RGBColors[][3]={
  /* black */
  {0.0,0.0,0.0},
  /* red */
  {0.8,0.0,0.0},
  /* green */
  {0.0,0.7,0.0},
  /* blue */
  {0.0,0.0,1.0},
  /* orange */
  {0.9,0.9,0.0},
  /* magenta */
  {1.0,0.0,1.0},
  /* gray */
  {0.7,0.7,0.7},
  /* turquoise */
  {0.0,0.7,0.7},
  /* white */
  {1.0,1.0,1.0}
};
const long     RGBMaxColors=sizeof(RGBColors)/sizeof(double)/3;
const char *PSColorNames[]={
  "black",
  "red",
  "green",
  "blue",
  "orange",
  "magenta",
  "gray",
  "turquoise",
  "white"
};

typedef struct format{
  double title_scale;
  double title_subscript_scale;
  double yaxis_scale;
  double yaxis_subscript_scale;
  double xticmark_scale;
  double xtic_subscript_scale;
  double yticmark_scale;
  double ytic_subscript_scale;
  double legend_scale;
  double legend_subscript_scale;
  long display_yticmarks;
  long yticmarkdigits;
  long display_legendframe;
  double vertspacing;
  double framelinewidth;
  double ticlinewidth;
  double yaxis_offsetx;
  double yaxis_offsety;
  double ylabel_offsetx;
  double xlabel_offsety;
  double frameleft;
  double frameright;
  double frametop;
  double framebottom;
}tsFORMAT;


typedef struct tsbanddata{
  double ef;
  long nk;
  long nband;
  long mspin;
  double *kdist;
  double *erg;
  double emin,emax;
} tsBANDDATA;


#include "versions.h"


const char BPINPUTFILE[BUFFER_MAX_STRING+1]="=.bp";
char FEDITFILE[BUFFER_MAX_STRING+1]="+fedit";
char TMPDIR[BUFFER_MAX_STRING+1]    ="+tmp";
const char  *DEFAULTHELPFILE="+fedithelp";
char EXECNAME[BUFFER_MAX_STRING+1]="bandplot";
char FULLEXECNAME[BUFFER_MAX_STRING+1]="bandplot";

char *BAND_PLOT_TYPES[]={"bands","bandweights" /* ,"bloch spectral density" */ };
const FPLOINTEGER BAND_TYPE_BANDS=1;
const FPLOINTEGER BAND_TYPE_BANDWEIGHTS=2;
const FPLOINTEGER BAND_TYPE_AKBL=3;
#define STRING_LEN 1000

/* global variable parser_control for io */
PARSER_CONTROLL bp_pc;

/* ***********************************************************************
 * forward prototypes
 * ***********************************************************************
 */
static long read_band_file(const char *fname,tsBANDDATA *bd);
/* static long read_BW_header(const char *file); */
static long varexists(PARSER_CONTROLL *pc,const char*name);
static void scanopts(long argc,char **argv,long *gv,long *intmp,long *doupdate);

static void ps_plot_bands(const tsBANDDATA *bd,FILE *FHO
			  ,const VIEWPORT *vp,const WORLD *world,long coltyp
			  );
void create_bpfedit(const char *fname);
void create_bpinfile(const char *fname,FPLOINTEGER band_plot_type
		     ,const char *title
		     ,const char * yaxis_title
		     ,FPLODOUBLEPRECISION emin
		     ,FPLODOUBLEPRECISION emax
		     ,FPLODOUBLEPRECISION plot_radius
		     ,FPLOINTEGER nstate_to_plot
		     ,const FPLOINTEGER *state_to_plot
		     ,FPLOLOGICAL print_labels
		     ,FPLOLOGICAL display_legend
		     ,FPLODOUBLEPRECISION legend_x
		     ,FPLODOUBLEPRECISION legend_y
		     ,long nfile,char **files
		     ,char **legstr
		     ,const char *outfile
		     ,const char *viewer
		     ,const tsFORMAT *format
		     ,long init
		     ,FPLOLOGICAL spin_legend
		     ,FPLOLOGICAL use_own_legend_string
		     ,const char *own_legend_string
		     ,const long *bw_colors
		     ,const char *bwfile
		     ,const char *bwbandfile
		     );
void read_bpinfile(const char *fname,FPLOINTEGER *band_plot_type
		   ,char *title,long title_len
		   ,char * yaxis_title,long yaxis_len
		   ,FPLODOUBLEPRECISION *emin
		   ,FPLODOUBLEPRECISION *emax
		   ,FPLODOUBLEPRECISION *plot_radius
		   ,FPLOINTEGER *nstate_to_plot
		   ,FPLOINTEGER **state_to_plot
		   ,FPLOLOGICAL *print_labels
		   ,FPLOLOGICAL *display_legend
		   ,FPLODOUBLEPRECISION *legend_x
		   ,FPLODOUBLEPRECISION *legend_y
		   ,long *nfile,char ***files
		   ,char ***legstr
		   ,char *outfile,long outfile_len
		   ,char *viewer,long viewer_len
		   ,tsFORMAT *format
		   ,FPLOLOGICAL *spin_legend
		   ,FPLOLOGICAL *use_own_legend_string
		   ,char *own_legend_string,long own_len
		   ,long *bw_colors
		   ,char *bwfile,long bw_len
		   ,char *bwbandfile
		   ,long doupdate);


void plotweights(const char * file,const char *underlaybandfile
		 ,VIEWPORT vp,WORLD world,FPLODOUBLEPRECISION plot_radius
		 ,FPLOINTEGER nstate_to_plot
		 ,const FPLOINTEGER *state_to_plot
		 ,FPLOLOGICAL print_labels,char *title,char *yaxis_title
		 ,FPLOLOGICAL display_legend
		 ,FPLODOUBLEPRECISION legend_x,FPLODOUBLEPRECISION legend_y
		 ,char **legstr,const char *outfile
		 ,const tsFORMAT *format
		 ,FPLOLOGICAL spin_legend
		 ,FPLOLOGICAL use_own_legend_string
		 ,const char *own_legend_string
		 ,long *bw_colors);
void plotband(VIEWPORT vp,WORLD world
	      ,FPLOLOGICAL print_labels,char *title,char *yaxis_title
	      ,FPLOLOGICAL display_legend
	      ,FPLODOUBLEPRECISION legend_x
	      ,FPLODOUBLEPRECISION legend_y,long nfile,char **files
	      ,char **legstr,const char *outfile
	      ,const tsFORMAT *format
	      ,FPLOLOGICAL spin_legend);
void plotakbl(VIEWPORT vp,WORLD world,FPLODOUBLEPRECISION plot_radius
	      ,FPLOLOGICAL print_labels,char *title,char *yaxis_title
	      ,FPLOLOGICAL display_legend
	      ,FPLODOUBLEPRECISION legend_x
	      ,FPLODOUBLEPRECISION legend_y,long nfile,char **files
	      ,char **legstr,const char *outfile
	      ,const tsFORMAT *format
	      ,FPLOLOGICAL spin_legend);



const char *  
#include "../mainversion.h"
;
const char *  
#include "../subversion.h"
;
const char *  
#include "../release.h"
;




long wpx0,wpy0;


/* ***********************************************************************
 * main
 * ***********************************************************************
 */
int main(int argc,char **argv){
  long wpx,wpy,i,nfile;
  FPLOINTEGER nstate_to_plot;
  FPLOINTEGER *state_to_plot;
  FPLOINTEGER band_plot_type;
  FPLODOUBLEPRECISION plot_radius,legend_x,legend_y;
  FPLODOUBLEPRECISION emin,emax;
  FPLOLOGICAL print_labels,display_legend,spin_legend,use_own_legend_string;
  VIEWPORT vp;
  WORLD world;
  int idum;
  long bw_colors[2];
  char **files,**legstr;
  char yaxis_title[STRING_LEN+1]="Energy  $~e$_n$.(k)  [eV]";
  char title[STRING_LEN+1]="Compound X$_2$.YZ$^$~m$_0";
  char outfile[STRING_LEN+1]="band";
  char own_legend_string[STRING_LEN+1]="";
  char viewer[STRING_LEN+1]="gv";
  char fname[STRING_LEN+1];
  char buf[STRING_LEN+1];
  char bwfile[STRING_LEN+1]="+bweights";
  char bwbandfile[STRING_LEN+1]="+band";
  tsFORMAT bwformat;
  long intmp=0;
  long gv=0;
  long doupdate=0;

  /* example code
  char *pbuffer;
  long lenpbuffer,ret;
  FILE *FH;

  FH=fopen("tin","r");

  lenpbuffer=0;
  pbuffer=NULL;
  while( 1 ) {
    if( (ret=allocreadbuf(FH,&pbuffer,&lenpbuffer)) == 0 ){
      break;
    }
    printf("%s",pbuffer);
  }
  fclose(FH);
  exit(0);
  */


  (void)strcpy(EXECNAME,argv[0]);
  (void)sprintf(FULLEXECNAME,"%s (%s-%s)",EXECNAME,FPLOMAINVERSION,FPLORELEASE);

  scanopts(argc,argv,&gv,&intmp,&doupdate);


  (&bwformat)->title_scale=1.;
  (&bwformat)->title_subscript_scale=0.6;
  (&bwformat)->yaxis_scale=1.;
  (&bwformat)->yaxis_subscript_scale=0.6;
  (&bwformat)->xticmark_scale=1.;
  (&bwformat)->xtic_subscript_scale=0.6;
  (&bwformat)->yticmark_scale=1.;
  (&bwformat)->ytic_subscript_scale=0.6;
  (&bwformat)->legend_scale=1.;
  (&bwformat)->legend_subscript_scale=0.6;

  (&bwformat)->display_yticmarks=1;
  (&bwformat)->yticmarkdigits=-1;
  (&bwformat)->display_legendframe=1;
  (&bwformat)->vertspacing=1.5;
  (&bwformat)->framelinewidth=1;
  (&bwformat)->ticlinewidth=1;
  (&bwformat)->yaxis_offsetx=0;
  (&bwformat)->yaxis_offsety=0;
  (&bwformat)->ylabel_offsetx=0;
  (&bwformat)->xlabel_offsety=0;

  /*  frame offset */
  (&bwformat)->frametop=0.1678;
  (&bwformat)->framebottom=0.1678;
  (&bwformat)->frameleft=0.1781;
  (&bwformat)->frameright=0.1188;

  world.e0=emin=1.0;
  world.e1=emax=-1.0;
    
  world.k0=0;
  world.k1=1;

  bw_colors[0]=PS_BLACK;
  bw_colors[1]=PS_GRAY;

  plot_radius=6.0;
  nstate_to_plot=1;
  state_to_plot=(FPLOINTEGER*)malloc(sizeof(FPLOINTEGER)*nstate_to_plot);
  state_to_plot[0]=1;
  spin_legend=L_TRUE;
  print_labels=L_TRUE;
  use_own_legend_string=L_FALSE;
  band_plot_type=BAND_TYPE_BANDS;
  display_legend=L_TRUE;
  legend_x=0.80;
  legend_y=-0.05;
  helpbp();
  create_bpfedit(FEDITFILE);

  nfile=1;
  files=(char**)malloc(sizeof(char*)*nfile);
  legstr=(char**)malloc(sizeof(char*)*nfile);
  files[0]=(char*)malloc(sizeof(char)*(STRING_LEN+1));
  (void)strcpy(files[0],"+band");
  legstr[0]=(char*)malloc(sizeof(char)*(STRING_LEN+1));
  (void)strcpy(legstr[0],"legend1");

  create_bpinfile(BPINPUTFILE,band_plot_type,title,yaxis_title,emin
		  ,emax,plot_radius
		  ,nstate_to_plot,state_to_plot,print_labels
		  ,display_legend,legend_x,legend_y
		  ,nfile,files,legstr		  
		  ,outfile,viewer
		  ,&bwformat
		  ,1L,spin_legend,use_own_legend_string
		  ,own_legend_string,bw_colors,bwfile,bwbandfile);
  
  free(files[0]);
  free(legstr[0]);
  free(files);
  free(legstr);
  free(state_to_plot);

  read_bpinfile(BPINPUTFILE,&band_plot_type,title,STRING_LEN
		,yaxis_title,STRING_LEN
		,&emin
		,&emax,&plot_radius
		,&nstate_to_plot,&state_to_plot,&print_labels
		,&display_legend,&legend_x,&legend_y
		,&nfile,&files,&legstr		  
		,outfile,STRING_LEN
		,viewer,STRING_LEN
		,&bwformat
		,&spin_legend,&use_own_legend_string
		,own_legend_string,STRING_LEN
		,bw_colors,bwfile,STRING_LEN
		,bwbandfile
		,doupdate);
  world.e0=emin;
  world.e1=emax;
  /*  printf("<"FPLOINTEGERFORMAT",%s>\n",band_plot_type,BAND_PLOT_TYPES[band_plot_type-1]);
  printf("<%s>\n",title);
  printf("<%s>\n",yaxis_title);
  printf("<%g>\n",world.e0);
  printf("<%g>\n",world.e1);
  printf("<%g>\n",plot_radius);
  printf("<"FPLOINTEGERFORMAT">\n",state_to_plot);
  printf("<%c>\n",(print_labels==L_TRUE?'t':'f'));
  printf("<%c>\n",(display_legend==L_TRUE?'t':'f'));
  printf("<%g>\n",legend_x);
  printf("<%g>\n",legend_y);
  for(i=0;i<nfile;i++){
    printf("<%s:%s>\n",files[i],legstr[i]);
  }
  */

  if(intmp){
    idum=chdir("..");
  }
  
  idum=idum;
  
  /* start viewer on created file */
  if(gv){
    if(band_plot_type == BAND_TYPE_BANDS){
      (void)sprintf(fname,"%s.ps",outfile);
    }else if(band_plot_type == BAND_TYPE_BANDWEIGHTS){
      (void)sprintf(fname,"%s_w.ps",outfile);
    }else if(band_plot_type == BAND_TYPE_AKBL){
      (void)sprintf(fname,"%s_akbl_1.ps",outfile);
    }
    (void)sprintf(buf,"%s %s > /dev/null  2>&1  &",viewer,fname);
    if(pathExists(fname)==1){
      idum=system(buf);      
      (void)printf("\nviewer %s started\n",viewer);
      
    }else{
      (void)printf("File '%s' does not exists\n"
	     "May be you have to created it first!\n",fname);
    }
    if(intmp){
      idum=chdir(TMPDIR);
    }
    
    exit(1);
  }  




  /*   printf("%10f%10f\n",world.e0,world.e1); */
  vp.scale=0.24;
  /* viewport dimension*/
  wpx0=596;
  wpy0=842;


  wpx=wpx0/vp.scale;
  wpy=wpy0/vp.scale;
  /* frame in viewport coordinates */
  vp.x0=wpx*(&bwformat)->frametop;
  vp.x1=wpx*(1-(&bwformat)->framebottom);
  vp.y0=wpy*(&bwformat)->frameleft;
  vp.y1=wpy*(1-(&bwformat)->frameright);
  

  if(band_plot_type==1){
    plotband(vp,world,print_labels,title,yaxis_title,display_legend
	     ,legend_x,legend_y,nfile,files,legstr,outfile,&bwformat
	     ,spin_legend);
  }else if(band_plot_type==2){


    plotweights(bwfile,bwbandfile
		,vp,world,plot_radius,nstate_to_plot,state_to_plot
		,print_labels,title
		,yaxis_title,display_legend
		,legend_x,legend_y,legstr,outfile,(&bwformat)
		,spin_legend,use_own_legend_string,own_legend_string
		,bw_colors);
  }else if(band_plot_type==3){
    plotakbl(vp,world,plot_radius,print_labels,title,yaxis_title,display_legend
	     ,legend_x,legend_y,nfile,files,legstr,outfile,&bwformat
	     ,spin_legend);
  }



  for(i=0;i<nfile;i++){
    free(files[i]);
    free(legstr[i]);
  }
  free(files);
  free(legstr);
  free(state_to_plot);


  if(intmp){
    idum=chdir(TMPDIR);
  }

  return 0;
  
}

/* ***********************************************************************
 *       bandweights plot
 * ***********************************************************************
 */
void plotweights(const char * file,const char *underlaybandfile
		 ,VIEWPORT vp,WORLD world,FPLODOUBLEPRECISION plot_radius
		 ,FPLOINTEGER nstate_to_plot
		 ,const FPLOINTEGER *state_to_plot
		 ,FPLOLOGICAL print_labels,char *title,char *yaxis_title
		 ,FPLOLOGICAL display_legend
		 ,FPLODOUBLEPRECISION legend_x
		 ,FPLODOUBLEPRECISION legend_y
		 ,char **legstr,const char *outfile
		 ,const tsFORMAT *format
		 ,FPLOLOGICAL spin_legend
		 ,FPLOLOGICAL use_own_legend_string
		 ,const char *own_legend_string
		 ,long *bw_colors)
{
  tsBANDDATA bd;
  FILE *FHI,*FH;
  long n,idum,nk,nband,mspin,i,ik,ms,in,nb,i1,j1,i2,j2,nleg,coltyp,is,filled;
  double ef,emin,emax,*x,*f1,k,e,wei,*wr;
  double radius=plot_radius;
#define READ_BUFFER_MAX_STRING 100000
/*   char readbuf[READ_BUFFER_MAX_STRING+1]; */
  char buf[BUFFER_MAX_STRING+1];
  char txtst[BUFFER_MAX_STRING+1];
  char fmt[BUFFER_MAX_STRING+1];
  char txt[BUFFER_MAX_STRING+1];
  char command[BUFFER_MAX_STRING+1];
  char *p,*tmp,**legends;
  long *orb_colors;
  long *orb_fill;
  char *pbuffer;
  long lenpbuffer;

  radius=radius/vp.scale;

  orb_colors=NULL;
  orb_fill=NULL;
  lenpbuffer=0;
  pbuffer=NULL;


  
  /* read_BW_header(file); */

  /*  open output     */
  (void)sprintf(buf,"%s_w.ps",outfile);
  
  FH=fopen(buf,"w");
  
  prefix(FH,wpx0,wpy0,vp.scale,landscape,buf);


  /* plot_radiusaw frame */
  pline(FH,vp.x0,vp.y0,vp.x1,vp.y0,format->framelinewidth,"",0.);
  pline(FH,vp.x1,vp.y0,vp.x1,vp.y1,format->framelinewidth,"",0.);
  pline(FH,vp.x1,vp.y1,vp.x0,vp.y1,format->framelinewidth,"",0.);
  pline(FH,vp.x0,vp.y1,vp.x0,vp.y0,format->framelinewidth,"",0.);



  bd.kdist=NULL;
  bd.erg=NULL;
  /*   printf("[%s]\n",underlaybandfile); */
  if(strlen(underlaybandfile)>0)
    read_band_file(underlaybandfile,&bd);	


  if  ( (FHI=fopen(file,"r"))==NULL ){
    (void)printf("File \'%s\' not found!\n",file);
    goto error;
  }

  (void)printf("Read file:%s:\n",file);

  if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
  n=sscanf(buf,"%*s%ld%lg%ld%ld%ld%ld",&idum,&ef,&nk,&idum,&nband,&mspin);



  if(spin_legend == L_TRUE){
    if( use_own_legend_string == L_FALSE){
      nleg=mspin*nstate_to_plot;
    }else{
      nleg=mspin;
    }
  }else{
    if( use_own_legend_string == L_FALSE){
      nleg=1*nstate_to_plot;
    }else{
      nleg=1;
    }
  }
  
  orb_colors=(long*)malloc(sizeof(long)*nleg);
  orb_fill=(long*)malloc(sizeof(long)*nleg);
  legends=(char**)malloc(sizeof(char*)*nleg);
  for(i=0;i<nleg;i++){
    legends[i]=(char*)malloc(sizeof(char)*200);
    legends[i][0]='\0';
    orb_colors[i]=PS_BLACK;
    orb_fill[i]=1;
  }
  

  coltyp=1;
  for(is=0;is<nstate_to_plot;is++){
    



    rewind(FHI);
    if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    
    /* cdum=fgets(readbuf,READ_BUFFER_MAX_STRING,FHI); */
    (void)allocreadbuf(FHI,&pbuffer,&lenpbuffer);


    
    /*   (void)printf("n %ld\n",n);  */
    /*   (void)printf("%ld\n",mspin);  */
    
    


    if( use_own_legend_string == L_FALSE){
      i=42+(state_to_plot[is]-1)*17;

      (void)strncpy(txtst,pbuffer+i,17);
      txtst[17]='\0';
      if(mspin==2 && spin_legend){
 	(void)sprintf(legends[2*is],"%s $.$~",txtst); 
 	(void)sprintf(legends[2*is+1],"%s $.$~",txtst); 
      }else{
	(void)sprintf(legends[is],"%s",txtst);
      }
      (void)printf("state_to_plot : %s\n",txtst);
    }else{
      

      if(mspin==2 && spin_legend){
	(void)sprintf(legends[0],"%s $.$~",own_legend_string);
	(void)sprintf(legends[1],"%s $.$~",own_legend_string);
      }else{
	(void)sprintf(legends[0],"%s",own_legend_string);
      }

    }



    emin=1000.;
    emax=-1000.;
    /*   alloc arrays */
    x=(double*)malloc(sizeof(double)*nk);
    f1=(double*)malloc(sizeof(double)*nk*nband*mspin);
    wr=(double*)malloc(sizeof(double)*nk*nband*mspin);
    /*   scan format */
    (void)strcpy(fmt,"%lg%lg");
    for(i=0;i<state_to_plot[is]-1;i++){
      (void)strcat(fmt,"%*g");
    }
    (void)strcat(fmt,"%lg");
    /* read data */
    /*   (void)printf("%s\n",fmt); */
    for(ik=0;ik<nk;ik++){
      for(ms=0;ms<mspin;ms++){
	for(in=0;in<nband;in++){
	  /* cdum=fgets(readbuf,READ_BUFFER_MAX_STRING,FHI); 
	     n=sscanf(readbuf,fmt,&k,&e,&wei);
	   */
	  (void)allocreadbuf(FHI,&pbuffer,&lenpbuffer);
          n=sscanf(pbuffer,fmt,&k,&e,&wei);
	  /* 	(void)printf("%f %f %f\n",k,e,wei); */
	  
	  x[ik]=k;
	  n=ik+in*nk+ms*nk*nband;
	  f1[n]=e;
	  emin=min(emin,f1[n]);
	  emax=max(emax,f1[n]);
	  wr[n]=wei*radius;
	  
	}
      }
    }
    if(world.e0>world.e1){
      k=emax-emin;
      world.e0=emin-k*.05;
      world.e1=emax+k*.05;
    }
    
    world.k0=x[0];
    world.k1=x[nk-1];
    
    /* zero line efermi */
    if(is==0){  /* only for first state */
      
      wctovc(&vp,&world,1.,0.,&ik,&in,landscape);
      pline(FH,ik,vp.y0,ik,vp.y1,format->ticlinewidth,"",0);
      
      
      
      
      /* bands */

      if(1){
	if((strlen(underlaybandfile)>0))
	  ps_plot_bands(&bd,FH,&vp,&world,PS_BLACK);  
      }else{
	
	for(in=0;in<nband;in++){
	  if(mspin==2){
	    ms=0;
	    n=in*nk+ms*nk*nband;
	    drawxy(FH,&vp,&world,x,&(f1[n]),nk,RGBColors[PS_BLACK],0,2,landscape);
	    ms=1;
	    n=in*nk+ms*nk*nband;
	    drawxy(FH,&vp,&world,x,&(f1[n]),nk,RGBColors[PS_BLACK],2,1,landscape);
	    
	  }else{
	    ms=0;
	    n=in*nk+ms*nk*nband;
	    drawxy(FH,&vp,&world,x,&(f1[n]),nk,RGBColors[PS_BLACK],0,1,landscape);
	  }
	}
      }
    } /* only for first state */

    /* bandweights */
    

    
    for(ms=0;ms<mspin;ms++){
      for(in=0;in<nband;in++){
	
	n=in*nk+ms*nk*nband;



	if((nstate_to_plot==1)) {
	  coltyp=bw_colors[ms];
	  if(spin_legend == L_TRUE){ 
	    orb_colors[ms]=coltyp;
	    orb_fill[ms]=1;
	  }else{ 
	    if(ms ==0 ) { 
	      orb_colors[0]=coltyp;
	      orb_fill[ms]=1;
	    }
	  } 
	}else{




	  if(use_own_legend_string){
	    if( mspin ==2 && spin_legend){
	      if(is==0){
		orb_colors[2*is+ms]=coltyp;
		if(ms==0)orb_fill[2*is+ms]=1;
		if(ms==1)orb_fill[2*is+ms]=0;
	      }
	    }else{
	      if(is==0){ 
		orb_colors[is]=coltyp; 
		orb_fill[is]=1; 
	      }
	    }
	  }else{
	    if( mspin ==2 && spin_legend){
	      orb_colors[2*is+ms]=coltyp;
	      orb_fill[2*is+ms]=1-ms;
	    }else{
	      orb_colors[is]=coltyp;
	      orb_fill[is]=1;
	    }
	  }
	}
	

	filled=1;
	if(nstate_to_plot>1){
	  if(ms==1) filled=0;
	}

	drawxyr(FH,&vp,&world,x,&(f1[n]),&(wr[n]),nk
		,RGBColors[coltyp],landscape,filled); 
      }
    }
    free(x);
    free(f1);
    free(wr);


    coltyp=(coltyp+1)%(RGBMaxColors-1);


  }
  (void)fclose(FHI);


  free(bd.kdist);bd.kdist=NULL;
  free(bd.erg);bd.erg=NULL;



  
  /* special k-points */
  if  ( (FHI=fopen("+points","r"))==NULL ){
    (void)printf("File \'+points\' not found!\n");
    goto error;
  }
  
  
  idum=fscanf(FHI,"%*s%ld\n",&nb);
  /*   (void)printf("nb %ld \n",nb);  */
  
  for(ik=0;ik<nb;ik++){
    if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    chop(buf);
    p=buf;
    if( (p=strposprotect(buf,'\'')) != NULL ) {
      p++;
      if( (tmp=strposprotect(p,'\'')) != NULL){
	*tmp='\0';
      }
      (void)strncpy(txt,p,100);
      txt[100]='\0';
    }else{
      (void)strncpy(txt,buf+1,100);
      txt[100]='\0';
    }

    /*     (void)printf("label:%s \n",txt);   */
    if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    /*     (void)printf("1%s \n",buf);   */
    (void)sscanf(buf,"%lg%lg",&k,&e);
/*     (void)printf("k,e %f %f \n",k,e);   */
    if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
/*     (void)printf("2%s \n",buf);   */
    if((!fgets(buf,BUFFER_MAX_STRING,FHI))&&ik< nb-1)goto error;
/*     (void)printf("3%s \n",buf);   */
    
    wctovc(&vp,&world,k,world.e0,&i1,&j1,landscape);
    wctovc(&vp,&world,k,world.e1,&i2,&j2,landscape);
    pline(FH,i1,j1,i2,j2,format->ticlinewidth,"",0.);

    /* labels */
    if(print_labels==L_TRUE){
      i=(vp.x1-vp.x0)*(.05+format->xlabel_offsety);
      (void)sprintf(command,"%g %g scale\n",format->xticmark_scale
		    ,format->xticmark_scale);
      formtext(FH,vp.x1+i,j1,txt,90,60,center,command,format->xtic_subscript_scale);
    }
    
  }      
  (void)fclose(FHI);

  /* x-titel */
  j1=(vp.y1+vp.y0)*.5;
  i=(vp.x1-vp.x0)*.08;
  /* printf("format->title_scale=%g\n",format->title_scale); */
  (void)sprintf(command,"%g %g scale\n",format->title_scale,format->title_scale);
  formtext(FH,vp.x0-i,j1,title,90,80,center,command,format->title_subscript_scale);
    
  /* y-autotics+titel */

  autotics(FH,&vp,&world,yaxis_title,landscape
	   ,format->yticmark_scale
	   ,format->ytic_subscript_scale
	   ,format->yaxis_scale
	   ,format->yaxis_subscript_scale,format->ticlinewidth
	   ,format->display_yticmarks,format->yticmarkdigits
	   ,format->yaxis_offsetx,format->yaxis_offsety
	   ,format->ylabel_offsetx);
  
  /* legend */
  
  


  if(display_legend == L_TRUE ){

    i=(vp.x1-vp.x0)*legend_y+vp.x0;
      
    j1=(vp.y1-vp.y0)*legend_x+vp.y0;
    
    bwlegend(FH,&vp,i,j1,nleg,legends,RGBColors,orb_colors,orb_fill
	     ,6/vp.scale/* radius */,format->legend_scale
	     ,format->legend_subscript_scale,format->framelinewidth
	     ,format->display_legendframe,format->vertspacing);
  }

  for(i=0;i<nleg;i++)        
    free(legends[i]);
  free(legends);



        

  free(pbuffer);
  postfix(FH);
  (void)fclose(FH);
  (void)printf("%s_w.ps created\n",outfile);
  return;

error:;
  postfix(FH);
  (void)fclose(FH);
  (void)printf("ERROR while creating %s_w.ps, picture incomplete!\n",outfile);
  return;
}





/* ***********************************************************************
 * bandstructure plot
 * ***********************************************************************
 */

void plotband(VIEWPORT vp,WORLD world,FPLOLOGICAL print_labels,char *title
	      ,char *yaxis_title,FPLOLOGICAL display_legend
	      ,FPLODOUBLEPRECISION legend_x
	      ,FPLODOUBLEPRECISION legend_y,long nfile,char **files
	      ,char **legstr,const char *outfile,const tsFORMAT *format
	      ,FPLOLOGICAL spin_legend)
{
  tsBANDDATA bd;
  FILE *FHP,*FH; 
  long coltyp,ifile,n,ix1,iy1,ix2,iy2,nb,ip,i;
  double k,dum,de;
/*   long n,idum,nk,nband,mspin,i,ik,ms,in,nb,i1,j1,i2,j2; */
/*   double ef,emin,emax,k,e; */
  char buf[BUFFER_MAX_STRING+1]; 
  char buf1[BUFFER_MAX_STRING+1];
  char dir[STRING_LEN+1]; 
  char txt[BUFFER_MAX_STRING+1]; 
  char command[BUFFER_MAX_STRING+1];
  char *p,*tmp;
  



  /*  open output     */
  (void)sprintf(buf,"%s.ps",outfile);
  FH=fopen(buf,"w");
  
  prefix(FH,wpx0,wpy0,vp.scale,landscape,buf);


  /* draw frame */
  pline(FH,vp.x0,vp.y0,vp.x1,vp.y0,format->framelinewidth,"",0.);
  pline(FH,vp.x1,vp.y0,vp.x1,vp.y1,format->framelinewidth,"",0.);
  pline(FH,vp.x1,vp.y1,vp.x0,vp.y1,format->framelinewidth,"",0.);
  pline(FH,vp.x0,vp.y1,vp.x0,vp.y0,format->framelinewidth,"",0.);

  coltyp=0;

  /* extract path of first file */
  if(nfile>0){
    (void)strcpy(dir,files[0]);
    p=dir+strlen(dir);
    
    while(p!=dir){
      if(*p != '/') {
	p--;
      }else{
	*p='\0';
	break;
      }
    }
    if(p==dir){
      (void)strcpy(dir,".");
    }
  }



  
  (void)printf("\n");
  /* for all files */
  for(ifile=0;ifile<nfile;ifile++){

    bd.kdist=NULL;
    bd.erg=NULL;
    read_band_file(files[ifile],&bd);
      
    
    /* autoscale of energy */
    if(world.e0>world.e1){
      de=bd.emax-bd.emin;
      world.e0=bd.emin-de*.05;
      world.e1=bd.emax+de*.05;
    }
    /*     world.k0=0; */
    /*     world.k1=1; */


    if(ifile==0){
      world.k0=bd.kdist[0];
      world.k1=bd.kdist[bd.nk-1];
    }
    
    /* plot bands */
    ps_plot_bands(&bd,FH,&vp,&world,coltyp);  

    if(ifile==0){
      /* zero line efermi */
      wctovc(&vp,&world,1.,0.,&ix1,&iy1,landscape);
      pline(FH,ix1,vp.y0,ix1,vp.y1,format->ticlinewidth,"",0);
      
      
      /* special k-points */
      (void)sprintf(buf1,"%s/+points",dir);
      if  ( (FHP=fopen(buf1,"r"))==NULL ){
	(void)printf("File \'%s\' does not exist.\n"
		     " This file usually is created by fplo.\n"
		     ,buf1
		     );
	goto ende;
      }
      
      
      if(fscanf(FHP,"%*s%ld\n",&nb)!=1)goto error;
      /*   (void)printf("nb %ld \n",nb);  */
      
      for(ip=0;ip<nb;ip++){
	if(!fgets(buf,BUFFER_MAX_STRING,FHP))goto error;
	chop(buf);
	p=buf;
	if( (p=strposprotect(buf,'\'')) != NULL ) {
	  p++;
	  if( (tmp=strposprotect(p,'\'')) != NULL){
	    *tmp='\0';
	  }
	  (void)strncpy(txt,p,100);
	  txt[100]='\0';
	}else{
	  (void)strncpy(txt,buf+1,100);
	  txt[100]='\0';
	}
	
/* 	(void)printf("label:%s \n",txt);   */
	if(!fgets(buf,BUFFER_MAX_STRING,FHP))goto error;
	/*     (void)printf("1%s \n",buf);   */
	(void)sscanf(buf,"%lg%lg",&k,&dum);
	/*     (void)printf("k,e %f %f \n",k,e);   */
	if(!fgets(buf,BUFFER_MAX_STRING,FHP))goto error;
	/*     (void)printf("2%s \n",buf);   */
	if(!fgets(buf,BUFFER_MAX_STRING,FHP)&&ip< nb-1)goto error;
	/*     (void)printf("3%s \n",buf);   */
	
	wctovc(&vp,&world,k,world.e0,&ix1,&iy1,landscape);
	wctovc(&vp,&world,k,world.e1,&ix2,&iy2,landscape);
	pline(FH,ix1,iy1,ix2,iy2,format->ticlinewidth,"",0.);
	
	/* labels */
	if(print_labels == L_TRUE){
	  i=(vp.x1-vp.x0)*(.05+format->xlabel_offsety);
	  (void)sprintf(command,"%g %g scale\n",format->xticmark_scale
			,format->xticmark_scale);
	  formtext(FH,vp.x1+i,iy1,txt,90,60,center,command,format->xtic_subscript_scale);
	}
	
      }
      (void)fclose(FHP);
      
      /* x-titel */
      iy1=(vp.y1+vp.y0)*.5;
      i=(vp.x1-vp.x0)*.08;
      /* printf("format->title_scale=%g\n",format->title_scale); */
      (void)sprintf(command,"%g %g scale\n",format->title_scale,format->title_scale);

      formtext(FH,vp.x0-i,iy1,title,90,80,center,command,format->title_subscript_scale);
      
      /* y-autotics+titel */
      autotics(FH,&vp,&world,yaxis_title,landscape
	       ,format->yticmark_scale
	       ,format->ytic_subscript_scale
	       ,format->yaxis_scale
	       ,format->yaxis_subscript_scale,format->ticlinewidth
	       ,format->display_yticmarks,format->yticmarkdigits
	       ,format->yaxis_offsetx,format->yaxis_offsety
	       ,format->ylabel_offsetx);
    }  /* first */

    coltyp=(coltyp+1)%(RGBMaxColors-1);

    free(bd.kdist);bd.kdist=NULL;
    free(bd.erg);bd.erg=NULL;
  
  
    
  }  /* ifile */
  /* legend */
  
  if(spin_legend == L_TRUE)
    n=bd.mspin;
  else
    n=1;
  if(display_legend == L_TRUE){
    
    i=(vp.x1-vp.x0)*legend_y+vp.x0;
    
    iy1=(vp.y1-vp.y0)*legend_x+vp.y0;
    
    legend(FH,&vp,i,iy1,nfile,legstr,RGBColors,RGBMaxColors,n
	   ,format->legend_scale
	   ,format->legend_subscript_scale,format->framelinewidth
	   ,format->display_legendframe,format->vertspacing);
  }
  
  
  
  
  
  
  
 ende:;

  postfix(FH);
  (void)fclose(FH);
  (void)printf("\nfile %s.ps created\n",outfile);
  return;
 error:;

  postfix(FH);
  (void)fclose(FH);
  (void)printf("some error occured\n");
  return;  
}


/* ***********************************************************************
 * bloch spectral density plot
 * ***********************************************************************
 */

void plotakbl(VIEWPORT vp,WORLD world,FPLODOUBLEPRECISION plot_radius
	      ,FPLOLOGICAL print_labels,char *title,char *yaxis_title
	      ,FPLOLOGICAL display_legend
	      ,FPLODOUBLEPRECISION legend_x
	      ,FPLODOUBLEPRECISION legend_y,long nfile,char **files
	      ,char **legstr,const char *outfile
	      ,const tsFORMAT *format
	      ,FPLOLOGICAL spin_legend)
{
  FILE *FHI,*FH=NULL;
  long n,idum,nk,nband,mspin,i,ik,ms,in,coltyp,nb,i1,j1,i2,j2,*f1,di,dj
    ,dti,dtj;
  double ef,emin,emax,*x,*y,k,e,wei,g;
  double radius=plot_radius;
  
  char command[BUFFER_MAX_STRING+1];
  char buf[BUFFER_MAX_STRING+1];
  char txtst[BUFFER_MAX_STRING+1];
  char txt[BUFFER_MAX_STRING+1];
  char *p,*tmp,**legends;


  
  ms=0;
  while(ms++<2){
  
    (void)sprintf(buf,"+akbl.%ld",ms);
    if  ( (FHI=fopen(buf,"r"))==NULL ){
      (void)printf("File \'%s\' not found!\n",buf);
      goto ende;
    }
    
    /*  open output     */
    (void)sprintf(buf,"%s_akbl_%ld.ps",outfile,ms);
    FH=fopen(buf,"w");
    
    prefix(FH,wpx0,wpy0,vp.scale,landscape,buf);
    
    n=fscanf(FHI,"%*s%ld%lg%ld%ld%ld%ld\n",&idum,&ef,&nk,&idum,&nband,&mspin);
    
    
    
    emin=1000.;
    emax=-1000.;
    /*   alloc arrays */
    x=(double*)malloc(sizeof(double)*nk);
    y=(double*)malloc(sizeof(double)*nband);
    f1=(long*)malloc(sizeof(long)*nk*nband);
    
    
    /* read data */
    
    for(ik=0;ik<nk;ik++){
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
      (void)sscanf(buf,"%*s%lf",&k);
      /*     (void)printf("k:%f\n",k);  */
      x[ik]=k;
      for(in=0;in<nband;in++){
	if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
	n=sscanf(buf,"%lf%lf",&e,&wei);
	/* (void)printf("%f %f %f\n",k,e,wei);  */
	
	y[in]=e;
	n=ik+in*nk;
	f1[n]=min((long)(wei/radius*250.),250);
	
	emin=min(emin,e);
	emax=max(emax,e);
	
      }
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    }
    (void)fclose(FHI);
    if(world.e0>world.e1){
      k=emax-emin;
      world.e0=emin-k*.05;
      world.e1=emax+k*.05;
    }
    
    world.k0=x[0];
    world.k1=x[nk-1];

    
    
    /* bandweights */
    coltyp=PS_BLACK;
    drawxyd(FH,&vp,&world,x,y,f1,nk,nband,RGBColors[coltyp] 
	    ,landscape);  
    

    /* draw frame */
    pline(FH,vp.x0,vp.y0,vp.x1,vp.y0,format->framelinewidth,"",0.);
    pline(FH,vp.x1,vp.y0,vp.x1,vp.y1,format->framelinewidth,"",0.);
    pline(FH,vp.x1,vp.y1,vp.x0,vp.y1,format->framelinewidth,"",0.);
    pline(FH,vp.x0,vp.y1,vp.x0,vp.y0,format->framelinewidth,"",0.);
    
    /* zero line efermi */
    wctovc(&vp,&world,1.,0.,&ik,&in,landscape);
    pline(FH,ik,vp.y0,ik,vp.y1,format->ticlinewidth,"",0);
    
    
    /* special k-points */
    if  ( (FHI=fopen("+points","r"))==NULL ){
      (void)printf("no +points present\n");
      goto ende;
    }
    
    
    idum=fscanf(FHI,"%*s%ld\n",&nb);
    /*   (void)printf("nb %ld \n",nb);  */
    
    for(ik=0;ik<nb;ik++){
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
      chop(buf);
      p=buf;
      if( (p=strposprotect(buf,'\'')) != NULL ) {
	p++;
	if( (tmp=strposprotect(p,'\'')) != NULL){
	  *tmp='\0';
	}
	(void)strncpy(txt,p,100);
	txt[100]='\0';
      }else{
	(void)strncpy(txt,buf+1,100);
	txt[100]='\0';
      }
      
/*       (void)printf("label:%s \n",txt);   */
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
      /*     (void)printf("1%s \n",buf);   */
      (void)sscanf(buf,"%lg%lg",&k,&e);
      /*     (void)printf("k,e %f %f \n",k,e);   */
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
      /*     (void)printf("2%s \n",buf);   */
      if((!fgets(buf,BUFFER_MAX_STRING,FHI))&&ik< nb-1)goto error;
      /*     (void)printf("3%s \n",buf);   */
      
      wctovc(&vp,&world,k,world.e0,&i1,&j1,landscape);
      wctovc(&vp,&world,k,world.e1,&i2,&j2,landscape);
      pline(FH,i1,j1,i2,j2,format->ticlinewidth,"",0.);
      
      /* labels */
	if(print_labels == L_TRUE){
	  i=(vp.x1-vp.x0)*(.05+format->xlabel_offsety);
	  (void)sprintf(command,"%g %g scale\n",format->xticmark_scale
			,format->xticmark_scale);
	  formtext(FH,vp.x1+i,j1,txt,90,60,center,command,format->xtic_subscript_scale);
	}
      
    }      
    (void)fclose(FHI);
    
    /* x-titel */
    j1=(vp.y1+vp.y0)*.5;
    i=(vp.x1-vp.x0)*.08;
    formtext(FH,vp.x0-i,j1,title,90,80,center,"",0.6);
    
    /* y-autotics+titel */
    autotics(FH,&vp,&world,yaxis_title,landscape
	     ,format->yticmark_scale
	     ,format->ytic_subscript_scale
	     ,format->yaxis_scale
	     ,format->yaxis_subscript_scale,format->ticlinewidth
	     ,format->display_yticmarks,format->yticmarkdigits
	     ,format->yaxis_offsetx,format->yaxis_offsety
	     ,format->ylabel_offsetx);
    
    /* legend */
/*     (void)printf("%s\n",txtst); */
    legends=(char**)malloc(sizeof(char*));
    legends[0]=(char*)malloc(sizeof(char)*200);
    (void)strcpy(legends[0],txtst);
/*     (void)printf("%s\n",legends[0]); */
    if(display_legend == L_TRUE){
      
      
      di=80;
      dj=110;
      
      j1=vp.y1+200;
      
      for (i=0;i<11;i++){
	i1=vp.x0+i*di;
	i2=i1+di;
	j2=j1+dj;
	
	wei=((double)(10-i))/10.;
/* 	(void)printf("%f\n",wei); */
	g=1.-wei;
	if(!i){
	  (void)sprintf(buf,">%3.2f\n",wei*radius);
	}else{
	  (void)sprintf(buf,"%3.2f\n",wei*radius);
	}
	(void)fprintf(FH,"v n %ld %ld m %ld %ld l %ld %ld l %ld %ld l c v %f g f r 0. g s r\n"
		,i1,j1,i2,j1,i2,j2,i1,j2,g);
	dti=di/2+50*.4;
	dtj=50;
	(void)fprintf(FH,"v \n/Helvetica-ISOLatin1 findfont 50 scalefont setfont \n"
		"%ld %ld translate 90 rotate 0 0 m \n"
		"(%s)RJ\n(%s)show\nr\n",i1+dti,j1-dtj,buf,buf);
	
      }
    }
    
    free(legends[0]);
    free(legends);
    
    
    
    
    free(x);
    free(f1);
    free(y);

    postfix(FH);
    (void)fclose(FH);

    (void)printf("%s_akbl_%ld.ps created\n",outfile,ms);

  ende:;
  }
    
  return;
 error:  
  if(FH){
    postfix(FH);
    (void)fclose(FH);
  }
  (void)printf("some error occured\n");
  return;  

  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void create_bpfedit(const char *fname)
{
  /* const char *nsubfunc="create_bpfedit"; */
  FILE *fh;
  long n,i;
  /* long  i,j,n,j0,l; */

  if(pathExists(fname)==1){
    /* (void)printf("File '%s' exists, not recreated!\n",fname); */
    return;
  }
  fh=fopen(fname,"w");
  (void)fprintf(fh,"%s\n\n",
"########################################################################\n\
# this file is part of the FPLO bandstructure package\n\
# it was created automatically\n\
# please dont delete this file nor\n\
# edit it, unless you really know what you are doing\n\
# any changes added by you may be lost if this file is recreated\n\
# by FPLO (which is the normal behaviour )\n\
# so keep duplicates of your changes somewhere else\n\
#\n\
########################################################################");
  
  /* header with version  numbers*/
  (void)fprintf(fh,"section header{\n");
  (void)fprintf(fh,"  struct {char[*] mainversion;char[*] subversion;} version={");
  (void)fprintf(fh,"\"%s\",\"%s\"};\n",FPLOMAINVERSION,FPLOSUBVERSION);
  (void)fprintf(fh,"};\n\n\n");
  (void)fprintf(fh,"section data{\n") ;
  (void)fprintf(fh,"%s","\
  struct{\n\
     char[*] screenname;\n\
     char[*] filename;\n\
     struct {\n\
        char[*] action;\n\
        char[*] menubarentry;\n\
        char[*] actionname;\n\
        }menus[*];\n\
     struct{\n\
        char[*] action;\n\
        char[*] variablename;\n\
        char[*] defaultvalue;\n\
        char[*] fieldname;\n");
  (void)fprintf(fh,"%s","\
        int namepos;\n\
        int fieldoffset;\n\
        int fieldwidth;\n\
        char[*] testflags;\n\
        char[*] testset[*];\n\
        real intervalleft;\n\
        real intervalright;\n\
        }entries[*];\n\
     }screens[*]=\n");
  (void)fprintf(fh,"  {\n");
  /*************************************************************************/
  /* main menu*/
  (void)fprintf(fh,"     {\"MAIN MENU\"\n"); 
  (void)fprintf(fh,"     ,\"%s\"\n",BPINPUTFILE); 
  /* main menu: menu entries*/
  (void)fprintf(fh,"          ,{\n");
  (void)fprintf(fh,"\
              {\"MAIN | MAINQUIT\",\"@(q)uit/save\",\"\"}\n\
             ,{\"MAIN | MAINEXIT\",\"e@(x)it/discard\",\"\"}\n\
             ,{\"MAIN | EXTERNEDIT\",\"@(e)xtern edit\",\"\"}\n\
             ,{\"MAIN | RUN\",\"@(+) plot\",\"-intmp\"}\n\
             ,{\"MAIN | RUN\",\"@(g)v\",\"-gv -intmp\"}\n\
             ,{\"MAIN | SEPARATOR\",\"\",\"\"}\n\
             ,{\"MAIN | HELP\",\"@(H)elp\",\"\"}\n\
             ,{\"ALT  | SUBMENU\",\"@(F)ormat\",\"FORMAT MENU\"}\n\
             ,{\"ALT  | RESTORE\",\"@(R)estore\",\"\"}\n\
             ,{\"ALT  | REBUILD\",\"R@(e)create\",\"=.bp\"}\n");
  (void)fprintf(fh,"           }\n"); 
  /* main menu: screen items*/
  (void)fprintf(fh,"          ,{\n");
  (void)fprintf(fh,"              {\"TITLE|EOL\",\"\",\"\",\"GENERAL\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");

  /* bandplot type */
  (void)fprintf(fh,"             ,{\"SELECTBOX|  AUTO_WIDTH |EOL \""
	  ",\"band_plot_type\",\"1\",\"@(p)lot type\",1,22,40,\"\",{\n");
  n=sizeof(BAND_PLOT_TYPES)/sizeof(char*);
  (void)fprintf(fh,"              ");
  for(i=0;i<n;i++){
    (void)fprintf(fh,"\"%ld\\n@(%ld)\\t%s\"\n",i+1,i+1,BAND_PLOT_TYPES[i]); 
    if(i<n-1)(void)fprintf(fh,"              ,");
  }
  (void)fprintf(fh,"              },0,0}\n");
  /* energy bounds*/

  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"energy_bounds.emin\""
	  ",\"1.\",\"@(l)ower energy bound\",1,22,20,\"\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"energy_bounds.emax\""
	  ",\"-1.\",\"@(u)pper energy bound\",1,22,20,\"\",{},-INF,+INF}\n");

  /*title*/
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM|EOL\",\"title\",\"1\",\"@(t)itle\""
	  ",1,22,40,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yaxis_title\",\"1\","
	  "\"@(y)axis title\",1,22,40,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"print_labels\",\"t\","
	  "\"@(a)uto x-labels\",1,22,40,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"display_yticmarks\",\"t\","
	  "\"@(dt)show y ticklabels\",1,22,40,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM\",\"legend.display\",\"t\","
	  "\"@(dl)\tdisplay legend\",1,22,10,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend.display_spin\",\"t\","
	  "\"@(ds)\tspin dependend\",42,22,10,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM|EOL\",\"legend.displayframe\",\"t\","
	  "\"@(dlf)\tdisplay leg. frame\",1,22,10,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend.x\",\"0.8\","
	  "\"@(r1)\\trelative x position\",1,22,40,\"\",{},-INF,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend.y\",\"-0.05\","
	  "\"@(r2)\\trelative y position\",1,22,40,\"\",{},-INF,+INF}\n"); 



  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,-1,0,\"\""
	  ",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"POSTSCRIPT\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"postscript.outfile\",\"band\","
	  "\"@(o)ut file prefix\",1,22,40,\"\",{},-INF,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"postscript.viewer\",\"gv\","
	  "\"@(v)iewer\",1,22,40,\"\",{},-INF,+INF}\n"); 
  



  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,-1,0,\"\""
	  ",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"BAND STRUCTURE\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");

  (void)fprintf(fh,"             ,{\"FIELD \",\"files\",\"\",\"\""
	  ",1,27,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"COUNTER|EOL\" ,\"\",\"\",\"@(N)umber of files to load\""
	  ",1,27,13,\"\",{},1,+INF}\n");
  /* (void)fprintf(fh,"             ,{\"DASHLINE\",\"\",\"\",\"\",1,50,0,\"\"" */
/* 	  ",{},0,0}\n"); */
  (void)fprintf(fh,"             ,{\"TITLE\",\"\",\"\","
	  "\"        file                     legend\""
	  ",1,-1,0,\"\",{},0,0}\n");

  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 

  (void)fprintf(fh,"             ,{\"ITEM\" ,\"file\",\"+band\",\"\",1,5,17,\"\""
	  ",{},1,+INF}\n");
  (void)fprintf(fh,"             ,{\"ITEM\" ,\"legend\",\"legendx\",\"\",31,0,20"
	  ",\"\",{},1,+INF}\n");
  (void)fprintf(fh,"             ,{\"ENDFIELD\",\"\",\"\",\"\",1,25,8,\"\""
	  ",{},0,0}\n");



  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,-1,0,\"\""
	  ",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"BANDWEIGHTS\""
	  ",1,-1,0,\"\",{},0,0}\n");

  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");

  /* bandweights plot radius*/
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"bw_file\",\"+bweights\","
	  "\"@(WF)\tweights file\",1,22,40,\"\",{},-INF,+INF}\n"); 

  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"bwband_file\",\"+band\"," 
		"\"@(WU)\tunderlying band file\",1,22,40,\"\",{},-INF,+INF}\n");  

  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"plot_radius\""
	  ",\"1.\",\"@(m)ax weights-radius\",1,22,20,\"LEFT_OPEN\","
	  "{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEMLIST | EOL\",\"states_to_plot\""
	  ",\"1\",\"@(s)tates to plot\",1,22,20,\"\","
	  "{},1,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM\",\"use_own_legend_string\",\"t\","
	  "\"@(wl)\tuse own legend str.\",1,22,4,\"\",{},0,+INF}\n"); 
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"bw_own_legend\",\"\","
		"\"@(ws)\tlegend string\",32,18,22,\"\",{},-INF,+INF}\n");   


  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  /* bandweights colors */
  (void)fprintf(fh,"             ,{\"SELECTBOX|  AUTO_WIDTH |EOL  \""
		",\"bw_colors_spinup\",\"0\",\"@(cu)\tspin up color\",1,22,10,\"\",{\n");
  n=RGBMaxColors-1;
  (void)fprintf(fh,"              ");
  for(i=0;i<n;i++){
    (void)fprintf(fh,"\"%ld\\n@(%ld)\\t%s\"\n",i,i,PSColorNames[i]); 
    if(i<n-1)(void)fprintf(fh,"              ,");
  }
  (void)fprintf(fh,"              },0,0}\n");
  (void)fprintf(fh,"             ,{\"SELECTBOX|  AUTO_WIDTH |EOL \""
		",\"bw_colors_spindown\",\"0\",\"@(cd)\tspin down color\",1,22,10,\"\",{\n");
  n=RGBMaxColors-1;
  (void)fprintf(fh,"              ");
  for(i=0;i<n;i++){
    (void)fprintf(fh,"\"%ld\\n@(%ld)\\t%s\"\n",i,i,PSColorNames[i]); 
    if(i<n-1)(void)fprintf(fh,"              ,");
  }
  (void)fprintf(fh,"              },0,0}\n");

  
  (void)fprintf(fh,"           }\n") ;
  (void)fprintf(fh,"     }\n") ;







  (void)fprintf(fh,"     ,{\"FORMAT MENU\"\n"); 
  (void)fprintf(fh,"     ,\"%s\"\n",BPINPUTFILE); 
  /* FORMAT menu: menu entries*/
  (void)fprintf(fh,"          ,{\n");
  (void)fprintf(fh,"\
              {\"MAIN | EXIT\",\"e@(x)it\",\"\"}\n\
             ,{\"MAIN | EXTERNEDIT\",\"@(e)xtern edit\",\"\"}\n\
             ,{\"MAIN | RUN\",\"@(+) plot\",\"-intmp\"}\n\
             ,{\"MAIN | RUN\",\"@(g)v\",\"-gv -intmp\"}\n\
             ,{\"MAIN | SEPARATOR\",\"\",\"\"}\n\
             ,{\"MAIN | HELP\",\"@(H)elp\",\"\"}\n\
             ");
  (void)fprintf(fh,"           }\n"); 
  /* FORMAT menu: screen items*/
  (void)fprintf(fh,"          ,{\n");
  (void)fprintf(fh,"              {\"TITLE|EOL\",\"\",\"\",\"TITLE FORMAT\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"title_format.title_scale\""
	",\"1.\",\"@(tt)\ttitle scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"title_format.subscript_scale\""
  ",\"1.\",\"@(ts)\tsubscript scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");


  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"YAXIS TITEL\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yaxis_title_format.title_scale\""
	",\"1.\",\"@(yt)\ttitle scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yaxis_title_format.subscript_scale\""
  ",\"1.\",\"@(ys)\tsubscript scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");

  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yaxis_title_format.offsetx\""
  ",\"1.\",\"@(yox)\tx-offset\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yaxis_title_format.offsety\""
  ",\"1.\",\"@(yoy)\ty-offset\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");

  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"X TICLABELS\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"xticmark_format.tic_scale\""
	",\"1.\",\"@(txt)\ttic label scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"xticmark_format.subscript_scale\""
  ",\"1.\",\"@(txs)\tsubscript scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"xticmark_format.offsety\""
  ",\"1.\",\"@(txoy)\ty-offset\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");

  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"Y TICLABELS\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yticmark_format.tic_scale\""
	",\"1.\",\"@(tyt)\ttic labels scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yticmark_format.subscript_scale\""
  ",\"1.\",\"@(tys)\tsubscript scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yticmark_format.digits\""
  ",\"-1\",\"@(tyd)\t# of digits\",1,22,20,\"\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"yticmark_format.offsetx\""
  ",\"1.\",\"@(tyox)\tx-offset\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");

  
  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"LEGEND FORMAT\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend_format.legend_scale\""
	",\"1.\",\"@(lt)\ttitle scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend_format.subscript_scale\""
  ",\"1.\",\"@(ls)\tsubscript scale\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"legend_format.vertspacing\""
  ",\"1.\",\"@(lv)\tvertical spacing\",1,22,20,\"\",{},-INF,+INF}\n");


  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"LINE WIDTH\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"frame.framelinewidth\""
  ",\"1.\",\"@(lwf)\tframe line width\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"ticlinewidth\""
  ",\"1.\",\"@(lwt)\ttic line width\",1,22,20,\"LEFT_OPEN\",{},0,+INF}\n");

  (void)fprintf(fh,"             ,{\"DASHLINE|EOL\",\"\",\"\",\"\",1,50,0,\"\"" 
 	  ",{},0,0}\n"); 
  (void)fprintf(fh,"             ,{\"TITLE|EOL\",\"\",\"\",\"FRAME SIZE\""
	  ",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"             ,{\"EOL\",\"\",\"\",\"\",1,-1,0,\"\",{},0,0}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"frame.left\""
  ",\"1.\",\"@(fl)\tframe left\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"frame.right\""
  ",\"1.\",\"@(fr)\tframe right\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"frame.top\""
  ",\"1.\",\"@(ft)\tframe top\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");
  (void)fprintf(fh,"              ,{\"ITEM | EOL\",\"frame.bottom\""
  ",\"1.\",\"@(fb)\tframe bottom\",1,22,20,\"LEFT_OPEN\",{},-INF,+INF}\n");
  
  /*     ! end one screen */
  (void)fprintf(fh,"           }\n"); 
  (void)fprintf(fh,"     }\n") ;

  /*     ! end struct screens */
  (void)fprintf(fh,"  };\n");
  /*     ! end section data */
  (void)fprintf(fh,"};\n");
  /* ! file end  */
  (void)fclose(fh);
  (void)printf("File '%s' created!\n",fname);
  exit(1);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void create_bpinfile(const char *fname,FPLOINTEGER band_plot_type
		     ,const char * title
		     ,const char * yaxis_title
		     ,FPLODOUBLEPRECISION emin
		     ,FPLODOUBLEPRECISION emax
		     ,FPLODOUBLEPRECISION plot_radius
		     ,FPLOINTEGER nstate_to_plot
		     ,const FPLOINTEGER *state_to_plot
		     ,FPLOLOGICAL print_labels
		     ,FPLOLOGICAL display_legend
		     ,FPLODOUBLEPRECISION legend_x
		     ,FPLODOUBLEPRECISION legend_y
		     ,long nfile,char **files
		     ,char **legstr
		     ,const char *outfile
		     ,const char *viewer
		     ,const tsFORMAT *format
		     ,long init
		     ,FPLOLOGICAL spin_legend
		     ,FPLOLOGICAL use_own_legend_string
		     ,const char *own_legend_string
		     ,const long *bw_colors
		     ,const char *bwfile
		     ,const char *bwbandfile
		     )
{
  /* const char *nsubfunc="create_bpinfile"; */
  FILE *fh;
  char host[5001];
  char date[5001];
  time_t t;  
  long i;
  if(init){
    if(pathExists(fname)==1){
      (void)printf("File '%s' exists, not recreated!\n",fname);
      return;
    }
  }
  
  fh=fopen(fname,"w");

    (void)fprintf(fh,"%s\n\n",
"########################################################################\n\
# this file is part of the FPLO bandstructure package\n\
# it was created automatically\n\
# please dont delete this file nor\n\
# edit it, unless you really know what you are doing\n\
# any changes added by you may be lost if this file is recreated\n\
# by FPLO (which is the normal behaviour )\n\
# so keep duplicates of your changes somewhere else\n\
#\n\
########################################################################");
  
  /* header with version  numbers*/
  (void)fprintf(fh,"section header{\n");
  (void)fprintf(fh,"  struct {char[*] mainversion;char[*] subversion;} version={");
  (void)fprintf(fh,"\"%s\",\"%s\"};\n",FPLOMAINVERSION,FPLOSUBVERSION);
  (void)fprintf(fh,"\n\n");


  (void)time(&t);
  (void)sprintf(date,"%s",ctime(&t));
  chop(date);   
  gethostname(host,5000);

  (void)fprintf(fh,"  struct {char[*] hostname;char[*] date;char[*] by;} last_changes\n");
  (void)fprintf(fh,"={\"%s\",\"%s\",\"%s\"};\n",host,date,FULLEXECNAME);
  (void)fprintf(fh,"};\n\n\n");
  /* section data*/
  (void)fprintf(fh,"section data{\n");
  
  (void)fprintf(fh," struct {int type;char[*] description;} band_plot_type\n");
  (void)fprintf(fh," ={"FPLOINTEGERFORMAT",\"%s\"};\n",band_plot_type,BAND_PLOT_TYPES[band_plot_type]);

  (void)fprintf(fh," struct {real emin;real emax;} energy_bounds\n");
  (void)fprintf(fh," ={%g,%g};\n",emin,emax);

  (void)fprintf(fh," char[*] title=\"%s\";\n",title);
  (void)fprintf(fh," char[*] yaxis_title=\"%s\";\n",yaxis_title);
  (void)fprintf(fh," logical display_yticmarks=%c;\n"
		,(format->display_yticmarks?'t':'f'));  


  (void)fprintf(fh," real plot_radius=%g;\n",plot_radius);

  /* HIERL(nstate_to_plot); */
  /* HIERX(state_to_plot); */
  (void)fprintf(fh," int state_to_plot="FPLOINTEGERFORMAT";\n",state_to_plot[0]);

  /* HIERL(nstate_to_plot); */
 
  (void)fprintf(fh," int states_to_plot[*]={\n");
  for(i=0;i<nstate_to_plot;i++){
    (void)fprintf(fh,""FPLOINTEGERFORMAT"",state_to_plot[i]);
    if(i<nstate_to_plot-1){
      (void)fprintf(fh,",\n");
    }
  }
  (void)fprintf(fh," };\n");
  

  (void)fprintf(fh," logical print_labels=%c;\n",(print_labels == L_TRUE?'t':'f'));
  (void)fprintf(fh," logical use_own_legend_string=%c;\n",(use_own_legend_string == L_TRUE?'t':'f'));

  (void)fprintf(fh," struct {logical display; real x; real y;logical display_spin;logical displayframe;} legend\n");
  (void)fprintf(fh," ={%c,%g,%g,%c,%c};\n",(display_legend == L_TRUE?'t':'f')
	  ,legend_x,legend_y
		,(spin_legend == L_TRUE?'t':'f')
		,(format->display_legendframe == L_TRUE?'t':'f')
		);

  (void)fprintf(fh," struct {char[*] file; char[*] legend;} files[*]\n");
  (void)fprintf(fh," ={\n");
  for(i=0;i<nfile;i++){
    (void)fprintf(fh,"{\"%s\",\"%s\"}",files[i],legstr[i]);
    if(i<nfile-1){
      (void)fprintf(fh,",\n");
    }
  }
  (void)fprintf(fh," };\n");
  
  
  (void)fprintf(fh," char[*] bw_own_legend=\"%s\";\n",own_legend_string);  

  (void)fprintf(fh," char[*] bwband_file=\"%s\";\n",bwbandfile);  
  (void)fprintf(fh," char[*] bw_file=\"%s\";\n",bwfile);  


  (void)fprintf(fh," struct {int type;char[*] description;} bw_colors_spinup\n");
  (void)fprintf(fh," ={%ld,\"%s\"};\n",bw_colors[0],PSColorNames[bw_colors[0]]);
  (void)fprintf(fh," struct {int type;char[*] description;} bw_colors_spindown\n");
  (void)fprintf(fh," ={%ld,\"%s\"};\n",bw_colors[1],PSColorNames[bw_colors[1]]);

  

  /* end section data */  
  (void)fprintf(fh,"};\n\n\n");


  /* section viewer */  
  (void)fprintf(fh,"section viewer{\n");
  
  (void)fprintf(fh," struct {char[*] outfile;char[*] viewer;} postscript=\n");  
  (void)fprintf(fh," {\"%s\",\"%s\"};\n",outfile,viewer);  
  
  /* end section viewer */  
  (void)fprintf(fh,"};\n\n\n");


  /* section text format */  
  (void)fprintf(fh,"section format{\n");
  
  (void)fprintf(fh," struct {real title_scale;real subscript_scale;} title_format=\n");  
  (void)fprintf(fh," {%g,%g};\n",format->title_scale
		,format->title_subscript_scale);  

  (void)fprintf(fh," struct {real title_scale;real subscript_scale;"
		"real offsetx;real offsety;} yaxis_title_format=\n");  
  (void)fprintf(fh," {%g,%g,%g,%g};\n",format->yaxis_scale
		,format->yaxis_subscript_scale
		,format->yaxis_offsetx
		,format->yaxis_offsety
		);  

  (void)fprintf(fh," struct {real tic_scale;real subscript_scale;int digits;"
		"real offsetx;} yticmark_format=\n");  
  (void)fprintf(fh," {%g,%g,%ld,%g};\n"
		,format->yticmark_scale
		,format->ytic_subscript_scale
		,format->yticmarkdigits
		,format->ylabel_offsetx
		);  

  (void)fprintf(fh," struct {real tic_scale;real subscript_scale;"
		"real offsety;} xticmark_format=\n");  
  (void)fprintf(fh," {%g,%g,%g};\n"
		,format->xticmark_scale
		,format->xtic_subscript_scale
		,format->xlabel_offsety
		);  

  (void)fprintf(fh," struct {real legend_scale;real subscript_scale;real vertspacing;} legend_format=\n");  
  (void)fprintf(fh," {%g,%g,%g};\n",format->legend_scale
		,format->legend_subscript_scale
		,format->vertspacing);  


  (void)fprintf(fh," struct {real left;real right;real top;real bottom;"
		"real framelinewidth;} frame=\n");  
  (void)fprintf(fh," {%g,%g,%g,%g,%g};\n"
		,format->frameleft
		,format->frameright
		,format->frametop
		,format->framebottom
		,format->framelinewidth
		);  
  

  (void)fprintf(fh," real ticlinewidth=%g;\n",format->ticlinewidth);  

  /* end section viewer */  
  (void)fprintf(fh,"};\n\n\n");


  (void)fclose(fh);
  if(init){
    (void)printf("File '%s' created!\n",fname);
  }else{
    (void)printf("File '%s' updated!\n",fname);
  }
  exit(1);

}





/* ***********************************************************************
 *
 * ***********************************************************************
 */

void read_bpinfile(const char *fname,FPLOINTEGER *band_plot_type
		   ,char *title,long title_len
		   ,char * yaxis_title,long yaxis_len
		   ,FPLODOUBLEPRECISION *emin
		   ,FPLODOUBLEPRECISION *emax
		   ,FPLODOUBLEPRECISION *plot_radius
		   ,FPLOINTEGER *nstate_to_plot
		   ,FPLOINTEGER **state_to_plot
		   ,FPLOLOGICAL *print_labels
		   ,FPLOLOGICAL *display_legend
		   ,FPLODOUBLEPRECISION *legend_x
		   ,FPLODOUBLEPRECISION *legend_y
		   ,long *nfile,char ***files
		   ,char ***legstr
		   ,char *outfile,long outfile_len
		   ,char *viewer,long viewer_len
		   ,tsFORMAT *format
		   ,FPLOLOGICAL *spin_legend
		   ,FPLOLOGICAL *use_own_legend_string
		   ,char *own_legend_string,long own_len
		   ,long *bw_colors
		   ,char *bwfile,long bw_len
		   ,char *bwbandfile
		   ,long doupdate
		   )
{
  /* const char *nsubfunc="read_bpinfile";  */
  FPLOINTEGER size[7];
  FPLOINTEGER rank,idum;
  FPLODOUBLEPRECISION dum;
  long i,n,j,ver;
  char mbuf[STRING_LEN+1];
  char sbuf[STRING_LEN+1];
  char name[STRING_LEN+1];
  char *buf;

  /* HIER(nsubfunc); */

  /* HIER("read_bpinfile"); */

  if(!init_parser_control(&bp_pc,0,1))
    goto error;
  
  if(!parsefile(&yyin,fname,&bp_pc) ) goto error;


  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"version.mainversion",mbuf,VTYPE_STRING
	       ,(long)STRING_LEN
	       ,rank,size,0L))goto error;
  mbuf[STRING_LEN]='\0';
  removetrailingspace(mbuf);

  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"version.subversion",sbuf,VTYPE_STRING,(long)STRING_LEN
	       ,rank,size,0L))goto error;
  sbuf[STRING_LEN]='\0';
  removetrailingspace(sbuf);
  
  ver=VERSION_OK;
  if(strcmp(FPLOMAINVERSION,mbuf)!=0){
    vercheck(error);
  }
  if(strcmp(FPLOSUBVERSION,sbuf)!=0){
    (void)sprintf(errorbuffer,"Version of file '%s' (%s:%s) differs "
		  "from bandplot version (%s:%s)!\n"
		  "I suggest to build a new %s!",fname,mbuf,sbuf
		  ,FPLOMAINVERSION,FPLOSUBVERSION,fname);
    goto error;
  }



  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"title",title,VTYPE_STRING,title_len
	       ,rank,size,0L))goto error;
  title[title_len]='\0';
  removetrailingspace(title);

  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"yaxis_title",yaxis_title,VTYPE_STRING,yaxis_len
	       ,rank,size,0L))goto error;
  yaxis_title[yaxis_len]='\0';
  removetrailingspace(yaxis_title);


  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"energy_bounds.emin",emin,VTYPE_REAL,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"energy_bounds.emax",emax,VTYPE_REAL,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"band_plot_type.type",band_plot_type,VTYPE_INT,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"plot_radius",plot_radius,VTYPE_REAL,0L
	       ,rank,size,0L))goto error;


  if(varexists(&bp_pc,"states_to_plot")){
    
    if(!getDimensions(&bp_pc.table,"states_to_plot",&rank,size))goto error;
    
    *nstate_to_plot=size[0];
    
    *state_to_plot=(FPLOINTEGER*)malloc(sizeof(FPLOINTEGER)**nstate_to_plot);
    
    if(!getValue(&bp_pc.table,"states_to_plot",*state_to_plot,VTYPE_INT,0L
		 ,rank,size,0L))goto error;
  }else{
    
    *nstate_to_plot=1;
    /* HIERL(*nstate_to_plot); */
    
    *state_to_plot=(FPLOINTEGER*)malloc(sizeof(FPLOINTEGER)**nstate_to_plot);
    
    /* HIERX(*state_to_plot); */
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,"state_to_plot",*state_to_plot,VTYPE_INT,0L
		 ,rank,size,0L))goto error;
  }

  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"print_labels",print_labels,VTYPE_LOGICAL,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"legend.display",display_legend,VTYPE_LOGICAL,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"legend.x",legend_x,VTYPE_REAL,0L
	       ,rank,size,0L))goto error;
  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"legend.y",legend_y,VTYPE_REAL,0L
	       ,rank,size,0L))goto error;



  strcpy(name,"legend.display_spin");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,spin_legend,VTYPE_LOGICAL,0L
		 ,rank,size,0L))goto error;

  }

  strcpy(name,"legend.displayframe");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&format->display_legendframe
		 ,VTYPE_LOGICAL,0L
		 ,rank,size,0L))goto error;

  }


  strcpy(name,"use_own_legend_string");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,use_own_legend_string,VTYPE_LOGICAL,0L
		 ,rank,size,0L))goto error;

  }


  if(!getDimensions(&bp_pc.table,"files",&rank,size))goto error;

  *nfile=size[0];

  *files=(char**)malloc(sizeof(char*)**nfile);
  *legstr=(char**)malloc(sizeof(char*)**nfile);
  for(i=0;i<*nfile;i++){
    (*files)[i]=(char*)malloc(sizeof(char)*(STRING_LEN+1));
    (*legstr)[i]=(char*)malloc(sizeof(char)*(STRING_LEN+1));
  }

  /* its not perfect, but we use the fortran interface here */
  buf=(char*)malloc(sizeof(char)*(*nfile)*(STRING_LEN+1));
  if(!getValue(&bp_pc.table,"files.file",buf,VTYPE_STRING,STRING_LEN
	       ,rank,size,0L))goto error;


  n=0;
  for(i=0;i<*nfile;i++){
    for(j=0;j<STRING_LEN;j++){
      (*files)[i][j]=buf[n];
      n++;
    }
    ((*files)[i])[STRING_LEN]='\0'; 
    removetrailingspace((*files)[i]); 

  }



  
  if(!getValue(&bp_pc.table,"files.legend",buf,VTYPE_STRING,STRING_LEN
	       ,rank,size,0L))goto error;


  n=0;
  for(i=0;i<*nfile;i++){
    for(j=0;j<STRING_LEN;j++){
      (*legstr)[i][j]=buf[n];
      n++;
    }
    ((*legstr)[i])[STRING_LEN]='\0'; 
    removetrailingspace((*legstr)[i]); 
  }


  free(buf);


  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"postscript.outfile",outfile,VTYPE_STRING
	       ,outfile_len
	       ,rank,size,0L))goto error;
  outfile[outfile_len]='\0';
  removetrailingspace(outfile);


  rank=1;
  size[0]=1;
  if(!getValue(&bp_pc.table,"postscript.viewer",viewer,VTYPE_STRING
	       ,viewer_len
	       ,rank,size,0L))goto error;
  viewer[viewer_len]='\0';
  removetrailingspace(viewer);



  /* HIER("read_bpinfile2"); */


  strcpy(name,"title_format.title_scale");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    /* printf("%g\n",format->title_scale); */
    format->title_scale=(double)dum;
  }
  strcpy(name,"title_format.subscript_scale");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->title_subscript_scale=(double)dum;
  }

  strcpy(name,"yaxis_title_format.title_scale");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->yaxis_scale=(double)dum;
  }
  strcpy(name,"yaxis_title_format.subscript_scale");
  if(varexists(&bp_pc,name)){
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->yaxis_subscript_scale=(double)dum;
  }
  strcpy(name,"yaxis_title_format.offsetx");
  if(varexists(&bp_pc,name)){
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->yaxis_offsetx=(double)dum;
  }
  strcpy(name,"yaxis_title_format.offsety");
  if(varexists(&bp_pc,name)){
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->yaxis_offsety=(double)dum;
  }

  strcpy(name,"yticmark_format.tic_scale");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->yticmark_scale=(double)dum;
  }
  strcpy(name,"yticmark_format.digits");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&idum,VTYPE_INT,0L
		 ,rank,size,0L))goto error;
    format->yticmarkdigits=(long)idum;
  }
  strcpy(name,"yticmark_format.subscript_scale");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->ytic_subscript_scale=(double)dum;
  }
  strcpy(name,"yticmark_format.offsetx");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->ylabel_offsetx=(double)dum;
  }
  strcpy(name,"xticmark_format.tic_scale");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->xticmark_scale=(double)dum;

  }
  strcpy(name,"xticmark_format.subscript_scale");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->xtic_subscript_scale=(double)dum;
  }
  strcpy(name,"xticmark_format.offsety");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->xlabel_offsety=(double)dum;
  }


  strcpy(name,"legend_format.legend_scale");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->legend_scale=(double)dum;
  }
  strcpy(name,"legend_format.subscript_scale");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->legend_subscript_scale=(double)dum;

  }
  strcpy(name,"legend_format.vertspacing");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->vertspacing=(double)dum;

  }

  strcpy(name,"frame.framelinewidth");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->framelinewidth=(double)dum;

  }
  strcpy(name,"frame.left");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->frameleft=(double)dum;

  }
  strcpy(name,"frame.right");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->frameright=(double)dum;

  }
  strcpy(name,"frame.top");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->frametop=(double)dum;

  }
  strcpy(name,"frame.bottom");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->framebottom=(double)dum;

  }

  strcpy(name,"ticlinewidth");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&dum,VTYPE_REAL,0L
		 ,rank,size,0L))goto error;
    format->ticlinewidth=(double)dum;

  }

  strcpy(name,"display_yticmarks");
  if(varexists(&bp_pc,name)){

    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&format->display_yticmarks,VTYPE_LOGICAL,0L
		 ,rank,size,0L))goto error;

  }


  strcpy(name,"bw_own_legend");
  if(varexists(&bp_pc,name)){
    
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,own_legend_string,VTYPE_STRING
		 ,own_len
		 ,rank,size,0L))goto error;
    own_legend_string[own_len]='\0';
    removetrailingspace(own_legend_string);
  }

  strcpy(name,"bw_file");
  if(varexists(&bp_pc,name)){
    
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,bwfile,VTYPE_STRING
		 ,bw_len
		 ,rank,size,0L))goto error;
    bwfile[bw_len]='\0';
    removetrailingspace(bwfile);
  }


  strcpy(name,"bwband_file");
  if(varexists(&bp_pc,name)){
    
    
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,bwbandfile,VTYPE_STRING
		 ,bw_len
		 ,rank,size,0L))goto error;
    bwbandfile[bw_len]='\0';
    removetrailingspace(bwbandfile);
  }
  


  strcpy(name,"bw_colors_spinup.type");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;

    if(!getValue(&bp_pc.table,name,&idum,VTYPE_INT,0L
		 ,rank,size,0L))goto error;
    bw_colors[0]=idum;
  }
  strcpy(name,"bw_colors_spindown.type");
  if(varexists(&bp_pc,name)){
    rank=1;
    size[0]=1;
    if(!getValue(&bp_pc.table,name,&idum,VTYPE_INT,0L
		 ,rank,size,0L))goto error;
    bw_colors[1]=idum;
  }


  if(!clean_vartable(&bp_pc.table))goto error;
  
  finalcleanup();

  if(ver!=VERSION_OK || doupdate){
    (void)printf("Version update of %s!\n",fname);
    create_bpinfile(fname,*band_plot_type,title
		     ,yaxis_title ,*emin ,*emax ,*plot_radius
		     ,*nstate_to_plot,*state_to_plot
		    ,*print_labels ,*display_legend
		     ,*legend_x ,*legend_y ,*nfile,*files
		    ,*legstr ,outfile ,viewer,format,0L
		    ,*spin_legend,*use_own_legend_string,own_legend_string
		    ,bw_colors,bwfile,bwbandfile);
    
  }

  return;
 error:
  (void)printf("%s\n",errorbuffer);
  exit(1);
}


/************************************************************************
 *
 *  get opts
 *
 ************************************************************************
 */

static void scanopts(long argc,char **argv,long *gv,long *intmp,long *doupdate)
{
  long i=1;

  *intmp=0;
  *gv=0;
  *doupdate=0;
  while(i<argc){
    if( strcmp(argv[i],"-gv")==0){
      *gv=1;
    }else if( strcmp(argv[i],"-intmp")==0){
      *intmp=1;
    }else if( strcmp(argv[i],"-update")==0){
      *doupdate=1;
    }
    i++;
  }

  return;
}



/************************************************************************
 *
 ************************************************************************
 */
/*
static long read_BW_header(const char *file)
{
  FILE *FHI;
  long idum,nband,mspin,nk,state_to_plot,i;
  double ef;
  char buf[BUFFER_MAX_STRING+1];
  char txtst[BUFFER_MAX_STRING+1];
  return 1;
  if  ( (FHI=fopen(file,"r"))==NULL ){
    (void)printf("File \'%s\' not found!\n",file);
    goto error;
  }
   
  fscanf(FHI,"%*s%ld%lg%ld%ld%ld%ld\n",&idum,&ef,&nk,&idum,&nband,&mspin);
  if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
  
  (void)printf("nband %ld\n",nband);  
  
  


  for(state_to_plot=1;state_to_plot<=nband;state_to_plot++){
    i=42+(state_to_plot-1)*17;
    (void)strncpy(txtst,buf+i,11);
    txtst[11]='\0';
    (void)printf("state_to_plot : <%s>\n",txtst);
  }
  

  

  fclose(FHI);

  return 1;
 error:;
  return 0;
  
}*/



/************************************************************************
 *
 ************************************************************************
 */
static long varexists(PARSER_CONTROLL *pc,const char*name)
{

  tsNAMSCAN namescan;

  return getNameScan(&(pc->table),name
		     ,(long) (VTYPE_ARRAY | VTYPE_SCALAR 
			      | VTYPE_STRUCT | VTYPE_STRUCTARRAY)
		     ,0L,&namescan,0L);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long read_band_file(const char *fname,tsBANDDATA *bd)
{
  long i1,i2,n,idum,ik,ms,in; 
  double k,e;
  char buf[BUFFER_MAX_STRING+1];
  FILE *FHI;

  bd->nk=0;
  bd->nband=0;
  bd->mspin=1;
  
  txtlen(fname,&i1,&i2);
  (void)strncpy(buf,fname+i1,i2-i1+1);
  buf[i2-i1+1]='\0';
  (void)printf("Read file:%s:\n",buf);


  if  ( (FHI=fopen(buf,"r"))==NULL ){
    (void)printf("file \'%s\' not found: ABORT!\n",buf);
    goto error;
  }
  if  ( (fgets(buf,BUFFER_MAX_STRING,FHI))==NULL ){
    (void)printf("can not read file \'%s\' : ABORT!\n",buf);
    goto error;
  }else{
    rewind(FHI);
  }

  if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
  n=sscanf(buf,"%*s%ld%lg%ld%ld%ld%ld",&idum,&(bd->ef),&(bd->nk),&idum
	   ,&(bd->nband),&(bd->mspin));

  /* todo: conditional read additional info */
  /*     (void)printf("n:%ld\n",n); */
  /*     (void)printf("mspin:%ld\n",bd->mspin); */
  /*     (void)printf("ef:%f\n",bd->ef); */
  
  
  bd->emin=1000.;
  bd->emax=-1000.;

  /*   alloc arrays */
  bd->kdist=(double*)malloc(sizeof(double)*bd->nk);
  bd->erg=(double*)malloc(sizeof(double)*bd->nk*bd->nband*bd->mspin);
  
  
  /* read kdist,erg data */
  for(ik=0;ik<bd->nk;ik++){
    if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    for(ms=0;ms<bd->mspin;ms++){
      idum=fscanf(FHI,"%lf",&k);
      bd->kdist[ik]=k;
      for(in=0;in<bd->nband;in++){
	idum=fscanf(FHI,"%lf",&e);
	/* 	(void)printf("%f %f %f\n",k,e,wei); */
	n=ik+in*bd->nk+ms*bd->nk*bd->nband;
	bd->erg[n]=e;
	bd->emin=min(bd->emin,bd->erg[n]);
	bd->emax=max(bd->emax,bd->erg[n]);
      }
      if(!fgets(buf,BUFFER_MAX_STRING,FHI))goto error;
    }
  }
  (void)fclose(FHI);


  

  return 1;

 error:;
  exit(2);
  return 0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void ps_plot_bands(const tsBANDDATA *bd,FILE *FHO
			  ,const VIEWPORT *vp,const WORLD *world,long coltyp
			  )
{
  long in,n;
  for(in=0;in<bd->nband;in++){
    if(bd->mspin==2){
      n=in*bd->nk+0*bd->nk*bd->nband;
      drawxy(FHO,vp,world,bd->kdist,&(bd->erg[n]),bd->nk
	     ,RGBColors[coltyp],0,2,landscape);
      n=in*bd->nk+1*bd->nk*bd->nband;
      drawxy(FHO,vp,world,bd->kdist,&(bd->erg[n]),bd->nk
	     ,RGBColors[coltyp],2,1,landscape);
      
    }else{
      n=in*bd->nk+0*bd->nk*bd->nband;
      drawxy(FHO,vp,world,bd->kdist,&(bd->erg[n]),bd->nk
	     ,RGBColors[coltyp],0,1,landscape);
      
    }
  }
  return;
}
