/*  postscript a line */
/*  by Klaus Koepernik Mar. 1999 */
/*   */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pline.h"
#include "main.h"
#include "tools.h"



/* ps line */

void pline(FILE *FH,long x0,long y0,long x1,long y1
      ,double width,char *dash,double grayscale)
{

  char *fmt="\
v\n\
%f g [ %s ] 0 setdash\n\
 n %ld %ld m %ld %ld l %g w s\n\
r\n";


  (void)fprintf(FH,fmt,grayscale,dash,x0,y0,x1,y1,width);

}

/* transform world into viewport */

void wctovc(const VIEWPORT *vp,const WORLD *w,double x,double y,long *a,long *b
       ,long typ)
{
  if (typ==landscape){
    *b=((vp->y1-vp->y0)/(w->k1-w->k0)*(x-w->k0)+vp->y0);
    *a=((vp->x1-vp->x0)/(w->e0-w->e1)*(y-w->e1)+vp->x0);
  }else{
    (void)printf("wctovc: no portraet implemented\n");
    exit(1);
  }
}

/* C*********************************************************************** */
/* draw a function f(x) with point radius r(x) */
/* by Klaus Koepernik Mar. 1999 */
/*  */
/* C*********************************************************************** */

void drawxyr(FILE *FH,VIEWPORT *vp,WORLD *w,double *x,double *f,double *wr
	     ,long n,const double color[3],long typ,long filled)
{

  long a,b,i;
  double xx,ff,tol=0.001;
  
  (void)fprintf(FH,"v\n"); 

  (void)fprintf(FH,"%f %f %f q \n",color[0],color[1],color[2]);
  for(i=0;i<n;i++){
    xx=x[i];
    ff=f[i];
    wctovc(vp,w,xx,ff,&a,&b,typ);
    if(inclip(a,b,vp)  && fabs(wr[i])>tol){
      if(filled){
	(void)fprintf(FH,"n %ld %ld %g 0 360 arc f s\n",a,b,wr[i]);
      }else{
	(void)fprintf(FH,"%f %f %f q \n",1.,1.,1.);
	(void)fprintf(FH,"n %ld %ld %g 0 360 arc f s\n",a,b,wr[i]);
	(void)fprintf(FH,"%f %f %f q \n",color[0],color[1],color[2]);
	(void)fprintf(FH,"n %ld %ld %g 0 360 arc  s\n",a,b,wr[i]);
      }
    }
  }
  (void)fprintf(FH,"r\n"); 
  
}

/* is point a,b inside rectangle  */
/* by Klaus Koepernik Mar. 1999 */
/*  */

long inclip(long a,long b,VIEWPORT *vp)
{
  long i;
  i=a>=vp->x0 && a<=vp->x1 && b>=vp->y0 && b<=vp->y1;
  return i;
}



/* C*********************************************************************** */
/*  postscript formated text */
/*  special characters are:  $~ : next letter is greek */
/*                           $i : switch on italic */
/*                           $n : switch on normal font */
/*                           $^ : go one superscript leve up */
/*                           $_ : go one subscript level down */
/*                           $. : go to normal level */
/*                           $$ : a single $$ */
/*                           $ not followed by ~^_. : print a single */
/*  */
    



void getrawtext(const char *txt,char *tmp,long *length)
{

  long l0,l1,j,i;

  *tmp='\0';
  txtlen(txt,&l0,&l1);

  j=0;
  i=l0;
  while(i<=l1) { 
    if(!strncmp(txt+i,"$_",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$.",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$~",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$^",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$i",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$n",2)){
      i=i+1;
    }else if(!strncmp(txt+i,"$$",2)){
      i=i+1;
      tmp[j]='$';
      j=j+1;
    }else if(*(txt+i)=='\\'){
      i=i+0;
      tmp[j++]='\\';
      tmp[j++]='\\';
    }else if(*(txt+i)=='('){
      tmp[j++]='\\';
      tmp[j++]='(';
    }else if(*(txt+i)==')'){
      tmp[j++]='\\';
      tmp[j++]=')';
    }else{
      tmp[j]=*(txt+i);
      j=j+1;
      tmp[j]='\0';
    } 
    i=i+1;
  }
  tmp[j]='\0';
  *length=j;

}

/************************************************************************
 *
 ************************************************************************
 */
void formtext(FILE *FH,long x,long y,char *txt,long angle,long fontsize
	      ,long just,char *command,double factor) 
{
  long i,l0,l1,length,size,msize,dy;
  char tmp[3000];
  char font[3000];

  size=fontsize;
  msize=size;
  
  txtlen(txt,&l0,&l1);

  getrawtext(txt,tmp,&length);

  /* (void)printf("tmp:%s\n",tmp);  */

  (void)fprintf(FH,"v\n");
  (void)fprintf(FH,"/Helvetica-ISOLatin1 findfont %ld scalefont setfont \n",size);
  (void)fprintf(FH,"%ld %ld translate\n",x,y);
  (void)fprintf(FH,"%ld  rotate 0 0 m\n",angle);
  

  (void)fprintf(FH,"%s",command);
  if(just==center){
    (void)fprintf(FH,"(%s) CS\n",tmp);
  }else if(just==block){
    ;
  }else if(just==right){
    (void)fprintf(FH,"(%s) RJ\n",tmp);
  }

  dy=0;
  i=l0;
  strcpy(font,"/Helvetica-ISOLatin1");
  while(i<=l1){
    if(!strncmp(txt+i,"$~",2)){
      (void)fprintf(FH,"/Symbol findfont %ld scalefont setfont\n",msize);
      i=i+2;
      (void)fprintf(FH,"(%c) show\n",*(txt+i));
      i=i+1;
      
    }else if(!strncmp(txt+i,"$i",2)){
      strcpy(font,"/Helvetica-Oblique-ISOLatin1");
      i=i+2;
    }else if(!strncmp(txt+i,"$n",2)){
      strcpy(font,"/Helvetica-ISOLatin1");
      i=i+2;
    }else if(!strncmp(txt+i,"$_",2)){
      (void)fprintf(FH,"0 %ld rm\n",-(long)(msize*.3));
      dy=dy-(long)(msize*.3);
      msize=msize*factor;
      i=i+2;
    }else if(!strncmp(txt+i,"$^",2)){
      (void)fprintf(FH,"0 %ld rm\n", (long)(msize*.6));
      dy=dy+ (long)(msize*.6);
      i=i+2;
      msize=msize*factor;
    }else if(!strncmp(txt+i,"$$",2)){
      (void)fprintf(FH,"%s findfont %ld scalefont setfont\n",font,msize);
      (void)fprintf(FH,"($) show\n");
      i=i+2;
    }else if(!strncmp(txt+i,"$.",2)){
      (void)fprintf(FH,"0 %ld rm\n", -dy);
      dy=0;
      i=i+2;
      msize=size;
    }else if(!strncmp(txt+i,"\\'",2)){
      (void)fprintf(FH,"%s findfont %ld scalefont setfont\n",font,msize);
      (void)fprintf(FH,"(') show\n");
      i=i+2;
    }else if(!strncmp(txt+i,"\\",1)){
      (void)fprintf(FH,"%s findfont %ld scalefont setfont\n",font,msize);
      (void)fprintf(FH,"(\\\\)show\n");
      i=i+1;
    }else if(*(txt+i)=='(' || *(txt+i)==')'){
      (void)fprintf(FH,"%s findfont %ld scalefont setfont\n",font,msize);
      (void)fprintf(FH,"(\\%c)show\n",*(txt+i));
      i=i+1;
    }else{
      (void)fprintf(FH,"%s findfont %ld scalefont setfont\n",font,msize);
      (void)fprintf(FH,"(%c) show\n",*(txt+i));
      i=i+1;
    }


  }
  (void)fprintf(FH,"r\n");
}

/*  calc len of character end is tab space or null */
/*  by Klaus Koepernik Mar. 1999 */
/*   */

void txtlen(const char *txt,long *i0,long *i1)
{
  long i,n;
  n=strlen(txt);
  for(i=n-1;i>-1;i--){	
    if((*(txt+i) !=' ') &&  *(txt+i)!='\t' && *(txt+i) !='\0') break;
  }	
  
  *i1=i;
  for(i=0;i<*i1;i++){
    if(*(txt+i) != ' ' && *(txt+i) != '\t' && *(txt+i) !='\0')break;
  }
  *i0=i;
}



/* C*********************************************************************** */
/*  split real number in mantisse and exponent */
/*  by Klaus Koepernik Mar. 1999 */
/*   */
/* C*********************************************************************** */

void split10(double d,double *m,long *e)
{
  double s;
  if(d==0.){
    *m=0.;
    *e=0;
    return;
  }
  s=ffsig(1.0,d);
  d=fabs(d);
  *e=(long)(log(d)/log(10.));
  /* (void)printf("%f %f %ld\n",log(d),log(10.),*e);  */
  if(d<1.){*e=*e-1;}
  *m=d/pow(10.,(double)*e);
  /* (void)printf("%f %f \n",*m,pow(10.,(double)*e));  */
  if( *m<1. || *m>=10. ){
    (void)printf("SPLIT10: mantisse out of range x: %g  = %f *10** %ld\n",d,*m,*e);
    exit(1);
  }
  *m=*m*s;
}

/* C*********************************************************************** */
/*  draw autotics */
/*  by Klaus Koepernik Mar. 1999 */
/*   */

void autotics(FILE *FH,VIEWPORT *vp,WORLD *world,char *yaxis_title,long typ
	      ,double ticmark_scale,double ticmark_subscript_scale
	      ,double title_scale,double title_subscript_scale
	      ,double ticlinewidth
	      ,long display_yticmarks,long yticmarkdigits
	      ,double yaxis_offsetx
	      ,double yaxis_offsety
	      ,double ylabel_offsetx
	      )
{
  long i,j,i0,j0,m,ninterv,l0,l1,labelshift,ytitelxpos
    ,ytitelshift,exponent,lengthmax,nmintics,intvtyp=0,facttyp=0;
  double tol,intervallength,mantisse,a,b,a0,mjtics;
  char txt[100],frm[100],txtmax[100],command[3000];
  long textheight=60;

  
  long intervall[3]={1,2,5};
  long intervallmin[3]={5,4,5};
  
  
  intervallength=world->e1-world->e0;
  m=1000;
  split10(intervallength,&mantisse,&exponent);
  /* (void)printf("interval: %f = %f * 10 ^ %ld\n",intervallength,mantisse,exponent);  */
  /* all subintervalls lengths (1,2,5) */
  for(i=0;i<3;i++){
    /* all factors 1,10,100 */
    for(j=0;j<3;j++){
      /* number of subintervalls */
      ninterv=(long)(mantisse/((double)intervall[i])*pow(10.,(double)j));
      /* (void)printf("%ld,%ld: %ld\n",i,j,ninterv);  */
      /* no more than 20 and not less than 1 */
      if(ninterv>0 &&  ninterv<20){
	/* best is 5 subdivisions */
	if(labs(5-ninterv)<m){

	  m=labs(5-ninterv);
	  intvtyp=i;
	  facttyp=j;
 	  /* (void)printf("%ld %ld %ld\n",m,intvtyp,facttyp);  */
	}
      }
    }
  }
  /* (void)printf("%ld %ld %ld\n",m,intvtyp,facttyp);  */
/*   (void)printf("%ld:%ld\n",intvtyp,facttyp);  */
  /* some problems */
  if(m==1000){
    (void)printf("TICS: auto-subdivision failed! Shame on the author.\n");
    exit(1);
  }

  


  ninterv=(long)(mantisse/((double)intervall[intvtyp])*pow(10.,(double)facttyp));
  mjtics=((double)intervall[intvtyp])*pow(10.,(double)(exponent-facttyp));

  
  lengthmax=0;
  labelshift=(vp->x1-vp->x0)*(.03+ylabel_offsetx);
  //printf("labelshift %ld %g",labelshift,ylabel_offsetx);
  intervallength=world->e1-world->e0;
  tol=1.e-3;



  a0= ((long)(world->e0/mjtics ))*mjtics;
  
  /* c      print *,'e0 ',e0 */
  /* c      print *,'d  ',mjtics */
  /* c      print *,'a0 ',a0 */
  
  for(i=-ninterv;i<=ninterv*2;i++){
    a=a0+mjtics*(i-1);
    if(a>=world->e0-tol*intervallength && a<=world->e1+tol*intervallength){
      wctovc(vp,world,0.,a,&i0,&j0,typ);
      j0=(vp->x1-vp->x0)*.03;
      pline(FH,i0,vp->y0,i0,vp->y0+j0,ticlinewidth,"",0.);

      long numdigits=max(0,-(exponent-facttyp)+1);
      if(yticmarkdigits>=0){
	numdigits=min(max(numdigits,yticmarkdigits),yticmarkdigits);
      }
      (void)sprintf(frm,"%%-#30.%ldf",numdigits);
      /*       (void)printf("%s:%ld %ld\n",frm,max(0,-exponent+1),exponent); */
      (void)sprintf(txt,frm,a);

      txtlen(txt,&l0,&l1) ;
/*       (void)printf("txt:%s:%ld\n",txt,l1-l0+1);  */
      if(l1-l0+1>lengthmax){
	lengthmax=l1-l0+1;
	(void)strncpy(txtmax,txt,lengthmax) ;
	txtmax[lengthmax]='\0';
/* 	(void)printf("txtmax:%s:%ld\n",txtmax,lengthmax);  */
      }



      if(display_yticmarks){
	(void)sprintf(command,"%g %g scale\n",ticmark_scale,ticmark_scale);
	
	formtext(FH,i0+textheight*0.4,vp->y0-labelshift,txt,90,textheight,right,command,ticmark_subscript_scale);
      }

    }
    nmintics=intervallmin[intvtyp];
    for(j=1;j<nmintics;j++){
      b=a+j*mjtics/((double)nmintics);
      if(b>=world->e0-tol*intervallength &&  b<=world->e1+tol*intervallength){
	wctovc(vp,world,0.,b,&i0,&j0,typ);
	j0=(vp->x1-vp->x0)*.015;
	pline(FH,i0,vp->y0,i0,vp->y0+j0,ticlinewidth,"",0.);
      }
    }
  }
  /* y-axis  titel */
  ytitelxpos=((vp->x1+vp->x0)*(.5+yaxis_offsety));
  ytitelshift=(long)((vp->y1-vp->y0)*(.02+yaxis_offsetx));

  if(!display_yticmarks){
    txtmax[1]='x';
    txtmax[0]='\0';
  }

  (void)sprintf(command,"%g %g scale\n(%s) stringwidth pop  0 x rm\n"
		,title_scale,title_scale,txtmax);

  formtext(FH,ytitelxpos,vp->y0-labelshift-ytitelshift,yaxis_title,180,textheight
	   ,center,command,title_subscript_scale);
}




/* C*********************************************************************** */
/*  postscript formated legend */
/*  */
/*  by Klaus Koepernik mar 1999 */
/*  */
/* C*********************************************************************** */

void legend(FILE *FH,VIEWPORT *vp,long x0,long y0,long nleg,char **leg
	    ,const double color[][3],long colmax,long mspin
	    ,double leg_scale
	    ,double subscript_scale
	    ,double framelinewidth,long display_legendframe
	    ,double vertspacing)
{

  long textheight,i0,i1,i,linelen,coltyp,length;
  char buf[400];
  char buf1[400];
  char tmp[4000];
  
  textheight=60;
  linelen=150;
  coltyp=0;
  
/*   (void)printf("l:%s\n",leg[0]); */
  (void)fprintf(FH,"v\n");
  (void)fprintf(FH,"%ld %ld translate 90 rotate\n",x0,y0);
  (void)fprintf(FH,"%g %g scale\n",leg_scale,leg_scale);
  (void)fprintf(FH,"/Helvetica-ISOLatin1 findfont %ld scalefont setfont\n"
	  ,textheight);
  (void)fprintf(FH,"0. g\n");
  for(i=0;i<nleg;i++){

    getrawtext(leg[i],tmp,&length);
    txtlen(tmp,&i0,&i1);    

    (void)strncpy(buf,tmp+i0,i1-i0+1);
    buf[i1-i0+1]='\0';

    if(mspin==2){
      (void)fprintf(FH,"(%s m)",buf);
    }else{
      (void)fprintf(FH,"(%s)",buf);
    }
  }
  (void)fprintf(FH," 0 %ld SL\n",nleg);

  if(display_legendframe)
    (void)fprintf(FH,"n 0 0 m %ld 0 rl dup 0 rl 0 %f rl neg 0 rl %ld neg 0 rl"
		  " c v 1 1 1 q f r %g %g div w s\n",textheight*3+linelen
		  ,-(((vertspacing*nleg*mspin+.7)*textheight)),textheight*3+linelen
		  ,framelinewidth,leg_scale);
      
  (void)fprintf(FH,"v\n");
  (void)fprintf(FH,"0 0 m\n");
      
      
  for(i=0;i<nleg;i++){
    (void)fprintf(FH,"%ld %ld m\n",textheight
	    ,-(long)(vertspacing*textheight*(i*mspin+1)-textheight*.4));

    
    (void)fprintf(FH,"v [ ] 0 setdash %ld w %f %f %f q %ld 0 rl s r\n"
		  ,mspin,color[coltyp][0],color[coltyp][1],color[coltyp][2]
		  ,linelen);

    txtlen(leg[i],&i0,&i1);
    (void)strncpy(buf1,leg[i]+i0,i1-i0+1);
    buf1[i1-i0+1]='\0';
    if(mspin==2){
      (void)sprintf(buf,"%s $.$~",buf1);
      formtext(FH,textheight*2+linelen
	       ,-(long)(vertspacing*textheight*(i*mspin+1)),buf,0,60,0,"",subscript_scale);
    }else{
      formtext(FH,textheight*2+linelen
	       ,-(long)(vertspacing*textheight*(i*mspin+1)),buf1,0,60,0
	       ,"",subscript_scale);
    }
    if(mspin==2){
      /* spin polarized minority */
      (void)fprintf(FH,"%ld %ld m\n",textheight
	      ,-(long)(vertspacing*textheight*(i+1)*mspin-textheight*.4));
      
      (void)fprintf(FH,"v [ 4 8 ] 0 setdash 1 w %f %f %f q %ld 0 rl s r\n"
		    ,color[coltyp][0],color[coltyp][1],color[coltyp][2]
		    ,linelen);

      (void)sprintf(buf,"%s $.$~",buf1);
      formtext(FH,textheight*2+linelen
	       ,-(long)(vertspacing*textheight*(i+1)*mspin),buf,0,60,0,"",subscript_scale);
         
    }

    coltyp=(coltyp+1)%(colmax-1);

  }
  (void)fprintf(FH,"r\n");
  
  (void)fprintf(FH,"r\n");
}







 


/* C*********************************************************************** */
/*  draw function f(x) */
/*  by Klaus Koepernik Mar. 1999 */
/*   */

void drawxy(FILE *FH,const VIEWPORT *vp,const WORLD *w
	    ,const double *x,const double *f,long n,const double *color
	    ,long linetyp,long linewidth,long typ)
{
  long i,a0,b0,a1,b1,p0x,p0y,p1x,p1y,xold,yold,last ;

  last=0;
  xold=0;
  yold=0;
  (void)fprintf(FH,"v n");
  if(linetyp==0) (void)fprintf(FH," [ ] 0 setdash");
  if(linetyp==1) (void)fprintf(FH," [ 40 20] 0 setdash");
  if(linetyp==2) (void)fprintf(FH," [ 4 8] 0 setdash");
  (void)fprintf(FH," %f %f %f q",color[0],color[1],color[2]);
  (void)fprintf(FH," %ld w\n",linewidth);
  for(i=0;i<n-1;i++){
    wctovc(vp,w,x[i],f[i],&a0,&b0,typ);
    wctovc(vp,w,x[i+1],f[i+1],&a1,&b1,typ);
    if(clipline(vp,a0,b0,a1,b1,&p0x,&p0y,&p1x,&p1y)){
      if(!(last && p0x==xold && p0y==yold )){
	(void)fprintf(FH,"%ld %ld m\n",p0x,p0y);
      }
      (void)fprintf(FH,"%ld %ld l\n",p1x,p1y);
      xold=p1x;
      yold=p1y;
      last=1;
    }else{
      last=0;
    }
  }
  (void)fprintf(FH,"s r\n");
}


/* C*********************************************************************** */
/*  clip a line  into a rectangle */
/*  by Klaus Koepernik Mar. 1999 */
/*   */

long clipline(const VIEWPORT *vp,long a0,long b0,long a1,long b1,long *p0x,long *p0y
	     ,long *p1x,long *p1y)
{
  long i,clip;
  double da,db,ox,oy,px,py,n[4][3],r,s,lmax,lmin;

  clip=1;
  /* D=p2-p1 */
  da=a1-a0;
  db=b1-b0;
  /* O */
  ox=(vp->x1+vp->x0)*.5;
  oy=(vp->y1+vp->y0)*.5;
  /* P1-O */
  px=a0-ox;
  py=b0-oy;
  /* n1<- x0y0,x0y1 */
  hess(vp->x0-ox,vp->y0-oy,vp->x0-ox,vp->y1-oy,&(n[0][0]));
  hess(vp->x0-ox,vp->y1-oy,vp->x1-ox,vp->y1-oy,&(n[1][0]));
  hess(vp->x1-ox,vp->y1-oy,vp->x1-ox,vp->y0-oy,&(n[2][0]));
  hess(vp->x1-ox,vp->y0-oy,vp->x0-ox,vp->y0-oy,&(n[3][0]));
  lmax=1.e30;
  lmin=-1.e30;
  for(i=0;i<4;i++){
    r=n[i][2]-n[i][0]*px-n[i][1]*py;
    s=n[i][0]*da+n[i][1]*db;
    if(s==0.){
      if(r<0.){
	clip=0;
      }
    }
    if(s>0.){
      lmax=min(lmax,r/s);
    }
    if(s<0.){
      lmin=max(lmin,r/s);
    }
  }

  lmax=min(lmax,1.);
  lmin=max(lmin,0.);

  if(lmax>lmin){
    *p1x=round(a0+lmax*da);
    *p1y=round(b0+lmax*db);
    *p0x=round(a0+lmin*da);
    *p0y=round(b0+lmin*db);
    
  }else{
    clip=0;
  }
  
  return clip;
}

/* C*********************************************************************** */
/*   */
/*  by Klaus Koepernik Mar. 1999 */
/*   */

void hess(double x0,double y0,double x1,double y1,double *n)
{
  double d,tol;

  tol=1.e-12;
  n[1]=(x1-x0);
  n[0]=-(y1-y0);
  d=sqrt(n[0]*n[0]+n[1]*n[1]);
  n[0]=n[0]/d;
  n[1]=n[1]/d;
  n[2]=n[0]*x0+n[1]*y0;
  if(fabs(n[0]*x1+n[1]*y1-n[2])>tol){
    (void)printf("HESS: plane does not fit both points\n");
    exit(1);
  }
}
 


/* C*********************************************************************** */
/*  draw a function f(x,y) with  grayscale */
/*  by Klaus Koepernik Mar. 1999 */
/*   */
/* C*********************************************************************** */

void drawxyd(FILE *FH,VIEWPORT *vp,WORLD *w,double *x,double *y
	     ,long *f,long nk,long ne,const double color[3],long typ)
{

  long a0,b0,a1,b1,ik,in,n,ff;
  double g,gray,rd;
  


  for(ik=0;ik<nk-1;ik++){
    for(in=0;in<ne-1;in++){
      wctovc(vp,w,x[ik],y[in],&a0,&b0,typ);
      wctovc(vp,w,x[ik+1],y[in+1],&a1,&b1,typ);
      a1--;
      b1--;
      n=ik+nk*in;
      ff=(f[n]+f[n+1]+f[n+nk]+f[n+nk+1]);
      gray=ff/4./250.;

      g=(1.-gray);
      if(inclip(a0,b0,vp) && inclip(a1,b1,vp)  && ff>1){
	rd=g;

	(void)fprintf(FH,"v %f g n %ld %ld m %ld %ld l %ld %ld l %ld %ld l c f r\n"
		,rd,a0,b0,a1,b0,a1,b1,a0,b1);
      }
    }
  }
  
}




/************************************************************************
 * bandweights legend
 ************************************************************************
 */
void bwlegend(FILE *FH,VIEWPORT *vp,long x0,long y0,long nleg,char **leg
	      ,const double color[][3],const long *coltyp,const long *orb_fill
	      ,double dr,double leg_scale
	      ,double subscript_scale,double framelinewidth
	      ,long display_legendframe,double vertspacing)
{

  long textheight,i0,i1,i,linelen,length;
  char buf[400];
  char tmp[4000];

  
  textheight=60;
  /* linelen=150; */
  linelen=10;
  
/*   (void)printf("l:%s\n",leg[0]); */
  (void)fprintf(FH,"v\n");
  (void)fprintf(FH,"%ld %ld translate 90 rotate\n",x0,y0);
  (void)fprintf(FH,"%g %g scale\n",leg_scale,leg_scale);
  (void)fprintf(FH,"/Helvetica-ISOLatin1 findfont %ld scalefont setfont\n"
	  ,textheight);
  (void)fprintf(FH,"0. g\n");
  for(i=0;i<nleg;i++){
    
    getrawtext(leg[i],tmp,&length);
    txtlen(tmp,&i0,&i1);    
    
    (void)strncpy(buf,tmp+i0,i1-i0+1);
    buf[i1-i0+1]='\0';
    (void)fprintf(FH,"(%s)",buf);
  }
  (void)fprintf(FH," 0 %ld SL\n",nleg);

      
  if(display_legendframe)
    (void)fprintf(FH,"n 0 0 m %ld 0 rl dup 0 rl 0 %f rl neg 0 rl %ld neg 0 rl"
		  " c v 1 1 1 q f r %g %g div w s\n",textheight*3+linelen
		  ,-(((vertspacing*nleg+.7)*textheight)),textheight*3+linelen
		  ,framelinewidth,leg_scale);
  
  (void)fprintf(FH,"v\n");
  (void)fprintf(FH,"0 0 m\n");
      
      
  for(i=0;i<nleg;i++){
    long y=-(long)(vertspacing*textheight*(i+1)-textheight*.4);
    (void)fprintf(FH,"%ld %ld m\n",textheight
		  ,y);

    if(orb_fill[i]){
      (void)fprintf(FH,"v %f %f %f q %ld %ld 2 copy m %ld 0 360 arc f r\n"
		    ,color[coltyp[i]][0],color[coltyp[i]][1],color[coltyp[i]][2]
		    ,linelen/2+textheight
		    ,y
		    ,(long)(dr+.5));
    }else{

      (void)fprintf(FH,"v %f %f %f q %ld %ld 2 copy m %ld 0 360 arc f r\n"
		    ,1.,1.,1.
		    ,linelen/2+textheight
		    ,y
		    ,(long)(dr+.5));
      (void)fprintf(FH,"v %f %f %f q n %ld %ld %ld 0 360 arc s r\n"
		    ,color[coltyp[i]][0],color[coltyp[i]][1],color[coltyp[i]][2]
		    ,linelen/2+textheight
		    ,y
		    ,(long)(dr+.5));
      
    }
    txtlen(leg[i],&i0,&i1);
    (void)strncpy(buf,leg[i]+i0,i1-i0+1);
    buf[i1-i0+1]='\0';
    formtext(FH,textheight*2+linelen
	     ,-(long)(vertspacing*textheight*(i+1)),buf,0,60,0,"",subscript_scale);


  }
  (void)fprintf(FH,"r\n");
  
  (void)fprintf(FH,"r\n");
}
