/*************************************************************************
 * @(#)CalcDriver.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Dec 2011
 *************************************************************************
 */

#include <iostream>
#include <sstream>
#include "CalcDriver.hh"



const struct Functions::tsfuncdef Functions::funcs[]={
  {1,Sqrt ,sqrt ,NULL },
  {1,Cos  ,cos  ,NULL },
  {1,Sin  ,sin  ,NULL },
  {1,Tan  ,tan  ,NULL },
  {1,Cot  ,Functions::cot  ,NULL },
  {1,Acos ,acos ,NULL },
  {1,Asin ,asin ,NULL },
  {1,Atan ,atan ,NULL },
  {1,Abs  ,fabs ,NULL },
  {1,Exp  ,exp  ,NULL },
  {1,Log  ,log  ,NULL },
  {2,Atan2,NULL ,atan2 },
  {2,Min  ,NULL ,Functions::dmin2 },
  {2,Max  ,NULL ,Functions::dmax2 },
  {2,Sign ,NULL ,Functions::sign2 },
  {1,Theta ,Functions::theta ,NULL }
};
const int Functions::MAXFUNS=sizeof(Functions::funcs)/
  sizeof(const struct Functions::tsfuncdef);


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::dmin2(double a, double b)
{
  return (a<b?a:b);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::dmax2(double a, double b)
{
  return (a>b?a:b);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::pow2(double a, double b)
{
  return ::pow(a,b);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::cot(double a)
{
  return 1./tan(a);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::theta(double a)
{
  return ((a>=0. ? 1.:0.));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Functions::sign2(double a,double b)
{
  return (fabs(a)*(b>=0. ? 1.:-1.) );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CalcDriver::error(const CALC::location& l, const std::string& m)
{
  if( m.length()!=0){
    std::ostringstream oss (std::ostringstream::out);
    oss << l << ": " << m << std::endl;
    calcerrormsg+=oss.str().c_str();
  }else{
    calcerrormsg+="error: unspecified\n";
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CalcDriver::error (const std::string& m)
{
  calcerrormsg+=Sprintf("%s\n",m.c_str());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int CalcDriver::parse(VarNode *vtable,const char *bufferorfilename,FILE *FH)
{
  pvtable=vtable;
  pFH=FH;

  if(pFH){
    pfilename=bufferorfilename;
  }else{
    pbuffer=bufferorfilename;
  }
  calcerrormsg="";
  scan_begin();
  CALC::CalcParser parser(*this,pscanner);
  parser.set_debug_level(ptraceparsing);
  int ret=parser.parse();
  scan_end();
  
  
  return ret;
}



/*EOF*/ 
