/* @(#)CalcDriver.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _CALCDRIVER_H
#define _CALCDRIVER_H 1

#include <string>
#include "CalcParse.hpp"
#include "CalcParseDecl.hh"
#include "VarNode.hh"
#include "FString.hh"
#include "CalcParseInternal.hh"

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* ***********************************************************************
 *
 * ***********************************************************************
 */
struct CalcLexerState{
  String stringlexbuffer;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CalcDriver{
 public:

  CalcDriver():presult(0.),pdebug(false){};
  ~CalcDriver(){};

  std::string &file(){return pfilename;};

  // Handling the scanner.
  void scan_begin ();
  void scan_end ();

  const String &errorMsg()const{return calcerrormsg;};

  bool traceScanning()const{return ptracescanning;};
  void setTraceScanning(bool t){ptracescanning=t;};

  bool traceParsing()const{return ptraceparsing;};
  void setTraceParsing(bool t){ptraceparsing=t;};

  // Run the parser.  Return 0 on success.
  int parse(VarNode *vtable,const char *bufferorfilename,FILE *FH=0);

  // Error handling.
  void error(const CALC::location& l, const std::string& m);
  void error(const std::string& m);

  VarData result()const{return presult;};
  void setResult(const VarData *r){presult=*r;};
  bool debug()const{return pdebug;};
  VarNode *vtable(){return pvtable;};
  
 private:
  bool ptracescanning;
  bool ptraceparsing;
  void *pscanner;
  FILE *pFH;
  const char *pbuffer;
  std::string pfilename;
  VarData presult;
  bool pdebug;
  String calcerrormsg;
  CalcLexerState plexerstate;
  YY_BUFFER_STATE lexstate;
  VarNode *pvtable;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _CALCDRIVER_H */

