/*************************************************************************
 * @(#)VarNode.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 Apr 2009
 *************************************************************************
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "VarNode.hh"
#include "constants.h"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
VarNode::VarNode()
  :ptype(Root),pname("root"),pnext(this),pprev(this)
{
  //printf("live root %p %s %g\n",(void*)this,pname(),pvalue);
  defineConst("pi",3.14159265358979323844);
  defineConst("hatoev",constant_hatoev);
  defineConst("aBtoAng",constant_aBtoAng);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VarNode::~VarNode()
{
  //printf("die %p %s %g\n",(void*)this,pname(),pvalue);
  clear(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void VarNode::clear(bool clearconstants)
{
  if(ptype!=Root) return ;
  //printf("clear %p %s %g\n",(void*)this,pname(),pvalue);

  VarNode *cur=this;
  while( cur->pnext != this ){
    VarNode *vn=cur->pnext;
    if((vn->ptype == Constant) && !clearconstants){
      cur=vn;
      continue;
    }
    cur->pnext=vn->pnext;
    vn->pnext->pprev=cur;
    delete vn;
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VarNode *VarNode::getNode(const String &name)
{
  VarNode *vn=this;

  while( (vn=vn->pnext) != this ){
    if( vn->pname==name ){
      return vn;
    }
  }  
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

VarNode *VarNode::append(const String &name,const VarData &value)
{
  //printf("append %p: %s %g\n",this,name(),value);
  //printf("next %p: prev %p\n",pnext,pprev);
  
  VarNode *vn=new VarNode(name,value);

  vn->pnext=this;
  vn->pprev=this->pprev;
  pprev->pnext=vn;
  pprev=vn;

  //printf("vn %p vn->next %p: vn->prev %p\n",vn,vn->pnext,vn->pprev);
  //printf("next %p: prev %p\n",pnext,pprev);

  vn->ptype=Variable;

  return vn;
  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */

String VarNode::printS()
{
  //printf("printS %p: %s %g\n",this,pname(),pvalue);
  //printf("next %p: prev %p\n",pnext,pprev);
  String out;
  VarNode *vn=this;
  while( (vn=vn->pnext) != this ){
    //printf("vn=%p next %p prev %p\n",vn,pnext,pprev);
    out+=Sprintf("%s=%s",vn->pname(),vn->pvalue.printS("%-30.16g")()).stripBlanks()+"\n";
  }
  return out;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const VarNode &v)
{
  const VarNode *vn=&v;
  while( (vn=vn->pnext) != &v ){
    dbg<<Sprintf("%s=%s",vn->pname(),vn->pvalue.printS("%-30.16g")()).stripBlanks()<<"\n";
  }
  return dbg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool VarNode::getValue(const String &name,VarData &value)
{
  VarNode *vn;

  if((vn=getNode(name))){
    value=vn->pvalue;
    return true;
  }
  return false;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VarNode *VarNode::define(const String &name,const VarData &value)
{
  VarNode *vn;

  if((vn=getNode(name))){
    if(vn->ptype==Variable){
      vn->pvalue=value;
      return vn;
    }else{
      return 0;
    }
  }  
  
  return append(name,value);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void VarNode::defineConst(const String &name,const VarData &value)
{
  VarNode *vn;
  //printf("defineConst %s %g\n",name(),value);
  if((vn=define(name,value))){
    vn->ptype=Constant;
  }
  
}
