/* @(#)VarNode.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef _VARNODES_H
#define _VARNODES_H 1

#include "FString.hh"
#include "MyDebug.hh"

class VarData{
public:
  enum VarDataType{VDReal=0,VDString=1};
    
  VarData():ptype(VDReal),pdvalue(0.),psvalue(""){}
  VarData(double d):ptype(VDReal),pdvalue(d),psvalue(""){}
  VarData(const String &s):ptype(VDString),pdvalue(0),psvalue(s){}
  
  VarDataType type()const{return ptype;}
  double valueD()const{return pdvalue;}

  
  const VarData &operator+=(const VarData &other){
    if(ptype==other.ptype){
      if(ptype==VDReal){
	pdvalue+=other.pdvalue;
      }else if(ptype==VDString){
	psvalue+=other.psvalue;
      }    
    }
    return *this;
  }

  const VarData &operator-=(const VarData &other){
    if(ptype==other.ptype){
      if(ptype==VDReal){
	pdvalue-=other.pdvalue;
      }    
    }
    return *this;
  }

  const VarData &operator/=(const VarData &other){
    if(ptype==other.ptype){
      if(ptype==VDReal){
	pdvalue/=other.pdvalue;
      }    
    }
    return *this;
  }
  const VarData &operator*=(const VarData &other){
    if(ptype==other.ptype){
      if(ptype==VDReal){
	pdvalue*=other.pdvalue;
      }    
    }
    return *this;
  }

  const VarData &operator-(){
    if(ptype==VDReal){
      pdvalue=-pdvalue;
    }
    return *this;
  }

  String printS(const char *fmt)const{
    if(ptype==VDReal){
      return Sprintf(fmt,pdvalue);
    }else{
      return psvalue;
    }
  }
private:
  VarDataType ptype;
  double pdvalue;
  String psvalue;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class VarNode{
  enum Type{Root=0,Constant=1,Variable=2};
 public:
  //create a root node
  VarNode();
 private:
  // create a variable node
  VarNode(const String &name,const VarData &value)
    :ptype(Constant),pname(name),pvalue(value),pnext(this),pprev(this){};
 public:
  ~VarNode();

  const String &name()const{return pname;};
  VarData value()const{return pvalue;};

  void clear(bool clearconstants=false);
  bool getValue(const String &name,VarData &value);
  VarNode *define(const String &name,const VarData &value);

  String printS();


 private:
  VarNode *getNode(const String &name);
  VarNode *append(const String &name,const VarData &value);
  void defineConst(const String &name,const VarData &value);

 private:
  Type ptype;
  String pname;
  VarData pvalue;
  VarNode *pnext,*pprev;
  friend class VarNodeIterator;
  friend MyDebug operator<<(MyDebug dbg, const VarNode &vn);
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const VarNode &vn);

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class VarNodeIterator
{
 public:
  VarNodeIterator(VarNode *v):proot(v),pcurrent(v){};
    
  VarNode *next(){
    if((pcurrent=pcurrent->pnext)==proot)return 0;
    return pcurrent;
  };

 private:
  VarNode *proot;
  VarNode *pcurrent;
};

#ifdef	__cplusplus
extern "C" {
#endif



#ifdef	__cplusplus
}
#endif


#endif /* _VARNODES_H */

