/*************************************************************************
 * @(#)CIFData.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   07 Aug 2012
 *************************************************************************
 */

#include "CIFData.hh"

namespace CIF{

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFMetaDict::value(DictKey key)const
{
  String s;
  const DictEntry *entry=pentries.find(DictEntry(key));
  if(!entry) return s;

  
  return entry->pcifname;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFMetaDict::subValue(DictKey key,DictKey categ)const
{
  String s;
  const DictEntry *entry=pentries.find(DictEntry(key));
  const DictEntry *centry=pentries.find(DictEntry(categ));
  if(!entry) return s;
  if(!centry)return entry->pcifname;
  s=entry->pcifname;
  //myDebug()<<s<<centry->pcifname;
  s.replace(centry->pcifname,"",0,false);
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFMetaDict::setDefault()
{
  pentries
    <<DictEntry(Cell          ,"cell_")
    <<DictEntry(CellLength    ,"cell_length_")
    <<DictEntry(CellLengthA   ,"cell_length_a")
    <<DictEntry(CellLengthB   ,"cell_length_b")
    <<DictEntry(CellLengthC   ,"cell_length_c")
    <<DictEntry(CellAngle     ,"cell_angle_")
    <<DictEntry(CellAngleAlpha,"cell_angle_alpha")
    <<DictEntry(CellAngleBeta ,"cell_angle_beta")
    <<DictEntry(CellAngleGamma,"cell_angle_gamma")
    <<DictEntry(CellVolume    ,"cell_volume")
    <<DictEntry(CellFormulaUnitsZ ,"cell_formula_units_z")
    <<DictEntry(CellSpecialDetails ,"cell_special_details")

    <<DictEntry(ChemicalName ,"chemical_name_")
    <<DictEntry(ChemicalFormula ,"chemical_formula_")


    <<DictEntry(AtomSite              ,"atom_site_")
    <<DictEntry(AtomSiteLabel         ,"atom_site_label")
    <<DictEntry(AtomSiteTypeSymbol    ,"atom_site_type_symbol")
    <<DictEntry(AtomSiteFractX        ,"atom_site_fract_x")
    <<DictEntry(AtomSiteFractY        ,"atom_site_fract_y")
    <<DictEntry(AtomSiteFractZ        ,"atom_site_fract_z")
    <<DictEntry(AtomSiteOccupancy     ,"atom_site_occupancy")
    <<DictEntry(AtomSiteWyckoffSymbol ,"atom_site_wyckoff_symbol")
    <<DictEntry(AtomSiteSymmetry      ,"atom_site_symmetry_")
    <<DictEntry(AtomSiteSymmetryMultiplicity
		,"atom_site_symmetry_multiplicity")


    <<DictEntry(SpaceGroupITNumber      ,"space_group_it_number")
    <<DictEntry(SpaceGroupNameHMFull    ,"space_group_name_h-m_full")
    <<DictEntry(SpaceGroupNameHMAlt     ,"space_group_name_h-m_alt")
    <<DictEntry(SpaceGroupNameHall      ,"space_group_name_hall")
    <<DictEntry(SpaceGroupCrystalSystem ,"space_group_crystal_system")
    
    

    <<DictEntry(SymmetryIntTablesNumber   ,"symmetry_int_tables_number")
    <<DictEntry(SymmetryCellSetting       ,"symmetry_cell_setting")
    <<DictEntry(SymmetrySpaceGroupNameHM  ,"symmetry_space_group_name_h-m")
    <<DictEntry(SymmetrySpaceGroupNameHall,"symmetry_space_group_name_hall")
    <<DictEntry(SymmetryEquivPos          ,"symmetry_equiv_pos_")
    <<DictEntry(SymmetryEquivPosSiteId    ,"symmetry_equiv_pos_site_id")
    <<DictEntry(SymmetryEquivPosAsXyz     ,"symmetry_equiv_pos_as_xyz")

    <<DictEntry(SpaceGroupSymop           ,"space_group_symop_")
    <<DictEntry(SpaceGroupSymopId         ,"space_group_symop_id")
    <<DictEntry(SpaceGroupSymopOperationXYZ ,"space_group_symop_operation_xyz")

    ;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFData::CIFData()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::Value::asString()const
{
  if(ptype==Character){
    return Sprintf("%c",pchar);
  }else if(ptype==Long){
    return Sprintf("%ld",plong);
  }else if(ptype==Double){
    if(puncertainty.length()>0){
      return pstring+"("+puncertainty+")";
    }else{
      return pstring;
    }
  }else if(ptype==CharacterString){
    return pstring;
  }
  return String();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::Value::printS()const
{
  String s;
  if(ptype==Character){
    s.sprintf("%c",pchar);
  }else if(ptype==Long){
    s.sprintf("%ld",plong);
  }else if(ptype==Double){
    if(puncertainty.length()>0){
      s=pstring+"("+puncertainty+")";
    }else{
      s=pstring;
    }
  }else if(ptype==CharacterString){
    if(pdelim==Unquoted){
      s=pstring;
    }else if(pdelim==SingleQuotes){
      s.sprintf("'%s'",pstring());
    }else if(pdelim==DoubleQuotes){
      s.sprintf("\"%s\"",pstring());
    }else{
      s.sprintf("\n;%s\n;\n",pstring());
    }
  }

  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFData::Value::print()const
{
  if(ptype==Character){
    printf("%c",pchar);
  }else if(ptype==Long){
    printf("%ld",plong);
  }else if(ptype==Double){
    if(puncertainty.length()>0){
      printf("%s(%s)",pstring(),puncertainty());
    }else{
      printf("%s",pstring());
    }
  }else if(ptype==CharacterString){
    if(pdelim==Unquoted){
      printf("%s",pstring());
    }else if(pdelim==SingleQuotes){
      printf("'%s'",pstring());
    }else if(pdelim==DoubleQuotes){
      printf("\"%s\"",pstring());
    }else{
      printf("\n;%s\n;\n",pstring());
    }
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataTable::printS()const
{
  String s;
  s=Sprintf("loop_\n");
  for(long i=0;i<ptags.size();i++){
    s+=Sprintf("_%s\n",(*ptags.at(i))());
  }
  for(long i=0;i<pvalues.size();i++){
    const PtrArray<Value> *va=pvalues.at(i);
    for(long j=0;j<va->size();j++){
      s+=va->at(j)->printS();
      if(j<va->size()-1)
	s+=" ";
    }
    s+="\n";
  }
  
  return s;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFData::DataTable::print()const
{
  printf("loop_\n");
  for(long i=0;i<ptags.size();i++){
    printf("_%s\n",(*ptags.at(i))());
  }
  for(long i=0;i<pvalues.size();i++){
    const PtrArray<Value> *va=pvalues.at(i);
    for(long j=0;j<va->size();j++){
      va->at(j)->print();
      if(j<va->size()-1)
	printf(" ");
    }
    printf("\n");
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlockItem::printS()const
{
  String s;
  if(ptype==Single){
    String n(*pname);
    n.rightPadd(' ',33);
    s=Sprintf("_%s %s\n",n(),pvalue->printS()());
  }else if(ptype==Table){
    s=ptable.printS();
  }else if(ptype==SaveFrame){
    s=psaveframe->printS();
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFData::DataBlockItem::print()const
{
  if(ptype==Single){
    String n(*pname);
    n.rightPadd(' ',33);
    printf("_%s ",n());
    pvalue->print();
    printf("\n");
  }else if(ptype==Table){
    ptable.print();
  }else if(ptype==SaveFrame){
    psaveframe->print();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlock::printS()const
{
  String s;
  if(ptype==Data)
    s+=Sprintf("data_%s\n",(*pname)());
  else
    s+=Sprintf("save_%s\n",(*pname)());
  for(long i=0;i<pdatablockitems.size();i++){
    s+=pdatablockitems.at(i)->printS();
  }
  if(ptype==SaveFrame)
    s+=Sprintf("save_\n");

  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFData::DataBlock::print()const
{
  if(ptype==Data)
    printf("data_%s\n",(*pname)());
  else
    printf("save_%s\n",(*pname)());
  for(long i=0;i<pdatablockitems.size();i++){
    pdatablockitems.at(i)->print();
  }
  if(ptype==SaveFrame)
    printf("save_\n");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::printS()const
{
  String s;
  for(long i=0;i<pdatablocks.size();i++){
    s+=pdatablocks.at(i)->printS();
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFData::print()const
{
  for(long i=0;i<pdatablocks.size();i++){
    pdatablocks.at(i)->print();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long CIFData::DataBlock
::spaceGroupNumber(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::SpaceGroupITNumber);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().L();
    }
  }

  tag=valuedict.value(CIFMetaDict::SymmetryIntTablesNumber);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().L();
    }
  }
  return -1;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlock
::spaceGroupNameHall(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::SpaceGroupNameHall);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }

  tag=valuedict.value(CIFMetaDict::SymmetrySpaceGroupNameHall);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }
  return "";

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlock
::spaceGroupNameHM(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::SpaceGroupNameHMFull);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }

  tag=valuedict.value(CIFMetaDict::SymmetrySpaceGroupNameHM);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }
  return "";

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlock
::crystalSystem(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::SpaceGroupCrystalSystem);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }

  tag=valuedict.value(CIFMetaDict::SymmetryCellSetting);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }
  return "";

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<Pair<long,String> > CIFData::DataBlock
::spaceGroupGeneratorsXYZ(const CIFMetaDict &valuedict)const
{
  ValueArray<Pair<long,String> > res;
  String tag=valuedict.value(CIFMetaDict::SymmetryEquivPosAsXyz);

  const DataTable *table=0;
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Table &&
       pdatablockitems.at(i)->table()->containsCategory(tag)){
      table=pdatablockitems.at(i)->table();
      break;
    }
  }

  String idtag;
  if(!table){
    tag=valuedict.value(CIFMetaDict::SpaceGroupSymopId);
    for(long i=0;i<pdatablockitems.size();i++){
      if(pdatablockitems.at(i)->type()==Table &&
	 pdatablockitems.at(i)->table()->containsCategory(tag)){
	table=pdatablockitems.at(i)->table();
	break;
      }
    }
    if(!table)return  res;
    tag=valuedict.value(CIFMetaDict::SpaceGroupSymopOperationXYZ);
    idtag=valuedict.value(CIFMetaDict::SpaceGroupSymopId);
  }else{
    tag=valuedict.value(CIFMetaDict::SymmetryEquivPosAsXyz);
    idtag=valuedict.value(CIFMetaDict::SymmetryEquivPosSiteId);
  }

  long itag=table->indexOf(tag);
  long iidtag=table->indexOf(idtag);
  if(itag<0)return res;

  for(long i=0;i<table->rows();i++){
    long id=i+1;
    if(iidtag>=0){
      id=table->value(i,iidtag).L();
    }
    res.append(Pair<long,String>(id,table->value(i,itag).S()));
  }
  return res;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Vec3 CIFData::DataBlock::axesAngles(const CIFMetaDict &valuedict)const
{
  Vec3 v;
  bool havea=false;
  bool haveb=false;
  bool havec=false;
  String a=valuedict.value(CIFMetaDict::CellAngleAlpha);
  String b=valuedict.value(CIFMetaDict::CellAngleBeta);
  String c=valuedict.value(CIFMetaDict::CellAngleGamma);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single){
      if(pdatablockitems.at(i)->nameLC()==a){
	v[0]=pdatablockitems.at(i)->value().D();
	havea=true;
      }
      if(pdatablockitems.at(i)->nameLC()==b){
	v[1]=pdatablockitems.at(i)->value().D();
	haveb=true;
      }
      if(pdatablockitems.at(i)->nameLC()==c){
	v[2]=pdatablockitems.at(i)->value().D();
	havec=true;
      }
    }
  }
  String msg="";
  if(!havea) msg+="cannot find cell-angle alpha\n";
  if(!haveb) msg+="cannot find cell-angle beta\n";
  if(!havec) msg+="cannot find cell-angle gamma\n";
  if(msg.length()>0)throw Except(msg);
  return v;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double CIFData::DataBlock::cellVolume(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::CellVolume);
  long L=tag.length()-1;
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single){ 
      String name=pdatablockitems.at(i)->nameLC();
      if(name.subString(0,L)==tag){
	double fac=1;
	if(name.subString(L+1)=="_nm"){fac=1000.;}
	if(name.subString(L+1)=="_pm"){fac=0.000001;}

	return pdatablockitems.at(i)->value().D()*fac;
      }
    }
  }
  throw Except(Sprintf("cellVolume: cannot find tag \'%s\'\n",tag()));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long CIFData::DataBlock::cellFormulaUnits(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::CellFormulaUnitsZ);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().L();
    }
  }
  throw Except(Sprintf("cellFormulaUnits: cannot find tag \'%s\'\n",tag()));

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataBlock
::cellSpecialDetails(const CIFMetaDict &valuedict)const
{
  String tag=valuedict.value(CIFMetaDict::CellSpecialDetails);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single &&
       pdatablockitems.at(i)->nameLC()==tag){
      return pdatablockitems.at(i)->value().S();
    }
  }
  throw Except(Sprintf("cellSpecialDetails: cannot find tag \'%s\'\n",tag()));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<Pair<String,String> > CIFData::DataBlock
::allOfSingleCategoryAsString(const String &categ)const
{
  ValueArray<Pair<String,String> > res;
  long L=categ.length()-1;
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single){
      String name=pdatablockitems.at(i)->nameLC();
      if(name.subString(0,L)==categ){
	res.append(Pair<String,String>(name.subString(L+1),
				       pdatablockitems.at(i)->value()
				       .asString()));
      }
    }
  }
  return res;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Vec3 CIFData::DataBlock
::latticeConstants(const CIFMetaDict &valuedict)const
{
  Vec3 v;
  bool havea=false;
  bool haveb=false;
  bool havec=false;
  String a=valuedict.value(CIFMetaDict::CellLengthA);
  String b=valuedict.value(CIFMetaDict::CellLengthB);
  String c=valuedict.value(CIFMetaDict::CellLengthC);
  long La=a.length()-1;
  long Lb=b.length()-1;
  long Lc=c.length()-1;
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single){
      String name=pdatablockitems.at(i)->nameLC();
      double fac=1;
      if(name.subString(0,La)==a){
	if(name.subString(La+1)=="_nm"){fac=10.;}
	if(name.subString(La+1)=="_pm"){fac=0.01;}
      	v[0]=pdatablockitems.at(i)->value().D()*fac;
	havea=true;
      }
      if(name.subString(0,Lb)==b){
	if(name.subString(Lb+1)=="_nm"){fac=10.;}
	if(name.subString(Lb+1)=="_pm"){fac=0.01;}
      	v[1]=pdatablockitems.at(i)->value().D()*fac;
	haveb=true;
      }
      if(name.subString(0,Lc)==c){
	if(name.subString(Lc+1)=="_nm"){fac=10.;}
	if(name.subString(Lc+1)=="_pm"){fac=0.01;}
      	v[2]=pdatablockitems.at(i)->value().D()*fac;
	havec=true;
      }
    }
  }
  String msg="";
  if(!havea) msg+="cannot find a-lattice constant\n";
  if(!haveb) msg+="cannot find b-lattice constant\n";
  if(!havec) msg+="cannot find c-lattice constant\n";
  if(msg.length()>0)throw Except(msg);
  return v;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const CIFData::DataTable *CIFData::DataBlock
::atomTable(const CIFMetaDict &valuedict)const
{
  String s=valuedict.value(CIFMetaDict::AtomSite);
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Table){
      if(pdatablockitems.at(i)->table()->containsCategory(s)){
	return pdatablockitems.at(i)->table();
      }
    }
  }
  throw Except(Sprintf("atomTable: cannot find tag \'%s\'\n",s()));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const CIFData::DataBlockItem *CIFData::DataBlock
::hasSingle1(const String &cifname)const
{
  for(long i=0;i<pdatablockitems.size();i++){
    if(pdatablockitems.at(i)->type()==Single){
      if(pdatablockitems.at(i)->nameLC()==cifname){
	return pdatablockitems.at(i);
      }
    }
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFData::DataBlockItem::~DataBlockItem()
{
  if(pname) delete pname;
  if(pvalue) delete pvalue;
  if(psaveframe) delete psaveframe;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFData::DataBlockItem *CIFData::DataBlockItem
::NewSingleTagValuePair(String *name,Value *v)
{
  DataBlockItem *dbi=new DataBlockItem(Single);
  dbi->pname=name;
  dbi->pvalue=v;
  return dbi;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFData::DataBlockItem *CIFData::DataBlockItem
::NewSaveFrame(String *name)
{
  DataBlockItem *dbi=new DataBlockItem(SaveFrame);
  dbi->psaveframe=new DataBlock(SaveFrame,name);
  return dbi;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFData::DataBlockItem *CIFData::DataBlockItem::NewTable()
{
  return new DataBlockItem(Table);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CIFData::DataTable::appendValue(Value *v)
{
  if((pcount%ptags.size())==0){
    pvalues.append(new PtrArray<Value>());
  }
  pvalues.last()->append(v);
  pcount++;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataTable::commonCategory()const
{
  String s;
  long N=ptagsnormalized.size();
  
  //myDebug()<<ptagsnormalized;
  //myDebug()<<"N";
  if(N<1) return s;
  long i=0;
  while(1){
    const String &t0=ptagsnormalized.at(0);
    if(i>=t0.length())return s;
    char ch=t0[i];
    //myDebug()<<i<<ch;
    for(long j=1;j<N;j++){
      const String &t=ptagsnormalized.at(j);
      if((i>=t.length()) || t[i]!=ch){
	return s;
      }
    }
    if(ch=='_'){
      s=t0.subString(0,i);
      //myDebug()<<"now"<<i<<s;
    }
    i++;
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataTable::atomLabel(long i,const CIFMetaDict &valuedict)const
{
  String t=valuedict.value(CIFMetaDict::AtomSiteTypeSymbol);
  long index;
  if((index=this->indexOf(t))>=0){
    String s=pvalues.at(i)->at(index)->S();
    long i0=s.reject(String::setazAZ);
    //    myDebug()<<s<<i0;
    s=s.subString(0,i0-1);
    //myDebug()<<"AtomSiteTypeSymbol:"<<s;
    return s;
  }


  t=valuedict.value(CIFMetaDict::AtomSiteLabel);
  if((index=this->indexOf(t))>=0){
    String s=pvalues.at(i)->at(index)->S();
    long i0=s.reject(String::setazAZ);
    //myDebug()<<s<<i0;
    s=s.subString(0,i0-1);
    //myDebug()<<"AtomSiteLabel:"<<s;
    return s;
  }
  throw Except(Sprintf("DataTable::label: cannot find tag \'%s\'\n",t()));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

Vec3 CIFData::DataTable::atomFracPosition(long i,const CIFMetaDict &valuedict)const
{
  String tx=valuedict.value(CIFMetaDict::AtomSiteFractX);
  String ty=valuedict.value(CIFMetaDict::AtomSiteFractY);
  String tz=valuedict.value(CIFMetaDict::AtomSiteFractZ);
  long index;
  Vec3 p;
  if((index=this->indexOf(tx))>=0){
    p[0]=pvalues.at(i)->at(index)->D();
    if((index=this->indexOf(ty))>=0){
      p[1]=pvalues.at(i)->at(index)->D();
      if((index=this->indexOf(tz))>=0){
	p[2]=pvalues.at(i)->at(index)->D();
	return p;
      }
    }
  }
  
  throw Except(Sprintf("DataTable::position: cannot find tag \'%s...\'\n",
		       tx()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CIFData::DataTable::atomWyckoffSymbol(long i,
					     const CIFMetaDict &valuedict)const
{
  String t=valuedict.value(CIFMetaDict::AtomSiteWyckoffSymbol);
  long index;
  if((index=this->indexOf(t))>=0){
    return pvalues.at(i)->at(index)->S();
  }
  
  throw Except(Sprintf("DataTable::WyckoffSymbol: cannot find tag \'%s\'\n",
		       t()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long CIFData::DataTable::atomSymmetryMultiplicity(long i,
                           const CIFMetaDict &valuedict)const
{
  String t=valuedict.value(CIFMetaDict::AtomSiteSymmetryMultiplicity);
  long index;
  if((index=this->indexOf(t))>=0){
    return pvalues.at(i)->at(index)->L();
  }
  
  throw Except(Sprintf("DataTable::SymmetryMultiplicity: cannot find "
		       "tag \'%s\'\n",t()));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double CIFData::DataTable::atomOccupancy(long i,
					 const CIFMetaDict &valuedict)const
{
  String t=valuedict.value(CIFMetaDict::AtomSiteOccupancy);
  long index;
  if((index=this->indexOf(t))>=0){
    return pvalues.at(i)->at(index)->D();
  }
  
  throw Except(Sprintf("DataTable::Occupancy: cannot find tag \'%s\'\n",
		       t()));
}




}//end namespace CIF

/*EOF*/ 
