/*************************************************************************
 * @(#)CIFDriver.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   06 Aug 2012
 *************************************************************************
 */

#include <iostream>
#include <sstream>
#include "CIFDriver.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFDriver::CIFDriver()
  :ptracescanning(false),ptraceparsing(false),pcifdata(0),pdatablock(0)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFDriver::~CIFDriver()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFDriver::parse(CIF::CIFData *cifdata,const char *bufferorfilename
		     ,bool frombuffer)
{
    
  if(!cifdata) {
    throw Except("internal error: cifdata==NULL?\n"); 
  }  
  pcifdata=cifdata;
  pcifdata->clear();

  pfrombuffer=frombuffer;
  if(!pfrombuffer){
    pfilename = bufferorfilename;
    pbuffer=0;
  }else{
    pfilename = "buffer";
    pbuffer=bufferorfilename;
  }

  ciferrormsg="";
  scan_begin();
  CIF::CIFParser parser(*this,pscanner);
  parser.set_debug_level(ptraceparsing);
  if(parser.parse()!=0)
    throw Except(ciferrormsg); 
  scan_end();

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFDriver::error(const CIF::location& l, const std::string& m)
{
  if( m.length()!=0){
    std::ostringstream oss (std::ostringstream::out);
    oss << l << ": " << m << std::endl;
    ciferrormsg+=oss.str().c_str();
  }else{
    ciferrormsg+="error: unspecified\n";
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFDriver::error (const std::string& m)
{
  ciferrormsg+=Sprintf("%s\n",m.c_str());
}

/*EOF*/ 
