/*************************************************************************
 * @(#)FMatrix.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   11 Sep 2012
 *************************************************************************
 */

#include "FMatrix.hh"
#include "IMatrix.hh"
#include "Vec3.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Matrix3x3 FMatrix::toMatrix3x3()const
{
  if(prows!=3 || pcols!=3) throw Except("FMatrix::toMatrix3x3: wrong dimensions!");
  Matrix3x3 res;
  for(int i=0;i<prows*pcols;i++){
    res.data()[i]=p[i].toDouble();
  }
    return res; 
  }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix FMatrix::inverse(Fractional &det)const
{
  // return inverse and det
  if(prows!=pcols){
    throw Except("FMatrix::inverse(): not square");
  }
  if(prows==3){
    
    FMatrix m(3,3);
    
    m.p[0]=p[4]*p[8]-p[5]*p[7];
    m.p[3]=p[5]*p[6]-p[3]*p[8];
    m.p[6]=p[3]*p[7]-p[4]*p[6];
    det=m.p[0]*p[0]+m.p[3]*p[1]+m.p[6]*p[2];
    Fractional i(1);
    if(det!=0){
      i=det.inv();
    }
    
    
    m.p[0]*=i;
    m.p[3]*=i;
    m.p[6]*=i;
    
    m.p[1]=i*(p[7]*p[2]-p[8]*p[1]);
    m.p[4]=i*(p[8]*p[0]-p[6]*p[2]);
    m.p[7]=i*(p[6]*p[1]-p[7]*p[0]);
    
    m.p[2]=i*(p[1]*p[5]-p[2]*p[4]);
    m.p[5]=i*(p[2]*p[3]-p[0]*p[5]);
    m.p[8]=i*(p[0]*p[4]-p[1]*p[3]);
    
    
    return m;
  }else{
    FMatrix m;
    throw Except("FMatrix::inverse(): not implemented");
    return m;
    
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix operator*(const FMatrix &oi,const FMatrix &of){// mat product
  if(oi.pcols!=of.prows)
    throw Except("FMatrix:: operator*(FMatrix,FMatrix): "
		 "incompatible dimensions");
  FMatrix res(oi.prows,of.pcols);
  for(long r=0;r<oi.prows;r++){
    for(long c=0;c<of.pcols;c++){
      Fractional d=0;
      for(long k=0;k<oi.pcols;k++){
	d+=of.p[k+of.prows*c]*oi.p[r+oi.prows*k];
      }
      res.p[r+oi.prows*c]=d;
    }
  }

  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix operator*(const IMatrix &oi,const FMatrix &of){// mat product
  if(oi.pcols!=of.prows)
    throw Except("FMatrix:: operator*(IMatrix,FMatrix): "
		 "incompatible dimensions");
  FMatrix res(oi.prows,of.pcols);
  for(long r=0;r<oi.prows;r++){
    for(long c=0;c<of.pcols;c++){
      Fractional d=0;
      for(long k=0;k<oi.pcols;k++){
	d+=of.p[k+of.prows*c]*oi.p[r+oi.prows*k];
      }
      res.p[r+oi.prows*c]=d;
    }
  }

  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix operator*(const FMatrix &oi,const IMatrix &of){// mat product
  if(oi.pcols!=of.prows)
    throw Except("FMatrix:: operator*(FMatrix,IMatrix): "
		 "incompatible dimensions");
  FMatrix res(oi.prows,of.pcols);
  for(long r=0;r<oi.prows;r++){
    for(long c=0;c<of.pcols;c++){
      Fractional d=0;
      for(long k=0;k<oi.pcols;k++){
	d+=oi.p[r+oi.prows*k]*of.p[k+of.prows*c];
      }
      res.p[r+oi.prows*c]=d;
    }
  }

  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix operator+(const FMatrix &of,const IMatrix &oi)
{
  if(oi.prows!=of.prows || oi.pcols!=of.pcols)
    throw Except("FMatrix:: operator+(FMatrix,IMatrix): "
		 "incompatible dimensions");
  FMatrix res(of.prows,of.pcols);
  for(long i=0;i<oi.prows*oi.pcols;i++){
    res.p[i]=of.p[i]+oi.p[i];
  }  
  return res;
}


/*EOF*/ 
