/* @(#)HallSymbol.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SPACEGROUP_HH
#define _SPACEGROUP_HH 1

#include "FString.hh"
#include "ValueArray.hh"
#include "SeitzMatrix.hh"
#include "SpaceGroupData.hh"


namespace SpaceGroup{


  class LatticeParameters;

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  class HallSymbol
  {
  private:
    class pSeitzSymbol;
  public:
    HallSymbol():psymbol("not set"),pisvalid(false),pparsed(false),
		 pcrystalsystem(Data::None),pbravais(Data::Simple),
		 pislayergroup(false){}
    HallSymbol(const String &symbol);
    ~HallSymbol(){}

    HallSymbol &fromSymbol(const String &symbol);
    
    const String &symbol()const{return psymbol;}

    const ValueArray<SeitzMatrix> &operations(bool analyse=false);

    

    // Tol is the error tolerance, when checking the lattice parameters according to
    // the hall symbols settings. If not OK  warnings are returned. 
    // Check it and if user thinks  we should go ahead with the correction, 
    // recall with adjust=true.
    String checkLatticeParameter(LatticeParameters &,bool adjust=false,
				 double tol=1.0e-4);

      

    
    Data::CrystalSystem crystalSystem()const{return pcrystalsystem;}
    Data::BravaisLattice bravaisLattice()const{return pbravais;}
    const  char *crystalSystemName()const{return Data::crystalSystemName(pcrystalsystem);}
    const char *bravaisLatticeName()const{return Data::bravaisLatticeName(pbravais);}
    bool isLayerGroup()const{return pislayergroup;}

  protected:
  private:
    void parse(ValueArray<SeitzMatrix> &ops);
    bool parseTranslationPart(const String &V,FMatrix &trans);
    bool parseSeitzSymbol(const String &ss,pSeitzSymbol *seitzsym,
			  SeitzMatrix &gen,long noOfOp);

  private:
    class pSeitzSymbol{
    public:
      pSeitzSymbol():improper(false),norder(0),axissymbol('\0'){};
      bool improper;
      long norder;
      char axissymbol;
      ValueArray<char> transsymbols;
    };
    
    static const char *translationSymbol;
    static const char *latticeSymbols;
    static const char *axisSymbols;

    static const long rotationpart[3][6][9];
    static const long rotationpart2prime[][9];
    static const long rotationpart2dprime[][9];
    static const long rotationpart3star[9];

  private:
    String psymbol;
    bool pisvalid;
    bool pparsed;
    ValueArray<SeitzMatrix> pops;
    Data::CrystalSystem pcrystalsystem;
    Data::BravaisLattice pbravais;
    long pmonoorientation;
    String pmonotranss;
    bool pislayergroup;
  };

}
#endif /* _SPACEGROUP_HH */

