/*************************************************************************
 * @(#)IntegerTricks.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   30 Aug 2012
 *************************************************************************
 */

#include "IMatrix.hh"
#include "FMatrix.hh"
#include "IntegerTricks.hh"
#include "MyMath.hh"
#include "Vec3.hh"

using namespace IntegerTricks;




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IntegerTricks::reduceToSmallestLatticeBasis(ValueArray<FMatrix> &tr)
{
  bool prot=false;
#define print					\
  for(long ir=0;ir<3;ir++){			\
    for(long i=0;i<tr.size();i++){		\
      printf("%10s ",tr.at(i)[ir].printS()());	\
    }						\
    printf("\n");				\
  }


  if(prot) {myDebug()<<"reduceToSmallestLatticeBasis: tr=";print;}


  
  
  long last=tr.size()-1;
  for(long irow=2;irow>=0;irow--){
    long ic0=0;
    while(true){
      while(tr.at(ic0)[irow]==0 && ic0<last)ic0++;
      if(ic0==last)break;
      if(prot) myDebug()<<"ic0"<<ic0;
      long imax=0;
      for(long i=0;i<=last;i++){
	if(abs(tr.at(i)[irow])>abs(tr.at(imax)[irow])){
	  imax=i;
	}
      }
      if(prot) myDebug()<<"imax="<<imax;
      long ii=-1;
      for(long i=0;i<=last;i++){
	if(i==imax)continue;
	if(tr.at(i)[irow]!=0){
	  ii=i;break;
	}
      }
      if(prot) myDebug()<<"ii="<<ii;
      if(ii<0) {
	break;
      }
      
      long k=tr.at(imax)[irow].backfolder(tr.at(ii)[irow]);
      if(prot) myDebug()<<"k="<<k;
      if(prot) myDebug()<<"a+k*b="<<tr.at(imax)[irow]+tr.at(ii)[irow]*k;
      for(long i=0;i<3;i++){
	tr[imax][i]+=tr.at(ii)[i]*k;
      }
      if(prot) print;
      if(tr[imax][irow]==0&& imax!=ic0){
	if(prot) myDebug()<<"swap imax with ic0";
	{ FMatrix d=tr.at(imax); tr[imax]=tr.at(ic0);tr[ic0]=d;}
	if(prot) print;
      }
    }
    if(prot) myDebug()<<"done with row "<<irow;
    
    if(prot) print;
    last--;
  }

  for(long i=tr.size()-1;i>=0;i--){
    if(tr.at(i)==0){
      tr.removeAt(i);
    }
  }
  if(tr.size()==3){
    if(FMatrix(tr.at(0),tr.at(1),tr.at(2)).determinant()<0)
      tr[2]*=-1;
  }


  if(prot) {myDebug()<<"reduceToSmallestLatticeBasis: out=";print;}

#undef print
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void shortenVV(long *A,long *V,long *pV,long rows,long cols,long c0,
		      long c1,bool userest)
{
  bool prot=false;;
#define printA(text,A,rows,cols){			\
    printf("%s=\n",text);				\
    for(long rrr=0;rrr<rows;rrr++){			\
      for(long ccc=0;ccc<cols;ccc++){			\
	printf("%10ld ",A[rrr+rows*ccc]);		\
      }							\
      printf("\n");					\
    }\
  }
#define modfold(A,a,rr,k)						\
  { rr=(A)%(a);								\
    k=((rr)-(A))/(a);							\
    if(labs((a))<2*labs((rr))||(labs((a))==2*labs((rr)) && (rr)<0)){	\
      k-=lSign(1,(a))*lSign(1,(rr));					\
      rr=A+a*k;								\
    }									\
  }
  long C0=c0;
  long C1=c1;
  if(userest){
    C0=c1+1;
    C1=cols-1;
  }
  if(prot) myDebug()<<"try to shorten from"<<c0<<" to "<<c1<<"=======";
  bool changed=true;
  while(changed){
    changed=false;
    for(long ll=c0;ll<=c1;ll++){
      long qq=pV[ll+cols*ll];
      if(prot) myDebug()<<"modify "<<ll<<" ======= qq="<<qq;
      long qqold=qq+1;
      while(qqold>qq && qq!=0){
	qqold=qq;
	for(long j=C0;j<=C1;j++){
	  if(j==ll)continue;
	  long pq=pV[j+cols*ll];
	  long pp=pV[j+cols*j];
	  if(prot) myDebug()<<"column j="<<j<<":pp="<<pp<<" pq="<<pq;
	  long L,R;
	  if(pp==0)continue;
	  modfold(pq,pp,R,L);
	  if(prot) myDebug()<<"L="<<L<<"rest="<<R;
	  if(prot) myDebug()<<"qq="<<qq<<"new="<<qq+(R+pq)*L;
	  
	  
	  if((R+pq)*L<0){
	    for(long i=0;i<cols;i++){
	      V[i+cols*ll]+=V[i+cols*j]*L;
	    }
	    for(long i=0;i<rows;i++){
	      A[i+rows*ll]+=A[i+rows*j]*L;
	    }
	    qq+=(R+pq)*L;
	    for(long i=0;i<cols;i++){
	      pV[i+cols*ll]+=pV[i+cols*j]*L;
	    }
	    for(long i=0;i<cols;i++){
	      pV[ll+cols*i]+=pV[j+cols*i]*L;
	    }
	    changed=true;
	    if(prot) printA("shortA",A,rows,cols);
	    if(prot) printA("V",V,cols,cols);
	    if(prot) printA("pV",pV,cols,cols);
	  }
	}
	if(qq==1) break;
      }
    }
    
  }//while(changed)
  if(prot) myDebug()<<"END try to shorten =======";
#undef modfold
#undef printA
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IntegerTricks::decompAVeqB(long *A,long *V,long rows,long cols,
				bool detnotone,long prot)
{
  /* solve A*V=B with B=(D,0,....0) for a rows x cols matrix A, return 
     V with det(V)=1 and the rank of B(A) (return value)
     A gets modified to be B.
     This is not the fastest possible algorithm, instead it tries to
     keep the absolute value of intermediate numbers at a minimum
     to avoid long overflow. It is suited for small matrices, since
     from a certain (unknown) size onwards the overflow will occur.
     The problems tackled with this routine should be resonably preconditioned
     such that A is already "close" to B (guided by geometrical considerations).
     If detnotone is true, B will be diagonalized, which may lead to  
     det(V)!=1.
   */
#define printA(text,A,rows,cols){			\
    printf("%s=\n",text);				\
    for(long rrr=0;rrr<rows;rrr++){			\
      for(long ccc=0;ccc<cols;ccc++){			\
	printf("%10ld ",A[rrr+rows*ccc]);		\
      }							\
      printf("\n");					\
    }\
  }
#define modfold(A,a,rr,k)						\
  { rr=(A)%(a);								\
    k=((rr)-(A))/(a);							\
    if(labs((a))<2*labs((rr))||(labs((a))==2*labs((rr)) && (rr)<0)){	\
      k-=lSign(1,(a))*lSign(1,(rr));					\
      rr=A+a*k;								\
    }									\
  }
#define swapcol(A,rows,c1,c2,s) {					\
    for(long i=0;i<rows;i++){						\
      long d=A[i+rows*(c1)];A[i+rows*(c1)]=(s)*A[i+rows*(c2)];		\
      A[i+rows*(c2)]=d;							\
    }									\
  }
#define swaprow(A,rows,cols,c1,c2) {					\
    for(long i=0;i<cols;i++){						\
      long d=A[c1+rows*i];A[c1+rows*i]=-A[c2+rows*i];A[c2+rows*i]=d;	\
    }									\
  }
  
  
  if(prot==10) myDebug()<<"================decompAVeqB=======================";
  long rank=cols;
  if(cols==0){return rank;}
  long *pV=new long[cols*cols];

  for(long r=0;r<cols;r++){
    for(long c=0;c<cols;c++){
      V[r+cols*c]=0;
      pV[r+cols*c]=0;
    }
    V[r+cols*r]=1;
    pV[r+cols*r]=1;
  }
  if(prot==1) printA("A",A,rows,cols);
  if(prot==10) printA("V",V,cols,cols);
  if(prot==10) printA("pV",pV,cols,cols);
  
  
  long cs=0;
  for(long ro=rows-1;ro>=0;ro--){
    long c0=cols;

    if(prot==10) myDebug()<<"======row "<<ro<<" and cs"<<cs<<" and c0"<<c0;


    
    while(c0>0){
      if(prot==10) myDebug()<<"====== c0"<<c0<<"cs="<<cs;
      if(cs>=c0-1)break;
      //pick largest in  [cs,c0-1]
      long cmax=cs;
      for(long i=cs;i<c0;i++){
	if(labs(A[ro+rows*i])>labs(A[ro+rows*cmax])){cmax=i;}
	//if(labs(A[r+rows*i])<labs(A[r+rows*cmax])){cmax=i;}
      }  
      if(prot==10) myDebug()<<"amax="<<cmax<<labs(A[ro+rows*cmax]);
      //pick second largest in  [cs,c0-1]
      long cred=cs;
      if(cred==cmax)cred++;
      for(long i=cs;i<c0;i++){
	if(i==cmax)continue;
	if(labs(A[ro+rows*i])>=labs(A[ro+rows*cred])){cred=i;}
	//if(labs(A[r+rows*i])<=labs(A[r+rows*cred])){cred=i;}
      }  
      if(prot==10) myDebug()<<"ared="<<cred<<labs(A[ro+rows*cred]);
      
      long amax=A[ro+rows*cmax];
      long ared=A[ro+rows*cred];
      if(ared==0){
	cmax=cred;
      }else if(amax!=0){
	long k=0;
	if(labs(amax+ared)<labs(amax)){
	  k=1;
	}else if(labs(amax-ared)<labs(amax)){
	  k=-1;
	}else{
	  myDebug()<<"cannot reduce amax?";
	  exit(1);
	}
	
	if(prot==10) myDebug()<<"k="<<k;
	for(long i=0;i<cols;i++){
	  V[i+cols*cmax]+=V[i+cols*cred]*k;
	  pV[i+cols*cmax]+=pV[i+cols*cred]*k;
	}
	for(long i=0;i<cols;i++){
	  pV[cmax+cols*i]+=pV[cred+cols*i]*k;
	}
	for(long j=0;j<rows;j++){
	  A[j+rows*cmax]+=A[j+rows*cred]*k;
	}
	
	if(prot==10) printA("redA",A,rows,cols);
	if(prot==10) printA("V",V,cols,cols);
	if(prot==10) printA("pV",pV,cols,cols);
      }
      if(A[ro+rows*cmax]==0){
	if(cmax<c0-1 && A[ro+rows*cmax]==0){
	  if(prot==10) myDebug()<<"======swap "<<cmax<<" and "<<c0-1;
	  swapcol(A,rows,cmax,c0-1,-1);
	  swapcol(V,cols,cmax,c0-1,-1);
	  swapcol(pV,cols,cmax,c0-1,-1);
	  swaprow(pV,cols,cols,cmax,c0-1);
	  if(prot==10) printA("swapA",A,rows,cols);
	  if(prot==10) printA("V",V,cols,cols);
	  if(prot==10) printA("pV",pV,cols,cols);
	}
	c0--;
      }
      
      //mix-reduce pV for c0..cols-1
      shortenVV(A,V,pV,rows,cols,c0,cols-1,false);
      //reduce pV with the mix for 0..cs-1
      shortenVV(A,V,pV,rows,cols,0,c0-1,true);
      
    }

    if((cs<cols && A[ro+rows*(cs)]!=0) || cs>=cols)cs++;
    if(cs-1<cols){
      long ared=A[ro+rows*(cs-1)];
      if(ared!=0){
	for(long c=0;c<cs-1;c++){
	  if(prot==10) myDebug()<<"reduce column"<<c<<"row"<<ro<<"with column"<<cs-1;
	  long amax=A[ro+rows*c];
	  
	  long l=1;
	  //if(detnotone && labs(amax)<labs(ared)
	  while(amax!=0){
	    long k=0;
	    if(labs(amax+ared)<labs(amax)){
	      k=1;
	    }else if(labs(amax-ared)<labs(amax)){
	      k=-1;
	    }else{
	      if(!detnotone) break;
	      long D=labs(MyMath::gcd(amax,ared));
	      l=ared/D;
	      k=-amax/D;
	      if(l<0){
		l=-l;
		k=-k;
	      }
	    }
	    amax=amax*l+ared*k;
	    for(long i=0;i<rows;i++){
	      A[i+rows*c]=A[i+rows*c]*l+k*A[i+rows*(cs-1)];
	    }
	    for(long i=0;i<cols;i++){
	      V[i+cols*c]=V[i+cols*c]*l+k*V[i+cols*(cs-1)];
	      pV[i+cols*c]=pV[i+cols*c]*l+k*pV[i+cols*(cs-1)];
	    }
	    for(long i=0;i<cols;i++){
	      pV[c+cols*i]=pV[c+cols*i]*l+k*pV[(cs-1)+cols*i];
	    }
	    if(prot==10) printA("0redA",A,rows,cols);
	    if(prot==10) printA("V",V,cols,cols);
	    if(prot==10) printA("pV",pV,cols,cols);
	    
	    shortenVV(A,V,pV,rows,cols,0,c0-1,true);
	  }
	}
      }
    }

  }// all rows


  rank=-1;
  for(long r=0;r<rows;r++){
    for(long c=cols-1;c>=0;c--){
      if(A[r+rows*c]!=0){
	rank=MyMath::max<long>(rank,c);break;
      }
    }
  }
  rank++;

  
  long i0=0;long i1=rank-1;
  while(i0<i1){
    //myDebug()<<"swap i0="<<i0<<" and i1="<<i1;
    swapcol(A,rows,i0,i1,-1);
    swapcol(V,cols,i0,i1,-1);
    i0++;i1--;
  }
  if(prot==10) printA("sortedA",A,rows,cols);
  if(prot==10) printA("V",V,cols,cols);

  if(prot==10) myDebug()<<"cols-rank"<<cols-rank;
  if(cols-rank>=2){
    long s=1;

    for(long c=rank-1;c>=0;c--){
      long r=rows-1;while(A[r+rows*c]==0)r--;
      if(A[r+rows*c]<0){
	s*=-1;
	for(long j=0;j<rows;j++){
	  A[j+rows*c]=-A[j+rows*c];
	}
	for(long j=0;j<cols;j++){
	  V[j+cols*c]=-V[j+cols*c];
	}
	if(prot==10) printA("mA",A,rows,cols);
	if(prot==10) printA("V",V,cols,cols);
      }      
    }
    

    if(prot==10) myDebug()<<"s="<<s;
    if(s<1){
      swapcol(A,rows,rank,rank+1,1);
      swapcol(V,cols,rank,rank+1,1);
    }

  }

  if(prot==1) printA("B",A,rows,cols);
  if(prot==1) printA("V",V,cols,cols);
  if(prot==1) printf("rank=%ld\n",rank);
  if(prot==10) myDebug()<<"================END decompAVeqB=======================";
  delete [] pV;
  return rank;

#undef swapcol
#undef swaprow
#undef modfold
#undef printA
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void shortenVVF(Fractional *F,long *V,long *pV,long rows,long cols,
		       long c0,long c1,bool userest)
{
  bool prot=false;;
#define printA(text,A,rows,cols){			\
    printf("%s=\n",text);				\
    for(long rrr=0;rrr<rows;rrr++){			\
      for(long ccc=0;ccc<cols;ccc++){			\
	printf("%10ld ",A[rrr+rows*ccc]);		\
      }							\
      printf("\n");					\
    }\
  }
#define printF(text,F,rows,cols){					\
    printf("%s=\n",text);						\
    for(long rrr=0;rrr<rows;rrr++){					\
      for(long ccc=0;ccc<cols;ccc++){					\
	printf("%s ",F[rrr+rows*ccc].printS().leftPadd(' ',12)());	\
      }									\
      printf("\n");							\
    }									\
  }
#define modfold(A,a,rr,k)						\
  { rr=(A)%(a);								\
    k=((rr)-(A))/(a);							\
    if(labs((a))<2*labs((rr))||(labs((a))==2*labs((rr)) && (rr)<0)){	\
      k-=lSign(1,(a))*lSign(1,(rr));					\
      rr=A+a*k;								\
    }									\
  }
  long C0=c0;
  long C1=c1;
  if(userest){
    C0=c1+1;
    C1=cols-1;
  }
  if(prot) myDebug()<<"try to shorten from"<<c0<<" to "<<c1<<"=======";
  bool changed=true;
  while(changed){
    changed=false;
    for(long ll=c0;ll<=c1;ll++){
      long qq=pV[ll+cols*ll];
      if(prot) myDebug()<<"modify "<<ll<<" ======= qq="<<qq;
      long qqold=qq+1;
      while(qqold>qq && qq!=0){
	qqold=qq;
	for(long j=C0;j<=C1;j++){
	  if(j==ll)continue;
	  long pq=pV[j+cols*ll];
	  long pp=pV[j+cols*j];
	  if(prot) myDebug()<<"column j="<<j<<":pp="<<pp<<" pq="<<pq;
	  long L,R;
	  if(pp==0)continue;
	  modfold(pq,pp,R,L);
	  if(prot) myDebug()<<"L="<<L<<"rest="<<R;
	  if(prot) myDebug()<<"qq="<<qq<<"new="<<qq+(R+pq)*L;
	  
	  
	  if((R+pq)*L<0){
	    for(long i=0;i<cols;i++){
	      V[i+cols*ll]+=V[i+cols*j]*L;
	    }
	    for(long i=0;i<rows;i++){
	      F[i+rows*ll]+=F[i+rows*j]*L;
	    }
	    qq+=(R+pq)*L;
	    for(long i=0;i<cols;i++){
	      pV[i+cols*ll]+=pV[i+cols*j]*L;
	    }
	    for(long i=0;i<cols;i++){
	      pV[ll+cols*i]+=pV[j+cols*i]*L;
	    }
	    changed=true;
	    if(prot) printF("shortA",F,rows,cols);
	    if(prot) printA("V",V,cols,cols);
	    if(prot) printA("pV",pV,cols,cols);
	  }
	}
	if(qq==1) break;
      }
    }
    
  }//while(changed)
  if(prot) myDebug()<<"END try to shorten =======";
#undef modfold
#undef printA
#undef printF
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IntegerTricks::decompAVeqBF(Fractional *F,long *V,long rows,long cols,
				bool detnotone)
{
  /* solve F*V=B with B=(D,0,....0) for a rows x cols matrix F, return 
     V with det(V)=1 and the rank of B(F) (return value)
     F gets modified to be B.
     This is not the fastest possible algorithm, instead it tries to
     keep the absolute value of intermediate numbers at a minimum
     to avoid long overflow. It is suite for smal matrices, since
     from a certain (unknown size onwards) the overflow will occur.
     The problems tackled with this routine should be resonably preconditioned
     such that F is already "close" to B (guided by geometrical considerations).
     If detnotone is tre, B will be diagonalized, which may lead to  
     det(V)!=1.
   */
  bool prot=false;
#define printA(text,A,rows,cols){			\
    printf("%s=\n",text);				\
    for(long rrr=0;rrr<rows;rrr++){			\
      for(long ccc=0;ccc<cols;ccc++){			\
	printf("%10ld ",A[rrr+rows*ccc]);		\
      }							\
      printf("\n");					\
    }\
  }
#define printF(text,A,rows,cols){					\
    printf("%s=\n",text);						\
    for(long rrr=0;rrr<rows;rrr++){					\
      for(long ccc=0;ccc<cols;ccc++){					\
	printf("%s ",F[rrr+rows*ccc].printS().leftPadd(' ',12)());	\
      }									\
      printf("\n");							\
    }\
  }
#define modfold(A,a,rr,k)						\
  { rr=(A)%(a);								\
    k=((rr)-(A))/(a);							\
    if(labs((a))<2*labs((rr))||(labs((a))==2*labs((rr)) && (rr)<0)){	\
      k-=lSign(1,(a))*lSign(1,(rr));					\
      rr=A+a*k;								\
    }									\
  }
#define swapcol(A,rows,c1,c2,s) {					\
    for(long i=0;i<rows;i++){						\
      long d=A[i+rows*(c1)];A[i+rows*(c1)]=(s)*A[i+rows*(c2)];		\
      A[i+rows*(c2)]=d;							\
    }									\
  }
#define swapcolF(F,rows,c1,c2,s) {					\
    for(long i=0;i<rows;i++){						\
      Fractional d=F[i+rows*(c1)];F[i+rows*(c1)]=F[i+rows*(c2)]*(s); \
      F[i+rows*(c2)]=d;							\
    }									\
  }
#define swaprow(A,rows,cols,c1,c2) {					\
    for(long i=0;i<cols;i++){						\
      long d=A[c1+rows*i];A[c1+rows*i]=-A[c2+rows*i];A[c2+rows*i]=d;	\
    }									\
  }
  
  
  if(prot) myDebug()<<"================decompAVeqBF=======================";
  long rank=cols;
  if(cols==0){return rank;}
  long *pV=new long[cols*cols];

  for(long r=0;r<cols;r++){
    for(long c=0;c<cols;c++){
      V[r+cols*c]=0;
      pV[r+cols*c]=0;
    }
    V[r+cols*r]=1;
    pV[r+cols*r]=1;
  }
  if(prot) printF("A",F,rows,cols);
  if(prot) printA("V",V,cols,cols);
  if(prot) printA("pV",pV,cols,cols);
  
  
  long cs=0;
  for(long r=rows-1;r>=0;r--){
    long c0=cols;

    if(prot) myDebug()<<"======row "<<r<<" and cs"<<cs<<" and c0"<<c0;


    
    while(c0>0){
      if(prot) myDebug()<<"====== c0"<<c0<<"cs="<<cs;
      if(cs>=c0-1)break;
      //pick largest in  [cs,c0-1]
      long cmax=cs;
      for(long i=cs;i<c0;i++){
	if(abs(F[r+rows*i])>abs(F[r+rows*cmax])){cmax=i;}
	//if(labs(A[r+rows*i])<labs(A[r+rows*cmax])){cmax=i;}
      }  
      if(prot) myDebug()<<"amax="<<cmax<<abs(F[r+rows*cmax]);
      //pick second largest in  [cs,c0-1]
      long cred=cs;
      if(cred==cmax)cred++;
      for(long i=cs;i<c0;i++){
	if(i==cmax)continue;
	if(abs(F[r+rows*i])>=abs(F[r+rows*cred])){cred=i;}
	//if(labs(A[r+rows*i])<=labs(A[r+rows*cred])){cred=i;}
      }  
      if(prot) myDebug()<<"ared="<<cred<<abs(F[r+rows*cred]);
      
      Fractional amax=F[r+rows*cmax];
      Fractional ared=F[r+rows*cred];
      if(ared==0){
	cmax=cred;
      }else if(amax!=0){
	long k=0;
	if(abs(amax+ared)<abs(amax)){
	  k=1;
	}else if(abs(amax-ared)<abs(amax)){
	  k=-1;
	}else{
	  myDebug()<<"cannot reduce amax?";
	  exit(1);
	}
	
	if(prot) myDebug()<<"k="<<k;
	for(long i=0;i<cols;i++){
	  V[i+cols*cmax]+=V[i+cols*cred]*k;
	  pV[i+cols*cmax]+=pV[i+cols*cred]*k;
	}
	for(long i=0;i<cols;i++){
	  pV[cmax+cols*i]+=pV[cred+cols*i]*k;
	}
	for(long j=0;j<rows;j++){
	  F[j+rows*cmax]+=F[j+rows*cred]*k;
	}
	
	if(prot) printF("redA",F,rows,cols);
	if(prot) printA("V",V,cols,cols);
	if(prot) printA("pV",pV,cols,cols);
      }
      if(F[r+rows*cmax]==0){
	if(cmax<c0-1 && F[r+rows*cmax]==0){
	  if(prot) myDebug()<<"======swap "<<cmax<<" and "<<c0-1;
	  swapcolF(F,rows,cmax,c0-1,-1);
	  swapcol(V,cols,cmax,c0-1,-1);
	  swapcol(pV,cols,cmax,c0-1,-1);
	  swaprow(pV,cols,cols,cmax,c0-1);
	  if(prot) printF("swapA",F,rows,cols);
	  if(prot) printA("V",V,cols,cols);
	  if(prot) printA("pV",pV,cols,cols);
	}
	c0--;
      }
      
      //mix-reduce pV for c0..cols-1
      shortenVVF(F,V,pV,rows,cols,c0,cols-1,false);
      //reduce pV with the mix for 0..cs-1
      shortenVVF(F,V,pV,rows,cols,0,c0-1,true);
      
    }

    if((cs<cols && F[r+rows*(cs)]!=0) || cs>=cols)cs++;
    if(cs-1<cols){
      Fractional ared=F[r+rows*(cs-1)];
      if(ared!=0){
	for(long c=0;c<cs-1;c++){
	  if(prot) myDebug()<<"reduce column"<<c<<"row"<<r<<"with column"<<cs-1;
	  Fractional amax=F[r+rows*c];
	  
	  long l=1;
	  while(amax!=0){
	    long k=0;
	    if(abs(amax+ared)<abs(amax)){
	      k=1;
	    }else if(abs(amax-ared)<abs(amax)){
	      k=-1;
	    }else{
	      if(!detnotone) break;
	      long a0=amax.n()*ared.m();
	      long a1=amax.m()*ared.n();
	      long D=labs(MyMath::gcd(a0,a1));
	      l=a1/D;
	      k=-a0/D;
	      if(l<0){
		l=-l;
		k=-k;
	      }
	    }
	    amax=amax*l+ared*k;
	    for(long i=0;i<rows;i++){
	      F[i+rows*c]=F[i+rows*c]*l+F[i+rows*(cs-1)]*k;
	    }
	    for(long i=0;i<cols;i++){
	      V[i+cols*c]=V[i+cols*c]*l+k*V[i+cols*(cs-1)];
	      pV[i+cols*c]=pV[i+cols*c]*l+k*pV[i+cols*(cs-1)];
	    }
	    for(long i=0;i<cols;i++){
	      pV[c+cols*i]=pV[c+cols*i]*l+k*pV[(cs-1)+cols*i];
	    }
	    if(prot) printF("0redA",F,rows,cols);
	    if(prot) printA("V",V,cols,cols);
	    if(prot) printA("pV",pV,cols,cols);
	    
	    shortenVVF(F,V,pV,rows,cols,0,c0-1,true);
	  }
	}
      }
    }

  }// all rows


  rank=-1;
  for(long r=0;r<rows;r++){
    for(long c=cols-1;c>=0;c--){
      if(F[r+rows*c]!=0){
	rank=MyMath::max<long>(rank,c);break;
      }
    }
  }
  rank++;

  
  long i0=0;long i1=rank-1;
  while(i0<i1){
    //myDebug()<<"swap i0="<<i0<<" and i1="<<i1;
    swapcolF(F,rows,i0,i1,-1);
    swapcol(V,cols,i0,i1,-1);
    i0++;i1--;
  }
  if(prot) printF("sortedA",F,rows,cols);
  if(prot) printA("V",V,cols,cols);

  if(prot) myDebug()<<"cols-rank"<<cols-rank;
  if(cols-rank>=2){
    long s=1;

    for(long c=rank-1;c>=0;c--){
      long r=rows-1;while(F[r+rows*c]==0)r--;
      if(F[r+rows*c]<0){
	s*=-1;
	for(long j=0;j<rows;j++){
	  F[j+rows*c]=-F[j+rows*c];
	}
	for(long j=0;j<cols;j++){
	  V[j+cols*c]=-V[j+cols*c];
	}
	if(prot) printF("mA",F,rows,cols);
	if(prot) printA("V",V,cols,cols);
      }      
    }
    

    if(prot) myDebug()<<"s="<<s;
    if(s<1){
      swapcolF(F,rows,rank,rank+1,1);
      swapcol(V,cols,rank,rank+1,1);
    }

  }

  if(prot) myDebug()<<"================END decompAVeqBF=======================";
  delete [] pV;
  return rank;

#undef swapcolF
#undef swapcol
#undef swaprow
#undef modfold
#undef printA
#undef printF
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void IntegerTricks::enlarged_lattice_site_list(double *U,ValueArray<Vec3> &sites,
					       long prot)
{
  // Return the sites, which are maped from (0,0,0) by cell enlargement.
  // Enlargment is defined by a matrix U, which maps an original cell 
  //   |a1|
  // A=|a2|
  //   |a3|
  // onto A'=U.A. The rows of A are the lattice basis vectors.
  // The input U as a real matrix stored in (fortran) column order.
  // The integral condition will be checked. 
  // The result is the array sites of 3-vectors expressed in A' coordinates
  // equivalent to A-cell site (0,0,0).
  //
  //
  double tol=1.0e-6;

#define printAlong(txt,A){			\
    printf("%s",txt);				\
    for(int i=-1;++i<3;){			\
      printf("[");				\
      for(int j=-1;++j<3;){			\
	printf("%8ld ",A[i+3*j]);		\
      }						\
      printf("\n");				\
    }						\
  }
#define printAdouble(txt,A){			\
    printf("%s",txt);				\
    for(int i=-1;++i<3;){			\
      printf("[");				\
      for(int j=-1;++j<3;){			\
	printf("%30.14f ",A[i+3*j]);		\
      }						\
      printf("\n");				\
    }						\
  }
  
  if(prot>=10) printAdouble("U=\n",U);

  // we call the integer U  A and its inverse-transposed G
  IMatrix A(3,3,U);

 
  double detU=Matrix3x3(U).determinant();

  long det;
  IMatrix G=A.inverseTtV(det);
  
  
  if(fabs(detU)<tol)
    throw Except("IntegerTricks::enlarged_lattice_site_list: determinant is zero\n");
  
  if(fabs(labs(det)-fabs(detU))>fabs(detU)*tol)
    throw Except("IntegerTricks::enlarged_lattice_site_list: determinant integer overflow\n");

  if(prot>=10){
    printf("A=\n%s det(A)=%ld\n",A.printS()(),det);
    printf("G=\n%s det(G)=det(A)^2*=%g\n",G.printS()(),
	   G.determinant()/detU/detU);
  }

  if(A*G.Td()!=IMatrix::diagMatrix(3,3,det)){
    throw Except("IntegerTricks::enlarged_lattice_site_list: A*G.T != 1\n");
  }
        
  if(prot>=1){
    printf("A    =\n%s det(A)=%ld\n",A.printS()(),det);
    printf("det*G=\n%s\n",G.printS()());
  }
  // now integer "SVD": G.V=B
  long rank;
  IMatrix Gsave=G;
  IMatrix V=G.decompAVeqB(rank,false);
  // now G contains B
  IMatrix &B=G;
  if(rank!=3)
    throw Except("IntegerTricks::enlarged_lattice_site_list: rank of SVDS != 3\n");
  

  if(prot>=1){ 
    printf("    B=\n%s det(B)=det^2*=%g\n",B.printS()(),
	   B.determinant()/detU/detU);
    printf("V    =\n%s det(V)=%ld\n",V.printS()(),V.determinant());
  }


  if (V.determinant()!=1)
    throw Except("IntegerTricks::enlarged_lattice_site_list: det(V)!=1");

  

  if (Gsave*V!=B)
    throw Except("IntegerTricks::enlarged_lattice_site_list: integer SVD failed");


  long n0=det/B(0,0);
  long n1=det/B(1,1);
  long n2=det/B(2,2);
  if((n0*B(0,0)!=det)||(n1*B(1,1)!=det)||(n2*B(2,2)!=det))
    throw Except("IntegerTricks::enlarged_lattice_site_list: "
		 "diagonal of B is not a divider of det(A)\n");

  if(prot>=1) printf("det/diag(B)=%ld,%ld,%ld\n",n0,n1,n2);

  // construct the list of new sites equivalent to 000

  sites.resize(det);
  

  long n=0;
  for(long i=-1;++i<labs(n0);){
    for(long j=-1;++j<labs(n1);){
      for(long k=-1;++k<labs(n2);){
	IMatrix t=B*IMatrix::vector3(i,j,k); // must be in the box [0,det)^3

	while(t[0]<0) t(0,0)+=det;
	while(t[0]>=det) t(0,0)-=det;
	while(t[1]<0) t(1,0)+=det;
	while(t[1]>=det) t(1,0)-=det;
	Vec3 &v=sites[n++];
	v[0]=t[0]/detU;
	v[1]=t[1]/detU;
	v[2]=t[2]/detU;
	if(prot>=1) printf("V*x=%12.8g %12.8g %12.8g\n",v[0],v[1],v[2]);
      }
    }
  }
  
}

/*EOF*/ 
