/* @(#)PeriodicHyperPlane.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PERIODICHYPERPLANE_HH
#define _PERIODICHYPERPLANE_HH 1

#include "Except.hh"
#include "FString.hh"
#include "IMatrix.hh"
#include "FMatrix.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class PeriodicHyperPlane{
public:
  
  PeriodicHyperPlane(long dim=3){reset(dim);}


  PeriodicHyperPlane(const PeriodicHyperPlane &other){
    reset(other.pdim);
    R=other.R;
    O=other.O;
    Delta=other.Delta;
    
  }

  PeriodicHyperPlane(const IMatrix &r,const FMatrix &o){
    reset(r.rows());
    R=r;
    O=o;
//     if(R.rows()!=r.rows())
//       throw Except("PeriodicHyperPlane: R has different dimension");
  }
  
  void reset(long dim=3){
    pdim=dim;
    R.setDimenions(pdim,0);
    Delta.setDimenions(pdim,0);
    O.setDimenions(pdim,0);
  }

  bool isEmpty()const{return O.columns()==0;}
  
  PeriodicHyperPlane &commonPoints(const PeriodicHyperPlane &p2,const FMatrix &delta=FMatrix(3,1))
  {
    bool prot=false;
    // created the merged PeriodicHyperPlane, which describes
    // the points in common
    if(pdim!=p2.pdim)
      throw Except("Cannot combine PeriodicHyperPlane with "
		   "different space dimensions");

    if(isEmpty())return *this;
    if(p2.isEmpty()){
      reset(pdim);
      return *this;
    }

    PeriodicHyperPlane p1(*this);
    
    reset(pdim);
    R=IMatrix(p1.R,-p2.R);
    Delta=FMatrix(p1.Delta,-p2.Delta);
    FMatrix O1(pdim,0); // expanded this->O
    for(long i1=0;i1<p1.O.columns();i1++){
      for(long i2=0;i2<p2.O.columns();i2++){
	O.addMatrix(p1.O.fromColumn(i1)-p2.O.fromColumn(i2)-delta);
	O1.addMatrix(p1.O.fromColumn(i1));
      }
    }
    if(prot) myDebug()<<"p1="<<p1.printS();
    if(prot) myDebug()<<"p2="<<p2.printS();
    if(prot) myDebug()<<"delta="<<delta.printS();
    if(prot) myDebug()<<"O1=\n"<<(O1).printS();
    if(prot) myDebug()<<"pm="<<this->printS();
    
    long mR=R.columns();
    long mD=Delta.columns();
    long mR1=p1.R.columns();
    long mD1=p1.Delta.columns();
    
    IMatrix PR1L=IMatrix::P(mR1,mR,mR1);
    IMatrix PD1L=IMatrix::P(mD1,mD,mD1);
    if(prot){
      myDebug()<<"PR1L=\n"<<PR1L.printS();
      myDebug()<<"PD1L=\n"<<PD1L.printS();
    }
    
    long rankR;
    IMatrix VR,GRT,b1;
    FMatrix QGDelta,b2;
    
    //GRT*R*VR=DR
    R.decompGTAVeqD(rankR,VR,GRT);
    O=GRT*O;
    Delta=GRT*Delta;
    
    b1=p1.R*PR1L*VR;
    b2=b1*R.diagInverseT(rankR);
    
    QGDelta=IMatrix::Q(pdim,pdim,pdim-rankR)*Delta;
    
    
    if(prot){
      
      myDebug()<<"rankR="<< rankR;
      myDebug()<<"BR=\n"<<R.printS();
      myDebug()<<"P/PRP=\n"<<R.diagInverseT(rankR).printS();
      myDebug()<<"VR=\n"<<VR.printS();
      myDebug()<<"GRT=\n"<<GRT.printS();
      
      
      myDebug()<<"QRL=\n"<<IMatrix::Q(pdim,pdim,pdim-rankR).printS();
      myDebug()<<"QRR=\n"<<IMatrix::Q(mR,mR-rankR,mR-rankR).printS();
      
      myDebug()<<"GRT*O=\n"<<O.printS();
      myDebug()<<"GRT*Delta=\n"<<Delta.printS();
      myDebug()<<"QR*GRT*Delta=\n"<<QGDelta.printS();
    }
    
    R=b1;R.removeColumns(0,rankR-1);
    
    
    
    IMatrix VD,GDT; 
    long rankQGDelta;
    QGDelta.decompGTAVeqD(rankQGDelta,VD,GDT);
    FMatrix PoPDP=QGDelta.diagInverseT(rankQGDelta);
    FMatrix b3=(p1.Delta*PD1L-b2*Delta)*VD;
    IMatrix QDL=IMatrix::Q(pdim,pdim,pdim-rankQGDelta);
    Delta=b3;Delta.removeColumns(0,rankQGDelta-1);
    if(prot){
      myDebug()<<"rankQGDelta="<< rankQGDelta;
      myDebug()<<"BD=\n"<<QGDelta.printS();
      myDebug()<<"P/PDP=\n"<<PoPDP.printS();
      myDebug()<<"VD=\n"<<VD.printS();
      myDebug()<<"GDT=\n"<<GDT.printS();
      myDebug()<<"QDL=\n"<<QDL.printS();
      myDebug()<<"QDR=\n"<<IMatrix::Q(mD,mD-rankQGDelta,mD-rankQGDelta).printS();
    }
    
    if(prot){
      myDebug()<<"b1=\n"<<b1.printS();
      myDebug()<<"b2=\n"<<b2.printS();
      myDebug()<<"b3=\n"<<b3.printS();
    }
    
    FMatrix GDQRGRdO=GDT*IMatrix::Q(pdim,pdim,pdim-rankR)*O;

    if(prot) myDebug()<<"GDT*QR*GRT*dO=\n"<<GDQRGRdO.printS();

    IMatrix solvable=(QDL*GDQRGRdO).areColumnsZero();

    if(prot) myDebug()<<"solvable=\n"<<solvable.printS();

    if(prot) myDebug()<<"Nraw=\n"<<(-PoPDP*GDQRGRdO).printS();

    IMatrix Np0=(-PoPDP*GDQRGRdO).toIMatrixColumnsWise(solvable);

    if(prot) myDebug()<<"solvable=\n"<<solvable.printS();

    if(prot) myDebug()<<"Np0=\n"<<Np0.printS();


    O=O1-b2*O+b3*Np0;

    if(prot) myDebug()<<"Oraw=\n"<<O.printS();

    O.removeColumns(solvable);
    
    if(prot) myDebug()<<"res before normalize="<<this->printS();
    
    normalize();
    
    
    
    if(prot) myDebug()<<"res="<<this->printS();
    
    
    return *this;
  }    
    
    
    
    
  
  
  PeriodicHyperPlane &normalize(){
    // hyperplane=O+R*x+Delta*N
    
    bool prot=false;

    // make minimal basis
    long rankD;
    Delta.decompAVeqB(rankD);
    Delta.removeColumns(rankD,Delta.columns());
    

    {
      //myDebug()<<"O=\n"<<O.printS();

      
      FMatrix  BD(Delta.Td());
      IMatrix GDT=BD.decompAVeqB(rankD,true).Td();
      FMatrix G=BD.T().diagInverseT(rankD)*GDT;

      //myDebug()<<"Delta=\n"<<BD.printS();
      
      bool gotone=true;
      while(gotone){
	gotone=false;
	for(long i1=0;i1<O.columns();i1++){
	  for(long i2=i1+1;i2<O.columns();i2++){
	    FMatrix t=O.fromColumn(i1)-O.fromColumn(i2);
	    t=t-Delta*(G*t).round();
	    //myDebug()<<"t=\n"<<t.printS();
	    
	    if(t==0) continue;
	    
	    ValueArray<long> m(O.columns(),-1);
	    bool found=true;
	    for(long i3=0;i3<O.columns();i3++){
	      m[i3]=-1;
	      for(long i4=0;i4<O.columns();i4++){
		FMatrix v=O.fromColumn(i3)-O.fromColumn(i4)-t;
		
		v=v-Delta*(G*v).round();
		//myDebug()<<"v=\n"<<v;
		if(v==0){
		  m[i3]=i4;
		  break;
		}
	      }
	      if(m[i3]==-1){
		found=false;
		break;
	      }
	    }
	    if(!found){
	      continue;
	    }else{
	      Delta.addMatrix(t);
	      Delta.decompAVeqB(rankD);
	      Delta.removeColumns(rankD,Delta.columns());
	      BD=Delta.Td();
	      GDT=BD.decompAVeqB(rankD,true).Td();
	      G=BD.T().diagInverseT(rankD)*GDT;
	      //myDebug()<<"Delta=\n"<<BD.printS();
	      
	      FMatrix o;
	      long j=0;
	      while(true){
		//myDebug()<<"while j="<<j;
		while(j<O.columns() && m[j]<0 )j++;
		//myDebug()<<"good j="<<j;
		if(j==O.columns())break;
		o.addMatrix(O.fromColumn(j));
		long k=j;
		while(m[k]>=0){
		  //myDebug()<<"chain k="<<k<<m[k];
		  long d=m[k];m[k]=-1;k=d;
		  //myDebug()<<" new  k="<<k<<m[k];
		}
		j++;
	      }
	      O=o;
	      gotone=true;
	      
	    }
	  }
	}
      }
    }
    
    // make minimal basis
    long rankR;
    // R*V=(B 0)
    R.decompAVeqB(rankR); //discard V by redefining x=V*x
    R.removeColumns(rankR,R.columns());

    // rescale x
    R.shortenColumns();

    // redefine x to make "diagonal elements positive"
    for(long c=0;c<rankR;c++){
      long r=pdim-1; while(R(r,c)==0)r--;
      R.scaleColumn(c,lSign(1,R(r,c)));
    }

    if(prot) myDebug()<<"R'=\n"<<(R).printS();

    IMatrix  BR(R.Td());
    IMatrix GRT=BR.decompAVeqB(rankR,true).Td();
    BR.T();

    if(prot){
      myDebug()<<"GRT=\n"<<GRT.printS();
      myDebug()<<"BR=\n"<<BR.printS();
      myDebug()<<"P/BR=\n"<<BR.diagInverseT(rankR);
      myDebug()<<"R*P/BR*GRT=\n"<<R*BR.diagInverseT(rankR)*GRT;
    }



    


    if(prot) {
      myDebug()<<"Delta'=\n"<<Delta.printS();
      myDebug()<<"QRL=\n"<<IMatrix::Q(pdim,pdim,pdim-rankR).printS();
      myDebug()<<"GRT*Delta=\n"<<(GRT*Delta).printS();
      myDebug()<<"Q*GRT*Delta=\n"<<(GRT*Delta).removeRows(0,rankR-1).printS();
    }
    
    
    IMatrix VD=(GRT*Delta).removeRows(0,rankR-1).decompAVeqB(rankD);
    
    if(prot) myDebug()<<"rankD=\n"<<rankD;
    if(prot) myDebug()<<"mD=\n"<<Delta.columns();
    if(prot) myDebug()<<"VD=\n"<<VD.printS();

    VD.removeColumns(rankD,Delta.columns()-1);


    if(prot) myDebug()<<"VDP=\n"<<VD.printS();

    Delta=Delta*VD;

    if(prot) myDebug()<<"Delta*VDP=\n"<<Delta.printS();


    // redefine x to make "diagonal elements positive"
    for(long c=0;c<rankD;c++){
      long r=pdim-1; while(Delta(r,c)==0)r--;
      Delta.scaleColumn(c,Delta(r,c).sign());
    }

    if(prot) myDebug()<<"new Delta=\n"<<Delta.printS();


    // remove nonorthogonal Delta-components from O
    FMatrix  BD(Delta.Td());
    IMatrix GDT=BD.decompAVeqB(rankD,true).Td();
    BD.T();

    if(prot) myDebug()<<"N=\n"<<(BD.diagInverseT(rankD)*GDT*O);
    if(prot) myDebug()<<"N.round()=\n"<<(BD.diagInverseT(rankD)*GDT*O).round();
    

    // remove nonorthogonal R-components from O
    O-=R*BR.diagInverseT(rankR)*GRT*O
      +Delta*(BD.diagInverseT(rankD)*GDT*O).round();
    for(long i2=O.columns()-1;i2>=0;i2--){
      for(long i1=0;i1<i2;i1++){
	if(O.fromColumn(i2)-O.fromColumn(i1)==0){
	  O.removeColumns(i2,i2);
	  break;
	}
      }
    }
    if(O.columns()==0){
      reset(pdim);
    }

    return *this;
  }

  
  String printS(long padding=6)const{
    String s="\n";
    for(int r=0;r<pdim;r++){
      s+="|";
      for(int c=0;c<O.columns();c++){
	s+=Sprintf("%s ",O(r,c).printS().leftPadd(' ',padding)());
      }
      s+=(r==1?"|  +  |":"|     |");
      for(int c=0;c<R.columns();c++){
	s+=Sprintf("%5ld ",R(r,c));
      }
      if(r==1)
	s+="| *x +  |";
      else
	s+="|       |";
      
      for(int c=0;c<Delta.columns();c++){
	s+=Sprintf("%s ",Delta(r,c).printS().leftPadd(' ',padding)());
      }
      if(r==1)
	s+="| *N\n";
      else
	s+="|\n";
    }
    return s;
  }

  String printS(const FMatrix &B)const{
    String s="\n";
    FMatrix o=B*O;
    FMatrix rr=B*R;
    FMatrix d=(B*Delta);
    for(int r=0;r<pdim;r++){
      s+="|";
      for(int c=0;c<o.columns();c++){
	s+=Sprintf("%s ",o(r,c).printS().leftPadd(' ',6)());
      }
      s+=(r==1?"|  +  |":"|     |");
      for(int c=0;c<rr.columns();c++){
	s+=Sprintf("%s ",rr(r,c).printS().leftPadd(' ',6)());
      }
      if(r==1)
	s+="| *x +  |";
      else
	s+="|       |";
      
      for(int c=0;c<Delta.columns();c++){
	s+=Sprintf("%s ",d(r,c).printS().leftPadd(' ',6)());
      }
      if(r==1)
	s+="| *N\n";
      else
	s+="|\n";
    }
    return s;
  }
  
public:
  long pdim; //space dimension
  IMatrix R;
  FMatrix O; 
  FMatrix Delta;
  /* R spans the hyperplane.
     O is a point on the hyperplane, defining the origin.
     Delta defines the periodic repetition of the plane.
     Only integer linear combinations are allowed;
     A point on a hyperplane is r=O+R*\vec{lambda}+Delta*\vec{N},
     where \vec{lambda} can be any number.
  */ 
};

inline MyDebug operator<<(MyDebug dbg,const PeriodicHyperPlane &p){
  dbg<<p.printS(6);
  return dbg;
}


#endif /* _PERIODICHYPERPLANE_HH */

