

//todo: check that orientation is not inverted!

//	todo: determine e-glides and d-glides


//todo: the PeriodicHyperPlane.R of the origin tells us, which
// further freedom of shift we have and .Delta tells, wether there are
// equivalent origins, unless we restricted them, which we did

/*************************************************************************
 * @(#)SeitzMatrix.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Aug 2012
 *************************************************************************
 */

#include <ctype.h>
#include "SeitzMatrix.hh"
#include "Fractional.hh"
#include "HallSymbol.hh"
#include "IntegerTricks.hh"
#include "IMatrix.hh"
#include "FMatrix.hh"
#include "PeriodicHyperPlane.hh"



using namespace SpaceGroup;

#define ASSERT(cond,msg) if(!(cond)) throw Except(msg);





typedef ValueArray<long> LongArray;
typedef ValueArray<FMatrix> FMatArray;
typedef ValueArray<IMatrix> IMatArray;






/* ***********************************************************************
 *
 * ***********************************************************************
 */
static const char *EXPECTING[]={
  "sign or integer/fractional or axis-symbol",
  "axis-symbol",
  "unsigned integer/fractional",
  "unsigned integer",
  " ---oops interal error--- "
};

static const long BEGIN=0;
static const long XYZ=1;
static const long UNSIGNEDFRAC=2;
static const long UNSIGNEDINTEGER=3;
static const long UNKOWN=4;

static long nprocount=0;
static long ncmpcount=0;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SeitzMatrix::groupAddOperation(ValueArray<SeitzMatrix> &ops,
				    const SeitzMatrix &op)
{
  //myDebug()<<"groupAddOperation adding "<<op;

  while(true){
    bool hadnew=false;
    long i=0;
    while(i<ops.size()){
      //myDebug()<<" II "<<i;
      SeitzMatrix prod=(ops.at(i)*op);
      //myDebug()<<" opsi "<<ops.at(i)<<"* opsj"<<ops.at(j)<<" = "<<prod;
      nprocount++;
      {
	bool isnew=true;
	for(long k=0;k<ops.size();k++){
	  //myDebug()<<" prod "<<prod<<" opsk"<<ops.at(k);
	  ncmpcount++;
	  if(ops.at(k)==prod){
	    //myDebug()<<" prod "<<prod<<" not new "<<ops.at(k);
	    isnew=false;
	    break;
	  }
	}
	if(isnew){
	  ops.append(prod);
	  hadnew=true;
	}
      }
      i++;
    }
    if(!hadnew) break;
  }
  //myDebug()<<"prod count="<<nprocount;
  //myDebug()<<"cmp count="<<ncmpcount;

  //myDebug()<<"end groupAddOperation ";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long parseFrac(const char *p,Fractional &f)
{
  long L=0;
  //myDebug()<<"parseFrac: p="<<p;
  f=0;
  if(*p=='\0')return L; //nothing found
  
  //myDebug()<<"parseFrac: p="<<p<<" not empty";
  char *endptr;
  f=strtol(p,&endptr,10);
  //myDebug()<<"parseFrac: n="<<n<<" endptr="<<String(endptr)<<" endptr-p"<<endptr-p;
  L=endptr-p;
  if(!L)return L; //no number at beginning
  if(*endptr=='\0'){
    //myDebug()<<"return int";
    return L; // pure integer, nothing follows
  }
  if(*endptr!='/')return L;//something else is following
  char *p0=endptr+1;
  if(*p0=='\0')return L;//after / must be a number, return and let them 
  // figure it out
  
  long m=strtol(p0,&endptr,10);
  if(endptr==p0){
    return L; //after / must be a number, return and let them 
    // figure it out
  }
  f=Fractional(f.n(),m);
  return endptr-p;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SeitzMatrix::SeitzMatrix(const String &xyz)
  :pactive(true),pinteger(true),panalysed(false)
{// convert xyz symbol to SeitzMatrix

  pW.setDimenions(3,3);
  pw.setDimenions(3,1);

  // x'=W(0,*)
  // y'=W(1,*)
  // z'=W(2,*)

  //myDebug()<<"SeitzMatrix::SeitzMatrix(const String &xyz)"<<xyz;
  StringList sl=xyz.lowerCase().removeFromSet(" \t")
    .replace("a","x")
    .replace("b","y")
    .replace("c","z").split(",");
  if(sl.size()!=3)
    throw Except("xyz-symbol needs 3 ','-seprated components!");


  
  for(long k=0;k<3;k++){
    //myDebug()<<"k="<<k;
    const char *p0=sl[k]();
    const char *p=sl[k]();
    
    long state=BEGIN;
    bool haverotpart=false;
    Fractional fac=1;
    long colref=0;
    while(true){

      //myDebug()<<"while start:"<<p<<"state:"<<state;
      
      Fractional f;
      long l;
      
      if((state==UNSIGNEDFRAC||state==UNSIGNEDINTEGER) && 
	 (*p=='-' || *p=='+')){ goto error; }
      
      if((l=parseFrac(p,f))>0){
	if(state==UNSIGNEDINTEGER && f.m()!=1){ goto error; }
	p+=l;
	//myDebug()<<"frac"<<f.n()<<"/"<<f.m()<<"rest"<<p;
	if(*p=='\0') {
	  if(state==BEGIN){
	    pw[k]+=f;
	    break;
	  }else if(state==UNSIGNEDFRAC){
	    pW(k,colref)*=f;
	    break;
	  }else if(state==UNSIGNEDINTEGER){
	    pW(k,colref)/=f;
	    break;
	  }else{
	    state=UNKOWN;
	    goto error;
	  }
	}else if(*p=='-' || *p=='+'){
	  if(state==BEGIN){
	    pw[k]+=f;
	    fac=1;
 	  }else if(state==UNSIGNEDFRAC){
	    pW(k,colref)*=f;
 	  }else if(state==UNSIGNEDINTEGER){
	    pW(k,colref)/=f;
	  }else{
	    state=UNKOWN;
	    goto error;
	  }
	  state=BEGIN;
	  continue;
	}else{
	  if(state==BEGIN){
	    if(*p=='*') p++;
	    fac=f;
	    state=XYZ;
	    continue;
	  }else{
	    state=BEGIN;
	    goto error;
	  }
	}
      }
      
      if(state==UNSIGNEDFRAC || state==UNSIGNEDINTEGER){ goto error; }
      
      if(state==BEGIN && *p=='-'){
	fac=-1;
	p++;
	state=XYZ;
      }
      if(state==BEGIN && *p=='+'){
	fac=1;
	p++;
	state=XYZ;
      }
      
      if((state==XYZ || state==BEGIN) && (*p=='x' || *p=='y' || *p=='z')){
	haverotpart=true;
	//myDebug()<<"rot "<<fac.n()<<"/"<<fac.m()<<*p;
	if(*p=='x') colref=0;
	if(*p=='y') colref=1;
	if(*p=='z') colref=2;
	pW(k,colref)=fac;
	//myDebug()<<"rot "<<facref->n()<<"/"<<facref->m()<<*p;
	p++;
	if(*p=='\0'){
	  break;
	}else if(*p=='*'){
	  p++;
	  //collect the next frac and finalize
	  state=UNSIGNEDFRAC;
	  continue;
	}else if(*p=='/'){
	  p++;
	  //collect int and finalize
	  state=UNSIGNEDINTEGER;
	  continue;
	}else if(*p=='+'){
	  // finalize and 
	  fac=1;
	  state=BEGIN;
	  continue;
	}else if(*p=='-'){
	  // finalize and 
	  fac=1;
	  state=BEGIN;
	  continue;
	}
      }
      
    error:;
      throw Except(Sprintf("Invalid xyz-symbol expecting %s at ^ in '%s ^ %s'.",
			   EXPECTING[state],String(p0,p-p0)(),p));
      
    }
    if(!haverotpart) {
      throw Except("Invalid xyz-symbol (missing rotation part)!");
    }
    //    myDebug()<<"DONE "<<res;
      
    
  }


  if(!pW.isInteger())
    throw Except("SeitzMatrix: Rotation matrix is not integer?");

  analyse(false);
  
  //myDebug()<<toXYZSymbol()<<"\n"<<printS()<<"\n\n";

  return;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SeitzMatrix::analyse(bool prot)
{
  // only works with lattices, not pointgroups!
  if(!pactive)return;

  if(prot)myDebug()<<"=================== analyse =======================";
  //myDebug()<<"W|w"<<*this;

  pinfo.pw0list.clear();
  pinfo.porlist.clear();
  
  IMatrix IW=pW.toIMatrix(); // implicit integer check in toLong

  long det=IW.determinant();
  if(labs(det)!=1) throw Except("SeitzMatrix: |det(operation)| != 1?");
  //myDebug()<<"det"<<det;

  long tr=IW.trace()*det;
  long c2=(tr-1);
  switch(c2){
  case 2:pinfo.pnorder=1*det;break;
  case 1:pinfo.pnorder=6*det;break;
  case 0:pinfo.pnorder=4*det;break;
  case -1:pinfo.pnorder=3*det;break;
  case -2:pinfo.pnorder=2*det;break;
  default:
    throw Except(Sprintf("SeitzMatrix: unknown rotation order trace=%ld",tr)());
  }
  //myDebug()<<"trace"<<tr;
  //myDebug()<<"c2"<<c2;
  //myDebug()<<"order"<<pinfo.pnorder;

  IMatrix OnemIW(3,3);
  if(labs(pinfo.pnorder)!=1){
    // determine the shortest lattice vector along the symmetry axis
    OnemIW=IMatrix::diagMatrix(3,3,1)-IW*lSign(1,pinfo.pnorder);
    pinfo.Pmin=OnemIW.fromRow(0).cross(OnemIW.fromRow(1));
    if(mhnorm(pinfo.Pmin)==0)
      pinfo.Pmin=OnemIW.fromRow(0).cross(OnemIW.fromRow(2));
    if(mhnorm(pinfo.Pmin)==0)
      pinfo.Pmin=OnemIW.fromRow(1).cross(OnemIW.fromRow(2));
    if(mhnorm(pinfo.Pmin)==0)
      throw Except("SeitzMatrix: rotation axis is zero");
    pinfo.Pmin.shorten();
    // determine the symmetry axis in a sign convention
    // if all nonzero make more -signs
    if(pinfo.Pmin[0]!=0 && pinfo.Pmin[1]!=0 && pinfo.Pmin[2]!=0 ){
      long s=lSign(1,pinfo.Pmin[0])+lSign(1,pinfo.Pmin[1])
	+lSign(1,pinfo.Pmin[2]);
      if(s==-3 || s==1)pinfo.Pmin*=-1;
    }
    if(pinfo.Pmin[0]==0){
      if(pinfo.Pmin[1]==0){
	pinfo.Pmin*=lSign(1,pinfo.Pmin[2]);
      }else{
	pinfo.Pmin*=lSign(1,pinfo.Pmin[1]);
      }
    }
    if(pinfo.Pmin[1]==0){
      if(pinfo.Pmin[2]==0){
	pinfo.Pmin*=lSign(1,pinfo.Pmin[0]);
      }else{
	pinfo.Pmin*=lSign(1,pinfo.Pmin[2]);
      }
    }
    if(pinfo.Pmin[2]==0){
      if(pinfo.Pmin[0]==0){
	pinfo.Pmin*=lSign(1,pinfo.Pmin[1]);
      }else{
	pinfo.Pmin*=lSign(1,pinfo.Pmin[0]);
      }
    }

    //determine Gmin from the inverse operation in G representation
    // IWG(op^-1)=IW^T
    pinfo.Gmin=OnemIW.fromColumn(0).cross(OnemIW.fromColumn(1));
    if(mhnorm(pinfo.Gmin)==0)
      pinfo.Gmin=OnemIW.fromColumn(0).cross(OnemIW.fromColumn(2));
    if(mhnorm(pinfo.Gmin)==0)
      pinfo.Gmin=OnemIW.fromColumn(1).cross(OnemIW.fromColumn(2));
    if(mhnorm(pinfo.Gmin)==0)
      throw Except("SeitzMatrix: reciprocal rotation axis is zero");
    pinfo.Gmin.shorten();
    pinfo.pp=pinfo.Pmin.dot(pinfo.Gmin);
    if(pinfo.pp<0){
      pinfo.Gmin*=-1;
      pinfo.pp*=-1;
    }
    if(labs(pinfo.pnorder)%pinfo.pp!=0)
      throw Except("SeitzMatrix: p is not prime factor of rotation order?");

    //myDebug()<<"Pmin="<<pinfo.Pmin;
    //myDebug()<<"Gmin="<<pinfo.Gmin;
    //myDebug()<<"p="<<pinfo.pp;

    //determin the "operation-lattice"
    {
      IMatrix g=pinfo.Gmin.Td();
      //myDebug()<<"g="<<g;
      long rank;
      pinfo.RA=g.decompAVeqB(rank).cycleColumnsL();
      //myDebug()<<"rank="<<rank;
      //myDebug()<<"B="<<g;
      //myDebug()<<"V="<<pinfo.RA;
      if(g(0,0)!=1)throw Except("SeitzMatrix: Gmin is not shortest vector?");

      long d=pinfo.RA.determinant();
      if(d<0){
	d*=-1;
	pinfo.RA.scaleColumn(1,-1);
      }
      if(pinfo.RA.determinant()!=1)
	throw Except("SeitzMatrix: det(RA)!=1?");
      if(pinfo.Gmin.dot(pinfo.RA.fromColumn(2))!=1)
	throw Except("SeitzMatrix: Gmin*RA(3)!=1?");
      if(pinfo.Gmin.dot(pinfo.RA.fromColumn(0))!=0)
	throw Except("SeitzMatrix: Gmin*RA(1)!=0?");
      if(pinfo.Gmin.dot(pinfo.RA.fromColumn(1))!=0)
	throw Except("SeitzMatrix: Gmin*RA(2)!=0?");
      
      pinfo.GA=pinfo.RA.inverseTtV(d);
      if(d!=1)
	throw Except("SeitzMatrix: det(GA)!=1?");
    }
    //myDebug()<<"RA=\n"<<pinfo.RA;
    // RA,GA depend on wether centering is inclued or excluded


  }
  
  //determine sense of rotation
  pinfo.pm1
    =lSign(1,pinfo.Pmin.dot(pinfo.RA.fromColumn(0)
			    .cross(IW*pinfo.RA.fromColumn(0)))
	   *lSign(1,pinfo.pnorder));
  

  FMatrix test(pinfo.RA.fromColumn(0),pinfo.RA.fromColumn(1),pinfo.Pmin);
  if(test.determinant()<0)
    throw Except("Something is wrong with the orientation!");

  //determine the origin of operation and the essential w0
  FMatrix Ppar=FMatrix(pinfo.Pmin*pinfo.Gmin.Td())/pinfo.pp;
  FMatrix Pper=FMatrix::diagMatrix(3,3,1)-Ppar;

  // w=w0+(1-W)*origin
  if(pinfo.pnorder==1){
    pinfo.pw00=pw;
    pinfo.porigin0=0;
    pinfo.pw0list.append(pw);
    pinfo.porlist.append(pinfo.porigin0);
  }else if(pinfo.pnorder>1){
    //       myDebug()<<"here";
    //       myDebug()<<"w"<<w;
    //myDebug()<<"Pmin=\n"<<pinfo.Pmin;
    //myDebug()<<"Gmin=\n"<<pinfo.Gmin;
    //       myDebug()<<"w*Gmin"<<(w*pinfo.Gmin);
    //       myDebug()<<"n,p,n/p"<<order()<<pinfo.pp<<(order()/pinfo.pp);
    //       myDebug()<<"x"<<((w*pinfo.Gmin)*(order()/pinfo.pp));
    // when taking the primitive centered lattice as basis, 
    // the non trivial rotation tau0 must be int/order()
    // if the centering vectorsa are taken as generators and the basis is the
    // conventional lattice the tau0 of rotations about non principal
    // axes can  be fractionals of int/norder().
    
    //myDebug()<<"Ppar=\n"<<Ppar;
    //myDebug()<<"pw="<<pw;
    pinfo.pw00=Ppar*pw;
    //myDebug()<<"pinfo.pw00="<<pinfo.pw00;
    
    
    
    
    
    Fractional dd;
    FMatrix XA=(Ppar+OnemIW).inverse(dd);
    if(XA*pinfo.Pmin!=pinfo.Pmin){
      myDebug() << "Par=\n"<<Ppar;
      myDebug() << "dd="<<dd;
      myDebug() << "XA=\n"<<XA;
      myDebug() << "XA*Ppar=\n"<<XA*Ppar;
      myDebug() << "XA*OnemIW=\n"<<XA*OnemIW;
      throw Except("SeitzMatrix:: inversion of 1-alpha failed!");
    }
    if(XA*OnemIW!=Pper){
      myDebug() << "Par=\n"<<Ppar;
      myDebug() << "dd="<<dd;
      myDebug() << "XA=\n"<<XA;
      myDebug() << "XA*Ppar=\n"<<XA*Ppar;
      myDebug() << "XA*OnemIW=\n"<<XA*OnemIW;
      throw Except("SeitzMatrix:: inversion of 1-alpha failed!");
    }
    //       myDebug() << "Par=\n"<<Ppar;
    //       myDebug() << "det="<<dd;
    //       myDebug() << "XA=\n"<<XA;
    //       myDebug() << "XA*Ppar=\n"<<XA*Ppar;
    //       myDebug() << "XA*OnemIW=\n"<<XA*OnemIW;
    
    pinfo.porigin0=XA*Pper*pw;
    
    
    //myDebug() << "pinfo.porigin0="<<pinfo.porigin0;
    
    
    
    pinfo.Delta=XA*pinfo.RA;
    
    //myDebug() << "pinfo.RA=\n"<<pinfo.RA;
    //myDebug() << "pinfo.Delta=\n"<<pinfo.Delta;
    
    FMatrix ff=pinfo.Delta.Td()*pinfo.Gmin;
    if(ff[0]!=0 || ff[1]!=0){
      myDebug()<<"Gmin*Delta="<<ff;
      throw Except("SeitzMatrix:: assumption about Delta vectors is wrong!");
    }
    
    
    
    //myDebug()<<"L3max="<<pinfo.pp;
    //myDebug()<<"nonsym="<<pinfo.pnonsymmorphic;
    // the operation appears at L*R3_perp with L<L3max
    pinfo.pnonsymmorphic=true;
    for(int L3=0;L3<pinfo.pp;L3++){
      //myDebug()<<"L3="<<L3;
      FMatrix pw0=pinfo.pw00+(FMatrix(pinfo.Pmin)*L3)/pinfo.pp;
      
      FMatrix por=pinfo.porigin0+Pper*pinfo.Delta.fromColumn(2)*L3;
      if(pw0.regularizedInterval(false)==0){
	pinfo.pnonsymmorphic=false;
	pinfo.pw0list.prepend(pw0);
	pinfo.porlist.prepend(por);
      }else{
	pinfo.pw0list.append(pw0);
	pinfo.porlist.append(por);
      }
    }
    pinfo.pw00=pinfo.pw0list.at(0);
    pinfo.porigin0=pinfo.porlist.at(0);
    
    //myDebug()<<"pw0 "<<(pinfo.pnonsymmorphic?"non symmorphic":"symmorphic");
    for(int i=0;i<pinfo.pw0list.size();i++){
      if(pinfo.pw0list.at(i).regularizedInterval(false)==0){
	pinfo.pw0list[i].regularizeInterval(false);
      }
    }
    //myDebug()<<"pw0list="<<pinfo.pw0list;
    //myDebug()<<"porlist="<<pinfo.porlist;
    
    pinfo.ptau0=(((pinfo.Gmin.dot(pinfo.pw00))/pinfo.pp)
		 .regularizeInterval(false))*labs(pinfo.pnorder);
    
    //myDebug()<<"pinfo.ptau0="<<pinfo.ptau0;
    
    
    
    
  }else if(pinfo.pnorder==-1){
    //inversion
    pinfo.porigin0=pw/2;
    pinfo.Delta=FMatrix::diagMatrix(3,3,1)/2;
    pinfo.pw0list.append(FMatrix(3,1));
    pinfo.porlist.append(pinfo.porigin0);
    
    //myDebug()<<"Delta=\n"<<pinfo.Delta;
    //myDebug()<<"pw0list="<<pinfo.pw0list;
    //myDebug()<<"porlist="<<pinfo.porlist;
    
  }else if(pinfo.pnorder==-2){
    //mirror
    pinfo.pw00=Pper*pw;
    pinfo.porigin0=Ppar*pw/2;
    pinfo.Delta.setColumn(0,FMatrix(pinfo.Pmin)/2);
    
    for(int L3=0;L3<pinfo.pp;L3++){
      FMatrix pw0=pinfo.pw00+Pper*pinfo.RA.fromColumn(2)*L3;
      FMatrix por=pinfo.porigin0+pinfo.Delta.fromColumn(0)
	*Fractional(L3,pinfo.pp);
      pinfo.pw0list.append(pw0);
      pinfo.porlist.append(por);
    }
    
    //sort according to complexity of pw0
    for(int i=0;i<pinfo.pw0list.size();i++){
      for(int j=i+1;j<pinfo.pw0list.size();j++){
	if(pinfo.pw0list.at(i).moreComplex(pinfo.pw0list.at(j))){
	  pinfo.pw0list.swap(i,j);
	  pinfo.porlist.swap(i,j);
	}
      }
    }
    pinfo.pnonsymmorphic=true;
    if(pinfo.pw0list.at(0).regularizedInterval(false)==0){
      pinfo.pnonsymmorphic=false;
    }
    
    
    
    pinfo.pw00=pinfo.pw0list.at(0);
    pinfo.porigin0=pinfo.porlist.at(0);
    
    //myDebug()<<"pw0 "<<(pinfo.pnonsymmorphic?"non symmorphic":"symmorphic");
    for(int i=0;i<pinfo.pw0list.size();i++){
      if(pinfo.pw0list.at(i).regularizedInterval(false)==0){
	pinfo.pw0list[i].regularizeInterval(false);
      }
    }
    //myDebug()<<"pw0list="<<pinfo.pw0list;
    //myDebug()<<"porlist="<<pinfo.porlist;
    
    
    
  }else{
    //rotoinversion
    Fractional d;
    FMatrix OnemW=FMatrix::diagMatrix(3,3,1)-IW;
    FMatrix XA=OnemW.inverse(d);
    pinfo.pw00=XA*pw; // this is actually the inversion center
    
    pinfo.porigin0=Pper*pinfo.pw00;
    
    pinfo.Delta=XA*pinfo.RA;
    
    
    FMatrix ff=pinfo.Delta.Td()*pinfo.Gmin;
    if(ff[0]!=0 || ff[1]!=0){
      myDebug()<<"Gmin*Delta="<<ff;
      throw Except("SeitzMatrix:: assumption about Delta vectors is wrong!");
    }
    
    
    //myDebug()<<"L3max="<<pinfo.pp;
    //myDebug()<<"nonsym="<<pinfo.pnonsymmorphic;
    //myDebug()<<"Delta=\n"<<pinfo.Delta;
    //myDebug()<<"inversion0="<<pinfo.pw00;
    // the center appears at L*XA*R3 with L<L3max
    pinfo.pnonsymmorphic=true; //todo: shouldn't that be false
    for(int L3=0;L3<pinfo.pp;L3++){
      FMatrix pw0=pinfo.pw00+pinfo.Delta.fromColumn(2)*L3;
      FMatrix por=Pper*pw0;
      pinfo.pw0list.append(pw0);
      pinfo.porlist.append(por);
    }
    pinfo.pw00=pinfo.pw0list.at(0);
    pinfo.porigin0=pinfo.porlist.at(0);
    
    //myDebug()<<"pw0 "<<(pinfo.pnonsymmorphic?"non symmorphic":"symmorphic");
    // no regularizeInterval since pw0 and origin have to be
    // regularized via Delta, which is more work and unnessecary
    //myDebug()<<"pw0list="<<pinfo.pw0list;
    //myDebug()<<"porlist="<<pinfo.porlist;
    
    
    //the inversion point on the axis repeats every pinfo.Delta.column(2)
    pinfo.Delta.setColumn(2,FMatrix(pinfo.Pmin)/2);
    
  }


  panalysed=true;
  if(prot){
    myDebug()<<"\nIT:"<<pinfo.symbol();
    myDebug()<<printWwS();
    myDebug()<<"Pmin          ="<<pinfo.Pmin;
    myDebug()<<"Gmin          ="<<pinfo.Gmin;
    myDebug()<<"pp            ="<<pinfo.pp;
    myDebug()<<"Delta=\n"<<pinfo.Delta;
    myDebug()<<"pw0list       ="<<pinfo.pw0list;
    myDebug()<<"porlist       ="<<pinfo.porlist;
    myDebug()<<"pinfo.ptau0="<<pinfo.ptau0;
    myDebug()<<"pinfo.pw00    ="<<pinfo.pw00
	     <<(pinfo.pnonsymmorphic?"non symmorphic":"symmorphic");
    myDebug()<<"pinfo.porigin0="<<pinfo.porigin0;
    myDebug()<<"=================== END analyse =======================";
  }
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String  SeitzMatrixInfo::symbol(bool sshort)const
{
  String res;
  if(sshort){
    if(pnorder==1) res="E";
    else if(pnorder==-1) res="I";
    else if(pnorder>1) res=Sprintf("C%ld%s",pnorder,(pm1>0?"":"-"));
    else if(pnorder<-2) res=Sprintf("%ld%s",pnorder,(pm1>0?"":"-"));
    else res="m";
    return res+locationSymbol(FMatrix(3,1),FMatrix(3,1),true);
  }
  for(long ii=0;ii<pw0list.size();ii++){
    
    const FMatrix &w0=pw0list.at(ii);
    const FMatrix &orig=porlist.at(ii);
    if(pnorder==1){
      // translation
      if(w0==0){
	res+="1";
      }else{
	res+="t"+w0.printS(0,"()",",",true);
      }
    }else if(pnorder==-1){
      
      // inversion
      res+=Sprintf("-1 %s,%s,%s",
		   orig[0].printS()(),
		   orig[1].printS()(),
		   orig[2].printS()()
		   );
    }else{
      // all the rest
      String cs;
      if(pnorder!=-2){
	cs.sprintf("%ld%s%s ",
		   pnorder,(labs(pnorder)!=2?(pm1==1?"+":"-"):""),
		   (w0==0||pnorder<0?" ":w0.Td().printS(0,"()",",")()));
      }else{
	cs=mirrorSignature(IMatrix::diagMatrix(3,3,1),w0,orig);
	if(cs.scan("ndg")>=0){
	  cs+=Sprintf(" %s ",w0.printS(0,"()",",",true)());
	}else{
	  cs+=" ";
	}
      }
      res+=cs+locationSymbol(w0,orig);
    }
    if(ii<pw0list.size()-1)res+=" , ";
  }
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SeitzMatrixInfo::mirrorSignature(const FMatrix &Bcenter,
					const FMatrix &w00,
					const FMatrix &orig)const
{
  // this only works as exepected when the conventional axis are used
  // 	todo: determine the symbol of the glide
  FMatrix w=Bcenter*w00;
  FMatrix o=orig;
  if(w==0){
    return "m";
  }else if(abs(w[0])==Fractional(1,2) && w[1]==0 && w[2]==0){
    return "a";
  }else if(w[0]==0 && abs(w[1])==Fractional(1,2) && w[2]==0){
    return "b";
  }else if(w[0]==0 && w[1]==0 && abs(w[2])==Fractional(1,2)){
    return "c";
  }else{
    if((w*2).isInteger()){
      return "n";
    }else if((w*4).isInteger()){
      //myDebug()<<"w"<<w;
      //myDebug()<<"orig"<<o;
      //myDebug()<<"test"<<((w0+o*2)*2);
      if(((w+o*2)*2).isInteger()){
	return "g";
      }else{
	return "d";
      }
    }else{
      return "g";
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SeitzMatrixInfo::mirrorSignatures(const FMatrix &Bcenter)const
{
  String s;
  for(int ii=0;ii<pw0list.size();ii++){
    FMatrix pw0=pw0list.at(ii);//.regularizedInterval(false);
    //todo: this should be forbidden, since backfolding must be done
    // in the tou0 space ... best witht the already determined lattice
    // vectos !!!
    s+=mirrorSignature(Bcenter,pw0,porlist.at(ii));
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SeitzMatrixInfo::locationSymbol(const FMatrix &w0,
				       const FMatrix &orig,bool sshort)const
{
  if(labs(pnorder)==1)return "";
  char c='x';
  String res;
  if(Pmin[0]==0){
    c='y';
    if(Pmin[1]==0){
      c='z';
    }
  }
  
  if(pnorder>1){
    //rotations
    
    FMatrix po=orig;
    if(Pmin[2]!=0 && po[2]!=0){
      //this is for getting the notation of IT
      po+=-(FMatrix(Pmin)*po[2])/Pmin[2];
    }
    if(Pmin[0]!=0 && Pmin[1]!=0 && Pmin[2]==0 ){
      po+=-(FMatrix(Pmin)*po[0])/Pmin[0];
      
    }
    if(Pmin[0]==0 && Pmin[1]!=0 && Pmin[2]==0 ){
      po+=-(FMatrix(Pmin)*po[1])/Pmin[1];
    }
    if(Pmin[0]!=0 && Pmin[1]==0 && Pmin[2]==0 ){
      po+=-(FMatrix(Pmin)*po[0])/Pmin[0];
    }
    if(sshort)
      return res+addlocdirshort(Pmin);
    else{
      return res+addlocdir(Pmin[0],c,po[0],true)
	+addlocdir(Pmin[1],c,po[1],true)
	+addlocdir(Pmin[2],c,po[2],false);
    }
  }else if(pnorder < -2){
    //rotoinversions
    
    FMatrix po=orig;
    if(Pmin[2]!=0 && po[2]!=0){
      //this is for getting the notation of IT
      po+=-(FMatrix(Pmin)*po[2])/Pmin[2];
    }
    
    if(sshort) return res+addlocdirshort(Pmin);
    else{
      return res+addlocdir(Pmin[0],c,po[0],true)
	+addlocdir(Pmin[1],c,po[1],true)
	+addlocdir(Pmin[2],c,po[2],false)
    	+Sprintf("; %s,%s,%s",
		 w0[0].printS()(),
		 w0[1].printS()(),
		 w0[2].printS()()
		 );
    }
  }else{//mirrors
    //myDebug()<<"Gmin"<<Gmin;
    if(Gmin[2]==0){
      char c='x';
      long r20=labs(Gmin[1]);
      long r21=-lSign(1,Gmin[1])*Gmin[0];
      if(Gmin[1]==0) {
	c='y';
	r21=labs(r21);
      }
      // Gmin=[g0,g1,0]
      // Rperp1=[0,0,1]
      // Rperp2=[r20,r21,0], s=sign(g0), r20=|g1|, r21=-s*g0
      // origin caneb shifted by Rperp1,2
      // [o0,o1,o2]+a[0,0,1]+b[r20,r21,0]=[u,v,0], with v=0 if possible
      Fractional o0=orig[0];
      Fractional o1=orig[1];
      if(r21!=0){
	o0-=(o1*r20)/r21;
	o1=0;
      }else if(r20!=0){
	o0=0;
	o1-=(o0*r21)/r20;
      }
      if(sshort) return addlocdirshort(Pmin);
      else{
	return 
	  addlocdir(r20,c,o0,true)+
	  addlocdir(r21,c,o1,true)+
	  addlocdir(1L,'z',0,false);
      }
    }else if(Gmin[0]==0){
      char c='y';
      long r21=labs(Gmin[2]);
      long r22=-lSign(1,Gmin[2])*Gmin[1];
      if(Gmin[2]==0){
	c='z';
	r22=labs(r22);
      }
      // Gmin=[0,g1,g2]
      // Rperp1=[1,0,0]
      // Rperp2=[0,r21,r22], s=sign(g2), r21=|g2|, r22=-s*g1
      // origin caneb shifted by Rperp1,2
      // [o0,o1,o2]+a[1,0,0]+b[0,r21,r22]=[0,u,v], with v=0 if possible
      Fractional o1=orig[1];
      Fractional o2=orig[2];
      if(r22!=0){
	o1-=(o2*r21)/r22;
	o2=0;
      }else if(r21!=0){
	o1=0;
	o2-=(o1*r22)/r21;
      }
      if(sshort) return addlocdirshort(Pmin);
      else return 
	     addlocdir(1L,'x',0,true)+
	     addlocdir(r21,c,o1,true)+
	     addlocdir(r22,c,o2,false);
    }else if(Gmin[1]==0){
      char c='x';
      long r20=labs(Gmin[2]);
      long r22=-lSign(1,Gmin[2])*Gmin[0];
      if(Gmin[2]==0){
 	c='z';
	r22=labs(r22);
      }
      // Gmin  =[g0,0,g2]
      // Rperp1=[0 ,1,0]
      // Rperp2=[r20,0,r22], s=sign(g2), r20=|g2|, r22=-s*g0
      // origin can be shifted by Rperp1,2
      // [o0,o1,o2]+a[0,1,0]+b[r20,0,r22]=[u,0,v], with v=0 if possible
      Fractional o0=orig[0];
      Fractional o2=orig[2];
      if(r22!=0){
	o0-=(o2*r20)/r22;
	o2=0;
      }else if(r20!=0){
	o0=0;
	o2-=(o0*r22)/r20;
      }
      if(sshort) return addlocdirshort(Pmin);
      else return 
	addlocdir(r20,c,o0,true)+
	addlocdir(1L,'y',0,true)+
	addlocdir(r22,c,o2,false);
    }else{
      return Sprintf("Perp{%ld,%ld,%ld}+ %s,%s,%s",
		     Gmin[0],Gmin[1],Gmin[2],
		     orig[0].printS()(),
		     orig[1].printS()(),
		     orig[2].printS()()
		     );
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SeitzMatrixInfo::addlocdirshort(const IMatrix &axis)const
{
{
  if(axis[0]==0 && axis[1]==0) return "(z)";
  else if(axis[0]==0 && axis[2]==0) return "(y)";
  else if(axis[1]==0 && axis[2]==0) return "(x)";
  else if(axis==IMatrix::vector3(1,1,0)) return "(xy)";
  else if(axis==IMatrix::vector3(1,-1,0)) return "(x-y)";
  else if(axis==IMatrix::vector3(1,0,1)) return "(xz)";
  else if(axis==IMatrix::vector3(-1,0,1)) return "(-xz)";
  else if(axis==IMatrix::vector3(0,1,1)) return "(yz)";
  else if(axis==IMatrix::vector3(0,1,-1)) return "(y-z)";
  else if(axis==IMatrix::vector3(1,1,1)) return "(111)";
  else if(axis==IMatrix::vector3(1,-1,-1)) return "(1-1-1)";
  else if(axis==IMatrix::vector3(-1,1,-1)) return "(-11-1)";
  else if(axis==IMatrix::vector3(-1,-1,1)) return "(-1-11)";
  return Sprintf("(%ld,%ld,%ld)",axis[0],axis[1],axis[2]);
  }}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SeitzMatrixInfo::addlocdir(long l,char c0,const Fractional &o,bool comma)const
{
  switch(l){
  case 0: 
    return Sprintf("%s%s",(o==0?"0":o.printS()()),(comma?",":""));
  case 1: 
    return Sprintf("%c%s%s",c0,(o==0?"":o.printS(true)()),(comma?",":""));
  case -1: 
    return Sprintf("-%c%s%s",c0,(o==0?"":o.printS(true)()),(comma?",":""));
  default:
    return Sprintf("%ld%c%s%s",l,c0,(o==0?"":o.printS(true)()),(comma?",":""));
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

FMatrix SeitzMatrix::findHexPlanarBasisFromC3()
{
  if(!panalysed)
    throw Except("SeitzMatrix: findHexPlanarBasisFromC3: operation is "
		 "not analysed yet?");
  if(pinfo.pm1!=1)
    throw Except("findTetPlanarBasisFromC4: need positive power operation ?");
  
  // given C3 in a basis, which has the axis at 001 but can have
  // mixed planar vectors ->determine the volume conserving transformation,
  // which transforms it into a real hexagonal (120degere)setup

  FMatrix U=FMatrix::diagMatrix(3,3,1);
  
  //some hex lattice basis with axis in 001
  IMatrix Mh(pinfo.RA.fromColumn(0),
	     pinfo.RA.fromColumn(1),
	     pinfo.Pmin);
  long d;
  IMatrix Gh=Mh.inversetV(d);
  if(labs(d)!=pinfo.pp)
    throw Except("findHexPlanarBasisFromC3: volume of RTG lattice is not "
		 "1/3 of hex lattice?");

  //myDebug()<<"Mh=\n"<<Mh<<"det="<<d;
  //myDebug()<<"Gh=\n"<<Gh;

  //myDebug()<<"C3=\n"<<opC3.pW;

  FMatrix ophexmod=(Gh*pW*Mh)/pinfo.pp;
  //myDebug()<<"C3alpha=\n"<<ophexmod;
  if(!ophexmod.isInteger())
    throw Except("findHexPlanarBasisFromC3: ophexmod is not integer?");
  if(ophexmod.determinant()!=1)
    throw Except("findHexPlanarBasisFromC3: |ophexmod|!=1 ?");

  if(ophexmod(0,0)+ophexmod(1,1)!=-1){
    throw Except("findHexPlanarBasisFromC3: ophexmod11!=-ophexmod22-1 ?");
  }
  //                            / 0 -1 0 )  
  // ophexmod*U=U*C3hex   C3hex=| 1 -1 0 |
  //                            \ 0  0 1 ) 
  
  Fractional det;
  while(true){
    //myDebug()<<"while U=\n"<<U<<U.inverse(det);
    FMatrix op=U.inverse(det)*ophexmod*U;
    //myDebug()<<"op=\n"<<op;
    long s=op(0,0).toLong();
    long a=op(1,0).toLong();
    long b=op(0,1).toLong();
    //myDebug()<<"s="<<s;
    //myDebug()<<"a="<<a;
    //myDebug()<<"b="<<b;
    if(s==0) {
      break;
    }
    if(abs(a)<=abs(s)){
      //myDebug()<<"case v2\n";
      // s-a*k->min
      // s=r+a*l
      long r=s%a;
      long k=(s-r)/a;
      //myDebug()<<"r="<<r;
      //myDebug()<<"k="<<k;
      if(k==0){
	if(r<0){r+=a;k=-1;
	}else if(r>0){r-=a;k=1;}
      }
      //myDebug()<<"r="<<r;
      //myDebug()<<"k="<<k;
      FMatrix v2(FMatrix::vector3(1,0,0,1),
		 FMatrix::vector3(k,1,0,1),
		 FMatrix::vector3(0,0,1,1));
      //myDebug()<<"case v2\n";
      U=U*v2;
    }else if(abs(b)<=abs(s)){
      //myDebug()<<"case v1\n";
      //s+b*k->min
      // s=r+b*l
      long r=s%b;
      long l=(s-r)/b;
      //myDebug()<<"r="<<r;
      //myDebug()<<"l="<<l;
      if(l==0){
	if(r<0){r+=b;l=-1;
	}else if(r>0){r-=b;l=1;}
      }
      long k=-l;
      //myDebug()<<"r="<<r;
      //myDebug()<<"k="<<k;
      FMatrix v1(FMatrix::vector3(1,k,0,1),
		 FMatrix::vector3(0,1,0,1),
		 FMatrix::vector3(0,0,1,1));
      U=U*v1;
    }
  }
  FMatrix op=U.inverse(det)*ophexmod*U;
  if(op(1,0)<0){
    throw Except("findHexPlanarBasisFromC3: U(1,0)<0?");
//     U=U*FMatrix(FMatrix::vector3(-1,0,0,1),FMatrix::vector3(0,1,0,1),
// 		FMatrix::vector3(0,0,1,1));
  }
  op=U.inverse(det)*ophexmod*U;
  //myDebug()<<"op=\n"<<op;
  if((op!=FMatrix(FMatrix::vector3(0,1,0,1),FMatrix::vector3(-1,-1,0,1),
		  FMatrix::vector3(0,0,1,1))))
    throw Except("findHexPlanarBasisFromC3: failed to find a hexagonal basis?");
  //myDebug()<<"op=\n"<<op;
  //myDebug()<<"U=\n"<<U;
  //myDebug()<<"det(U)="<<det;

  U=Mh*U;

  det=U.determinant();

  //myDebug()<<"det(U)="<<det;
  if(det<0){
    U=-U;
  }
  if(det!=pinfo.pp)
    throw Except("findHexPlanarBasisFromC3: transformation does "
		 "not conserve volume?");

  if(!U.isInteger())
    throw Except("findHexPlanarBasisFromC3: transformation not integer?");

  //myDebug()<<"U=\n"<<U;
  //myDebug()<<"det(U)="<<det;
  //myDebug()<<"U^-1*op*U="<<U.inverse(det)*pW*U;
  
  
  return U;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

FMatrix SeitzMatrix::findTetPlanarBasisFromC4()
{
  bool prot=false;
  if(prot) myDebug()<<"---------findTetPlanarBasisFromC4 ---------------";
  if(!panalysed)
    throw Except("SeitzMatrix: findTetPlanarBasisFromC4: operation is "
		 "not analysed yet?");
  if(pinfo.pm1!=1 && pinfo.pnonsymmorphic)
    throw Except("findTetPlanarBasisFromC4: need positive power operation ?");

  // given C4 in a basis, which has the axis at 001 but can have
  // mixed planar vectors ->determine the volume conserving transformation,
  // which transforms it into a standard tetragonal (90degere) setup

  FMatrix U=FMatrix::diagMatrix(3,3,1);
  
  //some tet lattice basis with axis in 001
  IMatrix Mh(pinfo.RA.fromColumn(0),
 	     pinfo.RA.fromColumn(1),
 	     pinfo.Pmin);
  long d;
  IMatrix Gh=Mh.inversetV(d);

  if(labs(d)!=pinfo.pp)
    throw Except(Sprintf("findTetPlanarBasisFromC4: volume of "
			 "bravais lattice is not "
			 "1/%ld of simple lattice?",pinfo.pp));

  if(prot) myDebug()<<"Mh=\n"<<Mh<<"det="<<d;
  if(prot) myDebug()<<"Gh=\n"<<Gh;


  Fractional det;
  FMatrix optetmod=(Gh*pW*Mh)/pinfo.pp*lSign(1,pinfo.pnorder);
  if(prot) myDebug()<<"C4alpha=\n"<<optetmod;
  if(!optetmod.isInteger())
    throw Except("findTetPlanarBasisFromC4: optetmod is not integer?");
  det=optetmod.determinant();
  if(det!=1)
    throw Except(Sprintf("findTetPlanarBasisFromC4: det(optetmod)=%s !=1 ?",
			 det.printS()()));

  if(optetmod(0,0)+optetmod(1,1)!=0){
    throw Except("findTetPlanarBasisFromC4: optetmod11!=-optetmod22 ?");
  }
  /*                            / 0 -1 0 \  
   * optetmod*U=U*C4tet   C4tet=| 1  0 0 |
   *                            \ 0  0 1 / 
   */
  
  while(true){
    if(prot) myDebug()<<"while U=\n"<<U<<U.inverse(det);
    FMatrix op=U.inverse(det)*optetmod*U;
    if(prot) myDebug()<<"op=\n"<<op;
    long s=op(0,0).toLong();
    long a=op(1,0).toLong();
    long b=op(0,1).toLong();
    if(prot) myDebug()<<"s="<<s;
    if(prot) myDebug()<<"a="<<a;
    if(prot) myDebug()<<"b="<<b;
    if(s==0) {
      break;
    }
    if(abs(a)<=abs(s)){
      if(prot) myDebug()<<"case v2\n";
      // s-a*k->min
      // s=r+a*l
      long r=s%a;
      long k=(s-r)/a;
      if(prot) myDebug()<<"r="<<r;
      if(prot) myDebug()<<"k="<<k;
      if(k==0){
 	if(r<0){r+=a;k=-1;
 	}else if(r>0){r-=a;k=1;}
      }
      if(prot) myDebug()<<"r="<<r;
      if(prot) myDebug()<<"k="<<k;
      FMatrix v2(FMatrix::vector3(1,0,0,1),
 		 FMatrix::vector3(k,1,0,1),
 		 FMatrix::vector3(0,0,1,1));
      U=U*v2;
    }else if(abs(b)<=abs(s)){
      if(prot) myDebug()<<"case v1\n";
      //s+b*k->min
      // s=r+b*l
      long r=s%b;
      long l=(s-r)/b;
      if(prot) myDebug()<<"r="<<r;
      if(prot) myDebug()<<"l="<<l;
      if(l==0){
	if(r<0){r+=b;l=-1;
	}else if(r>0){r-=b;l=1;}
      }
      long k=-l;
      if(prot) myDebug()<<"r="<<r;
      if(prot) myDebug()<<"k="<<k;
      FMatrix v1(FMatrix::vector3(1,k,0,1),
		 FMatrix::vector3(0,1,0,1),
		 FMatrix::vector3(0,0,1,1));
      U=U*v1;
    }
  }
  FMatrix op=U.inverse(det)*optetmod*U;
  if(op(1,0)<0){
    throw Except("findTetPlanarBasisFromC4: U(1,0)<0?");
//     U=U*FMatrix(FMatrix::vector3(-1,0,0,1),
// 		FMatrix::vector3(0,1,0,1),
// 		FMatrix::vector3(0,0,1,1));
  }
  op=U.inverse(det)*optetmod*U;
  if(prot) myDebug()<<"op=\n"<<op;
  if((op!=FMatrix(FMatrix::vector3(0,1,0,1),
		  FMatrix::vector3(-1,0,0,1),
 		  FMatrix::vector3(0,0,1,1))))
    throw Except("findTetPlanarBasisFromC4: failed to find a hexagonal basis?");
  if(prot) myDebug()<<"op=\n"<<op;
  if(prot) myDebug()<<"U=\n"<<U;
  if(prot) myDebug()<<"det(U)="<<det;

  U=Mh*U;

  det=U.determinant();

  if(prot) myDebug()<<"det(U)="<<det;
  if(det<0){
    U=-U;
  }
  if(det!=pinfo.pp)
    throw Except("findTetPlanarBasisFromC4: transformation does "
 		 "not conserve volume?");

  if(!U.isInteger())
    throw Except("findTetPlanarBasisFromC4: transformation not integer?");

  if(prot) myDebug()<<"U=\n"<<U;
  if(prot) myDebug()<<"det(U)="<<det;
  if(prot) myDebug()<<"U^-1*op*U=\n"<<U.inverse(det)*pW*U*lSign(1,pinfo.pnorder);
  

  if(prot) myDebug()<<"---------END findTetPlanarBasisFromC4 ---------------";

  return U;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SeitzMatrix::changeBasis(ValueArray<SeitzMatrix> &ops,
			      const FMatrix &U,
			      bool analyse,
			      bool prot)
{
  FMatrix Uinv=U.inverse();

  //  if(Uinv*U!=FMatrix::diagMatrix(3,3,1)) throw Except("inverse failed");

  for(long i=0;i<ops.size();i++){
    if(!ops[i].pactive) continue;
    if(prot)myDebug()<<i<<"old basis"<<ops.at(i);
    ops[i].changeBasis(U,Uinv);
    if(prot)myDebug()<<i<<"new basis"<<ops.at(i);
    if(analyse){
      if(prot)printf("ANALYSE changeBasis %ld\n",i);
      ops[i].analyse(prot);
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SeitzMatrix::changeOrigin(ValueArray<SeitzMatrix> &ops,
			       const FMatrix neworigin, /* we use copy
							since sometimes we
							want to use reference
							to pinfo data
						     */
			       bool analyse,
			       bool prot)
{
  for(long i=0;i<ops.size();i++){
    if(!ops[i].pactive) continue;
    //if(prot)myDebug()<<i<<"old basis"<<ops.at(i)<<" neworigin="<<neworigin;
    ops[i].changeOrigin(neworigin);
    //if(prot)myDebug()<<i<<"new basis"<<ops.at(i)<<" neworigin="<<neworigin;
    if(analyse){
      if(prot)printf("ANALYSE changeOrigin %ld\n",i);
      ops[i].analyse(prot);
    }
    //if(prot)myDebug()<<i<<"after analyse neworigin="<<neworigin;

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SeitzMatrix  SeitzMatrix::changeSetting(ValueArray<SeitzMatrix> &ops,// in primitive basis
					Data::BravaisLattice &pbravais,
					Data::CrystalSystem &pcrystalsystem,
					String &spgrnum,
					FMatrix &Bcenter,
					FMatrix &Bcenterinv,
					const String &newsetting,bool analyse)
{
  bool prot=false;
  StringList sl=spgrnum.split(":");
  
  if(prot)myDebug()<<"spgrnum input="<<spgrnum;
  if(prot)myDebug()<<"newsetting="<<newsetting;
  spgrnum=sl[0];
  if(prot)myDebug()<<"spgrnum="<<spgrnum;
  String modi("");
  if(sl.size()>1)modi=sl[1];
  if(prot)myDebug()<<"modi="<<modi;
  
  String setting=newsetting;
  if(newsetting=="default"){
    setting=Data::defaultSetting(spgrnum);
  }
  if(prot)myDebug()<<"setting="<<setting;
  
  String s=spgrnum;
  if(setting.length()>0) spgrnum+=":"+setting;
  if(prot)myDebug()<<"spgrnum output="<<spgrnum;
  
  String hm=Data::HMFromIT(spgrnum);
  FMatrix Bcenternew=Bcenter;
  if(hm[0]=='C') {
    pbravais=Data::CBaseCentered;
    Bcenternew=Data::bravaisMatrix(pbravais);
  }else if(hm[0]=='A'){
    pbravais=Data::ABaseCentered;
    Bcenternew=Data::bravaisMatrix(pbravais); 
  }else if(hm[0]=='B'){
    pbravais=Data::BBaseCentered;
    Bcenternew=Data::bravaisMatrix(pbravais);
  }else if(hm[0]=='I'){
    pbravais=Data::BodyCentered;
    Bcenternew=Data::bravaisMatrix(pbravais);
  }else if(hm[0]=='R') {
    if(prot)myDebug()<<"HERERERERERE"<<hm;
    if(hm.find(":R")>=0){
      pbravais=Data::Simple;
      pcrystalsystem=Data::Rhombohedral;
      Bcenternew=FMatrix::diagMatrix(3,3,1);
    }else if(hm.find(":H")>=0){
      pbravais=Data::BRhombohedral;
      pcrystalsystem=Data::Trigonal;
      Bcenterinv=Data::bravaisMatrix(pbravais).inverse();
      Bcenternew=Data::bravaisMatrix(pbravais);
    }
  }
  
  if(prot)myDebug()<<"HM="<<hm;
  
  if(prot){
    myDebug()<<"Bcenter=\n"<<Bcenter;
    myDebug()<<"settings change op=\n"<<SeitzMatrix::changeSettingOp(s,modi,setting);
    myDebug()<<"Bcenternew=\n"<<Bcenternew;
  }
  SeitzMatrix op=SeitzMatrix(Bcenterinv,false)
    *SeitzMatrix::changeSettingOp(s,modi,setting)
    *SeitzMatrix(Bcenternew,false) ;
  
  if(prot)myDebug()<<"here op="<<op;
  
  for(long i=0;i<ops.size();i++){
    if(!ops[i].pactive) continue;
    //myDebug()<<ops[i].pinfo.symbol()<<ops[i].printWwS();
    ops[i].apply(op);
    //myDebug()<<ops[i].pinfo.symbol()<<ops[i].printWwS();
    
    
    if(analyse) ops[i].analyse(false);
    
    
  }
  
  
  //todo: change R<->H
  
  if(prot)myDebug()<<"Bcenter=\n"<<Bcenter;
  Bcenter=Bcenternew;
  Bcenterinv=Bcenternew.inverse();
  if(prot)myDebug()<<"Bcenternew=\n"<<Bcenter;
  
  
  return op;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SeitzMatrix SeitzMatrix::changeSettingOp(const String &spgrnum, 
					 const String &oldsetting, 
					 const String &newsetting)
{
  bool prot=false;
  if(prot){
    myDebug()<<"changeSettingOp: spgrnum="<<spgrnum;
    myDebug()<<"changeSettingOp: oldsetting="<<oldsetting;
    myDebug()<<"changeSettingOp: newsetting="<<newsetting;
  }
  const long setops[][9]={
    {0,1,0,0,0,1,1,0,0},
    {0,0,1,0,1,0,-1,0,0},
    {0,0,1,1,0,0,0,1,0},
    {1,0,0,0,0,1,0,-1,0},
    {0,1,0,1,0,0,0,0,-1},
    {1,0,0,0,0,-1,0,1,0},
    {0,0,-1,0,1,0,1,0,0},
    {0,0,1,0,1,0,-1,0,-1}, //7  mono b2->b1, b1->b3
    {-1,0,-1,0,1,0,1,0,0}, //8  mono b1->b2, b3->b1
    {0,0,1,1,0,0,0,1,0},   //9  mono b1->c1
    {0,1,0,0,0,1,1,0,0},   //10 mono c1->b1=(b1->c1)^2
    {0,0,1,0,-1,0,1,0,0},   //11 mono b->-b
  };
  
  String oldorig="";
  String oldrot=oldsetting;
  if(oldsetting[0]=='2' || oldsetting[0]=='1'){
    oldorig=oldsetting[0];
    oldrot=oldsetting.subString(1);
  }
  String neworig="";
  String newrot=newsetting;
  if(newsetting[0]=='2' || newsetting[0]=='1'){
    neworig=newsetting[0];
    newrot=newsetting.subString(1);
  }

  SeitzMatrix monooldrot=FMatrix::diagMatrix(3,3,1);
  SeitzMatrix mononewrot=FMatrix::diagMatrix(3,3,1);

  long l=oldsetting.length();
  const char *os=oldsetting.data();
  long ios=0;
  bool minus=false;
  char abc='\0';
  char mono123='\0';
  
  if(l==1){
    if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
  }else if(l==2){
    if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
    if(abc!='\0' && (os[ios]=='1' || os[ios]=='2' || os[ios]=='3')){ 
      mono123=os[ios];ios++; } 
  }else if(l==3){
    if(os[ios]=='-'){ ios++; minus=true;
      if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
      if(abc!='\0' && (os[ios]=='1' || os[ios]=='2' || os[ios]=='3')){ 
	mono123=os[ios];ios++; } 
    }
  }

  if(abc!='\0' || mono123!='\0'){
    if(abc=='c'){
      monooldrot*=SeitzMatrix(FMatrix(3,3,setops[10]));
    }else if(abc=='a'){
      monooldrot*=SeitzMatrix(FMatrix(3,3,setops[9]));
    }
    if(minus) monooldrot*=SeitzMatrix(FMatrix(3,3,setops[11]));
    if(mono123=='2'){
      monooldrot*=SeitzMatrix(FMatrix(3,3,setops[7]));
    }else if(mono123=='3'){
      monooldrot*=SeitzMatrix(FMatrix(3,3,setops[8]));
    }
    oldrot="mo";
  }

  l=newsetting.length();
  os=newsetting.data();
  ios=0;
  minus=false;
  abc='\0';
  mono123='\0';
  
  if(l==1){
    if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
  }else if(l==2){
    if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
    if(abc!='\0' && (os[ios]=='1' || os[ios]=='2' || os[ios]=='3')){ 
      mono123=os[ios];ios++; } 
  }else if(l==3){
    if(os[ios]=='-'){ ios++; minus=true;
      if(os[ios]=='a' || os[ios]=='b' || os[ios]=='c'){ abc=os[ios];ios++; }
      if(abc!='\0' && (os[ios]=='1' || os[ios]=='2' || os[ios]=='3')){ 
	mono123=os[ios];ios++; } 
    }
  }

  if(abc!='\0' || mono123!='\0'){
    if(mono123=='2'){
      mononewrot*=SeitzMatrix(FMatrix(3,3,setops[8]));
    }else if(mono123=='3'){
      mononewrot*=SeitzMatrix(FMatrix(3,3,setops[7]));
    }
    if(minus) mononewrot*=SeitzMatrix(FMatrix(3,3,setops[11]));
    if(abc=='c'){
      mononewrot*=SeitzMatrix(FMatrix(3,3,setops[9]));
    }else if(abc=='a'){
      mononewrot*=SeitzMatrix(FMatrix(3,3,setops[10]));
    }
    newrot="mo";
  }


  if(prot){
    myDebug()<<"old rot="<<oldrot;
    myDebug()<<"old orig="<<oldorig;
    myDebug()<<"new rot="<<newrot;
    myDebug()<<"new orig="<<neworig;
  }


  SeitzMatrix op(1);
  if(oldrot=="" || oldrot=="R"|| oldrot=="H" ) ;
  else if(oldrot=="mo") op=monooldrot;
  else if(oldrot=="cab") op=SeitzMatrix(FMatrix(3,3,setops[0]));
  else if(oldrot=="-cba") op=SeitzMatrix(FMatrix(3,3,setops[1]));
  else if(oldrot=="bca") op=SeitzMatrix(FMatrix(3,3,setops[2]));
  else if(oldrot=="a-cb") op=SeitzMatrix(FMatrix(3,3,setops[3]));
  else if(oldrot=="ba-c") op=SeitzMatrix(FMatrix(3,3,setops[4]));
  else throw Except("unknown 'oldrot'");

  if(prot)myDebug()<<"op from "<<oldrot<<"="<<op;

  if(oldorig!="") {
    const long *delta=Data::deltaToDefaultT8(spgrnum+":"+oldorig);
    op=op*SeitzMatrix(-FMatrix::vector3(delta)/8);
    if(prot)myDebug()<<"op from "<<oldorig<<"="<<op;
  }

  if(prot)myDebug()<<"op from "<<oldsetting<<"="<<op;

  if(neworig!=""){
    if(prot)myDebug()<<"change into"<<spgrnum+":"+newsetting;
    const long *delta=Data::deltaToDefaultT8(spgrnum+":"+neworig);
    if(prot)myDebug()<<"delta="<<FMatrix::vector3(delta);
    op=op*SeitzMatrix(FMatrix::vector3(delta)/8);
    if(prot)myDebug()<<"S ="<<SeitzMatrix(FMatrix::vector3(delta)/8);
    if(prot)myDebug()<<"op to "<<newsetting<<"="<<op;
  }

  if(newrot=="" || newrot=="R"  || newrot=="H") ;
  else if(newrot=="mo")  op=op*mononewrot;
  else if(newrot=="cab")  op=op*SeitzMatrix(FMatrix(3,3,setops[2]));
  else if(newrot=="-cba") op=op*SeitzMatrix(FMatrix(3,3,setops[6]));
  else if(newrot=="bca")  op=op*SeitzMatrix(FMatrix(3,3,setops[0]));
  else if(newrot=="a-cb") op=op*SeitzMatrix(FMatrix(3,3,setops[5]));
  else if(newrot=="ba-c") op=op*SeitzMatrix(FMatrix(3,3,setops[4]));
  else throw Except("unknown 'newsetting'");

  //myDebug()<<"M="<<SeitzMatrix(FMatrix::matrix3x3(setops[0])).inverse();
  //myDebug()<<"op="<<op;
  
  return op;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
SeitzMatrix Tests::mixIntSeitzRepresentation(ValueArray<SeitzMatrix> &ops,
					     bool mixlat,bool rotlat,
					     bool shiftlat,bool shuffle)
{
  // todo: remove this debuging stuff
  
  FMatrix UT=FMatrix::diagMatrix(3,3,1);
  
  if(mixlat){
    long nmix=((rand()%5)+5)*1;
    while(nmix-->0){
      long mode=(rand()%3);
      if(mode==0){
	// interchange
	long in=(rand()%3),jn=in,kn=0;
	while(jn==in)jn=(rand()%3);
	while(kn==in || kn==jn)kn++;
	FMatrix u=FMatrix::diagMatrix(3,3,0);
	u(jn,in)=1;
	u(in,jn)=1;
	u(kn,kn)=1;
	UT=UT*u;
      }else if(mode==1){
	//invert
	long in=(rand()%3);
	FMatrix u=FMatrix::diagMatrix(3,3,1);
	u(in,in)=-1;
	UT=UT*u;
      }else if(mode==2){
	// mix
	long dir=(rand()%2);
	long c=(rand()%3);
	FMatrix u=FMatrix::diagMatrix(3,3,1);
	for(int i=0;i<3;i++){
	  if(i==c)continue;
	  if(dir==0){
	    u(i,c)=rand()%5-5;
	  }else{
	    u(c,i)=rand()%5-5;
	  }
	}
	UT=UT*u;
      }
    }
  }

  if(rotlat){
    
  }

  if(shuffle){
    if(ops.size()>1){
      for(long i=0;i<30;i++){
	long j=rand()%(ops.size()-1)+1;
	ops.swap(0,j);
      }
    }
  }

  Fractional det;
  FMatrix UTinv=UT.inverse(det);
  if(det<0){
    UT=-UT;
    UTinv=-UTinv;
    det=-det;
  }
  //myDebug()<<"UTmix=\n"<<UT<<det;
  //myDebug()<<"UTmixinv=\n"<<UTinv<<det;
  if(det!=1) 
    throw Except("SeitzMatrices: debuging: det(UTmix)!=1");
  if(!UT.isInteger()) 
    throw Except("SeitzMatrices: debuging: UTmix not integer");
  if(!UTinv.isInteger()) 
    throw Except("SeitzMatrices: debuging: UTmix^-1 not integer");
  
  
  //myDebug()<<"changebasis with UTminx";
  SeitzMatrix::changeBasis(ops,UT,!shiftlat,false);
  //myDebug()<<"changebasis with UTminx done";
  
  FMatrix vshift;
  if(shiftlat){
    // make random shift
    vshift=FMatrix::vector3(Fractional(rand()%26-13,rand()%12+1),
			   Fractional(rand()%26-13,rand()%12+1),
			   Fractional(rand()%26-13,rand()%12+1));
    //shift=FVec3(1,1,1,10);
    //myDebug()<<"shift="<<shift;
    SeitzMatrix::changeOrigin(ops,vshift,true);
  }  

  return SeitzMatrix(UT)*SeitzMatrix(vshift);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class SpaceGroup::MainAxisOperation{
public:

  MainAxisOperation(ValueArray<SeitzMatrix> &ops,
		    const IMatArray &primaxes,
		    const IMatArray &secaxes,
		    const IMatArray &tertaxes)
    :pops(&ops){
    detMainAxisOperations(primaxes,secaxes,tertaxes);
  }


  bool hasOnlyPrimary()const{return !hasSecondary() && !hasTertiary(); }

  bool hasSecondary()const{
    return pindC2second.size()>0 || pindmsecond.size()>0; }
  bool hasSecondaryRotation()const{return pindC2second.size()>0;}
  bool hasSecondaryMirror()const{return pindmsecond.size()>0; }
  bool hasTertiary()const{return pindC2tert.size()>0 || pindmtert.size()>0; }
  bool hasTertiaryRotation()const{return pindC2tert.size()>0; }
  bool hasTertiaryMirror()const{return pindmtert.size()>0; }
  
  const LongArray &primaryOperations(long order)const{
    if(order==2)
      return pindC2prim;
    else if(order==-2)
      return pindmprim;
    else if(order==4)
      return pindC4prim;
    else if(order==-4)
      return pindB4prim;
    else return pempty;
  }
  const LongArray &secondaryOperations(long order)const{
    if(order==2)
      return pindC2second;
    else if(order==-2)
      return pindmsecond;
    else if(order==3)
      return pindC3second;
    else if(order==-3)
      return pindB3second;
    else return pempty;
  }  
  const LongArray &tertiaryOperations(long order)const{
    if(order==2)
      return pindC2tert;
    if(order==-2)
      return pindmtert;
    else return pempty;
  }

  long primaryOperation(long order)const{
    if(order==2 && pindC2prim.size()>0)
      return pindC2prim.at(0);
    else if(order==-2  && pindmprim.size()>0)
      return pindmprim.at(0);
    else if(order==4 && pindC4prim.size()>0)
      return pindC4prim.at(0);
    else if(order==-4 && pindB4prim.size()>0)
      return pindB4prim.at(0);
    else return -1;
  }
  long secondaryOperation(long order)const{
    if(order==2 && pindC2second.size()>0)
      return pindC2second.at(0);
    else if(order==-2 && pindmsecond.size()>0)
      return pindmsecond.at(0);
    else if(order==3 && pindC3second.size()>0)
      return pindC3second.at(0);
    else if(order==-3 && pindB3second.size()>0)
      return pindB3second.at(0);
    else return -1;
  }
  long tertiaryOperation(long order)const{
    if(order==2 && pindC2tert.size()>0)
      return pindC2tert.at(0);
    if(order==-2 && pindmtert.size()>0)
      return pindmtert.at(0);
    else return -1;
  }

  
  PeriodicHyperPlane commonOrigin(const FMatrix &Bcenter,
				  const LongArray &inds,
				  long taurestrict,bool betweenpairs=false)
  {
						
    bool prot=false;
    if(prot){
      myDebug()<<"------------- det common origin ------------";
      myDebug()<<"inds="<<inds;
      myDebug()<<"taurestrict="<<taurestrict;
      myDebug()<<"betweenpairs="<<betweenpairs;
    }
    PeriodicHyperPlane pcs; 

    FMatrix delt(3,1);

    if(inds.size()>0) {
      
      long ip=0;
      if(prot) myDebug()<<"operation: "<<ip<<" "
			<<(*pops)[inds.at(ip)]<<(*pops)[inds.at(ip)].pinfo.symbol();
      pcs=(*pops)[inds.at(ip)].pinfo.toPHPlane(taurestrict,delt,betweenpairs);
      if(prot)myDebug()<<"pcs="<<pcs.printS();
      if(prot)myDebug()<<"conventional pcs="<<pcs.printS(Bcenter);
      
      for(ip=1;ip<inds.size();ip++){
	SeitzMatrixInfo &pinfo=(*pops)[inds.at(ip)].pinfo;
	if(prot)myDebug()<<"operation: "<<ip<<" "
			 <<(*pops)[inds.at(ip)]<<pinfo.symbol();
	
	PeriodicHyperPlane pp=pinfo.toPHPlane(taurestrict,delt,betweenpairs);
	if(prot){
	  myDebug()<<"pp="<<pp.printS();
	  myDebug()<<"conventional pp="<<pp.printS(Bcenter);
	}
	if(pcs.commonPoints(pp).isEmpty()){
	  break;
	}
	if(prot){
	  myDebug()<<"pcs="<<pcs.printS();
	  myDebug()<<"conventional pcs="<<pcs.printS(Bcenter);
	}
      }
    }

    if(prot){
      myDebug()<<"final pcs="<<pcs;
      myDebug()<<"final conventional pcs="<<pcs.printS(Bcenter);
      myDebug()<<"------------- END det common origin ------------";
    }
    return pcs;
  }



private:


  void detMainAxisOperations(const IMatArray &primaxes,
			     const IMatArray &secaxes,
			     const IMatArray &tertaxes)
  {
    bool prot=false;
    if(prot){
      myDebug()<<"detMainAxisOperations --------------------------";
      myDebug()<<"primaxes=\n"<<primaxes;
      myDebug()<<"secaxes ="<<secaxes;
      myDebug()<<"tertaxes="<<tertaxes;
    }
    pindC2second.clear();
    pindC3second.clear();
    pindB3second.clear();
    pindC2tert.clear();
    pindmsecond.clear();
    pindmtert.clear();
    pindC4prim.clear();
    pindC2prim.clear();
    pindB4prim.clear();
    pindmprim.clear();
    for(long i=0;i<pops->size();i++){
      if(!(*pops)[i].pactive)continue;
      const SeitzMatrixInfo &pinfo=(*pops)[i].pinfo;
      const IMatrix &Pmin=pinfo.Pmin;
      long order=pinfo.pnorder;

      if(primaxes.contains(Pmin)||primaxes.contains(-Pmin) ){
	bool isfirst=(primaxes.at(0)==Pmin||primaxes.at(0)==-Pmin);
	if(order==4){
	  if(isfirst) pindC4prim.prepend(i);
	  else pindC4prim<<i;
	}else if(order==2){
	  if(isfirst) pindC2prim.prepend(i);
	  else pindC2prim<<i;
	}else if(order==-4){
	  if(isfirst) pindB4prim.prepend(i);
	  else pindB4prim<<i;
	}else if(order==-2) {
	  if(isfirst) pindmprim.prepend(i);
	  else pindmprim<<i;
	}
      }else if(secaxes.contains(Pmin)||
	       secaxes.contains(-Pmin)){
	bool isfirst=(secaxes.at(0)==Pmin||secaxes.at(0)==-Pmin);
	if(order==2) {
	  if(isfirst) pindC2second.prepend(i);
	  else pindC2second<<i;
	}else if(order==-2) {
	  if(isfirst) pindmsecond.prepend(i);
	  else pindmsecond<<i;
	}else if(order==3) {
	  if(isfirst) pindC3second.prepend(i);
	  else pindC3second<<i;
	}else if(order==-3) {
	  if(isfirst) pindB3second.prepend(i);
	  else pindB3second<<i;
	}
      }else if(tertaxes.contains(Pmin)||tertaxes.contains(-Pmin)){
	bool isfirst=(tertaxes.at(0)==Pmin||tertaxes.at(0)==-Pmin);
	if(order==2) {
	  if(isfirst) pindC2tert.prepend(i);
	  else pindC2tert<<i;
	}else if(order==-2) {
	  if(isfirst) pindmtert.prepend(i);
	  else pindmtert<<i;
	}
      }
      
    }    
    if(prot)myDebug()<<"end detMainAxisOperations ----------------------";
  }

private:
  ValueArray<SeitzMatrix> *pops;
  LongArray pindC2second; //todo: make IMatrix
  LongArray pindC3second;
  LongArray pindB3second;
  LongArray pindC2tert;
  LongArray pindmsecond;
  LongArray pindmtert;
  LongArray pindC4prim;
  LongArray pindC2prim;
  LongArray pindB4prim;
  LongArray pindmprim;
  LongArray pempty;
};




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SeitzMatrix::analyseGroup(HallSymbol &hallsymbolout,SeitzMatrix &Stransform,
			       String &spgrnum,
			       const ValueArray<SeitzMatrix> &opsin,
			       const String &requiredsetting,
			       bool testmix)
{
  // We define the coordinate system by writing cartesian unit vectors
  // ex=\vec{e}_x ... where a vector is a column vector.
  // Lets organize them into a matrix
  //   | ex^T |
  // E=| ey^T |
  //   | ez^T |
  //
  //A vector \vec{r} is represented as
  // \vec{r}=E^T.r, with r being a column vector.
  //
  // Opertions are expressed in this cartesian system via 
  // 
  // \hat{D} E^T=E^T alpha, with alpha being the representation matrix
  // 
  // Vectors transform as \hat{D} \vec{r}= E^T.alpha.r: r'=alpha.r
  // 
  // 
  //
  // The lattice basis vectors are \vec{a}_i=E^T.a_i, with a coordinate column vector a_i 
  // A matrix can be defined via 
  //
  //   | a_1^T |
  // A=| a_2^T |, which gives the column vector of cartesian vectors A.E
  //   | a_3^T |
  //
  // Lattice vectors are \vec{R}=E^T.A^T.N,   N being integer
  //
  // Operations act as
  //
  //
  // \hat{D} \vec{R} =E^T .alpha . A^T.N:      R'=alpha . A^T.N
  //
  // In the lattice basis we can define reciprocal vectors vi A^T.G=1: G=A^{-T}
  //
  // R'=A^T.(G.alpha . A^T).N, which gives the lattice representation of alpha
  // alpha_A=G.alpha . A^T, and transformed lattice vectors are obtained via
  // N'=alpha_A.N
  //
  //
  //
  //
  //
  // We start out with a given lattice basis A, which is not explicitely given.
  // We know the lattice representation of all operations alpha_A and similarily
  // the translations \vec{tau}=E^T.A^T.tau are given relative to the lattice.
  //
  // We do transformations of the kind {W,w} (Seitzsymbol) to orient the lattice
  // and to define a standard origin.
  //


  bool protinit=false;
  bool prot=false;

  ValueArray<SeitzMatrix> ops(opsin);
  
  

  FMatrix C6(FMatrix::vector3( 1,1,0,1), //C6 in hex lattice basis
	     FMatrix::vector3(-1,0,0,1),
	     FMatrix::vector3( 0,0,1,1));
  FMatrix C4z(FMatrix::vector3( 0,1,0,1),
	      FMatrix::vector3(-1,0,0,1),
	      FMatrix::vector3( 0,0,1,1));
  FMatrix C3111(FMatrix::vector3( 0,1,0,1), // C3 around cubic-111 or RTG-z
		FMatrix::vector3(0,0,1,1),
		FMatrix::vector3( 1,0,0,1));
  FMatrix C3m111=C3111.inverse();
  FMatrix Bcenter=FMatrix::diagMatrix(3,3,1);
  FMatrix Bcenterinv=Bcenter;
  

  //find all information about the group from the SeitzMatrices
  Data::CrystalSystem pcrystalsystem=Data::None;
  Data::BravaisLattice pbravais=Data::Simple;
  
  spgrnum="0";

  
  if(protinit){
    for(long i=0;i<ops.size();i++){
      //myDebug()<<i<<"old basis"<<ops.at(i);
      printf("INITIAL ANALYSE %ld\n",i);
      ops[i].analyse(true);
    }
  }

  SeitzMatrix STmix;
  if(testmix) STmix=Tests::mixIntSeitzRepresentation(ops,true,true,true,
						     true);
  
  Stransform.pinteger=false;
  Stransform*=STmix;

  //contract the group to nessecary information
  //myDebug()<<"CONTRACT";

  // determine the smallest translational basis: primitive lattice
  FMatArray tr;
  //extract the pure translation from the operation set
  for(long i=ops.size()-1;i>=0;i--){
    if(!ops[i].pactive)continue;
    if(ops[i].pW==FMatrix::diagMatrix(3,3,1) && (ops[i].pw!=0)){
      tr<<ops[i].pw;
      //ops.removeAt(i);
      ops[i].pactive=false;
    }
  }  
  // add the conventional lattice vector basis
  tr<<FMatrix::vector3(1,0,0,1)<<FMatrix::vector3(0,1,0,1)
    <<FMatrix::vector3(0,0,1,1);
  // find smallest lattice spanned by all these translations
  IntegerTricks::reduceToSmallestLatticeBasis(tr);
  if(tr.size()!=3)
    throw Except("SeitzMatrices: something went wrong with the "
		 "primitive lattice reduction.");
  // this is actually B.Td(), since columns are vectors
  FMatrix BT(tr.at(0),tr.at(1),tr.at(2));
  Fractional detBT;
  FMatrix BTinv=BT.inverse(detBT);
  IMatrix IBTinv;
  //  throw Except(Sprintf("detBT=%s\n",BT.determinant().printS()()));
  if(detBT==0)
    throw Except("SeitzMatrices: primitive basis is flat?");
  if(detBT<0)
    throw Except("SeitzMatrices: primitive basis is left-handed?");
  try{
    IBTinv=BTinv.toIMatrix();
  }catch(Except &ex){
    throw Except("SeitzMatrices: B^(-T) is not integer?");
  }
  //myDebug()<<"BT="<<BT;
  //myDebug()<<"BTinv="<<BTinv<<BTinv*BT;

  // remove translation from the operation set
  for(long i=0;i<ops.size();i++){
    if(!ops[i].pactive) continue;
    //myDebug()<<"find similar to"<<i<<ops.at(i);
    for(long j=ops.size()-1;j>i;j--){
      if(!ops[j].pactive) continue;
      //myDebug()<<"try"<<j<<ops.at(j)<<" and "<<i<<ops.at(i);
      if(ops.at(j).pW!=ops.at(i).pW)continue;
      //myDebug()<<"same W";
      if((BTinv*(ops.at(j).pw-ops.at(i).pw)).isInteger()){
	//myDebug()<<"remove"<<j<<ops.at(j);
	//ops.removeAt(j);
	ops[j].pactive=false;
      }
    }
  }
  
  // re-express operations in primitive basis, backfold new tau
  // All lattice vectors A^T* tr *N can be expressed as A^T* BT *M.
  // Hence A'=B*A is the new lattice vectors.
  // Operations in the new basis are op'=B^{-T} * op *B^T 
  //                                tau'=B^{-T} * tau
  changeBasis(ops,BT,true);

  Stransform*=SeitzMatrix(BT,false);

  if(prot){
    for(long i=0;i<ops.size();i++){
      if(!ops[i].pactive)continue;
      //myDebug()<<i<<"old basis"<<ops.at(i);
      printf("BTTESTANALYSE %ld\n",i);
      ops[i].analyse(true);
    }
  }  
  // now we have the most general case of all possible ways to obtain
  // the group information .... let's find out the group and settings
  //todo: the removal for fcc is too slow, do indexing instead
  

  // remove the negative powers and find the common operations
  for(long i=ops.size()-1;i>=0;i--){
    if(!ops[i].pactive)continue;
    if(ops.at(i).pinfo.pm1<0){
      //ops.removeAt(i);
      ops[i].pactive=false;
      continue;
    }
    if(ops.at(i).pinfo.pnorder==1){
      ops[i].pactive=false;
      //ops.removeAt(i);
      continue;
    }
  }

  if(prot){
    for(long i=0;i<ops.size();i++){
      if(!ops[i].pactive)continue;
      //myDebug()<<i<<"old basis"<<ops.at(i);
      printf("SMALL SET ANALYSE %ld\n",i);
      ops[i].analyse(true);
      if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	throw Except("Negative operation power should not be here.");
      
    }
  }



  FMatrix UTbasis=FMatrix::diagMatrix(3,3,1);
  FMatrix commonorigin(3,1);

  long indI=-1;
  long indC3=-1;
  long indB3=-1;
  long indC6=-1;
  long indB6=-1;
  long indC4=-1;
  long indB4=-1;
  bool hasC4=false;
  bool hasB4=false;
  LongArray cubC3;
  LongArray orthoC2;
  LongArray orthom;
  for(long i=0;i<ops.size();i++){
    if(!ops[i].pactive)continue;

    //myDebug()<<"ops[i].pinfo.pnorder="<<ops[i].pinfo.pnorder;
    if(ops[i].pinfo.pnorder==-1){
      indI=i;
    }else if(ops[i].pinfo.pnorder==2){
      orthoC2<<i;
    }else if(ops[i].pinfo.pnorder==-2){
      orthom<<i;
    }else if(ops[i].pinfo.pnorder==3){
      indC3=i;
      cubC3<<i;
    }else if(ops[i].pinfo.pnorder==-3){
      indB3=i;
    }else if(ops[i].pinfo.pnorder==6){
      indC6=i;
    }else if(ops[i].pinfo.pnorder==-6){
      indB6=i;
    }else if(ops[i].pinfo.pnorder==4){
      hasC4=true;
      indC4=i;
    }else if(ops[i].pinfo.pnorder==-4){
      hasB4=true;
      indB4=i;
    }
  }

  //myDebug()<<"indI="<<indI;

  // myDebug()<<"cub3.size()="<<cubC3.size();
  // myDebug()<<"hasC4="<<hasC4;
  // myDebug()<<"hasB4="<<hasB4;


  String pointgroup="Unkown";
  if(cubC3.size()==4){
    //more than one C3 axis with different angle -> cubic
    pcrystalsystem=Data::Cubic;
    

    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("CUB START ANALYSE %ld\n",i);
	ops[i].analyse(true);
	if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	  throw Except("Negative operation power should not be here.");
	
      }
    }


    //myDebug()<<"cubC3="<<cubC3;
    //myDebug()<<"ppC3="<<ops[cubC3.at(0)].pinfo.pp;
    
    UTbasis.setDimenions(3,0);
    for(long i=0;i<4;i++){
      ops[cubC3.at(i)].analyse(false);
      for(long j=0;j<4;j++){
	SeitzMatrix op=ops[cubC3.at(i)].pW*ops[cubC3.at(j)].pW;
	op.analyse(false);
	if(op.pinfo.pnorder!=2){
	  //myDebug()<<"need inverse";
	  op=ops[cubC3.at(i)].pW*ops[cubC3.at(j)].pW.inverse();
	  op.analyse(false);
	}
	if(op.pinfo.pnorder==2){
	  //myDebug()<<op.pinfo.Pmin;
	  if(!UTbasis.containsColumn(op.pinfo.Pmin) &&
	     !UTbasis.containsColumn(-op.pinfo.Pmin)
	     ){
	    UTbasis.addMatrix(op.pinfo.Pmin);
	  }
	}
      }
    }
    //myDebug()<<"UTbasis=\n"<<UTbasis;
    //myDebug()<<"det=\n"<<UTbasis.determinant();

    Fractional detUT=UTbasis.determinant();
    if(detUT<0){
      UTbasis.swapColumns(0,1);
      detUT*=-1;
      //myDebug()<<"UTbasis=\n"<<UTbasis;
      //myDebug()<<"det=\n"<<UTbasis.determinant();
    }
    if(abs(detUT)==4){
      pbravais=Data::FaceCentered;
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Data::bravaisMatrix(Data::BodyCentered)*2;
    }else if(abs(detUT)==2){
      pbravais=Data::BodyCentered;
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Data::bravaisMatrix(Data::FaceCentered)*2;
    }
    
    UTbasis=UTbasis*Bcenter;
    
    if(!UTbasis.isInteger()){
      myDebug()<<"UTbasis=\n"<<UTbasis;
      throw Except("UTbasis not integer?");
    }
   
    //myDebug()<<"UTbasis=\n"<<UTbasis;
    //myDebug()<<"BT*UTbasis=\n"<<BT*UTbasis;
    //myDebug()<<"det=\n"<<UTbasis.determinant();


    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("test before changeBasis%ld: pm=%ld\n",i,ops.at(i).pinfo.pm1);
	if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	  throw Except("Negative operation power should not be here.");
      }
    }

    changeBasis(ops,UTbasis,true);
    
    Stransform*=UTbasis;

    
    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("DET LATTICE ANALYSE %ld\n",i);
	ops[i].analyse(true);
      }
    }

    IMatArray primaxes;
    primaxes<<Bcenterinv.toIMatrix()*IMatrix::vector3(0,0,1);
    primaxes<<Bcenterinv.toIMatrix()*IMatrix::vector3(1,0,0);
    primaxes<<Bcenterinv.toIMatrix()*IMatrix::vector3(0,1,0);

    

    IMatArray secoaxes;
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3( 1, 1, 1)).shorten();
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(-1, 1, 1)).shorten();
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3( 1,-1, 1)).shorten();
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3( 1, 1,-1)).shorten();
    
    IMatArray tertaxes;
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1, 1, 0)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,-1, 0)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1, 0, 1)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1, 0,-1)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0, 1, 1)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0, 1,-1)).shorten();
    
  loopcubchangebasis:

    MainAxisOperation maaxops(ops,primaxes,secoaxes,tertaxes);



    FMatrix delta111=Bcenterinv*FMatrix::vector3(-1,-1,-1,8)
      *(pbravais==Data::Simple?2:1);



    const LongArray &indsb4prim
      =LongArray(maaxops.primaryOperations(-4));
    const LongArray &indsc4prim
      =LongArray(maaxops.primaryOperations(4));
    const LongArray &indsc2prim
      =LongArray(maaxops.primaryOperations(2));
    const LongArray &indsmprim
      =LongArray(maaxops.primaryOperations(-2));
    const LongArray &indsc3seco
      =LongArray(maaxops.secondaryOperations(3));
    const LongArray &indsb3seco
      =LongArray(maaxops.secondaryOperations(-3));
    const LongArray &indsmtert
      =LongArray(maaxops.tertiaryOperations(-2));
    const LongArray &indsc2tert
      =LongArray(maaxops.tertiaryOperations(2));



    
    
    
    PeriodicHyperPlane ppb4p;
    ppb4p=maaxops.commonOrigin(Bcenter,indsb4prim,-1);
    //myDebug()<<"PHP all -4.."<<ppb4p.printS(Bcenter);
    PeriodicHyperPlane ppc4p;
    ppc4p=maaxops.commonOrigin(Bcenter,indsc4prim,0);
    //myDebug()<<"PHP all 4.."<<ppc4p.printS(Bcenter);
    PeriodicHyperPlane ppc2p;
    ppc2p=maaxops.commonOrigin(Bcenter,indsc2prim,0);
    //myDebug()<<"PHP all 2.."<<ppc2p.printS(Bcenter);
    PeriodicHyperPlane ppmp;
    ppmp=maaxops.commonOrigin(Bcenter,indsmprim,0);
    //myDebug()<<"PHP all m.."<<ppmp.printS(Bcenter);
    PeriodicHyperPlane ppb3s;
    ppb3s=maaxops.commonOrigin(Bcenter,indsb3seco,-1);
    //myDebug()<<"PHP all .-3."<<ppb3s.printS(Bcenter);
    PeriodicHyperPlane ppc3s;
    ppc3s=maaxops.commonOrigin(Bcenter,indsc3seco,0);
    //myDebug()<<"PHP all .3."<<ppc3s.printS(Bcenter);
    PeriodicHyperPlane ppmt;
    ppmt=maaxops.commonOrigin(Bcenter,indsmtert,0);
    //myDebug()<<"PHP all ..m"<<ppmt.printS(Bcenter);
    PeriodicHyperPlane ppc2t;
    ppc2t=maaxops.commonOrigin(Bcenter,indsc2tert,0);
    //myDebug()<<"PHP all ..2"<<ppc2t.printS(Bcenter);



    PeriodicHyperPlane ppmb3(ppmp);
    ppmb3.commonPoints(ppb3s);
    //myDebug()<<"PHP all m-3."<<ppmb3.printS(Bcenter);

    PeriodicHyperPlane pp23(ppc2p);
    pp23.commonPoints(ppc3s);
    //myDebug()<<"PHP all 23."<<pp23.printS(Bcenter);

    PeriodicHyperPlane ppmb3m(ppmb3);
    ppmb3m.commonPoints(ppmt);
    //myDebug()<<"PHP all m-3m"<<ppmb3m.printS(Bcenter);

    PeriodicHyperPlane ppb43m(ppb4p);
    ppb43m.commonPoints(ppc3s);
    ppb43m.commonPoints(ppmt);
    //myDebug()<<"PHP all -43m"<<ppb43m.printS(Bcenter);

    PeriodicHyperPlane ppc2z_0x0;
    if(indsc2prim.size()>0){
      ppc2z_0x0=ops[indsc2prim.at(0)].pinfo
	.toPHPlane(1,Bcenterinv*FMatrix::vector3(-1,0,0,4));
      //myDebug()<<"PHP 21z -(100)/4 "<<ppc2z_0x0.printS(Bcenter);
    }

    PeriodicHyperPlane ppb3_230;
    PeriodicHyperPlane ppb3111center;
    PeriodicHyperPlane ppb3333center;
    if(indsb3seco.size()>0){
      ppb3111center=ops[indsb3seco.at(0)].pinfo.toPHPlane(-1,delta111);
      ppb3333center=ops[indsb3seco.at(0)].pinfo.toPHPlane(-1,delta111*3);
      ppb3_230=ops[indsb3seco.at(0)].pinfo.toPHPlane(-1);
      ppb3_230.commonPoints(ppc2z_0x0);
      //myDebug()<<"PHP .-3. for 230"<<ppb3_230.printS(Bcenter);
    }

    PeriodicHyperPlane ppb43mfromcenter(ppb43m);
    ppb43mfromcenter.commonPoints(ppb3111center);
    //myDebug()<<"PHP all -43m -(111)/(4/8) from center"
    //<<ppb43mfromcenter.printS(Bcenter);

    PeriodicHyperPlane pp432(ppc4p);
    pp432.commonPoints(ppc3s);
    pp432.commonPoints(ppc2t);
    //myDebug()<<"PHP all 432"<<pp432.printS(Bcenter);

    PeriodicHyperPlane pp432fromcenter(pp432);
    pp432fromcenter.commonPoints(ppb3111center);
    //myDebug()<<"PHP all 432 -(111)/(4/8) from center"
    //<<pp432fromcenter.printS(Bcenter);

    PeriodicHyperPlane pp23fromcenter333(pp23);
    pp23fromcenter333.commonPoints(ppb3333center);
    //myDebug()<<"PHP all 23 -(333)/(4/8) from center"
    //     <<pp23fromcenter333.printS(Bcenter);
    


    PeriodicHyperPlane ppc3xxx;
    if(indsc3seco.size()>0){
      ppc3xxx=ops[indsc3seco.at(0)].pinfo.toPHPlane(0);
    }
    //myDebug()<<"PHP .3(111)."<<ppc3xxx.printS(Bcenter);

    PeriodicHyperPlane ppc21ppairs;
    ppc21ppairs=maaxops.commonOrigin(Bcenter,indsc2prim,1,true);
    //myDebug()<<"PHP all 21 pairs"<<ppc21ppairs.printS(Bcenter);

    PeriodicHyperPlane pp3_21pairs(ppc3xxx);
    pp3_21pairs.commonPoints(ppc21ppairs);
    pp3_21pairs.commonPoints(ppc2z_0x0);
    //myDebug()<<"PHP 3(111) and pairs 21"<<pp3_21pairs.printS(Bcenter);

    
    
    PeriodicHyperPlane pp;

    if(indI>=0){
      if(hasC4){
	pointgroup="m-3m";
	if(!ppmb3m.isEmpty()){
	  commonorigin=ppmb3m.O.fromColumn(0);
	  if(pbravais==Data::FaceCentered) spgrnum="225";
	  else if(pbravais==Data::BodyCentered) spgrnum="229";
	  else spgrnum="221";
	}else if(!ppmb3.isEmpty()){
	  commonorigin=ppmb3.O.fromColumn(0);
	  if(pbravais==Data::FaceCentered) spgrnum="226";
	  else spgrnum="223";
	}else if(!ppb43mfromcenter.isEmpty()){
	  commonorigin=ppb43mfromcenter.O.fromColumn(0);
	  if(pbravais==Data::FaceCentered) spgrnum="227:1";
	  else spgrnum="224:1";
	}else if(!pp432fromcenter.isEmpty()){
	  commonorigin=pp432fromcenter.O.fromColumn(0);
	  spgrnum="222:1";
	}else if(!pp23fromcenter333.isEmpty()){
	  commonorigin=pp23fromcenter333.O.fromColumn(0);
	  spgrnum="228:1";
	}else if(!ppb3_230.isEmpty()){
	  commonorigin=ppb3_230.O.fromColumn(0);
	  spgrnum="230";
	}
      }else{
	pointgroup="m-3";
	if(!ppmb3.isEmpty()){
	  commonorigin=ppmb3.O.fromColumn(0);
	  if(pbravais==Data::BodyCentered) spgrnum="204";
	  else if(pbravais==Data::FaceCentered) spgrnum="202";
	  else spgrnum="200";
	}else if(!(pp=PeriodicHyperPlane(pp23).commonPoints(ppb3111center)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	    if(pbravais==Data::FaceCentered) spgrnum="203:1";
	    else spgrnum="201:1";
	}else{
	  if(pbravais==Data::BodyCentered){
	    if(!ppb3_230.isEmpty()) spgrnum="206";
	  }else{
	    if(ppb3_230.isEmpty()){
	      FMatrix op=Bcenterinv*C4z*Bcenter;
	      UTbasis=UTbasis*op;
	      changeBasis(ops,op,true);
	      Stransform*=op;
	      goto loopcubchangebasis;
	    }else spgrnum="205";
	  }
	  commonorigin=ppb3_230.O.fromColumn(0);
	}
      }
    }else{
      if(hasC4){
	pointgroup="432";
	const SeitzMatrixInfo &pinfoC4p=ops[maaxops.primaryOperation(4)].pinfo;
	
	PeriodicHyperPlane pp43_010=
	  ops[indsc4prim.at(0)].pinfo.toPHPlane(1,Bcenterinv*FMatrix::vector3(0,-1,0,4));

	if(!pp432.isEmpty()){
	  commonorigin=pp432.O.fromColumn(0);
	  if(pbravais==Data::FaceCentered) spgrnum="209";
	  else if(pbravais==Data::BodyCentered) spgrnum="211";
	  else spgrnum="207";
	}else if(!pp23.isEmpty() && (pbravais==Data::Simple)){
	  commonorigin=pp23.O.fromColumn(0);
	  spgrnum="208";
	}else if(!(pp=PeriodicHyperPlane(pp23).commonPoints(pp43_010)).isEmpty() && 
		 (pbravais==Data::FaceCentered)){
	  commonorigin=pp.O.fromColumn(0);
	  spgrnum="210";
	}else if(!pp3_21pairs.isEmpty()){
	  commonorigin=pp3_21pairs.O.fromColumn(0);
	  if(pbravais==Data::BodyCentered) spgrnum="214";
	  else{
	    if(pinfoC4p.skrew()==1) spgrnum="213";
	    else if(pinfoC4p.skrew()==3) spgrnum="212";
	  }
	}
	
      }else{
	if(hasB4){
	  pointgroup="-43m";
	  if(!ppb43m.isEmpty()){
	    commonorigin=ppb43m.O.fromColumn(0);
	    if(pbravais==Data::FaceCentered) spgrnum="216";
	    else if(pbravais==Data::BodyCentered) spgrnum="217";
	    else spgrnum="215";
	  }else if(!pp23.isEmpty()){
	    commonorigin=pp23.O.fromColumn(0);
	    if(pbravais==Data::FaceCentered) spgrnum="219";
	    else spgrnum="218";
	  }else if(!pp3_21pairs.isEmpty()){
	    commonorigin=pp3_21pairs.O.fromColumn(0);
	    spgrnum="220";
	  }
	}else{
	  pointgroup="23";
	  if(!pp23.isEmpty()){
	    commonorigin=pp23.O.fromColumn(0);
	    if(pbravais==Data::BodyCentered) spgrnum="197";// I23
	    else if(pbravais==Data::FaceCentered) spgrnum="196";// F23
	    else spgrnum="195";// P23
	  }else if(!pp3_21pairs.isEmpty()){
	    commonorigin=pp3_21pairs.O.fromColumn(0);
	    if(pbravais==Data::BodyCentered) spgrnum="199";// I213
	    else spgrnum="198";// P213
	  }
	}
      }
    }

    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);

    if(spgrnum=="0")
      throw Except("SeitzMatrix: cannot determine for cubic group?");

  }else if(hasC4 || hasB4){//tetragonalsystem
    pcrystalsystem=Data::Tetragonal;
    if(ops[MyMath::max<long>(indC4,indB4)].pinfo.pp==2){
      pbravais=Data::BodyCentered;
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Data::bravaisMatrix(Data::FaceCentered)*2;
    }


    {
      // A^T*Umix^T*BT * UTbasis=some Ast^T (st=simple tetragonal)
      
      UTbasis=ops[MyMath::max<long>(indC4,indB4)].findTetPlanarBasisFromC4();
      //myDebug()<<"UTbasis after=\n"<<UTbasis;
      
      UTbasis=UTbasis*Bcenter;
      
      if(!UTbasis.isInteger()){
	myDebug()<<"UTbasis=\n"<<UTbasis;
	throw Except("UTbasis not integer?");
      }
      
      //myDebug()<<"UTbasis=\n"<<UTbasis;
      //myDebug()<<"BT*UTbasis=\n"<<BT*UTbasis;
      changeBasis(ops,UTbasis,true);
      Stransform*=UTbasis;
      
      if(prot){
	for(long i=0;i<ops.size();i++){
	  if(!ops[i].pactive)continue;
	  //myDebug()<<i<<"old basis"<<ops.at(i);
	  printf("DET LATTICE ANALYSE %ld\n",i);
	  ops[i].analyse(true);
	  if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	    throw Except("Negative operation power should not be here.");
	}
      }
      
    }



    IMatArray primaxes;
    primaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,0,1)).shorten();
    IMatArray secoaxes;
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,0,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,1,0)).shorten();
    IMatArray tertaxes;
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,1,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,-1,0)).shorten();
    
    
    MainAxisOperation maaxops(ops,primaxes,secoaxes,tertaxes);

    const LongArray &indsc2s=maaxops.secondaryOperations(2);
    const LongArray &indsms=maaxops.secondaryOperations(-2);
    const LongArray &indsmt=maaxops.tertiaryOperations(-2);

    long indmp=maaxops.primaryOperation(-2);
    long indC2p=maaxops.primaryOperation(2);
    long indC2t=maaxops.tertiaryOperation(2);

    SeitzMatrixInfo &pinfoC4=ops[MyMath::max<long>(0,indC4)].pinfo;
    

    FMatrix deltab110=Bcenterinv.toIMatrix()*FMatrix::vector3(-1,1,0,4);
    FMatrix delta110=Bcenterinv.toIMatrix()*FMatrix::vector3(-1,-1,0,4);
    FMatrix delta021=Bcenterinv.toIMatrix()*FMatrix::vector3(0,-2,-1,8);


    PeriodicHyperPlane ppb4;
    if(indB4>=0) ppb4=ops[indB4].pinfo.toPHPlane(-1);
    PeriodicHyperPlane ppc4p; // any 4_i axis
    PeriodicHyperPlane ppc40p; // any 4_0 axis
    PeriodicHyperPlane ppc42p; // any 4_2 axis
    PeriodicHyperPlane ppc43p; // any 4_3 axis
    if(indC4>=0){
      ppc4p=pinfoC4.toPHPlane(-1);
      ppc40p=pinfoC4.toPHPlane(0);
      ppc42p=pinfoC4.toPHPlane(2);
      ppc43p=pinfoC4.toPHPlane(3);
    }
    PeriodicHyperPlane ppc20p; // prim 2_0 axis
    if(indC2p>=0) ppc20p=ops[indC2p].pinfo.toPHPlane(0);
    PeriodicHyperPlane ppmp; // primary {m|0} mirror
    if(indmp>=0) ppmp=ops[indmp].pinfo.toPHPlane(0);
    PeriodicHyperPlane ppmas; // any secondary {m|tau} mirror
    if(indsms.size()>0) ppmas=ops[indsms.at(0)].pinfo.toPHPlane(-1);
    PeriodicHyperPlane ppallmas=maaxops.commonOrigin(Bcenter,indsms,-1);
    PeriodicHyperPlane ppmat; // any tertiary {m|tau} mirror
    if(indsmt.size()>0) ppmat=ops[indsmt.at(0)].pinfo.toPHPlane(-1);
    PeriodicHyperPlane ppc20xs; // secondary c2 
    if(indsc2s.size()>0) ppc20xs=ops[indsc2s.at(0)].pinfo.toPHPlane(0);
    PeriodicHyperPlane ppc20t; // tertiary c2 
    if(indC2t>=0) ppc20t=ops[indC2t].pinfo.toPHPlane(0);
    

    

    PeriodicHyperPlane pp422(ppc40p);
    pp422.commonPoints(ppc20xs);
    pp422.commonPoints(ppc20t);



    if(indI>=0){
      PeriodicHyperPlane ppI=ops[indI].pinfo.toPHPlane(-1);
      if(maaxops.hasSecondaryMirror()){
	pointgroup="4/mmm";//123-138,139-142
	const SeitzMatrixInfo &pinfoms=ops[indsms.at(0)].pinfo;
	const SeitzMatrixInfo &pinfomt=ops[indsmt.at(0)].pinfo;

	PeriodicHyperPlane pp4om11(ppc40p);
	pp4om11.commonPoints(ppmp);
	PeriodicHyperPlane ppms;
	ppms=ops[indsms.at(0)].pinfo.toPHPlane(0);
	PeriodicHyperPlane pp12om1(ppc20xs);
	pp12om1.commonPoints(ppms);
	PeriodicHyperPlane pp4m1ma(pp4om11);
 	pp4m1ma.commonPoints(ppmat);
	PeriodicHyperPlane ppb4ma1(ppb4);
	ppb4ma1.commonPoints(ppmas);
	PeriodicHyperPlane pp42om(ppc42p);
	pp42om.commonPoints(ppmp);
	PeriodicHyperPlane ppallmat=maaxops.commonOrigin(Bcenter,indsmt,-1);
	
	PeriodicHyperPlane pp;
	if(!(pp=PeriodicHyperPlane(pp4om11).commonPoints(pp12om1)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(pbravais==Data::BodyCentered) spgrnum="139";
	  else spgrnum="123";
	}else if(!(pp=PeriodicHyperPlane(pp4m1ma).commonPoints(ppmas)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(pbravais==Data::BodyCentered) spgrnum="140";
	  else spgrnum="124";
	}else if(!pp422.isEmpty()){
	  commonorigin=pp422.O.fromColumn(0);
	  if((!pinfomt.pnonsymmorphic)) spgrnum="125:1";
	  else spgrnum="126:1";
	}else if(!pp4m1ma.isEmpty()){
	  commonorigin=pp4m1ma.O.fromColumn(0);
	  if((!pinfomt.pnonsymmorphic)) spgrnum="127";
	  else spgrnum="128";
	}else if(!(pp=PeriodicHyperPlane(pp42om).commonPoints(ppallmas)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if((!pinfoms.pnonsymmorphic)) spgrnum="131";
	  else spgrnum="132";
	}else if(pbravais==Data::BodyCentered && 
		 !(pp=PeriodicHyperPlane(ppb4ma1).commonPoints(ppI,delta021)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if((!pinfoms.pnonsymmorphic)) spgrnum="141:1";
	  else spgrnum="142:1";
	}else if(!ppb4ma1.isEmpty()){
	  commonorigin=ppb4ma1.O.fromColumn(0);
	  if((!pinfoms.pnonsymmorphic)){
	    if((!pinfomt.pnonsymmorphic)) spgrnum="129:1"; 
	    else spgrnum="137:1";
	  }else{
	    if((!pinfomt.pnonsymmorphic))spgrnum="138:1";
	    else spgrnum="130:1";
	  }
	}else if(!(pp=PeriodicHyperPlane(pp42om).commonPoints(ppallmat)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if((!pinfomt.pnonsymmorphic)) {
	    commonorigin+=FMatrix::vector3(0,-1,0,2);
	    spgrnum="136";
	  }
	  else spgrnum="135";
	}else if(!(pp=PeriodicHyperPlane(ppb4).commonPoints(ppmat)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if((!pinfomt.pnonsymmorphic)) spgrnum="134:1";
	  else spgrnum="133:1";
	}
      }else{
	pointgroup="4/m";//83-86,87-88
	
	PeriodicHyperPlane pp;
	if(!(pp=PeriodicHyperPlane(ppc40p).commonPoints(ppmp)).isEmpty()){
	  if(pbravais==Data::BodyCentered) spgrnum="87";
	  else spgrnum="83";
	}else if(!(pp=PeriodicHyperPlane(ppc42p).commonPoints(ppmp)).isEmpty()){
	  spgrnum="84";
	}else if(!(pp=PeriodicHyperPlane(ppI)
		   .commonPoints(ppb4,delta021)
		   ).isEmpty()){
	  spgrnum="88:2";
	}else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(ppc40p,delta110)).isEmpty()){
	  spgrnum="85:2";
	}else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(ppc42p,deltab110)).isEmpty()){
	  spgrnum="86:2";
	}
	commonorigin=pp.O.fromColumn(0);
      }
    }else{
      if(hasB4){
	if(maaxops.hasSecondaryRotation() && maaxops.hasTertiaryMirror()){
	  pointgroup="-42m";//111-114,121-122
	  PeriodicHyperPlane pp;
	  PeriodicHyperPlane ppmt; //  tertiary {m|0} mirror
	  ppmt=ops[indsmt.at(0)].pinfo.toPHPlane(0);
	  const SeitzMatrixInfo &pinfomt=ops[indsmt.at(0)].pinfo;
	  if(pbravais==Data::BodyCentered){
	    if(!(pp=PeriodicHyperPlane(ppb4).commonPoints(ppmt)).isEmpty()){
	      spgrnum="121";
	    }else if(!(pp=PeriodicHyperPlane(ppb4)
		       .commonPoints(ppc20xs,delta021)).isEmpty()){
	      spgrnum="122";
	    }
	  }else{
	    if(!(pp=PeriodicHyperPlane(ppb4).commonPoints(ppmat)).isEmpty()){
	      if(!ops[indsc2s.at(0)].pinfo.pnonsymmorphic){
		if((!pinfomt.pnonsymmorphic)) spgrnum="111";
		else spgrnum="112";
	      }else{
		if((!pinfomt.pnonsymmorphic)) spgrnum="113";
		else spgrnum="114";
	      }
	    }
	  } 
	  commonorigin=pp.O.fromColumn(0);
	}else if(maaxops.hasSecondaryMirror() && maaxops.hasTertiaryRotation()){
	  pointgroup="-4m2";//115-118,119-120
	  PeriodicHyperPlane ppc21t; // tertiary c2 
	  ppc21t=ops[indC2t].pinfo.toPHPlane(1);
	  PeriodicHyperPlane pp;
	  const SeitzMatrixInfo &pinfoms=ops[indsms.at(0)].pinfo;
	  if(pbravais==Data::BodyCentered){
	    if(!ppb4.isEmpty()){
	      commonorigin=ppb4.O.fromColumn(0);
	      if((!pinfoms.pnonsymmorphic)){
		spgrnum="119";//I-4m2
	      }else{
		spgrnum="120";//I-4c2
	      }
	    }
	  }else{
	    if(!(pp=PeriodicHyperPlane(ppb4).commonPoints(ppallmas)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      if((!pinfoms.pnonsymmorphic)) spgrnum="115";
	      else spgrnum="116";
	    }else if(!(pp=PeriodicHyperPlane(ppb4).commonPoints(ppc21t)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      spgrnum="117";
	    }else if(!ppb4.isEmpty()){
	      commonorigin=ppb4.O.fromColumn(0);
	      spgrnum="118";
	    }
	  }
	}else{
	  pointgroup="-4";//81,82
	  if(!ppb4.isEmpty()){
	    commonorigin=ppb4.O.fromColumn(0);
	    if(pbravais==Data::BodyCentered) spgrnum="82";
	    else spgrnum="81";
	  }
	}
      }else{

	if(maaxops.hasSecondaryRotation() && maaxops.hasTertiaryRotation()){
	  pointgroup="422"; //89-96,97-98

	  PeriodicHyperPlane pp;
	  PeriodicHyperPlane ppc2p;
	  ppc2p=ops[indC2p].pinfo.toPHPlane(-1);
	  PeriodicHyperPlane pp212(ppc2p);
	  pp212.commonPoints(ppc20t);
	  PeriodicHyperPlane ppc20ys; // secondary c2y 
	  ppc20ys=ops[indsc2s.at(1)].pinfo.toPHPlane(0);
	  
	  if(!pp422.isEmpty()){
 	    commonorigin=pp422.O.fromColumn(0);
 	    if(pbravais==Data::BodyCentered) spgrnum="97";
	    else spgrnum="89";
	  }else if(!(pp=PeriodicHyperPlane(pp212)
		     .commonPoints(ppc20xs,delta021)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    if(pbravais==Data::BodyCentered) spgrnum="98";
	  }else if(!(pp=PeriodicHyperPlane(ppc4p).commonPoints(ppc20ys)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    spgrnum=Sprintf("%ld",89+pinfoC4.skrew()*2);
	  }else if(!pp212.isEmpty()){
	    commonorigin=pp212.O.fromColumn(0);
	    spgrnum=Sprintf("%ld",90+pinfoC4.skrew()*2);
	  }

	}else if(maaxops.hasSecondaryMirror() && maaxops.hasTertiaryMirror()){
	  pointgroup="4mm"; //99-106,107-110
	  PeriodicHyperPlane pp1mama(ppmas);
	  pp1mama.commonPoints(ppmat);
	  PeriodicHyperPlane pp40mama(ppc40p);// 40 and any seco+tert m
	  pp40mama.commonPoints(pp1mama);
	  //myDebug()<<"PHP 40 any-m any-m"<<pp40mama.printS(Bcenter);
	  PeriodicHyperPlane pp;
	  const SeitzMatrixInfo &pinfoms=ops[indsms.at(0)].pinfo;
	  const SeitzMatrixInfo &pinfomt=ops[indsmt.at(0)].pinfo;
	  if(pbravais==Data::BodyCentered){
	    if(!pp40mama.isEmpty()){
	      commonorigin=pp40mama.O.fromColumn(0);
	      if((!pinfoms.pnonsymmorphic) && (!pinfomt.pnonsymmorphic)) spgrnum="107";
	      else spgrnum="108";
	    }else if(!(pp=PeriodicHyperPlane(ppc20p).commonPoints(ppmas)
		       .commonPoints(ppc43p,delta110)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      if((!pinfoms.pnonsymmorphic)) spgrnum="109";// on 2m1
	      else spgrnum="110";// on 2c1
	    }
	  }else{
	    if(!(pp=PeriodicHyperPlane(ppc4p).commonPoints(pp1mama)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      if(!pinfoC4.pnonsymmorphic){
		if((!pinfomt.pnonsymmorphic)) spgrnum="99";
		else spgrnum="103";
	      }else{
		if((!pinfomt.pnonsymmorphic)) spgrnum="101";
		else spgrnum="105";
	      }
	    }else if(!(pp=PeriodicHyperPlane(ppc4p).commonPoints(ppmat)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      if(!pinfoC4.pnonsymmorphic){
		if((!pinfomt.pnonsymmorphic)) spgrnum="100";
		else spgrnum="104";
	      }else{
		if((!pinfomt.pnonsymmorphic)) { 
		  spgrnum="102";
		  commonorigin-=FMatrix::vector3(1,0,0,2);
		}else spgrnum="106";
	      }
	    }
	  }
	}else{
	  pointgroup="4"; //75-77,79-80
	  PeriodicHyperPlane pp;
	  if(pbravais==Data::BodyCentered){
	    if(!ppc40p.isEmpty()) {
	      commonorigin=ppc40p.O.fromColumn(0);
	      spgrnum="79";
	    }else if(!(pp=PeriodicHyperPlane(ppc20p)
		       .commonPoints(ppc43p,delta110)).isEmpty()){
	      commonorigin=pp.O.fromColumn(0);
	      spgrnum="80";
	    }
	  }else{
	    if(!ppc4p.isEmpty()){
	      commonorigin=ppc4p.O.fromColumn(0);
	      spgrnum=Sprintf("%ld",75+pinfoC4.skrew());
	    }
	  }
	}
      }
    }


    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);

    if(spgrnum=="0")
      throw Except("SeitzMatrix: non standard tau0 for tetragonal group?");

  }else if(indB6>=0 || indC6 >=0){//hexagonal system
    pcrystalsystem=Data::Hexagonal;
    ASSERT(indC3>=0,"SeitzMatrix: did not find 3 in hexagonal?");
    
    // A^T*Umix^T*BT * UTbasis=some Ahex^T
    UTbasis=ops[indC3].findHexPlanarBasisFromC3();
    //myDebug()<<"UTbasis=\n"<<UTbasis;
    //myDebug()<<"BT*UTbasis=\n"<<BT*UTbasis;

    if(!UTbasis.isInteger()){
      myDebug()<<"UTbasis=\n"<<UTbasis;
      throw Except("UTbasis not integer?");
    }

    changeBasis(ops,UTbasis,true);
    Stransform*=UTbasis;



    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("CHANGANALYSE %ld\n",i);
	ops[i].analyse(true);
	if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	  throw Except("Negative operation power should not be here.");
      }
    }

    IMatArray primaxes;
    primaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,0,1)).shorten();
    IMatArray secoaxes;
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,0,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,1,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,1,0)).shorten();
    IMatArray tertaxes;
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(2,1,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,2,0)).shorten()
	    <<(Bcenterinv.toIMatrix()*IMatrix::vector3(2,-1,0)).shorten();
    

    MainAxisOperation maaxops(ops,primaxes,secoaxes,tertaxes);

    const LongArray &indsc2s=maaxops.secondaryOperations(2);
    const LongArray &indsms=maaxops.secondaryOperations(-2);
    const LongArray &indsmt=maaxops.tertiaryOperations(-2);
    const LongArray &indsc2t=maaxops.tertiaryOperations(2);

    SeitzMatrixInfo &pinfoC6=ops[MyMath::max<long>(0,indC6)].pinfo;
    SeitzMatrixInfo &pinfoC3=ops[MyMath::max<long>(0,indC3)].pinfo;
    SeitzMatrixInfo &pinfoB6=ops[MyMath::max<long>(0,indB6)].pinfo;

    PeriodicHyperPlane ppc6p; // any 6_i axis
    if(indC6>=0) ppc6p=pinfoC6.toPHPlane(-1);
    

    long addspgrnum177168[]={0,1,3,5,4,2};

    PeriodicHyperPlane pp;

    if(indI>=0){
      ASSERT(indB6>=0,"SeitzMatrix: did not find -6 in 6/m or 6/mmm?");
      ASSERT(indB3>=0,"SeitzMatrix: did not find -3 ?");
      PeriodicHyperPlane pp6I(ppc6p);
      pp6I.commonPoints(ops[indI].pinfo.toPHPlane(-1));
      if(maaxops.hasSecondaryRotation()){
	pointgroup="6/mmm";
	const SeitzMatrixInfo &pinfoms=ops[indsms.at(0)].pinfo;
	if(!pp6I.isEmpty()){
	  commonorigin=pp6I.O.fromColumn(0);
	  if(!pinfoC6.pnonsymmorphic){
	    if((!pinfoms.pnonsymmorphic)) spgrnum="191";
	    else spgrnum="192";
	  }else if(pinfoC6.skrew()==3){
	    if((!pinfoms.pnonsymmorphic)) spgrnum="194";
	    else spgrnum="193";
	  }
	}        
      }else{
	pointgroup="6/m";
	if(!pp6I.isEmpty()){
	  commonorigin=pp6I.O.fromColumn(0);
	  if(!pinfoC6.pnonsymmorphic) spgrnum="175";
	  else if(pinfoC6.skrew()==3) spgrnum="176";
	}
      }
    }else{
      if(indB6>=0){
	PeriodicHyperPlane ppc30p;
	ppc30p=pinfoC3.toPHPlane(0);
	if(maaxops.hasSecondaryMirror() && maaxops.hasTertiaryRotation()){
	  pointgroup="-6m2";
	  PeriodicHyperPlane ppallc20t;
	  ppallc20t=maaxops.commonOrigin(Bcenter,indsc2t,0);
	  const SeitzMatrixInfo &pinfoms=ops[indsms.at(0)].pinfo;
	  if(!(pp=PeriodicHyperPlane(ppc30p).commonPoints(ppallc20t)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    if((!pinfoms.pnonsymmorphic)) spgrnum="187";
 	    else spgrnum="188";
	  }
	}else if(maaxops.hasSecondaryRotation() &&maaxops.hasTertiaryMirror()){
	  pointgroup="-62m";
	  PeriodicHyperPlane ppallc20s;
	  ppallc20s=maaxops.commonOrigin(Bcenter,indsc2s,0);
	  const SeitzMatrixInfo &pinfomt=ops[indsmt.at(0)].pinfo;
	  if(!(pp=PeriodicHyperPlane(ppc30p).commonPoints(ppallc20s)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    if((!pinfomt.pnonsymmorphic)) spgrnum="189";
	    else spgrnum="190";
	  }
	}else{
	  pointgroup="-6";
	  if(!(pp=pinfoB6.toPHPlane(-1)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    spgrnum="174";
	  }
	}
      }else{
	if(maaxops.hasSecondaryRotation()&& maaxops.hasTertiaryRotation()){
	  pointgroup="622";
	  PeriodicHyperPlane ppc20s;
	  ppc20s=ops[indsc2s.at(0)].pinfo.toPHPlane(0);
	  if(!(pp=PeriodicHyperPlane(ppc6p).commonPoints(ppc20s)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    spgrnum=Sprintf("%ld",177+addspgrnum177168[pinfoC6.skrew()]);
	  }
	}else if(maaxops.hasSecondaryMirror()&&maaxops.hasTertiaryMirror()){
	  pointgroup="6mm";
	  PeriodicHyperPlane ppallmas;
	  ppallmas=maaxops.commonOrigin(Bcenter,indsms,-1);
	  PeriodicHyperPlane pp6ma1(ppc6p);
	  pp6ma1.commonPoints(ppallmas);
	  PeriodicHyperPlane ppallmat;
	  ppallmat=maaxops.commonOrigin(Bcenter,indsmt,-1);
	  PeriodicHyperPlane ppallmt;
	  ppallmt=maaxops.commonOrigin(Bcenter,indsmt,0);
	  if(!(pp=PeriodicHyperPlane(pp6ma1).commonPoints(ppallmt)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    if(!pinfoC6.pnonsymmorphic) spgrnum="183";
	    else spgrnum="185";
	  }else if(!(pp=PeriodicHyperPlane(pp6ma1).commonPoints(ppallmat)).isEmpty()){
	    commonorigin=pp.O.fromColumn(0);
	    if(!pinfoC6.pnonsymmorphic) spgrnum="184";
	    else spgrnum="186";
	  }
	}else{
	  pointgroup="6";
	  if(!ppc6p.isEmpty()){
	    commonorigin=ppc6p.O.fromColumn(0);
	    spgrnum=Sprintf("%ld",168+addspgrnum177168[pinfoC6.skrew()]);
	  }
	}
      }
    }
    
    
    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);


    if(spgrnum=="0")
      throw Except("SeitzMatrix: non standard tau0 for hex group?");
    
  }else if(indC3>=0){//trigonal system


    if(ops[indC3].pinfo.pp==1){
      pcrystalsystem=Data::Trigonal;
    }else{
      pcrystalsystem=Data::Rhombohedral;
      Bcenter=Data::bravaisMatrix(Data::BRhombohedral);
      Bcenterinv=Bcenter.inverse();
    }
    



    // A^T*Umix^T*BT * UTbasis=some Ahex^T
    
    UTbasis=ops[indC3].findHexPlanarBasisFromC3();

    if(pcrystalsystem==Data::Rhombohedral){
      UTbasis=UTbasis*Bcenter;
    }
    
    if(!UTbasis.isInteger()){
      UTbasis=UTbasis*Bcenterinv*C6*Bcenter;
    }
    if(!UTbasis.isInteger()){
      myDebug()<<"UTbasis=\n"<<UTbasis;
      throw Except("UTbasis not integer?");
    }

    //myDebug()<<"UTbasis=\n"<<UTbasis;
    //myDebug()<<"BT*UTbasis=\n"<<BT*UTbasis;

    changeBasis(ops,UTbasis,true);
    Stransform*=UTbasis;


    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("CHANGANALYSE %ld\n",i);
	ops[i].analyse(true);
	if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	  throw Except("Negative operation power should not be here.");
      }
    }

    //todo: make from const data and use IMatrix instead of IMatArray

    IMatArray primaxes;
    IMatArray secoaxes;
    IMatArray tertaxes;

    primaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,0,1)).shorten();
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,1,0)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(2,1,0)).shorten();

    MainAxisOperation maaxops(ops,primaxes,secoaxes,tertaxes);

    const SeitzMatrixInfo &pinfoC3=ops[MyMath::max<long>(0,indC3)].pinfo;
    const SeitzMatrixInfo &pinfoB3=ops[MyMath::max<long>(0,indB3)].pinfo;
    const LongArray &indsms=maaxops.secondaryOperations(-2);
    const LongArray &indsmt=maaxops.tertiaryOperations(-2);
    const LongArray &indsc2s=maaxops.secondaryOperations(2);
    const LongArray &indsc2t=maaxops.tertiaryOperations(2);

    PeriodicHyperPlane ppb3p; //  -3
    PeriodicHyperPlane ppc3p; // any 3_i axis
    PeriodicHyperPlane ppc30p; // any 3_i axis
    if(indB3>=0) ppb3p=pinfoB3.toPHPlane(-1);
    if(indC3>=0){
      ppc3p=pinfoC3.toPHPlane(-1);
      ppc30p=pinfoC3.toPHPlane(0);
    }

    PeriodicHyperPlane ppmas; // any secondary {m|tau} mirror
    PeriodicHyperPlane ppmat; // any tetiary {m|tau} mirror
    if(indsms.size()>0) ppmas=ops[indsms.at(0)].pinfo.toPHPlane(-1);
    if(indsmt.size()>0) ppmat=ops[indsmt.at(0)].pinfo.toPHPlane(-1);
    


    PeriodicHyperPlane pp;
    if(indI>=0){
      ASSERT(indB3>=0,"SeitzMatrix: did not find -3 in -3?");
      if(maaxops.hasSecondaryMirror()){
	pointgroup="-3m1";
	if(!(pp=PeriodicHyperPlane(ppb3p).commonPoints(ppmas)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(pcrystalsystem==Data::Rhombohedral){
	    if(ops[indsms.at(0)].pinfo.pnonsymmorphic) spgrnum="167:R";
	    else spgrnum="166:R";
	  }else{
	    if(ops[indsms.at(0)].pinfo.pnonsymmorphic) spgrnum="165";
	    else spgrnum="164";
	  }
	}
      }else if(maaxops.hasTertiaryMirror()){
	pointgroup="-31m";
	if(!(pp=PeriodicHyperPlane(ppb3p).commonPoints(ppmat)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(ops[indsmt.at(0)].pinfo.pnonsymmorphic) spgrnum="163";
	  else spgrnum="162";
	}
      }else if(maaxops.hasOnlyPrimary()){
	pointgroup="-3";
	if(!ppb3p.isEmpty()){
	  commonorigin=ppb3p.O.fromColumn(0);
	  if(pcrystalsystem==Data::Rhombohedral) spgrnum="148:R";
	  else spgrnum="147";
	}
      }
    }else{ // noinversion
      if(maaxops.hasSecondaryMirror()){
	pointgroup="3m1";
	if(!(pp=PeriodicHyperPlane(ppc30p).commonPoints(ppmas)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(pcrystalsystem==Data::Rhombohedral){
	    if(ops[indsms.at(0)].pinfo.pnonsymmorphic) spgrnum="161:R";
	    else spgrnum="160:R";
	  }else{ 
	    if(ops[indsms.at(0)].pinfo.pnonsymmorphic) spgrnum="158";
	    else spgrnum="156";
	  }
	}	
      }else if(maaxops.hasTertiaryMirror()){
	pointgroup="31m";
	if(!(pp=PeriodicHyperPlane(ppc30p).commonPoints(ppmat)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  if(ops[indsmt.at(0)].pinfo.pnonsymmorphic) spgrnum="159";
	  else spgrnum="157";
	}
      }else if(maaxops.hasSecondaryRotation()){
	pointgroup="321";
	PeriodicHyperPlane ppc2xxs=ops[indsc2s.at(0)].pinfo.toPHPlane(0);
	if(!(pp=PeriodicHyperPlane(ppc30p).commonPoints(ppc2xxs)).isEmpty() && 
	   (pcrystalsystem==Data::Rhombohedral)){
	  commonorigin=pp.O.fromColumn(0);
	  spgrnum="155:R";
	}else if(!(pp=PeriodicHyperPlane(ppc3p).commonPoints(ppc2xxs)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  spgrnum=Sprintf("%ld",150+pinfoC3.skrew()*2);
	}
      }else if(maaxops.hasTertiaryRotation()){
	pointgroup="312";
	PeriodicHyperPlane ppc2xxt=ops[indsc2t.at(0)].pinfo.toPHPlane(0);
	if(!(pp=PeriodicHyperPlane(ppc3p).commonPoints(ppc2xxt)).isEmpty()){
	  commonorigin=pp.O.fromColumn(0);
	  spgrnum=Sprintf("%ld",149+pinfoC3.skrew()*2);
	}
      }else if(maaxops.hasOnlyPrimary()){
	pointgroup="3";
	if(!ppc30p.isEmpty() && (pcrystalsystem==Data::Rhombohedral)){
	  commonorigin=ppc30p.O.fromColumn(0);
	  spgrnum="146:R";
	}else if(!ppc3p.isEmpty()){
	  commonorigin=ppc3p.O.fromColumn(0);
	  spgrnum=Sprintf("%ld",143+pinfoC3.skrew());
	}
      }
    }
    

    
    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);
    
    // default setting here is simple rhombohedral as crystalsystem
    Bcenter=FMatrix::diagMatrix(3,3,1);
    Bcenterinv=FMatrix::diagMatrix(3,3,1);


    if(spgrnum=="0")
      throw Except("SeitzMatrix: non standard tau0 for trigonal group?");
    
  }else if(orthoC2.size()+orthom.size()>=3){//orthorhombic system
    pcrystalsystem=Data::Orthorhombic;
    
    //myDebug()<<"orthoC2="<<orthoC2;
    //myDebug()<<"orthom="<<orthom;



    // pp 1 1 1 S
    // pp 2 2 2 FC/BC
    // pp 1 2 2 AFC 
    // pp 2 1 2 BFC
    // pp 2 2 1 CFC
    pbravais=Data::Unkown;
    long ppsum=0;
    UTbasis.setDimenions(3,0);
    if(orthoC2.size()==3){
      if(orthom.size()==3) {
	pointgroup="mmm";
	// need to make sure that the order of orthoc2 fits orthom
	for(long i=0;i<3;i++){
	  for(long j=i+1;j<3;j++){
	    if( 
	       ops[orthoC2.at(i)].pinfo.Pmin==ops[orthom.at(j)].pinfo.Pmin || 
	       ops[orthoC2.at(i)].pinfo.Pmin==-ops[orthom.at(j)].pinfo.Pmin){
	      orthom.swap(i,j);
	    }
	  }
	}
	// for(long i=0;i<3;i++){
	//   myDebug()<<ops[orthoC2.at(i)].pinfo.Pmin-ops[orthom.at(i)].pinfo.Pmin;
	// }
	
	
      }else pointgroup="222";
      
      ppsum=ops[orthoC2.at(0)].pinfo.pp+ops[orthoC2.at(1)].pinfo.pp+
	ops[orthoC2.at(2)].pinfo.pp;
      
      if(ppsum==5){
	pbravais=Data::CBaseCentered;
	if(ops[orthoC2.at(0)].pinfo.pp==1){ //AFC->CFC
	  orthoC2.swap(0,2);
	  if(orthom.size()==3)orthom.swap(0,2);
	}else if(ops[orthoC2.at(1)].pinfo.pp==1){ // BFC->CFC
	  orthoC2.swap(1,2);
	  if(orthom.size()==3)orthom.swap(1,2);
	}
      }
      
      UTbasis.addMatrix(ops[orthoC2.at(0)].pinfo.Pmin);
      UTbasis.addMatrix(ops[orthoC2.at(1)].pinfo.Pmin);
      UTbasis.addMatrix(ops[orthoC2.at(2)].pinfo.Pmin);
    }else{
      pointgroup="mm2";
      
      ppsum=ops[orthom.at(0)].pinfo.pp+ops[orthom.at(1)].pinfo.pp+
	ops[orthoC2.at(0)].pinfo.pp;

      if(ppsum==5){
	pbravais=Data::ABaseCentered;
	if(ops[orthoC2.at(0)].pinfo.pp==1) pbravais=Data::CBaseCentered;
	else if(ops[orthom.at(1)].pinfo.pp==1) orthom.swap(0,1);//BFC->AFC
      }
      UTbasis.addMatrix(ops[orthom.at(0)].pinfo.Pmin);
      UTbasis.addMatrix(ops[orthom.at(1)].pinfo.Pmin);
      UTbasis.addMatrix(ops[orthoC2.at(0)].pinfo.Pmin);
    }
    
    if(ppsum==3) pbravais=Data::Simple;
    if(pbravais==Data::CBaseCentered){
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Bcenter.inverse();
    }else if(pbravais==Data::ABaseCentered){
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Bcenter.inverse();
    }
    
    Fractional detUT=UTbasis.determinant();
    //myDebug()<<"det=\n"<<detUT;
    if(detUT<0){
      detUT*=-1;
      UTbasis.scaleColumn(0,-1);
    }
    //myDebug()<<"UTbasis conv=\n"<<UTbasis.printS();
    //myDebug()<<"det=\n"<<UTbasis.determinant();

    if(abs(detUT)==4){
      pbravais=Data::FaceCentered;
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Data::bravaisMatrix(Data::BodyCentered)*2;
    }else if(abs(detUT)==2 && pbravais==Data::Unkown){
      pbravais=Data::BodyCentered;
      Bcenter=Data::bravaisMatrix(pbravais);
      Bcenterinv=Data::bravaisMatrix(Data::FaceCentered)*2;
    }
    
    //myDebug()<<"det=\n"<<UTbasis.determinant();
    UTbasis=UTbasis*Bcenter;
    
    //myDebug()<<"BCenter=\n"<<Bcenter;
    //myDebug()<<"BCenterinv=\n"<<Bcenter.inverse();
    //myDebug()<<"UTbasis prim=\n"<<UTbasis.printS();

    if(!UTbasis.isInteger()){
      myDebug()<<"UTbasis=\n"<<UTbasis;
      throw Except("UTbasis not integer?");
    }
    //myDebug()<<"det=\n"<<UTbasis.determinant();

    changeBasis(ops,UTbasis,true);
    Stransform*=UTbasis;

    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	//myDebug()<<i<<"old basis"<<ops.at(i);
	printf("DET LATTICE ANALYSE %ld\n",i);
	ops[i].analyse(true);
	if(ops.at(i).pinfo.pm1<0 && ops.at(i).pinfo.pnonsymmorphic)
	  throw Except("Negative operation power should not be here.");
      }
    }

    IMatArray primaxes;
    IMatArray secoaxes;
    IMatArray tertaxes;
    primaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(1,0,0)).shorten();
    secoaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,1,0)).shorten();
    tertaxes<<(Bcenterinv.toIMatrix()*IMatrix::vector3(0,0,1)).shorten();

    FMatrix delta100=Bcenterinv.toIMatrix()*FMatrix::vector3(-1,0,0,4);
    FMatrix delta010=Bcenterinv.toIMatrix()*FMatrix::vector3(0,-1,0,4);
    FMatrix delta001=Bcenterinv.toIMatrix()*FMatrix::vector3(0,0,-1,4);
    FMatrix delta011=Bcenterinv.toIMatrix()*FMatrix::vector3(0,-1,-1,4);
    FMatrix delta101=Bcenterinv.toIMatrix()*FMatrix::vector3(-1,0,-1,4);
    FMatrix delta110=Bcenterinv.toIMatrix()*FMatrix::vector3(-1,-1,0,4);
    FMatrix delta111=Bcenterinv*FMatrix::vector3(-1,-1,-1,8)
      *(pbravais==Data::Simple?2:1);



    MainAxisOperation maaxops(ops,primaxes,secoaxes,tertaxes);

    long indc2p=maaxops.primaryOperation(2);
    long indc2s=maaxops.secondaryOperation(2);
    long indc2t=maaxops.tertiaryOperation(2);
    long indms=maaxops.secondaryOperation(-2);
    long indmp=maaxops.primaryOperation(-2);
    long indmt=maaxops.tertiaryOperation(-2);
    LongArray indsc2; 
    if(indc2p>=0)indsc2<<indc2p;
    if(indc2s>=0)indsc2<<indc2s;
    if(indc2t>=0)indsc2<<indc2t;
    

    PeriodicHyperPlane pp20x;
    PeriodicHyperPlane pp20y;
    PeriodicHyperPlane pp20z;
    PeriodicHyperPlane pp2z;
    PeriodicHyperPlane pp21x;
    PeriodicHyperPlane pp21y;
    PeriodicHyperPlane pp21z;
    PeriodicHyperPlane ppmp;
    PeriodicHyperPlane ppmap;
    PeriodicHyperPlane ppms;
    PeriodicHyperPlane ppmas;
    PeriodicHyperPlane ppmt;
    PeriodicHyperPlane ppmat;
    if(indms>=0) {
      ppms=ops[indms].pinfo.toPHPlane(0);
      ppmas=ops[indms].pinfo.toPHPlane(-1);
    }
    if(indmt>=0) {
      ppmt=ops[indmt].pinfo.toPHPlane(0);
      ppmat=ops[indmt].pinfo.toPHPlane(-1);
    }
    if(indmp>=0) {
      ppmp=ops[indmp].pinfo.toPHPlane(0);
      ppmap=ops[indmp].pinfo.toPHPlane(-1);
    }
    if(indc2p>=0) {
      pp20x=ops[indc2p].pinfo.toPHPlane(0);
      pp21x=ops[indc2p].pinfo.toPHPlane(1);
    }
    if(indc2s>=0) {
      pp20y=ops[indc2s].pinfo.toPHPlane(0);
      pp21y=ops[indc2s].pinfo.toPHPlane(1);
    }
    if(indc2t>=0) {
      pp2z=ops[indc2t].pinfo.toPHPlane(-1);
      pp20z=ops[indc2t].pinfo.toPHPlane(0);
      pp21z=ops[indc2t].pinfo.toPHPlane(1);
    }
    //myDebug()<<"PHP ..21="<<pp21z.printS(Bcenter);
    //myDebug()<<"PHP ..2i="<<pp2z.printS(Bcenter);
    //myDebug()<<"PHP .m.="<<ppms.printS(Bcenter);
    //myDebug()<<"PHP m..="<<ppmp.printS(Bcenter);
    

    PeriodicHyperPlane pp202020(pp20x);
    pp202020.commonPoints(pp20y);
    pp202020.commonPoints(pp20z);
    //myDebug()<<"PHP 222="<<pp202020.printS(Bcenter);

    long afteropind=-1;
    FMatrix aftershiftcond(3,1);
    FMatrix aftershift(3,1);

    if(pointgroup=="222"){
      PeriodicHyperPlane pp212121_from2x;// (010)/4 from 2x fixes orientation
      pp212121_from2x=maaxops.commonOrigin(Bcenter,indsc2,1,true);
      pp212121_from2x.commonPoints(pp21x,delta010);
      //myDebug()<<"PHP nonintersecting paris of 21 axis="
      //	       <<pp212121_from2x.printS(Bcenter);
      PeriodicHyperPlane pp;
      if(!pp202020.isEmpty()){
	commonorigin=pp202020.O.fromColumn(0); //  222
	if(pbravais==Data::Simple) spgrnum="16";
	else if(pbravais==Data::CBaseCentered) spgrnum="21";
	else if(pbravais==Data::FaceCentered) spgrnum="22";
	else if(pbravais==Data::BodyCentered) spgrnum="23";
      }else if(pbravais==Data::BodyCentered && (!pp212121_from2x.isEmpty())) {
	commonorigin=pp212121_from2x.O.fromColumn(0);// 212121: 3 pairs 21
	spgrnum="24";
      }else if(!(pp=PeriodicHyperPlane(pp21y).commonPoints(pp20z)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::Simple) spgrnum="17:bca";
      }else if(!(pp=PeriodicHyperPlane(pp21x).commonPoints(pp20y)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::Simple) spgrnum="17:cab";
      }else if(!(pp=PeriodicHyperPlane(pp21z).commonPoints(pp20x)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::Simple) spgrnum="17";
	else if(pbravais==Data::CBaseCentered) spgrnum="20";
      }else if(!pp212121_from2x.isEmpty()){
	commonorigin=pp212121_from2x.O.fromColumn(0);
	spgrnum="19";
      }else if(!(pp=PeriodicHyperPlane(pp20z) .commonPoints(pp21x,delta010)
		 .commonPoints(pp21y,delta100) ).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="18";
      }else if(!(pp=PeriodicHyperPlane(pp20x) .commonPoints(pp21y,delta001)
		 .commonPoints(pp21z,delta010) ).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="18:cab";
      }else if(!(pp=PeriodicHyperPlane(pp20y) .commonPoints(pp21z,delta100)
		 .commonPoints(pp21x,delta001) ).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="18:bca";
      }
    }else if(pointgroup=="mm2"){
      PeriodicHyperPlane ppmm20(pp20z);
      ppmm20.commonPoints(ppms);
      ppmm20.commonPoints(ppmp);
      //myDebug()<<"PHP mm2="<<ppmm20.printS(Bcenter);
      PeriodicHyperPlane ppmma21(pp21z);
      ppmma21.commonPoints(ppmp);
      ppmma21.commonPoints(ppmas);
      //myDebug()<<"PHP m any-m 21="<<ppmma21.printS(Bcenter);
      PeriodicHyperPlane ppmam21(pp21z);
      ppmam21.commonPoints(ppmap);
      ppmam21.commonPoints(ppms);
      //myDebug()<<"PHP any-m m 21="<<ppmam21.printS(Bcenter);
      PeriodicHyperPlane ppmama20(pp20z);
      ppmama20.commonPoints(ppmap);
      ppmama20.commonPoints(ppmas);
      //myDebug()<<"PHP any-m any-m 20="<<ppmama20.printS(Bcenter);
      PeriodicHyperPlane ppmama2(pp2z);
      ppmama2.commonPoints(ppmap);
      ppmama2.commonPoints(ppmas);
      //myDebug()<<"PHP any-m any-m 2i="<<ppmama2.printS(Bcenter);
      PeriodicHyperPlane pp1ma2(pp2z);
      pp1ma2.commonPoints(ppmas);
      //myDebug()<<"PHP . any-m 2i="<<pp1ma2.printS(Bcenter);
      
      PeriodicHyperPlane ppma12(pp2z);
      ppma12.commonPoints(ppmap);
      //myDebug()<<"PHP  any-m . 2i="<<ppma12.printS(Bcenter);

      String mpsignature=ops[indmp].pinfo.mirrorSignatures(Bcenter);
      if(!ppmm20.isEmpty() && ((pbravais==Data::CBaseCentered)
			      ||(pbravais==Data::FaceCentered))){
	commonorigin=ppmm20.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered) spgrnum="35";
	else if(pbravais==Data::FaceCentered) spgrnum="42";
      }else if(!ppmma21.isEmpty() && (pbravais==Data::CBaseCentered)){
	commonorigin=ppmma21.O.fromColumn(0);
	spgrnum="36";
      }else if(!ppmam21.isEmpty() && (pbravais==Data::CBaseCentered)){
	commonorigin=ppmam21.O.fromColumn(0);
	 spgrnum="36:ba-c";
      }else if(!ppmama20.isEmpty() && ((pbravais==Data::ABaseCentered) ||
				       (pbravais==Data::BodyCentered))){
	commonorigin=ppmama20.O.fromColumn(0);
	if(pbravais==Data::BodyCentered){
	  if(!ops[indmp].pinfo.pnonsymmorphic &&!ops[indms].pinfo.pnonsymmorphic ) 
	    spgrnum="44";
	  else if(!ops[indmp].pinfo.pnonsymmorphic ) spgrnum="46";
	  else if(!ops[indms].pinfo.pnonsymmorphic ) spgrnum="46:ba-c";
	  else spgrnum="45";
	  
	}else if(pbravais==Data::ABaseCentered){
	  if(!ops[indmp].pinfo.pnonsymmorphic &&!ops[indms].pinfo.pnonsymmorphic ) 
	    spgrnum="38";
	  else if(!ops[indms].pinfo.pnonsymmorphic ) spgrnum="39";
	}
      }else if(!ppmama2.isEmpty()){
	commonorigin=ppmama2.O.fromColumn(0);
	if(pbravais==Data::Simple){
	  if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic ) 
	    spgrnum="25";
	  else if(!ops[indmp].pinfo.pnonsymmorphic ) spgrnum="26";
	  else if(!ops[indms].pinfo.pnonsymmorphic ) spgrnum="26:ba-c";
	  else spgrnum="27";
	}else if(pbravais==Data::CBaseCentered){
	  spgrnum="37";
	  afteropind=indmp;
	  aftershiftcond=FMatrix::vector3(1,1,1,2);
	  aftershift=FMatrix::vector3(1,0,0,2);
	}
      }else if(!pp1ma2.isEmpty()){
	commonorigin=pp1ma2.O.fromColumn(0);
	if(!ops[indc2t].pinfo.pnonsymmorphic ){
	  if(!ops[indmp].pinfo.pnonsymmorphic ) {
	    if(pbravais==Data::Simple) spgrnum="28";
	    else if(pbravais==Data::ABaseCentered) spgrnum="40";
	  }
	  else{
	    if(pbravais==Data::Simple) spgrnum="30:ba-c";
	    else if(pbravais==Data::ABaseCentered) spgrnum="41";
	  }
	}else{
	  if(!ops[indmp].pinfo.pnonsymmorphic ){
	    spgrnum="31";
	    commonorigin-=delta100;
	  }else spgrnum="29";
	}
      }else if(!ppma12.isEmpty()){
	commonorigin=ppma12.O.fromColumn(0);
	
	if(!ops[indc2t].pinfo.pnonsymmorphic ){
	  if(!ops[indms].pinfo.pnonsymmorphic )spgrnum="28:ba-c";
	  else spgrnum="30";
	}else{
	  if(!ops[indms].pinfo.pnonsymmorphic ){
	    spgrnum="31:ba-c";
	    commonorigin-=delta010;
	  }else spgrnum="29:ba-c";
	}
      }else if(!pp20z.isEmpty() && (pbravais==Data::FaceCentered)){
	commonorigin=pp20z.O.fromColumn(0);
	spgrnum="43";
	afteropind=indmp;
	aftershiftcond=FMatrix::vector3(3,3,3,4);
	aftershift=FMatrix::vector3(1,1,1,4);
      }else if(!pp2z.isEmpty()){
	commonorigin=pp2z.O.fromColumn(0);
	if(ops[indc2t].pinfo.pnonsymmorphic ){
	  if(mpsignature.contains("b")) spgrnum="33:ba-c";
	  else spgrnum="33";
	}else if(mpsignature.contains("b")) spgrnum="32";
	else spgrnum="34";
      }
    }else if(pointgroup=="mmm"){
      PeriodicHyperPlane ppI=ops[indI].pinfo.toPHPlane(-1);
      //myDebug()<<"PHP -1="<<ppI.printS(Bcenter);
      PeriodicHyperPlane ppmamamaI(ppI);
      ppmamamaI.commonPoints(ppmap);
      ppmamamaI.commonPoints(ppmas);
      ppmamamaI.commonPoints(ppmat);
      //myDebug()<<"PHP -1 on any-m any-m any-m="<<ppmamamaI.printS(Bcenter);
      PeriodicHyperPlane ppmama1I(ppI);
      ppmama1I.commonPoints(ppmap);
      ppmama1I.commonPoints(ppmas);
      //myDebug()<<"PHP -1 on any-m any-m .="<<ppmama1I.printS(Bcenter);
      PeriodicHyperPlane ppma1maI(ppI);
      ppma1maI.commonPoints(ppmap);
      ppma1maI.commonPoints(ppmat);
      //myDebug()<<"PHP -1 on any-m . any-m ="<<ppma1maI.printS(Bcenter);
      PeriodicHyperPlane pp1mamaI(ppI);
      pp1mamaI.commonPoints(ppmas);
      pp1mamaI.commonPoints(ppmat);
      //myDebug()<<"PHP -1 on . any-m any-m ="<<pp1mamaI.printS(Bcenter);
      PeriodicHyperPlane ppma11I(ppI);
      ppma11I.commonPoints(ppmap);
      //myDebug()<<"PHP -1 on  any-m .. ="<<ppma11I.printS(Bcenter);
      PeriodicHyperPlane pp1ma1I(ppI);
      pp1ma1I.commonPoints(ppmas);
      //myDebug()<<"PHP -1 on  . any-m . ="<<pp1ma1I.printS(Bcenter);
      PeriodicHyperPlane pp11maI(ppI);
      pp11maI.commonPoints(ppmat);
      //myDebug()<<"PHP -1 on  .. any-m ="<<pp11maI.printS(Bcenter);
      PeriodicHyperPlane pp2ma11I(ppI);
      pp2ma11I.commonPoints(pp20x);
      //myDebug()<<"PHP -1 on  20/any-m .. ="<<pp2ma11I.printS(Bcenter);
      PeriodicHyperPlane pp12ma1I(ppI);
      pp12ma1I.commonPoints(pp20y);
      //myDebug()<<"PHP -1 on  . 20/any-m . ="<<pp12ma1I.printS(Bcenter);
      PeriodicHyperPlane pp112maI(ppI);
      pp112maI.commonPoints(pp20z);
      //myDebug()<<"PHP -1 on  .. 20/any-m ="<<pp112maI.printS(Bcenter);
      
      PeriodicHyperPlane pp;
      if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp202020,delta111)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::FaceCentered){
	  spgrnum="70";
	}else spgrnum="48:2";
      }else if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp202020,delta110))
	       .isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered){
	  spgrnum="65";
	  commonorigin-=delta110;
	}else if(pbravais==Data::FaceCentered){
	  spgrnum="69";
	  commonorigin-=delta110;
	}else if(pbravais==Data::BodyCentered){
	  spgrnum="72";
	  commonorigin-=Bcenterinv*FMatrix::vector3(1,1,1,4);
	}else spgrnum="50:2";
      }else if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp202020,delta101))
	       .isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered){
	  spgrnum="68:2";
	  commonorigin-=delta110;
	}else if(pbravais==Data::BodyCentered){
	  spgrnum="72:bca";
	  commonorigin-=Bcenterinv*FMatrix::vector3(1,1,1,4);
	}else spgrnum="50:2bca";
      }else if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp202020,delta011))
	       .isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::BodyCentered){
	  spgrnum="72:cab";
	  commonorigin-=Bcenterinv*FMatrix::vector3(1,1,1,4);
	}else spgrnum="50:2cab";
      }else if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp20x,delta001))
	       .isEmpty() &&
	       ((pbravais==Data::CBaseCentered) || (pbravais==Data::BodyCentered))){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered) spgrnum="66";
	else if(pbravais==Data::BodyCentered) {
	  if(ops[indmp].pinfo.pnonsymmorphic && ops[indms].pinfo.pnonsymmorphic &&
	     ops[indmt].pinfo.pnonsymmorphic)
	    spgrnum="73";
	  else if(!ops[indms].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic )
	    spgrnum="74:cab";
	}
      }else if(!(pp=PeriodicHyperPlane(ppmamamaI).commonPoints(pp20x)).isEmpty() &&
	       ((pbravais==Data::CBaseCentered) || (pbravais==Data::BodyCentered))){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered) spgrnum="67";
	else if(pbravais==Data::BodyCentered) {
	  if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic &&
	     !ops[indmt].pinfo.pnonsymmorphic)
	    spgrnum="71";
	  else if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic )
	    spgrnum="74:a-cb";
	  else if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic )
	    spgrnum="74";
	}
      }else if(!ppmamamaI.isEmpty() && (pbravais==Data::Simple)){
	commonorigin=ppmamamaI.O.fromColumn(0);
	if(pbravais==Data::Simple){
	  if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic &&
	     !ops[indmt].pinfo.pnonsymmorphic) spgrnum="47";
	  else if(!ops[indmp].pinfo.pnonsymmorphic ) spgrnum="49:cab";
	  else if(!ops[indms].pinfo.pnonsymmorphic ) spgrnum="49:bca";
	  else if(!ops[indmt].pinfo.pnonsymmorphic ) spgrnum="49";
	}
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp21z,delta100)
		 .commonPoints(pp21y,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="64"; // 2/m 2/c 21/e
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp21z,delta100)
		 .commonPoints(pp21x,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="64:ba-c"; // 2/m 2/c 21/e
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp21z,delta100)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmp].pinfo.pnonsymmorphic) spgrnum="53";
	else spgrnum="54:-cba";
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp21z,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indms].pinfo.pnonsymmorphic) spgrnum="53:ba-c";
	else spgrnum="54:bca";
      }else if(!(pp=PeriodicHyperPlane(ppma1maI).commonPoints(pp21y,delta100)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmp].pinfo.pnonsymmorphic) spgrnum="53:a-cb";
	else spgrnum="54:cab";
      }else if(!(pp=PeriodicHyperPlane(ppma1maI).commonPoints(pp21y,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmt].pinfo.pnonsymmorphic) spgrnum="53:bca";
	else spgrnum="54:ba-c";
      }else if(!(pp=PeriodicHyperPlane(pp1mamaI).commonPoints(pp21x,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indms].pinfo.pnonsymmorphic) spgrnum="53:cab";
	else spgrnum="54:a-cb";
      }else if(!(pp=PeriodicHyperPlane(pp1mamaI).commonPoints(pp21x,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmt].pinfo.pnonsymmorphic) spgrnum="53:-cba";
	else spgrnum="54";
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp20x,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered){
	  if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic ){
	    spgrnum="63";
	    commonorigin-=delta110;
	  }
	}else spgrnum="52:cab";
      }else if(!(pp=PeriodicHyperPlane(ppmama1I).commonPoints(pp20y,delta100)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(pbravais==Data::CBaseCentered){
	  if(!ops[indms].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic ){
	    spgrnum="63:ba-c";
	    commonorigin-=delta110;
	  }
	}else spgrnum="52:a-cb";
      }else if(!(pp=PeriodicHyperPlane(ppma1maI).commonPoints(pp20x,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="52:-cba";
      }else if(!(pp=PeriodicHyperPlane(ppma1maI).commonPoints(pp20z,delta100)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="52";
      }else if(!(pp=PeriodicHyperPlane(pp1mamaI).commonPoints(pp20y,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="52:bca";
      }else if(!(pp=PeriodicHyperPlane(pp1mamaI).commonPoints(pp20z,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="52:ba-c";
      }else if(!ppmama1I.isEmpty()){
	commonorigin=ppmama1I.O.fromColumn(0);
	if(pbravais==Data::Simple){
	  if(!ops[indms].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic)
	    spgrnum="51:-cba";
	  else if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic)
	    spgrnum="51:bca";
	}
      }else if(!ppma1maI.isEmpty()){
	commonorigin=ppma1maI.O.fromColumn(0);
	if(pbravais==Data::Simple){
	  if(!ops[indms].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic)
	    spgrnum="51:cab";
	  else if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic)
	    spgrnum="51:ba-c";
	}
      }else if(!pp1mamaI.isEmpty()){
	commonorigin=pp1mamaI.O.fromColumn(0);
	if(pbravais==Data::Simple){
	  if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic) 
	    spgrnum="51:a-cb";
	  else if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic) 
	    spgrnum="51";
	}
      }else if(!(pp=PeriodicHyperPlane(pp112maI).commonPoints(pp21x,delta011)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="58";
      }else if(!(pp=PeriodicHyperPlane(pp112maI).commonPoints(pp21x,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="55";
      }else if(!(pp=PeriodicHyperPlane(pp12ma1I).commonPoints(pp21x,delta011)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="58:bca";
      }else if(!(pp=PeriodicHyperPlane(pp12ma1I).commonPoints(pp21x,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="55:bca";
      }else if(!(pp=PeriodicHyperPlane(pp2ma11I).commonPoints(pp21y,delta101)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="58:cab";
      }else if(!(pp=PeriodicHyperPlane(pp2ma11I).commonPoints(pp21y,delta001)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="55:cab";
      }else if(!(pp=PeriodicHyperPlane(pp1ma1I).commonPoints(pp21z,delta110)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60";
      }else if(!(pp=PeriodicHyperPlane(pp11maI)).commonPoints(pp21x,delta011).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60:cab";
      }else if(!(pp=PeriodicHyperPlane(pp1ma1I).commonPoints(pp21x,delta011)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60:-cba";
      }else if(!(pp=PeriodicHyperPlane(ppma11I).commonPoints(pp21y,delta101)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60:bca";
      }else if(!(pp=PeriodicHyperPlane(pp11maI).commonPoints(pp21y,delta101)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60:a-cb";
      }else if(!(pp=PeriodicHyperPlane(ppma11I).commonPoints(pp21z,delta110)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="60:ba-c";
      }else if(!(pp=PeriodicHyperPlane(ppma11I)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indms].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic)
	  spgrnum="59:2cab";
	else if(!ops[indms].pinfo.pnonsymmorphic)
	  spgrnum="57:a-cb";
	else if(!ops[indmt].pinfo.pnonsymmorphic)
	  spgrnum="57";
	else spgrnum="56:cab";
      }else if(!(pp=PeriodicHyperPlane(pp1ma1I)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indmt].pinfo.pnonsymmorphic)
	  spgrnum="59:2bca";
	else if(!ops[indmp].pinfo.pnonsymmorphic)
	  spgrnum="57:cab";
	else if(!ops[indmt].pinfo.pnonsymmorphic)
	  spgrnum="57:ba-c";
	else spgrnum="56:bca";
      }else if(!(pp=PeriodicHyperPlane(pp11maI)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmp].pinfo.pnonsymmorphic && !ops[indms].pinfo.pnonsymmorphic)
	  spgrnum="59:2";
	else if(!ops[indmp].pinfo.pnonsymmorphic) spgrnum="57:-cba";
	else if(!ops[indms].pinfo.pnonsymmorphic) spgrnum="57:bca";
	else spgrnum="56";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21x,delta011)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indms].pinfo.pnonsymmorphic) spgrnum="62";
	else if(!ops[indmt].pinfo.pnonsymmorphic) spgrnum="62:a-cb";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21y,delta101)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indmt].pinfo.pnonsymmorphic) spgrnum="62:cab";
	else if(!ops[indmp].pinfo.pnonsymmorphic) spgrnum="62:ba-c";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21z,delta110)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	if(!ops[indms].pinfo.pnonsymmorphic) spgrnum="62:-cba";
	else if(!ops[indmp].pinfo.pnonsymmorphic) spgrnum="62:bca";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21z,delta100)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="61";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21z,delta010)).isEmpty()){
	commonorigin=pp.O.fromColumn(0);
	spgrnum="61:ba-c";
      }
    }
    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);

	  
    
    if(afteropind>=0 && (ops[afteropind].pw-aftershiftcond).isInteger()){
      changeOrigin(ops,aftershift,true);
      Stransform*=SeitzMatrix(aftershift);
    }
    

    if(spgrnum=="0")
      throw Except("SeitzMatrix: non standard setting for orthorhombic group?");

    //orthorhombic systems
  }else if(orthoC2.size()+orthom.size()>0){ //monoclinic
    pcrystalsystem=Data::Monoclinic;
    pbravais=Data::Simple;

    //myDebug()<<"monoclinic"<<orthoC2;
    //myDebug()<<"monoclinic"<<orthom;
    long indC2=-1;
    long indm=-1;
    const SeitzMatrixInfo *pinfo=0;
    if(orthoC2.size()==1 && orthom.size()==1){
      pointgroup="2/m";
      indC2=orthoC2.at(0);
      indm=orthom.at(0);
      if(ops[indC2].pinfo.pp==2){
	pbravais=Data::CBaseCentered;
      }
      pinfo=&(ops[indC2].pinfo);
    }else if(orthoC2.size()==1){
      pointgroup="2";
      indC2=orthoC2.at(0);
      if(ops[indC2].pinfo.pp==2){
	pbravais=Data::CBaseCentered;
      }
      pinfo=&(ops[indC2].pinfo);
    }else if(orthom.size()==1){
      pointgroup="m";
      indm=orthom.at(0);
      if(ops[indm].pinfo.pp==2){
	pbravais=Data::CBaseCentered;
      }
      pinfo=&(ops[indm].pinfo);
    }
    //myDebug()<<pinfo->symbol();
    
    UTbasis.setDimenions(3,0);
    {// determin lattice vectors
      if(pbravais==Data::Simple){
	UTbasis.addMatrix(pinfo->RA.fromColumn(0));
	UTbasis.addMatrix(pinfo->Pmin);
	UTbasis.addMatrix(pinfo->RA.fromColumn(1));
      }else{
	Bcenter=Data::bravaisMatrix(pbravais);
	Bcenterinv=Bcenter.inverse();
	IMatrix R1=pinfo->RA.fromColumn(0);
	IMatrix R2=pinfo->RA.fromColumn(1);
	IMatrix a=(pinfo->RA.fromColumn(2)*2-pinfo->Pmin).shorten();
	IMatrix Rc(3,1);
	// myDebug()<<"RA=\n"<<pinfo->RA.printS();
	// myDebug()<<"detRA=\n"<<pinfo->RA.determinant();
	// myDebug()<<"Pmin=\n"<<pinfo->Pmin;
	// myDebug()<<"Gmin=\n"<<pinfo->Gmin;
	// myDebug()<<"pp=\n"<<pinfo->pp;
	// myDebug()<<"Gmin*Pmin=\n"<<pinfo->Gmin.dot(pinfo->Pmin);
	// myDebug()<<"Gmin*R3=\n"<<pinfo->Gmin.dot(pinfo->RA.fromColumn(2));
	// myDebug()<<"Gmin*a=\n"<<pinfo->Gmin.dot(a);

	// myDebug()<<"|R1R2Pmin|=\n"<<IMatrix(R1,R2,pinfo->Pmin).determinant();
	// myDebug()<<"|aR2Pmin|=\n"<<IMatrix(a,R2,pinfo->Pmin).determinant();
	// myDebug()<<"|R1aPmin|=\n"<<IMatrix(R1,a,pinfo->Pmin).determinant();
	// myDebug()<<"Pmin="<<pinfo->GA.Td()*pinfo->Pmin;

	//myDebug()<<"R1=\n"<<R1.printS();
	//myDebug()<<"R2=\n"<<R2.printS();
	//myDebug()<<"a=\n"<<a.printS();
	{
	  long k=1,l=0,fk=0,fl=0,stage=0; 
	  for(long c=0;c<3;c++){
	    //myDebug()<<"\nc="<<c<<"stage="<<stage;
	    //myDebug()<<"R1[c]="<<R1[c]<<"R2[c]="<<R2[c]<<"a[c]="<<a[c];
	    if(R1[c]==0 && R2[c]==0){
	      if(a[c]!=0)throw Except("Oops not solvable!");
	    }else if(stage==0){
	      long D=MyMath::gcd(R1[c],R2[c]);
	      //myDebug()<<"D="<<D;
	      if( (a[c] % D) !=0)throw Except("Oops not solvable!");
	      long s=a[c]/D;
	      ////myDebug()<<"s="<<s;
	      fk=(R2[c]/D);
	      fl=-(R1[c]/D);
	      //myDebug()<<"fk="<<fk;
	      //myDebug()<<"fl="<<fl;
	      if(fk==0){
		k=-s/fl;
		l=0;
	      }else{
		k=0;
 		while((s+fl*k)%fk!=0) k+=s;
		l=(s+fl*k)/fk;
	      }
	      //myDebug()<<"k="<<k<<"+"<<fk<<"*L";
	      //myDebug()<<"l="<<l<<"+"<<fl<<"*L";
	      //myDebug()<<"R1*k+R2*l-a="<<R1[c]*k+R2[c]*l-a[c];
	      stage=1;
	    }else if(stage==1){
	      long s=a[c]-R1[c]*k-R2[c]*l;
	      long D=R1[c]*fk+R2[c]*fl;
	      //myDebug()<<"D="<<D;
	      //myDebug()<<"s="<<s;
	      if(D==0){
		if( s !=0)throw Except("Oops not solvable!");
		stage=1;
	      }else{
		if( (s % D) !=0)throw Except("Oops not solvable!");
		long L=s/D;
		//myDebug()<<"L="<<L;
		k+=fk*L;
		l+=fl*L;
		stage=2;
	      }
	      //myDebug()<<"k="<<k;
	      //myDebug()<<"l="<<l;
	    }else if(stage==2){
	      if(R1[c]*k+R2[c]*l!=a[c]) throw Except("Oops not solvable!");
	    }
	  }
	  //myDebug()<<R1*k+R2*l-a;
	  long D=MyMath::gcd(k,l);
	  //myDebug()<<"D(k,l)="<<D;
	  if(labs(D)!=1) throw Except("Oops |D|!=1!");
	  if(R1*k+R2*l!=a) throw Except("Oops a is wrong!");
	  
	  //myDebug()<<"k,l="<<k<<l;
	  long s=0,r=0;
	  if(k==0) r=-1/l;
	  else{
	    r=0;while((1+l*r)%k!=0) r++;
	    s=(1+l*r)/k;
	  }
	  if(k*s-l*r!=1)throw Except("Oops det!=1!");
	  Rc=R1*r+R2*s;
	}
	
	UTbasis.addMatrix(a);
	UTbasis.addMatrix(pinfo->Pmin);
	UTbasis.addMatrix(Rc);
      }
    }

    Fractional detUT=UTbasis.determinant();
    //myDebug()<<"det UTbasis=\n"<<detUT;
    if(detUT<0){
      detUT*=-1;
      UTbasis.scaleColumn(0,-1);
    }
    //myDebug()<<"UTbasis conv=\n"<<UTbasis.printS();
    //myDebug()<<"det=\n"<<UTbasis.determinant();
    UTbasis=UTbasis*Bcenter;

    //myDebug()<<"BCenter=\n"<<Bcenter;
    //myDebug()<<"BCenterinv=\n"<<Bcenter.inverse();
    //myDebug()<<"UTbasis prim=\n"<<UTbasis.printS();
    //myDebug()<<"det=\n"<<UTbasis.determinant();

    changeBasis(ops,UTbasis,true);
    Stransform*=UTbasis;

    if(prot){
      for(long i=0;i<ops.size();i++){
	if(!ops[i].pactive)continue;
	printf("DET LATTICE ANALYSE %ld\n",i);
	ops[i].analyse(true);
      }
    }
    if(pointgroup=="2"){
      PeriodicHyperPlane ppc20=ops[indC2].pinfo.toPHPlane(0);
      PeriodicHyperPlane ppc21=ops[indC2].pinfo.toPHPlane(1);
      if(!ppc20.isEmpty()){
	commonorigin=ppc20.O.fromColumn(0);
	if(pbravais==Data::Simple) spgrnum="3:b";
	else  spgrnum="5:b1";
      }else if(!ppc21.isEmpty()){
	commonorigin=ppc21.O.fromColumn(0);
	spgrnum="4:b";
      }
    }else if(pointgroup=="m"){
      const SeitzMatrixInfo &pinfo=ops[indm].pinfo;
      String mpsignature=pinfo.mirrorSignatures(Bcenter);
      PeriodicHyperPlane ppma=pinfo.toPHPlane(-1);
      PeriodicHyperPlane ppm=pinfo.toPHPlane(0);
      PeriodicHyperPlane ppmc=pinfo.toPHPlane(3);
      if(pbravais==Data::Simple){
	if(!ppma.isEmpty()){
	  commonorigin=ppma.O.fromColumn(0);
	  if(!pinfo.pnonsymmorphic) spgrnum="6:b";
	  else {
	    if(mpsignature.contains("c")) spgrnum="7:b1";
	    else if(mpsignature.contains("a")) spgrnum="7:b3";
	    else spgrnum="7:b2";
	  }
	}
      }else{
	if(!ppm.isEmpty()){
	  commonorigin=ppm.O.fromColumn(0);
	  spgrnum="8:b1";
	}else if(!ppmc.isEmpty()){
	  commonorigin=ppmc.O.fromColumn(0);
	  spgrnum="9:b1";
	}
      }
      
    }else if(pointgroup=="2/m"){
      const SeitzMatrixInfo &pinfo2=ops[indC2].pinfo;
      const SeitzMatrixInfo &pinfom=ops[indm].pinfo;
      String mpsignature=pinfom.mirrorSignatures(Bcenter);
      PeriodicHyperPlane ppI=ops[indI].pinfo.toPHPlane(-1);
      PeriodicHyperPlane pp20=pinfo2.toPHPlane(0);
      PeriodicHyperPlane pp21=pinfo2.toPHPlane(1);
      PeriodicHyperPlane ppma=pinfom.toPHPlane(-1);
      PeriodicHyperPlane ppm=pinfom.toPHPlane(0);
      PeriodicHyperPlane ppa=pinfom.toPHPlane(1);
      PeriodicHyperPlane ppc=pinfom.toPHPlane(3);
      PeriodicHyperPlane pp;
      if(!(pp=PeriodicHyperPlane(pp20).commonPoints(ppm)).isEmpty()){
	if(pbravais==Data::Simple) spgrnum="10";
	else spgrnum="12:b1";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(pp21)).isEmpty()){
	spgrnum="11:b";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(ppc)).isEmpty()){
	if(pbravais==Data::Simple) spgrnum="13:b1";
	else spgrnum="15:b1";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(ppa)).isEmpty()){
	spgrnum="13:b3";
      }else if(!(pp=PeriodicHyperPlane(ppI).commonPoints(ppma)).isEmpty()){
	spgrnum="13:b2";
      }else if(!(pp=ppI).isEmpty()){
	if(mpsignature.contains("c")) spgrnum="14:b1";
	else if(mpsignature.contains("a")) spgrnum="14:b3";
	else  spgrnum="14:b2";
      }
      commonorigin=pp.O.fromColumn(0);
    }


    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);

    // printf("Crystalsystem: %s\n",Data::crystalSystemName[pcrystalsystem]);
    // printf("BravaisLattice: %s\n",Data::bravaisLaticeName[pbravais]);
    // printf("Pointgroup   : %s\n",pointgroup());
    // printf("Spacegroupnum: %s\n",spgrnum());
    // exit(1);
  }else{//triclinic
    pcrystalsystem=Data::Triclinic;
    pbravais=Data::Simple;
    if(indI>=0){
      pointgroup="-1";
      PeriodicHyperPlane ppI(ops[indI].pinfo.toPHPlane(-1));
      if(!ppI.isEmpty()){
	commonorigin=ppI.O.fromColumn(0);
	spgrnum="2";
      }
    }else{
      pointgroup="1";
      spgrnum="1";
    }

    changeOrigin(ops,commonorigin,true);
    Stransform*=SeitzMatrix(commonorigin);
  }//end group analysis///////////////////////////////////////////////////////

  ValueArray<SeitzMatrix> tmp(ops);
  ops.clear();
  for(long i=0;i<tmp.size();i++){
    if(tmp.at(i).pactive){
      ops<<tmp.at(i);
    }
  }
  

  //myDebug()<<"Bcenter\n="<<Bcenter;

  SeitzMatrix settingsop
    =changeSetting(ops,pbravais,pcrystalsystem,
		   spgrnum,Bcenter,Bcenterinv,requiredsetting,true); 
  //myDebug()<<"settingsop="<<settingsop;

  Stransform*=settingsop;



  if(prot){
    for(long i=0;i<ops.size();i++){
      if(!ops[i].pactive)continue;
      printf("ANALYSE CHANGE SETTINGS %ld\n",i);
      ops[i].analyse(true);
    }
  }


  /* ****************************************************
   * END OF GROUP CALCULATION
   * ****************************************************/


  //printf("Crystalsystem: %s\n",Data::crystalSystemName[pcrystalsystem]);
  // printf("BravaisLattice: %s\n",Data::bravaisLaticeName[pbravais]);
  // exit(1);
  if(pbravais!=Data::Simple){
    //myDebug()<<"here switch to conventional";

    // we are in the primitive basis here and want to switch to
    // the conventional: Ap^T=Ac^T*B^T
    changeBasis(ops,Bcenterinv,true);
    Stransform*=Bcenterinv;

    FMatrix trans;
    if(pbravais==Data::BodyCentered){
      trans=FMatrix::vector3(1,1,1,2);
    }else if(pbravais==Data::FaceCentered){
      trans=Bcenter;
    }else if(pbravais==Data::ABaseCentered){
      trans=FMatrix::vector3(0,1,1,2);
    }else if(pbravais==Data::CBaseCentered){
      trans=FMatrix::vector3(1,1,0,2);
    }else if(pbravais==Data::BBaseCentered){
      trans=FMatrix::vector3(1,0,1,2);
    }
    
    //todo: dont change ops here
    long N=ops.size();
    for(long k=0;k<trans.columns();k++){
      for(long i=0;i<N;i++){
	if(!ops[i].pactive)continue;
	ops.append(SeitzMatrix(ops.at(i)));
	ops.last()+=trans.fromColumn(k);
	ops.last().regularize();
      }
    }
    for(long i=0;i<ops.size();i++){
      if(!ops[i].pactive)continue;
      if(prot) printf("ANALYSE PRIMITIVE->BRAVAIS %ld\n",i);
      ops[i].analyse(prot);
    }
  }

  printf("Crystalsystem: %s\n",Data::crystalSystemName(pcrystalsystem));
  printf("BravaisLattice: %s\n",Data::bravaisLatticeName(pbravais));
  printf("Pointgroup   : %s\n",pointgroup());
  printf("Spacegroupnum: %s\n",spgrnum());

  //todo: collect all the transformations such that we can
  // apply them to the lattice parameters and wyckoffs
  // when applying it to lattice parameters, we need to keep
  // the euclidian rotation for convenience
  
  
  
  
  if(1){//testing the resulting transformation
    //myDebug()<<"compare against input "<<Stransform<<Stransform.pinteger;
    ValueArray<SeitzMatrix> tmp;
    for(long i=0;i<opsin.size();i++){
      tmp<<SeitzMatrix(opsin.at(i)).apply(Stransform);
      if(!tmp.last().isInteger()) throw Except("Transformed input op non-integer\n");
    }
    
    //myDebug()<<"here ";
    for(long i=0;i<ops.size();i++){
      SeitzMatrix op(ops.at(i));
      bool found=false;
      for(long j=0;j<tmp.size();j++){
	if(tmp.at(j)==op){
	  found=true;
	  break;
	}
      }
      if(!found) {
	throw Except("Cannot find analysed op in transformed original set\n"
		     +op.printWwS()+op.pinfo.symbol());
      }
    }
  }

  String hs=Data::HallFromIT(spgrnum);
  hallsymbolout.fromSymbol(hs);
  if(1){//testing the resulting group
    const ValueArray<SeitzMatrix> &hsops=hallsymbolout.operations();
    //myDebug()<<"compare against "<<hs;
    
    for(long i=0;i<ops.size();i++){
      SeitzMatrix op(ops.at(i));
      bool found=false;
      for(long j=0;j<hsops.size();j++){
	if(hsops.at(j)==op){
	  found=true;
	  break;
	}
      }
      if(!found) {
	throw Except("Cannot find analysed op in set created from calculated group information\n"
		     +op.printWwS()+op.pinfo.symbol());
      }
    }
  }
  
  if(1){// test other stuff
    if(hallsymbolout.crystalSystem()!=pcrystalsystem)
      throw Except("internal error with crystalSystem!");
    if(hallsymbolout.bravaisLattice()!=pbravais)
      throw Except("internal error with bravaislattice!");
  }

  //myDebug()<<"Stransform"<<Stransform;

  if(0){
    for(long i=0;i<ops.size();i++){
      SeitzMatrix &op=ops[i];
      op.analyse();
      printf("OPER%4ldIT: %-30s: %-30s\n\n",i,op.toXYZSymbol()(),op.symbol()());
    }
  }

  if(0){// do some adjustments
    const ValueArray<SeitzMatrix> &hsops=hallsymbolout.operations(true);
    long N=hsops.size();
    //long N=hallsymbolout.numberOfNonTranslationalOps();
    myDebug()<<"N="<<N;
    // the ops include the centering translations and hence refer to 
    // the conventional lattice
    IMatrix mtable(N,N);
    mtable=-1;
    //    long it=0;

    myDebug()<<"start table";
    for(long it=0;it<N;it++){
      for(long jt=0;jt<N;jt++){
	SeitzMatrix op=hsops.at(it)*hsops.at(jt);
	long i=0; while(i<N){if(hsops.at(i)==op)break; i++; }
	if(i<N) mtable(it,jt)=i;
	else{
	  myDebug()<<"cannot find"<<hsops.at(it).symbol()<<"*"<<hsops.at(jt).symbol()<<"="<<op;
	}
      }
    }

    {// print
      myDebug()<<"mtable=\n";
      printf("                ");
      for(long it=0;it<N;it++){
	const SeitzMatrixInfo &pinfo=hsops.at(it).pinfo;
	printf("%-10s ",pinfo.symbol(true)());
      }
      printf("\n");
      for(long it=0;it<N;it++){
	const SeitzMatrixInfo &pinfo=hsops.at(it).pinfo;
	printf("%-10s",pinfo.symbol(true)());
	for(long jt=0;jt<N;jt++){
	  printf("%10ld ",mtable(it,jt));
	}
	printf("\n");
      }
    }
    
  }
  
  //opsin=ops;
  
}

/*EOF*/ 
