/*************************************************************************
 * @(#)SpaceGroupData.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Aug 2012
 *************************************************************************
 */

#include "FString.hh"
#include "Except.hh"
#include "SpaceGroupData.hh"
#include "MyDebug.hh"
#include "FMatrix.hh"


using namespace SpaceGroup;


const char *Data::pcrystalsystemname[]={
  "Unknown",
  "Triclinic",
  "Monoclinic",
  "Orthorhombic",
  "Tetragonal",
  "Trigonal",
  "Rhombohedral",
  "Hexagonal",
  "Cubic"
};
const char *Data::pbravaislatticename[]={
  "Unknow",
  "Simple",
  "Body Centered",
  "Face Centered",
  "C-Base Centered",
  "A-Base Centered",
  "B-Base Centered",
  "Rhombohedral"
};

Data::tsITHMHall Data::ITHMHall[]={
  {"L1",true,{0,0,0},"p 1","p 1"},//triclinic
  {"L2",true,{0,0,0},"p -1","-p 1"},
  {"L3",true,{0,0,0},"p 1 1 2","p 2"},//monoclinic inclined
  {"L4",true,{0,0,0},"p 1 1 m","p -2"},
  {"L5",true,{0,0,0},"p 1 1 a","p -2a"},
  {"L6",true,{0,0,0},"p 1 1 2/m","-p 2"},
  {"L7",true,{0,0,0},"p 1 1 2/a","-p 2a"},
  {"L8:a",true,{0,0,0},"p 2 1 1","p 2x"},//monoclinic orthogonal
  {"L8:b",false,{0,0,0},"p 1 2 1","p 2y"},
  {"L9:a",true,{0,0,0},"p 21 1 1","p 2xa"},
  {"L9:b",false,{0,0,0},"p 1 21 1","p 2yb"},
  {"L10:a",true,{0,0,0},"c 2 1 1","c 2x"},
  {"L10:b",false,{0,0,0},"c 1 2 1","c 2y"},
  {"L11:a",true,{0,0,0},"p m 1 1","p -2x"},
  {"L11:b",false,{0,0,0},"p 1 m 1","p -2y"},
  {"L12:a",true,{0,0,0},"p b 1 1","p -2xb"},
  {"L12:b",false,{0,0,0},"p 1 a 1","p -2ya"},
  {"L13:a",true,{0,0,0},"c m 1 1","c -2x"},
  {"L13:b",false,{0,0,0},"c 1 m 1","c -2y"},
  {"L14:a",true,{0,0,0},"p 2/m 1 1","-p 2x"},
  {"L14:b",false,{0,0,0},"p 1 2/m 1","-p 2y"},
  {"L15:a",true,{0,0,0},"p 21/m 1 1","-p 2xa"},
  {"L15:b",false,{0,0,0},"p 1 21/m 1","-p 2yb"},
  {"L16:a",true,{0,0,0},"p 2/b 1 1","-p 2xb"},
  {"L16:b",false,{0,0,0},"p 1 2/a 1","-p 2ya"},
  {"L17:a",true,{0,0,0},"p 21/b 1 1","-p 2xab"},
  {"L17:b",false,{0,0,0},"p 1 21/a 1","-p 2yab"},
  {"L18:a",true,{0,0,0},"c 2/m 1 1","-c 2x"},
  {"L18:b",true,{0,0,0},"c 1 2/m 1","-c 2y"},

  {"L19",true,{0,0,0},"p 2 2 2","p 2 2y"},//orthorhombic
  {"L20",true,{0,0,0},"p 21 2 2","p 2 2xa"},
  {"L20:ba-c",false,{0,0,0},"p 2 21 2","p 2 2yb"},
  {"L21",true,{0,0,0},"p 21 21 2","p 2 2xab"},
  {"L22",true,{0,0,0},"c 2 2 2","c 2 2y"},
  {"L23",true,{0,0,0},"p m m 2","p 2 -2y"},
  {"L24",true,{0,0,0},"p m a 2","p 2 -2ya"},
  {"L24:ba-c",false,{0,0,0},"p b m 2","p 2 -2xb"},
  {"L25",true,{0,0,0},"p b a 2","p 2 -2yab"},
  {"L26",true,{0,0,0},"c m m 2","c 2 -2y"},
  {"L27",true,{0,0,0},"p m 2 m","p -2 2y"},
  {"L27:ba-c",false,{0,0,0},"p 2 m m","p -2 2x"},
  {"L28",true,{0,0,0},"p m 21 b","p -2b 2yb"},
  {"L28:ba-c",false,{0,0,0},"p 21 m a","p -2a 2xa"},
  {"L29",true,{0,0,0},"p b 21 m","p -2 2yb"},
  {"L29:ba-c",false,{0,0,0},"p 21 b m","p -2 2xa"},
  {"L30",true,{0,0,0},"p b 2 b","p -2b 2y"},
  {"L30:ba-c",false,{0,0,0},"p 2 a a","p -2a 2x"},
  {"L31",true,{0,0,0},"p m 2 a","p -2a 2y"},
  {"L31:ba-c",false,{0,0,0},"p 2 m b","p -2b 2x"},
  {"L32",true,{0,0,0},"p m 21 n","p -2ab 2yab"},
  {"L32:ba-c",false,{0,0,0},"p 21 m n","p -2ab 2xab"},
  {"L33",true,{0,0,0},"p b 21 a","p -2a 2yb"},
  {"L33:ba-c",false,{0,0,0},"p 21 a b","p -2b 2xa"},
  {"L34",true,{0,0,0},"p b 2 n","p -2ab 2y"},
  {"L34:ba-c",false,{0,0,0},"p 2 a n","p -2ab 2x"},
  {"L35",true,{0,0,0},"c m 2 m","c -2 2y"},
  {"L35:ba-c",false,{0,0,0},"c 2 m m","c -2 2x"},
  {"L36",true,{0,0,0},"c m 2 e","c -2a 2y"},
  {"L36:ba-c",false,{0,0,0},"c 2 m e","c -2a 2x"},
  {"L37",true,{0,0,0},"p m m m","-p -2 -2y"},
  {"L38",true,{0,0,0},"p m a a","-p -2a -2x"},
  {"L38:ba-c",false,{0,0,0},"p b m b","-p -2b -2y"},
  {"L39",true,{0,0,0},"p b a n","-p -2ab -2ya"},
  {"L40",true,{0,0,0},"p m a m","-p -2 -2ya"},
  {"L40:ba-c",false,{0,0,0},"p b m m","-p -2 -2xb"},
  {"L41",true,{0,0,0},"p m m a","-p -2a -2y"},
  {"L41:ba-c",false,{0,0,0},"p m m b","-p -2b -2x"},
  {"L42",true,{0,0,0},"p m a n","-p -2ab -2yab"},
  {"L42:ba-c",false,{0,0,0},"p b m n","-p -2ab -2xab"},
  {"L43",true,{0,0,0},"p b a a","-p -2a -2yab"},
  {"L43:ba-c",false,{0,0,0},"p b a b","-p -2b -2xab"},
  {"L44",true,{0,0,0},"p b a m","-p -2 -2yab"},
  {"L45",true,{0,0,0},"p b m a","-p -2a -2yb"},
  {"L45:ba-c",false,{0,0,0},"p m a b","-p -2b -2xa"},
  {"L46",true,{0,0,0},"p m m n","-p -2ab -2yb"},
  {"L47",true,{0,0,0},"c m m m","-c -2 -2y"},
  {"L48",true,{0,0,0},"c m m e","-c -2b -2yb"},

  {"L49",true,{0,0,0},"p 4","p 4"},//tetragonal
  {"L50",true,{0,0,0},"p -4","p -4"},
  {"L51",true,{0,0,0},"p 4/m","-p 4"},
  {"L52:1",false,{0,0,0},"p 4/n:1","p 4 -1ab"},
  {"L52:2",true,{0,0,0},"p 4/n:2","-p 4a "},
  {"L53",true,{0,0,0},"p 4 2 2","p 4 2y "},
  {"L54",true,{0,0,0},"p 4 21 2","p 4 2yab "},
  {"L55",true,{0,0,0},"p 4 m m","p 4 -2y"},
  {"L56",true,{0,0,0},"p 4 b m","p 4 -2yab"},
  {"L57",true,{0,0,0},"p -4 2 m","p -4 2"},
  {"L58",true,{0,0,0},"p -4 21 m","p -4 2ab"},
  {"L59",true,{0,0,0},"p -4 m 2","p -4 -2"},
  {"L60",true,{0,0,0},"p -4 b 2","p -4 -2ab"},
  {"L61",true,{0,0,0},"p 4/m m m","-p 4 2"},
  {"L62:1",false,{0,0,0},"p 4/n b m:1","p 4 2 -1ab"},
  {"L62:2",true,{0,0,0},"p 4/n b m:2","-p 4a 2b"},
  {"L63",true,{0,0,0},"p 4/m b m","-p 4 -2ab"},
  {"L64:1",false,{0,0,0},"p 4/n m m:1","p 4 -2 -1ab"},
  {"L64:2",true,{0,0,0},"p 4/n m m:2","-p 4a -2a "},

  {"L65",true,{0,0,0},"p 3","p 3"},//trigonal
  {"L66",true,{0,0,0},"p -3","p -3"},
  {"L67",true,{0,0,0},"p 3 1 2","p 3 2"},
  {"L68",true,{0,0,0},"p 3 2 1","p 3 2\""},
  {"L69",true,{0,0,0},"p 3 m 1","p 3 -2\""},
  {"L70",true,{0,0,0},"p 3 1 m","p 3 -2"},
  {"L71",true,{0,0,0},"p -3 1 m","p -3 -2"},
  {"L72",true,{0,0,0},"p -3 m 1","p -3 -2\""},

  {"L73",true,{0,0,0},"p 6","p 6"},//hexagonal
  {"L74",true,{0,0,0},"p -6","p -6"},
  {"L75",true,{0,0,0},"p 6/m","-p 6"},
  {"L76",true,{0,0,0},"p 6 2 2","p 6 2"},
  {"L77",true,{0,0,0},"p 6 m m","p 6 -2"},
  {"L78",true,{0,0,0},"p -6 m 2","p -6 -2\""},
  {"L79",true,{0,0,0},"p -6 2 m","p -6 -2"},
  {"L80",true,{0,0,0},"p 6/m m m","-p 6 2"},



  {"1",true,{0,0,0},"P 1","P 1"},
  {"2",true,{0,0,0},"P -1","-P 1"},
  {"3:b",true,{0,0,0},"P 1 2 1","P 2y"},
  {"3:c",false,{0,0,0},"P 1 1 2","P 2"},
  {"3:a",false,{0,0,0},"P 2 1 1","P 2x"},
  {"4:b",true,{0,0,0},"P 1 21 1","P 2yb"},
  {"4:c",false,{0,0,0},"P 1 1 21","P 2c"},
  {"4:a",false,{0,0,0},"P 21 1 1","P 2xa"},
  {"5:b1",true,{0,0,0},"C 1 2 1","C 2y"},
  {"5:b2",false,{0,0,0},"A 1 2 1","A 2y"},
  {"5:b3",false,{0,0,0},"I 1 2 1","I 2y"},
  {"5:c1",false,{0,0,0},"A 1 1 2","A 2"},
  {"5:c2",false,{0,0,0},"B 1 1 2","B 2"},
  {"5:c3",false,{0,0,0},"I 1 1 2","I 2"},
  {"5:a1",false,{0,0,0},"B 2 1 1","B 2x"},
  {"5:a2",false,{0,0,0},"C 2 1 1","C 2x"},
  {"5:a3",false,{0,0,0},"I 2 1 1","I 2x"},
  {"6:b",true,{0,0,0},"P 1 m 1","P -2y"},
  {"6:c",false,{0,0,0},"P 1 1 m","P -2"},
  {"6:a",false,{0,0,0},"P m 1 1","P -2x"},
  {"7:b1",true,{0,0,0},"P 1 c 1","P -2yc"},
  {"7:b2",false,{0,0,0},"P 1 n 1","P -2yac"},
  {"7:b3",false,{0,0,0},"P 1 a 1","P -2ya"},
  {"7:c1",false,{0,0,0},"P 1 1 a","P -2a"},
  {"7:c2",false,{0,0,0},"P 1 1 n","P -2ab"},
  {"7:c3",false,{0,0,0},"P 1 1 b","P -2b"},
  {"7:a1",false,{0,0,0},"P b 1 1","P -2xb"},
  {"7:a2",false,{0,0,0},"P n 1 1","P -2xbc"},
  {"7:a3",false,{0,0,0},"P c 1 1","P -2xc"},
  {"8:b1",true,{0,0,0},"C 1 m 1","C -2y"},
  {"8:b2",false,{0,0,0},"A 1 m 1","A -2y"},
  {"8:b3",false,{0,0,0},"I 1 m 1","I -2y"},
  {"8:c1",false,{0,0,0},"A 1 1 m","A -2"},
  {"8:c2",false,{0,0,0},"B 1 1 m","B -2"},
  {"8:c3",false,{0,0,0},"I 1 1 m","I -2"},
  {"8:a1",false,{0,0,0},"B m 1 1","B -2x"},
  {"8:a2",false,{0,0,0},"C m 1 1","C -2x"},
  {"8:a3",false,{0,0,0},"I m 1 1","I -2x"},
  {"9:b1",true,{0,0,0},"C 1 c 1","C -2yc"},
  {"9:b2",false,{0,0,0},"A 1 n 1","A -2yac"},
  {"9:b3",false,{0,0,0},"I 1 a 1","I -2ya"},
  {"9:-b1",false,{0,0,0},"A 1 a 1","A -2ya"},
  {"9:-b2",false,{0,0,0},"C 1 n 1","C -2ybc"},
  {"9:-b3",false,{0,0,0},"I 1 c 1","I -2yc"},
  {"9:c1",false,{0,0,0},"A 1 1 a","A -2a"},
  {"9:c2",false,{0,0,0},"B 1 1 n","B -2bc"},
  {"9:c3",false,{0,0,0},"I 1 1 b","I -2b"},
  {"9:-c1",false,{0,0,0},"B 1 1 b","B -2b"},
  {"9:-c2",false,{0,0,0},"A 1 1 n","A -2ac"},
  {"9:-c3",false,{0,0,0},"I 1 1 a","I -2a"},
  {"9:a1",false,{0,0,0},"B b 1 1","B -2xb"},
  {"9:a2",false,{0,0,0},"C n 1 1","C -2xbc"},
  {"9:a3",false,{0,0,0},"I c 1 1","I -2xc"},
  {"9:-a1",false,{0,0,0},"C c 1 1","C -2xc"},
  {"9:-a2",false,{0,0,0},"B n 1 1","B -2xbc"},
  {"9:-a3",false,{0,0,0},"I b 1 1","I -2xb"},
  {"10:b",true,{0,0,0},"P 1 2/m 1","-P 2y"},
  {"10:c",false,{0,0,0},"P 1 1 2/m","-P 2"},
  {"10:a",false,{0,0,0},"P 2/m 1 1","-P 2x"},
  {"11:b",true,{0,0,0},"P 1 21/m 1","-P 2yb"},
  {"11:c",false,{0,0,0},"P 1 1 21/m","-P 2c"},
  {"11:a",false,{0,0,0},"P 21/m 1 1","-P 2xa"},
  {"12:b1",true,{0,0,0},"C 1 2/m 1","-C 2y"},
  {"12:b2",false,{0,0,0},"A 1 2/m 1","-A 2y"},
  {"12:b3",false,{0,0,0},"I 1 2/m 1","-I 2y"},
  {"12:c1",false,{0,0,0},"A 1 1 2/m","-A 2"},
  {"12:c2",false,{0,0,0},"B 1 1 2/m","-B 2"},
  {"12:c3",false,{0,0,0},"I 1 1 2/m","-I 2"},
  {"12:a1",false,{0,0,0},"B 2/m 1 1","-B 2x"},
  {"12:a2",false,{0,0,0},"C 2/m 1 1","-C 2x"},
  {"12:a3",false,{0,0,0},"I 2/m 1 1","-I 2x"},
  {"13:b1",true,{0,0,0},"P 1 2/c 1","-P 2yc"},
  {"13:b2",false,{0,0,0},"P 1 2/n 1","-P 2yac"},
  {"13:b3",false,{0,0,0},"P 1 2/a 1","-P 2ya"},
  {"13:c1",false,{0,0,0},"P 1 1 2/a","-P 2a"},
  {"13:c2",false,{0,0,0},"P 1 1 2/n","-P 2ab"},
  {"13:c3",false,{0,0,0},"P 1 1 2/b","-P 2b"},
  {"13:a1",false,{0,0,0},"P 2/b 1 1","-P 2xb"},
  {"13:a2",false,{0,0,0},"P 2/n 1 1","-P 2xbc"},
  {"13:a3",false,{0,0,0},"P 2/c 1 1","-P 2xc"},
  {"14:b1",true,{0,0,0},"P 1 21/c 1","-P 2ybc"},
  {"14:b2",false,{0,0,0},"P 1 21/n 1","-P 2yn"},
  {"14:b3",false,{0,0,0},"P 1 21/a 1","-P 2yab"},
  {"14:c1",false,{0,0,0},"P 1 1 21/a","-P 2ac"},
  {"14:c2",false,{0,0,0},"P 1 1 21/n","-P 2n"},
  {"14:c3",false,{0,0,0},"P 1 1 21/b","-P 2bc"},
  {"14:a1",false,{0,0,0},"P 21/b 1 1","-P 2xab"},
  {"14:a2",false,{0,0,0},"P 21/n 1 1","-P 2xn"},
  {"14:a3",false,{0,0,0},"P 21/c 1 1","-P 2xac"},
  {"15:b1",true,{0,0,0},"C 1 2/c 1","-C 2yc"},
  {"15:b2",false,{0,0,0},"A 1 2/n 1","-A 2yac"},
  {"15:b3",false,{0,0,0},"I 1 2/a 1","-I 2ya"},
  {"15:-b1",false,{0,0,0},"A 1 2/a 1","-A 2ya"},
  {"15:-b2",false,{0,0,0},"C 1 2/n 1","-C 2ybc"},
  {"15:-b3",false,{0,0,0},"I 1 2/c 1","-I 2yc"},
  {"15:c1",false,{0,0,0},"A 1 1 2/a","-A 2a"},
  {"15:c2",false,{0,0,0},"B 1 1 2/n","-B 2bc"},
  {"15:c3",false,{0,0,0},"I 1 1 2/b","-I 2b"},
  {"15:-c1",false,{0,0,0},"B 1 1 2/b","-B 2b"},
  {"15:-c2",false,{0,0,0},"A 1 1 2/n","-A 2ac"},
  {"15:-c3",false,{0,0,0},"I 1 1 2/a","-I 2a"},
  {"15:a1",false,{0,0,0},"B 2/b 1 1","-B 2xb"},
  {"15:a2",false,{0,0,0},"C 2/n 1 1","-C 2xbc"},
  {"15:a3",false,{0,0,0},"I 2/c 1 1","-I 2xc"},
  {"15:-a1",false,{0,0,0},"C 2/c 1 1","-C 2xc"},
  {"15:-a2",false,{0,0,0},"B 2/n 1 1","-B 2xbc"},
  {"15:-a3",false,{0,0,0},"I 2/b 1 1","-I 2xb"},
  {"16",true,{0,0,0},"P 2 2 2","P 2 2"},
  {"17",true,{0,0,0},"P 2 2 21","P 2c 2"},
  {"17:cab",false,{0,0,0},"P 21 2 2","P 2a 2a"},
  {"17:bca",false,{0,0,0},"P 2 21 2","P 2 2b"},
  {"18",true,{0,0,0},"P 21 21 2","P 2 2ab"},
  {"18:cab",false,{0,0,0},"P 2 21 21","P 2bc 2"},
  {"18:bca",false,{0,0,0},"P 21 2 21","P 2ac 2ac"},
  {"19",true,{0,0,0},"P 21 21 21","P 2ac 2ab"},
  {"20",true,{0,0,0},"C 2 2 21","C 2c 2"},
  {"20:cab",false,{0,0,0},"A 21 2 2","A 2a 2a"},
  {"20:bca",false,{0,0,0},"B 2 21 2","B 2 2b"},
  {"21",true,{0,0,0},"C 2 2 2","C 2 2"},
  {"21:cab",false,{0,0,0},"A 2 2 2","A 2 2"},
  {"21:bca",false,{0,0,0},"B 2 2 2","B 2 2"},
  {"22",true,{0,0,0},"F 2 2 2","F 2 2"},
  {"23",true,{0,0,0},"I 2 2 2","I 2 2"},
  {"24",true,{0,0,0},"I 21 21 21","I 2b 2c"},
  {"25",true,{0,0,0},"P m m 2","P 2 -2"},
  {"25:cab",false,{0,0,0},"P 2 m m","P -2 2"},
  {"25:bca",false,{0,0,0},"P m 2 m","P -2 -2"},
  {"26",true,{0,0,0},"P m c 21","P 2c -2"},
  {"26:ba-c",false,{0,0,0},"P c m 21","P 2c -2c"},
  {"26:cab",false,{0,0,0},"P 21 m a","P -2a 2a"},
  {"26:-cba",false,{0,0,0},"P 21 a m","P -2 2a"},
  {"26:bca",false,{0,0,0},"P b 21 m","P -2 -2b"},
  {"26:a-cb",false,{0,0,0},"P m 21 b","P -2b -2"},
  {"27",true,{0,0,0},"P c c 2","P 2 -2c"},
  {"27:cab",false,{0,0,0},"P 2 a a","P -2a 2"},
  {"27:bca",false,{0,0,0},"P b 2 b","P -2b -2b"},
  {"28",true,{0,0,0},"P m a 2","P 2 -2a"},
  {"28:ba-c",false,{0,0,0},"P b m 2","P 2 -2b"},
  {"28:cab",false,{0,0,0},"P 2 m b","P -2b 2"},
  {"28:-cba",false,{0,0,0},"P 2 c m","P -2c 2"},
  {"28:bca",false,{0,0,0},"P c 2 m","P -2c -2c"},
  {"28:a-cb",false,{0,0,0},"P m 2 a","P -2a -2a"},
  {"29",true,{0,0,0},"P c a 21","P 2c -2ac"},
  {"29:ba-c",false,{0,0,0},"P b c 21","P 2c -2b"},
  {"29:cab",false,{0,0,0},"P 21 a b","P -2b 2a"},
  {"29:-cba",false,{0,0,0},"P 21 c a","P -2ac 2a"},
  {"29:bca",false,{0,0,0},"P c 21 b","P -2bc -2c"},
  {"29:a-cb",false,{0,0,0},"P b 21 a","P -2a -2ab"},
  {"30",true,{0,0,0},"P n c 2","P 2 -2bc"},
  {"30:ba-c",false,{0,0,0},"P c n 2","P 2 -2ac"},
  {"30:cab",false,{0,0,0},"P 2 n a","P -2ac 2"},
  {"30:-cba",false,{0,0,0},"P 2 a n","P -2ab 2"},
  {"30:bca",false,{0,0,0},"P b 2 n","P -2ab -2ab"},
  {"30:a-cb",false,{0,0,0},"P n 2 b","P -2bc -2bc"},
  {"31",true,{0,0,0},"P m n 21","P 2ac -2"},
  {"31:ba-c",false,{0,0,0},"P n m 21","P 2bc -2bc"},
  {"31:cab",false,{0,0,0},"P 21 m n","P -2ab 2ab"},
  {"31:-cba",false,{0,0,0},"P 21 n m","P -2 2ac"},
  {"31:bca",false,{0,0,0},"P n 21 m","P -2 -2bc"},
  {"31:a-cb",false,{0,0,0},"P m 21 n","P -2ab -2"},
  {"32",true,{0,0,0},"P b a 2","P 2 -2ab"},
  {"32:cab",false,{0,0,0},"P 2 c b","P -2bc 2"},
  {"32:bca",false,{0,0,0},"P c 2 a","P -2ac -2ac"},
  {"33",true,{0,0,0},"P n a 21","P 2c -2n"},
  {"33:ba-c",false,{0,0,0},"P b n 21","P 2c -2ab"},
  {"33:cab",false,{0,0,0},"P 21 n b","P -2bc 2a"},
  {"33:-cba",false,{0,0,0},"P 21 c n","P -2n 2a"},
  {"33:bca",false,{0,0,0},"P c 21 n","P -2n -2ac"},
  {"33:a-cb",false,{0,0,0},"P n 21 a","P -2ac -2n"},
  {"34",true,{0,0,0},"P n n 2","P 2 -2n"},
  {"34:cab",false,{0,0,0},"P 2 n n","P -2n 2"},
  {"34:bca",false,{0,0,0},"P n 2 n","P -2n -2n"},
  {"35",true,{0,0,0},"C m m 2","C 2 -2"},
  {"35:cab",false,{0,0,0},"A 2 m m","A -2 2"},
  {"35:bca",false,{0,0,0},"B m 2 m","B -2 -2"},
  {"36",true,{0,0,0},"C m c 21","C 2c -2"},
  {"36:ba-c",false,{0,0,0},"C c m 21","C 2c -2c"},
  {"36:cab",false,{0,0,0},"A 21 m a","A -2a 2a"},
  {"36:-cba",false,{0,0,0},"A 21 a m","A -2 2a"},
  {"36:bca",false,{0,0,0},"B b 21 m","B -2 -2b"},
  {"36:a-cb",false,{0,0,0},"B m 21 b","B -2b -2"},
  {"37",true,{0,0,0},"C c c 2","C 2 -2c"},
  {"37:cab",false,{0,0,0},"A 2 a a","A -2a 2"},
  {"37:bca",false,{0,0,0},"B b 2 b","B -2b -2b"},
  {"38",true,{0,0,0},"A m m 2","A 2 -2"},
  {"38:ba-c",false,{0,0,0},"B m m 2","B 2 -2"},
  {"38:cab",false,{0,0,0},"B 2 m m","B -2 2"},
  {"38:-cba",false,{0,0,0},"C 2 m m","C -2 2"},
  {"38:bca",false,{0,0,0},"C m 2 m","C -2 -2"},
  {"38:a-cb",false,{0,0,0},"A m 2 m","A -2 -2"},
  {"39",true,{0,0,0},"A b m 2","A 2 -2c"},
  {"39:ba-c",false,{0,0,0},"B m a 2","B 2 -2c"},
  {"39:cab",false,{0,0,0},"B 2 c m","B -2c 2"},
  {"39:-cba",false,{0,0,0},"C 2 m b","C -2b 2"},
  {"39:bca",false,{0,0,0},"C m 2 a","C -2b -2b"},
  {"39:a-cb",false,{0,0,0},"A c 2 m","A -2c -2c"},
  {"40",true,{0,0,0},"A m a 2","A 2 -2a"},
  {"40:ba-c",false,{0,0,0},"B b m 2","B 2 -2b"},
  {"40:cab",false,{0,0,0},"B 2 m b","B -2b 2"},
  {"40:-cba",false,{0,0,0},"C 2 c m","C -2c 2"},
  {"40:bca",false,{0,0,0},"C c 2 m","C -2c -2c"},
  {"40:a-cb",false,{0,0,0},"A m 2 a","A -2a -2a"},
  {"41",true,{0,0,0},"A b a 2","A 2 -2ac"},
  {"41:ba-c",false,{0,0,0},"B b a 2","B 2 -2bc"},
  {"41:cab",false,{0,0,0},"B 2 c b","B -2bc 2"},
  {"41:-cba",false,{0,0,0},"C 2 c b","C -2bc 2"},
  {"41:bca",false,{0,0,0},"C c 2 a","C -2bc -2bc"},
  {"41:a-cb",false,{0,0,0},"A c 2 a","A -2ac -2ac"},
  {"42",true,{0,0,0},"F m m 2","F 2 -2"},
  {"42:cab",false,{0,0,0},"F 2 m m","F -2 2"},
  {"42:bca",false,{0,0,0},"F m 2 m","F -2 -2"},
  {"43",true,{0,0,0},"F d d 2","F 2 -2d"},
  {"43:cab",false,{0,0,0},"F 2 d d","F -2d 2"},
  {"43:bca",false,{0,0,0},"F d 2 d","F -2d -2d"},
  {"44",true,{0,0,0},"I m m 2","I 2 -2"},
  {"44:cab",false,{0,0,0},"I 2 m m","I -2 2"},
  {"44:bca",false,{0,0,0},"I m 2 m","I -2 -2"},
  {"45",true,{0,0,0},"I b a 2","I 2 -2c"},
  {"45:cab",false,{0,0,0},"I 2 c b","I -2a 2"},
  {"45:bca",false,{0,0,0},"I c 2 a","I -2b -2b"},
  {"46",true,{0,0,0},"I m a 2","I 2 -2a"},
  {"46:ba-c",false,{0,0,0},"I b m 2","I 2 -2b"},
  {"46:cab",false,{0,0,0},"I 2 m b","I -2b 2"},
  {"46:-cba",false,{0,0,0},"I 2 c m","I -2c 2"},
  {"46:bca",false,{0,0,0},"I c 2 m","I -2c -2c"},
  {"46:a-cb",false,{0,0,0},"I m 2 a","I -2a -2a"},
  {"47",true,{0,0,0},"P m m m","-P 2 2"},
  {"48:1",false,{2,2,2},"P n n n:1","P 2 2 -1n"},
  {"48:2",true,{0,0,0},"P n n n:2","-P 2ab 2bc"},
  {"49",true,{0,0,0},"P c c m","-P 2 2c"},
  {"49:cab",false,{0,0,0},"P m a a","-P 2a 2"},
  {"49:bca",false,{0,0,0},"P b m b","-P 2b 2b"},
  {"50:1",false,{2,2,0},"P b a n:1","P 2 2 -1ab"},
  {"50:2",true,{0,0,0},"P b a n:2","-P 2ab 2b"},
  {"50:1cab",false,{0,0,0},"P n c b:1","P 2 2 -1bc"},
  {"50:2cab",false,{0,0,0},"P n c b:2","-P 2b 2bc"},
  {"50:1bca",false,{0,0,0},"P c n a:1","P 2 2 -1ac"},
  {"50:2bca",false,{0,0,0},"P c n a:2","-P 2a 2c"},
  {"51",true,{0,0,0},"P m m a","-P 2a 2a"},
  {"51:ba-c",false,{0,0,0},"P m m b","-P 2b 2"},
  {"51:cab",false,{0,0,0},"P b m m","-P 2 2b"},
  {"51:-cba",false,{0,0,0},"P c m m","-P 2c 2c"},
  {"51:bca",false,{0,0,0},"P m c m","-P 2c 2"},
  {"51:a-cb",false,{0,0,0},"P m a m","-P 2 2a"},
  {"52",true,{0,0,0},"P n n a","-P 2a 2bc"},
  {"52:ba-c",false,{0,0,0},"P n n b","-P 2b 2n"},
  {"52:cab",false,{0,0,0},"P b n n","-P 2n 2b"},
  {"52:-cba",false,{0,0,0},"P c n n","-P 2ab 2c"},
  {"52:bca",false,{0,0,0},"P n c n","-P 2ab 2n"},
  {"52:a-cb",false,{0,0,0},"P n a n","-P 2n 2bc"},
  {"53",true,{0,0,0},"P m n a","-P 2ac 2"},
  {"53:ba-c",false,{0,0,0},"P n m b","-P 2bc 2bc"},
  {"53:cab",false,{0,0,0},"P b m n","-P 2ab 2ab"},
  {"53:-cba",false,{0,0,0},"P c n m","-P 2 2ac"},
  {"53:bca",false,{0,0,0},"P n c m","-P 2 2bc"},
  {"53:a-cb",false,{0,0,0},"P m a n","-P 2ab 2"},
  {"54",true,{0,0,0},"P c c a","-P 2a 2ac"},
  {"54:ba-c",false,{0,0,0},"P c c b","-P 2b 2c"},
  {"54:cab",false,{0,0,0},"P b a a","-P 2a 2b"},
  {"54:-cba",false,{0,0,0},"P c a a","-P 2ac 2c"},
  {"54:bca",false,{0,0,0},"P b c b","-P 2bc 2b"},
  {"54:a-cb",false,{0,0,0},"P b a b","-P 2b 2ab"},
  {"55",true,{0,0,0},"P b a m","-P 2 2ab"},
  {"55:cab",false,{0,0,0},"P m c b","-P 2bc 2"},
  {"55:bca",false,{0,0,0},"P c m a","-P 2ac 2ac"},
  {"56",true,{0,0,0},"P c c n","-P 2ab 2ac"},
  {"56:cab",false,{0,0,0},"P n a a","-P 2ac 2bc"},
  {"56:bca",false,{0,0,0},"P b n b","-P 2bc 2ab"},
  {"57",true,{0,0,0},"P b c m","-P 2c 2b"},
  {"57:ba-c",false,{0,0,0},"P c a m","-P 2c 2ac"},
  {"57:cab",false,{0,0,0},"P m c a","-P 2ac 2a"},
  {"57:-cba",false,{0,0,0},"P m a b","-P 2b 2a"},
  {"57:bca",false,{0,0,0},"P b m a","-P 2a 2ab"},
  {"57:a-cb",false,{0,0,0},"P c m b","-P 2bc 2c"},
  {"58",true,{0,0,0},"P n n m","-P 2 2n"},
  {"58:cab",false,{0,0,0},"P m n n","-P 2n 2"},
  {"58:bca",false,{0,0,0},"P n m n","-P 2n 2n"},
  {"59:1",false,{2,2,0},"P m m n:1","P 2 2ab -1ab"},
  {"59:2",true,{0,0,0},"P m m n:2","-P 2ab 2a"},
  {"59:1cab",false,{0,0,0},"P n m m:1","P 2bc 2 -1bc"},
  {"59:2cab",false,{0,0,0},"P n m m:2","-P 2c 2bc"},
  {"59:1bca",false,{0,0,0},"P m n m:1","P 2ac 2ac -1ac"},
  {"59:2bca",false,{0,0,0},"P m n m:2","-P 2c 2a"},
  {"60",true,{0,0,0},"P b c n","-P 2n 2ab"},
  {"60:ba-c",false,{0,0,0},"P c a n","-P 2n 2c"},
  {"60:cab",false,{0,0,0},"P n c a","-P 2a 2n"},
  {"60:-cba",false,{0,0,0},"P n a b","-P 2bc 2n"},
  {"60:bca",false,{0,0,0},"P b n a","-P 2ac 2b"},
  {"60:a-cb",false,{0,0,0},"P c n b","-P 2b 2ac"},
  {"61",true,{0,0,0},"P b c a","-P 2ac 2ab"},
  {"61:ba-c",false,{0,0,0},"P c a b","-P 2bc 2ac"},
  {"62",true,{0,0,0},"P n m a","-P 2ac 2n"},
  {"62:ba-c",false,{0,0,0},"P m n b","-P 2bc 2a"},
  {"62:cab",false,{0,0,0},"P b n m","-P 2c 2ab"},
  {"62:-cba",false,{0,0,0},"P c m n","-P 2n 2ac"},
  {"62:bca",false,{0,0,0},"P m c n","-P 2n 2a"},
  {"62:a-cb",false,{0,0,0},"P n a m","-P 2c 2n"},
  {"63",true,{0,0,0},"C m c m","-C 2c 2"},
  {"63:ba-c",false,{0,0,0},"C c m m","-C 2c 2c"},
  {"63:cab",false,{0,0,0},"A m m a","-A 2a 2a"},
  {"63:-cba",false,{0,0,0},"A m a m","-A 2 2a"},
  {"63:bca",false,{0,0,0},"B b m m","-B 2 2b"},
  {"63:a-cb",false,{0,0,0},"B m m b","-B 2b 2"},
  {"64",true,{0,0,0},"C m c a","-C 2bc 2"},
  {"64:ba-c",false,{0,0,0},"C c m b","-C 2bc 2bc"},
  {"64:cab",false,{0,0,0},"A b m a","-A 2ac 2ac"},
  {"64:-cba",false,{0,0,0},"A c a m","-A 2 2ac"},
  {"64:bca",false,{0,0,0},"B b c m","-B 2 2bc"},
  {"64:a-cb",false,{0,0,0},"B m a b","-B 2bc 2"},
  {"65",true,{0,0,0},"C m m m","-C 2 2"},
  {"65:cab",false,{0,0,0},"A m m m","-A 2 2"},
  {"65:bca",false,{0,0,0},"B m m m","-B 2 2"},
  {"66",true,{0,0,0},"C c c m","-C 2 2c"},
  {"66:cab",false,{0,0,0},"A m a a","-A 2a 2"},
  {"66:bca",false,{0,0,0},"B b m b","-B 2b 2b"},
  {"67",true,{0,0,0},"C m m a","-C 2b 2"},
  {"67:ba-c",false,{0,0,0},"C m m b","-C 2b 2b"},
  {"67:cab",false,{0,0,0},"A b m m","-A 2c 2c"},
  {"67:-cba",false,{0,0,0},"A c m m","-A 2 2c"},
  {"67:bca",false,{0,0,0},"B m c m","-B 2 2c"},
  {"67:a-cb",false,{0,0,0},"B m a m","-B 2c 2"},
  {"68:1",false,{0,2,2},"C c c a:1","C 2 2 -1bc"},
  {"68:2",true,{0,0,0},"C c c a:2","-C 2b 2bc"},
  {"68:1ba-c",false,{0,0,0},"C c c b:1","C 2 2 -1bc"},
  {"68:2ba-c",false,{0,0,0},"C c c b:2","-C 2b 2c"},
  {"68:1cab",false,{0,0,0},"A b a a:1","A 2 2 -1ac"},
  {"68:2cab",false,{0,0,0},"A b a a:2","-A 2a 2c"},
  {"68:1-cba",false,{0,0,0},"A c a a:1","A 2 2 -1ac"},
  {"68:2-cba",false,{0,0,0},"A c a a:2","-A 2ac 2c"},
  {"68:1bca",false,{0,0,0},"B b c b:1","B 2 2 -1bc"},
  {"68:2bca",false,{0,0,0},"B b c b:2","-B 2bc 2b"},
  {"68:1a-cb",false,{0,0,0},"B b a b:1","B 2 2 -1bc"},
  {"68:2a-cb",false,{0,0,0},"B b a b:2","-B 2b 2bc"},
  {"69",true,{0,0,0},"F m m m","-F 2 2"},
  {"70:1",false,{7,7,7},"F d d d:1","F 2 2 -1d"},
  {"70:2",true,{0,0,0},"F d d d:2","-F 2uv 2vw"},
  {"71",true,{0,0,0},"I m m m","-I 2 2"},
  {"72",true,{0,0,0},"I b a m","-I 2 2c"},
  {"72:cab",false,{0,0,0},"I m c b","-I 2a 2"},
  {"72:bca",false,{0,0,0},"I c m a","-I 2b 2b"},
  {"73",true,{0,0,0},"I b c a","-I 2b 2c"},
  {"73:ba-c",false,{0,0,0},"I c a b","-I 2a 2b"},
  {"74",true,{0,0,0},"I m m a","-I 2b 2"},
  {"74:ba-c",false,{0,0,0},"I m m b","-I 2a 2a"},
  {"74:cab",false,{0,0,0},"I b m m","-I 2c 2c"},
  {"74:-cba",false,{0,0,0},"I c m m","-I 2 2b"},
  {"74:bca",false,{0,0,0},"I m c m","-I 2 2a"},
  {"74:a-cb",false,{0,0,0},"I m a m","-I 2c 2"},
  {"75",true,{0,0,0},"P 4","P 4"},
  {"76",true,{0,0,0},"P 41","P 4w"},
  {"77",true,{0,0,0},"P 42","P 4c"},
  {"78",true,{0,0,0},"P 43","P 4cw"},
  {"79",true,{0,0,0},"I 4","I 4"},
  {"80",true,{0,0,0},"I 41","I 4bw"},
  {"81",true,{0,0,0},"P -4","P -4"},
  {"82",true,{0,0,0},"I -4","I -4"},
  {"83",true,{0,0,0},"P 4/m","-P 4"},
  {"84",true,{0,0,0},"P 42/m","-P 4c"},
  {"85:1",false,{-2,2,0},"P 4/n:1","P 4ab -1ab"},
  {"85:2",true,{0,0,0},"P 4/n:2","-P 4a"},
  {"86:1",false,{2,2,2},"P 42/n:1","P 4n -1n"},
  {"86:2",true,{0,0,0},"P 42/n:2","-P 4bc"},
  {"87",true,{0,0,0},"I 4/m","-I 4"},
  {"88:1",false,{0,-2,-1},"I 41/a:1","I 4bw -1bw"},
  {"88:2",true,{0,0,0},"I 41/a:2","-I 4ad"},
  {"89",true,{0,0,0},"P 4 2 2","P 4 2"},
  {"90",true,{0,0,0},"P 42 1 2","P 4ab 2ab"},
  {"91",true,{0,0,0},"P 41 2 2","P 4w 2c"},
  {"92",true,{0,0,0},"P 41 21 2","P 4abw 2nw"},
  {"93",true,{0,0,0},"P 42 2 2","P 4c 2"},
  {"94",true,{0,0,0},"P 42 21 2","P 4n 2n"},
  {"95",true,{0,0,0},"P 43 2 2","P 4cw 2c"},
  {"96",true,{0,0,0},"P 43 21 2","P 4nw 2abw"},
  {"97",true,{0,0,0},"I 4 2 2","I 4 2"},
  {"98",true,{0,0,0},"I 41 2 2","I 4bw 2bw"},
  {"99",true,{0,0,0},"P 4 m m","P 4 -2"},
  {"100",true,{0,0,0},"P 4 b m","P 4 -2ab"},
  {"101",true,{0,0,0},"P 42 c m","P 4c -2c"},
  {"102",true,{0,0,0},"P 42 n m","P 4n -2n"},
  {"103",true,{0,0,0},"P 4 c c","P 4 -2c"},
  {"104",true,{0,0,0},"P 4 n c","P 4 -2n"},
  {"105",true,{0,0,0},"P 42 m c","P 4c -2"},
  {"106",true,{0,0,0},"P 42 b c","P 4c -2ab"},
  {"107",true,{0,0,0},"I 4 m m","I 4 -2"},
  {"108",true,{0,0,0},"I 4 c m","I 4 -2c"},
  {"109",true,{0,0,0},"I 41 m d","I 4bw -2"},
  {"110",true,{0,0,0},"I 41 c d","I 4bw -2c"},
  {"111",true,{0,0,0},"P -4 2 m","P -4 2"},
  {"112",true,{0,0,0},"P -4 2 c","P -4 2c"},
  {"113",true,{0,0,0},"P -4 21 m","P -4 2ab"},
  {"114",true,{0,0,0},"P -4 21 c","P -4 2n"},
  {"115",true,{0,0,0},"P -4 m 2","P -4 -2"},
  {"116",true,{0,0,0},"P -4 c 2","P -4 -2c"},
  {"117",true,{0,0,0},"P -4 b 2","P -4 -2ab"},
  {"118",true,{0,0,0},"P -4 n 2","P -4 -2n"},
  {"119",true,{0,0,0},"I -4 m 2","I -4 -2"},
  {"120",true,{0,0,0},"I -4 c 2","I -4 -2c"},
  {"121",true,{0,0,0},"I -4 2 m","I -4 2"},
  {"122",true,{0,0,0},"I -4 2 d","I -4 2bw"},
  {"123",true,{0,0,0},"P 4/m m m","-P 4 2"},
  {"124",true,{0,0,0},"P 4/m c c","-P 4 2c"},
  {"125:1",false,{2,2,0},"P 4/n b m:1","P 4 2 -1ab"},
  {"125:2",true,{0,0,0},"P 4/n b m:2","-P 4a 2b"},
  {"126:1",false,{2,2,2},"P 4/n n c:1","P 4 2 -1n"},
  {"126:2",true,{0,0,0},"P 4/n n c:2","-P 4a 2bc"},
  {"127",true,{0,0,0},"P 4/m b m","-P 4 2ab"},
  {"128",true,{0,0,0},"P 4/m n c","-P 4 2n"},
  {"129:1",false,{2,-2,0},"P 4/n m m:1","P 4ab 2ab -1ab"},
  {"129:2",true,{0,0,0},"P 4/n m m:2","-P 4a 2a"},
  {"130:1",false,{2,-2,0},"P 4/n c c:1","P 4ab 2n -1ab"},
  {"130:2",true,{0,0,0},"P 4/n c c:2","-P 4a 2ac"},
  {"131",true,{0,0,0},"P 42/m m c","-P 4c 2"},
  {"132",true,{0,0,0},"P 42/m c m","-P 4c 2c"},
  {"133:1",false,{2,-2,2},"P 42/n b c:1","P 4n 2c -1n"},
  {"133:2",true,{0,0,0},"P 42/n b c:2","-P 4ac 2b"},
  {"134:1",false,{2,-2,2},"P 42/n n m:1","P 4n 2 -1n"},
  {"134:2",true,{0,0,0},"P 42/n n m:2","-P 4ac 2bc"},
  {"135",true,{0,0,0},"P 42/m b c","-P 4c 2ab"},
  {"136",true,{0,0,0},"P 42/m n m","-P 4n 2n"},
  {"137:1",false,{2,-2,2},"P 42/n m c:1","P 4n 2n -1n"},
  {"137:2",true,{0,0,0},"P 42/n m c:2","-P 4ac 2a"},
  {"138:1",false,{2,-2,2},"P 42/n c m:1","P 4n 2ab -1n"},
  {"138:2",true,{0,0,0},"P 42/n c m:2","-P 4ac 2ac"},
  {"139",true,{0,0,0},"I 4/m m m","-I 4 2"},
  {"140",true,{0,0,0},"I 4/m c m","-I 4 2c"},
  {"141:1",false,{0,2,-1},"I 41/a m d:1","I 4bw 2bw -1bw"},
  {"141:2",true,{0,0,0},"I 41/a m d:2","-I 4bd 2"},
  {"142:1",false,{0,2,-1},"I 41/a c d:1","I 4bw 2aw -1bw"},
  {"142:2",true,{0,0,0},"I 41/a c d:2","-I 4bd 2c"},
  {"143",true,{0,0,0},"P 3","P 3"},
  {"144",true,{0,0,0},"P 31","P 31"},
  {"145",true,{0,0,0},"P 32","P 32"},
  {"146:H",false,{0,0,0},"R 3:H","R 3"},
  {"146:R",true,{0,0,0},"R 3:R","P 3*"},
  {"147",true,{0,0,0},"P -3","-P 3"},
  {"148:H",false,{0,0,0},"R -3:H","-R 3"},
  {"148:R",true,{0,0,0},"R -3:R","-P 3*"},
  {"149",true,{0,0,0},"P 3 1 2","P 3 2"},
  {"150",true,{0,0,0},"P 3 2 1","P 3 2\""},
  {"151",true,{0,0,0},"P 31 1 2","P 31 2c (0 0 1)"},
  {"152",true,{0,0,0},"P 31 2 1","P 31 2\""},
  {"153",true,{0,0,0},"P 32 1 2","P 32 2c (0 0 -1)"},
  {"154",true,{0,0,0},"P 32 2 1","P 32 2\""},
  {"155:H",false,{0,0,0},"R 32:H","R 3 2\""},
  {"155:R",true,{0,0,0},"R 32:R","P 3* 2"},
  {"156",true,{0,0,0},"P 3 m 1","P 3 -2\""},
  {"157",true,{0,0,0},"P 3 1 m","P 3 -2"},
  {"158",true,{0,0,0},"P 3 c 1","P 3 -2\"c"},
  {"159",true,{0,0,0},"P 3 1 c","P 3 -2c"},
  {"160:H",false,{0,0,0},"R 3 m:H","R 3 -2\""},
  {"160:R",true,{0,0,0},"R 3 m:R","P 3* -2"},
  {"161:H",false,{0,0,0},"R 3 c:H","R 3 -2\"c"},
  {"161:R",true,{0,0,0},"R 3 c:R","P 3* -2n"},
  {"162",true,{0,0,0},"P -3 1 m","-P 3 2"},
  {"163",true,{0,0,0},"P -3 1 c","-P 3 2c"},
  {"164",true,{0,0,0},"P -3 m 1","-P 3 2\""},
  {"165",true,{0,0,0},"P -3 c 1","-P 3 2\"c"},
  {"166:H",false,{0,0,0},"R -3 m:H","-R 3 2\""},
  {"166:R",true,{0,0,0},"R -3 m:R","-P 3* 2"},
  {"167:H",false,{0,0,0},"R -3 c:H","-R 3 2\"c"},
  {"167:R",true,{0,0,0},"R -3 c:R","-P 3* 2n"},
  {"168",true,{0,0,0},"P 6","P 6"},
  {"169",true,{0,0,0},"P 61","P 61"},
  {"170",true,{0,0,0},"P 65","P 65"},
  {"171",true,{0,0,0},"P 62","P 62"},
  {"172",true,{0,0,0},"P 64","P 64"},
  {"173",true,{0,0,0},"P 63","P 6c"},
  {"174",true,{0,0,0},"P -6","P -6"},
  {"175",true,{0,0,0},"P 6/m","-P 6"},
  {"176",true,{0,0,0},"P 63/m","-P 6c"},
  {"177",true,{0,0,0},"P 6 2 2","P 6 2"},
  {"178",true,{0,0,0},"P 61 2 2","P 61 2 (0 0 -1)"},
  {"179",true,{0,0,0},"P 65 2 2","P 65 2 (0 0 1)"},
  {"180",true,{0,0,0},"P 62 2 2","P 62 2c (0 0 1)"},
  {"181",true,{0,0,0},"P 64 2 2","P 64 2c (0 0 -1)"},
  {"182",true,{0,0,0},"P 63 2 2","P 6c 2c"},
  {"183",true,{0,0,0},"P 6 m m","P 6 -2"},
  {"184",true,{0,0,0},"P 6 c c","P 6 -2c"},
  {"185",true,{0,0,0},"P 63 c m","P 6c -2"},
  {"186",true,{0,0,0},"P 63 m c","P 6c -2c"},
  {"187",true,{0,0,0},"P -6 m 2","P -6 2"},
  {"188",true,{0,0,0},"P -6 c 2","P -6c 2"},
  {"189",true,{0,0,0},"P -6 2 m","P -6 -2"},
  {"190",true,{0,0,0},"P -6 2 c","P -6c -2c"},
  {"191",true,{0,0,0},"P 6/m m m","-P 6 2"},
  {"192",true,{0,0,0},"P 6/m c c","-P 6 2c"},
  {"193",true,{0,0,0},"P 63/m c m","-P 6c 2"},
  {"194",true,{0,0,0},"P 63/m m c","-P 6c 2c"},
  {"195",true,{0,0,0},"P 2 3","P 2 2 3"},
  {"196",true,{0,0,0},"F 2 3","F 2 2 3"},
  {"197",true,{0,0,0},"I 2 3","I 2 2 3"},
  {"198",true,{0,0,0},"P 21 3","P 2ac 2ab 3"},
  {"199",true,{0,0,0},"I 21 3","I 2b 2c 3"},
  {"200",true,{0,0,0},"P m -3","-P 2 2 3"},
  {"201:1",false,{2,2,2},"P n -3:1","P 2 2 3 -1n"},
  {"201:2",true,{0,0,0},"P n -3:2","-P 2ab 2bc 3"},
  {"202",true,{0,0,0},"F m -3","-F 2 2 3"},
  {"203:1",false,{-1,-1,-1},"F d -3:1","F 2 2 3 -1d"},
  {"203:2",true,{0,0,0},"F d -3:2","-F 2uv 2vw 3"},
  {"204",true,{0,0,0},"I m -3","-I 2 2 3"},
  {"205",true,{0,0,0},"P a -3","-P 2ac 2ab 3"},
  {"206",true,{0,0,0},"I a -3","-I 2b 2c 3"},
  {"207",true,{0,0,0},"P 4 3 2","P 4 2 3"},
  {"208",true,{0,0,0},"P 42 3 2","P 4n 2 3"},
  {"209",true,{0,0,0},"F 4 3 2","F 4 2 3"},
  {"210",true,{0,0,0},"F 41 3 2","F 4d 2 3"},
  {"211",true,{0,0,0},"I 4 3 2","I 4 2 3"},
  {"212",true,{0,0,0},"P 43 3 2","P 4acd 2ab 3"},
  {"213",true,{0,0,0},"P 41 3 2","P 4bd 2ab 3"},
  {"214",true,{0,0,0},"I 41 3 2","I 4bd 2c 3"},
  {"215",true,{0,0,0},"P -4 3 m","P -4 2 3"},
  {"216",true,{0,0,0},"F -4 3 m","F -4 2 3"},
  {"217",true,{0,0,0},"I -4 3 m","I -4 2 3"},
  {"218",true,{0,0,0},"P -4 3 n","P -4n 2 3"},
  {"219",true,{0,0,0},"F -4 3 c","F -4c 2 3"},
  {"220",true,{0,0,0},"I -4 3 d","I -4bd 2c 3"},
  {"221",true,{0,0,0},"P m -3 m","-P 4 2 3"},
  {"222:1",false,{2,2,2},"P n -3 n:1","P 4 2 3 -1n"},
  {"222:2",true,{0,0,0},"P n -3 n:2","-P 4a 2bc 3"},
  {"223",true,{0,0,0},"P m -3 n","-P 4n 2 3"},
  {"224:1",false,{2,2,2},"P n -3 m:1","P 4n 2 3 -1n"},
  {"224:2",true,{0,0,0},"P n -3 m:2","-P 4bc 2bc 3"},
  {"225",true,{0,0,0},"F m -3 m","-F 4 2 3"},
  {"226",true,{0,0,0},"F m -3 c","-F 4c 2 3"},
  {"227:1",false,{-1,-1,-1},"F d -3 m:1","F 4d 2 3 -1d"},
  {"227:2",true,{0,0,0},"F d -3 m:2","-F 4vw 2vw 3"},
  {"228:1",false,{1,1,1},"F d -3 c:1","F 4d 2 3 -1cd"},
  {"228:2",true,{0,0,0},"F d -3 c:2","-F 4cvw 2vw 3"},
  {"229",true,{0,0,0},"I m -3 m","-I 4 2 3"},
  {"230",true,{0,0,0},"I a -3 d","-I 4bd 2c 3"}

};

long Data::noOfITHMHall=sizeof(Data::ITHMHall)/sizeof(struct Data::tsithmhall);


static long datartgt[9]={2,1,1,-1,1,1,-1,-2,1};
static long databct[9]={-1,1,1,1,-1,1,1,1,-1};
static long datafct[9]={0,1,1,1,0,1,1,1,0};
static long datacfct[9]={1,1,0,-1,1,0,0,0,2};
static long dataafct[9]={0,1,-1,0,1,1,2,0,0};
static long databfct[9]={-1,0,1,1,0,1,0,2,0};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix Data::bravaisMatrix(BravaisLattice b)
{
  if(b==BRhombohedral){
    return FMatrix(3,3,datartgt,3);
  }else if(b==BodyCentered){
    return FMatrix(3,3,databct,2);
  }else if(b==FaceCentered){
    return FMatrix(3,3,datafct,2);
  }else if(b==CBaseCentered){
    return FMatrix(3,3,datacfct,2);
  }else if(b==ABaseCentered){
    return FMatrix(3,3,dataafct,2);
  }else if(b==BBaseCentered){
    return FMatrix(3,3,databfct,2);
  }else{
    throw Except(Sprintf("SpaceGroup::Data::bravaisMatrix: Unkown bravais lattice '%s'?",
			 bravaisLatticeName(b)));
  }
  return FMatrix(3,3);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Data::HallFromIT(const String &it)
{
  for(int i=0;i<noOfITHMHall;i++){
    //myDebug()<<ITHMHall[i].IT<<ITHMHall[i].Hall<<it;
    if(it==ITHMHall[i].IT)return String(ITHMHall[i].Hall);
  }
  for(int i=0;i<noOfITHMHall;i++){
    StringList sl=String(ITHMHall[i].IT).split(":");
    if(sl[0]==it && ITHMHall[i].defaultsetting) return String(ITHMHall[i].Hall);
  }
  throw Except("HallFromIT: unknown IT-number symbol");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Data::defaultSetting(const String num)
{
  //myDebug()<<"num="<<num;
  for(int i=0;i<noOfITHMHall;i++){
    StringList sl=String(ITHMHall[i].IT).split(":");
    //myDebug()<<ITHMHall[i].IT<<sl;
    if(sl[0]==num && ITHMHall[i].defaultsetting){
      if(sl.size()==1) return "";
      return sl[1];
    }
  }
  throw Except("defaultsetting: unknown IT-number or default setting");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const long *Data::deltaToDefaultT8(const String it)
{
  for(int i=0;i<noOfITHMHall;i++){
    if(strcmp(ITHMHall[i].IT,it())==0)return ITHMHall[i].deltatodefaultt8;
  }
  throw Except("deltaToDefaultT8: unknown IT-number or setting");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Data::HMFromIT(const String it)
{
  for(int i=0;i<noOfITHMHall;i++){
    if(strcmp(ITHMHall[i].IT,it())==0)return ITHMHall[i].HM;
  }
  throw Except("HMFromIT: unknown IT-number or setting");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Data::settingByIndex(const String it,long settingnumber)
{
  // go through settings of given spacegroup and return it,
  // if settingnumber is out of range, cycle through it
  String num=String(it).split(":")[0];
  //myDebug()<<"num="<<num;
  StringList gr;
  for(int i=0;i<noOfITHMHall;i++){
    StringList sl=String(ITHMHall[i].IT).split(":");
    //myDebug()<<"sl[0]="<<sl[0];
    if(sl[0]==num) {
      gr.append((sl.size()>1?sl[1]:""));
    }
  }
  //myDebug()<<"gr="<<gr;
  return gr[settingnumber%gr.size()];
  throw Except("settingByIndex: unknown IT-number or setting");
}


/* EOF */
