/*************************************************************************
 * @(#)SymmetryGroup.cpp
 * author: magru@unas
 * date:   13 Nov 2012
 *************************************************************************
 */

#include "SymmetryGroup.hh"


using namespace SpaceGroup;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Matrix3x3 LatticeParameters::getConvCell(Data::CrystalSystem pcr,
					 Data::BravaisLattice bra)
{
  Matrix3x3 cell;
  double toldp=1.0e-14;
  
  if(pcr==Data::Rhombohedral && bra==Data::Simple){
    double la=sqrt(MyMath::max<double>(0,2./3.*(1.-cos(pangles[0]/180.*Pi))));
    double mu=sqrt(MyMath::max<double>(0,1.-la*la));
    
    cell(0,0)=       pabc[0]*la;
    cell(2,0)=       pabc[0]*mu;
    cell(0,1)=  -0.5*pabc[0]*la;
    cell(1,1)= sq3o2*pabc[0]*la;
    cell(2,1)=       pabc[0]*mu;
    cell(0,2)=  -0.5*pabc[0]*la;
    cell(1,2)=-sq3o2*pabc[0]*la;
    cell(2,2)=       pabc[0]*mu;
  }else if(platticetype==Solid){
    cell(2,2)=pabc[2];
    double d=pangles[0]/180.*Pi;
    double ca=cos(d);
    double sa=sin(d);
    if(fabs(pangles[0]-90.)<toldp){ ca=0.; sa=1.; }
    cell(1,1)=pabc[1]*sa;
    cell(2,1)=pabc[1]*ca;
    d=pangles[1]/180.*Pi;
    double cb=cos(d);
    double sb=sin(d);
    if(fabs(pangles[1]-90.)<toldp){ cb=0.; sb=1.; }
    d=pangles[2]/180.*Pi;
    double cg=cos(d);
    if(fabs(pangles[2]-90.)<toldp){ cg=0.;  }
    double f=(cg-ca*cb)/sa;
    double e=sqrt(MyMath::max<double>(0.,sb*sb-f*f));
    cell(0,0)=pabc[0]*e;
    cell(1,0)=pabc[0]*f;
    cell(2,0)=pabc[0]*cb;
  }else if(platticetype==Slab){
    cell(2,2)=pabc[2];
    cell(1,1)=pabc[1];
    
    double d=pangles[2]/180.*Pi;
    double cg=cos(d);
    double sg=sin(d);
    if(fabs(pangles[2]-90.)<toldp){ cg=0.;sg=1;  }
    cell(0,0)=pabc[0]*sg;
    cell(1,0)=pabc[0]*cg;
  }else{
    cell=Matrix3x3::diagonal(1,1,1);
  }

  return cell;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LatticeParameters::convertLatticeParameters(const Matrix3x3 &S,
						 const SeitzMatrix &Stransform)
{

  //myDebug()<<"convertLatticeParameters:-----------------------";
  //myDebug()<<"S="<<S;

  Matrix3x3 U=Stransform.W().toMatrix3x3();
  //myDebug()<<"U="<<U;
  Matrix3x3 Sp=U.transposed()*S*U;
  //myDebug()<<"UT*S*U="<<Sp;

  pabc[0]=sqrt(Sp(0,0));
  pabc[1]=sqrt(Sp(1,1));
  pabc[2]=sqrt(Sp(2,2));
  pangles[0]=acos(Sp(1,2)/(pabc[1]*pabc[2]))/Pi*180.;
  pangles[1]=acos(Sp(2,0)/(pabc[2]*pabc[0]))/Pi*180.;
  pangles[2]=acos(Sp(0,1)/(pabc[0]*pabc[1]))/Pi*180.;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Matrix3x3 LatticeParameters::getMetric()const
{
  Matrix3x3 S;
  
  //myDebug()<<"abc   ="<<pabc;
  //myDebug()<<"albega="<<pangles;

  S(0,0)=pabc[0]*pabc[0];
  S(1,1)=pabc[1]*pabc[1];
  S(2,2)=pabc[2]*pabc[2];
  double d=pabc[0]*pabc[1]*cos(pangles[2]/180.*Pi);
  S(0,1)=S(1,0)=d;
  d=pabc[1]*pabc[2]*cos(pangles[0]/180.*Pi);
  S(1,2)=S(2,1)=d;
  d=pabc[2]*pabc[0]*cos(pangles[1]/180.*Pi);
  S(0,2)=S(2,0)=d;

  return S;
}



/*EOF*/ 
