/* @(#)SymmetryGroup.hh
 * author: magru@unas
 */

#ifndef _SYMMETRYGROUP_HH
#define _SYMMETRYGROUP_HH 1

#include "Vec3.hh"
#include "FString.hh"
#include "SpaceGroupData.hh"
#include "SeitzMatrix.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

namespace SpaceGroup{

  class HallSymbol;


  enum LatticeType{Molecule=0,Slab=2,Solid=3};
  
  class LatticeParameters{
  public:
    LatticeParameters():platticetype(Solid),pabc(Vec3(1,1,1)),pangles(Vec3(90.,90.,90.)){}

    const Vec3 &latticeConstants()const{return pabc;}
    const Vec3 &axisAngles()const{return pangles;}

    void setLatticeConstants(double a,double b,double c){
      pabc[0]=a;pabc[1]=b;pabc[2]=c;}
    void setAxisAngles(double al,double be,double ga){
      pangles[0]=al;pangles[1]=be;pangles[2]=ga;}


    Matrix3x3 getConvCell(Data::CrystalSystem pcr,Data::BravaisLattice bra);

    // Given a metric S calc U^T*S*U and determine pabc, pangles
    void convertLatticeParameters(const Matrix3x3 &S,
				  const SeitzMatrix &Stransform);

    // Return metric matrix, for conversion purposes. 
    Matrix3x3 getMetric()const;

    String printS()const{
      if(platticetype==Solid){
	return "abc="+pabc.printS("%30.15f"," ",true)+"\nalbega="+
	  pangles.printS("%30.15f"," ",true);
      }else if(platticetype==Slab){
	return Sprintf("ab   =%s %s\n"
		       "gamma=%s\n",
		       String::doubleToString("%30.15f",pabc[0],true)(),
		       String::doubleToString("%30.15f",pabc[1],true)(),
		       String::doubleToString("%30.15f",pangles[2],true)());
      }else if(platticetype==Molecule){
	return "";
      }
      return "";
    }


  private:
    LatticeType platticetype;
    Vec3 pabc;
    Vec3 pangles;


    friend class HallSymbol;

  };

  class SymmetryGroup{
  public:
    SymmetryGroup();
    ~SymmetryGroup();
  protected:
  private:
    
  };
  
}
#endif /* _SYMMETRYGROUP_HH */

