/*************************************************************************
 * @(#)main.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   06 Aug 2012
 *************************************************************************
 */

#include<stdlib.h>
#include<stdio.h>
#include"CIFDriver.hh"  
#include"CIFData.hh"  
#include"Assorted.hh"  
#include"MyDebug.hh"  
#include"HallSymbol.hh"  
#include"IntegerTricks.hh"  
#include"IMatrix.hh"  
#include"FMatrix.hh"  
#include"SeitzMatrix.hh"  
#include"SymmetryGroup.hh"  
#include"PeriodicHyperPlane.hh"  

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void usage()
{
  printf("\
Usage: readcif [-print|-extract-sym] [-debugp] [-debugl] [filename] \
[-datablock n]\n\
\t-print: print content to stdout\n\
	 \t-datablock: needs integer: the desired datablock\n\
\t-extract_sym: try to extract the symmetry information\n\
\t-debugp: parser debugging output\n\
\t-debugl: lexical analyser debugging output\n\
\tfilename: an existing filename\n\
\n");
}

using namespace CIF;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

static void allOfSingleSection(const CIFData::DataBlock *datablock,
			       CIFMetaDict &dict,
			       CIFMetaDict::DictKey key)
{
  String basename=dict.value(key);
  basename.replace("_"," ",0,true).capitalize(true);
  ValueArray<Pair<String,String> > names;
  try{
    names=datablock->allOfSingleCategoryAsString(dict,key);
  }catch(...){return;}

  printf("%s\n",basename());
  for(long i=0;i<names.size();i++){
    printf("\t%s: %s\n",names[i].v1(),names[i].v2.stripBlanks(true)());
  }
  printf("\n");
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long determinantREC(const long *A,const long *R,const long *C,long N,long cols)
{
//   myDebug()<<"determinantREC: R=";
//   for(long c=0;c<N;c++){
//     printf("%ld ",R[c]);
//   }
//   printf("\n");
//   myDebug()<<"determinantREC: C=";
//   for(long c=0;c<N;c++){
//     printf("%ld ",C[c]);
//   }
//   printf("\n");
  
  if(N==1){
    return A[R[0]+cols*C[0]];
  }
  long s=-1;
  long det=0;
  long *CC=new long[N-1];
  long *RR=new long[N-1];
  for(long j=0;j<N-1;j++) RR[j]=R[j+1];
  
  for(long c=0;c<N;c++){
    s*=-1;
    long n=0;
    for(long j=0;j<N;j++){
      if(j!=c) CC[n++]=C[j];
    }
    det+=s*A[R[0]+cols*C[c]]*determinantREC(A,RR,CC,N-1,cols);
  }
  delete [] CC;
  delete [] RR;
  return det;
}
long determinant(const long *A,long cols)
{
  long *C=new long[cols];
  long *R=new long[cols];
  
  for(long c=0;c<cols;c++){
    C[c]=c;
    R[c]=c;
  }
  return determinantREC(A,R,C,cols,cols);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void testdecompAVeqB( )
{
  
  //todo: check rows<cols
  long rows=3;
  long cols=rand()%10+1;
  IMatrix A(rows,cols);
  
  for(long r=0;r<rows;r++){
    for(long c=0;c<cols;c++){
      if(rand()%(rand()%110+1)!=0)
	A(r,c)=rand()%20-10;
      else
	A(r,c)=0;
    }
  }
  for(long c=0;c<cols;c++){
    //A(rows-3,c)=0;
  }
  IMatrix Asave(A);
  myDebug()<<"A="<<A.printS();
  bool detnotone=false;
  long rank;
  IMatrix V(A.decompAVeqB(rank,detnotone)); 
  A.addRows(cols-rank);
  
  myDebug()<<"rank="<< rank;
  myDebug()<<"A="<<A.printS();
  myDebug()<<"V="<<V.printS();
  
  IMatrix B=(Asave*V).Td();
  
  myDebug()<<"B="<<B.printS();
  
  long det=determinant(V.data(),cols);
  myDebug()<<"det="<<det;
  if(!detnotone && det!=1) {myDebug()<<"error det"<<det;exit(1);};
  if(detnotone && det<1) {myDebug()<<"error det"<<det;exit(1);};
  if(Fnorm2(B-A)!=0){myDebug()<<"error";exit(1);}
  
  exit(0);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void testdecompAVeqBF()
{
  
  //todo: check rows<cols
  long rows=3;
  long cols=rand()%10+1;
  FMatrix A(rows,cols);
  
  for(long r=0;r<rows;r++){
    for(long c=0;c<cols;c++){
      if(rand()%(rand()%110+1)!=0)
	A(r,c)=Fractional(rand()%4-4,rand()%4+1);
      else
	A(r,c)=0;
    }
  }
  for(long c=0;c<cols;c++){
    //A(rows-3,c)=0;
  }
  FMatrix Asave(A);
  myDebug()<<"A="<<A.printS();
  bool detnotone=false;
  long rank;
  IMatrix V(A.decompAVeqB(rank,detnotone)); 
  A.addRows(cols-rank);
  
  myDebug()<<"rank="<< rank;
  myDebug()<<"A="<<A.printS();
  myDebug()<<"V="<<V.printS();
  
  FMatrix B=(Asave*V).Td();
  
  myDebug()<<"B="<<B.printS();
  
  long det=determinant(V.data(),cols);
  myDebug()<<"det="<<det;
  if(!detnotone && det!=1) {myDebug()<<"error det"<<det;exit(1);};
  if(detnotone && det<1) {myDebug()<<"error det"<<det;exit(1);};
  if(Fnorm2(B-A)!=0){myDebug()<<"error";exit(1);}
  
  exit(0);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void testhypercrossing()
{

  try{
  


    PeriodicHyperPlane pl1(3);
    PeriodicHyperPlane pl2(3);
    if(0){
      // axis E
      pl1.reset();
      pl1.R.addColumn3(0,0,1);
      
      pl1.O.setColumn3(0,2,1,5,3);
      
      pl1.Delta.addColumn3(2,1,0,3);
      pl1.Delta.addColumn3(1,2,0,3);

      pl2.reset();
      pl2.R.addColumn3(1,0,0);
      pl2.R.addColumn3(0,1,0);
      pl2.R.addColumn3(0,0,1);
    }
    if(0){
      // axis point
      pl1.reset();
      pl1.R.addColumn3(0,0,1);
      pl1.O.setColumn3(0,2,1,5,3);
      pl1.Delta.addColumn3(2,1,0,3);
      pl1.Delta.addColumn3(1,2,0,3);

      pl2.reset();
      pl2.O.setColumn3(0,3,3,0,3);
    }
    if(0){
      // axis axis paral
      pl1.reset();
      pl1.R.addColumn3(0,0,1);
      pl1.O=FMatrix::vector3(2,1,5,3);
      pl1.Delta.addColumn3(2,1,0,3);
      pl1.Delta.addColumn3(1,2,0,3);

      pl2.reset();
      pl2.R.addColumn3(0,0,1);
      pl2.O=FMatrix::vector3(1,1,0,1);
      pl2.Delta.addColumn3(1,0,0,2);
      pl2.Delta.addColumn3(0,1,0,2);
    }
    if(0){
      // axis axis notparal
      pl1.reset();
      pl1.R.addColumn3(0,0,1);
      pl1.O.setColumn3(0,2,1,5,3);
      pl1.Delta.addColumn3(2,1,0,3);
      pl1.Delta.addColumn3(1,2,0,3);

      pl2.reset();
      pl2.R.addColumn3(2,1,0);
      pl2.O.setColumn3(0,0,0,0,1);
      pl2.Delta.addColumn3(0,2,0,2);
      pl2.Delta.addColumn3(0,0,1,2);
    }
    if(0){
      // axis mirror par
      pl1.reset();
      pl1.R.addColumn3(2,1,0);
      pl1.O.setColumn3(0,0,0,0,1);
      pl1.Delta.addColumn3(0,2,0,2);
      pl1.Delta.addColumn3(0,0,1,2);
      pl2.reset();
      pl2.R.addColumn3(1,0,0);
      pl2.R.addColumn3(0,1,0);
      pl2.O.setColumn3(0,0,0,2,4);
      pl2.Delta.addColumn3(0,0,1,4);
    }
    if(0){
      // axis mirror notpar 
      pl1.reset();
      pl1.R.addColumn3(0,0,1);
      pl1.O.setColumn3(0,0,0,0,1);
      pl1.Delta.addColumn3(1,0,0,2);
      pl1.Delta.addColumn3(0,1,0,2);
      pl2.reset();
      pl2.R.addColumn3(1,0,0);
      pl2.R.addColumn3(0,1,0);
      pl2.O.setColumn3(0,0,0,2,4);
      pl2.Delta.addColumn3(0,0,1,4);
    }
    if(0){
      // mirror E 
      pl1.reset();
      pl1.R.addColumn3(1,0,0);
      pl1.R.addColumn3(0,1,0);
      pl1.O.setColumn3(0,0,0,2,4);
      pl1.Delta.addColumn3(0,0,1,4);
      pl2.reset();
      pl2.R.addColumn3(1,0,0);
      pl2.R.addColumn3(0,1,0);
      pl2.R.addColumn3(0,0,1);
    }
    if(0){
      // mirror point 
      pl1.reset();
      pl1.R.addColumn3(1,0,0);
      pl1.R.addColumn3(0,1,0);
      pl1.O.setColumn3(0,0,0,2,4);
      pl1.Delta.addColumn3(0,0,1,4);
      pl2.reset();
      pl2.O.setColumn3(0,0,0,3,3);
      pl2.Delta.addColumn3(1,1,1,1);
    }
    if(0){
      // mirror mirror par
      pl1.reset();
      pl1.R.addColumn3(1,0,0);
      pl1.R.addColumn3(0,1,0);
      pl1.O.setColumn3(0,0,0,2,4);
      pl1.Delta.addColumn3(0,0,1,4);

      pl2.reset();
      pl2.R.addColumn3(1,0,0);
      pl2.R.addColumn3(0,1,0);
      pl2.O.setColumn3(0,0,0,0,7);
      pl2.Delta.addColumn3(0,0,2,3);
    }
    if(0){
      // mirror mirror notpar
      pl1.reset();
      pl1.R.addColumn3(1,0,0);
      pl1.R.addColumn3(0,1,0);
      pl1.O.setColumn3(0,0,0,2,4);
      pl1.Delta.addColumn3(0,0,1,4);

      pl2.reset();
      pl2.R.addColumn3(1,1,0);
      pl2.R.addColumn3(0,0,1);
      pl2.O.setColumn3(0,0,0,0,7);
    }
    if(1){
      pl1.reset();
      pl1.Delta.addColumn3(1,0,0,2);
      pl1.Delta.addColumn3(0,1,0,2);
      pl1.Delta.addColumn3(0,0,1,2);
      pl1.O.addColumn3(0,0,0,1);
      pl2.reset();
      pl2.R.addColumn3(1,1,1);
      pl2.Delta.addColumn3(1,-1,0,1);
      pl2.Delta.addColumn3(1,0,-1,1);
      pl2.Delta.addColumn3(1,1,1,2);
      pl2.O.addColumn3(0,0,0,1);
      pl2.O.addColumn3(1,0,0,1);
      pl2.O.addColumn3(-1,0,0,1);
      //      pl2.normalize();
      
    }



    if(1){
      myDebug()<<"pl1="<<pl1.printS();
      myDebug()<<"pl2="<<pl2.printS();
      if(pl1.commonPoints(pl2).isEmpty()){
	myDebug()<<"pm="<<pl1.printS();
      }else{
	myDebug()<<"NO COMMON POINTS";
      }
      
    }

    if(0){
      PeriodicHyperPlane pm;
      //      pm.R.addMatrix(IMatrix::vector3(0,0,1));
      pm.O.addMatrix(FMatrix::vector3(0,0,0,2));
      pm.O.addMatrix(FMatrix::vector3(1,0,0,2));
      pm.O.addMatrix(FMatrix::vector3(2,0,0,2));
      pm.O.addMatrix(FMatrix::vector3(3,0,0,2));
      pm.O.addMatrix(FMatrix::vector3(2,1,0,2));
      pm.O.addMatrix(FMatrix::vector3(2,-1,0,2));
      //      pm.O.addMatrix(FMatrix::vector3(1,1,0,2));
      //      pm.O.addMatrix(FMatrix::vector3(1,-1,0,2));
      pm.Delta.addMatrix(FMatrix::vector3(1,1,0,1));
      pm.Delta.addMatrix(FMatrix::vector3(1,-1,0,1));
      myDebug()<<"pm="<<pm.printS();
      pm.normalize();
      myDebug()<<"pm="<<pm.printS();
    }



    
    exit(0);

  }catch(Except &ex){
    myDebug()<<ex.msg();
    exit(1);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main(int argc,const char **argv)
{

  

  CIFData cifdata;
  CIFDriver cifdriver;
  String file="";
  long i=1;
  bool oprint=false;
  bool osymextract=false;
  long idata=-1;
  long seed=-1;
  long settingindex=-1;
  long itc=-1;
  String IT="";
  String hallsymboltest="";
  String setting="default";
  bool mixit=false;
  while(i<argc){
    //printf("%s\n",argv[i]);
    if(String(argv[i])=="-debugp"){
      cifdriver.setTraceParsing(true);
    }else if(String(argv[i])=="-print"){
      oprint=true;
    }else if(String(argv[i])=="-datablock"){
      i++;
      try{
	if(i>=argc) throw "";
	idata=String(argv[i]).convertToLong();
      }catch(...){
	usage();
	printf("-datablock needs integer argument\n\n");
	exit(1);
      }
    }else if(String(argv[i])=="-extract_sym"){
      osymextract=true;
    }else if(String(argv[i])=="-hall"){
      i++;
      try{
	if(i>=argc) throw "";
	hallsymboltest=String(argv[i]).stripBlanks(true);
      }catch(...){
	usage();
	printf("-hall needs Hall-symbol argument\n\n");
	exit(1);
      }
    }else if(String(argv[i])=="-mixit"){
      mixit=true;
    }else if(String(argv[i])=="-IT"){
      i++;
      try{
	if(i>=argc) throw "";
	IT=String(argv[i]).stripBlanks(true);
	hallsymboltest=SpaceGroup::Data::HallFromIT(IT);
      }catch(...){
	usage();
	printf("-IT needs IT-number-symbol argument\n\n");
	exit(1);
      }
      
    }else if(String(argv[i])=="-OC"){
      i++;
      try{
	if(i>=argc) throw "";
	setting=String(argv[i]).stripBlanks(true);
      }catch(...){
	usage();
	printf("-OC needs setting argument\n\n");
	exit(1);
      }
      
    }else if(String(argv[i])=="-OCN"){
      i++;
      try{
	if(i>=argc) throw "";
	settingindex=String(argv[i]).convertToLong();
      }catch(...){
	usage();
	printf("-OCN needs integer argument\n\n");
	exit(1);
      }
    }else if(String(argv[i])=="-ITC"){
      i++;
      try{
	if(i>=argc) throw "";
	itc=String(argv[i]).convertToLong();
      }catch(...){
	usage();
	printf("-ITC needs integer argument\n\n");
	exit(1);
      }
      
    }else if(String(argv[i])=="-seed"){
      i++;
      try{
	if(i>=argc) throw "";
	seed=String(argv[i]).convertToLong();
      }catch(...){
	usage();
	printf("-seed needs integer argument\n\n");
	exit(1);
      }
      
    }else if(String(argv[i])=="-debugl"){
      cifdriver.setTraceScanning(true); 
    }else if(String(argv[i])=="-h"){
      usage();exit(1);
    }else if(argv[i][0]=='-'){
      usage();
      printf("unknown argument: %s\n\n",argv[i]);
      exit(1);
    }else{
      file=argv[i];
    }
    i++;
  }

  if(seed>=0) srand(seed);

  if(0) testdecompAVeqB();
  if(0) testdecompAVeqBF();
  if(0) testhypercrossing();

  using namespace SpaceGroup;

  if(!hallsymboltest.empty()){
    //myDebug()<<hallsymboltest;
    try{
      HallSymbol hallSymbol(hallsymboltest);
      ValueArray<SeitzMatrix> ops;

      if(settingindex>=0) setting=SpaceGroup::Data::settingByIndex(IT,settingindex);


      String it=IT;
      if(itc>=0){
	it=(IT+":"+Data::settingByIndex(IT,itc));
	if(it.find(":")==it.length()-1) it=IT;
	hallSymbol.fromSymbol(Data::HallFromIT(it));
      }
      ops=hallSymbol.operations();
      
      myDebug()<<"Input:"<<hallSymbol.symbol()<<hallSymbol.bravaisLatticeName()
	       <<hallSymbol.crystalSystemName()<<it
	       <<(hallSymbol.isLayerGroup()?" Layer Group":"");
      myDebug();

      for(long i=0;i<ops.size();i++){
	ops[i].analyse();
	myDebug()<<ops[i].symbol();
	myDebug()<<ops[i].printWwS();
      }

      myDebug();
      
      LatticeParameters latparam;
      latparam.setLatticeConstants( 5.0+2*(rand()%20000)*0.0001,
				    5.0+2*(rand()%20000)*0.0001,
				    5.0+2*(rand()%20000)*0.0001);
      
      latparam.setAxisAngles( 90+59.99*(rand()%10000)/10000.-30.0,
			      90+59.99*(rand()%10000)/10000.-30.0,
			      90+59.99*(rand()%10000)/10000.-30.0);
      //myDebug()<<latparam.printS();

      String warnings=hallSymbol.checkLatticeParameter(latparam);
      if(!warnings.empty()){
	myDebug()<<warnings;
	// the user said take it
	hallSymbol.checkLatticeParameter(latparam,true);
      }

      myDebug()<<latparam.printS();


      Matrix3x3 cell=latparam.getConvCell(hallSymbol.crystalSystem(),
					  hallSymbol.bravaisLattice());
      myDebug()<<"cell=\n"<<cell.printS("%22.12f");
	

      Matrix3x3 S=latparam.getMetric();
      myDebug()<<"S=\n"<<S.printS("%22.12f");

      HallSymbol hallsout;
      SeitzMatrix Stransform;
      String spgrnum;
      SeitzMatrix::analyseGroup(hallsout,Stransform,spgrnum,ops,setting,mixit);
      myDebug()<<"Output:"<<hallsout.symbol()<<hallsout.bravaisLatticeName()
	       <<hallsout.crystalSystemName()<<spgrnum;
      myDebug();
      myDebug()<<"Stransform"<<Stransform;

      latparam.convertLatticeParameters(S,Stransform);

      
      myDebug()<<latparam.printS();

      Matrix3x3 cellout=latparam.getConvCell(hallsout.crystalSystem(),
					     hallsout.bravaisLattice());
      myDebug()<<"cell=\n"<<cellout.printS("%22.12f");


      Matrix3x3 U=cell*Stransform.W().toMatrix3x3()*cellout.inverse();
      myDebug()<<"E'^T=E^T*U, U=\n"<<U.printS("%12.5f")<<U.determinant();
      if( (fabs(U.determinant()-1.)>1.0e-6) ||
	  !(U*U.transposed()-Matrix3x3::unitMatrix()).isZero(1.0e-6)){
	myDebug()<<"U^T*U=\n"<<(U*U.transposed()).printS("%12.5f")<<
	  Sprintf("det=%30.15f",U.determinant());
	throw Except("Transformation of coordinate system is not orthogonal?");
      }
      
    }catch(Except &ex){
      printf("error: %s\n",ex.msg());
      exit(1);
    }
    printf("done\n");
    exit(0);
  }

  if(file.empty() ){
    usage();
    printf("Filename missing\n\n");
    exit(1);
  }
  if(oprint && osymextract ){
    usage();
    printf("Warning: -print overwrites -extraxt_sym \n\n");
    exit(1);
  }

  try{
    if(file=="-"){
      cifdriver.parse(&cifdata,"-");
    }else{
      cifdriver.parse(&cifdata,file());
    }
  }catch(Except &ex){
    fprintf(stderr,"%s\n",ex.msg());
    exit(1);
  }
  
  //printf("%s",cifdata.printS()());
  if(oprint) {
    cifdata.print();
    return 0;
  }

  

  try{

    
    ValueArray<String> datablocknames=cifdata.dataBlockNames();
    printf("\n------------------------------------------------------\n"
	   "  Available datablocks(records) in CIF file '%s'\n",file());
    for(long i=0;i<datablocknames.size();i++){
      printf("  %ld: %s\n",i,datablocknames.at(i)());
    }
    printf("\n------------------------------------------------------\n");
    if(idata<0){
      while(1){
	printf("Please give a number for the wanted record[0]: ");
	String in;
	getLine(stdin,in);
	in.stripBlanks(true);
	if(in.empty()){
	  idata=0;
	  break;
	}
	try{
	  idata=in.convertToLong();
	  if(0<=idata && idata<datablocknames.size()) break;
	}catch(...){continue;}
      }
    }
    printf("\nrecord: %s\n\n",datablocknames.at(idata)());

    const CIFData::DataBlock *datablock
      =cifdata.dataBlock(datablocknames.at(idata));

    CIFMetaDict dict=cifdata.metaDict();

    allOfSingleSection(datablock,dict,CIFMetaDict::ChemicalName);
    allOfSingleSection(datablock,dict,CIFMetaDict::ChemicalFormula);
      


    printf("\nlattice constants: %s [Ang]\n",
	   datablock->latticeConstants(dict).printS("%12.6f")());
    printf("axes angles      : %s\n",
	   datablock->axesAngles(dict).printS("%12.6f")());;
    try{
      double vol=datablock->cellVolume(dict);
      printf("cell volume      : %12.5f [Ang^3]\n",vol);
    }catch(...){};
    try{
      long N=datablock->cellFormulaUnits(dict);
      printf("formula units    : %ld\n",N);
    }catch(...){};
    try{
      String s=datablock->cellSpecialDetails(dict);
      printf("special details  : %s\n",s());
    }catch(...){};

    //
    // symmetry details
    //
    
    printf("\nSpacegroup information\n");
    String crystalsystem=datablock->crystalSystem(dict);
    if(!crystalsystem.empty()){
      printf("    Crystal system: %s\n",crystalsystem());
    }
    long spgrnum=datablock->spaceGroupNumber(dict);
    if(spgrnum>0){
      printf("    Number        : %ld\n",spgrnum);
    }
    String spgrhall=datablock->spaceGroupNameHall(dict);
    if(!spgrhall.empty()){
      printf("    Name-Hall     : %s\n",spgrhall());
    }
    String spgrhm=datablock->spaceGroupNameHM(dict);
    if(!spgrhm.empty()){
      printf("    Name-H-M      : %s\n",spgrhm());
    }

    ValueArray<Pair<long,String> > spgrgenxyz
      =datablock->spaceGroupGeneratorsXYZ(dict);
    if(spgrgenxyz.size()){
      printf("    Generat./Oper.: \n");
      for(long i=0;i<spgrgenxyz.size();i++){
	SeitzMatrix op(spgrgenxyz.at(i).v2);
	printf("    %ld: %s\n",spgrgenxyz.at(i).v1,spgrgenxyz.at(i).v2());
	//op.toXYZSymbol()()
      }
    }
    printf("\n");
    
    if(!spgrhall.empty()){
      printf("    Name-Hall     : %s\n",spgrhall());
    }
    //
    // atom table
    //

    const CIFData::DataTable *table=datablock->atomTable(dict);
    //for(long i=0;i<table->columns();i++){
    //  myDebug()<<"table tags"<<table->normalizedTag(i);
    //}
    bool haswycksymbol=table
      ->contains(dict.value(CIFMetaDict::AtomSiteWyckoffSymbol));
    //  myDebug()<<haswycksymbol;
    bool hasoccupancy=table
      ->contains(dict.value(CIFMetaDict::AtomSiteOccupancy));
    bool hasmult=table
      ->contains(dict.value(CIFMetaDict::AtomSiteSymmetryMultiplicity));
    
    printf("------------------------------------------------%s%s%s\n",
	   (haswycksymbol?"----------":""),
	   (hasoccupancy?"----------":""),
	   (hasmult?"----------":"")
	   );
    printf("El              fract_x    fract_y    fract_z   %s %s %s\n",
	   (haswycksymbol?"WyckSymbol":""),
	   (hasoccupancy?"Occupancy":""),
	   (hasmult?"Multipli.":"")
	   );
    printf("------------------------------------------------%s%s%s\n",
	   (haswycksymbol?"----------":""),
	   (hasoccupancy?"----------":""),
	   (hasmult?"----------":"")
	   );
    for(long i=0;i<table->rows();i++){
      printf("%-10s %33s",table->atomLabel(i,dict)(),
	     table->atomFracPosition(i,dict).printS("%10.5f")());
      if(haswycksymbol){
	printf("%10s",table->atomWyckoffSymbol(i,dict)());
      }
      if(hasoccupancy){
	try{
	  printf("%10.5f",table->atomOccupancy(i,dict));
	}catch (Except &ex){
	  printf("%10s","?");
	}
      }
      if(hasmult){
	printf("%10ld",table->atomSymmetryMultiplicity(i,dict));
      }
 

      printf("\n");
    }

    {
      ValueArray<SeitzMatrix> ops;
      for(long i=0;i<spgrgenxyz.size();i++){
	ops<<SeitzMatrix(spgrgenxyz.at(i).v2);
      }
      HallSymbol hallsout;
      SeitzMatrix Stransform;
      String spgrnum;
      bool mixit=false;
      myDebug()<<"settin="<<setting;
      SeitzMatrix::analyseGroup(hallsout,Stransform,spgrnum,ops,setting,mixit);
      myDebug()<<"Output:"<<hallsout.symbol()<<hallsout.bravaisLatticeName()
	       <<hallsout.crystalSystemName()<<spgrnum;

    }



  }catch(Except &ex){
    fprintf(stderr,"%s\n",ex.msg());
    return 1;
  }


  return 0;

}

/*EOF*/ 
