/*************************************************************************
 * @(#)Assorted.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Oct 2008
 *************************************************************************
 */
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h> 
#include <wordexp.h>
#include <errno.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <libgen.h>
#include <dirent.h>



#include "Assorted.hh"
#include "Except.hh"
#include "FString.hh"
#include "F90Interface.hh"
//#include "MyDebug.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */

static String ioerror(long err)
{
  switch (err){
  case ENOENT:
  case ENOTDIR:
    return "Path/file does not exist/is invalid.";
  case EACCES:
    return "Access denied.";
  case EPERM:
    return "Permisson denied.";
  case EEXIST:
    return "Path/file does allready exist.";
  case EISDIR:
    return "Can't create file in this  directory.";
  case ETXTBSY:
    return "Can't write on file, is busy.";
  case ENOSPC:
    return "No space left on device.";
  case ELOOP:
    return "Path contains a symbolic link which forms a loop.";
  case (EROFS):
    return "File system is read only!";
  default:
    return Sprintf("%d: %s\n",err,strerror(err));
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void die(const String &msg,int exitcode)
{
  if(msg.empty()) {
    FPLOF90Interface::PRINT("I must die!\n");
    fprintf(stderr,"I must die!\n");
  }else{
    FPLOF90Interface::PRINTF("%s\n",msg());
    fprintf(stderr,"%s\n",msg());
  }
  if(exitcode>=0){ exit(exitcode); }
}
// ============================================================================
// 
// ============================================================================

void die(const Except &ex,int exitcode)
{
  die(ex.msg(),exitcode);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FILE *openforread(String const &fname)
{
  FILE *FH;
  FileExistence ret=pathExistence(fname);
  
  if( ret==DoesNotExist  )
    throw Except(Sprintf("File %s does not exist.\n",fname()));
  
  if( ret!=RegularFile && ret!= SymLink )
    throw Except(Sprintf("File %s is not a regular file/symbolic link.\n",
			 fname()));
  
  errno=0;
  if((FH=fopen(fname(),"r"))==NULL) 
    throw Except(Sprintf("Error opening file '%s' for read: %s\n"
			    ,fname(),ioerror(errno)()));
    
  return FH;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FILE *openforwrite(String const &fname,const char *mode)
{
  FILE *FH;
  errno=0;
  if((FH=fopen(fname(),(mode[0]=='\0'?"w":mode)))==NULL) 
    throw Except(Sprintf("Error opening file '%s' with mode %s: %s\n"
			 ,fname(),mode,ioerror(errno)()));
  
  
  return FH;
}

/* ***********************************************************************
 * copy file1 onto file2
 * ***********************************************************************
 */


void backUp(String const &file1,String const &file2)
{
  //NSF("backup");

  FILE *FH,*FHo;

  FH=openforread(file1);
  FHo=openforwrite(file2);
  

  String s;
  while (getLine(FH,s)){
    (void)fprintf(FHo,"%s",s());
  }
  (void)fclose(FH);
  (void)fclose(FHo);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FILE *opentempfile(String &tempname,const char *mode)
{

  const char *tmpdir=getenv("TMPDIR");
  if(tmpdir==NULL){
    tmpdir=P_tmpdir;
  }

  tempname=String(tmpdir)+"/"+tempname;
  
  int fd=mktmpFileName(tempname);

  FILE *FH;
  errno=0;
  if((FH=fdopen(fd,mode))==NULL) 
    throw Except(Sprintf("Error opening file '%s' for read: %s\n"
			 ,tempname(),ioerror(errno)()));

  
  return FH;
}


/* ***********************************************************************
 *
 * create tmp name 
 * ***********************************************************************
 */
int mktmpFileName(String &templat)
{
  templat=templat+"XXXXXX";

  //mode_t oldmask=umask(S_IRUSR | S_IWUSR);

  errno=0;
  int fd=mkstemp(templat.data());
  //umask(oldmask);

  if(fd==-1){
    throw Except(Sprintf("Error creating temporary  file '%s': %s\n"
			 ,templat(),ioerror(errno)()));
  }
  
  return fd;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void removeDirectoryNonRecursive(String const &dirname,long prot)
{// remove the dir including all files in it, but not recursivly
  const char *subfunc="removeDirectory";  
  FileExistence ret=pathExistence(dirname);
  switch(ret)
    {
    case DoesNotExist:
      if(prot){
	(void)printf("Directory '%s' does not exist ... being done here.\n",dirname());
      }
      return;
    case RegularFile:
      throw Except(Sprintf("error(%s): Cannot delete directory '%s'!\n"
			   "It is a file!\n",subfunc,dirname()));
      
    case SymLink:
    case SomethingElse:
      throw
	Except(Sprintf("error(%s): Cannot delete "
				"directory '%s'!\n"
				"Something else with this name exists though!\n"
				,subfunc,dirname()));
    case Directory:
      if(prot){
	(void)printf("Will delete directory '%s'.\n",dirname());
      }
      break;
    }
  

  if(ret==Directory){


    struct dirent *dfile;
    DIR *ddir;

    ddir=opendir(dirname());

    while ( (dfile = readdir(ddir) )!=NULL){
      String name=Sprintf("%s/%s", dirname(), dfile->d_name);
      if((ret=pathExistence(name))==RegularFile){
	//if(dfile->d_name[0]=='+' || dfile->d_name[0]=='='){
	errno=0;
	if(remove(name())!=0 && errno!= ENOENT){
	  throw Except(Sprintf("Error removing file '%s': %s\n"
			       ,name(),ioerror(errno)()));
	  
	}
      }
    }
    closedir(ddir);
    errno=0;
    if(rmdir(dirname())!=0 && errno!= ENOENT){
      throw Except(Sprintf("Error removing directory '%s': %s\n"
			   ,dirname(),ioerror(errno)()));
      
    }
    

  }

  if(prot){
    (void)printf("Directory '%s' removed.\n",dirname());
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void createDirectory(String const &dirname,long prot)
{
  const char *subfunc="createDirectory";  

  FileExistence ret=pathExistence(dirname);
  
  switch(ret)
    {
    case DoesNotExist:
      if(prot){
	(void)printf("Will create directory '%s'.\n",dirname());
      }
      break;
    case RegularFile:
      throw
	Except(Sprintf("error(%s): Cannot create "
				" directory '%s'!\n"
				"File of this name allready exists!\n"
				,subfunc,dirname()));
    case Directory:
      if(prot){
	(void)printf("Directory '%s' exists.\n",dirname());
      }
      return;
      break;
    case SymLink:
    case SomethingElse:
      throw
	Except(Sprintf("error(%s): Cannot create "
				"directory '%s'!\n"
				"Something with this name allready exists!\n"
				,subfunc,dirname()));
    }
  
  if(ret==DoesNotExist){
    errno=0;
    if((mkdir(dirname(),0755))==-1)
      throw Except(Sprintf("Error creating directory"
				    " '%s': %s\n"
				    ,dirname(),ioerror(errno)()));
    
    
  }

  if(prot){
    (void)printf("Directory '%s' created.\n",dirname());
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */


/* ***********************************************************************
 *
 * ret: 0 if pathname does not exist
 * ret: 1 if pathname does exist and is a regular file
 * ret: 2 if pathname does exist and is dir
 * ret: 3 if pathname does exist and something else
 * ret: 4 is symlink , switched off, since not used and not as portable
 * ***********************************************************************
 */

FileExistence pathExistence(String const &pathname)
{
  struct stat buf;
  if(lstat(pathname(), &buf)!=0)
    return DoesNotExist;
  
  if (S_ISDIR(buf.st_mode) )return Directory;
  if (S_ISREG(buf.st_mode) )return RegularFile;
/*   if (S_ISLNK(buf.st_mode) )return SymLink; */
  if((buf.st_mode & S_IFMT)==S_IFLNK)return SymLink;
  return SomethingElse;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
//todo: use fread? as in BP/nused/sb_read_buffered -> put in FString
bool getLine(FILE *FH,String &line)
{
  const long l=20000;
  char buf[l+1];
  line="";
  bool ret=false;
  bool first=line.internalSize()>1;
  while(1){
    if(first){
      first=false;
      if(line.fgets(FH)==NULL) break;
      ret=true;
      if(line[line.length()-1]=='\n') break;
    }else{
      if(fgets(buf,l,FH)==NULL) break;
      line+=buf;
      ret=true;
      if(buf[::strlen(buf)-1]=='\n') break;
    }
  }
  return ret;
}


/* *********************************************************************** 
 * remove a file with checking
 * *********************************************************************** 
 */


void removeFile(String const &fname)
{
  errno=0;
  if(remove(fname())!=0 && errno!= ENOENT){
    throw Except(Sprintf("Error removing file '%s': %s\n"
				  ,fname(),ioerror(errno)()));
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
extern void chDir(String const &dname)
{
  errno=0;
  if(chdir(dname()))
    throw Except(Sprintf("Error changing to directory '%s': %s\n"
				  ,dname(),ioerror(errno)()));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
extern int systemCall(String const &command)
{
  return system(command());
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
time_t modificationDate(String const &pathname)
{
  struct stat buf;
  errno=0;
  if(stat(pathname(), &buf)!=0)
    throw Except(Sprintf("Error getting information for "
				  "path '%s': %s\n"
				  ,pathname(),ioerror(errno)()));
  
  
  return buf.st_mtime;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void splitPath(String &dir,String &file,String const &path)
{
  
  StringList sl=path.breakup("/");
  
  file=sl[sl.size()-1];
  

  if(sl.size()==1){
    dir=".";
    file=path;
  }else{
    sl.remove(sl.size()-1);
    dir=sl.join("/");
  }
  return ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int saveCurrentDir()
{
  int fd;
  errno=0;
  if((fd=open(".",O_RDONLY))==-1){
    throw Except(Sprintf("Error saving current directory: %s\n"
			 ,ioerror(errno)()));
  }
  return fd;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void restoreCurrentDir(int fd)
{
  errno=0;
  if(fchdir(fd)==-1){
    close(fd);
    throw Except(Sprintf("Cannot change to saved directory: %s\n"
			 ,ioerror(errno)()));
  }
  close(fd);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int mySystem(const String &cmd,const StringList &args,
	      String &sstdout,String &sstderr)
{
  int fdpipe[2];
  int fdpipeerr[2];

  if(pipe(fdpipe)<0){
    printf("error opening a pipe\n");
    exit(127);
  }
  if(pipe(fdpipeerr)<0){
    printf("error opening a pipe for err\n");
    exit(127);
  }
  
  pid_t pid;
  if((pid=fork())<0){
    printf("cannot fork\n");
    exit(127);
  }else if(pid==0){ //we are the child
    close(fdpipe[0]);//close read end in child
    close(fdpipeerr[0]);//close read end in child
    if(fdpipe[1] != STDOUT_FILENO){
      if(dup2(fdpipe[1],STDOUT_FILENO)!=STDOUT_FILENO){
	printf("dup2 to stdout error\n");
	_exit(127);
      }
    }
    if(fdpipeerr[1] != STDERR_FILENO){
      if(dup2(fdpipeerr[1],STDERR_FILENO)!=STDERR_FILENO){
	printf("dup2 to stderr error\n");
	_exit(127);
      }
    }
    const char **argv=(const char **)malloc(sizeof(char*)*(args.size()+2));
    for(int i=0;i<args.size();i++){
      argv[i+1]=args[i]();
    }
    argv[0]=cmd();
    argv[args.size()+1]=NULL;
    execvp(cmd(),(char *const*)argv);
    printf("couldn't exec %s\n",cmd());
    _exit(127);
  }
  //parent
  close(fdpipe[1]); // close write end in parent
  close(fdpipeerr[1]); // close write end in parent
  
  char line[1025];
  FILE *FH=fdopen(fdpipe[0],"r");
  sstdout="";
  while(fgets(line,1024,FH)!=NULL){
    sstdout+=line;
  }
  fclose(FH);

  FH=fdopen(fdpipeerr[0],"r");
  sstderr="";
  while(fgets(line,1024,FH)!=NULL){
    sstderr+=line;
  }
  fclose(FH);


  int status;
  if( (pid=waitpid(pid,&status,0))<0){
    printf("waitpid error\n");
    exit(127);
  }
  if (WIFEXITED(status)) {
    return WEXITSTATUS(status);
  }else{
    return 127;
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void expandFileName(String &fname)
{
  wordexp_t p;
  if(wordexp(fname(), &p, 0)){
    throw Except(Sprintf("error in file name expansion (\"%s\")\n",fname()));
  }
  char **w=p.we_wordv;
  if(p.we_wordc!=1){
    throw Except(Sprintf("error in file name expansion (\"%s\")\n",fname()));
  }
  fname=w[0];
  // for (size_t i = 0; i < p.we_wordc; i++){
  //   printf("%s\n", w[i]);
  // }
  
  wordfree(&p);

}

/* ***********************************************************************
*
* ***********************************************************************
*/
String expandAndCreatePath(const String &path)
{
  // create and return path, on error Except is raised
  // expand shell variables, only the dir part is created, the file part, if
  // present is not created but returned as full path

  String home=path;
  expandFileName(home);

  String dir;
  String file;
  splitPath(dir,file,home);
  home="";
  StringList sl=dir.split("/");
  for(int i=-1;++i<sl.size();){
    home+="/"+sl[i];
    createDirectory(home,false);
  }
  
  return home+"/"+file;
}

/*EOF*/ 
