/* @(#)BandWeights.hh
 * author: magru@unas
 */

#ifndef _BANDWEIGHTS_HH
#define _BANDWEIGHTS_HH 1

#include "FString.hh"
#include "MACROS.hh"
#include "ValueArray.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class BandWeights{
public:

  // ----------------------------------------------------------------
  // state index into orbitals/state in weight file header with factor
  class State
  {
  public:
    State():index(-1),factor(1.){}
    State(long i,double f=1.):index(i),factor(f){}
  public:
    long index;
    double factor;
  };

  // ----------------------------------------------------------------
  // list of state indeces into orbitals/state in weight file header with factors
  // to be summed up and stored with name in new weight file
  class StateSum
  {
  public:
    StateSum():name("SUM"){};
    StateSum(const String &na):name(na){
      if(name.length()>17) name=name.subString(0,16);
    };
    

  public:
    String name;
    ValueArray<State> data;
  };
  
  
  // ----------------------------------------------------------------
  BandWeights();
  ~BandWeights();


  // ----------------------------------------------------------------
  // interactive session
  void writeTemplate(const String &statefile);

  // ----------------------------------------------------------------
  // accumulate all the orbitals/states in the header in to labels
  void extractHeaderInfo(const String &infile,long prot=0);
  // ----------------------------------------------------------------
  // read state definitions from file
  void readStateDefinitions(const String &statefile,String &infile,
			    String &outfile,
			    ValueArray<StateSum> &statesums,
			    ValueArray<double> &ewindow,
			    bool getfilenamesonly=false,long prot=0);
  // ----------------------------------------------------------------
  // do the actual reading and adding
  void addWeights(const String &infile,const String &outfile,
		  const ValueArray<StateSum> &statesums,
		  const ValueArray<double> &ewindow,long prot=1);

protected:
private:
  ROPROPERTY(headerready,headerready,bool);
  ROPROPERTY(nsite,nsite,long);
  ROPROPERTY(ef,ef,double);
  ROPROPERTY(nk,nk,long);
  ROPROPERTY(nvdim,nvdim,long);
  ROPROPERTY(nband,nband,long);
  ROPROPERTY(nspin,nspin,long);
  ROPROPERTY(ilower,ilower,long);
  ROPROPERTY(iupper,iupper,long);
  ROPROPERTY(nweights,nweights,long);
  ROCPROPERTY(labels,labels,StringList);
};


#endif /* _BANDWEIGHTS_HH */

