/*************************************************************************
 * @(#)Cmplx.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Mar 2010
 *************************************************************************
 */

#include "Cmplx.hh"
#include "FString.hh"
#include "MyDebug.hh"

const Cmplx Cmplx::ci=Cmplx(0.,1.);
const Cmplx Cmplx::c1=Cmplx(1.,0.);


// ==========================================================================
// 
// ==========================================================================

void MyMath::zrootstester()
{
  // test zroot
  ValueArray<Cmplx> a;
  ValueArray<Cmplx> roots;
  double tol=1.0e-12;
  double maxerr=0.;
  typedef  ValueArray<Cmplx> VACX;
  struct Aa{ VACX a; };
  Aa data[]={
    {VACX()<<1},
    {VACX()<<0<<1},
    {VACX()<<0<<0<<1},
    {VACX()<<0<<0<<0<<1},
    {VACX()<<1<<1},
    {VACX()<<1<<1<<1},
    {VACX()<<1<<1<<1<<1},
    {VACX()<<1<<1<<1<<1<<1},
    {VACX()<<1<<0<<1},
    {VACX()<<-1<<0<<1},
    {VACX()<<1<<0<<0<<1},
    {VACX()<<-1<<0<<0<<1},
    {VACX()<<1<<0<<0<<0<<1},
    {VACX()<<-1<<0<<0<<0<<1},
    {VACX()<<1<<0<<0<<0<<0<<1},
    {VACX()<<-1<<0<<0<<0<<0<<1},
    
    {VACX()<<0<<0<<1<<0<<1},
    {VACX()<<0<<0<<-1<<0<<1},
    
    {VACX()<<1<<2<<3<<4<<5},
    {VACX()<<1<<2<<3<<4<<-5},
  };
  int Naa=sizeof(data)/sizeof(Aa);
  
  bool errocc=false;
  for(int i=-1;++i<Naa;){
    a.clear(); a=data[i].a;
    myDebug()<<"coeffs="<<a;
    int n=a.size()-1;roots.resize(n);
    MyMath::zroots(a.constDataRef(),n,roots.dataRef(),true);
    
    
    for(int j=-1;++j<n;){
      Cmplx p=horner(roots.at(j),a.constDataRef(),n);
      maxerr=MyMath::max(maxerr,p.abs());
      if( p.abs()>tol ){
	errocc=true;
	myDebug()<<"testing root "<<j<<"="<<roots[j]<<"P(root)="<<p<<"ERROR?";
      }else{
	myDebug()<<"testing root "<<j<<"="<<roots[j]<<"P(root)="<<p;
      }
    }
  }
  if(errocc) throw Except(Sprintf("Some roots where not accurate or wrong! maxerr=%g",maxerr));
  die(Sprintf("Maxerr=%g",maxerr),0);
}

// ==========================================================================
// 
// ==========================================================================

static void laguer(const Cmplx *a,long m,Cmplx &x,long &its)
{
  // laguere method of finding a root of a polynomial
  // order m
  // coeffs a[0:m]
  // return number of iterations in its
  
  long iter,j;
  double abx,abp,abm,err;
  Cmplx  dx,x1,b,d,f,g,h,sq,gp,gm,g2;
  const double epss=std::numeric_limits<double>::epsilon()*200.0; 
  const long mr=8;
  const long mt=10;
  const long maxit=mt*mr;
  double frac[]={0.0,0.5,0.25,0.75,0.13,0.38,0.62,0.88,1.0};
  
  for(iter=1;iter<=maxit;iter++){
    its=iter;
    b=a[m];
    err=b.abs();
    d=f=Cmplx();
    abx=x.abs();
    for(j=m-1;j>=0;j--){
      f=x*f+d;
      d=x*d+b;
      b=x*b+a[j];
      err=b.abs()+abx*err;
    }
    err*=epss;
    if(b.abs()<=err) { return; }
    
    g=d/b;
    g2=g*g;
    h=g2-2.0*f/b;
    
    sq=sqrt((m-1.)*(m*h-g2));
    gp=g+sq;
    gm=g-sq;
    abp=gp.abs();
    abm=gm.abs();
    if(abp<abm) { gp=gm; }
    if(MyMath::max(abp,abm)>0.0){
      dx=1.*m/gp;
    }else{
      dx=(1.0+abx)*exp(Cmplx::ci*iter);
    }
    
    x1=x-dx;
    if(x==x1){return;}
    if((iter%mt)){
      x=x1;
    }else{
      x=x-dx*frac[iter/mt];
    }
  }
  
  throw Except(" Too many iterations in laguer\n Please report!");
  
}

// ==========================================================================
// 
// ==========================================================================

void MyMath::zroots(const Cmplx *a,long m,Cmplx *roots,bool polish)
{
  // calcualte the complex roots of a compelx polynomial
  // order m
  // coeffs a[0:m]
  // roots needs to be root[0:m-1]
  
  const double eps=std::numeric_limits<double>::epsilon()*10.0; 
  
  long its;
  ValueArray<Cmplx> ad(m+1,Cmplx());
  Cmplx x,b,c;
  for(int j=0;j<=m;j++) ad[j]=a[j];
  
  for(int j=m;j>=1;j--){
    
    x=Cmplx();
    
    laguer(ad.constDataRef(),j,x,its);
    
    if(fabs(x.imag())<=2.0*eps*fabs(x.real())) { x.imag()=0.; }
    
    roots[j-1]=x;
    b=ad.at(j);
    for(int jj=j-1;jj>=0;jj--){
      c=ad.at(jj);
      ad[jj]=b;
      b=x*b+c;
    }
    
  }
  if(polish){
    for(int j=0;j<m;j++){
      laguer(a,m,roots[j],its);
    }
  }
  
  for(int j=1;j<m;j++){
    x=roots[j];
    int i=-1;
    for(i=j-1;i>=0;i--){
      if(roots[i].real()<= x.real()) { break; }
      roots[i+1]=roots[i];
    }
    roots[i+1]=x;
  }
}

/* ***********************************************************************
 * Clebsch-Gordon coefficients for coupling of l and s=1/2
 * ***********************************************************************
 */
double MyMath::ClebschGordonChi(long kap,long mu,long s)
{
  if(abs(s)!=1){
    return 0.;
  }
  long j=2*abs(kap)-1;
  if(!kap || abs(mu)>j) return 0.;
  
  double degk=2*kap+1.0;
  double ks=dSign(1.,kap);
  
  
  if(kap<0){
    if(mu==j){
      return (1+s)*0.5;
    }else if(mu==-j){
      return (1-s)*0.5;
    }
  }

  if(s==1){
    return -ks*sqrt( (1.0-mu/degk)*0.5 );
  }else{
    return sqrt( (1.0+mu/degk)*0.5 );
  }
    

}
// ============================================================================
// 
// ============================================================================

Cmplx sqrt(const Cmplx &z)
{
  
  if((z.re == 0.0) && (z.im == 0.0)) return Cmplx();

  Cmplx c;
  double x,y,w;

  x=fabs(z.re);
  y=fabs(z.im);
  if(x >= y){
    double r=y/x;
    w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
  } else {
    double r=x/y;
    w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
  }
  if(z.re >= 0.0){
    c.re=w;
    c.im=z.im/(2.0*w);
  }else{
    c.im=(z.im >= 0) ? w : -w;
    c.re=z.im/(2.0*c.im);
  }
  return c;
  
}

// ============================================================================
// 
// ============================================================================

Cmplx log(const Cmplx &other)
{
  if(other==Cmplx()) throw Except("Error in log(Cmplx): argument=0!\n");
  Cmplx c;
  c.im=::atan2(other.im,other.re);
  if(fabs(other.re)>fabs(other.im)){
    c.re=log(other.re/::cos(c.im));
  }else{
    c.re=log(other.im/::sin(c.im));
  }
  return c;
}
// ============================================================================
// 
// ============================================================================

String Cmplx::toString(const char *fmt)const
{
  if(re==0 && im==0){ return "0";
  }else if(im==0.){
    long l=lRound(re);
    if(fabs(re-l)<1.0e-12){
      if(l==1){
	return "1";
      }else{
	return Sprintf("%24ld",l,re).stripBlanks();
      }
    }else{
      return Sprintf(fmt,re).stripBlanks();
    }
  }else if(re==0.){
    long l=lRound(im);
    if(fabs(im-l)<1.0e-12){
      if(l==1){
	return "i";
      }else{
	return Sprintf("%24ld*i",l,im).stripBlanks();
      }
    }else{
      return Sprintf(fmt,im).stripBlanks()+"*i";
    }
  }else{
    long lr=lRound(re);
    long li=lRound(im);
    String s("(");
    if(fabs(re-lr)<1.0e-12){
      s+=Sprintf("%24ld",lr).stripBlanks();
    }else{
      s+=Sprintf(fmt,re).stripBlanks();
    }
    s+=",";
    
    if(fabs(im-li)<1.0e-12){
      s+=Sprintf("%24ld",li).stripBlanks();
    }else{
      s+=Sprintf(fmt,im).stripBlanks();
    }
    s+=")";
    
    return s;
  }
}
// ============================================================================
// 
// ============================================================================

MyDebug operator<<(MyDebug dbg,const Cmplx &d)
{
  return dbg<<d.toString();
}

/*EOF*/ 
