/* @(#)CoeffBasisInfo.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _COEFFBASISINFO_H
#define _COEFFBASISINFO_H 1


#include "FString.hh"
#include "MyDebug.hh"
#include "ValueArray.hh"
#include "MACROS.hh"

namespace XFPLO{

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class CoeffBasisInfo{
 public:
  CoeffBasisInfo();
  ~CoeffBasisInfo();
  
  void clean();


  inline long nsite()const{return pnsite;}
  inline const String &element(long isa)const{
    if(isa>=0 && isa<pnsite)
      return pelements[isa];
    else
      return defaultEl;
  };
  void setNsite(long n);
  inline StringList &elements(){return pelements;}
  inline ValueArray<long> &nRv(){return pnRv;}
  inline ValueArray<long> &blockstart (){return pblockstart;}
  inline ValueArray<long> &blocklength(){return pblocklength;}
  inline ValueArray<long> &nnRv (long isa){return pnnRv [isa];}
  inline ValueArray<long> &lRv  (long isa){return plRv  [isa];}
  inline ValueArray<long> &degRv(long isa){return pdegRv[isa];}
  inline ValueArray<long> &jRv  (long isa){return pjRv  [isa];}

  inline long nRv(long isa)const{return pnRv.at(isa);};
  inline long nnRv (long ivr,long isa)const{return pnnRv .at(isa).at(ivr);}
  inline long lRv  (long ivr,long isa)const{return plRv  .at(isa).at(ivr);}
  inline long degRv(long ivr,long isa)const{return pdegRv.at(isa).at(ivr);}
  inline long jRv  (long ivr,long isa)const{return pjRv  .at(isa).at(ivr);}
  inline long kapRv(long ivr,long isa)const{
    long l=plRv.at(isa).at(ivr);
    long j=pjRv.at(isa).at(ivr);
    return (j+1)/2*(2*l-j);
  };
  
  inline long blockstart(long isa)const{return pblockstart.at(isa);}
  inline long blocklength(long isa)const{return pblocklength.at(isa);}
    
  void getCBIFromLabels(const StringList &labels);

 private:
  StringList pelements;
  long    pnsite;
  ValueArray<long> pnRv;
  ValueArray<ValueArray<long> > pnnRv;
  ValueArray<ValueArray<long> > plRv;
  ValueArray<ValueArray<long> > pdegRv;
  ValueArray<ValueArray<long> > pjRv;
  ValueArray<long> pblockstart;
  ValueArray<long> pblocklength;
  PROPERTY(nvdim,nvdim,Nvdim,long);
  PROPERTY(FREL,FREL,FREL,bool);
  PROPERTY(isready,isReady,IsReady,bool);

  static const String defaultEl;
};

}
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _COEFFBASISINFO_H */

