/* @(#)F90Interface.hh
 * author: magru@unas
 */

#ifndef _F90INTERFACE_HH
#define _F90INTERFACE_HH 1

#include <stdio.h>
#include "FString.hh"

namespace FPLOF90Interface{
  
  
  
  
  // ============================================================================
  // typedefs for F90 interfacing
  // ============================================================================
  typedef void (*F90PRINTFUNC)(const char *str);
  
  // ============================================================================
  // fallback function to work with c/c++
  // ============================================================================
  
  //inline void localprint(const char *s){printf("localprint: %s",s);}
  inline void localprint(const char *s){printf("%s",s);}
  
  
  
  
  
  // ============================================================================
  // wrapper class to fork into the print functions
  // ============================================================================
  
  class F90Print{
  public:
    F90Print():pfunc(localprint){}
    
    void print(const char *s){pfunc(s);}
    
    void setPrintFunc(F90PRINTFUNC func){pfunc=func;}
    
    F90PRINTFUNC pfunc;
  };
  
  // ============================================================================
  // one global instance of this class
  // if we call from F90 linked stuff, we set this variable accordingly to
  // invoke the fortran interface
  // ============================================================================
  
  extern F90Print f90print;
  
  
  // ============================================================================
  // this is the actual print function
  // ============================================================================
  
  inline void PRINT(const  char *s)
  {
    f90print.print(s);
  }
  inline void PRINT(const String &s)
  {
    f90print.print(s());
  }

#define PRINTF(fmt,...) PRINT(Sprintf(fmt,__VA_ARGS__));
#define FPRINT(FILE,fmt)			\
  {if(FILE!=stdout){				\
      fprintf(FILE,"%s",fmt);			\
  }else{					\
    PRINT(fmt);					\
  }}
#define FPRINTF(FILE,fmt,...)			\
  {if(FILE!=stdout){				\
    fprintf(FILE,fmt,__VA_ARGS__);		\
  }else{					\
    PRINTF(fmt,__VA_ARGS__);			\
  }}

  extern "C" {
    // ============================================================================
    // call this to set print function from F90
    // ============================================================================
    void FPLOSetPrintFunc(F90PRINTFUNC func);
  }

  
}

#endif /* _F90INTERFACE_HH */

