/* @(#)FVec3.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _FVEC3_HH
#define _FVEC3_HH 1

#include "Fractional.hh"
#include "IVec3.hh"

class FMatrix3x3;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class FVec3{
public:
  FVec3(){};
  FVec3(const FVec3 &o){
    p[0]=o.p[0];
    p[1]=o.p[1];
    p[2]=o.p[2];
   };
  FVec3(const IVec3 &o){
    p[0]=o[0];
    p[1]=o[1];
    p[2]=o[2];
   };
  FVec3(const Fractional &a,const Fractional &b,const Fractional &c){
    p[0]=a;p[1]=b;p[2]=c;
  }
  FVec3(long a,long b,long c,long denom){
    p[0]=Fractional(a,denom);
    p[1]=Fractional(b,denom);
    p[2]=Fractional(c,denom);
  }
  FVec3 &operator=(const FVec3 &o){
    p[0]=o.p[0];
    p[1]=o.p[1];
    p[2]=o.p[2];
    return *this;
  };
  


  Fractional &operator[](long i){return p[i];}
  const Fractional &operator[](long i)const{return p[i];}

  bool operator==(const FVec3 &o)const{
    return (
	    (p[0]==o.p[0]) &&
	    (p[1]==o.p[1]) &&
	    (p[2]==o.p[2]) 
	    );
  };
  bool operator!=(const FVec3 &o)const{
    return (
	    (p[0]!=o.p[0]) ||
	    (p[1]!=o.p[1]) ||
	    (p[2]!=o.p[2]) 
	    );
  };

  bool operator==(const Fractional &o)const{
    return (
	    (p[0]==o) &&
	    (p[1]==o) &&
	    (p[2]==o) 
	    );
  };
  bool operator!=(const Fractional &o)const{
    return (
	    (p[0]!=o) ||
	    (p[1]!=o) ||
	    (p[2]!=o) 
	    );
  };
  bool operator==(long o)const{
    return (
	    (p[0]==o) &&
	    (p[1]==o) &&
	    (p[2]==o) 
	    );
  };
  bool operator!=(long o)const{
    return (
	    (p[0]!=o) ||
	    (p[1]!=o) ||
	    (p[2]!=o) 
	    );
  };

  FVec3 &operator+=(const FVec3 &a){
    p[0]+=a.p[0];
    p[1]+=a.p[1];
    p[2]+=a.p[2];
    return *this;
  }

  FVec3 &operator+=(const Fractional &a){
    p[0]+=a;
    p[1]+=a;
    p[2]+=a;
    return *this;
  }
  FVec3 &operator/=(const Fractional &a){
    p[0]/=a;
    p[1]/=a;
    p[2]/=a;
    return *this;
  }
  FVec3 &operator*=(const Fractional &a){
    p[0]*=a;
    p[1]*=a;
    p[2]*=a;
    return *this;
  }

  

  FVec3 operator-()const{
    return FVec3(-p[0],-p[1],-p[2]);
  }
  
  FVec3 operator+(const FVec3 &a)const{
    FVec3 v(*this);
    v+=a;
    return v;
  }
  FVec3 operator-(const FVec3 &a)const{
    return FVec3(p[0]-a.p[0],p[1]-a.p[1],p[2]-a.p[2]);
  }

  //dot product
  Fractional operator*(const FVec3 &a)const{
    return p[0]*a.p[0]+p[1]*a.p[1]+p[2]*a.p[2];
  }

  // vector product 
  FVec3 cross(const FVec3 &o) const{
    return FVec3(p[1]*o.p[2]-p[2]*o.p[1],
		 p[2]*o.p[0]-p[0]*o.p[2],
		 p[0]*o.p[1]-p[1]*o.p[0]);
  }

  // outer dyadic product
  FMatrix3x3 outer(const FVec3 &o) const;

  // scale
  FVec3 operator*(const Fractional &d)const{
    return FVec3(p[0]*d,p[1]*d,p[2]*d);
  }
  FVec3 operator/(const Fractional &d)const{
    return FVec3(p[0]/d,p[1]/d,p[2]/d);
  }

  bool isInteger()const{
    return (
	    p[0].isInteger() &&
	    p[1].isInteger() &&
	    p[2].isInteger() 
	    );
  };

  FVec3 &regularizeInterval(bool centered){
    p[0].regularizeInterval(centered);
    p[1].regularizeInterval(centered);
    p[2].regularizeInterval(centered);
    return *this;
   }
  FVec3 regularizedInterval(bool centered)const{
    FVec3 v(*this);
    return v.regularizeInterval(centered);
   }

  IVec3 commonDenominator(long &denom)const{
    long D=labs(MyMath::gcd(p[0].m(),p[1].m(),p[2].m()));
    long m0=p[0].m()/D;
    long m1=p[1].m()/D;
    long m2=p[2].m()/D;
    long n0=p[0].n()*(m1*m2);
    long n1=p[1].n()*(m0*m2);
    long n2=p[2].n()*(m0*m1);
    denom=D*(m0*m1*m2);
    D=labs(MyMath::gcd(n0,n1,n2,D));
    denom/=D;
    return IVec3(n0/D,n1/D,n2/D);
  }

  String printS(long padding=6,const char *paren="")const{
    if(paren[0]!='\0' && paren[1]!='\0')
      return Sprintf("%c%s,%s,%s%c",paren[0],
		     p[0].printS().rightPadd(' ',padding)(),
		     p[1].printS().rightPadd(' ',padding)(),
		     p[2].printS().rightPadd(' ',padding)(),paren[1]
		     );
    else
      return Sprintf("%s,%s,%s",
		     p[0].printS().rightPadd(' ',padding)(),
		     p[1].printS().rightPadd(' ',padding)(),
		     p[2].printS().rightPadd(' ',padding)()
		     );
  }
  
private:
  Fractional p[3];
  friend class FMatrix3x3;
  friend Fractional operator*(const IVec3 &a,const FVec3 &b);

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class FMatrix3x3{
public:
  FMatrix3x3(){};
  FMatrix3x3(const long *d){for(int k=0;k<9;k++) p[k]=d[k];}
  FMatrix3x3(const FMatrix3x3 &o){for(int k=0;k<9;k++) p[k]=o.p[k];}
  FMatrix3x3(const IMatrix3x3 &o){for(int k=0;k<9;k++) p[k]=o.p[k];}
  FMatrix3x3(const FVec3 &o1,const FVec3 &o2,const FVec3 &o3){
    for(int k=0;k<3;k++) {
      p[k  ]=o1.p[k];
      p[k+3]=o2.p[k];
      p[k+6]=o3.p[k];
    }
  }
  FMatrix3x3 &operator=(const FMatrix3x3 &o){
    for(int k=0;k<9;k++) p[k]=o.p[k]; return *this; }

  Fractional &operator()(long i,long j){return p[i+3*j];}
  const Fractional &operator()(long i,long j)const{return p[i+3*j];}

  FVec3 column(long i)const{return FVec3(p[3*i],p[1+3*i],p[2+3*i]);}

  void setColumn(long i,const FVec3 &v){
    p[0+3*i]=v[0]; 
    p[1+3*i]=v[1]; 
    p[2+3*i]=v[2]; 
  }

  bool isInteger()const{
    return (
	    p[0].isInteger() &&
	    p[1].isInteger() &&
	    p[2].isInteger() &&
	    p[3].isInteger() &&
	    p[4].isInteger() &&
	    p[5].isInteger() &&
	    p[6].isInteger() &&
	    p[7].isInteger() &&
	    p[8].isInteger() 
	    );
  };


  IMatrix3x3 toIMatrix3x3()const{
    if(!isInteger())
      throw Except("FMatrix3x3::toIMatrix3x3: matrix is not integer?"); 

    IMatrix3x3 res;
    for(int k=0;k<3;k++){
      res(k,0)=p[k].n();
      res(k,1)=p[k+3].n();
      res(k,2)=p[k+6].n();
    }
    return res; 
  }
  
  bool operator==(const FMatrix3x3 &o)const{
    return (
 	    (p[0]==o.p[0]) &&
 	    (p[1]==o.p[1]) &&
 	    (p[2]==o.p[2]) &&
 	    (p[3]==o.p[3]) &&
	    (p[4]==o.p[4]) &&
	    (p[5]==o.p[5]) &&
 	    (p[6]==o.p[6]) &&
 	    (p[7]==o.p[7]) &&
 	    (p[8]==o.p[8]) 
 	    );
  };
  bool operator!=(const FMatrix3x3 &o)const{
    return (
 	    (p[0]!=o.p[0]) ||
 	    (p[1]!=o.p[1]) ||
 	    (p[2]!=o.p[2]) ||
 	    (p[3]!=o.p[3]) ||
	    (p[4]!=o.p[4]) ||
	    (p[5]!=o.p[5]) ||
 	    (p[6]!=o.p[6]) ||
 	    (p[7]!=o.p[7]) ||
 	    (p[8]!=o.p[8]) 
 	    );
  };

  
  FMatrix3x3 &operator*=(const Fractional &d){
    for(int k=0;k<9;k++) p[k]*=d;
    return *this;
  }

  FMatrix3x3 operator-()const{
    FMatrix3x3 m;
    for(int k=0;k<9;k++) m.p[k]=-p[k];
    return m;
  }
  
  FMatrix3x3 operator+(const FMatrix3x3 &o)const{
    FMatrix3x3 m;
    for(int k=0;k<9;k++) m.p[k]=p[k]+o.p[k];
    return m;
  }
  FMatrix3x3 operator-(const FMatrix3x3 &o)const{
    FMatrix3x3 m;
    for(int k=0;k<9;k++) m.p[k]=p[k]-o.p[k];
    return m;
  }
  FMatrix3x3 operator*(const Fractional &s)const{
    FMatrix3x3 m;
    for(int k=0;k<9;k++) m.p[k]=p[k]*s;
    return m;
  }
  FMatrix3x3 operator/(const Fractional &s)const{
    FMatrix3x3 m;
    for(int k=0;k<9;k++) m.p[k]=p[k]/s;
    return m;
  }

  FMatrix3x3 operator*(const FMatrix3x3 &o)const{
    FMatrix3x3 m;
    m.p[0]=(p[0]*o.p[0] +p[3]*o.p[1] +p[6]*o.p[2]);
    m.p[1]=(p[1]*o.p[0] +p[4]*o.p[1] +p[7]*o.p[2]);
    m.p[2]=(p[2]*o.p[0] +p[5]*o.p[1] +p[8]*o.p[2]);
    
    m.p[3]=(p[0]*o.p[3] +p[3]*o.p[4] +p[6]*o.p[5]);
    m.p[4]=(p[1]*o.p[3] +p[4]*o.p[4] +p[7]*o.p[5]);
    m.p[5]=(p[2]*o.p[3] +p[5]*o.p[4] +p[8]*o.p[5]);
    
    m.p[6]=(p[0]*o.p[6] +p[3]*o.p[7] +p[6]*o.p[8]);
    m.p[7]=(p[1]*o.p[6] +p[4]*o.p[7] +p[7]*o.p[8]);
    m.p[8]=(p[2]*o.p[6] +p[5]*o.p[7] +p[8]*o.p[8]);
    
    return m;
  };

  FMatrix3x3 transposed()const{
    FMatrix3x3 m;
    
    //0 3 6
    //1 4 7
    //2 5 8
    
    m.p[0]=p[0];
    m.p[1]=p[3];
    m.p[2]=p[6];
    m.p[3]=p[1];
    m.p[4]=p[4];
    m.p[5]=p[7];
    m.p[6]=p[2];
    m.p[7]=p[5];
    m.p[8]=p[8];
    
    return m;
  }

  FMatrix3x3 inverseT(Fractional &det)const{
    return inverse(det).transposed();
  }
  
  FMatrix3x3 inverse()const{Fractional det;return inverse(det);}

  FMatrix3x3 inverse(Fractional &det)const{
     // return inverse and det
    FMatrix3x3 m;
    
    m.p[0]=p[4]*p[8]-p[5]*p[7];
    m.p[3]=p[5]*p[6]-p[3]*p[8];
    m.p[6]=p[3]*p[7]-p[4]*p[6];
    det=m.p[0]*p[0]+m.p[3]*p[1]+m.p[6]*p[2];
    Fractional i(1);
    if(det!=0){
      i=det.inv();
    }


    m.p[0]*=i;
    m.p[3]*=i;
    m.p[6]*=i;
    
    m.p[1]=i*(p[7]*p[2]-p[8]*p[1]);
    m.p[4]=i*(p[8]*p[0]-p[6]*p[2]);
    m.p[7]=i*(p[6]*p[1]-p[7]*p[0]);
    
    m.p[2]=i*(p[1]*p[5]-p[2]*p[4]);
    m.p[5]=i*(p[2]*p[3]-p[0]*p[5]);
    m.p[8]=i*(p[0]*p[4]-p[1]*p[3]);
    
    
    return m;
  };

   Fractional determinant()const{
     // return det
     return 
       p[0]*(p[4]*p[8]-p[5]*p[7])+
       p[1]*(p[5]*p[6]-p[3]*p[8])+
       p[2]*(p[3]*p[7]-p[4]*p[6]);
  }

  FVec3 operator*(const FVec3 &a)const{
    FVec3 v;
    v.p[0]=a.p[0]*p[0]+a.p[1]*p[3]+a.p[2]*p[6];
    v.p[1]=a.p[0]*p[1]+a.p[1]*p[4]+a.p[2]*p[7];
    v.p[2]=a.p[0]*p[2]+a.p[1]*p[5]+a.p[2]*p[8];
    return v;
  }


  String printS()const{
    return Sprintf("[%s,%s,%s\n"
		   " %s,%s,%s\n"
		   " %s,%s,%s]\n",
		   p[0].printS().rightPadd(' ',6)(),
		   p[3].printS().rightPadd(' ',6)(),
		   p[6].printS().rightPadd(' ',6)(),
		   p[1].printS().rightPadd(' ',6)(),
		   p[4].printS().rightPadd(' ',6)(),
		   p[7].printS().rightPadd(' ',6)(),
		   p[2].printS().rightPadd(' ',6)(),
		   p[5].printS().rightPadd(' ',6)(),
		   p[8].printS().rightPadd(' ',6)()
		   );
  }

  static FMatrix3x3 unitMatrix(const Fractional &d){ 
    FMatrix3x3 m; m.p[0]=m.p[4]=m.p[8]=d; return m; };


private:
  // order
  // 0 3 6
  // 1 4 7
  // 2 5 8
  Fractional p[9];
  friend class FVec3;
  friend FVec3 operator*(const FVec3 &v,const FMatrix3x3 &m);
  friend FMatrix3x3 operator*(const IMatrix3x3 &a,const FMatrix3x3 &b);

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
//outer dyadic product: m=this (outer) o
inline FMatrix3x3 FVec3::outer(const FVec3 &o) const
{
  FMatrix3x3 m;
  m.p[0]=p[0]*o.p[0]; m.p[1]=p[1]*o.p[0]; m.p[2]=p[2]*o.p[0];
  m.p[3]=p[0]*o.p[1]; m.p[4]=p[1]*o.p[1]; m.p[5]=p[2]*o.p[1];
  m.p[6]=p[0]*o.p[2]; m.p[7]=p[1]*o.p[2]; m.p[8]=p[2]*o.p[2];

  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
//scalar
inline Fractional operator*(const IVec3 &a,const FVec3 &b){
  return b.p[0]*a[0]+b.p[1]*a[1]+b.p[2]*a[2];
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline FVec3 operator*(const FVec3 &v,const FMatrix3x3 &m)
{
  FVec3 res;
  res[0]=v[0]*m.p[0]+v[1]*m.p[1]+v[2]*m.p[2];
  res[1]=v[0]*m.p[3]+v[1]*m.p[4]+v[2]*m.p[5];
  res[2]=v[0]*m.p[6]+v[1]*m.p[7]+v[2]*m.p[8];
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline FMatrix3x3 operator*(const IMatrix3x3 &a,const FMatrix3x3 &b)
{
  FMatrix3x3 m;

  m.p[0]=b.p[0]*a(0,0)+b.p[1]*a(0,1)+b.p[2]*a(0,2);
  m.p[3]=b.p[3]*a(0,0)+b.p[4]*a(0,1)+b.p[5]*a(0,2);
  m.p[6]=b.p[6]*a(0,0)+b.p[7]*a(0,1)+b.p[8]*a(0,2);
  
  m.p[1]=b.p[0]*a(1,0)+b.p[1]*a(1,1)+b.p[2]*a(1,2);
  m.p[4]=b.p[3]*a(1,0)+b.p[4]*a(1,1)+b.p[5]*a(1,2);
  m.p[7]=b.p[6]*a(1,0)+b.p[7]*a(1,1)+b.p[8]*a(1,2);
  
  m.p[2]=b.p[0]*a(2,0)+b.p[1]*a(2,1)+b.p[2]*a(2,2);
  m.p[5]=b.p[3]*a(2,0)+b.p[4]*a(2,1)+b.p[5]*a(2,2);
  m.p[8]=b.p[6]*a(2,0)+b.p[7]*a(2,1)+b.p[8]*a(2,2);
  
  
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

inline MyDebug operator<<(MyDebug dbg,const FVec3 &v){
  dbg<<v.printS(1,"{}");
  return dbg;
}

inline MyDebug operator<<(MyDebug dbg,const FMatrix3x3 &v){
  dbg<<v.printS();
  return dbg;
}

#endif /* _FVEC3_HH */

