/* @(#)IVec3.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _IVEC3_H
#define _IVEC3_H 1


#include "Vec3.hh"


class IVec3{
public:
  IVec3(){pp[0]=pp[1]=pp[2]=0;};
  IVec3(const long *d){memcpy(pp,d,sizeof(long)*3);};
  IVec3(long a,long b,long c){pp[0]=a;pp[1]=b;pp[2]=c;};
  
  IVec3 &operator=(const IVec3&other){
    memcpy(pp,other.pp,sizeof(long)*3);
    return *this;};
  IVec3 &operator+=(const IVec3 &b){
    pp[0]+=b.pp[0];
    pp[1]+=b.pp[1];
    pp[2]+=b.pp[2];
    return *this;
  };

  bool operator==(const IVec3 &other)const{
    return ( 
	    (pp[0]==other.pp[0])  && 
	    (pp[1]==other.pp[1])  && 
	    (pp[2]==other.pp[2]) );
  }

  inline IVec3 operator-()const{return IVec3(-pp[0],-pp[1],-pp[2]);};


  long &operator[](long i){return pp[i];};//todo: is sharing correct
  long operator[](long i)const{return pp[i];};
  const long *data()const{return pp;};

  long gcd()const{return MyMath::gcd(pp[0],pp[1],pp[2]);};

  // shortest possible integer vector in this direction
  IVec3 &shorten(){
    long D=labs(MyMath::gcd(pp[0],pp[1],pp[2]));
    pp[0]/=D; pp[1]/=D; pp[2]/=D;
    return *this;
  }

  IVec3 &operator*=(long s){
    pp[0]*=s;
    pp[1]*=s;
    pp[2]*=s;
    return *this;};
  IVec3 &operator/=(long s){
    pp[0]/=s;
    pp[1]/=s;
    pp[2]/=s;
    return *this;};
  IVec3 operator-(const IVec3 &b)const{
    return IVec3(pp[0]-b.pp[0],pp[1]-b.pp[1],pp[2]-b.pp[2]);
  };
  IVec3 operator+(const IVec3 &b)const{
    return IVec3(pp[0]+b.pp[0],pp[1]+b.pp[1],pp[2]+b.pp[2]);
  };
  IVec3 operator*(long s)const{
    return IVec3(pp[0]*s,pp[1]*s,pp[2]*s);
  };
  Vec3 operator*(double d)const{
    return Vec3(pp[0]*d,pp[1]*d,pp[2]*d);
  };


  // vector product 
  inline IVec3 cross(const IVec3 &o) const{
    return IVec3(pp[1]*o.pp[2]-pp[2]*o.pp[1],
		 pp[2]*o.pp[0]-pp[0]*o.pp[2],
		 pp[0]*o.pp[1]-pp[1]*o.pp[0]);
  }
  String printS1()const{
    return Sprintf("[%ld,%ld,%ld]\n",pp[0],pp[1],pp[2]);
  };
 

   
private:
  long pp[3];
  friend long operator*(const IVec3 &,const IVec3 &);
  friend long mhnorm(const IVec3 &v);
  friend IVec3 operator*(const IVec3 &a,const IMatrix3x3 &b);
  friend IVec3 operator*(const IMatrix3x3 &b,const IVec3 &a);
  friend long sum(const IVec3 &v);
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class IMatrix3x3{
  //storage of the linear array maps to the matrix as
  //0 3 6
  //1 4 7
  //2 5 8

public:
//   class IMatrix3x3Column{
//   public:
    
//     IMatrix3x3Column(long *d){p=d;};
    
//     long &operator[](long i){return p[i];};
//     long operator[](long i)const{return p[i];};
    
//     long operator*(const IMatrix3x3Column &other)const{
//       return p[0]*other.p[0]+p[1]*other.p[1]+p[2]*other.p[2];
//     };
    
//     long operator*(const IVec3 &other)const{
//       return p[0]*other[0]+p[1]*other[1]+p[2]*other[2];
//     };
    
//     IMatrix3x3Column &operator*=(long s){
//       p[0]*=s;
//       p[1]*=s;
//       p[2]*=s;
//       return *this;};
    
//     IVec3 operator*(long s)const{ return IVec3(p)*s;};
    
//     IVec3 operator+(const IVec3 &other)const{return IVec3(p)+other;};
// //     IVec3 operator-(const IMatrix3x3Column &other)const{
// //       return IVec3(p)-IVec3(other.p);};
    
//     IMatrix3x3Column &operator=(const IVec3 &other){
//       p[0]=other[0];
//       p[1]=other[1];
//       p[2]=other[2];
//       return *this;
//     };
    
//     inline long gcd()const{return IVec3(p).gcd();};
    
//     inline IVec3 cross(const IMatrix3x3Column &o) const{
//       return IVec3(p).cross(IVec3(o.p)); };

    
//   private:
//     long *p;
//   };

  IMatrix3x3(){p[0]=p[1]=p[2]=p[3]=p[4]=p[5]=p[6]=p[7]=p[8]=0;};
  
  IMatrix3x3(const long *d){memcpy(p,d,sizeof(long)*9);};
  IMatrix3x3(const IVec3 &v0,const IVec3 &v1,const IVec3 &v2){
    memcpy(p,v0.data(),sizeof(long)*3);
    memcpy(p+3,v1.data(),sizeof(long)*3);
    memcpy(p+6,v2.data(),sizeof(long)*3);
  };
  
  IMatrix3x3(const IMatrix3x3&other){
    memcpy(p,other.p,sizeof(long)*9);
  };
  
  IMatrix3x3 &operator=(const IMatrix3x3&other){
    memcpy(p,other.p,sizeof(long)*9);
    return *this;};

  IMatrix3x3 &operator/=(long s){
    p[0]/=s; p[1]/=s; p[2]/=s;
    p[3]/=s; p[4]/=s; p[5]/=s;
    p[6]/=s; p[7]/=s; p[8]/=s;
    return *this;};
  IMatrix3x3 &operator*=(long s){
    p[0]*=s; p[1]*=s; p[2]*=s;
    p[3]*=s; p[4]*=s; p[5]*=s;
    p[6]*=s; p[7]*=s; p[8]*=s;
    return *this;};

  void scaleColumn(long i,long s){
    p[0+3*i]*=s; 
    p[1+3*i]*=s; 
    p[2+3*i]*=s; 
  }
  void setColumn(long i,const IVec3 &v){
    p[0+3*i]=v[0]; 
    p[1+3*i]=v[1]; 
    p[2+3*i]=v[2]; 
  }
  void setColumn(long i,const long *v){
    p[0+3*i]=v[0]; 
    p[1+3*i]=v[1]; 
    p[2+3*i]=v[2]; 
  }
  
  bool operator==(const IMatrix3x3 &other)const{
    return ( 
	    (p[0]==other.p[0])  && 
	    (p[1]==other.p[1])  && 
	    (p[2]==other.p[2])  && 
	    (p[3]==other.p[3])  && 
	    (p[4]==other.p[4])  && 
	    (p[5]==other.p[5])  && 
	    (p[6]==other.p[6])  && 
	    (p[7]==other.p[7])  && 
	    (p[8]==other.p[8]) );
  }
  

  //return copy of column 
  IVec3 column(long i)const{return IVec3(p+3*i);};
  long *columnData(long i){return p+3*i;};
  
  //return copy of row
  IVec3 row(long i)const{return IVec3(p[i],p[i+3],p[i+6]);};


  //return element reference: for manipulation
  long operator()(long i,long j)const{return p[i+3*j];}
  long &operator()(long i,long j){return p[i+3*j];}
  
  IMatrix3x3 inverseTtV(long &idet)const{
    // return inverse.transposed * |V|
    IMatrix3x3 m;
    

    m.p[0]=p[4]*p[8]-p[5]*p[7];
    m.p[1]=p[5]*p[6]-p[3]*p[8];
    m.p[2]=p[3]*p[7]-p[4]*p[6];
    idet=m.p[0]*p[0]+m.p[1]*p[1]+m.p[2]*p[2];
    long i=1;
    if(idet<0){
      i=-1;
      idet*=i;
      m.p[0]*=i;
      m.p[1]*=i;
      m.p[2]*=i;
    }

    m.p[3]=i*(p[7]*p[2]-p[8]*p[1]);
    m.p[4]=i*(p[8]*p[0]-p[6]*p[2]);
    m.p[5]=i*(p[6]*p[1]-p[7]*p[0]);

    m.p[6]=i*(p[1]*p[5]-p[2]*p[4]);
    m.p[7]=i*(p[2]*p[3]-p[0]*p[5]);
    m.p[8]=i*(p[0]*p[4]-p[1]*p[3]);

    
    return m;
  };
  IMatrix3x3 inversetV(long &idet)const{
    // return inverse * |V|
    IMatrix3x3 m;
    
    m.p[0]=p[4]*p[8]-p[5]*p[7];
    m.p[3]=p[5]*p[6]-p[3]*p[8];
    m.p[6]=p[3]*p[7]-p[4]*p[6];
    idet=m.p[0]*p[0]+m.p[3]*p[1]+m.p[6]*p[2];
    long i=1;
    if(idet<0){
      i=-1;
      idet*=i;
      m.p[0]*=i;
      m.p[3]*=i;
      m.p[6]*=i;
    }
    m.p[1]=i*(p[7]*p[2]-p[8]*p[1]);
    m.p[4]=i*(p[8]*p[0]-p[6]*p[2]);
    m.p[7]=i*(p[6]*p[1]-p[7]*p[0]);

    m.p[2]=i*(p[1]*p[5]-p[2]*p[4]);
    m.p[5]=i*(p[2]*p[3]-p[0]*p[5]);
    m.p[8]=i*(p[0]*p[4]-p[1]*p[3]);

    return m;
  };


  IMatrix3x3 operator*(const IMatrix3x3 &o){
    IMatrix3x3 m;
      
    m.p[0]=(p[0]*o.p[0] +p[3]*o.p[1] +p[6]*o.p[2]);
    m.p[1]=(p[1]*o.p[0] +p[4]*o.p[1] +p[7]*o.p[2]);
    m.p[2]=(p[2]*o.p[0] +p[5]*o.p[1] +p[8]*o.p[2]);

    m.p[3]=(p[0]*o.p[3] +p[3]*o.p[4] +p[6]*o.p[5]);
    m.p[4]=(p[1]*o.p[3] +p[4]*o.p[4] +p[7]*o.p[5]);
    m.p[5]=(p[2]*o.p[3] +p[5]*o.p[4] +p[8]*o.p[5]);

    m.p[6]=(p[0]*o.p[6] +p[3]*o.p[7] +p[6]*o.p[8]);
    m.p[7]=(p[1]*o.p[6] +p[4]*o.p[7] +p[7]*o.p[8]);
    m.p[8]=(p[2]*o.p[6] +p[5]*o.p[7] +p[8]*o.p[8]);
    

    return m;
  };

  long determinant()const;


  IMatrix3x3 transposed()const{
    IMatrix3x3 m;
    
    //0 3 6
    //1 4 7
    //2 5 8
    
    m.p[0]=p[0];
    m.p[1]=p[3];
    m.p[2]=p[6];
    m.p[3]=p[1];
    m.p[4]=p[4];
    m.p[5]=p[7];
    m.p[6]=p[2];
    m.p[7]=p[5];
    m.p[8]=p[8];
    
    return m;
  }
  IMatrix3x3 &transpose(){
    
    long d=p[1];p[1]=p[3];p[3]=d;
    d=p[2];p[2]=p[6];p[6]=d;
    d=p[5];p[5]=p[7];p[7]=d;
    
    return *this;
  }
  String printS()const{
    return Sprintf("\n[%ld %ld %ld\n"
		   " %ld %ld %ld\n"
		   " %ld %ld %ld]\n",
		   p[0],p[3],p[6],
		   p[1],p[4],p[7],
		   p[2],p[5],p[8]
		   );
  };
  static inline IMatrix3x3 unitMatrix(long d){ 
    IMatrix3x3 m;
    m.p[0]=m.p[4]=m.p[8]=d;
    return m;
  };
private:
  long p[9];

  friend class FMatrix3x3;
  friend long trace(const IMatrix3x3 &m);
  friend IMatrix3x3 operator*(const IMatrix3x3 &ao,const IMatrix3x3 &b);
  friend IMatrix3x3 operator+(const IMatrix3x3 &a,const IMatrix3x3 &b);
  friend IMatrix3x3 operator-(const IMatrix3x3 &a,const IMatrix3x3 &b);
  friend IMatrix3x3 round(const Matrix3x3&other);
  friend IMatrix3x3 operator*(const IMatrix3x3 &b,long s);
  friend Matrix3x3 operator*(const Matrix3x3 &a,const IMatrix3x3 &b);
  friend Matrix3x3 operator-(const Matrix3x3 &a,const IMatrix3x3 &b);
  friend Matrix3x3 operator-(const IMatrix3x3 &a,const Matrix3x3 &b);
  friend IVec3 operator*(const IVec3 &a,const IMatrix3x3 &b);
  friend IVec3 operator*(const IMatrix3x3 &b,const IVec3 &a);
//   friend IVec3 operator*(const IMatrix3x3 &a,
// 			 const IMatrix3x3::IMatrix3x3Column &b);

};


  /* ***********************************************************************
   *
   * ***********************************************************************
   */
inline long sum(const IVec3 &v)
{
  return v.pp[0]+v.pp[1]+v.pp[2];
}

inline long mhnorm(const IVec3 &v){
  return labs(v.pp[0])+labs(v.pp[1])+labs(v.pp[2]);
}

inline long operator*(const IVec3 &a,const IVec3 &b){
  return a.pp[0]*b.pp[0]+a.pp[1]*b.pp[1]+a.pp[2]*b.pp[2];
}



inline Vec3 operator*(double d,const IVec3 &v){
  return Vec3(v[0]*d,v[1]*d,v[2]*d);
}

// inline Vec3 operator*(const Matrix3x3 &a,const IMatrix3x3::IMatrix3x3Column &b){
//   Vec3 v;
//   v[0]=a(0)[0]*b[0]+a(1)[0]*b[1]+a(2)[0]*b[2];
//   v[1]=a(0)[1]*b[0]+a(1)[1]*b[1]+a(2)[1]*b[2];
//   v[2]=a(0)[2]*b[0]+a(1)[2]*b[1]+a(2)[2]*b[2];
//   return v;
// }

// inline IVec3 operator-(const IVec3 &a,const IMatrix3x3::IMatrix3x3Column &b)
// {
//   return IVec3(a[0]-b[0],a[1]-b[1],a[2]-b[2]);
// }

inline Vec3 operator*(const Matrix3x3 &a,const IVec3 &b){
  Vec3 v;
  v[0]=a(0)[0]*b[0]+a(1)[0]*b[1]+a(2)[0]*b[2];
  v[1]=a(0)[1]*b[0]+a(1)[1]*b[1]+a(2)[1]*b[2];
  v[2]=a(0)[2]*b[0]+a(1)[2]*b[1]+a(2)[2]*b[2];
  return v;
}

inline Vec3 operator/(const IVec3 &a,double d){
  Vec3 v;
  v[0]=a[0]/d;
  v[1]=a[1]/d;
  v[2]=a[2]/d;
  return v;
}

inline double operator*(const Vec3 &a,const IVec3 &b){
  return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}
// inline double operator*(const Vec3 &a,const IMatrix3x3::IMatrix3x3Column &b)
// {
//   return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
// }

// inline long operator*(const IVec3 &a,const IMatrix3x3::IMatrix3x3Column &b)
// {
//   return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long trace(const IMatrix3x3 &m){
  return m.p[0]+m.p[4]+m.p[8];
}

inline IVec3 operator*(const IVec3 &a,const IMatrix3x3 &b)
{
  IVec3 v;
  v[0]=a.pp[0]*b.p[0]+a.pp[1]*b.p[1]+a.pp[2]*b.p[2];
  v[1]=a.pp[0]*b.p[3]+a.pp[1]*b.p[4]+a.pp[2]*b.p[5];
  v[2]=a.pp[0]*b.p[6]+a.pp[1]*b.p[7]+a.pp[2]*b.p[8];
  return v;
}
// inline IVec3 operator*(const IMatrix3x3 &a,
// 		       const IMatrix3x3::IMatrix3x3Column &b)
// {
//   IVec3 v;
//   v[0]=b[0]*a.p[0]+b[1]*a.p[3]+b[2]*a.p[6];
//   v[1]=b[0]*a.p[1]+b[1]*a.p[4]+b[2]*a.p[7];
//   v[2]=b[0]*a.p[2]+b[1]*a.p[5]+b[2]*a.p[8];
//   return v;
// }

inline IVec3 operator*(const IMatrix3x3 &b,const IVec3 &a)
{
  IVec3 v;
  v[0]=a.pp[0]*b.p[0]+a.pp[1]*b.p[3]+a.pp[2]*b.p[6];
  v[1]=a.pp[0]*b.p[1]+a.pp[1]*b.p[4]+a.pp[2]*b.p[7];
  v[2]=a.pp[0]*b.p[2]+a.pp[1]*b.p[5]+a.pp[2]*b.p[8];
  return v;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline IMatrix3x3 operator*(const IMatrix3x3 &a,const IMatrix3x3 &b)
{
  IMatrix3x3 m;
  
  m.p[0]=(a.p[0]*b.p[0] +a.p[3]*b.p[1] +a.p[6]*b.p[2]);
  m.p[1]=(a.p[1]*b.p[0] +a.p[4]*b.p[1] +a.p[7]*b.p[2]);
  m.p[2]=(a.p[2]*b.p[0] +a.p[5]*b.p[1] +a.p[8]*b.p[2]);
  
  m.p[3]=(a.p[0]*b.p[3] +a.p[3]*b.p[4] +a.p[6]*b.p[5]);
  m.p[4]=(a.p[1]*b.p[3] +a.p[4]*b.p[4] +a.p[7]*b.p[5]);
  m.p[5]=(a.p[2]*b.p[3] +a.p[5]*b.p[4] +a.p[8]*b.p[5]);
  
  m.p[6]=(a.p[0]*b.p[6] +a.p[3]*b.p[7] +a.p[6]*b.p[8]);
  m.p[7]=(a.p[1]*b.p[6] +a.p[4]*b.p[7] +a.p[7]*b.p[8]);
  m.p[8]=(a.p[2]*b.p[6] +a.p[5]*b.p[7] +a.p[8]*b.p[8]);
  
  
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline Matrix3x3 operator*(const Matrix3x3 &a,const IMatrix3x3 &b){
  Matrix3x3 m;
  
  m.p[0]=(a.p[0]*b.p[0] +a.p[3]*b.p[1] +a.p[6]*b.p[2]);
  m.p[1]=(a.p[1]*b.p[0] +a.p[4]*b.p[1] +a.p[7]*b.p[2]);
  m.p[2]=(a.p[2]*b.p[0] +a.p[5]*b.p[1] +a.p[8]*b.p[2]);
  
  m.p[3]=(a.p[0]*b.p[3] +a.p[3]*b.p[4] +a.p[6]*b.p[5]);
  m.p[4]=(a.p[1]*b.p[3] +a.p[4]*b.p[4] +a.p[7]*b.p[5]);
  m.p[5]=(a.p[2]*b.p[3] +a.p[5]*b.p[4] +a.p[8]*b.p[5]);
  
  m.p[6]=(a.p[0]*b.p[6] +a.p[3]*b.p[7] +a.p[6]*b.p[8]);
  m.p[7]=(a.p[1]*b.p[6] +a.p[4]*b.p[7] +a.p[7]*b.p[8]);
  m.p[8]=(a.p[2]*b.p[6] +a.p[5]*b.p[7] +a.p[8]*b.p[8]);
  
  
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline IMatrix3x3 operator+(const IMatrix3x3 &a,const IMatrix3x3 &b)
{
  IMatrix3x3 m;
  
  m.p[0]=(a.p[0]+b.p[0]);
  m.p[1]=(a.p[1]+b.p[1]);
  m.p[2]=(a.p[2]+b.p[2]);
  m.p[3]=(a.p[3]+b.p[3]);
  m.p[4]=(a.p[4]+b.p[4]);
  m.p[5]=(a.p[5]+b.p[5]);
  m.p[6]=(a.p[6]+b.p[6]);
  m.p[7]=(a.p[7]+b.p[7]);
  m.p[8]=(a.p[8]+b.p[8]);
  
  return m;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline IMatrix3x3 operator-(const IMatrix3x3 &a,const IMatrix3x3 &b)
{
  IMatrix3x3 m;
  
  m.p[0]=(a.p[0]-b.p[0]);
  m.p[1]=(a.p[1]-b.p[1]);
  m.p[2]=(a.p[2]-b.p[2]);
  m.p[3]=(a.p[3]-b.p[3]);
  m.p[4]=(a.p[4]-b.p[4]);
  m.p[5]=(a.p[5]-b.p[5]);
  m.p[6]=(a.p[6]-b.p[6]);
  m.p[7]=(a.p[7]-b.p[7]);
  m.p[8]=(a.p[8]-b.p[8]);
  
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline Matrix3x3 operator-(const Matrix3x3 &a,const IMatrix3x3 &b){
  Matrix3x3 m;
  
  m.p[0]=(a.p[0]-b.p[0]);
  m.p[1]=(a.p[1]-b.p[1]);
  m.p[2]=(a.p[2]-b.p[2]);
  m.p[3]=(a.p[3]-b.p[3]);
  m.p[4]=(a.p[4]-b.p[4]);
  m.p[5]=(a.p[5]-b.p[5]);
  m.p[6]=(a.p[6]-b.p[6]);
  m.p[7]=(a.p[7]-b.p[7]);
  m.p[8]=(a.p[8]-b.p[8]);
  
  
  return m;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline Matrix3x3 operator-(const IMatrix3x3 &a,const Matrix3x3 &b){
  Matrix3x3 m;
  
  m.p[0]=(a.p[0]-b.p[0]);
  m.p[1]=(a.p[1]-b.p[1]);
  m.p[2]=(a.p[2]-b.p[2]);
  m.p[3]=(a.p[3]-b.p[3]);
  m.p[4]=(a.p[4]-b.p[4]);
  m.p[5]=(a.p[5]-b.p[5]);
  m.p[6]=(a.p[6]-b.p[6]);
  m.p[7]=(a.p[7]-b.p[7]);
  m.p[8]=(a.p[8]-b.p[8]);
  
  
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline IMatrix3x3 operator*(const IMatrix3x3 &b,long s){
  IMatrix3x3 m;
  m.p[0]=b.p[0]*s;
  m.p[1]=b.p[1]*s;
  m.p[2]=b.p[2]*s;
  m.p[3]=b.p[3]*s;
  m.p[4]=b.p[4]*s;
  m.p[5]=b.p[5]*s;
  m.p[6]=b.p[6]*s;
  m.p[7]=b.p[7]*s;
  m.p[8]=b.p[8]*s;
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline IMatrix3x3 round(const Matrix3x3&other){
  IMatrix3x3 m;
  m.p[0]=lRound(other.p[0]);
  m.p[1]=lRound(other.p[1]);
  m.p[2]=lRound(other.p[2]);
  m.p[3]=lRound(other.p[3]);
  m.p[4]=lRound(other.p[4]);
  m.p[5]=lRound(other.p[5]);
  m.p[6]=lRound(other.p[6]);
  m.p[7]=lRound(other.p[7]);
  m.p[8]=lRound(other.p[8]);
  return m;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long IMatrix3x3::determinant() const 
{ 
  return (IVec3(p).cross(IVec3(p+3)))*IVec3(p+6); 
}


#if (defined QXFSF || defined QXFPL)
//extern QDebug operator<<(QDebug dbg, const IMatrix3x3::IMatrix3x3Column &v);
#endif



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _IVEC3_H */

