/* @(#)MachineTime.hh
 * author: magru@unas
 */

#ifndef _MACHINETIME_HH
#define _MACHINETIME_HH 1

#ifdef __MACH__
#include <mach/mach_time.h>
#define CLOCK_REALTIME 0
#define CLOCK_MONOTONIC 0
#include <time.h>
static int
clock_gettime(int clk_id, struct timespec *t){
    mach_timebase_info_data_t timebase;
    mach_timebase_info(&timebase);
    uint64_t time;
    time = mach_absolute_time();
    double nseconds = ((double)time * (double)timebase.numer)/((double)timebase.denom);
    double seconds = ((double)time * (double)timebase.numer)
      /((double)timebase.denom * 1e9);
    t->tv_sec = seconds;
    t->tv_nsec = nseconds;
    return 0;
}
#else
#include <time.h>
#endif

#include "MyDebug.hh"
#include "FString.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class MachineTime
{
public:
  MachineTime(){setNow();}
  MachineTime(const MachineTime &other){
    modtime=other.modtime;
  }
  MachineTime &operator=(const MachineTime &other){
    modtime=other.modtime;
    return *this;
  }


  bool operator>(const MachineTime &other)const{
    
    if(modtime.tv_sec>other.modtime.tv_sec) return true;
    else if(modtime.tv_sec<other.modtime.tv_sec) return false;

    if(modtime.tv_nsec>other.modtime.tv_nsec) return true;
    
    return false;
  }

  // set to current time
  inline void setNow(){
    clock_gettime(CLOCK_MONOTONIC,&modtime);
  }

  String toString()const{
    return Sprintf("%ld.%09ld s",modtime.tv_sec,modtime.tv_nsec);
  }

  void print(const String &txt,bool reset=true){
    MachineTime now;
    printf("%s: %s\n",txt(),(now-*this).toString()());
    if(reset) *this=now;
  }

  String printS(const String &txt,bool reset=true){
    MachineTime now;
    String res=Sprintf("%s: %s\n",txt(),(now-*this).toString()());
    if(reset) *this=now;
    return res;
  }


  MachineTime operator-(const MachineTime &other){
    MachineTime res;
    long n1=modtime.tv_nsec;
    long n2=other.modtime.tv_nsec;
    long n=n1-n2;
    res.modtime.tv_sec=modtime.tv_sec-other.modtime.tv_sec;
    res.modtime.tv_nsec=n;
    if(n<0){
      res.modtime.tv_sec--;
      res.modtime.tv_nsec+=1000000000;
    }
    return res;
    
  }

private:
  struct timespec modtime;
};




inline MyDebug operator<<(MyDebug dbg,const MachineTime &m)
{
  dbg<<m.toString()();
  return dbg;
}





#endif /* _MACHINETIME_HH */

