/*************************************************************************
 * @(#)MainBW.cpp
 * author: magru@unas
 * date:   22 Nov 2013
 *************************************************************************
 */

#include "BandWeights.hh"
#include "Assorted.hh"
#include "Except.hh"
#include "MyDebug.hh"

static String EXECNAME; 

bool DEBUG=false;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Control{
public:
  Control():statefile("=.addwei"),pinfile("+bweights"),poutfile("+bwsum"),
	    hasinfile(false),hasoutfile(false),printheader(false){}

  
  void setInfile(const String &s){hasinfile=true;pinfile=s;}
  void setOutfile(const String &s){hasoutfile=true;poutfile=s;}

  void check(){
    if(pinfile.empty())
      throw Except("need input file argument!\n");
    if(poutfile.empty())
      throw Except("need output file argument!\n");
    if(statefile.empty())
      throw Except("need statefile file argument!\n");
  }
  
public:
  String statefile;
private:
  ROCMODPROPERTY(infile,infile,String);
  ROCMODPROPERTY(outfile,outfile,String);
  bool hasinfile;
  bool hasoutfile;
  bool printheader;

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void usage()
{
  printf("\n\
Combine/extract/manipulate bandweight data\n\
\n\
Usage: %s [-p] [-f infile] [-o outfile] [-h] [-s statefile]\n\
-f infile : specify input bandweights file : default: +bweights\n\
-o outfile: specify output bandweights file: default: +bwsum\n\
-s statefile: specify file containing the weight sum definitions: default: =.addwei\n\
-p print weight header information.\n\
\n\
IMPORTANT If filenames are specified in the statefile the -f and -o options \n\
          overwrite these!!\n\
\n",EXECNAME());
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void scanopts(Control &control,int argc,const char **argv)
{

  long narg=1;
  while(narg<argc){
    if(DEBUG) myDebug()<<"argv["<<narg<<"]="<<argv[narg];
    if(strcmp(argv[narg],"-f")==0){
      narg++;
      if(narg<argc) control.setInfile(argv[narg]);
      else throw Except("-f needs filename as argument\n");
    }else if(strcmp(argv[narg],"-o")==0){
      narg++;
      if(narg<argc) control.setOutfile(argv[narg]);
      else throw Except("-o needs filename as argument\n");
    }else if(strcmp(argv[narg],"-s")==0){
      narg++;
      if(narg<argc) control.statefile=argv[narg];
      else throw Except("-s needs filename as argument\n");
    }else if(strcmp(argv[narg],"-p")==0){
      control.printheader=true;
    }else if(strcmp(argv[narg],"-h")==0 || strcmp(argv[narg],"-?")==0){
      usage();
      die("");
    }else{
      throw Except("Unkown argument\n");
    }
    narg++;
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main(int argc,const char **argv)
{
  EXECNAME=argv[0];
  EXECNAME=EXECNAME.subString(EXECNAME.find("/",true)+1);
  if(DEBUG) myDebug()<<"EXECNAME="<<EXECNAME;

  Control control;
  BandWeights bw;

  try{
    
    scanopts(control,argc,argv);


    ValueArray<BandWeights::StateSum> statesums;
    ValueArray<double> ewindow;
    String inf,outf;

    bw.writeTemplate(control.statefile);
    
    // passive run to get file names
    bw.readStateDefinitions(control.statefile,inf,outf,statesums,ewindow,true);
    if(!control.hasinfile && !inf.empty()) control.setInfile(inf);
    if(!control.hasoutfile && !outf.empty()) control.setOutfile(outf);

    control.check();
    printf("Read from '%s'\n",control.infile()());
    printf("Write to '%s'\n",control.outfile()());

    bw.extractHeaderInfo(control.infile());
    if(control.printheader){
      myDebug()<<"labels=\n"<<bw.labels();
      die("Abort\n");
    }

    // now fully scan and fill statesums
    bw.readStateDefinitions(control.statefile,control.infile(),
			    control.outfile(),statesums,ewindow);
    
    bw.addWeights(control.infile(),control.outfile(),statesums,ewindow);
    


  }catch(Except &ex){
    usage();
    die(ex.msg());
  }catch(TerminateExcept &ex){
    printf("%s",ex.msg());
  }
  return 0;

}


/*EOF*/ 
