//todo: read all values at once, make it faster
//todo: extract all data at once and do not reiterate over the lines
//todo: do not save them
/*************************************************************************
 * @(#)MainGrepFplo.cpp
 * author: magru@unas
 * date:   08 Nov 2012
 *************************************************************************
 */

#include <stdlib.h>
#include <unistd.h>
#include <glob.h>
#include <sys/types.h>
#include <regex.h>
#include "FString.hh"
#include "Except.hh"
#include "Assorted.hh"
#include "ValueArray.hh"
#include "PtrArray.hh"
#include "Pair.hh"
#include "Wigner3J.hh"
#include "OutGrep.hh"

static String EXECNAME; 


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void usage()
{
  printf("\
Extract data from the fplo output file(s)\n\
\n\
Usage: %s [-h] -m mode [options] (-p prefix) (-f outfile) ([-a(ll)]) \n\
          ([-x(fbp)])  ([-xm(grace)]) \n\
\n\
We assume that separate calculations are done in separate directories\n\
and that the directory names contain the running variable value.\n\
\n\
prefix: is the prefix string of a bunch of directories, of\n\
\twhich the data shall be extracted. (Default is . =current directory.)\n\
\tThe prefix is used to select the directories and is removed from the\n\
\tdirectory name to get the corresponding values;\n\
\tE.g. if the directories are named\n\
\tM=0.1 M=0.2 M=0.3, the prefix M= will result in a data set\n\
\t0.1 data1\n\
\t0.2 data2\n\
\t0.3 data3\n\
\t where data... is the data actually extracted from the output files. (See below.)\n\
\tIf the directories are named id=1_M=0.1 id=2_M=0.2 ..., the prefix\n\
\t'id*M=' (quotes are important) will give the same list as above.\n\
\tThe prefix is used to identify the directories but also\n\
\tis removed from the directory name to obtain the values of the\n\
\tfirst column (which are encoded in the dir-names).\n\
\n\
\tUse single quotes to protect * in shell context;\n\
\n\
\tIf prefix is . (simple dot), the file in the current directory is scanned.\n\
\tIn this case the option '-all' will make a list of the data indexed by the\n\
\titeration step.\n\
\n\
outfile: is the name of the output file, which should be the same in all \n\
\tdirectories. Default is out.\n\
\n\
\n\
modes: Default is EE.\n\
\tEE: extract the total energy. There is no options. Examples:\n\n\
\t    %s -m EE -p 'id*M=' -f out\n\n\
\t          This extracts all Etots from all directories named id*M=...\n\
\t          where * can be anything and ... after M= is usually the value of M for\n\
\t          this particular directory/calculation\n\
\t          The result is a table of M and Etot values\n\n\
\t    %s -m EE -p M= -f out\n\n\
\t          Same as above, but for directories called M=...\n\n\
\t    %s -m EE -p . -f out\n\n\
\t          Extract last total energy from file out in current directory\n\
\t          Equivalent to %s -m EE or %s\n\n\
\t    %s -m EE -p . -f out -a\n\n\
\t          Extract total energies for all iteration steps from current directory\n\
\t          The resulting table has columns 'iteration step' 'total energy'\n\
\t          Equivalent to %s -a\n\n\
\t    %s -m EE -p . -f out -a -x\n\
\t    %s -m EE -p . -f out -a -xm\n\n\
\t          Extract total energies for all iteration steps from current directory\n\
\t          and send them to xfbp (-x) or  xmgrace (-xm)\n\
\t          This is equivalent to %s -a -x or %s -a -xm\n\n\
\n\
\tSS: total gross spin. no options.\n\
\n\
\tit: iteration progress\n\
\n\
\tfit: force iteration progress\n\
\n\
\ttime: timing of one cycle\n\
\n\
\tterm: termination of process\n\
\n\
\tSSat: total gross spin of atom. Options: site-number. Examples\n\
\t      %s -m SSat 13 -p M= -f out\n\
\n\
\tN_net/N_gros/S_net/S_gros:  individual population numbers. \n\
\t      Options:  site-number orbital-number.\n\
\t      orbital-number is the number of the orbital in the order as printed in the \n\
\t      population analysis.\n\
\t      If orbital-number is out of range the total site population number is printed.\n\
\t      For the N_gros cases there is one more number than for N_net and S_..., which\n\
\t      is the number of excess electrons of the site.\n\
\tLzat: the orbital moment of an atom. Options: site-number. Example\n\
\t      %s -m Lzat 12 -p U=\n\
\n\
\tBfsm: the auxillary magnetic field needed to set the FSM moment.\n\
\n\
\tdBfsm:the change of the auxillary magnetic field needed to set the FSM moment.\n\
\t      This is for controlling what's going on in full relativistic FSM calculations.\n\
\n",EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),
	 EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),EXECNAME(),
	 EXECNAME(),EXECNAME());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Control{
public:
  Control():pmode(OutGrep::TotalEnergy),prefix("."),pref(prefix),file("out"),
	    element("\?\?"),site(-1),orbital(-1),all(false),xmgrace(false),
	    xfbp(false){}


  void setPrefix(const String &p){
    prefix=pref=p;
    pref.replace("*",".*",0,true);
    if(prefix[prefix.length()-1] != '*'  && prefix != "."){
      prefix+='*';
    }
  }

  OutGrep::Mode mode()const{return pmode;}

  void setMode(OutGrep::Mode m){
    pmode=m;
    psmode=OutGrep::modeName(m);
  }
  void setMode(const String ms){
    psmode=ms;
    pmode=OutGrep::modeFromString(psmode());
  }

  void checkArgumentSanity()const {
    if((pmode==OutGrep::AtomSpin||
	pmode==OutGrep::AtomOrbitalMoment||
	pmode==OutGrep::AtomEFG) && (site<1)){
      throw Except(Sprintf("mode %s: missing or invalid site argument\n",psmode()));
    }else if(pmode==OutGrep::AtomSpinGros||
	     pmode==OutGrep::AtomSpinNet ||
	     pmode==OutGrep::AtomOccuGros||
	     pmode==OutGrep::AtomOccuNet   ){
      if(site<1)
	throw Except(Sprintf("mode %s: missing or invalid site argument\n",psmode()));
;
      if(orbital<1)
	throw Except(Sprintf("mode %s: missing or invalid orbital argument\n",psmode()));
    }
  }

private:
  OutGrep::Mode pmode;
  String psmode;
public:
  String prefix;
  String pref;
  String file;
  String element;
  long site;
  long orbital;
  bool all,xmgrace,xfbp;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
class List{
public:
  class Node{
  public:
    Node():pprev(this),pnext(this){}
    Node(const T &v):pdata(v),pprev(this),pnext(this){}

    T &operator*(){return pdata;}
    
    void prepend(const T &v){
      Node *n=new Node(v);
      n->pprev=pprev;
      pprev->pnext=n;
      n->pnext=this;
      pprev=n;
    }
    
    void append(const T &v){
      Node *n=new Node(v);
      n->pnext=pnext;
      pnext->pprev=n;
      n->pprev=this;
      pnext=n;
    }
    
    inline Node *next()const{ return this->pnext;}
    inline Node *prev()const{ return this->pprev;}
    
  private:
    T pdata;
    Node *pprev;
    Node *pnext;
  };
public:
  List():psentinel(new Node()){}

  inline void append(const T &v){
    psentinel->prepend(v);
  }

  inline bool isEmpty()const{return psentinel->prev()==psentinel;}

  inline void insertAfter(Node *current,const T&v){
    current->append(v);
  }

  inline Node *last()const{return psentinel->prev();}
  inline Node *first()const{return  psentinel->next();}
  
  inline Node *begin()const{return  psentinel;}
  inline Node *end()const{return  psentinel;}

  //  typedef List<T>::Node ListNode;
  private:

  Node *psentinel;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */

typedef Pair<double,String> DSPair;
typedef Pair<String,String> SSPair;

// for -a option
class AData{
public:
  AData(){}
  AData(const String xp):xpref(xp){}
  
  String xpref;
  //               index  y
  ValueArray<DSPair > iandy;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static int qSortPairCmp(const SSPair *p1, const SSPair *p2)
{
  double d1,d2;
  try{
    d1=p1->v1.convertToReal();
    d2=p2->v1.convertToReal();
    if (d1<d2) 
      return -1;
    else if (d1>d2) 
      return 1;
    else 
      return 0;
  }catch(...){
    return String::cmp(p1->v1,p1->v2);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main(int argc,const char **argv)
{

  if(0){
    Wigner3J::Wigner3JTest();
    exit(1);
  }

  
  if(0){

    List<String> ilist;
    ilist.append("Hello");
    ilist.append("my");
    ilist.append("dear");
    for( List<String>::Node *n=ilist.first();n!=ilist.end();n=n->next()){
      myDebug()<<**n;
    }
    
    ilist.last()->append("world");
    ilist.first()->prepend("Hmm,");
    //    ilist.insertAfter(ilist.last(),"world");

    myDebug();
    for( List<String>::Node *n=ilist.last();n!=ilist.begin();n=n->prev()){
      myDebug()<<**n;
    }
    myDebug()<<"first"<<**ilist.first();
    myDebug()<<"last"<<**ilist.last();
    myDebug()<<ilist.isEmpty();
    exit(0);
  }

  bool DEBUG=false;

  EXECNAME=argv[0];
  if(DEBUG) myDebug()<<"EXECNAME="<<EXECNAME;
  

  Control control;
  
  try{
  
    long narg=1;
    while(narg<argc){
      if(DEBUG) myDebug()<<"argv["<<narg<<"]="<<argv[narg];
      if(strcmp(argv[narg],"-f")==0){
	narg++;
	if(narg<argc) control.file=argv[narg];
	else throw Except("-f needs filename as argument\n");
      }else if(strstr(argv[narg],"-a")==argv[narg]){
	control.all=true;
      }else if(strstr(argv[narg],"-xm")==argv[narg]){
	control.xmgrace=true;
      }else if(strstr(argv[narg],"-x")==argv[narg]){
	control.xfbp=true;
      }else if(strcmp(argv[narg],"-h")==0){
	usage();
	die("");
      }else if(strcmp(argv[narg],"-m")==0){
	narg++;
	if(narg<argc) control.setMode(argv[narg]);
	else throw Except("-m needs mode argument\n");
	
	if( control.mode()==OutGrep::AtomSpin||
	    control.mode()==OutGrep::AtomOrbitalMoment||
	    control.mode()==OutGrep::AtomEFG   ){
	  narg++;
	  if(narg<argc){
	    if(!String::islong(&control.site,argv[narg])) 
	      control.checkArgumentSanity();
	  }
	}else if( control.mode()==OutGrep::AtomSpinGros ||
		  control.mode()==OutGrep::AtomSpinNet  ||
		  control.mode()==OutGrep::AtomOccuGros ||
		  control.mode()==OutGrep::AtomOccuNet    ){
	  narg++;
	  if(narg<argc){
	    if(!String::islong(&control.site,argv[narg])) 
	      control.checkArgumentSanity();
	    narg++;
	  }
	  if(narg<argc){
	    if(!String::islong(&control.orbital,argv[narg])) 
	      control.checkArgumentSanity();
	  }
	}
	
      }else if(strcmp(argv[narg],"-p")==0){
	narg++;
	if(narg<argc) control.setPrefix(argv[narg]);
	else throw Except("-p needs prefix argument\n");
      }else{
	throw Except(Sprintf("unknown argument '%s'\n",argv[narg]));
      }
      narg++;
    }
    

    control.checkArgumentSanity();
  

    


    if(DEBUG){
      myDebug()<<"mode=<"<<control.mode()<<">";
      myDebug()<<"prefix=<"<<control.prefix<<">";
      myDebug()<<"pref=<"<<control.pref<<">";
      myDebug()<<"file=<"<<control.file<<">";
      myDebug()<<"all=<"<<control.all<<">";
      myDebug()<<"xmgrace=<"<<control.xmgrace<<">";
      myDebug()<<"xfbp=<"<<control.xfbp<<">";
      //myDebug()<<"element=<"<<control.element<<">";
      myDebug()<<"site=<"<<control.site<<">";
      myDebug()<<"orbital=<"<<control.orbital<<">";
    }


    OutGrep poutgreper;
    poutgreper.setSilentOpen(true);
    poutgreper.setSite(control.site);
    poutgreper.setOrbital(control.orbital);
    
    regex_t preg;
    regcomp(&preg,control.pref(),0);


    ValueArray<SSPair > data;
    PtrArray<AData> adata;

    glob_t globbuf;
    glob(control.prefix(),0,NULL,&globbuf);
    for(unsigned int i=0;i<globbuf.gl_pathc;i++){
      String d=globbuf.gl_pathv[i];
      //myDebug()<<d;
      String  x=d;
      if( d == ".." ) continue;

      const long nmatch=10;
      regmatch_t pmatch[nmatch];
      if(!regexec(&preg,x(),nmatch,pmatch,0)){
	if(pmatch[0].rm_so>=0){
	  x=x.subString(pmatch[0].rm_eo+0);
	}
      }
      
      
      poutgreper.setOutfile(Sprintf("%s/%s",d(),control.file()));
      ValueArray<double> vx;
      ValueArray<String> vy;
      poutgreper.grep(control.mode(),vx,vy);
      
      
      
      if(control.all){
	// xres<<vx;
	// yres<<vy;
	AData *ad=new AData(x);
	for(int j=0;j<vy.size();j++){
	  ad->iandy<<makePair(vx.at(j),vy.at(j));
	  //fprintf(FH,"%g %s\n",vx.at(j),vy.at(j)());
	}
	adata<<ad;
      }else{
	data<<makePair(x,(vy.size()>0?vy.last()():String("0")));
      }
    
    }// for all files

    regfree(&preg);

    FILE *FH=stdout;
    if (control.xmgrace) {
      FH=popen("xmgrace -nxy -","w");
    }
    if (control.xfbp) {
      FH=popen("xfbp -xny -","w");
    }

    if(control.all){
      for(long i=-1;++i<adata.size();){
	const AData *ad=adata.at(i);
	long N=ad->iandy.size();
	fprintf(FH,"# \"iteration index\" \"%s %s %s\"\n",
		poutgreper.modeString(control.mode(),true)(),control.pref(),ad->xpref());
	for(long j=-1;++j<N;){
	  fprintf(FH,"%g %s\n",ad->iandy.at(j).v1,ad->iandy.at(j).v2());
	}
	fprintf(FH,"\n");
      }
    }else{
      data.qSort(qSortPairCmp);
      
      fprintf(FH,"# \"x(%s)\" \"%s\"\n",control.pref(),
	      poutgreper.modeString(control.mode(),true)());
      long N=data.size();
      for(long i=-1;++i<N;){
	fprintf(FH,"%s %s\n",data.at(i).v1(),data.at(i).v2());
      }
    }

    if(control.xmgrace || control.xfbp){
      pclose(FH);
    }








  }catch(Except &ex){
    usage();
    die(ex.msg());
  }

  
  return 0;
}
 


/*EOF*/ 
