//todo: make unfold similar

/*************************************************************************
 * @(#)MolecularOrbital.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   30 Oct 2010
 *************************************************************************
 */

#include "MolecularOrbital.hh"
#include "Exception.hh"
#include "FString.hh"
#include "Except.hh"
#include "Assorted.hh"
#include "Cmplx.hh"
#include "MyMath.hh"
#include "DMQuantAxis.hh"
#include "CoeffBasisInfo.hh"
#include "MyDebug.hh"
#ifdef MPIFPLOVERSION
#include <mpi.h>
#endif

using namespace XFPLO;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MolecularOrbitalData::printS(const String &el,bool molecular)const
{
  if(!molecular){
    return Sprintf("%2s(%ld) %s x=%s z=%s",el(),psite+1
		   ,porbital()
		   ,pxaxis.printS()()
		   ,pzaxis.printS()()
		   );
  }else{
    return Sprintf("%2s(%ld) %s: at %s fac= %g+i*%g  x=%s z=%s",el(),psite+1
		   ,porbital()
		   ,pposition.printS()()
		   ,pxaxis.printS()()
		   ,pzaxis.printS()()
		   ,pfactor.real()
		   ,pfactor.imag()
		   );
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MolecularOrbital::~MolecularOrbital()
{
  //myDebug() <<"MolecularOrbital dies";

  // foreach (MolecularOrbitalData *c,pmolorbcontriblist){
  //   delete c;
  // }
  pmolorbcontriblist.clear();

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
MolecularOrbitals::MolecularOrbitals()
  :p_U(0),pglobalspinzaxis(Vec3(0.,0.,1.))
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
MolecularOrbitals::~MolecularOrbitals()
{
  //myDebug() <<"MolecularOrbitals die";
  clean();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::clean()
{
  if(p_U) delete [] p_U; p_U=0;
  pnames.clear();

  pmolorblist.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void MolecularOrbital::writeToFile(FILE *FH)const
{
  if(pismolecular){
    fprintf(FH,"bwdef molecular %s\n",(pactive?"on":"off"));
    // off
    //     emin -2.5
    //     emax  3.0
    //     de    1.0
    fprintf(FH,"\tname %s\n",pname());
    for(long im=0;im<pmolorbcontriblist.size();im++){
      fprintf(FH,"\tcontrib\n");
      fprintf(FH,"\t\tsite %ld\n",pmolorbcontriblist.at(im)->site()+1);
      fprintf(FH,"\t\tposition %s\n"
	      ,pmolorbcontriblist.at(im)->position()
	      .printS("%30.15f"," ",true)()
	      );
      fprintf(FH,"\t\txaxis %s\n"
	      ,pmolorbcontriblist.at(im)->xaxis()
	      .printS("%30.15f"," ",true)()
	      );
      fprintf(FH,"\t\tzaxis %s\n"
	      ,pmolorbcontriblist.at(im)->zaxis()
	      .printS("%30.15f"," ",true)()
	      );
      fprintf(FH,"\t\torbital %s\n",pmolorbcontriblist.at(im)->orbital()());
      fprintf(FH,"\t\tfac %s %s\n"
	      ,Sprintf("%12.8f",pmolorbcontriblist.at(im)->factor().real())
	      .stripBlanks().chop("0")()
	      ,Sprintf("%12.8f",pmolorbcontriblist.at(im)->factor().imag())
	      .stripBlanks().chop("0")()
	      );
    }    
  }else{
    fprintf(FH,"bwdef simple %s\n",(pactive?"on":"off"));
    fprintf(FH,"\tcontrib\n");
    fprintf(FH,"\t\tsite %ld\n",pmolorbcontriblist.at(0)->site()+1);
    fprintf(FH,"\t\torbital %s\n",pmolorbcontriblist.at(0)->orbital()());
    fprintf(FH,"\t\txaxis %s\n"
	    ,pmolorbcontriblist.at(0)->xaxis()
	    .printS("%30.15f"," ",true)()
	    );
    fprintf(FH,"\t\tzaxis %s\n"
	    ,pmolorbcontriblist.at(0)->zaxis()
	    .printS("%30.15f"," ",true)()
	    );
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void MolecularOrbitals::writeToFile(FILE *FH)
{
  if(!FH) return;

  //myDebug() << "writeToFile ";

  for(long ic=0;ic<pmolorblist.size();ic++){
    //myDebug() << "ic " <<ic ;
    pmolorblist.at(ic)->writeToFile(FH);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MolecularOrbitals::readFromFile(FILE *FH,String &coeffilename
				       ,String &bwfilename,bool prot,
				       StringList &output)
//,bool &useinfplo)
{
  if(!FH) return "";
  //  myDebug() << "readFromFile ";

  clean();

  String warn;

  //useinfplo=false;
  
  bool molecular=false;
  bool startdef=false;
  bool startcontrib=false;
  String line;
  long nline=0;
  bool havesite=false;
  bool haveorb=false;
  bool havefac=false;
  bool havepos=false;
  bool havexaxis=false;
  bool havezaxis=false;
  while (getLine(FH,line)){
    nline++;
    line=line.chop("\n").stripBlanks();

    if(line.subString(0,11).lowerCase()=="coefficients"){
      coeffilename=line.subString(12).stripBlanks();
    }else if(line.subString(0,7).lowerCase()=="bweights"){
      bwfilename=line.subString(8).stripBlanks();

    }else if(line.subString(0,4).lowerCase()=="bwdef"){
      if(startcontrib && 
	 !((havesite&&haveorb&&havexaxis&&havezaxis&&!molecular)||
	   (havesite&&haveorb&&havefac&&havepos&&havexaxis&&havezaxis&&molecular))
	 ){

	throw Exception(Sprintf("Line %ld: new \"bwdef\" but some keywords"
				" missing in previous definition."
				,nline));
      }
      havesite=false;
      haveorb=false;
      havefac=false;
      havepos=false;
      havexaxis=false;
      havezaxis=false;

      havexaxis=true;
      havezaxis=true;

      line=line.subString(5).stripBlanks();
      StringList sl=line.split(" \t");
      startdef=true;
      startcontrib=false;
      if(line.subString(0,2)=="sim"){
	molecular=false;
	pmolorblist.append(new MolecularOrbital());
	pmolorblist.last()->setName(Sprintf("%ld",pmolorblist.size()));
      }else if(line.subString(0,2)=="mol"){
	molecular=true;
	pmolorblist.append(new MolecularOrbital("name ??"));
      }
      if(sl.size()==2 && sl[1]=="off"){
	pmolorblist.last()->setActive(false);
      }
    }else if(line.subString(0,3).lowerCase()=="name"){
      line=line.subString(4).stripBlanks();
      if(!startdef){
	throw Exception(Sprintf("Line %ld: \"name\" outside of \"bwdef\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      pmolorblist.last()->setName(line);
      pmolorblist.last()->setName(line);
    }else if(line.subString(0,6).lowerCase()=="contrib"){
      if(startcontrib && 
	 !((havesite&&haveorb&&havexaxis&&havezaxis&&!molecular)||
	   (havesite&&haveorb&&havefac&&havepos&&havexaxis&&havezaxis&&molecular))
	 ){
	//myDebug() << molecular <<havesite << haveorb;
	throw Exception(Sprintf("Line %ld: new \"contrib\" but some keywords"
				" missing in previous definition."
				,nline));
      }
      havesite=false;
      haveorb=false;
      havefac=false;
      havepos=false;
      havexaxis=false;
      havezaxis=false;

      havexaxis=true;
      havezaxis=true;

      startdef=false;
      startcontrib=true;
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: \"contrib\" outside of \"bwdef\"?"
				,nline));
      }
      MolecularOrbital *mo=pmolorblist.last();
      mo->append(new MolecularOrbitalData());
      
    }else if(line.subString(0,3).lowerCase()=="site"){
      line=line.subString(4).stripBlanks();
      long site;
      if(!String::islong(&site,line())){
	throw Exception(Sprintf("Line %ld: \"site %s\" should be \"site INTEGER\"."
				,nline,line()));
      }
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"site\" outside of \"contrib\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setSite(site-1);
      havesite=true;
    }else if(line.subString(0,6).lowerCase()=="orbital"){
      line=line.subString(7).stripBlanks();
      String orbital=line;
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"orbital\" outside of \"contrib\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setOrbital(orbital);
      String::StateLabels ret=md->orbtype();
      if(mo->isMolecular()){
	// if(!(ret==String::SL_nlm || ret==String::SL_nljm))
	//   throw Exception(Sprintf("Line %ld: \"orbital %s\" should be "
	// 			  "\"orbital ORBITAL\", where "
	// 			  "ORBITAL is of nlm or nljm type and valid\"?"
	// 			  ,nline,orbital()));
      }else{
	if(ret==String::SL_invalid)
	  throw Exception(Sprintf("Line %ld: \"orbital %s\" should  be "
				  "\"orbital ORBITAL\" with valid ORBITAL ?"
				  ,nline,orbital()));
      }
      
      haveorb=true;

    }else if(line.subString(0,2).lowerCase()=="fac"){
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"fac\" outside of \"contrib\"?"
 				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      
      line=line.subString(3).stripBlanks();
      StringList sl=line.split(" \t");
      bool wrong=false;
      wrong|=(sl.size()<1||sl.size()>2);
      double re=0.,im=0.;
      try{
	re=sl[0].convertToFortranReal();
      }catch(Except ex){
	wrong|=true;
      }
      if(sl.size()==2){
	try{
	  im=sl[1].convertToFortranReal();
	}catch(Except ex){
	  wrong|=true;
	}
      }
      
      if(wrong){
	throw Exception(Sprintf("Line %ld: \"fac %s\" should be "
				"\"fac REAL [IMAG]\"."
				,nline,line()));
      }
      
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      if(!mo->isMolecular()){
	warn+=Sprintf("Line %ld: \"fac %s\" has a meaning only for "
		      "molecular types\"?"
		      ,nline,line());
      }
      
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setFactor(Cmplx(re,im));
      
      havefac=true;
 
    }else if(line.subString(0,7).lowerCase()=="position"){
      line=line.subString(8).stripBlanks();
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"position\" outside of \"contrib\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      StringList sl=line.split(" \t");
      bool wrong=false;
      wrong|=(sl.size()!=3);
      Vec3 p;
      try{
	p=Vec3(sl);
      }catch(Except ex){
	wrong|=true;
      }
      if(wrong){
	throw Exception(Sprintf("Line %ld: \"position %s\" should be "
				"\"position x y z\"."
				,nline,line()));
      }
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      if(!mo->isMolecular()){
	warn+=Sprintf("Line %ld: \"position %s\" has a meaning only for "
		      "molecular types\"?"
		      ,nline,line());
      }
      
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setPosition(p);

     
      havepos=true;
 
    }else if(line.subString(0,4).lowerCase()=="xaxis"){
      line=line.subString(5).stripBlanks();
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"xaxis\" outside of \"contrib\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      StringList sl=line.split(" \t");
      bool wrong=false;
      wrong|=(sl.size()!=3);
      Vec3 p;
      try{
	p=Vec3(sl);
      }catch(Except ex){
	wrong|=true;
      }
      if(wrong){
	throw Exception(Sprintf("Line %ld: \"xaxis %s\" should be "
				"\"xaxis x y z\"."
				,nline,line()));
      }
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setXaxis(p);

     
      havexaxis=true;
 
    }else if(line.subString(0,4).lowerCase()=="zaxis"){
      line=line.subString(5).stripBlanks();
      if(!startcontrib){
	throw Exception(Sprintf("Line %ld: \"zaxis\" outside of \"contrib\"?"
				,nline));
      }
      if(!pmolorblist.size()){
	throw Exception(Sprintf("Line %ld: missing  \"bwdef\"?"
				,nline));
      }
      StringList sl=line.split(" \t");
      bool wrong=false;
      wrong|=(sl.size()!=3);
      Vec3 p;
      try{
	p=Vec3(sl);
      }catch(Except ex){
	wrong|=true;
      }
      if(wrong){
	throw Exception(Sprintf("Line %ld: \"zaxis %s\" should be "
				"\"zaxis x y z\"."
				,nline,line()));
      }
      MolecularOrbital *mo=pmolorblist.last();
      if(!mo->numOfContribs()){
	throw Exception(Sprintf("Line %ld: missing \"contrib\"?",nline));
      }
      
      MolecularOrbitalData *md=(*mo)[mo->numOfContribs()-1];
      md->setZaxis(p);

     
      havezaxis=true;
 
    }
  }  
  if(startcontrib && 
     !((havesite&&haveorb&&!molecular)||
       (havesite&&haveorb&&havefac&&havepos&&molecular))
     ){
    //myDebug() << molecular <<havesite << haveorb;
    throw Exception(Sprintf("Line %ld: end of file but some keywords"
			    " missing in previous definition."
			    ,nline));
  }
  
  


  output.clear();
  if(prot){
    output<<"         Fatband definitions\n";
    output<<"\n";
    output<<"\n";
    output<<Sprintf("Number of Fatband definitions:%5ld\n",pmolorblist.size());
    output<<"\n";
    output<<Sprintf("bweights file: %s\n",bwfilename());
    output<<"\n";

    for(long ic=0;ic<pmolorblist.size();ic++){
      const MolecularOrbital *mo=pmolorblist.at(ic);
      if(!mo->isActive())continue;
      if(!mo->isMolecular()){
	
	const MolecularOrbitalData *md=mo->at(0);

	output<<Sprintf("bwdef simple\n");
	output<<Sprintf("    contrib\n");
	output<<Sprintf("         site    %5ld\n",md->site()+1);
	output<<Sprintf("         position%15.8f  %15.8f  %15.8f\n"
			,md->position()[0],md->position()[1],md->position()[2]);
	output<<Sprintf("         xaxis   %15.8f  %15.8f  %15.8f\n"
			,md->xaxis()[0],md->xaxis()[1],md->xaxis()[2]);
	output<<Sprintf("         zaxis   %15.8f  %15.8f  %15.8f\n"
			,md->zaxis()[0],md->zaxis()[1],md->zaxis()[2]);
	output<<Sprintf("         orbital %s\n",md->orbital()());
	output<<Sprintf("         factor  %s+i*%s\n"
			,String::doubleToString("%30.12f",md->factor().real(),true)()
			,String::doubleToString("%30.12f",md->factor().imag(),true)());
	output<<Sprintf("\n");
	output<<Sprintf("\n");
	
      }else{
	output<<Sprintf("bwdef molecular\n");
	output<<Sprintf("    name         %s\n",mo->name()());
	for(int im=0;im<mo->numOfContribs();im++){
	  const MolecularOrbitalData *md=mo->at(im);
	  output<<Sprintf("    contrib\n");
	  output<<Sprintf("         site    %5ld\n",md->site()+1);
	  output<<Sprintf("         position%15.8f  %15.8f  %15.8f\n"
			  ,md->position()[0],md->position()[1],md->position()[2]);
	  output<<Sprintf("         xaxis   %15.8f  %15.8f  %15.8f\n"
			  ,md->xaxis()[0],md->xaxis()[1],md->xaxis()[2]);
	  output<<Sprintf("         zaxis   %15.8f  %15.8f  %15.8f\n"
			  ,md->zaxis()[0],md->zaxis()[1],md->zaxis()[2]);
	  output<<Sprintf("         orbital %s\n",md->orbital()());
	  output<<Sprintf("         factor  %s+i*%s\n"
			  ,String::doubleToString("%30.12f",md->factor().real(),true)()
			  ,String::doubleToString("%30.12f",md->factor().imag(),true)());
	  output<<Sprintf("\n");
	
	}
	output<<Sprintf("\n");
      }
    }  
  }


  return warn;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::setup(const CoeffBasisInfo &cbi,bool prot,
			      StringList &output)
{
  pnumweights=0;
  for(long ibwd=-1;++ibwd<pmolorblist.size();){
    const MolecularOrbital *mo=pmolorblist.at(ibwd);
    if(!mo->isActive())continue;
    if(!mo->isMolecular()){
      if(mo->numOfContribs()!=1){
	throw Exception(Sprintf("simple bwdef %ld needs exactly one \"contrib\"?",ibwd+1));
      }
    }

    long nw=1;
    long ret0=0;
    for(long icn=-1;++icn<mo->numOfContribs();){
      const MolecularOrbitalData *md=mo->at(icn);
      long isa=md->site();

      if(isa<0 || isa >= cbi.nsite()){
	  throw Exception(Sprintf("bwdef %s, contrib %ld refers to an "
				  "invalid site number",mo->name()(),icn+1));
      }
	
      String::StateLabels ret=md->orbtype();
      //if(ret==String::SL_nlms) ret=String::SL_nlm;
      if(ret == String::SL_invalid){
	throw Exception(Sprintf("bwdef %s contrib %ld: orbital %s incorrect",
				mo->name()(),icn+1,md->orbital()()));
      }
      
      if(ret!=String::SL_all && ret!=String::SL_allnlm){
	bool found=false;
	for(long ivr=-1;++ivr<cbi.nRv(isa);){
	  if(cbi.FREL() && (ret==String::SL_nljm || ret==String::SL_nlj) ){
	    if(md->nmain()==cbi.nnRv(ivr,isa) &&
	       md->l()==cbi.lRv(ivr,isa) &&
	       md->j()==cbi.jRv(ivr,isa) ) found=true;
	  }else{
	    if(md->nmain()==cbi.nnRv(ivr,isa) &&
	       md->l()==cbi.lRv(ivr,isa) ) found=true;
	  }
	}
	if(!found){
	  throw Exception(Sprintf("bwdef %s contrib %ld: orbital %s "
				  "does not exist in current setup!",
				  mo->name()(),icn+1,md->orbital()()));
	}
      }
      if(ret==String::SL_nlj || ret==String::SL_nljm){
	if(!cbi.FREL()){
	  throw Exception(Sprintf("bwdef %s contrib %ld: orbital %s is "
				  "relativistic, calculation not!",
				  mo->name()(),icn+1,md->orbital()()));
	}
      }

      if(mo->isMolecular()){
	if(ret!=String::SL_nljm && ret!=String::SL_nlm){
	  throw Exception(Sprintf("bwdef %s contrib %ld: orbital %s "
				  "is not a simple orbital!",
				  mo->name()(),icn+1,md->orbital()()));
	}
	if(ret==String::SL_nlm && cbi.FREL()) nw=2;
	if(icn==0) {
	  ret0=ret;
	}else if(ret0!=ret){
	  throw Exception(Sprintf("bwdef %s contrib %ld: orbital %s "
				  "is of different type than contrib %ld: orbital %s.",
				  mo->name()(),icn+1,md->orbital()(),1,
				  mo->at(0)->orbital()()));
	}
      }else if(!mo->isMolecular()){
	long deg=0;
	if(ret == String::SL_nl){
	  deg=2*md->l()+1;
	  if(cbi.FREL()) deg=2*deg;
	}else if(ret == String::SL_nlj){
	  deg=md->j()+1;
	}else if(ret == String::SL_nlm){
	  deg=1;
	  if(cbi.FREL()) deg=2;
	}else if(ret == String::SL_nljm){
	  deg=1;
	}else if(ret == String::SL_all){
	  deg=cbi.blocklength(isa);
	}else if(ret == String::SL_allnlm){
	  deg=cbi.blocklength(isa);
	}
	pnumweights+=deg;
      }
    }//icn
    if(mo->isMolecular()){
      pnumweights+=nw;
    }
  }//ibwd

  pnames.adjustSize(pnumweights);

  long iw=-1;
  for(long ibwd=-1;++ibwd<pmolorblist.size();){
    const MolecularOrbital *mo=pmolorblist.at(ibwd);
    if(!mo->isActive())continue;
    if(!mo->isMolecular()){
      for(long icn=-1;++icn<mo->numOfContribs();){
	const MolecularOrbitalData *md=mo->at(icn);
	long isa=md->site();
	String::StateLabels ret=md->orbtype();
	
	long nmain=md->nmain();
	long l=md->l();
	long j=md->j();
	long m=md->m();

	if(ret == String::SL_nl){
	  for(long m=-1;++m<2*l+1;){
	    if(cbi.FREL()){
	      for(long ms=-1;++ms<2;){
		iw++;
		pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld%s"
				   ,cbi.element(isa)(),isa+1
				   ,nmain,String::intToSpdf(l),m-l,
				   (ms==0?"up":"dn"));

	      }
	    }else{
	      iw++;
	      pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld"
	       			 ,cbi.element(isa)(),isa+1
				 ,nmain,String::intToSpdf(l),m-l);
	    }
	  }
	}else if(ret == String::SL_nlj){
	  for(long m=-1;++m<j+1;){
	    iw++;
	    pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%1ld/2%+2ld/2"
			       ,cbi.element(isa)(),isa+1
			       ,nmain,String::intToSpdf(l),j,m*2-j);
	  }
	}else if(ret == String::SL_nlm){
	  if(cbi.FREL()){
	    for(long ms=-1;++ms<2;){
	      iw++;
	      pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld%s"
				 ,cbi.element(isa)(),isa+1
				 ,nmain,String::intToSpdf(l),m,
				 (ms==0?"up":"dn"));
	    }
	  }else{
	    iw++;
	    pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld"
			       ,cbi.element(isa)(),isa+1
			       ,nmain,String::intToSpdf(l),m);
	  }


	}else if(ret == String::SL_nljm){
	  iw++;
	  pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%1ld/2%+2ld/2"
			     ,cbi.element(isa)(),isa+1
			     ,nmain,String::intToSpdf(l),j,m);

	}else if(ret == String::SL_all){
	  for(long ivr=-1;++ivr<cbi.nRv(isa);){
	    for(long m=-1;++m<cbi.degRv(ivr,isa);){
	      iw++;
	      if(cbi.FREL()){
		pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%1ld/2%+2ld/2"
				   ,cbi.element(isa)(),isa+1
				   ,cbi.nnRv(ivr,isa),
				   String::intToSpdf(cbi.lRv(ivr,isa)),
				   cbi.jRv(ivr,isa),
				   2*m-cbi.jRv(ivr,isa));

	      }else{
		pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld"
				   ,cbi.element(isa)(),isa+1
				   ,cbi.nnRv(ivr,isa),
				   String::intToSpdf(cbi.lRv(ivr,isa)),
				   m-cbi.lRv(ivr,isa));
	      }
	    }
	  }
	}else if(ret == String::SL_allnlm){
	  for(long ivr=-1;++ivr<cbi.nRv(isa);){
	    if(cbi.FREL() && cbi.kapRv(ivr,isa)>0) continue;
	    long l=cbi.lRv(ivr,isa);
	    long deg=2*l+1;
	    for(long m=-1;++m<deg;){
	      if(cbi.FREL()){
		for(long ms=-1;++ms<2;){
		  iw++;
		  pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld%s"
				     ,cbi.element(isa)(),isa+1,cbi.nnRv(ivr,isa),
				     String::intToSpdf(l),m-l,(ms==0?"up":"dn"));

		}
	      }else{
		iw++;
		pnames[iw]=Sprintf("%2s(%03ld)%1ld%1c%+2ld"
				   ,cbi.element(isa)(),isa+1
				   ,cbi.nnRv(ivr,isa),
				   String::intToSpdf(l),m-l);
	      }
	    }
	  }
	}

      }
    }else if(mo->isMolecular()){
      const MolecularOrbitalData *md=mo->at(0);
      String::StateLabels ret=md->orbtype();
      if(cbi.FREL() && ret==String::SL_nlm){
	for(long ms=-1;++ms<2;){
	  iw++;
	  pnames[iw]=mo->name()+(ms==0?"up":"dn");
	}
      }else{
	iw++;
	pnames[iw]=mo->name();
      }
    }
  }

  
  if(prot){
    output<<Sprintf("\n");
    output<<Sprintf("Number of resulting weights %5ld\n",pnumweights);
    output<<Sprintf("\n");
    output<<Sprintf("will create the following fat bands\n");
    for(long iw=-1;++iw<pnames.size();){
      output<<Sprintf("%5ld %s\n",iw+1,pnames.at(iw)());
    }
    output<<Sprintf("\n");
    output<<Sprintf("======================"
		    "==================================================\n");
  }  

  resetTrans();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::rotatedUnsrel(long l,long mv0
				      ,long num,long iz
				      ,long &iw,long pnvdim
				      ,const String &el
				      ,MolecularOrbitalData *md
				      ,const Cmplx &fac
				      ,bool molcontrib)
{
  SqMatrix<Cmplx> &Dlm=md->Dlm(l);
  if(Dlm.size()==0){
    try{
      DMQuantAxis::calcAxesTrans(md->xaxis(),md->zaxis(),l,Dlm);
    }catch(Except ex){
      throw Exception(Sprintf("Orbital %s\n%s",md->printS(el,false)()
			      ,ex.msg()));
    }
  }
  
  long deg=2*l+1;
  for(long mv=mv0;mv<mv0+num;mv++){
    long iz0=iz;
    for(long m2=0;m2<deg;m2++){
      iz0++;
      p_U[iz0+pnvdim*iw]+=Dlm(m2,mv+l)*fac;
    }
    if(!molcontrib){
      iw++;
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::rotatedUfrel(long j,long mu0,
				     long num,long iz,
				     long &iw,long pnvdim,
				     const String &el,
				     MolecularOrbitalData *md,
				     const Cmplx &fac,
				     bool molcontrib)
{
  // j is 2*j-qn
  // mu0 is the 2*\mu-quantum number 
  // num is either 1 or \Delta_j=2 * j-qn + 1 = j+1
  SqMatrix<Cmplx> &Djmm=md->Djmm(j);
  if(Djmm.size()==0){
    try{
      DMQuantAxis::calcAxesTransJ(md->xaxis(),md->zaxis(),j,Djmm);
    }catch(Except ex){
      throw Exception(Sprintf("Orbital %s\n%s",md->printS(el,false)()
			      ,ex.msg()));
    }
  }

  long mv0=(mu0+j)/2; // zero-based index of mu0
  for(long mv=mv0;mv<mv0+num;mv++){ // zero based 
    long iz0=iz;
    for(long m2=0;m2<j+1;m2++){// zero based
      iz0++;
      p_U[iz0+pnvdim*iw]+=Djmm(m2,mv)*fac;
    }
    if(!molcontrib){
      iw++;
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::rotatedUfrelNL(long l,long mv0,
				       long num,long iz,
				       long &iw,long pnvdim,
				       const String &el,
				       MolecularOrbitalData *md,
				       const Vec3 &globalspinzaxis,
				       const Cmplx &fac)
{
  long degl=2*l+1;
  long dim=2*degl;
  SqMatrix<Cmplx> &Dlms=md->Dlms(l);
  if(Dlms.size()==0){
    try{
      SqMatrix<Cmplx> ytr;
      DMQuantAxis::calcAxesTrans(md->xaxis(),md->zaxis(),l,ytr);
      SqMatrix<Cmplx> T;
      if(md->useGlobalSpinZaxis()){
	T=relTOpseudo(globalspinzaxis,l);
      }else{
	T=relTOpseudo(md->spinZaxis(),l);
      }
      
      Dlms.resize(dim);
      for(int km=-1;++km<dim;){
	for(int ms=-1;++ms<2;){
	  for(int m=-1;++m<degl;){
	    Dlms(km,m+degl*ms)=SqMatrix<Cmplx>::dotcol(T,ms*degl,km,ytr,0,m,degl);
	  }
	}
      }
      
    }catch(Except ex){
      throw Exception(Sprintf("Orbital %s\n%s",md->printS(el,false)()
			      ,ex.msg()));
    }
  }

  
  for(long mv=mv0;mv<mv0+num;mv++){ // mv \in [-l,l]
    for(int ms=-1;++ms<2;){
      long iz0=iz;
      for(int km=-1;++km<dim;){
	p_U[(++iz0)+pnvdim*iw]+=Dlms(km,mv+l+ms*degl)*fac;
      }
      iw++;
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::buildMatrix(const Vec3 &kpoint,const CoeffBasisInfo &cbi)
{
  //printf("kpoint %16.7f %16.7f %16.7f:\n",kpoint[0],kpoint[1],kpoint[2]);

  //myDebug() << "MolecularOrbitals::buildMatrix";
  pnvdim=cbi.nvdim();
  
  if(p_U) delete [] p_U; p_U=0;
  
  
  //printf("No. of weight combinations=%ld\n",pnumweights);

  


  p_U=new Cmplx[pnvdim*pnumweights];
  for(long i=0;i<pnvdim*pnumweights;i++) p_U[i]=0.;

  
  long iw=0;
  for(long ic=0;ic<pmolorblist.size();ic++){

    MolecularOrbital &mo=(*pmolorblist[ic]);
    if(!mo.isActive())continue;

    
    if(!mo.isMolecular()){
      MolecularOrbitalData *md=mo[0];
      long isa=md->site();
      const String &el=cbi.element(isa);

      
      String::StateLabels ret=md->orbtype();
      long nmain=md->nmain();
      long l=md->l();
      long j=md->j();
      long m=md->m();
      if(ret==String::SL_nl){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  if(cbi.FREL() && cbi.kapRv(iv,isa)>0) continue;
	  long deg=2*cbi.lRv(iv,isa)+1;
	  if(cbi.nnRv(iv,isa)==nmain && cbi.lRv(iv,isa)==l ){
	    
	    if(cbi.FREL()){
	      rotatedUfrelNL(l,-l,deg,iz,iw,pnvdim,el,md,pglobalspinzaxis,
			     Cmplx::c1);
	    }else{
	      rotatedUnsrel(l,-l,deg,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	    }
	    
	  }
	  if(cbi.FREL()){
	    iz+=2*deg;
	  }else{
	    iz+=deg;
	  }
	  
	}//iv
      }else if(ret==String::SL_nlj){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  long deg=cbi.degRv(iv,isa);
	  if( cbi.nnRv(iv,isa)==nmain && cbi.lRv(iv,isa)==l && 
	      cbi.jRv(iv,isa)==j ){
	    rotatedUfrel(j,-j,deg,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	  }
	  iz+=deg;
	}
      }else if(ret==String::SL_nlm){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  if(cbi.FREL() && cbi.kapRv(iv,isa)>0) continue;
	  long deg=2*cbi.lRv(iv,isa)+1;
	  if(cbi.nnRv(iv,isa)==nmain && cbi.lRv(iv,isa)==l ){
	    if(cbi.FREL()){
	      rotatedUfrelNL(l,m,1,iz,iw,pnvdim,el,md,pglobalspinzaxis,Cmplx::c1);
	    }else{
	      rotatedUnsrel(l,m,1,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	    }
	  }
	  if(cbi.FREL()){
	    iz+=2*deg;
	  }else{
	    iz+=deg;
	  }
	}
      }else if(ret==String::SL_nljm){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  long deg=cbi.degRv(iv,isa);
	  if( cbi.nnRv(iv,isa)==nmain && cbi.lRv(iv,isa)==l && 
	      cbi.jRv(iv,isa)==j ){
	    rotatedUfrel(j,m,1,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	  }
	  iz+=deg;
	}
      }else if(ret==String::SL_all){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  long deg=cbi.degRv(iv,isa);
	  long lv=cbi.lRv(iv,isa);
	  long jv=cbi.jRv(iv,isa);
	  if(cbi.FREL()){
	    rotatedUfrel(jv,-jv,deg,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	  }else{
	    rotatedUnsrel(lv,-lv,deg,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	  }
	  iz+=deg;
	}
      }else if(ret==String::SL_allnlm){
	long iz=cbi.blockstart(isa)-1;
	for(long iv=0;iv<cbi.nRv(isa);iv++){
	  if(cbi.FREL() && cbi.kapRv(iv,isa)>0) continue;
	  long lv=cbi.lRv(iv,isa);
	  long deg=2*lv+1;
	  if(cbi.FREL()){
	    rotatedUfrelNL(lv,-lv,deg,iz,iw,pnvdim,el,md,pglobalspinzaxis,
			   Cmplx::c1);
	  }else{
	    rotatedUnsrel(lv,-lv,deg,iz,iw,pnvdim,el,md,Cmplx::c1,false);
	  }
	  if(cbi.FREL()){
	    iz+=2*deg;
	  }else{
	    iz+=deg;
	  }
	}
      }else if(ret==String::SL_invalid){
      }
     
    }else{

      long incr=1;


      if(cbi.FREL() && mo.numOfContribs()>0 ){
	String::StateLabels ret=mo.at(0)->orbtype();
	if(ret==String::SL_nlm){
	  incr=2;
	}
      }


      for(int im=0;im<mo.numOfContribs();im++){
	MolecularOrbitalData *md=mo[im];
	long isa=md->site();
	const String &el=cbi.element(isa);
	double qq=md->position()*kpoint;
	Cmplx cex=exp(-Cmplx::ci*Cmplx(qq))*md->factor();
	//printf("qq=%g cex*f= %g %g \n",qq,cex.real(),cex.imag());	
	
	

	//printf("%s(%03ld) mix: %s\n",cbi.element(isa)(),isa+1
	//       ,mo.orbital()()
	//       );
	//myDebug() << md->printS(el,true)()<<cex.real();
	
	long nmain=md->nmain();
	long l=md->l();
	long j=md->j();
	long m=md->m();
	String::StateLabels ret=md->orbtype();
	if(ret==String::SL_nlm){
	  long iz=cbi.blockstart(isa)-1;
	  for(long iv=0;iv<cbi.nRv(isa);iv++){
	    if(cbi.FREL() && cbi.kapRv(iv,isa)>0) continue;
	    long deg=2*cbi.lRv(iv,isa)+1;
 	    if( cbi.nnRv(iv,isa)==nmain && cbi.lRv(iv,isa)==l ){
	      if(cbi.FREL()){
		rotatedUfrelNL(l,m,1,iz,iw,pnvdim,el,md,pglobalspinzaxis,cex);
		iw-=2;
	      }else{
		rotatedUnsrel(l,m,1,iz,iw,pnvdim,el,md,cex,true);
	      }
	    }
	    if(cbi.FREL()){
	      iz+=2*deg;
	    }else{
	      iz+=deg;
	    }
	  } // iv
	}else if(ret==String::SL_nljm){
	  if(cbi.FREL()){
	    long iz=cbi.blockstart(isa)-1;
	    for(long iv=0;iv<cbi.nRv(isa);iv++){
	      long deg=cbi.degRv(iv,isa);
	      if(nmain==cbi.nnRv(iv,isa)&& l==cbi.lRv(iv,isa) && 
		 j==cbi.jRv(iv,isa)){
		rotatedUfrel(j,m,1,iz,iw,pnvdim,el,md,cex,true);
	      }
	      iz+=deg;
	    }
	  }
	}else{
	  throw Exception(Sprintf("MolecularOrbital \n%s\n"
				  "Contribution to orbital combination is "
				  "not a single orbital!"
				  ,md->printS(el,true)()));

	}// ret==...
      }// contribs
      iw+=incr;

    }
  }// for all pmolorblist
  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MolecularOrbitals::bcast_data()
{
#ifdef MPIFPLOVERSION
  bool prot=false;
  
  int nprocs,rank;
  MPI_Comm_size(MPI_COMM_WORLD,&nprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
#define IAmMaster (rank==0)
#define IAmSlave (rank!=0)
  
  
  MPI_Bcast(pglobalspinzaxis(),3,MPI_DOUBLE,0,MPI_COMM_WORLD);
  MPI_Bcast(&pnumweights,1,MPI_LONG,0,MPI_COMM_WORLD);
  MPI_Bcast(&pnvdim,1,MPI_LONG,0,MPI_COMM_WORLD);

  long count=0;
  if(IAmMaster){
    for(long i=-1;++i<pnames.size();){
      count+=pnames[i].length()+1;
    }
  }
  MPI_Bcast(&count,1,MPI_LONG,0,MPI_COMM_WORLD);
  //if(prot) printf("MPI: proc%3d: count=%ld\n",rank,count);
  char *buf=new char[count];

  if(IAmMaster){
    char *p=buf;
    for(long i=-1;++i<pnames.size();){
      const char *el=pnames[i]();
      strcpy(p,el);
      p+=strlen(el)+1;
    }
  }
  
  MPI_Bcast(buf,count,MPI_CHAR,0,MPI_COMM_WORLD);
  long len=pnames.size();
  MPI_Bcast(&len,1,MPI_LONG,0,MPI_COMM_WORLD);
  //if(prot) printf("MPI: proc%3d: len=%ld\n",rank,len);
  

  if(IAmSlave){
    pnames.adjustSize(len);

    
    const char *p=buf;
    for(long i=-1;++i<len;){
      long l=strlen(p);
      pnames[i].ncopy(p,l);
      p+=l+1;
    }
    
  }
  delete [] buf;
  
  if(prot){
    printf("MPI: proc%3d: pnumweights=%ld pnvdim=%ld\n",rank,pnumweights,pnvdim);
    for(long i=-1;++i<len;){
      printf("MPI: proc%3d: pnames[%ld]=%s\n",rank,i,pnames[i]());
      
    }
    printf("MPI: proc%3d: GlobalSpinZaxis=%g %g %g\n",rank,
	   pglobalspinzaxis[0],
	   pglobalspinzaxis[1],
	   pglobalspinzaxis[2]
	   );
    
  }
  
  len=pmolorblist.size();
  MPI_Bcast(&len,1,MPI_LONG,0,MPI_COMM_WORLD);
  //if(prot) printf("MPI: proc%3d: len=%ld\n",rank,len);  

  if(IAmSlave){
    pmolorblist.resize(len);
  }


  for(long ic=0;ic<pmolorblist.size();ic++){
    MolecularOrbital *mo=pmolorblist[ic];

    int a=mo->pactive;
    MPI_Bcast(&a,1,MPI_INT,0,MPI_COMM_WORLD);
    if(IAmSlave) mo->pactive=a;
    a=mo->pismolecular;
    MPI_Bcast(&a,1,MPI_INT,0,MPI_COMM_WORLD);
    if(IAmSlave) mo->pismolecular=a;
    
    //if(IAmMaster) printf("MPI: proc%3d: pname=%s\n",rank,mo->pname());
    a=mo->pname.length();
    MPI_Bcast(&a,1,MPI_INT,0,MPI_COMM_WORLD);
    char *buf=new char[a+1];
    if(IAmMaster) strcpy(buf,mo->pname());
    MPI_Bcast(buf,a+1,MPI_CHAR,0,MPI_COMM_WORLD);
    if(IAmSlave) mo->pname=buf;
    delete [] buf;

    a=mo->pmolorbcontriblist.size();
    MPI_Bcast(&a,1,MPI_INT,0,MPI_COMM_WORLD);

    if(IAmSlave) mo->pmolorbcontriblist.resize(a);

    #define ibufsize 7
    #define dbufsize 14
    long *ibuf=new long[ibufsize];
    double *dbuf=new double[dbufsize];
    for(int im=0;im<mo->numOfContribs();im++){
      MolecularOrbitalData *md=(*mo)[im];
      ibuf[0]=md->psite;
      ibuf[1]=md->porbtype;
      ibuf[2]=md->pnmain;
      ibuf[3]=md->pl;
      ibuf[4]=md->pj;
      ibuf[5]=md->pm;
      ibuf[6]=md->puseglobalspinzaxis;
      memcpy(dbuf+0,md->pposition(),sizeof(double)*3);
      memcpy(dbuf+3,md->pxaxis(),sizeof(double)*3);
      memcpy(dbuf+6,md->pzaxis(),sizeof(double)*3);
      memcpy(dbuf+9,md->pspinzaxis(),sizeof(double)*3);
      dbuf[12]=md->pfactor.real();
      dbuf[13]=md->pfactor.imag();
      MPI_Bcast(ibuf,ibufsize,MPI_LONG,0,MPI_COMM_WORLD);
      MPI_Bcast(dbuf,dbufsize,MPI_DOUBLE,0,MPI_COMM_WORLD);
    
      if(IAmSlave){
	md->psite=ibuf[0];
	md->porbtype=(String::StateLabels)ibuf[1];
	md->pnmain=ibuf[2];
	md->pl=ibuf[3];
	md->pj=ibuf[4];
	md->pm=ibuf[5];
	md->puseglobalspinzaxis=ibuf[6];
	memcpy(md->pposition(),dbuf+0,sizeof(double)*3);
	memcpy(md->pxaxis(),dbuf+3,sizeof(double)*3);
	memcpy(md->pzaxis(),dbuf+6,sizeof(double)*3);
	memcpy(md->pspinzaxis(),dbuf+9,sizeof(double)*3);
	md->pfactor.real()=dbuf[12];
	md->pfactor.imag()=dbuf[13];
      }

      a=md->porbital.length();
      MPI_Bcast(&a,1,MPI_INT,0,MPI_COMM_WORLD);
      char *buf=new char[a+1];
      if(IAmMaster) strcpy(buf,md->porbital());
      MPI_Bcast(buf,a+1,MPI_CHAR,0,MPI_COMM_WORLD);
      if(IAmSlave) md->porbital=buf;
      delete [] buf;

    // ValueArray<SqMatrix<Cmplx> > pDjmm;
    // ValueArray<SqMatrix<Cmplx> > pDlms;
    // ValueArray<SqMatrix<Cmplx> > pDlm;
    }
    delete [] ibuf;
  }

  if(prot){
    for(long ic=0;ic<pmolorblist.size();ic++){
      const MolecularOrbital *mo=pmolorblist.at(ic);
      if(!mo->isActive())continue;
      if(!mo->isMolecular()){
	const MolecularOrbitalData *md=mo->at(0);
	printf("MPI: proc%3d: bwdef simple\n",rank);
	printf("MPI: proc%3d:     contrib\n",rank);
	printf("MPI: proc%3d:     site    %5ld\n",rank,md->site()+1);
	printf("MPI: proc%3d:     position%15.8f  %15.8f  %15.8f\n",rank,
	       md->position()[0],md->position()[1],md->position()[2]);
	printf("MPI: proc%3d:     xaxis   %15.8f  %15.8f  %15.8f\n",rank
	       ,md->xaxis()[0],md->xaxis()[1],md->xaxis()[2]);
	printf("MPI: proc%3d:     zaxis   %15.8f  %15.8f  %15.8f\n",rank
	       ,md->zaxis()[0],md->zaxis()[1],md->zaxis()[2]);
	printf("MPI: proc%3d:     orbital %s\n",rank,md->orbital()());
	printf("MPI: proc%3d:     factor  %s+i*%s\n",rank
	       ,String::doubleToString("%30.12f",md->factor().real(),true)()
	       ,String::doubleToString("%30.12f",md->factor().imag(),true)());
	printf("MPI: proc%3d:     useGlobalSpinZaxis %d\n",rank,
	       md->useGlobalSpinZaxis());
	printf("MPI: proc%3d:     nljmot %ld%ld%ld%ld%d\n",rank,md->nmain(),md->l(),
	       md->j(),md->m(),md->orbtype());
      }else{
	printf("MPI: proc%3d: bwdef molecular\n",rank);
	printf("MPI: proc%3d:     name %s\n",rank,mo->name()());
	for(int im=0;im<mo->numOfContribs();im++){
	  const MolecularOrbitalData *md=mo->at(im);
	  printf("MPI: proc%3d:     contrib\n",rank);
	  printf("MPI: proc%3d:     site    %5ld\n",rank,md->site()+1);
	  printf("MPI: proc%3d:     position%15.8f  %15.8f  %15.8f\n",rank,
		 md->position()[0],md->position()[1],md->position()[2]);
	  printf("MPI: proc%3d:     orbital %s\n",rank,md->orbital()());
	  printf("MPI: proc%3d:     factor  %s+i*%s\n",rank
		 ,String::doubleToString("%30.12f",md->factor().real(),true)()
		 ,String::doubleToString("%30.12f",md->factor().imag(),true)());
	  printf("MPI: proc%3d:     useGlobalSpinZaxis %d\n",rank,
		 md->useGlobalSpinZaxis());
	  printf("MPI: proc%3d:     nljmot %ld%ld%ld%ld%d\n",rank,md->nmain(),md->l(),
		 md->j(),md->m(),md->orbtype());
	}
      }
    }
  }



#endif
    return;
}




/*EOF*/ 
