/* @(#)MyDebug.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#include <stdarg.h>
#include "MyDebug.hh"
#include "Except.hh"
#include "FString.hh"
#include "ValueArray.hh"
#include "Vec3.hh"
#include "Parser.hh"
#ifdef FMIN
#include "Variables.hh"
#endif
#ifdef QXFPL
#include "Point.hh"
#include "TicMarks.hh"
#endif
#if (defined QXFSF || defined QXFPL)
#include "Parser.hh"
#include <QString>
#endif

#ifdef QXFSF
#include "Plane.hh"
#include "Polyhedron.hh"
using namespace XFPLO;
#endif




// ============================================================================
// 
// ============================================================================


MyDebug myDebug(const char *fmt,...)
{
  MyDebug dbg(stdout);
  if(dbg.FH1!=stdout){
    va_list ap;
    va_start(ap, fmt);
    vfprintf(dbg.FH1, fmt, ap);
    va_end(ap);
  }else{
    long n;
    long pinternalsize=2;
    char *s=new char[pinternalsize+1];
    while(1){
      va_list ap;
      va_start(ap,fmt);
      n = vsnprintf(s,pinternalsize,fmt,ap);
      va_end(ap);
      if (n > -1 && n < (long)pinternalsize) break;
      if (n > -1) pinternalsize = n+1;
      else pinternalsize *= 2;
      
      char *tmp=new char[pinternalsize];
      strcpy(tmp,s);
      MyMath::swap(tmp,s);
      delete [] tmp;
      
    }
    PRINT(s);
    delete [] s;
  }
  return dbg;
}


#ifdef FMIN
MyDebug operator<<(MyDebug dbg,const Value &v)
{
  dbg<<v.printS();
  return dbg;
}
#endif

MyDebug operator<<(MyDebug dbg,const Except &ex)
{
  dbg<<ex.msg();
  return dbg;
}

MyDebug operator<<(MyDebug dbg,const Vec3 &v)
{
  dbg<<v.printS();
  return dbg;
}

#if (defined QXFPL)
MyDebug operator<<(MyDebug dbg,const Point &p)
{
  dbg<<" ( "<< p.x<<" , "<<p.y<<" ) ";
  return dbg;
}
#endif
MyDebug operator<<(MyDebug dbg,const Matrix3x3 &m)
{
  dbg<<" [\n"<< m.printS("%12.5g"," ")() <<" ] ";

  return dbg;
}



MyDebug operator<<(MyDebug dbg,const String &s)
{
  dbg<<s();
  return dbg;
}


MyDebug operator<<(MyDebug dbg,const StringList &s)
{
  FPRINT(dbg.FH1,"[");
  bool nl=false;
  long nlbreak=4;
  for(long i=0;i<s.size();i++){
    FPRINTF(dbg.FH1,"\"%s\"",s[i]());
    if(i<s.size()-1){FPRINT(dbg.FH1,",");}
    nl=false;
    if(((i+1) % nlbreak)==0 && i<s.size()-1){
      FPRINT(dbg.FH1,"\n");
      nl=true;
    }
  }
  if(!nl) {FPRINT(dbg.FH1,"] ");}
  return dbg;
}

MyDebug operator<<(MyDebug dbg,const ValueArray<double> &va)
{
  FPRINT(dbg.FH1,"[");
  bool nl=false;
  long nlbreak=4;
  for(long i=0;i<va.size();i++){
    FPRINTF(dbg.FH1,"%g ",va.at(i));
    nl=false;
    //FPRINTF(dbg.FH1,"%ld %ld %ld\n",i,nlbreak,i%nlbreak);
    if(((i+1) % nlbreak)==0 && i<va.size()-1){
      FPRINT(dbg.FH1,"\n");
      nl=true;
    }
  }
  if(!nl) FPRINT(dbg.FH1,"] ");
  return dbg;
}


#if (defined QXFSF || defined QXFPL)

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg,const QString &d)
{
  dbg<<d.toAscii().constData();
  return dbg;
}


#endif

#ifdef QXFPL

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Tics &p)
{
  for(int i=0;i<p.size();i++){
    const Tic *t=p.tic(i);
    dbg << " tic("<<i<<")=\n"
	<< "    drawtics ="<<p.drawTics(Xfp::Graph_X,Xfp::TKIMajor)
	<<" "<<p.drawTics(Xfp::Graph_X,Xfp::TKIMinor)
	<<" "<<p.drawTics(Xfp::Graph_Y,Xfp::TKIMajor)
	<<" "<<p.drawTics(Xfp::Graph_Y,Xfp::TKIMinor)<<"\n"
	<< *t;
    dbg << "    changed="<<p.changed()<<"\n";
    
  }
  return dbg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Tic &t)
{
  dbg << "        label="<<t.label()<<"\n";
  dbg << "        changed="<<t.changed()<<"\n";
  return dbg;
}

#endif

#ifdef QXFSF

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Plane &p)
{
  dbg << " n=[ "<< p.n().printS() <<" ] d="<<p.d()<<"\n";
  
  return dbg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const PolyhedronProperties &p)
{
   dbg << "calculated="<<p.calculated()<<"\n";
   dbg << "visible="<<p.isVisible()<<"\n";
   dbg << "facecolor="<<Sprintf("0x%x",p.faceMaterial().color())<<"\n";
   dbg << "drawfaces="<<p.drawFaces()<<"\n";
   dbg << "drawnormals="<<p.drawNormals()<<"\n";
   dbg << "drawrim="<<p.drawRim()<<"\n";
   dbg << "rimcolor="<<Sprintf("0x%x",p.rimMaterial().color())<<"\n";
   dbg << "rimradius="<<p.rimRadius().S<<"\n";
   dbg << "rimradiusunits="<<p.rimRadiusUnits()<<"\n";
   dbg << "disprimradius="<<p.dispRimRadius()<<"\n";
    
  return dbg;
}



#endif
/*EOF*/
