/* @(#)MyDebug.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _MYDEBUG_H
#define _MYDEBUG_H 1

#ifndef MY_UNUSED
template <typename T>
inline void myUnused(T &x) { (void)x; }
#define MY_UNUSED(x) myUnused(x);
//#define Q_UNUSED(x) (void)x;
#endif

#define NSF(x)  const char *nsubfunc=x ; MY_UNUSED(nsubfunc)
#define MASSERT(x,p) if(!(p)) { printf("ASSERT: %s: %s is null\n",x,#p);exit(2);};

#include <stdio.h>
#include <string.h>
#include "F90Interface.hh"
#include "Assorted.hh"
#include "Except.hh"

using FPLOF90Interface::PRINT;

template <class T>
class ValueArray;
template <class T>
class PtrArray;
class String;
class StringList;
class Value;
class Vec3;
class Except;
class Point;
class Matrix3x3;
class QString;

#define VWRAPD(x,n) ValueArray<double>((x),(n))
#define VWRAPL(x,n) ValueArray<long>((x),(n))

// ============================================================================
// 
// ============================================================================

class DebugFun{
public:

  // --------------------------------------------------------------------------
  DebugFun()
  {
    //printf("DebugFun constructor\n");
    FILE *FH=0;
    try{
      FH=openforread("=.debug");
      PRINT("found =.debug .. debugging\n");
    }catch(Except &ex){
      return;
    }      
    try{
      String line;
      while (getLine(FH,line)){
	line=line.stripBlanks(true);
	pfuncs<<line;
      }
      fclose(FH);
    }catch(Except &ex){
      if(FH) fclose(FH);
      PRINTF("%s\n",ex.msg());
      exit(2);
    }
    //myDebug()<<pfuncs;
    //exit(0);
  }
  // --------------------------------------------------------------------------
  ~DebugFun(){}
  // --------------------------------------------------------------------------
  bool operator()(const String &s)const
  {
    if(pfuncs.contains(s)){
      PRINTF("debugfun: start debugging \"%s\"\n",s());
      return true;
    }
    return false;
  }
  
protected:
private:
  ValueArray<String> pfuncs;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class MyDebug{
 public:
  MyDebug(FILE *f):FH1(f),plast(true){};
  ~MyDebug(){
    if(plast) {FPRINT(FH1,"\n");}
  };
  MyDebug(const MyDebug &other){
    FH1=other.FH1;
    plast=false;
  }
  MyDebug &operator<<(int i){ FPRINTF(FH1,"%d ",i);return *this;};
  MyDebug &operator<<(unsigned int i){ FPRINTF(FH1,"0x%x ",i);return *this;};
  MyDebug &operator<<(long i){ FPRINTF(FH1,"%ld ",i);return *this;};
  MyDebug &operator<<(unsigned long i){ FPRINTF(FH1,"0x%lx ",i);return *this;};
  MyDebug &operator<<(double d){ FPRINTF(FH1,"%g ",d);return *this;};
  MyDebug &operator<<(void *p){ FPRINTF(FH1,"%p ",p);return *this;};
  MyDebug &operator<<(const void *p){ FPRINTF(FH1,"%p ",p);return *this;};
  MyDebug &operator<<(char c){ FPRINTF(FH1,"%c ",c);return *this;};
  MyDebug &operator<<(const char *c){FPRINTF(FH1,"%s",c);
    if(c[strlen(c)-1]=='\n')
      return *this;
    else
      return space();
  };
  MyDebug &nospace(){return *this;};
  MyDebug &space(){FPRINT(FH1," ");return *this;};
 public:
  FILE *FH1;
  bool plast;
};



inline MyDebug myDebug(){return MyDebug(stdout);}
inline MyDebug myDebug(FILE *FH){return MyDebug(FH);}
MyDebug myDebug(const char *fmt,...);


MyDebug operator<<(MyDebug dbg,const Value &v);
MyDebug operator<<(MyDebug dbg,const String &s);
MyDebug operator<<(MyDebug dbg,const StringList &s);
MyDebug operator<<(MyDebug dbg,const ValueArray<double> &va);
MyDebug operator<<(MyDebug dbg,const Vec3 &v);
MyDebug operator<<(MyDebug dbg,const Except &ex);
MyDebug operator<<(MyDebug dbg,const Point &p);
MyDebug operator<<(MyDebug dbg,const Matrix3x3 &m);
MyDebug operator<<(MyDebug dbg,const QString &d);


template <class T>
MyDebug operator<<(MyDebug dbg, const ValueArray<T> &v){
  FPRINT(dbg.FH1,"[");
  bool nl=false;
  long nlbreak=8;
  long N=v.size();
  for(long i=0;i<N;i++){
    dbg<<v.at(i);
    if(i<N-1){FPRINT(dbg.FH1,",");}
    nl=false;
    if(((i+1) % nlbreak)==0 && i<N-1){
      FPRINT(dbg.FH1,"\n");
      nl=true;
    }
  }
  if(!nl) {FPRINT(dbg.FH1,"] ");}
  return dbg;
}
template <class T>
MyDebug operator<<(MyDebug dbg, const PtrArray<T> &v){
  dbg<<" [";
  bool nl=false;
  long nlbreak=4;
  for(long i=0;i<v.size();i++){
    dbg<<*(v.at(i));
    if(i<v.size()-1)dbg<<",";
    nl=false;
    if(((i+1) % nlbreak)==0 && i<v.size()-1){
      dbg<<"\n";
      nl=true;
    }
  }
  if(!nl) dbg<<"]";
  return dbg;
}
template <class T1,class T2>
class Pair;
template <class T1,class T2>
inline MyDebug operator<<(MyDebug dbg,const Pair<T1,T2> &v){
  dbg<<"Pair("<<v.v1<<","<<v.v2<<")\n";
  return dbg;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

#if (defined QXFSF || defined QXFPL)

class Size;
class Tics;
class Tic;
class Plane;
namespace XFPLO{
  class PolyhedronProperties;
}

MyDebug operator<<(MyDebug dbg, const Plane &v);
MyDebug operator<<(MyDebug dbg, const XFPLO::PolyhedronProperties &p);
MyDebug operator<<(MyDebug dbg, const Tics &p);
MyDebug operator<<(MyDebug dbg, const Tic &p);

#endif


extern DebugFun debugfun;


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _MYDEBUG_H */

