/* @(#)MyMath.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _MYMATH_H
#define _MYMATH_H 1


#include <stdlib.h>
#include <limits>
#include <math.h>
#include "ValueArray.hh"


#ifndef COMPARE
#define COMPARE(a,b) ((a)>(b) ? 1 : (((a)==(b))? 0 : -1))
#endif


const double sq2o2=.7071067811865475244008443621048490392848;
const double Pi = 3.1415926535897932384626433832795028841968;
const double sqPi = 1.7724538509055160272981674833411451827974;
const double twopi  = 6.2831853071795864769252867665590057683936;
const double fourpi =12.5663706143591729538505735331180115367872;
const double sq3o2 =0.8660254037844386467637231707529361834714;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double *dinit(double *a,double a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long *linit(long *a,long a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline int *iinit(int *a,int a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double *ainit(double *a,double a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double dSign(double x,double y)
{
  return ( fabs(x)* ((y)>=0. ? 1.:-1.) );
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double dRound(double x)
{
  return ((x)>0. ?( floor(x+.5) ) :(ceil(x-.5)));
}

inline long lRound(double x)
{
  return ((x)>0. ?((long)(x+.5)) :((long)(x-.5)));
}

inline unsigned long ulRound(double x)
{
  return ((x)>0. ?((unsigned long)(x+.5)) :((unsigned long)(x-.5)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long lSign(long x,long y)
{
  return ( labs(x)* ((y)>=0 ? 1:-1) );
}

namespace MyMath{


  void solve_quad_real(double a2,double a1,double a0,double x[2],long &nroot);
  void solve_cubic_real(double a3,double a2,double a1,double a0,
			double x[3],long &nroot);

  

  // ==========================================================================
  template<class T>
  T horner(T x,const T *coeffs,long N,T *fp=0)
  {
    // N is the order of the polynomial
    // at point x with coeffs[0:N] calc polynomial
    // first deriv in fp
    
    T d=coeffs[N];
    if(fp) (*fp)=d;
    
    d=coeffs[N-1]+x*d;
    for(int i=2;i<=N;i++){
      if(fp) (*fp)=d+x*(*fp);
      d=coeffs[N-i]+x*d;
    }
    return d;
  }

  // ==========================================================================
  template<class T>
  void getNextAfter(T &x,T y)
  {
    // return a value y \approx x, such that x!=y is grantied
    
    if(x==0){
      x=std::numeric_limits<T>::epsilon();
      return;
    }

    int ulp=1;
    while(x==y){
      x=y*(T(1.0)+std::numeric_limits<T>::epsilon()*(ulp++));
    }
    
  }
  
  template<class T>
  void cross(const T *a,const T *b,T *res)
  {
    res[0]=a[1]*b[2]-a[2]*b[1];
    res[1]=a[2]*b[0]-a[0]*b[2];
    res[2]=a[0]*b[1]-a[1]*b[0];
  }

  template<class T>
  inline T min(T x,const T &y){
    return (x<y?x:y);
  }

  template<class T>
  inline T min3(T x,const T &y,const T &z)
  {
    x=( x<y ? x : y  );
    return (x<z?x:z);
  }

  template<class T>
  inline T max(T x,const T &y){
    return (x>y?x:y);
  }
  template<class T>
  inline T max3(T x,const T &y,const T &z)
  {
    x=( x>y ? x : y  );
    return (x>z?x:z);
  }

  template<class T>
  T minVal(const ValueArray<T> &v){
    if(v.size()<1) return T(0);
    
    T m=v.at(0);;
    for(long i=0;++i<v.size();){
      m=MyMath::min<T>(m,v.at(i));
    }
    return m;
  }
  template<class T>
  T maxVal(const ValueArray<T> &v){
    if(v.size()<1) return T(0);
    
    T m=v.at(0);;
    for(long i=0;++i<v.size();){
      m=MyMath::max<T>(m,v.at(i));
    }
    return m;
  }

  template<class T,class R>
  T minVal(const T *v,R N){
    
    T m=v[0];
    for(R i=0;++i<N;){
      m=MyMath::min<T>(m,v[i]);
    }
    return m;
  }
  

  template<class T,class R>
  T maxVal(const T *v,R N){
    
    T m=v[0];
    for(R i=0;++i<N;){
      m=MyMath::max<T>(m,v[i]);
    }
    return m;
  }
  
  
  template <class T>
  T ror(T x,int N,unsigned int moves)
  {
    // rotate the first N bits right by moves bits
    // example:  ror(00001100,4,1)=00000110
    // example:  ror(00001100,4,2)=00000011
    // example:  ror(00001100,4,3)=00001001
    // example:  ror(00001100,4,4)=00001100
    
    T mask=(1<<N)-1;
    
    return (x>>moves) | ((x<<(N-moves))&mask);
    
  }
  
  // template <class T>
  // int countBits(T x,int N)
  // {
  //   int count=0;
  //   for(int c=-1;++c<N;){
  //     count+=(x&1);
  //     x>>=1;
  //   }
  //   return count;
  // }

  template <class T>
  int bitCount(T x)
  {
    T c;
    for (c = 0; x; c++){
      x &= x - 1; 
    }
    return c;
  }
  
  template <class T>
  inline void swap(T &a,T &b){
    T c=a;
    a=b;
    b=c;
  }
  
  template <class T>
  inline void swap3(T &a,T &b,T &c){
    T t=a;
    a=b;
    b=c;
    c=t;
  }

  template <class T>
  inline void swap4(T &a,T &b,T &c,T &d){
    T t=a;
    a=b;
    b=c;
    c=d;
    d=t;
  }
  
  template <class T>
  T Abs(const T &a);

  template <>
  inline int Abs(const int &a){
    return abs(a);
  }
  template <>
  inline long Abs(const long &a){
    return labs(a);
  }
  template <>
  inline float Abs(const float &a){
    return fabsf(a);
  }
  template <>
  inline double Abs(const double &a){
    return fabs(a);
  }
  template <class T>
  T Abs2(const T &a);

  template <>
  inline int Abs2(const int &a){
    return abs(a)*abs(a);
  }
  template <>
  inline long Abs2(const long &a){
    return labs(a)*labs(a);
  }
  template <>
  inline float Abs2(const float &a){
    return fabsf(a)*fabsf(a);
  }
  template <>
  inline double Abs2(const double &a){
    return fabs(a)*fabs(a);
  }

  template <class T>
  inline T mod(T x,T y){
    if(x<0) return (y-abs(x)%y)%y;
    else  return x%y;
  }
  template <class T>
  inline T moddiv(T x,T y,T &d){
    T r;
    if(x<0) r=(y-abs(x)%y)%y;
    else r=x%y;
    d=(x-r)/y;
    return r;
  }
  // we need this for generic algos
  template <class T>
  inline double toDouble(const T &a){
    return a;
  }

  long gcd(long i,long j);
  long gcd(long i,long j,long k);
  long gcd(long i,long j,long k,long l);
  void runningSum(const ValueArray<double> *x,
		  ValueArray<double> *y);


  void splitbaseN(double d,double base,double &mm,long &ee);

  template <class T>
  class Factorial{
  public:
    Factorial(){}
    
    T operator()(long i)
    {
      long N=pfac.size();
      if(i<N) return pfac.at(i);
      pfac.resize(i+1);
      if(!N) pfac[N++]=T(1);
      for(long k=N;k<=i;k++) pfac[k]=pfac[k-1]*k;
      return pfac.at(i);
    }    

  private:
    ValueArray<T> pfac;
    
  };

  typedef Factorial<double> dFactorial;


}

#ifdef	__cplusplus
extern "C" {
#endif

extern long lineCross2(double *c1,double *c2,double *c3,double *c4,double tol);


#ifdef	__cplusplus
}
#endif


#endif /* _MYMATH_H */

