/* @(#)Parser.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PARSERXX_H
#define _PARSERXX_H 1



#include <stdio.h>
#include "FString.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 * Parser wraps up the vartable fiasco.
 * An instance of parser can be used to query and write
 * the content, read from a file, in a much more convinient way.
 * ***********************************************************************
 */


class Parser;


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Dobj{
  //
public:
  // empty and copy constructor
  Dobj()
    :classname("Dobj"),node(0),parentname(""),name(""),structuretype(0)
    ,pindex(0),pparser(0){};
  Dobj(Dobj const &other)
    :classname("Dobj"),node(other.node),parentname(other.parentname)
    ,name(other.name),structuretype(other.structuretype)
    ,pindex(other.pindex),pparser(other.pparser){};
  // destructor
  ~Dobj(){};
  //
  //
  // Parser can access private constructors
  friend class Parser;
  //
  //
  Parser const *parser()const{return pparser;};
  //
  // return the value of a variable, if it is actually a scalar or 
  // array element
  // this is risky
  //
  String operator()() const;
  StringList list() const;

  long   valueL() const;
  double valueD() const;
  void getArray(long size,double *data) const;
  void getArray(long size,long size2,double *data) const;
  void getArray(long size,long   *data) const;
  void getArray(long size,String *data) const;
  void getStringArray(long sz,String *data) const;

  //
  // return (struct)array size
  long size() const ;
  void getSizes(long s[8]) const ;
  long size(long) const ;
  //
  long index()const {return pindex;};
  //
  long validNode()const{return (node!=0);};
  String nodeName()const{return name;};
  String fullName()const;
  String parentName()const{return parentname;};
  //
  long isScalar()const;
  long isArray()const;
  long isStructArray()const;
  long isStruct()const;
  //
  long isInt()const;
  long isReal()const;
  long isLogical()const;
  long isFlag()const;
  long isChar()const;
  long isString()const;
  //
  //
  long rank()const;
  long isDim() const;
  long hasIDRef() const;
  long hasIDRef(long i) const;
  String IDRef() const;
  String IDRef(long) const;
  long isVariableSize()const;
  long scalarType()const;
  long stringLength()const;
  long isVariableString()const;
  //
  //
  void setDefaultValue(String const &);
  //
  // write into element;
  //
  Dobj &operator=(String const &s);
  //
  // get an (struct)array memeber
  Dobj operator[](long index) const;
  // access a struct memeber
  Dobj operator()(String const &name) const;
  //
  Dobj first();
  Dobj next();
  //
  Dobj sap(String const &name) const;
  Dobj arp(String const &name) const;
  Dobj scp(String const &name) const;
  Dobj stp(String const &name) const;
  Dobj flagArp(String const &name) const;
  //
  long type()const{return structuretype;}; //todo remove
  String structureType()const;
  //
  Dobj sapTemplate() const;
  void createTemplateFromDefValue(String const &) ;
  void resizeSap(long newsize); 
  void resizeArp(long sizes[8]); 
  //

private:
  // these constructors should not be used from the outside world
  //
  Dobj(struct vartable *n,long st,String const &pn,String const &na
       ,Parser const *par)
    :classname("Dobj"),node(n),parentname(pn),name(na),structuretype(st)
    ,pindex(0),pparser(par){};
  //
  Dobj(struct vartable *n,long i,long st,String const &pn,String const &na
       ,Parser const *par) 
    :classname("Dobj"),node(n),parentname(pn),name(na),structuretype(st)
    ,pindex(i),pparser(par){}; 
  //
  //
private:
  String classname;
  struct vartable *node;
  String parentname;
  String name;
  long structuretype;
  long pindex;
  Parser const *pparser;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Parser{
public:
  Parser(long aschar);
  Parser();
  ~Parser();
  //
  //
  void parseFile(String const &filename);
  //
  //
  //
  void writeFile(FILE *fo);
  void writeFile(const String &filename);
  //
  void cleanup();
  //
  String fileName()const {return pfilename;};
  //
  //
  // return the root of the parse table
  Dobj operator()() const;
  //
  static void finalCleanUp();
  

private:
  String classname;
  struct parser_controll *pc; 
  long padding;
  long aschar;
  String pfilename;
  
};


MyDebug operator<<(MyDebug dbg,const Dobj &d);

/* ***********************************************************************
 *
 * ***********************************************************************
 */


#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _PARSERXX_H */

