#ifndef VEC3_H
#define VEC3_H

#if (defined QXFSF)
#include <QString>
#endif
#include "FString.hh"
#include "MyMath.hh"


class Vec3;
class IMatrix;
/* ***********************************************************************
 * 
 * ***********************************************************************
 */

class Matrix3x3
{
public:

  Matrix3x3(){ p[0]=p[1]=p[2]=0; p[3]=p[4]=p[5]=0; p[6]=p[7]=p[8]=0; };
  Matrix3x3(const Matrix3x3 &o){ memcpy(p,o.p,sizeof(double)*9); };
  inline const Matrix3x3 &operator=(const Matrix3x3 &o){
    memcpy(p,o.p,sizeof(double)*9);return*this;};
  
  Matrix3x3(const double m[9]){ memcpy(p,m,sizeof(double)*9); };
  Matrix3x3(const long m[9]){ for(int i=0;i<9;i++) p[i]=m[i]; };

  // vi are columns
  Matrix3x3(const Vec3 v0,const Vec3 v1,const Vec3 v2);

  void print(const char *name="mat",const char *delim=" ") const;
#if (defined QXFSF)
  QString printQ(const char *name="mat",const char *delim=" ") const;
#endif
  String printS(const String &fmt="%g",const String &delim=" ") const;

  //private:

  
  Matrix3x3(double m00,double m10,double m20,
	    double m01,double m11,double m21,
	    double m02,double m12,double m22){
    p[0]=m00; p[1]=m10; p[2]=m20;
    p[3]=m01; p[4]=m11; p[5]=m21;
    p[6]=m02; p[7]=m12; p[8]=m22; };

public:


  inline Matrix3x3 &operator+=(const Matrix3x3 &o){
    p[0]+=o.p[0]; p[1]+=o.p[1]; p[2]+=o.p[2];
    p[3]+=o.p[3]; p[4]+=o.p[4]; p[5]+=o.p[5];
    p[6]+=o.p[6]; p[7]+=o.p[7]; p[8]+=o.p[8];
    return *this;
  }
  inline Matrix3x3 &operator-=(const Matrix3x3 &o){
    p[0]-=o.p[0]; p[1]-=o.p[1]; p[2]-=o.p[2];
    p[3]-=o.p[3]; p[4]-=o.p[4]; p[5]-=o.p[5];
    p[6]-=o.p[6]; p[7]-=o.p[7]; p[8]-=o.p[8];
    return *this;
  }

  inline Matrix3x3 &operator*=(double s){
    p[0]*=s; p[1]*=s; p[2]*=s; p[3]*=s; p[4]*=s; p[5]*=s; p[6]*=s; 
    p[7]*=s; p[8]*=s; return *this;};

  inline Matrix3x3 operator-()const{
    Matrix3x3 res;
    res.p[0]=-p[0]; res.p[1]=-p[1]; res.p[2]=-p[2]; 
    res.p[3]=-p[3]; res.p[4]=-p[4]; res.p[5]=-p[5]; 
    res.p[6]=-p[6]; res.p[7]=-p[7]; res.p[8]=-p[8]; 
    return res;};

  inline Matrix3x3 &operator/=(double s){
    p[0]/=s; p[1]/=s; p[2]/=s; p[3]/=s; p[4]/=s; p[5]/=s; p[6]/=s; 
    p[7]/=s; p[8]/=s; return *this;};

  double determinant() const;
  double trace() const;
  // form the inverse transpose matrix and return new object
  Matrix3x3 inverseT(bool &indef,double &det) const;
  Matrix3x3 inverseT(bool &indef) const{
    double dum;return inverseT(indef,dum);};
  Matrix3x3 inverseT() const {bool indef; double dum;
    return inverseT(indef,dum);};

  Matrix3x3 inverse(bool &indef) const{
    double dum; return inverseT(indef,dum).transpose();};

  Matrix3x3 inverse() const{
    bool indef;
    double dum; return inverseT(indef,dum).transpose();};
  
  // these change the matrix and return a reference
/*   const Matrix3x3 &invertT(bool &indef) ; */
/*   const Matrix3x3 &invertT() {bool indef; return invertT(indef);}; */
  const Matrix3x3 &transpose();
  Matrix3x3 transposed()const{return Matrix3x3(*this).transpose();};

  inline Matrix3x3 operator*(const Matrix3x3 &o) const{
    return Matrix3x3(

		     (p[0]*o.p[0] +p[3]*o.p[1] +p[6]*o.p[2]),
		     (p[1]*o.p[0] +p[4]*o.p[1] +p[7]*o.p[2]),
		     (p[2]*o.p[0] +p[5]*o.p[1] +p[8]*o.p[2]),

		     (p[0]*o.p[3] +p[3]*o.p[4] +p[6]*o.p[5]),
		     (p[1]*o.p[3] +p[4]*o.p[4] +p[7]*o.p[5]),
		     (p[2]*o.p[3] +p[5]*o.p[4] +p[8]*o.p[5]),

		     (p[0]*o.p[6] +p[3]*o.p[7] +p[6]*o.p[8]),
		     (p[1]*o.p[6] +p[4]*o.p[7] +p[7]*o.p[8]),
		     (p[2]*o.p[6] +p[5]*o.p[7] +p[8]*o.p[8])); };


  inline const double*operator()() const { return p; };

  StringList toString(long mmax=10)const; 
  inline double *data() { return p; };


  // mat*v
  inline Vec3 operator*(const Vec3 &v) const;
  // mat*v, no class overhead
  inline Vec3 dot3(double a,double b,double c) const;
  // |mat*v|^2, no class overhead
  inline double dot3normsq(double a,double b,double c) const;

  
  inline const Vec3 operator()(int i) const ;

  inline const Matrix3x3 &scaleColumn(int i,double s){
    p[3*i]*=s; p[1+3*i]*=s; p[2+3*i]*=s; return *this;};


  inline Matrix3x3 operator*(double s)const{
    return Matrix3x3(p[0]*s,p[1]*s,p[2]*s,p[3]*s,p[4]*s,p[5]*s
		     ,p[6]*s,p[7]*s,p[8]*s);};

  inline Matrix3x3 operator/(double s)const{
    return Matrix3x3(p[0]/s,p[1]/s,p[2]/s,p[3]/s,p[4]/s,p[5]/s
		     ,p[6]/s,p[7]/s,p[8]/s);};

  inline const Matrix3x3 &setColumn(int i,const Vec3 &v);

  inline const Matrix3x3 &swapColumns(int i,int j){ 
    double d=p[0+3*i];p[0+3*i]=p[0+3*j];p[0+3*j]=d;
    d=p[1+3*i];p[1+3*i]=p[1+3*j];p[1+3*j]=d;
    d=p[2+3*i];p[2+3*i]=p[2+3*j];p[2+3*j]=d;
    return *this; };

  inline double operator()(long i,long j) const {return p[i+3*j];};
  inline double &operator()(long i,long j){return p[i+3*j];};

  inline bool isZero(double tol=1.0e-6)const{
    for(int i=0;i<9;i++){
      if(fabs(p[i])>tol){
	return false;
      }
    }
    return true;
  };

  static inline Matrix3x3 unitMatrix(){ 
    return Matrix3x3(1,0,0,0,1,0,0,0,1); };

  static inline Matrix3x3 diagonal(double a,double b,double c) {
    return Matrix3x3(a,0,0,0,b,0,0,0,c); };
  

private:
  double p[9]; // column wise: M(i,j)=p[i+3*j]

  friend class Vec3;
  friend double trace(const Matrix3x3 &m);
  friend Matrix3x3 operator*(const Matrix3x3 &a,const IMatrix &b);
  friend IMatrix round(const Matrix3x3&other);
  friend Matrix3x3 operator-(const Matrix3x3 &a,const IMatrix &b);
  friend Matrix3x3 operator-(const IMatrix &a,const Matrix3x3 &b);
  // Frobenius norm
  friend double Fnorm(const Matrix3x3 &m);


};




/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Vec3
{
public:
  Vec3(){p[0]=p[1]=p[2]=0;};
  Vec3(const double o[3]){p[0]=o[0];p[1]=o[1];p[2]=o[2];};
  Vec3(const Vec3 &o){p[0]=o.p[0];p[1]=o.p[1];p[2]=o.p[2];};
  Vec3(double o0,double o1,double o2){p[0]=o0;p[1]=o1;p[2]=o2;};
  
  Vec3(const String s[3]);
  Vec3(const StringList sl);
  Vec3(const String &x,const String &y,const String &z);

  inline const Vec3 &operator=(const Vec3 &o){
    p[0]=o.p[0];p[1]=o.p[1];p[2]=o.p[2]; return*this;};

  inline const Vec3 &operator=(double s){p[0]=p[1]=p[2]=s; return*this;};

  inline const Vec3 &operator+=(const Vec3 &o){
    p[0]+=o.p[0];p[1]+=o.p[1];p[2]+=o.p[2]; return *this; };
  inline const Vec3 &operator+=(const double *o){
    p[0]+=o[0];p[1]+=o[1];p[2]+=o[2]; return *this; };

  inline const Vec3 &operator*=(double s){
    p[0]*=s;p[1]*=s;p[2]*=s; return *this; };

  inline const Vec3 &operator/=(double s){
    p[0]/=s;p[1]/=s;p[2]/=s; return *this; };

  inline Vec3 operator-() const{ return Vec3(-p[0],-p[1],-p[2]); };

  inline Vec3 &operator-=(const Vec3 &o){
    p[0]-=o.p[0]; p[1]-=o.p[1]; p[2]-=o.p[2]; return *this; };

  inline Vec3 operator+(const Vec3 &o) const{
    return Vec3(p[0]+o.p[0], p[1]+o.p[1] ,p[2]+o.p[2] ); };

  inline Vec3 operator*(double s) const { 
    return Vec3(p[0]*s, p[1]*s ,p[2]*s ); };

  inline Vec3 operator/(double s) const { 
    return Vec3(p[0]/s, p[1]/s ,p[2]/s ); };

  // scalar product
  inline double operator*(const Vec3 &a) const{
    return p[0]*a.p[0]+p[1]*a.p[1]+p[2]*a.p[2]; };

  // vector product 
  inline Vec3 cross(const Vec3 &o) const{
    return Vec3(p[1]*o.p[2]-p[2]*o.p[1],
		p[2]*o.p[0]-p[0]*o.p[2],
		p[0]*o.p[1]-p[1]*o.p[0]);
  }

  //outer dyadic product: m=this (outer) o
  inline Matrix3x3 outer(const Vec3 &o) const{
    Matrix3x3 m;
    m.p[0]=p[0]*o.p[0]; m.p[1]=p[1]*o.p[0]; m.p[2]=p[2]*o.p[0];
    m.p[3]=p[0]*o.p[1]; m.p[4]=p[1]*o.p[1]; m.p[5]=p[2]*o.p[1];
    m.p[6]=p[0]*o.p[2]; m.p[7]=p[1]*o.p[2]; m.p[8]=p[2]*o.p[2];
    return m;
  }

  // res=v*M
  inline Vec3 operator*(const Matrix3x3 &m) const{
    return Vec3(
		p[0]*m.p[0] + p[1]*m.p[1] + p[2]*m.p[2],
		p[0]*m.p[3] + p[1]*m.p[4] + p[2]*m.p[5],
		p[0]*m.p[6] + p[1]*m.p[7] + p[2]*m.p[8] );
  };

  const Vec3 &swapColumns(int i,int j){ 
    double d=p[i];p[i]=p[j];p[j]=d; return *this; };

  inline const Vec3 &normalize(){ 
    double d=sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]) ;
    p[0]/=d; p[1]/=d; p[2]/=d; return *this;};

  inline Vec3 normalized()const{
    Vec3 t(*this);
    t.normalize();
    return t;
  };

  //  inline double norm() const{ return sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]); };

  bool eq(const Vec3 &other,double tol=1.0e-10) const;
  bool neq(const Vec3 &other,double tol=1.0e-10) const;
  bool lt(const Vec3 &other,double tol=1.0e-10) const;
  
  inline double &operator[](long i){return p[i];};

  StringList toString(long mmax=10)const; 

  inline const double &operator[](long i) const {return p[i];};
  
  // return raw data ... odd but simple Vec3 a; a() is pointer to double 
  inline double *operator()(){return p;};

  inline const double *operator()() const{return p;};


  long maxAbsAt()const{
    long r=0;double d=fabs(p[0]);
    if(fabs(p[1])>d) {r=1;d=fabs(p[1]);}
    if(fabs(p[2])>d) {r=2;}
    return r;
  }
  long smallestNonzeroAbsAt(double tol)const{
    int im=maxAbsAt();
    double d=fabs(p[im]);
    if(fabs(p[1])>tol && fabs(p[1])<d){ im=1;d=fabs(p[1]); }
    if(fabs(p[2])>tol && fabs(p[2])<d){ im=2;}
    return im;
  }

  inline Vec3 &backfold(){
    p[0]-=dRound(p[0]);
    p[1]-=dRound(p[1]);
    p[2]-=dRound(p[2]);
    return *this;
  };

  inline Vec3 &firstOctant(){
    p[0]-=floor(p[0]);
    p[1]-=floor(p[1]);
    p[2]-=floor(p[2]);
    if((long)p[0]<0) p[0]++;
    if((long)p[1]<0) p[1]++;
    if((long)p[2]<0) p[2]++;
    if((long)p[0]==1) p[0]--;
    if((long)p[1]==1) p[1]--;
    if((long)p[2]==1) p[2]--;
    return *this;
  };

  inline Vec3 backfolded()const{
    Vec3 v=*this;
    return v.backfold();
  };
  
  void findPerpDirections(Vec3 &x,Vec3 &y)const;
  
  static Vec3 eulerAngles(const Vec3 &z,const Vec3 &x);
  static Vec3 eulerAngles(const Vec3 &z);
  Matrix3x3 eulerRMatrix();

  void print(const char *name="vec",const char *delim=" ") const;
#if (defined QXFSF)
  QString printQ(const char *name="vec",const char *delim=" ") const;
#endif
  String printS(const String &fmt="%g",const String &delim=" "
		,bool contract=false) const;

private:
  double p[3];
  friend class Matrix3x3;
  friend double norm(const Vec3 &);
  friend double normsq(const Vec3 &);
  friend Vec3 operator-(const Vec3 &lhs,const Vec3 &rhs);
 
  
};


inline Vec3 operator-(const Vec3 &lhs, const Vec3 & rhs ) {
  return Vec3(lhs.p[0]-rhs.p[0],lhs.p[1]-rhs.p[1],lhs.p[2]-rhs.p[2]);
}

// inline Vec3 operator-( Vec3 lhs, const Vec3 & rhs ) {
//   lhs -= rhs;
//   return lhs;
// }

inline const Vec3 operator*(double s,const Vec3 &a){ return a*s; }

inline double norm(const Vec3 &v){ 
  return sqrt(v.p[0]*v.p[0]+v.p[1]*v.p[1]+v.p[2]*v.p[2]);}
inline double normsq(const Vec3 &v){ 
  return v.p[0]*v.p[0]+v.p[1]*v.p[1]+v.p[2]*v.p[2];}



inline const Vec3 Matrix3x3::operator()(int i) const { return Vec3(p+3*i); }


inline Vec3 Matrix3x3::operator*(const Vec3 &v) const { 
  return Vec3(
	      p[0]*v.p[0]+p[3]*v.p[1]+p[6]*v.p[2],
	      p[1]*v.p[0]+p[4]*v.p[1]+p[7]*v.p[2],
	      p[2]*v.p[0]+p[5]*v.p[1]+p[8]*v.p[2] ); }

inline Vec3 Matrix3x3::dot3(double a,double b,double c) const
{
  return Vec3(p[0]*a+p[3]*b+p[6]*c,p[1]*a+p[4]*b+p[7]*c,p[2]*a+p[5]*b+p[8]*c ); 
}

inline double Matrix3x3::dot3normsq(double a,double b,double c) const
{
  double d=p[0]*a+p[3]*b+p[6]*c;
  double res=d*d;
  d=p[1]*a+p[4]*b+p[7]*c;
  res+=d*d;
  d=p[2]*a+p[5]*b+p[8]*c;
  return res+d*d;
}

// ============================================================================
inline Matrix3x3 operator-(Matrix3x3 lhs,const Matrix3x3 &rhs){
  lhs-=rhs;
  return lhs;
}
// ============================================================================
inline Matrix3x3 operator+(Matrix3x3 lhs,const Matrix3x3 &rhs){
  lhs+=rhs;
  return lhs;
}


inline const Matrix3x3 &Matrix3x3::setColumn(int i,const Vec3 &v){
  p[0+3*i]=v[0]; p[1+3*i]=v[1]; p[2+3*i]=v[2]; return *this; }

extern bool lineCross3(const Vec3 &c1,const Vec3 &c2
		       ,const Vec3 &d1,const Vec3 &d2
		       ,double tol);

inline double trace(const Matrix3x3 &m){
    return m.p[0]+m.p[4]+m.p[8];
}

inline double Matrix3x3::determinant() const { 
  return (Vec3(p+3).cross(p+6))*Vec3(p); }

inline double Matrix3x3::trace() const { 
  return p[0]+p[4]+p[8]; }

inline double Fnorm(const Matrix3x3 &m)
{
  double d=0;
  for(int i=0;i<9;i++){
    d+=m.p[i]*m.p[i];
  }
  return sqrt(d);
}

inline long mhnorm(const Matrix3x3 &m){
  return 
    fabs(m(0,0))+fabs(m(0,1))+fabs(m(0,2))+
    fabs(m(1,0))+fabs(m(1,1))+fabs(m(1,2))+
    fabs(m(2,0))+fabs(m(2,1))+fabs(m(2,2))
    ;
}

template <>
bool ValueArray<Vec3>::contains(const Vec3&v,double tol)const;

#endif // VEC3_H
