/*************************************************************************
 * @(#)Wigner3J.cpp
 * author: magru@unas
 * date:   03 Mar 2014
 *************************************************************************
 */

#include <stdio.h>
#include "Wigner3J.hh"
#include "MyMath.hh"
#include "Except.hh"
#include "MyDebug.hh"


class APInteger{
public:
  // ----------------------------------------------------------------
  APInteger():psign(1),psize(0),pinternalsize(0),pp(NULL)
  {
    grow(sizeof(long));psize=0;
  }

  // ----------------------------------------------------------------
  APInteger(const APInteger &other):psize(0),pinternalsize(0),pp(NULL)
  {
    *this=other;
  }

  // ----------------------------------------------------------------
  APInteger(long x):psize(0),pinternalsize(0),pp(NULL)
  {
    *this=x;
  }

  // ----------------------------------------------------------------
  APInteger &operator=(const APInteger &other)
  {
    if(pp!=NULL) free(pp);
    
    pp=(unsigned char*)malloc(other.pinternalsize*sizeof(unsigned char));
    memcpy(pp,other.pp,other.pinternalsize*sizeof(unsigned char));
    pinternalsize=other.pinternalsize;
    psize=other.psize;
    psign=other.psign;
    return *this;
  }


  // ----------------------------------------------------------------
  ~APInteger()
  {
    if(pp!=NULL) free(pp);
  }

  // ----------------------------------------------------------------
  APInteger &operator=(long x)
  {
#undef debug 
#ifdef debug
    myDebug()<<"APInteger::operator=(long) x="<<x<<(unsigned long)x;
#endif
    psize=1;
    psign=1;
    if(!x) { psize=0;return *this;}
    unsigned char u=0;
    grow(sizeof(long));
    long i=0;
    if(x<0){
      psign=-1;
      u=1;
      x=labs(x);
#ifdef debug
      myDebug()<<"x="<<x<<(unsigned long)x;
#endif
      while(x!=0){
	psize=i+1;
	unsigned short s=((~(x&0xff))&0xff)+u;
	pp[i++]=(s&0xff);
	u=((s/0x100)&0x1);
	x/=0x100;
#ifdef debug
	myDebug()<<i<<dump(true,"tmp")<<"x="<<(unsigned long)x;
#endif
      }
    }else{
      while(x!=0){
	psize=i+1;
	pp[i++]=(x&0xff);
	x/=0x100;
      }
    }
    normalize();
    
    
    return *this;
  }

  // ----------------------------------------------------------------
  bool isZero()const
  {
    return (psize==0 && psign>0);
  }

  // ----------------------------------------------------------------
  bool isOne()const
  {
    return (psize==1 && psign>0 && pp[0]==1);
  }

  // ----------------------------------------------------------------
  bool isMinusOne()const
  {
    return (psize==0 && psign<0);
  }


  // ----------------------------------------------------------------
  long toLong()const
  {
    //myDebug()<<"APInteger::toLong: "<<dump(true,"this");
    long res=0;
    if(psign<0){
      unsigned long n=1;
      for(unsigned long i=0;i<psize;i++){
	res+=((~(pp[i]&0xff))&0xff)*n;
	//myDebug()<<"i="<<i<<"res="<<(unsigned long)res;
	n<<=8;
      }
      res=-(res+1);
    }else{
      for(long i=psize;--i>=0;){
	res<<=8;
	//myDebug()<<res;
	res+=pp[i];
	//myDebug()<<"i="<<i<<"res="<<res;
      }
    }
    return res;
  }

  // ----------------------------------------------------------------
  double toDouble()const
  {
    //myDebug()<<"APInteger::toDouble: "<<dump(true,"this");
    double res=0.;
    double pw=pow(256,psize-1);
    if(psign<0){
      //myDebug()<<"negative";
      for(long i=psize;--i>=0;){
	res+=((~(pp[i]&0xff))&0xff)*pw;
	//myDebug()<<"i="<<i<<"res="<<(unsigned long)res;
	pw/=256;
      }
      res=-(res+1);
    }else{
      //myDebug()<<"positive";
      for(long i=psize;--i>=0;){
	res+=pp[i]*pw;
	//myDebug()<<"i="<<i<<"res="<<res;
	pw/=256;
      }
    }
    //myDebug()<<"res="<<res;
    return res;
  }

  // ----------------------------------------------------------------
  APInteger &operator+=(const APInteger &a)
  {
#define debug 
#undef debug 
#ifdef debug
    myDebug()<<"APInteger::operator+=(long) "<<dump(true,"x")
	     <<a.dump(true,"a");
#endif

    unsigned char u=0;
    unsigned char f=(psign<0?0xff:0);
    unsigned char fa=(a.psign<0?0xff:0);
    unsigned long N=(psize<a.psize?psize:a.psize);

    for(unsigned long i=0;i<N;i++){
      unsigned short s=(pp[i]+a.pp[i]+u);
      pp[i]=(s&0xff);
      u=((s/0x100)&0x1);
    }
#ifdef debug
    myDebug()<<"common loop"<<dump(true,"this")<<"u="<<u;
#endif
    
    if(psize>a.psize){
#ifdef debug
      myDebug()<<"psize>a.psize";
#endif
      for(unsigned long i=a.psize;i<psize;i++){
	unsigned short s=(pp[i]+fa+u);
	pp[i]=(s&0xff);
	u=((s/0x100)&0x1);
      }
#ifdef debug
      myDebug()<<"x      loop"<<dump(true,"this")<<"u="<<u;
#endif
    }else if(psize<a.psize){
#ifdef debug
      myDebug()<<"psize<a.psize";
#endif
      grow(a.psize);
      for(unsigned long i=psize;i<a.psize;i++){
	unsigned short s=(a.pp[i]+f+u);
	pp[i]=(s&0xff);
	u=((s/0x100)&0x1);
      }
      psize=a.psize;
#ifdef debug
      myDebug()<<"a      loop"<<dump(true,"this")<<"u="<<u;
#endif
    }
    
    unsigned short s=f+fa+u;
    //  f fa u   s  add  sign done
    // 00 00 0 000   x    1    x
    // 00 00 1 001   1    1    x
    //
    // ff 00 0 0ff   x   -1    x
    // ff 00 1 100   x   +1    x
    //
    // ff ff 0 0fe   fe  -1
    // ff ff 1 0ff   x   -1    x
    if((s&0xff)==0){
      psign=1;
    }else if((s&0xff)==1){
      setMinSize(psize+1);
      pp[psize-1]=1;
      psign=1;
    }else if((s&0xff)==0xff){
      psign=-1;
    }else if((s&0xff)==0xfe){
      setMinSize(psize+1);
      pp[psize-1]=0xfe;
      psign=-1;
    }else{
      myDebug()<<"unkown s case" ; exit(1);
    }
    
#ifdef debug
    myDebug()<<"result     "<<dump(true,"this");
#endif
    
    normalize();
    return *this;
  }

  // ----------------------------------------------------------------
  APInteger operator-()const 
  {
    APInteger res=*this;
    return res.neg();
  }
  // ----------------------------------------------------------------
  APInteger &neg()
  {
#define debug 
#undef debug 
#ifdef debug
    myDebug()<<"APInteger::neg() "<<dump(true,"x");
#endif
    
    if(psize==0 && psign>0){ return *this;}


    unsigned char u=1;
    for(unsigned long i=0;i<psize;i++){
      unsigned short s=(((~pp[i])&0xff)+u);
      pp[i]=(s&0xff);
      u=((s/0x100)&0x01);
    }
#ifdef debug
    myDebug()<<"loop     "<<dump(true,"x")<<"u="<<u;
#endif
    if(u){
      setMinSize(psize+1);
      pp[psize-1]=1;
    }
    psign*=-1;

#ifdef debug
    myDebug()<<"result     "<<dump(true,"res");
#endif

    normalize();
    return *this;
  }

  // ----------------------------------------------------------------
  APInteger &operator*=(const APInteger &a)
  {
#define debug 
#undef debug 
#ifdef debug
    myDebug()<<"APInteger::operator*=(long) "<<dump(true,"x")
	     <<a.dump(true,"a");
#endif

    if(a.isZero()){ psign=1;psize=0; return *this; }
    if(isZero() || a.isOne()){ return *this; }
    if(isOne()){ *this=a;return *this; }
    if(a.isMinusOne()){ neg();return *this; }
    if(isMinusOne()){ *this=-a;return *this; }
    
    long N=psize;
    long M=a.psize;

    grow(N+M+1);
    
    pp[N+M-1]=0;
    pp[N+M]=0;


    psize=N+M+1;
    
    if(psign>0 && a.psign>0){
      
      for(long I=N+M-1;--I>=0;){
	long i0=MyMath::max<long>(I-N+1,0);
	long i1=MyMath::min<long>(I,M-1);
	for(long i=i0;i<=i1;i++){
	  unsigned short s=pp[I-i]*a.pp[i];
	  if(i==i0){
	    pp[I]=(s&0xff);
	  }else{
	    s+=pp[I];
	    pp[I]=(s&0xff);
	  }
	  s=((s/0x100)&0xff);
	  long k=I+1;
	  while(s){
	    //myDebug()<<"k="<<k;
	    //dump(true);
	    //dump(true);
	    s=pp[k]+s;
	    pp[k]=(s&0xff);
	    s=((s/0x100)&0xff);	  
	    //dump(true);
	    k++;
	  }
	}
      }
    }else {
      unsigned char fx=(  psign>0?0x0:0xff);
      unsigned char fa=(a.psign>0?0x0:0xff);
	
	for(long I=N+M-1+2;--I>=0;){
	  long i0=MyMath::max<long>(I-N+1,0);
	  long i1=MyMath::min<long>(I,M-1);
	  i0=0;
	  i1=I;
	  //myDebug()<<"I="<<I;
	  for(long i=i0;i<=i1;i++){
	    unsigned char aa=(I-i<N?pp[I-i]:fx);
	    unsigned char bb=(  i<M?a.pp[i]:fa);
	    //myDebug()<<"i="<<i<<"aa="<<aa<<"bb="<<bb;
	    unsigned short s=aa*bb;
	    if(i==i0){
	      pp[I]=(s&0xff);
	    }else{
	      s+=pp[I];
	      pp[I]=(s&0xff);
	    }
	    s=((s/0x100)&0xff);
	    long k=I+1;
	    while(s && k<N+M+1){
	      //myDebug()<<"k="<<k;
	      //dump(true);
	      //dump(true);
	      s=pp[k]+s;
	      pp[k]=(s&0xff);
	      s=((s/0x100)&0xff);	  
	      //dump(true);
	      k++;
	    }
	  }
	}
      }
      

    psign*=a.psign;

#ifdef debug
    myDebug()<<"result     "<<dump(true,"res");
#endif

    normalize();
    return *this;
  }


  // ----------------------------------------------------------------
  String dump(bool hex,const String &txt)const
  {
    String res=Sprintf("%s[%lu{%lu}]=%s[",txt(),psize,pinternalsize,(psign>0?"{00}":"{ff}"));
    for(long i=psize;--i>=0;){
      if(hex)
	res+=Sprintf(" %02x",pp[i]);
      else
	res+=Sprintf(" %u",pp[i]);
    }
    return res+"]";
  }

  // ----------------------------------------------------------------
  // String printS()
  // {
  //   String res=Sprintf("%c",(psign>0?'+':'-'));
    
  //   return res;
  // }
private:
  // ----------------------------------------------------------------
  void setMinSize(unsigned long newsize)
  {
    //todo: this is flawed, since it leaves un initialized bytes
    grow(newsize);psize=(psize>newsize?psize:newsize);
  }

  // ----------------------------------------------------------------
  void grow(unsigned long newsize)
  {
    // make sure the memory is at least newsize bytes
    if(newsize<=pinternalsize) return;

    if((pp=(unsigned char*)realloc(pp,newsize*sizeof(unsigned char)))==NULL){
      throw Except("APInteger::grow: bad realloc");
    }
    pinternalsize=newsize;
  }
  
  // ----------------------------------------------------------------
  void normalize()
  {
    if(psign<0){
      for(long i=psize-1;i>=0;i--){
	if(pp[i]==0xff){
	  psize--;
	}else break;
      }
    }else{
      for(long i=psize-1;i>=0;i--){
	if(pp[i]==0){
	  psize--;
	}else break;
      }
    }
    {//todo:remove
      pp=(unsigned char*)realloc(pp,psize*sizeof(unsigned char));
      pinternalsize=psize;
    }
  }
private:
  char psign;
  unsigned long psize,pinternalsize;
  unsigned char *pp;
};

// ------------------------------------------------------------------
APInteger operator+(APInteger a,const APInteger &b)
{
  return (a+=b);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Integer{
public:

  // --------------------------------------------------------------
  Integer():psize(0),psizeinternal(0),psign(1),pp(0)
  {
    grow(sizeof(long));psize=1;pp[0]=0;
  }

  // --------------------------------------------------------------
  Integer(const Integer &other):psize(0),psizeinternal(0),pp(0)
  {
    //myDebug()<<"Integer(const Integer &other)";
    *this=other;
  }

  // --------------------------------------------------------------
  Integer(long x):psize(0),psizeinternal(0),pp(0)
  {
    //int N=log(x)/log(256.);
    //myDebug()<<"x="<<x<<"N="<<N;
    grow(sizeof(long));
    fromLong(x);
  }

  // --------------------------------------------------------------
  ~Integer()
  {
    if(pp) delete [] pp;
  }

  // --------------------------------------------------------------
  void setSign(long s)
  {
    psign=lSign(1,s);
  }

  // --------------------------------------------------------------
  void mulSign(long s)
  {
    psign*=lSign(1,s);
  }

  // --------------------------------------------------------------
  Integer &operator=(long x)
  {
    fromLong(x);
    return *this;
  }

  // --------------------------------------------------------------
  Integer &operator=(const Integer &other)
  {
    //myDebug()<<pp;
    //if(pp) delete [] pp;
    if(psizeinternal>=other.psize){
      memcpy(pp,other.pp,other.psize*sizeof(unsigned char));
      memset(pp+other.psize,0,(psizeinternal-other.psize)*sizeof(unsigned char));
      psize=other.psize;
      psign=other.psign;
    }else{
      psizeinternal=other.psizeinternal;
      pp=new unsigned char[psizeinternal];
      psize=other.psize;
      psign=other.psign;
      memcpy(pp,other.pp,psizeinternal*sizeof(unsigned char));
    }
        
    //for(long i=-1;++i<psizeinternal;) pp[i]=other.pp[i];
    return *this;
  }


  // ----------------------------------------------------------------
  long abscmp(const Integer &a)
  {
    //myDebug()<<"abscmp-------";
    //dump();
    //a.dump();

    if(psize>a.psize){
      for(long i=psize-1;i>=a.psize;i--){
	if(pp[i]!=0)return 1;
      }
    }
    if(psize<a.psize) {
      for(long i=a.psize-1;i>=psize;i--){
	if(a.pp[i]!=0)return -1;
      }
    }
    //myDebug()<<"\tequal psize"<<psize;
    long N=(psize<a.psize?psize:a.psize);
    for(long i=N;--i>-1;){
      //myDebug()<<"\ti="<<i<<pp[i]<<a.pp[i];
      if(pp[i]>a.pp[i]) return 1;
      if(pp[i]<a.pp[i]) return -1;
    }
    return 0;
  }


  // --------------------------------------------------------------
  Integer &operator+=(const Integer &a)
  {
    //myDebug()<<" -------------- operator+="<<toLong()<<a.toLong();
    if(a.psize==1 && a.pp[0]==0) return *this;
    if(psize==1 && pp[0]==0) { *this=a; return *this; }
    if(psign*a.psign>0){
      add(a);
    }else{
      //different signs
      // |this|>|a| -> this+a=s*|this|+sa*|a|=s*(|this|-s*sa*|a|)=s*(|this|-|a|)
      // |this|<|a| -> this+a=s*|this|+sa*|a|=sa*(-|this|+|a|)
      //myDebug()<<"abscmp(a)="<<abscmp(a);
      if(abscmp(a)>0){
	//myDebug()<<"|a|>0";
	diff(a);
      }else{
	//myDebug()<<"|a|<0"<<a.psize<<a.psizeinternal;
	Integer aa(a); // aa=a
	swapAll(aa);  // this=aa, aa=this
	diff(aa);
      }
      return *this;

    }
    return *this;
  }
  // --------------------------------------------------------------
  Integer &operator*=(const Integer &a)
  {
    if( psize==1 && pp[0]==0 ) return *this;
    if( (a.psize==1 && a.pp[0]==0) || (psize==1 && pp[0]==0)) {
      psize=1; pp[0]=0;
      return *this;
    }

    mul(a);

    return *this;
  }  

    
  // --------------------------------------------------------------
  long toDouble()const
  {
    double res=0.;
    double pw=pow(256,psize-1);
    for(long i=psize;--i>=0;){
      res+=pp[i]*pw;
      pw/=256;
    }
    return res*psign;

  }
  
  // --------------------------------------------------------------
  long toLong()const
  {
    long res=0;
    long n=0;
    for(long i=-1;++i<psize;){
      res+=(pp[i]<<n);n+=8;
    }
    return res*psign;
  }

  // --------------------------------------------------------------
  void dump(bool hex,const String &txt="")const
  {
    printf("bytes(%s)=%d*[",txt(),psign);
    for(long i=psize;--i>-1;){
      if(hex)
	printf("%x ",pp[i]);
      else
	printf("%d ",pp[i]);
    }
    printf("]\n");
  }
private:

  // ---------------------------------------------------------------- 
  void diff(const Integer &a)
  {
    // required |*this|>|a|
    
    //myDebug()<<"----------- diff"<<toLong()<<a.toLong();
    //dump(false,"this");
    //a.dump(false,"a");
    long N=(psize<a.psize?psize:a.psize);
    unsigned char u=0;
    for(long i=-1;++i<N;){
      unsigned short c=(pp[i]-a.pp[i]-u);
      //myDebug()<<"pp="<<pp[i]<<"-a="<<a.pp[i]<<"="<<c<<"flag="<<(c&0x100)<<"rest="<<((0x100+c)&0xff);
      if((c&0x100)){
	pp[i]=((0x100+c)&0xff);
	u=1;
      }else{
	u=0;
	pp[i]=(c&0xff);
      }
      //myDebug()<<"dif="<<pp[i]<<"u="<<u;
    }
    // dump();
    // myDebug()<<"u="<<u;
    // myDebug()<<"N="<<N<<"psize="<<psize;
    for(long i=N-1;++i<psize;){
      unsigned short c=(pp[i]-u);
      if((c&0x100)){
	pp[i]=((0x100+c)&0xff);
	u=1;
      }else{
	u=0;
	pp[i]=(c&0xff);
      }
      //myDebug()<<"dif="<<pp[i]<<"u="<<u;
    }
    //dump();
    // myDebug()<<"u="<<u;
    // myDebug()<<"N="<<N<<"a.psize="<<a.psize;
    for(long i=psize;--i>0;){
      if(pp[i]!=0){
	psize=i+1;
	break;
      }
    }
    //dump();
    //myDebug()<<"------------------------------";
  }

  // ----------------------------------------------------------------
  void fromLong(long x)
  {
    psize=1;
    psign=1;
    if(x==0){ psize=1;pp[0]=0;return; }
    if(x<0)psign=-1;
    x=labs(x);
    long i=0;
    while(x!=0){
      grow(i+1);
      pp[i++]=(x&0xff);
      x>>=8; //todo: vulnearble if arithmetic right shift
    }
  }
  // ----------------------------------------------------------------
  void add(const Integer &a)
  {
    //myDebug()<<"----------- operator+="<<toLong()<<a.toLong();
    long N=(psize<a.psize?psize:a.psize);
    unsigned char u=0;
    for(long i=-1;++i<N;){
      unsigned short c=(pp[i]+a.pp[i]+u);
      //myDebug()<<"pp="<<pp[i]<<"a="<<a.pp[i]<<"sum="<<c<<(c&0xff);;
      pp[i]=(c&0xff);
      u=((c>>8)&0xff);
      //myDebug()<<"pp="<<pp[i]<<"u="<<u;
    }
    //dump();
    //myDebug()<<"u="<<u;
    //myDebug()<<"N="<<N<<"psize="<<psize;
    for(long i=N-1;++i<psize;){
      unsigned short c=(pp[i]+u);
      pp[i]=(c&0xff);
      u=((c>>8)&0xff);
    }
    //dump();
    //myDebug()<<"u="<<u;
    //myDebug()<<"N="<<N<<"a.psize="<<a.psize;
    for(long i=N-1;++i<a.psize;){
      unsigned short c=(a.pp[i]+u);
      grow(i+1);
      pp[i]=(c&0xff);
      u=((c>>8)&0xff);
    }
    //dump();
    //myDebug()<<"u="<<u;
    if(u!=0){
      grow(psize+1);
      pp[psize-1]=u;
    }
    //myDebug()<<"------------------------------";
  }
  // ----------------------------------------------------------------
  void mul(const Integer &a)
  {
    //myDebug()<<"\n--------mul"<<toLong()<<"*"<<a.toLong();
    long N=psize;
    long M=a.psize;
    psign*=a.psign;
    //myDebug()<<"N="<<N;
    //myDebug()<<"M="<<M;
    memset(pp+N,0,(psizeinternal-psize)*sizeof(unsigned char));

    //dump(true);
    //a.dump(true);
    for(long I=N+M-1;--I>=0;){
      //myDebug()<<"I="<<I;
      long i0=MyMath::max<long>(I-N+1,0);
      long i1=MyMath::min<long>(I,M-1);
      //myDebug()<<"i01="<<i0<<i1;
      for(long i=i0;i<=i1;i++){
	unsigned short s=pp[I-i]*a.pp[i];
	//myDebug()<<"\ta["<<I-i<<"]*b["<<i<<"]="<<((s>>8)&0xff)<<(s&0xff);
	//dump(true);
	grow(I+1,true);
	//dump(true);
	if(i==i0){
	  pp[I]=(s&0xff);
	}else{
	  s+=pp[I];
	  pp[I]=(s&0xff);
	}
	//dump(true);
	s=((s>>8)&0xff);
	//myDebug()<<"rest="<<s;
	long k=I+1;
	while(s){
	  //myDebug()<<"k="<<k;
	  //dump(true);
	  grow(k+1,true);
	  //dump(true);
	  s=pp[k]+s;
	  pp[k]=(s&0xff);
	  s=((s>>8)&0xff);	  
	  //dump(true);
	  k++;
	}
	
      }
    }
  }
  

  // --------------------------------------------------------------
  void swapAll(Integer &a)
  {
    MyMath::swap(psize,a.psize);
    MyMath::swap(psign,a.psign);
    MyMath::swap(psizeinternal,a.psizeinternal);
    MyMath::swap(pp,a.pp);
  }
  
  // --------------------------------------------------------------
  void grow(long newsize,bool dontshrink=false){
    // grow needed space and may be a bitmore, set psize=newsize
    //
    //myDebug()<<"grow: newsize="<<newsize;
    //myDebug()<<"grow: psize="<<psize<<"psizeinternal="<<psizeinternal;
    if(newsize<=psizeinternal) {
      if(dontshrink) {
	psize=(psize>newsize?psize:newsize);
      }else{
	psize=newsize;
      } 
      return;
    }
    long ns=(long)(psizeinternal*1.6);
    ns=(ns>newsize?ns:newsize);
    //myDebug()<<"ns="<<ns;
    unsigned char *tmp=new unsigned char[ns];
    //for(long i=-1;++i<psizeinternal;)tmp[i]=pp[i];
    memcpy(tmp,pp,psizeinternal*sizeof(unsigned char));
    //for(long i=psizeinternal-1;++i<ns;)tmp[i]=0;
    memset(tmp+psizeinternal,0,(ns-psizeinternal)*sizeof(unsigned char));
    
    MyMath::swap(tmp,pp);
    if(tmp) delete [] tmp;
    if(dontshrink) {
      psize=(psize>newsize?psize:newsize);
    }else{
      psize=newsize;
    } 
    psizeinternal=ns;
    //myDebug()<<"now psize="<<psize<<"psizeinternal="<<psizeinternal;
    //dump();
  }
    
private:
  long psize,psizeinternal;
  signed char psign;
  unsigned char *pp;
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrimeNumbers::setMaxPrime(long x)
{

  long pmax=0;
  if(pprimes.size()>0) pmax=pprimes.last();
  
  //myDebug()<<"pmax="<<pmax;

  if(pmax<x){
    
    for(long m=MyMath::max<long>(2,pmax+1);m<=x;m++){
      bool isp=true;
      for(long i=-1;++i<pprimes.size();){
	if(m!=i && (m%pprimes.at(i))==0){
	  isp=false;
	  break;
	}
      }
      if(isp) pprimes<<m;
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrimeNumbers::setMaxPrimeIndex(long ind)
{
  //myDebug()<<"setMaxPrimeIndex"<<ind;
  if(ind < pprimes.size()) return;
  
  long pmax=0;
  if(pprimes.size()>0) pmax=pprimes.last();
  
  //myDebug()<<"pmax="<<pmax;
  
  long m=MyMath::max<long>(2,pmax+1);

  //myDebug()<<"m="<<m;
  
  while(ind>=pprimes.size()){
    bool isp=true;
    for(long i=-1;++i<pprimes.size();){
      if(m!=i && (m%pprimes.at(i))==0){
	isp=false;
	break;
      }
    }
    if(isp) pprimes<<m;
    m++;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrimeNumbers::print()
{
  printf("prime numbers=\n");
  for(long i=-1;++i<pprimes.size();){
    printf("%ld ",pprimes.at(i));
    if((i+1)%10==0){
      printf("\n ");
    }
  }
  if(pprimes.size()%10!=0) printf("\n ");
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
IPrimeDecomposed &IPrimeDecompositions::number(long x)
{

  //myDebug()<<"number x="<<x;
  long xmax=pcache.size()-1;
  //myDebug()<<"number xmax="<<xmax;

  if(x<=xmax){
    //myDebug()<<"ret"<<pcache[x];
    return pcache[x];
  }

  pcache.resize(x+1);
  for(long n=xmax+1;n<=x;n++){
    primeDecompose(n,pcache[n],pprimes);
  }

  //myDebug()<<"ret"<<pcache[x];
  return pcache[x];
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IPrimeDecompositions::primeDecompose(long x,IPrimeDecomposed &pa,
					  PrimeNumbers &primes)
{


  //myDebug()<<"primeDecompose"<<x;
  primes.setMaxPrime(x);
  //primes.print();

  long a=labs(x);
  if(x==0){
    pa.setZero();
    return;
  }else{
    pa.setOne();
  }


  
  if(a<2) return;
  //printf("A=%6ld\n",a);
  for(long i=-1;++i<primes.size();){
    //while(a!=1){
    //printf("\tf[%ld]=%6ld\n",i,primes[i]);
    while((a%primes[i])==0){
      //myDebug()<<"\tpa  "<<pa;
      pa.setSize(i+2);
      //myDebug()<<"\tpare"<<pa;
      pa[i+1]++;
      a/=primes[i];
      //printf("\ta=%6ld p(%6ld)=%6ld\n",a,i,pa[i+1]);
      if(a==1) break;
    }
    if(a==1) break;
    
  }
  //myDebug()<<"ret"<<pa;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String IPrimeDecompositions::printS(IPrimeDecomposed &pa)
{
  String res;
  if(pa[0]==0) return "0";
  res=Sprintf("%ld",pa[0]);
  if(pa.size()==1) return res;
  for(long i=0;++i<pa.size();){
    if(pa[i]>0) {
      res+=Sprintf("*%ld^%ld",pprimes[i-1],pa[i]);
    }else if(pa[i]<0) {
      res+=Sprintf("*%ld^(%ld)",pprimes[i-1],pa[i]);
    }
  }
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double IPrimeDecompositions::toDouble(IPrimeDecomposed &pa)
{
  if(!pa[0]) return 0.;

  double res=pa[0];
  for(long i=0;++i<pa.size();){
    if(!pa[i])continue;
    res*=pow((double(pprimes[i-1])),pa[i]);
    //!Print *,i,primes(i),primfactors(i)
  }
  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IPrimeDecompositions::toLong(IPrimeDecomposed &pa)
{
  if(!pa[0]) return 0.;

  long res=pa[0];
  for(long i=0;++i<pa.size();){
    if(!pa[i])continue;
    if(pa[i]<0){
      myDebug()<<"IPrimeDecompositions::toLong: numbers must be integer!";
      exit(0);
    }
    long p=pprimes[i-1];
    for(long j=1;j<=pa[i];j++){
      res*=p;
    }
  }
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
APInteger IPrimeDecompositions::toAPInteger(IPrimeDecomposed &pa)
{
  APInteger res;
  if(!pa[0]) return res;

  res=pa[0];

  for(long i=0;++i<pa.size();){
    if(!pa[i])continue;
    if(pa[i]<0){
      myDebug()<<"IPrimeDecompositions::toAPInteger: numbers must be integer!";
      exit(0);
    }
    APInteger p(pprimes[i-1]);
    for(long j=1;j<=pa[i];j++){
      res*=p;
      //myDebug()<<i<<j<<pprimes[i-1];
    }
  }
  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IPrimeDecompositions::fac(long x,IPrimeDecomposed &pa)
{
  //myDebug()<<"IPrimeDecomposed::fac x="<<x;
  pa.setOne();
  
  //myDebug()<<"pa="<<pa;

  for(long i=x;i>=2;i--){
    //myDebug()<<"i="<<i;
    IPrimeDecomposed &pf=number(i);
    //myDebug()<<"\tpa="<<pa;
    //myDebug()<<"\tpf="<<pf;
    if(pf.size()==1) continue;
    for(long j=1;j<pf.size();j++){
      if(!pf[j]) continue;
      //myDebug()<<"\t\tj="<<j<<"pfj="<<pf[j];
      //myDebug()<<"\t\tpabe="<<pa.pp;
      pa.setMinSize(j+1);
      //myDebug()<<"\t\tpare="<<pa.pp;
      pa[j]+=pf[j];
      //myDebug()<<"\t\tpa="<<pa.pp;
    }
  }
  //myDebug()<<"pa="<<pa.pp;
  //myDebug()<<"pa="<<printS(pa);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
IPrimeDecomposed &IPrimeDecomposed::operator*=(const IPrimeDecomposed &b)
//void IPrimeDecompositions::muleq(IPrimeDecomposed &a,IPrimeDecomposed &b)
{
  if(pp[0]==0) return *this;
  if(b.pp[0]==0) {
    setZero();
    return *this;
  }
  pp[0]*=b.pp[0];
  
  for(long j=0;++j<b.size();){
    setMinSize(j+1);
    pp[j]+=b.pp[j];
  }

  return *this;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
IPrimeDecomposed &IPrimeDecomposed::operator/=(const IPrimeDecomposed &b)
{
  if(pp[0]==0) return *this;
  if(b.pp[0]==0) {
    printf("IPrimeDecompositions::diveq division by zero!\n");
    exit(1);
  }
  pp[0]*=b.pp[0];
  
  for(long j=0;++j<b.size();){
    setMinSize(j+1);
    pp[j]-=b.pp[j];
  }

  return *this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IPrimeDecompositions::add(APInteger &s,IPrimeDecomposed &f,IPrimeDecomposed &a)
{
  IPrimeDecomposed nf;
  nf.setOne();
  //myDebug()<<"\nadd:\nsin="<<s.toDouble();
  //myDebug()<<"fin="<<printS(f)<<f<<toDouble(f);
  //myDebug()<<"ain="<<printS(a)<<a<<toDouble(a);
  //  double d=s.toDouble()*toDouble(f)+toDouble(a);
  
  long N=MyMath::max(f.size(),a.size());
  
  for(long i=0;++i<N;){
    long d;
    if(a[i]<f[i]){
      d=a[i];
    }else{
      d=f[i];
    }
    nf.setMinSize(i+1);
    f.setMinSize(i+1);
    a.setMinSize(i+1);
    nf[i]=d;
    f[i]-=d;
    a[i]-=d;
  }

  //myDebug()<<"f~="<<f<<printS(f)<<toDouble(f)<<"asI"<<toInteger(f).toDouble();
  //myDebug()<<"a~="<<a<<printS(a)<<toDouble(a)<<"asI"<<toInteger(a).toDouble();;

  //s.dump(false,"s");
  //toInteger(f).dump(false,"f");
  s*=toAPInteger(f);
  //s.dump(false,"s*f");
  //myDebug()<<"f*s="<<s.toDouble();
  //toInteger(a).dump(false,"a");
  s+=toAPInteger(a);
  //s.dump(false,"s*f+a");
  //myDebug()<<"f*s+a="<<s.toDouble();
  f=nf;
  //myDebug()<<"new s="<<s.toDouble();
  //myDebug()<<"new f="<<printS(f)<<f<<toDouble(f);
  //myDebug()<<"new d="<<d<<s.toDouble()*toDouble(f);
  //  exit(0);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IPrimeDecompositions::add(double &s,IPrimeDecomposed &f,IPrimeDecomposed &a)
{
  IPrimeDecomposed nf;
  nf.setOne();
  //myDebug()<<"s="<<s;
  //myDebug()<<"f="<<f;
  //myDebug()<<"a="<<a;
  
  long N=MyMath::max(f.size(),a.size());
  
  for(long i=0;++i<N;){
    long d;
    if(a[i]<f[i]){
      d=a[i];
    }else{
      d=f[i];
    }
    nf.setMinSize(i+1);
    f.setMinSize(i+1);
    a.setMinSize(i+1);
    nf[i]=d;
    f[i]-=d;
    a[i]-=d;
  }

  //myDebug()<<"f="<<f<<toDouble(f);
  //myDebug()<<"a="<<a<<toDouble(a);

  s=s*toDouble(f)+toDouble(a);
  f=nf;
  //myDebug()<<"new s="<<s;
  //myDebug()<<"new f="<<f<<toDouble(f);
  //  exit(0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Wigner3J::Wigner3J()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Wigner3J::~Wigner3J()
{
}


// ------------------------------------------------------------------
Integer operator+(Integer a,Integer &b)
{
  a+=b;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Wigner3J::Wigner3JTest()
{
  long jmax=24;

  Wigner3J w3j;
  
  if(false){

    try{

      if(false){
	APInteger api(-0x87f73f);
	APInteger apj(-1);
	myDebug()<<"api="<<api.toLong();
	apj*=api;
	myDebug()<<apj.dump(true,"apr")<<"="<<apj.toLong();
	exit(0);
      }
      


      if(false){
	APInteger api;
	APInteger apj;
	long N0=-0xffffff;
	long N1= 0xffffff;
	for(long i=N0;i<=N1;i+=1){
	  api=i;
	  apj=-api;
	  if(!(api+apj).isZero()){
	    myDebug()<<api.dump(true,"api")<<"="<<api.toLong()<<"!="<<i;
	    myDebug()<<apj.dump(true,"apj")<<"="<<apj.toLong()<<"!="<<0;
	    myDebug()<<i;
	    exit(1);
	  }
	  
	}
	exit(0);
      }
      if(true){
	APInteger api;
	APInteger apj;
	APInteger &apr=apj;
	long N0=  -0xffffff*1;
	long N1=  +0xffffff*1;
	long dN=0x1267;
	
	
	for(long i=N0;i<=N1;i+=dN){
	  if(!(i%10)){
	    myDebug()<<"i="<<i;
	  }
	  api=i;
	  for(long j=N0;j<=N1;j+=dN){
	    apj=j;
	    apj*=api;
	    
	    //myDebug()<<apr.dump(true,"apr");
	    if(apr.toLong()!=i*j){
	      myDebug()<<apr.dump(true,"apr")<<"="<<apr.toLong()<<"!="<<i*j;
	      myDebug()<<i<<j;
	      exit(1);
	    }
	  }
	}
      }

      
    }catch(Except &ex){
      myDebug()<<ex.msg();
    }
    
    myDebug()<<"done";
    exit(0);
  }


  if(false){

    Integer in,jn;
    Integer &rn=jn;
    long N=10000;
    for(long i=-N;i<N;i++){
      in=i;
      //myDebug()<<in.toLong();
      for(long j=-N;j<N;j+=1){
       	jn=j;
	jn*=in;
	if(labs(rn.toLong()-i*j)!=0){
	  myDebug()<<"error:"<<i<<"*"<<j<<"/="<<rn.toLong();
	  exit(1);
	}

	if(fabs(rn.toLong()-rn.toDouble())!=0){
	  myDebug()<<"error:"<<i<<"*"<<j<<"/="<<rn.toLong()<<rn.toDouble();
	  exit(1);
	}
      }
    }

    exit(0);
  }
  if(false){
    long j1=40*2;
    long j2=40*2;
    long j3=60*2;
    long m1=0;
    long m2=0;
    long m3=0;
    double w=w3j.wigner3J(j1,m1,j2,m2,j3,m3);
    Wigner3J::printW3J(j1,m1,j2,m2,j3,m3,w);
    exit(0);
  }

  #define BOUND(x) (jmax)
  //#define BOUND(x) (x)
  long count=0;
  long count0=0;
  if(0){
    long l=2*1;
    long j1=l;
    long j3=l;
	    
    for(long j2=-1;++j2<=4;){
      if(j2%2) continue;
      for(long m2=-BOUND(j2);m2<=BOUND(j2);m2++){
	for(long m1=-BOUND(j1);m1<=BOUND(j1);m1++){
	  for(long m3=-BOUND(j3);m3<=BOUND(j3);m3++){
	    
	    double w=w3j.wigner3J(j1,m1,j2,m2,j3,m3);
	    if(fabs(w)>1.0e-10){
	      printf("#No %ld\n",count0);
	      Wigner3J::printW3J(j1,m1,j2,m2,j3,m3,w);
	      printf("\n");
	      ++count0;
	    }
	    ++count;
	  }
	}
      }
    }
  }else{
    for(long j1=-1;++j1<=jmax;){
      for(long m1=-BOUND(j1);m1<=BOUND(j1);m1++){
	for(long j2=-1;++j2<=jmax;){
	  for(long m2=-BOUND(j2);m2<=BOUND(j2);m2++){
	    for(long j3=-1;++j3<=jmax;){
	      for(long m3=-BOUND(j3);m3<=BOUND(j3);m3++){
		
		double w=w3j.wigner3J(j1,m1,j2,m2,j3,m3);
		if(fabs(w)>1.0e-10){
		  printf("#No %ld\n",count0);
		  Wigner3J::printW3J(j1,m1,j2,m2,j3,m3,w);
		  printf("\n");
		  ++count0;
		}
		++count;
	      }
	    }
	  }
	}
      }
    }
  }
  printf("# total   = %ld\n",count );
  printf("# nonzero = %ld\n",count0);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Wigner3J::printW3J(long j1,long m1,long j2,long m2,
			long j3,long m3,double w)
{
  // ----------------------------------------------------------------
  // input is 2*physical momentum to allow for half integer
  // ----------------------------------------------------------------
#define PRT(x)					\
  if((x)%2==0){					\
    printf("%5ld  ",(x)/2);			\
  }else{					\
    printf("%5ld/2",x);				\
  }
  printf("/");
  PRT(j1);
  PRT(j2);
  PRT(j3);
  printf("\\\n");
  printf("\\");
  PRT(m1);
  PRT(m2);
  PRT(m3);
  printf("/=%30.15E\n",w);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Wigner3J::isPhysical(long j1,long m1,long j2,long m2,long j3,long m3)
{
  // input: j1 equals 2*J1, where J1 is the physical momentum number
  
  if(m1+m2+m3!=0) return false;
  if(j1<0 || j2<0 || j3<0) return false;
  if(fabs(m1)>j1 || fabs(m2)>j2 || fabs(m3)>j3) return false;
  if((j1-m1)%2!=0 || (j2-m2)%2!=0 || (j3-m3)%2!=0) return false;
  if((j1+j2+j3)%2!=0) return false;
  if( (m1==0 && m2==0 && m3==0) && ((j1+j2+j3)%4!=0) ) return false;
  if(fabs(j1-j2)>j3 ||  j3>j1+j2) return false;
  return true;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Wigner3J::wigner3J(long j1,long m1,long j2,long m2,long j3,long m3)
{
  // Wigner 3J symbol
  // input: j1 equals 2*J1, where J1 is the physical momentum number
  // it is accurate at least up to J=4/2 (input j=9)


  if(!isPhysical(j1,m1,j2,m2,j3,m3)) return 0.;
    
  //  printf("# inp= %3ld %3ld %3ld %3ld %3ld %3ld\n", j1,m1,j2,m2,j3,m3);
    

  double  sig=pow(-1.0,(-j1+j2+m3)/2);

  IPrimeDecomposed ppre=wignerPre(j1,m1,j2,m2,j3,m3);
  APInteger su1=wignerSu(j1,m1,j2,m2,j3,m3,ppre);
  return sig*sqrt(pipdcmp.toDouble(ppre))*su1.toDouble();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
IPrimeDecomposed Wigner3J::wignerPre(long j1,long m1,long j2,long m2,long j3,long m3)
{
  // return square of prefactor
  // input: j1 equals 2*J1, where J1 is the physical momentum number

  IPrimeDecomposed pa;
  IPrimeDecomposed pb;

  pipdcmp.fac(((2+j1+j2+j3)/2),pa);
  pipdcmp.fac(((j1-m1)/2),pb);
  pa*=pb;
  pipdcmp.fac(((j1+m1)/2),pb);
  pa*=pb;
  pipdcmp.fac(((j3-m3)/2),pb);
  pa*=pb;
  pipdcmp.fac(((j3+m3)/2),pb);
  pa*=pb;

  pipdcmp.fac(((j1+j2-j3)/2),pb);
  pa/=pb;
  pipdcmp.fac(((j1-j2+j3)/2),pb);
  pa/=pb;
  pipdcmp.fac(((-j1+j2+j3)/2),pb);
  pa/=pb;
  pipdcmp.fac(((j2-m2)/2),pb);
  pa/=pb;
  pipdcmp.fac(((j2+m2)/2),pb);
  pa/=pb;


  //myDebug()<<"# MUL="<<pipdcmp.printS(pa)<<pipdcmp.toDouble(pa);
  
  return pa;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
APInteger Wigner3J::wignerSu(long j1,long m1,long j2,long m2,long j3,long m3,
			  IPrimeDecomposed &pre)
{
  MyMath::dFactorial fac;


  IPrimeDecomposed pa;
  IPrimeDecomposed pb;
  IPrimeDecomposed pfi;
  APInteger si;
  pfi.setOne();
  

  for(long k=0;k<=MyMath::min((j1-m1)/2,(j3+m3)/2);k++){

    pipdcmp.fac(((-2*k+j2+j3-m1)/2),pa);
    pipdcmp.fac(((-2*k+j1+j2+m3)/2),pb);
    pa*=pb;
    pipdcmp.fac(k,pb);
    pa/=pb;
    pipdcmp.fac(((2-2*k+j1+j2+j3)/2),pb);
    pa/=pb;
    pipdcmp.fac(((-2*k+j1-m1)/2),pb);
    pa/=pb;
    pipdcmp.fac(((-2*k+j3+m3)/2),pb);
    pa/=pb;

    pa[0]=(((2*k+j1-m1)/2)%2==0?1:-1);



    if(k==0){
      si=1;
      pfi=pa;
    }else{
      pipdcmp.add(si,pfi,pa);
    }



    
  }



  pre*=pfi;
  pre*=pfi;
  si*=pfi[0];
  


  return si;
}


/*EOF*/ 
