/*************************************************************************
 * @(#)App.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   16 Oct 2008
 *************************************************************************
 */

#include <unistd.h>
#include "MyDebug.hh"
#include "debugxx.hh"

#include "Screen.hh"
#include "Keys.hh"
#include "App.hh"
#include "Sysinfo.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

SObject::SObject(App *a,SObject *p,String const &cn)
  :classname(cn),app(a),pparent(p)
{
  //  a->registerObject(this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

SObject::~SObject()
{
  //NSF("~SObject");
  //HIER(nsubfunc);
//   app->unRegisterObject(this);
//   if(parent) app->registerObject(pparent);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
App::App(int argc,char *argv[])
  :s(0),appname(""),colorlayout(1),colorscheme(3),acs(1),reverse(1)
  ,screendebug(0),dumpmode(Screen::NO_DUMP),ppipein(0)
  ,cwdir(Sysinfo::getCWDir())
  ,classname("App"),mainwidget(0)
{
  
  // debugpipe(); 
  /*
    
  while ( (ch=getpipekey() ) != EOF ) {
  printf("line %d: <%d:%c>\n",pipeline(),ch,(char)ch);
  }
  exit(EXIT_NORMAL);
  */
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
App::~App()
{
  NSF("~App");
  //HIER(nsubfunc);

  if(s) delete s; s=0;
  
  if(mainwidget) delete mainwidget; mainwidget=0;
  
  if(chdir(cwdir())){
    (void)printf("internal(%s): Could not change to directory %s!\n"
		 ,nsubfunc,cwdir());
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void App::scanopts(int argc,char *argv[])
{
  long i=1;

  if(argc>0) appname=argv[0];
  
  StringList sl=appname.breakup("/");
  if(sl.size()>=0){
    appname=sl[sl.size()-1];
    sl.remove(appname);
    apppath=sl.join("/");
  }


  while( i<argc ) {
    if( argv[i][0]=='-' ){
      long j=i+1;
      StringList sl;
      while( j<argc && argv[j][0]!='-' ) {
	sl.append(argv[j]);
	j++;
      }
      onArgument(argv[i],sl);
    }
    i++;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void App::usage()
{
  (void)printf("\
     -c: the number n depends on the capabilities of your $TERM,\n\
         standard is COLORED \n\
         n : 0   use BOLD highlighting\n\
         n : 1   use COLOR highlighting (STANDARD)\n\
         color may work i.e. with $TERM=xterm|linux|aixterm|dtterm\n\n\
     -r: switch off the reverse mode for editing fields\n\n\
     -a: switch off the use off alternate character sets\n\
         alternate characters should work with $TERM=dtterm|linux\n\n\
     -cs: set color scheme\n");
     for(long i=0;i<Screen::colorSchemes.size();i++){
        if(i==0)
          printf("         values: %ld: %s\n",i,Screen::colorSchemes[i]());
        else
          printf("                 %ld: %s\n",i,Screen::colorSchemes[i]());
      }
   (void)printf("\n\
     -screendump: CTRL-S will create a screen dump of the current screen.\n\
                  dump_mode may be\n\
                     ascii - creates a file with asciis\n\
                     ps    - creates a ps file\n\n\
     -pipe: This is usefull for automation. \n\
            All key strokes are piped in from stdin.\n\
            All toggling switches are changed into normal\n\
            editing behaviour. This serves for automation, since then\n\
            the state is not depending an the number of edit invokations\n\
            for a certain item.\n\
            In pipemode a special syntax has to be obeyed.\n\
            See documentation.\n\n\
     -sd: debugging mode for screen layout\n\n\
     -h: prints help message\n\n");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
#define noargfmt "Option %s does not expect arguments\n"
#define oneargfmt "Option %s needs one argument.\n"
#define NO_ARG					\
    if(sl.size()!=0)\
      throw Except(Sprintf(noargfmt,opt()));
#define ONE_ARG\
    if(sl.size()!=1)\
      throw Except(Sprintf(oneargfmt,opt()));

long App::onArgument(String const &opt,StringList const &sl)
{
  long ok;
  long cs=-1;
  long l;

  if(opt=="-h"){
    usage();    
    throw Except("");
  }else if(opt=="-pipe"){
    ppipein=1; NO_ARG;
  }else if(opt=="-r"){
    reverse=0; NO_ARG;
  }else if(opt=="-sd"){
    screendebug=1; NO_ARG;
  }else if(opt=="-a"){
    acs=0; NO_ARG;
  }else if(opt=="-cs"){
    ok=0;
    if(sl.size()==1){
      ok=1;
      try{ cs=sl[0].convertToLong(); }catch(...){ok=0;};
      ok&=(cs >=0 && cs< Screen::colorSchemes.size());
    }
    if(!ok)
      throw 
	Except(Sprintf("Option -cs needs argument 0-%ld!\n"
				,Screen::colorSchemes.size()-1)); 
    colorscheme=cs;
  }else if(opt=="-c"){
    ok=0;
    if(sl.size()==1){
      ok=1;
      try{ l=sl[0].convertToLong(); }catch(...){ok=0;};
      ok&=(l==0 || l==1 );
    }
    if(!ok)
      throw 
	Except("Option -c needs argument 0 or 1!\n"); 
    colorlayout=l;
  }else if(opt=="-screendump"){
    dumpmode=Screen::ASCII_DUMP;
    ONE_ARG;
    if(sl[0]=="ps")
      dumpmode=Screen::PS_DUMP;
    else if(sl[0]=="ascii")
      dumpmode=Screen::ASCII_DUMP;
    else
      throw Except("Option -screendump: argument must be ascii or ps.\n");
  }else
    return 0;
  
  return 1;
      
}

#undef NO_ARG
#undef ONE_ARG
#undef noargfmt
#undef oneargfmt

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long App::exec()
{
  s=new Screen(colorlayout,colorscheme,acs,reverse,dumpmode,ppipein,screendebug);
  Event e;


  long ret=TerminateExcept::ExitNormal;
  try{
    if(mainwidget)
      mainwidget->exec();
    if(s) delete s; s=0;
  }catch(TerminateExcept &ex){
    ret=ex.retVal();
    if(s) delete s; s=0;
    printf("\n%s\n",ex.msg());
  }catch(Except &ex){
    if(s) delete s; s=0;
    printf("unhandled error: %s\n",ex.msg());
    ret=TerminateExcept::ExitEmergencyAbort;
  }
  
  if(ppipein && ret==TerminateExcept::ExitNormal){
    fprintf(stderr,"SUCCESS\n");
  }
  
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void App::quit(TerminateExcept:: ExitStatus st)
{
  //NSF("quit()");
  //  HIER(nsubfunc);
  abort(st);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Event  App::getKey()
{

  Event e=s->getKey();

  if(e.key==S_CTRL_C) abort(TerminateExcept::ExitUserAbort);

  return e;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void App::abort(TerminateExcept::ExitStatus rv)
{
  //  if(s) delete s; s=0;
  
  if(rv == TerminateExcept::ExitEmergencyAbort){
    throw TerminateExcept("Abort!",rv);
  }else if(rv == TerminateExcept::ExitUserAbort){
    throw TerminateExcept("User abort!",rv);
  }else{
    throw TerminateExcept("",TerminateExcept::ExitNormal);
  }
    
}







/* ***********************************************************************
 *
 * ***********************************************************************
 */
void App::setMainWidget(SObject *o)
{
  if(mainwidget)
    throw Except("setMainWidget called more than once.\n");

  mainwidget=o;
}


/*EOF*/ 
