/*************************************************************************
 * @(#)Dump.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Nov 2008
 *************************************************************************
 */

#include "debugxx.hh"
#include "Dump.hh"
#include "Screen.hh"
#include "Styles.hh"
#include "Assorted.hh"


static const double DUMP_BLACK[6]={0.,0.,0.,.5,.5,.5};
static const double DUMP_RED  [6]={0.8,0.,0.,1.,.0,.0};
static const double DUMP_GREEN[6]={0.0,0.8,0.0  ,0.0,1.0,0.0};
static const double DUMP_GRAY [6]={0.9,0.9,0.9,1.0,1.0,1.0};
static const double DUMP_BLUE [6]={0.0,0.0,0.8,0.0,0.0,1.0};
static const double DUMP_YELLOW[6]={0.8,0.4,0.0  ,1.0,1.0,0.0};
static const double DUMP_MAGENTA[6]={0.8,0.0,0.8  ,1.0,0.0,1.0};
static const double DUMP_WHITE[6] ={0.8,0.8,0.8,1.0,1.0,1.0};
static const double DUMP_BWWHITE[6] ={1.,1.,1.,1.0,1.0,1.0};
static const double DUMP_BWBLACK[6] ={0.,0.,0.,0.0,0.0,0.0};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Dump::Dump(long dumpmode,long color_scheme,long colored,long lines,long columns)
  :pdumpmode(dumpmode),filenr(0),buffer(0),attr(0),x(0),y(0),pcolored(colored)
  ,rimfactor(1.01),pbuffering(1)

{
  //  NSF("init_screendump"); 

  if(dumpmode==Screen::NO_DUMP) return;
  if(!pbuffering) return;

  file="file.ext";

  initDumpBuffer(lines,columns);

  dumpSetColors(color_scheme);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

Dump::~Dump()
{
  pdumpmode=Screen::NO_DUMP;

  delete [] buffer;
  delete [] attr;
}
/* ***********************************************************************
 * activate dump session
 * ***********************************************************************
 */

void Dump::initDumpBuffer(long lines,long columns)
{
  //  NSF("init_dump_buffer"); 


  
  if(pdumpmode==Screen::NO_DUMP) return;
  
  

  if(buffer)  delete [] buffer; buffer=0;
  if(attr)    delete [] attr;   attr=0;
  
  rows=lines;
  cols=columns;
  x=0;
  y=0;

  buffer=new char[(rows*cols)];
  attr  =new long[(rows*cols)];


  for(long i=0;i<rows;i++){
    for(long j=0;j<cols;j++){
      buffer[i+j*rows]=' ';
      attr[i+j*rows]=SS_STYLE_NORMAL;
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Dump::dumpSetColors(long color_scheme)
{
  if(pcolored){

    if (color_scheme== Screen::RED_DEVIL){
      dump_fgn =DUMP_WHITE; dump_bgn =DUMP_RED;
      dump_fgmh=DUMP_YELLOW; dump_bgmh=DUMP_RED;
      dump_fges=DUMP_YELLOW; dump_bges=DUMP_RED;
      dump_fgeh=DUMP_GREEN ; dump_bgeh=DUMP_RED;
      dump_fgehi=DUMP_MAGENTA ; dump_bgehi=DUMP_RED;
    }else if (color_scheme==       Screen::GREEN_LEAVES){
      
      dump_fgn =DUMP_BLACK ; dump_bgn =DUMP_GREEN;
      dump_fgmh=DUMP_RED   ; dump_bgmh=DUMP_GREEN;
      dump_fges=DUMP_RED   ; dump_bges=DUMP_GREEN;
      dump_fgeh=DUMP_YELLOW; dump_bgeh=DUMP_GREEN;
      dump_fgehi=DUMP_MAGENTA ; dump_bgehi=DUMP_GREEN;
    }else if (color_scheme==       Screen::SILVER_MINE){
      
      dump_fgn =DUMP_BLACK; dump_bgn =DUMP_GRAY;
      dump_fgmh=DUMP_RED  ; dump_bgmh=DUMP_GRAY;
      dump_fges=DUMP_RED  ; dump_bges=DUMP_GRAY;
      dump_fgeh=DUMP_BLUE ; dump_bgeh=DUMP_GRAY;
      dump_fgehi=DUMP_MAGENTA ; dump_bgehi=DUMP_GRAY;
      
    }else if (color_scheme==       Screen::HOT_DOG){
      
      dump_fgn =DUMP_BLACK; dump_bgn =DUMP_YELLOW;
      dump_fgmh=DUMP_RED  ; dump_bgmh=DUMP_YELLOW;
      dump_fges=DUMP_RED  ; dump_bges=DUMP_YELLOW;
      dump_fgeh=DUMP_WHITE; dump_bgeh=DUMP_YELLOW;
      dump_fgehi=DUMP_MAGENTA ; dump_bgehi=DUMP_YELLOW;
      
    }else if (color_scheme==       Screen::CLASSIC_BLACK||
	      1 ){
      
      dump_fgn =DUMP_WHITE; dump_bgn =DUMP_BLACK;
      dump_fgmh=DUMP_RED  ; dump_bgmh=DUMP_BLACK;
      dump_fges=DUMP_RED  ; dump_bges=DUMP_BLACK;
      dump_fgeh=DUMP_GREEN; dump_bgeh=DUMP_BLACK;
      dump_fgehi=DUMP_MAGENTA ; dump_bgehi=DUMP_BLACK;
      
    }
  
  }else{
    dump_fgn =DUMP_BWBLACK; dump_bgn =DUMP_BWWHITE;
    dump_fgmh =DUMP_BWBLACK; dump_bgmh =DUMP_BWWHITE;
    dump_fges =DUMP_BWBLACK; dump_bges =DUMP_BWWHITE;
    dump_fgeh =DUMP_BWBLACK; dump_bgeh =DUMP_BWWHITE;
    dump_fgehi =DUMP_BWWHITE; dump_bgehi =DUMP_BWBLACK;
    
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dump::screendumpOn()const 
{ 
  return (pdumpmode != Screen::NO_DUMP);
}


/* ***********************************************************************
 * setup file name
 * ***********************************************************************
 */
void Dump::setDumpFile()
{

  /* NSF("set_dump_file");  */

  if(pdumpmode==Screen::NO_DUMP) return;

  
  long n=-1;

  for(long i=filenr;i<9999;i++){
    if(pdumpmode==Screen::ASCII_DUMP){
      file.sprintf("dump%04ld.txt",i);
    }else{
      file.sprintf("dump%04ld.ps",i);
    }
    if(pathExistence(file)==DoesNotExist){
      n=i;
      break;
    }
  }
  if(n<0){
    throw Except(" cannot create a dump file \'dumpxxxx.[ps|txt]\', since"
		 " all numbers are used in the current directory!\n");
  }
  



}

/* ***********************************************************************
 * return current dump file name
 * ***********************************************************************
 */
String Dump::lastScreendumpFile()const
{

  if(pdumpmode==Screen::NO_DUMP) return "file.ext";

  if(!file.empty())
    return file;
  else
    return "file.ext";
}

/* ***********************************************************************
 * output of ps-header for postscript screen dump
 * ***********************************************************************
 */
void Dump::dumpPSHeader(FILE *FH,double scale,double pheight)const
{
  
  fprintf(FH,"\
%%!PS-Adobe-2.0  EPSF-1.2\n\
%%%%Creator: fedit\n\
%%%%Title: screenshot\n\
%%%%Pages: 1\n\
%%%%PageOrder: Ascend\n\
%%%%BoundingBox: 0 %g 595 %g\n\
%%%%%%DocumentPaperSizes: A4\n\
%%%%%%Orientation: Portrait\n\
%%%%DocumentFonts: Courier Courier-Bold\n\
%%%%%%\n\
%%%%%%  1000 pt = 23cm/29.5cm\n\
%%%%%%\n\
%%%%%%\n\
%%%%EndComments\n\
%%%%BeginProlog\n\
%%%%EndProlog\n\
/N {def} bind def\n\
/B {bind def} bind N\n\
/i {index} B\n\
/m {moveto} B\n\
/rm {rmoveto} B\n\
/l {lineto} B\n\
/rl {rlineto} B\n",(842.-pheight*rimfactor)*rimfactor,842.*rimfactor);
  fprintf(FH,"\
/n {newpath} B\n\
/c {closepath} B\n\
/v {gsave} B\n\
/r {grestore} B\n\
/s {stroke} B\n\
/g {setgray} B\n\
/q {setrgbcolor} B\n\
/f {fill} B\n\
/x {exch} B\n\
/w {setlinewidth} B\n\
/RJ {\n\
 stringwidth neg x neg x\n\
 rm\n\
} B\n\
/CS {\n\
stringwidth\n\
2 div neg x 2 div neg x\n\
rm\n\
} B\n\
/max { 2 copy ge{pop}{x pop}ifelse} B\n\
/min { 2 copy ge{x pop}{ pop}ifelse} B\n\
/SL {\n\
  { x stringwidth pop  max  }repeat\n\
} B\n\
1   setlinecap\n\
\n\
	mark             \n");
  fprintf(FH,"\
 	/ISOLatin1Encoding \n\
	  8#000 1 8#054 {StandardEncoding exch get} for \n\
	  /minus \n\
	  8#056 1 8#217 {StandardEncoding exch get} for \n\
	  /dotlessi \n\
	  8#301 1 8#317 {StandardEncoding exch get} for \n\
	  /space /exclamdown /cent /sterling /currency /yen /brokenbar /section \n\
	  /dieresis /copyright /ordfeminine /guillemotleft /logicalnot /hyphen \n\
	  /registered /macron /degree /plusminus /twosuperior /threesuperior /acute \n");
  fprintf(FH,"\
	  /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine \n\
	  /guillemotright /onequarter /onehalf /threequarters /questiondown /Agrave \n\
	  /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla /Egrave /Eacute\n\
	  /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n\
	  /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave \n\
	  /Uacute /Ucircumflex /Udieresis /Yacute /Thorn /germandbls /agrave /aacute \n");
  fprintf(FH,"\
	  /acircumflex /atilde /adieresis /aring /ae /ccedilla /egrave /eacute \n\
	  /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis /eth /ntilde\n\
	  /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave \n\
	  /uacute /ucircumflex /udieresis /yacute /thorn /ydieresis \n\
	  /ISOLatin1Encoding where not {256 array astore def} if \n\
	 cleartomark \n\
 \n\
	/makeISOEncoded \n\
	{ findfont /curfont exch def \n\
	  /newfont curfont maxlength dict def  \n");
  fprintf(FH,"\
	  /ISOLatin1 (-ISOLatin1) def\n\
	  /curfontname curfont /FontName get dup length string cvs def \n\
	  /newfontname curfontname length ISOLatin1 length add string \n\
    	 dup 0                  curfontname putinterval \n\
    	 dup curfontname length ISOLatin1   putinterval \n\
	  def \n\
	  curfont   \n\
	  { exch dup /FID ne  \n\
    	{ dup /Encoding eq  \n\
    	  { exch pop ISOLatin1Encoding exch }  \n\
    	  if  \n\
    	  dup /FontName eq  \n\
    	  { exch pop newfontname exch }  \n");
  fprintf(FH,"\
    	  if  \n\
    	  exch newfont 3 1 roll put  \n\
    	}  \n\
     	{ pop pop }  \n\
    	ifelse  \n\
	  }  \n\
	  forall \n\
	  newfontname newfont definefont \n\
	} def \n\
 \n\
	/Times-Roman makeISOEncoded pop \n\
	/Times-Bold makeISOEncoded pop \n\
	/Times-Italic makeISOEncoded pop \n\
	/Times-BoldItalic makeISOEncoded pop \n\
	/Helvetica makeISOEncoded pop \n\
	/Helvetica-Bold makeISOEncoded pop \n\
	/Helvetica-Oblique makeISOEncoded pop \n\
	/Helvetica-BoldOblique makeISOEncoded pop \n");
  fprintf(FH,"\
\n\
	/Courier   makeISOEncoded pop \n\
	/Courier-Bold makeISOEncoded pop \n\
\n\
\n\
/rectxywh {4 2 roll n m 1 i 0 rl 0 x rl neg 0 rl c s}B\n\
/fillrectxywh {4 2 roll n m 1 i 0 rl 0 x rl neg 0 rl c v q f r {s} {} ifelse}B\n\
/rect {2 i sub x 3 i sub x rectxywh}B\n\
/fillrect {2 i sub x 3 i sub x fillrectxywh}B\n\
");
  fprintf(FH,"\
/FFS { x findfont x scalefont }B\n\
/FS  { FFS setfont } B\n\
/textbox { v currentpoint translate 0 0 1 q dup stringwidth pop 0 descent neg 3 2\n\
 roll ascent descent add  rectxywh r }B\n");
  fprintf(FH,"\
/fontmetrics {\n\
  currentfont dup /FontBBox get aload pop\n\
  %% stack:  currentfont xll yll xur yur TOS\n\
  4 i /FontMatrix get transform \n\
  %% stack  currentfont T*xll T*yll xur yur TOS\n\
  /ascent x N %% store T*yur\n\
  /rscent x N %% store T*xur \n\
  %%stack:  currentfont xll yll TOS\n\
  3 2 roll\n\
  %% stack:  xll yll   currentfont TOS\n\
  /FontMatrix get transform\n\
  %% stack:  T*xll T*yll TOS\n\
  neg /descent x N  %% store T*yll\n\
  /lscent x N  %% store T*xll \n");
  fprintf(FH,"\
  %% stack: empty\n\
  %% modify rscent according to stringwidth for Courier fonts\n\
  /lscent 0 N\n\
  /rscent (W) stringwidth pop N\n\
  /BBheight ascent descent add N\n\
\n\
} B\n\
");
  fprintf(FH,"\
/drawTextBBox{\n\
  %% usage: true/false color string x0 y0 drawTextBBox\n\
  %%   color is a triple of numbers\n\
  %%   true for rim, false fpr no rim\n\
%%(drawTextBBox) =\n\
%%stack\n\
  v  translate 0 descent neg 3 2 roll stringwidth pop ascent  fillrect r\n\
} N\n\
/charOut{\n\
  %% usage: underline_flag fg bg string*1 charOut  \n\
  %%   prints a box with background color bg and the character with foreground\n\
  %%   color fg. The colors fg and bg are both a triple of numbers.\n");
  fprintf(FH,"\
  %%   Uses currentpoint and advances it horizontaly so next call to charOut \n\
  %%   will print right to the position of the previous call, such\n\
  %%   that the boxes touch\n\
  \n\
  7 1 roll 2 i 2 i 2 i q false 4 1 roll 7 i  \n\
  currentpoint  drawTextBBox q  x\n\
  %% make underline\n\
  {v dup \n\
    FontSize 30 div w 0 setlinecap  \n\
    currentpoint FontSize 10 div sub m \n\
    stringwidth pop 0 rl s r}if\n\
  true {\n\
    %% output string and let show advance to the right\n");
  fprintf(FH,"\
    currentpoint m show \n\
  }{\n\
    %% this is output with  explicit advancing to the right\n\
    dup v currentpoint m show r\n\
    currentpoint x 3 2 roll stringwidth pop  add x m\n\
  }ifelse\n\
  \n\
} N\n\
\n\
/Newline{\n\
  %%usage /x0 /y0 Newline\n\
  %%  will increase y0 by BBheight and move to x0 y0, which\n\
  %%  simulates pressing enter an a terminal\n\
  dup dup  load BBheight sub  N x load x load m\n\
}N\n\
/C { %% usage: (some_string) O\n");
  fprintf(FH,"\
  Underline RFG aload pop\n\
  Bold{ 6 3 roll pop pop pop }{pop pop pop }ifelse\n\
  RBG aload pop pop pop pop \n\
 8 7 roll charOut} N\n\
/Reverse false N \n\
/Underline false N \n\
/Bold false N \n\
/RFG { Reverse { BG} {FG} ifelse} N\n\
/RBG { Reverse { FG} {BG} ifelse} N\n");
  fprintf(FH,"\
/BON {/Bold true N /Courier-Bold-ISOLatin1 FontSize FS}N\n\
/BOFF { /Bold false N  /Courier-ISOLatin1 FontSize FS }N\n\
/RON  { /Reverse true  N }N\n\
/ROFF { /Reverse false N }N\n\
/UON  { /Underline true  N }N\n\
/UOFF  { /Underline false  N }N\n\
/CLN  { /FG FGN  N /BG BGN  N }N\n\
/CLMH  { /FG FGMH  N /BG BGMH  N }N\n\
/CLES  { /FG FGES  N /BG BGES  N }N\n\
/CLEH  { /FG FGEH  N /BG BGEH  N }N\n\
/CLEHI  { /FG FGEHI  N /BG BGEHI  N }N\n\
%%\n");
  fprintf(FH,"\
/FGN [%g %g %g %g %g %g] N\n\
/BGN [%g %g %g %g %g %g] N\n\
/FGMH[%g %g %g %g %g %g] N\n\
/BGMH[%g %g %g %g %g %g] N\n\
/FGES[%g %g %g %g %g %g] N\n\
/BGES[%g %g %g %g %g %g] N\n\
/FGEH[%g %g %g %g %g %g] N\n\
/BGEH[%g %g %g %g %g %g] N\n\
/FGEHI[%g %g %g %g %g %g] N\n\
/BGEHI[%g %g %g %g %g %g] N\n\
/FG FGN N\n\
/BG BGN N\n\
%%\n\
%%\n\
%%\n\
%%\n\
%%%%Page: 1 1\n\
%%\n\
%%\n\
%%\n\
%%\n\
v\n\
/Scale %g N \n\
Scale dup scale\n\
0.000000 %g Scale div translate\n\
0  rotate\n\
%%0.000000 0.000000 translate\n\
%%90  rotate\n\
"
		
		,dump_fgn[0],dump_fgn[1],dump_fgn[2]
		,dump_fgn[3],dump_fgn[4],dump_fgn[5]
		,dump_bgn[0],dump_bgn[1],dump_bgn[2]
		,dump_bgn[3],dump_bgn[4],dump_bgn[5]
		
		,dump_fgmh[0],dump_fgmh[1],dump_fgmh[2]
		,dump_fgmh[3],dump_fgmh[4],dump_fgmh[5]
		,dump_bgmh[0],dump_bgmh[1],dump_bgmh[2]
		,dump_bgmh[3],dump_bgmh[4],dump_bgmh[5]
		
		,dump_fges[0],dump_fges[1],dump_fges[2]
		,dump_fges[3],dump_fges[4],dump_fges[5]
		,dump_bges[0],dump_bges[1],dump_bges[2]
		,dump_bges[3],dump_bges[4],dump_bges[5]
		
		,dump_fgeh[0],dump_fgeh[1],dump_fgeh[2]
		,dump_fgeh[3],dump_fgeh[4],dump_fgeh[5]
		,dump_bgeh[0],dump_bgeh[1],dump_bgeh[2]
		,dump_bgeh[3],dump_bgeh[4],dump_bgeh[5]

		,dump_fgehi[0],dump_fgehi[1],dump_fgehi[2]
		,dump_fgehi[3],dump_fgehi[4],dump_fgehi[5]
		,dump_bgehi[0],dump_bgehi[1],dump_bgehi[2]
		,dump_bgehi[3],dump_bgehi[4],dump_bgehi[5]
		,scale,842*rimfactor);
}


/* ***********************************************************************
 * output of ps-footer for postscript screen dump
 * ***********************************************************************
 */

void Dump::dumpPSFooter(FILE *FH)const
{

  fprintf(FH,"\n\
r\n\
showpage\n\
%%%%Trailer\n\
%%%%DocumentFonts: \n\
%%%%EOF\n");
}


/* ***********************************************************************
 * escape ps special characters
 * ***********************************************************************
 */
String Dump::escapePSChar(char ch)const
{
  String p;
  
  if(ch=='(' || ch==')' || ch=='\\'){
    p+='\\';
    p+=ch;
  }else{
    p+=ch;
  }
  return p;
}

/* ***********************************************************************
 * deactivate dump session
 * ***********************************************************************
 */
void Dump::dumpScreen()
{
  /* NSF("dump_screen");  */

  if(pdumpmode==Screen::NO_DUMP) return;

  setDumpFile(); 



  String sfile=lastScreendumpFile();

  FILE *FH=openforwrite(sfile);




  long fontsize=20L;
  double sc=999./((cols+1)*fontsize);


  double width =595./(100. *10)*fontsize*(cols+1);
  double height=842./(100. *8)*fontsize*(rows+1);
  double pheight=height*sc;

  if(pdumpmode == Screen::ASCII_DUMP ){
    for(long i=0;i<rows;i++){
      for(long j=0;j<cols;j++){
	fprintf(FH,"%c",buffer[i+j*rows]);
      }
      fprintf(FH,"\n"); 
    }
    
  }else{
    dumpPSHeader(FH,sc,pheight);

    fprintf(FH,"\
%%\n\
%% set font size \n\
/FontSize %ld N\n\
%%\n\
%%\n\
%% load initial font \n\
/Courier-ISOLatin1 FontSize FS fontmetrics\n\
%%\n\
%%\n\
%% setof from bounding box\n\
/x0 FontSize 0.2 mul N\n\
/y0 ascent  FontSize 0.2 mul add neg N\n\
/yu y0 N\n\
%%\n\
%% goto beginnig\n\
x0 y0 m\n\
%%\n\
%%\n\
%%\n\
",fontsize);

  
    long at,at0;

#define MASK_ATTR(at)							\
    if(at & A_STANDOUT) at =((at & (~A_STANDOUT)) | (A_REVERSE));	\
    at&=(A_UNDERLINE|A_REVERSE|A_BOLD | SS_COLOR_MASK)  

    for(long i=0;i<rows;i++){
      fprintf(FH,"%%row:%ld\n",i);
      long j=0;
      at0=attr[i+j*rows];
      MASK_ATTR(at0);
      if((at0 & A_BOLD)) fprintf(FH," BON ");
      else fprintf(FH," BOFF ");

      if((at0 & A_UNDERLINE)) fprintf(FH," UON ");
      else fprintf(FH," UOFF ");

      if((at0 & A_REVERSE)) fprintf(FH," RON ");
      else fprintf(FH," ROFF ");

      if(pcolored==1){
	if(((at0 & SS_COLOR_MASK) == SS_STYLE_NORMAL))
	  fprintf(FH," CLN "); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_MENUHIGH))
	  fprintf(FH," CLMH "); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_ENTRYSEARCH))
	  fprintf(FH," CLES "); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_ENTRYHIGH))
	  fprintf(FH," CLEH "); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_HINT))
	  fprintf(FH," CLEHI "); 
      }
    
      fprintf(FH,"(");
      for(long j=0;j<cols;j++){
	String buf=escapePSChar(buffer[i+j*rows]);
	at=attr[i+j*rows];
	if(j==y && i==x){
	  at ^=A_REVERSE;
	}
	MASK_ATTR(at);
	if(at == at0){
	  fprintf(FH,"%s",buf());
	}else{
	  fprintf(FH,")C");
	  if((at0 & A_BOLD) && !(at & A_BOLD)) fprintf(FH," BOFF ");
	  else if(!(at0 & A_BOLD) && (at & A_BOLD)) fprintf(FH," BON ");

	  if((at0 & A_REVERSE) && !(at & A_REVERSE)) fprintf(FH," ROFF ");
	  else if(!(at0 & A_REVERSE) && (at & A_REVERSE))
	    fprintf(FH," RON ");

	  if((at0 & A_UNDERLINE) && !(at & A_UNDERLINE)) 
	    fprintf(FH," UOFF ");
	  else if(!(at0 & A_UNDERLINE) && (at & A_UNDERLINE))
	    fprintf(FH," UON ");

	  if(pcolored==1){
	    if( ((at0 & SS_COLOR_MASK)!=(at & SS_COLOR_MASK)) ){
	      if(((at & SS_COLOR_MASK) == SS_STYLE_NORMAL))
		fprintf(FH," CLN "); 
	      else if(((at & SS_COLOR_MASK) == SS_STYLE_MENUHIGH))
		fprintf(FH," CLMH "); 
	      else if(((at & SS_COLOR_MASK) == SS_STYLE_ENTRYSEARCH))
		fprintf(FH," CLES "); 
	      else if(((at & SS_COLOR_MASK) == SS_STYLE_ENTRYHIGH))
		fprintf(FH," CLEH "); 
	      else if(((at & SS_COLOR_MASK) == SS_STYLE_HINT))
		fprintf(FH," CLEHI "); 
	    }
	  }	
	
/* 	  if(screendump.x==i &&  screendump.y == j){ */
/* 	    fprintf(FH," RON "); */
/* 	  } */
	  
	  fprintf(FH,"(%s",buf());
	  
/* 	  if(screendump.x==i &&  screendump.y == j){ */
/* 	    fprintf(FH," RO1N "); */
/* 	  } */
	  

	  at0=at;
	}
      }
      fprintf(FH,")C");
      fprintf(FH,"/x0 /y0 Newline\n"); 
    }

    fprintf(FH,"2 Scale mul w 0 0 0 q "
		  " FontSize 0.2 mul dup neg %g y0 neg ascent sub "
		  " FontSize 0.1000 mul"
		  " add neg rect\n"
		  ,width); 
    dumpPSFooter(FH);
  } /* PS_DUMP */
  fclose(FH);FH=NULL;

  filenr++;



}




/* ***********************************************************************
 * move in dump mode
 * ***********************************************************************
 */
void Dump::dump_move(int xx,int yy)
{

  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;
  
  x=xx;
  y=yy;
    
}

/* ***********************************************************************
 * addstr in dump mode
 * ***********************************************************************
 */
void Dump::dump_addstr(String const &str)
{

  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;
  
  long l=str.length();

  for(long k=0;k<l;k++){
    long i=x;
    long j=y+k;
    if(j<cols){
      buffer[i+j*rows]=str[k];
      attr[i+j*rows]=cattr;
    }
  }
  y+=l;
}

/* ***********************************************************************
 * addch in dump mode
 * ***********************************************************************
 */
void Dump::dump_addch(char  ch)
{

  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;
  
  if(y<cols){
    buffer[x+y*rows]=ch;
    attr[x+y*rows]=cattr;
  }
  y++;
  
  
}

/* ***********************************************************************
 * clear line
 * ***********************************************************************
 */ 
void Dump::dump_clrtoeol()
{
  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;

  if(x<rows){
    for(long j=0;j<cols;j++){
      buffer[x+j*rows]=' ';
      attr[x+j*rows]=SS_STYLE_NORMAL;
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dump::dump_attron(int at)
{
  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;

  int fl=at & (~SS_COLOR_MASK);
  int c =at & (SS_COLOR_MASK);
  int sc=cattr & SS_COLOR_MASK;

  
  if(c!=0 && sc!=c)sc=c;
  
  cattr = ( (cattr & (~SS_COLOR_MASK)) | fl) | sc ;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dump::dump_attroff(int at)
{
  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;
  int fl=at & (~SS_COLOR_MASK);
  //int c =at & (SS_COLOR_MASK);
  

  cattr = (cattr & (~fl)); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dump::dump_attrset(int at)
{
  if(!pbuffering) return;
  if (pdumpmode == Screen::NO_DUMP) return;  

  cattr = at; 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dump::dump_scrollUpDn(long top,long bottom,long n)
{
  // n=1 is key down, means one more line at the bottom seen
  // n=-1 is key up

  long n0,n1;
  if(n==1){
    n0=top;
    n1=bottom;
  }else{
    n0=bottom;
    n1=top;
  }
  
  for(long i=n0;i!=n1;i+=n){
    for(long j=0;j<cols;j++){
      buffer[i+j*rows]=buffer[i+n+j*rows];
      attr[i+j*rows]=attr[i+n+j*rows];
    }
  }
  x=n1;
  dump_clrtoeol();
    
  

}

/*EOF*/ 
