/*************************************************************************
 * @(#)Fedit.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   16 Oct 2008
 *************************************************************************
 */


#include "FString.hh"
#include "Assorted.hh"
#include "Form.hh"
#include "Viewer.hh"
#include "Fedit.hh"
#include "Sysinfo.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fedit::Fedit(int argc,char *argv[])
  :App(argc,argv),expertmode(0),newfeditfile(1),editor("vi")
  ,feditfile("+fedit"),pprogram(""),platform(_PLATFORM_)
  ,tmpdir("+tmp")
{
  //  NSF("Fedit");
  {
    /* ../...version.h contains ...VERSION="..."  which is also used in fortran
       this is why it looks so strange: dont touch it
    */
    const char *  
#include "../mainversion.h" 
      ;
    const char *  
#include "../subversion.h" 
      ;
    const char *  
#include "../release.h"
      ;
    
    mainversion=FPLOMAINVERSION;
    subversion=FPLOSUBVERSION;
    release=FPLORELEASE;
  }  



  scanopts(argc,argv);

  
  if(pprogram.empty())
    pprogram="fplo"+mainversion+"-"+release+platform;
  
  
  fullexecname=Sprintf("%s (%s-%s%s)","fedit",mainversion(),release()
				,platform());
  
  
  
  (void)printf("FEDIT Version %s:%s Release %s Platform %s\n",mainversion()
	       ,subversion(),release(),platform());
  (void)printf("Program       %s\n\n",pprogram());


  //fprintf(stderr,"Welcome to %s in %s\n",appname(),cwdir());



  createDirectory(cwdir+"/"+tmpdir,1);
  

  helpfile=workPath("+fedithelp",IOTmp);

  MainForm *form=new MainForm(this,"MAIN MENU");
  //FileViewer *form=new FileViewer(this,"Main","+tmp/+symminfo");
  form->statusBar()->setRightStatus(Sprintf("(%s-%s:%s)"
						    ,mainversion()
						    ,release()
						    ,subversion()));
  
  
  //  form->startAtBottom();
  setMainWidget(form);
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fedit::~Fedit()
{
  NSF("~Fedit");

  try{
    removeDirectoryNonRecursive(cwdir+"/"+tmpdir,1);
    // remove(feditfile());
    // remove((feditfile+"help")());
  }catch(Except &ex){
    printf("%s\n",ex.msg());
  }
  
  // clean up the parser
  Parser::finalCleanUp();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
 void Fedit::usage()
 {
   (void)printf("\
 Edit the FPLO-package input files via hotkey-\
oriented menus\n\
 (C) Klaus Koepernik March 2002\n\n\
 usage:  %s [-bandplot | -atom ] [-c 0|1] [-ex] [-r] [-a]\n\
                [-e EDITOR] [-p FPLO-PROGRAM] [-ef ENTRYFILE ] [-sd] [-h]\n\
                [-cs color_scheme] [-screendump dump_mode] \n\n"
		,appname());

   App::usage();

   (void)printf("\
     -ex: run in expert mode\n\n\
     -e: in expert mode only: Set EDITOR to your choice\n\
         i.e.  %s -e 'emacs -nw'\n\
         STANDARD is vi\n\n\
     -p: set FPLO-PROGRAM to your name of the fplo/dirac/bandplot-executable.\n"
		,appname());
   (void)printf("\
         If you specify a program of the FPLO package, fedit will manage\n\
         its input. The specified program will overwrite the -atom or \n\
     -bandplot standard settings.\n\n\
     -ef:Set entry definition file to location ENTRYFILE (default: +fedit) \n\
         It is for developement or debuging purposes only.\n\
         This might be dangerous, so don't use it if you can achive the \n\
         desired result by other means.\n\n\
     -bandplot: manage bandplot  (overwritten by -p)\n\n\
     -atom: manage atom calculation (overwritten by -p)\n\n");
 /*     -update: force update of files\n\n\ */

   (void)printf("\
 EXPERTMODE: \n\
    *    Alternative exit, discarding all changes, using hotkey 'x'.\n\
    *    Extern editor available.\n\n\
 REMARKS:\n\
         The succesfull use of this tool depends on your\n\
         terminal type. I tested it for the xterm, aixterm, linux, vt100,\n\
         vt220. IF THERE IS ANY POSSIBILITY TO GET ANY OF THESE TERMS\n\
         , DO IT!\n\
         If you are working at a terminal which you don\'t know, \n");
   (void)printf("\
         or if the terminal type is not supported (Error message),\n\
         try to set and export the $TERM environment variable to\n\
         the terminals xterm|ansi|vt100|vt220|aixterm|dtterm\n\
         (or linux on linux-machines.) \n\
         Then, may be, you can use the options to make it work, may be with \n\
         less optical performance.\n\
 PROBLEMS:\n\
    *    The horizontal lines may look like long strings of one\n\
         arbitrary character:  use switch -a\n");
   (void)printf("\
    *    There are funny characters prefixing every string: Try\n\
         to set one of the above terminals, or contact me, that\n\
         I can try to make it work. At least, try to use -r -a -c 0.\n\
    *    Anyhow, the more switches you set, the less problems are\n\
         expected. Minimal terminal requirements are -r -a -c 0.\n\
    *    With default settings reverse mode is entered while editing.\n\
         On some recent xterms the character under the cursor is invisible.\n\
         I fixed that by putting an explicit cursor color into the\n\
         .Xdefaults file via\n\
              *XTerm.VT100.cursorColor: Black\n\
         Choose the color according to your settings. Note that my\n\
         default cursor color was black already, so it seems that \n\
         setting it explicitely changes some internal behaviour.\n\
 ERRORS:\n\
         If you get errors while using this tool, check the help\n\
         screen of the INPUT-FILE MENU\n\n"
 	 );

}

#define noargfmt "Option %s does not expect arguments\n"
#define oneargfmt "Option %s needs one argument.\n"
#define NO_ARG\
    if(sl.size()!=0)\
      throw Except(Sprintf(noargfmt,opt()));

#define ONE_ARG\
    if(sl.size()!=1)\
      throw Except(Sprintf(oneargfmt,opt()));

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Fedit::onArgument(String const &opt,StringList const &sl)
{

  if(App::onArgument(opt,sl)) return 1;


  
  if(opt=="-ex"){
    expertmode=1; NO_ARG;
  }else if(opt=="-e"){
    editor=sl[0]; ONE_ARG;
  }else if(opt=="-ef"){
    ONE_ARG;
    feditfile=sl[0];
    (void)printf("Using user defined entry definition file '%s'.\n"
		 "The usage of user defined files may lead to inconsistencies!\n"
		 "Its is better to manipulate  FPLO to create the "
		 "desired +fedit!\n\n"
		 ,feditfile());
    newfeditfile=0;
  }else if(opt=="-bandplot"){
    NO_ARG;
    if(pprogram.empty()) {
      pprogram="bandplot"+mainversion+"-"+release+platform;
    }
  }else if(opt=="-atom"){
    NO_ARG;
    if(pprogram.empty()) {
      pprogram="dirac"+mainversion+"-"+release+platform;
    }
  }else if(opt=="-p"){
    ONE_ARG;
    pprogram=sl[0];
    
//   }else if( strcmp(argv[i],"-update")==0){ 
//    /* not used since communication is hard to 
//       achieve at present */
//     force_update=0;		 
  }else{
    usage();
    throw 
      Except(Sprintf("Option %s %s not known.\n\n"
			      ,opt(),sl.join(" ")()));
  }

  return 1;
}

#undef NO_ARG
#undef ONE_ARG
#undef noargfmt
#undef oneargfmt

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::loadFeditParseTable()
{
  NSF("loadFeditParseTable");

  /* look for FEDITFILE  */
  if(pathExistence(feditfile)!=RegularFile){
    if(newfeditfile)
      throw 
	Except(String()
	       .sprintf(
			"error(%s): Cannot find entry definition file '%s'!\n"
			" It should be in the local tmp directory '%s'!\n"
			" One can overwrite the location using option -ef"
			" filename.\n\n"
			"\n Possible reasons:\n"
			"        1) the executable '%s' is not running"
			" correctly\n"
			"           or just does not exist!\n"
			"        2) the directory or some files are write"
			" protected\n"
			"        3) the executable '%s' is not in the PATH.\n"
			,nsubfunc,feditfile(),tmpdir(),pprogram(),pprogram()));
    else
      throw 
	Except(String()
	       .sprintf(
			"error(%s): Cannot find entry definition file '%s'!\n"
			,nsubfunc,feditfile()));
    
    
  }

  VersionStatus vs;
  String  msg;

  try{
    fedit_pc.parseFile(feditfile);
    vs=VersionCheck(fedit_pc,feditfile,msg);
  }catch (Except &ex){
    throw Except(Sprintf("error(%s,file:'%s'):\n%s\n"
				  ,nsubfunc,feditfile(),ex.msg()));
  }
  
  
  if(vs != VCheckOK) 
    throw Except(Sprintf("error(%s,file:'%s'):\n%s"
				  "may be the version of %s differs!\n"
				  ,nsubfunc,feditfile(),msg(),pprogram()));
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fedit::VersionStatus Fedit::VersionCheck(Parser &par,String const &fname
					 ,String &mesg)
{
  //NSF("VersionCheck");

  /* get  versions */
  String vm,vs;
  try{
    vm=par()("version")("mainversion")();
    vs=par()("version")("subversion")();
  }catch(Except &ex){
    throw Except(Sprintf("Cannot find correct version data in "
				  "file '%s':\n%s!",fname(),ex.msg()));
  }
  
  /* test them */
  if( mainversion != vm || subversion != vs ){
  
    mesg.sprintf("Version of file '%s' (%s:%s) differs from fedit "
		 "version (%s:%s)!\n",fname(),vm(),vs()
		 ,mainversion(),subversion());
    
    return VCheckWrongVersion;
  }
  return VCheckOK;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::setupParseTables()
{
  NSF("setupParseTables");
  
  if(parsetables.readIn())
    throw Except(Sprintf("error(%s): internal error: function"
				  " must not be called twice without"
				  " reseting parsetables!\n"
				  ,nsubfunc));

  loadFeditParseTable();

  Dobj d=fedit_pc()("screens");

  StringList sl;
  String fname;
  /* get filenames */
  for(long i=0;i<d.size();i++){
    fname=d[i]("filename")();
    if(sl.find(fname)<0) sl.append(fname);
  }
  
  parsetables.setup(sl);
  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Fedit::runProgram(String const &program,IOWorkMode ioflag,long report
			 ,long &exitcode)
{
  //NSF("runProgram");     
  
  exitcode=1;
  /* temporary file names */
  
  String efile="tmpfedit";
  String cfile="tmpfedit";
  {
    FILE *FH=opentempfile(cfile,"w");
    fclose(FH);
    FH=opentempfile(efile,"w");
    fclose(FH);
  }

  String prog=program;
  
  
  if(ioflag == IOTmp){
    if(program[0] =='.')
      prog="../"+program; 
    
    String dir=cwdir+"/"+tmpdir; 
    chDir(dir()); 
  }
  
  if(s) s->shellOut();
  systemCall((prog+" 2>"+efile+" |tee "+cfile)());
  if(s)s->shellIn();
  
  if(ioflag == IOTmp)
    chDir(cwdir);
  

  String txt;
  
  if(report){
    
    
    
    txt+="-----STDOUT:\n";
    FILE *FHi=openforread(cfile);
    
    String buf;
    while(getLine(FHi,buf)){
      if(buf.subString(0,12)=="TERMINATION: "){
	if(buf.subString(13,18)!="Normal"){
	  exitcode=0;
	}
      }
      txt+=buf;
    }
    
    (void)fclose(FHi);
    
    txt+="-----STDERR:\n";
    FHi=openforread(efile);
    
    
    while(getLine(FHi,buf))
      txt+=buf;
    
    (void)fclose(FHi);
    
    
  }
  

  remove(efile());
  remove(cfile());

  return txt;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Fedit::workPath(String const &path,IOWorkMode ioflag,long inclcwd)const
{
  String cwd="";
  if(inclcwd) cwd=cwdir+"/";

  switch(ioflag){
  case IOTmp:
    return cwd+tmpdir+"/"+path;
  case IOBak:
    return cwd+path+".bak";
  case IOOriginal:
    return cwd+path;
  }						
  return path;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::setFileStatus(Parser &parser,UpdateStatus status,String const &flag)
{
  NSF("Fedit::setFileStatus");

  
  Dobj ds=parser()("status");
  
  switch (status)
    {
    case UpdateStatusReady:
      ds="ready";
      break;
    case UpdateStatusChanged:
      if(flag.empty())
	ds="symmetry changed";
      else
	ds=Sprintf("trigger update %s",flag());
      break;
    default:
      throw Except(String().sprintf("error(%s): unknown file status"
				    " flag '%d'!\n",nsubfunc,status));
    }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Fedit::readFiles(IOWorkMode ioflag,long stoponvcheck)
{
  NSF("readFiles");

  for(long i=0;i<parsetables.size();i++){
    
    String fname=
      workPath(parsetables.fileName(i),ioflag,0);
  
    String title=String()
      .sprintf("File read '%s': Error Report",fname());
    
    fname=workPath(parsetables.fileName(i),ioflag);
    

    if(ioflag==IOOriginal){
      parsetables.setModificationDate(i,modificationDate(fname));
    }

    
    VersionStatus vs;
    
    String msg;
    try{
      parsetables.parser(i).parseFile(fname);
      vs=VersionCheck(parsetables.parser(i),fname,msg);
    }catch(Except &ex){
      throw Except(ex.msg(),title,0);
    }
    
    switch(vs)
      {
      case VCheckOK:
	break;
      case VCheckWrongVersion:
	if(stoponvcheck)
	  throw Except(msg,title,0);
	else
	  return 0;
	break;
      }
    

    /* test write of vartable, to catch undefined variables, its a hack */
    String sfile;
    FILE *FH=NULL;
    
    try{
      sfile="tmpfedit";
      FH=opentempfile(sfile,"w");
    }catch(Except &ex){ throw Except(ex.msg(),"Error Report",1); }
    
    
    try{
      parsetables.parser(i).writeFile(FH);
    }catch (Except &ex){
      if(FH!=NULL) fclose(FH);FH=NULL; 
      removeFile(sfile);
      throw Except(Sprintf("error(%s): in file '%s': %s\n"
				    ,nsubfunc,fname(),ex.msg())
		   ,"Error Report",0L);
    }
    
    (void)fclose(FH);FH=NULL;
    removeFile(sfile);
    

    
  }

  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::writeFiles(Form *form,IOWorkMode ioflag)
{ 
  NSF("writeFiles");

  String fname;
  String fullfname;
  String sfile;
  FILE *FH;
  
  for(long i=0;i<parsetables.size();i++){
    fname=parsetables.fileName(i);


    try{
      sfile="tmpfedit";
      FH=opentempfile(sfile,"w");
    }catch(Except &ex){ throw Except(ex.msg(),"Error Report"); }
    
    
    try{
      parsetables.parser(i).writeFile(FH);
    }catch (Except &ex){
      fclose(FH);
      removeFile(sfile);
      throw Except(Sprintf("error(%s): in file '%s': %s\n",nsubfunc,fname(),ex.msg())
		   ,"Error Report");
    }
    fclose(FH);
    removeFile(sfile);
    
    
    fullfname=workPath(fname,ioflag);
    
    
    long write=1;
    if(ioflag==IOOriginal){
      
      if(parsetables.modificationDate(i)<modificationDate(fullfname)){
	if(s->piper()){
	  s->piper()
	    ->message(Sprintf("File %s changed on disk! Not saved !!\n"
				   ,fullfname()));
	  throw TerminateExcept("Abort!");
	}else{
	  if(form->setStatusBar(Sprintf("File %s changed on disk, really overwrite"
					,fname())
				,StatusBar::StatusChoose)!=Accept)
	    write=0;
	}
	
      }
    }

    if(write){
      try{
	FH=openforwrite(fullfname);
      }catch(Except &ex){ throw Except(ex.msg(),"Error Report"); }
      
      
      try{
	Dobj d=parsetables.parser(i)()("last_changes");
	
	d("hostname")=Sysinfo::getHostname();
	d("date")=Sysinfo::getDateAndTime();
	d("by")=fullexecname;
	
	parsetables.parser(i).writeFile(FH);
      }catch (Except &ex){
	fclose(FH);
	backUp(fullfname+".bak",fullfname);
	throw Except(ex.msg(),"Error Report");
      }
      fclose(FH);
    }
    
  }
  form->setStatusBar("files written");
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::readScreenDefs()
{
    NSF("Fedit::readScreenDefs()");


  /* read fedit screen definitions */
  try{

    String at;
    String actiontype;
    String ID;
    Dobj d;
    StringList ids;
    StringList fnms;
    StringList acts;
  


    loadFeditParseTable();
    
    screens.clear();
    

    /* get definitions */
    Dobj ds=fedit_pc()("screens");
    
    /* for all screen definitions */
    for(long i=0;i<ds.size();i++){
     
 
      /* get name of screen */
      Dobj dsi=ds[i];
      
      ScreenDef *sde=screens.append(new ScreenDef());
      sde->setScreenName(dsi("screenname")().escapeDecode());

      String fname=dsi("filename")().escapeDecode();

      long pct=parsetables.findTable(fname);
      
      sde->setPtabIndex(pct);
      

      // get parser root of the screens data file
      Dobj dr=parsetables.parser(pct)();

      
      /* ******************************************************************
       * get menu entries 
       * ******************************************************************
       */
      Dobj da=dsi("menus");
      
      /* for all entries */
      for(long im=0;im<da.size();im++){
	
	sde->append(new MenuDef(
				da[im]("action")().escapeDecode()
				,da[im]("menubarentry")().escapeDecode()
				,da[im]("actionname")().escapeDecode()
				));
	
      } /* all menus*/
      
      /* ******************************************************************
       * get screen entries 
       * ******************************************************************
       */
      
      Dobj de=dsi("entries");
      
      /* for all items */
      for(long ie=0;ie<de.size();ie++){
	
	
	Dobj dei=de[ie];
	
	
 	/* get action type */
	StringList flags;
	at=BaseItemDef::actionType(dei("action")().escapeDecode(),flags);
	

	String ID=dei("variablename")().escapeDecode();

	long f=at.find("box");
	long box=(f==at.length()-3);
	actiontype=at.shorten(f);
	
	if(actiontype == "title")
	  sde->append(new TitleItemDef(dei,flags));

	else if(actiontype == "dashline")
	  sde->append(new DashLineItemDef(dei,flags));

	else if(actiontype == "emptyline" || 
		(actiontype.empty() && flags.contains("eol") ))
	  sde->append(new TitleItemDef(StringList("eol")));

	else if(actiontype=="item" || 
		actiontype == "itemlist"){
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(new SingleItemDef(dei,flags,dr));

	}else if(actiontype == "check" ){
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(new CheckGroupItemDef(dei,flags,dr,box));

	}else if(actiontype == "select"){
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(new SelectGroupItemDef(dei,flags,dr,box));
	  
	}else if(actiontype == "mselect" ){
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(new MSelectGroupItemDef(dei,flags,dr,box));

	}else if(actiontype == "field"){
	  FieldItemDef *fid=new FieldItemDef(dei,flags,dr);
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(fid);
	  readFieldScreenDefs(fid,de,ie,fname);
	}else if(actiontype=="compound"){
	  CompoundItemDef *cid=new CompoundItemDef(dei,flags,dr);
	  ids.append(ID); fnms.append(fname); acts.append(at);
	  sde->append(cid);
	  readCompoundScreenDefs(cid,de,ie,fname);
	}else if(actiontype=="counter") 
	  throw Except(Sprintf("error(%s): action %s: cannot have "
			       "%ss outside FIELDs or COMPOUNDs!\n"
			       ,nsubfunc,actiontype.upperCase()()
			       ,actiontype.upperCase()()));
	
	else if(actiontype=="endcompound" || actiontype == "endfield")
	  throw Except(Sprintf("in %s: action %s is not properly nested,"
			       " to many %ss.\n"
			       ,nsubfunc,actiontype.upperCase()()+3
			       ,actiontype.upperCase()()));
	else
	  throw Except(Sprintf("in %s: programmer warning: unknown action '%s'"
			       ,nsubfunc,actiontype()));
	
	
      } /* all items*/
      
      
    } /* all screens */
    
    //check the consistency
    String txt;
    for(long i=0;i<ids.size();i++){
      for(long j=i+1;j<ids.size();j++){
	if( (ids[i]==ids[j] ) && (fnms[i]==fnms[j] ) ){
	  txt+=Sprintf(" in file '%s': %s %s:%s  and "
			"%s %s:%s refer to the same variable\n"
		       ,feditfile()
		       ,acts[i].upperCase()(),fnms[i](),ids[i]()
		       ,acts[j].upperCase()(),fnms[j](),ids[j]());
	}
      }
    }  
    if(!txt.empty())
      throw Except(txt);
    
  }catch (Except &ex){
    throw Except(ex.msg(),Sprintf("File read '%s': Error Report",
				     feditfile()));
  }
  
  
  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::readFieldScreenDefs(FieldItemDef *fid,Dobj &de,long &ie0
				,String const &fname)
{
  NSF("Fedit::readFieldScreenDefs()");
  String actiontype;
  StringList mtab;
  StringList ids;
  StringList acts;
  BaseItemDef *b;

  Dobj dr=fid->data().sapTemplate();
  
  /* count struct array members and initialize members */
  Dobj dd=dr.first();
  while(dd.validNode()){mtab.append(dd.nodeName());dd=dd.next();}


  long firstitem=1;
  for(long ie=ie0+1;ie<de.size();ie++){

    Dobj dei=de[ie];
    /* get action type */
    StringList flags;
    actiontype=BaseItemDef::actionType(dei("action")().escapeDecode(),flags);



    if(actiontype == "endfield"){
      ie0=ie;
      if(mtab.size())
	throw Except(Sprintf("structarray '%s' contains more members\n in"
			     " file '%s' than"
			     " defined in FIELD in file '%s' \n"
			     " A FIELD edit structure must contain"
			     " all structarray members!\n missing: %s\n"
			     ,fid->data().nodeName()()
			     ,fname(),feditfile(),mtab.join(" ")()));
      //check the consistency
      String txt;
      for(long i=0;i<ids.size();i++){
	for(long j=i+1;j<ids.size();j++){
	  if( (ids[i]==ids[j] ) ){
	    txt+=Sprintf(" in file '%s', FIELD '%s': %s '%s'  and "
			 "%s '%s' refer to the same variable\n"
			 ,feditfile(),fid->data().nodeName()()
			 ,acts[i].upperCase()(),ids[i]()
			 ,acts[j].upperCase()(),ids[j]());
	  }
	}
      }  
      if(!txt.empty()) throw Except(txt);
      
      return;
    }else if(actiontype == "counter"){
      fid->appendToHeader(new CounterItemDef(dei,flags,fid->data()));
    }else if(actiontype == "dashline"){
      fid->appendToHeader(new DashLineItemDef(dei,flags));
    }else if(actiontype == "title"){
      fid->appendToHeader(new TitleItemDef(dei,flags));
    }else if(actiontype == "item" || actiontype=="itemlist"){
      fid->append(b=new SingleItemDef(dei,flags,dr,firstitem));
      firstitem=0;
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
    }else
      throw Except(Sprintf("in %s: action %s: FIELDs cannot contain %ss\n"
			   ,nsubfunc,actiontype.upperCase()()
			   ,actiontype.upperCase()()));
    

  }

  actiontype="field";
  throw Except(Sprintf("in %s: action %s is not properly nested,"
		       " missing END%s.\n"
		       ,nsubfunc,actiontype.upperCase()()
		       ,actiontype.upperCase()()));


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fedit::readCompoundScreenDefs(CompoundItemDef *cid,Dobj &de,long &ie0
				   ,String const &fname)
{
  NSF("Fedit::readCompoundScreenDefs()");
  String actiontype;
  String at;
  StringList mtab;
  StringList ids;
  StringList acts;
  BaseItemDef *b;
  
  Dobj dr=cid->data().sapTemplate();

  /* count struct array members and initialize members */
  Dobj dd=dr.first();
  while(dd.validNode()){mtab.append(dd.nodeName());dd=dd.next();}
  
  long body=0;
  for(long ie=ie0+1;ie<de.size();ie++){
    
    Dobj dei=de[ie];


    /* get action type */
    StringList flags;
    at=BaseItemDef::actionType(dei("action")().escapeDecode(),flags);
    long f=at.find("box");
    long box=(f==at.length()-3);
    actiontype=at.shorten(f);

    if(actiontype == "endcompound"){
      if(mtab.size())
	throw Except(Sprintf("structarray '%s' contains more members\n in"
			     " file '%s' than"
			     " defined in COMPOUND in file '%s' \n"
			     " A COMPOUND edit structure must contain"
			     " all structarray members!\n missing: %s\n"
			     ,cid->data().nodeName()() 
			     ,fname(),feditfile(),mtab.join(" ")()));
      //check the consistency
      String txt;
      for(long i=0;i<ids.size();i++){
	for(long j=i+1;j<ids.size();j++){
	  if( (ids[i]==ids[j] ) ){
	    txt+=Sprintf(" in file '%s', COMPOUND '%s': %s '%s'  and "
			 "%s '%s' refer to the same variable\n"
			 ,feditfile(),cid->data().nodeName()()
			 ,acts[i].upperCase()(),ids[i]()
			 ,acts[j].upperCase()(),ids[j]());
	  }
	}
      }  
      if(!txt.empty()) throw Except(txt);

      ie0=ie;
      return;
    }else if(actiontype == "body")
      body=1;
    else if(actiontype == "counter")
      cid->appendToHeader(new CounterItemDef(dei,flags,cid->data()));
    else if(actiontype == "dashline"){
      if(body)
	cid->append(new DashLineItemDef(dei,flags));
      else
	cid->appendToHeader(new DashLineItemDef(dei,flags));
    }else if(actiontype == "title"){
      if(body)
	cid->append(new TitleItemDef(dei,flags));
      else
	cid->appendToHeader(new TitleItemDef(dei,flags));
    }else if(actiontype == "emptyline" || 
	     (actiontype.empty() && flags.contains("eol") )){
      if(body)
	cid->append(new TitleItemDef(StringList("eol")));
      else
	cid->appendToHeader(new TitleItemDef(StringList("eol")));
    }else if(actiontype == "item" || actiontype == "itemlist"){
      cid->append(b=new SingleItemDef(dei,flags,dr));
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
    }else if(actiontype == "select"){
      cid->append(b=new SelectGroupItemDef(dei,flags,dr,box));
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
    }else if(actiontype == "check"){
      cid->append(b=new CheckGroupItemDef(dei,flags,dr,box));
      cid->data().createTemplateFromDefValue(b->ID());
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
    }else if(actiontype == "mselect"){
      cid->append(b=new MSelectGroupItemDef(dei,flags,dr,box));
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
    }else if(actiontype == "field"){
      FieldItemDef *fid=new FieldItemDef(dei,flags,dr);
      mtab.remove(dr(fid->ID()).nodeName());
      cid->append(fid);
      ids.append(fid->ID()); acts.append(actiontype);
      readFieldScreenDefs(fid,de,ie,fname);
    }else if(actiontype=="compound"){
      cid->append(b=new CompoundItemDef(dei,flags,dr));
      mtab.remove(dr(b->ID()).nodeName());ids.append(b->ID()); 
      acts.append(actiontype);
      readCompoundScreenDefs((CompoundItemDef*)b,de,ie,fname);

    }else if( actiontype == "endfield"){
      throw Except(Sprintf("in %s: action %s is not properly nested,"
       			   " to many %ss.\n"
			   ,nsubfunc,actiontype.upperCase()()+3
			   ,actiontype.upperCase()()));
    }else
      throw Except(Sprintf("in %s: action %s: COMPOUNDs cannot contain %ss\n" \
			   ,nsubfunc,actiontype.upperCase()()
			   ,actiontype.upperCase()()));
    
    
    
    
  }
  
  actiontype="compound";
  throw Except(Sprintf("in %s: action %s is not properly nested,"
		       " missing END%s.\n"
		       ,nsubfunc ,actiontype.upperCase()()
		       ,actiontype.upperCase()()));
}


/*EOF*/ 
