/*************************************************************************
 * @(#)Form.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   23 Oct 2008
 *************************************************************************
 */

#include "Keys.hh"
#include "Viewer.hh"
#include "Form.hh"
#include "Fedit.hh"
#include "Assorted.hh"
#include "debugxx.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainForm::onExec()
{
  //todo  if(screendump_on()){
  //    errorScreen("Information","\n\nThe screen dump mode is turned on ... "
  //		"CONTROL-S will dump screens",0);
    //  }

  /* recreate ENTRYDEFFILE */
  if(fapp->newfeditfile){

    try{

      String fname=fapp->workPath(fapp->feditfile,Fedit::IOTmp);
      removeFile(fname);
      runProgram(fapp->pprogram,Fedit::IOTmp,0);
      
      fname=fapp->workPath(fapp->feditfile,Fedit::IOOriginal);
      removeFile(fname);
      runProgram(fapp->pprogram,Fedit::IOOriginal,0);
    
    
    }catch(Except &ex){
      errorScreen("Error Report",ex.msg());
    }
    

  }

  /* read table of files to be loaded */
  try{
    fapp->setupParseTables();
  }catch (Except &ex){

    errorScreen("Read Entry definitions: Error Report",ex.msg());
  }


  try { 
    
    /* create INPUTFILEs if not exist, PROGRAM is responsible for
       correct order
    */
    
    
    for(long i=0;i<fapp->parsetables.size();i++){
      
      if(pathExistence(fapp->parsetables.fileName(i))!=RegularFile){
	
	
	runProgram(fapp->pprogram,Fedit::IOOriginal,1);
	
      }
    }
    
    /* backup input files */
    for(long i=0;i<fapp->parsetables.size();i++){
      
      String fname=fapp->parsetables.fileName(i);
      backUp(fname,fapp->workPath(fname,Fedit::IOBak));
    }
  }catch(Except &ex){
    errorScreen("Error Report",ex.msg());
  }
  
  
  // let's load the data and see if the versions match
  
  long ok=1;
  try{
    ok=fapp->readFiles(Fedit::IOOriginal,0L);
  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg(),1);
  }
  if(!ok){
    
    if(app->s->piper()){
      app->s->piper()->message("Old file version, will upgrade!!!");
    }else{
      if(setStatusBar("Old file version, shall I upgrade?"
		      ,StatusBar::StatusChoose)!=Accept)
	throw TerminateExcept("Version missmatch, aborted!");
      
    }  
    fapp->parsetables.cleanupTables();
    fapp->screens.clear();
    /*call PROGRAM to make new version files */
    
    
    runProgram(fapp->pprogram,Fedit::IOOriginal,1);
    
    
    /* try again */
    try{
      fapp->readFiles(Fedit::IOOriginal);
    }catch(Except &ex){
      // there might be an error , copy back backup to avoid more destruction
      for(long i=0;i<fapp->parsetables.size();i++){
	String fname=fapp->parsetables.fileName(i);
	backUp(fapp->workPath(fname,Fedit::IOBak)
	       ,fapp->workPath(fname,Fedit::IOOriginal));
      }
      errorScreen(ex.title(),String("")+ex.msg(),1);
    }
    
  }

  //  read screen definitions
  try{
    fapp->readScreenDefs();
  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg());
  }


}


















/* ***********************************************************************
 *
 * ***********************************************************************
 */
SubMenuForm::SubMenuForm(App *a,String const &title,SObject*p,String const &cn)
  :Form(a,title,p,cn),fapp(0)
{
  //HIER("SubMenuForm");

  fapp=feditApp();

  new MenuItem(pmenu,"e@(x)it","exit","exit");
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fedit *SubMenuForm::feditApp()const
{
  Fedit *f=dynamic_cast<Fedit*>(app);
  if(!f)
    throw TerminateExcept("\nForm has been instantiated from a non-Fedit "
		 "App-derived class\n");

  return f;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::aboutToShow()
{
  //  NSF("SubMenuForm::aboutToShow");   
  //HIER(subfunc);

  
  /* init menubars */
  try{
  
    pregisteredhooks.clear();
    pmenu->clear();
    paltmenu->clear();

    ScreenDef const *sde=fapp->screens.byName(screenname);

    if(sde){

      
      
      // setup alternative menubar marker
      for (long j=0;j<sde->msize();j++){
	MenuDef const *m=sde->menu(j);
	if( m->type() == "alt" && 
	    (m->action() != "eol" && m->action() != "separator" ) ){
	  new MenuItem(pmenu,"@_","","");
	  break;
	}
      }
      
      // load menus
      for (long j=0;j<sde->msize();j++){
	MenuDef const *m=sde->menu(j);
	
	if(m->type()=="main"){

	  if(m->action()=="help")
	    new MenuItem(pmenu,m->name(),m->action(),screenname);
	  
	  else if( fapp->expertmode ||  (
					 m->action()!="mainexit"    &&
					 m->action()!="externedit"    
					 )   )
	    new MenuItem(pmenu,m->name(),m->action(),m->data());
	}else if(m->type()=="alt")
	  new MenuItem(paltmenu,m->name(),m->action(),m->data());
	
      } /* all menues */
      
      
    } /* proper screenname */
  
    
  }catch(Except &ex){
    errorScreen("Error Report",ex.msg());
  }

  try{
    loadItems(screenname);
  }catch(Except &ex){
    errorScreen("Page data load: Error Report",ex.msg());
  }
  
  
//todo   PageLoadHotkeyTable(form);
  

  setStatusBar();
  
  return;
  
  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::loadItems(String const &sname)
{
  //NSF("SubMenuForm::loadItems");

  clear();

  for (long i=0;i<fapp->screens.size();i++){
    ScreenDef *sde=fapp->screens[i];

    if(sde->screenName()==sname || sname.length()==0){
      
      for (long j=0;j<sde->bsize();j++){
	BaseItemDef *bid=sde->baseitem(j);
	Dobj dr=fapp->parsetables.parser(sde->ptabIndex())();
	

	bid->addToForm(this,dr,1);

      }
    }
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long SubMenuForm::notify(SObject *caller,String action,String data1
			 ,String data2)
{

  if(action=="exit"){
    exitAction();
    return 1;
  }else if(action == "symmetryexit"){
    exitSymmetryAction();
    return 1;
  }else if(action == "mainexit"){
    Page::exitAction(); // exit without save
    return 1;
  }else if(action == "submenu"){
    submenuAction(data1);
    return 1;
  }else if(action == "mainquit"){
    quitAction();
    return 1;
  }else if(action == "viewer"){
    viewerAction(data1);
    return 1;
  }else if(action == "externedit"){
    externEditAction();
    return 1;
  }else if(action == "restore"){
    restoreAction();
    return 1;
  }else if(action == "rebuild"){
    rebuildAction(data1);
    return 1;
  }else if(action == "fplorun"){
    fploRunAction();
    return 1;
  }else if(action == "trigger_update"){
    fploRunAction(data1);
    return 1;
  }else if(action == "run"){
    runAction(data1);
    return 1;
  }else if(action == "registerhook"){
    pregisteredhooks.append(data1);
  }else if(action == "loadItems"){
    try{
      loadItems(screenname);
      paintObjects();
    }catch(Except &ex){
      errorScreen("Page data load: Error Report",ex.msg());
    }
    return 1;
  }else{
    if(!Page::notify(caller,action,data1,data2))
      errorScreen("Error Report",Sprintf("in SubMenuForm::notify: unknown "
					 "callback action '%s(\"%s\",\"%s\")'"
					 ,action(),data1(),data2()));
    return 1;
  }
  
  
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::quitAction()
{
  //  NSF("quitAction");     
  //HIER(subfunc);

   String msg;
   msg.sprintf("Save now the file '%s'",fapp->parsetables.fileName(0)());
   if(fapp->parsetables.size()==2)
     msg+=" and '"+fapp->parsetables.fileName(1)+"'";
   if(fapp->parsetables.size()>2)
     msg+=", '"+fapp->parsetables.fileName(1)+"',...";
   msg+="?  (Esc: abort)";

   ExecStatus ret=Accept;

   if(! fapp->s->piper())
     ret=setStatusBar(msg,StatusBar::StatusChoose);
   
   if (ret==Accept){
     if(!globalTestContent()) return;
     try{
       fapp->writeFiles(this,Fedit::IOOriginal);
     }catch(Except &ex){
       errorScreen(ex.title(),ex.msg());
     }
     
     Page::quitAction();
     return; 
     
   }else if(ret==Discard){
     Page::quitAction();
     return;
   }
   setStatusBar("Nothing done !",StatusBar::StatusMessage);
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::exitSymmetryAction()
{
  //HIER("symmetry submenu exit");



  if (! testContent()) return; 
  

  
  if(contentChanged()){
    
    /* TODO: set infile action 
       after abort and relaunch , check action and go to
       symmetry
       go to symmetry, if reloaded infile has wrong action
    */
    
    
    
     
    if(app->s->piper()){
      app->s->piper()
	->message(Sprintf(" WARNING: Screen %s: content changed, but not"
			  " updated!\n          "
			  "New symmetry settings discarded!\n",screenname()));
    // todo: provoke update 
    }else{
      if(setStatusBar("Content changed, not updated, really leave?"
		      ,StatusBar::StatusChoose)!=Accept){
	setStatusBar();
	return;
      }
    }
  }

  Page::exitAction();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::viewerAction(String const &filename)
{

  if(app->s->piper()){
    app->s->piper()
      ->message(Sprintf(" VIEWER \'%s\'",filename())
		,"Viewer mode is disabled in pipe mode!\n"
		" Please remove the according hotkeys.");
    throw TerminateExcept("Abort");
  }

  String path=filename;
  path.stripBlanks();
  
  String dir;
  String file;
  splitPath(dir,file,path);
  
  if(dir=="$tmp"){
    path=fapp->workPath(file,Fedit::IOTmp);

    if(pathExistence(path)!=RegularFile){
      path=fapp->workPath(file,Fedit::IOOriginal);
    }
  }


  FileViewer *form=new FileViewer(app,file,path,this);
  
  form->exec();

  delete form;

  show();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::submenuAction(String const &name)
{
  //  NSF("submenuAction");
  //HIER(subfunc);

  if(screenname.empty()){
    setStatusBar();
    return;
  }


  // on entering a submenu, the parent is tested

  if (! testContent()) return; 
  

  

   SubMenuForm *form=new SubMenuForm(app,name,this);

   form->exec();

   
   pcontentchanged|=form->contentChanged();

   
   delete form;

   show();

    

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::runProgram(String const &program,Fedit::IOWorkMode ioflag
		      ,long report)
{
  long exitcode;
  setStatusBar("Running "+program+" ...!",StatusBar::StatusMessage);
  String txt=fapp->runProgram(program,ioflag,report,exitcode);
  setStatusBar();

  if(report){
    String title=program+" OUTPUT";
    if(app->s->piper()){
      app->s->piper()->message(title,txt);
      if(!exitcode)throw TerminateExcept("Abort");
    }else{
      
      Viewer *v=new Viewer(app,title,this);
      v->setText(txt);
      v->startAtBottom();
      v->exec();
      delete v;
      setStatusBar();
      
    }
  }
  paint();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long SubMenuForm::globalTestContent()
{
  //NSF("SubMenuForm::globalTestContent");
  
  
  /* create a virtual form containing all items use testContent to validate all */

  
  SubMenuForm *form=new SubMenuForm(app,"global test",this);

  
  try{
    form->loadItems("");
  }catch(Except &ex){
    errorScreen("Error Report",ex.msg());
  }

  
  if (!form->testContent(1))return 0; 
  
  
  delete form;
  
  
  show();
  
  
  return 1;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::externEditAction()
{
  //NSF("externEditAction");
  //  HIER(subfunc);

  
  
  if(fapp->s->piper()){
    fapp->s->piper()
      ->message(Sprintf(" Function 'extern edit' not available in"
			" pipe mode in screen '%s'\n",screenname()));
    return;
  }

  globalTestContent();  
  
  fapp->writeFiles(this,Fedit::IOTmp);

 
  /* clean up forms */
  Page *p=this;
  while(p){
    p->clear() ;   
    p->menu()->clear();
    p->altMenu()->clear();
    p=dynamic_cast<Page*>(p->parent());
    
  }
  
  //  get the parse table the file to edit

  ScreenDef const *sde=fapp->screens.byName(screenname);
  long pct=sde->ptabIndex();
  
 editloop:    
  
  
  /* clean tables */
  fapp->parsetables.cleanupTables();
  
  fapp->screens.clear();

  /*TODO: here we have a risk, that the files in +fedit are changed
    if globalscreendefs are reloaded, but pctables are not !!!
  */
  
  
  
  
  
  Screen *s=app->s;
  if(s) s->shellOut();
  systemCall(fapp->editor+" "
	     +fapp->workPath(fapp->parsetables.fileName(pct),Fedit::IOTmp));
  
  

  if(s) s->shellIn();
  
  
  try{
    fapp->readFiles(Fedit::IOTmp);
    fapp->readScreenDefs();
  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg(),0);
    /* on error continue edit */
    goto editloop;
  }
  
  
  
  
  /* init forms */
  p=this;
  while(p){
    try{
      p->aboutToShow();
    }catch(Except &ex){
      errorScreen("Error Report",ex.msg());
    }
    p=dynamic_cast<Page*>(p->parent());
  }
  
  
  pcontentchanged=1;
  
  
  
  /* test data */
  if(globalTestContent()) show();  

  setStatusBar();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::restoreAction()
{
  //NSF("SubMenuForm::restoreAction");
  //  HIER(subfunc);


  

  if(app->s->piper()){
    fapp->s->piper()
      ->message(Sprintf(" Function 'restore' not available in"
			     " pipe mode in screen '%s'\n",screenname()));
    return;
  }else{
    if(setStatusBar("Really restore old content?",StatusBar::StatusChoose)!=Accept)
      return;
  }
  
  
  /* clean up forms */
  Form *p=this;
  while(p){
    p->clear() ;   
    p->menu()->clear();
    p->altMenu()->clear();
    p=dynamic_cast<Form*>(p->parent());
    
  }

  
   /* clean tables */
  fapp->parsetables.cleanupTables();
  
  fapp->screens.clear();
  /*TODO: here we have a risk, that the files in +fedit are changed
    if globalscreendefs are reloaded, but pctables are not !!!
  */
  

  try{
    fapp->readFiles(Fedit::IOBak);
    fapp->readScreenDefs();
  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg());
  }

  /* init forms */
  p=this;
  while(p){
    try{
      p->aboutToShow();
    }catch(Except &ex){
      errorScreen("Error Report",ex.msg());
    }
    p->resetContentChanged();
    p=dynamic_cast<Form*>(p->parent());
  }
  
  
  show();
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::rebuildAction(String const &filename)
{
  //NSF("SubMenuForm::rebuildAction");
  //  HIER(subfunc);


  if(app->s->piper()){
    app->s->piper()
      ->message(Sprintf(" Function 'rebuild/recreate' called in screen '%s'\n"
		       ,screenname()));
  }else{
    if(setStatusBar("Really rebuild default content ?"
		    ,StatusBar::StatusChoose)!=Accept)
      return;
  
  }

  /* recreate INPUTFILEs */

  fapp->writeFiles(this,Fedit::IOTmp);

  
  try{
    removeFile(fapp->workPath(filename,Fedit::IOTmp));
  }catch(Except &ex){
    throw Except(ex.msg(),"Error Report");
  }
  
  runProgram(fapp->pprogram,Fedit::IOTmp,1);
  

  setStatusBar();
  
  
  /* clean up forms */
  Form *p=this;
  while(p){
    p->clear() ;   
    p->menu()->clear();
    p->altMenu()->clear();
    p=dynamic_cast<Form*>(p->parent());
    
  }
  
  /* clean tables */
  fapp->parsetables.cleanupTables();
  
  fapp->screens.clear();
  /*TODO: here we have a risk, that the files in +fedit are changed
    if globalscreendefs are reloaded, but pctables are not !!!
  */
  

  try{
    fapp->readFiles(Fedit::IOTmp);
    fapp->readScreenDefs();

  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg());
  }
  
  /* init forms */
  p=this;
  while(p){
    try{
      p->aboutToShow();
    }catch(Except &ex){
      errorScreen("Error Report",ex.msg());
    }
    p->resetContentChanged();
    p=dynamic_cast<Form*>(p->parent());
  }

  show();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::fploRunAction(String const &update)
{
  //NSF("SubMenuForm::fploRunAction");
  //HIER(subfunc);

  
  if(!globalTestContent()) return;  
 

  //  get the parse table the file to edit
  
  ScreenDef const *sde=fapp->screens.byName(screenname);
  long pct=sde->ptabIndex();
  
  /* we assume that this is used in symmetry only, and that symmetry file has
     a status flag, it's a hack
  */
  
  try{
    Fedit::setFileStatus(fapp->parsetables.parser(pct),
			 Fedit::UpdateStatusChanged,update);
  }catch(Except &ex){ errorScreen("Error Report",ex.msg());}


  fapp->writeFiles(this,Fedit::IOTmp);
  
  
  /* clean up forms */
  Form *p=this;
  while(p){
    p->clear() ;   
    p->menu()->clear();
    p->altMenu()->clear();
    p=dynamic_cast<Form*>(p->parent());
    
  }
  
  /* clean tables */
  fapp->parsetables.cleanupTables();
  
  fapp->screens.clear();

  /*TODO: here we have a risk, that the files in +fedit are changed
    if globalscreendefs are reloaded, but pctables are not !!!
  */
  
  
  runProgram(fapp->pprogram,Fedit::IOTmp,1);


  try{
    fapp->readFiles(Fedit::IOTmp);
    fapp->readScreenDefs();

  }catch(Except &ex){
    errorScreen(ex.title(),ex.msg());
  }
  
  
  /*TODO: another fragil hack */
  try{
    Fedit::setFileStatus(fapp->parsetables.parser(pct)
			 ,Fedit::UpdateStatusReady,"");
  }catch (Except &ex){ errorScreen("Error Report",ex.msg());}
  
  /* init forms */
  p=this;
  while(p){
    try{
      p->aboutToShow();
    }catch(Except &ex){
      errorScreen("Error Report",ex.msg());
    }
    p->resetContentChanged();
    p=dynamic_cast<Form*>(p->parent());
  }

  
  
  /* test data */
  if(globalTestContent()) show();  
  
  
  setStatusBar();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubMenuForm::runAction(String const &options)
{
  //NSF("SubMenuForm::runAction");
  //  HIER(subfunc);
  
  
  if(!globalTestContent()) return;  
  
  
  fapp->writeFiles(this,Fedit::IOTmp);
  
  
  String p=fapp->pprogram+" "+options;
  p.stripBlanks();
  runProgram(p,Fedit::IOTmp,1);
  
  /* test data */
  //  show();
  
}
















/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Form::errorScreen(String const &title,String const &msg,long stop)
{
  if(app->s->piper()){
    app->s->piper()->message(title,msg);
  }else{
    Viewer *v=new Viewer(app,title,this);
    v->setText(msg);
    v->exec();
    delete v;
    setStatusBar();
    paint();
  }  
  
  if(stop)
    app->quit(TerminateExcept::ExitEmergencyAbort);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Form::searchObjectOnHotkey(String const &keysequence,long latestmatch)
{
  
  
  // reset on out-of-bound
  if(latestmatch>pobjects.size()-1 || latestmatch<0) latestmatch=0;
  
  
  for (long i=latestmatch;i<pobjects.size();i++){

    if(String::cmp(pobjects[i]->hotkey(),keysequence,keysequence.length())==0){
   
      makeObjectVisibile(i);

      removeObjectHighLighting();    
      highlightedobject=i;
      paintObject(highlightedobject,SearchHighLighting);	
      
      return i;
      break;
    }
    
  }
  
  return -1;

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Form::onKeyEvent(Event e)
{
  //NSF("Form::onKeyEvent");
  //HIER(subfunc);

  if(Page::onKeyEvent(e) ) return 1;

 
  long i;
  
  
  /* multi hotkeys */

  Piper *piper=app->s->piper();

  if(piper){
    String keysequence;
    while(e.key != '\n' ){
      //keysequence+=(char)e.key;
      keysequence+=(char)Sprintf("%c",e.key).toLower()[0];
      e=app->getKey();
    }
    for(long i=0;i<pobjects.size();i++){
      if(pobjects[i]->hotkey()==keysequence){
	editObjects(keysequence);
	execRegisteredHooks();
	piper->nextpipeline();
	return 1;
      }
    }
    piper->message(Sprintf(" ERROR: Hotkey '%s' not found in screen '%s'\n"
			,keysequence(),screenname()));
    throw TerminateExcept("Abort");
  }
  
   /* normal interactive run */
   
   if(!String::isPrint(e.key)) return 0;

   char key=Sprintf("%c",e.key).toLower()[0];
   
   long latestmatch=-1;
   long inseq=0;
   long ih1=0;
   long ih2=pobjects.size()-1;
   String keysequence;

 hotloop:
   
   keysequence+=key;
   // find first occurence of the keysequence
   for(long i=ih1;i<=ih2;i++){

     if(String::cmp(pobjects[i]->hotkey(),keysequence,keysequence.length())==0) {
       break;
     }
     ih1++;
   }
   
   if(ih1>ih2){
     if(inseq)
       setStatusBar("Hotkeys: aborted");
     else
       setStatusBar();
     removeObjectHighLighting();
     
     return 1;
   }
   
   // find last occurence of the keysequence
   for(long i=ih2;i>ih1;i--){
     if(String::cmp(pobjects[i]->hotkey(),keysequence,keysequence.length())==0) break;
     ih2--;
   }
   
   // if this narrow down to one single main entry, we have can edit it
   if(ih1==ih2) {
     keysequence=pobjects[ih1]->hotkey();
     goto edit;
   }
   
 newkey:
   inseq=1;
   setStatusBar("Hotkeys: R<["+keysequence
		+"]> (more keys or TAB search ,ENTER=take it)"
		,StatusBar::StatusUser);
   
   i=searchObjectOnHotkey(keysequence,latestmatch);
   if(i<0){
     latestmatch=0;
     i=searchObjectOnHotkey(keysequence,latestmatch);
   }
   
   e=app->getKey(); 
   
   if(String::isPrint(e.key)) 
     key=Sprintf("%c",e.key).toLower()[0];
   
   if(String::isPrint(e.key)) {
     goto hotloop;
   }
     
   
   if(
#if !( S_CTRL_I == '\t' )
      e.key=='\t'||
#endif
      e.key==S_CTRL_W||
      e.key==S_CTRL_I){
     
     latestmatch=i+1;
     goto newkey;
        
   }else if(e.key== S_ENTER){
     if(highlightedobject>-1){
       keysequence=pobjects[i]->hotkey();
     }
     goto edit;
     
   }else if(e.key== S_ESCESC||	  
	    e.key== S_CTRL_G){
     setStatusBar("Hotkeys: aborted");
     removeObjectHighLighting();
     return 1;
   }else{
     setStatusBar("Hotkeys: aborted");
     removeObjectHighLighting();
     return 1;
   }
   
   
   /* edit action */
 edit:
   removeObjectHighLighting();
   setStatusBar();
   editObjects(keysequence);
   execRegisteredHooks();


   
   
   return 1;
   

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Form::editObjects(String const &keysequence)
{
  
  long ret=0;

  for (long i=0;i<pobjects.size();i++){
    
    if(pobjects[i]->hotkey(0)!=keysequence  || keysequence.length()==0 )
      continue;
    
    makeObjectVisibile(i);

    if(!objectVisible(i)){
      latestfoundobject=-1;
      foundline=0;
    }
    

    pobjects[i]->edit(this,adrb-currentline);

    removeObjectHighLighting();
    highlightedobject=i;
    paintObject(highlightedobject,EditedHighLighting);	

    ret=1;
    
    
  }

  // move cursor to upper left corner
  app->s->moveTo(adrb,0);
 

  return ret;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Form::onSearchSelect(long selctedobject)
{
  editObjects(pobjects[latestfoundobject]->hotkey());
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Form::testContent(long globaltest)
{
  String txt;
  String title;
  
  if(globaltest)
    title="Global data test: Error Report";
  else
    title=Sprintf("Data test of screen '%s': Error Report",screenname());
  
  long ok=1;
  for (long i=0;i<pobjects.size();i++){
    if(pobjects[i]){
      String mesg;
      if(!pobjects[i]->testContent(mesg)){
	
	txt+=mesg+"\n\n"; 
	
	ok=0;
	
      }
    }
  }
  
  if(!ok){
    errorScreen(title,txt,0L);
    if(app->s->piper()) throw TerminateExcept("Abort");
  }
  return ok;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Form::contentChanged()
{
  for (long i=0;i<pobjects.size();i++){
    if(pobjects[i])
      pcontentchanged|=(pobjects[i]->contentChanged());
  }
  return pcontentchanged;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Form::exitAction()
{
  //  NSF("Form::exitAction");
  //  HIER(subfunc);
  

  if (! testContent()) return; 
  

  Page::exitAction();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Form::execRegisteredHooks()
{
  for(long i=pregisteredhooks.size()-1;i>=0;i--){
    notify(0,pregisteredhooks[i],"");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Form::notify(SObject *caller,String action,String data1
			 ,String data2)
{

//   if(action == "trigger_update"){
//     fploRunAction(data1);
//     return 1;
//   }else 
  if(action == "registerhook"){
    pregisteredhooks.append(data1);
  }else if(action == "loadItems"){
    try{
      loadCompoundItems(screenname);
      paintObjects();
    }catch(Except &ex){
      errorScreen("Page data load: Error Report",ex.msg());
    }
    return 1;
  }else{
    if(!Page::notify(caller,action,data1,data2))
      errorScreen("Error Report",Sprintf("in Form::notify: unknown "
					 "callback action '%s(\"%s\",\"%s\")'"
					 ,action(),data1(),data2()));
    return 1;
  }
  
  
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Form::loadCompoundItems(String const &screenname)
{
  if(!bid) return;
  if(!biddata) return;
  clear();
  bid->internalAddToForm(this,1,*biddata,1,bid->namePos());

}

/*EOF*/ 
