/*************************************************************************
 * @(#)Item.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   23 Oct 2008
 *************************************************************************
 */

#include "Item.hh"
#include "Screen.hh"
#include "Page.hh"
#include "Form.hh"
#include "LineEdit.hh"
#include "ValueArray.hh"


/* the macro version  calls j twice !! */
static long LMIN(long i,long j)
{
  return ( ((i)<j?i:j) );
}
/* the macro version  calls j twice !! */
static long LMAX(long i,long j)
{
  return ( ((i)>j?i:j) );
}

// {{{BaseItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
char BaseItem::clearArea(Screen *s,long y,long x,long len,HighLightState highlightmode)
{
  char c0;
  if(s->screenDebug()){
    c0='?';
  }else{
    c0=' ';
  }
  
  String ss;
  ss.rightPadd(c0,len);
  s->moveTo(y,x);
  paintHighlighted(s,ss,highlightmode);

  return c0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BaseItem::paintHighlighted(Screen *s,String const &text
				,HighLightState highlightmode)
{
  
  if(highlightmode==SearchHighLighting){
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOn);
    s->reverseOn();
  }
  if(highlightmode==EditedHighLighting)
    s->underlineOn();
  
  s->addStr(text);
  
  if(highlightmode==EditedHighLighting)
    s->underlineOff();
  if(highlightmode==SearchHighLighting){
    s->reverseOff();
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOff);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BaseItem::paintTokHighlighted(Screen *s,String const &text
				   ,HighLightState highlightmode)
{
  
  if(highlightmode==SearchHighLighting){
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOn);
    s->reverseOn();
    
    s->prtToken(text,Screen::StyleEntrySearch
		,Screen::StyleEntrySearch);
    
    s->reverseOff();
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOff);
  }else{
    if(highlightmode==EditedHighLighting)
      s->underlineOn();
    
    s->prtToken(text,Screen::StyleEntryHigh
		,Screen::StyleNormal);
    
    if(highlightmode==EditedHighLighting)
      s->underlineOff();
  }
  
}

// }}}

// {{{LineItem/HelpItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void LineItem::paint(Screen *s,long y0,HighLightState highlightmode)
{
  s->deleteLine(ypos+y0);
  s->moveTo(ypos+y0,0);
  paintHighlighted(s,text,highlightmode);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void HelpLineItem::paint(Screen *s,long y0,HighLightState highlightmode)
{
  s->deleteLine(ypos+y0);
  s->moveTo(ypos+y0,0);

  if(highlightmode==SearchHighLighting){
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOn);
    s->reverseOn();
  }

  s->moveTo(ypos+y0,0);
  s->prtMarkUpLine(text);


  if(highlightmode==SearchHighLighting){
    s->reverseOff();
    s->setStyle(Screen::StyleEntrySearch,Screen::AttOff);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String HelpLineItem::screenText()const
{
  
  char *v=new char[text.length()+1];
  long l=0;
  v[l]='\0';

  const char *p=text();


  ValueArray<char> mode;
  
  while( (*p) != '\0'){

    if( ( (*p) == 'U' || (*p) == 'B' || (*p) == 'A' || (*p) == 'R'  )
	&& (*(p+1)=='<')){
      mode.append(*p);
      p++;
    }else if( ( (*p)=='>') && ( mode.size()>0 ) ) {
      mode.removeLast();
    }else{
      v[l++]=*p;
    }
    p++;
  }
  v[l]='\0';

  String s(v);

  delete [] v;
  return s;
}

// }}}



// {{{TitleItem/DashLineItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TitleItem::paint(Screen *s,long y0,HighLightState highlightmode)
{
  //NSF("TitleItem::paint");

  long pos=namePos();
  long len=width();
  if(len<0){
    len=LMAX(s->cols()-pos-2+width(),offset()+1+3);
  }

  clearArea(s,ypos+y0,pos,len,highlightmode);
  

  String ss=screenText();
    
  
  s->moveTo(ypos+y0,pos);
  
  paintHighlighted(s,ss,highlightmode);
  
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DashLineItem::paint(Screen *s,long y0,HighLightState highlightmode)
{

  long pos=bid()->namePos()+ptab;
  long len=(bid()->offset() <=0 ? s->cols()+bid()->offset()-pos
	    : bid()->offset());
  s->moveTo(ypos+y0,pos);while(len--)s->addHLIN();
  
  
}

// }}}




// {{{EditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long EditItem::vWidth(Screen *s)
{
  
  long wid=width();
  if(wid<0){
    wid=LMAX(wid+s->cols()-2-3-offset()-namePos(),1);
  }
  return wid;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void EditItem::paint(Screen *s,long y0,HighLightState highlightmode)
{
  //  NSF("EditItem::paint");
  onLayout(s);

  long len=offset()+width()+3;
  if(width()<0){
    len=LMAX(s->cols()-namePos()-2+width(),offset()+1+3);
  }
  len+=2;
  char c0=clearArea(s,ypos+y0,namePos(),len,highlightmode);
  
  String ss=markupRep();
  long l=ss.length()-screenRepLength(ss);
  ss.rightPadd(c0,offset()+l);
  if(cutoffset)
    ss.shorten(offset()+l);
  
  if(printcolon){
    if(s->screenDebug()){
      ss+=Sprintf(" %c ",(offset()?':':'?'));
    }else{
      ss+=Sprintf(" %c ",(offset()?':':' '));
    }
  }  
  
  s->moveTo(ypos+y0,namePos());
  
  paintTokHighlighted(s,ss,highlightmode);

  
  paintValue(s,y0,highlightmode);
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void EditItem::paintValue(Screen *s,long y0
			  ,HighLightState highlightmode)
{
  //NSF("EditItem::paintValue");
  //HERE(subfunc);
  String ss=value();

  if(test()->autocompletion()){
    const SelectTest *t=dynamic_cast<const SelectTest*>(test());
    if(t){
      data()=t->mapToDescriptor(ss); // autocorrect
    }
  }


  if(data().isFlag() || ( data().isLogical() && data().isScalar())){
    paintValueCheckItem(s,y0,highlightmode);
    return;
  }
  
  long i=ss.length();
  long width=vWidth(s);
  if(i>width+2){
    ss=ss.subString(0,LMAX(width-1,0));
    ss+="<<";
  }
  
  s->moveTo(ypos+y0,namePos()+offset()+3);
  
  paintHighlighted(s,ss,highlightmode);
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void EditItem::edit(Form *f,long y0)
{
  //  NSF("EditItem::edit");
  if(!f) return;

  Screen *s=f->application()->s;
  Piper *piper=s->piper();
  
  long yad=ypos+y0;
  long xad=namePos()+offset()+3;
  
  if( (!piper) && test()->isToggle()){
    // this is the toggling edit mode
    String v=test()->toggleIt(value());
    paintValue(s,y0);
    setValue(f,v);
    return;
  }
  
  // and this is the lineedit mode
  
  f->setStatusBar("editing...");

  long lic;
  String v=value(lic);
  
  String hints="";

  const Test *tst=test();
  const SelectTest *slt=0;
  if(tst->autocompletion()){
    slt=dynamic_cast<const SelectTest*>(tst);
    if(slt){

      slt->mapToDescriptor(v); // manipulate value into descriptor equiv
      
      StringList sl=slt->getSelectDescriptorHints();
      hints+=String("possible values: H<")+sl.join("> H<")+">, ";
      // setup autocompletion stringlist test
      tst=new StringTest(1,-1,String("autocompletion")
			 ,slt->getSelectDescriptors()); 
    }
  }  

  f->setStatusBar(hints+"editing...");

  String save=v;
  
  
  while (1) {
    
    LineEdit le(f->application(),xad,yad,vWidth(s),f);
    le.setText(v);
    
    if(le.exec()!=Accept){
      v=save;
      break;
    }
    
    v=le.text();
    
    try{
      v.escapeEncode();
      tst->validate(v,lic,1L);//todo change like toggleit
      v.escapeDecode();
      break;
    }catch(Except &ex){
      String msg=String("invalid ")+ex.msg();
      if(slt)
	msg=hints;

      if(piper){
	piper->message(Sprintf(" ERROR: item '%s' %s in screen '%s'\n"
			       ,markupRep().removeFromSet("@")()
			       ,msg(),f->screenName()()));
	throw TerminateExcept("Abort");
      }
    
      f->setStatusBar(msg+"editing...",StatusBar::StatusError);
    }
  }


  // mapback
  if(slt){
    v=String(slt->mapFromDescriptor(v));
  }
  
  
  
  
  
  
  if(piper){
    piper->message(Sprintf(" item '%s' new value '%s' in screen '%s'\n"
			   ,markupRep().removeFromSet("@")()
			   ,v(),f->screenName()()));
  }
  

  
  f->setStatusBar();
  
  // repaint value is not really needed since we repaint the item later
  // but for logical correctness
  paintValue(s,y0);
  
  setValue(f,v);


  if(slt){
    data().next()=slt->getSelectDescriptor(data()());
    delete tst;
  }

  if(save!=v)
    if(flags().contains("trigger_update")){
      f->notify(0,"registerhook","trigger_update");
    }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void EditItem::paintValueCheckItem(Screen *s,long y0
			  ,HighLightState highlightmode)
{
  
  //  NSF("EditItem::paintValueCheckItem");
  
  long wid=LMAX(width(),3);
  String v=value();
  long i=v.length();
  String ss="[ ]"; 
  if(v()[0]=='+') ss="[X]";
  
  
  if(i>wid+2){
    ss=ss.subString(0,LMAX(wid-1,0));
    ss+="<<";
  }
  
  s->moveTo(ypos+y0,namePos()+offset()+3);

  paintHighlighted(s,ss,highlightmode);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String EditItem::value(long &lic)const
{
  //HIER("EditItem::value(long)");
  //HIER(constdata().fullName()());
  String v;
  if(constdata().isScalar()){
    if(constdata().isFlag()){
      v+=constdata()()[0]; 
    }else if(constdata().isLogical()){
      v+=(constdata()()[0]=='t'?'+':'-');
    }else{
      v=String::escapeDecode(constdata()());
    }
    lic=1;
  }else if(constdata().isArray()){
    lic=constdata().size();
    for(long ia=0;ia<lic;ia++){
      v+=String::escapeDecode(constdata()[ia]());
      if(ia<lic-1) v+=" ";
    }
  }


  return v;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void EditItem::setValue(Form *f,String const &v)
{
  NSF("EditItem::setValue");


  if(data().isScalar()){
    if(data().isFlag()){
      String s=data()();
      s.set(0,v[0]);
      data()=s;
    }else if(data().isLogical()){
      String s="f";
      if(v[0]=='t' || v[0]=='+')
	s="t";
      data()=s;
    }else{
      String s=v;
      s.escapeEncode();
      data()=s;
    }
    long c=0;
    if(data().isInt() && data().isDim()){
      // we have an int connected to an (struct)array dim-spec
      // we have to honor that
	
      String name=data().nodeName();
	
      String v=data()();
      long l;
      try{ l=v.convertToLong(); } catch(...){
 	  throw Except(String().sprintf("internal(%s): variable '%s'\n "
					"does not contain a"
					" valid integer value (%s)!"
 					,nsubfunc,data().fullName()(),v()));
      }
      Dobj d;
      try{ 
	Parser const *p=data().parser();
	d=(*p)().first();
      } catch(Except &ex){
	throw Except(String().sprintf("internal(%s):  "
				      "internal: %s"
				      ,nsubfunc,ex.msg()));
      }
      while(d.validNode()){
	
	
	if(d.isArray()){
	  
	  for(long ir=1;ir<d.rank();ir++){
	    if(d.hasIDRef(ir) && d.IDRef(ir)==name )
	      throw
		Except(Sprintf("internal(%s): in the moment"
			       " size update is\n "
			       "not implemented for multidimensional arrays with"
			       " size specifier, \n unless it is placed in the"
			       " last dimension!\n"
			       " i.e. the size specifier, which changed size is N\n"
			       " then  real a[,,,N] is     possible\n"
			       "       real a[,N,,] is NOT possible\n"
			       " Denied request: array %s [... ,%s, ...]\n"
			       ,nsubfunc,d.fullName()(),name()));
	    
	  }
	  
	  long rank=d.rank();
	  long sizes[8];
	  if(d.hasIDRef(rank) && d.IDRef(rank)==name ){
	    d.getSizes(sizes);
	    
	    c=(sizes[rank]!=l);
	    
	    if(sizes[rank]!=l){
	      sizes[rank]=l;
	      
 	      try{
 		d.resizeArp(sizes);
 	      } catch(Except &ex) {
 		throw Except(String().sprintf("internal(%s): resizing of "
 					      "array '%s' failed\n%s"
 					      ,nsubfunc,d.fullName()(),ex.msg()));
 	      }
 	    }
	  }
	  
	}else if(d.isStructArray()){
	  
	  if(d.hasIDRef() && d.IDRef()==name ){
	    
	    c=(d.size()!=l);
	    
	    if(d.size()!=l){
	      try{
		d.resizeSap(l);
	      } catch(Except &ex) {
		throw Except(String().sprintf("internal(%s): resizing of "
					      "structarray '%s' failed\n%s"
					      ,nsubfunc,d.fullName()(),ex.msg()));
	      }
	    }
	    
	  }
	} // struct array
	
	d=d.next();
      } // while validnode
      
      
    } // if dimspec

    if(c){
      f->contentChanged();/* Design flaw: we have to honor the changes here, even
			    if the changes are reverted back, we cannot
			    keep track of it, since the objects get recreated
			    and their former change status gets lost. */
      f->notify(0,"registerhook","loadItems");
    }


    
  }else{
    StringList sl=String(v).split(" \t");
    long n=sl.size();
    
    long rank=data().rank();
    if(rank!=1){
      throw Except(Sprintf("internal(%s): variable '%s':\n"
			   " only rank 1 arrays implemented!\n"
			   ,nsubfunc,data().fullName()()));  
    }
    
    
    long sizes[8];
    if(data().isVariableSize()) {
      data().getSizes(sizes);
      sizes[rank]=n;
      try{ data().resizeArp(sizes); } catch(Except &ex) {
	throw Except(Sprintf("internal(%s): resizing of "
			     "array '%s' failed\n%s"
			     ,nsubfunc,data().fullName()(),ex.msg()));
      }
    }
    
    while(n--){
      
      String vv=sl[n];
      vv.escapeEncode();
      Dobj di=data()[n];
      
      
      try{ di=vv; }catch(Except &ex){
	throw Except(Sprintf("internal(%s): screen item '%s',\n "
			     "variable '%s':\n %s\n",nsubfunc
			     ,bid()->bareRep()(),data().fullName()()
			     ,ex.msg()));
      }
    }// while
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long EditItem::testContent(String &msg)const 
{
  
  //HIER("EditItem::testContent");
  //HIER(constdata().fullName()());
  long lic;
  String v=value(lic);
  try{
    v.escapeEncode();
    test()->validate(v,lic,0L);//todo change liek toggleit
    v.escapeDecode();
    if(classname=="MSelectBoxEditItem"){
      // this is risky and also not implemented: setValue(0,v)
      ;
    }
  }catch (Except &ex){
    msg=Sprintf("screen item '%s', input variable '%s'='%s' not OK!\n %s"
		,bid()->bareRep()(),constdata().fullName()(),v(),ex.msg());
    return 0;
  }
  return 1;
}

// }}}


// {{{CheckGroupEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CheckGroupEditItem::calcFieldDimensions(long Ncolumns,long Autowidth
					     ,long cols,long lw,long fo
					     ,long lo[3],long Namepos[3])
{
  if(Ncolumns==2){
    if(Autowidth){
      lo[0]=LMAX((cols-1)/2-lw-2-3-Namepos[0],1);
      Namepos[1]=Namepos[0]+(cols-1)/2;
      lo[1]=LMAX((cols-1)-lw-2-3-Namepos[1],1);
    }else{
      lo[0]=LMAX(fo,1);
      Namepos[1]=Namepos[0]+(Namepos[0]+lw+2+3+lo[0]);
      lo[1]=LMIN(LMAX((cols-1)-lw-2-3-Namepos[1],1),lo[0]);
    }
  }else if(Ncolumns==3){
    if(Autowidth){
      lo[0]=LMAX((cols-1)/3-lw-2-3-Namepos[0],1);
      lo[1]=lo[0];
      lo[2]=lo[0];
      Namepos[1]=Namepos[0]+(Namepos[0]+lw+2+3+lo[0]);
      Namepos[2]=Namepos[1]+(Namepos[0]+lw+2+3+lo[1]);
    }else{
      lo[0]=LMAX(fo,1);
      Namepos[1]=Namepos[0]+(Namepos[0]+lw+2+3+lo[0]);
      lo[1]=LMAX(((cols-1)-3*(lw+2+3)-3*Namepos[0]-lo[0])/2,1);
      lo[2]=lo[1];
      Namepos[2]=Namepos[1]+(Namepos[0]+lw+2+3+lo[1]);
    }
  }else{
    if(Autowidth){
      lo[0]=LMAX(cols-1-lw-2-3-Namepos[0],1);
    }else{
      lo[0]=LMAX(fo,1);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CheckGroupEditItem::onLayout(Screen *s)
{
  //  NSF("onLayout");

  fo=bid()->offset()-1;

  if(inbox)
    namepos[0]=2;
  else
    namepos[0]=bid()->namePos()+ptab+1;

  calcFieldDimensions(ncolumn,autowidth,s->cols(),lw,fo,lo,namepos);//todo makeprivate vars

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

long CheckGroupEditItem::offset() const
{
  return lo[column];
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long CheckGroupEditItem::width() const
{
  return lw;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

long CheckGroupEditItem::namePos() const
{
  return namepos[column];
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CheckGroupEditItem::GetAutoHotkeyFromSet(String const &set,long i)
{
  
  String s;

  long ns=set.length();

  if(i>0){
    long n=i+1;
    while(n){
      long j=(n-1)%ns;
      n=(n-1)/ns;
      s+=set[j];
    }
  }else{
    s+=set[0];
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CheckGroupEditItem::markupRep()const
{
  //NSF("CheckGroupEditItem::markupRep");

  String v;
  if(pactive)
    v="@("+hotkey().upperCase()+") "+(constdata()[index]()()+1);
  else
    v=(constdata()[index]()()+1);
  return v;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CheckGroupEditItem::value(long &lic)const
{
  //NSF("CheckGroupEditItem::value");
  String v=" ";
  v.set(0,constdata()[index]()[0]);

  lic=1;

  return v;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CheckGroupEditItem::setValue(Form *,String const &v)
{
  //NSF("CheckGroupEditItem::setValue");
  String s=data()[index]();
  s.set(0,v[0]);
  data()[index]=s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CheckGroupEditItem::hotkey(long)const
{
  String alphanum_autokeys="0123456789abcdefghijklmnopqrstuvwxyz"; 
  String valid_set;

  if(inbox)
    valid_set=alphanum_autokeys.removeFromSet("xh");
  else
    valid_set=alphanum_autokeys;
  
  String h=GetAutoHotkeyFromSet(valid_set,index) ;
 
  if(!pactive) return "";
  
  if(inbox)
    return h;
  else
    return EditItem::hotkey()+h;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String CheckGroupEditItem::screenText()const
{
  //NSF("CheckGroupEditItem::screenText");
  String v=(constdata()[index]()()+1);
  return v;
}

// }}}

// {{{CheckBoxEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long CheckBoxEditItem::anyValueSet()
{
  //NSF("CheckBoxEditItem::anyValueSet");
  long ret=0;
  for(long ia=0;ia<data().size();ia++){
    if(data()[ia]()[0]=='+'){
      ret=1;
      break;
    }
  }

  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CheckBoxEditItem::paintValue(Screen *s,long y0
				  ,HighLightState highlightmode)
{
  if(!pshowvalue) return;

  long width=vWidth(s);

  String ss="none set";

  long i=ss.length();

  if(anyValueSet())
    ss="...";
  
  if(i>width+2){
    ss=ss.subString(0,LMAX(width-1,0));
    ss+="<<";
  }
  

  s->moveTo(ypos+y0,namePos()+offset()+3);

  paintHighlighted(s,ss,highlightmode);
  
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CheckBoxEditItem::edit(Form *f,long y0)
{
  NSF("CheckBoxEditItem::edit");
  if(!f) return;


  Form *box=new Form(f->application()
		     ,bid()->bareRep().upperCase(),f);

  Piper *piper=f->application()->s->piper();

  new MenuItem(box->menu(),"e@(x)it","exit","exit");
  new MenuItem(box->menu(),"@>","","");
  new MenuItem(box->menu(),"@(h)elp","help",data().fullName());
  
  long ok=1;
  String objname;
  if(className()=="CheckBoxEditItem"){
    CheckGroupItemDef *b=dynamic_cast<CheckGroupItemDef*>(bid());
    ok&=(b!=0);
    if(b)
      b->internalAddToForm(box,1,data(),1,ptab);
    objname="check box";
  }else if(className()=="MSelectBoxEditItem"){
    MSelectGroupItemDef *b=dynamic_cast<MSelectGroupItemDef*>(bid());
    ok&=(b!=0);
    if(b)
      b->internalAddToForm(box,1,data(),1,ptab);
    objname="multiple select box";
  }else if(className()=="SelectBoxEditItem"){
    SelectGroupItemDef *b=dynamic_cast<SelectGroupItemDef*>(bid());
    ok&=(b!=0);
    if(b)
      b->internalAddToForm(box,1,data(),1,ptab);
    objname="select box";
  }else
    throw TerminateExcept(Sprintf("internal(%s): unknown class '%s' !"
				  ,nsubfunc,className()()));

  if(!ok)
    throw TerminateExcept(Sprintf("internal(%s): the dynamic class"
				  " conversion for class '%s' failed!"
				  ,nsubfunc,className()()));
  
  if(piper){
    piper->message(Sprintf(" Entering %s '%s' in screen '%s'\n"
			   ,objname(),bid()->bareRep()(),f->screenName()()));
    piper->nextpipeline();
  }
  
  box->exec();

  //todo:  we do not have tests here

  delete box;


  f->show();
  
  
}

// }}}


// {{{SelectGroupEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SelectGroupEditItem::paint(Screen *s,long y0,HighLightState highlightmode)
{
  onLayout(s);
  
  long len=offset()+width()+3;
  if(width()<0){
    len=LMAX(s->cols()-namePos()-2+width(),offset()+1+3);
  }
  
  len+=2;
  clearArea(s,ypos+y0,namePos(),len,highlightmode);
  
  
  
  long wid=offset()+width()+2;
  String ss=" "+markupRep();
  
  if(ss.length()>wid){
    ss=ss.subString(0,wid-1);
    ss.set(wid-2,'<');
    ss.set(wid-1,'<');
    //    ss.set(wid  ,'\0');
  }
  
  
  
  s->moveTo(ypos+y0,namePos()+3);
  
  paintTokHighlighted(s,ss,highlightmode);

  
  s->moveTo(ypos+y0,namePos());
  if(valueSelected())   
    paintHighlighted(s,"[*]",highlightmode);
  else
    paintHighlighted(s,"[ ]",highlightmode);

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SelectGroupEditItem::markupRep()const
{
  
  String m;
  String prefix;
  if(pactive){
    if(inbox) 
      prefix="";
    else
      prefix=EditItem::hotkey();
    
    if(flags().contains("value_as_hotkey") ){
      m.sprintf("@(%s%ld) %s",prefix(),test()->testSet(index).i
		,test()->testSet(index).pbarerep());
    }else{
      if(flags().contains("print_value") )
	m.sprintf("%s (%ld)"
		  ,prependHotkey(test()->testSet(index).pmarkuprep(),prefix)()
		  ,test()->testSet(index).i);
      else
	m=prependHotkey(test()->testSet(index).pmarkuprep(),prefix);
      
    }
  }else{ // inactive
    
    if(flags().contains("value_as_hotkey") ){
      m=" "+test()->testSet(index).pbarerep;
    }else{
      if(flags().contains("print_value") )
	m.sprintf("%s (%ld)"
		  ,test()->testSet(index).pbarerep()
		  ,test()->testSet(index).i);
      else
	m=test()->testSet(index).pbarerep;
      
    }
  }
  return m;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SelectGroupEditItem::hotkey(long )const
{
  
  String h;
  String prefix;
  if(pactive){
    
    if(inbox) 
      prefix="";
    else
      prefix=EditItem::hotkey();
    
    if(flags().contains("value_as_hotkey") ){
      h.sprintf("%s%ld",prefix(),test()->testSet(index).i);
    }else{
      h=prefix+test()->testSet(index).photkey;
    }
  }else{//inactive
    h="";
  }
  return h;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SelectGroupEditItem::screenText()const
{
//   if(flags().contains("value_as_hotkey") )
//     return Sprintf("%ld %s",test()->testSet(index).i
// 			    ,test()->testSet(index).pbarerep());
//   else
    return test()->testSet(index).pbarerep;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long SelectGroupEditItem::valueSelected()
{
  return data().first()().convertToLong()==test()->testSet(index).i ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SelectGroupEditItem::edit(Form *f,long y0)
{
  //NSF("SelectGroupEditItem::edit");
  if(!f) return;

  Screen *s=f->application()->s;


  if(s->piper()){
    s->piper()->message(Sprintf(" Selecting select item '%s' in screen '%s'\n"
				,test()->testSet(index).pbarerep()
				,f->screenName()()));
    
    s->piper()->nextpipeline();
  }
  

  // deselect the selected one
  for (long i=0;i<f->pobjects.size();i++){ 
    BaseItem *b=f->pobjects[i];
    if(f-> objectVisible(i))
      if(b->bid()==bid() && b->className()==className()){
	SelectGroupEditItem *bs=dynamic_cast<SelectGroupEditItem*>(b);
	if(!bs)
	  throw TerminateExcept("dynamic_cast for SelectGroupEditItem failed!");
	if(bs->valueSelected()){
	  data().first()=test()->testSet(index).i;
	  s->moveTo(bs->ypos+y0,bs->namePos());
	  paintHighlighted(s,"[ ]",NoHighLighting);
	  
	}
      }
  }

  data().first()=test()->testSet(index).i; /* this we need in case that
					      the initial data are in valid
					      or invisible */


  // select the new one
  for (long i=0;i<f->pobjects.size();i++){ 
    BaseItem *b=f->pobjects[i];
    if(b->bid()==bid() && b->className()==className()){
      SelectGroupEditItem *bs=dynamic_cast<SelectGroupEditItem*>(b);
      if(bs->valueSelected()){
 	s->moveTo(bs->ypos+y0,bs->namePos());
	paintHighlighted(s,"[*]",NoHighLighting);
      }
    }
  }

  Dobj da=data().first().next();
  SelectTest *t=dynamic_cast<SelectTest*>(bid()->test());
  if(!t) throw TerminateExcept("dynamic_cast for SelectTest failed\n");
  da=t->getSelectDescriptor(data().first()());


}

// }}}


// {{{SelectBoxEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SelectBoxEditItem::paintValue(Screen *s,long y0
				   ,HighLightState highlightmode)
{
  
  if(!pshowvalue) return;
   long wid=vWidth(s);


   String ss;
   

   SelectTest const *t=dynamic_cast<SelectTest const*>(test());
   if(!t) throw TerminateExcept("dynamic_cast for SelectTest failed\n");
   
   long is=1;
   try{
     ss=t->getSelectDescriptor(data().first()()); 
   }catch(...){is=0;};
   
   
   if(!is){
     ss="(invalid)";
   }else{
     if(flags().contains("print_value") )
       ss.sprintf("%s (%s)",ss(),data().first()()());
     
   }
   
    

   long i=ss.length();
   
   if(i>wid+2){
     ss=ss.subString(0,LMAX(wid-1,0));
     ss+="<<";
   }
   
   
   s->moveTo(ypos+y0,namePos()+offset()+3);
   
   paintHighlighted(s,ss,highlightmode);
   
}

// }}}


// {{{MSelectGroupEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MSelectGroupEditItem::markupRep()const
{
  //NSF("MSelectGroupEditItem::markupRep");

  if(pactive){
    String prefix;
    if(inbox) 
      prefix="";
    else
      prefix=EditItem::hotkey().upperCase();
    
    return prependHotkey(bid()->test()->testSet(index).pmarkuprep(),prefix);
  }else{// not active
    
    return bid()->test()->testSet(index).pbarerep();
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MSelectGroupEditItem::hotkey(long)const
{
  String prefix;
  if(!pactive) return "";

  if(inbox) 
    prefix="";
  else
    prefix=EditItem::hotkey();

  return prefix+bid()->test()->testSet(index).photkey;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MSelectGroupEditItem::screenText()const
{
  return bid()->test()->testSet(index).pbarerep;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String MSelectGroupEditItem::value(long &count)const
{
  count =1;
  for(long ia=0;ia<constdata().size();ia++){
    long l=constdata()[ia].first()().convertToLong();
    if(bid()->test()->testSet(index).i == l){
      return "+";
      break;
    }
  }
  
  return "-";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MSelectGroupEditItem::setValue(Form *,String const &v)
{
  NSF("MSelectGroupEditItem::setValue");

  
  ValueArray<long> ipo;
  long found=0;
  for(long ia=0;ia<data().size();ia++){
    long l=data()[ia].first()().convertToLong();
    if(bid()->test()->testSet(index).i != l || v=="+")
      ipo.append(l);
    
    if(bid()->test()->testSet(index).i == l )
      found=1;
  }

  if(v=="+" && !found)
    ipo.append(bid()->test()->testSet(index).i);

  
  ipo.unify();

  
  try{ data().resizeSap(ipo.size()); } catch(Except &ex) {
    throw TerminateExcept(Sprintf("internal(%s): resizing of "
					   "struct array '%s' failed\n%s"
					   ,nsubfunc,data().fullName()()
					   ,ex.msg()));
  }

  for(long i=0;i<ipo.size();i++){
    Dobj di=data()[i];
    Dobj ds=di.first();
    ds=ipo[i];
    ds=ds.next();
    String s;
    long ok=1;

    MSelectTest *t=dynamic_cast<MSelectTest*>(bid()->test());
    if(!t) throw TerminateExcept("dynamic_cast for MSelectTest failed\n");

    try{ s=t->getSelectDescriptor(ipo[i]);  }catch(...){ok=0;}
    if(ok)
      ds=s;
    
    
  }
  
}

// }}}

// {{{CounterEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CounterEditItem::setValue(Form *f,String const &v)
{
  NSF("CounterEditItem::setValue");
  String s=v;
  
  
  long l;
  try{ l=v.convertToLong(); } catch(...){
    throw Except(String().sprintf("internal(%s): counter value"
				  " does not contain a"
				  " valid integer value (%s)!"
				  ,nsubfunc,v()));
  }

  
  if( (data().size()!=l) ){
    try{
      data().resizeSap(l);
    }catch(Except &ex){
      throw Except(String().sprintf("internal(%s): resizing of structarray '%s'"
				    " failed: %s\n"
				    ,nsubfunc,data().fullName()(),ex.msg()));
    }
    
    f->contentChanged(); /* Design flaw: we have to honor the changes here, even
			    if the changes are reverted back, we cannot
			    keep track of it, since the objects get recreated
			    and their former change status gets lost. */
    //todo: when in non-SubMenuForm we need to reload
    f->notify(0,"registerhook","loadItems");
    f->paintObjects();
  }  
}

// }}}


// {{{CompoundElementEditItem

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CompoundElementEditItem::paint(Screen *s,long y0
				    ,HighLightState highlightmode)
{
  //NSF("CompoundElementEditItem::paint");
  
  String ss=markupRep();

  long pos=namePos();
  
  clearArea(s,ypos+y0,pos,ss.length(),highlightmode);
  
  s->moveTo(ypos+y0,pos);
  
  paintTokHighlighted(s,ss,highlightmode);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CompoundElementEditItem::edit(Form *f,long y0)
{
  NSF("CheckBoxEditItem::edit");
  if(!f) return;

  String helpname=data().fullName().breakup("[")[0];

  String title=(bid()->bareRep()+" "+hotkey(0)).upperCase();

  Form *box=new Form(f->application(),title,f);

  Piper *piper=f->application()->s->piper();
  
  new MenuItem(box->menu(),"e@(x)it","exit","exit");
  new MenuItem(box->menu(),"@>","","");
  new MenuItem(box->menu(),"@(h)elp","help",helpname);
  
  long ok=1;
  if(className()=="CompoundElementEditItem"){
    CompoundItemDef *b=dynamic_cast<CompoundItemDef*>(bid());
    ok&=(b!=0);
    if(b){
      box->setCompoundItemDef(b,&data());
      b->internalAddToForm(box,1,data(),1,ptab);
    }
  }else
    throw TerminateExcept(Sprintf("internal(%s): unknown class '%s' !"
				  ,nsubfunc,className()()));
  
  if(!ok)
    throw TerminateExcept(Sprintf("internal(%s): the dynamic class"
				  " conversion for class '%s' failed!"
				  ,nsubfunc,className()()));
  
  if(piper){
    piper->message(Sprintf(" Entering Compound element '%s' in screen '%s'\n"
			   ,title(),f->screenName()()));
    piper->nextpipeline();
  }

  box->exec();

  //todo:  we do not have tests here

  delete box;


  f->notify(0,"registerhook","loadItems");
  f->show();
  
  
}

// }}}

/*EOF*/ 
