
/* @(#)Item.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ITEMXX_H
#define _ITEMXX_H 1


#include "FString.hh"
#include "ScreenDefs.hh"
#include "Test.hh"
#include "Page.hh"

class Screen;
class Form;




/* ***********************************************************************
 *
 * ***********************************************************************
 */
class BaseItem
{
public:
  BaseItem(String const &cn="BaseItem",long tab=0):classname(cn),ypos(0)
    ,ptab(tab),pid(0){};
    BaseItem(BaseItemDef *id,long tab,String const &cn="BaseItemDef")
    :classname(cn),ypos(0),ptab(tab),pid(id){};
  virtual ~BaseItem(){};
  //
  String const &className()const {return classname;};
  //
  virtual String screenText()const=0;
  virtual String hotkey(long supress=1)const{return "";};
  //
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting)=0;
  virtual void edit(Form *,long y0){};
  //
  virtual BaseItemDef *bid()const {return pid;};
  //
  void setY(long y){ypos=y;};
  //
  long y()const {return ypos;};
  //
  virtual long contentChanged()const {return 0;};
  virtual long testContent(String &msg)const {return 1;};
  //
 protected:
  //
  char clearArea(Screen *s,long y,long x,long len
		 ,HighLightState h=NoHighLighting);
  //
  void paintHighlighted(Screen *s,String const &text
			,HighLightState h=NoHighLighting);
  void paintTokHighlighted(Screen *s,String const &text
			   ,HighLightState h=NoHighLighting);
  //
 protected:
  String classname;
  long ypos;
  long ptab;
 private:
  BaseItemDef *pid;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class LineItem: public BaseItem
{
 public:
  LineItem(String const &s,String const &cn="LineItem")
    :BaseItem(cn,0),text(s){};
  virtual ~LineItem(){};
  //
  virtual String screenText()const{return text;};
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
public:
  String text;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class HelpLineItem: public LineItem
{
 public:
  HelpLineItem(String const &s):LineItem(s,"HelpLineItem"){};
  virtual ~HelpLineItem(){};
  //
  virtual String screenText()const;
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class DashLineItem: public BaseItem
{
public:
  DashLineItem(BaseItemDef *id,long tab)
    :BaseItem(id,tab,"DashLineItem"){};
  virtual ~DashLineItem(){};
  //
  virtual String screenText()const{return "";};
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class TitleItem: public BaseItem
{
public:
  TitleItem(BaseItemDef *id,long tab)
    :BaseItem("TitleItem",tab)
    ,ptext(id->bareRep())
    ,pnamepos(id->namePos())
    ,poffset(id->offset())
    ,pwidth(id->width())
  {};
  //
    TitleItem(String const &text,long np,long w,long tab)
      :BaseItem("TitleItem",tab),ptext(text),pnamepos(np),poffset(0)
      ,pwidth(w){};
  //
  virtual ~TitleItem(){};
  //
  virtual long offset()const {return poffset;};
  virtual long namePos()const {return pnamepos+ptab;};
  virtual long width()const {return pwidth;};
  virtual String screenText()const{return ptext;};
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
public:
  String ptext;
  long pnamepos;
  long poffset;
  long pwidth;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class EditItem: public BaseItem
{
public:
  EditItem(BaseItemDef *id,Dobj const &d,long active,long tab
	   ,String const &cn="EditItem")
    :BaseItem(id,tab,cn),photkey(id->hotkey()),pmarkuprep(id->markupRep())
    ,pdata(d),pactive(active),printcolon(1),pshowvalue(0),cutoffset(1)
  {
    if(!pactive){
      photkey="";
      pmarkuprep=id->bareRep();
    }
    if(cn=="EditItem")
      save();
  };
  virtual ~EditItem(){};
  //
  virtual String screenText()const{return bid()->bareRep();};
  virtual String hotkey(long supress=1)const{ 
    if( ( supress && !bid()->isFirstFieldItem() ) || 
        (!supress && flags().contains("fixed"))  )
      return "";
    else
      return photkey; 
  };
  //
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  virtual void edit(Form *,long y0);
  //
  virtual long contentChanged()const {  return (psave!=value()); };
  virtual long testContent(String &msg)const; 
  //
protected:
  //
  //
  virtual void onLayout(Screen *){};
  //
  virtual void save(){  psave=value(); };
  //
  Dobj &data()  {return pdata;};
  Dobj const &constdata() const  {return const_cast<Dobj&>(pdata);};
  StringList const &flags()const {return bid()->flags();};
  virtual Test const *test()const {return bid()->test();};
  //
  virtual long width()const {return bid()->width();};
  virtual long offset()const {return bid()->offset();};
  virtual long namePos()const {return bid()->namePos()+ptab;};
  virtual String markupRep()const{return pmarkuprep;};
  virtual String value(long &count)const ;
  virtual String value()const {long lic;return value(lic);};
  //
  virtual void setValue(Form *f,String const &v);
  //
  long vWidth(Screen *s);
  //
  virtual void paintValue(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
  void paintValueCheckItem(Screen *s,long y0
				   ,HighLightState h=NoHighLighting);

protected:
  // we need local copies for FIELDs
  String photkey;
  String pmarkuprep;
  Dobj pdata;
  long pactive;
  String psave;
  long printcolon;
  long pshowvalue;
  long cutoffset;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CheckGroupEditItem: public EditItem
{
public:
  CheckGroupEditItem(BaseItemDef *id,Dobj const &d,long active
		     ,long i,long c,long nc,long aw,long ib,long tab
		     ,String const &cn="CheckGroupEditItem")
    :EditItem(id,d,active,tab,cn),index(i),ncolumn(nc),column(c),autowidth(aw)
    ,inbox(ib)
    {
      lw=1;
      if(cn=="CheckGroupEditItem")
	save();
    };
  //
  virtual String screenText()const;
  virtual String hotkey(long supress=1)const;
  //
  //
protected:
  virtual void onLayout(Screen *);
  //
  virtual long testContent(String &msg)const{return 1;}; 
  virtual long contentChanged()const {return 0;} 
  //
  virtual long offset()const;
  virtual long width()const;
  virtual long namePos()const;
  virtual String markupRep()const;
  virtual String value(long &count)const;
  //
  virtual void setValue(Form *f,String const &v);
  //
  static String GetAutoHotkeyFromSet(String const &set,long i);
  //
private:
  //
  static void calcFieldDimensions(long ncolumns,long autowidth,long cols
				  ,long lw,long fo
				  ,long lo[3],long namepos[3]);
  //
protected:
  long lw;
  long index;
private:
  long ncolumn,column,autowidth;
  long fo,lo[3],namepos[3];
protected:
  long inbox;
};





/* ***********************************************************************
 *
 * ***********************************************************************
 */
class SelectGroupEditItem: public CheckGroupEditItem
{
public:
  SelectGroupEditItem(BaseItemDef *id,Dobj const &d,long active
		      ,long i,long c,long lc,long aw,long ib,long tab)
    :CheckGroupEditItem(id,d,active,i,c,lc,aw,ib,tab,"SelectGroupEditItem"){
    lw=0;  EditItem::save(); };
  //
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  virtual void edit(Form *,long y0);
  virtual String screenText()const; 
  virtual String hotkey(long supress=1)const; 
  //
protected: 
  // 
  //
  virtual String value(long &count)const{count=1;
    return const_cast<Dobj&>(constdata()).first()();};
  //
  virtual String markupRep()const; 
  // 
  // 
private: 
  long valueSelected();

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class MSelectGroupEditItem: public CheckGroupEditItem
{
public:
  MSelectGroupEditItem(BaseItemDef *id,Dobj const &d,long active
		       ,long i,long c,long lc,long aw,long ib,long tab)
    :CheckGroupEditItem(id,d,active,i,c,lc,aw,ib,tab,"MSelectGroupEditItem"){
    save();};
  //
  virtual String screenText()const;
  virtual String hotkey(long supress=1)const;
  //
  //
protected:
  //
  virtual String markupRep()const;
  virtual String value(long &count)const;
  virtual Test const *test()const {return &TestFlag;};
  virtual void paintValue(Screen *s,long y0,HighLightState h=NoHighLighting){
    paintValueCheckItem(s,y0,h);};
  //
  virtual void setValue(Form *f,String const &v);
  //
  // 
  // 
private: 

};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CheckBoxEditItem: public EditItem
{
public:
  CheckBoxEditItem(BaseItemDef *id,Dobj const &d,long active,long showvalue
		   ,long np,long tab
		   ,String const &cn="CheckBoxEditItem")
    :EditItem(id,d,active,tab,cn),pnamepos(np) 
  {
    pshowvalue=showvalue;
    if(!pshowvalue) cutoffset=0;
    printcolon=showvalue;
    
    if(cn=="CheckBoxEditItem")
      save();
  };
  //
  virtual void edit(Form *,long y0);
/*   virtual String hotkey(long supress=1)const{ */
/*     if(pshowvalue)  */
/*       return EditItem::hotkey(supress); */
/*     else */
/*       return ""; */
/*   } */
  //
protected:
  //
  //
  virtual long testContent(String &msg)const{return 1;}; 
  //
/*   virtual String markupRep()const{ */
/*     if(pshowvalue)  */
/*       return EditItem::markupRep(); */
/*     else */
/*       return bid()->bareRep(); */
/*   } */
  //
  virtual String value(long &count)const{count=1;
    String v="";
    for(long i=0;i<constdata().size();i++) v+=constdata()[i]()+" ";
    return v;
  }
  //
  virtual long namePos()const{return pnamepos+ptab;};
  virtual long width()const{return 10;};
  virtual void paintValue(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
  virtual long anyValueSet();
  //
protected:
  long pnamepos;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class SelectBoxEditItem: public CheckBoxEditItem
{
public:
  SelectBoxEditItem(SelectGroupItemDef *id,Dobj const &d,long active
		    ,long showvalue,long np,long tab)
    :CheckBoxEditItem(id,d,active,showvalue,np,tab,"SelectBoxEditItem")
  {save();};
  //
  //
protected:
  //
  //
  virtual long width()const{return bid()->width();};
  virtual void paintValue(Screen *s,long y0,HighLightState h=NoHighLighting);
  //
  virtual String value(long &count)const{count=1;
    return const_cast<Dobj&>(constdata()).first()();};
  virtual long testContent(String &msg)const{return EditItem::testContent(msg);}; 
  //
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class MSelectBoxEditItem: public CheckBoxEditItem
{
public:
  MSelectBoxEditItem(MSelectGroupItemDef *id,Dobj const &d,long active
		     ,long showvalue,long np,long tab)
    :CheckBoxEditItem(id,d,active,showvalue,np,tab,"MSelectBoxEditItem"){save();};
  //
protected:
  //
  virtual long anyValueSet(){return data().size()>0;};
  //
  virtual String value(long &count)const{count=1;
    String v="";
    for(long ia=0;ia<constdata().size();ia++)
      v+=constdata()[ia].first()()+" ";
    return v;
  };
  //
  virtual long testContent(String &msg)const{return EditItem::testContent(msg);}; 
  //
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CounterEditItem: public EditItem
{
public:
  CounterEditItem(BaseItemDef *id,Dobj const &d,long active,long tab
		  ,String const &cn="CounterEditItem") 
    :EditItem(id,d,active,tab,cn)   { if(cn=="CounterEditItem") save(); };
  virtual ~CounterEditItem(){};
  //
  //
protected:
  //
  //
  // virtual long testContent(String &msg)const{return 1;}; 
  // 
  virtual String value(long &count)const{count=1; return constdata().size();};
  //
  virtual void setValue(Form *f,String const &v);
  // 
  
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CompoundElementEditItem: public EditItem
{
public:
  CompoundElementEditItem(BaseItemDef *id,Dobj const &d,long active
			  ,String const &h,long tab
			  ,String const &cn="CompoundElementEditItem")
    :EditItem(id,d,active,tab,cn)
  {
    photkey=h.lowerCase();pmarkuprep="@("+photkey.upperCase()+")";
    if(!pactive){
      photkey="";
      pmarkuprep="("+h.lowerCase().upperCase()+")";
    }
  };
  virtual ~CompoundElementEditItem(){};
  //
  virtual String screenText()const{return "";};
  virtual String hotkey(long )const{return photkey;};
  //
  virtual void paint(Screen *s,long y0,HighLightState h=NoHighLighting);
  virtual void edit(Form *,long y0);
  //
protected:
  //
  virtual long contentChanged()const {return 0;};
  //
  virtual long testContent(String &msg)const{return 1;}; 
  //
  virtual long namePos()const {return bid()->namePos()+ptab;};
  //
};

#endif /* _ITEMXX_H */

