/*************************************************************************
 * @(#)LineEdit.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   22 Oct 2008
 *************************************************************************
 */


#include "Keys.hh"
#include "FString.hh"
#include "LineEdit.hh"
#include "Page.hh"




/* the macro version  calls j twice !! */
static long LMIN(long i,long j)
{
  return ( ((i)<j?i:j) );
}

/* the macro version  calls j twice !! */
static long LMAX(long i,long j)
{
  return ( ((i)>j?i:j) );
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSearchLineEdit::backAction(Event &e)
{
  if(pparent){
    app->s->reverseOff();
    
    Page *p=dynamic_cast<Page*>(pparent);
    p->searchCallBack(e,buffer);
    //	break; 	/* unclassified keys are escape like */
    
    app->s->reverseOn(); 
    
    //      if(e.key==S_CTRL_R){
    //	paint(); 
    //      }
    
    app->s->flush(xpos+1+cursorpos,yPos()); 
    
  }
  
}
















/* ***********************************************************************
 *
 * ***********************************************************************
 */
LineEdit::LineEdit(App *a,long x,long y,long bl,SObject *p,String const &cn)
  :SObject(a,p,cn),xpos(x),ypos(y),boxlength(bl)
  ,emptybuffer(Sprintf("%*c",boxlength,' ')),first(1)
{
  app->s->reverseOn(); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
LineEdit::~LineEdit()
{
  //NSF("~LineEdit");
  //  HIER(subfunc);
  app->s->reverseOff(); 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

long LineEdit::yPos()
{
  if(ypos<0)
    return app->s->rows()+ypos;
  else
    return ypos;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

ExecStatus LineEdit::exec()
{
  Piper *piper=app->s->piper();

  long len=buffer.length();
  
  if(len<boxlength) { 
    stringpos=0;
    cursorpos=len; 
  }else{
    cursorpos=boxlength-1; 
    stringpos=len-cursorpos;
    if( boxlength ==1) stringpos--;
  } 

  paint();

  Event e;

  ExecStatus ret=Discard;

  while(1){
    e=app->getKey();


    onKeyEvent(e);


    backAction(e);
    
    if(e.key==S_CTRL_R) {
      paint();
    }



    if(e.key==S_ENTER) {
      if(piper && buffer.length()==0){
	buffer=save;
	ret=Discard;
	break;
      }  
      ret=Accept;
      break;
    }else if(e.key==S_ESCESC || e.key==S_CTRL_G) {
      buffer=save;
      break;
    }
    
    
  }
  
  app->s->reverseOff(); 
  
  
  if(piper){
    piper->copyExpandPlaceHolders(buffer,save);
  }
  

  return ret;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void LineEdit::paint()
{
  
  Screen *s=app->s;
  s->moveTo(yPos(),xpos); 
  s->addCh('['); 
  s->moveTo(yPos(),xpos+boxlength+1); 
  s->addCh(']'); 


  paintBuffer();

  s->moveTo(yPos(),xpos+1+cursorpos); 

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LineEdit::paintBuffer()
{

  Screen *s=app->s;

  s->moveTo(yPos(),xpos+1);
  s->addStr(emptybuffer);

  s->moveTo(yPos(),xpos+1);
  s->addStr(buffer.subString(stringpos,stringpos+boxlength-1));
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long LineEdit::onKeyEvent(Event e)
{


  if(e.key==S_ENTER)  {
    ;
  }else if(e.key== S_BACKSPACE){
    
    first=0; 
    if(stringpos+cursorpos>0){
      buffer.removeSection(stringpos+cursorpos-1,stringpos+cursorpos-1);
      if(cursorpos>0){ 
	cursorpos--; 
      }else if(stringpos>0){ 
	long sh=LMAX((int)(boxlength/2),1);
	long n=LMIN(sh,stringpos); 
	stringpos-=n; 
	cursorpos+=n-1; 
      }
      paintBuffer(); 
    }
  }else if(e.key== S_KEY_LEFT){
    first=0; 
    if(cursorpos>0){ 
      cursorpos--; 
    }else if(stringpos>0){ 
      long sh=LMAX((int)(boxlength/2),1);
      long n=LMIN(sh,stringpos); 
      stringpos-=n; 
      cursorpos+=n-1; 
      paintBuffer(); 
    } 
  }else if(e.key== S_CTRL_K){
    first=0; 
    if(stringpos+cursorpos<buffer.length()){
      buffer.removeSection(stringpos+cursorpos,buffer.length());
      paintBuffer(); 
    }
  }else if(e.key== S_KEY_RIGHT){ 
    long len=buffer.length();
    first=0; 
    if((cursorpos<boxlength-1) && (cursorpos<(len-stringpos))){ 
      cursorpos++; 
    }else if(cursorpos==boxlength-1){ 
      if (stringpos+cursorpos<len) stringpos++; 
      paintBuffer(); 
    }; 
  }else if(e.key== S_DELETE){
    
    first=0; 
    if(stringpos+cursorpos<buffer.length()){
      buffer.removeSection(stringpos+cursorpos,stringpos+cursorpos);
      paintBuffer(); 
    }
  }else if(e.key== S_HOME){
    
    first=0; 
    stringpos=0; 
    paintBuffer(); 
    cursorpos=0; 
  }else if(e.key== S_END){
    
    long len=buffer.length();
    first=0; 
    if(len<boxlength) { 
      stringpos=0;cursorpos=len; 
    }else{ 
      stringpos=len-boxlength+1;
      cursorpos=boxlength-1; 
    } 
    paintBuffer(); 
  }
  
  if((e.key>31) &&(e.key<129) ){
    if (first){ 
      buffer="";
      stringpos=0;
      cursorpos=0; 
      paintBuffer(); 
      first=0; 
    }; 
    
    buffer.insertChar(stringpos+cursorpos,(char)e.key);
    if(cursorpos==boxlength-1) stringpos++; 
    paintBuffer(); 
    if(cursorpos<boxlength-1) cursorpos++; 
    
  }

  app->s->flush(xpos+1+cursorpos,yPos()); 
  
  
  return 1;
}

/*EOF*/ 
