/*************************************************************************
 * @(#)Menu.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 Oct 2008
 *************************************************************************
 */

//#include <stdio.h>



#include "Keys.hh"
#include "debugxx.hh"
#include "Menu.hh"
#include "FString.hh"
#include "Page.hh"
#include "Screen.hh"
#include "Except.hh"




// {{{MenuItem


/* ***********************************************************************
 *
 * ***********************************************************************
 */
MenuItem::MenuItem(Menu *p,String const &c,String const &a,String const &d)
  : classname("MenuItem"),parent(p),hotkey(""),screenrep(""),markuprep("")
  ,pdata(d),separator(0),paction(a)
{
  NSF("MenuItem");
  if(!p) THR(" argument parent is zero!");

  init(c);

  parent->addMenuItem(this);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MenuItem::MenuItem(String const &c,String const &a,String const &d)
  : classname("MenuItem"),parent(0),hotkey(""),screenrep(""),markuprep("")
  ,pdata(d),separator(0),paction(a)
{
  init(c);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MenuItem::init(String const &c)
{
  NSF("MenuItem::init");
  
  
  
  String s=c; s.stripBlanks();

  
  
  if(s=="@_"){
    hotkey="";
    screenrep="[]";
    markuprep="@_";
  }else if(s=="@>"){
    hotkey="";
    screenrep="";
    markuprep="";
    separator=1;
  }else{
    
    StringList sl=s.breakup("@(");
    //for(long i=0;i<sl.size();i++){ HIER(sl[i]()); }
    
    
    if(sl.size()!=2) 
      THR2(" argument '%s' is not valid!",s());
    
    screenrep=sl[0]; 
    markuprep=sl[0]+"@";

    
    StringList sl1=sl[1].breakup(")");

    if(sl1.size()!=2) 
      THR2(" argument '%s' is not valid!",s());

    //for(long i=0;i<sl1.size();i++){ HIER(sl1[i]()); }
    if(sl1.size()==2){
      hotkey=sl1[0].lowerCase();
      
      screenrep+="("+hotkey.upperCase()+")"+sl1[1];
      markuprep+="("+hotkey.upperCase()+")"+sl1[1];

      
    }
  }  

  if(hotkey.length()>1)
    THR2(" '%s': menu items cannot have multikey hotkeys!",c());




}


// }}}


// {{{Menu


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Menu::clear()
{
  //NSF("clear()");
  //HIER(subfunc);

  menuitems.clear();

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Menu::paint(Screen *s)
{
  
  long tab1=s->cols(); 
  for(long i=0;i<menuitems.size();i++){
    tab1-=menuitems[i]->screenrep.length()+1;
  }
  
  s->moveTo(1,0); 
  s->addCh(' ');
  s->moveTo(1,1); 
  

  for(long i=0;i<menuitems.size();i++){
    
    if(! menuitems[i]->separator ){
      
      s->prtToken(menuitems[i]->markuprep,Screen::StyleMenuHigh
		  ,Screen::StyleNormal);  
      s->addCh(' ');
      
    }else{
      for (long j=0;j<tab1;j++) {
	s->addCh(' ');
      }
    }
  }
  s->flush(0,3); 
  
  



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Menu::onKeyEvent(Event e)
{

  if(!page) return 0;
  
  if(!menuitems.size()) return 0;

  String ss;
  if(String::isPrint(e.key)) {
    ss.sprintf("%c",e.key).toLower();
  }
  
  Piper *piper=page->application()->s->piper();
  
  for (long i=0;i<menuitems.size();i++){ 
    if (!ss.empty() && menuitems[i]->hotkey==ss){ 


      if(piper){
	/*eat up keys */
	String keysequence;
	while(e.key != '\n' ){
	  keysequence+=(char)e.key;
	  e=page->application()->getKey();
	}
	if(keysequence.length()>1){
	  piper->message(Sprintf(" WARNING: Menu hotkey sequence '%s' longer"
				 " than 1 in screen '%s'\n",keysequence()
				 ,page->screenName()()));
	  
	}
	piper->message(Sprintf(" Entering/executing menu entry '%s' in"
			       " screen '%s'\n"
			       ,menuitems[i]->screenrep()
			       ,page->screenName()()));
		       
	
	piper->nextpipeline();
      }      

      page->notify(0,menuitems[i]->paction,menuitems[i]->pdata);
      return 1;
    }
  }

  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */


void AltMenu::paint(Screen *s)
{
  
  s->moveTo(s->rows()-1,1);
  s->deleteLine(s->rows()-1);
  
  String ss="CHOOSE: ";
  long l=ss.length();
  
  s->addStr(ss);
  
  
  String contmarker="[...]";
  
  long j=amenuloop;
  for(long i=amenuloop;i<menuitems.size();i++){
    if( menuitems[i]->separator ){
      j=i; continue;
    }
    l+=menuitems[i]->screenrep.length();
    if(i==menuitems.size()-1)
      if(l>=s->cols()) 
	break;
    
    
    if(i<menuitems.size()-1){
      l++;
      if(l+contmarker.length()>=s->cols()) 
	break;
      
    }
    
    j=i;
    
  }
  
  
  
  
  for(long i=amenuloop;i<=j;i++){
    
    if( menuitems[i]->separator )
      continue;
    
    s->prtToken(menuitems[i]->markuprep,Screen::StyleMenuHigh
		,Screen::StyleNormal);
    s->addCh(' ');
    
    if(i==menuitems.size()-1)
      amenuloop=i+1;
  }
  
  amenuloop=j+1;
  
  long more=(amenuloop<menuitems.size());
  
  if(more)
    s->addStr(contmarker);
  
  
  s->flush(0,3); 
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */


void AltMenu::exec()
{

  if(!page) return;
  
  if(!menuitems.size()) return;

  amenuloop=0;

  App *app=page->application();  
  Screen *s=app->s;
  Piper *piper=s->piper();

  if(piper){
    
    
    /*eat up keys */
    String keysequence;
    Event e(' ');
    while(e.key != '\n' ){
      keysequence+=(char)e.key;
      e=page->application()->getKey();
    }
    if(keysequence.empty()){
      piper->message(Sprintf(" ERROR: Submenu hotkey sequence '%s' does not"
			     " contain a hotkey after space in screen '%s'\n"
			     ,keysequence(),page->screenName()()));
      throw TerminateExcept("Abort");
    }
    String ss=keysequence;
    ss.removeLeadingBlanks();
    if(ss.length()>2){
      piper->message(Sprintf(" WARNING: Submenu hotkey sequence '%s' longer"
			     " than 2 in screen '%s'\n"
			     ,keysequence(),page->screenName()()));
      
    }
    e.key=ss[0];
    onKeyEvent(e);
    return;
  }

  while(1){

    paint(s);
    
    Event e=app->getKey();

    if(e.key == S_CTRL_R){
      if(page) page->onResize();
      break;
    }else if(e.key == S_SPACE){
      long more=(amenuloop<menuitems.size());
      if(!more) break;
    }else{

      onKeyEvent(e);
      break;
      
    }
    
    
  }
}

// }}}


/*EOF*/ 
