/*************************************************************************
 * @(#)Page.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 Oct 2008
 *************************************************************************
 */

//#include <stdio.h>
//#include "debugxx.h"
#include "Item.hh"
#include "Page.hh"
#include "Keys.hh"
#include "LineEdit.hh"
#include "Viewer.hh"

/* the macro version  calls j twice !! */
static long LMAX(long i,long j)
{
  return ( ((i)>j?i:j) );
}


static const  String STATUS[6]= { "STATUS: " ,"MESSAGE: ","WARNING: "
				  ,"ERROR: " ,"CHOOSE: ",""}; 

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecStatus StatusBar::exec()
{

  Event e;
  
  
  while(1){
    e=app->getKey();
    
    if(e.key == S_CTRL_R){
      if(pparent) pparent->onResize();
      else paint();
    }else if(e.key == 'y' || e.key == 'Y'){
      return Accept; 
    }else if(e.key == 'n' || e.key == 'N'){
      return Discard; 
    }else if(e.key == S_ESCESC || e.key == S_CTRL_G){
      return Abort; 
    }
    
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void StatusBar::paint()
{
  Screen *s=app->s;
  long lrow=s->rows()-1; 

  String out=STATUS[status]+statusmsg;


  s->deleteLine(lrow); 
  s->moveTo(lrow,1); 

  if(status == StatusChoose)
    out+=" (y/n):"; 
    
  long len=out.length()+ rstatusmsg.length();
  long fill=s->cols()-1-len;
  
  if(status != StatusChoose && status != StatusUser){
    //s->addString(" %s",rstatusmsg,s->cols()-rstatusmsg.length(),lrow); 
    for(long i=0;i<fill;i++)
      out+=" ";
    if(fill>0) out+=rstatusmsg;
  }

  s->moveTo(lrow,1); 
  if(out.length() > s->cols()-1){
    out.shorten(s->cols()-3);
    out+=" ...";
  }
  s->prtMarkUpLine(out);
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
Page::Page(App *a,String const &title,SObject *p,String const &cn)
  : SObject(a,p,cn),screenname(title),pmenu(new Menu(this))
  ,paltmenu(new AltMenu(this))
  ,statusbar(new StatusBar(a,this))
  ,currentline(0),adrb(3),adre(3)
  ,nlines(0),startatbottom(0),pexit(0),highlightedobject(-1),foundline(0)
  ,latestfoundobject(-1)
{

  Page *pp;
  if(p && (pp=dynamic_cast<Page*>(p)) ){
    statusBar()->setRightStatus(pp->statusBar()->rightStatusMsg());
  }

  clear();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Page::~Page()
{
  //  NSF("~Page");
  //  HIER(subfunc);
  

  delete statusbar;
  delete pmenu;
  delete paltmenu;

  clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::clearItems()
{ 
  //NSF("clearItems");
  //HIER(subfunc);

  nlines=0;  
  
  highlightedobject=-1;  
  latestfoundobject=-1;
  foundline=0;


  
  pobjects.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::clear()
{ 
  //  cleanHotkeyTable(&p->hotkeys);

  clearItems();
  

}
  




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::paint()
{
  //HIER("Page::paint");
  
  onLayout();

  Screen *s=app->s;
  // memorize the current screen atributes
  s->initialize(); 
  // clear the whole screen
  s->clrscr(0,s->rows()-1); 
  {//title
    s->deleteLine(0); 
    long x=( (s->cols()-screenname.length() )/2 ); 
    s->addString("%s",screenname,x,0); 
  }
  pmenu->paint(s);
  statusbar->paint();
  paintObjects();


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::paintFrame()
{
  Screen *s=app->s;
  long ade=s->rows()-adre;
  int L=ade-adrb+1;


  s->setStyle(Screen::StyleNormal,Screen::AttSet); 

  s->moveTo(adrb-1,0); 
  for(long i=0;i<s->cols();i++)s->addHLIN();  

  s->moveTo(ade+1,0); 
  for(long i=0;i<s->cols();i++)s->addHLIN();  
  
  if(currentline>0) {
    s->addString("%5s","[...]",1,adrb-1); 
  }
  if(nlines>currentline+L){
    s->addString("%5s","[...]",1,ade+1); 
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::paintObjects() 
{ 
  //HIER("Page::paintObjects");

  Screen *s=app->s;

  s->clrscr(adrb,s->rows()-adre); 

  /* output all lines, fitting into the frame adrb pd.row-adre */
  for (long i=0;i<pobjects.size();i++){ 
    if( objectVisible(i)) {
      pobjects[i]->paint(s,adrb-currentline); 
    }
  }
  /* output boundary lines */
  paintFrame();
  s->flush(0,adrb);
} 
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::removeObjectHighLighting()
{
  if(highlightedobject>=0){      
    paintObject(highlightedobject);
    highlightedobject=-1; 
  }
  app->s->flush(0,adrb);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::paintObject(long which,HighLightState highlightmode)
{ 

  if(which<0) return;

  Screen *s=app->s;



  /* output all lines, fitting into the frame adrb pd.row-adre */
  for (long i=0;i<pobjects.size();i++){ 

    if( objectVisible(i)) {
      if(i==which){
	pobjects[i]->paint(s,adrb-currentline,highlightmode); 
	break;
      }
    }
  }
  app->s->flush(0,adrb);
}
  




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::addItem(BaseItem *obj,long cr)
{ 
  //NSF("addLine");

  pobjects.append(obj);
  obj->setY(nlines);
  if(cr) nlines++;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::onLayout()
{
  Screen *s=app->s;

  int L=s->rows()-adre-adrb+1;

  if(currentline+L>=nlines)
    currentline=nlines-L;
  if(nlines<L) currentline=0;

  if(startatbottom) {
    currentline=LMAX(nlines-L,0); 
    startatbottom=0;
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::pageDown()
{
  Screen *s=app->s;
  
  int L=s->rows()-adre-adrb+1;
  if(currentline >= nlines-L) return;
  if (nlines >= currentline+2*L-1){
    currentline+=L-1;
  }else{
    currentline=nlines-L;
  };

  paintObjects();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::pageUp()
{
  if(currentline==0) return;

  Screen *s=app->s;
  long L=s->rows()-adre-adrb+1;
  currentline=LMAX(0,currentline-(L-1));

  paintObjects();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::firstPage()
{
  if(currentline==0) return;

  currentline=0;

  paintObjects();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::lastPage()
{
  Screen *s=app->s;
  
  int L=s->rows()-adre-adrb+1;
  
  currentline=LMAX(0,nlines-L);


  paintObjects();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::scrollPage(long updn)
{
  Screen *s=app->s;
  long ade=s->rows()-adre;
  int L=ade-adrb+1;
  if(updn==+1 && currentline+L>=nlines) return;
  if(updn==-1 && currentline==0) return;

  s->moveTo(adrb,0);
  s->scrollUpDn(adrb,ade,updn);
  currentline+=updn;
  for (long i=0;i<pobjects.size();i++){
    long j=pobjects[i]->y();
     if((updn==-1 && j-currentline==0)||(updn==+1 && j-currentline==L-1) ){
       pobjects[i]->paint(s,adrb-currentline);
     }
  }
  paintFrame();
  s->flush(0,adrb);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Page::notify(SObject *caller,String action,String data1,String data2)
{
  //HIER("action");
  
  if(action=="exit"){
    exitAction();
    return 1;
  }else if(action=="help"){
    helpAction(data1);
    return 1;
  }
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::quitAction()
{
  pexit=2;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::exitAction()
{
  //HIER("exitAction");
  pexit=1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::helpAction(String const &data)
{
  //  HIER("help");

  HelpViewer *help;
  try{
    help=new HelpViewer(app,"Help",data,this);
  }catch(Except &ex){
    setStatusBar(ex.msg(),StatusBar::StatusError);
    return;
  }

  

  help->exec();

  delete help;

  show();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::search()
{
  if(app->s->piper()){
    latestfoundobject=-1;
    foundline=0;
    searchstring="";
  }

  Screen *s=app->s;
  s->deleteLine(s->rows()-1);
  s->moveTo(s->rows()-1,1);
  s->addStr("Search:");


  if(searchstring.length()>0){

    const char *blank="                             ";

    long offset=35;
    
    long i=searchObject(latestfoundobject);
    if(i>-1){
      foundline=1;
      latestfoundobject=i;
      s->moveTo(s->rows()-1,offset);
      s->addStr(blank);
    }else{
      foundline=0;
      latestfoundobject=-1;      
      s->moveTo(s->rows()-1,offset);
      s->addStr("not found, start at beginning");
      s->flush(0,adrb);  
      latestfoundobject=-1;      
      i=searchObject(latestfoundobject);
      if(i<0){
	foundline=0;
	latestfoundobject=i;
	s->moveTo(s->rows()-1,offset);
	s->addStr(blank);
	s->moveTo(s->rows()-1,offset);
	s->addStr("not found");
	removeObjectHighLighting();
      }else{
	foundline=1;
	s->moveTo(s->rows()-1,offset);
	s->addStr(blank);
      }
      
    }
  }

  PageSearchLineEdit le(app,9,-1,20,this);
  le.setText(searchstring);
  le.exec();


  foundline=0;
  
  
  if(app->s->piper()){
    app->s->piper()->nextpipeline();
  }
  

  setStatusBar();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecStatus Page::exec()
{
  //  HIER("Page::exec");
  Event e;


  foundline=0;

  onExec();

  aboutToShow();

  paint();

  
  while(1){

    //HIERP(app->s);
    e=app->getKey();

    onKeyEvent(e);
    
    if(pexit) break;
  }
  //HIER("end exec");
  if(pexit==2) app->quit();

  return Accept;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::onResize()
{
  //HIER(className()());
  paint();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Page::onKeyEvent(Event e)
{
  //NSF("Page::onKeyEvent");
  //HIER(subfunc);
  
  



  removeObjectHighLighting();    // remove highlighting if any

  /* menu */
  if(pmenu->onKeyEvent(e)) return 1;
  
  /* alternative menu */
  if(e.key==S_SPACE) { 
    paltmenu->exec();
    setStatusBar();
    return 1;
  }
    
  

  if( e.key== S_CTRL_W || e.key== '/' ) {
    search();
    return 1;
  }else if(e.key== S_CTRL_R ){
    onResize();
    return 1;
  }else if(e.key== S_PAGE_UP ){
    pageUp();
    setStatusBar();
    return 1;
  }else if(e.key== S_PAGE_DOWN ){
    pageDown();
    setStatusBar();
    return 1;
  }else if(e.key== S_KEY_UP || e.key== S_CTRL_K ){
    scrollPage(-1);
    setStatusBar();
    return 1;
  }else if(e.key== S_KEY_DOWN ){
    scrollPage(+1);
    setStatusBar();
    return 1;
  }else if(e.key== S_HOME ){
    firstPage();
    setStatusBar();
    return 1;
  }else if(e.key== S_END ){
    lastPage();
    setStatusBar();
    return 1;
  }
  
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecStatus Page::setStatusBar(String const msg,StatusBar::Status st)
{
  statusBar()->setStatus(msg,st);
  statusBar()->paint();
  app->s->flush(0,adrb);
  if(st==StatusBar::StatusChoose){
    return statusBar()->exec();
  }
  return Discard;
}



/************************************************************************ 
 * search item 
 ************************************************************************ 
 */

long Page::searchObject(long latestmatch)
{

  if(searchstring.length()==0) return -1;

  long startover=0;
  if(latestmatch==pobjects.size() || latestmatch<0) startover=1;


  for (long i=0;i<pobjects.size();i++){
    if(app->s->piper() && pobjects[i]->hotkey().empty()) continue;
    if(i==latestmatch){startover=1;}
    if(startover ){

      if(searchCompare(pobjects[i])){
	
	
	makeObjectVisibile(i);
	
	removeObjectHighLighting();    // remove highlighting if any

	highlightedobject=i;
	paintObject(highlightedobject,SearchHighLighting);	
	return i;

      }
    }
  }
  
  return -1;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Page::searchCallBack(Event &e,String const &text)
{
  Screen *s=app->s;
  const char *blank="                             ";
  long offset=35;

  long key=e.key;
  searchstring=text;

  if(searchstring.length()==0){
    foundline=0;
    removeObjectHighLighting();
  }else if(String::isPrint(key)) {
    

    long i=searchObject(latestfoundobject);
    if(i>-1){
      latestfoundobject=i;
      foundline=1;
      s->moveTo(s->rows()-1,offset);
      s->addStr(blank);
    }else{
      foundline=0;
      s->moveTo(s->rows()-1,offset);
      s->addStr("not found");
      removeObjectHighLighting();    // remove highlighting if any

    }
    return 1;
  }

  if (key ==S_ENTER){
    removeObjectHighLighting();


    if( latestfoundobject>-1 &&  foundline==1){
      onSearchSelect(latestfoundobject);
      return 1;
    }
    
    if(app->s->piper()){
      app->s->piper()
	->message(Sprintf(" Searchstring '%s' not found in screen '%s'\n"
			  ,searchstring(),screenname()));
      throw TerminateExcept("Abort");
    }
    
    foundline=0;
    removeObjectHighLighting();
    return 0;
    
  }else if (key==S_ESCESC || key==S_CTRL_G){
    foundline=0;
    removeObjectHighLighting();
    return 0;
  }else if(key==S_KEY_LEFT|| key==S_KEY_RIGHT || key == S_HOME|| key==S_END){
  }else if (key==S_BACKSPACE|| key==S_DELETE || key==S_CTRL_H || key==S_CTRL_K){
    long i=searchObject(latestfoundobject);
    if(i>=-1){
      foundline=1;
      s->moveTo(s->rows()-1,offset);
      s->addStr(blank);
    }else{
      foundline=0;
    }
  }else if (key == S_CTRL_R) {
    long i=highlightedobject;
    paint(); 
    highlightedobject=i;
    paintObject(highlightedobject,SearchHighLighting);	 
  }else if(
#if !( S_CTRL_I == '\t' )
	   key=='\t' ||
#endif
	   key ==S_CTRL_I ||
	   key ==S_CTRL_W) {
    long i;
    if(latestfoundobject>-1){
      i=searchObject(latestfoundobject+1);
    }else{
      i=searchObject(latestfoundobject);
    }
    if(i>-1){
      foundline=1;
      latestfoundobject=i;
      s->moveTo(s->rows()-1,offset);
      s->addStr(blank);
    }else{
      foundline=0;
      s->moveTo(s->rows()-1,offset);
      s->addStr("not found, start at beginning");
      s->flush(0,adrb);
      latestfoundobject=-1;      
      long i=searchObject(latestfoundobject);
      if(i<0){
	foundline=0;
	s->moveTo(s->rows()-1,offset);
	s->addStr(blank);
	s->moveTo(s->rows()-1,offset);
	s->addStr("not found");
	removeObjectHighLighting();
      }else{
	latestfoundobject=i;
	foundline=1;
	s->moveTo(s->rows()-1,offset);
	s->addStr(blank);
      }
      
    }
  }else{
    foundline=0;
    removeObjectHighLighting();
    e.key=S_ESCESC;
  }
  
  
  
  
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Page::objectVisible(long i)
{
  
  long ade=app->s->rows()-adre;
  int L=ade-adrb+1;

  return ( pobjects[i]->y() >= currentline && pobjects[i]->y() < currentline+L);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::makeObjectVisibile(long i)
{
  Screen *s=app->s;
  long ade=s->rows()-adre;
  int L=ade-adrb+1;
  
  if(! objectVisible(i) ){
    
    currentline=LMAX(0,pobjects[i]->y()-(L-1)/2);
    
    paintObjects();
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Page::searchCompare(BaseItem *obj)
{
  String const &s=obj->screenText().lowerCase();
  return (s.find(searchstring.lowerCase())>=0);
}




/*EOF*/ 
