/* @(#)Page.hh
 * authors: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PAGEXX_H
#define _PAGEXX_H 1


#include "App.hh"
#include "Menu.hh"
#include "PtrArray.hh"


class BaseItem;


enum HighLightState{NoHighLighting=0,SearchHighLighting=1,EditedHighLighting=2};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class StatusBar: public SObject
{
 public:
  enum Status{ StatusOK=0,StatusMessage=1, StatusWarning=2, StatusError=3
	       ,StatusChoose=4,StatusUser=5};
  
 public:
  StatusBar(App *a,SObject*p=0)
    : SObject(a,p),status(StatusOK),statusmsg("OK"),rstatusmsg("") {};
  //
  void paint();
  virtual ExecStatus exec();
  virtual void show(){paint();};
  //
  void setRightStatus(String const &s ){rstatusmsg=s;};
  void setStatus(String const &s,Status st=StatusOK){statusmsg=s;status=st;};
  String const &rightStatusMsg()const {return rstatusmsg;};


 private:
  Status status;
  String statusmsg;
  String rstatusmsg;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Page: public SObject
{
 public:

  // the initial code in the constructor should not refer to the
  // Screen object, since this might get initiated only before
  // exec is called, use onExec() to make Screen dependend initializations
  Page(App *,String const &title="",SObject*p=0,String const &cn="Page");
  virtual ~Page();
  //
  StatusBar *statusBar(){return statusbar;};
  void addItem(BaseItem *obj,long cr=1);
  void addItem(){nlines++;};
  virtual ExecStatus exec();
  void startAtBottom(){startatbottom=1;};
  //
  virtual long notify(SObject *caller,
		      String action,String data1
		      ,String data2="");
  //
  virtual long searchCallBack(Event &e,String const &text);
  //
  virtual void show(){setStatusBar();paint();};
  //
  String const &screenName()const {return screenname;};
  //
  virtual void onResize();
  //
  ExecStatus setStatusBar(String const msg="OK"
		    ,StatusBar::Status st=StatusBar::StatusOK);
  //
  Menu *menu(){return pmenu;};
  AltMenu *altMenu(){return paltmenu;};
  //
  void clear();
  //
 protected:
  //
  void clearItems();
  virtual void quitAction();
  virtual void exitAction();
  virtual void helpAction(String const &data);
  virtual void onLayout();
  virtual void onExec(){};
  virtual void onSearchSelect(long selctedobject){};
  virtual long searchCompare(BaseItem *obj);
  virtual void paint();
  virtual long onKeyEvent(Event );
  void removeObjectHighLighting();
  long objectVisible(long);
  void paintObjects();
  void paintObject(long which,HighLightState h=NoHighLighting);
  void makeObjectVisibile(long);
  //
 private:
  void pageUp();
  void pageDown();
  void firstPage();
  void lastPage();
  void scrollPage(long updn);
  void paintFrame();
  void search();
  long searchObject(long latestmatch);


 protected:
  String screenname;
  Menu *pmenu;
  AltMenu *paltmenu;
 private:
  StatusBar *statusbar;
 protected:
  long currentline;
  long adrb,adre;

 protected:
  PtrArray<BaseItem> pobjects; 
  
  long nlines;

  long startatbottom;

  long pexit;

 protected:
  String searchstring;
  long highlightedobject;
  long foundline;
  long latestfoundobject;
};









#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PAGEXX_H */

