/*************************************************************************
 * @(#)ParseTable.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Oct 2008
 *************************************************************************
 */

#include "MyDebug.hh"
#include "debugxx.hh"

#include "ParseTable.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ParseTables::ParseTables()
  :classname("ParseTables"),psize(0),preadin(0),array(0)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ParseTables::~ParseTables()
{
  delete [] array;
}

/* ***********************************************************************
 * this will delete the whole data and setup a new table
 * ***********************************************************************
 */

void ParseTables::setup(StringList const &sl)
{
  if(array) delete [] array;
  array=new table[sl.size()];
  psize=sl.size();
  for(long i=0;i<sl.size();i++)
    array[i].filename=sl[i];
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String const &ParseTables::fileName(long i)const
{
  NSF("fileName(long)");
  if(i<0 || i>=psize)
    THR("index out of bounds");
  return array[i].filename;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
time_t ParseTables::modificationDate(long i)const
{
  NSF("modificationDate(long)");
  if(i<0 || i>=psize)
    THR("index out of bounds");
  return array[i].lastmod;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ParseTables::setModificationDate(long i,time_t d)
{
  NSF("setModificationDate(long)");
  if(i<0 || i>=psize)
    THR("index out of bounds");
  array[i].lastmod=d;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Parser  &ParseTables::parser(long i)const
{
  NSF("parser(long)");
  if(i<0 || i>=psize)
    THR("index out of bounds");
  return array[i].parser;
}

/* ***********************************************************************
 * cleanup the actual parse tables 
 * ***********************************************************************
 */
void ParseTables::cleanupTables()
{
  if(!array) return;
  for(long i=0;i<psize;i++){
    array[i].parser.cleanup();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long ParseTables::findTable(String const & fname)const
{
  NSF("findTable(String const &)");

  long pct=-1;
  
  for(long j=0;j<psize;j++)
    if(array[j].filename==fname){
      pct=j;
      break;
    }

  
  if(pct<0)
    THR2("internal(%s): cannot find entry in\n parse "
	 "control table for file '%s'!\n",fname());
  
  return pct;
}
/*EOF*/ 
