/*************************************************************************
 * @(#)Piper.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Nov 2008
 *************************************************************************
 */


#include "Keys.hh"
#include "Except.hh"
#include "Piper.hh"








#define SCAN_BEGIN 1
#define SCAN_WORD  2
#define SCAN_END   4







/* ***********************************************************************
 *
 * ***********************************************************************
 */
Piper::Piper(FILE *fh)
  :pline(1),linestart(1),hotstart(0),searchstart(0),commandstart(0)
  ,argstart(0),nextline(0),file(fh),pipedebug(0),pipemode(0)
{
  fprintf(stderr,"Pipe mode initialized\n");
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Piper::message(String const &msg) const
{
  if(!pipemode) return;
  fprintf(stderr,"Pipemode input line %ld:\n %s\n",pline,msg());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Piper::message(String const &title,String const &msg) const
{
  if(!pipemode) return;
  fprintf(stderr,"Pipemode input line %ld:\n %s\n\n%s\n",pline,title(),msg());
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Piper::eatup()
{
  int key;
  /*eat up rest of line */
  eatupbuffer="";

 loop1:
  key=fgetc(file);
  if(pipedebug) fprintf(stderr,"eatup=<%c>\n",key);
  if(key == '\t' ) key=' ';
  if(key == '\r' ) key='\n';
  if(key == EOF ){
    message(" ERROR: End of file before quit/save hotkey\n");
    throw TerminateExcept("Abort");
    
  }
  eatupbuffer=(char)key;
  if(key != '\n') goto loop1;
}


/* ***********************************************************************
 * get keys in pipe mode
 * ***********************************************************************
 */
int Piper::getPipeKey()
{

  if(!pipemode)return 0;

  int key;
 next:
  if(nextline){
    if(!linestart){
      eatup();
      linestart=1;
      hotstart=0;
      commandstart=0;
      searchstart=0;
      argstart=0;
      buffer="";
      pline++;
    }
    nextline=0;
  }
  key=fgetc(file);

  if(key == '\t' ) key=' ';
  if(key == '\r' ) key='\n';
  if(key == EOF ){
    message(" ERROR: End of file before quit/save hotkey\n");
    throw TerminateExcept("Abort");
  }
  if(pipedebug) fprintf(stderr,"fgetc=<%c>\n",key);
  if(linestart){
    if (key == ' ' ) goto next;
    
    if(key=='\n'){
      linestart=1;
      buffer="";
      hotstart=0;
      commandstart=0;
      searchstart=0;
      argstart=0;
      pline++;
      goto next;
    }
    if(key == '#' ) {
      eatup();
      message(eatupbuffer);
      linestart=1;
      hotstart=0;
      commandstart=0;
      searchstart=0;
      argstart=0;
      buffer="";
      pline++;
      goto next;
    }

    if(key == '@' ) {
      /* got @ */
      linestart=0;
      hotstart=1;
      commandstart=0;
      searchstart=0;
      argstart=0;
      buffer="";
      goto next;
    }else if(key == '!' ) {
      /* got ! */
      linestart=0;
      hotstart=0;
      commandstart=0;
      searchstart=1;
      argstart=0;
      buffer="";
      if(pipedebug) fprintf(stderr,"return=CTRL-W\n");
      return S_CTRL_W;
    }else {
      message(" ERROR: Expecting '@' or '!' at begin of line!\n");
      throw TerminateExcept("Abort");
    }
  }else{
    if(argstart){
      /* argument mode */
      if(key=='\n'){
	linestart=1;
	hotstart=0;
	commandstart=0;
	searchstart=0;
	argstart=0;
	buffer="";
	pline++;
	if(pipedebug) fprintf(stderr,"return=\\n\n");
	return '\n';
      }

      if(key=='@'){
	hotstart=0;
	commandstart=0;
	searchstart=0;
	argstart=1;
	buffer="";
	if(pipedebug) fprintf(stderr,"return=\\n\n");
	return '\n';
      }
      if(key=='!'){
	hotstart=0;
	commandstart=0;
	searchstart=0;
	argstart=1;
	buffer="";
	if(pipedebug) fprintf(stderr,"return=\\n\n");
	return '\n';
      }


    }else{
      /* hotkey/search mode */
      if(key=='\n'){
	if(hotstart){
	  message(" ERROR: Expecting closing '@'!\n");
	  throw TerminateExcept("Abort");

	  
	}
	if(searchstart){
	  message(" ERROR: Expecting closing '!'!\n");
	  throw TerminateExcept("Abort");
	}
	linestart=1;
	hotstart=0;
	commandstart=0;
	searchstart=0;
	argstart=0;
	buffer="";
	pline++;
	goto next;
      }
      if(key == '@' && hotstart ){
	if(buffer.length()==0){
	  message(" ERROR: Empty hotkey sequence!\n");
	  throw TerminateExcept("Abort");
	  
	}
	hotstart=0;
	searchstart=0;
	argstart=1;
	buffer="";
	if(commandstart){
	  commandstart=0;
	  if(pipedebug) fprintf(stderr,"eatup @, enter argument mode\n");
	  goto next;
	}
	if(pipedebug) fprintf(stderr,"return=\\n\n");
	return '\n';
      }
      if(key == '!' && searchstart){
	if(buffer.length()==0){
	  message(" ERROR: Empty search sequence!\n");
	  throw TerminateExcept("Abort");
	  
	}
	hotstart=0;
	commandstart=0;
	searchstart=0;
	argstart=1;
	buffer="";
	if(pipedebug) fprintf(stderr,"return=\\n\n");
	return '\n';
      }
    }
  }/* end hotkey/search mode */    
    
  if(!String::isPrint(key)){
    message(Sprintf(" ERROR: Unknown character '0x%x'!\n",key));
    throw TerminateExcept("Abort");
  }

  buffer=(char)key;
  /* if(pipedebug) fprintf(stderr,"[%s]\n",buf);  */
  if(buffer[0]==':' && hotstart) {
    commandstart=1;
    if(pipedebug) fprintf(stderr,"enter command mode\n");
  }
  if(pipedebug) fprintf(stderr,"return=<%c>\n",key);
  return key;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Piper::copyExpandPlaceHolders(String &dest,String const &src)
{

  if(!pipemode)return;
  StringList sl=src.split(" \t");


  dest=pipeRegexReplace(dest,sl);


}


/* ***********************************************************************
 *  the special placeholders '*' and '\n' and '\{nm}'
 *  are replaced by the words from a previous word-scan (in list)
 *  '\\n' and '\\{' will be escaped into '\n' and '\{'
 * ***********************************************************************
 */


String Piper::pipeRegexReplace(String const &text,StringList const &list)
{
  //  NSF("pipe_scan_line");
  long pos;
  long state,nw;

  String parg;
  String res;



  

  state=SCAN_BEGIN;
  pos=0;
  nw=0;

  while(state!=SCAN_END){
    
    switch (text[pos])
      {
      case ' ':
      case '\t':
	if(state==SCAN_WORD) state=SCAN_BEGIN;
	res+=text[pos++];
	break;
      case '\0':
	state=SCAN_END;
	break;
      default:

	if(state==SCAN_BEGIN){
	  state=SCAN_WORD;
	  nw++;

	  if(text[pos]=='*' && ( (text[pos-1]==' ' || text[pos-1]=='\t') || 
				 pos==0)){
	    if(text[pos]=='\0' || text[pos+1]==' ' || text[pos+1]=='\t'){
	      
	      
	      if(nw>list.size()){
		pline--;
		message(Sprintf(" string \'%s\' contains placeholders which"
				" do not match previous value.\n",text()));
		throw TerminateExcept("Abort");
	      }
	      res+=list[nw-1];
	      
	      
	      state=SCAN_BEGIN;
	      pos++;
	      break;
	    }
	  }
	  
	
	
	}
	if(state==SCAN_WORD){
	  
	  if(text[pos]=='\\'){ 
	    
	    if(text[pos+1]!='\0'){ 
	      
	      if(text[pos+1]=='\\'){ 
		if(text[pos+2]!='\0' && (String::isDigit(text[pos+2]) || 
					 text[pos+2]=='{')){ 
		  /* '\\n' is escaped '\n' and '\\{' is escaped '\{' */
		  pos++; 
		}
	      }else if(String::isDigit(text[pos+1])){ 
		String arg="";
		arg+=text[pos+1];
		parg=arg;
 		pos+=2;  

		long l=1,ok=1;
		try {l=arg.convertToLong();}catch(...){ok=0;};
		if(!ok) goto Larginvalid;
		if(l > list.size() || l<1)  goto Largnoexist;
		res+=list[l-1];
		
		state=SCAN_BEGIN;

		if(text[pos]!='\0' && String::isDigit(text[pos]))
		  goto Ldanger;
		
 		break;
	      }else if(text[pos+1]=='{'){ 
		String arg="";
 		pos+=2;  
		while(1){
		  if(text[pos]=='}'){
		    pos++;
		    break;
		  }else if(text[pos]=='\0'){
		    parg="{"+arg;
		    goto Larginvalid;
		  }else{
		    arg+=text[pos++];
		  }
		}
		parg="{"+arg+"}";
		if(arg.length()==0) goto Largnodigit;
		long l=1,ok=1;
		try {l=arg.convertToLong();}catch(...){ok=0;};
		if(!ok) goto Larginvalid;
		if(l > list.size() || l<1)  goto Largnoexist;
		res+=list[l-1];
		
		state=SCAN_BEGIN;
		
		break;
	      } 
	      	    
	    } /* not $  */
	  } /* *(p)=\ */
	}/* SCAN_WORD*/

	
	if(state==SCAN_WORD){
	  res+=text[pos];
	}
	
	pos++;
	break;
      }
    
  }

  
  return res;



 Largnoexist:;
  pline--;
  message(Sprintf(" string \'%s\' contains reference '\\%s', which does"
		  " not match previous value.\n",text(),parg()));
  throw TerminateExcept("Abort");

 Largnodigit:;
  pline--;
  message(Sprintf(" string \'%s\' contains empty reference '\\{}'.\n",text()));
  throw TerminateExcept("Abort");

 Larginvalid:
  pline--;
  message(Sprintf(" string \'%s\' contains reference '\\%s', which is no"
		  " integer or a runaway closing '}'.\n",text(),parg()));
  throw TerminateExcept("Abort");
  

 Ldanger:
  pline--;
  message(Sprintf(" string \'%s\' contains reference '\\%s', followed by a"
		  " digit.\n Put a space behind '\\%s' if you mean a single "
		  "digit\n reference or use \\{nm} if you want a reference >9.\n"
		  ,text(),parg(),parg()));
  throw TerminateExcept("Abort");
  
  
}


/*EOF*/ 
