/*************************************************************************
 * @(#)Screen.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   16 Oct 2008
 *************************************************************************
 */

#undef USE_ACS


#include <unistd.h>
#include <ncurses.h>
#include <term.h>
#include <string.h>

#include "Dump.hh"


#include "Styles.hh"
#include "Keys.hh"
#include "Piper.hh"
#include "Screen.hh"
#include "ValueArray.hh"




#define SS_STYLE_BOLD        A_BOLD

const StringList Screen::colorSchemes=
  String("CLASSIC_BLACK RED_DEVIL GREEN_LEAVES SILVER_MINE HOT_DOG").split(" ");

const long Screen::CLASSIC_BLACK=0;
const long Screen::RED_DEVIL=1;
const long Screen::GREEN_LEAVES=2;
const long Screen::SILVER_MINE=3;
const long Screen::HOT_DOG=4;
 
const long Screen::NO_DUMP=0;
const long Screen::ASCII_DUMP=1;
const long Screen::PS_DUMP=2;

/* ***********************************************************************
 *
 * ***********************************************************************
 */


const long  S_KEY_NO =      0x00;
const long  S_CTRL_A =     1;
const long  S_CTRL_B =     2;
const long  S_CTRL_C =     3;
const long  S_CTRL_D =     4;
const long  S_CTRL_E =     5;
const long  S_CTRL_F =     6;
const long  S_CTRL_G =     7;
const long  S_CTRL_H =     8;
const long  S_CTRL_I =     9;
const long  S_CTRL_J =     10;
const long  S_CTRL_K =     11;
const long  S_CTRL_L =     12;
const long  S_CTRL_M =     13;
const long  S_CTRL_N =     14;
const long  S_CTRL_O =     15;
const long  S_CTRL_P =     16;
const long  S_CTRL_R =     18;
const long  S_CTRL_S =     19;
const long  S_CTRL_U =     21;
const long  S_CTRL_V =     22;
const long  S_CTRL_W =     23;
const long  S_CTRL_X =     24;
const long  S_CTRL_Y =     25;
const long  S_CTRL_Z =     26;
const long  S_ENTER  =     10;
const long  S_SPACE  =     ' ';
const long  S_ESCAPE =     27 ;
const long  S_ESCESC =     27100 ;
const long  S_BACKSPACE=   KEY_BACKSPACE ;
const long  S_DELETE =     KEY_DC;

#ifdef KEY_HOME
const long S_HOME=        KEY_HOME;
#else
/* some old AIX stuff */
#ifdef KEY_FIND
const long S_HOME =       KEY_FIND ; 
#else
const long S_HOME  =      S_CTRL_A  ;
#endif
#endif

#ifdef KEY_END
const long S_END    =     KEY_END;
#else
#ifdef KEY_SELECT 
const long S_END     =    KEY_SELECT;
#else
const long S_END      =   S_CTRL_E;
#endif
#endif

const long  S_PAGE_UP   =  KEY_PPAGE ;
const long  S_PAGE_DOWN =  KEY_NPAGE ;
const long  S_KEY_RIGHT =  KEY_RIGHT;
const long  S_KEY_LEFT  =  KEY_LEFT;
const long  S_KEY_DOWN  =  KEY_DOWN ;
const long  S_KEY_UP    =  KEY_UP;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Screen::Screen(long layout,long cs,long acs,long rev,long dump,long pip
	       ,long sdebug)
  :COLORED(FALSE),ACS(acs),REVERSE(rev),pipein(pip),screendebug(sdebug)
  ,ppiper(0),pdump(0)
{
  short backgroundcolor;

  if(pip){
    ppiper=new Piper(stdin);
    ppiper->setPipeMode(pip);
    return;
  }

  initscr(); 
  
  
  putp(enter_ca_mode);  
  keypad(stdscr,TRUE); 
  noecho(); 
  raw(); 


  pdump=new Dump(dump,cs,(layout==1),lines,columns);
  

#if !defined (NO_COLOR_CURSES)  
  

  if((layout==1) &&((COLORED=(long)has_colors())==TRUE)) { 
    start_color(); 
    if(COLOR_PAIRS>=4){ 
      /*
	init_pair(4,COLOR_WHITE,COLOR_BLACK); 
      */
      switch (cs)
	{
	case RED_DEVIL:
	  backgroundcolor=COLOR_RED;
	  (void)init_pair(1,COLOR_WHITE,backgroundcolor); 
	  (void)init_pair(2,COLOR_YELLOW,backgroundcolor); 
	  (void)init_pair(3,COLOR_GREEN,backgroundcolor); 
	  (void)init_pair(4,COLOR_MAGENTA,backgroundcolor); 
	  break;
	case GREEN_LEAVES:
	  backgroundcolor=COLOR_GREEN;
	  (void)init_pair(1,COLOR_BLACK,backgroundcolor); 
	  (void)init_pair(2,COLOR_RED,backgroundcolor); 
	  (void)init_pair(3,COLOR_YELLOW,backgroundcolor); 
	  (void)init_pair(4,COLOR_MAGENTA,backgroundcolor); 
	  break;
	case SILVER_MINE:
	  backgroundcolor=COLOR_WHITE;
	  (void)init_pair(1,COLOR_BLACK,backgroundcolor); 
	  (void)init_pair(2,COLOR_RED,backgroundcolor); 
	  (void)init_pair(3,COLOR_BLUE,backgroundcolor); 
	  (void)init_pair(4,COLOR_MAGENTA,backgroundcolor); 
	  break;
	case HOT_DOG:
	  backgroundcolor=COLOR_YELLOW;
	  (void)init_pair(1,COLOR_BLACK,backgroundcolor); 
	  (void)init_pair(2,COLOR_RED,backgroundcolor); 
	  (void)init_pair(3,COLOR_WHITE,backgroundcolor); 
	  (void)init_pair(4,COLOR_MAGENTA,backgroundcolor); 
	  break;
	case CLASSIC_BLACK:
	default:
	  backgroundcolor=COLOR_BLACK;
	  (void)init_pair(1,COLOR_WHITE,backgroundcolor); 
	  (void)init_pair(2,COLOR_RED,backgroundcolor); 
	  (void)init_pair(3,COLOR_GREEN,backgroundcolor); 
	  (void)init_pair(4,COLOR_MAGENTA,backgroundcolor); 
	  break;
      }

      bkgd(SS_STYLE_NORMAL); 
      
      setAttribute(SS_STYLE_NORMAL,AttSet); 

    } 
  } 
  
#endif 


#if defined(USE_ACS)	 
  HLIN=ACS_HLINE;
#else
  HLIN='_';
#endif
  
  char *p=(char*)termname(); 
  if((strstr(p,"aixterm")!=NULL)||(!ACS)){ 
    HLIN='_'; 
  } 





}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Screen::~Screen()
{
  if(!ppiper){
    keypad(stdscr,FALSE); 
    echo();
    endwin();
    reset_shell_mode(); 
  }
  if(ppiper) delete ppiper;
  if(pdump) delete pdump;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Event Screen::getKey() 
{
  
  int key;
  int fkey;
  int esc=0;

  if(ppiper){
    return Event(ppiper->getPipeKey());
  }

 loop:
  
  key=getch();
  
  if(key==S_ESCAPE){
    key=getch()*1000+100;
    esc=1;
  };
  
  fkey=translateFunctionKey(key);
  
  if(fkey==S_CTRL_S && pdump->screendumpOn() ){
    makeScreenDump();
    goto loop;
  }
  

  return Event(fkey,esc);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::makeScreenDump()
{
  if(ppiper) return;
  
  pdump->dumpScreen();
  pdump->stopBuffering();

  String s=Sprintf("Screen dumped to '%s'.",pdump->lastScreendumpFile()());
  long l=s.length();
  long rim=2;
  long wx=l+2*rim+2;
  long wy=3;

  long x=(cols()+1-l-2*(1+rim))/2-1;
  long y=(rows()+1-3)/2-1;

  reverseOn();
  setAttribute(SS_STYLE_ENTRYHIGH,AttOn);
  moveTo(y,x);
  for(long i=0;i<wx;i++) addCh('-');
  moveTo(y+2,x);
  for(long i=0;i<wx;i++) addCh('-');

  moveTo(y+1,x);
  addCh('|');
  for(long i=1;i<wx-1;i++) addCh(' ');
  addCh('|');

  moveTo(y+1,x+rim+1);
  addStr(s);
  reverseOff();
  
  flush(0,3);
  sleep(1);
  copyFromDump(x,y,wx,wy);
  flush(0,3);

  pdump->startBuffering();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Screen::copyFromDump(long x,long y,long wx,long wy)
{
  if(ppiper) return;

#define MASK_ATTR(at)							\
    if(at & A_STANDOUT) at =((at & (~A_STANDOUT)) | (A_REVERSE));	\
    at&=(A_UNDERLINE|A_REVERSE|A_BOLD | SS_COLOR_MASK)  

  for(long j=0;j<wy;j++){
    moveTo(y+j,x);
    for(long i=0;i<wx;i++) {
      long at0=pdump->attrAt(y+j,x+i);
      char ch=pdump->charAt(y+j,x+i);
      MASK_ATTR(at0);
      if((at0 & A_BOLD)) boldOn();
      else boldOff();
      if((at0 & A_REVERSE)) reverseOn();
      else reverseOff();
      if((at0 & A_UNDERLINE)) underlineOn();
      else underlineOff();
      if(pdump->colored()){
	if(((at0 & SS_COLOR_MASK) == SS_STYLE_NORMAL))
	  setAttribute(SS_STYLE_NORMAL,AttOn); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_MENUHIGH))
	  setAttribute(SS_STYLE_MENUHIGH,AttOn); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_ENTRYSEARCH))
	  setAttribute(SS_STYLE_ENTRYSEARCH,AttOn); 
	else if(((at0 & SS_COLOR_MASK) == SS_STYLE_ENTRYHIGH))
	  setAttribute(SS_STYLE_ENTRYHIGH,AttOn); 
      }
      addCh(ch);
    }
  }

#undef MASK_ATTR
}

/************************************************************************ 
 * define emacs control keys and some function keys 
 ************************************************************************ 
 */

int Screen::translateFunctionKey(int key) const
{
  if(ppiper) return 0;
  switch (key)
    {
    case (KEY_BACKSPACE) :return S_BACKSPACE;
    case (S_CTRL_H) :return S_BACKSPACE;
    case (KEY_DC)        :return S_DELETE;
    case (S_CTRL_D)        :return S_DELETE;
#ifdef KEY_FIND
    case (KEY_FIND)      :return S_HOME;
#endif
    case (KEY_HOME)      :return S_HOME;
    case (S_CTRL_A)      :return S_HOME;
#ifdef KEY_SELECT
    case (KEY_SELECT)     :return S_END;
#endif
#ifdef KEY_END
    case (KEY_END)     :return S_END;
#endif
    case (S_CTRL_E)     :return S_END;
    case (KEY_PPAGE)     :return S_PAGE_UP;
    case (S_CTRL_P)     :return S_PAGE_UP;
    case (KEY_NPAGE)     :return S_PAGE_DOWN;
    case (S_CTRL_V)     :return S_PAGE_DOWN;
    case (KEY_RIGHT)     :return S_KEY_RIGHT;
    case (S_CTRL_F)     :return S_KEY_RIGHT;
    case (KEY_LEFT)      :return S_KEY_LEFT;
    case (S_CTRL_B)      :return S_KEY_LEFT;
    case (KEY_DOWN)      :return S_KEY_DOWN;
    case (S_CTRL_N)      :return S_KEY_DOWN;
    case (KEY_UP)        :return S_KEY_UP;

    }
    return key;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::deleteLine(long y)
{
  if(ppiper) return;
  moveTo(y,0); 
  clrToEOL(); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Screen::addString(String const &format,String const &c,long cx,long cy)
{ 
  if(ppiper) return;
  moveTo(cy,cx); 
  addStr(Sprintf(format(),c()));
} 

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::addStr(String const &s)
{
  if(ppiper) return;
  addstr(s());
  pdump->dump_addstr(s());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::addCh(char c)
{
  if(ppiper) return;
  addch(c);
  pdump->dump_addch(c);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::moveTo(long y,long x)
{
  if(ppiper) return;
  move((int)y,(int)x);
  pdump->dump_move(y,x);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::clrToEOL()
{
  if(ppiper) return;
  clrtoeol();
  pdump->dump_clrtoeol(); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::scrollUpDn(long top,long bottom,long n)
{
  if(ppiper) return;
  scrollok(stdscr,TRUE);
  setscrreg((int)top,(int)bottom);
  scrl((int)n);
  scrollok(stdscr,FALSE);
  pdump->dump_scrollUpDn(top,bottom,n);
}


/* ***********************************************************************
 *  clear screen from a till e   0,rows-1 is full screen  
 * ***********************************************************************
 */
void Screen::clrscr(long a,long e) 
{ 
  if(ppiper) return;
  for(long i=a;i<=e;i++){ 
    moveTo((int)i,0); 
    clrToEOL(); 
  }     
} 

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Screen::rows()const
{ 
  if(ppiper) return 100;
  return lines; 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Screen::cols()const
{ 
  if(ppiper) return 100;
  return columns; 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::initialize()
{ 
  if(ppiper) return;
  pdump->initDumpBuffer(lines,columns);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::flush(long cx,long cy) 
{
  if(ppiper) return;
  (void)moveTo(cy,cx); 
  (void)refresh(); 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::reverseOn()
{
  if(ppiper) return;
  if(REVERSE) {
    setAttribute(A_REVERSE,AttOn);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::reverseOff()
{
  if(ppiper) return;
  if(REVERSE) {
    setAttribute(A_REVERSE,AttOff);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::underlineOn()
{
  if(ppiper) return;
  setAttribute(A_UNDERLINE,AttOn);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::underlineOff()
{
  if(ppiper) return;
  setAttribute(A_UNDERLINE,AttOff);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::boldOn()
{
  if(ppiper) return;
    setAttribute(SS_STYLE_BOLD,AttOn);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::boldOff()
{
  if(ppiper) return;
  setAttribute(SS_STYLE_BOLD,AttOff);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Screen::setAttribute(long a,AttributeManipulation m)
{
  if(ppiper) return;
  switch(m){
  case AttOn:
    attron((int)a);
    pdump->dump_attron(a);
    return ;
  case AttOff:
    attroff((int)a);
    pdump->dump_attroff(a);
    return ;
  case AttSet:
    attrset((int)a);
    pdump->dump_attrset(a);
    return ;
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

// Screen::setSearchStyle(AttributeManipulation m)
// {
//   if(COLORED) {
//     setStyle(StyleEntrySearch,m);
//     setAttribute(A_REVERSE,m);
//   }else{
//     setAttribute(A_REVERSE,m);
//     setAttribute(A_UNDERLINE,m);
//   }
// }


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::setStyle(Styles s,AttributeManipulation m)
{
  if(ppiper) return;
  
  switch(s){
  case StyleMenuHigh: 
    if(COLORED && m==AttOff){
      setAttribute(SS_STYLE_NORMAL,AttOn);
    }else{
      setAttribute(SS_STYLE_MENUHIGH,m);
    }
    return;
  case StyleEntryHigh: 
    if(COLORED && m==AttOff){
      setAttribute(SS_STYLE_NORMAL,AttOn);
    }else{
      setAttribute(SS_STYLE_ENTRYHIGH,m);
    }
    return;
  case StyleEntrySearch: 
    if(COLORED && m==AttOff){
      setAttribute(SS_STYLE_NORMAL,AttOn);
    }else{
      setAttribute(SS_STYLE_ENTRYSEARCH,m); 
    }
    return;
  case StyleNormal: 
  default:
    if(COLORED && m==AttOff){
      setAttribute(SS_STYLE_NORMAL,AttOn);
    }else{
      setAttribute(SS_STYLE_NORMAL,m);
    }
    return;
  }
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::addHLIN()
{
  if(ppiper) return;
  addCh(HLIN);
}



/************************************************************************ 
 *  print @(..) highlighted 
 ************************************************************************ 
 */
void Screen::prtToken(String str,Styles high,Styles normal)
{
  if(ppiper) return;

  const char *g;
  long on,i,l;



  g=str();
  on=0;
  l=str.length();
  i=0;
  while(i<l){
 
    if(strncmp(g,"@(",2)==0){
      
      g+=2;i+=2;on++;
      setStyle(high,AttOn);boldOn();
      addCh('(');
      continue;
    }
    if( on && strncmp(g,")",1)==0){
      g++;i++;on--;
      addCh(')');
      setStyle(normal,AttOn); boldOff();
      continue;
    }
    if(strncmp(g,"@_",2)==0){
      setStyle(high,AttOn); boldOn();
      addStr("[]");
      setStyle(normal,AttOn); boldOff();
      g+=2;i+=2;
    }else if(strncmp(g,"@:",2)==0){
      setStyle(high,AttOn);boldOn();
      addStr("(:)");
      setStyle(normal,AttOn); boldOff();
      g+=2;i+=2;
    }else{
      addCh(g[0]);
      g++;i++;
    }
  }
}

/* ***********************************************************************
 * print with markup
 * ***********************************************************************
 */
void Screen::prtMarkUpLine(String const & line)
{
  if(ppiper) return;


  ValueArray<long> mode;

  const char*p =line();
  


  //  setAttribute(SS_STYLE_NORMAL,AttSet);

  while((*p)!='\0'){
    if(strncmp(p,"U<",2)==0){
      mode.append(A_UNDERLINE);
      p+=2;
    }else if(strncmp(p,"B<",2)==0){
      mode.append(SS_STYLE_BOLD);
      p+=2;
    }else if(strncmp(p,"R<",2)==0){
      mode.append(A_REVERSE);
      p+=2;
    }else if(strncmp(p,"A<",2)==0){
      p+=2;
      mode.append(A_STANDOUT);
    }else if(strncmp(p,"H<",2)==0){
      p+=2;
      if(COLORED){
	mode.append(SS_STYLE_HINT);
      }else{
	mode.append(A_STANDOUT);
      }
    }else if( ( (*p)=='>') && ( mode.size()>0 ) ) {
      if(mode.last()==SS_STYLE_HINT){
	setAttribute(SS_STYLE_NORMAL,AttOn);
      }else{
	setAttribute(mode.last(),AttOff);
      }
      mode.removeLast();
      p++;
    }else {
      
      for (long j=0;j<mode.size();j++)
	setAttribute(mode[j],AttOn);
     
      addCh(*p++);
    }
    
  }

  setAttribute(SS_STYLE_NORMAL,AttSet);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::shellOut()
{
  if(ppiper) return;
  reset_shell_mode();
  putp(exit_ca_mode); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Screen::shellIn()
{
  if(ppiper) return;
  putp(enter_ca_mode); 
  reset_prog_mode();
  bkgd(SS_STYLE_NORMAL);
  setAttribute(SS_STYLE_NORMAL,AttSet);
  wclear(stdscr);

}
/*EOF*/ 
