/*************************************************************************
 * @(#)ScreenDefs.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 Oct 2008
 *************************************************************************
 */

//#include "debugxx.h"


#include "ScreenDefs.hh"
#include "Except.hh"
#include "Form.hh"
#include "Item.hh"


/* the macro version  calls j twice !! */
static long LMIN(long i,long j)
{
  return ( ((i)<j?i:j) );
}

StringList const BaseItemDef::knownActions=String("\
            dashline emptyline title item itemlist select selectbox	\
            check checkbox field endfield counter compound body		\
            endcompound mselect mselectbox").split(" \t");
StringList const BaseItemDef::knownFlags=String("\
            eol print_value show_content value_as_hotkey auto_width fixed \
            trigger_update two_column three_column selector \
           ").split(" \t");

StringList const MenuDef::knownTypes=String("main alt").split(" \t");
StringList const MenuDef::knownActions=String("\
            mainquit mainexit externedit submenu separator help restore \
            rebuild eol exit fplorun viewer run				\
           ").split(" \t");
StringList const MenuDef::knownFlags=String("special_symmetry").split(" \t");

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String const BaseItemDef::actionType(String const a,StringList &flags)
{
  String ptype="";
  flags.clear();

  StringList sl=a.breakup("|");
  
  for(long i=0;i<sl.size();i++){
    
    String s=sl[i].stripBlanks().toLower();
    
    if(s.contains(" \t"))
      throw Except(Sprintf("error(actionType): action '%s': types/flags"
				   " must be separated by '|'!",a()));
    
    
    
    if(knownActions.contains(s)){
      if(!ptype.empty())
	throw Except(String()
		     .sprintf("action def '%s': more than one type "
			      "defined!",a()));
      ptype=s;
      
    }else if(knownFlags.contains(s))
      flags.append(s);
    else
      throw Except(String()
		   .sprintf("action def '%s': unknown type/flag "
			    "or wrong combination!",a()));
  }

  
  if(ptype.empty())
    if(!flags.contains("eol"))
      throw Except(String()
		   .sprintf("menu item '%s': need one type or just 'EOL'!",a()));
  
  

  return ptype;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MenuDef::setActions(String const & a) 
{ 

  ptype="";
  paction="";
  pflags.clear();
  
  StringList sl=a.breakup("|");
  

  for(long i=0;i<sl.size();i++){
    
    String s=sl[i].stripBlanks().toLower();
    
    if(s.contains(" \t"))
      throw Except(Sprintf("error(setAction): action '%s': "
				    "flags must be separated by '|'!",a()));
    
    
    if(knownTypes.contains(s)){
      if(!ptype.empty())
	throw Except(String()
		     .sprintf("menu item '%s': more than one type "
			      "defined!",a()));
      ptype=s;
    }else if(knownActions.contains(s)){
      if(!paction.empty())
	throw Except(String()
		     .sprintf("menu item '%s': more than one action "
			      "defined!",a()));
      paction=s;
    }else if(knownFlags.contains(s)){
      pflags.append(s);
    }else
      throw Except(String()
		   .sprintf("menu item '%s': unknown action, type or "
			    "flag, or wrong combination!",a()));
    
  }

  if(ptype.empty())
    throw Except(String()
		 .sprintf("menu item '%s': need one type!",a()));

  if(paction.empty())
    throw Except(String()
		 .sprintf("menu item '%s': need one action!",a()));
  
  if(ptype=="main" && paction == "exit" && pflags.contains("special_symmetry") )
    paction="symmetryexit";
  
  if(ptype=="main" && paction == "separator" )
    pname="@>";
  


  if(ptype=="alt" && paction == "eol" )
    pname="@>";
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ScreenDef::clear()
{
  
  
  md.clear();
  
  bd.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ScreenDef const *ScreenDefs::byName(String const &sn) const
{
  for (long i=0;i<sd.size();i++){
    
    if( const_cast<PtrArray<ScreenDef>&>(sd)[i]->screenName()==sn )
      return const_cast<PtrArray<ScreenDef>&>(sd)[i];
    
  }

  return 0;
}

 


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BaseItemDef::setupTest(Dobj const &itemdefdobj,Dobj const &d)
{
  
  // setup validators



  
  if(d.isInt())
    ptest=new IntTest(
		      itemdefdobj("intervalleft")()
		      ,itemdefdobj("intervalright")()
		      ,itemdefdobj("testflags")()
		      ,itemdefdobj("testset").list().escapeDecode()
		      ); 
  else if(d.isReal())
    ptest=new RealTest(
		       itemdefdobj("intervalleft")()
		       ,itemdefdobj("intervalright")()
		       ,itemdefdobj("testflags")()
		       ,itemdefdobj("testset").list().escapeDecode()
		       
		       ); 
  else if(d.isLogical())
    ptest=new LogicalTest(1);
  else if(d.isChar())
    ptest=new StringTest(0L,1L
			 ,itemdefdobj("testflags")()
			 ,itemdefdobj("testset").list().escapeDecode()
			 ); 
  else if(d.isFlag())
    ptest=new FlagTest(1);
  else if(d.isString())
    ptest=new StringTest(d.isVariableString()
			 ,d.stringLength()
			 ,itemdefdobj("testflags")()
			 ,itemdefdobj("testset").list().escapeDecode()
			 );
  else
    throw Except(Sprintf("unimplemented scalar/array type 0x%lx"
				  ,d.scalarType()));
  
  
  
  if(d.isArray()) { 
    if(d.rank() != 1)
      throw Except(" must be a rank 1 array"); 
    
    if(d.isVariableSize()) 
      ptest->setFixedVector(0); 
    
  }else if(!d.isScalar())
    throw Except("unknown structure type");
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SingleItemDef::SingleItemDef(Dobj const &itemdefdobj ,StringList const &flags
			     ,Dobj const &rootdobj,long isfirstfielditem)
  :BaseItemDef(itemdefdobj,flags)
{
  NSF("SingleItemDef::SingleItemDef");

  pisfirstfielditem=isfirstfielditem;
  
  try { 
    
    Dobj d=rootdobj(pID);
    
    if( !d.isScalar() ){
      pflags.remove("trigger_update");
    }
    
    // manipulate the default value of the parse object
    // in order to have automatic default value creation on resizing
    if( d.isFlag() ){
      if( d.isScalar() ){
	String defv=d();
	defv.set(0,itemdefdobj("defaultvalue")()[0]);
	d.setDefaultValue(defv);
      }else
	throw Except(String()
		     .sprintf("Cannot use flag arrays in ITEMLISTs, "
			      "use CHECKs instead!\n"));
      
    }else{
      d.setDefaultValue(itemdefdobj("defaultvalue")());
    }


    if(pflags.contains("selector")){
      long ok=1;
      Dobj ds=d;
      
      ds=ds.first(); ok&=ds.validNode();
      ok&=( ds.isScalar() && ds.isInt() );
      ds=ds.next(); ok&=ds.validNode();
      ok&=( ds.isScalar() && ds.isString() );
      ds=ds.next(); ok&=!ds.validNode();
      
      if(!ok)
	throw Except("A ITEM | SELECTOR edit structure must refer to "
		     "an input data structure of the shape"
		     " struct{int ; char[*] ;} ");
      ptest=new SelectTest(
			   itemdefdobj("testflags")()+" | autocompletion"
			   ,itemdefdobj("testset").list().escapeDecode()
			   );
      
      String defv=itemdefdobj("defaultvalue")();
      defv.stripBlanks();
      d.first().setDefaultValue(defv);
      try{
	d.first().next().setDefaultValue(ptest->getSelectDescriptor(defv));
      }catch(...){;}
      
    }else{
      if( ! (d.isScalar() || d.isArray() ) )
	throw Except(" must be a scalar or a 1D array");

      setupTest(itemdefdobj,d);
    }

    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " ITEM[LIST] '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SingleItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
			      ,long tab)
{
  NSF("SingleItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj(pID);
    
    
    
    
    
    if(d.isScalar()){
      if(d.isFlag()){
	pbarerep=d()()+1;
	pscreenrep="("+photkey+") "+pbarerep;
	pmarkuprep="@("+photkey+") "+pbarerep;
      }
    }else if(d.isArray()){

      if(d.rank() != 1) throw Except(" must be a rank 1 array"); 
      
      
      
      
    }else if(pflags.contains("selector") && d.first().isScalar()){
      d=d.first();    
      
    }else
      throw Except(" must be a scalar or a 1D array");
    
    
    f->addItem(new EditItem(this,d,active,tab),pflags.contains("eol"));
    

    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): ITEM[LIST] '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
TitleItemDef::TitleItemDef(Dobj const &itemdefdobj ,StringList const &flags)
  :BaseItemDef(itemdefdobj,flags)
{
  photkey="";
  if(pfieldoffset<0)
    pfieldwidth=pbarerep.length();
  else{
    pfieldwidth=LMIN(pbarerep.length(),pfieldoffset);
  }
  pfieldoffset=0;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void TitleItemDef::addToForm(Form *f,Dobj const &,long,long tab)
{
  //  NSF("TitleItemDef::addToForm");
  
  f->addItem(new TitleItem(this,tab),pflags.contains("eol"));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
DashLineItemDef::DashLineItemDef(Dobj const &itemdefdobj ,StringList const &flags)
  :BaseItemDef(itemdefdobj,flags)
{
  photkey="";
  pfieldwidth=0;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void DashLineItemDef::addToForm(Form *f,Dobj const &,long,long tab)
{
  f->addItem(new DashLineItem(this,tab),pflags.contains("eol"));
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
CheckGroupItemDef::CheckGroupItemDef(Dobj const &itemdefdobj
				     ,StringList const &flags
				     ,Dobj const &rootdobj,long inbox)
  :BaseItemDef(itemdefdobj,flags),pinbox(inbox)
{
  NSF("CheckGroupItemDef::CheckGroupItemDef");
  
  try { 
    
    Dobj d=rootdobj.flagArp(pID);


    // we need a dummy test here, to avoid trouble with ptest==0
    ptest=new FlagTest(1);

    String v=itemdefdobj("defaultvalue")();
    if(!v.empty())d.setDefaultValue(v);
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " CHECK[BOX] '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CheckGroupItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
				  ,long tab)
{
  NSF("CheckGroupItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj.flagArp(pID);
    
    if(d.rank() != 1)
      throw Except(" must be a rank 1 flag array"); 
    
    if(d.hasIDRef())
      throw Except(String()
		   .sprintf("Cannot use CHECK[BOX] as action type for arrays"
			    " with a dimension specifier ('%s')"
			    ,d.IDRef()()));
    
    

    if(pinbox)
      if(pflags.contains("show_content")) {
	f->addItem(new CheckBoxEditItem(this,d,active,0,namePos(),tab)
		   ,pflags.contains("eol"));
	internalAddToForm(f,0,d,0,tab);
      }else
	f->addItem(new CheckBoxEditItem(this,d,active,1,namePos(),tab)
		   ,pflags.contains("eol"));
    else
      internalAddToForm(f,0,d,active,tab);
    
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): CHECK(BOX) '%s': %s\n"
			 ,nsubfunc,pID(),ex.msg()));
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CheckGroupItemDef::internalAddToForm(Form *f,long inbox,Dobj const &d,long active,long tab)
{
  //NSF("CheckGroupItemDef::internalAddToForm");
  

  long lc=1;
  if(pflags.contains("two_column"))
    lc=2;
  else if(pflags.contains("three_column"))
    lc=3;
  
  
  long np;
  if(inbox)
    np=1;
  else
    np=namePos();
  
  if(!pflags.contains("show_content") || !pinbox || inbox) 
    f->addItem(new CheckBoxEditItem(this,d,0,0,np,tab),1);

  
  for(long ia=0;ia<d.size();ia++){
    
    // 	if(incompound) itm->compounditem=1;
    // 	if(incompound){ itm->hotkey[0]='\0'; }
    
    long eol=0;
    if( ia%lc ==lc-1 || ia== d.size()-1) eol=1;
    
    f->addItem(new CheckGroupEditItem(this,d,active,ia,ia%lc,lc
				      ,pflags.contains("auto_width")
				      ,inbox,tab)
	       ,eol);
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SelectGroupItemDef::SelectGroupItemDef(Dobj const &itemdefdobj
				       ,StringList const &flags
				       ,Dobj const &rootdobj,long inbox)
  :BaseItemDef(itemdefdobj,flags),pinbox(inbox)
{
  NSF("SelectGroupItemDef::SelectGroupItemDef");
  
  try { 
    
    Dobj ds=rootdobj.stp(pID);
    
    
    {
      long ok=1;
      Dobj d=ds;
      
      
      d=d.first(); ok&=d.validNode();
      ok&=( d.isScalar() && d.isInt() );
      d=d.next(); ok&=d.validNode();
      ok&=( d.isScalar() && d.isString() );
      d=d.next(); ok&=!d.validNode();
      
      if(!ok)
	throw Except("A SELECT(BOX) edit structure must refer to "
		     "an input data structure of the shape"
		     " struct{int ; char[*] ;} ");
    }
    
    
    ptest=new SelectTest(
			 itemdefdobj("testflags")()
			 ,itemdefdobj("testset").list().escapeDecode()
			 );

    String defv=itemdefdobj("defaultvalue")();
    defv.stripBlanks();
    ds.first().setDefaultValue(defv);
    try{
      ds.first().next().setDefaultValue(ptest->getSelectDescriptor(defv));
    }catch(...){;}
    
    
    
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
				  " SELECT(BOX) '%s': variable '%s'\n %s\n"
				  ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SelectGroupItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
				   ,long tab)
{
  NSF("SelectGroupItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj.stp(pID);
    

    if(pinbox){
      if(pflags.contains("show_content")) {
	f->addItem(new SelectBoxEditItem(this,d,active,0,namePos(),tab)
		   ,pflags.contains("eol"));
	internalAddToForm(f,0,d,0,tab);
      }else
	f->addItem(new SelectBoxEditItem(this,d,active,1,namePos(),tab)
		   ,pflags.contains("eol"));
	
    }else
      internalAddToForm(f,0,d,active,tab);
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): SELECT(BOX) '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SelectGroupItemDef::internalAddToForm(Form *f,long inbox,Dobj const &d
					   ,long active,long tab)
{
  //NSF("SelectGroupItemDef::internalAddToForm");
  
  
  long lc=1;
  if(pflags.contains("two_column"))
    lc=2;
  else if(pflags.contains("three_column"))
    lc=3;
  
  
  long np;
  if(inbox)
    np=1;
  else
    np=namePos();
  

  if(!pflags.contains("show_content") || !pinbox || inbox) 
    f->addItem(new SelectBoxEditItem(this,d,0,0,np,tab),1);
  
  long kc=0;
  for(long k=0;k<ptest->testSetSize();k++){
    
    //     if(incompound) itm->compounditem=1;
    
    
    if(!ptest->testSet(k).ignore){ /* an alternative */
      
      long column=kc%lc;
      long eol=0;
      if( kc%lc ==lc-1 || k== ptest->testSetSize()-1) {
	eol=1;
	kc=0;
      }else
	kc++;
      
      f->addItem(new SelectGroupEditItem(this,d,active,k,column,lc
					 ,pflags.contains("auto_width")
					 ,inbox,tab)
		 ,eol);
      
    }else{/* a subtitle */
      
      
      f->addItem();
      if(kc!=0) f->addItem();
      f->addItem(new TitleItem(ptest->testSet(k).s,np,-1,tab),1);
      kc=0;
    }
  } /* for all testset members*/
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MSelectGroupItemDef::MSelectGroupItemDef(Dobj const &itemdefdobj
					 ,StringList const &flags
					 ,Dobj const &rootdobj,long inbox)
  :BaseItemDef(itemdefdobj,flags),pinbox(inbox)
{
  NSF("MSelectGroupItemDef::MSelectGroupItemDef");
  
  try { 
    
    
    Dobj ds=rootdobj.sap(pID);
    
    
    
    {
      long ok=1;
      Dobj d=ds.sapTemplate();
      
      ok&=ds.isVariableSize();
      d=d.first(); ok&=d.validNode();
      ok&=( d.isScalar() && d.isInt() );
      d=d.next(); ok&=d.validNode();
      ok&=( d.isScalar() && d.isString() );
      d=d.next(); ok&=!d.validNode();
      
      if(!ok)
	throw Except("An MSELECT(BOX) edit structure must refer to "
		     "an input data structure of the shape"
		     " struct{int ; char[*] ;}[*]  ");
      
    }

    ptest=new MSelectTest(
			  itemdefdobj("testflags")()
			  ,itemdefdobj("testset").list().escapeDecode()
			  );
    
    /* todo: set default value */
    
    
    
    
    
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " MSELECT(BOX) '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void MSelectGroupItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
				    ,long tab)
{
  NSF("MSelectGroupItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj.sap(pID);
    
    
    if(pinbox)
      if(pflags.contains("show_content")) {
	f->addItem(new MSelectBoxEditItem(this,d,active,0,namePos(),tab)
		   ,pflags.contains("eol"));
	internalAddToForm(f,0,d,0,tab);
      }else{
	f->addItem(new MSelectBoxEditItem(this,d,active,1,namePos(),tab)
		   ,pflags.contains("eol"));
      }
    else
      internalAddToForm(f,0,d,active,tab);
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): MSELECT(BOX) '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void MSelectGroupItemDef::internalAddToForm(Form *f,long inbox,Dobj const &d
					    ,long active,long tab)
{
  //NSF("MSelectGroupItemDef::internalAddToForm");
  
 
  
  long lc=1;
  if(pflags.contains("two_column"))
    lc=2;
  else if(pflags.contains("three_column"))
    lc=3;
  
  
  long np;
  if(inbox)
    np=1;
  else
    np=namePos();
      
  if(!pflags.contains("show_content") || !pinbox || inbox) 
    f->addItem(new MSelectBoxEditItem(this,d,0,0,np,tab),1);

  long kc=0;
  for(long k=0;k<ptest->testSetSize();k++){
    
    //     if(incompound) itm->compounditem=1;
    
    
    if(!ptest->testSet(k).ignore){ /* an alternative */
      
      long column=kc%lc;
      long eol=0;
      if( kc%lc ==lc-1 || k== ptest->testSetSize()-1) {
	eol=1;
	kc=0;
      }else
	kc++;
      
      f->addItem(new MSelectGroupEditItem(this,d,active,k,column,lc
					  ,pflags.contains("auto_width")
					  ,inbox,tab)
		 ,eol);
      
    }else{/* a subtitle */
      
      f->addItem();
      if(kc!=0) f->addItem();
      f->addItem(new TitleItem(ptest->testSet(k).s,np,-1,tab),1);
      kc=0;
    }
  } /* for all testset members*/
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FieldItemDef::FieldItemDef(Dobj const &itemdefdobj ,StringList const &flags
			   ,Dobj const &rootdobj)
  :BaseItemDef(itemdefdobj,flags)
{
  NSF("FieldItemDef::FieldItemDef");
  
  
  try { 
    
    pdata=rootdobj.sap(pID);
    
     
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " FIELD '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void FieldItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
			     ,long tab)
{
  NSF("FieldItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj.sap(pID);
    


    if(!pbarerep.empty()){
      // if the fieldname i given create a title
      f->addItem(new TitleItem(pbarerep,pnamepos,-1,tab),1);
      if(flags().contains("eol"))
	f->addItem();
    }
      
    // create the headers
    for(long ie=0;ie<bdhead.size();ie++){
      bdhead[ie]->addToForm(f,d,active,tab);
      f->addItem();
    }


    String prefix=photkey;
    
    // create the editable fields
    for(long ia=0;ia<d.size();ia++){
      for(long ie=0;ie<bd.size();ie++){
	if(ie==0){
	  if(active){
	    bd[ie]->setFieldName(Sprintf("@(%s%ld)\\t%s",prefix()
					 ,ia+1,bd[ie]->bareRepSaved()()));
	  }else{
	    bd[ie]->setFieldName(Sprintf("@(%s%ld)%s",prefix()
					 ,ia+1,bd[ie]->bareRepSaved()()));
	    
	  }
	}else{
	  bd[ie]->setHotkey(Sprintf("%s%ld",prefix(),ia+1));
	}
	bd[ie]->addToForm(f,d[ia],active,tab);
      }
      f->addItem();

    }
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): FIELD '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
CounterItemDef::CounterItemDef(Dobj const &itemdefdobj ,StringList const &flags
			       ,Dobj const &sapobj)
  :BaseItemDef(itemdefdobj,flags)
{
  NSF("CounterItemDef::CounterItemDef");
  try { 
    
    if(!sapobj.isVariableSize())
      throw Except(Sprintf("error(%s): variable '%s': COUNTERS are allowed"
			   " for variable size structarrays only!\n"
			   ,nsubfunc,sapobj.fullName()()));
    
    
    ptest=new IntTest(
		      itemdefdobj("intervalleft")()
		      ,itemdefdobj("intervalright")()
		      ,itemdefdobj("testflags")()
		      ,itemdefdobj("testset").list().escapeDecode()
		      ); 
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " ITEM[LIST] '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CounterItemDef::addToForm(Form *f,Dobj const &sapobj,long active
			       ,long tab)
{
  NSF("CounterItemDef::addToForm");
  
  try { 
    
    f->addItem(new CounterEditItem(this,sapobj,active,tab)
	       ,pflags.contains("eol"));
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): ITEM[LIST] '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
CompoundItemDef::CompoundItemDef(Dobj const &itemdefdobj ,StringList const &flags
			   ,Dobj const &rootdobj)
  :BaseItemDef(itemdefdobj,flags)
{
  NSF("CompoundItemDef::CompoundItemDef");
  
  
  try { 
    
    pdata=rootdobj.sap(pID);
    
     
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s):"
			 " FIELD '%s': variable '%s'\n %s\n"
			 ,nsubfunc,pbarerep(),pID(),ex.msg())); 
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CompoundItemDef::addToForm(Form *f,Dobj const &rootdobj,long active
				,long tab)
{
  NSF("CompoundItemDef::addToForm");
  
  try { 
    Dobj d=rootdobj.sap(pID);
    

    if(!pbarerep.empty()){
      // if the fieldname is given create a title
      f->addItem(new TitleItem(pbarerep,(active?1:pnamepos),-1,tab),1);
      if(flags().contains("eol"))
 	f->addItem();
    }
      
    // create the headers
    for(long ie=0;ie<bdhead.size();ie++){
      bdhead[ie]->addToForm(f,d,active,tab);
      f->addItem();
    }
    
    String prefix=photkey;
     // create the editable fields
     for(long ia=0;ia<d.size();ia++){
       
       f->addItem(new DashLineItem(this,tab),1);
       f->addItem(new CompoundElementEditItem(this,d[ia],active,prefix+(ia+1),tab),1);
       
       internalAddToForm(f,0,d[ia],0,tab);
       f->addItem();

     }
    
    
    
  }catch (Except &ex){ 
    throw Except(Sprintf("error(%s): COMPOUND '%s': %s\n"
				  ,nsubfunc,pID(),ex.msg()));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CompoundItemDef::internalAddToForm(Form *f,long inbox,Dobj const &d
					,long active,long tab)
{
  //NSF("CompoundItemDef::internalAddToFormc");

  String title=(pbarerep+" "+photkey+(d.index()+1)).upperCase();

  if(inbox){
    f->addItem(new TitleItem(title,(active?1:pnamepos),-1,tab),1);
    f->addItem();
  }
  
  for(long ie=0;ie<bd.size();ie++){
    bd[ie]->addToForm(f,d,active,(active?0:pnamepos+tab));
  }
  f->addItem();
  
}
/*EOF*/ 
