/* @(#)ScreenDefs.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SCREENDEFSXX_H
#define _SCREENDEFSXX_H 1


#include "Test.hh"
#include "PtrArray.hh"
#include "FString.hh"
#include "Parser.hh"
#include "Tools.hh"

class Form;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class BaseItemDef
{
public:
  BaseItemDef(StringList const &flags)
    :pflags(flags),pnamepos(0),pfieldoffset(0),pfieldwidth(0),ptest(0)
    ,pisfirstfielditem(1){};
  BaseItemDef( Dobj const &itemdefdobj ,StringList const &flags)
    :pflags(flags),ptest(0),pisfirstfielditem(1)
  {
    pID=itemdefdobj("variablename")().escapeDecode();
    pnamepos=itemdefdobj("namepos")().convertToLong();
    pfieldoffset=itemdefdobj("fieldoffset")().convertToLong();
    pfieldwidth=itemdefdobj("fieldwidth")().convertToLong();
    


    setFieldName(itemdefdobj("fieldname")().escapeDecode());

    if(flags.contains("fixed")){
      photkey="";
      pscreenrep=pbarerep;
      pmarkuprep=pbarerep;
    }
    
    pbarerepsave=pbarerep;

  };
  virtual ~BaseItemDef(){ if(ptest) delete ptest;  };
  //
  void setFieldName(String const &fn){
    extractMarkup(fn,photkey,pscreenrep,pmarkuprep,pbarerep);
  };
  void setHotkey(String const &h){photkey=h;};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0)=0;
  //
  static String const actionType(String const a,StringList &flags);
  //
  String const &markupRep()const {return pmarkuprep;};
  String const &bareRep()const {return pbarerep;};
  String const &bareRepSaved()const {return pbarerepsave;};
  String const &screenRep()const {return pscreenrep;};
  String const &hotkey()const {return photkey;};
  String const &ID()const {return pID;};
  long isFirstFieldItem()const {return pisfirstfielditem;}
  long width()const {return pfieldwidth;};
  long offset()const {return pfieldoffset;};
  long namePos()const {return pnamepos;};
  Test *test()const {return ptest;};
  StringList const &flags()const {return pflags;};
  //
protected:
  //
  void setupTest(Dobj const &itemdefdobj,Dobj const &itemdataobj);
  //
  //
protected:
  //
  StringList pflags;
  String pID;
  String pscreenrep;
  String pmarkuprep;
  String pbarerep;
  String pbarerepsave;
  String photkey;
  long pnamepos;
  long pfieldoffset;
  long pfieldwidth;
  Test *ptest;
  long pisfirstfielditem;
  static StringList const knownActions;
  static StringList const knownFlags;
};




/* ***********************************************************************
 *
 * ***********************************************************************
 */

class TitleItemDef: public BaseItemDef
{
public:
  TitleItemDef(StringList const &flags):BaseItemDef(flags){};
  TitleItemDef(Dobj const &itemdefdobj ,StringList const &flags);
  virtual ~TitleItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  //
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class DashLineItemDef: public BaseItemDef
{
public:
  DashLineItemDef(Dobj const &itemdefdobj ,StringList const &flags);
  virtual ~DashLineItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  //
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */

class SingleItemDef: public BaseItemDef
{
public:
  SingleItemDef(Dobj const &itemdefdobj ,StringList const &flags
		,Dobj const &rootdobj,long isfirstfielditem=1);
  virtual ~SingleItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  //
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CheckGroupItemDef: public BaseItemDef
{
public:
  CheckGroupItemDef(Dobj const &itemdefdobj ,StringList const &flags
		    ,Dobj const &rootdobj,long inbox=0);
  virtual ~CheckGroupItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  virtual void internalAddToForm(Form *f,long inbox,Dobj const &d,long active
				 ,long tab);
  //
private:
  long pinbox;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class SelectGroupItemDef: public BaseItemDef
{
public:
  SelectGroupItemDef(Dobj const &itemdefdobj ,StringList const &flags
		     ,Dobj const &rootdobj,long inbox=0);
  virtual ~SelectGroupItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  virtual void internalAddToForm(Form *f,long inbox,Dobj const &d,long active
				 ,long tab);
  //
private:
  long pinbox;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class MSelectGroupItemDef: public BaseItemDef
{
 public:
  MSelectGroupItemDef(Dobj const &itemdefdobj ,StringList const &flags
		     ,Dobj const &rootdobj,long inbox=0);
  virtual ~MSelectGroupItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  virtual void internalAddToForm(Form *f,long inbox,Dobj const &d,long active
				 ,long tab);
  //
private:
  long pinbox;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class FieldItemDef: public BaseItemDef
{
public:
  FieldItemDef(Dobj const &itemdefdobj ,StringList const &flags
	       ,Dobj const &rootdobj);
  virtual ~FieldItemDef(){
    bd.clear();
    bdhead.clear();
  };
  //
  Dobj &data() {return pdata;};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  //
  BaseItemDef *appendToHeader(BaseItemDef *o){ bdhead.append(o); return o; };
  BaseItemDef *append(BaseItemDef *o){ bd.append(o); return o; };
  //
  //
private:
  PtrArray<BaseItemDef> bdhead;
  PtrArray<BaseItemDef> bd;
  Dobj pdata;

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class CompoundItemDef: public BaseItemDef
{
public:
  CompoundItemDef(Dobj const &itemdefdobj ,StringList const &flags
		  ,Dobj const &rootdobj);
  virtual ~CompoundItemDef(){
    bd.clear();
    bdhead.clear();
  };
  //
  Dobj &data() {return pdata;};
  // 
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  virtual void internalAddToForm(Form *f,long inbox,Dobj const &d,long active
				 ,long tab);
  //
  BaseItemDef *appendToHeader(BaseItemDef *o){ bdhead.append(o); return o; };
  BaseItemDef *append(BaseItemDef *o){ bd.append(o); return o; };
  //
  //
private:
  PtrArray<BaseItemDef> bdhead;
  PtrArray<BaseItemDef> bd;
  Dobj pdata; 
  
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */

class CounterItemDef: public BaseItemDef
{
public:
  CounterItemDef(Dobj const &itemdefdobj ,StringList const &flags
		 ,Dobj const &sapobj);
  virtual ~CounterItemDef(){};
  //
  virtual void addToForm(Form *f,Dobj const &rootdobj,long active,long tab=0);
  //
 private:
  long ptriggerupdate;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class MenuDef
{
public:
  MenuDef(String const & a,String const &n,String const &d)
    :pname(n),pdata(d) {  setActions(a); };
  String const &type()const {return ptype;};
  String const &action()const {return paction;};
  StringList const &flags()const {return pflags;};
  String const &name()const {return pname;};
  String const &data()const {return pdata;};
  //
private:
   void setActions(String const & a) ;
private:
  String ptype;  		// an specifier for the menu action
  String paction;		// an specifier for the menu action
  StringList pflags;		// an specifier for the menu action flags
  String pname;			// the token decribing the visible menu item
  String pdata;			// some data passed on to the app
  //
  static StringList const knownTypes;
  static StringList const knownActions;
  static StringList const knownFlags;
};




/* ***********************************************************************
 *
 * ***********************************************************************
 */

class ScreenDef
{
public:
  ScreenDef() :pct(-1) {};
  ~ScreenDef(){clear();};
  //
  void clear();
  //
  String const &screenName()const{return screenname;};
  void setScreenName(String const & n){screenname=n;};
  //
  long ptabIndex()const{return pct;};
  void  setPtabIndex(long p){pct=p;};
  //
  //
  //
  MenuDef *append(MenuDef *o){ md.append(o); return o; };
  long msize()const {return md.size();};
  MenuDef const *menu(long i)const {
    return const_cast<PtrArray<MenuDef>&>(md)[i];};
  //
  BaseItemDef *append(BaseItemDef *o){ bd.append(o); return o; };
  BaseItemDef *baseitem(long i) { return (bd)[i];};
  long bsize()const {return bd.size();};
  //
private:
  String screenname;
  long     pct;
  PtrArray<MenuDef> md;
  PtrArray<BaseItemDef> bd;
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */

class ScreenDefs
{
public:
  ScreenDefs() {
  };
  ~ScreenDefs(){ clear(); };
  //
  //
  //
  ScreenDef *operator[](long i) const{ 
    return const_cast<PtrArray<ScreenDef>&>(sd)[i];};
  //
  ScreenDef *append(ScreenDef *o) { sd.append(o);return o; };
  //
  ScreenDef const *byName(String const &sname) const;
  //
  long size()const {return sd.size();};
  //
  void clear(){
    sd.clear();
  };
  //
private:
  PtrArray<ScreenDef> sd;
};

#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _SCREENDEFSXX_H */


