/*************************************************************************
 * @(#)Test.cpp author: Klaus.Koepernik@ifw-dresden.de date: 03 Oct 2008
 *************************************************************************
 */

#include <ctype.h>

#include "Test.hh"
#include "FString.hh"
#include "Tools.hh"


long MAX_MOMENTUM=3;

class FlagTest TestFlag(1L);

// {{{ Test


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Test::Test()
  :classname("Test"),dataname("")
  ,toggle(0),leftopen(0),rightopen(0)
  ,fixedsize(1),state(0),strlen(1),pautocompletion(0),noWS(0)
{
  left.i=I_NEG_INFINITY;
  right.i=I_POS_INFINITY;	/* todo */
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Test::setTestFlags(String const &test)
{
  
  
  StringList sl=test.breakup("|");
  
  
  for(long i=0;i<sl.size();i++){
    
    String s=sl[i].stripBlanks().toLower();
    
    if(s.contains(" \t"))
      throw Except(Sprintf("error(setTests): test flags '%s': flags"
				    " must be separated by '|'!",test()));
    
    
    if(s=="toggle")
      toggle=1;
    else if(s=="left_open")
      leftopen=1;
    else if(s=="right_open")
      rightopen=1;
    else if(s=="state")
      state=1;
    else if(s=="autocompletion")
      pautocompletion=1;
    else if(s=="word")
      noWS=1;
    else if(s.length())
      throw Except(String()
		   .sprintf("test flags '%s': unknown flag !",test()));
    
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList Test::valueList(String const &value,long,long) const
{
 
  return   String(value).expandComma().split(" \t");
}

/* ***********************************************************************
 * test of integer| real|fractional|logical|string n-vector with conditions
 * , seperators are expanded  
 * ***********************************************************************
 */


void Test::validate(String &value,long count,long editing)  const
{
  
  //  NSF("validate");

  StringList srclist=valueList(value,count,editing);
  
  long n=srclist.size();

  if(!fixedsize) count=n; 
  
  if(n != count) 
    throw Except(Sprintf(": must be %ld %s%c!",count,dataname()
				  ,(count==1?' ':'s')));
  
  
  while(n--){
    testIt(srclist[n],count,editing);
  }
  
  value=srclist.join(" ");

}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Test::isToggle() const 
{
  long tg=1;
  
  tg=tg && toggle;
  
  //  tg=tg && (tset->size()==2 || tset->size()==0);
  
  
  return tg;
}






// }}}

// {{{ TestSet

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long TestSet::isInSetInsensitiveTruncate(const String & value) const 
{
  long index=-1;
  String v(value);
  v.toUpper().stripBlanks();
  long l=v.length();
  
  for(long k=0;k<tsm.size();k++){ 
    String t=const_cast<ValueArray<TestSetMember>&>(tsm)[k].s;
    if(l<=t.length()){
      if(String::cmp(v,t.toUpper().stripBlanks(),l)==0){
	index=k;
	break;
      } 
    }
  }
  return index;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long TestSet::isInSetInsensitive(const String & value) const 
{
  String v(value);
  v.toUpper().stripBlanks();
  long index=-1;
  for(long k=0;k<tsm.size();k++){ 
    String t=const_cast<ValueArray<TestSetMember>&>(tsm)[k].s;
    if(v==t.toUpper().stripBlanks()){
      index=k;
      break;
    } 
  }
  return index;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long TestSet::isInSet(FPLOINTEGER l)  const 
{
  long found=-1;
  for(long k=0;k<tsm.size();k++){ 
    if(const_cast<ValueArray<TestSetMember>&>(tsm)[k].ignore) continue;
    if(const_cast<ValueArray<TestSetMember>&>(tsm)[k].i==l){
      found=k;
      break;
    } 
  }
  return found;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long TestSet::isInSet(FPLODOUBLEPRECISION r) const 
{
  long found=0;
  for(long k=0;k<tsm.size();k++){ 
    if(const_cast<ValueArray<TestSetMember>&>(tsm)[k].d==r){ 
      found=1;
      break;
    } 
  }
  return found;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TestSet::initInt(StringList const &set)
{
  
  for(long k=0;k<set.size();k++){
    
    long l=0,ok=1;
    String s=set[k];
    
    ok&=!s.stripBlanks().contains(" \t");
    
    try{ l=s.convertToLong(); }catch (...){ok=0;}
    
    tsm.resize(k+1);
    
    if(!ok) 
      throw Except(Sprintf(": test set contains invalid integer"
				    " value '%s'",tsm[k].s()));
    
    
    tsm[k].s=s;
    tsm[k].i=l;
  }
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TestSet::initReal(StringList const &set)
{
  
  for(long k=0;k<set.size();k++){
    
    long ok=1;
    double d=0;
    String s=set[k];
    
    ok&=!s.stripBlanks().contains(" \t");
    
    try{ d=s.convertToFortranReal(); }catch (...){ok=0;}
    
    
    if(!ok) 
      throw Except(Sprintf(": test set contains invalid integer"
				    " value '%s'",tsm[k].s()));
    
    tsm.resize(k+1);

    tsm[k].s=s;
    tsm[k].d=d;
  }
  return;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TestSet::initString(StringList const &set,long slen)
{

  for(long k=0;k<set.size();k++){

    String s=set[k];
    s.stripBlanks();

    if(s.contains(" \t"))
      throw Except(Sprintf(": test set value '%s' "
				    "is not a single word.\n"
				    ,s()));
    
    if(slen >=0 ){    
      if(s.length() > slen )
	throw Except(String()
		     .sprintf(": test set contains value '%s' which"
			      " is longer than the declared string length of "
			      "the data\n",s()));
    
    }
    tsm.resize(k+1);
    tsm[k].s=s;

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TestSet::initSelect(StringList const &set)
{

  long l=0,ok=1;

  if(set.size()<1)
    throw Except(Sprintf(":SELECT must "
				  "have at least one element in test set!"));

  for(long k=0;k<set.size();k++){
    /* normalize testset */
    StringList sl=set[k].breakup("\\n");
    if( sl.size()==1 ){
      tsm.resize(k+1);
      tsm[k].s=set[k];
      tsm[k].ignore=1;
      continue;
    }
    if(sl.size()>1) 
      try{ l=sl[0].convertToLong(); }catch (...){ ok=0; }

    if(sl.size()!=2 || (!ok) )
      throw Except(Sprintf(": the test set element '%s...' must "
				    "have the structure \"int-number \\n "
				    "descriptor \"!"
				    ,set[k]()));

    tsm.resize(k+1);
    tsm[k].i=l;
    tsm[k].s=sl[1];

    extractMarkup(sl[1],tsm[k].photkey,tsm[k].pscreenrep
		  ,tsm[k].pmarkuprep,tsm[k].pbarerep);

    
  }
}





// }}}


// {{{LogicalTest

/* ***********************************************************************
 *
 * ***********************************************************************
 */
LogicalTest::LogicalTest(long tg)
  :Test()
{
  classname="LogicalTest";
  dataname="logical";
  toggle=tg;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LogicalTest::testIt(String &value,long count,long editing)  const
{
  char c=tolower(value[0]);
  long ok = ((value.length() == 1 )  && (c=='t'||c=='f'||c=='+'||c=='-'));
  
  if(!ok){
    throw Except(String()
		 .sprintf(": must be %ld logical%s (\'t\' or \'f\')!"
			  ,count,(count>1?"s":"")));
  }
  
  value=c;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String LogicalTest::toggleIt(String const &v)  const
{
  String s=v;
  if(s.length()<1) s='t';
  else{
    if(s[0]=='t' || s[0]=='+')
      s="f";
    else if(s[0]=='f' || s[0]=='-')
      s="t";
  }
  return s;
}

// }}}

// {{{FlagTest

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FlagTest::FlagTest(long tg)
  :Test()
{
  classname="FlagTest";
  dataname="flag";
  toggle=tg;
  return ;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FlagTest::testIt(String &value,long ,long editing)  const
{
  long ok=(value.length()==1) && (value[0]=='+' || value[0]=='-');
  
  if(!ok){
    throw Except(": must be '+' or '-'!");
  }
  return ;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String FlagTest::toggleIt(String const &v)  const
{
  String s=v;
  if(s.length()<1) s='+';
  else
    s=(s[0]=='+' ? "-" : "+");

  return s;

}

// }}}

// {{{IntTest



/* ***********************************************************************
 *
 * ***********************************************************************
 */
IntTest::IntTest(String const &l,String const &r,String const &flags
		 ,StringList const &testset)
  :Test()
{
  classname="IntTest";
  dataname="integer";
  
  left.i=valFromString(l);
  right.i=valFromString(r);

  setTestFlags(flags);

  tset.initInt(testset);

  if(tset.size()>0){
    if(tset.size()!=2) toggle=0;
  }else{
    toggle=0;
  }

  return ;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FPLOINTEGER IntTest::valFromString(String ss)
{
  if(ss=="+INF"){
    return I_POS_INFINITY;
  }else if(ss=="-INF"){
    return I_NEG_INFINITY;
  }else
    return ss.convertToLong();
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
const String IntTest::intervalToString ()  const
{
  String s;
  s+="[";
  if(left.i==I_POS_INFINITY){
    s+="+INF";
  }else if(left.i==I_NEG_INFINITY){
    s+="-INF";
  }else{
    s+=Sprintf(FPLOINTEGERFORMAT,left.i);
  }
  s+=",";
  if(right.i==I_POS_INFINITY){
    s+="+INF";
  }else if(right.i==I_NEG_INFINITY){
    s+="-INF";
  }else{
    s+=Sprintf(FPLOINTEGERFORMAT,right.i);
  }
  s+="]";
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IntTest::isInIntervall(FPLOINTEGER l) const
{
  return ((l>=left.i)&&(l<=right.i));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String IntTest::toggleIt(String const &v) const
{
  String s=v;
  if(testSet(0).s==s){
    s=testSet(1).s;
  }else{
    s=testSet(0).s;
  }

  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IntTest::testIt(String &value,long count,long editing) const
{
  long l=0;
  long ok=1;

  try{ l=value.convertToLong(); }catch (...){ ok=0; }
  
  if(tset.size()<1){
    ok&=isInIntervall(l);
    if(!ok){
      throw Except(Sprintf(": must be %ld integer in %s !"
				    ,count,intervalToString()() ));
  }
  }else{
    ok&=(tset.isInSet(l)>=0);
    if(!ok){
      throw Except(Sprintf(": must be %ld integer, element of "
			   "test set (in fedit-file) !"
			   ,count ));
    }
  }
  
  
}

// }}}

// {{{RealTest


/* ***********************************************************************
 *
 * ***********************************************************************
 */
RealTest::RealTest(String const &l,String const &r,String const &flags
		 ,StringList const &testset)
  :Test()
{
  classname="RealTest";
  dataname="real";
  
  left.r=valFromString(l);
  right.r=valFromString(r);

  setTestFlags(flags);

  tset.initReal(testset);

  if(tset.size()>0){
    if(tset.size()!=2) toggle=0;
  }else{
    toggle=0;
  }

  return ;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FPLODOUBLEPRECISION RealTest::valFromString(String ss)
{
  if(ss=="+INF"){
    return R_POS_INFINITY;
  }else if(ss=="-INF"){
    return R_NEG_INFINITY;
  }else
    return ss.convertToFortranReal();
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
const String RealTest::intervalToString ()  const
{
  String s;
  s+=(leftopen?"(":"[");
  if(left.r==R_POS_INFINITY){
    s+="+INF";
  }else if(left.r==R_NEG_INFINITY){
    s+="-INF";
  }else{
    s+=Sprintf("%g",left.r);
  }
  s+=",";
  if(right.r==R_POS_INFINITY){
    s+="+INF";
  }else if(right.r==R_NEG_INFINITY){
    s+="-INF";
  }else{
    s+=Sprintf("%g",right.r);
  }
  s+=(rightopen?")":"]");
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long RealTest::isInIntervall(FPLODOUBLEPRECISION r) const
{
  long ret=1;
  
  if(leftopen) 
    ret&=(r>left.r);
  else
    ret&=(r>=left.r);
  
  if(rightopen) 
    ret&=(r<right.r);
  else
    ret&=(r<=right.r);
  
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RealTest::testIt(String &value,long count,long editing) const 
{
  //NSF("testIt");
  double d=0.;
  long ok=1;

  
  try{ d=value.convertToFortranReal(); }catch (...){ ok=0; }
  
  
  if(tset.size()<1){
    ok&=isInIntervall(d);
    if(!ok)
      throw Except(Sprintf(": must be %ld real in %s !"
				    ,count,intervalToString()() ));
    
  }else{
    ok&=tset.isInSet(d);
    if(!ok)
      throw Except(Sprintf(": must be %ld real, element of "
			   "test set (in fedit-file) !"
			   ,count ));
    
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String RealTest::toggleIt(String const &v) const
{
  String s=v;
  if(testSet(0).s==s){
    s=testSet(1).s;
  }else{
    s=testSet(0).s;
  }
  return s;
  
}

// }}}

// {{{StringTest

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringTest::StringTest(long variable,long slen,String const &flags
		       ,StringList const &testset)
  :Test()
{

  setTestFlags(flags);

  noWS=noWS || pautocompletion;

  dataname="string";
  if(slen ==1 && ! variable) dataname="char";
  if(noWS) dataname="string without whitespace";

  classname="StringTest";


  strlen=slen;
  if(variable)strlen=-1;


  tset.initString(testset,strlen);


  
}


/* ***********************************************************************
 * test values string length or if value is in testset
 * the length test for testset members is not performed, this must be done
 * while loading the items, its an IO issue
 * If testset is  defined, the value is replaced with the testset value
 * thus the value does not has to be of proper case
 * this implies that the testset is not case sensitive, if testset contains
 * 'A' and 'a', the value 'a' will be replaces by 'A'
 * 'a' and 'A', the value 'A' will be replaces by 'a'
 * ***********************************************************************
 */
void StringTest::testIt(String &value,long count,long editing) const 
{
  
 
  if(tset.size()>0){
    

    long index;
    if((index=tset.isInSetInsensitive(value))<0){
      /* if no exact match, lets make completion */
      
      if(pautocompletion && editing){
	index=tset.isInSetInsensitiveTruncate(value);
      }      
      
      if(index<0)
	throw Except(Sprintf(": string(s) must be in test set, "
			     "defined in fedit-file!")); 
      
    }
    // return normalized value
    value=testSet(index).s;

  }else{
    
    if(state){
      value.stripBlanks();
    }
    if(strlen >= 0){
      if( value.length() > strlen)
        throw Except(Sprintf(": string(s) to long, maximum is %ld!"
				      ,strlen)); 
      
    }
    
    if(state){
      char p[2];
      p[0]=value[0]; p[1]='\0';
      
      if( value.length() != 2) 
	throw Except(": state incomplete");

      long l;
      if (!String::islong(&l,p,1L,8L)) 
	throw Except(": unknown main qn.");

      if(!String::isSPDF(value[1],MAX_MOMENTUM)) 
	throw Except(": unknown momentum");
      
    }
  }
   
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String StringTest::toggleIt(String const &v) const 
{
  
  String s=v;
  if(testSet(0).s==s){
    s=testSet(1).s;
  }else{
    s=testSet(0).s;
  }
  return s;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList StringTest::valueList(String const &value,long count,long ed) const
{
  //NSF("valueList");


  StringList l;
  if( count==1 && fixedsize &&  !noWS ){
    return StringList(value);
  }else{
    if(noWS && value.empty()){
      return StringList("");
    }else
      return String(value).expandComma().split(" \t");
  }
}

// }}}

// {{{SelectTest

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SelectTest::SelectTest(String const &flags,StringList const &testset)
  :Test()
{

  classname="SelectTest";
  dataname="select";

  setTestFlags(flags);

  tset.initSelect(testset);
  
  return ;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SelectTest::testIt(String &value,long count,long editing) const
{
  //NSF("testIt");
  long l=0;
  long ok=1;
  
  
  try{ l=value.convertToLong(); }catch (...){ ok=0; }
  
  ok&=(tset.isInSet(l)>=0);
  
  if(!ok)
    throw Except(Sprintf(": select must be in set of "
				  "possible values, defined in fedit-file!" ));
  

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

String const& SelectTest::getSelectDescriptor(String const& value) const
{
  
  if( tset.size() >0 ){
    
    for(long i=0;i< tset.size();i++){
      
      if(testSet(i).ignore)continue;
      
      if(value==testSet(i).i ) {
	
	return testSet(i).pbarerep;
	
      }
      
    }
    throw Except("internal(getSelectDescriptor): not found!\n");
  }
  
  throw Except("internal(getSelectDescriptor): test set is empty!\n");
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

StringList SelectTest::getSelectDescriptors() const
{
  StringList sl;
  if( tset.size() >0 ){
    
    for(long i=0;i< tset.size();i++){
      
      if(testSet(i).ignore)continue;
      
      sl.append(testSet(i).s);
	
	
	
    }
  }
  return sl;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

StringList SelectTest::getSelectDescriptorHints() const
{
  StringList sl;
  if( tset.size() >0 ){
    
    for(long i=0;i< tset.size();i++){
      
      if(testSet(i).ignore)continue;
      
      sl.append(testSet(i).s);
	
	
	
    }
  }
  StringList slo;
  for(int i=0;i<sl.size();i++){
    String s=sl[i];
    long l=s.length();
    long n=-1;

    for(int k=0;k<l;k++){

      String o=s.subString(0,k);
      long index=tset.isInSetInsensitiveTruncate(o);
      if(index==i) {
	n=k;
	break;
      }

    }
    if(n<0 || n>=s.length()-1)
      slo.append(s);
    else 
      slo.append(s.subString(0,n)+"["+s.subString(n+1)+"]");
  }
  return slo;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long SelectTest::mapToDescriptor(String &value) const
{
  long l=0;
  if(!String::islong(&l,value())) l=testSet(0).i; // correct data

  long index=0;
  if((index=tset.isInSet(l))<0){
    index=0;
  }

  value=getSelectDescriptor(index);

  return testSet(index).i;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long SelectTest::mapFromDescriptor(const String &value) const
{
  long index=0;
  if((index=tset.isInSetInsensitive(value))<0){
    index=0;
  }
  return testSet(index).i;
  
}

// }}}

// {{{MSelectTest

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MSelectTest::MSelectTest(String const &flags,StringList const &testset)
  :Test()
{

  classname="MSelectTest";
  dataname="multiple select";
  fixedsize=0;

  setTestFlags(flags);

  tset.initSelect(testset);
  
  return ;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MSelectTest::testIt(String &value,long count,long editing) const
{
  //NSF("MSelectTest::testIt");
 
  ValueArray<long> vpo=value.splitToLongArray(" \t");
  
  ValueArray<long> rpo;
  
  for(long j=0;j<vpo.size();j++){
      
    if(tset.isInSet(vpo[j]) >=0) {
      rpo.append(vpo[j]) ;//todo must be error ??
    }
      
  }
  rpo.unify();
  
  value="";
  for(long j=0;j<rpo.size();j++) {
    value+=String(rpo[j]);
    if(j<rpo.size()-1) value+=" ";
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList MSelectTest::valueList(String const &value,long count,long ed) const
{
  //NSF("valueList");
  count=1;
  return StringList(value);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

String const& MSelectTest::getSelectDescriptor(String const& value) const
{

  if( tset.size() >0 ){
    
    for(long i=0;i< tset.size();i++){
      
      if(testSet(i).ignore)continue;
      
      if(value==testSet(i).i) {
	
	return testSet(i).pbarerep;
	
      }
      
    }
    throw Except("internal(getSelectDescriptor): not found!\n");
  }
  
  throw Except("internal(getSelectDescriptor): test set is empty!\n");
  
}

// }}}




/*EOF*/ 



