/* @(#)Test.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _TESTXX_H
#define _TESTXX_H 1


#include "fortrantypes.h"
#include "debugxx.hh"

#include "MyDebug.hh"
#include "ValueArray.hh"
#include "Except.hh"
#include "FString.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class TestSetMember
{
public:
  TestSetMember(): ignore(0),s(""),d(0.),i(0){};
  TestSetMember const &operator=(long){
    s="";ignore=0;d=0.;i=0;
    return *this;
  };
public:
  long ignore;
  String s;
  String pbarerep;
  String pscreenrep;
  String pmarkuprep;
  String photkey;
  FPLODOUBLEPRECISION d;
  FPLOINTEGER i;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class TestSet //todo reduce to simpler form
{
public:
  TestSet(){
  };
  //
  long size() const {return tsm.size();};
  //
  long isInSet(FPLOINTEGER l)  const ; 
  long isInSet(FPLODOUBLEPRECISION r) const ; 
  long isInSetInsensitive(const String & v) const ;
  long isInSetInsensitiveTruncate(const String & v)  const ;
  //
  void initInt(StringList const &set);
  void initReal(StringList const &set);
  void initString(StringList const &set,long slen);
  void initSelect(StringList const &set);
  //
  //
  friend class Test;
private:
  ValueArray<TestSetMember> tsm;
};





/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Test
{
public:
  //
  Test();
  virtual ~Test(){};
  //
  //
  long isToggle() const ;
  //
  void setTestFlags(String const &f);
  //
  void setFixedVector(long a){fixedsize=a;};
  //
  //
  //
  TestSetMember const & testSet(long i) const   {
    return const_cast<ValueArray<TestSetMember>&>(tset.tsm)[i];};
  long testSetSize()   const {return tset.tsm.size();};
  //
  //
  void validate(String &value,long count,long editing) const;
  long autocompletion()const{return pautocompletion;};
  //
  virtual String toggleIt(String const &value)const {return value;};
  virtual String const& getSelectDescriptor(String const& value) const
  {return value;};
  //
  //
  //
  //
 private:
  virtual StringList valueList(String const &,long,long) const;
  virtual void testIt(String&,long count,long editing)  const{
    throw Except(Sprintf(": unknown test class %s!",classname()));
  };
 protected:
  String classname;
  String dataname;
  long toggle; /* is the editing action a toggling of two states */
  long leftopen;   /* a left open interval */
  long rightopen;   /* a left open interval */
  long fixedsize;   /* fixed number of vector components
		       although the data in the files say otherwise */
  long state;      /* a char[2] containing atomic state definitions  */

  //
  union {
    FPLOINTEGER   i;
    FPLODOUBLEPRECISION r;
  } left,right;


  /* length of strings: negativ means arbitrary length */
  long strlen;
  long pautocompletion; 
  long noWS; // string cannot contain whitespace
  
  /* a set of possible values */
  TestSet tset;
};




/* ***********************************************************************
 *
 * ***********************************************************************
 */
class StringTest: public Test
{
 public:  
  StringTest(long variable,long slen,String const &flags
	     ,StringList const &testset);
  virtual String toggleIt(String const &value)const;
  StringList valueList(String const &,long,long) const;
 private:
  void testIt(String&,long count,long editing)  const; 
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class RealTest: public Test
{
 public:  
  RealTest(String const &l,String const &r,String const &flags
	  ,StringList const &testset);
  virtual String toggleIt(String const &value)const;
 private:
  void testIt(String&,long count,long editing)  const; 
  FPLODOUBLEPRECISION valFromString(String ss);
  long isInIntervall(FPLODOUBLEPRECISION r) const;
  const String intervalToString() const;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class IntTest: public Test
{
 public:  
  IntTest(String const &l,String const &r,String const &flags
	  ,StringList const &testset);
  virtual String toggleIt(String const &value)const;
 private:
  //
  void testIt(String&,long count,long editing)  const; 
  FPLOINTEGER valFromString(String ss);
  //
  long isInIntervall(FPLOINTEGER l) const;
  const String intervalToString() const;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class FlagTest: public Test
{
 public:  
  FlagTest(long tg);
  virtual String toggleIt(String const &value)const;
 private:
  void testIt(String&,long count,long editing)  const; 
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class LogicalTest: public Test
{
 public:  
  LogicalTest(long tg);
  virtual String toggleIt(String const &value)const;
 private:
  void testIt(String&,long count,long editing)  const;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class SelectTest: public Test
{
 public:  
  SelectTest(String const &flags,StringList const &testset);

  String const& getSelectDescriptor(String const& value) const;
  StringList getSelectDescriptors() const;
  StringList getSelectDescriptorHints() const;
  long mapToDescriptor(String &value) const;
  long mapFromDescriptor(const String &value) const;
 private:
  void testIt(String&,long count,long editing)  const;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class MSelectTest: public Test
{
 public:  
  MSelectTest(String const &flags,StringList const &testset);
  StringList valueList(String const &,long,long) const;
  String const& getSelectDescriptor(String const& value) const; 
 private:
  void testIt(String&,long count,long editing)  const;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
extern class FlagTest TestFlag;

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _TESTXX_H */

