/*************************************************************************
 * @(#)Viewer.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   20 Oct 2008
 *************************************************************************
 */


#include "Viewer.hh"
#include "Menu.hh"
#include "Assorted.hh"
#include "ValueArray.hh"

// {{{HelpViewer

/* ***********************************************************************
 *
 * ***********************************************************************
 */
HelpViewer::HelpViewer(App *a,String const &title,String const &topic,SObject*p
		       ,String const &cn)
  :Viewer(a,"HELP",p,cn)
{

  FILE *file;  

  try{
    file=openforread(app->helpFile());
  }catch(Except &ex){
    throw Except("Can't find helpfile");
  } 
  
  String txt;
  String line;
  while(getLine(file,line)){ 
    if(line[0]=='#') continue;
    line.chop(); 
    if(isSectionMarker(line,topic)){
      long tabb=0;
      String buf;
      while(getLine(file,line)){ 
	if(line[0]=='#') continue;
	line.chop(); 

	if(isSectionMarker(line,""))
	  break;
	buf="";
	for (long j=0;j<tabb;j++) buf+="\t";
	
	long i;
	if( (i=line.find("=H1")) >=0 ){
	  tabb=1;
	  buf="A<";
	  buf+=line.subString(i+3).removeLeadingBlanks()
	    .removeTrailingBlanks()+">";
	}else if( (i=line.find("=H2")) >=0 ){
	  tabb=2; 
	  buf="\tB<";
	  buf+=line.subString(i+3).removeLeadingBlanks()
	    .removeTrailingBlanks()+">";
	}else if( (i=line.find("=H+")) >=0 ){
	  tabb++; 
	  buf="";
	  for (long j=0;j<tabb;j++) buf+="\t";
	  
	  buf+=line.subString(i+3).removeLeadingBlanks();
	}else if( (i=line.find("=H-")) >=0 ){
	  tabb--; 
	  buf="";
	  for (long j=0;j<tabb;j++) buf+="\t";
	  
	  buf+=line.subString(i+3).removeLeadingBlanks();
	}else{
	  buf+=line.removeLeadingBlanks();
	}

	buf=buf.breakup("\t").join("    ");
	
	txt+=buf+"\n";
      }
      break;
    } // if isSectionMarker
  } // while getLint


  setText(txt);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long HelpViewer::isSectionMarker(String const &str,String const &topic)
{

  
  StringList sl=str.breakup("M<");

  if(sl.size()!=2) return 0;

  StringList sl1=sl[1].breakup(">");

  if(topic.empty()){
    if( sl1.size()==2 ) return 1;
  }else{
    if(sl1[0]==topic) return  1;
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void HelpViewer::onLayout()
{
  //  NSF("subfunc");
  //  HIER(subfunc);
  

  clearItems();

  
  ValueArray<char> mode;

  
  
  long l=0;
  long rl=0;
  
  char *v=new char[3*buffer.length()+1];
  v[l]='\0';
  
  const char *p=buffer();
  
  while( (*p) != '\0'){
    if( (*p) =='\n'){

      addItem(new HelpLineItem(v));
      l=0;
      rl=0;
      for(long i=0;i<mode.size();i++){
	v[l++]=mode[i];
	v[l++]='<';
      }
      v[l]='\0';
    }else if( ( (*p) == 'U' || (*p) == 'B' || (*p) == 'A' || (*p) == 'R'  )
	      && (*(p+1)=='<')){
      mode.append(*p);
      v[l++]=*p++;
      v[l++]=*p;
      v[l]='\0';
    }else if( ( (*p)=='>') && ( mode.size()>0 ) ) {
      mode.removeLast();
      v[l++]='>';
      v[l]='\0';
    }else{
      rl++;
      v[l++]=*p;
      v[l]='\0';
      if(rl==app->s->cols()){
	addItem(new HelpLineItem(v));
	l=0;
	rl=0;
	for(long i=0;i<mode.size();i++){
	  v[l++]=mode[i];
	  v[l++]='<';
	}
	v[l]='\0';
      }
    }
    p++;
  }
  if(l){
    addItem(new HelpLineItem(v));
  }
  
  delete [] v;
  
  
  
  
  Page::onLayout();
}


// }}}

// {{{FileViewer

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FileViewer::FileViewer(App *a,String const &title,String const &fname,SObject*p
		       ,String const &cn)
  :Viewer(a,title,p,cn)
{


  new MenuItem(pmenu,"@>","","");
  new MenuItem(pmenu,"@(h)elp","help",screenname);


  FILE *FH;
  StringList sl;
  String s;


  try{
    FH=openforread(fname);
  }catch(Except &ex){
    String msg=Sprintf("internal(viewer): can not open file \'%s\'"
				": %s",fname(),ex.msg());
    setText(msg);

    return;
  }

  String txt;
  while(getLine(FH,s))
    txt+=s;
  
  (void)fclose(FH);

  setText(txt);
  
}

// }}}

// {{{Viewer

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Viewer::Viewer(App *a,String const &title,SObject*p,String const &cn)
  :Page(a,title,p,cn)
{
  //HIER("Viewer");

  new MenuItem(pmenu,"e@(x)it","exit","exit");

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Viewer::~Viewer()
{
  //HIER("~Viewer");
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Viewer::onLayout()
{
  //  NSF("subfunc");
  //  HIER(subfunc);
  StringList sl=buffer.breakup("\n");
  String st;
  StringList sl1;
  

  clearItems();

  for(long i=0;i<sl.size();i++){
    
    st=sl[i].breakup("\t").join("     ");
    
    sl1=st.wrapAround(app->s->cols());
    
    for(long j=0;j<sl1.size();j++){
      addItem(new LineItem(sl1[j]));
    } 
    
  }
  
  
  Page::onLayout();
}


// }}}


/*EOF*/ 
