/*************************************************************************
 * @(#)cfatband.cpp
 * author: magru@unas
 * date:   28 Oct 2013
 *************************************************************************
 */

#include "fortrantypes.h"
#include "cio.h"
#include "Assorted.hh"
#include "Exception.hh"
#include "SqMatrix.hh"
#include "MolecularOrbital.hh"
#include "CoeffBasisInfo.hh"
#include "Vec3.hh"

struct FCOMPLEX{
  FPLODOUBLEPRECISION r,i;  
};

static XFPLO::MolecularOrbitals globalmolorbdefs;
static XFPLO::CoeffBasisInfo globalcoeffbasisinfo;
static StringList globaloutput;

extern "C"{



  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  FPLOINTEGER CFUNC(cfatbandnumweights)()
  {
    return (FPLOINTEGER)(globalmolorbdefs.numWeights());
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  FPLOLOGICAL CFUNC(cfatbandissimple)(FPLOINTEGER *ibwd)
  {
    return (FPLOLOGICAL)(!globalmolorbdefs[(*ibwd)-1]->isMolecular());
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  FPLOLOGICAL CFUNC(cfatbandisactive)(FPLOINTEGER *ibwd)
  {
    return (FPLOLOGICAL)(globalmolorbdefs[(*ibwd)-1]->isActive());
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  FPLOINTEGER CFUNC(cfatbandsite)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->site()+1);
  }
  FPLOINTEGER CFUNC(cfatbandorbtype)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->orbtype());
  }
  FPLOINTEGER CFUNC(cfatbandnv)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->nmain());
  }
  FPLOINTEGER CFUNC(cfatbandlv)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->l());
  }
  FPLOINTEGER CFUNC(cfatbandjv)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->j());
  }
  FPLOINTEGER CFUNC(cfatbandmv)(FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    return (FPLOINTEGER)(globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->m());
  }
  void CFUNC(cfatbandaxes)(char *res,INTERNAL_INTEGER res_len,
			   FPLOINTEGER *icn,FPLOINTEGER *ibwd)
  {
    Sprintf("xaxis=%s zaxis=%s",
	    globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->xaxis().printS()(),
	    globalmolorbdefs[(*ibwd)-1]->at((*icn)-1)->zaxis().printS()()
	    ).convertToFortranChar(res,(long)res_len);
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  FPLOINTEGER CFUNC(cfatbandnumbw)()
  {
    return (FPLOINTEGER)(globalmolorbdefs.size());
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cfatbandweightlabel)(char *num,INTERNAL_INTEGER num_len,FPLOINTEGER *iw)
  {
    globalmolorbdefs.name((*iw-1)).convertToFortranChar(num,(long)num_len);
  }
  
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cbuildfatbandmatrix)(FPLODOUBLEPRECISION *qq,struct FCOMPLEX *res)
  {

    globalmolorbdefs.buildMatrix(Vec3(qq[0],qq[1],qq[2]),globalcoeffbasisinfo);
    const Cmplx *U=globalmolorbdefs.U();
    long Nw=globalmolorbdefs.numWeights();
    long nvdim=globalcoeffbasisinfo.nvdim();
    long i=0;
    for(long iw=-1;++iw<Nw;){
      for(long iz=-1;++iz<nvdim;){
	res[i].r=U[iz+nvdim*iw].real();
	res[i].i=U[iz+nvdim*iw].imag();
	i++;
      }
    }

  }
 
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cfatbandsetglobalspinzaxis)(FPLODOUBLEPRECISION *axis)
  {
    globalmolorbdefs.setGlobalSpinZaxis(Vec3(axis[0],axis[1],axis[2]));
  }
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cfatbandload)(const char *fn,char *currentbwfile,FPLOLOGICAL *err,
			   char *errormsg,
			   INTERNAL_INTEGER fn_len,
			   INTERNAL_INTEGER currentbwfile_len,
			   INTERNAL_INTEGER err_len)
  {
    String fname;
    fname.ncopy(fn,fn_len).removeTrailingBlanks();

    String pcurrentcoeffsfile,pcurrentbwfile,warn;

    *err=L_FALSE;
    globalmolorbdefs.clean();
    FILE *FH;
    try{
      FH=openforread(fname);
    }catch(Except &ex) {
      String(ex.msg()).convertToFortranChar(errormsg,(long)err_len);
      *err=L_TRUE;
      return;
    }
    try{
      warn=globalmolorbdefs.readFromFile(FH,pcurrentcoeffsfile,pcurrentbwfile,
					 true,globaloutput);
      globalmolorbdefs.setup(globalcoeffbasisinfo,true,globaloutput);
    }catch(Exception &ex) {
      ex.msg().convertToFortranChar(errormsg,(long)err_len);
      *err=L_TRUE;
      return;
    }

    pcurrentbwfile.convertToFortranChar(currentbwfile,(long)currentbwfile_len);

    fclose(FH);

    if(!warn.empty()){
      warn.convertToFortranChar(errormsg,(long)err_len);
      *err=L_TRUE;
      return;
    }
    
    return;

  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cgetoutputsize)(FPLOINTEGER *size)
  {
    *size=(FPLOINTEGER) (globaloutput.size());
  }
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cgetoutput)(FPLOINTEGER *index,char *line,INTERNAL_INTEGER line_len)
  {
    globaloutput[*index-1].chop("\n").convertToFortranChar(line,(long)line_len);
  }

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(csetcoeffbasisinfo)(FPLOINTEGER *nsite,FPLOINTEGER *isrt,char *el,
				 FPLOINTEGER *nRvmx,FPLOINTEGER *nRv,FPLOINTEGER *nnRv,
				 FPLOINTEGER *lRv,FPLOINTEGER *degRv,
				 FPLOINTEGER *blockstart,FPLOINTEGER *blocklength,
				 FPLOLOGICAL *rel,
				 INTERNAL_INTEGER el_len)
  {

    globalcoeffbasisinfo.clean();
    globalcoeffbasisinfo.setFREL(*rel);
    globalcoeffbasisinfo.setNsite(*nsite);
    for(long isa=-1;++isa<*nsite;){
      long issa=isrt[isa]-1;
      globalcoeffbasisinfo.elements()[isa].ncopy(el+issa*el_len,2);
      globalcoeffbasisinfo.nRv()[isa]=nRv[issa];
      globalcoeffbasisinfo.blockstart ()[isa]=blockstart[isa]-1;
      globalcoeffbasisinfo.blocklength()[isa]=blocklength[isa];
      globalcoeffbasisinfo.nnRv (isa).resize(nRv[issa]);
      globalcoeffbasisinfo.lRv  (isa).resize(nRv[issa]);
      globalcoeffbasisinfo.jRv  (isa).resize(nRv[issa]);
      globalcoeffbasisinfo.degRv(isa).resize(nRv[issa]);
      for(long ivr=-1;++ivr<nRv[issa];){
	globalcoeffbasisinfo.nnRv(isa)[ivr]=nnRv[ivr+(*nRvmx)*issa];
	globalcoeffbasisinfo.lRv(isa)[ivr]=lRv[ivr+(*nRvmx)*issa];
	globalcoeffbasisinfo.degRv(isa)[ivr]=degRv[ivr+(*nRvmx)*issa];
	globalcoeffbasisinfo.jRv(isa)[ivr]=degRv[ivr+(*nRvmx)*issa];
	if(*rel) globalcoeffbasisinfo.jRv(isa)[ivr]--;
      }
    }
    globalcoeffbasisinfo.setNvdim(globalcoeffbasisinfo.blockstart(*nsite-1)+
				  globalcoeffbasisinfo.blocklength(*nsite-1));


    // if(0){
    //   for(long isa=-1;++isa<globalcoeffbasisinfo.nsite();){
    // 	myDebug()<<globalcoeffbasisinfo.element(isa)<<globalcoeffbasisinfo.nRv(isa);
    // 	for(long ivr=-1;++ivr<globalcoeffbasisinfo.nRv(isa);){
    // 	  myDebug()<<ivr<<Sprintf("%1ld x %1ld%c%1ld/2 k=%2ld",
    // 				  globalcoeffbasisinfo.degRv(ivr,isa),
    // 				  globalcoeffbasisinfo.nnRv(ivr,isa),
    // 				  String::intToSpdf(globalcoeffbasisinfo.lRv(ivr,isa)),
    // 				  globalcoeffbasisinfo.jRv(ivr,isa),
    // 				  globalcoeffbasisinfo.kapRv(ivr,isa))
    // 		   <<"starts at "<<globalcoeffbasisinfo.blockstart(isa)
    // 		   <<"len "<<globalcoeffbasisinfo.blocklength(isa);
    // 	}
    //   }
    //   myDebug()<<"nvdim="<<globalcoeffbasisinfo.nvdim();
    //   exit(2);
    // }
  }
  
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  void CFUNC(cmkdir)(const char *dn,FPLOLOGICAL *err,char *errormsg
		     ,INTERNAL_INTEGER dn_len,INTERNAL_INTEGER err_len)
  {
    String dname;
    dname.ncopy(dn,dn_len).removeTrailingBlanks();

    *err=L_FALSE;

    try{
      createDirectory(dname,false);
    }catch(Except &ex) {
      // printf("%s",ex.msg());
      // exit(1);
      String(ex.msg()).convertToFortranChar(errormsg,(long)err_len);
      *err=L_TRUE;
      return;
    }

    
    return;
    
  }
  

}


