/* @(#)csym.c
 */


#include <stdlib.h>
#include "fortrantypes.h"
#include "merror.h"
#include "cbuffer.h"
#include "fsymmetry.h"
#include "cio.h"
#include "tools.h"
#include "allocate.h"
#include "generatetau.h"



static tsSYMMETRY sym;
static long initialized=0;
static const char *cbmsg=NULL;
static tsCBUFFER cb;
static double *taus=NULL;
static long *isrt=NULL;
static long nsite=0;
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgeterrormsg)(char *msg,FPLOINTEGER *index,INTERNAL_INTEGER msg_len)
{
  char c=errorgetbuffer()[*index];
  if(c=='\n'){
    *msg=(char)0x0d;
  }else{
    *msg=c ;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(ccalcsymmetry)
(FPLOINTEGER *spgrnum,FPLOINTEGER *str_type,FPLODOUBLEPRECISION *latticeconstants
 ,FPLODOUBLEPRECISION *axisangles
 ,FPLOINTEGER *nugen
 ,FPLOINTEGER *iugen
 ,FPLODOUBLEPRECISION *cell,FPLODOUBLEPRECISION *rcell,FPLODOUBLEPRECISION *ccell
 ,FPLODOUBLEPRECISION *vuc,FPLOINTEGER *pgrpnum,char *pgr
 ,FPLOLOGICAL *HEX
 ,FPLOLOGICAL *RTG
 ,char *lattice
 ,FPLOINTEGER *rcentering
 ,FPLOINTEGER *gcentering
 ,FPLOINTEGER * numops
 ,FPLODOUBLEPRECISION *opmat
 ,FPLODOUBLEPRECISION *optau
 ,FPLODOUBLEPRECISION *opgen
 ,FPLODOUBLEPRECISION *opeuler
 ,FPLOINTEGER         *opeulerinv
 ,FPLOINTEGER *opind
 ,FPLOINTEGER *inversion
 ,char *opsymbol
 ,FPLOINTEGER *err
 ,FPLOINTEGER *complain_subgroups
 ,INTERNAL_INTEGER pgr_len,INTERNAL_INTEGER lattice_len
 ,INTERNAL_INTEGER opsymbol_len)
{
  long i,j,in,ig,k,ix;
  double alat[3];
  double angle[3];
  double gdum[9],d;
  double gen[9*64];
  long  num;
  const long *Cr;
  const long *Cg;

  *err=0;
  /* catch errors into buffer */
  seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER);

  
  num=*spgrnum;
  for(i=0;i<3;i++){
    alat[i]=(double)(latticeconstants[i]);
    angle[i]=(double)(axisangles[i]);
  }

  sym.nugen=(long)*nugen;
  for(i=0;i<sym.nugen;i++){
    sym.iugen[i]=iugen[i];
  }

  /* calc symmetry info */
  *err=calcsym(&sym,num,"",alat,angle,(*complain_subgroups? 1L:0L),*str_type);
  if(*err>WARNING) return;

  Cr=bravais_centering(sym.bravais);
  Cg=bravais_centering(SYM_SIMPLE); 

  if(sym.bravais == SYM_FACECENTERED ){
    Cg=bravais_centering(SYM_BODYCENTERED); 
  }else if(sym.bravais == SYM_BODYCENTERED ){   
    Cg=bravais_centering(SYM_FACECENTERED); 
  }else if(sym.bravais == SYM_CBASECENTERED ){   
    Cg=bravais_centering(SYM_CBASECENTERED); 
  }else if(sym.bravais == SYM_ABASECENTERED ){   
    Cg=bravais_centering(SYM_ABASECENTERED); 
  }else if(sym.bravais == SYM_BBASECENTERED ){   
    Cg=bravais_centering(SYM_BBASECENTERED); 
  }else if(sym.bravais == SYM_RHOMBOHEDRAL ){   
    Cg=bravais_centering(SYM_RHOMBOHEDRAL); 
  }


  for(i=0;i<3;i++){
    latticeconstants[i]=(FPLODOUBLEPRECISION)sym.alat[i];
    axisangles[i]=(FPLODOUBLEPRECISION)sym.angle[i];
    for(j=0;j<3;j++){
      cell[i+3*j]=sym.cell[i+3*j];
      rcell[i+3*j]=sym.rcell[i+3*j];
      ccell[i+3*j]=sym.ccell[i+3*j];
      rcentering[i+3*j]=Cr[i+3*j];
      gcentering[i+3*j]=Cg[i+3*j];
    }
  }
  *vuc=(FPLODOUBLEPRECISION)sym.vuc;
  *pgrpnum=(FPLOINTEGER)sym.npgr;
  *RTG=(sym.RTG ? L_TRUE : L_FALSE);
  *HEX=(sym.hex ? L_TRUE : L_FALSE);
  *inversion=(FPLOINTEGER)(sym.inversion ? 1 : 0);
  *numops=sym.ng;

  for(ig=0;ig<sym.ng;ig++){
    in=sym.itable[ig];
    ix=operationindex(sym.opmat+9*in,sym.hex);
    
    if(!c2fstrncpy(opsymbol+ig*opsymbol_len
		   ,operationsymbols[ix+1],(long)opsymbol_len,0L)){
      errmsg("ccalcsymmetry",errorbuffer,SEVERE_ERROR);
      *err=SEVERE_ERROR;
      return;
    }

    transbtoa(gdum,sym.opmat+0+9*in,sym.hex);
    for(i=0;i<3;i++){
      for(j=0;j<3;j++){
	opmat[i+3*j+9*ig]=(FPLODOUBLEPRECISION)(gdum[i+3*j]);
      }
      for(k=0;k<3;k++){
	d=0.0;
	for(j=0;j<3;j++){
	  d+=sym.v[j+3*in] *sym.alat[j] *sym.scal[j] *sym.usheer[j+3*k];
	}
	optau[k+3*ig]=(FPLODOUBLEPRECISION)(d);
      }


    }
    opind[ig]=(FPLOINTEGER)(in)+1;
  }
  for(ig=0;ig<64;ig++){
    for(i=0;i<3;i++){
      opeuler[i+ig*3]=(FPLODOUBLEPRECISION)sym.eulerangles[i+ig*3];
    }
    opeulerinv[ig]=(FPLOINTEGER)(sym.eulerinv[ig]?1:0);
  }
  
  getrotations(gen);
  for(i=0;i<64*9;i++){
    opgen[i]=(FPLODOUBLEPRECISION)gen[i];
  }

  /* seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER); */
  if(!c2fstrncpy(pgr,pointgroup(sym.npgr),(long)pgr_len,0L)){
    errmsg("ccalcsymmetry",errorbuffer,SEVERE_ERROR);
    *err=SEVERE_ERROR;
    return;
  }
  if(!c2fstrncpy(lattice,sym.lattice,(long)lattice_len,0L)){
    errmsg("ccalcsymmetry",errorbuffer,SEVERE_ERROR);
    *err=SEVERE_ERROR;
    return;
  }

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(creportcalcsym)(void)
{
  if(!initialized){
    (void)printf("error(creportcalcsym): csyminit must be called before this "
	   "function!\n");
    (void)fprintf(stderr,"error(creportcalcsym): csyminit must be called before "
	    "this function!\n");
    exit(2);
  }
  CBEnd(&cb);
  (void)CBStart(&cb);
  cbmsg=NULL;
  reportcalcsym(&cb,&sym); 
  cbmsg=CBGetBuffer(&cb);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(csymexit)(void)
{
  if(initialized){
    CBEnd(&cb);
    initialized=0;
    cbmsg=NULL;
    if(taus!=NULL) {
      free(taus);
      taus=NULL;
    }
    if(isrt!=NULL) {
      free(isrt);
      isrt=NULL;
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(csyminit)(void)
{
  if(!initialized){
    CBInit(&cb,1000,1000);
    initialized=1;
    cbmsg=NULL;
    taus=NULL;
    isrt=NULL;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetreportbuffer)(char *msg,FPLOINTEGER *index,INTERNAL_INTEGER msg_len)
{
  if(cbmsg[*index]=='\n'){
    *msg=(char)0x0d;
  }else{
    *msg=cbmsg[*index] ;
  }
}


/* ***********************************************************************
 * calculate unit cell 
 *
 * ***********************************************************************
 */
void CFUNC(ccalcunitcell)
( const FPLOINTEGER         *i_nsort /* number of WPS */
 ,const FPLODOUBLEPRECISION *i_wtau /* WPS */
 ,      FPLOINTEGER         *o_nsite /* number of sites in unit cell */
 ,char                  *str_type
 ,      FPLODOUBLEPRECISION *i_tol  /* tolerance for backfolding 
			     in cell */
 ,      FPLOINTEGER         *err
 ,INTERNAL_INTEGER      strtype_len 
 )
{
  const char *nsubfunc="ccalcunitcell";
  long nsort,i,is,is_crystal;
  double *wtau;
  char *p=NULL;
  
  if(!initialized){
    (void)printf("error(ccalcunitcell): csyminit must be called before this "
	   "function!\n");
    (void)fprintf(stderr,"error(ccalcunitcell): csyminit must be called before "
	    "this function!\n");
    exit(2);
  }

  seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER);
  
  if((p=f2cstrcpy(str_type,strtype_len))==NULL) {
    FREEE(p,"");p=NULL;
    errmsg(nsubfunc,"Cannot allocate string!",SEVERE_ERROR);
    *err=SEVERE_ERROR;
    return;
  }
  
  is_crystal=((strstri(p,"crystal",0L))==p);

  /*printf("%s %d\n",p,is_crystal);
    exit(0);*/
  
  /* generate sites from WPS */
  nsort=*i_nsort;

  wtau=(double*)malloc(sizeof(double)*3* (nsort)); 
  for(is=0;is<nsort;is++){
    for(i=0;i<3;i++){
      wtau[i+3*is]=(double)(i_wtau[i+3*is]);
    }
  }
  taus=(double*)realloc(taus,sizeof(double)*3* (nsort)*48); 
  isrt=(long*)realloc(isrt,sizeof(long)*3*(nsort)*48); 

  if(is_crystal){
    generatetau(&sym,&nsite,nsort,wtau,taus,isrt,(double)(*i_tol));
  }else{
    generatetau_mol(&sym,&nsite,nsort,wtau,taus,isrt,(double)(*i_tol));
  }
  *err=severitylevel();
  
  (*o_nsite)=(FPLOINTEGER)nsite;



  free(wtau);
  FREEE(p,"");p=NULL;

  return ;
}


/* ***********************************************************************
 * calculate unit cell 
 *
 * ***********************************************************************
 */
void CFUNC(cgetsites)
( const FPLOINTEGER         *i_nsite /* number of sites from previous call to 
				    ccalcunitcell
				 */
  , FPLODOUBLEPRECISION *o_tau   /* sites */
  , FPLOINTEGER         *o_isrt  /* mapping of sites onto sorts */
  )
{
  long i,is;
  
  
  if(taus==NULL || isrt == NULL ){
    (void)printf("error(cgetsites): internal buffer not valid !\n");
    (void)fprintf(stderr,"error(cgetsites): internal buffer not valid !\n");
    exit(2);
  }
  
  if((*i_nsite) != nsite ){
    (void)printf("error(cgetsites): dummy i_nsite ("FPLOINTEGERFORMAT
	   ") != internal nsite (%ld)!\n"
	   " Did you call ccalcunitcell first?\n",*i_nsite,nsite);
    (void)fprintf(stderr,"error(cgetsites): dummy i_nsite ("FPLOINTEGERFORMAT
	   ") != internal nsite (%ld)!\n"
	   " Did you call ccalcunitcell first?\n",*i_nsite,nsite);

    
    
    exit(2);
  }
  
  
  for(is=0;is<nsite;is++){
    for(i=0;i<3;i++){
      o_tau[i+3*is]=(FPLODOUBLEPRECISION)(taus[i+3*is]);
    }
    o_isrt[is]=(FPLOINTEGER)(isrt[is]+1);
  }
  
  return ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cgetspacegroupname)
(char *result,FPLOINTEGER *spgrnum,FPLOINTEGER *err,INTERNAL_INTEGER result_len)
{
  const char *sginfo;
  long n=*spgrnum;
  seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER);  
  sginfo=getSpacegroupDescriptor("",&n);
  if(!(*err=severitylevel())){
    (void)c2fstrncpy(result,spcgrp_name(sginfo),(long)result_len,1L);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cgetcrystallsystemname)
(char *result,FPLOINTEGER *num,FPLOINTEGER *err,INTERNAL_INTEGER result_len)
{
  *err=0;
  seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER);  
  if(*num<SYM_CUBIC || *num> SYM_TRICLINIC){
    errmsg("cgetcrystallsystemname"
	   ,"Number of crystall system out of bound!\n",ERROR);
    *err=ERROR;
  }
  
  if(!c2fstrncpy(result,crystalsystems_longname(*num),(long)result_len,0L)){
    errmsg("cgetcrystallsystemname",errorbuffer,SEVERE_ERROR);
    *err=SEVERE_ERROR;
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cgetspacegroupcrystallsystem)
(FPLOINTEGER *result,FPLOINTEGER *spgrnum,FPLOINTEGER *err)
{
  const char *sginfo;
  long n=*spgrnum,ng;
  *result=0;
  seterrorhandling(ERR_NOCATCH | ERR_BUFFERED |ERR_RESETBUFFER);
  sginfo=getSpacegroupDescriptor("",&n);
  
  if(!(*err=severitylevel())){
    
    n=sscanf(spcgrp_crystallsystem(sginfo),"%ld",&(ng))-1;
    *result=ng-1;    
    if(n){
      (void)printf("error(cgetspacegroupcrystallsystem): error in "
	     "spacegroup_table!\n");
      (void)fprintf(stderr,"error(cgetspacegroupcrystallsystem): error in "
	      "spacegroup_table!\n");
      exit(1);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cconvertvecrtgtostg)(FPLODOUBLEPRECISION *result,FPLODOUBLEPRECISION *v)
{
  long i;
  double v1[3],v2[3];
  for(i=0;i<3;i++) v1[i]=(double)(v[i]);
  ConvertVecRTGtoSTG(v2,v1);
  for(i=0;i<3;i++) result[i]=(FPLODOUBLEPRECISION)(v2[i]);
}
