#ifndef _FORTRANTYPES_H
#define _FORTRANTYPES_H 1

#ifndef _DARWIN_COMPAT
#include <values.h>
#endif
#include <limits.h>

/* dont touch this */
#ifndef FLT_MAX 
#define FLT_MAX 1.e+20
#endif 
#ifndef DBL_MAX 
#define DBL_MAX 1.e+20
#endif 
/* end dont touch this */




#ifdef AUTO_DATA_TYPES
/* START OF SECTION DATA TYPE AUTO CHOICE */


/* ***********************************************************************
 * nothing to be done here, the settings come from prerocessor arguments
 * which where set by the config process!
 * ***********************************************************************
 */



/* END OF SECTION DATA TYPE AUTO CHOICE */
#else
/* START OF SECTION HAND MADE DECISION */


/* ***********************************************************************
 * This section is where data types may be set by hand.
 * Possibly, this might be nesseccary if error messages say so.
 * Just try to uncomment the proper definition (only one for each type may be 
 * active) or, if this does not work, make more difficult changes by hand
 * in the section DATA DEFINITION
 * ***********************************************************************
 */




/* uncomment the correct INTEGER type */
/* #define INTEGER_IS_longlong   */
#define INTEGER_IS_long  
/* #define INTEGER_IS_int */
/* #define INTEGER_IS_short */
/* #define INTEGER_IS_signedchar  */

/* uncomment the correct LOGICAL type */
/* #define LOGICAL_IS_longlong   */
/* #define LOGICAL_IS_long   */
#define LOGICAL_IS_int 
/* #define LOGICAL_IS_short */
/* #define LOGICAL_IS_signedchar  */


/* uncomment the correct DOUBLEPRECISION type */
#define DOUBLEPRECISION_IS_double
/* #define DOUBLEPRECISION_IS_float */



/* this is used as data type for internal character
   lenght dummy arguments hidden to fortran, but
   visible to C
*/

/* uncomment the correct INTEGER type */
/* #define INTERNAL_INTEGER_IS_longlong   */
/* #define INTERNAL_INTEGER_IS_long   */
#define INTERNAL_INTEGER_IS_int 
/* #define INTERNAL_INTEGER_IS_short */
/* #define INTERNAL_INTEGER_IS_signedchar  */


#endif
/* END OF SECTION HAND MADE DECISION */






/* START OF SECTION DATA DEFINITIONS */

/* ***********************************************************************
 * here come the data type definitions, 
 * its best practice to change the macros above and to leave the ones
 * below unchanged if ever possible
 * ***********************************************************************
 */





/* ***********************************************************************
 * first the data type representing the F90 Integer(t_int)
 * ***********************************************************************
 */
/*  this is normally integer*8 */
#ifdef INTEGER_IS_longlong
  #define FPLOINTEGER longlong   /* not yet clear */
  #define FPLOINTEGERFORMAT "%ld"   /* not yet clear */
  #define I_POS_INFINITY LONGLONG_MAX
  #define I_NEG_INFINITY LONGLONG_MIN
#endif

/*  this is normally standard integer*4 */
#ifdef INTEGER_IS_long
  #define FPLOINTEGER long 
  #define FPLOINTEGERFORMAT "%ld" 
  #define I_POS_INFINITY LONG_MAX
  #define I_NEG_INFINITY LONG_MIN
#endif


/* this also can be integer*4 */
#ifdef INTEGER_IS_int
  #define FPLOINTEGER int
  #define FPLOINTEGERFORMAT "%d" 
  #define I_POS_INFINITY INT_MAX
  #define I_NEG_INFINITY INT_MIN
#endif

/* this is normally  integer*2*/
#ifdef INTEGER_IS_short
  #define FPLOINTEGER short
  #define FPLOINTEGERFORMAT "%hd"
  #define I_POS_INFINITY SHRT_MAX
  #define I_NEG_INFINITY SHRT_MIN
#endif


/* This is normally  integer*1, this will most probably crash.
   In fact it is stupid to use 1 byte integers as fortran standard.
   It is just added here for completenes.
*/
#ifdef INTEGER_IS_signedchar
  #define FPLOINTEGER signed char 
  #define FPLOINTEGERFORMAT "%d"
  #define I_POS_INFINITY SCHAR_MAX
  #define I_NEG_INFINITY SCHAR_MIN
#endif



/* ***********************************************************************
 * second the data type representing the F90 Logical(t_log)
 * ***********************************************************************
 */

/*  this is normally logical*8 */
#ifdef LOGICAL_IS_longlong
  #define FPLOLOGICAL longlong  /* not yet clear */
  #define FPLOLOGICALFORMAT "%ld"   /* not yet clear */
#endif

/*  this is normally standard logical*4 */
#ifdef LOGICAL_IS_long
  #define FPLOLOGICAL long 
  #define FPLOLOGICALFORMAT "%ld" 
#endif


/* this also can be logical*4 */
#ifdef LOGICAL_IS_int
  #define FPLOLOGICAL int
  #define FPLOLOGICALFORMAT "%d" 
#endif

/* this is normally  logical*2*/
#ifdef LOGICAL_IS_short
  #define FPLOLOGICAL short
  #define FPLOLOGICALFORMAT "%hd"
#endif


/* this is normally  logical*1
*/
#ifdef LOGICAL_IS_signedchar
  #define FPLOLOGICAL signed char 
  #define FPLOLOGICALFORMAT "%d"
#endif

/* ***********************************************************************
 * third the data type representing the F90 Real(t_real)
 * ***********************************************************************
 */


/* this is normally standard real*8 */
#ifdef DOUBLEPRECISION_IS_double
  #define FPLODOUBLEPRECISION double  
  #define R_POS_INFINITY  DBL_MAX
  #define R_NEG_INFINITY -DBL_MAX
#endif

/* this is normally standard real*4 */
#ifdef DOUBLEPRECISION_IS_float 
  #define FPLODOUBLEPRECISION float  
  #define R_POS_INFINITY  FLT_MAX
  #define R_NEG_INFINITY -FLT_MAX
#endif





/* ***********************************************************************
 * Last the data type used by F90 internally to transfer hidden 
 * character data length dummy arguments to external subroutines.
 * This coresponds to the standard F90 calling convention.
 * On some machines the character data are transfered in a completely
 * different way and the settings below are no longer significant.
 * More over, the code, in its current layout, will not run on such machines.
 * We will update the interface in the future, so the code runs on such
 * architectures.
 * ***********************************************************************
 */


/*  this is normally integer*8 */
#ifdef INTERNAL_INTEGER_IS_longlong
  #define INTERNAL_INTEGER longlong      /* not yet clear */
  #define INTERNAL_INTEGERFORMAT "%ld"   /* not yet clear */
#endif

/*  this is normally standard integer*4 */
#ifdef INTERNAL_INTEGER_IS_long
  #define INTERNAL_INTEGER long 
  #define INTERNAL_INTEGERFORMAT "%ld" 
#endif


/* this also can be integer*4 */
#ifdef INTERNAL_INTEGER_IS_int
  #define INTERNAL_INTEGER int
  #define INTERNAL_INTEGERFORMAT "%d" 
#endif

/* this is normally  integer*2*/
#ifdef INTERNAL_INTEGER_IS_short
  #define INTERNAL_INTEGER short
  #define INTERNAL_INTEGERFORMAT "%hd"
#endif


/* this is normally  integer*1
*/
#ifdef INTERNAL_INTEGER_IS_signedchar
  #define INTERNAL_INTEGER signed char 
  #define INTERNAL_INTEGERFORMAT "%d"
#endif

/* END OF SECTION DATA DEFINITIONS */




/* ***********************************************************************
 * these values are set by sizetest.c, called initially by FORTRAN 
 * ***********************************************************************
 */
extern FPLOLOGICAL L_TRUE;
extern FPLOLOGICAL L_FALSE;




#endif /* _FORTRANTYPES_H */
/*EOF*/
