/*************************************************************************
 * @(#)idparser.c
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   07 Nov 2003
 *************************************************************************
 */


#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "idparser.h"
#include "table.h"
#include "merror.h"
#include "helper.h"


/* ***********************************************************************
 * typedefs for variable name parser callbacks
 * ***********************************************************************
 */


#define VNP_INIT               1
#define VNP_EXIT               2
#define VNP_NAME               3
#define VNP_INDEX              4
#define VNP_SCALAR             5
#define VNP_STRUCT             6
#define VNP_ARRAY              7
#define VNP_STRUCTARRAY        8


/* ***********************************************************************
 * the variable name  lexer token data structure
 * ***********************************************************************
 */

typedef union tutoken{
  char id[10000+1];
  long num;
}tuTOKEN;

#define TOKEN_ERROR 0
#define TOKEN_EOL   1
#define TOKEN_CHAR  2
#define TOKEN_NUM   3
#define TOKEN_ID    4




/* ***********************************************************************
 * a callback function for the vnparser, which is used for VARTABLE creation
 * ***********************************************************************
 */



VARTABLE *vt_creation_cb(long vnptype,const void *id,void *data)
{
  const char *nsubfunc="vt_creation_cb";
  struct vartable *node;
  tsTABLESCAN *ts=(tsTABLESCAN*)data;
  long i,in;
  long si,string,structurtype,sti;
  tsSTRUCTARRAY *sap;
  tsARRAY7 *arp;

  if(vnptype==VNP_INIT){
    /* ************************************************************ */
    ts->currentnode=ts->list;
    ts->node=NULL;
    ts->undef=0;
  }else if(vnptype==VNP_EXIT){
    /* ************************************************************ */
    return ts->node;
  }else if(vnptype==VNP_NAME){
    /* ************************************************************ */
    strcpy(ts->name,(const char*)id);
    ts->rank=0;
  }else if(vnptype==VNP_INDEX){
    /* ************************************************************ */
    for(i=1;i<8;i++){
      in=((const long*)id)[i];
      if(in>=0 || in==-2 ){
	ts->rank=i;
	ts->index[i]=in;
      }
    }
  }else if(vnptype==VNP_SCALAR){ 
    /* ************************************************************ */
    structurtype=ts->structuretype;
    if( ! structurtype){
      structurtype=VTYPE_SCALAR;
    }
    if(!structuretypeindex(&sti,structurtype)) return NULL;
    
    if((ts->node
	=lookup_list(ts->currentnode,ts->name,NULL,structurtype,ts->scalartype))==NULL){
      
      if(ts->scalartype && (structurtype==VTYPE_SCALAR||structurtype==VTYPE_ARRAY)){ 
	if(!scalartypeindex(&si,ts->scalartype,&string,1L)) return NULL; 
	sprintf(errorbuffer,"%s: %s \"%s\" of type %s not declared"
		,nsubfunc,structuretypes[sti],ts->name,scalartypes[si]);
      }else{
	sprintf(errorbuffer,"%s: %s \"%s\" not declared"
		,nsubfunc,structuretypes[sti],ts->name);
      }
      ts->undef=1;
      return NULL;
    }	/* end lookup */
    ts->undef=0;
    
  }else if(vnptype==VNP_STRUCT){ 
    /* ************************************************************ */
    if((node=lookup_list(ts->currentnode,ts->name,NULL,VTYPE_STRUCT,0))==NULL){
      sprintf(errorbuffer,"%s: cannot find struct \"%s\"",nsubfunc,ts->name);
      ts->undef=1;
      return NULL;
    }
    ts->undef=0;
    ts->currentnode=node->data.list;
  }else if(vnptype==VNP_ARRAY){ 
    /* ************************************************************ */
    structurtype=ts->structuretype;
    if( ! structurtype){
      structurtype=VTYPE_ARRAY;
    }
    if(!structuretypeindex(&sti,structurtype)) return NULL;

    if((ts->node
	=lookup_list(ts->currentnode,ts->name,NULL,structurtype,ts->scalartype))==NULL){
      
      if(ts->scalartype && (structurtype==VTYPE_ARRAY)){ 
	if(!scalartypeindex(&si,ts->scalartype,&string,1L)) return NULL; 
	sprintf(errorbuffer,"%s: %s \"%s\" of type %s not declared"
		,nsubfunc,structuretypes[sti],ts->name,scalartypes[si]);
      }else{
	sprintf(errorbuffer,"%s: %s \"%s\" not declared"
		,nsubfunc,structuretypes[sti],ts->name);
      }
      ts->undef=1;
      return NULL;
    }	/* end lookup */
    ts->undef=0;
    if(ts->node->structuretype==VTYPE_ARRAY){
      arp=ARP(ts->node);
      if(arp->rank!=ts->rank){
	sprintf(errorbuffer,"%s: array \"%s\" has rank %ld not %ld"
		,nsubfunc,ts->name,arp->rank,ts->rank);
	return NULL;
      }
      if(arp->variable){
	for(i=1;i<arp->rank;i++){
	  if(arp->Size[i]!=ts->index[i]){
	    sprintf(errorbuffer,"%s: array \"%s\", dimension %ld: size is %ld not %ld!"
		    ,nsubfunc,ts->name,i,arp->Size[i],ts->index[i]);
	    return NULL;
	    
	  }
	} 
	if(ts->index[arp->rank]>=0){
	  sprintf(errorbuffer,"%s: array \"%s\", dimension %ld: has variable size!"
		  ,nsubfunc,ts->name,arp->rank);
	  return NULL;
	}

      }else{ /* not variable */

	if(ts->index[arp->rank]<0){
	  sprintf(errorbuffer,"%s: array \"%s\", dimension %ld: has fixed size!"
		  ,nsubfunc,ts->name,arp->rank);
	  return NULL;
	}
	for(i=1;i<=arp->rank;i++){
	  if(arp->Size[i]!=ts->index[i]){
	    sprintf(errorbuffer,"%s: array \"%s\", dimension %ld: size is %ld not %ld!"
		    ,nsubfunc,ts->name,i,arp->Size[i],ts->index[i]);
	    return NULL;
	    
	  }
	} 
      }

    }else{
      sprintf(errorbuffer,"%s: structarray element logic not yet implementd! Sorry.",nsubfunc);
      return NULL;
    }

  }else if(vnptype==VNP_STRUCTARRAY){ 
    /* ************************************************************ */

    sprintf(errorbuffer,"%s: structarray member logic not yet implementd! Sorry.",nsubfunc);
    return NULL;

    if(ts->rank!=1){
      sprintf(errorbuffer,"%s: struct array \"%s\" has rank 1!",nsubfunc,ts->name);
      return NULL;
    }
    if(ts->index[1]>=0){
      /* printf("structarray  %s[%ld]\n",ts->name,ts->index[1]);  */
    }else{
      sprintf(errorbuffer,"%s: full structarray logic not yet implementd! Sorry.",nsubfunc);
      return NULL;
    }

    if((node=lookup_list(ts->currentnode,ts->name,NULL,VTYPE_STRUCTARRAY,0))
       ==NULL){
      sprintf(errorbuffer,"%s: cannot find structarray \"%s\"",nsubfunc,ts->name);
      ts->undef=1;
      return NULL;
    }
    ts->undef=0;
    sap=&node->data.structarray;

    if(ts->index[1]>=sap->Size){
      if(sap->variable){

	if((node=insert_structelement_into_structarray(node
						       ,ts->index[1]+1
						       ))==NULL)
	  return NULL;
	ts->currentnode=node;
      }else{
	sprintf(errorbuffer,"%s: structarray \"%s\" has no element %ld"
		,nsubfunc,ts->name,ts->index[1]);
	return NULL;
      }
    }else{
      if(sap->array==NULL){
	if(!allocate_structarray(node )) return NULL;   
      }
      ts->currentnode=sap->array[ts->index[1]];
    }


  }else{
    /* ************************************************************ */
    sprintf(errorbuffer,"%s: unknown VNP type %ld\n",nsubfunc,vnptype);
    return NULL;
  }
  return (VARTABLE*)1;
}

/* ***********************************************************************
 * a lexer for variable names
 * ***********************************************************************
 */


long lexer(const char *(*stream),tuTOKEN *token,long *colnr)
{
  int i;
  long l,ret=TOKEN_ERROR;
  char *err;
  const char *p;
  p=*stream;
  while( (*p==' ' || *p=='\t') && *p!='\0'){p++;(*colnr)++;};
  
  if( *p=='['){
    p++;(*colnr)++;
    token->id[0]='[';
    ret=TOKEN_CHAR;
  }else if(*p==']'){
    p++;(*colnr)++;
    token->id[0]=']';
    ret=TOKEN_CHAR;
  }else if(*p=='.'){
    p++;(*colnr)++;
    token->id[0]='.';
    ret=TOKEN_CHAR;
  }else if(*p==','){
    p++;(*colnr)++;
    token->id[0]=',';
    ret=TOKEN_CHAR;
  }else if(*p=='*'){
    p++;(*colnr)++;
    token->id[0]='*';
    ret=TOKEN_CHAR;
  }else if(isdigit(*p)){
    i=0;
    while( isdigit(*p)){
      token->id[i++]=(*p);
      p++;(*colnr)++;
    }
    token->id[i]='\0';
    l= strtol(token->id,&err,10);
    if (*err!='\0' ) {
      ret=TOKEN_ERROR;
      return ret;
    }
    token->num=l;
    ret=TOKEN_NUM;
  }else if(islower(*p) || isupper(*p)){
    i=0;
    token->id[i++]=(*p);p++;(*colnr)++;
    while( islower(*p) || isupper(*p) || isdigit(*p) || (*p)=='_'){
      token->id[i++]=(*p);
      p++;(*colnr)++;
    }
    token->id[i]='\0';
    ret=TOKEN_ID;
  }else if ((*p)=='\0'){
    ret=TOKEN_EOL;
  }

  *stream=p;
  return ret;
}

/* ***********************************************************************
 * a parser for scanning variable names, which uses callback
 * for performing actions depending in the parse state
 * ***********************************************************************
 */

VARTABLE *parse_varname(const char *name,VNPARSER_CALLBACK callback
			,tsTABLESCAN *data)
{
  const char *nsubfunc="parse_varname";
  const char (*p);
  long ret,colnr,variable=0;
  tuTOKEN token;
  long index[8];
  long rank=0,i;
  
  p=name;
  colnr=1;

  if(callback==NULL) {
    sprintf(errorbuffer,"%s: callback function is NULL!",nsubfunc);
    goto error;
  }
  if(callback(VNP_INIT,NULL,data)==NULL) goto error;

  
 expectid:  
  if((ret=lexer(&p,&token,&colnr))==TOKEN_ERROR) goto lexerror;
  if(ret!=TOKEN_ID){
    sprintf(errorbuffer,"%s: expect an identifier near character %ld"
	    " in variable name \"%s\"",nsubfunc,colnr-1,name);
    goto error;
    
  }
  rank=0;
  if(callback(VNP_NAME,token.id,data)==NULL) goto error;
  goto expectelormem;

 expectelormem:  
  if((ret=lexer(&p,&token,&colnr))==TOKEN_ERROR) goto lexerror;
  if(ret==TOKEN_EOL) {
    if(callback(VNP_SCALAR,NULL,data)==NULL) goto error;
    goto ende;
  }
  

  if(ret==TOKEN_CHAR && token.id[0]=='['){
    rank=1;
    for(i=0;i<8;i++)index[i]=-1;
    goto expectnum;
  }else if(ret==TOKEN_CHAR && token.id[0]=='.'){
    if(callback(VNP_STRUCT,NULL,data)==NULL) goto error;
    goto expectid;
  }else {
    sprintf(errorbuffer,"%s: expect '[' or '.' near character %ld"
	    " in variable name \"%s\"",nsubfunc,colnr-1,name);
    goto error;
  }


 expectmem:  
  if((ret=lexer(&p,&token,&colnr))==TOKEN_ERROR) goto lexerror;
  if(ret==TOKEN_EOL) {
    if(callback(VNP_ARRAY,NULL,data)==NULL) goto error;
    goto ende;
  }



  

  if(ret==TOKEN_CHAR && token.id[0]=='.'){
    if(callback(VNP_STRUCTARRAY,NULL,data)==NULL) goto error;
    goto expectid;
  }else {
    sprintf(errorbuffer,"%s: expect '.' near character %ld"
	    " in variable name \"%s\"",nsubfunc,colnr-1,name);
    goto error;
  }

 expectnum:;
  if((ret=lexer(&p,&token,&colnr))==TOKEN_ERROR) goto lexerror;

  if(ret==TOKEN_NUM){
    index[rank]=token.num;
  }else if(ret==TOKEN_CHAR && token.id[0]=='*'){
    variable=1;
    index[rank]=-2;
  }else{
    sprintf(errorbuffer,"%s: expect a number or '*' near character %ld"
	    " in variable name \"%s\"",nsubfunc,colnr-1,name);
    goto error;
    
  }

  
  if((ret=lexer(&p,&token,&colnr))==TOKEN_ERROR) goto lexerror;

  if(ret==TOKEN_CHAR && token.id[0]==']'){
    if(callback(VNP_INDEX,&index,data)==NULL) goto error;
    goto expectmem;
  }else if(ret==TOKEN_CHAR && token.id[0]==','){
    if(variable){
      sprintf(errorbuffer,"%s:  near character %ld"
	      " in variable name \"%s\":"
	      " only last dimension may be variable",nsubfunc,colnr-1,name);
      goto error;
    }
    rank++;
    goto expectnum;
  }else{
    sprintf(errorbuffer,"%s: expect ']' or ',' near character %ld"
	    " in variable name \"%s\"",nsubfunc,colnr-1,name);
    goto error;
  }
  
  

 ende:
  return callback(VNP_EXIT,NULL,data);
 error:
  return NULL;
 lexerror:
  sprintf(errorbuffer,"%s: lexical error near character %ld "
	  "in variable name \"%s\"",nsubfunc,colnr-1,name);
  return NULL;

}
