/* @(#)table.h
 */

#ifndef _TABLE_H
#define _TABLE_H 1

#include <stdio.h>



/* ***********************************************************************
 * convinience macros please use only with pointer!!!
 * ***********************************************************************
 */
#define SAP(node) (&(node->data.structarray))
#define SCP(node) (&(node->data.scalar))
#define STP(node) (node->data.list)
#define ARP(node) (&node->data.array7)

#define WRITE_COPY  1
#define WRITE_REF   2

/* ***********************************************************************
 *
 * ***********************************************************************
 */

#define VTYPE_NONE           0x00000000

/* data structure types */
#define VTYPE_SCALAR         0x00000001
#define VTYPE_ARRAY          0x00000002
#define VTYPE_STRUCT         0x00000004
#define VTYPE_STRUCTARRAY    0x00000008
#define VTYPE_SECTION        0x00000010
#define VTYPE_SECTIONEND     0x00000020
#define VTYPE_PLACEMENT      0x00000040
#define VTYPE_COMMENT        0x00000080
#define VTYPE_ERROR          0x00000100
#define VTYPE_ROOT           0x00000200


/* scalar types */
/* this constants are also defined in ioadapter.f90 */
/* you cannot change them independently, otherwise  it crashes */
#define VTYPE_INT     0x00000001 
#define VTYPE_REAL    0x00000002
#define VTYPE_LOGICAL 0x00000004
#define VTYPE_FLAG    0x00000008
#define VTYPE_CHAR    0x00000010
#define VTYPE_STRING  0x00000020

#define VTYPE_POS_INF 0x00000040
#define VTYPE_NEG_INF 0x00000080



/* placement flags */
#define VTYPE_INPLACE  0x00000001
#define VTYPE_SEPERATE 0x00000002
#define VTYPE_DATA     0x00000004
/* TODO: new placement for in struct definintions, so one can  */
/* print seperate parts of table */
/* copy placenment */

/* ***********************************************************************
 * typedefs
 * ***********************************************************************
 */

struct vartable;		/* forward of node type */


/* ******************************
 * array specific data
 * ******************************
 */

typedef struct tsarray7{
  long scalartype;
  long strlength;
  long strvariable;
  long Size[8];
  long currentsize;
  long variable;
  long rank;
  long lowdim;
  long external;
  void *array;
  char *idref[8];
  long allocated;
  void *defaultvalue;		/* pointer to the (array) of default values */
} tsARRAY7;

/* ******************************
 * scalar specific data
 * ******************************
 */
typedef struct tsscalar{
  long scalartype;
  long strlength;
  long strvariable; 
  void *defaultvalue;
  char *value;
  long isdim;
}tsSCALAR;


/* ******************************
 * structarray specific data
 * ******************************
 */

typedef struct tsstructarray{
  long  Size;
  long  currentsize;
  long  variable;
  long allocated;
  long typetemplateiscopy; 
  struct vartable *typetemplate; 
  struct vartable **array;
  char *idref;
}tsSTRUCTARRAY;

/* ******************************
 * a union to mimic all node data types
 * ******************************
 */

typedef union tudata{
  tsSCALAR         scalar;
  tsARRAY7         array7;
  tsSTRUCTARRAY    structarray;
  struct vartable *noderef;	/* for placements */
  struct vartable *list;	/* for placements */
}tuDATA;

/* ******************************
 * the node data type
 * ******************************
 */

typedef struct vartable{
  long structuretype; /* scalar,  array, structarray, flag */
  tuDATA data;			/* actual data held by node */
  struct vartable *prev,*next;
  struct vartable *parent;
  long aschar;  
  char *name;
  long dataplacement;
} VARTABLE;


extern const char * scalartypes[];
extern const char * structuretypes[];


#ifdef	__cplusplus
extern "C" {
#endif

  extern long init_vartable(VARTABLE *table,long aschar);
  extern VARTABLE* append_placement_to_list(VARTABLE *list,VARTABLE *noderef);
  extern VARTABLE* append_scalar_to_list(VARTABLE *list,const char *name
					 ,long scalartype
					 ,long strlength
					 ,long strvariable
					 ,void *defaultvalue
					 );
  extern VARTABLE* append_struct_to_list(VARTABLE *list,char *name
					 ,VARTABLE *typetemplate
					 ,long allocarray
					 );
  extern VARTABLE* append_structarray_to_list(VARTABLE *list,char *name
					      ,long size
					      ,long variable
					      ,char *idref
					      ,VARTABLE *structtable
					      ,long allocarray);
  
  extern long clean_vartable(VARTABLE *table);

  extern VARTABLE *lookup_list(const VARTABLE * list
			       ,const char *name
			       ,const VARTABLE *noderef
			       ,const long structuretype
			       ,const long scalartype);

  extern VARTABLE* append_section_to_list(VARTABLE *list,const char *name);
  extern VARTABLE* append_section_end_to_list(VARTABLE *list,const char *name);
  extern VARTABLE *append_array_to_list(VARTABLE *list,const char *name
					,long scalartype
					,long rank
					,long size[8]
					,char *idref[8]
					,long variable
					,long strlength,long strvariable
					,void *defaultvalue
					,void *externalarray
					,long allocarray
					,long reallocarray
					);


  extern VARTABLE* append_comment_to_list(VARTABLE *list,char *comment
					  ,long copy);
  extern long print_vartab(FILE *fh,const VARTABLE *table);
  extern long scalartypeindex(long *sctyind,long scalartype,long *string,long acceptstring);
  extern long structuretypeindex(long *sctyind,long structuretype);

  extern long write_into_scalar(VARTABLE *node,const void *value,long copy);
  extern long set_defaultvalue_scalar(VARTABLE *node,void *defaultvalue);
  extern long write_into_array(VARTABLE *node,long indx[8],const void *value
			       ,long copy);
  extern long set_defaultvalue_array(VARTABLE *node,void *defaultvalue);
  extern long resize_vtarray(VARTABLE *node,long sizes[8]);

  extern VARTABLE *insert_structelement_into_structarray(VARTABLE *node
							 ,long indx );
  extern long resize_structarray(VARTABLE *node,long size);
  extern long allocate_array(VARTABLE *node,long reallocarray );
  extern long allocate_structarray(VARTABLE *node );
  extern long create_template_from_defaultvalue(VARTABLE *typetemplate
						,const char *name);
  extern long vartable_clean_scalar(VARTABLE *node);
  extern long vartable_clean_array(VARTABLE *node);
  extern long vartable_clean_structarray(VARTABLE *node);


#ifdef	__cplusplus
}
#endif

#endif /* _TABLE_H */

