#include <string.h>
#include <ctype.h>
#include "tools.h"  
#include "cio.h"
#include "allocate.h"  
#include "ioparser.h"
#include "tparser.h"
#include "convtests.h"  
#include "merror.h"  

#define EXPECT_NAME     1
#define EXPECT_BRACKETS 2
#define EXPECT_MEMBER   3

#define LMAX(x,y) ((x)>=(y)?(x):(y))


static long getArrayNaturalValue(VARTABLE *var,void  *result
				 ,const long result_len,const FPLOINTEGER rank
				 ,const FPLOINTEGER size[7]
				 ,const long asstring
				 );

static long getArrayNaturalSection(const char *name,const void *array
				   ,void *result,const long result_len
				   ,const long asize,const long rsize
				   ,const long aoffset,const long roffset
				   ,const long scalartype
				   ,const long aschar
				   ,const long asstring
				   );


static long getScalarNaturalValue(struct vartable *var,void *result
				    ,const long roffset
				    ,const long result_len
				    ,const long asstring
				    );

static long nameScanRemove(struct vartable *table,const char *varname
			   ,struct tsnamescan *namescan
			   ,long killdefinition
			   ,int  depth);

/* ***********************************************************************
 * In table look for variable of varname=name1.name2.name3 ....
 * to arbitrary nesting depth. This is for the fortran interface,
 * which returns whole arrays instead of calling the overhead
 * for each value needed in fortran.
 * ***********************************************************************
 */
long getNameScan(VARTABLE *table,const char *varname
		    ,long structuretypefilter /* do we look for something */ 
		    ,long scalartypefilter    /* specific? */
		    
		    

		    ,struct tsnamescan *namescan  /* output: path through
						    table */
		    ,long depth
		    )
{
  char *subfunc="getNameScan";
  char *buffer,*p0,*p1;
  long si,string,sti;
  struct vartable *var;

  /* HIER("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"); */
  /* HIERL(depth) */
  /* HIER(varname); */
  if((buffer=allocate(sizeof(char)*(strlen(varname)+1),"",subfunc))==NULL){
    (void)sprintf(errorbuffer,"error(%s): cannot allocate memeory!",subfunc);
    return 0;
  }
  (void)strcpy(buffer,varname);

  namescan->next=NULL;
  namescan->var=NULL;

  p0=buffer;
  p1=p0;
  var=table;  
/*   HIER(p0); */
  if((p1=strstr(p0,"."))!=NULL){
    /* if we are here, there is a name1.name2 */
    *p1++='\0';
    /* HIER(p0); */
    if((var
	=lookup_list(var,p0,NULL,VTYPE_STRUCT|VTYPE_STRUCTARRAY,0L))==NULL){ 
      (void)sprintf(errorbuffer
	      ,"error(%s): Struct or structarray variable '%s' not declared!"
	      ,subfunc,buffer);
      goto errorend;
    } 
    /* if we are here, name1 is either a struct or a structarray */
    p0=p1;
    /* *(p1-1)='.'; */

    /* HIER(p1); */

    /* HIER(var->name) ; */
    namescan->var=var;
    if(var->structuretype == VTYPE_STRUCT){ 
      var=var->data.list;  
    }else if(var->structuretype == VTYPE_STRUCTARRAY){ 
      /* var is pattern  not the true array  */
      var=var->data.structarray.typetemplate;
    }else goto internalerror; 
    if((namescan->next=allocate(sizeof(struct tsnamescan),"",subfunc))==NULL){
      (void)sprintf(errorbuffer,"error(%s): cannot allocate memeory!",subfunc);
      return 0;
    }
    /* HIERP(namescan->next); */
    if(!getNameScan(var,p1,structuretypefilter,scalartypefilter
		       ,namescan->next,depth+1))
      return 0;
    
  }else{
    /* if we are here, we have a final name1  */
    /* HIER("simple"); */
    /* HIER(p0); */
    if(scalartypefilter){ 
      if(!scalartypeindex(&si,scalartypefilter,&string,1L)) 
	goto internalerror; 
    } 
    if(!structuretypeindex(&sti,structuretypefilter))goto internalerror;
    
    if((var=lookup_list(var,p0,NULL,structuretypefilter,scalartypefilter))
       ==NULL){ 
      if(scalartypefilter){
	(void)sprintf(errorbuffer
		      ,"error(%s): %s variable '%s' of requested type '%s'"
		      " not declared!"
		      ,subfunc,structuretypes[sti],varname,scalartypes[si]);
	goto errorend;
      }else{
	(void)sprintf(errorbuffer
		      ,"error(%s): %s variable '%s' not declared!"
		      ,subfunc,structuretypes[sti],varname);
      } 
      goto errorend;
    }
    namescan->var=var;

  }
  
  

  /* HIER("???????????????????"); */
  FREEE(buffer,"");buffer=NULL;
  return 1;

 errorend:
  FREEE(buffer,"");buffer=NULL;
  return 0;
 internalerror:
  (void)sprintf(errorbuffer
	  ,"error(%s): variable %s: internal error!"
	  ,subfunc,varname);
  goto errorend;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void cleanup_namescan(struct tsnamescan *namescan)
{
  if(!namescan) return;
  cleanup_namescan(namescan->next);
  if(namescan->next){
    FREEE(namescan->next,"namescan->next");
  }
  return; 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static int scanIndices(const char *varname,char *buf,long *rank,long indx[8])
{
  char *subfunc="scanIndices";
  long i;
  char *p0=buf;
  
  (*rank)=0;
  while(1){
    p0+=strspn(p0," \t");
    i=strcspn(p0,",");
    
    if((unsigned long)i==strlen(p0)){
      /* no more commas */
      (*rank)++;
      removetrailingspace(p0);
      if(strlen(p0)==0) goto missingindexerror;
      /*       printf("ind[%ld]=<%s>\n",(*rank),p0); */
      if(sscanf(p0,"%ld",indx+(*rank))!=1) goto conversionerror;
      
      break;
    }else{
      p0[i]='\0';
      removetrailingspace(p0);
      if(strlen(p0)==0) goto missingindexerror;
      (*rank)++;
      /*       printf("ind[%ld]=<%s>\n",(*rank),p0); */
      if(sscanf(p0,"%ld",indx+(*rank))!=1) goto conversionerror;
      p0+=i+1;
    }
  }

  return 1;
  
 missingindexerror:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', missing index !"
		,subfunc,varname);
  return 0;

 conversionerror:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', non-integer index !"
		,subfunc,varname);
  return 0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long traverse(long expect,VARTABLE **var,long indx[8]
		     ,const char *varname
		     ,const char *p,char *buf)
{
  char *subfunc="traverse";
  tsARRAY7 *arp;
  tsSTRUCTARRAY *sap;
  long i,rank;
/*   printf("%s: p<%s> var<%p,%s,%ld>\n",subfunc,p */
/* 	 ,(void*)(*var) */
/* 	 ,(*var)->name */
/* 	 ,(*var)->structuretype); */

  if(expect==EXPECT_NAME){
/*     HIER("EXPECT_NAME"); */
    p+=strspn(p," \t");
    i=strcspn(p," \t\n.[");
    strncpy(buf,p,i);buf[i]='\0';
    p+=i;
    p+=strspn(p," \t");
/*     printf("rest<%s>: name<%s> var<%p>\n",p,buf,(void*)(*var)); */

    
    if((*var
	=lookup_list(*var,buf,NULL,VTYPE_SCALAR|VTYPE_ARRAY|
		     VTYPE_STRUCT|VTYPE_STRUCTARRAY,0L))==NULL)
      goto varerror;
      
    if((*var)->structuretype == VTYPE_SCALAR){
      if(strlen(p)==0){
	return 1;
      }else{ goto varerror;}
    }else if(
	     (*var)->structuretype == VTYPE_ARRAY||
	     (*var)->structuretype == VTYPE_STRUCTARRAY ){
      return traverse(EXPECT_BRACKETS,var,indx,varname,p,buf);
    }else if((*var)->structuretype == VTYPE_STRUCT){
      (*var)=STP((*var)); 
      return traverse(EXPECT_MEMBER,var,indx,varname,p,buf);
    }
    
    
  }else if(expect==EXPECT_BRACKETS){
/*     HIER("EXPECT_BRACKETS"); */
    p+=strspn(p," \t");
    if(p[0]!='[')  goto missingopenbracket;
    p++;
    p+=strspn(p," \t");
    i=strcspn(p,"]");
    if((unsigned long)i==strlen(p)) goto missingclosingbracket;
    
    strncpy(buf,p,i);buf[i]='\0';
    p+=i;
    p+=strspn(p," \t");
    removetrailingspace(buf);
/*     printf("pp<%s>: bbuf<%s>\n",p,buf); */
    
    if(!scanIndices(varname,buf,&rank,indx)) return 0;
    p++;
/*     printf("ind[1]=%ld\n",indx[1]); */
    /*       printf("ind[2]=%ld\n",indx[2]);  */
    /*       printf("ind[3]=%ld\n",indx[3]);  */
    
    if((*var)->structuretype == VTYPE_ARRAY){
      arp=ARP((*var));
      if(arp->rank!=rank) goto rankerror;
      return 1;
    }else if((*var)->structuretype == VTYPE_STRUCTARRAY){
      sap=SAP((*var));
      if(rank!=1) goto rankerror;
      i=indx[1]-1;
      if(i<0 || i>sap->Size-1) goto indexerror;
      *var=sap->array[indx[1]-1];
      return traverse(EXPECT_MEMBER,var,indx,varname,p,buf);
    }
    
   
  }else if(expect==EXPECT_MEMBER){ 
/*     HIER("EXPECT_MEMBER"); */
    p+=strspn(p," \t");
    if(p[0]!='.')  goto missingmemberdot;
    p++;
/*     printf("pp<%s>: bbuf<%s> var<%p>\n",p,buf,(void*)(*var)); */
    
    return traverse(EXPECT_NAME,var,indx,varname,p,buf);

  }else{
    (void)sprintf(errorbuffer,"error(%s): variable '%s' not yet imple !"
		  ,subfunc,varname);
    return 0;

  }


 varerror:
  (void)sprintf(errorbuffer,"error(%s): variable '%s' not declared "
		"or incomplete or wrong syntax!"
		,subfunc,varname);
  return 0;
  
 missingopenbracket:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', missing opening '['!"
		,subfunc,varname);
  return 0;

 missingclosingbracket:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', missing closing ']'!"
		,subfunc,varname);
  return 0;


 rankerror:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', has different rank!"
		,subfunc,varname);
  return 0;

 missingmemberdot:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', missing '.'!"
		,subfunc,varname);
  return 0;

 indexerror:
  (void)sprintf(errorbuffer,"error(%s): variable '%s', index out of bounds!"
		,subfunc,varname);
  return 0;


  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long setValue(VARTABLE *table,const char *varname,const char *value)
{
  char *subfunc="setValue";
  char *buffer=NULL;
  char *name=NULL;
  char *p;
  long indx[8];
  VARTABLE *var=table;

  if(!(table->aschar)){
    (void)sprintf(errorbuffer 
	    ,"error(%s): Cannot set variable %s as string while parser\n"
	    " works in non character mode (i.e. external arrays used)!" 
	    ,subfunc,varname); 
    goto error;
  }

  if((buffer=allocate(sizeof(char)*(strlen(varname)+1),"",subfunc))==NULL)
    goto error;
  if((name=allocate(sizeof(char)*(strlen(varname)+1),"",subfunc))==NULL)
    goto error;
  
  strcpy(buffer,varname);
  p=buffer;


  

  if(!traverse(EXPECT_NAME,&var,indx,varname,p,name)) goto error;

  if(var->structuretype==VTYPE_SCALAR){
    if(!write_into_scalar(var,value,WRITE_COPY))goto error;
  }else if(var->structuretype==VTYPE_ARRAY){
    if(!write_into_array(var,indx,value,WRITE_COPY)) goto error;
  }

  
  if(buffer) {FREEE(buffer,"");buffer=NULL;}
  if(name) {FREEE(name,"");name=NULL;}
  return 1;
  
  
 error:
  if(buffer) {FREEE(buffer,"");buffer=NULL;}
  if(name) {FREEE(name,"");name=NULL;}
  return 0;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long removeVariable(VARTABLE *table,const char *varname)
{
  tsNAMSCAN namescan,*nx;
  long st;
  
  if(!getNameScan(table,varname
		  ,(long) (VTYPE_ARRAY | VTYPE_SCALAR 
			   | VTYPE_STRUCT | VTYPE_STRUCTARRAY)
		  ,0L,&namescan,0L)) 
    goto errorend;
  
  st=namescan.var->structuretype;
  nx=namescan.next;
  
  if(!nameScanRemove(table,varname,&namescan,0L,0)) goto errorend;
  
  if(st == VTYPE_STRUCTARRAY && nx){
    if(!nameScanRemove(table,varname,&namescan,1L,0))
      goto errorend;
  }
  
  
  cleanup_namescan(&namescan);
  return 1;
  
 errorend:
  cleanup_namescan(&namescan);
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long nameScanRemove(VARTABLE *table,const char *varname
		    ,struct tsnamescan *namescan
		    ,long killdefinition,int depth)
{
  char *subfunc="nameScanRemove";
  VARTABLE *v;
  VARTABLE *stp;
  tsSTRUCTARRAY *sap;
  tsARRAY7 *arp;
  long scalartype=0L;
  long i;
  long prot=0;

  if(!namescan) return 1;

  if(prot){
    if(depth==0) printf("\n");
    printf("%*s: varname=%s\n",2*depth+24,subfunc,varname);
    printf("%*s: structuretype=%ld\n",2*depth+24,subfunc
	   ,namescan->var->structuretype);
    printf("%*s: table=%p %s\n",2*depth+24,subfunc,(void*)table,table->name);
    printf("%*s: tabletype=%ld\n",2*depth+24,subfunc,table->structuretype);
    printf("%*s: next=%p\n",2*depth+24,subfunc,(void*)namescan->next);
    printf("%*s: name=%s\n",2*depth+24,subfunc,namescan->var->name);
  }
  
  if( namescan->var->structuretype == VTYPE_STRUCTARRAY){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCTARRAY,scalartype))==NULL) 
      goto internalerror;
    sap=SAP(v);
    if(prot){
      printf("%*s: sap=%p name=%s \n",2*depth+25,subfunc,(void*)sap,v->name);
    }
    if(namescan->next){
      /*  remove in all members followed by removing from template */
      if(killdefinition){
	if(!nameScanRemove(sap->typetemplate,varname,namescan->next
			   ,1L,depth+1))return 0;
      }else{
	for(i=0;i<sap->Size;i++){
	  if(prot)printf("%*s: i=%ld\n",2*depth+24,subfunc,i);
	  if(!nameScanRemove(sap->array[i],varname,namescan->next
			     ,killdefinition,depth+1))return 0;
	}
      }
    }else{
      /* remove the sap */
      if(!vartable_clean_structarray(v)) return 0;
      goto cleannode;
    }

    
  }else if( namescan->var->structuretype == VTYPE_STRUCT){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCT,scalartype))==NULL) 
      goto internalerror;
    stp=STP(v);
    if(prot){
      printf("%*s: stp=%p name=%s \n",2*depth+25,subfunc,(void*)stp,v->name);
    }

    if(namescan->next){
      if(!nameScanRemove(stp,varname,namescan->next
			 ,killdefinition,depth+1)) return 0;
    }else{
      if(!clean_vartable(v->data.list)) return 0;
      FREEE(v->data.list,v->name);  v->data.list=NULL;
      goto cleannode;
    }
    
  }else if( namescan->var->structuretype == VTYPE_SCALAR){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_SCALAR,scalartype))==NULL) 
      goto internalerror;
    
    if(prot){
      printf("%*s: scp=%p name=%s value=%c namescan->var=%p,%p\n"
	     ,2*depth+25,subfunc,(void*)SCP(v)
	     ,v->name,SCP(v)->value[0],(void*)namescan->var,(void*)v);
    }
    
    if(!vartable_clean_scalar(v)) return 0;
    goto cleannode;

  }else if( namescan->var->structuretype == VTYPE_ARRAY){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_ARRAY,scalartype))==NULL) 
      goto internalerror;
    arp=ARP(v);
    if(prot) printf("%*s: arp=%p arprank=%ld  namescan->var=%p,%p\n"
		    ,2*depth+24,subfunc,(void*)arp,arp->rank
		    ,(void*)namescan->var,(void*)v);
    if(!vartable_clean_array(v)) return 0;
    goto cleannode;
  }
  
  return 1;
 cleannode:
  FREEE(v->name,"name"); v->name=NULL;
  if(v->next){
    v->next->prev=v->prev;
  }
  v->prev->next=v->next;
  FREEE(v,"v");v=NULL;
  return 1;
 internalerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': internal error!"
		,subfunc,varname);
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long getNameScanValue(VARTABLE *table,const char *varname,void *result
		      ,struct tsnamescan *namescan,long baseindex
		      ,long scalartype,const long result_len
		      ,const FPLOINTEGER rank,const FPLOINTEGER size[7]
		      ,const long asstring
		      ,int depth
		      )
{
  char *subfunc="getNameScanValue";
  tsSTRUCTARRAY *sap;
  tsARRAY7 *arp;
  VARTABLE *stp;
  VARTABLE *v;
  long i,n,dim;
  long prot=0;

  if(!namescan) return 1;

  if(prot){
    if(depth==0) printf("\n");
    printf("%*s: varname=%s\n",2*depth+24,subfunc,varname);
    printf("%*s: structuretype=%ld\n",2*depth+24,subfunc
	   ,namescan->var->structuretype);
    printf("%*s: table=%p %s\n",2*depth+24,subfunc,(void*)table,table->name);
    printf("%*s: tabletype=%ld\n",2*depth+24,subfunc,table->structuretype);
    printf("%*s: next=%p\n",2*depth+24,subfunc,(void*)namescan->next);
    printf("%*s: name=%s\n",2*depth+24,subfunc,namescan->var->name);
  }

 if( namescan->var->structuretype == VTYPE_STRUCTARRAY){
   dim=rank-1;
   
   if((v=lookup_list(table,namescan->var->name,NULL
		     ,VTYPE_STRUCTARRAY,0L))==NULL) 
     goto internalerror;
   sap=SAP(v);
   
   if(prot){
     printf("%*s: sap=%p size=%ld sarank=%ld\n",2*depth+24,subfunc,(void*)sap
	    ,sap->Size,rank);
     printf("%*s: sasize[%ld]=%ld\n",2*depth+24,subfunc,dim,size[dim]);
   }

   if(size[dim] <sap->Size) goto sapsizeerror;

   n=1;
   for(i=0;i<rank-1;i++)
     n*=size[i];

   if(namescan->next){
     for(i=0;i<sap->Size;i++){
       
       if(prot)printf("%*s: i=%ld\n",2*depth+24,subfunc,i);
       if(!getNameScanValue(sap->array[i],varname,result,namescan->next
			    ,baseindex+n*i
			    ,scalartype,result_len
			    ,dim,size,asstring,depth+1)) return 0;
     }

     if(size[dim]>sap->Size){
       if(prot)printf("%*s: dim=%ld padding n=%ld from=%ld to=%ld\n"
		      ,2*depth+24,subfunc,dim,n
		      ,baseindex+i*n
		      ,baseindex+i*n+n*(size[dim]-sap->Size)-1
		      );
       if(!getArrayNaturalSection(varname,(void*)1L
				  ,result,result_len
				  ,0L,(size[dim]-sap->Size)*n
				  ,0L,baseindex+i*n
				  ,scalartype,table->aschar,asstring))
	 return 0;
     }

   }else
     goto internalerror;
   

   
 }else if( namescan->var->structuretype == VTYPE_STRUCT){
   
   if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCT,0L))==NULL) 
      goto internalerror;
    stp=STP(v);
    if(prot) printf("%*s: stp=%p \n",2*depth+25,subfunc,(void*)stp);


    if(namescan->next){
      if(!getNameScanValue(stp,varname,result,namescan->next,baseindex
			   ,scalartype,result_len
			   ,rank,size,asstring,depth+1)) return 0;


    }else{
      goto internalerror;
    }
    
 }else if( namescan->var->structuretype == VTYPE_ARRAY){
   dim=rank-1;

   if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_ARRAY,0L))==NULL) 
      goto internalerror;
   arp=ARP(v);

   if(prot) printf("%*s: arp=%p arprank=%ld dim=%ld\n"
		   ,2*depth+24,subfunc,(void*)arp,arp->rank,dim);

   if(depth==0){
     if(!getArrayNaturalValue(v,result,result_len,rank,size,asstring)) 
       return 0;
   }else{
     if(arp->rank!=1) goto rankerror;
     
     if(size[dim] <arp->Size[1]) goto arpsizeerror;

     if(!getArrayNaturalSection(v->name,arp->array
				,result,result_len
				,arp->Size[1],size[0]
				,0L,baseindex
				,scalartype,table->aschar,asstring))
       return 0;
     
   }

 }else if( namescan->var->structuretype == VTYPE_SCALAR){
   if((v=lookup_list(table,namescan->var->name,NULL
		     ,VTYPE_SCALAR,scalartype))==NULL) 
     goto internalerror;
   if(prot) printf("%*s: scp=%p name=%s value=%s at %ld\n"
		   ,2*depth+25,subfunc,(void*)SCP(v)
		   ,v->name,SCP(v)->value,baseindex);

   if(!getScalarNaturalValue(v,result,baseindex
			     ,result_len,asstring)) return 0;

 }  
  
  return 1;
 internalerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': internal error!"
		,subfunc,varname);
  return 0;

 sapsizeerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': dummy argument result, "
		"dimension %ld:\n"
		"size ("FPLOINTEGERFORMAT") < maximum array size (%ld)!"
		,subfunc,varname,dim+1,size[dim],sap->Size);
  return 0;
 arpsizeerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': dummy argument result, "
		"dimension %ld:\n"
		"size ("FPLOINTEGERFORMAT") < maximum array size (%ld)!"
		,subfunc,varname,dim+1,size[dim],arp->Size[0]);
  return 0;

 rankerror:
  (void)sprintf(errorbuffer
	  ,"error(%s): Internal error: (level>1)-array variable '%s' "
		"has rank > 1!"
		,subfunc,varname);
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long getValue(VARTABLE *table,const char *name,void *result
	      ,long scalartype,const long result_len,const FPLOINTEGER rank
	      ,const FPLOINTEGER size[7]
	      ,const long asstring)

{
  char *subfunc="getValue";
  long lrank;
  tsNAMSCAN namescan,*nx;

  
  if((!table->aschar) && asstring){
    (void)sprintf(errorbuffer 
	    ,"error(%s): Cannot read variable %s as string while parser\n"
	    " works in non character mode (i.e. external arrays used)!" 
	    ,subfunc,name); 
    return 0; 
  }
  /*if(asstring && scalartype!=VTYPE_REAL){
    (void)sprintf(errorbuffer 
	    ,"error(%s): Cannot read variable %s as string, only real\n"
	    " variables have this option!" 
	    ,subfunc,name); 
    return 0; 
  }*/

  /* fill in namescan to traverse the data */
  if(!getNameScan(table,name,VTYPE_ARRAY | VTYPE_SCALAR,scalartype
		  ,&namescan,0L)) goto errorend;

  nx=&namescan;
  lrank=0;
  while(nx){
    if(nx->var->structuretype==VTYPE_STRUCTARRAY) lrank++;
    if(nx->var->structuretype==VTYPE_ARRAY) {
      lrank+=ARP(nx->var)->rank;
    }
    nx=nx->next;
  }

  if(rank<lrank) goto rankerror;

  /*
    printf("rank=%ld\n",rank);
    for(i=0;i<rank;i++){
    printf("size[%ld]=%ld\n",i,size[i]);
    }
  */

  if(!getNameScanValue(table,name,result,&namescan,0L,scalartype,result_len
		       ,rank,size,asstring,0L)) goto errorend;

  cleanup_namescan(&namescan);
  return 1;



 errorend:
  cleanup_namescan(&namescan);
  return 0;

 rankerror:
  (void)sprintf(errorbuffer,"error(%s): Variable '%s': "
		"rank of dummy variable sizearrays does not fit\n "
		"rank of data object!"
		,subfunc,name);
  goto errorend;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long getNameScanSizeArray(VARTABLE *table,const char *varname
			  ,struct tsnamescan *namescan
			  ,long maxdim
			  ,const FPLOINTEGER sarank,const FPLOINTEGER sasize[7]
			  ,long baseindex
			  ,FPLOINTEGER *sizearray
			  ,int rdepth
			  ,int depth
			  )
{
  char *subfunc="getNameScanSizeArray";
  VARTABLE *v;
  tsSTRUCTARRAY *sap;
  VARTABLE *stp;
  tsARRAY7 *arp;
  long i,j,dim,n;
  long prot=0;
  if(!namescan) return 1;

  if(prot){
    if(depth==0) printf("\n");
    printf("%*s: varname=%s\n",2*depth+24,subfunc,varname);
    printf("%*s: structuretype=%ld\n",2*depth+24,subfunc
	   ,namescan->var->structuretype);
    printf("%*s: table=%p %s\n",2*depth+24,subfunc,(void*)table,table->name);
    printf("%*s: tabletype=%ld\n",2*depth+24,subfunc,table->structuretype);
    printf("%*s: next=%p\n",2*depth+24,subfunc,(void*)namescan->next);
    printf("%*s: name=%s\n",2*depth+24,subfunc,namescan->var->name);
  }

  if( namescan->var->structuretype == VTYPE_STRUCTARRAY){
    dim=sarank-1;

    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCTARRAY,0L))==NULL) 
      goto internalerror;
    sap=SAP(v);

    if(prot){
      printf("%*s: sap=%p size=%ld sarank=%ld\n",2*depth+24,subfunc,(void*)sap
	     ,sap->Size,sarank);
      printf("%*s: sasize[%ld]=%ld\n",2*depth+24,subfunc,dim,sasize[dim]);
    }

    if(sarank==0){
      if(namescan->next){
 	if(!namescan->next->next){
	  if(namescan->next->var->structuretype & (VTYPE_SCALAR)){ 
	  }else
	    goto internalerror; 
	}else
	  goto internalerror; 
      }else{
	if(namescan->var->structuretype & (VTYPE_STRUCTARRAY)){ 
	}else
	  goto internalerror; 
      }

      if(baseindex>=maxdim)goto dimerror;
      sizearray[baseindex]=sap->Size;


      if(prot) printf("%*s: sizearray[%ld]=%ld\n",2*depth+24,subfunc
		      ,baseindex,sap->Size); 
      
      
    }else{
      if(sasize[dim] <sap->Size) goto sapsizeerror;
      n=1;
      for(i=0;i<sarank-1;i++)
	n*=sasize[i];
      if(namescan->next){
	for(i=0;i<sap->Size;i++){
	  
	  if(prot)printf("%*s: i=%ld\n",2*depth+24,subfunc,i);
	  
	  if(!getNameScanSizeArray(sap->array[i],varname,namescan->next
				   ,maxdim,dim,sasize,baseindex+n*i,sizearray
				   ,rdepth+1
				   ,depth+1)){
	    return 0;
	  }
	}
	for(i=sap->Size;i<sasize[dim];i++){
	  for(j=0;j<n;j++){
	    if(baseindex+n*i+j>=maxdim)goto dimerror;
	    sizearray[baseindex+n*i+j]=0;


	    if(prot) printf("%*s: sizearray[%ld]=%ld\n",2*depth+24,subfunc
			    ,baseindex+n*i+j,0L); 


	  }
	}
	
      }
    }
  }else if( namescan->var->structuretype == VTYPE_STRUCT){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCT,0L))==NULL) 
      goto internalerror;
    stp=STP(v);
    
    
    if(prot) printf("%*s: stp=%p \n",2*depth+25,subfunc,(void*)stp);
    

    if(namescan->next){
      if(!getNameScanSizeArray(stp,varname,namescan->next
			       ,maxdim,sarank,sasize,baseindex,sizearray,rdepth
			       ,depth+1)){
	return 0;
      }
    }else{
      if(rdepth==0) goto rank0error;
    }
    
  }else if( namescan->var->structuretype == VTYPE_ARRAY){
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_ARRAY,0L))==NULL) 
      goto internalerror;
    
    arp=ARP(v);
    
    if(prot) printf("%*s: arp=%p sarank=%ld\n",2*depth+24,subfunc,(void*)arp
		    ,sarank);


    n=1;
    for(i=arp->rank;i>=2;i--){
      dim=sarank-1+i-arp->rank;
      n=n*LMAX(arp->Size[i],sasize[dim]);
      if(sasize[dim] <arp->Size[i]) goto arpsizeerror;
      
      if(prot){
	printf("%*s: arp[%ld].size=%ld\n",2*depth+24,subfunc,i,arp->Size[i]);
	printf("%*s: sasize[%ld]=%ld\n",2*depth+24,subfunc,dim,sasize[dim]);
      }


    }
    for(j=0;j<n;j++){ 
      if(baseindex+j>=maxdim)goto dimerror;
      sizearray[baseindex+j]=arp->Size[1];
      
      if(prot) printf("%*s: sizearray[%ld]=%ld\n",2*depth+24,subfunc
		      ,baseindex+j,arp->Size[1]); 
      
    }
  }else if( namescan->var->structuretype == VTYPE_SCALAR){
    if(rdepth==0) goto rank0error;
  }
  return 1;
 internalerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': internal error!"
		,subfunc,varname);
  return 0;
 arpsizeerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': dummy argument sizearray, "
		"dimension %ld:\n"
		"size ("FPLOINTEGERFORMAT") < maximum array size (%ld)!"
		,subfunc,varname,dim+1,sasize[dim],arp->Size[i]);
  return 0;
 sapsizeerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable '%s': dummy argument sizearray, "
		"dimension %ld:\n"
		"size ("FPLOINTEGERFORMAT") < maximum array size (%ld)!"
		,subfunc,varname,dim+1,sasize[dim],sap->Size);
  return 0;
 rank0error:
  (void)sprintf(errorbuffer
	  ,"error(%s): variable '%s' is not an object which has dimensions!"
	  ,subfunc,varname);
  return 0;
 dimerror:
  (void)sprintf(errorbuffer
	  ,"error(%s): variable '%s', dimensions of sizearray to small!"
	  ,subfunc,varname);
  return 0;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long getSizeArray(VARTABLE *table,const char *name
		  ,const FPLOINTEGER sarank,const FPLOINTEGER sasize[7],FPLOINTEGER *sizearray)
{
  char *subfunc="getSizeArray";
  long i,rank,maxdim;
  tsNAMSCAN namescan,*nx;

  maxdim=1;
  for(i=0;i<sarank;i++){
    maxdim*=sasize[i];
    /*    printf("sasize[%ld]=%ld\n",i,sasize[i]); */
  }

  /* fill in namescan to traverse the data */
  if(!getNameScan(table,name,VTYPE_ARRAY|VTYPE_STRUCTARRAY
		     |VTYPE_SCALAR | VTYPE_STRUCT,0L
		     ,&namescan,0L))goto errorend;

  nx=&namescan;
  rank=0;
  while(nx){
    if(nx->var->structuretype==VTYPE_STRUCTARRAY) rank++;
    if(nx->var->structuretype==VTYPE_ARRAY) {
      rank+=ARP(nx->var)->rank;
    }
    nx=nx->next;
  }

  if(sarank+1!=rank) goto rankerror;


  if(sarank<1)goto rankonemeaningerror;


  if(!getNameScanSizeArray(table,name,&namescan,maxdim,sarank,sasize,0L
			   ,sizearray,0L,0L))
    goto errorend;


  cleanup_namescan(&namescan);

  return 1;


  
 errorend:
  cleanup_namescan(&namescan);
  return 0;


 rankonemeaningerror:
  (void)sprintf(errorbuffer,"error(%s): Variable '%s': "
		"sizearrays not meaningfull\nfor "
		"rank one objects. Use normal rank and size "
		"query function instead!"
		,subfunc,name);
  goto errorend;

 rankerror:
  (void)sprintf(errorbuffer,"error(%s): Variable '%s': "
		"rank of dummy variable sizearrays does not fit\n "
		"rank of data object!"
		,subfunc,name);
  goto errorend;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long getNameScanDimensions(VARTABLE *table,const char *varname
		      ,struct tsnamescan *namescan
		      ,long *rank,long size[7]
		      ,int depth)
{
  char *subfunc="getNameScanDimensions";
  tsSTRUCTARRAY *sap;
  VARTABLE *stp;
  tsARRAY7 *arp;
  VARTABLE *v;
  struct tsnamescan *nx;
  long i,n;
  long prot=0;

  if(!namescan) return 1;
  if(prot){
    printf("%*s: varname=%s\n",2*depth+25,subfunc,varname);
    printf("%*s: structuretype=%ld\n",2*depth+25,subfunc
	   ,namescan->var->structuretype);
    printf("%*s: table=%p %s\n",2*depth+25,subfunc,(void*)table,table->name);
    printf("%*s: tabletype=%ld\n",2*depth+25,subfunc,table->structuretype);
    printf("%*s: next=%p\n",2*depth+25,subfunc,(void*)namescan->next);
    printf("%*s: name=%s\n",2*depth+25,subfunc,namescan->var->name);
  }  
  if( namescan->var->structuretype == VTYPE_STRUCTARRAY){
    n=depth;
    (*rank)=LMAX((*rank),n+1);

    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCTARRAY,0L))==NULL) 
      goto internalerror;
    
    sap=SAP(v);
    size[n]=LMAX(size[n],sap->Size);
    if(prot){
      printf("%*s: sap=%p size=%ld\n",2*depth+25,subfunc,(void*)sap,sap->Size);
      printf("%*s: maxsize[%ld]=%ld\n",2*depth+25,subfunc,n,size[n]);
    }

    if(namescan->next){
      for(i=0;i<sap->Size;i++){
	if(prot) printf("%*s: i=%ld\n",2*depth+25,subfunc,i);
	if(!getNameScanDimensions(sap->array[i],varname,namescan->next
			     ,rank,size,depth+1)){
	  return 0;
	}
      }
      if(!sap->Size){
	nx=namescan->next;
	(*rank)=depth+1;
	while(nx){
	  if( nx->var->structuretype&(VTYPE_STRUCTARRAY|VTYPE_ARRAY)){
	    if(prot){
	      printf("%*s: maxsize[%ld]=%ld\n",2*depth+25,subfunc,(*rank)
		     ,size[(*rank)]);
	    }
	    (*rank)++;
	  }
	  nx=nx->next;
	}
      }
    }
  }else if( namescan->var->structuretype == VTYPE_STRUCT){
    /*v=table->next;
      while(v){
      HIER(v->name);
      v=v->next;
    }*/
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_STRUCT,0L))==NULL) {
      goto internalerror;
    }

    stp=STP(v);
    if(prot) printf("%*s: stp=%p \n",2*depth+25,subfunc,(void*)stp);
    if(namescan->next){
      if(!getNameScanDimensions(stp,varname,namescan->next
			   ,rank,size,depth)){
	return 0;
      }
    }
    
  }else if( namescan->var->structuretype == VTYPE_ARRAY){
    /* (*rank)=LMAX((*rank),depth+1); */
    if((v=lookup_list(table,namescan->var->name,NULL
		      ,VTYPE_ARRAY,0L))==NULL) 
      goto internalerror;
    arp=ARP(v);
    if(prot) printf("%*s: arp=%p\n",2*depth+25,subfunc,(void*)arp);
    for(i=1;i<=arp->rank;i++){
      n=depth+arp->rank-i;
      (*rank)=LMAX((*rank),n+1);
      size[n]=LMAX(size[n],arp->Size[i]);
      if(prot){
	printf("%*s: arp[%ld].size=%ld\n",2*depth+25,subfunc,i,arp->Size[i]);
	printf("%*s: maxsize[%ld]=%ld\n",2*depth+25,subfunc,n,size[n]);
	printf("%*s: i=%ld rank=%ld\n",2*depth+25,subfunc,i,*rank);
      }
    }
  }else if( namescan->var->structuretype == VTYPE_SCALAR){
    /*HIER("SCP");*/
  }else{
    /*HIER("unkown");*/
  }

  return 1;
 internalerror:
  (void)sprintf(errorbuffer
		,"error(%s): variable %s: internal error!"
		,subfunc,varname);
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long getDimensions(VARTABLE *table,const char *name
		   ,FPLOINTEGER *rank,FPLOINTEGER size[7])
{
  /* char *subfunc="getDimensions"; */
  long i;
  tsNAMSCAN namescan;
  long lrank,lsize[7];

  /* fill in namescan to traverse the data */
  if(!getNameScan(table,name,VTYPE_ARRAY|VTYPE_STRUCTARRAY
		     |VTYPE_SCALAR | VTYPE_STRUCT,0L
		     ,&namescan,0L))goto errorend;
  

  lrank=0; for(i=0;i<7;i++)lsize[i]=0;
  if(!getNameScanDimensions(table,name,&namescan,&lrank,lsize,0L)) 
    goto errorend;
  *rank=lrank; for(i=0;i<*rank;i++) size[*rank-1-i]=(FPLOINTEGER)lsize[i];
  
  /*
    if(*rank==0){
     (void)sprintf(errorbuffer
		  ,"error(%s): variable '%s' is not an object,"
		  " which has dimensions!",subfunc,name);
    goto errorend;
  }
  */
  cleanup_namescan(&namescan);
  return 1;

 errorend:
  cleanup_namescan(&namescan);
  return 0;
}












/* ***********************************************************************
 *
 * ***********************************************************************
 */

long parsefile(FILE **yyin,const char *fname,PARSER_CONTROLL *pc)
{
  long ret=1;
  char *subfunc="parsefile";
 
  errorbuffer[0]='\0';
  
  if((*yyin = fopen(fname,"r"))==NULL){
    (void)sprintf(errorbuffer,"error(%s): Could not open file %s!",subfunc,fname);
    return 0;
  }
  
  /* parse file */
  if(yyparse (pc)){
    ret=0;
  }
  
  (void)fclose(*yyin);   
  yyin=NULL;
  return ret;
}



/* ***********************************************************************
 * returns in result the natural value (INT as INTEGER,LOGICAL as L_FALSE|L_TRUE
 * ... )
 * ***********************************************************************
 */
long getScalarNaturalValue(VARTABLE *var,void *result,const long roffset
			   ,const long result_len,long asstring)
{
  char *subfunc="getScalarNaturalValue";
  void *value;
  long i;

  if(var==NULL){
    (void)sprintf(errorbuffer,"error(%s): dummy argument var == NULL!"
	    ,subfunc);
    return 0;
  }
  if(result==NULL){
    (void)sprintf(errorbuffer,"error(%s): dummy argument result == NULL!"
	    ,subfunc);
    return 0;
  }
  
  value=var->data.scalar.value;
  
  if(value==NULL){
    (void)sprintf(errorbuffer,"error(%s): Value of scalar variable %s == NULL!"
	    ,subfunc,var->name);
    return 0;
  }
  
  if(var->data.scalar.scalartype == VTYPE_INT) {
    if(var->aschar){
      if(asstring){
	/* this is fortran specific conversion code*/
	if(!c2fstrncpy((char*)result+roffset*result_len
		       ,(char*)value,result_len,0L))
	  return 0;
      }else{
	if(!convertlong(value,((FPLOINTEGER*)result)+roffset)) 
	  return 0; 
      }
    }else{
      ((FPLOINTEGER*)result)[roffset]=*((FPLOINTEGER*)value);
    }
  }else if(var->data.scalar.scalartype == VTYPE_REAL) {
    if(var->aschar){
      if(asstring){
	/* this is fortran specific conversion code*/
	if(!c2fstrncpy((char*)result+roffset*result_len
		       ,(char*)value,result_len,0L))
	  return 0;
      }else{
	if(!convertfortranreal(value,((FPLODOUBLEPRECISION*)result)+roffset))
	  return 0; 
      }
    }else{
      ((FPLODOUBLEPRECISION*)result)[roffset]=*((FPLODOUBLEPRECISION*)value);
    }
  }else if(var->data.scalar.scalartype == VTYPE_CHAR) {
    ((char*)result)[roffset*result_len]=*((char*)value);
    /* this is fortran specific conversion code*/
    for(i=1;i<result_len;i++){
      ((char*)result)[i+roffset*result_len]=' '; /* paddout whole string */
    }
  }else if(var->data.scalar.scalartype == VTYPE_LOGICAL) {
    if(var->aschar){
      if(asstring){
	((char*)result)[roffset*result_len]=*((char*)value);
	/* this is fortran specific conversion code*/
	for(i=1;i<result_len;i++){
	  ((char*)result)[i+roffset*result_len]=' '; /* paddout whole string */
	}
      }else{
	((FPLOLOGICAL*)result)[roffset]
	  =(tolower(*((char*)value))=='f'?L_FALSE:L_TRUE);
      }
    }else{
      ((FPLOLOGICAL*)result)[roffset]=*((FPLOLOGICAL*)value);
    }
    
  }else if(var->data.scalar.scalartype & ( VTYPE_FLAG | VTYPE_STRING )) {
    /* this is fortran specific conversion code*/
    if(!c2fstrncpy((char*)result+roffset*result_len
		   ,(char*)value,result_len,0L))
      return 0;

  }else{
    (void)sprintf(errorbuffer,"error(%s): internal error: scalar variable %s: unknown type!"
	    ,subfunc,var->name);
    return 0;
  }
  
  
  return 1;
}


/* ***********************************************************************
 * character arrays are padded with space, if array or arraymembers
 * are shorter than fortran definition
 * ***********************************************************************
 */
static long getArrayNaturalValue(VARTABLE *var
				 ,void  *result
				 ,const long result_len
				 ,const FPLOINTEGER rank
				 ,const FPLOINTEGER size[7]
				 ,const long asstring
				 )
{
  char *subfunc="getArrayNaturalValue";
  tsARRAY7 *arp;
  long ind,maxind;
  void *array;


  if(var==NULL){
    (void)sprintf(errorbuffer,"error(%s): dummy argument var == NULL!"
	    ,subfunc);
    return 0;
  }
  
  arp=&var->data.array7;  
  array=arp->array;  

  if(array==NULL){
    (void)sprintf(errorbuffer,"error(%s): Arraypointer of array %s == NULL!"
	    ,subfunc,var->name);
    return 0;
  }
  

  if(arp->rank!=rank){
    (void)sprintf(errorbuffer
	    ,"error(%s): Data array %s has rank %ld, not "FPLOINTEGERFORMAT"!"
		  ,subfunc,var->name,arp->rank,rank); 
    return 0;
  }



  if(arp->Size[rank]>size[rank-1]){
    (void)sprintf(errorbuffer
	    ,"error(%s): Data array %s, last dimension ("FPLOINTEGERFORMAT"): "
	    "size (%ld) > size of dummy argument ("FPLOINTEGERFORMAT")! "
	    ,subfunc,var->name
	    ,rank
	    ,arp->Size[rank]	    
	    ,size[rank-1]); 
    return 0;
  }

  
  ind=1;
  switch ((int)rank)
    {
    case 7: 
      ind=ind*arp->Size[6]; 
      if(arp->Size[6]!=size[5]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 6: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name,arp->Size[6],size[5]); 
	return 0;
      }
    case 6: 
      ind=ind*arp->Size[5]; 
      if(arp->Size[5]!=size[4]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 5: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name ,arp->Size[5] ,size[4] ); 
	return 0;
      }
    case 5: 
      ind=ind*arp->Size[4]; 
      if(arp->Size[4]!=size[3]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 4: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name  ,arp->Size[4] ,size[3] ); 
	return 0;
      }
    case 4: 
      ind=ind*arp->Size[3]; 
      if(arp->Size[3]!=size[2]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 3: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name,arp->Size[3] ,size[2] ); 
	return 0;
      }
    case 3: 
      ind=ind*arp->Size[2]; 
      if(arp->Size[2]!=size[1]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 2: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name,arp->Size[2] ,size[1] ); 
	return 0;
      }
    case 2: 
      ind=ind*arp->Size[1]; 
      if(arp->Size[1]!=size[0]){
	(void)sprintf(errorbuffer
		,"error(%s): Data array %s, "
		"dimension 1: size is %ld, not "FPLOINTEGERFORMAT"! "
		,subfunc,var->name,arp->Size[1] ,size[0] ); 
	return 0;
      }
    }


    
  maxind=ind*size[rank-1];
  ind=ind*arp->Size[rank];
  
  
  if(!getArrayNaturalSection(var->name,array,result,result_len
			     ,ind,maxind,0L,0L,arp->scalartype
			     ,var->aschar,asstring))
    return 0;
      
      
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long getArrayNaturalSection(const char *name,const void *array
				   ,void *result,const long result_len
				   ,const long asize,const long rsize
				   ,const long aoffset,const long roffset
				   ,const long scalartype
				   ,const long aschar
				   ,const long asstring
				   )
{
  char *subfunc="getArraySection";
  long j,i;

  if(array==NULL){
    (void)sprintf(errorbuffer,"error(%s): dummy argument array == NULL!"
	    ,subfunc);
    return 0;
  }
  if(result==NULL){
    (void)sprintf(errorbuffer,"error(%s): dummy argument result == NULL!"
	    ,subfunc);
    return 0;
  }

  if(scalartype==VTYPE_REAL){
    if(aschar){
      if(asstring){
	for(j=0;j<asize;j++){
	  /* this is fortran specific conversion code*/
	  if(!c2fstrncpy(((char*)result)+result_len*(j+roffset)
			 ,((char**)array)[j+aoffset],result_len,0L))
	    return 0;
	  
	}
	
      }else{
	for(j=0;j<asize;j++){
	  if(!convertfortranreal(((char**)array)[j+aoffset]
				 ,(FPLODOUBLEPRECISION*)result+j+roffset))
	    return 0; 
	}
      }
    }else{
      for(j=0;j<asize;j++){
	((FPLODOUBLEPRECISION*)result)[j+roffset]=((FPLODOUBLEPRECISION*)array)[j+aoffset];
      }
    }
    if(asstring){
      for(j=asize;j<rsize;j++){
	for(i=0;i<result_len;i++){
	  *(((char*)result)+result_len*(j+roffset)+i)=' '; 
	}
      } 
    }else{
      /* padding with 0 */
      for(j=asize;j<rsize;j++){
	((FPLODOUBLEPRECISION*)result)[j+roffset]=0.0; 
      }
    }
  }else if(scalartype==VTYPE_LOGICAL){
    if(aschar){
      if(asstring){
	/* this is fortran specific conversion code*/
	for(j=0;j<asize;j++){
	  *(((char*)result)+result_len*(j+roffset))=((char*)array)[j+aoffset];
	  for(i=1;i<result_len;i++){
	    *(((char*)result)+result_len*(j+roffset)+i)=' ';
	  }
	  
	}
	/* padding with space */
	for(j=asize;j<rsize;j++){
	  for(i=0;i<result_len;i++){
	    *(((char*)result)+result_len*(j+roffset)+i)=' '; 
	  }
	  
	} 

      }else{
	for(j=0;j<asize;j++){
	  ((FPLOLOGICAL*)result)[j+roffset]
	    =(((char*)array)[j+aoffset]=='f'?L_FALSE:L_TRUE);
	}
      }
    }else{
      for(j=0;j<asize;j++){
	  ((FPLOLOGICAL*)result)[j+roffset]
	    =((FPLOLOGICAL*)array)[j+aoffset];
      }
    }
    /* padding with 0 */
    for(j=asize;j<rsize;j++){
      ((FPLOINTEGER*)result)[j+roffset]=L_FALSE; 
    }
    
  }else if(scalartype==VTYPE_INT){
    if(aschar){
      if(asstring){
	for(j=0;j<asize;j++){
	  /* this is fortran specific conversion code*/
	  if(!c2fstrncpy(((char*)result)+result_len*(j+roffset)
			 ,((char**)array)[j+aoffset],result_len,0L))
	    return 0;
	  
	}
      }else{
	for(j=0;j<asize;j++){
	  if(!convertlong(((char**)array)[j+aoffset],(FPLOINTEGER*)result+j+roffset))
	    return 0; 
	}
      }
    }else{
      for(j=0;j<asize;j++){
	((FPLOINTEGER*)result)[j+roffset]=((FPLOINTEGER*)array)[j+aoffset];
      }
    }
    /* padding with 0 */
    for(j=asize;j<rsize;j++){
      ((FPLOINTEGER*)result)[j+roffset]=0; 
    }
    
      
  }else if(scalartype==VTYPE_CHAR){
    /* this is fortran specific conversion code*/
    for(j=0;j<asize;j++){
      *(((char*)result)+result_len*(j+roffset))=((char*)array)[j+aoffset];
      for(i=1;i<result_len;i++){
	*(((char*)result)+result_len*(j+roffset)+i)=' ';
      }
      
    }
    /* padding with space */
    for(j=asize;j<rsize;j++){
      for(i=0;i<result_len;i++){
	*(((char*)result)+result_len*(j+roffset)+i)=' '; 
      }
      
    } 
  }else if(scalartype & ( VTYPE_FLAG | VTYPE_STRING )){
    for(j=0;j<asize;j++){
      /* this is fortran specific conversion code*/
      if(!c2fstrncpy(((char*)result)+result_len*(j+roffset)
		     ,((char**)array)[j+aoffset],result_len,0L))
	return 0;
      
    }
    for(j=asize;j<rsize;j++){
      for(i=0;i<result_len;i++){
	*(((char*)result)+result_len*(j+roffset)+i)=' '; 
      }
      
    } 
  }else{
    (void)sprintf(errorbuffer,"error(%s): internal error: array variable %s: unknown type!"
	    ,subfunc,name);
    return 0;
  }
  return 1;
}
