/* @(#)vartable.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "table.h"
#include "fortrantypes.h"
#include "helper.h"
#include "merror.h"

#undef  UNDEFINED_ABORT
#define UNDEFINED_ABORT


#define INDENT 4

const char * scalartypes[]={
  "?"
  ,"int"
  ,"real"
  ,"logical"
  ,"flag"
  ,"char"
  ,"string"
  ,"char or string or flag"
};

const char * structuretypes[]={
  "?"
  ,"scalar"
  ,"array"
  ,"struct"
  ,"structarray"
  ,"section"
  ,"sectionend"
  ,"placement"
  ,"comment"
  ,"scalar or array"
  ,"scalar or array or structarray"
  ,"scalar, array, struct or structarray "
  ,"array or structarray "
};








static long printstructheader(FILE *fh,VARTABLE *list,int indent);
static long printscalarbody(FILE *fh,VARTABLE *node);
static long printvar(FILE *fh,VARTABLE * node,long placement,int indent);


static long printscalarvalue(FILE *fh,long scalartype,void *array,long ind
		      ,long aschar);
static long printarraybody(FILE *fh,VARTABLE *node,int indent);
static long printstructarraybody(FILE *fh,VARTABLE *node,int indent);
static long printscalarheader(FILE *fh,VARTABLE *node);


static char escapebuffer[3];

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static const char *escapechar(char c)
{
  
  switch (c){
  case ('\"'):escapebuffer[1]='\"';   break;
  case ('\''):escapebuffer[1]='\'';   break;
  case ('\\'):escapebuffer[1]='\\';   break;
  case ('\0'):escapebuffer[1]='0';   break;
  case ('\a'):escapebuffer[1]='a';   break;
  case ('\b'):escapebuffer[1]='b';   break;    
  case ('\f'):escapebuffer[1]='f';   break;    
  case ('\n'):escapebuffer[1]='n';   break;    
  case ('\r'):escapebuffer[1]='r';   break;    
  case ('\t'):escapebuffer[1]='t';   break;
  case ('\v'):escapebuffer[1]='v';   break;
  default: escapebuffer[0]=c;escapebuffer[1]='\0'; return escapebuffer;
  }
  escapebuffer[0]='\\';
  escapebuffer[2]='\0';
  
  return escapebuffer;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long print_vartab(FILE *fh,const VARTABLE *table)
{
  char *subfunc="print_vartab"; 
  VARTABLE *node;
  long structuretype;
  int indent=0;
  
  if(!myassert(table,subfunc,"table")) return 0;
  if(!myassert(fh,subfunc,"fh")) return 0;
  
  node=table->next;
  /* fprintf(fh,"===============================================!\n");  */
  while(node!=NULL){
    structuretype=node->structuretype;

    if(structuretype==VTYPE_SECTION){
      indent+=INDENT;
      (void)fprintf(fh,"section %s{\n",node->name);
    }else if(structuretype==VTYPE_PLACEMENT){
      (void)fprintf(fh,"%*.*s",indent,indent,""); 
      if(!printvar(fh,node->data.noderef,VTYPE_DATA,indent))return 0;
    }else if(structuretype==VTYPE_COMMENT){
      if(node->name)
	(void)fprintf(fh,"%*.*s#%s\n",indent,indent,"",node->name);
    }else if(structuretype==VTYPE_SECTIONEND){
      indent=0;
      (void)fprintf(fh,"};\n\n\n");
    }else  if(structuretype==VTYPE_SCALAR){
      if(node->dataplacement==VTYPE_SEPERATE || 
	 !node->dataplacement ||
	 node->dataplacement==VTYPE_INPLACE)
	(void)fprintf(fh,"%*.*s",indent,indent,""); 
      if(!printvar(fh,node,node->dataplacement,indent))return 0;
      if(node->dataplacement==VTYPE_SEPERATE || !node->dataplacement)
	(void)fprintf(fh,"\n"); 
    }else  if(structuretype==VTYPE_ARRAY){
      if(node->dataplacement==VTYPE_SEPERATE ||
	 !node->dataplacement ||
	 node->dataplacement==VTYPE_INPLACE
	 )
	(void)fprintf(fh,"%*.*s",indent,indent,""); 
      if(!printvar(fh,node,node->dataplacement,indent))return 0;
      if(node->dataplacement==VTYPE_SEPERATE)
	(void)fprintf(fh,"\n"); 
    }else if(structuretype==VTYPE_STRUCT){
      (void)fprintf(fh,"\n"); 
      if(!printvar(fh,node,node->dataplacement,indent))return 0;
      if( node->dataplacement==VTYPE_SEPERATE ||  !node->dataplacement)
	(void)fprintf(fh,"\n"); 
      (void)fprintf(fh,"\n"); 
    }else if(structuretype==VTYPE_STRUCTARRAY){
      (void)fprintf(fh,"\n"); 
      if(!printvar(fh,node,node->dataplacement,indent))return 0;
      if( node->dataplacement==VTYPE_SEPERATE ||  !node->dataplacement)
	(void)fprintf(fh,"\n"); 
      (void)fprintf(fh,"\n"); 
      
    }else{
      goto error;
    }
    
    node=node->next;
  }
  /* fprintf(fh,"===============================================!\n");  */
  
  return 1;
 error:
  (void)sprintf(errorbuffer,"error(%s): unknown type 0x%lx!\n",subfunc,structuretype);
  return 0;
  
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printarrayheader(FILE *fh,VARTABLE *node)
{
  char *subfunc="printarrayheader";   
  long sctyind,string,i;
  tsARRAY7 *arp;
  
  if(!myassert(node,subfunc,"node")) return 0;
  arp=&node->data.array7;

  if(!scalartypeindex(&sctyind,arp->scalartype,&string,0L))return 0;
  
  (void)fprintf(fh,"%s",scalartypes[sctyind]);
  if(string){
    (void)fprintf(fh,"[");
    if(arp->strvariable){
      (void)fprintf(fh,"*");
    }else{
      (void)fprintf(fh,"%ld",arp->strlength);
    }
    (void)fprintf(fh,"]");
  }
  (void)fprintf(fh," %s[",node->name);
  for(i=1;i<=arp->rank;i++){
    if(arp->idref[i]!=NULL){
      (void)fprintf(fh,"%s",arp->idref[i]);
    }else if(i==arp->rank && arp->variable){
      (void)fprintf(fh,"*");
    }else
      (void)fprintf(fh,"%ld",arp->Size[i]);
    if(i<arp->rank)(void)fprintf(fh,",");
  }
  (void)fprintf(fh,"]");
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printstructheader(FILE *fh,VARTABLE *list,int indent)
{
  char *subfunc="printstructheader";   
  VARTABLE *table,*node;
  tsSTRUCTARRAY *sap=NULL;
  long structuretype;
  int ind=indent;
  int hascompound=0;
  int lastnoncomp=0;
  int iscomp=0;
  int previscomp=0;
  
  if(!myassert(list,subfunc,"list")) return 0;
  
  if(list->structuretype==VTYPE_STRUCT){
    table=STP(list);  
    if(!myassert(table,subfunc,"table")) return 0;
  }else if(list->structuretype==VTYPE_STRUCTARRAY){
    sap=SAP(list);
    table=sap->typetemplate;
    if(!myassert(table,subfunc,"table")) return 0;
  }else{
      (void)sprintf(errorbuffer,"error(%s): wrong argument type 0x%lx!\n"
		    ,subfunc,list->structuretype);
      return 0;
  }


  /* (void)fprintf(fh,"A\n%*.*sstruct {",indent,indent,""); */
  (void)fprintf(fh,"%*.*sstruct {",indent,indent,"");
  node=table->next;
  ind+=INDENT;
  while(node!=NULL){
    structuretype=node->structuretype;

    iscomp=(structuretype==VTYPE_STRUCT||structuretype==VTYPE_STRUCTARRAY);

    if(node->prev){
      previscomp=(node->prev->structuretype==VTYPE_STRUCT||
		  node->prev->structuretype==VTYPE_STRUCTARRAY);
      if( iscomp && ( !previscomp ) ){
	(void)fprintf(fh,"\n");
      }
      if( previscomp  && (!iscomp) ){
	(void)fprintf(fh,"%*.*s",ind,ind,"");
      }
    }

    if(structuretype==VTYPE_SCALAR){
      if(!printvar(fh,node,VTYPE_SEPERATE,ind))return 0;
    }else  if(structuretype==VTYPE_ARRAY){ 
      if(!printvar(fh,node,VTYPE_SEPERATE,ind))return 0; 
    }else  if(structuretype==VTYPE_STRUCTARRAY){ 
      hascompound=1;
      if(!printvar(fh,node,VTYPE_SEPERATE,ind))return 0; 
    }else  if(structuretype==VTYPE_STRUCT){ 
      hascompound=1;
      if(!printvar(fh,node,VTYPE_SEPERATE,ind))return 0; 
    }else{
      goto error;
    }
    if(iscomp){
      (void)fprintf(fh,"\n");
    }
    if(node->next==NULL && (!iscomp) ){
      lastnoncomp=1;
    }

    node=node->next;
  }
  if(hascompound){
    if(lastnoncomp) (void)fprintf(fh,"\n");
    (void)fprintf(fh,"%*.*s} %s",indent,indent,"",list->name);
  }else{
    (void)fprintf(fh,"} %s",list->name);
  }

  if(list->structuretype==VTYPE_STRUCTARRAY){
    (void)fprintf(fh,"[");
    if(sap->idref!=NULL){
      (void)fprintf(fh,"%s",sap->idref);
    }else if( sap->variable){
      (void)fprintf(fh,"*");
    }else
      (void)fprintf(fh,"%ld",sap->Size);
    
    (void)fprintf(fh,"]");
    
  }
  
  return 1;
  
 error:
  (void)sprintf(errorbuffer,"error(%s): unknown type 0x%lx!\n",subfunc,structuretype);
  return 0;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printstructbody(FILE *fh,VARTABLE *list,int indent,long *iscomp)
{
  char *subfunc="printstructbody";   
  VARTABLE *node;
  tsSCALAR *scp;
  long structuretype,sctyind,string,iscomp1;
  

  if(!myassert(list,subfunc,"list")) return 0;
  
  
  
  /* (void)fprintf(fh,"%*.*s",indent,indent,"");    */
  node=list->next;
  
  (*iscomp)=0;
  while(node!=NULL){
    structuretype=node->structuretype;
    
    (*iscomp)|=(structuretype == VTYPE_STRUCTARRAY ||
		structuretype == VTYPE_STRUCT );
    
    if(structuretype==VTYPE_SCALAR){
      if(node->prev!=list)
	(void)fprintf(fh,",");

      scp=&node->data.scalar;
      if((scp->value)!=NULL){
	if(!scalartypeindex(&sctyind,scp->scalartype,&string,0L))return 0;
	if(scp->scalartype & (VTYPE_FLAG | VTYPE_STRING)
	   || (node->aschar && scp->scalartype & (VTYPE_INT | VTYPE_REAL))){
	  if(!printscalarvalue(fh,scp->scalartype,&(scp->value),0L,node->aschar))return 0;
	}else{
	  if(!printscalarvalue(fh,scp->scalartype,scp->value,0L,node->aschar))return 0;
	}
      }else{
	(void)fprintf(fh,"(undefined)");
	return 0;
      }
    }else  if(structuretype==VTYPE_ARRAY){ 
      if(node->prev!=list)
	(void)fprintf(fh,",");

      (void)fprintf(fh,"{");

      /*changed */      
      if(! printarraybody(fh,node,indent) ) return 0;
      
      (void)fprintf(fh,"}");
    }else  if(structuretype==VTYPE_STRUCTARRAY){ 
      if(node->prev!=list){
	(void)fprintf(fh,"\n%*.*s,{",indent,indent,"");
      }else{
	(void)fprintf(fh,"\n%*.*s {",indent,indent,"");
      }
      if(! printstructarraybody(fh,node,indent+INDENT))return 0;

      (void)fprintf(fh,"\n%*.*s}",indent,indent,"");
      if(node->next && 
	 (!(node->next->structuretype==VTYPE_STRUCT ||
	    node->next->structuretype==VTYPE_STRUCTARRAY))){
	(void)fprintf(fh,"\n%*.*s",indent,indent,"");	  
      }

    }else  if(structuretype==VTYPE_STRUCT){ 
      if(node->prev!=list){
	(void)fprintf(fh,"\n%*.*s,{",indent,indent,"");
      }else{
	(void)fprintf(fh,"\n%*.*s {",indent,indent,"");
      }
      if(! printstructbody(fh,STP(node),indent+INDENT,&iscomp1))return 0;
      
      if(!iscomp1){
	(void)fprintf(fh,"}");
      }else{
	(void)fprintf(fh,"\n%*.*s}",indent,indent,"");
      }

      
      if(node->next && 
	 (!(node->next->structuretype==VTYPE_STRUCT ||
	    node->next->structuretype==VTYPE_STRUCTARRAY))){
	(void)fprintf(fh,"\n%*.*s",indent,indent,"");	  
      }

    }else{
      goto error;
    }
    
    node=node->next;
    /* (void)fprintf(fh,"\n"); */
  }
  
  
  
  return 1;
  
 error:
  (void)sprintf(errorbuffer,"error(%s):struct member has unknown type 0x%lx!\n",subfunc
	  ,structuretype);
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printarraybody(FILE *fh,VARTABLE *node,int indent)
{
  char *subfunc="printarraybody";   
  tsARRAY7 *arp;
  long n,i,j,k,l,m,br=4;
  
  if(!myassert(node,subfunc,"node")) return 0;
  arp=&node->data.array7;
  

  if(arp->array==NULL){
    (void)sprintf(errorbuffer,"\nerror(%s):  array %s=(undefined)\n"
		  ,subfunc,node->name);
    
    return 0;
  }

  if(arp->scalartype==VTYPE_REAL){
    br=4;
  }else if(arp->scalartype==VTYPE_INT){
    br=20;
  }else if(arp->scalartype==VTYPE_LOGICAL){
    br=30;
  }else if(arp->scalartype==VTYPE_CHAR){
    br=10;
  }else if(arp->scalartype==VTYPE_FLAG){
    br=4;
  }else{
    br=4;
  }
  
  if(arp->rank==1){
    l=0;
    /* (void)fprintf(fh,"%*.*s",indent,indent,""); */
    for(i=0;i<arp->Size[1];i++){
      l++;
      
      if(!printscalarvalue(fh,arp->scalartype,arp->array,i,node->aschar))return 0;
      
      if(l%br==0 && i<arp->Size[1]-1)
	(void)fprintf(fh,"\n%*.*s",indent,indent,"");
      if(i<arp->Size[1]-1)
	(void)fprintf(fh,",");
    }
  }else if(arp->rank==2){
    
    /* printf("%s",indent); */
    for(j=0;j<arp->Size[2];j++){
      if(j==0)(void)fprintf(fh,"{");
      else (void)fprintf(fh,",{");
      l=0;
      for(i=0;i<arp->Size[1];i++){
	l++;
	if(!printscalarvalue(fh,arp->scalartype,arp->array,i+arp->Size[1]*j
			     ,node->aschar))
	  return 0;
	
	if(l%br==0 && i<arp->Size[1]-1)
	  (void)fprintf(fh,"\n%*.*s",indent,indent,"");
	if(i<arp->Size[1]-1)
	  (void)fprintf(fh,",");
      }
      (void)fprintf(fh,"}");
      if(j<arp->Size[2]-1)
	(void)fprintf(fh,"\n%*.*s",indent,indent,"");
    }
  }else if(arp->rank==3){
    
    
    for(k=0;k<arp->Size[3];k++){
      if(k==0)(void)fprintf(fh,"%*.*s {",indent,indent,"");
      else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
      for(j=0;j<arp->Size[2];j++){
	if(j==0)(void)fprintf(fh,"{");
	else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	l=0;
	for(i=0;i<arp->Size[1];i++){
	  l++;
	  if(!printscalarvalue(fh,arp->scalartype,arp->array,
			       i+arp->Size[1]*(j+arp->Size[2]*k),node->aschar))
	    return 0;
	  
	  if(l%br==0 && i<arp->Size[1]-1)
	    (void)fprintf(fh,"\n%*.*s",indent,indent,"");
	  if(i<arp->Size[1]-1)
	    (void)fprintf(fh,",");
	}
	(void)fprintf(fh,"}");
	if(j<arp->Size[2]-1)
	  (void)fprintf(fh,"\n");
	
      }
      (void)fprintf(fh,"}");
      if(k<arp->Size[3]-1)
	(void)fprintf(fh,"\n");
    }
  }else if(arp->rank==4){
    
    
    for(m=0;m<arp->Size[4];m++){
      if(m==0)(void)fprintf(fh,"%*.*s {",indent,indent,"");
      else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
      for(k=0;k<arp->Size[3];k++){
	if(k==0)(void)fprintf(fh,"{");
	else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	for(j=0;j<arp->Size[2];j++){
	  if(j==0)(void)fprintf(fh,"{");
	  else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	  l=0;
	  for(i=0;i<arp->Size[1];i++){
	    l++;
	    if(!printscalarvalue(fh,arp->scalartype,arp->array,
				 i+arp->Size[1]*(j+arp->Size[2]
						   *(k+arp->Size[3]*m))
				 ,node->aschar))
	      return 0;
	    
	    if(l%br==0 && i<arp->Size[1]-1)
	      (void)fprintf(fh,"\n%*.*s",indent,indent,"");
	    if(i<arp->Size[1]-1)
	      (void)fprintf(fh,",");
	  }
	  (void)fprintf(fh,"}");
	  if(j<arp->Size[2]-1)
	    (void)fprintf(fh,"\n");
	  
	}
	(void)fprintf(fh,"}");
	if(k<arp->Size[3]-1)
	  (void)fprintf(fh,"\n");
      }
      (void)fprintf(fh,"}");
      if(m<arp->Size[4]-1)
	(void)fprintf(fh,"\n");
      
    }
  }else if(arp->rank==5){
    
    
    for(n=0;n<arp->Size[5];n++){
      if(n==0)(void)fprintf(fh,"%*.*s {",indent,indent,"");
      else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
      for(m=0;m<arp->Size[4];m++){
	if(m==0)(void)fprintf(fh,"{");
	else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	for(k=0;k<arp->Size[3];k++){
	  if(k==0)(void)fprintf(fh,"{");
	  else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	  for(j=0;j<arp->Size[2];j++){
	    if(j==0)(void)fprintf(fh,"{");
	    else (void)fprintf(fh,"%*.*s,{",indent,indent,"");
	    l=0;
	    for(i=0;i<arp->Size[1];i++){
	      l++;
	      if(!printscalarvalue(fh,arp->scalartype,arp->array,
				   i+arp->Size[1]*(j+arp->Size[2]
						     *(k+arp->Size[3]
						       *(m+arp->Size[4]*n)))
				   ,node->aschar))
		return 0;
	      
	      if(l%br==0 && i<arp->Size[1]-1)
		(void)fprintf(fh,"\n%*.*s",indent,indent,"");
	      if(i<arp->Size[1]-1)
		(void)fprintf(fh,",");
	    }
	    (void)fprintf(fh,"}");
	    if(j<arp->Size[2]-1)
	      (void)fprintf(fh,"\n");
	    
	  }
	  (void)fprintf(fh,"}");
	  if(k<arp->Size[3]-1)
	    (void)fprintf(fh,"\n");
	}
	(void)fprintf(fh,"}");
	if(m<arp->Size[4]-1)
	  (void)fprintf(fh,"\n");
      }
      (void)fprintf(fh,"}");
      if(n<arp->Size[5]-1)
	(void)fprintf(fh,"\n");
      
    }
  }else{
    
    (void)sprintf(errorbuffer,"error(%s): unimplemented rank !\n",subfunc);
    return 0;
    
  }
  return 1;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printscalarvalue(FILE *fh,long scalartype,void *array,long ind,long aschar)
{
  char *subfunc="printscalarvalue";   
  char *p;
  double d;
  FPLOINTEGER l;

  switch ((int)scalartype)
    {
    case VTYPE_INT: 
      if(aschar){
	(void)fprintf(fh,"%s",(char*)((char**)(array))[ind]);break;
      }else{
	l=((FPLOINTEGER*)array)[ind];
	if(l>=I_POS_INFINITY)
	  (void)fprintf(fh,"+INF");
	else if(l<=I_NEG_INFINITY)
	  (void)fprintf(fh,"-INF");
	else
	  (void)fprintf(fh,FPLOINTEGERFORMAT,l);
	break;
      }
    case VTYPE_REAL:
      if(aschar){
	(void)fprintf(fh,"%s",(char*)((char**)(array))[ind]);break;
      }else{
	d=((FPLODOUBLEPRECISION*)array)[ind];
	if(d>=R_POS_INFINITY)
	  (void)fprintf(fh,"+INF");
	else if(d<=R_NEG_INFINITY)
	  (void)fprintf(fh,"-INF");
	else
	  (void)fprintf(fh,"%#.16g",d);
	break;
      }
      
    case VTYPE_LOGICAL:
      if(aschar){
	(void)fprintf(fh,"%c",((char*)array)[ind]);break;
      }else{
	(void)fprintf(fh,"%c",(((FPLOLOGICAL*)array)[ind]==L_FALSE?'f':'t'));break;
      }
    case VTYPE_FLAG: 
      (void)fprintf(fh,"%s(%c)",((char**)(array))[ind]+1
	      ,((char**)(array))[ind][0]);break;
    case VTYPE_STRING: 
      {
	(void)fprintf(fh,"\"");
	p=(char*)((char**)(array))[ind];
	while(*p != '\0'){
	  (void)fprintf(fh,"%s",escapechar(*p));
	  p++;
	}
	(void)fprintf(fh,"\"");
      }
      break;
    case VTYPE_CHAR: 
      {
	(void)fprintf(fh,"'%s'",escapechar(((char*)array)[ind]));break;
      }
    default: 
      {
	(void)sprintf(errorbuffer,"error(%s): Unknown type 0x%lx!\n"
		,subfunc,scalartype);
	return 0;
      }
    }
  return 1;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long printscalarbody(FILE *fh,VARTABLE *node)
{
  char *subfunc="printscalarbody";    
  
#ifdef UNDEFINED_ABORT
  if((node->data.scalar.value)==NULL){
    (void)sprintf(errorbuffer,"\nerror(%s):  scalar %s=(undefined)\n"
	    ,subfunc,node->name);
    
    return 0;; /* may be recovery 
		  this is a check which belongs elsewhere, but is
		  important if fedit use, may be an undefined 
		  value should be installed and parsed */
    
  }
#endif
  
  if((node->data.scalar.value)!=NULL){
    if(node->data.scalar.scalartype & (VTYPE_FLAG | VTYPE_STRING)
       || (node->aschar && node->data.scalar.scalartype & (VTYPE_INT | VTYPE_REAL))){
      if(!printscalarvalue(fh,node->data.scalar.scalartype,&(node->data.scalar.value),0,node->aschar))return 0;
    }else{
      if(!printscalarvalue(fh,node->data.scalar.scalartype,node->data.scalar.value,0,node->aschar))return 0;
    }
  }else{
    (void)fprintf(fh,"(undefined)");
    return 0;
  }
  
  return 1;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printvar(FILE *fh,VARTABLE * node,long placement,int indent)
{
  char *subfunc="printvar"; 
  long structuretype;
  long iscomp;
  if(!myassert(node,subfunc,"node"))
    return 0;
    
  
  
  structuretype=node->structuretype;
  
  
  
  
  /* (void)fprintf(fh,"%*.*s",indent,indent,"");  */
  if(structuretype==VTYPE_SCALAR){
    if(placement==VTYPE_SEPERATE){ 
      if(!printscalarheader(fh,node))return 0;
      (void)fprintf(fh,";");
    }else if(placement==VTYPE_INPLACE){  
      if(!printscalarheader(fh,node))return 0;
      (void)fprintf(fh,"=");
      if(!printscalarbody(fh,node))return 0;
      (void)fprintf(fh,";\n");
    }else if(placement==VTYPE_DATA){ 
      (void)fprintf(fh,"%s=",node->name);
      if(!printscalarbody(fh,node))return 0;
      (void)fprintf(fh,";\n");
    }else { 
      (void)sprintf(errorbuffer,"scalar %s=(undefined);\n",node->name);
      return 0; 
    } 
    
  }else  if(structuretype==VTYPE_STRUCTARRAY){
#ifdef UNDEFINED_ABORT
    if(placement==VTYPE_SEPERATE){ 
#else
    if(placement==VTYPE_SEPERATE||!placement){  
#endif
      if(!printstructheader(fh,node,indent))return 0;
      (void)fprintf(fh,";");
    }else if(placement==VTYPE_INPLACE){  
      if(!printstructheader(fh,node,indent))return 0;
      (void)fprintf(fh,"\n%*.*s={",indent,indent,"");
      if(!printstructarraybody(fh,node,indent+INDENT))return 0; 
      (void)fprintf(fh,"\n%*.*s};\n",indent,indent,"");
    }else if(placement==VTYPE_DATA){ 
      (void)fprintf(fh,"%s={",node->name);
      if(!printstructarraybody(fh,node,indent+INDENT))return 0; 
      (void)fprintf(fh,"\n%*.*s};\n\n",indent,indent,"");
    }else { 
      (void)sprintf(errorbuffer,"structarray %s=(undefined);\n",node->name);
      return 0; 
    } 
  }else  if(structuretype==VTYPE_STRUCT){
    if(placement==VTYPE_SEPERATE){ 
      if(!printstructheader(fh,node,indent))return 0;
      (void)fprintf(fh,";");
    }else if(placement==VTYPE_INPLACE){  
      if(!printstructheader(fh,node,indent))return 0;
      (void)fprintf(fh,"\n%*.*s={",indent,indent,"");
      if(!printstructbody(fh,STP(node),indent+INDENT,&iscomp))return 0;
      if(iscomp){
	(void)fprintf(fh,"\n%*.*s};\n",indent,indent,"");
      }else{
	(void)fprintf(fh,"};\n");
      }
    }else if(placement==VTYPE_DATA){ 
      (void)fprintf(fh,"%s={",node->name); 
      if(!printstructbody(fh,STP(node),indent+INDENT,&iscomp))return 0;
      if(iscomp){
	(void)fprintf(fh,"\n%*.*s};\n\n",indent,indent,"");
      }else{
	(void)fprintf(fh,"};\n\n");
      }

    }else { 
      (void)sprintf(errorbuffer,"struct %s=(undefined);\n",node->name);
      return 0; 

    } 
  }else  if(structuretype==VTYPE_ARRAY){
    
    
#ifdef UNDEFINED_ABORT      
    if(placement==VTYPE_SEPERATE){ 
#else
    if(placement==VTYPE_SEPERATE|| !placement){ 
#endif
      if(!printarrayheader(fh,node))return 0;
      (void)fprintf(fh,";");
#ifndef UNDEFINED_ABORT      
      (void)fprintf(fh,"\n");
#endif
    }else if(placement==VTYPE_DATA){ 
      (void)fprintf(fh,"%s={",node->name);
      if(!printarraybody(fh,node,indent))return 0;
      (void)fprintf(fh,"\n%*.*s};\n",indent,indent,"");
    }else if(placement==VTYPE_INPLACE){ 
      if(!printarrayheader(fh,node))return 0;
      (void)fprintf(fh,"={");
      if(!printarraybody(fh,node,indent))return 0;
      (void)fprintf(fh,"\n%*.*s};\n",indent,indent,"");
    }else {
      (void)sprintf(errorbuffer,"array %s=(undefined);\n",node->name);
      return 0; 
      
    }
    
    
  }else{
    goto error;
  }
  return 1;
 error:
  
  (void)sprintf(errorbuffer,"error(%s): unknown type 0x%lx!\n",subfunc,structuretype);
  return 0;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long scalartypeindex(long *sctyind,long scalartype,long *string,long acceptstring)
{
  char *subfunc="scalartypeindex";   
  long i;


  *sctyind=1;
  i=scalartype;
  *string=0;

  if(i==(VTYPE_CHAR|VTYPE_STRING|VTYPE_FLAG)){
    *sctyind=7;
    return 1;
  }


  while( ! (i & 0x1) ) {(*sctyind)++; i=i>>1;}
  
  if(*sctyind<1 || *sctyind> 6  ){
    (void)sprintf(errorbuffer,"error(%s): unknown type 0x%lx!\n"
	    ,subfunc,scalartype);
    return 0;
  }
  if(*sctyind==6){
    if(!acceptstring) *sctyind=5;
    *string=1;
  }

  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

long structuretypeindex(long *sctyind,long structuretype)
{
  char *subfunc="structuretypeindex";   
 
  *sctyind=1;
  
  if(structuretype==(VTYPE_SCALAR)){
    *sctyind=1;
  }else   if(structuretype==(VTYPE_ARRAY)){
    *sctyind=2;
  }else   if(structuretype==(VTYPE_STRUCT)){
    *sctyind=3;
  }else   if(structuretype==(VTYPE_STRUCTARRAY)){
    *sctyind=4;
  }else   if(structuretype==(VTYPE_SECTION)){
    *sctyind=5;
  }else   if(structuretype==(VTYPE_SECTIONEND)){
    *sctyind=6;
  }else   if(structuretype==(VTYPE_PLACEMENT)){
    *sctyind=7;
  }else   if(structuretype==(VTYPE_COMMENT)){
    *sctyind=8;
  }else if(structuretype==(VTYPE_ARRAY|VTYPE_SCALAR)){
    *sctyind=9;
  }else if(structuretype==(VTYPE_ARRAY|VTYPE_SCALAR|VTYPE_STRUCTARRAY)){
    *sctyind=10;
  }else if(structuretype==(VTYPE_ARRAY|VTYPE_SCALAR|VTYPE_STRUCTARRAY|VTYPE_STRUCT)){
    *sctyind=11;
  }else if(structuretype==(VTYPE_ARRAY|VTYPE_STRUCTARRAY)){
    *sctyind=12;
  }else{  
    (void)sprintf(errorbuffer,"error(%s): unknown type 0x%lx!\n"
	    ,subfunc,structuretype);
    return 0;
  }
  return 1;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long printstructarraybody(FILE *fh,VARTABLE *node,int indent)
{
  char *subfunc="printstructarraybody";   
  tsSTRUCTARRAY *sap;
  long i,iscomp;
  
  if(!myassert(node,subfunc,"node")) return 0;
  sap=&node->data.structarray;

  
  if(sap->array==NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): structarray %s: structarray->array is NULL!"
	    ,subfunc,node->name);
    return 0;
    
  }


 
  if(sap->Size>0) (void)fprintf(fh,"\n%*.*s{",indent,indent,"");
  for(i=0;i<sap->Size;i++){
    
    if(!printstructbody(fh,sap->array[i],indent+INDENT,&iscomp))
      return 0;
    
    if(!iscomp){
      if(i<sap->Size-1){
	(void)fprintf(fh,"}\n");
      }else{
	(void)fprintf(fh,"}");
      }
      (void)fprintf(fh,"%*.*s",indent,indent,""); 
    }else{
      (void)fprintf(fh,"\n%*.*s}",indent,indent,"");
    }
    
    if(i<sap->Size-1){
      (void)fprintf(fh,",{");
    }else{
      ;
    }
  }

  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long printscalarheader(FILE *fh,VARTABLE *node)
{
  char *subfunc="printscalarheader";    
  long sctyind,string;
  subfunc=subfunc;
  if(!scalartypeindex(&sctyind,node->data.scalar.scalartype,&string,0L))return 0;
  
  
  (void)fprintf(fh,"%s",scalartypes[sctyind]);
  if(string){
    (void)fprintf(fh,"[");
    if(node->data.scalar.strvariable){
      (void)fprintf(fh,"*");
    }else{
      (void)fprintf(fh,"%ld",node->data.scalar.strlength);
    }
    (void)fprintf(fh,"]");
  }  
  (void)fprintf(fh," %s",node->name);      
  
  
  return 1;
}
