/* TDOD: check resize/default  of nested structarrays */
/* @(#)main.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "heapcheck.h" 
#include "tparser.h" 
#include "merror.h"
#include "table.h"
#include "ioparser.h"
#include "tools.h"
#include "fortrantypes.h"
#include "allocate.h"




/* ***********************************************************************
 * defines
 * ***********************************************************************
 */
#define MODE_VAR_EXISTS        1
#define MODE_GET_RANK_AND_SIZE 2
#define MODE_GET_SIZE_ARRAY    3
#define MODE_GET_VALUE         4
#define MODE_CHECK             5
#define MODE_PRINT             6
#define MODE_SET_SCALAR       20
#define MODE_REMOVE           30
#define FORMAT_PLAIN           1
#define FORMAT_PERL            2
#define FORMAT_SHELL           3
/* ***********************************************************************
 * global vars
 * ***********************************************************************
 */
long mode=MODE_CHECK;
char *name="";
char *fname=NULL;
char *val=NULL;
long format;
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void usage(char *name);
void usage(char *name)
{
  (void)fprintf(stdout,"\
Extract data from input files of the FPLO-package\n\
(C) Klaus Koepernik March 2003\n\n\
usage:  %s -mode MODE [EXIST|RANK|SIZEARRAY|VALUE|SET_SCALAR] \n\
           -var variable_name -f file_name -format [plain|perl|shell] \n\
           [ -value value ]\n\
\n\
%s extracts information about the variables coded in the FPLO input files.\n\
The information may be used in shell scripts to process information, which\n"
      ,name,name);
  (void)fprintf(stdout,"\
otherwise is hard to abtain (due to the syntax of the input files).\n\
Additionally, the program may be used to set the value of scalar variable.\n\
Normally, FEDIT is used to manipulate the values in the input files.\n\
However, there is some information, which is not accessible in this way.\n\
These are especially the version numbers, which should only be modified with \n\
great care. \n");
  (void)fprintf(stdout,"\
FEDIT will automatically upgrade versions, so you will need this program\n\
for downgrading only.\n\
\n\
Options:\n\
 -mode : takes one of the following arguments\n\
         PRINT: check syntax and print the content\n\
         EXIST: check existance of variable (return value is 1 if variable exist,\n\
					     0 otherwise)\n\
         RANK:  print rank and dimensions of a data structure which may be interpreted \n");
  (void)fprintf(stdout,"\
                as an array\n\
         SIZEARRAY: print array containing the actual dimensions of the data object\n\
                    (there are objects of variable dimension)\n\
         VALUE:   print values of the object in form of a one dimensional array,\n\
                  you will need the SIZEARRAY information to reconstruct the data\n\
                  structure from this.\n\
         SET_SCALAR: set the value of a scalar data object. \n\
         REMOVE: remove a data component\n");
  (void)fprintf(stdout,"\
\n\
 -var : specifies the name of the data object.\n\
        i.e. \"basis.valencestates.state\" is a two-dimensional array with\n\
        variable dimensions.\n\
\n\
 -f   : name of the input file\n\
\n\
 -format : format of output\n\
           possible values:  plain (default), perl, shell\n");
  (void)fprintf(stdout,"\
\n\
 -value: (optional) specify the value for the \"-mode SET_VALUE\"\n\
         The content of the changed input file is written to standard output.\n\
         The file is not changed. You need to use shell commands to save the \n\
         changed file.\n");
  exit(0);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void scanopts(int argc,char **argv);
void scanopts(int argc,char **argv)
{
  long i=1;
  mode=-1;
  name=NULL;
  fname=NULL;
  format=FORMAT_PLAIN;
  while(i<argc){
    if( strcmp(argv[i],"-h")==0){
      usage(argv[0]);
    }else if( strcmp(argv[i],"-format")==0){
      i++;
      if((i>=argc) || ( *(argv[i])=='-')){
	(void)fprintf(stderr,"Option -format needs one argument!\n"); 
	exit(0);
      }

      if(strcmp(argv[i],"plain")==0){
	format=FORMAT_PLAIN;
      }else if(strcmp(argv[i],"perl")==0){
	format=FORMAT_PERL;
      }else if(strcmp(argv[i],"shell")==0){
	format=FORMAT_SHELL;
      }else{
	(void)fprintf(stderr,"Unknown format in option -format!\n"); 
	exit(0);
      }

    }else if( strcmp(argv[i],"-var")==0){
      i++;
      if((i>=argc) || ( *(argv[i])=='-')){
	(void)fprintf(stderr,"Option -var needs one argument!\n"); 
	exit(0);
      }
      name=argv[i];

    }else if( strcmp(argv[i],"-value")==0){
      i++;
      if((i>=argc) ){
	(void)fprintf(stderr,"Option -value needs one argument!\n"); 
	exit(0);
      }
      val=argv[i];
      /* fprintf(stderr,"val=[%s]\n",val); */

    }else if( strcmp(argv[i],"-f")==0){
      i++;
      if((i>=argc) || ( *(argv[i])=='-')){
	(void)fprintf(stderr,"Option -f needs one argument!\n"); 
	exit(0);
      }
      fname=argv[i];

    }else if( strcmp(argv[i],"-mode")==0){
      i++;
      if((i>=argc) || ( *(argv[i])=='-')){
	(void)fprintf(stderr,"Option -mode needs one argument!\n"); 
	exit(0);
      }
      if(strcmp(argv[i],"EXIST")==0){
	mode=MODE_VAR_EXISTS;
      }else if(strcmp(argv[i],"RANK")==0){
	mode=MODE_GET_RANK_AND_SIZE;
      }else if(strcmp(argv[i],"PRINT")==0){
	mode=MODE_PRINT;
      }else if(strcmp(argv[i],"CHECK")==0){
	mode=MODE_CHECK;
      }else if(strcmp(argv[i],"SIZEARRAY")==0){
	mode=MODE_GET_SIZE_ARRAY;
      }else if(strcmp(argv[i],"VALUE")==0){
	mode=MODE_GET_VALUE;
      }else if(strcmp(argv[i],"SET_SCALAR")==0){
	mode=MODE_SET_SCALAR;
      }else if(strcmp(argv[i],"REMOVE")==0){
	mode=MODE_REMOVE;
      }else{
	(void)fprintf(stderr,"Unknown mode in option -mode!\n"); 
	exit(0);
      }
    }else{/* END -mode*/
      (void)fprintf(stderr,"Unknown option \"%s\"!\n",argv[i]); 
      usage(argv[0]);      
    }
    i++;
  }/* END while */
  
  if(mode==-1){
    (void)fprintf(stderr,"option -mode must be specified!\n"); 
    exit(0);
  }
  if(name==NULL && mode!=MODE_CHECK && mode!=MODE_PRINT){
    (void)fprintf(stderr,"option -var must be specified!\n"); 
    exit(0);
  }
  if(fname==NULL){
    (void)fprintf(stderr,"option -f must be specified!\n"); 
    exit(0);
  }
  if(mode==MODE_SET_SCALAR &&  val==NULL){
    (void)fprintf(stderr,"option -value must be specified for -mode SET_VALUE!\n"); 
    exit(0);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main (int argc,char ** argv)
{

  /* FILE *FO; */

  VARTABLE *var; 
  long i,scalartyperet,result_len;
  char *result;
  PARSER_CONTROLL pc;
  FPLOINTEGER rank,size[7],dim,*sizearray,asize[7];
  tsNAMSCAN namescan,*nx;
  long prot=0;

  namescan.next=NULL;
  
  scanopts(argc,argv);


#ifdef EXTENDED_HEAP_TEST
  INITHEAP();
#endif

  (void)init_parser_control(&pc,0,1);


  

  
  if(!parsefile(&yyin,fname,&pc)){
    (void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
    errorbuffer[0]='\0';
    exit(0);
  }else{    

    if(mode==MODE_VAR_EXISTS){
      if(!getNameScan(&pc.table,name
		  ,(long) (VTYPE_ARRAY | VTYPE_SCALAR 
			   | VTYPE_STRUCT | VTYPE_STRUCTARRAY)
		  ,0L,&namescan,0L)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	exit(0);
      }else{
	/* (void)fprintf(stdout,"var %s exists\n",name); */
      }
    }else if(mode==MODE_GET_RANK_AND_SIZE){
      if(!getDimensions(&pc.table,name,&rank,size)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	errorbuffer[0]='\0';
	exit(0);
      }else{
	
	if(format==FORMAT_PERL){
	  (void)fprintf(stdout,"$rank="FPLOINTEGERFORMAT";\n",rank);
	  (void)fprintf(stdout,"@size=(");
	  for(i=0;i<rank;i++){
	    (void)fprintf(stdout," "FPLOINTEGERFORMAT"",size[i]);
	    if(i<rank-1)
	      (void)fprintf(stdout,",");
	  }
	  (void)fprintf(stdout,");\n");
	}else if(format==FORMAT_SHELL){
	  (void)fprintf(stdout,"rank="FPLOINTEGERFORMAT";\n",rank);
	  (void)fprintf(stdout,"size=(");
	  for(i=0;i<rank;i++){
	    (void)fprintf(stdout," "FPLOINTEGERFORMAT"",size[i]);
	    if(i<rank-1)
	      (void)fprintf(stdout," ");
	  }
	  (void)fprintf(stdout,") \n");
	}else {
	  (void)fprintf(stdout,"rank: "FPLOINTEGERFORMAT"\n",rank);
	  (void)fprintf(stdout,"size:");
	  for(i=0;i<rank;i++){
	    (void)fprintf(stdout," "FPLOINTEGERFORMAT"",size[i]);
	  }
	  (void)fprintf(stdout,"\n");
	}


      }
    }else if(mode==MODE_GET_SIZE_ARRAY){
      if(!getDimensions(&pc.table,name,&rank,size)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	errorbuffer[0]='\0';
	exit(0);
      }else{

	if(prot){
	  (void)fprintf(stdout,"rank: "FPLOINTEGERFORMAT"\n",rank); 
	  (void)fprintf(stdout,"size:"); 
	}
	dim=1;
	for(i=1;i<rank;i++){
	  if(prot)(void)fprintf(stdout," "FPLOINTEGERFORMAT"",size[i]); 
	  dim=dim*size[i];
	  asize[i-1]=size[i];
	}
	if(prot){
	  (void)fprintf(stdout,"\n"); 
	  (void)fprintf(stdout,"dim="FPLOINTEGERFORMAT"\n",dim); 
	}
	
	

	sizearray=(FPLOINTEGER*)allocate(sizeof(FPLOINTEGER)*dim,"sizearray","main"); 

	for(i=0;i<dim;i++) sizearray[i]=0;
	

	if(!getSizeArray(&pc.table,name,rank-1,asize,sizearray)){
	  (void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	  errorbuffer[0]='\0';
	  exit(0);

	}

	if(format==FORMAT_PERL){
	  fprintf(stdout,"@sizearray=(");
	  for(i=0;i<dim;i++){
	    fprintf(stdout,""FPLOINTEGERFORMAT"",sizearray[i]);
	    if(i<dim-1)fprintf(stdout,",");
	  }
	  fprintf(stdout,");\n");
	}else if(format==FORMAT_SHELL){
	  fprintf(stdout,"sizearray=(");
	  for(i=0;i<dim;i++){
	    fprintf(stdout," "FPLOINTEGERFORMAT"",sizearray[i]);
	  }
	  fprintf(stdout,")\n");
	}else{
	  fprintf(stdout,"sizearray:");
	  for(i=0;i<dim;i++){
	    fprintf(stdout," "FPLOINTEGERFORMAT"",sizearray[i]);
	  }
	  fprintf(stdout,"\n");
	}
	FREEE(sizearray,"sizearray");
      }      
    }else if(mode==MODE_GET_VALUE){
      if(!getNameScan(&pc.table,name
		  ,(long) (VTYPE_ARRAY | VTYPE_SCALAR )
		  ,0L,&namescan,0L)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	exit(0);
      }
      /* (void)fprintf(stdout,"datatype: %ld\n",datatype); */


      nx=&namescan;
      
      scalartyperet=0;
      while(nx){
	if(nx->var->structuretype &( VTYPE_SCALAR )){
	  scalartyperet=SCP(nx->var)->scalartype;
	}else if(nx->var->structuretype &( VTYPE_ARRAY )){
	  scalartyperet=ARP(nx->var)->scalartype;
	}

	nx=nx->next;
      }

      /* (void)fprintf(stdout,"type: %ld\n",scalartyperet);  */

      /* extract values */
      result_len=501;
      rank=1;
      size[0]=1;
      dim=1;

      if(!getDimensions(&pc.table,name,&rank,size)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	errorbuffer[0]='\0';
	exit(0);
      }
      /* (void)fprintf(stdout,"rank: "FPLOINTEGERFORMAT"\n",rank); */
      /* (void)fprintf(stdout,"size:"); */
      dim=1;
      for(i=0;i<rank;i++){
	/* (void)fprintf(stdout," "FPLOINTEGERFORMAT"",size[i]); */
	dim=dim*size[i];
      }
      /* (void)fprintf(stdout,"\n"); */
      /* (void)fprintf(stdout,"dim="FPLOINTEGERFORMAT"\n",dim); */
      

      


      result=(char*)allocate(sizeof(char)*dim*result_len,"sizearray","main"); 	
      if(!getValue(&pc.table,name,result,scalartyperet,(long)result_len
		   ,rank,size,1L)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	errorbuffer[0]='\0';
	exit(0);
	
      }

      if(format==FORMAT_PERL){
	fprintf(stdout,"@value=(");
	for(i=0;i<dim;i++){
	  result[i*result_len+result_len-1]='\0';
	  removetrailingspace(result+i*result_len);
	  fprintf(stdout,"\"%s\"",result+i*result_len);
	  if(i<dim-1) fprintf(stdout,",");
	}
	fprintf(stdout,");\n");      
      
      }else if(format==FORMAT_SHELL){
	fprintf(stdout,"value=(");
	for(i=0;i<dim;i++){
	  result[i*result_len+result_len-1]='\0';
	  removetrailingspace(result+i*result_len);
	  fprintf(stdout," \'%s\'",result+i*result_len);
	}
	fprintf(stdout,")\n");      
      
      }else{
	fprintf(stdout,"value:");
	for(i=0;i<dim;i++){
	  result[i*result_len+result_len-1]='\0';
	  removetrailingspace(result+i*result_len);
	  fprintf(stdout," \"%s\"",result+i*result_len);
	}
	fprintf(stdout,"\n");
      }

      FREEE(result,"result");
      
    }else if(mode==MODE_REMOVE){
      if(!removeVariable(&pc.table,name)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	exit(0);
      }
    }



    

    if(mode==MODE_SET_SCALAR){
      /* set scalar variable */
      if(!getNameScan(&pc.table,name
		      ,(long) ( VTYPE_SCALAR )
		      ,0L,&namescan,0L)){
	fprintf(stderr,"ERROR: variable '%s': %s\n",name,errorbuffer);
	exit(0);
      }else{
	nx=&namescan;
	var=nx->var;
	while(nx){
	  if(nx->var->structuretype &( VTYPE_STRUCTARRAY | VTYPE_ARRAY )){
	    fprintf(stderr,"ERROR: variable '%s' is not a simple"
		    " scalar value!\n",name);
	    exit(0);
	  }
	  var=nx->var;
	  nx=nx->next;
	}
	if(!write_into_scalar(var,val,WRITE_COPY)){ 
	  (void)fprintf(stderr,"ERROR: internal(%s):  variable "
			"'%s':\n %s\n","main",var->name
			,errorbuffer);  
	  exit(0);
	}  
      }
    }
    
    /* output stuff */
    if(mode==MODE_SET_SCALAR || mode==MODE_CHECK || mode==MODE_PRINT  || 
       mode==MODE_REMOVE){
      if(!print_vartab(stdout,&pc.table)){
	(void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
	exit(0);
	
      }
    }
  }  
  
  
  
  if(!clean_vartable(&pc.table)){
    (void)fprintf(stderr,"ERROR: %s\n",errorbuffer);
    exit(0);
  }
  
  
  finalcleanup();
  
#ifdef EXTENDED_HEAP_TEST
  TESTHEAP();
#endif  
  
  
  
  
  cleanup_namescan(&namescan);

  return 1;
}

