/*************************************************************************
 * @(#)Calculations.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   02 Dec 2011
 *************************************************************************
 */

#include <stdio.h>
#include <unistd.h>
//#include <dirent.h> //todo
#include <glob.h>
#include "Calculations.hh"
#include "Assorted.hh"
#include "Except.hh"
#include "OutGrep.hh"
#include "Variables.hh"
#include "MethodControl.hh"
#include "Config.hh"
#include "MyMath.hh"

const Calculation::tsStatusInfo Calculation::pstatusinfo[]={
  {New,"New"},
  {Created,"Created"},
  {Running,"Running"},
  {Ready,"Ready"},
  {Stoped,"Stoped"},
  {NotUsed,"Not used"}
};

const long Calculation::pmaxstatusinfo=sizeof(Calculation::pstatusinfo)/
  sizeof(Calculation::tsStatusInfo);



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Calculations::registerCalculation(long norder,long idsc)
{
  const Variables &vars=pcontrol->variables();
  long D=vars.size();
  long nsc=false;
  const char *marker="REGISTER_CALC:";


  if(idsc>=0) nsc=true;
  
  

  if(nsc) printf("%s id of sc calc=%ld\n",marker,idsc);
  
  

  printf("%s register at: dim=%ld %s\n",marker,D,vars.stringify()());


  // use current pcontrol->variables() values and evaluate all formulas
  // from pcontrol->crunchContent() --> pvartable
  setupCurrentVarTable();

  if(vlevel>=Vlevel::All){
    //pvartable.print(String(marker)+" initial vartable:");
  }
  
  
  
  // find the id of an existing calculation definition, with
  // exactly the current values
  long id=valuesAlreadyDefined();



  String from_dir;
  if(nsc){
    from_dir=getById(idsc)->dir();
  }else{
    from_dir=getClosestCalcDir();
  }
  
  
  Calculation *c=0;
  if( id<0 ){
    id=getFreshId();
    String dir=constructDirName(id);
    if(nsc){
      pcalculations.append(new Calculation(id,norder,vars.currentValues(),dir,
					   Calculation::New,false,true));
    }else{
      pcalculations.append(new Calculation(id,norder,vars.currentValues(),dir,
				       Calculation::New,true,true));
    }
    c=pcalculations.last();
  }else{
    c=getById(id);
    c->values()=vars.currentValues();
    c->setSc(!nsc);
    c->setOrder(norder);
    c->setUsed(true);
    if(c->status()==Calculation::NotUsed){
      c->setStatus(Calculation::Created);
    }
    //c->setStatus(Calculation::New);
  }

  //todo: before deleting old dirs, the copy_from must be fulfilled

  writeStatusToFile();

  

  createFploInput(c,from_dir);
    
  if(nsc){
    throw Except("todo set_fplo_input_singlestep\n");
    //set_fplo_input_singlestep($control,$dir);
  }
  writeStatusToFile();


  return id;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::run()
{
  const Variables &vars=pcontrol->variables();
  
  if(vlevel>=Vlevel::Many)  printf("RUN: start\n");

  
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    if(c->status() == Calculation::Created ){

      if(c->status()==Calculation::Created){
	printf("RUN: start x=%s: in '%s', status=\"%s\"\n",
	       vars.stringify(c->values())(),
	       c->dir()(),
	       Calculation::statusGuiName(c->status())
	       );
      }
      
      
      if(pcontrol->dontRun()){
	printf("stoping before any run\n");
	exit(0);
      }
      pcontrol->jobInterface().submitFplo(c->dir());
      c->setStatus(Calculation::Running);

    }
  }
  
  writeStatusToFile();
  
  // now wait until all calcs finished
  while(true){
    
    bool ready=true;
    if(vlevel>=Vlevel::All){
      printf("RUN: \n");
    }

    for(long i=0;i<pcalculations.size();i++){
      Calculation *c=pcalculations[i];
      if(!c->used() )continue;

      String flagfile=Sprintf("%s/+done",c->dir()());
      FileExistence ex=pathExistence(flagfile);
      //printf("ex=%d\n",ex);
      if( ex==RegularFile ){
	c->setStatus(Calculation::Ready);
	removeFile(flagfile);
      }else if(c->status()!=Calculation::Ready && 
	       c->status()!=Calculation::Stoped){
	ready=false;
      }
      //printf("stat=%s\n",Calculation::statusGuiName(c->status()));
      
      
      c->setEtot(0.);
      c->setIterationProgress(1e33);
      c->setTermination("");
      OutGrep poutgreper;
      try{
	poutgreper.setOutfile(Sprintf("%s/out",c->dir()()));
	if(c->status()==Calculation::Ready){
	  String s;
	  if(!poutgreper.grepLast(OutGrep::Termination,s)){
	  }
	  c->setTermination(s);
	  
	  for(long i=0;i<pcontrol->report().size();i++){
	    const String &ps=pcontrol->report()[i];
	    String s;
	    OutGrep::Mode mode=OutGrep::modeFromString(ps());
	    if(mode==OutGrep::AtomSpinGros||
	       mode==OutGrep::AtomSpinNet ||
	       mode==OutGrep::AtomOccuGros ||
	       mode==OutGrep::AtomOccuNet 
	       ){
	      StringList sl=ps.split(" ");
	      poutgreper.setSite(-1);
	      poutgreper.setOrbital(-1);
	      if(sl.size()>=1) 
		try{ poutgreper.setSite(sl[1].convertToLong()); }catch(...){}
	      if(sl.size()>=2) 
		try{ poutgreper.setOrbital(sl[2].convertToLong()); }catch(...){}
	      
	    }
	    if(!poutgreper.grepLast(mode,s)){
	      s="undef";
	    }
	    //myDebug()<<ps<<mode<<s;
	    c->setResult(i,s);
	  }

	}
	{
	  double e;
	  if(!poutgreper.grepLast(OutGrep::TotalEnergy,e)){
	    //todo: react on missing total energy
	  }
	  c->setEtot(e);
	  if(!poutgreper.grepLast(OutGrep::Iteration,e)){
	  }
	  c->setIterationProgress(e);
	}
      }catch(Except &ex){
	printf("RUN: checking output: %s\n",ex.msg());
      }
      if(0 && vlevel>=Vlevel::All){
	printf("RUN: x=%s: in '%s', status=\"%s\" y=%17.8f iter=%g\n",
	       vars.stringify(c->values())(),
	       c->dir()(),
 	       Calculation::statusGuiName(c->status()),
	       c->etot(),c->iterationProgress()
	     );

      }

    }
    
    reportResults();
    

    writeStatusToFile();
 
    if(ready) {
      for(long i=0;i<pcalculations.size();i++){
	Calculation *c=pcalculations[i];
	printf("RUN: x=%s: in '%s', status=\"%s\" y=%17.8f iter=%g\n",
	       vars.stringify(c->values())(),
	       c->dir()(),
	       Calculation::statusGuiName(c->status()),
	       c->etot(),c->iterationProgress()
	       );
      }
      break;
    }
    
    respondToQuitRequest();
    
    sleep(pcontrol->sleepTime());
  }

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Calculations::stopRunningJobs()
{
  const Variables &vars=pcontrol->variables();

  bool somestoped=false;
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    if(c->status()==Calculation::Running){
      printf("STOP: i=%ld id=%ld %s, status=%s\n",i,c->ID(),
	     vars.stringify(c->values())(),
	     Calculation::statusGuiName(c->status()));
      pcontrol->jobInterface().killFplo(c->dir());
      c->setStatus(Calculation::Stoped);
      somestoped=true;
    }else if(c->status()!=Calculation::NotUsed && 
	     c->status()!=Calculation::Ready){
      c->setStatus(Calculation::Stoped);
    }
  }  
  writeStatusToFile();
  return true;
  return somestoped;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::forceFileTransfer()
{
  const Variables &vars=pcontrol->variables();
  
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    if(c->status()==Calculation::Running){
      printf("transfering files: i=%ld id=%ld %s, status=%s\n",i,c->ID(),
	     vars.stringify(c->values())(),
	     Calculation::statusGuiName(c->status()));
      pcontrol->jobInterface().transferFiles(c->dir());
    }  
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::attemptRefresh(bool registernew)
{
  Variables &vars=pcontrol->variables();
  bool possible=true;
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    if(c->status()==Calculation::Running){
      printf("STILL RUNNING: i=%ld id=%ld %s, status=%s\n",i,c->ID(),
	     vars.stringify(c->values())(),
	     Calculation::statusGuiName(c->status()));
      possible=false;
    }   
  }
  if(!possible)
    throw Except("There are running calculations.\n"
		 "Input refresh is not possible before you stopped\n "
		 "all calulations\n");


  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    c->setStatus(Calculation::New);
  }

  writeStatusToFile();

  if(registernew){
    for(long i=0;i<pcalculations.size();i++){
      Calculation *c=pcalculations[i];
      vars.setCurrentValue(c->values());//.at(0));
      registerCalculation(-1);
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::restartJobs(bool onlyunfinished)
{
  const Variables &vars=pcontrol->variables();

  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    if( c->status()!=Calculation::Running && 
	(c->status()==Calculation::Stoped || !onlyunfinished)  ){
      printf("SET RESTART: i=%ld id=%ld %s, status=%s\n",i,c->ID(),
	     vars.stringify(c->values())(),
	     Calculation::statusGuiName(c->status()));
      //pcontrol->jobInterface().killFplo(c->dir());
      c->setStatus(Calculation::Created);
    }
  }  
  writeStatusToFile();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::sortCalculationOrder()
{
  for(long i=0;i<pcalculations.size();i++){
    for(long j=i+1;j<pcalculations.size();j++){
      if(pcalculations[i]->order()>pcalculations[j]->order()){
	Calculation *c=pcalculations[i];
	pcalculations[i]=pcalculations[j];
	pcalculations[j]=c;
      }
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::initCheckCalculations()
{
/*
  const Variables &vars=pcontrol->variables();
  //todo: my $redo=shift;
  

  //todo: when new values added, does it create correct new ids?  


  my @rem=();

  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=&(pcalculations[i]);

    String dir=c->dir();
    printf("Checking old calculation at x=%s\n",
	   vars.stringify(c->values())());
    // if($redo){
//       if( ! -d $dir){
// 	printf("\tCannot find directory $dir, do you have an old =.status_min\n");
// 	printf("\tremove cal form index list?\n");
// 	<STDIN>;
// 	push @rem,$dir;
// 	#die("\tCannot find directory $dir\n");
//       }else{
// 	$c->{status}="new";
// 	setup_var_table($control,$x);
// 	&create_fplo_input($control,$dir,"");
//       }
//       next;
//     }

    if( ! -d $dir){
      printf("\tCannot find directory $dir\nremove cal form index list?\n");
      <STDIN>;
      push @rem,$dir;
      #die("\tCannot find directory $dir\n");
      #$c->{status}="new";
      #$control->{table}={$control->{vars}->[0]=>$x};
      #&create_fplo_input($control,$dir,"");
    }else{

      if( $c->{status} eq "new" ){
	printf("\tCalculation in directory $dir seems to be unfinished\n".
	       "\tBetter stop here? [<ctrl-C> to interupt, <enter> to continue]\n");
	<STDIN>;
      }
      my $e=`grEE $dir out`;
      $e=~s/^\s+//g;
      $e=~s/\s+$//g;
      my @e=split(/\s+/,$e);
      $c->{erg}=$e[0];
    }
  }
  
  if($#rem>=0){
    foreach my $dir (@rem){
      for(my $i=0;$i<=$#{$control->{calculations}};$i++){
	my $c=$control->{calculations}->[$i];
	if($dir eq $c->{dir}){
	  printf("remove %s\n",$c->{dir});
	  splice(@{$control->{calculations}},$i,1);
	  last;
	}
      }
    }
  }

  foreach my $c (@{sorted_calcs($control)}){
    printf("calculation at x=%s erg=%s: %s\n",stringify_vector($c->{x})
	   ,$c->{status},$c->{erg});
  }

  &run($control);
*/
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Vartable::replaceVariablesIn(const String &s)
{
  //myDebug()<<"Vartable::replaceVariablesIn"<<s;
  String text=s;
  VarNodeIterator it(pvtable);
  while(VarNode *v=it.next()){
    String t=text;
    text="";
    
    const String &key="$"+v->name();
    long lkey=key.length();
    String value=Sprintf("%s",v->value().printS("%30.15g")()).stripBlanks();
//     myDebug()<<"key="<<key;
//     myDebug()<<"value="<<value;
    //myDebug()<<"text="<<t;
    
    long ind=0;
    while(ind>=0){
      ind=t.find(key);
      if(ind>=0){
	//myDebug()<<"ind="<<ind;
	if((ind+lkey==t.length()) || 
	   (ind+lkey<t.length() && 
	    !isalnum(t[ind+lkey]) &&  t[ind+lkey]!='_'
	    )){
	  //myDebug()<<"replace="<<t.subString(0,ind-1);
	  text+=t.subString(0,ind-1);
	  text+=value;
	  t=t.subString(ind+lkey);
	}else{
	  text+=t.subString(0,ind-1);
	  text+=key;
	  t=t.subString(ind+lkey);
	}
      }else{
	text+=t;
      }
      //myDebug()<<"text="<<text;
      //myDebug()<<"rest="<<t;

    }
  }
  return text;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::readStatusFromFile()
{
  const Variables &vars=pcontrol->variables();
  
  FileExistence ex=pathExistence(pstatusfilename);
  if(ex!=RegularFile) return;
  
  FILE *FH;
  FH=openforread(pstatusfilename);

  pcalculations.clear();
  String line;
  bool incalc=false;
  while(getLine(FH,line)){
    line.stripBlanks(true);
    if(incalc){
      String s;
      long id;
      if(MethodControl::getTag(line,"endcalculation")){
	incalc=false;
	continue;
      }else if(MethodControl::getFromKey(line,"id",id,'=')){
	pcalculations.last()->setID(id);
	continue;
      }else if(MethodControl::getFromKey(line,"sc",id,'=')){
	pcalculations.last()->setSc(id);
	continue;
      }else if(MethodControl::getFromKey(line,"status",s,'=')){
	s.stripBlanks();
	if(s[0]=='"')s=s.subString(1);
	if(s[s.length()-1]=='"')s=s.subString(0,s.length()-2);
	pcalculations.last()->setStatusFromString(s);
	continue;
      }else if(MethodControl::getFromKey(line,"dir",s,'=')){
	s.stripBlanks();
	if(s[0]=='"')s=s.subString(1);
	if(s[s.length()-1]=='"')s=s.subString(0,s.length()-2);
	// make sure the new id format is fulfilled
	long j0=s.find("=");
	long j1=s.find("_");
	String snum="00000"+s.subString(j0+1,j1-1).stripBlanks();
	snum=snum.subString(snum.length()-5,snum.length()-1);
	String snew=s.subString(0,j0)+snum+s.subString(j1);
	if(s!=snew){
	  FileExistence ex=pathExistence(s);
	  if( ex==Directory ){
	    systemCall(Sprintf("mv %s %s\n",s(),snew())());
	  }else{
	  }
	}
	pcalculations.last()->setDir(snew);
	continue;
      }else if(MethodControl::getFromKey(line,"x",s,'=')){
	s.replace("["," ",0,true).replace("]"," ",0,true)
	  .replace(","," ",0,true).stripBlanks();
	StringList sl=s.split(" \t");
	ValueArray<double> &va=pcalculations.last()->values();
	for(long i=0;i<vars.size();i++){
	  va.resize(i+1);
	  try{
	    va[i]=sl[i].convertToFortranReal();
	  }catch(Except &ex){
	    throw Except(Sprintf("file '%s': x-value: %s\n",pstatusfilename(),
				 ex.msg())());
	  }
	}
	
	continue;
      }
      
    }else{
      if(MethodControl::getTag(line,"calculation")){
	incalc=true;
	pcalculations.resize(pcalculations.size()+1);
	continue;
      }
    }
    printf("file '%s': unscanned input: %s\n",line(),pstatusfilename());
  }
  if(incalc)
    throw Except(Sprintf("file '%s': missing \"endcalculation:\"\n",
			 pstatusfilename())()); 

  
  fclose(FH);




}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::analyseDirectories()
{
  // analyse the directories and find unused ones and their values
  const Variables &vars=pcontrol->variables();
  String dir=constructDirName(0,true);
  double tol=1.0e-12;
  
  
  StringList sl=dir.split("*");
  
  
  glob_t globbuf;
  glob(dir(),0,NULL,&globbuf);
  for(unsigned int i=0;i<globbuf.gl_pathc;i++){
    String d=globbuf.gl_pathv[i];
    if( d == ".." ) continue;

    
    ValueArray<double> va;
    long id;
    String  x=d;
    long i2,i1,i0=0;
    for(long j=-1;++j<sl.size();){
      i1=d.find(sl[j],i0);;
      if(j<sl.size()-1){
	i2=d.find(sl[j+1],i1);
      }else{
	i2=d.length()-1;
      }
      String n=d.subString(i1+sl[j].length(),i2-1);
      if(j==0){
	id=n.convertToLong();
      }else{
	for(long i=0;i<vars.size();i++){
	  va.resize(i+1);
	  try{
	    va[i]=n.convertToFortranReal();
	  }catch(Except &ex){
	    throw Except(Sprintf("in existing directory name '%s': value: %s\n",d(),
				 ex.msg())());
	  }
	}
      }
    }

    Calculation *c0=0;
    for(long i=0;i<pcalculations.size();i++){
      Calculation *cc=pcalculations[i];
      if(norm(va-cc->values())<tol){ 
	//myDebug()<<id<<"already there"<<cc->ID()<<va;
	c0=cc;
      }
    }    
    if(c0){
      c0->setID(id);
    }else{
      pcalculations.resize(pcalculations.size()+1);    
      Calculation *c=pcalculations.last();
      c->setID(id);
      c->setStatus(Calculation::NotUsed);
      c->setSc(true);
      c->setDir(d);
      c->setValues(va);
    }    
  }// scan directory
  
  // for(long i=0;i<pcalculations.size();i++){
  //   Calculation *cc=pcalculations[i];
  //   //if(norm(va-cc->values())<tol){ 
  //   //myDebug()<<cc->ID()<<cc->values();
  //     //}
  // }    
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::writeStatusToFile(bool tostdout)
{
//   my $what=shift;
//   my $tab="";



//   if($what eq "control"){
//     &write_hash_ref($fo,"",$control,$tab,0);
//   }elsif($what eq "calculations"){
//     &write_hash_ref($fo,"",$control->{calculations},$tab,0);
  FILE *FH;
  if(!tostdout){
    FH=openforwrite(pstatusfilename);
  }else{
    FH=stdout;
  }
  
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    c->writeToFile(FH);
  }

  if(!tostdout) fclose(FH);


}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::createFploInput(Calculation *c,const String &from_dir_in)
{
  
  const char*marker="CREATE_FPLO_INPUT:";
  String from_dir=from_dir_in;

  bool makeinput=c->status()==Calculation::New;

  FileExistence ex=pathExistence(c->dir());
  if( ex!=Directory ){
    printf("%s in %s: create directory\n",marker,c->dir()());
    createDirectory(c->dir(),false);
    makeinput=true;

    if(from_dir == c->dir()) from_dir="SC";
    if(from_dir.length()>0 ){
      printf("%s from_dir=%s\n",marker,from_dir());
      ex=pathExistence(from_dir);
      if( ex==Directory ){
	printf("%s in %s: copy from %s\n",marker,c->dir()(),from_dir());
	systemCall(Sprintf("cp %s/=.* %s",from_dir(),c->dir()()));
      }
    }
  }
  
  if(makeinput){
    int fdpwd=saveCurrentDir();
    chDir(c->dir());
    
    writePipeFile();
    
    pcontrol->jobInterface().runFeditPipe();
    
    
    executePerjob();
    
    restoreCurrentDir(fdpwd);
    c->setStatus(Calculation::Created);

  }
  writeStatusToFile();

 }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::executePerjob()
{
  const char *marker="CREATE_FPLO_INPUT:";
  printf("%s executing per-job\n",marker);


  String job;
  for(int i=0;i<pcontrol->perJobShellCommands().size();i++){
    String line=pcontrol->perJobShellCommands()[i];
    String s=pvartable.replaceVariablesIn(line);
    job+=s+"\n";
    
  }
  systemCall(job);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::writePipeFile()
{
  const char *marker="CREATE_FPLO_INPUT:";

  printf("%s writing =.pipe\n",marker);
  FILE *FO;
  FO=openforwrite("=.pipe");
  for(int i=0;i<pcontrol->pipeContent().size();i++){
    fprintf(FO,"%s\n",pvartable
	    .replaceVariablesIn(pcontrol->pipeContent()[i])());
    
  }
  fclose(FO);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::setupCurrentVarTable()
{
  const Variables &vars=pcontrol->variables();

  pvartable.clear();
  for(int i=0;i<vars.size();i++){
    pvartable.defineVariable(vars.at(i).name(),vars.currentValues().at(i));
  }
  // now load crunchContent and evaluate all formulas
  try{
    pvartable.parse(pcontrol->crunchContent());
  }catch(Except &ex){
    throw Except(Sprintf("crunch section: %s",ex.msg())());
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Calculations::valuesAlreadyDefined()
{
  // find an already registered calc and return its' id or -1
  const Variables &vars=pcontrol->variables();
  double tol=1.0e-12;
  
  for(long i=0;i<pcalculations.size();i++){
    const Calculation *c=pcalculations.at(i);
    if(norm(vars.currentValues()-c->values())<tol){ 
      return c->ID();
    }
  }
  return -1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Calculation *Calculations::getById(long id)
{
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(c->ID()==id) {
      return c;
    }
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Calculations::getClosestCalcDir()
{
  
  Calculation *c=getClosestCalc();

  if(!c) { 
    return "SC"; 
  }
  
  return c->dir();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Calculation *Calculations::getClosestCalc()
{
  const Variables &vars=pcontrol->variables();
  const char *marker="GETCLOSESTCALC:";
  
  
  if(vlevel>=Vlevel::All){
    printf("%s No of calcs: %ld: %s\n",marker,pcalculations.size(),
	   vars.stringify()());
  }
  if(pcalculations.size()<=0){
    // no calcs there, return 0
    return 0;
  }
  Calculation *c0=0;


  double dx;

  
  bool start=true;
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];

    FileExistence ex=pathExistence(c->dir());  
    if( ex!=Directory ) continue;


    double dxc=norm(vars.currentValues()-c->values());
    if(dxc<dx || start){
      dx=dxc;
      c0=c;
      if(start){
	if(vlevel>=Vlevel::All){
	  printf("%s c0[id:%ld] %s dx=%g \n",marker,
		 c0->ID(),vars.stringify(c0->values())(),
		 dx) ;
	}
      }
      start=false;
      if(vlevel>=Vlevel::All){
	printf("%s closer point: c[id:%ld] %s dx=%g \n",marker,
	       c0->ID(),vars.stringify(c0->values())(),dx);
      }
    }
  }
  return c0;
 }

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long  Calculations::getFreshId()
{
  
  long id=-1;
  for(long i=0;i<pcalculations.size();i++){
     Calculation *c=pcalculations[i];
    id=MyMath::max<long>(id,c->ID());
  }
  return id+1;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::reportResults()
{
  const Variables &vars=pcontrol->variables();
  
  FILE *FH;
  FILE *FHI;
  FH=openforwrite("results.dat");
  FHI=openforwrite("iteration.dat");
  fprintf(FH,"#");
  for(long i=0;i<vars.size();i++){
    fprintf(FH," %s ",vars.at(i).name()());
  }
  for(long i=0;i<pcontrol->report().size();i++){
    fprintf(FH," '%s'",pcontrol->report()[i]());
  }
  fprintf(FH," ID ");
  fprintf(FH,"\n");

  fprintf(FHI,"#");
  for(long i=0;i<vars.size();i++){
    fprintf(FHI," %s ",vars.at(i).name()());
  }
  fprintf(FHI," last dev  termination ID");
  fprintf(FHI,"\n");

  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(!c->used() )continue;
    //    if(c->status()!=Calculation::Ready) continue;
    fprintf(FH,"%s ",vars.stringify(c->values(),true)());
    for(long i=0;i<pcontrol->report().size();i++){
      fprintf(FH," %s",c->result(i)());
    }
    fprintf(FH," %5ld",c->ID());
    fprintf(FH,"\n");
    
    fprintf(FHI,"%s ",vars.stringify(c->values(),true)());
    fprintf(FHI," %12.3e",c->iterationProgress());
    fprintf(FHI," %s",c->termination()());
    fprintf(FHI," %5ld",c->ID());
    fprintf(FHI,"\n");
  }
  fclose(FH);
  fclose(FHI);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::deleteUnusedDirs()
{
  printf("Calculations::deleteUnusedDirs\n");

  // DIR *dh;
  // struct dirent *dir;
  // dh=opendir(".");
  // if(dh==NULL) return;
  

  // while( (dir=readdir(dh))!=NULL ){
  //   if(strcmp(dir->d_name,".")==0||
  //      strcmp(dir->d_name,"..")==0){
  //     continue;
  //   }
  //   FileExistence ex=pathExistence(dir->d_name);
  //   if( ex==Directory ){
  //     printf("checking dir=%s\n",dir->d_name);
  //     bool dirisused=false;
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    if(c->status()==Calculation::NotUsed){
      printf("deleting dir=%s\n",c->dir()());
      systemCall(Sprintf("rm -rf %s",c->dir()()));
    }
    
  }
  // }
  //   }
  //   closedir(dh);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::respondToQuitRequest()
{
  String  fname="=.min_command";
  FileExistence ex=pathExistence(fname);
  if( ex==RegularFile ){
    try{
      FILE *FI=openforread(fname);
      
      String line;
      while(getLine(FI,line)){
	line.stripBlanks(true);
	if(line.find("stop")==0){
	  printf("stoping on user request ('=.min_control:stop')'\n");
	  exit(0);
	}
      }

      fclose(FI);
    }catch(...){};
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Calculations::constructDirName(long id,bool asmask)
{
  String dir;
  const Variables &vars=pcontrol->variables();
  String t=vars.stringify(asmask).replace(" ","_",0,true);
  
  if(asmask){
    if(pcontrol->useLongNames()){
      dir.sprintf("id=*_%s",t());
    }else{
      dir="id=*";
    }
  }else{
    if(pcontrol->useLongNames()){
      dir.sprintf("id=%05ld_%s",id,t());
    }else{
      dir.sprintf("id=%05ld",id);
    }
  }
  return dir;
}


/*EOF*/ 
