/*************************************************************************
 * @(#)CalculationsMin1D.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Dec 2011
 *************************************************************************
 */

#include "MethodControl.hh"
#include "Calculations.hh"
#include "MyMath.hh"
#include "Assorted.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
static void swap(T &a,T &b){
  T d=a;
  a=b;
  b=d;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void cycle_values(double &a,double &b,double &c,double &d)
{
  // result is cyclic shifting of values to the left
  // say: arg=a,b,c,d
  //     result= a has value of b, b has value of c,c of d  and d of a

  double dum=a;
  a=b;
  b=c;
  c=d;
  d=dum;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void leftshift_values(double &a,double &b,double &c,double d)
{
  // result is cyclic shifting of values to the left
  // say: arg=a,b,c,d
  //     result= a has value of b, b has value of c,c of d  and d of a

  a=b;
  b=c;
  c=d;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::minimize1D()
{

  const char *marker="MINIMIZE1D:";
  
  const Variables &vars=pcontrol->variables();
  
  
  
  run();//finish unfinished previous jobs
  
  
  //  reconsider:;
  
  //todo:   &init_check_calculations($control,$redo && !$rescan);
  
  
  // now check old calcs for braketing and start accordingly
  
  sortCalculation1D();
  
  long ncalc=pcalculations.size();
  ValueArray<Min1DPoint> initialPoints;
  ValueArray<long> bracketids;
  if(ncalc>=3){
    // we have more than 2 calcs from previous runs
    
    
    long imin;
    double fmin;
    yminInCalcs(imin,fmin);
    
    printf("%s minimum from previous runs: id=%ld, fmin=%30.15e\n",marker,
	   pcalculations.at(imin)->ID(),fmin);
    
    
    if(imin==0 ){
      //myDebug()<<"imin=0";
      initialPoints.resize(2);
      initialPoints[0]=Min1DPoint(pcalculations.at(0)->ID(),Min1DPoint::Old);
      initialPoints[1]=Min1DPoint(pcalculations.at(1)->ID(),Min1DPoint::Old);
    }else if(imin==ncalc-1){
      //myDebug()<<"imin=ncalc-1";
      initialPoints.resize(2);
      initialPoints[0]=Min1DPoint(pcalculations.at(ncalc-2)->ID(),
				  Min1DPoint::Old);
      initialPoints[1]=Min1DPoint(pcalculations.at(ncalc-1)->ID(),
				  Min1DPoint::Old);
    }else{
      
      //myDebug()<<"imin=else";
      long i0=-1;
      long  i1=-1;
      for(long i=imin-1;i>=0;i--){
	if(pcalculations.at(i)->etot()>fmin){
	  i0=i;
	  break;
	}
      }
      for(long i=imin+1;i<ncalc;i++){
	if(pcalculations.at(i)->etot()>fmin){
	  i1=i;
	  break;
	}
      }
      //myDebug()<<"i0="<<i0;
      //myDebug()<<"i1="<<i1;
      if(i0 >= 0 && i1 >=0){
	// we have already a braket
	//myDebug()<<"have braket";
	bracketids.clear();
	bracketids.append(pcalculations.at(i0)->ID());
	bracketids.append(pcalculations.at(imin)->ID());
	bracketids.append(pcalculations.at(i1)->ID());
      }else if(i0>=0){
	// it is a rare case of a plateau
	initialPoints.resize(2);
	initialPoints[0]=Min1DPoint(pcalculations.at(imin)->ID(),
				    Min1DPoint::Old);
	initialPoints[1]=Min1DPoint(pcalculations.at(i0)->ID(),
				    Min1DPoint::Old);
	
      }else if(i1>=0){
	// it is a rare case of a plateau
	initialPoints.resize(2);
	initialPoints[0]=Min1DPoint(pcalculations.at(imin)->ID(),
				    Min1DPoint::Old);
	initialPoints[1]=Min1DPoint(pcalculations.at(i1)->ID(),
				    Min1DPoint::Old);
      }else{
	// it is a rare case of a plateau
	initialPoints.resize(2);
	initialPoints[0]=Min1DPoint(pcalculations.at(imin)->ID(),
				    Min1DPoint::Old);
	initialPoints[1]=Min1DPoint(pcalculations.at(imin-1)->ID(),
				    Min1DPoint::Old);
      }
    }
    //myDebug()<<"here";
	
    
  }else if(ncalc==2){
    initialPoints.resize(2);
    initialPoints[0]=Min1DPoint(pcalculations.at(0)->ID(),
				Min1DPoint::Old);
    initialPoints[1]=Min1DPoint(pcalculations.at(1)->ID(),
				    Min1DPoint::Old);
  }else if(ncalc<=1){
    initialPoints.resize(3);
    for(int i=0;i<3;i++){
      initialPoints[i]=Min1DPoint(vars.at(0).bracket()[i],Min1DPoint::Guess);
    }
  }
  
//   if($rescan){
//     #&write_hash_ref(\*STDOUT,"inital_points",$inital_points,"",0);
//     #&write_hash_ref(\*STDOUT,"braket",$braket,"",0);
//     if($#$braket>=2){
//       print "have a braket\n";
//       for(my $i=$#$cal;$i>=0;$i--){
// 	#print  "id=$cal->[$i]->{id}\n";
// 	my $haveit=0;
// 	foreach my $b (@{$braket}){ 
// 	  if($b->{id} == $cal->[$i]->{id}){ $haveit=1; last; }
// 	}
// 	#print  "haveit\n" if ($haveit);
// 	#$cal->[$i]->{status}="new" if ($haveit);
// 	splice(@$cal,$i,1) if (!$haveit);
//       }
//       &deleteUnusedDirs($control);
//       &write_status($control,"calculations");
//       #&run($control);
//       $rescan=0;
//       $redo=1;
//       goto reconsider;
      
//     }elsif($#{$inital_points}==1){
      
//       print "have 2 initial points\n";
//       for(my $i=$#$cal;$i>=0;$i--){
// 	#print  "id=$cal->[$i]->{id}\n";
// 	my $haveit=0;
// 	foreach my $b (@{$inital_points}){ 
// 	  if($b->{id} == $cal->[$i]->{id}){ $haveit=1; last; }
// 	}
// 	print  "haveit\n" if ($haveit);
// 	#$cal->[$i]->{status}="new" if ($haveit);
// 	splice(@$cal,$i,1) if (!$haveit);
//       }
//       &deleteUnusedDirs($control);
//       &write_status($control,"calculations");
//       $rescan=0;
//       $redo=1;
//       goto reconsider;
//       die("two ini points\n");
      
//     }else{
//       print "not enough points star over\n";
//       $rescan=0;
//       $redo=0;
//       goto reconsider;
      
//       #die ("have no braket and less than 2 calcs -> implement it!\n");
//     }
//   }

//   printf("initialPoints.size=%ld\n",initialPoints.size());
//   for(int i=0;i<initialPoints.size();i++){
//     printf("%d: id=%ld\n",i,initialPoints.at(i).id);
//   }

  if( initialPoints.size()>=1){
    findBracket_1D(initialPoints,bracketids);
  }

  reportResults();

  brent(bracketids);
  


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::findBracket_1D(ValueArray<Min1DPoint> initialPoints,
				  ValueArray<long> &bracket,long idsc)
{
  Variables &vars=pcontrol->variables();

  double GLIMIT=100;
  double STEPMAX=vars.at(0).maxStep();
  double TINY=1.0e-20;
  double GOLD=1.618034;
  double ax,bx,cx,fa,fb,fc;
  const char *marker="BRAKET_MIN_1D:";

  
  printf("%s START\n",marker);
  printf("%s GLIMIT=%30.15g\n",marker,GLIMIT);
  printf("%s STEPMAX=%30.15g\n",marker,STEPMAX);

  if(initialPoints.size()==2){
    printf("%s have two initial points\n",marker);
    
    if(initialPoints.at(0).type==Min1DPoint::Old){

      long id=initialPoints.at(0).id;
      ax=getById(id)->values().at(0);
      fa=getById(id)->etot();

      
      id=initialPoints.at(1).id;
      bx=getById(id)->values().at(0);
      fb=getById(id)->etot();
      
    }else{ // type == Guess or Unknown
      
      ax=initialPoints.at(0).value;
      bx=initialPoints.at(1).value;
      
      vars.setCurrentValue(ax);
      long ida=registerCalculation(-1,idsc);
      vars.setCurrentValue(bx);
      long idb=registerCalculation(-1,idsc);
      
      writeStatusToFile();

      run();
    
      fa=getById(ida)->etot();
      fb=getById(idb)->etot();

    }
    
    printf("%s xa=%30.15g erg=%30.15g\n",marker,ax,fa);
    printf("%s xb=%30.15g erg=%30.15g\n",marker,bx,fb);

    
    if(fb > fa ){
      printf("%s exchange a<->b\n",marker);
      swap(ax,bx);
      swap(fa,fb);
      printf("%s xa=%30.15g erg=%30.15g\n",marker,ax,fa);
      printf("%s xb=%30.15g erg=%30.15g\n",marker,bx,fb);
    }
    
    cx=bx+GOLD*(bx-ax); // guess new downhill point
    fc=func(cx,idsc);
    
  }else if(initialPoints.size()==3){
    printf("%s have three initial points\n",marker);

    if(initialPoints.at(0).type==Min1DPoint::Old){
      throw Except("type==old\n"); 
      long id=initialPoints.at(0).id;
      ax=getById(id)->values().at(0);
      fa=getById(id)->etot();

      id=initialPoints.at(1).id;
      bx=getById(id)->values().at(0);
      fb=getById(id)->etot();

      id=initialPoints.at(2).id;
      cx=getById(id)->values().at(0);
      fc=getById(id)->etot();

    }else{
    
      ax=initialPoints.at(0).value;
      bx=initialPoints.at(1).value;
      cx=initialPoints.at(2).value;
      
      vars.setCurrentValue(ax);
      long ida=registerCalculation(-1,idsc);
      vars.setCurrentValue(bx);
      long idb=registerCalculation(-1,idsc);
      vars.setCurrentValue(cx);
      long idc=registerCalculation(-1,idsc);
      
      writeStatusToFile();
      
      run();
      
      printf("%s ida=%ld\n",marker,ida);
      printf("%s idb=%ld\n",marker,idb);
      printf("%s idc=%ld\n",marker,idc);
      fa=getById(ida)->etot();
      fb=getById(idb)->etot();
      fc=getById(idc)->etot();
      printf("%s ida=%ld erg=%30.15e\n",marker,ida,fa);
      printf("%s idb=%ld erg=%30.15e\n",marker,idb,fb);
      printf("%s idc=%ld erg=%30.15e\n",marker,idc,fc);

     }
    
    if(ax > bx ){ 
      swap(ax,bx);
      swap(fa,fb);
    }
    if(ax > cx ){ 
      swap(ax,cx);
      swap(fa,fc);
    }
    if(bx > cx ){ 
      swap(bx,cx);
      swap(fb,fc);
    }

    // now a<b<c

    if(fa < fc ){
      printf("%s exchange a<->c\n",marker);
      swap(ax,cx);
      swap(fa,fc);
    }
    // now a is the larger of two corners a,c
    
   }else{
    throw Except("more than 3 or less than 2 initial points to"
		 " be implemented\n");
   }





  printf("%s xa=%30.15g, ya=%30.15g\n",marker,ax,fa);
  printf("%s xb=%30.15g, yb=%30.15g\n",marker,bx,fb);
  printf("%s xc=%30.15g, yc=%30.15g\n",marker,cx,fc);



  while ( fb > fc ){// keep runing until we got b<a and b<c

    printf("%s --------------------------------\n",marker);
    printf("%s still b>c\n",marker);

    // parabolic extrapolation
    
    double r=(bx-ax)*(fb-fc);
    double q=(bx-cx)*(fb-fa);
    double u=bx-((bx-cx)*q-(bx-ax)*r)/
      (2.0*dSign(MyMath::max<double>(fabs(q-r),TINY),q-r));
    double du=GLIMIT*(cx-bx);
    double ulim=cx+dSign(MyMath::min<double>(fabs(du),STEPMAX),du);
    double fu;

    printf("%s u=%30.15g, ulim=%30.15g\n",marker,u,ulim);
    
    if((bx-u)*(u-cx) >0 ){ // parabolic u is between b and c
      printf("%s b<u<c\n",marker);

      fu=func(u,idsc);
      if(fu<fc){
	// got a minimum between b and c
	printf("%s minimum between b and c\n",marker);
	ax=bx;
 	fa=fb;
 	bx=u;
 	fb=fu;
 	printf("%s xa=%30.15g, erg=%30.15g\n",marker,ax,fa);
 	printf("%s xb=%30.15g, erg=%30.15g\n",marker,bx,fb);
 	printf("%s xc=%30.15g, erg=%30.15g\n",marker,cx,fc);
	goto end;
      }else if( fu > fb){
	// min between a and u
	printf("%s minimum between a and u\n",marker);
 	cx=u;
 	fc=fu;
 	printf("%s xa=%30.15g, erg=%30.15g\n",marker,ax,fa);
 	printf("%s xb=%30.15g, erg=%30.15g\n",marker,bx,fb);
 	printf("%s xc=%30.15g, erg=%30.15g\n",marker,cx,fc);
	goto end;
      }
      u=cx+GOLD*(cx-bx); // parabolic fir was no use -> magnify step
      fu=func(u,idsc);
      
    }else if( (cx-u)*(u-ulim) > 0){  //parabolic fit is between c and limit
      printf("%s c<u<ulimit\n",marker);
      
      fu=func(u,idsc);
      printf("%s u=%30.15g, erg=%30.15g\n",marker,u,fu);
      if(fu<fc){
 	printf("%s magnify\n",marker);
 	double r=u+GOLD*(u-bx);
	double fr=func(r,idsc);
	printf("%s r=%30.15g, erg=%30.15g\n",marker,r,fr);
	cycle_values(bx,cx,u,r);
 	cycle_values(fb,fc,fu,fr);
      }else{
 	printf("%s do nothing\n",marker);
      }

    }else if( (u-ulim)*(ulim-cx) >=0 ){// limit u to max allowed u
      printf("%s restrict u to ulimit\n",marker);
      u=ulim;
      fu=func(u,idsc);
    }else{ // reject parabolic u, use magnification
      printf("%s just magnify\n",marker);
      u=cx+GOLD*(cx-bx);
      fu=func(u,idsc);
      printf("%s u=%30.15g, erg=%30.15g\n",marker,u,fu);
    }
    printf("%s cycle values\n",marker);
    leftshift_values(ax,bx,cx,u);
    leftshift_values(fa,fb,fc,fu);

    printf("%s xa=%30.15g, erg=%30.15g\n",marker,ax,fa);
    printf("%s xb=%30.15g, erg=%30.15g\n",marker,bx,fb);
    printf("%s xc=%30.15g, erg=%30.15g\n",marker,cx,fc);
    
  } // while
  
  printf("%s after while\n",marker);


 end:;

  bracket.resize(3);
  vars.setCurrentValue(ax);
  bracket[0]=getClosestCalc()->ID();
  vars.setCurrentValue(bx);
  bracket[1]=getClosestCalc()->ID();
  vars.setCurrentValue(cx);
  bracket[2]=getClosestCalc()->ID();
  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::brent(ValueArray<long> bracketids,long idsc)
{
  Variables &vars=pcontrol->variables();

  double tol=pcontrol->minimizerXTol();

  const char *marker="BRENT:";

  if(idsc>=0) printf("%s idsc=%ld\n",marker,idsc) ;


  bool stepsexceede=false;
  long ITMAX=100;
  double  ZEPS=1.0e-10;
  double GOLD=.3819660;

  printf("%s bracket:",marker);
  for(int i=0;i<bracketids.size();i++){
    printf(" %ld",bracketids.at(i));
  }
  printf("\n");

  double e=0;

  double aa,bb,x,fx;
  {
    double ax=getById(bracketids.at(0))->values().at(0);
    double bx=getById(bracketids.at(1))->values().at(0);
    double cx=getById(bracketids.at(2))->values().at(0);

    double fa=getById(bracketids.at(0))->etot();
    double fb=getById(bracketids.at(1))->etot();
    double fc=getById(bracketids.at(2))->etot();

    printf("%s xa=%30.15g, ya=%30.15g\n",marker,ax,fa);
    printf("%s xb=%30.15g, yb=%30.15g\n",marker,bx,fb);
    printf("%s xc=%30.15g, yc=%30.15g\n",marker,cx,fc);
      
    aa=MyMath::min<double>(ax,cx);
    bb=MyMath::max<double>(ax,cx);
    x=bx;
    fx=fb;
  }
  
  double w=x;
  double v=x;
  

  double fw=fx;
  double fv=fx;



  double d,u,fu;

  for(long iter=1;iter<=ITMAX;iter++){
    printf("%s iteration %ld\n",marker,iter);
    double xm=0.5*(aa+bb);
    double tol1=tol*fabs(x)+ZEPS;
    double tol2=2.0*tol1;
    if(fabs(x-xm) <= (tol2-0.5*(bb-aa))){
      printf("%s done x=%30.15g: fx=%30.15g\n",marker,x,fx);
      goto end;
    }
    if(fabs(e) > tol1){
      
      // trial parabola
      double r=(x-w)*(fx-fv);
      double q=(x-v)*(fx-fw);
      double p=(x-v)*q-(x-w)*r;
      q=2.0*(q-r);
      if (q > 0.0) {p=-p;}
      q=fabs(q);
      double etemp=e;
      e=d;
      if(fabs(p)>=fabs(0.5*q*etemp) || p <= q*(aa-x) || p >=q*(bb-x)){
	e=(x>=xm? aa-x:bb-x);
	d=GOLD*e;
      }else{
	// take the parabolic step
	d=p/q;
	u=x+d;
	if(u-aa < tol2 || bb-u < tol2 ){
	  d=dSign(tol1,xm-x);
	}
      }
    }else{
      printf("%s first step: golden rule\n",marker);
      e=(x>= xm ? aa-x : bb-x);
      d=GOLD*e;
    }
    u=(fabs(d) >= tol1 ? x+d : x+dSign(tol1,d));
    fu=func(u,idsc);
    printf("%s u=%30.15g: fu=%30.15g\n",marker,u,fu);
    //
    if (fu <= fx ){
      if(u >= x ) {aa=x;} else {bb=x;}
      leftshift_values(v,w,x,u);
      leftshift_values(fv,fw,fx,fu);
    }else{
      if(u < x ) {aa=u;} else {bb=u;}
      if(fu <= fw || w == x){
	v=w;
	w=u;
	fv=fw;
	fw=fu;
      }else if(fu <= fv || v == x || v ==w){
	v=u;
	fv=fu;
      }
    }
  }// iter loop
  printf("%s x=%30.15g, erg=%30.15g\n",marker,x,fx);

 end:

  String varmin=Sprintf("%s_min",vars.at(0).name()());
  FILE *FH=openforwrite("=.result_min");
  fprintf(FH,"%s=%30.15g\n",varmin(),x);
  varmin=Sprintf("e_at_%s_min",vars.at(0).name()());
  fprintf(FH,"%s=%30.15g\n",varmin(),fx);
  fclose(FH);
  if(stepsexceede)
    throw Except(Sprintf("%s Too many iterations in brent\n",marker));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::yminInCalcs(long &imin,double &ymin)
{

  imin=0;
  ymin=0.;
  if(pcalculations.size()<1) return;
  ymin=pcalculations.at(imin)->etot();

  for(long i=1;i<pcalculations.size();i++){
    const Calculation *c=pcalculations.at(i);
    double f=c->etot();
    if(f<ymin){
      imin=i;
      ymin=f;
    }
  }
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::sortCalculation1D()
{
  const Variables &vars=pcontrol->variables();
  if(vars.size()!=1){
    throw Except("Calculations::sortCalculation1D called for D!=1\n");
  }
  
  for(long i=0;i<pcalculations.size();i++){
    for(long j=i+1;j<pcalculations.size();j++){
      if(pcalculations[i]->values().at(0)>pcalculations[j]->values().at(0)){
	Calculation *c=pcalculations[i];
	pcalculations[i]=pcalculations[j];
	pcalculations[j]=c;
      }
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Calculations::func(double x,long idsc)
{
  Variables &vars=pcontrol->variables();

  if(vars.size()!=1)
    throw Except("func(double) called for D!=1\n");
  
  vars.setCurrentValue(x);
  long id=registerCalculation(-1,idsc);

  sortCalculation1D();

  writeStatusToFile();

  run();
  reportResults();
  
  
  return getById(id)->etot();
}



/*EOF*/ 
