//todo: restart with new simplex around best point
//todo: better new point with saddle point

      //      todo: the corners should be referenced by ids not by
      // indices, since otherwise, we cannot register connectivities
/*************************************************************************
 * @(#)CalculationsSimplex.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Dec 2011
 *************************************************************************
 */

#include "Calculations.hh"
#include "Assorted.hh"
#include "MethodControl.hh"
#include "MyDebug.hh"
#include "SqMatrix.hh"
#include "MyMath.hh"
#include "Config.hh"


static long nSupTetOfD(long D)
{  
  return (D+1)*(D+2)/2;
}

static double power(double x,double y)
{
  return ::pow(x,y);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class PointStatistics{
public:
  PointStatistics(const  char *marker,double weightpower,
		  const Variables &vars,
		  const PtrArray<Calculation> &calcs)
    :pweightpower(weightpower){
    long D=vars.size();  

    long N=calcs.size();
    
    const Calculation *c0=calcs.at(0);
    ValueArray<double> x0=c0->values();
    double y0=c0->etot();

    pweights.resize(N); pweights[0]=0.;
    pxaverage.resize(D);
    pxvariation.resize(D);
    pxdeviation_cwise.resize(D);
    pxdeviation_cwise_rel.resize(D);
    pxaverage=0.;
    pyaverage=0.;
    pxvariation=0.;
    pxdeviation_cwise=0.;
    pxdeviation_cwise_rel=0.;

    double ssum=0.;
    //for(long i=1;i<N;i++){
      //const Calculation *c=calcs.at(i);
      //double de=MyMath::max<double>(c->etot()-y0,1.0e-8);
      //ssum+=pweights[i];
      //myDebug()<<"de="<<de<<" wei="<<power(de,-pweightpower);
      //}

    for(long i=1;i<N;i++){
      const Calculation *c=calcs.at(i);
      double de=MyMath::max<double>(c->etot()-y0,1.0e-8);
      pweights[i]=power(de,-pweightpower);
      ssum+=pweights[i];
      //myDebug()<<"de="<<de<<" wei="<<pweights[i];
    }

    //myDebug()<<"sum="<<pweights.sum()<<ssum;
    pweights/=ssum;//pweights.sum();
    pweights[0]=0.;
    //     printf("%s weights(power=%g)=%s\n%s sum=%g\n",marker,pweightpower,
    // 	   printS(pweights,"%12.5g")(),marker,pweights.sum());
    
    for(long i=1;i<N;i++){
      const Calculation *c=calcs.at(i);
      ValueArray<double> dx=c->values()-x0;
      pxaverage+=dx*pweights.at(i);
      pyaverage+=c->etot()*pweights.at(i);
      //myDebug()<<"y="<<c->etot()<<" wei="<<pweights.at(i)<< "yav="<<pyaverage;
      pxdeviation_cwise+=pow(dx,2)*pweights.at(i);
      if(i<=nSupTetOfD(D))
	pxvariation=max(pxvariation,abs(dx));
    }
    pxaverage+=x0;
    pxdeviation_cwise=sqrt(pxdeviation_cwise);
    
    for(long i=0;i<D;i++){
      if(x0.at(i)!=0){
	pxdeviation_cwise_rel[i]=pxdeviation_cwise.at(i)/x0.at(i);
      }else{
	pxdeviation_cwise_rel[i]=pxdeviation_cwise.at(i);
      }
    }

  };

  void report(const  char *marker){
    printf("%s x-average=%s\n",marker,printS(pxaverage,"%12.5g")());
    printf("%s y-average=%17.8f\n",marker,pyaverage);
    printf("%s x-deviation=%s\n",marker,
	   printS(pxdeviation_cwise,"%12.5g")());
    printf("%s x-deviation relative=%s\n",marker,
	   printS(pxdeviation_cwise_rel,"%12.5g")());
    printf("%s variation of N best x=%s\n",marker,
	   printS(pxvariation,"%12.5g")());

  };

  
  const ValueArray<double> &xaverage()const{return pxaverage;};
  double yaverage()const{return pyaverage;};

  const ValueArray<double> &xdeviation_cwise_rel()const{
    return pxdeviation_cwise_rel;};

  const ValueArray<double> &xvariation()const{
    return pxvariation;};
  const ValueArray<double> &weights()const{
    return pweights;};
  
private:
  // the points are weighted according to 1/(delta_E)^pweightpower
  double pweightpower;
  ValueArray<double> pweights;
  ValueArray<double> pxaverage;
  double pyaverage;
  // averaged deviation, component wise
  ValueArray<double> pxdeviation_cwise;
  // relative averaged deviation, component wise
  ValueArray<double> pxdeviation_cwise_rel;
  // variation  of x for the best points
  ValueArray<double> pxvariation;
  
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::minimizeSimplex()
{
  Variables &vars=pcontrol->variables();

  const char *marker="minimizeSimplex:";

  long NITER=200;
  
  long D=vars.size();

  piterationcounter=0;
  
    
  printCalculations();

  run();

  sortCalcsAccordingToY();
  printCalculations();

//   if($delete_worst){
//     print "really delete worst calculations?";
//     my $l=<STDIN>;
//     if(strip_ws($l) =~/[yY]/){
      
//       my $cal=$control->{calculations};
//       my $N=nSupTetOfD($D);
//       for(my $i=$N;$i<$#$cal+1;$i++){
// 	#print "$cal->[$i]->{id} ym=$cal->[$i]->{erg}\n";
// 	delete $cal->[$i];
//       }
//       &deleteUnusedDirs($control);
//       &printCalculations($control);
//     }else{
//       die("abort\n");
//     }
//   };
  

//   &init_check_calculations($control,$redo);

//   &sortCalcsAccordingToY($control);
//   &printCalculations($control);

  long  ncalc=pcalculations.size();

  printf("%s D=%ld N_S=%ld N_calc=%ld\n",marker,D,nSupTetOfD(D),ncalc);
  ValueArray<double> start_point_vec(D,0.);
  ValueArray<double> start_scale_vec(D,0.);
  

  
  bool newsimplex=pcontrol->newSimplex()||(ncalc<nSupTetOfD(D))||
    pcontrol->permanentSimplex();
  
  if(newsimplex){
    if((ncalc<nSupTetOfD(D))){
      
      for(long j=0;j<D;j++){
	start_point_vec[j]=vars.at(j).bracket()[1];
	start_scale_vec[j]=fabs(vars.at(j).bracket()[2]-start_point_vec[j])*8/5;
      }
    }else{
      PointStatistics pointstats(marker,pweightpower,vars,pcalculations);
      start_point_vec=pcalculations.at(0)->values();
      start_scale_vec=pointstats.xvariation()*0.75;
    }
    
    
    
    createStartSimplex(start_point_vec,start_scale_vec);
    printCalculations();
    
    
    run();
    sortCalcsAccordingToY();
    printCalculations();
  }
  
   // now the minimization
   long iter=0;
   while(iter<NITER){
    
     bool newsimplex=calcGeneralExtremum(start_point_vec,start_scale_vec)
       || pcontrol->permanentSimplex();

     
     if(newsimplex){
       PointStatistics pointstats(marker,pweightpower,vars,pcalculations);
       start_point_vec=pcalculations.at(0)->values();
       start_scale_vec=pointstats.xvariation()*0.5;
     }

     
     run();
     sortCalcsAccordingToY();
     printCalculations();
     
     if(newsimplex){
       
       
       createStartSimplex(start_point_vec,start_scale_vec);
       printCalculations();
       
       
       run();
       sortCalcsAccordingToY();
       printCalculations();
     
     }
     
     iter++;
   }
   
   writeResult(pcalculations.at(0));
   printf("stop because: iterations exceeded\n");
    
   //die("todo");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::createStartSimplex(const ValueArray<double> &start_point_vec,
				      const ValueArray<double> &start_scale_vec)
{
  const char *marker="createStartSimplex:";
  
  Variables &vars=pcontrol->variables();
  
  long D=vars.size();
  
  printf("%s point=%s\n",marker,printS(start_point_vec,"%30.15g",",")());
  printf("%s scale=%s\n",marker,printS(start_scale_vec,"%30.15g",",")());

  for(long i=0;i<=D;i++){
    ValueArray<double> x=start_point_vec-start_scale_vec*0.375;

    ValueArray<double> z(D,0.);
    if(i>0){
      z[i-1]=start_scale_vec.at(i-1);
    }
    x+=z;

    vars.setCurrentValue(x);
    long id=registerCalculation(-1);
    getById(id)->setEtot(0.);
    
  }


  for(long  i=0;i<=D;i++){
    for(long j=i+1;j<=D;j++){
      ValueArray<double> x=(pcalculations.at(i)->values()+
			    pcalculations.at(j)->values())*0.5;

      vars.setCurrentValue(x);
      long id=registerCalculation(-1);
      getById(id)->setEtot(0.);
    }
  }
  pnbetween=0;
  piterationcounter++;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::printCalculations()
{
  const Variables &vars=pcontrol->variables();
  
  long D=vars.size();

  long npoints=nSupTetOfD(D);

  printf("-----------------------------------------------------"
 	 "-------------------\nSimplex data: last %ld iteration count=%ld\n",
	 pcalculations.size()-1,piterationcounter);
  // print simplex corners
  for(long i=0;i<pcalculations.size();i++){
    Calculation *c=pcalculations[i];
    ValueArray<double> &x=c->values();
    if(i<npoints){
      printf("* id_%3ld: x=",c->ID());
    }else{
      printf("  id_%3ld: x=",c->ID());
    }
    for(long i=0;i<D;i++){ 
      printf("%15g ",x.at(i)); 
    }
    printf(" erg=%17.8f status=%s\n" ,c->etot(),
	   Calculation::statusGuiName(c->status()));
    
  }


  printf("-----------------------------------------------------"
 	 "-------------------\n");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::sortCalcsAccordingToY()
{

  for(long i=0;i<pcalculations.size();i++){
    for(long j=i+1;j<pcalculations.size();j++){
      if(pcalculations[i]->etot()>pcalculations[j]->etot()){
	Calculation *c=pcalculations[i];
	pcalculations[i]=pcalculations[j];
	pcalculations[j]=c;
      }
    }
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Calculations::calcGeneralExtremum(ValueArray<double> &start_point_vec,
				       ValueArray<double> &start_scale_vec)
{
  const char *marker="calcGeneralExtremum:";
  const Variables &vars=pcontrol->variables();
  printf("%s \n",marker);
  
  long D=vars.size();

  PointStatistics pointstats(marker,pweightpower,vars,pcalculations);

  pointstats.report(marker);

  const Calculation *c0=pcalculations.at(0);
  double y0=c0->etot();
  ValueArray<double> x0=c0->values();
  


  bool conv=true;
  for(long i=0;i<D;i++){
    if(pointstats.xdeviation_cwise_rel().at(i)>pcontrol->minimizerXTol()){
      conv=false;
    }
  }
  if(vlevel>=Vlevel::Many) {
    printf("%s %s\n",marker,(conv?"converged":"not converged"));
    printf("%s best x=%s\n",marker,printS(x0)());
    printf("%s best y=%17.8f\n",marker,y0);
  }
  
  if(conv && pdidone){
    sortCalcsAccordingToY();
    printCalculations();
    writeResult(c0);
    throw Except("x does not change much -> might be converged\n");
  }

  printf("%s \n",marker);

  SqMatrix<double> interpol_A;
  ValueArray<double> interpol_B;
  calcInterpolatorGeneral(interpol_B,interpol_A,pointstats);
  
  if(pinterpolindef){
    printf("%s interpolation is bad, create new simplex\n",marker);
    printf("%s \n",marker);
    return true;
  }
     
  printf("%s \n",marker);
  printf("%s constant  term B=%s\n",marker,printS(interpol_B,"%17.8f")());
  printf("%s quadratic term A=\n",marker); printSqMatrix(interpol_A,"%17.8f",marker);
  printf("%s \n",marker);

  // take minmax of last N points points
  long idmin,idmax2,idmax;
  classifyAllPoints(idmin,idmax2,idmax);

  ValueArray<double> mostDecrDir;
  
  String exttype;
  {
    SqMatrix<double> V;
    ValueArray<double> w;
    try{
      String msg;
      diagonalizeSymmetric(interpol_A,w,V,msg);
      if(msg.length()){
	printf("diagonalization: %s\n",msg());
      }
    }catch(Except &ex){
      throw Except("diagonalization failed: %s\n",ex.msg());
    }
    printf("%s \n%s diag(A)=%s\n%s \n",marker,marker,printS(w,"%17.8f")()
	   ,marker);
    
    bool min=true;
    bool max=true;
    long imin=-1;
    double wmin;
    for(long i=0;i<w.size();i++){
      if(w.at(i)<0){
	min=false;
	if(imin<0){
	  wmin=w.at(i);
	  imin=i;
	  //myDebug()<<"initial   imin="<<imin<<wmin;
	}else{
	  //myDebug()<<"continous imin="<<imin<<wmin;
	  if(w.at(i)<wmin){
	    wmin=w.at(i);
	    imin=i;
	  }
	}
      }
      if(w.at(i)>0) max=false;
    }
    if(imin>=0){
      printf("%s \n%s most decreasing direction=%ld w=%17.8f\n%s \n",
	     marker,marker,imin,w.at(imin),marker);
    }
    //it can be a saddlepoint
    if(min && !max){
      exttype="min";
    }else if(!min && max){
      exttype="max";
    }else{
      exttype="saddle";
    }
    if(exttype!="min"){
      mostDecrDir=V.column(imin);
      printf("%s \n%s mostDecrDir=%s\n%s \n",
	     marker,marker,printS(mostDecrDir,"%17.8f")(),marker);
      printf("%s \n%s A*mostDecrDir=%17.8f\n%s \n",marker,marker,
	     mostDecrDir*(interpol_A*mostDecrDir),marker);
    }
  }

  // y=y0+(x-x0)*b+1/2*(x-x0)*A*(x-x0)
  // (xmin-x0)=-1/A*b

  // determine the position x_ext of the extremum
  SqMatrix<double> Ai=interpol_A.inverse();
  if(vlevel>=Vlevel::All){
    printf("%s \n",marker);
    printf("%s 1/A=\n",marker); printSqMatrix(Ai,"%17.8f",marker);
    printf("%s 1=\n",marker); printSqMatrix(Ai*interpol_A,"%17.8f",marker);
    printf("%s \n",marker);
  }
  // the point of the extremum
  ValueArray<double> dxext=-Ai*interpol_B;
  ValueArray<double> xext=dxext+x0;
    
  printf("%s \n",marker);
  printf("%s dx_sol=%s\n",marker,printS(dxext,"%17.8f")());
  printf("%s 0=%s\n",marker,printS(interpol_B+interpol_A*dxext,"%17.8f")());
  printf("%s \n",marker);

  
  double dx=norm(dxext);
  printf("%s |dxext|=%17.8f\n",marker,dx);
  printf("%s \n",marker);


  // determine scaling such that none violates maxstep
  double scale=1.;
  for(long i=0;i<D;i++){
    double m=vars.at(i).maxStep();
    scale=MyMath::min<double>(scale,m/fabs(dxext[i]));
    printf("%s old dx[%ld] d%s=%17.8f maxstep[%ld]=%17.8f\n",
	   marker,
	   i,vars.at(i).name()(),dxext[i],
	   i,m);
  }
  scale=MyMath::min<double>(1.,scale);
  printf("%s maxstep-scale=%17.8f\n",marker,scale);
  printf("%s \n",marker);



  // y=y0+b*dx+1/2*dx*A*dx
  // dx=-Ai*b
  // y_sol=y0+1/2*b*dx

  ValueArray<double> xsol;
  double ysol;
  if(exttype=="min"){
    printf("%s case of solution=minimum\n",marker);
    ValueArray<double> dxextscaled=dxext*scale; 
    for(long i=0;i<D;i++){
      double m=vars.at(i).maxStep();
      printf("%s old dx[%ld] d%s=%17.8f maxstep[%ld]=%17.8f\n"
	     "%s new dx[%ld] d%s=%17.8f maxstep[%ld]=%17.8f\n",
	     marker,i,vars.at(i).name()(),dxext[i],i,m,
	     marker,i,vars.at(i).name()(),dxextscaled[i],i,m);
    }
    xsol=dxextscaled+x0;
    ysol=y0+interpol_B*dxextscaled*0.5;
    printf("%s xsol[%s]=%s\n",marker,exttype(),printS(xsol,"%17.8f")());
    printf("%s ysol 2nd order= %17.8f\n",marker,ysol);
    printf("%s y0            = %17.8f\n",marker,y0);
  }else{
    printf("%s \n",marker);
    printf("%s case of solution=saddle point or maximum\n",marker);
    printf("%s \n",marker);
    if(norm(x0-xext)>1.0e-6){
      // the smallest point and the saddle are differnt
      // go along downhill direction
      dxext=-interpol_B;
      double scale=1.0e+30;
      for(long i=0;i<D;i++){
	double m=vars.at(i).maxStep();
	if(fabs(dxext[i])>0)
	  scale=MyMath::min<double>(scale,m/fabs(dxext[i]));
      }
      printf("%s downhill-maxstep-scale=%17.8f\n",marker,scale);
      printf("%s \n",marker);
      ValueArray<double> dxextscaled=dxext*scale;
      for(long i=0;i<D;i++){
	double m=vars.at(i).maxStep();
	printf("%s old dx[%ld] d%s=%17.8f maxstep[%ld]=%17.8f\n"
	       "%s new dx[%ld] d%s=%17.8f maxstep[%ld]=%17.8f\n",
	       marker,i,vars.at(i).name()(),dxext[i],i,m,
	       marker,i,vars.at(i).name()(),dxextscaled[i],i,m);
      }
      printf("%s \n",marker);
      xsol=dxextscaled+x0;
      ysol=y0+(interpol_B*dxextscaled)
	+(dxextscaled*(interpol_A*dxextscaled))*0.5;

      printf("%s xdownhill[%s]=%s\n",marker,exttype(),printS(xsol,"%17.8f")());
      printf("%s ydownhill = %17.8f\n",marker,ysol);
      printf("%s y0        = %17.8f\n",marker,y0);
      printf("%s \n",marker);
    }else{
      // the smallest known point is close to the saddle
      //(xext+mostDecrDir*lam)-x_i
      throw Except("the smallest known point is close to the saddle");
    }
  }
  



  



  
 
  long nloop=0;
  double yext=func(xsol);
  printf("%s \n",marker);
  printf("%s y at new point=%17.8f\n",marker,yext);
  printf("%s \n",marker);
  
 loop:;

  if(conv && pdidone){
    sortCalcsAccordingToY();
    printCalculations();
    writeResult(c0);
    throw Except("x does not change much -> might be converged\n");
  }

  pdidone=true;
  
  if(yext<getById(idmin)->etot()){
    printf("%s ext is smaller\n",marker);
    pnbetween=0;
  }else if(yext<getById(idmax)->etot()){
    if(pnbetween>MyMath::min<double>(2,nSupTetOfD(D)/4)) {
      printf("%s ext is in between, create new simplex\n",marker);
      printf("%s \n",marker);
      return true;
    }else{
      pnbetween++;
      printf("%s ext is in between\n",marker);
      printf("%s \n",marker);
    }
  }else{
    printf("%s ext is larger\n",marker);
    xsol=(xsol+getById(idmin)->values())*0.5;
    yext=func(xsol);
    printf("%s y at line search trial=%17.8f\n",marker,yext);
    //todo: better line search

    nloop++;
    if(nloop<=0){
      goto loop;
    }else{
      printf("%s failed line search, create new simplex\n",marker);
      printf("%s \n",marker);
      return true;
    }
  }
  
  
  
  printf("%s \n",marker);

  sortCalcsAccordingToY();
  printCalculations();
  

  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::writeResult(const Calculation *c0)
{
  const Variables &vars=pcontrol->variables();
  
  const ValueArray<double> &x=c0->values();
  const double &y=c0->etot();
  
  long D=vars.size();


  if(D>1){
    FILE *FH=openforwrite("=.result_min");
    String varmin=Sprintf("%s_min",vars.at(0).name()());
    fprintf(FH,"id=%ld\n",c0->ID());
    fprintf(FH,"%s=%30.15g\n",vars.at(0).name()(),x.at(0));
    for(long ii=1;ii<D;ii++){
      fprintf(FH,"%s=%30.15g\n",vars.at(ii).name()(),x.at(ii));
      varmin+=Sprintf("%s_min",vars.at(ii).name()());
    }
    fprintf(FH,"e_at_%s_min=%30.15g\n",varmin(),y);
    fclose(FH);

    String resdir="min";
    FileExistence ex=pathExistence(resdir);
    //printf("resdir=%s, ex=%d\n",resdir(),ex);
    if( ex==SymLink ){
      printf("reset link from best directory to '%s'\n",resdir());
      systemCall(Sprintf("rm %s ; ln -s %s %s",resdir(),c0->dir()(),resdir()));
    }else if( ex==DoesNotExist ){
      printf("set link from best directory '%s' to '%s'\n",c0->dir()(),
	     resdir());
      //myDebug()<<Sprintf("ln -s %s %s",c0->dir()(),resdir());
      systemCall(Sprintf("ln -s %s %s",c0->dir()(),resdir()));
    }else{
      printf("cannot set link from best directory to '%s', because it exists\n"
	     ,resdir());
      while(1){
	resdir+="1";
	FileExistence ex1=pathExistence(resdir);
	if(ex1==DoesNotExist) break;
      }
      printf("will set link to '%s' instead\n",resdir());
      systemCall(Sprintf("ln -s %s %s",c0->dir()(),resdir()));
    }
    
  }else{
    throw Except("writeResult: D==1 not yet implemented.");
  }  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::calcInterpolatorGeneral(ValueArray<double> &B,
					   SqMatrix<double> &A,
					   const PointStatistics &pointstats)
{
  const char *marker="CALC_INTERPOLATOR:";
  printf("%s \n",marker);

  const Variables &vars=pcontrol->variables();
  
  long D=vars.size();
  long N=nSupTetOfD(D);

  double tol=1.0e-12;

  long Nv=(D*(D+3))/2;
  long maxCalc=pcalculations.size();
  printf("%s N=%ld Nv=%ld maxCalc=%ld\n",marker,N,Nv,maxCalc);



  if(vlevel>=Vlevel::Many){
    for(long i=0;i<N;i++){
      const Calculation *c=pcalculations.at(i);
      printf("%s best points: id=%ld erg=%17.8f\n",marker,c->ID(),c->etot()); 
    }
  }



  SqMatrix<double> M(maxCalc-1,Nv);
  ValueArray<double> m(maxCalc-1,0.);

  const Calculation *c0=pcalculations.at(0);
  double y0=c0->etot();
  ValueArray<double> x0=c0->values();
  
  printf("%s \n",marker);
  for(long i=1;i<maxCalc;i++){
    const Calculation *c=pcalculations.at(i);
    ValueArray<double> dx=(c->values()-x0)/pointstats.xvariation();
    double dy=c->etot()-y0;
    printf("%s dx=%s wei=%17.8f\n",marker,printS(dx,"%10.5f")(),
	   pointstats.weights().at(i));
    
    long k0=0;
    for(long k=0;k<D;k++){
      M(i-1,k+k0)=dx.at(k);
    }
    k0=D;
    for(long k=0;k<D;k++){
      M(i-1,k+k0)=0.5*power(dx.at(k),2);
    }
    k0=2*D-1;
    for(long j=0;j<D;j++){
      for(long jj=j+1;jj<D;jj++){
  	k0++;
	//myDebug()<<"i="<<i<<"k0="<<k0<<"j="<<j<<"jj="<<jj;
 	M(i-1,k0)=dx.at(j)*dx.at(jj);
      }
    }
    m[i-1]=dy;
  }
  printf("%s \n",marker);

  if(vlevel>=Vlevel::All){
    printf("%s M=\n",marker); printSqMatrix(M,"%17.8f",marker);
    printf("%s \n%s m=%s\n%s \n",marker,marker,printS(m)(),marker);
  }

  // m=M*A
  // S=M^T*M
  // S=D*s*D^T
  // M^T*m=S*A
  // A=D*P(1/s)*D^T*M^T*m

  SqMatrix<double> mwei=SqMatrix<double>(pointstats.weights())
    .removeRows(0,0).removeCols(0,0);
  SqMatrix<double> S=M.transposed()*mwei*M;
  ValueArray<double> b=m*mwei*M;



  if(vlevel>=Vlevel::All){
    printf("%s S=\n",marker); printSqMatrix(S,"%17.8f",marker);
  }

  SqMatrix<double> V;
  ValueArray<double> w;
  try{
    String msg;
    diagonalizeSymmetric(S,w,V,msg);
    if(msg.length()){
      printf("diagonalization: %s\n",msg());
    }
  }catch(Except &ex){
    throw Except("diagonalization failed: %s\n",ex.msg());
  }
  SqMatrix<double> ww(w);

  if(vlevel>=Vlevel::All){
    printf("%s \n%s s=%s\n%s \n",marker,marker,printS(w)(),marker);
    printf("%s V=\n",marker); printSqMatrix(V,"%17.8f",marker);
    printf("%s V*S*V-s=\n",marker); 
    printSqMatrix(V.transposed()*S*V-ww,"%17.8f",marker);
  }

  pinterpolindef=false;
  //todo: scaling
  SqMatrix<double> PwP(ww);
  if(vlevel>=Vlevel::All){
    printf("%s PwP=\n",marker); printSqMatrix(PwP,"%17.8f",marker);
  }
  for(long i=0;i<Nv;i++){
    if( PwP(i,i) < fabs(PwP(Nv-1,Nv-1))*tol ){
      PwP(i,i)=0.;
      // this is the worst convergence criterion, but anyway
      writeResult(c0);
      printf("%s \n%s SVD zero\n%s \n",marker,marker,marker);
      // die("SVD zero\n");
      pinterpolindef=true;
    }else{
      PwP(i,i)=1./PwP(i,i);
    }
  }
  if(vlevel>=Vlevel::All){
    printf("%s P/1wP=\n",marker); printSqMatrix(PwP,"%17.8f",marker);
  }

  ValueArray<double> v=V*PwP*V.transposed()*b;
  
  
  
  printf("%s \n%s parameter-vector=%s\n%s \n",marker,marker,
	 printS(v,"%17.8f")(),marker);
  
  if(vlevel>=Vlevel::All){
    printf("%s \n%s M*v-m=%s\n%s \n",marker,marker,printS(M*v-m,"%17.8f")(),
	   marker);
   }
  
  B.resize(D);
  A.resize(D);
  
  long k0=0;
  for(long k=0;k<D;k++){
    B[k]=v.at(k+k0)/pointstats.xvariation().at(k);
  }
  k0=D;
  for(long k=0;k<D;k++){
    A(k,k)=v.at(k+k0)/(pointstats.xvariation().at(k)
		       *pointstats.xvariation().at(k));
  }
  k0=2*D-1;
  for(long j=0;j<D;j++){
    for(long jj=j+1;jj<D;jj++){
      k0++;
      A(j,jj)=v.at(k0)/(pointstats.xvariation().at(j)
			*pointstats.xvariation().at(jj));
      A(jj,j)=A(j,jj);
    }
  }
  
  //printf("B=%s\n",printS(B,"%17.8f")());
  //printf("A=\n"); printSqMatrix(A,"%17.8f");

  bool failed=false;
  for(long i=0;i<maxCalc;i++){
    const Calculation *c=pcalculations.at(i);
    ValueArray<double> dx=c->values()-x0;
    
    double y=y0+B*dx+dx*(A*dx)*0.5;
    
    if( i<N &&
	(((c->etot()!=0. ) && (fabs(y-c->etot())>1.0e-4*fabs(y)))||
	 ((c->etot()==0. ) && (fabs(y-c->etot())>1.0e-4)))
       ){
      printf("%s error: id=%ld: interpolated=%17.8f real=%17.8f\n",
	     marker,c->ID(),y,c->etot());
      failed=true;
    }
    if(vlevel>=Vlevel::All){
      printf("%s id=%ld: interpolated=%17.8f real=%17.8f err=%17.8f\n",
	     marker,c->ID(),y,c->etot(),y-c->etot());
    }
  }
  if(vlevel>=Vlevel::All){
    printf("%s \n",marker);
  }
  if(failed) throw Except("interpolation failed\n");
  printf("%s \n",marker);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Calculations::classifyAllPoints(long &idmin,long &idmax2,long &idmax)
{
  const char *marker="classifyAllPoints";
  const Variables &vars=pcontrol->variables();
  
  long D=vars.size();
  long N=nSupTetOfD(D);
  
  
  idmin=pcalculations.at(0)->ID();
  idmax=idmin;
  double emin=pcalculations.at(0)->etot();
  double emax=emin;
  for(long i=0;i<N;i++){
    const Calculation *c=pcalculations.at(i);
    if(c->etot()<emin){
      idmin=c->ID();
      emin=c->etot();
    }
    if(c->etot()>emax){
      idmax=c->ID();
      emax=c->etot();
    }
  }
  idmax2=idmin;
  double emax2=emin;
  for(long i=0;i<N;i++){
    const Calculation *c=pcalculations.at(i);
    if(c->etot()>=emax2 && c->ID() != idmax&& c->ID() != idmin){
      idmax2=c->ID();
      emax2=c->etot();
    }
  }

  printf("%s \n",marker);
  printf("%s min   id=%ld y=%17.8f\n",marker,idmin,getById(idmin)->etot());
  printf("%s max2  id=%ld y=%17.8f\n",marker,idmax2,getById(idmax2)->etot());
  printf("%s max   id=%ld y=%17.8f\n",marker,idmax,getById(idmax)->etot());
  printf("%s \n",marker);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Calculations::func(const ValueArray<double> &x,long idsc)
{
  Variables &vars=pcontrol->variables();
  
  
  vars.setCurrentValue(x);
  long id=registerCalculation(-1,idsc);

  //todo: why?  sortCalculation();

  writeStatusToFile();

  run();
  reportResults();

  piterationcounter++;

  return getById(id)->etot();

}


/*EOF*/ 
