/*************************************************************************
 * @(#)JobInterface.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   02 Dec 2011
 *************************************************************************
 */

#include <sys/stat.h>
#include "Assorted.hh"
#include "Except.hh"
#include "JobInterface.hh"
#include "MyDebug.hh"
#include "MethodControl.hh"





/* ***********************************************************************
*
* ***********************************************************************
*/
void JobInterface::writePBS_SH_default()
{
    String fname=expandAndCreatePath("$HOME/.minner/pbs/pbs.sh");
    FileExistence ex=pathExistence(fname);
    if(ex==DoesNotExist){
      FILE *FH=openforwrite(fname);
      fprintf(FH,"%s",pbs_sh_default);
      fclose(FH);
    }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static String readRunningFile()
{
  FILE *FH=openforread("=.running");
  String line;
  String jobid;
  while(getLine(FH,line)){
    if(MethodControl::getFromKey(line,"jobid",jobid,'=')){
      continue;
    }
  }
  fclose(FH);
  return jobid;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void JobInterface::setDefault()
{
  pfplo="/home/magru/FPLO/bin/fplo14.00-45-x86_64";
  prunfplo="/home/magru/FPLO/bin/fplo14.00-45-x86_64";
  pfedit="/home/magru/FPLO/bin/fedit14.00-45-x86_64";
  pallotedtime="01:00:00";
  pallotedmemory="500mb";
  pjobname="jobname";
  pmethod="PBS";
  pforce_initial_spin_split=false;
  pforce_convergence=false;
  pcopy=true;
  pdeleteonnode=true;
  
  try{

    writePBS_SH_default();

    String fname=expandAndCreatePath("$HOME/.minner/defaults");

    FileExistence ex=pathExistence(fname);
    if(ex==DoesNotExist){
      FILE *FH=openforwrite(fname);
      writeToFile(FH);
      fclose(FH);
    }else{
      FILE *FH=openforread(fname);

      bool injob=false;
      String line;
      while(getLine(FH,line)){
	line.stripBlanks(true);
	if(injob){
	  // -------------- job information section ---------------
	  if(MethodControl::getTag(line,"endjob")){
	    injob=false;
	    continue;
	  }
	  if(MethodControl::getFromKey(line,"fplo",pfplo)) continue;
	  if(MethodControl::getFromKey(line,"runfplo",prunfplo)) continue;
	  if(MethodControl::getFromKey(line,"fedit",pfedit)) continue;
	  if(MethodControl::getFromKey(line,"time",pallotedtime)) continue;
	  if(MethodControl::getFromKey(line,"memory",pallotedmemory)) continue;
	  if(MethodControl::getFromKey(line,"name",pjobname)) continue;
	  if(MethodControl::getFromKey(line,"method",pmethod)) continue;
	  if(MethodControl::getFromKey(line,"copydata",line)){
	    if(line.lowerCase().find("t")==0){
	      pcopy=true;
	    }else{
	      pcopy=false;
	    }
	    continue;
	  }
	  if(MethodControl::getFromKey(line,"deleteonnode",line)){
	    if(line.lowerCase().find("t")==0){
	      pdeleteonnode=true;
	    }else{
	      pdeleteonnode=false;
	    }
	    continue;
	  }
	  if(MethodControl::getFromKey(line,"forceinitialspinsplit",line)){
	    if(line.lowerCase().find("t")==0){
	      pforce_initial_spin_split=true;
	    }else{
	      pforce_initial_spin_split=false;
	    }
	    continue;
	  }
	  if(MethodControl::getFromKey(line,"forceconvergence",line)){
	    if(line.lowerCase().find("t")==0){
	      pforce_convergence=true;
	    }else{
	      pforce_convergence=false;
	    }
	    continue;
	  }
	}else{
	  if(MethodControl::getTag(line,"job")){
	    //enter job information section
	    injob=true;
	    clear();
	    continue;
	  }
	}

      }
      fclose(FH);

    }


  }catch(Except &ex){
    die(ex.msg());
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void JobInterface::writeToFile(FILE *FH)
{
  fprintf(FH,"\njob:\nfplo:    %s\nrunfplo: %s\nfedit:   %s\n"
	  "method: %s\n"
	  "forceinitialspinsplit: %s\n"
	  "forceconvergence: %s\n"
	  "copydata: %s\n"
	  "deleteonnode: %s\n"
	  "\ntime: %s\nmemory: %s\nname: %s\nendjob:\n\n",
	  pfplo(),prunfplo(),pfedit(),pmethod(),
	  (pforce_initial_spin_split?"t":"f"),
	  (pforce_convergence?"t":"f"),(pcopy?"t":"f"),(pdeleteonnode?"t":"f"),
	  pallotedtime(),pallotedmemory(),pjobname());
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void JobInterface::runFeditPipe()const
{
  //myDebug()<<"START JobInterface::runFeditPipe()";

  int ret=systemCall(Sprintf("%s -pipe -p %s < =.pipe 2>./+log >/dev/null",
			     pfedit(),pfplo()));
  if(ret != 256 ){
    systemCall("cat ./+log");
    systemCall("cat =.pipe");
    throw Except(" -------------------------------\n");
   }

  //myDebug()<<"END JobInterface::runFeditPipe()";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void JobInterface::submitFplo(const String &dir)
{
  int fdpwd=saveCurrentDir();

  chDir(dir()) ;
  
  // remove marker for a finished job
  FileExistence ex=pathExistence("+done");
  if( ex==RegularFile ){
    removeFile("+done");
  }else if( ex!=DoesNotExist ){
    throw Except(Sprintf("cannot remove file %s/+done\n",dir()));
  }

  String runner,program,path;
  
  splitPath(path,program,prunfplo);
  if(path==".") 
    throw Except("the fplo/fedit variable should have a full "
		 "absolute pathname\n");

  
  if(pmethod.find("QS")==0){
    runner=Sprintf("echo \"y\" |QSUB -p %s -hp %s -c %s %s -m %s -n %s %s ",
		   program(),path(),pallotedtime(),
 		   (pforce_initial_spin_split?"-i":""),
 		   pallotedmemory(),pjobname(),
		   (pforce_convergence?"":"-norestart")
		   );
    systemCall(runner);
  }else if(pmethod.find("FQ-l")==0){
    
    runner=Sprintf("echo \"y\" |FQcalc -l -p %s -hp %s %s %s >/dev/null",
		   program(),path(),
		   (pforce_convergence?"":"-norestart"),
 		   (pforce_initial_spin_split?"-i":"")
		   );
    systemCall(runner);
  }else if(pmethod.find("PBS")==0){
    makePBSFile();
    runner="qsub";
    String serr,jobid;
    StringList args;
    args.append("pbs.sh");
    int ret=mySystem(runner,args,jobid,serr);
    if(serr.length()>0 || ret ){
      printf("problems launching %s: :%s\n",runner(),serr());
    }
    jobid.stripBlanks(true);
    FILE *FH=openforwrite("=.running");
    fprintf(FH,"jobid=%s\n",jobid());
    fclose(FH);
  }else if(pmethod.find("condor")==0){
    makeCondorFile();
    runner="condor_submit";
    String serr,jobid;
    StringList args;
    args.append("condor.cmd");
    int ret=mySystem(runner,args,jobid,serr);
    if(serr.length()>0 || ret ){
      printf("problems launching %s: :%s\n",runner(),serr());
    }
    jobid.stripBlanks(true);
    jobid=jobid.subString(jobid.find("cluster")+7);
    jobid.replace("."," ",jobid.find("."),true);
    FILE *FH=openforwrite("=.running");
    fprintf(FH,"jobid=%s\n",jobid());
    fclose(FH);
  }else{
    die(Sprintf("unkown running/job method: %s!",pmethod()));
  }
  
  
  restoreCurrentDir(fdpwd);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void JobInterface::killFplo(const String &dir)const
{
  
  int fdpwd=saveCurrentDir();
  
  chDir(dir()) ;
  
  
  
  //todo: this does not work if the directory gets copied
  if(pmethod.find("FQ")==0){
    systemCall("FQcalc -kill\n");
  }else if(pmethod.find("QS")==0){
    systemCall("QSUB -kill\n");
  }else if(pmethod.find("PBS")==0){
    FileExistence ex=pathExistence("=.running");
    if( ex==RegularFile ){
      String jobid=readRunningFile();
      printf("jobid=%s\n",jobid());
      StringList args;
      String sout,serr;
      args.append("-W");
      args.append("300");
      args.append(jobid);
      int ret=mySystem("qdel",args,sout,serr);
      if(serr.length()>0 || ret ){
	printf("problems killing :%s\n",serr());
      }
    }
  }else if(pmethod.find("condor")==0){
    FileExistence ex=pathExistence("=.running");
    if( ex==RegularFile ){
      String jobid=readRunningFile();
      printf("jobid=%s\n",jobid());

      StringList args;
      String sout,serr;

      args<<jobid<<"-runv";
      int ret=mySystem("condor_q",args,sout,serr);
      if(sout.find(jobid)>=0){

	args.clear();
	args<<"-v"<<"-e"<<"condor_ssh_to_job"<<Sprintf("%s:*",jobid())<<".";
	ret=mySystem("rsync",args,sout,serr);
      }      
      args.clear();
      args<<jobid;
      ret=mySystem("condor_rm",args,sout,serr);
      if(serr.length()>0 || ret ){
	printf("problems killing :%s\n",serr());
      }
      
    }
  }else{
    //todo: this does not work if the directory gets copied
    FileExistence ex=pathExistence("+run");
    if( ex==RegularFile ){
      
      
      FILE *FH=openforread("+run");
      String line;
      String host;
      String pid;
      while(getLine(FH,line)){
	line.stripBlanks(true);
	if(MethodControl::getFromKey(line,"host",host)){
	  continue;
	}
	if(MethodControl::getFromKey(line,"pid",pid)){
	  continue;
	}
      }
      fclose(FH);
      printf("host=%s\npid=%s\n",host(),pid());
      systemCall(Sprintf("ssh %s \"kill -9 %s\"",host(),pid()));
    }else{
      printf("Could not kill, since '%s/+run' does not exist.\n"
	     " May be it is not running.\n",dir());
    }
  }
  
  

  restoreCurrentDir(fdpwd);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void JobInterface::transferFiles(const String &dir)const
{// force transfer of files from remote machines for intermediate checking
  int fdpwd=saveCurrentDir();
  chDir(dir()) ;

  if(pmethod.find("condor")==0){
    FileExistence ex=pathExistence("=.running");
    if( ex==RegularFile ){
      String jobid=readRunningFile();
      //printf("jobid=%s\n",jobid());
    
      StringList args;
      String sout,serr;

      args<<jobid<<"-runv";
      int ret=mySystem("condor_q",args,sout,serr);

      if(sout.find(jobid)>=0){
	printf("INFO: transfering files for job %s ...\n",jobid());

	args.clear();
	args<<"-v"<<"-e"<<"condor_ssh_to_job"<<Sprintf("%s:*",jobid())<<".";
	ret=mySystem("rsync",args,sout,serr);
	//args<<"-ssh"<<"scp -r"<<jobid<<"remote:*"<<".";
	//int ret=mySystem("condor_ssh_to_job",args,sout,serr);
	// if(serr.length()>0 || ret ){
	//   printf("problems transfering files: %s\n",serr());
	// }
	ret=ret;
      }else{
	printf("INFO: job %s seems not to be running ... skip transfering files\n",
	       jobid());
      }      
    }
  }else{
    throw Except(Sprintf("cannot transfer files for current queue system %s\n",
			 pmethod()));
  }

  restoreCurrentDir(fdpwd);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void JobInterface::makePBSFile()const
{
  // make pbs file in current calculation directory
  String pbs_sh;

  try{
    String fname=expandAndCreatePath("$HOME/.minner/pbs/pbs.sh");
    FileExistence ex=pathExistence(fname);
    if(ex==DoesNotExist){
      writePBS_SH_default();
    }else{
      FILE *FH=openforread(fname);
      String line;
      while(getLine(FH,line)){
	pbs_sh+=line;
      }
      fclose(FH);
    }
  }catch(Except &ex){
    die(ex.msg());
  }


  pbs_sh
    .replace("%job%",pjobname)
    .replace("%mem%",pallotedmemory,0,true)
    .replace("%cputime%",pallotedtime,0,true)
    .replace("%executable%",prunfplo,0,true)
    .replace("%initialspinsplit%",pforce_initial_spin_split,0,true)
    .replace("%copydata%",pcopy,0,true)
    .replace("%deleteonnode%",pdeleteonnode,0,true)
    ;
  FILE *FH;
  FH=openforwrite("pbs.sh");
  fprintf(FH,"%s",pbs_sh());
  fclose(FH);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void JobInterface::makeCondorFile()const
{
  const char *fname="condor.sh";
  FILE *FH;
  FH=openforwrite("condor.cmd");
  fprintf(FH,"\
universe=vanilla\n\
executable=%s\n\
output=condor.out\n\
error=condor.err\n\
log=condor.log\n\
+WantCheckpoint = False\n\
should_transfer_files = IF_NEEDED\n\
when_to_transfer_output = ON_EXIT\n\
transfer_input_files = .\n\
transfer_output_files = .\n\
queue\n\
",fname
	  );
  fclose(FH);


  FH=openforwrite(fname);
  fprintf(FH,"\
#!/bin/sh\n\
\n\
\n\
\n\
\n\
\n\
# This job's working directory\n\
echo Working directory is `pwd`\n\
\n\
echo Running on host `hostname`\n\
echo Time is `date`\n\
echo Directory is `pwd`\n\
echo This jobs runs on the following processors:\n\
EXEC=%s\n\
init_split=%d\n\
\n\
\n\
[ -f ./+term ] && rm -f ./+term\n\
[ -f ./+kill ] && rm -f ./+kill\n\
[ -f ./+done ] && rm -f ./+done\n\
\n\
\n\
# Run the executable fplo\n\
\n\
echo pwd=`pwd`\n\
\n\
if [ x$init_split = x0 ] ; then\n\
  # if fplo is asking  questions like \"Have 2 spin sorts, want to skip\n\
  #                                   do initial spinplit?\" \n\
  # we assume that the answer yes is what we want, in most cases its \n\
  # correct. If not, redo i.e. the splitting be hand in every subdirectory\n\
  # and proceed with this script afterwards\n\
\n\
echo \"y\" > ./+yes\n\
echo \"y\" >> ./+yes\n\
echo \"y\" >> ./+yes\n\
echo \"y\" >> ./+yes\n\
\n\
$EXEC <  ./+yes > out\n\
\n\
else\n\
\n\
echo \"n\" > ./+no\n\
echo \"n\" >> ./+no\n\
echo \"n\" >> ./+no\n\
echo \"n\" >> ./+no\n\
\n\
$EXEC <  ./+no > out\n\
\n\
fi\n\
\n\
\n\
touch ./+term\n\
touch ./+done\n\
\n\
\n\
"
	  ,prunfplo(),pforce_initial_spin_split
	  //,pjobname(),pallotedmemory(),pallotedmemory(),pallotedtime(),
	  //
);
  fclose(FH);

  chmod("condor.cmd",S_IRUSR|S_IWUSR|S_IROTH);
  chmod(fname,S_IRUSR|S_IWUSR|S_IXUSR|S_IROTH);
  
  FileExistence ex=pathExistence(".job.ad");
  if( ex==RegularFile ){
    removeFile(".job.ad");
  }
  ex=pathExistence(".machine.ad");
  if( ex==RegularFile ){
    removeFile(".machine.ad");
  }

}
/*EOF*/ 
