/*************************************************************************
 * @(#)Main.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 Dec 2011
 *************************************************************************
 */

#include <stdlib.h>
#include <stdio.h>
#include "Except.hh"
#include "Assorted.hh"
#include "SqMatrix.hh"
#include "Config.hh"
#include "MethodControl.hh"


String startargs;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void usage()
{
  //todo
  printf("\
%s [--tol tolerance] [--delete_unused] [-stop_running] \n\
   [--restart] [--refresh] [--new_simplex] [--always_simplex] \n\
   [--sleep time_in_sec] [--dont_run] [--restart_unfinished]\n\
   [--force_file_transfer]\n\
\t--stop_running:   stop all running jobs\n\
\t--restart: recalculate all valid dirs\n\
\t--refresh: re-create all input and restart\n\
\t--delete_unused:   deletes unused directories\n\
\t--new_simplex:   start by creating a new simplex from previous calculations\n\
\t--always_simplex:   always try a new simplex from previous calculations\n\
\t--sleep time:   sleep for tims seconds\n\
\t--dont_run:   stop when before running\n\
\t--restart_unfinished: restart all jobs, which are not having status 'Ready'\n\
\t--force_file_transfer: force transfer of files from remote machine, \n\
                         even when not ready\n\
",EXECNAMESHORT());
;
//
//\t--i     :   force initial spin split, if set in fedit\n	
//\t--norestart  :   do not attempt to force convergence\n	
// \t--d     :   delete worst points from history\n
// \t--redo  :   scan old calculations in all existing directories and \n
// \t--rescan:   recalculate all existing directories\n
// \t            use the starting point according to the old energies\n
// \t            then do a minimization, by recalculating from there using\n
// \t            the old calculations as good starting points, this leaves\n
// \t            inconsistent directories\n

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void processCommandLineArgs(int argc,const char** argv,
				   MethodControl &mc)
{

  int i=0;
//   bool redo=false;
//   bool rescan=false;
  startargs="";
  while(i<argc){
    startargs+=argv[i++];
    startargs+=" ";
  }
  
  i=1;
  while(i<argc){
    String arg=argv[i++];
    //printf("arg=[%s]\n",arg());
    //    if(arg == "--redo"){
    //    }else
    if(arg == "--stop_running"){
      mc.setStopRunning(true);
    }else if(arg == "--restart_unfinished"){
      mc.setRestartUnfinished(true);
    }else if(arg == "--force_file_transfer"){
      mc.setForceFileTransfer(true);
    }else if(arg == "--dont_run"){
      mc.setDontRun(true);
    }else if(arg == "--restart"){
      mc.setRestart(true);
    }else if(arg == "--refresh"){
      mc.setRefreshInput(true);
    }else if(arg == "--delete_unused"){
      mc.setDeleteUnused(true);
    }else if(arg == "--new_simplex"){
      mc.setNewSimplex(true);
    }else if(arg == "--always_simplex"){
      mc.setPermanentSimplex(true);
//       redo=true;
//     }else if(arg =="--rescan"){
//       rescan=true;
      
      //   }elsif($arg =~/--norestart/){
      //     $restart=0;
//     }else if(arg == "--i" || arg == "-i"){
//       mc.jobInterface().setFlags(JobInterface::ForceInitialSpinSplit,"true");
      //   }elsif($arg =~/^--model$/){
      //     &run_as_model();
      //   }elsif($arg =~/^--d$/){
      //     $delete_worst=1;
//     }else if(arg == "--run"){
//       if(i>=argc) {
// 	usage();
// 	die("option --run needs an argument\n");
//       }
//       arg=argv[i++];
//       mc.jobInterface().setFlags(JobInterface::Queing,arg);
    }else if(arg =="--sleep"){
      if(i>=argc) {
 	usage();
 	die("option --sleep needs an argument\n");
      }
      arg=argv[i++];
      try{
	mc.setSleepTime(arg.convertToLong());
      }catch(Except &ex){
	die(Sprintf("argument for --sleep: %s\n",ex.msg())());
      }
      
    }else if(arg =="--tol"){
      if(i>=argc) {
 	usage();
 	die("option --tol needs an argument\n");
      }
      arg=argv[i++];
      try{
	mc.setMinimizerXTol(arg.convertToFortranReal());
      }catch(Except &ex){
	die(Sprintf("argument for --tol: %s\n",ex.msg())());
      }
    }else{
      usage();
      die("abort\n");
    }
  }
}






/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main(int argc,const char** argv)
{

  MethodControl methodcontrol;


  vlevel=Vlevel::All;
  
  if(vlevel>=Vlevel::All){
    printf("START\n");
    printf("verbosity level: %s\n",vlevel.name());
  }
  

  EXECNAME=argv[0];
  EXECNAMESHORT=argv[0];
  long ind;
  EXECNAMESHORT.chop("/");
  if((ind=EXECNAMESHORT.find("/",-1))>=0){
    EXECNAMESHORT=EXECNAMESHORT.subString(ind+1);
  }

  
// my $redo=0;
// my $rescan=0;
// my $delete_worst=0;
// my $did_one=0;

// my $LIBRARY='
// ';

// my $control={value_diff_tol=>1.0e-10
// 	     ,minimizer_xtol=>1.0e-3
// 	    };


  processCommandLineArgs(argc,argv,methodcontrol);


  try{
    FILE *FH=openforwrite("+minner_arguments");
    fprintf(FH,"%s\n",startargs());
    fprintf(stdout,"%s\n",startargs());
    fclose(FH);
  }catch(Except &ex){
    die(ex.msg());
  }

  FileExistence ex=pathExistence(methodcontrol.controlFile());
  switch(ex){
  case DoesNotExist:
    {
      printf("file '%s' does not exist, will create default, which "
	     "you will have to edit\n"
	     "which method: [m]inimize, [c]urve:",
	     methodcontrol.controlFile()());
      MinEnums::Method whichmethod=MinEnums::None;
      while(whichmethod==MinEnums::None){
	char c=(char)getc(stdin);
	if(c=='m'||c=='M'){
	  whichmethod=MinEnums::Minimize;
	}else if(c=='c'||c=='C'){
	  whichmethod=MinEnums::Curve;
	}
      }
      methodcontrol.setDefault(whichmethod);
      methodcontrol.writeControlFile(methodcontrol.controlFile());
      exit(0);
      break;
    }
  case RegularFile:
    {
      break;
    }
  case Directory:
    {
      die(Sprintf("file '%s' is a directory?\n",
		  methodcontrol.controlFile()())());
      break;
    }
  case SymLink:
    {
      break;
    }
  case SomethingElse:
    {
      die(Sprintf("file '%s' seems not to be a file?\n",
		  methodcontrol.controlFile()())());
      break;
    }
  }

    
    

  try{
    methodcontrol.readControlFile();
  }catch(Except &ex){
    die(ex.msg());
  }

  //methodcontrol.writeControlFile("test");




  //$D=pvariables.size();



    
    
    
  try{
    methodcontrol.doCalculations();
  }catch(Except &ex){
    die(ex.msg());
  }


//if($control->{method} eq "minimize-nsc" && $control->{D}==1){

//   &init_check_calculations($control,0);

//   &miniNSC($control);
  
  

  
// &messagedialog("done");

// exit(0);


// ########################################################################
// ########################################################################
// ########################################################################
// sub messagedialog{
//   my $text=shift;
//   $text.=" in `pwd`";
//   system "zenity --info --text=\"$text\"";
// }
// ########################################################################

// ########################################################################
// sub deleteUnusedDirs{
//   my $control=shift;
  
//   my $cal=$control->{calculations};
//   foreach my $f (glob("id=*")){
//     my $id=$f;
//     if($control->{long_names}){
//       $id=~s/id=(\d+)_.*$/\1/g;
//     }else{
//       $id=~s/id=(\d+)$/\1/g;
//     }
//     print "f=$id\n";
//     my $haveit=0;
//     foreach my $c (@{$cal}){
//       if($c->{id}==$id){ $haveit=1;last;}
//     }
//     if(!$haveit){
//       print "have to kill id=$id\n";
//       `rm -rf $f`;
//     }
//   }
// }########################################################################
// ########################################################################


// ########################################################################
// sub sign1{
//   my $x=shift;
//   my $value=shift;
//   if($x>=0) {
//     return abs($value);
//   }else{
//     return -abs($value);
//   }
// }
// ########################################################################
// sub prep_open_dx{
//   my $control=shift;
//   my $interpol=shift;
//   my $which=shift;
  
//   printf("which=%s %d\n",$which,($which eq "tet only"));

//   my $D=$control->{D};
//   my $cal=$control->{calculations};

//   my $N=30;
//   my $scale=2;

//   my $x0=new Math::MatrixReal($D,1);
//   my $x1=new Math::MatrixReal($D,1);

//   $x0=$x0->each(sub{shift; 1.0e50});
//   $x1=$x1->each(sub{shift;-1.0e50});


//   foreach my $c (@$cal){
//     my $x=$c->{x};
//     for(my $i=1;$i<=$D;$i++){
//       $x0->assign($i,1,Math::MatrixReal::min($x0->element($i,1)
// 					      ,$x->element($i,1)));
//     }
//     for(my $i=1;$i<=$D;$i++){
//       $x1->assign($i,1,Math::MatrixReal::max($x1->element($i,1)
// 					     ,$x->element($i,1)));
//     }
//   }


//   {  
//     my $x=$control->{ext}->{x};
//     for(my $i=1;$i<=$D;$i++){
//       $x0->assign($i,1,Math::MatrixReal::min($x0->element($i,1)
// 					      ,$x->element($i,1)));
//     }
//     for(my $i=1;$i<=$D;$i++){
//       $x1->assign($i,1,Math::MatrixReal::max($x1->element($i,1)
// 					     ,$x->element($i,1)));
//     }
//   }



//   my $dx=$x1-$x0;
//   $x0=$x0-($scale-1.)/2*$dx;
//   $x1=$x1+($scale-1.)/2*$dx;

//   print "x0\n=",$x0;
//   print "x1\n=",$x1;

//   my $radius=$x1->element(1,1)-$x0->element(1,1);

//   #printf("radius=%g\n",$radius);
  
  
// #  $x0=[-20,-20];
// #  $x1=[20,20];
  
//   my $xx=new Math::MatrixReal($N*$N,$D);
//   #print "xx=\n",$xx;
//   for(my $i=0;$i<$N;$i++){
//     for(my $j=0;$j<$N;$j++){
//       my $x;
//       $x=Math::MatrixReal->new_diag([$i,$j])/($N-1)*($x1-$x0)+$x0;
//       #print "x=\n",$x;
//       $xx->assign_row(1+$i+$j*$N,~$x);
//     }
//   }

//   if(($which ne "tet only")){
//     open(fd,">surf.dx");
//     printf(fd "#scalar data".
// 	   "\nobject 1 class array type float rank 1 shape 1 items %ld".
// 	   " data follows\n",$N*$N);
//     for(my $i=1;$i<=$N*$N;$i++){
//       my $x=$xx->row($i);
//       #print "x=\n",$x;
//       my $y=&calcY(~$x);
//       printf(fd "%15g\n",$y);
//     }
//     printf(fd "attribute \"dep\" string \"positions\"\n");
//     #
//     printf(fd "#position data".
// 	   "\nobject 2 class array type float rank 1 shape 2 items %ld ".
// 	   "data follows\n",$N*$N);
//     for(my $i=1;$i<=$N*$N;$i++){
//       for(my $k=1;$k<=$D;$k++){ printf(fd "%15g ",$xx->element($i,$k)); }
//       printf(fd "\n");
//     }
//     printf(fd "attribute \"dep\" string \"positions\"\n#\n");
//     ###############################
//     printf(fd "#scalar data interpol".
// 	   "\nobject 3 class array type float rank 1 shape 1 items %ld".
// 	   " data follows\n",$N*$N);
//     print "b=\n",$interpol->{B};
//     print "A=\n",$interpol->{A};
    
//     for(my $i=1;$i<=$N*$N;$i++){
//       my $x=$xx->row($i);
//       my $dx=(~$x-$interpol->{x0});
//       my $yi=$interpol->{y0}+$interpol->{B}->scalar_product($dx)
// 	+0.5*$dx->scalar_product($interpol->{A}*$dx);
//       printf(fd "%15g\n",$yi);
//     }
//     printf(fd "attribute \"dep\" string \"positions\"\n");
//     #
//     printf(fd "# the whole object\n".
// 	   "object \"surf-field\" class field\n".
// 	   "component \"data\" value 1\n".
// 	   "component \"positions\" value 2\n".
// 	   "attribute \"name\" string \"surf-field\"\n".
// 	   "#\n");
//     printf(fd "# the whole object\n".
// 	   "object \"interpol-field\" class field\n".
// 	   "component \"data\" value 3\n".
// 	   "component \"positions\" value 2\n".
// 	   "attribute \"name\" string \"interpol-field\"\n".
// 	   "#\n");
    
//     printf(fd "# the whole object\n".
// 	   "object \"surf group\" class group\n".
// 	   "member \"surf\" value \"surf-field\"\n".
// 	   "member \"interpol\" value \"interpol-field\"\n".
// 	   "#\n");
//     printf(fd "end\n");
    
    
//     close(fd);
//   }
  
//   &write_tet_dx($control);

//   if(($which ne "tet only")){

//     open(fo,">ext.dx");
//     printf(fo "#scalar data".
// 	   "\nobject 1 class array type float rank 1 shape 1 items 1".
// 	   " data follows\n");
//     printf(fo "%15g ",$control->{ext}->{erg});
//     printf(fo "\n");
//     printf(fo "attribute \"dep\" string \"positions\"\n");
//     #
//     printf(fo "#radius data".
// 	   "\nobject 3 class array type float rank 1 shape 1 items 1".
// 	   " data follows\n");
//     printf(fo "%15g ",$radius/20.);
//     printf(fo "\n");
//     printf(fo "attribute \"dep\" string \"positions\"\n");
//     #
//     printf(fo "#position data".
// 	   "\nobject 2 class array type float rank 1 shape 2 items 1 ".
// 	 "data follows\n");
//     for(my $j=1;$j<=$D;$j++){
//       printf(fo "%15g ",$control->{ext}->{x}->element($j,1));
//     }
//     printf(fo "\n");
//     printf(fo "attribute \"dep\" string \"positions\"\n#\n");

//     printf(fo "# the whole object\n".
// 	   "object \"ext\" class field\n".
// 	   "component \"data\" value 3\n".
// 	   "component \"positions\" value 2\n".
// 	   "attribute \"name\" string \"tet\"\n".
// 	   "end\n");


//     close(fo);

//   }
  
//   system("dx -script surf.net");

// }

// ########################################################################
// sub write_tet_dx{
//   my $control=shift;

//   my $D=$control->{D};
//   my $cal=$control->{calculations};

//   my $N=nSupTetOfD($D);

  
//   open(fo,">tet.dx");
//   #
//   printf(fo "#scalar data".
// 	 "\nobject 1 class array type float rank 1 shape 1 items %ld".
// 	 " data follows\n",$N);
  
//   for(my $i=0;$i<$N;$i++){
//     my $y=$cal->[$i]->{erg};
//     printf(fo "%15g ",$y);
//   }
//   printf(fo "\n");
//   printf(fo "attribute \"dep\" string \"positions\"\n");
//   #
//   #
//   printf(fo "#position data".
// 	 "\nobject 2 class array type float rank 1 shape 2 items %ld ".
// 	 "data follows\n",$N);
  
  
//   for(my $i=0;$i<$N;$i++){
//     my $x=$cal->[$i]->{x};
//     for(my $j=1;$j<=$D;$j++){
//       printf(fo "%15g ",$x->element($j,1));
//     }
//     printf(fo "\n");
//   }
//   printf(fo "attribute \"dep\" string \"positions\"\n#\n");
//   #
//   #
//   printf(fo "#connection data\n".
// 	 "object 3 class array type int rank 1 shape 2 items %ld "
// 	 ."data follows\n",$N-1);
  
  
//   for(my $i=1;$i<$N;$i++){
//     printf(fo "%ld %ld\n",0,$i);
//   }
  
  
//   printf(fo "attribute \"element type\" string \"lines\"\n");
//   printf(fo "attribute \"ref\" string \"positions\"\n");
//   #
//   printf(fo "# the whole object\n".
// 	 "object \"tet\" class field\n".
// 	 "component \"data\" value 1\n".
// 	 "component \"positions\" value 2\n".
// 	 "component \"connections\" value 3\n".
// 	 "attribute \"name\" string \"tet\"\n".
// 	 "end\n");
  
//   close(fo);
// }


// ########################################################################
// sub run_as_model{


//   my $return;
//   my $stat_file="=.in";
//   unless ($return = eval `cat $stat_file`) {
//     die "couldn't parse $stat_file: $@" if $@;
//     die "couldn't do $stat_file: $!"    unless defined $return;
//     die "couldn't run $stat_file:"      unless $return;
//   }
//   my $dat=$return;


//   #printf("x=%s\n",$dat->[0]);
//   #printf("y=%s\n",$dat->[1]);
  
//   my $D=$#$dat+1;
  
//   my $x=new Math::MatrixReal($D,1);
  
//   for(my $i=1;$i<=$D;$i++){
//     $x->assign($i,1,$dat->[$i-1]);
//   }
  
  
//   #print "x=\n",$x;
  
//   my $y=&calcY($x);
  
  
//   `>+done`;
//   printf("EE: %s\n",$y);
//   print "TERMINATION: Finished:\n";
  
  
//   exit(1);
  
// }
// ########################################################################
// sub calcY{
//   my $x=shift;
//   #printf(" calcY: -------------------\n");
//   #print "x=\n",$x;
//   my $y0=1;
//   my $b=Math::MatrixReal->new_from_cols([[-3,5]]);
//   my $A=Math::MatrixReal->new_from_rows([[3,-0.2],[-0.2,1]]);
//   #my $b=Math::MatrixReal->new_from_cols([[0,0]]);
//   #my $A=Math::MatrixReal->new_from_rows([[1,0],[0,1]]);
//   #print "b=\n",$b;
//   #print "A=\n",$A;


//   my $a=0.5*$x->scalar_product($A*$x);
  
  
//   my $y=$y0 +$b->scalar_product($x)+$a+0.5*$a*$a
//     +0.000*$x->scalar_product($x);


//   return $y;
// }
// ########################################################################
// ########################################################################
// ########################################################################
// ########################################################################
// ########################################################################
// ########################################################################
// ########################################################################






// ########################################################################
// sub ABS{
//   my $v=shift;
//   if(ref $v eq "Math::MatrixReal"){
//     return $v->norm_p(2);
//   }else{
//     return abs($v);
//   }
// }

// ########################################################################
// sub stringify_vector{
//   my $v=shift;
//   my $wc=shift;
//   my $wc=defined($wc);
//   #print "defined wc=",$wc,"\n";
//   if(ref $v eq "Math::MatrixReal"){
//     my $t="[";
//     (my $D,my $dum)=$v->dim();
//     for(my $i=1;$i<=$D;$i++){
//       $t.=sprintf("%s ",$v->element($i,1));
//       if($wc && $i<$D){
// 	$t.=",";
//       }
//     }
//     $t.="]";
//     return $t;
//   }else{
//     return $v;
//   }
// }


// ########################################################################
// sub set_fplo_input_singlestep{
//   my $control=shift;
//   my $dir=shift;
  
//   my $marker="SET_FPLO_INPUT_SINGLESTEP:";
//   printf("\n$marker directory: %s\n",$dir);


//   my $ROOT=`pwd`;
//   $ROOT=~s/\s+$//g;
//   chdir $dir;
//   printf("$marker %s",`pwd`);
//   open(fo,">=.pipe_nsc");
//   printf(fo "\@n\@1\n");
//   printf(fo "\@q\@\n");
//   close(fo);

  
//   `$control->{fedit} -p $control->{fplo} -pipe < =.pipe_nsc 2>./+log`;
//   if($? != 256 ){
//     system "cat ./+log";
//     system "cat =.pipe";
//     die(" -------------------------------\n");
//   }

//   `rm  =.pipe_nsc`;
  

//   chdir $ROOT;

// }







// ########################################################################
// sub strip_ws{
//   my $t=shift;
//   $t=~s/^\s+//g;
//   $t=~s/\s+$//g;
//   return $t;
// }



// ########################################################################
// sub  make_EX_script{
//   my $control=shift;

//   if(! -f "EX" ){
 
//     open(fo,">EX");
//    print fo <<EOF
// prefix=\'id=*_$control->{prefix}\'
// (
// echo "------------- iter ----------------"
// grit \$prefix
// echo "------------- erg ----------------"
// grEE \$prefix | tee e
// echo "------------- eV ----------------"
// grEV \$prefix out 1 | tee ev
// echo "------------- gap ----------------"
// grGAP \$prefix | tee g
// echo "------------- spin ----------------"
// grSS \$prefix | tee s
// echo "------------- spin 1 ----------------"
// grSSat Al 1 \$prefix | tee sAl1
// echo "------------- time ----------------"
// grTI \$prefix
// echo "------------- term ----------------"
// grTE \$prefix
// )| less -S
// EOF
// ;
//     close(fo);
//     `chmod u+x EX`
    
//   }
// }











// ########################################################################
// sub escapify{
//   my $text=shift;

// #  $text=~s/\\n/\\\n/g;
// #  $text=~s/\n/\\n/g;
//   $text=~s/\'/\\\'/g;
  
//   return $text;
// }




// ########################################################################
// sub write_hash_ref{
//   my $fo=shift;
//   my $name=shift;
//   my $h=shift;
//   my $tab=shift;
//   my $level=shift;
//   my $c="=>";
//   if(!$level){ $c="="; }
//   $level++;

//   if(ref $h eq "HASH"){
//     if($name =~ /^$/ ){
//       printf($fo "{\n");
//     }else{
//       printf($fo "$tab%s${c}{\n",$name);
//     }
//     $tab.="  ";
//     foreach my $k (sort keys %$h){
//       if( ref ($h->{$k}) eq "HASH"){
// 	&write_hash_ref($fo,$k,$h->{$k},$tab,$level);
// 	printf($fo ",\n");
//       }elsif( ref ($h->{$k}) eq ""){
// 	printf($fo "$tab%s=>'%s',\n",$k,&escapify($h->{$k}));
//       }elsif(ref ($h->{$k}) eq "ARRAY"){
// 	&write_hash_ref($fo,$k,$h->{$k},$tab,$level);
// 	printf($fo ",\n");
//       }elsif(ref ($h->{$k}) eq "Math::MatrixReal"){
// 	(my $rows,my $cols)=$h->{$k}->dim();
// 	if($cols != 1){
// 	  printf($fo "$tab%s=>%s\n",$k,ref ($h->{$k}));
// 	  die("oops, forgot to implement the data type ".(ref ($h->{$k}))
// 	      ." for other than cols=1\n");
// 	}
// 	printf($fo "$tab%s=>[",$k);
// 	for(my $i=1;$i<=$rows;$i++){
// 	  printf($fo "'%s'",escapify($h->{$k}->element($i,1)));
// 	  printf($fo ",") if($i<$rows);
// 	}
// 	printf($fo "],\n");
//       }else{
// 	printf($fo "$tab%s=>%s\n",$k,ref ($h->{$k}));
// 	die("oops, forgot to implement the data type ".(ref ($h->{$k}))."\n");
//       }
//     }
//     printf($fo "$tab}");
//   }elsif(ref $h eq "ARRAY"){
//     if($name =~ /^$/ ){
//       printf($fo "[");
//     }else{
//       printf($fo "$tab%s${c}[",$name);
//     }
//     $tab.="  ";
//     for(my $i=0;$i<=$#$h;$i++){
//       if(ref ($h->[$i]) eq ""){
// 	printf($fo "'%s'",escapify($h->[$i]));
// 	printf($fo ",") if($i<$#$h);
//       }elsif(ref ($h->[$i]) eq "HASH"){
// 	&write_hash_ref($fo,"",$h->[$i],$tab,$level);
// 	printf($fo ",\n$tab") if($i<$#$h);
//       }elsif(ref ($h->[$i]) eq "ARRAY"){
// 	&write_hash_ref($fo,"",$h->[$i],$tab,$level);
// 	printf($fo ",") if($i<$#$h);
//       }else{
// 	die("oops in ARRAY, forgot to implement the data type ".(ref ($h))."\n");
//       }
      
//     }
    
   
//     printf($fo "]");
//   }

//   printf($fo "\n") if($level==1);
// }  








// ########################################################################
// sub tablesorter{
//   if(length($a)<length($b)) {return 1;}
//   if(length($a)>length($b)) {return -1;}
//   return ($a cmp $b);
// }

// ########################################################################
// sub miniNSC{
//   my $control=shift;
//   my $tol=$control->{minimizer_xtol};
//   my $marker="miniSC";

//   printf("$marker START\n");

//   my $cal=$control->{calculations};
//   my $ncalc=$#$cal+1;
//   my $inital_points=[];
//   my $braket=[];
//   my $sc=0;
  
//   if($ncalc>=3){
//     # we have more than 2 calcs from previous runs
//     my $calsort;

//     @$calsort=sort {$a->{x} <=> $b->{x}} @$cal;
//     (my $imin,my $fmin)=&ymin_in_calc($calsort);

//     printf("at imin=%ld, fmin=%s\n",$imin,$fmin);

//     if($imin==0 ){
//       $inital_points=[ {id=>$calsort->[0]->{id},type=>"old"},
// 		       {id=>$calsort->[1]->{id},type=>"old"} ];
//     }elsif($imin==$ncalc-1){
//       $inital_points=[ {id=>$calsort->[$ncalc-2]->{id},type=>"old"} ,
// 		       {id=>$calsort->[$ncalc-1]->{id},type=>"old"} ];
//       $sc=1;
//     }else{

//       my $i0=-1;
//       my $i1=-1;
//       for(my $i=$imin-1;$i>=0;$i--){
// 	if($calsort->[$i]->{erg}>$fmin){
// 	  $i0=$i;
// 	  last;
// 	}
//       }
//       for(my $i=$imin+1;$i<$ncalc;$i++){
// 	if($calsort->[$i]->{erg}>$fmin){
// 	  $i1=$i;
// 	  last;
// 	}
//       }
//       if($i0 >= 0 && $i1 >=0){
// 	# we have already a braket
// 	$inital_points=[
// 			{id=>$calsort->[$i0]->{id},type=>"old"},
// 			{id=>$calsort->[$imin]->{id},type=>"old"},
// 			{id=>$calsort->[$i1]->{id},type=>"old"}
// 		];
// 	$sc=1;
//       }elsif($i0>=0){
// 	# it is a rare case of a plateau
// 	$inital_points=[ {id=>$calsort->[$imin]->{id},type=>"old"} 
// 			 ,{id=>$calsort->[$i0]->{id},type=>"old"} ];
//       }elsif($i1>=0){
// 	# it is a rare case of a plateau
// 	$inital_points=[ {id=>$calsort->[$imin]->{id},type=>"old"} 
// 			 ,{id=>$calsort->[$i1]->{id},type=>"old"} ];
//       }else{
// 	# it is a rare case of a plateau
// 	$inital_points=[ {id=>$calsort->[$imin]->{id},type=>"old"} 
// 			 ,{id=>$calsort->[$imin-1]->{id},type=>"old"} ];
//       }
//     }

//   }elsif($ncalc==2){
//     $inital_points=[ {id=>$cal->[0]->{id},type=>"old"} ,
// 		     {id=>$cal->[1]->{id},type=>"old"} ];
//     $sc=1 if($cal->[0]->{erg}>$cal->[1]->{erg});
//   }else{
//     if( $#{$control->{guess}->[0]} !=2 ){
//       die("you need to specify an initial guess in the form\n".
// 	  "guess: [x0 x1 x2]\n".
// 	  "preferably such that the minimum is bracketed\n");
//     }
//     for(my $i=0;$i<=$#{$control->{guess}->[0]};$i++){
//       $inital_points->[$i]={x=>$control->{guess}->[0]->[$i],type=>"guess"};
//     }
//     $sc=1;
//   }

//   my $idsc;
// #     my $x=$control->{guess}->[0]->[1];
// #     my $idsc=&register_calc($control,$x);
// #     &run($control);
// #     my $fx=&getById($control,$idsc)->{erg};
// #     printf("$marker start_x=%s f=%s\n",$x,$fx);
// #     my $idx=$idsc;
    
    
// #     my $a=$control->{guess}->[0]->[0];
// #     my $ida=&register_calc($control,$a,$idsc);
// #     &run($control);
// #     my $fa=&getById($control,$ida)->{erg};
// #     printf("$marker x      =%s f=%s\n",$x,$fx);
// #     printf("$marker a      =%s f=%s\n",$a,$fa);
    
    
    
// #     my $inital_points=[ {id=>$ida,type=>"old"} ,
// # 			{id=>$idx,type=>"old"} ];
  



//   if( $#$inital_points>=0){


//     my $x;
//     if($inital_points->[$sc]->{type} eq "guess" ){
//       $x=$inital_points->[$sc]->{x};
//     }else{
//       $x=&getById($control,$inital_points->[$sc]->{id})->{x};
//     }
//     $idsc=&register_calc($control,$x);
    
//     &run($control);
//     #$inital_points->[$sc]->{id}=$idsc;
//     $inital_points->[1]->{id}=$idsc;

//     my $x0=$x;

//     $x=$x0*(1.+$tol*3);
//     my $id=&register_calc($control,$x,$idsc);
//     &run($control);
//     $inital_points->[2]->{id}=$id;

//     $x=$x0*(1.-$tol*3);
//     $id=&register_calc($control,$x,$idsc);
//     &run($control);
//     $inital_points->[0]->{id}=$id;
    
// #     for(my $j=0;$j<=$#$inital_points;$j++){
// #       next if($j==$sc);
// #       if($inital_points->[$j]->{type} eq "guess" ){
// # 	$x=$inital_points->[$j]->{x};
// #       }else{
// # 	$x=&getById($control,$inital_points->[$j]->{id})->{x};
// #       }
// #       my $id=&register_calc($control,$x,$idsc);
      
// #       &run($control);
// #       $inital_points->[$j]->{id}=$id;
      
// #     }
    
    
    
//     print "have inital_points\n";
//     for(my $i=$#$cal;$i>=0;$i--){
//       #print  "id=$cal->[$i]->{id}\n";
//       my $haveit=0;
//       foreach my $b (@{$inital_points}){ 
// 	if($b->{id} == $cal->[$i]->{id}){ $haveit=1; last; }
//       }
//       #print  "haveit\n" if ($haveit);
//       #$cal->[$i]->{status}="new" if ($haveit);
//       splice(@$cal,$i,1) if (!$haveit);
//     }
//     &deleteUnusedDirs($control);
//     &write_status($control,"calculations");
//     &write_status($control,"calculations",$stat_file);
//     &report_results($control);


    
  
//     $braket=&braket_min_1d( { control       =>$control
// 			      ,idsc         =>$idsc
// 			      ,inital_points=>$inital_points
// 			      ,GMAX         =>100
// 			      ,STEPMAX      =>1
// 			    });
//   }else{
//     die("no inital_points!?");
//   }
  
//   &brent({control=>$control,braket=>$braket,
// 	  tol=>$control->{minimizer_xtol},idsc=>$idsc});

  
//   printf("$marker END\n");
//   return;

// }


  if(vlevel>=Vlevel::All) 
    printf("END\n");
  return 0;
}



/*EOF*/ 
