/*************************************************************************
 * @(#)MethodControl.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 Dec 2011
 *************************************************************************
 */

#include "Assorted.hh"
#include "Except.hh"
#include "MethodControl.hh"
#include "Config.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */

static void readBracket(StringList &sl,long &startindex,
			const String &text,const char *elnames[],
			double values[])
{
  if(startindex>=sl.size()|| sl[startindex++]!="[")
    throw Except(Sprintf("%s: missing '['\n",text())());
  
  
  for(int j=0;j<3;j++){
    try{
      if(startindex>=sl.size()) throw Except("missing number");
      values[j]=sl[startindex++].convertToFortranReal();
    }catch(Except &ex){
      throw Except(Sprintf("%s-%s: %s\n",text(),elnames[j],ex.msg())());
    }
  }

  if(startindex>=sl.size()||sl[startindex++]!="]")
    throw Except(Sprintf("%s: missing ']'\n",text())());
  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void readList(StringList &sl,long &startindex,
		     const String &text,Variable *v)
{
  const char *elnames[]={"minvalue","maxvalue","stepwidth"};
  if(startindex>=sl.size()|| sl[startindex++]!="{")
    throw Except(Sprintf("%s: missing '{'\n",text())());

  if(startindex>=sl.size()|| sl[sl.size()-1]!="}")
    throw Except(Sprintf("%s: missing '}'\n",text())());

  long lastindex=sl.size()-2;
  long i=startindex;
  while(i<=lastindex){
    if(sl[i]=="["){
      double d[3];
      readBracket(sl,i,text+" interval",elnames,d);
      v->values().append(Value(d));
    }else{
      double d;
      try{
	d=sl[i++].convertToFortranReal();
      }catch(Except &ex){
	throw Except(Sprintf("%s: %s\n",text(),ex.msg())());
      }
      v->values().append(Value(d));
    }
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MethodControl::MethodControl()
  :pcontrolfile("=.min"),
   pmethod(MinEnums::None),
   plongnames(true),
   psleeptime(5),
   pcalculations(new Calculations(this)),
   pstoprunning(false),
   pdontrun(false),
   prestartunfinished(false),
   prestart(false),
   prefreshinput(false),
   pdeleteunused(false),
   pnewsimplex(false),ppermanentsimplex(false),
   pforcefiletransfer(false),
   pminimizer_xtol(5.0e-3)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MethodControl::~MethodControl()
{
  if(pcalculations) delete pcalculations; pcalculations=0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MethodControl::readControlFile()
{
  
  const char *elnames[]={"lower-bound","guessed-minimum","upper-bound"};
  
  pjobinterface.clear();
  pperjobshellcommands.clear();
  pcrunchcontent="";
  ppipecontent.clear();
  preport.clear();
  FILE *FH=NULL;
  FH=openforread(pcontrolfile);
  
  bool gotvars=false;
  bool injob=false;
  bool inperjob=false;
  bool incrunch=false;
  bool inpipe=false;
  bool inreport=false;
  bool invars=false;

  String line;
  while(getLine(FH,line)){
    line.stripBlanks(true);

    if(injob){
      // -------------- job information section ---------------
      if(getTag(line,"endjob")){
	injob=false;
	continue;
      }
      if(getFromKey(line,"fplo",pjobinterface.pfplo)) continue;
      if(getFromKey(line,"runfplo",pjobinterface.prunfplo)) continue;
      if(getFromKey(line,"fedit",pjobinterface.pfedit)) continue;
      if(getFromKey(line,"time",pjobinterface.pallotedtime)) continue;
      if(getFromKey(line,"memory",pjobinterface.pallotedmemory)) continue;
      if(getFromKey(line,"name",pjobinterface.pjobname)) continue;
      if(getFromKey(line,"method",pjobinterface.pmethod)) continue;
      if(getFromKey(line,"copydata",line)){
	if(line.lowerCase().find("t")==0){
	  pjobinterface.pcopy=true;
	}else{
	  pjobinterface.pcopy=false;
	}
	continue;
      }
      if(getFromKey(line,"deleteonnode",line)){
	if(line.lowerCase().find("t")==0){
	  pjobinterface.pdeleteonnode=true;
	}else{
	  pjobinterface.pdeleteonnode=false;
	}
	continue;
      }
      if(getFromKey(line,"forceinitialspinsplit",line)){
	if(line.lowerCase().find("t")==0){
	  pjobinterface.pforce_initial_spin_split=true;
	}else{
	  pjobinterface.pforce_initial_spin_split=false;
	}
	continue;
      }
      if(getFromKey(line,"forceconvergence",line)){
	if(line.lowerCase().find("t")==0){
	  pjobinterface.pforce_convergence=true;
	}else{
	  pjobinterface.pforce_convergence=false;
	}
	continue;
      }
    }else if(inperjob){
      // -------------- per-job-shell-commands section ---------------
      if(getTag(line,"endperjob")){
	inperjob=false;
	continue;
      }
      pperjobshellcommands.append(line);
      continue;
    }else if(incrunch){
      // -------------- crunch section ---------------
      if(getTag(line,"endcrunch")){
	incrunch=false;
	continue;
      }
      pcrunchcontent+=line+"\n";
      continue;
    }else if(inpipe){
      // -------------- pipe section ---------------
      if(getTag(line,"endpipe")){
	inpipe=false;
	continue;
      }
      ppipecontent.append(line);
      continue;

    }else if(inreport){
      // -------------- report section ---------------
      if(getTag(line,"endreport")){
	inreport=false;
	continue;
      }
      preport.append(line);
      continue;

    }else if(invars){
      // -------------- variables section ---------------
      if(getTag(line,"endvars")){
	invars=false;
	gotvars=true;
	pvariables.check(pmethod);
	pvariables.print(stdout,pmethod);
	continue;
      }
      if(line[0]!='#' && line.length()){
	// we have some content
	line.replace("["," [ ",0,true).replace("]"," ] ",0,true)
	  .replace("{"," { ",0,true).replace("}"," } ",0,true);
	StringList sl=line.split(" \t");
	long i=0;
	Variable *v=pvariables.appendNew();
	v->setName(sl[i++]);
	if(i>=sl.size())continue;
	v->setXformat(sl[i++]);
	if(pmethod==MinEnums::Minimize){
	  readBracket(sl,i,Sprintf("variable %s: solution-bracket",
				   v->name()()),elnames,v->bracket());
	  try{
	    if(i>=sl.size()) throw Except("missing number");
	    v->setMaxStep(sl[i++].convertToFortranReal());
	  }catch(Except &ex){
	    throw Except(Sprintf("variable %s: maxstep: %s\n",
				 v->name()(),ex.msg())());
	  }
	}else if(pmethod==MinEnums::Curve){
	  readList(sl,i,Sprintf("variable %s: value-list",v->name()()),v);
	}
      }
      continue;

    }else{
      // ---------------- zeroth level commands ------------------
      
      if(getFromKey(line,"method",line)) {
	pmethod=MinEnums::methodFromString(line);
	continue;
      }
      String s;
      if(getFromKey(line,"longnames",s)) {
	s.toLower();
	if(s=="t" || s=="true")
	  plongnames=true;
	else
	  plongnames=false;
	continue;
      }
      if(getTag(line,"job")){
	//enter job information section
	injob=true;
	pjobinterface.clear();
	continue;
      }
      if(getTag(line,"perjob")){
	//enter per-job-shell-commands information section
	inperjob=true;
	pperjobshellcommands.clear();
	continue;
      }
      if(getTag(line,"crunch")){
	//enter crunch section
	incrunch=true;
	pcrunchcontent="";
	continue;
      }
      if(getTag(line,"pipe")){
	//enter pedit ipe section
	inpipe=true;
	ppipecontent.clear();
	continue;
      }
      if(getTag(line,"report")){
	//enter report section
	inreport=true;
	preport.clear();
	continue;
      }
      if(getTag(line,"vars")){
	//enter variables section
	invars=true;
	pvariables.clear();
	continue;
      }
    }  // end zeroth level commands  


    if(line[0]!='#' && line.length()){
      if(vlevel>=Vlevel::All)
	printf("unscanned input: %s\n",line());
    }
    
  }
  if(injob){
    throw Except("missing \"endjob:\"\n");
  }
  if(inperjob){
    throw Except("missing \"endperjob:\"\n");
  }
  if(incrunch){
    throw Except("missing \"endcrunch:\"\n");
  }
  if(inpipe){
    throw Except("missing \"endpipe:\"\n");
  }
  if(inreport){
    throw Except("missing \"endreport:\"\n");
  }
  if(invars){
    throw Except("missing \"endvars:\"\n");
  }
  if(pmethod==MinEnums::None){
    throw Except("missing \"method:\" tag\n");
  }
  if(!gotvars){
    throw Except("missing \"vars:\" section\n");
  }
  if(pvariables.size()<1){
    throw Except("no variables?\n");
  }
  if(FH) fclose(FH);

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool MethodControl::getFromKey(const String &line,const String &key,
			       String &value,char enddelim)
{
  //printf("key =%s\n",key());
  //printf("line=%s\n",line());
  if(line.find(key)==0){
    String s=line.subString(key.length());
    //printf("here1 =%s\n",s());
    s.stripBlanks();
    if(s[0]==enddelim){
      value=s.subString(1).stripBlanks(true);
      //printf("here2 =%s\n",value());
      return true;
    }
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool MethodControl::getFromKey(const String &line,const String &key,
			       long &value,char enddelim)
{
  String s;
  bool ret=getFromKey(line,key,s,enddelim);
  if(ret){
    value=s.convertToLong();
  }
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool MethodControl::getTag(const String &line,const String &tag)
{
  if(line.find(tag)==0){
    String s=line.subString(tag.length());
    s.stripBlanks();
    if(s[0]==':'){
      return true;
    }
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MethodControl::writeControlFile(const String &fname)
{
  const char *separator="\
########################################################################";

  FILE *FH=NULL;
  try{
    FH=openforwrite(fname);
  }catch(Except &ex){
    throw Except(ex.msg());
  }

  

  // job interface information
  pjobinterface.writeToFile(FH);

  fprintf(FH,"\n\nlongnames: %s\n",(plongnames?"true":"false"));
  
  
  // report section
  fprintf(FH,"\nreport:%sendreport:\n\n",preport.join("\n",true,true)());


  // method specificator
  fprintf(FH,"method: %s\n",MinEnums::methodGuiName(pmethod));


  //the variables
  pvariables.writeToFile(FH,pmethod);

  // per-job-shell-commands section
  fprintf(FH,"perjob:%sendperjob:\n",pperjobshellcommands.join("\n",true,true)());
  
  // crunch section
  fprintf(FH,"\n\n\n%s\ncrunch:\n%sendcrunch:\n\n\n",
	  separator,pcrunchcontent());


  // pipe section
  fprintf(FH,"\n%s\npipe:%sendpipe:\n",
	  separator,ppipecontent.join("\n",true,true)());


  if(FH) fclose(FH);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MethodControl::setDefault(MinEnums::Method whichmethod)
{


  pmethod=whichmethod;

  pjobinterface.setDefault();
  pperjobshellcommands.clear();
  pcrunchcontent="";
  ppipecontent.clear();
  preport.clear();
  


  ppipecontent.append("# reset to default input");
  ppipecontent.append("@ e@");
  ppipecontent.append("");
  ppipecontent.append("#symmetry menu");
  ppipecontent.append("@+@");
  ppipecontent.append("");
  ppipecontent.append("@s@");
  ppipecontent.append("@225@");
  ppipecontent.append("@x@");
  ppipecontent.append("");
  ppipecontent.append("@u@");
  ppipecontent.append("!bohr!");
  ppipecontent.append("@x@");
  ppipecontent.append("");
  ppipecontent.append("@l@$a0,$c0");
  ppipecontent.append("");
  ppipecontent.append("@n@1");
  ppipecontent.append("@1@Al@0,,");
  ppipecontent.append("");
  ppipecontent.append("# update");
  ppipecontent.append("@+@");
  ppipecontent.append("@x@");
  ppipecontent.append("");
  ppipecontent.append("#main menu");
  ppipecontent.append("");
  ppipecontent.append("#k-points");
  ppipecontent.append("@k@$nkx,$nkz");
  ppipecontent.append("");
  ppipecontent.append("#iterat");
  ppipecontent.append("@ t@");
  ppipecontent.append("@v@");
  ppipecontent.append("!LCiterat!");
  ppipecontent.append("@x@");
  ppipecontent.append("@x@");
  ppipecontent.append("");
  ppipecontent.append("");
  ppipecontent.append("# quit");
  ppipecontent.append("@q@");
  preport.append("total energy");

  if(pmethod==MinEnums::Minimize){
    
    
    pcrunchcontent+="nkx=12\n";
    pcrunchcontent+="nkz=12\n";
    if(pmethod==MinEnums::Curve){
      pcrunchcontent+="coa=1.\n";
    }
    pcrunchcontent+="a0=(V/coa)^(1/3)\n";
    pcrunchcontent+="c0=coa*a0\n";
    
    Variable *v=pvariables.appendNew();
    v->setName("V");
    v->setXformat("%010.5f");
    v->bracket()[0]=100;
    v->bracket()[1]=110;
    v->bracket()[2]=120;
    v->setMaxStep(5);
    
    v=pvariables.appendNew();
    v->setName("coa");
    v->setXformat("%010.5f");
    v->bracket()[0]=0.9;
    v->bracket()[1]=1.0;
    v->bracket()[2]=1.1;
    v->setMaxStep(0.08);
  }else if(pmethod==MinEnums::Curve){
    pcrunchcontent+="nkx=12\n";
    pcrunchcontent+="nkz=12\n";
    if(pmethod==MinEnums::Curve){
      pcrunchcontent+="coa=1.\n";
    }
    pcrunchcontent+="a0=(4*V/coa)^(1/3)\n";
    pcrunchcontent+="c0=coa*a0\n";
    
    Variable *v=pvariables.appendNew();
    v->setName("V");
    v->setXformat("%010.5f");
    double d[3];
    d[0]=100;
    d[1]=115;
    d[2]=5;
    v->values().append(Value(d));
  }


  pvariables.print(stdout,pmethod);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MethodControl::doCalculations()
{




  if(pmethod==MinEnums::Curve){
    printf("curve\n");

    pcalculations->readStatusFromFile();
    //myDebug()<<*pcalculations;
    pcalculations->analyseDirectories();

    if(prefreshinput) pcalculations->attemptRefresh();


    //    pcalculations->initCheckCalculations();
    
    //    &make_EX_script($control);
    
    
    // we load the calculations
    GridLooper gridlooper(&pvariables);
    // this->pvariables->currentValues() contains the current N-tuple
    long norder=0;
    while(gridlooper.nextNtuple()){
      //printf("\t%s\n",gridlooper.printSCounter()());
      //printf("<%s>\n",pvariables.stringify()());
      pcalculations->registerCalculation(norder++);
    }
    pcalculations->sortCalculationOrder();

    pcalculations->writeStatusToFile();

    if(pdeleteunused){
      myDebug()<<*pcalculations;
      pcalculations->deleteUnusedDirs();
      return;
    }
    
    if(pstoprunning) {
      //pcalculations->stopRunningJobs();
      if(pcalculations->stopRunningJobs()){
	throw Except("\nSome jobs where stopped, you should make sure \n"
		     "that all calcs are sound and re-run the program or\n"
		     "use --restart\n");
      }
    }
    if(pforcefiletransfer){
      pcalculations->forceFileTransfer();
      return;
    }

    if(prestart) pcalculations->restartJobs();
    if(prestartunfinished) pcalculations->restartJobs(true);
    


    //throw Except("debug");
    pcalculations->run();

    
  }else if(pmethod==MinEnums::Minimize && pvariables.size()==1){
    
    pcalculations->readStatusFromFile();

    if(pstoprunning) {
      if(pcalculations->stopRunningJobs()){
	throw Except("\nSome jobs where stopped, you should make sure \n"
		     "that all calcs are sound and re-run the program or\n"
		     "use --restart\n");
      }
    }

    if(prefreshinput) {
      pcalculations->attemptRefresh(true);
    }

    if(prestart) pcalculations->restartJobs();
    


    pcalculations->run();

    pcalculations->minimize1D();
    
  }else if(pmethod==MinEnums::Minimize && pvariables.size()>1){
    
    pcalculations->readStatusFromFile();

    if(pstoprunning) {
      if(pcalculations->stopRunningJobs()){
	throw Except("\nSome jobs where stopped, you should make sure \n"
		     "that all calcs are sound and re-run the program or\n"
		     "use --restart\n");
      }else{
	throw Except("\tNo more running jobs.\n");
      }
    }

    if(prefreshinput) {
      pcalculations->attemptRefresh(true);
    }

    if(prestart) pcalculations->restartJobs();

    pcalculations->minimizeSimplex();

    //pcalculations->minimizeMulti1D();
    
  }else {
    //pcalculations->readStatusFromFile();
  }

}



/*EOF*/ 
