/*************************************************************************
 * @(#)Variables.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   02 Dec 2011
 *************************************************************************
 */

//#include "math.h"
#include "Variables.hh"
#include "Except.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Variable *Variables::appendNew()
{
  pvariables.resize(pvariables.size()+1);
  return &(pvariables.last());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Variables::print(FILE *FH,MinEnums::Method method)
{
  for(int i=0;i<pvariables.size();i++){
    const Variable &v=pvariables.at(i);
    printf(
	   "variable%d: name    : %s\n"
 	   "           prefix  : %s\n"
 	   "           x-format: %s\n",
	   i+1,v.name()(),v.prefix()(),v.xformat()());
    if(method==MinEnums::Minimize){
      printf(
	     "           solution-bracket: [%g %g %g]\n",
	     v.bracket()[0],v.bracket()[1],v.bracket()[2]
	     );
      printf(
	     "           maxstep : %g\n",
	     v.maxStep()
	     );
    }else if(method==MinEnums::Curve){
      printf("           value:");
      for(int i=0;i<v.values().size();i++){
	const Value &va=v.values().at(i);
	printf(" %s",va.printS()());
      }
      printf("\n");
    }

  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Variables::check(MinEnums::Method method)
{
  for(int i=0;i<pvariables.size();i++){
    const Variable &v=pvariables.at(i);
    if(!v.name().length()){
      throw Except(Sprintf("variable%d has no name\n",i+1)());
    }
    for(int j=i+1;j<pvariables.size();j++){
      const Variable &vj=pvariables.at(j);
      if(v.name().lowerCase()==vj.name().lowerCase()){
	throw Except(Sprintf("variable%d %s and variable%d %s have "
			     "the same name\n",
			     i+1,v.name()(),j+1,vj.name()())());
      }
    }
    if(!v.xformat().length()){
      throw Except(Sprintf("variable %s has no x-format\n",v.name()())());
    }
    if(method==MinEnums::Minimize){
      if(v.bracket()[0]>=v.bracket()[1])
	throw Except(Sprintf("variable %s: solution-bracket: lower-bound>"
			     "guessed-minimum\n",
			     v.name()())());
      if(v.bracket()[1]>=v.bracket()[2])
	throw Except(Sprintf("variable %s: solution-bracket: guessed-minimum>"
			     "upper-bound\n",
			     v.name()())());
      if(v.maxStep()==0.)
	throw Except(Sprintf("variable %s: maxstep=0\n",
			     v.name()())());
      
	
    }else if(method==MinEnums::Curve){
      if(v.values().size()==0){
	throw Except(Sprintf("variable %s: no values?\n",
			     v.name()())());
      }
      for(int i=0;i<v.values().size();i++){
	const Value &va=v.values().at(i);
	String msg;
	if(!va.check(msg))
	  throw Except(Sprintf("variable %s: values: range %s: %s\n",
			       v.name()(),va.printS()(),msg())());
	
      }
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Variables::writeToFile(FILE *FH,MinEnums::Method method)
{
  fprintf(FH,"\n\nvars:\n");
  if(method==MinEnums::Minimize){
    fprintf(FH,"# name  format solution-bracket    maxstep\n");
  }else if(method==MinEnums::Curve){
    fprintf(FH,"# name format  list-of-values-and-ranges\n");
  }

  for(int i=0;i<pvariables.size();i++){
    const Variable &v=pvariables.at(i);
    fprintf(FH,"%s %s ",v.name()(),v.xformat()());
     if(method==MinEnums::Minimize){
       fprintf(FH,"[%g %g %g] %g",
	       v.bracket()[0],v.bracket()[1],v.bracket()[2],v.maxStep()
	       );
     }else if(method==MinEnums::Curve){
       fprintf(FH,"{");
       for(int i=0;i<v.values().size();i++){
	 const Value &va=v.values().at(i);
	 fprintf(FH,"%s",va.printS()());
	 if(i<v.values().size()-1)
	   fprintf(FH," ");
       }
       fprintf(FH,"}");
     }
     fprintf(FH,"\n");
     
  }
  fprintf(FH,"endvars:\n\n");
}


/*EOF*/ 
