/* @(#)Cmplx.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _CMPLX_H
#define _CMPLX_H 1

#include <math.h>
#include "MyMath.hh"

class MyDebug;
class String;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Cmplx{
 public:
  Cmplx():re(0.),im(0.){};
  Cmplx(double r):re(r),im(0.){};
  Cmplx(double r,double i):re(r),im(i){};

  static const Cmplx ci;
  static const Cmplx c1;

  inline bool isReal(double tol=1.0e-10)const{
    return (fabs(im)<tol);
  };
    
  Cmplx(const Cmplx &other){
    re=other.re;
    im=other.im;
  };

  inline bool operator==(double other)const{
    if(re==other && im==0)return true;
    return false;
  }

  inline bool operator==(const Cmplx &other)const{
    if(re==other.re && im==other.im)return true;
    return false;
  }

  inline bool operator!=(const Cmplx &other)const{
    if(re!=other.re || im!=other.im)return true;
    return false;
  }

  inline const Cmplx &operator=(const Cmplx &other){
    re=other.re;
    im=other.im;
    return *this;
  };

  inline const Cmplx &operator=(double r){
    re=r;
    im=0.;
    return *this;
  }

  inline Cmplx &operator+=(const Cmplx &other){
    re+=other.re;
    im+=other.im;
    return *this;
  }

  inline Cmplx &operator-=(const Cmplx &other){
    re-=other.re;
    im-=other.im;
    return *this;
  }


  inline Cmplx &operator*=(const Cmplx &other){
    // double d=other.re*(re+im);
    // re=d-im*(other.im+other.re);
    // im=d+re*(other.im-other.re);
    double d=re*other.re-im*other.im;
    im=re*other.im+im*other.re;
    re=d;
    return *this;
  }

  inline Cmplx operator-()const{
    return Cmplx(-re,-im);
  }

  inline Cmplx &operator/=(const Cmplx &other){
    double d=other.normsq();
    double r=(re*other.re+im*other.im)/d;
    im=(im*other.re-re*other.im)/d;
    re=r;
    return *this;
  }

  inline Cmplx &conjugate(){
    im=-im;
    return *this;
  }
  
  inline double real() const{return re;};
  inline double imag() const{return im;};

  inline double &real(){return re;};
  inline double &imag(){return im;};

  //  inline operator double ()const{return re;}


  inline double normsq() const{return re*re+im*im; };
  inline double abs() const{return sqrt(re*re+im*im); };

  String toString(const char *fmt="%12.5e")const;

  friend Cmplx exp(const Cmplx &other);
  friend Cmplx power(const Cmplx &other,long n);
  friend Cmplx log(const Cmplx &other);
  friend Cmplx sqrt(const Cmplx &other);
  friend Cmplx conjg(Cmplx a);

 private:
  double re,im;
};

inline Cmplx operator+( Cmplx lhs, const Cmplx & rhs ) {
  lhs += rhs;
  return lhs;
}

inline Cmplx operator-( Cmplx lhs, const Cmplx & rhs ) {
  lhs -= rhs;
  return lhs;
}

inline Cmplx operator*( Cmplx lhs, const Cmplx & rhs ) {
  lhs *= rhs;
  return lhs;
}

inline Cmplx operator/( Cmplx lhs, const Cmplx & rhs ) {
  lhs /= rhs;
  return lhs;
}

Cmplx sqrt(const Cmplx &c);

inline Cmplx exp(const Cmplx &c){
  double d=::exp(c.re);
  return Cmplx(d*::cos(c.im),d*::sin(c.im));
}

inline Cmplx power(const Cmplx &z,long n){
  return exp(n*log(z));
}


inline Cmplx conjg(Cmplx a)
{
  a.im=-a.im;
  return a;
}

inline long conjg(const long &a)
{
  return a;
}

inline double conjg(const double &a)
{
  return a;
}

inline int conjg(const int &a)
{
  return a;
}

// override Abs from MyMath for Array purposes
namespace MyMath{
  template <>
  inline Cmplx Abs(const Cmplx &a){
    return a.abs();
  }
  template <>
  inline Cmplx Abs2(const Cmplx &a){
    return a.normsq();
  }
  template <>
  inline double toDouble(const Cmplx &a){
    return a.real();
  }
  
  double ClebschGordonChi(long kap,long mu,long s);

  void zrootstester();
  void zroots(const Cmplx *a,long m,Cmplx *roots,bool polish);

}

MyDebug operator<<(MyDebug dbg,const Cmplx &d);

#define CXREIM(c) ((c).real()),((c).imag())

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _CMPLX_H */

