/* @(#)OutGrep.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OUTGREP_H
#define _OUTGREP_H 1

#include "FString.hh"
#include "MyDebug.hh"
 

template <class T>
class ValueArray;

class OutGrep{
 public:

  class SiteInfo{
  public:
    SiteInfo():site(-1),el("??"){};
    SiteInfo(const String &e,long s):site(s),el(e){};
      
    static String getElementfromSite(const ValueArray<SiteInfo>&sites,
				     long site);

    long site;
    String el;
  };

  enum Mode{Iteration=1,IterationBest=13,TotalEnergy=2,TotalSpin=3,CycleTime=4,
	    AtomSpin=50,AtomSpinGros=51,AtomSpinNet=52,
	    AtomOccuGros=61,AtomOccuNet=62,
	    AtomEFG=7,Pressure=8,Bfsm=9,dBfsm=10,Termination=11,
	    ForceIteration=12,AtomOrbitalMoment=70
  };

  OutGrep();
  OutGrep(const String &outfile);
  ~OutGrep(){clear();};
  void clear();
  void setOutfile(const String &outfile){
    poutfile=outfile;
    readFile();
  };
  void setSilentOpen(bool s){psilentopen=s;}
  void setSite(long site){psite=site;};
  void setOrbital(long o){porbital=o;};
  // grep for all iterations
  void grep(Mode mode,ValueArray<double> &x,ValueArray<double> &y);
  void grep(Mode mode,ValueArray<double> &x,ValueArray<String> &y);
  // grep for last iteration, return true if found
  bool grepLast(Mode mode,double &);
  bool grepLast(Mode mode,String &);
  //const StringList &content()const{ return pcontent; };

  ValueArray<SiteInfo> extractSites();


  static Mode modeFromString(const char *mode);
  // return generic mode name
  static const char *modeName(Mode mode,bool alternativestring=false);
  // return specific mode name (with arguments (e.g. "SSat 5") included)
  String modeString(Mode mode,bool alternativestring=false);

 private:
  void readFile();
  void grepinternal(Mode mode,ValueArray<double> &x,ValueArray<double> &yd,
		    ValueArray<String> &ys,bool asstring);

  struct tsModes{
    const char *name;
    Mode mode;
    const char *altname;
  };
  static tsModes modes[];
  static long maxmodes;

 private:
  String poutfile;
  char *pbuffer;
  char **plines;
  long plinessize;
  long pnlines;
  long psite;
  long porbital;
  bool psilentopen;
};

inline MyDebug operator<<(MyDebug dbg,const OutGrep::SiteInfo &){
  dbg<<"SiteInfo";
  return dbg;
}


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OUTGREP_H */

