/* @(#)PtrArray.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PTRARRAY_H
#define _PTRARRAY_H 1

//#include <stdio.h> 
//#include <stdlib.h> 
#include "Except.hh"
#include "MyDebug.hh"


#define PTRARRAYDEBUG 
#undef  PTRARRAYDEBUG 

/* ***********************************************************************
 * class to store pointer, automatic deletion of ptr members on destruction
 * 
 * ***********************************************************************
 */
template <class T>

class PtrArray
{
 public:

  // ----------------------------------------------------------------
  PtrArray(bool autodelete=true):psize(0),array(0),pautodelete(autodelete){};

  // ----------------------------------------------------------------
  ~PtrArray(){clear();};

  // ----------------------------------------------------------------
  PtrArray(const PtrArray &other){
    // Copy not only pointers but the stuff pointed to., requires clone method in T
    psize=other.psize;
    pautodelete=other.pautodelete;
    array=new T*[psize];
    for(long i=0; i<psize; i++) {
      array[i]=other.array[i]->clone();
    }
  }

  // ----------------------------------------------------------------
  const PtrArray &operator=(const PtrArray &other){
    // Copy not only pointers but the stuff pointed to.
    clear();
    psize=other.psize;
    pautodelete=other.pautodelete;
    array=new T*[psize];
    for(long i=0; i<psize; i++) {
      //myDebug()<<"cloning array["<<i<<"]"<<other.array[i]->printS()();
      array[i]=other.array[i]->clone();
    }
    return *this;
  }

  // ----------------------------------------------------------------
  void setAutoDelete(bool a){pautodelete=a;};

  // ----------------------------------------------------------------
  void clear(){
#ifdef PTRARRAYDEBUG
    printf("clear \n");
#endif
    for(long i=0;i<psize;i++){
      if(pautodelete && array[i]){
#ifdef PTRARRAYDEBUG
	printf("\tdelete array[%ld]=%p\n",i,array[i]);	
#endif
	delete array[i];
      }
    }
    if(array) delete [] array;
    array=0;
    psize=0;
  };

  // ----------------------------------------------------------------
  long size() const{return psize;};

  // ----------------------------------------------------------------
  const T *at(long index)const{
#ifdef MY_DEBUGGING
    if(index<0 || index>=psize){
      throw Except("PtrArray::at: index out of bounds\n");
    }
#endif
    return array[index];
  };
  
  // ----------------------------------------------------------------
  T *&operator[](long index){
#ifdef MY_DEBUGGING
    if(index<0 || index>=psize){
      throw Except("PtrArray:operator[]: index out of bounds\n");
    }
#endif
    return array[index];
  };

  // ----------------------------------------------------------------
  void resize(long newsize,bool shrink=false){
    // the array grows to have at least size newsize
    // or shrinks to newsize
    // new elements are allocated so calling resize 
    // makes the new elments available
    if(newsize==psize) return;
    if((!shrink) && (newsize<psize)) return;
    
    if(newsize>0){
      T* *temp=new T*[newsize];
      if(newsize>psize){
	for(long i=0; i<psize; i++) temp[i] = array[i];	
	for(long i=psize; i<newsize; i++) temp[i] = new T();
      }else{
	for(long i=0; i<newsize; i++) temp[i] = array[i];	
	for(long i=newsize; i<psize; i++) {
	  if(pautodelete &&  array[i]) delete array[i];
	}
      }
      swap(temp);
      if(temp) delete [] temp;
    }else{
      for(long i=0; i<psize; i++) {
	if(pautodelete &&  array[i]) delete array[i];
      }
      delete [] array;
      array=0;
    }
    psize=newsize;
  };

  // ----------------------------------------------------------------
  long indexOf(const T *t)const{
    for(long i=0;i<psize;i++){
      if(array[i]==t)
	return i;
    }
    return -1;
  };

  // ----------------------------------------------------------------
  long indexOf(const T &t)const{
    for(long i=0;i<psize;i++){
      if(*(array[i])==t)
	return i;
    }
    return -1;
  };

  // ----------------------------------------------------------------
  T** dataRef()const{
    return array;
  }

  // ----------------------------------------------------------------
  const T *last()const{
#ifdef MY_DEBUGGING
    if(psize<1){
      throw Except("PtrArray: no elements\n");
    }
#endif
    return array[psize-1];
  }

  // ----------------------------------------------------------------
  T *last(){
#ifdef MY_DEBUGGING
    if(psize<1){
      throw Except("PtrArray: no elements\n");
    }
#endif
    return array[psize-1];
  }

  // ----------------------------------------------------------------
  PtrArray &operator<<(T *v){
    this->append(v);
    return *this;
  }


  // ----------------------------------------------------------------
  T *append(T *l){
#ifdef PTRARRAYDEBUG
    printf("append %p\n",l);
#endif
    insert(psize,l);
    return l;
  };

  // ----------------------------------------------------------------
  void insert(long index,T *l){
#ifdef PTRARRAYDEBUG
    printf("insert %p at %ld\n",l,index);
#endif
    index=(index<0?0:(index>=psize?psize:index));
    psize++;
    T* *temp=new T*[psize];
    for(long i=0; i<index; i++) {
      temp[i] = array[i];
#ifdef PTRARRAYDEBUG
      printf("\ttemp[%ld]=%p\n",i,temp[i]);
#endif
    }
    temp[index]=l;
#ifdef PTRARRAYDEBUG
    printf("\ttemp[index=%ld]=%p\n",index,temp[index]);
#endif
    for(long i=index+1; i<psize; i++) {
      temp[i] = array[i-1];
#ifdef PTRARRAYDEBUG
      printf("\ttemp[%ld]=%p\n",i,temp[i]);
#endif
    }
    swap(temp);
    if(temp) delete [] temp;
    for(long i=0; i<psize; i++) {
#ifdef PTRARRAYDEBUG
      printf("\tarray[%ld]=%p\n",i,array[i]);
#endif
    }
  };

  // ----------------------------------------------------------------
  T *take(T *l){return removeInternal(l,false);};

  // ----------------------------------------------------------------
  void remove(T *l){removeInternal(l,pautodelete);};

  // ----------------------------------------------------------------
  T *takeAt(long j){return removeAtInternal(j,false);};

  // ----------------------------------------------------------------
  void removeAt(long j){removeAtInternal(j,pautodelete);};


  typedef int (*sortCallBackType)(const T* *p1, const T* *p2) ;


  void qSort(sortCallBackType sortFunc){
    qsort(array,psize,sizeof(T*),(int (*)(const void *p1, const void *p2) )sortFunc);
  }
  

 private:

  // ----------------------------------------------------------------
  T *removeInternal(T *l,bool deleteT=false){
#ifdef PTRARRAYDEBUG
    printf("removeInternal %p deleteT=%d\n",l,deleteT);
#endif
    if(!l)return l;
    long j=-1;
    for(long i=0;i<psize;i++){ 
      if(array[i] == l){ 
	j=i;
	if(deleteT && array[i]) delete array[i];
	break;
      }
    }
#ifdef PTRARRAYDEBUG
    printf("removeInternal found %p at %ld\n",l,j);
#endif

    removeElementAt(j);
    
    return l;
  }

  // ----------------------------------------------------------------
  T *removeAtInternal(long j,bool deleteT=false){
    if(j<0||j>=psize)return 0;

    T *tb=array[j];
    if(deleteT && array[j]) {
      tb=0;
      delete array[j];
    }

    removeElementAt(j);
      
    return tb;
  }
  
  // ----------------------------------------------------------------
  void removeElementAt(long j){
#ifdef PTRARRAYDEBUG
    printf("removeElementAt at %ld\n",j);
#endif
    if(j<0||j>=psize) return;

    psize--;
    T* *temp=new T*[psize];
    
    for(long i=0;i<j;i++) {
      temp[i]=array[i];
#ifdef PTRARRAYDEBUG
      printf("\ttemp[%ld]=%p\n",i,temp[i]);
#endif
    }
    for(long i=j;i<psize;i++) {
      temp[i]=array[i+1];
#ifdef PTRARRAYDEBUG
      printf("\ttemp[%ld]=%p\n",i,temp[i]);
#endif
    }
    
    swap(temp);
    if(temp) delete [] temp;
#ifdef PTRARRAYDEBUG
    for(long i=0; i<psize; i++) {
      printf("\tarray[%ld]=%p\n",i,array[i]);
    }
#endif
  };

  // ----------------------------------------------------------------
  void swap(T* *&temp){
    T* *c=array;
    array=temp;
    temp=c;
  }

 private:
  long psize;
  T* *array;
  bool pautodelete;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PTRARRAY_H */

