Module atomdefs
  !!
  Use precisiondefs
  Use constants
  Use errors
  !!
  Implicit None 
  !!
  Private
  !!
  Public :: lqntable,nqntable,lqnchar,lcharqn,getelementnname&
       &,getelementnumber,charmult,multchars,MAX_ELEMENT,isElement
  !!
  Character(*),Parameter::nmodule="ATOMDEFS"  
  !!***********************************************************************
  !! shell definitions
  !! table of angular momentum numbers in each shell
  Integer(t_int) :: lqntable(19)
  Data lqntable/ 0_t_int,  0_t_int,1_t_int,  0_t_int,1_t_int,2_t_int&
       &, 0_t_int,1_t_int,2_t_int,3_t_int, 0_t_int,1_t_int,2_t_int,3_t_int&
       &, 0_t_int,1_t_int,2_t_int, 0_t_int,1_t_int /
  !! table of main quantum number in each shell
  Integer(t_int) :: nqntable(19)
  Data nqntable / 1_t_int,  2_t_int,2_t_int,  3_t_int,3_t_int,3_t_int&
       &, 4_t_int,4_t_int,4_t_int,4_t_int, 5_t_int,5_t_int,5_t_int,5_t_int&
       &, 6_t_int,6_t_int,6_t_int, 7_t_int,7_t_int /
  !!
  !!***********************************************************************
  !! verbose angular momenta
  Character(1) :: lqnchar(0:6)
  Character(3) :: multchars
  Data lqnchar/"s","p","d","f","g","h","i"/
  Data multchars/"SDT"/
  !!
  !!***********************************************************************
  Character(300),Parameter :: ELEMENT_TABLE=&
       &"E H HeLiBeB C N O F NeNaMgAlSiP S ClArK CaScTiV CrMnFe"//&
       &"CoNiCuZnGaGeAsSeBrKrRbSrY ZrNbMoTcRuRhPdAgCdInSnSbTeI XeC"//&
       &"sBaLaCePrNdPmSmEuGdTbDyHoErTmYbLuHfTaW ReOsIrPtAuHgTlPbBi"//&
       &"PoAtRnFrRaAcThPaU NpPuAmCmBkCfEsFmMdNoLr"


  Integer(t_int),Parameter :: MAX_ELEMENT=103

Contains
  !!***********************************************************************
  Integer(t_int) Function charmult(c)
    Implicit None
    !!IO
    Character(1),Intent(in) :: c
    !!LOCAL
    Character(*),Parameter::nsubfunc="charmult"
    Integer(t_int) :: l
    !!
    l=-1_t_int
    If(c=='S') l=1_t_int
    If(c=='D') l=2_t_int
    If(c=='T') l=3_t_int
    !only up to fstates allowed
    If(l<0_t_int .Or. l> 3) Then
       Write(errbuf,"('''Multiplicity ''''',a1&
            &,''''' not implemented!''')") c
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    charmult=l
    Return
  End Function charmult
  !!***********************************************************************
  Integer(t_int) Function lcharqn(c,throwerror)
    Implicit None
    !!IO
    Character(1),Intent(in) :: c
    Logical(t_log),Intent(in),Optional    :: throwerror
    !!LOCAL
    Character(*),Parameter::nsubfunc="lcharqn"
    Logical(t_log) :: lthrow
    Integer(t_int) :: l
    !!
    lthrow=.Not.Present(throwerror)
    If(.Not.lthrow) lthrow=throwerror
    !!
    l=-1_t_int
    If(c=='s') l=0_t_int
    If(c=='p') l=1_t_int
    If(c=='d') l=2_t_int
    If(c=='f') l=3_t_int
    If(c=='g') l=4_t_int
    If(c=='h') l=5_t_int
    If(c=='i') l=6_t_int
    !only up to fstates allowed
    If(lthrow .And. l<0_t_int .Or. l> ORBITAL_MAX_L) Then
       Write(errbuf,"('''State  ''''',a1,''''' not implemented!''')") c
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    lcharqn=l
    Return
  End Function lcharqn
  !!***********************************************************************
  Integer(t_int) Function getelementnumber(el)
    !!
    Implicit None 
    !!IO
    Character(*),Intent(in) ::el
    !!LOCAL    
    Character(*),Parameter::nsubfunc="getelementnumber"
    Integer(t_int) :: i 
    getelementnumber=-1
    Do i=0_t_int,MAX_ELEMENT
       If(ELEMENT_TABLE(i*2_t_int+1_t_int:i*2_t_int+2_t_int) == el(1:2))Then
          getelementnumber=i
          Return
       Endif
    Enddo
    Write(errbuf,"('''Element   ',a2,' not implemented!''')") &
         &el(1_t_int:2_t_int)
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    
  End Function getelementnumber
  !!***********************************************************************
  Logical(t_log) Function isElement(el)
    !!
    Implicit None 
    !!IO
    Character(*),Intent(in) ::el
    !!LOCAL    
    !Character(*),Parameter::nsubfunc="isElement"
    Integer(t_int) :: i 
    Character(2)   :: c
    isElement=.False.
    If(Len_trim(el)<1.Or.Len_trim(el)>2) Return
    c=Trim(el)
    If(Len_trim(c)==1) c(2:2)=' '
    Do i=0_t_int,MAX_ELEMENT
       If(ELEMENT_TABLE(i*2_t_int+1_t_int:i*2_t_int+2_t_int) == c)Then
          isElement=.True.
          Return
       Endif
    Enddo
    !!
    Return
    !!
  End Function isElement
  !!***********************************************************************
  Character(2) Function getelementnname(n)
    !!
    Implicit None 
    !!IO
    !! nuclear charge
    Integer(t_int),Intent(in) :: n
    !!LOCAL
    Character(*),Parameter::nsubfunc="getelementnname"
    If(n<0_t_int .Or. n> MAX_ELEMENT)Then
       Write(errbuf,"('''Element number ',i5,' not implemented!''')")n
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    End If
    getelementnname=ELEMENT_TABLE(2_t_int*n+1_t_int:2_t_int*n+2_t_int)
    return
  End Function getelementnname
  !!***********************************************************************
  
End Module atomdefs
