Module atompot
  !!
  Use precisiondefs
  Use file_iounits
  Use constants
  Use errors
  Use integrate1d
  Use tmesh
  Use xc
  !!
  Implicit None 
  !!
  Private
  !!
  Public :: sicpot,potatcoul,potatxc,gam_fac_mesh
  !!
  Character(*),Parameter::nmodule="ATOMPOT"
  !!***********************************************************************
Contains
  !!***********************************************************************
  !!     
  !!     recalculate the potential from the orbdensity and spin density
  !!
  Subroutine  sicpot(orbdens,mesh,vsic,Esic,mesg,xreduce)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Intent(in)      :: orbdens(mesh% nr)
    Real(t_real),Intent(out)     :: Esic
    Real(t_real),Intent(out)     :: vsic(mesh% nr)
    Character(*),Intent(in)      :: mesg
    !!     
    !!LOCAL
    !Character(*),Parameter::nsubfunc="sicpot"
    Real(t_real)   :: bsic(mesh% nr),e
    !!
    !!
    e=0.0_t_real
    vsic=0.0_t_real
    bsic=0.0_t_real
    

    Call potatcoul(mesh,orbdens,vsic,0.0_t_real,1.0_t_real,e,.True.,mesg)

    !! TODO: GGA expression for orbdens
    Call potatxc(mesh,orbdens,orbdens,orbdens,orbdens&
         &,orbdens,orbdens&
         &,vsic,bsic,2_t_int,e&
         &,.False.,.False.,mesg,xreduce)

    !! sic functional is minus LSDA functional of the orbital density
    vsic=-(vsic+bsic)
    Esic=-e
    
    !! open(17,file='sicpot',status='unknown')
    !! Do i=1_t_int,nr
    !!        write(17,12)r(i),vvv(i),bbb(i),pot(i),bf(i)
    !! Enddo
    !! 12 Format(5(e30.15e3,2x))
    !! close(17)
    Return
    !!     
  End Subroutine sicpot
  !!***********************************************************************
  !! coulomb potential + energy
  !!
  Subroutine potatcoul(mesh,rhotrr,vtr,Z,Q,Etot,comment,mesg)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)   :: mesh
    Logical(t_log),Intent(in) :: comment       !!      silent or not
    Character(*),Intent(in)      :: mesg
    !!
    Real(t_real),Intent(in)   :: Z         !!     nuclear charge
    Real(t_real),Intent(in)   :: Q         !!     charge content of density
    Real(t_real),Intent(inout):: Etot     !!     total energy
    Real(t_real),Intent(in)   :: rhotrr(mesh% nr)
    Real(t_real),Intent(out)  :: vtr(mesh% nr)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="potatcoul"
    Integer(t_int) :: ir
    Real(t_real) :: cp
    Real(t_real) :: ee(mesh% nr),y1(mesh% nr),y(mesh% nr)
    !!
    !!  
    vtr=0.0_t_real
    y=0.0_t_real
    y1=0.0_t_real
    !!     
    y=rhotrr*mesh% p_jmesh
    y1(1)=0.0_t_real
    Do ir=2,mesh% nr
       y1(ir)=rhotrr(ir)/mesh% p_rmesh(ir)*mesh% p_jmesh(ir)
    Enddo
!    Open(12,file='+y')
!    Do ir=1,mesh% nr
!       Write(12,"(3(e30.15e3,2x))")mesh% p_rmesh(ir),y(ir),y1(ir)
!    Enddo
!    Close(12)
    Call lag4int(mesh% nr,y1,mesh% hx,cp,lagint_in)
    Call lag4int(mesh% nr,y,mesh% hx,cp,lagint_out)


    If(Abs(cp-Q)>1.e-10_t_real)Then
!       Write(errbuf,10)cp,Q
!       Call errmsg(nmodule,nsubfunc,E_warning,msg=trim(errbuf))
    Endif
    
    vtr=vtr+y1*mesh% p_rmesh+y-Z


!    If(Abs(vtr(mesh% nr-1_t_int))>1.e-10_t_real*Z)Then
!       If(comment)Write(outfile,*)'v(nr)-Z ',y(mesh% nr-1_t_int)-Z&
!            &,vtr(mesh% nr-1_t_int)
!    Endif
    !!
    !!     energy
    !!     1/2(n v) -(n Z)
    
    ee(1)=0.0_t_real
    Do ir=2,mesh% nr
       ee(ir)=rhotrr(ir)*(vtr(ir)-Z)/mesh% p_rmesh(ir)*mesh% p_jmesh(ir)
    Enddo

!    Open(12,file='+e')
!    Do ir=1,mesh% nr
!       Write(12,"(3(e30.15e3,2x))")mesh% p_rmesh(ir),ee(ir)
!    Enddo
!    Close(12)

    Call tlag4int(mesh% nr,ee,mesh% hx,cp)
    !!     r->0
    If(comment)Then
       Write(outfile,12)Trim(mesg),0.50_t_real*cp
    Endif
    Etot=etot+0.50_t_real*cp
    !!
    !!
    Return
    !!     
!10  Format('''Ionic potential!'',/,''Coulomb potential charge error',g30.15e2&
!         &,'!=',g30.15e2,'''')
12  Format(a,' COULOMB ENERGY     : ',f30.15)
    !!
  End Subroutine potatcoul
  !!***********************************************************************
  !! xc-potential + energy
  !!
  Subroutine potatxc(mesh,rhotrr,sptrr,drhotrr,dsptrr,d2rhotrr,d2sptrr&
       &,vtr,btr,mspin,Etot&
       &,comment,standalone,mesg,xreduce)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Character(*),Intent(in)      :: mesg
    Logical(t_log),Intent(in)    :: comment,standalone
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Intent(in)      :: rhotrr(mesh% nr),sptrr(mesh% nr)
    Real(t_real),Intent(in)      :: drhotrr(mesh% nr),dsptrr(mesh% nr)
    Real(t_real),Intent(in)      :: d2rhotrr(mesh% nr),d2sptrr(mesh% nr)
    Real(t_real),Intent(inout)   :: vtr(mesh% nr),btr(mesh% nr),Etot
    !!LOCAL
    Character(*),Parameter::nsubfunc="potatxc"
    Logical(t_log) :: alert
    Integer(t_int):: i
    Real(t_real)   :: rhos(2),vex(2_t_int),cp,error,tol,fs12,f3s2,eps,r,r2
    Real(t_real)   :: gradrho,gradrhos(2),laprhos(2),mixrhos(2),mixrho,ex,r0
    Real(t_real)   :: rho
    Real(t_real)   :: ee(mesh% nr)

    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    

    r0=50.0_t_real*mesh% p_rmesh(2) !!TODO: check this out !!
    
    f3s2=0.50_t_real*(3.0_t_real-mspin)
    fs12=0.50_t_real*(mspin-1_t_int)

    !!
    Do i=1_t_int,mesh% nr
       r=mesh% p_rmesh(i)
       r2=r**2_t_int
       If(i==1)Then
          !! avoid division by zero
          r2=1.0_t_real
       Endif
       rhos(1)=(rhotrr(i)+sptrr(i))/r2*f3s2/fourpi
       rhos(2)=(rhotrr(i)-sptrr(i))/r2*fs12/fourpi

       gradrhos(1)=(drhotrr(i)+dsptrr(i))/r2*f3s2/fourpi
       gradrhos(2)=(drhotrr(i)-dsptrr(i))/r2*fs12/fourpi
       laprhos(1)=(d2rhotrr(i)+d2sptrr(i))/r2*f3s2/fourpi
       laprhos(2)=(d2rhotrr(i)-d2sptrr(i))/r2*fs12/fourpi
       !!
       gradrho=gradrhos(1)+gradrhos(2)
       mixrhos(1)=laprhos(1)*gradrhos(1)**2
       mixrhos(2)=laprhos(2)*gradrhos(2)**2
       mixrho=(laprhos(1)+laprhos(2))*gradrho**2
       !!
       rho=(rhos(1)+rhos(2))
       If(i==1 .Or. rho == 0.0_t_real)Then
          !! put some value, to get through
          rhos(1:2)=0.0_t_real
          gradrhos(1:2)=0.0_t_real
          laprhos(1:2)=0.0_t_real
          mixrhos=0.0_t_real
          mixrho=0.0_t_real
          gradrho=0.0_t_real
       Else
          laprhos(1:2)=laprhos(1:2)+2.0_t_real/r*gradrhos(1:2)
       Endif
       !!
       !!
       gradrho=Abs(gradrho)
       gradrhos(1)=Abs(gradrhos(1))
       gradrhos(2)=Abs(gradrhos(2))
       !!
       !!
       !!
       !!       xc-potential
       Call adjustrho(rhos,1.0_t_real,error,mspin,tol,alert)
       Call vxcrho(mspin,rhos,gradrhos,gradrho,laprhos,mixrhos,mixrho&
            &,vex,eps,xreduce)


       If(getxcversion()==XCVER_PBE96)Then
          ex=1.0_t_real-Exp(-(r/r0)**2)
       Else
          ex=1.0_t_real
       Endif

       If(i>1)Then
          vtr(i)=vtr(i)+(vex(1_t_int)+vex(2_t_int))*r*0.5_t_real*ex
          btr(i)=btr(i)+(vex(1_t_int)-vex(2_t_int))*r*fs12*ex
       Endif
       ee(i)=rho*r2*mesh% p_jmesh(i)*fourpi*eps
       If(i==1)Then
          !! set correct values for r==0
          !vtr(i)=0.0_t_real
          !btr(i)=0.0_t_real
          ee(i)=0.0_t_real
       Endif
    Enddo! i
    If(comment)Then
       !Write(outfile, *)'vxc(nr-1)',vtr(mesh% nr-1_t_int)
       !Write(outfile, *)'vxc(nr)',vtr(mesh% nr)
    Endif
    !!
!!$    If(standalone) Then
!!$       Call writefunc(mesh% nr,mesh% p_rmesh,vtr,12,'+so_vtr')
!!$       Call writefunc(mesh% nr,mesh% p_rmesh,ttt,12,'+so_ttt')
!!$    Else
!!$       Call writefunc(mesh% nr,mesh% p_rmesh,vtr,12,'+fp_vtr')
!!$       Call writefunc(mesh% nr,mesh% p_rmesh,ttt,12,'+fp_ttt')
!!$    Endif
    !!
    Call tlag4int(mesh% nr,ee,mesh% hx,cp)


    !!
    Etot=Etot+cp
    !!
    If(comment)Then
       Write(outfile,11)Trim(mesg),cp
       Write(outfile,12)Trim(mesg),Etot
       Write(outfile,13)Trim(mesg),Etot*2.0_t_real
       Write(outfile,14)Trim(mesg),Etot*hatoev
       If(standalone)Then
          Write(outfile,112)Trim(mesg),Trim(mesg),Etot
       Endif
    Endif
    !!
    If(alert)Then
       Write(errbuf,15) error
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)

    Endif
    
    !!
    Return
    !!
11  Format(a,' EXCHANGE ENERGY    : ',f30.15)
12  Format(a,' TOTAL ENERGY (Ha)  : ',f30.15)
112 Format(a,' ',/,a,' EE: ',f30.15)
13  Format(a,' TOTAL ENERGY (Ry)  : ',f30.15)
14  Format(a,' TOTAL ENERGY (eV)  : ',f30.15)
15  Format('''Numerical inaccuracy in vxc/exc &
         &calculation! '',/,'' maxerror=',e12.4e3,'''')
    !!
  End Subroutine potatxc
  !!***********************************************************************
  Subroutine  gam_fac_mesh(mesh,gam_factor,gamma,texp)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Logical(t_log),Intent(in)    :: gam_factor
    Real(t_real),Intent(in)      :: gamma
    Real(t_real),Intent(out)     :: texp(1:mesh% nr)
    !!LOCAL
    If(gam_factor)Then
       If(Abs(gamma)<1.0e-12_t_real)Then
          texp(1:mesh% nr)=1.0_t_real
       Else
          texp(1:mesh% nr)=mesh% p_rmesh(1:mesh% nr)**(gamma)
       Endif
    Else
       texp(1:mesh% nr)=1.0_t_real
    Endif
    !!
    Return
    !!
  End Subroutine gam_fac_mesh
  !!***********************************************************************
End Module atompot
