Module bandplot
  !!
  Use precisiondefs
  Use memory
  Use file_iounits
  Use constants
  Use tsymmetry
  Use errors
  Use tio
  Use tbandplot
  Use matserv
  Use torbitalbasis
  Use diracdefs
  Use ioaux
  Use sort
  Use optionstest
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="BANDPLOT"  
  !!
  Public :: define_special_bz_points,calc_bandplot_mesh&
       &,read_bandplot_mesh,prepaxisrot,read_old_ldos_file&
       &,setup_ldos,frel_prepaxisrot,setup_WeightRot,setup_ldosRot,&
       &averageDegenerateSubspaces
  !!***********************************************************************
contains
  !!***********************************************************************
  !!   define special points in BZ
  Subroutine define_special_bz_points(symmetry,bandplotcontrol)
    !!
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(in) :: symmetry
    Type(T_Bandplot),Intent(inout) :: bandplotcontrol
    !!LOCAL
    Character(*),Parameter::nsubfunc="define_special_bz_points"
    Integer(t_int),Parameter :: maxpoints=20_t_int!,maxnpath=100_t_int
    !! labels of special symmetry points
    Character(20) :: plabel(maxpoints)
    !! table of special symemtry points
    Real(t_real)  :: points(3,maxpoints)
    !! number of points on path
    Integer(t_int) :: npath
    !! path through points (sequence of indices into plabel and points)
    Integer(t_int) :: path(100)
    Real(t_real)  :: aob,aoc,bg,bm,coa,mu,la,alpha,fa,a,b,c
    Integer(t_int) :: i
    !!
    a=symmetry% lattice_constants(1)
    b=symmetry% lattice_constants(2)
    c=symmetry% lattice_constants(3)
    aoc=a/c
    aob=a/b
    !boa=b/a
    coa=c/a
    !boc=b/c
    !cob=c/b
    
    If(symmetry % lattice == 'SC' )Then 
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(2_t_int)='X'
       points(:, 2_t_int)=(/ 0.5_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(3_t_int)='M'
       points(:, 3_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.0_t_real /)
       plabel(4_t_int)='R'
       points(:, 4_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.5_t_real /)
       npath=8_t_int
       path(1_t_int:npath)=(/1_t_int,2_t_int,3_t_int,1_t_int,4_t_int&
            &,3_t_int,2_t_int,4_t_int/)
    Else If(symmetry % lattice == 'BCC')Then
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(2_t_int)='H'
       points(:, 2_t_int)=(/ 1.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(3_t_int)='N'
       points(:, 3_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.0_t_real /)
       plabel(4_t_int)='P'
       points(:, 4_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.5_t_real /)
       npath=7_t_int
       path(1_t_int:npath)=(/1_t_int,2_t_int,3_t_int,1_t_int,4_t_int&
            &,3_t_int,4_t_int/)
    Else If(symmetry % lattice =='FCC')Then
       
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(2_t_int)='X'
       points(:, 2_t_int)=(/ 1.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(3_t_int)='U'
       points(:, 3_t_int)=(/ 1.0_t_real ,0.25_t_real ,0.25_t_real /)
       plabel(4_t_int)='W'
       points(:, 4_t_int)=(/ 1.0_t_real ,0.5_t_real ,0.0_t_real /)
       plabel(5_t_int)='L'
       points(:, 5_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.5_t_real /)
       plabel(6_t_int)='K'
       points(:, 6_t_int)=(/ 0.75_t_real ,0.75_t_real ,0.0_t_real /)
       npath=9_t_int
       path(1_t_int:npath)=(/1_t_int,2_t_int,4_t_int,6_t_int,1_t_int&
            &,5_t_int,4_t_int,3_t_int,2_t_int/)
    Else If(symmetry % lattice =='ST')Then
       
       
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(2_t_int)='X'
       points(:, 2_t_int)=(/ 0.0_t_real ,0.5_t_real ,0.0_t_real /)
       plabel(3_t_int)='M'
       points(:, 3_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.0_t_real /)
       plabel(4_t_int)='Z'
       points(:, 4_t_int)=(/ 0.0_t_real ,0.0_t_real ,aoc*0.5_t_real /)
       plabel(5_t_int)='R'
       points(:, 5_t_int)=(/ 0.0_t_real ,0.5_t_real ,aoc*0.5_t_real /)
       plabel(6_t_int)='A' 
       points(:, 6_t_int)=(/ 0.5_t_real ,0.5_t_real ,aoc*0.5_t_real /)
       npath=7_t_int
       path(1_t_int:npath)=(/1_t_int,2_t_int,3_t_int,1_t_int,4_t_int&
            &,5_t_int,6_t_int/)
       
    Else If(symmetry % lattice =='BCT')Then
       If(coa > 1.0_t_real)Then
          bg=0.5_t_real*(1.0_t_real+aoc**2_t_int)
          bm=0.5_t_real*(1.0_t_real-aoc**2_t_int)
          plabel(1_t_int)='$~G'
          points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
          plabel(2_t_int)='$~G$_3'
          points(:, 2_t_int)=(/ bm  ,bm ,aoc /)
          plabel(3_t_int)='Z'
          points(:, 3_t_int)=(/ 0.0_t_real  ,0.0_t_real ,aoc /)
          plabel(4_t_int)='$~G$_1'
          points(:, 4_t_int)=(/ 0.0_t_real  ,bg ,0.0_t_real /)
          plabel(5_t_int)='$~G$_2'
          points(:, 5_t_int)=(/ bm  ,bg ,0.0_t_real /)
          plabel(6_t_int)='X'
          points(:, 6_t_int)=(/ 0.5_t_real  ,0.5_t_real ,0.0_t_real /)
          plabel(7_t_int)='P'
          points(:, 7_t_int)=(/ 0.5_t_real  ,0.5_t_real ,aoc*0.5_t_real /)
          npath=8_t_int
          path(1_t_int:npath)=(/1_t_int,6_t_int,7_t_int,2_t_int,3_t_int&
               &,1_t_int,4_t_int,5_t_int/)
       Else
          bg=0.5_t_real*(aoc+coa)
          bm=0.5_t_real*(aoc-coa)
          plabel(1_t_int)='$~G'
          points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
          plabel(2_t_int)='X'
          points(:, 2_t_int)=(/ 0.5_t_real ,0.5_t_real ,0.0_t_real /)
          plabel(3_t_int)='L'
          points(:, 3_t_int)=(/ 1.0_t_real ,0.0_t_real ,0.0_t_real /)
          plabel(4_t_int)='H$_1'
          points(:, 4_t_int)=(/ 1.0_t_real ,0.0_t_real ,bm /)
          plabel(5_t_int)='H'
          points(:, 5_t_int)=(/ 0.0_t_real ,0.0_t_real ,bg /)
          plabel(6_t_int)='P'
          points(:, 6_t_int)=(/ 0.5_t_real ,0.5_t_real ,aoc*0.5_t_real /)
          npath=7_t_int
          path(1_t_int:npath)=(/1_t_int,3_t_int,4_t_int,6_t_int,5_t_int&
               &,1_t_int,2_t_int/)
       Endif
    Else If( (symmetry % lattice == 'SH') .Or. &
         &   (symmetry % lattice == 'STG')        )Then
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       plabel(2_t_int)='K'
       points(:, 2_t_int)=(/ sq3o3, c1o3 ,0.0_t_real /)
       plabel(3_t_int)='M'
       points(:, 3_t_int)=(/ sq3o3, 0.0_t_real ,0.0_t_real /)
       plabel(4_t_int)='A'
       points(:, 4_t_int)=(/ 0.0_t_real, 0.0_t_real ,aoc*0.5_t_real /)
       plabel(5_t_int)='H'
       points(:, 5_t_int)=(/ sq3o3, c1o3 ,aoc*0.5_t_real /)
       plabel(6_t_int)='L'
       points(:, 6_t_int)=(/ sq3o3, 0.0_t_real ,aoc*0.5_t_real /)
       npath=8_t_int
       path(1_t_int:npath)=(/1_t_int,3_t_int,2_t_int,1_t_int,4_t_int&
            &,6_t_int,5_t_int,4_t_int/)
    Else If(symmetry % lattice =='RTG')Then  !!
       alpha=symmetry% axis_angles(1)
       mu=sq3o3*Sqrt(1.0_t_real+2.0_t_real*Cos(alpha*Pi/180.0_t_real))
       la=Sqrt(Max(0.0_t_real,1.0_t_real-mu**2))
       fa=(2.0_t_real-la**2/mu**2)

       If(alpha<=90.0_t_real)Then
          
          plabel(1_t_int)='$~G'
          points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
          plabel(2_t_int)='T'
          points(:, 2_t_int)=(/ 0.0_t_real ,0.0_t_real &
               &,1.0_t_real/(2.0_t_real*mu) /)
          plabel(3_t_int)='X'
          points(:, 3_t_int)=(/ &
               & 1.0_t_real/(6.0_t_real*la) &
               &,1.0_t_real/(6.0_t_real*sq3o3*la) &
               &,1.0_t_real/(3.0_t_real*mu) /)
          plabel(4_t_int)='L'
          points(:, 4_t_int)=(/ &
               & 1.0_t_real/(3.0_t_real*la) &
               &,0.0_t_real &
               &,1.0_t_real/(6.0_t_real*mu) /)
          plabel(5_t_int)='U'
          points(:, 5_t_int)=(/ &
               & fa/(6.0_t_real*la) &
               &,0.0_t_real &
               &,1.0_t_real/(2.0_t_real*mu) /)
          plabel(6_t_int)='W'
          points(:, 6_t_int)=(/ &
               & fa/(6.0_t_real*la) &
               &,fa/(18.0_t_real*sq3o3*la) &
               &,1.0_t_real/(2.0_t_real*mu) /)
          plabel(7_t_int)='Ux'
          points(:, 7_t_int)=(/ &
               & fa/(12.0_t_real*la) &
               &,fa/(12.0_t_real*sq3o3*la) &
               &,1.0_t_real/(2.0_t_real*mu) /)
          npath=8_t_int
          path(1_t_int:npath)=(/1_t_int,4_t_int,5_t_int,2_t_int,6_t_int&
               &,7_t_int,3_t_int,1_t_int/)
       Else
          !! RTG alpha>90
          plabel(1_t_int)='$~G'
          points(:, 1_t_int)=(/ 0.0_t_real,0.0_t_real,0.0_t_real /)

          plabel(2_t_int)='L1'
          points(:, 2_t_int)=(/ &
               &1.0_t_real/(3.0_t_real*la)&
               &,0.0_t_real&
               &,1.0_t_real/(6.0_t_real*mu) /)

          plabel(3_t_int)='L'
          points(:, 3_t_int)=(/ &
               &1.0_t_real/(-6.0_t_real*la)&
               &,1.0_t_real/( 6.0_t_real*sq3o3*la)&
               &,1.0_t_real/(6.0_t_real*mu) /)

          plabel(4_t_int)='F'
          points(:, 4_t_int)=(/ &
               &1.0_t_real/( 2.0_t_real*la)&
               &,1.0_t_real/( 6.0_t_real*sq3o3*la)&
               &,0.0_t_real /)


          plabel(5_t_int)='T'
          points(:, 5_t_int)=(/ &
               &1.0_t_real/( 3.0_t_real*la)&
               &,1.0_t_real/( 3.0_t_real*sq3o3*la)&
               &,1.0_t_real/( 6.0_t_real*mu) /)

          plabel(6_t_int)='T1'
          points(:, 6_t_int)=(/ &
               & 0.0_t_real&
               &,0.0_t_real&
               &,( 1.0_t_real/6.0_t_real+2.0_t_real/3.0_t_real&
               &*mu**2/la**2)/mu /)

          npath=6_t_int
          path(1_t_int:npath)=(/1_t_int,4_t_int,5_t_int,3_t_int,6_t_int&
               &,1_t_int/)
       Endif
    Else If(symmetry % lattice =='SO')Then
       
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)        
       plabel(2_t_int)='X'
       points(:, 2_t_int)=(/ 0.5_t_real ,0.0_t_real ,0.0_t_real /)        
       plabel(3_t_int)='S'
       points(:, 3_t_int)=(/ 0.5_t_real ,aob*0.5_t_real ,0.0_t_real /)        
       plabel(4_t_int)='Y'
       points(:, 4_t_int)=(/ 0.0_t_real ,aob*0.5_t_real ,0.0_t_real /)        
       plabel(5_t_int)='Z'
       points(:, 5_t_int)=(/ 0.0_t_real ,0.0_t_real ,aoc*0.5_t_real /)        
       plabel(6_t_int)='U'
       points(:, 6_t_int)=(/ 0.5_t_real ,0.0_t_real ,aoc*0.5_t_real /)        
       plabel(7_t_int)='R'
       points(:, 7_t_int)=(/ 0.5_t_real ,aob*0.5_t_real ,aoc*0.5_t_real /)
       plabel(8_t_int)='T'
       points(:, 8_t_int)=(/ 0.0_t_real ,aob*0.5_t_real ,aoc*0.5_t_real /)
       npath=9_t_int
       path(1_t_int:npath)=(/1_t_int,2_t_int,3_t_int,4_t_int,1_t_int&
            &,5_t_int,6_t_int,7_t_int,8_t_int/)
       
    Else If((symmetry % lattice =='CFCO') .Or. &
         &  (symmetry % lattice =='AFCO')        )Then  !!
       !!
       Call  makeABCCOSpecPoints(a,b,c,symmetry% lattice&
            &,plabel,points,npath,path)
       !!
    Else If(symmetry % lattice =='FCO')Then  !!
       !!
       Call  makeFCOSpecPoints(a,b,c&
            &,plabel,points,npath,path)
       !!
    Else If(symmetry % lattice =='BCO')Then  !!
       !!
       Call  makeBCOSpecPoints(a,b,c&
            &,plabel,points,npath,path)
       !!

    Else If(symmetry % lattice =='SM')Then  !1
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       npath=1_t_int
       path(1_t_int:npath)=(/1_t_int/)
    Else If(symmetry % lattice =='CFCM')Then !1

       plabel(1)='$~G'
       points(:, 1)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       
       plabel(2)='X'
       points(:, 2)=(/ 0.5_t_real ,0.0_t_real ,0.0_t_real /)

       plabel(3)='XY'
       points(:, 3)=(/ 0.5_t_real ,aob*0.5_t_real ,0.0_t_real /)

       plabel(4)='Y'
       points(:, 4)=(/ 0.0_t_real ,aob*0.5_t_real ,0.0_t_real /)

       plabel(5)='YZ'
       points(:, 5)=(/ 0.0_t_real ,aob*0.5_t_real ,aoc*0.5_t_real /)

       plabel(6)='XYZ'
       points(:, 6)=(/ 0.5_t_real ,aob*0.5_t_real ,aoc*0.5_t_real /)

       plabel(7)='Z'
       points(:, 7)=(/ 0.0_t_real ,    0.0_t_real ,aoc*0.5_t_real /)


       npath=8
       path(1:npath)=(/1,2,3,4,5,6,7,1/)
    Else If(symmetry % lattice =='STC')Then  !1
       plabel(1_t_int)='$~G'
       points(:, 1_t_int)=(/ 0.0_t_real ,0.0_t_real ,0.0_t_real /)
       npath=1_t_int
       path(1_t_int:npath)=(/1_t_int/)
    Else
       Write(errbuf,"('''Unknown lattic type:',a8,'''')") symmetry % lattice
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
    Endif
    bandplotcontrol % npoints=npath
  
 
    Call palloc('p_labels',bandplotcontrol % p_labels&
         &,1_t_int,Max(1_t_int,npath)&
         &,nmodule,nsubfunc)
    Call palloc('p_points',bandplotcontrol % p_points&
         &,1_t_int,3_t_int&
         &,1_t_int,Max(1_t_int,npath)&
         &,nmodule,nsubfunc)
      
    Do i=1_t_int,npath
       bandplotcontrol % p_labels(i)=plabel(path(i))
       bandplotcontrol % p_points(:,i)=points(:,path(i))
    Enddo
  End Subroutine define_special_bz_points
  !!***********************************************************************
  Subroutine  makeBCOSpecPoints(a,b,c,plabel,points,npath,path)
    !!
    Implicit None 
    !!IO
    Real(t_real),Intent(in)      :: a,b,c
    Character(20),Intent(out)    :: plabel(:)
    Real(t_real),Intent(out)     :: points(:,:)
    Integer(t_int),Intent(out)   :: npath
    Integer(t_int),Intent(out)   :: path(:)
    !!LOCAL
    Real(t_real)   :: aa,bb,cc,P(3,3),res(3),v(3),bmab,bgac,bmac
    Real(t_real),Parameter :: c1=1.0_t_real
    Real(t_real),Parameter :: c0=0.0_t_real
    Real(t_real),Parameter :: c1o2=0.5_t_real
    !!
    v(1)=1.0_t_real/a
    v(2)=1.0_t_real/b
    v(3)=1.0_t_real/c
    !!
    P(:,:)=c0
    P(1,1)=c1
    P(2,2)=c1
    P(3,3)=c1
    !!
    If(v(2)<v(1))Then
       res(:)=P(:,1)
       P(:,1)=P(:,2)
       P(:,2)=res(:)
       res(1)=v(1)
       v(1)=v(2)
       v(2)=res(1)
    Endif
    !!
    If(v(3)<v(1))Then
       res(:)=P(:,1)
       P(:,1)=P(:,3)
       P(:,3)=res(:)
       res(1)=v(1)
       v(1)=v(3)
       v(3)=res(1)
    Endif
    !!
    If(v(3)<v(2))Then
       res(:)=P(:,2)
       P(:,2)=P(:,3)
       P(:,3)=res(:)
       res(1)=v(2)
       v(2)=v(3)
       v(3)=res(1)
    Endif
    !!
    !!
    P=Transpose(P)
    !!
    !Print *,P(:,1)
    !Print *,P(:,2)
    !Print *,P(:,3)
    v(1)=a
    v(2)=b
    v(3)=c
    res=Matmul(P,v)
    AA=res(1)
    BB=res(2)
    CC=res(3)
    !Print *,'ABC=',AA,BB,CC
    !!
    bmab=c1o2*(1.-(BB*BB)/(AA*AA));
    bmac=c1o2*(1.-(CC*CC)/(AA*AA));
    bgac=c1o2*(1.+(CC*CC)/(AA*AA));
    !!
    plabel(1)='$~G'
    points(:,1)=(/ c0 ,c0 ,c0 /)
    
    plabel(2)='X'
    Call makeSpecPointPerm(P,a,b,c,(/ c1,c0,c0 /),points(:, 2))

    plabel(3)='T'
    Call makeSpecPointPerm(P,a,b,c,(/ c1o2,c1o2,c0 /),points(:, 3))

    plabel(4)='R'
    Call makeSpecPointPerm(P,a,b,c,(/ c1o2,c0,c1o2 /),points(:, 4))

    plabel(5)='S'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,c1o2,c1o2 /),points(:, 5))

    plabel(6)='W'
    Call makeSpecPointPerm(P,a,b,c,(/ c1o2,c1o2,c1o2 /),points(:, 6))

    plabel(7)='U1'
    Call makeSpecPointPerm(P,a,b,c,(/ c1,bmab,c0 /),points(:, 7))

    plabel(8)='G1'
    Call makeSpecPointPerm(P,a,b,c,(/ c1,c0,bmac /),points(:, 8))

    plabel(9)='A'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,bmab,bgac /),points(:, 9))

    plabel(10)='Z'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,c0,bgac /),points(:, 10))

    !!
    npath=7
    path(1:npath)=(/1,2,7,3,6,4,1/)
    !!
    !!
    Return
    !!
  End Subroutine makeBCOSpecPoints
  !!***********************************************************************
  Subroutine  makeFCOSpecPoints(a,b,c,plabel,points,npath,path)
    !!
    Implicit None 
    !!IO
    Real(t_real),Intent(in)      :: a,b,c
    Character(20),Intent(out)    :: plabel(:)
    Real(t_real),Intent(out)     :: points(:,:)
    Integer(t_int),Intent(out)   :: npath
    Integer(t_int),Intent(out)   :: path(:)
    !!LOCAL
    Integer(t_int) :: typ
    Real(t_real)   :: aa,bb,cc,P(3,3),res(3),v(3),bgcamba
    Real(t_real)   :: bmbaca,bgabmcb,bgacmbc,bgbcmac,bgcbmab,bgbagca,bgbamca
    Real(t_real),Parameter :: c1=1.0_t_real
    Real(t_real),Parameter :: c0=0.0_t_real
    Real(t_real),Parameter :: c1o2=0.5_t_real
    !!
    v(1)=1.0_t_real/a
    v(2)=1.0_t_real/b
    v(3)=1.0_t_real/c
    !!
    P(:,:)=c0
    P(1,1)=c1
    P(2,2)=c1
    P(3,3)=c1
    !!
    typ=1
    If(v(1)**2 >= v(2)**2+v(3)**2)Then
       typ=0
    Else If(v(2)**2 >= v(1)**2+v(3)**2)then
       typ=0
       res(:)=P(:,1)
       P(:,1)=P(:,2)
       P(:,2)=res(:)
       res(1)=v(1)
       v(1)=v(2)
       v(2)=res(1)
    Else If(v(3)**2 >= v(1)**2+v(2)**2)Then
       typ=0
       res(:)=P(:,1)
       P(:,1)=P(:,3)
       P(:,3)=res(:)
       res(1)=v(1)
       v(1)=v(3)
       v(3)=res(1)
    Endif
    !!
    P=Transpose(P)
    !!
    !Print *,P(:,1)
    !Print *,P(:,2)
    !Print *,P(:,3)
    v(1)=a
    v(2)=b
    v(3)=c
    res=Matmul(P,v)
    AA=res(1)
    BB=res(2)
    CC=res(3)
    !Print *,'ABC=',AA,BB,CC
    !!
    If(typ == 0)Then
       
       bmbaca =c1o2*(c1-(AA*AA)/(BB*BB)-(AA*AA)/(CC*CC));
       bgcamba=c1o2*(c1-(AA*AA)/(BB*BB)+(AA*AA)/(CC*CC));
       bgbamca=c1o2*(c1+(AA*AA)/(BB*BB)-(AA*AA)/(CC*CC));
       bgbagca=c1o2*(c1+(AA*AA)/(BB*BB)+(AA*AA)/(CC*CC));
       

       !!
       plabel(1)='$~G'
       points(:,1)=(/ c0 ,c0 ,c0 /)
       
       plabel(2)='Y'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c0 /),points(:, 2))

       plabel(3)='Z'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c0,c1 /),points(:, 3))

       plabel(4)='T'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c1 /),points(:, 4))

       plabel(5)='L'
       Call makeSpecPointPerm(P,a,b,c,(/ c1o2,c1o2,c1o2 /),points(:, 5))

       plabel(6)='U1'
       Call makeSpecPointPerm(P,a,b,c,(/ bmbaca,c1,c1 /),points(:, 6))

       plabel(7)='C1'
       Call makeSpecPointPerm(P,a,b,c,(/ bgcamba,c1,c0 /),points(:, 7))

       plabel(8)='A1'
       Call makeSpecPointPerm(P,a,b,c,(/ bgbamca,c0,c1 /),points(:, 8))

       plabel(9)='X1'
       Call makeSpecPointPerm(P,a,b,c,(/ bgbagca,c0,c0 /),points(:, 9))

       plabel(10)='Z1'
       Call makeSpecPointPerm(P,a,b,c,(/ c1,c1,c0 /),points(:, 10))

       !!
       npath=8
       path(1:npath)=(/1,2,4,3,1,10,2,1/)
       !!
    Else
       !!
       bgcbmab=c1o2*(c1+(B*B)/(C*C)-(B*B)/(A*A));
       bgbcmac=c1o2*(c1+(C*C)/(B*B)-(C*C)/(A*A));
       bgacmbc=c1o2*(c1+(C*C)/(A*A)-(C*C)/(B*B));
       bgcamba=c1o2*(c1-(A*A)/(B*B)+(A*A)/(C*C));
       bgbamca=c1o2*(c1+(A*A)/(B*B)-(A*A)/(C*C));
       bgabmcb=c1o2*(c1+(B*B)/(A*A)-(B*B)/(C*C));
       !!
       plabel(1)='$~G'
       points(:,1)=(/ c0 ,c0 ,c0 /)
       
       plabel(2)='Y'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c0 /),points(:, 2))

       plabel(3)='T'
       Call makeSpecPointPerm(P,a,b,c,(/ c1,c0,c0 /),points(:, 3))

       plabel(4)='Z'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c0,c1 /),points(:, 4))

       plabel(5)='D1'
       Call makeSpecPointPerm(P,a,b,c,(/ c1,bgcbmab,c0 /),points(:, 5))

       plabel(6)='C1'
       Call makeSpecPointPerm(P,a,b,c,(/ bgcamba,c1,c0 /),points(:, 6))

       plabel(7)='E1'
       Call makeSpecPointPerm(P,a,b,c,(/ c1,c0,bgbcmac /),points(:, 7))

       plabel(8)='A1'
       Call makeSpecPointPerm(P,a,b,c,(/ bgbamca,c0,c1 /),points(:, 8))

       plabel(9)='H1'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,bgacmbc /),points(:, 9))

       plabel(10)='B1'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,bgabmcb,c1 /),points(:, 10))

       plabel(11)='Z1'
       Call makeSpecPointPerm(P,a,b,c,(/ c1,c1,c0 /),points(:, 11))

       plabel(12)='T1'
       Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c1 /),points(:, 12))

       npath=7
       path(1:npath)=(/1,3,11,2,12,4,1/)

       !!
    Endif
    !!
    Return
    !!
  End Subroutine makeFCOSpecPoints
  !!***********************************************************************
  Subroutine  makeABCCOSpecPoints(a,b,c,lattice,plabel,points,npath,path)
    !!
    Implicit None 
    !!IO
    Character(*),Intent(in)      :: lattice
    Real(t_real),Intent(in)      :: a,b,c
    Character(20),Intent(out)    :: plabel(:)
    Real(t_real),Intent(out)     :: points(:,:)
    Integer(t_int),Intent(out)   :: npath
    Integer(t_int),Intent(out)   :: path(:)
    !!LOCAL
    Real(t_real)   :: bg,bm,aa,bb,P(3,3),res(3),v(3)
    Real(t_real),Parameter :: c1=1.0_t_real
    Real(t_real),Parameter :: c0=0.0_t_real
    Real(t_real),Parameter :: c1o2=0.5_t_real
    !!
    v(1)=1.0_t_real/a
    v(2)=1.0_t_real/b
    v(3)=1.0_t_real/c
    !!
    P(:,:)=c0
    P(1,1)=c1
    P(2,2)=c1
    P(3,3)=c1
    !!
    If(lattice == 'AFCO')Then
       
       res(:)=P(:,1)
       P(:,1)=P(:,3)
       P(:,3)=res(:)
       res(1)=v(1)
       v(1)=v(3)
       v(3)=res(1)
    Endif
    
    If(v(2)>v(1))Then
       res(:)=P(:,1)
       P(:,1)=P(:,2)
       P(:,2)=res(:)
       res(1)=v(1)
       v(1)=v(2)
       v(2)=res(1)
    End If
    !!
    P=Transpose(P)
    !!
    !Print *,P(:,1)
    !Print *,P(:,2)
    !Print *,P(:,3)
    v(1)=a
    v(2)=b
    v(3)=c
    res=Matmul(P,v)
    AA=res(1)
    BB=res(2)
    !CC=res(3)
    !Print *,'ABC=',AA,BB,CC
    !!
    bg=c1o2*(c1+(AA*AA)/(BB*BB));
    bm=c1o2*(c1-(AA*AA)/(BB*BB));

    !!
    plabel(1)='$~G'
    points(:,1)=(/ c0 ,c0 ,c0 /)
    plabel(2)='Y'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c0 /)&
         &,points(:, 2))

    plabel(3)='K'
    Call makeSpecPointPerm(P,a,b,c,(/ bg,c0,c0 /)&
         &,points(:, 3))

    plabel(4)='Z'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,c0,c1o2 /)&
         &,points(:, 4))

    plabel(5)='U'
    Call makeSpecPointPerm(P,a,b,c,(/ bm,c1,c0 /)&
         &,points(:, 5))

    plabel(6)='T'
    Call makeSpecPointPerm(P,a,b,c,(/ c0,c1,c1o2 /)&
         &,points(:, 6))

    plabel(7)='P'
    Call makeSpecPointPerm(P,a,b,c,(/ bg,c0,c1o2 /)&
         &,points(:, 7))

    plabel(8)='C'
    Call makeSpecPointPerm(P,a,b,c,(/ bm,c1,c1o2 /)&
         &,points(:, 8))
    !!
    npath=8
    path(1:npath)=(/1,3,5,2,4,6,8,7/)
    !!
    Return
    !!
  End Subroutine makeABCCOSpecPoints
  !!***********************************************************************
  Subroutine  makeSpecPointPerm(Perm,a,b,c,x,res)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: a,b,c
    Real(t_real),Intent(in)      :: Perm(3,3),x(3)
    Real(t_real),Intent(out)     :: res(3)
    !!LOCAL
    res=Matmul(x,Perm)
    res(2)=res(2)*a/b
    res(3)=res(3)*a/c
    !!
  End Subroutine makeSpecPointPerm
  !!***********************************************************************
  
  !!***********************************************************************
  !***********************************************************************
  !!***********************************************************************
  !!
  !! calcs k-points along symmetry lines, defined via hspnt
  !! pnt(4,np) contains the euclidian norm along the path
  !! the number of points is proportional to the length of the intervall
  !! the max number is ndiv
  !! outputs band points as datas producing vertical 
  !!    lines e.g. in 'xmgr'
  !!    x:    k-distances  
  !!    y:    emin,emax
  !!    each line is separated by blank line
  !!    output to file '+points'
  !!
  Subroutine calc_bandplot_mesh(bandplot,pointsfilename)
    !!
    Implicit None
    !!IO
    Type(T_Bandplot),Intent(inout)::bandplot 
    Character(*),Intent(in)      :: pointsfilename
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_bandplot_mesh"
    Integer(t_int) :: nd,np,i,n,nlabel,lastvalidindex
    Real(t_real)  :: dmax,d,dd(3),aa(3),dist
    !!
    !!***********************************************************************
    !!     determine max distance between sym-points
    dmax=0.0_t_real
    lastvalidindex=bandplot % npoints
    np=0_t_int
    nlabel=0
    Do nd=1_t_int,bandplot % npoints-1_t_int
       If(Trim(bandplot% p_labels(nd))=="<JUMP>") Cycle
       If(Trim(bandplot% p_labels(nd+1))=="<JUMP>") Cycle
       dmax=Max(dmax,Sqrt(Sum((bandplot % p_points(:,nd)&
            &-bandplot % p_points(:,nd+1_t_int))**2_t_int)))
       nlabel=nlabel+1
    Enddo
    !! count number of resulting k-points
    Do nd=1_t_int,bandplot % npoints-1_t_int
       If(Trim(bandplot% p_labels(nd))=="<JUMP>") Cycle
       If(Trim(bandplot% p_labels(nd+1))=="<JUMP>") Cycle
       dd(:)=-bandplot % p_points(:,nd)+bandplot % p_points(:,nd+1_t_int)
       d=Sqrt(Sum(dd*dd))
       n=Max(1_t_int,Nint(bandplot % ndiv*d/dmax,t_int))
       d=d/Real(n,t_real)
       np=np+n
    Enddo                     ! npoints
    np=np+1_t_int
    bandplot % nkpnts=np
    !!   
    Call palloc('p_kpnts',bandplot% p_kpnts&
         &,1_t_int,4_t_int&
         &,1_t_int,Max(1_t_int,bandplot % nkpnts)&
         &,nmodule,nsubfunc)

    np=0_t_int
    !!     calculate mesh
    Open(13,file=Trim(pointsfilename),status='unknown')
    Write(13,"('# ',i4)")nlabel+1
    dist=0.0_t_real
    Do nd=1_t_int,bandplot % npoints-1_t_int
       If(Trim(bandplot% p_labels(nd))=="<JUMP>") Cycle
       If(Trim(bandplot% p_labels(nd+1))=="<JUMP>") Cycle
       lastvalidindex=nd+1
       dd(:)=-bandplot % p_points(:,nd)+bandplot % p_points(:,nd+1_t_int)
       d=Sqrt(Sum(dd*dd))
       n=Max(1_t_int,Nint(bandplot % ndiv*d/dmax,t_int))
       d=d/Real(n,t_real)
       Do i=1_t_int,n
          np=np+1_t_int
          aa(:)=dd(:)*(i-1_t_int)/n
          bandplot % p_kpnts(1_t_int:3_t_int,np)=bandplot % p_points(:,nd)+aa(:)
          bandplot % p_kpnts(4_t_int,np)=dist
          !!Print 122,np,( bandplot % p_kpnts(k,np),k=1,4)
          If(i == 1_t_int)Then
             Write(13,12) Trim(Adjustl(bandplot % p_labels(nd)))
             Write(13,13) dist,bandplot % emin*hatoev
             Write(13,13) dist,bandplot % emax*hatoev
             Write(13,14)
          Endif
          dist=dist+d
       Enddo                   ! n
    Enddo                     ! npoints
    
    Write(13,12) Trim(Adjustl(bandplot % p_labels(lastvalidindex)))
    Write(13,13) dist,bandplot % emin*hatoev
    Write(13,13) dist,bandplot % emax*hatoev
    np=np+1_t_int
    bandplot % p_kpnts(1_t_int:3_t_int,np)=bandplot % p_points(:,lastvalidindex)
    bandplot % p_kpnts(4_t_int,np)=dist
    !!Print 122,np,( bandplot % p_kpnts(k,np),k=1,4)
    !!122 Format(i5,4(f30.15,2x))
    Close(13)
    !!     
    Return
    !!
12  Format('# '' ',a,' ''')
13  Format(2(f10.5,2x))
14  Format(' ')
    !!     
  End Subroutine calc_bandplot_mesh
  !!
  !!***********************************************************************
  !!     
  Subroutine read_bandplot_mesh(bandplot)
    !!     
    Implicit None
    !!IO
    Type(T_Bandplot),Intent(inout)::bandplot 
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_bandplot_mesh"
    Integer(t_int):: i,io,iu
    Real(t_real) :: xx(3),x0(3),d
    Logical(t_log)::exist,onlyoccupied
    Character(5000) :: buf,word
    !!     
    Inquire(file=f_bandplot_mesh,exist=exist)
    If(.not. exist) return 
    !!
    bandplot% suffix='_kp'
    !!
    Open(12,file=f_bandplot_mesh,status='unknown')
    !!
    Read(12,1)buf
    !!
    !!
    If(.Not.next_word(buf,word))Then
       Call terminate(TERM_CRASH,Trim(nsubfunc)//': in '''&
            &//Trim(f_bandplot_mesh)&
            &//''': error in first line!')
    Endif
    Read(word,*)bandplot% nkpnts
    !!
    onlyoccupied=.False.
    io=0_t_int
    iu=0_t_int
    If(next_word(buf,word))Then
       Read(word,*)onlyoccupied
       If(next_word(buf,word))Then
          Read(word,*)io
          If(next_word(buf,word))Then
             Read(word,*)iu
          Endif
       Endif
    Endif
    !!
    bandplot% output_partoccu_bands=onlyoccupied
    bandplot% part_occu_offset(1)=io
    bandplot% part_occu_offset(2)=iu
    !!
1   Format(a)
    !!
    Call palloc('p_kpnts',bandplot% p_kpnts&
         &,1_t_int,4_t_int&
         &,1_t_int,Max(1_t_int,bandplot % nkpnts)&
         &,nmodule,nsubfunc)

    
    Read(12,*)bandplot % p_kpnts(1_t_int:3_t_int,1_t_int)
    x0(:)=bandplot % p_kpnts(1_t_int:3_t_int,1_t_int)
    bandplot % p_kpnts(4_t_int,1_t_int)=0.0_t_real
    d=0.0_t_real
    Do i=2_t_int,bandplot % nkpnts
       Read(12,*)bandplot % p_kpnts(1_t_int:3_t_int,i)
       xx(:)=bandplot % p_kpnts(1_t_int:3_t_int,i)
       d=d+Sqrt(Sum((xx(:)-x0(:))**2_t_int))
       bandplot % p_kpnts(4_t_int,i)=d
       x0=xx
    Enddo
    Close(12)
    !!
  End Subroutine read_bandplot_mesh
  !!
  !!***********************************************************************
  !!
  Subroutine  prepaxisrot(xaxis,zaxis,lmax,wu,prot,trans)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)   :: prot,trans
    Integer(t_int),Intent(in)   :: lmax
    Real(t_real),Intent(in)     :: xaxis(3),zaxis(3)
    Real(t_real),Intent(out)    :: wu(1:(2*lmax+1)**2,0:lmax)
    !!LOCAL
    Integer(t_int) :: l,n1,j
    Real(t_real)   :: eulang(3),fac(0:2*lmax+1)
    Real(t_real)   :: r(3),ro(3)
    !!
    !!     new z-axis
    !! new x-axis, must be orthogonal, if not it is corrected in a
    !! certain way
    !! calc euler angles for the new system
    Call reuler2(zaxis,xaxis,eulang)
    !! prepare ytr input, for l<=lmax
    fac(0_t_int)=1.0_t_real
    Do j=1_t_int,2_t_int*lmax+1_t_int
       fac(j)=Real(j,t_real)*fac(j-1_t_int)
    Enddo
    !! calc rotation matrices
    Do l=0_t_int,lmax
       n1=l*(l*l*4_t_int-1_t_int)/3_t_int
       Call ytr(eulang,0_t_int,fac,l,wu(:,l),trans=trans)
    Enddo
    !! controll output
    If(prot) Write(outfile,400) zaxis
    r=zaxis
    Call rotxyz(r,wu(:,1),ro,trans)
    If(prot) Write(outfile,401)ro
    !!     
    If(prot) Write(outfile,402)xaxis
    r=xaxis
    Call rotxyz(r,wu(:,1),ro,trans)
    If(prot) Write(outfile,401)ro
    !!     
400 Format('z-Vector: ',3(f16.7,2x))
402 Format('x-Vector: ',3(f16.7,2x))
401 Format('becomes : ',3(f16.7,2x))
    !!
  End Subroutine prepaxisrot
  !!***********************************************************************
  Subroutine frel_prepaxisrot(xaxis,zaxis,lmax,cdjmm,prot)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: lmax !! maximum basis momentum
    Real(t_real),Intent(in)      :: xaxis(3),zaxis(3)
    Complex(t_real),Pointer      :: cdjmm(:,:) !! must be nullify and
    !!                                            pfreeed later
    !!LOCAL
    Character(*),Parameter::nsubfunc="frel_prepaxisrot"
    Integer(t_int) :: l,j,ij,m,mp
    Real(t_real)   :: fac(0:2*lmax+1),eulang(3),MM(3,3)
    !!
    Call palloc('cdjmm',cdjmm,1_t_int,(2*lmax+2)**2,1_t_int,(lmax+1) &
         &,nmodule,nsubfunc)

    !!
    Call make_factorials(fac,2*lmax+1)
    !!
    Call reuler2(zaxis,xaxis,eulang)
    Call m_euler(MM,eulang)
    !!
    If(prot) Then
       Write(outfile,400) zaxis
       Write(outfile,401) Dot_product(xaxis,MM(:,3))
       Write(outfile,402) xaxis
       Write(outfile,401) Dot_product(zaxis,MM(:,1))
    Endif
    !!
    !! prepare the rotation matrices (Dj functions)
    Do l=0,lmax
       j=2*l+1  !! this is 2*J
       
       !! degeneracy of j-shell is 2*j+1=2*l+2
       
       !! index of j-shell one-based: 
       ij=l+1 !! =(j+1)/2 = Abs(kappa)
       Call wignerDJ(j,eulang,cdjmm(:,ij),fac,.False.)
    Enddo
    !!
    If(prot)Then
       Do l=0,lmax
          j=2*l+1
          ij=l+1
          Write(outfile,"('J=',i1,'/2')")j
          Do m=1,j+1
             Do mp=1,j+1
                Write(outfile,"('(',f8.3,',',f8.3,')')",advance='no')&
                     &cdjmm(m+(j+1)*(mp-1),ij)
             Enddo
             Write(outfile,*)
          Enddo

       Enddo
    Endif
    !!
    Return
    !!
400 Format('z-Vector: ',3(f16.7,2x))
402 Format('x-Vector: ',3(f16.7,2x))
401 Format('becomes : ',3(f16.7,2x))
    !!
  End Subroutine frel_prepaxisrot
  !!***********************************************************************
  !! rotate vector: ro=wu*r, where wu is the p-state
  !! Ylm rotation matrix (see ytr), this makes the permutation neccesary.
  !!
  Subroutine rotxyz(r,wu,ro,trans)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: trans
    Real(t_real),Intent(in)     ::  r(3),wu(3,3)
    Real(t_real),Intent(out)    ::  ro(3)
    !!LOCAL
    Real(t_real)  :: dd,perm(3,3)
    !!
    dd=Sqrt(r(1)**2+r(2)**2+r(3)**2)
    !!
    perm(1,:)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
    perm(2,:)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
    perm(3,:)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
    If(trans)Then
       ro=Matmul(Matmul(Transpose(perm),Matmul(Transpose(wu),perm)),r)/dd
    Else
       ro=Matmul(Matmul(Transpose(perm),Matmul(wu,perm)),r)/dd
    Endif
    !!
  End Subroutine rotxyz
  !!***********************************************************************
  Subroutine read_old_ldos_file(ldos)
    !!
    Implicit None
    !!IO
    Type(T_LDOS),Intent(inout)   :: ldos
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_old_ldos_file"
    Integer(t_int) :: i
    Logical(t_log)       :: exist
    Character(200),Parameter   :: f_ldos='=.ldos'
    !!
    ldos% nldos=0_t_int
    !!     
    Call palloc('ldos% p_ldsites',ldos% p_ldsites &
         &,1_t_int,Max(1_t_int,ldos% nldos)&
         &,nmodule,nsubfunc)
    !!
    !! exists =.ldos
    Inquire(file=f_ldos,exist=exist)
    If(exist) Then
       !!     
       !!
       Open(12,file=f_ldos)
       Read(12,*) ldos% nldos
       !!
       Call palloc('ldos% p_ldsites',ldos% p_ldsites &
            &,1_t_int,Max(1_t_int,ldos% nldos)&
            &,nmodule,nsubfunc)
       !!
       Do i=1_t_int,ldos% nldos
          Read(12,*)ldos% p_ldsites(i)
       Enddo
       !!
       Close(12)
       !!
       
       !!
    Endif
    !!
    !!
  End Subroutine read_old_ldos_file
  !!***********************************************************************
  Subroutine setup_ldos(nsite,ldos,plotbands)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: plotbands
    Integer(t_int),Intent(in)    :: nsite
    Type(T_LDOS),Intent(inout)   :: ldos
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_ldos"
    Integer(t_int) :: i,isa,nuniq
    Logical(t_log)       :: doscalc
    !!
    !!
    Call palloc('ldos% p_siteld',ldos% p_siteld &
         &,1_t_int,nsite,nmodule,nsubfunc)
    !!
    ldos% plotldos=.False.
    !!
    !!
    doscalc=isoption('CALC_DOS')
    !!     
    !! dos or band calc ?
    If(.Not.(doscalc .Or. plotbands)) Goto 1000
    !!
    if(ldos% nldos < 1) goto 1000
    !!
    If(isoption('NO_DOS')) Goto 1000
    !!
    !!     
    ldos% plotldos=.True.
    !!
    Do i=1_t_int,ldos% nldos
       If(ldos% p_ldsites(i) < 1 .Or. ldos% p_ldsites(i) > nsite )Then
          ldos% p_ldsites(i)=nsite+1
       Endif
    Enddo
    Call ipiksrt1(ldos% p_ldsites,ldos% nldos)
    Call iuniq1(ldos% p_ldsites,ldos% nldos,nuniq)
    !!
    If(ldos% p_ldsites(nuniq) == nsite+1 )Then
       nuniq=nuniq-1
    Endif
    !!
    ldos% nldos=nuniq
    !!
    If(ldos% nldos <1 )Then
       ldos% plotldos=.False.
       Goto 1000
    Endif
    !!
    !!
    Do isa=1_t_int,nsite
       ldos% p_siteld(isa)=0_t_int
       Do i=1_t_int,ldos% nldos
          If(ldos% p_ldsites(i)==isa)Then
             ldos% p_siteld(isa)=i
          End If
       End Do
    End Do
    !!
    Write(outfile,1)ldos% nldos
    Write(outfile,2)
    Do isa=1_t_int,nsite
       If(ldos% p_siteld(isa)> 0_t_int)Then
          Write(outfile,3)isa,ldos% p_siteld(isa)
       End If
    Enddo
    !!
1   Format('local dos will be calculated for ',i4,' sites ')
2   Format(' site   definition ')
3   Format(i5,2x,i5)
    !!
    
    !!
    !!
1000 Continue
    !!
    !!
  End Subroutine setup_ldos
  !!***********************************************************************
  Subroutine setup_WeightRot(wr,lmax,rel,bweight_rot,xaxis,zaxis&
       &,lmx,relTOpseudo,prot)
    Implicit None 
    !!IO
    Type(T_WeightRot),Intent(inout) :: wr
    Integer(t_int),Intent(in)       :: lmax,lmx! max N/SREL basis angmomentum
    Real(t_real),Intent(in)         :: xaxis(3),zaxis(3)
    Logical(t_log),Intent(in)       :: rel,bweight_rot,prot
    Complex(t_real),Intent(in)      :: relTOpseudo(2*lmx+3,2*lmx+2,2&
         &,-lmx-1:lmx+1) 
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_WeightRot"
    Integer(t_int) :: l,deg,k,degj,ns,m2,mv,i0,ij
    Complex(t_real):: ca
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    !!
    If(bweight_rot)Then
       If(rel)Then
          Call frel_prepaxisrot(xaxis,zaxis,lmax,wr% p_cdjmm,.False.)
       Else
          Call palloc('wr% p_cdjmm',wr% p_cdjmm,1_t_int,1_t_int&
               &,1_t_int,1_t_int,nmodule,nsubfunc)
       Endif
       Call palloc('wr% p_wu',wr% p_wu,1_t_int,(2*lmax+1)**2&
            &,0_t_int,lmax,nmodule,nsubfunc)
       Call prepaxisrot(xaxis,zaxis,lmax,wr% p_wu,prot,.True.)
    Endif
    
    
    If(rel)Then
       Call palloc('wr% p_rotwhole',wr% p_rotwhole&
            &,1_t_int,2*(2*lmax+1)&
            &,1_t_int,2*(2*lmax+1)&
            &,0_t_int,lmax&
            &,nmodule,nsubfunc)
       wr% p_rotwhole=c0
       !!
       Do l=0,lmax 
          deg=2*l+1
          i0=2*l
          Do k=-l-1,l,deg
             If(k==0)Cycle
             degj=2*Abs(k)
             ij=degj/2
             If(bweight_rot)Then
                Do m2=1,degj
                   wr% p_rotwhole(i0+1:i0+degj,i0+m2,l)=&
                        & wr% p_cdjmm(1+degj*(m2-1):degj*m2,ij)
                Enddo
             Else
                Do m2=1,degj
                   wr% p_rotwhole(i0+m2,i0+m2,l)=c1
                Enddo
             Endif
             i0=i0-2*l
          Enddo! k
       Enddo! l
    Else
       Call palloc('wr% p_rotwhole',wr% p_rotwhole&
            &,1_t_int,2*lmax+1&
            &,1_t_int,2*lmax+1&
            &,0_t_int,lmax&
            &,nmodule,nsubfunc)
       wr% p_rotwhole=c0
       Do l=0,lmax 
          deg=2*l+1
          If(bweight_rot)Then
             Do m2=1,deg
                wr% p_rotwhole(1:deg,m2,l)=wr% p_wu(1+deg*(m2-1):deg*m2,l)
             Enddo
          Else
             Do m2=1,deg
                wr% p_rotwhole(m2,m2,l)=c1
             Enddo
          Endif
       Enddo! l

    Endif
    !!
    !!
    !! lms-projection
    If(rel)Then
       Call palloc('wr% p_relToNrel',wr% p_relToNrel&
            &,1_t_int,(2*lmax+3)&
            &,1_t_int,(2*lmax+2)&
            &,1_t_int,2_t_int&
            &,(-lmax-1),(lmax)&
            &,nmodule,nsubfunc)
       Call palloc('wr% p_relToNrelwhole',wr% p_relToNrelwhole&
            &,1_t_int,2*(2*lmax+1)&
            &,1_t_int,2*(2*lmax+1)&
            &,0_t_int,lmax&
            &,nmodule,nsubfunc)

       wr% p_relToNrelwhole=c0
       
       Do l=0,lmax 
          deg=2*l+1
          i0=2*l
          Do k=-l-1,l,deg
             If(k==0)Cycle
             degj=2*Abs(k)
             If(bweight_rot)Then
                Do m2=1,degj
                   Do mv=1,deg
                      Do ns=1,2
                         ca=Sum(wr% p_wu(1+deg*(mv-1):deg*mv,l)&
                              &*relTOpseudo(1:deg,m2,ns,k))
                         wr% p_relToNrel(mv,m2,ns,k)=ca
                         wr% p_relToNrelwhole(i0+m2,2*(mv-1)+ns,l)=Conjg(ca)
                      Enddo
                   Enddo
                Enddo
             Else
                Do ns=1,2
                   wr% p_relToNrel(1:deg,1:degj,ns,k)=&
                        &relTOpseudo(1:deg,1:degj,ns,k)
                   Do m2=1,degj
                      Do mv=1,deg
                         wr% p_relToNrelwhole(i0+m2,2*(mv-1)+ns,l)=&
                              & Conjg(relTOpseudo(mv,m2,ns,k))
                      Enddo
                   Enddo
                Enddo
             Endif
             i0=i0-2*l
          Enddo ! k
       Enddo!l
    Endif
    !!
  End Subroutine setup_WeightRot
  !!***********************************************************************
  Subroutine setup_ldosRot(symmetry,basis,wr,rel)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_WeightRot),Intent(inout) :: wr
    Logical(t_log),Intent(in)    :: rel
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_ldosRot"
    Integer(t_int) :: l,jops,deg,fac,lmax
    !!
    lmax=basis% lvmx
    fac=1
    If(rel) Then
       fac=2
       lmax=2*basis% lvmx+1
    Endif
    !!
    Call palloc('wr% p_wu_ks',wr% p_wu_ks&
         &,1_t_int,fac*(2*basis% lvmx+1)&
         &,1_t_int,fac*(2*basis% lvmx+1)&
         &,0_t_int,lmax&
         &,1_t_int,48_t_int&
         &,nmodule,nsubfunc)
    !!
    Do jops=1,symmetry% Rnumops
       Do l=0,basis% lvmx
          deg=fac*(2*l+1)
          !! w^{+}*K*D^{+}*C*K * K*C^{+}*D*K*w
          If(symmetry% Roptimerev(jops)==1)Then
             wr% p_wu_ks(1:deg,1:deg,l,jops)=&
                  &Matmul(symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)&
                  &,Conjg(wr% p_rotwhole(1:deg,1:deg,l)))
          Else
             wr% p_wu_ks(1:deg,1:deg,l,jops)=&
                  &Matmul(symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)&
                  &,wr% p_rotwhole(1:deg,1:deg,l))
          Endif
          If(rel)Then
             !! w^{+}*K*D^{+}*C*K * K*C^{+}*D*K*w
             If(symmetry% Roptimerev(jops)==1)Then
                wr% p_wu_ks(1:deg,1:deg,l+basis% lvmx+1,jops)=&
                     &Matmul(symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)&
                     &,Conjg(wr% p_relToNrelwhole(1:deg,1:deg,l)))
             Else
                wr% p_wu_ks(1:deg,1:deg,l+basis% lvmx+1,jops)=&
                     &Matmul(symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)&
                     &,wr% p_relToNrelwhole(1:deg,1:deg,l))
             Endif
          Endif
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine setup_ldosRot
  !!***********************************************************************
  Subroutine averageDegenerateSubspaces(io,iu,eb,nw,p_weights,p_wsum,wtol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: io,iu! lower upper band index
    Integer(t_int),Intent(in)    :: nw! number of weights
    Real(t_real),Intent(in)      :: wtol! tolerance to determine degenracy
    Real(t_real),Intent(in)      :: eb(:)! band energies
    Real(t_real),Intent(inout)   :: p_weights(:,:),p_wsum(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="averageDegenerateSubspaces"
    Integer(t_int) :: in,in1,in2,iw
    Real(t_real)   :: e,d,w
    !!
    !!
    !! average over degenerate energies
    If(iu>=io)Then
       in=io
       e=eb(in)
       in1=in
       in2=in
       bandloop: Do in=io+1,iu
          d=Abs(e-eb(in))
          If(Abs(e)>1.0_t_real)Then
             d=d/Abs(e)
          Endif
          If(d<wtol)Then
             in2=in
          Else
             !! take average of weights for degenerate bands
             Do iw=1,nw
                w=Sum(p_weights(iw,in1-io+1:in2-io+1))/Real(in2-in1+1,t_real)
                p_weights(iw,in1-io+1:in2-io+1)=w
             Enddo
             w=Sum(p_wsum(in1-io+1:in2-io+1))/Real(in2-in1+1,t_real)
             p_wsum(in1-io+1:in2-io+1)=w
             e=eb(in)
             in1=in
             in2=in
          Endif
       Enddo bandloop
       Do iw=1,nw
          w=Sum(p_weights(iw,in1-io+1:in2-io+1))/Real(in2-in1+1,t_real)
          p_weights(iw,in1-io+1:in2-io+1)=w
       Enddo
       w=Sum(p_wsum(in1-io+1:in2-io+1))/Real(in2-in1+1,t_real)
       p_wsum(in1-io+1:in2-io+1)=w
    Endif
    !!
    Return
    !!
  End Subroutine averageDegenerateSubspaces
  !!***********************************************************************
  
End Module bandplot
