Module clock
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  !!
  Private 
  !!
  Public :: watch,cpu_start,cpu_stop,cpu_delete,cpu_reset,cpu_read
  !!***********************************************************************
  Character(*),Parameter::nmodule="CLOCK"
  !!
  Integer(t_int),Parameter :: msgsize=80_t_int
  !!
  Type T_Time
     Integer(t_int) :: ntask
     Real(t_real),Pointer :: p_tstart(:)
     Real(t_real),Pointer :: p_ttot(:)
     Character(msgsize),Pointer :: p_ctask(:)
     Logical(t_log),Pointer :: p_running(:)!! is it currently running
     Logical(t_log),Pointer :: p_used(:)!! is it in use?
     !! for progress reporting
     Integer(t_int),Pointer :: p_nsteps(:)
     Integer(t_int),Pointer :: p_subdiv(:)
     Logical(t_log),Pointer :: p_printeta(:)
     Real(t_real),Pointer   :: p_eta(:)
  End Type T_Time
  !!
  Logical(t_log) :: timesinitialized=.False.
  Type(T_Time) ::  times
  !!
  !!
  Integer(t_int),Parameter :: cpu_start  =1_t_int
  Integer(t_int),Parameter :: cpu_stop   =2_t_int
  Integer(t_int),Parameter :: cpu_delete =3_t_int
  Integer(t_int),Parameter :: cpu_reset  =4_t_int
  Integer(t_int),Parameter :: cpu_read   =5_t_int
Contains

  !!***********************************************************************
  Subroutine init_time(ti)
    !!
    Implicit None
    !!IO
    Type(T_Time),Intent(inout)         :: ti
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_time"
    !!
    ti% ntask=0_t_int
    Nullify(ti% p_tstart)
    Nullify(ti% p_ttot)
    Nullify(ti% p_ctask)
    Nullify(ti% p_running)
    Nullify(ti% p_used)
    Nullify(ti% p_nsteps)
    Nullify(ti% p_subdiv)
    Nullify(ti% p_printeta)
    Nullify(ti% p_eta)
    !!
    Return
    !!
  End Subroutine init_time
  !!***********************************************************************
  Subroutine init_task(ti,itask)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: itask
    Type(T_Time),Intent(inout)   :: ti
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_task"
    !!
    !!
    ti% p_running(itask)=.False.
    ti% p_ttot(itask)=0.0_t_real
    ti% p_tstart(itask)=0.0_t_real
    ti% p_nsteps(itask)=0_t_int
    ti% p_subdiv(itask)=1_t_int
    ti% p_printeta(itask)=.False.
    ti% p_eta(itask)=0.0_t_real
    !!
    Return
    !!
  End Subroutine init_task
  !!***********************************************************************
  Subroutine timesrealloc(ti,newsize)
    !!
    Implicit None
    !!IO
    Type(T_Time),Intent(inout)         :: ti
    Integer(t_int),Intent(in)    :: newsize
    !!LOCAL
    Character(*),Parameter::nsubfunc="timesrealloc"
    !!
    !!
    If(ti% ntask>newsize) Return
    !!
    Call realloc('ti% p_tstart',ti% p_tstart,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_ttot',ti% p_ttot,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_ctask',ti% p_ctask,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_running',ti% p_running,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_used',ti% p_used,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_nsteps',ti% p_nsteps,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_subdiv',ti% p_subdiv,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_printeta',ti% p_printeta,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('ti% p_eta',ti% p_eta,1_t_int,newsize,nmodule,nsubfunc)
    ti% ntask=newsize
    !!
    Return
    !!
  End Subroutine timesrealloc
  !!***********************************************************************
  Subroutine watch(ctask0,action,reset,start,delete,Stop,Print&
       &,printprogress,mesg,sccmesg,Nsteps,istep,deltaProgress)
    !!- Calculates cpu- time for a certain task.   	
    !!-----------------------------------------------------------------------
    !!i Inputs:
    !!i   ctask0:string for the task.
    !!i   print:if T then result is printed
    !!i   start :if T then starts clock again after reset
    !!i   stp   :if T stops clock after read
    !!i   delete :if T deletes clock
    !!i   reset   :if T resets clock
    !!o Outputs:
    !!o   time  :cpu time of the task.
    !!r Remarks:
    !!r    cpu looks if ctask0 is already in the list; if it
    !!r    is not found, a new task is added and ntask incremented by 1.
    !!r
    !!r    entries:
    !!r    cpu_start: (re)start clock
    !!r    cpu_stop : stop clock
    !!r    cpu_reset: reset clock
    !!r    cpu_delete: delete this clock
    !!r    cpu_read : read clock
    !!r
    !!r    cpu invokes subroutines which are supplied by the system and
    !!r    thus may depend on the installation.
    !!r    cpu has to be called twice: at start time
    !!r    and at end time of the task.
    !!r    cpu allows to handle maxtsk time measurements simultaneoulsy.
    !!-----------------------------------------------------------------------
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: action
    Character(*),Intent(in),Optional   :: sccmesg
    Logical(t_log),Intent(in),Optional :: delete
    Logical(t_log),Intent(in),Optional :: print
    Logical(t_log),Intent(in),Optional :: printprogress
    Logical(t_log),Intent(in),Optional :: start
    Logical(t_log),Intent(in),Optional :: reset
    Logical(t_log),Intent(in),Optional :: stop
    Integer(t_int),Intent(in),Optional    :: Nsteps,istep
    Real(t_real),Intent(in),Optional      :: deltaProgress
    Character(*),Intent(in),Optional :: mesg
    Character(*),Intent(in):: ctask0
    !!LOCAL
    Character(*),Parameter::nsubfunc="watch"
    Integer(t_int)    :: itask,i,jtask,nst,ist
    Real(t_real)      :: factor,tnow,ddum,dpr,time,p,dest
    Character(msgsize):: ltask0
    Logical(t_log)    :: del,pr,strt,stp,rst,prpro
    Character(100)    :: scc,est
    External cgetcpuclock
    !!
    If(.Not.timesinitialized) Then
       Call init_time(times)
       timesinitialized=.True.
    Endif
    !!
    ltask0=Trim(ctask0)

    rst=.False.
    del=.False.
    pr=.False.
    prpro=.False.
    strt=.False.
    stp=.False.
    nst=0_t_int
    ist=0_t_int
    dpr=0.1_t_real
    scc=""
    !!
    If(Present(reset))   rst=reset
    If(Present(delete))  del=delete
    If(Present(print))   pr=print
    If(Present(printprogress))   prpro=printprogress
    If(Present(start))   strt=start
    If(Present(Stop))    stp=Stop
    If(Present(sccmesg)) scc=Trim(sccmesg)
    If(Present(Nsteps))  nst=Nsteps
    If(Present(istep))   ist=istep
    If(Present(deltaProgress))  dpr=deltaProgress
    !!
    
!!$    Print *,'ltask0 ',Trim(ltask0)
!!$    Do i=1,times% ntask
!!$       Print "('task ',i4,': ',a,' used ',l1,' run ',l1)"&
!!$            &,i,Trim(times% p_ctask(i)),times% p_used(i),times% p_running(i)
!!$    Enddo
    
    !! --- check if clock in list
    itask=-1
    jtask=-1
    Do i=1,times% ntask
       If(.Not.times% p_used(i).And.jtask<0)Then
          jtask=i
       endif
       If (times% p_used(i) .And. Trim(ltask0)==Trim(times% p_ctask(i))) Then
          itask=i
          Exit
       Endif
    Enddo
    If(itask<0)Then
       !! got none
       If(jtask>0)Then
          !! got unused
          itask=jtask
       Else
          !! need new
          itask=times% ntask+1
          Call timesrealloc(times,itask)
       Endif
       Call init_task(times,itask)
       times% p_ctask(itask)=ltask0
    Endif
    times% p_used(itask)=.True.

    !Print *,'itask ',itask,Trim(times% p_ctask(itask))
    
    !! get actual time
    Call cgetcpuclock(ddum)
    tnow=Real(ddum,t_real)
    factor=1.0_t_real
    !!#elseifC TRANSPUTER
    !!      call iclock(secst)
    !!#endif
    tnow=tnow*factor
    
    !Print *,'clock ',Trim(times% p_ctask(itask))
    !Print *,'running ',times% p_running(itask)
    !Print *,'action ',action
    
    !! do actions
    If (action==cpu_start) Then
       If (times% p_running(itask)) Then
          Write(errbuf,400) Trim(times% p_ctask(itask))
          Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       Endif
       times% p_tstart(itask)=tnow
       times% p_running(itask)=.True.
       If (rst) times% p_ttot(itask)=0.0_t_real
       time=times% p_ttot(itask)
       If(nst>=0)Then
          times% p_nsteps(itask)=nst
          times% p_subdiv(itask)=(Max(1_t_int,nst/(100/Nint(1.0_t_real/dpr))))
          times% p_printeta(itask)=.True.
       Endif
       !Print *,'running ',times% p_running(itask)
       Return
    Elseif (action==cpu_stop) Then
       If (.Not.times% p_running(itask)) Then
          Write(errbuf,401)Trim(times% p_ctask(itask))
          Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
          pr=.False.
       Else
          times% p_ttot(itask)=times% p_ttot(itask)+tnow-times% p_tstart(itask)
          times% p_running(itask)=.False.
          time=times% p_ttot(itask)
       Endif
    Elseif (action==cpu_read) Then
       !! read clock
       If (times% p_running(itask)) Then
          time=times% p_ttot(itask)+tnow-times% p_tstart(itask)
          If (stp) Then
             times% p_ttot(itask)=time
             times% p_running(itask)=.False.
          Endif
       Else
          time=times% p_ttot(itask)
       Endif
    Elseif (action==cpu_reset) Then
       !! reset clock
       times% p_ttot(itask)=0.0_t_real
       time=0.0_t_real
       If (strt) Then
          times% p_tstart(itask)=tnow
          times% p_running(itask)=.True.
       Else
          times% p_running(itask)=.False.
       Endif
       !Print *,'running ',times% p_running(itask)
       Return
    Elseif (action==cpu_delete) Then
       !! delete clock
       time=times% p_ttot(itask)
       del=.True.
    Endif
    
    !Print *,'nstep=',times% p_nsteps(itask)
    !Print *,'subdi=',times% p_subdiv(itask)
    !Print *,'istep=',ist
    !Print *,'preta=',times% p_printeta(itask)
    !Print *,'prpro1=',prpro
    If(times% p_nsteps(itask)==0) prpro=.False.
    !Print *,'prpro2=',prpro
    If(prpro.And..Not.pr)Then
       !Print *,'GOT THROUGH'
       p=100.0_t_real*ist/times% p_nsteps(itask)
       !Print *,'p     =',p
       !Print *,'deltat=',time
       dest=time*times% p_nsteps(itask)/ist
       If(time>0.0_t_real.And.ist>0)Then
          !Print *,'dest=',dest
          !Print *,'eta =',times% p_eta(itask)
          times% p_printeta(itask)=.False.
          If(Abs(times% p_eta(itask)-dest)/dest>0.10_t_real)Then
             times% p_printeta(itask)=.True.
          Endif
          !Print *,'preta=',times% p_printeta(itask)
          times% p_eta(itask)=dest
          Write(est,"(f10.2,'s')")dest
       Else
          est='N/A'
       Endif
       If(times% p_printeta(itask) .Or. &
            &Mod(ist,times% p_subdiv(itask)) ==0)Then
          pr=.True.
       Endif
    Endif


    If(pr)Then
       If(Present(mesg))Then
          If(prpro)Then
             Write(outfile,3001)Trim(scc),mesg,Trim(ltask0),time,p,Trim(Adjustl(est))
          Else
             Write(outfile,3000)Trim(scc),Trim(scc),mesg,Trim(ltask0),time,Trim(scc)
          Endif
       Else
          If(prpro)Then
             Write(outfile,301)Trim(scc),Trim(ltask0),time,p,Trim(Adjustl(est))
          Else
             Write(outfile,300)Trim(scc),Trim(scc),Trim(ltask0),time,Trim(scc)
          Endif
       Endif
    Endif

!!$    Print *,'del ',del
!!$    Print *,'running ',times% p_running(itask)


    If (del) Then
       times% p_used(itask)=.False.
       Call init_task(times,itask)
    Endif
    
    
300 Format(a,' ',/,a,' CPU   : ',a,':',' cpu time:',f15.2,' sec',/,a,' ')
301 Format(a,' CPU   : ',a,':',' cpu time:',f15.2,' sec ',f6.2,'% done ETA='&
         &,a)
3000 Format(a,' ',/a,' CPU   : ',a,a,':',' cpu time:',f15.2,' sec',/,a,' ')
3001 Format(a,' CPU   : ',a,a,':',' cpu time:',f15.2,' sec ',f6.2,'% done ETA='&
          &,a)
400 Format( "'clock """,a,""" already started!'")
401 Format( "'clock ",a," not running!'")
  End Subroutine watch
  !!***********************************************************************

End Module clock
