Module DENSIO
  !!
  Use precisiondefs
  Use errors
  Use constants
  Use file_iounits
  Use memory
  Use tlsdau
  Use topc
  Use tio
  Use clock
  Use io
  Use titeration
  Use iteration
  Use finuc
  Use tatom
  Use diracdata
  Use trelativistic
  Use diracdriver
  Use diracio
  Use diracpotorb
  Use tnumerics
  Use ioadapter
  Use tsymmetry
  Use tpotentialdensity
  Use tbzone
  Use tmesh
  Use lsdau
  Use mesh
  Use matserv
  Use ioaux
  Use opcorrection
  Use density
  Use atomdefs
  Use prnt
  Use optionstest
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="DENSIO"  
  !!
  Public :: densload,denswrite,init_dens,debugdensvectorout
  !!
  Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
  !!
  Character(*),Parameter :: f_densmap='=.densmap'
  !!***********************************************************************
  Type T_MapData
     Integer(t_int) :: newsite
     Integer(t_int) :: oldsite
     Integer(t_int) :: oldsort
     Character(1000) :: file
  End Type T_MapData
  !!
  Type T_DensMapData
     Integer(t_int) :: nslots
     ! for the followiong nsite is used in allocation
     Integer(t_int),Pointer :: p_ninslot(:) ! nslots 
     Type(T_MapData),Pointer:: p_md(:,:)    ! p_nonslot(),nslots
  End Type T_DensMapData
  !!***********************************************************************
Contains 
  !!***********************************************************************
  !!
  !!   Read density from disc and storage in array
  !!
  Subroutine densload(compound,symmetry,structure,potential,numerics,mspin&
       &,initial_spin_split,spin_split,efermi,Bfsm,opc,lsdau&
       !&,bzone&
       &,relativistic)
    Implicit None
    !!IO
    Character(*),Intent(in)          :: compound
    Type(T_Lsdau),Intent(inout) :: lsdau
    Type(T_OPC),Intent(inout)   :: opc
    Type(T_Structure),Intent(inout)  :: structure
    Type(T_Symmetry),Intent(in)      :: symmetry
    Type(T_Numerics),Intent(in)      :: numerics
    Type(T_Potential),Intent(inout)  :: potential
    !Type(T_Bzone),Intent(in)         :: bzone
    Integer(t_int),Intent(in)        :: relativistic
    Logical(t_log),Intent(inout)     :: initial_spin_split
    Integer(t_int),Intent(in)        :: mspin
    Real(t_real),Intent(in)          :: spin_split(*)
    Real(t_real),Intent(out)         :: efermi,Bfsm
    !!LOCAL
    Character(*),Parameter::nsubfunc="densload"
    Type(T_DensMapData):: dmd
    Integer(t_int) :: ver,i,j
    Logical(t_log) :: exists,rel
    Logical(t_log) :: mapexists
    !!------------------------------------------------------------------------
    !!
    !! load dens from file and reinterpolate if different settings
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    If(lsdau% lsdau .And. lsdau% functional == LSDAU_FIXEDPOT)Then
       Inquire(file='=.vmat',exist=exists)
       If(.Not.exists)Then
          Call write_u_dmat_to_file(mspin,structure,lsdau,'vmat',rel)
          Write(outfile,"('File ''=.vmat'' created... edit it!')")
          Call terminate(TERM_NORMAL,'Created =.vmat!')
       Endif
       Write(outfile,"('Reading potential matrix from file ''=.vmat''')")
       Call read_u_dmat_from_file(mspin,symmetry,structure,lsdau,'vmat',rel)
    Endif
    
    potential % p_lcd=0.0_t_real
    Inquire(file=f_densmap,exist=mapexists)
    If(mapexists)Then
       Write(outfile,"(/,/,72('='),/,72('='),/&
            &,'=.densmap exists: will start mapping')")
       Call init_densmapdata(dmd)
       Call readdensmap(structure% nsite,dmd)
       !!
       !! todo: opc and lsdau including =.dmat_init and scratch
       If(Any(dmd% p_md(1,1:dmd% nslots)% oldsite ==0))Then
          Write(outfile,"('Need some atomic start densities: &
               &running atom program ...')")
          Call init_dens(compound,structure&
               &,potential,initial_spin_split,opc,lsdau,relativistic&
               !&,bzone&
               &,potential% make_vca,.True.)
          Call densin(f_dens,symmetry,structure % nsite&
               &,structure % p_isrt&
               &,structure % p_elements,structure% p_equivsites&
               &,efermi,Bfsm,potential% p_pmeshs,potential% nrp&
               &,potential % mlmp,potential % p_nlmp&
               &,potential % p_lmp,potential % p_lcd&
               &,mspin,opc,lsdau,rel)
       Endif
       
       Write(outfile,"(/,/,'Different =.dens files: ',i3)")dmd% nslots

       Do i=1,dmd% nslots
          If(Len_trim(dmd% p_md(1,i)% file)>0)Then
             Write(outfile,"(/,'Mapping from file ',a,/&
                  &,' current-site site/sort-from file')")&
                  &Trim(dmd% p_md(1,i)% file)
             Do j=1,dmd% p_ninslot(i)
                Write(outfile,"(i7,5x,i5,i5)")&
                     &dmd% p_md(j,i)% newsite&
                     &,dmd% p_md(j,i)% oldsite&
                     &,dmd% p_md(j,i)% oldsort
             Enddo
             Write(outfile,*)

             Call densin(dmd% p_md(1,i)% file,symmetry,structure % nsite&
                  &,structure % p_isrt&
                  &,structure % p_elements,structure% p_equivsites&
                  &,efermi,Bfsm,potential% p_pmeshs,potential% nrp&
                  &,potential % mlmp,potential % p_nlmp&
                  &,potential % p_lmp,potential % p_lcd&
                  &,mspin,opc,lsdau,rel,dmd% p_ninslot(i),dmd% p_md(:,i))
          Endif
       Enddo

       Call density_analysis(structure,mspin,potential,numerics,0_t_int&
            &,relativistic,dontshow=.True.)
       
       !!Print *,Sum(potential% p_charges(:,1))-potential% tot_nuc_charge
       
       Call recharge(mspin,structure,potential)
    
       !Call density_analysis(structure,mspin,potential,numerics,0_t_int&
       !     &,relativistic,dontshow=.false.)
       !Print *,Sum(potential% p_charges(:,1))-potential% tot_nuc_charge

       
       Call denswrite(compound,structure,mspin&
            &,potential% p_pmeshs,potential% nrp,efermi,Bfsm&
            &,potential % mlmp,potential % p_nlmp,potential % p_lmp&
            &,potential % p_lcd,opc,lsdau,relativistic,.False.&
            !&,bzone&
            &)
       
       Write(errbuf,"('''File ',a,' was used to map the density!'',/&
            &,''In order to avoid unwanted overwrites, move/rename &
            &the file.''')")&
            &Trim(f_densmap)
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       
       Call clean_densmapdata(dmd)
       
       Call terminate(TERM_NORMAL,'Exported mapped density.')
    Else
       
       Call densin(f_dens,symmetry,structure % nsite&
            &,structure % p_isrt&
            &,structure % p_elements&
            &,structure% p_equivsites&
            &,efermi,Bfsm,potential% p_pmeshs,potential% nrp&
            &,potential % mlmp,potential % p_nlmp&
            &,potential % p_lmp,potential % p_lcd&
            &,mspin,opc,lsdau,rel)
       
    Endif
    
    Call read_u_dmat_from_file(mspin,symmetry,structure,lsdau,'dmat',rel)
    
    !!     make initial spin split
    Call initsplit(potential % p_lcd,potential% p_pmeshs,potential% nrp&
         &,potential % mlmp,structure% nsite,structure% p_isrt&
         &,structure% p_isrtTOIOisrt&
         &,potential% p_natom,mspin&
         &,initial_spin_split,spin_split&
         &,lsdau,symmetry,structure% p_equivsites,structure% p_elements,rel) 
    
    
    !!START export old density
    Call write_fplo1_density(mspin,structure% nsite,structure% nsort&
         &,structure% p_isrt&
         &,structure% p_elements,potential% mlmp,potential% p_nlmp&
         &,potential% p_lmp,potential% p_pmeshs,potential% nrp,efermi,Bfsm&
         &,potential% p_lcd,8_t_int)
    
    !!END   export old density
    
    
    Inquire(file='=.densconvert',exist=exists)
    If(exists)Then
       Open(12,file='=.densconvert')
       Read(12,*)ver
       Close(12)
       If(ver==3)Then
          Call denswrite(compound,structure,mspin&
               &,potential% p_pmeshs,potential% nrp,efermi,Bfsm&
               &,potential% mlmp,potential% p_nlmp,potential% p_lmp&
               &,potential% p_lcd,opc,lsdau,relativistic,.True.&
               !&,bzone&
               &,.True.)
          Call terminate(TERM_NORMAL,'Exported version 3 density format!')
       Elseif(ver<=9) Then
          Call denswrite(compound,structure,mspin&
               &,potential% p_pmeshs,potential% nrp,efermi,Bfsm&
               &,potential % mlmp,potential % p_nlmp,potential % p_lmp&
               &,potential % p_lcd,opc,lsdau,relativistic,.False.&
               !&,bzone&
               &,.True.)
          Call terminate(TERM_NORMAL&
               &,'Exported version 4...9 density format!')
       Endif
    Endif
    !! 
    !!
    Return
    !!
  End Subroutine densload
  !!***********************************************************************
  Subroutine read_u_dmat_from_file(mspin,symmetry,structure,lsdau,which,rel)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: which
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lsdau),Intent(inout)  :: lsdau
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_u_dmat_from_file"
    Integer(t_int) :: isa,ms,l,m,m1,deg,isau,i,j,nspin
    Character(2)   :: label,ele
    Character(20)   :: str
    Character(10000):: buf,word
    Integer :: eof
    Logical(t_log) :: exists
    Character(100) :: filename
    Real(t_real)   :: d,unit
    Complex(t_real):: cc
    Real(t_real),Pointer :: rdmat(:,:)
    Real(t_real),Pointer :: idmat(:,:)
    Complex(t_real),Pointer :: dmat(:,:,:,:)
    Complex(t_real),Pointer :: ldmat(:,:,:,:)
    Complex(t_real),Pointer :: ldmatoff(:,:,:)
    !!
    Nullify(rdmat,idmat,ldmat,ldmatoff)
    !!
    !!
    !!
    If(Trim(which)=='dmat')Then
       filename='=.dmat_init'
       dmat=>lsdau% p_dmat
       unit=1.0_t_real
    Else
       filename='=.vmat'
       dmat=>lsdau% p_vmat
       unit=1.0_t_real/hatoev
    Endif
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    !!
    Inquire(file=Trim(filename),exist=exists)
    If(.Not. exists) Return
    !!
    nspin=mspin
    If(rel) nspin=1
    !!
    Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
    !!
    ldmat(:,:,:,:)=c0
    !!
    !!
    !!
    Open(12,file=Trim(filename))
    
    readloop: Do
       
       Read(12,1,iostat=eof)buf
       If(eof < 0 ) Exit readloop
       
       
       If(Len_trim(buf)==0)Then
          !Print "('[empty line]')"
          Cycle readloop
       Endif
       
       
       word=Trim(buf)
       If(word(1:1)=='#')Then
          !Print "('[comment line]')"
          Cycle readloop
       Endif
       
       
       !Print "('[',a,']')",Trim(buf)
       
       
       isa=0
       label="  "
       str="  "
       If(next_word(buf,word))Then
          !Print *,'word=[',Trim(word),']'
          ele=word(1:2)
       Endif
       
       If(next_word(buf,word))Then
          !Print *,'word=[',Trim(word),']'
          Read(word,*)isa
       Endif
       
       If(next_word(buf,word))Then
          !Print *,'word=[',Trim(word),']'
          Read(word,*)label
       Endif
       l=lcharqn(label(2:2))
       If(next_word(buf,word))Then
          !Print *,'word=[',Trim(word),']'
          Read(word,*)str
       Endif
       If(uc_compare(str(1:2),'UP'))Then
          ms=1
       Else If(uc_compare(str(1:4),'DOWN'))Then
          ms=2
       Else
          ms=12
       Endif
       Write(outfile,"('element=',a2,' site=',i5,' state=',a2,' spin=',i2,' l=',i1)")&
            &ele,isa,label,ms,l
       
       deg=2*l+1
       Call palloc('rdmat',rdmat,1_t_int,deg,1_t_int,deg,nmodule,nsubfunc)
       Call palloc('idmat',idmat,1_t_int,deg,1_t_int,deg,nmodule,nsubfunc)

       !Print *,'before real',ms,deg
       
       !! read 2l+1 lines real part
       m=0
       mloop: Do
          
          Read(12,1,iostat=eof)buf
          If(eof < 0 ) Exit readloop
          
          If(Len_trim(buf)==0)Then
             !Print "('[empty line]')"
             Cycle mloop
          Endif
          
          
          word=Trim(buf)
          If(word(1:1)=='#')Then
             !Print "('[comment line]')"
             Cycle mloop
          Endif
          
          m=m+1
          !Print "('dmat[',a,']')",Trim(buf)          
          
          !! here we may read the values
          m1loop: Do m1=1,deg
             If(next_word(buf,word))Then
                Read(word,*)d
                rdmat(m,m1)=d
             Else
                Exit m1loop
             Endif
          Enddo m1loop
          !!
          
          If(m >= deg) Then
             !! store dmat
             !Do isau=1,lsdau% nusite
             !Enddo
             Exit mloop
          Endif
       Enddo mloop
       
       !Print *,'before imag',ms
       
       !! read 2l+1 lines imag part
       m=0
       mloop2: Do
          
          Read(12,1,iostat=eof)buf
          If(eof < 0 ) Exit readloop
          
          If(Len_trim(buf)==0)Then
             !Print "('[empty line]')"
             Cycle mloop2
          Endif
          
          
          word=Trim(buf)
          If(word(1:1)=='#')Then
             !Print "('[comment line]')"
             Cycle mloop2
          Endif
          
          m=m+1
          !Print "('dmat[',a,']')",Trim(buf)          
          
          !! here we may read the values
          m1loop2: Do m1=1,deg
             If(next_word(buf,word))Then
                Read(word,*)d
                idmat(m,m1)=d
             Else
                Exit m1loop2
             Endif
          Enddo m1loop2
          !!
          If(m >= deg) Then
             !! store imag part dmat
             Do isau=1,lsdau% nusite
                If( (isa == lsdau% p_isiteu(isau)) .And. &
                     & ( label == lsdau% p_labelsu(isau) )) Then
                   Do i=1,deg
                      Do j=1,deg
                         cc=Cmplx(rdmat(i,j),idmat(i,j),t_real)*unit
                         If(rel)Then
                            If(ms/=12)Then
                               If(mspin==2)Then
                                  ldmat(i+deg*(ms-1),j+deg*(ms-1),isau,1)=cc
                               Else
                                  ldmat(i+deg*(ms-1),j+deg*(ms-1),isau,1)=cc
                                  If(ms==1) ldmat(i+deg,j+deg,isau,1)=cc
                               Endif
                            Else
                               ldmat(i,j+deg,isau,1)=cc
                               ldmat(j+deg,i,isau,1)=Conjg(cc)
                            Endif
                         Else
                            If(ms/=12)Then
                               If(mspin==2)Then
                                  ldmat(i,j,isau,ms)=cc
                               Else
                                  ldmat(i,j,isau,1)=ldmat(i,j,isau,1)+cc*0.5_t_real
                               Endif
                            Endif
                         Endif
                      Enddo
                   Enddo
                Endif
             Enddo
             Exit mloop2
          Endif
       Enddo mloop2
       
       
    Enddo readloop
    Close(12)
    !!
    !!
    Call pfree(rdmat)
    Call pfree(idmat)
    !!
    !!
    Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,'fromspin')
    !!
    Call force_u_mat_sym(symmetry,structure% p_equivsites,lsdau,rel,mspin,ldmat,dmat)
    !!
    Call force_u_hermit(lsdau,nspin,dmat)
    !!
    !!
    Call pfree(ldmat)
    Call pfree(ldmatoff)
    !!
    !!
    !!
    Call print_u_dmat(mspin,structure% p_elements,lsdau,rel,dmat,which)
    !!
!    Stop
    !!
    Return
    !!
1   Format(a)
    !!
  End Subroutine read_u_dmat_from_file
  !!***********************************************************************
  Subroutine write_u_dmat_to_file(mspin,structure,lsdau,which,rel)
    !!
    Implicit None  
    !!IO
    Character(*),Intent(in)      :: which
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lsdau),Intent(in)     :: lsdau
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="write_u_dmat_to_file"
    Integer :: lunit
    Character(100) :: filename
    Complex(t_real),Pointer :: dmat(:,:,:,:)
    !!
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    If(Trim(which)=='dmat')Then
       filename='=.dmat_init'
       dmat=>lsdau% p_dmat
    Else
       filename='=.vmat'
       dmat=>lsdau% p_vmat
    Endif
    !!
    Write(outfile,"('START: ',a)")nsubfunc
    !!
    !!
    lunit=12
    Open(lunit,file=Trim(filename),status='unknown')
    Call print_u_dmat(mspin,structure% p_elements,lsdau,rel,dmat,which,lunit)
    Close(lunit)
    !!
    Write(outfile,"('END  : ',a)")nsubfunc
    !!
    !!
    !!
    Return
    !!
  End Subroutine write_u_dmat_to_file
  !!***********************************************************************
  Subroutine write_fplo1_density(mspin,nsite,nsort,isrt,elements,mlmp,nlmp,lmp&
       &,pmeshs,nrp,efermi,Bfsm,lcd,degree)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: mspin,nsite,nsort,mlmp,isrt(*),degree,nrp
    Character(2),Intent(in)      :: elements(:)
    Integer(t_int),Intent(in)    :: nlmp(nsite),lmp(mlmp,nsite)
    Real(t_real),Intent(in)      :: efermi,Bfsm
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    !!LOCAL
    Integer(t_int) :: ms,isa,issa,i,ir
    Real(t_real)   :: ff,ri,rmax
    !Real(t_real),Parameter :: zero=0.0_t_real
    Type(T_Mesh),Pointer :: pmesh
    Type(T_Mesh)      :: ldmesh
    !!
    Return
    !!
    !! old FPLO1 dens is allways log-mesh 
    !!
    rmax=0.0_t_real
    Do issa=1,nsort
       rmax=Max(rmax,pmeshs(issa)% rmax)
    Enddo
    !!
    pmesh => pmeshs(1)
    !!
    Call init_mesh(ldmesh)
    ldmesh% Type=LOG_MESH
    ldmesh% rmin=pmesh% rmin
    ldmesh% rmax=rmax
    ldmesh% nr=nrp
    ldmesh% with_zero=.False.
    Call setup_mesh(ldmesh)
    
    Open(10,file='dens',status='unknown')
    
    Write(10,100)':COMPOUND: XYZ  Version 1.62 M-CPA'
    Write(10,101)mspin,0,0,nsite,mlmp,ldmesh% nr,ldmesh% hx&
         &,ldmesh% rmin,efermi-Bfsm,efermi+Bfsm
    
    Do ms=1_t_int,mspin
       Write(10,102)ms,'MSPIN'
       Write(10,105)'LOCAL-SITE DENSITY'
       Do isa=1_t_int,nsite
          issa=isrt(isa)
          !!
          pmesh => pmeshs(issa)
          !!
          Write(10,106)elements(issa),isa
          Write(10,200)nlmp(isa),'NLM'
          Do i=1_t_int,nlmp(isa)
             ff=1.0_t_real
             If(i==1_t_int)Then
                ff=1.0_t_real/sq4pi
             Endif
             Write(10,201)i,lmp(i,isa)
             Do ir=1,ldmesh% nr
                ri=ldmesh% p_rmesh(ir)
                Write(10,202)minterpol(pmesh,degree,lcd(:,i,isa,ms),ri)*ff
             Enddo
          Enddo
       Enddo ! is
    Enddo ! ms
    Close(10)
    !!
    Call cleanup_mesh(ldmesh)
    !!
    Call terminate(TERM_NORMAL,' denisty converted')
    Return
    !!
100 Format(a,3x,a)
101 Format(6i5,4(1pd16.8,3x))
102 Format(i5,3x,a)
105 Format(a)
106 Format(a,i5)
200 Format(i5,3x,a)
201 Format(2i5)
202 Format(5(1pd16.8))
    !! 202  format(5(1pd30.17))
  End Subroutine write_fplo1_density
  !!***********************************************************************
  !! make initial spinsplit 
  !!
  Subroutine initsplit(lcd,pmeshs,nrp,mlmp,nsite,isrt,isrtTOIOisrt&
       &,natom,mspin&
       &,initial_spin_split,spin_split,lsdau,symmetry,equivsites&
       &,elements,rel) 
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)      :: lsdau
    Type(T_Symmetry),Intent(in)      :: symmetry
    Type(T_Mesh),Pointer             :: pmeshs(:)
    Character(2),Intent(in)          :: elements(:)
    Logical(t_log),Intent(in)        :: rel
    Integer(t_int),Intent(in)        :: equivsites(48,*)
    Logical(t_log),Intent(inout)     :: initial_spin_split
    Integer(t_int),Intent(in) ::  mlmp,nsite,mspin,isrt(nsite),nrp
    Integer(t_int),Intent(in)    :: isrtTOIOisrt(*)
    Real(t_real),Intent(in)  ::  spin_split(*),natom(*)
    Real(t_real),Intent(inout) ::  lcd(nrp,mlmp,nsite,*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="initsplit"
    Integer(t_int) :: isa,j,ir,issa,isau,deg,i,issaIO,nspin
    Real(t_real)  :: d,a,al,r,f,Z
    Complex(t_real):: cd,ca
    Type(T_Mesh),Pointer :: pmesh
    Complex(t_real),Pointer :: ldmat(:,:,:,:)
    !!***********************************************************************
    !!
    Nullify(ldmat)
    !!
    nspin=mspin
    if(rel)nspin=1
    !!
    !print *,'initial_spin_split TODO: to be tested'
    If(mspin == 2_t_int .And. initial_spin_split)Then
       Write(outfile,44)
       Do isa=1_t_int,nsite
          issa=isrt(isa)
          !!
          pmesh => pmeshs(issa)
          !!
          issaIO=isrtTOIOisrt(issa)
          Z=natom(issa)
          If(Abs(Z) > 1.e-3_t_real)Then
             al=12.0_t_real/2.0_t_real        
             a=(spin_split(issaIO))*0.5_t_real
             Do j=1_t_int,1_t_int     
                Do ir=1_t_int,pmesh% nr
                   r=pmesh% p_rmesh(ir)
                   d=lcd(ir,j,isa,1_t_int)+lcd(ir,j,isa,2_t_int)
                   f=r**4_t_int*Exp(-r*al)*al**5_t_int/24.0_t_real/sq4pi
                   lcd(ir,j,isa,1_t_int)=d*0.5_t_real+a*f
                   lcd(ir,j,isa,2_t_int)=d*0.5_t_real-a*f      
                Enddo             ! ir
             Enddo               ! j
          Endif
       Enddo                   ! isa
       !!START LSDA+U
       If(lsdau% lsdau)Then
          Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
               &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
          
          ldmat(:,:,:,:)=lsdau% p_dmat(:,:,:,:)

          Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,'tospin')
          
          
          Do isau=1_t_int,lsdau% nusite
             issaIO=lsdau% p_isrtu(lsdau% p_udefs(isau))
             deg=2*lsdau% p_lvsu(isau)+1
             !! calc trace
             If(rel)Then
                cd=   trace(lsdau% p_dmat(:,:,isau,1),0_t_int,0_t_int,2*deg)
             Else
                cd=   trace(lsdau% p_dmat(:,:,isau,1),0_t_int,0_t_int,deg)+&
                     &trace(lsdau% p_dmat(:,:,isau,2),0_t_int,0_t_int,deg)
             Endif
             cd=cd/deg
             a=(spin_split(issaIO))*0.5_t_real/deg
             ca=Cmplx(a,0.0_t_real,t_real)
             Do i=1,deg
                If(rel)Then
                   ldmat(i,i,isau,1)=cd*0.5_t_real+ca
                   ldmat(deg+i,deg+i,isau,1)=cd*0.5_t_real-ca
                Else
                   ldmat(i,i,isau,1_t_int)=cd*0.5_t_real+ca
                   ldmat(i,i,isau,2_t_int)=cd*0.5_t_real-ca
                Endif
             Enddo
          Enddo !isau
          Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,'fromspin')
          Call force_u_mat_sym(symmetry,equivsites,lsdau,rel,mspin,ldmat&
               &,lsdau% p_dmat)
          Call force_u_hermit(lsdau,nspin,lsdau% p_dmat)
          !!
          Call pfree(ldmat)
          Call print_u_dmat(mspin,elements,lsdau,rel,lsdau% p_dmat,'dmat')
       End If
       !!END LSDA+U
       
       initial_spin_split=.False.
    Endif
    !!
    !!
    Return
    !!
44  Format('-----------------------------------------------------------------&
         &-------'/&
         &,'Will make initial spin splitting !',/&
         &,'-----------------------------------------------------------------&
         &-------'/)
    !!     
  End Subroutine initsplit
  !!***********************************************************************
  Subroutine densin(densfile,symmetry,nsite,isrt,elements,equivsites&
       &,efermi,Bfsm&
       &,pmeshs,nrp,mlmp,nlmp,lmp,lcd,mspin,opc,lsdau,rel,ninslot,md)
    Implicit None
    !!IO
    Type(T_Mesh),Pointer        :: pmeshs(:)
    Type(T_Lsdau),Intent(inout) :: lsdau
    Type(T_OPC),Intent(inout)   :: opc
    Type(T_Symmetry),Intent(in) :: symmetry
    Type(T_MapData),Intent(in),Optional :: md(:)
    Logical(t_log),Intent(in)   :: rel
    Character(*),Intent(in)     :: densfile
    Integer(t_int),Intent(in),Optional :: ninslot
    Integer(t_int),Intent(in)   :: nsite,isrt(nsite)
    Integer(t_int),Intent(in)   :: mlmp,mspin,nlmp(nsite)
    Integer(t_int),Intent(in)   :: lmp(mlmp,nsite),nrp
    Character(2),Intent(in)     :: elements(:)
    Integer(t_int),Intent(in)   :: equivsites(48,*)
    Real(t_real),Intent(out)    :: efermi,Bfsm
    Real(t_real),Intent(inout)  :: lcd(:,:,:,:)
    !!LOCAL
    Logical(t_log) :: mapping
    !Type(T_Mesh),Pointer :: pmesh
    Type(T_Mesh) :: ldmesh !! for reading old files
    Type(T_Mesh),Pointer :: ldmeshs(:) !! for new files
    Character(*),Parameter::nsubfunc="densin"
    Integer(t_int) :: dnsite,dmspin,dmlmp,lsdau_maxdim,lsdau_nsite,isau
    Integer(t_int) :: dnrp,n,deg,nspin
    Real(t_real)   :: ef(2)
    !Integer(t_int) :: ir,ilm
    !Real(t_real)   :: texp,ri
    Integer(t_int) :: opc_nodef,iodef,issao,jodef
    Integer(t_int) :: isa,isbu,i1,i2,ms!,ver
    Complex(t_real):: cd(2)
    Logical(t_log) :: gotit,uniformmesh
    Character(2)   :: label,state
    Character(2),Pointer   :: delements(:)
    Integer(t_int),Pointer :: dnlmp(:)
    Integer(t_int),Pointer :: dlmp(:,:)
    Integer(t_int),Pointer :: lsdau_siteu(:)
    Integer(t_int),Pointer :: lsdau_udef(:)
    Character(2),Pointer   :: lsdau_el(:)
    Character(2),Pointer   :: lsdau_label(:)
    Real(t_real),Pointer :: dens(:,:,:,:)
    Real(t_real),Pointer :: lsdau_dmat(:,:,:,:,:)
    Real(t_real),Pointer :: lsdau_dmatoff(:,:,:,:)
    Complex(t_real),Pointer :: ldmat(:,:,:,:)
    !!OPC
    Integer(t_int),Pointer :: opc_isrto(:)
    Character(2),Pointer   :: opc_el(:)
    Character(2),Pointer   :: opc_state(:)
    Real(t_real),Pointer :: opc_Bracah(:,:)
    Real(t_real),Pointer :: opc_dBracah(:,:)
    Real(t_real),Pointer :: opc_Morb(:,:)
    Real(t_real),Pointer :: opc_occu(:,:)
    Integer(t_int),Pointer :: itmp(:)
    Real(t_real),Pointer :: dtmp(:)
    Logical(t_log),Pointer :: ltmp(:)
    
    Character(200)  ::  mainversion,subversion
    Logical(t_log) :: exist ,have_ldmat,have_ldmatoff,have_lmp,have_opcparams
    Integer(t_int),Parameter::degree=8_t_int
    !!-------------------------------------------------------------------------
    !!
    Nullify(delements,dnlmp,dlmp,dens,ldmeshs,itmp,dtmp,ltmp,lsdau_dmatoff)
    Nullify(lsdau_siteu,lsdau_udef,lsdau_el,lsdau_label,lsdau_dmat,ldmat)
    Nullify(opc_isrto,opc_el,opc_state,opc_Bracah,opc_dBracah,opc_Morb,opc_occu)
    !! ldmeshs is saved for every site in the file in order
    !! to transfer files between structures with less restrictions
    !!
    nspin=mspin
    if(rel) nspin=1
    !!
    mapping=Present(ninslot).And.Present(md)
    !!
    Call init_mesh(ldmesh)
    !!
    Inquire(file=densfile,exist=exist)
    If(.Not.exist) Then 
       Write(errbuf,"('''File ',a,' does not exist!''')")Trim(densfile)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    End If
    
1   Format('========================================================================',/,'LOADING DENSITY from file ',a,/)
    Write(outfile,1) Trim(densfile)    
    
    Call watch(nsubfunc,cpu_start,reset=.True.)     
    
    
    Call initparser
    Call setparsestop("data_header")
    Call readfile(densfile)
    Call getstring("version.mainversion",mainversion)
    Call getstring("version.subversion",subversion)
    
    !ver=checkversion(mainversion,subversion,densfile)
    
    Call getinteger("nsite",dnsite)
    
    If(nsite/=dnsite .And. .Not. mapping)Then
       Write(errbuf,"('''File ',a,' seems to belong to a different &
            &structure!'',/,'' error: Number of sites different!''')")&
            &Trim(densfile)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    
    Call getinteger("mspin",dmspin)
    Call getinteger("mlmp",dmlmp)
    
    uniformmesh=variable_exists('rmin')
    
    Call getinteger("nr",dnrp)
    
    If(uniformmesh)Then
       ldmesh % nr=dnrp
       Call getreal("rmin",ldmesh % rmin)
       Call getreal("rmax",ldmesh % rmax)
       !!
       If( variable_exists('mesh_alpha'))Then
          Call getreal("mesh_alpha",ldmesh % alpha)
       Else
          ldmesh% alpha=3.0_t_int
       Endif
       If(variable_exists('mesh_type.type').And.&
            &variable_exists('mesh_with_zero'))Then
          Call getinteger("mesh_type.type",ldmesh% Type)
          Call getlogical("mesh_with_zero",ldmesh% with_zero)
       Else
          ldmesh% Type=LOG_MESH
          ldmesh% with_zero=.False.
       Endif
       Call setup_mesh(ldmesh)
    Else
       If(.Not.variable_exists('pmeshs'))Then
          Write(errbuf,"('''File ',a,' seems to be corrupted, &
               &missing old AND new style mesh information!''')")Trim(densfile)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       !!
       Call palloc('itmp',itmp,1_t_int,dnsite,nmodule,nsubfunc)
       Call palloc('dtmp',dtmp,1_t_int,dnsite,nmodule,nsubfunc)
       Call palloc('ltmp',ltmp,1_t_int,dnsite,nmodule,nsubfunc)
       Call getinteger("pmeshs.nr",itmp)
       !!
       Call palloc_meshs('ldmeshs',ldmeshs,dnsite,nmodule,nsubfunc)

       n=0
       Do isa=1,dnsite
          ldmeshs(isa)% nr=itmp(isa)
          n=Max(n,ldmeshs(isa)% nr)
       Enddo
       If(dnrp<n)Then
          Write(errbuf,"('''File ',a,' seems to be corrupted, &
               & number of mesh points inconsisten!''')")Trim(densfile)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       Call getreal("pmeshs.rmin",dtmp)
       Do isa=1,dnsite
          ldmeshs(isa)% rmin=dtmp(isa)
       Enddo
       Call getreal("pmeshs.rmax",dtmp)
       Do isa=1,dnsite
          ldmeshs(isa)% rmax=dtmp(isa)
       Enddo
       Call getreal("pmeshs.alpha",dtmp)
       Do isa=1,dnsite
          ldmeshs(isa)% alpha=dtmp(isa)
       Enddo
       Call getlogical("pmeshs.with_zero",ltmp)
       Do isa=1,dnsite
          ldmeshs(isa)% with_zero=ltmp(isa)
       Enddo
       Call getinteger("pmeshs.mesh_type.type",itmp)
       Do isa=1,dnsite
          ldmeshs(isa)% Type=itmp(isa)
       Enddo
       
       Do isa=1,dnsite
          Call setup_mesh(ldmeshs(isa))
       Enddo
    Endif
    
    Call getreal("efermi",ef)
    efermi=(ef(1)+ef(2))*0.5_t_real
    Bfsm=(ef(2)-ef(1))*0.5_t_real
    
    
    
    Call palloc('delements',delements,1_t_int,dnsite,nmodule,nsubfunc)
    Call palloc('dnlmp',dnlmp,1_t_int,dnsite,nmodule,nsubfunc)
    Call palloc('dlmp',dlmp,1_t_int,dmlmp,1_t_int,dnsite,nmodule,nsubfunc)
    
    Call getstring("elements",delements)
    
    
    Do isa=1_t_int,nsite
       If(Trim(elements(isrt(isa))) /= Trim(delements(isa)).And. .Not.mapping)Then
          Write(errbuf,"('''File ',a,' seems to belong to a different &
               &structure!'',/,'' error:  different elements!''')")Trim(densfile)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
    Enddo ! isa
    
    
    
    Call getinteger("nlmp",dnlmp)
    have_lmp=variable_exists('lmp')
    If(have_lmp)Then
       Call getinteger("lmp",dlmp)
    Endif
    
    
    
!!$    Do isa=1,dnsite
!!$       Do j=1,dnlmp(isa)
!!$          Print *,isa,j,dlmp(j,isa)
!!$       Enddo
!!$    Enddo
    
    !!START LSDA+U
    lsdau_nsite=0_t_int
    have_ldmat=.False.
    have_ldmatoff=.False.
    Call palloc('lsdau_dmatoff',lsdau_dmatoff,1_t_int,1_t_int,1_t_int,1_t_int&
         &,1_t_int,1_t_int,1_t_int,1_t_int,nmodule,nsubfunc)

    If(lsdau% lsdau.And. mapping)Then
       Write(errbuf,"('''Density mapping for LSDA+U not implemented!''')")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       
    Endif
    If(lsdau% lsdau)Then
       
       If( variable_exists('lsdau_nsite'))Then
          !! if this exist, we have at least the header data
          Call getinteger("lsdau_nsite",lsdau_nsite)
          Call getinteger("lsdau_maxdim",lsdau_maxdim)
          !! if we have the LSDA+U header 
          If(lsdau_nsite > 0_t_int)Then
             have_ldmat=.True.
             Call palloc('lsdau_dmat',lsdau_dmat,1_t_int,2_t_int&
                  &,1_t_int,lsdau_maxdim,1_t_int,lsdau_maxdim&
                  &,1_t_int,lsdau_nsite,1_t_int,dmspin,nmodule,nsubfunc)
             
             If(rel)Then
                Call palloc('lsdau_dmatoff',lsdau_dmatoff,1_t_int,2_t_int&
                     &,1_t_int,lsdau_maxdim,1_t_int,lsdau_maxdim&
                     &,1_t_int,lsdau_nsite,nmodule,nsubfunc)
             Endif
             Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim&
                  &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite&
                  &,1_t_int,nspin,nmodule,nsubfunc)
             
             lsdau_dmat(:,:,:,:,:)=0.0_t_real
             If(rel) lsdau_dmatoff(:,:,:,:)=0.0_t_real
             ldmat(:,:,:,:)=c0
             
             Call palloc('lsdau_siteu',lsdau_siteu,1_t_int,lsdau_nsite&
                  &,nmodule,nsubfunc)
             Call palloc('lsdau_udef',lsdau_udef,1_t_int,lsdau_nsite&
                  &,nmodule,nsubfunc)
             Call palloc('lsdau_el',lsdau_el,1_t_int,lsdau_nsite&
                  &,nmodule,nsubfunc)
             Call palloc('lsdau_label',lsdau_label,1_t_int,lsdau_nsite&
                  &,nmodule,nsubfunc)
             
             !! read the other data
             Call getinteger('lsdau.site',lsdau_siteu)
             Call getstring('lsdau.element',lsdau_el)
             Call getinteger('lsdau.udef',lsdau_udef)
             Call getstring('lsdau.state',lsdau_label)
             
          Else
             
             Write(outfile,2)'No LSDA+U occupation matrix found!&
                  &, Will start with default!'
          Endif
          
       Else
          Write(outfile,2)'No LSDA+U data found!, Will start with default!'
       Endif
    Endif
    !!END LSDA+U
2   Format(a)
    
    
    
    
    !!START OPC
    opc_nodef=0_t_int
    have_opcparams=.False.
    If(opc% make_opc.And. mapping)Then
       Write(errbuf,"('''Density mapping for OPC not implemented !''')")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       
    Endif
    If(opc% make_opc)Then
       
       If( variable_exists('opc_nodef'))Then
          !! if this exist, we have at least the header data
          Call getinteger("opc_nodef",opc_nodef)
          !! if we have the OPC header 
          If(opc_nodef > 0_t_int)Then
             have_opcparams=.True.
             Call palloc('opc_Bracah',opc_Bracah,1_t_int,2_t_int&
                  &,1_t_int,opc_nodef,nmodule,nsubfunc)
             Call palloc('opc_dBracah',opc_dBracah,1_t_int,2_t_int&
                  &,1_t_int,opc_nodef,nmodule,nsubfunc)
             Call palloc('opc_Morb',opc_Morb,1_t_int,2_t_int&
                  &,1_t_int,opc_nodef,nmodule,nsubfunc)
             Call palloc('opc_occu',opc_occu,1_t_int,2_t_int&
                  &,1_t_int,opc_nodef,nmodule,nsubfunc)
             
             
             opc_Bracah(:,:)=0.0_t_real
             opc_dBracah(:,:)=0.0_t_real
             opc_Morb(:,:)=0.0_t_real
             opc_occu(:,:)=0.0_t_real
             
             Call palloc('opc_isrto',opc_isrto,1_t_int,opc_nodef,nmodule,nsubfunc)
             Call palloc('opc_el',opc_el,1_t_int,opc_nodef,nmodule,nsubfunc)
             Call palloc('opc_state',opc_state,1_t_int,opc_nodef,nmodule,nsubfunc)
             
             
             !! read the other data
             Call getinteger('opc.sort',opc_isrto)
             Call getstring('opc.element',opc_el)
             Call getstring('opc.state',opc_state)
             
          Else
             
             Write(outfile,2)'No OPC matrices found!&
                  &, Will start with default!'
          Endif
          
       Else
          Write(outfile,2)'No OPC data found!, Will start with default!'
       Endif
    Endif
    !!END OPC
    
    
    Call cleanreadfile
    
    
    
    Call palloc('dens',dens,1_t_int,dnrp,1_t_int,dmlmp,1_t_int,dnsite&
         &,1_t_int,dmspin,nmodule,nsubfunc)
    
    
    
    !! reinitialize parser
    Call initparser  
    dens=0.0_t_real
    
    !! set direct input for array dens
    Call addexternalrealarray('dens',dens)


    
    !!START LSDA+U
    !! if we have the header and are in LSDA+U
    If(have_ldmat)Then
       Call addexternalrealarray('lsdau_dmat',lsdau_dmat)
    Endif

    If(rel.And.have_ldmat)Then
       Call addexternalrealarray('lsdau_dmat_offdiagonal',lsdau_dmatoff)
    Endif
    !!END LSDA+U
    
    !!START OPC
    !! if we have the header and are in OPC
    If(have_opcparams)Then
       Call addexternalrealarray('opc_Bracah',opc_Bracah)
       Call addexternalrealarray('opc_dBracah',opc_dBracah)
       Call addexternalrealarray('opc_Morb',opc_Morb)
       Call addexternalrealarray('opc_occu',opc_occu)
    Endif
    !!END OPC
    
    !! read the file
    Call readfile(densfile)

    If(rel.And.variable_exists('lsdau_dmat_offdiagonal'))Then
       have_ldmatoff=.True.
    Endif

    Call cleanreadfile
    
    
    
    !!START LSDA+U
    !! if we have LSDA+U data
    If(have_ldmat)Then
       
       
       ldmat(:,:,:,:)=c0
       isauloop: Do isau=1,lsdau% nusite
          isa=lsdau% p_isiteu(isau)
          label=lsdau% p_labelsu(isau)
          deg=2*lsdau% p_lvsu(isau)+1
          gotit=.False.
          isbloop: Do isbu=1,lsdau_nsite
             If(isa == lsdau_siteu(isbu).And.&
                  &label == lsdau_label(isbu))Then
                gotit=.True.
                Do i1=1_t_int,deg
                   Do i2=1_t_int,deg
                      cd(1:2)=c0
                      If(dmspin == mspin ) Then
                         Do ms=1,mspin
                            
                            cd(ms)=Cmplx(lsdau_dmat(1_t_int,i1,i2,isbu,ms)&
                                 &,lsdau_dmat(2_t_int,i1,i2,isbu,ms),t_real)
                         Enddo
                      Elseif(dmspin == 2_t_int .And. mspin == 1_t_int) Then
                         cd(1)=Cmplx(&
                              &  lsdau_dmat(1_t_int,i1,i2,isbu,1)&
                              & +lsdau_dmat(1_t_int,i1,i2,isbu,2)&
                              &, lsdau_dmat(2_t_int,i1,i2,isbu,1)&
                              & +lsdau_dmat(2_t_int,i1,i2,isbu,2)&
                              &,t_real)*0.5_t_real
                         cd(2)=cd(1)
                      Elseif(dmspin == 1_t_int .And. mspin == 2_t_int) Then
                         
                         Do ms=1,mspin
                            cd(ms)=Cmplx(&
                                 & lsdau_dmat(1_t_int,i1,i2,isbu,1)&
                                 &,lsdau_dmat(2_t_int,i1,i2,isbu,1)&
                                 &,t_real)
                         End Do
                      Endif
                      If(rel)Then
                         If(dmspin==1)Then
                            !! enforce time reversal symmetry
                            ms=1
                            ldmat(i1+deg*(ms-1),i2+deg*(ms-1),isau,1)=cd(1)
                            ms=2
                            ldmat(i1+deg*(ms-1),i2+deg*(ms-1),isau,1)=Conjg(cd(1))
                         Else
                            Do ms=1,2
                               ldmat(i1+deg*(ms-1),i2+deg*(ms-1),isau,1)=cd(ms)
                            Enddo
                         Endif
                      Else
                         Do ms=1,mspin
                            ldmat(i1,i2,isau,ms)=cd(ms)
                         Enddo
                      Endif
                      !!
                      If(have_ldmatoff)Then
                         cd(1)=Cmplx(lsdau_dmatoff(1,i1,i2,isbu),&
                              &      lsdau_dmatoff(2,i1,i2,isbu),t_real)
                         ldmat(i1    ,i2+deg,isau,1)=cd(1)
                         ldmat(i2+deg,i1    ,isau,1)=Conjg(cd(1))
                      Endif
                      !!
                   End Do
                End Do
                Exit isbloop
             Endif
          End Do isbloop
          If(.Not.gotit)Then
3            Format('Could not find LSDA+U matrix for site ',i5&
                  &,' state ',a2,' ! Take default!')
             Write(outfile,3)isa,label
          Endif
       End Do isauloop
       !!
       !!
       Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,'fromspin')
       Call force_u_mat_sym(symmetry,equivsites,lsdau,rel,mspin,ldmat&
            &,lsdau% p_dmat)
       !!
       Call force_u_hermit(lsdau,nspin,lsdau% p_dmat)
       
       
    Endif
    !!END LSDA+U
    
    
    
    
    
    !!START OPC
    !! if we have OPC data
    If(have_opcparams)Then
       
       
       !opc% p_Bracah(1:2,1:opc% nodef)=0.0_t_real
       !opc% p_Morb  (1:2,1:opc% nodef)=0.0_t_real
       !opc% p_occu  (1:2,1:opc% nodef)=0.0_t_real
       
       iodefloop: Do iodef=1,opc% nodef
          issao=opc% p_isrto(iodef)
          state=opc% p_state(iodef)
          
          gotit=.False.
          jodefloop: Do jodef=1,opc_nodef
             If(issao == opc_isrto(jodef) .And.&
                  & state == opc_state(jodef) .And.&
                  & elements(issao)==opc_el(jodef) )Then
                gotit=.True.
                Do ms=1,2
                   opc% p_Bracah(ms,iodef,1)=opc_Bracah(ms,jodef)
                   opc% p_Bracah(ms,iodef,2)=opc_dBracah(ms,jodef)
                   opc% p_Morb  (ms,iodef)=opc_Morb  (ms,jodef)
                   opc% p_occu  (ms,iodef)=opc_occu  (ms,jodef)
                Enddo
                Exit jodefloop
             Endif
          End Do jodefloop
          If(.Not.gotit)Then
113          Format('Could not find OPC data for sort ',i5&
                  &,' el ',a2,' state ',a2,' ! Take default!')
             Write(outfile,113)issao,elements(issao),state
          Endif
       End Do iodefloop
       !!
    Endif
    !!END OPC
    
    
    
    !!
    If(uniformmesh)Then
       Write(outfile,20)
20     Format('Density reinterpolation:',/)
       Call densreint(degree,elements,delements&
            &, mspin,nsite,isrt,mlmp,nlmp,lmp,pmeshs,nrp,lcd&
            &,dmspin,dnsite,dmlmp,dnlmp,dlmp,ldmesh,ldmeshs,dnrp,dens,have_lmp&
            &,uniformmesh,mapping,ninslot,md)
    Else
       Call densreint(degree,elements,delements&
            &, mspin,nsite,isrt,mlmp,nlmp,lmp,pmeshs,nrp,lcd&
            &,dmspin,dnsite,dmlmp,dnlmp,dlmp,ldmesh,ldmeshs,dnrp,dens,have_lmp&
            &,uniformmesh,mapping,ninslot,md)
    Endif
    
    
    
    
    
    Call pfree(dens)
    Call cleanup_mesh(ldmesh)
    Call pfree_meshs(ldmeshs)
    
    !!LSDA+U
    Call pfree(lsdau_siteu)
    Call pfree(lsdau_udef)
    Call pfree(lsdau_el)
    Call pfree(lsdau_label)
    Call pfree(lsdau_dmat)
    Call pfree(lsdau_dmatoff)
    Call pfree(ldmat)
    !!OPC
    Call pfree(opc_isrto)
    Call pfree(opc_el)
    Call pfree(opc_state)
    Call pfree(opc_Bracah)
    Call pfree(opc_dBracah)
    Call pfree(opc_Morb)
    Call pfree(opc_occu)
    !! dens
    Call pfree(delements)
    Call pfree(dnlmp)
    Call pfree(dens)
    Call pfree(dlmp)
    Call pfree(itmp)
    Call pfree(dtmp)
    Call pfree(ltmp)
    
    Call watch(nsubfunc,cpu_stop,delete=.True.,Print=.True.)    
    
11  Format(/,'END DENSITY LOADING',/,'========================================================================')
    Write(outfile,11)
    
    
    
    !!
    If(opc% make_opc)Then
       Write(outfile,1000) 
       Write(outfile,1001) nsubfunc
       Write(outfile,1000) 
       Write(outfile,1010) 
       Call print_opc_params(elements,opc)
       Write(outfile,1000) 
    Endif
1000 Format('OPC: ',72('-'))
1001 Format('OPC: ',a)
1010 Format('OPC: Racah parameters B from density file:')
    !!
    !Stop
    !!
    Return
    !!
  End Subroutine densin
  !!***********************************************************************
  Subroutine denswrite(compound,structure,mspin,pmeshs,nrp,efermi,Bfsm&
       &,mlmp,nlmp,lmp,lcd,opc,lsdau,relativistic,write_log&
       !&,bzone&
       &,arguniformmesh)
    !!
    Implicit None 
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lsdau),Intent(in) :: lsdau
    !Type(T_Bzone),Intent(in) :: bzone
    Type(T_OPC),Intent(in)   :: opc
    Type(T_Mesh),Pointer     :: pmeshs(:)
    Logical(t_log),Intent(in):: write_log
    Logical(t_log),Intent(in),Optional  :: arguniformmesh
    Character(*),Intent(in)  :: compound
    Integer(t_int),Intent(in):: mspin,nrp,relativistic
    Real(t_real),Intent(in)  :: efermi,Bfsm
    !! density
    Integer(t_int),Intent(in):: mlmp,nlmp(*),lmp(mlmp,*)
    Real(t_real),Intent(in)  :: lcd(nrp,mlmp,structure% nsite,mspin) 
    !!LOCAL
    Character(*),Parameter::nsubfunc="denswrite"
    Type(T_Mesh),Pointer  :: pmesh
    Type(T_Mesh)   :: ldmesh
    Logical(t_log) :: uniformmesh,rel
    Character(200) :: buf
    Integer(t_int) :: isa,issa,ilm,ms,ir,i,isau,i1,i2,j,iudef,degree
    Integer(t_int) :: iodef,issao,msp,l,deg,ii0,mms
    Real(t_real)   :: rmax
    Complex(t_real):: cc
    Complex(t_real),Pointer::  ldmat(:,:,:,:)
    Complex(t_real),Pointer::  pdmat(:,:,:,:)
    !!
    Nullify(ldmat)
    !!
    !!
    If(Present(arguniformmesh))Then
       uniformmesh=arguniformmesh
    Else
       uniformmesh=.False.
    Endif
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    degree=8_t_int
    !!
    Call init_mesh(ldmesh)
    !!
    If(uniformmesh)Then
       !!
       !!
       rmax=0.0_t_real
       Do issa=1,structure% nsort
          rmax=Max(rmax,pmeshs(issa)% rmax)
       Enddo
       !!
       !! this is not thebest solution, since it does not
       !! store all informaion, however as a quick fix
       !!
       !!
       pmesh => pmeshs(1)
       ldmesh% Type=pmesh% Type
       ldmesh% rmin=pmesh% rmin
       ldmesh% rmax=rmax
       ldmesh% alpha=pmesh% alpha
       ldmesh% nr  =nrp
       ldmesh% with_zero=pmesh% with_zero
       If(write_log)Then
          ldmesh% Type=LOG_MESH
          ldmesh% with_zero=.False.
       Endif
       Call setup_mesh(ldmesh)
    Endif
    !!
    
1   Format(/,'------------------------------------------------------------------------',/,'Writing ',a,' ...!')
    Write(outfile,1)Trim(f_dens)
    
    !!
    Open(12,file=f_dens)
    
2   Format(a)
    Write(12,2)&
         &"########################################################################"&
         &,"# this file is part of the FPLO bandstructure package"&
         &,"# it was created automatically"&
         &,"# please dont delete this file nor"&
         &,"# edit it directly, unless you really know what you are doing"&
         &,"# Use FPLO to change its content"&
         &,"#"&
         &,"########################################################################"
    
3   Format(/,/)
    Write(12,3)
    
    
    !! SECTION header
    Write(12,2)  'section header{'
    Call write_version(12_t_int)
    Call write_changed(12_t_int)
    
4   Format('  char[*] compound="',a,'";')
    Write(12,4) Trim(compound)
5   Format('};',2/)
    Write(12,5) 
    Write(12,2)  'section data_header{'
6   Format(2x,'int  mspin=',i10,';')
    Write(12,6)mspin
7   Format(2x,'int  nsite=',i10,';')
    Write(12,7)structure% nsite
8   Format(2x,'int  mlmp =',i10,';')
    Write(12,8)mlmp
9   Format(2x,'int  nr =',i10,';')
    If(uniformmesh)Then
       Write(12,9)ldmesh% nr
10     Format(2x,'real rmin =',e30.15e3,';')
       Write(12,10) ldmesh% rmin
11     Format(2x,'real rmax =',e30.15e3,';')
       Write(12,11) ldmesh% rmax
411    Format(2x,'real mesh_alpha =',e30.15e3,';')
       Write(12,411) ldmesh% alpha
211    Format('  struct {int type;char[*] description;} mesh_type'&
            &,'  ={',i3,',"',a,'"};')
       Write(12,211) ldmesh% Type,Trim(MESH_TYPE(ldmesh% Type))
212    Format(2x,'logical mesh_with_zero =',l1,';')
       Write(12,212) ldmesh% with_zero
    Else
       Write(12,9)nrp
    Endif
    
12  Format(2x,'real efermi[2]={',e30.15e3,',',e30.15e3,'};')
    Write(12,12) efermi-Bfsm,efermi+Bfsm
    
    
    Write(12,2)'  char[2] elements[nsite]={'
    Write(12,2,advance='no')'    '
    Do isa=1_t_int,structure% nsite
       issa=structure% p_isrt(isa)
13     Format('"',a,'"')
       Write(12,13,advance='no')Trim(structure% p_elements(issa))
14     Format(/,'  ')
       If(Mod(isa,15_t_int) == 0_t_int) Write(12,14,advance='no')
33     Format(',')
       If(isa < structure% nsite) Write(12,33,advance='no')
    End Do
    Write(12,2)'  };'
    
15  Format('  int nlmp[nsite]={',/,'  ')
    Write(12,15,advance='no')
    Do isa=1_t_int,structure% nsite
16     Format(i5)
       Write(12,16,advance='no')nlmp(isa)
17     Format(/,'   ')
       If(Mod(isa,15_t_int) == 0_t_int) Write(12,17,advance='no')
       If(isa < structure% nsite) Write(12,33,advance='no')
    End Do
    Write(12,2)'  };'
    
    !!START: store in array, which L-componenets are in dens[]
    Write(12,200)
    Do isa=1_t_int,structure% nsite
       Write(12,2,advance='no')'    {'
       Do j=1,nlmp(isa)
          Write(buf,16)lmp(j,isa)
          Write(12,2,advance='no') Trim(Adjustl(buf))
          If(Mod(j,15_t_int) == 0_t_int) Write(12,217,advance='no')
          If(j < mlmp) Write(12,33,advance='no')
       Enddo
       Do j=nlmp(isa)+1,mlmp
          Write(12,2,advance='no') '0'
          If(Mod(j,15_t_int) == 0_t_int) Write(12,217,advance='no')
          If(j < mlmp) Write(12,33,advance='no')
       Enddo
       Write(12,2,advance='no')'    }'
       If(Mod(isa,15_t_int) == 0_t_int) Write(12,217,advance='no')
       If(isa < structure% nsite) Write(12,33)
       
    Enddo
    Write(12,2)'  };'
200 Format('  int lmp[mlmp,nsite]={')
217 Format(/,'   ')
    !!END  : store in array, which L-componenets are in dens[]
    
    !! new: we store all mesh details -> no longer downwards compatible
    Write(12,"('  struct {int nr;real rmin;real rmax;real alpha;&
         &logical with_zero;',/&
         &,'          struct{int type;char[*] description;}mesh_type;'&
         &,'}pmeshs[nsite]={')")
    Do isa=1,structure% nsite
       pmesh => pmeshs(structure% p_isrt(isa))
       
       Write(12,2,advance='no')'    {'
       Write(12,"(i5,2(',',e30.15e3),/,t11,1(',',e30.15e3),',',l1,',{',i3,',""',a,'""}')",advance='no')&
            &pmesh% nr,pmesh% rmin,pmesh% rmax,pmesh% alpha,pmesh% with_zero&
            &,pmesh% Type,Trim(MESH_TYPE(pmesh% Type))
       Write(12,2,advance='no')'}'
       If(isa < structure% nsite) Write(12,33)
    Enddo
    Write(12,2)  '  };'
    
    
    
    
    !!START LSDA+U

18  Format(2/,'# LSDA+U definitions',2/)
    Write(12,18)
19  Format(2x,'int  lsdau_nsite=',i10,';')
    Write(12,19)lsdau% nusite    
20  Format(2x,'int  lsdau_maxdim=',i10,';')
    Write(12,20)lsdau% umaxdim    
    
21  Format('  struct {int site;char[2] element; int udef;char[2] state;}&
         &lsdau[lsdau_nsite]={',/,'  ')
    Write(12,21,advance='no')
    
    
    Do isau=1_t_int,lsdau% nusite
       iudef=lsdau% p_udefs(isau)
       isa=lsdau% p_isiteu(isau)
       issa=lsdau% p_isortu(isau)
       
31     Format('{')
       Write(12,31,advance='no')
22     Format(i5,',')
       Write(12,22,advance='no')isa
23     Format('"',a2,'",')
       Write(12,23,advance='no')structure% p_elements(issa)
       Write(12,22,advance='no')iudef
24     Format('"',a2,'"')
       Write(12,24,advance='no')lsdau% p_labelsu(isau)
32     Format('}')
       Write(12,32)
       If(isau < lsdau% nusite) Write(12,33,advance='no')
    End Do
25  Format(2x,'};')    
    Write(12,25)    
    !!END LSDA+U
    
    
    
    !!START OPC
118 Format(2/,'# OPC definitions',2/)
    Write(12,118)
119 Format(2x,'int  opc_nodef=',i10,';')
    Write(12,119)opc% nodef    
    
121 Format('  struct {int sort;char[2] element;char[2] state;}&
         &opc[opc_nodef]={',/,'  ')
    Write(12,121,advance='no')
    
    
    Do iodef=1,opc% nodef
       issao=opc% p_isrto(iodef)
       
       Write(12,31,advance='no')
       Write(12,22,advance='no')issao
       Write(12,23,advance='no')structure% p_elements(issao)
       Write(12,24,advance='no')opc% p_state(iodef)
       Write(12,32)
       If(iodef < opc% nodef) Write(12,33,advance='no')
    End Do
    Write(12,25)    
    !!END OPC
    
    
    !! end section header
    Write(12,5) 
    
    
    
    
    
    Write(12,2)  'section data{'
    
    !!START LSDA+U
    !! occupation matrix
    If(lsdau% nusite > 0_t_int)Then

       If(rel)Then
          msp=mspin!!TODO: does that check 
          Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
               &,1_t_int,lsdau% nusite,1_t_int,1_t_int,nmodule,nsubfunc)
          pdmat => ldmat
          pdmat(:,:,:,1)=lsdau% p_dmat(:,:,:,1)
          Call lsdau_d_mat_trans_spin(lsdau,rel,pdmat,'tospin')
       Else
          msp=mspin
          pdmat => lsdau% p_dmat
       Endif

26     Format(2x,'real lsdau_dmat[2,lsdau_maxdim,lsdau_maxdim,lsdau_nsite,mspin]={')
       Write(12,26)
       !i=0_t_int
       Do ms=1,msp
          Write(12,31,advance='no')
          Do isau=1_t_int,lsdau% nusite
             l=lsdau% p_lvsu(isau)
             deg=2*l+1
             ii0=0
             mms=ms
             If(rel)Then
                ii0=deg*(ms-1)
                mms=1
             Endif
             
             Write(12,31,advance='no')
             Do i2=1,lsdau% umaxdim
                Write(12,31,advance='no')
                Do i1=1,lsdau% umaxdim
                   Write(12,31,advance='no')
                   cc=pdmat(i1+ii0,i2+ii0,isau,mms)
                   If(i1>deg .Or. i2>deg) cc=c0
27                 Format(e30.15e3)
                   Write(12,27,advance='no') Real(cc,t_real)
                   Write(12,33,advance='no')
                   Write(12,27,advance='no')Aimag(cc)
                   Write(12,32)
                   If(i1 < lsdau% umaxdim) Write(12,33,advance='no')
                Enddo
                Write(12,32,advance='no')
                If(i2 < lsdau% umaxdim) Write(12,33,advance='no')
             End Do
             Write(12,32,advance='no')
             If(isau < lsdau% nusite) Write(12,33,advance='no')
          End Do
          Write(12,32,advance='no')
          If(ms < mspin) Write(12,33,advance='no')
       End Do
       
       Write(12,5)
       
       If(rel)Then
326        Format(2x,'real lsdau_dmat_offdiagonal[2,lsdau_maxdim,lsdau_maxdim,lsdau_nsite]={')
          Write(12,326)

          Do isau=1_t_int,lsdau% nusite
             l=lsdau% p_lvsu(isau)
             deg=2*l+1
             
             Write(12,31,advance='no')
             Do i2=1,lsdau% umaxdim
                Write(12,31,advance='no')
                Do i1=1,lsdau% umaxdim
                   Write(12,31,advance='no')
                   cc=pdmat(i1,i2+deg,isau,1)
                   If(i1>deg .Or. i2>deg) cc=c0
                   Write(12,27,advance='no') Real(cc,t_real)
                   Write(12,33,advance='no')
                   Write(12,27,advance='no')Aimag(cc)
                   Write(12,32)
                   If(i1 < lsdau% umaxdim) Write(12,33,advance='no')
                Enddo
                Write(12,32,advance='no')
                If(i2 < lsdau% umaxdim) Write(12,33,advance='no')
             End Do
             Write(12,32,advance='no')
             If(isau < lsdau% nusite) Write(12,33,advance='no')
          End Do
          Write(12,5)
       Endif
       
    End If
    !!END LSDA+U
    
    
    
    !!START OPC
    !! occupation matrix
    If(opc% nodef > 0_t_int)Then
126    Format(2x,'real opc_Bracah[2,opc_nodef]={')
       Write(12,126)
       !i=0_t_int
       Do iodef=1,opc% nodef
          Write(12,31,advance='no')
          Do ms=1,2
             Write(12,27,advance='no')&
                  &opc% p_Bracah(ms,iodef,1)
             If(ms < 2) Write(12,33,advance='no')
             !If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
          Enddo
          Write(12,32,advance='no')
          If(iodef < opc% nodef) Write(12,33)
       End Do
       
       Write(12,5)
226    Format(2x,'real opc_dBracah[2,opc_nodef]={')
       Write(12,226)
       !i=0_t_int
       Do iodef=1,opc% nodef
          Write(12,31,advance='no')
          Do ms=1,2
             Write(12,27,advance='no')&
                  &opc% p_Bracah(ms,iodef,2)
             If(ms < 2) Write(12,33,advance='no')
             !If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
          Enddo
          Write(12,32,advance='no')
          If(iodef < opc% nodef) Write(12,33)
       End Do
       
       Write(12,5)
1126   Format(2x,'real opc_Morb[2,opc_nodef]={')
       Write(12,1126)
       !i=0_t_int
       Do iodef=1,opc% nodef
          Write(12,31,advance='no')
          Do ms=1,2
             Write(12,27,advance='no')&
                  &opc% p_Morb(ms,iodef)
             If(ms < 2) Write(12,33,advance='no')
             !If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
          Enddo
          Write(12,32,advance='no')
          If(iodef < opc% nodef) Write(12,33)
       End Do
       
       Write(12,5)
11126  Format(2x,'real opc_occu[2,opc_nodef]={')
       Write(12,11126)
       !i=0_t_int
       Do iodef=1,opc% nodef
          Write(12,31,advance='no')
          Do ms=1,2
             Write(12,27,advance='no')&
                  &opc% p_occu(ms,iodef)
             If(ms < 2) Write(12,33,advance='no')
             !If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
          Enddo
          Write(12,32,advance='no')
          If(iodef < opc% nodef) Write(12,33)
       End Do
       
       Write(12,5)
    End If
    !!END OPC
    
    
    
    !! density
28  Format(2x,'real dens[nr,mlmp,nsite,mspin]={')
    Write(12,28)
    i=0_t_int
    Do ms=1_t_int,mspin
       Write(12,31,advance='no')
       Do isa=1_t_int,structure% nsite
          
          pmesh => pmeshs(structure% p_isrt(isa))
          
          Write(12,31,advance='no')
          Do ilm=1_t_int,mlmp
             Write(12,31,advance='no')
             If(uniformmesh)Then
                Do ir=1_t_int,ldmesh% nr
                   Write(12,27,advance='no')&
                        & minterpol(pmesh,degree,lcd(1,ilm,isa,ms)&
                        &,ldmesh% p_rmesh(ir))
                   i=i+1_t_int
                   If(ir < ldmesh% nr) Write(12,33,advance='no')
                   If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
                Enddo
             Else
                Do ir=1_t_int,pmesh% nr
                   Write(12,27,advance='no')lcd(ir,ilm,isa,ms)
                   i=i+1_t_int
                   If(ir < pmesh% nr) Write(12,33,advance='no')
                   If(Mod(i,5_t_int) == 0_t_int) Write(12,40)
                Enddo
             Endif
             Write(12,32)
             If(ilm < mlmp) Write(12,33,advance='no')
             
          Enddo
          Write(12,32,advance='no')
          If(isa < structure% nsite) Write(12,33,advance='no')
          
       End Do
       Write(12,32,advance='no')
       If(ms < mspin) Write(12,33,advance='no')
    End Do
    Write(12,5)
    !! end section data
    Write(12,5)
    
    Close(12)
    !!
    Call cleanup_mesh(ldmesh)
    Call pfree(ldmat)
    !!
    
    If( lsdau% dmat_mode == LSDAU_DMAT_MODE_SC ) &
         &Call write_u_dmat_to_file(mspin,structure,lsdau,'dmat',rel)
    !!
!!$    Open(12,file='=.kT')!!TODO
!!$    Write(12,"('# current kT')")
!!$    Write(12,"(e30.15e3)")bzone% kTcur
!!$    Close(12)
    !!
    !!
    Return
    !!
100 Format('End Writing ',a,'!',/&
         &,'------------------------------------------------------------------------',/)
    Write(outfile,100)Trim(f_dens)
    !!
40  Format()
    !!
  End Subroutine denswrite
  !!***********************************************************************
  !!
  Subroutine densreint(degree,elements,delements&
       &,mspin,nsite,isrt,mlmp,nlmp,lmp,nmeshs,nrp,lcd&
       &,dmspin,dnsite,dmlmp,dnlmp,dlmp,ldmesh,ldmeshs,dnrp,dens&
       &,have_lmp,uniformmesh,mapping,ninslot,md)
    !!
    Implicit None 
    !!IO
    !! do we have a new file with more information?
    Logical(t_log),Intent(in) :: have_lmp,uniformmesh
    Character(2),Intent(in)   :: elements(:)
    Character(2),Intent(in)   :: delements(:)
    Integer(t_int),Intent(in) :: degree
    Logical(t_log),Intent(in) :: mapping
    !! current FPLO settings
    Type(T_Mesh),Pointer      :: nmeshs(:)
    !! mapping of densities
    Type(T_MapData),Intent(in),Optional :: md(:)
    Integer(t_int),Intent(in),Optional :: ninslot
    Integer(t_int),Intent(in) :: mspin,mlmp,nsite,nlmp(nsite),lmp(mlmp,nsite),nrp
    Integer(t_int),Intent(in) :: isrt(nsite)
    Real(t_real),Intent(inout):: lcd(nrp,mlmp,nsite,mspin)
    !! settings from dens file
    Type(T_Mesh),Target,Intent(in)   :: ldmesh
    Type(T_Mesh),Target,Intent(in)   :: ldmeshs(:)
    Integer(t_int),Intent(in) :: dmspin,dmlmp,dnsite,dnlmp(nsite),dnrp
    Real(t_real),Intent(in)   :: dens(dnrp,dmlmp,dnsite,dmspin)
    Integer(t_int),Intent(in) :: dlmp(dmlmp,nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="densreint"
    Integer(t_int) :: dil,nil,dms,nms,isa,isold,nnsite,i,issa
    Real(t_real)   :: tmp(nrp)
    Type(T_Mesh),Pointer :: nmesh
    Type(T_Mesh),Pointer :: dmesh
    !!

    nnsite=nsite
    If(mapping)nnsite=ninslot
    !!
    If(have_lmp)Then
       !!  new file more information
       isaloop1: Do i=1,nnsite
              
          !! isa is new site , isold=mapnewtoold(isa) is site in dens file
          If(mapping) Then
             isold=md(i)% oldsite
             isa=md(i)% newsite
          Else
             isa=i
             isold=i
          Endif
          issa=isrt(isa)

          If(Trim(elements(issa)) /= Trim(delements(isold)))Then
             Write(errbuf,"('''You try to map different &
                  &elements!'',/,'' error:  new site ',a,' elem. ',a2,' !=&
                  &old site ',a,' elem. ',a2,'!''')")&
                  &Trim(intToChar10(isa)),elements(issa)&
                  &,Trim(intToChar10(isold)),delements(isold)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif

          !!
          nmesh => nmeshs(issa)
          If(uniformmesh)Then 
             dmesh => ldmesh
          Else
             dmesh => ldmeshs(isold) !! yes isold not issa
          Endif
          !!
          lcd(:,:,isa,:)=0.0_t_real

          !! for all dnlmp
          !! if lmp in nlmp, reinterpol
          Do dil=1,dnlmp(isold)
             Do nil=1,nlmp(isa)
                If(dlmp(dil,isold)==lmp(nil,isa))Then
                   !1                  Format('inter isa=',i3,' dil ',i3,1x,i3,' -> nil ',i3,1x,i3)
                   !                   Print 1,isa,dil,dlmp(dil,isa),nil,lmp(nil,isa)
                   Do dms=1,dmspin
                      If(meshesEqual(dmesh,nmesh))Then
                         tmp(1:nmesh%nr)=dens(1:nmesh%nr,dil,isold,dms)
                      Else
                         Call mreinter(degree,dmesh,dens(1,dil,isold,dms)&
                              &,nmesh,tmp)
                      Endif
                      nms=Min(dms,mspin)
                      If(dmspin ==1_t_int .And. mspin == 2_t_int) Then
                         Do nms=1,mspin
                            lcd(1:nmesh%nr,nil,isa,nms)=&
                                 &tmp(1:nmesh%nr)*0.5_t_real
                         Enddo
                      Else
                         lcd(1:nmesh%nr,nil,isa,nms)=lcd(1:nmesh%nr,nil,isa,nms)&
                              &+tmp(1:nmesh%nr)
                      Endif
                   End Do
                Endif
             Enddo
          Enddo
       End Do isaloop1
    Else
       !! old file , less information
       isaloop2: Do i=1_t_int,nnsite
          !!
          !! isa is new site , mapnewtoold is site in dens file
          If(mapping) Then
             isold=md(i)% oldsite
             isa=md(i)% newsite
          Else
             isa=i
             isold=i
          Endif
          issa=isrt(isa)

          nmesh => nmeshs(issa)
          If(uniformmesh)Then 
             dmesh => ldmesh
          Else
             dmesh => ldmeshs(isold) !! yes isold not issa
          Endif
          !!
          
          lcd(:,:,isa,:)=0.0_t_real

          !! for all dnlmp
          !! if lmp in nlmp, reinterpol
          Do nil=1,Min(dnlmp(isold),nlmp(isa))
             dil=nil
             Do dms=1,dmspin
                Call mreinter(degree,dmesh,dens(1,dil,isold,dms),nmesh,tmp)
                nms=Min(dms,mspin)
                If(dmspin ==1_t_int .And. mspin == 2_t_int) Then
                   Do nms=1,mspin
                      lcd(1:nmesh%nr,nil,isa,nms)=&
                           &tmp(1:nmesh%nr)*0.5_t_real
                   Enddo
                Else
                   lcd(1:nmesh%nr,nil,isa,nms)=lcd(1:nmesh%nr,nil,isa,nms)&
                        &+tmp(1:nmesh%nr)
                Endif
             End Do
          Enddo
       End Do isaloop2
    Endif
    !!
    Return
    !!
  End Subroutine densreint
  !!***********************************************************************
  Subroutine init_densmapdata(dmd)
    !!
    Implicit None
    !!IO
    Type(T_DensMapData),Intent(inout) :: dmd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_densmapdata"
    !!
    dmd% nslots=0
    Nullify(dmd% p_ninslot)
    Nullify(dmd% p_md)
    !!
    !!
    Return
    !!
  End Subroutine init_densmapdata
  !!***********************************************************************
  Subroutine clean_densmapdata(dmd)
    !!
    Implicit None
    !!IO
    Type(T_DensMapData),Intent(inout) :: dmd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="clean_densmapdata"
    !!
    If(Associated(dmd% p_ninslot))Then
       Deallocate(dmd% p_ninslot)
    Endif
    If(Associated(dmd% p_md))Then
       Deallocate(dmd% p_md)
    Endif
    Nullify(dmd% p_ninslot)
    Nullify(dmd% p_md)
    !!
    !!
    Return
    !!
  End Subroutine clean_densmapdata
  !!***********************************************************************
  Subroutine readdensmap(nsite,dmd)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite
    Type(T_DensMapData),Intent(inout):: dmd
    !!LOCAL
    Character(*),Parameter::nsubfunc="readdensmap"
    Integer(t_int) :: nline,isnew,isold,i,j,ip,issold
    Character(5000)   :: buf,word,file
    Character(100)    :: elold
    Integer :: eof,icheck
    Type(T_MapData),Pointer :: p_md(:)
    Type(T_MapData) :: d
    !!
    Nullify(p_md)
    Allocate(p_md(nsite),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for p_md !'")
    Endif
    !!
    !!
    p_md(1:nsite)% file=""
    p_md(1:nsite)% oldsite=0
    p_md(1:nsite)% oldsort=0
    p_md(1:nsite)% newsite=0
    !!
    Open(12,file=f_densmap)
    nline=0
    readloop: Do 
       Read(12,12,iostat=eof) buf
       If(eof < 0 ) Exit readloop
       nline=nline+1
       If( Len_trim(buf)==0) Cycle readloop
       buf=Adjustl(buf)
       If( buf(1:1)=='#') Cycle readloop
       If(.Not.next_word(buf,word)) Goto 6651
       Read(word,*)isnew
       If(.Not.next_word(buf,word)) Goto 6652
       Read(word,*)elold
       If(.Not.next_word(buf,word)) Goto 6653
       Read(word,*)isold
       If(.Not.next_word(buf,word)) Goto 6654
       Read(word,*)issold
       file=""
       If(isold>=1)Then
          If(.Not.next_word(buf,word)) Goto 6655
          Read(word,"(a)")file
       Endif
       !Print "(i3,i3,'<',a,'>')",isnew,isold,Trim(file)
       If(isnew<1 .Or. isnew>nsite)Then
          Write(errbuf,100)Trim(f_densmap),nline,isnew
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
       Endif
       p_md(isnew)%newsite=isnew
       p_md(isnew)%oldsite=isold
       p_md(isnew)%oldsort=issold
       p_md(isnew)%file=Trim(file)
    End Do readloop
    Close(12)
    !!
    Allocate(dmd% p_md(nsite,nsite),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for p_md !'")
    Endif
    Allocate(dmd% p_ninslot(nsite),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for p_md !'")
    Endif
    !!
    dmd% p_ninslot(1:nsite)=0
    !!
    i=1
    dmd% nslots=1
    dmd% p_md(i,dmd% nslots)=p_md(i)
    Do While(i<=nsite)
       !Print *,'iwhile:',i
       ip=i+1
       iploop: Do While(ip<=nsite)
          If(Trim(p_md(i)% file)==Trim(p_md(ip)% file))Then
             !Print *,ip-i+1,dmd% nslots
             dmd% p_md(ip-i+1,dmd% nslots)=p_md(ip)
             ip=ip+1
          Else
             Exit iploop
          Endif
       End Do iploop
       !Print *,'next new:',i,ip
       jloop: Do j=ip,nsite
          If(Trim(p_md(i)% file)==Trim(p_md(j)% file))Then
             d=p_md(ip)
             p_md(ip)=p_md(j)
             p_md(j)=d
             dmd% p_md(ip-i+1,dmd% nslots)=p_md(ip)
             ip=ip+1
          Endif
       Enddo jloop
       !Print *,'collected:',ip-i
       dmd% p_ninslot(dmd% nslots)=ip-i
       i=ip
       If(i<=nsite) Then
          dmd% nslots=dmd% nslots+1
          dmd% p_md(1,dmd% nslots)=p_md(ip)

       Endif
    Enddo
    !!
    
    !Print "('slots=',i3)",dmd% nslots
    !Do i=1,nsite
    !   Write(outfile,"(i3,i3,1x,a)")&
    !        &p_md(i)% newsite&
    !        &,p_md(i)% oldsite&
    !        &,Trim(p_md(i)%file)
    !Enddo

    If(.False.)Then
       Write(outfile,"(/,/,'different =.dens files: ',i3)")dmd% nslots
       Do i=1,dmd% nslots
          If(dmd% p_md(1,i)% oldsite==0)Cycle
          Write(outfile,"('From file: ',a,/,' current-site site- sort- from file')")&
               &Trim(dmd% p_md(1,i)% file)
          Do j=1,dmd% p_ninslot(i)
             Write(outfile,"(i7,1x,i5,1x,i5)")&
                  &dmd% p_md(j,i)% newsite&
                  &,dmd% p_md(j,i)% oldsite&
                  &,dmd% p_md(j,i)% oldsort
          Enddo
          Write(outfile,*)
       Enddo
    Endif
    !!
    Deallocate(p_md)
    !!
    !!
    Return
    !!
6651 Continue
    Write(errbuf,"('''File ''''',a,''''' line ',a,' corrupted.'',/,'' &
         &Expected int!''')")&
         &Trim(f_densmap),Trim(intToChar10(nline))
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
6652 Continue
    Write(errbuf,"('''File ''''',a,''''' line ',a,' corrupted.'',/,'' &
         &Expected int el!''')")&
         &Trim(f_densmap),Trim(intToChar10(nline))
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
6653 Continue
    Write(errbuf,"('''File ''''',a,''''' line ',a,' corrupted.'',/,'' &
         &Expected int el int!''')")&
         &Trim(f_densmap),Trim(intToChar10(nline))
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
6654 Continue
    Write(errbuf,"('''File ''''',a,''''' line ',a,' corrupted.'',/,'' &
         &Expected int el int int !''')")&
         &Trim(f_densmap),Trim(intToChar10(nline))
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
6655 Continue
    Write(errbuf,"('''File ''''',a,''''' line ',a,' corrupted.'',/,'' &
         &Expected int el int int filename!''')")&
         &Trim(f_densmap),Trim(intToChar10(nline))
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)

!2   Format('new site ',i5,' has new mapping')
100 Format('"File ''',a,''', line ',i5,' : Incorrect new site number (''',i5&
         &,''') !?"')
!101 Format('"File ''',a,''', line ',i5,' : Incorrect old site number (''',i5&
!         &,''') !?"')
12  Format(a)
    !!
  End Subroutine readdensmap
  !!***********************************************************************
  !! when mapping correct uncompensated charges
  Subroutine recharge(mspin,structure,potential)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(inout) :: potential
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    !Character(*),Parameter::nsubfunc="recharge"
    Integer(t_int) :: isa,issa,ms,j,ir
    Real(t_real)   :: dq,q,f
    Type(T_Mesh),Pointer :: pmesh
    !!
    dq=(Sum(potential %dens_charge(:))+potential% molecule_ionicity&
         &-potential% tot_nuc_charge)&
         &/structure% nsite/Real(mspin,t_real)
    !!
    Do isa=1_t_int, structure % nsite
       issa=structure% p_isrt(isa)
       pmesh => potential% p_pmeshs(issa)
       Do ms=1_t_int,mspin
          q=potential% p_charges(isa,ms)
          f=1.0_t_real
          If(q/=0.0_t_real)Then
             f=(q-dq)/q
          Endif
          Do j=1,potential% p_nlmp(isa)
             Do ir=1,pmesh% nr
                potential% p_lcd(ir,j,isa,ms)=&
                     &potential% p_lcd(ir,j,isa,ms)&
                     &-dq*func2(pmesh% p_rmesh(ir))
                
             Enddo
          Enddo
       Enddo
    End Do
    !!
    Return
    !!
  Contains
    Real(t_real) Function func2(r)
      !!
      Implicit None  
      !!IO
      Real(t_real),Intent(in)      :: r
      !!LOCAL
      Real(t_real),Parameter   :: r0=2.0_t_real
      Real(t_real)   :: x
      x=r/r0
      func2=2.0_t_real*x**2/Pi/r0*Exp(-x**2)
    End Function func2
    !!
    Real(t_real) Function func3(r)
      !!
      Implicit None  
      !!IO
      Real(t_real),Intent(in)      :: r
      !!LOCAL
      Real(t_real),Parameter   :: r0=3.0_t_real
      Real(t_real)   :: x
      x=r/r0
      func3=x**3/sqpi/r0*Exp(-x**2)
    End Function func3
    !!
    !!
  End Subroutine recharge
  !!***********************************************************************
  Subroutine debugdensvectorout(fname,mspin,structure,potential,lcd)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin
    Type(T_Structure),Intent(in) :: structure 
    Type(T_Potential),Intent(in) :: potential 
    Real(t_real),Intent(in)      :: lcd(:,:,:,:)
    Character(*),Intent(in)      :: fname
    !!LOCAL
    !Character(*),Parameter::nsubfunc="debugdensvectorout"
    Integer(t_int) :: isa,j
    Type(T_Mesh),Pointer :: pmesh
    !!
    !!
    Print "(' -> ',a)",Trim(fname)
    Open(15,file=Trim(fname))
    If(mspin==2)Then
       Do isa=1,structure% nsite
          pmesh => potential% p_pmeshs(structure% p_isrt(isa))
          Do j=1,potential% p_nlmp(isa)
             Write(15,"(e30.15e3)")&
                  &0.0_t_real,&
                  &(lcd(2:,j,isa,1)+lcd(2:,j,isa,2))&
                  &*Sqrt(pmesh% p_jmesh(2:)/pmesh% p_rmesh(2:)**2/structure% vuc)
          Enddo
       Enddo
       Do isa=1,structure% nsite
          pmesh => potential% p_pmeshs(structure% p_isrt(isa))
          Do j=1,potential% p_nlmp(isa)
             Write(15,"(e30.15e3)")&
                  &0.0_t_real,&
                  &(lcd(2:,j,isa,1)-lcd(2:,j,isa,2))&
                  &*Sqrt(pmesh% p_jmesh(2:)/pmesh% p_rmesh(2:)**2/structure% vuc)
          Enddo
       Enddo
    Else
       Do isa=1,structure% nsite
          pmesh => potential% p_pmeshs(structure% p_isrt(isa))
          Do j=1,potential% p_nlmp(isa)
             Write(15,"(e30.15e3)")&
                  &0.0_t_real,&
                  &potential% p_lcd(2:,j,isa,1)&
                  &*Sqrt(pmesh% p_jmesh(2:)/pmesh% p_rmesh(2:)**2/structure% vuc)
          Enddo
       Enddo
    Endif
    Close(15)
    
    !!
    Return
    !!
  End Subroutine debugdensvectorout
  !!***********************************************************************
  !! make initial density
  Subroutine init_dens(compound,structure&
       &,potential,initial_spin_split,opc,lsdau,relativistic&
       !&,bzone&
       &,vca,forceit)
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(in) :: lsdau
    !Type(T_Bzone),Intent(in) :: bzone
    Type(T_OPC),Intent(in)   :: opc
    Logical(t_log),Intent(in):: vca,forceit
    Logical(t_log),Intent(inout)            :: initial_spin_split
    Character(*),Intent(in)          :: compound
    Integer(t_int),Intent(in)    :: relativistic
    Type(T_Structure),Intent(in)     :: structure   
    Type(T_Potential),Intent(inout)  :: potential   
    !!LOCAL
    Character(*),Parameter::nsubfunc="init_dens"
    logical(t_log) :: exist,rel
    Real(t_real) :: eff,Bfsm
    Integer(t_int) :: mlmp,ms
    Integer(t_int) :: nlmp(structure % nsite)
    Integer(t_int) :: lmp(1,structure % nsite)
    Real(t_real),Pointer  :: lcd(:,:,:,:)
    !!
    Nullify(lcd)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    !!........................................................................
    !! look for dens-file
    !! if exist only tests are done
    !! if not exist initial density is calculated
    !!
    mlmp=1_t_int
    nlmp(:)=1_t_int
    lmp(1,:)=1_t_int
    Inquire(file=f_dens,exist=exist)
    If(.Not.exist.Or.forceit) Then
       !!
       !!....................................................................
       !! something we have to write
       eff=-0.08_t_real
       Bfsm=-0.0_t_real
       !!
       !!.....................................................................
       !!     calculate atomic density
       Write(outfile,40)
       !! tmp dens array
       !! the start density should be nonpolarized, thus ms=1
       !! this is good for the initial spin split test
       ms=1_t_int
       Call palloc('lcd',lcd&
            &,1_t_int,potential% nrp&
            &,1_t_int,mlmp&
            &,1_t_int,structure % nsite&
            &,1_t_int,ms&
            &,nmodule,nsubfunc)
       
       Call startatom(ms,structure % nsite&
            &,structure % nsort,structure % p_isrt,structure% p_elements&
            &,potential % p_natom,potential% p_pmeshs,potential% nrp&
            &,mlmp,lcd,potential% molecule_ionicity,vca) 
       
       Call denswrite(compound,structure,ms,potential% p_pmeshs,potential% nrp&
            &,eff,Bfsm,mlmp,nlmp,lmp,lcd,opc,lsdau,relativistic,.False.&
            !&,bzone&
            &)

       If(outlevel>=L_many) Write(outfile,41)
       Call pfree(lcd)
       
       !!
       !!....................................................................
       !!     check initial spin splitting
    Else ! does exist
       Call initparser
       Call setparsestop("data_header")
       Call readfile(f_dens)
       Call getinteger('mspin',ms)
       Call cleanreadfile

       If(ms == 2_t_int .And. initial_spin_split)Then
          Write(outfile,42) trim(f_dens)
          Write(outfile,43)
          If(lyn())Then
             initial_spin_split=.False.
             Write(outfile,"('Spin split skipped!')")
          Endif
       Endif
       If(initial_spin_split)Then
          Write(outfile,"('Will make initial spin split!')")
       Endif
       !!
    End If
    
    !!.......................................................................
40  Format('-------------INITIAL ATOM CALCULATION-----------------')
41  Format('------------------------------------------------------' &
         & ,/'Please be aware that the atomic total energies, given above' &
         & ,/'are not correct, because, the compression is ' &
         & ,/'switched on, and the accuracy of the initial atomic ' &
         & ,/'calculations is low !')
42  Format(72('-'),/'''',a,''' Contains aready 2 spin sorts, which may be splitted allready.',/)
43  Format('Do you want to skip the possibly repeated initial spin splitting ?')
    !!
  End Subroutine init_dens
  !!***********************************************************************
  !! interface between FPLO and DIRAC
  !! set global dirac data and make atom calculation
  !! calculate for all sorts an initial density via a atomic calculation
  !!
  Subroutine startatom(mspin,nsite,nsort,isrt,elements,natom,pmeshs,nrp,mlmp&
       &,lcd,molecule_ionicity,vca)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer     :: pmeshs(:)    ! FPLO mesh
    Logical(t_log),Intent(in):: vca
    Integer(t_int),Intent(in):: nsite,nsort,isrt(*),nrp
    Real(t_real) ,Intent(in) :: natom(nsort)
    Character(2),Intent(in)  :: elements(nsort)
    Integer(t_int),Intent(in):: mspin,mlmp
    Real(t_real),Intent(in)  :: molecule_ionicity
    Real(t_real) ,Intent(out):: lcd(nrp,mlmp,nsite,mspin) !  density 
    !!LOCAL
    !Character(*),Parameter::nsubfunc="startatom"
    Logical(t_log) :: prot
    Integer(t_int) ::isa,issa,ir
    Real(t_real)  :: r,d
    Integer(t_int) ::nrat          !! local settings, good for atom program
    Real(t_real)  ::rminat,rmaxat !! local settings, good for atom program
    Type(T_Mesh),Pointer :: pmesh
    Character(100)   :: str
    !!........................................................................
    !!
    
    prot=(outlevel>=L_many)
    !!
    !! this are good settings for atom program
    nrat=1000_t_int
    rminat=1.0e-4_t_real

    lcd(:,:,:,:)=0.0_t_real
    
    !Open(12,file='+initdens')
    Do issa=1_t_int,nsort

       Write(str,"('startatom',a)")intToChar10(issa)

       pmesh => pmeshs(issa)

       rmaxat=pmesh% rmax
       
      
       !! do not make initial dens for empty spheres
       If(natom(issa) > 1.0e-5_t_real) Then 
          !!
          Call init_tatom(g_atomat)
          !! default init input data
          Call diracinit(g_atomat)
          
          g_atomat% standalone=.False.
          g_atomat% relativistic = SCALAR_RELATIVISTIC_KH
          g_atomat% gam_factor=.False.
          g_atomat% gam_factor=.True.!!needed for GGA

       
          !! set specifics
          g_atomat% mesh% nr=nrat
          g_atomat% mesh% rmin=rminat
          g_atomat% mesh% rmax=rmaxat
          
          g_atomat% noUnOccupied=.True.
          g_atomat% force_pos_spin=.True.
          g_atomat% wftol=1.e-6_t_real
          
          !! allways calculate compressed states for initiation
          g_atomat% compr=.True.
          g_atomat% r0=2.0_t_real**1.5
          g_atomat% exn0=4.0_t_real
          g_atomat% Z=natom(issa)
          g_atomat% vca=vca
          g_atomat% element=elements(issa)
          g_atomat% nuc_type=FINUC_NONE
          g_atomat% ionicity=Max(molecule_ionicity/nsite,0.0_t_real)
          Call set_def_atom_basis_and_occus(g_atomat)
          Call updateRawOccus(g_atomat)
          
          g_atomat% iteration_control% scheme   =IT_SCHEME_ITERAT
          g_atomat% iteration_control% maxdim   =5_t_int
          g_atomat% iteration_control% tolerance=1.e-3_t_real
          g_atomat% iteration_control% mixing   = 0.2_t_real
          g_atomat% iteration_control% progress = 0.1_t_real
          g_atomat% iteration_control% steps    = 100_t_int

          !! setup allocation
          Call diracsetup(g_atomat)
          !! do calculation
          Call iterate(sccatom,str,.False.,prot)

          !! reinterpolation and storage of density
          Do isa=1_t_int,nsite
             If(isrt(isa) == issa )Then   

                Do ir=1_t_int,pmesh% nr
                   r=pmesh% p_rmesh(ir)
                   d=minterpol(g_atomat% mesh,8_t_int,g_atomat% p_rho,r)
                   lcd(ir,1_t_int,isa,1_t_int)=d/Real(mspin,t_real)/sq4pi
                   If(mspin == 2_t_int) lcd(ir,1_t_int,isa,2_t_int)=&
                        &d/Real(mspin,t_real)/sq4pi
                End Do
                
                !Do ir=1,g_atomat% mesh% nr
                !   Write(12,"(2(e30.15e3,2x))")g_atomat% mesh% p_rmesh(ir)&
                !        &,g_rhotrrat(ir)
                !End Do

                !Do ir=1,pmesh% nr
                !   Write(13,"(2(e30.15e3,2x))")&
                !        &pmesh% p_rmesh(ir),lcd(ir,isa,1)
                !End Do
             End If
          End Do ! isa
          !Write(12,"(' ')")
          !Write(13,"(' ')")
          
          !! cleanup allocation          
          Call clean_tatom(g_atomat)
          !!
       End If
       
    Enddo
    !Close(12)

        
  End Subroutine startatom
  !!***********************************************************************
End Module DENSIO
