  !!----DEBUG options--------------------|
  !! DENSITY:calc_density:NO_OVERDENS    |
  !! DENSITY:calc_density:NO_NETDENS     |
  !! DENSITY:calc_density:NO_ROTLOC      |
  !!-------------------------------------|
Module density
  !!
  Use precisiondefs
  Use constants
  Use tmol
  Use tsymmetry
  Use tpotentialdensity
  Use errors
  Use memory
  Use clock
  Use trelativistic
  Use symmetry
  Use matrixinvers
  Use netdensity
  Use tbzone
  Use occu
  Use overlapdensity
  Use tforces
  Use forces
  Use integrate1d
  Use tdiracdefs
  Use torbitalbasis
  Use tnumerics
  Use tmesh
  Use mesh
  Use tkohnsham
  Use finuc
  Use optionstest
  Use file_iounits
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="DENSITY"  
  !!
  Public :: calc_density,density_analysis,print_charge_moment_summary
  !!***********************************************************************
Contains 
  !!***********************************************************************
  !! calculates the charge density
  Subroutine calc_density(mspin&
       &,symmetry,structure,lattice,potential,mol,forces&
       &,nsite&
       &,bzone&
       &,ncsmx,nvsmx&
       &,kohnsham,basis&
       &,relativistic&
       &,numerics,quant,force_loop)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(inout) :: potential
    Type(T_Mol),Intent(in)       :: mol
    Type(T_Forces),Intent(inout) :: forces
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Bzone), Intent(in)    :: bzone
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Quant),Intent(in)     :: quant
    Type(T_Numerics),Intent(in)  :: numerics
    Logical(t_log),Intent(in)    :: force_loop
    Integer(t_int),Intent(in)    :: mspin,relativistic,nsite
    Integer(t_int),Intent(in)    :: ncsmx,nvsmx
    !!LOCAL
    Character(*),Parameter  ::nsubfunc="calc_density"
    Character(*),Parameter  :: cpumesg_netdens='Net density'
    Character(*),Parameter  :: cpumesg_dndV='dndV-correction'
    Character(*),Parameter  :: cpumesg_ovldens='Overlap density'
    Logical(t_log) :: rel,debnoovl,debnonet,debnorotloc
    Integer(t_int) :: ms,isa,issa,n,mapnvsmx,mapncsmx,onr,one
    Type(T_Mesh),Pointer :: pmesh
    Type(T_Mesh)   :: imesh
    !! automatic arrays
    Real(t_real),Pointer     :: onc(:,:,:,:,:,:)
    Real(t_real),Pointer     :: onv(:,:,:,:,:,:)
    Real(t_real),Pointer     :: lcdnet(:,:,:,:)
    !!-----------------------------------------------------------------------
    Nullify(onc,onv)
    Nullify(lcdnet)
    Call init_mesh(imesh)
    !!
    debnoovl=debugfun(nmodule,nsubfunc,'NO_OVERDENS')
    debnonet=debugfun(nmodule,nsubfunc,'NO_NETDENS')
    debnorotloc=debugfun(nmodule,nsubfunc,'NO_ROTLOC')
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    !!
    !!
    !! initialize density=lcd
    potential% p_lcd(:,:,:,:)=0.0_t_real
    !!
    If(force_loop)Then
       Call palloc('lcdnet',lcdnet,1_t_int,potential% nrp&
            &,1_t_int,potential% mlmp,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       lcdnet(:,:,:,:)=0.0_t_real
    Endif
    !!
    !! START net/onsite density        
    !!
    Call watch(cpumesg_netdens,cpu_start,reset=.True.)
    !!
    !!
    !!
    !!
    !!
    If(.Not.debnonet)Then
       !!
       Call calc_net_density(mspin,structure,potential&
            &,potential% max_L,potential% p_nlmp,potential% p_lmp&
            &,basis&
            &,kohnsham% p_nonvv,kohnsham% p_noncv,kohnsham% p_noncc&
            &,kohnsham% p_ronvv(:,:,:,1),kohnsham% p_roncv(:,:,:,1)&
            &,kohnsham% p_roncc(:,:,:,1)&
            &,quant,potential% p_pmeshs,potential% p_lcd&
            &,numerics% degree,relativistic,printorbmom=.True.,useOVO=use_OVO)
       !!
    Endif
    !!
    !!
    !!
    If(force_loop)Then
       lcdnet(:,:,:,:)=potential% p_lcd(:,:,:,:)
    Endif
    !!
    Call watch(cpumesg_netdens,cpu_stop,delete=.True.,Print=.True.)
    !!
    !! END net/onsite density        
    !!     
    !!
    !!
    !! START occupation numbers
    !!
    !!
    If(rel)Then
       mapnvsmx=nvsmx
       mapncsmx=ncsmx
       Do issa=1,structure% nsort
          n=Sum(2*basis% p_lRv(1:basis% p_nRv(issa),issa)+1)
          mapnvsmx=Max(mapnvsmx,n)
          n=Sum(2*basis% p_lRc(1:basis% p_nRc(issa),issa)+1)
          mapncsmx=Max(mapncsmx,n)
       Enddo
       If(mapnvsmx - nvsmx /=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'assumption of relativistic state counting wrong!'")
       Endif
       If(mapncsmx - ncsmx /=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'assumption of relativistic state counting wrong!'")
       Endif
    Endif
    !!
    Call palloc('onc',onc,1_t_int,ncsmx,1_t_int,nvsmx&
         &,1_t_int,nsite,1_t_int,nsite,1_t_int,mspin&
         &,1_t_int,Max(1_t_int,lattice% npairsmax),nmodule,nsubfunc)
    Call palloc('onv',onv,1_t_int,nvsmx,1_t_int,nvsmx&
         &,1_t_int,nsite,1_t_int,nsite,1_t_int,mspin&
         &,1_t_int,Max(1_t_int,lattice% npairsmax),nmodule,nsubfunc)
    !!
    !!
    !!
    !!
    Call make_ovloccus(mspin,symmetry,structure,lattice,basis,bzone&
         &,kohnsham,kohnsham% p_bweitk,relativistic,onc,onv,.False.&
         &,force_loop)
    !!
    !!
    
    !! START overlap/gross density        
    !!
    If(isoption('NO_CORE'))Then
       onc(:,:,:,:,:,:)=0.0_t_real
    Endif
    !!
    If(debnoovl) Goto 1000
    Call watch(cpumesg_ovldens,cpu_start,reset=.True.)            
    !!
    !Call watch('OVDENS-occu',cpu_start,reset=.True.)
    !Call watch('OVDENS-occu',cpu_stop)
    !! make large mesh, leave input unchanged
    !!
    onr=Nint(numerics% ovldns_nr  *2.0_t_real,t_int)
    one=Nint(numerics% ovldns_neta*2.0_t_real,t_int)
    !!
    !!
    Do ms=1_t_int,mspin
       Do isa=1_t_int,nsite
          issa=structure% p_isrt(isa)
          !!
          pmesh => potential% p_pmeshs(issa)
          !!
          !!smaller radial temporay mesh 
          imesh% Type=numerics% ovldns_meshtype
          imesh% rmax=pmesh% rmax
          imesh% rmin=numerics% ovldns_rmin
          imesh% nr=onr
          imesh% with_zero=.True.
          Call setup_mesh(imesh)
          !!
          Call calc_overlapdensity(mspin,ms,nsite,structure% nsort,isa,issa&
               &,structure% p_isrt&
               &,structure% p_conao&
               &,lattice% npairsmax,lattice% p_npairs,lattice% p_pairs&
               &,lattice% p_npshell,lattice% p_ninpshell&
               &,lattice% maxinpshell,lattice% p_pairabs&
               &,pmesh,potential% nrp,basis% p_basmeshs,basis% nrb,imesh&
               &,basis% ncomp,nvsmx,ncsmx&
               &,basis% lmx,basis% nRvmx,basis% p_nRv,basis% p_lRv&
               &,basis% nRcmx,basis% p_nRc,basis% p_lRc&
               &,basis% p_fv,basis% p_fc&
               &,basis% p_fgam&
               &,basis% p_rtfcmax,basis% p_rtfvmax,basis% p_rfvmax&
               &,basis% p_rfcmax&
               &,potential% max_L,potential% mlmp,potential% p_nlmp&
               &,potential% p_lmp,potential% p_lcd,onv,onc&
               &,one,numerics% degree)
          !!
          Call cleanup_mesh(imesh)
          !!
       Enddo                   ! isa
    Enddo                     ! ms
    !!
    Call cleanup_mesh(imesh)
    !!
    !Call watch('OVDENS-occu',cpu_read,delete=.True.&
    !     &,Print=.True.,mesg='Total ')

    Call watch(cpumesg_ovldens,cpu_stop,delete=.True.,Print=.True.)
    !!
    !!
1000 Continue
    !! END overlap/gross density        
    !!
    !!rotation of density to correct for irreduzible Brioullin zone integration
    !!TODO:
    !!TODO: test this for rotated quantization axis
    !!TODO:
    If(.Not.debnorotloc)Then
       Do ms=1_t_int,mspin
          Call rotloc(mspin,ms,potential% p_lcd,potential% mlmp&
               &,potential% p_nlmp,potential% p_lmp,potential% nrp&
               &,nsite,structure% p_equivsites&
               &,symmetry% Rnumops,symmetry% opeuler&
               &,symmetry% opeulerinv,symmetry% Ropind &
               &,potential% max_L)
       Enddo ! ms
       If(force_loop)Then
          Do ms=1_t_int,mspin
             Call rotloc(mspin,ms,lcdnet,potential% mlmp&
                  &,potential% p_nlmp,potential% p_lmp,potential% nrp&
                  &,nsite,structure% p_equivsites&
                  &,symmetry% Rnumops,symmetry% opeuler&
                  &,symmetry% opeulerinv,symmetry% Ropind &
                  &,potential% max_L)
          Enddo ! ms
       Endif
    Endif
    !!
    If(force_loop)Then
       !!
       Call watch(cpumesg_dndV,cpu_start,reset=.True.)
       Call dndV_forces(mspin,symmetry,structure,lattice,basis,potential&
            &,mol,forces,numerics% degree,relativistic,lcdnet,onc,onv)
       Call watch(cpumesg_dndV,cpu_stop,delete=.True.,Print=.True.)
    Endif
    !!
    !!
    !!
    !!
    !!
    Call pfree(onc)
    Call pfree(onv)
    Call pfree(lcdnet)
    !!
    Return
    !!
  End Subroutine calc_density
  !!***********************************************************************
  !! print out and convergence organization of total energy
  Subroutine print_charge_moment_summary(mspin,structure,potential)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Integer(t_int) :: isa,issa
    Real(t_real)   :: a,b,dnat
    !!
    Write(outfile,*)
    Write(outfile,200)
    Write(outfile,210)
    a=potential% dens_charge(1_t_int)
    b=potential% dens_charge(2_t_int)
    If(mspin == 1_t_int) Then
       a=potential% dens_charge(1_t_int)*0.5_t_real
       b=potential% dens_charge(1_t_int)*0.5_t_real
    Endif
    Write(outfile,220)potential% tot_nuc_charge,a+b,a-b,a,b
    
    Write(outfile,*)
    Write(outfile,300)
    Write(outfile,310)
    Write(outfile,300)
    Do isa=1_t_int,structure% nsite
       issa=structure% p_isrt(isa)
       dnat=potential% p_natom(issa)
       a=potential% p_charges(isa,1_t_int)
       b=potential% p_charges(isa,2_t_int)
       If(mspin == 1_t_int) Then
          a=potential% p_charges(isa,1_t_int)*0.5_t_real
          b=potential% p_charges(isa,1_t_int)*0.5_t_real
       Endif
       Write(outfile,320)structure% p_elements(issa),isa,a-b,dnat
    Enddo
    Write(outfile,300)
    !!
    Return
    !!
200 Format(72('-')/10('='),t33,'CHARGE',t63,10('=')/72('-'))
210 Format( t3,'nu. charge',t17,'el. charge',t31,'mag. moment',t47&
         &,'spin up',t61,'spin down')
220 Format( t3,f10.2,t15,f12.7,t30,f12.7,t44,f12.7,t59,f12.7)
300 Format('-------------------------------------------------')
310 Format('|  ATOM   |  SITE  |  MAG.MOMENT  |  NU.CHARGE  |')
320 Format('|',t5,a,t11,'|',t13,i3,t20,'|',t22,f10.7,t35,'|',t39,f7.2,t49,'|')
    !!
  End Subroutine print_charge_moment_summary
  !!***********************************************************************
  !!
  Subroutine density_analysis(structure,mspin,potential,numerics,titerstep&
       &,relativistic,dontshow)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Potential),Intent(inout) :: potential
    Integer(t_int),Intent(in)    :: mspin,titerstep,relativistic
    Logical(t_log),Intent(in),Optional    :: dontshow
    !!LOCAL
    Character(*),Parameter::nsubfunc="density_analysis"
    Integer(t_int) :: isa,ms,ig,issa,ir
    Real(t_real)  :: ss,corss,d,xg,rg,jg,al,x,M(3,3),b(3),x0,coeff(0:4)
    Logical(t_log) :: indef,lshow
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_ww(:),p_xx(:)
    !!
    corss=0.0_t_real
    !!
    If(Present(dontshow)) Then
       lshow=.Not.dontshow
    Else
       lshow=.True.
    Endif
    
    If(lshow)Then
       Write(outfile,1)
1      Format(78('-'))
       Write(outfile,2)
       Write(outfile,1)
    End If
    !!
    !!
    Call get_gauss(numerics% coulerg_nr,p_xx,p_ww)
    al=3.0_t_real
    !!

    potential % dens_charge(:)=0.0_t_real
    Do isa=1_t_int, structure % nsite
       issa=structure% p_isrt(isa)
       !!
       pmesh => potential% p_pmeshs(issa)
       !!
       potential% p_charges(isa,:)=0.0_t_real
       Do ms=1_t_int,mspin

          
          ss=0.0_t_real
          Do ig=1,numerics% coulerg_nr
             xg=(p_xx(ig)+1.0_t_real)*0.5_t_real
             rg=xg**al*pmesh% rmax
             jg=al*rg/xg*p_ww(ig)*0.5_t_real
             
             
             d=minterpol(pmesh,numerics% degree,potential % p_lcd(:,1,isa,ms),rg)
             ss=ss+d*jg
          Enddo

          ss=(ss+corss)*sq4pi
          potential% p_charges(isa,ms)=ss
          potential % dens_charge(ms)=potential % dens_charge(ms)+ss&
               &*structure % p_conao(structure % p_isrt(isa))
       Enddo
       If(lshow)Then
          If(mspin == 1_t_int) Then
             Write(outfile,3) isa,structure% p_elements(structure% p_isrt(isa))&
                  &,potential% p_charges(isa,1_t_int)*0.5_t_real&
                  &,potential% p_charges(isa,1_t_int)*0.5_t_real&
                  &,Sum(potential% p_charges(isa,:))&
                  &,potential % p_natom(structure % p_isrt(isa))&
                  &,Sum(potential% p_charges(isa,:))&
                  &-potential % p_natom(structure % p_isrt(isa))
          Else
             Write(outfile,3) isa,structure% p_elements(structure% p_isrt(isa))&
                  &,potential% p_charges(isa,1_t_int)&
                  &,potential% p_charges(isa,2_t_int)&
                  &,Sum(potential% p_charges(isa,:))&
                  &,potential % p_natom(structure% p_isrt(isa))&
                  &,Sum(potential% p_charges(isa,:))&
                  &-potential% p_natom(structure % p_isrt(isa))
          Endif
       Endif
    Enddo

    If(lshow)Then
       Write(outfile,1)
       
       If(mspin ==  1_t_int) Then
          Write(outfile,4) &
               & potential % dens_charge(1_t_int)*0.5_t_real&
               &,potential % dens_charge(1_t_int)*0.5_t_real&
               &,Sum(potential %dens_charge(:)),potential %tot_nuc_charge&
               &,Sum(potential %dens_charge(:))-potential %tot_nuc_charge
       Else
          Write(outfile,4) &
               & potential % dens_charge(1_t_int)&
               &,potential % dens_charge(2_t_int)&
               &,Sum(potential %dens_charge(:)),potential %tot_nuc_charge&
               &,Sum(potential %dens_charge(:))-potential %tot_nuc_charge
       Endif
       Write(outfile,1)
       !!
       If(Abs(Sum(potential % dens_charge(:))+potential% molecule_ionicity&
            &-potential %tot_nuc_charge)&
            &/structure% nsite > 1.e-4_t_real)Then
567       Format("'Total ionicity deviates to much from zero !'&
               &,/,' Check the calculation!'") 
          Write(errbuf,567)
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf,titerstep=titerstep)
       Endif
    End If
    !!
2   Format(32x,'Density Analysis',/&
         & ,'site elem. Spin up       Spin down    Sum           &
         &Nuclear        Excess',/&
         & ,'           (electrons)   (electrons)  (electrons)   &
         &(protons)      electrons')
3   Format(i3,2x,a2,1x,3(f13.8,1x),2(f13.8,1x))
4   Format('sum  :  ',3(f13.8,1x),2(f13.8,1x))
    
    !! maximum extension of density
    !! Call dens_rmax(structure,mspin,potential,numerics)
    !!
    If(lshow)&
         & Call print_charge_moment_summary(mspin,structure,potential)
    !!
    If(.Not.lshow) Return
    If(potential% nuc_type==FINUC_NONE .And. &
         &relativistic/=NON_RELATIVISTIC)Then
       Write(outfile,101)
    Else
       Write(outfile,"(/,/)")
       Do isa=1_t_int, structure % nsite
          issa=structure% p_isrt(isa)
          !!
          pmesh => potential% p_pmeshs(issa)
          !!
          If(Abs(pmesh% p_rmesh(1))>1.0e-12_t_real)Then
             Write(outfile,100)
             Cycle
          Endif
          x0=pmesh% p_rmesh(2)
          Do ir=2,4
             x=pmesh% p_rmesh(ir)/x0
             b(ir-1)=Sum(potential% p_lcd(ir,1,isa,1:mspin))
             M(ir-1,1)=x**2
             M(ir-1,2)=x**3
             M(ir-1,3)=x**4
          Enddo
          !!
!!$       Print *,'b=',b(1:3)
!!$       Print *,'M='
!!$       Print *,M(1,1:3)
!!$       Print *,M(2,1:3)
!!$       Print *,M(3,1:3)
!!$       !!
          Call rinv(M,M,3_t_int,3_t_int,indef)
          !!
          coeff(0:1)=0.0_t_real
          coeff(2:4)=Matmul(M,b)
!!$       !!
!!$       Print *,'c=',coeff(2:4)
!!$       !!
!!$       Do ir=1,5
!!$          x=pmesh% p_rmesh(ir)/x0
!!$          Print *,horner(x,coeff,4_t_int),Sum(potential% p_lcd(ir,1,isa,1:mspin))
!!$       Enddo
          !!
          Write(outfile,200) structure% p_elements(issa),isa,coeff(2)/sq4pi/x0**2
       Enddo
       Write(outfile,"(/,/)")
    Endif
    !!
    Return
    !!
100 Format('Mesh does not start with zero ... no r=0 densities.')
101 Format('(Scalar)Relativistic and no finite nucleus: no r=0 densities.')
200 Format('DENSITY AT ORIGIN: ',a2,'(',i3.3,') ',f30.15)
    !!
    !!
    !!
  End Subroutine density_analysis
!!$  !!***********************************************************************
!!$  !! determine rdmax
!!$  !!
!!$  Subroutine dens_rmax(structure,mspin,potential,numerics)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Structure),Intent(in) :: structure
!!$    Type(T_Potential),Intent(inout) :: potential
!!$    Type(T_Numerics),Intent(in)  :: numerics
!!$    Integer(t_int) ,Intent(in)    :: mspin
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="dens_rmax"
!!$    Integer(t_int) :: ms,isa,j,ir,imin
!!$    Real(t_real)  :: rdmin,rdmax
!!$    Type(T_Mesh),Pointer :: pmesh
!!$    !!
!!$    rdmax=0.0_t_real
!!$    Do ms=1_t_int,mspin
!!$       Do isa=1_t_int,structure% nsite
!!$          !!
!!$          pmesh => potential% p_pmeshs(structure% p_isrt(isa))
!!$          !!
!!$          !dr=lattice% p_rsite(structure% p_isrt(isa))
!!$          Do j=1_t_int,potential% p_nlmp(isa)
!!$             rdmin=pmesh% rmax
!!$             imin=pmesh% nr
!!$             rloop: Do ir=pmesh% nr,2_t_int,-1_t_int
!!$                If(Abs(potential% p_lcd(ir,j,isa,ms)&
!!$                     &/pmesh% p_rmesh(ir)**2) >  numerics% cuttol_dens) &
!!$                     &Exit rloop
!!$                rdmin=pmesh% p_rmesh(ir)
!!$                imin=ir
!!$             Enddo rloop
!!$             If(imin >=pmesh% nr-2_t_int )Then
!!$                rdmax=Max(rdmax,pmesh% rmax)
!!$             Else
!!$                rdmax=Max(rdmax,rdmin)
!!$             Endif
!!$          Enddo
!!$       Enddo
!!$    Enddo
!!$    
!!$1   Format(a,': max extension of density=',f15.8)
!!$    Write(outfile,1) Trim(nsubfunc),rdmax
!!$
!!$    !!
!!$  End Subroutine dens_rmax
  !!***********************************************************************

End Module density
  
