!!----------------------------------!
!! DIAGONALIZE:dontuselapack        |
!!----------------------------------!
Module diagonalize
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="DIAGONALIZE"  
  !!
  Public :: &
       & rbchd,cdiagno,cdiagno_noO,ctestdia,ctestdia_noO& !! complex hermitian
       &,rdiagno,rtestdia& !! real symmetric
       &,rdiagno_noO,rtestdia_noO& !! rela symmetric no overlap
       &,mainaxis_decopose,cdiagno_unitary_noO
  !!***********************************************************************
contains 
  !!***********************************************************************
  !! diagonalize a real symmetric 3x3 problem and sort eigenvectors such
  !! that the 1st vector has largest 1st component, the 2nd vector has largest
  !! 2nd component among vector 2 and 3, and all i,i-elements positive 
  Subroutine mainaxis_decopose(h,z,e)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)      :: h(3,3)
    Real(t_real),Intent(out)     :: z(3,3),e(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mainaxis_decopose"
    Real(t_real)   :: dum(3),d
    !!
    z=h
    !!
    Call rdiagno_noO(3_t_int,z,e)
    !!
    If( Abs(z(1,1)) < Abs(z(1,2)) ) Then
       dum=z(:,1)
       z(:,1)=z(:,2)
       z(:,2)=dum
       d=e(1)
       e(1)=e(2)
       e(2)=d
    Endif
    If( Abs(z(1,1)) < Abs(z(1,3)) ) Then
       dum=z(:,1)
       z(:,1)=z(:,3)
       z(:,3)=dum
       d=e(1)
       e(1)=e(3)
       e(3)=d
    Endif
    If( z(1,1) < 0.0_t_real ) z(:,1)=-z(:,1)
    If( Abs(z(2,2)) < Abs(z(2,3)) ) Then
       dum=z(:,2)
       z(:,2)=z(:,3)
       z(:,3)=dum
       d=e(2)
       e(2)=e(3)
       e(3)=d
    Endif
    !!
    If( z(2,2) < 0.0_t_real ) z(:,2)=-z(:,2)
    If( z(3,3) < 0.0_t_real ) z(:,3)=-z(:,3)
    !!
    Return
    !!
  End Subroutine mainaxis_decopose
  !!***********************************************************************
  Subroutine cdiagno_unitary_noO(nvdim,M,e)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(inout):: M(:,:)
    Complex(t_real),Intent(out)     :: e(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="cdiagno_unitary_noO"
    Logical(t_log) :: uselapack
#if defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK)
    Integer :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: dim
    Complex(t_real)   :: w(1)
    Complex(t_real),Pointer:: work(:)
    Complex(t_real),Pointer:: vr(:,:),rwork(:)
#endif
    !!
    uselapack=.False.
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    uselapack=.True.
    If(debugfun(nmodule,'dontuselapack',''))uselapack=.False.
#endif
    If(uselapack)Then
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
       !!
       !Print *,'nvdim=',nvdim
       Nullify(work,rwork,vr)
       Call palloc('rwork',rwork,1_t_int,Max(1_t_int,2_t_int*nvdim)&
            &,nmodule,nsubfunc)
       Call palloc('vr',vr,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
       !!
       !Print *,'trying lwork'
       lnvdim=nvdim
       lwork=-1
       Call zgeev('N','V',lnvdim,M,lnvdim,e,w,1,vr,lnvdim, w, lwork, rwork, info)
       lwork=Max(1,Int(w(1)))
       dim=lwork
       !Print *,'lwork=',w,lwork
       !!
       Call palloc('work',work,1_t_int,dim,nmodule,nsubfunc)
       !!
       Call zgeev('N','V',lnvdim,M,lnvdim,e,w,1,vr,lnvdim, work, lwork, rwork, info)
       If(info/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in lapack::dsyev'")
       Endif
       !!
       M=vr
       !!
       Call pfree(work)
       Call pfree(rwork)
       Call pfree(vr)
       !!
#endif
    Else
       !!
       If(nvdim==2)Then
          Call diagnounit2x2(M,e)
       Else If(nvdim==1)Then
          e(1)=M(1,1)
          M(1,1)=Cmplx(1.0_t_real,0.0_t_real,t_real)
       Else
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'diagonalization of a unitary matrix with dimension>2 not &
               &implemented',/,'Consider using the MKL library option?'")
       Endif
       !!
    Endif
    !!
    Return
    !!
  End Subroutine cdiagno_unitary_noO
  !!***********************************************************************
  Subroutine diagnounit2x2(M,e)
    !!
    Implicit None
    !!IO
    Complex(t_real),Intent(inout):: M(2,2)
    Complex(t_real),Intent(out)  :: e(2)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="diagnounit2x2"
    Real(t_real)   :: ar,ai,d
    Complex(t_real):: det,cb,A,C
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!todo: protect against small b and small ar
    !!
    cb=M(1,2)
    det=Sqrt(M(1,1)*M(2,2)-M(2,1)*cb)

    ar=Real(M(1,1)/det,t_real)
    ai=Aimag(M(1,1)/det)
    cb=cb/det
    d=Sqrt(1.0_t_real-ar**2)
    e(1)=Cmplx(ar,-d,t_real)*det
    e(2)=Cmplx(ar,+d,t_real)*det
    !!
    If(Abs(cb)==0.0_t_real)Then
       M(1,1)=Cmplx(1.0_t_real,0.0_t_real,t_real)
       M(2,2)=Cmplx(1.0_t_real,0.0_t_real,t_real)
       M(1,2)=Cmplx(0.0_t_real,0.0_t_real,t_real)
       M(2,1)=Cmplx(0.0_t_real,0.0_t_real,t_real)
    Endif
    !!
    !!
    If(ai<0)Then
       A=Sqrt((d-ai)/d*0.5_t_real)
       C=-ci*(d+ai)/cb*A
    Else
       A=(d-ai)*Sqrt((d+ai)/d*0.5_t_real)/abs(cb)
       C=-ci*(d+ai)/cb*A
    Endif
    M(1,1)=A
    M(2,1)=C
    M(2,2)=A
    M(1,2)=-Conjg(C)
    
    Return
    !!
  End Subroutine diagnounit2x2
  !!***********************************************************************
  !!
  !!- Diagonalizes secular equation  (h*z-z*eband)=0
  !!----------------------------------------------------------------------
  !!i Inputs:
  !!i    nvdim  :dimension of problem
  !!io   h     :hamilton matrix
  !!o Outputs:
  !!io   h     :hamilton matrix is OVERWRITTEN in this routine
  !!o    z     :eigenvectors
  !!o    eband :eigenvalues
  !!r Remarks:
  !!r    h,z are dimensioned (nvdim,nvdim) 
  !!     Find the eigenvalues and vectors of  H   ---
  !!----------------------------------------------------------------------
  !!***********************************************************************
  Subroutine rdiagno_noO(nvdim,h,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    real(t_real),Intent(inout):: h(:,:)
    Real(t_real),Intent(out)     :: eband(:)
    !!LOCAL
    Logical(t_log) :: uselapack
#if defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK)
    Integer :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    Character(*),Parameter::nsubfunc="rdiagno_noO"
    Integer(t_int) :: dim
    Real(t_real)   :: w(1)
    Real(t_real),Pointer:: work(:)
#endif
    !!
    uselapack=.False.
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    uselapack=.True.
    If(debugfun(nmodule,'dontuselapack',''))uselapack=.False.
#endif
    !!
    If(uselapack)Then
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
       !Print *,'use lapack routines'
       Nullify(work)
       !!
       lnvdim=nvdim
       lwork=-1
       Call dsyev('V','U',lnvdim,h,lnvdim,eband,w,lwork,info)
       lwork=Max(1,Int(w(1)))
       dim=lwork
       !!
       Call palloc('work',work,1_t_int,dim,nmodule,nsubfunc)
       !!
       Call dsyev('V','U',lnvdim,h,lnvdim,eband,work,lwork,info)
       If(info/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in lapack::dsyev'")
       Endif
       !!
       !!
       Call pfree(work)
#endif
    Else
       !Print *,'use internal routines'
       Call rdiagno_noO_internal(nvdim,h,eband)
    Endif
    !!
    Return
    !!
  End Subroutine rdiagno_noO
  !!***********************************************************************
  Subroutine rdiagno_noO_internal(nvdim,hh,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)   :: nvdim
    Real(t_real),Intent(inout)  :: hh(nvdim,nvdim)
    Real(t_real),Intent(out)    :: eband(nvdim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rdiagno_noO_internal"
    Integer(t_int)  ::  ierr,i
    !! automatic arrays
    Real(t_real)  :: tau(2,nvdim)
    Real(t_real)  :: d(nvdim)
    Real(t_real)  :: e(nvdim)
    Real(t_real)  :: e2(nvdim)
    Real(t_real)  :: e3(nvdim)
    Real(t_real)  :: rv1(nvdim)
    Real(t_real)  :: rv2(nvdim)
    Real(t_real)  :: rv3(nvdim)
    Real(t_real)  :: rv4(nvdim)
    Real(t_real)  :: rv5(nvdim)
    Integer(t_int):: iwk(nvdim)
    Real(t_real)  :: zz(nvdim,nvdim)
    !!

    !! --- Reduction of a real symmetric matrix to a real symmetric
    !! --- tridiagonal matrix using unitary similarity transformations

    Call rbtridi(nvdim,hh,d,e,e2,tau)


    !! ----- Make unit eigenvector matrix ---
    zz(:,:)=0.0_t_real 
    Do i=1_t_int,nvdim
       zz(i,i)=1.0_t_real
    Enddo
    eband(:)=d(:)
    !! ----- Get eigenvalues and eigenvectors of symmetric tridiag. matrix
    !! ----- by implicit QL method
    e3=e
    Call imtql2(nvdim,nvdim,eband,e3,zz,ierr)

    If (ierr /= 0_t_int) Then
       Call errmsg(nmodule,nsubfunc,E_information,level=Int(L_all,t_int)&
            &,msg="'error in IMTQL2: trying another subroutine.'")

       Call imtqlv(nvdim,d,e,e2,eband,iwk,ierr,rv1)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in IMTQLV.'")
       Endif
       Call binvit(nvdim,d,e,e2,nvdim,eband,iwk,zz,ierr,rv1,rv2,rv3,rv4,rv5)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in BINVIT.'")
       End If
    Endif

    
    !! --- Get eigenvectors of real symmetric matrix by back transforming
    !! --- those of the corresponding real symmetric tridiagonal matrix
    Call rbtribk(nvdim,hh,tau,nvdim,zz)
    hh=zz
    !!
    Return
    !!
  End Subroutine rdiagno_noO_internal
  !!***********************************************************************
  !!
  !!  test if HP*ZZ=ZZ*EB
  !!
  Subroutine rtestdia_noO(nvdim,nband,hp,zz,eb,tol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim,nband
    Real(t_real),Intent(in)      :: hp(nvdim,nvdim)
    Real(t_real),Intent(in)      :: zz(nvdim,nvdim),eb(*)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: k,iz,ii
    Real(t_real)  ::  a,c,b,d
    !!
    Do k=1_t_int,nband
       Do iz=1_t_int,nvdim
          a=0.0_t_real
          b=0.0_t_real
          Do ii=1_t_int,nvdim
             a=a+hp(iz,ii)*zz(ii,k)
             b=b+zz(ii,iz)*zz(ii,k)
          Enddo
          c=(zz(iz,k))*eb(k)
          d=0.0_t_real
          if(k==iz)d=1.0_t_real
          If(Abs(a-c) > tol) Then
             Write(outfile,14) a,a-c
          Endif
          If(Abs(b-d) > tol) Then
             Write(outfile,15) b,b-d
          Endif
       Enddo
    Enddo
    !!
14  Format('RTESTDIA: H.C     =',f15.7,',  err|H.C-C.e|   = ',e15.5e2,' ')
15  Format('RTESTDIA: C^{T}.C =',f15.7,',  err|C^{T}.C-1| =' ,e15.5e2,' ')
    !!     
  End Subroutine rtestdia_noO
  !!***********************************************************************
  !!
  !!- Diagonalizes secular equation with overlap (h-o*eband)z=0
  !!----------------------------------------------------------------------
  !!i Inputs:
  !!i    nvdim  :dimension of problem
  !!io   h     :hamilton matrix
  !!io   o     :overlap matrix
  !!o Outputs:
  !!io   h     :hamilton matrix is OVERWRITTEN in this routine
  !!io   o     :overlap matrix is OVERWRITTEN in this routine
  !!o    z     :eigenvectors
  !!o    eband :eigenvalues
  !!r Remarks:
  !!r    h,o,z are dimensioned (nvdim,nvdim) 
  !!     Find the eigenvalues and vectors of O^-1/2  H  O^-1/2 ---
  !!----------------------------------------------------------------------
  Subroutine rdiagno(nvdim,h,o,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Real(t_real),Intent(inout):: h(nvdim,nvdim)
    Real(t_real),Intent(inout):: o(nvdim,nvdim)
    Real(t_real),Intent(out)     :: eband(nvdim)
    !!LOCAL
    Logical(t_log) :: uselapack
#if defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: lwork,info,lnvdim,itype
#endif
#if defined (USE_MKL_LAPACK)
    Integer :: lwork,info,lnvdim,itype
#endif
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    Character(*),Parameter::nsubfunc="rdiagno"
    Integer(t_int) :: dim
    Real(t_real)   :: w(1)
    Real(t_real),Pointer:: work(:)
#endif
    !!
    uselapack=.False.
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    uselapack=.True.
    If(debugfun(nmodule,'dontuselapack',''))uselapack=.False.
#endif
    !!
    If(uselapack)Then
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
       !Print *,'use lapack routines'
       Nullify(work)
       !!
       itype=1
       lnvdim=nvdim
       lwork=-1
       Call dsygv(itype,'V','U',lnvdim,h,lnvdim,o,lnvdim,eband,w,lwork,info)
       lwork=Max(1,Int(w(1)))
       dim=lwork
       !!
       Call palloc('work',work,1_t_int,dim,nmodule,nsubfunc)
       !!
       Call dsygv(itype,'V','U',lnvdim,h,lnvdim,o,lnvdim,eband,work,lwork,info)
       If(info>nvdim)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'S is indefinite in lapack::dsygv'")
       Else If(info/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in lapack::dsygv'")
       Endif
       !!
       !!
       Call pfree(work)
#endif
    Else
       !Print *,'use internal routines'
       Call rdiagno_internal(nvdim,h,o,eband)
    Endif
    !!
    Return
    !!
  End Subroutine rdiagno
  !!***********************************************************************
  Subroutine rdiagno_internal(nvdim,hh,o,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Real(t_real),Intent(inout)  :: hh(nvdim,nvdim)
    Real(t_real),Intent(inout)  :: o(nvdim,nvdim)
    Real(t_real),Intent(out)    :: eband(nvdim)

    !!LOCAL
    Character(*),Parameter::nsubfunc="rdiagno_internal"
    Integer(t_int)  ::  ierr,i
    !! automatic arrays
    Real(t_real)  :: d(nvdim)
    Real(t_real)  :: e(nvdim)
    Real(t_real)  :: e2(nvdim)
    Real(t_real)  :: rv1(nvdim)
    Real(t_real)  :: rv2(nvdim)
    Real(t_real)  :: rv3(nvdim)
    Real(t_real)  :: rv4(nvdim)
    Real(t_real)  :: rv5(nvdim)
    Real(t_real)  :: tau(2,nvdim)
    Integer(t_int):: iwk(nvdim)
    Real(t_real)  :: zz(nvdim,nvdim)
    !!
    !! --- Cholesky decomposition of overlap matrix
    Call rbchd (o,nvdim,ierr)

    If(ierr /= 0_t_int)Then
       Write(errbuf,"('''RBCHD: overlap matrix indefinite'',/&
            &,'' possible reasons:  '',/&
            &,'' (1) max LOIs should be small enough'',/&
            &,'' (2) basis orbitals do not fall off'',/&
            &,'' sufficiently fast or are ill behaved'',/&
            &,'' either the atom program crashed'',/&
            &,'' or the lattice constants are to small'',/&
            &,'' or the highest core states should be'',/&
            &,'' valence states ...''')")
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    End If
    !! --- Reduction of nonorthogonal symmetric matrix to orthogonal form


    Call rbred(nvdim,hh,o,zz)

    !! --- Reduction of a real symmetric matrix to a real symmetric
    !! --- tridiagonal matrix using unitary similarity transformations

    Call rbtridi(nvdim,hh,d,e,e2,tau)


    Call imtqlv(nvdim,d,e,e2,eband,iwk,ierr,rv1)
    If (ierr /= 0_t_int) Then
       Call errmsg(nmodule,nsubfunc,E_information,level=Int(L_all,t_int)&
            &,msg="'error in IMTQLV: trying another subroutine.'")
    Else
       Call binvit(nvdim,d,e,e2,nvdim,eband,iwk,zz,ierr,rv1,rv2,rv3,rv4,rv5)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_information,level=Int(L_all,t_int)&
               &,msg="'error in BINVIT: trying another subroutine.'")
       End If
    Endif
    If (ierr /= 0_t_int) Then
       !! ----- previous method did not work, try this one:
       !! ----- Make unit eigenvector matrix ---
       zz(:,:)=0.0_t_real 
       Do i=1_t_int,nvdim
          zz(i,i)=1.0_t_real
       Enddo
       eband(:)=d
       !! ----- Get eigenvalues and eigenvectors of symmetric tridiag. matrix
       !! ----- by implicit QL method
       Call imtql2(nvdim,nvdim,eband,e,zz,ierr)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'error in IMTQL2'")
       Endif
    Endif
    !! --- Get eigenvectors of real symmetric matrix by back transforming
    !! --- those of the corresponding real symmetric tridiagonal matrix
    Call rbtribk(nvdim,hh,tau,nvdim,zz)

    !! --- Backtransformation of eigenvectors to nonorthogonal representation
    !! --- Get the eigenvectors of H - E O ---
    Call rbbak(nvdim,o,zz)
    hh=zz
    !!
  End Subroutine rdiagno_internal
  !!***********************************************************************
  !!
  !!  test if HP*ZZ=SP*ZZ*EB
  !!
  Subroutine rtestdia(nvdim,nband,hp,sp,zz,eb,tol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim,nband
    Real(t_real),Intent(in)      :: hp(nvdim,nvdim),sp(nvdim,nvdim)
    Real(t_real),Intent(in)      :: zz(nvdim,nvdim),eb(*)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: k,iz,ii
    Real(t_real)  ::  a,c
    !!
    Do k=1_t_int,nband
       Do iz=1_t_int,nvdim
          a=0.0_t_real
          c=0.0_t_real
          Do ii=1_t_int,nvdim
             a=a+hp(iz,ii)*zz(ii,k)
             c=c+(sp(iz,ii)*zz(ii,k))*eb(k)
          Enddo
          If(Abs(a-c) > tol) Then
             Write(outfile,14) a,a-c
          Endif
       Enddo
    Enddo
    !!
14  Format('RTESTDIA: H.C  =',f15.7,',  err|H.C-S.C.e|   = ',e15.5e2,' ')
    !!     
  End Subroutine rtestdia
  !!***********************************************************************
  !!
  !!- Diagonalizes secular equation with overlap (h-o*eband)z=0
  !!----------------------------------------------------------------------
  !!i Inputs:
  !!i    nvdim  :dimension of problem
  !!io   h     :hamilton matrix
  !!io   o     :overlap matrix
  !!o Outputs:
  !!io   h     :hamilton matrix is OVERWRITTEN in this routine
  !!io   o     :overlap matrix is OVERWRITTEN in this routine
  !!o    z     :eigenvectors
  !!o    eband :eigenvalues
  !!r Remarks:
  !!r    h,o,z are dimensioned (nvdim,nvdim,2) (real followed by imag. parts)
  !!     Find the eigenvalues and vectors of O^-1/2  H  O^-1/2 ---
  !!----------------------------------------------------------------------
  Subroutine cdiagno(nvdim,h,o,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(inout):: h(nvdim,nvdim)
    Complex(t_real),Intent(inout):: o(nvdim,nvdim)
    Real(t_real),Intent(out)     :: eband(nvdim)
    !!LOCAL
    Logical(t_log) :: uselapack
#if defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: lwork,info,lnvdim,itype
#endif
#if defined (USE_MKL_LAPACK)
    Integer :: lwork,info,lnvdim,itype
#endif
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    Character(*),Parameter::nsubfunc="cdiagno"
    Integer(t_int) :: dim
    Real(t_real)   :: w(1)
    Real(t_real),Pointer :: rwork(:)
    Complex(t_real),Pointer:: work(:)
#endif
    !!
    uselapack=.False.
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    uselapack=.True.
    If(debugfun(nmodule,'dontuselapack',''))uselapack=.False.
#endif
    !!
    If(uselapack)Then
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
       !Print *,'use lapack routines'
       Nullify(rwork,work)
       Call palloc('rwork',rwork,1_t_int,Max(1_t_int,3_t_int*nvdim-2_t_int)&
            &,nmodule,nsubfunc)
       !!
       itype=1
       lnvdim=nvdim
       lwork=-1
       Call zhegv(itype,'V','U',lnvdim,h,lnvdim,o,lnvdim,eband,w,lwork,rwork,info)
       lwork=Max(1,Int(w(1)))
       dim=lwork
       !!
       Call palloc('work',work,1_t_int,dim,nmodule,nsubfunc)
       !!
       Call zhegv(itype,'V','U',lnvdim,h,lnvdim,o,lnvdim,eband,work,lwork,rwork,info)
       If(info>nvdim)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'S is indefinite in lapack::zhegv'")
       Else If(info/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in lapack::zhegv'")
       Endif
       !!
       !!
       Call pfree(work)
       Call pfree(rwork)
#endif
    Else
       !Print *,'use internal routines'
       Call cdiagno_internal(nvdim,h,o,eband)
    Endif
    !!
    Return
    !!
  End Subroutine cdiagno
  !!***********************************************************************
  Subroutine cdiagno_internal(nvdim,hh,o,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(inout):: hh(nvdim,nvdim)
    Complex(t_real),Intent(inout):: o(nvdim,nvdim)
    Real(t_real),Intent(out)     :: eband(nvdim)

    !!LOCAL
    Character(*),Parameter::nsubfunc="cdiagno_internal"
    Integer(t_int)  ::  ierr,i
    !! automatic arrays
    Real(t_real)  :: e(nvdim)
    Real(t_real)  :: e2(nvdim)
    Real(t_real)  :: e3(nvdim)
    Real(t_real)  :: d(nvdim)
    Real(t_real)  :: wz(nvdim,nvdim)
    Real(t_real)  :: rv1(nvdim)
    Real(t_real)  :: rv2(nvdim)
    Real(t_real)  :: rv3(nvdim)
    Real(t_real)  :: rv4(nvdim)
    Real(t_real)  :: rv6(nvdim)
    Real(t_real)  :: tau(2,nvdim)
    Integer(t_int) :: iwk(nvdim)
    Complex(t_real):: zz(nvdim,nvdim)
    !!
    !! --- Cholesky decomposition of overlap matrix

    Call cbchd (o,nvdim,ierr)


    If(ierr /= 0_t_int)Then
       Write(errbuf,"('''BCHD: overlap matrix indefinite'',/&
            &,'' possible reasons:  '',/&
            &,'' (1) max LOIs should be small enough'',/&
            &,'' (2) basis orbitals do not fall off'',/&
            &,'' sufficiently fast or are ill behaved'',/&
            &,'' either the atom program crashed'',/&
            &,'' or the lattice constants are to small'',/&
            &,'' or the highest core states should be'',/&
            &,'' valence states ...''')")
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    End If
    !! --- Reduction of nonorthogonal hermitian matrix to orthogonal form


    Call cbred(nvdim,hh,o,zz)


    !! --- Reduction of a complex hermitian matrix to a real symmetric
    !! --- tridiagonal matrix using unitary similarity transformations

    Call cbtridi(nvdim,hh,d,e,e2,tau)




    !! ----- Make unit eigenvector matrix ---
    wz(:,:)=0.0_t_real 
    Do i=1_t_int,nvdim
       wz(i,i)=1.0_t_real
    Enddo
    eband(:)=d
    !! ----- Get eigenvalues and eigenvectors of symmetric tridiag. matrix
    !! ----- by implicit QL method
    e3=e
    Call imtql2(nvdim,nvdim,eband,e3,wz,ierr)
    !!
    !!
    !!
    If (ierr /= 0_t_int) Then
       Call errmsg(nmodule,nsubfunc,E_information,level=Int(L_all,t_int)&
            &,msg="'error in IMTQL2: trying another subroutine.'")
       !!
       !! ----- previous method did not work, try this one:

       Call imtqlv(nvdim,d,e,e2,eband,iwk,ierr,rv1)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in IMTQLV.'")
       Endif
       !!
       Call binvit(nvdim,d,e,e2,nvdim,eband,iwk,wz,ierr,rv1,rv2,rv3,rv4,rv6)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in BINVIT.'")
       End If
    Endif


    !! --- Get eigenvectors of complex hermitian matrix by back transforming
    !! --- those of the corresponding real symmetric tridiagonal matrix
    Call cbtribk(nvdim,hh,tau,nvdim,zz,wz)



    !! --- Backtransformation of eigenvectors to nonorthogonal representation
    !! --- Get the eigenvectors of H - E O ---
    Call cbbak(nvdim,o,zz)
    !!
    hh=zz
    !!
    Return
    !!
  End Subroutine cdiagno_internal
  !!***********************************************************************
  Subroutine cdiagno_noO(nvdim,h,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(inout):: h(:,:)
    Real(t_real),Intent(out)     :: eband(:)
    !!LOCAL
    Logical(t_log) :: uselapack
#if defined (USE_MKL_LAPACK_64)
    Integer(t_int) :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK)
    Integer :: lwork,info,lnvdim
#endif
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    Character(*),Parameter::nsubfunc="cdiagno_noO"
    Integer(t_int) :: dim
    Real(t_real)   :: w(1)
    Real(t_real),Pointer :: rwork(:)
    Complex(t_real),Pointer:: work(:)
#endif
    !!
    uselapack=.False.
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
    uselapack=.True.
    If(debugfun(nmodule,'dontuselapack',''))uselapack=.False.
#endif
    !!
    If(uselapack)Then
#if defined (USE_MKL_LAPACK) || defined (USE_MKL_LAPACK_64)
       !Print *,'use lapack routines'
       Nullify(rwork,work)
       Call palloc('rwork',rwork,1_t_int,Max(1_t_int,3_t_int*nvdim-2_t_int)&
            &,nmodule,nsubfunc)
       !!
       lnvdim=nvdim
       lwork=-1
       Call zheev('V','U',lnvdim,h,lnvdim,eband,w,lwork,rwork,info)
       lwork=Max(1,Int(w(1)))
       dim=lwork
       !!
       Call palloc('work',work,1_t_int,dim,nmodule,nsubfunc)
       !!
       Call zheev('V','U',lnvdim,h,lnvdim,eband,work,lwork,rwork,info)
       If(info/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in lapack::zheev'")
       Endif
       !!
       !!
       Call pfree(work)
       Call pfree(rwork)
#endif
    Else
       !Print *,'use internal routines'
       Call cdiagno_noO_internal(nvdim,h,eband)
    Endif
    !!
    Return
    !!
  End Subroutine cdiagno_noO
  !!***********************************************************************
  Subroutine cdiagno_noO_internal(nvdim,hh,eband)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(inout):: hh(nvdim,nvdim)
    Real(t_real),Intent(out)     :: eband(nvdim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="cdiagno_noO_internal"
    Integer(t_int)  ::  ierr,i,j
    !! automatic arrays
    Real(t_real)  :: e(nvdim)
    Real(t_real)  :: e2(nvdim)
    Real(t_real)  :: e3(nvdim)
    Real(t_real)  :: d(nvdim)
    Real(t_real)  :: wz(nvdim,nvdim)
    Real(t_real)  :: rv1(nvdim)
    Real(t_real)  :: rv2(nvdim)
    Real(t_real)  :: rv3(nvdim)
    Real(t_real)  :: rv4(nvdim)
    Real(t_real)  :: rv6(nvdim)
    Real(t_real)  :: tau(2,nvdim)
    Integer(t_int) :: iwk(nvdim)
    Complex(t_real):: zz(nvdim,nvdim)
    !!
    !!
    !! --- setup required form 
    !!
    Do i=1,nvdim
       Do j=i,nvdim
          hh(i,j)=Conjg(hh(i,j))
       Enddo
    Enddo
    !!
    !!
    
    !! --- Reduction of a complex hermitian matrix to a real symmetric
    !! --- tridiagonal matrix using unitary similarity transformations

    Call cbtridi(nvdim,hh,d,e,e2,tau)




    !! ----- Make unit eigenvector matrix ---
    wz(:,:)=0.0_t_real 
    Do i=1_t_int,nvdim
       wz(i,i)=1.0_t_real
    Enddo
    eband(:)=d
    !! ----- Get eigenvalues and eigenvectors of symmetric tridiag. matrix
    !! ----- by implicit QL method
    e3=e
    Call imtql2(nvdim,nvdim,eband,e3,wz,ierr)
    !!
    !!
    !!
    If (ierr /= 0_t_int) Then
       Call errmsg(nmodule,nsubfunc,E_information,level=Int(L_all,t_int)&
            &,msg="'error in IMTQL2: trying another subroutine.'")
       !!
       !! ----- previous method did not work, try this one:

       Call imtqlv(nvdim,d,e,e2,eband,iwk,ierr,rv1)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in IMTQLV.'")
       Endif
       !!
       Call binvit(nvdim,d,e,e2,nvdim,eband,iwk,wz,ierr,rv1,rv2,rv3,rv4,rv6)
       If (ierr /= 0_t_int) Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'error in BINVIT.'")
       End If
    Endif


    !! --- Get eigenvectors of complex hermitian matrix by back transforming
    !! --- those of the corresponding real symmetric tridiagonal matrix
    Call cbtribk(nvdim,hh,tau,nvdim,zz,wz)
    !!
    hh=zz
    !!
    !!
    !!
  End Subroutine cdiagno_noO_internal
  !!***********************************************************************
  !!
  !!  test if HP*ZZ=SP*ZZ*EB
  !!
  Subroutine ctestdia_noO(nvdim,nband,hp,zz,eb,tol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim,nband
    Complex(t_real),Intent(in)   :: hp(nvdim,nvdim)
    Real(t_real),Intent(in)      :: eb(*)
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: k,iz,ii
    Complex(t_real):: ca,cd
    !!
    Do k=1_t_int,nband
       Do iz=1_t_int,nvdim
          ca=Cmplx(0.0_t_real,0.0_t_real,t_real)
          Do ii=1_t_int,nvdim
             ca=ca+hp(iz,ii)*zz(ii,k)
          Enddo
          cd=ca-zz(iz,k)*eb(k)
          If(Abs(cd) > tol) Then
             Write(outfile,14) Real(ca,t_real),Aimag(ca)&
                  &,Real(cd,t_real),Aimag(cd)

          Endif
       Enddo
    Enddo
    !!
14  Format('CTESTDIA: H.C : (',f15.7,',',f15.7,')  H.C-S.C.e : (' &
         & ,e15.5e3,',',e15.5e3,')')
    !!     
  End Subroutine ctestdia_noO
  !!***********************************************************************
  !!
  !!  test if HP*ZZ=SP*ZZ*EB
  !!
  Subroutine ctestdia(nvdim,nband,hp,sp,zz,eb,tol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvdim,nband
    Complex(t_real),Intent(in)   :: hp(nvdim,nvdim),sp(nvdim,nvdim)
    Real(t_real),Intent(in)      :: eb(*)
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: k,iz,ii
    Complex(t_real):: ca,cc,cd
    !!
    Do k=1_t_int,nband
       Do iz=1_t_int,nvdim
          ca=Cmplx(0.0_t_real,0.0_t_real,t_real)
          cc=Cmplx(0.0_t_real,0.0_t_real,t_real)
          Do ii=1_t_int,nvdim
             ca=ca+hp(iz,ii)*zz(ii,k)
             cc=cc+sp(iz,ii)*zz(ii,k)*eb(k)
          Enddo
          cd=ca-cc
          If(Abs(cd) > tol) Then
             Write(outfile,14) Real(ca,t_real),Aimag(ca)&
                  &,Real(cd,t_real),Aimag(cd)

          Endif
       Enddo
    Enddo
    !!
14  Format('CTESTDIA: H.C : (',f15.7,',',f15.7,')  H.C-S.C.e : (' &
         & ,e15.5e2,',',e15.5e2,')')
    !!     
  End Subroutine ctestdia
  !!***********************************************************************
  !!- Cholesky decomposition of hermetian matrix
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system
  !!io  o     :overlap matrix (only full upper triangle is used)
  !!o Outputs:
  !!io  o     :full lower triangle of Cholesky factor l of o
  !!o          The strict upper triangle is the cc of the lower one
  !!o   ierr  :nonzero if decomposition finds matrix not pos. def.
  !!r Remarks:
  !!r   Adapted from hchd which is adapted from eispack reduc for
  !!r   hermetian matrices. bchd is written for square matrices.
  !!r   At the end the strict lower triangle is set to the
  !!r   complex conjugated of the strict upper one. Then it is possible to
  !!r   change the indices in particular for the innermost loop so
  !!r   that the first array index equals the loop variable in bbak
  !! ----------------------------------------------------------------------
  Subroutine cbchd(co,n,ierr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Integer(t_int),Intent(out)   :: ierr
    Complex(t_real),Intent(inout):: co(n*n)
    !!LOCAL
    Integer(t_int) :: i,j,k,im1,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: xr,y,det,ndet
    Complex(t_real):: cx
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          co(ji) = co(ij)
       Enddo
    Enddo

    !! --- form l in the arrays o
    det=1.0_t_real
    ndet=1.0_t_real
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       y=1.0_t_real
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          cx =  Conjg(co(ji))
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             cx = cx-co(ki)*Conjg(co(kj))
          Enddo
          If (j /= i) Then
             co(ji) = cx / y
             co(ij) =  Conjg(co(ji))
          Else
             ierr = i
             xr=Real(cx,t_real)
             If (xr <=  0.0_t_real) Return
             y = Sqrt(xr)
             ndet=ndet*Real(co(ji),t_real)
             det=det*xr
!             Print 12,i,xr
!             Print 13,i,xr/or(ji)
!             Print 14,i,det/ndet
             co(ji) = Cmplx(y,0.0_t_real,t_real)
!12           Format('bchd: xr   ',i5,'  =',f30.15)
!13           Format('bchd: ||xrn',i5,'||=',f30.15)
!14           Format('bchd: ||Sn ',i5,'||=',f30.15)
          Endif
       Enddo
    Enddo
!    Print *,'bchd: det=',det
!    Print *,'bchd: <det>=',Exp(Log(det)/n)
!    Print *,'bchd: ndet=',ndet
!    Print *,'bchd: <ndet>=',Exp(Log(ndet)/n)
!    Print *,'bchd: ||Sn||=',det/ndet
!    Print *,'bchd: <||Sn||>=',Exp(Log(det/ndet)/n)
    ierr = 0_t_int
  End Subroutine cbchd
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine bchd(or,oi,n,ierr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Integer(t_int),Intent(out)   :: ierr
    Real(t_real),Intent(inout)  :: or(n*n),oi(n*n)
    !!LOCAL
    Integer(t_int) :: i,j,k,im1,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: xr,xi,y,det,ndet
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          or(ji) = or(ij)
          oi(ji) = oi(ij)
       Enddo
    Enddo

    !! --- form l in the arrays o
    det=1.0_t_real
    ndet=1.0_t_real
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       y=1.0_t_real
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          xr =  or(ji)
          xi = -oi(ji)
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             !!             xr = xr - or(ki)*or(kj) - oi(ki)*oi(kj)
             !!             xi = xi - or(ki)*oi(kj) + oi(ki)*or(kj)
             xr = xr - or(ki)*or(kj) - oi(ki)*oi(kj)
             xi = xi + or(ki)*oi(kj) - oi(ki)*or(kj)
          Enddo
          If (j /= i) Then
             or(ji) = xr / y
             oi(ji) = xi / y
             or(ij) =  or(ji)
             oi(ij) = -oi(ji)
             !!             oi(ij) =  oi(ji)
          Else
             ierr = i
             If (xr <=  0.0_t_real) Return
             y = Sqrt(xr)
             ndet=ndet*or(ji)
             det=det*xr
!             Print 12,i,xr
!             Print 13,i,xr/or(ji)
!             Print 14,i,det/ndet
             or(ji) = y
             oi(ji) = 0.0_t_real
!12           Format('bchd: xr   ',i5,'  =',f30.15)
!13           Format('bchd: ||xrn',i5,'||=',f30.15)
!14           Format('bchd: ||Sn ',i5,'||=',f30.15)
          Endif
       Enddo
    Enddo
!    Print *,'bchd: det=',det
!    Print *,'bchd: <det>=',Exp(Log(det)/n)
!    Print *,'bchd: ndet=',ndet
!    Print *,'bchd: <ndet>=',Exp(Log(ndet)/n)
!    Print *,'bchd: ||Sn||=',det/ndet
!    Print *,'bchd: <||Sn||>=',Exp(Log(det/ndet)/n)
    ierr = 0_t_int
  End Subroutine bchd
#endif
  !!***********************************************************************
  !!- Reduces nonorthogonal hermetian matrix to orthogonal form
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system.
  !!io  ch     :hamiltonian matrix(only full upper triangle is used)
  !!i   co     :Cholesky-decomposed overlap matrix
  !!w Work arrays
  !!w   ct    :work array
  !!o Outputs:
  !!io  h     :reduced hermetian matrix:  a <-  ldag^-1 a l^-1
  !!o          The strict upper triangle
  !!r Remarks:
  !!r   Adapted from hred which is adapted from eispack reduc for
  !!r   hermetian matrices. bred is written for square matrices.
  !!r   At the beginning the strict upper triangle of ct is set to the
  !!r   complex conjugated of the strict lower one of h. 
  !!r   Then it is possible to
  !!r   change the indices in particular for the innermost loop so
  !!r   that the first array index equals the loop variable.
  !! ----------------------------------------------------------------------
  Subroutine cbred(n,ch,co,ct)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Complex(t_real),Intent(in)   :: co(n*n)
    Complex(t_real),Intent(inout):: ch(n*n)
    Complex(t_real),Intent(out)  :: ct(n*n)
    !!LOCAL
    Integer(t_int) :: i,ii,im1,j,jm1,k,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: y
    Complex(t_real):: cx
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          ij=i+jm1n
          ct(ij) =  Conjg(ch(ij))
       Enddo
    Enddo

    !! --- form the transpose of the upper triangle of inv(l)*h
    !! --- in the lower triangle of the array h
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       ii=i+im1n
       y = Real(co(ii),t_real)

       !!i ----  (l^-1 * h)_ij  =  (h_ij - sum_k<i  l_ik * (l^-1)_kj) / l_ii
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          cx = ch(ij)
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             cx = cx - Conjg(co(ki))*ct(kj)
          Enddo
          ch(ji) = cx / y
          ct(ij) = ch(ji)
       Enddo
    Enddo
    !! --- pre-multiply by inv(l) and overwrite
    Do j = 1_t_int, n
       jm1 = j - 1_t_int
       jm1n=jm1*n
       Do i = j, n
          im1 = i - 1_t_int
          im1n=im1*n
          ii=i+im1n
          ij=i+jm1n
          ji=j+im1n
          cx =  Conjg(ch(ij))
          Do k = j, im1
             ki=k+im1n
             kj=k+jm1n
             cx = cx -Conjg(co(ki))*ct(kj)
          Enddo
          Do k = 1_t_int, jm1
             ki=k+im1n
             kj=k+jm1n

             cx = cx -Conjg(co(ki)*ch(kj))
          Enddo
          ch(ji) = cx / Real(co(ii),t_real)
          ct(ij) = ch(ji)
       Enddo
    Enddo
  End Subroutine cbred
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine bred(n,hr,hi,or,oi,htr,hti)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)     :: or(n*n),oi(n*n)
    Real(t_real),Intent(inout)  :: hr(n*n),hi(n*n)
    Real(t_real),Intent(out)    :: htr(n*n),hti(n*n)
    !!LOCAL
    Integer(t_int) :: i,ii,im1,j,jm1,k,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: xr,xi,y
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          ij=i+jm1n
          htr(ij) =  hr(ij)
          hti(ij) = -hi(ij)
       Enddo
    Enddo

    !! --- form the transpose of the upper triangle of inv(l)*h
    !! --- in the lower triangle of the array h
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       ii=i+im1n
       y = or(ii)

       !!i ----  (l^-1 * h)_ij  =  (h_ij - sum_k<i  l_ik * (l^-1)_kj) / l_ii
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          xr = hr(ij)
          xi = hi(ij)
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             !!             xr = xr - or(ki)*htr(kj) + oi(ki)*hti(kj)
             !!             xi = xi - or(ki)*hti(kj) - oi(ki)*htr(kj)
             xr = xr - or(ki)*htr(kj) - oi(ki)*hti(kj)
             xi = xi - or(ki)*hti(kj) + oi(ki)*htr(kj)
          Enddo
          hr(ji) = xr / y
          hi(ji) = xi / y
          htr(ij) = hr(ji)
          hti(ij) = hi(ji)
       Enddo
    Enddo
    !! --- pre-multiply by inv(l) and overwrite
    Do j = 1_t_int, n
       jm1 = j - 1_t_int
       jm1n=jm1*n
       Do i = j, n
          im1 = i - 1_t_int
          im1n=im1*n
          ii=i+im1n
          ij=i+jm1n
          ji=j+im1n
          xr =  hr(ij)
          xi = -hi(ij)
          Do k = j, im1
             ki=k+im1n
             kj=k+jm1n
             !!             xr = xr - htr(kj)*or(ki) + hti(kj)*oi(ki)
             !!             xi = xi - htr(kj)*oi(ki) - hti(kj)*or(ki)
             xr = xr - htr(kj)*or(ki) - hti(kj)*oi(ki)
             xi = xi + htr(kj)*oi(ki) - hti(kj)*or(ki)
          Enddo
          Do k = 1_t_int, jm1
             ki=k+im1n
             kj=k+jm1n
             !!             xr = xr - hr(kj)*or(ki) - hi(kj)*oi(ki)
             !!             xi = xi - hr(kj)*oi(ki) + hi(kj)*or(ki)
             xr = xr - hr(kj)*or(ki) + hi(kj)*oi(ki)
             xi = xi + hr(kj)*oi(ki) + hi(kj)*or(ki)
          Enddo
          hr(ji) = xr / or(ii)
          hi(ji) = xi / or(ii)
          htr(ij) = hr(ji)
          hti(ij) = hi(ji)
       Enddo
    Enddo
  End Subroutine bred
#endif
  !!***********************************************************************
  !!- Reduces a complex hermitian matrix to a real symmetric tridiagonal
  !!- matrix using unitary similarity transformations
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of matrix
  !!io  ar    :real part of the complex hermitian input matrix
  !!io         only the upper triangle of the matrix need be supplied
  !!io  ai    :imaginary part of the complex hermitian input matrix
  !!io         only the upper triangle of the matrix need be supplied
  !!o Outputs:
  !!io  ar    :contains information about the unitary transformations used
  !!io         in the reduction (in strict upper triangle)
  !!io         the strict lower triangle and the diagonal unaltered
  !!io  ai    :contains information about the unitary transformations used
  !!io         in the reduction (in full upper triangle)
  !!io         the strict lower triangle is unaltered
  !!o   d     :contains the diagonal elements of the the tridiagonal matrix
  !!o   e     :contains the subdiagonal elements of the tridiagonal matrix
  !!o          in its last n-1 positions.  e(1) is set to zero.
  !!o   e2    :contains the squares of the corresponding elements of e
  !!r          e2 may coincide with e if the squares are not needed.
  !!o   tau   :contains further information about the transformations
  !!r Remarks:
  !!r     this subroutine is a translation of a complex analogue of
  !!r     the algol procedure tred1, num. math. 11, 181-195(1968)
  !!r     by martin, reinsch, and wilkinson.
  !!r     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
  !!r     this version is adapted from the eispack htridi. In contrast
  !!r     only the upper triangle of a is used for input and output
  !!r     questions and comments should be directed to burton s. garbow,
  !!r     mathematics and computer science div, argonne national laboratory
  !! ----------------------------------------------------------------------
  Subroutine cbtridi(n,ca,d,e,e2,tau)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Complex(t_real),Intent(inout):: ca(n,n)
    Real(t_real),Intent(out)    ::  tau(2,n),d(n),e(n),e2(n)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1,jp1
    Real(t_real)  :: f,g,h,hh,si,scale
    Complex(t_real):: cc,cg,cf
    !!
    tau(1_t_int,n) = 1.0_t_real
    tau(2_t_int,n) = 0.0_t_real
    !!
    Do i = 1_t_int, n
       d(i) = Real(ca(i,i),t_real)
    Enddo

    Do i = n,1_t_int,-1_t_int
       im1 = i - 1_t_int
       h = 0.0_t_real
       scale = 0.0_t_real
       If (im1 >= 1_t_int) Then
          !! -------- scale row
          Do k = 1_t_int, im1
             scale = scale + Abs(Real(ca(k,i),t_real)) + Abs(Aimag(ca(k,i)))
          Enddo
          If (scale /=  0.0_t_real) Goto 140
          tau(1,im1) = 1.0_t_real
          tau(2,im1) = 0.0_t_real
       Endif
       e (i) = 0.0_t_real
       e2(i) = 0.0_t_real
       Goto 290
       !!
140    Do k = 1_t_int, im1
          ca(k,i) = ca(k,i) / scale
          h = h + Abs(ca(k,i))**2
       Enddo

       e2(i) = scale * scale * h
       g = Sqrt(h)
       e(i) = scale * g
       f = Abs(ca(im1,i))
       !! ------- form next diagonal element of matrix t
       !If (Abs(f)>1.0e-100_t_real) Then
       If (f /= 0.0_t_real) Then
          cc=Conjg(ca(im1,i)*Cmplx(tau(1,i),tau(2,i),t_real))/f
          tau(1,im1) = -Real(cc,t_real) 
          si         = -Aimag(cc) 
          h = h + f * g
          g = 1.0_t_real + g / f
          ca(im1,i) = g * ca(im1,i)
          If (im1 == 1_t_int) Goto 270
       Else
          tau(1_t_int,im1) = -tau(1_t_int,i)
          si       =    tau(2_t_int,i)
          ca(im1,i) = Cmplx(g,Aimag(ca(im1,i)),t_real)
       Endif
       f = 0.0_t_real

       Do j = 1_t_int, im1
          !! --------- form element of a*u
          cg = Dot_product(ca(1:j,i),ca(1:j,j))
          !!
          jp1 = j + 1_t_int
          If (im1  >= jp1) Then
             Do  k = jp1, im1
                cg  = cg  + Conjg(ca(j,k)*ca(k,i))
             Enddo
          Endif
          !! --------- form element of p
          e(j)     = Real(cg,t_real)  / h
          tau(2,j) = Aimag(cg) / h
          f = f + e(j) * Real(ca(j,i),t_real) - tau(2,j) * Aimag(ca(j,i))
       Enddo

       hh = f / (h + h)
       !! ------ form reduced a
       Do  j = 1_t_int, im1

          cf=Conjg(ca(j,i))
          cg=Cmplx(e(j),tau(2,j),t_real) -hh*cf

          e(j) = Real(cg,t_real)
          tau(2,j) = -Aimag(cg)
          

          Do k = 1_t_int, j
             ca(k,j) = ca(k,j) - cf*Cmplx(e(k),tau(2,k),t_real) - cg*ca(k,i)
          Enddo
       Enddo

270    Continue
       Do k = 1_t_int, im1
          ca(k,i) = scale * ca(k,i)
       Enddo

       tau(2,im1) = -si
290    hh = d(i)
       d(i) = Real(ca(i,i),t_real)
       ca(i,i) = Cmplx(hh,scale * Sqrt(h),t_real)
    Enddo
  End Subroutine cbtridi
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine btridi(n,ar,ai,d,e,e2,tau)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(inout)  ::  ar(n,n),ai(n,n)
    Real(t_real),Intent(out)    ::  tau(2,n),d(n),e(n),e2(n)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1,jp1
    Real(t_real)  :: f,g,h,fi,gi,hh,si,scale
    !!
    tau(1_t_int,n) = 1.0_t_real
    tau(2_t_int,n) = 0.0_t_real
    !!
    Do i = 1_t_int, n
       d(i) = ar(i,i)
    Enddo

    Do i = n,1_t_int,-1_t_int
       im1 = i - 1_t_int
       h = 0.0_t_real
       scale = 0.0_t_real
       If (im1 >= 1_t_int) Then
          !! -------- scale row
          Do k = 1_t_int, im1
             scale = scale + Abs(ar(k,i)) + Abs(ai(k,i))
          Enddo
          If (scale /=  0.0_t_real) Goto 140
          tau(1_t_int,im1) = 1.0_t_real
          tau(2_t_int,im1) = 0.0_t_real
       Endif
       e (i) = 0.0_t_real
       e2(i) = 0.0_t_real
       Goto 290
       !!
140    Do k = 1_t_int, im1
          ar(k,i) = ar(k,i) / scale
          ai(k,i) = ai(k,i) / scale
          h = h + ar(k,i) * ar(k,i) + ai(k,i) * ai(k,i)
       Enddo

       e2(i) = scale * scale * h
       g = Sqrt(h)
       e(i) = scale * g
       f = Sqrt(ar(im1,i)**2_t_int+ai(im1,i)**2_t_int)
       !! ------- form next diagonal element of matrix t
       If (f /= 0.0_t_real) Then
          tau(1_t_int,im1) = (ai(im1,i)*tau(2_t_int,i) - ar(im1,i)*tau(1_t_int,i)) / f
          si         = (ar(im1,i)*tau(2_t_int,i) + ai(im1,i)*tau(1_t_int,i)) / f
          h = h + f * g
          g = 1.0_t_real + g / f
          ar(im1,i) = g * ar(im1,i)
          ai(im1,i) = g * ai(im1,i)
          If (im1 == 1_t_int) Goto 270
       Else
          tau(1_t_int,im1) = -tau(1_t_int,i)
          si       =    tau(2_t_int,i)
          ar(im1,i) = g
       Endif
       f = 0.0_t_real

       Do j = 1_t_int, im1
          g  = 0.0_t_real
          gi = 0.0_t_real
          !! --------- form element of a*u
          Do k = 1_t_int, j
             g  = g  + ar(k,j) * ar(k,i) + ai(k,j) * ai(k,i)
             gi = gi - ar(k,j) * ai(k,i) + ai(k,j) * ar(k,i)
          Enddo
          jp1 = j + 1_t_int
          If (im1  >= jp1) Then
             Do  k = jp1, im1
                g  = g  + ar(j,k) * ar(k,i) - ai(j,k) * ai(k,i)
                gi = gi - ar(j,k) * ai(k,i) - ai(j,k) * ar(k,i)
             Enddo
          Endif
          !! --------- form element of p
          e(j)     = g  / h
          tau(2_t_int,j) = gi / h
          f = f + e(j) * ar(j,i) - tau(2_t_int,j) * ai(j,i)
       Enddo

       hh = f / (h + h)
       !! ------ form reduced a
       Do  j = 1_t_int, im1
          f = ar(j,i)
          g = e(j) - hh * f
          e(j) = g
          fi = -ai(j,i)
          gi = tau(2_t_int,j) - hh * fi
          tau(2_t_int,j) = -gi
          Do k = 1_t_int, j
             ar(k,j) = ar(k,j) - f * e(k) - g * ar(k,i) &
                  & + fi * tau(2_t_int,k) + gi * ai(k,i)
             ai(k,j) = ai(k,j) - f * tau(2_t_int,k) - g * ai(k,i) &
                  & - fi * e(k) - gi * ar(k,i)
          Enddo
       Enddo

270    Continue
       Do k = 1_t_int, im1
          ar(k,i) = scale * ar(k,i)
          ai(k,i) = scale * ai(k,i)
       Enddo

       tau(2_t_int,im1) = -si
290    hh = d(i)
       d(i) = ar(i,i)
       ar(i,i) = hh
       ai(i,i) = scale * Sqrt(h)
    Enddo
  End Subroutine btridi
#endif
  !!***********************************************************************
  !!- Finds the eigenvalues of a symmetric tridiagonal matrix by the
  !!- implicit QL method
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of matrix
  !!i   d     :diagonal elements of matrix
  !!i   e     :subdiagonal elements of matrix in its last n-1 positions
  !!i   e2    :squares of the corresponding elements of e
  !!o Outputs:
  !!io  e2    :neglegible elements have been replaced by zero
  !!i   w     :eigenvalues in ascending order
  !!i   ind   :index to submatrix
  !!i   ierr  :nonzero (j) if the j-th eigenvalue has not been determined
  !!i          after 30 iterations
  !!i   rv1   :work file
  !!r Remarks:
  !!r    THIS ROUTINE IS A VARIANT OF  IMTQL1  WHICH IS A TRANSLATION OF
  !!r    ALGOL PROCEDURE IMTQL1, NUM. MATH. 12, 377-383(1968) BY MARTIN AND
  !!r    WILKINSON, AS MODIFIED IN NUM. MATH. 15, 450(1970) BY DUBRULLE.
  !!r    HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 241-248(1971).
  !!r    QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
  !!r    MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
  !!r    THIS VERSION DATED AUGUST 1983.
  !!r
  !!r    IND CONTAINS THE SUBMATRIX INDICES ASSOCIATED WITH THE
  !!r    CORRESPONDING EIGENVALUES IN W -- 1 FOR EIGENVALUES
  !!r    BELONGING TO THE FIRST SUBMATRIX FROM THE TOP,
  !!r    2 FOR THOSE BELONGING TO THE SECOND SUBMATRIX, ETC..
  !!r
  !!r    IF AN ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT AND
  !!r    ORDERED FOR INDICES 1,2,...IERR-1, BUT MAY NOT BE
  !!r    THE SMALLEST EIGENVALUES.
  !! ----------------------------------------------------------------------
  Subroutine imtqlv(n,d,e,e2,w,ind,ierr,rv1)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Integer(t_int),Intent(inout) :: ind(*)
    Integer(t_int),Intent(out)   :: ierr
    Real(t_real),Intent(in)     :: d(n),e(n)
    Real(t_real),Intent(out)    :: w(n),rv1(n),e2(n)
    !!LOCAL 
    Integer(t_int) :: i,j,k,l,m,ii,mml,tag
    Real(t_real)  :: b,c,f,g,p,r,s,tst1,tst2
    !!
    ierr = 0_t_int
    k = 0_t_int
    tag = 0_t_int
    !!
    Do i = 1_t_int, n
       w(i) = d(i)
       If (i /= 1_t_int) rv1(i-1_t_int) = e(i)
    Enddo
    !!
    e2(1_t_int) = 0.0_t_real
    rv1(n) = 0.0_t_real
    !!
    Do l = 1_t_int, n
       j = 0_t_int
       !!     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
105    Do m = l, n
          If (m == n) Goto 120
          tst1 = Abs(w(m)) + Abs(w(m+1_t_int))
          tst2 = tst1 + Abs(rv1(m))
          If (tst2 == tst1) Goto 120
          !! .......... GUARD AGAINST UNDERFLOWED ELEMENT OF E2 ..........
          If (e2(m+1_t_int) == 0.0_t_real) Goto 125
       Enddo
       !!
120    If (m <= k) Goto 130
       If (m /= n) e2(m+1_t_int) = 0.0_t_real
125    k = m
       tag = tag + 1_t_int
130    p = w(l)
       If (m == l) Goto 215
       If (j == 30_t_int) Goto 1000
       j = j + 1_t_int
       !!     .......... FORM SHIFT ..........
       g = (w(l+1_t_int) - p) / (2.0_t_real * rv1(l))
       r = Abs(Cmplx(g,1.0_t_real,t_real))
       g = w(m) - p + rv1(l) / (g + Sign(r,g))
       s = 1.0_t_real
       c = 1.0_t_real
       p = 0.0_t_real
       mml = m - l
       !!     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
       Do ii = 1_t_int, mml
          i = m - ii
          f = s * rv1(i)
          b = c * rv1(i)
          r = Abs(Cmplx(f,g,t_real))
          rv1(i+1_t_int) = r
          If (r == 0.0_t_real) Goto 210
          s = f / r
          c = g / r
          g = w(i+1_t_int) - p
          r = (w(i) - g) * s + 2.0_t_real * c * b
          p = s * r
          w(i+1_t_int) = g + p
          g = c * r - b

       Enddo
       !!
       w(l) = w(l) - p
       rv1(l) = g
       rv1(m) = 0.0_t_real
       Goto 105
       !!     .......... RECOVER FROM UNDERFLOW ..........
210    w(i+1_t_int) = w(i+1_t_int) - p
       rv1(m) = 0.0_t_real
       Goto 105
       !!     .......... ORDER EIGENVALUES ..........
215    If (l == 1_t_int) Goto 250
       !!     .......... FOR I=L STEP -1 UNTIL 2 DO -- ..........
       Do ii = 2_t_int, l
          i = l + 2_t_int - ii
          If (p >= w(i-1_t_int)) Goto 270
          w(i) = w(i-1_t_int)
          ind(i) = ind(i-1_t_int)
       Enddo
       !!
250    i = 1_t_int
270    w(i) = p
       ind(i) = tag
    Enddo
    !!
    Goto 1001
    !!     .......... SET ERROR -- NO CONVERGENCE TO AN
    !!                EIGENVALUE AFTER 30 ITERATIONS ..........
1000 ierr = l
1001 Return
  End Subroutine imtqlv
  !!***********************************************************************
  !!- Finds eigenvectors of a tridiagonal symmetric matrix
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of input matrix
  !!i   d     :diagonal elements of input matrix
  !!i   e     :subdiagonal elements of input matrix in position 2,...,n-1
  !!i   e2    :square of e
  !!i   m     :number of specified eigenvalues
  !!i   w     :eigenvalues
  !!i   ind   :index to submatrix
  !!w Work arrays:
  !!w   rv1   :work array
  !!w   rv2   :work array
  !!w   rv3   :work array
  !!w   rv4   :work array
  !!w   rv6   :work array
  !!o Outputs:
  !!o   z     :orthonormal eigenvector
  !!o   ierr  :0 for normal return
  !!r Remarks:
  !!r    AUTHORS -
  !!r       THIS ROUTINE IS A MODIFICATION OF EISPACK ROUTINE TINVIT
  !!r       WHICH IS A TRANSLATION OF THE INVERSE ITERATION TECHNIQUE
  !!r       IN THE ALGOL PROCEDURE TRISTURM BY PETERS AND WILKINSON.
  !!r       HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 418-439(1971).
  !!r       THE EISPACK VERSION WAS MODIFIED BY C. MOLER AND D. SPANGLER
  !!r       (NRCC - 1 APR 1979).  THIS VERSION IS BY S. T. ELBERT
  !!r       (AMES LABORATORY - USDOE), WHO FURTHER STREAMLINED THE CODE.
  !!r
  !!r    PURPOSE -
  !!r       THIS ROUTINE FINDS THOSE EIGENVECTORS OF A TRIDIAGONAL
  !!r       SYMMETRIC MATRIX CORRESPONDING TO SPECIFIED EIGENVALUES,
  !!r       USING INVERSE ITERATION.
  !!r
  !!r    METHOD -
  !!r       THE CALCULATIONS PROCEED AS FOLLOWS.  FIRST, THE  E2  ARRAY
  !!r       IS INSPECTED FOR THE PRESENCE OF A ZERO ELEMENT DEFINING A
  !!r       SUBMATRIX.  THE EIGENVALUES BELONGING TO THIS SUBMATRIX ARE
  !!r       IDENTIFIED BY THEIR COMMON SUBMATRIX INDEX IN  IND.
  !!r
  !!r       THE EIGENVECTORS OF THE SUBMATRIX ARE THEN COMPUTED BY
  !!r       INVERSE ITERATION.  FIRST, THE  LU  DECOMPOSITION OF THE SUB-
  !!r       MATRIX WITH AN APPROXIMATE EIGENVALUE SUBTRACTED FROM ITS
  !!r       DIAGONAL ELEMENTS IS ACHIEVED BY GAUSSIAN ELIMINATION USING
  !!r       PARTIAL PIVOTING.  THE MULTIPLIERS DEFINING THE LOWER
  !!r       TRIANGULAR MATRIX  L  ARE STORED IN THE TEMPORARY ARRAY  RV4
  !!r       AND THE UPPER TRIANGULAR MATRIX  U  IS STORED IN THE THREE
  !!r       TEMPORARY ARRAYS  RV1,  RV2, AND  RV3.  SAVING THESE
  !!r       QUANTITIES IN RV1,  RV2,  RV3, AND  RV4 AVOIDS REPEATING
  !!r       THE  LU  DECOMPOSITION IF FURTHER ITERATIONS ARE REQUIRED.
  !!r       AN APPROXIMATE VECTOR, STORED IN  RV6, IS COMPUTED STARTING
  !!r       FROM AN INITIAL VECTOR, AND THE NORM OF THE APPROXIMATE
  !!r       VECTOR IS COMPARED WITH A NORM OF THE SUBMATRIX TO DETERMINE
  !!r       WHETHER THE GROWTH IS SUFFICIENT TO ACCEPT IT AS AN
  !!r       EIGENVECTOR.  IF THIS VECTOR IS ACCEPTED, ITS EUCLIDIAN NORM
  !!r       IS MADE 1.  IF THE GROWTH IS NOT SUFFICIENT, THIS VECTOR IS
  !!r       USED AS THE INITIAL VECTOR IN COMPUTING THE NEXT APPROXIMATE
  !!r       VECTOR.  THIS ITERATION PROCESS IS REPEATED AT MOST  5  TIMES.
  !!r
  !!r       EIGENVECTORS COMPUTED IN THE ABOVE WAY CORRESPONDING TO WELL-
  !!r       SEPARATED EIGENVALUES OF THIS SUBMATRIX WILL BE ORTHOGONAL.
  !!r       HOWEVER, EIGENVECTORS CORRESPONDING TO CLOSE EIGENVALUES OF
  !!r       THIS SUBMATRIX MAY NOT BE SATISFACTORILY ORTHOGONAL.  HENCE,
  !!r       TO INSURE ORTHOGONAL EIGENVECTORS, EACH APPROXIMATE VECTOR IS
  !!r       MADE ORTHOGONAL TO THOSE PREVIOUSLY COMPUTED EIGENVECTORS
  !!r       WHOSE EIGENVALUES ARE CLOSE TO THE CURRENT EIGENVALUE, AS
  !!r       DETERMINED BY THE PARAMETER GRPTOL.
  !!r       IF THE ORTHOGONALIZATION PROCESS PRODUCES A ZERO VECTOR, A
  !!r       COLUMN OF THE IDENTITY MATRIX IS USED AS AN INITIAL VECTOR
  !!r       FOR THE NEXT ITERATION.
  !!r
  !!r       IDENTICAL EIGENVALUES ARE PERTURBED SLIGHTLY IN AN ATTEMPT TO
  !!r       OBTAIN INDEPENDENT EIGENVECTORS.  THESE PERTURBATIONS ARE NOT
  !!r       RECORDED IN THE EIGENVALUE ARRAY  W.
  !!r
  !!r       THE ABOVE STEPS ARE REPEATED ON EACH SUBMATRIX UNTIL ALL THE
  !!r       EIGENVECTORS ARE COMPUTED.
  !!r
  !!r    ON ENTRY-
  !!r
  !!r       N      - INTEGER
  !!r                THE ORDER OF THE MATRIX.
  !!r
  !!r       D      - W.P. REAL (N)
  !!r                THE DIAGONAL ELEMENTS OF THE INPUT MATRIX.
  !!r
  !!r       E      - W.P. REAL (N)
  !!r                CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
  !!r                IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
  !!r
  !!r       E2     - W.P. REAL (N)
  !!r          CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E,
  !!r          WITH ZEROS CORRESPONDING TO NEGLIGIBLE ELEMENTS OF E.
  !!r          E(I) IS CONSIDERED NEGLIGIBLE IF IT IS NOT LARGER THAN
  !!r          THE PRODUCT OF THE RELATIVE MACHINE PRECISION AND THE SUM
  !!r          OF THE MAGNITUDES OF D(I) AND D(I-1).  E2(1) MUST CONTAIN
  !!r          0.0 IF THE EIGENVALUES ARE IN ASCENDING ORDER, OR 2.0
  !!r          IF THE EIGENVALUES ARE IN DESCENDING ORDER.  IF  BISECT,
  !!r          TRIDIB, OR  EIMQLV  HAS BEEN USED TO FIND THE EIGENVALUES,
  !!r          THEIR OUTPUT E2 ARRAY IS EXACTLY WHAT IS EXPECTED HERE.
  !!r
  !!r       M      - INTEGER
  !!r                THE NUMBER OF SPECIFIED EIGENVALUES.
  !!r
  !!r       W      - W.P. REAL (M)
  !!r                THE M EIGENVALUES IN ASCENDING OR DESCENDING ORDER.
  !!r
  !!r       IND    - INTEGER (M)
  !!r           CONTAINS IN ITS FIRST M POSITIONS THE SUBMATRIX INDICES
  !!r           ASSOCIATED WITH THE CORRESPONDING EIGENVALUES IN W --
  !!r           1 FOR EIGENVALUES BELONGING TO THE FIRST SUBMATRIX FROM
  !!r           TOP, 2 FOR THOSE BELONGING TO THE SECOND SUBMATRIX, ETC.
  !!r
  !!r    ON EXIT-
  !!r
  !!r       Z      - W.P. REAL (NM,M)
  !!r          CONTAINS THE ASSOCIATED SET OF ORTHONORMAL EIGENVECTORS.
  !!r          ANY VECTOR WHICH FAILS TO CONVERGE IS SET TO ZERO.
  !!r
  !!r       IERR   - INTEGER
  !!r                SET TO
  !!r                ZERO   FOR NORMAL RETURN,
  !!r                -R     IF THE EIGENVECTOR CORRESPONDING TO THE R-TH
  !!r                       EIGENVALUE FAILS TO CONVERGE IN 5 ITERATIONS.
  !!r
  !!r       RV1, RV2, RV3, RV4, AND RV6  -  W.P. REAL (N)
  !!r                TEMPORARY STORAGE ARRAYS.
  !!r
  !!r    COMPLEXITY -
  !!r       O(N*M)
  !!r
  !!r    NOTE -
  !!r    QUESTIONS AND COMMENTS ABOUT EISPACK SHOULD BE DIRECTED TO
  !!r    B. S. GARBOW, APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LAB.
  !!r
  !!-----------------------------------------------------------------------

  Subroutine binvit(n,d,e,e2,m,w,ind,z,ierr,rv1,rv2,rv3,rv4,rv6)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: ind(*),n,m
    Integer(t_int),Intent(out)   :: ierr
    Real(t_real),Intent(in)     :: d(n),e(n),e2(n),w(m)
    Real(t_real),Intent(out)    :: z(n,m)
    Real(t_real),Intent(inout)  :: rv1(n),rv2(n),rv3(n),rv4(n),rv6(n)
    !!LOCAL
    Integer(t_int) :: i,igroup,ip,ipm1,ipp1,iq,iqm1,iqm2,iqp1,ir,is,itag,iters
    Integer(t_int) :: j,jj,nsub,nsubup
    Real(t_real)  :: anorm,anorm2,epmach,eps2,eps3,eps4,order
    Real(t_real)  :: test,u,uk,v,xu,rtpre,rtcur
    !!
    ierr = 0_t_int
    If (m  <=  0_t_int) Return
    !! --- EPMACH IS A MACHINE DEPENDENT PARAMETER SPECIFYING
    !! --- THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
    epmach = 0.5_t_real**20_t_int
    Do i=1_t_int,100_t_int
       epmach = epmach*0.5_t_real
       test = 1.0_t_real + epmach
       If(test == 1.0_t_real) Goto 120
    Enddo
120 Continue
    epmach=epmach+epmach
    epmach=epmach+epmach

    itag = 0_t_int
    order = 1.0_t_real - e2(1_t_int)
    ip = 1_t_int
    Do iq=1_t_int,n
       !! this a bugfix for the case when the compiler does not know
       !! to stop boolean evaluation after the result is definite
       !! in this case the e2(...) statement produces a segfault
        If ( iq /= n) Then
           If(  e2(iq+1_t_int)  /=  0.0_t_real ) Cycle
        Endif
       !! end bugfix
       !skip: If (iq == n .Or. e2(iq+1_t_int)  ==  0.0_t_real) Then
          !! -- ESTABLISH AND PROCESS NEXT SUBMATRIX WITH INDEX RANGE IP TO IQ
          itag = itag + 1_t_int
          is = 0_t_int
          eps2=1.0_t_int
          eps3=1.0_t_int
          eps4=1.0_t_int
          uk=1.0_t_int
          igroup = 0_t_int
          rtcur=w(1_t_int)
          Do ir = 1_t_int, m
             If (ind(ir)  ==  itag) Then
                rtpre = rtcur
                rtcur = w(ir)
                ipp1 = ip + 1_t_int
                ipm1 = ip - 1_t_int
                nsub = iq - ipm1
                If (is  ==  0_t_int) Then
                   !! ----------- CHECK FOR ISOLATED ROOT
                   xu = 1.0_t_real
                   If (ip  ==  iq) Then
                      !! ------------- SET ISOLATED ROOT TO UNIT VECTOR
                      Do i=1_t_int,n
                         z(i,ir)=0.0_t_real
                      Enddo
                      z(ip,ir)=1.0_t_real
                      Goto 920
                   Endif
                   anorm = Abs(d(ip))
                   Do i = ipp1, iq
                      anorm = anorm + Abs(d(i)) + Abs(e(i))
                   Enddo
                   !! ----------- EPS2 IS THE CRITERION FOR GROUPING,
                   !! ----------- EPS3 REPLACES ZERO PIVOTS AND EQUAL ROOTS ARE
                   !! ----------- MODIFIED BY EPS3
                   !! ----------- EPS4 IS TAKEN VERY SMALL TO AVOID OVERFLOW
                   uk = Real(nsub,t_real)
                   anorm2 = anorm / uk
                   eps2 = 1.e-5_t_real * anorm2
                   eps3 = epmach * anorm
                   eps4 = uk * eps3
                   uk = eps4 / Sqrt(uk)
                   is = ip
                   igroup = 0_t_int
                   !! ----------- LOOK FOR CLOSE OR COINCIDENT ROOTS
                Else
                   igroup = igroup + 1_t_int
                   If (Abs(rtcur-rtpre)  >=  eps2) igroup =0_t_int
                   If (order * (rtcur - rtpre)  <=  0.0_t_real) &
                        & rtcur = rtpre + order * eps3
                   !!------ ELIMINATION WITH INTERCHANGES AND INITIALIZATION OF
                   !!------ VECTOR
                Endif
                v = 0.0_t_real
                iqm1 = iq - 1_t_int
                rv6(ip) = uk
                u = d(ip) - rtcur
                If(ip  /= iq) Then
                   v = e(ipp1)
                   If (ipp1  <=  iqm1) Then
                      Do i = ipp1,iqm1
                         rv6(i) = uk
                         If (Abs(e(i))  >=  Abs(u)) Then
                            !! WARNING -- A DIVIDE CHECK MAY OCCUR HERE
                            !! IF E2 ARRAY HAS NOT BEEN SPECIFIED CORRECTLY
                            xu = u / e(i)
                            rv4(i) = xu
                            rv1(i-1_t_int) = e(i)
                            rv2(i-1_t_int) = d(i) - rtcur
                            rv3(i-1_t_int) = e(i+1_t_int)
                            u = v - xu * rv2(i-1_t_int)
                            v = -xu * rv3(i-1_t_int)
                         Else
                            xu = e(i) / u
                            rv4(i) = xu
                            rv1(i-1_t_int) = u
                            rv2(i-1_t_int) = v
                            rv3(i-1_t_int) = 0.0_t_real
                            u = d(i) - rtcur - xu * v
                            v = e(i+1_t_int)
                         Endif
                      Enddo
                   Endif
                   rv6(iq) = uk
                   rv3(iqm1) = 0.0_t_real
                   If (Abs(e(iq))  >=  Abs(u)) Then
                      xu = u / e(iq)
                      rv4(iq) = xu
                      rv1(iqm1) = e(iq)
                      rv2(iqm1) = d(iq) - rtcur
                      u = v - xu * rv2(iqm1)
                      v = 0.0_t_real
                   Else
                      xu = e(iq) / u
                      rv4(iq) = xu
                      rv1(iqm1) = u
                      rv2(iqm1) = v
                      u = d(iq) - rtcur - xu * v
                   Endif
                Endif
                If (Abs(u)  <  eps3) u = eps3
                rv1(iq) = u
                rv2(iq) = 0.0_t_real
                rv3(iq) = 0.0_t_real
                !! --------- BACK SUBSTITUTION FOR I=IQ STEP -1 UNTIL IP DO
                Do iters=1_t_int,5_t_int
                   rv6(iq) = rv6(iq)/rv1(iq)
                   u = rv6(iq)
                   rv6(iqm1) = (rv6(iqm1) - u * rv2(iqm1)) / rv1(iqm1)
                   v = u
                   u = rv6(iqm1)
                   anorm = Abs(u) + Abs(v)
                   If (ipp1  <=  iqm1) Then
                      iqm2=iqm1-1_t_int
                      Do i = iqm2,ip,-1_t_int
                         rv6(i) = (rv6(i) - u*rv2(i) - v*rv3(i)) / rv1(i)
                         v = u
                         u = rv6(i)
                         anorm = anorm + Abs(u)
                      Enddo
                   Endif
                   If (igroup  /= 0_t_int) Then
                      !! ORTHOGONALIZE WITH RESPECT TO PREVIOUS MEMBERS OF 
                      !! GROUP
                      j = ir
                      Do jj = 1_t_int,igroup
630                      j = j - 1_t_int
                         If (ind(j)  /= itag) Goto 630
                         xu = 0.0_t_real
                         nsubup = nsub+ip-1_t_int
                         Do i = ip, nsubup
                            xu = xu + rv6(i)*z(i,j)
                         Enddo
                         xu = -xu
                         If (xu  /= 0.0_t_real) Then
                            Do i = ip, nsubup
                               rv6(i) = rv6(i) + xu*z(i,j)
                            Enddo
                         Endif
                      Enddo
                      anorm = 0.0_t_real
                      Do i = ip,iq
                         anorm = anorm + Abs(rv6(i))
                      Enddo
                   Endif
                   If (anorm  <  1.0_t_real) Then
                      !! ------------- FORWARD SUBSTITUTION
                      If (anorm  ==  0.0_t_real) Then
                         rv6(is) = eps4
                         is = is + 1_t_int
                         If (is > iq) is = ip
                      Else
                         xu = eps4 / anorm
                         Do i =ip,iq
                            rv6(i) = rv6(i) * xu
                         Enddo
                      Endif
                      !!  ELIMINATION OPERATIONS ON NEXT VECTOR ITERATE
                      Do i = ipp1,iq
                         u = rv6(i)
                         !! -- IF RV1(I-1)  ==  E(I), A ROW INTERCHANGE
                         !! -- WAS PERFORMED EARLIER IN THE
                         !! -- TRIANGULARIZATION PROCESS
                         If (rv1(i-1_t_int)  ==  e(i)) Then
                            u = rv6(i-1_t_int)
                            rv6(i-1_t_int) = rv6(i)
                         Endif
                         rv6(i) = u - rv4(i) * rv6(i-1_t_int)
                      Enddo
                   Else
                      Goto 840
                   Endif
                Enddo
                !! --------- SET ERROR -- NON-CONVERGED EIGENVECTOR
                ierr = -ir
                Do i=1_t_int,n
                   z(i,ir)=0.0_t_real
                Enddo
                Goto 920
                !! -------NORMALIZE SO THAT SUM OF SQUARES IS 1 AND EXPAND 
                !!TO FULL ORDER
840             u = 0.0_t_real
                Do i = ip,iq
                   u = u + rv6(i)*rv6(i)
                Enddo
                xu = 1.0_t_real / Sqrt(u)
                If(ipm1 >= 1_t_int) Then
                   Do i = 1_t_int, ipm1
                      z(i,ir) = 0.0_t_real
                   Enddo
                Endif
                Do i = ip,iq
                   z(i,ir) = rv6(i) * xu
                Enddo
                iqp1 = iq + 1_t_int
                If(iqp1  <=  n) Then
                   Do i=iqp1,n
                      z(i,ir) = 0.0_t_real
                   Enddo
                Endif
             Endif
920          Continue
          Enddo
          ip = iq + 1_t_int
       !Endif skip
    Enddo
    Return
  End Subroutine binvit
  !!***********************************************************************
  !!
  !!     this subroutine is a translation of the algol procedure imtql2,
  !!     num. math. 12, 377-383(1968) by martin and wilkinson,
  !!     as modified in num. math. 15, 450(1970) by dubrulle.
  !!     handbook for auto. comp., vol.ii-linear algebra, 241-248(1971).
  !!
  !!     this subroutine finds the eigenvalues and eigenvectors
  !!     of a symmetric tridiagonal matrix by the implicit ql method.
  !!     the eigenvectors of a full symmetric matrix can also
  !!     be found if  tred2  has been used to reduce this
  !!     full matrix to tridiagonal form.
  !!
  !!     on input
  !!
  !!        nm must be set to the row dimension of two-dimensional
  !!          array parameters as declared in the calling program
  !!          dimension statement.
  !!
  !!        n is the order of the matrix.
  !!
  !!        d contains the diagonal elements of the input matrix.
  !!
  !!        e contains the subdiagonal elements of the input matrix
  !!          in its last n-1 positions.  e(1) is arbitrary.
  !!
  !!        z contains the transformation matrix produced in the
  !!          reduction by  tred2, if performed.  if the eigenvectors
  !!          of the tridiagonal matrix are desired, z must contain
  !!          the identity matrix.
  !!
  !!      on output
  !!
  !!        d contains the eigenvalues in ascending order.  if an
  !!          error exit is made, the eigenvalues are correct but
  !!          unordered for indices 1,2,...,ierr-1.
  !!
  !!        e has been destroyed.
  !!
  !!        z contains orthonormal eigenvectors of the symmetric
  !!          tridiagonal (or full) matrix.  if an error exit is made,
  !!          z contains the eigenvectors associated with the stored
  !!          eigenvalues.
  !!
  !!        ierr is set to
  !!          zero       for normal return,
  !!          j          if the j-th eigenvalue has not been
  !!                     determined after 30 iterations.
  !!
  !!     calls dcabs2 for  dsqrt(a*a + b*b) .
  !!
  !!     questions and comments should be directed to burton s. garbow,
  !!     mathematics and computer science div, argonne national laboratory
  !!
  !!     this version dated august 1983.
  !!
  !!     -----------------------------------------------------------------
  Subroutine imtql2(nm,n,d,e,z,ierr)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(out)   :: ierr
    Integer(t_int),Intent(in)    :: n,nm
    Real(t_real),Intent(inout)  :: e(n),d(n),z(nm,n)
    !!LOCAL
    Integer(t_int) :: i,j,k,l,m,ii,mml
    Real(t_real)  :: b,c,f,g,p,r,s,tst1,tst2
    !!
    ierr = 0_t_int
    If (n == 1_t_int) go to 1001
    !!
    Do i = 2_t_int, n
       e(i-1_t_int) = e(i)
    Enddo
    !!
    e(n) = 0.0_t_real
    !!
    Do l = 1_t_int, n
       !Print *,' l=',l
       j = 0_t_int
       !!     .......... look for small sub-diagonal element ..........
105    Do m = l, n
          If (m  ==  n) go to 120
          tst1 = Abs(d(m)) + Abs(d(m+1_t_int))
          tst2 = tst1 + Abs(e(m))
          If (tst2  ==  tst1) go to 120
       Enddo
       !!
120    p = d(l)
       If (m  ==  l) go to 240
       If (j  ==  30_t_int) go to 1000
       j = j + 1_t_int
       !!     .......... form shift ..........
       g = (d(l+1_t_int) - p) / (2.0_t_real * e(l))
       r = Abs(Cmplx(g,1.0_t_real,t_real))
       g = d(m) - p + e(l) / (g + Sign(r,g))
       s = 1.0_t_real
       c = 1.0_t_real
       p = 0.0_t_real
       mml = m - l
       !!     .......... for i=m-1 step -1 until l do -- ..........
       Do ii = 1_t_int, mml
          i = m - ii
          f = s * e(i)
          b = c * e(i)
          r = Abs(Cmplx(f,g,t_real))
          e(i+1_t_int) = r
          !Print *,' e=',j,r
          If (r  ==  0.0_t_real) go to 210
          s = f / r
          c = g / r
          g = d(i+1_t_int) - p
          r = (d(i) - g) * s + 2.0_t_real * c * b
          p = s * r
          d(i+1_t_int) = g + p
          g = c * r - b
          !!     .......... form vector ..........
          Do k = 1_t_int, n
             f = z(k,i+1_t_int)
             z(k,i+1_t_int) = s * z(k,i) + c * f
             z(k,i) = c * z(k,i) - s * f

          Enddo
          !!
       Enddo
       !!
       d(l) = d(l) - p
       e(l) = g
       !Print *,' e=',j,g
       e(m) = 0.0_t_real
       go to 105
       !!     .......... recover from underflow ..........
210    d(i+1_t_int) = d(i+1_t_int) - p
       e(m) = 0.0_t_real
       go to 105
240    Continue
    Enddo
    !!     .......... order eigenvalues and eigenvectors ..........
    Do ii = 2_t_int, n
       i = ii - 1_t_int
       k = i
       p = d(i)
       !!
       Do j = ii, n
          If (d(j) >= p) go to 260
          k = j
          p = d(j)
260       Continue
       Enddo
       !!
       If (k  ==  i) go to 300
       d(k) = d(i)
       d(i) = p
       !!
       Do j = 1_t_int, n
          p = z(j,i)
          z(j,i) = z(j,k)
          z(j,k) = p
       Enddo
       !!
300    Continue
    Enddo
    !!
    go to 1001
    !!     .......... set error -- no convergence to an
    !!                eigenvalue after 30 iterations ..........
1000 ierr = l
1001 Return
  End Subroutine imtql2
  !!***********************************************************************
  !!- Get eigenvectors of a complex hermitian matrix by back transforming
  !!- those of the corresponding real symmetric tridiagonal matrix
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix
  !!i   ar    :contains information about the unitary transformations used
  !!i          in the reduction by btridi (in strict upper triangle)
  !!i   ai    :contains information about the unitary transformations used
  !!i          in the reduction by btridi (in full upper triangle)
  !!i   tau   :contains further information about the transformations
  !!i         (output from btridi)
  !!i   m     :number of eigenvectors to be back transformed
  !!i   zr    :contains the eigenvectors to be back transformed
  !!o Outputs:
  !!o   zr    :contain the real parts of the transformed eigenvectors
  !!o   zi    :contain the imaginary parts of the transformed eigenvectors
  !!r Remarks:
  !!r    this subroutine is a translation of a complex analogue of
  !!r    the algol procedure trbak1, num. math. 11, 181-195(1968)
  !!r    by martin, reinsch, and wilkinson.
  !!r    handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
  !!r
  !!r    this subroutine forms the eigenvectors of a complex hermitian
  !!r    matrix by back transforming those of the corresponding
  !!r    real symmetric tridiagonal matrix determined by  btridi.
  !!r
  !!r    note that the last component of each returned vector
  !!r    is real and that vector euclidean norms are preserved.
  !!r
  !!r    questions and comments should be directed to burton s. garbow,
  !!r    mathematics and computer science div, argonne national laboratory
  !!r
  !!r    this version is adapted from the eispack htridi. In contrast
  !!r    only the upper triangle of a is used for input
  !! ----------------------------------------------------------------------
  Subroutine cbtribk(n,ca,tau,m,cz,wz)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: m,n
    Real(t_real),Intent(in)      :: tau(2,n)
    Complex(t_real),Intent(in)   :: ca(n*n) 
    Real(t_real),Intent(in)      :: wz(n*m)
    Complex(t_real),Intent(out)  :: cz(n*m)
    !!LOCAL
    Integer(t_int) :: i,j,jm1,k,l,im1n,jm1n,ki,kj
    Real(t_real)  :: h
    Complex(t_real):: cs
    !!
    If (m  ==  0_t_int) Return
    !! --- transform the eigenvectors of the real symmetric
    !! --- tridiagonal matrix to those of the hermitian tridiagonal matrix.

    Do j = 1_t_int, m
       jm1=j-1_t_int
       Do k = 1_t_int, n
          kj=k+jm1*n
          cz(kj) =  wz(kj) * Cmplx(tau(1_t_int,k),-tau(2_t_int,k),t_real)
       Enddo
    Enddo

    If (n  ==  1_t_int) Return
    !! --- recover and apply the householder matrices
    Do i = 2_t_int, n
       im1n=(i-1_t_int)*n
       l = i - 1_t_int
       h = Aimag(ca(i+im1n))
       If (h  /=  0.0_t_real) Then
          Do j = 1_t_int, m
             jm1n=(j-1_t_int)*n
             !!ifdefc BLAS
             !!             s  = ddot(l,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int) 
             !!     .          - ddot(l,ai(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!             si = ddot(l,ar(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!     .          + ddot(l,ai(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!else
             cs  = Cmplx(0.0_t_real,0.0_t_real,t_real)
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                cs =cs  +ca(ki)*cz(kj)
             Enddo
             !!endif

             !! --------- double divisions avoid possible underflow
             cs= (cs / h) / h
             
             !!ifdefc BLAS
             !!            call daxpy(l,smin,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,si  ,ai(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,si  ,ar(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,s   ,ai(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!else
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                cz(kj)=cz(kj) - cs *Conjg(ca(ki))
             Enddo
             !!endif
          Enddo
       Endif
    Enddo
  End Subroutine cbtribk
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine btribk(n,ar,ai,tau,m,zr,zi)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: m,n
    Real(t_real),Intent(in)     :: ar(n*n),ai(n*n),tau(2,n)
    Real(t_real),Intent(inout)  :: zr(n*m),zi(n*m)
    !!LOCAL
    Integer(t_int) :: i,j,jm1,k,l,im1n,jm1n,ki,kj
    Real(t_real)  :: h,s,si,smin
    !!
    If (m  ==  0_t_int) Return
    !! --- transform the eigenvectors of the real symmetric
    !! --- tridiagonal matrix to those of the hermitian tridiagonal matrix.

    Do j = 1_t_int, m
       jm1=j-1_t_int
       Do k = 1_t_int, n
          kj=k+jm1*n
          zi(kj) = -zr(kj) * tau(2_t_int,k)
          zr(kj) =  zr(kj) * tau(1_t_int,k)
       Enddo
    Enddo

    If (n  ==  1_t_int) Return
    !! --- recover and apply the householder matrices
    Do i = 2_t_int, n
       im1n=(i-1_t_int)*n
       l = i - 1_t_int
       h = ai(i+im1n)
       If (h  /=  0.0_t_real) Then
          Do j = 1_t_int, m
             jm1n=(j-1_t_int)*n
             !!ifdefc BLAS
             !!             s  = ddot(l,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int) 
             !!     .          - ddot(l,ai(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!             si = ddot(l,ar(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!     .          + ddot(l,ai(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!else
             s  = 0.0_t_real
             si = 0.0_t_real
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                s =  s  + ar(ki) * zr(kj) - ai(ki) * zi(kj)
                si = si + ar(ki) * zi(kj) + ai(ki) * zr(kj)
             Enddo
             !!endif

             !! --------- double divisions avoid possible underflow
             s = (s / h) / h
             smin = -s
             si = - (si / h) / h

             !!ifdefc BLAS
             !!            call daxpy(l,smin,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,si  ,ai(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,si  ,ar(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!            call daxpy(l,s   ,ai(im1n+1_t_int),1_t_int,zi(jm1n+1_t_int),1_t_int)
             !!else
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                zr(kj) = zr(kj) + smin * ar(ki) + si * ai(ki)
                zi(kj) = zi(kj) + si   * ar(ki) + s  * ai(ki)
             Enddo
             !!endif
          Enddo
       Endif
    Enddo
  End Subroutine btribk
#endif
  !!***********************************************************************
  !!- Back-transforms eigenvectors to nonorthogonal representation
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system
  !!i   o     :Cholesky-decomposed overlap matrix
  !!io  z     :eigenvectors to be back transformed
  !!o Outputs:
  !!io  z     :transformed eigenvectors
  !!r Remarks:
  !!r   Adapted from rebakh, eispack
  !! ----------------------------------------------------------------------
  Subroutine cbbak(n,co,cz)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Complex(t_real),Intent(in)   :: co(*)
    Complex(t_real),Intent(inout):: cz(*)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1n,jm1n,ij,ii,ki,kj
    Complex(t_real):: cx
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = n, 1_t_int, -1_t_int
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ii=i+im1n
          cx = cz(ij)
          Do k = i+1_t_int, n
             ki=k+im1n
             kj=k+jm1n
             cx = cx -Conjg(co(ki))*cz(kj)

          Enddo
          cz(ij) = cx/Real(co(ii),t_real)
       Enddo
    Enddo
  End Subroutine cbbak
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine bbak(n,or,oi,zr,zi)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)     :: or(*),oi(*)
    Real(t_real),Intent(inout)  :: zr(*),zi(*)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1n,jm1n,ij,ii,ki,kj
    Real(t_real)  :: xr,xi
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = n, 1_t_int, -1_t_int
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ii=i+im1n
          xr = zr(ij)
          xi = zi(ij)
          Do k = i+1_t_int, n
             ki=k+im1n
             kj=k+jm1n
             xr = xr - or(ki)*zr(kj) - oi(ki)*zi(kj)
             xi = xi - or(ki)*zi(kj) + oi(ki)*zr(kj)
          Enddo
          zr(ij) = xr/or(ii)
          zi(ij) = xi/or(ii)
       Enddo
    Enddo
  End Subroutine bbak
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine readtestwritematc2(n1,n2,aa,lunin,lunout)
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: n1,n2,lunin,lunout
    Real(t_real),Intent(in)     :: aa(n1,n2,2)
    !!LOCAL
    Integer(t_int) :: count,i1,i2
    Real(t_real)  :: dum
    !!
    Open(lunin)
    Open(lunout)
    count=1_t_int
    Do i1=1_t_int,n1
       Do i2=1_t_int,n2
          count=count+1_t_int
          Read(lunin,"(15x,e32.17e3)")dum
          Write(lunout,"(i15,2(e32.17e3,2x))")count&
               &,(aa(i1,i2,1_t_int))-(dum),Abs(aa(i1,i2,1_t_int))-Abs(dum)
          count=count+1_t_int
          Read(lunin,"(15x,e32.17e3)")dum
          Write(lunout,"(i15,2(e32.17e3,2x))")count&
               &,(aa(i1,i2,2_t_int))-(dum),Abs(aa(i1,i2,2_t_int))-Abs(dum)
       Enddo
    Enddo
    Close(lunin)
    Close(lunout)
  End Subroutine readtestwritematc2
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine readtestwritemat2(n1,n2,aa,lunin,lunout)
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: n1,n2,lunin,lunout
    Real(t_real),Intent(in)     :: aa(n1,n2)
    !!LOCAL
    Integer(t_int) :: count,i1,i2
    Real(t_real)  :: dum
    !!
    Open(lunin)
    Open(lunout)
    count=1_t_int
    Do i1=1_t_int,n1
       Do i2=1_t_int,n2
          count=count+1_t_int
          Read(lunin,"(15x,e32.17e3)")dum
          Write(lunout,"(i15,2(e32.17e3,2x))")count&
               &,(aa(i1,i2))-(dum),Abs(aa(i1,i2))-Abs(dum)
       Enddo
    Enddo
    Close(lunin)
    Close(lunout)
  End Subroutine readtestwritemat2
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine readtestwritemat1(n1,aa,lunin,lunout)
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: n1,lunin,lunout
    Real(t_real),Intent(in)     :: aa(n1)
    !!LOCAL
    Integer(t_int) :: count,i1
    Real(t_real)  :: dum
    !!
    Open(lunin)
    Open(lunout)
    count=1_t_int
    Do i1=1_t_int,n1
       count=count+1_t_int
       Read(lunin,"(15x,e32.17e3)")dum
       Write(lunout,"(i15,2(e32.17e3,2x))")count&
            &,(aa(i1))-(dum),Abs(aa(i1))-Abs(dum)
    Enddo
    Close(lunin)
    Close(lunout)
  End Subroutine readtestwritemat1
#endif
  !!***********************************************************************
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine readc2(n1,n2,aa,lunin)
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: n1,n2,lunin
    Real(t_real),Intent(out)     :: aa(n1,n2,2)
    !!LOCAL
    Integer(t_int) :: count,i1,i2
    Real(t_real)  :: dum
    !!
    Open(lunin)
    count=1_t_int
    Do i1=1_t_int,n1
       Do i2=1_t_int,n2
          count=count+1_t_int
          Read(lunin,"(15x,e32.17e3)")dum
          aa(i1,i2,1_t_int)=dum
          count=count+1_t_int
          Read(lunin,"(15x,e32.17e3)")dum
          aa(i1,i2,2_t_int)=dum
       Enddo
    Enddo
    Close(lunin)
  End Subroutine readc2
#endif
  !!***********************************************************************
  !!- Reduces a real symmetric matrix to a real symmetric tridiagonal
  !!- matrix using unitary similarity transformations
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of matrix
  !!io  ar    :real symmetric input matrix
  !!io         only the upper triangle of the matrix need be supplied
  !!o Outputs:
  !!io  ar    :contains information about the unitary transformations used
  !!io         in the reduction (in strict upper triangle)
  !!io         the strict lower triangle and the diagonal unaltered
  !!o   d     :contains the diagonal elements of the the tridiagonal matrix
  !!o   e     :contains the subdiagonal elements of the tridiagonal matrix
  !!o          in its last n-1 positions.  e(1) is set to zero.
  !!o   e2    :contains the squares of the corresponding elements of e
  !!r          e2 may coincide with e if the squares are not needed.
  !!o   tau   :contains further information about the transformations
  !!r Remarks:
  !!r     this subroutine is a translation of a complex analogue of
  !!r     the algol procedure tred1, num. math. 11, 181-195(1968)
  !!r     by martin, reinsch, and wilkinson.
  !!r     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
  !!r     this version is adapted from the eispack htridi. In contrast
  !!r     only the upper triangle of a is used for input and output
  !!r     questions and comments should be directed to burton s. garbow,
  !!r     mathematics and computer science div, argonne national laboratory
  !! ----------------------------------------------------------------------
  Subroutine rbtridi(n,ar,d,e,e2,tau)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(inout)  ::  ar(n,n)
    Real(t_real),Intent(out)    ::  tau(2,n),d(n),e(n),e2(n)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1,jp1
    Real(t_real)  :: f,g,h,hh,scale
    !!
    tau(1_t_int,n) = 1.0_t_real
    !!
    Do i = 1_t_int, n
       d(i) = ar(i,i)
    Enddo

    Do i = n,1_t_int,-1_t_int
       im1 = i - 1_t_int
       h = 0.0_t_real
       scale = 0.0_t_real
       If (im1 >= 1_t_int) Then
          !! -------- scale row
          Do k = 1_t_int, im1
             scale = scale + Abs(ar(k,i)) 
          Enddo
          If (scale /=  0.0_t_real) Goto 140
          tau(1_t_int,im1) = 1.0_t_real
       Endif
       e (i) = 0.0_t_real
       e2(i) = 0.0_t_real
       Goto 290
       !!
140    Do k = 1_t_int, im1
          ar(k,i) = ar(k,i) / scale
          h = h + ar(k,i) * ar(k,i) 
       Enddo

       e2(i) = scale * scale * h
       g = Sqrt(h)
       e(i) = scale * g
       f = Sqrt(ar(im1,i)**2_t_int)
       !! ------- form next diagonal element of matrix t
       If (f /= 0.0_t_real) Then
          tau(1_t_int,im1) = ( - ar(im1,i)*tau(1_t_int,i)) / f

          h = h + f * g
          g = 1.0_t_real + g / f
          ar(im1,i) = g * ar(im1,i)

          If (im1 == 1_t_int) Goto 270
       Else
          tau(1_t_int,im1) = -tau(1_t_int,i)
          ar(im1,i) = g
       Endif
       f = 0.0_t_real

       Do j = 1_t_int, im1
          g  = 0.0_t_real
          
          !! --------- form element of a*u
          Do k = 1_t_int, j
             g  = g  + ar(k,j) * ar(k,i) 

          Enddo
          jp1 = j + 1_t_int
          If (im1  >= jp1) Then
             Do  k = jp1, im1
                g  = g  + ar(j,k) * ar(k,i) 
             Enddo
          Endif
          !! --------- form element of p
          e(j)     = g  / h
          f = f + e(j) * ar(j,i) 
       Enddo

       hh = f / (h + h)
       !! ------ form reduced a
       Do  j = 1_t_int, im1
          f = ar(j,i)
          g = e(j) - hh * f
          e(j) = g

          Do k = 1_t_int, j
             ar(k,j) = ar(k,j) - f * e(k) - g * ar(k,i) 
          Enddo
       Enddo

270    Continue
       Do k = 1_t_int, im1
          ar(k,i) = scale * ar(k,i)
       Enddo

290    hh = d(i)
       d(i) = ar(i,i)
       ar(i,i) = hh
       tau(2_t_int,i) = scale * Sqrt(h)

    Enddo
  End Subroutine rbtridi
  !!***********************************************************************
  !!- Get eigenvectors of a real symmetric matrix by back transforming
  !!- those of the corresponding real symmetric tridiagonal matrix
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix
  !!i   ar    :contains information about the unitary transformations used
  !!i          in the reduction by rbtridi (in strict upper triangle)
  !!i   ai    :contains information about the unitary transformations used
  !!i          in the reduction by rbtridi (in full upper triangle)
  !!i   tau   :contains further information about the transformations
  !!i         (output from rbtridi)
  !!i   m     :number of eigenvectors to be back transformed
  !!i   zr    :contains the eigenvectors to be back transformed
  !!o Outputs:
  !!o   zr    :contain  the transformed eigenvectors
  !!r Remarks:
  !!r    this subroutine is a translation of a complex analogue of
  !!r    the algol procedure trbak1, num. math. 11, 181-195(1968)
  !!r    by martin, reinsch, and wilkinson.
  !!r    handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
  !!r
  !!r    this subroutine forms the eigenvectors of a real symmetric
  !!r    matrix by back transforming those of the corresponding
  !!r    real symmetric tridiagonal matrix determined by  rbtridi.
  !!r
  !!r    note that the last component of each returned vector
  !!r    is real and that vector euclidean norms are preserved.
  !!r
  !!r    questions and comments should be directed to burton s. garbow,
  !!r    mathematics and computer science div, argonne national laboratory
  !!r
  !!r    this version is adapted from the eispack htridi. In contrast
  !!r    only the upper triangle of a is used for input
  !! ----------------------------------------------------------------------
  Subroutine rbtribk(n,ar,tau,m,zr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: m,n
    Real(t_real),Intent(in)     :: ar(n*n),tau(2,n)
    Real(t_real),Intent(inout)  :: zr(n*m)
    !!LOCAL
    Integer(t_int) :: i,j,jm1,k,l,im1n,jm1n,ki,kj
    Real(t_real)  :: h,s,smin
    !!
    If (m  ==  0_t_int) Return
    !! --- transform the eigenvectors of the real symmetric
    !! --- tridiagonal matrix to those of the symmetric tridiagonal matrix.

    Do j = 1_t_int, m
       jm1=j-1_t_int
       Do k = 1_t_int, n
          kj=k+jm1*n
          zr(kj) =  zr(kj) * tau(1_t_int,k)
       Enddo
    Enddo

    If (n  ==  1_t_int) Return
    !! --- recover and apply the householder matrices
    Do i = 2_t_int, n
       im1n=(i-1_t_int)*n
       l = i - 1_t_int
       h = tau(2_t_int,i)
       If (h  /=  0.0_t_real) Then
          Do j = 1_t_int, m
             jm1n=(j-1_t_int)*n
             !!ifdefc BLAS
             !!             s  = ddot(l,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int) 

             !!else
             s  = 0.0_t_real
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                s =  s  + ar(ki) * zr(kj) 
             Enddo
             !!endif

             !! --------- double divisions avoid possible underflow
             s = (s / h) / h
             smin = -s

             !!ifdefc BLAS
             !!            call daxpy(l,smin,ar(im1n+1_t_int),1_t_int,zr(jm1n+1_t_int),1_t_int)
             !!else
             Do k = 1_t_int, l
                ki=k+im1n
                kj=k+jm1n
                zr(kj) = zr(kj) + smin * ar(ki) 
             Enddo
             !!endif
          Enddo
       Endif
    Enddo
  End Subroutine rbtribk
  !!***********************************************************************
  !!- Cholesky decomposition of symmetric matrix
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system
  !!io  o     :overlap matrix (only full upper triangle is used)
  !!o Outputs:
  !!io  o     :full lower triangle of Cholesky factor l of o
  !!o          The strict upper triangle is the cc of the lower one
  !!o   ierr  :nonzero if decomposition finds matrix not pos. def.
  !!r Remarks:
  !!r   Adapted from hchd which is adapted from eispack reduc for
  !!r   hermetian matrices. rbchd is written for square matrices.
  !!r   At the end the strict lower triangle is set to the
  !!r   transposed of the strict upper one. Then it is possible to
  !!r   change the indices in particular for the innermost loop so
  !!r   that the first array index equals the loop variable in rbbak
  !! ----------------------------------------------------------------------
  Subroutine rbchd(or,n,ierr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Integer(t_int),Intent(out)   :: ierr
    Real(t_real),Intent(inout)  :: or(n*n)
    !!LOCAL
    Integer(t_int) :: i,j,k,im1,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: xr,y,det,ndet
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          or(ji) = or(ij)
       Enddo
    Enddo

    !! --- form l in the arrays o
    det=1.0_t_real
    ndet=1.0_t_real
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       y=1.0_t_real
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          xr =  or(ji)
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             !!             xr = xr - or(ki)*or(kj) 
             xr = xr - or(ki)*or(kj) 
          Enddo
          If (j /= i) Then
             or(ji) = xr / y
             or(ij) =  or(ji)
          Else
             ierr = i
             If (xr <=  0.0_t_real) Return
             y = Sqrt(xr)
             ndet=ndet*or(ji)
             det=det*xr
!             Print 12,i,xr
!             Print 13,i,xr/or(ji)
!             Print 14,i,det/ndet
             or(ji) = y
!12           Format('rbchd: xr   ',i5,'  =',f30.15)
!13           Format('rbchd: ||xrn',i5,'||=',f30.15)
!14           Format('rbchd: ||Sn ',i5,'||=',f30.15)
          Endif
       Enddo
    Enddo
!    Print *,'rbchd: det=',det
!    Print *,'rbchd: <det>=',Exp(Log(det)/n)
!    Print *,'rbchd: ndet=',ndet
!    Print *,'rbchd: <ndet>=',Exp(Log(ndet)/n)
!    Print *,'rbchd: ||Sn||=',det/ndet
!    Print *,'rbchd: <||Sn||>=',Exp(Log(det/ndet)/n)
    ierr = 0_t_int
  End Subroutine rbchd
  !!***********************************************************************
  !!- Reduces nonorthogonal symmetric matrix to orthogonal form
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system.
  !!io  h     :hamiltonian matrix(only full upper triangle is used)
  !!i   o     :Cholesky-decomposed overlap matrix
  !!w Work arrays
  !!w   ht    :work array
  !!o Outputs:
  !!io  h     :reduced hermetian matrix:  a <-  ldag^-1 a l^-1
  !!o          The strict upper triangle
  !!r Remarks:
  !!r   Adapted from hred which is adapted from eispack reduc for
  !!r   hermetian matrices. rbred is written for square matrices.
  !!r   At the beginning the strict upper triangle of ht is set to the
  !!r   transposed of the strict lower one of h. 
  !!r   Then it is possible to
  !!r   change the indices in particular for the innermost loop so
  !!r   that the first array index equals the loop variable.
  !! ----------------------------------------------------------------------
  Subroutine rbred(n,hr,or,htr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)     :: or(n*n)
    Real(t_real),Intent(inout)  :: hr(n*n)
    Real(t_real),Intent(out)    :: htr(n*n)
    !!LOCAL
    Integer(t_int) :: i,ii,im1,j,jm1,k,im1n,jm1n,ij,ji,ki,kj
    Real(t_real)  :: xr,y
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = 1_t_int, j
          ij=i+jm1n
          htr(ij) =  hr(ij)
       Enddo
    Enddo

    !! --- form the transpose of the upper triangle of inv(l)*h
    !! --- in the lower triangle of the array h
    Do i = 1_t_int, n
       im1 = i - 1_t_int
       im1n=im1*n
       ii=i+im1n
       y = or(ii)

       !!i ----  (l^-1 * h)_ij  =  (h_ij - sum_k<i  l_ik * (l^-1)_kj) / l_ii
       Do j = i, n
          jm1n=(j-1_t_int)*n
          ij=i+jm1n
          ji=j+im1n
          xr = hr(ij)
          Do k = 1_t_int, im1
             ki=k+im1n
             kj=k+jm1n
             xr = xr - or(ki)*htr(kj) 
          Enddo
          hr(ji) = xr / y
          htr(ij) = hr(ji)
       Enddo
    Enddo
    !! --- pre-multiply by inv(l) and overwrite
    Do j = 1_t_int, n
       jm1 = j - 1_t_int
       jm1n=jm1*n
       Do i = j, n
          im1 = i - 1_t_int
          im1n=im1*n
          ii=i+im1n
          ij=i+jm1n
          ji=j+im1n
          xr =  hr(ij)
          Do k = j, im1
             ki=k+im1n
             kj=k+jm1n
             xr = xr - htr(kj)*or(ki) 
          Enddo
          Do k = 1_t_int, jm1
             ki=k+im1n
             kj=k+jm1n
             xr = xr - hr(kj)*or(ki) 
          Enddo
          hr(ji) = xr / or(ii)
          htr(ij) = hr(ji)
       Enddo
    Enddo
  End Subroutine rbred
  !!***********************************************************************
  !!- Back-transforms eigenvectors to nonorthogonal representation
  !! ----------------------------------------------------------------------
  !!i Inputs:
  !!i   n     :order of the matrix system
  !!i   o     :Cholesky-decomposed overlap matrix
  !!io  z     :eigenvectors to be back transformed
  !!o Outputs:
  !!io  z     :transformed eigenvectors
  !!r Remarks:
  !!r   Adapted from rebakh, eispack
  !! ----------------------------------------------------------------------
  Subroutine rbbak(n,or,zr)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)     :: or(*)
    Real(t_real),Intent(inout)  :: zr(*)
    !!LOCAL 
    Integer(t_int) :: i,j,k,im1n,jm1n,ij,ii,ki,kj
    Real(t_real)  :: xr
    !!
    Do j = 1_t_int, n
       jm1n=(j-1_t_int)*n
       Do i = n, 1_t_int, -1_t_int
          im1n=(i-1_t_int)*n
          ij=i+jm1n
          ii=i+im1n
          xr = zr(ij)
          Do k = i+1_t_int, n
             ki=k+im1n
             kj=k+jm1n
             xr = xr - or(ki)*zr(kj) 
          Enddo
          zr(ij) = xr/or(ii)
       Enddo
    Enddo
  End Subroutine rbbak
  !!***********************************************************************

  
End Module diagonalize
