Module diracdefs
  !!
  Use precisiondefs
  Use file_iounits
  Use constants
  Use errors
  Use memory
  Use trelativistic
  Use matserv
  Use tdiracdefs
  Use atomdefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="DIRACDEFS"  
  !!
  Public :: init_diracdef,cgoOFKapMu&
       &,makekrkgaunt,makekrkgaunt2,setup_quant,cxylm,ryhzy4,sphsp&
       &,spherical_spinor&
       &,LARGE_COMP1,SMALL_COMP1,LARGE_COMP2,SMALL_COMP2,c_mukk&
       &,c_kmuLzkmu,wignerDJ,djmm,wignerD2,chi_km
  !!***********************************************************************
  !! the components of the relativistic orbitals are counted as
  !! large1,small1,large2,small2, where large2... occurs in magnetic orbitals
  !!
  !! where we have  |kap,mu>=/   g_{1} * \chi_{ kap,mu}+   g_{2} * \chi_{-kap-1,mu}  \
  !!                         \ i f_{1} * \chi_{-kap,mu}+ i f_{2} * \chi_{ kap+1,mu}  /
  !!
  Integer(t_int),Parameter :: LARGE_COMP1=1_t_int
  Integer(t_int),Parameter :: SMALL_COMP1=2_t_int
  Integer(t_int),Parameter :: LARGE_COMP2=3_t_int
  Integer(t_int),Parameter :: SMALL_COMP2=4_t_int
  !! the Clebsch-Gordons, see chi_km
  Real(t_real),Allocatable   :: cgoOFKapMu(:,:,:)!!2,-lmax-1:lmax+1,-2*lmax-1:2*lmax+1
  
Contains
  !!***********************************************************************
  Subroutine  setup_quant(quant,relativistic,lmax)
    !!
    Implicit none  
    !!IO
    Type(T_Quant),Intent(inout)  :: quant
    Integer(t_int),Intent(in)    :: relativistic
    !! maximum angular momentum for the basis orbitals
    Integer(t_int),Intent(in)    :: lmax
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_quant"
    Logical(t_log) :: prot
    Integer(t_int) :: i,l,j,ij
!!$    Integer(t_int) :: ka,kb,mma,mmb
!!$    Complex(t_real):: c
    Real(t_real)   :: aprime(3),d,fac(0:2*lmax+1)
    Complex(t_real),Pointer:: cdjmm(:,:)
    Complex(t_real):: SR(2,2,3),DSD(2,2,3),sigma(2,2,3)
    Complex(t_real),Parameter   :: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter   :: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter   :: ci=(0.0_t_real,1.0_t_real)
    !!
    Nullify(cdjmm)
    !!
    Call make_factorials(fac,2*lmax+1)
    !!
    prot=(relativistic==FULL_RELATIVISTIC)
    !!
    If(prot)Then
1      Format('Quantization axis (magnetic field) along: ('&
            &,2(f10.5,','),f10.5,')')
       !!
       Write(outfile,1) quant% axis(1:3)
       !!
    Endif
    !!
    d=Sqrt(Sum(quant% axis(:)**2))
    If(d<1.0e-8_t_real)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Quantization axis input invalid, norm is tiny ?!'")

       quant% axis(1:2)=0.0_t_real
       quant% axis(3)=1.0_t_real
       d=1.0_t_real
    Endif
    !!
    Call reuler(quant% axis,quant% eulang)
    !!
    If(prot)Then
       Write(outfile,3) quant% eulang*180.0_t_real/pi
    Endif
    !!
    Call m_euler(quant% MM,quant% eulang)
    !!
    !! axis in \Sigma^\prime is axis.M
    Do i=1,3
       aprime(i)=Dot_product(quant% axis(:),quant% MM(:,i))
    Enddo
    If(prot)Then
       Write(outfile,2) "Quan.-axis becomes: " ,aprime(:)/d
!!$       Write(outfile,2) "new x-axis is old : " &
!!$            &,Matmul((/1.0_t_real,0.0_t_real,0.0_t_real/),Transpose(quant% MM(:,:)))
       Write(outfile,4)
    Endif
    !!
    !!
    !!
    Call wignerD2(quant% eulang,quant% D1o2)
    !!
    If(prot)Then
       Write(outfile,22) quant% D1o2(1,1),quant% D1o2(1,2)
       Write(outfile,23) quant% D1o2(2,1),quant% D1o2(2,2)
       Write(outfile,4)
    Endif
    !!
    !! paranoia
    !!
    sigma(:,:,:)=c0
    sigma(1,1,3)=c1
    sigma(2,2,3)=-c1
    sigma(1,2,1)=c1
    sigma(2,1,1)=c1
    sigma(2,1,2)=ci
    sigma(1,2,2)=-ci
    !!    
    Do i=1,2
       Do j=1,2
          SR(i,j,:)=Matmul(sigma(i,j,:),quant% MM(:,:))
       End Do
    End Do

    
    Do i=1,3
       DSD(:,:,i)=Matmul(Matmul(quant% D1o2,sigma(:,:,i))&
            &,Conjg(Transpose(quant% D1o2)))
    Enddo
    !!
    !!
    If(Any(Abs(SR-DSD)>1.0e-10_t_real))Then
       Write(outfile,918)
918    Format('Symmetry assumption in D1o2 matrices is wrong ?',/&
            &,'Please contact authors!')
       Write(outfile,322) 'sig_x=',((sigma(i,j,1),j=1,2),i=1,2)
       Write(outfile,322) 'sig_y=',((sigma(i,j,2),j=1,2),i=1,2)
       Write(outfile,322) 'sig_z=',((sigma(i,j,3),j=1,2),i=1,2)
       Write(outfile,322) 'sigR_x=',((SR(i,j,1),j=1,2),i=1,2)
       Write(outfile,322) 'sigR_y=',((SR(i,j,2),j=1,2),i=1,2)
       Write(outfile,322) 'sigR_z=',((SR(i,j,3),j=1,2),i=1,2)
       Write(outfile,322) 'DsigD+_x=',((DSD(i,j,1),j=1,2),i=1,2)
       Write(outfile,322) 'DsigD+_y=',((DSD(i,j,2),j=1,2),i=1,2)
       Write(outfile,322) 'DsigD+_z=',((DSD(i,j,3),j=1,2),i=1,2)
322    Format(a,t41,': ',2('(',f10.5,',',f10.5,') '),/&
            &,t43,2('(',f10.5,',',f10.5,') '),/)
       Call terminate(TERM_CRASH,' D1o2 is wrong!?')
    Endif

    !!
    !!
    !!
    Call palloc('quant% p_chiSigmaRchi',quant% p_chiSigmaRchi&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,nmodule,nsubfunc)
    Call palloc('quant% p_chiVSigmachi',quant% p_chiVSigmachi&
         &,1_t_int,3_t_int&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,nmodule,nsubfunc)
    Call palloc('quant% p_chiLRchi',quant% p_chiLRchi&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,nmodule,nsubfunc)
    Call palloc('quant% p_chipSLRchi',quant% p_chipSLRchi&
         &,1_t_int,2_t_int&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,-lmax-1,lmax+1 ,1_t_int,2*(lmax+1)&
         &,nmodule,nsubfunc)
    !!
    Call palloc('cdjmm',cdjmm,1_t_int,(2*lmax+2)**2,1_t_int,(lmax+1) &
         &,nmodule,nsubfunc)
    
    !! prepare the rotation matrices (Dj functions)
    Do l=0,lmax
       j=2*l+1
       ij=l+1 !! =(j+1)/2 = Abs(kappa)
       Call wignerDJ(j,quant% eulang,cdjmm(:,ij),fac,.False.)
    Enddo

    !!

    Call make_chiLRchi(quant% D1o2,cdjmm,lmax&
         &,quant% p_chiSigmaRchi,quant% p_chiLRchi,quant% p_chipSLRchi&
         &,quant% p_chiVSigmachi)
    
    !!
!!$    Do ka=-lmax-1,lmax+1
!!$       Do mma=1,2*Abs(ka)
!!$          Do kb=-lmax-1,lmax+1
!!$             Do mmb=1,2*Abs(kb)
!!$                c=quant% p_chiSigmaRchi(ka,mma,kb,mmb)&
!!$                     &-Dot_product(quant% axis(1:3)&
!!$                     &   ,quant% p_chiVSigmachi(1:3,ka,mma,kb,mmb))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err',ka,mma,kb,mmb,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$       Enddo
!!$    Enddo
    !!
    Call pfree(cdjmm)
    !!
22  Format('s=1/2 Wigner D-function: ',t41,': ',2('(',f10.5,',',f10.5,') '))
23  Format(t41,': ',2('(',f10.5,',',f10.5,') '))
2   Format(a,t41,': (',2(f10.5,','),f10.5,')')
3   Format('Euler angles: alpha=',f10.5,' beta=',f10.5,' gamma=',f10.5)
4   Format(/)
    !!
  End Subroutine setup_quant
  !!***********************************************************************
  Subroutine make_chiLRchi(D1o2,cdjmm,lmax,chiSigmaRchi,chiLRchi,chipSLRchi&
       &,chiVSigmachi)
    !!
    Implicit none  
    !!IO
    Integer(t_int) :: lmax
    Complex(t_real),Intent(in)   :: D1o2(2,2)
    Complex(t_real),Intent(in)   :: cdjmm((2*lmax+2)**2,lmax+1)
    Complex(t_real),Intent(out)  :: chiSigmaRchi(-lmax-1:lmax+1,2*lmax+2&
         &,-lmax-1:lmax+1,2*lmax+2)
    !! <chi|\vec{Sigma}|chi>
    Complex(t_real),Intent(out)  :: chiVSigmachi(1:3,-lmax-1:lmax+1,2*lmax+2&
         &,-lmax-1:lmax+1,2*lmax+2)
    Complex(t_real),Intent(out)  :: chiLRchi(-lmax-1:lmax+1,2*lmax+2&
         &,-lmax-1:lmax+1,2*lmax+2)
    !! spin projected L_z expectation value along quantization axis
    Complex(t_real),Intent(out)  :: chipSLRchi(1:2,-lmax-1:lmax+1,2*lmax+2&
         &,-lmax-1:lmax+1,2*lmax+2)
    !!LOCAL
    Logical(t_log) :: exists
    Integer(t_int) :: kapa,kpa,kapb,kpb,la,lb,mma,mmb,mua,mub,n,ms,kp,nu
    Integer(t_int) :: dega,degb,na,nb
    Real(t_real)   :: fs,f
    Complex(t_real):: c,d,cc1,cc2
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
!    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    !!
    !! lmax:   (  \chi_{\kappa=lmax}  )   (  \chi_{\kappa=-lmax-1}  )
    !!         (  \chi_{\kappa=-lmax} )   (  \chi_{\kappa=lmax+1 } )
    !! 
    !!
    kapaloop: Do kapa=-lmax-1,lmax+1
       kpa=Abs(kapa)
       dega=2*kpa
       If(kapa==0) Cycle kapaloop
       la=(Abs(2*kapa+1)-1)/2
       kapbloop: Do kapb=-lmax-1,lmax+1
          kpb=Abs(kapb)
          degb=2*kpb
          If(kapb==0) Cycle kapbloop
          lb=(Abs(2*kapb+1)-1)/2
          mmaloop: Do mma=1,dega
             mua=2*(mma-kpa)-1
             mmbloop: Do mmb=1,degb
                mub=2*(mmb-kpb)-1
                

                d=c0
                If(kapa == kapb ) Then
                   
                   !!        sum_{n}  D^{j}_{mua,n} n    D^{j*}_{mub,n}
                   Do n=1,degb
                      nu=2*(n-kpb)-1
                      d=d+           cdjmm(dega*(n-1)+mma,kpa) &
                           & * nu*0.5_t_real &
                           & * Conjg(cdjmm(degb*(n-1)+mmb,kpb))
                   Enddo
                Endif

                Call c_kmuVSigmakmu(kapa,mua,kapb,mub,chiVSigmachi(:,kapa,mma,kapb,mmb))


                Call c_kmuSigmaRkmu(kapa,mua,kapb,mub,D1o2,exists,c)
                chiSigmaRchi(kapa,mma,kapb,mmb)=c
                chiLRchi    (kapa,mma,kapb,mmb)=d-c*0.5_t_real
                
                
                !!        sum_{n}  D^{j}_{mua,n} n    D^{j*}_{mub,n}
                kp=Min(kpa,kpb)
                Do ms=1,2
                   fs=3.0_t_real-2.0_t_real*ms
                   d=c0
                   If(la == lb) Then
                      Do n=1,2*kp
                         nu=2*(n-kp)-1
                         na=(nu+1)/2+kpa
                         nb=(nu+1)/2+kpb
                         
                         cc1=cdjmm(dega*(na-1)+mma,kpa) 
                         cc2=Conjg(cdjmm(degb*(nb-1)+mmb,kpb))
                         f=       cgoOFKapMu(ms,kapa,nu) &
                              & * (nu-fs)*0.5_t_real &
                              & * cgoOFKapMu(ms,kapb,nu)
                         
                         
                         d=d+           cc1 * f * cc2
                         
                         
                      Enddo
                   Endif
                   chipSLRchi    (ms,kapa,mma,kapb,mmb)=d
                   
                Enddo

                

             Enddo mmbloop
          End Do mmaloop
       Enddo kapbloop
    Enddo kapaloop
    !!
    !!
!!$    kapaloop1: Do kapa=-lmax-1,lmax+1
!!$       kpa=Abs(kapa)
!!$       If(kapa==0) Cycle kapaloop1
!!$       la=(Abs(2*kapa+1)-1)/2
!!$       kapbloop1: Do kapb=-lmax-1,lmax+1
!!$          kpb=Abs(kapb)
!!$          If(kapb==0) Cycle kapbloop1
!!$          lb=(Abs(2*kapb+1)-1)/2
!!$          Write(outfile,10)lqnchar(la),2*kpa-1,lqnchar(lb),2*kpb-1
!!$10        Format(a1,i1,'/2',' - ',a1,i1,'/2')
!!$          mmaloop1: Do mma=1,2*kpa
!!$             mua=2*(mma-kpa)-1
!!$             mmbloop1: Do mmb=1,2*kpb
!!$                mub=2*(mmb-kpb)-1
!!$                d=c0
!!$                Do ms=1,2
!!$                   d=d+chipSLRchi    (ms,kapa,mma,kapb,mmb)
!!$                Enddo
!!$                !Write(outfile,12,advance='no') chiLRchi(kapa,mma,kapb,mmb)
!!$12              Format('(',f12.5,',',f12.5,') ')
!!$                If(Abs(d - chiLRchi(kapa,mma,kapb,mmb)) > 1.0e-10_t_real&
!!$                     & ) Then
!!$                   Write(outfile,12,advance='no') d-chiLRchi(kapa,mma,kapb,mmb)
!!$                   
!!$                Endif
!!$             End Do mmbloop1
!!$             Write(outfile,*)
!!$          End Do mmaloop1
!!$       End Do kapbloop1
!!$       Write(outfile,*)
!!$    End Do kapaloop1
!!$    
!!$    Stop' UNEQUAL'
    !!
    Return
    !!
  End Subroutine make_chiLRchi
  !!***********************************************************************
  !!
  !!
  !! \chi_{kappa^\prime,\mu^\prime} ( \sigma \cdot R )_z \chi_{kappa,\mu}
  !! with rotation R
  !! 
  Subroutine c_kmuSigmaRkmu(ka,ma,kb,mb,D1o2,exists,res)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: ka,ma,kb,mb
    Complex(t_real),Intent(in)   :: D1o2(2,2)
    Logical(t_log),Intent(out)   :: exists
    Complex(t_real),Intent(out)  :: res
    !!LOCAL
    Integer(t_int) :: la,lb,s
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    res=c0
    exists=.False.
    !!
    la=(Abs(2*ka+1)-1)/2
    lb=(Abs(2*kb+1)-1)/2
    !!
    If(la /= lb) Return
    If(ka == 0) Return
    If(kb == 0) Return
    If(Abs(ma) > 2*Abs(ka)-1) Return
    If(Abs(mb) > 2*Abs(kb)-1) Return
    !!
    If( ma == mb ) Then
       exists=.True.
       Do s=1,2
          res=res&
               &+cgoOFKapMu(s,ka,ma)  &
               &*(D1o2(s,1)*Conjg(D1o2(s,1))-D1o2(s,2)*Conjg(D1o2(s,2)))&
               &*cgoOFKapMu(s,kb,mb)
       Enddo
    Else If( ma == mb-2 ) Then
       exists=.True.
       res=cgoOFKapMu(2,ka,ma)  &
            & *  (D1o2(2,1)*Conjg(D1o2(1,1))-D1o2(2,2)*Conjg(D1o2(1,2)))&
            & *  cgoOFKapMu(1,kb,mb)
    Else If( ma == mb+2 ) Then
       exists=.True.
       res= cgoOFKapMu(1,ka,ma)  &
            & *  (D1o2(1,1)*Conjg(D1o2(2,1))-D1o2(1,2)*Conjg(D1o2(2,2)))&
            & *  cgoOFKapMu(2,kb,mb)
    Endif
    !!
    Return
    !!
  End Subroutine c_kmuSigmaRkmu
  !!***********************************************************************
  !!
  !!
  !! \chi_{kappa^\prime,\mu^\prime} ( \vec_{\sigma}  )_z \chi_{kappa,\mu}
  !! 
  Subroutine c_kmuVSigmakmu(ka,ma,kb,mb,res)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: ka,ma,kb,mb
    Complex(t_real),Intent(out)  :: res(3)
    !!LOCAL
    Integer(t_int) :: la,lb,s,t,k
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: cm1=(-1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Complex(t_real),Parameter:: cmi=(0.0_t_real,-1.0_t_real)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real):: sigma(2,2,3)
    Data sigma(1,1:2,1)/c0, c1/
    Data sigma(2,1:2,1)/c1, c0/
    Data sigma(1,1:2,2)/c0, cmi/
    Data sigma(2,1:2,2)/ci, c0/
    Data sigma(1,1:2,3)/c1, c0/
    Data sigma(2,1:2,3)/c0, cm1/
    !!
    res=c0
    !!
    la=(Abs(2*ka+1)-1)/2
    lb=(Abs(2*kb+1)-1)/2
    !!
    If(la /= lb) Return
    If(ka == 0) Return
    If(kb == 0) Return
    If(Abs(ma) > 2*Abs(ka)-1) Return
    If(Abs(mb) > 2*Abs(kb)-1) Return
    !!
    Do t=1,2
       Do s=1,2
          !!updown(t)=-2*(t-1)+1
          If(ma+2*t==mb+2*s)Then
             Do k=1,3
                res(k)=res(k)&
                     &+cgoOFKapMu(t,ka,ma)*sigma(t,s,k)*cgoOFKapMu(s,kb,mb)
             Enddo
          Endif
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine c_kmuVSigmakmu
  !!***********************************************************************
  Subroutine  init_diracdef(lmax)
    !!
    Implicit None  
    !!IO
    !! maximum angular momentum of occuring orbitals
    Integer(t_int),Intent(in)    :: lmax
    !!LOCAL
    Character(*),Parameter::nsubfunc="init_diracdef"
    Integer(t_int) :: kap,j,m,mu!,i,jj,l
    Integer        :: icheck
    !!
    Allocate(cgoOFKapMu(2,-lmax-1:lmax+1,-2*lmax-1:2*lmax+1),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for cgoOFKapMu !'")
    Endif
    !!
    !!
    !! Clebsch-Gordon coefficients of \chi_{kap,mu}
    !!
    cgoOFKapMu(1:2,-lmax-1:lmax+1,-2*lmax-1:2*lmax+1)=0.0_t_real
    !!
    Do kap=-lmax-1,lmax+1
       j=2*Abs(kap)-1
       Do m=1,j+1
          mu=-j+2*(m-1)
          Call chi_km(kap,mu,cgoOFKapMu(1,kap,mu))
       Enddo
    Enddo
    !!
    !!
  End Subroutine init_diracdef
  !!***********************************************************************
  !!***********************************************************************
  !!
  !! Returns bispinors Chi(kappa,mu)=Omega(j,l,mu)
  !!
  Subroutine chi_km(kap,mu,cgordon)
    !!
    !!DOC This subroutine returns the Clebsch Gordon coefficients
    !!DOC $c_{\kappa\mu\sigma}$ and the values of $l$ and $m_{\sigma}$
    !!DOC for a spin angular functions
    !!DOC \begin{equation}\chi_{\kappa\mu}=\Omega_{jl\mu}=\left (
    !!DOC \begin{array}{c}
    !!DOC c_{\kappa\mu\uparrow} {\cal Y}_{lm_{\uparrow}}\\
    !!DOC c_{\kappa\mu\downarrow} {\cal Y}_{lm_{\downarrow}}\\
    !!DOC \end{array}\right )\nonumber\end{equation}
    !!DOC characterized by $\kappa$ and $\mu$.\\
    !!DOC {\bf If $\chi_{\kappa\mu}$ does not exist, the Clebsch Gordon
    !!DOC coefficients are set to zero!}
    !!
    !!
    Implicit None
    !!IO
    !!DOC io={kap}{$\kappa$ quantum number}endio
    Integer(t_int),Intent(in)    :: kap
    !!DOC io={mu}{$\mu$ quantum number in units (1/2) it is the 
    !!DOC z-component of total torque}endio
    Integer(t_int),Intent(in)    :: mu
    !!DOC io={cgordon}{Clebsch Gordon coefficients 
    !!DOC $(c_{\kappa\mu\uparrow},c_{\kappa\mu\downarrow})$}endio
    Real(t_real),Intent(out)     :: cgordon(2)
    !!LOCAL
    !! total torque j in units 1/2
    Integer(t_int) :: j,ks!,l,ms
    Real(t_real)   :: deg
    !!
    !! check if bispinor exists
    j=2*Abs(kap)-1
    If ((kap==0).Or.(Abs(mu)>j)) Then
       cgordon(1)=0.0_t_real
       cgordon(2)=0.0_t_real
    Else 
       !l=(Abs(2*kap+1)-1)/2
       deg=Real(2*kap+1,t_real)
       ks=Sign(1_t_int,kap)
       
       cgordon(1)=-ks*Sqrt( (1.0_t_real-mu/deg)*0.5_t_real )
       cgordon(2)=    Sqrt( (1.0_t_real+mu/deg)*0.5_t_real )
       

       If(kap<0)Then 
          If(mu==j)Then
             cgordon(1)=1.0_t_real
             cgordon(2)=0.0_t_real
          Else If(mu==-j)Then
             cgordon(1)=0.0_t_real
             cgordon(2)=1.0_t_real
          Endif
       Endif
    Endif
    !!
    Return
    !!
  End Subroutine chi_km
  !!***********************************************************************
  !! This function returns the value of the matrix element
  !! $\langle \chi_{\kappa_1\mu}|\sigma_z|\chi_{\kappa_2\mu}\rangle$.
  !! (integrated over a sphere)
  !!
  Real(t_real) Function c_mukk(mu,kap1,kap2,exists)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: mu
    !! $\mu$ quantum number (in units (1/2))
    !! $\kappa_1$ quantum number
    Integer(t_int),Intent(in)    :: kap1
    !! $\kappa_2$ quantum number
    Integer(t_int),Intent(in)    :: kap2
    !! does the matrix element exist ?
    Logical(t_log),Intent(out)   :: exists
    !!LOCAL
    !!
    c_mukk=0.0_t_real
    exists=.False.
    If(Abs(mu)<=2*Abs(kap1)-1 .And. kap1 /= 0)Then
       If(kap1==kap2 )Then
          exists=.True.
          c_mukk=-Real(mu,t_real)/(2.0_t_real*kap1+1.0_t_real)
       Else If(-kap1-1==kap2 .And. kap2 /= 0 .And. Abs(mu)<= 2*Abs(kap2)-1)Then
          exists=.True.
          c_mukk=-Sqrt(1.0_t_real&
               &-(Real(mu,t_real)/(2.0_t_real*kap1+1.0_t_real))**2&
               &)
       Endif
    Endif
    !!
    Return
    !!
  End Function c_mukk
  !!***********************************************************************
  !! calculate all spherical spinor remainders up to momentum lmax
  !! for relativistic kinetic and overlap integration
  Subroutine  ryhzy4(lmax,y,z,ss)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: lmax
    Real(t_real),Intent(in)      :: y,z
    Real(t_real),Intent(out)     :: ss(2,(lmax+1),-(lmax+1):(lmax+1))
    !!LOCAL
    Real(t_real)   :: ylm4,ylm5,ylm6,ylm7,ylm8,ylm9,ylm10
    Real(t_real)   :: ylm11,ylm12,ylm13,ylm14,ylm15,ylm16,ylm17,ylm18,ylm19
    Real(t_real)   :: ylm20,ylm21
    !!    
    !!
    !!

    If(lmax>4) Then
       Call terminate(TERM_CRASH,' ryhzy4 called with lmax>4')
    Endif

    !!--------------------------------
    !!
    
 
    !! kap=-1 , s1/2  1/2
    ss(1,1,-1)=0.70710678118654752440_t_real
    ss(2,1,-1)=0.0_t_real
    !!
    !! kap=+1 , p1/2  1/2
    ss(1,1,1)=-0.70710678118654752438_t_real*z
    ss(2,1,1)=+0.70710678118654752438_t_real*y
    !!
    If(lmax==0) Goto 1000
    !!------------------------------
    !! kap=-2 , p3/2  1/2
    ss(1,1,-2)=1.0_t_real*z
    ss(2,1,-2)=0.5_t_real*y
    !! kap=-2 , p3/2  3/2
    ss(1,2,-2)=0.86602540378443864676_t_real*y
    ss(2,2,-2)=0.0_t_real
    !!

    ylm4=(z**2*2.37170824512628449903_t_real-.79056941504209483301_t_real)
    ylm5=(z*1.93649167310370844257_t_real*y)
    ylm6=y**2*0.96824583655185422124_t_real

    !! kap= 2 , d3/2  1/2
    ss(1,1,2)=(-0.63245553203367586639_t_real)*ylm4
    ss(2,1,2)=( 0.77459666924148337703_t_real)*ylm5
    !! kap= 2 , d3/2  3/2
    ss(1,2,2)=(-0.44721359549995793928_t_real)*ylm5
    ss(2,2,2)=( 0.89442719099991587856_t_real)*ylm6
    !!
    If(lmax==1) Goto 1000
    !!------------------------------
    !! kap=-3 d5/2  1/2
    ss(1,1,-3)=( 0.77459666924148337703_t_real)*ylm4
    ss(2,1,-3)=( 0.63245553203367586639_t_real)*ylm5
    !! kap=-3 d5/2  3/2
    ss(1,2,-3)=( 0.89442719099991587856_t_real)*ylm5
    ss(2,2,-3)=( 0.44721359549995793928_t_real)*ylm6
    !! kap=-3 d5/2  5/2
    ss(1,3,-3)=ylm6
    ss(2,3,-3)=0.0_t_real
    !!


    ylm7=z*(1.97202659436653868084_t_real*ylm4-1.24721912892464712851_t_real)
    ylm8=(z*2.09165006633518886990_t_real*ylm5-.81009258730098252885_t_real*y)
    ylm9=(z*2.64575131106459059023_t_real*ylm6)
    ylm10=1.04582503316759443485_t_real*y**3


    !! kap= 3 f5/2  1/2
    ss(1,1,3)=(-0.65465367070797714379_t_real)*ylm7
    ss(2,1,3)=( 0.75592894601845445442_t_real)*ylm8
    !! kap= 3 f5/2  3/2
    ss(1,2,3)=(-0.53452248382484876937_t_real)*ylm8
    ss(2,2,3)=( 0.84515425472851657750_t_real)*ylm9
    !! kap= 3 f5/2  5/2
    ss(1,3,3)=(-0.37796447300922722721_t_real)*ylm9
    ss(2,3,3)=( 0.92582009977255146156_t_real)*ylm10
    !!
    If(lmax==2) Goto 1000
    !!------------------------------
    !! kap=-4 f7/2  1/2
    ss(1,1,-4)=( 0.75592894601845445442_t_real)*ylm7
    ss(2,1,-4)=( 0.65465367070797714379_t_real)*ylm8
    !! kap=-4 f7/2  3/2
    ss(1,2,-4)=( 0.84515425472851657750_t_real)*ylm8
    ss(2,2,-4)=( 0.53452248382484876937_t_real)*ylm9
    !! kap=-4 f7/2  5/2
    ss(1,3,-4)=( 0.92582009977255146156_t_real)*ylm9
    ss(2,3,-4)=( 0.37796447300922722721_t_real)*ylm10
    !! kap=-4 f7/2  7/2
    ss(1,4,-4)=ylm10
    ss(2,4,-4)=0.0_t_real
    !!
    
    ylm11=(z*1.98431348329844294286_t_real*ylm7&
         &  -1.00623058987490536338_t_real*ylm4)
    ylm12=(z*2.04939015319191967665_t_real*ylm8&
         &  -0.97979589711327123926_t_real*ylm5)
    ylm13=(z*2.29128784747792000364_t_real*ylm9&
         &  -0.86602540378443864680_t_real*ylm6)
    ylm14=(z*3.0_t_real*ylm10)
    ylm15=1.10926495933117797240_t_real*y**4
    

    !!
    !! kap= 4 g7/2  1/2
    ss(1,1,4)=(-0.66666666666666666666_t_real)*ylm11
    ss(2,1,4)=( 0.74535599249992989879_t_real)*ylm12
    !! kap= 4 g7/2  3/2
    ss(1,2,4)=(-0.57735026918962576450_t_real)*ylm12
    ss(2,2,4)=( 0.81649658092772603272_t_real)*ylm13
    !! kap= 4 g7/2  5/2
    ss(1,3,4)=(-0.47140452079103168293_t_real)*ylm13
    ss(2,3,4)=( 0.88191710368819686349_t_real)*ylm14
    !! kap= 4 g7/2  7/2
    ss(1,4,4)=(-0.33333333333333333333_t_real)*ylm14
    ss(2,4,4)=( 0.94280904158206336586_t_real)*ylm15

    If(lmax==3) Goto 1000
    !!------------------------------
    !!
    !!
    !! kap=-5 g7/2  1/2
    ss(1,1,-5)=( 0.74535599249992989879_t_real)*ylm11
    ss(2,1,-5)=( 0.66666666666666666666_t_real)*ylm12
    !! kap=-5 g7/2  3/2
    ss(1,2,-5)=( 0.81649658092772603272_t_real)*ylm12
    ss(2,2,-5)=( 0.57735026918962576450_t_real)*ylm13
    !! kap=-5 g7/2  5/2
    ss(1,3,-5)=( 0.88191710368819686349_t_real)*ylm13
    ss(2,3,-5)=( 0.47140452079103168293_t_real)*ylm14
    !! kap=-5 g7/2  7/2
    ss(1,4,-5)=( 0.94280904158206336586_t_real)*ylm14
    ss(2,4,-5)=( 0.33333333333333333333_t_real)*ylm15
    !! kap=-5 g7/2  9/2
    ss(1,5,-5)=ylm15
    ss(2,5,-5)=0.0_t_real 
    !!

    ylm16=(z*1.98997487421323990945_t_real*ylm11&
         &  -1.00285307284481394972_t_real*ylm7)
    ylm17=(z*2.03100960115899009012_t_real*ylm12&
         &  -0.99103120896511485333_t_real*ylm8)
    ylm18=(z*2.17124059336723766014_t_real*ylm13&
         &  -0.94760708295868567065_t_real*ylm9)
    ylm19=(z*2.48746859276654986209_t_real*ylm14&
         &  -0.82915619758884995052_t_real*ylm10)
    ylm20=(z*3.31662479035539984915_t_real*ylm15)
    ylm21=1.16340690431164126724_t_real*y**5
    !!
    !! kap= 5 h7/2  1/2
    ss(1,1,5)=(-0.67419986246324208624_t_real)*ylm16
    ss(2,1,5)=( 0.73854894587599639640_t_real)*ylm17
    !! kap= 5 h7/2  3/2
    ss(1,2,5)=(-0.60302268915552724529_t_real)*ylm17
    ss(2,2,5)=( 0.79772403521746564185_t_real)*ylm18
    !! kap= 5 h7/2  5/2
    ss(1,3,5)=(-0.52223296786709351452_t_real)*ylm18
    ss(2,3,5)=( 0.85280286542244173718_t_real)*ylm19
    !! kap= 5 h7/2  7/2
    ss(1,4,5)=(-0.42640143271122086859_t_real)*ylm19
    ss(2,4,5)=( 0.90453403373329086793_t_real)*ylm20
    !! kap= 5 h7/2  9/2
    ss(1,5,5)=(-0.30151134457776362264_t_real)*ylm20
    ss(2,5,5)=( 0.95346258924559231544_t_real)*ylm21

    !!
    !!
    !!
1000 Continue
    !!
    !!
    Return
    !!
  End Subroutine ryhzy4
  !!***********************************************************************
  !! Return the spherical spinor in ss for qn.s kap and mu.
  !! The array cy of (complex) spherical harmonics has to be provided such
  !! that it contains the harmonics for all lm needed in the spinor.
  !! The array cy is 1-based and the order of Y_lm in cy is
  !! Y_{0,0}, Y_{1,-1}, Y_{1,0}, Y_{1,1}, Y_{2,-2},\ldots.
  Subroutine  spherical_spinor(kap,mu,cy,ss)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: kap
    !! mu qn. in units of 1/2
    Integer(t_int),Intent(in)    :: mu
    Complex(t_real),Intent(in)   :: cy(*)
    Complex(t_real),Intent(out)  :: ss(2)
    !!LOCAL
    Integer(t_int) :: l,ml
    !!
    l=(Abs(2*kap+1)-1)/2
    ml=((mu-1)/2)
    ss(1)=0.0_t_real
    ss(2)=0.0_t_real
    !!
    If(Abs(ml)<=l)Then
       ss(1)=cgoOFKapMu(1,kap,mu)*cy(l*(l+1)+1+ml)
    Endif
    If(Abs(ml+1)<=l)Then
       ss(2)=cgoOFKapMu(2,kap,mu)*cy(l*(l+1)+1+ml+1)
    Endif
    !!
    Return
    !!    
  End Subroutine spherical_spinor
  !!***********************************************************************
  !! calculate all spherical spinor remainders up to momentum lmax
  !! for relativistic kinetic and overlap integration
  !! not speed optimized !!!
#ifdef my_compile_unused
  Subroutine  ryhzy(lmax,y,z,ss)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: lmax
    Real(t_real),Intent(in)      :: y,z
    Real(t_real),Intent(out)     :: ss(2,(lmax+1),-(lmax+1):(lmax+1))
    !!LOCAL
    Integer(t_int) :: m,l1,kap,mu,ind!,l
    Real(t_real)   :: fac(0:2*(lmax+1)),s,dfac(0:2*(lmax+1)),p
    Real(t_real)   :: pl,pl1
    Real(t_real)   :: ylm(((lmax+2)*(lmax+3))/2)
    !!    
    fac(0)=1.0_t_real
    dfac(0)=1.0_t_real
    Do m=1,2*(lmax+1)
       fac(m)=fac(m-1)*m
       dfac(m)=dfac(m-1)*(2*m-1)
    Enddo
    !!
    !!
    !!
    pl1=0.0_t_real
    Do m=0,lmax+1
       
       If(m==0)Then
          pl=dfac(m)
       Else
          pl=dfac(m)*y**m
       Endif
       l1=m
       s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-m)/fac(l1+m))
       ylm((l1*(l1+1))/2+1+m)=pl*s
       If(m+1<=lmax+1)Then
          l1=m+1
          pl1=(z*(2*l1-1)*pl)
          s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-m)/fac(l1+m))
          ylm((l1*(l1+1))/2+1+m)=pl1*s
       Endif
       Do l1=m+2,lmax+1
          p=(z*(2*l1-1)*pl1-(l1+m-1)*pl)/(l1-m)
          s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-m)/fac(l1+m))
          ylm((l1*(l1+1))/2+1+m)=p*s
          pl=pl1
          pl1=p
       Enddo
    Enddo
    
    
    !!
    Do kap=1,lmax+1
       Do m=1,kap
          !! positive kap l=kap
          !!
          mu=2*m-1
          ind=(kap*(kap+1))/2
          ss(1,m,kap)=cgoOFKapMu(1,kap,mu)*ylm(ind+m)
          ss(2,m,kap)=cgoOFKapMu(2,kap,mu)*ylm(ind+1+m)

          !! negative kap l=kap-1
          ind=(kap*(kap-1))/2
          ss(1,m,-kap)=cgoOFKapMu(1,-kap,mu)*ylm(ind+m)
          If( mu < 2*kap-1 )Then
             ss(2,m,-kap)=cgoOFKapMu(2,-kap,mu)*ylm(ind+1+m)
          Else
             ss(2,m,-kap)=0.0_t_real
          Endif
       Enddo
    Enddo
    !!
    !!
    !!
!!$    !!
!!$    !!
!!$    pl1=0.0_t_real
!!$    Do m=0,l
!!$       
!!$       If(m==0)Then
!!$          pl=dfac(m)
!!$       Else
!!$          pl=dfac(m)*y**m
!!$       Endif
!!$       l1=m
!!$       s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-Abs(m))/fac(l1+Abs(m)))
!!$       ylm((l1*(l1+1))/2+1+m)=pl*(-1)**m*s
!!$       If(m+1<=l)Then
!!$          l1=m+1
!!$          pl1=(z*(2*l1-1)*pl)
!!$          s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-Abs(m))/fac(l1+Abs(m)))
!!$          ylm((l1*(l1+1))/2+1+m)=pl1*(-1)**m*s
!!$       Endif
!!$       Do l1=m+2,l
!!$          p=(z*(2*l1-1)*pl1-(l1+m-1)*pl)/(l1-m)
!!$          s=Sqrt((2*l1+1)*0.5_t_real * fac(l1-Abs(m))/fac(l1+Abs(m)))
!!$          ylm((l1*(l1+1))/2+1+m)=p*(-1)**m*s
!!$          pl=pl1
!!$          pl1=p
!!$!          Write(6,"(i1,i1,'  ')",advance='no')l1,m
!!$       Enddo
!!$!       Write(6,"()")
!!$    Enddo
!!$    
    !!
    Return
    !!
  End Subroutine ryhzy
#endif
  !!***********************************************************************
  !! calculate complex spherical spinors  up to momentum l
  !! not speed optimized
  Subroutine sphsp(lmax,ss,ldim,y,z,phi)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: lmax,ldim
    Real(t_real),Intent(in)      :: y,z,phi
    Complex(t_real),Intent(out)  :: ss(2,2*(ldim+1),-(ldim+1):(ldim+1))
    !!s,m,kap
    !!LOCAL
    Integer(t_int) :: m,l1,kap,mu,ind
    Real(t_real)   :: fac(0:2*(lmax+1)),s,dfac(0:2*(lmax+1)),pl,pl1,p
    Complex(t_real):: ce
    Complex(t_real):: ylm((lmax+2)**2)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!    
    fac(0)=1.0_t_real
    dfac(0)=1.0_t_real
    Do m=1,2*(lmax+1)
       fac(m)=fac(m-1)*m
       dfac(m)=dfac(m-1)*(2*m-1)
    Enddo
    !!
    !!
    !!
    !!
    pl1=0.0_t_real
    Do m=0,lmax+1
       
       ce=Exp(ci*m*phi)
       If(m==0)Then
          pl=dfac(m)
       Else
          pl=dfac(m)*y**m
       Endif
       l1=m
       s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
       ylm(l1*(l1+1)+1+m)=pl*(-1)**m*s*ce
       ylm(l1*(l1+1)+1-m)=pl*s/ce
       If(m+1<=lmax+1)Then
          l1=m+1
          pl1=(z*(2*l1-1)*pl)
          s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
          ylm(l1*(l1+1)+1+m)=pl1*(-1)**m*s*ce
          ylm(l1*(l1+1)+1-m)=pl1*s/ce
       Endif
       Do l1=m+2,lmax+1
          p=(z*(2*l1-1)*pl1-(l1+m-1)*pl)/(l1-m)
          s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
          ylm(l1*(l1+1)+1+m)=p*(-1)**m*s*ce
          ylm(l1*(l1+1)+1-m)=p*s/ce
          pl=pl1
          pl1=p
       Enddo
    Enddo
    !!
    !!
    !!
    Do kap=1,lmax+1
       Do m=1,2*kap
          !!
          mu=2*(m-1-kap)+1
          !! positive kap l=kap
          !!
          ind=kap*(kap+1)-kap+m
          ss(1,m, kap)=cgoOFKapMu(1,kap,mu)*ylm(ind)
          ss(2,m, kap)=cgoOFKapMu(2,kap,mu)*ylm(ind+1)
          !! negative kap l=abs(kap)-1
          !!
          !!
          ind=kap*(kap-1)-kap+m
          If(m>1)Then
             ss(1,m,-kap)=cgoOFKapMu(1,-kap,mu)*ylm(ind)
          Else
             ss(1,m,-kap)=0.0_t_real
          Endif
          If( m<2*kap )Then
             ss(2,m,-kap)=cgoOFKapMu(2,-kap,mu)*ylm(ind+1)
          Else
             ss(2,m,-kap)=0.0_t_real
          Endif
       Enddo
    Enddo
    !!
    !!
!556 Format(2('(',f30.15,',',f30.15,')'))
    !!
    Return
    !!
  End Subroutine sphsp
  !!***********************************************************************
  !! calculate all complex spherical harmonics  up to momentum l
  !! this is not optimized for speed, it is for test reasons
  Subroutine  cxylm(l,y,theta,phi)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: l
    Real(t_real),Intent(in)      :: theta,phi
    Complex(t_real),Intent(out)  :: y(*)
    !!LOCAL
    Integer(t_int) :: m,l1
    Real(t_real)   :: fac(0:2*l),s,ct,st,dfac(0:2*l),pl,pl1,p
    Complex(t_real):: ce
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!    
    fac(0)=1.0_t_real
    dfac(0)=1.0_t_real
    Do m=1,2*l
       fac(m)=fac(m-1)*m
       dfac(m)=dfac(m-1)*(2*m-1)
    Enddo
    !!
    st=Sin(theta)
    ct=Cos(theta)
!!$    Do m=0,2*l
!!$       Print *,m,fac(m)
!!$    Enddo
!!$    Do m=0,l
!!$       Print *,m,dfac(m)
!!$    Enddo
    !!
    !!
    !!
    pl1=0.0_t_real
    Do m=0,l
       
       ce=Exp(ci*m*phi)
       If(m==0)Then
          pl=dfac(m)
       Else
          pl=dfac(m)*st**m
       Endif
       l1=m
       s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
       y(l1*(l1+1)+1+m)=pl*(-1)**m*s*ce
       y(l1*(l1+1)+1-m)=pl*s/ce
!       Write(6,"(i1,i1,'  ')",advance='no')l1,m
       If(m+1<=l)Then
          l1=m+1
          pl1=(ct*(2*l1-1)*pl)
          s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
          y(l1*(l1+1)+1+m)=pl1*(-1)**m*s*ce
          y(l1*(l1+1)+1-m)=pl1*s/ce
!          Write(6,"(i1,i1,'  ')",advance='no')l1,m
       Endif
       Do l1=m+2,l
          p=(ct*(2*l1-1)*pl1-(l1+m-1)*pl)/(l1-m)
          s=Sqrt((2*l1+1)/fourpi * fac(l1-Abs(m))/fac(l1+Abs(m)))
          y(l1*(l1+1)+1+m)=p*(-1)**m*s*ce
          y(l1*(l1+1)+1-m)=p*s/ce
          pl=pl1
          pl1=p
!          Write(6,"(i1,i1,'  ')",advance='no')l1,m
       Enddo
!       Write(6,"()")
    Enddo
    
    !!
  End Subroutine cxylm
  !!***********************************************************************
  !! calculate all gaunt coefficients (\chi_{kapa,mua}|Y_L|\chi_{kapb,mub})
  !! up to the maximum angular momenta lmxa,lmx,lmxb.
  !! All kappa,mu which belong to the l_a and l_b shell are considered.
  !! the pointer array krk will be allocated and must be freed with
  !! a call to pfree after usage
  !! A typical situation is that one needs the Gaunt coefficients for products
  !! of orbitals with potential componenets. Then the maximum angular momentum
  !! of the orbitals is lmxa=lmxb. The maximum |kappa| will be lmxa+1.
  !! Hence, the maximum L will be 2*(lmxa+1)
  !! The needed call will be
  !!        
  !!        call makekrkgaunt(lmxa,2*(lmxa+1),lmxa,krk)
  !!
  !!  
  !! Do not forget to nullify the pointer krk before the call!!!
  !!
  !! The usage of the coefficients will include a test
  !! if(l<=l_a+l_b) then
  !!   do_something_with(krk(i_{kappa_a,mu_b},i_{l,m},i_{kappa_b,mu_b}))
  !! endif
  !!
  !! with i_{l,m}=l*(l+1)+1+m and 
  !!    i_{kappa,mu}=2*kappa**2+kappa-Abs(kappa)+1++i_{mu}
  !! where i_{mu} runs form 0 to 2j=2*abs(kappa)-1 
  !! and 2mu is 2*i_{mu}-j
  Subroutine  makekrkgaunt(lmxa,lmx,lmxb,krk)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: lmxa,lmx,lmxb
    Complex(t_real),Pointer      :: krk(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="makekrkgaunt"
    Integer(t_int) :: kmxa,kmxb
    Integer(t_int) :: l,kapa,kapb,ira,irb,mua,mub,mla,mlb,ia,ib,ja,jb
    Integer(t_int) :: la,ma,lb,mb,ms
    !!
    Complex(t_real),Pointer ::crc(:,:,:) 
    !!
    !!
    crc => Null()
    !!
    kmxa=lmxa+1
    kmxb=lmxb+1
    !!
    Call palloc('krk',krk&
         &,1_t_int,2_t_int*kmxa*(kmxa+1_t_int)&
         &,1_t_int,(lmx+1_t_int)**2           &
         &,1_t_int,2_t_int*kmxb*(kmxb+1_t_int)&
         &,nmodule,nsubfunc)
    !!
    krk(:,:,:)=Cmplx(0.0_t_real,0.0_t_real,t_real)
    !!
    !! aux array
    Call palloc('crc',crc,1_t_int,(kmxa+1)**2,1_t_int,(lmx+1)**2&
         &,1_t_int,(kmxb+1)**2,nmodule,nsubfunc)
    !!
    !!
    Call makecrcgaunt(kmxa,lmx,kmxb,crc)
    !!
    Do l=1,(lmx+1)**2
       Do kapa=-kmxa,kmxa
          ira=2*kapa**2+kapa-Abs(kapa)+1
          ia=ira-kapa**2
          ja=2*Abs(kapa)-1
          la=(Abs(2*kapa+1)-1)/2
          Do ma=0,ja
             mua=2*ma-ja
             mla=((mua-1)/2)
             Do kapb=-kmxb,kmxb
                irb=2*kapb**2+kapb-Abs(kapb)+1
                ib=irb-kapb**2
                jb=2*Abs(kapb)-1
                lb=(Abs(2*kapb+1)-1)/2
                Do mb=0,jb
                   mub=2*mb-jb
                   mlb=((mub-1)/2)
                   !!
                   krk(ira+ma,l,irb+mb)=0.0_t_real
                   !!
                   Do ms=1,2
                      If(Abs(mla+ms-1)<=la .And. Abs(mlb+ms-1)<=lb)Then
                         krk(ira+ma,l,irb+mb)=krk(ira+ma,l,irb+mb) &
                              &+   cgoOFKapMu(ms,kapa,mua)&
                              &   *crc(ia+ma+ms-1  ,l,ib+mb+ms-1  )&
                              &   *cgoOFKapMu(ms,kapb,mub)
                      Endif
                   End Do
                End Do
             Enddo
          End Do
       Enddo
    Enddo
    !!
    Call pfree(crc)
    !!
  End Subroutine makekrkgaunt
  !!***********************************************************************
  !! calculate all gaunt coefficients 
  !!    (\chi_{kapa,mua}|Y_L|\chi_{kapb,mub}) and
  !!    (\chi_{kapa,mua}|Y_L\sigma_{z}|\chi_{kapb,mub})
  !! up to the maximum angular momenta lmxa,lmx,lmxa.
  !! All kappa,mu which belong to the l_a and l_b shell are considered.
  !! All Y_L which could contribute are considered.
  !! The pointer array krk will be allocated and must be freed with
  !! a call to pfree after usage
  !! A typical situation is that one needs the Gaunt coefficients for products
  !! of orbitals with potential componenets. Then the maximum angular momentum
  !! of the orbitals is lmxa. The maximum |kappa| will be lmxa+1.
  !! Hence, the maximum L will be 2*(lmxa+1)
  !! All these coefficients are calculated. krk(1,...) are the Y_L and
  !! krk(2,...) are the Y_L \sigma_{z} coefficients.
  !! The needed call will be
  !!        
  !!        call makekrkgaunt2(lmxa,krk,Nlm,kmx)
  !!
  !! The dimensions Nlm and kmx are set on output.
  !!  
  !! To avoid f90 pointer-overhead, one should not declare krk as pointer
  !! inside the calculation subroutine. (build krk outside)
  !! The dimensioning in side the calculation subroutine is
  !!!   krk(2,Nlmx,-kmx:kmx,2*kmx,-kmx:kmx,2*kmx)
  !!
  !! Do not forget to nullify the pointer krk before the call to makekrkgaunt2!
  !!
  !! The usage of the coefficients should include tests to avoid
  !! unesseccary calculations. see calc_rocis.
  !!
  !! The result is stored as real array but
  !! in fact the coefficients for m_L<0 are imaginary, the others are real
  !! m_L is m-qn of the middle (real) spherical harmonic.
  !!
  Subroutine  makekrkgaunt2(lmxa,krk,Nlmx,kmx)
    !!
    Implicit None  
    !!IO
    !! the maximum orbital momentum
    Integer(t_int),Intent(in)    :: lmxa
    !! The size of the second dimension of the 
    Integer(t_int),Intent(out)   :: Nlmx
    !! The maximum kapa
    Integer(t_int),Intent(out)   :: kmx
    !! The coefficients
    Real(t_real),Pointer         :: krk  (:,:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="makekrkgaunt2"
    Logical(t_log) :: exists
    Integer(t_int) :: lmx
    Integer(t_int) :: l,kapa,kapb,ira,irb,mua,mub,mla,mlb,ia,ib,ja,jb
    Integer(t_int) :: la,ma,lb,mb,ms,imua,imub,m,i
    Real(t_real)   :: alphb,alphbq
    !!
    Real(t_real),Pointer ::crc(:,:,:) 
    !!
    !!
    crc => Null()
    !!
    lmx=2_t_int*(lmxa+1_t_int)
    kmx=lmxa+1_t_int
    Nlmx=(lmx+1_t_int)**2_t_int
    !!
    !!
    Call palloc('krk',krk&
         &,1_t_int,2_t_int&
         &,1_t_int,Nlmx&
         &,-kmx,kmx&
         &,1_t_int,2_t_int*kmx&
         &,-kmx,kmx&
         &,1_t_int,2_t_int*kmx&
         &,nmodule,nsubfunc)
    !!
    krk(:,:,:,:,:,:)=0.0_t_real
    !!
    !! aux array
    Call palloc('crc',crc,1_t_int,(kmx+1_t_int)**2_t_int,1_t_int,Nlmx&
         &,1_t_int,(kmx+1_t_int)**2_t_int,nmodule,nsubfunc)
    !!
    !!
    Call makecrcgaunt2(kmx,lmx,kmx,crc)
    !!
    Do l=0,lmx
       Do m=-l,l
          i=l*(l+1)+1+m
          Do kapa=-kmx,kmx
             ira=2*kapa**2+kapa-Abs(kapa)+1
             ia=ira-kapa**2
             ja=2*Abs(kapa)-1
             la=(Abs(2*kapa+1)-1)/2
             Do ma=0,ja
                mua=2*ma-ja
                mla=((mua-1)/2)
                imua=(Abs(2*mua-1)+1)/2
                
                Do kapb=-kmx,kmx
                   irb=2*kapb**2+kapb-Abs(kapb)+1
                   ib=irb-kapb**2
                   jb=2*Abs(kapb)-1
                   lb=(Abs(2*kapb+1)-1)/2
                   !!
                   Do mb=0,jb
                      mub=2*mb-jb
                      mlb=((mub-1)/2)
                      imub=(Abs(2*mub-1)+1)/2
                      !!
                      !!
                      
                      alphb=c_mukk(mub,kapb,kapb,exists)
                      alphbq=c_mukk(mub,-kapb-1,kapb,exists)
                      !!
                      krk(1:2,i,kapa,imua,kapb,imub)=0.0_t_real
                      !!
                      !! kapb-term (potential and sigma-z)
                      Do ms=1,2
                         If(Abs(mla+ms-1)<=la .And. Abs(mlb+ms-1)<=lb)Then
                            krk(1,i,kapa,imua,kapb,imub)=&
                                 & krk(1,i,kapa,imua,kapb,imub) &
                                 &+   cgoOFKapMu(ms,kapa,mua)&
                                 &   *crc(ia+ma+ms-1  ,i,ib+mb+ms-1  )&
                                 &   *cgoOFKapMu(ms,kapb,mub)
                            
                            krk(2,i,kapa,imua,kapb,imub)=&
                                 & krk(2,i,kapa,imua,kapb,imub) &
                                 &+   cgoOFKapMu(ms,kapa,mua)&
                                 &   *crc(ia+ma+ms-1  ,i,ib+mb+ms-1  )&
                                 &   *cgoOFKapMu(ms,kapb,mub)*alphb
                         Endif
                      End Do
                      !! \bar{kapb}-term (sigma-z)
                      Do ms=1,2
                         If(Abs(mla+ms-1)<=la .And. Abs(mlb+ms-1)<=lb)Then
                            krk(2,i,kapa,imua,kapb,imub)=&
                                 & krk(2,i,kapa,imua,kapb,imub) &
                                 &+   cgoOFKapMu(ms,kapa,mua)&
                                 &   *crc(ia+ma+ms-1  ,i,ib+mb+ms-1  )&
                                 &   *cgoOFKapMu(ms,-kapb-1,mub)*alphbq
                         Endif
                      End Do
                   End Do
                Enddo
             End Do
          Enddo
       Enddo
    Enddo
    !!
    Call pfree(crc)
    !!
  End Subroutine makekrkgaunt2
  !!***********************************************************************
  !! This function returns the value of the matrix element
  !! $\langle \chi_{\kappa_1\mu_1}|\L_z|\chi_{\kappa_2\mu_2}\rangle$.
  !! (integrated over a sphere)
  !!
  Real(t_real) Function c_kmuLzkmu(kap1,mu1,kap2,mu2,exists)
    !!
    Implicit none  
    !!IO
    !! $\mu$ quantum number (in units (1/2))
    Integer(t_int),Intent(in)    :: mu1,mu2
    !! $\kappa_1$ quantum number
    Integer(t_int),Intent(in)    :: kap1
    !! $\kappa_2$ quantum number
    Integer(t_int),Intent(in)    :: kap2
    !! does the matrix element exist ?
    Logical(t_log),Intent(out)   :: exists
    !!LOCAL
    !!
    c_kmuLzkmu=0.0_t_real
    exists=.False.
    If(mu1/=mu2) Return
    !!
    If(Abs(mu1)<=2*Abs(kap1)-1 .And. kap1 /= 0)Then
       If(kap1==kap2 )Then
          exists=.True.
          c_kmuLzkmu=Real(mu1,t_real)*&
               &((kap1+1.0_t_real)/(2.0_t_real*kap1+1.0_t_real))
       Else If(-kap1-1==kap2 .And. kap2 /= 0 .And. Abs(mu1)<= 2*Abs(kap2)-1)Then
          exists=.True.
          c_kmuLzkmu=Sqrt(1.0_t_real&
               &-(Real(mu1,t_real)/(2.0_t_real*kap1+1.0_t_real))**2&
               &)*0.5_t_real
       Endif
    Endif
    !!
    Return
    !!
  End Function c_kmuLzkmu
  !!***********************************************************************
  !! The wigner D-functions for half integer j
  !! D^j_{m,m^\prime}=<jm|\hat{D}|jm^\prime>
  !!
  !! the matrix index refers to the order:  -m to m , m half integer
  !! j must be half integer
  !! on input J=2j
  Subroutine  wignerDJ(J,eulang,D,fac,trans)
    !!
    Implicit None  
    !!IO
    Logical(t_log),Intent(in)    :: trans
    Integer(t_int),Intent(in)    :: J !! equals 2j
    Real(t_real),Intent(in)      :: eulang(3),fac(0:J)
    Complex(t_real),Intent(out)  :: D(0:J,0:J)
    !!LOCAL
    Integer(t_int) :: M,MP !! equals 2*m,2*mp
    Integer(t_int) :: l,lp !! equals m+j,mp+j
    Real(t_real)   :: cb,sb
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    cb=Cos(eulang(2)*0.5_t_real)
    sb=Sin(eulang(2)*0.5_t_real)
    !!
    Do l=0,J
       Do lp=0,J
          M=(-J+2*l)
          MP=(-J+2*lp)
          If(trans)Then
             D(lp,l)=Exp(ci*M*eulang(1)*0.5_t_real)&
                  & *djmm(J,M,MP,sb,cb,fac)&
                  & *Exp(ci*MP*eulang(3)*0.5_t_real)
          Else
             D(l,lp)=Exp(-ci*M*eulang(1)*0.5_t_real)&
                  & *djmm(J,M,MP,sb,cb,fac)&
               & *Exp(-ci*MP*eulang(3)*0.5_t_real)
          Endif
       Enddo
    Enddo
    !!
  End Subroutine wignerDJ
  !!***********************************************************************
  !! returns the dj,mm'(beta) functions
  !!
  !! The input quantum numbers are J=2j, M=2m, MP=2m'
  !! with |m|,|m'|<=j
  !! with j,m,m' half integer
  Real(t_real) Function djmm(J,M,MP,sb,cb,fac)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: J   !! equals 2*j
    Integer(t_int),Intent(in)    :: M,MP !! equals 2*m,2*mp
    Real(t_real),Intent(in)      :: cb,sb,fac(0:J)
    !!LOCAL
    Integer(t_int) :: k,k1,k2,l,lp
    Real(t_real)   :: ck,sk,f,d
    !!
    !!
    l=(M+J)/2
    lp=(MP+J)/2
    !! M=(-J+2*l)
    !! MP=(-J+2*lp)
    !!
    d=0.0_t_real
    Do k=Max(-J+l+lp,0_t_int),Min(l,lp)
       f=Real((-1_t_int)**k,t_real)/(fac(k)*fac(l-k)*fac(lp-k)*fac(k+J-l-lp))
       k1=2_t_int*k+J-l-lp
       k2=J-k1
       If(k1 == 0_t_int)Then
          ck=1.0_t_real
       Else
          ck=cb**k1
       Endif
       If(k2 == 0_t_int)Then
          sk=1.0_t_real
       Else
          sk=sb**k2
       Endif
       d=d+f*ck*sk
    Enddo
    djmm=d*Real((-1_t_int)**(l),t_real)&
         &*Sqrt(fac(l)*fac(J-l)*fac(lp)*fac(J-lp))
    !!    
    Return
    !!
  End Function djmm

  !!***********************************************************************
  !! The wigner D-functions for j=1/2
  !! D_{m,m^\prime}=<m|\hat{D}|m^\prime>,   m,m^\prime=+-1/2
  !!
  !! positive m,mp refer to matrix index 1
  !! negative m,mp refer to matrix index 2
  !!
  Subroutine  wignerD2(eulang,D)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: eulang(3)
    Complex(t_real),Intent(out)  :: D(2,2)
    !!LOCAL
    Real(t_real)   :: cb,sb
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Complex(t_real):: em,emp
    !!
    cb=Cos(eulang(2)*0.5_t_real)
    sb=Sin(eulang(2)*0.5_t_real)
    !!
    em=Exp(-ci*eulang(1)*0.5_t_real)
    emp=Exp(-ci*eulang(3)*0.5_t_real)
    D(1,1)= cb*em*emp
    D(1,2)=-sb*em/emp
    D(2,1)= sb/em*emp
    D(2,2)= cb/em/emp
    !!
  End Subroutine wignerD2
  !!***********************************************************************
End Module diracdefs
