Module diracdriver
  !!
  Use precisiondefs
  Use titeration
  Use file_iounits
  Use constants
  Use errors
  Use diracdata
  Use diracpotorb
  Use clock  
  Use trelativistic
  Use finuc
  Use tatom
  Use memory
  Use tmesh
  Use mesh
  Use xc  
  Use versioncontrol
  !!
  Implicit None
  !!
  Private
  !!
  Public :: sccatom,diracsetup
  !!
  !!***********************************************************************
  Character(*),Parameter::nmodule="DIRACDRIVER"
  !!
  Integer(t_int),Parameter ::it_checkin =1_t_int
  Integer(t_int),Parameter ::it_checkout=0_t_int
  
  !! global data for scatom, must be retained in subsequent calls
  Integer(t_int) :: restart,tstep
  Logical(t_log) :: sicstart

Contains
  !!***********************************************************************
  !! iteration control handler
  Subroutine sccatom(Data,Entry)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)       ::Entry
    Type(T_iteration),Intent(inout) :: Data
    !!LOCAL
    Character(*),Parameter::nsubfunc="sccatom"
    Integer(t_int) :: ixc
    !!
    data% erg=0.0_t_real
    !!
    !!
    !!------------------------------------------------------
    If(outlevel>L_many)&
         &Write(outfile,"(a,' ',72('='),/a,' ',15('-'),' START ',a,'(',a,')',a)")&
         & Trim(data%name),Trim(data%name),nsubfunc&
         &,Trim(iteration_entries(Entry))&
         &,Repeat('-',72_t_int-15_t_int-9_t_int-Len(nsubfunc)&
         &-Len(Trim(iteration_entries(Entry))))
    
    If(Entry==it_entry_init)Then
       !!------------------------------------------------------
       !! put initialization of iteration scheme here 
       !! make what ever you want to do first
       !!------------------------------------------------------
       
       If(data% prot) Then
          Write(outfile,"(a,' RADIAL MESH:')")Trim(data%name)
          Write(outfile,"(a,' type:',a)")Trim(data%name)&
               &,Trim(MESH_TYPE(g_atomat% mesh% Type))
          Write(outfile,"(a,' nr:  ',i6)")Trim(data%name),g_atomat% mesh% nr
          Write(outfile,"(a,' r0:  ',g30.15e2)")Trim(data%name)&
               &,g_atomat% mesh% rmin
          Write(outfile,"(a,' r1:  ',g30.15e2)")Trim(data%name)&
               &,g_atomat% mesh% rmax
          Write(outfile,"(a,' ',a,' calculation!')") Trim(data%name),&
               &Trim(relat_name(g_atomat% relativistic))
          
          Write(outfile,"(a,' TOLERANCE:')")Trim(data%name)
          Write(outfile,"(a,' WF:  ',g30.15e2)")Trim(data%name),g_atomat% wftol
          Write(outfile,"(a,' SPIN: ',i2)")Trim(data%name),g_atomat% mspin
          Write(outfile,"(a,' Force positive spin: ',l2)")Trim(data%name)&
               &,g_atomat% force_pos_spin
          Call getXCVersionIndex(ixc) !! get index into xcversion (global xcversion)
          Write(outfile,"(a,' XC version : ',i4,' - ',a)")Trim(data%name)&
               &,getxcversion(),XC_VERSIONS(ixc)
          
          Write(outfile,"(a,' relat. corr.: ',l4)")Trim(data%name)&
               &,getxcrelat()
          
          Write(outfile,"(a,' Confining potential',l4)")Trim(data%name)&
               &,g_atomat% compr
          Write(outfile,"(a,' cf(r) =(r/',g12.5e2,')^',g12.5e2)")&
               &Trim(data%name),g_atomat% r0,g_atomat% exn0
          
          Write(outfile, "(a,' output: basis  density potential overwrite')")&
               &Trim(data%name)
          Write(outfile, "(a,' ',4(l10))")Trim(data%name)&
               &,g_atomat% basout,g_atomat% rhoout&
               &,g_atomat% potout,g_atomat% overwriteout
          Write(outfile,"(a,' ',/,a,' ')")Trim(data%name)&
               &,Trim(data%name)
          
          Write(outfile,"(a,' ',a2)")&
               &Trim(data%name),g_atomat% element
          Write(outfile,"(a,' ionicity: ',f15.6)")Trim(data%name)&
               &,g_atomat% ionicity
          Write(outfile,"(a,' Finite nucleus model: ',a)")&
               &Trim(data%name),Trim(FINUC_TYPE(g_atomat% nuc_type))
          
          Write(outfile,"(a,' Z=',f10.5)")Trim(data%name)&
               &,g_atomat% Z
       Endif
       !! initialize shell data
       Call atominit(g_atomat,data% name,data% prot)

       

       Call watch('total atom calculation',cpu_start&
            &,reset=.True.,sccmesg=data%name)

       
       !! total iteration step counter, for output in atstates,potat
       tstep=0_t_int
       !! overwriteoutat=.False.
       
       !! first calculate in Z/r-potential       
       Call potat(g_atomat,0_t_int,tstep,data%name,data% prot,g_atomat% xreduce)
       
       data%scheme=g_atomat% iteration_control% scheme
       
       !! return and let iteration module make default settings
    Elseif(Entry==it_entry_prepare)Then
       !!------------------------------------------------------
       !! put initialization of iteration control settings here 
       !! make what ever you want to do before chekin of initial data
       !!------------------------------------------------------
       
       !! dimension of vector
       data%vecdim=g_atomat% mesh% nr*g_atomat% mspin
       data%tolerance=g_atomat% iteration_control% tolerance
       data%maxdim=g_atomat% iteration_control% maxdim 
       data%mixing=g_atomat% iteration_control% mixing  
       data%progress=g_atomat% iteration_control% progress
       data%steps   =g_atomat% iteration_control% steps
       data%convcond =CONVCOND_ITERAT
       !! return and let iterate initialize arrays
    Elseif(Entry==it_entry_startdata)Then
       !!------------------------------------------------------
       !! initialize iteration vector
       !!------------------------------------------------------


       Call Checkinout(g_atomat% mspin,g_atomat% mesh&
            &,g_atomat% p_vtr,g_atomat% p_btr,data% vector,data%vecdim&
            &,it_checkin)

       data%reset=.False.
       restart=0_t_int
       sicstart=.True.
       !! return and let iteration module start the scheme
    Elseif(Entry==it_entry_loop)Then
       !!------------------------------------------------------
       !! this is one step of iteration loop which transforms
       !! old vector into new vector
       !!------------------------------------------------------

       tstep=tstep+1_t_int
       !! get potential from vector
       Call checkinout(g_atomat% mspin,g_atomat% mesh&
            &,g_atomat% p_vtr,g_atomat% p_btr,data% vector,data%vecdim&
            &,it_checkout)
       
       !! one cycle of iteration
       
       Call watch('cycle',cpu_start,reset=.True.,sccmesg=data%name)     
       !!
       !! calculate orbitals
       Call atstates(g_atomat,sicstart,tstep,restart,data%name,data% prot&
            &,g_atomat% xreduce)
       !! calculate potential
       Call potat(g_atomat,1_t_int,tstep,data%name,data% prot,g_atomat% xreduce)
       !!

       Call watch('cycle',cpu_stop,delete=.True.,Print=data% prot&
            &,sccmesg=data%name)
       
       !! put potential into vectotr
       Call checkinout(g_atomat% mspin,g_atomat% mesh&
            &,g_atomat% p_vtr,g_atomat% p_btr,data% vector,data%vecdim&
            &,it_checkin)

       !!     check possible convergence problems, restart 'ITERAT'
       data%reset=.False.
       data%letsstop=.False.

       If(Mod(data%step,20_t_int)==0_t_int .And. &
            &Sqrt(data%deviation)>1.e-4_t_real)Then
          data%reset=.True.
          Call errmsg(nmodule,nsubfunc,E_information&
               &,msg="'convergence to slow, restart iteration!'")
          
       Endif
       

       If(restart>20_t_int)Then

          Call errmsg(nmodule,nsubfunc,E_warning&
               &,msg="/,'To many problems in wfdirac, restart iteration!'"&
               &//"/'Please check output of orbital calculation'")
          restart=0_t_int
          data%reset=.True.          
          
       Endif
       
       !! return and let iteration module proceed
    Elseif(Entry==it_entry_exit)Then
       !!------------------------------------------------------
       !! this is called before vector is destroyed
       !! make cleanup work
       !!------------------------------------------------------

       Call watch('total atom calculation',cpu_stop&
            &,delete=.True.,Print=data% prot,sccmesg=data%name)

       
       !! return and let iteration module cleanup
    Elseif(Entry==it_entry_best)Then
       !!------------------------------------------------------
       !! this is called if a new better iteration vector was found
       !! but only in LCITERAT method
       !!------------------------------------------------------
    Else
       !!------------------------------------------------------
       !! some new action was implemented in modeule iteration
       !!------------------------------------------------------
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'unknown iteration entry!'")
    Endif
    !! Ciao
    If(outlevel>L_many)&
         &Write(outfile,"(a,' ',15('-'),' END   ',a&
         &,'(',a,')',a,/,a,' ',72('='))")Trim(data%name),nsubfunc&
         &,Trim(iteration_entries(Entry))&
         &,Repeat('-',72_t_int-15_t_int-9_t_int-Len(nsubfunc)&
         &-Len(Trim(iteration_entries(Entry))))&
         &,Trim(data%name)
    !!
    !!
  End Subroutine sccatom
  !!***********************************************************************

  !!***********************************************************************
  !! put/get potential into/from iteration vector
  !!***********************************************************************
  Subroutine checkinout(mspin,meshat,vtr,btr,vit,nvec,in) 
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)   :: meshat
    Integer(t_int),Intent(in) :: in,mspin,nvec
    Real(t_real),Intent(inout) ::  vtr(meshat% nr),vit(nvec),btr(meshat% nr)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="checkinout"
    Logical(t_log) :: use_v_spin
    Integer(t_int) :: n,ms
    Real(t_real)  :: fsp,f3s2,fs12
    !!
    use_v_spin=.False.
    use_v_spin=.True.
    fs12=0.50_t_real*(mspin-1_t_int)
    f3s2=0.50_t_real*(3.0_t_real-mspin)

    If (in==it_checkout) Then
       vtr=0.0_t_real
       btr=0.0_t_real
       n=1_t_int
       If(use_v_spin)Then
          Do ms=1_t_int,mspin
             fsp=3.0_t_real-2.0_t_real*ms
             vtr=vtr+vit(n:n+meshat% nr-1_t_int)*f3s2
             btr=btr+vit(n:n+meshat% nr-1_t_int)*fs12*fsp
             n=n+meshat% nr
          Enddo
       Else
          vtr=vtr+vit(n:n+meshat% nr-1_t_int)
          n=n+meshat% nr
          If(mspin==2)Then
             btr=btr+vit(n:n+meshat% nr-1_t_int)
             n=n+meshat% nr
          Endif
       Endif
    Else
       n=1_t_int
       If(use_v_spin)Then
          Do ms=1_t_int,mspin
             fsp=3.0_t_real-2.0_t_real*ms
             vit(n:n+meshat% nr-1_t_int)=(vtr+btr*fsp)
             n=n+meshat% nr
          Enddo
       else
          vit(n:n+meshat% nr-1_t_int)=vtr
          n=n+meshat% nr
          If(mspin==2)Then
             vit(n:n+meshat% nr-1_t_int)=btr
             n=n+meshat% nr
          Endif
       Endif
    Endif
    !!
  End Subroutine checkinout
  !!***********************************************************************
  Subroutine diracsetup(atom)
    !!
    Implicit None 
    !!IO
    Type(T_Atom),Intent(inout)   :: atom
    !!LOCAL
    Character(*),Parameter::nsubfunc="diracsetup"
    !!
    
    Call setup_mesh(atom% mesh)


    !! allocate global data
    Call palloc('atom% p_rho',atom% p_rho&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_sp',atom% p_sp&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)

    Call palloc('atom% p_drho',atom% p_drho&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_dsp',atom% p_dsp&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)

    Call palloc('atom% p_d2rho',atom% p_d2rho&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_d2sp',atom% p_d2sp&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)


    Call palloc('atom% p_vtr',atom% p_vtr&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_btr',atom% p_btr&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    

    Call palloc('atom% p_vhar',atom% p_vhar&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_vxc',atom% p_vxc&
         &,1_t_int,atom% mesh% nr&
         &,nmodule,nsubfunc)
    
  End Subroutine diracsetup
  

!!***********************************************************************
  

  
End Module diracdriver
