!! ***********************************************************************
!! DIRACPOTORB:atstates:special
!! ***********************************************************************
Module diracpotorb
  !!
  Use precisiondefs
  Use file_iounits
  Use memory
  Use atomdefs
  Use constants
  Use integrate1d
  Use errors
  Use diracsolver
  Use tmesh
  Use finuc
  Use versioncontrol
  Use diracdefs
  Use mesh
  Use trelativistic
  Use tatom
  Use diracspecial
  Use atompot
  Use matserv
  Use atomdata
  Use bspline
  !!
  !!
  Implicit None 
  !!
  Private
  !!
  Public :: atstates,potat,atominit,make_occus,atcalc_rho&
       &,atom_transfer_fv
  !!*********************************************************************** 
  !!
  !!
  Character(*),Parameter::nmodule="DIRACPOTORB"
  !!
  
Contains
  !!***********************************************************************
  !! make all states
  !! 
  !!
  !! relativistic=SCALAR_RELATIVISTIC mspin=1  :   nonpolarized
  !! relativistic=NON_RELATIVISTIC    mspin=2  :   polarized
  !!
  !!
  !!
  Subroutine atstates(atom,sicstart,tstep,restart,mesg,prot,xreduce)
    !!
    Implicit None
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout) :: atom
    Character(*),Intent(in)    :: mesg
    !! true if first sic step
    Logical(t_log),Intent(inout) ::sicstart
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in) ::tstep !! total iteration step counter
    Integer(t_int),Intent(inout):: restart
    Real(t_real),Intent(in)      :: xreduce
    
    !!LOCAL
    Character(*),Parameter::nsubfunc="atstates"
    !!
    Character(3_t_int) :: mes
    Type(T_DIRACSOLVER)  :: dsdata
    !!
    Logical(t_log) :: rel,relmag,alldone,zoverr_erg_out,sortJ,khsrel,nrel
    !! radial mesh index
    Integer(t_int) :: ir
    !! non relativistic shell index
    Integer(t_int) :: iv0
    !! main quantum number
    Integer(t_int) :: nmain
    !! angular momentum
    Integer(t_int) :: la
    !! kappa value
    Integer(t_int) :: kap
    !! mu qn.
    Integer(t_int) :: mu
    !! J qn.
    Integer(t_int) :: j
    !! shell index for kappa and \bar{kappa}
    Integer(t_int) :: ivr,ivrq
    !! componenet index
    Integer(t_int) :: icomp
    !! spin index
    Integer(t_int) :: ms
    !! dummy
    Integer(t_int) :: jj,nj,ii,m
    !! energy of kappa and \bar{kappa}, HOMO and LUMO for tests
    Real(t_real)   :: E,Eq,homo,lumo
    !! dummy
    Real(t_real)   :: v,val,spc
    !! eigenvalue sum,SIC contrib to ETOT,SIC contr to kinetic
    Real(t_real)   :: Esum,Esic,vnsic
    !! spin factor
    Real(t_real)   :: fsp
    !! occupation number
    Real(t_real)   :: ona
    !! relativistic spin channels
    Integer(t_int) :: nspin
    Real(t_real):: zoverc,c2
    !! charges and energy contribs
    Real(t_real)   :: vtn
    !! compression radius, not needed here
    Real(t_real)   :: rx0
    !! order of ODE
    Integer(t_int) :: odeorder
    !! these data are not used here
    !! d(E)/dW, d(Eq)/dW, kink at largest mesh radius
    Real(t_real)   :: dedw,dedwq,Edum,fx0
    Real(t_real)   :: fx2,rx2,exn2,exn0,gamma
    Real(t_real)   :: rmean
    !!     
    !! automatic arrays
    !! wave functions
    Integer(t_int) :: nr
    Real(t_real)   :: wfn(atom% mesh% nr,4_t_int,2_t_int)
    Real(t_real)   :: gradwf(atom% mesh% nr,2_t_int,4_t_int)
    Real(t_real)   :: wkfn(atom% mesh% nr,4_t_int)
    Real(t_real)   :: dwfdwn(atom% mesh% nr,4_t_int,2_t_int)
    !! aux arrays for sic potential
    Real(t_real)   :: vvv(atom% mesh% nr),bbb(atom% mesh% nr)
    Real(t_real)   :: vsic(atom% mesh% nr)
    !! aux
    Real(t_real)   :: tmp(atom% mesh% nr)
    Real(t_real)   :: texp(atom% mesh% nr)
    
    !!
    Integer(t_int) :: ldo2
    !!
    !Logical(t_log) :: print_gradwf=.True.
    Logical(t_log) :: print_gradwf=.False.
    !Logical(t_log) :: print_kinwf=.True.
    Logical(t_log) :: print_kinwf=.False.
    !!
    Real(t_real),Pointer :: gradfvp(:,:,:,:)
    Real(t_real),Pointer :: gradfvm(:,:,:,:)
    Real(t_real),Pointer :: kinwf(:,:,:,:)
    !!***********************************************************************
    !!
    Nullify(gradfvp)
    Nullify(gradfvm)
    Nullify(kinwf)
    !!
    !! avoid sorting of nominal kappa states with respect to J-weight
    !! to have a simple occupation scheme
    sortJ=.False.
    !!
    nr=atom% mesh% nr
    zoverr_erg_out=.True.
    zoverr_erg_out=.False.
    !!
    Call init_diracsolver(dsdata,nr)
    !!
    nspin=atom% mspin
    rel=(atom% relativistic == FULL_RELATIVISTIC) 
    khsrel=(atom% relativistic == SCALAR_RELATIVISTIC_KH) 
    nrel=.Not.(rel.Or.khsrel)
    relmag=(rel.And.atom% mspin==2)
    If(relmag) Then
       nspin=1_t_int
    Endif
    !!
!!$    If(.True.)Then
!!$       atom% p_vtr(:)=-atom% Z
!!$    Endif

    rx0=1.0_t_real !! not needed here
    !!
    atom% p_fv(:,:,:,:)=0.0_t_real
    atom% fgamv=0.0_t_real
    vsic(:)=0.0_t_real
    !!
    Esic=0.0_t_real
    vnsic=0.0_t_real
    zoverc=atom% Z/speed_of_light
    c2=speed_of_light**2_t_int
    !! 
    If(prot) Then
       Write(outfile,3) Trim(mesg),Trim(relat_name(atom% relativistic))
       Write(outfile,4) Trim(mesg),speed_of_light
       Write(outfile,5) Trim(mesg),atom% sic
       Write(outfile,6) Trim(mesg),atom% sic4f
    End If
    
    
    If(print_kinwf .And. .Not.relmag) Then
       Call palloc("kinwf",kinwf,1_t_int,atom% mesh% nr&
            &,1_t_int,atom% nRv,1_t_int,atom% ncomp,1_t_int,2_t_int&
            &,nmodule,nsubfunc)
    Else
       Call palloc("kinwf",kinwf,1_t_int,1_t_int,1_t_int,1_t_int&
            &,1_t_int,1_t_int,1_t_int,1_t_int,nmodule,nsubfunc)
    Endif
    !!
    If(atom% gam_factor .And. print_gradwf)Then
       Call palloc("gradfvp",gradfvp,1_t_int,atom% mesh% nr&
            &,1_t_int,atom% nRv,1_t_int,atom% ncomp,1_t_int,2_t_int&
            &,nmodule,nsubfunc)
       Call palloc("gradfvm",gradfvm,1_t_int,atom% mesh% nr&
            &,1_t_int,atom% nRv,1_t_int,atom% ncomp,1_t_int,2_t_int&
            &,nmodule,nsubfunc)
    Else
       Call palloc("gradfvp",gradfvp,1_t_int,1_t_int,1_t_int,1_t_int&
            &,1_t_int,1_t_int,1_t_int,1_t_int,nmodule,nsubfunc)
       Call palloc("gradfvm",gradfvm,1_t_int,1_t_int,1_t_int,1_t_int&
            &,1_t_int,1_t_int,1_t_int,1_t_int,nmodule,nsubfunc)
    Endif
    
    
    !! iteration counter
    mes=""
    
    If(.Not.atom% overwriteout)Then
       Write(mes,2)tstep
    Endif
    
    
    
    
    Esum=0.0_t_real
    If(.Not.rel)Then
       If(atom% mspin==2)Then
          If(prot) Write(outfile,876)Trim(mesg),Trim(mesg)
       Else
          If(prot) Write(outfile,877)Trim(mesg),Trim(mesg)
       Endif
    Else
       If(relmag)Then
          If(prot) Write(outfile,879)Trim(mesg),Trim(mesg)
       Else
          If(prot) Write(outfile,878)Trim(mesg),Trim(mesg)
       Endif
    Endif
    !!.......................................................................
    !!     all main numbers
    !!.......................................................................
    ii=0_t_int
    ivrloop: Do ivr=1_t_int,atom% nRv
       !! angular momentum
       la=atom% p_lRv(ivr)
       !! main quantum number
       nmain=atom% p_nnRv(ivr)
       !! kappa qn
       kap=atom% p_kapRv(ivr)
       !! mu qn. for polarized FULL_RELATIVISTIC
       mu=atom% p_muRv(1,ivr)
       !! total angular momentum
       j=2*Abs(kap)-1
       !! nonrelat state index
       iv0=atom% p_ivrTOiv(ivr)
       If(atom% noUnOccupied .And. &
            &All(Abs(atom% p_nroccu(iv0,1:2))<1.0e-8_t_real))Then
          If(prot) Write(outfile,654)Trim(mesg),nmain,lqnchar(la)
          Cycle ivrloop
       Endif
       !!
       If(ii/=iv0)Then
          If(rel)Then
             !If(prot) Write(outfile,789) Trim(mesg),nmain,lqnchar(la)
          Endif
          If(prot) Write(outfile,"(a,' ')")Trim(mesg)
          ii=iv0
       Endif
       !!..................................................................
       !!         all spins
       !!..................................................................
       spinloop: Do ms=1_t_int,nspin
          
          !! calculate state
          alldone=.False.
          If (relmag .And.(Abs(mu)<(2*la+1))) Then
             !! the \bar{kappa}-state for double spinors is calculated
             !! together with kappa-state
             odeorder=4_t_int
             If (j == (2*la+1) ) Then
                ivrq=ivr-j-2
                alldone=.True. ! already calculated
             Else
                ivrq=ivr+j+2
                If (atom% p_muRv(1,ivrq) /= mu) Then
                   Write(errbuf,5020)
                   Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
                Endif
             Endif
          Else
             !! non relativistic, KH-scalar relat, relativistic non polarized,
             !! relativistic polarized and |mu|==l+1/2
             odeorder=2_t_int
             ivrq=ivr
          Endif
          
          !! occupation number of this (degenerate) subshell
          !!TODO: possibly: make an array of occus for all states including
          !!      the degenerate angular states, then the aufbau principle
          !!      may decide about correct choice.
          If(rel)Then
             ona=atom% p_roccu(ivr)
          Else
             ona=atom% p_nroccu(ivr,ms)
          Endif
          
          !!
          fsp=(3.0_t_real-2.0_t_real*ms)*(atom% mspin-1_t_int)
          
          !! do not calculate states with zero occupation
          If(atom% noUnOccupied .And. Abs(ona)<1.e-5_t_real) Cycle spinloop 
          !!...............................................................
          !!           calculate wave function starting with etab, vartab
          !!...............................................................
          
          vvv=atom% p_vtr
          If(atom% mspin==1_t_int)Then
             bbb=0.0_t_real
          Else
             If(rel)Then
                bbb=atom% p_btr
             Else
                bbb=0.0_t_real
                vvv=atom% p_vtr+fsp*atom% p_btr
             Endif
          End If
          
          
          If(atom% sic.And..Not.sicstart)Then
             !!TODO: rel pol sic
             If((atom% sic4f.And.nmain==4_t_int.And.la==3_t_int)&
                  &.Or.(.Not.atom% sic4f))Then
                
                Call sicpot(atom% p_orbdens(:,ivr,ms),atom% mesh,vsic,Edum,mesg&
                     &,xreduce)
                !!
                vvv=vvv+vsic
                !!
                Esic=Esic+Edum*ona
                !Print *,'esic'
                !Print 678,esic,edum,ona
                
             Endif
          Endif
          
          fx0=0.0_t_real
          rx0=1.0_t_real
          exn0=1.0_t_real
          fx2=0.0_t_real
          rx2=1.0_t_real
          exn2=1.0_t_real
          If(atom% compr)Then
             fx2=1.0_t_real
             rx2 =atom% r0
             exn2=atom% exn0
          Endif
          
          
          If(.Not. alldone) Then
             !!
             !!
             Call wfdirac(atom% mesh&
                  &,wfn,wkfn,dwfdwn,dedw,dedwq&
                  &,fx0,rx0,exn0,fx2,rx2,exn2&
                  &,nmain,la,atom% Z&
                  &,vvv,bbb,mu,atom% relativistic,kap&
                  &,atom% p_etab(ivr,ms),atom% p_etab(ivrq,ms)&
                  &,E,Eq&
                  &,atom% p_vartab(1:2,ivr,ms)&
                  &,atom% p_vartab(1:2,ivrq,ms)&
                  &,odeorder,atom% wftol,restart,sortJ,dsdata,atom% nuc_type&
                  &,gamma,atom% gam_factor,gradwf)
             !!
             !! store energies
             atom% p_ev(ivr,ms)=E
             If(odeorder==4)Then
                atom% p_ev(ivrq,ms)=Eq
             Endif
             !! store behaviour
             If(Abs(atom% fgamv)<1.0e-10_t_real)Then
                atom% fgamv=gamma
             Else
                If(Abs(gamma-atom% fgamv)>1.0e-10_t_real)Then
                   Write(errbuf,898)gamma,atom% fgamv 
898                Format("'multiple gamm-values detected, not yet impl&
                        &emented, sorry!',/,' gamma1="&
                        &,g30.15e3," gamma2=",g30.15e3,"'")
                   Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
                Endif
             Endif
             
             
             
             !! store wavefunctions
             Do icomp=1,atom% ncomp
                atom% p_fv(1:nr,ivr,icomp,ms)=wfn(1:nr,icomp,1)
                If (odeorder ==4 )Then
                   atom% p_fv(1:nr,ivrq,icomp,ms)=wfn(1:nr,icomp,2)
                Endif
             Enddo
             !!
             !!
             Call gam_fac_mesh(atom% mesh,atom% gam_factor,atom% fgamv,texp)
             !!
             tmp=0.0_t_real
             Do icomp=1,atom% ncomp
                tmp(1:nr)=tmp(1:nr)+(atom% p_fv(1:nr,ivr,icomp,ms)**2)
             Enddo
             tmp(1:nr)=tmp(1:nr)&
                  &*atom% mesh% p_jmesh(1:nr)*texp&
                  &*atom% mesh% p_rmesh(1:nr)**2
             !!
             Call lag4int(nr,tmp,atom% mesh% hx,rmean,lagint_out)
             !!
             rmean=Sqrt(rmean)
             !!
             !!
             If(atom% gam_factor .And. print_gradwf)Then
                If(nrel)Then
                   icomp=1
                   gradfvp(1:nr,ivr,icomp,ms)=gradwf(1:nr,1,icomp)
                   gradfvm(1:nr,ivr,icomp,ms)=gradwf(1:nr,2,icomp)
                Else If(.Not.relmag)Then
                   Do icomp=1,atom% ncomp
                      gradfvp(1:nr,ivr,icomp,ms)=gradwf(1:nr,1,icomp)
                      gradfvm(1:nr,ivr,icomp,ms)=gradwf(1:nr,2,icomp)
                   Enddo
                Endif
             Endif
             If(print_kinwf)Then
                If(nrel)Then
                   icomp=1
                   kinwf(1:nr,ivr,icomp,ms)=wkfn(1:nr,icomp)
                Else If(.Not.relmag)Then
                   Do icomp=1,atom% ncomp
                      kinwf(1:nr,ivr,icomp,ms)=wkfn(1:nr,icomp)
                   Enddo
                Endif
             Endif
             !!
          Endif!! alldone
          !!
          relatif: If(relmag) Then ! non relat
             !!  
             !! orbital spin density
             !!
             Call gam_fac_mesh(atom% mesh,atom% gam_factor,atom% fgamv,texp)
             !!
             Call calc_rel_orb_sp(kap,la,mu,nr,tmp&
                  &,atom% p_fv(:,ivr,LARGE_COMP1,1)&
                  &,atom% p_fv(:,ivr,SMALL_COMP1,1)&
                  &,atom% p_fv(:,ivr,LARGE_COMP2,1)&
                  &,atom% p_fv(:,ivr,SMALL_COMP2,1))
             !!
             tmp=tmp*texp**2*atom% mesh% p_jmesh
             !!
             Call tlag4int(nr,tmp,atom% mesh% hx,spc)
          Endif relatif
          
          
          !!..............................................................
          !!           output energies
          !!..............................................................
          !! exact energies for comlomb potential
          v=Sqrt(kap**2_t_int-(zoverc)**2_t_int)+Real(nmain-Abs(kap),t_real)
          v=-((1.0_t_real-1.0_t_real/Sqrt((zoverc/v)**2_t_int&
               &+1.0_t_real))*c2)
          !!
          E=atom% p_ev(ivr,ms)
          If(nrel.Or.khsrel) Then
             v=-(0.5_t_real*atom% Z**2_t_int/(nmain)**2_t_int)
             If(atom% mspin==1)Then
                If(zoverr_erg_out)Then
                   If(prot) Write(outfile,2001)Trim(mesg)&
                        &,nmain,lqnchar(la),E,v,(E-v)/v
                Else
                   If(prot) Write(outfile,2002)Trim(mesg)&
                        &,nmain,lqnchar(la),E,ona,rmean
                Endif
             Else
                If(zoverr_erg_out)Then
                   If(prot) Write(outfile,2003)Trim(mesg)&
                        &,nmain,lqnchar(la),3-2*ms,E,v,(E-v)/v
                Else
                   If(prot) Write(outfile,2004)Trim(mesg)&
                        &,nmain,lqnchar(la),3-2*ms,E,ona,rmean
                Endif
             Endif
          Else
             If(relmag)Then
                If(zoverr_erg_out)Then
                   If(prot) Write(outfile,2007)Trim(mesg)&
                        &,nmain,lqnchar(la),j,mu,kap,E,v,(E-v)/v
                Else
                   If(prot) Write(outfile,2007)Trim(mesg)&
                        &,nmain,lqnchar(la),j,mu,kap,E,ona,spc&
                        &,rmean
                Endif
             Else
                If(zoverr_erg_out)Then
                   If(prot) Write(outfile,2005)Trim(mesg)&
                        &,nmain,lqnchar(la),j,kap,E,v,(E-v)/v
                Else
                   If(prot) Write(outfile,2006)Trim(mesg)&
                        &,nmain,lqnchar(la),j,kap,E,ona,rmean
                Endif
             Endif
          Endif
          
          
          
          
          !!..............................................................
          !!           band sum
          !!..............................................................
          Esum=Esum+atom% p_ev(ivr,ms)*ona     !*(3.0_t_real-mspin)
          
          !!..............................................................
          !!           end output wave function 
          !!..............................................................
          !!..............................................................
          !!           sic kinetic contribution
          !!..............................................................
          If(atom% sic)Then
             If((atom% sic4f.And.nmain==4_t_int.And.la==3_t_int)&
                  &.Or.(.Not.atom% sic4f))Then
                ! relat non polarized
                tmp(1)=0.0_t_real
                Do ir=2,nr
                   tmp(ir)=atom% p_orbdens(ir,ivr,ms)*vsic(ir)&
                        &/atom% mesh% p_rmesh(ir)
                Enddo
                Call tlag4int(nr,tmp*atom% mesh% p_jmesh(:)&
                     &,atom% mesh% hx,val)
                
                vnsic=vnsic+(val)*ona
                
             Endif             ! sic4f
          Endif               ! sic
          !!...........................................................
          !!           end SIC
          !!...........................................................
          
       Enddo spinloop
    Enddo ivrloop ! shells ivr
    
    !!.................................................................
    !!       check consistency of energies and occupation
    !!.................................................................
    !!
    checkif: If((.Not.atom% noUnOccupied).And.rel.And.&
         & (speed_of_light<200.0_t_real))Then
       ivr=0
       ivcheckloop: Do iv0=1,atom% nv
          nmain=atom% p_nnv(iv0)
          la=atom% p_lv(iv0)
          homo=-1.0e30_t_real
          lumo= 1.0e30_t_real
          jjloop: Do jj=Max(2-la,1_t_int),2
             kap=la-(jj-1)*(2*la+1)
             j=2*Abs(kap)-1
             If(relmag)Then
                nj=j+1
             Else
                nj=1
             Endif
             Do m=1,nj
                ivr=ivr+1
                E=atom% p_ev(ivr,1)
                ona=atom% p_roccu(ivr)
                If(Abs(ona)<1.e-5_t_real)Then
                   lumo=Min(lumo,E)
                Else
                   homo=Max(homo,E)
                Endif
             Enddo
          Enddo jjloop
          If(homo>lumo+1.e-6_t_real)Then
435          Format("' Occupation numbers are not consistent with aufbau "&
                  &,"principle !',/,'"&
                  &,"nl-shell: ",i1,a1," HOMO=",f12.7," LUMO=",f12.7,"'") 
             Write(errbuf,435) nmain,lqnchar(la),homo,lumo
             Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
          Endif
       End Do ivcheckloop
       
    Endif checkif
    
    If(prot) Write(outfile,13)Trim(mesg)
    
    !!.............................................................
    !!           output wave functions
    !!           approx r->0
    !!.............................................................
    
    !!
    !    If(rel) Call bastest(atom)
    !!
    
    
    If(atom% basout) Then    
       Call statesout(atom,gradfvp,gradfvm,kinwf,mes,print_gradwf&
            &,print_kinwf)
    Endif
    !!
    Call pfree(gradfvp)
    Call pfree(gradfvm)
    Call pfree(kinwf)
    !!
    Call clean_diracsolver(dsdata)
    !!.......................................................................
    !! density
    !!.......................................................................
    
    Call atcalc_rho(atom,mesg,prot)
    
    
    !!     output density
    If(atom% rhoout)Then
       Open(18,file='+atrho'//mes,status='unknown')
       If(.True.)Then
          Write(18,100) Trim(FPLOMAINVERSION)
100       Format('# Version: ',a)
          Write(18,101) atom% element
101       Format('# element: ',a2)
          Call write_mesh_info(18_t_int,atom% mesh)
          Call write_mesh_function(18_t_int,atom% mesh,atom% p_rho/fourpi&
               &,' density ')
          If(atom% mspin==2_t_int)Then
             Call write_mesh_function(18_t_int,atom% mesh,atom% p_sp/fourpi&
                  &,' spin density ')
          Endif
       Else
          Write(18,1501)
          Do ir=1_t_int,nr
             Write(18,500)atom% mesh% p_rmesh(ir),atom% p_rho(ir)/fourpi
          Enddo
          If(atom% mspin==2_t_int)Then
             Write(18,501)
             Write(18,1502)
             Do ir=1_t_int,nr
                Write(18,500)atom% mesh% p_rmesh(ir),atom% p_sp(ir)/fourpi
             Enddo
          End If
       Endif
       Close(18)
       
       If(atom% nonspheredens)Then
          !!
          Open(18,file='+atrhoL'//mes,status='unknown')
          Do ldo2=0,atom% lmax
             Write(18,11501)ldo2*2
             Do ir=1_t_int,nr
                Write(18,500)atom% mesh% p_rmesh(ir)&
                     &,atom% p_rhol(ir,ldo2)/sq4pi
             Enddo
             Write(18,501)
          Enddo
          If(atom% mspin==2_t_int)Then
             Do ldo2=0,atom% lmax
                Write(18,11502)ldo2*2
                Do ir=1_t_int,nr
                   Write(18,500)atom% mesh% p_rmesh(ir)&
                        &,atom% p_spl(ir,ldo2)/sq4pi
                Enddo
                Write(18,501)
             Enddo
          End If
          Close(18)
       End If
       
    End If
    !!.......................................................................
    !!     calculate energies
    !!.......................................................................
    If(prot) Write(outfile,16)Trim(mesg),Esum
    
    !!     int(v*n)
    tmp(1)=0.0_t_real
    Do ir=2_t_int,nr
       tmp(ir)=(atom% p_rho(ir)*atom% p_vtr(ir)+atom% p_sp(ir)*atom% p_btr(ir))&
            &/atom% mesh% p_rmesh(ir)*atom% mesh% p_jmesh(ir)
    Enddo
    Call tlag4int(nr,tmp,atom% mesh% hx,vtn)
    !!
    If(prot .And. atom% sic) Then
       Write(outfile,18)Trim(mesg),vnsic
       Write(outfile,19)Trim(mesg),Esic-vnsic
       Write(outfile,191)Trim(mesg),Esic
    Endif
    
    !If(prot) Write(outfile,20)Trim(mesg),vtn,vnsic
    If(prot) Write(outfile,21)Trim(mesg),vtn+vnsic
    If(prot) Write(outfile,22)Trim(mesg),Esum-vtn-vnsic
    atom% Etot=Esum-vtn-vnsic+Esic
    !!
    sicstart=.False.
    !!
    If(atom% standalone.And.debugfun(nmodule,nsubfunc,'special')) Then
       Call special_orb(atom,xreduce)
    Endif
    !!
    Return
    !!
2   Format(i3.3) 
3   Format(a,' Relativistic   : ',a) 
4   Format(a,' speed of light : ',f12.5) 
5   Format(a,' SIC            : ',l1) 
6   Format(a,' SIC4F          : ',l1) 
13  Format(a,' ----------------------------------------------------') 
16  Format(a,' SUM OF EIGENVALUES : ',f30.15)
18  Format(a,' V*N sic            : ',f30.15)
191 Format(a,' Esic               : ',f30.15)
19  Format(a,' SIC total          : ',f30.15)
    !20  Format(a,' ',3(g30.15e3,2x))
21  Format(a,' V*N                : ',f30.15)
22  Format(a,' KINETIC ENERGY     : ',f30.15)
500 Format(2(g30.15e3,2x))
501 Format(' ') 
1501 Format('# density') 
11501 Format('# density l,m=',i3,',0') 
11502 Format('# spin density l,m=',i3,',0') 
1502 Format('# spin density') 
654 Format(a,' Occus zero, ignore the shell ',i1,a1)
    !789 Format(a,' Shell: ',i1,a1) 
877 Format(a,' ',72('-'),/,a,' STATE                         &
         &ENERGY       OCCUPATION    Sqrt(<r^2>)') 
876 Format(a,' ',72('-'),/,a,' STATE      SPIN               &
         &ENERGY       OCCUPATION    Sqrt(<r^2>)') 
878 Format(a,' ',72('-'),/,a,' STATE             KAP         &
         &ENERGY       OCCUPATION    sqrt(<r^2>)') 
879 Format(a,' ',72('-'),/,a,' STATE       J_z   KAP         &
         &ENERGY       OCCUPATION    SPIN Char.  Sqrt(<r^2>)    ') 
2002 Format(a,' ',2x,i1,a1,17x,f19.10,1x,f12.7,1x,f12.7)
2004 Format(a,' ',2x,i1,a1,7x,i2,'/2',6x,f19.10,1x,f12.7,1x,f12.7)
    
    
2006 Format(a,' ',2x,i1,a1,i1,'/2',10x,i3,1x,f19.10,1x,f12.7,1x,f12.7)
2007 Format(a,' ',2x,i1,a1,i1,'/2',4x,i2,'/2',2x,i3,1x,f19.10,1x,f12.7&
          &,1x,f12.7,1x,f12.7)
2001 Format(a,' ',2x,i1,a1,17x,3(f30.15,1x)) 
2003 Format(a,' ',2x,i1,a1,7x,i2,'/2',6x,3(f30.15,1x)) 
2005 Format(a,' ',2x,i1,a1,i1,'/2',10x,i3,1x,3(f30.15,1x))
5020 Format("'error in numbering of states!'")
    !!     
  End Subroutine atstates
  !!     end subroutine atstates
  !!***********************************************************************
  !!***********************************************************************
  !! calculate potentail from density, see potat1
  !! 
  !!
  Subroutine potat(atom,init,tstep,mesg,prot,xreduce)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(inout) :: atom
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in) :: init,tstep
    Character(*),Intent(in)      :: mesg
    Real(t_real),Intent(in)      :: xreduce
    !!LOCAL
    !Character(*),Parameter::nsubfunc="potat"
    Integer(t_int) :: i,lun,iZ,n0
    Real(t_real) :: s0,ri!,r,r0,x,v
    Logical(t_log) :: exist,reinit
    Character(3_t_int) :: mes
    !Character(*),Parameter :: fmt12="(3(e30.15e3,2x))"
    !!
    Type(T_Mesh)         :: imesh
    Real(t_real)   :: tt(50),tb(50)
    !!
    
    initif: If(init==0_t_int) Then
       !!
       reinit=.True.
       !!
       Inquire(file='=.atpot',exist=exist)
       existif:If(exist.And.atom% standalone)Then
          !!
          Call read_atpot(atom,reinit,mesg,prot)
          !! force nuclear condition in order to be able to use
          !! =.atpot files with differing Z as starting value
          atom% p_vtr(1) = -atom% Z
          !!
          !!
       Endif existif
       !!
       If(reinit)Then
          iZ=Int(atom% Z+0.1_t_real,t_int)
          
          
          s0=0.2_t_real
          If(atom% mspin==1_t_int) s0=0.0_t_real
          Do i=1_t_int,atom% mesh% nr
             ri=atom% mesh% p_rmesh(i)
             !!
             atom% p_btr(i)=-(s0*Exp(-(ri*0.5))*ri)
             !!          atom% p_btr(i)=-s0*r(i)
             !!          atom% p_btr(i)=0.0_t_real
             !!          atom% p_btr(i)=0.1*r(i)
          Enddo
          !!
          Call getatpotfit(iZ,atom% mspin,atom% Z,atom% mesh,atom% p_vtr&
               &,atom% p_btr&
               &,atom% relativistic,atom% vca)
          !!
       Endif
    Else initif
       !!
       !!       recalculate the potential from the density and spin density
       !!
       atom% p_vtr=0.0_t_real
       atom% p_btr=0.0_t_real
       
       atom% p_vhar=0.0_t_real
       atom% p_vxc =0.0_t_real
       
       Call potatcoul(atom% mesh,atom% p_rho,atom% p_vtr,atom% Z,atom% Z&
            &,atom% Etot,prot,mesg)
       atom% p_vhar=atom% p_vtr+atom% Z
       
       Call potatxc(atom% mesh&
            &,atom% p_rho,atom% p_sp,atom% p_drho,atom% p_dsp&
            &,atom% p_d2rho,atom% p_d2sp&
            &,atom% p_vtr,atom% p_btr&
            &,atom% mspin,atom% Etot,prot,atom% standalone,mesg,xreduce)
       
       atom% p_vxc=atom% p_vtr-atom% p_vhar+atom% Z
       
    End If initif
    
    !!     
    
    mes=""
    If(.Not.atom% overwriteout)Then
       Write(mes,2)tstep
    Endif
    
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_vxc,12,'+dvxc')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_vhar,12,'+dvhar')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_vtr,12,'+dvtr')
    
    If(atom% potout.And.atom% standalone)Then
       lun=16_t_int
       Open(lun,file='=.atpot'//mes,status='unknown')
       !!
       Write(lun,100) Trim(FPLOMAINVERSION)
100    Format('# Version: ',a)
       Write(lun,101) atom% element
101    Format('# element: ',a2)
       Call write_mesh_info(lun,atom% mesh)
       !!
       Call write_mesh_function(lun,atom% mesh,atom% p_vtr,' v*r ')
       If(atom% mspin==2_t_int)Then
          Call write_mesh_function(lun,atom% mesh,atom% p_btr,' b*r ')
       Endif
       Close(lun)
       
       
       If(.False.)Then
          Call init_mesh(imesh)
          imesh% Type=POW_MESH
          imesh% rmax=20.0_t_real
          imesh% alpha=3
          imesh% rmin=1.0e-4_t_real
          imesh% nr=50
          imesh% with_zero=.True.
          Call setup_mesh(imesh)
          
          Call mreinter(8_t_int,atom% mesh,atom% p_vtr,imesh,tt)
          Call mreinter(8_t_int,atom% mesh,atom% p_btr,imesh,tb)
          
          n0=imesh% nr
          Do i=imesh% nr,1,-1
             If(Abs(tt(i))>1.0e-6_t_real)Then
                n0=i
                Exit
             Endif
          Enddo
          
          
          iZ=Int(atom% Z+0.1_t_real,t_int)
          Open(12,file='cc')
          If(atom% relativistic == NON_RELATIVISTIC) Then
             Write(12,"('!!',/,'Data ncoefnrel(',i3,') / ',i2,' /')") iZ,n0
             Write(12,"('Data vscoefnrel(1:',i2,',',i3,') /&',/,'& ')",advance='no') n0,iZ
          Else
             Write(12,"('!!',/,'Data ncoeffrel(',i3,') / ',i2,' /')") iZ,n0
             Write(12,"('Data vscoeffrel(1:',i2,',',i3,') /&',/,'& ')",advance='no') n0,iZ
          Endif
          Do i=1,n0
             If(i>1) Write(12,"(',')",advance='no')
             Write(12,"(f13.6,'_t_real')",advance='no')tt(i)
             If(Mod(i,3_t_int)==0 .And. i< n0)Then
                Write(12,"(' &',/,'&')",advance='no')
             Endif
          Enddo
          Write(12,"('/')")
          
          If(atom% relativistic == NON_RELATIVISTIC) Then
             Write(12,"('Data bscoefnrel(1:',i2,',',i3,') /&',/,'& ')",advance='no') n0,iZ
          Else
             Write(12,"('Data bscoeffrel(1:',i2,',',i3,') /&',/,'& ')",advance='no') n0,iZ
          Endif
          Do i=1,n0
             If(i>1) Write(12,"(',')",advance='no')
             Write(12,"(f13.6,'_t_real')",advance='no')tb(i)
             If(Mod(i,3_t_int)==0 .And. i< n0)Then
                Write(12,"(' &',/,'&')",advance='no')
             Endif
          Enddo
          Write(12,"('/')")
          Close(12)
          
       Endif
       
!!$       Open(lun,file='=.atpot_nuc',status='unknown')
!!$       r0=nuc_max_rad(atom% nuc_type,atom% Z)
!!$       Do i=1,atom% mesh% nr
!!$          r=atom% mesh% p_rmesh(i)
!!$          x=r/r0
!!$          v=(atom% p_vtr(i)-atom% Z*nuc_zeta(x,atom% nuc_type))
!!$          Write(lun,89)r,v
!!$89        Format(2(e30.15e3,2x))
!!$       Enddo
!!$       Close(lun)
!!$       Open(lun,file='=.atpotxc'//mes,status='unknown')
!!$       !!
!!$       Write(lun,100) Trim(FPLOMAINVERSION)
!!$       Write(lun,101) atom% element
!!$       Call write_mesh_info(lun,atom% mesh)
!!$       !!
!!$       Call write_mesh_function(lun,atom% mesh,atom% p_vxc,' vxc*r ')
!!$       Close(lun)
    End If
    !!
    !!
    Return
    !!
2   Format(i3.3)
    !!
  End Subroutine potat
  !!***********************************************************************
  Subroutine statesout(atom,gradfvp,gradfvm,kinwf,mes&
       &,print_gradwf,print_kinwf)
    !!
    Implicit None
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout) :: atom
    Logical(t_log),Intent(in)    :: print_gradwf,print_kinwf
    Character(*),Intent(in)      :: mes
    Real(t_real),Intent(in)      :: gradfvp(:,:,:,:)
    Real(t_real),Intent(in)      :: gradfvm(:,:,:,:)
    Real(t_real),Intent(in)      :: kinwf(:,:,:,:)
    !!LOCAL
    Logical(t_log) :: rel,relmag,relnsp,khsrel,grads,nrel
    Integer(t_int) :: nspin,ivr,la,mu,ms,kap,nmain,j,ir
    Real(t_real)   :: r,t
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
!!$    Integer(t_int) :: kk,jj,i
!!$    Real(t_real)   :: g(atom% mesh% nr,2)
!!$    Real(t_real)   :: f(atom% mesh% nr,2)
!!$    Real(t_real)   :: gs(atom% mesh% nr)
!!$    Real(t_real)   :: gd(atom% mesh% nr)
!!$    Real(t_real)   :: fs(atom% mesh% nr)
!!$    Real(t_real)   :: fd(atom% mesh% nr)
!!$    Character(100)   :: str
    !!
    grads=(atom% gam_factor .And. print_gradwf)
    !!
    !!
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,atom% fgamv,texp)
    !!
    nspin=atom% mspin
    rel=(atom% relativistic == FULL_RELATIVISTIC) 
    khsrel=(atom% relativistic == SCALAR_RELATIVISTIC_KH) 
    nrel=.Not.(rel.Or.khsrel)
    relnsp=(rel.And.atom% mspin==1)
    relmag=(rel.And.atom% mspin==2)
    If(relmag) Then
       nspin=1_t_int
    Endif
    !!
    !!
    If(nrel) Then
       Open(19,file='+bas'//mes,status='unknown')
    Else
       Open(19,file='+bas.large'//mes,status='unknown')
    Endif
    If(.Not.nrel) Open(29,file='+bas.small'//mes,status='unknown')
    If(khsrel) Open(30,file='+bas.small2'//mes,status='unknown')
    If(relmag) Then
       Open(20,file='+bas.largeq'//mes,status='unknown')
       Open(30,file='+bas.smallq'//mes,status='unknown')
    Endif
    !!
    If(grads) Then
       If(nrel) Then
          Open(39,file='+bas.grad_plus'//mes,status='unknown')
          Open(49,file='+bas.grad_minus'//mes,status='unknown')
       Else 
          Open(39,file='+bas.grad_plus.large'//mes,status='unknown')
          Open(49,file='+bas.grad_minus.large'//mes,status='unknown')
       Endif
       If(relnsp.Or.khsrel)Then
          Open(38,file='+bas.grad_plus.small'//mes,status='unknown')
          Open(48,file='+bas.grad_minus.small'//mes,status='unknown')
       Endif
       If(khsrel) Then
          Open(37,file='+bas.grad_plus.small2'//mes,status='unknown')
          Open(47,file='+bas.grad_minus.small2'//mes,status='unknown')
       Endif
    Endif
    If(print_kinwf )Then
       If(nrel)Then
          Open(59,file='+bas.kin'//mes,status='unknown')
       Else 
          Open(59,file='+bas.kin.large'//mes,status='unknown')
       Endif
       If(relnsp.Or.khsrel) Open(69,file='+bas.kin.small'//mes,status='unknown')
       If(khsrel) Open(70,file='+bas.kin.small2'//mes,status='unknown')
    Endif
    
    
    !!
    ivrloop: Do ivr=1_t_int,atom% nRv
       !! main momentum
       nmain=atom% p_nnRv(ivr)
       !! angular momentum
       la=atom% p_lRv(ivr)
       !! kappa qn
       kap=atom% p_kapRv(ivr)
       j=2*Abs(kap)-1
       !! mu qn. for polarized FULL_RELATIVISTIC
       mu=atom% p_muRv(1,ivr)
       !!
       spinloop: Do ms=1_t_int,nspin
          
          !! header
          If(nrel) Then
             If(atom% mspin==1)Then
                Write(19,400)nmain,lqnchar(la)
                If(grads ) Then
                   Write(39,400)nmain,lqnchar(la)
                   Write(49,400)nmain,lqnchar(la)
                Endif
                If(print_kinwf) Write(59,400)nmain,lqnchar(la)
             Else
                Write(19,401)nmain,lqnchar(la),3-2*ms
                If(grads ) Then
                   Write(39,401)nmain,lqnchar(la),3-2*ms
                   Write(49,401)nmain,lqnchar(la),3-2*ms
                End If
                If(print_kinwf) Write(59,401)nmain,lqnchar(la),3-2*ms
             Endif
          Else If (khsrel) Then
             If(atom% mspin==1)Then
                Write(19,400)nmain,lqnchar(la)
                Write(29,400)nmain,lqnchar(la)
                Write(30,400)nmain,lqnchar(la)
                If(grads ) Then
                   Write(39,400)nmain,lqnchar(la)
                   Write(38,400)nmain,lqnchar(la)
                   Write(37,400)nmain,lqnchar(la)
                   Write(49,400)nmain,lqnchar(la)
                   Write(48,400)nmain,lqnchar(la)
                   Write(47,400)nmain,lqnchar(la)
                Endif
                If(print_kinwf) Then
                   Write(59,400)nmain,lqnchar(la)
                   Write(69,400)nmain,lqnchar(la)
                   Write(70,400)nmain,lqnchar(la)
                Endif
             Else
                Write(19,401)nmain,lqnchar(la),3-2*ms
                Write(29,401)nmain,lqnchar(la),3-2*ms
                Write(30,401)nmain,lqnchar(la),3-2*ms
                If(grads ) Then
                   Write(39,401)nmain,lqnchar(la),3-2*ms
                   Write(38,401)nmain,lqnchar(la),3-2*ms
                   Write(37,401)nmain,lqnchar(la),3-2*ms
                   Write(49,401)nmain,lqnchar(la),3-2*ms
                   Write(48,401)nmain,lqnchar(la),3-2*ms
                   Write(47,401)nmain,lqnchar(la),3-2*ms
                End If
                If(print_kinwf)Then
                   Write(59,401)nmain,lqnchar(la),3-2*ms
                   Write(69,401)nmain,lqnchar(la),3-2*ms
                   Write(70,401)nmain,lqnchar(la),3-2*ms
                Endif
             Endif
          Else
             If(relmag)Then
                Write(19,403)nmain,lqnchar(la),j,mu
                Write(20,403)nmain,lqnchar(la),j,mu
                Write(29,403)nmain,lqnchar(la),j,mu
                Write(30,403)nmain,lqnchar(la),j,mu
             Else
                Write(19,402)nmain,lqnchar(la),j
                Write(29,402)nmain,lqnchar(la),j
                If(grads ) Then
                   Write(39,402)nmain,lqnchar(la),j
                   Write(38,402)nmain,lqnchar(la),j
                   Write(49,402)nmain,lqnchar(la),j
                   Write(48,402)nmain,lqnchar(la),j
                Endif
                If(print_kinwf) Then
                   Write(59,402)nmain,lqnchar(la),j
                   Write(69,402)nmain,lqnchar(la),j
                Endif
             Endif
          End If
          !! functions 
          Do ir=1_t_int,atom% mesh% nr
             r=atom% mesh% p_rmesh(ir)
             t=texp(ir)
             !! wave functions
             Write(19,500) r,atom% p_fv(ir,ivr,LARGE_COMP1,ms)*t
             If(.Not.nrel) Write(29,500) r,atom% p_fv(ir,ivr,SMALL_COMP1,ms)*t
             If(khsrel) Write(30,500) r,atom% p_fv(ir,ivr,LARGE_COMP2,ms)*t
             If(relmag)Then
                Write(20,500)r,atom% p_fv(ir,ivr,LARGE_COMP2,ms)*t
                Write(30,500)r,atom% p_fv(ir,ivr,SMALL_COMP2,ms)*t
             Endif
             !! gradient functions
             If(grads)Then
                If(.Not.relmag)Then
                   Write(39,500) r,gradfvp(ir,ivr,LARGE_COMP1,ms)*t
                   Write(49,500) r,gradfvm(ir,ivr,LARGE_COMP1,ms)*t
                Endif
                If(relnsp.Or.khsrel)Then
                   Write(38,500) r,gradfvp(ir,ivr,SMALL_COMP1,ms)*t
                   Write(48,500) r,gradfvm(ir,ivr,SMALL_COMP1,ms)*t
                Endif
                If(khsrel)Then
                   Write(37,500) r,gradfvp(ir,ivr,LARGE_COMP2,ms)*t
                   Write(47,500) r,gradfvm(ir,ivr,LARGE_COMP2,ms)*t
                Endif
             Endif
             !! kinetic functions
             If(print_kinwf) Then
                If( .Not.relmag) Write(59,500) r,kinwf(ir,ivr,LARGE_COMP1,ms)*t
                If(relnsp.Or.khsrel) Write(69,500) r,kinwf(ir,ivr,SMALL_COMP1,ms)*t
                If(khsrel) Write(70,500) r,kinwf(ir,ivr,LARGE_COMP2,ms)*t
             Endif
          Enddo
          !!
          Write(19,501)
          If(.Not.nrel) Write(29,501)
          If(khsrel) Write(30,501)
          If(relmag)Then
             Write(20,501)
             Write(30,501)
          Endif
          !!
          If(grads)Then
             If(.Not.relmag)Then
                Write(39,501)
                Write(49,501)
             Endif
             If(relnsp.Or.khsrel)Then
                Write(38,501)
                Write(48,501)
             Endif
             If(khsrel)Then
                Write(37,501)
                Write(47,501)
             Endif
          Endif
          !!
          If(print_kinwf) Then
             If(.Not.relmag)Write(59,501)
             If(relnsp.Or.khsrel) Write(69,501)
             If(khsrel) Write(70,501)
          Endif
          !!
       End Do spinloop
       !!
    End Do ivrloop
    !!
    Close(19)
    If(.Not.nrel) Close(29)
    If(khsrel) Close(30)
    If(relmag)Then
       Close(20)
       Close(30)
    Endif
    !!
    If(grads)Then
       If(.Not.relmag)Then
          Close(39)
          Close(49)
       Endif
       If(relnsp.Or.khsrel)Then
          Close(38)
          Close(48)
       Endif
       If(khsrel)Then
          Close(37)
          Close(47)
       Endif
    Endif
    !!
    If(print_kinwf) Then
       If(.Not.relmag)Close(59)
       If(relnsp.Or.khsrel) Close(69)
       If(khsrel) Close(70)
    Endif
    !!
    !!
!!$    !! avergaes
!!$    !!
!!$    ms=1
!!$    ivr=1
!!$    Do While(ivr<=atom% nRv)
!!$       !! main momentum
!!$       nmain=atom% p_nnRv(ivr)
!!$       !! angular momentum
!!$       la=atom% p_lRv(ivr)
!!$       !! kappa qn
!!$       kap=atom% p_kapRv(ivr)
!!$       j=2*Abs(kap)-1
!!$       !!
!!$       Write(outfile,400)nmain,lqnchar(la)
!!$       gs=0.0_t_real
!!$       gd=0.0_t_real
!!$       fs=0.0_t_real
!!$       fd=0.0_t_real
!!$       g=0.0_t_real
!!$       f=0.0_t_real
!!$       i=0
!!$       jjloop:Do jj=Max(2-la,1_t_int),2
!!$          kk=la-(jj-1)*(2*la+1)
!!$          j=2*Abs(kk)-1
!!$          Write(outfile,402) nmain,lqnchar(la),j
!!$ !!TODO: gam
!!$          g(:,i+1)=atom% p_fv(:,ivr+i,LARGE_COMP1,ms)
!!$          f(:,i+1)=atom% p_fv(:,ivr+i,SMALL_COMP1,ms)
!!$          gs=gs+atom% p_fv(:,ivr+i,LARGE_COMP1,ms)*Abs(kk)/(2*la+1)
!!$          fs=fs+atom% p_fv(:,ivr+i,SMALL_COMP1,ms)*Abs(kk)/(2*la+1)
!!$          gd=gd+atom% p_fv(:,ivr+i,LARGE_COMP1,ms)*Sign(1_t_int,kk)/(2*la+1)
!!$          fd=fd+atom% p_fv(:,ivr+i,SMALL_COMP1,ms)*Sign(1_t_int,kk)/(2*la+1)
!!$          i=i+1
!!$       End Do jjloop
!!$       !! 
!!$       Write(str,"('+state.large',i1,a1)")nmain,lqnchar(la)
!!$       Open(20,file=Trim(str),status='unknown')
!!$       Do ir=1,atom% mesh% nr
!!$          Write(20,700)atom% mesh%p_rmesh(ir),g(ir,1),g(ir,2),gs(ir),gd(ir)
!!$       Enddo
!!$       Close(20)
!!$       Write(str,"('+state.small',i1,a1)")nmain,lqnchar(la)
!!$       Open(20,file=Trim(str),status='unknown')
!!$       Do ir=1,atom% mesh% nr
!!$          Write(20,700)atom% mesh%p_rmesh(ir),f(ir,1),f(ir,2),fs(ir),fd(ir)
!!$       Enddo
!!$       Close(20)
!!$       !!
!!$       ivr=ivr+Min(la+1,2_t_int)
!!$    Enddo
    !!
    !!
    Return
    !!
400 Format('# ',i1,a1)
401 Format('# ',i1,a1,' ms=',i2,'/2')
402 Format('# ',i1,a1,i1,'/2')
403 Format('# ',i1,a1,i1,'/2',2x,i3,'/2')
500 Format(2(g30.15e3,2x))
!!$700 Format(5(g30.15e3,2x))
501 Format(' ') 
    !!
  End Subroutine statesout
  !!***********************************************************************
  !! initialize atomic calculaiton, define shell occupation
  !! 
  !! 
  !!
  Subroutine atominit(atom,mesg,prot)
    !!
    Implicit None
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout) :: atom
    Character(*),Intent(in)    :: mesg
    Logical(t_log),Intent(in)    :: prot
    !!LOCAL
    !Character(*),Parameter::nsubfunc="atominit"
    !Logical(t_log) :: rel
    !Integer(t_int) :: nmain,j,l,iv,ivr
    !Integer(t_int) :: m,mv,jj,kap,mu
    !!----------------------------------------------------------------------
    !!
    !!
    !!
    !!
    !!
    Call make_occus(atom,mesg,prot)
    !!
    
    !!
    !!
    Call alloc_atom(atom)
    !!
    !! 
    !!
!!$    Do ivr=1,atom% nRv
!!$       nmain=atom% p_nnRv(ivr)
!!$       l=atom% p_lRv(ivr)
!!$       kap=atom% p_kapRv(ivr)
!!$       j=2*Abs(kap)-1
!!$       mu=atom% p_muRv(1,ivr)
!!$       alph=c_mukk(mu,kap,kap,exists)
!!$       Write(outfile,6) nmain,lqnchar(l),j,mu,alph
!!$6      Format(i1,a1,i1,'/2',i3,'/2',2x,f12.7) 
!!$    Enddo
    !!
    !!
    Return
    !!
  End Subroutine atominit
  !!***********************************************************************
  Subroutine read_atpot(atom,reinit,mesg,prot)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(inout)   :: atom
    Logical(t_log),Intent(inout) :: reinit
    Logical(t_log),Intent(in)    :: prot
    Character(*),Intent(in)      :: mesg
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_atpot"
    Integer(t_int) :: lun,j,j2
    Integer :: eof
    Type(T_Mesh)           :: tmesh
    Character(10000)   :: buffer
    Character(5000)    :: ele
    Real(t_real),Pointer :: vtmp(:)
    !!
    If(prot)Write(outfile,10)Trim(mesg),Trim(mesg),Trim(mesg)
    !!
    Nullify(vtmp)
    Call init_mesh(tmesh)
    !!
    lun=12_t_int
    !!
    atom% p_vtr(:)=0.0_t_real
    atom% p_btr(:)=0.0_t_real
    !!
    Open(lun,file='=.atpot',status='old')
    !!
    !!
    readloop: Do 
       Read(lun,3,iostat=eof) buffer
       If(eof /=0 ) Exit readloop
       !!
       j=Index(buffer,'Version: ')
       If(j > 0) Then
          !!j=j+9
          !!Print *,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'element: ')
       If(j > 0) Then
          j=j+8
          Read(buffer(j:),*)ele
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start mesh>')
       If(j > 0) Then
          Call read_mesh_info(lun,tmesh)
          Call setup_mesh(tmesh)
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start function')
       If(j > 0) Then
          j=j+15
          !Print *,'<',Trim(buffer(j:)),'>'
          !!
          Call palloc('vtmp',vtmp ,1_t_int,tmesh% nr ,nmodule,nsubfunc)
          Call read_mesh_function(lun,vtmp)
          !!
          j2=Index(buffer(j:),'v*r')
          If(j2>0)Then
             !Print *,'reinter v*r'
             Call mreinter(8_t_int,tmesh,vtmp,atom% mesh,atom% p_vtr)
          Endif
          j2=Index(buffer(j:),'b*r')
          If(j2>0 .And. atom% mspin==2 )Then
             !Print *,'reinter b*r'
             Call mreinter(8_t_int,tmesh,vtmp,atom% mesh,atom% p_btr)
          Endif
          Call pfree(vtmp)
          Cycle readloop
       Endif
       !!
       !Print *,'[',Trim(buffer),']'
       !!
    Enddo readloop
    !!
    !!
    Close(lun)
    !!
    Call cleanup_mesh(tmesh)
    !!
    !Print *,'ele=',Trim(ele),atom% element
    If(Trim(ele) == Trim(atom% element) ) reinit=.False.
    Return
    !!
3   Format(a)
10  Format(a,' ',/,a&
         &,' Found file =.atpot, will use it as start potential!',/&
         &,a,' ')
    !!
  End Subroutine read_atpot
  !!***********************************************************************
  Subroutine alloc_atom(atom)
    !!
    Implicit None  
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout) :: atom
    !!LOCAL
    Character(*),Parameter::nsubfunc="alloc_atom"
    Logical(t_log) :: rel,relmag
    Integer(t_int) :: ivr,nmain
    Real(t_real)   :: Z,E
    !!
    rel=(atom% relativistic==FULL_RELATIVISTIC)
    relmag=(rel .And. atom% mspin==2_t_int)
    !!
    Z=atom% Z
    !!
    !!     prepare initial energies, and fit values
    !!
    Call palloc('atom% p_etab',atom% p_etab &
         &,1_t_int,atom% nRv&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_vartab',atom% p_vartab &
         &,1_t_int,2_t_int&
         &,1_t_int,atom% nRv&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    Do ivr=1_t_int,atom% nRv
       nmain=atom% p_nnRv(ivr)
       E=-(0.5_t_real*Z**2_t_int/(nmain)**2_t_int)
       atom% p_etab(ivr,1:2)=E
       atom% p_vartab(1:2,ivr,1:2)=1.0_t_real
    Enddo
    !!
    !! allocate arrays to store the solutions
    !! this is not really nesseccary, but in this way we have the
    !! information at hand when further things shall be implemented
    !!
    !!
    !! determine dimensions of arrays
    atom% ncomp=2
    If(atom% relativistic == SCALAR_RELATIVISTIC_KH) atom% ncomp=3
    If(relmag) atom% ncomp=4
    !!
    Call palloc('atom% p_ev',atom% p_ev &
         &,1_t_int,atom% nRv&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_fv',atom% p_fv &
         &,1_t_int,atom% mesh% nr&
         &,1_t_int,atom% nRv&
         &,1_t_int,atom% ncomp&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_orbdens',atom% p_orbdens &
         &,1_t_int,atom% mesh% nr&
         &,1_t_int,atom% nRv&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    
    If(atom% nonspheredens)Then
       Call palloc('atom% p_rhol',atom% p_rhol &
            &,1_t_int,atom% mesh% nr&
            &,0_t_int,atom% lmax&
            &,nmodule,nsubfunc)
       Call palloc('atom% p_spl',atom% p_spl &
            &,1_t_int,atom% mesh% nr&
            &,0_t_int,atom% lmax&
            &,nmodule,nsubfunc)
    Endif
    atom% p_ev(:,:)=0.0_t_real
    atom% p_fv(:,:,:,:)=0.0_t_real
    atom% p_orbdens(:,:,:)=0.0_t_real
    !!
    Return
    !!
  End Subroutine alloc_atom
  !!***********************************************************************
  !! make occus from config
  Subroutine make_occus(atom,mesg,prot)
    !!
    Implicit None  
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout)   :: atom
    Logical(t_log),Intent(in)    :: prot
    Character(*),Intent(in)      :: mesg
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_occus"
    Logical(t_log) :: rel,relmag
    Integer(t_int) :: iv,ms,ivr,kap,j,mu,l,l2,m,deg,nmain,ldo2
    Integer(t_int) :: mv,jj,j0,j1,dj,jvr,iv0
    Real(t_real)   :: Q,d,ona,d1,d2,Z,dm_min,dm_max,MCU
    Real(t_real)   :: Nup,Ndn,degmax,mmin,mmax,magmom
    Real(t_real),Pointer :: p_moccu(:,:)
    !!
    Nullify(p_moccu)
    !!
    rel=(atom% relativistic==FULL_RELATIVISTIC)
    relmag=(rel .And. atom% mspin==2_t_int)
    !!
    If(atom% Z<=0.0_t_real)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Atomic charge <=0 !'")
    !!
    !!
    !!----------------------------------------------------------------
    !!
    !!
    !! new organization
    !!
    !!
    !!
    !!
    !!
    atom% degRvmx=0_t_int
    atom% nRv=atom% nv
    !!
    !Print *,'atom% nRv=',atom% nRv
    !Print *,'atom% degRvmx=',atom% degRvmx
    !!
    Do iv=1,atom% nv
       l=atom% p_lv(iv)
       atom% degRvmx=Max(atom% degRvmx,2*l+1)
    Enddo
    !!
    !! make relativistic
    !!
    If(rel)Then
       atom% nRv=0_t_int
       atom% degRvmx=0_t_int
       If(atom% mspin==2)Then ! rel pol
          atom% degRvmx=1
          ivr=0_t_int
          Do iv=1_t_int,atom% nv
             ivr=ivr+2*atom% p_lv(iv)+1
          Enddo
          atom% nRv=ivr*2
       Else !! rel non pol
          ivr=0_t_int
          Do iv=1_t_int,atom% nv
             l=atom% p_lv(iv)
             ivr=ivr+Min(2_t_int,l+1)
             atom% degRvmx=Max(atom% degRvmx,2*l+2)
          End Do
          atom% nRv=ivr
       Endif
    Endif
    
    Call palloc('atom% p_degRv',atom% p_degRv &
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_nnRv',atom% p_nnRv &
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_lRv',atom% p_lRv &
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_kapRv',atom% p_kapRv &
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_ivrTOiv',atom% p_ivrTOiv &
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    Call palloc('atom% p_muRv',atom% p_muRv &
         &,1_t_int,atom% degRvmx&
         &,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    !!
    !!
    !!
    relif: If(.Not.rel)Then
       Do ivr=1_t_int,atom% nv
          l=atom% p_lv(ivr)
          atom% p_ivrTOiv(ivr)=ivr
          !! there are 2*l+1 m-qns for one spin direction
          atom% p_degRv(ivr)=2*l+1
          atom% p_nnRv(ivr)=atom% p_nnv(ivr)
          atom% p_lRv(ivr)=atom% p_lv(ivr)
          !! store the j+1/2 case only
          atom% p_kapRv(ivr)=-l-1
          Do m=1,atom% p_degRv(ivr)
             !! mu=nonrelat m (-l,...,0,...,l)
             atom% p_muRv(m,ivr)=-l+(m-1)
             !!
             !!
             !!
          Enddo
       End Do
    Else ! relif
       !! relativistic
       !! 
       ivr=0
       ivloop: Do iv=1_t_int,atom% nv
          l=atom% p_lv(iv)
          mv=0
          jjloop:Do jj=Max(2-l,1_t_int),2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             polarized1v:If(atom% mspin==2)Then
                !! relativistic spinpolarized
                mvloop1:Do m=1,j+1
                   ivr=ivr+1
                   mv=mv+1
                   atom% p_ivrTOiv(ivr)=iv
                   atom% p_degRv(ivr)=1
                   atom% p_nnRv(ivr)=atom% p_nnv(iv)
                   atom% p_lRv(ivr)=atom% p_lv(iv)
                   atom% p_kapRv(ivr)=kap
                   mu=-j+2*(m-1)
                   atom% p_muRv(1,ivr)=mu
                Enddo mvloop1
             Else !! relativistic nonpolarized
                ivr=ivr+1
                atom% p_ivrTOiv(ivr)=iv !!map relativistic index to nonrel
                atom% p_degRv(ivr)=j+1
                atom% p_nnRv(ivr)=atom% p_nnv(iv)
                atom% p_lRv(ivr)=atom% p_lv(iv)
                atom% p_kapRv(ivr)=kap
                mvloop2: Do m=1,atom% p_degRv(ivr)
                   mu=-j+2*(m-1)
                   atom% p_muRv(m,ivr)=mu
                   mv=mv+1
                Enddo mvloop2
             Endif polarized1v
          Enddo jjloop
       End Do ivloop
    Endif relif
    !!
    If(prot) Write(outfile,167)Trim(mesg),atom% lmax
167 Format(a,' Maximum orbital angular momentum lmax=',i2)
    !!
    !!
    !!
    !!
    !!
    Z=atom% Z
    !!
    !!
    !!
    !!
    !!
    !! remove charge Q from occupation numbers
    !!
    
    If(atom% ionicity<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Ionicity must be positive in the moment !'")
    Endif
    Q=atom% ionicity
    ivloop1:Do iv=atom% nv,1,-1
       If(prot)&
            &Write(outfile,17)Trim(mesg),atom% p_nnv(iv)&
            &,lqnchar(atom% p_lv(iv))&
            &,atom% p_nroccu(iv,1),atom% p_nroccu(iv,2),Q
       
17     Format(a,' ',i1,a1,' : ',2(2x,f15.6),' Q=',f15.6)
       Do ms=2,1,-1
          d=atom% p_nroccu(iv,ms)
          If(d>=Q)Then
             d=d-Q
             Q=0.0_t_real
             atom% p_nroccu(iv,ms)=d
             Exit ivloop1
          Else
             Q=Q-d
             d=0.0_t_real
             atom% p_nroccu(iv,ms)=d
          Endif
       Enddo
    Enddo ivloop1
    !!
    magmom=Abs(atom% Magmom)
!!$    If(atom% mspin==2.And.magmom<1.0e-6_t_real)Then
!!$       Do iv=1,atom% nv
!!$          atom% p_nroccu(iv,1:2)=Sum(atom% p_nroccu(iv,1:2))*0.5_t_real
!!$       Enddo
!!$    Else 
    !!
    !! IMPORTANT: the case magmom==0 should be treated via mspin==1 !!
    !!
    If(atom% mspin==2.And.magmom>1.0e-6_t_real)Then
       !! maximal possible magnetic change
       !Print "('magmom=',f10.5)",magmom
       Nup=Sum(atom% p_nroccu(1:atom% nv,1))
       Ndn=Sum(atom% p_nroccu(1:atom% nv,2))
       mcu=Nup-Ndn
       !Print "('Nup,Ndn=',2f10.5)",Nup,Ndn
       Print "('Z=',f10.5)",Nup+Ndn
       !Print "('M=',f10.5)",mcu
       !Print "('dM=',f10.5)",magmom-mcu
       
       degmax=Sum(2*atom% p_lv(1:atom% nv)+1)
       !Print "('deg_maxM=',f10.5)",degmax
       
       dm_min=Max(-Nup,Ndn-degmax,-mcu*0.5_t_real)
       dm_max=Min(degmax-Nup,Ndn)
       
       !Print "('dM_min,max=',2f10.5)",dm_min,dm_max
       
       mmin=mcu+dm_min*2.0_t_real
       mmax=mcu+dm_max*2.0_t_real
       Print "('M range=',2f10.5)",mmin,mmax
       
       If(magmom>mmax)Then
          Write(errbuf,66) Trim(mesg),mmax,Trim(mesg),magmom,Trim(mesg)
          Call errmsg(nmodule,nsubfunc,E_information,msg=Trim(errbuf))
66        Format("'",a,": Maximal possible magnetization  ",g30.15e2&
               &," ',/,' ",a,": exceeds requested magnetization "&
               &,g30.15e2,"',/,' ",a,": Adjusting ...'")
          magmom=mmax
       Endif
       
       Nup=Nup+(magmom-mcu)*0.5_t_real
       Ndn=Ndn-(magmom-mcu)*0.5_t_real
       
       Print "('Magmom=',2f10.5)",magmom
       !Print "('new Nup,Ndn=',2f10.5)",Nup,Ndn
       
       
       Do ms=1,2
          Q=-(3-2*ms)*(magmom-mcu)*0.5_t_real
          !Print "('spin=',i1,' Q=',f10.5)",ms,Q
          ivloop2:Do iv0=atom% nv,1,-1
             iv=atom% p_magorder(iv0)
             l=atom% p_lv(iv)
             deg=2*l+1
             d=atom% p_nroccu(iv,ms)
             !Print "(i1,a1,' d=',f10.5)",atom% p_nnv(iv),lqnchar(l),d
             If(Q>0)Then
                If(d>=Q)Then
                   d=d-Q
                   Q=0.0_t_real
                   atom% p_nroccu(iv,ms)=d
                   !Print "(i1,a1,' d>=Q: new d=',f10.5)"&
                   !     &,atom% p_nnv(iv),lqnchar(l),d
                   Exit ivloop2
                Else
                   Q=Q-d
                   d=0.0_t_real
                   atom% p_nroccu(iv,ms)=d
                   !Print "(i1,a1,' d<Q: new d=',f10.5)"&
                   !     &,atom% p_nnv(iv),lqnchar(l),d
                Endif
             Else If(Q<0)Then 
                If(deg>=d-Q)Then
                   d=d-Q
                   Q=0.0_t_real
                   atom% p_nroccu(iv,ms)=d
                   !Print "(i1,a1,' deg>=d-Q: new d=',f10.5)"&
                   !     &,atom% p_nnv(iv),lqnchar(l),d
                   Exit ivloop2
                Else
                   Q=Q+(deg-d)
                   d=deg
                   atom% p_nroccu(iv,ms)=d
                   !Print "(i1,a1,' deg-d<-Q: new d=',f10.5)"&
                   !     &,atom% p_nnv(iv),lqnchar(l),d
                Endif
             Endif
          Enddo ivloop2
       Enddo
       !!
       !Stop
       !!
    Endif
    !!
    !! relativistic non polarized occupation numbers
    Call palloc("atom% p_roccu" ,atom% p_roccu,1_t_int,atom% nRv&
         &,nmodule,nsubfunc)
    atom% p_roccu(:)=0.0_t_real
    !!
    If(rel)Then
       If(relmag)Then
          !! magnetic
          If(.False.)Then !!OLD
             Do ivr=1_t_int,atom% nRv
                iv=atom% p_ivrTOiv(ivr)
                kap=atom% p_kapRv(ivr)
                j=2*Abs(kap)-1
                mu =atom% p_muRv(1,ivr)
                l=atom% p_lRv(ivr)
                ona=Sum(atom% p_nroccu(iv,1:2))
                l2=Max(2_t_int,2_t_int*l)
                d1=Min(Real(l2,t_real),ona)
                
                If(kap>0 .Or. l==0)Then
                   If(l==0)Then
                      !! occupy positive mu first
                      m=(j-mu)/2
                   Else
                      !! occupy negativ mu first
                      m=(j+mu)/2
                   Endif
                   
                   atom% p_roccu(ivr)=Min(1.0_t_real,Max(0.0_t_real,d1-m))
                Else
                   !! occupy positive mu first
                   m=(j-mu)/2
                   atom% p_roccu(ivr)=Min(1.0_t_real,Max(0.0_t_real,ona-d1-m))
                Endif
             Enddo
             !!
          Else
             ivr=0
             Do iv=1,atom% nv
                l=atom% p_lv(iv)
                ona=atom% p_nroccu(iv,1)
                mv=0
                Do jj=Max(2-l,1_t_int),2
                   kap=l-(jj-1)*(2*l+1)
                   j=2*Abs(kap)-1
                   j0=1
                   j1=j+1
                   dj=1
                   If(kap<0)Then
                      j0=j+1
                      j1=1
                      dj=-1
                   Endif
                   Do m=j0,j1,dj
                      mu=-j+2*(m-1)
                      !Print *,'mv==',mv,2*l+1
                      If(mv==2*l+1)Then
                         ona=atom% p_nroccu(iv,2)
                      Endif
                      mv=mv+1
                      d=Max(Min(ona,1.0_t_real),0.0_t_real)
                      ona=ona-1.0_t_real
                      jvr=ivr+m
                      If(kap<0)Then
                         jvr=ivr+2*l+m
                      Endif
                      !Print "(i3,': ',i1,a1,1x,i1,'/2 ',spi2,'/2',f10.5)"&
                      !     &,jvr,atom% p_nnv(iv),lqnchar(l)&
                      !     &,j,mu,d
                      atom% p_roccu(jvr)=d
                   Enddo
                   !Print *
                Enddo
                !Print *
                ivr=ivr+2*(2*l+1)
             Enddo
          Endif
          !!
       Else
          !! non magnetic
          Do ivr=1_t_int,atom% nRv
             iv=atom% p_ivrTOiv(ivr)
             kap=atom% p_kapRv(ivr)
             l=atom% p_lRv(ivr)
             ona=Sum(atom% p_nroccu(iv,1:2))
             l2=Max(2_t_int,2_t_int*l)
             d1=Min(Real(l2,t_real),ona)
             If(kap>0 .Or. l==0)Then
                atom% p_roccu(ivr)=d1
             Else
                atom% p_roccu(ivr)=ona-d1
             Endif
          Enddo
       Endif
    Endif
    
    !! print non relativistic occupation numbers
    !!
    If(prot)Write(outfile,1000)Trim(mesg),Trim(mesg)
    
    Do iv=1,atom% nv
       nmain=atom% p_nnv(iv)
       l=atom% p_lv(iv)
       d1=atom% p_nroccu(iv,1)
       d2=atom% p_nroccu(iv,2)
       If(atom% mspin==1)Then
          d1=(d1+d2)*0.5_t_real
          d2=d1
       Endif
       If(prot)Write(outfile, 10021)Trim(mesg),iv,d1,d2,nmain,lqnchar(l)
       
    Enddo
    !!
    !! print relativistic occupation numbers
    !!
    
    If(rel)Then
       If(relmag)Then
          If(prot)Write(outfile,2001)Trim(mesg),Trim(mesg)
       Else
          If(prot)Write(outfile,2001)Trim(mesg),Trim(mesg)
       Endif
    Else
       If(prot)Write(outfile,1000)Trim(mesg),Trim(mesg)
    Endif
    
    Do ivr=1,atom% nRv
       nmain=atom% p_nnRv(ivr)
       l=atom% p_lRv(ivr)
       kap=atom% p_kapRv(ivr)
       j=2*Abs(kap)-1
       If(l /= (Abs(2*kap+1)-1)/2 )Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Corrupted array p_lRv'")
       Endif
       !!
       deg=atom% p_degRv(ivr)
       If(.Not.relmag)deg=1
       Do m=1,deg
          mu=atom% p_muRv(m,ivr)
          If(rel)Then
             If(relmag)Then
                d1=atom% p_roccu(ivr)
                If(prot)Write(outfile, 2003)Trim(mesg),ivr,d1,nmain&
                     &,lqnchar(l),j,mu
             Else 
                d1=atom% p_roccu(ivr)
                If(prot)Write(outfile, 2002)Trim(mesg),ivr,d1,nmain&
                     &,lqnchar(l),j
             Endif
          Else
             d1=atom% p_nroccu(ivr,1)
             d2=atom% p_nroccu(ivr,2)
             If(atom% mspin==1)Then
                d1=(d1+d2)*0.5_t_real
                d2=d1
             Endif
             If(prot)Write(outfile, 1002)Trim(mesg),ivr,d1,d2,nmain&
                  &,lqnchar(l)
          Endif
       End Do
       
       
    Enddo
    !!
    !! check occupation
    !!
    ona=Sum(atom% p_nroccu(1:atom% nv,1:2))
    !!
    If(atom% standalone .And. Abs(ona-Z)>1.e-5_t_real) Then
       Write(errbuf,10)Z,ona
       Call errmsg(nmodule,nsubfunc,E_information,msg=Trim(errbuf))
    Endif
    !!
    !!
    !!
    !!
    !!
    If(atom% nonspheredens)Then
       !!
       !! prepare symmetry part for nonspherical densities
       !!
       !!
       Call palloc("atom% p_gsum" ,atom% p_gsum&
            &,0_t_int,atom% lmax&
            &,1_t_int,2_t_int&
            &,1_t_int,atom% nv&
            &,nmodule,nsubfunc)
       atom% p_gsum(:,:,:)=0.0_t_real
       !!
       !!
       If(.Not.rel)Then
          !!
          If(prot)Write(outfile,6000)Trim(mesg),Trim(mesg)
6000      Format(a,' ---- nonrelativistic nonspherical m-occupation numbers ----'&
               &,/,a,' Shell  m    Spin up      Spin down        ')
          !!
          Call palloc("p_moccu",p_moccu,-atom% lmax,atom% lmax&
               &,1_t_int,2_t_int,nmodule,nsubfunc)
          !!
          Do iv=1,atom% nv
             nmain=atom% p_nnv(iv)
             l=atom% p_lv(iv)
             d1=atom% p_nroccu(iv,1)
             d2=atom% p_nroccu(iv,2)
             If(atom% mspin==1)Then
                d1=(d1+d2)*0.5_t_real
                d2=d1
             Endif
             !!
             !! spin up
             d=d1
             Do m=-l,l
                ona=Max(Min(1.0_t_real,d),0.0_t_real)
                p_moccu(m,1)=ona
                d=d-1.0_t_real
             Enddo
             !! spin down
             d=d2
             Do m=l,-l,-1
                ona=Max(Min(1.0_t_real,d),0.0_t_real)
                p_moccu(m,2)=ona
                d=d-1.0_t_real
             Enddo
             !! form gaunt symmetry factors
             Do m=-l,l
                Do ldo2=0,l
                   Do ms=1,2
                      atom% p_gsum(ldo2,ms,iv)=atom% p_gsum(ldo2,ms,iv)&
                           &+gaunt(l,m,ldo2*2,0_t_int,l,m)*p_moccu(m,ms)
                   Enddo
                Enddo
             Enddo
             !!
             !!
             !! check ld=0 to be equal to occus/sqrt(4pi)
             If(Abs(atom% p_gsum(0,1,iv)*sq4pi-d1)>1.0e-6_t_real)Then
                Print *,iv,ms,atom% p_gsum(0,1,iv)*sq4pi,d1
                Stop 'gaunt error'
             Endif
             If(Abs(atom% p_gsum(0,2,iv)*sq4pi-d2)>1.0e-6_t_real)Then
                Print *,iv,ms,atom% p_gsum(0,2,iv)*sq4pi,d2
                Stop 'gaunt error'
             Endif
             !!
             !!
             !!
             ona=Sum(p_moccu(-l:l,1:2)) !! sum of all m-occus
             If(   (Abs(ona-atom% p_degRv(iv)*2)>1.0e-6_t_real) .And.&
                  &(Abs(ona)>1.0e-6_t_real) )Then
                !!
                !! here the  shell is incomplete
                If(prot)Write(outfile, 10022)Trim(mesg),iv,nmain,lqnchar(l)
                !!
                Do m=-l,l
                   If(prot)Write(outfile,364)Trim(mesg),m&
                        &,p_moccu(m,1),p_moccu(m,2)
                Enddo
                If(prot)Write(outfile,637)Trim(mesg)
637             Format(a,'       -- nonspherical  occus --')
                Do ldo2=1,l
                   If(prot) Write(outfile,627)Trim(mesg),2*ldo2&
                        &,atom% p_gsum(ldo2,1,iv),atom% p_gsum(ldo2,2,iv)
627                Format(a,'       l=',i2,f10.5,6x,f10.5)
                Enddo
             Else
                Do ldo2=1,l
                   If(   (Abs(atom% p_gsum(ldo2,1,iv))>1.0e-6_t_real).Or.&
                        &(Abs(atom% p_gsum(ldo2,2,iv))>1.0e-6_t_real) )Then
                      Stop 'gfactor /= 0 for full shell?'
                   Endif
                Enddo
                
             End If
364          Format(a,'       m=',i2,f10.5,6x,f10.5)
             !!
             !! make nonspin g-occus
             If(atom% mspin==1)Then
                Do ldo2=0,l
                   atom% p_gsum(ldo2,1,iv)=Sum(atom% p_gsum(ldo2,1:2,iv))
                   atom% p_gsum(ldo2,2,iv)=0.0_t_real
                Enddo
             Endif
             !!
             !If(prot)Write(outfile, 10022)Trim(mesg),iv,nmain,lqnchar(l)
             !Do ldo2=1,l
             !   If(prot)Write(outfile,627) 2*ldo2&
             !        &,atom% p_gsum(ldo2,1,iv),atom% p_gsum(ldo2,2,iv)
             !Enddo
             
             !!
             !!
          Enddo
          Call pfree(p_moccu)
       Else
          Call errmsg(nmodule,nsubfunc,E_warning&
               &,msg="'Non-spherical density not implemented for "&
               &//"relativistic case!'")
          
       Endif
    End If
    !!
    !!
    !!
    !!
    !!
    Return
    !!
10  Format("'Sum of occupation numbers .ne. Z !'/'Z=",g30.15e2&
         &,' sum occu=',g30.15e2,"'")
1000 Format(a,' ---- nonrelativistic occupation numbers ----'/,&
          & a,' Shell       Spin up      Spin down        state')
1002 Format(a,' ',i4,5x,f10.5,5x,f10.4,i10,a1,i1,'/2')
2001 Format(a,' ----    relativistic occupation numbers ----'/,&
          & a,' Shell       Spin up        state')
2002 Format(a,' ',i4,5x,f10.5,i10,a1,i1,'/2')
2003 Format(a,' ',i4,5x,f10.5,i10,a1,i1,'/2',i3,'/2')
10021 Format(a,' ',i4,5x,f10.5,5x,f10.4,i10,a1,'   ')
10022 Format(a,' ',i4,5x,10x,5x,10x,i10,a1,'   ')
    !!
  End Subroutine make_occus
  !!***********************************************************************
  Subroutine  atcalc_rho(atom,mesg,prot)
    !!
    Implicit None
    !!IO
    !! atom organization
    Type(T_Atom),Intent(inout)   :: atom !! TODO: intent_in after change of rhol,spl
    Logical(t_log),Intent(in)    :: prot
    Character(*),Intent(in)      :: mesg
    !!LOCAL
    Character(*),Parameter::nsubfunc="atcalc_rho"
    Logical(t_log) :: rel,relmag,khsrel,nrel
    Integer(t_int) :: nspin,ivr,la,kap,mu,iv0,ms,ldo2,nr,ir
    Real(t_real)   :: ona,fsp,gsum,spc,fac,r
    !! charges 
    Real(t_real)   :: charge,spin,val
    !! aux
    Real(t_real)   :: osp(atom% mesh% nr)
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
    !!
    nspin=atom% mspin
    nr=atom% mesh% nr
    rel=(atom% relativistic == FULL_RELATIVISTIC) 
    khsrel=(atom% relativistic == SCALAR_RELATIVISTIC_KH)
    nrel=.Not.(rel.Or.khsrel)
    relmag=(rel.And.atom% mspin==2)
    If(relmag) Then
       nspin=1_t_int
    Endif
    !!
    atom% p_rho=0.0_t_real
    atom% p_sp=0.0_t_real
    atom% p_drho=0.0_t_real
    atom% p_dsp=0.0_t_real
    atom% p_d2rho=0.0_t_real
    atom% p_d2sp=0.0_t_real
    If(atom% nonspheredens)Then
       atom% p_rhol(:,:)=0.0_t_real
       atom% p_spl(:,:)=0.0_t_real
    Endif
    !!
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,atom% fgamv,texp)
    !!
    !!.......................................................................
    !!     all main numbers
    !!.......................................................................
    ivrloop: Do ivr=1_t_int,atom% nRv
       !! angular momentum
       la=atom% p_lRv(ivr)
       !! kappa qn
       kap=atom% p_kapRv(ivr)
       !! mu qn. for polarized FULL_RELATIVISTIC
       mu=atom% p_muRv(1,ivr)
       !!
       !!
       !!
       !! nonrelat state index
       iv0=atom% p_ivrTOiv(ivr)
       If(atom% noUnOccupied .And. &
            &All(Abs(atom% p_nroccu(iv0,1:2))<1.0e-8_t_real))Then
          Cycle ivrloop
       Endif
       !!
       !!..................................................................
       !!         all spins
       !!..................................................................
       spinloop: Do ms=1_t_int,nspin
          !!
          !! occupation number of this (degenerate) subshell
          !!TODO: possibly: make an array of occus for all states including
          !!      the degenerate angular states, then the aufbau principle
          !!      may decide about correct choice.
          If(rel)Then
             ona=atom% p_roccu(ivr)
          Else
             ona=atom% p_nroccu(ivr,ms)
          Endif
          
          !!
          fsp=(3.0_t_real-2.0_t_real*ms)*(atom% mspin-1_t_int)
          
          !! do not calculate states with zero occupation
          If(atom% noUnOccupied .And. Abs(ona)<1.e-5_t_real) Cycle spinloop 
          !!
          !!...............................................................
          !!           rho : [total spherical density] * r^2 * (4 pi)
          !!  int_r rho(r) dr == particle number
          !!...............................................................
          relatif: If(nrel) Then ! non relat
             If(atom% sic)Then
                atom% p_orbdens(:,ivr,ms)=atom% p_fv(:,ivr,1,ms)**2*texp**2
             Endif
             atom% p_rho=atom% p_rho&
                  &+atom% p_fv(:,ivr,LARGE_COMP1,ms)**2*ona
             atom% p_sp =atom% p_sp &
                  &+atom% p_fv(:,ivr,LARGE_COMP1,ms)**2*ona*fsp
             
             If(atom% nonspheredens)Then
                Do ldo2=0,la
                   gsum=atom% p_gsum(ldo2,ms,ivr)
                   If(ldo2>0 .And. Abs(gsum)>1.0e-6_t_real)Then
                      If(prot)Write(outfile,764)Trim(mesg),ldo2,gsum
                   Endif
764                Format(a,' ',20x,'ld=',i3,' gsum=',2(f10.5,2x))
                   !!
                   !!
                   atom% p_rhol(:,ldo2)=atom% p_rhol(:,ldo2)&
                        &+atom% p_fv(:,ivr,LARGE_COMP1,ms)**2*texp(:)**2*gsum
                   atom% p_spl(:,ldo2)=atom% p_spl(:,ldo2)&
                        &+atom% p_fv(:,ivr,LARGE_COMP1,ms)**2*texp(:)**2*gsum*fsp
                Enddo
             Endif
             
          Else If(khsrel) Then
             If(atom% sic)Then
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg="'SIC not yet implemented for KH SREL case, sorry!'")
             Endif
             atom% p_rho=atom% p_rho&
                  &+(&
                  &  atom% p_fv(:,ivr,LARGE_COMP1,ms)**2&
                  & +atom% p_fv(:,ivr,SMALL_COMP1,ms)**2&
                  & +atom% p_fv(:,ivr,LARGE_COMP2,ms)**2&
                  &)*ona
             atom% p_sp =atom% p_sp &
                  &+(&
                  &  atom% p_fv(:,ivr,LARGE_COMP1,ms)**2&
                  & +atom% p_fv(:,ivr,SMALL_COMP1,ms)**2&
                  & +atom% p_fv(:,ivr,LARGE_COMP2,ms)**2&
                  &)*ona*fsp
             
             If(atom% nonspheredens)Then
                Do ldo2=0,la
                   gsum=atom% p_gsum(ldo2,ms,ivr)
                   If(ldo2>0 .And. Abs(gsum)>1.0e-6_t_real)Then
                      If(prot)Write(outfile,764)Trim(mesg),ldo2,gsum
                   Endif
                   !!
                   !!
                   atom% p_rhol(:,ldo2)=atom% p_rhol(:,ldo2)&
                        &+(&
                        &   atom% p_fv(:,ivr,LARGE_COMP1,ms)**2&
                        &  +atom% p_fv(:,ivr,SMALL_COMP1,ms)**2&
                        &  +atom% p_fv(:,ivr,LARGE_COMP2,ms)**2&
                        &)*texp(:)**2*gsum
                   atom% p_spl(:,ldo2)=atom% p_spl(:,ldo2)&
                        &+(&
                        &   atom% p_fv(:,ivr,LARGE_COMP1,ms)**2&
                        &  +atom% p_fv(:,ivr,SMALL_COMP1,ms)**2&
                        &  +atom% p_fv(:,ivr,LARGE_COMP2,ms)**2&
                        &)*texp(:)**2*gsum*fsp
                Enddo
             Endif
             
          Else relatif              ! relat non polarized
             If(relmag)Then
                If(atom% sic)Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'SIC not yet implemented for magnetic "&
                        &//"relativistic case, sorry!'")
                Endif
                !!  
                !!
                atom% p_rho=atom% p_rho&
                     &+(atom% p_fv(:,ivr,LARGE_COMP1,1)**2_t_int&
                     & + atom% p_fv(:,ivr,SMALL_COMP1,1)**2_t_int)*ona
                If ((Abs(mu)<(2*la+1))) Then
                   atom% p_rho=atom% p_rho&
                        &+(atom% p_fv(:,ivr,LARGE_COMP2,1)**2_t_int&
                        & + atom% p_fv(:,ivr,SMALL_COMP2,1)**2_t_int)*ona
                Endif
                !!
                !!
                Call calc_rel_orb_sp(kap,la,mu,nr,osp&
                     &,atom% p_fv(:,ivr,LARGE_COMP1,1)&
                     &,atom% p_fv(:,ivr,SMALL_COMP1,1)&
                     &,atom% p_fv(:,ivr,LARGE_COMP2,1)&
                     &,atom% p_fv(:,ivr,SMALL_COMP2,1))
                !!
                atom% p_sp=atom% p_sp+osp*ona
                !!
                !!
                Call tlag4int(nr,osp*atom% mesh% p_jmesh,atom% mesh% hx,spc)
             Else
                If(atom% sic)Then
                   !!
                   atom% p_orbdens(:,ivr,ms)=&
                        &(atom% p_fv(:,ivr,LARGE_COMP1,1)**2_t_int&
                        &+atom% p_fv(:,ivr,SMALL_COMP1,1)**2_t_int)*texp(:)**2
                Endif
                
                !!
                atom% p_rho=atom% p_rho&
                     &+(atom% p_fv(:,ivr,LARGE_COMP1,1)**2_t_int&
                     & + atom% p_fv(:,ivr,SMALL_COMP1,1)**2_t_int)*ona
                atom% p_sp =atom% p_sp &
                     &+(atom% p_fv(:,ivr,LARGE_COMP1,1)**2_t_int&
                     & + atom% p_fv(:,ivr,SMALL_COMP1,1)**2_t_int)*ona*fsp
             Endif
          Endif relatif
          !!
          !!
          !!
          !!
          !!
       Enddo spinloop
    Enddo ivrloop ! shells ivr
    !!
    
    !! calculate density gradient
    !!
    !!
    !!
    !!
    Call dfspline(3_t_int,atom% mesh% nr,atom% mesh% p_rmesh,atom% p_rho&
         &,atom% p_drho,atom% p_d2rho)
    Call dfspline(3_t_int,atom% mesh% nr,atom% mesh% p_rmesh,atom% p_sp&
         &,atom% p_dsp,atom% p_d2sp)
    !!
    
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_rho,12,'+rho1')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_drho,12,'+drho1')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_d2rho,12,'+d2rho1')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_sp,12,'+sp1')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_dsp,12,'+dsp1')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,atom% p_d2sp,12,'+d2sp1')
    !Stop
    !!
    fac=2.0_t_real*(atom% fgamv-1.0_t_real)
    atom% p_drho(1)=0.0_t_real
    atom% p_dsp(1)=0.0_t_real
    Do ir=2,atom% mesh% nr
       r=atom% mesh% p_rmesh(ir)
       atom% p_drho(ir)=atom% p_drho(ir)+fac/r*atom% p_rho(ir)
       atom% p_dsp (ir)=atom% p_dsp (ir)+fac/r*atom% p_sp (ir)
       atom% p_d2rho(ir)=&
            & atom% p_d2rho(ir)&
            &+2.0_t_real*fac/r*atom% p_drho(ir)&
            &+fac*(fac-1.0_t_real)/r**2*atom% p_rho(ir)
       atom% p_d2sp(ir)=&
            & atom% p_d2sp(ir)&
            &+2.0_t_real*fac/r*atom% p_dsp(ir)&
            &+fac*(fac-1.0_t_real)/r**2*atom% p_sp(ir)
    Enddo
    !!
    atom% p_rho=atom% p_rho*texp(:)**2
    atom% p_sp =atom% p_sp *texp(:)**2
    !!
    atom% p_drho=atom% p_drho*texp(:)**2
    atom% p_dsp =atom% p_dsp *texp(:)**2
    !!
    atom% p_d2rho=atom% p_d2rho*texp(:)**2
    atom% p_d2sp =atom% p_d2sp *texp(:)**2
    !!
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_rho,12,'+rho')
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_drho,12,'+drho')
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_d2rho,12,'+d2rho')
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_sp,12,'+sp')
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_dsp,12,'+dsp')
    !Call writefunc(atom% mesh% nr-1,atom% mesh% p_rmesh(2:),atom% p_d2sp,12,'+d2sp')
    !!
    !!
    Call tlag4int(nr,atom% p_rho*atom% mesh% p_jmesh,atom% mesh% hx,charge)
    Call tlag4int(nr,atom% p_sp *atom% mesh% p_jmesh,atom% mesh% hx,spin)
    !!
    If(atom% force_pos_spin)Then
       If(spin<0.0_t_real)Then
          If(prot)Write(outfile,333)Trim(mesg),spin,Trim(mesg)
          spin=-spin
          atom% p_sp=-atom% p_sp
       Endif
    Endif
    !!
    If(prot)Write(outfile,7)Trim(mesg),charge
    If(prot)Write(outfile,10)Trim(mesg),spin
    !!
    !!.......................................................................
    !!     <r>
    !!.......................................................................
    Call tlag4int(nr,atom% p_rho*atom% mesh% p_rmesh*atom% mesh% p_jmesh&
         &,atom% mesh% hx,val)
    If(prot)Write(outfile,14)Trim(mesg),val/charge
    !!
    !!.......................................................................
    !!     <r^-1>
    !!.......................................................................
    osp(2:nr)=atom% p_rho(2:nr)/atom% mesh% p_rmesh(2:nr)
    osp(1)=0.0_t_real
    Call tlag4int(nr,osp*atom% mesh% p_jmesh,atom% mesh% hx,val)
    If(prot)Write(outfile,141)Trim(mesg),val/charge
    !!
    !!.......................................................................
    !!     <r>spin
    !!.......................................................................
    If(Abs(spin)>1.e-5_t_real)Then
       Call tlag4int(nr,atom% p_sp*atom% mesh% p_rmesh*atom% mesh% p_jmesh&
            &,atom% mesh% hx,val)
       If(prot)Write(outfile,15)Trim(mesg),val/spin
    End If
    !!
    !!
    !!
    !!
    !!
    !!
    Return
    !!
7   Format(a,' CHARGE             : ',f30.15)
10  Format(a,' SPIN               : ',f30.15)
14  Format(a,' <R>                : ',f30.15)
141 Format(a,' <R^-1>             : ',f30.15)
15  Format(a,' <R>spin            : ',f30.15)
333 Format(a,' OPTION: FORCE_POSITIVE_SPIN is set and spin is ',f12.5&
         &,/,a,' will invert it!') 
    !!
  End Subroutine atcalc_rho
  !!***********************************************************************
  !! relativistic orbital spin density
  Subroutine  calc_rel_orb_sp(kap,la,mu,nr,osp,fvl1,fvs1,fvl2,fvs2)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: kap,la,mu,nr
    Real(t_real),Intent(in)      :: fvl1(nr),fvs1(nr),fvl2(nr),fvs2(nr)
    !! orb spin dens
    Real(t_real),Intent(out)     :: osp(nr)
    !!LOCAL
    Logical(t_log) :: exists
    !! <kapa1|sigma_z|kapa2>
    Real(t_real)   :: alph
    !!
    osp(:)=0.0_t_real
    alph=c_mukk(mu,kap,kap,exists)
    If(exists)Then
       osp =osp +(fvl1(:)*fvl1(:))*alph
    Endif
    alph=c_mukk(mu,-kap,-kap,exists)
    If(exists)Then
       osp =osp -(fvs1(:)*fvs1(:))*alph
    Endif
    If ((Abs(mu)<(2*la+1))) Then
       alph=c_mukk(mu,-kap-1,kap,exists)
       If(exists)Then
          osp =osp +(fvl1(:)*fvl2(:))*alph*2.0_t_real
       Endif
       alph=c_mukk(mu,-kap-1,-kap-1,exists)
       If(exists)Then
          osp =osp +(fvl2(:)*fvl2(:))*alph
       Endif
       alph=c_mukk(mu,kap+1,kap+1,exists)
       If(exists)Then
          osp =osp -(fvs2(:)*fvs2(:))*alph
       Endif
    Endif
    !!
    !!
    Return
    !!
  End Subroutine calc_rel_orb_sp
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine bastest(atom)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(inout)   :: atom
    !!LOCAL
    Character(*),Parameter::nsubfunc="bastest"
    Integer(t_int) :: ivr,la,kap,j,nmain,nr,ms,iv0,icomp,ir,jvr,nmainb,lb
    Integer(t_int) :: jv0,ii,jj,kapb,jb
    Real(t_real)   :: jsig,jdeg,sL,r,t,rdeg,d,err
    Real(t_real)   :: texp(atom% mesh% nr),dd
    Real(t_real)   :: tmp(atom% mesh% nr)
    Real(t_real),Pointer :: p_fav(:,:,:,:)
    !!
    Nullify(p_fav)
    !!
    If(atom% mspin == 2)  Return
    ms=1
    !!
    nr=atom% mesh% nr
    !!
    Print *,'START: rel bas test'
    !!
    Call palloc('p_fav',p_fav &
         &,1_t_int,atom% mesh% nr&
         &,1_t_int,atom% nRv&
         &,1_t_int,atom% ncomp&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    p_fav(:,:,:,:)=0.0_t_real
    !!
    ivrloop: Do ivr=1,atom% nRv
       la=atom% p_lRv(ivr)
       nmain=atom% p_nnRv(ivr)
       kap=atom% p_kapRv(ivr)
       j=2*Abs(kap)-1
       iv0=atom% p_ivrTOiv(ivr)
       rdeg=2.0_t_real*la+1.0_t_real
       sL=Sqrt(la*(la+1.0_t_real))
       jdeg=Abs(kap)/rdeg
       jsig=Sign(1_t_int,kap)/rdeg
       
       
       Do icomp=1,2
          p_fav(1:nr,iv0,icomp,ms)=p_fav(1:nr,iv0,icomp,ms)&
               &+ atom% p_fv(1:nr,ivr,icomp,ms)*jdeg
          p_fav(1:nr,iv0,icomp+2,ms)=p_fav(1:nr,iv0,icomp+2,ms)&
               &+ atom% p_fv(1:nr,ivr,icomp,ms)*jsig*sL
       Enddo
       
    Enddo ivrloop
    !!
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,atom% fgamv,texp)
    !!
    Open(12,file='+SRELbas.1')
    Open(13,file='+SRELbas.2')
    Open(14,file='+SRELbas.3')
    Open(15,file='+SRELbas.4')
    Do ivr=1,atom% nRv
       la=atom% p_lRv(ivr)
       kap=atom% p_kapRv(ivr)
       nmain=atom% p_nnRv(ivr)
       iv0=atom% p_ivrTOiv(ivr)
       If(la /=0 .And. kap < 0 ) Cycle
       
       Print "(i3,1x,i1,a1)",iv0,nmain,lqnchar(la)
       
       Write(12,11) nmain,lqnchar(la)
       Write(13,11) nmain,lqnchar(la)
       Write(14,11) nmain,lqnchar(la)
       Write(15,11) nmain,lqnchar(la)
11     Format('# ',i1,a1)
       Do ir=1,nr
          r=atom% mesh% p_rmesh(ir)
          t=texp(ir)
          Write(12,10)r,p_fav(ir,iv0,LARGE_COMP1,ms)*t
          Write(13,10)r,p_fav(ir,iv0,SMALL_COMP1,ms)*t
          Write(14,10)r,p_fav(ir,iv0,LARGE_COMP2,ms)*t
          Write(15,10)r,p_fav(ir,iv0,SMALL_COMP2,ms)*t
       Enddo
       Write(12,2)
       Write(13,2)
       Write(14,2)
       Write(15,2)
10     Format(2(e30.15e3,2x))
2      Format(' ')
    Enddo
    Close(12)
    Close(13)
    Close(14)
    Close(15)
    !!
    Do ivr=1,atom% nRv
       la=atom% p_lRv(ivr)
       sL=1.0_t_real
       If(la > 0 ) sL=1.0_t_real/Sqrt(la*(la+1.0_t_real))
       
       kap=atom% p_kapRv(ivr)
       nmain=atom% p_nnRv(ivr)
       iv0=atom% p_ivrTOiv(ivr)
       If(la /=0 .And. kap < 0 ) Cycle
       
       
       Do jvr=1,atom% nRv
          lb=atom% p_lRv(jvr)
          nmainb=atom% p_nnRv(jvr)
          kapb=atom% p_kapRv(jvr)
          jb=2*Abs(kapb)-1
          jv0=atom% p_ivrTOiv(jvr)
          If(lb /=0 .And. kapb < 0 ) Cycle
          
          If(la /= lb )Cycle
          
          ii=iv0
          jj=jv0
          tmp=0.0_t_real
          Do icomp=1,4
             tmp(1:nr)=tmp(1:nr)+(&
                  &   p_fav(1:nr,ii,icomp,ms)&
                  & * p_fav(1:nr,jj,icomp,ms)&
                  &)
          Enddo
          
          tmp(1:nr)=tmp(1:nr)*atom% mesh% p_jmesh(1:nr)*texp**2
          
          !!
          Call lag4int(nr,tmp,atom% mesh% hx,d,lagint_out)
          
          tmp=0.0_t_real
          
          Do icomp=1,2
             tmp(1:nr)=tmp(1:nr)+(&
                  &  (&
                  &    p_fav(1:nr,ii,icomp  ,ms)&
                  &   +p_fav(1:nr,ii,icomp+2,ms)*sL&
                  &  ) * p_fav(1:nr,jj,icomp+2,ms)*sL&
                  &+ p_fav(1:nr,ii,icomp+2,ms)*sL * p_fav(1:nr,jj,icomp,ms)&
                  &)
          Enddo
          
          
          
          
          
          tmp(1:nr)=tmp(1:nr)*atom% mesh% p_jmesh(1:nr)*texp**2
          
          !!
          Call lag4int(nr,tmp,atom% mesh% hx,dd,lagint_out)
          
          err=0.0_t_real
          If(ivr==jvr) err=1.0_t_real
          err=err-d
          !!
          Print "(i3,1x,i1,a1,4x,i3,1x,i1,a1,' : ',f15.8,f15.8,f15.8)"&
               &,iv0,nmain,lqnchar(la),jv0,nmainb,lqnchar(lb),d,err,dd
          
          
          
       Enddo
       
    Enddo
    !!
    Print *,'END  : rel bas test'
    !!
    Call pfree(p_fav)
    !!
    Return
    !!
  End Subroutine bastest
#endif
  !!***********************************************************************
  Subroutine atom_transfer_fv(from,to)
    !! transfer the orbitals from one atom to another
    Implicit None
    !!IO
    Type(T_Atom),Intent(in)      :: from
    Type(T_Atom),Intent(inout)   :: to
    !!LOCAL
    Character(*),Parameter::nsubfunc="atom_transfer_fv"
    Integer(t_int) :: la,nmain,ivr,deg,jvr,kap,icomp,mu,ncomp
    Real(t_real)   :: d
    !!
    If(from% relativistic /= to% relativistic)Then 
       Write(errbuf,15) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
15     Format('''orbital transfer between different relativistic-settings!''')
    Endif
    !!
    If(from% relativistic==FULL_RELATIVISTIC)Then
       !Print *,from% mspin,to% mspin
       !Print *,from% nRv,to% nRv
       If(from% mspin==to% mspin)Then
          to% p_fv=from% p_fv
       Else If(to% mspin==2 .And. from% mspin==1)Then
          to% p_fv=0.0_t_real
          Do ivr=1,to% nRv
             la=to% p_lRv(ivr)
             deg=to% p_degRv(ivr)
             nmain=to% p_nnRv(ivr)
             kap=to% p_kapRv(ivr)
             mu=to% p_muRv(1,ivr)
             !Print "(i3,1x,i1,a1,1x,i1,'/2',1x,2f10.5,/,t12,2f10.5)",ivr,nmain,lqnchar(la)&
             !     &,2*Abs(kap)-1&
             !     &,cgoOFKapMu(1,kap,mu),cgoOFKapMu(1,-kap-1,mu)&
             !     &,cgoOFKapMu(2,kap,mu),cgoOFKapMu(2,-kap-1,mu)
             Do jvr=1,from% nRv
                If(from% p_nnRv(jvr)==nmain.And.from% p_kapRv(jvr)==kap)Then
                   If(Abs(mu)==2*Abs(kap)-1 .And. kap<0)Then
                      to% p_fv(:,ivr,1:2,1)=from% p_fv(:,jvr,1:2,1)
                   Else
                      If(kap>0)Then
                         d=-cgoOFKapMu(1,kap,mu)
                         to% p_fv(:,ivr,1:2,1)=to% p_fv(:,ivr,1:2,1)&
                              &+from% p_fv(:,jvr,1:2,1)*d
                      Else !! kap<0
                         d=cgoOFKapMu(2,kap,mu)
                         to% p_fv(:,ivr,1:2,1)=to% p_fv(:,ivr,1:2,1)&
                              &+from% p_fv(:,jvr,1:2,1)*d
                      Endif
                   Endif
                   !!
                Endif
                If(from% p_nnRv(jvr)==nmain.And.&
                     &from% p_kapRv(jvr)==-kap-1.And. kap/=-1)Then
                   If(Abs(mu)==2*Abs(kap)-1 .And. kap<0)Then
                      !to% p_fv(:,ivr,1:2,1)=from% p_fv(:,jvr,1:2,1)
                      !Print "('!!![',i3,'] L1 1*',i3)",to% p_kapRv(ivr),from% p_kapRv(jvr)
                   Else
                      If(kap>0)Then
                         d=-cgoOFKapMu(2,kap,mu)
                         to% p_fv(:,ivr,3:4,1)=to% p_fv(:,ivr,3:4,1)&
                              &+from% p_fv(:,jvr,1:2,1)*d
                      Else
                         d=cgoOFKapMu(1,kap,mu)
                         to% p_fv(:,ivr,3:4,1)=to% p_fv(:,ivr,3:4,1)&
                              &+from% p_fv(:,jvr,1:2,1)*d
                      End If
                      !!
                   Endif
                Endif
             Enddo
          Enddo
          !!
       Else
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg='''combination of spins not implemented!''')
       Endif
       !Else If(from% relativistic==SCALAR_RELATIVISTIC)Then
    Else If(from% relativistic==SCALAR_RELATIVISTIC_KH.Or.&
         &  from% relativistic==NON_RELATIVISTIC)Then
       
       ncomp=1
       If(from% relativistic==SCALAR_RELATIVISTIC_KH) ncomp=3
       
       If(from% mspin==to% mspin)Then
          to% p_fv=from% p_fv
       Else If(to% mspin==2 .And. from% mspin==1)Then
          Do ivr=1,to% nRv
             Do icomp=1,ncomp
                to% p_fv(:,ivr,icomp,1)=from% p_fv(:,ivr,icomp,1)
                to% p_fv(:,ivr,icomp,2)=from% p_fv(:,ivr,icomp,1)
             Enddo
          Enddo
       Endif
    Else
       Write(errbuf,16) Trim(relat_name(from%relativistic))
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
16     Format('''orbital transfer for ',a,' mode not implemented!''')
       
    Endif
    !!
    Return
    !!
  End Subroutine atom_transfer_fv
  !!***********************************************************************
  
End Module diracpotorb
