Module diracspecial
  !!
  Use precisiondefs
  Use memory
  Use tatom
  Use constants
  Use file_iounits
  Use atomdefs
  Use trelativistic
  Use errors
  Use diracdefs
  Use integrate1d
  Use tmesh
  Use matserv
  Use mesh
  Use xc
  Use nsphintirred
  Use sort
  Use atompot
  !!
  !!
  Implicit None 
  !!
  Private
  !!
  Public :: special_orb
  !!
  Character(*),Parameter::nmodule="diracspecial"
  !!
  !!***********************************************************************
  Type T_OpenShell
     Integer(t_int) :: iv
     Integer(t_int) :: nmain
     Integer(t_int) :: l
     Integer(t_int) :: deg
     Integer(t_int) :: ncomb
     Real(t_real),Pointer   :: p_comb(:,:,:) !! 1:deg,0:ncomb.1:2(mspin)
     Real(t_real),Pointer   :: p_agsum(:,:,:) !! 0:l,0:ncomb,1:2(mspin)
     Real(t_real),Pointer   :: p_bgsum(:,:,:) !! 0:l,0:ncomb,1:2(mspin)
     Real(t_real),Pointer   :: p_exphar(:) !! 1:ncomb
     Real(t_real),Pointer   :: p_exphar0(:) !! 1:ncomb
     Real(t_real),Pointer   :: p_expxc (:) !! 1:ncomb
     Real(t_real),Pointer   :: p_expxc0(:) !! 1:ncomb
     Real(t_real),Pointer   :: p_exp   (:) !! 1:ncomb
  End Type T_OpenShell
  !!***********************************************************************
  
Contains
  !!***********************************************************************
  Subroutine init_openshell(os)
    !!
    Implicit none  
    !!IO
    Type(T_OpenShell),Intent(inout) :: os
    !!LOCAL
    os% iv=0_t_int
    Nullify(os% p_comb)
    Nullify(os% p_agsum)
    Nullify(os% p_bgsum)
    Nullify(os% p_exphar)
    Nullify(os% p_exphar0)
    Nullify(os% p_expxc)
    Nullify(os% p_expxc0)
    Nullify(os% p_exp)
    !!
  End Subroutine init_openshell
  !!***********************************************************************
  Subroutine clean_openshell(os)
    !!
    Implicit none  
    !!IO
    Type(T_OpenShell),Intent(inout) :: os
    !!LOCAL
    Call pfree(os% p_comb)
    Call pfree(os% p_agsum)
    Call pfree(os% p_bgsum)
    Call pfree(os% p_exphar)
    Call pfree(os% p_exphar0)
    Call pfree(os% p_expxc)
    Call pfree(os% p_expxc0)
    Call pfree(os% p_exp)
    !!
  End Subroutine clean_openshell
  !!***********************************************************************
  Subroutine find_open_shell(atom,tol,os)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Type(T_OpenShell),Intent(inout) :: os
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Character(*),Parameter::nsubfunc="find_open_shell"
    Integer(t_int) :: ms,deg,nmain,l,iv,nopen,ioccu(2),iopen
    Real(t_real)   :: dsum,doccu(2)
    Character(9)   :: spin(2)
    Data spin(1)/'spin up  '/
    Data spin(2)/'spin down'/
    !!
    If(atom% mspin /= 2 .Or. &
         & (atom% relativistic /= NON_RELATIVISTIC .And.&
         &atom% relativistic /= SCALAR_RELATIVISTIC_KH)&
         &) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Need spin polarized, non/KH-scalar relativistic calculation !'")
    Endif
    !!
    Write(outfile, 2)
    nopen=0
    iopen=0
    ivloop: Do iv=1,atom% nv
       nmain=atom% p_nnv(iv)
       l=atom% p_lv(iv)
       deg=atom% p_degRv(iv)
       !! total shell occupation
       dsum=Sum(atom% p_nroccu(iv,1:atom% mspin))
       Do ms=1,atom% mspin
          doccu(ms)=atom% p_nroccu(iv,ms)
          ioccu(ms)=Nint(doccu(ms))
          If(Abs(doccu(ms)-ioccu(ms))>tol)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Noninteger occupation not supported !'")
          Endif
       Enddo
       !Print *,dsum,deg
       If(atom% element == 'Zn' .And. nmain==3 .And. l==2)Goto 99
       If(atom% element == 'Yb' .And. nmain==4 .And. l==3)Goto 99
       If(atom% element == 'Hg' .And. nmain==5 .And. l==2)Goto 99
       !! full shell
       If(Abs(dsum-2*deg)<tol) Cycle 
       !! empty shell
       If(Abs(dsum)<tol) Cycle 
       !!
99     Continue
       !Print *,ioccu
       If(ioccu(2) == 0_t_int ) Then
          If(ioccu(1) == deg )Then
             Write(outfile, 10020)iv,nmain,lqnchar(l),spin(1),doccu(1)
          Else
             Write(outfile, 10021)iv,nmain,lqnchar(l),spin(1),doccu(1)
          Endif
          Write(outfile, 10022)iv,nmain,lqnchar(l),spin(2)
          nopen=nopen+1
          iopen=iv
       Elseif(ioccu(1) == deg ) Then
          Write(outfile, 10020)iv,nmain,lqnchar(l),spin(1),doccu(1)
          Write(outfile, 10021)iv,nmain,lqnchar(l),spin(2),doccu(2)
          nopen=nopen+1
          iopen=iv
       Else
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'First hunds rule is violated by shell occupation !'")
       Endif
    Enddo ivloop
    Write(outfile, 2)
    Write(outfile, 30)nopen
    !!
    If(nopen==0) Then
       Return
    Endif
    If(nopen>1)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'More than one open shells are not supported !'")
    Endif
    !!
    os% iv    =iopen
    os% nmain =atom% p_nnv(iopen)
    os% l     =atom% p_lv(iopen)
    os% deg   =atom% p_degRv(iopen)
    !!
    Return
    !!
2   Format()
30  Format('Found ',i4,' open shells',/)
10022 Format(i4,i10,a1,2x,a,' is           unoccupied ')
10021 Format(i4,i10,a1,2x,a,' is partially occupied with ',5x,f10.5&
           &,' electrons')
10020 Format(i4,i10,a1,2x,a,' is           occupied with ',5x,f10.5&
           &,' electrons')
    
    !!
    !!
  End Subroutine find_open_shell
  !!***********************************************************************
  Subroutine  find_combinations(ioccu,maxoccu,ms,ncomb,p_comb,prot)
    !!
    Implicit none  
    !!IO
    Logical(t_log) :: prot
    !! occupation number
    Integer(t_int),Intent(in)    :: ioccu,ms
    Integer(t_int),Intent(in)    :: maxoccu
    Integer(t_int),Intent(out)   :: ncomb
    Real(t_real),Pointer         :: p_comb(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="find_combinations"
    Integer(t_int) :: maxX,x,bit,nbits,icomb
    !!
    if(prot)Write(outfile,10) nsubfunc,ioccu,maxoccu
    !!
    maxX=2**maxoccu-1
    if(prot)Write(outfile,20) maxX
    !!
    icomb=0
    Do x=0,maxX
       nbits=0
       Do bit=0,maxoccu-1
          If(Btest(x,bit))Then
             nbits=nbits+1
          Endif
          if(prot)Write(outfile,100)x,bit,Btest(x,bit),nbits
       Enddo
       If(nbits == ioccu)Then
          icomb=icomb+1
          if(prot)Write(outfile,200)x,x
       Endif
    Enddo
    ncomb=icomb
    !!
    !!
    !!
    Call palloc('p_comb',p_comb&
         &,1_t_int,maxoccu&
         &,0_t_int,ncomb&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    p_comb(:,:,:)=0.0_t_real
    icomb=0
    p_comb(:,icomb,ms)=ioccu/Real(maxoccu,t_real) 
    icomb=0
    Do x=0,maxX
       nbits=0
       Do bit=0,maxoccu-1
          If(Btest(x,bit))Then
             nbits=nbits+1
          Endif
       Enddo
       If(nbits == ioccu)Then
          icomb=icomb+1
          Do bit=0,maxoccu-1
             If(Btest(x,bit))Then
                p_comb(bit+1,icomb,ms)=1.0_t_real
             Endif
          End Do
       Endif
    Enddo
    !!
    !!
    !!
    !!
10  Format(a,': of ',i5,' of ',i5)
20  Format(' max X=',i10)
100 Format(' x=',i10,' bit=',i10,' test=',i2,' nbits=',i10)
200 Format(t20,' x=',i10,' =',Z10)

    !!
  End Subroutine find_combinations
  !!***********************************************************************
  Subroutine define_open_shell_occupations(atom,os)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)         :: atom
    Type(T_OpenShell),Intent(inout) :: os
    !!LOCAL
    Character(*),Parameter::nsubfunc="define_open_shell_occupations"
    Logical(t_log) :: prot
    Integer(t_int) :: ioccu(2),ms,icomb,la,ldo2,m,m1,k
    !!
    !!
    !!
    !!
    Do ms=1,atom% mspin
       ioccu(ms)=Nint(atom% p_nroccu(os% iv,ms))
    Enddo
    Write(outfile, 40)os% iv,os% nmain,lqnchar(os% l),ioccu(1),ioccu(2)
    !!
    !!
    !! find all combinations of possible occupation
    !!
    prot=.True.
    prot=.False.
    If(ioccu(2)==0)Then
       ms=1
       Call find_combinations(ioccu(ms),os% deg,ms,os% ncomb,os% p_comb,prot)
       os% p_comb(:,:,2)=0.0_t_real
    Elseif(ioccu(1)==os% deg)Then
       ms=2
       Call find_combinations(ioccu(ms),os% deg,ms,os% ncomb,os% p_comb,prot)
       os% p_comb(:,:,1)=1.0_t_real
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'unknown case of occupation !'")
    Endif
    !!
    !!
    Write(outfile,300) os% ncomb
    !!
    !!
    Do icomb=0,os% ncomb
       Write(outfile,400,advance='no')icomb,os% p_comb(1:os% deg,icomb,1)
       Write(outfile,401)os% p_comb(1:os% deg,icomb,2)
    Enddo
    !!
    If(os% ncomb==0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'no combinations? no open shell?!'")
    Endif
    !!
    Call palloc('os% p_exphar',os% p_exphar&
         &,0_t_int,os% ncomb&
         &,nmodule,nsubfunc)
    Call palloc('os% p_exphar0',os% p_exphar0&
         &,0_t_int,os% ncomb&
         &,nmodule,nsubfunc)
    Call palloc('os% p_expxc',os% p_expxc&
         &,0_t_int,os% ncomb&
         &,nmodule,nsubfunc)
    Call palloc('os% p_expxc0',os% p_expxc0&
         &,0_t_int,os% ncomb&
         &,nmodule,nsubfunc)
    Call palloc('os% p_exp',os% p_exp&
         &,0_t_int,os% ncomb&
         &,nmodule,nsubfunc)

    !!
    !! define the gaunt factors
    !!
    Call palloc('os% p_agsum',os% p_agsum&
         &,0_t_int,os% l&
         &,0_t_int,os% ncomb&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    Call palloc('os% p_bgsum',os% p_bgsum&
         &,0_t_int,os% l&
         &,0_t_int,os% ncomb&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    la=os% l
    os% p_agsum(:,:,:)=0.0_t_real
    os% p_bgsum(:,:,:)=0.0_t_real
    !!
    Do icomb=0,os% ncomb
       Do ldo2=0,la
          Do m=-la,la
             Do ms=1,atom% mspin
                os% p_agsum(ldo2,icomb,ms)=os% p_agsum(ldo2,icomb,ms)&
                     &+gaunt(la,m,ldo2*2,0_t_int,la,m)&
                     &*os% p_comb(m+la+1,icomb,ms)
             Enddo
             Do m1=-la,la
                k=m1-m
                If(k>=-ldo2*2 .And. k<=ldo2*2) Then
                   Do ms=1,atom% mspin
                      os% p_bgsum(ldo2,icomb,ms)=os% p_bgsum(ldo2,icomb,ms)&
                           &+gaunt(la,m1,ldo2*2,k,la,m)**2&
                           &*os% p_comb(m+la+1,icomb,ms)&
                           &*os% p_comb(m1+la+1,icomb,ms)
                      !Print *,icomb,ldo2,m,m1,k,gaunt(la,m1,ldo2*2,k,la,m)
                   Enddo
                End If
             Enddo
          Enddo
       Enddo
    End Do
    
    !!
    !!
    Return
    !!
40  Format('Will treat ',i4,1x,i1,a1,'  occu(up)',i10,5x,'  occu(down)',i10,/)
300 Format('Found ',i5,' combinations')
400 Format(i5,' : ',100(f5.2,1x))
401 Format('  --- ',100(f5.2,1x))
    !!
  End Subroutine define_open_shell_occupations
  !!***********************************************************************
  Subroutine calc_spherical_dens(atom,os,p_rho0,p_rhol,p_spl)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Type(T_OpenShell),Intent(in)  :: os
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_rho0(:,:)
    Real(t_real),Pointer :: p_spl (:,:)
    !!LOCAL
    Integer(t_int) :: iv,ldo2,ms
    Real(t_real)   :: fsp,gsum
    Real(t_real)   :: d(atom% mesh% nr)
    !!
    !!
    p_rhol(:,0)=0.0_t_real
    p_rho0(:,:)=0.0_t_real
    p_spl(:,0)=0.0_t_real
    !!
    Do iv=1,atom% nv
       !la=atom% p_lv(iv)
       !!
       !If (     iv == os% iv ) Cycle
       Do ms=1,atom% mspin
          fsp=(3.0_t_real-2.0_t_real*ms)*(atom% mspin-1_t_int)
          !!
          !! full shells and L=0 component of open shells
          gsum=atom% p_nroccu(iv,ms)/sq4pi
          !!
          ldo2=0
          !!
          !!
          Call spsisquare(atom,iv,ms,d)
          d(:)=d(:)*gsum
          !!!
          If(iv/=os% iv)Then
             p_rho0(:,1)=p_rho0(:,1)+d(:)
          Else
             p_rho0(:,2)=p_rho0(:,2)+d(:)
          Endif

          p_rhol(:,ldo2)=p_rhol(:,ldo2)+d(:)
          p_spl (:,ldo2) =p_spl (:,ldo2)+d(:)*fsp

       End Do
    End Do
    !!
    !!
  End Subroutine calc_spherical_dens
  !!***********************************************************************
  Subroutine calc_open_shell_dens(atom,os,p_rhol,p_spl,icomb)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Type(T_OpenShell),Intent(in)  :: os
    Integer(t_int),Intent(in)    :: icomb
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_spl (:,:)
    !!LOCAL
    Integer(t_int) :: iv,ms,ldo2
    Real(t_real)   :: fsp,gsum
    Real(t_real)   :: d(atom% mesh% nr)
    !!
    iv=os% iv
    !!
    !!
    p_rhol(:,1:)=0.0_t_real
    p_spl(:,1:)=0.0_t_real
    !!
    Do ms=1,atom% mspin
       fsp=(3.0_t_real-2.0_t_real*ms)*(atom% mspin-1_t_int)
       !!
       !! open shell
       Do ldo2=1,os% l
          gsum=os% p_agsum(ldo2,icomb,ms)
          !!
          If(Abs(gsum)>1.0e-6_t_real)Then
             !Write(outfile,764) ldo2,ms,gsum
          Endif
!764       Format(t20,'ld=',i3,' ms=',i2,' gsum=',2(f10.5,2x))
          !!
          call spsisquare(atom,iv,ms,d)
          d(:)=d(:)*gsum
          !!
          p_rhol(:,ldo2)=p_rhol(:,ldo2)+d(:)
          p_spl(:,ldo2) =p_spl(:,ldo2)+d(:)*fsp
       Enddo
    End Do
    !!
  End Subroutine calc_open_shell_dens
  !!***********************************************************************
  Subroutine  write_L_density(atom,maxL,p_rhol,p_spl)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_spl (:,:)
    Integer(t_int),Intent(in)    :: maxL
    !!LOCAL
    Integer(t_int) :: ldo2,ir
    !!
    !!
    Open(18,file='+spatrhoL',status='unknown')
    Do ldo2=0,maxl
       Write(18,11501)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_rhol(ir,ldo2)/sq4pi !! -atom% p_rhol(ir,ldo2)/sq4pi
       Enddo
       Write(18,501)
    Enddo
    !!
    Do ldo2=0,maxL
       Write(18,11502)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_spl(ir,ldo2)/sq4pi !! -atom% p_spl(ir,ldo2)/sq4pi
       Enddo
       Write(18,501)
    Enddo
    Close(18)
    !!
    Return
    !!
500 Format(2(g30.15e3,2x))
501 Format(' ') 
11501 Format('# density l,m=',i3,',0') 
11502 Format('# spin density l,m=',i3,',0') 
    !!
  End Subroutine write_L_density
  !!***********************************************************************
  Subroutine  write_L_potential(atom,maxL,p_potharl,p_potxcl,p_bfl)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Real(t_real),Pointer :: p_potharl(:,:)
    Real(t_real),Pointer :: p_potxcl(:,:)
    Real(t_real),Pointer :: p_bfl (:,:)
    Integer(t_int),Intent(in)    :: maxL
    !!LOCAL
    Integer(t_int) :: ldo2,ir
    Real(t_real)   :: d
    !!
    !!
    !! write pure Hartree potential
    !!
    Open(18,file='+spatpotharL',status='unknown')
    Do ldo2=0,maxl
       Write(18,11501)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          d=0.0_t_real
          If(ldo2 ==0 )Then
             !!test d= atom% p_vhar(ir)
          Endif
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_potharl(ir,ldo2)/sq4pi -d
       Enddo
       Write(18,501)
    Enddo
    Close(18)
    !!
    !! write potential contribution and magnetic fied of xc
    !!
    Open(18,file='+spatpotxcL',status='unknown')
    Do ldo2=0,maxl
       Write(18,11501)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          d=0.0_t_real
          If(ldo2 ==0 )Then
             !!test d= atom% p_vxc(ir)
          Endif
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_potxcl(ir,ldo2)/sq4pi -d
       Enddo
       Write(18,501)
    Enddo
    Do ldo2=0,maxl
       Write(18,11502)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          d=0.0_t_real
          If(ldo2 ==0 )Then
             !!test d= atom% p_btr(ir)
          Endif
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_bfl(ir,ldo2)/sq4pi -d
       Enddo
       Write(18,501)
    Enddo
    Close(18)
    !!
    !! write total potential and field
    !!
    !!
    Open(18,file='+spatpotL',status='unknown')
    Do ldo2=0,maxl
       Write(18,11501)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          d=0.0_t_real
          If(ldo2 ==0 )Then
             !!test d= atom% p_vtr(ir)
          Endif
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,(p_potharl(ir,ldo2)+p_potxcl(ir,ldo2))/sq4pi -d
       Enddo
       Write(18,501)
    Enddo
    !!
    Do ldo2=0,maxL
       Write(18,11502)ldo2*2
       Do ir=1_t_int,atom% mesh% nr
          d=0.0_t_real
          If(ldo2 ==0 )Then
             !!test d= atom% p_btr(ir)
          Endif
          Write(18,500)atom% mesh% p_rmesh(ir)&
               &,p_bfl(ir,ldo2)/sq4pi -d
       Enddo
       Write(18,501)
    Enddo
    Close(18)
    !!
    Return
    !!
500 Format(2(g30.15e3,2x))
501 Format(' ') 
11501 Format('# potential l,m=',i3,',0') 
11502 Format('# magnetic field l,m=',i3,',0') 
    !!
  End Subroutine write_L_potential
  !!***********************************************************************
  !! for all l=2*minL,2*maxL,2 cal Hartree potential
  Subroutine calc_vhartree(atom,p_rhol,minL,maxL,p_potharl)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Integer(t_int),Intent(in)    :: minL,maxL
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_potharl(:,:)
    !!LOCAL
    Integer(t_int) :: ir,ldo2,l
    Real(t_real)   :: ri,rl,cp,rjac,fac,sd
    Real(t_real)   :: yout(atom% mesh% nr)
    Real(t_real)   :: yin (atom% mesh% nr)
    !!
    Do ldo2=minL,maxL
       l=ldo2*2
       fac=fourpi/Real(2_t_int*l+1_t_int,t_real)
       Do ir=1,atom% mesh% nr
          ri  =atom% mesh% p_rmesh(ir)
          rjac=atom% mesh% p_jmesh(ir)
          If(l == 0)Then
             rl=1.0_t_real
          Else
             rl=ri**l
          Endif
          yout(ir)=p_rhol(ir,ldo2)*rl*rjac
          If(atom% mesh% with_zero .and. (ir ==1 ))Then
             yin (ir)=0.0_t_real
          Else
             yin (ir)=p_rhol(ir,ldo2)/rl/ri*rjac
          Endif
       Enddo
       !!
       Call lag4int(atom% mesh% nr,yout,atom% mesh% hx,cp,lagint_out)
       Call lag4int(atom% mesh% nr,yin ,atom% mesh% hx,cp,lagint_in)
       !!
       Do ir=1,atom% mesh% nr
          ri  =atom% mesh% p_rmesh(ir)
          If(l == 0)Then
             rl=1.0_t_real
          Else
             rl=ri**l
          Endif
          If(atom% mesh% with_zero .And. (ir ==1 ))Then
             sd=0.0_t_real
          Else
             sd=yout(ir)/rl
          Endif
          p_potharl(ir,ldo2)=(sd+yin(ir)*rl*ri)*fac
       Enddo
    Enddo
    !!
  End Subroutine calc_vhartree
  !!***********************************************************************
  Subroutine calc_shell_hartree_expectation(atom,os,icomb,p_potharl)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Type(T_OpenShell),Intent(inout)  :: os
    Integer(t_int),Intent(in)    :: icomb
    Real(t_real),Pointer         :: p_potharl(:,:)
    !!LOCAL
    Integer(t_int) :: iv,la,ms,ir,ldo2
    Real(t_real)   :: cp,cp0,ri,rjac,rinv,d,gamma
    Real(t_real)   :: yout(atom% mesh% nr)
    Real(t_real)   :: yout0(atom% mesh% nr)
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
    iv=os% iv
    la=os% l
    !!
    gamma=atom% fgamv
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,gamma,texp)
    !!
    !! sum_L{vl_10* sum_{s}{ (r*Rlm)^2 * sum_m{Nslm*(Ylm|Yl_10|Ylm)}}
    !!
    yout0(:)=0.0_t_real
    yout(:)=0.0_t_real
    !!
    !!
    Do ir=1,atom% mesh% nr
       ri  =atom% mesh% p_rmesh(ir)
       rjac=atom% mesh% p_jmesh(ir)
       !!
       If(atom% mesh% with_zero .And. (ir ==1 ))Then
          rinv=0.0_t_real
       Else             
          rinv=1.0_t_real/ri
       Endif
       !!
       Do ldo2=0,la
          Do ms=1,atom% mspin
             !!
             !!
             d=psisquare(atom,ir,iv,ms,texp)&
                  &  *p_potharl(ir,ldo2)*rinv&
                  &  *os% p_agsum(ldo2,icomb,ms)&
                  &  *rjac
             !!
             yout(ir)=yout(ir)+d
             If(ldo2 == 0)Then
                yout0(ir)=yout0(ir)+d
             Endif
             !!
          Enddo
          !!
          !!
       Enddo
    Enddo
    !!
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,yout,12,'+test')
    !!
    Call lag4int(atom% mesh% nr,yout,atom% mesh% hx,cp,lagint_out)
    Call lag4int(atom% mesh% nr,yout0,atom% mesh% hx,cp0,lagint_out)
    cp =cp *0.5_t_real
    cp0=cp0*0.5_t_real
    !!
    os% p_exphar0(icomb)=cp0
    os% p_exphar(icomb)=cp
    os% p_exp   (icomb)=os% p_exp   (icomb)+cp
    !!
    Return
    !!
  End Subroutine calc_shell_hartree_expectation
  !!***********************************************************************
  Subroutine calc_shell_xc_expectation(atom,os,icomb,p_excl,p_potxcl,p_bfl&
       &,xc_v,p_rhol,p_spl,full_rho)
    !!
    Implicit none  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Type(T_OpenShell),Intent(inout)  :: os
    Logical(t_log),Intent(in)    :: xc_v,full_rho
    Integer(t_int),Intent(in)    :: icomb
    Real(t_real),Pointer         :: p_potxcl(:,:)
    Real(t_real),Pointer         :: p_excl  (:,:)
    Real(t_real),Pointer         :: p_bfl   (:,:)
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_spl (:,:)
    !!LOCAL
    Integer(t_int) :: iv,la,ms,ir,ldo2
    Real(t_real)   :: cp,cp0,ri,rjac,rinv,vpm(2),d,ecp,ecp0,eps,rho(2),f,gamma
    Real(t_real)   :: yout(atom% mesh% nr)
    Real(t_real)   :: eyout(atom% mesh% nr)
    Real(t_real)   :: yout0(atom% mesh% nr)
    Real(t_real)   :: eyout0(atom% mesh% nr)
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
    iv=os% iv
    la=os% l
    !!
    gamma=atom% fgamv
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,gamma,texp)
    !!
    !! sum_L{ sum_{s}{v_{sl_10}* (r*Rlm)^2 * sum_m{Nslm*(Ylm|Yl_10|Ylm)}}
    !!
    yout0(:)=0.0_t_real
    eyout0(:)=0.0_t_real
    yout(:)=0.0_t_real
    eyout(:)=0.0_t_real
    !!
    !!
    Do ir=1,atom% mesh% nr
       ri  =atom% mesh% p_rmesh(ir)
       rjac=atom% mesh% p_jmesh(ir)
       !!
       If(atom% mesh% with_zero .And. (ir ==1 ))Then
          rinv=0.0_t_real
       Else             
          rinv=1.0_t_real/ri
       Endif
       !!
       Do ldo2=0,la
          vpm(1)=p_potxcl(ir,ldo2)+p_bfl(ir,ldo2)
          vpm(2)=p_potxcl(ir,ldo2)-p_bfl(ir,ldo2)
          eps   =p_excl(ir,ldo2)

          rho(1)=(p_rhol(ir,ldo2) +p_spl (ir,ldo2))*0.5_t_real
          rho(2)=(p_rhol(ir,ldo2) -p_spl (ir,ldo2))*0.5_t_real
          

          Do ms=1,atom% mspin
             !!
             If(full_rho)Then
                f=rho(ms)
             Else
                !!
                f=psisquare(atom,ir,iv,ms,texp)*os% p_agsum(ldo2,icomb,ms)
             Endif
             !!
             d=f*vpm(ms)*rinv *rjac
             !!
             yout(ir)=yout(ir)+d
             If(ldo2 == 0) Then
                yout0(ir)=yout0(ir)+d
             Endif

             !!
             d=f *eps*rinv*rjac
             !!
             eyout(ir)=eyout(ir)+d
             If(ldo2 == 0) Then
                eyout0(ir)=eyout0(ir)+d
             Endif

             !!
          Enddo
          !!
          !!
       Enddo
    Enddo
    !!
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,yout,12,'+test')
    !!
    Call lag4int(atom% mesh% nr,yout,atom% mesh% hx,cp,lagint_out)
    Call lag4int(atom% mesh% nr,eyout,atom% mesh% hx,ecp,lagint_out)
    Call lag4int(atom% mesh% nr,yout0,atom% mesh% hx,cp0,lagint_out)
    Call lag4int(atom% mesh% nr,eyout0,atom% mesh% hx,ecp0,lagint_out)
    !!
    If(xc_v)Then
       os% p_expxc0(icomb)=cp0
       os% p_expxc(icomb)=cp
       os% p_exp  (icomb)=os% p_exp  (icomb)+cp
    Else
       os% p_expxc0(icomb)=ecp0
       os% p_expxc(icomb)=ecp
       os% p_exp  (icomb)=os% p_exp  (icomb)+ecp
    Endif
    !!
    Print *,'ratio=',ecp/cp
    !!
    Return
    !!
  End Subroutine calc_shell_xc_expectation
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine calc_vxc(atom,p_rhol,p_spl,maxL,p_potxcl,p_bfl,xreduce)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Integer(t_int),Intent(in)    :: maxL
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Pointer :: p_rhol  (:,:)
    Real(t_real),Pointer :: p_spl   (:,:)
    Real(t_real),Pointer :: p_potxcl(:,:)
    Real(t_real),Pointer :: p_bfl(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_vxc"
    Logical(t_log) :: HEX,alert
    Integer(t_int) :: nvg
    Integer(t_int) :: vnga
    Real(t_real)   :: vag(3,2000)
    Real(t_real)   :: vwg(2000)
    Real(t_real)   :: ylm((2*maxl+1)**2)
    Real(t_real)   :: fL(0:maxL,2)
    Integer(t_int) :: ir,ig,ldo2,l,m,ilm
    Real(t_real)   :: ri,wrsh,r2,dp,dm,dums(2),dum
    Real(t_real)   :: rsh(3)
    Real(t_real)   :: error,tol,ff,vex(2),epsxc,rho(2)
    !!
    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    ff=1.0_t_real
    !!
    !errtol=1.0e-12_t_real
    !!
    nvg=10_t_int !! maximum mesh size
    HEX=.False.
    !!
    Call gsphirred(nvg&
         &,vag&
         &,vwg&
         &,vnga,HEX,1_t_int,.False.)
    !!
2   Format(a,': Spherical mesh internal type=',i5)
3   Format(a,': Spherical mesh size=',i5)
    Write(outfile,2)Trim(nsubfunc),nvg
    Write(outfile,3)Trim(nsubfunc),vnga

    !! 
    irloop: Do ir=1,atom% mesh% nr
       ri=atom% mesh% p_rmesh(ir) 
       !!
       !!
       !! result array of all L-components
       fL(0:maxL,1:2)=0.0_t_real
       igloop: Do ig=1,vnga
          rsh(:)=ri*vag(:,ig)
          wrsh  =vwg(ig)
          !!
          r2=Dot_product(rsh,rsh)
          Call ylms12(ylm,rsh(1),rsh(2),rsh(3),2*maxl)
          !!
          !!
          rho(:)=0.0_t_real
          Do ldo2=0,maxl
             l=ldo2*2
             m=0
             ilm=l*l+l+1+m
             dp=(p_rhol(ir,ldo2) +p_spl (ir,ldo2))*0.5_t_real
             dm=(p_rhol(ir,ldo2) -p_spl (ir,ldo2))*0.5_t_real
             rho(1)=rho(1)+dp*ylm(ilm)
             rho(2)=rho(2)+dm*ylm(ilm)
          Enddo
          !!
          !!
          !! put some values to get through
          If(atom% mesh% with_zero .And. (ir ==1 ))Then
             rho(1:2)=0.0_t_real
          Else
             rho(1)=rho(1)/r2
             rho(2)=rho(2)/r2
          Endif
          !!
          !! xc-potential
          Call adjustrho(rho,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rho,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          
          !!
          !! now make the real Ylm expansion
          !!
          Do ldo2=0,maxl
             l=ldo2*2
             m=0
             ilm=l*l+l+1+m
             !Print *,l,m,ilm
             fL(ldo2,1)=fL(ldo2,1)+vex(1)*ylm(ilm)*wrsh
             fL(ldo2,2)=fL(ldo2,2)+vex(2)*ylm(ilm)*wrsh
          Enddo
          !!
       Enddo igloop
       !!
       !!
       !! save result
       Do ldo2=0,maxl
          dp=(fL(ldo2,1)+fL(ldo2,2))*0.5_t_real*ri
          dm=(fL(ldo2,1)-fL(ldo2,2))*0.5_t_real*ri
          
          p_potxcl(ir,ldo2)=dp
          p_bfl   (ir,ldo2)=dm
       Enddo
       !!
       !!
    Enddo irloop
    !!
    !!
    Return
    !!
  End Subroutine calc_vxc
#endif
  !!***********************************************************************
  Subroutine calc_vxc_L0(atom,p_rhol,p_spl,maxL,p_excl,p_potxcl,p_bfl,x_only&
       &,xreduce)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Logical(t_log),Intent(in)    :: x_only
    Integer(t_int),Intent(in)    :: maxL
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Pointer :: p_rhol  (:,:)
    Real(t_real),Pointer :: p_spl   (:,:)
    Real(t_real),Pointer :: p_potxcl(:,:)
    Real(t_real),Pointer :: p_excl  (:,:)
    Real(t_real),Pointer :: p_bfl(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_vxc_L0"
    Logical(t_log) :: alert
    Integer(t_int) :: nvg,olxcv
    Real(t_real)   :: ylm((2*maxl+1)**2)
    Real(t_real)   :: fL(0:maxL,2)
    Real(t_real)   :: eL(0:maxL)
    Integer(t_int) :: ir,ig,ldo2,l,m,ilm
    Real(t_real)   :: ri,wrsh,r2,dp,dm,z,x,dums(2),dum
    Real(t_real)   :: rsh(3)
    Real(t_real)   :: error,tol,ff,vex(2),epsxc,rho(2)
    !! DONT FREE THEM !
    Real(t_real),Pointer   :: p_cc(:),p_xx(:)

    
    !!
    If(x_only)Then
       olxcv=getxcversion()
       Print *,'before XC=',olxcv
       Call setxcversion(XCVER_XONLY)
       Print *,'inside XC=',getxcversion()
    End If
    !!
    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    ff=1.0_t_real
    !!
    !errtol=1.0e-12_t_real
    !!
    nvg=8*maxL+2_t_int !! maximum mesh size
    !!
    Call get_gauss(nvg,p_xx,p_cc)
    !! 
    irloop: Do ir=1,atom% mesh% nr
       ri=atom% mesh% p_rmesh(ir) 
       !!
       !!
       !! result array of all L-components
       fL(0:maxL,1:2)=0.0_t_real
       eL(0:maxL)=0.0_t_real
       igloop: Do ig=1,nvg
          z=p_xx(ig)
          x=Sqrt(1.0_t_real-z*z)
          wrsh=p_cc(ig)*twopi
          !!
          rsh(:)=ri*(/x,0.0_t_real,z/)
          !!
          r2=Dot_product(rsh,rsh)
          Call ylms12(ylm,rsh(1),rsh(2),rsh(3),2*maxl)
          !!
          !!
          rho(:)=0.0_t_real
          Do ldo2=0,maxl
             l=ldo2*2
             m=0
             ilm=l*l+l+1+m
             dp=(p_rhol(ir,ldo2) +p_spl (ir,ldo2))*0.5_t_real
             dm=(p_rhol(ir,ldo2) -p_spl (ir,ldo2))*0.5_t_real
             rho(1)=rho(1)+dp*ylm(ilm)
             rho(2)=rho(2)+dm*ylm(ilm)
          Enddo
          !!
          !!
          !! put some values to get through
          If(atom% mesh% with_zero .And. (ir ==1 ))Then
             rho(1:2)=0.0_t_real
          Else
             rho(1)=rho(1)/r2
             rho(2)=rho(2)/r2
          Endif
          !!
          !!  make unpolarized
          !!
          !rho(1)=(rho(1)+rho(2))*0.5_t_real
          !rho(2)=rho(1)
          !! xc-potential
          Call adjustrho(rho,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rho,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          
          !!
          !! now make the real Ylm expansion
          !!
          Do ldo2=0,maxl
             l=ldo2*2
             m=0
             ilm=l*l+l+1+m
             !Print *,l,m,ilm
             fL(ldo2,1)=fL(ldo2,1)+vex(1)*ylm(ilm)*wrsh
             fL(ldo2,2)=fL(ldo2,2)+vex(2)*ylm(ilm)*wrsh
             eL(ldo2)  =eL(ldo2  )+epsxc *ylm(ilm)*wrsh
          Enddo
          !!
       Enddo igloop
       !!
       !!
       !! save result
       Do ldo2=0,maxl
          dp=(fL(ldo2,1)+fL(ldo2,2))*0.5_t_real*ri
          dm=(fL(ldo2,1)-fL(ldo2,2))*0.5_t_real*ri
          
          p_excl  (ir,ldo2)=eL(ldo2)*ri
          p_potxcl(ir,ldo2)=dp
          p_bfl   (ir,ldo2)=dm
       Enddo
       !!
       !!
    Enddo irloop
    !!
    If(alert)Then
      
3      Format('''Numerical inaccuracy in vxc/exc &
            &calculation! '',/,'' maxerror=',e12.4e3,'''')
       Write(errbuf,3) error
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif

    !!
    If(x_only) Then
       Call setxcversion(olxcv)
       Print *,'after XC=',getxcversion()
    End If
    !!
    Return
    !!
  End Subroutine calc_vxc_L0
  !!***********************************************************************
  Subroutine calc_slater_F_integrals(atom,iv,ms,Fl)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Integer(t_int),Intent(in)    :: iv,ms
    Real(t_real),Intent(out)     :: Fl(0:)
    !!LOCAL
    Integer(t_int) :: la,ir,ldo2,l
    Real(t_real)   :: ri,rjac,rl,ftr,cp,sd,gamma
    Real(t_real)   :: yout(atom% mesh% nr)
    Real(t_real)   :: yin (atom% mesh% nr)
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
    gamma=atom% fgamv
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,gamma,texp)
    !!
    la=atom% p_lv(iv)
    !!
    Do ldo2=0,la
       l=ldo2*2
       !!
       Do ir=1,atom% mesh% nr
          ri  =atom% mesh% p_rmesh(ir)
          rjac=atom% mesh% p_jmesh(ir)
          !!
          ftr=psisquare(atom,ir,iv,ms,texp)
          If(l == 0)Then
             rl=1.0_t_real
          Else
             rl=ri**l
          Endif
          yout(ir)=ftr *rl*rjac
          If(atom% mesh% with_zero .And. (ir ==1 ))Then
             yin (ir)=0.0_t_real
          Else
             yin (ir)=ftr/rl/ri*rjac
          Endif
       Enddo
       !!
       Call lag4int(atom% mesh% nr,yout,atom% mesh% hx,cp,lagint_out)
       Call lag4int(atom% mesh% nr,yin ,atom% mesh% hx,cp,lagint_in)
       !!
       Do ir=1,atom% mesh% nr
          ri  =atom% mesh% p_rmesh(ir)
          rjac=atom% mesh% p_jmesh(ir)
          !!
          ftr=psisquare(atom,ir,iv,ms,texp)
          If(l == 0)Then
             rl=1.0_t_real
          Else
             rl=ri**l
          Endif
          sd=yin(ir)*rl
          If(atom% mesh% with_zero .And. (ir ==1 ))Then
             sd=sd+0.0_t_real
          Else
             sd=sd+yout(ir)/rl/ri
          Endif
          yout(ir)=sd*ftr*rjac
       Enddo
       !!
       Call lag4int(atom% mesh% nr,yout,atom% mesh% hx,cp,lagint_out)
       !!
       !Print *,ldo2,cp
       Fl(ldo2)=cp
    Enddo
    !!
  End Subroutine calc_slater_F_integrals
  !!***********************************************************************
  Subroutine sort_exp(os)
    !!
    Implicit none  
    !!IO
    Type(T_OpenShell),Intent(inout)  :: os
    !!LOCAL
    Character(*),Parameter::nsubfunc="sort_exp"
    Integer(t_int) :: i,icomb,icombmin
    Real(t_real)   :: tolsort
    Integer(t_int),Pointer :: p_iwk(:)
    !!
    Nullify(p_iwk)
    !!
    tolsort=1.0e-12_t_real
    Call palloc('p_iwk' ,p_iwk ,1_t_int,os% ncomb,nmodule,nsubfunc)
    Call dshel2(1_t_int,os% ncomb,os% p_exp(1:),p_iwk,tolsort)
    !!
    icombmin=p_iwk(1)+1
    !!
    Write(outfile,2)
    Write(outfile,1400)
    Do icomb=1,os% ncomb
       i=p_iwk(icomb)+1
       Write(outfile,400,advance='no')i&
            &,os% p_exp(i)*hatoev,os% p_exphar(i)*hatoev,os% p_expxc(i)*hatoev&
            &,os% p_exphar0(i)*hatoev,os% p_expxc0(i)*hatoev&
            &,Nint(os% p_comb(1:os% deg,i,1))
       Write(outfile,401)Nint(os% p_comb(1:os% deg,i,2))
    Enddo
    Write(outfile,2)
    !!
    !! write deepest
    i=icombmin
    Write(outfile,199,advance='no')&
         & os% p_exp(i)*hatoev,os% p_exphar(i)*hatoev,os% p_expxc(i)*hatoev&
         &,os% p_exphar0(i)*hatoev,os% p_expxc0(i)*hatoev&
         &,Nint(os% p_comb(1:os% deg,i,1))
    Write(outfile,401)Nint(os% p_comb(1:os% deg,i,2))
    Write(outfile,2)
    !!
    !! write hund
    i=1
    Write(outfile,300,advance='no')&
         & os% p_exp(i)*hatoev,os% p_exphar(i)*hatoev,os% p_expxc(i)*hatoev&
         &,os% p_exphar0(i)*hatoev,os% p_expxc0(i)*hatoev&
         &,Nint(os% p_comb(1:os% deg,i,1))
    Write(outfile,401)Nint(os% p_comb(1:os% deg,i,2))
    Write(outfile,2)
    !!
    
    !!
    Call pfree(p_iwk)
    !!
    Return
    !!

300 Format('hund<v...>=',f24.15,4(f30.15),' : ',100(i1,1x))
199 Format('min<v...>=',f25.15,4(f30.15),' : ',100(i1,1x))
400 Format(i5,5(f30.15),' : ',100(i1,1x))
1400 Format('icomb=                <v>  eV                       <vhar> eV&
          &                     <vxc>  eV                     &
          &<vhar0> eV                    <vxc0> eV')
    !!
401 Format('  --- ',100(i1,1x))
2 Format()
  End Subroutine sort_exp
  !!***********************************************************************
  Subroutine  special_orb(atom,xreduce)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)       :: atom
    Real(t_real),Intent(in)      :: xreduce
    !!LOCAL
    Character(*),Parameter::nsubfunc="special_orb"
    Integer(t_int) :: maxL,icomb,ms,ldo2,m
    Real(t_real)   :: avexphar,avexpxc,avexp,Fl(0:12,1:2),vhar,vhar0
    Real(t_real)   :: vxc,vxc0,J(2),dd,EHUiso,EXUSIiso,EXJiso
    Type(T_OpenShell) :: os
    Logical(t_log) :: xc_v,x_only,full_rho,config_average,HF
    Real(t_real),Parameter :: tol=1.0e-6_t_real
    Real(t_real),Pointer :: p_rho0(:,:)
    Real(t_real),Pointer :: p_rhol(:,:)
    Real(t_real),Pointer :: p_spl(:,:)
    Real(t_real),Pointer :: p_potharl(:,:)
    Real(t_real),Pointer :: p_potxcl(:,:)
    Real(t_real),Pointer :: p_excl(:,:)
    Real(t_real),Pointer :: p_bfl(:,:)
    !!
    xc_v=.True.  !! use vxc expression
    xc_v=.False. !! use exc expression
    x_only=.True.  !! use x-only
    x_only=.False. !! use Perdew-Wang
    full_rho=.False. !! use shell density
    full_rho=.True.  !! use full density
    config_average=.False. !! spherical refference point
    config_average=.True.  !! configuration average
    HF=.False.  !! LSDA
    HF=.True.   !! Hartree-Fock expression
    !!
    Write(outfile,1)nsubfunc
    Nullify(p_rho0)
    Nullify(p_rhol)
    Nullify(p_spl)
    Nullify(p_potharl)
    Nullify(p_potxcl)
    Nullify(p_excl)
    Nullify(p_bfl)
    !!
    Call init_openshell(os)
    !!
    Call find_open_shell(atom,tol,os)
    !!
    Call define_open_shell_occupations(atom,os)
    !!
    maxL=os% l
    !!
    Call palloc('p_rho0',p_rho0&
         &,1_t_int,atom% mesh% nr&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    Call palloc('p_rhol',p_rhol&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)
    Call palloc('p_spl',p_spl&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)
    !!
    Call palloc('p_potharl',p_potharl&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)
    Call palloc('p_potxcl',p_potxcl&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)
    Call palloc('p_excl',p_excl&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)
    Call palloc('p_bfl',p_bfl&
         &,1_t_int,atom% mesh% nr&
         &,0_t_int,maxL&
         &,nmodule,nsubfunc)

    !!
    p_rho0(:,:)=0.0_t_real
    p_rhol(:,:)=0.0_t_real
    p_spl(:,:)=0.0_t_real
    p_potharl(:,:)=0.0_t_real
    p_potxcl(:,:)=0.0_t_real
    p_excl(:,:)=0.0_t_real
    p_bfl(:,:)=0.0_t_real
    !!
    !! create spherical density resulting from all shells
    !!
    !!
    Call calc_spherical_dens(atom,os,p_rho0,p_rhol,p_spl)
    !!
    !! calculate spherical Hartree potential
    !!
    Call calc_vhartree(atom,p_rhol,0_t_int,0_t_int,p_potharl)
    !!
    Do ms=1,atom% mspin
       Call calc_slater_F_integrals(atom,os% iv,ms,Fl(0:12,ms))
       Write(outfile,90,advance='no') atom% p_nnv(os% iv)&
            &,lqnchar(atom% p_lv(os% iv)),ms
90     Format('SlaterFl: ',i1,a1,2x,i2,4x)
91     Format(f12.5,2x)
2      Format()
       ldo2=0
       Write(outfile,91,advance='no') Fl(ldo2,ms)
       Do ldo2=1,maxL
          Write(outfile,91,advance='no') Fl(ldo2,ms)!/Fl(0,ms)
       Enddo
       Write(outfile,2)
       !!
       !!
       !!
       !!
    Enddo

    Do ms=1,atom% mspin
       !!
       Write(outfile,94,advance='no') atom% p_nnv(os% iv)&
            &,lqnchar(atom% p_lv(os% iv)),ms
       !!
94     Format('SlaterUJ: ',i1,a1,2x,i2,4x)
92     Format(' U=',f12.5,' eV ',2x)
93     Format(' J=',f12.5,' eV ',2x)
       Write(outfile,92,advance='no') Fl(0,ms)*hatoev

       
       If (maxl == 0) J(ms)=0.0_t_real
       If (maxl == 1) J(ms)=Fl(1,ms)/5.0_t_real
       If (maxl == 2) J(ms)=(Fl(1,ms)+Fl(2,ms))/14.0_t_real
       If (maxl == 3) J(ms)=(286.0_t_real*Fl(1,ms)+195.0_t_real*Fl(2,ms)&
            &+250.0_t_real*Fl(3,ms))/6435.0_t_real


       Write(outfile,93,advance='no') J(ms)*hatoev
       Write(outfile,2)

       !!
    Enddo
    !!
    os% p_exphar(:)=0.0_t_real
    os% p_exphar0(:)=0.0_t_real
    os% p_expxc(:)=0.0_t_real
    os% p_expxc0(:)=0.0_t_real
    os% p_exp(:)=0.0_t_real
    !!
    avexphar=0.0_t_real
    avexpxc =0.0_t_real
    avexp   =0.0_t_real
    Do icomb=0,os% ncomb
       !!
400    Format('icomb=',i5,' : ',100(i1,1x))
401    Format('  --- ',100(i1,1x))
       Write(outfile,400,advance='no') icomb&
            &,Nint(os% p_comb(1:os% deg,icomb,1))
       Write(outfile,401)Nint(os% p_comb(1:os% deg,icomb,2))
       !! create nonspherical density resulting from open shell occupation
       Call calc_open_shell_dens(atom,os,p_rhol,p_spl,icomb)
       !!
       If(.False.)Then
          Call write_L_density(atom,maxL,p_rhol,p_spl)
       Endif
       !!
       !! calculate nonspherical Hartree potential
       Call calc_vhartree(atom,p_rhol,1_t_int,maxL,p_potharl)
       !!
       !!
       !! calculate xc potential
       Call calc_vxc_L0(atom,p_rhol,p_spl,maxL,p_excl,p_potxcl,p_bfl,x_only&
            &,xreduce)
       !!
       If(.False.)Then
          Call write_L_potential(atom,maxL,p_potharl,p_potxcl,p_bfl)
       Endif
       !!
       !! calcualte expectation value of VH,Vxc
       Call calc_shell_hartree_expectation(atom,os,icomb,p_potharl)
       Call calc_shell_xc_expectation(atom,os,icomb,p_excl,p_potxcl,p_bfl,xc_v&
            &,p_rhol,p_spl,full_rho)


       !!
       !! play with the second order approximation to Exc
       !!
       Call calc_vxc_L0_kernel(atom,os,p_rho0,p_rhol,p_spl,x_only,icomb,xreduce)
       !!
       !!
       !! open-shell nonspherical contrib to shell expectation value
       !!
       !!
       vhar=0.0_t_real
       vxc =0.0_t_real
       vxc0 =0.0_t_real
       vhar0=0.0_t_real
       Do ms=1,atom% mspin
          Write(outfile,190,advance='no')ms
190       Format('al,snl: ',i1,4x)
          ldo2=0
          !vhar0=vhar0+fourpi/Real(4_t_int*ldo2+1_t_int,t_real)*Fl(ldo2,ms)&
          !     &*os% p_agsum(ldo2,icomb,ms)**2
          Write(outfile,91,advance='no') os% p_agsum(ldo2,icomb,ms)
          !Write(outfile,91,advance='no') os% p_bgsum(ldo2,icomb,ms)
          !!
          ldo2=0
          vhar0=vhar0+fourpi/Real(4_t_int*ldo2+1_t_int,t_real)*Fl(ldo2,ms)&
               &*os% p_agsum(ldo2,icomb,ms)**2
          vxc0 =vxc0 -fourpi/Real(4_t_int*ldo2+1_t_int,t_real)*Fl(ldo2,ms)&
               &*os% p_bgsum(ldo2,icomb,ms)
          !!
          Do ldo2=1,maxL
             vhar=vhar+fourpi/Real(4_t_int*ldo2+1_t_int,t_real)*Fl(ldo2,ms)&
                  &*os% p_agsum(ldo2,icomb,ms)**2
             vxc=vxc-fourpi/Real(4_t_int*ldo2+1_t_int,t_real)*Fl(ldo2,ms)&
                  &*os% p_bgsum(ldo2,icomb,ms)
             Write(outfile,91,advance='no') os% p_agsum(ldo2,icomb,ms)
             !Write(outfile,91,advance='no') os% p_bgsum(ldo2,icomb,ms)
          Enddo
          Write(outfile,2)
       Enddo
       vhar=vhar*0.5_t_real
       vhar0=vhar0*0.5_t_real
       vxc=vxc*0.5_t_real
       vxc0=vxc0*0.5_t_real

       


       If(HF)Then

          EHUiso=0.0_t_real
          EXUSIiso =0.0_t_real
          EXJiso=0.0_t_real
          Do ms=1,atom% mspin
             !! this is the isotropic Hartree U-term
             EHUiso=EHUiso+atom% p_nroccu(os% iv,ms)**2*0.5_t_real*Fl(0,ms)
             !! this is the isotropic U-SI term  -U Tr (n^s)^2
             !! thus it is the isotropic Hartree SIC
             dd=0.0_t_real
             Do m=-os% l,os% l
                dd=dd+os% p_comb(m+os% l+1,icomb,ms)&
                     &*os% p_comb(m+os% l+1,icomb,ms)
             Enddo
             EXUSIiso =EXUSIiso -Fl(0,ms)*dd
             !! this is the isotropic X-J-term: -(J sum_s (Ns)^2 )
             EXJiso=EXJiso-atom% p_nroccu(os% iv,ms)**2*J(ms)

             !! this is the isotropic X-J-SI-term: -( -J Tr (n^s)^2)
             dd=0.0_t_real
             Do m=-os% l,os% l
                dd=dd+os% p_comb(m+os% l+1,icomb,ms)&
                     &*os% p_comb(m+os% l+1,icomb,ms)
             Enddo
             EXJiso =EXJiso +J(ms)*dd

          Enddo
          EXUSIiso=EXUSIiso*0.5_t_real
          EXJiso=EXJiso*0.5_t_real

          Print *,'VDECOMP EH      isotropic=',vhar0*hatoev,EHUiso*hatoev
          Print *,'VDECOMP EX U-SI isotropic=',vxc0*hatoev,EXUSIiso*hatoev
          Print *,'VDECOMP EX J    isotropic=',EXJiso*hatoev
          
          Print *,'VDECOMP EH J  anisotropic=',vhar*hatoev
          Print *,'VDECOMP EX J  anisotropic=',(vxc-EXJiso)*hatoev
          

          Print *,'VDECOMP '
          Print *,'VDECOMP '

          os% p_exphar (icomb)=vhar+vhar0
          os% p_exphar0(icomb)=vhar0
          os% p_expxc  (icomb)=vxc+vxc0
          os% p_expxc0 (icomb)=vxc0
          os% p_exp    (icomb)=vhar+vxc+vhar0+vxc0
       Endif
       !Print *,'hartree from slater Fs'&
       !     &,ms,vhar,vxc,os% p_exphar(icomb)-os% p_exphar0(icomb)
       !!
       If(icomb>0)Then
          avexphar=avexphar+os% p_exphar(icomb)
          avexpxc =avexpxc +os% p_expxc (icomb)
          avexp   =avexp   +os% p_exphar(icomb) +os% p_expxc (icomb)
       Endif
    Enddo
    !!
    If(config_average) Then
       avexphar=avexphar/os% ncomb
       avexpxc =avexpxc /os% ncomb
       avexp   =avexp   /os% ncomb
    Else

       !icomb=0
       !Print *,'0: ',os% p_exphar(icomb),os% p_exphar0(icomb)
       !Print *,'0: ',os% p_expxc (icomb),os% p_expxc0 (icomb)
       !Print *,'0: ',os% p_exp   (icomb)
       !!
       avexphar=os% p_exphar(0)
       avexpxc =os% p_expxc (0)
       avexp   =os% p_exp   (0)
    Endif
    !!
    os% p_exphar=os% p_exphar-avexphar
    os% p_exphar0=os% p_exphar0-avexphar
    os% p_expxc =os% p_expxc -avexpxc
    os% p_expxc0 =os% p_expxc0 -avexpxc
    os% p_exp   =os% p_exp   -avexp
    !!
    !! sort expectation values according to total values for output
    Call sort_exp(os)
    !!
    !!
    Call pfree(p_rhol)
    Call pfree(p_rho0)
    Call pfree(p_spl)
    Call pfree(p_potharl)
    Call pfree(p_potxcl)
    Call pfree(p_excl)
    Call pfree(p_bfl)

    !!
    !!
    !! calc spherical vhartree and nonspherical compare with previous
    !! and Fl result!!!
    !!
    Call clean_openshell(os)
    !!
    Write(outfile,11)nsubfunc
    !!
    Return
    !Stop nsubfunc
    !!
1   Format(/,/,'============================== START ',a&
         &,'============================== ')
11  Format('============================== END   ',a&
         &,'============================== ',/,/)
    !!
  End Subroutine special_orb
  !!***********************************************************************
  Subroutine calc_vxc_L0_kernel(atom,os,p_rho0,p_rhol,p_spl,x_only,icomb&
       &,xreduce)
    !!
    Implicit None  
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Type(T_OpenShell),Intent(in)  :: os
    Logical(t_log),Intent(in)    :: x_only
    Integer(t_int),Intent(in)    :: icomb
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Pointer :: p_rho0  (:,:)
    Real(t_real),Pointer :: p_rhol  (:,:)
    Real(t_real),Pointer :: p_spl   (:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_vxc_L0_kernel"
    Logical(t_log) :: alert
    Integer(t_int) :: nvg,olxcv,ms
    Real(t_real)   :: ylm((2*os% l+1)**2)
    Integer(t_int) :: ir,ig,ldo2,l,m,ilm,la,iv0,mt,n,deg
    Real(t_real)   :: ri,wrsh,z,x,rinv,K,Q,P,mratio,rho,mag,e0,e,dums(2),dum
    Real(t_real)   :: rjac,fn,fm,fnm,B0,K0,Q0,P0,dE,Exc,bfl0,gamma,fsp
    Real(t_real)   :: rsh(3),rhl,spl,fr,kmm,Kst(2,2),Ns(2),NN,MM,pmm,Pst(2,2)
    Real(t_real)   :: Pstav,Kstav,Pstpm,Pstmm,Kstpm,Kstmm,ss,tt
    Real(t_real)   :: error,tol,ff,vex(2),epsxc,rhos(2),gsumn,gsumm,d
    Real(t_real)   :: U0,U0im,V0,UP
    Real(t_real)   :: texp(atom% mesh% nr)
    Real(t_real),Pointer :: p_kout   (:)
    Real(t_real),Pointer :: p_qout   (:)
    Real(t_real),Pointer :: p_pout   (:)
    Real(t_real),Pointer :: p_b0out  (:)
    Real(t_real),Pointer :: p_e0out  (:)
    Real(t_real),Pointer :: p_eout   (:)
    Real(t_real),Pointer :: p_kkk    (:)
    Real(t_real),Pointer :: p_ppp    (:)
    Real(t_real),Pointer :: p_V0out    (:)
    Real(t_real),Pointer :: p_U0out    (:)
    Real(t_real),Pointer :: p_UPout    (:)
    Real(t_real),Pointer :: p_U0imout    (:)
    !! DONT FREE THEM !
    Real(t_real),Pointer   :: p_cc(:),p_xx(:)
    !!
    la=os% l
    iv0=os% iv
    !!
    gamma=atom% fgamv
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,gamma,texp)
    !!
    Nullify(p_V0out)
    Nullify(p_U0out)
    Nullify(p_UPout)
    Nullify(p_U0imout)
    Nullify(p_kout)
    Nullify(p_qout)
    Nullify(p_pout)
    Nullify(p_b0out)
    Nullify(p_e0out)
    Nullify(p_eout)
    Nullify(p_kkk)
    Nullify(p_ppp)
    !!
    Call palloc('p_V0out ',p_V0out ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_U0out ',p_U0out ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_UPout ',p_UPout ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_U0imout ',p_U0imout ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    !!
    Call palloc('p_kout ',p_kout ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_qout ',p_qout ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_pout ',p_pout ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_b0out ',p_b0out ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_e0out ',p_e0out ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_eout  ',p_eout  ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_kkk   ',p_kkk   ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    Call palloc('p_ppp   ',p_ppp   ,1_t_int,atom% mesh% nr,nmodule,nsubfunc)
    !!
    !!
    If(x_only)Then
       olxcv=getxcversion()
       Print *,'before XC=',olxcv
       Call setxcversion(XCVER_XONLY)
       Print *,'inside XC=',getxcversion()
    End If
    !!
    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    ff=1.0_t_real
    !!
    !errtol=1.0e-12_t_real
    !!
    nvg=8*la+2_t_int !! maximum mesh size
    !!
    Call get_gauss(nvg,p_xx,p_cc)
    !! 
    !!
    irloop: Do ir=1,atom% mesh% nr
       ri=atom% mesh% p_rmesh(ir) 
       rjac=atom% mesh% p_jmesh(ir)
       !! put some values to get through
       If(atom% mesh% with_zero .And. (ir ==1 ))Then
          rinv=0.0_t_real
       Else
          rinv=1.0_t_real/ri
       Endif
       !!
       !!
       !! result array of all L-components
       UP=0.0_t_real
       U0=0.0_t_real
       V0=0.0_t_real
       U0im=0.0_t_real
       e=0.0_t_real
       igloop: Do ig=1,nvg
          z=p_xx(ig)
          x=Sqrt(1.0_t_real-z*z)
          wrsh=p_cc(ig)*twopi
          !!
          rsh(:)=ri*(/x,0.0_t_real,z/)
          !!
          Call ylms12(ylm,rsh(1),rsh(2),rsh(3),2*la)
          !!
          !!
          rho=0.0_t_real
          mag=0.0_t_real
          Do ldo2=0,la
             l=ldo2*2
             ilm=l*l+l+1
             rho=rho+p_rhol(ir,ldo2) *ylm(ilm)
             mag=mag+p_spl (ir,ldo2) *ylm(ilm)
          Enddo
          !!
          rhos(1)=(rho+mag)*0.5_t_real*rinv**2
          rhos(2)=(rho-mag)*0.5_t_real*rinv**2
          !!
          !! xc-potential
          Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rhos,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          !!
          !!
          e=e+rho*epsxc*wrsh*rjac
          !!
          !! Perdew
          rho=p_rho0(ir,2)/sq4pi
          mag=p_rho0(ir,2)/sq4pi
          !!
          rhos(1)=(rho+mag)*0.5_t_real*rinv**2
          rhos(2)=(rho-mag)*0.5_t_real*rinv**2
          Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rhos,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          UP=UP+rho*epsxc*wrsh*rjac
          !!
          !! U[rho0]
          !!
          rho=p_rho0(ir,1)/sq4pi
          mag=0.0_t_real
          !!
          rhos(1)=(rho+mag)*0.5_t_real*rinv**2
          rhos(2)=(rho-mag)*0.5_t_real*rinv**2
          Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rhos,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          U0=U0+rho*epsxc*wrsh*rjac
          !!
          !! spherical part of orbital density
          rho=rho+p_rho0(ir,2)/sq4pi
          mag=p_rho0(ir,2)/sq4pi
          !Do ldo2=1,la
          !   l=ldo2*2
          !   ilm=l*l+l+1
          !   rho=rho+p_rhol(ir,ldo2) *ylm(ilm)
          !Enddo


          V0=V0+(vex(1)+vex(2))*0.5_t_real*mag*wrsh*rjac


          rhos(1)=(rho+mag)*0.5_t_real*rinv**2
          rhos(2)=(rho-mag)*0.5_t_real*rinv**2
          !!
          Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
          Call vxcrho(atom% mspin,rhos,dums,dum,dums,dums,dum,vex,epsxc&
               &,xreduce)
          U0im=U0im+rho*epsxc*wrsh*rjac
          !!
       Enddo igloop
       !!
       p_V0out(ir)=V0
       p_U0out(ir)=U0
       p_UPout(ir)=UP
       p_U0imout(ir)=U0im
       !!
       p_eout(ir)=e
       !!
       !!
       !!
       !!
       !! calculate the spherical expansion
       !!
       !! xc-potential
       !mratio=1.0_t_real
       !mratio=0.7_t_real
       !mratio=0.5_t_real
       !mratio=0.3_t_real
       mratio=0.0_t_real
       !!
       rho=p_rhol(ir,0)
       mag=mratio*p_spl (ir,0)
       !!
       rhos(1)=(rho +mag)*0.5_t_real/sq4pi*rinv**2
       rhos(2)=(rho -mag)*0.5_t_real/sq4pi*rinv**2
       !!
       If(ir == atom% mesh% nr .Or. ir == 1) Then
          K=0.0_t_real
          Q=0.0_t_real
          P=0.0_t_real
          epsxc=0.0_t_real
          bfl0=0.0_t_real
       Else
          Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
          Call kxcrho(atom% mspin,rhos,K,Q,P,vex,epsxc,1.0_t_real)
          bfl0=(vex(1)-vex(2))*0.5_t_real*sq4pi
       Endif
       !!
       !!
       !!
       !! n0,m0 expansion
       !!
       p_e0out(ir)=rho*epsxc*sq4pi*rjac
       p_b0out(ir)=(1.0_t_real-mratio)*p_spl (ir,0)*bfl0*rjac
       !!
       !!
       !!
       !!
       !!
       !!
       fn=0.0_t_real
       fm=0.0_t_real
       fnm=0.0_t_real
       !!
       !fm =fm +((1.0_t_real-mratio)*p_spl (ir,0))**2
       fnm=0.0_t_real
       !!
       Do ldo2=0,la

          fr=1.0_t_real
          If(ldo2==0)fr=1.0_t_real-mratio

          rhl=0.0_t_real
          spl=0.0_t_real

          Do ms=1,atom% mspin
             fsp=(3.0_t_real-2.0_t_real*ms)*(atom% mspin-1_t_int)
             gsumn=os% p_agsum(ldo2,icomb,ms)
             gsumm=os% p_agsum(ldo2,icomb,ms)*fsp
             !!
             !!
             !!
             If(ldo2>0)Then
                rhl=rhl+psisquare(atom,ir,iv0,ms,texp)*gsumn
             Endif
             spl =spl+psisquare(atom,ir,iv0,ms,texp)*gsumm
          Enddo

          fn =fn +rhl**2
          fm =fm +spl**2*fr**2
          fnm=fnm+rhl*spl*fr
       Enddo
       p_kout(ir)=0.5_t_real*K*fm*rinv**2*rjac
       p_qout(ir)=1.0_t_real*Q*fnm*rinv**2*rjac
       p_pout(ir)=0.5_t_real*P*fn*rinv**2*rjac
       !!
       !!
    Enddo irloop
    !!
    !!
    !p_kkk=p_rho0(:,2)*sq4pi*atom% mesh% p_jmesh
    !Call lag4int(atom% mesh% nr,p_kkk,atom% mesh% hx,V0,lagint_out)
    !Print *,'U   int rho           =',V0
    Call lag4int(atom% mesh% nr,p_V0out,atom% mesh% hx,V0,lagint_out)
    Call lag4int(atom% mesh% nr,p_U0out,atom% mesh% hx,U0,lagint_out)
    Call lag4int(atom% mesh% nr,p_UPout,atom% mesh% hx,UP,lagint_out)
    Call lag4int(atom% mesh% nr,p_U0imout,atom% mesh% hx,U0im,lagint_out)
    Print *,'UP[rhoim]            =',UP*hatoev
    Print *,'U[rho0]              =',U0*hatoev
    Print *,'V[rho0]rhoim         =',V0*hatoev
    Print *,'U[rho0+rhoim]        =',U0im*hatoev
    Print *,'U[SIC]               =',(U0im-V0-U0)*hatoev
    Print *,'U '

    !!
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,p_rho0(:,1)+p_rho0(:,2)-p_rhol(:,0),12,'+drho')
    !!
    !!
    !!
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,p_kkk,12,'+ne')
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,p_ppp,12,'+neapprox')
    !!
    !! P,K-integrals
    !!
    Do ms=1,atom% mspin
       Do mt=1,atom% mspin
          !! 
          p_kkk(:)=0.0_t_real
          p_ppp(:)=0.0_t_real
          !!
          irloop1: Do ir=1,atom% mesh% nr
             ri=atom% mesh% p_rmesh(ir) 
             rjac=atom% mesh% p_jmesh(ir)
             !! put some values to get through
             If(atom% mesh% with_zero .And. (ir ==1 ))Then
                rinv=0.0_t_real
             Else
                rinv=1.0_t_real/ri
             Endif
             !!
             !! calculate the spherical expansion
             !!
             !! xc-potential
             rhos(1)=(p_rhol(ir,0) +mratio*p_spl (ir,0))*0.5_t_real/sq4pi*rinv**2
             rhos(2)=(p_rhol(ir,0) -mratio*p_spl (ir,0))*0.5_t_real/sq4pi*rinv**2
             !!
             If(ir == atom% mesh% nr  .Or. ir == 1) Then
                K=0.0_t_real
                Q=0.0_t_real
                P=0.0_t_real
             Else
                Call adjustrho(rhos,ff,error,atom% mspin,tol,alert)
                Call kxcrho(atom% mspin,rhos,K,Q,P,vex,epsxc,1.0_t_real)
             Endif
             !!
             !!
             !!
             !! now, we lost the polarization of the rest
             !!
             !!
             !!
             d=psisquare(atom,ir,iv0,ms,texp)&
                  &*psisquare(atom,ir,iv0,mt,texp)&
                  &*rinv**2*rjac/fourpi
             
             p_kkk(ir)=p_kkk(ir)+2.0_t_real*K*d
             p_ppp(ir)=p_ppp(ir)+2.0_t_real*P*d

             !!
          Enddo irloop1
          !!
          Call lag4int(atom% mesh% nr,p_kkk,atom% mesh% hx,Kst(ms,mt),lagint_out)
          Call lag4int(atom% mesh% nr,p_ppp,atom% mesh% hx,Pst(ms,mt),lagint_out)
          !!
       Enddo
    Enddo
    !!
    !!
    !! sum the energies
    !!
    deg=2*la+1
    !!
    NN=0.0_t_real
    MM=0.0_t_real
    Ns(:)=0.0_t_real
    Do ms=1,atom% mspin
       Do m=-la,la
          Ns(ms)=Ns(ms) + os% p_comb(m+la+1,icomb,ms)
       Enddo
       NN=NN+Ns(ms)
       MM=MM+Ns(ms)*(3.0_t_real-2.0_t_real*ms)
    Enddo
    !!
    !!
    Print *,'Ns=',Ns(1:2)
    Print *,'N,M=',NN,MM
    !!
    Pstav=0.0_t_real
    Pstpm=0.0_t_real
    Pstmm=0.0_t_real

    Kstav=0.0_t_real
    Kstpm=0.0_t_real
    Kstmm=0.0_t_real
    Do ms=1,atom% mspin
       Do mt=1,atom% mspin
          Pstav=Pstav+Pst(ms,mt)
          Pstpm=Pstpm+Pst(ms,mt)*(3-2*mt)
          Pstmm=Pstmm+Pst(ms,mt)*(3-2*mt)*(3-2*ms)

          Kstav=Kstav+Kst(ms,mt)
          Kstpm=Kstpm+Kst(ms,mt)*(3-2*mt)
          Kstmm=Kstmm+Kst(ms,mt)*(3-2*mt)*(3-2*ms)
       Enddo
    Enddo
    Pstav=Pstav*0.25_t_real
    Pstpm=Pstpm*0.25_t_real
    Pstmm=Pstmm*0.25_t_real

    Kstav=Kstav*0.25_t_real
    Kstpm=Kstpm*0.25_t_real
    Kstmm=Kstmm*0.25_t_real
    !!
    !pmm=-0.25_t_real*(Pstav*NN**2+2.0_t_real*Pstpm*NN*MM+Pstmm*MM**2 )
    pmm=0.0_t_real !! todo: is this correct?
    kmm=+0.25_t_real*(Kstav*MM**2+2.0_t_real*Kstpm*NN*MM+Kstmm*NN**2 )&
         &*((1.0_t_real-mratio)**2-1.0_t_real)

    !Print *,'kmm=',kmm*hatoev
    kmm=kmm+0.25_t_real*(2.0_t_real*Kstpm*NN*MM) !! big
    !Print *,'kmm=',kmm*hatoev
    kmm=kmm+0.25_t_real*(Kstmm*NN**2)*1 !! small
    !Print *,'kmm=',kmm*hatoev


    !!
    Do ms=1,atom% mspin
       Do mt=1,atom% mspin

          ss=3.0_t_real-2.0_t_real*ms
          tt=3.0_t_real-2.0_t_real*mt


          !!
          !!
          !!
          spl=0.0_t_real
          Do m=-la,la
             Do n=-la,la
                
                gsumm=0.0_t_real
                Do ldo2=0,la
                   gsumm=gsumm&
                        &  +gaunt(la,m,ldo2*2,0_t_int,la,m)&
                        &  *gaunt(la,n,ldo2*2,0_t_int,la,n)*fourpi
                Enddo
                
                If(la<=2)Then
                   d=gsumm-(deg)/(deg+2.0_t_real)&
                        &*(1.0_t_real+delta(m,n)+delta(m,-n))
                   If(Abs(d)>1.0e-14_t_real)Then
                      Print "('dWWWW ',e30.15e3)",d
                      Stop 'WWWW'
                   Endif
                Endif
                
                
                spl=spl   + gsumm&
                     &*os% p_comb(m+la+1,icomb,ms)&
                     &*os% p_comb(n+la+1,icomb,mt)


             Enddo
          Enddo
          !!
          !!

          kmm=kmm+0.25_t_real*spl *Kstav *ss *tt !! big
          
          !!
          rhl=0.0_t_real
          spl=0.0_t_real
          Do m=-la,la
             !dd=0.0_t_real
             Do n=-la,la
                
                gsumm=0.0_t_real
                Do ldo2=0,la
                   gsumm=gsumm&
                        &  +gaunt(la,m,ldo2*2,0_t_int,la,m)&
                        &  *gaunt(la,n,ldo2*2,0_t_int,la,n)*fourpi
                Enddo

                If(la<=2)Then
                   d=gsumm-deg/(deg+2.0_t_real)&
                     &*(1.0_t_real+delta(m,n)+delta(m,-n))
                   If(Abs(d)>1.0e-14_t_real)Then
                      Print "('dWWWW ',e30.15e3)",d
                      Stop 'WWWW'
                   Endif
                Endif


                spl=spl   + gsumm&
                     &*(os% p_comb(m+la+1,icomb,ms)-Ns(ms)/deg)&
                     &*(os% p_comb(n+la+1,icomb,mt)-Ns(mt)/deg)

                rhl=rhl   + gsumm&
                     &*(os% p_comb(m+la+1,icomb,ms)-Ns(ms)/deg)&
                     &*(os% p_comb(n+la+1,icomb,mt)-Ns(mt)/deg)

                !dd=dd+gsumm
                
             Enddo
             !Print *,'sum a_mn=',dd
          Enddo
          !!
          !!

          kmm=kmm+0.25_t_real*spl *(1*Kstpm*(ss+tt)+1*Kstmm*ss*tt)*ss*tt*1 !!small
          
          pmm=pmm+0.25_t_real*rhl*(Pstav+Pstpm*( ss+tt)+Pstmm*ss*tt)
          
       End Do
    End Do
    !!
    !Call writefunc(atom% mesh% nr,atom% mesh% p_rmesh,p_potxcl(:,0),12,'+v')
    !!
    !!
    If(alert)Then

3      Format('''Numerical inaccuracy in vxc/exc &
            &calculation! '',/,'' maxerror=',e12.4e3,'''')
       Write(errbuf,3) error
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif

    !!
    If(x_only) Then
       Call setxcversion(olxcv)
       Print *,'after XC=',getxcversion()
    End If
    !!
    Call lag4int(atom% mesh% nr,p_eout,atom% mesh% hx,Exc,lagint_out)
    !!
    Call lag4int(atom% mesh% nr,p_e0out,atom% mesh% hx,E0,lagint_out)
    Call lag4int(atom% mesh% nr,p_b0out,atom% mesh% hx,B0,lagint_out)
    Call lag4int(atom% mesh% nr,p_kout,atom% mesh% hx,K0,lagint_out)
    Call lag4int(atom% mesh% nr,p_qout,atom% mesh% hx,Q0,lagint_out)
    Call lag4int(atom% mesh% nr,p_pout,atom% mesh% hx,P0,lagint_out)
    !!
    Print *,'<EXC    >=',Exc*hatoev
    !!
    Print *,'<E0>     =',E0*hatoev
    Print *,'<EXC-E0 >=',(Exc-E0)*hatoev
    !!
    Print *,'<B0 m>   =',B0*hatoev
    Print *,'<K0 m^2> =',K0*hatoev
    Print *,'<Q0 n*m> =',Q0*hatoev
    Print *,'<P0 n^2> =',P0*hatoev
    Print *,'<E2    > =',(P0+Q0+K0)*hatoev
    dE=B0+(P0+Q0+K0)
    Print *,'<dE=B0+E2>=',dE*hatoev
    Print *,'<err dE>=',(Exc-e0-dE)/dE*100.0_t_real
    !!
    Print *,'<Kmm>    =',kmm*hatoev
    Print *,'<dKmm>    =',(kmm-K0)/K0*100.0_t_real
    Print "('<Kst[eV]> =',4(f15.8,1x))",Kst(:,:)*hatoev
    Print "('<Kstav[eV]> =',3(f15.8,1x))",Kstav*hatoev,Kstpm*hatoev,Kstmm*hatoev
    !!
    Print *,'<Pmm>    =',pmm*hatoev
    Print "('<Pst[eV]> =',4(f15.8,1x))",Pst(:,:)*hatoev
    Print "('<Pstav[eV]> =',3(f15.8,1x))",Pstav*hatoev,Pstpm*hatoev,Pstmm*hatoev
    !!
    Print *,'K'
    Print *,'P'
    Call pfree(p_kout )
    Call pfree(p_qout )
    Call pfree(p_pout )
    Call pfree(p_b0out )
    Call pfree(p_e0out )
    Call pfree(p_eout )
    Call pfree(p_kkk )
    Call pfree(p_ppp )
    Call pfree(p_V0out )
    Call pfree(p_U0out )
    Call pfree(p_UPout )
    Call pfree(p_U0imout )
    !!
    Return
    !!
  End Subroutine calc_vxc_L0_kernel
  !!***********************************************************************
  Real(t_real) Function  delta(i,j)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: i,j
    !!LOCAL
    !!
    !!
    delta=0.0_t_real
    If(i==j)delta=1.0_t_real
    !!
    Return
  End Function delta
  !!***********************************************************************
  Real(t_real) Function psisquare(atom,ir,iv,ms,texp)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Integer(t_int),Intent(in)    :: ir,iv,ms
    Real(t_real),Intent(in)      :: texp(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="psisquare"
    Logical(t_log) :: khsrel,rel,nrel
    !!
    rel=(atom% relativistic == FULL_RELATIVISTIC) 
    khsrel=(atom% relativistic == SCALAR_RELATIVISTIC_KH)
    nrel=.Not.(rel.Or.khsrel)
    !!
    If(khsrel)Then
       psisquare=(&
            &atom% p_fv(ir,iv,LARGE_COMP1,ms)**2+&
            &atom% p_fv(ir,iv,SMALL_COMP1,ms)**2+&
            &atom% p_fv(ir,iv,LARGE_COMP2,ms)**2&
            &)*texp(ir)**2
    Else If(nrel) Then
       psisquare=(&
            &atom% p_fv(ir,iv,LARGE_COMP1,ms)**2&
            &)*texp(ir)**2
    Else
       Stop 'psisquare relativistic mode not implemented'
    Endif
    !!
    Return
    !!
  End Function psisquare
  !!***********************************************************************
  Subroutine spsisquare(atom,iv,ms,psisq)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Integer(t_int),Intent(in)    :: iv,ms
    Real(t_real),Intent(out)     :: psisq(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="psisquare"
    Logical(t_log) :: khsrel,rel,nrel
    Real(t_real)   :: gamma
    Real(t_real)   :: texp(atom% mesh% nr)
    !!
    rel=(atom% relativistic == FULL_RELATIVISTIC) 
    khsrel=(atom% relativistic == SCALAR_RELATIVISTIC_KH)
    nrel=.Not.(rel.Or.khsrel)
    !!
    gamma=atom% fgamv
    Call gam_fac_mesh(atom% mesh,atom% gam_factor,gamma,texp)
    !!
    If(khsrel)Then
       psisq(:)=(&
            & atom% p_fv(:,iv,LARGE_COMP1,ms)**2&
            &+atom% p_fv(:,iv,SMALL_COMP1,ms)**2&
            &+atom% p_fv(:,iv,LARGE_COMP2,ms)**2&
            &)*texp(:)**2
    Else If(nrel) Then
       psisq(:)=(&
            &atom% p_fv(:,iv,LARGE_COMP1,ms)**2&
            &)*texp(:)**2
    Else
       Stop 'spsisquare relativistic mode not implemented'
    Endif
    !!
    Return
    !!
  End Subroutine spsisquare
  !!***********************************************************************

End Module diracspecial
