!!----DEBUG options-----------------|
!! SUBFUNC    | MODES               |
!!----------------------------------|
!! EFG:vlocefg:                     !
!!----------------------------------|
Module efg
  !!
  Use precisiondefs
  Use errors
  Use memory
  Use matserv
  Use constants
  Use file_iounits
  Use mesh
  Use vecharm
  Use tsymmetry
  Use finuc
  Use diagonalize
  Use ioaux
  Use trelativistic
  Use tmesh
  Use tnumerics
  Use atomdefs
  Use integrate1d
  Use torbitalbasis
  Use tpotentialdensity
  Use tkohnsham
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="EFG"  
  !!
  Public :: calc_efg,efg_contributions
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine efg_contributions(mspin,structure,kohnsham,basis,numerics&
       &,relativistic)
    !!
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin,relativistic
    Type(T_Structure),Intent(in) :: structure
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Basis),Intent(in)     :: basis
    !!LOCAL
    Character(*),Parameter::nsubfunc="efg_contributions"
    Character(*),Parameter   :: mesg='EFG: '
    Logical(t_log) :: prot
    Integer(t_int) :: isa,issa,i,j,la,na,lb,nb,iva,ivb,nRc,nRv,glmax,ica,icb
    Integer(t_int) :: par
    Real(t_real)   :: unit,efgmatsum(3,3),d,e(3),z(3,3),v(3),Vzz,eta,Vzzsum
    Type(T_Mesh),Pointer :: bmesh
    Real(t_real),Pointer     :: wgaunt(:,:,:)
    Real(t_real),Pointer :: efgmatvv(:,:,:,:)
    Real(t_real),Pointer :: efgmatcv(:,:,:,:)
    Real(t_real),Pointer :: efgmatcc(:,:,:,:)
    Real(t_real),Pointer :: p_n(:,:)
    Real(t_real),Pointer :: p_z(:,:)
    Real(t_real),Pointer :: p_e(:)
    !!
    Return
    prot=(outlevel>=L_many)
    !!
    Nullify(wgaunt,efgmatvv,efgmatcv,efgmatcc,p_n,p_z,p_e)
    !!
    unit=hatoev/brtoan**2/10.0_t_real !! in 10^21 V/m^2
    !!
    If(relativistic==FULL_RELATIVISTIC)Then
       Write(errbuf,"('''Orbital resolved onsite EFG not yet &
            &implemented for full relativistic.''')")
       Call errmsg(nmodule,nsubfunc,E_information,msg=errbuf)
       Return
    Endif
    !!
    glmax=basis% lmx
    !If(rel) glmax=basis% lmx+1
    !!
    Call palloc('wgaunt',wgaunt&
         &,1_t_int,9_t_int&
         &,1_t_int,(glmax+1_t_int)**2_t_int&
         &,1_t_int,(glmax+1_t_int)**2_t_int&
         &,nmodule,nsubfunc)
    !!
    Call makergaunt(wgaunt,2_t_int,glmax,glmax) 
    !!
    !!
    isaloop: Do isa=1,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !!
       Write(outfile,2) Trim(mesg)
       Write(outfile,10) Trim(mesg),structure% p_elements(issa)&
            &,Trim(intToChar10(isa))
       Write(outfile,2) Trim(mesg)
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       !!
       nRc=basis% p_nRc(issa)
       nRv=basis% p_nRv(issa)
       !!
       Call palloc('efgmatcc',efgmatcc,1_t_int,3_t_int,1_t_int,3_t_int&
            &,1_t_int,nRc,1_t_int,nRc,nmodule,nsubfunc)
       !!
       Call snrel_orbital_efg(mspin,isa,bmesh,basis% ncomp,nRc,nRc&
            &,basis% p_lRc(:,issa),basis% p_lRc(:,issa)&
            &,basis% p_fc(:,:,:,issa),basis% p_fc(:,:,:,issa)&
            &,basis% p_fgam(issa)&
            &,numerics% degree,numerics% oci_nr,wgaunt&
            &,kohnsham% p_noncc&
            &,efgmatcc)
       !!
       Call palloc('efgmatcv',efgmatcv,1_t_int,3_t_int,1_t_int,3_t_int&
            &,1_t_int,nRc,1_t_int,nRv,nmodule,nsubfunc)
       !!
       Call snrel_orbital_efg(mspin,isa,bmesh,basis% ncomp,nRc,nRv&
            &,basis% p_lRc(:,issa),basis% p_lRv(:,issa)&
            &,basis% p_fc(:,:,:,issa),basis% p_fv(:,:,:,issa)&
            &,basis% p_fgam(issa)&
            &,numerics% degree,numerics% oci_nr,wgaunt&
            &,kohnsham% p_noncv&
            &,efgmatcv)
       !!
       !!
       Call palloc('efgmatvv',efgmatvv,1_t_int,3_t_int,1_t_int,3_t_int&
            &,1_t_int,nRv,1_t_int,nRv,nmodule,nsubfunc)
       !!
       Call snrel_orbital_efg(mspin,isa,bmesh,basis% ncomp,nRv,nRv&
            &,basis% p_lRv(:,issa),basis% p_lRv(:,issa)&
            &,basis% p_fv(:,:,:,issa),basis% p_fv(:,:,:,issa)&
            &,basis% p_fgam(issa)&
            &,numerics% degree,numerics% oci_nr,wgaunt&
            &,kohnsham% p_nonvv&
            &,efgmatvv)

       !!
       efgmatsum=0.0_t_real
       Do iva=1,nRv
          Do ivb=1,nRv
             efgmatsum=efgmatsum+efgmatvv(:,:,iva,ivb)
          Enddo
       Enddo
       Do ica=1,nRc
          Do ivb=1,nRv
             efgmatsum=efgmatsum+efgmatcv(:,:,ica,ivb)*2.0_t_real
          Enddo
       Enddo
       Do ica=1,nRc
          Do icb=1,nRc
             efgmatsum=efgmatsum+efgmatcc(:,:,ica,icb)
          Enddo
       Enddo
       !!
       !!
       Call mainaxis_decopose(efgmatsum,z,e)
       If(Abs(e(1))>Abs(e(2)) )Then
          d=e(1)
          e(1)=e(2)
          e(2)=d
          v=Z(:,1)
          Z(:,1)=Z(:,2)
          Z(:,2)=v
       Endif
       If(Abs(e(2))>Abs(e(3)) )Then
          d=e(2)
          e(2)=e(3)
          e(3)=d
          v=Z(:,2)
          Z(:,2)=Z(:,3)
          Z(:,3)=v
       Endif
       If(Abs(e(1))>Abs(e(3)) )Then
          d=e(1)
          e(1)=e(3)
          e(3)=d
          v=Z(:,1)
          Z(:,1)=Z(:,3)
          Z(:,3)=v
       Endif
       !!
       If(prot)Then
          !!
          Call print_contrib_efgmat(nRc,nRc&
               &,basis% p_lRc(:,issa),basis% p_lRc(:,issa)&
               &,basis% p_nnRc(:,issa),basis% p_nnRc(:,issa)&
               &,efgmatcc,mesg,unit)
          Call print_contrib_efgmat(nRc,nRv&
               &,basis% p_lRc(:,issa),basis% p_lRv(:,issa)&
               &,basis% p_nnRc(:,issa),basis% p_nnRv(:,issa)&
               &,efgmatcv,mesg,unit)
          Call print_contrib_efgmat(nRv,nRv&
               &,basis% p_lRv(:,issa),basis% p_lRv(:,issa)&
               &,basis% p_nnRv(:,issa),basis% p_nnRv(:,issa)&
               &,efgmatvv,mesg,unit)
          Write(outfile,2) Trim(mesg)
          Write(outfile,15) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') efgmatsum(i,j)*unit
             Enddo
             Write(outfile,*)
          Enddo
       Endif
       !!
       !!
       !!
       Write(outfile,2)Trim(mesg)
       Write(outfile,2)Trim(mesg)//' Sum of all onsite orbital contributions'
       Write(outfile,2)Trim(mesg)
       d=Max(Abs(e(1)),Abs(e(2)))
       If(d<1.0e-6_t_real)d=1.0_t_real
       If(Abs(e(1)-e(2))/d<1.0e-6_t_real)Then
          Write(outfile,415)Trim(mesg)
       Else
          Write(outfile,416)Trim(mesg),z(:,1)
          Write(outfile,426)Trim(mesg),z(:,2)
       Endif
       Write(outfile,436)Trim(mesg),z(:,3)
       Write(outfile,2)Trim(mesg)
       !!
       If(All(Abs(e(1:3))<1.0e-8_t_real))Then
          Vzz=0.0_t_real
          eta=0.0_t_real
          Write(outfile,446)Trim(mesg),Vzz*unit,eta
       Else If(Abs(e(3))<1.0e-8_t_real)Then
          Vzz=e(3)
          If(Abs(e(1)-e(2))<1.0e-8_t_real)Then
             eta=0.0_t_real
             Write(outfile,446)Trim(mesg),Vzz*unit,eta
          Else
             Write(outfile,4461)Trim(mesg),Vzz*unit
          Endif
       Else
          Vzz=e(3)
          eta=(e(1)-e(2))/Vzz
          Write(outfile,446)Trim(mesg),Vzz*unit,eta
       Endif
       Write(outfile,2)Trim(mesg)
       Write(outfile,2)Trim(mesg)//' Single orbital contributions along &
            &these axes'
       Write(outfile,2)Trim(mesg)
       !!
       Do par=0,1
          !!cc
          Vzzsum=0.0_t_real
          !! table header
          Write(outfile,"(a,'  Vzz     ')",advance='no')Trim(mesg)
          Do icb=1,nRc
             lb=basis% p_lRc(icb,issa)
             nb=basis% p_nnRc(icb,issa)
             If(Mod(lb,2_t_int)/=par) Cycle
             Write(outfile,"(i1,a1,'(',i2,')  ')",advance='no')nb,lqnchar(lb),icb
          Enddo
          Write(outfile,*)
          !! even parity table 
          Do ica=1,nRc
             la=basis% p_lRc(ica,issa)
             na=basis% p_nnRc(ica,issa)
             If(Mod(la,2_t_int)/=par) Cycle
             Write(outfile,"(a,1x,i1,a1,'(',i2,') ')",advance='no')&
                  &Trim(mesg),na,lqnchar(la),ica
             Do icb=1,nRc
                lb=basis% p_lRc(icb,issa)
                nb=basis% p_nnRc(icb,issa)
                If(Mod(lb,2_t_int)/=par) Cycle
                
                Vzz=0.0_t_real
                If(Abs(la-2)<=lb .And. lb<=la+2)Then
                   Vzz=Dot_product(z(1:3,3)&
                        &,Matmul(efgmatcc(1:3,1:3,ica,icb),z(1:3,3)))*unit
                   !!eta=(e(1)-e(2))/Vzz
                Endif
                Vzzsum=Vzzsum+Vzz
                If(Abs(Vzz)<1.0e-4_t_real)Then
                   Write(outfile,"('    0   ')",advance='no') 
                Else
                   Write(outfile,"(f8.4)",advance='no') Vzz
                Endif
             End Do
             Write(outfile,*)
          End Do
          Write(outfile,2)Trim(mesg)
          Write(outfile,447)Trim(mesg),'cc',Vzzsum
          Write(outfile,2)Trim(mesg)
       End Do
       !!
       !!
       !! cv
       Do par=0,1
          Vzzsum=0.0_t_real
          !! table header
          Write(outfile,"(a,'  Vzz     ')",advance='no')Trim(mesg)
          Do ivb=1,nRv
             lb=basis% p_lRv(ivb,issa)
             nb=basis% p_nnRv(ivb,issa)
             If(Mod(lb,2_t_int)/=par) Cycle
             Write(outfile,"(i1,a1,'(',i2,')  ')",advance='no')nb,lqnchar(lb),ivb
          Enddo
          Write(outfile,*)
          !! even parity table 
          Do ica=1,nRc
             la=basis% p_lRc(ica,issa)
             na=basis% p_nnRc(ica,issa)
             If(Mod(la,2_t_int)/=par) Cycle
             Write(outfile,"(a,1x,i1,a1,'(',i2,') ')",advance='no')&
                  &Trim(mesg),na,lqnchar(la),ica
             Do ivb=1,nRv
                lb=basis% p_lRv(ivb,issa)
                nb=basis% p_nnRv(ivb,issa)
                If(Mod(lb,2_t_int)/=par) Cycle
                
                Vzz=0.0_t_real
                If(Abs(la-2)<=lb .And. lb<=la+2)Then
                   Vzz=Dot_product(z(1:3,3)&
                        &,Matmul(efgmatcv(1:3,1:3,ica,ivb),z(1:3,3)))*unit&
                        &*2.0_t_real
                   !!eta=(e(1)-e(2))/Vzz
                Endif
                Vzzsum=Vzzsum+Vzz
                If(Abs(Vzz)<1.0e-4_t_real)Then
                   Write(outfile,"('    0   ')",advance='no') 
                Else
                   Write(outfile,"(f8.4)",advance='no') Vzz
                Endif
             End Do
             Write(outfile,*)
          End Do
          Write(outfile,2)Trim(mesg)
          Write(outfile,447)Trim(mesg),'cv',Vzzsum
          Write(outfile,2)Trim(mesg)
       End Do
       !!
       !! vv
       Do par=0,1
          Vzzsum=0.0_t_real
          !! table header
          Write(outfile,"(a,'  Vzz     ')",advance='no')Trim(mesg)
          Do ivb=1,nRv
             lb=basis% p_lRv(ivb,issa)
             nb=basis% p_nnRv(ivb,issa)
             If(Mod(lb,2_t_int)/=par) Cycle
             Write(outfile,"(i1,a1,'(',i2,')  ')",advance='no')nb,lqnchar(lb),ivb
          Enddo
          Write(outfile,*)
          !! even parity table 
          Do iva=1,nRv
             la=basis% p_lRv(iva,issa)
             na=basis% p_nnRv(iva,issa)
             If(Mod(la,2_t_int)/=par) Cycle
             Write(outfile,"(a,1x,i1,a1,'(',i2,') ')",advance='no')&
                  &Trim(mesg),na,lqnchar(la),iva
             Do ivb=1,nRv
                lb=basis% p_lRv(ivb,issa)
                nb=basis% p_nnRv(ivb,issa)
                If(Mod(lb,2_t_int)/=par) Cycle
                
                Vzz=0.0_t_real
                If(Abs(la-2)<=lb .And. lb<=la+2)Then
                   Vzz=Dot_product(z(1:3,3)&
                        &,Matmul(efgmatvv(1:3,1:3,iva,ivb),z(1:3,3)))*unit
                   !!eta=(e(1)-e(2))/Vzz
                Endif
                Vzzsum=Vzzsum+Vzz
                If(Abs(Vzz)<1.0e-4_t_real)Then
                   Write(outfile,"('    0   ')",advance='no') 
                Else
                   Write(outfile,"(f8.4)",advance='no') Vzz
                Endif
             End Do
             Write(outfile,*)
          End Do
          Write(outfile,2)Trim(mesg)
          Write(outfile,447)Trim(mesg),'vv',Vzzsum
          Write(outfile,2)Trim(mesg)
       End Do
       !!
       !!
!!$       lenv=kohnsham% p_vblocklength(isa)
!!$       !!
!!$       Call palloc('p_n',p_n,1_t_int,lenv,1_t_int,lenv,nmodule,nsubfunc)
!!$       Call palloc('p_z',p_z,1_t_int,lenv,1_t_int,lenv,nmodule,nsubfunc)
!!$       Call palloc('p_e',p_e,1_t_int,lenv,nmodule,nsubfunc)
!!$       
!!$       !!
!!$       Do i=1,lenv
!!$          Do j=1,lenv
!!$             p_n(i,j)=Sum(kohnsham% p_nonvv(i,j,isa,1:mspin))
!!$             Write(outfile,"(1x,f12.8)",advance='no')p_n(i,j)
!!$          Enddo
!!$          Write(outfile,*)
!!$       Enddo
!!$       !!
!!$       !!
!!$       Call rdiagno_noO(lenv,p_n,p_z,p_e)
!!$       !!
!!$       !!
!!$       Do j=lenv,1,-1
!!$          Write(outfile,"('n=',f9.5)",advance='no')p_e(j)
!!$          i=0
!!$          Do iva=1,nRv
!!$             la=basis% p_lRv(iva,issa)
!!$             na=basis% p_nnRv(iva,issa)
!!$             Do ma=1,2*la+1
!!$                i=i+1
!!$                d=Abs(p_z(i,j))**2
!!$                If(d>0.1_t_real)Then
!!$                   Write(outfile,"(3x,i1,a1,'(',a,')',sp,i2,ss,' : ',f6.2,'%')",advance='no')&
!!$                        &na,lqnchar(la),Trim(intToChar10(iva)),ma-la-1,d
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          Write(outfile,*)
!!$       Enddo
       !!
    Enddo isaloop
    !!
    !!
    !!
    Call pfree(wgaunt)
    Call pfree(efgmatvv)
    Call pfree(efgmatcv)
    Call pfree(efgmatcc)
    Call pfree(p_n)
    Call pfree(p_z)
    Call pfree(p_e)
    !!
    !Stop
    !!
    Return
    !!
2   Format(a)
9   Format(f12.7,1x)
15  Format(a,' efg(sum) [10^21 V/m^2]')
415 Format(a,' 1st and 2nd axis degenerate and perp. to 3rd axis        ')
416 Format(a,' 1st main axis        ',3(f10.4,1x))
426 Format(a,' 2nd main axis        ',3(f10.4,1x))
436 Format(a,' 3rd main axis        ',3(f10.4,1x))
10  Format(a,' Site ',a2,'(',a,')')
446 Format(a,' Vzz [10^21 V/m^2] ,eta',2(f10.4,1x))
4461 Format(a,' Vzz [10^21 V/m^2] ,eta',1(f10.4,1x),' divergent? ')
447 Format(a,' Sum(Vzz_',a2,') [10^21 V/m^2]',1(f10.4,1x))
    !!
  End Subroutine efg_contributions
  !!***********************************************************************
  Subroutine print_contrib_efgmat(nRa,nRb,lRa,lRb,nnRa,nnRb,efgmatab&
       &,mesg,unit)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: nRa,nRb,lRa(nRa),lRb(nRb)
    Integer(t_int),Intent(in)    :: nnRa(nRa),nnRb(nRb)
    Real(t_real),Intent(in)      :: unit
    Real(t_real),Intent(in)      :: efgmatab(:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_contrib_efgmat"
    Integer(t_int) :: ia,la,na,ib,lb,nb,i,j
    !!
    Do ia=1,nRa
       la=lRa(ia)
       na=nnRa(ia)
       Do ib=1,nRb
          lb=lRb(ib)
          nb=nnRb(ib)
          
          If(Mod(la+lb,2_t_int)/=0.Or..Not.(Abs(la-2)<=lb .And. lb<=la+2))Then
             Cycle
          Endif
          
          
          Write(outfile,2) Trim(mesg)
          Write(outfile,14) Trim(mesg),na,lqnchar(la),nb,lqnchar(lb)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') efgmatab(i,j,ia,ib)*unit
             Enddo
             Write(outfile,*)
          Enddo
          
       End Do
    End Do
    !!
    !!
    Return
    !!
2   Format(a)
9   Format(f12.7,1x)
14  Format(a,' efg(',i1,a1,',',i1,a1,') [10^21 V/m^2]')
    !!
  End Subroutine print_contrib_efgmat
  !!***********************************************************************
  Subroutine snrel_orbital_efg(mspin,isa&
       &,bmesh,ncomp,nRa,nRb,lRa,lRb,fna,fnb,fgam&
       &,degree,oci_nr,wgaunt,nonab&
       &,efgmatab)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: mspin,isa
    Integer(t_int),Intent(in)    :: degree,oci_nr,ncomp
    Integer(t_int),Intent(in)    :: nRa,nRb,lRa(nRa),lRb(nRb)
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: fna(:,:,:)
    Real(t_real),Intent(in)      :: fnb(:,:,:)
    Real(t_real),Intent(in)      :: nonab(:,:,:,:)
    Real(t_real),Intent(in)      :: wgaunt(:,:,:)
    Real(t_real),Intent(out)     :: efgmatab(3,3,nRa,nRb)
    !!LOCAL
    Character(*),Parameter::nsubfunc="snrel_orbital_efg"
    Integer(t_int) :: ia,iva,la,dega,lqa,ib,ivb,lb,degb,lqb,ma,mb
    Integer(t_int) :: mL
    Real(t_real)   :: d,s,efgl(5)
    Real(t_real),Parameter :: sfac=Sqrt(3.0_t_real*fourpi/5.0_t_real)
    Real(t_real),Pointer :: p_oci(:,:)
    !!
    Nullify(p_oci)
    !!
    Call palloc('p_oci',p_oci,1_t_int,nRa,1_t_int,nRb,nmodule,nsubfunc)
    !!
    Call snrel_efgradialintegral(bmesh,ncomp,nRa,nRb,lRa,lRb,fna,fnb,fgam&
         &,degree,oci_nr,p_oci)
    !!
    efgmatab=0.0_t_real
    !!
    ia=0
    ivaloop: Do iva=1,nRa
       la=lRa(iva)
       dega=2*la+1
       lqa=la**2
       ib=0
       ivbloop: Do ivb=1,nRb 
          lb=lRb(ivb)
          degb=2*lb+1
          lqb=lb**2
          If(Mod(la+lb,2_t_int)/=0.Or..Not.(Abs(la-2)<=lb .And. lb<=la+2))Then
             ib=ib+degb
             Cycle
          Endif
          !!
          !! V_{2m}=sqrt(3.0_t_real*fourpi/5.0_t_real)
          !!        *int 1/x*R*R*n_{ab}*<Y_bY_{2m}Y_a>
          Do mL=1,5
             d=0.0_t_real
             Do ma=1,dega
                Do mb=1,degb
                   s=Sum(nonab(ia+ma,ib+mb,isa,1:mspin))
                   d=d+s*wgaunt(mL+4,lqb+mb,lqa+ma)
                Enddo
             Enddo
             efgl(mL)=-d*sfac*p_oci(iva,ivb)
          Enddo ! mL
          Call efgl_to_mat(efgmatab(:,:,iva,ivb),efgl)
          !!
          !!
          ib=ib+degb
       Enddo ivbloop
       ia=ia+dega
    Enddo ivaloop
    !!
    !!
    Call pfree(p_oci)
    !!
    Return
    !!
  End Subroutine snrel_orbital_efg
  !!***********************************************************************
  Subroutine snrel_efgradialintegral(bmesh,ncomp&
       &,nRa,nRb,lRa,lRb&
       &,fna,fnb,fgam,degree,oci_nr,res)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: nRa,nRb,lRa(:),lRb(:)
    Integer(t_int),Intent(in)    :: degree,oci_nr,ncomp
    Real(t_real),Intent(in)      :: fna(:,:,:)
    Real(t_real),Intent(in)      :: fnb(:,:,:)
    Real(t_real),Intent(out)     :: res(:,:)
    Real(t_real),Intent(in)      :: fgam
    !!LOCAL
    !Character(*),Parameter::nsubfunc="snrel_efgradialintegral"
    Integer(t_int) :: ir,index,ia,ib,la,lb,icomp
    Real(t_real)   :: xp,rp,wp,alfac,al1,al,dr,alm1,texp,d,alll
    Real(t_real)   :: ffa(ncomp,nRa)
    Real(t_real)   :: ffb(ncomp,nRb)
    Real(t_real)   :: coefa(0:degree)
    !! DONT FREE THESE AFTERWARDS !!
    Real(t_real),Pointer   :: p_ar(:),p_wtr(:)
    !!
    Call  get_gauss(oci_nr,p_ar,p_wtr)
    !!
    alfac=1.0_t_real/(2.0_t_real*fgam-1.0_t_real)
    al1=alfac*(1.0_t_real+2.0_t_real)
    al=Max(al1,3.0_t_real)
    !!
    !!
!!$    Write(outfile,12)fgam
!!$    Write(outfile,13)alfac
!!$    Write(outfile,14)al1
!!$    Write(outfile,14)al
!!$12  Format('minimum gamma value =',f10.5)
!!$13  Format('maximum alpha factor=',f10.5)
!!$14  Format('maximum alpha       =',f10.5)
    !!
    alm1=al-1.0_t_real
    dr=bmesh% rmax
    !!
    res=0.0_t_real
    irloop: Do ir=1,oci_nr
       xp=(p_ar(ir)+1.0_t_real)*0.5_t_real
       rp=xp**al*dr
       wp=p_wtr(ir)*0.5_t_real*al*xp**alm1*dr
       !!
       index=mintpl_coefs(degree,bmesh,rp,coefa)
       !!
       texp=rp**fgam
       !!
       Do ia=1,nRa
          la=lRa(ia)
          Do icomp=1,ncomp
             d=1.0_t_real
             If(icomp>=3 .And. la>0) d=1.0_t_real/Sqrt(la*(la+1.0_t_real))
             !!
             ffa(icomp,ia)=Dot_product(fna(index:index+degree,ia,icomp)&
                  &,coefa(0:degree))*texp/rp*d
          Enddo
       Enddo
       !!
       Do ib=1,nRb
          lb=lRb(ib)
          Do icomp=1,ncomp
             d=1.0_t_real
             If(icomp>=3 .And. lb>0) d=1.0_t_real/Sqrt(lb*(lb+1.0_t_real))
             !!
             ffb(icomp,ib)=Dot_product(fnb(index:index+degree,ib,icomp)&
                  &,coefa(0:degree))*texp/rp*d
          Enddo
       Enddo
       !!
       Do ia=1,nRa
          la=lRa(ia)
          Do ib=1,nRb
             lb=lRb(ib)
             !!
             d=0.0_t_real
             Do icomp=1,ncomp
                alll=1.0_t_real
                If(icomp>=3) alll=(la*(la+1)+lb*(lb+1))*0.5_t_real-3.0_t_real
                d=d+ffa(icomp,ia)*ffb(icomp,ib)/rp*alll
             Enddo
             res(ia,ib)=res(ia,ib)+d*wp
          Enddo
       Enddo
    End Do irloop
    !!
    !!
    Return
    !!
  End Subroutine snrel_efgradialintegral
  !!***********************************************************************
  Subroutine calc_efg(mspin,structure,lattice,potential,fgam,degree&
       &,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: degree
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: relativistic
    Real(t_real),Intent(in)      :: fgam(structure% nsort)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_efg"
    Logical(t_log) :: dolew,prot
    Integer(t_int) :: isa,issa,i,j
    Real(t_real)   :: unit,d
    Real(t_real)   :: efgmat_site(3,3),efgmat_on(3,3),efgmat_total(3,3)
    Real(t_real)   :: efgmat_lattice(3,3)
    Real(t_real)   :: Z(3,3),E(3),v(3),Vzz,eta
    Character(*),Parameter   :: mesg='EFG: '
    Real(t_real),Pointer :: p_efg_offsite(:,:,:)
    Real(t_real),Pointer :: p_efg_ew(:,:,:)
    Real(t_real),Pointer :: p_Qlr(:,:,:)
    Real(t_real),Pointer :: p_efgl(:,:) !! 5,site
    Real(t_real),Pointer :: p_efgl_site(:,:) !! 5,site
    !!
    Return
    !!
    prot=(outlevel>=L_many)
    !!
    !!
    Nullify(p_efg_ew,p_efg_offsite,p_Qlr,p_efgl,p_efgl_site)
    !!
    dolew=.False.
    If(structure% str_type == STR_TYPE_MOLECULE)Then
       dolew=.True.
    Endif
    !!
    If(.Not.Associated(potential% p_vft))Then
31     Format("'Subroutine ",a," called with dead p_vft!?'")
       Write(errbuf,31) Trim(nsubfunc)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Call palloc('p_efg_offsite',p_efg_offsite,1_t_int,3_t_int&
         &,1_t_int,3_t_int,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!
    Call palloc('p_efg_ew',p_efg_ew,1_t_int,3_t_int&
         &,1_t_int,3_t_int,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!
    Call palloc('p_Qlr',p_Qlr,1_t_int,potential% nrp,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!
    Call palloc('p_efgl',p_efgl,1_t_int,5_t_int,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    Call palloc('p_efgl_site',p_efgl_site,1_t_int,5_t_int&
         &,1_t_int,structure% nsite,nmodule,nsubfunc)

    Call vlocefg(structure% nsite,structure% p_isrt,mspin,degree&
         &,fgam,potential% max_L,potential% mlmp&
         &,potential% p_nlmp,potential% p_lmp&
         &,potential% p_pmeshs,potential% nrp,potential% p_natom&
         &,potential% p_lcd,potential% ew_p_r,potential% p_ac&
         &,potential% p_Nl,p_Qlr,relativistic&
         &,p_efgl,p_efgl_site,potential% nuc_type,dolew)
    !!
    Call calcOffsiteEfg(mspin,structure,lattice,potential,degree&
         &,p_Qlr,p_efg_offsite)
    

    Call ew_efg(structure,lattice,potential,p_efg_ew)
    !!
    !!
    !! prepare onsite part and output
    !!
    unit=hatoev/brtoan**2/10.0_t_real !! in 10^21 V/m^2
    !!
    Write(outfile,1)Trim(mesg)
    If(prot) Write(outfile,2)Trim(mesg)
    Do isa=1,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !!
       !!
       !!
       If(prot) Then
          Write(outfile,10) Trim(mesg),structure% p_elements(issa)&
               &,Trim(intToChar10(isa))
          Write(outfile,2) Trim(mesg)
          Write(outfile,11) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -p_efg_ew(i,j,isa)*unit
             Enddo
             Write(outfile,*)
          Enddo
       End If
       !!
       Call efgl_to_mat(efgmat_site,p_efgl_site(:,isa))
       !!
       If(prot) Then
          Write(outfile,2) Trim(mesg)
          Write(outfile,12) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -efgmat_site(i,j)*unit
             Enddo
             Write(outfile,*)
          Enddo
       End If
       !!
       Call efgl_to_mat(efgmat_on,p_efgl(:,isa))
       !!
       If(prot) Then
          Write(outfile,2) Trim(mesg)
          Write(outfile,13) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -efgmat_on(i,j)*unit
             Enddo
             Write(outfile,*)
          Enddo
       End If
       !!
       !!
       If(prot) Then
          
          Write(outfile,2) Trim(mesg)
          Write(outfile,16) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -p_efg_offsite(i,j,isa)*unit
             Enddo
             Write(outfile,*)
          Enddo
       End If
       
       !!
       !! total
       efgmat_total=efgmat_on+p_efg_offsite(:,:,isa)+p_efg_ew(:,:,isa)
       !!
       If(prot) Then
          
          Write(outfile,2) Trim(mesg)
          Write(outfile,14) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -efgmat_total(i,j)*unit
             Enddo
             Write(outfile,*)
          Enddo
       Endif
       !!
       !! lattice
       efgmat_lattice=efgmat_total-efgmat_site
       !!
       If(prot) Then
          
          Write(outfile,2) Trim(mesg)
          Write(outfile,15) Trim(mesg)
          Do i=1,3
             Write(outfile,2,advance='no')Trim(mesg)
             Do j=1,3
                Write(outfile,9,advance='no') -efgmat_lattice(i,j)*unit
             Enddo
             Write(outfile,*)
          Enddo
       Endif
       !!
    Enddo
    If(prot) Write(outfile,2)Trim(mesg)
    !!
    !!
    Write(outfile,2)Trim(mesg)
    Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       !!
       !!
       !!
       Write(outfile,10) Trim(mesg),structure% p_elements(issa)&
            &,Trim(intToChar10(isa))
       Write(outfile,2) Trim(mesg)
       !! decompose total
       Call efgl_to_mat(efgmat_on,p_efgl(:,isa))
       efgmat_total=efgmat_on+p_efg_offsite(:,:,isa)+p_efg_ew(:,:,isa)
       !!
       efgmat_total=-efgmat_total
       Call mainaxis_decopose(efgmat_total,z,e)
       !!
       If(Abs(e(1))>Abs(e(2)) )Then
          d=e(1)
          e(1)=e(2)
          e(2)=d
          v=Z(:,1)
          Z(:,1)=Z(:,2)
          Z(:,2)=v
       Endif
       If(Abs(e(2))>Abs(e(3)) )Then
          d=e(2)
          e(2)=e(3)
          e(3)=d
          v=Z(:,2)
          Z(:,2)=Z(:,3)
          Z(:,3)=v
       Endif
       If(Abs(e(1))>Abs(e(3)) )Then
          d=e(1)
          e(1)=e(3)
          e(3)=d
          v=Z(:,1)
          Z(:,1)=Z(:,3)
          Z(:,3)=v
       Endif
       !!
       Write(outfile,2)Trim(mesg)
       d=Max(Abs(e(1)),Abs(e(2)))
       If(d<1.0e-6_t_real)d=1.0_t_real
       If(Abs(e(1)-e(2))/d<1.0e-6_t_real)Then
          Write(outfile,415)Trim(mesg)
       Else
          Write(outfile,416)Trim(mesg),z(:,1)
          Write(outfile,426)Trim(mesg),z(:,2)
       Endif
       Write(outfile,436)Trim(mesg),z(:,3)
       Write(outfile,2) Trim(mesg)
       If(All(Abs(e(1:3))<1.0e-8_t_real))Then
          Vzz=0.0_t_real
          eta=0.0_t_real
          Write(outfile,446)Trim(mesg),Vzz*unit,eta
       Else If(Abs(e(3))<1.0e-8_t_real)Then
          Vzz=e(3)
          If(Abs(e(1)-e(2))<1.0e-8_t_real)Then
             eta=0.0_t_real
             Write(outfile,446)Trim(mesg),Vzz*unit,eta
          Else
             Write(outfile,4461)Trim(mesg),Vzz*unit
          Endif
       Else
          Vzz=e(3)
          eta=(e(1)-e(2))/Vzz
          Write(outfile,446)Trim(mesg),Vzz*unit,eta
       Endif
       !!
       Write(outfile,2) Trim(mesg)
       !!
    Enddo
    !!
    Write(outfile,21) Trim(mesg)
    !!
    Call pfree(p_efg_offsite)
    Call pfree(p_efg_ew)
    Call pfree(p_Qlr)
    Call pfree(p_efgl)
    Call pfree(p_efgl_site)
    !!
    !!
    Return
    !!
1   Format(/,a,72('='))
21  Format(a,72('='),/)
2   Format(a)
9   Format(f12.7,1x)
10  Format(a,' Site ',a2,'(',a,')')
11  Format(a,' Ewald   [10^21 V/m^2]')
12  Format(a,' Site    [10^21 V/m^2]')
13  Format(a,' Onsite  [10^21 V/m^2]')
16  Format(a,' Offsite [10^21 V/m^2]')
14  Format(a,' Total   [10^21 V/m^2]')
15  Format(a,' Lattice [10^21 V/m^2]')
415 Format(a,' 1st and 2nd axis degenerate and perp. to 3rd axis        ')
416 Format(a,' 1st main axis        ',3(f15.9,1x))
426 Format(a,' 2nd main axis        ',3(f15.9,1x))
436 Format(a,' 3rd main axis        ',3(f15.9,1x))
446 Format(a,' Vzz [10^21 V/m^2] ,eta',3(f15.6,1x))
4461 Format(a,' Vzz [10^21 V/m^2] ,eta',1(f10.4,1x),' divergent? ')
    !!
  End Subroutine calc_efg
  !!***********************************************************************
  Subroutine efgl_to_mat(efgmat,efgl)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)      :: efgl(5)
    Real(t_real),Intent(out)     :: efgmat(3,3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="efgl_to_mat"
    Real(t_real)   :: zz
    !!
    !!
    zz=efgl(3)*sq3o3 !! V00/sqrt(3)
    !!
    efgmat(1,1)= efgl(5)-zz !!  V22-V20/sqrt(3)
    efgmat(2,2)=-efgl(5)-zz !! -V22-V20/sqrt(3)
    efgmat(3,3)=  2.0_t_real*zz !!    2*V20/sqrt(3)

    efgmat(1,2)=efgl(1) !! V2-2
    efgmat(1,3)=efgl(4) !! V21
    efgmat(2,3)=efgl(2) !! V2-1
    !!
    efgmat(2,1)=efgmat(1,2)
    efgmat(3,2)=efgmat(2,3)
    efgmat(3,1)=efgmat(1,3)
    !!
    Return
    !!
  End Subroutine efgl_to_mat
  !!***********************************************************************
  Subroutine calcOffsiteEfg(mspin,structure,lattice,potential,degree&
       &,Qlr,efgmat_off)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin,degree
    Real(t_real),Intent(in)      :: Qlr(:,:,:)
    Real(t_real),Intent(out)     :: efgmat_off(3,3,structure% nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calcOffsiteEfg"
    Logical(t_log) :: dolew
    Integer(t_int) :: isa,isb,issb,irs,diml,indexb,j,ilm,ms,l
    Integer(t_int) :: ii,jj,nn
    Real(t_real)   :: pvec(3),p,p2,dlm,vlm,efgmat(3,3),afdp,afpmpm,nvec(3)
    Real(t_real)   :: gfdp(3),gqv(5),a,fpmpm,oop,oop2,Qlm
    Real(t_real)   :: coefb(0:degree)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    Type(T_Mesh),Pointer :: pmeshb
    !!
    dolew=(structure% str_type == STR_TYPE_MOLECULE) 
    !!
    If(.Not.Associated(potential% p_vc))Then
31     Format("'Subroutine ",a," called with dead p_vc!?'")
       Write(errbuf,31) Trim(nsubfunc)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Nullify(p_yb,p_rvylm,p_qvylm)
    !!
    diml=(potential% max_L+1)**2
    Call palloc('p_yb',p_yb,1_t_int,diml,nmodule,nsubfunc)
    Call palloc('p_rvylm',p_rvylm,1_t_int,3_t_int,1_t_int,diml&
         &,nmodule,nsubfunc)
    Call palloc('p_qvylm',p_qvylm,1_t_int,5_t_int,1_t_int,diml&
         &,nmodule,nsubfunc)
    !!
    efgmat_off(:,:,:)=0.0_t_real
    !!
    !! isa is site of EFG
    isaloop: Do isa=1,structure% nsite
       !!
       !!
       !!
       efgmat(1:3,1:3)=0.0_t_real
       !! neighbor loop
       isbloop: Do isb=1_t_int,structure% nsite
          issb=structure% p_isrt(isb)
          !!
          pmeshb    => potential% p_pmeshs(issb)
          !!
          irloop: Do irs=1,lattice% p_npairs(isb,isa)
             pvec(:)=-lattice% p_pairs(:,irs,isb,isa)
             !! pvec=-(R+sb-sa)
             p2=Dot_product(pvec,pvec)
             p=Sqrt(p2)
             !! p=|R+sb-sa|
             If(p > pmeshb% rmax) Cycle irloop
             !!
             If(p==0.0_t_real)Cycle irloop !! should never happen
             !!
             oop=1.0_t_real/p
             oop2=oop*oop
             !!
             nvec=pvec*oop
             !!
             !!
             Call ylms12(p_yb,pvec(1),pvec(2),pvec(3),potential% max_L)
             !!
             Call calc_real_quad_harmonics_opt(potential% max_L&
                  &,p_yb,p_rvylm,p_qvylm)
             !!
             indexb=mintpl_coefs(degree,pmeshb,p,coefb)
             !!
             afdp=0.0_t_real
             afpmpm=0.0_t_real
             gfdp(:)=0.0_t_real
             gqv(:)=0.0_t_real
             ilmloop: Do j=1,potential% p_nlmp(isb)
                ilm=potential% p_lmp(j,isb)
                l=llilm(ilm)
                !!
                dlm=0.0_t_real
                Do ms=1,mspin
                   dlm=dlm+Dot_product(&
                        &potential% p_lcd(indexb:indexb+degree,j,isb,ms)&
                        &,coefb(0:degree))
                Enddo
                If(.Not.dolew) Then
                   dlm=dlm-Exp(-(potential% ew_p_r*p)**2)&
                        &*potential% p_ac(j,isb)/potential% p_Nl(l)&
                        &*spowi(p,l+2)
                Endif
                !!
                !! V
                vlm=Dot_product(&
                     &potential% p_vc(indexb:indexb+degree,j,isb)&
                     &,coefb(0:degree))
                If(dolew)Then
                   vlm=vlm+Dot_product(&
                        &potential% p_vew(indexb:indexb+degree,j,isb)&
                        &,coefb(0:degree))*p
                Endif
                !!
                !! 1/p D_{-l} V 
                Qlm=Dot_product(Qlr(indexb:indexb+degree,j,isb)&
                     &,coefb(0:degree))
                !!
                !! D_l^{++} V
                fpmpm=(2.0_t_real*l+3.0_t_real)*Qlm-fourpi*dlm
                !!
                !! 1/p D_{-l} V Y
                afdp   =afdp   -Qlm  *p_yb(ilm)
                !! D_l^{++}V Y
                afpmpm =afpmpm +fpmpm*p_yb(ilm)
                !!  1/p D_{-l} V (deg*P_l)
                gfdp(:)=gfdp(:)-Qlm  *p_rvylm(:,ilm)
                !! V (deg_l*deg_{l-1}*Q^1_{lm})
                gqv(:) =gqv(:) +vlm  *p_qvylm(:,ilm)
                !!
             End Do ilmloop
             !!
             !!
             Do jj=1,3
                efgmat(jj,jj)=efgmat(jj,jj)+afdp*oop2 !! 1/p D_{-l}V Y \bf{1}
                Do ii=1,3
                   efgmat(ii,jj)=efgmat(ii,jj)&
                        !! D_l^{++}V Y \vec{e}_p \circ \vec{e}_p
                        &+afpmpm*nvec(ii)*nvec(jj)*oop2& 
                        !! 1/p D_{-l}V (deg*P_l \vec{e}_p +h.c.)
                        &+(gfdp(ii)*nvec(jj) +gfdp(jj)*nvec(ii))*oop2
                Enddo
             Enddo
             !!
             !! V /p^2 (deg_l *deg_{l-1}*Q^1_{lm})
             nn=0
             Do jj=1,3
                Do ii=jj,3
                   nn=nn+1
                   If(nn==6) Then
                      a=(-gqv(1)-gqv(4))*oop2*oop
                   Else
                      a=gqv(nn)*oop2*oop
                   Endif
                   efgmat(ii,jj)=efgmat(ii,jj)+a
                   If(ii /= jj) efgmat(jj,ii)=efgmat(jj,ii)+a
                Enddo
             Enddo
             
             !!
             !!
          End Do irloop
          !!
       End Do isbloop
       !!
       efgmat_off(:,:,isa)=efgmat(:,:)
       a=0.0_t_real
       Do ii=1,3
          a=a+efgmat(ii,ii)
       Enddo
       a=a*c1o3
       Do ii=1,3
          efgmat_off(ii,ii,isa)=efgmat_off(ii,ii,isa)-a
       Enddo
       !!
    End Do isaloop
    !!
    Call pfree(p_qvylm)
    Call pfree(p_rvylm)
    Call pfree(p_yb)
    !!
    Return
    !!
  End Subroutine calcOffsiteEfg
  !!***********************************************************************
  !! calculate the lm-dependend Q(r) functions
  !! 
  !!  Q(r)=fourpi * \int_{0}^{r} r^(l+2)*[n(r)] dr
  !!  from the ewald corrected density plus nuclear charges
  !! 
  !!***********************************************************************
  Subroutine vlocefg(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
       &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl,Qlr,relativistic,efgl,efgl_site&
       &,nuc_type,dolew)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Logical(t_log),Intent(in)    :: dolew
    Integer(t_int),Intent(in)    :: relativistic,nuc_type
    Integer(t_int),Intent(in)    :: nsite,mspin,degree,isrt(nsite)
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite),nrp
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: ew_p_r
    Real(t_real),Intent(in)      :: natom(*)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(in)      :: ac(mlmp,nsite),Nl(0:max_L)
    Real(t_real),Intent(out)     :: Qlr(nrp,mlmp,nsite)
    Real(t_real),Intent(out)     :: efgl(5,nsite),efgl_site(5,nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="vloc"
    Logical(t_log) :: prot
    Integer(t_int) :: ir,ig,ilm,l,isa,issa,ng,index,degr,m
    Real(t_real)   :: summ,sumew,ri,rl,r0,d,dew,xg,rg,jg,rgl,sd,ex,exf
    Real(t_real)   :: deg,ori
    Real(t_real)   :: sdew,sdew1,d1,d1ew,p,pp,Z,norm,texpout,texpin
    Real(t_real)   :: y(nrp),y1(nrp)
    Real(t_real)   :: y1ew(nrp)
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrp)
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    prot=debugfun(nmodule,nsubfunc,'')
    !!
    ng=10
    p=ew_p_r
    pp=p*p
    !!
    Call get_gauss(ng,p_xx,p_ww)
    !!
    Qlr(:,:,:)=0.0_t_real
    !ehz0=0.0_t_real
    Do isa=1_t_int, nsite
       !!
       issa=isrt(isa)
       !!
       pmesh => pmeshs(issa)
       !!
       Z=natom(issa)
       !!
       efgl(1:5,isa)=0.0_t_real
       efgl_site(1:5,isa)=0.0_t_real
       !!
       Do ilm=1,nlmp(isa)
          l=llilm(lmp(ilm,isa))
          m=mmilm(lmp(ilm,isa))
      
          deg=Real(2*l+1,t_real)
          
          
          exf=-1.0_t_real
          If(l==0) exf=2.0_t_real*fgam(issa)-2.0_t_real
          If(l==1) exf=fgam(issa)-1.0_t_real
          If(l==2) Then

             If(relativistic==FULL_RELATIVISTIC .Or.&
                  &relativistic==SCALAR_RELATIVISTIC)Then
                If(nuc_type /= FINUC_NONE)Then
                   exf=4.0_t_real-2.0_t_real
                Else
                   exf=fgam(issa)+Sqrt(fgam(issa)**2-1+4)-2.0_t_real
                Endif
             Else If(relativistic==NON_RELATIVISTIC)Then
                exf=4.0_t_real-2.0_t_real
             Else If(relativistic==SCALAR_RELATIVISTIC_KH)Then
                !If(nuc_type /= FINUC_NONE)Then
                !   exf=2.0_t_real-2.0_t_real 
                !Else
                   exf=2.0_t_real*Sqrt(fgam(issa)**2+2)-2.0_t_real 
                !Endif
             Else
                Call terminate(TERM_CRASH,Trim(nsubfunc)&
                     &//': unknown relativistic option!')
             Endif
             
             
             
             If(prot) Then
                Open(33,file='+leading'//intToChar10(ilm))
                Do ir=3,20
                   d=Log(Abs(Sum(lcd(ir-1,ilm,isa,1:mspin))))
                   d1=Log(Abs(Sum(lcd(ir,ilm,isa,1:mspin))))
                   Write(33,"(2(e30.15e3,2x))")&
                        &Log(pmesh% p_rmesh(ir))&
                        &,(d1-d)&
                        &/(Log(pmesh% p_rmesh(ir))-Log(pmesh% p_rmesh(ir-1)))
                Enddo
                Close(33)
             Endif
              !Print *,'gam=',ilm,fgam(issa)
              !Print *,'fac=',ilm,exf-l          

          Endif
          ex=-exf-1.0_t_real
          Do ir=2,pmesh% nr
             dens(ir)=Sum(lcd(ir,ilm,isa,1:mspin))&
                  &*spow(pmesh% p_rmesh(ir),ex)
          Enddo
          dens(1)=0.0_t_real

          !! dens=lcd*r**(-exf-1)=n*r**(1-exf)
          !! l==0: dens=n*r**(1-(2*gam-2))
          !!       n=r**(2*gam-2)*P
          !!       dens=P*r
          !!
          
          If(prot) Then
             Call writefunc(pmesh% nr,pmesh% p_rmesh,lcd(:,ilm,isa,1)&
                  & ,17_t_int,'+d_'//intToChar10(ilm))
             Call writefunc(pmesh% nr,pmesh% p_rmesh,dens,17_t_int&
                  &,'+md_'//intToChar10(ilm))
          Endif

          summ=0.0_t_real
          sumew=0.0_t_real
          y(1)=summ
          If(prot) Open(33,file='+int_'//intToChar10(ilm))
          Do ir=2,pmesh% nr
             ri=pmesh% p_rmesh(ir)
             r0=pmesh% p_rmesh(ir-1)
             rl=spowi(ri,l)
             
             norm=1.0_t_real/Nl(l)
             
             d=0.0_t_real
             dew=0.0_t_real
             d1=0.0_t_real
             d1ew=0.0_t_real
             Do ig=1,ng
                xg=(p_xx(ig)+1.0_t_real)*0.5_t_real !! xi \in [0,1]
                rg=(ri-r0)*xg+r0 !! r \in [r0,ri]
                jg=(ri-r0)*p_ww(ig)*0.5_t_real
                !!TODO: mesh transform
                
                rgl=spowi(rg,l)
                
                texpout=spow(rg,exf+l+1)
                texpin=spow(rg,exf-l)

                index=rmintpl_coefs1(degree,2_t_int,degr,pmesh,rg,coef)
                sd=Dot_product(dens(index:index+degr),coef(0:degr))
      
                
                ex=Exp(-(pp*rg*rg))
                
                sdew=rgl*ex*rg*rg*ac(ilm,isa)*norm
                sdew1=ex*ac(ilm,isa)*norm
                
                d=d+sd*texpout*jg
                dew=dew+sdew*rgl*jg

                !! y1 <- lcd*r**(-1-l) = n*r**(1-l)
                
                d1=d1+sd*texpin*jg
                d1ew=d1ew+sdew1*rg*jg
                
                If(prot) Write(33,"(2(e30.15e3,2x))")rg,sd*texpin!*(ri-r0)
             Enddo
             summ=summ+d
             sumew=sumew+dew

             If(dolew)sumew=0.0_t_real
             y(ir)=(summ-sumew)/rl*fourpi
             
             y1(ir-1)=d1
             y1ew(ir-1)=d1ew
             
          Enddo
          If(prot) Close(33)
          !!
          If(prot)Then
             Call writefunc(pmesh% nr,pmesh% p_rmesh,y1,17_t_int&
                  &,'+y1'//intToChar10(ilm))
          Endif
          !!
          summ=0.0_t_real
          sumew=0.0_t_real
          Do ir=pmesh% nr-1,1,-1
             ri=pmesh% p_rmesh(ir)
             If(ir==1)Then
                ori=1.0_t_real
             Else
                ori=1.0_t_real/ri
             Endif
             
             If(l==2)summ=summ+y1(ir)*fourpi/deg
             If(l==2)sumew=sumew+y1ew(ir)*fourpi/deg
             
             
             If(l==0) Then
                Qlr(ir,ilm,isa)=(y(ir)-Z*sq4pi)*ori
             Else
                Qlr(ir,ilm,isa)=(y(ir))*ori
             Endif
             
          Enddo
          !!
          !!
          If(prot)Then
             Call writefunc(pmesh% nr,pmesh% p_rmesh,qlr(:,ilm,isa)&
                  &,17_t_int,'+qlr_'//intToChar10(ilm))
          Endif
          !!
          If(l==2)Then
             If(dolew)sumew=0.0_t_real
             efgl(m,isa)=(summ-sumew)*Sqrt(15.0_t_real/fourpi)
             efgl_site(m,isa)=summ*Sqrt(15.0_t_real/fourpi)
          Endif
          
          !!
          If(ilm== 1 )Then
             If(.Not. pmesh% with_zero) Stop 'mesh must be a with_zero-mesh'
          Endif
          !!
       Enddo !! ilm
       !!
    Enddo !! isa
    !!
    Return
    !!
  End Subroutine vlocefg
  !!***********************************************************************
  Subroutine ew_efg(structure,lattice,potential,p_efg_ew)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Real(t_real),Intent(out)     :: p_efg_ew(:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="ew_efg"
    Integer(t_int) :: isa,ig,i,j
    Real(t_real)   :: tau(3),G(3),g2,gs,ex,d
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    !!
    !! calculate ewald part
    p_efg_ew(:,:,:)=0.0_t_real
    !!
    If(structure% str_type == STR_TYPE_MOLECULE) Then
       Call calcOffsiteMolEw(structure,potential,p_efg_ew)
       Return
    Endif
    !!
    Do isa=1,structure% nsite
       tau=structure% p_tau(:,isa)
       
       Do ig=2_t_int,lattice% nlattg
          G(:)=lattice% p_glat(:,ig)
          g2=Dot_product(G,G)*c1o3
          gs=Dot_product(G,tau)
          ex=Real(potential% p_vft(ig)*Exp(ci*gs),t_real)
          Do i=1,3
             Do j=1,3
                d=0.0_t_real
                If(i==j)d=g2
                p_efg_ew(i,j,isa)=p_efg_ew(i,j,isa)-ex*(G(i)*G(j)-d)
             End Do
          End Do
       Enddo ! ig
    Enddo ! isa
    !!
    Return
    !!
  End Subroutine ew_efg
  !!***********************************************************************
  Subroutine calcOffsiteMolEw(structure,potential,efgmat_ew)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Real(t_real),Intent(out)     :: efgmat_ew(3,3,structure% nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calcOffsiteMolEw"
    Integer(t_int) :: isa,isb,issb,diml,j,ilm,l
    Integer(t_int) :: ii,jj,nn
    Real(t_real)   :: pvec(3),p,p2,efgmat(3,3),afdp,afpmpm,nvec(3)
    Real(t_real)   :: gfdp(3),gqv(5),a,fpmpm,oop,oop2,Qlm,deg,plp1
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    Type(T_Mesh),Pointer :: pmeshb
    !!
    If(structure% str_type /= STR_TYPE_MOLECULE) Return
    !!
    !!
    If(.Not.Associated(potential% p_ac))Then
31     Format("'Subroutine ",a," called with dead p_ac!?'")
       Write(errbuf,31) Trim(nsubfunc)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Nullify(p_yb,p_rvylm,p_qvylm)
    !!
    diml=(potential% max_L+1)**2
    Call palloc('p_yb',p_yb,1_t_int,diml,nmodule,nsubfunc)
    Call palloc('p_rvylm',p_rvylm,1_t_int,3_t_int,1_t_int,diml&
         &,nmodule,nsubfunc)
    Call palloc('p_qvylm',p_qvylm,1_t_int,5_t_int,1_t_int,diml&
         &,nmodule,nsubfunc)
    !!
    efgmat_ew(:,:,:)=0.0_t_real
    !!
    !! isa is site of EFG
    isaloop: Do isa=1,structure% nsite
       !!
       !!
       !!
       efgmat(1:3,1:3)=0.0_t_real
       !! neighbor loop
       isbloop: Do isb=1_t_int,structure% nsite
          issb=structure% p_isrt(isb)
          !!
          pmeshb    => potential% p_pmeshs(issb)
          !!
          pvec(:)=structure % p_tau(:,isa)-structure % p_tau(:,isb)
          !! pvec=-(sb-sa)
          p2=Dot_product(pvec,pvec)
          p=Sqrt(p2)
          !! p=|sb-sa|
          If(p <= pmeshb% rmax) Cycle isbloop
          !!
          !!
          oop=1.0_t_real/p
          oop2=oop*oop
          !!
          nvec=pvec*oop
          !!
          !!
          Call ylms12(p_yb,pvec(1),pvec(2),pvec(3),potential% max_L)
          !!
          Call calc_real_quad_harmonics_opt(potential% max_L&
               &,p_yb,p_rvylm,p_qvylm)
          !!
          !!
          afdp=0.0_t_real
          afpmpm=0.0_t_real
          gfdp(:)=0.0_t_real
          gqv(:)=0.0_t_real
          ilmloop: Do j=1,potential% p_nlmp(isb)
             ilm=potential% p_lmp(j,isb)
             l=llilm(ilm)
             deg=2.0_t_real*l+1.0_t_real
             plp1=1.0_t_real/p**(l+1)
             
             !!
             !!
             !! V
             Qlm=fourpi*potential% p_ac(j,isb)*plp1
             !!
             !! 1/p D_{-l} V 
             !!
             !! D_l^{++} V
             fpmpm=(2.0_t_real*l+3.0_t_real)*Qlm
             !!
             !! 1/p D_{-l} V Y
             afdp   =afdp   -Qlm  *p_yb(ilm)
             !! D_l^{++}V Y
             afpmpm =afpmpm +fpmpm*p_yb(ilm)
             !!  1/p D_{-l} V (deg*P_l)
             gfdp(:)=gfdp(:)-Qlm  *p_rvylm(:,ilm)
             !! V (deg_l*deg_{l-1}*Q^1_{lm})
             gqv(:) =gqv(:) +Qlm/deg  *p_qvylm(:,ilm)
             !!
          End Do ilmloop
          !!
          !!
          Do jj=1,3
             efgmat(jj,jj)=efgmat(jj,jj)+afdp*oop2 !! 1/p D_{-l}V Y \bf{1}
             Do ii=1,3
                efgmat(ii,jj)=efgmat(ii,jj)&
                     !! D_l^{++}V Y \vec{e}_p \circ \vec{e}_p
                     &+afpmpm*nvec(ii)*nvec(jj)*oop2& 
                     !! 1/p D_{-l}V (deg*P_l \vec{e}_p +h.c.)
                     &+(gfdp(ii)*nvec(jj) +gfdp(jj)*nvec(ii))*oop2
             Enddo
          Enddo
          !!
          !! V /p^2 (deg_l *deg_{l-1}*Q^1_{lm})
          nn=0
          Do jj=1,3
             Do ii=jj,3
                nn=nn+1
                If(nn==6) Then
                   a=(-gqv(1)-gqv(4))*oop2
                Else
                   a=gqv(nn)*oop2
                Endif
                efgmat(ii,jj)=efgmat(ii,jj)+a
                If(ii /= jj) efgmat(jj,ii)=efgmat(jj,ii)+a
             Enddo
          Enddo
          
          !!
          !!
          !!
       End Do isbloop
       !!
       efgmat_ew(:,:,isa)=efgmat(:,:)
       a=0.0_t_real
       Do ii=1,3
          a=a+efgmat(ii,ii)
       Enddo
       a=a*c1o3
       Do ii=1,3
          efgmat_ew(ii,ii,isa)=efgmat_ew(ii,ii,isa)-a
       Enddo
       !!
    End Do isaloop
    !!
    Call pfree(p_qvylm)
    Call pfree(p_rvylm)
    Call pfree(p_yb)
    !!
    Return
    !!
  End Subroutine calcOffsiteMolEw
  !!***********************************************************************

End Module efg
