!!----DEBUG options---------------------|
!! MODULE:SUBFUNC:                      |
!!--------------------------------------|
!!--------------------------------------!
!! ERG:erg_forces:                      !
!!--------------------------------------!

Module erg
  !!
  Use precisiondefs
  Use errors
  Use memory
  Use file_iounits
  Use constants
  Use clock
  Use matserv
  Use tpotentialdensity
  Use integrate1d
  Use mesh
  Use tmol
  Use tmesh
  Use diagonalize
  Use tsymmetry
  Use terg
  Use tforces
  Use tlsdau
  Use topc
  Use opcorrection
  Use ioaux
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="ERG"  
  !!
  Public :: calc_nv_erg,erg_etot,erg_org,erg_hart_0,erg_forces
  !!***********************************************************************
  !!***********************************************************************
  Character(*),Parameter :: fmtstart=&
       &"(/,/,'============================== START ',a&
       &,'============================== ')"
  Character(*),Parameter :: fmtend=&
       &"('============================== END   ',a&
         &,'============================== ',/,/)"
  !!***********************************************************************
Contains 
  !!***********************************************************************
  !!
  Subroutine calc_nv_erg(mspin,nsite,isrt&
!!!       &,vuc&
       &,npshell,ninpshell,pairabs,pairs&
       &,pmeshs,nrpmax,fgam&
       &,max_L,mlmp,nlmp,lmp,lcd,vcoul,mol&
!!!       &,tot_nuc_charge,dens_charge&
       &,coulerg_nr,coulerg_nxi,coulerg_neta,degree,etot&
       &,chargemode,ionicity,vuc)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Type(T_Etot),Intent(inout)   :: etot
    Type(T_Mol),Intent(in)       :: mol
    Integer(t_int),Intent(in)    :: mspin,nsite,isrt(nsite),chargemode
    Integer(t_int),Intent(in)    :: max_L,nrpmax
    Integer(t_int),Intent(in)    :: npshell(nsite,nsite)
    Integer(t_int),Intent(in)    :: ninpshell(:,:,:)
    Integer(t_int),Intent(in)    :: mlmp,nlmp(nsite),lmp(mlmp,nsite)
    Integer(t_int),Intent(in)    :: coulerg_nr,coulerg_nxi,coulerg_neta
    Integer(t_int),Intent(in)    :: degree
    Real(t_real),Intent(in)      :: vuc,ionicity
    Real(t_real),Intent(in)      :: pairabs(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(in)      :: lcd(nrpmax,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: vcoul(nrpmax,mlmp,nsite)
!!!    Real(t_real),Intent(in)      :: vuc,tot_nuc_charge,dens_charge(2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_nv_erg"
    Integer(t_int) :: isa,issa,i,il,l,ir,isb,issb,ish,n,ne,nf,ninp,ms
    Integer(t_int) :: ixi,ieta,m,indexpaa,indexpab,indexpba,indexpbb
    Integer(t_int) :: indexaba,indexabb,index,deg
    Real(t_real)   :: r,d,v,rbig,ns0(2),nns(2)
!!!    Real(t_real)   :: dq
    Real(t_real)   :: eu(3),xi,wxi,eta,weta,ra,rb,zs,alx,texp
    Real(t_real)   :: za,zb,ee00,e0,envoff,ds(2),envxc0,envxc00,ez0
    Real(t_real)   :: jac,vv,nn,v0,n0,vat0,epsxc0,exc0,exc00,vxc0(2)
    Real(t_real)   :: r1,b1_eta,b1_xi
    Real(t_real)   :: x,al,ex,exf0,exf1
    !!automatic arrays
    Real(t_real)   :: hza(0:max_L),hzb(0:max_L)
    Real(t_real)   :: fac(0:2*max_L+1)
    Real(t_real)   :: wu((2*max_L+1)**2)
    Real(t_real)   :: nrl(nrpmax,0:max_L,2)
    Real(t_real)   :: vrl(nrpmax,0:max_L)
    Real(t_real)   :: denss(nrpmax,2)
    Real(t_real)   :: sdlm((max_L+1)**2)
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: coefpaa(0:degree),coefpab(0:degree)
    Real(t_real)   :: coefpba(0:degree),coefpbb(0:degree)
    Real(t_real)   :: coefaba(0:degree),coefabb(0:degree)
    Type(T_Mesh),Pointer :: pmesha,pmeshb
    Type(T_Mesh),Pointer :: atommesha,atommeshb
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_aet(:),p_wtet(:)
    Real(t_real),Pointer  :: p_axi(:),p_wtxi(:)
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    !!
    !!charge correction
!!!    dq=(tot_nuc_charge-dens_charge(1_t_int)-dens_charge(2_t_int))&
!!!         &/vuc/Real(mspin,t_real) !! TODO: for non crystals vuc needs
    !! to be replaced with molecule volume, which is hard to calculate
    !!
    !! onsite terms
    !!
    !!
    !!
    al=3.0_t_real
    !!
    !!
    Call get_gauss(coulerg_nr,p_xx,p_ww)
    !!
    !!
    !!
    ez0=0.0_t_real
    e0=0.0_t_real
    exc0=0.0_t_real
    envxc0=0.0_t_real
    etot% env=0.0_t_real
    etot% exc=0.0_t_real
    etot% envxc=0.0_t_real
    !edq=0.0_t_real

    Do isa=1_t_int,nsite
       issa=isrt(isa)
       
       pmesha    => pmeshs(issa)
       atommesha => mol% p_atommeshs(issa)

       exf0=2.0_t_real*fgam(issa)-2.0_t_real

       ex=-exf0-1.0_t_real
       Do ir=2,pmesha% nr
          Do ms=1,mspin
             denss(ir,ms)=lcd(ir,1,isa,ms)*spow(pmesha% p_rmesh(ir),ex)
          Enddo
       Enddo
       denss(1,1)=0.0_t_real
       denss(1,2)=0.0_t_real

       
       Do ir=1,coulerg_nr
          !!
          x=(p_xx(ir)+1.0_t_real)*0.5_t_real
          r=x**al*pmesha% rmax
          jac=al*r/x*p_ww(ir)*0.5_t_real
          !!
          texp=spow(r,exf0)
          !!
          !!
          index=rmintpl_coefs1(degree,3_t_int,deg,pmesha,r,coef)
          d=0.0_t_real
          Do ms=1,mspin
             ds(ms)=Dot_product(denss(index:index+deg,ms),coef(0:deg))*texp
             d=d+ds(ms)
          Enddo
          
          !!

          !!
          v=minterpol(pmesha,degree,vcoul(1,1,isa),r)
          !edq=edq+v*r*jac*dq*conao(issa)*sq4pi
          ez0=ez0+r*v*jac
          e0=e0+d*v*jac

          v=minterpol(atommesha,degree,mol% p_vatom(:,0,issa),r)
          etot% env=etot% env+d*v*jac

          v=minterpol(atommesha,degree,mol% p_excatom(:,issa),r)
          exc0=exc0+d*v*jac
          
          Do ms=1,mspin
             v=minterpol(atommesha,degree,mol% p_vxcatom(:,ms,issa),r)
             envxc0=envxc0+ds(ms)*v*jac
          Enddo
       Enddo               ! ir
    Enddo                   ! isa
    !Write(outfile,100) edq
    !e1=e1+edq

    e0=e0*0.5_t_real
    ez0=ez0*ionicity*sq4pi*0.5_t_real/vuc

    etot% exc=etot% exc+exc0
    etot% envxc=etot% envxc+envxc0

    Write(outfile,121) nsubfunc,etot% env
    Write(outfile,122) nsubfunc,exc0
    Write(outfile,123) nsubfunc,envxc0
    Write(outfile,101) nsubfunc,e0
    If(chargemode==CHARGEMODE_JELLIUM)Then
       Write(outfile,102) nsubfunc,ez0
    Else
       ez0=0.0_t_real
    Endif
    !!
    !!
    !!
    !!...................  two-center terms ..............................
    !!

    Call make_factorials(fac,2*max_L+1)

    !!      
    envoff=0.0_t_real
    ee00=0.0_t_real
    exc00=0.0_t_real
    envxc00=0.0_t_real
    !!
    Call get_gauss(coulerg_neta,p_aet,p_wtet)
    Call get_gauss(coulerg_nxi,p_axi,p_wtxi)
    !!
    Do isa=1_t_int,nsite
       issa=isrt(isa)
       !!
       pmesha => pmeshs(issa)
       !!
       exf1=fgam(issa)-1.0_t_real
       exf0=2.0_t_real*exf1
       Do isb=1_t_int,nsite
          issb=isrt(isb)
          !!
          pmeshb    => pmeshs(issb)
          atommeshb => mol% p_atommeshs(issb)
          !!
          !!For all shells
          n=0_t_int
          ishloop: Do ish=1_t_int,npshell(isa,isb)
             nf=n+1_t_int
             ninp=ninpshell(ish,isa,isb)
             ne=n+ninp
             n=ne
             rbig=pairabs(ish,isa,isb)*0.5_t_real


             r1=Max(pmesha% rmax,pmeshb% rmax)/rbig-1.0_t_real
             b1_eta= Min(r1,1.0_t_real)
             If(b1_eta <= 0.0_t_real) Cycle ishloop


             !!rotation matrice
             sdlm(:)=0.0_t_real
             Do ir=nf,ne
                Call reuler(pairs(1:3,ir,isa,isb),eu)
                Do l=0_t_int,max_L
                   Call ytr(eu,0_t_int,fac,l,wu(1_t_int),.True.)
                   Do m=1,2*l+1
                      sdlm(l*l+m)=sdlm(l*l+m)+wu(m+(2*l+1)*l)
                   Enddo
                Enddo
             Enddo
             !!
             vrl(:,:)=0.0_t_real
             !!
             Do ir=1,pmeshb% nr
                Do i=1,nlmp(isb)
                   il=lmp(i,isb)
                   l=llilm(il)
                   vrl(ir,l)=vrl(ir,l)+vcoul(ir,i,isb)*sdlm(il)
                Enddo
             Enddo
             !!
             !!
             !!
             nrl(:,:,:)=0.0_t_real
             Do ir=2,pmesha% nr
                r=pmesha% p_rmesh(ir)
                Do i=1,nlmp(isa)
                   il=lmp(i,isa)
                   l=llilm(il)

                   ex=-exf0-1.0_t_real
                   if(l==1) ex=-exf1-1.0_t_real
                   texp=spow(r,ex)
                   
                   Do ms=1,mspin
                      nrl(ir,l,ms)=nrl(ir,l,ms)&
                           &+lcd(ir,i,isa,ms)*sdlm(il)*texp
                   Enddo
                   
                Enddo
             Enddo
             !!
             !! some preporations for integration
             !! 
             etaloop: Do ieta=1_t_int, coulerg_neta
                al=2.0_t_real
                x=(p_aet(ieta)+1.0_t_real)*0.5_t_real
                eta=(x**al-1.0_t_real)*b1_eta
                weta=p_wtet(ieta)*0.5_t_real*b1_eta*al*x**(al-1.0_t_real)


                !b1_xi=Log(r1-Abs(eta)+1.0_t_real)
                b1_xi=(r1-Abs(eta))


                xiloop: Do ixi=1_t_int, coulerg_nxi
                   x=(p_axi(ixi)+1.0_t_real)*0.5_t_real
                   !xi=Exp(x*b1_xi)
                   alx=2.0_t_real
                   xi=x**alx*b1_xi+1.0_t_real
                   wxi=p_wtxi(ixi)*0.5_t_real*b1_xi&
                                !&*xi!&
                        &*alx*x**(alx-1.0_t_real)

                   ra=(xi-eta)
                   rb=(xi+eta)
                   zs=eta*xi
                   za=(zs-1.0_t_real)/ra
                   zb=(zs+1.0_t_real)/rb
                   ra=ra*rbig
                   rb=rb*rbig
                   jac=rbig*weta*wxi
                   !!
                   Call yhzy0(hza,za,max_L)
                   Call yhzy0(hzb,zb,max_L)
                   !!
                   indexpaa=mintpl_coefs(degree,pmesha,ra,coefpaa)
                   indexpab=mintpl_coefs(degree,pmesha,rb,coefpab)
                   indexpbb=mintpl_coefs(degree,pmeshb,rb,coefpbb)
                   indexpba=mintpl_coefs(degree,pmeshb,ra,coefpba)
                   !!
                   indexabb=mintpl_coefs(degree,atommeshb,rb,coefabb)
                   indexaba=mintpl_coefs(degree,atommeshb,ra,coefaba)
                   !!
                   !! interpolation of density and potential
                   nns(1:2)=0.0_t_real
                   !!
                   If(max_L>0) Then
                      texp=spow(ra,exf1)
                      Do ms=1,mspin
                         nns(ms)=nns(ms)&
                              &+Dot_product(nrl(indexpaa:indexpaa+degree,1,ms)&
                              &       ,coefpaa(0:degree))*hza(1)*jac*texp
                      Enddo
                   Endif
                   !!
                   texp=spow(ra,exf0)
                   n0=0.0_t_real
                   Do ms=1,mspin
                      ns0(ms)=Dot_product(nrl(indexpaa:indexpaa+degree,0,ms)&
                           &,coefpaa(0:degree))*sq2o2*jac/ninp*texp
                      n0=n0+ns0(ms)
                   Enddo
                   !!
                   Do l=2,max_L
                      Do ms=1,mspin
                         nns(ms)=nns(ms)&
                              &+Dot_product(nrl(indexpaa:indexpaa+degree,l,ms)&
                              &,coefpaa(0:degree))*hza(l)*jac*texp
                      Enddo
                   Enddo       ! l
                   !!               
                   nn=Sum(nns(1:mspin))
                   !!
                   !!
                   vat0=Dot_product(mol% p_vatom(indexabb:indexabb+degree,0,issb)&
                        &,coefabb(0:degree))*sq2o2
                   epsxc0=Dot_product(mol% p_excatom(indexabb:indexabb&
                        &+degree,issb),coefabb(0:degree))*sq2o2
                   Do ms=1,mspin
                      vxc0(ms)=Dot_product(mol% p_vxcatom(indexabb:indexabb&
                           &+degree,ms,issb),coefabb(0:degree))*sq2o2
                   Enddo
                   !!
                   v0=Dot_product(vrl(indexpbb:indexpbb+degree,0)&
                        &,coefpbb(0:degree))*sq2o2/ninp
                   !!
                   vv=0.0_t_real
                   Do l=1,max_L
                      vv=vv+Dot_product(vrl(indexpbb:indexpbb+degree,l)&
                           &,coefpbb(0:degree))*hzb(l)
                   End Do ! l
                   !!
                   exc00=exc00+n0*epsxc0*ninp+nn*epsxc0
                   Do ms=1,mspin
                      envxc00=envxc00+ns0(ms)*vxc0(ms)*ninp+nns(ms)*vxc0(ms)
                   Enddo
                   envoff=envoff+n0*vat0*ninp+nn*vat0
                   ee00=ee00+n0*v0*ninp+n0*vv+v0*nn
                   !!

                   !! eta->-eta
                   !! ra->rb, za->-zb
                   !! h(za,l)->h(-zb,l)=>h(zb,l)*(-1)**l
                   !!
                   nns(1:2)=0.0_t_real
                   !!
                   If(max_L>0) Then
                      texp=spow(rb,exf1)
                      Do ms=1,mspin
                         nns(ms)=nns(ms)&
                              &+Dot_product(nrl(indexpab:indexpab+degree,1,ms)&
                              &,coefpab(0:degree))&
                              &*hzb(1)*texp*jac*(-1.0_t_real)**1
                      Enddo
                   Endif
                   !!
                   texp=spow(rb,exf0)
                   
                   n0=0.0_t_real
                   Do ms=1,mspin
                      ns0(ms)=Dot_product(nrl(indexpab:indexpab+degree,0,ms)&
                           &,coefpab(0:degree))*sq2o2*jac/ninp*texp
                      n0=n0+ns0(ms)
                   Enddo
                   !!
                   Do l=2,max_L
                      Do ms=1,mspin
                         nns(ms)=nns(ms)&
                              &+Dot_product(nrl(indexpab:indexpab+degree,l,ms)&
                              &,coefpab(0:degree))*hzb(l)*texp&
                              &*jac*(-1.0_t_real)**l
                      Enddo
                   Enddo       ! l
                   !!               
                   nn=Sum(nns(1:mspin))
                   !!
                   vat0=Dot_product(mol% p_vatom(indexaba:indexaba+degree,0,issb)&
                        &,coefaba(0:degree))*sq2o2
                   epsxc0=Dot_product(mol% p_excatom(indexaba:indexaba&
                        &+degree,issb),coefaba(0:degree))*sq2o2

                   Do ms=1,mspin
                      vxc0(ms)=Dot_product(mol% p_vxcatom(indexaba:indexaba&
                           &+degree,ms,issb),coefaba(0:degree))*sq2o2
                   Enddo

                   !!
                   v0=Dot_product(vrl(indexpba:indexpba+degree,0)&
                        &,coefpba(0:degree))*sq2o2/ninp
                   !!
                   vv=0.0_t_real
                   Do l=1,max_L
                      vv=vv+Dot_product(vrl(indexpba:indexpba+degree,l)&
                           &,coefpba(0:degree))*hza(l)*(-1.0_t_real)**l
                   End Do ! l
                   !!
                   exc00=exc00+n0*epsxc0*ninp+nn*epsxc0
                   Do ms=1,mspin
                      envxc00=envxc00+ns0(ms)*vxc0(ms)*ninp+nns(ms)*vxc0(ms)
                   Enddo
                   envoff=envoff+n0*vat0*ninp+nn*vat0
                   ee00=ee00+n0*v0*ninp+n0*vv+v0*nn


                Enddo xiloop
             Enddo etaloop

          Enddo ishloop ! ish
       Enddo ! isb
    Enddo ! isa
    !!
    ee00=ee00*0.5_t_real
    etot% env=etot% env + envoff
    !!
    etot% envxc=etot% envxc + envxc00
    etot% exc=etot% exc + exc00

    Write(outfile,103) nsubfunc,ee00
    etot% envc=e0+ee00+ez0
    Write(outfile,105) nsubfunc,etot% envc

    Write(outfile,106) nsubfunc,envoff

    Write(outfile,108) nsubfunc,exc00
    Write(outfile,109) nsubfunc,envxc00
    Write(outfile,107) nsubfunc,etot% exc
    Write(outfile,1010) nsubfunc,etot% envxc


101 Format(a,': onsit e0     : ',f30.8)
102 Format(a,': jellium-n*Z0 : ',f30.8)
121 Format(a,': onsit env    : ',f30.8)
122 Format(a,': onsit exc0   : ',f30.8)
123 Format(a,': onsit envxc0 : ',f30.8)
103 Format(a,': offsite      : ',f30.8)
108 Format(a,': offsit exc0  : ',f30.8)
109 Format(a,': offsit envxc0: ',f30.8)
105 Format(a,': n*vc-11      : ',f30.8)
106 Format(a,': n*vat        : ',f30.8)
107 Format(a,': n*excat      : ',f30.8)
1010 Format(a,': n*vxcat      : ',f30.8)

    !!
  End Subroutine calc_nv_erg
  !!***********************************************************************
  !! calculate all spherical Pl0 Legendre function up to momentum l
  !! introduce a convinience factor sqrt(2 pi)
  !! and the normalization factor c_l^{|m|} for the spherical harmonics
#ifdef my_compile_unused
  Subroutine  yhz0(hz,z,maxl)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: maxl
    Real(t_real),Intent(in)      :: z
    Real(t_real),Intent(out)     :: hz(0:maxl)
    !!LOCAL
    Integer(t_int) :: l
    Real(t_real)   :: pl,pl1,p,s
    !!    
    !!                  convinience factor
    pl=1.0_t_real/sq4pi * (sq4pi/Sqrt(2.0_t_real))
    hz(0)=pl
    !!
    pl1=0.0_t_real
    If(maxl>=1)Then
       pl1=z*pl
       s=Sqrt(3.0_t_real)
       hz(1)=pl1*s
    Endif
    Do l=2,maxl
       p=(z*Real(2*l-1,t_real)*pl1-Real(l-1,t_real)*pl)/Real(l,t_real)
       s=Sqrt(Real(2*l+1,t_real))
       hz(l)=p*s
       pl=pl1
       pl1=p
    Enddo
    !!
  End Subroutine yhz0
#endif
  !!***********************************************************************
  !! Yl-m=SIN(|m| phi) /sqrt(pi)* H * SIN(theta)^|m|
  !! Yl+m=COS(|m| phi) /sqrt(pi)* H * SIN(theta)^|m|
  !! Yl0=1 /sqrt(2 pi)* H
  !! maximum L=12 implemented
  !! Output: H= H * SIN(Theta)^|m|  for |m|=0
  !!***********************************************************************
  !!
  Subroutine yhzy0(h,z,max_L)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: max_L
    Real(t_real),Intent(in)  :: z
    Real(t_real),Intent(out) :: h(0:max_L)
    !!LOCAL
    Character(*),Parameter::nsubfunc="yhzy"
    Real(t_real)   :: z2,z4,z6,z8,z10,z12
    !!
    If(max_L > 12_t_int) Then
       Call terminate(TERM_CRASH,Trim(nsubfunc)&
            &//': max_L .gt. 12 not implemented')
    Endif
    !!
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=0  xxxxxxxxxxxxxxx
    H(0)=0.70710678118654752440_t_real
    If(max_L < 1_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=1  xxxxxxxxxxxxxxx
    H(1)=z*1.2247448713915890490_t_real
    If(max_L < 2_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=2  xxxxxxxxxxxxxxx
    z2=z*z
    H(2)=(3.0_t_real*z2-1.0_t_real)*0.79056941504209483299_t_real
    If(max_L < 3_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=3  xxxxxxxxxxxxxxx
    H(3)=z*(5.0_t_real*z2-3.0_t_real)*0.93541434669348534639_t_real
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=4 xxxxxxxxxxxxxxx
    If(max_L < 4_t_int) Return
    z4=z2*z2
    H(4)=(35.0_t_real*z4-30.0_t_real*z2+3.0_t_real)&
         &*0.26516504294495532165_t_real
    If(max_L < 5_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=5  xxxxxxxxxxxxxxx
    H(5)=z*(63.0_t_real*z4-70.0_t_real*z2+15.0_t_real)&
         &*0.29315098498896434716_t_real
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=6  xxxxxxxxxxxxxxx
    If(max_L < 6_t_int) Return
    z6=z2*z4
    H(6)=(231.0_t_real*z6-315.0_t_real*z4+105.0_t_real*z2-5.0_t_real)*&
         & 0.15934435979977452593_t_real
    If(max_L < 7_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=7  xxxxxxxxxxxxxxx
    H(7)=z*(429.0_t_real*z6-693.0_t_real*z4+315.0_t_real*z2-35.0_t_real)* &
         & 0.17116329922036441045_t_real
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=8  xxxxxxxxxxxxxxx
    If(max_L < 8_t_int) Return
    z8=z4*z4
    H(8)=(6435.0_t_real*z8-12012.0_t_real*z6+6930.0_t_real*z4&
         &-1260.0_t_real*z2+35.0_t_real) &
         & *0.022777155839239454964_t_real
    If(max_L < 9_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=9  xxxxxxxxxxxxxxx
    H(9)=z*(12155.0_t_real*z8-25740.0_t_real*z6+18018.0_t_real*z4&
         &-4620.0_t_real*z2+315.0_t_real)&
         & *0.024079742199097564258_t_real
    If(max_L < 10_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=10 xxxxxxxxxxxxxxx
    z10=z6*z4
    H(10)=(46189.0_t_real*z10-109395.0_t_real*z8+90090.0_t_real*z6&
         &-30030.0_t_real*z4+3465.0_t_real &
         & *z2-63.0_t_real)*0.012657696676577852013_t_real
    If(max_L < 11_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=11 xxxxxxxxxxxxxxx
    H(11)=z*(88179.0_t_real*z10-230945.0_t_real*z8+218790.0_t_real*z6&
         &-90090.0_t_real*z4+&
         & 15015.0_t_real*z2-693.0_t_real)*0.013246738248291539334_t_real
    If(max_L < 12_t_int) Return
    !!     xxxxxxxxxxxxxxxxxxxxxxxxx L=12 xxxxxxxxxxxxxxx
    z12=z6*z6
    H(12)=(676039.0_t_real*z12-1939938.0_t_real*z10+2078505.0_t_real*z8&
         &-1021020.0_t_real*z6+&
         & 225225.0_t_real*z4-18018.0_t_real*z2+231.0_t_real)&
         &*0.0034526698300124390839_t_real
    !!
  End Subroutine yhzy0
  !!     
  !!***********************************************************************
  !! calculate the local part of the E^{C0} energy
  !!
  Subroutine erg_hart_0(mspin,nsite,isrt,conao&
       &,pmeshs,nrpmax,fgam,mlmp,lcd,charges&
       &,ew_p_r,natom&
       &,degree,coulerg_nr,etot)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: mspin,nsite,isrt(*)
    Integer(t_int),Intent(in)    :: degree,coulerg_nr,nrpmax
    Integer(t_int),Intent(in)    :: mlmp
    Real(t_real),Intent(in)      :: conao(*),charges(nsite,2)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(in)      :: ew_p_r,natom(*)
    Real(t_real),Intent(in)      :: lcd(nrpmax,mlmp,nsite,mspin)
    Type(T_Etot),Intent(inout)   :: etot
    !!LOCAL
    Character(*),Parameter::nsubfunc="erg_hart_0"
    Integer(t_int) :: isa,issa,ir,deg,index
    Real(t_real)   :: dnat,x,r,w,d,al,p,ehart0,texp,ex,exf
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrpmax)
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_ww(:),p_xx(:)
    !!
    !!
    al=3.0_t_real
    !!
    Call get_gauss(coulerg_nr,p_xx,p_ww)
    !!
    ehart0=0.0_t_real
    Do isa=1_t_int,nsite
       issa=isrt(isa)
       !!
       pmesh => pmeshs(issa)
       !!
       p=ew_p_r
       dnat=natom(issa)
       
       exf=2.0_t_real*fgam(issa)-2.0_t_real
       ex=-exf-1.0_t_real
       Do ir=2,pmesh% nr
          dens(ir)=Sum(lcd(ir,1,isa,1:mspin))*spow(pmesh% p_rmesh(ir),ex)
       Enddo
       dens(1)=0.0_t_real

       Do ir=1,coulerg_nr
          x=(p_xx(ir)+1.0_t_real)*0.5_t_real
          r=x**al*pmesh% rmax
          w=al*r/x*p_ww(ir)*0.5_t_real
          !!
          texp=spow(r,exf)
          !!
          !!
          index=rmintpl_coefs1(degree,2_t_int,deg,pmesh,r,coef)
          d=Dot_product(dens(index:index+deg),coef(0:deg))*texp
          !!
          ehart0=ehart0+dnat*w*d*conao(issa)*sq4pi
       Enddo                   ! ir
       d=charges(isa,1_t_int)+charges(isa,2_t_int)-dnat
       ehart0=ehart0-p*2.0_t_real/sqpi*dnat*d*conao(issa)
    Enddo                     ! is
    !!
    !!
    ehart0=-ehart0*0.5_t_real
    Write(outfile,10) nsubfunc,ehart0
    etot% eh0=etot% eh0+ehart0
    Write(outfile,103) nsubfunc,etot% eh0
    !!
    etot% envc=etot% envc+etot% eh0
    !!
    Return
    !!

10  Format(a,':secnd term  onsite  : ',f20.6)
103 Format(a,': Int(Z*VCoulomb) tot: ',f20.6)
    !!
  End Subroutine erg_hart_0
  !!***********************************************************************
  !! print out and convergence organization of total energy
  Subroutine erg_org(etot,estack)
    !!
    Implicit None  
    !!IO
    Type(T_Etotstack),Intent(inout) ::  estack
    Real(t_real),Intent(in)      :: etot
    !!LOCAL
    Integer(t_int) :: i
    Real(t_real)   :: eav,deav
    !!
    Write(outfile,100)
    !!convergency control
    If(estack% icur < estack% nmax) Then
       estack% icur=estack% icur+1_t_int
       estack% p_stack(estack% icur)=etot
    Else
       Do i=1_t_int,estack% nmax-1_t_int
          estack% p_stack(i)=estack% p_stack(i+1_t_int)
       Enddo
       estack% p_stack(estack% icur)=etot
    Endif
    eav=0.0_t_real
    Do i=1_t_int,estack% icur
       eav=eav+estack% p_stack(i)
    Enddo
    eav=eav/estack% icur
    deav=0.0_t_real
    Do i=1_t_int,estack% icur
       deav=deav+(estack% p_stack(i)-eav)**2_t_int
    Enddo
    deav=Sqrt(deav)/estack% icur
    Write(outfile,20)(estack% p_stack(i),i=1,estack% icur)
    Write(outfile,21)eav,deav

    estack% etotconv=.False.
    !    If (estack% icur == estack% nmax &
    If (estack% icur >= Min(3_t_int,estack% nmax) &
         &.And. Abs(deav) < estack% etot_tolerance) Then
333    Format(/,'Total energy converged!',/)
       Write(outfile,333)
       estack% etotconv=.True.
    Endif
    !!
    Write(outfile,100)
    !!
    Return
    !!
100 Format(/,72('-'),/)
20  Format('estack     : ',f20.8)
21  Format('eav  ,deav : ',f20.8,2x,f20.8)
    !!
  End Subroutine erg_org
  !!***********************************************************************
  !! construct total energy from contribs
  !!
  Subroutine erg_etot(structure,lsdau,etot,opc)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Etot),Intent(inout)   ::  etot
    Type(T_Lsdau),Intent(in)     :: lsdau 
    Type(T_OPC),Intent(in)       :: opc
    !!LOCAL
    !Character(*),Parameter::nsubfunc="erg_etot"
    Real(t_real)   :: ekin,eband,etot_opc,etot_opc_dc,Eent
    Real(t_real)   :: pkin,penvc,pxc,pressure
    Real(t_real),Parameter :: c2o3=0.666666666666666666666666666_t_real
    !!_______________________________________________________________________
    !!
    !!
    !!
    !!
!!!todo: pressure->Eent
    Eent=Sum(etot% Eent(1:2))
    eband=etot% ecore+Sum(etot% bandenergy(1:2))
    ekin=eband-etot% env
    !!
    !!
    pxc=-(etot% exc-etot% envxc)/structure% vuc
    penvc=c1o3*etot% envc/structure% vuc
    pkin=c2o3*ekin/structure% vuc
    pressure=pkin+penvc+pxc
    !!
    etot% etot=ekin+etot% exc+etot% envc+Eent
    !!
    Call calc_opc_etot(structure,opc,etot_opc,etot_opc_dc)
    !!
    If (opc%  make_opc ) Then
!!!todo: pressure
       ekin=ekin-etot_opc_dc
       etot% etot=etot% etot+etot_opc-etot_opc_dc
    Endif
    !!
    !!
    If (lsdau% lsdau ) Then
!!!todo: pressure
       etot% etot=etot% etot-lsdau% eeU+lsdau% eeD
       ekin=ekin-2.0_t_real*lsdau% eeU+lsdau% eeD
    End If
    !!
    Write(outfile,*)
    Write(outfile,100)

    Write(outfile,1)etot% ecore
    Write(outfile,654)Eband
    Write(outfile,754)Eent


    !!START LSDA+U
    If (lsdau% lsdau ) Then
       Write(outfile,111)
       Write(outfile,121)'EE',etot% etot,ekin,etot% envc,etot% exc,lsdau% eeU
!!!todo: pressure and lsdau
    Else
       
       Write(outfile,110)
       Write(outfile,120)'EE',etot% etot,ekin,etot% envc,etot% exc
       
!       Write(outfile,200)pressure,pkin,penvc,pxc
!!!todo: units
    End If
    !!
    !!
    !!
    !!
    Return
    !!
100 Format(72('-'),/,10('='),t30,'TOTAL ENERGY'&
         &,t63,10('=')/72('-'))
110 Format( t13,'total energy',t34,'kinetic energy',t52,'potential energy'&
         &,t72,'ex.-corr. energy')
111 Format( t13,'total energy',t34,'kinetic energy',t52,'potential energy'&
         &,t72,'ex.-corr. energy',t95,'LSAD+U energy')
!120 Format(a,': ',f20.8,t25,f20.8,t47,f20.8,t68,f20.8)
!121 Format(a,': ',f20.8,t25,f20.8,t47,f20.8,t68,f20.8,t90,f20.8)
120 Format(a,': ',f22.10,t28,f20.8,t48,f20.8,t68,f20.8)
121 Format(a,': ',f22.10,t28,f20.8,t48,f20.8,t68,f20.8,t88,f20.8)
1   Format('Ecore: ',f17.8)
654 Format('EBand: ',f17.8)
754 Format('T*S:   ',f17.8)
!200 Format(/,/,t20,'total',t43,'kinetic',t66,'Coulomb',t89,'ex.corr.',/&
!         &,'Pressure: ',4(f22.10,1x),/,/)
    !!
    !!
    !!
  End Subroutine erg_etot
  !!*********************************************************************** 
  Subroutine erg_forces(symmetry,structure,forces,force_loop)
    !!
    Implicit None  
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Forces),Intent(inout) :: forces
    Logical(t_log),Intent(in)    :: force_loop
    !!LOCAL
    Character(*),Parameter::nsubfunc="erg_forces"
    Logical(t_log) :: deb
    Integer(t_int) :: isa,jops,isga,k,i,j
    Real(t_real)   :: fm(3),t(3),torque(3),btau(3),tmat(3,3)
    Real(t_real)   :: aa(3,3),E(3)
    Real(t_real),Pointer :: for(:,:)
    !!
    Nullify(for)
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    !!
    If(.Not. force_loop) Return
    !!
    If(deb) Then
       Write(outfile,102)
       Write(outfile,1102)
1102   Format(/,/)
       Do isa=1,structure% nsite
          Write(outfile,10)isa,forces% p_hfforces(:,isa)
10        Format('HF-FORCES: isa=',i5,' : ',3(f15.8,2x))
       Enddo
       Write(outfile,1102)
       Do isa=1,structure% nsite
          Write(outfile,13)isa,forces% p_cplforces(:,isa)
13        Format('CORE-PL-FORCES: isa=',i5,' : ',3(f15.8,2x))
       Enddo
       Write(outfile,1102)
       Do isa=1,structure% nsite
          Write(outfile,11)isa,forces% p_plforces(:,isa)&
               &+forces% p_cplforces(:,isa)
11        Format('PL-FORCES: isa=',i5,' : ',3(f15.8,2x))
       Enddo
       Write(outfile,1102)
       Do isa=1,structure% nsite
          Write(outfile,15)isa,forces% p_dndvforces(:,isa)
15        Format('dndV-FORCES: isa=',i5,' : ',3(f15.8,2x))
       Enddo
    Endif
    !!
    Call palloc('for',for,1_t_int,3_t_int,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    !!collect total force
    Do isa=1,structure% nsite
       for(:,isa)=forces% p_hfforces(:,isa)&
            & +forces% p_cplforces(:,isa)&
            &+forces% p_plforces(:,isa)&
            &+forces% p_dndvForces(:,isa)
    Enddo
    !! and symmetrize again
    Do isa=1,structure% nsite
       forces% p_forces(:,isa)=0.0_t_real
       Do jops=1,symmetry% Rnumops
          isga=structure% p_equivsites(jops,isa)
          forces% p_forces(:,isa)=forces% p_forces(:,isa)&
               &+Matmul(symmetry% Ropmat(:,:,jops),for(:,isga))&
               &/symmetry% Rnumops
       Enddo
    Enddo
    !!
    If(deb)Then
       Write(outfile,1102)
       Do isa=1,structure% nsite
          Write(outfile,12)isa,forces% p_forces(:,isa)
12        Format('TOTAL FORCES: isa=',i5,' : ',3(f17.10,2x))
       Enddo
    Endif
    !!
    Do k=1,3
       fm(k)= Sum(forces% p_forces(k,1:structure% nsite))
    Enddo
    !!
    fm(1:3)=fm(1:3)/structure% nsite
    !!
    If(deb)Then
       Write(outfile,"('averaged force : ',3(f17.10,2x))") fm(1:3)
    Endif
    !!
    Do isa=1,structure% nsite
       for(:,isa)=forces% p_forces(:,isa)-fm(:)
    Enddo
    !!
    !! and symmetrize again
    Do isa=1,structure% nsite
       forces% p_forces(:,isa)=0.0_t_real
       Do jops=1,symmetry% Rnumops
          isga=structure% p_equivsites(jops,isa)
          forces% p_forces(:,isa)=forces% p_forces(:,isa)&
               &+Matmul(symmetry% Ropmat(:,:,jops),for(:,isga))&
               &/symmetry% Rnumops
       Enddo
    Enddo
    !!
    !!
    !!
    !!
    torque(:)=0.0_t_real
    Do isa=1,structure% nsite
       Call cross(structure% p_tau(:,isa),forces% p_forces(:,isa),t)
       torque(:)=torque(:)+t(:)
    Enddo
    !!
    If(deb)Then
       Write(outfile,17) torque
17     Format('TORQUE=',3(f17.10,2x))
    Endif
    !!
    !!
    !! START: torque correction
    If(structure% str_type == STR_TYPE_MOLECULE) Then
       !! free standing molecules can have a torque error
       btau(:)=0.0_t_real
       Do isa=1,structure% nsite
          btau(:)=btau(:)+structure% p_tau(:,isa)
       Enddo
       btau(:)=btau(:)/structure% nsite
       !!
       tmat(:,:)=0.0_t_real
       Do isa=1,structure% nsite
          fm(:)=structure% p_tau(:,isa)-btau(:)
          Do i=1,3
             Do j=1,3
                If(i==j)Then
                   tmat(i,j)=tmat(i,j)+Sum(fm**2)
                Endif
                tmat(i,j)=tmat(i,j)-fm(i)*fm(j)
             Enddo
          Enddo
       Enddo
       !!
       aa=Matmul(tmat,tmat)
       Call rdiagno_noO(3_t_int,aa,e)
       !!
       t=-Matmul(tmat,torque)
       !!
       fm=0.0_t_real
       Do i=1,3
          If(Abs(e(i))>1.0e-10_t_real)Then
             fm=fm+aa(:,i)/e(i)*Dot_product(aa(:,i),t)
          Endif
       Enddo
       !!
       !!
       !!
       Do isa=1,structure% nsite
          Call cross(fm,structure% p_tau(:,isa)-btau(:),t)
          for(:,isa)=forces% p_forces(:,isa)+t(:)
       Enddo
       !!
       !!
       !! and symmetrize again
       Do isa=1,structure% nsite
          forces% p_forces(:,isa)=0.0_t_real
          Do jops=1,symmetry% Rnumops
             isga=structure% p_equivsites(jops,isa)
             forces% p_forces(:,isa)=forces% p_forces(:,isa)&
                  &+Matmul(symmetry% Ropmat(:,:,jops),for(:,isga))&
                  &/symmetry% Rnumops
          Enddo
       Enddo
       !!
       !!
       torque(:)=0.0_t_real
       Do isa=1,structure% nsite
          Call cross(structure% p_tau(:,isa),forces% p_forces(:,isa),t)
          torque(:)=torque(:)+t(:)
       Enddo
       !!
       If(deb)Then
          Write(outfile,177)torque
177       Format('corrected TORQUE=',3(f17.10,2x))
       Endif
       !!
       !!
    Endif
    !! END: torque correction
    !!
    !!
    Call pfree(for)
    !!
    Write(outfile,1102)
    Do isa=1,structure% nsite
       Write(outfile,14)isa,forces% p_forces(:,isa)
14     Format('TOTAL NEUTRAL FORCES: isa=',i5,' : ',3(f17.10,2x))
    Enddo
    !!
    Write(outfile,102)
    !!
    !!
    !   Write(outfile,"('stop after force calculation')")
    !   Call terminate(TERM_FINISH,'SCF calculation')  
    !!
    Return
    !!
102 Format(72('-'))
    !!
  End Subroutine erg_forces
  !!***********************************************************************
  
End Module erg

