Module errors
  !!
  Use precisiondefs
  Use file_iounits
  Use tio
  Use tmpi
  !!
  implicit none 
  !!
  Private
  !!
  Public :: &
       !! subroutine
       & errmsg&
       &,cerrmsg&
       &,setoutlevel&
       !! constants
       &, E_information&
       &, E_warning&
       &, E_error&
       &, E_fatalerror&
       &, L_all&
       &, L_info&
       &, L_many&
       &, L_more&
       &, L_silent&
       &,errbuf&
       &,outlevel&
       &,ContinueOnError &
       &,VERBOSITY_LEVELS,MAP_VERBOSITY_LEVELS&
       &,TERMINATION_STATUS,TERM_NORMAL,TERM_ERROR,TERM_CRASH,TERM_FINISH&
       &,terminate,debugfun,g_debug

  Character(*),Parameter::nmodule="ERRORS"         

  
  !!***********************************************************************
  !!  constants
  !! These constants are also defined in merror.h, dont change them
  Integer(t_int),Parameter :: E_information=1_t_int
  Integer(t_int),Parameter :: E_warning    =2_t_int
  Integer(t_int),Parameter :: E_error      =3_t_int
  Integer(t_int),Parameter :: E_fatalerror =4_t_int
  !! output levels
  !! These constants are also defined in vlevel.h, dont change them
  Integer(t_int),Parameter :: L_silent = 0_t_int
  Integer(t_int),Parameter :: L_info   = 100_t_int
  Integer(t_int),Parameter :: L_more   = 200_t_int
  Integer(t_int),Parameter :: L_many   = 300_t_int
  Integer(t_int),Parameter :: L_all    = 1000_t_int
  Integer(t_int),Parameter :: MAP_VERBOSITY_LEVELS(5)=(/L_silent,L_info,L_more&
       &,L_many,L_all/)
  !!***********************************************************************
  Integer(t_int),Parameter :: TERM_FINISH= 1_t_int
  Integer(t_int),Parameter :: TERM_NORMAL= 2_t_int
  Integer(t_int),Parameter :: TERM_ERROR = 3_t_int
  Integer(t_int),Parameter :: TERM_CRASH = 4_t_int
  Character(8),Parameter      :: TERMINATION_STATUS(4)=&
       &(/ "Finished","Normal  " ,"Error   ","Crash   "/)
  !!***********************************************************************
  !! global output level
  Character(11),Parameter      :: e_types(4)=(/ "INFORMATION" ,"WARNING    " ,"ERROR      " ,"FATAL ERROR" /)
  Integer(t_int)             :: outlevel=L_all
  ! for debugging or other reasons continuation on fatal errors may be
  ! desired
  Logical(t_log)            :: ContinueOnError=.False.
  Character(5000)           :: errbuf
  !!***********************************************************************
  !! debug
  Logical(t_log) :: g_debug
  Logical(t_log),Private             :: debug_initialized
  Integer(t_int),Private             :: nentries
  Character(200),Allocatable,Private :: entries(:,:)
  !!
  Data g_debug             /.False./
  Data debug_initialized   /.False./


Contains
  !!***********************************************************************
  !! handle error messages
  Subroutine errmsg(nmodul,nsubfun,Type,level,msg,titerstep)
    !!
    Implicit None 
    !!IO
    Character(*),Intent(in) :: nmodul,nsubfun
    Integer(t_int),Intent(in):: Type
    Character(*),Intent(in),Optional :: msg
    Integer(t_int),Intent(in),Optional:: level
    Integer(t_int),Intent(in),Optional:: titerstep
    !!LOCAL
    Character(*),Parameter::nsubfunc="errmsg"
    Integer(t_int):: lv
    !!
    Open(errfile,file=f_error,position='append')
    !!
    If(Type<E_information .Or. Type>E_fatalerror)Then
1      Format('ERROR in MODULE ',a,'(',a,'): unknown error level!')
       Write(outfile,1)Trim(nmodule),Trim(nsubfunc)
       Write(errfile,1)Trim(nmodule),Trim(nsubfunc)
3      Format('ABORT!')
       Write(outfile,3)
       Write(errfile,3)
       Close(errfile)
       Call terminate(TERM_CRASH)
    Endif
    !! give all
    lv=L_info
    If(Present(level))lv=level
    !! allways give warnings and errors
    If(Type>E_information)lv=L_silent

    If(outlevel>=lv)Then

       If(Present(msg))Then
          If(Present(titerstep))Then
             Write(unit=outfile,fmt="(a,' in MODULE ',a,'(',a,'):',/&
                  &,' Total iteration step: ',i5,/,' ',"//msg//")")&
                  &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun),titerstep
             If(Type>e_information)Then
                Write(errfile,fmt="(a,' in MODULE ',a,'(',a,'):',/&
                     &,' Total iteration step: ',i5,/,' ',"//msg//")")&
                     &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun),titerstep
             Endif
          Else
             Write(unit=outfile,fmt="(a,' in MODULE ',a,'(',a,'):',/,' ',"//msg//")")&
                  &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
             If(Type>e_information)Then
                Write(errfile,fmt="(a,' in MODULE ',a,'(',a,'):',/,' ',"//msg//")")&
                     &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
             Endif
          Endif
       Else
          Write(outfile,20)&
               &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
20        Format(a,' in MODULE ',a,'(',a,')')
          If(Type>e_information)Then
             Write(errfile,20)&
                  &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
          End If
          
       Endif
       
       If(Type==E_fatalerror.and. .not.ContinueOnError) Then
          Write(outfile,3)
          Write(errfile,3)
          Close(errfile)
          Call terminate(TERM_ERROR)
       Endif
    Endif
    
    Close(errfile)
  End Subroutine errmsg
  !!***********************************************************************
  !! handle error messages
  Subroutine cerrmsg(nmodul,nsubfun,Type)
    !!
    Implicit None 
    !!IO
    Character(*),Intent(in) :: nmodul,nsubfun
    Integer(t_int),Intent(in):: Type
    !!LOCAL
    Character(*),Parameter::nsubfunc="cerrmsg"
    Integer(t_int) :: i
    Character(1) :: msg
    !!
    External cgeterrormsg
    !!
    If(Type<E_information) return

    Open(errfile,file=f_error,position='append')

    If(Type>E_fatalerror)Then
       Write(unit=outfile,fmt="('ERROR in MODULE ',a,'(',a,'): unknown error level!')")Trim(nmodule),Trim(nsubfunc)
       Write(unit=errfile,fmt="('ERROR in MODULE ',a,'(',a,'): unknown error level!')")Trim(nmodule),Trim(nsubfunc)
       Write(unit=outfile,fmt="('ABORT!')")
       Write(unit=errfile,fmt="('ABORT!')")
       Close(errfile)
       Call terminate(TERM_CRASH)
    Endif
    
    
    Write(unit=outfile,fmt="(a,' in MODULE ',a,'(',a,'):')")&
         &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
    Write(unit=errfile,fmt="(a,' in MODULE ',a,'(',a,'):')")&
         &Trim(e_types(Type)),Trim(nmodul),Trim(nsubfun)
    
    i=0_t_int;
    Call cgeterrormsg(msg,i)
    Do While(Ichar(msg)/=0)
       If(Ichar(msg)==13)Then
          Write(outfile,"()")
          Write(errfile,"()")
       Else
          Write(errfile,"(a1)",advance='no')msg
          Write(outfile,"(a1)",advance='no')msg
       End If
       i=i+1_t_int
       Call cgeterrormsg(msg,i)
    End Do
    Write(errfile,"()")
    Write(outfile,"()")
    
    If(Type>=E_error) Then
       Write(unit=outfile,fmt="('ABORT!!')")
       Write(unit=errfile,fmt="('ABORT!!')")
       Close(errfile)
       Call terminate(TERM_ERROR)
    Endif
    Close(errfile)
    
  End Subroutine cerrmsg
  !!***********************************************************************
  !! set output level
  Subroutine setoutlevel(level)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)::level
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setoutlevel"
    !!
    outlevel=level
  End Subroutine setoutlevel
  !!***********************************************************************
  !! set output level
  Integer(t_int) Function getoutlevel() 
    !!
    implicit none 
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="getoutlevel"
    !!
    getoutlevel=outlevel
  End Function getoutlevel
  !!***********************************************************************
  Character(100) Function VERBOSITY_LEVELS(level)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in) :: level
    !!LOCAL
    Character(*),Parameter::nsubfunc="VERBOSITY_LEVELS"
    !!
    Select Case(Int(level))
       
    Case (Int(L_silent))
       VERBOSITY_LEVELS= 'silent'
    Case (Int(L_info))
       VERBOSITY_LEVELS= 'basic information'
    Case (Int(L_more))
       VERBOSITY_LEVELS= 'more information '
    Case (Int(L_many))
       VERBOSITY_LEVELS= 'much information '
    Case (Int(L_all))
       VERBOSITY_LEVELS= 'complete information  '
    Case default
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Unknown verbosity level !'")
    End Select
    
  End Function VERBOSITY_LEVELS
  !!***********************************************************************
  Subroutine  terminate(status,msg)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: status
    Character(*),Optional        :: msg
    !!LOCAL
    !!
    If(Present(msg))Then
       Write(outfile,1)Trim(TERMINATION_STATUS(status)),Trim(msg)
    Else
       Write(outfile,2)Trim(TERMINATION_STATUS(status))
    Endif
    Call request_slave_termination()
    Call clean_mpi()
    Stop
    !!
1   Format('TERMINATION: ',a,' : ',a)
2   Format('TERMINATION: ',a)
    !!
  End Subroutine terminate
  !!***********************************************************************
  Logical(t_log) Function debugfun(namemodule,namesubfunc,mode)
    !!
    Implicit None  
    !!IO
    Character(*),Intent(in)      :: namesubfunc,namemodule
    Character(*),Intent(in)      :: mode
    !!LOCAL
    Character(*),Parameter::nsubfunc="debugfun"
    Logical(t_log) :: exist_debug
    Integer(t_int) :: j,ic
    Integer        :: eof,icheck
    Character(5000):: buffer
    !!
    debugfun=.False.
    !!
    If(.Not.g_debug) Return
    !!
    initif: If(.Not. debug_initialized) Then
       !!
       debug_initialized=.True.
       Inquire(file=f_fplo_debug ,exist=exist_debug)
       If(.Not. exist_debug)Return
       !!
       !!
1      Format("'File ''",a,"'' found, will read debuging "&
            &,"information!',/")
       Write(errbuf,1)Trim(f_fplo_debug)
       Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       !!
       Do ic=1,2
          Open(debug_unit,file=f_fplo_debug,status='unknown')
          !!
          nentries=0
          readloop: Do
             !!
             Read(debug_unit,"(a)",iostat=eof) buffer
             !!
             If(eof < 0) Exit readloop
             !!
             nentries=nentries+1
             If(ic==2)entries(1:3,nentries)=''
             !!
             j=Index(buffer,":",.False.)
             If(j>1) Then
                If(ic==2) Then
                   entries(1,nentries)=Trim(Adjustl(buffer(1:j-1)))
                Endif
                !!
                buffer=Adjustl(buffer(j+1:))
                j=Index(buffer,":",.False.)
                If(j>1)Then
                   If(ic==2) Then
                      entries(2,nentries)=Trim(Adjustl(buffer(1:j-1)))
                   Endif
                   !!
                   buffer=Adjustl(buffer(j+1:))
                   If(ic==2) Then
                      entries(3,nentries)=Adjustl(buffer(1:200))
                   Endif
                Else If(j==0)Then
                   If(ic==2) Then
                      entries(2,nentries)=Adjustl(buffer(1:200))
                   Endif
                Endif
                !!
             Endif
             !!
          Enddo readloop
          !!
          Close(debug_unit)
          !!
          If (ic==1)Then
             Allocate(entries(3,nentries),STAT=icheck)
             If(icheck/=0)Then
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg="'Could not allocate memory for entries !'")
             Endif
          Endif
          !!
       Enddo

       Do ic=1,nentries
100       Format(a,': entry ',i2,': ',a,'/',a,'/',a)
          Write(outfile,100)nsubfunc,ic&
               &,Trim(entries(1,ic))&
               &,Trim(entries(2,ic))&
               &,Trim(entries(3,ic))
       Enddo
       Write(outfile,101)
101    Format(/,/)
    Endif initif
    !!
    !! have allready read the file, check it
    !!
    checkloop: Do ic=1,nentries
       
       If(Trim(entries(1,ic))  /= Trim(namemodule)) Cycle checkloop
       If(Trim(entries(2,ic))  /= Trim(namesubfunc)) Cycle checkloop
       If(Trim(entries(3,ic))  /= Trim(mode)) Cycle checkloop
       
       debugfun=.True.
       Exit checkloop
    Enddo checkloop
    
    If(debugfun)Then
2      Format("'Start debuging of routine ",a,"::",a,", mode ",a,"!',/")
       Write(errbuf,2)&
            &Trim(namemodule),Trim(namesubfunc),Trim(mode)
       Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       
    Endif
    !!
    !!
    Return 
    !!
  End Function debugfun
  !!***********************************************************************
  
End Module errors
