!!----DEBUG options-----------------|
!! SUBFUNC    | MODES               |
!!----------------------------------|
!!wave_coef_dW_rel4                 |
!!wave_coef_dW_kh                   |
!!----------------------------------|
!!TODO: one has to make sure that rmin is below the finite nucleus radius of all atoms
Module finuc
  !!
  Use precisiondefs
  Use constants
  Use errors
  Use file_iounits
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="FINUC"  
  !!
  Public :: nuc_zeta,nuc_pot_gauss&
       &,wave_coef_nrel,wave_coef_dW_nrel&
       &,wave_coef_dW_rel4,wave_coef_dW_kh&
       &,horner&
       &,nuc_max_rad,nuc_pot_coefs&
       &,FINUC_NONE&
       &,FINUC_POW0,FINUC_POW1,FINUC_POW2&
       &,FINUC_HUMP3,FINUC_HUMP5,FINUC_GAUSS&
       &,FINUC_TYPE
  !!
  !!***********************************************************************
  !!
  !!
  Integer(t_int),Parameter :: FINUC_NONE=1_t_int
  Integer(t_int),Parameter :: FINUC_POW0=2_t_int
  Integer(t_int),Parameter :: FINUC_POW1=3_t_int
  Integer(t_int),Parameter :: FINUC_POW2=4_t_int
  Integer(t_int),Parameter :: FINUC_HUMP3=5_t_int
  Integer(t_int),Parameter :: FINUC_HUMP5=6_t_int
  Integer(t_int),Parameter :: FINUC_GAUSS=7_t_int
  !!
  Character(17)              :: FINUC_TYPE(7)
  !!
  Data FINUC_TYPE&
       &/"Point charge     "&
       &,"Constant density "&
       &,"Linear density   "&
       &,"Square density   "&
       &,"Cubic density    "&
       &,"Quintic density  "&
       &,"Gauss density    "&
       &/

  !!
  Integer(t_int),Parameter :: max_isotope_Z=109
  Real(t_real)   :: isotope_number(0:max_isotope_Z)
  Data isotope_number(0:10)    / 0,   1,   4,  7,  9, 11, 12, 14, 16, 19, 20/
  Data isotope_number(11:20)   / 23, 24, 27, 28, 31, 32, 35, 40, 39, 40/
  Data isotope_number(21:30)   / 45, 48, 51, 52, 55, 56, 59, 58, 63, 64/
  Data isotope_number(31:40)   / 69, 74, 75, 80, 79, 84, 85, 88, 89, 90/
  Data isotope_number(41:50)   / 93, 98, 98,102,103,106,107,114,115,120/
  Data isotope_number(51:60)   /121,130,127,132,133,138,139,140,141,144/
  Data isotope_number(61:70)   /145,152,153,158,159,162,162,168,169,174/
  Data isotope_number(71:80)   /175,180,181,184,187,192,193,195,197,202/
  Data isotope_number(81:90)   /205,208,209,209,210,222,223,226,227,232/
  Data isotope_number(91:100)  /231,238,237,244,243,247,247,251,252,257/
  Data isotope_number(101:109) /258,259,262,261,262,263,262,265,266/
  !!
  Real(t_real)   :: pow0_coeffs(0:3),pow1_coeffs(0:4),pow2_coeffs(0:5)
  Real(t_real)   :: hump3_coeffs(0:6),hump5_coeffs(0:8)
  !!
  Data pow0_coeffs(0:3) /0.0_t_real,1.5_t_real,0.0_t_real,-0.5_t_real/
  Data pow1_coeffs(0:4) /0.0_t_real,2.0_t_real,0.0_t_real,-2.0_t_real,1.0_t_real/
  Data pow2_coeffs(0:5) /0.0_t_real,2.5_t_real,0.0_t_real,-5.0_t_real,5.0_t_real&
       &,-1.5_t_real/
  Data hump5_coeffs(0:8) /0.0_t_real,2.4_t_real,0.0_t_real,-2.8_t_real,0.0_t_real&
       &,0.0_t_real,5.6_t_real,-6.0_t_real,1.8_t_real/
  Data hump3_coeffs(0:6) /0.0_t_real,2.25_t_real,0.0_t_real,-2.5_t_real,0.0_t_real&
       &,2.25_t_real,-1.0_t_real/
  !!
  !!***********************************************************************
  !!
Contains
  !!***********************************************************************
  !! phenomanological nuclear root mean square <r^2> from linear interpolation
  !! of isotope numbers ---- in bohr radii
  Real(t_real) Function nuc_rms(Z)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: Z
    !!LOCAL
    Character(*),Parameter:: nsubfunc="nuc_rms"
    Integer(t_int) :: iZ0,iZ1
    Real(t_real)   :: A0,A1,A
    !!
    iZ0=Floor(Z)
    iZ1=Ceiling(Z)
    If(iZ1==iZ0) iZ1=iZ1+1
    If(iZ0 >= max_isotope_Z) Then
       iZ0=max_isotope_Z-1
       iZ1=max_isotope_Z
    Endif
    !Print *,'Z=',Z,iZ0,iZ1
    If(Z> max_isotope_Z+1.0e-3_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'nuclear charge larger than max_isotope_Z requested!'")
    Endif
    !!
    !!
    A0=isotope_number(iZ0)
    A1=isotope_number(iZ1)
    !!
    A=(Z-iZ1)/Real(iZ0-iZ1,t_real)*A0+(Z-iZ0)/Real(iZ1-iZ0,t_real)*A1 
    !!
    !Print *,'A=',A,A0,A1
    !!
    nuc_rms=(0.836*A**c1o3+0.570) *1.0e-5_t_real/brtoan 
    !!
    Return
    !!
  End Function nuc_rms
  !!***********************************************************************
  Real(t_real) Function nuc_pot_gauss(r,Z)
    !!
    Implicit none  
    !!IO
    Real(t_real),Intent(in)      :: r,Z
    !!LOCAL
    Real(t_real)   :: rms,r0,x,eps,alpha
    !!
    rms=nuc_rms(Z)
    !!
    eps=(1.0e-6_t_real)**2
    alpha=Sqrt(3.0_t_real/2.0_t_real)
    r0=Sqrt(Log((1.5_t_real/pi/rms**2)**3/eps))/Sqrt(3.0_t_real)*rms
    !Print *,'r0=',r0
    !!
    If(r>r0)Then
       nuc_pot_gauss=1.0_t_real/r
    Else if(r==0.0_t_real)then
       nuc_pot_gauss=2.0_t_real/sqpi*alpha/rms
    Else
       x=r/rms
       nuc_pot_gauss=erf(alpha*x)/r
    Endif
    !!
    Return
    !!
  End Function nuc_pot_gauss
  !!***********************************************************************
  !! Vcoeff is potential expansion
  !! r*V =-Z * sum_{n=0} Vcoeff_n x^n   x=r/R
  !!
  Subroutine wave_coef_nrel(l,r0,ee,ZZ,Vcoeff,Nv,Gcoeff,Ng)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: l !! angular momentum
    Integer(t_int),Intent(in)    :: Ng !! nunmber of coeffs in G
    Integer(t_int),Intent(in)    :: Nv !! nunmber of coeffs in r*V
    Real(t_real),Intent(in)      :: r0 !! nuclear confinement radius
    Real(t_real),Intent(in)      :: ee !! energy
    Real(t_real),Intent(in)      :: ZZ !! nuclear charge
    Real(t_real),Intent(in)      :: Vcoeff(0:Nv)
    Real(t_real),Intent(out)     :: Gcoeff(0:Ng)
    !!LOCAL
    Integer(t_int) :: deg,n,k
    Real(t_real)   :: v
    !!
    deg=2*l+1
    !!
    Gcoeff(:)=0.0_t_real
    !!
    Gcoeff(0)=1.0_t_real
    !! finite nucleus is contained in the potential coefficients

    Do n=1,Ng
       !!
       v=0.0_t_real
       Do k=1,Min(n,Nv+1)
          v=v-ZZ*Vcoeff(k-1)*Gcoeff(n-k)
       Enddo
       !!
       If(n==1)Then
          Gcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(v)
       Else
          Gcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(v-(ee*r0)*Gcoeff(n-2))
       Endif
    Enddo
    !!
    Return
    !!
  End Subroutine wave_coef_nrel
  !!***********************************************************************
  !! Vcoeff is potential derivative expansion
  !! r*V =-Z * sum_{n=0} Vcoeff_n x^n   x=r/R
  !!
  Subroutine wave_coef_dW_nrel(l,r0,ee,dedw,ZZ,Vcoeff,Nv&
       &,Gcoeff,dGcoeff,Ng)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: l !! angular momentum
    Integer(t_int),Intent(in)    :: Ng !! nunmber of coeffs in G
    Integer(t_int),Intent(in)    :: Nv !! nunmber of coeffs in r*V
    Real(t_real),Intent(in)      :: r0 !! nuclear confinement radius
    Real(t_real),Intent(in)      :: ee !! energy
    Real(t_real),Intent(in)      :: dedw !! energy derivative
    Real(t_real),Intent(in)      :: ZZ !! nuclear charge
    Real(t_real),Intent(in)      :: Vcoeff(0:Nv) 
    Real(t_real),Intent(out)     :: dGcoeff(0:Ng)
    Real(t_real),Intent(out)     :: Gcoeff(0:Ng)
    !!LOCAL
    Integer(t_int) :: deg,n,k
    Real(t_real)   :: v,vdg
    !!
    deg=2*l+1
    !!
    dGcoeff(:)=0.0_t_real
    Gcoeff(:)=0.0_t_real
    !!
    Gcoeff(0)=1.0_t_real
    dGcoeff(0)=0.0_t_real
    !! finite nucleus is contained in the potential coefficients

    Do n=1,Ng
       !!
       vdg=0.0_t_real
       Do k=1,Min(n,Nv+1)
          vdg=vdg-ZZ*Vcoeff(k-1)*dGcoeff(n-k)
       Enddo
       v=0.0_t_real
       Do k=1,Min(n,Nv+1)
          v=v-ZZ*Vcoeff(k-1)*Gcoeff(n-k)
       Enddo
       !!
       If(n==1)Then
          Gcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(v)
          dGcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(vdg)
       Else
          Gcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(v-(ee*r0)*Gcoeff(n-2))
          dGcoeff(n)=2.0_t_real*r0/(n**2+deg*n) *(vdg-(ee*r0)*dGcoeff(n-2)&
               &-(dedw*r0)*Gcoeff(n-2))
       Endif
    Enddo
    !!
    Return
    !!
  End Subroutine wave_coef_dW_nrel
  !!***********************************************************************
  !! the confinement radius of the nuclear material
  !! outside this radius there is no or very little nuclear density left
  Real(t_real) Function nuc_max_rad(nuc_type,Z)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nuc_type !! which model
    Real(t_real),Intent(in)      :: Z !! nuclear charge
    !!LOCAL
    Character(*),Parameter:: nsubfunc="nuc_max_rad"
    Real(t_real)   :: rms
    !!
    rms=nuc_rms(Z)
    !!
    If(nuc_type ==FINUC_NONE) Then
       nuc_max_rad= rms
    Else If(nuc_type ==FINUC_POW0) Then
       nuc_max_rad=Sqrt(5.0_t_real/3.0_t_real)*rms
    Else if(nuc_type ==FINUC_POW1) Then
       nuc_max_rad=Sqrt(5.0_t_real/2.0_t_real)*rms
    Else if(nuc_type ==FINUC_POW2) Then
       nuc_max_rad=Sqrt(7.0_t_real/2.0_t_real)*rms
    Else if(nuc_type ==FINUC_HUMP3) Then
       nuc_max_rad=Sqrt(28.0_t_real/9.0_t_real)*rms
    Else if(nuc_type ==FINUC_HUMP5) Then
       nuc_max_rad=Sqrt(25.0_t_real/7.0_t_real)*rms
    Else
       nuc_max_rad= rms
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'nuclear charge model unknown!'")
    Endif
    !!
    Return
    !!
  End Function nuc_max_rad
  !!***********************************************************************
  !! coefficients of f(x)=r*v(r)  x=r/R
  !! v_{n-1}=f_n R^{-n}
  Subroutine nuc_pot_coefs(nuc_type,coeffs,N)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nuc_type !! model
    Integer(t_int),Intent(in)    :: N
    Real(t_real),Intent(out)     :: coeffs(0:N)
    !!LOCAL
    Character(*),Parameter:: nsubfunc="nuc_pot_coefs"
    Integer(t_int) :: nmin
    !!
    coeffs(:)=0.0_t_real
    !!
    If(nuc_type ==FINUC_NONE) Then
       coeffs(0)=1.0_t_real
    Else If(nuc_type ==FINUC_POW0) Then
       nmin=Min(N,3_t_int)
       coeffs(0:nmin)=pow0_coeffs(0:nmin)
    Else If(nuc_type ==FINUC_POW1) Then
       nmin=Min(N,4_t_int)
       coeffs(0:nmin)=pow1_coeffs(0:nmin)
    Else If(nuc_type ==FINUC_POW2) Then
       nmin=Min(N,5_t_int)
       coeffs(0:nmin)=pow2_coeffs(0:nmin)
    Else If(nuc_type ==FINUC_HUMP3) Then
       nmin=Min(N,6_t_int)
       coeffs(0:nmin)=hump3_coeffs(0:nmin)
    Else If(nuc_type ==FINUC_HUMP5) Then
       nmin=Min(N,8_t_int)
       coeffs(0:nmin)=hump5_coeffs(0:nmin)
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'nuclear charge model unknown!'")
    Endif
    !!
    Return
    !!
  End Subroutine nuc_pot_coefs
  !!***********************************************************************
  Real(t_real) Function horner(x,coeffs,N,fp)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: N
    Real(t_real),Intent(in)      :: x,coeffs(0:N)
    Real(t_real),Intent(out),Optional :: fp
    !!LOCAL
    Logical(t_log) :: pres
    Integer(t_int) :: i
    Real(t_real)   :: d
    !!
    pres=Present(fp)
    !!
    d=coeffs(N)
    If(pres) fp=d
    d=coeffs(N-1)+x*d
    Do i=2,N
       if(pres) fp=d+x*fp
       d=coeffs(N-i)+x*d
    Enddo
    horner=d
    !!
  End Function horner
  !!***********************************************************************
  Real(t_real) Function nuc_zeta(xx,nuc_type)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nuc_type
    Real(t_real),Intent(in)      :: xx
    !!LOCAL
    Character(*),Parameter:: nsubfunc="nuc_zeta"
    !!
    If(xx>1.0_t_real)Then
       nuc_zeta=0.0_t_real
       Return
    Endif
    !!
    !!
    !!
    If(nuc_type ==FINUC_NONE) Then
       nuc_zeta=0.0_t_real
    Else If(nuc_type ==FINUC_POW0) Then
       nuc_zeta=horner(xx,pow0_coeffs(0:3),3_t_int)-1.0_t_real
    Else If(nuc_type ==FINUC_POW1) Then
       nuc_zeta=horner(xx,pow1_coeffs(0:4),4_t_int)-1.0_t_real
    Else If(nuc_type ==FINUC_POW2) Then
       nuc_zeta=horner(xx,pow2_coeffs(0:5),5_t_int)-1.0_t_real
    Else If(nuc_type ==FINUC_HUMP3) Then
       nuc_zeta=horner(xx,hump3_coeffs(0:6),6_t_int)-1.0_t_real
    Else If(nuc_type ==FINUC_HUMP5) Then
       nuc_zeta=horner(xx,hump5_coeffs(0:8),8_t_int)-1.0_t_real
    Else
       nuc_zeta=0.0_t_real
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'nuclear charge model unknown!'")
    Endif
    !!
    !!
    !!
    Return
    !!
  End Function nuc_zeta
  !!***********************************************************************
  !! Vcoeff is potential expansion
  !! r*V =-Z * sum_{n=0} Vcoeff_n x^n   x=r/R
  !!
  Subroutine wave_coef_dW_rel4(ipart,kap,cm,r0,ee,dedw,ZZ,Vcoeff,Bcoeff,Nv&
       &,Gcoeff,dGcoeff,Ng,gamma)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: ipart !! which partial wave (ODE 4)
    Integer(t_int),Intent(in)    :: kap!! angular momentum
    Integer(t_int),Intent(in)    :: Ng !! nunmber of coeffs in G
    Integer(t_int),Intent(in)    :: Nv !! nunmber of coeffs in r*V
    Real(t_real),Intent(in)      :: cm(5) !! B-coupling coefficients
    Real(t_real),Intent(in)      :: r0 !! nuclear confinement radius
    Real(t_real),Intent(in)      :: ee,dedw !! energy
    Real(t_real),Intent(in)      :: ZZ !! nuclear charge
    Real(t_real),Intent(in)      :: Vcoeff(0:Nv)
    Real(t_real),Intent(in)      :: Bcoeff(0:Nv)
    Real(t_real),Intent(out)     :: Gcoeff(0:Ng,1:4)
    Real(t_real),Intent(out)     :: dGcoeff(0:Ng,1:4)
    Real(t_real),Intent(out)     :: gamma
    !!LOCAL
    Logical(t_log) :: prot,lv0
    Integer(t_int) :: n,k,kapq,j
    Real(t_real)   :: c,c2,vm1,M(4,4),det,A(4,4),V(4),vk,bk,ekF,ekG,dek
    Real(t_real)   :: dA(4,4),dV(4)
    Real(t_real),Parameter :: zero=0.0_t_real
    !!
    prot=debugfun(nmodule,'wave_coef_dW_rel4','')
    !!
    If(prot) Write(outfile,"('========== START ==============')")
    c=speed_of_light
    c2=c**2
    !!
    vm1=-ZZ*Vcoeff(0)
    !!
    lv0=(Abs(vm1)<1.0e-6_t_real)
    !!
    dGcoeff(:,:)=zero
    Gcoeff(:,:)=zero
    !!
    kapq=-kap-1
    If(ipart ==1 )Then 
       gamma=Sqrt(kap**2-(vm1/c)**2)
    Else
       gamma=Sqrt(kapq**2-(vm1/c)**2)
    Endif
    !!
    !! first partial wave
    If(ipart == 1 ) Then
       If(kap<0) Then
          Gcoeff(0,1)=1.0_t_real
          Gcoeff(0,2)=vm1/(gamma-kap)
       Else
          If(lv0)Then
             Gcoeff(0,1)=zero
             Gcoeff(0,2)=1.0_t_real
          Else
             Gcoeff(0,1)=1.0_t_real
             Gcoeff(0,2)=-c2*(gamma+kap)/vm1
          Endif
       Endif
    Else
       If(kapq<0) Then
          Gcoeff(0,3)=1.0_t_real
          Gcoeff(0,4)=vm1/(gamma-kapq)
       Else
          If(lv0)Then
             Gcoeff(0,3)=zero
             Gcoeff(0,4)=1.0_t_real
          Else
             Gcoeff(0,3)=1.0_t_real
             Gcoeff(0,4)=-c2*(gamma+kapq)/vm1
          Endif
       Endif
    Endif
    !! finite nucleus is contained in the potential coefficients
    !!
    !!
    !!
    M=zero
    Do n=1,Ng
       !!
       If(prot) Write(outfile,"('n=',i5)") n
       !!
       If(n==1 .And. ipart == 2 .And. kapq>0 .And. (lv0) ) Then
          det=2*(gamma+1.0_t_real)
          M(1,1:2)=(/ zero     , zero /)
          M(2,1:2)=(/ zero     , 1.0_t_real/det /)
       Else
          det=(gamma+n)**2-kap**2+(vm1/c)**2
          If(prot) Write(outfile,"('det1=',e30.15e3)") det
          M(1,1:2)=(/ gamma+n-kap, -vm1/c2 /)/det
          M(2,1:2)=(/ vm1     , gamma+n+kap /)/det
       Endif

       If(n==1 .And. ipart == 1 .And. kap>0 .And. (lv0) ) Then
          det=2*(gamma+1.0_t_real)
          M(3,3:4)=(/ zero       , zero /)
          M(4,3:4)=(/ zero     , 1.0_t_real/det /)
       Else
          det=(gamma+n)**2-kapq**2+(vm1/c)**2
          If(prot) Write(outfile,"('det2=',e30.15e3)") det
          M(3,3:4)=(/ gamma+n-kapq, -vm1/c2 /)/det
          M(4,3:4)=(/ vm1     , gamma+n+kapq /)/det
       Endif

       !!
       If(prot) Then
          Do j=1,4
             Write(outfile,"(4(e12.5e2,2x))")M(j,1:4)
          Enddo
          Write(outfile,*)
       Endif
       V=zero
       dV=zero
       !!
       Do k=1,Min(n,Nv)
          If(prot) Write(outfile,"('k=',i5)") k
       
          vk=-ZZ*Vcoeff(k)
          bk=-ZZ*Bcoeff(k)
          If(prot) Write(outfile,"('VB=',2(e30.15e3,2x))") vk,bk
          ekF=zero
          ekG=zero
          dek=zero
          If(k==1) Then
             ekF=(ee+2*c2)*r0
             ekG=(ee)*r0
             dek=dedw*r0
          Endif
          A=zero
          A(1,2)=(ekF-vk+bk*cm(4))/c2
          A(2,1)=-(ekG-vk-bk*cm(1))
          A(2,3)= bk*cm(2) 
          A(3,4)= (ekF-vk+bk*cm(5))/c2 
          A(4,1)= bk*cm(2)
          A(4,3)=  -(ekG-vk-bk*cm(3)) 

          dA=zero
          dA(1,2)= (dek)/c2
          dA(2,1)=-(dek)
          dA(3,4)= (dek)/c2 
          dA(4,3)=-(dek) 

          V=V+Matmul(A,Gcoeff(n-k,1:4))

          If(prot) Then
             Do j=1,4
                Write(outfile,"('A=',4(e30.15e3,2x))")A(j,1:4)
             Enddo
             Write(outfile,*)
             
             Write(outfile,"('V=',4(e30.15e3,2x))")V(1:4)
             Write(outfile,*)
          Endif


          dV=dV+Matmul(dA,Gcoeff(n-k,1:4))+Matmul(A,dGcoeff(n-k,1:4))
       Enddo
       !!
       Gcoeff(n,1:4)= Matmul(M,V)
       dGcoeff(n,1:4)= Matmul(M,dV)
    Enddo
    !!
    If(prot) Write(outfile,"('========== END   ==============')")
    !!
    Return
    !!
  End Subroutine wave_coef_dW_rel4
  !!***********************************************************************
  !! Vcoeff is potential expansion
  !! r*V =-Z * sum_{n=0} Vcoeff_n x^n   x=r/R
  !!
  Subroutine wave_coef_dW_kh(la,r0,ee,ZZ,Vcoeff,Nv,Gcoeff,Ng,gamma)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: la!! angular momentum
    Integer(t_int),Intent(in)    :: Ng !! nunmber of coeffs in G
    Integer(t_int),Intent(in)    :: Nv !! nunmber of coeffs in r*V
    Real(t_real),Intent(in)      :: r0 !! nuclear confinement radius
    Real(t_real),Intent(in)      :: ee !! energy
    Real(t_real),Intent(in)      :: ZZ !! nuclear charge
    Real(t_real),Intent(in)      :: Vcoeff(0:Nv)
    Real(t_real),Intent(out)     :: Gcoeff(0:Ng,1:3)
    Real(t_real),Intent(out)     :: gamma
    !!LOCAL
    Logical(t_log) :: prot,lv0
    Integer(t_int) :: n,k,j
    Real(t_real)   :: M(3,3),det,gnm1,gnp1,A(3),V(3),vk,ekF,ekG
    Real(t_real)   :: L,sL,vm1,c,c2,m21,P(0:Ng,1:3)
    Real(t_real),Parameter :: zero=0.0_t_real
    !!
    prot=debugfun(nmodule,'wave_coef_dW_kh','')
    !!
    If(prot) Write(outfile,"('========== START ==============')")
    c=speed_of_light
    c2=c**2
    !!
    vm1=-ZZ*Vcoeff(0)
    !!
    Gcoeff(:,:)=zero
    !!
    lv0=(Abs(vm1)<1.0e-6_t_real)
    !!
    L=Real(la*(la+1),t_real)
    sL=Sqrt(L)
    !!
    !!
    If( .Not.lv0 .Or. la == 0  )Then
       !!
       !!
       gamma=Sqrt(L+1-(vm1/c)**2)
       !!
       If(lv0.And.  la==0)Then
          Gcoeff(0,1)=1.0_t_real
          Gcoeff(0,2)=0.0_t_real
          Gcoeff(0,3)=0.0_t_real
       Else
          Gcoeff(0,1)=vm1/c2
          Gcoeff(0,2)=(1.0_t_real-gamma)
          Gcoeff(0,3)=-sL
       Endif
       !! finite nucleus is contained in the potential coefficients
       !!
       !!
       !!
       M=zero
       Do n=1,Ng
          !!
          If(prot) Write(outfile,"('n=',i5)") n
          !!
          det=(2.0_t_real*gamma+n)*n
          gnm1=gamma+n-1.0_t_real
          gnp1=gamma+n+1.0_t_real
          If(prot) Write(outfile,"('det1=',e30.15e3)") det
          If(prot) Write(outfile,"('gnm1=',e30.15e3)") gnm1
          If(prot) Write(outfile,"('gnp1=',e30.15e3)") gnp1
          If(lv0.And.la==0)Then
             M(1,1:3)=(/ 0.0_t_real , gnp1           , 0.0_t_real /)/det
             M(2,1:3)=(/ -gnm1      , 0.0_t_real     , 0.0_t_real /)/det
             M(3,1:3)=(/ 0.0_t_real , 0.0_t_real     , 1.0_t_real /)/det
          Else
             M(1,1:3)=(/ vm1/c2 , gnp1               , -sL            /)/det
             M(2,1:3)=(/ -gnm1  , -(L*c2-vm1**2)/vm1 , sL*c2*gnm1/vm1 /)/det
             M(3,1:3)=(/ -sL    , -sL*c2*gnp1/vm1    , c2*(L+det)/vm1 /)/det
          Endif

          !!
          If(prot) Then
             Do j=1,3
                Write(outfile,"(3(e12.5e2,2x))")M(j,1:3)
             Enddo
             Write(outfile,*)
          Endif
          V=zero
          !!
          Do k=1,Min(n,Nv)
             If(prot) Write(outfile,"('k=',i5)") k

             vk=-ZZ*Vcoeff(k)
             If(prot) Write(outfile,"('Vk=',1(e30.15e3,2x))") vk
             ekG=zero
             ekF=zero
             If(k==1) Then
                ekG=(ee)*r0
                ekF=(ee+2*c2)*r0
             Endif
             A=zero
             A(1)=(ekG-vk)
             A(2)=(ekF-vk)/c2
             A(3)=(ekF-vk)/c2


             V=V+A(1:3)*Gcoeff(n-k,1:3)

             If(prot) Then
                Write(outfile,"('A=',3(e30.15e3,2x))")A(1:3)
                Write(outfile,*)

                Write(outfile,"('V=',3(e30.15e3,2x))")V(1:3)
                Write(outfile,*)
             Endif


          Enddo
          !!
          Gcoeff(n,1:3)= Matmul(M,V)
       Enddo
       !!
    Else
       !! Vcoeff(0) == 0
       gamma=la
       vk=-ZZ*Vcoeff(1)
       m21=2.0_t_real+(ee-vk/r0)/c2
       !Print *,'m21=',m21
       P(0,1)=1.0_t_real
       P(0,2)=la/m21
       P(0,3)=sL/m21
       !!
       !!
       M=zero
       Do n=1,Ng
          !!
          If(prot) Write(outfile,"('n=',i5)") n
          !!
          det=(2.0_t_real*la+1.0_t_real+n)*n
          gnm1=la+n
          gnp1=la+n+1.0_t_real
          If(prot) Write(outfile,"('det1=',e30.15e3)") det
          M(1,1:3)=(/ -m21   , gnp1           , -sL            /)/det
          M(2,1:3)=(/ -gnm1  , L/m21          , -sL*gnm1/m21   /)/det
          M(3,1:3)=(/ -sL    , sL*gnp1/m21    , -(L+det)/m21   /)/det

          !!
          If(prot) Then
             Do j=1,3
                Write(outfile,"(3(e12.5e2,2x))")M(j,1:3)
             Enddo
             Write(outfile,*)
          Endif
          V=zero
          !!
          !!
          Do k=1,Min(n,Nv)
             If(prot) Write(outfile,"('k=',i5)") k

             A=zero
             vk=-ZZ*Vcoeff(k+1)
             If(prot) Write(outfile,"('Vk+1=',1(e30.15e3,2x))") vk
             A(2:3)=-vk/c2/r0

             If(k>1) Then

                vk=-ZZ*Vcoeff(k-1)
                If(prot) Write(outfile,"('Vk-1=',1(e30.15e3,2x))") vk

                ekG=0.0_t_real
                If(k==2) Then
                   ekG=ee
                Endif
                A(1)=ekG-vk*r0
             Endif

             V=V+A(1:3)*P(n-k,1:3)

             If(prot) Then
                Write(outfile,"('A=',3(e30.15e3,2x))")A(1:3)
                Write(outfile,*)

                Write(outfile,"('V=',3(e30.15e3,2x))")V(1:3)
                Write(outfile,*)
             Endif


          Enddo
          !!
          P(n,1:3)= Matmul(M,V)
       Enddo
       !!
       Gcoeff(0,1)=0.0_t_real
       Gcoeff(0,2:3)=P(0,2:3)
       Do n=1,Ng
          Gcoeff(n,1)=P(n-1,1)*r0
          Gcoeff(n,2:3)=P(n,2:3)
       Enddo
    Endif
    !!
    !!
    If(prot) Write(outfile,"('========== END   ==============')")
    !!
    Return
    !!
  End Subroutine wave_coef_dW_kh
  !!***********************************************************************
End Module finuc
