!!----DEBUG options---------------------|
!! MODULE:SUBFUNC:MODES                 |
!!--------------------------------------|
!---------------------------------------!
! FORCEDRIVER:sccforce:separate_outfile !
! FORCEDRIVER:sccforce:log_history      |
! FORCEDRIVER:checkinout:               |
!---------------------------------------!

Module forcedriver
  !!
  Use precisiondefs
  Use constants
  Use file_iounits
  Use tio
  Use titeration
  Use iteration
  Use tsymmetry
  Use errors
  Use tforces
  Use clock
  Use fplodriver
  Use fplodata
  Use ioaux
  Use forces
  Use fplostartup
  Use io
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="FORCEDRIVER"  
  !!
  Public :: sccforce
  !!
  !!***********************************************************************
  Integer(t_int),Parameter ::it_checkin =1_t_int
  Integer(t_int),Parameter ::it_checkout=0_t_int
  !!***********************************************************************
  Integer(t_int) :: tstep
  !!
  !!
  !!
  !!***********************************************************************
Contains
  Subroutine sccforce(Data,Entry)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)       ::Entry
    Type(T_iteration),Intent(inout) :: Data
    !!LOCAL
    Character(*),Parameter  :: nsubfunc="sccforce"
    Character(*),Parameter  :: cpumesg1='total force calculation'
    Character(*),parameter  :: cpumesg2='force cycle'
    Logical(t_log) :: deb,debc
    Character(100) :: fname
    Integer(t_int) :: isa
    !!------------------------------------------------------
    deb=debugfun(nmodule,nsubfunc,'separate_outfile')
    debc=debugfun('FORCEDRIVER','checkinout','')
    !!
!!$1   Format(72('='),/,15('-'),' START ',a,'(',a,')',a)
!!$    If(outlevel>L_many)&
!!$         &Write(outfile,1)nsubfunc&
!!$         &,Trim(iteration_entries(Entry))&
!!$         &,Repeat('-',72_t_int-15_t_int-9_t_int-Len(nsubfunc)&
!!$         &-Len(Trim(iteration_entries(Entry))))

    data% erg=0.0_t_real
    

    If(Entry==it_entry_init)Then
       !!------------------------------------------------------
       !! put initialization of iteration scheme here 
       !! make what ever you want to do first
       !!------------------------------------------------------
       
       !! initialize 


       Call watch(cpumesg1,cpu_start,reset=.True.,sccmesg=data%name)

       
       data%scheme=g_forces% forceiteration_control % scheme
       
       !! return and let iteration module make default settings
    Elseif(Entry==it_entry_prepare)Then
       !!------------------------------------------------------
       !! put initialization of iteration control settings here 
       !! make what ever you want to do before checkin of initial data
       !!------------------------------------------------------
       
       !!
       !!
       !!
       !! dimension of vector
       data%vecdim=g_forces% Ndegfree 
       If(data%vecdim==0 )Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'it seems that there is nothing to optimize. ',/,'All Wyckoff positions are determined by symmetry!'")
          
       Endif
       !!
       data%tolerance=g_forces% forceiteration_control % tolerance 
       data%maxdim=g_forces% forceiteration_control % maxdim
       If(data%maxdim==0) data%maxdim=data%vecdim
       data%steps   =g_forces% forceiteration_control % steps      
       
       data%convcond =CONVCOND_ITERAT

       
       !! return and let iterate initialize arrays
    Elseif(Entry==it_entry_startdata)Then
       !!------------------------------------------------------
       !! initialize iteration vector
       !!------------------------------------------------------

       !!
       !!
       !! make savfty backups
       !!
       Write(fname,"(a,'.bak.forces')")Trim(f_fplo_in)
       Call copy_file_to_file(fname,f_fplo_in)
       Write(fname,"(a,'.bak.forces')")Trim(f_fplo_sym)
       Call copy_file_to_file(fname,f_fplo_sym)
       !!
       !!
       !!
       !!
       !!
       !! put sites into vector
       Call checkinout(it_checkin,data%vecdim,data% vector,g_symmetry&
            &,g_structure,g_forces,data% name)

       
       data%reset=.False.
       tstep=0

       !! return and let iteration module start the scheme
    Elseif(Entry==it_entry_loop)Then
       !!------------------------------------------------------
       !! this is one step of iteration loop which transforms
       !! old vector into new vector
       !!------------------------------------------------------
       data%reset=.False.       
       
       Call watch(cpumesg2,cpu_start,reset=.True.,sccmesg=data%name)
       !!
       !! get sites from vector
       Call checkinout(it_checkout,data%vecdim,data% vector,g_symmetry&
            &,g_structure,g_forces,data% name)
       
       tstep=tstep+1


       If(deb)Then
          outfile=8
          Open(outfile,file='out'//Trim(intToChar10(tstep)))
       Endif
       !! one full fplo cycle is one force cycle

       Write(outfile,900)&
            & g_structure% nsite&
            &,g_structure% naps&
            &,g_structure% naps_a
900    Format(/,72('='),/,'Force-step: START SCF iteration ',/,72('='),/,/&
            &,'                 Atom sites',/&
            &,'Number of sites               : ',i5,/&
            &,'Number of real sites          : ',i5,/&
            &,'Number of nonempty real sites : ',i5,/&
            &,'No.  Element WPS CPA-Block    X                      &
            &Y                      Z'&
            &)
       !!
       Do isa=1,g_structure% nsite
          Write(outfile,901) isa&
               &,g_structure % p_IOelements( g_structure % p_IOisrt(isa))&
               &,g_structure % p_IOisrt(isa),g_structure % p_kaps(isa)&
               &,g_structure % p_tau(:,isa)
       Enddo
       !!
       Write(outfile,902)
901    Format(i4,3x,a2,2x,i4,3x,i4,3x,3(f22.15,1x))
902    Format(/,/)
       !!
       !Call write_in_sym_file(g_symmetry,g_structure)
       !!
       If(debugfun(nmodule,nsubfunc,'log_history'))&
            &Call log_site_history(g_symmetry,g_structure,g_forces)
       !!
       Call fplo_init_fixed(g_compound,g_mspin,g_atommeshio,g_pmeshio&
            &,g_radialmesh_rmaxfactor&
            &,g_symmetry,g_structure,g_lattice,g_sic&
            &,g_numerics,g_potential,g_basis,g_kohnsham,g_bandplot,g_bzone&
            &,g_loi,g_mol,g_quant,g_wannier&
            &,g_forces,g_etotstack,g_relativistic&
            &,g_initial_spin_split,gp_spin_split,g_opc,g_lsdau,g_stm,g_optics)
       !!
       !! do calculation
       Call iterate(sccfplo,"SCF",.True.,.True.)
       !!
       if(deb) Close(outfile)
       !!
       !Call write_in_sym_file(g_symmetry,g_structure)
       !!
       outfile=stdout


!!$       data%letsstop =g_etotstack% etotconv
       
      
       data% erg=g_etot% etot

       !! put density into vector
       Call checkinout(it_checkin,data%vecdim,data% vector,g_symmetry&
            &,g_structure,g_forces,data% name)

       Call watch(cpumesg2,cpu_stop,delete=.True.,Print=.True.&
            &,sccmesg=data%name)


       !! return and let iteration module proceed
    Elseif(Entry==it_entry_exit)Then
       !!------------------------------------------------------
       !! this is called before vector is destroyed
       !! make cleanup work
       !!------------------------------------------------------

       !! here the last dens+positions should be present

       !! get sites from vector
       !Call checkinout(it_checkout,data%vecdim,data% vector,g_symmetry&
       !     &,g_structure,g_forces,data% name)
       !!
       !Do isa=1,g_structure% nsite
       !   Write(outfile,"('tau(',i5,')=',3(e30.15e3,1x))") &
       !        &isa,g_structure% p_tau(1:3,isa)
       !Enddo
       !!
       !Call write_in_sym_file(g_symmetry,g_structure)

       !!

       Call watch(cpumesg1,cpu_stop,delete=.True.,Print=.True.&
            &,sccmesg=data%name)
       !!

       !!
       
       !! return and let iteration module cleanup
    Elseif(Entry==it_entry_best)Then
       !!------------------------------------------------------
       !! this is called if a new better iteration vector was found
       !!------------------------------------------------------
       
       Call checkinout(it_checkout,data%vecdim,data% vector,g_symmetry&
            &,g_structure,g_forces,data% name)
       !!
       Call write_in_sym_file(g_symmetry,g_structure)

       

    Else
       !!------------------------------------------------------
       !! some new action was implemented in module iteration
       !!------------------------------------------------------
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'unknown iteration entry!'")
       
    Endif
    !! Ciao
    !!
!!$2   format(15('-'),' END   ',a,'(',a,')',a,/,72('='))
!!$    If(outlevel>L_many)&
!!$         &Write(outfile,2)nsubfunc&
!!$         &,Trim(iteration_entries(Entry))&
!!$         &,Repeat('-',72_t_int-15_t_int-9_t_int-Len(nsubfunc)&
!!$         &-Len(Trim(iteration_entries(Entry))))
    !!
    Return
    !!
!111 Format( t2,'total energy')
!121 Format(a,': ',f22.10)
    !!
  End Subroutine sccforce
  !!***********************************************************************
  Subroutine checkinout(in,nvec,vit,symmetry,structure,forces,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(inout):: structure
    Type(T_Forces),Intent(in),Target  :: forces
    Integer(t_int),Intent(in)   :: in
    Integer(t_int),Intent(in)   :: nvec
    Real(t_real),Intent(inout)  :: vit(nvec)
    !!LOCAL
    Character(*),Parameter::nsubfunc="checkinout"
    Logical(t_log) :: deb
    Integer(t_int) :: n,m,issa,isa,i,jops,isga,m0
    Real(t_real)   :: force(3),xparam(3),yparam(3)
    Type(T_SymTrans),Pointer :: sp
    Type(T_SymTrans),Pointer :: st
    !External ccalccode
    !Character(5000)   :: emsg,code,str
    !Real(t_real)   :: res,
    !Integer(t_int) :: err
    !!
    st => forces% symtrans
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    !!
    checkif: If (in==it_checkout) Then
       !!
       If(deb) Write(outfile,"(a,' Checkout iteration data')")Trim(mesg)
       !!
!!$       If(deb) Then
!!$          Do isa=1,structure% nsite
!!$             Write(outfile,"(a,' tau0(',i3,')=',3(f30.15,1x))")&
!!$                  &Trim(mesg),isa,structure% p_tau(:,isa)
!!$          Enddo
!!$       Endif
       !!
       n=0
       Do issa=1,structure% nsort
          sp => forces% p_sitefreeparam(issa)
          !!
          isa=structure% p_isst(issa)
          !!
          !!
          If(.Not. sp% do_vary) Cycle
          !!   
          structure% p_tau(1:3,isa)=sp% Qtau
          !!
          If(issa==st% issashift)Then
             m=sp% nparal-st% nparal
             m0=st% nparal
             structure% p_tau(1:3,isa)=structure% p_tau(1:3,isa)&
                  &+st% Pshift(1:3)
          Else
             m=sp% nparal
             m0=0
          Endif
          xparam(1:m)=vit(n+1:n+m)
          Do i=1,m
             structure% p_tau(1:3,isa)=structure% p_tau(1:3,isa)&
                  &+sp% paral(:,i+m0)*xparam(i)
          Enddo
          !!
          !!
          n=n+m
          !!
       Enddo ! issa
       !!
       If(deb) Then
          Do isa=1,structure% nsite
             Write(outfile,"(a,' raw tau(',i3,')=',3(f30.15,1x))")&
                  &Trim(mesg),isa,structure% p_tau(:,isa)
          Enddo
       Endif
       
       !!
       !!
       !! let the user do some tricks with the positions
       !!
       !!
!!$       Do issa=1,structure% nsort
!!$          Write(str,"()") structure% p_tau(:,isa)
!!$       Enddo
!!$       !!
!!$       Call ccalccode(code,res,table,emsg,err)
!!$       !Print *,Trim(code)
!!$       Write(*,"(a)") Trim(table)
!!$       Print *,'res=',res
!!$       !!
!!$       Stop
       !!
       !! create sites from wyckoffs
       !!
       Do issa=1,structure% nsort
          !!
          isa=structure% p_isst(issa)
          Do jops=1,symmetry% Rnumops
             isga=structure% p_equivsites(jops,isa)
             If(isga /= isa)Then
                structure% p_tau(:,isga)=Matmul(structure% p_tau(:,isa)&
                     &                     ,symmetry% Ropmat(:,:,jops))&
                     &+symmetry% Roptau(:,jops)
             Endif
          Enddo
       Enddo
       !!
       If(deb) Then
          Do isa=1,structure% nsite
             Write(outfile,"(a,' copy tau(',i3,')=',3(f30.15,1x))")&
                  &Trim(mesg),isa,structure% p_tau(:,isa)
          Enddo
       Endif
       !!
       !!
    Else checkif ! checkin: vector=sites+forces
       If(deb) Write(outfile,"(a,' Checkin iteration data')")Trim(mesg)

    
       !!
       If(deb) Then
          Do isa=1,structure% nsite
             Write(outfile,"(a,' shift tau(',i3,')=',3(f30.15,1x))")&
                  &Trim(mesg),isa,structure% p_tau(:,isa)
          Enddo
       Endif


       

       n=0
       Do issa=1,structure% nsort
          !!
          sp => forces% p_sitefreeparam(issa)
          !!
          !!
          If(.Not. sp% do_vary) Cycle
          !!
          If(deb) Write(outfile,"(a,' issa=',i3)") Trim(mesg),issa
          !!
          !!
          isa=structure% p_isst(issa)
          !!
          force=0.0_t_real
          !! we check in wyckoff positions, thus we have
          !! to calculate the "wyckoff-force"
          Do jops=1,symmetry% Rnumops
             isga=structure% p_equivsites(jops,isa)
             force=force&
                  &+Matmul(symmetry% Ropmat(:,:,jops)&
                  &        ,forces% p_forces(:,isga))&
                  &*structure% p_mult(issa)/symmetry% Rnumops
          Enddo

          If(issa==st% issashift)Then
             m=sp% nparal-st% nparal
             Do i=1,m
                xparam(i)=Dot_product(structure% p_tau(1:3,isa)&
                     &               ,sp% paral(:,i+st% nparal))
                yparam(i)=Dot_product(-force(1:3)&
                     &               ,sp% paral(:,i+st% nparal))
             Enddo
             !Write(outfile,"(a,' m=',i3)") Trim(mesg),m
          Else
             m=sp% nparal
             !Write(outfile,"(a,' m=',i3)") Trim(mesg),m
             !Write(outfile,"(a,' tau       =',3(f30.15,1x))") &
             !     &Trim(mesg),structure% p_tau(:,isa)
             !Write(outfile,"(a,' tau-Pshift=',3(f30.15,1x))") &
             !     &Trim(mesg),structure% p_tau(:,isa)&
             !     &-st% Pshift(:)
             Do i=1,m
                !Write(outfile,"(a,' Paral     =',3(f30.15,1x))") &
                !     &Trim(mesg),sp% paral(:,i)
                xparam(i)=Dot_product(structure% p_tau(1:3,isa)&
                     &               ,sp% paral(:,i))
                yparam(i)=-Dot_product(force,sp% paral(:,i))
             Enddo
          Endif
       
          If(deb) Then
             Write(outfile,"(a,' xparam(',i3,')=',3(f30.15,1x))") &
                  &Trim(mesg),isa,xparam(1:m)
             Write(outfile,"(a,' yparam(',i3,')=',3(f30.15,1x))") &
                  &Trim(mesg),isa,yparam(1:m)
          Endif

          vit(n+1:n+m)=xparam(1:m)+yparam(1:m)
          n=n+m
       Enddo ! issa
       !!
       !!
    Endif checkif
    !!
    !!
  End Subroutine checkinout
  !!***********************************************************************


  

End Module forcedriver
