!!***********************************************************************
!!
!!  FULL-POTENTIAL LOCAL-ORBITAL MINIMUM BASIS BANDSTRUCTURE CODE
!!
!!                         by Klaus Koepernik
!!  part of subroutines based on the 
!!         MIX-BASIS-CODE of Arthur Ernst and Helmut Eschrig
!!
!!  full relativistic band structure part by Ingo Opahle 
!!        and Klaus Koepernik
!! 
!!  LSDA+U by Igor Chaplygin
!! 
!!thanks   authors are very grateful to , V.N. Antonov, Arnulf Moebius
!!thanks   A. Perlov, A. Yaresko P.M., Oppeneer for supplying some code
!!
!!***********************************************************************
Program fplo
  !!
  Use precisiondefs
  Use constants
  Use file_iounits
  Use errors
  Use memory
  Use versioncontrol
  Use tio
  Use ioaux, Only : setFPLOPrintFunc
  Use tmpi
  Use mpicentral
  Use optionstest
  Use ioadapter
  Use develtest
  Use dump
  Use tforces
  Use fplodata
  Use fplodriver
  Use forcedriver
  Use fplostartup
  Use iteration
  !!
  Implicit None 
  !!
  Character(*),Parameter::nmodule="PROGRAM"    
  Character(*),Parameter::nsubfunc="main"    
  Logical(t_log) :: deb,exists


  !!***********************************************************************
  !!
  Call setFPLOPrintFunc() !! initialize c++ interface
  !!
  outfile =stdout
  !!
  Include '../mainversion.h' 
  Include '../subversion.h' 
  Include '../release.h' 
  !!
  APPLICATIONNAME="fplo"
  !!
  !!
  g_debug=.True.
  !!
  Call init_mpi()
  !!
  If(g_mpi% my_rank /=0)Then
     Call wait_for_tasks()
  Endif
  !!
  Open(errfile,file=f_error,status='unknown')
  Close(errfile,status='delete')
  !!
  Call sizetest
  If( debugfun(nmodule,nsubfunc,'mem_stat') ) Then
     Call init_mem_stat(.True.)
  Else
     Call init_mem_stat(.False.)
  Endif
  !!
  !!
  !!
  Inquire(file='=.speed_of_light',exist=exists)
  If(exists)Then
     Open(12,file='=.speed_of_light')
     Read(12,*) speed_of_light
     Close(12)
     Write(outfile,"('speed of light=',f30.15)")speed_of_light
  Endif
  !!ONLY FOR TESTS: speed_of_light=1.0e6_t_real
  !Stop 'speed_of_light'
  !!
  Call init_options
  Call init_fplo_data
  Call init_dumps
  !!
  Call develtests
  !!
  !!
  Call fplo_init
  !!
  !!
  !!
  !Call opticsDos3d(g_symmetry,g_structure,g_lattice,g_bzone)
  !!

  If( g_forces% force_mode /= FORCE_MODE_GEO_OPT)Then
     !!
     Call fplo_init_fixed(g_compound,g_mspin,g_atommeshio,g_pmeshio&
          &,g_radialmesh_rmaxfactor&
          &,g_symmetry,g_structure,g_lattice,g_sic&
          &,g_numerics,g_potential,g_basis,g_kohnsham,g_bandplot,g_bzone&
          &,g_loi,g_mol,g_quant,g_wannier&
          &,g_forces,g_etotstack,g_relativistic&
          &,g_initial_spin_split,gp_spin_split,g_opc,g_lsdau,g_stm,g_optics)
     !!
!     Call lindhard_suscep_3d(g_symmetry,g_structure,g_bzone)
     !! do calculation
     Call iterate(sccfplo,"SCF",.False.,.True.)
  Else
     deb=debugfun('FORCEDRIVER','sccforce','separate_outfile')
     If(deb) Open(9,file='outplot')
     Call iterate(sccforce,"FORCES",.False.,.True.)
     If(deb) Close(9)
     Call clean_forces(g_forces)
  Endif
  !!
  Call cfinalcleanup
  !!
  Call terminate(TERM_FINISH,'SCF calculation')  
  !!
  
End Program fplo
!!***********************************************************************
