Module fplodata
  !!
  Use precisiondefs
  Use constants
  Use tsymmetry
  Use torbitalbasis
  Use tmesh
  Use tnumerics
  Use tdiracdefs
  Use diracdefs
  Use tpotentialdensity
  Use titeration
  Use tcpa
  Use tkohnsham
  Use tcoreoccupation
  Use tbzone
  Use tbandplot
  Use tlsdau
  Use optionstest
  Use integrate1d
  Use tmol
  Use ttopoins
  Use toptics
  Use tstm
  Use tunfold
  Use tforces
  Use tgrid
  Use twannier
  Use tfatband
  Use terg
  Use tloi
  Use topc
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="FPLODATA"  
  !!
  Public :: init_fplo_data,g_symmetry,g_structure,g_compound &
       &,g_mspin,g_initial_spin_split &
       &,g_relativistic ,g_numerics  &
       &,g_cpacontrol,g_bzone &
       &,g_basis,gp_spin_split,g_bandplot,g_lattice,g_potential &
       &,g_iteration_control,g_etotstack,g_BEB &
       &,g_kohnsham,g_trace,g_sic,g_unfold&
       &,g_coreoccu,g_lsdau,g_Ufactor,g_OPCfactor&
       &,g_quant,g_diracdefs,g_ldos,g_opc,g_mol,g_etot,g_loi,g_wannier&
       &,g_gridplot,g_pmeshio,g_atommeshio,g_forces&
       &,g_radialmesh_rmaxfactor,g_symtested,g_bwdefs,g_stm&
       &,g_optics,g_topoins
  !!
  !!***********************************************************************
  !!***********************************************************************
  !! symmetry information
  !! a title for the calculation
  Character(200)    :: g_compound
  !! symmetry information
  Type(T_Symmetry)  :: g_symmetry
  !! structure information 
  Type(T_Structure) :: g_structure
  Type(T_Coreoccu) :: g_coreoccu
  !! potential and density information 
  Type(T_Potential) :: g_potential
  !! lattice vectors, shlees ...
  Type(T_lattice) :: g_lattice
  Type(T_Basis) :: g_basis
  Type(T_SIC) :: g_sic
  Type(T_Kohnsham) :: g_kohnsham
  Type(T_Trace) :: g_trace
  Type(T_Unfold) :: g_unfold
  !!
  Type(T_Mol)       :: g_mol
  !! local dos control
  Type(T_LDOS)       :: g_ldos
  !! CPA ??
  Logical(t_log)     :: g_BEB
  !! number of spins
  Integer(t_int)     :: g_mspin
  Real(t_real),Pointer :: gp_spin_split(:)!!IOnsort
  !! make initial spinsplit ?
  Logical(t_log)     :: g_initial_spin_split
  !! type of relativistic calculation
  Integer(t_int)     :: g_relativistic
  !!
  Type(T_Mesh)       :: g_atommeshio,g_pmeshio
  !!
  Type(T_Etotstack) :: g_etotstack
  Type(T_Etot)      :: g_etot
  Type(T_Forces)    :: g_forces
  !! settings for various numerical procedures
  Type(T_Numerics)  :: g_numerics
  !! CPA 
  Type(T_Cpacontrol) :: g_cpacontrol
  !! Brouillin zone integration
  Type(T_Bzone)      :: g_bzone
  !! band plot controll
  Type(T_Bandplot)   :: g_bandplot
  !!
  Type(T_iteration)  :: g_iteration_control
  !!
  !!
  Type(T_Lsdau)      :: g_lsdau
  !!
  Type(T_Quant)      :: g_quant
  !!
  Type(T_Diracdefs)  :: g_diracdefs
  !!
  Type(T_OPC)        :: g_opc
  !!
  Type(T_LOI)        :: g_loi
  !!
  Type(T_Wannier)    :: g_wannier
  !!
  Type(T_BWdefs)     :: g_bwdefs
  !!
  Type(T_GridPlot)   :: g_gridplot
  !!
  !! scaling factor dens/U-densmatrix in iteration vector
  Real(t_real)       :: g_Ufactor
  !! scaling factor dens/OPC-parameter in iteration vector
  Real(t_real)       :: g_OPCfactor
  !!
  Real(t_real)       :: g_radialmesh_rmaxfactor
  !! did we do the symm-test
  Logical(t_log) :: g_symtested
  !!
  Type(T_STM) :: g_stm  
  !!
  Type(T_Optics) :: g_optics  
  !!
  Type(T_TopoIns) :: g_topoins  
  !!
  !!***********************************************************************
Contains
  Subroutine init_fplo_Data
    implicit none 
    !!IO
    !!LOCAL
    !!
    Call init_diracdef((ORBITAL_MAX_L+1)*2)
    Call init_diracdefs(g_diracdefs)
    Call init_quant(g_quant)
    Call init_gmesh()
    Call init_unfold(g_unfold)
    Call init_symmetry(g_symmetry)
    Call init_structure(g_structure)
    Call init_lattice(g_lattice)
    Call init_bandplot(g_bandplot)
    Call init_bzone(g_bzone)
    Call init_trace(g_trace)
    Call init_cpacontrol(g_cpacontrol)
    Call init_numerics(g_numerics)
    Call init_etotstack(g_etotstack)
    Call init_etot(g_etot)
    Call init_basis(g_basis)
    Call init_potential(g_potential)
    Call init_iterationcontrol(g_iteration_control)
    Call init_kohnsham(g_kohnsham)
    Call init_sic(g_sic)
    Call init_coreoccu(g_coreoccu)
    Call init_lsdau(g_lsdau)
    Call init_ldos(g_ldos)
    Call init_opc(g_opc)
    Call init_mol(g_mol)
    Call init_loi(g_loi)
    Call init_wannier(g_wannier)
    Call init_bwdefs(g_bwdefs)
    Call init_stm(g_stm)
    Call init_optics(g_optics)
    Call init_topoins(g_topoins)
    !!
    Call init_mesh(g_atommeshio)
    Call init_mesh(g_pmeshio)
    Call init_forces(g_forces)
    !!
    Nullify(gp_spin_split)
    !!
    Call init_gridplot(g_gridplot)
    !!
    g_OPCfactor=1.0_t_real
    g_radialmesh_rmaxfactor=1.0_t_real
    !!
    g_symtested=.False.
    !!
    !!
    Return
    !!
  End Subroutine init_fplo_data
  
End Module fplodata
