!!----DEBUG options-----------------|
!! SUBFUNC    | MODES               |
!!----------------------------------|
!!fplo_step   | load_vat_v0         |
!!----------------------------------|


Module fplostep
  !!
  Use precisiondefs
  Use constants
  Use file_iounits
  Use memory
  Use errors
  Use io
  Use prnt
  Use clock
  Use tsymmetry
  Use tpotentialdensity
  Use tmesh
  Use tlsdau
  Use toptics
  Use mpicentral
  Use tstm
  Use stm
  Use tunfold
  Use tfatband
  Use efg
  Use bhf
  Use tforces
  Use tgrid
!!  Use cpa
  Use tmol
  Use mol
  Use molpot
  Use molcore
  Use plotdens
  Use loi
  Use occu
  Use ttopoins
  Use topoins
  Use erg
  Use vlm
  Use optionstest
  Use tnumerics
  Use terg
  Use tdiracdefs
  Use torbitalbasis
  Use orbitalbasis
  Use tkohnsham
  Use kohnsham
  Use tbandplot
  Use trelativistic
  Use tbzone
  Use density
  Use tcpa
  Use tloi
  Use topc
  Use opcorrection
  Use loiorg
  Use twannier
  Use loif  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="FPLOSTEP"  
  !!
  Public :: fplo_step
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine fplo_step(symmetry,structure,lattice,potential,mspin&
       &,numerics,titerstep,basis&
       !&,sic&
       &,relativistic,quant&
       &,kohnsham,bzone,bandplot,ldos,BEB&
       !&,cpacontrol,trace&
       &,estack,etot,forces,lsdau,opc&
       &,mol,loi,wannier,unfold,bwdefs,gridplot,deviation,restart,force_loop&
       &,stm,optics,topoins)
    !!
    Implicit None 
    !!IO
    Logical(t_log),Intent(in)               :: BEB,force_loop
    Type(T_Symmetry),Intent(in)      :: symmetry
    Type(T_Structure), Intent(in)    :: structure
    Type(T_Lattice),Intent(in)       :: lattice
    Type(T_Unfold),Intent(in)        :: unfold
    Type(T_BWDefs),Intent(in)        :: bwdefs
    Type(T_Numerics), Intent(in)     :: numerics
    Type(T_Potential), Intent(inout) :: potential
    Type(T_STM), Intent(inout)       :: stm
    Type(T_Optics), Intent(inout)    :: optics
    Type(T_TopoIns), Intent(inout)   :: topoins
    Type(T_Basis),Intent(inout)      :: basis
    !Type(T_SIC),Intent(inout)        :: sic
    Type(T_Kohnsham),Intent(inout)   :: kohnsham
    Type(T_Bandplot),Intent(inout)   :: bandplot
    Type(T_LDOS),Intent(inout)       :: ldos
    Type(T_Bzone),Intent(inout)      :: bzone
    Type(T_Quant),Intent(inout)      :: quant
    Type(T_Lsdau),Intent(inout)      :: lsdau
    Type(T_Etotstack),Intent(inout)  :: estack
    Type(T_Etot),Intent(inout)       :: etot
    Type(T_Forces),Intent(inout)     :: forces
    !Type(T_Cpacontrol),Intent(inout) :: cpacontrol
    !Type(T_Trace),Intent(in)         :: trace
    Type(T_OPC),Intent(inout)        :: opc
    Type(T_Mol),Intent(inout)        :: mol
    Type(T_Wannier),Intent(inout)    :: wannier
    Type(T_GridPlot),Intent(inout)   :: gridplot
    Type(T_Loi),Intent(inout)        :: loi
    Integer(t_int),Intent(in)        :: mspin
    Integer(t_int),Intent(in)        :: relativistic
    Integer(t_int),Intent(inout)     :: titerstep
    Real(t_real),Intent(in)          :: deviation
    Integer(t_int),Intent(inout)     :: restart
    !!LOCAL
    Character(*),Parameter::nsubfunc="fplo_step"
    Character(*),Parameter  :: cpumesg_pot='make potential'
    Character(*),Parameter  :: cpumesg_step='fplo step'
    Character(*),Parameter  :: cpumesg_band='Kohn-Sham equation'
    Character(*),Parameter  :: cpumesg_lo='LO integrals'
    Character(*),Parameter  :: cpumesg_lof='LOF integrals'
    Character(*),Parameter  :: cpumesg_dens='Density calculation'
    !!
    !!
    Integer(t_int) :: nspin
    !!
    Logical(t_log) :: rel,prot
    !!
    !!
    Call mem_status()
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    nspin=mspin
    If(rel)Then
       nspin=1_t_int
    Endif
    !!
    !!
    Call watch(cpumesg_step,cpu_start,reset=.True.)            
    !!
    Call calc_Z2_invariant(mspin,symmetry,structure,lattice,basis,loi&
         &,kohnsham,lsdau,opc,topoins,relativistic,atstart=.True.)
    !!
    !! write density information
    If(isoption('PLOT_REALFUNC'))Then
       Call printfunc(potential% p_pmeshs,potential% nrp,potential % p_lcd,mspin&
            &,structure % nsite,structure % p_isrt&
            &,structure % p_elements,potential % mlmp&
            &,potential % p_nlmp,potential % p_lmp,'+dens',PRINTFUNC_DENS)

    End If

    !! charge analysis of start density
    Call density_analysis(structure,mspin,potential,numerics,titerstep&
         &,relativistic)

    
    Call calc_racah_params(opc,structure% p_elements,basis,relativistic,quant&
         &,potential% make_vca,potential% p_natom)

    !! make potential


    Call watch(cpumesg_pot,cpu_start,reset=.True.)            

    Call mol_makepot(symmetry,structure,lattice,mspin&
         &,basis% p_fgam,potential,etot,forces,numerics,mol,gridplot&
         &,BEB,force_loop,relativistic)

    !!
    !!
    !Stop
    !!
    !!
    Call test_mpi()

    !!
    Call watch(cpumesg_pot,cpu_stop,delete=.True.,Print=.True.)         
    !! END of making potential
    !!-------------------------------
    !!


    !!----- LO block -----------------------------------
    Call watch(cpumesg_lo,cpu_start,reset=.True.)
    !!
    !!
    Call alloc_loi(loi,mspin,relativistic,structure% nsite&
         &,kohnsham% nchismx,kohnsham% ncsmx,kohnsham% nvsmx&
         &,kohnsham% nchidim,need_sigma=bandplot% plotkspintextures)
    !!
    !!
    Call loi_int(symmetry,structure,mspin,lattice&
         &,kohnsham% ncsmx,kohnsham% nvsmx&
         !&,kohnsham% p_cblocklength,kohnsham% p_vblocklength&
         &,basis,mol,numerics,loi,relativistic)

    Call watch(cpumesg_lo,cpu_stop,delete=.True.,Print=.True.)

    If(force_loop.Or.optics% active)Then
       Call watch(cpumesg_lof,cpu_start,reset=.True.)

       Call loif_int(mspin,symmetry,structure,lattice,basis&
            &,mol,loi,numerics,kohnsham% ncsmx,kohnsham% nvsmx&
            &,kohnsham% p_cblocklength,kohnsham% p_vblocklength&
            &,calc_THCI=force_loop)
       Call watch(cpumesg_lof,cpu_stop,delete=.True.,Print=.True.)
    Endif
    !!
    !!
    !!
    !!
    !!----- START bandstructure/diagonalization block --------------------
    Call watch(cpumesg_band,cpu_start,reset=.True.)

    !!
    prot=(outlevel >= L_many)
    Call mol_core(mspin,structure,basis,kohnsham,loi,rel,etot,forces&
         &,prot,force_loop)
    !!
    !Print *,'after mol_core'
    !!
    Call mol_core_trans(mspin,structure,lattice,loi,kohnsham,rel&
         &,need_sigma=bandplot% plotkspintextures&
         !&,quant=quant&
         &)
    !!
    CPAIF: If(BEB) Then

       Call terminate(TERM_CRASH,Trim(nsubfunc)//&
            &': CPA and new numerics not yet implemented!')

!!$       Call cpa_driver(symmetry,structure,lattice,mspin,potential% p_natom&
!!$            &,efermi,etot% bandenergy,basis&
!!$            &,bzone% nkp,bzone% p_bk,bzone% p_wbk&
!!$            &,cpacontrol&
!!$            &,trace,kohnsham&
!!$            &,bandplot&
!!$!!TODO: nspin,rel
!!$            &,loi,lsdau)
    Else

       !!
       Call calc_KS_states(symmetry,structure,lattice,mspin,nspin,kohnsham&
            &,potential% p_natom,etot% bandenergy,etot% Eent,basis&
            &,bzone,bandplot&
            &,ldos,loi,lsdau,relativistic,opc&
            &,forces,deviation,restart,force_loop,wannier,bwdefs% active,optics)!,quant)

       Call calc_Z2_invariant(mspin,symmetry,structure,lattice,basis,loi&
            &,kohnsham,lsdau,opc,topoins,relativistic,atstart=.False.)

       If(bandplot% plotbands.Or.wannier% do_wanniers) Then
          Call create_bandstructure(mspin,structure,lattice,nspin,basis&
               &,kohnsham,bandplot,bzone,symmetry% lattice_constants&
               &,loi,lsdau,relativistic,opc,wannier,unfold,bwdefs,stm,optics&
               &,quant)
       Endif
    End If CPAIF
    !! 
    Call watch(cpumesg_band,cpu_stop,delete=.True.,Print=.True.)

    If(kohnsham% DOS_savememory_onstep)Then
       Write(errbuf,110) 
       Call errmsg(nmodule,nsubfunc,E_information,msg=errbuf)
110    Format("'debug option DOS_savememory_onstep was set in ''=.debug''',/&
            &,' must stop here (have no wave function coefficients)!'") 
       Call terminate(TERM_NORMAL,'debug option DOS_savememory_onstep')
    Endif
    !!
    !!
    Call stm_calc_dos(mspin,nspin,symmetry,structure,stm,kohnsham,basis,bzone)
    !!
    Call stm_make_map(mspin,nspin,symmetry,structure,potential&
         &,basis,mol,kohnsham,numerics,stm,bzone)
    !!
    Call plot_dens_resolved(symmetry,structure,lattice,potential,basis&
         &,kohnsham,loi,lsdau,opc,bzone,numerics% degree,mspin&
         &,relativistic,quant&
         &,gridplot)


    !!----- END bandstructure/diagonalization block --------------------
    !!
    !!
    Call make_netoccus(mspin,symmetry,structure,kohnsham,kohnsham% p_bweitk&
         &,kohnsham% p_coccu,bzone% nkp,basis&
         &,relativistic,kohnsham% p_noncc,kohnsham% p_noncv,kohnsham% p_nonvv&
         &,kohnsham% p_roncc,kohnsham% p_roncv,kohnsham% p_ronvv,.False.)
    !!
    !! hyper fine fields
    !!
    !!
    Call calc_hyperfineFields(mspin,nspin,symmetry,structure&
         &,basis,potential,bzone% nkp,kohnsham&
         &,kohnsham% nband,kohnsham% ncdim,kohnsham% nvdim&
         &,kohnsham% p_ccck,kohnsham% p_ccvk&
         &,numerics,relativistic)
    !!
    Call efg_contributions(mspin,structure,kohnsham,basis,numerics&
         &,relativistic)
    !!
    !! END of live time of LO-integrals
    !!
    Call cleanup_loi(loi)
    !!
    !!
    !! density calculation
    Call watch(cpumesg_dens,cpu_start,reset=.True.)            
    !! overwrite old density with new  density=potential% p_lcd

    Call calc_density(mspin,symmetry,structure,lattice,potential,mol&
         &,forces,structure% nsite,bzone&
         &,kohnsham% ncsmx,kohnsham% nvsmx,kohnsham,basis&
         &,relativistic,numerics,quant,force_loop)
    !!
    Call density_analysis(structure,mspin,potential,numerics,titerstep&
         &,relativistic)
    !!
    !!
    !! write density information
    If(isoption('PLOT_REALFUNC'))Then
       Call printfunc(potential% p_pmeshs,potential% nrp,potential% p_lcd,mspin&
            &,structure% nsite,structure% p_isrt&
            &,structure% p_elements,potential% mlmp,potential% p_nlmp&
            &,potential% p_lmp,'+dens',PRINTFUNC_DENS)
    End If
    !! TODO: clearify
    !! write density of this SCC step
    !! after convergency the density which entered vector is written
    !! thus the last dens is discarded if convergency is reached
    !! call denswrite('dens')
    !!
    !!
    !!
    Call watch(cpumesg_dens,cpu_stop,delete=.True.,Print=.True.)
    !!
    !!
    Call erg_etot(structure,lsdau,etot,opc)
    !!
    Call erg_org(etot% etot,estack)
    !!
    Call erg_forces(symmetry,structure,forces,force_loop)
    !!
    Call watch(cpumesg_step,cpu_stop,delete=.True.,Print=.True.)         
    !!
  End Subroutine fplo_step
  !!***********************************************************************
End Module fplostep
