Module grid
  !!
  Use precisiondefs
  Use constants
  Use memory
  Use file_iounits
  Use errors
  Use tgrid
  Use tsymmetry
  Use matserv
  Use ioaux
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="GRID"  
  !!
  Public :: check_grid_type,valid_grid_type&
       &,valid_grid_basis,check_grid_basis,make_grid&
       &,print_grid_props,is_in_box
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine make_grid(gr,symmetry,structure,xrunsfirst)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Grid),Intent(inout)   :: gr
    Logical(t_log),Intent(in),Optional :: xrunsfirst
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_grid"
    Integer(t_int) :: i,i1,i2,i3,n
    Real(t_real)   :: basis(3,3),tmp(3,3),tmpi(3,3),det,x(3),pcell(3,3)
    Logical(t_log) :: lxrf,exist
    Integer        :: eof
    Character(500):: buffer,buf,word
    !!
    lxrf=.False.
    If(Present(xrunsfirst))Then
       lxrf=xrunsfirst
    Endif
    !!
    If( .Not. valid_grid_type(gr) ) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="' Gridtype not implemented?'")
    Endif
    !!
    If(gr% latticetype == GRID_DIRECT_LATTICE) Then
       basis=symmetry% lattice_vectors
    Else
       basis=symmetry% reciprocial_lattice_vectors
    Endif
    !!
    If(gr% basis == GRID_BASIS_CARTESIAN)Then
       basis(:,1)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
       basis(:,2)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
       basis(:,3)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
    Elseif(gr% basis == GRID_BASIS_CONVENTIONAL)Then
       !!
       If(gr% latticetype == GRID_DIRECT_LATTICE) Then
          tmp(:,:)=Real(symmetry% rcentering(:,:),t_real)/12.0_t_real
       Else
          tmp(:,:)=Real(symmetry% gcentering(:,:),t_real)/12.0_t_real
          If(symmetry% HEX .And. .Not. symmetry% RTG)Then
             tmp(:,1)=(/1.0_t_real,1.0_t_real,0.0_t_real/)
             tmp(:,2)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
             tmp(:,3)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
          Endif
       Endif
       !!
       Call dinv33(tmp,1_t_int,tmpi,det)
       basis=Matmul(basis,tmpi)
    Endif
    !!
    If(structure% str_type==STR_TYPE_MOLECULE)Then
       basis(:,1)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
       basis(:,2)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
       basis(:,3)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
    Endif
    !!
    Write(outfile,"('GRID CREATION:')")
    Write(outfile,"('lattice basis according to setting:')")
    Do i=1,3
       Write(outfile,44) basis(:,i)
    Enddo
    !!
    !!
    Write(outfile,"('resulting lattice directions:')")
    pcell=Matmul(basis,gr% directions)
    !!
    Do i=1,3
       Write(outfile,44) pcell(:,i)
    Enddo
44  Format(3(f25.15,1x))
    !!
    If(gr% gridtype==GRID_TYPE_REGULAR) Then
       !!
       gr% n123(1:3)=gr% subdivisions(1:3)
       !!
       If(gr% InclPeriodicPoints)Then
          gr% n123(1:3)=gr% n123(1:3)+1
       Endif
       !!
       !!
       gr% npoints=Product(gr% n123(1:3))
       !!
       Call palloc('gr% points',gr% points,1_t_int,3_t_int,1_t_int&
            &,gr% npoints,nmodule,nsubfunc)
       !!
       gr% o=Matmul(pcell(1:3,1:3),gr% origin(1:3))
       !!
       Write(outfile,"('resulting origin: ',3(f19.13,1x))")gr% o
       !!
       gr% delta(1:3,1)=pcell(1:3,1)/Real(Max(gr% subdivisions(1),1_t_int),t_real)
       gr% delta(1:3,2)=pcell(1:3,2)/Real(Max(gr% subdivisions(2),1_t_int),t_real)
       gr% delta(1:3,3)=pcell(1:3,3)/Real(Max(gr% subdivisions(3),1_t_int),t_real)
       !!
       Call dinv33(pcell,0_t_int,tmpi,det)
       Do i=1,3
          gr% ibox(i,1:3)=tmpi(i,1:3)
       Enddo
       !!
       n=0
       If(lxrf)Then
          Do i1=0,gr% n123(1)-1
             x(1)=Real(i1,t_real)
             Do i2=0,gr% n123(2)-1
                x(2)=Real(i2,t_real)
                Do i3=0,gr% n123(3)-1
                   n=n+1
                   x(3)=Real(i3,t_real)
                   
                   gr% points(1:3,n)=Matmul(gr% delta(1:3,:),x(1:3))+gr% o
                   ! Print *,gr% points(1:3,n)!/twopi*symmetry% lattice_constants(1)
                   
                Enddo
             Enddo
          Enddo
       Else
          
          Do i3=0,gr% n123(3)-1
             x(3)=Real(i3,t_real)
             Do i2=0,gr% n123(2)-1
                x(2)=Real(i2,t_real)
                Do i1=0,gr% n123(1)-1
                   n=n+1
                   x(1)=Real(i1,t_real)
                   
                   
                   gr% points(1:3,n)=Matmul(gr% delta(1:3,:),x(1:3))+gr% o
                   ! Print *,gr% points(1:3,n)!/twopi*symmetry% lattice_constants(1)
                   
                Enddo
             Enddo
          Enddo
       Endif
       !!
       Write(outfile,"('number of grid points=',i8)") gr% npoints
    Else If(gr% gridtype==GRID_TYPE_CUSTOM) Then
       !! we have to read the points from file
       
       !!
       !!
       !!
       Write(outfile,"('Reading grid data from file ''',a,'''')")&
            &Trim(gr% pointfile)
       !!
       !!
       !!
       Inquire(file=gr% pointfile,exist=exist)
       If(.Not.exist)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="' The user specified grid data-point file ''"&
               &//Trim(gr% pointfile)//"'' does not exist!'")
  
       Endif

       Open(12,file=gr% pointfile)
       n=0
       Do 
          Read(12,1,iostat=eof) buffer
          If(eof < 0_t_int) Exit 
          buf=Adjustl(buffer)
          If(buf(1:1) /= '#' .And. Len_trim(buf)/=0) n=n+1
       Enddo 
       Rewind(12)
       !Print *,'number of points ',3

       gr% npoints=n
       !!
       Call palloc('gr% points',gr% points,1_t_int,3_t_int,1_t_int&
            &,gr% npoints,nmodule,nsubfunc)
       !!
       n=0
       i=0
       readloop: Do 
          Read(12,1,iostat=eof) buffer
          If(eof < 0_t_int) Exit readloop
          buf=Adjustl(buffer)
          n=n+1
          If(buf(1:1) /= '#' .And. Len_trim(buf)/=0) Then
             
             If(.Not.next_word(buf,word)) Goto 666
             Read(word,fmt=*,iostat=eof) x(1)
             If(eof/=0) Goto 666

             If(.Not.next_word(buf,word)) Goto 666
             Read(word,fmt=*,iostat=eof) x(2)
             If(eof/=0) Goto 666

             If(.Not.next_word(buf,word)) Goto 666
             Read(word,fmt=*,iostat=eof) x(3)
             If(eof/=0) Goto 666

             i=i+1
             gr% points(1:3,i)=Matmul(pcell(1:3,:),x(1:3))

!             Print *,i,gr% points(1:3,i)
          Endif
       Enddo readloop
       !!
       Close(12)
       !!
       Write(outfile,"('number of grid points=',i8)") gr% npoints
       !!
    Endif
    !!
!!$    Do i=1,gr% npoints
!!$       Print *,gr% points(1:3,i)
!!$    Enddo
    !!
    Return
    !!
1   Format(a)
    !!
666 Continue
    !!
    !!
    Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="' File: ''"//Trim(gr% pointfile)//"'' line  "//&
                  &Trim(intToChar10(n))//":  3 real numbers expected!'")
    !!
  End Subroutine make_grid
  !!***********************************************************************
  Logical(t_log) Function is_in_box(gr,r)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    Real(t_real),Intent(in)      :: r(3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="is_in_box"
    Real(t_real)   :: x(3)
    !!
    If (gr% gridtype==GRID_TYPE_CUSTOM)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="' wrong grid type?'")

    is_in_box=.False.
    !!
    x=Matmul(gr% ibox(:,:),r-gr% o)
    !!
    If(&
         & (0.0_t_real <= x(1)  .And. x(1) <= 1.0_t_real).And.&
         & (0.0_t_real <= x(2)  .And. x(2) <= 1.0_t_real).And.&
         & (0.0_t_real <= x(3)  .And. x(3) <= 1.0_t_real) ) Then
       !!
       is_in_box=.True.
       !!
    Endif
    !!
    Return
    !!
  End Function is_in_box
  !!***********************************************************************
  Logical(t_log) Function valid_grid_basis(gr)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="dum"
    !!
    valid_grid_basis=.True.
    !!
    If (   ( gr% basis < GRID_BASIS_PRIMITIVE ) .Or. &
         & ( gr% basis > GRID_BASIS_CARTESIAN )    ) Then
       valid_grid_basis=.False.
    Endif
    !!
    Return
    !!
  End Function valid_grid_basis
  !!***********************************************************************
  Subroutine check_grid_basis(gr,caller)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: caller
    Type(T_Grid),Intent(in)      :: gr
    !!LOCAL
    Character(*),Parameter::nsubfunc="dum"
    !!
    !!
    If (   ( gr% basis < GRID_BASIS_PRIMITIVE ) .Or. &
         & ( gr% basis > GRID_BASIS_CARTESIAN )    ) Then
1      Format("'Wrong basis type: ",i4," in subroutine/function ",a," !'")
       Write(errbuf,1) gr% basis,Trim(caller)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Return
    !!
  End Subroutine check_grid_basis
  !!***********************************************************************
  Logical(t_log) Function valid_grid_type(gr)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="dum"
    !!
    valid_grid_type=.True.
    !!
    If (   ( gr% gridtype < GRID_TYPE_REGULAR ) .Or. &
         & ( gr% gridtype > GRID_TYPE_CUSTOM )    ) Then
       valid_grid_type=.False.
    Endif
    !!
    Return
    !!
  End Function valid_grid_type
  !!***********************************************************************
  Subroutine check_grid_type(gr,caller)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: caller
    Type(T_Grid),Intent(in)      :: gr
    !!LOCAL
    Character(*),Parameter::nsubfunc="dum"
    !!
    !!
    If (   ( gr% gridtype < GRID_TYPE_REGULAR ) .Or. &
         & ( gr% gridtype > GRID_TYPE_CUSTOM )    ) Then
1      Format("'Wrong grid type: ",i4," in subroutine/function ",a," !'")
       Write(errbuf,1) gr% gridtype,Trim(caller)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Return
    !!
  End Subroutine check_grid_type
  !!***********************************************************************
  Subroutine print_grid_props(lun,gr)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    Integer(t_int),Intent(in)    :: lun
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_grid_props"
    Integer(t_int) :: i,j
    !!
    !!
    Write(lun,1)
    Write(lun,10) Trim(GRID_TYPES(gr% gridtype))
    Write(lun,11) Trim(GRID_LATTICE_TYPES(gr% latticetype))
    Write(lun,12) Trim(GRID_BASIS_TYPES(gr% basis))
    !!
    Write(lun,13) ((Trim(stringify_double100(gr% directions(j,i))),j=1,3),i=1,3)
    Write(lun,14) (Trim(intToChar20(gr% subdivisions(i))),i=1,3)
    Write(lun,16) (Trim(stringify_double100(gr% origin(i))),i=1,3)
    If(gr% InclPeriodicPoints)Then
       Write(lun,15) 'yes'
    Else
       Write(lun,15) 'no'
    Endif
    !!
    Write(lun,2)
    !!
    Return
    !!
1   Format(/,'------------- Grid definition ------------ ')
10  Format('Grid type              : ',a)
11  Format('Grid lattice       type: ',a)
12  Format('Grid lattice basis type: ',a)
13  Format('Grid directions in terms of the basis: ',/&
         &,t10,'d1=',3(a,1x),/&
         &,t10,'d2=',3(a,1x),/&
         &,t10,'d3=',3(a,1x),/)
14  Format('Grid subdivisions : Nxyz=',3(a,1x),/)
15  Format('Grid includes periodic boundary points: ',a,/)
16  Format('Grid origin in relative coords : o=',3(a,1x),/)

2   Format('------------------------------------------ ',/)
    !!
  End Subroutine print_grid_props
  !!***********************************************************************
End Module grid
