Module integrate1d
  !!
  Use precisiondefs 
  Use constants 
  Use errors
  Use file_iounits
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Public :: lagint_out,lagint_in,lag4int,lag2int,simpson,tlag4int&
       &,tjlag4int,simpintleft,get_gauss,init_gmesh&
       &,IJAC_TYPE_LIN,IJAC_TYPE_LOG,IJAC_TYPE_POW,IJAC_TYPE_SIN&
       &,quad_gkp,QUADSTART,clean_quadgkp,T_Quadgkp,init_quadgkp&
       &,alloc_quadgkp&
       &,clag4int,cjlag4int
  !!***********************************************************************
  Character(*),Parameter::nmodule="INTEGRATE1D"  
  !!***********************************************************************
  Type T_Quadgkp
     Integer(t_int) :: ndim
     Logical(t_log) :: conv
     Integer(t_int) :: typ,j1,j2,j,jl,jh,ip,k,state,kmin
     Real(t_real)   :: epsabs,epsrel,b0,b1,exx
     Real(t_real)   :: hx,xm,rx
     Real(t_real),Pointer :: p_current(:) ! ndim
     Real(t_real),Pointer :: p_old(:) ! ndim
     Real(t_real),Pointer :: p_fval(:) ! ndim
     Real(t_real),Pointer :: p_ff(:) ! ndim
     Real(t_real),Pointer :: p_res(:) ! ndim
     Real(t_real),Pointer :: p_work(:,:) ! 17,ndim
  End Type T_Quadgkp
  !!***********************************************************************
  !! organization of gauss meshs
  Type T_Gauss
     Integer(t_int) :: ng
     Real(t_real),Pointer   :: p_w(:)
     Real(t_real),Pointer   :: p_a(:)
     Type (T_Gauss),Pointer :: next
  End Type T_Gauss
  !!
  Type(T_Gauss),Target      :: gfirst
  !!***********************************************************************
  !!
  Integer(t_int),Parameter :: lagint_out=0_t_int
  Integer(t_int),Parameter :: lagint_in =1_t_int
  
  ! data
  Real(t_real),Private :: cof2(2_t_int,3_t_int)
  Real(t_real),Private :: cof4(4_t_int,5_t_int)
  Data cof2/&
       &  0.41666666666666666666666666666666666666666_t_real&
       &, 0.33333333333333333333333333333333333333333_t_real&
       &, 0.66666666666666666666666666666666666666666_t_real&
       &, 1.33333333333333333333333333333333333333333_t_real&
       &,-0.08333333333333333333333333333333333333333_t_real&
       &, 0.33333333333333333333333333333333333333333_t_real&
       &/
  Data cof4/&
       &  0.348611111111111111111111111111_t_real&
       &, 0.322222222222222222222222222222_t_real&
       &, 0.3375_t_real&
       &, 0.311111111111111111111111111111_t_real&
       &, 0.897222222222222222222222222222_t_real&
       &, 1.377777777777777777777777777777_t_real&
       &, 1.275_t_real&
       &, 1.422222222222222222222222222222_t_real&
       &,-0.366666666666666666666666666666_t_real&
       &, 0.266666666666666666666666666666_t_real&
       &, 0.9_t_real&
       &, 0.533333333333333333333333333333_t_real&
       &, 0.147222222222222222222222222222_t_real&
       &, 0.0444444444444444444444444444444_t_real&
       &, 0.525_t_real&
       &, 1.422222222222222222222222222222_t_real&
       &,-0.0263888888888888888888888888888_t_real&
       &,-0.0111111111111111111111111111111_t_real&
       &,-0.0375_t_real&
       &, 0.311111111111111111111111111111_t_real&
       &/
  !!***********************************************************************
  !!START DATA quad_gkp:
  !!     algorithm 699 , collected algorithms from acm.
  !!     this work published in transactions on mathematical software,
  !!     vol. 17, no. 4, december, 1991, pp. 457-461.
  !!     this quadrature program uses formulae due to t. n. l. patterson,
  !!     mathematics of computation, volume 22, 1968, pages 847-856, as
  !!     modified by f. t. krogh and w. v. snyder, acm transactions on
  !!     mathematical software 17, 4 (december 1991) pp 457-461.  it is a
  !!     functional replacement for algorithm 468, t. n. l. patterson,
  !!     communications of the acm 16, 11 (november 1973) 694-699.
  !!***********************************************************************
  !!     *****     data statements     ************************************
  !!
  Integer(t_int) ::  fh_d(2:8), fk_d(2:8)
  Integer(t_int) ::  kh_d(11), kk, kl_d(11), kx_d(1:8)
  Real(t_real) ::  p_d(305)
  !!
  Data fk_d /2, 3, 5, 9,12,14, 1/
  Data fh_d /2, 4, 8,16,17,17, 0/
  Data kl_d    /1, 1, 1, 1, 1, 3, 5, 9, 5, 9,12/
  Data kh_d    /1, 2, 4, 8,16, 3, 6,17, 5, 9,17/
  Data kx_d /0, 1, 2, 3, 4, 5,       8,      11/
  !!
  !!     in the comments below, f(k,i) refers to the function value
  !!     computed for the i'th node of the k'th formula.  the abscissae and
  !!     weights are stored in order according to the distance from the
  !!     boundary of the region, not from the center.  since we store
  !!     1 - |abscissa|, the first "node" coefficient for each formula is
  !!     the smallest.
  !!
  !! fh_d, fk_d  contain subscripts to indicate where to store integrand
  !!         samples in work.
  !!
  !! kh_d, kl_d  are bounds for indexes at which integrand samples are
  !!         retrieved from work in order to begin applying a quadrature
  !!         formula.
  !! kx_d      is a list of bounds of subscripts into kh_d and kl_d.  kh_d is
  !!         indexed by k.  the set of indexes from which to retrieve
  !!         integrand samples is given by the set of bounds in kl_d and kh_d
  !!         indexed by kx_d(k-1)+1 to kx_d(k) inclusively.
  !! p_d     contains the coefficients necessary to the quadrature
  !!         formulae.  their organization is described below in the
  !!         section on data statements.

  !!     corrections, nodes and weights for the 3-point formula.
  !!
  !!     correction for f(1,1).
  Data p_d(  1)/-.11111111111111111111e+00_t_real/
  !!     node and weight for f(2,1).
  Data p_d(  2)/+.22540333075851662296e+00_t_real/
  Data p_d(  3)/+.55555555555555555556e+00_t_real/
  !!
  !!     corrections, nodes and weights for the 7-point formula.
  !!
  !!     corrections for f(1,1) and f(2,1).
  Data p_d(  4)/+.647209421402969791e-02_t_real/
  Data p_d(  5)/-.928968790944433705e-02_t_real/
  !!     nodes and weights for f(3,1-2)
  Data p_d(  6)/+.39508731291979716579e-01_t_real/
  Data p_d(  7)/+.10465622602646726519e+00_t_real/
  Data p_d(  8)/+.56575625065319744200e+00_t_real/
  Data p_d(  9)/+.40139741477596222291e+00_t_real/
  !!
  !!     corrections, nodes and weights for the 15-point formula.
  !!
  !!     corrections for f(1,1), f(2,1), f(3,1-2).
  Data p_d( 10)/+.5223046896961622e-04_t_real/
  Data p_d( 11)/+.17121030961750000e-03_t_real/
  Data p_d( 12)/-.724830016153892898e-03_t_real/
  Data p_d( 13)/-.7017801099209042e-04_t_real/
  !!     nodes and weights for f(4,1-4).
  Data p_d( 14)/+.61680367872449777899e-02_t_real/
  Data p_d( 15)/+.17001719629940260339e-01_t_real/
  Data p_d( 16)/+.11154076712774300110e+00_t_real/
  Data p_d( 17)/+.92927195315124537686e-01_t_real/
  Data p_d( 18)/+.37889705326277359705e+00_t_real/
  Data p_d( 19)/+.17151190913639138079e+00_t_real/
  Data p_d( 20)/+.77661331357103311837e+00_t_real/
  Data p_d( 21)/+.21915685840158749640e+00_t_real/
  !!
  !!     corrections, nodes and weights for the 31-point formula.
  !!
  !!     corrections for f(1,1), f(2,1), f(3,1-2), f(4,1-4).
  Data p_d( 22)/+.682166534792e-08_t_real/
  Data p_d( 23)/+.12667409859336e-06_t_real/
  Data p_d( 24)/+.59565976367837165e-05_t_real/
  Data p_d( 25)/+.1392330106826e-07_t_real/
  Data p_d( 26)/-.6629407564902392e-04_t_real/
  Data p_d( 27)/-.704395804282302e-06_t_real/
  Data p_d( 28)/-.34518205339241e-07_t_real/
  Data p_d( 29)/-.814486910996e-08_t_real/
  !!     nodes and weights for f(5,1-8).
  Data p_d( 30)/+.90187503233240234038e-03_t_real/
  Data p_d( 31)/+.25447807915618744154e-02_t_real/
  Data p_d( 32)/+.18468850446259893130e-01_t_real/
  Data p_d( 33)/+.16446049854387810934e-01_t_real/
  Data p_d( 34)/+.70345142570259943330e-01_t_real/
  Data p_d( 35)/+.35957103307129322097e-01_t_real/
  Data p_d( 36)/+.16327406183113126449e+00_t_real/
  Data p_d( 37)/+.56979509494123357412e-01_t_real/
  Data p_d( 38)/+.29750379350847292139e+00_t_real/
  Data p_d( 39)/+.76879620499003531043e-01_t_real/
  Data p_d( 40)/+.46868025635562437602e+00_t_real/
  Data p_d( 41)/+.93627109981264473617e-01_t_real/
  Data p_d( 42)/+.66886460674202316691e+00_t_real/
  Data p_d( 43)/+.10566989358023480974e+00_t_real/
  Data p_d( 44)/+.88751105686681337425e+00_t_real/
  Data p_d( 45)/+.11195687302095345688e+00_t_real/
  !!
  !!     corrections, nodes and weights for the 63-point formula.
  !!
  !!     corrections for f(1,1), f(2,1), f(3,1-2), f(4,1-4), f(5,1-8).
  Data p_d( 46)/+.371583e-15_t_real/
  Data p_d( 47)/+.21237877e-12_t_real/
  Data p_d( 48)/+.10522629388435e-08_t_real/
  Data p_d( 49)/+.1748029e-14_t_real/
  Data p_d( 50)/+.3475718983017160e-06_t_real/
  Data p_d( 51)/+.90312761725e-11_t_real/
  Data p_d( 52)/+.12558916e-13_t_real/
  Data p_d( 53)/+.54591e-15_t_real/
  Data p_d( 54)/-.72338395508691963e-05_t_real/
  Data p_d( 55)/-.169699579757977e-07_t_real/
  Data p_d( 56)/-.854363907155e-10_t_real/
  Data p_d( 57)/-.12281300930e-11_t_real/
  Data p_d( 58)/-.462334825e-13_t_real/
  Data p_d( 59)/-.42244055e-14_t_real/
  Data p_d( 60)/-.88501e-15_t_real/
  Data p_d( 61)/-.40904e-15_t_real/
  !!     nodes and weights for f(6,1-16).
  Data p_d( 62)/+.12711187964238806027e-03_t_real/
  Data p_d( 63)/+.36322148184553065969e-03_t_real/
  Data p_d( 64)/+.27937406277780409196e-02_t_real/
  Data p_d( 65)/+.25790497946856882724e-02_t_real/
  Data p_d( 66)/+.11315242452570520059e-01_t_real/
  Data p_d( 67)/+.61155068221172463397e-02_t_real/
  Data p_d( 68)/+.27817125251418203419e-01_t_real/
  Data p_d( 69)/+.10498246909621321898e-01_t_real/
  Data p_d( 70)/+.53657141626597094849e-01_t_real/
  Data p_d( 71)/+.15406750466559497802e-01_t_real/
  Data p_d( 72)/+.89628843042995707499e-01_t_real/
  Data p_d( 73)/+.20594233915912711149e-01_t_real/
  Data p_d( 74)/+.13609206180630952284e+00_t_real/
  Data p_d( 75)/+.25869679327214746911e-01_t_real/
  Data p_d( 76)/+.19305946804978238813e+00_t_real/
  Data p_d( 77)/+.31073551111687964880e-01_t_real/
  Data p_d( 78)/+.26024395564730524132e+00_t_real/
  Data p_d( 79)/+.36064432780782572640e-01_t_real/
  Data p_d( 80)/+.33709033997521940454e+00_t_real/
  Data p_d( 81)/+.40715510116944318934e-01_t_real/
  Data p_d( 82)/+.42280428994795418516e+00_t_real/
  Data p_d( 83)/+.44914531653632197414e-01_t_real/
  Data p_d( 84)/+.51638197305415897244e+00_t_real/
  Data p_d( 85)/+.48564330406673198716e-01_t_real/
  Data p_d( 86)/+.61664067580126965307e+00_t_real/
  Data p_d( 87)/+.51583253952048458777e-01_t_real/
  Data p_d( 88)/+.72225017797817568492e+00_t_real/
  Data p_d( 89)/+.53905499335266063927e-01_t_real/
  Data p_d( 90)/+.83176474844779253501e+00_t_real/
  Data p_d( 91)/+.55481404356559363988e-01_t_real/
  Data p_d( 92)/+.94365568695340721002e+00_t_real/
  Data p_d( 93)/+.56277699831254301273e-01_t_real/
  !!
  !!     corrections, nodes and weights for the 127-point formula.
  !!
  !!     corrections for f(3,1), f(4,1-2), f(5,1-3), f(6,1-6).
  Data p_d( 94)/+.1041098e-15_t_real/
  Data p_d( 95)/+.249472054598e-10_t_real/
  Data p_d( 96)/+.55e-20_t_real/
  Data p_d( 97)/+.290412475995385e-07_t_real/
  Data p_d( 98)/+.367282126e-13_t_real/
  Data p_d( 99)/+.5568e-18_t_real/
  Data p_d(100)/-.871176477376972025e-06_t_real/
  Data p_d(101)/-.8147324267441e-09_t_real/
  Data p_d(102)/-.8830920337e-12_t_real/
  Data p_d(103)/-.18018239e-14_t_real/
  Data p_d(104)/-.70528e-17_t_real/
  Data p_d(105)/-.506e-19_t_real/
  !!     nodes and weights for f(7,1-32).
  Data p_d(106)/+.17569645108401419961e-04_t_real/
  Data p_d(107)/+.50536095207862517625e-04_t_real/
  Data p_d(108)/+.40120032808931675009e-03_t_real/
  Data p_d(109)/+.37774664632698466027e-03_t_real/
  Data p_d(110)/+.16833646815926074696e-02_t_real/
  Data p_d(111)/+.93836984854238150079e-03_t_real/
  Data p_d(112)/+.42758953015928114900e-02_t_real/
  Data p_d(113)/+.16811428654214699063e-02_t_real/
  Data p_d(114)/+.85042788218938676006e-02_t_real/
  Data p_d(115)/+.25687649437940203731e-02_t_real/
  Data p_d(116)/+.14628500401479628890e-01_t_real/
  Data p_d(117)/+.35728927835172996494e-02_t_real/
  Data p_d(118)/+.22858485360294285840e-01_t_real/
  Data p_d(119)/+.46710503721143217474e-02_t_real/
  Data p_d(120)/+.33362148441583432910e-01_t_real/
  Data p_d(121)/+.58434498758356395076e-02_t_real/
  Data p_d(122)/+.46269993574238863589e-01_t_real/
  Data p_d(123)/+.70724899954335554680e-02_t_real/
  Data p_d(124)/+.61679602220407116350e-01_t_real/
  Data p_d(125)/+.83428387539681577056e-02_t_real/
  Data p_d(126)/+.79659974529987579270e-01_t_real/
  Data p_d(127)/+.96411777297025366953e-02_t_real/
  Data p_d(128)/+.10025510022305996335e+00_t_real/
  Data p_d(129)/+.10955733387837901648e-01_t_real/
  Data p_d(130)/+.12348658551529473026e+00_t_real/
  Data p_d(131)/+.12275830560082770087e-01_t_real/
  Data p_d(132)/+.14935550523164972024e+00_t_real/
  Data p_d(133)/+.13591571009765546790e-01_t_real/
  Data p_d(134)/+.17784374563501959262e+00_t_real/
  Data p_d(135)/+.14893641664815182035e-01_t_real/
  Data p_d(136)/+.20891506620015163857e+00_t_real/
  Data p_d(137)/+.16173218729577719942e-01_t_real/
  Data p_d(138)/+.24251603361948636206e+00_t_real/
  Data p_d(139)/+.17421930159464173747e-01_t_real/
  Data p_d(140)/+.27857691462990108452e+00_t_real/
  Data p_d(141)/+.18631848256138790186e-01_t_real/
  Data p_d(142)/+.31701256890892077191e+00_t_real/
  Data p_d(143)/+.19795495048097499488e-01_t_real/
  Data p_d(144)/+.35772335749024048622e+00_t_real/
  Data p_d(145)/+.20905851445812023852e-01_t_real/
  Data p_d(146)/+.40059606975775710702e+00_t_real/
  Data p_d(147)/+.21956366305317824939e-01_t_real/
  Data p_d(148)/+.44550486736806745112e+00_t_real/
  Data p_d(149)/+.22940964229387748761e-01_t_real/
  Data p_d(150)/+.49231224246628339785e+00_t_real/
  Data p_d(151)/+.23854052106038540080e-01_t_real/
  Data p_d(152)/+.54086998801016766712e+00_t_real/
  Data p_d(153)/+.24690524744487676909e-01_t_real/
  Data p_d(154)/+.59102017877011132759e+00_t_real/
  Data p_d(155)/+.25445769965464765813e-01_t_real/
  Data p_d(156)/+.64259616216846784762e+00_t_real/
  Data p_d(157)/+.26115673376706097680e-01_t_real/
  Data p_d(158)/+.69542355844328595666e+00_t_real/
  Data p_d(159)/+.26696622927450359906e-01_t_real/
  Data p_d(160)/+.74932126969651682339e+00_t_real/
  Data p_d(161)/+.27185513229624791819e-01_t_real/
  Data p_d(162)/+.80410249728889984607e+00_t_real/
  Data p_d(163)/+.27579749566481873035e-01_t_real/
  Data p_d(164)/+.85957576684743982540e+00_t_real/
  Data p_d(165)/+.27877251476613701609e-01_t_real/
  Data p_d(166)/+.91554595991628911629e+00_t_real/
  Data p_d(167)/+.28076455793817246607e-01_t_real/
  Data p_d(168)/+.97181535105025430566e+00_t_real/
  Data p_d(169)/+.28176319033016602131e-01_t_real/
  !!
  !!     corrections, nodes and weights for the 255-point formula.
  !!
  !!     corrections for f(4,1), f(5,1), f(6,1-2), f(7,1-4).
  Data p_d(170)/+.3326e-18_t_real/
  Data p_d(171)/+.114094770478e-11_t_real/
  Data p_d(172)/+.2952436056970351e-08_t_real/
  Data p_d(173)/+.51608328e-15_t_real/
  Data p_d(174)/-.110177219650597323e-06_t_real/
  Data p_d(175)/-.58656987416475e-10_t_real/
  Data p_d(176)/-.23340340645e-13_t_real/
  Data p_d(177)/-.1248950e-16_t_real/
  !!     nodes and weights for f(8,1-64).
  Data p_d(178)/+.24036202515353807630e-05_t_real/
  Data p_d(179)/+.69379364324108267170e-05_t_real/
  Data p_d(180)/+.56003792945624240417e-04_t_real/
  Data p_d(181)/+.53275293669780613125e-04_t_real/
  Data p_d(182)/+.23950907556795267013e-03_t_real/
  Data p_d(183)/+.13575491094922871973e-03_t_real/
  Data p_d(184)/+.61966197497641806982e-03_t_real/
  Data p_d(185)/+.24921240048299729402e-03_t_real/
  Data p_d(186)/+.12543855319048853002e-02_t_real/
  Data p_d(187)/+.38974528447328229322e-03_t_real/
  Data p_d(188)/+.21946455040427254399e-02_t_real/
  Data p_d(189)/+.55429531493037471492e-03_t_real/
  Data p_d(190)/+.34858540851097261500e-02_t_real/
  Data p_d(191)/+.74028280424450333046e-03_t_real/
  Data p_d(192)/+.51684971993789994803e-02_t_real/
  Data p_d(193)/+.94536151685852538246e-03_t_real/
  Data p_d(194)/+.72786557172113846706e-02_t_real/
  Data p_d(195)/+.11674841174299594077e-02_t_real/
  Data p_d(196)/+.98486295992298408193e-02_t_real/
  Data p_d(197)/+.14049079956551446427e-02_t_real/
  Data p_d(198)/+.12907472045965932809e-01_t_real/
  Data p_d(199)/+.16561127281544526052e-02_t_real/
  Data p_d(200)/+.16481342421367271240e-01_t_real/
  Data p_d(201)/+.19197129710138724125e-02_t_real/
  Data p_d(202)/+.20593718329137316189e-01_t_real/
  Data p_d(203)/+.21944069253638388388e-02_t_real/
  Data p_d(204)/+.25265540247597332240e-01_t_real/
  Data p_d(205)/+.24789582266575679307e-02_t_real/
  Data p_d(206)/+.30515340497540768229e-01_t_real/
  Data p_d(207)/+.27721957645934509940e-02_t_real/
  Data p_d(208)/+.36359378430187867480e-01_t_real/
  Data p_d(209)/+.30730184347025783234e-02_t_real/
  Data p_d(210)/+.42811783890139037259e-01_t_real/
  Data p_d(211)/+.33803979910869203823e-02_t_real/
  Data p_d(212)/+.49884702478705123440e-01_t_real/
  Data p_d(213)/+.36933779170256508183e-02_t_real/
  Data p_d(214)/+.57588434808916940190e-01_t_real/
  Data p_d(215)/+.40110687240750233989e-02_t_real/
  Data p_d(216)/+.65931563842274211999e-01_t_real/
  Data p_d(217)/+.43326409680929828545e-02_t_real/
  Data p_d(218)/+.74921067092924347640e-01_t_real/
  Data p_d(219)/+.46573172997568547773e-02_t_real/
  Data p_d(220)/+.84562412844234959360e-01_t_real/
  Data p_d(221)/+.49843645647655386012e-02_t_real/
  Data p_d(222)/+.94859641186738404810e-01_t_real/
  Data p_d(223)/+.53130866051870565663e-02_t_real/
  Data p_d(224)/+.10581543166444097714e+00_t_real/
  Data p_d(225)/+.56428181013844441585e-02_t_real/
  Data p_d(226)/+.11743115975265809315e+00_t_real/
  Data p_d(227)/+.59729195655081658049e-02_t_real/
  Data p_d(228)/+.12970694445188609414e+00_t_real/
  Data p_d(229)/+.63027734490857587172e-02_t_real/
  Data p_d(230)/+.14264168911376784347e+00_t_real/
  Data p_d(231)/+.66317812429018878941e-02_t_real/
  Data p_d(232)/+.15623311732729139895e+00_t_real/
  Data p_d(233)/+.69593614093904229394e-02_t_real/
  Data p_d(234)/+.17047780536259859981e+00_t_real/
  Data p_d(235)/+.72849479805538070639e-02_t_real/
  Data p_d(236)/+.18537121234486258656e+00_t_real/
  Data p_d(237)/+.76079896657190565832e-02_t_real/
  Data p_d(238)/+.20090770903915859819e+00_t_real/
  Data p_d(239)/+.79279493342948491103e-02_t_real/
  Data p_d(240)/+.21708060588171698360e+00_t_real/
  Data p_d(241)/+.82443037630328680306e-02_t_real/
  Data p_d(242)/+.23388218069623990928e+00_t_real/
  Data p_d(243)/+.85565435613076896192e-02_t_real/
  Data p_d(244)/+.25130370638306339718e+00_t_real/
  Data p_d(245)/+.88641732094824942641e-02_t_real/
  Data p_d(246)/+.26933547875781873867e+00_t_real/
  Data p_d(247)/+.91667111635607884067e-02_t_real/
  Data p_d(248)/+.28796684463774796540e+00_t_real/
  Data p_d(249)/+.94636899938300652943e-02_t_real/
  Data p_d(250)/+.30718623022088529711e+00_t_real/
  Data p_d(251)/+.97546565363174114611e-02_t_real/
  Data p_d(252)/+.32698116976958152079e+00_t_real/
  Data p_d(253)/+.10039172044056840798e-01_t_real/
  Data p_d(254)/+.34733833458998250389e+00_t_real/
  Data p_d(255)/+.10316812330947621682e-01_t_real/
  Data p_d(256)/+.36824356228880576959e+00_t_real/
  Data p_d(257)/+.10587167904885197931e-01_t_real/
  Data p_d(258)/+.38968188628481359983e+00_t_real/
  Data p_d(259)/+.10849844089337314099e-01_t_real/
  Data p_d(260)/+.41163756555233745857e+00_t_real/
  Data p_d(261)/+.11104461134006926537e-01_t_real/
  Data p_d(262)/+.43409411457634557737e+00_t_real/
  Data p_d(263)/+.11350654315980596602e-01_t_real/
  Data p_d(264)/+.45703433350168850951e+00_t_real/
  Data p_d(265)/+.11588074033043952568e-01_t_real/
  Data p_d(266)/+.48044033846254297801e+00_t_real/
  Data p_d(267)/+.11816385890830235763e-01_t_real/
  Data p_d(268)/+.50429359208123853983e+00_t_real/
  Data p_d(269)/+.12035270785279562630e-01_t_real/
  Data p_d(270)/+.52857493412834112307e+00_t_real/
  Data p_d(271)/+.12244424981611985899e-01_t_real/
  Data p_d(272)/+.55326461233797152625e+00_t_real/
  Data p_d(273)/+.12443560190714035263e-01_t_real/
  Data p_d(274)/+.57834231337383669993e+00_t_real/
  Data p_d(275)/+.12632403643542078765e-01_t_real/
  Data p_d(276)/+.60378719394238406082e+00_t_real/
  Data p_d(277)/+.12810698163877361967e-01_t_real/
  Data p_d(278)/+.62957791204992176986e+00_t_real/
  Data p_d(279)/+.12978202239537399286e-01_t_real/
  Data p_d(280)/+.65569265840056197721e+00_t_real/
  Data p_d(281)/+.13134690091960152836e-01_t_real/
  Data p_d(282)/+.68210918793152331682e+00_t_real/
  Data p_d(283)/+.13279951743930530650e-01_t_real/
  Data p_d(284)/+.70880485148175331803e+00_t_real/
  Data p_d(285)/+.13413793085110098513e-01_t_real/
  Data p_d(286)/+.73575662758907323806e+00_t_real/
  Data p_d(287)/+.13536035934956213614e-01_t_real/
  Data p_d(288)/+.76294115441017027278e+00_t_real/
  Data p_d(289)/+.13646518102571291428e-01_t_real/
  Data p_d(290)/+.79033476175681880523e+00_t_real/
  Data p_d(291)/+.13745093443001896632e-01_t_real/
  Data p_d(292)/+.81791350324074780175e+00_t_real/
  Data p_d(293)/+.13831631909506428676e-01_t_real/
  Data p_d(294)/+.84565318851862189130e+00_t_real/
  Data p_d(295)/+.13906019601325461264e-01_t_real/
  Data p_d(296)/+.87352941562769803314e+00_t_real/
  Data p_d(297)/+.13968158806516938516e-01_t_real/
  Data p_d(298)/+.90151760340188079791e+00_t_real/
  Data p_d(299)/+.14017968039456608810e-01_t_real/
  Data p_d(300)/+.92959302395714482093e+00_t_real/
  Data p_d(301)/+.14055382072649964277e-01_t_real/
  Data p_d(302)/+.95773083523463639678e+00_t_real/
  Data p_d(303)/+.14080351962553661325e-01_t_real/
  Data p_d(304)/+.98590611358921753738e+00_t_real/
  Data p_d(305)/+.14092845069160408355e-01_t_real/
  !!
  !!
  Integer(t_int),Parameter   :: QUADSTART=0_t_int
  Integer(t_int),Parameter   :: L1I=100_t_int
  Integer(t_int),Parameter   :: L2B2=220_t_int
  Integer(t_int),Parameter   :: L2B3=230_t_int
  !!
  Integer(t_int),Parameter   :: IJAC_TYPE_LIN=1_t_int
  Integer(t_int),Parameter   :: IJAC_TYPE_LOG=2_t_int
  Integer(t_int),Parameter   :: IJAC_TYPE_POW=3_t_int
  Integer(t_int),Parameter   :: IJAC_TYPE_SIN=4_t_int
  !!END DATA quad_gkp:
  !!***********************************************************************

Contains
  !***********************************************************************
  !
  Subroutine lag4int(n,f,h,val,way)
    !!
    !! integration via 4-th Lagrange-interpolation
    !! use intermediate integrals of P4 to get integral for each mesh point
    !! way=0 replaces f by int(f,from x(1) to x(i))
    !! way=1 replaces f by int(f,from x(i)  to x(n))
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) ::  n ! number of points
    Integer(t_int),Intent(in) ::  way ! outside 0 or inside 1
    Real(t_real),Intent(inout):: f(n) ! function
    
    Real(t_real),Intent(in):: h ! step width
    Real(t_real),Intent(out):: val ! itegral
    !!LOCAL
    Character(*),Parameter::nsubfunc="LAG4INT"
    Integer(t_int) ::  i,k,j
    !
    Real(t_real) ::  ff(5_t_int),ll(5_t_int)
    !      
    !***********************************************************************
    !
    !
    k=Mod(n-1_t_int,4_t_int) 
    j=(n-k-1_t_int)/4_t_int  
    If(j*4_t_int+k+1_t_int/=n) Then
       Call terminate(TERM_CRASH,Trim(nsubfunc)//': error, sorry ')
    Endif
    If(n<5_t_int)&
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<5!')
    !
    
    outward: If(way==lagint_out)Then
       
       ff=f(1_t_int:5_t_int)
       ll=f(n-4_t_int:n)
       
       f(1_t_int)=0.0_t_real
       
       Do i=5_t_int,n-k,4_t_int
          f(i-3_t_int:i)=f(i-4_t_int)+Matmul(cof4,ff)*h
          If(i+4_t_int<=n-k)Then
             ff(1_t_int)=ff(5_t_int)
             ff(2_t_int:)=f(i+1_t_int:i+4_t_int)
          Endif
       Enddo
       !
       f(n-k+1_t_int:n) =f(n-4_t_int)+Matmul(cof4(5_t_int-k:4_t_int,:),ll)*h
       !       
       val=f(n)
       Return
       
    Else ! way==lag4int_in
       
       ff=f(n-k:n-k-4_t_int:-1_t_int)
       ll=f(n:n-4_t_int:-1_t_int)
       !
       f(n)=0.0_t_real
       f(n-1_t_int:n-k:-1_t_int)=Matmul(cof4(1_t_int:k,:),ll)*h
       !
       Do i=n-k-4_t_int,1_t_int,-4_t_int
          f(i+3_t_int:i:-1_t_int)=f(i+4_t_int)+Matmul(cof4,ff)*h
          If(i-4_t_int>0_t_int)Then
             ff(1_t_int)=ff(5_t_int)
             ff(2_t_int:)=f(i-1_t_int:i-4_t_int:-1_t_int)
          Endif
       Enddo
       !
       val=f(1_t_int)
       Return
       !
    End If outward
  End Subroutine lag4int
  !
  !***********************************************************************
  !***********************************************************************
  !
  Subroutine tlag4int(n,f,h,val)
    !!
    !! integration via 4-th Lagrange-interpolation
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) ::  n   ! number of points
    Real(t_real),Intent(in)  :: f(n) ! function
    
    Real(t_real),Intent(in):: h    ! step width
    Real(t_real),Intent(out):: val ! itegral
    !!LOCAL
    Character(*),Parameter::nsubfunc="TLAG4INT"
    Integer(t_int) ::  k,j
    !
    !***********************************************************************
    !
    !
    ! n= 1+4*j+k,   k=[0-3]
    ! k=mod(n-1,4)
    ! j=(n-k-1)/4
    !
    k=Mod(n-1_t_int,4_t_int) 
    j=(n-k-1_t_int)/4_t_int  
    !
    If(n<5_t_int)&
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<5!')
    !
    
    val=(   Sum(f(2_t_int:4_t_int*j-2_t_int:4_t_int))&
         &       *64.00_t_real/45.0_t_real &
         & +Sum(f(3_t_int:4_t_int*j-1_t_int:4_t_int))&
         &       *8.00_t_real/15.0_t_real &
         & +Sum(f(4_t_int:4_t_int*j-0_t_int:4_t_int))&
         &       *64.00_t_real/45.0_t_real &
         & +Sum(f(5_t_int:4_t_int*j-3_t_int:4_t_int))&
         &       *28.00_t_real/45.0_t_real &
         & +    f(1_t_int)*14.00_t_real/45.0_t_real &
         & +    f(1_t_int+4_t_int*j)*14.00_t_real/45.0_t_real)*h

    If(k == 1_t_int)Then
       val=val+h*Sum(f(n-4_t_int:n)*(/&
            & -19.0_t_real/720.0_t_real&
            &,53.0_t_real/360.0_t_real&
            &,-11.0_t_real/30.0_t_real&
            &,323.0_t_real/360.0_t_real&
            &,251.0_t_real/720.0_t_real/))
    Else if(k == 2_t_int)Then
       val=val+h*Sum(f(n-4_t_int:n)*(/&
            & -1.0_t_real/90.0_t_real&
            &,2.0_t_real/45.0_t_real&
            &,4.0_t_real/15.0_t_real&
            &,62.0_t_real/45.0_t_real&
            &,29.0_t_real/90.0_t_real/))
    Else if(k == 3_t_int)Then
       val=val+h*Sum(f(n-4_t_int:n)*(/&
            & -3.0_t_real/80.0_t_real&
            &,21.0_t_real/40.0_t_real&
            &,9.0_t_real/10.0_t_real&
            &,51.0_t_real/40.0_t_real&
            &,27.0_t_real/80.0_t_real/))
    Endif
    
    Return
  End Subroutine tlag4int
  !!***********************************************************************
  !***********************************************************************
  !
  Subroutine tjlag4int(n,f,jac,h,val)
    !!
    !! integration f*jac via 4-th Lagrange-interpolation
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) ::  n   ! number of points
    Real(t_real),Intent(in)  :: f(n) ! function
    Real(t_real),Intent(in)  :: jac(n) ! function
    
    Real(t_real),Intent(in):: h    ! step width
    Real(t_real),Intent(out):: val ! itegral
    !!LOCAL
    Character(*),Parameter::nsubfunc="TJLAG4INT"
    Integer(t_int) ::  k,j
    !
    !***********************************************************************
    !
    !
    ! n= 1+4*j+k,   k=[0-3]
    ! k=mod(n-1,4)
    ! j=(n-k-1)/4
    !
    k=Mod(n-1_t_int,4_t_int) 
    j=(n-k-1_t_int)/4_t_int  
    !
    If(n<5_t_int)&
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<5!')
    !
    
    val=(   Sum(jac(2_t_int:4_t_int*j-2_t_int:4_t_int)&
         &       *f(2_t_int:4_t_int*j-2_t_int:4_t_int))&
         &    *64.00_t_real/45.0_t_real &
         & +Sum(jac(3_t_int:4_t_int*j-1_t_int:4_t_int)&
         &       *f(3_t_int:4_t_int*j-1_t_int:4_t_int))&
         &    *8.00_t_real/15.0_t_real &
         & +Sum(jac(4_t_int:4_t_int*j-0_t_int:4_t_int)&
         &       *f(4_t_int:4_t_int*j-0_t_int:4_t_int))&
         &    *64.00_t_real/45.0_t_real &
         & +Sum(jac(5_t_int:4_t_int*j-3_t_int:4_t_int)&
         &       *f(5_t_int:4_t_int*j-3_t_int:4_t_int))&
         &    *28.00_t_real/45.0_t_real &
         & +    jac(1_t_int)*f(1_t_int)*14.00_t_real/45.0_t_real &
         & +    jac(1_t_int+4_t_int*j)*f(1_t_int+4_t_int*j)&
         &*14.00_t_real/45.0_t_real)*h

    If(k == 1_t_int)Then
       val=val+h*Sum(jac(n-4_t_int:n)*f(n-4_t_int:n)*(/&
            & -19.0_t_real/720.0_t_real&
            &,53.0_t_real/360.0_t_real&
            &,-11.0_t_real/30.0_t_real&
            &,323.0_t_real/360.0_t_real&
            &,251.0_t_real/720.0_t_real/))
    Else if(k == 2_t_int)Then
       val=val+h*Sum(jac(n-4_t_int:n)*f(n-4_t_int:n)*(/&
            & -1.0_t_real/90.0_t_real&
            &,2.0_t_real/45.0_t_real&
            &,4.0_t_real/15.0_t_real&
            &,62.0_t_real/45.0_t_real&
            &,29.0_t_real/90.0_t_real/))
    Else if(k == 3_t_int)Then
       val=val+h*Sum(jac(n-4_t_int:n)*f(n-4_t_int:n)*(/&
            & -3.0_t_real/80.0_t_real&
            &,21.0_t_real/40.0_t_real&
            &,9.0_t_real/10.0_t_real&
            &,51.0_t_real/40.0_t_real&
            &,27.0_t_real/80.0_t_real/))
    Endif
    
    Return
  End Subroutine tjlag4int
  !!***********************************************************************

  !!***********************************************************************
  !!
  !!
  !! this code gives an integral for each mesh points not only for all
  !! odd, using the integral of the interpolation polynomial for every
  !! second step
  !! way=0 replaces f by int(f,from x(1) to x(i))
  !! way=1 replaces f by int(f,from x(i)  to x(n))
  !!
  Subroutine lag2int(n,f,h,val,way)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: n    !!     number of points
    Integer(t_int),Intent(in) :: way  !!     outside 0 or inside 1

    Real(t_real),Intent(inout) :: f(n) !!     function
    Real(t_real),Intent(in) :: h    !!     step width
    Real(t_real),Intent(out) :: val  !!     itegral at final point (n or 1 according to way)
    !!LOCAL
    Character(*),Parameter::nsubfunc="LAG2INT"
    Integer(t_int) :: i,k,j
    !!
    Real(t_real) :: ff(3_t_int),ll(3_t_int)
    !!      
    !!***********************************************************************
    !!      
    
    k=Mod(n-1_t_int,2_t_int) 
    j=(n-k-1_t_int)/2_t_int  
    If(j*2_t_int+k+1_t_int/=n) &
         &Call terminate(TERM_CRASH,Trim(nsubfunc)//': error, sorry ')
    If(n<3_t_int)&
         &Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<3!')
    
    !!
    outward: If(way==lagint_out)Then
       
       ff=f(1_t_int:3_t_int)
       ll=f(n-2_t_int:n)
       
       f(1_t_int)=0.0_t_real
       
       Do i=3_t_int,n-k,2_t_int
          f(i-1_t_int:i)=f(i-2_t_int)+Matmul(cof2,ff)*h
          If(i+2_t_int<=n-k)Then
             ff(1_t_int)=ff(3_t_int)
             ff(2_t_int:)=f(i+1_t_int:i+2_t_int)
          Endif
       Enddo
       ! f(n-k+1_t_int:n)=f(n-2_t_int)+Matmul(cof2(3_t_int-k:2_t_int,:),ll)*h
       ! simpler
       If(k==1_t_int)Then
          f(n)=f(n-2_t_int)+Dot_product(cof2(2_t_int,:),ll)*h
       End If
       !!
       val=f(n)
       Return
    Else ! way==lag4int_in
       
       ff=f(n-k:n-k-2_t_int:-1_t_int)
       ll=f(n:n-2_t_int:-1_t_int)
       
       f(n)=0.0_t_real
       
       ! f(n-1_t_int:n-k:-1_t_int)=Matmul(cof2(:k,:),ll)*h
       ! simpler
       If(k==1_t_int)Then
          f(n-1_t_int)=Dot_product(cof2(1_t_int,:),ll)*h
       End If
       
       Do i=n-k-2_t_int,1_t_int,-2_t_int
          f(i+1_t_int:i:-1_t_int)=f(i+2_t_int)+Matmul(cof2,ff)*h
          If(i-2_t_int>0_t_int)Then
             ff(1_t_int)=ff(3_t_int)
             ff(2_t_int:)=f(i-1_t_int:i-2_t_int:-1_t_int)
          End If
          
       Enddo
       !!
       val=f(1_t_int)
       !!
       Return
    Endif outward
    !!
  End Subroutine lag2int
  !!
  !!***********************************************************************
  Subroutine simpson(n,f,h,val)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in) :: n
    Real(t_real),Intent(in)  :: f(n),h
    Real(t_real),Intent(out)  :: val
    !!LOCAL
    Integer(t_int)::k,m,i
    Real(t_real) ::sum,sum1
    !!
    k=Mod(n,2_t_int)
    m=n-1_t_int+k
    sum = 0.0_t_real
 
    Do i = 2_t_int,m-1_t_int,2_t_int
       sum=sum+f(i)
    Enddo ! i
    
    sum=sum+sum
    sum=sum+sum
    
    sum1=0.0_t_real
    Do i = 3_t_int,m-2_t_int,2_t_int
       sum1=sum1+f(i)
    Enddo
    
    sum=sum+sum1+sum1
    val=sum+f(1_t_int)+f(m)
    val=val*h/3.0_t_real
    If(k == 0_t_int)Then
       val=val+h/12.0_t_real*(5.0_t_real*f(n)+8.0_t_real*f(n-1_t_int)-f(n-2_t_int))
    Endif
    Return
  End Subroutine simpson
  !!***********************************************************************
  !!***********************************************************************
  !!
  !! Gauss abscissias and weights
  !!
  !!
  !! 18.12.95 by Arthur Ernst
  !!
  !! n   : number of points
  !! x1  : lower  limit
  !! x2  : upper  limit
  !! w   : weights
  !! x   : abscissas
  !! ----------------------------------------------------------------------
  Subroutine gauss(n,w,x,x1,x2)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: n
    Real(t_real),Intent(in)  :: x1,x2
    Real(t_real),Intent(out) :: x(*),w(*)
    !!LOCAL
    Integer(t_int) ::  i,j,m
    Real(t_real)  :: p1,p2,p3,pp,xl,xm,z,z1
    !!
    Real(t_real),Parameter :: eps=3.e-14_t_real
    !!
    m=(n+1_t_int)/2_t_int
    xm=0.5_t_real*(x2+x1)
    xl=0.5_t_real*(x2-x1)
    
    Do i=1_t_int,m
       z=Cos(pi*(Real(i,t_real)-0.25_t_real)/(Real(n,t_real)+0.5_t_real))
1      Continue
       p1=1.0_t_real
       p2=0.0_t_real
       Do j=1_t_int,n
          p3=p2
          p2=p1
          p1=((2.0_t_real*Real(j,t_real)-1.0_t_real)*z*p2&
               &-(Real(j,t_real)-1.0_t_real)*p3)/Real(j,t_real)
       Enddo
       pp=Real(n,t_real)*(z*p1-p2)/(z*z-1.0_t_real)
       z1=z
       z=z1-p1/pp
       If(Abs(z-z1) > eps) Goto 1
       x(i)=xm-xl*z
       x(n+1_t_int-i)=xm+xl*z
       w(i)=2.0_t_real*xl/((1.0_t_real-z*z)*pp*pp)
       w(n+1_t_int-i)=w(i)
    Enddo
    Return
  End Subroutine gauss
  !!***********************************************************************
  !!***********************************************************************
  !! given a tabulated function r,f and a  known value f0 at r0 < r(1)
  !! this approximats the integral from r0 to r1
  Subroutine simpintleft(r,f,r0,f0,val)
    !!
    Implicit None 
    !!IO
    Real(t_real),Intent(in) ::r(2),f(2),r0,f0
    Real(t_real),Intent(out)::val
    !!LOCAL
    Real(t_real) :: d12,d13,d23
    d12=(r0-r(1_t_int))
    d13=(r0-r(2_t_int))
    d23=(r(1_t_int)-r(2_t_int))
    val= &
         & -f0  *d12*(2.0_t_real*d13+d23)/6.0_t_real/d13 &
         & -f(1_t_int)*d12*(2.0_t_real*d23+d13)/6.0_t_real/d23 &
         & +f(2_t_int)*d12**3_t_int/6.0_t_real/d23/d13 
    
  End Subroutine simpintleft
  !!***********************************************************************
  !! provide new gauss meshs, without makeing it twice
  Subroutine  get_gauss(ng,p_ag,p_wg)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)   :: ng
    Real(t_real),Pointer        :: p_ag(:)
    Real(t_real),Pointer        :: p_wg(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="get_gauss"
    Type(T_Gauss), Pointer :: p_current,p_new
    Logical(t_log) :: found
    Integer :: icheck
    !!
    If(ng == 0_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Cannot call get_gauss with ng==0!'")
    Endif
    !!first lookup table of calculated meshes
    p_current=>gfirst
    !!
    !Print *,nsubfunc,'gfirst % next', Associated(p_current% next) 
    !Print *,nsubfunc,'gfirst % ng', p_current% ng 
    found=.False.
    searchloop: Do While (Associated(p_current% next))
       p_current=>p_current%next
       !Print *,nsubfunc,'p_current% ng', p_current% ng 
       If( p_current%ng == ng) Then
          !Print *,nsubfunc,'p_current% ng == ng' 
          found=.True.
          Exit searchloop
       Endif
       !Print *,nsubfunc,'p_current% next', Associated(p_current% next) 
    End Do searchloop
    !!
    If(found)Then
       !Print *,nsubfunc,' found ',ng
       !! do with p_current
       p_wg=>p_current% p_w
       p_ag=>p_current% p_a
    Else
       !Print *,nsubfunc,' not found ',ng

       Allocate(p_new,STAT=icheck)
       If(icheck/=0)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Could not allocate memory for p_new!'")
       Endif
       Nullify(p_new% next)
       Nullify(p_new% p_w)
       Nullify(p_new% p_a)
       p_new% ng=0_t_int
       !!
       p_current% next=>p_new
       p_current=>p_new
       !! do with p_current
       If (outlevel > L_more) Then
1         Format(a,' Create gauss > ngmax: ',i5)
          Write(outfile,1)Trim(nsubfunc),ng
       Endif
       Call palloc('p_current% p_w',p_current% p_w,1_t_int,ng,nmodule,nsubfunc)
       Call palloc('p_current% p_a',p_current% p_a,1_t_int,ng,nmodule,nsubfunc)
       Call gauss(ng,p_current% p_w,p_current% p_a,-1.0_t_real,1.0_t_real)
       p_current% ng=ng
       p_wg=>p_current% p_w
       p_ag=>p_current% p_a
    Endif
    !!
  End Subroutine get_gauss
  !!***********************************************************************
  Subroutine  init_gmesh
    !!
    Implicit None  
    !!IO
    !!LOCAL
    !!
    Nullify(gfirst% next)
    Nullify(gfirst% p_w)
    Nullify(gfirst% p_a)
    gfirst% ng=0_t_int
    !!
  End Subroutine init_gmesh
  !!***********************************************************************
  Subroutine clean_quadgkp(gkp)
    !!
    Implicit none  
    !!IO
    Type(T_Quadgkp),Intent(inout):: gkp
    !!LOCAL
    !!
    Call pfree(gkp% p_current)
    Call pfree(gkp% p_old)
    Call pfree(gkp% p_fval)
    Call pfree(gkp% p_ff)
    Call pfree(gkp% p_res)
    Call pfree(gkp% p_work)
    !!
    !!
    return
  End Subroutine clean_quadgkp
  !!***********************************************************************
  Subroutine  init_quadgkp(gkp)
    !!
    Implicit none  
    !!IO
    Type(T_Quadgkp),Intent(inout):: gkp
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_quadgkp"
    !!
    Nullify (gkp% p_current)
    Nullify (gkp% p_old)
    Nullify (gkp% p_fval)
    Nullify (gkp% p_ff)
    Nullify (gkp% p_res)
    Nullify (gkp% p_work)
    !!
    !!
    return
  End Subroutine init_quadgkp
  !!***********************************************************************
  Subroutine  alloc_quadgkp(gkp,ndim)
    !!
    Implicit none  
    !!IO
    Type(T_Quadgkp),Intent(inout):: gkp
    Integer(t_int),Intent(in)    :: ndim
    !!LOCAL
    Character(*),Parameter::nsubfunc="alloc_quadgkp"
    !!
    gkp% ndim=ndim
    gkp% state=QUADSTART
    Call palloc('gkp% p_current',gkp% p_current,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('gkp% p_old    ',gkp% p_old    ,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('gkp% p_fval   ',gkp% p_fval   ,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('gkp% p_ff     ',gkp% p_ff     ,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('gkp% p_res    ',gkp% p_res    ,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('gkp% p_work   ',gkp% p_work   ,1_t_int,17_t_int&
         &,1_t_int,ndim,nmodule,nsubfunc)
    !!
    !!
    return
  End Subroutine alloc_quadgkp
  !!***********************************************************************
  Subroutine  quad_gkp(gkp,r,ijac,calc)
    !!
    Implicit None  
    !!IO
    Type(T_Quadgkp),Intent(inout):: gkp
    Logical(t_log),Intent(out)   :: calc
    Real(t_real),Intent(out)     :: r,ijac
    !!LOCAL
    !Character(*),Parameter::nsubfunc="quad_gkp"
    Integer(t_int) :: ia,ndim
    Real(t_real)   :: a,b
    !!
    ndim=gkp% ndim
    !!Print *,nsubfunc,' memory=',22*ndim*8
    !!
    calc=.True.
    gkp% conv=.True.
    !!
    If(gkp% state == L1I)Goto 100    
    If(gkp% state == L2B2)Goto 220    
    If(gkp% state == L2B3)Goto 230    

    !!************ L0I ********************
    gkp% p_res(1:ndim)=0.0_t_real
    !!
    If(gkp% b0>=gkp% b1)Then
       calc=.False.
       !Print *,nsubfunc,' return b0>=b1'
       Return 
    Endif
    !Print *,'================',nsubfunc,'================='
    Call jac_bound(gkp% b0,gkp% b1,a,b,gkp% typ,gkp% exx)
    gkp% hx=0.5_t_real*(b-a)
    gkp% xm=0.5_t_real*(b+a)/gkp% hx
    Call jac_rx(0.0_t_real,gkp% xm,gkp% hx,r,gkp% rx,ijac,gkp% typ,gkp% exx)
    gkp% state=L1I
    Return
    !!************ L1I ********************
100 Continue 
    gkp% p_work(1,1:ndim)=gkp% p_ff(1:ndim)
    gkp% p_current(1:ndim)=gkp% p_ff(1:ndim)*2.0_t_real
    !!
    gkp% ip=1_t_int
    gkp% jh=0_t_int
    gkp% k=2_t_int
    !!************ L1T ********************
105 Continue
    If(gkp% k > 8_t_int) Then
       calc=.False.
       gkp% conv=.False.
       Return 
    Endif
    gkp% p_old(1:ndim)=gkp% p_current(1:ndim)
    gkp% p_current(1:ndim)=0.0_t_real
    !!
    Do kk = kx_d(gkp% k-1)+1, kx_d(gkp% k)
       Do ia=1,ndim
          gkp% p_current(ia)=gkp% p_current(ia)&
               &+Dot_product(p_d(gkp% ip:gkp% ip+kh_d(kk)-kl_d(kk))&
               &,gkp% p_work(kl_d(kk):kh_d(kk),ia))
       Enddo
       gkp% ip=gkp% ip+kh_d(kk)-kl_d(kk)+1
    Enddo
    gkp% jl=gkp% jh+1
    gkp% jh=gkp% jl+gkp% jl-1
    gkp% j1=fk_d(gkp% k)
    gkp% j2=fh_d(gkp% k)
    gkp% j=gkp% jl
    !!************ L2T ********************
205 Continue     
    If(gkp% j > gkp% jh) Then
       !!************ L2X ********************
       gkp% p_current(1:ndim)=gkp% p_current(1:ndim)+0.5_t_real&
            &*gkp% p_old(1:ndim)
       gkp% p_res(1:ndim)=gkp% p_current(1:ndim)

       !!
       If (gkp% k >= gkp% kmin .And. &
            &All(Abs(gkp% p_current(1:ndim)-gkp% p_old(1:ndim))&
            &    <= Max(gkp% epsabs,Abs(gkp% epsrel*gkp% p_res(1:ndim))))&
            &)Then
          calc=.False.
          Return
       End If
       
       gkp% k=gkp% k+1
       Goto 105 !! L1T
    Endif
    Call jac_rx(p_d(gkp% ip)-1.0_t_real,gkp% xm,gkp% hx,r,gkp% rx,ijac&
         &,gkp% typ,gkp% exx)
    gkp% state=L2B2
    Return
    !!************ L2B2 ********************
220 Continue
    gkp% p_fval(1:ndim)=gkp% p_ff(1:ndim)
    Call jac_rx(1.0_t_real-p_d(gkp% ip),gkp% xm,gkp% hx,r,gkp% rx,ijac&
         &,gkp% typ,gkp% exx)
    gkp% state=L2B3
    Return
    !!************ L2B3 ********************
230 Continue
    gkp% p_fval(1:ndim)=gkp% p_fval(1:ndim)+gkp% p_ff(1:ndim)
    gkp% p_current(1:ndim)=gkp% p_current(1:ndim)+p_d(gkp% ip+1)&
         &*gkp% p_fval(1:ndim)
    If (gkp% j1 <= gkp% j2) Then
       gkp% p_work(gkp% j1,1:ndim)=gkp% p_fval(1:ndim)
       gkp% j1=gkp% j1+1
    End If
    gkp% ip=gkp% ip+2
    gkp% j=gkp% j+1
    Goto 205 !!L2T
    !!
  End Subroutine quad_gkp
  !!***********************************************************************
  Subroutine  jac_rx(x,xm,hx,r,rx,ijac,Type,exx)
    !!
    Implicit none 
    !!IO
    Integer(t_int),Intent(in)    :: Type
    Real(t_real),Intent(in)      :: x,xm,hx,exx
    Real(t_real),Intent(out)     :: r,rx,ijac
    !!LOCAL
    !!
    Select Case(Int(Type))
    Case (Int(IJAC_TYPE_LIN))
       rx=(xm+x)*hx
       r=rx
       ijac=hx
    Case (Int(IJAC_TYPE_LOG))
       rx=(xm+x)*hx
       r=Exp(rx)
       ijac=r*hx
    Case (Int(IJAC_TYPE_POW))
       rx=(xm+x)*hx
       r=rx**exx
       ijac=exx*rx**(exx-1_t_real)*hx
    Case (Int(IJAC_TYPE_SIN))
       rx=(xm+x)*hx
       r=(xm+Sin(pi2*x))*hx
       ijac=pi2*Cos(pi2*x)*hx
    End Select
    !!
  End Subroutine jac_rx
  !!***********************************************************************
  Subroutine  jac_bound(b0,b1,a,b,type,exx)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: Type
    Real(t_real),Intent(in)      :: b0,b1,exx
    Real(t_real),Intent(out)     :: a,b
    !!LOCAL
    !!
    Select Case(Int(Type))
    Case (Int(IJAC_TYPE_LIN))
       a=b0
       b=b1
    Case (Int(IJAC_TYPE_LOG))
       a=Log(b0)
       b=Log(b1)
    Case (Int(IJAC_TYPE_POW))
       a=b0**(1.0_t_real/exx)
       b=b1**(1.0_t_real/exx)
    Case (Int(IJAC_TYPE_SIN))
       a=b0
       b=b1
    Case default
       a=b0
       b=b1
    End Select
    !!
  End Subroutine jac_bound
  !***********************************************************************
  Subroutine clag4int(n,h,c)
    !!
    !! integration weights for 4-th Lagrange-interpolation
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: n    ! number of points
    Real(t_real),Intent(in)   :: h    ! step width
    Real(t_real),Intent(out)  :: c(n) ! weights
    !!LOCAL
    Character(*),Parameter::nsubfunc="clag4int"
    Integer(t_int) ::  k,j,ir
    !
    !***********************************************************************
    !
    !
    ! n= 1+4*j+k,   k=[0-3]
    ! k=mod(n-1,4)
    ! j=(n-k-1)/4
    !
    k=Mod(n-1_t_int,4_t_int) 
    j=(n-k-1_t_int)/4_t_int  
    !
    If(n<5_t_int)&
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<5!')
    !
    c(1:n)=0.0_t_real
    Do ir=2,4*j-2,4
       c(ir)=64.00_t_real/45.0_t_real*h
    Enddo
    Do ir=3,4*j-1,4
       c(ir)=8.00_t_real/15.0_t_real*h
    Enddo
    Do ir=4,4*j-0,4
       c(ir)=64.00_t_real/45.0_t_real*h
    Enddo
    Do ir=5,4*j-3,4
       c(ir)=28.00_t_real/45.0_t_real*h
    Enddo
    c(1)=c(1)+14.00_t_real/45.0_t_real*h
    c(1+4*j)=c(1+4*j)+14.00_t_real/45.0_t_real*h
    
    If(k == 1_t_int)Then
       c(n-4)=c(n-4)-19.0_t_real/720.0_t_real*h
       c(n-3)=c(n-3)+ 53.0_t_real/360.0_t_real*h
       c(n-2)=c(n-2)- 11.0_t_real/30.0_t_real*h
       c(n-1)=c(n-1)+323.0_t_real/360.0_t_real*h
       c(n-0)=c(n-0)+251.0_t_real/720.0_t_real*h
    Else if(k == 2_t_int)Then
       c(n-4)=c(n-4)- 1.0_t_real/90.0_t_real*h
       c(n-3)=c(n-3)+ 2.0_t_real/45.0_t_real*h
       c(n-2)=c(n-2)+ 4.0_t_real/15.0_t_real*h
       c(n-1)=c(n-1)+62.0_t_real/45.0_t_real*h
       c(n-0)=c(n-0)+29.0_t_real/90.0_t_real*h
    Else If(k == 3_t_int)Then
       c(n-4)=c(n-4)- 3.0_t_real/80.0_t_real*h
       c(n-3)=c(n-3)+21.0_t_real/40.0_t_real*h
       c(n-2)=c(n-2)+ 9.0_t_real/10.0_t_real*h
       c(n-1)=c(n-1)+51.0_t_real/40.0_t_real*h
       c(n-0)=c(n-0)+27.0_t_real/80.0_t_real*h
    Endif
    
    Return
  End Subroutine clag4int
  !!***********************************************************************
  Subroutine cjlag4int(n,jr,h,c)
    !!
    !! integration via 4-th Lagrange-interpolation
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: n     ! number of points
    Real(t_real),Intent(in)   :: h     ! step width
    Real(t_real),Intent(in)   :: jr(n) ! jacobian of the mesh
    Real(t_real),Intent(out)  :: c(n)  ! weights
    !!LOCAL
    Character(*),Parameter::nsubfunc="cjlag4int"
    Integer(t_int) ::  k,j,ir
    !
    !***********************************************************************
    !
    !
    ! n= 1+4*j+k,   k=[0-3]
    ! k=mod(n-1,4)
    ! j=(n-k-1)/4
    !
    k=Mod(n-1_t_int,4_t_int) 
    j=(n-k-1_t_int)/4_t_int  
    !
    If(n<5_t_int)&
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//': called with n<5!')
    !
    c(1:n)=0.0_t_real
    Do ir=2,4*j-2,4
       c(ir)=64.00_t_real/45.0_t_real*h*jr(ir)
    Enddo
    Do ir=3,4*j-1,4
       c(ir)=8.00_t_real/15.0_t_real*h*jr(ir)
    Enddo
    Do ir=4,4*j-0,4
       c(ir)=64.00_t_real/45.0_t_real*h*jr(ir)
    Enddo
    Do ir=5,4*j-3,4
       c(ir)=28.00_t_real/45.0_t_real*h*jr(ir)
    Enddo
    c(1)=c(1)+14.00_t_real/45.0_t_real*h*jr(1)
    c(1+4*j)=c(1+4*j)+14.00_t_real/45.0_t_real*h*jr(1+4*j)
    
    If(k == 1_t_int)Then
       c(n-4)=c(n-4)- 19.0_t_real/720.0_t_real*h*jr(n-4)
       c(n-3)=c(n-3)+ 53.0_t_real/360.0_t_real*h*jr(n-3)
       c(n-2)=c(n-2)- 11.0_t_real/30.0_t_real*h *jr(n-2)
       c(n-1)=c(n-1)+323.0_t_real/360.0_t_real*h*jr(n-1)
       c(n-0)=c(n-0)+251.0_t_real/720.0_t_real*h*jr(n-0)
    Else if(k == 2_t_int)Then
       c(n-4)=c(n-4)- 1.0_t_real/90.0_t_real*h*jr(n-4)
       c(n-3)=c(n-3)+ 2.0_t_real/45.0_t_real*h*jr(n-3)
       c(n-2)=c(n-2)+ 4.0_t_real/15.0_t_real*h*jr(n-2)
       c(n-1)=c(n-1)+62.0_t_real/45.0_t_real*h*jr(n-1)
       c(n-0)=c(n-0)+29.0_t_real/90.0_t_real*h*jr(n-0)
    Else If(k == 3_t_int)Then
       c(n-4)=c(n-4)- 3.0_t_real/80.0_t_real*h*jr(n-4)
       c(n-3)=c(n-3)+21.0_t_real/40.0_t_real*h*jr(n-3)
       c(n-2)=c(n-2)+ 9.0_t_real/10.0_t_real*h*jr(n-2)
       c(n-1)=c(n-1)+51.0_t_real/40.0_t_real*h*jr(n-1)
       c(n-0)=c(n-0)+27.0_t_real/80.0_t_real*h*jr(n-0)
    Endif
    
    Return
  End Subroutine cjlag4int
  !!***********************************************************************
End Module integrate1d

