Module ioadapter
  !!Important: One can call the multidimensional array
  !! functions with array sections of proper dimensions, but then
  !! the element order is wrong!!
  !! similar holds for character subsections of whole arrays, like c(1:3)(1:4)
  !!Example:
  !!   fortran type: integer(t_int): ita3(4,5,6)
  !!   input type  : int ita3[2,3,4]={...};
  !!   call getinteger("ita3",ita3(2:3,3:5,2:5)) will give no error message
  !!   but the data are stored as if ita3 would be ita3(2,3,4) which results
  !!   in unexpected assignements!
  !!Only call it with full arrays !!!
  !!Important: the functions act like an assignement
  !!   if the last dimension of the array argument is larger than
  !!   the input data types last dimension, the trailing elements are
  !!   padded with type specific zeros. Thus all elements which where
  !!   in the array before the call are lost!
  !!Exception: One can get a scalar data using an array dummy argumenr
  !!           than no padding takes place. The data ar written into 
  !!           the first element of the array argument
  Use precisiondefs
  Use errors
  !!
  Implicit None 
  !!
  Private
  Public :: &
       ! control function
       &sizetest&
       ! start end parser
       &,initparser,readfile,cleanreadfile,setparsestop&
       &,addexternalrealarray&
       &,addexternalintegerarray&
       ! size query functions
       &,getrankandsize&
       &,getsizearray  &
       ! query functions
       &,getinteger,getreal,getchar,getlogical,getflag,getstring&
       &,getrealasstring,variable_exists&
       ! manipulation
       &,writeParsFile,setParserVarValue,mkdirectory

  Interface setParserVarValue
     Module Procedure setParserVarValueS,setParserVarValueL
  End Interface setParserVarValue
  

  Interface addexternalarray
     Module Procedure addexternalrealarray,addexternalintegerarray
  End Interface
  
  Interface getsizearray
     Module Procedure getsizearray0,getsizearray1,getsizearray2  
  End Interface
  
       
  Interface getinteger
     Module Procedure getintegerscalar,getintegerarray1d,getintegerarray2d&
       &,getintegerarray3d,getintegerarray4d,getintegerarray5d
  End Interface

  Interface getreal
     Module Procedure getrealscalar,getrealarray1d,getrealarray2d&
          &,getrealarray3d,getrealarray4d,getrealarray5d
  End Interface

  Interface getrealasstring
     Module Procedure getrealasstringscalar,getrealasstringarray1d&
          &,getrealasstringarray2d,getrealasstringarray3d&
          &,getrealasstringarray4d,getrealasstringarray5d
  End Interface
  
  Interface getlogical
     Module Procedure getlogicalscalar,getlogicalarray1d,getlogicalarray2d&
          &,getlogicalarray3d,getlogicalarray4d,getlogicalarray5d
  End Interface

  Interface getchar
     Module Procedure getcharscalar,getchararray1d,getchararray2d&
          &,getchararray3d,getchararray4d,getchararray5d
  End Interface

  Interface getflag
     Module Procedure getflagscalar,getflagarray1d,getflagarray2d&
          &,getflagarray3d,getflagarray4d,getflagarray5d
  End Interface

  Interface getstring
     Module Procedure getstringscalar,getstringarray1d &
          &,getstringarray2d&
          &,getstringarray3d,getstringarray4d,getstringarray5d
  End Interface
     
  !!***********************************************************************
  Character(*),Parameter::nmodule="IOADAPTER"  
  ! this constants are also defined in vartable.h
  ! you cannot change them independently, otherwise  it crashes
  Integer(t_int),Parameter :: VTYPE_INT =2_t_int**0_t_int
  Integer(t_int),Parameter :: VTYPE_REAL=2_t_int**1_t_int
  Integer(t_int),Parameter :: VTYPE_LOGICAL=2_t_int**2_t_int
  Integer(t_int),Parameter :: VTYPE_FLAG=2_t_int**3_t_int
  Integer(t_int),Parameter :: VTYPE_CHAR=2_t_int**4_t_int
  Integer(t_int),Parameter :: VTYPE_STRING=2_t_int**5_t_int

Contains 
  !!***********************************************************************
  Subroutine getrankandsize(name,rank,asize)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Integer(t_int),Intent(out) :: rank
    Integer(t_int),Intent(out) :: asize(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrankandsize"
    Logical(t_log)               :: err
    External cgetarrayrankandsize
    !!
    If(Size(asize)<7_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg='"dummy argument asize must have size>=7"')
    End If
    
    Call cgetarrayrankandsize(name,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    
  End Subroutine getrankandsize
  !!***********************************************************************
  Subroutine getsizearray0(name,sizearray)
    Implicit None 
    !!IO
    Character(*),Intent(in)  :: name
    Integer(t_int),Intent(out) :: sizearray
    !!LOCAL
    Character(*),Parameter::nsubfunc="getsizearray0"
    Logical(t_log)               :: err
    Integer(t_int)         ::rank,asize(7)
    Integer(t_int)  :: lsizearray(1)
    External cgetsizearray
    !!
Print*,nsubfunc
stop
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetsizearray(name,rank,asize,lsizearray,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    sizearray=lsizearray(1)
  End Subroutine getsizearray0
  !!***********************************************************************
  Subroutine getsizearray1(name,sizearray)
    Implicit None 
    !!IO
    Character(*),Intent(in)  :: name
    Integer(t_int),Intent(out) :: sizearray(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getsizearray1"
    Logical(t_log)               :: err
    Integer(t_int)         ::rank,asize(7)
    External cgetsizearray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(sizearray,1)
    Call cgetsizearray(name,rank,asize,sizearray,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    
  End Subroutine getsizearray1
  !!***********************************************************************
  Subroutine getsizearray2(name,sizearray)
    Implicit None 
    !!IO
    Character(*),Intent(in)  :: name
    Integer(t_int),Intent(out) :: sizearray(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getsizearray2"
    Logical(t_log)               :: err
    Integer(t_int)         ::rank,asize(7)
    External cgetsizearray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(sizearray,1)
    asize(2_t_int)=Size(sizearray,2)
    Call cgetsizearray(name,rank,asize,sizearray,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    
  End Subroutine getsizearray2
  !!***********************************************************************
  Subroutine getintegerarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Integer(t_int),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetintarray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getintegerarray1d
  !!***********************************************************************
  Subroutine getintegerarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Integer(t_int),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetintarray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getintegerarray2d
  !!***********************************************************************
  Subroutine getintegerarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Integer(t_int),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetintarray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getintegerarray3d
  !!***********************************************************************
  Subroutine getintegerarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Integer(t_int),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetintarray
    !!
    rank=4_t_int
    Asize(1_t_int)=Size(array,1)
    Asize(2_t_int)=Size(array,2)
    Asize(3_t_int)=Size(array,3)
    Asize(4_t_int)=Size(array,4)
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getintegerarray4d
  !!***********************************************************************
  Subroutine getintegerarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Integer(t_int),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetintarray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getintegerarray5d
  !!***********************************************************************
  Subroutine getrealarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Real(t_real),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealarray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealarray1d
  !!***********************************************************************
  Subroutine getrealarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Real(t_real),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealarray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealarray2d
  !!***********************************************************************
  Subroutine getrealarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Real(t_real),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealarray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealarray3d
  !!***********************************************************************
  Subroutine getrealarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Real(t_real),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealarray
    !!
    rank=4_t_int
    Asize(1_t_int)=Size(array,1)
    Asize(2_t_int)=Size(array,2)
    Asize(3_t_int)=Size(array,3)
    Asize(4_t_int)=Size(array,4)

    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealarray4d
  !!***********************************************************************
  Subroutine getrealarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Real(t_real),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealarray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealarray5d
  !!***********************************************************************
  Subroutine getlogicalarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Logical(t_log),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getlogicalarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetlogarray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getlogicalarray1d
  !!***********************************************************************
  Subroutine getlogicalarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Logical(t_log),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getlogicalarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetlogarray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)

    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getlogicalarray2d
  !!***********************************************************************
  Subroutine getlogicalarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Logical(t_log),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getlogicalarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetlogarray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)

    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getlogicalarray3d
  !!***********************************************************************
  Subroutine getlogicalarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Logical(t_log),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getlogicalarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetlogarray
    !!
    rank=4_t_int
    Asize(1_t_int)=Size(array,1)
    Asize(2_t_int)=Size(array,2)
    Asize(3_t_int)=Size(array,3)
    Asize(4_t_int)=Size(array,4)

    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getlogicalarray4d
  !!***********************************************************************
  Subroutine getlogicalarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Logical(t_log),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getlogicalarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetlogarray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)

    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getlogicalarray5d

  !!***********************************************************************
  Subroutine getchararray1d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Character(*),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getchararray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getchararray1d
  !!***********************************************************************
  Subroutine getchararray2d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getchararray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getchararray2d
  !!***********************************************************************
  Subroutine getchararray3d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getchararray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getchararray3d
  !!***********************************************************************
  Subroutine getchararray4d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getchararray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=4_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getchararray4d
  !!***********************************************************************
  Subroutine getchararray5d(name,array)
    Implicit None 
    !!IO
    Character(*),intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getchararray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getchararray5d
  !!***********************************************************************
  Subroutine getstringarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getstringarray1d
  !!***********************************************************************
  Subroutine getstringarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getstringarray2d
  !!***********************************************************************
  Subroutine getstringarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getstringarray3d
  !!***********************************************************************
  Subroutine getstringarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=4_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getstringarray4d
  !!***********************************************************************
  Subroutine getstringarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getstringarray5d
  !!***********************************************************************
  
  Subroutine getflagarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getflagarray1d
  !!***********************************************************************
  Subroutine getflagarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getflagarray2d
  !!***********************************************************************
  Subroutine getflagarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getflagarray3d
  !!***********************************************************************
  Subroutine getflagarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=4_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getflagarray4d
  !!***********************************************************************
  Subroutine getflagarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetchararray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getflagarray5d
  !!***********************************************************************
  Subroutine getintegerscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: name
    Integer(t_int),Intent(out) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getintegerscalar"
    Logical(t_log)         :: err
    Integer(t_int)         :: rank,asize(7)
    Integer(t_int)         :: array(1)
    External cgetintarray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetintarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
    
  End Subroutine getintegerscalar
  !!***********************************************************************

  Subroutine getrealscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Real(t_real),Intent(out) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealscalar"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    Real(t_real)  :: array(1)
    External cgetrealarray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetrealarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getrealscalar
  !!***********************************************************************
  Subroutine getcharscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Character(*),Intent(inout) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getcharscalar"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    Character(Len(value))  :: array(1)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetchararray(name,array,VTYPE_CHAR,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getcharscalar
  !!***********************************************************************

  Subroutine getlogicalscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Logical(t_log), Intent(out)     :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getcharscalar"
    Logical(t_log)         :: err
    Integer(t_int)         :: rank,asize(7)
    Logical(t_log)         :: array(1)
    External cgetlogarray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetlogarray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getlogicalscalar
!!***********************************************************************

  Subroutine getflagscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Character(*),Intent(inout) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getflagscalar"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    Character(Len(value))  :: array(1)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetchararray(name,array,VTYPE_FLAG,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getflagscalar
  !!***********************************************************************
  Subroutine getstringscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Character(*),Intent(inout) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getstringscalar"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    Character(Len(value))  :: array(1)
    External cgetchararray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetchararray(name,array,VTYPE_STRING,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getstringscalar
  !!***********************************************************************
  Subroutine getrealasstringscalar(name,value)
    Implicit None 
    !!IO
    Character(*),intent(in)  :: name
    Character(*),Intent(inout) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringscalar"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    Character(Len(value))  :: array(1)
    External cgetrealchararray
    !!
    rank=1_t_int
    asize(1_t_int)=1_t_int
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    value=array(1)
  End Subroutine getrealasstringscalar
  !!***********************************************************************
  Subroutine getrealasstringarray1d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringarray1d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealchararray
    !!
    rank=1_t_int
    asize(1_t_int)=Size(array,1)
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealasstringarray1d
  !!***********************************************************************
  Subroutine getrealasstringarray2d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringarray2d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealchararray
    !!
    rank=2_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealasstringarray2d
  !!***********************************************************************
  Subroutine getrealasstringarray3d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringarray3d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealchararray
    !!
    rank=3_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealasstringarray3d
  !!***********************************************************************
  Subroutine getrealasstringarray4d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringarray4d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealchararray
    !!
    rank=4_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealasstringarray4d
  !!***********************************************************************
  Subroutine getrealasstringarray5d(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)    :: name
    Character(*),Intent(inout) :: array(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getrealasstringarray5d"
    Logical(t_log)               :: err
    Integer(t_int)         :: rank,asize(7)
    External cgetrealchararray
    !!
    rank=5_t_int
    asize(1_t_int)=Size(array,1)
    asize(2_t_int)=Size(array,2)
    asize(3_t_int)=Size(array,3)
    asize(4_t_int)=Size(array,4)
    asize(5_t_int)=Size(array,5)
    Call cgetrealchararray(name,array,rank,asize,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine getrealasstringarray5d
  !!***********************************************************************
  Logical(t_log) Function variable_exists(name)
    Implicit None 
    !!IO
    Character(*),Intent(in)   :: name
    !!LOCAL
    Character(*),Parameter::nsubfunc="variable_exists"
    Logical(t_log)         :: err,exists
    External cvarexists
    !!

    Call cvarexists(name,exists,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    variable_exists=exists
    
  End Function variable_exists
  !!***********************************************************************
  Subroutine sizetest()
    !!
    Implicit None 
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="sizetest"
    Integer(t_int) :: ltest(4)
    Integer        :: itest(4)
    character(4)   :: ctest="ABCD"
    Logical(t_log)       :: lotest(4)
    Logical(t_log),parameter    :: t=.true._t_log,f=.false._t_log
    Real(t_real)  :: dtest(4)
    External csizetest
    !!

    Call csizetest(ltest(1_t_int),ltest(4_t_int),itest(1_t_int),itest(4_t_int)&
         &,dtest(1_t_int),dtest(4_t_int)&
         & ,ctest(1_t_int:1_t_int),ctest(4_t_int:4_t_int)&
         &,lotest(1_t_int),lotest(4_t_int)&
         & ,t,f)
    
    !!
  End Subroutine sizetest

  !!***********************************************************************
  Subroutine readfile(name)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: name
    !!LOCAL
    Character(*),Parameter::nsubfunc="readfile"
    Logical(t_log)               ::err 
    External cparsefile
    !!
    Call cparsefile(name,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine readfile
  !!***********************************************************************
  Subroutine setParserVarValueS(var,Value)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: var
    Character(*),intent(in)   :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="setParserVarValueS"
    Logical(t_log)               ::err 
    External csetvalue
    !!
    Call csetvalue(var,Trim(Value),err,errbuf)
    !!
    If(err)Then
       Call errmsg(nmodule&
            &,nsubfunc//"('"//Trim(var)//"','"//Trim(value)//"')"&
            &,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine setParserVarValueS
  !!***********************************************************************
  Subroutine setParserVarValueL(var,Value)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: var
    Logical(t_log),Intent(in) :: value
    !!LOCAL
    Character(*),Parameter::nsubfunc="setParserVarValueL"
    Logical(t_log)               ::err 
    Character(3)   :: str
    External csetvalue
    !!
    Write(str,"(l1)")value
    Call csetvalue(var,Trim(str),err,errbuf)
    !!
    If(err)Then
       Call errmsg(nmodule&
            &,nsubfunc//"('"//Trim(var)//"','"//Trim(str)//"')"&
            &,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine setParserVarValueL
  !!***********************************************************************
  Subroutine writeParsFile(name)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: name
    !!LOCAL
    Character(*),Parameter::nsubfunc="writeParsFile"
    Logical(t_log)               ::err 
    External cwriteparsefile
    !!
    Call cwriteparsefile(name,err,errbuf)
    !!
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name='"//Trim(name)//"')"&
            &,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine writeParsFile
  !!***********************************************************************
  Subroutine initparser
    Implicit None 
    !!IO
    !!LOCAL
    Character(*),Parameter::nsubfunc="initparser"
    Logical(t_log)               ::err 
    External cinitparser
    !!
    Call cinitparser(err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine initparser
  !!***********************************************************************
  Subroutine setparsestop(name)
    Implicit None 
    !!IO
    Character(*),intent(in)   :: name
    !!LOCAL
    Character(*),Parameter::nsubfunc="setparsestop"
    Logical(t_log)               ::err 
    External csetparsestop
    !!
    Call csetparsestop(name,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine setparsestop
  !!***********************************************************************
  Subroutine addexternalrealarray(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)   :: name
    Real(t_real),Intent(in)   :: array(*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="addexternalrealarray"
    Logical(t_log)               ::err 
    External caddexternalrealarray
    !!
    Call caddexternalrealarray(name,array,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine addexternalrealarray
  !!***********************************************************************
  Subroutine addexternalintegerarray(name,array)
    Implicit None 
    !!IO
    Character(*),Intent(in)   :: name
    Integer(t_int),Intent(in)   :: array(*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="addexternalintegerarray"
    Logical(t_log)               ::err 
    External caddexternalintarray
    !!
    Call caddexternalintarray(name,array,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc//"(name="//Trim(name)//")",E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
  End Subroutine addexternalintegerarray
  !!***********************************************************************
  Subroutine cleanreadfile()
    Implicit None 
    !!IO
    !!LOCAL
    Character(*),Parameter::nsubfunc="cleanreadfile"
    Logical(t_log)               ::err 
    External ccleanparsefile
    !!
    Call ccleanparsefile(err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
           
  End Subroutine cleanreadfile

  
  !!***********************************************************************
  Subroutine mkdirectory(dname)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: dname
    !!LOCAL
    Character(*),Parameter::nsubfunc="mkdirectory"
    Logical(t_log)               ::err 
    External cmkdir
    Character(5000)   :: errm
    !!
    Call cmkdir(dname,err,errm)
    If(err)Then
       Write(errbuf,'("""mkdir ",a,": ",a,"""")')&
            &Trim(dname),Trim(errm)
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    !!
    Return
    !!
  End Subroutine mkdirectory
  !!***********************************************************************


End Module ioadapter
