Module ioaux
  !!
  Use ISO_C_BINDING
  Use precisiondefs
  Use errors
  Use constants
  Use versioncontrol
  Use file_iounits
  Use atomdefs, Only: lqnchar,lcharqn
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="IOAUX"  
  !!
  Public :: write_version,write_status,make_ratio,double_to_string_as_ratio&
       &,write_changed,next_word,uc_compare,char_to_uc,char_to_lc&
       &,intToChar10,intToChar20,stringify_double100,replacechar&
       &,copy_file_to_output,write_codified,copy_file_to_file&
       &,strip_leading_blanks,kapToString,replace_tabs_by_blanks,capitalize&
       &,readOrbitalDescription,T_OrbitalDescriptor&
       &,ORB_INVALID,ORB_ALL,ORB_ALLNLM,ORB_NL,ORB_NLJ,ORB_NLM,ORB_NLMS,ORB_NLJM&
       &,palloc_orbitaldescriptor,pfree_orbitaldescriptor,orbDescToString&
       &,orbDescEqual,delete_existing_file,setFPLOPrintFunc
  !!***********************************************************************,
  Type T_OrbitalDescriptor
     Integer(t_int) :: Type !! ORB_...
     Integer(t_int) :: nmain
     Integer(t_int) :: l
     Integer(t_int) :: j
     Integer(t_int) :: kap
     Integer(t_int) :: m
     Integer(t_int) :: ms !!  for FREL need spin
  End Type T_OrbitalDescriptor
  !!
  Integer(t_int),Parameter :: ORB_INVALID= 0_t_int
  Integer(t_int),Parameter :: ORB_ALL    = 1_t_int
  Integer(t_int),Parameter :: ORB_NL     = 2_t_int
  Integer(t_int),Parameter :: ORB_NLJ    = 3_t_int
  Integer(t_int),Parameter :: ORB_NLM    = 4_t_int
  Integer(t_int),Parameter :: ORB_NLJM   = 5_t_int
  Integer(t_int),Parameter :: ORB_ALLNLM = 6_t_int
  Integer(t_int),Parameter :: ORB_NLMS   = 7_t_int
contains 
  !!***********************************************************************
  Subroutine palloc_orbitaldescriptor(name,od,N1,N2,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N1,N2
    Type(T_OrbitalDescriptor),Pointer         :: od(:,:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    !Character(*),Parameter::nsubfunc="palloc_orbitaldescriptor"
    Integer :: icheck
    !!
    If(N1<=0 .or. N2<=0) Return
    !!
    Call pfree_orbitaldescriptor(od)
    !!
    Allocate(od(1:N1,1:N2),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
!!$    Do i=1,N
!!$       Call init_grid(od(i))
!!$    Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_orbitaldescriptor
  !!***********************************************************************
  Subroutine pfree_orbitaldescriptor(od)
    Implicit None 
    !!IO
    Type(T_OrbitalDescriptor),Pointer         :: od(:,:)
    !!LOCAL
    External crealmemaddress
    !!
    !!
    If(Associated(od))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
!!$       Do i=1,Size(od)
!!$          Call cleanup_grid(od(i))
!!$       Enddo
       !!
       Deallocate(od)
       Nullify(od)
    Endif
    !!
  End Subroutine pfree_orbitaldescriptor
  !!***********************************************************************
  ! write_status
  ! {{{ 

  Subroutine write_status(lunit,status)
    !!
    implicit none 
    !!IO
    Integer(t_int),Intent(in) :: lunit
    Character(*),Intent(in)   :: status
    !!LOCAL
    !Character(*),Parameter::nsubfunc="write_status"

1   Format('  ',a,a,a,/)
    Write(lunit,1) 'char[*] status="',Trim(status),'";'    
    
  End Subroutine write_status

  ! }}}

  !!***********************************************************************
  ! write_version
  ! {{{ 

  Subroutine write_version(lunit)

    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)  :: lunit 
    !!LOCAL
    !Character(*),Parameter::nsubfunc="write_version"

1   Format(/,'    struct {char[*] mainversion;char[*] subversion;} version',/&
         &,'    ={"',a,'","',a,'"};',/)
    Write(lunit,1) Trim(FPLOMAINVERSION),Trim(FPLOSUBVERSION)
    
  End Subroutine write_version
  ! }}}
  !!***********************************************************************
  !!***********************************************************************
  ! write_version
  ! {{{ 

  Subroutine write_changed(lunit)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)  :: lunit 
    !!LOCAL
    Character(*),Parameter::nsubfunc="write_changed"
    Character(400) :: date,host
    Logical(t_log) :: err
    External cgetdate,cgethostname
    !!
    !!
    Call cgetdate(date,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    Call cgethostname(host,err,errbuf)
    If(err)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg='"'//Trim(errbuf)//'"')
    End If
    
    !!
    
1   Format(/,'    struct {char[*] hostname;char[*] date;char[*] by;} last_changes') 
    Write(lunit,1) 
2   Format('    ={"',a,'","',a,'","',a,' (',a,'-',a,')"};',/)
    Write(lunit,2) Trim(host),Trim(date),Trim(APPLICATIONNAME)&
         &,Trim(FPLOMAINVERSION),Trim(FPLORELEASE)
    
  End Subroutine write_changed
  ! }}}
  !!***********************************************************************
  !! decompose d into m/n upto n=maxn With tolerance tol
  !! if n==0, no fractional result
  !! if n==1, integer result
  !! else d==m/n
  !!***********************************************************************
  
  Subroutine make_ratio(d,m,n,maxn,tol)
    !!
    Implicit None 
    !!IO
    Real(t_real),Intent(in)     :: d,tol
    Integer(t_int),Intent(in)    :: maxn
    Integer(t_int),Intent(out)   :: m,n
    !!LOCAL    
    Integer(t_int) :: i,s
    Real(t_real)  :: da
    !!
    da=Abs(d)
    s=Nint(Sign(1.0_t_real,d),t_int)
  
    n=0_t_int
    m=0_t_int
    Do i=1_t_int,maxn
       m=Nint(da*i,t_int)
       If(Abs(Real(m,t_real)/i-da)<tol)Then
          n=i
          Exit
       End If
    End Do
    m=m*s
  End Subroutine make_ratio
  !!***********************************************************************
  Subroutine double_to_string_as_ratio(d,str,maxn,tol,fmt)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: maxn
    Real(t_real),Intent(in)     :: d,tol
    Character(*),Intent(in)     :: fmt
    Character(*),Intent(out)    :: str
    !!LOCAL
    Character(2000)  :: mstr,nstr
    Integer(t_int)    :: m,n
    !!
    Call make_ratio(d,m,n,maxn,tol)
    If(n==0_t_int)Then
       Write(mstr,fmt)d
       Write(str,1) Trim(Adjustl(mstr))
    Else If(n==1_t_int)Then
       Write(mstr,2)m
       Write(str,1) Trim(Adjustl(mstr))
    Else
       Write(mstr,2)m
       Write(nstr,2)n
       Write(str,3) Trim(Adjustl(mstr)),Trim(Adjustl(nstr))
    Endif
1   Format(a)
2   Format(i15)
3   Format(a,'/',a)
  End Subroutine double_to_string_as_ratio
  !!***********************************************************************
  !! It returns .False., if 'buf' contains only space
  !! or .True. and the next non-space word of 'buf' in 'word'.
  !! 
  Logical(t_log) Function next_word(buf,word)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(inout)   :: buf
    Character(*),Intent(out)     :: word
    !!LOCAL
    Integer(t_int) :: l,i0,i1
    !!
    !Print "(72('-'))"
    !Print "('buf=<'a,'>')",Trim(buf)
    word=''
    next_word=.False.
    !!
    l=Len(buf)
    i0=1
    Do While (buf(i0:i0)==' '.Or.buf(i0:i0)==Char(9))
       i0=i0+1
       If(i0>l) Return
    Enddo
    !!
    !Print "('buf(i0:)=<'a,'>')",Trim(buf(i0:))
    !!
    i1=i0
    Do While (buf(i1:i1)/=' '.And.buf(i1:i1)/=Char(9).And.i1<=Len(buf))
       i1=i1+1
    Enddo
    i1=i1-1
    !!
    !Print "('buf(i0:i1)=<'a,'>')",Trim(buf(i0:i1))
    !Print "('buf(i1:)=<'a,'>')",Trim(buf(i1:))
    !!
    word=buf(i0:i1)
    !!
    !ch1=Adjustl(buf)
    !Print *,'ch1=<',Trim(ch1),'>'
    !l=Index(ch1,' ')-1
    !Print *,'l=<',l,'>'
    !If(l<=0) Return
    !!
    !word=ch1(1:l)
    !Print "('word=<'a,'>')",Trim(word)
    buf=buf(i1+1:)
    next_word=.True.
    !!
    !Print "(72('-'))"
    Return
  End Function next_word
  !!***********************************************************************
  !! This function is limited to the buf of size 5000.
  !! It returns .False., if 'buf' contains only space
  !! or .True. and the next non-space word of 'buf' in 'word'.
  !! 
  Logical(t_log) Function next_word_old(buf,word)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(inout)   :: buf
    Character(*),Intent(out)     :: word
    !!LOCAL
    Character(5000)   :: ch1
    Integer(t_int) :: l
    !!
    word=''
    next_word_old=.False.
    !!
    ch1=Adjustl(buf)
    !Print *,'ch1=<',Trim(ch1),'>'
    l=Index(ch1,' ')-1
    !Print *,'l=<',l,'>'
    If(l<=0) Return
    !!
    word=ch1(1:l)
    !Print *,'word=<',Trim(word),'>'
    buf=ch1(l+1:)
    next_word_old=.True.
    !!
    Return
  End Function next_word_old
  !!***********************************************************************
  Logical(t_log) Function uc_compare(str1,str2)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(in)      :: str1,str2
    !!LOCAL
    Integer(t_int) :: len,i
    Character(1)   :: ch1,ch2
    !!
    uc_compare=.False.
    !!
    len=Len_trim(str1)
    If(len /= Len_trim(str2) ) Then
       Return
    Endif
    !!
    Do i=1,len
       ch1=str1(i:i)
       ch2=str2(i:i)
       If(char_to_uc(ch1)  /= char_to_uc(ch2))Then
          Return
       Endif
    Enddo
    !!
    uc_compare=.True.
    !!
    Return
    !!
  End Function uc_compare
  !!***********************************************************************
  Character(1) Function char_to_uc(ch) 
    !!
    Implicit none  
    !!IO
    Character(1)   :: ch
    !!LOCAL
    External ctoupper
    !!
    char_to_uc=ch
    Call ctoupper(char_to_uc)
    !!
    Return
    !!
  End Function char_to_uc
  !!***********************************************************************
  Character(1) Function char_to_lc(ch) 
    !!
    Implicit none  
    !!IO
    Character(1)   :: ch
    !!LOCAL
    External ctolower
    !!
    char_to_lc=ch
    Call ctolower(char_to_lc)
    !!
    Return
    !!
  End Function char_to_lc
  !!***********************************************************************
  Character(10) Function intToChar10(i)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: i
    !!LOCAL
    !Character(*),Parameter::nsubfunc="intToChar"
    !!
    !!
    Write(intToChar10,1)i
    intToChar10=Adjustl(intToChar10)
    !!
    Return
    !!
1   Format(i10)
    !!
  End Function intToChar10
  !!***********************************************************************
  Subroutine capitalize(str)
    !!
    Implicit None
    !!IO
    Character(*),Intent(inout)   :: str
    !!LOCAL
    !Character(*),Parameter::nsubfunc="capitalize"
    Integer(t_int) :: len,i
    External ctolower
    !!
    len=Len_trim(str)
    Call ctoupper(str(1:1))
    Do i=2,len
       Call ctolower(str(i:i))
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine capitalize
  !!***********************************************************************
  Character(20) Function intToChar20(i)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: i
    !!LOCAL
    !Character(*),Parameter::nsubfunc="intToChar"
    !!
    !!
    Write(intToChar20,1)i
    intToChar20=Adjustl(intToChar20)
    !!
    Return
    !!
1   Format(i20)
    !!
  End Function intToChar20
  !!***********************************************************************
  Character(100) Function stringify_double100(d)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)      :: d
    !!LOCAL
    !Character(*),Parameter::nsubfunc="stringify_double100"
    Integer(t_int) :: k
    !!
    !!
    If(Abs(d)>1.e-8)Then
       Write(stringify_double100,12) d
    Else
       Write(stringify_double100,13) d
    Endif
    stringify_double100=Adjustl(stringify_double100)
    k=Verify(stringify_double100,' ',.True.)
    k=Verify(stringify_double100(1:k),'0',.True.)
    stringify_double100=stringify_double100(1:k)

    If((stringify_double100(k:k)=='+' .Or. stringify_double100(k:k)=='-' ).And. (&
         &stringify_double100(k-1:k-1)=='e'.Or.&
         &stringify_double100(k-1:k-1)=='E'.Or.&
         &stringify_double100(k-1:k-1)=='d'.Or.&
         &stringify_double100(k-1:k-1)=='D'&
         &))then

       stringify_double100=stringify_double100(1:k-2)
       k=k-2
    Endif

    k=Verify(stringify_double100(1:k),'0',.True.)
    stringify_double100=stringify_double100(1:k)

    k=Verify(stringify_double100,' ',.True.)
    If(stringify_double100(k:k) == '.') stringify_double100(k:k)=' '
    !!
    !!
    !!
    Return
    !!
12  Format(f30.15)
13  Format(g30.15)
    !!
    !!
  End Function stringify_double100
  !!***********************************************************************
  Subroutine copy_file_to_output(lun,fname)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: fname
    Integer(t_int),Intent(in)    :: lun
    !!LOCAL
    !Character(*),Parameter::nsubfunc="copy_file_to_output"
    Integer :: eof
    Character(5000)     :: buffer
    !!
    !!
    Write(lun,1) Trim(fname)
    Open(12,file=fname)
    Do 
       Read(12,2,iostat=eof)buffer
       If(eof < 0_t_int) Exit
       Write(lun,2)Trim(buffer)
    Enddo
    Close(12)
    Write(lun,3)Trim(fname)
    !!
    Return
    !!
1   Format(72('-'),/,'Start: content of ',a,/,72('-'))
2   Format(a)
3   Format(72('-'),/,'End  : content of ',a,/,72('-'))
    !!
  End Subroutine copy_file_to_output
  !!***********************************************************************
  Subroutine copy_file_to_file(fnameout,fnamein)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: fnameout
    Character(*),Intent(in)      :: fnamein
    !!LOCAL
    !Character(*),Parameter::nsubfunc="copy_file_to_file"
    Integer :: eof
    Character(5000)     :: buffer
    !!
    !!
    Open(12,file=Trim(fnamein))
    Open(13,file=Trim(fnameout))
    Do 
       Read(12,2,iostat=eof)buffer
       If(eof < 0_t_int) Exit
       Write(13,2)Trim(buffer)
    Enddo
    Close(12)
    Close(13)
    Return
    !!
2   Format(a)
    !!
  End Subroutine copy_file_to_file
  !!***********************************************************************
  Subroutine write_codified(fname,text)
    !!
    Implicit None  
    !!IO
    Character(*),Intent(in)      :: fname
    Character(*),Intent(in)      :: text
    !!LOCAL
    Integer(t_int) :: i,nbs,n,k
    Character(1) :: ch
    Character(2),Parameter :: bs2='\\'
    Character(1) :: bs
    Logical(t_log) :: cr
    !!
    bs=bs2(1:1)
    n=Len(text)
    Open(12,file=Trim(fname))
    i=0
    outerloop: Do While (i<n)
       i=i+1
       ch=text(i:i)
       cr=.False.
       nbs=0
       If(text(i:i)==bs) Then
          nbs=1
          ch=Char(0)
          Do While(i<n)
             i=i+1
             If(text(i:i) /= bs)Then
                ch=text(i:i)
                Exit
             Endif
             nbs=nbs+1
          Enddo
          !!
          If(ch=='n')Then
             If(Mod(nbs,2_t_int)==0)Then
                nbs=nbs/2
             Else
                nbs=(nbs-1)/2
                cr=.True.
             Endif
          Endif
       Endif
       !!
       Do k=1,nbs
          Write(12,2,advance='no')bs
       Enddo
       If(cr)Then
          Write(12,1)
       Else
          If(ch/=Char(0)) Write(12,2,advance='no')ch
       Endif
       !!
    Enddo outerloop
    Write(12,1)
    Close(12)
    !!
    Return
    !!
1   Format()
2   Format(a1)
    !!
  End Subroutine write_codified
  !!***********************************************************************
  Subroutine strip_leading_blanks(text)
    !!
    Implicit None
    !!IO
    Character(*),Intent(inout)   :: text
    !!LOCAL
    !Character(*),Parameter::nsubfunc="strip_leading_blanks"
    Integer(t_int) :: iq,l
    !!
    l=Len_trim(text)
    iq=1
    Do While((text(iq:iq)==' '.Or.text(iq:iq)==Char(9)).And.iq<=l)
       iq=iq+1
    Enddo
    !!
    text=text(iq:)
    !!
    Return
    !!
  End Subroutine strip_leading_blanks
  !!***********************************************************************
  Subroutine replace_tabs_by_blanks(text)
    !!
    Implicit None
    !!IO
    Character(*),Intent(inout)   :: text
    !!LOCAL
    !Character(*),Parameter::nsubfunc="strip_tabs_blanks"
    Integer(t_int) :: iq,l
    !!
    l=Len_trim(text)
    iq=1
    Do iq=1,l
       If(text(iq:iq)==Char(9)) text(iq:iq)=' '
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine replace_tabs_by_blanks
  !!***********************************************************************
  Character(4) Function kapToString(kap)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: kap
    !!LOCAL
    !Character(*),Parameter::nsubfunc="kapToString"
    !!
    Write(kapToString,"(a1,i1,'/2')") &
         &lqnchar((Abs(2*kap+1)-1)/2)&
         &,(2*Abs(kap)-1)
    !!
    Return
    !!
  End Function kapToString
  !!***********************************************************************
  Subroutine readOrbitalDescription(str,od)
    !!
    Implicit None
    !!IO
    Type(T_OrbitalDescriptor),Intent(inout) :: od
    Character(*),Intent(in)      :: str
    !!LOCAL
    !Character(*),Parameter::nsubfunc="readOrbitalDescription"
    !!
    !!
    od% Type=orbitalToInts(str,od% nmain,od% l,od% j,od% m,od% ms)
    !!
    od% kap=od% l
    If(od% j==2*od% l+1) od% kap=-od% l-1
    !!
    Return
    !!
  End Subroutine readOrbitalDescription
  !!***********************************************************************
  Integer(t_int) Function orbitalToInts(orb,nmain,l,j,m,ms)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: orb
    Integer(t_int),Intent(out)   :: nmain,l,j,m,ms
    !!LOCAL
    !Character(*),Parameter::nsubfunc="orbitalToInts"
    Integer(t_int) :: i,i0,j1
    Character(100) :: buf,text
    Character(1)   :: ch1
    Integer ::eof
    !!
    j=-1
    !!
    orbitalToInts=ORB_INVALID
    !!
    !!
    !!Print "('<',a,'>')",Trim('    '//orb)
    !!
    buf=Adjustl('    '//orb)
    i0=1
    Do i=1,Len_trim(buf)
       buf(i0:i0)=buf(i:i)
       If(buf(i:i)/=' ')Then
          i0=i0+1
       Endif
    Enddo
    Do i=i0,Len_trim(buf)
       buf(i:i)=' '
    Enddo
    !!Print "('<',a,'>')",Trim(buf)
    !!
    If(buf(1:3)=='all')Then
       orbitalToInts=ORB_ALL
       return
    Endif
    If(buf(1:6)=='allnlm')Then
       orbitalToInts=ORB_ALLNLM
       return
    Endif
    !!
    Read(buf(1:1),123,iostat=eof) nmain  
    If(eof /=0) Return 
    ch1=char_to_lc(buf(2:2))
    l=lcharqn(ch1,.False.)
    If(l<0.Or.l>ORBITAL_MAX_L) Return
    If(Len_trim(buf(3:))==0)Then
       orbitalToInts=ORB_NL
       Return
    Endif
    !!
    j1=Index(buf(3:),'/')
    If(j1>=1)Then
       text=buf(3:1+j1)
       Read(text,*,iostat=eof) j
       If(eof /=0 ) Return
       text=buf(3+j1:)
       If(text(1:1)/='2') return
       buf=text(2:)
       !!Print "('jbuf<',a,'>',i5)",Trim(buf),j
       If(Mod(j,2_t_int)/=1)Return
       If(j/= 2*l-1 .And. j/=2*l+1)Return
       If(Len_trim(buf)==0)Then
          orbitalToInts=ORB_NLJ
          Return
       Endif
       
       !!
       j1=Index(buf,'/')
       If(j1>=1)Then
          text=buf(1:j1-1)
          Read(text,*,iostat=eof) m
          If(eof /=0 ) Return
          text=buf(j1+1:)
          If(text(1:1)/='2') Return
          buf=text(2:)
          !!Print "('mbuf<',a,'>',i5)",Trim(buf),m
          If(Abs(Mod(m,2_t_int))/=1 .Or. Abs(m)>j)Return
          If(Len_trim(buf)==0)Then
             orbitalToInts=ORB_NLJM
             Return
          Endif
          
       Endif
    Endif
    !!
    j1=scan(buf(3:),'ud')
    !Print *,Trim(buf(3:j1+1)),j1
    If(j1>0)Then
       text=buf(3:j1+1)
       Read(text,*,iostat=eof) m
       If(eof /=0 ) Return
       !Print *,j1,Trim(buf(j1+2:))
       If(buf(j1+2:j1+2)=='u')ms=1
       If(buf(j1+2:j1+2)=='d')ms=2
       orbitalToInts=ORB_NLMS
    Else
       Read(buf(3:),*,iostat=eof) m
       If(eof /=0 ) Return
       orbitalToInts=ORB_NLM
    Endif
    If(Abs(m)>l)Return
    !!
    Return
    !!
123 Format(i1)
    !!
  End Function orbitalToInts  
  !!***********************************************************************
  Character(20) Function orbDescToString(od,outputtype)
    !!
    Implicit None
    !!IO
    Type(T_OrbitalDescriptor),Intent(in) :: od
    Integer(t_int),Intent(in)    :: outputtype
    !!LOCAL
    !Character(*),Parameter::nsubfunc="orbDescToString"
    Character(*),Parameter:: updn='ud'
    !!
    orbDescToString=""
    !!
    If(outputtype==ORB_NL)Then
       Write(orbDescToString,"(i1,a1)")od% nmain,lqnchar(od% l)
    Else If(outputtype==ORB_NLJ)Then
       Write(orbDescToString,"(i1,a1,i1,'/2')")od% nmain,lqnchar(od% l),od%j
    Else If(outputtype==ORB_NLM)Then
       Write(orbDescToString,"(i1,a1,sp,i2,ss)")od% nmain,lqnchar(od% l),od% m
    Else If(outputtype==ORB_NLMS)Then
       Write(orbDescToString,"(i1,a1,sp,i2,ss,1x,a1)")od% nmain,lqnchar(od% l),od% m&
            &,updn(od% ms:od% ms)
    Else If(outputtype==ORB_NLJM)Then
       Write(orbDescToString,"(i1,a1,i1,'/2',sp,i2,ss,'/2')")&
            &od% nmain,lqnchar(od% l),od%j,od% m
    Else
       Call terminate(TERM_CRASH,' orbDescToString: outputtype not implemented. ')
    Endif
    !!
    Return
    !!
  End Function orbDescToString
  !!***********************************************************************
  Logical(t_log) Function orbDescEqual(od1,od2,nom)
    !!
    Implicit None
    !!IO
    Type(T_OrbitalDescriptor),Intent(in) :: od1,od2
    Logical(t_log),Intent(in)    :: nom
    !!LOCAL
    !Character(*),Parameter::nsubfunc="orbDescEqual"
    !!
    !!
    orbDescEqual=.False.
    !!
    If(od1% Type /= od1% type) return
    If(od1% Type == ORB_NL .Or. (od1% Type == ORB_NLM .And. nom))Then
       If(od1% nmain == od2% nmain .And. od1% l == od2% l) Then
          orbDescEqual=.True.
          Return
       Endif
    Else if(od1% Type == ORB_NLM)Then
       If(od1% nmain == od2% nmain .And. od1% l == od2% l  .And. od1% m == od2% m) Then
          orbDescEqual=.True.
          Return
       Endif
    Else if(od1% Type == ORB_NLMS)Then
       If(od1% nmain == od2% nmain .And. od1% l == od2% l  .And. od1% m == od2% m&
            &  .And. od1% ms == od2% ms) Then
          orbDescEqual=.True.
          Return
       Endif
    Else If(od1% Type == ORB_NLJ .Or. (od1% Type == ORB_NLJM .And. nom))Then
       If(od1% nmain == od2% nmain .And. od1% l == od2% l .And.od1% j == od2% j) Then
          orbDescEqual=.True.
          Return
       Endif
    Else If(od1% Type == ORB_NLJM)Then
       If(od1% nmain == od2% nmain .And. od1% l == od2% l .And.od1% j == od2% j&
            &.And.od1% m == od2% m) Then
          orbDescEqual=.True.
          Return
       Endif
    Endif
    !!
    Return
    !!
  End Function orbDescEqual
  !!***********************************************************************
  Subroutine replacechar(str,old,snew)
    !!
    Implicit None
    !!IO
    Character(1),Intent(in) :: old,snew
    Character(*),Intent(inout) :: str
    !!LOCAL
    !Character(*),Parameter::nsubfunc="replacechar"
    Integer(t_int) :: i,l
    !!
    l=Len_trim(str)
    Do i=1,l
       If(str(i:i)==old(1:1))Then
          str(i:i)=snew(1:1)
       Endif
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine replacechar
  !!***********************************************************************
  Subroutine delete_existing_file(lun,fname)
    !!
    Implicit None
    !!IO
    Integer,Intent(in)           :: lun
    Character(*),Intent(in)      :: fname
    !!LOCAL
    !Character(*),Parameter::nsubfunc="delete_existing_file"
    Logical(t_log) :: exists
    Integer :: err
    !!
    Inquire(file=Trim(fname),exist=exists)
    If(.Not.exists) return
    !!
    Open(lun,file=Trim(fname),status='old',iostat=err)
    If(err==0) Close(lun,status='delete')
    !!
    Return
    !!
  End Subroutine delete_existing_file
  !!***********************************************************************
  Integer(t_int) Function indexCS(cstr,ch)
    !! like index but for c-strings
    Implicit None
    !!IO
    Character(kind=c_char,len=1),Intent(in)   :: cstr(*)
    Character(kind=c_char,len=1) :: ch
    !!LOCAL
    !Character(*),Parameter::nsubfunc="indexCS"
    Integer(t_int) :: i
    !!
    !!
    indexCS=0_t_int
    i=1
    Do 
       If(cstr(i)==c_null_char) Then
          indexCS=i
          Return
       Endif
       If(cstr(i)==ch)Then
          indexCS=i
          Return
       Endif
       i=i+1
    Enddo
    !!
    Return
    !!
  End Function indexCS
  !! *********************************************************************
  Subroutine print_c_string(str) Bind(c)
    !! 
    Implicit None
    !!IO
    Character(kind=c_char,len=1),Intent(in)   :: str(*)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_c_string"
    Integer(t_int) :: len
    !!
    !!
    len=indexCS(str,c_null_char)-1
    Call prcstrN(str,len)
    !!
    Return
    !!
  Contains
    !!***********************************************************************
    Subroutine prcstrN(cstr,L)
      !!
      Implicit None
      !!IO
      Character(kind=c_char,len=1),Intent(in)   :: cstr(*)
      Integer(t_int),Intent(in)    :: L
      !!LOCAL
      !Character(*),Parameter::nsubfunc="prcstrN"
      Character(len=L)   :: str
      Integer(t_int) :: i,ll
      !!
      ll=L
      Do i=1,L
         str(i:i)=cstr(i)
      Enddo
      !!
      Do 
         i=Index(str,c_new_line)
         If(i>0)Then
            Write(outfile,"(a)")str(1:i-1)
            str=str(i+1:)
            ll=ll-i
            If(Len_trim(str)==0) Exit
         Else
            Write(outfile,"(a)",advance='no')str(1:ll)
            Exit
         Endif
      Enddo
      !!
      !!
      Return
      !!
    End Subroutine prcstrN
    !!***********************************************************************
  End Subroutine print_c_string
  !! *********************************************************************
!!$  Subroutine print_c_string(str) Bind(c)
!!$    !! this cannot be used with older gfortran compilers :-(
!!$    Implicit None
!!$    !!IO
!!$    Character(kind=c_char,len=1),Intent(in)   :: str(*)
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="print_c_string"
!!$    Character(len=:),Allocatable :: str1
!!$    Integer(t_int) :: i,len
!!$    !!
!!$    !!
!!$    i=1
!!$    Do 
!!$       If(str(i)==c_null_char) Exit
!!$       i=i+1
!!$    Enddo
!!$    len=i-1;
!!$    Allocate(Character(len=len) :: str1)
!!$    str1=Transfer(str(1:len),str1)
!!$
!!$    Do 
!!$       i=Index(str1,c_new_line)
!!$       !Print *,'i=',i
!!$       If(i>0)Then
!!$          Write(outfile,"(a)")str1(1:i-1)
!!$          str1=str1(i+1:)
!!$          If(Len_trim(str1)==0) Exit
!!$       Else
!!$          !Print *,'L',Len_trim(str1),Trim(str1)
!!$          Write(outfile,"(a)",advance='no')str1
!!$          Exit
!!$       Endif
!!$    Enddo
!!$    Deallocate(str1)
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine print_c_string
  !!***********************************************************************
  Subroutine setFPLOPrintFunc()
    !! set print callback for c++ interface
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setFPLOPrintFunc"
    !!
    Interface
       Subroutine FPLOSetPrintFunc(prtfunc) Bind(c,name='FPLOSetPrintFunc')
         Use,Intrinsic :: ISO_C_BINDING
         Type(C_FUNPTR), Intent(IN), Value :: prtfunc
       End Subroutine FPLOSetPrintFunc
    End Interface
    !!
    Call FPLOSetPrintFunc(C_funloc(print_c_string)) 
    !!
    Return
    !!
  End Subroutine setFPLOPrintFunc
  !!***********************************************************************

End Module ioaux
