  !!----DEBUG options-----------------------|
  !! MODULE:SUBFUNC:MODES                   |
  !!----------------------------------------|
  !! ITMETHODS:eigen_analyse:               |
  !! ITMETHODS:clean_x_space:               |
  !! ITMETHODS:clean_x_space:weights        |
  !! ITMETHODS:make_step:call xfplo         |
  !! ITMETHODS:make_step:vectors            |
  !! ITMETHODS:calc_hpoint:                 |
  !! ITMETHODS:calc_hpoint:vectors          |
  !! ITMETHODS:calc_hpoint:eigenvectors     |
  !! ITMETHODS:calc_hpoint:plot xspace      |
  !! ITMETHODS:eigen_analyse_e:             |
  !! ITMETHODS:eigen_analyse_S:             |
  !! ITMETHODS:iterat_minimize:             |
  !! ITMETHODS:iterat_minimize:save_history |
  !! ITMETHODS:iterat_minimize:vectors      |
  !! ITMETHODS:line_search:                 |
  !! ITMETHODS:line_search:plot             |
  !! ITMETHODS:line_search:vectors          |
  !! ITMETHODS:plot_along_line:vectors      |
  !!----------------------------------------|
Module itmethods
  !!
  Use precisiondefs
  Use titeration
  Use constants
  Use file_iounits !! todo remove
  Use ioaux  !! todo remove
  Use memory
  Use diagonalize
  Use matserv
  Use errors
  !!
  !!
  Implicit None 
  !!
  Private 
  !!
  Public :: iterat_minimize
  !!***********************************************************************
  Character(*),Parameter::nmodule="ITMETHODS"
  !!***********************************************************************
  Integer(t_int),Parameter :: BETTER                   = 1_t_int
  Integer(t_int),Parameter :: FIRST_LINESEARCH_FAILED  = 2_t_int
  !!***********************************************************************
  !! keep things together
  Type T_it
     Logical(t_log) :: stepsexceed,converged
     !! the first line search data point
     Integer(t_int) :: L1
     !! the current point, for book keeping purposes
     Integer(t_int) :: XN
     Integer(t_int) :: DX
     !! the hypothetical sub-space minimum
     Integer(t_int) :: XZ1
     Integer(t_int) :: XL1
     Integer(t_int) :: XL2
     Integer(t_int) :: XMU
     Integer(t_int) :: XTMP
     !! maximum allowed dimension of known subspace
     Integer(t_int) :: mdim
     !! dimension of known subspace
     Integer(t_int) :: m
     !! dimension of vector
     Integer(t_int) :: nvec
     !! known x-points
     Real(t_real),Pointer :: xx(:,:)
     !! knwon energies
     Real(t_real),Pointer :: ee(:)
     !! known y-points
     Real(t_real),Pointer :: yy(:,:)
     !! deviations of known points
     Real(t_real),Pointer :: u2(:)
     !! current mixing
     Real(t_real)   :: p
     !! maxstepwidth per dimension
     Real(t_real)   :: stepMax
     !!  tolerance for SVD of x-space
     Real(t_real)   :: xsvdtol
     !! tolerance to decide, which x to remove
     Real(t_real)   :: removextol
     !! below which u2 do we cross over to gradient dominated
     !! algorithm
     Real(t_real)   :: u2crossover
     !! the fraction of the slope*dx which is required as minimum improvement
     Real(t_real)   :: improvtol
  End Type T_it
  !!***********************************************************************
  
  !!***********************************************************************
  
Contains
  !!***********************************************************************
  Subroutine init_it(it,nvec,maxdim,mixing,vec,md,stepmax)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvec,maxdim,md
    Type(T_it),Intent(inout)     :: it
    Real(t_real),Intent(in)      :: vec(:)
    Real(t_real),Intent(in)      :: mixing
    Real(t_real),Intent(in)      :: stepmax
    !!LOCAL
    Character(*),Parameter::nsubfunc="init_it"
    Integer(t_int) :: dim
    !!
    Nullify(it% xx)
    Nullify(it% yy)
    Nullify(it% ee)
    Nullify(it% u2)
    !!
    !!
    it% m=-1
    it% mdim=Min(maxdim,nvec)
    it% XN=it% mdim+1
    it% DX=it% XN+1
    it% XMU=it% DX+1
    it% XL1=it% XMU+1
    it% XL2=it% XL1+1
    it% XTMP=it% XL2+1
    it% XZ1=it% XTMP+1
    dim= it% XZ1
    it% nvec=nvec
    it% p=mixing
    it% stepMax=stepmax
    it% xsvdtol=1.0e-8_t_real
    it% removextol=1.0e-4_t_real
    it% u2crossover=1.0e-2_t_real
    it% stepsexceed=.False.
    it% converged=.False.
    it% improvtol=0.001_t_real
    !!
    Call palloc('it% xx',it% xx,1_t_int,it% nvec,0_t_int,dim&
         &,nmodule,nsubfunc)
    Call palloc('it% ee',it% ee,0_t_int,dim,nmodule,nsubfunc)
    Call palloc('it% yy',it% yy,1_t_int,it% nvec,0_t_int,dim&
         &,nmodule,nsubfunc)
    Call palloc('it% u2',it% u2,0_t_int,dim,nmodule,nsubfunc)
    !!
    !! save initial x in current point
    it% xx(1:it% nvec,md)=vec(1:it% nvec)
    !!
    Return
    !!
  End Subroutine init_it
  !!***********************************************************************
  Subroutine clean_it(it)
    !!
    Implicit None
    !!IO
    Type(T_it),Intent(inout)     :: it
    !!LOCAL
    !Character(*),Parameter::nsubfunc="clean_it"
    !!
    call pfree(it% xx)
    Call pfree(it% yy)
    Call pfree(it% u2)
    Call pfree(it% ee)
    !!
    !!
    Return
    !!
  End Subroutine clean_it
  !!***********************************************************************
  Subroutine save_state(it)
    !!
    Implicit None
    !!IO
    Type(T_it),Intent(inout)     :: it
    !!LOCAL
    !Character(*),Parameter::nsubfunc="save_state"
    Integer(t_int) :: i
    !!
    Open(12,file='=.iterat_minimize')
    !!
    Write(12,1)'m'
    Write(12,2)it% m
    Write(12,1)'p'
    Write(12,3)it% p
    !!
    Write(12,1)'xx'
    Do i=0,it% m
       Write(12,3)it% xx(1:it% nvec,i)
    Enddo
    Write(12,1)'yy'
    Do i=0,it% m
       Write(12,3)it% yy(1:it% nvec,i)
    Enddo
    Write(12,1)'ee'
    Do i=0,it% m
       Write(12,3)it% ee(i)
    Enddo
    Write(12,1)'u2'
    Do i=0,it% m
       Write(12,3)it% u2(i)
    Enddo
    !!
    Close(12)
    !!
    !!
    Return
    !!
1   Format(a)
2   Format(i10)
3   Format(e30.16e3)
  End Subroutine save_state
  !!***********************************************************************
  Subroutine read_state(it)
    !!
    Implicit None
    !!IO
    Type(T_it),Intent(inout)     :: it
    !!LOCAL
    !Character(*),Parameter::nsubfunc="read_state"
    Integer(t_int) :: i
    !!
    Open(12,file='=.iterat_minimize')
    !!
    Read(12,*)
    Read(12,*)it% m
    Read(12,*)
    Read(12,*)it% p
    !!
    Read(12,*)
    Do i=0,it% m
       Read(12,*)it% xx(1:it% nvec,i)
    Enddo
    Read(12,*)
    Do i=0,it% m
       Read(12,*)it% yy(1:it% nvec,i)
    Enddo
    Read(12,*)
    Do i=0,it% m
       Read(12,*)it% ee(i)
    Enddo
    Read(12,*)
    Do i=0,it% m
       Read(12,*)it% u2(i)
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine read_state
  !!***********************************************************************
  Subroutine make_step(it,md,Data,handler,lun)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Integer(t_int),Intent(in)    :: md !! at which point to calculate y(x(md))
    Type(T_it),Intent(inout)     :: it
    Type(T_iteration),Intent(inout)  :: Data
    Procedure(IterationHandler)  ::handler
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_step"
    Real(t_real)   :: ubest,fmax,d
    !!
    Write(lun,1)Trim(data%name)
    !!
    
    data% step=data% step+1
    !! calc corresponding g(x)=y(x)+x
    data% vector(1:it% nvec)=it% xx(1:it% nvec,md)
    Call handler(Data,it_entry_loop)
    !! calc new y(x)=x(:,2)
    it% ee(md)=data% erg
    it% yy(1:it% nvec,md)=data% vector(1:it% nvec)-it% xx(1:it% nvec,md)
    !! and its deviation from zero length
    data% deviation=Sqrt(Sum(it% yy(1:it% nvec,md)**2))
    it% u2(md)=data% deviation
    If(it% u2(md)<=data% tolerance/(hatoev/brtoan)) it% converged=.True.
    !!

    ubest=Min(Minval(it% u2(0:it% m)),it% u2(md))
    !!
    fmax=Maxval(Abs(it% yy(1:it% nvec,md)))
    !!
    If(debugfun(nmodule,nsubfunc,'vectors'))Then
       Write(lun,2)Trim(data%name)
       Write(lun,"(a,' XX(n)=',100(f12.5))")Trim(data%name),it% xx(:,md)
       Write(lun,"(a,' YY(n)=',100(f12.5))")Trim(data%name),it% yy(:,md)
       Write(lun,2)Trim(data%name)
    Endif

    !!
    !!
    Write(lun,2)Trim(data%name)
    Write(lun,4,advance='no')Trim(data%name),data% step,it% m
    d=data% deviation*(hatoev/brtoan)
    If(d<100.0_t_int .And. d>1.0e-6_t_real)Then
       Write(lun,411,advance='no') d
    else
       Write(lun,41,advance='no') d
    Endif
    d=ubest*hatoev/brtoan
    If(d<100.0_t_int .And. d>1.0e-6_t_real)Then
       Write(lun,421,advance='no') d
    else
       Write(lun,42,advance='no') d
    Endif
    Write(lun,43) data% erg
    d=fmax*hatoev/brtoan
    If(d<100.0_t_int .And. d>1.0e-6_t_real)Then
       Write(lun,31)Trim(data%name),d
    Else
       Write(lun,3)Trim(data%name),d
    Endif
    Write(lun,2)Trim(data%name)
    !!
    !!
    If(Data% step>Data% steps)it% stepsexceed=.True.
    !!
    If(debugfun(nmodule,nsubfunc,'call xfplo'))&
         & Call system('xfplo =.sym >/dev/null 2>&1 &')
    !!
    !!
    Return
    !!
1   Format(a,' ------------------------------------------------')
2   Format(a,' ')
4   Format(a,' No=',i4,' m=',i4)
411 Format(' |force| last dev=',f10.6,' eV/A')
41  Format(' |force| last dev=',e14.3e3,' eV/A')
421 Format(' best=',f10.6)
42  Format(' best=',e14.3e3)
43  Format(' E=',f19.10,' Ha')
31  Format(a,' maximum force left=',f10.6,' eV/Ang')
3   Format(a,' maximum force left=',e14.3e3,' eV/Ang')
    !!
  End Subroutine make_step
  !!***********************************************************************
  !! make eigen analysis of 
  !! [ (v_k1-v_k0) ... (v_km-v_k0) ]
  Subroutine eigen_analyse(ww,it,k0,k1,km,D,E,Z,lun,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: lun
    Character(*),Intent(in)      :: ww
    Type(T_it),Intent(inout)     :: it
    Integer(t_int),Intent(in)    :: k0,k1,km
    Real(t_real),Pointer         :: D(:),E(:),Z(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="eigen_analyse"
    Integer(t_int) :: i,j,m,nv
    Real(t_real),Pointer   :: ltr(:,:)
    Real(t_real),Pointer   :: vv(:,:)
    Logical(t_log) :: deb
    !!
    Nullify(ltr)
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    nv=it% nvec
    m=km-k1+1
    !!
    Call palloc('D',D,1_t_int,m,nmodule,nsubfunc)
    Call palloc('E',E,1_t_int,m,nmodule,nsubfunc)
    Call palloc('Z',Z,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    !!
    Call palloc('ltr',ltr,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    vv=>it% yy
    If(ww=='xx') vv=>it% xx
    !!
    Do i=1,m
       ltr(i,i)=Dot_product((vv(1:nv,i+k1-1)-vv(1:nv,k0))&
            &,(vv(1:nv,i+k1-1)-vv(1:nv,k0)))
       D(i)=Sqrt(Abs(ltr(i,i)))
       If(D(i)/=0.0_t_real) Then
          D(i)=1.0_t_real/D(i)
       Else
          D(i)=1.0_t_real
       Endif
    Enddo
    !!
    Do i=1,m
       ltr(i,i)=ltr(i,i)*D(i)**2
       Do j=1,i-1
          ltr(i,j)=Dot_product(&
               & vv(1:nv,i+k1-1)-vv(1:nv,k0)&
               &,vv(1:nv,j+k1-1)-vv(1:nv,k0)&
               &)*D(i)*D(j)
          ltr(j,i)=ltr(i,j)
       Enddo
    Enddo
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,3,advance='no')Trim(mesg)
          Do j=1,m
             Write(lun,4,advance='no') ltr(i,j)
          Enddo
          Write(lun,*)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    Call rdiagno_noO(m,ltr,e)
    Z=ltr
    !!
    !!
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,5)Trim(mesg),ww,i,1.0_t_real/D(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,6)Trim(mesg),ww,i,e(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    !!
    Call pfree(ltr)
    !!
    Return
    !!
2   Format(a,' ')
3   Format(a,' ltr: ')
4   Format(e13.5e3,1x)
5   Format(a,' D',a,'(',i3,')=',e30.15e3)
6   Format(a,' e',a,'(',i3,')=',e30.15e3)
    !!
  End Subroutine eigen_analyse
  !!***********************************************************************
  Subroutine eigen_analyse_e(Syx,it,k0,k1,km,D,E,Z,lun,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    Integer(t_int),Intent(in)    :: k0,k1,km
    Real(t_real),Intent(in)      :: Syx(:,:)
    Real(t_real),Pointer         :: D(:),E(:),Z(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="eigen_analyse_e"
    Integer(t_int) :: i,j,m,nv
    Real(t_real),Pointer   :: ltr(:,:)
    Logical(t_log) :: deb
    !!
    i=k0!! avoid warnings
    Nullify(ltr)
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    !!
    nv=it% nvec
    m=km-k1+1
    !!
    Call palloc('D',D,1_t_int,m,nmodule,nsubfunc)
    Call palloc('E',E,1_t_int,m,nmodule,nsubfunc)
    Call palloc('Z',Z,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    !!
    Call palloc('ltr',ltr,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    Do i=1,m
       ltr(i,i)=Dot_product(Syx(i,:),Syx(i,:))
       D(i)=Sqrt(Abs(ltr(i,i)))
       If(D(i)/=0.0_t_real) Then 
          D(i)=1.0_t_real/D(i)
       Else
          D(i)=1.0_t_real
       Endif
    Enddo
    !!
    Do i=1,m
       ltr(i,i)=ltr(i,i)*D(i)**2
       Do j=1,i-1
          ltr(i,j)=Dot_product(Syx(i,:),Syx(j,:))*D(i)*D(j)
          ltr(j,i)=ltr(i,j)
       Enddo
    Enddo
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,3,advance='no')Trim(mesg)
          Do j=1,m
             Write(lun,4,advance='no') ltr(i,j)
          Enddo
          Write(lun,*)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    Call rdiagno_noO(m,ltr,e)
    Z=ltr
    !!
    !!
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,5)Trim(mesg),i,1.0_t_real/D(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,6)Trim(mesg),i,e(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    !!
    Call pfree(ltr)
    !!
    Return
    !!
2   Format(a,' ')
3   Format(a,' ltr: ')
4   Format(e13.5e3,1x)
5   Format(a,' D Syx*Sxy(',i3,')=',e30.15e3)
6   Format(a,' E Sxy*Sxy(',i3,')=',e30.15e3)
    !!
  End Subroutine eigen_analyse_e
  !!***********************************************************************
  Subroutine eigen_analyse_S(S,it,k0,k1,km,D,E,Z,lun,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    Integer(t_int),Intent(in)    :: k0,k1,km
    Real(t_real),Intent(in)      :: S(:,:)
    Real(t_real),Pointer         :: D(:),E(:),Z(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="eigen_analyse_S"
    Integer(t_int) :: i,j,m,nv
    Real(t_real),Pointer   :: ltr(:,:)
    Logical(t_log) :: deb
    !!
    i=k0!! avoid warnings
    Nullify(ltr)
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    !!
    nv=it% nvec
    m=km-k1+1
    !!
    Call palloc('D',D,1_t_int,m,nmodule,nsubfunc)
    Call palloc('E',E,1_t_int,m,nmodule,nsubfunc)
    Call palloc('Z',Z,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    !!
    Call palloc('ltr',ltr,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    Do i=1,m
       D(i)=Sqrt(Abs(S(i,i)))
       If(D(i)/=0.0_t_real) Then
          D(i)=1.0_t_real/D(i)
       Else
          D(i)=1.0_t_real
       Endif
    Enddo
    !!
    Do i=1,m
       Do j=1,m
          ltr(i,j)=D(i)*S(i,j)*D(j)
       End Do
    End Do
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,3,advance='no')Trim(mesg)
          Do j=1,m
             Write(lun,4,advance='no') ltr(i,j)
          Enddo
          Write(lun,*)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    Call rdiagno_noO(m,ltr,e)
    Z=ltr
    !!
    !!
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,5)Trim(mesg),i,1.0_t_real/D(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,6)Trim(mesg),i,e(i)
       Enddo
       Write(lun,2)Trim(mesg)
       !!
    Endif
    !!
    !!
    Call pfree(ltr)
    !!
    Return
    !!
2   Format(a,' ')
3   Format(a,' ltr: ')
4   Format(e13.5e3,1x)
5   Format(a,' D Sav(',i3,')=',e30.15e3)
6   Format(a,' E Sav(',i3,')=',e30.15e3)
    !!
  End Subroutine eigen_analyse_S
  !!***********************************************************************
  Subroutine clean_x_space(it,Zx,Dx,Ex,lun,mesg)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    Character(*),Intent(in)      :: mesg
    !!LOCAL
    Character(*),Parameter::nsubfunc="clean_x_space"
    Integer(t_int) :: m,i,j,ind
    Logical(t_log) :: indefinite
    Real(t_real),Pointer :: Zx(:,:),Ex(:),Dx(:)
    Real(t_real)   :: emax,cmin,cmax
    Logical(t_log) :: deb,deb2
    !!
    !!
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    deb2=debugfun(nmodule,nsubfunc,'weights')
    !!
    If(deb) Write(lun,6)Trim(mesg)
    !!
    m=it% m
    !!
    Do While(m>=1) 
       !!
       Call eigen_analyse('xx',it,0_t_int,1_t_int,m,Dx,Ex,Zx,lun,mesg)
       !!
       !!
       If(deb)Then
          Do i=1,m
             Write(lun,7) Trim(mesg),i,Ex(i)
             If(deb2)Then
                Do j=1,m
                   Write(lun,8)Trim(mesg),j,Zx(j,i)*Sqrt(Abs(Ex(i)))
                Enddo
             Endif
          Enddo
       End If
       !!
       emax=Maxval(Abs(Ex(1:m)))
       !!
       If(deb) Write(lun,9)Trim(mesg),emax

       If(m<=1) Return
       !!
       indefinite=.False.
       Do i=1,m
          If(Abs(Ex(i))<it% xsvdtol*emax)Then
             indefinite=.True.
          Endif
       Enddo
       !!
       If(.Not.indefinite) Exit
       !!
       i=m
       If(Abs(Ex(i))>it% xsvdtol*emax)Then
          ind=m
          cmin=Abs(Zx(m,i)*Sqrt(Abs(Ex(i))))
          cmax=Maxval(Abs(Zx(1:m,i)*Sqrt(Abs(Ex(i)))))
          Do j=m-1,1,-1
             If( Abs(Zx(j,i)*Sqrt(Abs(Ex(i)))) < cmin)Then
                ind=j
                cmin=Abs(Zx(j,i)*Sqrt(Abs(Ex(i))))
             Endif
          Enddo
          If(cmin<cmax*it% removextol)Then
             if(deb) Write(lun,"(a,' remove weakest ',i3,2(e30.15e3))")&
                  &Trim(mesg),ind,cmin,cmax
             !Stop 'remove weakest'
          Else
             If(deb) Write(lun,10)Trim(mesg),m,cmin,cmax
             it% m=it% m-1
             m=it% m
          Endif
       Endif
    Enddo
    !!
    !!
    Return
    !!
6   Format(a,' clean_x_space ---------------------------------------')
7   Format(a,' Ex(',i3,')=',e30.15e3)
8   Format(a,'    W DX(',i3,')=',e30.15e3)
9   Format(a,' Ex_max=',e30.15e3)
10  Format(a,' remove largest ',i3,2(e30.15e3))
    !!
  End Subroutine clean_x_space
  !!***********************************************************************
  Subroutine limitStep(scale,nvec,xx,stepmax)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nvec
    Real(t_real),Intent(in)      :: stepmax
    Real(t_real),Intent(inout)   :: xx(:)
    Real(t_real),Intent(out)     :: scale
    !!LOCAL
    !Character(*),Parameter::nsubfunc="limitStep"
    Real(t_real)   :: xmax
    !!
    xmax=Maxval(Abs(xx(1:nvec)))
    !!

    

!    Do i=1,nvec
!       xx(i)=Sign(Min(stepMax,Abs(xx(i))),xx(i))
!    Enddo
    scale=1.0_t_real
    If(Any(Abs(xx(1:nvec))>stepMax))Then
       scale=stepMax/xmax
       xx(1:nvec)=xx(1:nvec)*scale
    Endif


    !!
    !!
    Return
    !!
  End Subroutine limitStep
  !!***********************************************************************
  Subroutine show_plot()
    !!
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="show_plot"
    !!
    Open(57,file='SHOW')
    Write(57,111)
    Write(57,112)
    Write(57,113)
    Write(57,114)
    Write(57,115)
    Write(57,116)
    Write(57,117)
    Write(57,118)
    Write(57,119)
111 Format('#! /usr/bin/perl',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'use strict;',/&
         &,'my $data=[ eval `cat tt ttt` ];',/&
         &,'',/&
         &,'#print $data->[0],"\n";',/&
         &,'#exit(0);',/&
         &,'',/&
         &,'my $N=1000;',/&
         &,'my $h=0.03*spread($data);',/&
         &,'',/&
         &,'',/&
         &,'my $i0=0;',/&
         &,'my $i1=1;',/&
         &,'',/&
         &,'my $dx=$data->[$i1]->{x}-$data->[$i0]->{x};',/&
         &,'my $X=$data->[2]->{x};',/&
         &,'$X=($X>$data->[$i1]->{x}? $X :$data->[$i1]->{x});',/&
         &,'my $DX=$X-$data->[$i0]->{x};',/&
         &,'',/&
         &,'#my $dy=$y1-$y0;',/&
         &,'',/&
         &,'',/&
         &,'# shift Y data into visible E-region',/&
         &,'my $Y0=$data->[$i0]->{Y};',/&
         &,'for(my $i=0;$i<=$#$data;$i++){',/&
         &,'  $data->[$i]->{Y}=$data->[$i]->{Y}-$Y0+$data->[$i1]->{E};',/&
         &,'  $data->[$i]->{DY}/=$dx;',/&
         &,'}',/&
         &,'',/&
         &,'open(FO,">t");',/&
         &,'',/&
         &,'my $nS=0;',/&
         &,'my $txt="";',/&
         &,'',/&
         &,'&draw_dots(\*FO,$data,$dx,"E",\$txt,\$nS,1);',/,'',/,'',/)
112 Format(&
         &'&draw_tangents(\*FO,$data,$dx,$h,"E","y",\$txt,\$nS);',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'&draw_quad_y0_dy0_y1(\*FO,$data,$N,$dx,$DX,$i0,$i1,"E","y",\$txt,\$nS,3);',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'&draw_quad_y0_dy0_dy1(\*FO,$data,$N,$dx,$DX,$i0,$i1,"E","y",\$txt,\$nS,8);',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'&draw_cubic(\*FO,$data,$N,$dx,$DX,$i0,$i1,"E","y",\$txt,\$nS,9);',/&
         &,'# $b=$b/$a;',/&
         &,'# $c=$c/$a;',/&
         &,'',/&
         &,'# #my $x1=-($b-sqrt($b*$b-4*$c))/2;',/&
         &,'# #my $x2=-($b+sqrt($b*$b-4*$c))/2;',/&
         &,'',/&
         &,'# #print "x1=$x1 ".($x1*$dx)."\n";',/&
         &,'# #print "x2=$x2 ".($x2*$dx)."\n";',/&
         &,'',/&
         &,'',/&
         &,'',/&
         &,'# draw tangents for the Y curve',/&
         &,'&draw_dots(\*FO,$data,$dx,"Y",\$txt,\$nS,11);',/&
         &,'',/&
         &,'# # draw tangents for the Y curve',/&
         &,'&draw_tangents(\*FO,$data,$dx,$h,"Y","DY",\$txt,\$nS);',/&
         &,'',/&
         &,'&draw_quad_y0_dy0_dy1(\*FO,$data,$N,$dx,$DX,$i0,$i1,"Y","DY",\$txt,\$nS,12);',/&
         &,'',/&
         &,'close(FO);',/&
         &,'',/&
         &,'',/&
         &,'open(fp,"|xmgrace -batch - ");',/&
         &,'print fp <<EOF',/&
         &,'read "t"',/&
         &,'$txt',/)
113 Format(&
         &'autoscale',/&
         &,'EOF',/&
         &,';',/&
         &,'close(fp);',/&
         &,'`rm t`;',/&
         &,'',/&
         &,'',/&
         &,'########################################################################',/&
         &,'sub spread{',/&
         &,'  my $data=shift;',/&
         &,'',/&
         &,'  my $x0=0;',/&
         &,'  my $x1=0;',/&
         &,'  for(my $i=0;$i<=$#$data;$i++){',/&
         &,'    $x0=($x0<$data->[$i]->{x}?$x0:$data->[$i]->{x});',/&
         &,'    $x1=($x1>$data->[$i]->{x}?$x1:$data->[$i]->{x});',/&
         &,'  }',/&
         &,'  return $x1-$x0;',/&
         &,'}',/&
         &,'',/&
         &,'',/&
         &,'########################################################################',/&
         &,'sub draw_dots{',/&
         &,'  my $fo=shift;',/&
         &,'  my $data=shift;',/&
         &,'  my $dx=shift;',/&
         &,'  my $w=shift;',/&
         &,'  my $txt=shift;',/&
         &,'  my $nS=shift;',/&
         &,'  my $color=shift;',/&
         &,'  ',/&
         &,'  my $iwk=&sortdata($data);',/&
         &,'  ',/&
         &,'  for(my $I=0;$I<=$#$data;$I++){',/&
         &,'    my $i=$iwk->[$I];',/&
         &,'    printf($fo "%30.15g %30.15g\n",$data->[$i]->{x}/$dx,$data->[$i]->{$w});',/&
         &,'  }',/&
         &,'  printf($fo "\n\n");',/&
         &,'  $$txt.=sprintf("s%d symbol 1\n",$$nS);',/)
114 Format(&
         &'  $$txt.=sprintf("s%d line color %d\n",$$nS,$color);',/&
         &,'  $$nS++;',/&
         &,'  ',/&
         &,'  my $i=0;',/&
         &,'  printf($fo "%30.15g %30.15g\n",$data->[$i]->{x}/$dx,$data->[$i]->{$w});',/&
         &,'  printf($fo "\n\n");',/&
         &,'  $$txt.=sprintf("s%d symbol 1\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol linewidth 2\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol color 1\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color 0\n",$$nS);',/&
         &,'  $$nS++;',/&
         &,'  $i=1;',/&
         &,'  printf($fo "%30.15g %30.15g\n",$data->[$i]->{x}/$dx,$data->[$i]->{$w});',/&
         &,'  printf($fo "\n\n");',/&
         &,'  $$txt.=sprintf("s%d symbol 1\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol linewidth 2\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol color 2\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color 0\n",$$nS);',/&
         &,'  $$nS++;',/&
         &,'  $i=2;',/&
         &,'  printf($fo "%30.15g %30.15g\n",$data->[$i]->{x}/$dx,$data->[$i]->{$w});',/&
         &,'  printf($fo "\n\n");',/&
         &,'  $$txt.=sprintf("s%d symbol 1\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol linewidth 2\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d symbol color 3\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color 0\n",$$nS);',/&
         &,'  $$nS++;',/&
         &,'',/&
         &,'',/&
         &,'}',/&
         &,'',/&
         &,'########################################################################',/&
         &,'sub draw_tangents{',/&
         &,'  my $fo=shift;',/&
         &,'  my $data=shift;',/&
         &,'  my $dx=shift;',/&
         &,'  my $h=shift;',/&
         &,'  my $w=shift;',/&
         &,'  my $dw=shift;',/)
115 Format(&
         &'  my $txt=shift;',/&
         &,'  my $nS=shift;',/&
         &,'  for(my $i=0;$i<=$#$data;$i++){',/&
         &,'    printf($fo "%30.15g %30.15g\n"',/&
         &,'       ,$data->[$i]->{x}/$dx-$h/$dx',/&
         &,'       ,$data->[$i]->{$w}-$h*$data->[$i]->{$dw});',/&
         &,'    ',/&
         &,'    printf($fo "%30.15g %30.15g\n"',/&
         &,'       ,$data->[$i]->{x}/$dx+$h/$dx',/&
         &,'       ,$data->[$i]->{$w}+$h*$data->[$i]->{$dw});',/&
         &,'    printf($fo "\n\n");',/&
         &,'    $$txt.=sprintf("s%d line color 2\n",$$nS);',/&
         &,'    $$nS++;',/&
         &,'  }',/&
         &,'  ',/&
         &,'}',/&
         &,'',/&
         &,'########################################################################',/&
         &,'sub sortdata{',/&
         &,'  my $data=shift;',/&
         &,'  my $iwk=[];',/&
         &,'  for(my $i=0;$i<=$#$data;$i++){',/&
         &,'    $iwk->[$i]=$i;',/&
         &,'  }',/&
         &,'  for(my $i=0;$i<=$#$data;$i++){',/&
         &,'    for(my $j=$i;$j<=$#$data;$j++){',/&
         &,'      if(',/&
         &,'     $data->[$iwk->[$j]]->{x} <',/&
         &,'     $data->[$iwk->[$i]]->{x} ){',/&
         &,'    my $dum=$iwk->[$i];',/&
         &,'    $iwk->[$i]=$iwk->[$j];',/&
         &,'    $iwk->[$j]=$dum;',/&
         &,'      }',/&
         &,'    }',/&
         &,'  }',/&
         &,'  return $iwk;',/&
         &,'}',/&
         &,'',/&
         &,'########################################################################',/)
116 Format(&
         &'sub draw_quad_y0_dy0_y1{',/&
         &,'  my $fo=shift;',/&
         &,'  my $data=shift;',/&
         &,'  my $N=shift;',/&
         &,'  my $dx=shift;',/&
         &,'  my $DX=shift;',/&
         &,'  my $i0=shift;',/&
         &,'  my $i1=shift;',/&
         &,'  my $w=shift;',/&
         &,'  my $dw=shift;',/&
         &,'  my $txt=shift;',/&
         &,'  my $nS=shift;',/&
         &,'  my $color=shift;',/&
         &,'  ',/&
         &,'  my $x0=$data->[$i0]->{x};',/&
         &,'  my $x1=$data->[$i1]->{x};',/&
         &,'  my $y0=$data->[$i0]->{$dw};',/&
         &,'  my $y1=$data->[$i1]->{$dw};',/&
         &,'  my $E0=$data->[$i0]->{$w};',/&
         &,'  my $E1=$data->[$i1]->{$w};',/&
         &,'',/&
         &,'  print "dx=$dx\n";',/&
         &,'  for(my $i=0;$i<$N;$i++){',/&
         &,'    my $x=$i/($N-1.)*(1+2*0.2)*$DX+$x0-0.2*$DX;',/&
         &,'    my $l0=($x1-$x)/$dx;',/&
         &,'    my $l1=($x-$x0)/$dx;',/&
         &,'    my $lam=($y0-($E1-$E0)/$dx)*$dx;',/&
         &,'    my $E=($E0*$l0 +$E1*$l1+$l0*$l1*$lam);',/&
         &,'    printf($fo "%30.15g %30.15g\n",$x/$dx,$E);',/&
         &,'  }',/&
         &,'  printf($fo "\n\n");',/&
         &,'  ',/&
         &,'  $$txt.=sprintf("s%d symbol 0\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color %d\n",$$nS,$color);',/&
         &,'  $$nS++;',/&
         &,'',/&
         &,'}',/&
         &,'',/&
         &,'',/)
117 Format(&
         &'########################################################################',/&
         &,'sub draw_quad_y0_dy0_dy1{',/&
         &,'  my $fo=shift;',/&
         &,'  my $data=shift;',/&
         &,'  my $N=shift;',/&
         &,'  my $dx=shift;',/&
         &,'  my $DX=shift;',/&
         &,'  my $i0=shift;',/&
         &,'  my $i1=shift;',/&
         &,'  my $w=shift;',/&
         &,'  my $dw=shift;',/&
         &,'  my $txt=shift;',/&
         &,'  my $nS=shift;',/&
         &,'  my $color=shift;',/&
         &,'',/&
         &,'  my $x0=$data->[$i0]->{x};',/&
         &,'  my $x1=$data->[$i1]->{x};',/&
         &,'  my $y0=$data->[$i0]->{$dw};',/&
         &,'  my $y1=$data->[$i1]->{$dw};',/&
         &,'  my $E0=$data->[$i0]->{$w};',/&
         &,'  my $E1=$data->[$i1]->{$w};',/&
         &,'  my $dy=$y1-$y0;',/&
         &,'  ',/&
         &,'  for(my $i=0;$i<$N;$i++){',/&
         &,'    my $x=$i/($N-1.)*(1+2*0.2)*$DX+$x0-0.2*$DX;',/&
         &,'    my $l1=($x-$x0)/$dx;',/&
         &,'    my $E=($E0+$l1*($l1*$dy*$dx/2+$dx*$y0));',/&
         &,'    printf($fo "%30.15g %30.15g\n",$x/$dx,$E);',/&
         &,'  }',/&
         &,'  printf($fo "\n\n");',/&
         &,'  $$txt.=sprintf("s%d symbol 0\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color %d\n",$$nS,$color);',/&
         &,'  $$nS++;',/&
         &,'',/&
         &,'}',/&
         &,'',/&
         &,'',/&
         &,'########################################################################',/&
         &,'sub draw_cubic{',/)
118 Format(&
         &'  my $fo=shift;',/&
         &,'  my $data=shift;',/&
         &,'  my $N=shift;',/&
         &,'  my $dx=shift;',/&
         &,'  my $DX=shift;',/&
         &,'  my $i0=shift;',/&
         &,'  my $i1=shift;',/&
         &,'  my $w=shift;',/&
         &,'  my $dw=shift;',/&
         &,'  my $txt=shift;',/&
         &,'  my $nS=shift;',/&
         &,'  my $color=shift;',/&
         &,'    ',/&
         &,'  my $x0=$data->[$i0]->{x};',/&
         &,'  my $x1=$data->[$i1]->{x};',/&
         &,'  my $y0=$data->[$i0]->{$dw};',/&
         &,'  my $y1=$data->[$i1]->{$dw};',/&
         &,'  my $E0=$data->[$i0]->{$w};',/&
         &,'  my $E1=$data->[$i1]->{$w};',/&
         &,'  my $dE=$E1-$E0;',/&
         &,'',/&
         &,'  my $a=3*(($y0+$y1)*$dx-2*$dE);',/&
         &,'  my $b=(6*$dE-2*(2*$y0+$y1)*$dx);',/&
         &,'  my $c=$y0*$dx;',/&
         &,'  ',/&
         &,'  print "a=$a\n";',/&
         &,'  print "b=$b\n";',/&
         &,'  print "c=$c\n";',/&
         &,'  ',/&
         &,'  for(my $i=0;$i<$N;$i++){',/&
         &,'    my $x=$i/($N-1.)*(1+2*0.2)*$DX+$x0-0.2*$DX;',/&
         &,'    my $l1=($x-$x0)/$dx;',/&
         &,'    my $E=($E0',/&
         &,'       +$y0*$dx*$l1*(1-$l1)*(1-$l1)',/&
         &,'       -$y1*$dx*$l1*$l1*(1-$l1)',/&
         &,'       +$dE*(3-2*$l1)*$l1*$l1',/&
         &,'      );',/&
         &,'    printf($fo "%30.15g %30.15g\n",$x/$dx,$E);',/&
         &,'  }',/)
119 Format(&
         &'  printf($fo "\n\n");',/&
         &,'',/&
         &,'  $$txt.=sprintf("s%d symbol 0\n",$$nS);',/&
         &,'  $$txt.=sprintf("s%d line color %d\n",$$nS,$color);',/&
         &,'  $$nS++;',/&
         &,'  ',/&
         &,'}')
    Close(57)
    !!
    Call system('/usr/bin/perl ./SHOW >/dev/null 2>&1 &')
    !!
    !!
    Return
    !!
  End Subroutine show_plot
  !!***********************************************************************
  Subroutine plot_along_line(txt,it,Data,handler,lun,xk0,xk1,N,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Character(*),Intent(in)      :: txt
    Integer(t_int),Intent(in)    :: xk0,xk1,N
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    Type(T_iteration),Intent(inout)  :: Data
    Procedure(IterationHandler)  :: handler
    !!LOCAL
    Character(*),Parameter::nsubfunc="plot_along_line"
    Integer(t_int) :: i,step,sav,osav
    Real(t_real)   :: x0,x1,l,dx
    Real(t_real)   :: e(N),xx(N),y(N),yy(N),dyy(N)
    Character(5000)   :: str
    Logical(t_log) :: debv,debout
    Real(t_real)   :: ddxx(it% nvec)
    Real(t_real)   :: ddyy(it% nvec)
    !!
    debv=debugfun(nmodule,nsubfunc,'vectors')
    debout=debugfun('FORCEDRIVER','sccforce','separate_outfile')
    !!
    Write(lun,1)Trim(mesg),Trim(txt)
    Read(*,*) str
    If (str(1:1)/='p') return
    !!
    step=data% step
    !!
    sav=stdout
    osav=outfile
    If(debout)Then
       outfile=9
       stdout=9
    Endif
    !!
    !!
    If(debv)Then
       Write(lun,2)Trim(mesg),'XX(k0)',it% xx(:,xk0)
       Write(lun,2)Trim(mesg),'XX(k1)',it% xx(:,xk1)
    Endif
    !!
    ddxx(:)=it% xx(:,xk1)-it% xx(:,xk0)
    ddyy(:)=it% yy(:,xk1)-it% yy(:,xk0)
    dx=Sqrt(Sum(ddxx(:)**2))
    !!
    x0=-0.2
    x1= 1.2
    Do i=1,N
       l=(i-1)/Real(N-1,t_real)*(x1-x0)+x0
       it% xx(:,it% XTMP)=it% xx(:,xk0)+ddxx(:)*l
       Write(lun,3)Trim(mesg),i,l
       If(debv) Write(lun,2)Trim(mesg),'XX',it% xx(:,it% XTMP)
       Call make_step(it,it% XTMP,Data,handler,lun)
       xx(i)=l*dx
       e(i)=it% ee(it% XTMP)
       y(i)=Dot_product(ddxx(:),it% yy(:,it% XTMP))/dx
       yy(i)=Sum((it% yy(:,it% XTMP))**2)
       dyy(i)=2.0_t_real*Dot_product(it% yy(:,it% XTMP),ddyy(:))
    Enddo
    Open(57,file='ttt')
    Do i=1,N
       Write(57,11)xx(i),e(i),y(i),yy(i),dyy(i)
    Enddo
    Close(57)
    !!
    Call show_plot()
    !!
    !!
    If(debout)Then
       outfile=osav
       stdout=sav
    Endif
    !!
    data% step=step
    !!
    Write(lun,"('continue? ')")
    Read(*,*)str
    !Pause
    !!
    Return
    !!
1   Format(a,' plot because of ',a,'?')
2   Format(a,' plot ',a,'=',100(f12.5))
3   Format(a,'  i,x',i3,g30.15)
11  Format('      {x=>',f30.15,',E=>',f30.15,',y=>',f30.15,',Y=>'&
         &,f30.15,',DY=>',f30.15,'},')
    !!
  End Subroutine plot_along_line
  !!***********************************************************************
  Subroutine create_line_plot_file(it,k0,k1,res,mu)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: k0,k1,res
    Type(T_it),Intent(inout)     :: it
    Real(t_real),Intent(in)      :: mu
    !!LOCAL
    !Character(*),Parameter::nsubfunc="create_line_plot_file"
    Integer(t_int) :: nv
    Real(t_real)   :: y0DX,y1DX,Y0,Y1,y0DY,y1DY,dx,yDX,yDY,xx,Y
    !!
    Open(13,file='tt')
    !!
    nv=it% nvec
    !!
    it% xx(1:nv,it% XTMP)=it% xx(1:nv,k1)-it% xx(1:nv,k0)
    it% yy(1:nv,it% XTMP)=it% yy(1:nv,k1)-it% yy(1:nv,k0)
    !!
    dx=Sqrt(Sum(it% xx(1:nv,it% XTMP)**2))
    !!
    y0DX=Dot_product(it% yy(1:nv,k0),it% xx(1:nv,it% XTMP))
    y1DX=Dot_product(it% yy(1:nv,k1),it% xx(1:nv,it% XTMP))
    Y0=Dot_product(it% yy(1:nv,k0),it% yy(1:nv,k0))
    Y1=Dot_product(it% yy(1:nv,k1),it% yy(1:nv,k1))
    y0DY=Dot_product(it% yy(1:nv,k0),it% yy(1:nv,it% XTMP))*2.0_t_real
    y1DY=Dot_product(it% yy(1:nv,k1),it% yy(1:nv,it% XTMP))*2.0_t_real
    !!
    Write(13,"('      {x=>',f30.15,',E=>',f30.15,',y=>'&
         &,f30.15,',Y=>',f30.15,',DY=>',f30.15,'},')") &
         &0.0_t_real,it% ee(k0),y0DX/dx,Y0,y0DY

    Write(13,"('      {x=>',f30.15,',E=>',f30.15,',y=>'&
         &,f30.15,',Y=>',f30.15,',DY=>',f30.15,'},')") &
         &dx,it% ee(k1),y1DX/dx,Y1,y1DY
    !!
    yDX=Dot_product(it% yy(1:nv,res),(it% xx(:,k1)-it% xx(:,k0)))
    yDY=Dot_product(it% yy(1:nv,res),(it% yy(:,k1)-it% yy(:,k0)))*2.0_t_real
    Y  =Dot_product(it% yy(1:nv,res),it% yy(1:nv,res))
    !!
    xx=Sign(Sqrt(Sum((it% xx(:,res)-it% xx(:,k0))**2)),mu)
    !!
    Write(13,"('      {x=>',f30.15,',E=>',f30.15,',y=>'&
         &,f30.15,',Y=>',f30.15,',DY=>',f30.15,'},')") &
         &xx,it% ee(res),yDX/dx,Y,yDY
    !!
    Close(13)
    !!
    Return
    !!
  End Subroutine create_line_plot_file
  !!***********************************************************************
  Subroutine login_it(from,crit,it,lun,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Character(*),Intent(in)      :: crit
    Integer(t_int),Intent(in)    :: lun
    Integer(t_int),Intent(in)    :: from
    Type(T_it),Intent(inout)     :: it
    !!LOCAL
!    Character(*),Parameter::nsubfunc="login_it"
    Integer(t_int) :: j,i
    !!
    j=0
    If(crit=='e')Then
       
       j=0
       Do i=0,it% m
          If(it% ee(from) > it% ee(i)) j=j+1
       Enddo
       
    Else If(crit=='u')Then
       j=0
       Do i=0,it% m
          If(it% u2(from) > it% u2(i)) j=j+1
       Enddo
    Else if(crit=='0')then
       j=0
    Else if(crit=='m')then
       j=it% m
    Endif

    If(it% m< it% mdim) it% m=it% m+1
 
    If(j>it% m )Then
       j=it% m
    Endif
    
    Do i=it% m,j+1,-1
       it% u2(i)=it% u2(i-1)
       it% ee(i)=it% ee(i-1)
       it% xx(1:it% nvec,i)=it% xx(1:it% nvec,i-1)
       it% yy(1:it% nvec,i)=it% yy(1:it% nvec,i-1)
    Enddo
    
    it% u2(j)=it% u2(from)
    it% ee(j)=it% ee(from)
    it% xx(1:it% nvec,j)=it% xx(1:it% nvec,from)
    it% yy(1:it% nvec,j)=it% yy(1:it% nvec,from)
    
    
    Call print_history(it,lun,mesg)

    !!
    Return
    !!
  End Subroutine login_it
  !!***********************************************************************
  Subroutine print_history(it,lun,mesg)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_history"
    Integer(t_int) :: i
    !!
    !!
    Write(lun,2)Trim(mesg)
    Do i=0,it% m
       Write(lun,3) Trim(mesg),i,it% u2(i),i,it% ee(i)
    Enddo
    Write(lun,2)Trim(mesg)
    !!
    !!
    Return
    !!
2   Format(a,' ')
3   Format(a,' sorted u(',i5,')=',e15.5e3,' e(',i5,')=',f30.10)
    !!
  End Subroutine print_history
  !!***********************************************************************
  Subroutine calc_hepoint(it,xk0,xk1,res,lun,mesg,Data,handler)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(in)    :: lun
    Integer(t_int),Intent(in)    :: xk0,xk1,res
    Type(T_it),Intent(inout)     :: it
    Type(T_iteration),Intent(inout)  :: Data
    Procedure(IterationHandler)  :: handler
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_hpoint"
    Integer(t_int) :: i,j,m,nv,k0,k1
    Real(t_real),Pointer :: b(:),lam(:),Z(:,:),e(:),D(:)
    Real(t_real),Pointer :: Syx(:,:)
    Real(t_real),Pointer :: A2(:,:),Wr(:)
    Real(t_real),Pointer :: Dx(:),Ex(:),Zx(:,:),Ux(:,:)
    Real(t_real),Pointer :: Dy(:),Ey(:),Zy(:,:)
    Real(t_real)   :: dum,etol,emax,deh,maxdev,dEyh,slope,deH0
    Logical(t_log) :: deb,debv,debz,havemax,old
    !!
    !old=.False.
    old=.True.
    !!
    etol=1.0e-8_t_real
    !!
    Nullify(Dx,Zx,Ex,Ux)
    Nullify(Dy,Zy,Ey)
    Nullify(b,lam,D,Z,e,Syx)
    Nullify(A2,Wr)
    !!
    deb=debugfun(nmodule,nsubfunc,'')
    debv=debugfun(nmodule,nsubfunc,'vectors')
    debz=debugfun(nmodule,nsubfunc,'eigenvectors')
    !!
    k0=xk0
    k1=xk1
    nv=it% nvec
    !!
    !!
    If (k0==0 .And. k1==0 )Then

       If(debv)Then
          Write(lun,2)Trim(mesg)
          Write(lun,3)Trim(mesg),'XX',0,it% xx(:,0)
          Write(lun,3)Trim(mesg),'YY',0,it% yy(:,0)
          Write(lun,2)Trim(mesg)
       Endif
       
       it% xx(1:nv,res)=it% xx(1:nv,0)
       it% yy(1:nv,res)=it% yy(1:nv,0)

       If(deb)Write(lun,4)Trim(mesg)

       !! goto cleanup and return
       Goto 2222
    Endif
    !!
    !! reduce dimenisonality of X-space and return 
    !! its eigensystem
    !!
    !! Sxx=DX*DX=(1/Dx)*Zx*Ex*Zx^(T)*(1/Dx)
    !! where Dx=1/sqrt(diag(Sxx))
    !! the orthogonalized DX is
    !!
    !! DX~=DX*Dx*Zx*(1/sqrt(Ex))
    !! with
    !! 1=DX~*DX~
    !!
    !!
    Call clean_x_space(it,Zx,Dx,Ex,lun,mesg)
    !!
    !!
    If(deb) Write(lun,1)Trim(mesg)
    !!

    
    m=it% m-k1+1
    If(deb)Write(lun,5)Trim(mesg),k0,k1,it% m,m
    !!
    If(debv)Then
       Write(lun,2)Trim(mesg)
       Do i=0,m
          Write(lun,3)Trim(mesg),'XX',i,it% xx(:,i)
       Enddo
       Do i=0,m
          Write(lun,3)Trim(mesg),'YY',i,it% yy(:,i)
       Enddo
       Write(lun,2)Trim(mesg)
    Endif

    !!
    !!
    Call palloc('b'   ,b,1_t_int,m,nmodule,nsubfunc)
    Call palloc('lam' ,lam,1_t_int,m,nmodule,nsubfunc)
    Call palloc('Syx' ,Syx,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    Call palloc('Ux'  ,Ux ,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    Call palloc('A2'  ,A2 ,1_t_int,m,1_t_int,m,nmodule,nsubfunc)
    !!
    !! DX_ortho=DX*Ux
    !!
    Do i=1,m
       Do j=1,m
          Ux(i,j)=Dx(i)*Zx(i,j)/Sqrt(Ex(j))
       Enddo
    Enddo
    !!
    If(.Not.old)Then
       Ux=0.0_t_real
       Do i=1,m
          Ux(i,i)=1.0_t_real
       Enddo
    Endif
    !!
    Do i=1,m
       Do j=1,m
          Syx(i,j)=Dot_product(it% yy(1:nv,k1+i-1)-it% yy(1:nv,k0)&
               &,it% xx(1:nv,k1+j-1)-it% xx(1:nv,k0))
       Enddo
    Enddo
    maxdev=0.0_t_real
    Do i=1,m
       maxdev=Max(maxdev,Maxval(Abs(Syx(i,i+1:m)-Syx(i+1:m,i))))
    Enddo
    If(deb) Write(lun,23)Trim(mesg),maxdev/Abs(Maxval(Syx(1:m,1:m)))
    
    A2(1:m,1:m)=(Syx(1:m,1:m)+Transpose(Syx(1:m,1:m)))*0.5_t_real
    A2(:,:)=Matmul(A2(:,:),Ux(:,:))
    A2(:,:)=Matmul(Transpose(Ux(:,:)),A2(:,:))

       

    If(old)Then
       Syx(:,:)=Matmul(Syx(:,:),Ux(:,:))
    Else
       Syx(1:m,1:m)=(Syx(1:m,1:m)+Transpose(Syx(1:m,1:m)))*0.5_t_real
       Syx(:,:)=Matmul(Syx(:,:),Ux(:,:))
       Syx(:,:)=Matmul(Transpose(Ux(:,:)),Syx(:,:))
    Endif


    !!
    !!

    If(old)Then
       Call eigen_analyse_e(Syx,it,k0,k1,it% m,D,E,Z,lun,mesg)
    Else
       Call eigen_analyse_S(Syx,it,k0,k1,it% m,D,E,Z,lun,mesg)
    Endif





    Do i=1,m
       If(deb) Write(lun,6)Trim(mesg),i,E(i)
       Do j=1,m
          if(debz)Write(lun,7) Trim(mesg),j,Z(j,i)*Sqrt(Abs(E(i)))
       Enddo
    Enddo
    !!
    !!
    Do i=1,m
       b(i)=-Dot_product(&
            &it% xx(1:nv,i+k1-1)-it% xx(1:nv,k0)&
            &,it% yy(1:nv,k0)&
            &)
    Enddo
    b(1:m)=Matmul(Transpose(Ux(1:m,1:m)),b(1:M))
    If(old)Then
       b(1:m)=Matmul(Syx(1:m,1:m),b(1:M))
    Endif
    b(1:m)=D(1:m)*b(1:m)

!!$    Write(lun,2)Trim(mesg)
!!$    Do i=1,m
!!$       Write(lun,"(a,' Ux')",advance='no')Trim(mesg)
!!$       Do j=1,m
!!$          Write(lun,"(e13.5e3,1x)",advance='no') Ux(i,j)
!!$       Enddo
!!$       Write(lun,*)
!!$    Enddo
!!$    Write(lun,2)Trim(mesg)
    
    !!
    !!
    !! |DX~^T*(y0+DY*lam)|^2 ->Min
    !! 
    !! gives equation
    !! Syx~*DX~^T*y0=Syx~*Sxy~*lam
    !! with
    !! Syx~=Syx*Ux
    !!
    !! and decomposition
    !! Syx~*Sxy~=(1/D)*Z*E*Z^T*(1/D)
    !!
    !! giving
    !! b=Z*E*Z^T*(1/D)*lam
    !!
    !! with
    !! b=D*Syx~*Ux^T*DX*y0
    !!
    !! solution
    !! lam=D*Z*(1/E)*Z^T*b
    !!
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,8)Trim(mesg),i,b(i)/D(i)
       Enddo
       Write(lun,2)Trim(mesg)
    Endif
    !!
    emax=Maxval(Abs(e(1:m)))
    !!
    If(deb) Write(lun,9)Trim(mesg),emax
    !!
    lam(:)=0.0_t_real
    Do j=1,m
       If(Abs(e(j))>etol*emax)Then
          If(old)Then
             lam(:)=lam(:)+Z(:,j)/e(j)*Dot_product(Z(:,j),b)
          Else
             lam(:)=lam(:)+Z(:,j)/Abs(E(j))*Dot_product(Z(:,j),b)
          Endif
       Else
          If(deb) Write(lun,10)Trim(mesg),j
       Endif
    Enddo

    lam(1:m)=D(1:m)*lam(1:m)
    If(.Not.old)Then
       lam(1:m)=Matmul(Ux(1:m,1:m),lam(1:m))
    Endif
    !!
    !!
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,11)Trim(mesg),i,lam(i)
       Enddo
       Write(lun,2)Trim(mesg)
    Endif
    !!
    !! we take the same lam although it is not 100% the theory
    !! otherwise we have a 1/Sxy problem
    !!
    Do i=1,nv
       it% xx(i,res)=it% xx(i,k0)+&
            &Dot_product((it% xx(i,k1:it% m)-it% xx(i,k0)),lam(1:m))
       it% yy(i,res)=it% yy(i,k0)+&
            &Dot_product(it% yy(i,k1:it% m)-it% yy(i,k0),lam(1:m))
    Enddo
    !!
    !!
    !! here we should introduce the maximum direction search
    deH0=&
         &+Dot_product(it% xx(:,res)-it% xx(:,k0),it% yy(:,k0))&
         &*0.5_t_real

    deH=0.5_t_real*Dot_product((it% yy(:,res)+it% yy(:,k0)),&
         &                     (it% xx(:,res)-it% xx(:,k0)))

    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Write(lun,12)Trim(mesg),it% m,deH,deH0
       Write(lun,2)Trim(mesg)
       !!
       Write(lun,2)Trim(mesg)
       Do i=1,m
          Write(lun,13)Trim(mesg),i&
               &,Dot_product(it% xx(:,k1+i-1)-it% xx(:,k0),it% yy(:,res))
       Enddo
       Write(lun,2)Trim(mesg)
    Endif
    !!
    !!
    If(debugfun(nmodule,nsubfunc,'plot xspace'))Then
       Do i=1,m
          Call create_line_plot_file(it,0_t_int,i,i,1.0_t_real)
          Call plot_along_line(' DX'//Trim(intToChar10(i))//'0'&
               &,it,Data,handler,lun&
               &,0_t_int,i,5_t_int,mesg)
       Enddo
       !Call create_line_plot_file(it,0,i,res,1.0_t_real)
       Call plot_along_line(' DXH0 ',it,Data,handler,lun&
            &,0_t_int,res,5_t_int,mesg)
       
       it% xx(:,it% XMU)=it% xx(:,res)-it% yy(:,res)
       !Call create_line_plot_file(it,0,i,it% XMU,1.0_t_real)
       Call plot_along_line(' (XH-YH)-X0 ',it,Data,handler,lun&
            &,0_t_int,it% XMU,5_t_int,mesg)
    Endif
    !!
    If(.Not.old)Then
       havemax=Any(E(:)<0.0_t_real)
       !havemax=.False.
       it% ee(res)=it% ee(k0)+deH
       Goto 898
    Endif

    havemax=.False.
    If(deH<=0.0_t_real)Then
       it% ee(res)=it% ee(k0)+deH
       If(deb) Write(lun,14)Trim(mesg)
    Else
       havemax=.True.
       If(deb) Write(lun,15)Trim(mesg)

       Call palloc('Wr'  ,Wr ,1_t_int,m,nmodule,nsubfunc)

       
       If(deb) Then
          Write(lun,2)Trim(mesg)
          Do i=1,m
             Write(lun,"(a,' Syx: ')",advance='no')Trim(mesg)
             Do j=1,m
                Write(lun,"(e13.5e3,1x)",advance='no') Syx(i,j)
             Enddo
             Write(lun,*)
          Enddo
          Write(lun,2)Trim(mesg)
          !!
          Write(lun,2)Trim(mesg)
          Do i=1,m
             Write(lun,"(a,' A2: ')",advance='no')Trim(mesg)
             Do j=1,m
                Write(lun,"(e13.5e3,1x)",advance='no') A2(i,j)
             Enddo
             Write(lun,*)
          Enddo
          Write(lun,2)Trim(mesg)
       Endif
       
       Call rdiagno_noO(m,A2,Wr)

       
       Do i=1,m
          If(deb) Write(lun,66)Trim(mesg),i,Wr(i)
       Enddo

       Do i=1,m
          b(i)=-Dot_product(&
               &it% xx(1:nv,i+k1-1)-it% xx(1:nv,k0)&
               &,it% yy(1:nv,k0)&
               &)
       Enddo
       b(1:m)=Matmul(Transpose(Ux(1:m,1:m)),b(1:M))
       b(1:m)=Matmul(Transpose(A2(1:m,1:m)),b(1:M))
       

       emax=Maxval(Abs(Wr(1:m)))
       !!
       If(deb) Write(lun,9)Trim(mesg),emax

       
       lam(:)=0.0_t_real
       Do j=1,m
          If(Abs(Wr(j))>etol*emax)Then
             lam(:)=lam(:)+A2(:,j)/Abs(Wr(j))*b(j)
          Else
             If(deb) Write(lun,10)Trim(mesg),j
          Endif
       Enddo
       lam(1:m)=Matmul(Ux(1:m,1:m),lam(1:m))
       
       If(deb)Then
          Write(lun,2)Trim(mesg)
          Do i=1,m
             Write(lun,11)Trim(mesg),i,lam(i)
          Enddo
          Write(lun,2)Trim(mesg)
       Endif
       
       Do i=1,nv
          it% xx(i,res)=it% xx(i,k0)+&
               &Dot_product((it% xx(i,k1:it% m)-it% xx(i,k0)),lam(1:m))
          it% yy(i,res)=it% yy(i,k0)+&
               &Dot_product(it% yy(i,k1:it% m)-it% yy(i,k0),lam(1:m))
       Enddo
       !!
       !!
       !! here we should introduce the maximum direction search
       
       deH=0.5_t_real*Dot_product((it% yy(:,res)+it% yy(:,k0)),&
            &                     (it% xx(:,res)-it% xx(:,k0)))
       
       it% ee(res)=it% ee(k0)+deH

       If(deb)Then
          Write(lun,2)Trim(mesg)
          Write(lun,12)Trim(mesg),it% m,deH
          Write(lun,2)Trim(mesg)
          !!
          Write(lun,2)Trim(mesg)
          Do i=1,m
             Write(lun,13)Trim(mesg),i&
                  &,Dot_product(it% xx(:,k1+i-1)-it% xx(:,k0),it% yy(:,res))
          Enddo
          Write(lun,2)Trim(mesg)
       Endif
       
    Endif
    !!
898 Continue
    !!
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Write(lun,18)Trim(mesg),Sqrt(Sum(it% yy(:,res)**2))
       Write(lun,2)Trim(mesg)
    Endif
    !!
    !! Qyx*Yh
    !!
    If(.Not.havemax)Then
       Do i=1,m
          b(i)=-Dot_product(&
               &it% yy(1:nv,i+k1-1)-it% yy(1:nv,k0),it% yy(1:nv,res))
       Enddo
       If(.Not.old)Then
          b(1:m)=Matmul(Transpose(Ux(1:m,1:m)),b(1:M))
       Endif
       b(1:m)=D(1:m)*b(1:m)
       !!
       !!
       lam(1:m)=0.0_t_real
       Do j=1,m
          If(Abs(e(j))>etol*emax)Then
             If(old)Then
                lam(:)=lam(:)+Z(:,j)/e(j)*Dot_product(Z(:,j),b)
             Else
                lam(:)=lam(:)+Z(:,j)/Abs(e(j))*Dot_product(Z(:,j),b)
             Endif
          Endif
       Enddo
       lam(1:m)=D(1:m)*lam(1:m)
       If(old)Then
          lam(1:m)=Matmul(Transpose(Syx(1:m,1:m)),lam(1:m))
       Endif
       lam(1:m)=Matmul(Ux(1:m,1:m),lam(1:m))
       !!
       !!
       Do i=1,nv
          it% yy(i,res)=it% yy(i,res)+&
               &Dot_product(it% xx(i,k1:it% m)-it% xx(i,k0),lam(1:m))
       Enddo
       !!
       If(it% u2(0) < it % u2crossover/10.0_t_real)Then
          !!lets try whether gradient can be minimized as well
          !!
          !! |y0+DY*lam|^2 -> Min
          !!
          !! lam=-1/DYDY y0*DY
          !!
          Call eigen_analyse('yy',it,0_t_int,1_t_int,m,Dy,Ey,Zy,lun,mesg)
          !!
          Do i=1,m
             b(i)=-Dot_product(&
                  &it% yy(1:nv,i+k1-1)-it% yy(1:nv,k0)&
                  &,it% yy(1:nv,k0)&
                  &)
          Enddo
          b(1:m)=Dy(1:m)*b(1:m)
          !!
          emax=Maxval(Abs(e(1:m)))
          !!
          lam(:)=0.0_t_real
          Do j=1,m
             If(Abs(Ey(j))>etol*emax)Then
                lam(:)=lam(:)+Zy(:,j)/Ey(j)*Dot_product(Zy(:,j),b)
             !Else
                !If(deb) Write(lun,10)Trim(mesg),j
             Endif
          Enddo
          lam(1:m)=Dy(1:m)*lam(1:m)
          !!
          Do i=1,nv
             it% xx(i,it% XTMP)=it% xx(i,k0)+&
                  &Dot_product((it% xx(i,k1:it% m)-it% xx(i,k0)),lam(1:m))
             it% yy(i,it% XTMP)=it% yy(i,k0)+&
                  &Dot_product((it% yy(i,k1:it% m)-it% yy(i,k0)),lam(1:m))
          Enddo
          !!
          !!
          slope=Dot_product(it% yy(:,k0),(it% xx(:,it% XTMP)-it% xx(:,k0)))
          dEyh=0.5_t_real*(Dot_product((it% yy(:,it% XTMP)),&
               &                     (it% xx(:,it% XTMP)-it% xx(:,k0)))+slope)
          !!
          If(deb)Then
             Write(lun,2)Trim(mesg)
             Write(lun,112)Trim(mesg),it% m,dEyh
             Write(lun,114)Trim(mesg),it% m,-0.01_t_real*Abs(slope)
             Write(lun,2)Trim(mesg)
          Endif
          !!
          If(dEyh < -0.01_t_real*Abs(slope))Then
             If(deb)Then
                Write(lun,2)Trim(mesg)
                Write(lun,113)Trim(mesg),it% m
                Write(lun,2)Trim(mesg)
             Endif
             it% xx(1:nv,res)=it% xx(1:nv,it% XTMP)
             it% yy(1:nv,res)=it% yy(1:nv,it% XTMP)
          Endif
          !!
          !!
       Endif
       !!
    Endif
    !!
    !!
    it% u2(res)=Sqrt(Sum(it% yy(1:nv,res)**2))
    dum=Sqrt(Sum((it% yy(1:nv,res)-it% yy(1:nv,k0))**2))
    !!
    If(debv)Then
       Write(lun,2)Trim(mesg)
       Write(lun,16)Trim(mesg),it% xx(:,res)
       Write(lun,17)Trim(mesg),it% yy(:,res)
       Write(lun,2)Trim(mesg)
    Endif
    If(deb)Then
       Write(lun,2)Trim(mesg)
       Write(lun,118)Trim(mesg),it% u2(res)
       Write(lun,2)Trim(mesg)
       Write(lun,21)Trim(mesg),it% m,it% ee(res)
       Write(lun,2)Trim(mesg)
       !!
       !!
       Write(lun,2)Trim(mesg)
       Write(lun,22)Trim(mesg),dum
       Write(lun,2)Trim(mesg)
    Endif
    !!
2222 Continue
    !!
    Call pfree(Wr)
    Call pfree(A2)
    Call pfree(D)
    Call pfree(lam)
    Call pfree(b)
    Call pfree(E)
    Call pfree(Z)
    Call pfree(Syx)
    Call pfree(Dx)
    Call pfree(Ex)
    Call pfree(Zx)
    Call pfree(Ux)
    Call pfree(Dy)
    Call pfree(Ey)
    Call pfree(Zy)
    !!
    Return
    !!
1   Format(a,' calc_hpoint ----------------------------------------')
2   Format(a,' ')
3   Format(a,' ',a2,'(',i3,')=',100(f12.5))
4   Format(a,' case m=0')
5   Format(a,' ',4(i5,2x))
6   Format(a,' E(',i3,')=',e30.15e3)
66  Format(a,' EA(',i3,')=',e30.15e3)
7   Format(a,'    W DY(',i3,')=',e30.15e3)
8   Format(a,' by(',i3,')=',e30.15e3)
9   Format(a,' emax=',e30.15e3)
10  Format(a,' skip jer=',i3)
11  Format(a,' lam(',i3,')=',e30.15e3)
12  Format(a,' m=',i5,'  estimated DE   = ',f30.15,'  DE0   = ',f30.15)
112 Format(a,' m=',i5,'  estimated DEy  = ',f30.15)
114 Format(a,' m=',i5,'  estimated slope= ',f30.15)
113 Format(a,' m=',i5,'  gradient point seems better E= ')
13  Format(a,' DX*Yh(',i3,')=',e30.15e3)
14  Format(a,' supposed minimum at XX(h)')
15  Format(a,' supposed maximum at XX(h)')
16  Format(a,' XX(h)=',100(f12.5))
17  Format(a,' YY(h)=',100(f12.5))
18  Format(a,' |YY(h)|=',e30.15e3) 
118  Format(a,' |Qyx*YY(h)|=',e30.15e3) 
!19  Format(a,' curv(',i3,')=',e30.15e3)
!20  Format(a,' m=',i5,'  estimated u= ',e30.15e3)
21  Format(a,' m=',i5,'  estimated e= ',f17.10)
22  Format(a,' |yh-y0|=',e30.15e3)
23  Format(a,' max deviation from symmetric model=',e15.3e3)
    !!
    !!
  End Subroutine calc_hepoint
  !!***********************************************************************
  Subroutine iterat_minimize(Data,handler,lun)
    !!
    !! here we try to use the linear relation between x and y
    !! in the sense of the gradient of a quadratic energy function
    !!
    !! E=a+b*x+1/2*x^T*B*x
    !!
    !! the forces are
    !!
    !! y=grad E =b+B*x
    !!
    !! This we take as iterator in order to avoid maxima
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Type(T_iteration),Intent(inout)  :: Data
    Procedure(IterationHandler)  ::handler
    !!LOCAL
    Character(*),Parameter::nsubfunc="iterat_minimize"
    Integer(t_int) :: nv,ret,k1
    Logical(t_log) :: debv,deb,debhist,exists
    Real(t_real)   :: preduce,pmax,pmin
    Type(T_it)     :: it
    !!
    debhist=debugfun(nmodule,nsubfunc,'save_history')
    deb=debugfun(nmodule,nsubfunc,'')
    debv=debugfun(nmodule,nsubfunc,"vectors")
    !!
    !!
    !! initialize the point 0 with the starting x
    Call init_it(it,data% vecdim,data% maxdim,data% mixing,data% vector,0_t_int&
         &,0.5_t_real)
    !!
    preduce=1.4_t_real
    pmax=1.0_t_real
    pmin=0.0_t_real
    it% p=1.0_t_real
    data% step=0
    nv=it% nvec
    If(deb)Then
       Write(lun,1)Trim(data%name),nv
       Write(lun,3)Trim(data%name),it% mdim
       Write(lun,4)Trim(data%name),it% p
    Endif

    !!
    k1=1
    !!
    If(debhist)Then
       Inquire(file='=.iterat_minimize',exist=exists)
       If(exists)Then
          Call read_state(it)
          Goto 444
       Endif
    Endif
    !!
    if(deb)Write(lun,5)Trim(data%name)
    it% m=0
    Call make_step(it,0_t_int,Data,handler,lun)
    If(it% stepsexceed) Goto 7777
    If(it% converged) Goto 8888

    !!
    If(deb)Write(lun,6)Trim(data%name),it% m
    Call print_history(it,lun,data%name)
    !!
    k1=0
    !!
444 Continue
    iterloop: Do

       !!TODO: check progress of interpolation
       If( data% deviation <= data% tolerance/(hatoev/brtoan)) Goto 8888


       If(debhist) Call save_state(it)
       !!
       !!
       Call calc_hepoint(it,0_t_int,k1,it% XZ1,lun,Data%name,Data,handler)
       !!
       !!here XZ1 is the last subspace hypothetical best point
       !!
       !! take y(xh) to get a new dimension
       it% xx(:,it% DX)=(it% xx(:,it% XZ1)-it% xx(:,0)&
            &-it% yy(:,it% XZ1))*it% p

       
       Call line_search(it,Data,handler,lun,ret,Data% name)
       If(it% stepsexceed) Goto 7777
       If(ret==BETTER) Then
          k1=1
          !!Write(lun,"('todo: converge')")
          !todo If(it% converged) Goto 8888
          Cycle iterloop
       Else
          k1=0
          Cycle iterloop
       Endif


       
!!$       y0DX=it% p*Dot_product(it% yy(:,0),it% xx(:,it% DX))
!!$       y1DX=it% p*Dot_product(it% yy(:,it% XN),it% xx(:,it% DX))
!!$       Write(lun,14)Trim(data%name),y0DX
!!$       Write(lun,15)Trim(data%name),y1DX
!!$       pnew=-1.0_t_real
!!$       If(y0DX<0.0_t_real .And. (Abs(y1DX) < Abs(y0DX)).Or.y1DX>0.0_t_real)Then
!!$          mu=-y0DX/(y1DX-y0DX)
!!$          Write(lun,16)Trim(data%name),mu
!!$          If(mu>0.0_t_real)Then
!!$             Write(lun,17)Trim(data%name),it% p
!!$             pnew=Max(pmin,Min(pmax,Max(0.01_t_real,(it%p+Min(pmax,mu))*0.5_t_real)))
!!$             Write(lun,18)Trim(data%name),pnew
!!$          Endif
!!$       Endif
!!$       
!!$       If(it% ee(it% XN) < &
!!$            &it% ee(0)-0.01_t_real*Abs(slope) ) Then
!!$          Write(lun,20)Trim(data%name)
!!$          Call login_it(it% XN,'0',it,lun,data%name) !! login m+1
!!$
!!$          If(pnew>0.0_t_real)Then
!!$             it% p=pnew
!!$          Else
!!$             it% p=Max(pmin,Min(pmax,it% p*preduce))
!!$          Endif
!!$          
!!$          If(it% converged) Goto 8888
!!$
!!$          Cycle iterloop
!!$
!!$       Else If(it% u2(0)< it% u2crossover .And. it% u2(it% XN) < &
!!$            &it% u2(0)-0.01_t_real*Abs(yslope) ) Then
!!$          Write(lun,21)Trim(data%name)
!!$          Call login_it(it% XN,'0',it,lun,data%name) !! login m+1
!!$          If(it% converged) Goto 8888
!!$
!!$          Cycle iterloop
!!$
!!$       Endif
!!$
!!$
!!$       If(it% converged) Goto 8888
!!$       
!!$       !! we where not succesfull in the heuristic step
!!$       !! lets be tight
!!$       !!
!!$       !Pause 'first line search'
!!$       !!
!!$       Write(lun,23)Trim(data%name)
!!$       If(pnew>0.0_t_real)Then
!!$          it% p=pnew
!!$       Else
!!$          it% p=Max(pmin,Min(pmax,it% p/preduce))
!!$       Endif

          

       !!
       !!
    End Do iterloop
    !!
    Goto 7777
    !!
7777 Continue
    !!
    Write(lun,511)Trim(data%name),data% step,data% steps
    !!
    data% vector(1:it% nvec)=it% xx(1:it% nvec,0)
    Call clean_it(it)
    Return
    !!
8888 Continue
    !! converged
    data% vector(1:it% nvec)=it% xx(1:it% nvec,0)
    !!
    Write(lun,1000)Trim(data%name)
    Call clean_it(it)
    Return
    !!
1000 Format(a,' Force optimization is converged!')
1   Format(a,' nvec=',i10)
!2   Format(a,' ')
3   Format(a,' mdim=',i10)
4   Format(a,' mixing=',f12.5)
5   Format(a,' calc at x0')
6   Format(a,' login m=',i3)
!7   Format(a,' calc at x0-p*y0')
!12  Format(a,' calc at new dimension')
!13  Format(a,' ',a,'=',100(f12.5))
!14  Format(a,' grad at x0 =',e14.3e3)
!15  Format(a,' grad at x1 =',e14.3e3)
!16  Format(a,' better mixing would have been',e14.3e3)
!17  Format(a,' old mixing=',e14.3e3)
!18  Format(a,' new mixing=',e14.3e3)
!19  Format(a,' reduction criterion dE =',e30.15e3)
!20  Format(a,' hypothetical point is BETTER')
!21  Format(a,' hypothetical point is BETTER u2')
!22  Format(a,' hypothetical point is gradient update')
!23  Format(a,' hypothetical point is WORSE THAN BEST')
511 Format(a,' Abort iteration, to many steps (step='&
         & ,i4,' steps=',i4,')!')

    !!
  End Subroutine iterat_minimize 
  !!***********************************************************************
 Subroutine line_search(it,Data,handler,lun,ret,mesg)
    !!
    Implicit None  
    !!IO
    Character(*),Intent(in)      :: mesg
    Integer(t_int),Intent(out)   :: ret
    Integer(t_int),Intent(in)    :: lun
    Type(T_it),Intent(inout)     :: it
    Type(T_iteration),Intent(inout)  :: Data
    Procedure(IterationHandler)  :: handler
    !!LOCAL
    Character(*),Parameter::nsubfunc="line_search"
    Logical(t_log) :: deb,debv,debplot,gotit
    Character(100) :: wher
    Integer(t_int) :: nv,count,nroot,res
    Real(t_real)   :: mu,scale,slope0,slope1,erequire,a,b,c,dE,xr(2)
    Real(t_real)   :: xprog,xm,Em,E0,u2require,yslope0,yslope1,xx
    Real(t_real)   :: dy0,dy1,Y0,Y1
    !!
    u2require=0.0_t_real
    yslope0=0.0_t_real
    deb=debugfun(nmodule,nsubfunc,'')
    debv=debugfun(nmodule,nsubfunc,"vectors")
    debplot=debugfun(nmodule,nsubfunc,"plot")
    !!
    xprog=1.0e-4_t_real
    !!
    nv=it% nvec
    !!
    Call limitStep(scale,it% nvec,it% xx(:,it% DX),it% stepmax)
    !! DXnew=DXold*scale
    !! dx=Dot_product(it% xx(:,it% DX),it% xx(:,it% DX))
    !!
    slope0=Dot_product(it% yy(:,0),it% xx(:,it% DX))
    erequire=it% ee(0)-it% improvtol*Abs(slope0)
    E0=it% ee(0)
    !!
    count=1
    !!
    Do 
       !!
       If(deb) Write(lun,"(a,' loop ',i3)")Trim(mesg),count
       If(count==1) Then
          res=it% XN
          wher='new dimension'
          mu=1.0_t_real
       Elseif(count==2) Then
          res=it% XL1
          dE=it% ee(it% XN)-it% ee(0)
          slope1=Dot_product(it% yy(:,it% XN),it% xx(:,it% DX))
          gotit=.False.
          !! chase energy minimum
          If(slope0<0.0_t_real .And. slope1 > 0.0_t_real)Then
             !! 3rd order
             c=slope0
             b=+6.0_t_real*dE-2.0_t_real*(2.0_t_real*slope0+slope1)
             a=3.0_t_real*(slope0+slope1-2.0_t_real*dE)
             Call solve_quad_real(a,b,c,xr,nroot)
             If( nroot==2)Then
                If(deb) Write(lun,"(a,' xr=',2(f30.15))")Trim(mesg),xr
                If(0.0_t_real < xr(1) .And. xr(1) < 1.0_t_real)Then
                   mu=Min(1.0_t_real-xprog,Max(xr(1),xprog))
                Else If(0.0_t_real < xr(2) .And. xr(2) < 1.0_t_real)Then
                   mu=Min(1.0_t_real-xprog,Max(xr(2),xprog))
                Endif
                gotit=.True.
                wher='line: 3rd order y0,y1,E0,dE'
             Endif
          Endif
          If(.Not.gotit)Then
             !! 3rd order not succesfull try second order
             !!
             gotit=.True.
             wher='line: 2nd order y0,E0,E1 '
             c=slope0
             b=2.0_t_real*(dE-slope0)
             If(deb) Write(lun,"(a,' b=',1(f30.15))")Trim(mesg),b
             !!
             If(b/=0.0_t_real)Then
                mu=-c/Abs(b) !! abs takes care of maxima
             Else
                If(c<0.0_t_real) mu=2.0_t_real
                If(c>0.0_t_real) mu=-1.0_t_real
             Endif
             mu=Min(2.0_t_real,Max(mu,-1.0_t_real))
             If(Abs(mu)<xprog) mu=Sign(xprog,mu)
             If(Abs(mu-1.0_t_real)<xprog) &
                  &mu=1.0_t_real+Sign(xprog,mu-1.0_t_real)
             !!
          Endif
          If(it% u2(0)<it% u2crossover)Then
             !! chase gradient minimum
             wher='line: 2nd order gradient '
             !! make sure we see a minimum between x0 and x1
             c=yslope0
             b=yslope1-yslope0

             If(b/=0.0_t_real)Then
                xx=-c/Abs(b) !! abs takes care of maxima
             Else
                If(c<0.0_t_real) xx=2.0_t_real
                If(c>0.0_t_real) xx=-1.0_t_real
             Endif
             xx=Min(2.0_t_real,Max(xx,-1.0_t_real))
             If(Abs(xx)<xprog) xx=Sign(xprog,xx)
             If(Abs(xx-1.0_t_real)<xprog) &
                  &xx=1.0_t_real+Sign(xprog,xx-1.0_t_real)

             !! x=x0+DX*lam
             !! dE=y0(x-x0)+1/2*(x-x0)*B*(x-x0)
             !! dE=(y+y0))*(x-x0)/2
             If(deb) Write(lun,"(a,' my(gradient)=',1(f30.15))")Trim(mesg),xx
             de=Dot_product(&
                  &(2.0_t_real*it% yy(:,0)&
                  &+(it% yy(:,it% XN)-it% yy(:,0))*xx)&
                  &,(it% xx(:,it% DX)*xx))
             If(deb) Write(lun,"(a,' E(my_y)=',e30.15e3)")Trim(mesg),de
             If(de<-it% improvtol*Abs(slope0))Then
                If(deb) Write(lun,"(a,' y-interpol seems good')")Trim(mesg)
                mu=xx
             Endif
             
          Endif
       Elseif(count==3) Then
          gotit=.True.
          wher='line: 2nd order E0,Em,E1 '
          Em=it% ee(res)
          res=it% XL2
          xm=mu
          !!
          c=((Em-E0)-dE*xm**2)/xm/(1.0_t_real-xm)
          b=2.0_t_real*(dE*xm-(Em-E0))/xm/(1.0_t_real-xm)

          erequire=it% ee(0)-it% improvtol*Abs(c)
          
          If(deb) Write(lun,"(a,' b=',1(f30.15))")Trim(mesg),b
          !!
          If(b/=0.0_t_real)Then
             mu=-c/Abs(b) !! abs takes care of maxima
          Else
             If(c<0.0_t_real) mu=2.0_t_real
             If(c>0.0_t_real) mu=-1.0_t_real
          Endif
          mu=Min(2.0_t_real,Max(mu,-1.0_t_real))
          If(Abs(mu)<xprog) mu=Sign(xprog,mu)
          If(Abs(mu-1.0_t_real)<xprog) &
               &mu=1.0_t_real+Sign(xprog,mu-1.0_t_real)

          !!
       Endif
       !!
       !! form new point
       it% xx(:,res)=it% xx(:,0)+it% xx(:,it% DX)*mu
       !!
       If(debv)Then
          Write(lun,2)Trim(data%name)
          Write(lun,13)Trim(data%name),'DX',it% xx(:,it% DX)
          Write(lun,13)Trim(data%name),'XN',it% xx(:,res)
          Write(lun,2)Trim(data%name)
       Endif
       !!
       If(deb) Then 
          Write(lun,2)Trim(mesg)
          Write(lun,12)Trim(data%name),Trim(wher)
          Write(lun,2)Trim(mesg)
          Write(lun,4)Trim(mesg),mu
          Write(lun,2)Trim(mesg)
       Endif
       Call make_step(it,res,Data,handler,lun)
       If(it% stepsexceed) Return
       !!
       If(count==1)Then
          yslope0=Dot_product(it% yy(:,0),it% yy(:,res)-it% yy(:,0))&
               &*2.0_t_real
          yslope1=Dot_product(it% yy(:,res),it% yy(:,res)-it% yy(:,0))&
               &*2.0_t_real
          If(it% u2(0) >0.0_t_real)Then
             u2require=Sqrt(Max(0.0_t_real,it% u2(0)**2-it% improvtol*Abs(yslope0)))
          Else
             u2require=0.0_t_real
          Endif
       Endif
       If(count>=2)Then
          dy0=yslope0
          dy1=yslope1
          Y0=it% u2(0)
          Y1=it% u2(it% XN)
!!$          If(Y0>Y1)Then
!!$             dum=Y0
!!$             Y0=Y1
!!$             Y1=dum
!!$             dum=dy0
!!$             dy0=-dy1
!!$             dy1=-dy0
!!$          Endif
          If(Y0 >0.0_t_real)Then
             u2require=Sqrt(Max(0.0_t_real&
                  &,Y0**2-it% improvtol*Abs(dy0)))
          Else
             u2require=0.0_t_real
          Endif

       Endif
       !!
       If(deb)Write(lun,19)Trim(data%name),erequire
       If(deb)Write(lun,119)Trim(data%name),u2require*hatoev/brtoan
       !!
       Call print_history(it,lun,data%name)
       !!
       !!
       If(count>1)Then
          If(debplot)Then
             Call create_line_plot_file(it,0_t_int,it% XN,res,mu)
             Call show_plot()
             Call plot_along_line('line search',it,Data,handler,lun&
                  &,0_t_int,it% XN,5_t_int,mesg)
          Endif
       Endif
       !!
       If(it% ee(res) < erequire ) Then
          If(count==1)Then
             Write(lun,20)Trim(data%name)
          Else
             Write(lun,220)Trim(data%name)
          Endif
          Call login_it(res,'0',it,lun,data%name) !! login at 0
          !!
          data% vector(1:it% nvec)=it% xx(1:it% nvec,0)
          Call handler(Data,it_entry_best)
          !!
          Goto 8888
          !!
       Else If(count == 1 .And. &
            & it% u2(0)< it% u2crossover .And.&
            & it% u2(res) < u2require ) Then
          Write(lun,21)Trim(data%name)
          Call login_it(res,'0',it,lun,data%name) !! login m+1
          data% vector(1:it% nvec)=it% xx(1:it% nvec,0)
          Call handler(Data,it_entry_best)
          Goto 8888
       Else If(count >= 2 .And. &
            & it% u2(res) < u2require ) Then
          Write(lun,216)Trim(data%name)
          Call login_it(res,'0',it,lun,data%name) !! login m+1
          data% vector(1:it% nvec)=it% xx(1:it% nvec,0)
          Call handler(Data,it_entry_best)
          Goto 8888
       Else
          If(count==1)Then
             If(deb)Write(lun,31)Trim(data%name)
          Endif
       Endif
       !!
!       If(count>=2)Then
!          If(it% ee(res) < it% ee(it% m) )Then
!             Call login_it(res,'e',it,lun,data%name)
!          Endif
!       Endif
       !!
       !!
       count=count+1
       If(count>3) Exit
    Enddo
    !!
    ret=FIRST_LINESEARCH_FAILED
    !!
    Return
8888 Continue
    !!
    If(deb) Write(lun,"('todo: pnew')")
    !If(pnew>0.0_t_real)Then
    !   it% p=pnew
    !Else
    !   it% p=Max(pmin,Min(pmax,it% p*preduce))
    !Endif
    ret=BETTER
    !!
    Return
    !!
2   Format(a,' ')
4   Format(a,' on line  mu          = ',e30.15e3)
12  Format(a,' calc at ',a)
13  Format(a,' ',a,'=',100(f12.5))
19  Format(a,' reduction upper bound E=',f17.10)
119 Format(a,' reduction upper bound u2=',f17.10)
20  Format(a,' hypothetical point is BETTER')
220 Format(a,' hypothetical point at line is BETTER')
21  Format(a,' hypothetical point is BETTER u2')
216 Format(a,' point along line is BETTER u2')
31  Format(a,' mu=1 not succesfull, start backtracking')
    !!
  End Subroutine line_search
  !!***********************************************************************
  

End Module itmethods
