Module ksaux
  !!
  Use precisiondefs
  Use file_iounits
  Use memory
  Use torbitalbasis
  Use constants
  Use tdiracdefs
  Use diracdefs
  Use tloi
  Use atomdefs
  Use ioaux, Only: kapToString
  Use optionstest
  Use diagonalize
  Use sort, Only: det_degbands
  Use prnt
  Use matserv
  Use tsymmetry
  Use tkohnsham
  Use tbzone
  !!
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="KSAUX"  
  !!
  Public :: rel_sym_check,backfold,check_RealatSym,analyze_S&
       &,check_Sym_coeff
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine check_Sym_coeff(symmetry,structure,basis,bzone,kohnsham)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_Sym_coeff"
    Logical(t_log) :: ismykvector
    Integer(t_int) :: jops,ikp,gikp,i,isa,isga,issa,ndegbands,j
    Integer(t_int) :: iva,dega,iga0,istga,nband,nvdim,iband
    Integer(t_int) :: la,ista,ia0,in1,in2,id,ms
    Real(t_real)   :: gk(3),dk(3),G(3),dgtau(3)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    Integer(t_int),Pointer :: degbands(:)
    Complex(t_real),Pointer:: p_Cg(:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    !Return
    ms=1
    !!
    If (.Not.isoption('FULLBZ'))Then
       Print *,'check_Sym_coeff:  need FULLBZ'
       Stop
    Endif
    !!
    Nullify(p_Cg)
    !!
    nband=kohnsham% nband
    nvdim=kohnsham% nvdim
    !!
    Call palloc('p_Cg',p_Cg,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    !!
    !!
    !!
    !!
    !!
    jopsloop: Do jops=1,symmetry% Rnumops
       Write(outfile,"(a)")Trim((symmetry% Ropsymbol(jops)))
       
       kloop: Do ikp=1,bzone% nkp
          gk(:)=Matmul(bzone% p_bk(:,ikp),symmetry% Ropmat(:,:,jops))
          gikp=-1
          ksearchloop: Do i=1,bzone% nkp
             dk=Matmul(gk-bzone% p_bk(:,i),symmetry% lattice_vectors(:,:))&
                  &/twopi
             Call backfold(dk)
             If(All(Abs(dk(:))<tol))Then
                gikp=i
                Exit ksearchloop
             Endif
          Enddo ksearchloop
          If(gikp<0)Then
             Print *,'not found g(ikp,jops,k)=',ikp,jops&
                  &,bzone% p_bk(:,ikp)
             Stop
          Endif
          !!
          G=gk-bzone% p_bk(:,gikp)

          !!
          dk=Matmul(bzone% p_bk(:,ikp),symmetry% lattice_vectors(:,:))&
               &/twopi

          ismykvector=.False.
          If(jops==16.And.Abs(dk(3)-0.5_t_real)<tol)Then
             ismykvector=.True.
             Print "(&
                  & 'k =',3f12.8,/&
                  &,'gk=',3f12.8,/&
                  &,'kg=',3f12.8,/&
                  &,'G =',3f12.8,/&
                  &)"&
                  &,Matmul(bzone% p_bk(:,ikp)&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(gk&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(bzone% p_bk(:,gikp)&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(G&
                  &,symmetry% lattice_vectors(:,:))/twopi
          Endif
          !!
          p_Cg=c0
          isaloop: Do isa=1,structure% nsite
             issa=structure% p_isrt(isa)
             isga=structure% p_equivsites(jops,isa)
             ista=kohnsham% p_vblockstart(isa)-1
             istga=kohnsham% p_vblockstart(isga)-1

             dgtau=structure% p_tau(:,isga)
             If(jops==16.And.ismykvector)Then
                Print *,ikp,gikp
                Print *,isa,isga
             Endif
             ia0=ista
             iga0=istga
             ivaloop: Do iva=1,basis% p_nRv(issa)
                dega=basis% p_degRv(iva,issa)
                la=basis% p_lRv(iva,issa)
                p_Cg(ia0+1:ia0+dega,1:nband)=p_Cg(ia0+1:ia0+dega,1:nband)&
                     &+Matmul(Transpose(&
                     &   symmetry% Rwu_ks(1:dega,1:dega,la,jops))&
                     &,Transpose(kohnsham% p_ccvk(1:nband,gikp,iga0+1:iga0+dega&
                     &            ,ms)))&
                     &*Exp(-ci*Dot_product(G,dgtau))
                !!
                ia0=ia0+dega
                iga0=iga0+dega
             End Do ivaloop
          Enddo isaloop
          !!
          Call palloc('degbands',degbands,1_t_int,nband+1,nmodule,nsubfunc)
          !!
          Call det_degbands(nband,kohnsham% p_ebk0(:,ikp,1),ndegbands,degbands&
               &,1.0e-6_t_real)
          !!
          Do id=1,ndegbands-1
             in1=degbands(id)
             in2=degbands(id+1)-1
             Do j=in1,in2
                !If(Any(degbands(2:ndegbands)-degbands(1:ndegbands-1)>1))&
                !     &Print *,i,j,kohnsham% p_ebk0(iband,ikp,1)
             Enddo
             !If(Any(degbands(2:ndegbands)-degbands(1:ndegbands-1)>1))&
             !     &Print *
             If(.Not.check_unitary_equivalence(&
                  &Transpose(kohnsham% p_ccvk(in1:in2,ikp,1:nvdim,ms))&
                  &,p_Cg(1:nvdim,in1:in2)&
                  &,tol))Then
                Write(outfile,"(&
                     &'error: op=',a,/&
                     &,'k (',i5,')=',3(f12.7,1x),/&
                     &,'gk(',i5,')=',3(f12.7,1x),/&
                     &,'iband=',i6,':',i6,' ek=',2f15.8)")&
                     & Trim(symmetry% Ropsymbol(jops))&
                     &,ikp,Matmul(bzone% p_bk(:,ikp)&
                     &,symmetry% lattice_vectors(:,:))/twopi&
                     &,gikp,Matmul(bzone% p_bk(:,gikp)&
                     &,symmetry% lattice_vectors(:,:))/twopi&
                     &,in1,in2,kohnsham% p_ebk0(in1,ikp,1)
                Do i=1,nvdim
                   Do iband=in1,in2
                      Write(outfile,"(4f12.7)",advance='no')&
                           &kohnsham% p_ccvk(iband,ikp,i,ms),p_Cg(i,iband)
                   Enddo
                   Write(outfile,*)
                Enddo
                !Pause
             Endif

          Enddo
          !!
       Enddo kloop
    Enddo jopsloop
    !!
    Call pfree(p_Cg)
    !!
    Stop
    !!
    Return
    !!
  End Subroutine check_Sym_coeff
  !!***********************************************************************
  Subroutine check_RealatSym(symmetry,structure,basis,bzone,kohnsham,Hk)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Complex(t_real),Intent(in)   :: Hk(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_RealatSym"
    Logical(t_log) :: timerev
    Integer(t_int) :: jops,ikp,gikp,i,isa,isb,isga,isgb,degmax,issa,issb
    Integer(t_int) :: iva,ivb,dega,degb,iga0,igb0,istga,istgb
    Integer(t_int) :: kapa,kapb,ista,istb,ia0,ib0
    Real(t_real)   :: gk(3),dk(3),G(3),dgtau(3)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    Complex(t_real),Pointer:: p_Ag(:,:)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Character(4),Parameter :: trev(0:1)=(/"  ","T*"/)
    !!
    !Return
    !!
    If (.Not.isoption('FULLBZ'))Then
       Print *,'check_RealatSym:  need FULLBZ'
       Stop
    Endif
    !!
    Nullify(p_Ag)
    !!
    degmax=basis% degRvmx
    !!
    Call palloc('p_Ag',p_Ag,1_t_int,degmax,1_t_int,degmax,nmodule,nsubfunc)
    !!
    !!
    Do jops=1,symmetry% Rnumops
       timerev=(symmetry% Roptimerev(jops)==1)
       If(timerev)Then
          Write(outfile,"('T*',a)")Trim(symmetry% Ropsymbol(jops))
       Else
          Write(outfile,"(a)")Trim((symmetry% Ropsymbol(jops)))
       Endif
       
       Do ikp=1,bzone% nkp
          gk(:)=Matmul(bzone% p_bk(:,ikp),symmetry% Ropmat(:,:,jops))
          If(timerev)gk=-gk
          gikp=-1
          ksearchloop: Do i=1,bzone% nkp
             dk=Matmul(gk-bzone% p_bk(:,i),symmetry% lattice_vectors(:,:))&
                  &/twopi
             Call backfold(dk)
             If(All(Abs(dk(:))<tol))Then
                gikp=i
                Exit ksearchloop
             Endif
          Enddo ksearchloop
          If(gikp<0)Then
             Print *,'not found g(ikp,jops,k)=',ikp,jops&
                  &,bzone% p_bk(:,ikp)
             Stop
          Endif
          !!
          G=gk-bzone% p_bk(:,gikp)
          !!
          Do isa=1,structure% nsite
             issa=structure% p_isrt(isa)
             isga=structure% p_equivsites(jops,isa)
             ista=kohnsham% p_vblockstart(isa)-1
             istga=kohnsham% p_vblockstart(isga)-1
             Do isb=1,structure% nsite
                issb=structure% p_isrt(isb)
                isgb=structure% p_equivsites(jops,isb)
                istb=kohnsham% p_vblockstart(isb)-1
                istgb=kohnsham% p_vblockstart(isgb)-1

                dgtau=structure% p_tau(:,isgb)-structure% p_tau(:,isga)

                ia0=ista
                iga0=istga
                Do iva=1,basis% p_nRv(issa)
                   dega=basis% p_degRv(iva,issa)
                   kapa=basis% p_kapRv(iva,issa)
                   ib0=istb
                   igb0=istgb
                   Do ivb=1,basis% p_nRv(issb)
                      degb=basis% p_degRv(ivb,issb)
                      kapb=basis% p_kapRv(ivb,issb)
                      If(timerev)Then
                         p_Ag(1:dega,1:degb)=&
                              &Matmul(Transpose(&
                              &   symmetry% Rwu_tj_ks(1:dega,1:dega,kapa,jops))&
                              &,Conjg(Matmul(&
                              &Hk(iga0+1:iga0+dega&
                              &            ,igb0+1:igb0+degb,gikp)&
                              &,((symmetry% Rwu_tj_ks(1:degb,1:degb,kapb,jops))))))&
                              &*Exp(-ci*Dot_product(G,dgtau))
                      Else
                         p_Ag(1:dega,1:degb)=&
                              &Matmul(Transpose(&
                              &   Conjg(symmetry% Rwu_tj_ks(1:dega,1:dega,kapa,jops)))&
                              &,Matmul(&
                              &Hk(iga0+1:iga0+dega&
                              &            ,igb0+1:igb0+degb,gikp)&
                              &,((symmetry% Rwu_tj_ks(1:degb,1:degb,kapb,jops)))))&
                              &*Exp(ci*Dot_product(G,dgtau))
                      Endif
                      If(Any(Abs(Hk(ia0+1:ia0+dega,ib0+1:ib0+degb,ikp)&
                           &-p_Ag(1:dega,1:degb))>tol))Then
                         Print "('error: op=',a,a,/&
                              &,'k (',i5,')=',3(f12.7,1x),/&
                              &,'gk(',i5,')=',3(f12.7,1x),/&
                              &,'isa,isb=',2i4,/&
                              &,'isga,isgb=',2i4,/&
                              &,'nka=',i4,a,/&
                              &,'nkb=',i4,a,/&
                              &)"&
                              &,trev(symmetry% Roptimerev(jops))&
                              &,Trim(symmetry% Ropsymbol(jops))&
                              &,ikp,Matmul(bzone% p_bk(:,ikp)&
                              &,symmetry% lattice_vectors(:,:))/twopi&
                              &,gikp,Matmul(bzone% p_bk(:,gikp)&
                              &,symmetry% lattice_vectors(:,:))/twopi&
                              &,isa,isb,isga,isgb&
                              &,basis% p_nnRv(iva,issa),kapToString(kapa)&
                              &,basis% p_nnRv(ivb,issb),kapToString(kapb)
                         
                         Print *,ia0,ib0
                         Print *,iga0,igb0
                         Call printmat('Re Hkab',"(f12.8,' ')","('     0       ')"&
                              &,Real(Hk(ia0+1:ia0+dega,ib0+1:ib0+degb,ikp)&
                              &,t_real)&
                              &,dega,degb,1.0e-6_t_real)

                         Call printmat('Im Hkab',"(f12.8,' ')","('     0       ')"&
                              &,Aimag(Hk(ia0+1:ia0+dega,ib0+1:ib0+degb,ikp))&
                              &,dega,degb,1.0e-6_t_real)

                         Call printmat('Re DHgkD',"(f12.8,' ')","('     0       ')"&
                              &,Real(p_Ag(1:dega,1:degb),t_real)&
                              &,dega,degb,1.0e-6_t_real)
                         Call printmat('Im DHgkD',"(f12.8,' ')","('     0       ')"&
                              &,Aimag(p_Ag(1:dega,1:degb))&
                              &,dega,degb,1.0e-6_t_real)


                         If(.True.)Then
                            Call printmat('Re Hgkgagb',"(f12.8,' ')","('     0       ')"&
                                 &,Real(Hk(iga0+1:iga0+dega,igb0+1:igb0+degb,gikp)&
                                 &,t_real)&
                                 &,dega,degb,1.0e-6_t_real)
                            Call printmat('Im Hgkgagb',"(f12.8,' ')","('     0       ')"&
                                 &,Aimag(Hk(iga0+1:iga0+dega,igb0+1:igb0+degb,gikp))&
                                 &,dega,degb,1.0e-6_t_real)
                            
                         Endif


                         Call printmat('delta',"(f12.8,' ')","('     0       ')"&
                              &,abs(Hk(ia0+1:ia0+dega,ib0+1:ib0+degb,ikp)&
                              &-p_Ag(1:dega,1:degb))&
                              &,dega,degb,1.0e-6_t_real)

                         !Pause
                      Endif
                      !!
                      ib0=ib0+degb
                      igb0=igb0+degb
                   End Do ! ivb 
                   ia0=ia0+dega
                   iga0=iga0+dega
                End Do ! iva
             Enddo !isb
          Enddo!isa
          !!
          !!
       Enddo

    Enddo
    !!
    Call pfree(p_Ag)
    !!
    Stop
    !!
    Return
    !!
  End Subroutine check_RealatSym
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine check_RealatSym_coeff(symmetry,structure,basis,bzone,kohnsham,Ck)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Complex(t_real),Intent(in)   :: Ck(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_RealatSym_coeff"
    Logical(t_log) :: timerev,ismykvector
    Integer(t_int) :: jops,ikp,gikp,i,isa,isga,degmax,issa,ndegbands,j
    Integer(t_int) :: iva,dega,iga0,istga,nband,nvdim,iband
    Integer(t_int) :: kapa,ista,ia0,in1,in2,id
    Real(t_real)   :: gk(3),dk(3),G(3),dgtau(3)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    Integer(t_int),Pointer :: degbands(:)
    Complex(t_real),Pointer:: p_Cg(:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Character(4),Parameter :: trev(0:1)=(/"  ","T*"/)
    !!
    !Return
    !!
    If (.Not.isoption('FULLBZ'))Then
       Print *,'check_RealatSym:  need FULLBZ'
       Stop
    Endif
    !!
    Nullify(p_Cg)
    !!
    degmax=basis% degRvmx
    nband=kohnsham% nband
    nvdim=kohnsham% nvdim
    !!
    Call palloc('p_Cg',p_Cg,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    !!
    !!
    !!
    !!
    !!
    jopsloop: Do jops=1,symmetry% Rnumops
       timerev=(symmetry% Roptimerev(jops)==1)
       If(timerev)Then
          Write(outfile,"('T*',a)")Trim(symmetry% Ropsymbol(jops))
       Else
          Write(outfile,"(a)")Trim((symmetry% Ropsymbol(jops)))
       Endif
       
       kloop: Do ikp=1,bzone% nkp
          gk(:)=Matmul(bzone% p_bk(:,ikp),symmetry% Ropmat(:,:,jops))
          If(timerev)gk=-gk
          gikp=-1
          ksearchloop: Do i=1,bzone% nkp
             dk=Matmul(gk-bzone% p_bk(:,i),symmetry% lattice_vectors(:,:))&
                  &/twopi
             Call backfold(dk)
             If(All(Abs(dk(:))<tol))Then
                gikp=i
                Exit ksearchloop
             Endif
          Enddo ksearchloop
          If(gikp<0)Then
             Print *,'not found g(ikp,jops,k)=',ikp,jops&
                  &,bzone% p_bk(:,ikp)
             Stop
          Endif
          !!
          G=gk-bzone% p_bk(:,gikp)

          !!
          dk=Matmul(bzone% p_bk(:,ikp),symmetry% lattice_vectors(:,:))&
               &/twopi

          ismykvector=.False.
          If(jops==16.And.Abs(dk(3)-0.5_t_real)<tol)Then
             ismykvector=.True.
             Print "(&
                  & 'k =',3f12.8,/&
                  &,'gk=',3f12.8,/&
                  &,'kg=',3f12.8,/&
                  &,'G =',3f12.8,/&
                  &)"&
                  &,Matmul(bzone% p_bk(:,ikp)&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(gk&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(bzone% p_bk(:,gikp)&
                  &,symmetry% lattice_vectors(:,:))/twopi&
                  &,Matmul(G&
                  &,symmetry% lattice_vectors(:,:))/twopi
          Endif
          !!
          p_Cg=c0
          isaloop: Do isa=1,structure% nsite
             issa=structure% p_isrt(isa)
             isga=structure% p_equivsites(jops,isa)
             ista=kohnsham% p_vblockstart(isa)-1
             istga=kohnsham% p_vblockstart(isga)-1

             dgtau=structure% p_tau(:,isga)
             If(jops==16.And.ismykvector)Then
                Print *,ikp,gikp
                Print *,isa,isga
             Endif
             ia0=ista
             iga0=istga
             ivaloop: Do iva=1,basis% p_nRv(issa)
                dega=basis% p_degRv(iva,issa)
                kapa=basis% p_kapRv(iva,issa)
                If(timerev)Then
                   p_Cg(ia0+1:ia0+dega,1:nband)=p_Cg(ia0+1:ia0+dega,1:nband)&
                        &+Conjg(&
                        &Matmul(Transpose(&
                        &   Conjg(symmetry% Rwu_tj_ks(1:dega,1:dega,kapa,jops)))&
                        &,Ck(iga0+1:iga0+dega&
                        &            ,1:nband,gikp))&
                        &*Exp(-ci*Dot_product(G,dgtau))&
                        &)
                Else
                   p_Cg(ia0+1:ia0+dega,1:nband)=p_Cg(ia0+1:ia0+dega,1:nband)&
                        &+Matmul(Transpose(&
                        &   Conjg(symmetry% Rwu_tj_ks(1:dega,1:dega,kapa,jops)))&
                        &,Ck(iga0+1:iga0+dega&
                        &            ,1:nband,gikp))&
                        &*Exp(-ci*Dot_product(G,dgtau))
                Endif
                !!
                ia0=ia0+dega
                iga0=iga0+dega
             End Do ivaloop
          Enddo isaloop
          !!
          Call palloc('degbands',degbands,1_t_int,nband+1,nmodule,nsubfunc)
          !!
          Call det_degbands(nband,kohnsham% p_ebk0(:,ikp,1),ndegbands,degbands&
               &,1.0e-6_t_real)
          !!
          Do id=1,ndegbands-1
             in1=degbands(id)
             in2=degbands(id+1)-1
             Do j=in1,in2
                !If(Any(degbands(2:ndegbands)-degbands(1:ndegbands-1)>1))&
                !     &Print *,i,j,kohnsham% p_ebk0(iband,ikp,1)
             Enddo
             !If(Any(degbands(2:ndegbands)-degbands(1:ndegbands-1)>1))&
             !     &Print *
             If(.Not.check_unitary_equivalence(&
                  &Ck(1:nvdim,in1:in2,ikp)&
                  &,p_Cg(1:nvdim,in1:in2)&
                  &,tol))Then
                Write(outfile,"(&
                     &'error: op=',a,a,/&
                     &,'k (',i5,')=',3(f12.7,1x),/&
                     &,'gk(',i5,')=',3(f12.7,1x),/&
                     &,'iband=',i6,':',i6,' ek=',2f15.8)")&
                     & trev(symmetry% Roptimerev(jops))&
                     &,Trim(symmetry% Ropsymbol(jops))&
                     &,ikp,Matmul(bzone% p_bk(:,ikp)&
                     &,symmetry% lattice_vectors(:,:))/twopi&
                     &,gikp,Matmul(bzone% p_bk(:,gikp)&
                     &,symmetry% lattice_vectors(:,:))/twopi&
                     &,in1,in2,kohnsham% p_ebk0(in1,ikp,1)
                Do i=1,nvdim
                   Do iband=in1,in2
                      Write(outfile,"(4f12.7)",advance='no')&
                           &Ck(i,iband,ikp),p_Cg(i,iband)
                   Enddo
                   Write(outfile,*)
                Enddo
                !Pause
             Endif

          Enddo
          !!
       Enddo kloop
    Enddo jopsloop
    !!
    Call pfree(p_Cg)
    !!
    Stop
    !!
    Return
    !!
  End Subroutine check_RealatSym_coeff
#endif
  !!***********************************************************************
  Logical Function check_unitary_equivalence(A,B,tol)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)      :: tol
    Complex(t_real),Intent(in)   :: A(:,:)
    Complex(t_real),Intent(inout):: B(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_unitary_equivalence"
    Integer(t_int) :: nvdim,nband,i
    Complex(t_real),Pointer:: U(:,:)
    Complex(t_real),Pointer:: S(:,:)
    Real(t_real),Pointer :: E(:)
    !!
    Nullify(U,S,E)
    !!
    nvdim=Size(A,1)
    nband=Size(A,2)
    !!
    Call palloc('U',U,1_t_int,nband,1_t_int,nband,nmodule,nsubfunc)
    Call palloc('S',S,1_t_int,nband,1_t_int,nband,nmodule,nsubfunc)
    Call palloc('E',E,1_t_int,nband,nmodule,nsubfunc)
    !!
    !! A=B*U
    !! we need to establish unitary B first, since we have a non-orthognal
    !! basis we got B^{+}*S*B=1 and not B^{+}*B=1
    !!
    !! U=1/(B^{+}*B)*B^{+}*A
    !!
    !! then B*U = P_B*A, where P_B =B*1/S_B*B, S_B=B^{+}*B
    !! hence if A is the same subspace as B we get B*U=P_B*A=A
    !!
    !! 
    !!
    S=Matmul(Transpose(Conjg(B)),B)
    !!
    !!
    Call cdiagno_noO(nband,S,E)
    !!
    !! 1/S*B^{+}*A = Z*1/E*Z^{+}*B^{+}*A
    !!
    U=Matmul(Transpose(Conjg(B)),A)
    U=Matmul(Transpose(Conjg(S)),U)
    Do i=1,nband
       If(Abs(E(i))<1.0e-6_t_real)Then
          Print *,'B^{+}*B indefinite'
          Stop
       Endif
       U(i,:)=U(i,:)/E(i)
       !! todo check E for zero
    Enddo
    U=Matmul(S,U)
    !!
    !!
    !!
    !! now U should be 1
    S=Matmul(Transpose(Conjg(U)),U)
    !!
    Do i=1,nband
       S(i,i)=S(i,i)-1.0_t_real
    Enddo
    !!
    !B=Matmul(B,U)
    !!
    check_unitary_equivalence=All(Abs(S)<tol)
    If(.Not.check_unitary_equivalence)Then
       Call printmat('S_B',"('(',f12.8,',',f12.8,') ')"&
            &,"('       0          ')"&
            &,Matmul(Transpose(Conjg(B)),B),nband,nband,1.0e-6_t_real)
       Call printmat('A^+*A',"('(',f12.8,',',f12.8,') ')","('       0          ')"&
            &,Matmul(Transpose(Conjg(A)),A),nband,nband,1.0e-6_t_real)
       Call printmat('B^+*A',"('(',f12.8,',',f12.8,') ')","('       0          ')"&
            &,Matmul(Transpose(Conjg(B)),A),nband,nband,1.0e-6_t_real)
       Call printmat('U',"('(',f12.8,',',f12.8,') ')","('       0          ')"&
            &,U,nband,nband,1.0e-6_t_real)
    Endif
    !!
    Call pfree(U)
    Call pfree(S)
    Call pfree(E)
    !!
    Return
    !!
  End Function check_unitary_equivalence
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine f_analyzse(ghp,gsp,hp,nsite,ndim,zzT,eb,bw&
       &,vblockstart,vblockend,nmode)
    !!
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nsite,nmode(nsite),ndim
    Integer(t_int),Intent(in)    :: vblockstart(:),vblockend(:)
    Real(t_real),Intent(in)      :: eb(:)
    Real(t_real),Intent(in)      :: bw(:)
    Complex(t_real),Intent(in)   :: zzT(:,:)
    Complex(t_real),Intent(in)   :: ghp(:,:,:)
    Complex(t_real),Intent(in)   :: gsp(:,:,:)
    Complex(t_real),Intent(in)   :: hp(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="f_analyzse"
    Integer(t_int) :: i,ik,isa,ncdim,nvdim
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Real(t_real),Pointer :: sdelta(:,:,:),csdelta(:,:,:),vsdelta(:,:,:)
    Real(t_real)   :: fS(3),foScc(3),foSvv(3),foSdvc(3),foSvcd(3),fH0(3)
    Real(t_real)   :: foHvv(3),foHdvc(3),foHvcd(3),foHcc(3),foHdcc(3)
    Real(t_real)   :: foHdses(3),foHsesd(3),foHdsh(3),foHshd(3),fHcor(3)
    Complex(t_real):: zz(ndim,ndim),zzp(ndim,ndim),ee(ndim,ndim),nn(ndim,ndim)
    Complex(t_real),Pointer:: zzcc(:,:)
    Complex(t_real),Pointer:: eecc(:,:),eevc(:,:),eecv(:,:),eevv(:,:)
    Complex(t_real),Pointer:: nncc(:,:),nnvc(:,:),nncv(:,:),nnvv(:,:)
    Complex(t_real),Pointer:: nncc0(:,:),eecc0(:,:),Hcc(:,:),Hcv(:,:)
    Complex(t_real),Pointer:: gspcc(:,:,:),gspvc(:,:,:),gspcv(:,:,:),gspvv(:&
         &,:,:)
    Complex(t_real),Pointer:: ghpcc(:,:,:),ghpvc(:,:,:),ghpcv(:,:,:),ghpvv(:,:,:)
    Real(t_real),Pointer :: eebcc(:,:)
    Real(t_real)   :: eeb(ndim,ndim),wwb(ndim,ndim)
    Real(t_real),Parameter   :: two=2.0_t_real
    !Real(t_real),Parameter   :: tol=1.0e-10_t_real

    !!
    Write(outfile,"(/,72('='),/)")
    !!
    Call prep_f_reduce(nsite,nmode,ndim,vblockstart,vblockend&
         &,ncdim,nvdim,sub_c,sub_v,sdelta,csdelta,vsdelta&
         &,gspcc,gspcv,gspvc,gspvv&
         &,ghpcc,ghpcv,ghpvc,ghpvv&
         &,eecc,eecv,eevc,eevv&
         &,nncc,nncv,nnvc,nnvv&
         &,zzcc,nncc0,eecc0,eebcc,Hcc,Hcv&
         &)
    !!
    !!
    zz=Transpose(zzT)
    zzp=Transpose(Conjg(zz))
    !!
    eeb=0.0_t_real
    wwb=0.0_t_real
    Do i=1,ndim
       eeb(i,i)=-eb(i)*bw(i)
       wwb(i,i)=bw(i)
    Enddo
    eebcc=0.0_t_real
    Do i=1,ncdim
       eebcc(i,i)=eb(i)
    Enddo
    !!
    ee=Matmul(Matmul(zz,eeb),zzp)
    nn=Matmul(Matmul(zz,wwb),zzp)
    !!
    zzcc=zz(sub_c,1:ncdim)
    eecc0=Matmul(zzcc,Matmul(eebcc,Transpose(Conjg(zzcc))))*two!!TODO: mspin
    nncc0=Matmul(zzcc,Transpose(Conjg(zzcc)))*two!!TODO: mspin
    !!
    !! nncc0=2*CC^+=2*S_cc^-1
    !!
    !Print *,'compile gsp'
    !!
    Do ik=1,3
       gspcc(ik,:,:)=gsp(ik,sub_c,sub_c)
       gspcv(ik,:,:)=gsp(ik,sub_c,sub_v)
       gspvc(ik,:,:)=gsp(ik,sub_v,sub_c)
       gspvv(ik,:,:)=gsp(ik,sub_v,sub_v)
       ghpcc(ik,:,:)=ghp(ik,sub_c,sub_c)
       ghpcv(ik,:,:)=ghp(ik,sub_c,sub_v)
       ghpvc(ik,:,:)=ghp(ik,sub_v,sub_c)
       ghpvv(ik,:,:)=ghp(ik,sub_v,sub_v)
    Enddo
    !!
    !Print *,'compile ee'
    !!
    eecc(:,:)=ee(sub_c,sub_c)
    eecv(:,:)=ee(sub_c,sub_v)
    eevc(:,:)=ee(sub_v,sub_c)
    eevv(:,:)=ee(sub_v,sub_v)
    !!
    nncc(:,:)=nn(sub_c,sub_c)
    nncv(:,:)=nn(sub_c,sub_v)
    nnvc(:,:)=nn(sub_v,sub_c)
    nnvv(:,:)=nn(sub_v,sub_v)
    !!
    Hcc(:,:)=hp(sub_c,sub_c)
    Hcv(:,:)=hp(sub_c,sub_v)
    !!
    !Print *,'compile forces'

    !!
    Do isa=1,nsite
       Write(outfile,"('isa=',i5)")isa
       Do ik=1,3
          foScc(ik)=two&
               &*re_trace(Matmul(csdelta(:,:,isa),Matmul(gspcc(ik,:,:),eecc)))
          foSdvc(ik)=two&
               &*re_trace(Matmul(vsdelta(:,:,isa),Matmul(gspvc(ik,:,:),eecv)))
          foSvcd(ik)=two&
               &*re_trace(Matmul(csdelta(:,:,isa),Matmul(gspcv(ik,:,:),eevc)))
          foSvv(ik)=two&
               &*re_trace(Matmul(vsdelta(:,:,isa),Matmul(gspvv(ik,:,:),eevv)))

          foHcc(ik)=two&
               &*re_trace(Matmul(csdelta(:,:,isa),Matmul(ghpcc(ik,:,:),nncc0)))
          foHdcc(ik)=two&
               &*re_trace(Matmul(csdelta(:,:,isa),Matmul(ghpcc(ik,:,:),nncc-nncc0)))
          foHdvc(ik)=two&
               &*re_trace(Matmul(vsdelta(:,:,isa),Matmul(ghpvc(ik,:,:),nncv)))
          foHvcd(ik)=two&
               &*re_trace(Matmul(csdelta(:,:,isa),Matmul(ghpcv(ik,:,:),nnvc)))
          foHvv(ik)=two&
               &*re_trace(Matmul(vsdelta(:,:,isa),Matmul(ghpvv(ik,:,:),nnvv)))



          foHdses(ik)=-two*re_trace(Matmul(vsdelta(:,:,isa),Matmul(gspvc(ik,:&
               &,:),Matmul(nncc0/two,Matmul(Hcc,nncv)))))

          foHsesd(ik)= two*re_trace(Matmul(gspvc(ik,:,:),Matmul(csdelta(:,:&
               &,isa),Matmul(nncc0/two,Matmul(Hcc,nncv)))))

          foHdsh(ik)=-two*re_trace(Matmul(Matmul(Matmul(Matmul(vsdelta(:,:&
               &,isa),gspvc(ik,:,:)),nncc0/two),Hcv),nnvv))

          foHshd(ik)= two*re_trace(Matmul(Matmul(Matmul(Matmul(gspvc(ik,:,:)&
               &,vsdelta(:,:,isa)),nncc0/two),Hcv),nnvv))

          fHcor(ik)=foHdses(ik)+foHsesd(ik)+foHdsh(ik)+foHshd(ik)

          fS(ik)=two*re_trace(Matmul(sdelta(:,:,isa),Matmul(gsp(ik,:,:),ee)))
          fH0(ik)=two*re_trace(Matmul(sdelta(:,:,isa),Matmul(ghp(ik,:,:),nn)))
       Enddo
       Write(outfile,9)isa
       Write(outfile,10) 'fdetails: Scc       =',foScc(:)
       Write(outfile,10) 'fdetails: Svcd      =',foSvcd(:)
       Write(outfile,10) 'fdetails: Sdvc      =',foSdvc(:)
       Write(outfile,10) 'fdetails: Svv       =',foSvv(:)
       Write(outfile,10) 'fdetails: S         =',fS(:)
       Write(outfile,8)
       Write(outfile,10) 'fdetails: Hcc       =',foHcc(:)
       Write(outfile,10) 'fdetails: Hdcc      =',foHdcc(:)
       Write(outfile,10) 'fdetails: Hvcd      =',foHvcd(:)
       Write(outfile,10) 'fdetails: Hdvc      =',foHdvc(:)
       Write(outfile,8)
       Write(outfile,10) 'fdetails: HdSeS     =',foHdses(:)
       Write(outfile,10) 'fdetails: HSeSd     =',foHsesd(:)
       Write(outfile,10) 'fdetails: HdSH      =',foHdsh(:)
       Write(outfile,10) 'fdetails: HSHd      =',foHshd(:)
       Write(outfile,10) 'fdetails: Hvv       =',foHvv(:)
       Write(outfile,8)
       Write(outfile,10) 'fdetails: Hcor      =',fHcor(:)
       Write(outfile,10) 'fdetails: H0        =',fH0(:)
       Write(outfile,10) 'fdetails: H         =',fH0(:)+fHcor(:)

8      Format('fdetails: ')
9      Format('fdetails: S isa=',i3)
10     Format(a,1x,3(f20.10,2x))

    Enddo
    !!
    !!
    Call clean_f_reduce(sub_c,sub_v,sdelta,csdelta,vsdelta&
         &,gspcc,gspcv,gspvc,gspvv&
         &,ghpcc,ghpcv,ghpvc,ghpvv&
         &,eecc,eecv,eevc,eevv&
         &,nncc,nncv,nnvc,nnvv&
         &,zzcc,nncc0,eecc0,eebcc,Hcc,Hcv)
    !!
    !!
    Write(outfile,"(/,72('='),/)")
    !!
    Return
    !!
  End Subroutine f_analyzse
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine prep_f_reduce(nsite,nmode,ndim,vblockstart,vblockend&
       &,ncdim,nvdim,sub_c,sub_v,sdelta,csdelta,vsdelta&
       &,gspcc,gspcv,gspvc,gspvv&
       &,ghpcc,ghpcv,ghpvc,ghpvv&
       &,eecc,eecv,eevc,eevv&
       &,nncc,nncv,nnvc,nnvv&
       &,zzcc,nncc0,eecc0,eebcc,Hcc,Hcv)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nsite,nmode(nsite),ndim
    Integer(t_int),Intent(in)    :: vblockstart(:),vblockend(:)
    Integer(t_int),Intent(out)   :: ncdim,nvdim
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Real(t_real),Pointer   :: sdelta(:,:,:),csdelta(:,:,:),vsdelta(:,:,:)
    Complex(t_real),Pointer:: gspcc(:,:,:),gspcv(:,:,:),gspvc(:,:,:),gspvv(:,:,:)
    Complex(t_real),Pointer:: ghpcc(:,:,:),ghpcv(:,:,:),ghpvc(:,:,:),ghpvv(:,:,:)
    Complex(t_real),Pointer:: eecc(:,:),eecv(:,:),eevc(:,:),eevv(:,:)
    Complex(t_real),Pointer:: nncc(:,:),nncv(:,:),nnvc(:,:),nnvv(:,:)
    Complex(t_real),Pointer:: zzcc(:,:),nncc0(:,:),eecc0(:,:)
    Complex(t_real),Pointer:: Hcc(:,:),Hcv(:,:)
    Real(t_real),Pointer :: eebcc(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_f_reduce"
    Integer(t_int) :: jc,jv,isa,i
    !!
    !!
    Nullify(sub_c)
    Nullify(sub_v)
    Nullify(sdelta)
    Nullify(csdelta)
    Nullify(vsdelta)
    Nullify(gspcc,gspcv,gspvc,gspvv)
    Nullify(ghpcc,ghpcv,ghpvc,ghpvv)
    Nullify(eecc,eecv,eevc,eevv)
    Nullify(nncc,nncv,nnvc,nnvv)
    Nullify(zzcc,nncc0,eecc0,eebcc,Hcc,Hcv)
    !!
    ncdim=Sum(nmode(1:nsite))
    nvdim=ndim-ncdim
    !!
    Write(outfile,"('ncdim=',i5)") ncdim
    Write(outfile,"('nvdim=',i5)") nvdim
    !!
    Call palloc('sub_c',sub_c,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('sub_v',sub_v,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('sdelta',sdelta,1_t_int,ndim,1_t_int,ndim,1_t_int,nsite&
         &,nmodule,nsubfunc)
    Call palloc('csdelta',csdelta,1_t_int,ncdim,1_t_int,ncdim,1_t_int,nsite&
         &,nmodule,nsubfunc)
    Call palloc('vsdelta',vsdelta,1_t_int,nvdim,1_t_int,nvdim,1_t_int,nsite&
         &,nmodule,nsubfunc)
    !!
    jc=0
    jv=0
    Do isa=1,nsite
       !Write(outfile,"('isa=',i5)") isa
       Do i=1,nmode(isa)
          jc=jc+1
          sub_c(jc)=vblockstart(isa)-1+i
          !Write(outfile,"('sub(',i3,')=',i5)") jc,sub_c(jc)
       Enddo
       Do i=vblockstart(isa)+nmode(isa),vblockend(isa)
          jv=jv+1
          sub_v(jv)=i
          !Write(outfile,"('sub(',i3,')=',i5)") jv,sub_v(jv)
       Enddo
    Enddo
    !!
    
    Write(outfile,"('sub_c=',/,5(i5))") sub_c(1:ncdim)
    Write(outfile,"('sub_v=',/,5(i5))") sub_v(1:nvdim)
    Write(outfile,*)
    !!
    Do isa=1,nsite
       sdelta(:,:,isa)=0.0_t_real
       Do i=vblockstart(isa),vblockend(isa)
          sdelta(i,i,isa)=1.0_t_real
       Enddo
       csdelta(:,:,isa)=sdelta(sub_c,sub_c,isa)
       vsdelta(:,:,isa)=sdelta(sub_v,sub_v,isa)
    Enddo
    !!
    !!
    Call palloc('gspcc',gspcc,1_t_int,3_t_int,1_t_int,ncdim,1_t_int,ncdim&
         &,nmodule,nsubfunc)
    Call palloc('gspcv',gspcv,1_t_int,3_t_int,1_t_int,ncdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('gspvc',gspvc,1_t_int,3_t_int,1_t_int,nvdim,1_t_int,ncdim&
         &,nmodule,nsubfunc)
    Call palloc('gspvv',gspvv,1_t_int,3_t_int,1_t_int,nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    !!
    Call palloc('ghpcc',ghpcc,1_t_int,3_t_int,1_t_int,ncdim,1_t_int,ncdim&
         &,nmodule,nsubfunc)
    Call palloc('ghpcv',ghpcv,1_t_int,3_t_int,1_t_int,ncdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('ghpvc',ghpvc,1_t_int,3_t_int,1_t_int,nvdim,1_t_int,ncdim&
         &,nmodule,nsubfunc)
    Call palloc('ghpvv',ghpvv,1_t_int,3_t_int,1_t_int,nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    !!
    Call palloc('eecc',eecc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('eecv',eecv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('eevc',eevc,1_t_int,nvdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('eevv',eevv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    Call palloc('nncc',nncc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('nncv',nncv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('nnvc',nnvc,1_t_int,nvdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('nnvv',nnvv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    Call palloc('zzcc',zzcc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('nncc0',nncc0,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('eecc0',eecc0,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    !!
    Call palloc('eebcc',eebcc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('Hcc',Hcc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('Hcv',Hcv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    !!
    !Print *,'END ',nsubfunc
    Return
    !!
  End Subroutine prep_f_reduce
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine  clean_f_reduce(sub_c,sub_v,sdelta,csdelta,vsdelta&
       &,gspcc,gspcv,gspvc,gspvv&
       &,ghpcc,ghpcv,ghpvc,ghpvv&
       &,eecc,eecv,eevc,eevv&
       &,nncc,nncv,nnvc,nnvv&
       &,zzcc,nncc0,eecc0,eebcc,Hcc,Hcv)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Real(t_real),Pointer   :: sdelta(:,:,:),csdelta(:,:,:),vsdelta(:,:,:)
    Complex(t_real),Pointer:: gspcc(:,:,:),gspcv(:,:,:),gspvc(:,:,:),gspvv(:&
         &,:,:)
    Complex(t_real),Pointer:: ghpcc(:,:,:),ghpcv(:,:,:),ghpvc(:,:,:),ghpvv(:&
         &,:,:)
    Complex(t_real),Pointer:: eecc(:,:),eecv(:,:),eevc(:,:),eevv(:,:)
    Complex(t_real),Pointer:: nncc(:,:),nncv(:,:),nnvc(:,:),nnvv(:,:)
    Complex(t_real),Pointer:: zzcc(:,:),nncc0(:,:),eecc0(:,:)
    Complex(t_real),Pointer:: Hcc(:,:),Hcv(:,:)
    Real(t_real),Pointer :: eebcc(:,:)
    !!LOCAL
    !!
    Call pfree(sub_c)
    Call pfree(sub_v)
    Call pfree(sdelta)
    Call pfree(csdelta)
    Call pfree(vsdelta)
    Call pfree(gspcc)
    Call pfree(gspcv)
    Call pfree(gspvc)
    Call pfree(gspvv)

    Call pfree(ghpcc)
    Call pfree(ghpcv)
    Call pfree(ghpvc)
    Call pfree(ghpvv)

    Call pfree(eecc)
    Call pfree(eecv)
    Call pfree(eevc)
    Call pfree(eevv)
    Call pfree(nncc)
    Call pfree(nncv)
    Call pfree(nnvc)
    Call pfree(nnvv)
    !!
    Call pfree(zzcc)
    Call pfree(nncc0)
    Call pfree(eecc0)
    Call pfree(eebcc)
    Call pfree(Hcc)
    Call pfree(Hcv)
    !!
    Return
    !!
  End Subroutine clean_f_reduce
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine f_analyzse0(ghp,gsp,ndim,zzT,eb,bw,vblockstart,vblockend,nmode)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nmode,ndim
    Integer(t_int),Intent(in)    :: vblockstart(*)
    Integer(t_int),Intent(in)    :: vblockend(*)
    Complex(t_real),Intent(in)   :: ghp(:,:,:)
    Complex(t_real),Intent(in)   :: gsp(:,:,:)
    Complex(t_real),Intent(in)   :: zzT(:,:)
    Real(t_real),Intent(in)      :: eb(:)
    Real(t_real),Intent(in)      :: bw(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="f_analyzse0"
    Integer(t_int) :: ncdim,nvdim,i,j,kk
    Complex(t_real):: zz(Size(zzT,1),Size(zzT,1))
    !!
    Real(t_real)   :: tol,tol0
    Real(t_real)   :: fo(3),ff(3),foS(3),foH(3),foC(3),fovc(3)
    !!
    Integer(t_int) :: cindex(2*nmode),zzcindex(2*nmode)
    Integer(t_int) :: vindex(ndim-2*nmode),zzvindex(ndim-2*nmode)
    Real(t_real)   :: csdelta(2*nmode,2*nmode)
    Real(t_real)   :: vsdelta(ndim-2*nmode,ndim-2*nmode)
    !!
    Real(t_real)   :: eCcc(2*nmode,2*nmode),nCcc(2*nmode,2*nmode)
    Real(t_real)   :: eCvc(ndim-2*nmode,2*nmode),nCvc(ndim-2*nmode,2*nmode)
    Real(t_real)   :: eCvv(ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: nCvv(ndim-2*nmode,ndim-2*nmode)
    !!
    Real(t_real)   :: eVcc(2*nmode,2*nmode),nVcc(2*nmode,2*nmode)
    Real(t_real)   :: eVvc(ndim-2*nmode,2*nmode),nVvc(ndim-2*nmode,2*nmode)
    Real(t_real)   :: eVvv(ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: nVvv(ndim-2*nmode,ndim-2*nmode)
    !!
    Real(t_real)   :: ec(2*nmode),eec(2*nmode,2*nmode)
    Real(t_real)   :: ev(ndim-2*nmode),eev(ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: bwv(ndim-2*nmode),bwvd(ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: zzcc(2*nmode,2*nmode)
    Real(t_real)   :: zzcv(2*nmode,ndim-2*nmode)
    Real(t_real)   :: zzvc(ndim-2*nmode,2*nmode)
    Real(t_real)   :: zzvv(ndim-2*nmode,ndim-2*nmode)
    !!
    Real(t_real)   :: ghpcc(3,2*nmode,2*nmode)
    Real(t_real)   :: ghpvc(3,ndim-2*nmode,2*nmode)
    Real(t_real)   :: ghpvv(3,ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: gspcc(3,2*nmode,2*nmode)
    Real(t_real)   :: gspLvc(3,ndim-2*nmode,2*nmode)
    Real(t_real)   :: gspvLc(3,ndim-2*nmode,2*nmode)
    Real(t_real)   :: gspvv(3,ndim-2*nmode,ndim-2*nmode)
    Real(t_real)   :: gspLvv(3,ndim-2*nmode,ndim-2*nmode)
    !!
    !!
    !!
    tol=1.0e-10_t_real
    tol0=1.0e-14_t_real
    ncdim=2*nmode
    nvdim=ndim-ncdim
    zz=Transpose(zzT)
    !!
    Do kk=1,3
!!$       Call printmat('Re GScc',"(f15.8,' ')","('       0        ')"&
!!$            &,Real(gsp(kk,:,:),t_real),ndim,ndim,tol)
       If(Any(Aimag(gsp(kk,:,:))>tol))Then
          Call printmat('Im GScc',"(f15.8,' ')","('       0        ')"&
               &,Aimag(gsp(kk,:,:)),ndim,ndim,tol)
          Stop 'Im gsp  /=0 '
       Endif
       If(Any(Aimag(ghp(kk,:,:))>tol))Then
          Call printmat('Im GHcc',"(f15.8,' ')","('       0        ')"&
               &,Aimag(ghp(kk,:,:)),ndim,ndim,tol)
          Stop 'Im ghp  /=0 '
       Endif
    Enddo
!!$    Call printmat('Re ZZ',"(f15.8,' ')","('       0        ')"&
!!$         &,Real(zz,t_real),ndim,ndim,tol)
    If(Any(Aimag(zz)>tol))Then
       Call printmat('Im ZZ',"(f15.8,' ')","('       0        ')"&
            &,Aimag(zz),ndim,ndim,tol)
       Stop 'Im ZZ  /=0 '
    Endif
    !!
    !!
    cindex=(/(i,i=1,nmode)&
         &,(i,i=vblockstart(2),vblockstart(2)+nmode-1)/)
    vindex=(/(i,i=vblockstart(1)+nmode,vblockend(1))&
         &,(i,i=vblockstart(2)+nmode,vblockend(2))/)
    zzcindex=(/(i,i=1,nmode)/)
    zzvindex=(/(i,i=2*nmode+1,ndim)/)
    !!
    csdelta=0.0_t_real
    Do i=1,ncdim/2
       csdelta(i,i)=1.0_t_real
    Enddo
    vsdelta=0.0_t_real
    Do i=1,nvdim/2
       vsdelta(i,i)=1.0_t_real
    Enddo
    !!
    !!
    !!
    !!
    ec=eb(zzcindex)
    ev=eb(zzvindex)
    bwv=bw(zzvindex)
    Do i=1,ncdim
       Do j=1,ncdim
          eec(i,j)=0.0_t_real
          If(i==j) eec(i,j)=ec(i)
       Enddo
    Enddo
    Do i=1,nvdim
       Do j=1,nvdim
          bwvd(i,j)=0.0_t_real
          eev(i,j)=0.0_t_real
          If(i==j) bwvd(i,j)=bwv(i)
          If(i==j) eev(i,j)=ev(i)
       Enddo
    Enddo
    !!
    zzcc=Real(zz(cindex,zzcindex),t_real)
    zzvc=Real(zz(vindex,zzcindex),t_real)
    zzcv=Real(zz(cindex,zzvindex),t_real)
    zzvv=Real(zz(vindex,zzvindex),t_real)
    !!
    !!
    !Call printmat('Re ZZcc',"(f15.8,' ')","('       0        ')"&
    !     &,Real(zzcc,t_real),ncdim,ncdim,tol)
    !!
    !!
    !!
    eCcc=-Matmul(zzcc,Matmul(eec,Transpose(zzcc)))*2.0_t_real
    nCcc=Matmul(zzcc,Transpose(zzcc))*2.0_t_real
    eCvc=-Matmul(zzvc,Matmul(eec,Transpose(zzcc)))*2.0_t_real
    nCvc=Matmul(zzvc,Transpose(zzcc))*2.0_t_real
    eCvv=-Matmul(zzvc,Matmul(eec,Transpose(zzvc)))*2.0_t_real
    nCvv=Matmul(zzvc,Transpose(zzvc))*2.0_t_real
    !!
    eVcc=-Matmul(zzcv,Matmul(Matmul(eev,bwvd),Transpose(zzcv)))
    nVcc=Matmul(zzcv,Matmul(bwvd,Transpose(zzcv)))
    eVvc=-Matmul(zzvv,Matmul(Matmul(eev,bwvd),Transpose(zzcv)))
    nVvc=Matmul(zzvv,Matmul(bwvd,Transpose(zzcv)))
    eVvv=-Matmul(zzvv,Matmul(Matmul(eev,bwvd),Transpose(zzvv)))
    nVvv=Matmul(zzvv,Matmul(bwvd,Transpose(zzvv)))
    !!
    !!
    !!
    If(Any(Abs(eCvc)>tol)) Then
       Stop 'eCvc /=0 ?'
    Endif
    If(Any(Abs(eCvv)>tol)) Then
       Stop 'eCvv /=0 ?'
    Endif
    If(Any(Abs(nCvc)>tol)) Then
       Stop 'nCvc /=0 ?'
    Endif
    If(Any(Abs(nCvv)>tol)) Then
       Stop 'nCvv /=0 ?'
    Endif

    !!

    Do kk=1,3
       If(Any(Abs(gsp(kk,cindex,vindex)+Transpose(gsp(kk,vindex,cindex)))>tol))Then
          !Print *,Maxval((gsp(kk,cindex,vindex)+Transpose(gsp(kk,vindex,cindex))))
          Stop 'SLcv /= - (SLvc)^+'
       Endif
    Enddo
    !!
!!$    Call printmat('Re eCcc',"(f15.8,' ')","('       0        ')"&
!!$         &,eCcc,ncdim,ncdim,tol)
!!$    Call printmat('Re nCcc',"(f15.8,' ')","('       0        ')"&
!!$         &,nCcc,ncdim,ncdim,tol)
!!$    Call printmat('Re eCvc',"(f15.8,' ')","('       0        ')"&
!!$         &,eCvc,nvdim,ncdim,tol)
!!$    Call printmat('Re nCvc',"(f15.8,' ')","('       0        ')"&
!!$         &,nCvc,nvdim,ncdim,tol)
    !!
    !!
    Do kk=1,3
       gspcc(kk,:,:)=Real(Matmul(csdelta,gsp(kk,cindex,cindex))&
            &+Matmul(Transpose(gsp(kk,cindex,cindex)),csdelta),t_real)

       ghpcc(kk,:,:)=Real(Matmul(csdelta,ghp(kk,cindex,cindex))&
            &+Matmul(Transpose(ghp(kk,cindex,cindex)),csdelta),t_real)

       gspLvc(kk,:,:)=Real(Matmul(vsdelta,gsp(kk,vindex,cindex)),t_real)

       gspvLc(kk,:,:)=-Real(Matmul(gsp(kk,vindex,cindex),csdelta),t_real)

       
       ghpvc(kk,:,:)=Real(Matmul(vsdelta,ghp(kk,vindex,cindex))&
            &+Matmul(Transpose(ghp(kk,cindex,vindex)),csdelta),t_real)

       gspvv(kk,:,:)=Real(Matmul(vsdelta,gsp(kk,vindex,vindex))&
            &+Matmul(Transpose(gsp(kk,vindex,vindex)),vsdelta),t_real)

       gspLvv(kk,:,:)=Real(Matmul(vsdelta,gsp(kk,vindex,vindex)),t_real)

       ghpvv(kk,:,:)=Real(Matmul(vsdelta,ghp(kk,vindex,vindex))&
            &+Matmul(Transpose(ghp(kk,vindex,vindex)),vsdelta),t_real)

    Enddo
    !!
    Do kk=1,3
       !       Print *,'kk=',kk
       !       Call printmat('ghpcc',"(f15.8,' ')","('       0        ')"&
       !            &,ghpcc(kk,:,:),ncdim,ncdim,tol)
    Enddo
    !!
    !!
    ff=0.0_t_real
    foC=0.0_t_real
    !!
    !!
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(gspcc(kk,:,:),eCcc))
    Enddo
    ff=ff+fo
    Print 10,' dScc eCcc =',fo
    !!
!!$    If(Any(Abs(eCvc)>tol))Then
!!$       Do kk=1,3
!!$          fo(kk)=trace(Matmul(Transpose(gspvc(kk,:,:)),eCvc))
!!$       Enddo
!!$       ff=ff+fo
!!$       Print 10,' dScv eCvc =',fo
!!$       !!
!!$       Do kk=1,3
!!$          fo(kk)=trace(Matmul(gspvc(kk,:,:),Transpose(eCvc)))
!!$       Enddo
!!$       ff=ff+fo
!!$       Print 10,' dSvc eCcv =',fo
!!$    Endif
!!$    !!
!!$    If(Any(Abs(eCvv)>tol)) Then
!!$       Do kk=1,3
!!$          fo(kk)=trace(Matmul(gspvv(kk,:,:),eCvv))
!!$       Enddo
!!$       ff=ff+fo
!!$       Print 10,' dSvv eCvv =',fo
!!$    Endif
    !!
    Print *
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(ghpcc(kk,:,:),nCcc))
    Enddo
    ff=ff+fo
    foC=foC+fo
    Print 11,' dHcc nCcc =',fo,' -> core'
    !!
!!$    focor=0.0_t_real
!!$    !!
!!$    If(Any(Abs(nCvc)>tol)) Then
!!$       Do kk=1,3
!!$          fo(kk)=trace(Matmul(Transpose(ghpvc(kk,:,:)),nCvc))*2.0_t_real
!!$       Enddo
!!$       focor=focor+fo
!!$       ff=ff+fo
!!$       Print 10,' dHcv nCvc =',fo
!!$    Endif
!!$    If(Any(Abs(nCvv)>tol)) Then
!!$       Do kk=1,3
!!$          fo(kk)=trace(Matmul(ghpvv(kk,:,:),nCvv))
!!$       Enddo
!!$       focor=focor+fo
!!$       ff=ff+fo
!!$       Print 10,' dHvv nCvv =',fo
!!$    Endif
!!$    If(Any(Abs(focor)>tol)) Then
!!$       Print 11,' dHvv nCvv =',focor,' ->0'
!!$    Endif
    !!
    !!
    Print *
    Print *
    Print *
    Print *
    !!
    foS=0.0_t_real
    foH=0.0_t_real
    Do kk=1,3
       fo(kk)=trace(Matmul(gspcc(kk,:,:),eVcc))
    Enddo
    ff=ff+fo
    foS=foS+fo
    Print 10,' dScc eVcc =',fo
    !!
    fovc=0.0_t_real
    Do kk=1,3
       fo(kk)=trace(Matmul(gspLvc(kk,:,:),Transpose(eVvc)))*2.0_t_real
    Enddo
    fovc=fovc+fo
    Print 10,' dSdvc eVcv =',fo
    Do kk=1,3
       fo(kk)=trace(Matmul(gspvLc(kk,:,:),Transpose(eVvc)))*2.0_t_real
    Enddo
    fovc=fovc+fo
    Print 10,' dSvdc eVcv =',fo
    !!
    ff=ff+fovc
    foS=foS+fovc
    Print 10,' dScv eVvc =',fovc
    !!
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(gspLvv(kk,:,:),eVvv))*2.0_t_real
    Enddo
    ff=ff+fo
    foS=foS+fo
    Print 11,' dSvv eVvv =',fo,' -> Svv'
    Print 11,' dSvv      =',foS,' -> S total'
    !!
    !!
    Print *
    Print *
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(ghpcc(kk,:,:),nVcc))
    Enddo
    ff=ff+fo
    foH=foH+fo
    Print 10,' dHcc nVcc =',fo
    !!
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(Transpose(ghpvc(kk,:,:)),nVvc))*2.0_t_real
    Enddo
    foH=foH+fo
    ff=ff+fo
    Print 10,' dHcv nVvc =',fo
    !!    Print 11,' dHcv nVvc =',fo+focor,' rest'
    !!
    !!
    Do kk=1,3
       fo(kk)=trace(Matmul(ghpvv(kk,:,:),nVvv))
    Enddo
    foH=foH+fo
    ff=ff+fo
    Print 11,' dHvv nVvv =',fo,' -> Hvv'
    Print 11,' dHvv      =',foH,' -> H total'
    !!
    Print *
    Print *
    Print 10,' total Pulay =',ff

    !!
    !!==============================
    !!
    !!
    !Stop
    !!
    Return
    !!
10  Format(a,1x,3(f20.10,2x))
11  Format(a,1x,3(f20.10,2x),a)
    !!
  End Subroutine f_analyzse0
#endif
  !!***********************************************************************
  Real(t_real) Function re_trace(a)
    !!
    Implicit none  
    !!IO
    Complex(t_real),Intent(in)      :: a(:,:)
    !!LOCAL
    Integer(t_int) :: i,n
    !!
    n=Size(a,1)
    !!
    re_trace=0.0_t_real
    !!
    Do i=1,n
       re_trace=re_trace+Real(a(i,i),t_real)
    Enddo
    !!
    Return
    !!
  End Function re_trace
  !!***********************************************************************
  Integer(t_int) Function delta(i,ia)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: i,ia(:)
    !!LOCAL
    Integer(t_int) :: k
    !!
    delta=0
    Do k=1,Size(ia,1)
       If(i==ia(k))Then
          delta=1
          Return
       Endif
    Enddo
    !!
    Return
    !!
  End Function delta
  !!***********************************************************************
  Integer(t_int) Function theta(i,ia)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: i,ia(:)
    !!LOCAL
    Integer(t_int) :: k
    !!
    theta=1
    Do k=1,Size(ia,1)
       If(i==ia(k))Then
          theta=0
          Return
       Endif
    Enddo
    !!
    Return
    !!
  End Function theta
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine reltest(structure,basis,nvdim,eb,zz)
    !!
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: nvdim
    Real(t_real),Intent(in)      :: eb(nvdim)
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="reltest"
    Integer(t_int) :: isa,issa,iv,ivr,l,jj,j0,kap,j,iz,mmu,mu,kp,in,nvm
    Integer(t_int) :: degl,degj,ms,m,i,ldim,jdim,n
    Real(t_real)   :: del
    Complex(t_real):: c
    Complex(t_real),Pointer :: cc(:,:,:,:,:,:)
    Complex(t_real),Pointer :: A(:,:,:,:,:)
    Complex(t_real),Pointer :: p_U(:,:,:)
    Complex(t_real),Pointer :: TT(:,:,:)
    !Complex(t_real) :: MM(nvdim,nvdim)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Integer(t_int),Pointer :: p_nv(:),p_lv(:,:)

    !!
    Nullify(p_nv)
    Nullify(p_lv)
    Nullify(cc)
    Nullify(A)
    Nullify(p_U)
    Nullify(TT)
    !!
    Call palloc('p_nv',p_nv,1_t_int,structure% nsort,nmodule,nsubfunc)
    Call palloc('p_lv',p_lv,1_t_int,basis% nRvmx,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    !!
    nvm=0
    Do issa=1,structure% nsort
       iv=0
       ivrloop1: Do ivr=1,basis% p_nRv(issa)
          kap=basis% p_kapRv(ivr,issa)
          l=(Abs(2*kap+1)-1)/2
          If(kap>0) Cycle ivrloop1
          iv=iv+1
          p_lv(iv,issa)=l
       Enddo ivrloop1
       p_nv(issa)=iv
       nvm=Max(nvm,iv)
    Enddo
    !!
    Write(outfile,2)
    Write(outfile,"(32(' '))",advance='no')
    Do in=1,nvdim
       Write(outfile,13,advance='no') in,eb(in)
    Enddo
    Write(outfile,2)
    Write(outfile,2)
    !!
    isaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       ivr=0
       iz=0
       ivloop: Do iv=1,p_nv(issa)
          l=p_lv(iv,issa)
          degl=2*l+1
          j0=1
          If(l==0) j0=2
          jjloop: Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             kp=Abs(kap)
             degj=2*kp
             j=2*Abs(kap)-1
             ivr=ivr+1
             mmuloop: Do mmu=1,degj
                iz=iz+1
                mu=2*(mmu-1)-j
                Write(outfile,11,advance='no') isa,basis% p_nnRv(ivr,issa)&
                     &,lqnchar(l),j,mu
                inloop: Do in=1,nvdim
                   Write(outfile,12,advance='no') zz(iz,in)
                Enddo inloop
                Write(outfile,2)
             Enddo mmuloop
          Enddo jjloop
       Enddo ivloop
    Enddo isaloop

    Call palloc('cc',cc&
         &,1_t_int,2*basis% lmx+1&
         &,1_t_int,2_t_int&
         &,1_t_int,2_t_int&
         &,1_t_int,nvm&
         &,1_t_int,structure% nsite&
         &,1_t_int,nvdim&
         &,nmodule,nsubfunc)

    Call palloc('A',A& 
         &,1_t_int,2*basis% lmx+1   & ! degl
         &,1_t_int,2*(basis% lmx+1) & ! degj
         &,1_t_int,2_t_int          & ! j=l+-1/2
         &,1_t_int,2_t_int          & ! spin
         &,0_t_int,basis% lmx       & ! l
         &,nmodule,nsubfunc)

    !!
    Alloop:Do l=0,basis% lmx
       j0=1 ; If(l==0) j0=2
       Ajjloop: Do jj=j0,2
          kap=l-(jj-1)*(2*l+1)
          j=2*Abs(kap)-1
          Amloop: Do m=1,2*l+1
             Ammuloop: Do mmu=1,j+1
                mu=2*(mmu-1)-j
                Amsloop: Do ms=1,2
                   del=0.0_t_real
                   If(m == (mu-1)/2 + ms + l ) del=1.0_t_real
                   !!
                   A(m,mmu,jj,ms,l)=cgoOFKapMu(ms,kap,mu)*del
                Enddo Amsloop
             End Do Ammuloop
          Enddo Amloop
       End Do Ajjloop
    End Do Alloop


    Call palloc('p_U',p_U,1_t_int,2*basis% lmx+1,1_t_int,2*basis% lmx+1&
         &,0_t_int,basis% lmx,nmodule,nsubfunc)


    Call make_ylm_U_matrix(basis% lmx,p_U)


    Do l=0,basis% lmx
       degl=2*l+1
       Do jj=1,2
          Do ms=1,2
             Do mmu=1,2*basis% lmx+2
                A(1:degl,mmu,jj,ms,l)=&
                     &Matmul(p_U(1:degl,1:degl,l),A(1:degl,mmu,jj,ms,l))
             Enddo
          Enddo
       Enddo
    Enddo
    
    !!
    cc(:,:,:,:,:,:)=c0
    isaloop1: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       iz=0
       ivloop1: Do iv=1,p_nv(issa)
          l=p_lv(iv,issa)
          degl=2*l+1
          j0=1 ; If(l==0) j0=2
          jjloop1: Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             degj=2*Abs(kap)
             msloop1: Do ms=1,2
                Do in=1,nvdim
                   cc(1:degl,1,ms,iv,isa,in)=cc(1:degl,1,ms,iv,isa,in)&
                      &+Matmul(A(1:degl,1:degj,jj,ms,l),zz(iz+1:iz+degj,in))
                Enddo
             Enddo msloop1
             iz=iz+degj
          End Do jjloop1
       Enddo ivloop1
    Enddo isaloop1
    !!


    Do in=1,nvdim
       m=2
       ms=1
       iv=1
       isa=1
       c=cc(m,1,ms,iv,isa,in)
       cc(:,1,:,:,:,in)=cc(:,1,:,:,:,in)*Conjg(c)/Abs(c)
    Enddo

    Do ms=1,2
       Write(outfile,2)
       isaloop2: Do isa=1,structure% nsite
          issa=structure% p_isrt(isa)
          ivloop2: Do iv=1,p_nv(issa)
             l=p_lv(iv,issa)
             degl=2*l+1
             !If(l /= 1 ) Cycle ivloop2
             mmuloop2: Do m=1,degl
                Write(outfile,311,advance='no') isa,0&!basis% p_nnRv(ivr,issa)&
                     &,lqnchar(l),m-l-1,3-2*ms
                !inloop2: Do in=7,10
                inloop2: Do in=1,nvdim
                   Write(outfile,12,advance='no') cc(m,1,ms,iv,isa,in)
                Enddo inloop2
                Write(outfile,2)
             Enddo mmuloop2
          Enddo ivloop2
       Enddo isaloop2
    End Do
    !!
    
    !!
    !!
    !!
!!$    m=1
!!$    iv=1
!!$    isa=1
!!$    Do ms=1,2
!!$       Print *,ms,Sum(Abs(cc(m,1,ms,iv,isa,7:10))**2)
!!$    Enddo


    !!
    ldim=0
    jdim=0
    Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       Do iv=1,p_nv(issa)
          l=p_lv(iv,issa)
          degl=2*l+1
          ldim=ldim+degl
       Enddo
       Do ivr=1,basis% p_nRv(issa)
          kap=basis% p_kapRv(ivr,issa)
          degj=2*Abs(kap)
          jdim=jdim+degj
       Enddo
    Enddo
          
    !!
    Call palloc('TT',TT ,1_t_int,ldim ,1_t_int,jdim, 1_t_int,2_t_int & 
         &,nmodule,nsubfunc)
    TT=c0
    !!
    i=0
    ivr=0
    isaloop4: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       ivloop4: Do iv=1,p_nv(issa)
          l=p_lv(iv,issa)
          degl=2*l+1
          j0=1 ; If(l==0) j0=2
          mloop4: Do m=1,degl
             i=i+1
             !!
             iz=ivr
             jjloop4: Do jj=j0,2
                kap=l-(jj-1)*(2*l+1)
                j=2*Abs(kap)-1
                mmuloop4: Do mmu=1,j+1
                   mu=2*(mmu-1)-j
                   iz=iz+1
                   msloop4: Do ms=1,2
                      !! U_mn d_n,mu-s/2
                      n=(mu-1)/2 + ms + l
                      If(1 <=n .And. n <= 2*l+1  ) Then
                         TT(i,iz,ms)=TT(i,iz,ms)+p_U(m,n,l)*cgoOFKapMu(ms,kap,mu)
                      Endif
                   Enddo msloop4
                Enddo mmuloop4
             Enddo jjloop4
             !!
          Enddo mloop4
          ivr=ivr+2*degl
       Enddo ivloop4
    Enddo isaloop4
    !!
    !!
!!$    Call printmat('TT1',"('(',f12.5,',',f12.5,') ')","('              0     &
!!$         &        ')"&
!!$         &,TT(:,:,1),ldim,jdim,1.0e-5_t_real)
!!$    !!
!!$    Call printmat('TT2',"('(',f12.5,',',f12.5,') ')","('              0     &
!!$         &        ')"&
!!$         &,TT(:,:,2),ldim,jdim,1.0e-5_t_real)



    !!
!!$    Do ms=1,2
!!$    !!
!!$       Do i=1,2
!!$          Call printmat('TT*TT',"('(',f12.5,',',f12.5,') ')","('              0     &
!!$         &        ')"&
!!$         &,Matmul(TT(:,:,ms),Conjg(Transpose(TT(:,:,i)))),ldim,ldim&
!!$         &,1.0e-7_t_real)
!!$       Enddo
!!$    Enddo


!!$    Print *,'UU'
!!$    Do i=1,2*basis% lmx+1
!!$       Do j=1,2*basis% lmx+1
!!$          Write(outfile,"('(',f12.5,',',f12.5,') ')",advance='no')UU(i,j,1)
!!$       Enddo
!!$       Write(outfile,*)
!!$    Enddo

    !!
2   Format(' ')
12  Format('(',f12.5,',',f12.5,') ')
13  Format(3x,i3,2x,f12.5,8x)
11  Format(i3,' : ',i1,a1,i1,'/2 ',i3,'/2             : ')
!211 Format(i3,' : ',i1,a1,' l',sp,i2,'/2 ml=',s,i3,' spin ',i2,' : ')
311 Format(i3,' : ',i1,a1,' ml=',s,i3,' spin ',i2,' : ')
    !!
    Call pfree(cc)
    Call pfree(A)
    Call pfree(p_U)
    Call pfree(TT)
    !!
    !!
    Call pfree(p_nv)
    Call pfree(p_lv)
    !!
    !!
    Return
    !!
  End Subroutine reltest
#endif
  !!***********************************************************************
  !! this are test routines, not optimized
#ifdef my_compile_unused
  Subroutine  matRelToNonrel(h,ndim,nsite,nsort,isrt,basis)
    !!
    Implicit None  
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: nsite,ndim,nsort,isrt(*)
    Complex(t_real),Intent(in)   :: h(ndim,ndim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="matRelToNonrel"
    Integer(t_int),Parameter :: lmax=ORBITAL_MAX_L
    Integer(t_int) :: l,jj,kap,j,mu,m,ms,i1,i2,isa,issa,isb,issb
    Integer(t_int) :: la,iva,dega,ivb,lb,degb,msb,i2s,i1s,msa,ivr,iv
    !!
    !Character(1),Parameter   :: SGN(2)=(/'+','-'/)
    Real(t_real)   :: T(2*(2*lmax+1),2*(2*lmax+1),0:lmax)
    !Real(t_real)   :: A(2*(2*lmax+1),2*(2*lmax+1),0:lmax)
    Complex(t_real)   :: MM(2*lmax+1,2*lmax+1)
    Complex(t_real)   :: ht(ndim,ndim)
    Integer(t_int),Pointer :: p_nv(:),p_lv(:,:)
    !!
    Nullify(p_nv)
    Nullify(p_lv)
    !!
    Call palloc('p_nv',p_nv,1_t_int,nsort,nmodule,nsubfunc)
    Call palloc('p_lv',p_lv,1_t_int,basis% nRvmx,1_t_int,nsort&
         &,nmodule,nsubfunc)
    !!
    Do issa=1,nsort
       iv=0
       ivrloop1: Do ivr=1,basis% p_nRv(issa)
          kap=basis% p_kapRv(ivr,issa)
          l=(Abs(2*kap+1)-1)/2
          If(kap>0) Cycle ivrloop1
          iv=iv+1
          p_lv(iv,issa)=l
       Enddo ivrloop1
       p_nv(issa)=iv
    Enddo
    !!
    !Call printmat('Re S',h(1,1,1),ndim)
    !Call printmat('Im S',h(1,1,2),ndim)
    !!
!!$    Do l=0,lmax
!!$       Print *,'l=',l
!!$       jj0=1
!!$       If(l==0) jj0=2
!!$       Do jj=jj0,2
!!$          kap=l-(jj-1)*(2*l+1)
!!$          j=2*Abs(kap)-1
!!$          jdeg=2*Abs(kap)
!!$          Do m=1,jdeg
!!$             mu=2*(m-1)-j
!!$             Print 12,l,kap,j,mu
!!$12           Format(i1,1x,i3,1x,i1,'/2',1x,i3,'/2') 
!!$             
!!$          Enddo
!!$       Enddo
!!$    Enddo
    !!
    T(:,:,:)=0.0_t_real
    Do l=0,lmax
       !Print *,'l=',l
       i2=0
       Do ms=1,2
          Do m=-l,l
             i2=i2+1
             mu=2*m+(3-2*ms)
             !Write(outfile,13,advance='no') SGN(ms),l,m
!13           Format(a1,1x,i1,',',i3,':')

             kap=3*l+1
             Do jj=1,2
                kap=kap-2*l-1
                j=2*Abs(kap)-1
                If(Abs(mu)<=j)Then
                   i1=(2*l)*(jj-1)+(mu+j)/2+1
                   !Write(outfile,14,advance='no') kap,mu,i1
                   T(i1,i2,l)=cgoOFKapMu(ms,kap,mu)
                Endif
!14              Format(i2,i3,'/2','   ',i2,'   and   ')

             Enddo
             !Write(outfile,*)
          Enddo
       Enddo
    Enddo
    !!
!!$    Do l=0,lmax
!!$       dega=2*(2*l+1)
!!$       A(1:dega,1:dega,l)=Matmul(Transpose(T(1:dega,1:dega,l)),T(1:dega,1:dega,l))
!!$    Enddo
!!$    !!
!!$    Do l=0,lmax
!!$       Print *,'l=',l
!!$       Do i1=1,2*(2*l+1)
!!$          Do i2=1,2*(2*l+1)
!!$             d=A(i1,i2,l)
!!$             If(Abs(d)<1.0e-12_t_real)Then
!!$                Write(outfile,115,advance='no')
!!$             Else
!!$                Write(outfile,15,advance='no')d
!!$             Endif
!!$          Enddo
!!$          Write(outfile,15)
!!$       Enddo
!!$    Enddo
!!$15  Format(f10.5,1x)
!!$!115 Format('   0.0     ')
!!$115 Format('           ')
    !!
    i1=0
    Do msa=0,1
       i1s=0
       Do isa=1,nsite
          issa=isrt(isa)
          Do iva=1,p_nv(issa)
             la=p_lv(iva,issa)
             dega=(2*la+1)
             !Print *,iva,la,dega,i1
             i2=0
             Do msb=0,1
                i2s=0
                Do isb=1,nsite
                   issb=isrt(isb)
                   Do ivb=1,p_nv(issb)
                      lb=p_lv(ivb,issb)
                      degb=(2*lb+1)
                      !Print *,ivb,lb,degb,i1,i2
                      MM(1:dega,1:degb)=&
                           & Matmul(Transpose(T(1:dega*2,msa*dega+1:&
                           &                        msa*dega+dega,la))&
                           &       ,Matmul(h(i1s+1:i1s+dega*2&
                           &               ,i2s+1:i2s+degb*2)&
                           &               ,T(1:degb*2,msb*degb+1:&
                           &                  msb*degb+degb,lb)))
                      !!
                      !!
                      Call MatCmplxHarmToReal(MM,Int(lmax,t_int),la,lb)
                      !!
                      ht(i1+1:i1+dega,i2+1:i2+degb)=MM(1:dega,1:degb)
                      i2=i2+degb
                      i2s=i2s+degb*2
                   Enddo
                Enddo
             Enddo! msb
             i1=i1+dega
             i1s=i1s+dega*2
          Enddo
       Enddo
    Enddo! msa
    !!
    Call printmat('St',"('(',f12.5,',',f12.5,') ')","('            0               ')"&
         &,ht,ndim,ndim,1.0e-5_t_real)

    !!
    !! 
    Call pfree(p_nv)
    Call pfree(p_lv)
    !!
    !!
    Return
    !!
  End Subroutine matRelToNonrel
#endif
  !!***********************************************************************
  !! this are test routines, not optimized
#ifdef my_compile_unused
  Subroutine  MatCmplxHarmToReal(MM,lmax,la,lb)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: lmax,la,lb
    Complex(t_real),Intent(inout)   :: MM(2*lmax+1,2*lmax+1)
    !!LOCAL
    Integer(t_int) :: l,ma,mb,i,j
    Complex(t_real):: c
    !!
    Complex(t_real):: U(2*lmax+1,2*lmax+1,0:lmax)
    Complex(t_real):: A(2*lmax+1,2*lmax+1)
    !!
    U(:,:,:)=0.0_t_real
    Do l=0,lmax
       Do ma=-l,l
          Do mb=-l,l
             If(ma == mb)Then
                If(ma<0)Then
                   c=Cmplx(0.0_t_real,1.0_t_real,t_real)*sq2o2
                Elseif(ma>0)Then
                   c=Cmplx((-1.0_t_real)**ma,0.0_t_real,t_real)*sq2o2
                Else
                   c=Cmplx(1.0_t_real,0.0_t_real,t_real)
                Endif
                U(ma+l+1,mb+l+1,l)=c
             Else If(ma == -mb)Then
                If(ma<0)Then
                   c=Cmplx(1.0_t_real,0.0_t_real,t_real)*sq2o2
                Elseif(ma>0)Then
                   c=Cmplx(0.0_t_real,-(-1.0_t_real)**ma,t_real)*sq2o2
                Else
                   c=Cmplx(1.0_t_real,0.0_t_real,t_real)
                Endif
                U(ma+l+1,mb+l+1,l)=c
             Endif

          Enddo
       End Do
    End Do
    !!
!!$    Do l=0,lmax
!!$       Print *,'l=',l
!!$       Do ma=1,2*l+1
!!$          Do mb=1,2*l+1
!!$             c=U(ma,mb,l)
!!$             If(Abs(c)<1.0e-12_t_real)Then
!!$                Write(outfile,115,advance='no')
!!$             Else
!!$                Write(outfile,15,advance='no')c
!!$             Endif
!!$          Enddo
!!$          Write(outfile,*)
!!$       Enddo
!!$    Enddo
!!$15  Format('(',f7.3,',',f7.3,')',1x)
!!$!115 Format('   0.0     ')
!!$115 Format('   0             ',1x)
    !!
    Do ma=1,2*la+1
       Do mb=1,2*lb+1
          c=Cmplx(0.0_t_real,0.0_t_real,t_real)
          Do i=1,2*la+1
             Do j=1,2*lb+1
                c=c+Conjg(U(i,ma,la))&
                     &*MM(i,j)&
                     &*U(j,mb,lb)
!                c=c+U(ma,i,la)&
!                     &*Cmplx(MM(i,j,1),MM(i,j,2),t_real)&
!                     &*Conjg(U(mb,j,lb))
             Enddo
          Enddo
          A(ma,mb)=c
       Enddo
    Enddo
    !!
    Do ma=1,2*la+1
       Do mb=1,2*lb+1
          MM(ma,mb)=A(ma,mb)
       Enddo
    Enddo
    !!
  End Subroutine MatCmplxHarmToReal
#endif
  !!***********************************************************************
  Subroutine analyze_S(sp,nvdim,ms,qq)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nvdim,ms
    Real(t_real),Intent(in)      :: qq(3)
    Complex(t_real),Intent(in)     :: sp(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="analyze_S"
    Complex(t_real),Pointer :: p_s(:,:)
    Real(t_real),Pointer    :: p_E(:)
    !!
    Nullify(p_s,p_E)
    !!
    Call palloc('p_s',p_s,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('p_E',p_E,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    p_s=sp
    !!
    Call cdiagno_noO(nvdim,p_s,p_E)
    !!
    !Write(outfile,*) 
    Write(outfile,1)qq(1:3)
1   Format('K-vector:',3f10.4)
22  Format('Overlap sigmas at spin',i4)
3   Format(5f14.7)
    Write(outfile,22)ms
    Write(outfile,3) p_E(1),p_E(nvdim)
    !Write(outfile,3)(p_E(in),in=1,nvdim)
    !!
    !!
    !! Call printmat('raw Re H',"(f12.5,' ')","('     0       ')"&
    !! &,Real(hp,t_real),nvdim,nvdim)
    !!
    !!
    Call pfree(p_E)
    Call pfree(p_s)
    !!
    !!
    !!
    Return
    !!
  End Subroutine analyze_S
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine svd_diagno(nvdim,hp,sp,zz,eb)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nvdim
    Complex(t_real),Intent(in)   :: hp(:,:)
    Complex(t_real),Intent(in)   :: sp(:,:)
    Complex(t_real),Intent(out)  :: zz(:,:)
    Real(t_real),Intent(out)     :: eb(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="analyze_S"
    Integer(t_int) :: i,j,ival,i0,m,n,ndim
    Complex(t_real):: cd
    Real(t_real)   :: emax
    Complex(t_real),Pointer :: p_SS(:,:)
    Complex(t_real),Pointer :: p_HH(:,:)
    Complex(t_real),Pointer :: p_u(:,:)
    !Complex(t_real),Pointer :: work(:)
    !Real(t_real),Pointer    :: rwork(:)
    Real(t_real),Pointer    :: p_s(:)
    Real(t_real),Pointer    :: p_e(:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    !Integer(t_int) :: lwork,info
    !!
    !!
    Nullify(p_SS)
    Nullify(p_HH)
    Nullify(p_u)
    Nullify(p_s)
    Nullify(p_e)
    !!
    Call palloc('p_SS',p_SS,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('p_u' ,p_u ,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('p_s' ,p_s ,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    p_SS=sp
    !!
    p_u=c0
    Do i=1,nvdim
       p_u(i,i)=c1
    Enddo
    Print *,'cdiagno_noO SS'
    Call cdiagno(nvdim,p_SS,p_u,p_s)
    !!
    Open(12,file='=.ival')
    Read(12,*)ival
    Close(12)
    !!
    i0=Max(1_t_int,nvdim-ival+1)
    ndim=nvdim-i0+1
    !!
    Print *,nvdim,i0,ndim
    !!
    Do i=1,nvdim
       p_SS(i,1:i0-1)=c0
       Do j=i0,nvdim
          p_SS(i,j)=p_SS(i,j)/Sqrt(p_s(j))
       Enddo
    Enddo
    !!
    Call palloc('p_HH',p_HH ,1_t_int,ndim,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('p_u' ,p_u  ,1_t_int,ndim,1_t_int,ndim,nmodule,nsubfunc)
    Call palloc('p_e' ,p_e  ,1_t_int,ndim,nmodule,nsubfunc)
    !!
    Do i=i0,nvdim
       Do j=i0,nvdim
          cd=c0
          Do m=1,nvdim
             Do n=1,nvdim
                cd=cd+Conjg(p_SS(m,i))*hp(m,n)*p_SS(n,j)
             Enddo
          Enddo
          p_HH(i-i0+1,j-i0+1)=cd
       Enddo
    Enddo
    !!
    p_u=c0
    Do i=1,ndim
       p_u(i,i)=c1
    Enddo


    Call printmat('Re H',"(f12.5,' ')","('     0       ')"&
         &,Real(p_HH,t_real),ndim,ndim,1.0e-10_t_real)


    Print *,'cdiagno_noO HH reduced'
    Call cdiagno(ndim,p_HH,p_u,p_e)
    !!
    Print *,'after cdiagno_noO HH reduced'
    Do i=1,ndim
       eb(i)=p_e(i)
    Enddo
    emax=p_e(ndim)+1.0_t_real
    Do i=ndim+1,nvdim
       eb(i)=emax
    Enddo
    !!
    Do i=1,nvdim
       Do j=1,ndim
          cd=c0
          Do n=1,ndim
             cd=cd+p_SS(i,n+i0-1)*p_HH(n,j)
          Enddo
          zz(i,j)=cd
       Enddo
       zz(i,ndim+1:nvdim)=c0
    Enddo
    !!
    !!
    !!
    Call pfree(p_e)
    Call pfree(p_u)
    Call pfree(p_HH)
    !!
    !!
    Call pfree(p_s)
    Call pfree(p_SS)
    !!
    !!
    !!
    Return
    !!
  End Subroutine svd_diagno
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine print_resonaces(mspin,nsite,kohnsham,oci,efermi)
    !!
    Implicit none  
    !!IO
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Integer(t_int),Intent(in)    :: mspin,nsite
    Real(t_real),Intent(in)      :: efermi(2)
    Real(t_real),Intent(in)      :: oci(:,:)
    !!LOCAL
    Integer(t_int) :: nva,ioci,ms,isa,i
    !!
    ioci=0
    Do isa=1,nsite
       Write(outfile,10) isa
       nva=kohnsham % p_vblocklength(isa)
       !!
       Do ms=1,mspin
          Do i=1,nva
             Write(outfile,12) (oci(ioci+i+(i-1)*nva,ms)-efermi(ms))*hatoev
          End Do
       End Do
       ioci=ioci+nva**2
    Enddo
    !!
10  Format('site_A ',i5)
12  Format(f12.6,'  ')
    !!
  End Subroutine print_resonaces
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine drot_mat(symmetry,structure,basis,kohnsham,bzone,nvdim,hp,eb,quant,ik,qq)
    !!
    Implicit None  
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Quant),Intent(in)     :: quant
    Integer(t_int),Intent(in)    :: nvdim,ik
    Real(t_real),Intent(in)      :: qq(1:3),eb(nvdim)
    Complex(t_real),Intent(in)   :: hp(nvdim,nvdim)
    !!LOCAL
    Integer(t_int) :: isa,ivra,iza,mma,ja,ija,kapa,mmb,issa,i,j,jk,nk
    Real(t_real)   :: fac(0:2*basis% lmx+1),qqR(3),nq(3),nb(3),bb(3),re,im
    Character(100)   :: fname
    Complex(t_real):: D(nvdim,nvdim),cdjmm((2*basis% lmx+2)**2)
    Complex(t_real):: A(nvdim,nvdim)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Call make_factorials(fac,2*basis% lmx+1)
    
    
    qqR=Matmul(qq(:),(quant% MM(:,:)))
    nq(:)=Matmul(qqR(:),symmetry% lattice_vectors(:,:))/twopi
    Call backfold(nq)
    nk=-1
    Do jk=1,bzone% nkp
       bb=bzone% p_bk(1:3,jk)
       nb(:)=Matmul(bb(:),symmetry% lattice_vectors(:,:))/twopi
       Call backfold(nb)

       If( Sqrt(Sum((nb(1:3)-nq(1:3))**2)) < 1.0e-6_t_real )Then
          nk=jk
       Endif
    Enddo

    If(nk == -1 ) Stop 'nk=-1'
    
    Write(fname,"('+DhpD_',i5.5)") nk
    Open(12,file=Trim(fname),status='unknown')

    Write(fname,"('+eb_',i5.5)") nk
    Open(13,file=Trim(fname),status='unknown')

    Write(13,1)ik,qq(1:3),nk,qqR(1:3),nq
    Do i=1,nvdim
       Write(13,4)i,eb(i) 
    Enddo

    Close(13)

    Write(12,1)ik,qq(1:3),nk,qqR(1:3),nq

    
    !!
    D(:,:)=c0
    Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       iza=kohnsham% p_vblockstart(isa)-1
       Do ivra=1,basis% p_nRv(issa)
          kapa=basis% p_kapRv(ivra,issa)
          ja=2*Abs(kapa)-1
          ija=Abs(kapa)
          
          Call wignerDJ(ja,quant% eulang,cdjmm(:),fac,.False.)

          Do mma=1,ja+1
             !Print *,isa,ivra,mma,iza
             Do mmb=1,ja+1
             D(iza+mma,iza+mmb)=cdjmm(mma+(mmb-1)*(ja+1))
          Enddo
          Enddo
          iza=iza+ja+1
       Enddo
    Enddo
    
    A=Matmul(Transpose(Conjg(D)),Matmul(hp,D))
    !!
!!$    Call printmat('D',"('(',f12.5,',',f12.5,') ')"&
!!$         &,"('            0               ')"&
!!$         &,D,nvdim,nvdim,1.0e-5_t_real)
!!$    Call printmat('hp',"('(',f12.5,',',f12.5,') ')"&
!!$         &,"('            0               ')"&
!!$         &,hp,nvdim,nvdim,1.0e-5_t_real)
    Do i=1,nvdim
       Do j=1,nvdim
          re=Real(A(i,j),t_real)
          im=Aimag(A(i,j))
          If(Abs(re)<1.0e-14_t_real) re=0.0_t_real
          If(Abs(im)<1.0e-14_t_real) im=0.0_t_real
          Write(12,2) i,j,re,im
       Enddo
    Enddo
    !!
    Close(12)
    !!
    Return
    !!
1   Format('K-vector ',i5,' : ',3f10.4,'  ->  ',i5,' : ',3f10.4,' = ',3f10.4)
2   Format(i5,1x,i5,1x,2(e30.11e3))
4   Format(i5,1x,e30.11e3)
    !!
  End Subroutine drot_mat
#endif
  !!***********************************************************************
  Subroutine  backfold(nq)
    !!
    Implicit none  
    !!IO
    Real(t_real),Intent(inout)   :: nq(3)
    !!LOCAL
    Integer(t_int) :: i
    !!
    Do i=1,3
       Do While(nq(i) < -0.5_t_real) 
          nq(i)=nq(i)+1.0_t_real
       Enddo
       Do While(nq(i) >  0.5_t_real)
          nq(i)=nq(i)-1.0_t_real
       Enddo
       If(nq(i) == -0.5_t_real) then
          nq(i)=nq(i)+1.0_t_real
       Endif
    Enddo

    !!
    Return
    !!
  End Subroutine backfold
  !!***********************************************************************
  !! reduce the hamilton problem to the orthogonalized core-problem
  !! for test reasons
#ifdef my_compile_unused
  Subroutine e_reduce(hp0,sp0,nsite,ndim,zz,eb,vblockstart,vblockend,nmode)
    !!
    Implicit None  
    !!IO
    Integer(t_int) :: ndim,nsite,nmode(*)
    Integer(t_int),Intent(in)    :: vblockstart(:),vblockend(:)
    Complex(t_real),Intent(in)   :: hp0(ndim,ndim),sp0(ndim,ndim)
    Complex(t_real),Intent(inout):: zz(ndim,ndim)
    Real(t_real),Intent(inout)   :: eb(ndim)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="e_reduce"
    Integer(t_int) :: i,ncdim,nvdim
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Complex(t_real),Pointer:: hcc(:,:),hcv(:,:),hvv(:,:)
    Complex(t_real),Pointer:: scc(:,:),scv(:,:),svv(:,:)
    Complex(t_real),Pointer:: zzcc(:,:),zzvv(:,:),zzcv(:,:)
    Real(t_real),Pointer :: ecc(:),eecc(:,:)
    Real(t_real),Pointer :: evv(:),eevv(:,:)
    !!
    Call prep_e_reduce(nsite,nmode,ndim,vblockstart,vblockend&
         &,ncdim,nvdim,sub_c,sub_v,hcc,hcv,hvv,scc,scv,svv&
         &,zzcc,ecc,eecc,zzvv,evv,eevv,zzcv)
    !!
    hcc(:,:)=hp0(sub_c,sub_c)
    hcv(:,:)=hp0(sub_c,sub_v)
    hvv(:,:)=hp0(sub_v,sub_v)
    !!
    scc(:,:)=sp0(sub_c,sub_c)
    scv(:,:)=sp0(sub_c,sub_v)
    svv(:,:)=sp0(sub_v,sub_v)
    !!
    !!core diagonalize
    !!
    Call cdiagno(ncdim,hcc,scc,ecc)
    zzcc=hcc
    !!
    Write(outfile,"('core energies=',/,4(f30.15,2x))") ecc(1:ncdim)
    !!
    !! core transform
    !!
    eecc=0.0_t_real
    Do i=1,ncdim
       eecc(i,i)=ecc(i)
    Enddo
    !!
    hcv=Matmul(Transpose(Conjg(zzcc)),hcv)
    scv=Matmul(Transpose(Conjg(zzcc)),scv)
    !!
    svv=svv-Matmul(Transpose(Conjg(scv)),scv)
    hvv=hvv&
         &-Matmul(Transpose(Conjg(scv)),hcv)&
         &-Matmul(Transpose(Conjg(hcv)),scv)&
         &+Matmul(Transpose(Conjg(scv)),Matmul(eecc,scv))
    !!
    !! diagonalize valence
    !!
    Call cdiagno(nvdim,hvv,svv,evv)
    zzvv=hvv
    !!
    Write(outfile,"('valence energies=',/,4(f30.15,2x))") evv(1:nvdim)
    !!
    !!
    eevv=0.0_t_real
    Do i=1,nvdim
       eevv(i,i)=evv(i)
    Enddo

    !!
    !!
    !!
    zzcv=-Matmul(Matmul(zzcc,scv),zzvv)
    zz(sub_c,1:ncdim)=zzcc
    zz(sub_v,ncdim+1:nvdim)=zzvv
    zz(sub_c,ncdim+1:nvdim)=zzcv
    zz(sub_v,1:ncdim)=0.0_t_real
    !!
    eb(1:ncdim)=ecc
    eb(ncdim+1:nvdim)=evv
    !!
!!$    ee0=0.0_t_real
!!$    Do i=1,ndim
!!$       ee0(i,i)=eb(i)
!!$    Enddo
!!$    !!
!!$    s0=Matmul(Transpose(Conjg(zz)),Matmul(hp0,zz))-ee0
!!$    !!
!!$    Call printmat('Re 1',"(f15.8,' ')","('       0        ')"&
!!$         &,s0(:,:),ndim,ndim,tol)

    !!
    Call clean_e_reduce(sub_c,sub_v,hcc,hcv,hvv,scc,scv,svv,zzcc,ecc,eecc&
         &,zzvv,evv,eevv,zzcv)
    !!
!    Stop nsubfunc
    !!
    Return
    !!
  End Subroutine e_reduce
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine prep_e_reduce(nsite,nmode,ndim,vblockstart,vblockend&
       &,ncdim,nvdim,sub_c,sub_v&!,zzcindex,zzvindex&
       &,hcc,hcv,hvv,scc,scv,svv&
       &,zzcc,ecc,eecc,zzvv,evv,eevv,zzcv)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nsite,nmode(nsite),ndim
    Integer(t_int),Intent(in)    :: vblockstart(:),vblockend(:)
    Integer(t_int),Intent(out)   :: ncdim,nvdim
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Complex(t_real),Pointer:: hcc(:,:),hcv(:,:),hvv(:,:)
    Complex(t_real),Pointer:: scc(:,:),scv(:,:),svv(:,:)
    Complex(t_real),Pointer:: zzcc(:,:),zzvv(:,:),zzcv(:,:)
    Real(t_real),Pointer :: ecc(:),eecc(:,:)
    Real(t_real),Pointer :: evv(:),eevv(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_e_reduce"
    Integer(t_int) :: jc,jv,isa,i
    !!
    !!
    Nullify(sub_c)
    Nullify(sub_v)
    Nullify(hcc,hcv,hvv)
    Nullify(scc,scv,svv)
    Nullify(zzcc)
    Nullify(ecc,eecc)
    Nullify(zzvv)
    Nullify(evv,eevv)
    Nullify(zzcv)
    !!
    ncdim=Sum(nmode(1:nsite))
    nvdim=ndim-ncdim
    !!
    Write(outfile,"('ncdim=',i5)") ncdim
    Write(outfile,"('nvdim=',i5)") nvdim
    !!
    Call palloc('sub_c',sub_c,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('sub_v',sub_v,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    jc=0
    jv=0
    Do isa=1,nsite
       !Write(outfile,"('isa=',i5)") isa
       Do i=1,nmode(isa)
          jc=jc+1
          sub_c(jc)=vblockstart(isa)-1+i
          !Write(outfile,"('sub(',i3,')=',i5)") jc,sub_c(jc)
       Enddo
       Do i=vblockstart(isa)+nmode(isa),vblockend(isa)
          jv=jv+1
          sub_v(jv)=i
          !Write(outfile,"('sub(',i3,')=',i5)") jv,sub_v(jv)
       Enddo
    Enddo
    !!
!!$    Do i=1,ncdim
!!$       zzcindex(i)=i
!!$    Enddo
!!$    Do i=ncdim+1,ndim
!!$       zzvindex(i-ncdim)=i
!!$    Enddo
          

    Write(outfile,"('sub_c=',/,5(i5))") sub_c(1:ncdim)
    Write(outfile,"('sub_v=',/,5(i5))") sub_v(1:nvdim)
    Write(outfile,*)
!!$    Write(outfile,"('zz_c=',/,5(i5))") zzcindex(1:ncdim)
!!$    Write(outfile,"('zz_v=',/,5(i5))") zzvindex(1:nvdim)
    !!
    !!
    Call palloc('hcc',hcc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('scc',scc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('hcv',hcv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('scv',scv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('hvv',hvv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('svv',svv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    Call palloc('zzcc',zzcc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('zzvv',zzvv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('zzcv',zzcv,1_t_int,ncdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    Call palloc('ecc',ecc,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('eecc',eecc,1_t_int,ncdim,1_t_int,ncdim,nmodule,nsubfunc)
    Call palloc('evv',evv,1_t_int,nvdim,nmodule,nsubfunc)
    Call palloc('eevv',eevv,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    Return
    !!
  End Subroutine prep_e_reduce
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine  clean_e_reduce(sub_c,sub_v,hcc,hcv,hvv,scc,scv,svv&
       &,zzcc,ecc,eecc,zzvv,evv,eevv,zzcv)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Pointer :: sub_c(:),sub_v(:)
    Complex(t_real),Pointer:: hcc(:,:),hcv(:,:),hvv(:,:)
    Complex(t_real),Pointer:: scc(:,:),scv(:,:),svv(:,:)
    Complex(t_real),Pointer:: zzcc(:,:),zzvv(:,:),zzcv(:,:)
    Real(t_real),Pointer :: ecc(:),eecc(:,:)
    Real(t_real),Pointer :: evv(:),eevv(:,:)
    !!LOCAL
    !!
    Call pfree(sub_c)
    Call pfree(sub_v)
    Call pfree(hcc)
    Call pfree(scc)
    Call pfree(hcv)
    Call pfree(scv)
    Call pfree(hvv)
    Call pfree(svv)
    Call pfree(zzcc)
    Call pfree(ecc)
    Call pfree(eecc)
    Call pfree(zzvv)
    Call pfree(evv)
    Call pfree(eevv)
    Call pfree(zzcv)
    !!
    Return
    !!
  End Subroutine clean_e_reduce
#endif
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine e_reduce0(hp0,sp0,ndim,zz,eb,vblockstart,vblockend,nmode)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nmode,ndim
    Integer(t_int),Intent(in)    :: vblockstart(:),vblockend(:)
    Complex(t_real),Intent(in)   :: hp0(ndim,ndim),sp0(ndim,ndim)
    Complex(t_real),Intent(inout):: zz(ndim,ndim)
    Real(t_real),Intent(inout)   :: eb(ndim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="e_reduce0"
    Integer(t_int) :: i,ncdim,nvdim
    Real(t_real)   :: tol
    Real(t_real):: hvv(ndim-(2*nmode),ndim-(2*nmode))
    Real(t_real):: hvc(ndim-(2*nmode),(2*nmode))
    Real(t_real):: hcc((2*nmode),(2*nmode))
    Real(t_real):: svv(ndim-(2*nmode),ndim-(2*nmode))
    Real(t_real):: svc(ndim-(2*nmode),(2*nmode))
    Real(t_real):: zzcv((2*nmode),ndim-(2*nmode))
    Real(t_real):: scc((2*nmode),(2*nmode))
    Real(t_real):: zzcc((2*nmode),(2*nmode))
    Real(t_real):: ecc((2*nmode))
    Real(t_real):: zzvv(ndim-(2*nmode),ndim-(2*nmode))
    Real(t_real):: evv(ndim-(2*nmode))
    Real(t_real):: eecc((2*nmode),(2*nmode))
    Real(t_real):: eevv(ndim-(2*nmode),ndim-(2*nmode))
    Real(t_real)   :: s0(ndim,ndim)
    Real(t_real)   :: ee0(ndim,ndim)
    Integer(t_int) :: cindex((2*nmode)),zzcindex((2*nmode))
    Integer(t_int) :: vindex(ndim-(2*nmode)),zzvindex(ndim-(2*nmode))
    !!
    tol=1.0e-10_t_real
    ncdim=2*nmode
    nvdim=ndim-ncdim
    !!
    !!
    Write(outfile,"('START: ',a)") nsubfunc
    !Call printmat('Re S',"(f15.8,' ')","('       0        ')"&
    !     &,Real(sp0(:,:),t_real),ndim,ndim,tol)
    If(Any(Aimag(sp0(:,:))>tol))Then
       Call printmat('Im S',"(f15.8,' ')","('       0        ')"&
            &,Aimag(sp0(:,:)),ndim,ndim,tol)
       Stop 'S  /=0 '
    Endif
    !Call printmat('Re H',"(f15.8,' ')","('       0        ')"&
    !     &,Real(hp0(:,:),t_real),ndim,ndim,tol)
    If(Any(Aimag(hp0(:,:))>tol))Then
       Call printmat('Im H',"(f15.8,' ')","('       0        ')"&
            &,Aimag(hp0(:,:)),ndim,ndim,tol)
       Stop 'H  /=0 '
    Endif
    !!
    cindex=(/(i,i=1,nmode)&
         &,(i,i=vblockstart(2),vblockstart(2)+nmode-1)/)
    vindex=(/(i,i=vblockstart(1)+nmode,vblockend(1))&
         &,(i,i=vblockstart(2)+nmode,vblockend(2))/)
    zzcindex=(/(i,i=1,nmode)/)
    zzvindex=(/(i,i=2*nmode+1,ndim)/)
    !!
    Print *,'ndim=',ndim
    Print *,'nvdim=',nvdim
    Print *,'ncdim=',ncdim
    Print *,'cindex',cindex
    Print *,'vindex',vindex
    Print *,'zzcindex',zzcindex
    Print *,'zzvindex',zzvindex
    !!
    !!
    hcc(:,:)=Real(hp0(cindex,cindex),t_real)
    hvc(:,:)=Real(hp0(vindex,cindex),t_real)
    hvv(:,:)=Real(hp0(vindex,vindex),t_real)
    !!
    scc(:,:)=Real(sp0(cindex,cindex),t_real)
    svc(:,:)=Real(sp0(vindex,cindex),t_real)
    svv(:,:)=Real(sp0(vindex,vindex),t_real)
    !!
    !!core diagonalize
    !!
    Call rdiagno(ncdim,hcc,scc,ecc)
    zzcc=hcc
    !!
    scc(:,:)=Real(sp0(cindex,cindex),t_real)
    hcc(:,:)=Real(hp0(cindex,cindex),t_real)
    !!
    eecc=0.0_t_real
    Do i=1,ncdim
       Print *,i,ecc(i)
       eecc(i,i)=ecc(i)
    Enddo
    !! core transform
    hvc=Matmul(hvc,zzcc)
    svc=Matmul(svc,zzcc)
    !!
    !!
    svv=svv-Matmul(svc,Transpose(svc))
    hvv=hvv&
         &-Matmul(hvc,Transpose(svc))&
         &-Matmul(svc,Transpose(hvc))&
         &+Matmul(Matmul(svc,eecc),Transpose(svc))
    !!
    !Call printmat('Re Svv',"(f15.8,' ')","('       0        ')"&
    !    &,svv(:,:),nvdim,nvdim,tol)
    !!
    !h0=Real(hp0,t_real)
    !s0=Real(sp0,t_real)
    !Call rdiagno(ndim,h0,s0,zz0,e0)
    !!
    Call rdiagno(nvdim,hvv,svv,evv)
    zzvv=hvv
    !!
    Write(outfile,"('END: ',a)") nsubfunc
    !!
    eevv=0.0_t_real
    Do i=1,nvdim
       Print *,i,evv(i)
       eevv(i,i)=evv(i)
    Enddo
    !!
    !!
    !!
    zzcv=-Matmul(Matmul(zzcc,Transpose(svc)),zzvv)
    zz(cindex,zzcindex)=zzcc
    zz(vindex,zzvindex)=zzvv
    zz(cindex,zzvindex)=zzcv
    zz(vindex,zzcindex)=0.0_t_real
    !!
    eb(zzcindex)=ecc
    eb(zzvindex)=evv
    !!
    ee0=0.0_t_real
    Do i=1,ndim
       ee0(i,i)=eb(i)
    Enddo
    !!
    s0=Real(Matmul(Transpose(Conjg(zz)),Matmul(hp0,zz)),t_real)-ee0
    !!
    Call printmat('Re 1',"(f15.8,' ')","('       0        ')"&
         &,s0(:,:),ndim,ndim,tol)
    !!
    Stop 'stop e_reduce0'
    !!
    Return
  End Subroutine e_reduce0
#endif
  !!***********************************************************************
  Subroutine rel_sym_check(symmetry,structure,lattice,basis,loi)  
    !!
    Use loisym
    !
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Basis),Intent(inout)  :: basis
    Type(T_Loi),Intent(in)       :: loi
    !!LOCAL
    Character(*),Parameter::nsubfunc="rel_sym_check"
    Integer(t_int) :: isa,issa,ioci,jops,isga,gioci,lena,kinv,j,ma,mb,l,kap
    Integer(t_int) :: mub,i,itci,gitci,irab,irgagb,isb,issb,isgb,lenb
    Real(t_real)   :: eu(3),tol,finv,s
    !!
    Integer(t_int),Parameter :: KMAX=ORBITAL_MAX_L+1
    Integer(t_int),Parameter :: JMAX=2*ORBITAL_MAX_L+1
    !!
    Real(t_real)   :: fac(0:JMAX)
    Complex(t_real):: u((JMAX+1)**2)
    Complex(t_real)  :: wu_ks(JMAX+1,JMAX+1,-KMAX:KMAX,48)
    !!
    Integer(t_int),Pointer :: p_gdiff(:,:,:,:)
    Complex(t_real),Pointer :: p_oci(:)
    Complex(t_real),Pointer :: p_tci(:)
    !!
    Nullify(p_gdiff)
    Nullify(p_oci)
    Nullify(p_tci)
    !!
    tol=1.0e-10_t_real
    !!
    Call make_factorials(fac,JMAX)
    !!
    Do jops=1,symmetry% Rnumops
       !Write(outfile,1) symmetry% Ropsymbol(jops)
       kinv=symmetry% opeulerinv(symmetry% Ropind(jops))
       eu=symmetry% opeuler(:,symmetry% Ropind(jops))
       Call write_euler(eu)
       
       Do kap=-KMAX,KMAX
          If(kap==0) Cycle
          j=2*Abs(kap)-1
          l=(Abs(2*kap+1)-1)/2
          finv=1.0_t_real
          If(kinv==1) finv=(-1.0_t_real)**l

          Call wignerDJ(j,eu,u(:),fac,.True.)
          Do ma=1,j+1
             Do mb=1,j+1
                i=mb
                s=1.0_t_real
                If(symmetry% Roptimerev(jops)==1) Then 
                   i=j+1-(mb-1) !! index of negative mu
                   mub=2*(mb-1)-j !! mu of index mb
                   s=(-1.0_t_real)**((mub+1)/2)*Sign(1_t_int,kap)
                Endif
                   
                wu_ks(ma,mb,kap,jops)=u(ma+(j+1)*(i-1))*s*finv
                !Print *,wu_ks(ma,mb,kap,jops)
             Enddo
          Enddo
          !!
          !Print *,'J=',j
          !Call printmat('Wj  ',"(f12.5,1x)","('      0      ')"&
          !     &,Real (wu_ks(1:j+1,1:j+1,kap,jops),t_real),j+1,j+1,tol)
          !Call printmat('Wj  ',"(f12.5,1x)","('      0      ')"&
          !     &,Aimag (wu_ks(1:j+1,1:j+1,kap,jops)      ),j+1,j+1,tol)

          
       Enddo
    Enddo
    !!
    !!
    Call palloc("p_oci",p_oci&
         &,1_t_int,loi% ocivvdim&
         &,nsubfunc,nmodule)
    Call palloc("p_tci",p_tci&
         &,1_t_int,loi% tcivvdim&
         &,nsubfunc,nmodule)
    !!
    Call gvecdifn(structure% nsite,lattice% p_npairs,lattice% npairsmax&
         &,lattice% p_pairs&
         &,symmetry% Rnumops,structure% p_equivsites,symmetry% Ropmat&
         &,p_gdiff)
    
    !!
    ojopsloop: Do jops=1,symmetry% Rnumops
    
       If(symmetry% Roptimerev(jops)==1)Then
          Write(outfile,11) symmetry% Ropsymbol(jops)
       Else
          Write(outfile,1) symmetry% Ropsymbol(jops)
       Endif

       p_oci(:)=-loi% p_roci_vHv(:)
       p_tci(:)=-loi% p_rtci_vHv(:)

       Do isa=1,structure% nsite
          issa=structure% p_isrt(isa)
          !!
          lena=Sum(basis% p_degRv(1:basis% p_nRv(issa),issa))
          !!
          ioci=loi% p_ocivvindex(isa)
          !!
          !!
          isga=structure% p_equivsites(jops,isa)
          
          Print *,'site=',isa,' G site=',isga
          !!
          !!
          gioci=loi% p_ocivvindex(isga)
          !!
          !!
          Call sym_loi_block(&
               &basis% p_nRv(issa),basis% p_kapRv(:,issa)&
               &,basis% p_nRv(issa),basis% p_kapRv(:,issa)&
               &,loi% p_roci_vHv,ioci,gioci,p_oci&
               &,lena,lena&
               &,wu_ks(:,:,:,jops),symmetry% Roptimerev(jops))
          
          !!
          Do isb=1,structure% nsite
             issb=structure% p_isrt(isb)

             lenb=Sum(basis% p_degRv(1:basis% p_nRv(issb),issb))
             !!
             isgb=structure% p_equivsites(jops,isb)
             !!
             irabloop: Do irab=1,lattice% p_npairs(isa,isb)
                !!
                itci=loi% p_tcivvindex(irab,isa,isb)
                !!
                irgagb=p_gdiff(irab,jops,isb,isa)
                !!
                gitci=loi% p_tcivvindex(irgagb,isga,isgb)
                !!
                Call sym_loi_block(&
                     &basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                     &,basis% p_nRv(issb),basis% p_kapRv(:,issb)&
                     &,loi% p_rtci_vHv,itci,gitci,p_tci&
                     &,lena,lenb&
                     &,wu_ks(:,:,:,jops),symmetry% Roptimerev(jops))
                !!
             Enddo irabloop
             !!
             !!
          Enddo ! isb
          
       End Do ! isa
       

       !TODO: tcis
       !TODO: k-matrices
       If(Any( Abs(p_oci(:))>tol ) )Then
          !Call printmat('Re H  ',"(f12.5,1x)","('      0      ')"&
          !     &,Real (loi% p_roci_vHv(ioci+1:),t_real),lena,lena,tol)
          !Call printmat('Re Hg ',"(f12.5,1x)","('      0      ')"&
          !     &,Real (p_oci(ioci+1:),t_real),lena,lena,tol)
          !Call printmat('Im H  ',"(f12.5,1x)","('      0      ')"&
          !     &,Aimag (loi% p_roci_vHv(ioci+1:)),lena,lena,tol)
          !Call printmat('Im Hg ',"(f12.5,1x)","('      0      ')"&
          !     &,Aimag (p_oci(ioci+1:)),lena,lena,tol)
          Print *,'trouble in wonderland oci'
       Endif
       !!
       If(Any( Abs(p_tci(:))>tol ) )Then
          
          !Call printmat('Re H  ',"(f12.5,1x)","('      0      ')"&
          !     &,Real (loi% p_roci_vHv(ioci+1:),t_real),lena,lena,tol)
          !Call printmat('Re Hg ',"(f12.5,1x)","('      0      ')"&
          !     &,Real (p_oci(ioci+1:),t_real),lena,lena,tol)
          !Call printmat('Im H  ',"(f12.5,1x)","('      0      ')"&
          !     &,Aimag (loi% p_roci_vHv(ioci+1:)),lena,lena,tol)
          !Call printmat('Im Hg ',"(f12.5,1x)","('      0      ')"&
          !     &,Aimag (p_oci(ioci+1:)),lena,lena,tol)
          Print *,'trouble in wonderland tci'
       Endif
       !!
       !!
       !!
    Enddo ojopsloop
    !!
    !!
    !!
    Call pfree(p_oci)
    Call pfree(p_tci)
    Call pfree(p_gdiff)
    !!
    Stop
    !!
    Return
    !!
11   Format('OP: ',a15,' time reversal') 
1   Format('OP: ',a15) 
    !!
  End Subroutine rel_sym_check
  !!***********************************************************************
  Subroutine sym_loi_block(nRa,kapRa,nRb,kapRb&
       &,loi,iloi,giloi,lloi&
       &,lena,lenb&
       &,wu_ks,timerev)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nRa,kapRa(:)
    Integer(t_int),Intent(in)    :: nRb,kapRb(:)
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: iloi,giloi
    Integer(t_int),Intent(in)    :: timerev
    Integer(t_int),Parameter :: KMAX=ORBITAL_MAX_L+1
    Integer(t_int),Parameter :: JMAX=2*ORBITAL_MAX_L+1
    Complex(t_real),Intent(in)      :: wu_ks(1:JMAX+1,1:JMAX+1,-KMAX:KMAX)
    Complex(t_real),Intent(in)      :: loi(:)
    Complex(t_real),Intent(inout)   :: lloi(:)
    !!LOCAL
    Integer(t_int) :: ii,jj,ma,kapa,ja,iva,ivb,kapb,jb,degb,dega
    Complex(t_real)   :: A(lena,lenb)
    !!
    !!
    !!
    !! Oin,Pjm=( wuT* Ogin,DPgjm)*wu )
    ii=giloi
    Do iva=1,lena
       jj=0
       Do ivb=1,nRb
          kapb=kapRb(ivb)
          jb=2*Abs(kapb)-1
          degb=jb+1
          !!
          A(iva,jj+1:jj+degb)=Matmul(loi(ii+1:ii+degb)&
               &             ,wu_ks(1:degb,1:degb,kapb))
          jj=jj+degb
          ii=ii+degb
       Enddo
    Enddo
    !!
    ii=iloi
    jj=0
    Do iva=1,nRa
       kapa=kapRa(iva)
       ja=2*Abs(kapa)-1
       dega=ja+1
       Do ma=1,dega
          !!
          If(timerev==1)Then
             lloi(ii+1:ii+lenb)=lloi(ii+1:ii+lenb)&
                  &+Conjg(Matmul(Conjg(wu_ks(1:dega,ma,kapa))&
                  & ,A(jj+1:jj+dega,1:lenb)))
          Else
             lloi(ii+1:ii+lenb)=lloi(ii+1:ii+lenb)&
                  &+Matmul(Conjg(wu_ks(1:dega,ma,kapa))&
                  & ,A(jj+1:jj+dega,1:lenb))
          Endif
          !!
          ii=ii+lenb
       Enddo
       jj=jj+dega
    Enddo
    !!
    
    !!
  End Subroutine sym_loi_block
  !!***********************************************************************
  !! this are test routines, not optimized
#ifdef my_compile_unused
  Subroutine  calcRelTrans(nvdim,nsite,nsort,isrt,basis,nrdim,p_UT)
    !!
    Implicit None  
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: nsite,nvdim,nsort,isrt(*)
    Integer(t_int),Intent(out)   :: nrdim
    Complex(t_real),Pointer      :: p_UT(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calcRelTrans"
    Integer(t_int),Parameter :: lmax=ORBITAL_MAX_L
    Integer(t_int) :: issa,iv,ivr,kap,jj,j0,j,l,mu,m,deg,i1,ms,n,il,ik,isa
    !!
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Logical(t_log),Parameter :: prot=.False.
    !!
    Complex(t_real),Pointer  :: p_T(:,:,:,:)
    Complex(t_real),Pointer  :: p_U(:,:,:)
    Integer(t_int),Pointer   :: p_nv(:),p_lv(:,:)
    !!
    !!
    Nullify(p_U)
    Nullify(p_T)
    Nullify(p_nv)
    Nullify(p_lv)
    !!
    !!
    !!
    Call palloc('p_U',p_U,1_t_int,2*lmax+1,1_t_int,2*lmax+1&
         &,0_t_int,lmax,nmodule,nsubfunc)
    !!
    Call make_ylm_U_matrix(lmax,p_U)
    !!
    !!
    Call palloc('p_T',p_T,1_t_int,2*(2*lmax+1),1_t_int,2*(2*lmax+1)&
         &,1_t_int,2_t_int,0_t_int,lmax,nmodule,nsubfunc)
    !!
    !!
    Call palloc('p_nv',p_nv,1_t_int,nsort,nmodule,nsubfunc)
    Call palloc('p_lv',p_lv,1_t_int,basis% nRvmx,1_t_int,nsort&
         &,nmodule,nsubfunc)
    !!
    Do issa=1,nsort
       iv=0
       ivrloop1: Do ivr=1,basis% p_nRv(issa)
          kap=basis% p_kapRv(ivr,issa)
          l=(Abs(2*kap+1)-1)/2
          If(kap>0) Cycle ivrloop1
          iv=iv+1
          p_lv(iv,issa)=l
       Enddo ivrloop1
       p_nv(issa)=iv
    Enddo
    !!
    !!
    If(prot)Then
       Do l=0,lmax
          Print *,'l=',l
          j0=1
          If(l==0) j0=2
          Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             Do m=1,2*Abs(kap)
                mu=2*(m-1)-j
                Print 12,l,kap,j,mu
12              Format(i1,1x,i3,1x,i1,'/2',1x,i3,'/2') 
                
             Enddo
          Enddo
       Enddo
    Endif
    !!
    p_T(:,:,:,:)=c0
    Do l=0,lmax
       deg=2*l+1
       j0=1 ; If(l==0)j0=2
       Do m=1,deg
          i1=0
          Do jj=j0,2
             kap=l-(jj-1)*deg
             j=2*Abs(kap)-1
             Do mu=-j,j,2
                i1=i1+1
                Do ms=1,2
                   !! U_mn d_n,mu-s/2
                   n=(mu-1)/2 + ms + l
                   If(1 <=n .And. n <= deg  ) Then
                      p_T(i1,m,ms,l)=p_U(m,n,l)*cgoOFKapMu(ms,kap,mu)
                   Endif
                Enddo
             Enddo
          Enddo
       Enddo
       If(prot)Then
          Print *,'l==================================',l
          Do ms=1,2
             Do i1=1,2
                Print *,'T*T=',ms,i1
                Call printmat('TT*TT^+',"(f12.5,' ')" ,"('     0       ')"&
                     &,Real(Matmul(Transpose(p_T(1:2*deg,1:deg,ms,l))&
                     &,Conjg(p_T(1:2*deg,1:deg,i1,l))))&
                     &,deg,deg ,1.0e-7_t_real)
             Enddo
          Enddo
       Endif
    Enddo
    !!
    !!
    nrdim=0
    Do isa=1,nsite
       issa=isrt(isa)
       nrdim=nrdim+Sum(2*p_lv(1:p_nv(issa),issa)+1)
    Enddo
    !!
    Call palloc('p_UT',p_UT,1_t_int,nrdim,1_t_int,nvdim,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    !!
    Do ms=1,2
       il=0
       ik=0
       Do isa=1,nsite
          issa=isrt(isa)
          Do iv=1,p_nv(issa)
             l=p_lv(iv,issa)
             deg=(2*l+1)
             Do m=1,deg
                il=il+1
                !Print "('isa=',i5,1x,a1,sp,i2,ss,' il=',i5)"&
                !     &,isa,lqnchar(l),m-l-1,il
                i1=ik
                j0=1; If(l==0) j0=2
                Do jj=j0,2
                   kap=l-(jj-1)*deg
                   j=2*Abs(kap)-1
                   Do mu=-j,j,2
                      i1=i1+1
                      !Print "('i1=',i4,' iT=',i4)",i1,i1-ik
                      p_UT(il,i1,ms)=p_T(i1-ik,m,ms,l)
                   Enddo
                Enddo
             Enddo
             ik=ik+2*deg
          Enddo
       Enddo
    Enddo! ms
    !!
    If(prot) Then
       Print *,'nrdim=',nrdim,Size(p_UT,1),Size(p_UT,2),Size(p_UT,3)
       !!
       Do ms=1,2
          Do i1=1,2
             !!
             Call printmat('TT*TT^+',"(f12.5,' ')" ,"('     0       ')"&
                  &,Real(Matmul(p_UT(1:nrdim,:,ms)&
                  &,Transpose(Conjg(p_UT(1:nrdim,:,i1)))))&
                  &,nrdim,nrdim ,1.0e-7_t_real)
          Enddo
       Enddo
    Endif
    !! 
    !!
    Call pfree(p_nv)
    Call pfree(p_lv)
    Call pfree(p_T)
    Call pfree(p_U)
    !!
    Return
    !!
  End Subroutine calcRelTrans
#endif
!!$  !!***********************************************************************
!!$  Subroutine dtest(structure,basis,kohnsham,loi,ms)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Structure),Intent(in) :: structure
!!$    Type(T_Basis),Intent(in)     :: basis
!!$    Type(T_Kohnsham),Intent(in)  :: kohnsham
!!$    Type(T_Loi),Intent(in)       :: loi
!!$    Integer(t_int),Intent(in)    :: ms
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="dtest"
!!$    Integer(t_int) :: isa,issa,ma,la,mb,lb,i,j,ia,ib,nva,it,ioci
!!$    Real(t_real)   :: ham,ovl
!!$    Real(t_real)   :: ah(2,2)
!!$    Real(t_real)   :: as(2,2)
!!$    Real(t_real)   :: az(2,2)
!!$    Real(t_real)   :: ae(2)
!!$    !!
!!$    !!
!!$    !Do isa=1,structure% nsite
!!$    isa=1
!!$    issa=structure% p_isrt(isa)
!!$
!!$    nva=basis% p_nRv(issa)
!!$
!!$
!!$    ah=0.0_t_real
!!$    as=0.0_t_real
!!$    
!!$    it=kohnsham% p_vblockstart(isa)-1
!!$    ioci=loi% p_ocivvindex(isa)
!!$    
!!$    i=it
!!$    Do ia=1,nva
!!$       la=basis% p_lRv(ia,issa)
!!$       Do ma=1,2*la+1
!!$          i=i+1
!!$          j=it
!!$          Do ib=1,nva
!!$             lb=basis% p_lRv(ib,issa)
!!$             Do mb=1,2*lb+1
!!$                j=j+1
!!$                ioci=ioci+1_t_int
!!$                ham=loi% p_oci_vHv(ioci,ms)
!!$                ovl=loi% p_oci_vv0(ioci)
!!$                
!!$                If( la==2 .And. basis% p_nnRv(ia,issa)==4 .And. &
!!$                     &lb==2 .And. basis% p_nnRv(ib,issa)==4 ) Then
!!$                   as(1,1)=as(1,1)+ovl
!!$                   ah(1,1)=ah(1,1)+ham
!!$                Endif
!!$                If( la==2 .And. basis% p_nnRv(ia,issa)==4 .And. &
!!$                     &lb==2 .And. basis% p_nnRv(ib,issa)==5 ) Then
!!$                   as(1,2)=as(1,2)+ovl
!!$                   ah(1,2)=ah(1,2)+ham
!!$                   
!!$                Endif
!!$                
!!$                If( la==2 .And. basis% p_nnRv(ia,issa)==5 .And. &
!!$                     &lb==2 .And. basis% p_nnRv(ib,issa)==4 ) Then
!!$                   as(2,1)=as(2,1)+ovl
!!$                   ah(2,1)=ah(2,1)+ham
!!$                   
!!$                Endif
!!$                If( la==2 .And. basis% p_nnRv(ia,issa)==5 .And. &
!!$                     &lb==2 .And. basis% p_nnRv(ib,issa)==5 ) Then
!!$                   as(2,2)=as(2,2)+ovl
!!$                   ah(2,2)=ah(2,2)+ham
!!$                   
!!$                Endif
!!$             Enddo             ! mb
!!$          Enddo               ! ib
!!$       Enddo                 ! ma
!!$    Enddo                   ! ia
!!$    !!
!!$    ah=ah/5.0_t_real
!!$    as=as/5.0_t_real
!!$    !!
!!$    Call printmat('hp d',"(f12.5,' ')","('     0       ')"&
!!$         &,ah,2_t_int,2_t_int,1.0e-5_t_real)
!!$    Call printmat('sp d',"(f12.5,' ')","('     0       ')"&
!!$         &,as,2_t_int,2_t_int,1.0e-5_t_real)
!!$    !!
!!$    Call rdiagno(2_t_int,ah,as,az,ae)
!!$    !!
!!$    Call printmat('Z d',"(f12.5,' ')","('     0       ')"&
!!$         &,az,2_t_int,2_t_int,1.0e-5_t_real)
!!$    !!
!!$    Call printmat('E d',"(f12.5,' ')","('     0       ')"&
!!$         &,ae,2_t_int,1_t_int,1.0e-5_t_real)
!!$    !!
!!$    Stop
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine dtest
  !!***********************************************************************
End Module ksaux

