Module loi0
  !!
  Use precisiondefs
  Use constants
  Use memory
  Use file_iounits
  Use errors
  Use tsymmetry
  Use torbitalbasis
  Use tmol
  Use tdiracdefs
  Use finuc
  Use tnumerics
  Use integrate1d
  Use trelativistic
  Use clock
  Use optionstest
  Use mesh
  Use matserv
  Use tmesh
  Use tloi
  Use loiorg
  Use loiaux

  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="LOI0"
  !!
  Public :: loi_int0,calc_tcis
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine loi_int0(structure,lattice,basis,mol,numerics,loi&
       &,cblocklength,vblocklength,ncsmx,nvsmx&
       &,relativistic,mspin,quant,natom,nuc_type,need_sigma)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Mol),Intent(inout)    :: mol
    Type(T_Basis),Intent(inout)  :: basis
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Quant),Intent(in)     :: quant
    Type(T_Loi),Intent(inout)    :: loi
    Logical(t_log),Intent(in)    :: need_sigma
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: mspin,nuc_type
    Integer(t_int),Intent(in)    :: ncsmx,cblocklength(*)
    Integer(t_int),Intent(in)    :: nvsmx,vblocklength(*)
    Real(t_real),Intent(in)      :: natom(*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="loi_int0"
    Logical(t_log) :: rel,relmag
    Integer(t_int) :: isa,issa,isb,issb,wudim,hzlmx,ncomp
    Real(t_real),Pointer :: p_wu    (:)
    Real(t_real),Pointer :: p_fac   (:)
    Real(t_real),Pointer :: p_tci_vc0   (:)
    Real(t_real),Pointer :: p_tci_vHc0  (:)
    Real(t_real),Pointer :: p_tci_vv0   (:)
    Real(t_real),Pointer :: p_tci_vHv0  (:)
    Character(*),Parameter :: cpumesg_oci='One-center integrals'
    Character(*),Parameter :: cpumesg_tci='Two-center integrals'
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    Integer(t_int) :: itcivc,itci,ir,icomp
    Real(t_real)   :: rffmax,ZZa,rnuca,ZZb,rnucb
    Type(T_Mesh),Pointer :: bmesha,bmeshb
    Type(T_Mesh),Pointer :: atommesha,atommeshb
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    relmag=(rel .And. mspin==2)
    ncomp=basis% ncomp
    !!

    !!
    Nullify(p_tci_vc0)
    Nullify(p_fac)
    Nullify(p_tci_vHc0)
    Nullify(p_tci_vv0)
    Nullify(p_tci_vHv0)
    Nullify(p_wu)
    !!
    !! everything, which is not crystal potential related
    !!
    Call watch(cpumesg_oci,cpu_start,reset=.True.)
    Call watch(cpumesg_oci,cpu_stop)
    Call watch(cpumesg_tci,cpu_start,reset=.True.)
    Call watch(cpumesg_tci,cpu_stop)
    !!
    !! initialize LOIs
    loi% p_oci_cc0 (:) =0.0_t_real
    loi% p_oci_cHc0(:) =0.0_t_real
    loi% p_oci_vv0 (:) =0.0_t_real
    loi% p_oci_vHv0(:) =0.0_t_real
    loi% p_oci_vc0 (:) =0.0_t_real
    loi% p_oci_vHc0(:) =0.0_t_real
    If(rel)Then

       !! need temporary arrays
       Call palloc("p_tci_vv0" ,p_tci_vv0&
            &,1_t_int,loi% tcivvdim,nmodule,nsubfunc)
       Call palloc("p_tci_vHv0",p_tci_vHv0&
            &,1_t_int,loi% tcivvdim,nmodule,nsubfunc)
       Call palloc("p_tci_vc0" ,p_tci_vc0&
            &,1_t_int,loi% tcivcdim,nmodule,nsubfunc)
       Call palloc("p_tci_vHc0" ,p_tci_vHc0&
            &,1_t_int,loi% tcivcdim,nmodule,nsubfunc)
       

       p_tci_vv0 (:) =0.0_t_real
       p_tci_vHv0(:) =0.0_t_real
       p_tci_vc0 (:) =0.0_t_real
       p_tci_vHc0 (:) =0.0_t_real

       loi% p_vbSv0(:,:,:) =c0
       loi% p_cbSc0(:,:,:) =c0
       loi% p_vbSc0 (:,:,:) =c0
       
       loi% p_rtci_vbSv0 (:) =c0

       loi% p_rtci_vv0 (:) =c0
       loi% p_rtci_vHv0(:) =c0
       loi% p_rtci_vc0 (:) =c0
       loi% p_rtci_vHc0 (:) =c0
       loi% p_rtci_vbSc0 (:) =c0

       !! sigma-vector in global coords
       loi% p_cbVSc0(:,:,:,:) =c0
       loi% p_vbVSv0(:,:,:,:) =c0
       loi% p_vbVSc0 (:,:,:,:) =c0
       loi% p_rtci_vbVSv0 (:,:) =c0
       loi% p_rtci_vbVSc0 (:,:) =c0

    Else
       loi% p_tci_vv0 (:) =0.0_t_real
       loi% p_tci_vHv0(:) =0.0_t_real
       loi% p_tci_vc0 (:) =0.0_t_real
       loi% p_tci_vHc0 (:) =0.0_t_real
    Endif
    !!
    !! for tci_gkp we need at least lmx==3
    !! in the moment it may be 4 at maximum, yhzy... must be
    !! generalized in tci_gkp, if more is needed
    hzlmx=Max(3_t_int,basis% lmx)
    !! valence/core Wigner D-functions (rotation matrices)
    !! for calc_tcis
    wudim=(basis% lmx+1)*(4*(basis% lmx+1)**2-1)/3
    Call palloc('p_wu',p_wu,1_t_int,wudim,nmodule,nsubfunc) 
    !!
    !! factorials for the ytr calls to calculate the Wigner D-functions
    !!
    !!
    Call palloc('p_fac',p_fac ,0_t_int,2*basis% lmx+1 ,nmodule,nsubfunc) 
    !!
    Call make_factorials(p_fac,2*basis% lmx+1)
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !!
       ZZa=natom(issa)
       !!
       rnuca=nuc_max_rad(nuc_type,ZZa)
       !!
       bmesha    => basis% p_basmeshs(issa)
       atommesha => mol% p_atommeshs(issa)
       !!
       !! calculate one-center integrals 
       !!
       Call watch(cpumesg_oci,cpu_start)
       !!
       If(rel)Then
          Call rel_calc_oci_vcs(quant,mol% p_vatom(:,0,issa)&
               &,numerics% oci_nr,atommesha,mol% nra,bmesha,basis% nrb&
               &,numerics% degree,basis% nRcmx,basis% nRcmx&
               &,basis% p_nRc(issa),basis% p_kapRc(:,issa)&
               &,basis% p_degRc(:,issa)&
               &,basis% p_nRc(issa),basis% p_kapRc(:,issa)&
               &,basis% p_degRc(:,issa)&
               &,basis% p_fc(:,:,:,issa),basis% p_fkc(:,:,:,issa)&
               &,basis% p_fc(:,:,:,issa),basis% p_fkc(:,:,:,issa)&
               &,basis% p_fgam(issa)&
               &,ZZa,rnuca,nuc_type&
               &,loi% p_ociccindex(isa)&
               &,loi% ociccdim,loi% p_oci_cc0,loi% p_oci_cHc0&
               &,ncsmx,ncsmx,loi% p_cbSc0(:,:,isa),loi% p_cbVSc0&
               &,need_sigma,isa)

       Else
          Do icomp=1,ncomp
             Call nrel_calc_oci_vcs(mol% p_vatom(:,0,issa),numerics% oci_nr&
                  &,atommesha,mol% nra,bmesha,basis% nrb,numerics% degree&
                  &,basis% nRcmx,basis% nRcmx&
                  &,basis% p_nRc(issa),basis% p_lRc(:,issa)&
                  &,basis% p_nRc(issa),basis% p_lRc(:,issa)&
                  &,basis% p_fc(:,:,icomp,issa),basis% p_fkc(:,:,icomp,issa)&
                  &,basis% p_fc(:,:,icomp,issa),basis% p_fkc(:,:,icomp,issa)&
                  &,basis% p_fgam(issa)&
                  &,ZZa,rnuca,nuc_type&
                  &,loi% p_ociccindex(isa)&
                  &,loi% ociccdim,loi% p_oci_cc0,loi% p_oci_cHc0)
          Enddo
       Endif
       !!
       !Stop
       !! non relativistic
       If(rel)Then

          Call rel_calc_oci_vcs(quant,mol% p_vatom(:,0,issa)&
               &,numerics% oci_nr,atommesha,mol% nra,bmesha,basis% nrb&
               &,numerics% degree,basis% nRvmx,basis% nRvmx&
               &,basis% p_nRv(issa),basis% p_kapRv(:,issa)&
               &,basis% p_degRv(:,issa)&
               &,basis% p_nRv(issa),basis% p_kapRv(:,issa)&
               &,basis% p_degRv(:,issa)&
               &,basis% p_fv(:,:,:,issa),basis% p_fkv(:,:,:,issa)&
               &,basis% p_fv(:,:,:,issa),basis% p_fkv(:,:,:,issa)&
               &,basis% p_fgam(issa)&
               &,ZZa,rnuca,nuc_type&
               &,loi% p_ocivvindex(isa)&
               &,loi% ocivvdim,loi% p_oci_vv0,loi% p_oci_vHv0&
               &,nvsmx,nvsmx,loi% p_vbSv0(:,:,isa),loi% p_vbVSv0&
               &,need_sigma,isa)

          Call rel_calc_oci_vcs(quant,mol% p_vatom(:,0,issa)&
               &,numerics% oci_nr,atommesha,mol% nra,bmesha,basis% nrb&
               &,numerics% degree,basis% nRvmx,basis% nRcmx&
               &,basis% p_nRv(issa),basis% p_kapRv(:,issa)&
               &,basis% p_degRv(:,issa)&
               &,basis% p_nRc(issa),basis% p_kapRc(:,issa)&
               &,basis% p_degRc(:,issa)&
               &,basis% p_fv(:,:,:,issa),basis% p_fkv(:,:,:,issa)&
               &,basis% p_fc(:,:,:,issa),basis% p_fkc(:,:,:,issa)&
               &,basis% p_fgam(issa)&
               &,ZZa,rnuca,nuc_type&
               &,loi% p_ocivcindex(isa)&
               &,loi% ocivcdim,loi% p_oci_vc0,loi% p_oci_vHc0&
               &,nvsmx,ncsmx,loi% p_vbSc0(:,:,isa),loi% p_vbVSc0&
               &,need_sigma,isa)

          
       Else
          
          Do icomp=1,ncomp
             Call nrel_calc_oci_vcs(mol% p_vatom(:,0,issa),numerics% oci_nr&
                  &,atommesha,mol% nra,bmesha,basis% nrb,numerics% degree&
                  &,basis% nRvmx,basis% nRvmx&
                  &,basis% p_nRv(issa),basis% p_lRv(:,issa)&
                  &,basis% p_nRv(issa),basis% p_lRv(:,issa)&
                  &,basis% p_fv(:,:,icomp,issa),basis% p_fkv(:,:,icomp,issa)&
                  &,basis% p_fv(:,:,icomp,issa),basis% p_fkv(:,:,icomp,issa)&
                  &,basis% p_fgam(issa)&
                  &,ZZa,rnuca,nuc_type&
                  &,loi% p_ocivvindex(isa)&
                  &,loi% ocivvdim,loi% p_oci_vv0,loi% p_oci_vHv0)
             !!
             Call nrel_calc_oci_vcs(mol% p_vatom(:,0,issa),numerics% oci_nr&
                  &,atommesha,mol% nra,bmesha,basis% nrb,numerics% degree&
                  &,basis% nRvmx,basis% nRcmx&
                  &,basis% p_nRv(issa),basis% p_lRv(:,issa)&
                  &,basis% p_nRc(issa),basis% p_lRc(:,issa)&
                  &,basis% p_fv(:,:,icomp,issa),basis% p_fkv(:,:,icomp,issa)&
                  &,basis% p_fc(:,:,icomp,issa),basis% p_fkc(:,:,icomp,issa)&
                  &,basis% p_fgam(issa)&
                  &,ZZa,rnuca,nuc_type&
                  &,loi% p_ocivcindex(isa),loi% ocivcdim&
                  &,loi% p_oci_vc0,loi% p_oci_vHc0)

          Enddo
       Endif
       !!
       Call watch(cpumesg_oci,cpu_read,Stop=.True.,Print=(outlevel >= L_many))
       !!
       !! calculation of two-center integrals
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          issb=structure% p_isrt(isb)
          !!
          ZZb=natom(issb)
          !!
          rnucb=nuc_max_rad(nuc_type,ZZb)
          !!
          bmeshb    => basis% p_basmeshs(issb)
          atommeshb => mol% p_atommeshs(issb)
          !!
          Call watch(cpumesg_tci,cpu_start)
          !!
          rffmax=Max(basis% p_rtfcmax(0,issa),basis% p_rtfvmax(0,issa)&
               &    ,basis% p_rtfcmax(0,issb),basis% p_rtfvmax(0,issb))
          
          !!
          If( rel)Then
             
             
             
             Call calc_tcis(&
                  !&structure% nsite,&
                  &isa,isb&
                  &,lattice% p_npshell(isa,isb)&
                  &,lattice% p_pairs(:,:,isa,isb),lattice% p_pairabs(:,isa,isb)&
                  &,lattice% p_ninpshell(:,isa,isb)&
                  &,atommesha,atommeshb,bmesha,bmeshb,basis% nrb&
                  &,mol% p_vatom(:,0,issa),mol% p_vatom(:,0,issb)&
                  &,rffmax&
                  &,basis% p_nRv(issa),basis% p_nRv(issb)&
                  &,basis% p_nRc(issb),basis% nRvmx,basis% nRcmx&
                  &,hzlmx,basis% lmx,basis% p_lRv(:,issa),basis% p_lRv(:,issb)&
                  &,basis% p_lRc(:,issb)&
                  &,basis% p_fv (:,:,1,issa)&
                  &,basis% p_fv (:,:,1,issb)&
                  &,basis% p_fkv(:,:,1,issa)&
                  &,basis% p_fkv(:,:,1,issb)&
                  &,basis% p_fc (:,:,1,issb)&
                  &,basis% p_fkc(:,:,1,issb)&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fgam(issb)&
                  &,ZZa,ZZb,rnuca,rnucb,nuc_type&
                  &,numerics% degree,basis% lmx,p_fac,p_wu&
                  &,p_tci_vHv0,p_tci_vv0&
                  &,p_tci_vHc0,p_tci_vc0&
                  &,loi% p_tcivvindex(:,isa,isb),loi% p_tcivcindex(:,isa,isb)&
                  &,prot=.True.)


          Else             
             !Do icomp=1,ncomp
             !   If(icomp==2 .Or. icomp==4 ) Cycle
             icomp=1
             Call calc_tcis(&
                  !&structure% nsite,&
                  &isa,isb&
                  &,lattice% p_npshell(isa,isb)&
                  &,lattice% p_pairs(:,:,isa,isb),lattice% p_pairabs(:,isa,isb)&
                  &,lattice% p_ninpshell(:,isa,isb)&
                  &,atommesha,atommeshb,bmesha,bmeshb,basis% nrb&
                  &,mol% p_vatom(:,0,issa),mol% p_vatom(:,0,issb)&
                  &,rffmax&
                  &,basis% p_nRv(issa),basis% p_nRv(issb)&
                  &,basis% p_nRc(issb),basis% nRvmx,basis% nRcmx&
                  &,hzlmx,basis% lmx,basis% p_lRv(:,issa),basis% p_lRv(:,issb)&
                  &,basis% p_lRc(:,issb)&
                  &,basis% p_fv (:,:,icomp,issa)&
                  &,basis% p_fv (:,:,icomp,issb)&
                  &,basis% p_fkv(:,:,icomp,issa)&
                  &,basis% p_fkv(:,:,icomp,issb)&
                  &,basis% p_fc (:,:,icomp,issb)&
                  &,basis% p_fkc(:,:,icomp,issb)&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fgam(issb)&
                  &,ZZa,ZZb,rnuca,rnucb,nuc_type&
                  &,numerics% degree,basis% lmx,p_fac,p_wu&
                  &,loi% p_tci_vHv0,loi% p_tci_vv0&
                  &,loi% p_tci_vHc0,loi% p_tci_vc0&
                  &,loi% p_tcivvindex(:,isa,isb),loi% p_tcivcindex(:,isa,isb)&
                  &,prot=.True.)
          !Enddo
          Endif
          !!
          Call watch(cpumesg_tci,cpu_read,Stop=.True.,Print=(outlevel >= L_many))
          !!
          !!
       End Do isbloop
       !!
    End Do isaloop
    !!
    !!
    !!
    !!
    !!
    !!
    If(rel) Then
       !print *,' reltrans_loi_block0'
       Do isa=1,structure% nsite
          issa=structure% p_isrt(isa)
          Do isb=1,structure% nsite
             issb=structure% p_isrt(isb)
             Do ir=1,lattice% p_npairs(isa,isb)

                itci  =loi% p_tcivvindex(ir,isa,isb)
                itcivc=loi% p_tcivcindex(ir,isa,isb)
                
                 
                Call reltrans_loi_block0(&
                     & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                     &,basis% p_nRv(issb),basis% p_kapRv(:,issb)&
                     &,itci,p_tci_vv0,loi% p_rtci_vv0&
                     &,nvsmx,nvsmx,vblocklength(isa),vblocklength(isb)&
                     &,basis% lmx,basis% p_relTOpseudo,1.0_t_real)

                If(relmag)Then
                   Call reltrans_loi_block0(&
                        & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                        &,basis% p_nRv(issb),basis% p_kapRv(:,issb)&
                        &,itci,p_tci_vv0,loi% p_rtci_vbSv0&
                        &,nvsmx,nvsmx,vblocklength(isa),vblocklength(isb)&
                        &,basis% lmx,basis% p_relTOpseudo,-1.0_t_real)
                Endif

                Call reltrans_loi_block0(&
                     & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                     &,basis% p_nRv(issb),basis% p_kapRv(:,issb)&
                     &,itci,p_tci_vHv0,loi% p_rtci_vHv0&
                     &,nvsmx,nvsmx,vblocklength(isa),vblocklength(isb)&
                     &,basis% lmx,basis% p_relTOpseudo,1.0_t_real)
                
                Call reltrans_loi_block0(&
                     & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                     &,basis% p_nRc(issb),basis% p_kapRc(:,issb)&
                     &,itcivc,p_tci_vc0,loi% p_rtci_vc0&
                     &,nvsmx,ncsmx,vblocklength(isa),cblocklength(isb)&
                     &,basis% lmx,basis% p_relTOpseudo,1.0_t_real)

                If(relmag)Then
                   Call reltrans_loi_block0(&
                        & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                        &,basis% p_nRc(issb),basis% p_kapRc(:,issb)&
                        &,itcivc,p_tci_vc0,loi% p_rtci_vbSc0&
                        &,nvsmx,ncsmx,vblocklength(isa),cblocklength(isb)&
                        &,basis% lmx,basis% p_relTOpseudo,-1.0_t_real)
                Endif
                
                Call reltrans_loi_block0(&
                     & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                     &,basis% p_nRc(issb),basis% p_kapRc(:,issb)&
                     &,itcivc,p_tci_vHc0,loi% p_rtci_vHc0&
                     &,nvsmx,ncsmx,vblocklength(isa),cblocklength(isb)&
                     &,basis% lmx,basis% p_relTOpseudo,1.0_t_real)
                
                If(need_sigma)Then

                   Call reltrans_loi_blockS0(&
                        & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                        &,basis% p_nRv(issb),basis% p_kapRv(:,issb)&
                        &,itci,p_tci_vv0,loi% p_rtci_vbVSv0&
                        &,nvsmx,nvsmx,vblocklength(isa),vblocklength(isb)&
                        &,basis% lmx,basis% p_relTOpseudoNoD)

                   Call reltrans_loi_blockS0(&
                        & basis% p_nRv(issa),basis% p_kapRv(:,issa)&
                        &,basis% p_nRc(issb),basis% p_kapRc(:,issb)&
                        &,itcivc,p_tci_vc0,loi% p_rtci_vbVSc0&
                        &,nvsmx,ncsmx,vblocklength(isa),cblocklength(isb)&
                        &,basis% lmx,basis% p_relTOpseudoNoD)
                   
                Endif
                !!
             Enddo
          Enddo
       Enddo
    Endif
    !!
    !!
    If (isoption('NO_CORE')) Then
       If (rel) Then
          loi% p_oci_vc0 (:)   =0.0_t_real
          loi% p_oci_vHc0 (:)   =0.0_t_real
          loi% p_vbSc0 (:,:,:) =c0
          loi% p_vbVSc0 (:,:,:,:) =c0
          loi% p_rtci_vc0 (:) =c0
          loi% p_rtci_vHc0 (:) =c0
          loi% p_rtci_vbSc0 (:) =c0
       Else
          loi% p_oci_vc0 (:) =0.0_t_real
          loi% p_oci_vHc0 (:) =0.0_t_real
          loi% p_tci_vc0 (:) =0.0_t_real
          loi% p_tci_vHc0 (:) =0.0_t_real
       Endif

    Endif

    !!
    !Call print_loi0s(structure% nsite,lattice,cblocklength,vblocklength,loi)
    !!
    Call pfree(p_fac)
    Call pfree(p_wu)
    Call pfree(p_tci_vc0)
    Call pfree(p_tci_vHc0)
    Call pfree(p_tci_vv0)
    Call pfree(p_tci_vHv0)
    !!
    Call watch(cpumesg_oci,cpu_read,delete=.True.,Print=.True.,mesg='Total ')
    Call watch(cpumesg_tci,cpu_read,delete=.True.,Print=.True.,mesg='Total ')
    !!
    Return
    !!
  End Subroutine loi_int0
  !!***********************************************************************
  !! calculates the one-center hamilton and overlap elements
  !! between core and valence
  Subroutine nrel_calc_oci_vcs(vatom,oci_nr,atommesh,nramax,bmesh,nrbmax&
       &,degree,nRmxa,nRmxb,nRa,lRa,nRb,lRb,fna,fkna,fnb,fknb,fgam&
       &,ZZ,rnuc,nuc_type&
       &,ioci0,ocidim,oci_nn0,oci_nHn0)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: atommesh
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: nuc_type
    Real(t_real),Intent(in)      :: ZZ,rnuc
    Integer(t_int),Intent(in)    :: oci_nr,degree
    Integer(t_int),Intent(in)    :: nrbmax,nramax
    Integer(t_int),Intent(in)    :: ioci0
    Integer(t_int),Intent(in)    :: nRa,nRb,nRmxa,nRmxb
    Integer(t_int),Intent(in)    :: lRa(*),lRb(*)
    !! dimension of oci-arrays
    Integer(t_int),Intent(in)    :: ocidim
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: fna (nrbmax,nRmxa)
    Real(t_real),Intent(in)      :: fkna(nrbmax,nRmxa)
    Real(t_real),Intent(in)      :: fnb (nrbmax,nRmxb)
    Real(t_real),Intent(in)      :: fknb(nrbmax,nRmxb)
    Real(t_real),Intent(in)      :: vatom(nramax)
    !! output
    Real(t_real),Intent(inout)   :: oci_nn0 (ocidim)
    Real(t_real),Intent(inout)   :: oci_nHn0(ocidim)
    !!
    !!LOCAL
    Integer(t_int) :: ir,ia,ib,ioci,nr
    Integer(t_int) :: la,ma,mb,lb
    Real(t_real)   :: fa,fb,fka,fkb
    Real(t_real)   :: rp,we,xp,vloc,texp
    Real(t_real)   :: ffa(nRa),ffb(nRb),ffka(nRa),ffkb(nRb)
    Real(t_real),Pointer :: p_xx(:),p_rr(:),p_ww(:)
    !!
    !!
    !!
    !! loop over xp-integration mesh 
    !!
    Call prep_oci_int_mesh(oci_nr,rnuc,bmesh% rmax,nuc_type,p_xx,p_rr,p_ww,nr)
    !!
    !! for all points of integration mesh
    Do ir=1,nr
       xp=p_xx(ir)
       rp=p_rr(ir)
       we=p_ww(ir)
       !! find the values of potentials and functions
       !! for now take the atom potential as the local potential
       vloc=(minterpol(atommesh,degree,vatom,rp))/sq4pi/rp
       !! finite nucleus
       vloc=vloc-ZZ*nuc_zeta(rp/rnuc,nuc_type)/rp
       !!
       Do ia=1,nRa ! valence
          !! basis orbital
          texp=rp**fgam
          ffa(ia) = minterpol(bmesh,degree,fna (1_t_int,ia),rp)*texp
          ffka(ia)= minterpol(bmesh,degree,fkna(1_t_int,ia),rp)*texp/rp
       Enddo                 ! iv

       Do ib=1,nRb ! core
          !! basis orbital
          texp=rp**fgam
          ffb(ib) = minterpol(bmesh,degree,fnb (1_t_int,ib),rp)*texp
          ffkb(ib)= minterpol(bmesh,degree,fknb(1_t_int,ib),rp)*texp/rp
       Enddo                 ! iv
       
       
       !! end of potential calculation
       !!
       ioci=ioci0 !! iocivc
       !!
       Do ia=1,nRa
          la=lRa(ia)
          fa=ffa(ia)
          !! kinetic
          fka=(fa*vloc+ffka(ia))*0.5_t_real
          Do ma=1,2*la+1
             Do ib=1,nRb
                lb=lRb(ib)
                fb=ffb(ib)
                !! kinetic
                fkb=(fb*vloc+ffkb(ib))*0.5_t_real
                mbloop: Do mb=1,2*lb+1
                   ioci=ioci+1
                   
                   !! delta lm,l'm'
                   If( (la /= lb) .Or. (ma /=mb) ) Cycle mbloop
                   !!
                   !!
                   !! Hamilton (i|H|i)
                   oci_nHn0(ioci)=oci_nHn0(ioci)+(fka*fb+fa*fkb)*we
                   !! overlap (i|i)
                   oci_nn0(ioci)=oci_nn0(ioci)+fb*fa*we
                Enddo mbloop
             Enddo !! ib
          Enddo !! ma
       Enddo !! ia
    Enddo !! ir
    !!
    !!
    Call pfree(p_xx)
    Call pfree(p_rr)
    Call pfree(p_ww)
    !!
    Return
    !!
  End Subroutine nrel_calc_oci_vcs  
  !!***********************************************************************
  Subroutine rel_calc_oci_vcs(quant,vatom,oci_nr&
       &,atommesh,nramax,bmesh,nrbmax,degree&
       &,nRmxa,nRmxb,nRa,kapRa,degRa,nRb,kapRb,degRb&
       &,fna,fkna,fnb,fknb,fgam&
       &,ZZ,rnuc,nuc_type&
       &,ioci0,ocidim,oci_nn0,oci_nHn0&
       &,nsmxa,nsmxb,nbSn0,nbVSn0,need_sigma,isa)
    !!
    Implicit None
    !!IO
    Type(T_Quant),Intent(in)     :: quant
    Type(T_Mesh),Intent(in)      :: atommesh
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: nuc_type,isa
    Logical(t_log),Intent(in)    :: need_sigma
    Real(t_real),Intent(in)      :: ZZ,rnuc
    Integer(t_int),Intent(in)    :: oci_nr,degree,nsmxa,nsmxb
    Integer(t_int),Intent(in)    :: ioci0
    Integer(t_int),Intent(in)    :: nrbmax,nramax
    Integer(t_int),Intent(in)    :: nRmxa,nRmxb
    Integer(t_int),Intent(in)    :: nRa,kapRa(nRmxa),degRa(nRmxa)
    Integer(t_int),Intent(in)    :: nRb,kapRb(nRmxb),degRb(nRmxb)
    !! dimension of oci-arrays
    Integer(t_int),Intent(in)    :: ocidim
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: fna (nrbmax,nRmxa,2)
    Real(t_real),Intent(in)      :: fkna(nrbmax,nRmxa,2)
    Real(t_real),Intent(in)      :: fnb (nrbmax,nRmxb,2)
    Real(t_real),Intent(in)      :: fknb(nrbmax,nRmxb,2)
    Real(t_real),Intent(in)      :: vatom(nramax)
    !! output
    Real(t_real),Intent(inout)   :: oci_nn0 (ocidim)
    Real(t_real),Intent(inout)   :: oci_nHn0(ocidim)
    Complex(t_real),Intent(inout):: nbSn0(nsmxa,nsmxb)
    Complex(t_real),Intent(inout):: nbVSn0(:,:,:,:)
    !!
    !!LOCAL
    Integer(t_int) :: ir,ia,ib,ioci,icomp,nr,ks
    Integer(t_int) :: ma,mb,kapa,kapb,mua,mub,ja,jb,i,j
    Real(t_real)   :: fa(2),fb(2),fka(2),fkb(2)
    Complex(t_real):: alpha,alphaq
    Real(t_real)   :: vloc
    Real(t_real)   :: rp,we,xp,texp
    Real(t_real)   :: ffa(nRa,2),ffka(nRa,2)
    Real(t_real)   :: ffb(nRb,2),ffkb(nRb,2)
    Real(t_real),Pointer :: p_xx(:),p_rr(:),p_ww(:)
    !!
    !!
    !!
    !! loop over xp-integration mesh 
    !!
    Call prep_oci_int_mesh(oci_nr,rnuc,bmesh% rmax,nuc_type,p_xx,p_rr,p_ww,nr)
    !!
    !!
    !!
    !! for all points of integration mesh
    Do ir=1,nr
       xp=p_xx(ir)
       rp=p_rr(ir)
       we=p_ww(ir)
       !! find the values of potentials and functions
       !! for now take the atom potential as the local potential
       vloc=(minterpol(atommesh,degree,vatom,rp))/sq4pi/rp
       !! finite nucleus
       vloc=vloc-ZZ*nuc_zeta(rp/rnuc,nuc_type)/rp
       !!
       Do ia=1,nRa
          !! basis orbital
          texp=rp**fgam
          Do icomp=1,2
             ffa(ia,icomp)= minterpol(bmesh,degree&
                  &,fna (1_t_int,ia,icomp),rp)*texp
             ffka(ia,icomp)=minterpol(bmesh,degree&
                  &,fkna(1_t_int,ia,icomp),rp)*texp/rp
          Enddo
       Enddo                 ! iv
       Do ib=1,nRb
          !! basis orbital
          texp=rp**fgam
          Do icomp=1,2
             ffb(ib,icomp)= minterpol(bmesh,degree&
                  &,fnb (1_t_int,ib,icomp),rp)*texp
             ffkb(ib,icomp)=minterpol(bmesh,degree&
                  &,fknb(1_t_int,ib,icomp),rp)*texp/rp
          Enddo
       Enddo                 ! iv
       
       
       !! end of potential calculation
       !!
       ioci=ioci0
       !!
       i=0
       Do ia=1_t_int,nRa
          
          kapa=kapRa(ia)
          ja=2*Abs(kapa)-1
          
          fa(1:2)=ffa(ia,1:2)
          !! kinetic
          fka(1:2)=(fa(1:2)*vloc+ffka(ia,1:2))*0.5_t_real
          !!
          Do ma=1,degRa(ia)
             mua=-ja+2*(ma-1)
             i=i+1
             j=0
             Do ib=1_t_int,nRb
                kapb=kapRb(ib)
                jb=2*Abs(kapb)-1
                
                fb(1:2)=ffb(ib,1:2)
                !! second kinetic term, for symmetric treatment
                fkb(1:2)=(fb(1:2)*vloc+ffkb(ib,1:2))*0.5_t_real
                !!
                mbloop: Do mb=1,degRb(ib)
                   mub=-jb+2*(mb-1)

                   alpha=quant% p_chiSigmaRchi(kapa,ma,kapb,mb)
                   alphaq=quant% p_chiSigmaRchi(-kapa,ma,-kapb,mb)

                   j=j+1
                   ioci=ioci+1_t_int
                   
                   !! delta kap,mu,kap'mu'
                   If(kapa == kapb .And. mua == mub) Then
                      !!
                      !!
                      !! Hamilton (i|H|i)
                      oci_nHn0(ioci)=oci_nHn0(ioci)&
                           &+(Dot_product(fka,fb)&
                           &+Dot_product(fa,fkb))*we
                      !! overlap (i|i)
                      oci_nn0(ioci)=oci_nn0(ioci)+Dot_product(fa,fb)*we
                   Endif
                   !!
                   !! (i|beta sigma_Z |i)
                   nbSn0(i,j)=nbSn0(i,j)+alpha*fa(1)*fb(1)*we 
                   !! (i|beta sigma_Z |i)
                   nbSn0(i,j)=nbSn0(i,j)-alphaq*fa(2)*fb(2)*we
                   !!
                   !!
                   If(need_sigma)Then
                      Do ks=1,3
                         alpha=quant% p_chiVSigmachi(ks,kapa,ma,kapb,mb)
                         alphaq=quant% p_chiVSigmachi(ks,-kapa,ma,-kapb,mb)
                         !! (i|beta sigma_Z |i)
                         nbVSn0(i,j,isa,ks)=nbVSn0(i,j,isa,ks)+(alpha*fa(1)*fb(1) &
                              &-alphaq*fa(2)*fb(2))*we
                      Enddo
                   Endif
                   !!
                Enddo mbloop
             Enddo               ! ib
          Enddo                 ! ma
       Enddo                   ! ia
    Enddo                     ! ir
    !!
    !!
    Call pfree(p_xx)
    Call pfree(p_rr)
    Call pfree(p_ww)
    !!
    !!
    Return
    !!
  End Subroutine rel_calc_oci_vcs
  !!***********************************************************************
  Subroutine calc_tcis(&
       !& nsite,&
       &isa,isb&
       &,npshell,pairs,pairabs,ninpshell &
       &,atommesha,atommeshb,bmesha,bmeshb,nrbmax&
       &,vata,vatb&
       &,rffmax&
       &,nva,nvb,ncb,nvm,ncm&
       &,hzlmx,lmx,lva,lvb,lcb&
       &,fva,fvb,fkva,fkvb,fcb,fkcb,fgama,fgamb&
       &,ZZa,ZZb,rnuca,rnucb,nuc_type&
       &,degree&
       &,faclmx,fac,wu&
       &,tci_vHv0&
       &,tci_vv0&
       &,tci_vHc0&
       &,tci_vc0&
       &,tcivvindex,tcivcindex,prot)
    !!
    Implicit  None
    !!IO
    Type(T_Mesh),Intent(in)     :: atommesha,atommeshb
    Type(T_Mesh),Intent(in)     :: bmesha,bmeshb
    Logical(t_log),Intent(in)   :: prot
    Integer(t_int),Intent(in)   :: nrbmax
    Integer(t_int),Intent(in)   :: nuc_type
    Real(t_real),Intent(in)     :: ZZa,ZZb,rnuca,rnucb
    !Integer(t_int),Intent(in)   :: relativistic
    Integer(t_int),Intent(in)   :: degree
    !Integer(t_int),Intent(in)   :: nsite,isa,isb
    Integer(t_int),Intent(in)   :: isa,isb
    Integer(t_int),Intent(in)   :: npshell,ninpshell(:)
    Integer(t_int),Intent(in)   :: nva,nvb,ncb,nvm,ncm
    Integer(t_int),Intent(in)   :: lva(:),lvb(nvm),lcb(ncm)
    Integer(t_int),Intent(in)   :: tcivvindex(:)
    Integer(t_int),Intent(in)   :: tcivcindex(:)
    Integer(t_int),Intent(in)   :: hzlmx,lmx,faclmx
    Real(t_real),Intent(in)     :: fac(0:2*faclmx+1)
    Real(t_real),Intent(inout)  :: wu(*)
    Real(t_real),Intent(in)     :: pairs(:,:)
    Real(t_real),Intent(in)     :: pairabs(:)
    Real(t_real),Intent(in)     :: rffmax
    Real(t_real),Intent(in)     :: fva (:,:)
    Real(t_real),Intent(in)     :: fkva(:,:)
    Real(t_real),Intent(in)     :: fvb (nrbmax,nvm)
    Real(t_real),Intent(in)     :: fkvb(nrbmax,nvm)
    Real(t_real),Intent(in)     :: fcb (nrbmax,ncm)
    Real(t_real),Intent(in)     :: fkcb(nrbmax,ncm)
    Real(t_real),Intent(in)     :: fgama,fgamb
    Real(t_real),Intent(in)     :: vata(:),vatb(:)
    Real(t_real),Intent(inout)  :: tci_vHv0(:)            !! (i|H|j)
    Real(t_real),Intent(inout)  :: tci_vv0(:)             !! (i|j)
    Real(t_real),Intent(inout)  :: tci_vHc0(:)            !! (i|H|c)
    Real(t_real),Intent(inout)  :: tci_vc0(:)             !! (i|c)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="twocenterint"
    Logical(t_log) :: test!,rel
    Integer(t_int) :: n,ish,nf,ne,i,lb
    Integer(t_int) :: ir,n1
    Integer(t_int) :: itci,itcivc
    Real(t_real)  :: s_ij_max,s_iHj_max,s_ic_max,s_iHc_max
    Real(t_real)  :: rbig
    Real(t_real)  :: pp(3),eu(3)
    !!automatic arrays
    Real(t_real)  :: fap(nva),fbp(nvb),fkap(nva),fkbp(nvb),fcp(ncb),fkcp(ncb)
    Real(t_real)  :: fam(nva),fbm(nvb),fkam(nva),fkbm(nvb),fcm(ncb),fkcm(ncb)
    Real(t_real)  :: hza3(0:hzlmx,0:hzlmx) 
    Real(t_real)  :: hzb3(0:hzlmx,0:hzlmx) 
    !!WORK
    !!
    !! GKP
    Type(T_Quadgkp) :: gkpxi,gkpeta
    Logical(t_log) :: calcxi,calceta
    Integer(t_int) :: nvmax_gk,ncmax_gk,ndim_gk
    Integer(t_int) :: n_ij,n_iHj,n_ic,n_iHc
    Integer(t_int) :: count,countxi,counteta,mcountxi
    Real(t_real)   :: epsabs,epsrel,r1,b1_eta,etah,xi,eta,xijac,etajac,jac
    !!
    !rel=(relativistic==FULL_RELATIVISTIC)
    !!
    Call init_quadgkp(gkpxi)
    Call init_quadgkp(gkpeta)
    !!
    !epsabs=1.e-8_t_real
    epsabs=1.e-6_t_real
    epsrel=1.e-6_t_real
    !!
    !!
    nvmax_gk=0
    Do i=1,nva
       nvmax_gk=nvmax_gk+Sum(Min(lva(i),lvb(1:nvb))+1_t_int)
    Enddo
    ncmax_gk=0
    Do i=1,nva
       ncmax_gk=ncmax_gk+Sum(Min(lva(i),lcb(1:ncb))+1_t_int)
    Enddo
    !!
    n_ij=0_t_int
    n_iHj=nvmax_gk
    n_ic=nvmax_gk*2
    n_iHc=nvmax_gk*2+ncmax_gk
    ndim_gk=nvmax_gk*2+ncmax_gk*2
    !!
    If(prot.And.outlevel >= L_many)Then
       Write(outfile,"('nvmax_gk =',i5)")nvmax_gk
       Write(outfile,"('ncmax_gk =',i5)")ncmax_gk
       Write(outfile,"('ndim_gk  =',i5)")ndim_gk
       Write(outfile,"('n_ij     =',i5)")n_ij
       Write(outfile,"('n_iHj    =',i5)")n_iHj
       Write(outfile,"('n_ic     =',i5)")n_ic
       Write(outfile,"('n_iHc    =',i5)")n_iHc
    Endif
    !!
    Call alloc_quadgkp(gkpxi,ndim_gk)
    Call alloc_quadgkp(gkpeta,ndim_gk)
    !!


    !!preparations
    test=isoption('PROT_TCI')
    !!
    !!     
    
    
    s_ij_max=0.0_t_real
    s_iHj_max=0.0_t_real
    s_ic_max=0.0_t_real
    s_iHc_max=0.0_t_real
    
    
    n=0_t_int
    shellloop: Do ish=1_t_int,npshell
       nf=n+1_t_int
       ne=n+ninpshell(ish)
       n=ne
       s_ij_max=0.0_t_real
       s_iHj_max=0.0_t_real
       s_ic_max=0.0_t_real
       s_iHc_max=0.0_t_real

       rbig=pairabs(ish)*0.5_t_real
       
       
       !!gkpeta% p_res(1:ndim_gk)=0.0_t_real       
       
       
       
       
       !!
       !Print *,'rbig=',rbig*2.0_t_real
       !!
       r1=Min(Max(bmesha% rmax,bmeshb% rmax),rffmax)/rbig-1.0_t_real
       !!
       gkpxi% typ=IJAC_TYPE_POW
       gkpxi% exx =2.0_t_real
       gkpxi% kmin=3_t_int
       gkpxi% epsabs=epsabs
       gkpxi% epsrel=epsrel
       gkpeta% typ=IJAC_TYPE_POW
       gkpeta% exx  =2.0_t_real
       gkpeta% kmin=3_t_int
       gkpeta% epsabs=epsabs
       gkpeta% epsrel=epsrel
       !!
       !!
       !Print *,'epsabs=',epsabs
       !Print *,'epsrel=',epsrel
       !Print *,'kmin   =',kmin
       !Print *,'typ_xi=',typxi
       !If(typxi == IJAC_TYPE_POW) Print *,'ex_xi =',exxi
       !Print *,'typ_eta =',typeta
       !If(typeta == IJAC_TYPE_POW) Print *,'ex_eta  =',exeta
       
       
       
       b1_eta= Min(r1,1.0_t_real)
       
       gkpeta% b0=0.0_t_real
       gkpeta% b1=b1_eta
       
       If(b1_eta <= 0.0_t_real)Exit shellloop
       
       
       mcountxi=0_t_int
       count=0_t_int
       counteta=0_t_int
       gkpeta% state=QUADSTART
       
       etaloop: Do 
          gkpeta% p_ff=gkpxi% p_res
          Call quad_gkp(gkpeta,etah,etajac,calceta)
          If(.Not.calceta) Exit etaloop
          
          
          eta=(etah-b1_eta)
          
          
          gkpxi% b0=0.0_t_real
          gkpxi% b1=(r1+eta)
          
          countxi=0_t_int
          gkpxi% state=QUADSTART
          
          xiloop: Do 
             
             Call quad_gkp(gkpxi,xi,xijac,calcxi)
             If(.Not.calcxi) Exit xiloop
             
             xi=xi+1.0_t_real
             
             jac=xijac*etajac*rbig
             
             Call tci_kernq(xi,eta,rbig,jac&
                  &,atommesha,atommeshb,bmesha,bmeshb,nrbmax&
                  &,nvm,ncm,nva,nvb,ncb,lva,lvb,lcb&
                  &,fva,fvb,fkva,fkvb,fcb,fkcb,fgama,fgamb&
                  &,ZZa,ZZb,rnuca,rnucb,nuc_type&
                  &,degree,vata,vatb&
                  &,fap,fkap,fbp,fkbp,fcp,fkcp&
                  &,fam,fkam,fbm,fkbm,fcm,fkcm&
                  &,hzlmx,hza3,hzb3&
                  &,gkpxi% p_ff,gkpxi% ndim,n_ij,n_iHj,n_ic,n_iHc)
             
             
             
             
             !!
             countxi=countxi+1
             count=count+2
          Enddo xiloop
          !If(.Not.gkpxi% conv) Print "('xi not converged!')"
          
          counteta=counteta+1
          mcountxi=Max(mcountxi,countxi)
          !Print *,'countxi=',countxi
       End Do etaloop
       !If(.Not.gkpeta% conv) Print "('eta not converged!')"
       !!
       !Print *,'mcountxi=',mcountxi
       !Print *,'counteta =',counteta
       !Print *,'count  =',count

       



       !!Rotation
       Do ir=nf,ne

          pp(1:3)=pairs(1:3,ir)*0.5_t_real
          Call reuler(pp,eu)
          Do lb=0_t_int,lmx
             n1=lb*(lb*lb*4_t_int-1_t_int)/3_t_int
             Call ytr(eu,0_t_int,fac,lb,wu(n1+1_t_int),.False.)
          Enddo
          itci=tcivvindex(ir)
          itcivc=tcivcindex(ir)


          !! rotate all SKIs
          Call rotski(nva,nvb,ncb,lva,lvb,lcb,wu,itci,itcivc,tci_vHv0 &
               &,tci_vv0,tci_vHc0,tci_vc0 &
               &,s_iHj_max,s_ij_max,s_ic_max,s_iHc_max&
               &,gkpeta% p_res,ndim_gk,n_ij,n_iHj,n_ic,n_iHc)

       Enddo                   ! ir
       !!
       !!
       If(test)Then
          Write(outfile,134)isa,isb
          Write(outfile,133)ish,rbig
          Write(outfile,13)ish,s_ij_max
          Write(outfile,14)ish,s_iHj_max
          Write(outfile,15)ish,s_ic_max
          Write(outfile,156)ish,s_iHc_max
       Endif


    Enddo shellloop
    !!
    Call clean_quadgkp(gkpxi)
    Call clean_quadgkp(gkpeta)
    !!
    !!     End of slater-Koster integrals      
    If (prot.And.outlevel >= L_many) Then
       Write(outfile,16)ish-1_t_int,s_ij_max
       Write(outfile,17)ish-1_t_int,s_iHj_max
       Write(outfile,18)ish-1_t_int,s_ic_max
       Write(outfile,19)ish-1_t_int,s_iHc_max
    Endif
    Return
    !!
13  Format('TWO: max (v|v)    in shell ',i4,' : ',f20.9)
14  Format('TWO: max (v|H|v)  in shell ',i4,' : ',f20.9)
15  Format('TWO: max (v|c)    in shell ',i4,' : ',f20.9)
156 Format('TWO: max (v|H|c)  in shell ',i4,' : ',f20.9)
16  Format('TWO: max (v|v)    in last shell ',i4,' : ',f20.9)
17  Format('TWO: max (v|H|v)  in last shell ',i4,' : ',f20.9)
18  Format('TWO: max (v|c)    in last shell ',i4,' : ',f20.9)
19  Format('TWO: max (v|H|c)  in last shell ',i4,' : ',f20.9)
133 Format('rbig in shell ',i4,' : ',f15.6)
134 Format('isa  ',i4,' isb ',i4)
    !!
  End Subroutine calc_tcis
  !!***********************************************************************
  !! lmx must be 3 or 4
  !! 4 is the maximum orbital angular momentum implemented in the moment
  !! if more should be implemented, yhzy... must be enhanced
  Subroutine tci_kernq(xi,eta,rbig,jac&
       &,atommesha,atommeshb,bmesha,bmeshb,nrbmax&
       &,nvm,ncm,nva,nvb,ncb,lva,lvb,lcb&
       &,fva,fvb,fkva,fkvb,fcb,fkcb,fgama,fgamb&
       &,ZZa,ZZb,rnuca,rnucb,nuc_type&
       &,degree,vata,vatb&
       &,fap,fkap,fbp,fkbp,fcp,fkcp&
       &,fam,fkam,fbm,fkbm,fcm,fkcm&
       &,lmx,hza,hzb&
       &,wij,ndim_gk,n_ij,n_iHj,n_ic,n_iHc&
       &)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Intent(in)      :: atommesha,atommeshb
    Type(T_Mesh),Intent(in)      :: bmesha,bmeshb
    Integer(t_int),Intent(in)    :: nrbmax
    Integer(t_int),Intent(in)    :: nuc_type
    Real(t_real),Intent(in)      :: ZZa,ZZb,rnuca,rnucb
    Real(t_real),Intent(in)      :: xi,eta,rbig,jac
    Integer(t_int),Intent(in)    :: degree
    Integer(t_int),Intent(in)    :: nvm,ncm,nva,nvb,ncb,lmx
    Integer(t_int),Intent(in)    :: lva(:),lvb(nvm),lcb(ncm)
    Real(t_real),Intent(in)      :: fgama,fgamb
    Real(t_real),Intent(in)      :: fva (:,:),fvb (nrbmax,nvm)
    Real(t_real),Intent(in)      :: fkva(:,:),fkvb(nrbmax,nvm)
    Real(t_real),Intent(in)      :: fcb (nrbmax,ncm),fkcb(nrbmax,ncm)
    Real(t_real),Intent(in)      :: vata(:),vatb(:)
    !!AUX
    !! positive eta
    Real(t_real),Intent(inout)   :: fap(nva),fkap(nva)
    Real(t_real),Intent(inout)   :: fbp(nvb),fkbp(nvb)
    Real(t_real),Intent(inout)   :: fcp(ncb),fkcp(ncb)
    !! negative eta
    Real(t_real),Intent(inout)   :: fam(nva),fkam(nva)
    Real(t_real),Intent(inout)   :: fbm(nvb),fkbm(nvb)
    Real(t_real),Intent(inout)   :: fcm(ncb),fkcm(ncb)
    Real(t_real),Intent(inout)   :: hza(0:lmx,0:lmx) 
    Real(t_real),Intent(inout)   :: hzb(0:lmx,0:lmx) 
    !!OUT
    Integer(t_int),Intent(in)    :: ndim_gk
    Integer(t_int),Intent(in)    :: n_ij,n_iHj
    Integer(t_int),Intent(in)    :: n_ic,n_iHc
    Real(t_real),Intent(out)     :: wij(ndim_gk)
    !!LOCAL
    Character(*),Parameter::nsubfunc="tci_kernq"
    !!
    Integer(t_int) :: i,j,m,m2,la,lb,ic,ia
    Real(t_real)   :: ccp,ccm,a,b,ha,hb,texp
    Real(t_real)   :: ra,rb,za,zb,ya,yb,zs,fk
    Real(t_real)   :: fcoefaa(0:degree),fcoefba(0:degree)
    Real(t_real)   :: fcoefab(0:degree),fcoefbb(0:degree)
    !! positive eta
    Real(t_real)   :: vap,vbp
    !! negative eta
    !! eta->-eta
    !! ra->rb, za->-zb, ya->yb
    !! rb->ra, zb->-za, yb->ya
    Real(t_real)   :: vam,vbm
    !! interpolation 
    Integer(t_int) :: findexaa,findexab,findexbb,findexba,index
    !!
    !wij(1:ndim_gk)=0.0_t_real    
    !!
    ra=(xi-eta)
    rb=(xi+eta)
    zs=eta*xi
    za= (zs-1.0_t_real)/ra
    zb= (zs+1.0_t_real)/rb
    ya=Sqrt(Abs(1.0_t_real-za*za))
    yb=Sqrt(Abs(1.0_t_real-zb*zb))
    ra=ra*rbig
    rb=rb*rbig
    !!
    If(lmx==3)Then
       Call yhzy3(hza,za,ya)
       Call yhzy3(hzb,zb,yb)
    Else If(lmx==4)Then
       Call yhzy4(hza,za,ya)
       Call yhzy4(hzb,zb,yb)
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'lmx has invalid value (3 or 4 allowed)!'")
    Endif
    !!
    !! use atom potential for now oto compensate the large L=0 terms
    !!
    !! potential at A and B
    index=mintpl_coefs(degree,atommesha,ra,fcoefaa)
    vap=(Dot_product(vata(index:index+degree)&
         &,fcoefaa(0:degree))/sq4pi)/ra
    vap=vap-ZZa*nuc_zeta(ra/rnuca,nuc_type)/ra

    index=mintpl_coefs(degree,atommeshb,ra,fcoefba)
    vbm=(Dot_product(vatb(index:index+degree)&
         &,fcoefba(0:degree))/sq4pi)/ra
    vbm=vbm-ZZb*nuc_zeta(ra/rnucb,nuc_type)/ra
    
    index=mintpl_coefs(degree,atommeshb,rb,fcoefbb)    
    vbp=(Dot_product(vatb(index:index+degree)&
         &,fcoefbb(0:degree))/sq4pi)/rb
    vbp=vbp-ZZb*nuc_zeta(rb/rnucb,nuc_type)/rb
    
    index=mintpl_coefs(degree,atommesha,rb,fcoefab)    
    vam=(Dot_product(vata(index:index+degree)&
         &,fcoefab(0:degree))/sq4pi)/rb
    vam=vam-ZZa*nuc_zeta(rb/rnuca,nuc_type)/rb

    


    !! - interpolation of function fa and tfa at the point ra
    
    findexaa=mintpl_coefs(degree,bmesha,ra,fcoefaa)
    findexab=mintpl_coefs(degree,bmesha,rb,fcoefab)
    findexbb=mintpl_coefs(degree,bmeshb,rb,fcoefbb)
    findexba=mintpl_coefs(degree,bmeshb,ra,fcoefba)


    Do i=1_t_int,nva 
       texp=ra**fgama
       fap(i)=Dot_product(fva(findexaa:findexaa+degree,i),fcoefaa(0:degree))&
            &*texp*jac
       fk=Dot_product(fkva(findexaa:findexaa+degree,i),fcoefaa(0:degree))&
            &*texp*jac*0.5_t_real/ra
       !!
       fkap(i)=vap*fap(i)+fk
       !!eta->-eta
       texp=rb**fgama
       la=(-1)**lva(i)
       fam(i)=Dot_product(fva(findexab:findexab+degree,i),fcoefab(0:degree))&
            &*texp*la*jac
       fk=Dot_product(fkva(findexab:findexab+degree,i),fcoefab(0:degree))&
            &*texp*la*jac*0.5_t_real/rb
       fkam(i)=vam*fam(i)+fk
    Enddo               ! i
    !!           - interpolation of function fb at the point rb
    Do i=1_t_int, nvb
       texp=rb**fgamb
       fbp(i)=Dot_product(fvb(findexbb:findexbb+degree,i),fcoefbb(0:degree))*texp
       fk=Dot_product(fkvb(findexbb:findexbb+degree,i),fcoefbb(0:degree))*texp/rb
       !!
       fkbp(i)=vbp*fbp(i)+fk*0.5_t_real
       !!eta->-eta
       texp=ra**fgamb
       lb=(-1)**lvb(i)
       fbm(i)=Dot_product(fvb(findexba:findexba+degree,i),fcoefba(0:degree))&
            &*texp*lb
       fk=Dot_product(fkvb(findexba:findexba+degree,i),fcoefba(0:degree))&
            &*texp*lb/ra
       !!
       fkbm(i)=vbm*fbm(i)+fk*0.5_t_real
    Enddo
    !! core function at B
    Do i=1_t_int,ncb
       texp=rb**fgamb
       fcp(i)=Dot_product(fcb(findexbb:findexbb+degree,i),fcoefbb(0:degree))*texp
       fk=Dot_product(fkcb(findexbb:findexbb+degree,i),fcoefbb(0:degree))*texp/rb
       !!
       fkcp(i)=vbp*fcp(i)+fk*0.5_t_real
       !!eta->-eta
       texp=ra**fgamb
       lb=(-1)**lcb(i)
       fcm(i)=Dot_product(fcb(findexba:findexba+degree,i),fcoefba(0:degree))&
            &*texp*lb
       fk=Dot_product(fkcb(findexba:findexba+degree,i),fcoefba(0:degree))&
            &*texp*lb/ra
       !!
       fkcm(i)=vbm*fcm(i)+fk*0.5_t_real
    Enddo
    
    !! Y*Y*radial A * radial B

    ic=0_t_int
    ia=0_t_int
    !! 
    Do i=1_t_int,nva
       la=lva(i)
       Do j=1_t_int,nvb
          lb=lvb(j)
          a=fap(i)*fbp(j)
          b=fam(i)*fbm(j)
          ha=(fkap(i)*fbp(j)+fap(i)*fkbp(j))
          hb=(fkam(i)*fbm(j)+fam(i)*fkbm(j))
          m2=Min(la,lb)
          Do m=0_t_int,m2
             ia=ia+1_t_int
             ccp=hza(m,la)*hzb(m,lb)
             ccm=hzb(m,la)*hza(m,lb)
             !!  (a|b) 
             wij(ia+n_ij)=a*ccp+b*ccm
             !!  
             wij(ia+n_iHj)=ha*ccp+hb*ccm
          Enddo           !m
       Enddo             ! j
       Do j=1_t_int,ncb
          lb=lcb(j)
          a=fap(i)*fcp(j)
          b=fam(i)*fcm(j)
          ha=(fkap(i)*fcp(j)+fap(i)*fkcp(j))
          hb=(fkam(i)*fcm(j)+fam(i)*fkcm(j))
          m2=Min(la,lb)
          Do m=0_t_int,m2
             ic=ic+1
             ccp=hza(m,la)*hzb(m,lb)
             ccm=hzb(m,la)*hza(m,lb)
             !!  (a|b) 
             wij(ic+n_ic)=a*ccp+b*ccm
             !!  
             wij(ic+n_iHc)=ha*ccp+hb*ccm
          Enddo           !m
       Enddo             ! j
    Enddo               ! i
    !!
  End Subroutine tci_kernq
  !!***********************************************************************
  Subroutine rotski(nva,nvb,ncb,lva,lvb,lcb,wu ,itci,itcivc,tci_vHv0&
       &,tci_vv0,tci_vHc0,tci_vc0&
       &,s_iHj_max,s_ij_max,s_ic_max,s_iHc_max &
       &,wij,ndim_gk ,n_ij,n_iHj,n_ic,n_iHc)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)   :: nva,nvb,ncb,lva(*),lvb(*),lcb(*)
    Real(t_real),Intent(in)     :: wu(*)
    Integer(t_int),Intent(inout) :: itci,itcivc
    Real(t_real),Intent(inout)  :: s_iHj_max,s_ij_max,s_ic_max,s_iHc_max
    Real(t_real),Intent(inout)  :: tci_vHv0(*),tci_vv0(*)
    Real(t_real),Intent(inout)  :: tci_vHc0(*),tci_vc0(*)
    !!
    !OUT
    Integer(t_int),Intent(in)    :: ndim_gk
    Integer(t_int),Intent(in)    :: n_ij,n_iHj
    Integer(t_int),Intent(in)    :: n_ic,n_iHc
    Real(t_real),Intent(in)      :: wij(ndim_gk)
    !!LOCAL
    Integer(t_int) :: i,la,n1,ma,j,lb,n2,mb,m2,m,inda,indb
    Integer(t_int) :: ic2,ic1,ic,ia2,ia1,ia
    Real(t_real)  :: aa1,aa2,cc
    !!
    ia2=0_t_int
    Do i=1_t_int, nva
       la=lva(i)
       n1=la*(la*la*4_t_int-1_t_int)/3_t_int+la+1_t_int
       Do ma=1_t_int,la+la+1_t_int
          inda=(2_t_int*la+1_t_int)*(ma-1_t_int)+n1
          ia1=ia2
          Do j=1_t_int, nvb
             lb=lvb(j)
             n2=lb*(lb*lb*4_t_int-1_t_int)/3_t_int+lb+1_t_int
             m2=Min(la,lb)
             Do mb=1_t_int,lb+lb+1_t_int
                indb=(2_t_int*lb+1_t_int)*(mb-1_t_int)+n2
                aa1=0.0_t_real
                aa2=0.0_t_real
                !!m=0
                ia=ia1+1_t_int
                cc=wu(inda)*wu(indb)
                aa1=aa1+wij(ia+n_ij )*cc
                aa2=aa2+wij(ia+n_iHj)*cc
                
                Do m=1,m2
                   ia=ia+1_t_int
                   cc=wu(inda-m)*wu(indb-m)+wu(inda+m)*wu(indb+m)
                   aa1=aa1+wij(ia+n_ij )*cc
                   aa2=aa2+wij(ia+n_iHj)*cc
                Enddo
                s_iHj_max=Max(s_iHj_max,Abs(aa2))
                s_ij_max=Max(s_ij_max,Abs(aa1))

                itci=itci+1_t_int
                tci_vHv0(itci)=tci_vHv0(itci)+aa2 
                tci_vv0 (itci)=tci_vv0 (itci)+aa1
             Enddo ! mb
             ia1=ia1+m2+1
          Enddo ! j
          !!
       Enddo ! ma
       ia2=ia2+Sum(Min(la,lvb(1:nvb)))+nvb
    Enddo ! i
    !!     
    !!core-valence-overlap            
    ic2=0_t_int
    Do i=1_t_int, nva
       la=lva(i)
       n1=la*(la*la*4_t_int-1_t_int)/3_t_int+la+1_t_int
       Do ma=1_t_int, la+la+1_t_int
          inda=(2_t_int*la+1_t_int)*(ma-1_t_int)+n1
          ic1=ic2
          Do j=1_t_int, ncb
             lb=lcb(j)
             n2=lb*(lb*lb*4_t_int-1_t_int)/3_t_int+lb+1_t_int
             m2=Min(la,lb)
             Do mb=1_t_int, lb+lb+1_t_int
                indb=(2_t_int*lb+1_t_int)*(mb-1_t_int)+n2
                aa1=0.0_t_real
                aa2=0.0_t_real
                !!m=0
                ic=ic1+1_t_int
                cc=wu(inda)*wu(indb)
                aa1=aa1+wij(ic+n_ic)*cc
                aa2=aa2+wij(ic+n_iHc)*cc
                Do m=1,m2
                   ic=ic+1_t_int
                   cc=(wu(inda-m)*wu(indb-m)+wu(inda+m)*wu(indb+m))
                   aa1=aa1+wij(ic+n_ic)*cc
                   aa2=aa2+wij(ic+n_iHc)*cc
                Enddo
                s_iHc_max=Max(s_iHc_max,Abs(aa2))
                s_ic_max=Max(s_ic_max,Abs(aa1))
                itcivc=itcivc+1_t_int
                tci_vHc0(itcivc)=tci_vHc0(itcivc)+aa2
                tci_vc0(itcivc)=tci_vc0(itcivc)+aa1
             Enddo           ! mb
             ic1=ic1+m2+1
          Enddo             ! j 
       Enddo               ! ma
       ic2=ic2+(Sum(Min(la,lcb(1:ncb)))+ncb)
    Enddo                 ! i
    
  End Subroutine rotski
  !!***********************************************************************
  Subroutine reltrans_loi_block0(nRva,kapRva,nRcb,kapRcb&
       &,PNRindex,PNRloi,Rloi,nvsmx,ncsmx,lena,lenb,lmx,TT,sfac)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nRva,nRcb,kapRva(*),kapRcb(*)
    Integer(t_int),Intent(in)    :: nvsmx,ncsmx
    Integer(t_int),Intent(in)    :: PNRindex
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: lmx
    Complex(t_real),Intent(in)   :: TT(2*lmx+3,2*lmx+2,2,-lmx-1:lmx+1)
    Real(t_real),Intent(in)      :: PNRloi(:)
    Complex(t_real),Intent(out)  :: Rloi(:)
    Real(t_real),Intent(in)      :: sfac
    !!LOCAL
    Integer(t_int) :: ii,deglb,degjb,jj,degla,degja,ia,icb,kapb,kapa,lb,la,ma
    Integer(t_int) :: iva,k
    Complex(t_real):: A(nvsmx,ncsmx,2)
    !!
    !!
    !!
    !! Oin,jm=sum_s ( G^{s+}  * O^s_{in,jm} * G^s )
    ii=PNRindex !! pseudo-nonrel index
    Do ia=1,lena !! lena is pseudo-nonrel 
       jj=0
       Do icb=1,nRcb  
          kapb=kapRcb(icb)
          lb=(Abs(2*kapb+1)-1)/2
          deglb=2*lb+1
          degjb=2*Abs(kapb)
          !!
          A(ia,jj+1:jj+degjb,1)=Matmul(PNRloi(ii+1:ii+deglb)&
               &             ,TT(1:deglb,1:degjb,1,kapb))
          A(ia,jj+1:jj+degjb,2)=Matmul(PNRloi(ii+1:ii+deglb)&
               &             ,TT(1:deglb,1:degjb,2,kapb))
          jj=jj+degjb
          ii=ii+deglb
       Enddo
    Enddo
    !!
    ii=PNRindex
    jj=0
    Do iva=1,nRva
       kapa=kapRva(iva)
       la=(Abs(2*kapa+1)-1)/2
       degla=2*la+1
       degja=2*Abs(kapa)
       
       Do ma=1,degja
          !!
          Do k=1,lenb
             ii=ii+1
             Rloi(ii)= &
                  & Dot_product(TT(1:degla,ma,1,kapa),A(jj+1:jj+degla,k,1))&
                  &+Dot_product(TT(1:degla,ma,2,kapa),A(jj+1:jj+degla,k,2))&
                  &*sfac
          Enddo
       Enddo
       jj=jj+degla
    Enddo
    !!
    
    !!
  End Subroutine reltrans_loi_block0
  !!***********************************************************************
  Subroutine reltrans_loi_blockS0(nRva,kapRva,nRcb,kapRcb&
       &,PNRindex,PNRloi,Rloi,nvsmx,ncsmx,lena,lenb,lmx,TT)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nRva,nRcb,kapRva(*),kapRcb(*)
    Integer(t_int),Intent(in)    :: nvsmx,ncsmx
    Integer(t_int),Intent(in)    :: PNRindex
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: lmx
    Complex(t_real),Intent(in)   :: TT(2*lmx+3,2*lmx+2,2,-lmx-1:lmx+1)
    Real(t_real),Intent(in)      :: PNRloi(:)
    Complex(t_real),Intent(out)  :: Rloi(:,:)
    !!LOCAL
    Integer(t_int) :: ii,deglb,degjb,jj,degla,degja,ia,icb,kapb,kapa,lb,la,ma
    Integer(t_int) :: iva,k,ks
    Complex(t_real):: A(nvsmx,ncsmx,2)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: cm1=(-1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Complex(t_real),Parameter:: cmi=(0.0_t_real,-1.0_t_real)
    Complex(t_real):: sigma(2,2,3)
    Data sigma(1,1:2,1)/c0, c1/
    Data sigma(2,1:2,1)/c1, c0/
    Data sigma(1,1:2,2)/c0, cmi/
    Data sigma(2,1:2,2)/ci, c0/
    Data sigma(1,1:2,3)/c1, c0/
    Data sigma(2,1:2,3)/c0, cm1/
    !!
    !!
    !!
    !! Oin,jm=sum_s ( G^{s+}  * O^s_{in,jm} * G^s )
    ii=PNRindex !! pseudo-nonrel index
    Do ia=1,lena !! lena is pseudo-nonrel 
       jj=0
       Do icb=1,nRcb  
          kapb=kapRcb(icb)
          lb=(Abs(2*kapb+1)-1)/2
          deglb=2*lb+1
          degjb=2*Abs(kapb)
          !!
          A(ia,jj+1:jj+degjb,1)=Matmul(PNRloi(ii+1:ii+deglb)&
               &             ,TT(1:deglb,1:degjb,1,kapb))
          A(ia,jj+1:jj+degjb,2)=Matmul(PNRloi(ii+1:ii+deglb)&
               &             ,TT(1:deglb,1:degjb,2,kapb))
          jj=jj+degjb
          ii=ii+deglb
       Enddo
    Enddo
    !!
    Do ks=1,3
       ii=PNRindex
       jj=0
       Do iva=1,nRva
          kapa=kapRva(iva)
          la=(Abs(2*kapa+1)-1)/2
          degla=2*la+1
          degja=2*Abs(kapa)
          
          Do ma=1,degja
             !!
             Do k=1,lenb
                ii=ii+1
                Rloi(ii,ks)= &
                     & Dot_product(TT(1:degla,ma,1,kapa),A(jj+1:jj+degla,k,1))&
                     &  *sigma(1,1,ks)&
                     &+Dot_product(TT(1:degla,ma,1,kapa),A(jj+1:jj+degla,k,2))&
                     &  *sigma(1,2,ks)&
                     &+Dot_product(TT(1:degla,ma,2,kapa),A(jj+1:jj+degla,k,1))&
                     &  *sigma(2,1,ks)&
                     &+Dot_product(TT(1:degla,ma,2,kapa),A(jj+1:jj+degla,k,2))&
                     &  *sigma(2,2,ks)
             Enddo
          Enddo
          jj=jj+degla
       Enddo
    Enddo
    !!
    !!
  End Subroutine reltrans_loi_blockS0
  !!***********************************************************************
End Module loi0
