Module loiaux
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  Use prnt
  Use finuc
  Use integrate1d
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="LOIAUX"  
  !!
  Public :: print_tci,print_oci,print_oci_cx,print_tci_cx,prep_thcis,vecdifn&
       &,prep_oci_int_mesh

  !!&,prntocivc,prntoci
  !!***********************************************************************
Contains
  !!***********************************************************************
  !! make auxillary arrays for the thci-integration
  !! p_fabl, p_isitebl,p_ipairbl and p_vloc has to be nullified 
  !! on calling and has to be freed
  !! after final usage
  Subroutine  prep_thcis(nsort,nsite,isrt,npairs,npairsmax&
       &,nm,nR,lR&
       &,lmx,blocklen&
       &,mxblsize,p_fabl,p_vloc,p_isitebl,p_ipairbl,p_gfabl)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nsort,nsite,isrt(nsite),npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: nm,nR(*),lR(nm,*)
    Integer(t_int),Intent(out)   :: mxblsize
    Integer(t_int),Intent(out)   :: lmx(nsort),blocklen(nsite)
    Integer(t_int),Pointer       :: p_isitebl(:)
    Integer(t_int),Pointer,Optional :: p_ipairbl(:)
    Real(t_real),Pointer         :: p_fabl(:)
    Real(t_real),Pointer,Optional :: p_gfabl(:,:)
    Real(t_real),Pointer         :: p_vloc(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_thcis"
    Integer(t_int) :: isa,issa,na,isp,inbl
    !!
    !! determine max valence-l and blocklenght
    !!
    lmx(1:nsort)=0_t_int
    blocklen(1:nsite)=0_t_int
    !!
    Do isa=1,nsite
       issa=isrt(isa)
       na=nR(issa)
       If(na /= 0_t_int)Then
          lmx(issa)=Maxval(lR(1:na,issa))
          blocklen(isa)=Sum(2*lR(1:na,issa)+1)
       End If
    Enddo
    !!
    !! determine maximal blocksize=max aux array length
    !!
    mxblsize=0_t_int
    Do isp=1,nsite
       inbl=0_t_int
       Do isa=1,nsite
          inbl=inbl+blocklen(isa)*(npairs(isa,isp)+1)
       Enddo
       If(outlevel >= L_many) Write(outfile,20) isp,inbl
       mxblsize=Max(mxblsize,inbl)
    Enddo
    If(outlevel >= L_many)Write(outfile,21) mxblsize
20  Format('isp=',i5,' inbl=',i7)
21  Format('mxblsize=',i7)
    !!
    Call palloc('p_fabl',p_fabl,1_t_int,mxblsize,nsubfunc,nmodule)
    If(Present(p_gfabl))Then
       Call palloc('p_gfabl',p_gfabl,1_t_int,3_t_int,1_t_int,mxblsize&
            &,nsubfunc,nmodule)
    Endif
    Call palloc('p_vloc',p_vloc,1_t_int,nsite*Max(1_t_int,npairsmax)&
         &,nsubfunc,nmodule)
    Call palloc('p_isitebl',p_isitebl,1_t_int,nsite*Max(1_t_int,npairsmax)&
         &,nsubfunc,nmodule)
    If(Present(p_ipairbl))Then
       Call palloc('p_ipairbl',p_ipairbl,1_t_int,nsite*Max(1_t_int,npairsmax)&
            &,nsubfunc,nmodule)
    Endif
    !!
    Return
    !!
  End Subroutine prep_thcis
  !!***********************************************************************
  Subroutine prep_oci_int_mesh(oci_nr,rnuc,rmax,nuc_type,p_xx,p_rr,p_ww,nr)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: oci_nr,nuc_type
    Real(t_real),Intent(in)      :: rnuc,rmax
    Integer(t_int),Intent(out)   :: nr
    Real(t_real),Pointer :: p_xx(:),p_rr(:),p_ww(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_oci_int_mesh"
    Integer(t_int) :: ir,n1
    Real(t_real)   :: al,alm1,x0,x1,dx,r0,r1,lam
    !! DONT FREE THESE AFTERWARDS !!
    Real(t_real),Pointer   :: p_ar(:),p_wtr(:)
    !!
    Nullify(p_xx)
    Nullify(p_rr)
    Nullify(p_ww)
    !!
    al=3.0_t_real
    alm1=al-1.0_t_real
    !!
    If(nuc_type == FINUC_NONE ) Then
       n1=0
       lam=0.0_t_real
    Else
       n1=oci_nr
       lam=2.0_t_real
    Endif
    !!
    nr=oci_nr+n1
    Call palloc('p_xx',p_xx,1_t_int,nr,nmodule,nsubfunc)
    Call palloc('p_rr',p_rr,1_t_int,nr,nmodule,nsubfunc)
    Call palloc('p_ww',p_ww,1_t_int,nr,nmodule,nsubfunc)
    !!
    If(n1 == 0 ) Goto 100
    !!
    Call  get_gauss(n1,p_ar,p_wtr)
    !!
    !!
    r0=0.0_t_real
    r1=rnuc*lam
    !!
    x0=r0**(1.0_t_real/al)
    x1=r1**(1.0_t_real/al)
    dx=(x1-x0)
    !!
    !!
    Do ir=1,n1
       p_xx(ir)=((p_ar(ir)+1.0_t_real)*0.5_t_real*dx+x0)
       p_rr(ir)=p_xx(ir)**al
       p_ww(ir)=p_wtr(ir)*0.5_t_real*dx*al*p_xx(ir)**alm1
    Enddo
    !!
100 Continue
    !!
    Call  get_gauss(oci_nr,p_ar,p_wtr)
    !!
    r0=rnuc*lam
    r1=rmax
    !!
    x0=r0**(1.0_t_real/al)
    x1=r1**(1.0_t_real/al)
    dx=(x1-x0)
    !!
    !!
    Do ir=1,oci_nr
       p_xx(n1+ir)=((p_ar(ir)+1.0_t_real)*0.5_t_real*dx+x0)
       p_rr(n1+ir)=p_xx(n1+ir)**al
       p_ww(n1+ir)=p_wtr(ir)*0.5_t_real*dx*al*p_xx(n1+ir)**alm1
    Enddo
    !!
    Return
    !!
  End Subroutine prep_oci_int_mesh
  !!***********************************************************************
  Subroutine vecdifn(nsite,isp,itcidif,npairs,npairsmax,pairs,tcivvindex)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,isp,npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: tcivvindex(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Integer(t_int),Intent(out)   :: itcidif(0:npairsmax,nsite&
         &,0:npairsmax,nsite)
    !!LOCAL
    Integer(t_int) :: isb,isa,irbp,irap,irab,m,itci,irb0,ira0
    Real(t_real)  :: rbp(3),aa(3)
    !!

    Do isb=1,nsite
       irb0=1
       If(isb==isp)Then
          irb0=0
       Endif
       Do irbp=irb0,npairs(isb,isp)
          If(irbp==0)Then
             rbp(:)=0.0_t_real
          Else
             rbp(:)=pairs(:,irbp,isb,isp)
          Endif
          Do isa=1,nsite
             ira0=1
             If(isa==isp)Then
                ira0=0
             Endif
             Do irap=ira0,npairs(isa,isp)
                If(irap==0)Then
                   aa(:)=-rbp(:)
                Else
                   aa(:)=pairs(:,irap,isa,isp)-rbp(:)
                Endif
                irab=0_t_int
                mloop:Do m=1_t_int,npairs(isa,isb)
                   If(All(Abs(pairs(:,m,isa,isb)-aa(:)) < 1.e-3_t_real)) Then
                      irab=m
                      Exit mloop
                   Endif
                Enddo mloop
                itci=0
                If(irab /= 0 ) Then
                   itci=tcivvindex(irab,isa,isb)+1
                Endif
                itcidif(irap,isa,irbp,isb)=itci
             Enddo
          Enddo
       Enddo
    Enddo
    Return
  End Subroutine vecdifn
  !!***********************************************************************
  
!!$  Subroutine  prntoci(txt,fmt,fmt0,oci,nR,lR,ioci0,tol)
!!$    !!
!!$    Implicit None  
!!$    !!IO
!!$    Character(*),Intent(in)      :: txt,fmt,fmt0
!!$    Integer(t_int),Intent(in)    :: nR,ioci0,lR(*)
!!$    Real(t_real),Intent(in)      :: tol
!!$    Real(t_real),Intent(in)      :: oci(*)
!!$    !!LOCAL
!!$    Integer(t_int) :: ioci,ia,la,ma,ib,lb,mb
!!$    Real(t_real)   :: d
!!$    !!
!!$    Write(outfile,2)Trim(txt)
!!$2   Format('prntoci: ',a)
!!$    !!
!!$    ioci=ioci0
!!$    Do ia=1,nR
!!$       la=lR(ia)
!!$       Do ma=1,2*la+1
!!$          Do ib=1,nR
!!$             lb=lR(ib)
!!$             Do mb=1,2*lb+1
!!$                ioci=ioci+1
!!$                d=oci(ioci)
!!$                If(Abs(d)<tol)Then
!!$                   Write(outfile,fmt0,advance='no') 
!!$                Else
!!$                   Write(outfile,fmt,advance='no') d
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          Write(outfile,*)
!!$       Enddo
!!$    Enddo
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine prntoci
!!$  !!***********************************************************************
!!$  Subroutine  prntocivc(txt,fmt,fmt0,oci,nRv,nRc,lRv,lRc,ioci0,tol)
!!$    !!
!!$    Implicit None  
!!$    !!IO
!!$    Character(*),Intent(in)      :: txt,fmt,fmt0
!!$    Integer(t_int),Intent(in)    :: nRv,nRc,ioci0,lRv(*),lRc(*)
!!$    Real(t_real),Intent(in)      :: tol
!!$    Real(t_real),Intent(in)      :: oci(*)
!!$    !!LOCAL
!!$    Integer(t_int) :: ioci,ia,la,ma,ib,lb,mb
!!$    Real(t_real)   :: d
!!$    !!
!!$    Write(outfile,2)Trim(txt)
!!$2   Format('prntoci: ',a)
!!$    !!
!!$    ioci=ioci0
!!$    Do ia=1,nRv
!!$       la=lRv(ia)
!!$       Do ma=1,2*la+1
!!$          Do ib=1,nRc
!!$             lb=lRc(ib)
!!$             Do mb=1,2*lb+1
!!$                ioci=ioci+1
!!$                d=oci(ioci)
!!$                If(Abs(d)<tol)Then
!!$                   Write(outfile,fmt0,advance='no') 
!!$                Else
!!$                   Write(outfile,fmt,advance='no') d
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          Write(outfile,*)
!!$       Enddo
!!$    Enddo
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine prntocivc
  !!***********************************************************************
  Subroutine print_oci_cx(txt,nsite,blocklengtha,blocklengthb,fmt,fmt0&
       &,oci,tol,ociindex,subblock)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(in)      :: fmt,fmt0,txt
    Integer(t_int),Intent(in)    :: nsite,blocklengtha(*),blocklengthb(*)
    Integer(t_int),Intent(in)    :: ociindex(*)
    Integer(t_int),Intent(in),Optional    :: subblock(4,*)
    Complex(t_real),Intent(in)   :: oci(:)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: bla,blb,ioci,isa
    !!
    Do isa=1,nsite
       bla=blocklengtha(isa)
       blb=blocklengthb(isa)
       !!
       ioci=ociindex(isa)
       Write(outfile,10) isa
       If(Present(subblock)) Then
          Call printmatsb('Re '//txt,fmt,fmt0,Real (oci(ioci+1:),t_real)&
               &,blb,bla,tol&
               &,subblock=(/subblock(3,isa),subblock(4,isa)&
               &,subblock(1,isa),subblock(2,isa)/)&
               &,trans=.True.)
       Else
          Call printmatsb('Re '//txt,fmt,fmt0,Real (oci(ioci+1:),t_real)&
               &,blb,bla,tol,trans=.True.)
       Endif
       Write(outfile,10) isa
       If(Present(subblock)) Then
          Call printmatsb('Im '//txt,fmt,fmt0,Aimag(oci(ioci+1:)       )&
               &,blb,bla,tol&
               &,subblock=(/subblock(3,isa),subblock(4,isa)&
               &,subblock(1,isa),subblock(2,isa)/)&
               &,trans=.True.)

       Else
          Call printmatsb('Im '//txt,fmt,fmt0,Aimag(oci(ioci+1:)       )&
               &,blb,bla,tol,trans=.True.)
       Endif
       !!
    Enddo
    !!
10  Format('site=',i5)
    !!
  End Subroutine print_oci_cx
  !!***********************************************************************
  Subroutine print_tci_cx(txt,nsite,blocklengtha,blocklengthb,fmt,fmt0,tci&
       &,tciindex,npairs,tol,subblock)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(in)      :: fmt,fmt0,txt
    Integer(t_int),Intent(in)    :: nsite,blocklengtha(*),blocklengthb(*)
    Complex(t_real),Intent(in)   :: tci(:)
    Integer(t_int),Intent(in)    :: npairs(:,:)
    Integer(t_int),Intent(in)    :: tciindex(:,:,:)
    Integer(t_int),Intent(in),Optional    :: subblock(4,*)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Integer(t_int) :: na,nb,itci,isa,isb,irp
    !!
    Do isa=1,nsite
       na=blocklengtha(isa)
       Do isb=1,nsite
          nb=blocklengthb(isb)
          !!
          Do irp=1,npairs(isa,isb)

             itci=tciindex(irp,isa,isb)
             Write(outfile,10) isa,isb,irp
             If(Present(subblock)) Then
                Call printmatsb('Re '//txt,fmt,fmt0,Real(tci(itci+1:),t_real)&
                     &,nb,na,tol&
                     &,subblock=(/subblock(3,isb),subblock(4,isb)&
                     &,subblock(1,isa),subblock(2,isa)/)&
                     &,trans=.True.)
                
             Else
                Call printmatsb('Re '//txt,fmt,fmt0,Real(tci(itci+1:),t_real)&
                     &,nb,na,tol,trans=.True.)
             Endif
             Write(outfile,10) isa,isb,irp
             If(Present(subblock)) Then
                Call printmatsb('Im '//txt,fmt,fmt0,Aimag(tci(itci+1:)      )&
                     &,nb,na,tol&
                     &,subblock=(/subblock(3,isb),subblock(4,isb)&
                     &,subblock(1,isa),subblock(2,isa)/)&
                     &,trans=.True.)

             Else
                Call printmatsb('Im '//txt,fmt,fmt0,Aimag(tci(itci+1:)      )&
                     &,nb,na,tol,trans=.True.)
             Endif
             !!
          Enddo
       Enddo
    Enddo
    !!
10  Format('siteA=',i5,' siteB=',i5,' pair=',i5)
    !!
  End Subroutine print_tci_cx
  !!***********************************************************************
  Subroutine print_oci(txt,nsite,blocklengtha,blocklengthb,fmt,fmt0,oci,tol&
       &,ociindex,subblock,trans,fac)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(in)      :: fmt,fmt0,txt
    Integer(t_int),Intent(in)    :: nsite,blocklengtha(*),blocklengthb(*)
    Integer(t_int),Intent(in)    :: ociindex(*)
    Logical(t_log),Intent(in),Optional    :: trans
    Integer(t_int),Intent(in),Optional    :: subblock(4,*)
    Real(t_real),Intent(in),Optional      :: fac
    Real(t_real),Intent(in)      :: oci(:)
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Logical(t_log) :: dotr
    Integer(t_int) :: bla,blb,ioci,isa
    Real(t_real)   :: f
    !!
    f=1.0_t_real
    If(Present(fac)) Then
       f=fac
    Endif
    !!
    dotr=.True.
    If(Present(trans)) Then
       If(trans)Then
          dotr=.False.
       Else
          dotr=.True.
       Endif
    Endif
    !!
    Do isa=1,nsite
       bla=blocklengtha(isa)
       blb=blocklengthb(isa)
       !!
       ioci=ociindex(isa)
       Write(outfile,10) isa
       If(Present(subblock)) Then
          Call printmatsb(txt,fmt,fmt0,f*oci(ioci+1:),blb,bla,tol&
               &,subblock=(/subblock(3,isa),subblock(4,isa)&
               &,subblock(1,isa),subblock(2,isa)/)&
               &,trans=dotr)
       Else
          Call printmatsb(txt,fmt,fmt0,f*oci(ioci+1:),blb,bla,tol&
               &,trans=dotr)
       Endif
       !!
    Enddo
    !!
10  Format('site=',i5)
    !!
  End Subroutine print_oci
  !!***********************************************************************
  Subroutine print_tci(txt,nsite,blocklengtha,blocklengthb,fmt,fmt0,tci&
       &,tciindex,npairs,pairs,tol,subblock,trans,fac)
    !!
    Implicit none  
    !!IO
    Character(*),Intent(in)      :: fmt,fmt0,txt
    Integer(t_int),Intent(in)    :: nsite,blocklengtha(*),blocklengthb(*)
    Real(t_real),Intent(in)      :: tci(:)
    Integer(t_int),Intent(in)    :: npairs(:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Integer(t_int),Intent(in)    :: tciindex(:,:,:)
    Logical(t_log),Intent(in),Optional    :: trans
    Integer(t_int),Intent(in),Optional    :: subblock(4,*)
    Real(t_real),Intent(in),Optional      :: fac
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    Logical(t_log) :: dotr
    Integer(t_int) :: na,nb,itci,isa,isb,irp
    Real(t_real)   :: f
    !!
    f=1.0_t_real
    If(Present(fac)) Then
       f=fac
    Endif
    !!
    dotr=.True.
    If(Present(trans)) Then
       If(trans)Then
          dotr=.False.
       Else
          dotr=.True.
       Endif
    Endif
    !!
    Do isa=1,nsite
       na=blocklengtha(isa)
       Do isb=1,nsite
          nb=blocklengthb(isb)
          !!
          Do irp=1,npairs(isa,isb)

             itci=tciindex(irp,isa,isb)
             Write(outfile,10) isa,isb,irp,pairs(:,irp,isa,isb)
             If(Present(subblock)) Then
                Call printmatsb(txt,fmt,fmt0,f*tci(itci+1:),nb,na,tol&
                     &,subblock=(/subblock(3,isb),subblock(4,isb)&
                     &,subblock(1,isa),subblock(2,isa)/)&
                     &,trans=dotr)
             Else
                Call printmatsb(txt,fmt,fmt0,f*tci(itci+1:),nb,na,tol&
                     &,trans=dotr)
             Endif
             !!
          Enddo
       Enddo
    Enddo
    !!
10  Format('siteA=',i5,' siteB=',i5,' pair=',i5,' p=',3f12.5)
    !!
  End Subroutine print_tci  
  !!***********************************************************************
End Module loiaux

