Module loisym
  !!
  Use precisiondefs
  Use file_iounits
  Use memory
  Use clock
  Use errors
  Use tsymmetry
  Use tkohnsham
  Use torbitalbasis
  Use tloi
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="LOISYM"  
  !!
  Public :: symmetrize_oci,symmetrize_tci,gvecdifn&
       &,symmetrize_foci,symmetrize_ftci
  !!
Contains
  !!***********************************************************************
  Subroutine symmetrize_oci(mspin,nsite,isrt&
       &,numops,equivsites,wu_ks&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,ocidim,oci,ociindex)
    
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin,isrt(nsite)
    Integer(t_int),Intent(in)    :: ocidim
    Integer(t_int),Intent(in)    :: numops,equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: ociindex(nsite)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(inout)   :: oci(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_oci"
    Logical(t_log) :: lsp
    Integer(t_int) :: isa,issa,lena,lenb,isga
    Integer(t_int) :: ioci,gioci,jops
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_oci(:,:)
    !!
    !!
    Nullify(p_oci)
    !!
    Call palloc("p_oci",p_oci&
         &,1_t_int,ocidim&
         &,1_t_int,mspin&
         &,nsubfunc,nmodule)
    !!
    lsp=(mspin == 2)
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    !! onsite
    !!
    p_oci(:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       lenb=Sum(2*lRb(1:nRb(issa),issa)+1)

       !!
       ioci=ociindex(isa)
       !!
       ojopsloop: Do jops=1,numops
          isga=equivsites(jops,isa)
          !!
          !!
          !!
          gioci=ociindex(isga)
          !!
          !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
                     
          Call symmetrize_loi_block(lsp&
               &,nma,nRa,lRa,nmb,nRb,lRb&
               &,oci,ioci,gioci,p_oci&
               &,lena,lenb&
               &,issa,issa,jops,norm,wu_ks)
          
          !!
          !Write(buffer,10)isa,symmetry% Ropsymbol(jops),isga
          !Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
          !     &,bla,blb,oci(ioci+1:,1),loci(ioci+1:,1))
          !!
       Enddo ojopsloop
       !!
       !Write(buffer,20)isa
       !Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
       !     &,bla,blb,oci(ioci+1:,1),p_oci(ioci+1:,1))
       !!
    End Do isaloop
    !!
    oci(:,:)=p_oci(:,:)
    !!
    Call pfree(p_oci)
    !!
    Return
    !!
!10  Format('OCI: ','Sa=',i4,' G=',a15,' GSa=',i4)
!20  Format('OCI: ','Sa=',i4)
    !!
  End Subroutine symmetrize_oci
  !!***********************************************************************
  Subroutine symmetrize_tci(mspin,nsite,isrt&
       &,npairsmax,npairs&
       &,numops,equivsites,wu_ks&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,tcidim,tci,tciindex,gdiff)
    
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin,isrt(nsite)
    Integer(t_int),Intent(in)    :: tcidim
    Integer(t_int),Intent(in)    :: numops,equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: tciindex(:,:,:)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Integer(t_int),Intent(in)    :: gdiff(0:npairsmax,48,nsite,nsite)
    Real(t_real),Intent(inout)   :: tci(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_tci"
    Logical(t_log) :: lsp
    Integer(t_int) :: isa,isb,issa,issb,isga,isgb,irab,irgagb,lena,lenb
    Integer(t_int) :: itci,gitci,jops
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_tci(:,:)

    !!
    !!
    !! Oin,Rjm=norm * sum_g ( wuT* Ogin,[DPRji-gj+gi]gjm)*wu )
    !!
    
    Nullify(p_tci)
    !!
    Call palloc("p_tci",p_tci&
         &,1_t_int,tcidim&
         &,1_t_int,mspin&
         &,nsubfunc,nmodule)
    !!
    lsp=(mspin == 2)
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    !!
    !! twosite
    !!
    p_tci(:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       !!
       isbloop: Do isb=1,nsite
          !!
          issb=isrt(isb)
          lenb=Sum(2*lRb(1:nRb(issb),issb)+1)
          !!
          irabloop: Do irab=1,npairs(isa,isb)
             !!
             itci=tciindex(irab,isa,isb)
             !!
             ojopsloop: Do jops=1,numops
                isga=equivsites(jops,isa)
                isgb=equivsites(jops,isb)
                !!
                irgagb=gdiff(irab,jops,isb,isa)
                !!
                gitci=tciindex(irgagb,isga,isgb)
                !!
                !!
                !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
                !!
                Call symmetrize_loi_block(lsp&
                     &,nma,nRa,lRa,nmb,nRb,lRb&
                     &,tci,itci,gitci,p_tci&
                     &,lena,lenb&
                     &,issa,issb,jops,norm,wu_ks)
                
                !!
                !Write(buffer,11)isa,isb,irab,symmetry% Ropsymbol(jops)&
                !     &,isga,isgb,irgagb
                !Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
                !  &,bla,blb,tci(itci+1:,1),p_tci(itci+1:,1))

                !!
             Enddo ojopsloop
             !!
             !Do ms=1,mspin
             !   Write(buffer,21)isa,isb,irab,ms
             !   Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
             !        &,bla,blb,tci(itci+1:,ms),p_tci(itci+1:,ms))
             !Enddo
             !!
          End Do irabloop
       End Do isbloop
    End Do isaloop
    !!
    tci(:,:)=p_tci(:,:)
    Call pfree(p_tci)
    !!
    Return
    !!
!20  Format('OCI: ','Sa=',i4)
!21  Format('TCI: ','Sa=',i4,' Sb=',i4,' Pab=',i4,' spin=',i2)

    !!
  End Subroutine symmetrize_tci
  !!***********************************************************************
  Subroutine symmetrize_loi_block(lsp&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,loi,iloi,giloi,lloi&
       &,lena,lenb&
       &,issa,issb,jops,norm,wu_ks)
    !!
    Implicit None  
    !!IO
    Logical(t_log),Intent(in)    :: lsp
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: iloi,giloi
    Integer(t_int),Intent(in)    :: issa,issb
    Integer(t_int),Intent(in)    :: jops
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(in)      :: norm
    Real(t_real),Intent(in)      :: loi(:,:)
    Real(t_real),Intent(inout)   :: lloi(:,:)
    !!LOCAL
    Integer(t_int) :: ii,jj,ma,la,iva,ivb,lb,degb,dega
    Real(t_real)   :: A(lena,lenb,2)
    !!
    !!
    !!
    !! Oin,Pjm=( wuT* Ogin,DPgjm)*wu )
    ii=giloi
    Do iva=1,lena
       jj=0
       Do ivb=1,nRb(issb)
          lb=lRb(ivb,issb)
          degb=2*lb+1
          !!
          A(iva,jj+1:jj+degb,1)=Matmul(loi(ii+1:ii+degb,1)&
               &             ,wu_ks(1:degb,1:degb,lb,jops))
          If(lsp)Then
             A(iva,jj+1:jj+degb,2)=Matmul(loi(ii+1:ii+degb,2)&
                  &             ,wu_ks(1:degb,1:degb,lb,jops))
             
          Endif
          jj=jj+degb
          ii=ii+degb
       Enddo
    Enddo
    !!
    ii=iloi
    jj=0
    Do iva=1,nRa(issa)
       la=lRa(iva,issa)
       dega=2*la+1
       Do ma=1,dega
          !!
          lloi(ii+1:ii+lenb,1)=lloi(ii+1:ii+lenb,1)&
               &+Matmul(wu_ks(1:dega,ma,la,jops)&
               & ,A(jj+1:jj+dega,1:lenb,1))*norm
          If(lsp)Then
             lloi(ii+1:ii+lenb,2)=lloi(ii+1:ii+lenb,2)&
                  &+Matmul(wu_ks(1:dega,ma,la,jops)&
                  & ,A(jj+1:jj+dega,1:lenb,2))*norm
          Endif
          !!
          ii=ii+lenb
       Enddo
       jj=jj+dega
    Enddo
    !!
    
    !!
  End Subroutine symmetrize_loi_block
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine sympro(txt,fmt,fmt0,tol,dima,dimb,ociold,ocinew)
    !!
    Implicit None  
    !!IO
    Character(*),Intent(in)      :: txt,fmt,fmt0
    Integer(t_int),Intent(in)    :: dima,dimb
    Real(t_real),Intent(in)      :: tol
    Real(t_real),Intent(in)      :: ociold(:)
    Real(t_real),Intent(in)      :: ocinew(:)
    !!LOCAL
    Integer(t_int) :: ioci,ia,ib
    Real(t_real)   :: d,err,dnew,maxerr!,rel
    !!
    maxerr=0.0_t_real
    ioci=0_t_int
    Do ia=1_t_int,dima
       Do ib=1_t_int,dimb
          ioci=ioci+1_t_int
          d=ociold(ioci)
          dnew=ocinew(ioci)
          err=Abs(d-dnew)
          !rel=Abs(err)
          !If(dnew/=0.0_t_real) rel=rel/Abs(dnew)
          maxerr=Max(err,maxerr)
       Enddo               ! ib
    Enddo                   ! ia
    !!
    If(maxerr<tol)Return
    !!
    Write(outfile,21)Trim(txt)
21  Format(a)
    !!
    ioci=0_t_int
    Do ia=1_t_int,dima
       Do ib=1_t_int,dimb
          ioci=ioci+1_t_int
          d=ociold(ioci)
          dnew=ocinew(ioci)
          err=Abs(d-dnew)
          !rel=Abs(err)
          !If(dnew/=0.0_t_real) rel=rel/Abs(dnew)
          d=err
          If(Abs(d)<tol)Then
             Write(outfile,fmt0,advance='no') 
          Else
             Write(outfile,fmt,advance='no') d
          Endif
       Enddo               ! ib
       Write(outfile,*)
    Enddo                   ! ia
    !!
  End Subroutine sympro
#endif
  !!***********************************************************************
  Subroutine gvecdifn(nsite,npairs,npairsmax,pairs&
       &,numops,equivsites,opmat,p_gdiff)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite
    Integer(t_int),Intent(in)    :: numops
    Integer(t_int),Intent(in)    :: equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Real(t_real),Intent(in)      :: opmat(3,3,48)
    Integer(t_int),Pointer       :: p_gdiff(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="gvecdifn"
    Integer(t_int) :: isa,isb,irab,jops,isga,isgb,irgagb,m,irab0,ir0
    Real(t_real)   :: rab(3),rgagb(3),DP(3)
    !!
    !!Write(outfile,"('START',a)")nsubfunc
    Call palloc('p_gdiff',p_gdiff&
         &,0_t_int,npairsmax&
         &,1_t_int,48_t_int&
         &,1_t_int,nsite&
         &,1_t_int,nsite&
         &,nmodule,nsubfunc)
    !!
    Write(outfile,1000) &
         & 1+npairsmax,48_t_int,nsite,nsite&
         &,(1+npairsmax)*48_t_int*(nsite)**2*4/1.0e6_t_real
1000 Format('Allocated p_gdiff(',i6,',',i6,',',i6,',',i6,') = ',f10.5&
          &,' MByte')
    !!
    isaloop: Do isa=1,nsite
       !!Write(outfile,"('isa=',i5)")isa
       isbloop: Do isb=1,nsite
          !!Write(outfile,"('isb=',i5)")isb
          jopsloop: Do jops=1,numops
             !!Write(outfile,"('jops=',i5)")jops
             !! isga=G isa
             isga=equivsites(jops,isa)
             !! isgb=G isb
             isgb=equivsites(jops,isb)
             !!
             !!Write(outfile,"('isa=',i5,'isga=',i5)")isa,isga
             !!Write(outfile,"('isb=',i5,'isgb=',i5)")isb,isgb
             irab0=1
             If(isb==isa)Then
                irab0=0
             Endif
             irabloop: Do irab=irab0,npairs(isa,isb)
                !!
                !! rab=R+Sa-Sb
                !!
                If(irab==0)Then
                   rab(:)=0.0_t_real
                Else
                   rab(:)=pairs(:,irab,isa,isb)
                Endif
                !!
                !!Write(outfile,"('rab=',3(f30.15))")rab

                !! \hat{D}(W)*rab =DR(W)*rab =rab*DR(W^-1)=rab*opmat
                !!
                !!
                DP=Matmul(rab,opmat(:,:,jops))
                !!
                !!Write(outfile,"('DP=',3(f30.15))")DP
                !! search irgagb such that pairs(:,irgagb,isga,isgb) ==DP
                irgagb=-1
                ir0=1
                If(isgb==isga)Then
                   ir0=0
                Endif
                mloop: Do m=ir0,npairs(isga,isgb)
                   !!
                   !! rgagb=R+Sga-Sgb
                   !!
                   If(m==0)Then
                      rgagb(:)=0.0_t_real
                   Else
                      rgagb(:)=pairs(:,m,isga,isgb)
                   Endif
                   !!
                   !!Write(outfile,"('rgagb   =',3(f30.15))")rgagb
                   !!Write(outfile,"('rgagb-DP=',3(f30.15))")rgagb-DP

                   !!
                   If(All(Abs(rgagb(:)-DP(:)) < 1.e-5_t_real)) Then
                      irgagb=m
                      Exit mloop
                   Endif
                   !!
                   !!
                End Do mloop
                !!
                If(irgagb==-1)Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'Could not find a symmetry related vector !'&
                        &,/,'Please report all calculational detail to the&
                        & authors!'")
                Endif
                !!
                p_gdiff(irab,jops,isb,isa)=irgagb
                !!
             End Do irabloop
          End Do jopsloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!Write(outfile,"('END',a)")nsubfunc
    Return
    !!
  End Subroutine gvecdifn
  !!***********************************************************************
  Subroutine symmetrize_ftci(mspin,nsite,isrt&
       &,npairsmax,npairs&
       &,numops,opmat,equivsites,wu_ks&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,tcidim,tci,tciindex,gdiff)
    
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin,isrt(nsite)
    Integer(t_int),Intent(in)    :: tcidim
    Integer(t_int),Intent(in)    :: numops,equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: tciindex(:,:,:)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(in)      :: opmat(3,3,*)
    Integer(t_int),Intent(in)    :: gdiff(0:npairsmax,48,nsite,nsite)
    Real(t_real),Intent(inout)   :: tci(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_ftci"
    Logical(t_log) :: lsp
    Integer(t_int) :: isa,isb,issa,issb,isga,isgb,irab,irgagb,lena,lenb
    Integer(t_int) :: itci,gitci,jops
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_tci(:,:,:)

    !!
    !!
    !! Oin,Rjm=norm * sum_g ( wuT* Ogin,[DPRji-gj+gi]gjm)*wu )
    !!
    
    Nullify(p_tci)
    !!
    Call palloc("p_tci",p_tci&
         &,1_t_int,3_t_int&
         &,1_t_int,tcidim&
         &,1_t_int,mspin&
         &,nsubfunc,nmodule)
    !!
    lsp=(mspin == 2)
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    !!
    !! twosite
    !!
    p_tci(:,:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       !!
       isbloop: Do isb=1,nsite
          !!
          issb=isrt(isb)
          lenb=Sum(2*lRb(1:nRb(issb),issb)+1)
          !!
          irabloop: Do irab=1,npairs(isa,isb)
             !!
             itci=tciindex(irab,isa,isb)
             !!
             ojopsloop: Do jops=1,numops
                isga=equivsites(jops,isa)
                isgb=equivsites(jops,isb)
                !!
                irgagb=gdiff(irab,jops,isb,isa)
                !!
                gitci=tciindex(irgagb,isga,isgb)
                !!
                !!
                !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
                !!
                Call symmetrize_floi_block(lsp&
                     &,nma,nRa,lRa,nmb,nRb,lRb&
                     &,tci,itci,gitci,p_tci&
                     &,lena,lenb&
                     &,issa,issb,jops,norm,wu_ks,opmat(:,:,jops))
                
                !!
                !Write(buffer,11)isa,isb,irab,symmetry% Ropsymbol(jops)&
                !     &,isga,isgb,irgagb
                !Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
                !  &,nva,nvb,tci(itci+1:,1),p_tci(itci+1:,1))

                !!
             Enddo ojopsloop
             !!
             !Do ms=1,mspin
             !   Write(buffer,21)isa,isb,irab,ms
             !   Call sympro(buffer,'(f10.6,1x)','("    0     ",1x)',1.0e-6_t_real&
             !        &,nva,nvb,tci(itci+1:,ms),p_tci(itci+1:,ms))
             !Enddo
             !!
          End Do irabloop
       End Do isbloop
    End Do isaloop
    !!
    tci(:,:,:)=p_tci(:,:,:)
    Call pfree(p_tci)
    !!
    Return
    !!
!20  Format('OCI: ','Sa=',i4)
!21  Format('TCI: ','Sa=',i4,' Sb=',i4,' Pab=',i4,' spin=',i2)

    !!
  End Subroutine symmetrize_ftci
  !!***********************************************************************
  Subroutine symmetrize_foci(mspin,nsite,isrt&
       &,numops,opmat,equivsites,wu_ks&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,ocidim,oci,ociindex)
    
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin,isrt(nsite)
    Integer(t_int),Intent(in)    :: ocidim
    Integer(t_int),Intent(in)    :: numops,equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: ociindex(nsite)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(in)      :: opmat(3,3,*)
    Real(t_real),Intent(inout)   :: oci(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_foci"
    Logical(t_log) :: lsp
    Integer(t_int) :: isa,issa,lena,lenb,isga
    Integer(t_int) :: ioci,gioci,jops
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_oci(:,:,:)
    !!
    !!
    Nullify(p_oci)
    !!
    Call palloc("p_oci",p_oci&
         &,1_t_int,3_t_int&
         &,1_t_int,ocidim&
         &,1_t_int,mspin&
         &,nsubfunc,nmodule)
    !!
    lsp=(mspin == 2)
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    !! onsite
    !!
    p_oci(:,:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       lenb=Sum(2*lRb(1:nRb(issa),issa)+1)
       !!
       ioci=ociindex(isa)
       !!
       ojopsloop: Do jops=1,numops
          
          isga=equivsites(jops,isa)
          !!
          !!
          !!
          gioci=ociindex(isga)
          !!
          !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
                     
          Call symmetrize_floi_block(lsp&
               &,nma,nRa,lRa,nmb,nRb,lRb&
               &,oci,ioci,gioci,p_oci&
               &,lena,lenb&
               &,issa,issa,jops,norm,wu_ks,opmat(:,:,jops))
          !!
          !!
          !!
       Enddo ojopsloop
       !!
       !!
    End Do isaloop
    !!
    oci(:,:,:)=p_oci(:,:,:)
    !!
    Call pfree(p_oci)
    !!
    Return
    !!
!10  Format('OCI: ','Sa=',i4,' G=',a15,' GSa=',i4)
!20  Format('OCI: ','Sa=',i4)
    !!
  End Subroutine symmetrize_foci
  !!***********************************************************************
  Subroutine symmetrize_floi_block(lsp&
       &,nma,nRa,lRa,nmb,nRb,lRb&
       &,loi,iloi,giloi,lloi&
       &,lena,lenb&
       &,issa,issb,jops,norm,wu_ks,MM)
    !!
    Implicit None  
    !!IO
    Logical(t_log),Intent(in)    :: lsp
    Integer(t_int),Intent(in)    :: nma,nRa(*),lRa(nma,*)
    Integer(t_int),Intent(in)    :: nmb,nRb(*),lRb(nmb,*)
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: iloi,giloi
    Integer(t_int),Intent(in)    :: issa,issb
    Integer(t_int),Intent(in)    :: jops
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:),MM(3,3)
    Real(t_real),Intent(in)      :: norm
    Real(t_real),Intent(in)      :: loi(:,:,:)
    Real(t_real),Intent(inout)   :: lloi(:,:,:)
    !!LOCAL
    Integer(t_int) :: ii,jj,ma,la,iva,ivb,lb,degb,dega,ik
    Real(t_real)   :: A(3,lena,lenb,2)
    !!
    !!
    !!
    !! Oin,Pjm=( wuT* Ogin,DPgjm)*wu )
    ii=giloi
    Do iva=1,lena
       jj=0
       Do ivb=1,nRb(issb)
          lb=lRb(ivb,issb)
          degb=2*lb+1
          !!
          A(1:3,iva,jj+1:jj+degb,1)=Matmul(MM,Matmul(loi(1:3,ii+1:ii+degb,1)&
               &             ,wu_ks(1:degb,1:degb,lb,jops)))
          If(lsp)Then
             A(1:3,iva,jj+1:jj+degb,2)=Matmul(MM,Matmul(loi(1:3,ii+1:ii+degb,2)&
                  &             ,wu_ks(1:degb,1:degb,lb,jops)))
          Endif
          
          jj=jj+degb
          ii=ii+degb
       Enddo
    Enddo
    !!
    ii=iloi
    jj=0
    Do iva=1,nRa(issa)
       la=lRa(iva,issa)
       dega=2*la+1
       Do ma=1,dega
          !!
          Do ik=1,3
             lloi(ik,ii+1:ii+lenb,1)=lloi(ik,ii+1:ii+lenb,1)&
                  &+Matmul(wu_ks(1:dega,ma,la,jops)&
                  & ,A(ik,jj+1:jj+dega,1:lenb,1))*norm
             If(lsp)Then
                lloi(ik,ii+1:ii+lenb,2)=lloi(ik,ii+1:ii+lenb,2)&
                     &+Matmul(wu_ks(1:dega,ma,la,jops)&
                     & ,A(ik,jj+1:jj+dega,1:lenb,2))*norm
             Endif
          Enddo
          !!
          ii=ii+lenb
       Enddo
       jj=jj+dega
    Enddo
    !!
    !!
    !!
  End Subroutine symmetrize_floi_block
  !!***********************************************************************
End Module loisym
