!!----DEBUG options-----------------|
!! SUBFUNC    | MODES               |
!!----------------------------------|
!!----------------------------------|
Module lsdau
  !!
  Use precisiondefs
  Use constants
  Use file_iounits
  Use errors
  Use atomdefs
  Use tdiracdefs
  Use tsymmetry
  Use torbitalbasis
  Use trelativistic
  Use tlsdau
  Use memory
  Use optionstest
  Use diagonalize
  Use matrixinvers
  Use matserv
  Use prnt
  !!TEST Use fplodata
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="LSDAU"  
  !!
  Public :: io_init_lsdau,lsdau_calc_u_cs,lsdau_calc_u_dmat&
       &,lsdau_calc_u_potmat,lsdau_merge_h,lsdau_norm_conv&
       &,force_u_mat_sym,force_u_hermit&
       &,print_u_dmat,lsda_sdcmp,lsda_scmp,lsdau_aux_alloc,lsdau_aux_free&
       &,lsdau_save_dmat,lsdau_d_mat_trans_spin
  !!***********************************************************************
  Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
  Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
  Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
  !!
  Logical(t_log) :: diagonalonly=.False. !!todo
  Logical(t_log) :: checkspinrep=.True.
  Integer(t_int) :: indices(3)!!todo
  Data indices/1_t_int,2_t_int,3_t_int/
  !!
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine io_init_lsdau(&
!!!       &symmetry,&
       &structure,basis,lsdau,quant,nspin,BEB&
       &,relativistic)
    !!-----------------------------------------------------------------------
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
!!!    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Lsdau),Intent(inout)  :: lsdau 
    Type(T_Quant),Intent(inout)  :: quant 
    Integer(t_int),Intent(in)    :: nspin,relativistic
    Logical(t_log),Intent(in)    :: BEB
    !!LOCAL
    Character(*),Parameter::nsubfunc="io_init_lsdau"
    Logical(t_log) :: rel
    Integer(t_int) :: judef,iudef,issau,ivu,n,l,iv,i,isa,issa,iz,isau,deg
    Integer(t_int) :: issb,nmain,i0,kap,k,mu,m,ms
    Integer(t_int),Pointer :: p_nvu(:)
!!!    Integer(t_int) :: npar
    Real(t_real)   :: d,eulang(3)
    Character(2)   :: state
    Logical(t_log) :: apply_to_double,exists
    Real(t_real),Pointer :: fac(:),wu(:)
    !!
    Nullify(p_nvu,fac,wu)
    !!
    apply_to_double=.False.
    !!
    If(lsdau% lsdau .And. BEB)Then
       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg="'CPA and LSDA+U is an experimental implementation!'")
    Endif
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
!!$    If(lsdau% lsdau .And. rel)Then
!!$       Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$            &,msg="'LSDA+U and full relativistic not yet implemented!'")
!!$    Endif
    !!
    If( BEB .And. lsdau% lsdau .And. lsdau% projection /=LSDAU_ORTHOGONAL)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Only orthogonal projection implemented for CPA+LSDA+U !'")
    Endif
    If( lsdau% lsdau .And. &
         & ( lsdau% projection /=LSDAU_ORTHOGONAL).And.  &
         & ( lsdau% projection /=LSDAU_GROSS)            &
         &)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized LSDA+U projection no longer supported !'")
    Endif
    !!
    !!

    lsdau% nusite=0_t_int
    If(.Not.lsdau% lsdau)Then
       lsdau% nudef=0_t_int
       Return
    Endif
    If(lsdau% nudef==0_t_int)Then
       lsdau% lsdau=.False.
       Return
    End If
    
    !!
    lsdau% umaxdim=0_t_int
    !!
    !!
    !!
    !!

    Write(outfile,1)
    !!
    Call check_lsdau_version(lsdau)
    !!
1   Format('LSDA+U: ',70('-'))
2   Format('LSDA+U: Projection                            : ',a)
3   Format('LSDA+U: Functional                            : ',a)
4   Format('LSDA+U: Occupation matrix iteration tolerance : ',e12.5e2)
    Write(outfile,2) Trim(LSDAU_PROJECTIONS(lsdau% projection))
    Write(outfile,3) Trim(LSDAU_FUNCTIONALS(lsdau% functional))
    !!
    !!
    Write(outfile,4) lsdau% omat_dev
    !!
    If(  lsdau% projection == LSDAU_ORTHOGONAL ) Then
       apply_to_double=.True.
    Else If(  lsdau% projection == LSDAU_GROSS ) Then
       apply_to_double=.False.
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Unimplemented branch for current projection!'")
    Endif
    !!
    !!
    Call palloc('lsdau% p_u_j',lsdau% p_u_j ,1_t_int,2_t_int &
         &,1_t_int,lsdau% nudef ,nmodule,nsubfunc)
    Call palloc('p_nvu',p_nvu ,1_t_int,lsdau% nudef ,nmodule,nsubfunc)
    !!
    lsdau% p_u_j(1_t_int:2_t_int,1_t_int:lsdau% nudef)=0.0_t_real
    !!
    lsdau% lmx=0_t_int
    Do iudef=1_t_int,lsdau% nudef
       !!
       issau=lsdau% p_isrtu(iudef)
       state=lsdau% p_state(iudef)
       
       lsdau% p_u_f(0_t_int:3_t_int,iudef)&
            &=lsdau% p_u_f(0_t_int:3_t_int,iudef)/hatoev
       
       !!
       If(issau > structure% IOnsort .Or. issau < 1_t_int )Then
5         Format("'LSDA+U: input sort: ",i4," does not exist! '")
          Write(errbuf,5)issau
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=errbuf)
       Endif
       
       !!
       Read(state(1_t_int:1_t_int),"(i1)") n
       l=lcharqn(state(2_t_int:2_t_int))
       lsdau% umaxdim=Max(lsdau% umaxdim,2_t_int*l+1_t_int)
       lsdau% lmx=Max(lsdau% lmx,l)
       !!
       If(l>3)Then
          Write(errbuf,636)structure % p_IOelements(issau),issau,n&
               &,lqnchar(l)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
636    Format('''LSDA+U: input atom ',a2,' sort(',i3,'): orbital ',i1,a1&
            &,' not allowed for LSDA+U!''') 
          
       !!
       issa=-1
       Do issb=1,structure% nsort
          If(structure% p_isrtTOIOisrt(issb) == issau)Then
             issa=issb
          Endif
       Enddo
       
       If(issa == -1)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="('issa = -1 ?!?!?')")
       Endif
       !!
       ivu=-1_t_int
       Do iv=1,basis % p_nNLv(issa)
          If( basis% p_lNLv(iv,issa) == l .And. &
               & basis% p_nnNLv(iv,issa) == n )Then
             ivu=iv
             Exit
          Endif
       Enddo
       If(ivu<0_t_int)Then
6         Format("'LSDA+U: valence state (",i2,a&
               &,") not defined for input sort ",i5," (",a2,")!'")
          Write(errbuf,6) n,lqnchar(l),issau ,structure % p_elements(issa)
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=Trim(errbuf))
       End If
       
       p_nvu(iudef)=ivu
       
       
    Enddo

    !!
    !! sort definitions into kohn-sham-matrix-order
    !!
    Do iudef=1_t_int,lsdau% nudef

       Do judef=iudef+1_t_int,lsdau% nudef

          If( (lsdau% p_isrtu(judef) < lsdau% p_isrtu(iudef)) .Or.&
               & ( (lsdau% p_isrtu(judef) == lsdau% p_isrtu(iudef) ) .And.&
               &   (p_nvu(judef) < p_nvu(iudef) ) ) )Then
             issau=lsdau% p_isrtu(judef)
             lsdau% p_isrtu(judef)=lsdau% p_isrtu(iudef)
             lsdau% p_isrtu(iudef)=issau
             state=lsdau% p_state(judef)
             lsdau% p_state(judef)=lsdau% p_state(iudef)
             lsdau% p_state(iudef)=state
             i=p_nvu(judef)
             p_nvu(judef)=p_nvu(iudef)
             p_nvu(iudef)=i
             Do i=0_t_int,3_t_int
                d=lsdau% p_u_f(i,judef)
                lsdau% p_u_f(i,judef)=lsdau% p_u_f(i,iudef)
                lsdau% p_u_f(i,iudef)=d
             End Do

          Endif
          If( (lsdau% p_isrtu(judef) == lsdau% p_isrtu(iudef)) .And.&
               & (p_nvu(judef) == p_nvu(iudef) ) )Then
7            Format("'LSDA+U: input sort ",i4," state ",a2&
                  &," multiply defined !'")
             Write(errbuf,7)lsdau% p_isrtu(iudef),lsdau% p_state(iudef)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          End If
          
       Enddo
    Enddo


    !! output and organization

8   Format('LSDA+U:                      ',i5,'  Correlated states '&
         &,/,'LSDA+U: sort  el.  state   F0       F2       F4       &
         &F6       U        J'&
         &,/,'LSDA+U:                                (in eV)' )
    
    Write(outfile,1)
    Write(outfile,8)lsdau% nudef
    Write(outfile,1)
    !!

    Do iudef=1_t_int,lsdau% nudef
       !!
       issau=lsdau% p_isrtu(iudef)
       ivu=p_nvu(iudef)
       l=lcharqn(lsdau% p_state(iudef)(2:2))


       
       lsdau% p_u_j(1,iudef)=lsdau% p_u_f(0,iudef)
       If (l == 0_t_int) lsdau% p_u_j(2,iudef)=0.0_t_real
       If (l == 1_t_int) lsdau% p_u_j(2,iudef)&
            & =lsdau% p_u_f(1,iudef)/5.0_t_real
       If (l == 2_t_int) lsdau% p_u_j(2,iudef)&
            &=(lsdau% p_u_f(1,iudef)+lsdau% p_u_f(2,iudef))/14.0_t_real
       If (l == 3_t_int) lsdau% p_u_j(2,iudef)&
            &=(286.0_t_real*lsdau% p_u_f(1,iudef)&
            & +195.0_t_real*lsdau% p_u_f(2,iudef)&
            & +250.0_t_real*lsdau% p_u_f(3,iudef))/6435.0_t_real
       !!
9      Format('LSDA+U: ',i3,3x,a2,5x,i1,a1,2x,6(f8.4,1x))

       issa=-1
       Do issb=1,structure% nsort
          If(structure% p_isrtTOIOisrt(issb) == issau)Then
             issa=issb
          Endif
       Enddo
  
       Do iv=1,basis % p_nNLv(issa)
          If( (apply_to_double .And. &
               &(basis% p_firstNLv(iv,issa) == basis% p_firstNLv(ivu,issa)) ) .Or.&
               &(.Not.apply_to_double .And. (iv == ivu) )&
               &) Then

             
             
             Write(outfile,9) issau,structure% p_elements(issa)&
                  &,basis% p_nnNLv(iv,issa)&
                  &,lqnchar(l),lsdau% p_u_f(0_t_int:3_t_int,iudef)*hatoev &
                  &,lsdau% p_u_j(1_t_int,iudef)*hatoev &
                  &,lsdau% p_u_j(2_t_int,iudef)*hatoev
          Endif
       Enddo
       !!
       Do i=0_t_int,l
          lsdau% p_u_f(i,iudef)=lsdau% p_u_f(i,iudef)*fourpi&
               &/Real(4_t_int*i+1_t_int,t_real)
       Enddo

    Enddo
    Write(outfile,1)

       
    !! make indicees into matrix blocks
    !!
    !! fisrt count number of occurencuies
    lsdau% nusite=0_t_int
    Do isa=1_t_int,structure% nsite
       issa=structure% p_isrt(isa)
       Do iv=1_t_int,basis% p_nNLv(issa)
          Do iudef=1_t_int,lsdau% nudef
             issau=lsdau% p_isrtu(iudef)
             ivu=p_nvu(iudef)
             If(structure% p_isrtTOIOisrt(issa) == issau .And.&
                  & (&
                  &   (apply_to_double .And. &
                  &    (basis% p_firstNLv(iv,issa) == basis% p_firstNLv(ivu,issa))&
                  &   ) .Or. &
                  &   (.Not.apply_to_double .And. (iv == ivu))  &
                  & )&
                  & )Then

                lsdau% nusite=lsdau% nusite+1_t_int
             End If
          End Do
       End Do
    End Do
    
    lsdau% rumaxdim=lsdau% umaxdim
    If(rel) lsdau% rumaxdim=lsdau% rumaxdim*2

    !!
11  Format('LSDA+U: site-state pairs        : ',i5)
12  Format('LSDA+U: maximum block dimension : ',i5)
13  Format('LSDA+U: maximum angular momentum: ',i5)
    Write(outfile,11)lsdau% nusite
    Write(outfile,12)lsdau% umaxdim
    Write(outfile,13)lsdau% lmx
    !!
    Call palloc('lsdau% p_udefs',lsdau% p_udefs &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_isiteu',lsdau% p_isiteu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_isortu',lsdau% p_isortu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_ubi1',lsdau% p_ubi1 &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_ubi2',lsdau% p_ubi2 &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_lvsu',lsdau% p_lvsu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_degvsu',lsdau% p_degvsu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_nvsu',lsdau% p_nvsu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    Call palloc('lsdau% p_labelsu',lsdau% p_labelsu &
         &,1_t_int,lsdau% nusite &
         &,nmodule,nsubfunc)
    !!
    iz=0_t_int
    lsdau% nusite=0_t_int
    Do isa=1_t_int,structure% nsite
       issa=structure% p_isrt(isa)
       Do iv=1_t_int,basis% p_nNLv(issa)
          nmain=basis% p_nnNLv(iv,issa)
          l=basis% p_lNLv(iv,issa)
          deg=basis% p_degNLv(iv,issa)
          udefloop:Do iudef=1_t_int,lsdau% nudef
             issau=lsdau% p_isrtu(iudef)
             ivu=p_nvu(iudef)
             
             If(structure% p_isrtTOIOisrt(issa) == issau .And.&
                  &(&
                  &   (apply_to_double .And. &
                  &    (basis% p_firstNLv(iv,issa) == basis% p_firstNLv(ivu,issa))&
                  &   ) .Or. &
                  &   (.Not.apply_to_double .And. (iv == ivu))  &
                  & ) )Then
                If(lqnchar(l) /= lsdau% p_state(iudef)(2:2))Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="' state-definition  is corrupted!'")
                Endif
                !! caught a state to be modified
                lsdau% nusite=lsdau% nusite+1_t_int
                lsdau% p_udefs(lsdau% nusite)=iudef
                lsdau% p_lvsu(lsdau% nusite)=l
                lsdau% p_degvsu(lsdau% nusite)=deg
                lsdau% p_nvsu(lsdau% nusite)=iv
                Write(lsdau% p_labelsu(lsdau% nusite),99)nmain,lqnchar(l)
99              Format(i1,a1)
                lsdau% p_isiteu(lsdau% nusite)=isa
                lsdau% p_isortu(lsdau% nusite)=issa
                lsdau% p_ubi1(lsdau% nusite)=iz+1_t_int
                lsdau% p_ubi2(lsdau% nusite)=iz+deg
                Exit udefloop
             End If
          End Do udefloop
          iz=iz+deg
       End Do
    End Do

    !!




15  Format('LSDA+U: site  el.    udef state ubi1 ubi2')
    Write(outfile,1)
    Write(outfile,15)
    Write(outfile,1)
    Do isau=1_t_int,lsdau% nusite
       iudef=lsdau% p_udefs(isau)
       isa=lsdau% p_isiteu(isau)
       issa=lsdau% p_isortu(isau)
17     Format('LSDA+U: ',i5,1x,a2,2x,i5,4x,a2,2(1x,i5))
       Write(outfile,17)&
            & isa,structure% p_elements(issa),iudef&
            &,lsdau% p_labelsu(isau)&
            &,lsdau% p_ubi1(isau),lsdau% p_ubi2(isau)
    Enddo

    Write(outfile,1)




18  Format('LSDA+U: iteration vector dim.   : ',i5)

    Inquire(file='=.lsdau',exist=exists)
    If(exists)Then
       Open(12,file='=.lsdau')
       Read(12,*)
       Read(12,*)checkspinrep,diagonalonly
       Close(12)
       Print *,'checkspinrep=',checkspinrep
       Print *,'diagonalonly=',diagonalonly
    Endif
    


    lsdau% n_u_it=0_t_int
    Do isau=1,lsdau% nusite
       deg=lsdau% p_degvsu(isau)
       lsdau% n_u_it=lsdau% n_u_it+deg**2*nspin
    Enddo
    Write(outfile,18)lsdau% n_u_it
    lsdau% n_u_it=lsdau% n_u_it*2_t_int !! 2 for complex
    If(checkspinrep) lsdau% n_u_it=lsdau% n_u_it+Size(indices)*lsdau% nusite !! N,M,Lz...

    !!
    If(rel)Then
       !!
       Call palloc('lsdau% p_T',lsdau% p_T,1_t_int,lsdau% rumaxdim&
            &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite&
            &,nmodule,nsubfunc)
       !!
       Do isau=1,lsdau% nusite
          l=lsdau% p_lvsu(isau)
          deg=2*l+1
          !!
          i0=-2*l
          kap=l+deg
          Do k=1,2
             kap=kap-deg
             i0=i0+2*l
             If(kap==0) Cycle
             Do mu=1,2*Abs(kap)
                Do ms=1,2
                   Do m=1,deg
                      lsdau% p_T(m+deg*(ms-1),mu+i0,isau)&
                           &=basis% p_relTOpseudo(m,mu,ms,kap)
                   Enddo
                Enddo
             Enddo
          Enddo
          !Call cprintmat('n',"('(',f8.5,',',f8.5,')')","('         0         ')"&
          !     &,lsdau% p_T(:,:,isau),1.0e-6_t_real)
       Enddo
    Endif
    !!
    !!
    !! allocate lsdau matrix
    Call palloc('lsdau% p_dmat',lsdau% p_dmat,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite,1_t_int,nspin &
         &,nmodule,nsubfunc)
    
    Call palloc('lsdau% p_vmat',lsdau% p_vmat,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite,1_t_int,nspin &
         &,nmodule,nsubfunc)
    
    lsdau% p_dmat(1:lsdau% rumaxdim,1:lsdau% rumaxdim,1:lsdau% nusite,1:nspin)=c0
    lsdau% p_vmat(1:lsdau% rumaxdim,1:lsdau% rumaxdim,1:lsdau% nusite,1:nspin)=c0
    !!
    !!
    Call palloc('fac',fac,0_t_int,2*lsdau% lmx,nmodule,nsubfunc)
    Call palloc('wu',wu,1_t_int,(2*lsdau% lmx+1)**2,nmodule,nsubfunc)
    Call palloc('lsdau% p_Dlquant',lsdau% p_Dlquant,1_t_int,lsdau% umaxdim&
         &,1_t_int,lsdau% umaxdim,1_t_int,lsdau% nusite,nmodule,nsubfunc)
    !!
    lsdau% axis=quant% axis
    Call make_factorials(fac,2*lsdau% lmx)
    Call reuler(lsdau% axis,eulang)
    !! calc rotation matrices
    Do isau=1,lsdau% nusite
       l=lsdau% p_lvsu(isau)
       deg=2*l+1
       Call ytr(eulang,0_t_int,fac,l,wu(:),.True.)
       Do m=1,deg   
          Do n=1,deg   
             lsdau% p_Dlquant(m,n,isau)=wu(m+deg*(n-1))
          Enddo
       Enddo
    Enddo
    !!
    !!
    Call pfree(p_nvu)
    Call pfree(fac)
    Call pfree(wu)
    !!
    Return
    !!
  End Subroutine io_init_lsdau
  !!***********************************************************************

  Subroutine check_lsdau_version(lsdau)
    !!
    Implicit None  
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau 
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_lsdau_version"
    !!
    If (( lsdau% functional < 1_t_int ) &
         &.Or. ( lsdau% functional > 3_t_int )) Then
1      Format("'Wrong LSDA+U functional version: ",i4," ! '")
       Write(errbuf,1) lsdau% functional
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    Endif
    If (( lsdau% projection < 1_t_int ) &
         &.Or. ( lsdau% projection > 4_t_int )) Then
2      Format("'Wrong LSDA+U projection version: ",i4," ! '")
       Write(errbuf,2) lsdau% projection
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    Endif
  End Subroutine check_lsdau_version
  !!***********************************************************************
  !!
  !! calculate vv=C or vv=SC which enters the density matrix expression
  !! n=vv*vv^+  for the correlated orbitals
  !!
  Subroutine lsdau_calc_u_cs(lsdau,sp,zz,nvdim,nband,ms,ik,kintest)
    !! to be executed at every k-point    
    !!-----------------------------------------------------------------------
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Logical(t_log),Intent(in)    :: kintest
    Integer(t_int),Intent(in)    :: nvdim,ms,ik
    Integer(t_int),Intent(in)    :: nband
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim) ! eigenvectors
    Complex(t_real),Intent(in)   :: sp(nvdim,nvdim) ! overlap matrix
    !! if projection=LSDAU_HYBRIDIZED_CORE
    !!    vv=S*zz
    !! if projection=LSDAU_ORTHOGONAL
    !!    vv=zz
    ! !if projection=LSDAU_GROSS
    !!    vv=S*zz
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_calc_u_cs"
    Integer(t_int) :: isau,ib1,ib2,ib,deg
    !!
    If(.Not. lsdau% lsdau ) Return
    !!
    If (   lsdau% projection /= LSDAU_ORTHOGONAL .And. &
         & lsdau% projection /= LSDAU_GROSS ) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized LSDA+U projection no longer supported !'")
    Endif
    !!
    Do isau=1,lsdau% nusite
       ib1=lsdau% p_ubi1(isau) !! start index of u-block
       ib2=lsdau% p_ubi2(isau) !! end   index of u-block
       deg=lsdau% p_degvsu(isau)
       
       !! lsdau% projection == LSDAU_ORTHOGONAL 
       Do ib=1,nband
          lsdau% p_cvv(1:deg,isau,ib,ik,ms)=zz(ib1:ib2,ib)
       Enddo
       
       If (lsdau% projection == LSDAU_GROSS ) Then
          !!
          Do ib=1,nband
             lsdau% p_Scvv(1:deg,isau,ib,ik,ms)=&
                  &Matmul(sp(ib1:ib2,1:nvdim),zz(1:nvdim,ib))
          Enddo
       Endif
       
    Enddo
    !!
    !! ENERGY TESTS
    If(kintest) Call lsdau_calc_u_hcc(lsdau,zz,nvdim,nband,ms,ik)
    !!
    Return
    !!
  End Subroutine lsdau_calc_u_cs
  !!***********************************************************************
  !!
  !! calc C*HPU*C^(+) for test reasons
  !!
  Subroutine lsdau_calc_u_hcc(lsdau,zz,nvdim,nband,ms,ik)
    !! to be executed at every k-point    
    !!-----------------------------------------------------------------------
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Integer(t_int),Intent(in)    :: nvdim,ms,ik
    Integer(t_int),Intent(in)    :: nband
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim) ! eigenvectors
    !!LOCAL
    Integer(t_int) :: ib,ii
    Real(t_real)   :: d
    !!
    If(.Not. lsdau% lsdau ) Return
    !!
    Do ib=1_t_int,nband
       d=0.0_t_real
       Do ii=1,nvdim
          d=d+Real(Dot_product(zz(1:nvdim,ib),lsdau% p_hpu(1:nvdim,ii))*zz(ii,ib),t_real)
       Enddo
       lsdau% p_hh(ib,ik,ms)=d
    Enddo
    !!
  End Subroutine lsdau_calc_u_hcc
  !!***********************************************************************
  !! for tests calc kinetic contrib 
  Subroutine lsdau_calc_u_kin(lsdau,nspin,nband,nkp,bweitk)
    !!
    Implicit None  
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Integer(t_int),Intent(in)    :: nspin,nkp,nband
    Real(t_real),Intent(in)      :: bweitk(nband,nkp,nspin)
    !!LOCAL
    Integer(t_int) :: ms,ikp,ib
    Real(t_real)   :: d,de
    !!
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    d=0.0_t_real
    Do ms=1_t_int,nspin
       Do ikp=1_t_int,nkp
          Do ib=1_t_int,nband
             d=d+lsdau% p_hh(ib,ikp,ms)*bweitk(ib,ikp,ms)
          End Do
       End Do
    End Do
    If(lsdau% functional == LSDAU_AMF)Then
       de=2.0_t_real*lsdau% eeU
    Else
       de=2.0_t_real*lsdau% eeU-lsdau% eeD
    Endif
1   Format('LSDA+U: kinetic correction : ',2(e30.15e3,2x))
    Write(outfile,1)d ,de
    
    !!
  End Subroutine lsdau_calc_u_kin
  !!***********************************************************************
  !! sum up and integrate the density matrix contributions
  !! ----------------------------------------------------------------------
  !!  Inputs:
  !!    bweitk:bandweights
  !!    nvdim :number of valence orbitals
  !!    nkp     :number of irreducible k-points
  !!    mspin   :spin channels
  !!    vv      :matrix vv_{in}(k)= C
  !!             n - band index; i - orbital index
  !!             c^_i - LO coeff. of i-th orbital in n-th KS state
  !!    Svv      :matrix vv_{in}(k)=SC
  !!             n - band index; i - orbital index
  !!             c^_i - LO coeff. of i-th orbital in n-th KS state
  !!  Outputs:
  !!    dmat    :dmat_{ij}=\sum_{kn}^{occ} vmat_{in}(k)^*.vmat_{nj}(k)
  !! ----------------------------------------------------------------------
  !!    if mspin==1 dmat will contain half the occupation, it is dmat for
  !!    one of the two degenerate spin directions
  !!
  Subroutine lsdau_calc_u_dmat(symmetry,equivsites,elements,lsdau,rel,nspin,mspin&
       &,nband,nkp&
       &,bweitk,kintest)
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Type(T_Symmetry),Intent(in)  :: symmetry
    Character(2),Intent(in)      :: elements(:)
    Logical(t_log),Intent(in)    :: kintest,rel
    Integer(t_int),Intent(in)    :: equivsites(48,*)
    Integer(t_int),Intent(in)    :: nspin,mspin,nkp,nband
    Real(t_real),Intent(in)      :: bweitk(nband,nkp,nspin)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_calc_u_dmat"
    Integer(t_int) :: ib,ikp,j,i,isau,ms,deg
    Real(t_real)   :: wt,fspin
    Complex(t_real),Pointer :: ldmat(:,:,:,:) 
    Complex(t_real),Pointer :: ldmat2(:,:,:,:) 
    Logical(t_log) :: fbz,prtmat
    Complex(t_real):: ct
    !!
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    If( lsdau% dmat_mode /= LSDAU_DMAT_MODE_SC ) Then
       Write(errbuf,"('''LSDA+U Occupation matrix is kept fixed on user request.''&
            &,/,'' THIS IS NOT A FULLY SELF CONSISTENT CALCULATION!''')") 
       Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       Call print_u_dmat(mspin,elements,lsdau,rel,lsdau% p_dmat,'dmat')
       Return
    Endif
    !!
    Nullify(ldmat)
    Nullify(ldmat2)
    !!
    prtmat=.true.
    !!
    Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
    Call palloc('ldmat2',ldmat2,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
    
    !! fullbz
    fbz=isoption('FULLBZ')
    !!
    fspin=3.0_t_real-Real(mspin,t_real)
    if(rel) fspin=1.0_t_real
!1   Format('LSDA+U: ------------sumdens ------------')
    !Write(outfile,1)
    !!
    ldmat(:,:,:,:)=c0
    !!
    !! --- loop over spins and correlated orbitals
    !!
    Do ms=1_t_int,nspin
       Do isau=1_t_int,lsdau% nusite
          deg=lsdau% p_degvsu(isau)
          Do i=1,deg
             Do j=1,deg
                ct=c0
                !! --- sum over bands and k-points
                If (lsdau% projection == LSDAU_ORTHOGONAL ) Then
                   Do ikp=1_t_int,nkp
                      Do ib=1_t_int,nband
                         wt=bweitk(ib,ikp,ms)
                         ct=ct+wt*lsdau% p_cvv(i,isau,ib,ikp,ms)&
                              &*Conjg(lsdau% p_cvv(j,isau,ib,ikp,ms))
                      Enddo ! ib
                   Enddo ! ikp
                Else If (lsdau% projection == LSDAU_GROSS ) Then
                   Do ikp=1_t_int,nkp
                      Do ib=1_t_int,nband
                         wt=bweitk(ib,ikp,ms)
                         ct=ct+wt*lsdau% p_Scvv(i,isau,ib,ikp,ms)&
                              &*Conjg(lsdau% p_cvv(j,isau,ib,ikp,ms))
                              !&+wt*lsdau% p_cvv(i,isau,ib,ikp,ms)&
                              !&*Conjg(lsdau% p_Scvv(j,isau,ib,ikp,ms)))*0.5_t_real
                      Enddo ! ib
                   Enddo ! ikp
                Endif
                !! --- accumulate the density matrix elements
                ct=ct/fspin
                If (i /= j) Then
                   ldmat(i,j,isau,ms)= ct
                Else
                   ldmat(i,i,isau,ms)= Cmplx(Real(ct,t_real),0.0_t_real,t_real)
                   
                Endif
             Enddo ! j
          Enddo ! i
          !!
          Do i=1,deg
             Do j=i+1,deg
                ct=(ldmat(i,j,isau,ms)+Conjg(ldmat(j,i,isau,ms)))*0.5_t_real
                ldmat(i,j,isau,ms)=ct
                ldmat(j,i,isau,ms)=Conjg(ct)
             Enddo
          Enddo
          !!
          If(.False.)Then
             Call check_hermes(deg,isau,ldmat)!!todo: remove
          Endif
          !!
       Enddo ! isau
    Enddo ! ms
    !!
    !!
    !! in: ldmat
    If(rel.And.diagonalonly)Then
       Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,mode='tospin')
       
       Do isau=1,lsdau% nusite
          deg=2_t_int*lsdau% p_lvsu(isau)+1_t_int
          ldmat(1:deg,deg+1:2*deg,isau,1)=c0
          ldmat(deg+1:2*deg,1:deg,isau,1)=c0
       Enddo ! isau

       Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,mode='fromspin')
    Endif
    !!
    !!
    !! make symmetrization to correct for irreducible brilloun zone
    !! integration
    !!
    If(.Not.fbz)Then

       Call force_u_mat_sym(symmetry,equivsites,lsdau,rel,mspin,ldmat,ldmat2)

    Else
       ldmat2=ldmat
    Endif
    !!
    !! force hermitization of numerical noice
    Call force_u_hermit(lsdau,nspin,ldmat2)
    !!
    !!
    !! and print matrices 
    If(prtmat)Then
       Call print_u_dmat(mspin,elements,lsdau,rel,ldmat2,'dmat')
    Endif
    !!
    lsdau% p_dmat(:,:,:,:)=ldmat2(:,:,:,:)
    
    !!
    Call pfree(ldmat)
    Call pfree(ldmat2)
    !!
    If(kintest) Call lsdau_calc_u_kin(lsdau,nspin,nband,nkp,bweitk)

    
  End Subroutine lsdau_calc_u_dmat
  !!***********************************************************************
  Subroutine print_u_dmat(mspin,elements,lsdau,rel,dmati,which,lunit1,mode)
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: which
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Character(2),Intent(in)      :: elements(*)
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    Integer,Intent(in),Optional :: lunit1
    Character(*),Intent(in),Optional :: mode
    Complex(t_real),Intent(in),Target   :: dmati(:,:,:,:) 
    !!LOCAL
    Character(*),Parameter::nsubfunc="print_u_dmat"
    Integer(t_int) :: ms,isau,i,j,degl,l,msp,ns,i0,i1,mms,llunit
    Integer(t_int) :: nspin,deg
    Logical(t_log) :: tofile
    Real(t_real)   :: d,spmom,Lmom(3),Jzmom,charge
    Character(13)  :: updn(3)
    Character(10)  :: lmode
    Real(t_real),Pointer :: charge_s(:,:),p_bb(:)
    Complex(t_real),Pointer::  p_Uylm(:,:,:)
    Complex(t_real),Pointer::  p_vv(:,:) 
    Complex(t_real),Pointer::  ldmat(:,:,:,:)
    Complex(t_real),Pointer::  psmat(:,:,:,:)
    Complex(t_real),Pointer::  p_Bop(:,:,:,:,:)
    
    Data updn(1) /'up'/
    Data updn(2) /'down'/
    Data updn(3) /'mixed(updown)'/
    !!
    Nullify(p_Uylm,p_vv,ldmat,charge_s,p_Bop,p_bb)
    !!
    nspin=mspin
    If(rel)nspin=1
    llunit=outfile
    tofile=.False.
    If(Present(lunit1))Then
       llunit=lunit1
       tofile=.True.
    Endif
    lmode='tospin'
    If(Present(mode))lmode=mode
    !!
    Call palloc('p_Uylm',p_Uylm&
         &,1_t_int,2*lsdau% lmx+1,1_t_int,2*lsdau% lmx+1&
         &,0_t_int,lsdau% lmx,nmodule,nsubfunc)
    !!
    Call  make_ylm_U_matrix(lsdau% lmx,p_Uylm)
    !!
    !!
    If(rel) Then
       msp=3
       Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
            &,1_t_int,lsdau% nusite,1_t_int,1_t_int,nmodule,nsubfunc)
       psmat => ldmat
       psmat(:,:,:,1)=dmati(:,:,:,1)
       Call lsdau_d_mat_trans_spin(lsdau,rel,psmat,lmode)
    Else
       msp=mspin
       If(tofile) msp=2
       psmat => dmati
    Endif
    

    !!
    If(.Not.tofile)Then
       Write(llunit,1)
       If(Trim(which)=="dmat")Then
          Write(llunit,210)
       Else If(Trim(which)=="vmat")Then
          Write(llunit,211)
       Endif
       Write(llunit,1)
    Endif
    !!
    Call palloc('charge_s',charge_s,1_t_int,3_t_int,1_t_int,lsdau% nusite&
         &,nmodule,nsubfunc)
    !!
    If(.Not.tofile.And.(Trim(which)=="dmat"))Then
       Write(llunit,22)lsdau% axis
    Endif
    !!
    Call dmat_decom(nspin,mspin,lsdau,rel,p_Bop)
    !!
    isauloop: Do isau=1_t_int,lsdau% nusite
       l=lsdau% p_lvsu(isau)
       deg=lsdau% p_degvsu(isau)
       degl=2_t_int*l+1_t_int
       !!
       Call decom_b(nspin,deg,isau,psmat,p_Bop,p_bb)
       !!
       charge=p_bb(1)
       spmom =p_bb(2)
       Lmom(1)=p_bb(5)
       Lmom(2)=p_bb(6)
       Lmom(3)=p_bb(3)
       Jzmom =p_bb(4)
       !!
       charge_s(1,isau)=(p_bb(1)+p_bb(2))*0.5_t_real
       charge_s(2,isau)=(p_bb(1)-p_bb(2))*0.5_t_real
       If(rel) charge_s(3,isau)=Real(trace(psmat(:,:,isau,1),0_t_int,degl    ,degl),t_real)
       !!
       If(.Not.tofile.And.(Trim(which)=="dmat"))Then
          Write(llunit,24)&
               & lsdau% p_isiteu(isau),elements(lsdau% p_isortu(isau))&
               &,lsdau% p_labelsu(isau),charge&
               &,spmom,Lmom(1:3) ,Jzmom
          !!
       Endif
    Enddo isauloop
    If(.Not.tofile.And.(Trim(which)=="dmat"))Then
       Write(llunit,1)
       Write(llunit,1)
    Endif
    !!
    isauloop2: Do isau=1_t_int,lsdau% nusite
       l=lsdau% p_lvsu(isau)
       degl=2_t_int*l+1_t_int
       
       Call palloc('p_vv',p_vv,1_t_int,degl,1_t_int,degl,nmodule,nsubfunc)
      
       !!
       !!
       nsloop: Do ns=1_t_int,msp
          mms=ns
          ms=ns
          i0=0
          If(rel)Then
             i0=degl*(ms-1)
             mms=1
          Endif
          i1=i0
          If(ns==3) Then
             ms=12
             i0=0
             i1=degl
          Endif
          !!
          !! print matrices
          !!
          tofileif:If(tofile)Then
             Write(llunit,15)
             Write(llunit,12)elements(lsdau% p_isortu(isau))&
                  &,lsdau% p_isiteu(isau)&
                  &,lsdau% p_labelsu(isau),Trim(updn(ns))
             !!
             !!
             Write(llunit,13)
             Do i=1,degl
                Do j=1,degl
                   If(mspin==2)Then
                      d=Real(psmat(i+i0,j+i1,isau,mms),t_real)
                   Else
                      d=Real(psmat(i+i0,j+i1,isau,1),t_real)
                   Endif
                   Write(llunit,27,advance='no') d 
                Enddo
                Write(llunit,3)
             Enddo
             !!
             Write(llunit,3)
             Write(llunit,14)
             !!
             Do i=1,degl
                Do j=1,degl
                   If(mspin==2)Then
                      d=Aimag(psmat(i+i0,j+i1,isau,mms))
                   Else
                      d=Aimag(psmat(i+i0,j+i1,isau,1))
                   Endif
                   Write(llunit,27,advance='no') d
                Enddo
                Write(llunit,3)
             Enddo
             Write(llunit,3)
             Write(llunit,3)
             !!             
          Else tofileif
             
             Write(llunit,2)lsdau% axis
             Write(llunit,4)&
                  & lsdau% p_isiteu(isau),elements(lsdau% p_isortu(isau))&
                  &,lsdau% p_labelsu(isau),ms,charge_s(ns,isau)
             Write(llunit,1115)
             Do i=1_t_int,degl
                Write(llunit,6,advance='no')
                Do j=1_t_int,degl
                   Write(llunit,5,advance='no') &
                        & Real(psmat(i+i0,j+i1,isau,mms),t_real) &
                        &,Aimag(psmat(i+i0,j+i1,isau,mms))
                End Do
                Write(llunit,3)
             End Do
             Write(llunit,6)

             Write(llunit,1116)
             p_vv=Matmul(Transpose(Conjg(p_Uylm(1:degl,1:degl,l)))&
                  &,Matmul(psmat(i0+1:i0+degl,i1+1:i1+degl,isau,mms),p_Uylm(1:degl,1:degl,l)))
             
             Do i=1_t_int,degl
                Write(llunit,6,advance='no')
                Do j=1_t_int,degl
                   Write(llunit,5,advance='no') Real(p_vv(i,j),t_real),Aimag(p_vv(i,j))
                End Do
                Write(llunit,3)
             End Do
             Write(llunit,6)
          Endif tofileif
          
       Enddo nsloop
    Enddo isauloop2
    !!
    If(.Not. tofile) Write(llunit,1)
    !!
    Call pfree(p_Bop)
    Call pfree(p_bb)
    Call pfree(p_vv)
    Call pfree(p_Uylm)
    Call pfree(ldmat)
    Call pfree(charge_s)
    !!
    !!
    Return
    !!
210 Format('LSDA+U: ------------density matrix  --------------------------------------')
211 Format('LSDA+U: ------------potential matrix--------------------------------------')
1   Format('LSDA+U: ',66('-'))
!11  Format('LSDA+U:       ',50('-'))
2   Format(&
         &'LSDA+U: spin    z-axis: (',2(f12.5,','),f12.5,')',/&
         &'LSDA+U: orbital z-axis: global',/&
         &,'LSDA+U: site   el.  state  spin trace')
22  Format('LSDA+U:       ',50('-'),/,&
         &'LSDA+U:        z-axis: (',2(f12.5,','),f12.5,')',/,&
         &'LSDA+U:       ',50('-'),/,&
         &'LSDA+U: site   el.  state  charge    spinmoment     Lx        Ly        Lz         Jz')
3   Format()
4   Format('LSDA+U: ',i5,2x,a2,5x,a2,4x,i2,2x,f8.5)
24  Format('LSDA+U: ',i5,2x,a2,5x,a2,4x,f8.5,2x,f8.5,2x,'(',3(f8.5,2x),')',f8.5)
1115 Format('LSDA+U: real harmonics')
1116 Format('LSDA+U: complex harmonics')
6   Format('LSDA+U: ')
15  Format("# Element site  state spin")
12  Format(a2,1x,i6,1x,a2,1x,a)
13  Format("# real part")
14  Format("# imaginary part")
5   Format('(',f8.4,',',f8.4,') ')
27  Format(f15.10,' ')
    !!
  End Subroutine print_u_dmat
  !!***********************************************************************
  Subroutine dmat_decom(nspin,mspin,lsdau,rel,Bop)
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau
    Integer(t_int),Intent(in)    :: nspin,mspin
    Logical(t_log),Intent(in)    :: rel
    Complex(t_real),Pointer:: Bop(:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="dmat_decom"
    Integer(t_int) :: i,j,m,ns,i0,mb,degl,l,ms,isau,deg,dim,n,si
    Complex(t_real):: cdx,cdy,cd
    Complex(t_real),Pointer::  p_Uylm(:,:,:),DU(:,:)
    !!
    Nullify(p_Uylm,DU)
    !!
    dim=6
    !!
    Call palloc('p_Uylm',p_Uylm,1_t_int,lsdau% umaxdim,1_t_int,lsdau% umaxdim&
         &,0_t_int,lsdau% lmx,nmodule,nsubfunc)
    !!
    Call  make_ylm_U_matrix(lsdau% lmx,p_Uylm)
    !!
    Call palloc('Bop',Bop,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,nspin,1_t_int,dim,1_t_int,lsdau% nusite,nmodule,nsubfunc)
    !!
    Bop(:,:,:,:,:)=c0
    Do isau=1,lsdau% nusite
       l=lsdau% p_lvsu(isau)
       deg=lsdau% p_degvsu(isau)
       degl=2*l+1
       !!
       Call palloc('DU',DU,1_t_int,degl,1_t_int,degl,nmodule,nsubfunc)
       DU(1:degl,1:degl)=&
            !&Matmul(Transpose(Conjg(p_Uylm(1:degl,1:degl,l))),&
            &Matmul(lsdau% p_Dlquant(1:degl,1:degl,isau),p_Uylm(1:degl,1:degl,l))!)
       !!
       !Call printmat('U',"(f12.5,f12.5)","('      0                 ')"&
       !  &,p_Uylm(1:degl,1:degl,l))
       !Call printmat('Dl',"(f12.5,f12.5)","('      0                 ')"&
       !  &,lsdau% p_Dlquant(1:degl,1:degl,isau))
       !Call printmat('DU',"(f12.5,f12.5)","('      0                 ')"&
       !  &,DU(1:degl,1:degl))
       !!
       reliflms: If(rel)Then
          Do i=1,deg
             !! N
             Bop(i,i,1,1,isau)=c1
             !! 2*Sz
             If(i<=degl)Then
                Bop(i,i,1,2,isau)=c1
             Else
                Bop(i,i,1,2,isau)=-c1
             Endif
          Enddo
          !! Lz in quant axis
          Do i=1,degl
             Do j=1,degl
                Do ns=0,1
                   i0=degl*ns
                   Do m=1,degl
                      mb=degl+1-m
                      Bop(i0+i,i0+j,1,3,isau)=Bop(i0+i,i0+j,1,3,isau)&
                           &-ci*lsdau% p_Dlquant(i,mb,isau)&
                           &*lsdau% p_Dlquant(j,m,isau)*(m-l-1)
                   Enddo
                Enddo
             Enddo
          Enddo
          !! Lx,Ly in quant axis
          Do i=1,degl
             Do j=1,degl
                Do ns=0,1
                   i0=degl*ns
                   !!     
                   cdx=c0
                   cdy=c0
                   Do m=1,degl
                      Do n=1,degl
                         Do si=-1,1,2
                            If(m==n+si)Then
                               cd=Conjg(DU(i,n))&
                                    &*Sqrt(l*(l+1.0_t_real)-(n-l-1)*((n-l-1)+si))*0.5_t_real&
                                    &*DU(j,m)
                               cdx=cdx+cd
                               cdy=cdy-ci*si*cd
                            Endif
                         Enddo
                      Enddo
                   Enddo
                   Bop(i0+i,i0+j,1,5,isau)=cdx
                   Bop(i0+i,i0+j,1,6,isau)=cdy
                End Do
             End Do
          End Do
          

          !! Jz
          Bop(:,:,1,4,isau)=Bop(:,:,1,2,isau)*0.5_t_real+Bop(:,:,1,3,isau)




       Else reliflms
          Do ms=1,mspin
             Do i=1,degl
                Bop(i,i,ms,1,isau)=c1*(3.0_t_real-mspin)
                Bop(i,i,ms,2,isau)=c1*(3.0_t_real-2.0_t_real*ms)*(mspin-1.0_t_real)
             Enddo
          Enddo
          Do i=1,degl
             Do j=1,degl
                Do m=1,degl
                   mb=degl+1-m
                   cd=-ci*lsdau% p_Dlquant(i,mb,isau)&
                        &*lsdau% p_Dlquant(j,m,isau)*(m-l-1)&
                        &*(3.0_t_real-mspin)
                   Do ms=1,mspin
                      Bop(i,j,ms,3,isau)=Bop(i,j,ms,3,isau)+cd
                   Enddo
                Enddo
             Enddo
          Enddo

          !! Lx,Ly in quant axis
          Do i=1,degl
             Do j=1,degl
                !!     
                cdx=c0
                cdy=c0
                Do m=1,degl
                   Do n=1,degl
                      Do si=-1,1,2
                         If(m==n+si)Then
                            cd=Conjg(DU(i,n))&
                                 &*Sqrt(l*(l+1.0_t_real)-(n-l-1)*((n-l-1)+si))*0.5_t_real&
                                 &*DU(j,m)
                            cdx=cdx+cd
                            cdy=cdy-ci*si*cd
                         Endif
                      Enddo
                   Enddo
                Enddo
                Do ms=1,mspin
                   Bop(i,j,ms,5,isau)=cdx
                   Bop(i,j,ms,6,isau)=cdy
                End Do
             End Do
          End Do

          Do ms=1,mspin
             Bop(:,:,ms,4,isau)=Bop(:,:,ms,2,isau)*0.5_t_real+Bop(:,:,ms,3,isau)
          Enddo

       Endif reliflms
       !!
       
!!$    Call printmat('B1',"(f12.5,f12.5)","('      0                 ')"&
!!$         &,Bop(:,:,1,1,isau))
!!$    Call printmat('B1',"(f12.5,f12.5)","('      0                 ')"&
!!$         &,Bop(:,:,nspin,1,isau))
!!$    Call printmat('B2',"(f12.5,f12.5)","('      0                 ')"&
!!$         &,Bop(:,:,1,2,isau))
!    Call printmat('B3',"(f12.5,f12.5)","('      0                 ')"&
!         &,Bop(1:degl,1:degl,1,3,isau))
!    Call printmat('B5',"(f12.5,f12.5)","('      0                 ')"&
!         &,Bop(1:degl,1:degl,1,5,isau))
!!$    Call printmat('B3',"(f12.5,f12.5)","('      0                 ')"&
!!$         &,Bop(:,:,nspin,3,isau))
    
       
    Enddo
    !!
    Call pfree(p_Uylm)
    Call pfree(DU)
    !!
    Return
    !!
  End Subroutine dmat_decom
  !!***********************************************************************
  Subroutine decom_S_b(nspin,deg,indices,Bop,bb)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: indices(:),nspin,deg
    Complex(t_real),Intent(in)   :: Bop(:,:,:,:)
    Real(t_real),Intent(inout)   :: bb(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="decom_S_b"
    Integer(t_int) :: dim,ms,a1,a2,i,aa1,aa2
    Complex(t_real):: sumlog
    Real(t_real)   :: suminf
    Logical(t_log) :: sing,indef,infinite
    Complex(t_real),Pointer:: p_BS(:,:)
    !!
    Nullify(p_BS)
    !!
    dim=Size(indices)
    Call palloc('p_BS',p_BS,1_t_int,dim,1_t_int,dim,nmodule,nsubfunc)
    !!
    p_BS=c0
    Do ms=1,nspin
       Do a1=1,dim
          aa1=indices(a1)
          Do a2=1,dim
             aa2=indices(a2)
             Do i=1,deg
                p_BS(a1,a2)=p_BS(a1,a2)&
                     &+Dot_product(Bop(:,i,ms,aa1),Bop(:,i,ms,aa2))
             Enddo
          Enddo
       Enddo
    Enddo
    !!
    !Call printmat('S',"(f12.5,f12.5)","('      0                 ')",p_BS)
    Call cinv(p_BS,dim,dim,sumlog,sing,infinite,indef,suminf)
    !Call printmat('S',"(f12.5,f12.5)","('      0                 ')",p_BS)
    !!
    !Do a1=1,Size(bb)
    !   Print *,'b=',bb(a1)
    !Enddo
    !!
    bb(:)=Real(Matmul(p_BS(:,:),bb(indices(:))),t_real)
    !!
    !Do a1=1,dim
    !   Print *,'b=',bb(a1)
    !Enddo
    
    Call pfree(p_BS)
    !!
    Return
    !!
  End Subroutine decom_S_b
  !!***********************************************************************
  Subroutine decom_b(nspin,deg,isau,psmat,Bop,bb)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nspin,deg,isau
    Complex(t_real),Intent(in)   :: psmat(:,:,:,:)
    Complex(t_real),Intent(in)   :: Bop(:,:,:,:,:)
    Real(t_real),Pointer         :: bb(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="decom_b"
    Integer(t_int) :: dim,ms,a1,i,j
    !!
    dim=Size(Bop,4)
    Call palloc('bb',bb,1_t_int,dim,nmodule,nsubfunc)
    !!
    bb=0.0_t_real
    Do ms=1,nspin
       Do a1=1,dim
          Do i=1,deg
             Do j=1,deg
                bb(a1)=bb(a1)+Real(psmat(j,i,isau,ms)*Bop(j,i,ms,a1,isau),t_real)
             Enddo
          Enddo
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine decom_b
  !!***********************************************************************
  Subroutine  mmult(a,b,wu,dim,norm,timerev)
    !!
    !! a=a+K0*wu^(+)*b*wu*K0*norm
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: dim
    Logical(t_log),Intent(in)    :: timerev
    Complex(t_real),Intent(in)   :: b(:,:)
    Complex(t_real),Intent(inout):: a(:,:)
    Complex(t_real),Intent(in)   :: wu(:,:)
    Real(t_real),Intent(in)      :: norm
    !!LOCAL
    Character(*),Parameter::nsubfunc="mmult"
    Integer(t_int) :: i,j,k
    Complex(t_real):: ct
    Complex(t_real),Pointer:: v(:)
    !!
    Nullify(v)
    !!
    Call palloc('v',v,1_t_int,dim,nmodule,nsubfunc)
    !!
    Do i=1,dim
       Do k=1,dim
          v(k)=Dot_product(wu(1:dim,i),b(1:dim,k))
       Enddo
       Do j=1_t_int,dim
          ct=Sum(v(1:dim)*wu(1:dim,j))
          If(timerev) ct=Conjg(ct)
          a(i,j)=a(i,j)+ct*norm
       Enddo
    Enddo
    !!
    Call pfree(v)
    !!
  End Subroutine mmult
  !!***********************************************************************
  !!***********************************************************************
  !!
  !! calulate LSDA+U potential from input occupation numver matrix
  Subroutine lsdau_calc_u_potmat(lsdau,rel,elements,nspin,mspin,energy)
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Logical(t_log),Intent(in)    :: energy,rel
    Integer(t_int),Intent(in)    :: nspin,mspin
    Character(2),Intent(in)      :: elements(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_calc_u_potmat"
    Integer(t_int) :: isau,iudef,l,degl,m,ms,msb,msp3,msp2
    Integer(t_int) :: glmax,n,nu,mu,ib0,mms,mmsb,i1,i0,deg2
    Real(t_real)   :: wc,wx,v_dc(2),uc0,ux0,tol,d,dc,charge_s(2),charge
    Real(t_real)   :: d_dc(2),fac,UU,JJ
    Complex(t_real):: cu,co,d_dc12,charge_12,v_dc12,cul(2,2),de(2,2)
    Real(t_real),Pointer   :: wgaunt(:,:,:),MMC(:,:),EM(:)!!todo remove
    Complex(t_real),Pointer::  sdmat(:,:,:,:)
    Complex(t_real),Pointer::  svmat(:,:,:,:)
    Complex(t_real),Pointer::  psmat(:,:,:,:)
    Complex(t_real),Pointer::  pvmat(:,:,:,:)
    Logical(t_log) :: prtmat
    !!
    Nullify(wgaunt,sdmat,svmat,MMC,EM)
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    !!
    tol=1.e-10_t_real
    fac=1.0_t_real
    msp2=mspin
    msp3=mspin
    If(rel) Then 
       msp2=2
       msp3=3
    Else
       If(mspin==1) Then
          fac=2.0_t_real
       Endif
    Endif
    prtmat=.False.
    If(outlevel >= L_many) prtmat=.True.
    !!
    If(lsdau% functional == LSDAU_FIXEDPOT)Then

       If(energy)Then
          lsdau% eeU=0.0_t_real
          lsdau% eeD=0.0_t_real
       End If

       !! TODO: symmetry
       Call force_u_hermit(lsdau,nspin,lsdau% p_vmat)

       isauloop1: Do isau=1_t_int,lsdau% nusite
          deg2=lsdau% p_degvsu(isau)

          spinloop1: Do ms=1,nspin          

             !!
             !!

             !! --- energy calculation
             If(energy)Then


                Do n=1,deg2
                   Do m=1,deg2
                      lsdau% eeD= lsdau% eeD &
                           &-Real(lsdau% p_vmat(n,m,isau,ms)&
                           &*lsdau% p_dmat(m,n,isau,ms),t_real)*fac
                   Enddo ! j
                Enddo ! i
             End If
             !!
          End Do spinloop1
          !!
       Enddo isauloop1
       !!
       Call print_u_dmat(mspin,elements,lsdau,rel,lsdau% p_vmat,'vmat')
       !!
       If(energy) Then
          lsdau% eeU=lsdau% eeD
          Write(outfile,1) lsdau% eeU
          Write(outfile,2) lsdau% eeD
       End If
       !!
       !!
       Return
       !!
    Endif
    !!
    !!
    !!
    !!
    glmax=lsdau% lmx
    Call palloc('wgaunt',wgaunt,1_t_int,(glmax+1)**2&
         &,1_t_int,(glmax+1)**2,1_t_int,(2*glmax+1)**2,nmodule,nsubfunc)

    Call makergaunt(wgaunt,glmax,glmax,2*glmax) 
    !!
    !! make s-rep matrices
    !!
    If(rel)Then
       Call palloc('sdmat',sdmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
            &,1_t_int,lsdau% nusite,1_t_int,1_t_int,nmodule,nsubfunc)
       Call palloc('svmat',svmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
            &,1_t_int,lsdau% nusite,1_t_int,1_t_int,nmodule,nsubfunc)
       psmat=>sdmat
       pvmat=>svmat
       psmat(:,:,:,1)=lsdau% p_dmat(:,:,:,1)
       Call lsdau_d_mat_trans_spin(lsdau,rel,psmat,'tospin')
    Else
       psmat=>lsdau% p_dmat
       pvmat=>lsdau% p_vmat
    Endif
    !!
    !! initalize
    !!
    pvmat(:,:,:,:)=c0

    !!
    If(energy)Then
       lsdau% eeU=0.0_t_real
       lsdau% eeD=0.0_t_real
    End If

    !!
    Do isau=1_t_int,lsdau% nusite
       iudef=lsdau% p_udefs(isau)
       l=lsdau% p_lvsu(isau)
       deg2=lsdau% p_degvsu(isau)
       degl=2_t_int*l+1_t_int  

       UU=lsdau% p_u_j(1_t_int,iudef)
       JJ=lsdau% p_u_j(2_t_int,iudef)

       If(rel) Goto 2000
       
       !! calculate trace of occupation matrix
       If(rel)Then
          charge_s(1)=Real(trace(psmat(:,:,isau,1),0_t_int,0_t_int,degl),t_real)
          charge_s(2)=Real(trace(psmat(:,:,isau,1),degl   ,degl   ,degl),t_real)
          charge_12  =Real(trace(psmat(:,:,isau,1),0_t_int,degl   ,degl),t_real)
          If(diagonalonly)Then
             charge_12=c0
          Endif
       Else
          charge_12=c0
          Do ms=1,mspin
             charge_s(ms)=Real(trace(psmat(:,:,isau,ms),0_t_int,0_t_int,degl),t_real)
          End Do
       Endif


       !! cancel the dmat trace (depending on the functional)

       charge=0.0_t_real       
       d_dc12=c0
       v_dc12=c0
       If (lsdau% functional == LSDAU_AMF ) Then
          !!
          d_dc(1:msp2)=charge_s(1:msp2)/degl
          d_dc12=charge_12/degl
          v_dc=0.0_t_real
          v_dc12=c0
          !!
       Else If (lsdau% functional == LSDAU_AL ) Then
          !!
          d_dc(1:2)=0.0_t_real
          d_dc12=c0
          charge=Sum(charge_s(1:msp2))*fac
          !!
          v_dc(1:msp2)=UU*(charge-0.5_t_real)-JJ*(charge_s(1:msp2)-0.5_t_real)
          If(rel) v_dc12=-JJ*charge_12
       Endif

       
       !! --- compute the upot matrix


       spinloop: Do ms=1_t_int,msp2
          !!
          !! v_(s,n,m)= sum_(t,mu,nu) n(t,mu,nu) [ (n,nu|w|m,mu)-dst (n,nu|w|mu,m) ]
          !!
          i0=0
          i1=0
          ib0=0
          msb=1_t_int+mspin-ms
          mms=ms
          mmsb=msb
          If(rel)Then
             i0=degl*(ms-1)
             i1=degl*(2-ms)
             mms=1
             mmsb=1
             msb=3-ms
             ib0=degl*(msb-1)
          Endif
          !!
          nloop: Do n=1_t_int,degl
             mloop: Do m=1_t_int,degl
                cu=c0
                co=c0
                uc0=0.0_t_real
                ux0=0.0_t_real
                Do mu=1_t_int,degl
                   Do nu=1_t_int,degl
                      !!
                      Call coul_matel(l,n,m,nu,mu,wgaunt,lsdau% p_u_f(0:,iudef),wc,wx)
                      !!
                      If(mu == nu )Then
                         cu=cu+  wc *(psmat(ib0+mu,ib0+mu,isau,mmsb) -d_dc(msb)) &
                              &+(wc-wx)*(psmat(i0+mu,i0+mu,isau,mms) -d_dc(ms))
                         !!
                         If(rel) co=co-wx*(psmat(i0+mu,i1+nu,isau,mms)-d_dc12)
                         !!
                         !! contraction of matrix elements for tests
                         uc0=uc0+  wc    
                         ux0=ux0+  wx
                      Else
                         cu=cu+  wc *psmat(ib0+mu,ib0+nu,isau,mmsb) &
                              &+(wc-wx)*psmat(i0+mu,i0+nu,isau,mms)
                         !!
                         If(rel) co=co-wx*psmat(i0+mu,i1+nu,isau,mms)
                      Endif
                      !!
                   Enddo ! nu
                Enddo ! mu
                !!
                If(diagonalonly) co=c0
                !!
                If (n /= m) Then
                   pvmat(i0+n,i0+m,isau,mms)= cu
                   If(rel) pvmat(i0+n,i1+m,isau,mms)= co
                Else
                   pvmat(i0+n,i0+n,isau,mms)=&
                        &Cmplx(Real(cu-v_dc(ms),t_real),0.0_t_real,t_real)
                   If(rel) pvmat(i0+n,i1+n,isau,mms)=co-v_dc12
                Endif
                !!
                Call check_contractions(n,m,l,uc0,ux0,UU,JJ,tol)
                !!
             Enddo mloop
          Enddo nloop

       Enddo spinloop

       !Goto 1000
2000   Continue
       relif:If(rel)Then
          !! calculate trace of occupation matrix
          charge_s(1)=Real(trace(psmat(:,:,isau,1),0_t_int,0_t_int,degl),t_real)
          charge_s(2)=Real(trace(psmat(:,:,isau,1),degl   ,degl   ,degl),t_real)
          charge_12  =Real(trace(psmat(:,:,isau,1),0_t_int,degl   ,degl),t_real)
          
          If(diagonalonly)Then
             charge_12=c0
          Endif
          
          !! cancel the dmat trace (depending on the functional)
          
          charge=0.0_t_real       
          v_dc12=c0
          d_dc12=c0
          If (lsdau% functional == LSDAU_AMF ) Then
             !!
             d_dc(1:2)=charge_s(1:2)/degl
             d_dc12=charge_12/degl
             v_dc=0.0_t_real
             v_dc12=c0
             !!
          Else If (lsdau% functional == LSDAU_AL ) Then
             !!
             d_dc(1:2)=0.0_t_real
             d_dc12=c0
             charge=Sum(charge_s(1:2))
             !!
             v_dc(1)=UU*(charge-0.5_t_real)-JJ*(charge_s(1)-0.5_t_real)
             v_dc(2)=UU*(charge-0.5_t_real)-JJ*(charge_s(2)-0.5_t_real)
             v_dc12=                       -JJ*charge_12
          Endif
          
          
          !! --- compute the upot matrix
          
          
          !!
          !! v_(s,n,m)= sum_(t,mu,nu) n(t,mu,nu) [ (n,nu|w|m,mu)-dst (n,nu|w|mu,m) ]
          !!
!!$          Call palloc('MMC',MMC,1_t_int,4*degl*degl,1_t_int,4*degl*degl,nmodule,nsubfunc)
!!$          Call palloc('EM',EM,1_t_int,4*degl*degl,nmodule,nsubfunc)
!!$          MMC=0.0_t_real
          !!
          nloop1: Do n=1,degl
             mloop1: Do m=1,degl
                cul=c0
                uc0=0.0_t_real
                ux0=0.0_t_real
                Do mu=1_t_int,degl
                   Do nu=1_t_int,degl
                      !!
                      Call coul_matel(l,n,m,nu,mu,wgaunt,lsdau% p_u_f(0:,iudef),wc,wx)
                      !!
                      de(1,1)=psmat(     mu,     nu,isau,1)
                      de(2,2)=psmat(degl+mu,degl+nu,isau,1)
                      de(1,2)=psmat(     mu,degl+nu,isau,1)
                      de(2,1)=psmat(degl+mu,     nu,isau,1)
                      If(mu==nu)Then
                         de(1,1)=de(1,1)-d_dc(1)
                         de(2,2)=de(2,2)-d_dc(2)
                         de(1,2)=de(1,2)-d_dc12
                         de(2,1)=de(2,1)-Conjg(d_dc12)
                      Endif
                      
                      cul(1,1)=cul(1,1)+wc*de(2,2)+(wc-wx)*de(1,1)
                      cul(2,2)=cul(2,2)+wc*de(1,1)+(wc-wx)*de(2,2)
                      !!
                      cul(1,2)=cul(1,2)-wx*de(1,2)
                      cul(2,1)=cul(2,1)-wx*de(2,1)
                      !!
                      If(mu==nu)Then
                         !! contraction of matrix elements for tests
                         uc0=uc0+  wc    
                         ux0=ux0+  wx
                      Endif
                      !!
!!$                      i=n+degl*(m-1)
!!$                      j=nu+degl*(mu-1)
!!$
!!$                      If(n==m .And. mu==nu)Then
!!$                         !! the density dependend AL-dc-term, the rest is N*(U-J)/2
!!$                         wc=wc-UU
!!$                         wx=wx-JJ
!!$                      Endif
!!$                      !wx=0
!!$                      MMC(i+degl*degl*0,j+degl*degl*0)=wc-wx
!!$                      MMC(i+degl*degl*0,j+degl*degl*3)=wc
!!$                      MMC(i+degl*degl*3,j+degl*degl*0)=wc
!!$                      MMC(i+degl*degl*3,j+degl*degl*3)=wc-wx
!!$
!!$                      MMC(i+degl*degl*2,j+degl*degl*1)=-wx
!!$                      MMC(i+degl*degl*1,j+degl*degl*2)=-wx
                      !!
                   Enddo ! nu
                Enddo ! mu
                !!
                If(diagonalonly) Then
                   cul(1,2)=c0
                   cul(2,1)=c0
                Endif
                !!
                pvmat(     n,     m,isau,1)= cul(1,1)
                pvmat(degl+n,degl+m,isau,1)= cul(2,2)
                pvmat(     n,degl+m,isau,1)= cul(1,2)
                pvmat(degl+n,     m,isau,1)= cul(2,1)
                
                If (n == m) Then
                   pvmat(     n,     n,isau,1)=&
                        &Cmplx(Real(pvmat(     n,     n,isau,1)-v_dc(1),t_real)&
                        &,0.0_t_real,t_real)
                   pvmat(degl+n,degl+n,isau,1)=&
                        &Cmplx(Real(pvmat(degl+n,degl+n,isau,1)-v_dc(2),t_real)&
                        &,0.0_t_real,t_real)
                   
                   pvmat(     n,degl+n,isau,1)=pvmat(n,degl+n,isau,1)-v_dc12
                   pvmat(degl+n,     n,isau,1)=pvmat(n,degl+n,isau,1)-Conjg(v_dc12)
                Endif
                !!
                Call check_contractions(n,m,l,uc0,ux0,UU,JJ,tol)
                !!
             Enddo mloop1
          Enddo nloop1
          
!!$          Call printmat('MM',"(f7.3)","('  0    ')",MMC)
!!$          Call rdiagno_noO(4*degl*degl,MMC,EM)
!!$          Do n=1,4*degl*degl
!!$             Print *,n,EM(n)
!!$          Enddo
!!$          Print *,'f0',lsdau% p_u_f(0,iudef)/(fourpi/(4*0+1))
!!$          Print *,'f2',lsdau% p_u_f(1,iudef)/(fourpi/(4*1+1))
!!$          Print *,'f4',lsdau% p_u_f(2,iudef)/(fourpi/(4*2+1))
!!$          Print *,'f6',lsdau% p_u_f(2,iudef)/(fourpi/(4*3+1))
!!$
!!$          stop
       Endif relif
!1000   Continue


       If(.False.)Then
          Call check_hermes(deg2,isau,pvmat)!!todo: remove
       Endif

       !! TODO: symmetry
       Call force_u_hermit(lsdau,nspin,pvmat)

       If(lsdau% functional == LSDAU_AMF )Then
          !! check potential trace
          Do ms=1,msp3
             i0=0
             i1=0
             mms=ms
             If(rel)Then
                i0=degl*(ms-1)
                i1=degl*(ms-1)
                If(ms==3)Then
                   i0=0
                   i1=degl
                Endif
                mms=1
             Endif
             d=Real(trace(pvmat(:,:,isau,mms),i0,i1,degl),t_real)
             If(Abs(d)>tol)Then
                Write(errbuf,14)&
                     &lsdau% p_isiteu(isau),elements(lsdau% p_isortu(isau))&
                     &,lsdau% p_labelsu(isau),d
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg=errbuf)

             Endif
          End Do
       End If


       If(energy)Then
          !! --- energy calculation
          spinloop3: Do ms=1,msp2
             !!
             i0=0
             mms=ms
             If(rel)Then
                i0=degl*(ms-1)
                mms=1
             Endif
             !!
             Do n=1,degl
                Do m=1,degl
                   lsdau% eeU= lsdau% eeU +Real(pvmat(i0+n,i0+m,isau,mms)&
                        &*psmat(i0+m,i0+n,isau,mms),t_real)*0.5_t_real*fac
                   If(rel.And.ms==1)Then
                      lsdau% eeU= lsdau% eeU&
                           & +Real(pvmat(n,degl+m,isau,mms)&
                           &*psmat(degl+m,n,isau,mms),t_real)*0.5_t_real*fac&
                           & +Real(pvmat(degl+n,m,isau,mms)&
                           &*psmat(m,degl+n,isau,mms),t_real)*0.5_t_real*fac
                   Endif
                Enddo ! j
             Enddo ! i
             
          Enddo spinloop3
          !!
          If (lsdau% functional == LSDAU_AL) Then
             !! ---  eU=\sum_{mm's} [n^s_{mm'}v^s_{mm'}]/2 + eD/2
             !! ---  eD=\sum_i [n_i dE^{dc}/dn_i] - 2E^{dc}=(U-J)N/2
             dc=(UU-JJ)*charge*0.5_t_real
             lsdau% eeD= lsdau% eeD+dc
             lsdau% eeU= lsdau% eeU+dc*0.5_t_real
          Endif
          !!
       End If
       !!
       
    Enddo ! isau
    !!
    !!
    If(rel)Then
       lsdau% p_vmat=pvmat
       Call lsdau_d_mat_trans_spin(lsdau,rel,lsdau% p_vmat,'fromspin')
    Endif
    !!
    !!
    !!
    If(prtmat) Call print_u_dmat(mspin,elements,lsdau,rel,lsdau% p_vmat,'vmat')


    If(energy) Then
       Write(outfile,1) lsdau% eeU
       Write(outfile,2) lsdau% eeD
    End If
    !!
    !!
    !!
    !!
    Call pfree(sdmat)
    Call pfree(svmat)
    Call pfree(wgaunt)
    Call pfree(MMC)
    Call pfree(EM)
    !!
    Return
    !!
1   Format('LSDA+U: eU=',f15.8)
2   Format('LSDA+U: eD=',f15.8)
14  Format("'LSDA+U: site ",i4," (",a2,") state ",a2&
         &," : potential trace(",e12.2e3,") !=0! '")
    !!
  End Subroutine lsdau_calc_u_potmat
  !!***********************************************************************
  Subroutine check_hermes(deg,isau,mat)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: deg,isau
    Complex(t_real),Intent(in)   :: mat(:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="check_hermes"
    Integer(t_int) :: n,m
    Real(t_real)   :: d
    Logical(t_log) :: err
    !!
    !!
    err=.False.
    Do n=1,deg
       Do m=n+1,deg
          d=Abs(mat(n,m,isau,1)-Conjg(mat(m,n,isau,1)))
          If(d>1.0e-12_t_real)Then
             Print "('herm error: nm=',i5,1x,i5,' d=',e14.5e3)",n,m,d
             err=.True.
          Endif
       Enddo
    Enddo
    !!
    If(err) Then
       Stop 'matrix not hermition'
    Endif
    !!
    Return
    !!
  End Subroutine check_hermes
  !!***********************************************************************
  Subroutine coul_matel(l,n,m,nu,mu,wgaunt,u_f,wc,wx)
    !!
    !!                             m,mu
    !! return wc=(n nu|W|m mu)=   w
    !!                             n,nu
    !!
    !!                             mu,m
    !! and    wx=(n nu |W|mu m)   w
    !!                             n ,nu
    !!
    !! (n,nu|w|m,mu)=Sum_(K) 4Pi/(2K+1) F_K 
    !!       sum_M gaunt(ln,KM,lm) gaunt(lnu,KM,lmu)
    !! (n,nu|w|mu,m)=Sum_(K) 4Pi/(2K+1) F_K 
    !!       sum_M gaunt(ln,KM,lmu) gaunt(lnu,KM,lm)
    !!
    !! 4PI/(2K+1)*F_K is in u_f(k/2)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: l,n,m,mu,nu
    Real(t_real),Intent(in)      :: wgaunt(:,:,:),u_f(0:)
    Real(t_real),Intent(out)     :: wc,wx
    !!LOCAL
    !Character(*),Parameter::nsubfunc="Coul_matel"
    Integer(t_int) :: k,LL2,ll,MM
    Real(t_real)   :: ac,ax
    !!
    ll=l*l
    !!
    wc=0.0_t_real
    wx=0.0_t_real
    Do k=0,l ! K 
       LL2=(2*k)**2
       ac=0.0_t_real
       ax=0.0_t_real
       Do MM=1,4*k+1 ! M
          
          ac=ac+ wgaunt(ll+n ,ll+m ,LL2+MM)&
               &*wgaunt(ll+nu,ll+mu,LL2+MM)
          ax=ax+ wgaunt(ll+n ,ll+mu,LL2+MM)&
               &*wgaunt(ll+nu,ll+m ,LL2+MM)
       Enddo ! m
       wc=wc+ac*u_f(k) ! 4PI/(2K+1) F
       wx=wx+ax*u_f(k)
    Enddo ! k
    
    !!
    Return
    !!
  End Subroutine Coul_matel 
  !!***********************************************************************
  Subroutine check_contractions(n,m,l,uc0,ux0,UU,JJ,tol)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n,m,l
    Real(t_real),Intent(in)      :: uc0,ux0,UU,JJ,tol
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_contractions"
    Real(t_real)   :: d
    !!
    !!
    If(n/=m)Then
       If( Abs(uc0) >tol )Then
10        Format("'LSDA+U: contraction of direct &
               &term is not diagonal, error:",e12.2e3," ! '")
          Write(errbuf,10)uc0
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=errbuf)
          
       Endif
       If( Abs(ux0) >tol )Then
11        Format("'LSDA+U: contraction of xc &
               &term is not diagonal, error:",e12.2e3," ! '")
          Write(errbuf,11)uc0
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=errbuf)
       Endif
    Else
       d=uc0-UU*(2*l+1)
       If( Abs(d) >tol )Then
12        Format("'LSDA+U: contraction of direct &
               &term /=  (2l+1)U, error:",e12.2e3," ! '")
          Write(errbuf,12)d
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=errbuf)
          
       Endif
       d=ux0-(UU+2.0_t_real*l*JJ)
       If( Abs(d) >tol )Then
13        Format("'LSDA+U: contraction of xc &
               &term /=  U+2l J error:",e12.2e3," ! '")
          Write(errbuf,13)d
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=errbuf)
          
       Endif
       
    Endif
    !!
    Return
    !!
  End Subroutine check_contractions
  !!***********************************************************************
  !! add LSDA+U potential matrix to Hamiltonian matrix
  !! to be executed at every k-point
  !!
  Subroutine lsdau_merge_h(lsdau,nvdim,hp,sp,ms,kintest)
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Logical(t_log),Intent(in)    :: kintest
    Integer(t_int),Intent(in)    :: nvdim,ms
    Complex(t_real),Intent(inout):: hp(nvdim,nvdim)
    Complex(t_real),Intent(in)   :: sp(nvdim,nvdim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_merge_h"
    Integer(t_int) :: i1,j1,i,j,ib1,ib2,isau,deg
    Complex(t_real):: cx
    !!   
    If(.Not. lsdau% lsdau ) Return
    !!
    If(kintest)Then
       lsdau% p_hpu(1_t_int:nvdim,1_t_int:nvdim)=0.0_t_real
    Endif
    !!
    If (lsdau% projection == LSDAU_ORTHOGONAL ) Then
       Do isau=1_t_int,lsdau% nusite
          ib1=lsdau% p_ubi1(isau) !! start index of u-block
          ib2=lsdau% p_ubi2(isau) !! end   index of u-block
          Do i=ib1,ib2
             i1=i-ib1+1
             Do j=ib1,ib2
                j1=j-ib1+1
                hp(i,j)=hp(i,j) +lsdau% p_vmat(i1,j1,isau,ms)
                If(kintest)Then
                   lsdau% p_hpu(i,j)=lsdau% p_vmat(i1,j1,isau,ms)
                Endif
             Enddo ! j
          Enddo ! i
       Enddo ! isau
    Else If (lsdau% projection == LSDAU_GROSS ) Then
       !!
       !! dH=(PvPS+SPvP)*0.5_t_real
       !!
       Do isau=1_t_int,lsdau% nusite
          !!
          ib1=lsdau% p_ubi1(isau) !! start index of u-block
          ib2=lsdau% p_ubi2(isau) !! end   index of u-block
          deg=lsdau% p_degvsu(isau) !! end   index of u-block
          !!
          !!
          Do j=1,nvdim
             Do i=ib1,ib2
                i1=i-ib1+1
                !!
                !!
                cx=Sum(lsdau% p_vmat(i1,1:deg,isau,ms)*sp(ib1:ib2,j))
                !!
                hp(i,j)=hp(i,j)+cx*0.5_t_real
                If(kintest)Then
                   lsdau% p_hpu(i,j)=lsdau% p_hpu(i,j)+cx*0.5_t_real
                Endif
                !!
                !!
                cx=Sum(sp(j,ib1:ib2)*lsdau% p_vmat(1:deg,i1,isau,ms))
                !!
                hp(j,i)=hp(j,i)+cx*0.5_t_real
                If(kintest)Then
                   lsdau% p_hpu(j,i)=lsdau% p_hpu(j,i)+cx*0.5_t_real
                Endif
                
             Enddo! i
             !!
          Enddo! j
       Enddo ! isau
       !!
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized LSDA+U projection no longer supported !'")
    Endif
    !!
    
    !!
  End Subroutine lsdau_merge_h
  !!***********************************************************************
  !! 
  Logical(t_log) Function lsdau_norm_conv(lsdau,nspin)
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Integer(t_int),Intent(in)    :: nspin
    !!LOCAL
    Integer(t_int) :: m,deg,i,j,ms,isau
    Real(t_real)   :: norm
    !!
    lsdau_norm_conv=.True.
    !!
    If(.Not. lsdau% lsdau ) Return
    If( lsdau% dmat_mode /= LSDAU_DMAT_MODE_SC ) Return
    !!
    norm=0.0_t_real
    m=0_t_int
    Do isau=1,lsdau% nusite
       deg=lsdau% p_degvsu(isau)
       Do ms=1,nspin
          m=m+deg*deg
          Do i=1,deg
             Do j=1,deg
                norm=norm+Abs(lsdau% p_odmat(i,j,isau,ms)&
                     &-lsdau% p_dmat(i,j,isau,ms))**2
             Enddo ! j
          Enddo ! i
       Enddo ! ms
    Enddo ! isite
    norm=Sqrt(norm/Real(m,t_real))
1   Format('LSDA+U: dmat: deviation ',e12.3e3)
    Write(outfile,1) norm
    !!
    If (norm > lsdau% omat_dev) Then
       lsdau_norm_conv=.False.
    Endif
    !!
    Return
    !!
  End Function lsdau_norm_conv
  !!***********************************************************************
  !! force hermitization of numerical noice
  Subroutine force_u_hermit(lsdau,nspin,mat)
    !!
    Implicit None  
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Integer(t_int),Intent(in)    :: nspin
    Complex(t_real),Intent(inout)   :: mat(:,:,:,:) 
    
    !!LOCAL
    Integer(t_int) :: isau,ms,i1,i2,deg
    Complex(t_real):: c
    !!
    !!
    Do ms=1,nspin
       Do isau=1_t_int,lsdau% nusite
          deg=lsdau% p_degvsu(isau)
          Do i1=1,deg
             Do i2=i1+1,deg
                c=(mat(i1,i2,isau,ms)+Conjg(mat(i2,i1,isau,ms)))*0.5_t_real
                mat(i1,i2,isau,ms)=c
                mat(i2,i1,isau,ms)=Conjg(c)
             End Do !i2
             mat(i1,i1,isau,ms)=&
                  &Cmplx(Real(mat(i1,i1,isau,ms),t_real),0.0_t_real,t_real)
          End Do !i1
       Enddo ! isau
    Enddo ! ms
    !!
  End Subroutine force_u_hermit
  !!***********************************************************************
  Subroutine lsdau_d_mat_trans_spin(lsdau,rel,dmat,mode)
    !! transform a density matrix representation
    !!    mode='tospin'   from ljm to lms  
    !!    mode='fromspin' from lms to ljm  
    !! 
    !! in ljm rep first come the j=l-1/2 states and second the j=l+1/2 states
    !! in lms rep first come the spin up and then the spin down states
    !!
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau
    Logical(t_log),Intent(in)    :: rel
    Character(*),Intent(in)      :: mode
    Complex(t_real),Intent(inout):: dmat(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_d_mat_trans_spin"
    Integer(t_int) :: isau,i,j,deg
    Complex(t_real),Pointer:: lmat(:,:)
    !!
    Nullify(lmat)
    !!
    If(.Not. rel) Return
    !!
    !!
    If(Len_trim(mode)==0) Return
    !!
    !!
    Call palloc('lmat',lmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,nmodule,nsubfunc)
    
    !!
    If(Trim(mode)=='fromspin')Then
       !! n=T^+ ns T
       Do isau=1_t_int,lsdau% nusite
          deg=lsdau% p_degvsu(isau)
          Do i=1,deg
             Do j=1,deg
                lmat(j,i)=Dot_product(lsdau% p_T(1:deg,i,isau),dmat(1:deg,j,isau,1))
             Enddo
          Enddo

          Do i=1,deg
             Do j=1,deg
                dmat(i,j,isau,1)=Sum(lmat(1:deg,i)*lsdau% p_T(1:deg,j,isau))
             Enddo
          Enddo
       Enddo
       !!
    Else If(Trim(mode)=='tospin')Then
       !! n=T^+ ns T
       Do isau=1_t_int,lsdau% nusite
          deg=lsdau% p_degvsu(isau)
          Do i=1,deg
             Do j=1,deg
                lmat(j,i)=Sum(lsdau% p_T(i,1:deg,isau)*dmat(1:deg,j,isau,1))
             Enddo
          Enddo

          Do i=1,deg
             Do j=1,deg
                dmat(i,j,isau,1)=Dot_product(lsdau% p_T(j,1:deg,isau),lmat(1:deg,i))
             Enddo
          Enddo
       Enddo
       !!
    Endif
    !!
    !!
    Call pfree(lmat)
    !!
    !!
    Return
    !!
  End Subroutine lsdau_d_mat_trans_spin
  !!***********************************************************************
  Subroutine force_u_mat_sym(symmetry,equivsites,lsdau,rel,mspin,lmat,mat)
    !! make symmetrization to correct for irreducible brilloun zone
    !! integration
    !! mat = symmetrization of lmat
    !! for rel and mspin==1 time reversal is enforced
    !! the matrix representation must be ljmu for rel
    Implicit None
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Type(T_Symmetry),Intent(in)  :: symmetry
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: equivsites(48,*)
    Complex(t_real),Intent(inout):: lmat(:,:,:,:)
    Complex(t_real),Intent(out)  :: mat(:,:,:,:) 
    !!LOCAL
    Character(*),Parameter::nsubfunc="force_u_mat_sym"
    Integer(t_int) :: ms,isbu,iscu,isc,l,ivsu,isa,isau,nspin
    Integer(t_int) :: jops,deg,i,j,ib,jb,degl
    Logical(t_log) :: timerev
    Real(t_real)   :: norm,ski,skj,s
    Complex(t_real),Pointer:: ldmat(:,:)
    !!
    Nullify(ldmat)
    !!
    nspin=mspin
    if(rel) nspin=1
    !!
    mat(:,:,:,:)=c0
    !!
!!$    Do isau=1_t_int,lsdau% nusite
!!$       Print *,'isau=',isau
!!$       Call cprintmat('n',"('(',f8.5,',',f8.5,')')","('         0         ')"&
!!$            &,lmat(:,:,isau,1),1.0e-6_t_real)
!!$    Enddo
    !!
    !!
    !Do isau=1_t_int,lsdau% nusite
    !   Call cprintmat('n',"('(',f8.5,',',f8.5,')')","('         0         ')"&
    !        &,lmat(:,:,isau,1),1.0e-6_t_real)
    !Enddo
    !!
    !!
    norm=1.0_t_real/Real(symmetry% Rnumops,t_real)
    !!
    isauloop: Do isau=1_t_int,lsdau% nusite
       isa=lsdau% p_isiteu(isau)
       ivsu=lsdau% p_nvsu(isau)
       l=lsdau% p_lvsu(isau)
       deg=lsdau% p_degvsu(isau)
       
       
       
       oploop: Do jops=1_t_int,symmetry% Rnumops
          timerev=( symmetry% Roptimerev(jops)==1)
          isc=equivsites(jops,isa)
          iscu=0_t_int
          isbuloop:  Do isbu=1_t_int,lsdau% nusite
             
             
             If(ivsu == lsdau% p_nvsu(isbu) .And.&
                  & isc== lsdau% p_isiteu(isbu))Then
                iscu=isbu
                Exit isbuloop
             End If
             
          End Do isbuloop
          If(iscu == 0_t_int)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Symmetry is incorrect!'")
          Endif
          
          !!
          !! mat(isau)=mat(isau)+K0*wu^(+)*lmat(iscu)*wu*K0
          !!
          !!
          !!
          Do ms=1_t_int,nspin
             Call mmult(&
                  & mat(:,:,isau,ms)&
                  &,lmat(:,:,iscu,ms)&
                  &,symmetry% p_Rwu_tjnl_ks(:,:,l,jops)&
                  &,deg,norm,timerev)
          Enddo
          
          
          
       Enddo oploop                   ! jops
       
       !! time reversal
       If(rel.And.mspin==1)Then
          degl=2*l+1
          Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim&
               &,1_t_int,lsdau% rumaxdim,nmodule,nsubfunc)
          ldmat=mat(:,:,isau,1)
          Do i=1,deg
             If(i>=degl)Then
                ski=-1.0_t_real
                ib=3*degl-i
             Else
                !! kap=l
                ski=1.0_t_real
                ib=degl-i
             Endif
             Do j=1,deg
                If(j>=degl)Then
                   skj=-1.0_t_real
                   jb=3*degl-j
                Else
                   !! kap=l
                   skj=1.0_t_real
                   jb=degl-j
                Endif
                s=-ski*skj*(-1.0_t_real)**((j-jb+i-ib)*0.5_t_real)
                mat(i,j,isau,1)=(ldmat(i,j)+s*Conjg(ldmat(ib,jb)))*0.5_t_real
             Enddo
             
          Enddo
          !!
          Call pfree(ldmat)
          !!
          !!
       Endif
       !!
    Enddo isauloop
    !!
!!$    !!
!!$    Do isau=1_t_int,lsdau% nusite
!!$       Print *,'isau=',isau
!!$       Call cprintmat('n',"('(',f8.5,',',f8.5,')')","('         0         ')"&
!!$            &,lmat(:,:,isau,1),1.0e-6_t_real)
!!$    Enddo
    !!
    !!
  End Subroutine force_u_mat_sym
  !!***********************************************************************
  !! make symmetry decomposition of LSDA+U density matrix
  !! and add the parameter to the iteration vector
  Subroutine lsda_sdcmp(lsdau,rel,mspin,vit,nit,Ufactor)
    !!
    Implicit none  
    !!IO
    Type(T_Lsdau),Intent(in)     :: lsdau     
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(inout) :: nit
    Real(t_real),Intent(in)      :: Ufactor
    Real(t_real),Intent(inout)   :: vit(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsda_sdcmp"
    Integer(t_int) :: deg,isau,m,n,ms,nspin,a1,dim,degl
    Complex(t_real):: cd
    Real(t_real),Pointer :: p_bb(:)
    Complex(t_real),Pointer:: pdmat(:,:,:,:)
    Complex(t_real),Pointer::  p_Bop(:,:,:,:,:)
    Complex(t_real),Pointer:: ldmat(:,:,:,:)
    !!
    Nullify(ldmat,p_Bop,p_bb)
    !!
    nspin=mspin
    If(rel) nspin=1
    !!

    If(rel) Then
       nspin=1
       Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
            &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
       !!
       ldmat=lsdau% p_dmat
       Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,mode='tospin')
       pdmat => ldmat
    Else
       pdmat => lsdau% p_dmat
    Endif
    !!
    If(checkspinrep)Then
       Print *,'checking in spin rep'
       Call dmat_decom(nspin,mspin,lsdau,rel,p_Bop)
    Else
       pdmat => lsdau% p_dmat
    Endif
    !!
    isauloop: Do isau=1,lsdau% nusite
       deg=lsdau% p_degvsu(isau)
       degl=2*lsdau% p_lvsu(isau)+1
       If(checkspinrep)Then
          !!
          Call decom_b(nspin,deg,isau,pdmat,p_Bop,p_bb)
          !!
          dim=Size(indices)
          Do a1=1,dim
             vit(nit)=p_bb(indices(a1))*Ufactor
             nit=nit+1
          Enddo
          !!
          Call decom_S_b(nspin,deg,indices,p_Bop(:,:,:,:,isau),p_bb)
          !!
       Endif
       !!
       !!
       If(rel)Then
          If(checkspinrep)Then
             !!spinsum
             Do m=1,degl
                Do n=m,degl
                   cd=pdmat(m,n,isau,1)+pdmat(degl+m,degl+n,isau,1)
                   vit(nit)=Real(cd,t_real)*Ufactor
                   nit=nit+1_t_int
                   vit(nit)=Aimag(cd)*Ufactor
                   nit=nit+1_t_int
                Enddo
             Enddo

             !!spindiff
             Do m=1,degl
                Do n=m,degl
                   cd=pdmat(m,n,isau,1)-pdmat(degl+m,degl+n,isau,1)
                   vit(nit)=Real(cd,t_real)*Ufactor
                   nit=nit+1_t_int
                   vit(nit)=Aimag(cd)*Ufactor
                   nit=nit+1_t_int
                Enddo
             Enddo

             !! spin12
             Do m=1,degl
                Do n=1,degl
                   cd=pdmat(m,degl+n,isau,1)
                   vit(nit)=Real(cd,t_real)*Ufactor
                   nit=nit+1_t_int
                   vit(nit)=Aimag(cd)*Ufactor
                   nit=nit+1_t_int
                Enddo
             Enddo
          Else
             Do m=1,deg
                Do n=m,deg
                   cd=pdmat(m,n,isau,1)
                   vit(nit)=Real(cd,t_real)*Ufactor
                   nit=nit+1_t_int
                   vit(nit)=Aimag(cd)*Ufactor
                   nit=nit+1_t_int
                Enddo
             Enddo
          Endif
       Else
          Do m=1,deg
             Do n=m,deg
                Do ms=1,mspin
                   If(checkspinrep)Then
                      cd=(pdmat(m,n,isau,1)-Sum(p_Bop(m,n,1,indices(1:dim),isau)*p_bb(1:dim)))&
                           &+(3-2*ms)*(pdmat(m,n,isau,mspin)&
                           &   -Sum(p_Bop(m,n,mspin,indices(1:dim),isau)*p_bb(1:dim)))
                   Else
                      cd=pdmat(m,n,isau,1)+(3-2*ms)*pdmat(m,n,isau,mspin)
                   Endif
                   
                   vit(nit)=Real(cd,t_real)*Ufactor
                   nit=nit+1_t_int
                   vit(nit)=Aimag(cd)*Ufactor
                   nit=nit+1_t_int
                Enddo
             Enddo
          Enddo
       Endif
    Enddo isauloop
    !!
    Call pfree(ldmat)
    Call pfree(p_Bop)
    Call pfree(p_bb)
    !!
  End Subroutine lsda_sdcmp
  !!***********************************************************************
  !! revers of lsda_sdcmp : 
  !!        make LSDA+U density matrix form symmetry decomposition 
  !! get the parameter from the iteration vector
  Subroutine lsda_scmp(lsdau,symmetry,structure,rel,mspin,vit,nit,Ufactor)
    !!
    Implicit none  
    !!IO
    Type(T_Lsdau),Intent(inout)  :: lsdau     
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(inout) :: nit
   Real(t_real),Intent(in)      :: Ufactor
    Real(t_real),Intent(in)      :: vit(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsda_scmp"
    Integer(t_int) :: deg,isau,m,n,ms,nspin,dim,a1,degl
    Complex(t_real):: cd(2)
    Real(t_real),Pointer :: p_bb(:)
    Complex(t_real),Pointer :: ldmat(:,:,:,:) 
    Complex(t_real),Pointer::  p_Bop(:,:,:,:,:)
    !!
    Nullify(ldmat,p_Bop,p_bb)
    !!
    !!
    !!
    nspin=mspin
    if(rel) nspin=1
    !!
    Call palloc('ldmat',ldmat,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
    ldmat(:,:,:,:)=c0
    !!
    If(checkspinrep)Then
       Print *,'checking out spin rep'
       Call dmat_decom(nspin,mspin,lsdau,rel,p_Bop)
    Endif
    !!
    isauloop: Do isau=1,lsdau% nusite
       deg=lsdau% p_degvsu(isau)
       degl=2*lsdau% p_lvsu(isau)+1
       
       If(checkspinrep)Then
          !!
          Call decom_b(nspin,deg,isau,ldmat,p_Bop,p_bb)
          !!
          p_bb=0.0_t_real
          dim=Size(indices)
          Do a1=1,dim
             p_bb(indices(a1))=vit(nit)/Ufactor
             nit=nit+1
          Enddo
          !!
          Call decom_S_b(nspin,deg,indices,p_Bop(:,:,:,:,isau),p_bb)
          !!
          !!
       Endif
       
       If(rel)Then
          If(checkspinrep)Then
             Do m=1,degl
                Do n=m,degl
                   cd(1)=Cmplx(vit(nit)/Ufactor,vit(nit+1)/Ufactor,t_real)
                   nit=nit+2
                   ldmat(m,n,isau,1)=cd(1)*0.5_t_real
                   ldmat(degl+m,degl+n,isau,1)=cd(1)*0.5_t_real
                Enddo
             Enddo
             !! spin2
             Do m=1,degl
                Do n=m,degl
                   cd(1)=Cmplx(vit(nit)/Ufactor,vit(nit+1)/Ufactor,t_real)
                   nit=nit+2
                   ldmat(m,n,isau,1)=ldmat(m,n,isau,1)+cd(1)*0.5_t_real
                   ldmat(degl+m,degl+n,isau,1)=ldmat(degl+m,degl+n,isau,1)-cd(1)*0.5_t_real
                Enddo
             Enddo
             !! spin12
             Do m=1,degl
                Do n=1,degl
                   cd(1)=Cmplx(vit(nit)/Ufactor,vit(nit+1)/Ufactor,t_real)
                   nit=nit+2
                   ldmat(m,degl+n,isau,1)=cd(1)
                Enddo
             Enddo
          Else
             Do m=1,deg
                Do n=m,deg
                   cd=c0
                   cd(1)=Cmplx(vit(nit)/Ufactor,vit(nit+1)/Ufactor,t_real)
                   nit=nit+2
                   ldmat(m,n,isau,1)=cd(1)
                Enddo
             Enddo
          Endif
          !!
          Do m=1,deg
             ldmat(m,m,isau,1)=Cmplx(Real(ldmat(m,m,isau,1),t_real),0.0_t_real,t_real)
             Do n=m+1,deg
                ldmat(n,m,isau,1)=Conjg(ldmat(m,n,isau,1))
             Enddo
          Enddo
          !!
       Else
          
          
          Do m=1,deg
             Do n=m,deg
                cd=c0
                Do ms=1,mspin
                   cd(ms)=Cmplx(vit(nit)/Ufactor,vit(nit+1)/Ufactor,t_real)
                   nit=nit+2
                Enddo
                
                ldmat(m,n,isau,1)=(cd(1)+cd(2))*0.5_t_real
                ldmat(m,n,isau,mspin)=(cd(1)-cd(2))*0.5_t_real
                
                
                If(checkspinrep)Then
                   Do ms=1,mspin
                      ldmat(m,n,isau,ms)=ldmat(m,n,isau,ms)&
                           &+Sum(p_Bop(m,n,ms,indices(1:dim),isau)*p_bb(1:dim))
                   Enddo
                Endif
                
                If(m==n)Then
                   ldmat(m,n,isau,1    )&
                        &=Cmplx(Real(ldmat(m,n,isau,1   ),t_real),0.0_t_real,t_real)
                   ldmat(m,n,isau,mspin)&
                        &=Cmplx(Real(ldmat(m,n,isau,mspin),t_real),0.0_t_real,t_real)
                Else
                   ldmat(n,m,isau,1   )=Conjg(ldmat(m,n,isau,1   ))
                   ldmat(n,m,isau,mspin)=Conjg(ldmat(m,n,isau,mspin))
                Endif
                
             Enddo
          Enddo
       Endif

    Enddo isauloop
    !!
    If(checkspinrep)Then
       If(rel)Then
          Call lsdau_d_mat_trans_spin(lsdau,rel,ldmat,mode='fromspin')
       Endif
    Endif
    !!
    !!
    Call force_u_hermit(lsdau,nspin,ldmat)
    Call force_u_mat_sym(symmetry,structure% p_equivsites,lsdau,rel,mspin&
         &,ldmat,lsdau% p_dmat)
    !!
    !!
    Call pfree(ldmat)
    Call pfree(p_Bop)
    Call pfree(p_bb)
    !!!
    Return
    !!
    !!
    Return
    !!
  End Subroutine lsda_scmp
  !!***********************************************************************
  Subroutine  lsdau_aux_alloc(lsdau,nvdim,nband,nkp,nspin,kintest)
    !!
    Implicit None 
    !!IO
    Type(T_LSDAU),Intent(inout)  :: lsdau
    Logical(t_log),Intent(in)    :: kintest
    Integer(t_int),Intent(in)    :: nvdim,nband,nkp,nspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="lsdau_aux_alloc"
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    Call palloc('lsdau% p_cvv',lsdau% p_cvv&
         &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite&
         &,1_t_int,nband ,1_t_int,nkp ,1_t_int,nspin ,nmodule,nsubfunc)
    Call palloc('lsdau% p_Scvv',lsdau% p_Scvv&
         &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% nusite&
         &,1_t_int,nband ,1_t_int,nkp ,1_t_int,nspin ,nmodule,nsubfunc)
    !!
    !!
    Call palloc('lsdau% p_odmat',lsdau% p_odmat&
         &,1_t_int,lsdau% rumaxdim,1_t_int,lsdau% rumaxdim&
         &,1_t_int,lsdau% nusite,1_t_int,nspin,nmodule,nsubfunc)
    !!
    !! ENERGY TESTS
    If(kintest) Then
       Call palloc('lsdau% p_hpu',lsdau% p_hpu&
            &,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
       Call palloc('lsdau% p_hh',lsdau% p_hh&
            &,1_t_int,nband,1_t_int,nkp,1_t_int,nspin,nmodule,nsubfunc)
    Endif
    !!
    Return
    !!
  End Subroutine lsdau_aux_alloc
  !!***********************************************************************
  Subroutine  lsdau_aux_free(lsdau)
    !!
    Implicit None 
    !!IO
    Type(T_LSDAU),Intent(inout)  :: lsdau
    !!LOCAL
    !Character(*),Parameter::nsubfunc="lsdau_aux_free"
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    Call pfree(lsdau% p_cvv)
    Call pfree(lsdau% p_Scvv)
    Call pfree(lsdau% p_hpu)
    Call pfree(lsdau% p_hh)
    Call pfree(lsdau% p_odmat)
    !!
    !!
    Return
    !!
  End Subroutine lsdau_aux_free
  !!***********************************************************************
  Subroutine  lsdau_save_dmat(lsdau)
    !!
    Implicit None 
    !!IO
    Type(T_LSDAU),Intent(inout)  :: lsdau
    !!LOCAL
    !Character(*),Parameter::nsubfunc="lsdau_save_dmat"
    !!
    If(.Not. lsdau% lsdau) Return
    !!
    !! save density matrix
    lsdau% p_odmat(:,:,:,:)=lsdau% p_dmat(:,:,:,:)
    !!
    !!
    Return
    !!
  End Subroutine lsdau_save_dmat
  !!***********************************************************************
  
End Module lsdau
