Module mesh
  !!
  Use precisiondefs
  Use memory
  Use errors
  Use tmesh
  Use interpolation
  Use bspline
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="MESH"  
  !!
  Public :: setup_mesh,mesh_r_to_x,mesh_x_to_r&
       &,setup_bspline_mesh,makebsplint,bsplindex,bspvalue&
       &,minterpol,mintpl_coefs,mreinter,writefunc,det_logzero_params&
       &,write_mesh_info,read_mesh_info,write_mesh_function&
       &,read_mesh_function,read_mesh_function_rf&
       &,mbsp_coefs,mesh_x_to_jac,mesh_r_to_jac&
       &,rmintpl_coefs,rmintpl_coefs1,dfspline,dfspline1,dfspline3&
       &,meshesEqual
  !!***********************************************************************
contains 
  !!***********************************************************************
  !! return the jacobian of the mesh transform r=g(x)
  !! jac(x)= dr/dx =d/dx g(x)
  !!
  Real(t_real) Function mesh_r_to_jac(mesh,r)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: r
    !!LOCAL
    Character(*),Parameter::nsubfunc="mesh_r_to_jac"
    !!
    mesh_r_to_jac=1.0_t_real
    !!
    If(mesh% Type ==LOG_MESH)Then
       !!
       mesh_r_to_jac=r
       !!
    Else If(mesh% Type ==LIN_MESH)Then
       !!
       mesh_r_to_jac=1.0_t_real
       !!
    Else If(mesh% Type ==ROOT_MESH .Or. mesh% Type ==POW_MESH)Then
       !!
       If(r==0.0_t_real  ) Then
          If(mesh% alpha> 1 ) Then
             mesh_r_to_jac=0.0_t_real
          Else If(mesh% alpha == 1 ) Then
             mesh_r_to_jac=1.0_t_real
          Else
             Write(errbuf,2) mesh% alpha
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
       Else
          mesh_r_to_jac=mesh% alpha*r/mesh_r_to_x(mesh,r)
       Endif
       !!
    Else If(mesh% Type ==LOGZERO_MESH)Then
       !!
       mesh_r_to_jac=r+mesh% alpha
       !!
    Else
       Write(errbuf,1) mesh% Type
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
1   Format("'Mesh type ",i4," not implemented!'")
2   Format("'Mesh parameter alpha ",i4," < 1!'")
    !!
  End Function mesh_r_to_jac
  !!***********************************************************************
  !! return the jacobian of the mesh transform r=g(x)
  !! jac(x)= dr/dx =d/dx g(x)
  !!
  Real(t_real) Function mesh_x_to_jac(mesh,x)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: x
    !!LOCAL
    Character(*),Parameter::nsubfunc="mesh_x_to_jac"
    !!
    mesh_x_to_jac=1.0_t_real
    !!
    If(mesh% Type ==LOG_MESH)Then
       !!
       mesh_x_to_jac=Exp(x)
       !!
    Else If(mesh% Type ==LIN_MESH)Then
       !!
       mesh_x_to_jac=1.0_t_real
       !!
    Else If(mesh% Type ==ROOT_MESH .Or. mesh% Type ==POW_MESH)Then
       !!
       If(x==0.0_t_real  ) Then
          If(mesh% alpha> 1 ) Then
             mesh_x_to_jac=0.0_t_real
          Else If(mesh% alpha == 1 ) Then
             mesh_x_to_jac=1.0_t_real
          Else
             Write(errbuf,2) mesh% alpha
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
       Else
          mesh_x_to_jac=mesh% alpha*x**(mesh% alpha)/x
       Endif
       !!
    Else If(mesh% Type ==LOGZERO_MESH)Then
       !!
       mesh_x_to_jac=mesh% alpha*Exp(x)
       !!
    Else
       Write(errbuf,1) mesh% Type
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
1   Format("'Mesh type ",i4," not implemented!'")
2   Format("'Mesh parameter alpha ",i4," < 1!'")
    !!
  End Function mesh_x_to_jac
  !!***********************************************************************
  !! return the mesh transforn r=g(x)
  Real(t_real) Function mesh_x_to_r(mesh,x)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: x
    !!LOCAL
    Character(*),Parameter::nsubfunc="mesh_x_to_r"
    !!
    mesh_x_to_r=x
    If(mesh% Type ==LOG_MESH)Then
       mesh_x_to_r=Exp(x)
    Else If(mesh% Type ==LIN_MESH)Then
       mesh_x_to_r=x
    Else If(mesh% Type ==ROOT_MESH)Then
       mesh_x_to_r=x**(mesh% alpha)
    Else If(mesh% Type ==LOGZERO_MESH)Then
       mesh_x_to_r=mesh% alpha*(Exp(x)-1.0_t_real)
    Else If(mesh% Type ==POW_MESH)Then
       mesh_x_to_r=x**(mesh% alpha)
    Else
       Write(errbuf,1) mesh% Type
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
1   Format("'Mesh type ",i4," not implemented!'")
    !!
  End Function mesh_x_to_r
  !!***********************************************************************
  !! return the inverse mesh transform x=g^{-1}(r)
  Real(t_real) Function mesh_r_to_x(mesh,r)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: r
    !!LOCAL
    Character(*),Parameter::nsubfunc="mesh_r_to_x"
    !!
    mesh_r_to_x=r
    !!
    If(mesh% Type ==LOG_MESH)Then
       !! special code to avoid overflow for some unimportant cases
       !! however, if this is used in another context be shure to
       !! consider this special situation
       If(r<1.0e-20_t_real)Then
          mesh_r_to_x=Log(1.0e-20_t_real)
       Else
          mesh_r_to_x=Log(r)
       Endif
    Else If(mesh% Type ==LIN_MESH)Then
       mesh_r_to_x=r
    Else If(mesh% Type ==ROOT_MESH)Then
       mesh_r_to_x=r**(1.0_t_real/mesh% alpha)
    Else If(mesh% Type ==LOGZERO_MESH)Then
       mesh_r_to_x=Log(r/mesh% alpha+1.0_t_real)
    Else If(mesh% Type ==POW_MESH)Then
       mesh_r_to_x=r**(1.0_t_real/mesh% alpha)
    Else
       Write(errbuf,1) mesh% Type
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
1   Format("'Mesh type ",i4," not implemented!'")
    !!
  End Function mesh_r_to_x
  !!***********************************************************************
  Subroutine  setup_mesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_mesh"
    !!
    If(mesh% Type ==LOG_MESH)Then
       Call setup_logmesh(mesh)
    Else If(mesh% Type ==LIN_MESH)Then
       Call setup_linmesh(mesh)
    Else If(mesh% Type ==ROOT_MESH)Then
       Call setup_rootmesh(mesh)
    Else If(mesh% Type ==LOGZERO_MESH)Then
       Call setup_logzeromesh(mesh)
    Else If(mesh% Type ==POW_MESH)Then
       Call setup_powmesh(mesh)
    Else
       Write(errbuf,1) mesh% Type
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
1   Format("'Mesh type ",i4," not implemented!'")
    !!
  End Subroutine setup_mesh
  !!***********************************************************************
  !! 
  !!    rmesh(i)=rmin*exp((i-1)*h)=exp(xmesh(i)
  !!    xmesh(i)=log(rmin)+(i-1)*h
  Subroutine setup_logmesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_logmesh"
    Integer(t_int) :: ir
    Real(t_real)  :: x0,x1
    !!
    Call palloc('mesh % p_rmesh',mesh % p_rmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_xmesh',mesh % p_xmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_jmesh',mesh % p_jmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)

    mesh% type=LOG_MESH
    mesh% with_zero=.False.
    
    x0=Log(mesh % rmin)
    x1=Log(mesh % rmax)
    mesh % hx=(x1-x0)/Real(mesh % nr-1_t_int,t_real)
    Do ir=1_t_int,mesh % nr
       mesh % p_xmesh(ir)=Real(ir-1_t_int,t_real)*mesh % hx+x0
       mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
       mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
    Enddo ! ir
    Return
  End Subroutine setup_logmesh
  !!***********************************************************************
  !! 
  !!    rmesh(i)=xmesh(i)
  !!    xmesh(i)=rmin+(i-1)*h
  Subroutine setup_linmesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_linmesh"
    Integer(t_int) :: ir
    Real(t_real)  :: x0,x1
    !!

    Call palloc('mesh % p_rmesh',mesh % p_rmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_xmesh',mesh % p_xmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_jmesh',mesh % p_jmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)

    mesh% type=LIN_MESH
    mesh% with_zero=.False.
    
    x0=mesh % rmin
    x1=mesh % rmax
    mesh % hx=(x1-x0)/Real(mesh % nr-1_t_int,t_real)
    Do ir=1_t_int,mesh % nr
       mesh % p_xmesh(ir)=Real(ir-1_t_int,t_real)*mesh % hx+x0
       mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
       mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
    Enddo ! ir
    Return
  End Subroutine setup_linmesh
  !!***********************************************************************
  !! 
  !!    xmesh(i)=i*h
  !!    rmesh(i)=xmesh(i)**al
  Subroutine setup_rootmesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_rootmesh"
    Integer(t_int) :: ir
    !!

    Call palloc('mesh % p_rmesh',mesh % p_rmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_xmesh',mesh % p_xmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_jmesh',mesh % p_jmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)

    mesh% type=ROOT_MESH

    If(mesh% with_zero)Then
       mesh% alpha=Log(mesh% rmax/mesh% rmin)&
            &/Log(Real(mesh% nr-1_t_int,t_real))
       
       mesh% hx=mesh% rmin**(1.0_t_real/mesh% alpha)
       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir-1_t_int,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
       Enddo ! ir
    Else
       mesh% alpha=Log(mesh% rmax/mesh% rmin)/Log(Real(mesh% nr,t_real))

       mesh% hx=mesh% rmin**(1.0_t_real/mesh% alpha)
       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
       Enddo ! ir
    Endif
    !!
    If(mesh % alpha < 1.0_t_real) Then
       Write(errbuf,1) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
1      Format("' The root-mesh parameter give alpha<1, which makes no &
            &sense',/,' The rmin or the number of points have&
            & to be adjusted!'")
    Endif
    !!
    Return
  End Subroutine setup_rootmesh
  !!***********************************************************************
  !! 
  !!    xmesh(i)=i*h
  !!    rmesh(i)=xmesh(i)**al
  Subroutine setup_powmesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_powmesh"
    Integer(t_int) :: ir
    Real(t_real)   :: rmin
    !!

    Call palloc('mesh % p_rmesh',mesh % p_rmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_xmesh',mesh % p_xmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_jmesh',mesh % p_jmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)

    mesh% type=POW_MESH

    If(mesh% with_zero)Then
       rmin=mesh% rmax/(Real(mesh% nr-1,t_real)**mesh% alpha)
       mesh% hx=rmin**(1.0_t_real/mesh% alpha)
       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir-1_t_int,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
       Enddo ! ir
    Else
       rmin=mesh% rmax/(Real(mesh% nr,t_real)**mesh% alpha)
       mesh% hx=rmin**(1.0_t_real/mesh% alpha)
       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_x_to_jac(mesh,mesh % p_xmesh(ir))
       Enddo ! ir
    Endif
    Return
  End Subroutine setup_powmesh
  !!***********************************************************************
  !! given rmin,rmax,nr determine the parameter of the log-zero mesh
  !! r=a*(exp(x)-1),   x=i*h
  !! such that
  !! rmin=a*(exp(h)-1)
  !! rmax=a*(exp(h*n)-1)
  Subroutine  det_logzero_params(rmin,rmax,n,h,a)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)      :: rmin,rmax
    Real(t_real),Intent(out)     :: h,a
    !!LOCAL
    Character(*),Parameter::nsubfunc="det_logzero_params"    
    Logical(t_log) :: found
    Integer(t_int) :: MAXITER,niter
    Real(t_real)   :: q,s0,s,tol
    !!
    MAXITER=1000
    tol=1.e-10_t_real
    !!
    !! iterative solution of the coefficient equation
    q=(rmax-rmin)/rmin
    s0=1.0_t_real
    s=1.1_t_real
    found=.False.
    niter=0_t_int
    Do 
       s=(s*(1.0_t_real+q)-q)**(1.0_t_real/Real(n,t_real))
       !Print *,'s=',s
       If(Abs(s-s0)<1.0e-14_t_real) Then
          found=.True.
          Exit
       Endif
       If(niter>MAXITER) Then
          found=.False.
          Exit
       Endif
       s0=s
       niter=niter+1_t_int
    Enddo
    If(.Not.found) Then
       Write(errbuf,1)s,s0
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
1      Format("'Cannot solve coefficient equation: current s="&
            &,e30.15e3," last s=",e30.15e3,"!'")
    Endif
    !!
    h=Log(s)
    a=rmin/(Exp(h)-1.0_t_real)
    !Print *,'h =',h
    !Print *,'a =',a

    !! this must be zero
    s0=a*(Exp(h)-1.0_t_real)-rmin
    !Print *,'rmin,err=',rmin,s0
    !!
    If(Abs(s0)>tol)Then
       Write(errbuf,3) s0,rmin
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
3      Format("'Cannot solve coefficient equation  r(1)="&
            &,e30.15e3," rmin=",e30.15e3,"!'")

    Endif
    !! and this must be zero
    s0=a*(Exp(h*n)-1.0_t_real)-rmax
    !Print *,'rmax,err=',rmax,s0
    !!
    If(Abs(s0)>tol)Then
       Write(errbuf,2) s0,rmax
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
2      Format("'Cannot solve coefficient equation  r(N)="&
            &,e30.15e3," rmax=",e30.15e3,"!'")
    Endif
    !!

    !!
    Return
    !!
  End Subroutine det_logzero_params
  !!***********************************************************************
  !! 
  !!    xmesh(i)=i*h,  i=1,N
  !!    rmesh(i)=al*(exp(xmesh(i))-1)
  Subroutine setup_logzeromesh(mesh)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_logzeromesh"
    Integer(t_int) :: ir,n
    !!
    !!
    Call palloc('mesh % p_rmesh',mesh % p_rmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_xmesh',mesh % p_xmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)
    Call palloc('mesh % p_jmesh',mesh % p_jmesh &
         &,1_t_int,mesh % nr &
         &,nmodule,nsubfunc)

    mesh% Type=LOGZERO_MESH
    
    n=mesh% nr
    If(mesh% with_zero) n=mesh% nr-1

    Call det_logzero_params(mesh% rmin,mesh% rmax,n,mesh% hx,mesh% alpha)

    If(mesh% with_zero)Then

       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir-1,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_r_to_jac(mesh,mesh % p_rmesh(ir))
       Enddo ! ir
       !! make sure that first point is rmin, correction for roundoff
       mesh % p_xmesh(2)=mesh_r_to_x(mesh,mesh % rmin)
       mesh % p_rmesh(2)=mesh% rmin
       mesh % p_jmesh(2)=mesh_r_to_jac(mesh,mesh % rmin)
       !! make sure that last point is rmax
       mesh % p_xmesh(mesh% nr)=mesh_r_to_x(mesh,mesh % rmax)
       mesh % p_rmesh(mesh% nr)=mesh% rmax
       mesh % p_jmesh(mesh% nr)=mesh_r_to_jac(mesh,mesh % rmax)

    Else

       Do ir=1_t_int,mesh % nr
          mesh % p_xmesh(ir)=Real(ir,t_real)*mesh % hx
          mesh % p_rmesh(ir)=mesh_x_to_r(mesh,mesh % p_xmesh(ir))
          mesh % p_jmesh(ir)=mesh_r_to_jac(mesh,mesh % p_rmesh(ir))
       Enddo ! ir
       !! make sure that first point is rmin, correction for roundoff
       mesh % p_xmesh(1)=mesh_r_to_x(mesh,mesh % rmin)
       mesh % p_rmesh(1)=mesh% rmin
       mesh % p_jmesh(1)=mesh_r_to_jac(mesh,mesh % rmin)
       !! make sure that last point is rmax
       mesh % p_xmesh(mesh% nr)=mesh_r_to_x(mesh,mesh % rmax)
       mesh % p_rmesh(mesh% nr)=mesh% rmax
       mesh % p_jmesh(mesh% nr)=mesh_r_to_jac(mesh,mesh % rmax)
    Endif
    !!
    Return
  End Subroutine setup_logzeromesh
  !!***********************************************************************
  Subroutine  setup_bspline_mesh(mesh,k)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(inout) :: mesh
    !! desired order
    Integer(t_int) :: k
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_bspline_mesh"
!!$    Integer(t_int) :: mj
    Real(t_real),Pointer :: bcoef(:)
    !!
    Nullify(bcoef)
    !!
    Call pfree(mesh% p_q)
    Call pfree(mesh% p_knots)

    !!
    mesh% k=k
    mesh% p=k-1
!!$    mj=2*mesh% p+1
    !!
    Call palloc("mesh% p_q",mesh% p_q&
         &,1_t_int,(2_t_int*mesh% k-1_t_int)*mesh% nr,nmodule,nsubfunc)
    Call palloc("mesh% p_knots",mesh% p_knots,1_t_int,mesh% nr+mesh% k&
         &,nmodule,nsubfunc)

    Call palloc("bcoef",bcoef,1_t_int,mesh% nr,nmodule,nsubfunc)
    !!
    If(mesh% knot_x) Then
       Call set_natural_knots_X(mesh% nr,mesh% k,mesh% p_xmesh,mesh% p_knots)
       Call bsplint(mesh% p_xmesh,mesh% p_knots,mesh% nr,mesh% k,mesh% p_q&
            &,bcoef)
    Else
       Call set_natural_knots(mesh% nr,mesh% k,mesh% p_rmesh,mesh% p_knots)
       Call bsplint(mesh% p_rmesh,mesh% p_knots,mesh% nr,mesh% k,mesh% p_q&
            &,bcoef)
    Endif
    Call pfree(bcoef)
    !!
    Return
    !!
  End Subroutine setup_bspline_mesh
  !!***********************************************************************
  Subroutine makebsplint(mesh,bcoef)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)    :: mesh
    Real(t_real),Intent(inout) :: bcoef(*)
    !!LOCAL
    !!
    Call banslv(mesh% p_q,2*mesh% k-1_t_int&
         &,mesh% nr,mesh% k-1_t_int,mesh% k-1_t_int&
         &,bcoef)
    !!
    Return
    !!
  End Subroutine makebsplint
  !!***********************************************************************
  Integer(t_int) Function bsplindex(mesh,x)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)    :: mesh
    Real(t_real),Intent(in)      :: x
    !!LOCAL
    !!
    !!
    bsplindex=natural_knot_index(mesh% k,mesh% nr,mesh% hx,mesh% p_xmesh,x)
    !!
    Return
    !!
  End Function bsplindex
  !!***********************************************************************
!!$  Real(t_real) Function  bspvalue(mesh,bcoef,r)
!!$    !!
!!$    Implicit None  
!!$    !!IO
!!$    Type(T_Mesh),Intent(in)    :: mesh
!!$    Real(t_real),Intent(in)    :: bcoef(*)
!!$    Real(t_real),Intent(in)    :: r
!!$    !!LOCAL
!!$    Integer(t_int) :: i,index
!!$    Real(t_real)   :: r1,r2,x2,b1,b2,c1,c,x
!!$    !!
!!$    x=mesh_r_to_x(mesh,r)
!!$    index=natural_knot_index(mesh% k,mesh% nr&
!!$         &,mesh% hx,mesh% p_xmesh,x)
!!$    !!
!!$    bspvalue=0.0_t_real
!!$    If(index >=1 .And. index <= mesh% nr)Then
!!$       !!
!!$       bspvalue=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,x,index)
!!$       !!
!!$    Endif
!!$    !!
!!$    Return
!!$    !!
!!$  End Function bspvalue
!!$  !!***********************************************************************
  Real(t_real) Function  bspvalue(mesh,bcoef,index,r,r0,f0)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)    :: mesh
    Integer(t_int),Intent(in)  :: index
    Real(t_real),Intent(in)    :: bcoef(*)
    Real(t_real),Intent(in)    :: r
    Real(t_real),Intent(in)    :: r0,f0
    !!LOCAL
    Integer(t_int) :: i
    Real(t_real)   :: r1,r2,x2,b1,b2,c1,c,x,x1
    !!
    !!
    If(index==0)Then
       r1=mesh% p_rmesh(1)
       x1=mesh% p_xmesh(1)
       i=mesh% k !!TODO: This is not general, if no natural parametrization
       !! is used the first knot index may differ from k,
       !! but if index determination is used, round off may give index==0
       !! which is not what we want
       If(mesh% knot_x) Then
          b1=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,x1,i)
       Else
          b1=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,r1,i)
       Endif
       !!
       r2=mesh% p_rmesh(2)
       x2=mesh% p_xmesh(2)
       i=natural_knot_index(mesh% k,mesh% nr,mesh% hx,mesh% p_xmesh,x2)
       If(mesh% knot_x) Then
          b2=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,x2,i)
       Else
          b2=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,r2,i)
       Endif
       !!
       c1=(r-r2)*(b2-b1)/(r2-r1)
       c=b2-b1+c1 -(r-r1)*(b1-f0)/(r1-r0)
       bspvalue = b2+c1+(r-r2)*(c)/(r2-r0)
       !!
    else
       !!
       x=mesh_r_to_x(mesh,r)
       If(mesh% knot_x) Then
          bspvalue=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,x,index)
       Else
          bspvalue=bvalue0(mesh% p_knots,bcoef,mesh% nr,mesh% k,r,index)
       Endif
       !!
    Endif
    !!
    Return
    !!
  End Function bspvalue
  !!***********************************************************************
  Real(t_real) Function minterpol(mesh,degree,f,r)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)    :: mesh
    Integer(t_int),Intent(in)  :: degree
    Real(t_real),Intent(in)    :: r,f(*)
    !!LOCAL
    !!
    minterpol=dinterpol(degree&
         &,mesh% nr,mesh% p_rmesh&
         &,mesh% p_xmesh,mesh% hx&
         &,f,r&
         &,mesh_r_to_x(mesh,r))
    !!
    Return
    !!
  End Function minterpol
  !!***********************************************************************
  Integer(t_int) Function mintpl_coefs(degree,mesh,rr,coef)
    Implicit None 
    !!IO
    Type(T_Mesh),Intent(in)   :: mesh
    Integer(t_int),Intent(in) :: degree !! interpolation degree < 9
    Real(t_real),Intent(in)   :: rr      !! interpolation point
    Real(t_real),Intent(out)  :: coef(0:degree)
    !!LOCAL
    !!
    mintpl_coefs=interpl_coefs(degree,mesh% nr,mesh% p_rmesh&
         &,mesh% p_xmesh,mesh% hx&
         &,rr,mesh_r_to_x(mesh,rr),coef)
    !!
  End Function mintpl_coefs
  !!***********************************************************************
  Integer(t_int) Function rmintpl_coefs(degi,deg0,deg,mesh,rr,coef)
    Implicit None 
    !!IO
    Type(T_Mesh),Intent(in)   :: mesh
    Integer(t_int),Intent(in) :: degi !! interpolation degree < 9
    Integer(t_int),Intent(in) :: deg0   !! interpolation degree for r->0
    Integer(t_int),Intent(out)   :: deg !! the degree to be used for the
    !! interplation outside
    Real(t_real),Intent(in)   :: rr      !! interpolation point
    Real(t_real),Intent(out)  :: coef(0:degi)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rmintpl_coefs"
    !!
    If(deg0>degi) &
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//'deg0 < degi!')
    !!
    If(rr<mesh% p_rmesh(2) .And. mesh% with_zero )Then
       !! for the first interval make direct rmesh interpolation
       !! to assure a certain analytical behaviour
       rmintpl_coefs=1
       deg=deg0
       Call rinterpl_coefs(mesh% nr,mesh% p_rmesh,rmintpl_coefs,deg,coef,rr)
    Else
       deg=degi
       rmintpl_coefs=interpl_coefs(deg,mesh% nr,mesh% p_rmesh&
            &,mesh% p_xmesh,mesh% hx&
            &,rr,mesh_r_to_x(mesh,rr),coef)
    Endif
    
    !!
  End Function rmintpl_coefs
  !!***********************************************************************
  Integer(t_int) Function rmintpl_coefs1(degi,deg0,deg,mesh,rr,coef)
    !!
    !!
    Implicit None 
    !!IO
    Type(T_Mesh),Intent(in)   :: mesh
    Integer(t_int),Intent(in) :: degi !! interpolation degree < 9
    Integer(t_int),Intent(in) :: deg0   !! interpolation degree for r->0
    Integer(t_int),Intent(out)   :: deg !! the degree to be used for the
    !! interplation outside
    Real(t_real),Intent(in)   :: rr      !! interpolation point
    Real(t_real),Intent(out)  :: coef(0:degi)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rmintpl_coefs1"
    !!
    If(deg0>degi) &
         & Call terminate(TERM_CRASH,Trim(nsubfunc)//'deg0 < degi!')
    !!
    If(rr<mesh% p_rmesh(2) .And. mesh% with_zero )Then
       !! for the first interval make direct rmesh interpolation
       !! to assure a certain analytical behaviour
       rmintpl_coefs1=1
       deg=2_t_int
       Call rinterpl_coefs(mesh% nr,mesh% p_rmesh,rmintpl_coefs1,deg,coef,rr)
    Else If(rr<mesh% p_rmesh(3) .And. mesh% with_zero )Then
       !! for the first interval make direct rmesh interpolation
       !! to assure a certain analytical behaviour
       rmintpl_coefs1=1
       deg=3_t_int
       Call rinterpl_coefs(mesh% nr,mesh% p_rmesh,rmintpl_coefs1,deg,coef,rr)
    Else If(rr<mesh% p_rmesh(4) .And. mesh% with_zero )Then
       !! for the first interval make direct rmesh interpolation
       !! to assure a certain analytical behaviour
       rmintpl_coefs1=1
       deg=4_t_int
       Call rinterpl_coefs(mesh% nr,mesh% p_rmesh,rmintpl_coefs1,deg,coef,rr)
    Else
       deg=degi
       rmintpl_coefs1=interpl_coefs(deg,mesh% nr,mesh% p_rmesh&
            &,mesh% p_xmesh,mesh% hx&
            &,rr,mesh_r_to_x(mesh,rr),coef)
    Endif
    
    !!
  End Function rmintpl_coefs1
  !!***********************************************************************
  Integer(t_int) Function mbsp_coefs(mesh,rr,nderiv,BiAtX)
    Implicit None 
    !!IO
    Type(T_Mesh),Intent(in)   :: mesh
    Integer(t_int),Intent(in) :: nderiv
    Real(t_real),Intent(in)   :: rr      !! interpolation point
    Real(t_real),Intent(out)  :: BiAtX(1:mesh% k,nderiv)
    !!LOCAL
    Real(t_real)   :: work(((mesh% k+1)*(mesh% k+2))/2)
    Real(t_real)   :: xx

    !!
    xx=mesh_r_to_x(mesh,rr)
    mbsp_coefs=bsplindex(mesh,xx)
    If(mesh% knot_x) Then
       Call bsp_bspvd(mesh% nr,mesh% p_knots,mesh% k,nderiv,xx&
            &,mbsp_coefs,mesh% k,BiAtX,work,.False.)
    Else
       Call bsp_bspvd(mesh% nr,mesh% p_knots,mesh% k,nderiv,rr&
            &,mbsp_coefs,mesh% k,BiAtX,work,.False.)
    Endif
    mbsp_coefs=mbsp_coefs-(mesh% k-1)
    !!
  End Function mbsp_coefs
  !!***********************************************************************
!!$  Subroutine  mbspreinter(oldmesh,oldfunc,newmesh,newfunc)
!!$    !!
!!$    Implicit None  
!!$    !!IO
!!$    Type(T_Mesh),Intent(in)      :: newmesh,oldmesh
!!$    Real(t_real),Intent(in)      :: oldfunc(*)
!!$    Real(t_real),Intent(out)     :: newfunc(*)
!!$    !!LOCAL
!!$    Integer(t_int) :: ir
!!$    !!
!!$    Do ir=1_t_int,newmesh % nr
!!$       newfunc(ir)=bspvalue(oldmesh,oldfunc,newmesh% p_rmesh(ir))
!!$    End Do
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine mbspreinter
  !!***********************************************************************
  Subroutine  mreinter(degree,oldmesh,oldfunc,newmesh,newfunc)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: degree
    Type(T_Mesh),Intent(in)      :: newmesh,oldmesh
    Real(t_real),Intent(in)      :: oldfunc(*)
    Real(t_real),Intent(out)     :: newfunc(*)
    !!LOCAL
    Integer(t_int) :: ir
    !!
    Do ir=1_t_int,newmesh % nr
       newfunc(ir)=minterpol(oldmesh,degree,oldfunc,newmesh% p_rmesh(ir))
    End Do
    !!
    Return
    !!
  End Subroutine mreinter
  !!***********************************************************************
  Subroutine writefunc(nr,r,f,lunit,fname)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nr
    Integer(t_int),Intent(in)    :: lunit
    Character(*),Intent(in),Optional :: fname
    Real(t_real),Intent(in)      :: r(nr),f(nr)
    !!LOCAL
    Integer(t_int) :: ir
    !!
    If(Present(fname))Then
       Open(lunit,file=Trim(fname))
    Endif
    Do ir=1,nr
       Write(lunit,1)r(ir),f(ir)
    Enddo
    !!
    Write(lunit,2)
    !!
    If(Present(fname))Then
       Close(lunit)
    Endif
    !!
    Return
    !!
1   Format(3(e30.15e3,2x))
2   Format()
    !!
  End Subroutine writefunc
  !!***********************************************************************
  Subroutine write_mesh_info(lun,mesh)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Type(T_Mesh),Intent(in)      :: mesh
    !!LOCAL
    !!
    Write(lun, 99) 
    Write(lun,101) mesh% with_zero
    Write(lun,102) mesh% Type,Trim(MESH_TYPE(mesh% Type))
    Write(lun,103) mesh% nr
    Write(lun,104) mesh% rmin
    Write(lun,105) mesh% rmax
    Write(lun,106) mesh% alpha
    Write(lun,107) 
    !!
    Return
    !!
99  Format('# <start mesh>')
101 Format('# mesh% with_zero: ',l1)
102 Format('# mesh% type: ',i2,' (',a,')')
103 Format('# mesh% nr: ',i6)
104 Format('# mesh% rmin: ',f30.15)
105 Format('# mesh% rmax: ',f30.15)
106 Format('# mesh% alpha: ',f30.15)
107 Format('# <end mesh>')
    !!
  End Subroutine write_mesh_info
  !!***********************************************************************
  Subroutine read_mesh_info(lun,mesh)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Type(T_Mesh),Intent(inout)   :: mesh
    !!LOCAL
    Character(10000)   :: buffer
    Integer(t_int) :: j,j2
    !!
    readloop: Do 
       !!
       Read(lun,3) buffer
       !!
       j=Index(buffer,'mesh% type: ')
       If(j > 0) Then
          j=j+11
          j2=Index(buffer,'(')-1
          Read(buffer(j:j2),*)mesh% Type
          !Print *,mesh% Type,'<',buffer(j:j2),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'mesh% rmax: ')
       If(j > 0) Then
          j=j+11
          Read(buffer(j:),*)mesh% rmax
          !Print *,mesh% rmax,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'mesh% rmin: ')
       If(j > 0) Then
          j=j+11
          Read(buffer(j:),*)mesh% rmin
          !Print *,mesh% rmin,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'mesh% nr: ')
       If(j > 0) Then
          j=j+9
          Read(buffer(j:),*)mesh% nr
          !Print *,mesh% nr,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'mesh% alpha: ')
       If(j > 0) Then
          j=j+12
          Read(buffer(j:),*)mesh% alpha
          !Print *,mesh% alpha,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'mesh% with_zero: ')
       If(j > 0) Then
          j=j+16
          Read(buffer(j:),*)mesh% with_zero
          !Print *,mesh% with_zero,'<',Trim(buffer(j:)),'>'
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<end mesh>')
       If(j > 0) Then
          Exit readloop
       Endif
       !!
       !Print *,'[',Trim(buffer),']'
       !!
    End Do readloop
    !!
    !!
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_mesh_info
  !!***********************************************************************
  Subroutine write_mesh_function(lun,mesh,func,txt)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Character(*),Intent(in)      :: txt
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: func(:)
    !!LOCAL
    Integer(t_int) :: ir
    !!
    Write(lun, 99) Trim(txt)
    Do ir=1,mesh% nr
       Write(lun,101) mesh% p_rmesh(ir),func(ir)
    Enddo
    Write(lun,107) Trim(txt)
    Write(lun,108) 
    !!
    Return
    !!
99  Format('# <start function ',a,'>')
101 Format(2(e30.15e3,2x))
107 Format('# <end function ',a,'>')
108 Format(' ')
    !!
  End Subroutine write_mesh_function
  !!***********************************************************************
  Subroutine read_mesh_function_rf(lun,r,func)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Real(t_real),Intent(inout)   :: r(:),func(:)
    !!LOCAL
    Character(10000)   :: buffer
    Integer(t_int) :: ir,j
    !!
    r(:)=0.0_t_real
    func(:)=0.0_t_real
    ir=0
    readloop: Do
       !!       
       Read(lun,3) buffer
       !!
       j=Index(buffer,'<end function')
       If(j > 0) Then
          Exit readloop
       Endif
       !!
       ir=ir+1
       Read(buffer,*)r(ir),func(ir)
       !Print *,ir,d,func(ir)
       !!
       !Print *,'[',Trim(buffer),']'
       !!
    End Do readloop
    !!
    !!
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_mesh_function_rf
  !!***********************************************************************
  Subroutine read_mesh_function(lun,func)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lun
    Real(t_real),Intent(inout)   :: func(:)
    !!LOCAL
    Character(10000)   :: buffer
    Integer(t_int) :: ir,j
    Real(t_real)   :: d
    !!
    func(:)=0.0_t_real
    ir=0
    readloop: Do
       !!       
       Read(lun,3) buffer
       !!
       j=Index(buffer,'<end function')
       If(j > 0) Then
          Exit readloop
       Endif
       !!
       ir=ir+1
       Read(buffer,*)d,func(ir)
       d=d
       !Print *,ir,d,func(ir)
       !!
       !Print *,'[',Trim(buffer),']'
       !!
    End Do readloop
    !!
    !!
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_mesh_function
  !!***********************************************************************
  !! calculate the derivative function
  Subroutine dfspline1(p,nr,r,f,dfs)
    !!
    Implicit None 
    !!
    !!IO
    Integer(t_int),Intent(in)    :: nr
    Integer(t_int),Intent(in)    :: p
    Real(t_real),Intent(in)      :: r(:)
    Real(t_real),Intent(in)      :: f(:)
    Real(t_real),Intent(out)     :: dfs(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="dfspline1"
    Integer(t_int) :: k,i
    Real(t_real),Pointer :: work(:),knots(:),p_q(:)
    !!
    Nullify(work,knots,p_q)
    !!
    k=p+1
    !!
    Call palloc("work",work,1_t_int,nr,nmodule,nsubfunc)

    !!
    !!

        
    Call palloc("knots",knots,1_t_int,nr+k,nmodule,nsubfunc)
    Call palloc("p_q",p_q,1_t_int,(2_t_int*k-1_t_int)*nr,nmodule,nsubfunc)

    Call set_natural_knots_x(nr,k,r,knots)

    Call bsplint(r,knots,nr,k,p_q,work)
    !!
    work=f
    Call banslv(p_q,2*k-1_t_int,nr,p,p,work)
    !!
    !!
    !!
    !!
    !!
    Do i=1,nr
       dfs(i)=bvalue(knots,work,nr,k,r(i),1_t_int)
    Enddo
    !!
    !! 
    !!
    !!
    Call pfree(p_q)
    Call pfree(knots)
    Call pfree(work)
    !!
    !!
    Return
    !!
    !!
  End Subroutine dfspline1
  !!***********************************************************************
  !! calculate the derivative function
  Subroutine dfspline(p,nr,r,f,dfs,d2fs)
    !!
    Implicit None 
    !!
    !!IO
    Integer(t_int),Intent(in)    :: nr
    Integer(t_int),Intent(in)    :: p
    Real(t_real),Intent(in)      :: r(:)
    Real(t_real),Intent(in)      :: f(:)
    Real(t_real),Intent(out)     :: dfs(:)
    Real(t_real),Intent(out)     :: d2fs(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="dfspline"
    Integer(t_int) :: k,i
    Real(t_real),Pointer :: work(:),knots(:),p_q(:)
    !!
    Nullify(work,knots,p_q)
    !!
    k=p+1
    !!
    Call palloc("work",work,1_t_int,nr,nmodule,nsubfunc)

    !!
    !!

        
    Call palloc("knots",knots,1_t_int,nr+k,nmodule,nsubfunc)
    Call palloc("p_q",p_q,1_t_int,(2_t_int*k-1_t_int)*nr,nmodule,nsubfunc)

    Call set_natural_knots_x(nr,k,r,knots)

    Call bsplint(r,knots,nr,k,p_q,work)
    !!
    work=f
    Call banslv(p_q,2*k-1_t_int,nr,p,p,work)
    !!
    !!
    !!
    !!
    !!
    Do i=1,nr
       dfs(i)=bvalue(knots,work,nr,k,r(i),1_t_int)
       d2fs(i)=bvalue(knots,work,nr,k,r(i),2_t_int)
    Enddo
    !!
    !! 
    !!
    !!
    Call pfree(p_q)
    Call pfree(knots)
    Call pfree(work)
    !!
    !!
    Return
    !!
    !!
  End Subroutine dfspline
  !!***********************************************************************
  !! calculate the derivative function
  Subroutine dfspline3(p,nr,r,f,dfs,d2fs,d3fs)
    !!
    Implicit None 
    !!
    !!IO
    Integer(t_int),Intent(in)    :: nr
    Integer(t_int),Intent(in)    :: p
    Real(t_real),Intent(in)      :: r(:)
    Real(t_real),Intent(in)      :: f(:)
    Real(t_real),Intent(out)     :: dfs(:)
    Real(t_real),Intent(out)     :: d2fs(:)
    Real(t_real),Intent(out)     :: d3fs(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="dfspline3"
    Integer(t_int) :: k,i
    Real(t_real),Pointer :: work(:),knots(:),p_q(:)
    !!
    Nullify(work,knots,p_q)
    !!
    k=p+1
    !!
    Call palloc("work",work,1_t_int,nr,nmodule,nsubfunc)

    !!
    !!

        
    Call palloc("knots",knots,1_t_int,nr+k,nmodule,nsubfunc)
    Call palloc("p_q",p_q,1_t_int,(2_t_int*k-1_t_int)*nr,nmodule,nsubfunc)

    Call set_natural_knots_x(nr,k,r,knots)

    Call bsplint(r,knots,nr,k,p_q,work)
    !!
    work=f
    Call banslv(p_q,2*k-1_t_int,nr,p,p,work)
    !!
    !!
    !!
    !!
    !!
    Do i=1,nr
       dfs(i)=bvalue(knots,work,nr,k,r(i),1_t_int)
       d2fs(i)=bvalue(knots,work,nr,k,r(i),2_t_int)
       d3fs(i)=bvalue(knots,work,nr,k,r(i),3_t_int)
    Enddo
    !!
    !! 
    !!
    !!
    Call pfree(p_q)
    Call pfree(knots)
    Call pfree(work)
    !!
    !!
    Return
    !!
    !!
  End Subroutine dfspline3
  !!***********************************************************************
  Logical(t_log) Function meshesEqual(mesha,meshb)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: mesha,meshb
    !!LOCAL
    !Character(*),Parameter::nsubfunc="meshesEqual"
    Real(t_real)   :: Mrmin,Mrmax,Malpha
    Real(t_real),Parameter :: tol=1.0e-6_t_real
    !!
    Mrmin=Max(Abs(mesha% rmin),Abs(meshb% rmin))
    If(Mrmin==0.0_t_real)Then
       Mrmin=1.0_t_real
    Endif
    Mrmax=Max(Abs(mesha% rmax),Abs(meshb% rmax))
    If(Mrmax==0.0_t_real)Then
       Mrmax=1.0_t_real
    Endif
    Malpha=Max(Abs(mesha% alpha),Abs(meshb% alpha))
    If(Malpha==0.0_t_real)Then
       Malpha=1.0_t_real
    Endif
    !!
    meshesEqual=( &
         &(mesha% with_zero .eqv. meshb% with_zero) .And. &
         &(mesha% type == meshb% type) .And. &
         &(mesha% nr == meshb% nr) .And. &
         &(Abs(mesha% rmin - meshb% rmin)   < tol*Mrmin ) .And. &
         &(Abs(mesha% rmax - meshb% rmax)   < tol*Mrmax ) .And. &
         &(Abs(mesha% alpha - meshb% alpha) < tol*Malpha)       &
         &)
    !!
    !!
    !!
    Return
    !!
  End Function meshesEqual
  !!***********************************************************************

End Module mesh
