Module molcore
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  Use clock
  Use optionstest
  Use tsymmetry
  Use tkohnsham
  Use torbitalbasis
  Use terg
  Use tforces
  Use tmesh
  Use tmol
  Use tloi
  Use diagonalize
  Use integrate1d
  Use prnt
  Use diracdefs 
  Use matserv   
  Use sort
  Use atomdefs  
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="MOLCORE"  
  !!
  Public :: mol_core,mol_core_trans,mol_core_trans_gcv,&
       &mol_core_trans_gcv_noH
  !!
  !!***********************************************************************
  !!
Contains

  !!***********************************************************************
  Subroutine mol_core(mspin,structure,basis,kohnsham,loi,rel,etot,forces&
       &,prot,force_loop)
    !!
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Kohnsham),Intent(inout):: kohnsham
    Type(T_Etot),Intent(inout)   :: etot
    Type(T_Forces),Intent(inout) :: forces
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Loi),Intent(in)       :: loi
    Logical(t_log),Intent(in)    :: rel,prot,force_loop
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="mol_core"
    Logical(t_log) :: tstdia
    Integer(t_int) :: isa,ia,ib,issa,ioci,ms,ioci0,iz,nmain,l,ica,deg,m
    Integer(t_int) :: ndim,i0,ic,imax,nchidim,k,i,icb
    Real(t_real)   :: con,ee,tol,dmax,emax,occu,fspin
    Integer(t_int),Pointer :: p_in(:)
    Real(t_real),Pointer   :: p_ec(:),p_w(:)
    Real(t_real),Pointer   :: p_hc(:,:),p_sc(:,:)
    !! testdiagno
    Real(t_real),Pointer   :: p_hc1(:,:),p_sc1(:,:),p_dhcc(:,:,:)
    !!
    tol=1.0e-10_t_real
    !!
    If(rel)Then
       Call rel_mol_core(mspin,structure,basis,kohnsham,loi,rel,etot% ecore&
            &,prot)
       Return
    Endif
    !!
    Nullify(p_hc)
    Nullify(p_dhcc)
    Nullify(p_sc)
    Nullify(p_ec)
    Nullify(p_w)
    Nullify(p_in)
    Nullify(p_hc1)
    Nullify(p_sc1)
    !!
    fspin=3.0_t_real-Real(mspin,t_real)
    !!
    If(force_loop)Then
       !!
       forces% p_cplforces(:,:)=0.0_t_real
       !!
       !!
    Endif
    !!
    kohnsham% p_nczz(:,:,:,:)=0.0_t_real
    !!
    tstdia=isoption('TEST_DIAGNO')
    !!
    !!
    etot% ecore=0.0_t_real
    !!
    isaloop: Do isa=1,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !!
       con=structure% p_conao(issa)
       !!
       ioci0=loi% p_ociccindex(isa)
       !!
       ndim=kohnsham% p_cblocklength(isa)
       nchidim=kohnsham% p_chiblocklength(isa)
       !!
       If(ndim==0_t_int) Cycle isaloop
       !!
       Call palloc('p_hc',p_hc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Call palloc('p_sc',p_sc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Call palloc('p_ec',p_ec ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_w' ,p_w  ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_in',p_in ,1_t_int,ndim, nmodule,nsubfunc)
       !!
       If(force_loop)Then
          Call palloc('p_dhcc',p_dhcc,1_t_int,3_t_int,1_t_int,ndim&
               & ,1_t_int,ndim,nmodule,nsubfunc)
       Endif
       !!
       If(tstdia) Then
          Call palloc('p_hc1',p_hc1 ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
          Call palloc('p_sc1',p_sc1 ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Endif
       !!
       msloop: Do ms=1,mspin
          !!
          p_hc(:,:)=0.0_t_real
          p_sc(:,:)=0.0_t_real
          !!
          ioci=ioci0
          Do ia=1,ndim
             Do ib=1,ndim
                ioci=ioci+1_t_int
                !!
                p_hc(ia,ib)=p_hc(ia,ib)+loi% p_oci_cHc(ioci,ms)
                p_sc(ia,ib)=p_sc(ia,ib)+loi% p_oci_cc0(ioci)
                !!
             End Do
          End Do
          !!
          If(prot)Then
             Call printmat('Hcc',"(f12.5,' ')","('     0       ')"&
                  &,p_hc(:,:),ndim,ndim,tol)
             Call printmat('Scc',"(f12.5,' ')","('     0       ')"&
                  &,p_sc(:,:),ndim,ndim,tol)
          Endif
          !!
          !!diagonalization
          !!
          If(tstdia) Then
             p_hc1=p_hc
             p_sc1=p_sc
          Endif
          !!
          Call rdiagno(ndim,p_hc,p_sc,p_ec)
          !!
          If(tstdia)Then
             Call rtestdia(ndim,ndim,p_hc1,p_sc1,p_hc,p_ec,1.0e-13_t_real)
          Endif
          !!
          If(prot)Then
             Call printmat('CC0',"(f12.5,' ')","('     0       ')"&
                  &,p_hc(:,:)**2,ndim,ndim,tol)
             Do ia=1,ndim
                Write(outfile,10) ia,p_ec(ia)
10              Format('Ec0(',i3,')=',f30.15)
             Enddo
          Endif
          !!
          !! search solutions according to character
          !!
          p_in(1:ndim)=0
          i0=0
          iz=kohnsham% p_chiblockstart(isa)-1
          emax=-Huge(1.0_t_real)
          Do ica=1,basis% p_nRchi(issa)
             nmain=basis% p_nnRc(ica,issa)
             l=basis% p_lRc(ica,issa)
             deg=2*l+1
             occu=basis% p_Rcoreoccu(ica,issa,ms)/deg*fspin
             Do m=1,deg
                If(prot) Write(outfile,"(i1,a1,'state ',i2,1x)",advance='no') &
                     &nmain,lqnchar(l),m
                Do ic=1,ndim
                   !! w is weight of ica-th nl-state in ic-th solution
                   p_w(ic)=p_hc(i0+m,ic)**2
                   if(prot)  Write(outfile,"(f10.5,' ')",advance='no') p_w(ic)
                Enddo
                If(prot)  Write(outfile,*)
                dmax=0.0_t_real
                imax=-1
                Do ic=1,ndim
                   If( (p_in(ic)==0) .And. ( p_w(ic)>dmax) ) Then
                      imax=ic
                      dmax=p_w(ic)
                   Endif
                Enddo
                If(imax<0)Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'Cannot find a solution for some core states ?!'")
                Endif
                !!
                p_in(imax)=1
                ee=p_ec(imax)
                !! coeficients of occupied solutions
                kohnsham% p_nczz(1:ndim,i0+m,isa,ms)=p_hc(1:ndim,imax)
                !! energies of the diagonalized core
                kohnsham% p_hc(iz+i0+m,ms)=ee
                kohnsham% p_coccu(i0+m,isa,ms)=occu
                !!
                etot% ecore=etot% ecore+ee*occu*con
                !! highest occupied energy for tests later
                emax=Max(emax,ee)
                !!
                If(prot)  Write(outfile,16) imax,dmax,ee,occu
16              Format('sol_max=',i5,' wmax=',g12.5,' e=',g12.5,' occu=',g12.5)
                !!
             Enddo !! m
             i0=i0+deg
          Enddo !! ica
          !!
          Do ic=1,ndim
             If( (p_in(ic)==0) .And. (p_ec(ic) < emax))Then
                Write(outfile,"('core state ',i5,' energy ',g12.5,' HOMO ',g12.5)")&
                     &ic,p_ec(ic),emax
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg="'Aufbau principle violated by some core states ?!'")
             Endif
          Enddo
          !!
          Do ica=1,ndim
             Do icb=1,ndim
                !! ib is right index but Dot_product makes left
                !! factor conjg -> thus other order in next line
                kohnsham% p_ncfc(ica,icb,isa,ms)=&
                     & Dot_product(kohnsham% p_nczz(icb,1:nchidim,isa,ms)&
                     &            ,kohnsham% p_nczz(ica,1:nchidim,isa,ms))
             Enddo
          Enddo
          !!
          If(prot)Then
             !!
             Call printmat('nczz',"(f12.5,' ')","('     0       ')"&
                  &,kohnsham% p_nczz(1:ndim,1:nchidim,isa,ms),ndim,nchidim,tol)
             !!
             Do ia=1,nchidim
                Write(outfile,11) ia,kohnsham% p_hc(iz+ia,ms)
11              Format('Ec(',i2,')=',f30.15)
             Enddo
             !!
             !!
          Endif
          !!
          If(force_loop)Then
             !!
             p_dhcc(:,:,:)=0.0_t_real
             !!
             ioci=ioci0
             Do ia=1,ndim
                Do ib=1,ndim
                   ioci=ioci+1_t_int
                   !!
                   p_dhcc(:,ia,ib)=p_dhcc(:,ia,ib)+loi% p_oci_gcHc(:,ioci,ms)
                   !!
                End Do
             End Do
             !!
             !Do k=1,3
             !Call printmat('dHcc',"(f12.5,' ')","('     0       ')"&
             !     &,p_dHcc(k,1:ndim,1:ndim),ndim,ndim,tol)
             !Enddo
             !!
             !!
             !!
             Do k=1,3
                ee=0.0_t_real
                Do i=1,nchidim
                   ee=ee+&
                        &2.0_t_real&
                        &*Dot_product(kohnsham% p_nczz(1:ndim,i,isa,ms)&
                        &,Matmul(p_dhcc(k,1:ndim,1:ndim)&
                        &,kohnsham% p_nczz(1:ndim,i,isa,ms)))&
                        &*kohnsham% p_coccu(i,isa,ms)
                Enddo
                forces% p_cplforces(k,isa)=forces% p_cplforces(k,isa)+ee
             Enddo
             !!
             !Print *,'trace=',forces% p_cplforces(3,isa)
             !Stop
          Endif
          !!
          !!
       Enddo msloop
       !!
       Call pfree(p_hc)
       Call pfree(p_sc)
       Call pfree(p_ec)
       Call pfree(p_w)
       Call pfree(p_in)
       Call pfree(p_sc1)
       Call pfree(p_hc1)
       Call pfree(p_dhcc)
       !!
    End Do isaloop
    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_core
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine  sph_aver(mspin,structure,basis,kohnsham,loi)
    !!
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Kohnsham),Intent(inout):: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Loi),Intent(in)       :: loi
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="sph_aver"
    Integer(t_int) :: isa,issa,ioci0,ioci,ms,ndim
    !!
    Integer(t_int) :: ica,nmaina,lca,mca
    Integer(t_int) :: icb,nmainb,lcb,mcb,l,maxldim
    Integer(t_int) :: ldim(0:basis% lcmx)
    Integer(t_int) :: ninda(0:basis% lcmx)
    Integer(t_int) :: nindb(0:basis% lcmx)
    Real(t_real)   :: tol
    Real(t_real),Pointer   :: p_hc(:,:),p_sc(:,:)
    Real(t_real),Pointer :: p_Hnn(:,:,:),p_Snn(:,:,:)! n,n,l
    !!
    !!
    Nullify(p_Hnn)
    Nullify(p_Snn)
    !!
    tol=1.0e-10_t_real
    !!
    Nullify(p_hc)
    Nullify(p_sc)
    !!
    !!
    isaloop: Do isa=1,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !!
       ioci0=0_t_int
       If(isa>1) ioci0=Sum(kohnsham% p_cblocklength(1:isa-1)**2)
       !!
       ldim(:)=0_t_int
       Do l=0,basis% lcmx
          Do ica=1,basis% p_nRc(issa)
             nmaina=basis% p_nnRc(ica,issa)
             lca=basis% p_lRc(ica,issa)
             If(l == lca) Then
                ldim(l)=ldim(l)+1
             Endif
          Enddo
       Enddo
       Print "('ldim:',100(a1,'=',i2,2x))"&
            &,(lqnchar(l),ldim(l),l=0,basis% lcmx)
       maxldim=Maxval(ldim(0:basis% lcmx))
       Print "('max_ldim=',i3)",maxldim
       !!
       Call palloc('p_Hnn', p_Hnn ,1_t_int,maxldim ,1_t_int,maxldim&
            &,0_t_int,basis% lcmx ,nmodule,nsubfunc)
       Call palloc('p_Snn', p_Snn ,1_t_int,maxldim ,1_t_int,maxldim&
            &,0_t_int,basis% lcmx ,nmodule,nsubfunc)
       !!
       msloop: Do ms=1,mspin
          !!
          p_Snn(:,:,:)=0.0_t_real
          p_Hnn(:,:,:)=0.0_t_real
          !!
          ninda(:)=0_t_int
          ioci=ioci0
          Do ica=1,basis% p_nRc(issa)
             nmaina=basis% p_nnRc(ica,issa)
             lca=basis% p_lRc(ica,issa)
             ninda(lca)=ninda(lca)+1
             Do mca=1,2*lca+1
                nindb(:)=0_t_int
                Do icb=1,basis% p_nRc(issa)
                   nmainb=basis% p_nnRc(icb,issa)
                   lcb=basis% p_lRc(icb,issa)
                   nindb(lcb)=nindb(lcb)+1
                   Do mcb=1,2*lcb+1
                      ioci=ioci+1
                      If(lca == lcb .And. mca == mcb) Then
                         Print 1112,ica,nmaina,lqnchar(lca),mca-lca-1&
                              &,icb,nmainb,lqnchar(lcb),mcb-lcb-1&
                              &,ioci&
                              &,ninda(lca),nindb(lcb)

                         p_Hnn(ninda(lca),nindb(lcb),lca)=&
                              &p_Hnn(ninda(lca),nindb(lcb),lca)&
                              &+loi% p_oci_cHc(ioci,ms)/(2*lca+1)
                         p_Snn(ninda(lca),nindb(lcb),lca)=&
                              &p_Snn(ninda(lca),nindb(lcb),lca)&
                              &+loi% p_oci_cc0(ioci)/(2*lca+1)
                      Endif
                   Enddo
                Enddo
             Enddo
          Enddo
1112      Format(i5,1x,i1,a1,sp,i2,ss,i5,1x,i1,a1,sp,i2,ss,':',i6,' i ',i5,i5)
          !!
          dialloop: Do l=0,basis% lcmx
             !!
             ndim=ldim(l)
             !!
             If(ndim<1)Cycle dialloop
             !!
             Write(outfile,1113) l,ndim
1113         Format('subshells l=',i2,' dimension=',i3)
             !!
             !! allocate diagonalization arrays
             Call palloc('p_hc',p_hc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
             Call palloc('p_sc',p_sc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
             !!
             !!
             p_hc(:,:)=p_Hnn(1:ndim,1:ndim,l)
             p_sc(:,:)=p_Snn(1:ndim,1:ndim,l)
             !!
             Call printmat('Hnn',"(f12.5,' ')","('     0       ')"&
                  &,p_hc(:,:),ndim,ndim,tol)
             Call printmat('Snn',"(f12.5,' ')","('     0       ')"&
                  &,p_sc(:,:),ndim,ndim,tol)
          End Do dialloop
       End Do msloop
    End Do isaloop
    !!
    Call pfree(p_hc)
    Call pfree(p_sc)
    !!
    Call pfree(p_Snn)
    Call pfree(p_Hnn)
    !!
    Stop 'sph_aver'
    !!
    Return
    !!
  End Subroutine sph_aver
#endif
  !!***********************************************************************
  Subroutine rel_mol_core(mspin,structure,basis,kohnsham,loi,rel,ecore,prot)
    !!
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Kohnsham),Intent(inout):: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Loi),Intent(in)       :: loi
    Logical(t_log),Intent(in)    :: rel,prot
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(out)     :: ecore
    !!LOCAL
    Character(*),Parameter::nsubfunc="mol_core"
    Logical(t_log) :: tstdia,opencore
    Integer(t_int) :: isa,ia,ib,issa,ioci,ioci0,iz,i0,ica,j,kap,ll,nmain,m
    Integer(t_int) :: ndim,nchidim,ic,imax,icb,degl
    Real(t_real)   :: con,ee,tol,emax,dmax,fspin,occuN,occuM,mav,wc
    Integer(t_int),Pointer :: p_in(:),p_index(:),p_inc(:)
    Real(t_real),Pointer   :: p_ec(:),p_w(:),p_spchar(:),p_occu(:)
    Complex(t_real),Pointer   :: p_hc(:,:),p_sc(:,:)
    !! testdiagno
    Complex(t_real),Pointer   :: p_hc1(:,:),p_sc1(:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    tol=1.0e-10_t_real
    !!
    If(.Not. rel) Return
    !!
    Nullify(p_hc)
    Nullify(p_sc)
    Nullify(p_ec)
    Nullify(p_w,p_spchar,p_occu)
    Nullify(p_in)
    Nullify(p_index,p_inc)
    Nullify(p_hc1)
    Nullify(p_sc1)
    !!
    fspin=3.0_t_real-Real(mspin,t_real)
    !!
    !!
    tstdia=isoption('TEST_DIAGNO')
    !!
    !!
    ecore=0.0_t_real
    !!
    isaloop: Do isa=1,structure% nsite
       !!
       issa=structure% p_isrt(isa)
       !issaIO=structure% p_isrtTOIOisrt(issa)
       con=structure% p_conao(issa)
       !!
       ioci0=loi% p_ociccindex(isa)
       !!
       ndim  =kohnsham% p_cblocklength(isa)
       nchidim=kohnsham% p_chiblocklength(isa)
       !!
       If(ndim==0_t_int) Cycle isaloop
       !!
       Call palloc('p_hc',p_hc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Call palloc('p_sc',p_sc ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Call palloc('p_ec',p_ec ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_w' ,p_w  ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_spchar' ,p_spchar  ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_occu' ,p_occu  ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_in',p_in ,1_t_int,ndim, nmodule,nsubfunc)
       Call palloc('p_index',p_index ,1_t_int,2*ndim, nmodule,nsubfunc)
       Call palloc('p_inc',p_inc ,1_t_int,ndim, nmodule,nsubfunc)
       !!
       If(tstdia) Then
          Call palloc('p_hc1',p_hc1 ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
          Call palloc('p_sc1',p_sc1 ,1_t_int,ndim ,1_t_int,ndim,nmodule,nsubfunc)
       Endif
       !!
       !!
       p_hc(:,:)=c0
       p_sc(:,:)=c0
       !!
       ioci=ioci0
       Do ia=1,ndim
          Do ib=1,ndim
             ioci=ioci+1_t_int
             !!
             p_hc(ia,ib)=p_hc(ia,ib)+loi% p_roci_cHc(ioci)
             p_sc(ia,ib)=p_sc(ia,ib)&
                  &+Cmplx(loi% p_oci_cc0(ioci),0.0_t_real,t_real)
             !!
          End Do
       End Do
       !!
       If(prot)Then
          Call printmat('Re Hcc',"(f12.5,' ')","('     0       ')"&
               &,Real(p_hc(:,:),t_real),ndim,ndim,tol)
          Call printmat('Im Hcc',"(f12.5,' ')","('     0       ')"&
               &,Aimag(p_hc(:,:)),ndim,ndim,tol)
          Call printmat('Re Scc',"(f12.5,' ')","('     0       ')"&
               &,Real(p_sc(:,:),t_real),ndim,ndim,tol)
          Call printmat('Im Scc',"(f12.5,' ')","('     0       ')"&
               &,Aimag(p_sc(:,:)),ndim,ndim,tol)
       Endif
       !!
       !!
       If(.False.)Then
          Call corereltest(structure,basis,p_hc,isa,ndim)
       Endif

       !!
       !!diagonalization
       !!
       If(tstdia) Then
          p_hc1=p_hc
          p_sc1=p_sc
       Endif
       !!
       Call cdiagno(ndim,p_hc,p_sc,p_ec)
       !!
       If(tstdia)Then
          Call ctestdia(ndim,ndim,p_hc1,p_sc1,p_hc,p_ec,1.0e-13_t_real)
       Endif
       !!
       If(prot)Then
          Call printmat('Re CC0',"(f12.5,' ')","('     0       ')"&
               &,Real(p_hc(:,:),t_real),ndim,ndim,tol)
          Call printmat('Im CC0',"(f12.5,' ')","('     0       ')"&
               &,Aimag(p_hc(:,:)),ndim,ndim,tol)
          Do ia=1,ndim
             Write(outfile,10) ia,p_ec(ia)
10           Format('Ec0(',i3,')=',f30.15)
          Enddo
       Endif
       !!
       !!
       !! search solutions according to character
       !!
       !!
       !!
       p_in(1:ndim)=0
       i0=0
       iz=kohnsham% p_chiblockstart(isa)-1
       emax=-Huge(1.0_t_real)
       Do ica=1,basis% p_nRchi(issa)
          nmain=basis% p_nnRc(ica,issa)
          kap=basis% p_kapRc(ica,issa)
          j=2*Abs(kap)-1
          ll=(Abs(2*kap+1)-1)/2
          degl=2*(2*ll+1)
          !!total charge
          occuN=Sum(basis% p_Rcoreoccu(ica,issa,1:mspin))*fspin
          occuM=basis% p_Rcoreoccu(ica,issa,1)-basis% p_Rcoreoccu(ica,issa,2)
          opencore=(occuN<(degl-1.0e-6_t_real))
          !!
          If(kap>0) Cycle
          !!
          If(prot) Write(outfile,"(i1,a1,i1,'/2 N=',f12.5,' M=',f12.5)")&
               &nmain,lqnchar(ll),j,occuN,occuM
          !!
          !!
          !find 2*degl large overlap levels and later check If i0+1:i0+degl are
          !amongst them
          !!
          p_index=ndim+1_t_int
          p_inc=0_t_int
          Do m=1,degl
             !If(prot)&
             !     & Write(outfile,"(i1,a1,i1,'/2 state ',i2,' i0+m=',i5)")&
             !     &nmain,lqnchar(ll),j,m,i0+m
             dmax=0.0_t_real
             imax=-1
             Do ic=1,ndim
                If(p_in(ic)/=0) Cycle
                If(p_inc(ic)/=0) Cycle
                wc=Abs(p_hc(i0+m,ic))**2
                If(( wc>dmax) ) Then
                   imax=ic
                   dmax=wc
                Endif
             Enddo
             If(imax<0)Then
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg="'Cannot find a solution for some core states1 ?!'")
             Endif
             p_inc(imax)=1
             p_index(m)=imax
          Enddo
          !!
          Do m=1,degl
             !If(prot)&
             !     & Write(outfile,"(i1,a1,i1,'/2 state ',i2,' i0+m=',i5)")&
             !     &nmain,lqnchar(ll),j,m,i0+m
             dmax=-1.0_t_real
             imax=-1
             Do ic=1,ndim
                If(p_in(ic)/=0) Cycle
                If(p_inc(ic)/=0) Cycle
                wc=Abs(p_hc(i0+m,ic))**2
                If(( wc>dmax) ) Then
                   imax=ic
                   dmax=wc
                Endif
             Enddo
             If(imax>0)Then
                p_inc(imax)=1
                p_index(m+degl)=imax
             Endif
          Enddo
          !!
          Call ipiksrt1(p_index,2*degl)
          !Write(outfile,"('indices=',100i7)") p_index(1:2*degl)
          !!
          Do m=1,degl
             imax=p_index(m)
             p_in(imax)=1

             p_spchar(m)=Real(Dot_product(p_hc(1:ndim,imax),&
                  &Matmul(loi% p_cbSc0(1:ndim,1:ndim,isa),p_hc(1:ndim,imax)))&
                  &,t_real)

             If(prot) Write(outfile,"(i1,a1,i1,'/2 state ',i2,' sol_max=',i5&
                  &,' e=',g12.5,' spinchar=',g12.5)") &
                  &nmain,lqnchar(ll),j,m,imax,p_ec(imax),p_spchar(m)
          Enddo
          !!
          mav=Sum(p_spchar(1:degl))/degl
          If(mspin==2.And. opencore)Then
             p_occu(1:degl)=(occuN+(occuM-occuN*mav)&
                  &/(Sum(p_spchar(1:degl)**2)/degl -mav**2)&
                  &*(p_spchar(1:degl)-mav))/degl
          Else
             p_occu(1:degl)=occuN/degl
          Endif
          !!
          !Write(outfile,"('indices=',100i7)") p_index(1:degl)
          !Write(outfile,"('in     =',100i7)") p_in(p_index(1:degl))
          !Write(outfile,"('ec     =',100f12.5)") p_ec(p_index(1:degl))
          !Write(outfile,"('spchar=',100f12.5)") p_spchar(1:degl)
          !Write(outfile,"('nup   =',100f12.5)") (1.0_t_real+p_spchar(1:degl))*0.5_t_real
          !Write(outfile,"('ndn   =',100f12.5)") (1.0_t_real-p_spchar(1:degl))*0.5_t_real
          !Write(outfile,"('p_occu=',100f12.5)") p_occu(1:degl)
          !!
          !!
          !!
          Do m=1,degl
             !!
             imax=p_index(m)
             ee=p_ec(imax)
             !! coeficients of occupied solutions
             kohnsham% p_rczz(1:ndim,i0+m,isa)=p_hc(1:ndim,imax)
             !! energies of the diagonalized core
             kohnsham% p_hc(iz+i0+m,1)=ee
             kohnsham% p_coccu(i0+m,isa,1)=p_occu(m)
             !!
             ecore=ecore+ee*p_occu(m)*con
             !! highest occupied energy for tests later
             emax=Max(emax,ee)
             !!
             If(prot)  Write(outfile,116) imax,p_ec(imax),p_spchar(m),p_occu(m)
116          Format('sol_max=',i5,' e=',g12.5,' spinchar=',g12.5,' occu=',g12.5)
             !!
          Enddo !! m
          i0=i0+degl
       Enddo !! ica
       !!
       Do ic=1,ndim
          If( (p_in(ic)==0) .And. (p_ec(ic) < emax))Then
             Write(outfile,"('core state ',i5,' energy ',g12.5,' HOMO ',g12.5)")&
                  &ic,p_ec(ic),emax
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Aufbau principle vialoted by some core states ?!'")
          Endif
       Enddo
       !!
       Do ica=1,ndim
          Do icb=1,ndim
             !! ib is right index but Dot_product makes left
             !! factor conjg -> thus other order in next line
             kohnsham% p_rcfc(ica,icb,isa)=&
                  & Dot_product(kohnsham% p_rczz(icb,1:nchidim,isa)&
                  &            ,kohnsham% p_rczz(ica,1:nchidim,isa))
          Enddo
       Enddo
       !!
       If(prot)Then
          !!
          Call printmat('Re rczz',"(f12.5,' ')","('     0       ')"&
               &,Real(kohnsham% p_rczz(1:ndim,1:nchidim,isa),t_real),ndim,nchidim,tol)
          Call printmat('Im rczz',"(f12.5,' ')","('     0       ')"&
               &,Aimag(kohnsham% p_rczz(1:ndim,1:nchidim,isa)),ndim,nchidim,tol)
          !!
          Do ia=1,nchidim
             Write(outfile,11) ia,kohnsham% p_hc(iz+ia,1)
11           Format('Ec(',i2,')=',f30.15)
          Enddo
       Endif



       !!
       !!
       Call pfree(p_hc)
       Call pfree(p_sc)
       Call pfree(p_ec)
       Call pfree(p_w)
       Call pfree(p_spchar)
       Call pfree(p_occu)
       Call pfree(p_in)
       Call pfree(p_index)
       Call pfree(p_inc)
       Call pfree(p_sc1)
       Call pfree(p_hc1)
       !!
    End Do isaloop
    !!
    !!
    !Stop
    !!
    !!
    Return
    !!
  End Subroutine rel_mol_core
  !!***********************************************************************
  Subroutine mol_core_trans(mspin,structure,lattice,loi,kohnsham,rel&
       &,need_sigma&
!!$       &,quant&
       &)
    !!
!!$    Use tdiracdefs!! todo: remove
    Implicit None
    !!IO
!!$    Type(T_Quant),Intent(in)     :: quant!!todo:remove
    Type(T_LOI),Intent(inout)    :: loi
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Integer(t_int),Intent(in)    :: mspin 
    Logical(t_log),Intent(in)    :: rel,need_sigma
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mol_core_trans"
    Character(*),Parameter:: core_trans_cv_cpumesg='core transformation of S/Hcv'
    Integer(t_int) :: ms
    Logical(t_log) :: relmag
!!$    Integer(t_int) :: i,j,isa,lechi,lec,lev!todo: remove
!!$    Complex(t_real):: c
    !!
    Call watch(core_trans_cv_cpumesg,cpu_start,reset=.True.)
    !!
    relmag=.False.
    relmag=(rel .And. mspin==2)
    !!
    If(rel)Then
       Call rel_mol_core_trans_cv(structure,lattice,kohnsham&
            &,loi,loi% p_roci_vchi,loi% p_rtci_vchi&
            &,loi% p_roci_vHchi,loi% p_rtci_vHchi&
            &,relmag,loi% p_vbSchi,loi% p_rtci_vbSchi&
            &,need_sigma)
       Call rel_mol_rawcore_trans_Scv(structure,lattice,kohnsham&
            &,loi,loi% p_roci_vcraw,loi% p_rtci_vcraw)
       If(relmag)Then
          Call setup_core_spin_char(structure% nsite,loi% p_cspchar&
               &,kohnsham% ncsmx,kohnsham% nchismx&
               &,kohnsham% p_chiblocklength,kohnsham% p_cblocklength&
               &,kohnsham% p_rczz,loi% p_cbSc0)
          
       Endif
    Else
       Do ms=1,mspin
          Call mol_core_trans_cv(structure,lattice,kohnsham&
               &,loi,ms,loi% p_oci_vchi(:,ms),loi% p_tci_vchi(:,ms)&
               &,loi% p_oci_vHchi(:,ms),loi% p_tci_vHchi(:,ms))
          Call mol_rawcore_trans_Scv(structure,lattice,kohnsham&
               &,loi,ms,loi% p_oci_vcraw(:,ms),loi% p_tci_vcraw(:,ms))
       Enddo
    Endif
    !!
    Call watch(core_trans_cv_cpumesg,cpu_read,delete=.True.,Print=.True.&
         &,mesg='Total ')
    !!
    !!
    !!
!!$    If(need_sigma)Then!!todo: remove
!!$       Do isa=1,Size(loi% p_vbVSv0,3)
!!$          lev=kohnsham% p_vblocklength(isa)
!!$          lec=kohnsham% p_cblocklength(isa)
!!$          lechi=kohnsham% p_chiblocklength(isa)
!!$          Print *,'checking',lec,lechi
!!$          Do i=1,lec
!!$             Do j=1,lec
!!$                c=loi% p_cbSc0(i,j,isa)-Sum(loi% p_cbVSc0(i,j,isa,1:3)*quant% axis(1:3))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err cbVSc0',i,j,isa,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          Do i=1,lev
!!$             Do j=1,lev
!!$                c=loi% p_vbSv0(i,j,isa)-Sum(loi% p_vbVSv0(i,j,isa,1:3)*quant% axis(1:3))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err vbBSv0',i,j,isa,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          Do i=1,lev
!!$             Do j=1,lec
!!$                c=loi% p_vbSc0(i,j,isa)-Sum(loi% p_vbVSc0(i,j,isa,1:3)*quant% axis(1:3))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err vbVSc0',i,j,isa,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$
!!$          Do i=1,lev
!!$             Do j=1,lechi
!!$                c=loi% p_vbSchi(i,j,isa)-Sum(loi% p_vbVSchi(i,j,isa,1:3)*quant% axis(1:3))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err vbVSchi',i,j,isa,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          
!!$          Do i=1,lechi
!!$             Do j=1,lechi
!!$                c=loi% p_chibSchi(i,j,isa)-Sum(loi% p_chibVSchi(i,j,isa,1:3)&
!!$                     &*quant% axis(1:3))&
!!$                     &/Sqrt(Sum(quant% axis(1:3)**2))
!!$
!!$                If(Abs(c)>1.0e-10_t_real)Then
!!$                   Print *,'err chibVSchi',i,j,isa,c
!!$                   Stop
!!$                Endif
!!$             Enddo
!!$          Enddo
!!$          
!!$
!!$       Enddo
!!$       Do i=1,loi% tcivvdim
!!$          c=loi% p_rtci_vbSv0(i)-Sum(loi% p_rtci_vbVSv0(i,1:3)*quant% axis(1:3))&
!!$               &/Sqrt(Sum(quant% axis(1:3)**2))
!!$          If(Abs(c)>1.0e-10_t_real)Then
!!$             Print *,'err rtci_vbVSv0',i,j,isa,c
!!$             Stop
!!$          Endif
!!$       Enddo
!!$       Do i=1,loi% tcivcdim
!!$          c=loi% p_rtci_vbSc0(i)-Sum(loi% p_rtci_vbVSc0(i,1:3)*quant% axis(1:3))&
!!$               &/Sqrt(Sum(quant% axis(1:3)**2))
!!$          If(Abs(c)>1.0e-10_t_real)Then
!!$             Print *,'err rtci_vbVSc0',i,j,isa,c
!!$             Stop
!!$          Endif
!!$       Enddo
!!$       !!
!!$       Do i=1,loi% tcivcdim
!!$          c=loi% p_rtci_vbSchi(i)-Sum(loi% p_rtci_vbVSchi(i,1:3)*quant% axis(1:3))&
!!$               &/Sqrt(Sum(quant% axis(1:3)**2))
!!$          If(Abs(c)>1.0e-10_t_real)Then
!!$             Print *,'err rtci_vbVSchi',i,j,isa,c
!!$             Stop
!!$          Endif
!!$       Enddo
!!$    Endif
!!$    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_core_trans
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine rel_mol_rawcore_trans_Scv(structure,lattice,kohnsham,loi&
       &,roci_vcraw,rtci_vcraw)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(in)       :: loi
    !!  the contracted core-valence overlap matrix elements
    Complex(t_real),Intent(out)  :: roci_vcraw(:),rtci_vcraw(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mol_rawcore_trans_Scv"
    Integer(t_int) :: isa,isb,lev,lecraw,iocivc,ir,itcivc
    !!
    !!
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call rel_mul_cc_scv_r_raw(lev,lecraw,kohnsham% ncsmx&
            &,kohnsham% p_rcfc(:,:,isa)&
            &,loi% p_oci_vc0(iocivc+1:iocivc+lev*lecraw)&
            &,roci_vcraw(iocivc+1:iocivc+lev*lecraw))
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             Call rel_mul_cc_scv_raw(lev,lecraw,kohnsham% ncsmx&
                  &,kohnsham% p_rcfc(:,:,isb)&
                  &,loi% p_rtci_vc0(itcivc+1:itcivc+lev*lecraw)&
                  &,rtci_vcraw(itcivc+1:itcivc+lev*lecraw))
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine rel_mol_rawcore_trans_Scv
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine rel_mol_core_trans_cv(structure,lattice,kohnsham,loi&
       &,roci_vc,rtci_vc,roci_vHc,rtci_vHc,relmag,vbSchi,rtci_vbSchi&
       &,need_sigma)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(inout)    :: loi
    Logical(t_log),Intent(in)    :: relmag,need_sigma
    !!  the contracted core-valence overlap matrix elements
    Complex(t_real),Intent(out)  :: roci_vc(:),rtci_vc(:),rtci_vbSchi(:)
    Complex(t_real),Intent(out)  :: roci_vHc(:),rtci_vHc(:)
    Complex(t_real),Intent(out)  :: vbSchi(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rel_mol_core_trans_cv"
    Integer(t_int) :: isa,isb,lev,lechi,lecraw,iocivc,ir,itcivc,ks
    Complex(t_real),Pointer:: A(:,:)
    !!
    Nullify(A)
    !!
    Call palloc('A',A,1_t_int,kohnsham% ncsmx,1_t_int,kohnsham% nchismx&
         &,nmodule,nsubfunc)
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lechi=kohnsham% p_chiblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call rel_mul_cc_scv_r(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_rczz(:,:,isa)&
            &,loi% p_oci_vc0(iocivc+1:iocivc+lev*lecraw)&
            &,roci_vc(iocivc+1:iocivc+lev*lechi))
       Call rel_mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_rczz(:,:,isa)&
            &,loi% p_roci_vHc(iocivc+1:iocivc+lev*lecraw)&
            &,roci_vHc(iocivc+1:iocivc+lev*lechi))
       If(relmag)Then
          vbSchi(1:lev,1:lechi,isa)=Matmul(loi% p_vbSc0(1:lev,1:lecraw,isa)&
               &,kohnsham% p_rczz(1:lecraw,1:lechi,isa))

          !! this is the new better treatment
#if defined (NEW_FULL_SIGMA_Z)
          !!todo: use better sigma_cv-ortho
          Call rel_mul_cc_cc_cc(lecraw,lechi,A,loi% p_cbSc0(:,:,isa)&
               &,kohnsham% p_rczz(:,:,isa),loi% p_chibSchi(:,:,isa))
#endif

          
       Endif
       If(need_sigma)Then

          Do ks=1,3
             Call rel_mul_cc_cc_cc(lecraw,lechi,A,loi% p_cbVSc0(:,:,isa,ks)&
                  &,kohnsham% p_rczz(:,:,isa),loi% p_chibVSchi(:,:,isa,ks))

             loi% p_vbVSchi(1:lev,1:lechi,isa,ks)=&
                  &Matmul(loi% p_vbVSc0(1:lev,1:lecraw,isa,ks)&
                  &,kohnsham% p_rczz(1:lecraw,1:lechi,isa))
          Enddo
       Endif
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lechi=kohnsham% p_chiblocklength(isb)
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             Call rel_mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
                  &,kohnsham% p_rczz(:,:,isb)&
                  &,loi% p_rtci_vc0(itcivc+1:itcivc+lev*lecraw)&
                  &,rtci_vc(itcivc+1:itcivc+lev*lechi))
             Call rel_mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
                  &,kohnsham% p_rczz(:,:,isb)&
                  &,loi% p_rtci_vHc(itcivc+1:itcivc+lev*lecraw)&
                  &,rtci_vHc(itcivc+1:itcivc+lev*lechi))
             If(relmag)Then
                Call rel_mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx&
                     &,kohnsham% nchismx&
                     &,kohnsham% p_rczz(:,:,isb)&
                     &,loi% p_rtci_vbSc0(itcivc+1:itcivc+lev*lecraw)&
                     &,rtci_vbSchi(itcivc+1:itcivc+lev*lechi))
             Endif
             If(need_sigma)Then
                Do ks=1,3
                   Call rel_mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx&
                        &,kohnsham% nchismx&
                        &,kohnsham% p_rczz(:,:,isb)&
                        &,loi% p_rtci_vbVSc0(itcivc+1:itcivc+lev*lecraw,ks)&
                        &,loi% p_rtci_vbVSchi(itcivc+1:itcivc+lev*lechi,ks))
                Enddo
             Endif
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    Call pfree(A)
    !!
    !!
    Return
    !!
  End Subroutine rel_mol_core_trans_cv
  !!***********************************************************************
  Subroutine rel_mul_cc_scv(lev,lecraw,lechi,ncsmxraw,nchismx,rczz,rloi_vc0,rloi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! contracted core block length
    Integer(t_int),Intent(in)    :: lechi
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! max dimension of a site contracted core block
    Integer(t_int),Intent(in)    :: nchismx
    !! the eigenvectors of the core diagonalization problem
    Complex(t_real),Intent(in)   :: rczz(ncsmxraw,nchismx)
    !! core-valence LOI block to be pre-multiplied by C^T
    Complex(t_real),Intent(in)   :: rloi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C^T
    Complex(t_real),Intent(out)  :: rloi_vc(lev*lechi)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mul_cc_scv"
    Integer(t_int) :: ivc,ivcraw,i,j
    !!
    !!
    ivc=0_t_int
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lechi
          rloi_vc(ivc+j)=&
               &Sum(rczz(1:lecraw,j)*rloi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivc=ivc+lechi
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine rel_mul_cc_scv
  !!***********************************************************************
  Subroutine rel_mul_cc_scv_raw(lev,lecraw,ncsmxraw,rcfc,rloi_vc0,rloi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! the eigenvectors of the core diagonalization problem
    Complex(t_real),Intent(in)   :: rcfc(ncsmxraw,ncsmxraw)
    !! core-valence LOI block to be pre-multiplied by C*f*C^T
    Complex(t_real),Intent(in)   :: rloi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C*f*C^T
    Complex(t_real),Intent(out)  :: rloi_vc(lev*lecraw)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mul_cc_scv_raw"
    Integer(t_int) :: ivcraw,i,j
    !!
    !!
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lecraw
          rloi_vc(ivcraw+j)=&
               &Sum(rcfc(1:lecraw,j)*rloi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine rel_mul_cc_scv_raw
  !!***********************************************************************
  Subroutine rel_mul_cc_scv_r(lev,lecraw,lechi,ncsmxraw,nchismx,rczz,loi_vc0,rloi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! contracted core block length
    Integer(t_int),Intent(in)    :: lechi
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! max dimension of a site contracted core block
    Integer(t_int),Intent(in)    :: nchismx
    !! the eigenvectors of the core diagonalization problem
    Complex(t_real),Intent(in)   :: rczz(ncsmxraw,nchismx)
    !! core-valence LOI block to be pre-multiplied by C^T
    Real(t_real),Intent(in)      :: loi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C^T
    Complex(t_real),Intent(out)  :: rloi_vc(lev*lechi)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mul_cc_scv_r"
    Integer(t_int) :: ivc,ivcraw,i,j
    !!
    !!
    ivc=0_t_int
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lechi
          rloi_vc(ivc+j)=&
               &Sum(rczz(1:lecraw,j)*loi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivc=ivc+lechi
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine rel_mul_cc_scv_r
  !!***********************************************************************
  Subroutine rel_mul_cc_scv_r_raw(lev,lecraw,ncsmxraw,rcfc,loi_vc0,rloi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! the eigenvectors of the core diagonalization problem
    Complex(t_real),Intent(in)   :: rcfc(ncsmxraw,ncsmxraw)
    !! core-valence LOI block to be pre-multiplied by C*f*C^T
    Real(t_real),Intent(in)      :: loi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C*f*C^T
    Complex(t_real),Intent(out)  :: rloi_vc(lev*lecraw)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mul_cc_scv_r_raw"
    Integer(t_int) :: ivcraw,i,j
    !!
    !!
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lecraw
          rloi_vc(ivcraw+j)=&
               &Sum(rcfc(1:lecraw,j)*loi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine rel_mul_cc_scv_r_raw
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine mol_rawcore_trans_Scv(structure,lattice,kohnsham,loi,ms&
       &,oci_vcraw,tci_vcraw)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(in)       :: loi
    Integer(t_int),Intent(in)    :: ms
    !!  the CfC*Scv core-valence overlap matrix elements
    Real(t_real),Intent(out)     :: oci_vcraw(:),tci_vcraw(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mol_rawcore_trans_Scv"
    Integer(t_int) :: isa,isb,lev,lecraw,iocivc,ir,itcivc
    !!
    !!
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call mul_cc_scv_raw(lev,lecraw,kohnsham% ncsmx&
            &,kohnsham% p_ncfc(:,:,isa,ms)&
            &,loi% p_oci_vc0(iocivc+1:iocivc+lev*lecraw)&
            &,oci_vcraw(iocivc+1:iocivc+lev*lecraw))
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             Call mul_cc_scv_raw(lev,lecraw,kohnsham% ncsmx&
                  &,kohnsham% p_ncfc(:,:,isb,ms)&
                  &,loi% p_tci_vc0(itcivc+1:itcivc+lev*lecraw)&
                  &,tci_vcraw(itcivc+1:itcivc+lev*lecraw))
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_rawcore_trans_Scv
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine mol_core_trans_cv(structure,lattice,kohnsham,loi,ms&
       &,oci_vc,tci_vc,oci_vHc,tci_vHc)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(in)       :: loi
    Integer(t_int),Intent(in)    :: ms
    !!  the contracted core-valence overlap matrix elements
    Real(t_real),Intent(out)     :: oci_vc(:),tci_vc(:)
    Real(t_real),Intent(out)     :: oci_vHc(:),tci_vHc(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mol_core_trans_cv"
    Integer(t_int) :: isa,isb,lev,lechi,lecraw,iocivc,ir,itcivc
    !!
    !!
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lechi=kohnsham% p_chiblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,loi% p_oci_vc0(iocivc+1:iocivc+lev*lecraw)&
            &,oci_vc(iocivc+1:iocivc+lev*lechi))
       Call mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,loi% p_oci_vHc(iocivc+1:iocivc+lev*lecraw,ms)&
            &,oci_vHc(iocivc+1:iocivc+lev*lechi))
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lechi=kohnsham% p_chiblocklength(isb)
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             Call mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,loi% p_tci_vc0(itcivc+1:itcivc+lev*lecraw)&
                  &,tci_vc(itcivc+1:itcivc+lev*lechi))
             Call mul_cc_scv(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,loi% p_tci_vHc(itcivc+1:itcivc+lev*lecraw,ms)&
                  &,tci_vHc(itcivc+1:itcivc+lev*lechi))
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_core_trans_cv
  !!***********************************************************************
  Subroutine mul_cc_scv(lev,lecraw,lechi,ncsmxraw,nchismx,nczz,loi_vc0,loi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! contracted core block length
    Integer(t_int),Intent(in)    :: lechi
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! max dimension of a site contracted core block
    Integer(t_int),Intent(in)    :: nchismx
    !! the eigenvectors of the core diagonalization problem
    Real(t_real),Intent(in)      :: nczz(ncsmxraw,nchismx)
    !! core-valence LOI block to be pre-multiplied by C^T
    Real(t_real),Intent(in)      :: loi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C^T
    Real(t_real),Intent(out)     :: loi_vc(lev*lechi)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mul_cc_scv"
    Integer(t_int) :: ivc,ivcraw,i,j
    !!
    !!
    ivc=0_t_int
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lechi
          loi_vc(ivc+j)=&
               &Dot_product(nczz(1:lecraw,j),loi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivc=ivc+lechi
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine mul_cc_scv
  !!***********************************************************************
  Subroutine mul_cc_scv_raw(lev,lecraw,ncsmxraw,ncfc,loi_vc0,loi_vc)
    !!
    Implicit None  
    !!IO
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! the eigenvectors of the core diagonalization problem
    Real(t_real),Intent(in)      :: ncfc(ncsmxraw,ncsmxraw)
    !! core-valence LOI block to be pre-multiplied by C*f*C^T
    Real(t_real),Intent(in)      :: loi_vc0(lev*lecraw)
    !! core-valence LOI after pre-multiplication by C*f*C^T
    Real(t_real),Intent(out)     :: loi_vc(lev*lecraw)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mul_cc_scv_raw"
    Integer(t_int) :: ivcraw,i,j
    !!
    !!
    ivcraw=0_t_int
    Do i=1,lev
       Do j=1,lecraw
          loi_vc(ivcraw+j)=&
               &Dot_product(ncfc(1:lecraw,j),loi_vc0(ivcraw+1:ivcraw+lecraw))
       Enddo ! j
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine mul_cc_scv_raw
  !!***********************************************************************
  Subroutine mul_cc_sccg(lecraw,lechi,nczz,i0,loi_cc0,loi_cc)
    !!
    !!  loi_cc(:,i,j)=nczz(l,i)*loi_cc0(:,l,n)*nczz(n,j)
    !!
    !!
    Implicit None  
    !!IO
    !! base index
    Integer(t_int),Intent(in)    :: i0
    !! contracted core block length
    Integer(t_int),Intent(in)    :: lechi
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! the eigenvectors of the core diagonalization problem
    Real(t_real),Intent(in)      :: nczz(:,:)
    !! core-core LOI block to be pre-multiplied by C^T
    Real(t_real),Intent(in)      :: loi_cc0(:,:)
    !! core-core LOI after pre-multiplication by C^T
    Real(t_real),Intent(out)     :: loi_cc(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mul_cc_sccg"
    Integer(t_int) :: icc,iccraw,i,j,k
    Real(t_real)   :: tmp(3,lecraw,lechi)
    !!
    !!
    !!
    iccraw=i0
    Do i=1,lecraw
       Do j=1,lechi
          Do k=1,3
             tmp(k,i,j)&
                  &=Dot_product(loi_cc0(k,iccraw+1:iccraw+lecraw),&
                  &nczz(1:lecraw,j))
          Enddo
       Enddo
       iccraw=iccraw+lecraw
    Enddo
    !!
    icc=i0
    Do i=1,lechi
       Do j=1,lechi
          Do k=1,3
             loi_cc(k,icc+j)=Dot_product(nczz(1:lecraw,i)&
                  &,tmp(k,1:lecraw,j))
          Enddo
       Enddo
       icc=icc+lechi
    Enddo
    !!
    Return
    !!
  End Subroutine mul_cc_sccg
  !!***********************************************************************
  Subroutine mul_cc_scvg(lev,lecraw,lechi,ncsmxraw,nchismx,nczz,i0,loi_vc0,loi_vc)
    !!
    Implicit None  
    !!IO
    !! base index
    Integer(t_int),Intent(in)    :: i0
    !! valence block length
    Integer(t_int),Intent(in)    :: lev
    !! contracted core block length
    Integer(t_int),Intent(in)    :: lechi
    !! raw core block length
    Integer(t_int),Intent(in)    :: lecraw
    !! max dimension of a site contracted core block
    Integer(t_int),Intent(in)    :: ncsmxraw
    !! max dimension of a site raw core block
    Integer(t_int),Intent(in)    :: nchismx
    !! the eigenvectors of the core diagonalization problem
    Real(t_real),Intent(in)      :: nczz(ncsmxraw,nchismx)
    !! core-valence LOI block to be pre-multiplied by C^T
    Real(t_real),Intent(in)      :: loi_vc0(:,:)
    !! core-valence LOI after pre-multiplication by C^T
    Real(t_real),Intent(out)     :: loi_vc(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mul_cc_scvg"
    Integer(t_int) :: ivc,ivcraw,i,j,k
    !!
    !!
    ivc=i0
    ivcraw=i0
    Do i=1,lev
       Do j=1,lechi
          Do k=1,3
             loi_vc(k,ivc+j)=&
                  &Dot_product(nczz(1:lecraw,j)&
                  &,loi_vc0(k,ivcraw+1:ivcraw+lecraw))
          Enddo
       Enddo ! j
       ivc=ivc+lechi
       ivcraw=ivcraw+lecraw
    Enddo ! i
    !!
    !!
    Return
    !!
  End Subroutine mul_cc_scvg
  !!***********************************************************************
  Subroutine corereltest(structure,basis,H,isa,ndim)
    !!
    Implicit none  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: isa,ndim
    Complex(t_real),Intent(in)   :: H(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="corereltest"
    Integer(t_int) :: issa,ldim,jdim,ic,l,degl,degj,icr,kap
    Integer(t_int) :: i,j0,m,mmu,mu,jj,iz,n,ms,j,kap0
    Complex(t_real),Pointer :: TT(:,:,:)
    Complex(t_real),Pointer :: p_U(:,:,:)
    Complex(t_real),Pointer :: p_A(:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    Nullify(TT)
    Nullify(p_U)
    Nullify(p_A)
    !!
    Call palloc('p_U',p_U,1_t_int,2*basis% lmx+1,1_t_int,2*basis% lmx+1&
         &,0_t_int,basis% lmx,nmodule,nsubfunc)
    !!
    Call make_ylm_U_matrix(basis% lmx,p_U)
    !!
    Print *,'p_U'
    Do l=0,basis% lmx
       Write(outfile,*)'U l=',l,':'
       Do i=1,2*l+1
          Do j=1,2*l+1
             Write(outfile,"('(',f12.5,',',f12.5,') ')",advance='no')p_U(i,j,l)
          Enddo
          Write(outfile,*)
       Enddo
    Enddo

    issa=structure% p_isrt(isa)
    !!
    ldim=0
    jdim=0
    Do ic=1,basis% p_nRc(issa)
       kap0=basis% p_kapRc(ic,issa)
       If(kap0>0) Cycle
       l=basis% p_lRc(ic,issa)
       degl=2*l+1
       ldim=ldim+degl
    Enddo
    Do icr=1,basis% p_nRc(issa)
       kap=basis% p_kapRc(icr,issa)
       l=(Abs(2*kap+1)-1)/2
       degj=2*Abs(kap)
       jdim=jdim+degj
       Print 1,icr,basis% p_nnRc(icr,issa),lqnchar(l),degj-1,degj,jdim
1      Format(i4,':',i1,a1,i1,'/2  deg=',i3,' dim=',i3)
    Enddo
    !!
    If(jdim /= ndim ) Then
       Print *,'   ndim=',ndim
       Print *,'/= jdim=',jdim
       Stop 
    Endif
    Call palloc('TT',TT ,1_t_int,ldim ,1_t_int,jdim, 1_t_int,2_t_int & 
         &,nmodule,nsubfunc)
    Call palloc('p_A',p_A ,1_t_int,ldim ,1_t_int,ldim,nmodule,nsubfunc)
    TT=c0
    !!
    i=0
    icr=0
    icloop4: Do ic=1,basis% p_nRc(issa)
       kap0=basis% p_kapRc(ic,issa)
       If(kap0>0)Cycle icloop4
       l=basis% p_lRc(ic,issa)
       degl=2*l+1
       j0=1 ; If(l==0) j0=2
       mloop4: Do m=1,degl
          i=i+1
          !!
          iz=icr
          jjloop4: Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             mmuloop4: Do mmu=1,j+1
                mu=2*(mmu-1)-j
                iz=iz+1
                msloop4: Do ms=1,2
                   !! U_mn d_n,mu-s/2
                   n=(mu-1)/2 + ms + l
                   If(1 <=n .And. n <= 2*l+1  ) Then
                      TT(i,iz,ms)=TT(i,iz,ms)+p_U(m,n,l)*cgoOFKapMu(ms,kap,mu)
                   Endif
                Enddo msloop4
             Enddo mmuloop4
          Enddo jjloop4
          !!
       Enddo mloop4
       icr=icr+2*degl
    Enddo icloop4
    !!

    Call printmat('TT1',"('(',f12.5,',',f12.5,') ')","('              0     &
         &        ')"&
         &,TT(:,:,1),ldim,jdim,1.0e-5_t_real)
    !!
    Call printmat('TT2',"('(',f12.5,',',f12.5,') ')","('              0     &
         &        ')"&
         &,TT(:,:,2),ldim,jdim,1.0e-5_t_real)


    !!
    Do ms=1,2
    !!
       Do i=1,2
          Call printmat('TT*TT',"('(',f12.5,',',f12.5,') ')","('              0     &
         &        ')"&
         &,Matmul(TT(:,:,ms),Conjg(Transpose(TT(:,:,i)))),ldim,ldim&
         &,1.0e-7_t_real)
       Enddo
    Enddo

    Do ms=1,2
       !Do i=1,2
       Do i=ms,ms

          p_A=Matmul(TT(:,:,ms),Matmul(H,Transpose(Conjg(TT(:,:,i)))))
          
          Call printmat('TT*H*TT^+',"(f12.5,' ')" ,"('     0       ')"&
               &,Real(p_A),ldim,ldim ,1.0e-7_t_real)


       Enddo
    Enddo

    !!
    Call pfree(TT)
    Call pfree(p_U)
    Call pfree(p_A)
    !!
    !Stop
    Return
    !!
  End Subroutine corereltest
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine mol_core_trans_gcv(structure,lattice,kohnsham,loi,ms&
       &,oci_gvc,tci_gvc&
       &,oci_gvHc,tci_gvHc,oci_gvHcr,tci_gvHcr)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(in)       :: loi
    Integer(t_int),Intent(in)    :: ms
    !!  the contracted core-valence overlap matrix elements
    Real(t_real),Intent(out)     :: oci_gvc(:,:),tci_gvc(:,:)
    Real(t_real),Intent(out)     :: oci_gvHc(:,:),tci_gvHc(:,:)
    Real(t_real),Intent(out)     :: oci_gvHcr(:,:),tci_gvHcr(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mol_core_trans_gcv"
    Integer(t_int) :: isa,isb,lev,lechi,lecraw,iocivc,ir,itcivc
    !!
    !!
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lechi=kohnsham% p_chiblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,iocivc,loi% p_oci_gvc0(:,:)&
            &,oci_gvc(:,:))
       Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,iocivc,loi% p_oci_gvHc(:,:,ms)&
            &,oci_gvHc(:,:))
       Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,iocivc,loi% p_oci_gvHcr(:,:,ms)&
            &,oci_gvHcr(:,:))
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lechi=kohnsham% p_chiblocklength(isb)
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             
             Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx&
                  &,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,itcivc,loi% p_tci_gvc0(:,:)&
                  &,tci_gvc(:,:))
             Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx&
                  &,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,itcivc,loi% p_tci_gvHc(:,:,ms)&
                  &,tci_gvHc(:,:))
             Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx&
                  &,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,itcivc,loi% p_tci_gvHcr(:,:,ms)&
                  &,tci_gvHcr(:,:))
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_core_trans_gcv
  !!***********************************************************************
  !! manipulate core-valence overlap with the core-diagonalization results
  Subroutine mol_core_trans_gcv_noH(structure,lattice,kohnsham,loi,ms&
       &,oci_gcc,oci_gvc,tci_gvc)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Loi),Intent(in)       :: loi
    Integer(t_int),Intent(in)    :: ms
    !!  the contracted core-valence overlap matrix elements
    Real(t_real),Intent(out)     :: oci_gcc(:,:),oci_gvc(:,:),tci_gvc(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mol_core_trans_gcv_noH"
    Integer(t_int) :: isa,isb,lev,lechi,lecraw,iocivc,iocicc,ir,itcivc
    !!
    !!
    !!
    !!
    isaloop: Do isa=1_t_int,structure% nsite
       !!
       !! onsite core valence overlap
       !!
       lev=kohnsham% p_vblocklength(isa)
       lechi=kohnsham% p_chiblocklength(isa)
       lecraw=kohnsham% p_cblocklength(isa)
       !!
       iocicc=loi% p_ociccindex(isa)
       !!
       Call mul_cc_sccg(lecraw,lechi,kohnsham% p_nczz(:,:,isa,ms)&
            &,iocicc,loi% p_oci_gcc0(:,:),oci_gcc(:,:))
       !!
       iocivc=loi% p_ocivcindex(isa)
       !!
       Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx,kohnsham% nchismx&
            &,kohnsham% p_nczz(:,:,isa,ms)&
            &,iocivc,loi% p_oci_gvc0(:,:)&
            &,oci_gvc(:,:))
       !!
       !! twosite core valence overlap
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          lechi=kohnsham% p_chiblocklength(isb)
          lecraw=kohnsham% p_cblocklength(isb)
          !!
          irloop: Do ir=1_t_int,lattice% p_npairs(isa,isb)
             !!
             itcivc=loi% p_tcivcindex(ir,isa,isb)
             
             Call mul_cc_scvg(lev,lecraw,lechi,kohnsham% ncsmx&
                  &,kohnsham% nchismx&
                  &,kohnsham% p_nczz(:,:,isb,ms)&
                  &,itcivc,loi% p_tci_gvc0(:,:)&
                  &,tci_gvc(:,:))
             !!
          Enddo irloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine mol_core_trans_gcv_noH
  !!***********************************************************************
  Subroutine setup_core_spin_char(nsite,cspchar,ncsmx,nchismx&
       &,chiblocklength,cblocklength,rczz,cbSc0)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nsite,ncsmx,nchismx
    Integer(t_int),Intent(in)    :: cblocklength(nsite),chiblocklength(nsite)
    Real(t_real),Intent(out)     :: cspchar(*)
    Complex(t_real),Intent(in)   :: cbSc0(ncsmx,ncsmx,nsite)
    Complex(t_real),Intent(in)   :: rczz(ncsmx,nchismx,nsite)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setup_core_spin_char"
    Integer(t_int) :: iz,isa,lechi,lecraw,j
    !!
    iz=0_t_int
    Do isa=1_t_int,nsite
       lechi=chiblocklength(isa)
       lecraw=cblocklength(isa)
       Do j=1,lechi
          iz=iz+1_t_int
          cspchar(iz)=Real(Dot_product(rczz(1:lecraw,j,isa)&
               &,Matmul(cbSc0(1:lecraw,1:lecraw,isa),rczz(1:lecraw,j,isa)))&
               &,t_real)
          !!TODO: check the reality
       Enddo ! j
    Enddo ! isa
    !!
    Return
    !!
  End Subroutine setup_core_spin_char
  !!***********************************************************************
  Subroutine rel_mul_cc_cc_cc(lecraw,lechi,Aux,cbSc0,rczz,chibSchi)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: lecraw,lechi
    Complex(t_real),Intent(in)   :: cbSc0(:,:),rczz(:,:)
    Complex(t_real),Intent(inout):: Aux(:,:)
    Complex(t_real),Intent(out)  :: chibSchi(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="rel_mul_cc_cc_cc"
    Integer(t_int) :: i,j
    !!
    !!
    Aux(1:lecraw,1:lechi)=&
         &Matmul(cbSc0(1:lecraw,1:lecraw),rczz(1:lecraw,1:lechi))
    !!
    Do i=1,lechi
       Do j=1,lechi
          chibSchi(i,j)=Dot_product(rczz(1:lecraw,i),Aux(1:lecraw,j))
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine rel_mul_cc_cc_cc
  !!***********************************************************************

  
End Module molcore
