!!----DEBUG options--------------------------------|
!! MOLPOT:det_pot_l_components:take_all_components |
!!-------------------------------------------------|

!!TODO: see if lcd in potderivs needs to be changed
!!TODO: change lcd
Module molpot
  !!
  Use precisiondefs
  Use constants
  Use memory
  Use trelativistic
  Use file_iounits
  Use errors
  Use clock
  Use tsymmetry
  Use bspline
  Use symmetry
  Use tmesh
  Use tnumerics
  Use tgrid
  Use torbitalbasis
  Use atomdefs
  Use ioaux
  Use dump
  Use efg
  Use tforces
  Use terg
  Use tpotentialdensity
  Use integrate1d
  Use matserv
  Use tmol
  Use mesh
  Use optionstest
  Use xc
  Use prnt
  Use plotdens
  Use vlm
  Use pot
  Use vecharm
  Use erg
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="MOLPOT"  
  !!
  Public :: mol_makepot,det_pot_l_components,alloc_tpotential&
       &,setup_potential_restrictions
  !!***********************************************************************
  Character(*),Parameter :: fmtstart=&
       &"(/,/,'============================== START ',a&
       &,'============================== ')"
  Character(*),Parameter :: fmtend=&
       &"('============================== END   ',a&
         &,'============================== ',/,/)"
  !!***********************************************************************
Contains
  !!***********************************************************************
  !! allocate potential and density
  Subroutine alloc_tpotential(mspin,structure,potential)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)        :: mspin
    Type(T_Structure),Intent(inout)  :: structure
    Type(T_Potential),Intent(inout)  :: potential
    !!LOCAL
    Character(*),Parameter::nsubfunc="alloc_tpotential"
    !!
    !!
    Call palloc('potential% p_lcd',potential% p_lcd &
         &,1_t_int,potential% nrp&
         &,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite&
         &,1_t_int,mspin&
         &,nmodule,nsubfunc)
    !!
    !!
    Call palloc('potential% p_charges',potential% p_charges &
         &,1_t_int,structure% nsite&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    Return
    !!
  End Subroutine alloc_tpotential
  !!***********************************************************************
  !info
  !info   determines nonzero L-components of the local potential
  !info   may be wrong, check it for each new structure, till time
  !info   will prove it
  !info
  Subroutine det_pot_l_components(symmetry,structure,potential&
       &,maxl,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Potential),Intent(inout) :: potential
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: maxl(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="det_pot_l_components"
    Logical(t_log) ::  deb,equiv
    Integer(t_int) :: isa,jops,ma,mb,ii,kinv,la,ig,glmax,issa
    Real(t_real)  :: tol,eu(3),s2(3),b(3,3),r(3)
    Real(t_real) :: fac(0_t_int:Max(3_t_int,2_t_int*potential % max_L+1_t_int))
    Real(t_real),Pointer :: wu(:) 
    Real(t_real),Pointer :: aa(:,:,:) 
    Integer(t_int),Pointer :: llmp(:,:)
    Real(t_real),Parameter :: etol=1.0e-5_t_real !! tolerance for symmetry equivalence
    !!
    deb=debugfun(nmodule,nsubfunc,'take_all_components')
    !!
    Call make_factorials(fac,Max(3_t_int,2_t_int*potential % max_L+1_t_int))

    !!
    Nullify(llmp)
    Nullify(aa)
    Nullify(wu)
    !!
    !!

    Call palloc('potential % p_nlmp',potential % p_nlmp&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    Call palloc('llmp',llmp&
         &,1_t_int,(potential % max_L+1_t_int)**2_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('aa',aa&
         &,1_t_int,2_t_int*potential % max_L+1_t_int&
         &,1_t_int,2_t_int*potential % max_L+1_t_int&
         &,1_t_int,potential % max_L+1_t_int&
         &,nmodule,nsubfunc)
    Call palloc('wu',wu&
         &,1_t_int,(2_t_int*potential % max_L+1_t_int)**2_t_int&
         &,nmodule,nsubfunc)
    !!

    potential % mlmp=0_t_int
    tol=1.e-12_t_real
    !
    siteloop: Do isa=1_t_int,structure% nsite

       issa=structure% p_isrt(isa)

       If(relativistic ==FULL_RELATIVISTIC)Then
          glmax=Max(potential% max_LIO,(maxl(issa)+0)*2)
          !! todo: test wheter +1 in relat makes a difference for the
          !! force correction term adn energies
          !! todo: change the +0 into +1 in fplostartup too!!!
       Else
          glmax=Max(potential% max_LIO,maxl(issa)*2)
       Endif

       aa(:,:,:)=0.0_t_real
       ig=0_t_int

       If(outlevel >= L_many)Then
          Write(outfile,400) isa,structure% p_tau(:,isa)
       Endif
400    Format(' Site ',i5,' tau=',t25,3(f15.7,2x),/&
            &,' has local symmetry operations:')
       oploop: Do jops=1_t_int,symmetry % Rnumops

          s2=Matmul(structure% p_tau(:,isa),symmetry% Ropmat(:,:,jops))&
               &+symmetry% Roptau(:,jops)-structure% p_tau(:,isa)



          If(structure% str_type == STR_TYPE_MOLECULE ) Then
             equiv=All(Abs(s2(1:3))<etol)
          Else
             r(1)=Sum(s2(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
             r(2)=Sum(s2(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
             r(3)=Sum(s2(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
             equiv=vecIsOnGrid(3_t_int,r,etol)
          Endif


          
          ifgo: If(equiv)Then


             If(outlevel >= L_many)Then
                Write(outfile,401) symmetry% Ropsymbol(jops)
             Endif
401          Format(t20,a30)
             !Print *,'OP ',jops,' = ',opsymbol(jops)
             !Print "('OP*tau-tau ',t25,3(f15.7,2x))",s2
             !Print "('OP*tau indices ',t25,3(f15.7,2x))",r
             !Print "(i5, 'Its a valid oeraiton ')",jops

             ig=ig+1_t_int
             If(Any(Abs(symmetry% Ropmat(:,:,jops)&
                  &     -symmetry% opgen(:,:,symmetry% Ropind(jops)))&
                  & >1.e-8_t_real))&
                  &Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Problems with the order of symmetry operations!'")

             kinv=symmetry% opeulerinv(symmetry% Ropind(jops))
             eu=symmetry% opeuler(:,symmetry% Ropind(jops))


             Call ytr(eu,kinv,fac,1_t_int,b,.false.)
             If(potential % max_L > 0_t_int)&
                  &Call paranoia(symmetry% Ropmat(:,:,jops),b&
                  &,jops,1.e-14_t_real)

             Do la=0_t_int,glmax
                Call ytr(eu,kinv,fac,la,wu,.false.)
                Do ma=1_t_int,la+la+1_t_int
                   Do mb=1_t_int,la+la+1_t_int
                      ii=(mb-1_t_int)*(2_t_int*la+1_t_int)+ma
                      aa(ma,mb,la+1_t_int)=aa(ma,mb,la+1_t_int)+wu(ii)
                   Enddo           ! mb
                Enddo             ! ma
             Enddo               ! la
          End If ifgo

       Enddo oploop
       If(ig == 0_t_int) Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'There should be at least the identity operation!'")
       ii=0_t_int
       Do la=0_t_int,glmax
          Do mb=1_t_int,la+la+1_t_int
             ! wu(ma,mb)=0 forall ma
             Do ma=1_t_int,la+la+1_t_int
                If(Abs(aa(ma,mb,la+1_t_int)) > tol) Goto 444
             Enddo               ! ma
             If(.Not.deb) Goto 555
444          Continue
             ii=ii+1_t_int
             llmp(ii,isa)=mb+la*la
555          Continue
          Enddo                 ! mb

       Enddo                   ! la
       potential % mlmp=Max(potential % mlmp,ii)
       potential % p_nlmp(isa)=ii
    Enddo siteloop

    !! make final array p_lmp and copy data
    Call palloc('potential % p_lmp',potential % p_lmp&
         &,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)


    potential % p_lmp(:,:)=0_t_int
    Do isa=1_t_int,structure% nsite
       potential % p_lmp(1_t_int:potential % p_nlmp(isa),isa)&
            &=llmp(1_t_int:potential % p_nlmp(isa),isa)
    Enddo
    !!
    Call pfree(wu)
    Call pfree(aa)
    Call pfree(llmp)
    !!
    !!
    If(outlevel >= L_many) Then

10     Format('Angular momentum components of potential and density',/)
       Open(14,file='+lpot')
       Write(outfile,10)
       Write(14,10)
       Write(outfile,11)potential % mlmp
       Write(14,11)potential % mlmp
       Do isa=1_t_int,structure% nsite
2         Format(i4)
          Write(outfile,2,advance='no')potential % p_nlmp(isa)
          Write(14,2,advance='no')potential % p_nlmp(isa)
          Do ii=1_t_int,potential % p_nlmp(isa)
             Write(outfile,2,advance='no')potential % p_lmp(ii,isa)
             Write(14,2,advance='no')potential % p_lmp(ii,isa)
          Enddo
3         Format()
          Write(outfile,3)
          Write(14,3)
       Enddo
       Write(outfile,15)
       Write(14,15)
       Close(14)
    End If


    !!
    Return
    !!
11  Format(' - mlmp -',/,i5,/ ,' - nlmp -   - lmp -')
15  Format(/,'#1-1 2-4 5-9 10-16 17-25 26-36 37-49 50-64 65-81 '&
         & ,'82-100 101-121 122-144 145-169',/'#0    1   2    3     4     '&
         & ,'5     6     7     8     9       10      11      12',/)
    !!
  End Subroutine det_pot_l_components
  !!***********************************************************************
  Subroutine mol_makepot(symmetry,structure,lattice,mspin&
       &,fgam,potential,etot,forces,numerics,mol,gridplot,BEB,force_loop&
       &,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Mol),Intent(inout)    :: mol
    Type(T_Potential),Intent(inout) :: potential
    Type(T_Etot),Intent(inout)   :: etot
    Type(T_Forces),Intent(inout) :: forces
    Type(T_GridPlot),Intent(inout):: gridplot
    Logical(t_log),Intent(in)    :: BEB,force_loop
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(in)      :: fgam(structure% nsort)
    !!LOCAL
    Character(*),Parameter::nsubfunc="mol_makepot"
    Character(*),Parameter  :: cpumesg_nverg ='calc_nv_erg'
    Logical(t_log) :: refu
    !!
    !!
    refu=isoption('PLOT_REALFUNC')
    !!
    !!
    Call palloc('potential% p_ac',potential% p_ac&
         &,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite,nmodule,nsubfunc)
    Call palloc('potential% p_Nl',potential% p_Nl&
         &,0_t_int,potential% max_L,nmodule,nsubfunc)
    Call palloc('potential% p_vc',potential% p_vc&
         &,1_t_int,potential% nrp,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite,nmodule,nsubfunc)
    Call palloc('potential% p_vft',potential% p_vft&
         &,1_t_int,lattice% nlattg,nmodule,nsubfunc)
    !!
    potential% p_Nl(0:potential% max_L)=0.0_t_real
    potential% p_ac(:,:)=0.0_t_real
    potential% p_vc(:,:,:)=0.0_t_real
    !!
    If(structure% str_type == STR_TYPE_MOLECULE)Then
       Call palloc('potential% p_vew',potential% p_vew&
            &,1_t_int,potential% nrp,1_t_int,potential% mlmp&
            &,1_t_int,structure% nsite,nmodule,nsubfunc)
       potential% p_vew(:,:,:)=0.0_t_real
    Endif
    !!
    emptif: If(.Not.isoption('EMPTY_LATTICE_TEST')) Then
       !!
       !!start calculations
       !!local Coulomb potential + Ewald Fourier potential
       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          Call vcoulomb(structure % nsite,structure% p_tau,structure% vuc&
               &,structure% p_isrt,structure% p_conao&
               &,lattice% nlattg,lattice% p_glat,mspin,numerics% degree&
               &,fgam&
               &,potential% max_L,potential% mlmp,potential% p_nlmp&
               &,potential% p_lmp,potential% p_pmeshs,potential% nrp&
               &,potential% p_natom,potential% p_lcd&
               &,potential% ew_p_r,potential% p_vc&
               &,potential% p_vft,potential% p_ac,potential% p_Nl&
               &,relativistic,potential% nuc_type,potential% p_vew)
          
       Else
          Call vcoulomb(structure % nsite,structure% p_tau,structure% vuc&
               &,structure% p_isrt,structure% p_conao&
               &,lattice% nlattg,lattice% p_glat,mspin,numerics% degree&
               &,fgam&
               &,potential% max_L,potential% mlmp,potential% p_nlmp&
               &,potential% p_lmp,potential% p_pmeshs,potential% nrp&
               &,potential% p_natom,potential% p_lcd&
               &,potential% ew_p_r,potential% p_vc&
               &,potential% p_vft,potential% p_ac,potential% p_Nl&
               &,relativistic,potential% nuc_type)
       Endif
       !!
       !!
       !! write local Hartree-potential
       If(refu) Then
          Call printfunc(potential% p_pmeshs,potential% nrp,potential% p_vc&
               &,1_t_int,structure% nsite,structure% p_isrt&
               &,structure% p_elements,potential% mlmp&
               &,potential% p_nlmp,potential% p_lmp,'+har',PRINTFUNC_POT)
          If(structure% str_type == STR_TYPE_MOLECULE)Then
             Call printfunc(potential% p_pmeshs,potential% nrp,potential% p_vew&
                  &,1_t_int,structure% nsite,structure% p_isrt&
                  &,structure% p_elements,potential% mlmp&
                  &,potential% p_nlmp,potential% p_lmp,'+vew',PRINTFUNC_VEWLOC)
          End If
       End If
       !!
       !!
    End If emptif
    !!
    !!
    !!
    If(dumpread(DUMP_MOLPOT))Then
       Call dumpmolpot(mol,etot,'read')
       gdump(DUMP_MOLPOT)% doread=.False.
    Else
       !!
       Call watch(cpumesg_nverg,cpu_start,reset=.True.)
       !!
       Call calc_nv_erg(mspin,structure% nsite,structure% p_isrt&
            &,lattice% p_npshell,lattice% p_ninpshell&
            &,lattice% p_pairabs,lattice% p_pairs&
            &,potential% p_pmeshs,potential% nrp,fgam,potential% max_L&
            &,potential% mlmp,potential% p_nlmp,potential% p_lmp,potential% p_lcd&
            &,potential% p_vc,mol&
            &,numerics% coulerg_nr,numerics% coulerg_nxi&
            &,numerics% coulerg_neta,numerics% degree,etot&
            &,potential% chargemode,potential% molecule_ionicity&
            &,structure% vuc)
       !!
       Call watch(cpumesg_nverg,cpu_stop,delete=.True.,Print=.True.)
       !!
       Call molpottogrid(structure,mspin,lattice,potential&
            &,mol,numerics% degree,etot)
       
       Call erg_hart_0(mspin,structure% nsite,structure% p_isrt&
            &,structure% p_conao,potential% p_pmeshs,potential% nrp,fgam&
            &,potential% mlmp,potential% p_lcd,potential% p_charges&
            &,potential% ew_p_r,potential% p_natom&
            &,numerics% degree,numerics% coulerg_nr,etot)
       
       If(dumpwrite(DUMP_MOLPOT)) Call dumpmolpot(mol,etot,'write')
    Endif
    !!
    !!
    !! plot density/potential
    !! if user used old style =.griddens
    Call plot_dens(symmetry,structure,lattice,mspin,potential&
         &,numerics% degree,BEB,gridplot)
    !!
    Call plot_denspot(symmetry,structure,lattice,mspin,potential&
         &,numerics% degree,gridplot,BEB)

    !!
    !!
    !!
    If(force_loop)Then
       Call calc_HF_force(structure,mspin,lattice,fgam,potential&
            &,numerics% degree,numerics% coulerg_nr,forces% p_hfforces)
    Endif

    !!

    !Stop
    !!
    !!
    Call calc_efg(mspin,structure,lattice,potential,fgam,numerics% degree&
         &,relativistic)
    !!
    !! calculate crystal field potential for
    !! crystalfield models
    Call calc_vlm(mspin,symmetry,structure,lattice,potential,numerics% degree)

    !!
    Call pfree(potential% p_vew)
    Call pfree(potential% p_vft)
    Call pfree(potential% p_vc)
    Call pfree(potential% p_ac)
    Call pfree(potential% p_Nl)
    !!
    Return
    !!
  End Subroutine mol_makepot
  !!***********************************************************************
  Subroutine calc_HF_force(structure,mspin,lattice,fgam,potential&
       &,degree,coulerg_nr,hfforces)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: degree,coulerg_nr
    Real(t_real),Intent(in)      :: fgam(structure% nsort)
    Real(t_real),Intent(out)     :: hfforces(3,*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_HF_force"
    Logical(t_log) :: dolew
    Integer(t_int) :: isa,issa,isb,issb,irs,j,l,ilm
    Real(t_real)   :: rb,rb2,r(3),gv(3),gg(3),vp,vm,deg
    Real(t_real)   :: tt(3),fon(3),fewon(3)
    Real(t_real)   :: p,gvon(3),gvoff(3),gvtail(3)
    Real(t_real),Pointer :: rvylm(:,:,:)
    Type(T_Mesh),Pointer :: pmesha,pmeshb
    Character(*),Parameter  :: cpumesg_pot=' forces'
    !!
    !!
    Nullify(rvylm)
    !!
    dolew=.False.
    !!
    If(structure% str_type /=STR_TYPE_MOLECULE .And. &
         & structure% str_type /=STR_TYPE_CRYSTAL) Then
31     Format('''This subroutine is for crystals/molecules only! ''')
       Write(errbuf,31) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    Write(outfile,fmtstart)nsubfunc
    !!
    Call watch(nmodule//cpumesg_pot,cpu_start,reset=.True.)
    !!
    !!
    !! 
    Call palloc('rvylm',rvylm,1_t_int,3_t_int,1_t_int,2_t_int&
         &,1_t_int,(potential% max_L+1)**2,nmodule,nsubfunc)
    !!
    !!
    !!
    !!
    isaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       !!
       pmesha => potential% p_pmeshs(issa)
       !!
       tt(:)=structure % p_tau(:,isa)
       !!
       !!
       gv(:)=0.0_t_real
       gvon(:)=0.0_t_real
       gvoff(:)=0.0_t_real
       gvtail(:)=0.0_t_real
       !!
       !! onsite contribs
       !!
       p=potential% ew_p_r
       !!
       !!
       Call HFonsite(isa,structure% nsite,fgam(issa),mspin,degree,coulerg_nr&
            &,pmesha,potential% nrp&
            &,potential% max_L,potential% mlmp,potential% p_nlmp&
            &,potential% p_lmp,potential% ew_p_r,potential% p_lcd&
            &,potential% p_ac,potential% p_Nl,fon,fewon)
       !!
       !!
       !Print *,'fon(1:3)=',fon(1:3)
       gvon(1:3)=gvon(1:3)+fon(1:3)
       If(structure% str_type ==STR_TYPE_MOLECULE )Then
          dolew=.True.
          gvon(1:3)=gvon(1:3)+fewon(1:3)!! molecule ewald back compensation
       Endif

       !!
       !!
       !!
       !! offsite contribs
       !!
       isbloop: Do isb=1_t_int,structure% nsite
          !!
          issb=structure% p_isrt(isb)
          !!
          pmeshb => potential% p_pmeshs(issb)
          !!
          irloop: Do irs=1,lattice% p_npairs(isb,isa)
             r(:)=-lattice% p_pairs(:,irs,isb,isa)
             rb2=Dot_product(r,r)
             rb=Sqrt(rb2)
             
         
             Call calc_real_vec_Ylm(r,potential% max_L,rvylm)
            
             
             
             rltrhoif: If(rb <= pmeshb% rmax)Then
                
                If(rb /= 0.0_t_real ) Then
                
                   gg(:)=0.0_t_real

                   Do j=1_t_int,potential% p_nlmp(isb)
                      !!
                      ilm=potential% p_lmp(j,isb)
                      l=llilm(ilm)
                      deg=Sqrt((2.0_t_real*l+1.0_t_real))
                      !!
                      Call potderivs(rb,mspin,pmeshb,degree,coulerg_nr&
                           &,potential,isb,issb,j,vp,vm,dolew)

                      !!
                      gg(:)=gg(:)+(&
                           &    -rvylm(:,2,ilm)*Sqrt(l+1.0_t_real)&
                           &     *vp&
                           &    +rvylm(:,1,ilm)*Sqrt(l*1.0_t_real)&
                           &     *vm&
                           &      )/deg/rb2

                   Enddo ! j
                   !!
                   gvoff(:)=gvoff(:)+gg(:)
                   !!
                Else
         
                   !! should not happen, since it means that atoms 
                   !! are very close, the true r=0 case is treated 
                   !! differently

                   Write(errbuf,3) isa,isb
                   Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)


                Endif
                                
             Endif rltrhoif
             
          End Do irloop
       End Do isbloop
       !!
       !!
       !!
       !!

       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          !!
          isbloop2: Do isb=1_t_int,structure% nsite
             !!
             issb=structure% p_isrt(isb)
             !!
             pmeshb => potential% p_pmeshs(issb)
             !!
             r(:)=tt(:)-structure % p_tau(:,isb)
             rb2=Dot_product(r,r)
             rb=Sqrt(rb2)
             !!
             !!
             Call calc_real_vec_Ylm(r,potential% max_L,rvylm)
             !!
             !!
             If(rb > pmeshb% rmax)Then
                
                !! add up the multipole tails 
                gg(:)=0.0_t_real
                Do j=1_t_int,potential% p_nlmp(isb)
                   ilm=potential% p_lmp(j,isb)
                   l=llilm(ilm)
                   deg=(2.0_t_real*l+1.0_t_real)
                   gg(:)=gg(:)+potential% p_ac(j,isb)*fourpi&
                        &*(rvylm(:,2,ilm)*Sqrt((l+1.0_t_real)/deg))*rb**(-l-2)
                   
                Enddo
                gvtail(:)=gvtail(:)+gg(:)
                !!
             Endif
             !!
          End Do isbloop2
          
       Else !! STR_TYPE_CRYSTAL
          !!
          !!
          Call vew_force(structure% p_tau(1:3,isa),potential% p_vft&
               &,lattice% nlattg,lattice% p_glat,gg)
          !!
          gvtail(1:3)=gg(1:3)
          !!
          !!
          !!
       End If !! STR_TYPE_MOLECULE
       !!
       !!
       gvon(:)=gvon(:)*potential% p_natom(issa)
       gvoff(:)=gvoff(:)*potential% p_natom(issa)
       gvtail(:)=gvtail(:)*potential% p_natom(issa)
       !!
       hfforces(:,isa)=gvon(:)+gvoff(:)+gvtail(:)
       If(outlevel >= L_many) Then
          Write(outfile,11)isa,gvon(1:3)
          Write(outfile,12)isa,gvoff(1:3)
          Write(outfile,13)isa,gvtail(1:3)
          Write(outfile,14)isa,hfforces(:,isa)
          Write(outfile,*)
       Endif
11     Format('FORCES HF on: isa=',i5,' : ',3(f15.8,2x))
12     Format('FORCES HF of: isa=',i5,' : ',3(f15.8,2x))
13     Format('FORCES HF ta: isa=',i5,' : ',3(f15.8,2x))
14     Format('FORCES HF   : isa=',i5,' : ',3(f15.8,2x))
       !!
    Enddo isaloop

    !!
    !!
    !!
    !!
    Call pfree(rvylm)
    !!
    !!
    Call watch(nmodule//cpumesg_pot,cpu_stop,delete=.True.,Print=.True.)
    !!
    Write(outfile,fmtend) nsubfunc
    !!
    !Stop 'HF'
    Return
    !!
3   Format("'Atoms isa=",i5," and isb=",i5," extremely close'")
    !!
  End Subroutine calc_HF_force
  !!***********************************************************************
  !! ft-potential (Ewald procedure) at point(x,y,z)
  !!
  Subroutine vew_force(tau,vft,nlattg,glatt,f)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nlattg
    Real(t_real),Intent(in)      :: tau(3)
    Complex(t_real),Intent(in)   :: vft(nlattg)
    Real(t_real),Intent(in)      :: glatt(3,nlattg)
    Real(t_real),Intent(out)     :: f(3)
    !!LOCAL
    Integer(t_int) :: ig
    Real(t_real)  :: g,d
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    f(1:3)=0.0_t_real
    !!
    Do ig=2_t_int,nlattg
       g=Dot_product(glatt(:,ig),tau(:))
       d=Real(ci*vft(ig)*Exp(ci*g),t_real)
       f(1:3)=f(1:3)+d*glatt(1:3,ig)
    Enddo ! ig
    !!
  End Subroutine vew_force
  !!***********************************************************************
  Subroutine potderivs(rb,mspin,pmesh,degree,coulerg_nr&
       &,potential,isa,issa,j,vp,vm,dolew)
    !!
    Implicit None
    !!IO
    Type(T_Potential),Intent(in) :: potential
    Type(T_Mesh),Intent(in)      :: pmesh
    Logical(t_log),Intent(in)    :: dolew
    Integer(t_int),Intent(in)    :: isa,issa,j,mspin,degree,coulerg_nr
    Real(t_real),Intent(in)      :: rb
    Real(t_real),Intent(out)     :: vp,vm
    !!LOCAL
    Character(*),Parameter::nsubfunc="potderivs"
    Integer(t_int) :: ilm,l,nn,ir,ms
    Real(t_real)   :: Z,p,pp,al,y,y1,xi,ri,jr,rr,ex,rl,sd,cp,sdew,x1
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    !!
    Z=potential% p_natom(issa)
    p=potential% ew_p_r
    pp=p*p
    !!
    ilm=potential% p_lmp(j,isa)
    l=llilm(ilm)
    !!
    nn=coulerg_nr
    Call get_gauss(nn,p_xx,p_ww)
    !!
    al=3.0_t_real
    !!
    y=0.0_t_real
    Do ir=1,nn
       xi=(p_xx(ir)+1.0_t_real)*0.5_t_real
       ri=xi**al*rb
       jr=al*ri/xi*p_ww(ir)*0.5_t_real

       rr=ri*ri
       ex=Exp(-(pp*rr))*rr
       If(l==0)Then
          rl=1.0_t_real
       Else
          rl=ri**l
       End If
       
       
       !!TODO: adjust integration to better scheme

       sd=0.0_t_real
       Do ms=1,mspin
          sd=sd+minterpol(pmesh,degree,potential% p_lcd(:,j,isa,ms),ri)
       Enddo
       sdew=potential% p_ac(j,isa)*rl*ex/potential% p_Nl(l)
       If(dolew)Then
          y=y+sd*rl*jr
       Else
          y=y+(sd-sdew)*rl*jr
       Endif
    Enddo                 ! ir

    x1=(rb/pmesh% rmax)**(1.0_t_real/al)
    y1=0.0_t_real
    Do ir=1,nn
       xi=(p_xx(ir)+1.0_t_real)*0.5_t_real
       xi=(1.0_t_real-x1)*xi+x1
       ri=xi**al*pmesh% rmax
       jr=al*ri/xi*(1.0_t_real-x1)*p_ww(ir)*0.5_t_real
       
       rr=ri*ri
       ex=Exp(-(pp*rr))*rr
       If(l==0)Then
          rl=1.0_t_real
       Else
          rl=ri**l
       End If

       !!TODO: adjust integration to better scheme

       sd=0.0_t_real
       Do ms=1,mspin
          sd=sd+minterpol(pmesh,degree,potential% p_lcd(:,j,isa,ms),ri)
       Enddo
       sdew=potential% p_ac(j,isa)*rl*ex/potential% p_Nl(l)
       If(dolew)Then
          y1=y1+sd/rl/ri*jr
       Else
          y1=y1+(sd-sdew)/rl/ri*jr
       Endif
    Enddo                 ! ir
    !!
    !!
    If(rb<=1.0e-4_t_real)Then
       Write(errbuf,3) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
3      Format('''Subroutine called for small interatomic distance! &
            &'',/,'' Wrong geometry?''')
    Endif
    !!
    !!
    ri=rb
    If(l==0)Then
       rl=1.0_t_real
    Else
       rl=ri**l
    Endif
    sd=y/rl
    cp=y1*rl*ri
    If(j == 1_t_int)sd=sd-Z/sq4pi
    vp=-sd*fourpi
    vm=+cp*fourpi
    !!
    !!
    !!
    Return
    !!
  End Subroutine potderivs
  !!***********************************************************************
  Subroutine molpottogrid(structure,mspin,lattice&
       &,potential,mol,degree,etot)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Type(T_Mol),Intent(inout)    :: mol
    Type(T_Etot),Intent(inout)   :: etot
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: degree
    !!LOCAL
    Character(*),Parameter::nsubfunc="molpottogrid"
    Logical(t_log) :: alert
    Integer(t_int) :: isa,issa,j,ms,ir,im,isb,issb,irs0,irs,indexa,indexb,in
    Integer(t_int) :: l,ilm,ii,jj,nn,i
    Real(t_real)   :: rsh(3),p(3),r(3),rb2,rb,vh,oorb,oorb2
    Real(t_real)   :: grho(3,2),fn,df,d2f,fdp,trho,a,afdp,gfn(3),gfdp(3)
    Real(t_real)   :: tgrad(3),gradrhos(2),gradrho,afpmpm,gqv(5)
    Real(t_real)   :: ggrho(3,3,2),laprhos(2)
    Real(t_real)   :: mixrhos(2),mixrho
    Real(t_real)   :: fpmpm,d
    Real(t_real)   :: coefa(0:degree)
    Real(t_real)   :: coefb(0:degree)
    Real(t_real)   :: tt(3),vc,vc0,rho(2),rho0(2),dd,rhot(2),ttt(3),vvew
    Real(t_real)   :: error,tol,ff,ww,vat,excat,vex(2),epsxc,veff(2),vxcat(2)
    Real(t_real)   :: exc,atexc,envv,envc,envxc,atvxc
    Real(t_real)   :: ydn(1:degree+1)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    Type(T_Mesh),Pointer :: atommeshb,pmeshb
    Real(t_real),Pointer :: p_dlcd(:,:,:,:)
    Real(t_real),Pointer :: p_d2lcd(:,:,:,:)
    Character(*),Parameter  :: cpumesg_pot='pot -> grid'
    !Character(100)   :: str!!TODO:remove
    !Type(T_Mesh) :: imesh!!TODO:remove
    !Real(t_real)   :: tmp(10000) !!TODO:remove
    !!
    Nullify(p_dlcd)
    Nullify(p_d2lcd)
    !!
    Nullify(p_yb,p_rvylm,p_qvylm)
    !!
    !!
    Write(outfile,fmtstart)nsubfunc
    !!
    Call watch(nmodule//cpumesg_pot,cpu_start,reset=.True.)
    !!
    !!
!!$    mol% calc_vatomlm=.False.
!!$    If(mol% calc_vatomlm)Then
!!$       Call palloc('mol% p_vatomlm',mol% p_vatomlm&
!!$            &,1_t_int,atommesh% nr&
!!$            &,1_t_int,structure% nsort&
!!$            &,1_t_int,mspin&
!!$            &,nmodule,nsubfunc)
!!$       mol% p_vatomlm(:,:,:)=0.0_t_real
!!$       !!
!!$       Call palloc('vat',vat &
!!$            &,1_t_int,mol% mesh% nr&
!!$            &,1_t_int,structure% nsort&
!!$            &,1_t_int,mspin&
!!$            &,nmodule,nsubfunc)
!!$       vat(:,:,:)=0.0_t_real
!!$    Endif

    !!
    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    ff=1.0_t_real
    !!
    !!
    !! prepare gradient functions
    !!
    Call prep_for_gga(structure,mspin,potential,p_yb,p_dlcd,p_d2lcd&
         &,p_rvylm,p_qvylm)
    !!
    exc=0.0_t_real
    envxc=0.0_t_real
    atexc=0.0_t_real
    atvxc=0.0_t_real
    envv=0.0_t_real
    envc=0.0_t_real
    etot% eh0=0.0_t_real
    !!
    isaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       tt(:)=structure % p_tau(:,isa)
       !!
       imloop: Do im=1,mol% p_nm(isa)
          rsh(:)=mol% p_rm(:,im,isa)
          ww=mol% p_wm(im,isa)
          ir=mol% p_rindxm(im,isa)
          !!wgw=mol% p_wgm(im,isa)
          !!ra=mol% mesh% p_rmesh(ir)
          !!
          !Print *,isa,im,isb,irs,rsh,ra,wgw
          !Stop
          !!
          vc=0.0_t_real
          vc0=0.0_t_real
          rho(1:2)=0.0_t_real
          grho(1:3,1:2)=0.0_t_real

          ggrho(1:3,1:3,1:2)=0.0_t_real
          laprhos(1:2)=0.0_t_real
          rho0(1:2)=0.0_t_real
          vat=0.0_t_real
          excat=0.0_t_real
          vxcat=0.0_t_real
          isbloop: Do isb=1_t_int,structure% nsite

             issb=structure% p_isrt(isb)

             atommeshb => mol% p_atommeshs(issb)
             pmeshb    => potential% p_pmeshs(issb)
             
             irs0=1
             If(isb==isa)Then
                irs0=0
             Endif

             irloop: Do irs=irs0,lattice% p_npairs(isb,isa)
                If(irs==0)Then
                   p(:)=0.0_t_real
                Else
                   p(:)=lattice% p_pairs(:,irs,isb,isa)
                Endif
                r(:)=rsh(:)-p(:)
                rb2=Dot_product(r,r)
                rb=Sqrt(rb2)
                !!
                If(rb > pmeshb% rmax) Cycle irloop
                !!
                If(rb==0.0_t_real)Then
                   !! whenever rb==0 the resulting divergence
                   !! is screened in the relevant integrals
                   !! at the next neighbours the shape function
                   !! is zero and at the origin the jacobian
                   !! Thus, we may ignore the case rb==0
                   oorb=0.0_t_real
                Else
                   oorb=1.0_t_real/rb
                Endif
                oorb2=oorb*oorb

                !!
                Call ylms12(p_yb,r(1),r(2),r(3),potential% max_L)
                !!
                If(getxcversion() == XCVER_PBE96 )Then
                   Call calc_real_quad_harmonics_opt(potential% max_L&
                        &,p_yb,p_rvylm,p_qvylm)
                Endif

                
                
                
                indexa=mintpl_coefs(degree,atommeshb,rb,coefa)
                indexb=mintpl_coefs(degree,pmeshb,rb,coefb)

                
                
                msloop: Do ms=1_t_int,mspin


                   dd=Dot_product(&
                        &potential% p_lcd(indexb:indexb+degree,1,isb,ms)&
                        &,coefb(0:degree))/sq4pi
                   rho0(ms)=rho0(ms)+dd*oorb2
                   !!
                   dd=0.0_t_real
                   Do j=2_t_int,potential% p_nlmp(isb)
                      dd=dd+Dot_product(&
                           &potential% p_lcd(indexb:indexb+degree,j,isb,ms)&
                           &,coefb(0:degree))*p_yb(potential% p_lmp(j,isb))
                   Enddo
                   rho(ms)=rho(ms)+dd*oorb2
                   !!
                   !! calculate gradients for GGA
                   ggaif: If(getxcversion() == XCVER_PBE96 )Then
                      !!
                      afdp=0.0_t_real
                      afpmpm=0.0_t_real
                      gfn(:)=0.0_t_real
                      gfdp(:)=0.0_t_real
                      gqv(:)=0.0_t_real
                      !!
                      ggajloop: Do j=1,potential% p_nlmp(isb)
                         ilm=potential% p_lmp(j,isb)
                         l=llilm(ilm)
                         
                         fn=Dot_product(&
                              &potential% p_lcd(indexb:indexb+degree,j,isb,ms)&
                              &,coefb(0:degree))*oorb2
                         df=Dot_product(&
                              & p_dlcd(indexb:indexb+degree,j,isb,ms)&
                              &,coefb(0:degree))*oorb
                         d2f=Dot_product(&
                              & p_d2lcd(indexb:indexb+degree,j,isb,ms)&
                              &,coefb(0:degree))
                         
                         !! gradient of density
                         fdp=df-(2.0_t_real+l)*fn
                         
                         !!
                         fpmpm=d2f-(2.0_t_real*l+5.0_t_real)*df&
                              &+(l+4.0_t_real)*(l+2.0_t_real)*fn
                         !!
                         afdp   =afdp   +fdp  *p_yb(ilm)
                         afpmpm =afpmpm +fpmpm*p_yb(ilm)
                         gfn(:) =gfn(:) +fn   *p_rvylm(:,ilm)
                         gfdp(:)=gfdp(:)+fdp  *p_rvylm(:,ilm)
                         gqv(:) =gqv(:) +fn   *p_qvylm(:,ilm)
                         !!
                      Enddo ggajloop
                      !!
                      !! collect the separate sums
                      grho(:,ms)=grho(:,ms)+gfn(:)*oorb+afdp*r(:)*oorb2
                      !!
                      Do jj=1,3
                         ggrho(jj,jj,ms)=ggrho(jj,jj,ms)+afdp*oorb2
                         Do ii=1,3
                            ggrho(ii,jj,ms)=ggrho(ii,jj,ms)&
                                 &+afpmpm*r(ii)*r(jj)*oorb2*oorb2&
                                 &+(gfdp(ii)*r(jj) +gfdp(jj)*r(ii))&
                                 & *oorb*oorb2
                         Enddo
                      Enddo
                      !!
                      !!
                      nn=0
                      Do jj=1,3
                         Do ii=jj,3
                            nn=nn+1
                            If(nn==6) Then
                               a=(-gqv(1)-gqv(4))*oorb2
                            Else
                               a=gqv(nn)*oorb2
                            Endif
                            ggrho(ii,jj,ms)=ggrho(ii,jj,ms)+a
                            If(ii /= jj) ggrho(jj,ii,ms)=ggrho(jj,ii,ms)+a
                         Enddo
                      Enddo
                      !!
                   Endif ggaif
                   !!
                End Do msloop
                !!
                dd=Dot_product(potential% p_vc(indexb:indexb+degree,1,isb)&
                     &,coefb(0:degree))/sq4pi
                vc0=vc0+dd*oorb
                !!
                Do in=0_t_int,degree
                   dd=0.0_t_real
                   Do j=2_t_int,potential% p_nlmp(isb)
                      dd=dd+potential% p_vc(indexb+in,j,isb)&
                           &*p_yb(potential% p_lmp(j,isb))
                   Enddo
                   ydn(in+1_t_int)=dd
                Enddo
                dd=Dot_product(ydn(1:1+degree),coefb(0:degree))
                vc=vc+dd*oorb
                !!
                dd=Dot_product(mol% p_vatom(indexa:indexa+degree,0,issb)&
                     &,coefa(0:degree))/sq4pi
                !!
                vat=vat+dd*oorb
                !!
                dd=Dot_product(mol% p_excatom(indexa:indexa+degree,issb)&
                     &,coefa(0:degree))/sq4pi
                !!
                excat=excat+dd*oorb
                !!
                Do ms=1,mspin
                   dd=Dot_product(mol% p_vxcatom(indexa:indexa+degree,ms,issb)&
                        &,coefa(0:degree))/sq4pi
                   !!
                   vxcat(ms)=vxcat(ms)+dd*oorb
                Enddo
                !!
             End Do irloop
          End Do isbloop
          !!
          rhot(1:2)=rho(1:2)+rho0(1:2)
          !! xc-potential
          If(ir==1)Then
             !! we are at the origin, the density is not well defined in relat
             !! but this point gives zero anyway for all relevant integrals,
             !! thus we can set the density to zero and avoid the adjustrho
             !! error message, which has no meaning.
             !!
             rhot(1:2)=0.0_t_real
             grho(1:3,1:2)=0.0_t_real
             ggrho(1:3,1:3,1:2)=0.0_t_real
             laprhos(1:2)=0.0_t_real
          Endif
          !!
          !! GGA
          If (getxcversion() == XCVER_PBE96 )Then
             trho=rhot(1)+rhot(2)
             tgrad(1:3)=grho(1:3,1)+grho(1:3,2)
             gradrhos(1)=Sqrt(Sum(grho(1:3,1)**2))
             gradrhos(2)=Sqrt(Sum(grho(1:3,2)**2))
             gradrho=Sqrt(Sum(tgrad(1:3)**2))
             !!
             Do ms=1,mspin
                !!laprhos(ms)=Sum((/(ggrho(nn,nn,ms),nn=1,3)/))
                d=0.0_t_real
                Do nn=1,3
                   d=d+ggrho(nn,nn,ms)
                Enddo
                laprhos(ms)=d
             Enddo
             !!
             mixrhos(1)=Dot_product(grho(1:3,1)&
                  &,Matmul(ggrho(1:3,1:3,1),grho(1:3,1))) 
             mixrhos(2)=Dot_product(grho(1:3,2)&
                  &,Matmul(ggrho(1:3,1:3,2),grho(1:3,2))) 
             
             !!mixrho=Dot_product(tgrad(1:3)&
             !!     &,Matmul(ggrho(1:3,1:3,1)+ggrho(1:3,1:3,2),tgrad(1:3)))
             mixrho=0.0_t_real
             Do i=1,3
                d=0.0_t_real
                Do nn=1,3
                   d=d+(ggrho(i,nn,1)+ggrho(i,nn,2))*tgrad(nn)
                Enddo
                mixrho=mixrho+tgrad(i)*d
             Enddo
          Endif
          !!
          !!
          Call adjustrho(rhot,ff,error,mspin,tol,alert)
          Call vxcrho(mspin,rhot,gradrhos,gradrho,laprhos,mixrhos,mixrho&
               &,vex,epsxc,potential% xreduce)
          !Print *,'rhot',rhot
          !Print *,'gradrhos',gradrhos
          !Print *,'gradrho',gradrho
          !Print *,'laprhos',laprhos
          !Print *,'mixrhos',mixrhos
          !Print *,'mixrho',mixrho
          !Print *,'ggrho1',Sum(Abs(ggrho(1:3,1:3,1)))
          !Print *,'ggrho2',Sum(Abs(ggrho(1:3,1:3,2)))
          !Print *,'vex',vex
          !!
          ttt(:)=rsh(:)+tt(:)
          If(structure% str_type == STR_TYPE_CRYSTAL)Then
             vvew=vew(ttt(:),potential% p_vft&
                  &,lattice% nlattg,lattice% p_glat)
          Else If(structure% str_type == STR_TYPE_MOLECULE)Then
             vvew=vew_sum(ttt(:),structure% nsite,structure% p_isrt&
                  &,structure% p_tau,potential% p_pmeshs,potential% nrp,degree&
                  &,potential% max_L,potential% mlmp,potential% p_nlmp&
                  &,potential% p_lmp,potential% p_vew&
                  &,potential% p_ac)
          Else
             vvew=0.0_t_real
             Call terminate(TERM_CRASH,Trim(nsubfunc)//'unknown str_type!')
          Endif
          !!
          !! total Coulomb potential of new density
          vh=vc+vc0+vvew
          !!
          !! integral n_new*(veff_old-vat)
          veff(1:mspin)=vh+vex(1:mspin)-vat
          !!
          envv=envv+Dot_product(rhot(1:mspin),veff(1:mspin))*ww
          !!
          !! integral n_new*vc_new
          envc=envc+Sum(rhot(1:mspin))*(vvew)*ww&
               &+Sum(rho (1:mspin))*vc *ww
          !!
          If(  ir == 1_t_int )Then
             etot% eh0=etot% eh0+potential% p_natom(issa)*(vh)
          Endif
          !!
          !!
          exc=exc+(epsxc)*Sum(rhot(1:mspin))*ww
          atexc=atexc+(-excat)*Sum(rhot(1:mspin))*ww
          !!todo: GGA
          envxc=envxc+Dot_product(rhot(1:mspin),vex(1:mspin))*ww
          atvxc=atvxc+Dot_product(rhot(1:mspin),-vxcat(1:mspin))*ww
          !!
          !!
          !!
          !! the effective potential
          Do ms=1,mspin
             mol% p_vm2(ms,im,isa)=(vh+vex(ms))
          Enddo
          !!
!!$          If(mol% calc_vatomlm)Then
!!$             Do ms=1,mspin
!!$                vat(ir,issa,ms)=vat(ir,issa,ms)&
!!$                     &+mol% p_vm2(ms,im,isa)*wgw*ra/sq4pi&
!!$                     &/structure% p_mult(issa)
!!$                If(ir==1)Then
!!$                   vat(ir,issa,ms)=-potential% p_natom(issa)*sq4pi
!!$                Endif
!!$             Enddo
!!$          Endif

          !!
       Enddo imloop
    Enddo isaloop
    !!
    !!
!!$    If(mol% calc_vatomlm)Then
!!$       !!
!!$       !! output p_vatomlm
!!$       !!
!!$
!!$       Call setup_bspline_mesh(mol% mesh,4_t_int)
!!$       Nullify(bcoef)
!!$       Call palloc("bcoef",bcoef,1_t_int,mol% mesh% nr,nmodule,nsubfunc)
!!$
!!$       
!!$       Do ms=1,mspin
!!$          Do issa=1,structure% nsort
!!$             !!
!!$             bcoef(:)=vat(:,issa,ms)
!!$             Call makebsplint(mol% mesh,bcoef)
!!$             !!
!!$             Do ir=1,atommesh% nr
!!$                ra=atommesh% p_rmesh(ir)
!!$                !!
!!$                dd=bvalue(mol% mesh% p_knots,bcoef,mol% mesh% nr&
!!$                     &,mol% mesh% k, ra, 0_t_int )
!!$                !!
!!$                If(ir==1) dd=-potential% p_natom(issa)*sq4pi
!!$                !!
!!$                mol% p_vatomlm(ir,issa,ms)=dd
!!$                !!
!!$             Enddo
!!$          End Do
!!$       End Do
!!$       !!
!!$       Call pfree(bcoef)
!!$       !!
!!$       
!!$       Open(12,file='+vatomlm')
!!$       Do ms=1,mspin
!!$          Do issa=1,structure% nsort
!!$             Do ir=1,atommesh% nr
!!$                ra=atommesh% p_rmesh(ir)
!!$                
!!$                Write(12,34)ra,mol% p_vatomlm(ir,issa,ms)/sq4pi
!!$             Enddo
!!$             Write(12,2)
!!$          Enddo
!!$       Enddo
!!$       Close(12)
!!$2      Format(' ')
!!$34     Format(10(e39.15e3,2x))
!!$    Endif
    !!
    !!
    etot% env=etot% env+envv
    !!
    envc=envc*0.5_t_real
    !!
    etot% envc=etot% envc+envc
    !!
    etot% eh0=-etot% eh0*0.5_t_real
    !!
    etot% exc=etot% exc+exc+atexc
    !!
    etot% envxc=etot% envxc+envxc+atvxc
    !!
    Write(outfile,10 ) Trim(nsubfunc),exc
    Write(outfile,15 ) Trim(nsubfunc),atexc
    Write(outfile,16 ) Trim(nsubfunc),etot% exc

    Write(outfile,110 ) Trim(nsubfunc),envxc
    Write(outfile,115 ) Trim(nsubfunc),atvxc
    Write(outfile,116 ) Trim(nsubfunc),etot% envxc

    Write(outfile,11 ) Trim(nsubfunc),envv
    Write(outfile,13 ) Trim(nsubfunc),envc
    Write(outfile,14 ) Trim(nsubfunc),etot% eh0
10  Format(a,': EXC(n)             : ',f30.8)
15  Format(a,': EXC(at)            : ',f30.8)
16  Format(a,': EXC                : ',f30.8)
110 Format(a,': n*VXC(n)           : ',f30.8)
115 Format(a,': n*VXC(at)          : ',f30.8)
116 Format(a,': ENVXC              : ',f30.8)
11  Format(a,': Int(n*v)           : ',f30.8)
13  Format(a,': Int(n*VCoulomb)    : ',f30.8)
14  Format(a,': Int(Z*VCoulomb)    : ',f30.8)
    !!
    !!
    !!
    If(alert)Then
      
3      Format('''Numerical inaccuracy in vxc/exc &
            &calculation! '',/,'' maxerror=',e12.4e3,'''')
       Write(errbuf,3) error
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
    Endif
    !!
    Call watch(nmodule//cpumesg_pot,cpu_stop,delete=.True.,Print=.True.)
    !!
    Write(outfile,fmtend) nsubfunc
    !!
    !!
    Call pfree(p_dlcd)
    Call pfree(p_d2lcd)
    Call pfree(p_qvylm)
    Call pfree(p_rvylm)
    Call pfree(p_yb)
    !Stop
    !!
    Return
    !!
  End Subroutine molpottogrid
  !!***********************************************************************
  Subroutine dumpmolpot(mol,etot,mode)
    !!
    Implicit None  
    !!IO
    Type(T_Mol),Intent(inout)    :: mol
    Type(T_Etot),Intent(inout)   :: etot
    Character(*),Intent(in)      :: mode
    !!LOCAL
    Character(*),Parameter::nsubfunc="dumpmolpot"
    Integer(t_int) :: lmspin,lmaxpnts,lnsite
    Integer ::err
    Character(30)  :: str30
    If(Trim(mode) == 'write')Then
       lmspin=Size(mol% p_vm2,1)
       lmaxpnts=Size(mol% p_vm2,2)
       lnsite=Size(mol% p_vm2,3)
       Open(12,file=Trim(gdump(DUMP_MOLPOT)% fname),form='UNFORMATTED')
       Write(12) '#mspin                       =',lmspin
       Write(12) '#mol% maxpnts                =',lmaxpnts
       Write(12) '#nsite                       =',lnsite
       Write(12) '#mol% p_vm2                  =',mol% p_vm2
       Write(12) '#etot% eh0                   =',etot% eh0
       Write(12) '#etot% env                   =',etot% env
       Write(12) '#etot% envc                  =',etot% envc
       Write(12) '#etot% exc                   =',etot% exc
       Write(12) '#etot% envxc                 =',etot% envxc
       Close(12)
    Else
       Open(12,file=Trim(gdump(DUMP_MOLPOT)% fname),status='old'&
            &,form='UNFORMATTED')
       err=0
       Read(12,iostat=err) str30,lmspin
       Read(12,iostat=err) str30,lmaxpnts
       Read(12,iostat=err) str30,lnsite
       If(err/=0.Or.&
            &lmspin/=Size(mol% p_vm2,1).Or.&
            &lmaxpnts/=Size(mol% p_vm2,2).Or.&
            &lnsite/=Size(mol% p_vm2,3))Then
          Write(errbuf,34) Trim(gdump(DUMP_MOLPOT)% fname)
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Endif
       Read(12,iostat=err) str30,mol% p_vm2
       Read(12,iostat=err) str30,etot% eh0
       Read(12,iostat=err) str30,etot% env
       Read(12,iostat=err) str30,etot% envc
       Read(12,iostat=err) str30,etot% exc
       Read(12,iostat=err) str30,etot% envxc
       Close(12)
    Endif
    !!
    Return
    !!
34  Format("'data on dump file ",a," seem to belong to different settings.',/&
         &,' I will ignore the file''s content and proceed with normal &
         &calculation!'")
    !!
  End Subroutine dumpmolpot
  !!***********************************************************************
  Subroutine setup_potential_restrictions(structure,basis,potential)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Potential),Intent(inout) :: potential
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_potential_restrictions"
    Logical(t_log) :: exists
    Integer(t_int) :: nline,sort,nmain,l,ivr,i
    Integer :: eof,err
    Character(1):: ch
    Character(5000):: buf,word
    Character(*),Parameter :: fname='=.pot_restriction'
    !!
    !!
    !!
    Inquire(file=Trim(fname),exist=exists)
    If(.Not.exists) return
    !!
    Write(outfile,"(/,'File ''',a,''' found, you know what you do?',/)")&
         &Trim(fname)
    !!
    Call copy_file_to_output(outfile,Trim(fname))
    !!
    Open(12,file=Trim(fname))
    nline=0
    potential% nspherdens=0
    readloop: Do 
       Read(12,12,iostat=eof) buf
       If(eof < 0 ) Exit readloop
       nline=nline+1
       If( Len_trim(buf)==0) Cycle readloop
       word=Adjustl(buf)
       If( word(1:1)=='#') Cycle readloop
       If(word(1:Len_trim(word))=='netvv') Then
          potential% spheroption=SPHER_NETVV
          Cycle
       Endif
       If(word(1:Len_trim(word))=='netvvall') Then
          potential% spheroption=SPHER_NETVVALL
          Cycle
       Endif
       If(word(1:Len_trim(word))=='netall') Then
          potential% spheroption=SPHER_NETALL
          Cycle
       Endif
       If(.Not.next_word(buf,word)) Goto 661
       Read(word,*,iostat=err) sort
       If(err/=0) Goto 661
       if(sort<1 .or. sort> structure% nsort) goto 661
       If(.Not.next_word(buf,word)) Goto 662
       Read(word,"(i1,a1)",iostat=err) nmain,ch
       l=lcharqn(ch,.False.)
       If(l<0) Goto 662
       If(err/=0) Goto 662
       !Print *,Trim(word),nmain,l
       !!
       i=-1_t_int
       Do ivr=1,basis % p_nRv(sort)
          If( basis % p_lRv(ivr,sort) == l .And. &
               & basis % p_nnRv(ivr,sort) == nmain )Then
             i=ivr
             Exit
          Endif
       Enddo
       If(i < 0_t_int) goto 661
       !!
       potential% nspherdens=potential% nspherdens+1
       !!
       Call realloc('p_spherrestrictsort',potential% p_spherrestrictsort&
            &,1_t_int,potential% nspherdens,nmodule,nsubfunc)
       Call realloc('p_spherrestrictivr',potential% p_spherrestrictivr&
            &,1_t_int,potential% nspherdens,nmodule,nsubfunc)
       !!
       potential% p_spherrestrictsort(potential% nspherdens)=sort
       potential% p_spherrestrictivr(potential% nspherdens)=i
       !!
    Enddo readloop
    Close(12)
    !!
    Write(outfile,"('SPHERICAL DENSITY OPTION :',a)")spheroptions(potential% spheroption)
    !!
    Do i=1,potential% nspherdens
       sort=potential% p_spherrestrictsort(i)
       ivr=potential% p_spherrestrictivr(i)
       Write(outfile,"('SPHERICAL DENSITY FOR sort ',i5,' orbital ',i1,a1)")&
            &sort,basis% p_nnRv(ivr,sort),lqnchar(basis% p_lRv(ivr,sort))
    Enddo
    !!
    !!
    Return
    !!
661 Continue
    !!
    Write(errbuf,"('''File ''''',a,''''' line ',i5,': expect valid sort number &
         &at beginning of line''')")&
         &Trim(fname),nline
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    !!
662 Continue
    !!
    Write(errbuf,"('''File ''''',a,''''' line ''''',i5,''''': expect valid orbital after sort''')")&
         &Trim(fname),nline
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    !!
12  Format(a)
    !!
  End Subroutine setup_potential_restrictions
  !!***********************************************************************
  
    
End Module molpot
