Module MPICENTRAL
  !!
  Use precisiondefs
  Use errors
  Use file_iounits
  Use memory
  Use tmpi
  !!
  !!
  Implicit None 
  !!
#if defined (MPI)
  Include 'mpif.h'
#endif
  !!
  Private
  !!
  Character(*),Parameter::nmodule="MPICENTRAL"  
  !!
  Public :: init_mpi,wait_for_tasks,test_mpi
  !!***********************************************************************
  !!START DO NOT CHANGE, will be set by install/MMakefile
  Logical(t_log),Parameter :: do_mpi=.False.
  !!END DO NOT CHANGE
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine init_mpi()
    !!
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_mpi"
#if defined (MPI)
    Integer :: ierr
#endif
    !!
    g_mpi% mpion=do_mpi
    g_mpi% my_rank=0
    !!
    If(.Not. g_mpi% mpion)Return
    !!
#if defined (MPI)
    Call mpi_init(ierr)
    If(ierr/=0)Then
       g_mpi% mpion=.False.
       Call terminate(TERM_CRASH,"cannot initialize mpi")
    Endif
    !!
    Call MPI_Comm_rank(MPI_COMM_WORLD, g_mpi% my_rank, ierr)
    If(ierr/=0)Then
       Call terminate(TERM_CRASH,"cannot call MPI_Comm_rank")
    Endif
    Call MPI_Comm_size(MPI_COMM_WORLD, g_mpi% nprocs, ierr)
    If(ierr/=0)Then
       Call terminate(TERM_CRASH,"cannot call MPI_Comm_size")
    Endif
    !!
    Call check_mpi_environment()
    !!
#endif
    Return
    !!
  End Subroutine init_mpi
  !!***********************************************************************
#if defined (MPI)
  Subroutine check_mpi_environment()
    !!
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="check_mpi_environment"
    Integer(t_int) :: t_int_size,internal_int_size,t_real_size,t_log_size
    External cgettypesizes
    !!
    If(.Not. g_mpi% mpion)Return
    If(g_mpi% my_rank/=0)Return
    !!
    !!
    Call cgettypesizes(t_int_size,internal_int_size,t_real_size,t_log_size)
    !!
    Write(outfile,"('Integer(t_int) has ',i3,' bytes')")t_int_size
    Write(outfile,"('Integer        has ',i3,' bytes')")internal_int_size
    Write(outfile,"('Real(t_real)   has ',i3,' bytes')")t_real_size
    Write(outfile,"('Logical(t_log) has ',i3,' bytes')")t_log_size
    !!
    !!
    Return
    !!
  End Subroutine check_mpi_environment
#endif
  !!***********************************************************************
  Subroutine wait_for_tasks()
    !!
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="wait_for_tasks"
#if defined (MPI)
    Integer :: tag,source,size,ierr
    Integer:: status(MPI_STATUS_SIZE)
    Character(100)   :: message
    !!
    !!
    Print *,'here I am waiting',g_mpi% my_rank
    !!
    Do 
       !!
       source=0
       tag=0
       Call MPI_Recv(message,100, MPI_CHARACTER, source&
            &,tag, MPI_COMM_WORLD, status, ierr)
       Call MPI_Get_count(status, MPI_CHARACTER, size, ierr)
       !!
       Print *,'got message: ',message(1:size)
       !!
       If(message(1:size)=='doit')Then
       Else If(message(1:size)=='die')Then
          Call clean_mpi()
          Stop
       Else
          Write(outfile,"('unknown task')")
          Call mpi_abort(MPI_COMM_WORLD,1,ierr)
          Stop
       Endif
       !!
    Enddo
    !!
#endif
    Return
    !!
  End Subroutine wait_for_tasks
  !!***********************************************************************
  Subroutine test_mpi()
    !!
    Implicit None
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="test_mpi"
#if defined (MPI)
    Integer :: proc,tag,ierr
    Character(100) :: msg
    !!
    !!
    Do proc=1,g_mpi% nprocs-1
       tag=0
       msg='doit'
       Call MPI_Send(Trim(msg), Len_trim(msg), MPI_CHARACTER&
            &,proc, tag, MPI_COMM_WORLD, ierr)
    Enddo
    !!
#endif
    Return
    !!
  End Subroutine test_mpi
  !!***********************************************************************

End Module MPICENTRAL
