Module netdensity
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  Use optionstest
  Use matserv
  Use tsymmetry
  Use tpotentialdensity
  Use torbitalbasis
  Use tdiracdefs
  Use tmesh
  Use mesh
  Use integrate1d
  Use trelativistic
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="NETDENSITY"  
  !!
  Public :: calc_net_density
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine calc_net_density(mspin,structure,potential&
       &,max_L,nlmp,lmp,basis&
       &,nonvv,noncv,noncc&
       &,ronvv,roncv,roncc&
       &,quant,pmeshs,lcd,degree,relativistic,printorbmom,useOVO)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Quant),Intent(in)     :: quant
    Type(T_Potential),Intent(in) :: potential
    Type(T_Mesh),Intent(in),Target :: pmeshs(:)
    Logical(t_log),Intent(in)    :: printorbmom,useOVO
    Integer(t_int),Intent(in)    :: mspin,degree,relativistic
    Integer(t_int),Intent(in)    :: max_L,nlmp(:),lmp(:,:)
    Real(t_real),Intent(in)      :: nonvv(:,:,:,:)
    Real(t_real),Intent(in)      :: noncv(:,:,:,:)
    Real(t_real),Intent(in)      :: noncc(:,:,:,:)
    Complex(t_real),Intent(in)   :: ronvv(:,:,:)
    Complex(t_real),Intent(in)   :: roncv(:,:,:)
    Complex(t_real),Intent(in)   :: roncc(:,:,:)
    Real(t_real),Intent(out)     :: lcd(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_net_density"
    Integer(t_int) :: ms,isa,issa,icomp,ia,la,lb,ll
    Integer(t_int) :: glmax,nrp
    Logical(t_log) :: rel
    Real(t_real)   :: d
    Type(T_Mesh),Pointer :: pmesh,tmesh
    !!
    Real(t_real),Pointer     :: wgaunt(:,:,:)
    Real(t_real),Pointer     :: p_alll(:,:,:)
    Real(t_real),Pointer     :: omtmp(:),Lz(:)
    Real(t_real),Pointer     :: tfca(:,:,:)
    Real(t_real),Pointer     :: tfva(:,:,:)
    !!
    Nullify(wgaunt,p_alll,omtmp,Lz)
    Nullify(tfca,tfva)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    !! prepare for gaunt coeffs
    glmax=basis% lmx
    If(rel) glmax=basis% lmx+1
    !!
    Call palloc('wgaunt',wgaunt&
         &,1_t_int,(max_L+1_t_int)**2_t_int&
         &,1_t_int,(glmax+1_t_int)**2_t_int&
         &,1_t_int,(glmax+1_t_int)**2_t_int&
         &,nmodule,nsubfunc)
    !!
    Call makergaunt(wgaunt,max_L,glmax,glmax) 
    !!
    !!
    Call palloc('p_alll',p_alll&
         &,0_t_int,glmax&
         &,0_t_int,glmax&
         &,0_t_int,max_L&
         &,nmodule,nsubfunc)
    !!
    If(.Not.rel)Then
       p_alll(0:glmax,0:glmax,0:max_L)=0.0_t_real
       Do la=0,glmax
          Do lb=0,glmax
             Do ll=0,max_L
                d=0.0_t_real
                If(Abs(la-lb)<=LL .And. LL <= la+lb.And.la>0.And.lb>0)&
                     & d=(la*(la+1.0_t_real)+lb*(lb+1)-ll*(ll+1))&
                     &*0.5_t_real&
                     &/Sqrt(la*(la+1.0_t_real)*lb*(lb+1.0_t_real))
                p_alll(la,lb,LL)=d
                !If(LL==0) Print *,la,lb,p_alll(la,lb,LL)
             Enddo
          Enddo
       Enddo
    Endif
    !!
    If(rel)Then
       !!
       nrp=Maxval(pmeshs(1:structure% nsort)% nr)
       !! allocate and initialize dummy array for orbital moments
       Call palloc("omtmp",omtmp,1_t_int,nrp,nmodule,nsubfunc)
       Call palloc("Lz",Lz,1_t_int,structure% nsite,nmodule,nsubfunc)
       omtmp(:)=0.0_t_real
       Lz(:)=0.0_t_real
       !!
       !!
    Endif
    netisaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       !!
       pmesh => pmeshs(issa)
       tmesh => basis% p_basmeshs(issa)
       !!
       !issaIO=isrtTOIOisrt(issa)
       !!
       Call palloc("tfca",tfca,1_t_int,pmesh% nr,1_t_int,basis% nRcmx&
            &,1_t_int,basis% ncomp,nmodule,nsubfunc)
       Call palloc("tfva",tfva,1_t_int,pmesh% nr,1_t_int,basis% nRvmx&
            &,1_t_int,basis% ncomp,nmodule,nsubfunc)
       !!
       !! if meshs are different: make temporary interpolant
       !!
       Do icomp=1,basis% ncomp
          Do ia=1_t_int,basis% p_nRc(issa)
             Call mreinter(degree,tmesh&
                  &,basis% p_fc(:,ia,icomp,issa)&
                  &,pmesh,tfca(:,ia,icomp))
          Enddo
       Enddo
       !! 
       Do icomp=1,basis% ncomp
          Do ia=1_t_int,basis% p_nRv(issa)
             Call mreinter(degree,tmesh&
                  &,basis% p_fv(:,ia,icomp,issa)&
                  &,pmesh,tfva(:,ia,icomp))
          Enddo
       Enddo
       !!
       If(.Not.rel)Then
          !!
          !! non relativistic calculation:
          !!
          !!
          Do ms=1,mspin
             !!
             Call net_dens(potential,issa,mspin&
                  &,pmesh% nr,pmesh% p_rmesh,basis% ncomp&
                  &,basis% nRcmx,basis% p_nRc(issa),basis% p_lRc(:,issa)&
                  &,basis% nRvmx,basis% p_nRv(issa),basis% p_lRv(:,issa)&
                  &,tfca,tfva,basis% p_fgam(issa)&
                  &,nonvv(:,:,isa,ms)&
                  &,noncv(:,:,isa,ms)&
                  &,noncc(:,:,isa,ms)&
                  &,max_L,nlmp(isa),lmp(:,isa),lcd(:,:,isa,ms)&
                  &,glmax,wgaunt,relativistic&
                  &,p_alll,useOVO)
             !!
          Enddo
       Else
          !!
          !!
          omtmp(:)=0.0_t_real
          !!
          Call  rel_net_dens(mspin,isa,issa,basis&
               &,pmesh% nr,pmesh% p_rmesh&
               &,ronvv(:,:,isa)&
               &,roncv(:,:,isa)&
               &,roncc(:,:,isa)&
               &,tfca,tfva,basis% p_fgam(issa),glmax,wgaunt&
               &,nlmp(isa),lmp(:,isa),max_L,lcd,omtmp(:)&
               &,quant)
          !!
          Call tjlag4int(pmesh% nr,omtmp(1:pmesh% nr),pmesh% p_jmesh&
               &,pmesh% hx,Lz(isa))
          !!
       Endif
    Enddo netisaloop
    !!
    !!
    !! output of orbital moments
    !!
!    If (printorbmom.And.rel.And.(mspin==2)) Then
    If (printorbmom.And.rel) Then
       !!
       Write(outfile,2002)
       Write(outfile,2000)
       Write(outfile,2002)
       issaloop: Do isa=1,structure% nsite
          issa=structure% p_isrt(isa)
          !!
          Write(outfile,2001) structure% p_elements(issa),isa,Lz(isa)
       Enddo issaloop
       Write(outfile,2002)
2000   Format(  '|      ONSITE ORBITAL MOMENTS      |'&
            &,/,'| atom   site    orbital moment    |')
2002   Format(  '|----------------------------------|')
2001   Format('|',1x,a2,4x,i4,8x,f8.5,t36,'|')
       !!
    Endif
    Call pfree(omtmp)
    Call pfree(Lz)
    !!
    !!
    Call pfree(tfva)
    Call pfree(tfca)
    !!
    Call pfree(wgaunt)
    Call pfree(p_alll)
    !!
    Return
    !!
  End Subroutine calc_net_density
  !!***********************************************************************
  Subroutine net_dens(potential,issa,mspin,nr,rmesh,ncomp&
       &,nRcmx,nRca,lRca,nRvmx,nRva,lRva&
       &,tfca,tfva,fgam,nonvv,noncv,noncc&
       &,max_L,nlmpa,lmpa,lcda&
       &,glmax,wgaunt,relativistic,alll,useOVO)

    !!
    Implicit None
    !!IO
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin,issa
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: nr
    Integer(t_int),Intent(in)    :: ncomp
    Integer(t_int),Intent(in)    :: glmax
    Integer(t_int),Intent(in)    :: max_L,nlmpa,lmpa(:)
    Integer(t_int),Intent(in)    :: nRcmx,nRca,lRca(nRcmx)
    Integer(t_int),Intent(in)    :: nRvmx,nRva,lRva(nRvmx)
    Logical(t_log),Intent(in)    :: useOVO
    Real(t_real),Intent(in)      :: rmesh(nr)
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: tfca(:,:,:)
    Real(t_real),Intent(in)      :: tfva(:,:,:)
    Real(t_real),Intent(in)      :: wgaunt((max_L+1_t_int)**2_t_int&
         &                               ,(glmax+1_t_int)**2_t_int&
         &                               ,(glmax+1_t_int)**2_t_int)
    Real(t_real),Intent(in)      :: alll(0:glmax,0:glmax,0:max_L)
    Real(t_real),Intent(inout)   :: lcda(:,:)
    Real(t_real),Intent(in)      :: nonvv(:,:)
    Real(t_real),Intent(in)      :: noncv(:,:)
    Real(t_real),Intent(in)      :: noncc(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="net_dens"
    Logical(t_log) :: nocore,srel,srelOVO
    Integer(t_int) :: ia,la,lqa,ma,ila,ib,lb,lqb,mb,ilb,il,ild
    Integer(t_int) :: i,j
    Integer(t_int) :: icomp,l,LL
    Real(t_real)   :: c,d,fspin,f,f2
    Real(t_real)   :: texp(nr)
    !!
    srel=(relativistic==SCALAR_RELATIVISTIC)
    srel=srel.Or.(relativistic==SCALAR_RELATIVISTIC_KH)
    srelOVO=(srel.And.useOVO)
    !srel=.False.
!!$    Real(t_real)   :: delta_n(5,2)
!!$    Character(100)  :: mode
!!$    Logical(t_log) :: exists
!!$    !!
!!$    If(special_code)Then
!!$       Inquire(file='=.delta_n',exist=exists)
!!$       If(exists)Then
!!$          Open(12,file='=.delta_n')
!!$          Read(12,*)
!!$          Read(12,*)mode
!!$          Read(12,*)
!!$          Read(12,*)delta_n(1:5,1)
!!$          Read(12,*)delta_n(1:5,2)
!!$          Close(12)
!!$       Else
!!$          mode='none'
!!$       Endif
!!$    Endif
    !!
    fspin=3.0_t_real-Real(mspin,t_real)
    !!
    If (relativistic == FULL_RELATIVISTIC ) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'full relativistic calculations need&
            & another routine!'")
    Endif
    !!
    !! core - core
    !!
    !!
    nocore=isoption('NO_CORE')
    !!
    If(nocore)Goto 333
    i=0
    Do ia=1,nRca
       la=lRca(ia)
       lqa=la*la
       Do ma=1,la+la+1
          i=i+1
          ila=lqa+ma
          j=0
          Do ib=1,nRca
             lb=lRca(ib)
             lqb=lb*lb
             Do mb=1,lb+lb+1
                j=j+1
                ilb=lqb+mb
                c=noncc(i,j)
                Do il=1,nlmpa
                   ild=lmpa(il)
                   LL=llilm(ild)
                   d=wgaunt(ild,ila,ilb)
                   f=d*c
                   Do icomp=1,ncomp
                      f2=f
                      If(srelOVO.And.icomp>=3) f2=f*alll(la,lb,LL)
                      lcda(1:nr,il)=lcda(1:nr,il)&
                           &+tfca(1:nr,ia,icomp)*tfca(1:nr,ib,icomp)*f2
                   Enddo
                Enddo
             Enddo
          Enddo
       Enddo
    Enddo
333 Continue
    !! valence - valence
    i=0
    Do ia=1,nRva
       la=lRva(ia)
       lqa=la*la
       Do ma=1,la+la+1
          i=i+1
          ila=lqa+ma
          j=0
          Do ib=1,nRva
             lb=lRva(ib)
             lqb=lb*lb
             
             Do mb=1,lb+lb+1
                j=j+1
                ilb=lqb+mb
                c=nonvv(i,j)
                illoopvv: Do il=1,nlmpa
                   ild=lmpa(il)
                   LL=llilm(ild)
                   If(LL/=0)Then
                      If(LL/=0 .And. spheroptiontest(potential,issa,ia,ib))Then
                         Print "('skip vv ivra=',i5,' ivrb=',i5,' ild=',i5)",ia,ib,ild
                         Cycle illoopvv
                      Endif
                   Endif
                   !!
                   !!
                   d=wgaunt(ild,ila,ilb)*c
                   Do icomp=1,ncomp
                      f2=d
                      If(srelOVO.And.icomp>=3) f2=d*alll(la,lb,LL)
                      lcda(1:nr,il)=lcda(1:nr,il)&
                           &+f2*tfva(1:nr,ia,icomp)*tfva(1:nr,ib,icomp)
                   Enddo
                Enddo illoopvv
             Enddo
          Enddo
       Enddo
    Enddo
    If(nocore) Goto 666
    !! core - valence
    i=0
    coreloop3: Do ia=1,nRca
       la=lRca(ia)
       lqa=la*la
       Do ma=1,la+la+1
          i=i+1
          ila=lqa+ma
          j=0
          Do ib=1,nRva
             lb=lRva(ib)
             lqb=lb*lb
             
             Do mb=1,lb+lb+1
                j=j+1
                ilb=lqb+mb
                c=noncv(i,j)*2.0_t_real
                illoopcv: Do il=1,nlmpa
                   ild=lmpa(il)
                   LL=llilm(ild)
                   If(LL/=0 .And. spheroptiontest(potential,issa,ib,0_t_int))Then
                      Print "('skip cv ivra=',i5,' ivrb=',i5,' ild=',i5)",ia,ib,ild
                      Cycle illoopcv
                   Endif
                   d=wgaunt(ild,ila,ilb)*c
                   Do icomp=1,ncomp
                      f2=d
                      If(srelOVO.And.icomp>=3) f2=d*alll(la,lb,LL)
                      lcda(1:nr,il)=lcda(1:nr,il)&
                           &+f2*tfca(1:nr,ia,icomp)*tfva(1:nr,ib,icomp)
                   Enddo
                Enddo illoopcv
             Enddo
          Enddo
       Enddo
    Enddo coreloop3
    !!
666 Continue
    !!
    texp(:)=rmesh(:)**(2*fgam)
    !!
    Do il=1,nlmpa
       ild=lmpa(il)
       l=llilm(ild)
       
       If(l==1)Then
          !Print *,lcda(1,il,isa,ms)
          If(Abs(lcda(1,il)) >1.0e-12_t_real)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="(' Assumption about L=1m density is "&
                  &//"wrong, Please report case!')")
          Endif
       Endif

       lcda(1:nr,il)=lcda(1:nr,il)*texp(1:nr)
    Enddo
    !!
    Return
    !!
  End Subroutine net_dens
  !!***********************************************************************
  Subroutine  rel_net_dens(mspin,isa,issa,basis,nr,rmesh&
       &,ronvv,roncv,roncc&
       &,rfca,rfva,fgam&
       &,glmax,wgaunt,nlmp,lmp,max_L,lcd,omtmp,quant)
    !!
    Implicit none  
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Quant),Intent(in)     :: quant
    Integer(t_int),Intent(in)    :: mspin,isa,issa
    Integer(t_int),Intent(in)    :: nr,max_L,glmax
    Integer(t_int),Intent(in)    :: nlmp,lmp(:)
    Real(t_real),Intent(in)      :: rmesh(nr)
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: rfca(nr,basis% nRcmx,2)
    Real(t_real),Intent(in)      :: rfva(nr,basis% nRvmx,2)
    Complex(t_real),Intent(in)   :: ronvv(:,:)
    Complex(t_real),Intent(in)   :: roncv(:,:)
    Complex(t_real),Intent(in)   :: roncc(:,:)
    Real(t_real),Intent(in)      :: wgaunt((max_L+1)**2&
         &                               ,(glmax+1)**2&
         &                               ,(glmax+1)**2)
    Real(t_real),Intent(inout)   :: lcd(:,:,:,:)
    Real(t_real),Intent(inout)   :: omtmp(:)
    !!LOCAL
    Character(*),Parameter  ::nsubfunc="rel_net_dens"
    Logical(t_log) :: nocore
    Integer(t_int) :: ia,icra,ivra,kapa,la,ja,ma,mua,mla,lla,ka
    Integer(t_int) :: ib,icrb,ivrb,kapb,lb,jb,mb,mub,mlb,llb,kb
    Integer(t_int) :: ilm,j,icomp,ms,bfac,l
    Real(t_real)   :: cs,den,mag,d,texp(nr),omtmpl(nr)
    Complex(t_real):: rex,ang(2)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    !!
    cs=1.0_t_real/Real(mspin,t_real)
    !!
    !!
    nocore=isoption('NO_CORE')
    !!
    !!
    omtmpl(1:nr)=0.0_t_real
    !!
    !! setup kappa array
    !!    
    !!    
    If(nocore)Goto 333
    !!    
    !! core-core
    ia=0
    ialoopcc: Do icra=1,basis% p_nRc(issa)
       kapa=basis% p_kapRc(icra,issa)
       !! angular momentum
       la=(Abs(2*kapa+1)-1)/2
       !! j-quantum number
       ja=2*Abs(kapa)-1
       maloopcc: Do ma=1,ja+1
          mua=-ja+2*(ma-1)

          !!
          ia=ia+1
          !!
          ib=0
          ibloopcc: Do icrb=1,basis% p_nRc(issa)
             kapb=basis% p_kapRc(icrb,issa)
             !! angular momentum
             lb=(Abs(2*kapb+1)-1)/2
             !! j-quantum number
             jb=2*Abs(kapb)-1
             mbloopcc: Do mb=1,jb+1
                mub=-jb+2*(mb-1)

                !!
                ib=ib+1
                !!
                rex=roncc(ia,ib)
                !!
                !!
                !!
                icomploopcc: Do icomp=1,2
                   bfac=3_t_int-2_t_int*icomp
                   !!
                   ka=kapa*bfac
                   kb=kapb*bfac
                   !!
                   lla=(Abs(2*ka+1)-1)/2
                   llb=(Abs(2*kb+1)-1)/2
                   !!
                   lmploopcc: Do j=1,nlmp
                      ilm=lmp(j)
                      !!
                      ang(1:2)=c0
                      Do mla=1,2*lla+1
                         Do mlb=1,2*llb+1
                            ang(icomp)=ang(icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,1,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,1,kb))
                            ang(3-icomp)=ang(3-icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,2,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,2,kb))
                         Enddo
                      Enddo
                      !!
                      !! 
                      den=Real((ang(1)+ang(2))*rex,t_real)*cs
                      mag=Real((ang(1)-ang(2))*rex,t_real)*cs
                      !!
                      Do ms=1,mspin
                         d=(den+(mspin-1)*(3-2*ms)*mag)
                         lcd(1:nr,j,isa,ms)=lcd(1:nr,j,isa,ms)&
                              &+(rfca(1:nr,icra,icomp)*rfca(1:nr,icrb,icomp))*d
                      Enddo
                      !!
                   Enddo lmploopcc

                   d=Real(Conjg(quant% p_chiLRchi(ka,ma,kb,mb))*rex,t_real)
                   omtmpl(1:nr)=omtmpl(1:nr)&
                        &+rfca(1:nr,icra,icomp)*rfca(1:nr,icrb,icomp)*d

                Enddo icomploopcc 
                !!
             End Do mbloopcc
          Enddo ibloopcc
       End Do maloopcc
    Enddo ialoopcc
    !!
    !!
    !!
333 Continue
    !!     valence-valence
    ia=0
    ialoopvv: Do ivra=1,basis% p_nRv(issa)
       kapa=basis% p_kapRv(ivra,issa)
       !! angular momentum
       la=(Abs(2*kapa+1)-1)/2
       !! j-quantum number
       ja=2*Abs(kapa)-1
       maloopvv: Do ma=1,ja+1
          mua=-ja+2*(ma-1)

          !!
          ia=ia+1
          !!
          ib=0
          ibloopvv: Do ivrb=1,basis% p_nRv(issa)
             kapb=basis% p_kapRv(ivrb,issa)
             !! angular momentum
             lb=(Abs(2*kapb+1)-1)/2
             !! j-quantum number
             jb=2*Abs(kapb)-1
             mbloopvv: Do mb=1,jb+1
                mub=-jb+2*(mb-1)

                !!
                ib=ib+1
                !!
                rex=ronvv(ia,ib)
                !!
                icomploopvv: Do icomp=1,2
                   bfac=3_t_int-2_t_int*icomp
                   !!
                   ka=kapa*bfac
                   kb=kapb*bfac
                   !!
                   lla=(Abs(2*ka+1)-1)/2
                   llb=(Abs(2*kb+1)-1)/2
                   !!
                   lmploopvv: Do j=1,nlmp
                      ilm=lmp(j)
                      !!
                      ang(1:2)=c0
                      Do mla=1,2*lla+1
                         Do mlb=1,2*llb+1
                            ang(icomp)=ang(icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,1,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,1,kb))
                            ang(3-icomp)=ang(3-icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,2,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,2,kb))
                         Enddo
                      Enddo
                      !!
                      den=Real((ang(1)+ang(2))*rex,t_real)*cs
                      mag=Real((ang(1)-ang(2))*rex,t_real)*cs
                      !! 
                      !!
                      Do ms=1,mspin
                         d=(den+(mspin-1)*(3-2*ms)*mag)
                         lcd(1:nr,j,isa,ms)=lcd(1:nr,j,isa,ms)&
                              &+(rfva(1:nr,ivra,icomp)*rfva(1:nr,ivrb,icomp))*d
                      Enddo
                      !!
                   Enddo lmploopvv

                   d=Real(Conjg(quant% p_chiLRchi(ka,ma,kb,mb))*rex,t_real)
                   omtmpl(1:nr)=omtmpl(1:nr)&
                        &+rfva(1:nr,ivra,icomp)*rfva(1:nr,ivrb,icomp)*d
                   
                Enddo icomploopvv
                !!
             End Do mbloopvv
          Enddo ibloopvv
       End Do maloopvv
    Enddo ialoopvv
    !!
    !!
    !!
    If(nocore) Goto 666
    !!
    !!
    !!
    !!
    !! core-valence
    ia=0
    ialoopcv: Do icra=1,basis% p_nRc(issa)
       kapa=basis% p_kapRc(icra,issa)
       !! angular momentum
       la=(Abs(2*kapa+1)-1)/2
       !! j-quantum number
       ja=2*Abs(kapa)-1
       maloopcv: Do ma=1,ja+1
          mua=-ja+2*(ma-1)

          !!
          ia=ia+1
          !!
          ib=0
          ibloopcv: Do ivrb=1,basis% p_nRv(issa)
             kapb=basis% p_kapRv(ivrb,issa)
             !! angular momentum
             lb=(Abs(2*kapb+1)-1)/2
             !! j-quantum number
             jb=2*Abs(kapb)-1
             mbloopcv: Do mb=1,jb+1
                mub=-jb+2*(mb-1)

                !!
                ib=ib+1
                !!
                rex=roncv(ia,ib)*2.0_t_real
                !!
                icomploopcv: Do icomp=1,2
                   bfac=3_t_int-2_t_int*icomp
                   !!
                   ka=kapa*bfac
                   kb=kapb*bfac
                   !!
                   lla=(Abs(2*ka+1)-1)/2
                   llb=(Abs(2*kb+1)-1)/2
                   !!
                   lmploopcv: Do j=1,nlmp
                      ilm=lmp(j)
                      !!
                      ang(1:2)=c0
                      Do mla=1,2*lla+1
                         Do mlb=1,2*llb+1
                            ang(icomp)=ang(icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,1,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,1,kb))
                            ang(3-icomp)=ang(3-icomp)&
                                 &+basis% p_relTOpseudo(mla,ma,2,ka)&
                                 &*wgaunt(ilm,lla**2+mla,llb**2+mlb)&
                                 &*Conjg(basis% p_relTOpseudo(mlb,mb,2,kb))
                         Enddo
                      Enddo
                      !!
                      !! 
                      den=Real((ang(1)+ang(2))*rex,t_real)*cs
                      mag=Real((ang(1)-ang(2))*rex,t_real)*cs
                      !!
                      Do ms=1,mspin
                         d=(den+(mspin-1)*(3-2*ms)*mag)
                         lcd(1:nr,j,isa,ms)=lcd(1:nr,j,isa,ms)&
                              &+(rfca(1:nr,icra,icomp)*rfva(1:nr,ivrb,icomp))*d
                      Enddo
                      !!
                   Enddo lmploopcv

                   d=Real(Conjg(quant% p_chiLRchi(ka,ma,kb,mb))*rex,t_real)
                   omtmpl(1:nr)=omtmpl(1:nr)&
                        &+rfca(1:nr,icra,icomp)*rfva(1:nr,ivrb,icomp)*d
                   
                Enddo icomploopcv
                !!
             End Do mbloopcv
          Enddo ibloopcv
       End Do maloopcv
    Enddo ialoopcv
    !!
    !!
666 Continue
    !!
    !!
    texp(1:nr)=rmesh(1:nr)**(fgam+fgam)
    !!
    omtmp(1:nr)=omtmpl(1:nr)*texp(1:nr)
    !!
    Do j=1,nlmp
       ilm=lmp(j)
       l=llilm(ilm)
       !!
       If(l==1)Then
          
          !Print *,lcd(1,j,isa,1:mspin)
          
          If(Abs(Sum(lcd(1,j,isa,1:mspin))) >1.0e-12_t_real)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="(' Assumption about L=1m density is "&
                  &//"wrong, Please report case!')")
          Endif
       Endif
       !!
       Do ms=1,mspin
          lcd(1:nr,j,isa,ms)=lcd(1:nr,j,isa,ms)*texp(1:nr)
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine rel_net_dens
  !!***********************************************************************

End Module netdensity
