Module nsphintirred
  !!
  Use precisiondefs
  Use errors
  Use file_iounits
  Use constants
  Use points
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="NSPHINTIRRED"  
  !!
  Public :: gsphirred
  !!***********************************************************************
  !! integrate Ylm s upto lmax=2*nv+1
  !!
  !! cubic meshes
  !!
  !! nv  lmax  N   accuracy integrating YlmYl'm' from ylms12/ylms30
  !!                Linux PC
  !! 1   3    6     5.0e-16
  !! 2   5   18     5.0e-16
  !! 3   7   26     5.0e-16
  !! 4   9   42     4.4e-16
  !! 5  11   50     1.0e-15
  !! 6  13   78     3.2e-15
  !! 7  15   90     2.7e-15
  !! 8  17  110     2.8e-15
  !! 9  19  146     3.4e-15
  !!10  21  170     3.6e-15
  !!11  23  194     1.5e-14
  !!12  25  258     3.0e-14
  !!13  27  278     7.0e-14
  !!14  29  314     1.3e-13
  !!15  31  356     5.8e-13
  !!16  33  402     5.1e-13
  !!17  35  450     2.3e-12     
  !!18  37  494     2.4e-12
  !!19  39  542     6.1e-12
  !!20  41  602     1.6e-11
  !!21  43  668     9.7e-11
  !!
  !!----------------------------------------------------------------------
  !!
  !! hexagonal meshes
  !!
  !! nv  lmax  N   accuracy integrating YlmYl'm' from ylms12/ylms30
  !!                Linux PC
  !! 1   3     8     5.0e-16
  !! 2   5    14     5.0e-16
  !! 3   7    26     5.6e-16
  !! 4   9    44     8.3e-16
  !! 5  11    50     1.0e-15
  !! 6  13    74     3.0e-15
  !! 7  15    92     3.4e-15
  !! 8  17   116     2.7e-15
  !! 9  19   146     3.3e-15
  !!10  21   170     3.9e-15
  !!11  23   200     8.8e-15
  !!12  25   242     2.4e-14
  !!13  27   272     4.9e-14
  !!14  29   314     1.5e-13
  !!15  31   350     7.8e-14
  !!16  33   398     1.2e-12     
  !!17  35   446     2.3e-12
  !!18  37   500     5.6e-12  bad mesh distribution
  !!
  !!20  41   602     2.7e-11
  !!----------------------------------------------------------------------
  !!
  !! IMPORTANT REMARK: most of the errors come form ylms**
  !!
  !!***********************************************************************

  !! cubic
  Integer(t_int),Parameter  :: maxtypes=21
  Integer(t_int),Parameter  :: maxpoints=201
  Integer(t_int),Parameter  :: maxvars=358
  !!
  Integer(t_int) :: nvtypes(maxtypes)
  Integer(t_int) :: npoints(0:maxtypes)
  Integer(t_int) :: nvars(maxtypes)
  Integer(t_int) :: pointlist(maxpoints)
  Real(t_real)   :: weights(maxpoints)
  Real(t_real)   :: vars(maxvars)
  !! hexagonal
  Integer(t_int),Parameter  :: maxtypeshex=20
  Integer(t_int),Parameter  :: maxpointshex=267
  Integer(t_int),Parameter  :: maxvarshex=605
  !!
  Integer(t_int) :: nvtypeshex(maxtypeshex)
  Integer(t_int) :: npointshex(0:maxtypeshex)
  Integer(t_int) :: nvarshex(maxtypeshex)
  Integer(t_int) :: pointlisthex(maxpointshex)
  Real(t_real)   :: weightshex(maxpointshex)
  Real(t_real)   :: varshex(maxvarshex)
  !!-------------------------------------------------------------------
  !!cubic
  Data nvtypes /1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,20,21,18,19/
  Data npoints /0,1,3,6,9,13,17,22,28,35,43,52,61,71,83,96,110,125,145,166&
       &,183,201/
  Data nvars   /0,0,0,0,2,4,10,16,24,33,44,57,74,93,114,139,167,198,239,285&
       &,320/
  !!-------------------------------------------------------------------
  !! hexagonal
  !!               this must be updated (19)                ----
  !!                                                            |
  !!                                                            v

  Data nvtypeshex /1,2,3,4,5,6,7,8,9,11,10,13,14,15,16,17,18,19,19,12/
  Data npointshex /0,2,4,7,11,17,24,32,41,51,65,77,112,127,145,165,188,212&
       &,238,267,0/
  Data nvarshex   /0,0,2,6,11,18,28,43,61,84,114,142,228,265,307,352,405,465&
       &,532,0/
  !!-------------------------------------------------------------------
  !! hexagonal
  !! lmax=    3
  Data pointlisthex(1:2) /2,60/
  !! lmax=    5
  Data pointlisthex(3:4) /2,121/
  !! lmax=    7
  Data pointlisthex(5:7) /2,120,121/
  !! lmax=    9
  Data pointlisthex(8:11) /2,60,121,243/
  !! lmax=   11
  Data pointlisthex(12:17) /2,60,61,121,121,122/
  !! lmax=   13
  Data pointlisthex(18:24) /2,60,61,121,121,122,243/
  !! lmax=   15
  Data pointlisthex(25:32) /2,60,121,121,122,243,122,120/
  !! lmax=   17
  Data pointlisthex(33:41) /2,60,121,121,122,243,122,121,243/
  !! lmax=   19
  Data pointlisthex(42:51) /2,121,121,122,243,122,121,243,120,243/
  !! lmax=   23
  Data pointlisthex(52:65) /2,121,121,122,243,122,121,243,121,243,61,243,120&
       &,121/
  !! lmax=   21
  Data pointlisthex(66:77) /2,121,121,122,243,122,121,243,121,243,120,122/
  !! lmax=   41
  Data pointlisthex(78:112) /2,121,121,243,122,121,243,121,243,243,120,121&
       &,243,122,122,121,121,122,122,243,243,243,243,120,121,243,243,243,243&
       &,243,243,243,243,60,61/
  !! lmax=   25
  Data pointlisthex(113:127) /2,121,121,122,243,122,121,243,121,243,243&
       &,120,122,243,243/
  !! lmax=   27
  Data pointlisthex(128:145) /121,121,122,243,122,121,243,121,243,243,120&
       &,122,243,122,2,243,61,120/
  !! lmax=   29
  Data pointlisthex(146:165) /121,121,122,243,122,121,243,121,243,243,122&
       &,243,121,2,243,243,121,243,60,61/
  !! lmax=   31
  Data pointlisthex(166:188) /121,121,122,243,122,121,243,121,243,243,122&
       &,243,121,2,243,243,121,243,60,122,122,120,61/
  !! lmax=   33
  Data pointlisthex(189:212) /121,121,122,243,122,121,243,121,243,243,122&
       &,243,121,2,243,243,121,243,122,120,243,243,120,121/
  !! lmax=   35
  Data pointlisthex(213:238) /121,121,122,243,122,121,243,121,243,243,122,243&
       &,121,2,243,243,121,243,122,120,243,243,120,243,122,243/
  !! lmax=   37
  Data pointlisthex(239:267) /121,121,122,243,122,121,243,121,243,243,122&
       &,243,121,2,243,243,121,243,122,121,243,243,243,122,243,243,243,121&
       &,61/


  !! lmax=    3
  Data weightshex(1:2) /&
       & 2.0943951023931954923_t_real &
       &,1.3962634015954636615_t_real &
       &/
  !! lmax=    5
  Data weightshex(3:4) /&
       & 1.047197551196597746154214461093172000_t_real &
       &,0.872664625997164788461845384244305400_t_real &
       &/
  !! lmax=    7
  Data weightshex(5:7) /&
       & 0.628318530717958647692528676655901700_t_real &
       &,0.372336907092123643077054030610903800_t_real &
       &,0.570140888984814328461738984372946800_t_real &
       &/
  !! lmax=    9
  Data weightshex(8:11) /&
       & 0.403919055461544844945197006421651300_t_real &
       &,0.212763946909784938901173731777655800_t_real &
       &,0.377776894371067007992140858980220100_t_real &
       &,0.247859420396857063943643617576922200_t_real &
       &/
  !! lmax=   11
  Data weightshex(12:17) /&
       & 0.292342649709050204134718203721857100_t_real &
       &,0.205427259085309596180443603095616700_t_real &
       &,0.283685262546379918534898309036911900_t_real &
       &,0.279502846819009128864434847390917000_t_real &
       &,0.220909057712924189642396134183948100_t_real &
       &,0.253505610897311302933926156165062100_t_real &
       &/
  !! lmax=   13
  Data weightshex(18:24) /&
       & 0.200084871852604867815058291623112100_t_real &
       &,0.172966364486977880148235396046324800_t_real &
       &,0.148488929233863613710981897851333500_t_real &
       &,0.192595647728958036920482271257123700_t_real &
       &,0.148847699960138500268613598483544700_t_real &
       &,0.176016559044404916052727090736748600_t_real &
       &,0.167831259480287367340136569198201300_t_real &
       &/
  !! lmax=   15
  Data weightshex(25:32) /&
       & 0.167029645266690409703760069814857100_t_real &
       &,0.131539234445411406431894928075896700_t_real &
       &,0.161881005580153330458671594203946100_t_real &
       &,0.151786178658123958204112486669720900_t_real &
       &,0.120532485991007055429636392389528300_t_real &
       &,0.125075092588079846779394635662007500_t_real &
       &,0.150252125085195693076022046385383100_t_real &
       &,0.118987679272137243927075194446816100_t_real &
       &/
  !! lmax=   17
  Data weightshex(33:41) /&
       & 0.132013665846982845562457892880588000_t_real &
       &,0.117439208280340857537135756547105000_t_real &
       &,0.129525646203150100243594309339354600_t_real &
       &,0.103892386464819488681990819299337100_t_real &
       &,0.078912352232135639617278201153301700_t_real &
       &,0.119759537070657967374038141780261100_t_real &
       &,0.124422616101046574518547533657932000_t_real &
       &,0.111549115095182571145257073335607000_t_real &
       &,0.089327239588806600418579690163398030_t_real &
       &/
  !! lmax=   19
  Data weightshex(42:51) /&
       & 0.106985082438354120472693785024723600_t_real &
       &,0.105387360125647066824427192479406100_t_real &
       &,0.087169006150272923447715081948106440_t_real &
       &,0.071562847847415463281805887282165870_t_real &
       &,0.076187119441150700047156547222582600_t_real &
       &,0.101905112729155235589286634403075600_t_real &
       &,0.095297029814113153898133973552899440_t_real &
       &,0.078547126626462437832362476442884460_t_real &
       &,0.075591076806135759671968306444663660_t_real &
       &,0.091492889257786423801528520073897820_t_real &
       &/
  !! lmax=   23
  Data weightshex(52:65) /&
       & 0.076220965801480095510278409383985370_t_real &
       &,0.075245519892401026541309646405847920_t_real &
       &,0.065863579429244899136104435635097010_t_real &
       &,0.064454211578227666027000341171285500_t_real &
       &,0.062058469454058625646911295623345860_t_real &
       &,0.073246613423905079138189257377230960_t_real &
       &,0.072126608532920175960893286423648490_t_real &
       &,0.068968361507139578900967611680473850_t_real &
       &,0.053485849634931999509860732532189420_t_real &
       &,0.061469093136847779131239735491300280_t_real &
       &,0.067353257517859563736928030145004890_t_real &
       &,0.052129179426854683015946566630093330_t_real &
       &,0.062706399263518659096097135155396750_t_real &
       &,0.044438439332470442901118790905544830_t_real &
       &/
  !! lmax=   21
  Data weightshex(66:77) /&
       & 0.088970582055229273680845853506875270_t_real &
       &,0.074052281829046303029149475258662640_t_real &
       &,0.082679163887837034977971982152050200_t_real &
       &,0.083086037169446882474335529164177040_t_real &
       &,0.068180468281076059338557855762581600_t_real &
       &,0.084932067776473944968651387359057260_t_real &
       &,0.087681692650750697240393581626674740_t_real &
       &,0.078305674717291414582137245560634350_t_real &
       &,0.067065663366985864695181777046986680_t_real &
       &,0.072213972311463634260731012983014200_t_real &
       &,0.071659773389420394878434439378318370_t_real &
       &,0.043812210164436195247103084910301210_t_real &
       &/
  !! lmax=   41
  Data weightshex(78:112) /&
       & 0.024454816306724334716341349590615394_t_real &
       &,0.024332238626907405110358283125003471_t_real &
       &,0.023332009642562877691497109805908756_t_real &
       &,0.021828536754549171169045563439174862_t_real &
       &,0.024084487521917703529535930083544895_t_real &
       &,0.023958442809307477810974429152116375_t_real &
       &,0.023582824934896536845152382616607780_t_real &
       &,0.022479737624961087740505835142869919_t_real &
       &,0.022827031582713626536787748771847734_t_real &
       &,0.020486761607390921115218969843800732_t_real &
       &,0.021905268432925277948843900511049445_t_real &
       &,0.021368878599216988225307958236661392_t_real &
       &,0.020392831017611680269275870298138979_t_real &
       &,0.007929376925894832927994168879309512_t_real &
       &,0.022936047756391475965809304803951773_t_real &
       &,0.020839079725571191401265690832569955_t_real &
       &,0.018875720510925144760561882976206847_t_real &
       &,0.020256093030818038879158538957847771_t_real &
       &,0.023093803476695089691171646457401510_t_real &
       &,0.021939424710868436209865150133471583_t_real &
       &,0.019788576784432247085178321074666641_t_real &
       &,0.020782660600986244565602072100880935_t_real &
       &,0.020335844448328129726284560463853628_t_real &
       &,0.021232015972505847566122948108941733_t_real &
       &,0.015320166036976579678551891096451556_t_real &
       &,0.016760364002144769086857357383850588_t_real &
       &,0.021722580187724554698205930731715386_t_real &
       &,0.020639097262940837630767715581352645_t_real &
       &,0.022025142313511068669763228041272087_t_real &
       &,0.018949528278017761200101489995392226_t_real &
       &,0.019252019205223571040874523705353336_t_real &
       &,0.021273834667401010709655551050461113_t_real &
       &,0.022037960376123005238227286391412374_t_real &
       &,0.022033662120063138841914359481817136_t_real &
       &,0.021823026510949250184960190008299300_t_real &
       &/
  !! lmax=   25
  Data weightshex(113:127) /&
       & 0.065099871537693298896122440096561940_t_real &
       &,0.064390010805291504629039435275669103_t_real &
       &,0.057409777089982379071601221410626802_t_real &
       &,0.056687857264198554818106825532195619_t_real &
       &,0.055983578888327035115512509289703128_t_real &
       &,0.062931073817799219891526621792477624_t_real &
       &,0.062100140248520140719720564989632589_t_real &
       &,0.059798607081722321374028605968736014_t_real &
       &,0.047279169350599483865530566429604072_t_real &
       &,0.054239931507990175811149770298175269_t_real &
       &,0.046655689068506378440195696002860407_t_real &
       &,0.040462768693967576018690219245497096_t_real &
       &,0.052654628873274003583117870557208105_t_real &
       &,0.050552876441928939317141883799145784_t_real &
       &,0.028985390243199816812401899230127628_t_real &
       &/
  !! lmax=   27
  Data weightshex(128:145) /&
       & 0.055758235266961227418309670933328809_t_real &
       &,0.051201509464459328039383459759235595_t_real &
       &,0.048618997910405728954211012091075176_t_real &
       &,0.045067934099647076058089739800129358_t_real &
       &,0.054634838946090329594974526956902004_t_real &
       &,0.054084274713239271086312353997679531_t_real &
       &,0.052297895386733560568006664759371743_t_real &
       &,0.047434102524565561086960712277902933_t_real &
       &,0.048207087005714964383926575522002418_t_real &
       &,0.034799929820482404651861711073086329_t_real &
       &,0.032807200618848258712226617517159672_t_real &
       &,0.048695475404608212850718044576366844_t_real &
       &,0.041360234974370372282835207906419649_t_real &
       &,0.040112063753451000019923978669550167_t_real &
       &,0.056314688535241875569439428118754946_t_real &
       &,0.043950356018527030006684568092424241_t_real &
       &,0.052743390301893867599574460920274931_t_real &
       &,0.046726501409530766093691348193835959_t_real &
       &/
  !! lmax=   29
  Data weightshex(146:165) /&
       & 0.048648507197930574236487504163502928_t_real &
       &,0.044819373652446071073019538244481828_t_real &
       &,0.042432116347924424372019852304571630_t_real &
       &,0.042863980409616369345146701318273822_t_real &
       &,0.047801856295951124623343378299999808_t_real &
       &,0.047337904433885798180584807975135572_t_real &
       &,0.046019638906758729152842587655935009_t_real &
       &,0.039993201757767933408094016515445117_t_real &
       &,0.043014843534894250838465898628932583_t_real &
       &,0.039320636092728319328131363686285822_t_real &
       &,0.044078107729807032085810101818806648_t_real &
       &,0.030410255855360987020421955081849694_t_real &
       &,0.035966782516428920490500657769567909_t_real &
       &,0.049062399651867699818443713005895131_t_real &
       &,0.038108900666024292259310281334622667_t_real &
       &,0.042743816521529531001440429945408914_t_real &
       &,0.029020036243468403204780847256307116_t_real &
       &,0.035159582102657221822985147814288229_t_real &
       &,0.035301732128117097972962016458415060_t_real &
       &,0.011976848338289794638393464167931000_t_real &
       &/
  !! lmax=   31
  Data weightshex(166:188) /&
       & 0.040344326937278565247985288124867802_t_real &
       &,0.034338618998174443964676856012616073_t_real &
       &,0.036376270922825412974562246758644275_t_real &
       &,0.032126662256004021173525651506019730_t_real &
       &,0.031301627716010413598936288260089792_t_real &
       &,0.038865870726537730798136813862607229_t_real &
       &,0.037387632623045821374480339090965575_t_real &
       &,0.035382722196997570413366315225384868_t_real &
       &,0.034273692272835403785867616934727194_t_real &
       &,0.035615211888854582269931475051903828_t_real &
       &,0.037517020868545638165742537887793179_t_real &
       &,0.036695692151595467250057887177967196_t_real &
       &,0.035779931990084411386476495439753357_t_real &
       &,0.040737988161224186283885884593856403_t_real &
       &,0.035872068540643013413088346627390459_t_real &
       &,0.037885470527859726964094106163654258_t_real &
       &,0.036942340070693768601719093533144882_t_real &
       &,0.038050175526509594173069621597294973_t_real &
       &,0.038186081933482955784063160628437181_t_real &
       &,0.039486162496778861319273087276179230_t_real &
       &,0.033358208238367991606380737362635824_t_real &
       &,0.038314699996097458810462014441425495_t_real &
       &,0.014987665606462752371174741723302471_t_real &
       &/
  !! lmax=   33
  Data weightshex(189:212) /&
       & 0.037441832393345537223428222651250099_t_real &
       &,0.035025884706969513368939925912408881_t_real &
       &,0.029897964862973018666307946733263948_t_real &
       &,0.033243995917188589831120449251822707_t_real &
       &,0.033167200499100238240677557045730462_t_real &
       &,0.036610235047660997475739259437442788_t_real &
       &,0.035769731939359711210285741552119963_t_real &
       &,0.032187053551897782941907299895288001_t_real &
       &,0.033868306440444394424830417505319100_t_real &
       &,0.031115121481105673656630842011280256_t_real &
       &,0.034495480643727235400892111677610631_t_real &
       &,0.027910812127335008453663386154823962_t_real &
       &,0.027669782001983006682810235660832927_t_real &
       &,0.037703396174610179716258181771656184_t_real &
       &,0.031746822259863537306549609914111185_t_real &
       &,0.032374475651130878924094259259881973_t_real &
       &,0.030226632999923376306881615424307068_t_real &
       &,0.030186667386369691523588636446728910_t_real &
       &,0.036904414121129563041940570241018671_t_real &
       &,0.031152301934195456957535455607198347_t_real &
       &,0.030169027290665371055161609892987783_t_real &
       &,0.029090049614211813982423174853067820_t_real &
       &,0.026529625884041371088162949554902846_t_real &
       &,0.018655222971866278069585360605682901_t_real &
       &/
  !! lmax=   35
  Data weightshex(213:238) /&
       & 0.034035975744596101398363465821781304_t_real &
       &,0.032215414122092493109727470408549026_t_real &
       &,0.031017393937132256584368406719569787_t_real &
       &,0.030643655512530117748528066845483911_t_real &
       &,0.021683873624574845163955711376967197_t_real &
       &,0.033395837990032811961557728885937884_t_real &
       &,0.032748692469017881827392076481128037_t_real &
       &,0.030139888533973124429396084173370974_t_real &
       &,0.031310738290084100145492804776874052_t_real &
       &,0.029163143410220729039508041121350977_t_real &
       &,0.031725737719177921959462273495132825_t_real &
       &,0.027408605115090668501816982611339973_t_real &
       &,0.027093843561806067813061056080755631_t_real &
       &,0.034240699020856441299866425010692368_t_real &
       &,0.029104923421458984487139013626146289_t_real &
       &,0.023895712876829313636810927327169788_t_real &
       &,0.027055379743261927004400839908909716_t_real &
       &,0.024496860500457749623892055282969220_t_real &
       &,0.033618487208797943304743178963874584_t_real &
       &,0.024417810917067070431909822345486784_t_real &
       &,0.025369947197363172166126475092814759_t_real &
       &,0.026407776853017528840141233876664330_t_real &
       &,0.032844968022730812304503096924806629_t_real &
       &,0.023344838173254566363994573526394851_t_real &
       &,0.025479759237031648000729862024479918_t_real &
       &,0.024488305012765178854853279329212062_t_real &
       &/
  !! lmax=   37
  Data weightshex(239:267) /&
       & 0.030448409047882179330726609029277245_t_real &
       &,0.028840328155220280096042916831532002_t_real &
       &,0.012120558397900010302453136213741410_t_real &
       &,0.026319920496797261156817068497728638_t_real &
       &,0.020672271486387655915417531775014737_t_real &
       &,0.029897042136449382401221802246439008_t_real &
       &,0.029327316620459484577949691580420267_t_real &
       &,0.027482869388630253134402568929311940_t_real &
       &,0.028062193009532733538962884272146828_t_real &
       &,0.026574994885641793452734565236630666_t_real &
       &,0.028463001611494699183757588650343162_t_real &
       &,0.025387897126919392201216061247054317_t_real &
       &,0.024135366791224385478667004818774093_t_real &
       &,0.030623232680901862141936751601785690_t_real &
       &,0.026560000751966278750907682977159133_t_real &
       &,0.025258979743190799521479912024728593_t_real &
       &,0.025406972573570616195617538257822596_t_real &
       &,0.024360595879522121299766550161708514_t_real &
       &,0.030089480649535585189120495528232018_t_real &
       &,0.025234479245746015648078207434278460_t_real &
       &,0.025747508380244813679578674429556966_t_real &
       &,0.020490234643894566965672147719073851_t_real &
       &,0.025974300687804859735324374749198177_t_real &
       &,0.029020482477815503738402767094899583_t_real &
       &,0.013898707163142403178172820682277046_t_real &
       &,0.029146927215097048540584857926295601_t_real &
       &,0.023244612305748751062369610644382486_t_real &
       &,0.024791214888573395079002294671542613_t_real &
       &,0.009565648825519050895814806763211235_t_real &
       &/

  !! lmax=    5
  Data varshex(1:2) /&
       & 0.894427190999915878563669467492511000_t_real &
       &,0.447213595499957939281834733746254200_t_real &
       &/
  !! lmax=    7
  Data varshex(3:6) /&
       & 0.952305702455787987605161224888739500_t_real &
       &,0.305145619451087969887872182748875600_t_real &
       &,0.755928946018454454429033072468360500_t_real &
       &,0.654653670707977143798292456246857900_t_real &
       &/
  !! lmax=    9
  Data varshex(7:11) /&
       & 0.632455532033675866399778708886544500_t_real &
       &,0.774596669241483377035853079956479300_t_real &
       &,0.893192777063704836638788011576299700_t_real &
       &,0.301820396743022919235197584560576400_t_real &
       &,0.333333333333333333333333333333332500_t_real &
       &/
  !! lmax=   11
  Data varshex(12:18) /&
       & 0.550209452804018763170522104571064600_t_real &
       &,0.835026681037858962849463146664317500_t_real &
       &,0.914116775359537338569932197785147400_t_real &
       &,0.405451009378791841748740817145931200_t_real &
       &,0.738548945875996396405443960751454500_t_real &
       &,0.426401432711220868596875464867676600_t_real &
       &,0.522232967867093514531873769838091500_t_real &
       &/
  !! lmax=   13
  Data varshex(19:28) /&
       & 0.462322663628821302700547935946242200_t_real &
       &,0.886711765285175266109143149949028300_t_real &
       &,0.804620120703346951191845779197392400_t_real &
       &,0.593789913487364134230876602273912900_t_real &
       &,0.643604405706694314157523318144259700_t_real &
       &,0.371585176886389074853204863632187800_t_real &
       &,0.669102253226847062145040793178365300_t_real &
       &,0.912934053215830706474357110861507700_t_real &
       &,0.261198338006437104488288910391131800_t_real &
       &,0.313571112670778112475981573135831500_t_real &
       &/
  !! lmax=   15
  Data varshex(29:43) /&
       & 0.425317238995104122470361478383073700_t_real &
       &,0.905044333838724986261690907227550700_t_real &
       &,0.772167209832495512206565386259367300_t_real &
       &,0.635419389112024851167441794343039600_t_real &
       &,0.783983550434694101029391518491145700_t_real &
       &,0.452633113883709186275808039758592300_t_real &
       &,0.424844744422887501853702467050005700_t_real &
       &,0.925962237804143411443107547329197700_t_real &
       &,0.189966214723641323334752350083993200_t_real &
       &,0.326353751969108460863802600231165900_t_real &
       &,0.596595751680601487860129593444573300_t_real &
       &,0.344444717830182396613190395685925700_t_real &
       &,0.724866432824381871283209241810123200_t_real &
       &,0.927749760224921677804069357944615000_t_real &
       &,0.373202870303271294047646811166708000_t_real &
       &/
  !! lmax=   17
  Data varshex(44:61) /&
       & 0.381100578389705478446061405676836400_t_real &
       &,0.924533584652841018510154606544763800_t_real &
       &,0.945291174198688966726848655944062100_t_real &
       &,0.326227828337902330346578071586216900_t_real &
       &,0.756953945744587214503624169202407600_t_real &
       &,0.437027564339786804247662193350794500_t_real &
       &,0.485826751042935078948267919886930700_t_real &
       &,0.925841368460474796727226750194463600_t_real &
       &,0.336134894006537489763851562944282500_t_real &
       &,0.172716801378583755508773081360469100_t_real &
       &,0.546185285901798278818686123052566000_t_real &
       &,0.315340221842815946262481935210858000_t_real &
       &,0.776042639261847840319634429107351400_t_real &
       &,0.698165858168896703185080866138463100_t_real &
       &,0.715936054747411092127848007221313700_t_real &
       &,0.830589540486509654725335108064803300_t_real &
       &,0.211485942302199806005289112582218400_t_real &
       &,0.515164742041766047075478772769653300_t_real &
       &/
  !! lmax=   19
  Data varshex(62:84) /&
       & 0.344692442781230359033937143942716700_t_real &
       &,0.938715675745061921648785053574653600_t_real &
       &,0.916041917651312049070175474248661500_t_real &
       &,0.401082541511977101145857212373697000_t_real &
       &,0.693048222691800278564890198730161400_t_real &
       &,0.400131577932502593993353711963181400_t_real &
       &,0.599648965116243999169607668001786600_t_real &
       &,0.980705888822935778116844483106100500_t_real &
       &,0.140259395545143052313390928997423600_t_real &
       &,0.136173644988032227247392594582943600_t_real &
       &,0.498390703165961419102528595336651800_t_real &
       &,0.287746006634474695074564294720111600_t_real &
       &,0.817807399491867754440335879997720700_t_real &
       &,0.643196045171096778267994457964466600_t_real &
       &,0.765701539423984985357676454773480800_t_real &
       &,0.792647950164444818459827054429654100_t_real &
       &,0.182715522184949055964844606153479800_t_real &
       &,0.581656483719372969949858310669001100_t_real &
       &,0.915045758536754987387256269702506200_t_real &
       &,0.403350046217791683450503635785301900_t_real &
       &,0.883659927642474322928832739582848100_t_real &
       &,0.342684033561659680250316850764371400_t_real &
       &,0.318924419605661971127189133615715200_t_real &
       &/
  !! lmax=   23
  Data varshex(85:114) /&
       & 0.292460854002049054773533344624186300_t_real &
       &,0.956277495749216168587523989032615600_t_real &
       &,0.775822753176865180728616801223590800_t_real &
       &,0.630950913822199744124365997543917400_t_real &
       &,0.745043170033307855993966504599376000_t_real &
       &,0.430150874776622410327815552891268600_t_real &
       &,0.509785150544448504165955462533707300_t_real &
       &,0.888409276985765298799149296048498700_t_real &
       &,0.377314608743070050747940565782789300_t_real &
       &,0.261462507053485046749174311502987300_t_real &
       &,0.426306766121183824967073641236790300_t_real &
       &,0.246128326177424314384341252395027900_t_real &
       &,0.870450106676079223481692740307118400_t_real &
       &,0.559554050540603374855859789000986800_t_real &
       &,0.828793861297007848451597849563048500_t_real &
       &,0.668980085006507877636163197216400000_t_real &
       &,0.232849365249277210323312929116795300_t_real &
       &,0.705866006383431237554786833035018400_t_real &
       &,0.920149398450329695482040349642833100_t_real &
       &,0.391567471237711980858147796005791800_t_real &
       &,0.850462872576063672383347714159833900_t_real &
       &,0.212904106420671298677422204229505100_t_real &
       &,0.481024681111983645189587360871027200_t_real &
       &,0.960823342140211891339804048529282300_t_real &
       &,0.159636782912492186430572372312769300_t_real &
       &,0.226571407595625716484570787341244400_t_real &
       &,0.963016686427654240143114408665338200_t_real &
       &,0.269441759313401651534554539111997400_t_real &
       &,0.996275179828736593162782087335155100_t_real &
       &,0.086230888069290826010006067594629440_t_real &
       &/
  !! lmax=   21
  Data varshex(115:142) /&
       & 0.973018307425451361797032023565096600_t_real &
       &,0.230727920752798843588369984544728900_t_real &
       &,0.597503977911433457408985261544088100_t_real &
       &,0.801865946639469931184052079206897500_t_real &
       &,0.780899746739224511963462662441758300_t_real &
       &,0.450852678990001856313924996927367700_t_real &
       &,0.432351069606810797036021213276652200_t_real &
       &,0.925660805762421472863503372824535900_t_real &
       &,0.327700315396095115272259777104460600_t_real &
       &,0.189115245193412190262133092909121900_t_real &
       &,0.457612515427541253906031222533149100_t_real &
       &,0.264202708966632716038146578137727500_t_real &
       &,0.848992175640489098208043923523877400_t_real &
       &,0.315240811665260223547809792383413400_t_real &
       &,0.949011712604553650333368070374425500_t_real &
       &,0.709498495036679469092403589060800700_t_real &
       &,0.248144728654929111724370688645246200_t_real &
       &,0.659572648903408154169248014656128300_t_real &
       &,0.809733560256199762813963821508673000_t_real &
       &,0.586797717612142138353471642923299800_t_real &
       &,0.891273077638990083991615483503322000_t_real &
       &,0.181858110904124976483888263135051900_t_real &
       &,0.415403332406356285668195350706528800_t_real &
       &,0.987709235194678636700657022308742300_t_real &
       &,0.156302484660810815625078458002627300_t_real &
       &,0.862138179264489429417626295064095100_t_real &
       &,0.497755709877006805577477423516662900_t_real &
       &,0.094641497977093105915187918381763570_t_real &
       &/
  !! lmax=   41
  Data varshex(143:170) /&
       & 0.167238771020876170706776680019786692_t_real &
       &,0.985916423165486653886382865439990388_t_real &
       &,0.482553842191510805400706132090447597_t_real &
       &,0.875866307941006762700246041580387954_t_real &
       &,0.681672857940088290688222931357341883_t_real &
       &,0.132393547934224812437848838290775496_t_real &
       &,0.719579087531857282729158819410963787_t_real &
       &,0.248462508922096291787713831084040158_t_real &
       &,0.143449896409702087313752780501345369_t_real &
       &,0.957960598814055066248541624181221432_t_real &
       &,0.329663247545277798919094942723698039_t_real &
       &,0.944098587658037380291246569732570612_t_real &
       &,0.406135360898695350533620554091204543_t_real &
       &,0.140714097423657743906309098808483486_t_real &
       &,0.902913955708921267769102091516692457_t_real &
       &,0.621446344536751468508112659183862999_t_real &
       &,0.783456725583429520558083385407081744_t_real &
       &,0.552025695364419731087025076199354006_t_real &
       &,0.136734034211243146442105910106375430_t_real &
       &,0.822539625517061307392578305052630246_t_real &
       &,0.789515152875738819223840219533340359_t_real &
       &,0.131826531396453712375116424393179444_t_real &
       &,0.599406030166179229875033317742610179_t_real &
       &,0.984008614686028876193173845858519092_t_real &
       &,0.178120875316966585683040743935781148_t_real &
       &,0.741622351694417648298094882333888809_t_real &
       &,0.670817626085690670788212627970627972_t_real &
       &,0.729706213816097004119003960694237317_t_real /
  Data varshex(171:200) /&
       & 0.257735323424501824583928153838935836_t_real &
       &,0.633325622865081910444413915846133393_t_real &
       &,0.781536903848907755110275774249020827_t_real &
       &,0.451220541818793560653566078067113391_t_real &
       &,0.430813289677810990894867087251815559_t_real &
       &,0.475594321400551262803916534906764320_t_real &
       &,0.274584509485665665119920257667072403_t_real &
       &,0.835711306972728190271379117958550764_t_real &
       &,0.970842894023095925072115387665915640_t_real &
       &,0.239716655918731968544611188764059684_t_real &
       &,0.835199287091317571909027301856972218_t_real &
       &,0.549947407341970191321163758627788524_t_real &
       &,0.661461597761523921770313977737341420_t_real &
       &,0.381895031526215795252997760580242325_t_real &
       &,0.645464746971019770011453493537621367_t_real &
       &,0.833795088877641801251330244576985133_t_real &
       &,0.481391819012494433210496535085550399_t_real &
       &,0.270273317867981952508806837431439495_t_real &
       &,0.612218048865156089784428276607983963_t_real &
       &,0.265596468077342309256201748158026864_t_real &
       &,0.744746652754198444748171348632657510_t_real &
       &,0.955441271816102165660448291567240785_t_real &
       &,0.181358418635286477748827566580366873_t_real &
       &,0.232897187833037932366384639337393770_t_real &
       &,0.839759963535678332040653929863981331_t_real &
       &,0.376520750708395308951465252521193021_t_real &
       &,0.391197300512851994778223551206126512_t_real &
       &,0.829609043904260108981611166109705597_t_real &
       &,0.244398515994591301048888005801281766_t_real &
       &,0.502014142880358424375740209272875428_t_real /
  Data varshex(201:228) /&
       & 0.935649492325393652681180756945633411_t_real &
       &,0.352930627051879877272715443219741271_t_real &
       &,0.915966395873053727981882630666034271_t_real &
       &,0.401254983310273945457464823282490275_t_real &
       &,0.937301942648005322887245277907935965_t_real &
       &,0.101622070648028655727056802381525148_t_real &
       &,0.333373698820831415021646255401173598_t_real &
       &,0.988491846287157400955303582387018668_t_real &
       &,0.089342585419164330090772517721266778_t_real &
       &,0.122072815378470363334604307621048043_t_real &
       &,0.897270672191654270256485237595422725_t_real &
       &,0.235351577505365077067061186857167485_t_real &
       &,0.373517035475589840307227313226426333_t_real &
       &,0.907622314199267274856817266860374006_t_real &
       &,0.332917599502251705679975560697348028_t_real &
       &,0.255710005101921830513331797165090302_t_real &
       &,0.766973247289971051262051099419422721_t_real &
       &,0.363938560943721612289694500673210021_t_real &
       &,0.528489131202988831480817004908517047_t_real &
       &,0.876395482620504835440556293009449343_t_real &
       &,0.107837046724319657815555187810804557_t_real &
       &,0.469363536500386133153208882972899302_t_real &
       &,0.956387086917881804468238135114944867_t_real &
       &,0.267179900581067107824695434907968980_t_real &
       &,0.118062020574861799035816659701532296_t_real &
       &,0.896045272757978341871763837801300511_t_real &
       &,0.423429218740289449724607211590003564_t_real &
       &,0.133456232095276488332497003229139016_t_real &
       &/

  !! lmax=   25
  Data varshex(229:265) /&
       & 0.270851737734663044200332839073552673_t_real &
       &,0.962621076107371184955234550279669034_t_real &
       &,0.731709192837792112258436066947408025_t_real &
       &,0.681616943096829684616232675832397290_t_real &
       &,0.707591173426340267369423647307567136_t_real &
       &,0.408527954453900722272164119115846504_t_real &
       &,0.576558446056292256675199420497928058_t_real &
       &,0.860302705370372238726631123393176356_t_real &
       &,0.363937894598084060546877502476558963_t_real &
       &,0.356971236947645900914318923922202345_t_real &
       &,0.396495618864666964127513972599848083_t_real &
       &,0.228916852284022731186174088879335461_t_real &
       &,0.889037850128721835354988235303605462_t_real &
       &,0.521606553312549989423972391797861586_t_real &
       &,0.853186148235777410675607995818331271_t_real &
       &,0.627872257100425807965347086524326026_t_real &
       &,0.218373572761165270095344411428621031_t_real &
       &,0.747053821008326582247573010758113848_t_real &
       &,0.882751744251140467556765975270967240_t_real &
       &,0.469839715245070669927772777739645984_t_real &
       &,0.810176917164076605483097802625085882_t_real &
       &,0.201828868115872919547309772951967700_t_real &
       &,0.550343956893848821543400818008803282_t_real &
       &,0.934401249789086308017399638373587765_t_real &
       &,0.150986268750922976084388261885077126_t_real &
       &,0.322641365979732355248679984756422474_t_real &
       &,0.993464560115771458450543883500420501_t_real &
       &,0.114140999618790433678610288977750984_t_real &
       &,0.859619258349240137853029111050692486_t_real &
       &,0.496301410208520248586305576774306856_t_real &
       &,0.121406922785055600513013633067638894_t_real &
       &,0.945890024667334018825975205909949743_t_real &
       &,0.301140830432632256261276620030424355_t_real &
       &,0.120856367151982802474827456949371229_t_real &
       &,0.981549436065844901614665858648532488_t_real &
       &,0.072442775152006947809147719031933552_t_real &
       &,0.176954086946579497894991819632055435_t_real &
       &/
  !! lmax=   27
  Data varshex(266:295) /&
       & 0.252304726709864151719671272668572622_t_real &
       &,0.967647831021111090519221672000507894_t_real &
       &,0.692639845781897114399257161830667490_t_real &
       &,0.721283608600132949216821691854785473_t_real &
       &,0.776636160384887349268064177528052215_t_real &
       &,0.448391096260612080293047957617664239_t_real &
       &,0.442472258087245561520825772586930588_t_real &
       &,0.868809640049766877258405410053851578_t_real &
       &,0.258611510687969038655203559420939020_t_real &
       &,0.422243882011665137938867611647624943_t_real &
       &,0.370449289613240548170383998137294231_t_real &
       &,0.213878997079310066202660466664848073_t_real &
       &,0.903893300358728128947700369259714295_t_real &
       &,0.488500888817208376993270274533503127_t_real &
       &,0.872563396908670137947195329465671233_t_real &
       &,0.590830055460994710768214253483727718_t_real &
       &,0.205208542661066533063414183321824861_t_real &
       &,0.780262327414875997026504233056594873_t_real &
       &,0.848657833577903353272585634282282138_t_real &
       &,0.528942228893533982433458471308742563_t_real &
       &,0.770823650887807756006876350302331705_t_real &
       &,0.195545226874632246872845679571405489_t_real &
       &,0.606294452785558373457932741207936468_t_real &
       &,0.930529949656588071514174789609887291_t_real &
       &,0.093504306826319725784850070730739413_t_real &
       &,0.354077614933558215045931444261533842_t_real &
       &,0.996305293115537819988215918821137036_t_real &
       &,0.085882261905251818971607660074599268_t_real &
       &,0.671143565757372339410557725997375427_t_real &
       &,0.387484918354904220643982701084908734_t_real /
  Data varshex(296:307) /&
       & 0.631998221666780357423565895325531104_t_real &
       &,0.977012017108340354431675824713546808_t_real &
       &,0.115183637593728685277214779840085253_t_real &
       &,0.179388539367955796122148785796768159_t_real &
       &,0.842310598210294814114966612357225305_t_real &
       &,0.486308250617988420638366571244897141_t_real &
       &,0.232415880532047679577710166617516313_t_real &
       &,0.922291949089365035265779653541996771_t_real &
       &,0.321707388575821273425919322240938675_t_real &
       &,0.214200646079010774555201659832936420_t_real &
       &,0.955890807057383510412823063056753785_t_real &
       &,0.293722258235878356190248019996195575_t_real &
       &/
  !! lmax=   29
  Data varshex(308:337) /&
       & 0.235831457563452459817255412824779639_t_real &
       &,0.971793971798290957882150864153834943_t_real &
       &,0.654443425666958771060591501534028607_t_real &
       &,0.756110972411653215271910416399828938_t_real &
       &,0.821846138668068127017931201722722746_t_real &
       &,0.474493089392463637197953067391021003_t_real &
       &,0.315317669145232086707041417829659482_t_real &
       &,0.794043899752784368683049201073752688_t_real &
       &,0.340105125022960763422003153122228540_t_real &
       &,0.503808286155067477741510576318204338_t_real &
       &,0.347280855099748199698644075659227485_t_real &
       &,0.200502695176243042575958548458305096_t_real &
       &,0.916075693874829461291356152180715562_t_real &
       &,0.458459153411107118519990828605441275_t_real &
       &,0.888715480147370308049197990625322040_t_real &
       &,0.557075883920437152957047126381668563_t_real &
       &,0.193401405017891996955951966100917386_t_real &
       &,0.807627609787684840881812938758775062_t_real &
       &,0.811092206067344176129024104407157445_t_real &
       &,0.584918313319739452351311525452302783_t_real &
       &,0.734369992703816603549733200183178148_t_real &
       &,0.182498171272233450582879606019626644_t_real &
       &,0.653754641512002493378112994582303215_t_real &
       &,0.871404443517397746389470276404293478_t_real &
       &,0.154443543891407525357776974209401187_t_real &
       &,0.465619466483519391512857349316598312_t_real &
       &,0.637730564018396520519925936162739310_t_real &
       &,0.368193912806473097823682348320959133_t_real &
       &,0.676559657597936549412961917291566366_t_real &
       &,0.958402667211238062749866890926157535_t_real /
  Data varshex(338:352) /&
       & 0.143871817310696503854888685075571163_t_real &
       &,0.246506039816679614244274818333098882_t_real &
       &,0.930467168830699827895045735918789482_t_real &
       &,0.366375282638146787238126587932772775_t_real &
       &,0.904006961040739895737021913522754983_t_real &
       &,0.295386826926504347651033044259474187_t_real &
       &,0.309059924396835291808020412016676262_t_real &
       &,0.910421263284967773205612819819354342_t_real &
       &,0.397458081263450829196674427573948130_t_real &
       &,0.114717901815625567325803927522147328_t_real &
       &,0.987654203348619379402779014820385644_t_real &
       &,0.156649847136229278382549152062321402_t_real &
       &,0.973905289696162572989873603249764558_t_real &
       &,0.211081002058988836786906951449332692_t_real &
       &,0.083386433378618642552820683287042803_t_real &
       &/
  !! lmax=   31
  Data varshex(353:382) /&
       & 0.215089730546948525235379843018046636_t_real &
       &,0.976594290282940922369738446569770861_t_real &
       &,0.597389603805396284958222183398125487_t_real &
       &,0.801951158902605275419345486816207618_t_real &
       &,0.815459797020366963917691759873226046_t_real &
       &,0.470805933323026452533055001716436827_t_real &
       &,0.336700300848300224138441495589580126_t_real &
       &,0.734120197882312122738813591056433370_t_real &
       &,0.284823663888727723018813441676290090_t_real &
       &,0.616395178071856716200477166917259534_t_real &
       &,0.738248308608094743102999504861569310_t_real &
       &,0.426227859703669415425398627119428028_t_real &
       &,0.522799432334921732339249174927529544_t_real &
       &,0.418867272615691842197099970292381095_t_real &
       &,0.908047470087049849357456156693814756_t_real &
       &,0.510379928011407179477447441857111773_t_real &
       &,0.177255582315147151706296092873193852_t_real &
       &,0.841482493948144424163394496161479405_t_real &
       &,0.766280281123934616916474968343491424_t_real &
       &,0.642506444139374942060851339657433523_t_real &
       &,0.677453460469226772414991419946668155_t_real &
       &,0.148812107739283378517685084932935172_t_real &
       &,0.720355305032496942452102005342139493_t_real &
       &,0.830933750700287458828690616980906809_t_real &
       &,0.143924301410587973147364883462655785_t_real &
       &,0.537433621399542471709179950067151971_t_real &
       &,0.586494138281240824676994731209170187_t_real &
       &,0.338612548614811987132759459700002466_t_real &
       &,0.735775895013098641574903828657904989_t_real &
       &,0.932982944452667532722218775286300665_t_real /
  Data varshex(383:405) /&
       & 0.155608874153796817943328403090504908_t_real &
       &,0.324543839326859682999449451338294385_t_real &
       &,0.900313723087422447939128784515918785_t_real &
       &,0.435241542158447935803368397803349780_t_real &
       &,0.853483477761272034121514521100659895_t_real &
       &,0.300364595922014319535851389866666697_t_real &
       &,0.425848638256751179173049776530742369_t_real &
       &,0.917701453913468065178547361529080876_t_real &
       &,0.330985209290634803800965549781487038_t_real &
       &,0.219710793353311446178937000868967442_t_real &
       &,0.976791409129333581740842077051721200_t_real &
       &,0.214192770772336889119695645561055305_t_real &
       &,0.978681825146831872638701682485668933_t_real &
       &,0.174530913588167907086893550561646279_t_real &
       &,0.108262852952180563833093128707558725_t_real &
       &,0.317368744062862608104535458062799378_t_real &
       &,0.183232929817067170423423821930494916_t_real &
       &,0.930431498672854984465208952513218815_t_real &
       &,0.857934437532019094669889166674425021_t_real &
       &,0.495328678456161394036532644489091543_t_real &
       &,0.136374488800107702055946151322440214_t_real &
       &,0.938373670971619011915253801193853472_t_real &
       &,0.345622414821793210256368874337391914_t_real &
       &/
  !! lmax=   33
  Data varshex(406:435) /&
       & 0.207156554805443007948602880744442703_t_real &
       &,0.978307805243901489854575605892483804_t_real &
       &,0.584997754886739657049547086176096365_t_real &
       &,0.811034910948643484902726132286180594_t_real &
       &,0.856576774508251667270507945580293242_t_real &
       &,0.494544831343920468545234225005040098_t_real &
       &,0.147314762206823778905977407943982620_t_real &
       &,0.725166120488861000235106028124354151_t_real &
       &,0.312165504263740393158487199520721453_t_real &
       &,0.613748153270459469380864379928319589_t_real &
       &,0.766082852390802186492255128469848044_t_real &
       &,0.442298141049385982059046906083398740_t_real &
       &,0.466357607096774879595211932978800292_t_real &
       &,0.405253496320953381512429227287791271_t_real &
       &,0.914204355557138561631943946721033497_t_real &
       &,0.495565265114383424387569761131424433_t_real &
       &,0.171937658812301299105165499586467896_t_real &
       &,0.851382704483862263273963629705163965_t_real &
       &,0.737478344220123012892360125152280142_t_real &
       &,0.675370780983561468957730082192058540_t_real &
       &,0.662346117191864782337792158408614055_t_real &
       &,0.164330204287510554790346566322993922_t_real &
       &,0.730953627119864267725620608765456216_t_real &
       &,0.800045932060032458864316156262643175_t_real &
       &,0.148240451033722512697616690608866374_t_real &
       &,0.581335768099222022236219150962201666_t_real &
       &,0.573406158251400893570757976366035315_t_real &
       &,0.331056199821435456662593332908476985_t_real &
       &,0.749404543780700060180382162415739877_t_real &
       &,0.906528182104700968891338378228530519_t_real /
  Data varshex(436:465) /&
       & 0.117190299153800172353294688745128346_t_real &
       &,0.405552818796995837908473737828765812_t_real &
       &,0.859366632108829432345301381621432667_t_real &
       &,0.511359943306012003584338038010716859_t_real &
       &,0.846698124065696444322535542043171419_t_real &
       &,0.277364129151819529842647012935417859_t_real &
       &,0.454061038367622208530888680895237768_t_real &
       &,0.869057605675964267321090185928949290_t_real &
       &,0.391704351444550573686451109027963805_t_real &
       &,0.302169785180722877306272554854150256_t_real &
       &,0.956948831404060926002206830918355179_t_real &
       &,0.290256669302192223125519241429774488_t_real &
       &,0.981922064250219138097142981530969810_t_real &
       &,0.110259257041673346777878623356291075_t_real &
       &,0.153856933464848198129004845831577025_t_real &
       &,0.306301401365092083841545714251585859_t_real &
       &,0.176843196531295519219766185798832629_t_real &
       &,0.935364065678372291486464305113237791_t_real &
       &,0.978954700557774296100106411884414142_t_real &
       &,0.204077667214809328523838676969004723_t_real &
       &,0.938834513425439818770616371847850462_t_real &
       &,0.318082897602995118562869818557929317_t_real &
       &,0.131958427747909238811696507662735861_t_real &
       &,0.934522287136588761164091912073730729_t_real &
       &,0.221758904990734178949693191063268878_t_real &
       &,0.278372202100550405726401040972464633_t_real &
       &,0.909496739800720204403281559538729847_t_real &
       &,0.415711053848536782983295423210828756_t_real &
       &,0.999227230397810771553864954109323056_t_real &
       &,0.039305750616422391302894331728412338_t_real &
       &/
  !! lmax=   35
  Data varshex(466:500) /&
       & 0.197561587974592078878379616358337734_t_real &
       &,0.980290476826617710506650748319548320_t_real &
       &,0.561640018731595813911679405747714775_t_real &
       &,0.827381707169775843405118445744314970_t_real &
       &,0.746854907677937092740455577587856650_t_real &
       &,0.431196881993450078704618527513870844_t_real &
       &,0.506238082167379767659521457134316591_t_real &
       &,0.701189802730161419224274609163327084_t_real &
       &,0.303031709903646668849998380985898324_t_real &
       &,0.645371709435817648868297087862510389_t_real &
       &,0.855417028410322159368268934463211073_t_real &
       &,0.493875251622089249074382521168739856_t_real &
       &,0.156041479552304047627242180975017587_t_real &
       &,0.387400089338796895169369559966684723_t_real &
       &,0.921911693591252905853186629412774478_t_real &
       &,0.474817996337129261940656357076465960_t_real &
       &,0.164681982978596106583110720321014438_t_real &
       &,0.864539018689515993909044327244063974_t_real &
       &,0.712416487278297257908511388143664223_t_real &
       &,0.701756901393959731167301385310912441_t_real &
       &,0.637839525970223974089920408844359068_t_real &
       &,0.158410863557796204695651953182245352_t_real &
       &,0.753702021635177286768810860246217778_t_real &
       &,0.775177918749571818215214020262681158_t_real &
       &,0.146283932200478162514910640598418624_t_real &
       &,0.614573189671537934019984529694748142_t_real &
       &,0.551482676606853065531954494810010603_t_real &
       &,0.318398671792381948793474728123892296_t_real &
       &,0.771031220641154166050231707224244236_t_real &
       &,0.885275252971980956950399874930207571_t_real &
       &,0.122706516935637221911596969517013855_t_real &
       &,0.448587602567123222868175265098554874_t_real &
       &,0.835760126634677414681050797746010976_t_real &
       &,0.549094719267621414147296533105398087_t_real &
       &,0.823888613330637135009116912500864409_t_real /
  Data varshex(501:532) /&
       & 0.274557118233118018895881119166940883_t_real &
       &,0.495808371905563473266301906605319858_t_real &
       &,0.872268844742656454441232315192295786_t_real &
       &,0.339971573528408690345675624773416082_t_real &
       &,0.351520115617768441915542346697260585_t_real &
       &,0.936301427601114645685492206746983143_t_real &
       &,0.351197432610368992069195118023643829_t_real &
       &,0.972666495858480093188577884467153098_t_real &
       &,0.087713869095440390869656101600641657_t_real &
       &,0.215002709291518637904244900849066495_t_real &
       &,0.292544282552339104652494039021982404_t_real &
       &,0.168900520281478921882104322361386696_t_real &
       &,0.941217539676446376774373079104133127_t_real &
       &,0.969294329480367922678792315406958890_t_real &
       &,0.245903442101170987649053180666514960_t_real &
       &,0.955686625216705328091703026051555621_t_real &
       &,0.255971439534200568941480505771954044_t_real &
       &,0.145401844983802422614763027117464914_t_real &
       &,0.928004056037872400433225683979620250_t_real &
       &,0.206293772263402791714023915614099437_t_real &
       &,0.310244019285775574462412295770887970_t_real &
       &,0.910552376829120198600038710943299462_t_real &
       &,0.413393721590979039463433489481463119_t_real &
       &,0.904139035418247037351304067988850576_t_real &
       &,0.380971526957121854571255511455760638_t_real &
       &,0.193373473571016987656239082856212289_t_real &
       &,0.813730869760306346925667216384115450_t_real &
       &,0.469807736704021173202933958288936120_t_real &
       &,0.342232029670193882905072479400418317_t_real &
       &,0.993791764632391524584493490090631164_t_real &
       &,0.083519674026819324829722690465413838_t_real &
       &,0.073500970056803791513442736575836271_t_real &
       &/
  !! lmax=   37
  Data varshex(533:562) /&
       & 0.186945618551577026104971543108852854_t_real &
       &,0.982370264057482686412455603928075825_t_real &
       &,0.534111809164531733789609677406388306_t_real &
       &,0.845413848544599389346524587146763077_t_real &
       &,0.693041565469688738577422021112573120_t_real &
       &,0.400127734383524441252752295006867235_t_real &
       &,0.599659223816686447302437372938064069_t_real &
       &,0.669504971114849151207400380121582282_t_real &
       &,0.287329584124723697062792637168277370_t_real &
       &,0.684985258045177445036527922421998976_t_real &
       &,0.736591597816458085434309803029466241_t_real &
       &,0.425271357282148633119685056589095557_t_real &
       &,0.525905971350008866793255641753060364_t_real &
       &,0.367254325763929003708812721556340341_t_real &
       &,0.930120562189484075275449741802555167_t_real &
       &,0.450920778258169611036876099141412926_t_real &
       &,0.156334191739111964420563474087572882_t_real &
       &,0.878766221601812144110198926039947611_t_real &
       &,0.682110859750105369647965162421987424_t_real &
       &,0.731248777784258646876416054601749986_t_real &
       &,0.608233736991684526244183060757213906_t_real &
       &,0.149917105671554795258404963377812242_t_real &
       &,0.779471989626435651174733689905285151_t_real &
       &,0.744345604482620250838894794058938687_t_real &
       &,0.142436484675117905881693142865540283_t_real &
       &,0.652427366777940700973770837795214828_t_real &
       &,0.525087882765234643217123333506284388_t_real &
       &,0.303159630462718876313373089031618477_t_real &
       &,0.795221323802896354680305027404285962_t_real &
       &,0.855615884265754788775474041538584115_t_real /
  Data varshex(563:592) /&
       & 0.121348331791640837123278379802298457_t_real &
       &,0.503185891061659594083740840901403354_t_real &
       &,0.994375477305397970422267029887283992_t_real &
       &,0.105912275651418074323487214185311395_t_real &
       &,0.791695847815474884602826308712270761_t_real &
       &,0.271699626158258204632536698049130194_t_real &
       &,0.547171817345520058304340535722184166_t_real &
       &,0.832366114552957995571680052136573488_t_real &
       &,0.386603213669417407659405690747575877_t_real &
       &,0.397120392984911814902589212791835472_t_real &
       &,0.909854561887364917220619272182723127_t_real &
       &,0.414927314372952848874578910129879890_t_real &
       &,0.931134183321948443020138562766178238_t_real &
       &,0.150443342045825036756660528779608334_t_real &
       &,0.332198635583367504325664031018164108_t_real &
       &,0.277149282651352163387578215872681415_t_real &
       &,0.160012212944469849351475053086983292_t_real &
       &,0.947409292140653717757622033480563696_t_real &
       &,0.965915138336536821039156633652921401_t_real &
       &,0.258858929790550080637815224236380262_t_real &
       &,0.970511066411502613521283895191993771_t_real &
       &,0.161826332444830447756420684059810935_t_real &
       &,0.178663113429334315872831136398297175_t_real &
       &,0.877647806909587029925369970067126473_t_real &
       &,0.252728442929813213782227154463245225_t_real &
       &,0.407262398412945049296044046667452376_t_real &
       &,0.903689582906846968457562977937866365_t_real &
       &,0.418274963320885705776838169676407364_t_real &
       &,0.091603454108241338334843052989273067_t_real &
       &,0.839398549381422121539705435487637755_t_real /
  Data varshex(593:605) /&
       & 0.484626978442745437127822272335680490_t_real &
       &,0.246062526732168215105358131400995256_t_real &
       &,0.992613520955622918501320886814437303_t_real &
       &,0.118912836096536668851868640251667775_t_real &
       &,0.024044446917310181343581639623311380_t_real &
       &,0.914693199278454821232542149035074758_t_real &
       &,0.320720302194915304285122628809695969_t_real &
       &,0.245916325106218074264410074734033156_t_real &
       &,0.958256094000543790774667807064221288_t_real &
       &,0.276488737290828587713083126671729440_t_real &
       &,0.072795854704400273752525337287798492_t_real &
       &,0.804583146813490324189127146372630493_t_real &
       &,0.593840012009717542226544533495291819_t_real &
       &/
  
  
  !!-------------------------------------------------------------------
  !! cubic
  !! lmax=3
  Data pointlist(1) /6/ 
  !! lmax=5
  Data pointlist(2:3) /6,12/ 
  !! lmax=7
  Data pointlist(4:6) /6,12,8/ 
  !! lmax=9
  Data pointlist(7:9) /6,12,241/ 
  !! lmax=11
  Data pointlist(10:13)/6,12,8,241/ 
  !! lmax=13
  Data pointlist(14:17)/6,241,242,240/ 
  !! lmax=15
  Data pointlist(18:22)/6,12,242,241,240/ 
  !! lmax=17
  Data pointlist(23:28)/6,8,242,241,241,240/ 
  !! lmax=19
  Data pointlist(29:35)/6,8,12,242,241,241,48/
  !! lmax=21
  Data pointlist(36:43)/6,8,12,242,241,241,240,48/
  !! lmax=23
  Data pointlist(44:52)/6,12,8,241,241,241,242,240,48/
  !! lmax=25
  Data pointlist(53:61)/6,12,242,242,241,240,48,48,48/
  !! lmax=   27
  Data pointlist(62:71) /6,8,241,241,242,240,240,48,48,48/
  !! lmax=   29
  Data pointlist(72:83) /6,12,8,241,242,242,241,241,240,48,48,48/
  !! lmax=   31
  Data pointlist(84:96) /12,8,241,242,241,241,241,242,240,240,48,48,48/
  !! lmax=   33
  Data pointlist(97:110) /6,12,241,241,242,241,242,241,240,240,48,48,48,48/
  !! lmax=   35
  Data pointlist(111:125) /241,241,241,241,242,242,240,240,48,48,48,48,48,6,12/
  !! lmax=   41
  Data pointlist(126:145) /12,242,241,242,241,242,241,241,240,240,240,48&
       &,48,48,48,48,48,48,6,8/
  !! lmax=   43
  Data pointlist(146:166) /8,12,242,241,242,241,241,241,241,240,240,240&
       &,48,48,48,48,48,48,48,48,241/
  !! lmax=   37
  Data pointlist(167:183) /6,241,242,241,241,242,241,240,240,48,48,48,48,48&
       &,8,242,240/
  !! lmax=   39
  Data pointlist(184:201) /6,8,242,241,242,241,242,241,241,240,240,240,48,48&
       &,48,48,48,48/



  !! lmax=3
  Data weights(1) /2.0943951023931954923_t_real/ 
  !! lmax=5
  Data weights(2:3) /&
       & 0.41887902047863909846_t_real&
       &,0.83775804095727819692_t_real/ 
  !! lmax=7
  Data weights(4:6) /&
       & 0.59839860068377014066_t_real&
       &,0.47871888054701611253_t_real&
       &,0.40391905546154484495_t_real/ 
  !! lmax=9
  Data weights(7:9) /&
       & 0.333278048348211352147259684195928_t_real&
       &,0.25044962153254212235583796252282_t_real&
       &,0.315054452744974973862373328236192_t_real/ 
  !! lmax=11
  Data weights(10:13) /&
       & 0.159572960182338704175880298833228_t_real&
       &,0.283685262546379918534898309036881_t_real&
       &,0.265071880146638804495285535464217_t_real&
       &,0.253505610897311302933926156165097_t_real/ 
  !! lmax=13
  Data weights(14:17) /&
       & 0.174252735543320663464277340667401_t_real&
       &,0.164002846648077844946819192482287_t_real&
       &,0.165956808711311495905776654460396_t_real&
       &,0.150075936353079366358442048437051_t_real/ 
  !! lmax=15
  Data weights(18:22) /&
       & 0.165769565398111410581497436518502_t_real&
       &,0.138532559921300900115337700802595_t_real&
       &,0.146485688586445280022905704952145_t_real&
       &,0.132436616909935439897051460681148_t_real&
       &,0.133967798791739850454106855382369_t_real/ 
  !! lmax=17
  Data weights(23:28) /&
       & 0.04810746585139659457672226569522064_t_real&
       &,0.123071735281670180596632273188645_t_real&
       &,0.124945096872513315109704113231751_t_real&
       &,0.120580249028527899553419278960188_t_real&
       &,0.103191734088330407050941401820932_t_real&
       &,0.121830917385521375853317779047025_t_real/ 
  !! lmax=19
  Data weights(29:35) /&
       & 0.007535190013117138117896271027691597_t_real&
       &,0.09061000833610514010361827896390239_t_real&
       &,0.09265184700375432421283713871231646_t_real&
       &,0.08942676055004592969337902704313919_t_real&
       &,0.08487112439121475736042614737415046_t_real&
       &,0.09518264418191036893468101417757567_t_real&
       &,0.08785259467896816187576115592533489_t_real&
       &/ 
  !! lmax=21
  Data weights(36:43) /&
       & 0.06967855090540037740686733510876141_t_real&
       &,0.08021962308552601053975350877296547_t_real&
       &,0.07629461771935279216515968930511552_t_real&
       &,0.07939343745253054342072232666157940_t_real&
       &,0.06513636946550791140554986667454729_t_real&
       &,0.07793248373075363082373873347292932_t_real&
       &,0.06882781368562169231160173760301136_t_real&
       &,0.07500092515800830058397335905688989_t_real&
       &/ 
  !! lmax=23
  Data weights(44:52) /&
       & 0.02239755062103846740100076759903218_t_real&
       &,0.07184075893484736519791925142563366_t_real&
       &,0.07003719860124849565597200853496767_t_real&
       &,0.06935092759371099524284176102465906_t_real&
       &,0.06482032680351046659129495742226639_t_real&
       &,0.05160728216651316735193055430327014_t_real&
       &,0.07048105416807012741903369551220522_t_real&
       &,0.06348336993464156093533630795491276_t_real&
       &,0.06949515747104321210106806693585305_t_real&
       &/ 
  !! lmax=25
  Data weights(53:61) /&
       & 0.042091101987723483700913605560546240_t_real &
       &,0.048182320399190557204636530023044240_t_real &
       &,0.050074291691079134100578400862829580_t_real &
       &,0.045340977164248614918479274648858130_t_real &
       &,0.053149815799209153083575493153551140_t_real &
       &,0.054594954630805126842344538094752830_t_real &
       &,0.049063220615871379921142734040412460_t_real &
       &,0.049594081391117789002810097680745440_t_real &
       &,0.044255098301226178378338596971308840_t_real &
       &/
  !! lmax=   27
  Data weights(62:71) /&
       & 0.038871628860071923143022268349401960_t_real &
       &,0.044561992014312167858306562549548080_t_real &
       &,0.047367952764703650974424460855555220_t_real &
       &,0.048388969721292617018158841866026440_t_real &
       &,0.046523908564783017036074813297299960_t_real &
       &,0.052114752746930676619805706984971660_t_real &
       &,0.047994304629514810190717168602884730_t_real &
       &,0.046143832163826054625459032426112210_t_real &
       &,0.038487995901894623666509985674765190_t_real &
       &,0.043686663243255353957731890733778910_t_real &
       &/
  !! lmax=   29
  Data weights(72:83) /&
       & 0.041655934593063774828299620334769430_t_real &
       &,0.030075849117194776773358802061107690_t_real &
       &,0.044688479775788260067199801397404060_t_real &
       &,0.044439598067619509856781183696826110_t_real &
       &,0.043908587206725508690116234976443040_t_real &
       &,0.039166497625833700495499358934740070_t_real &
       &,0.044279846397446236660766765950642700_t_real &
       &,0.037753454203311751746616678982846620_t_real &
       &,0.039130035341462149116397109574929300_t_real &
       &,0.043504163023385939522366180525746160_t_real &
       &,0.031190694759939155990135024063054810_t_real &
       &,0.042591486528561870018219958002919860_t_real &
       &/
  !! lmax=   31
  Data weights(84:96) /&
       & 0.038433864493675153361901706327886110_t_real &
       &,0.038570694034990256804956261519156480_t_real &
       &,0.037199854732676814161995862233047410_t_real &
       &,0.038276484271511222542429980390820860_t_real &
       &,0.037873595480523061838372667526310880_t_real &
       &,0.034625638856585703531190146710485560_t_real &
       &,0.017808237470986037043053546568743130_t_real &
       &,0.037905687943313245274924645955574960_t_real &
       &,0.034302688830060495122367531984709070_t_real &
       &,0.033471076242556405098091712382556200_t_real &
       &,0.035334582267026954600812696481690740_t_real &
       &,0.037549128748385505428497927714111790_t_real &
       &,0.037147129740379986395061807366201060_t_real &
       &/
  !! lmax=   33
  Data weights(97:110) /&
       & 0.035532910207025995454076672833333290_t_real &
       &,0.027270174116406568069045886323482980_t_real &
       &,0.032738688642694489077320809954908450_t_real &
       &,0.027795707510791754551461702900557650_t_real &
       &,0.028299073671883829234884239853476130_t_real &
       &,0.030655804821937084288044089255845360_t_real &
       &,0.035427984461605022039445253326799490_t_real &
       &,0.032012261188180920225808813942503230_t_real &
       &,0.034343934470154603702794823439969210_t_real &
       &,0.028660371905346711605776220969091250_t_real &
       &,0.032178181082158693511174075383037240_t_real &
       &,0.032441850758808038037699214495505220_t_real &
       &,0.028344370776164547913410908756915630_t_real &
       &,0.032608914540741058264480384131220980_t_real &
       &/  
  !! lmax=   35
  Data weights(111:125) /&
       & 0.028476670242873356109391990800629770_t_real &
       &,0.028957287773881556033271571300520770_t_real &
       &,0.030499460153126933719719990253400660_t_real &
       &,0.029891025945857647568130826371135310_t_real &
       &,0.029869689429547394312240557606801030_t_real &
       &,0.027410887056827974850217646559812770_t_real &
       &,0.030386422189169699087376526410609550_t_real &
       &,0.021361561673874716906751681853674750_t_real &
       &,0.027596654848391411664661264679990220_t_real &
       &,0.028685805825331925183770550026347280_t_real &
       &,0.027551430219546668404744058727950470_t_real &
       &,0.027319957839301620149430113535417430_t_real &
       &,0.025527879394508531684952646267626100_t_real &
       &,0.024481957956847800356880818375418660_t_real &
       &,0.034523650779534660451337936642963090_t_real &
       &/  
  !! lmax=   41
  Data weights(126:145) /&
       & 0.021583168294555804020455187854161430_t_real &
       &,0.021737491820467039527771223194212010_t_real &
       &,0.022784316371533098591819091759314950_t_real &
       &,0.021896714463173841902437565116934810_t_real &
       &,0.022010572503689728502722478190996810_t_real &
       &,0.021240569708875920458611165661841740_t_real &
       &,0.020930395258868578260687043725598120_t_real &
       &,0.019694669583526070184694906298911480_t_real &
       &,0.021862717310316276691575307268388900_t_real &
       &,0.019788743340604361441820551051551240_t_real &
       &,0.022302065520683010671378112407774010_t_real &
       &,0.017795960673030863419873643142429560_t_real &
       &,0.021455710437970339770308386913495530_t_real &
       &,0.019849488643598556143716609533901050_t_real &
       &,0.020346825000176638277098964187079510_t_real &
       &,0.022205628997354489272085456302253790_t_real &
       &,0.022204466239410679859624172928870280_t_real &
       &,0.021181813643283251731512087171166360_t_real &
       &,0.005781092068515899960291102897512510_t_real &
       &,0.021101625847513298684552327583626260_t_real &
       &/
  !! lmax=   43
  Data weights(146:166) /&
       & 0.020098315347609935991626581768826850_t_real &
       &,0.022426677695955586660649477249881620_t_real &
       &,0.015597892598958995495793642276034200_t_real &
       &,0.015531441983364976858009940033235750_t_real &
       &,0.022669451485297578509777180469195420_t_real &
       &,0.020360925329179815977869703846132400_t_real &
       &,0.018058724540960619740316177131960790_t_real &
       &,0.018444122437330188674537026503147240_t_real &
       &,0.016858103759326642566727319602432460_t_real &
       &,0.020388814808171723330927671201582860_t_real &
       &,0.018044319573962021825772416681167290_t_real &
       &,0.020059571988545167233692981264559080_t_real &
       &,0.019244177965090699665818893903768990_t_real &
       &,0.020531112760547134537822844421500280_t_real &
       &,0.017438880407282591639174516941689010_t_real &
       &,0.018514496969305855549696081642727170_t_real &
       &,0.018949788873723964206542380293300200_t_real &
       &,0.020494901550404202131420803446567640_t_real &
       &,0.017516852980204411169697360980028190_t_real &
       &,0.019913174570659018618127919424977260_t_real &
       &,0.014465857641584282499547970213469010_t_real &
       &/
  !! lmax=   37
  Data weights(167:183) /&
       & 0.026154262425462597066901333624854118_t_real &
       &,0.024740968894256663882313197240207608_t_real &
       &,0.027381339318287350085633959480575738_t_real &
       &,0.026724751969062706877225195951957092_t_real &
       &,0.026119130928180827104856483432770280_t_real &
       &,0.028015138211494130068797957734783263_t_real &
       &,0.027920911398125075045925804403788289_t_real &
       &,0.025225324595386407561692008036162638_t_real &
       &,0.013895893743468266856725753860277242_t_real &
       &,0.023914444510528743299670755583606733_t_real &
       &,0.026352208739994599638524660070378636_t_real &
       &,0.023118156159700428249626299410177953_t_real &
       &,0.027413937018005898002555830718565383_t_real &
       &,0.026753896760754955389534464184867138_t_real &
       &,0.028254022995617444801111268838400100_t_real &
       &,0.026579478721748524056765506941363133_t_real &
       &,0.025933978168748208176918253843826627_t_real &
       &/
  !! lmax=   39
  Data weights(184:201) /&
       & 0.024642771787659930165645024187625918_t_real &
       &,0.022309621334308867075927438957115224_t_real &
       &,0.022848477799561861809616383581819400_t_real &
       &,0.022350235701075973510946506418171838_t_real &
       &,0.020868005666562533027619914465031200_t_real &
       &,0.024552363398197283100560435433088961_t_real &
       &,0.022457362986291908352937165219581102_t_real &
       &,0.024760455069467748653443210071036637_t_real &
       &,0.024002297593178091876699033286953202_t_real &
       &,0.022656940419112881224011103266103187_t_real &
       &,0.021186061231245843184245984720220196_t_real &
       &,0.026738504960138034415267920320735906_t_real &
       &,0.023585557322198785230225020664918652_t_real &
       &,0.020247030092353467971432901854812445_t_real &
       &,0.024465134092924337685181284613536282_t_real &
       &,0.021937268466130453782043969197185359_t_real &
       &,0.025016842765254478139476634599321718_t_real &
       &,0.023538585952029531702493108102507509_t_real &
       &/
  !! lmax=9
  Data vars(1:2) /&
       & 0.83609559674910521169857046024601_t_real&
       &,0.387907304066807722797010001351143_t_real/
  !! lmax=11
  Data vars(3:4) /&
       & 0.904534033733290867940436200910191_t_real&
       &,0.301511344577763622646812066970057_t_real/
  !! lmax=13
  Data vars(5:10) /&
       & 0.914152532416420162167480391540728_t_real&
       &,0.286640146766503980402863673376484_t_real&
       &,0.659905001656390535727456372692715_t_real&
       &,0.359236381200120665890719062362076_t_real&
       &,0.841991943784699350740331899130458_t_real&
       &,0.539490098705864747757261708252716_t_real/
  !! lmax=15
  Data vars(11:16) /&
       & 0.65851167678212717856102782791129_t_real&
       &,0.364314072035630446327900575090596_t_real&
       &,0.878522265967329617671220782295486_t_real&
       &,0.337785899794254704343343922475565_t_real&
       &,0.916866318264100158942094806649518_t_real&
       &,0.3991943817651167736751546955371_t_real/
  !! lmax=17
  Data vars(17:24) /&
       & 0.690421048382292178153032396103913_t_real&
       &,0.215957291845848832354930328945833_t_real&
       &,0.828769981252592210694031500711147_t_real&
       &,0.395689473055941908779727135592324_t_real&
       &,0.965124035086594105655529546158105_t_real&
       &,0.185115635344736169151586543356317_t_real&
       &,0.878158910604066133449110592671584_t_real&
       &,0.478369028812150196668193543052489_t_real/
  !! lmax=19
  Data vars(25:33) /&
       & 0.676441040011426369868857172216994_t_real&
       &,0.291298882209526746288335878318565_t_real&
       &,0.807089818359582501629574989260428_t_real&
       &,0.417496122796545335940470554302347_t_real&
       &,0.974888643677173235480043489930166_t_real&
       &,0.15746766720390817632447875386306_t_real&
       &,0.882270011260322631916665753047258_t_real&
       &,0.449332832326955734884695382702087_t_real&
       &,0.140355381171318328571556780751702_t_real/
  !! lmax=21
  Data vars(34:44) /&
       & 0.674360146036276574240465476865946_t_real&
       &,0.300793595137701468300356883599137_t_real&
       &,0.932642590312690564450361693247777_t_real&
       &,0.255125262111413443724659641862272_t_real&
       &,0.791795559393492143466037460949991_t_real&
       &,0.431891069671941037837573204456768_t_real&
       &,0.965232421976448368780051836299341_t_real&
       &,0.261393136033598847230784207849514_t_real&
       &,0.854415804684658782831663598377717_t_real&
       &,0.499045316179603714588195352462233_t_real&
       &,0.144663074432511538886159042626734_t_real&
       &/
  !! lmax=23
  Data vars(45:57) /&
       & 0.777493219314767127213777705408861_t_real&
       &,0.444693317871743731104577366252498_t_real&
       &,0.912509096867473692992168393889743_t_real&
       &,0.289246562757543855705028028092384_t_real&
       &,0.982972302707253296863729600560754_t_real&
       &,0.12993354476500668622350319060905_t_real&
       &,0.671297344269522628017298568226789_t_real&
       &,0.314196994182586079225390950149911_t_real&
       &,0.938319218137591520905616391486685_t_real&
       &,0.3457702197611282708569170905689_t_real&
       &,0.836036015482458885943746382165861_t_real&
       &,0.525118572443642024905268201469165_t_real&
       &,0.159041710538352952424263579125666_t_real&
   &/
  !! lmax=   25
  Data vars(58:74) /&
       & 0.688309142218006211438512037007169500_t_real &
       &,0.229043771969955556181620802044313100_t_real &
       &,0.624512475929750932246830934037822800_t_real &
       &,0.469007819568271723870866709080431100_t_real &
       &,0.977535791879453033161732498998298600_t_real &
       &,0.149036531754148595707174865353991200_t_real &
       &,0.942283353128282824104104583948858200_t_real &
       &,0.334816490659166684548292442480188500_t_real &
       &,0.777062794968741082385849223076056900_t_real &
       &,0.515775835991624327563423262369864700_t_real &
       &,0.360761277969946909995116017075061500_t_real &
       &,0.835575485065536673030374885071339000_t_real &
       &,0.537361403147623114259418491974940000_t_real &
       &,0.114264303982962079220284698910702400_t_real &
       &,0.898622099196517052479586323911266700_t_real &
       &,0.370347554045070894888740293875399500_t_real &
       &,0.235204192242566882409648540683883400_t_real &
       &/
  !! lmax=   27
  Data vars(75:93) /&
       & 0.948960275193177679321853299914014100_t_real &
       &,0.223018380526458499768169150748247600_t_real &
       &,0.856619784243603955815909655378044200_t_real &
       &,0.364830471070085789178566811319308600_t_real &
       &,0.692564881725506522353696938279748700_t_real &
       &,0.201761664349475278789085922885395100_t_real &
       &,0.790597263391429956227974010255689900_t_real &
       &,0.612336481942715811225672650676126800_t_real &
       &,0.979398140337109853388327699571117300_t_real &
       &,0.201938809306707741412611814416545700_t_real &
       &,0.722954017454501775521806045850823100_t_real &
       &,0.562184155402609965466190635844987100_t_real &
       &,0.401604860603865239718534647173074800_t_real &
       &,0.822366683101256717225536136123886700_t_real &
       &,0.525558535259220024056508903957583300_t_real &
       &,0.217947848214246869628593330549239200_t_real &
       &,0.907020924181417884841915038241110900_t_real &
       &,0.408588676422929708516775800154991600_t_real &
       &,0.101825029320128523209093815053726800_t_real &
       &/
  !! lmax=   29
  Data vars(94:114) /&
       & 0.883891427556494490502642093593143600_t_real &
       &,0.330708288595963566631215031556742300_t_real &
       &,0.653608780334129153328704994573451400_t_real &
       &,0.381564050377212825154773701031581600_t_real &
       &,0.695426311904478538612796654879105200_t_real &
       &,0.181009639030273364111420965322824100_t_real &
       &,0.748193910256371904991627044084877500_t_real &
       &,0.469151293643788274226113478662881700_t_real &
       &,0.962330955909753349690140997947071400_t_real &
       &,0.192248707795163547269337457283314500_t_real &
       &,0.979294690791006006205008107327884900_t_real &
       &,0.202439888827641713129123325265811800_t_real &
       &,0.914746363222945111253089769430067200_t_real &
       &,0.388827544293737575530607939301231500_t_real &
       &,0.109782656958634934383818946503517300_t_real &
       &,0.823349243056872562371176979577684900_t_real &
       &,0.562476142748494757258441224246961500_t_real &
       &,0.075608285236804156376369109045640730_t_real &
       &,0.807966193393655149150558777186670000_t_real &
       &,0.525151517928472997508549389866927900_t_real &
       &,0.267219972214629478066399019795392600_t_real &
       &/  
  !! lmax=   31
  Data vars(115:139) /&
       & 0.960636941489162741898619750720101700_t_real &
       &,0.196439133889287598212075351048738100_t_real &
       &,0.650100257739484502478210433137196600_t_real &
       &,0.393369177458162881508640873083092400_t_real &
       &,0.735086055357023748990851500671892700_t_real &
       &,0.479399880694421229065650701906181500_t_real &
       &,0.861114649794155792504056434172209300_t_real &
       &,0.359500737071489483018918199109868900_t_real &
       &,0.996180977060192533666002339217933100_t_real &
       &,0.061739213403639007673780834905954210_t_real &
       &,0.693238020028101552180516331605091100_t_real &
       &,0.197083980006074929144906662549814300_t_real &
       &,0.975673020276206200937247570163869100_t_real &
       &,0.219230831556844949858561412472913200_t_real &
       &,0.920314408157191257167046790258325400_t_real &
       &,0.391179485835183819408149750873664200_t_real &
       &,0.906944238959543857048427446852971200_t_real &
       &,0.380574904024794347996446620838915500_t_real &
       &,0.180595929756493629966220790685964200_t_real &
       &,0.826101910740545573114899187933150200_t_real &
       &,0.555028724660080157978539261528533100_t_real &
       &,0.097461519960570092630978118079862820_t_real &
       &,0.793794535100575487662729623115531000_t_real &
       &,0.534434821349659374841803823031813900_t_real &
       &,0.290292366026767741688622851849041800_t_real &
       &/
  !! lmax=   33
  Data vars(140:167) /&
       & 0.931855278529831987402600811403674800_t_real &
       &,0.256559681045258208251450955995647100_t_real &
       &,0.746660585570964028140908975550196100_t_real &
       &,0.470371114097594806876074170276658700_t_real &
       &,0.682387029061023541911958452426381000_t_real &
       &,0.262098998736240166002216919843705700_t_real &
       &,0.981951284469439747998620156932532600_t_real &
       &,0.133737943248760591151195724841609500_t_real &
       &,0.621104190136198432872459119537479500_t_real &
       &,0.477974026481056950171400951256089000_t_real &
       &,0.853091120245525051528230958021257700_t_real &
       &,0.368941418492309641596289524927145800_t_real &
       &,0.862218258038519092570140887607560100_t_real &
       &,0.506536943869863954276991148009949900_t_real &
       &,0.972827182499064350319279778017295500_t_real &
       &,0.231532444791074895770555141044721600_t_real &
       &,0.777692958415106715394153098077838300_t_real &
       &,0.616322863959680045343947320264095200_t_real &
       &,0.123853905033700172488610439803364200_t_real &
       &,0.925408506115446416315418261493387500_t_real &
       &,0.363326563079075271281943012070153200_t_real &
       &,0.107763191166207168064752325186114600_t_real &
       &,0.750458113514514805977604060386617600_t_real &
       &,0.565604771598950342117741800993305700_t_real &
       &,0.341912067942526454457360735883671500_t_real &
       &,0.849549973225721149138318320609634800_t_real &
       &,0.478444993722054837522602529289718500_t_real &
       &,0.222160372196481939877359423330616000_t_real &
       &/
  !! lmax=   35
  Data vars(168:198) /&
       & 0.874652422831084066219061420003533000_t_real &
       &,0.342770432823277898095878769339675500_t_real &
       &,0.779944825054023987907234488161597500_t_real &
       &,0.442541562947169921191831159184544700_t_real &
       &,0.986953916476549401822863784847583300_t_real &
       &,0.113846314722085640245277116186650600_t_real &
       &,0.663281893877240222553487703376800300_t_real &
       &,0.529177252560340805589111339871911900_t_real &
       &,0.691727407342792411370712673827812900_t_real &
       &,0.207428030559124478078987128669171500_t_real &
       &,0.648004891716373670888498597960123200_t_real &
       &,0.400224088009831895595493533419287300_t_real &
       &,0.967590800481255648921285028018196800_t_real &
       &,0.252523351047072324170224087716678500_t_real &
       &,0.838282188908566436056459738905435500_t_real &
       &,0.545236619972157166600612303858866500_t_real &
       &,0.856952439709180200016463227345091800_t_real &
       &,0.462092504378556978565898043461932800_t_real &
       &,0.228260889058194053771426414866213100_t_real &
       &,0.761595245694233735343999108687836100_t_real &
       &,0.562835667639254200417186374238239100_t_real &
       &,0.321229968977078371612012208571633300_t_real &
       &,0.940610318419123554454079963264026300_t_real &
       &,0.290046268080514921306814275048672400_t_real &
       &,0.176423896499428521318417878783381500_t_real &
       &,0.796743064105113434772150875726059400_t_real &
       &,0.590881262693055610050936706392550600_t_real &
       &,0.126727357735633833133038349620448600_t_real &
       &,0.905988990648181199171123996115861600_t_real &
       &,0.415932864766447673500163719816126600_t_real &
       &,0.078637146638632502929192581035990800_t_real &
       &/
  !! lmax=   41
  Data vars(199:234) /&
       & 0.673089303933418801507557227972894500_t_real &
       &,0.306433643487218188645182381952342900_t_real &
       &,0.911213003401744589267572674598295600_t_real &
       &,0.291282390843981678881255232883178600_t_real &
       &,0.697471581140683902740450288796053000_t_real &
       &,0.164519868107863439588503605855098500_t_real &
       &,0.993466631896775909299471371230863300_t_real &
       &,0.080697122958863767190912750028607740_t_real &
       &,0.633506444674948709032344225274978600_t_real &
       &,0.444228735124836674407633716583462300_t_real &
       &,0.698122260001346727338680866313781100_t_real &
       &,0.506273300743092731845160127180220500_t_real &
       &,0.806568433015534150066704455980936100_t_real &
       &,0.417999618936946999249077073503858400_t_real &
       &,0.975087764009895416259629894590201300_t_real &
       &,0.221819414114685879924481530989812200_t_real &
       &,0.866188342800131919372181072993640900_t_real &
       &,0.499717675089806238199588739883918000_t_real &
       &,0.930146161626016792306528031720083700_t_real &
       &,0.367189485160438446938996241419337000_t_real &
       &,0.963730211169906079931549501159125000_t_real &
       &,0.226799621934067201884899937003242500_t_real &
       &,0.140662758287232579851968053079889500_t_real &
       &,0.791251478528672017713039267388265100_t_real &
       &,0.606065943785004880366017762671936400_t_real &
       &,0.081272193954524133798890058997165500_t_real &
       &,0.925041551277560208930597865525574200_t_real &
       &,0.349714292322953828015551961734265700_t_real &
       &,0.148317369701126185601018808905371700_t_real &
       &,0.868118890885336295813167219136376800_t_real &
       &,0.476038922411904040174863631121186700_t_real &
       &,0.140557944054851555536610992348718700_t_real &
       &,0.854812572304245581113635930196954800_t_real &
       &,0.433333521074299038540141047340614600_t_real &
       &,0.285512741824159139442544956041287700_t_real &
       &,0.785925679753782042151793451062424900_t_real /
  Data vars(235:239) /&
       & 0.573031536242873401713086805821191500_t_real &
       &,0.232283629157734873948531146248246900_t_real &
       &,0.750319616677112529311757066584150200_t_real &
       &,0.547071401298283744017351938245455900_t_real &
       &,0.371124446393717245001105858695359100_t_real &
       &/

  !! lmax=   43
  Data vars(240:273) /&
       & 0.698030306440546300260779584791497000_t_real &
       &,0.159710308311749409442785643124882200_t_real &
       &,0.891857999161706165739606177960090400_t_real &
       &,0.319835355559136179256675313621316900_t_real &
       &,0.638919256424711706581753654656417400_t_real &
       &,0.428444124174187942969759459550151400_t_real &
       &,0.685819626446031518955450562555899300_t_real &
       &,0.514612203499599193469465367944481100_t_real &
       &,0.938245919051743237480970420203711900_t_real &
       &,0.244637073419739139292498068593305700_t_real &
       &,0.976118483353220505916262581277719300_t_real &
       &,0.153611045267273313474001355626108900_t_real &
       &,0.995795340880058308600319653900664000_t_real &
       &,0.064775146011292301681118069497063730_t_real &
       &,0.922230899118315987301632247836694200_t_real &
       &,0.386639585029032535965820280435826000_t_real &
       &,0.806265519826902071477878609521928000_t_real &
       &,0.591553811194091946805146600704251200_t_real &
       &,0.979643067254758622392587756350650600_t_real &
       &,0.200747255970507538502765745489106300_t_real &
       &,0.950587350568481983650613784756095500_t_real &
       &,0.293444582738008727835421221533079600_t_real &
       &,0.101360573207287482790812528450844900_t_real &
       &,0.866431306492185174351274154828640800_t_real &
       &,0.491968904423874975629911286826290000_t_real &
       &,0.085225513845428039030768414062626790_t_real &
       &,0.842178997186780318828150328365243900_t_real &
       &,0.437427087160409643954742023400608300_t_real &
       &,0.315265094984884827220453545762274400_t_real &
       &,0.750423653713292578889026073892877200_t_real &
       &,0.539851493855023124954545347656033800_t_real &
       &,0.381345911909505428530854295780300200_t_real &
       &,0.723685916485156254379855620188225700_t_real &
       &,0.628046483450103581201199815233004100_t_real /
  Data vars(274:285) /&
       & 0.286070461437384715411875772430820200_t_real &
       &,0.900400753713359591841636579514020100_t_real &
       &,0.393722155550866687684788438430493500_t_real &
       &,0.185098208907577220493861773848037500_t_real &
       &,0.774955522663755083671321020716796200_t_real &
       &,0.619345002221436023484756773437666900_t_real &
       &,0.125919442963648739296249446331087700_t_real &
       &,0.816264370750028961354334112768138700_t_real &
       &,0.532105764687567457971362369189349700_t_real &
       &,0.224890934077650966278166610067792300_t_real &
       &,0.790311851299496617415584776888803600_t_real &
       &,0.433247722264966066350135628053959600_t_real &
       &/

  !! lmax=   37
  Data vars(286:320) /&
       & 0.975293609901472018663320569649156575_t_real &
       &,0.156208793743110572036733883519973422_t_real &
       &,0.683203395788872622764488972635589673_t_real &
       &,0.257810472954661274315826114402846707_t_real &
       &,0.830625884443252018257589633192295571_t_real &
       &,0.393738898315155893363685467334545371_t_real &
       &,0.922792176474345378763953581737897021_t_real &
       &,0.272446874672605315320974107725871419_t_real &
       &,0.641027490692422760572737212576836069_t_real &
       &,0.422098936688013586526178728399047009_t_real &
       &,0.714979913376317416603485599536007958_t_real &
       &,0.494370166711338702768367459972541822_t_real &
       &,0.986978786268028532967204730492666250_t_real &
       &,0.160850475463671700535866786241718184_t_real &
       &,0.949554036579188923037973641187402621_t_real &
       &,0.313603462379113959791697536200897261_t_real &
       &,0.884081310039659922622175286043164976_t_real &
       &,0.461376440745633279373005560117405805_t_real &
       &,0.074377531307847605816464626248818353_t_real &
       &,0.872678223664570574484496167882757733_t_real &
       &,0.427581654060356081340227135150963985_t_real &
       &,0.235810616920993852819605869651699590_t_real &
       &,0.942572343875142482604982752272007074_t_real &
       &,0.314711155776180993489092967389225521_t_real &
       &,0.111867175667129856059098335748307125_t_real &
       &,0.768448516009512455145859454182769506_t_real &
       &,0.545392839593017676007233339008961931_t_real &
       &,0.334714100036797031820061740410067340_t_real &
       &,0.800026068531681338245387242062969688_t_real &
       &,0.575174628659306812027050482165313270_t_real &
       &,0.170682266847994580004037887617404584_t_real &
       &,0.704165975553083554391279515163011469_t_real &
       &,0.091107396772974849665901769043187657_t_real &
       &,0.801819323969416264480672169421353160_t_real &
       &,0.597566541658105456782064294805679717_t_real &
       &/

  !! lmax=   39
  Data vars(321:358) /&
       & 0.681954146521450498735838777953845442_t_real &
       &,0.264342739798920994064474656000535077_t_real &
       &,0.987830541031106348035027945292890686_t_real &
       &,0.109979139399687334030639621677012517_t_real &
       &,0.629338279881628869288702871004111316_t_real &
       &,0.455923961808616871018243054852042443_t_real &
       &,0.717862918713939571023629495303811865_t_real &
       &,0.492276766634128178225189365929204522_t_real &
       &,0.705308417259076590507750948199713336_t_real &
       &,0.071274631440594781476446289967644386_t_real &
       &,0.818837078470578687901072356464946939_t_real &
       &,0.405897671169574068694932771662253814_t_real &
       &,0.953489332438132106979029558528899983_t_real &
       &,0.213140907531479273029373190635928302_t_real &
       &,0.978241169981870069912079377749006946_t_real &
       &,0.207470994002780996080143416220602622_t_real &
       &,0.900446958314447024031481209741684989_t_real &
       &,0.434965832292906086624211887985852601_t_real &
       &,0.813415157269177109454667934048381702_t_real &
       &,0.581683575429769092454684152808205684_t_real &
       &,0.865975237641113428734921378525009399_t_real &
       &,0.480857357761112199347355413232188947_t_real &
       &,0.137342962249322673866079850580850596_t_real &
       &,0.727597293165943878673553824420060759_t_real &
       &,0.582212304303990948055902743837005809_t_real &
       &,0.362809883678254499297754335924479195_t_real &
       &,0.941686701124473937593371955110107126_t_real &
       &,0.322103479996736758298142567399916876_t_real &
       &,0.097342206155898781825045814148972752_t_real &
       &,0.810397988823026670533383297936529707_t_real &
       &,0.510107995052155561059538748187941204_t_real &
       &,0.288175177792022151843875873012541410_t_real &
       &,0.894005849469231569331479352810996153_t_real &
       &,0.369627877283054885001978430567788450_t_real &
       &,0.253236595795356130491342125051901721_t_real &
       &,0.775167184716982424207869655990163249_t_real &
       &,0.609624083113703525529966777540896140_t_real &
       &,0.165753772282032682556097734915255179_t_real /

Contains
  !!***********************************************************************
  !!
  !! irreducible spherical mesh for each pointgroup
  !!
  !!
  !!***********************************************************************
  !!
  !! Abcissas and Weight Factors for Surface or Sphere Integration
  !! all  point symmetries in the irreducible part
  !!
  Subroutine gsphirred(nv,aa,ww,nw,HEX,ipgr,prot)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)  :: prot
    Logical(t_log),Intent(in)  :: HEX
    Integer(t_int),Intent(in)  :: ipgr
    Integer(t_int),Intent(in)  :: nv
    Integer(t_int),Intent(out) :: nw
    Real(t_real) ,Intent(out)  :: aa(3,*),ww(*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="gsphirred"
    Integer(t_int) :: nw0,ivar,ip,np,np0,ind,nvv,p
    Real(t_real)   :: wsum
    !!
    nvv=Max(1_t_int,nv)
    If(prot)Then
       Write(outfile,400)nsubfunc,nvv
       If(HEX)Then
          Write(outfile,401)nsubfunc
       Else
          Write(outfile,402)nsubfunc
       Endif
       Write(outfile,403)nsubfunc,ipgr
    Endif

    wsum=0.0_t_real
    !!
    !!
    nw=0_t_int
    !! map argument onto mesh type
    If(HEX)Then
       ind=0
       If(nvv<= maxtypeshex)Then
          ind=nvtypeshex(nvv)
       End If
    Else
       ind=0
       If(nvv<= maxtypes)Then
          ind=nvtypes(nvv)
       End If
       
    Endif
    
    If(prot)Then
       Write(outfile,2)nsubfunc,ind
    Endif

    If(ind==0)Then
       Write(errbuf,1001)nvv
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    Endif
    If(hex)Then
       np0=npointshex(ind-1)+1_t_int
       np=npointshex(ind)
       ivar=nvarshex(ind)
    Else
       np0=npoints(ind-1)+1_t_int
       np=npoints(ind)
       ivar=nvars(ind)
    Endif

    If(prot)Then
       Write(outfile,4)nsubfunc,np0
       Write(outfile,3)nsubfunc,np
       Write(outfile,5)nsubfunc,ivar
    Endif

    Do ip=np0,np
       nw0=nw+1_t_int

       If(HEX)Then
          p=pointlisthex(ip)
       Else
          p=pointlist(ip)
       End If
       
       If(prot)Then
          Write(outfile,6)nsubfunc,p
       End If
       
       Select Case (p)
          !!hex
       Case (2_t_int)
          Call hex2i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (60_t_int)
          Call hex60i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (61_t_int)
          Call hex61i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (120_t_int)
          ivar=ivar+2_t_int
          !Write(outfile,7)nsubfunc,ivar
          Call hex120i(aa(1,nw0),ww(nw0),nw&
               &,varshex(ivar-1_t_int),varshex(ivar),ipgr)
       Case (121_t_int)
          ivar=ivar+2_t_int
          !Write(outfile,7)nsubfunc,ivar
          Call hex121i(aa(1,nw0),ww(nw0),nw&
               &,varshex(ivar-1_t_int),varshex(ivar),ipgr)
       Case (122_t_int)
          ivar=ivar+3_t_int
          !Write(outfile,7)nsubfunc,ivar
          Call hex122i(aa(1,nw0),ww(nw0),nw&
               &,varshex(ivar-2_t_int),varshex(ivar-1_t_int),varshex(ivar)&
               &,ipgr)
       Case (243_t_int)
          ivar=ivar+3_t_int
          !Write(outfile,7)nsubfunc,ivar
          Call hex24i(aa(1,nw0),ww(nw0),nw&
               &,varshex(ivar-2_t_int),varshex(ivar-1_t_int),varshex(ivar)&
               &,ipgr)

          !! cubic
       Case (6_t_int)
          Call poi6i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (8_t_int)
          Call poi8i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (12_t_int)
          Call poi12i(aa(1,nw0),ww(nw0),nw,ipgr)
       Case (240_t_int)
          ivar=ivar+2_t_int
          Call poi240i(aa(1,nw0),ww(nw0),nw&
               &,vars(ivar-1_t_int),vars(ivar),ipgr)
       Case (241_t_int)
          ivar=ivar+2_t_int
          Call poi24i1(aa(1,nw0),ww(nw0),nw&
               &,vars(ivar-1_t_int),vars(ivar),ipgr)
       Case (242_t_int)
          ivar=ivar+2_t_int
          Call poi24i2(aa(1,nw0),ww(nw0),nw&
               &,vars(ivar-1_t_int),vars(ivar),ipgr)
       Case (48_t_int)
          ivar=ivar+3_t_int
          Call poi48i(aa(1,nw0),ww(nw0),nw&
               &,vars(ivar-2_t_int)&
               &,vars(ivar-1_t_int)&
               &,vars(ivar)&
               &,ipgr)
       Case default

          Write(errbuf,1000)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          
       End Select

       If(prot)Then
          Write(outfile,10)nsubfunc,nw-nw0+1,ww(nw0)
       End If

       wsum=wsum+Sum(ww(nw0:nw))
       If(HEX)Then
          ww(nw0:nw)=weightshex(ip)*ww(nw0:nw)
       Else
          ww(nw0:nw)=weights(ip)*ww(nw0:nw)
       End If
       
    Enddo
    !!
    If(prot)Then
       Write(outfile,200) nsubfunc,nw
       Write(outfile,201) nsubfunc,wsum
    Endif
    !!
    Return
    !!
200 Format(a,': number of created irreducible points=',i4)
201 Format(a,': number of total points              =',f12.4)
400 Format(a,': required mesh number    =',i4) 
401 Format(a,': required mesh symmetry  = hexagonal') 
402 Format(a,': required mesh symmetry  = cubic') 
403 Format(a,': required point group    =',i4) 
2   Format(a,': internal mesh index =',i4)
3   Format(a,':','np   =',i4)
4   Format(a,':','np0  =',i4)
5   Format(a,':','ivar =',i4)
6   Format(a,':','point=',i4)
!7   Format(a,':','ivar=',i4)
10  Format(a,':',t20,'#=',i4,t40,'mul=',f10.5)
1000 Format("'Unknown pointtype'")
1001 Format("'nv =",i5," not a valid argument!'")
  End Subroutine gsphirred
  !!***********************************************************************
  
End Module nsphintirred
