Module occu
  !!
  Use precisiondefs
  Use memory
  Use errors
  Use clock
  Use tkohnsham
  Use tsymmetry
  Use trelativistic
  Use torbitalbasis
  Use optionstest
  Use file_iounits
  Use tbzone
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="OCCU"  
  !!
  Public :: make_netoccus,rel_to_pseudo,make_ovloccus
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine make_netoccus(mspin,symmetry,structure,kohnsham,bweitk,coccu&
       &,nkp,basis,relativistic&
       &,noncc,noncv,nonvv&
       &,roncc,roncv,ronvv,projected)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry), Intent(in) :: symmetry
    Type(T_Structure), Intent(in):: structure
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Logical(t_log),Intent(in)    :: projected
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: nkp
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(in)      :: coccu(:,:,:)
    Type(T_Basis),Intent(in)     :: basis
    Real(t_real),Intent(out)     :: noncc(:,:,:,:),noncv(:,:,:,:)
    Real(t_real),Intent(out)     :: nonvv(:,:,:,:)
    Complex(t_real),Intent(out)  :: roncc(:,:,:,:),roncv(:,:,:,:)
    Complex(t_real),Intent(out)  :: ronvv(:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="make_netoccus"
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    !!
    If(relativistic== FULL_RELATIVISTIC)Then
       roncc=c0
       ronvv=c0
       roncv=c0
       Call make_frel_netoccus(mspin,symmetry,structure,kohnsham,bweitk,coccu&
            &,nkp,basis,roncc,roncv,ronvv,projected)
    Else
       noncc=0.0_t_real
       nonvv=0.0_t_real
       noncv=0.0_t_real
       Call make_nrel_netoccus(mspin,symmetry,structure,kohnsham,bweitk,coccu&
            &,nkp,basis,noncc,noncv,nonvv)
    Endif
    !!
    Return
    !!
  End Subroutine make_netoccus
  !!***********************************************************************
  Subroutine make_frel_netoccus(mspin,symmetry,structure,kohnsham,bweitk&
       &,coccu,nkp,basis,roncc,roncv,ronvv,projected)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin
    Type(T_Structure), Intent(in):: structure
    Type(T_Symmetry), Intent(in) :: symmetry
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Logical(t_log),Intent(in)    :: projected
    Integer(t_int),Intent(in)    :: nkp
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(in)      :: coccu(:,:,:)
    Complex(t_real),Intent(out)  :: roncc(:,:,:,:),roncv(:,:,:,:)
    Complex(t_real),Intent(out)  :: ronvv(:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="make_frel_netoccus"
    Integer(t_int) :: nband,isa,iv,i,jv,j,ik,n13,ic,jc
    Integer(t_int) :: lenchi,lenc,lenv,istc,istv,nns,ms
    Logical(t_log) :: fbz
    Complex(t_real):: c
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    fbz=isoption('FULLBZ')
    !!
    nband=kohnsham% nband
    !!
    nns=1
    if(projected)nns=mspin
    !!
    isaloop: Do isa=1,structure% nsite
       !! 
       lenchi=kohnsham% p_chiblocklength(isa)
       lenc=kohnsham% p_cblocklength(isa)
       lenv=kohnsham% p_vblocklength(isa)
       istc=kohnsham% p_cblockstart(isa)-1
       istv=kohnsham% p_vblockstart(isa)-1
       !!
       msloop1: Do ms=1,nns
          iv=istv
          Do i=1,lenv
             iv=iv+1
             jv=istv
             Do j=1,lenv
                jv=jv+1
                c=c0
                Do ik=1,nkp
                   c=c+Sum(&
                        &kohnsham% p_ccvk(1:nband,ik,iv,1)&
                        &*Conjg(kohnsham% p_ccvk(1:nband,ik,jv,1))&
                        &*bweitk(1:nband,ik,ms))
                Enddo
                ronvv(i,j,isa,ms)=c
             End Do
          End Do
          !! core val
          ic=istc
          Do i=1,lenc
             ic=ic+1
             jv=istv
             Do j=1,lenv
                jv=jv+1
                c=c0
                Do ik=1,nkp
                   c=c+Sum(&
                        &kohnsham% p_ccck(1:nband,ik,ic,1)&
                        &*Conjg(kohnsham% p_ccvk(1:nband,ik,jv,1))&
                        &*bweitk(1:nband,ik,ms))
                Enddo
                roncv(i,j,isa,ms)=c
             End Do
          End Do
          !! core core
          ic=istc
          Do i=1,lenc
             ic=ic+1
             jc=istc
             Do j=1,lenc
                jc=jc+1
                c=c0
                Do ik=1,nkp
                   c=c+Sum(&
                        &kohnsham% p_ccck(1:nband,ik,ic,1)&
                        &*Conjg(kohnsham% p_ccck(1:nband,ik,jc,1))&
                        &*bweitk(1:nband,ik,ms))
                Enddo
                c=c+Sum(     kohnsham% p_rczz(i,1:lenchi,isa)&
                     &*Conjg(kohnsham% p_rczz(j,1:lenchi,isa))&
                     &      *coccu(1:lenchi,isa,ms))
                roncc(i,j,isa,ms)=c
             End Do
          End Do
       End Do msloop1
       !!
    End Do isaloop
    !!
    If(.Not. fbz)Then
       n13=Lbound(symmetry% Rwu_tj_ks,3)
       Do ms=1,nns
          Call symmetrize_frel_occus(structure% nsite,structure% p_isrt&
               &,basis% p_nRv,basis% p_kapRv&
               &,basis% p_nRv,basis% p_kapRv&
               &,symmetry% Rnumops,structure% p_equivsites&
               &,symmetry% Roptimerev&
               &,n13,symmetry% Rwu_tj_ks&
               &,ronvv(:,:,:,ms))
          
          Call symmetrize_frel_occus(structure% nsite,structure% p_isrt&
               &,basis% p_nRc,basis% p_kapRc&
               &,basis% p_nRv,basis% p_kapRv&
               &,symmetry% Rnumops,structure% p_equivsites&
               &,symmetry% Roptimerev&
               &,n13,symmetry% Rwu_tj_ks&
               &,roncv(:,:,:,ms))
          
          Call symmetrize_frel_occus(structure% nsite,structure% p_isrt&
               &,basis% p_nRc,basis% p_kapRc&
               &,basis% p_nRc,basis% p_kapRc&
               &,symmetry% Rnumops,structure% p_equivsites&
               &,symmetry% Roptimerev&
               &,n13,symmetry% Rwu_tj_ks&
               &,roncc(:,:,:,ms))
       Enddo
    Endif
    !!
    Return
    !!
  End Subroutine make_frel_netoccus
  !!***********************************************************************
  Subroutine symmetrize_frel_occus(nsite,isrt&
       &,nRa,kapRa,nRb,kapRb&
       &,numops,equivsites,optimerev,n13,wu_ks&
       &,on)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,isrt(:),n13
    Integer(t_int),Intent(in)    :: nRa(:),kapRa(:,:)
    Integer(t_int),Intent(in)    :: nRb(:),kapRb(:,:)
    Integer(t_int),Intent(in)    :: numops,equivsites(:,:),optimerev(:)
    Complex(t_real),Intent(in)   :: wu_ks(:,:,n13:,:)
    Complex(t_real),Intent(inout):: on(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_frel_occus"
    Integer(t_int) :: isa,issa,lena,lenb,isga,jops,m1,m2
    Logical(t_log) :: timerev
    Real(t_real)   :: norm
    Complex(t_real),Pointer :: p_on(:,:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    !! Oin,Rjm=norm * sum_g ( K_{0g}*wuT* Ogin,[DPRji-gj+gi]gjm)*wu*K_{0g} )
    !!
    !!
    Nullify(p_on)
    !!
    m1=Size(on,1)
    m2=Size(on,2)
    Call palloc("p_on",p_on,1_t_int,m1,1_t_int,m2,1_t_int,nsite&
         &,nsubfunc,nmodule)
    !!
    !! onsite
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    p_on(:,:,:)=c0
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*Abs(kapRa(1:nRa(issa),issa)))
       lenb=Sum(2*Abs(kapRb(1:nRb(issa),issa)))
       !!
       ojopsloop: Do jops=1,numops
          timerev=(optimerev(jops)==1)
          isga=equivsites(jops,isa)
          !!
          !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
          !!
          Call symmetrize_ron_block(&
               &nRa,kapRa,nRb,kapRb&
               &,on(:,:,isga)&
               &,p_on(:,:,isa)&
               &,lena,lenb&
               &,issa,issa,jops,timerev,n13,wu_ks)
          !!
       End Do ojopsloop
       !!
    End Do isaloop
    !!
    on(:,:,:)=p_on(:,:,:)*norm
    !!
    Call pfree(p_on)
    !!
    Return
    !!
  End Subroutine symmetrize_frel_occus
  !!***********************************************************************
  Subroutine symmetrize_ron_block(&
       &nRa,kapRa,nRb,kapRb&
       &,on,lon&
       &,lena,lenb&
       &,issa,issb,jops,timerev,n13,wu_ks)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nRa(:),kapRa(:,:),n13
    Integer(t_int),Intent(in)    :: nRb(:),kapRb(:,:)
    Logical(t_log),Intent(in)    :: timerev
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: issa,issb
    Integer(t_int),Intent(in)    :: jops
    Complex(t_real),Intent(in)   :: wu_ks(:,:,n13:,:)
    Complex(t_real),Intent(in)   :: on(:,:)
    Complex(t_real),Intent(inout):: lon(:,:)
    !!LOCAL
    Integer(t_int) :: iva,dega,kapa,degb,kapb,ivb,jj
    Complex(t_real):: A(lena,lenb)
    !!
    !!
    !!
    !! Oin,Pjm=K_{0g}*( wuT* Ogin,DPgjm)*wu )*K_{0g}
    jj=0
    Do ivb=1,nRb(issb)
       kapb=kapRb(ivb,issb)
       degb=2*Abs(kapb)
       !!
       A(1:lena,jj+1:jj+degb)=Matmul(on(1:lena,jj+1:jj+degb)&
            &             ,wu_ks(1:degb,1:degb,kapb,jops))
       jj=jj+degb
    Enddo
    !!
    jj=0
    Do iva=1,nRa(issa)
       kapa=kapRa(iva,issa)
       dega=2*Abs(kapa)
       !!
       If(timerev)Then
          lon(jj+1:jj+dega,1:lenb)=lon(jj+1:jj+dega,1:lenb)&
               &+Matmul(Transpose(wu_ks(1:dega,1:dega,kapa,jops))&
               & ,Conjg(A(jj+1:jj+dega,1:lenb)))
       Else
          lon(jj+1:jj+dega,1:lenb)=lon(jj+1:jj+dega,1:lenb)&
               &+Matmul(Conjg(Transpose(wu_ks(1:dega,1:dega,kapa,jops)))&
               & ,A(jj+1:jj+dega,1:lenb))
       Endif
       !!
       jj=jj+dega
    Enddo
    !!
    !!
  End Subroutine symmetrize_ron_block
  !!***********************************************************************
  Subroutine make_nrel_netoccus(mspin,symmetry,structure,kohnsham&
       &,bweitk,coccu&
       &,nkp,basis,noncc,noncv,nonvv)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin
    Type(T_Structure), Intent(in):: structure
    Type(T_Symmetry), Intent(in) :: symmetry
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Integer(t_int),Intent(in)    :: nkp
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(in)      :: coccu(:,:,:)
    Type(T_Basis),Intent(in)     :: basis
    Real(t_real),Intent(out)     :: noncc(:,:,:,:),noncv(:,:,:,:)
    Real(t_real),Intent(out)     :: nonvv(:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="make_nrel_netoccus"
    Integer(t_int) :: nband,isa,ms,iv,jv,i,j,ik,ic,jc,lenchi,lenc,lenv
    Integer(t_int) :: istc,istv
    Logical(t_log) :: fbz
    Real(t_real)   :: d
    !!
    fbz=isoption('FULLBZ')
    !!
    nband=kohnsham% nband
    !!
    !!
    Do isa=1,structure% nsite
       !!
       lenchi=kohnsham% p_chiblocklength(isa)
       lenc=kohnsham% p_cblocklength(isa)
       lenv=kohnsham% p_vblocklength(isa)
       istc=kohnsham% p_cblockstart(isa)-1
       istv=kohnsham% p_vblockstart(isa)-1
       !!
       Do ms=1,mspin
          iv=istv
          Do i=1,lenv
             iv=iv+1
             jv=istv
             Do j=1,lenv
                jv=jv+1
                d=0.0_t_real
                Do ik=1,nkp
                   d=d+Sum(&
                        &Real(kohnsham% p_ccvk(1:nband,ik,iv,ms)&
                        &*Conjg(kohnsham% p_ccvk(1:nband,ik,jv,ms))&
                        &*bweitk(1:nband,ik,ms),t_real))
                Enddo
                nonvv(i,j,isa,ms)=d
             Enddo
          Enddo
          !! core val
          ic=istc
          Do i=1,lenc
             ic=ic+1
             jv=istv
             Do j=1,lenv
                jv=jv+1
                d=0.0_t_real
                Do ik=1,nkp
                   d=d+Sum(&
                        &Real(kohnsham% p_ccck(1:nband,ik,ic,ms)&
                        &*Conjg(kohnsham% p_ccvk(1:nband,ik,jv,ms))&
                        &*bweitk(1:nband,ik,ms),t_real))
                Enddo
                noncv(i,j,isa,ms)=d
             Enddo
          Enddo
          !! core core and core
          ic=istc
          Do i=1,lenc
             ic=ic+1
             jc=istc
             Do j=1,lenc
                jc=jc+1
                d=0.0_t_real
                Do ik=1,nkp
                   d=d+Sum(&
                        &Real(kohnsham% p_ccck(1:nband,ik,ic,ms)&
                        &*Conjg(kohnsham% p_ccck(1:nband,ik,jc,ms))&
                        &*bweitk(1:nband,ik,ms),t_real))
                Enddo

                d=d+Sum(&
                     &     kohnsham% p_nczz(i,1:lenchi,isa,ms)&
                     &    *kohnsham% p_nczz(j,1:lenchi,isa,ms)&
                     &     *coccu(1:lenchi,isa,ms)&
                     &)

                noncc(i,j,isa,ms)=d
                
             Enddo
          Enddo

       Enddo !! ms
       !!
    Enddo !! isa
    !!
    If(.Not. fbz)Then
       Call symmetrize_nrel_netoccus(mspin,structure% nsite,structure% p_isrt&
            &,basis% p_nRv,basis% p_lRv&
            &,basis% p_nRv,basis% p_lRv&
            &,symmetry% Rnumops,structure% p_equivsites,symmetry% Rwu_ks&
            &,nonvv)
       !!
       Call symmetrize_nrel_netoccus(mspin,structure% nsite,structure% p_isrt&
            &,basis% p_nRc,basis% p_lRc&
            &,basis% p_nRv,basis% p_lRv&
            &,symmetry% Rnumops,structure% p_equivsites,symmetry% Rwu_ks&
            &,noncv)

       Call symmetrize_nrel_netoccus(mspin,structure% nsite,structure% p_isrt&
            &,basis% p_nRc,basis% p_lRc&
            &,basis% p_nRc,basis% p_lRc&
            &,symmetry% Rnumops,structure% p_equivsites,symmetry% Rwu_ks&
            &,noncc)
    Endif
    !!
    !!
    Return
    !!
  End Subroutine make_nrel_netoccus
  !!***********************************************************************
  Subroutine symmetrize_nrel_netoccus(mspin,nsite,isrt&
       &,nRa,lRa,nRb,lRb&
       &,numops,equivsites,wu_ks&
       &,on)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: nsite,isrt(:)
    Integer(t_int),Intent(in)    :: nRa(:),lRa(:,:)
    Integer(t_int),Intent(in)    :: nRb(:),lRb(:,:)
    Integer(t_int),Intent(in)    :: numops,equivsites(:,:)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(inout)   :: on(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_nrel_netoccus"
    Integer(t_int) :: isa,issa,lena,lenb,isga,jops,m1,m2,ms
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_on(:,:,:,:)
    !!
    !! Oin,Rjm=norm * sum_g ( wuT* Ogin,[DPRji-gj+gi]gjm)*wu )
    !!
    Nullify(p_on)
    !!
    m1=Size(on,1)
    m2=Size(on,2)
    Call palloc("p_on",p_on,1_t_int,m1,1_t_int,m2,1_t_int,nsite&
         &,1_t_int,mspin,nsubfunc,nmodule)
    !!
    !! onsite
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    p_on(:,:,:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       lenb=Sum(2*lRb(1:nRb(issa),issa)+1)
       !!
       ojopsloop: Do jops=1,numops
          isga=equivsites(jops,isa)
          !!
          !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
          !!
          Do ms=1,mspin
             Call symmetrize_on_block(&
                  &nRa,lRa,nRb,lRb&
                  &,on(:,:,isga,ms)&
                  &,p_on(:,:,isa,ms)&
                  &,lena,lenb&
                  &,issa,issa,jops,wu_ks)
          Enddo
          !!
       End Do ojopsloop
       !!
    End Do isaloop
    !!
    on(:,:,:,:)=p_on(:,:,:,:)*norm
    !!
    Call pfree(p_on)
    !!
    Return
    !!
  End Subroutine symmetrize_nrel_netoccus
  !!***********************************************************************
  Subroutine make_ovloccus(mspin,symmetry,structure,lattice,basis&
       &,bzone,kohnsham,bweitk,relativistic,onc,onv,projected,symmetrize)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice), Intent(in)  :: lattice
    Type(T_Basis), Intent(in)    :: basis
    Type(T_Bzone), Intent(in)    :: bzone
    Type(T_Kohnsham), Intent(in) :: kohnsham
    Logical(t_log),Intent(in)    :: projected !! for energy resolved
    !! densities, we need spin projected version and hence twice
    !! the effort. The data will be organized  bit different from 
    !! the normal SCF density calculation
    Logical(t_log),Intent(in)    :: symmetrize
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: relativistic
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(out)     :: onc(:,:,:,:,:,:)
    Real(t_real),Intent(out)     :: onv(:,:,:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="make_ovloccus"
    Character(*),Parameter  :: cpumesg_occu='Occupation numbers'
    Character(*),Parameter  :: cpumesg_sym='Symmetrize occus'
    Logical(t_log) :: rel
    !!
    Call watch(cpumesg_occu,cpu_start,reset=.True.)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    onc(:,:,:,:,:,:)=0.0_t_real
    onv(:,:,:,:,:,:)=0.0_t_real
    !!
    If(rel)Then
       Call rel_ovloccu(structure% nsite,structure% p_isrt&
            &,mspin&
            &,lattice% p_npairs,lattice% npairsmax,lattice% p_pairs&
            &,bzone% nkp,bzone% p_bk,bweitk&
            &,kohnsham% p_ccck&
            &,kohnsham% p_ccvk&
            &,kohnsham% nband,kohnsham% ncsmx,kohnsham% nvsmx&
            &,kohnsham% p_vblockstart,kohnsham% p_vblocklength&
            &,kohnsham% p_cblockstart,kohnsham% p_cblocklength&
            &,basis% lmx,basis% p_relTOpseudo&
            &,basis% p_nRv,basis% p_kapRv&
            &,basis% p_nRc,basis% p_kapRc&
            &,onc,onv,projected)
    Else
       Call nrel_ovloccu(structure% nsite&
            &,mspin&
            &,lattice% p_npairs,lattice% npairsmax,lattice% p_pairs&
            &,bzone% nkp,bzone% p_bk,bweitk&
            &,kohnsham% p_ccck,kohnsham% p_ccvk&
            &,kohnsham% nband&
            &,kohnsham% p_vblockstart,kohnsham% p_vblocklength&
            &,kohnsham% p_cblockstart,kohnsham% p_cblocklength&
            &,onc,onv)
    Endif
    !!
    Call watch(cpumesg_occu,cpu_stop,delete=.True.,Print=.True.)
    !!
    If(symmetrize)Then
       Call watch(cpumesg_sym,cpu_start,reset=.True.)
       Call symmetrize_nrel_ovloccus(mspin,symmetry,structure&
            &,lattice,basis,onc,onv)
       Call watch(cpumesg_sym,cpu_stop,delete=.True.,Print=.True.)
    Endif
    !!
    !!
    Return
    !!
  End Subroutine make_ovloccus
  !!***********************************************************************
  Subroutine nrel_ovloccu(nsite,mspin&
       &,npairs,npairsmax,pairs&
       &,nkp,bk,bweitk&
       &,ccck,ccvk&
       &,nband&
       &,vblockstart,vblocklength,cblockstart,cblocklength&
       &,onc,onv)
    !!
    !! make overlap occupation numbers
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin
    Integer(t_int),Intent(in)    :: nkp,npairsmax
    Integer(t_int),Intent(in)    :: npairs(:,:)
    Integer(t_int),Intent(in)    :: vblockstart(:),cblockstart(:)
    Integer(t_int),Intent(in)    :: vblocklength(:),cblocklength(:)
    Integer(t_int),Intent(in)    :: nband
    Real(t_real),Intent(in)      :: bk(:,:)
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Complex(t_real),Intent(in)   :: ccvk(:,:,:,:)
    Complex(t_real),Intent(in)   :: ccck(:,:,:,:)
    Real(t_real),Intent(inout)   :: onc(:,:,:,:,:,:)
    Real(t_real),Intent(inout)   :: onv(:,:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="nrel_ovloccu"
    Integer(t_int) :: isa,isb,istva,istca,istvb,lena,i,lenc,lenb
    Integer(t_int) :: np,ir,ik,j,ms
    Real(t_real)   :: rk
    Real(t_real)   :: rr(3)
    Complex(t_real):: rex
    Complex(t_real),Pointer:: eikr(:,:)
    !!
    Nullify(eikr)
    !!
    Call palloc('eikr', eikr ,1_t_int,Max(1_t_int,npairsmax) ,1_t_int,nkp&
         &,nmodule,nsubfunc)
    !!
    !!
    !!
    !!
    !!
    !!
    Do isa=1_t_int,nsite
       istva=vblockstart(isa)-1
       istca=cblockstart(isa)-1
       lena=vblocklength(isa)
       lenc=cblocklength(isa)
       Do isb=1_t_int,nsite
          istvb=vblockstart(isb)-1
          lenb=vblocklength(isb)
          np=npairs(isb,isa)
          Do ir=1,np
             rr(:)=pairs(:,ir,isb,isa)
             Do ik=1,nkp
                rk=Dot_product(rr,bk(:,ik))
                eikr(ir,ik)=Exp(Cmplx(0.0_t_real,-rk,t_real))
             End Do !ik
          End Do !ir
          !!
          !! valence - valence
          !!
          Do ms=1,mspin
             Do j=1,lenb
                Do i=1,lena
                   Do ik=1,nkp
                      rex=Sum(ccvk(1:nband,ik,i+istva,ms)&
                           &*Conjg(ccvk(1:nband,ik,j+istvb,ms))&
                           &*bweitk(1:nband,ik,ms))
                      Do ir=1,np
                         onv(i,j,isa,isb,ms,ir)=onv(i,j,isa,isb,ms,ir)&
                              &+Real(rex*eikr(ir,ik),t_real)
                      Enddo ! ir
                   Enddo ! ik
                Enddo ! i
             Enddo ! j
          Enddo ! ms
          !!
          !! core - valence
          !!
          Do ms=1,mspin
             Do j=1,lenb
                Do i=1,lenc
                   Do ik=1,nkp
                      rex=Sum(ccck(1:nband,ik,i+istca,ms)&
                           &*Conjg(ccvk(1:nband,ik,j+istvb,ms))&
                           &*bweitk(1:nband,ik,ms))
                      Do ir=1,np
                         onc(i,j,isa,isb,ms,ir)=onc(i,j,isa,isb,ms,ir)&
                              &+Real(rex*eikr(ir,ik),t_real)
                      Enddo            ! ir
                   Enddo ! ik
                Enddo ! i
             Enddo ! j
          Enddo ! ms
          !!
       Enddo ! isb
    Enddo ! isa
    !!
    !!
    Call pfree(eikr)
    !!
    Return
    !!
  End Subroutine nrel_ovloccu
  !!***********************************************************************
  Subroutine rel_ovloccu(nsite,isrt,mspin&
       &,npairs,npairsmax,pairs&
       &,nkp,bk,bweitk&
       &,ccck,ccvk&
       &,nband,ncsmx,nvsmx&
       &,vblockstart,vblocklength,cblockstart,cblocklength&
       &,lmx,relTOpseudo&
       &,nRv,kapRv,nRc,kapRc&
       &,onc,onv,projected)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: projected !! for energy resolved
    !! densities, we need spin projected version and hence twice
    !! the effort. The data will be organized  bit different from 
    !! the normal SCF density calculation
    Integer(t_int),Intent(in)    :: nsite,mspin,nvsmx,ncsmx
    Integer(t_int),Intent(in)    :: nkp,npairsmax
    Integer(t_int),Intent(in)    :: isrt(:)
    Integer(t_int),Intent(in)    :: npairs(:,:),lmx
    Integer(t_int),Intent(in)    :: vblockstart(:),cblockstart(:)
    Integer(t_int),Intent(in)    :: vblocklength(:),cblocklength(:)
    Integer(t_int),Intent(in)    :: nband
    Integer(t_int),Intent(in)    :: nRv(:),kapRv(:,:)
    Integer(t_int),Intent(in)    :: nRc(:),kapRc(:,:)
    Real(t_real),Intent(in)      :: bk(:,:)
    Real(t_real),Intent(in)      :: bweitk(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Complex(t_real),Intent(in)   :: relTOpseudo(2*lmx+3,2*lmx+2,2,-lmx-1:lmx+1)
    Complex(t_real),Intent(in)   :: ccvk(:,:,:,:)
    Complex(t_real),Intent(in)   :: ccck(:,:,:,:)
    Real(t_real),Intent(inout)   :: onc(:,:,:,:,:,:)
    Real(t_real),Intent(inout)   :: onv(:,:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rel_ovloccu"
    Logical(t_log) :: relmag
    Integer(t_int) :: isa,isb,issa,issb,istva,istca,istvb,lena,i,lenc,lenb
    Integer(t_int) :: np,ir,ik,j,iv,jv,ic,nns,ms
    Real(t_real)   :: rk
    Real(t_real)   :: rr(3)
    Complex(t_real):: rex
    Complex(t_real),Pointer:: eikr(:,:)
    Complex(t_real),Pointer:: onnv(:,:,:)
    Complex(t_real),Pointer:: onnc(:,:,:)
    Real(t_real),Pointer :: onnvms(:,:,:)
    Real(t_real),Pointer :: onncms(:,:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    Nullify(onnv,onnc,onnvms,onncms,eikr)
    !!
    Call palloc('eikr', eikr ,1_t_int,Max(1_t_int,npairsmax) ,1_t_int,nkp&
         &,nmodule,nsubfunc)
    Call palloc('onnv', onnv ,1_t_int,nvsmx,1_t_int,nvsmx&
         &,1_t_int,Max(1_t_int,npairsmax),nmodule,nsubfunc)
    Call palloc('onnc', onnc ,1_t_int,ncsmx,1_t_int,nvsmx&
         &,1_t_int,Max(1_t_int,npairsmax),nmodule,nsubfunc)
    !!
    If(projected)Then
       Call palloc('onnvms', onnvms ,1_t_int,nvsmx&
            &,1_t_int,nvsmx,1_t_int,mspin,nmodule,nsubfunc)
       Call palloc('onncms', onncms ,1_t_int,ncsmx&
            &,1_t_int,nvsmx,1_t_int,mspin,nmodule,nsubfunc)
    Endif

    !!
    relmag=(mspin==2)
    !!
    !!
    !!
    nns=1
    if(projected)nns=mspin
    !!
    Do isa=1_t_int,nsite
       issa=isrt(isa)

       istva=vblockstart(isa)-1
       istca=cblockstart(isa)-1

       lena=vblocklength(isa)
       lenc=cblocklength(isa)

       !! setup kappa  array


       
       Do isb=1_t_int,nsite
          issb=isrt(isb)
          
          istvb=vblockstart(isb)-1

          lenb=vblocklength(isb)
             
          np=npairs(isb,isa)
          Do ir=1_t_int,np
             rr(:)=pairs(:,ir,isb,isa)
             Do ik=1_t_int,nkp
                rk=Dot_product(rr,bk(:,ik))
                eikr(ir,ik)=Exp(Cmplx(0.0_t_real,-rk,t_real))
             End Do !ik
          End Do !ir

          msloop1: Do ms=1,nns
             
             onnv(:,:,1:np)=c0
             
             !! valence - valence
             jv=istvb
             Do j=1,lenb
                jv=jv+1
                iv=istva
                Do i=1,lena
                   iv=iv+1
                   Do ik=1,nkp
                      rex=Sum(ccvk(1:nband,ik,iv,1)&
                           &*Conjg(ccvk(1:nband,ik,jv,1))&
                           &*bweitk(1:nband,ik,ms))
                      Do ir=1,np
                         onnv(i,j,ir)=onnv(i,j,ir)+rex*eikr(ir,ik)
                      Enddo ! ir
                   Enddo ! ik
                Enddo ! j
             Enddo ! i

             projif1: If(projected)Then
                Do ir=1,np
                   Call rel_to_pseudo(nRv(issa),kapRv(:,issa)&
                        &,nRv(issb),kapRv(:,issb)&
                        &,lmx,relTOpseudo&
                        &,relmag&
                        &,onnv(:,:,ir)&
                        &,onnvms(:,:,1)&
                        &,onnvms(:,:,mspin))
                   onv(:,:,isa,isb,ms,ir)=onnvms(:,:,ms)
                Enddo
             Else projif1
                Do ir=1,np
                   Call rel_to_pseudo(nRv(issa),kapRv(:,issa)&
                        &,nRv(issb),kapRv(:,issb)&
                        &,lmx,relTOpseudo&
                        &,relmag&
                        &,onnv(:,:,ir)&
                        &,onv(:,:,isa,isb,1,ir)&
                        &,onv(:,:,isa,isb,mspin,ir))
                Enddo
             Endif projif1
          Enddo msloop1
          
          msloop2: Do ms=1,nns
             onnc(:,:,1:np)=c0
          
             !! core - valence
             jv=istvb
             Do j=1,lenb
                jv=jv+1
                ic=istca
                Do i=1,lenc
                   ic=ic+1
                   Do ik=1,nkp
                      rex=Sum(ccck(1:nband,ik,ic,1)&
                           &*Conjg(ccvk(1:nband,ik,jv,1))&
                           &*bweitk(1:nband,ik,ms))
                      Do ir=1,np
                         onnc(i,j,ir)=onnc(i,j,ir)+rex*eikr(ir,ik)
                      Enddo            ! ir
                   Enddo ! ik
                Enddo ! j
             Enddo ! i

             projif2: If(projected)Then
                Do ir=1,np
                   Call rel_to_pseudo(nRc(issa),kapRc(:,issa)&
                        &,nRv(issb),kapRv(:,issb)&
                        &,lmx,relTOpseudo&
                        &,relmag&
                        &,onnc(:,:,ir)&
                        &,onncms(:,:,1)&
                        &,onncms(:,:,mspin))
                   onc(:,:,isa,isb,ms,ir)=onncms(:,:,ms)
                Enddo
             Else projif2
                Do ir=1,np
                   Call rel_to_pseudo(nRc(issa),kapRc(:,issa)&
                        &,nRv(issb),kapRv(:,issb)&
                        &,lmx,relTOpseudo&
                        &,relmag&
                        &,onnc(:,:,ir)&
                        &,onc(:,:,isa,isb,1,ir)&
                        &,onc(:,:,isa,isb,mspin,ir))
                Enddo
             Endif projif2
          Enddo msloop2


       Enddo ! isb
    Enddo ! isa
    !!
    !!
    Call pfree(onnvms)
    Call pfree(onncms)
    Call pfree(onnv)
    Call pfree(onnc)
    Call pfree(eikr)
    !!
    Return
    !!
  End Subroutine rel_ovloccu
  !!***********************************************************************
  Subroutine symmetrize_nrel_ovloccus(mspin,symmetry,structure,lattice,basis&
       &,onc,onv)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(inout)   :: onc(:,:,:,:,:,:)
    Real(t_real),Intent(inout)   :: onv(:,:,:,:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="symmetrize_nrel_ovloccus"
    Integer(t_int),Pointer :: p_gdiff(:,:,:,:)
    !!
    Nullify(p_gdiff)
    !!
    If(structure% str_type==STR_TYPE_MOLECULE) Return
    !!
    Call gvecdifoccu(structure% nsite,lattice% p_npairs,lattice% npairsmax&
         &,lattice% p_pairs&
         &,symmetry% Rnumops,structure% p_equivsites,symmetry% Ropmat&
         &,p_gdiff)
    !!
    !!
    Call symmetrize_nrel_ovloccu(mspin,structure% nsite,structure% p_isrt&
         &,lattice% npairsmax,lattice% p_npairs&
         &,symmetry% Rnumops,structure% p_equivsites&
         &,symmetry% Rwu_ks&
         &,basis% p_nRv,basis% p_lRv&
         &,basis% p_nRv,basis% p_lRv&
         &,p_gdiff,onv)
    !!
    Call symmetrize_nrel_ovloccu(mspin,structure% nsite,structure% p_isrt&
         &,lattice% npairsmax,lattice% p_npairs&
         &,symmetry% Rnumops,structure% p_equivsites&
         &,symmetry% Rwu_ks&
         &,basis% p_nRc,basis% p_lRc&
         &,basis% p_nRv,basis% p_lRv&
         &,p_gdiff,onc)
    !!
    Call pfree(p_gdiff)
    !!
    Return
    !!
    !!
  End Subroutine symmetrize_nrel_ovloccus 
  !!***********************************************************************
  Subroutine symmetrize_nrel_ovloccu(mspin,nsite,isrt&
       &,npairsmax,npairs&
       &,numops,equivsites,wu_ks&
       &,nRa,lRa,nRb,lRb&
       &,gdiff,on)
    
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,mspin,isrt(:)
    Integer(t_int),Intent(in)    :: numops,equivsites(:,:)
    Integer(t_int),Intent(in)    :: nRa(:),lRa(:,:)
    Integer(t_int),Intent(in)    :: nRb(:),lRb(:,:)
    Integer(t_int),Intent(in)    :: npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: gdiff(0:npairsmax,48,nsite,nsite)
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(inout)   :: on(:,:,:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="symmetrize_nrel_ovloccu"
    Integer(t_int) :: isa,isb,issa,issb,isga,isgb,irba,irgbga,lena,lenb
    Integer(t_int) :: jops,ms,m1,m2
    Real(t_real)   :: norm
    Real(t_real),Pointer :: p_on(:,:,:,:,:,:)
    !!
    !!
    !! Oin,Rjm=norm * sum_g ( wuT* Ogin,[DPRji-gj+gi]gjm)*wu )
    !!
    Nullify(p_on)
    !!
    m1=Size(on,1)
    m2=Size(on,2)
    Call palloc("p_on",p_on,1_t_int,m1,1_t_int,m2&
         &,1_t_int,nsite,1_t_int,nsite,1_t_int,mspin&
         &,1_t_int,Max(1_t_int,npairsmax),nsubfunc,nmodule)
    !!
    !! offsite
    !!
    norm=1.0_t_int/Real(numops,t_real)
    !!
    !!
    !! 
    !!
    p_on(:,:,:,:,:,:)=0.0_t_real
    isaloop: Do isa=1,nsite
       !!
       issa=isrt(isa)
       lena=Sum(2*lRa(1:nRa(issa),issa)+1)
       !!
       isbloop: Do isb=1,nsite
          !!
          issb=isrt(isb)
          lenb=Sum(2*lRb(1:nRb(issb),issb)+1)
          !!
          irbaloop: Do irba=1,npairs(isb,isa)
             !!
!!$             Call printmat1('onIN',"(f12.5,' ')","('     0       ')"&
!!$                  &,on(1:lena,1:lenb,isa,isb,1,irba),lena,lenb&
!!$                  &,1.0e-6_t_real)

             !!
             ojopsloop: Do jops=1,numops
                isga=equivsites(jops,isa)
                isgb=equivsites(jops,isb)
                !!
                irgbga=gdiff(irba,jops,isb,isa)
                !!
                !!
                !!
                !! Oin,im+=1/Ng * ( wuT* Ogin,gim)*wu )
                !!
                !Write(outfile,22)isga,isgb,irgbga
                Do ms=1,mspin
                   Call symmetrize_on_block(&
                        &nRa,lRa,nRb,lRb&
                        &,on(:,:,isga,isgb,ms,irgbga)&
                        &,p_on(:,:,isa,isb,ms,irba)&
                        &,lena,lenb&
                        &,issa,issb,jops,wu_ks)
                Enddo
                
                !!
                !!
             Enddo ojopsloop
             !!
             !!
          End Do irbaloop
       End Do isbloop
    End Do isaloop
    !!
    on(:,:,:,:,:,:)=p_on(:,:,:,:,:,:)*norm
    !!
    Call pfree(p_on)
    !!
    Return
    !!
!21  Format('ON: ','Sa =',i4,' Sb =',i4,' Pab  =',i4)
!22  Format('ON: ','Sga=',i4,' Sgb=',i4,' Pgbga=',i4)
    !!
  End Subroutine symmetrize_nrel_ovloccu
  !!***********************************************************************
  Subroutine symmetrize_on_block(&
       &nRa,lRa,nRb,lRb&
       &,on,lon&
       &,lena,lenb&
       &,issa,issb,jops,wu_ks)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nRa(:),lRa(:,:)
    Integer(t_int),Intent(in)    :: nRb(:),lRb(:,:)
    Integer(t_int),Intent(in)    :: lena,lenb
    Integer(t_int),Intent(in)    :: issa,issb
    Integer(t_int),Intent(in)    :: jops
    Real(t_real),Intent(in)      :: wu_ks(:,:,0:,:)
    Real(t_real),Intent(in)      :: on(:,:)
    Real(t_real),Intent(inout)   :: lon(:,:)
    !!LOCAL
    Integer(t_int) :: iva,dega,la,degb,lb,ivb,jj
    Real(t_real)   :: A(lena,lenb)
    !!
    !!
    !!
    !! Oin,Pjm=( wuT* Ogin,DPgjm)*wu )
    jj=0
    Do ivb=1,nRb(issb)
       lb=lRb(ivb,issb)
       degb=2*lb+1
       !!
       A(1:lena,jj+1:jj+degb)=Matmul(on(1:lena,jj+1:jj+degb)&
            &             ,wu_ks(1:degb,1:degb,lb,jops))
       jj=jj+degb
    Enddo
    !!
    jj=0
    Do iva=1,nRa(issa)
       la=lRa(iva,issa)
       dega=2*la+1
       !!
       lon(jj+1:jj+dega,1:lenb)=lon(jj+1:jj+dega,1:lenb)&
            &+Matmul(Transpose(wu_ks(1:dega,1:dega,la,jops))&
            & ,A(jj+1:jj+dega,1:lenb))
       !!
       jj=jj+dega
    Enddo
    !!
    !!
  End Subroutine symmetrize_on_block
  !!***********************************************************************
  Subroutine gvecdifoccu(nsite,npairs,npairsmax,pairs&
       &,numops,equivsites,opmat,p_gdiff)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite
    Integer(t_int),Intent(in)    :: numops
    Integer(t_int),Intent(in)    :: equivsites(48,nsite)
    Integer(t_int),Intent(in)    :: npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Real(t_real),Intent(in)      :: opmat(3,3,48)
    Integer(t_int),Pointer       :: p_gdiff(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="gvecdifoccu"
    Integer(t_int) :: isa,isb,irba,jops,isga,isgb,irgbga,m,irba0,ir0
    Real(t_real)   :: rba(3),rgbga(3),DP(3)
    !!
    !!Write(outfile,"('START',a)")nsubfunc
    Call palloc('p_gdiff',p_gdiff&
         &,0_t_int,npairsmax&
         &,1_t_int,48_t_int&
         &,1_t_int,nsite&
         &,1_t_int,nsite&
         &,nmodule,nsubfunc)
    !!
    Write(outfile,1000) &
         & 1+npairsmax,48_t_int,nsite,nsite&
         &,(1+npairsmax)*48_t_int*(nsite)**2*4/1.0e6_t_real
1000 Format('Allocated p_gdiff(',i6,',',i6,',',i6,',',i6,') = ',f10.5&
          &,' MByte')
    !!
    isaloop: Do isa=1,nsite
       !!Write(outfile,"('isa=',i5)")isa
       isbloop: Do isb=1,nsite
          !!Write(outfile,"('isb=',i5)")isb
          jopsloop: Do jops=1,numops
             !!Write(outfile,"('jops=',i5)")jops
             !! isga=G isa
             isga=equivsites(jops,isa)
             !! isgb=G isb
             isgb=equivsites(jops,isb)
             !!
             !!Write(outfile,"('isa=',i5,'isga=',i5)")isa,isga
             !!Write(outfile,"('isb=',i5,'isgb=',i5)")isb,isgb
             irba0=1
             If(isb==isa)Then
                irba0=0
             Endif
             irbaloop: Do irba=irba0,npairs(isb,isa)
                !!
                !! rab=R+Sa-Sb
                !!
                If(irba==0)Then
                   rba(:)=0.0_t_real
                Else
                   rba(:)=pairs(:,irba,isb,isa)
                Endif
                !!
                !!Write(outfile,"('rab=',3(f30.15))")rab

                !! \hat{D}(W)*rab =DR(W)*rab =rab*DR(W^-1)=rab*opmat
                !!
                !!
                DP=Matmul(rba,opmat(:,:,jops))
                !!
                !!Write(outfile,"('DP=',3(f30.15))")DP
                !! search irgagb such that pairs(:,isga,isgb,irgagb) ==DP
                irgbga=-1
                ir0=1
                If(isgb==isga)Then
                   ir0=0
                Endif
                mloop: Do m=ir0,npairs(isgb,isga)
                   !!
                   !! rgagb=R+Sga-Sgb
                   !!
                   If(m==0)Then
                      rgbga(:)=0.0_t_real
                   Else
                      rgbga(:)=pairs(:,m,isgb,isga)
                   Endif
                   !!
                   !!Write(outfile,"('rgagb   =',3(f30.15))")rgagb
                   !!Write(outfile,"('rgagb-DP=',3(f30.15))")rgagb-DP

                   !!
                   If(All(Abs(rgbga(:)-DP(:)) < 1.e-5_t_real)) Then
                      irgbga=m
                      Exit mloop
                   Endif
                   !!
                   !!
                End Do mloop
                !!
                If(irgbga==-1)Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'Could not find a symmetry related vector !'&
                        &,/,'Please report all calculational detail to the&
                        & authors!'")
                Endif
                !!
                p_gdiff(irba,jops,isb,isa)=irgbga
                !!
             End Do irbaloop
          End Do jopsloop
       Enddo isbloop
    Enddo isaloop
    !!
    !!Write(outfile,"('END',a)")nsubfunc
    Return
    !!
  End Subroutine gvecdifoccu
  !!***********************************************************************
  Subroutine rel_to_pseudo(nRva,kapRva,nRvb,kapRvb&
       &,lmx,TT,relmag,on,onv1,onv2)
    !!
    Implicit none  
    !!IO
    Logical(t_log),Intent(in)    :: relmag
    Integer(t_int),Intent(in)    :: lmx
    Integer(t_int),Intent(in)    :: nRva,kapRva(:)
    Integer(t_int),Intent(in)    :: nRvb,kapRvb(:)
    Complex(t_real),Intent(in)   :: on(:,:)
    Complex(t_real),Intent(in)   :: TT(2*lmx+3,2*lmx+2,2,-lmx-1:lmx+1)
    Real(t_real),Intent(out)     :: onv1(:,:),onv2(:,:)
    !!LOCAL
    Integer(t_int) :: i,ir0,iva,kapa,degja,ma,j,jr0,ivb,kapb,degjb,mb
    Integer(t_int) :: ms,dega,degb
    Real(t_real)   :: d(2)
    !!
    i=0
    ir0=0
    ivaloop: Do iva=1,nRva
       kapa=kapRva(iva)
       degja=2*Abs(kapa)
       dega  =Abs(2*kapa+1)
       maloop: Do ma=1,dega !! pseudo m_l
          i=i+1
          !!
          j=0
          jr0=0
          ivbloop: Do ivb=1,nRvb
             kapb=kapRvb(ivb)
             degjb=2*Abs(kapb)
             degb  =Abs(2*kapb+1)
             mbloop: Do mb=1,degb
                j=j+1
                !!
                msloop: Do ms=1,2
                   !!
                   !! Nab= C_{a} Exp(Rsa-Rsb)  C^{*}_{b} 
                   !! 
                   !! Nab^{s} = Ta^{s} Nba Tb^{s,+}
                   d(ms)=Real(&
                        &Dot_product(TT(mb,1:degjb,ms,kapb)&
                        &,Matmul(TT(ma,1:degja,ms,kapa)&
                        &,on(ir0+1:ir0+degja,jr0+1:jr0+degjb)))&
                        &,t_real)
                   !!
                Enddo msloop
                If(relmag)Then
                   onv1(i,j)=d(1)
                   onv2(i,j)=d(2)
                Else
                   onv1(i,j)=d(1)+d(2)
                Endif
                !!

             End Do mbloop
             jr0=jr0+degjb
          End Do ivbloop
       End Do maloop
       ir0=ir0+degja
    End Do ivaloop
    !!
    !!
    Return
    !!
  End Subroutine rel_to_pseudo
  !!***********************************************************************
End Module occu

