!!----DEBUG options------------------|
!! SUBFUNC           | MODES         |
!!-----------------------------------|
!! <empty>           |<empty>        |
!! calc_lop_kn       |               |
!! calc_sls          |               |
!! calc_racah_params !               !
!!-----------------------------------|
Module opcorrection
  !!
  Use precisiondefs
  Use constants
  Use trelativistic
  Use topc
  Use memory
  Use errors
  Use atomdefs
  Use tmesh
  Use tkohnsham
  Use optionstest
  Use file_iounits
  Use prnt
  Use tdiracdefs
  Use mesh
  Use tsymmetry
  Use diracdefs
  Use torbitalbasis
  Use integrate1d
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="OPCORRECTION"
  !!
  Public :: io_init_opc,calc_racah_params,opc_calc_o_cs,opc_calc_M_orb&
       &,opc_merge_h,calc_opc_etot,print_opc_params,opc_checkout&
       &,opc_checkin,opc_alloc,opc_free
  !!***********************************************************************
  Real(t_real)   :: Pcoef_3d(0:MAX_ELEMENT)
  Real(t_real)   :: Pcoef_4f(0:MAX_ELEMENT)
  Data Pcoef_3d(0:18)/ 19*0.0_t_real /
  Data Pcoef_3d(19)/ 46.4_t_real / !! for VCA interpolation
  Data Pcoef_3d(20:30)/ &
       !! Ca
       &  48.0_t_real&
       !! Sc
       &, 49.6_t_real&
       !! Ti
       &, 51.2_t_real&
       !! V
       &, 52.8_t_real&
       !! Cr
       &, 54.4_t_real&
       !! Mn
       &, 56.0_t_real&
       !! Fe
       &, 57.6_t_real&
       !! Co
       &, 59.2_t_real&
       !! Ni
       &, 60.8_t_real&
       !! Cu
       &, 62.4_t_real&
       !! Zn
       &, 64.0_t_real&
       &/
  Data Pcoef_3d(31)/ 65.6_t_real / !! for VCA interpolation
  !!
  Data Pcoef_3d(32:MAX_ELEMENT)/ 72*0.0_t_real /
  !!
  Data Pcoef_4f(0:55)/ 56*0.0_t_real /
  Data Pcoef_4f(56)/ 18.785715_t_real / !! for VCA interpolation 
  Data Pcoef_4f(57:71)/ &
       !! La
       & 19.000000_t_real&
       !! Ce
       &,19.214285_t_real&
       !! Pr
       &,19.428571_t_real&
       !!Nd
       &,19.642857_t_real&
       !! Pm
       &,19.857142_t_real&
       !! Sm
       &,20.071428_t_real&
       !! Eu
       &,20.285714_t_real&
       !! Gd
       &,20.500000_t_real&
       !! Tb
       &,20.714285_t_real&
       !! Dy
       &,20.928571_t_real&
       !! Ho
       &,21.142857_t_real&
       !! Er
       &,21.357142_t_real&
       !! Tm
       &,21.571428_t_real&
       !! Yb
       &,21.785714_t_real&
       !! Lu
       &,22.000000_t_real&
       & /
  Data Pcoef_4f(72)/ 22.214285_t_real / !! for VCA interpolation 
  Data Pcoef_4f(73:MAX_ELEMENT)/ 31*0.0_t_real /
  !!
Contains 
  !!***********************************************************************
  Subroutine io_init_opc(structure,basis,opc,BEB,relativistic)
    !!
    !!
    !! Initialization for orbital polarization correction (OPC).
    !! Orbitals and initial values for $B_{\vec{s}l\sigma}$ and
    !! $L_{\vec{s}l\sigma}$ are read from files {\em =.opc} and
    !! {\em +opc} (if it exists, otherwise initial polarization).
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_OPC),Intent(inout)    :: opc
    Logical(t_log),Intent(in)    :: BEB
    Integer(t_int),Intent(in)    :: relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="io_init_opc"
    Logical(t_log) :: rel,prot
    Integer(t_int) :: IOiodef,iodef,jodef,IOissao,nmain,l,ivo,i,isa,issa
    Integer(t_int) :: relfac,iz,deg,issao,ivr,isao,ms,kap,j
    Character(2)   :: state
    Integer(t_int),Pointer :: p_odefTOIOodef(:)
    !!
    !!
    Nullify(p_odefTOIOodef)
    !!
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    relfac=1_t_int
    If(rel)Then
       relfac=2_t_int
    Endif


    !!
    If(opc% make_opc .And. .Not.rel)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'OPC needs full relativistic mode!'")
    Endif
    !!
    If(opc% make_opc .And. BEB)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'CPA and OPC not yet implemented!'")
    Endif
    !!
    If( BEB .And. opc% make_opc .And. opc% projection /=OPC_ORTHOGONAL)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Only orthogonal projection implemented for CPA+OPC !'")
    Endif
    If( opc% make_opc .And. opc% projection /=OPC_ORTHOGONAL)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized OPC projection no longer supported !'")
    Endif
    !!
    !!
    opc% nosite=0
    opc% nodef=0
    If(.Not. opc% make_opc)Then
       opc% IOnodef=0
       Return
    Endif
    If(opc% IOnodef==0_t_int)Then
       opc% make_opc=.False.
       Return
    End If
    !!
    opc% omaxdim=0_t_int
    !!
    !!
    prot=  debugfun(nmodule,'','') .Or. debugfun(nmodule,nsubfunc,'')
    !!
    Write(outfile,1)
    !!
    Call check_opc_version(opc)
    !!
    opc% spin_independend = .True.
    If(   (opc% functional == OPC_FUNC_DEFAULT) .Or. &
         &(opc% functional == OPC_FUNC_ESCHRIG)      &
         &)Then
       opc% spin_independend = .False.
    Endif
    !!
    Write(outfile,2) Trim(OPC_PROJECTIONS(opc% projection))
    Write(outfile,3) Trim(OPC_FUNCTIONALS(opc% functional))
    If(opc% spin_independend)Then
       Write(outfile,4) 'no'
    Else
       Write(outfile,4) 'yes'
    Endif
    !!
1   Format('OPC: ',70('-'))
2   Format('OPC: Projection                            : ',a)
3   Format('OPC: Functional                            : ',a)
4   Format('OPC: spin dependend                        : ',a)
    !!
    !!
    Write(outfile,1)
    Write(outfile,18)opc% IOnodef
    Write(outfile,1)
    !!
    Do IOiodef=1,opc% IOnodef
       !!
       IOissao=opc% p_IOisrto(IOiodef)
       !!
       !!
       Write(outfile,19) IOiodef,IOissao,structure% p_IOelements(IOissao)&
            &,opc% p_IOstate(IOiodef)
       !!
       !!
    Enddo
    Write(outfile,1)
    !!
    !!
    !! translate input arrays into relativistic sorts
    !!
    opc% nodef=0
    Do IOiodef=1,opc% IOnodef
       IOissao=opc% p_IOisrto(IOiodef)
       !Print *,'IOiodef=',IOiodef,IOissao,opc% p_IOstate(IOiodef)
       !!
       If(IOissao > structure% IOnsort .Or. IOissao < 1_t_int )Then
5         Format("'OPC: input sort: ",i4," does not exist! '")
          Write(errbuf,5)IOissao
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       !!
       Do issa=1,structure% nsort
          !Print *,'issa=',issa,structure% nsort
          If(structure% p_isrtTOIOisrt(issa)==IOissao)Then
             opc% nodef=opc% nodef+1
             !Print *,'opc% nodef=',opc% nodef
          Endif
       Enddo
    Enddo
    !!
    Call palloc('opc% p_isrto',opc% p_isrto,1_t_int,opc% nodef&
         &,nmodule,nsubfunc)
    Call palloc('opc% p_state',opc% p_state,1_t_int,opc% nodef&
         &,nmodule,nsubfunc)
    Call palloc('p_odefTOIOodef',p_odefTOIOodef,1_t_int,opc% nodef&
         &,nmodule,nsubfunc)
    !!
    iodef=0
    Do IOiodef=1,opc% IOnodef
       IOissao=opc% p_IOisrto(IOiodef)
       !!
       !!
       Do issa=1,structure% nsort
          If(structure% p_isrtTOIOisrt(issa)==IOissao)Then
             iodef=iodef+1
             opc% p_isrto(iodef)=issa
             opc% p_state(iodef)=opc% p_IOstate(IOiodef)
             p_odefTOIOodef(iodef)=IOiodef
          Endif
       Enddo
    Enddo
    !!
    !!
    Call palloc('opc% p_nvo',opc%p_nvo,1_t_int,opc% nodef,nmodule,nsubfunc)
    Call palloc('opc% p_lvo',opc%p_lvo,1_t_int,opc% nodef,nmodule,nsubfunc)

    !! extract the state information of the input definitions
    !! state and sort info is non-relativistic
    !!
    opc% lmx=0_t_int
    iodefinitloop: Do iodef=1,opc% nodef
       !!
       issao=opc% p_isrto(iodef)
       state=opc% p_state(iodef)
       IOissao=opc% p_IOisrto(p_odefTOIOodef(iodef))
       !!
       
       !!
       Read(state(1:1),"(i1)") nmain
       l=lcharqn(state(2:2))
       opc% lmx=Max(opc% lmx,l)
       !!
       If(l>3 .Or. l<2)Then
          Write(errbuf,636)structure % p_elements(issao),IOissao,nmain&
               &,lqnchar(l)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       !!
       ivo=-1_t_int
       Do ivr=1,basis % p_nRv(issao)
          If( basis % p_lRv(ivr,issao) == l .And. &
               & basis % p_nnRv(ivr,issao) == nmain )Then
             ivo=ivr
             Exit
          Endif
       Enddo
       If(ivo < 0_t_int)Then
          Write(errbuf,6) nmain,lqnchar(l),IOissao&
               &,structure % p_elements(issao)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
       End If
       !!
       opc% p_nvo(iodef)=ivo
       opc% p_lvo(iodef)=l
       !!       
    Enddo iodefinitloop
    !!
    opc% omaxdim=(2*opc% lmx+1)*relfac
    
    !!
    !Do iodef=1,opc% nodef
    !   Print *,opc% p_nvo(iodef),opc% p_lvo(iodef)
    !End Do
    !Stop
    !!
    !!
    !! sort definitions into kohn-sham-matrix-order
    !!
    !!
    Do iodef=1,opc% nodef
       !!
       Do jodef=iodef+1,opc% nodef
          !!
          If( (opc% p_isrto(jodef) < opc% p_isrto(iodef)) .Or.&
               & ( (opc% p_isrto(jodef) == opc% p_isrto(iodef) ) .And.&
               &   (opc% p_nvo(jodef) < opc% p_nvo(iodef) ) ) )Then
             !!
             issao=opc% p_isrto(jodef)
             opc% p_isrto(jodef)=opc% p_isrto(iodef)
             opc% p_isrto(iodef)=issao
             !!
             state=opc% p_state(jodef)
             opc% p_state(jodef)=opc% p_state(iodef)
             opc% p_state(iodef)=state
             !!
             i=opc% p_nvo(jodef)
             opc% p_nvo(jodef)=opc% p_nvo(iodef)
             opc% p_nvo(iodef)=i
             !!
             i=opc% p_lvo(jodef)
             opc% p_lvo(jodef)=opc% p_lvo(iodef)
             opc% p_lvo(iodef)=i
             !!
             i=p_odefTOIOodef(jodef)
             p_odefTOIOodef(jodef)=p_odefTOIOodef(iodef)
             p_odefTOIOodef(iodef)=i
             !!
          Endif
          If( (opc% p_isrto(jodef) == opc% p_isrto(iodef)) .And.&
               & (opc% p_nvo(jodef) == opc% p_nvo(iodef) ) )Then
             Write(errbuf,7)structure% p_isrtTOIOisrt(opc% p_isrto(iodef))&
                  &,opc% p_state(iodef)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          End If
          
       Enddo
    Enddo
    !!
    !! output
    !!
    Write(outfile,1)
    Write(outfile,28)opc% nodef
    Write(outfile,1)
    !!
    Do iodef=1,opc% nodef
       !!
       ivo=opc% p_nvo(iodef)
       l=opc% p_lvo(iodef)
       IOiodef=p_odefTOIOodef(iodef)
       issao=opc% p_isrto(iodef)
       !!
       nmain=basis% p_nnRv(ivo,issao)
       !!
       Write(outfile,29) iodef,issao,structure% p_elements(issao)&
            &,nmain,lqnchar(l),IOiodef
       !!
       !!
    Enddo
    Write(outfile,1)
    !!
    !! make indicees into matrix blocks
    !!
    !! fisrt count number of occurences
    opc% nosite=0_t_int
    Do isa=1,structure% nsite
       !Print *,'isa=',isa
       issa=structure% p_isrt(isa)
       ivloop0: Do ivr=1,basis% p_nRv(issa)
          iodefloop0: Do iodef=1,opc% nodef
             If(issa == opc% p_isrto(iodef) .And.&
                  & ivr == opc% p_nvo(iodef))Then
                opc% nosite=opc% nosite+1
                Exit iodefloop0
             End If
          End Do iodefloop0
       End Do ivloop0
    End Do
    !!
    Write(outfile,11)opc% nosite
    Write(outfile,12)opc% omaxdim
    Write(outfile,13)opc% lmx
    !!
    !!
    Call palloc('opc% p_odefs',opc% p_odefs,1_t_int,opc% nosite&
         &,nmodule,nsubfunc)
    Call palloc('opc% p_isiteo',opc% p_isiteo,1_t_int,opc% nosite &
         &,nmodule,nsubfunc)
    Call palloc('opc% p_obi1',opc% p_obi1 ,1_t_int,opc% nosite &
         &,nmodule,nsubfunc)
    Call palloc('opc% p_obi2',opc% p_obi2 ,1_t_int,opc% nosite &
         &,nmodule,nsubfunc)
    !!
    !!
    !! the following orbital count only works for full relativistic
    !! orbital organization
    !!
    opc% nosite=0
    iz=0
    Do isa=1,structure% nsite
       !! relativistic sort
       issa=structure% p_isrt(isa)
       !! corresponding input sort
       !! loop over non-relat basis nl-shells
       ivloop: Do ivr=1,basis% p_nRv(issa)
          !! kappa momentum
          kap=basis% p_kapRv(ivr,issa)
          nmain=basis% p_nnRv(ivr,issa)
          !!
          j=2*Abs(kap)-1
          !!
          l=(Abs(2*kap+1)-1)/2
          !! degeneracy
          deg=2*l+1
          If(prot) Write(outfile,80) isa,structure% p_elements(issa)&
               &,nmain,lqnchar(l),j,iz+1
80        Format('OPC: ',i5,1x,a2,4x,i1,a1,i1,'/2    iz=',i5)
          odefloop:Do iodef=1,opc% nodef
             !!
             If( issa == opc% p_isrto(iodef) .And.&
                  & ivr == opc% p_nvo(iodef))Then
                If(l /= opc% p_lvo(iodef))Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'p_lvo is corrupted!'")
                Endif
                !! catched a state to be modified
                opc% nosite=opc% nosite+1
                opc% p_odefs(opc% nosite)=iodef
                opc% p_isiteo(opc% nosite)=isa
                opc% p_obi1(opc% nosite)=iz+1
                opc% p_obi2(opc% nosite)=iz+deg*relfac
                Exit odefloop
             End If
          End Do odefloop
          iz=iz+j+1
       End Do ivloop
       If(prot) Write(outfile,81)
81     Format('OPC:')
    End Do
    !!
    !!
    !!
    !!
    !!
    Write(outfile,1)
    Write(outfile,15)
    Write(outfile,1)
    Do isao=1,opc% nosite
       iodef=opc% p_odefs(isao)
       isa=opc% p_isiteo(isao)
       issa=structure% p_isrt(isa)
       IOiodef=p_odefTOIOodef(iodef)
       IOissao=opc% p_IOisrto(IOiodef)
       Write(outfile,17)&
            & isao,iodef,isa,structure% p_elements(issa),issa,IOiodef,IOissao&
            &,opc% p_state(iodef)&
            &,opc% p_obi1(isao),opc% p_obi2(isao)
    Enddo
    Write(outfile,1)
    !!
    !!
    !!
    !!
    Call palloc('opc% p_Bracah',opc% p_Bracah&
         &,1_t_int,2_t_int,1_t_int,opc% nodef,1_t_int,2_t_int,nmodule,nsubfunc)
    Call palloc('opc% p_Morb',opc% p_Morb&
         &,1_t_int,2_t_int,1_t_int,opc% nodef,nmodule,nsubfunc)
    Call palloc('opc% p_occu',opc% p_occu&
         &,1_t_int,2_t_int,1_t_int,opc% nodef,nmodule,nsubfunc)
    Call palloc('opc% p_MSmat',opc% p_MSmat,1_t_int,2_t_int&
         &,1_t_int,opc% omaxdim,1_t_int,opc% omaxdim&
         &,1_t_int,opc% nodef,nmodule,nsubfunc)
    Call palloc('opc% p_MLmat',opc% p_MLmat,1_t_int,2_t_int&
         &,1_t_int,opc% omaxdim,1_t_int,opc% omaxdim&
         &,1_t_int,opc% nodef,nmodule,nsubfunc)
    

    !! initialize the B-Racahs and Morbs
    Do iodef=1,opc% nodef
       Do ms=1,2
          opc% p_Bracah(ms,iodef,1)=0.130_t_real/hatoev
          opc% p_Bracah(ms,iodef,2)=0.0_t_real
          opc% p_Morb  (ms,iodef)=0.05_t_real
          opc% p_occu  (ms,iodef)=1.00_t_real
       Enddo
    Enddo
    !!
    Write(outfile,1000) 
    Write(outfile,1001) nsubfunc
    Write(outfile,1000) 
    Write(outfile,1010) 
    Call print_opc_params(structure% p_elements,opc)
    Write(outfile,1000) 
1000 Format('OPC: ',72('-'))
1001 Format('OPC: ',a)
1010 Format('OPC: Racah parameters B initial values:')
    !!
    !!
    Call pfree(p_odefTOIOodef)
    !!
    Return
    !!
    !!
6   Format("'OPC: valence state (",i2,a1,") not defined for input sort "&
         &,i5," (",a2,")!'")
7   Format("'OPC: input sort ",i4," state ",a2," multiply defined !'")
!!!8   Format('OPC:   relat sorts:       ',i5,'  OPC states '&
!!!         &,/,'OPC: sort  el.  state   ')
28  Format('OPC:   relat sorts:       ',i5,'  OPC states '&
         &,/,'OPC: OPC-def sort  el.  state   IO-OPC-def   ')
18  Format('OPC:           INPUT:     ',i5,'  OPC states '&
         &,/,'OPC: IO-OPC-def IOsort EL. state   ')
!!!9   Format('OPC: ',i3,3x,a2,5x,i1,a1,2x,6(f8.4,1x))
29  Format('OPC: ',i5,3x,i3,3x,a2,5x,i1,a1,2x,i5)
19  Format('OPC: ',i5,7x,i3,3x,a2,5x,a2)
11  Format('OPC: site-state pairs        : ',i5)
12  Format('OPC: maximum block dimension : ',i5)
13  Format('OPC: maximum angular momentum: ',i5)
15  Format('OPC: No. OPC-def site El. relat.-sort IO-OPC-def IO-sort &
         &state blk-st blk-end')
17  Format('OPC: ',i4,1x,i5,1x,i5,1x,a2,2x,i5,8x,i5,4x,i5,5x,a2,2(3x,i5))
636 Format('''OPC: input atom ',a2,' sort(',i3,'): orbital type ',i1,a1&
         &,' not allowed for OPC!''') 
    !!
  End Subroutine io_init_opc
  !!***********************************************************************
  Subroutine check_opc_version(opc)
    !!
    Implicit None  
    !!IO
    Type(T_OPC),Intent(in)       :: opc
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_opc_version"
    !!
    If (( opc% functional < 1_t_int ) .Or. &
         &( opc% functional > 4_t_int )) Then
1      Format("'Wrong OPC functional version: ",i4," ! '")
       Write(errbuf,1) opc% functional
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    If (( opc% projection < 1_t_int ) .Or. &
         &( opc% projection > 2_t_int )) Then
2      Format("'Wrong OPC projection version: ",i4," ! '")
       Write(errbuf,2) opc% projection
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
  End Subroutine check_opc_version
  !!***********************************************************************
  !! for l=0...lmax calculate Fl(l) == SLater F(k) with k=2*l
  Subroutine calc_slater_F_integrals(mesh,lmax,f,Fl)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Integer(t_int),Intent(in)    :: lmax
    Real(t_real),Intent(in)      :: f(:)
    Real(t_real),Intent(out)     :: Fl(0:lmax)
    !!LOCAL
    Integer(t_int) :: l,k,ir
    Real(t_real)   :: ri,rjac,rk,ftr,cp,sd
    Real(t_real)   :: yout(mesh% nr)
    Real(t_real)   :: yin (mesh% nr)
    !!
    !!
    !!
    Do l=0,lmax
       k=l*2
       !!
       Do ir=1,mesh% nr
          ri  =mesh% p_rmesh(ir)
          rjac=mesh% p_jmesh(ir)
          ftr=f(ir)**2
          If(k == 0)Then
             rk=1.0_t_real
          Else
             rk=ri**k
          Endif
          yout(ir)=ftr *rk *rjac
          If(mesh% with_zero .And. (ir ==1 ))Then
             yin (ir)=0.0_t_real
          Else
             yin (ir)=ftr / rk /ri *rjac
          Endif
       Enddo
       !!
       Call lag4int(mesh% nr,yout,mesh% hx,cp,lagint_out)
       Call lag4int(mesh% nr,yin ,mesh% hx,cp,lagint_in)
       !!
       Do ir=1,mesh% nr
          ri  =mesh% p_rmesh(ir)
          rjac=mesh% p_jmesh(ir)
          ftr=f(ir)**2
          If(k == 0)Then
             rk=1.0_t_real
          Else
             rk=ri**k
          Endif
          sd=yin(ir)*rk
          If(mesh% with_zero .And. (ir == 1 ))Then
             sd=sd+0.0_t_real
          Else
             sd=sd+yout(ir)/rk/ri
          Endif
          yout(ir)=sd*ftr*rjac
       Enddo
       !!
       Call lag4int(mesh% nr,yout,mesh% hx,cp,lagint_out)
       !!
       !Print *,k,cp
       Fl(l)=cp
    Enddo
    !!
    Return
    !!
  End Subroutine calc_slater_F_integrals
  !!***********************************************************************
  !! Calculation of Racah paramters (see M. Richter, HMM p.46) for OPC states.
  Subroutine calc_racah_params(opc,elements,basis,relativistic,quant,vca,natom)
    !!
    Implicit None
    !!     
    !!IO
    Logical(t_log),Intent(in)    :: vca
    Type(T_Basis),Intent(in)     :: basis
    Type(T_OPC),Intent(inout)    :: opc
    Type(T_Quant),Intent(in)     :: quant
    Character(2),Intent(in)      :: elements(:)
    Integer(t_int),Intent(in)    :: relativistic
    Real(t_real),Intent(in)      :: natom(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_racah_params"
    Logical(t_log) :: prot,rel
    Integer(t_int) :: iodef,issao,j,l,k,iz,ms,relfac,msb
    Integer(t_int) :: i,ivra,kapa,la,ja,ma,mua,ivrb,kapb,lb,jb,mb,mub,icomp
    Real(t_real)   :: racahb,fs,del,ff(2),jdeg,ZZ
    Real(t_real)   :: Fk(0:3),FFk(0:3) !! Only up to f-states
    Complex(t_real):: alpha(2)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Real(t_real),Pointer   :: cc(:)
    Type(T_Mesh),Pointer   :: tmesh
    !!
    Nullify(cc)
    !!
    If(.Not. opc% make_opc) Return
    !!
    !!
    prot=  debugfun(nmodule,'','') .Or. debugfun(nmodule,nsubfunc,'')
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    relfac=1_t_int
    If(rel)Then
       relfac=2_t_int
    Endif
    !!
    opc% p_MLmat(:,:,:,:)=0.0_t_real
    opc% p_MSmat(:,:,:,:)=0.0_t_real
    !!
    !!
    iopcloop: Do iodef=1,opc% nodef
       !!
       issao=opc% p_isrto(iodef)
       !!
       ZZ=natom(issao)
       !!
       tmesh => basis% p_basmeshs(issao)
       !!
       Call palloc('cc',cc,1_t_int,tmesh% nr, nmodule,nsubfunc)
       Call cjlag4int(tmesh% nr,tmesh% p_jmesh,tmesh% hx,cc)
       !!
       opc% p_Bracah(1:2,iodef,:)=0.0_t_real
       !!
       opc% p_MSmat(:,:,:,iodef)=c0 !! complex
       opc% p_MLmat(:,:,:,iodef)=c0 !! complex
       i=0
       Do ivra=1,basis% p_nRv(issao)
          kapa=basis% p_kapRv(ivra,issao)
          la=(Abs(2*kapa+1)-1)/2
          ja=2*Abs(kapa)-1
          If( (ivra==opc% p_nvo(iodef)) .Or. &
               & ((la>0) .And. (ivra-1==opc% p_nvo(iodef))) ) Then
             !Print  *,'first j=',kapa,la,ja
             Do ma=1,ja+1
                mua=-ja+2*(ma-1)
                i=i+1
                j=0
                Do ivrb=1,basis% p_nRv(issao)
                   kapb=basis% p_kapRv(ivrb,issao)
                   lb=(Abs(2*kapb+1)-1)/2
                   jb=2*Abs(kapb)-1
                   If( (ivrb==opc% p_nvo(iodef)) .Or. &
                        & ((lb>0) .And. (ivrb-1==opc% p_nvo(iodef))) ) Then
                      !Print  *,'first j=',kapb,lb,jb
                      Do mb=1,jb+1
                         mub=-jb+2*(mb-1)
                         
                         j=j+1
                         
                         If(kapa == kapb .And. mua== mub )Then
                            del=1.0_t_real
                         Else
                            del=0.0_t_real
                         Endif

                         !! prepare radial integrals
                         Do icomp=1,2
                            ff(icomp)=Sum(&
                                 &    (basis% p_fv(1:tmesh% nr,ivra,icomp,issao)&
                                 &    *basis% p_fv(1:tmesh% nr,ivrb,icomp,issao))&
                                 &    *tmesh% p_rmesh(1:tmesh% nr)**(basis% p_fgam(issao)*2)&
                                 &    *cc(1:tmesh% nr)&
                                 &   )
                         Enddo

                         Do ms=1,2
                            msb=3-ms
                            fs=3.0_t_real-2.0_t_real*ms

                            !! prepare spin projectors
                            alpha(1)=(del&
                                 &      +fs*quant% p_chiSigmaRchi( kapa,ma, kapb,mb))&
                                 &*0.5_t_real
                            alpha(2)=(del&
                                 &-fs*quant% p_chiSigmaRchi(-kapa,ma,-kapb,mb))&
                                 &*0.5_t_real


                            
                            
                            !! (i|beta sigma_Z |i)
                            opc% p_MSmat(ms,i,j,iodef)= opc% p_MSmat(ms,i,j,iodef)&
                                 &+Sum(alpha(1:2)*ff(1:2))

                            !! prepare spin-L_z projectors
                            alpha(1)= quant% p_chipSLRchi(ms , kapa,ma, kapb,mb)
                            alpha(2)= quant% p_chipSLRchi(msb, -kapa,ma,-kapb,mb)


                            !! (i|beta sigma_Z |i)
                            opc% p_MLmat(ms,i,j,iodef)= opc% p_MLmat(ms,i,j,iodef)&
                                 &+Sum(alpha(1:2)*ff(1:2))
                         Enddo
                         
                      Enddo
                   Endif
                Enddo
             Enddo
          Endif
       Enddo
       !!


       jmuif: If ( &
            & (opc% functional /= OPC_FUNC_ESCHRIG        ) .And.&
            & (opc% functional /= OPC_FUNC_ESCHRIG_NO_SPIN)  ) Then

          Fk(0:3)=0.0_t_real
          Do ivra=1,basis% p_nRv(issao)
             kapa=basis% p_kapRv(ivra,issao)
             la=(Abs(2*kapa+1)-1)/2
             ja=2*Abs(kapa)-1
             jdeg=Abs(kapa)/Real(2*la+1,t_real)
             !Print *,'jdeg=',jdeg
             If( (ivra==opc% p_nvo(iodef)) .Or. &
                  & ((la>0) .And. (ivra-1==opc% p_nvo(iodef))) ) Then
                
                
                !!
                !! Slater F- integrals
                !!
                Call calc_slater_F_integrals(tmesh,3_t_int &
                     &,basis% p_fv(:,ivra,LARGE_COMP1,issao)&
                     &*tmesh% p_rmesh(1:tmesh% nr)**basis% p_fgam(issao)&
                     &,FFk)
                
                !! j-average
                Fk(0:3)=Fk(0:3)+FFk(0:3)*jdeg
                !!
                
             End If
          End Do
          
          !! output Fk
          If(prot) Then
             Write(outfile,1020,advance='no') 
             Do k=1,3
                Write(outfile,1021,advance='no') k,Fk(k)
             Enddo
             Write(outfile,2)
          Endif !! prot
          
          !!
          l=opc% p_lvo(iodef)
          !!     ----- Racah parameter for d- and f-states ----
          !!
          racahb=0.0_t_real
          If (l == 2) Then
             !! d-states
             racahb=(9.0_t_real*Fk(1) -5.0_t_real*Fk(2))/441.0_t_real
          Else If (l == 3) Then
             !! f-states
             racahb=(&
                  &   (  5.0_t_real /  225.00_t_real ) *Fk(1) &
                  &  +(  6.0_t_real / 1089.00_t_real ) *Fk(2) &
                  &  -( 91.0_t_real / 7361.64_t_real ) *Fk(3) &
                  &  )/3.0_t_real
          Else
             Write(errbuf,5000) l
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          !!
          !! TODO: how to do spin-dependend racahs
          !!
          opc% p_Bracah(1:2,iodef,1)=racahb
          !!

       End If jmuif
       

       !!TODO: avoid overshooting
       !!
       If(opc% functional == OPC_FUNC_ESCHRIG_NO_SPIN)Then
          racahb=Pcoef(elements(issao),opc% p_state(iodef),ZZ,vca)
          Do ms=1,2
             opc% p_Bracah(ms,iodef,1)=racahb&
                  &*MLofml(opc% p_occu(1,iodef),opc% p_lvo(iodef))
             opc% p_Bracah(ms,iodef,2)=racahb&
                  &*dMLofml(opc% p_occu(1,iodef),opc% p_lvo(iodef))
          Enddo
       Endif
       If(opc% functional == OPC_FUNC_ESCHRIG)Then
          racahb=Pcoef(elements(issao),opc% p_state(iodef),ZZ,vca)
          Do ms=1,2
             opc% p_Bracah(ms,iodef,1)=racahb&
                  &*MLofml(opc% p_occu(ms,iodef),opc% p_lvo(iodef))
             opc% p_Bracah(ms,iodef,2)=racahb&
                  &*dMLofml(opc% p_occu(ms,iodef),opc% p_lvo(iodef))
          Enddo
          
       Endif
       !!
       If ( opc% spin_independend ) Then
          !!
          opc% p_Bracah(1,iodef,1)=Sum(opc% p_Bracah(1:2,iodef,1))*0.5_t_real
          opc% p_Bracah(2,iodef,1)=opc% p_Bracah(1,iodef,1)
          
          opc% p_Bracah(1,iodef,2)=Sum(opc% p_Bracah(1:2,iodef,2))*0.5_t_real
          opc% p_Bracah(2,iodef,2)=opc% p_Bracah(1,iodef,2)
          
          !!
       Endif
       !!
    Enddo iopcloop
    !!
    !!
    !!
    If (prot) Then
       Write(outfile,11000)
11000  Format('OPC: projected operator matrices')
       Do iodef=1,opc% nodef
          l=opc% p_lvo(iodef)
          iz=(2*l+1)*relfac
          Write(outfile,11010) iodef,iz
11010     Format('OPC: iodef=',i5,'  dim=',i5)
          
          Do ms=1,2
             Call printmat('Re MSmat  ',"(f12.5,1x)","('      0      ')"&
                  &,Real (opc% p_MSmat(ms,1:iz,1:iz,iodef),t_real),iz,iz&
                  &,1.0e-10_t_real)
             Call printmat('Im MSmat  ',"(f12.5,1x)","('      0      ')"&
                  &,Aimag(opc% p_MSmat(ms,1:iz,1:iz,iodef)       ),iz,iz&
                  &,1.0e-10_t_real)
          Enddo
          Do ms=1,2
             Call printmat('Re MLmat  ',"(f12.5,1x)","('      0      ')"&
                  &,Real (opc% p_MLmat(ms,1:iz,1:iz,iodef),t_real),iz,iz&
                  &,1.0e-10_t_real)
             Call printmat('Im MLmat  ',"(f12.5,1x)","('      0      ')"&
                  &,Aimag(opc% p_MLmat(ms,1:iz,1:iz,iodef)       ),iz,iz&
                  &,1.0e-10_t_real)
       Enddo

          
       Enddo
    Endif                     ! prot
    
    !!
    
    Write(outfile,1000) 
    Write(outfile,1001) nsubfunc
    Write(outfile,1000) 
    Write(outfile,1010) 
    !!
    Call print_opc_params(elements,opc)
    !!
    Write(outfile,1000) 
    !!
    Call pfree(cc)
    !!
    !!
    Return
    !!
1000 Format('OPC: ',72('-'))
1001 Format('OPC: ',a)
1010 Format('OPC: Racah parameters B updated:')
1020 Format('OPC: ')
1021 Format('  F(',i2,')=',f15.8)
2   Format()
5000 Format("'Racah-parameter for l=",i2," not yet defined!'")
    !!
  End Subroutine calc_racah_params
  !!***********************************************************************
  !!
  !! Output of OPC parameters $B_{\vec{s}l\sigma}$ and $L_{\vec{s}l\sigma}$.
  !!
  Subroutine print_opc_params(elements,opc)
    !!
    !!
    !!
    Implicit None
    !!IO
    Type(T_OPC),Intent(in)       :: opc
    Character(2),Intent(in)      :: elements(*)
    !!
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_opc_params"
    Integer(t_int) :: iodef,issao
    !!
    If(.Not. opc% make_opc) Return
    !!
    Write(outfile,1020) 
    Write(outfile,1030) 
    Write(outfile,1020) 
    !!
    Do iodef=1,opc% nodef
       !!
       issao=opc% p_isrto(iodef)
       !!
       Write(outfile,1040) elements(issao),issao&
            & ,opc% p_state(iodef) &
            & ,opc% p_Bracah(1,iodef,1)*1000._t_real*hatoev&
            & ,opc% p_Bracah(1,iodef,2)*1000._t_real*hatoev&
            & ,opc% p_Bracah(2,iodef,1)*1000._t_real*hatoev&
            & ,opc% p_Bracah(2,iodef,2)*1000._t_real*hatoev&
            & ,opc% p_Morb(1,iodef),opc% p_Morb(2,iodef) &
            & ,opc% p_occu(1,iodef),opc% p_occu(2,iodef)
    Enddo                     ! iop
    Write(outfile,1020)
    Write(outfile,1021)
    !!
    Return
    !!
1020 Format('OPC: ',75('-'))
1021 Format('OPC:')
1030 Format('OPC: El srt Orb | Bup[meV] dBup  | Bdn[meV] dBdn  | Mup     &
          &Mdn  |  Nup    Ndn ')
1040 Format('OPC: ',a2,' ',i3,' ',a2,'  |',f7.2,' ',f7.2,' |',f7.2,' ',f7.2&
          &,' |',f6.3,' ',f6.3,' |',f6.3,' ',f6.3)
    !!
  End Subroutine print_opc_params
  !!***********************************************************************
  !! add the opc to the Hamiltonian
  Subroutine opc_merge_h(opc,nvdim,hvv&
!!$       &,sp&
       &)
    !!
    !! Add OP correction to Hamiltonian
    !!
    Implicit None
    !!IO
    Type(T_OPC),Intent(in)       :: opc
    !!dimension of valence-valence Hamilton matrix
    Integer(t_int),Intent(in)    :: nvdim
    !! Hamiltonain matrix
    Complex(t_real),Intent(inout):: hvv(nvdim,nvdim)
!!$    !! overlap matrix
!!$    Complex(t_real),Intent(inout):: sp(nvdim,nvdim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="opc_merge_h"
    Integer(t_int) :: isao,iodef,ib1,ib2,i,j,i1,j1,ms!,ii,jj
    Complex(t_real):: cd!,cx
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    If(.Not. opc% make_opc) Return
    !!
    If (opc% projection == OPC_ORTHOGONAL ) Then
       Do isao=1,opc% nosite
          iodef=opc% p_odefs(isao)
          ib1=opc% p_obi1(isao) !! start index of o-block
          ib2=opc% p_obi2(isao) !! end   index of o-block
          Do i=ib1,ib2
             i1=i-ib1+1_t_int
             Do j=ib1,ib2
                j1=j-ib1+1_t_int
                cd=c0
                Do ms=1,2
                   cd=cd-(opc% p_Bracah(ms,iodef,1)*opc% p_Morb(ms,iodef)&
                        &*opc% p_MLmat(ms,i1,j1,iodef)&
                        &+ 0.5_t_real*opc% p_Bracah(ms,iodef,2)&
                        &*opc% p_Morb(ms,iodef)**2&
                        &*opc% p_MSmat(ms,i1,j1,iodef)&
                        &)
                Enddo
                hvv(i,j)=hvv(i,j)+cd
             Enddo ! j
          Enddo ! i
       Enddo ! isao
    Else
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized OPC projection no longer supported !'")
!!$       Do i=1,nvdim
!!$          Do j=1,nvdim
!!$             cx=c0
!!$             
!!$             Do isao=1,opc% nosite
!!$                iodef=opc% p_odefs(isao)
!!$                ib1=opc% p_obi1(isao) !! start index of o-block
!!$                ib2=opc% p_obi2(isao) !! end   index of o-block
!!$                
!!$                Do ii=ib1,ib2
!!$                   i1=ii-ib1+1
!!$                   Do jj=ib1,ib2
!!$                      j1=jj-ib1+1
!!$                   
!!$                      cd=c0
!!$                      Do ms=1,2
!!$                         cd=cd-(opc% p_Bracah(ms,iodef,1)*opc% p_Morb(ms,iodef)&
!!$                              &*opc% p_MLmat(ms,i1,j1,iodef)&
!!$                              &+ 0.5_t_real*opc% p_Bracah(ms,iodef,2)&
!!$                              &*opc% p_Morb(ms,iodef)**2&
!!$                              &*opc% p_MSmat(ms,i1,j1,iodef)&
!!$                              &)
!!$                      Enddo
!!$                      
!!$                      cx=cx+sp(i,ii)*cd*sp(jj,j)
!!$                      
!!$                   Enddo
!!$                Enddo ! ii
!!$             Enddo ! isau
!!$             !!
!!$             hvv(i,j)=hvv(i,j)+cx
!!$          Enddo
!!$       Enddo
       
    Endif
    !!
    Return
    !!
  End Subroutine opc_merge_h
  !!***********************************************************************
  !!
  !! calculate vv=C or vv=S*C which enters the orbital moment expression
  !! for the OPC state
  !!
  Subroutine opc_calc_o_cs(opc,zz&
!!$       &,sp&
       &,nvdim,nband,ik)
    !! to be executed at every k-point    
    !!-----------------------------------------------------------------------
    Implicit None
    !!IO
    Type(T_OPC),Intent(inout)    :: opc     
    Integer(t_int),Intent(in)    :: nvdim,ik
    Integer(t_int),Intent(in)    :: nband
    Complex(t_real),Intent(in)   :: zz(nvdim,nvdim) ! eigenvectors
!!$    Complex(t_real),Intent(in)   :: sp(nvdim,nvdim) ! overlap matrix
    !! if projection=OPC_HYBRIDIZED
    !!    vv=S*zz
    !! if projection=OPC_ORTHOGONAL
    !!    vv=zz
    !!LOCAL
    Character(*),Parameter::nsubfunc="opc_calc_o_cs"
    Integer(t_int) :: isao,ib1,ib2,i,i1,ib!,k
    !    Complex(t_real):: c
    !Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    If(.Not. opc% make_opc) Return
    !!
    If (opc% projection /= OPC_ORTHOGONAL ) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Hybridized OPC projection no longer supported !'")
    Endif

    !!
    Do isao=1,opc% nosite
       ib1=opc% p_obi1(isao) !! start index of o-block
       ib2=opc% p_obi2(isao) !! end   index of o-block
       Do i=ib1,ib2
          i1=i-ib1+1_t_int
!!$          If (opc% projection == OPC_ORTHOGONAL ) Then
          Do ib=1,nband
             opc% p_cvv(i1,isao,ib,ik)=zz(i,ib)
          Enddo
!!$          Else
!!$             Do ib=1,nband
!!$                c=c0
!!$                Do k=1,nvdim
!!$                   c=c+sp(i,k)*zz(k,ib)
!!$                Enddo
!!$                opc% p_cvv(i1,isao,ib,ik)=c
!!$             Enddo
!!$          Endif
       Enddo
    Enddo
    !!
  End Subroutine opc_calc_o_cs
  !!*********************************************************************** 
  !!
  !! sum up and integrate the orbital moment contributions
  !! ----------------------------------------------------------------------
  !!  Inputs:
  !!    bweitk:bandweights
  !!    nvdim :number of valence orbitals
  !!    nkp     :number of irreducible k-points
  !!    nspin   :spin channels
  !!    vv      :matrix vv_{in}(k)=SC or C
  !!             n - band index; i - orbital index
  !!             c^_i - LO coeff. of i-th orbital in n-th KS state
  !!  Outputs:
  !!    dmat    :dmat_{ij}=\sum_{kn}^{occ} vmat_{in}(k)^*.vmat_{nj}(k)
  !! ----------------------------------------------------------------------
  !!    if mspin==1 dmat will contain half the occupation, it is dmat for
  !!    one of the two degenerate spin directions
  !!
  Subroutine opc_calc_M_orb(structure,opc,nspin,nband,nkp,bweitk&
       &,relativistic)
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_OPC),Intent(inout)    :: opc     
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: nspin,nkp,nband
    Real(t_real),Intent(in)      :: bweitk(nband,nkp,nspin)
    !!LOCAL
    Character(*),Parameter::nsubfunc="opc_calc_M_orb"
    Logical(t_log) :: rel
    Integer(t_int) :: isao,ms,i,j,l2,iodef,ikp,ib,relfac,issao,ns
    Real(t_real)   :: xl(2),xs(2)
    Complex(t_real):: c
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    If(.Not. opc% make_opc) Return
    !!
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    relfac=1_t_int
    If(rel)Then
       relfac=2_t_int
    Endif
    !!
    If(.Not.rel) Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'OP correction makes not much sense for non&
         & relativistic calculations !'")
    !!
    !prot= debugfun(nmodule,'','') .Or. debugfun(nmodule,nsubfunc,'')
    !!
    opc% p_Morb(:,:)=0.0_t_real
    opc% p_occu(:,:)=0.0_t_real
    !!
    ns=1
    Do isao=1,opc% nosite
       iodef=opc% p_odefs(isao)
       issao=opc% p_isrto(iodef)
       
       l2=(2*opc% p_lvo(iodef)+1)*relfac
       
       
       !! --- sum over bands and k-points
       !!
       xl(1:2)=0.0_t_real
       xs(1:2)=0.0_t_real
       !!
       Do i=1,l2
          Do j=1,l2
             c=c0
             Do ikp=1,nkp
                Do ib=1,nband
                   c=c+         opc% p_cvv(i,isao,ib,ikp)&
                        &*bweitk(ib,ikp,ns)&
                        &*Conjg(opc% p_cvv(j,isao,ib,ikp))
                Enddo
             Enddo
             Do ms=1,2
                xl(ms)=xl(ms)+Real(c*opc% p_MLmat(ms,j,i,iodef),t_real)
                xs(ms)=xs(ms)+Real(c*opc% p_MSmat(ms,j,i,iodef),t_real)
             Enddo
          Enddo ! j
       Enddo ! i
       
       xl(1:2)=xl(1:2)/structure% p_mult(issao)
       xs(1:2)=xs(1:2)/structure% p_mult(issao)
       
       
       !!
       If (opc% spin_independend) Then
          xs(1)=Sum(xs(1:2))
          xs(2)=xs(1)
          xl(1)=Sum(xl(1:2))
          xl(2)=xl(1)
       Endif
       !!
       !!
       !!
       Do ms=1,2
          opc% p_occu(ms,iodef)=opc% p_occu(ms,iodef)+xs(ms)
          opc% p_Morb(ms,iodef)=opc% p_Morb(ms,iodef)+xl(ms)
       Enddo ! ms
       !!
    Enddo ! isao
    !!

    
    !!
    Write(outfile,1000) 
    If (opc% spin_independend) Then 
       Write(outfile,1020)
    Else
       Write(outfile,1010)
    Endif
    !!
    Call print_opc_params(structure% p_elements,opc)
    !!
    Write(outfile,1000)
    !!
    !!
    Return
    !!
1000 Format('OPC:',72('-'))
1010 Format('OPC: L updated (from hybrid expression)')
1020 Format('OPC: L updated (from hybrid expression, spin sum)')
    !!
  End Subroutine opc_calc_M_orb
  !!***********************************************************************
  !!
  Subroutine calc_opc_etot(structure,opc,etot_opc,etot_opc_dc)
    !!
    !! Calculate total energy correction $0.5 \sum_X B_X M_X^2$ 
    !! within orbital polarization scheme. X are (spin-) shells
    !! with OP corrections.
    !!
    Implicit None
    !!
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_OPC),Intent(in)       :: opc
    !!total energy correction due to OPC
    Real(t_real),Intent(out)     :: etot_opc
    Real(t_real),Intent(out)     :: etot_opc_dc
    !!LOCAL
    !Character(*),Parameter::nsubfunc="calc_opc_etot"
    Integer(t_int) :: nospin,iodef,ms,issao
    Real(t_real)   :: d,dc,mult
    !!
    !!
    If(.Not. opc% make_opc) Return
    !!
    !!
    nospin=2
    If (opc% spin_independend) Then 
       nospin=1
    Endif
    !!
    !!
    etot_opc_dc=0.0_t_real
    etot_opc   =0.0_t_real
    !!
    Do iodef=1,opc% nodef
       issao=opc% p_isrto(iodef)
       Do ms=1,nospin
          !!
          mult=Real(structure% p_mult(issao),t_real)
          d=-mult&
               &*opc% p_Bracah(ms,iodef,1)*opc% p_Morb(ms,iodef)**2*0.5_t_real
          dc=-mult&
               &*opc% p_Bracah(ms,iodef,2)*opc% p_Morb(ms,iodef)**2*0.5_t_real&
               &*opc% p_occu(ms,iodef)&
               &+2.0_t_real*d
          !!
          etot_opc_dc =etot_opc_dc+dc
          etot_opc    =etot_opc   +d
          !!
          Write(outfile,1010) iodef,ms,d,dc
          !!
       Enddo ! ms
    Enddo ! iopc
    !!
    Write(outfile,1000) etot_opc-etot_opc_dc
    !!
    Return
    !!
1000 Format('OPC: OP-correction to total energy:',f13.9)
1010 Format('OPC: OPC-def ',i3,' spin',i2,' e ',f13.9,' dc ',f13.9)
    !!
  End Subroutine calc_opc_etot
  !!***********************************************************************
  Real(t_real) Function Pcoef(element,state,ZZ,vca)
    !!
    Implicit None  
    !!IO
    Character(2),Intent(in)      :: element
    Character(*),Intent(in)      :: state
    Logical(t_log),Intent(in)    :: vca
    Real(t_real),Intent(in)      :: ZZ
    !!LOCAL
    Character(*),Parameter::nsubfunc="Pcoef"
    Integer(t_int) :: iat,i0
    Real(t_real)   :: Z,x,a
    !!
    Pcoef=0.0_t_real
    !!
    iat=getelementnumber(element)
    Z=Real(iat,t_real)
    !!
    !!
    If(vca .And. Abs(Z-ZZ)>1.0e-6_t_real)Then
       i0=Floor(ZZ,t_int)
       x=(ZZ-Real(i0,t_real))
       If( (i0 < 0 .Or. i0+1 > MAX_ELEMENT ) )Then
          Write(errbuf,"('''VCA Element charge ',g11.4,' out of range!''')")ZZ
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
       End If
       !!
       a=0.0_t_real
       If(state=='3d')Then
          a=Pcoef_3d(i0)*1.0e-3_t_real/hatoev
       Elseif(state=='4f')Then
          a=Pcoef_4f(i0)*1.0e-3_t_real/hatoev
       Else
          Write(errbuf,5000) state
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Endif
       Pcoef=a*(1.0_t_real-x)
       
       If(state=='3d')Then
          a=Pcoef_3d(i0+1)*1.0e-3_t_real/hatoev
       Elseif(state=='4f')Then
          a=Pcoef_4f(i0+1)*1.0e-3_t_real/hatoev
       Else
          Write(errbuf,5000) state
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Endif
       
       Pcoef=Pcoef+a*x
       
       Return
    Endif
    !!
    !!
    !!
    If(state=='3d')Then
       Pcoef=Pcoef_3d(iat)*1.0e-3_t_real/hatoev
    Elseif(state=='4f')Then
       Pcoef=Pcoef_4f(iat)*1.0e-3_t_real/hatoev
    Else
       Write(errbuf,5000) state
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
    Endif
    !!
    !!
    !!
    Return
    !!
5000 Format("' No P coefficient for Eschrig functional implemented for ",a2&
          &," orbitals!'")
    !!
  End Function Pcoef
  !!***********************************************************************
  !! return the total M_L value for a certain occupation of the l-shell
  !! MLofml=sum_{m occu.} m
  Real(t_real) Function MLofml(noccu,l)
    !!
    Implicit none
    !!IO
    !! angular momentum of the shell
    Integer(t_int),Intent(in)    :: l
    !! occupation of tge shell
    Real(t_real),Intent(in)      :: noccu
    !!LOCAL
    Real(t_real)   :: x,deg
    !!
    deg=2.0_t_real*Real(l,t_real)+1.0_t_real
    x=noccu
    If(noccu>deg)Then
       x=x-deg
    Endif
    MLofml=Abs(x-deg)*x*0.5_t_real
    !!
    Return
    !!
  End Function MLofml
  !!***********************************************************************
  !! return the derivative of total M_L value for a certain occupation of
  !! the l-shell 
  !! dMLofml=d/dN (sum_{m occu.} m)
  Real(t_real) Function dMLofml(noccu,l)
    !!
    Implicit none
    !!IO
    !! angular momentum of the shell
    Integer(t_int),Intent(in)    :: l
    !! occupation of tge shell
    Real(t_real),Intent(in)      :: noccu
    !!LOCAL
    Real(t_real)   :: x,deg
    !!
    deg=2.0_t_real*Real(l,t_real)+1.0_t_real
    x=noccu
    If(noccu>deg)Then
       x=x-deg
    Endif
    dMLofml=deg*0.5_t_real-x
    !!
    Return
    !!
  End Function dMLofml
  !!***********************************************************************
  !! get the OPC parameter from the iteration vector
  Subroutine opc_checkout (opc,vit,nit,OPCfactor)
    !!
    Implicit None  
    !!IO
    Type(T_OPC),Intent(inout)    :: opc
    Integer(t_int),Intent(inout) :: nit
    Real(t_real),Intent(in)      :: OPCfactor
    Real(t_real),Intent(in)      :: vit(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="opc_checkout"
    Integer(t_int) :: iodef,ms
    Real(t_real)   :: d
    !!
    Do iodef=1,opc% nodef
       Do ms=1,2
          !!
          d=vit(nit)/OPCfactor
          nit=nit+1_t_int
          opc% p_Morb(ms,iodef)=d
          d=vit(nit)/OPCfactor
          nit=nit+1_t_int
          opc% p_occu(ms,iodef)=d
          !!
       Enddo
    Enddo
    !!
    !!
    Return
  End Subroutine opc_checkout
  !!***********************************************************************
  !! put the OPC parameter into the iteration vector
  Subroutine opc_checkin (opc,vit,nit,OPCfactor)
    !!
    Implicit None  
    !!IO
    Type(T_OPC),Intent(in)       :: opc
    Integer(t_int),Intent(inout) :: nit
    Real(t_real),Intent(in)      :: OPCfactor
    Real(t_real),Intent(inout)   :: vit(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="opc_checkout"
    Integer(t_int) :: iodef,ms
    Real(t_real)   :: d
    !!
    Do iodef=1,opc% nodef
       Do ms=1,2
          !!
          d=opc% p_Morb(ms,iodef)*OPCfactor
          vit(nit)=d
          nit=nit+1_t_int
          
          d=opc% p_occu(ms,iodef)*OPCfactor
          vit(nit)=d
          nit=nit+1_t_int
          !!
       Enddo
    Enddo
    !!
    !!
    Return
  End Subroutine opc_checkin
  !!***********************************************************************
  Subroutine  opc_alloc(opc,nband,nkp)
    !!
    Implicit None 
    !!IO
    Type(T_OPC),Intent(inout)    :: opc
    Integer(t_int),Intent(in)    :: nband,nkp
    !!LOCAL
    Character(*),Parameter::nsubfunc="opc_alloc"
    !!
    If(.Not. opc% make_opc) Return
    !!
    Call palloc('opc% p_cvv',opc% p_cvv&
         &,1_t_int,opc% omaxdim,1_t_int,opc% nosite&
         &,1_t_int,nband,1_t_int,nkp,nmodule,nsubfunc)
    !!
    !!
    Return
    !!
  End Subroutine opc_alloc
  !!***********************************************************************
  Subroutine opc_free(opc)
    !!
    Implicit None 
    !!IO
    Type(T_OPC),Intent(inout)    :: opc
    !!LOCAL
    !!
    If(.Not. opc% make_opc) Return
    !!
    Call pfree(opc% p_cvv)
    !!
    !!
    Return
    !!
  End Subroutine opc_free
  !!***********************************************************************
  
End Module opcorrection
