!!----DEBUG options-----------------|
!! SUBFUNC    | MODES               |
!!----------------------------------|
!!----------------------------------|
Module orbitalbasis
  !!
  Use precisiondefs
  Use memory
  Use file_iounits
  Use constants
  Use atomdefs
  Use tio
  Use tsymmetry
  Use errors
  Use torbitalbasis
  Use matrixinvers
  Use ioaux
  Use tdiracdefs
  Use versioncontrol
  Use integrate1d
  Use diracdefs
  Use trelativistic
  Use optionstest
  Use finuc
  Use tnumerics
  Use tmesh
  Use mesh
  Use diracsolver
  Use tpotentialdensity
  Use matserv
  Use tmol
  Use diagonalize
  Use prnt
  Use tbasdef
  Use basdef
  Use tcoreoccupation
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="ORBITALBASIS"  
  !!
  Public :: funcmin,print_basis,setup_relat_basis,calc_basis&
       &,build_map_trans,allocate_basis,make_core_occupation
  !!***********************************************************************
  !!
  Logical(t_log) :: newsrel=.True.
  !!
Contains
  !!***********************************************************************
  !!
  !! Program for calculating the atom-like basis functions 
  !!
  Subroutine calc_basis(nsort,elements,rcut,rsite,esic&
       &,relativistic,degree,natom,nuc_type,basis,mol,calc_forces)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(inout)  :: basis
    Type(T_Mol),Intent(in)       :: mol
    Logical(t_log),Intent(in)    :: calc_forces
    Integer(t_int),Intent(in)    :: nsort,relativistic,degree,nuc_type
    Character(2),Intent(in)      :: elements(:)
    Real(t_real),Intent(in)      :: rsite(nsort),rcut(nsort)
    Real(t_real),Intent(out)     :: esic(nsort,2)
    Real(t_real),Intent(in)      :: natom(nsort)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_basis"
    Type(T_Mesh)   :: cmesh
    Logical(t_Log) :: refu,nobasis,sic4f,cconf,srel,rel,basis_exists,khsrel
    Logical(t_log) :: nrel
    Logical(t_log) :: sortJ
    character(2)  :: element
    Character(200):: fnamec
    Integer(t_int) :: issa,ir,lc,lv,icr,nmain,j0,jj,kap,degc,degv,nmaincalc
    Integer(t_int) :: j,icomp,multo,ncomp_nat
    Integer(t_int) :: restart,ivr,ivat,index,i1,i2
    Real(t_real)   :: coef(0:degree),sL
    Real(t_real)  :: wftol,ZZ,QQ,r,E,fx0,rx0,pex0,exn0,dedw,dedwq,rinf,gamma
    Real(t_real)  :: jdeg,dg,fcmax,fx2,rx2,exn2,Eq,f,fk,jsig,texp
    Real(t_real)  :: etab,vartab(1:2),etabq,vartabq(1:2),fdp,fdm
    Real(t_real),Pointer :: vat(:,:),vvv(:),bbb(:),vvvc(:),wfn(:,:,:),wkfn(:,:)
    Real(t_real),Pointer :: dwfdwn(:,:,:),wfnc(:,:,:),wkfnc(:,:)
    Real(t_real),Pointer :: gradwf(:,:,:),gradwfc(:,:,:) !! r,\pm,icomp
    Real(t_real),Pointer :: p_ec(:,:)
    Real(t_real),Pointer :: p_ev(:,:)
    Type(T_DIRACSOLVER)  :: dsdata
    Type(T_Mesh),Pointer :: atommesh,bmesh
!    Logical(t_log),Parameter :: same_nmain=.True.
    !!
    !!.......................................................................
    Nullify(p_ec)
    Nullify(p_ev)
    Nullify(vat,vvv,bbb,vvvc,wfn,gradwf,wkfn,dwfdwn,wfnc,wkfnc,gradwfc)

    !!
    !!
    srel=(relativistic == SCALAR_RELATIVISTIC)
    khsrel=(relativistic == SCALAR_RELATIVISTIC_KH)
    rel=(relativistic == FULL_RELATIVISTIC)
    nrel=(relativistic == NON_RELATIVISTIC)
    
    !! sort nominal magnetic [\kappa]\mu-states with respect to
    !! J_{\kappa}-weight, say call a state \Psi_{[\kappa],\mu}
    !! a nl(j=l+1/2),\mu state if <J^2> gives the larger j
    !! and call it a nl(j=l-1/2),\mu state if <J^2> gives the smaller j
    !! compared are the two solutions of the magnetic ODE4 for a nominal
    !! [\kappa].
    sortJ=.True.
    !!
    ncomp_nat=basis% ncomp
    If(srel) ncomp_nat=2
    !!
    basis% gam_factor=.False.
    basis% gam_factor=.True.
!    Open(12,file='=.gam')
!    Read(12,*)
!    Read(12,*)basis% gam_factor
!    Close(12)
    !!
    Call init_mesh(cmesh)
    !!
    restart=1_t_int
    refu=isoption('PLOT_REALFUNC')
    nobasis=isoption('NO_BASIS')
    sic4f=isoption('SIC_4F')
    !!
    Call palloc('p_ec',p_ec,1_t_int,basis% nRcmx,1_t_int,nsort&
         &,nmodule,nsubfunc)
    Call palloc('p_ev',p_ev,1_t_int,basis% nRvmx,1_t_int,nsort&
         &,nmodule,nsubfunc)
    !!
    esic(:,:)=0.0_t_real
    p_ev(:,:)=0.0_t_real
    p_ec(:,:)=0.0_t_real
    
    basis% p_fgam(:)      =0.0_t_real
    basis% p_fv  (:,:,:,:)=0.0_t_real
    basis% p_fkv (:,:,:,:)=0.0_t_real
    basis% p_fc  (:,:,:,:)=0.0_t_real
    basis% p_fkc (:,:,:,:)=0.0_t_real
    If(calc_forces)Then
       basis% p_fdpv(:,:,:,:)=0.0_t_real
       basis% p_fdmv(:,:,:,:)=0.0_t_real
       basis% p_fdpc(:,:,:,:)=0.0_t_real
       basis% p_fdmc(:,:,:,:)=0.0_t_real
    End If
    
    If(nobasis)Then
       Inquire(file=f_basis_dump,exist=basis_exists)
       If(basis_exists)Then
          Write(errbuf,"(''' BASIS dumping option set by user ...  ''&
               &,/,'' THIS IS A DEBUGGING OPTION!''&
               &,/,'' Reading BASIS ...'',/')")
          Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))

          !!read basis dump
          Call dump_basis(basis,nsort,calc_forces,'read')
          !!
          Goto 9999
          !!
       Else
          Write(errbuf,"(''' BASIS dumping option set by user ...  ''&
               &,/,'' THIS IS A DEBUGGING OPTION!''&
               &,/,'' BASIS dump does not exist .... &
               & calculating...'',/')")
          Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       Endif
    Endif
    !!
    !!
    !! TODO: special treatment if SIC_4F
    
    !!
    wftol=1.e-12_t_real
    !!
    !!
    !!
901 Format()
    Write(outfile,901)
    If (nrel) Then
       Write(outfile,100)    ! nonrelativistic atomic calculation
    Else If (khsrel) Then
       Write(outfile,101)    ! KH scalar relativistic atomic calculation
    Else 
       Write(outfile,102)    ! (scalar) relativistic atomic calculation
    Endif
    Write(outfile,110)
    
    !!
    !!...................................................................
    !!     loop over sorts
#if defined (BASCREATE)
    Open(88,file='+rconf') !!TODO: remove
#endif
    sortloop: Do issa=1_t_int,nsort
       !!
       bmesh    => basis% p_basmeshs(issa)
       atommesh => mol% p_atommeshs(issa)
       !!
       Call init_diracsolver(dsdata,atommesh% nr)
       !!
       !! atomic charge
       ZZ=natom(issa)
       !!Print *,'Z=',Z,issa,isa
       element=elements(issa)
       

       Call palloc('vat',vat,1_t_int,atommesh% nr,0_t_int,basis% maxNvat&
            &,nmodule,nsubfunc)
       Call palloc('vvv',vvv,1_t_int,atommesh% nr,nmodule,nsubfunc)
       Call palloc('bbb',bbb,1_t_int,atommesh% nr,nmodule,nsubfunc)
       Call palloc('vvvc',vvvc,1_t_int,atommesh% nr,nmodule,nsubfunc)
       Call palloc('wfn',wfn,1_t_int,atommesh% nr,1_t_int,4_t_int&
            &,1_t_int,2_t_int,nmodule,nsubfunc)
       Call palloc('gradwf',gradwf,1_t_int,atommesh% nr,1_t_int,2_t_int&
            &,1_t_int,4_t_int,nmodule,nsubfunc)
       Call palloc('wkfn',wkfn,1_t_int,atommesh% nr&
            &,1_t_int,4_t_int,nmodule,nsubfunc)
       Call palloc('dwfdwn',dwfdwn,1_t_int,atommesh% nr,1_t_int,4_t_int&
            &,1_t_int,2_t_int,nmodule,nsubfunc)
       Call palloc('wfnc',wfnc,1_t_int,atommesh% nr,1_t_int,4_t_int&
            &,1_t_int,2_t_int,nmodule,nsubfunc)
       Call palloc('wkfnc',wkfnc,1_t_int,atommesh% nr&
            &,1_t_int,4_t_int,nmodule,nsubfunc)
       Call palloc('gradwfc',gradwfc,1_t_int,atommesh% nr,1_t_int,2_t_int&
            &,1_t_int,4_t_int,nmodule,nsubfunc)
       !!
       !!
       pex0=0.8_t_real
       !!
       fx0=0.0_t_real
       rx0=pex0*rcut(issa)
       exn0=18.0_t_real
       !!
       !!
       fx2=1.0_t_real
       rx2=basis% p_pex(issa)*rcut(issa)
       exn2=basis% p_exn2(issa)
       
       !!...............................................................
       !! atomic potential 
       
       Do ivat=0,basis% p_Nvat(issa)
          !!
          If(ivat==0)Then
             QQ=0.0_t_real
          Else
             QQ=basis% p_Qvat(ivat,issa)
          Endif
          !!
          irloop: Do ir=1_t_int,atommesh% nr
             r=atommesh% p_rmesh(ir)
             !!
             vat(ir,ivat)=minterpol(atommesh,degree&
                  &,mol% p_vatom(:,ivat,issa),r)/sq4pi
             !!
             !!
          Enddo irloop ! ir
       Enddo
       !!
       bbb(:)=0.0_t_real
       !!
       If(refu)Then
          Do ivat=0,basis% p_Nvat(issa)
             Write(fnamec,911)issa,ivat
             Open(12,file=fnamec,status='unknown')
             Do lv=0_t_int,3_t_int
                Do ir=2_t_int,atommesh% nr
                   r=atommesh% p_rmesh(ir)
                   dg=(vat(ir,ivat)&
                        &+0.5_t_real*lv*(lv+1_t_int)/r)/r
                   If(Abs(dg) < 50.0_t_real)Write(12,912)r,dg
                Enddo             ! lv
                Write(12,920)
             Enddo               ! ir
             Close(12)
          Enddo
911       Format('+atpot.',i3.3,'.',i3.3)
912       Format(2(e30.15e3,2x))
920       Format(' ')
       Endif                 ! refu
       !!         
       !!..............................................................
       !!         calculation of core-states
       !!         loop over core orbitals
       coreloop: Do icr=1_t_int,basis% p_nRc(issa)
          !! main quantum number
          nmain=basis% p_nnRc(icr,issa)
          !! kappa qn
          kap=basis% p_kapRc(icr,issa)
          !! total angular momentum
          j=2*Abs(kap)-1
          !! angular momentum
          lc=(Abs(2*kap+1)-1)/2
          sL=Sqrt(Real(lc*(lc+1),t_real))
          degc=2*lc+1
          !!
          ivat=basis% p_vatclsc(icr,issa)
          If(ivat==0) Stop 'ivat==0'
          !!
          QQ=basis% p_Qvat(ivat,issa)
          etab=-(0.5_t_real*(ZZ/(nmain))**2)
          
          j0=2_t_int
          If(srel .And. lc > 0_t_int ) j0=1_t_int
          !! loop over spin orbit quantum number kappa (if scalar relativistic)
          !! for full relativistic kappa is included in the icr loop by array 
          !! organization
          Jloop: Do jj=j0,2_t_int
             !!
             jdeg=1.0_t_real
             jsig=1.0_t_real
             If(srel)Then
                kap=lc-(jj-1_t_int)*degc
                j=2*Abs(kap)-1
                ! degeneracy of relativistic state
                jdeg=Abs(kap)/Real(degc,t_real)
                jsig=Sign(1_t_int,kap)/Real(degc,t_real)
                If(lc==0) jsig=0.0_t_real
             Endif
             !!             
             vvv(:)=vat(:,ivat)
             !!SIC---------SIC---------------------------
             sic4fif: If(sic4f) Then
                Call errmsg(nmodule,nsubfunc,E_fatalerror&
                     &,msg="'SIC is not yet finaly implemented!'")
                
                !!SIC                if(nmain.eq.4_t_int.and.lc.eq.3_t_int)then
                !!SIC                  if(scfloop.lt.2_t_int)then
                !!SIC                    do ir=1_t_int,nra
                !!SIC                      r=ra(ir)
                !!SIC                      vvv(ir)=v(ir)+exp(-r)-1.d0
                !!SIC                    enddo       ! ir
                !!SIC!!                    open(19,file='vsic0')
                !!SIC!!                    do ir=1_t_int,nra
                !!SIC!!                      write(19,*)ra(ir),v(ir),vvv(ir)
                !!SIC!!                    enddo
                !!SIC!!                    close(19)
                !!SIC                  else          ! scfloop
                !!SIC                    do ir=1_t_int,nra
                !!SIC                      r=ra(ir)
                !!SIC                      vsic(ir)=orbdens(ir,issa,ms)*jac(...)
                !!SIC                    enddo       ! ir
                !!SIC                    call simpson(nra,vsic,dr,ddum)
                !!SIC                    if(dabs(ddum-1.d0).gt.1.d-6)then
                !!SIC                      write(outfile,*)'norm of 4f-orbital density ne 1'
                !!SIC                    endif
                !!SIC                    call sicpot(orbdens(1,issa,ms),nra,ra,xa,dr,vsic
                !!SIC     &               ,bsic,v,bf,vvv,bbb,nonrelat,C,ms,mspin,ddum)
                !!SIC                    do ir=1,nra
                !!SIC                      vvv(ir)=vvv(ir)+bbb(ir)*(3-2*ms)
                !!SIC                      bbb(ir)=0.d0
                !!SIC                    enddo
                !!SIC!!                    open(19,file='vsic')
                !!SIC!!                    do ir=1,nra
                !!SIC!!                      write(19,*)ra(ir),v(ir),vvv(ir)
                !!SIC!!                    enddo
                !!SIC!!                    close(19)
                !!SIC
                !!SIC                  endif         ! scfloop
                !!SIC                endif           ! 4f-state
             Endif sic4fif             ! sic4f
             !!SIC---------end SIC---------------------------
             !! calculate state
             !! usually
             !! additional core compression should not be neccesarry
             !! if so, the concepts of core-valence orthog. 
             !! may be demaged
             fx2=0.0_t_real
             !!
             !!
             !!TODO: wfn
             !! select state to correct
             !! 
             vvvc=vvv
             cmesh% Type=atommesh% Type
             cmesh% with_zero=atommesh% with_zero
             cmesh% nr=atommesh% nr
             cmesh% rmin=atommesh% rmin
             cmesh% rmax=atommesh% rmax
             !! 
             cconf=.True.
             cconf=.False.
             rinf=rsite(issa)*1.5_t_real
!!$                If(basis% p_IOcnfc(ic0,issaIO)==CORE_CONF_INF&
!!$                     &.And. Abs(basis% p_IOrcinf(ic0,issaIO))>1.0e-3_t_real)Then
!!$                   !! core confinment
!!$                   rinf=basis% p_IOrcinf(ic0,issaIO)
!!$                   cconf=.True.
!!$                Endif
             If(cconf)Then
                cmesh% rmax=rinf
                Write(outfile,*) 'Core confinement rinf=',rinf
             Endif
             !!
             Call setup_mesh(cmesh)
             !! 
             If(cconf)Then
                Call mreinter(9_t_int,atommesh,vvv,cmesh,vvvc)
                !Call writefunc(atommesh% nr,atommesh% p_rmesh&
                !     &,vvv,12_t_int,'+vvv')
                !Call writefunc(cmesh% nr,cmesh% p_rmesh&
                !     &,vvvc,12_t_int,'+vvvc')
                !!
             Endif
             !!
             vartab(1:2)=1.0_t_real
             Call wfdirac(cmesh,wfnc,wkfnc,dwfdwn,dedw,dedwq&
                  &,fx0,rx0,exn0,fx2,rx2,exn2,nmain,lc,ZZ&
                  &,vvvc,bbb,0_t_int,relativistic,kap&
                  &,etab,etabq,E,Eq&
                  &,vartab,vartabq&
                  &,2_t_int,wftol,restart,sortJ,dsdata,nuc_type&
                  &,gamma,basis% gam_factor,gradwfc)
             !!
             !!
             If(Abs(basis% p_fgam(issa))<1.0e-10_t_real)Then
                basis% p_fgam(issa)=gamma
             Else
                If(Abs(gamma-basis% p_fgam(issa))>1.0e-10_t_real)Then
898                Format("'multiple gamm-values detected, not yet impl&
                        &emented, sorry!',/,'sort ",i5," gamma1="&
                        &,g30.15e3," gamma2=",g30.15e3,"'")
                   Write(errbuf,898) issa,gamma,basis% p_fgam(issa) 
                   Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
                Endif
             Endif

             If(cconf)Then
                Do icomp=1, ncomp_nat
                   Call mreinter(9_t_int,cmesh,wfnc(:,icomp,1)&
                        &,atommesh,wfn(:,icomp,1))
                   Call mreinter(9_t_int,cmesh,wkfnc(:,icomp)&
                        &,atommesh,wkfn(:,icomp))
                   Call mreinter(9_t_int,cmesh,gradwfc(:,1,icomp)&
                        &,atommesh,gradwf(:,1,icomp))
                   Call mreinter(9_t_int,cmesh,gradwfc(:,2,icomp)&
                        &,atommesh,gradwf(:,2,icomp))
                Enddo
             Else
                wfn=wfnc
                wkfn=wkfnc
                gradwf=gradwfc
             Endif
             !!
             Call cleanup_mesh(cmesh)
             
             
             !!SIC---------SIC---------------------------
             !!4f-orbital density
             sic4fif2: If(sic4f) Then
                !!SIC                if(nmain.eq.4.and.lc.eq.3)then
                !!SIC                  do ir=1,nra
                !!SIC                    r=ra(ir)
                !!SIC                    x=xa(ir)
                !!SIC                    if(nonrelat)then
                !!SIC                      orbdens(ir,issa,ms)=wg(ir,1)**2
                !!SIC                    else
                !!SIC                      stop 'SIC not yet scalar relativistic!'
                !!SIC                    endif
                !!SIC                  enddo         ! ir
                !!SIC                endif           ! 4f-state
             Endif sic4fif2             ! sic4f
             !!SIC---------end SIC---------------------------                
             !! output
             multo=coreMultIndex(icr,issa,basis)
             If (nrel .Or. khsrel) Then
                Write(outfile,115)element,nmain,lqnchar(lc),multo,e &
                     & ,'     none    ','  core',issa
             Else
                Write(outfile,116)element,nmain,lqnchar(lc),multo,j,e&
                     & ,'     none    ', '  core',issa
             Endif
             
             !!..........................................................
             !! reinterpolate the values of functions at the radial-mesh
             !!    
             !!
             !!
!             Write(outfile,12)icr,jj,nmain,lqnchar(lc),j
!12           Format(i5,1x,i2,' : ',i1,a1,i1,'/2')
             !!
             !!
             Do ir=1_t_int,bmesh% nr
                r=bmesh% p_rmesh(ir)
                
                
                If (srel) Then
                   !! SREL: components are averaged with resp. to J
                   !!
                   Do icomp=1,2
                      f= minterpol(atommesh,degree,wfn(:,icomp,1_t_int),r)
                      fk= minterpol(atommesh,degree,wkfn(:,icomp),r)
                           
                      if(newsrel) fk=fk-E*r*f
                      
                      !! averages
                      basis% p_fc(ir,icr,icomp,issa)=&
                           &basis% p_fc(ir,icr,icomp,issa)+f*jdeg
                      basis% p_fkc(ir,icr,icomp,issa)=&
                           &basis% p_fkc(ir,icr,icomp,issa)+fk*jdeg

                      fdp=minterpol(atommesh,degree,gradwf(:,1,icomp),r)
                      fdm=minterpol(atommesh,degree,gradwf(:,2,icomp),r)

                      If(calc_forces) Then
                         basis% p_fdpc(ir,icr,icomp,issa)=&
                              &basis% p_fdpc(ir,icr,icomp,issa)+fdp*jdeg
                         basis% p_fdmc(ir,icr,icomp,issa)=&
                              &basis% p_fdmc(ir,icr,icomp,issa)+fdm*jdeg
                      End If
                      
                      
                      !! difference averages
                      basis% p_fc(ir,icr,icomp+2,issa)=&
                           &basis% p_fc(ir,icr,icomp+2,issa)+f*jsig*sL
                      basis% p_fkc(ir,icr,icomp+2,issa)=&
                           &basis% p_fkc(ir,icr,icomp+2,issa)+fk*jsig*sL
                      
                      If(calc_forces) Then
                         basis% p_fdpc(ir,icr,icomp+2,issa)=&
                              &basis% p_fdpc(ir,icr,icomp+2,issa)+fdp*jsig*sL
                         basis% p_fdmc(ir,icr,icomp+2,issa)=&
                              &basis% p_fdmc(ir,icr,icomp+2,issa)+fdm*jsig*sL
                      End If
                      
                      
                   Enddo !! icomp
                   
                Else !! non, KH scalar and full relativistic
                   Do icomp=1,basis% ncomp
                      basis% p_fc(ir,icr,icomp,issa)=&
                           & minterpol(atommesh,degree,wfn(:,icomp,1_t_int),r)
                      basis% p_fkc(ir,icr,icomp,issa)=&
                           & minterpol(atommesh,degree,wkfn(:,icomp),r)

                      If(calc_forces) Then
                         basis% p_fdpc(ir,icr,icomp,issa)=&
                              & minterpol(atommesh,degree,gradwf(:,1,icomp),r)
                         basis% p_fdmc(ir,icr,icomp,issa)=&
                              & minterpol(atommesh,degree,gradwf(:,2,icomp),r)
                      End If
                      
                   Enddo
                Endif
             Enddo             !ir
             !!..........................................................
             !! sic kinetic contribution
             !!..........................................................
             !!SIC---------SIC---------------------------
             If(sic4f)Then
                !!SIC                if(nmain.eq.4.and.lc.eq.3)then
                !!SIC                  do ir=1,nra
                !!SIC                    wg(ir,1)=-(orbdens(ir,issa,ms)*(vsic(ir)+bsic(ir)
                !!SIC     &               *(3-2*ms)))/r(...)*jac(...)
                !!SIC                  enddo
                !!SIC                  call simpson(nra,wg,dr,ddum)
                !!SIC                  print *,'kinetic SIC: ',ddum
                !!SIC                  esic(issa,ms)=-ddum
                !!SIC                  call sicpot(orbdens(1,issa,ms),nra,ra,xa,dr,vsic
                !!SIC     &             ,bsic,v,bf,vvv,bbb,nonrelat,C,ms,mspin,ddum)
                !!SIC                  print *,'potential SIC: ',ddum
                !!SIC                  esic(issa,ms)=esic(issa,ms)-ddum
                !!SIC                endif
             Endif
             !!SIC---------end SIC---------------------------
             !!.......................................................
             if (srel) Then
                !! J-averaged energies
                p_ec(icr,issa)=p_ec(icr,issa)+E*jdeg 
             Else
                p_ec(icr,issa)=E
             Endif
          Enddo Jloop               ! j

          !! correct SREL kinetic function according to newer formula
          If (srel .And. newsrel) Then
             Do ir=1_t_int,bmesh% nr
                r=bmesh% p_rmesh(ir)
                Do icomp=1,basis% ncomp
                   basis% p_fkc(ir,icr,icomp,issa)=&
                        & basis% p_fkc(ir,icr,icomp,issa)&
                        &+p_ec(icr,issa)*r*basis% p_fc(ir,icr,icomp,issa)
                Enddo
             Enddo
          Endif
          !!
          
       Enddo coreloop                 ! icr
       !!...............................................................
       !! calculation of valence-states
       !! loop over valence states
       valenceloop: Do ivr=1_t_int,basis% p_nRv(issa)
          !! main quantum number
          nmain=basis% p_nnRv(ivr,issa)
          nmaincalc=nmain
!          If(same_nmain) Then
!             Read(basis% p_firstRv(ivr,issa),567)nmaincalc
!567          Format(i1)
!          Endif
          !! kappa qn
          kap=basis% p_kapRv(ivr,issa)
          !! total angular momentum
          j=2*Abs(kap)-1
          !! angular momentum
          lv=(Abs(2*kap+1)-1)/2
          sL=Sqrt(Real(lv*(lv+1),t_real))
          degv=2*lv+1
          !!
          ivat=basis% p_vatclsv(ivr,issa)
          If(ivat==0) Stop 'ivat==0'
          !!
          QQ=basis% p_Qvat(ivat,issa)
          etab=-(0.5_t_real*(ZZ/(nmaincalc))**2)
          fx2=1.0_t_real

          rx2=basis% p_pex(issa)*rcut(issa)*basis% p_pexRv(ivr,issa)

          If(Abs(rx2)==0.0_t_real) Then
             fx2=0.0_t_real
             rx2=1.0_t_real
          Endif

          !Print *,'confinement pex,exn2=',basis% p_pex(issa),exn2
          !Print *,'confinement ivr     =',basis% p_pexRv(ivr,issa)




          j0=2_t_int
          If(srel .And. lv > 0_t_int) j0=1_t_int
          !! loop over spin orbit quantum number kappa (if scalar relativistic)
          !! for full relativistic kappa is included in the ivr loop by array 
          !! organization
          Jloop2: Do jj=j0,2_t_int
             !!
             jdeg=1.0_t_real
             jsig=1.0_t_real
             If(srel)Then
                kap=lv-(jj-1_t_int)*degv
                j=2*Abs(kap)-1
                !! degeneracy of relativstistic state
                jdeg=Abs(kap)/Real(degv,t_real)
                jsig=Sign(1_t_int,kap)/Real(degv,t_real)
                If(lv==0) jsig=0.0_t_real
             Endif
             !! calculate state
             !!
#if defined (BASCREATE)
             If(jj==j0 .And. ivr ==1)Then !!TODO: remove
                Write(88,987) issa,basis% p_pex(issa)*rcut(issa),exn2
987             Format(i5,2(f30.6,2x))
             Endif
#endif
             !!
             vartab(1:2)=1.0_t_real
             Call wfdirac(atommesh,wfn,wkfn,dwfdwn,dedw,dedwq&
                  &,fx0,rx0,exn0,fx2,rx2,exn2,nmaincalc,lv,ZZ&
                  &,vat(:,ivat),bbb,0_t_int&
                  &,relativistic,kap&
                  &,etab,etabq,E,Eq&
                  &,vartab,vartabq&
                  &,2_t_int,wftol,restart,sortJ,dsdata,nuc_type&
                  &,gamma,basis% gam_factor,gradwf)
             
             !Print *,'gamma=',gamma
             If(Abs(basis% p_fgam(issa))<1.0e-10_t_real)Then
                basis% p_fgam(issa)=gamma
             Else
                If(Abs(gamma-basis% p_fgam(issa))>1.0e-10_t_real)Then
                   Write(errbuf,898)issa,gamma,basis% p_fgam(issa) 
                   Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
                Endif
             Endif
             !! output
             If (nrel.Or.khsrel) Then
                If(Abs(fx2)> 1.0e-6_t_real)Then
                   Write(outfile,120)element,nmain,lqnchar(lv),e&
                        &,rx2,exn2&
                        &,'valence',issa
                Else
                   Write(outfile,1115)element,nmain,lqnchar(lv),e &
                        & ,'     none    ', 'valence',issa
                Endif
                
             Else
                If(Abs(fx2)> 1.0e-6_t_real)Then
                   Write(outfile,121)element,nmain,lqnchar(lv),j,e&
                        &,rx2,exn2&
                        &,'valence',issa
                Else
                   Write(outfile,1116)element,nmain,lqnchar(lv),j &
                        & ,e,'     none    ', 'valence',issa
                Endif
             Endif
             !!........................................................
             !! reinterpolate the values of functions at the radial-mesh
             !!

             !Write(outfile,12)ivr,jj,nmain,lqnchar(lv),j
             !!
             !!
             Do ir=1_t_int,bmesh% nr
                r=bmesh% p_rmesh(ir)

                index=mintpl_coefs(degree,atommesh,r,coef)
                i1=index
                i2=index+degree
                !!
                If (srel) Then
                   !! SREL: large component is averaged with resp. to J
                   !!
                   Do icomp=1,2

                      f =Dot_product(wfn (i1:i2,icomp,1),coef(0:degree))
                      fk=Dot_product(wkfn(i1:i2,icomp  ),coef(0:degree))

                      if(newsrel) fk=fk-E*r*f


                      !! averages
                      basis% p_fv(ir,ivr,icomp,issa)=&
                           &basis% p_fv(ir,ivr,icomp,issa)+f*jdeg
                      basis% p_fkv(ir,ivr,icomp,issa)=&
                           &basis% p_fkv(ir,ivr,icomp,issa)+fk*jdeg


                      

                      If(calc_forces) Then

                         fdp=Dot_product(gradwf(i1:i2,1,icomp),coef(0:degree))
                         fdm=Dot_product(gradwf(i1:i2,2,icomp),coef(0:degree))


                         basis% p_fdpv(ir,ivr,icomp,issa)=&
                              &basis% p_fdpv(ir,ivr,icomp,issa)+fdp*jdeg
                         basis% p_fdmv(ir,ivr,icomp,issa)=&
                              &basis% p_fdmv(ir,ivr,icomp,issa)+fdm*jdeg
                      End If
                      

                      !! difference averages
                      basis% p_fv(ir,ivr,icomp+2,issa)=&
                           &basis% p_fv(ir,ivr,icomp+2,issa)+f*jsig*sL
                      basis% p_fkv(ir,ivr,icomp+2,issa)=&
                           &basis% p_fkv(ir,ivr,icomp+2,issa)+fk*jsig*sL
                      If(calc_forces) Then
                         basis% p_fdpv(ir,ivr,icomp+2,issa)=&
                              &basis% p_fdpv(ir,ivr,icomp+2,issa)+fdp*jsig*sL
                         basis% p_fdmv(ir,ivr,icomp+2,issa)=&
                              &basis% p_fdmv(ir,ivr,icomp+2,issa)+fdm*jsig*sL
                      End If
                         

                   Enddo !! icomp
                   
                Else !! non, KH scalar and full relativistic
                   Do icomp=1,basis% ncomp

                      f =Dot_product(wfn (i1:i2,icomp,1),coef(0:degree))
                      fk=Dot_product(wkfn(i1:i2,icomp  ),coef(0:degree))

                      basis% p_fv (ir,ivr,icomp,issa)=f 
                      basis% p_fkv(ir,ivr,icomp,issa)=fk

                      If(calc_forces) Then

                         fdp=Dot_product(gradwf(i1:i2,1,icomp),coef(0:degree))
                         fdm=Dot_product(gradwf(i1:i2,2,icomp),coef(0:degree))

                         basis% p_fdpv(ir,ivr,icomp,issa)=fdp
                         basis% p_fdmv(ir,ivr,icomp,issa)=fdm
                      End If
                   End Do
                Endif           ! nonrelat
             Enddo             ! ir
             If (srel) Then
                !! J-averaged energies
                p_ev(ivr,issa) = p_ev(ivr,issa)+e*jdeg
             Else
                p_ev(ivr,issa)=e
             Endif             ! nonrelat
          Enddo Jloop2               ! j

          !! correct SREL kinetic function according to newer formula
          If (srel .And. newsrel) Then
             Do ir=1_t_int,bmesh% nr
                r=bmesh% p_rmesh(ir)
                Do icomp=1,basis% ncomp
                   basis% p_fkv(ir,ivr,icomp,issa)=&
                        & basis% p_fkv(ir,ivr,icomp,issa)&
                        &+p_ev(ivr,issa)*r*basis% p_fv(ir,ivr,icomp,issa)
                Enddo
             Enddo
          Endif
          !!
          
       Enddo valenceloop                 ! ivr
       !!
       Call clean_diracsolver(dsdata)
    Enddo sortloop                   ! issa
#if defined (BASCREATE)
    Close(88)
#endif
    !!
    Call cleanup_mesh(cmesh)
    !!
    !!
    !!
    !! output of scalarrelativistic atomic energies
    If (srel) Then
       Write(outfile,101)
       Do issa=1_t_int,nsort
          Do icr=1_t_int,basis% p_nRc(issa)
             fx2=0.0_t_real
             kap=basis% p_kapRc(icr,issa)
             lc=(Abs(2*kap+1)-1)/2 ! angular momentum
             nmain=basis% p_nnRc(icr,issa) ! main quantum number
             multo=coreMultIndex(icr,issa,basis)
             Write(outfile,115)elements(issa),nmain,lqnchar(lc),multo,&
                  & p_ec(icr,issa),'     none    ', '  core',issa
          Enddo               ! icr
          Do ivr=1_t_int,basis% p_nRv(issa)
             fx2=1.0_t_real
             kap=basis% p_kapRv(ivr,issa)
             lv=(Abs(2*kap+1)-1)/2 ! angular momentum
             nmain=basis% p_nnRv(ivr,issa) ! main quantum number
             e=p_ev(ivr,issa)
             If(Abs(fx2)  > 1.e-5_t_real)Then
                rx2=basis% p_pex(issa)*rcut(issa)*basis% p_pexRv(ivr,issa)
                Write(outfile,120)elements(issa),nmain,lqnchar(lv),e&
                     &,rx2,exn2,'valence',issa
             Else
                Write(outfile,1115)elements(issa),nmain,lqnchar(lv),e &
                     & ,'     none    ', 'valence',issa
             Endif
          Enddo               ! ivr
       Enddo                 ! issa
    Endif                     ! scalrel
    Write(outfile,130)
    !!
    !!
    !!
    !basis% p_fc (:,:,3:4,:)=0.0_t_real
    !basis% p_fkc(:,:,3:4,:)=0.0_t_real
    !basis% p_fv (:,:,3:4,:)=0.0_t_real
    !basis% p_fkv(:,:,3:4,:)=0.0_t_real
    !!
    !!
    Call ortho_bas(nsort,basis,rel,calc_forces)
    !!
    !!
    !!
    !!
    !! make the throw-away approx in SREL now
    If(srel.And..Not.newsrel) Then
       Do issa=1,nsort
          Do ivr=1,basis% p_nRv(issa)
             If(basis% ncomp >2) Then
                If( basis% p_tRv(ivr,issa) /= ORB_TYPE_SEMICORE )Then
                   basis% p_fv (:,ivr,3:4,issa)=0.0_t_real
                   basis% p_fkv(:,ivr,3:4,issa)=0.0_t_real
                   If(calc_forces) Then
                      basis% p_fdpv(:,ivr,3:4,issa)=0.0_t_real
                      basis% p_fdmv(:,ivr,3:4,issa)=0.0_t_real
                   Endif
                Endif
             Endif
          Enddo
       Enddo
    End If
    !!

    If(nobasis)Then
       Write(errbuf,"(''' dumping BASIS .... '',/')")
       Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       !!write basis dump
       Call dump_basis(basis,nsort,calc_forces,'write')
    Endif


9999 Continue


    !!
    !! check basis states
    Call spin_character(nsort,elements,basis,relativistic)
    Call bascheck(nsort,elements,basis,relativistic)
    !!
    !!check core overlap at rsite
    Do issa=1_t_int,nsort
       bmesh => basis% p_basmeshs(issa)
       r=rsite(issa)
       texp=r**basis% p_fgam(issa)
       Do icr=1_t_int,basis% p_nRc(issa)
          
          kap=basis% p_kapRc(icr,issa)
          nmain=basis% p_nnRc(icr,issa)
          j=2*Abs(kap)-1
          lc=(Abs(2*kap+1)-1)/2
          multo=coreMultIndex(icr,issa,basis)
          fcmax=0.0_t_real
          !Print "('icr=',i4,1x,i1,a1,i1,'/2')",icr,nmain,lqnchar(lc),j
          Do icomp=1_t_int,basis% ncomp
             fcmax= Max(fcmax,Abs(minterpol(bmesh,degree&
                  &,basis% p_fc(1_t_int:bmesh% nr,icr,icomp,issa)&
                  &,r)))*texp
          Enddo !! 
          If(fcmax > 0.01_t_real)Then
             Write(errbuf,201)nmain,lqnchar(lc),multo&
                  &,elements(issa),issa,r,fcmax
             Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
          Endif
       Enddo ! icr
    Enddo ! issa
    !!
    !!
    !!
    !!
    !!
201 Format('''The core states ',i2,a1,i1,' of atom ',a2,' sort '&
         &,i4,' at R_NN/2=',f10.4,''',/,'' seem to be rather big: '&
         &,e15.4e2,''',/,'' I suggest to treat it as semi core !''')
130 Format(72('-'))
100 Format(72('-')/10('='),t18,'NONRELATIVISTIC ATOMIC ENERGIES',t63,10('='))
101 Format(72('-')/10('='),t18,'SCALARRELATIVISTIC ATOMIC ENERGIES'&
         &,t63,10('='))
102 Format(72('-'),/,10('='),t20,'RELATIVISTIC ATOMIC ENERGIES',t63,10('='))
110 Format(72('-')/'|',t3,'ATOM',t9,'ELECTRON',t28,&
          & 'ENERGY',t40,'COMPRESSION',t56,'TYPE ',t64,'SORT',t72,'|')
115 Format(72('-')/'|',t4,a2,t11,i1,a1,i1,t23,f16.8,t40,a,t55,a,t63,i3,t72,'|')
1115 Format(72('-')/'|',t4,a2,t11,i1,a1,t23,f16.8,t40,a,t55,a,t63,i3,t72,'|')
116  Format(72('-')/'|',t4,a2,t11,i1,a1,i1,1x,i1,'/2',t23,f16.8,t40,a,t55&
          &,a,t63,i3,t72,'|')
1116 Format(72('-')/'|',t4,a2,t11,i1,a1,1x,i1,'/2',t23,f16.8,t40,a,t55&
          &,a,t63,i3,t72,'|')
120 Format(72('-')/'|',t4,a2,t11,i1,a,t23,f16.8,t40,&
         & f6.3,2x,f6.3,t55,a,t63,i3,t72,'|')
121 Format(72('-')/'|',t4,a2,t11,i1,a1,1x,i1,'/2',t23,f16.8,t40&
         &,f6.3,2x,f6.3,t55,a,t63,i3,t72,'|')
    !!
    !!
    !Stop 'rcinf test'

    

    !!
    !! determine the lowest valence atom level
    !! for semicores this will be the lowest peak in the DOS
    basis% valence_erg_bottom=1.e30_t_real
    Do issa=1_t_int,nsort
       Do ivr=1_t_int,basis% p_nRv(issa)
          basis% valence_erg_bottom=&
               &Min(basis% valence_erg_bottom,p_ev(ivr,issa))
       End Do
    End Do
    !!
    !!
    !!
    !!
    Call printbasis(nsort,elements,basis,relativistic,calc_forces,mol)
    !!
    Call pfree(vat)
    Call pfree(vvv)
    Call pfree(bbb)
    Call pfree(vvvc)
    Call pfree(wfn)
    Call pfree(gradwf)
    Call pfree(wkfn)
    Call pfree(dwfdwn)
    Call pfree(wfnc)
    Call pfree(wkfnc)
    Call pfree(gradwfc)
    Call pfree(p_ev)
    Call pfree(p_ec)
    !!
    !!
    !Stop
    Return
    !!
  End Subroutine calc_basis
  !!***********************************************************************
  Subroutine  ortho_bas(nsort,basis,rel,calc_forces)
    !!
    Implicit none  
    !!IO
    Type(T_Basis),Intent(inout)  :: basis
    Logical(t_log),Intent(in)    :: rel,calc_forces
    Integer(t_int),Intent(in)    :: nsort
    !!LOCAL
    Character(*),Parameter::nsubfunc="ortho_bas"
    Logical(t_log) :: prot,second
    Integer(t_int) :: ivr,jvr,issa,nv,nr,ierr,icomp,ir,j,ncomp,lv
    Integer(t_int) :: jcr,nc,kapi,kapj,i,nchi,nc1,kp,kp1
    Real(t_real)   :: tol
    Real(t_real),Pointer :: p_S(:,:),p_Sc1v(:,:),p_Svv(:,:)
    Real(t_real),Pointer :: p_S11(:,:),p_S12(:,:),p_S22(:,:)
    Real(t_real),Pointer :: p_tr(:),p_texp(:)
    Type(T_Mesh),Pointer :: bmesh
    !!
    prot=(outlevel >= L_many) 
    !!
    If(prot) Write(outfile,1)'START:',Trim(nsubfunc)
    tol=1.0e-10_t_real
    !!
    !!
    Nullify(p_tr)
    Nullify(p_texp)
    Nullify(p_S)
    Nullify(p_S11)
    Nullify(p_Sc1v)
    Nullify(p_Svv)
    Nullify(p_S12)
    Nullify(p_S22)
    !!
    ncomp=basis% ncomp
    !!
    issaloop: Do issa=1_t_int,nsort
       !!
       nc=basis% p_nRc(issa)
       nchi=basis% p_nRchi(issa)
       nc1=nc-nchi
       !!
       bmesh => basis% p_basmeshs(issa)
       nr=bmesh% nr
       !!
       Call palloc('p_tr'  ,p_tr  ,1_t_int,nr,nmodule,nsubfunc)
       Call palloc('p_texp',p_texp,1_t_int,nr,nmodule,nsubfunc)
       !!
       p_texp(1:nr)=bmesh% p_rmesh(1:nr)**(basis% p_fgam(issa))
       !!

       If(prot) Then
          Write(outfile,13) issa
          Write(outfile,14) nchi,nc,nc1
       Endif
       !!
       Call palloc('p_S'  ,p_S  ,1_t_int,nc ,1_t_int,nc ,nmodule,nsubfunc)
       Call palloc('p_S11',p_S11,1_t_int,nchi,1_t_int,nchi,nmodule,nsubfunc)
       If(nc1>0)Then
          Call palloc('p_S12',p_S12,1_t_int,nchi,1_t_int,nc1,nmodule,nsubfunc)
          Call palloc('p_S22',p_S22,1_t_int,nc1,1_t_int,nc1,nmodule,nsubfunc)
       Endif
       !!
       ortholoopc: Do i=1,2        !! repeat ortho two times .... improves errors
          !!
          !!
          If(prot) Then
             Write(outfile,15) 
             Write(outfile,16) i
          Endif
          !!
          !!
          !!
          Call create_overlap(bmesh,p_texp,1_t_int,nchi,1_t_int,nchi&
               &,basis% p_kapRc(:,issa)&
               &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
               &,basis% p_fc(:,:,:,issa),p_tr,p_S11)
          !!
          !!
          !!
          If(prot) Then
             Call printmat("S11","(e15.5e2,2x)","('         0       ')"&
                  &,p_S11,nchi,nchi,tol)
             Write(outfile,*)
          Endif
          !!
          !
          Call rbchd (p_S11,nchi,ierr)
          !!
          If(ierr /= 0_t_int)Then
             Write(errbuf,"('''RBCHD: overlap matrix indefinite'',/&
                  &,'' possible reasons:  '',/&
                  &,'' (1) bad basis definitions''')")
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          End If
          !!
          If(prot) Then
             Call printmat("S11R","(e15.5e2,2x)","('         0       ')"&
                  &,p_S11,nchi,nchi,tol)
             Write(outfile,*)
          Endif
          !!
          !!
          Do ir=1,bmesh% nr
             Do icomp=1,ncomp
                Do jcr=1,nchi
                   j=jcr
                   !!
                   basis% p_fc(ir,jcr,icomp,issa)=&
                        &(basis% p_fc(ir,jcr,icomp,issa)&
                        &-Sum(basis% p_fc(ir,1:jcr-1,icomp,issa)&
                        &*p_S11(1:j-1,j)))/p_S11(j,j)
                   !!
                   basis% p_fkc(ir,jcr,icomp,issa)=&
                        &(basis% p_fkc(ir,jcr,icomp,issa)&
                        &-Sum(basis% p_fkc(ir,1:jcr-1,icomp,issa)&
                        &*p_S11(1:j-1,j)))/p_S11(j,j)
                   !!
                   If(calc_forces) Then
                      basis% p_fdpc(ir,jcr,icomp,issa)=&
                           &(basis% p_fdpc(ir,jcr,icomp,issa)&
                           &-Sum(basis% p_fdpc(ir,1:jcr-1,icomp,issa)&
                           &*p_S11(1:j-1,j)))/p_S11(j,j)
                      !!
                      basis% p_fdmc(ir,jcr,icomp,issa)=&
                           &(basis% p_fdmc(ir,jcr,icomp,issa)&
                           &-Sum(basis% p_fdmc(ir,1:jcr-1,icomp,issa)&
                           &*p_S11(1:j-1,j)))/p_S11(j,j)
                   Endif
                   !!
                Enddo
             Enddo
          Enddo

          Call create_overlap(bmesh,p_texp,1_t_int,nchi,1_t_int,nchi&
               &,basis% p_kapRc(:,issa)&
               &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
               &,basis% p_fc(:,:,:,issa),p_tr,p_S11)



          If(prot) Then
             Call printmat("S11","(e15.5e2,2x)","('         0       ')"&
                  &,p_S11,nchi,nchi,tol)
             Write(outfile,*)
          Endif

       Enddo ortholoopc


       If(nc1>0)Then
          ortholoopc1: Do i=1,2

             If(prot) Then
                Write(outfile,15) 
                Write(outfile,16) i
             Endif

             Call create_overlap(bmesh,p_texp,1_t_int,nchi,nchi+1,nc&
                  &,basis% p_kapRc(:,issa)&
                  &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
                  &,basis% p_fc(:,:,:,issa),p_tr,p_S12)

             Call create_overlap(bmesh,p_texp,nchi+1,nc,nchi+1,nc,basis% p_kapRc(:,issa)&
                  &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
                  &,basis% p_fc(:,:,:,issa),p_tr,p_S22)

             If(prot) Then

                Call printmat("S12","(e15.5e2,2x)","('         0       ')"&
                     &,p_S12,nchi,nc1,tol)
                Write(outfile,*)

                Call printmat("S22","(e15.5e2,2x)","('         0       ')"&
                     &,p_S22,nc1,nc1,tol)
                Write(outfile,*)
             Endif



             Do ir=1,bmesh% nr
                Do icomp=1,ncomp
                   Do jcr=nchi+1,nc
                      basis% p_fc(ir,jcr,icomp,issa)=&
                           & basis% p_fc(ir,jcr,icomp,issa)&
                           &-Dot_product(basis% p_fc(ir,1:nchi,icomp,issa)&
                           &                  ,p_S12(1:nchi,jcr-nchi))
                      basis% p_fkc(ir,jcr,icomp,issa)=&
                           & basis% p_fkc(ir,jcr,icomp,issa)&
                           &-Dot_product(basis% p_fkc(ir,1:nchi,icomp,issa)&
                           &                  ,p_S12(1:nchi,jcr-nchi))
                      If(calc_forces) Then
                         
                         basis% p_fdpc(ir,jcr,icomp,issa)=&
                              & basis% p_fdpc(ir,jcr,icomp,issa)&
                              &-Dot_product(basis% p_fdpc(ir,1:nchi,icomp,issa)&
                              &                  ,p_S12(1:nchi,jcr-nchi))
                         basis% p_fdmc(ir,jcr,icomp,issa)=&
                              & basis% p_fdmc(ir,jcr,icomp,issa)&
                              &-Dot_product(basis% p_fdmc(ir,1:nchi,icomp,issa)&
                              &                  ,p_S12(1:nchi,jcr-nchi))
                      End If
                   End Do
                End Do
             End Do

             Call create_overlap(bmesh,p_texp&
                  &,nchi+1,nc,nchi+1,nc,basis% p_kapRc(:,issa)&
                  &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
                  &,basis% p_fc(:,:,:,issa),p_tr,p_S22)

             If(prot) Then
                Call printmat("S22","(e15.5e2,2x)","('         0       ')"&
                     &,p_S22,nc1,nc1,tol)
                Write(outfile,*)
             Endif

             Call rbchd (p_S22,nc1,ierr)

             If(ierr /= 0_t_int)Then
                Write(errbuf,"('''RBCHD: overlap matrix indefinite'',/&
                     &,'' possible reasons:  '',/&
                     &,'' (1) bad basis definitions''')")
                Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
             End If

             If(prot) Then
                Call printmat("S22R","(e15.5e2,2x)","('         0       ')"&
                     &,p_S22,nc1,nc1,tol)
                Write(outfile,*)
             Endif

             Do ir=1,bmesh% nr
                Do icomp=1,ncomp
                   Do jcr=nchi+1,nc
                      j=jcr-nchi

                      basis% p_fc(ir,jcr,icomp,issa)=&
                           &(basis% p_fc(ir,jcr,icomp,issa)&
                           &-Sum(basis% p_fc(ir,nchi+1:jcr-1,icomp,issa)&
                           &*p_S22(1:j-1,j)))/p_S22(j,j)

                      basis% p_fkc(ir,jcr,icomp,issa)=&
                           &(basis% p_fkc(ir,jcr,icomp,issa)&
                           &-Sum(basis% p_fkc(ir,nchi+1:jcr-1,icomp,issa)&
                           &*p_S22(1:j-1,j)))/p_S22(j,j)
                      If(calc_forces) Then
                         
                         basis% p_fdpc(ir,jcr,icomp,issa)=&
                              &(basis% p_fdpc(ir,jcr,icomp,issa)&
                              &-Sum(basis% p_fdpc(ir,nchi+1:jcr-1,icomp,issa)&
                              &*p_S22(1:j-1,j)))/p_S22(j,j)
                         basis% p_fdmc(ir,jcr,icomp,issa)=&
                              &(basis% p_fdmc(ir,jcr,icomp,issa)&
                              &-Sum(basis% p_fdmc(ir,nchi+1:jcr-1,icomp,issa)&
                              &*p_S22(1:j-1,j)))/p_S22(j,j)
                      End If
                   Enddo
                Enddo
             Enddo


          End Do ortholoopc1
       End If
       !!
       Write(outfile,15) 
       !!
       !!
       !! now the valence part
       !! here we do it a bit step by step, since we apply a 
       !! srel approx and want to do this a bit more controlled.
       !! Up to here the basis is the complete transformed FREL
       !! basis, which means the scalar products are all exact!
       !! Meaning, we can make a orthogonalization.
       !! First we make semi ortho to core, then real valence ortho
       !! to semi and core and then orthogonalize the rest.
       If(prot) Then
          Call create_overlap(bmesh,p_texp,1_t_int,nc,1_t_int,nc&
               &,basis% p_kapRc(:,issa)&
               &,basis% p_kapRc(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
               &,basis% p_fc(:,:,:,issa),p_tr,p_S)
          Call printmat("S","(e15.5e2,2x)","('         0       ')"&
               &,p_S,nc,nc,tol)
          Write(outfile,*)
       Endif
       !!
       !!
       nv=basis% p_nRv(issa)
       !!
       If(prot) Then
          Write(outfile,13) issa
          Write(outfile,114) nv
       Endif
       !!
       !!
       !!
       !!
       !!
       Call palloc('p_Sc1v',p_Sc1v,1_t_int,nchi,1_t_int,nv,nmodule,nsubfunc)
       Call palloc('p_Svv',p_Svv,1_t_int,nv,1_t_int,nv,nmodule,nsubfunc)

       !!
       ortholoopcv: Do i=1,2        !! repeat ortho two times .... improves errors
          !!
          If(prot) Then
             Write(outfile,15) 
             Write(outfile,16) i
          Endif
          !!
          Call create_overlap(bmesh,p_texp,1_t_int,nchi,1_t_int,nv&
               &,basis% p_kapRc(:,issa)&
               &,basis% p_kapRv(:,issa),ncomp,basis% p_fc(:,:,:,issa)&
               &,basis% p_fv(:,:,:,issa),p_tr,p_Sc1v)
          !!
          !!
          !!
          If(prot) Then
             Call printmat("Sc1v","(e15.5e2,2x)","('         0       ')"&
                  &,p_Sc1v,nchi,nv,tol)
             Write(outfile,*)
          Endif


          Do ir=1,bmesh% nr
             Do icomp=1,ncomp
                Do jvr=1,nv
                   basis% p_fv(ir,jvr,icomp,issa)=&
                        & basis% p_fv(ir,jvr,icomp,issa)&
                        &-Dot_product(basis% p_fc(ir,1:nchi,icomp,issa)&
                        &                  ,p_Sc1v(1:nchi,jvr))
                   basis% p_fkv(ir,jvr,icomp,issa)=&
                        & basis% p_fkv(ir,jvr,icomp,issa)&
                        &-Dot_product(basis% p_fkc(ir,1:nchi,icomp,issa)&
                        &                  ,p_Sc1v(1:nchi,jvr))
                   If(calc_forces) Then
                      
                      basis% p_fdpv(ir,jvr,icomp,issa)=&
                           & basis% p_fdpv(ir,jvr,icomp,issa)&
                           &-Dot_product(basis% p_fdpc(ir,1:nchi,icomp,issa)&
                           &                  ,p_Sc1v(1:nchi,jvr))
                      basis% p_fdmv(ir,jvr,icomp,issa)=&
                           & basis% p_fdmv(ir,jvr,icomp,issa)&
                           &-Dot_product(basis% p_fdmc(ir,1:nchi,icomp,issa)&
                           &                  ,p_Sc1v(1:nchi,jvr))
                   End If
                End Do
             End Do
          End Do
          !!
          !!
          If(prot) Then
             Call create_overlap(bmesh,p_texp,1_t_int,nv,1_t_int,nv&
                  &,basis% p_kapRv(:,issa)&
                  &,basis% p_kapRv(:,issa),ncomp,basis% p_fv(:,:,:,issa)&
                  &,basis% p_fv(:,:,:,issa),p_tr,p_Svv)
             Call printmat("Svv","(e15.8e2,2x)","('         0       ')"&
                  &,p_Svv,nv,nv,tol)
             Write(outfile,*)
          Endif
          !!
       Enddo ortholoopcv
       
       !!
       !! the whole valence sector is ortho to core
       !! Lets orthonormalize the semi-core sector

       ortholoopsemi: Do i=1,2
          Do ivr=1,nv
             kapi=basis% p_kapRv(ivr,issa)
             lv=(Abs(2*kapi+1)-1)/2
             If(basis% p_tRv(ivr,issa) == ORB_TYPE_SEMICORE )Then
                Do jvr=1,ivr-1
                   kapj=basis% p_kapRv(jvr,issa)
                   If(basis% p_tRv(jvr,issa) == ORB_TYPE_SEMICORE )Then
                      !!
                      If(kapi == kapj)Then
                         Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis&
                              &,ivr,jvr,issa,calc_forces)
                      Endif
                      !!
                   End If
                End Do
                !!
                Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis,ivr,ivr&
                     &,issa,calc_forces)
                !!
             End If
          End Do
       Enddo ortholoopsemi
       !!
       If(prot) Then
          Call create_overlap(bmesh,p_texp,1_t_int,nv,1_t_int,nv&
               &,basis% p_kapRv(:,issa)&
               &,basis% p_kapRv(:,issa),ncomp,basis% p_fv(:,:,:,issa)&
               &,basis% p_fv(:,:,:,issa),p_tr,p_Svv)
          Call printmat("Svv (semi OK)","(e15.8e2,2x)","('         0       ')"&
               &,p_Svv,nv,nv,tol)
          Write(outfile,*)
       Endif
       !!
       !! Now, we orthogonalize the valence to the semi core, 
       !!

       ortholoopval1: Do i=1,2
          
          kp1=-1
          If(rel) kp1=basis% lvmx
          kploop: Do kp=-basis% lvmx-1,kp1
             lv=(Abs(2*kp+1)-1)/2
             ivrloop: Do ivr=1,nv
                kapi=basis% p_kapRv(ivr,issa)
                !!
                If(basis% p_tRv(ivr,issa) /= ORB_TYPE_SEMICORE .And. kapi == kp )Then
                   !Print "(i1,a1,1x,i1,'/2')",basis% p_nnRv(ivr,issa)&
                   !     &,lqnchar((Abs(2*kapi+1)-1)/2),2*Abs(kapi)-1
                   
                   Do jvr=1,nv
                      kapj=basis% p_kapRv(jvr,issa)
                      If(basis% p_tRv(jvr,issa) == ORB_TYPE_SEMICORE .And. &
                           &kapj == kp )Then
                         !Print "('  -> ',i1,a1,1x,i1,'/2')",basis% p_nnRv(jvr,issa)&
                         !     &,lqnchar((Abs(2*kapj+1)-1)/2),2*Abs(kapj)-1
                         
                         !!
                         Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis&
                              &,ivr,jvr,issa,calc_forces)
                         !!
                         !!
                      Endif
                   Enddo
                   
                   !!
                   Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis,ivr,ivr&
                        &,issa,calc_forces)
                   !!
                   !!Exit ivrloop
                Endif
             Enddo ivrloop
          Enddo kploop
       End Do ortholoopval1
       !!
       !!
       If(prot) Then
          Call create_overlap(bmesh,p_texp,1_t_int,nv,1_t_int,nv&
               &,basis% p_kapRv(:,issa)&
               &,basis% p_kapRv(:,issa),ncomp,basis% p_fv(:,:,:,issa)&
               &,basis% p_fv(:,:,:,issa),p_tr,p_Svv)
          Call printmat("Svv (val ortho to semi)"&
               &,"(e15.8e2,2x)","('         0       ')",p_Svv,nv,nv,tol)
          Write(outfile,*)
       Endif
       !!
       !! Finally, the rest valence gets orthoed too
       !!
       ortholoopval2: Do i=1,2
          kp1=-1
          If(rel) kp1=basis% lvmx
          kploop2: Do kp=-basis% lvmx-1,kp1
             lv=(Abs(2*kp+1)-1)/2
             second=.False.
             ivrloop2: Do ivr=1,nv
                kapi=basis% p_kapRv(ivr,issa)
                If(basis% p_tRv(ivr,issa) /= ORB_TYPE_SEMICORE .And. kapi == kp)Then
                   
                   If(.Not. second)Then
                      second=.True.
                      Cycle ivrloop2
                   Else
                      !Print "(i1,a1,1x,i1,'/2')",basis% p_nnRv(ivr,issa)&
                      !     &,lqnchar((Abs(2*kapi+1)-1)/2),2*Abs(kapi)-1
                      Do jvr=1,ivr-1
                         kapj=basis% p_kapRv(jvr,issa)
                         If(basis% p_tRv(jvr,issa) /= ORB_TYPE_SEMICORE .And.&
                              & kapj == kp)Then
                            !!
                            !Print "(' -> ',i1,a1,1x,i1,'/2')",basis% p_nnRv(jvr,issa)&
                            !     &,lqnchar((Abs(2*kapj+1)-1)/2),2*Abs(kapj)-1
                            !!                            
                            Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis&
                                 &,ivr,jvr,issa,calc_forces)
                            !!
                         Endif
                      Enddo
                      !!
                      Call orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis,ivr,ivr&
                           &,issa,calc_forces)
                      !!
                   Endif
                Endif
             End Do ivrloop2
          End Do kploop2
       End Do ortholoopval2
       !!
       If(prot) Then
          Call create_overlap(bmesh,p_texp,1_t_int,nv,1_t_int,nv&
               &,basis% p_kapRv(:,issa)&
               &,basis% p_kapRv(:,issa),ncomp,basis% p_fv(:,:,:,issa)&
               &,basis% p_fv(:,:,:,issa),p_tr,p_Svv)
          Call printmat("Svv (all ortho)"&
               &,"(e15.8e2,2x)","('         0       ')",p_Svv,nv,nv,tol)
          Write(outfile,*)
       Endif
       !!
       !!
       Call pfree(p_texp)
       Call pfree(p_tr)
       !!
    Enddo issaloop
    !!
    !!
    !!
    !!
    Call pfree(p_Sc1v)
    Call pfree(p_Svv)
    Call pfree(p_S22)
    Call pfree(p_S12)
    Call pfree(p_S11)
    Call pfree(p_S)
    !!
    If(prot) Write(outfile,1)'END:',Trim(nsubfunc)
    !!
    !Stop
    !!
    Return
    !!
1   Format(72('='),/,a,a,/,72('='))
13  Format(72('-'),/,'sort=',i5)
14  Format('nchi=',i5,' nc=',i5,' nc1=',i5)
114 Format('nv=',i5)
15  Format(50('-'))
16  Format('ortho step: ',i1)
    !!
  End Subroutine ortho_bas
  !!***********************************************************************
  Subroutine create_overlap(bmesh,p_texp&
       &,ni1,ni2,nj1,nj2,kapi,kapj,ncomp,fi,fj,tr,S)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: ni1,ni2,nj1,nj2,kapi(:),kapj(:),ncomp
    Real(t_real),Intent(in)      :: p_texp(:)
    Real(t_real),Intent(in)      :: fi(:,:,:)
    Real(t_real),Intent(in)      :: fj(:,:,:)
    Real(t_real),Intent(inout)   :: tr(:)
    Real(t_real),Intent(out)     :: S(:,:)
    !!LOCAL
    Integer(t_int) :: i,j,icomp,l
    Real(t_real)   :: dg
    !!
    !!
    Do i=ni1,ni2
       Do j=nj1,nj2
          
          dg=0.0_t_real
          
          If( kapi(i) == kapj(j) )Then 
             l=(Abs(2*kapi(i)+1)-1)/2

             tr(1:bmesh% nr)=0.0_t_real
             Do icomp=1,ncomp
                tr(1:bmesh% nr)=tr(1:bmesh% nr)&
                     &+fi(1:bmesh% nr,i,icomp)*&
                     & fj(1:bmesh% nr,j,icomp)*p_texp(1:bmesh% nr)**2
             Enddo
             Call tjlag4int(bmesh% nr,tr,bmesh% p_jmesh,bmesh% hx,dg)
          Endif
          S(i-ni1+1,j-nj1+1)=dg
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine create_overlap
  !!***********************************************************************
  Subroutine scal_prod_f(bmesh,p_texp,ncomp,fi,fj,tr,S)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Intent(in)      :: bmesh
    Integer(t_int),Intent(in)    :: ncomp
    Real(t_real),Intent(in)      :: p_texp(:)
    Real(t_real),Intent(in)      :: fi(:,:)
    Real(t_real),Intent(in)      :: fj(:,:)
    Real(t_real),Intent(inout)   :: tr(:)
    Real(t_real),Intent(out)     :: S
    !!LOCAL
    Integer(t_int) :: icomp
    !!
    !!
    !!
    S=0.0_t_real
    !!
    tr(1:bmesh% nr)=0.0_t_real
    Do icomp=1,ncomp
       tr(1:bmesh% nr)=tr(1:bmesh% nr)&
            &+fi(1:bmesh% nr,icomp)*&
            & fj(1:bmesh% nr,icomp)*p_texp(1:bmesh% nr)**2
    Enddo
    !!
    Call tjlag4int(bmesh% nr,tr,bmesh% p_jmesh,bmesh% hx,S)
    !!
    Return
    !!
  End Subroutine scal_prod_f
  !!***********************************************************************
  Subroutine orthogonalize_two_fvs(bmesh,p_texp,p_tr,basis,ivr,jvr,issa&
       &,calc_forces)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: bmesh
    Type(T_Basis),Intent(inout)  :: basis
    Logical(t_log),Intent(in)    :: calc_forces
    Integer(t_int),Intent(in)    :: ivr,jvr,issa
    Real(t_real),Intent(in)      :: p_texp(:)
    Real(t_real),Intent(inout)   :: p_tr(:)
    !!LOCAL
!    Character(*),Parameter::nsubfunc="orthogonalize_two_fvs"
    Real(t_real)   :: dg,norm
    !!
    !!
    Call scal_prod_f(bmesh,p_texp,basis% ncomp,basis% p_fv(:,ivr,:,issa)&
         &,basis% p_fv(:,jvr,:,issa),p_tr,dg)
    !!
    If(ivr==jvr) Then
       norm=1.0_t_real/Sqrt(dg)
       dg=0.0_t_real
    Else
       norm=1.0_t_real
    Endif
    !!
    basis% p_fv  (:,ivr,:,issa)=basis% p_fv  (:,ivr,:,issa)*norm&
         &-basis% p_fv  (:,jvr,:,issa)*dg
    basis% p_fkv (:,ivr,:,issa)=basis% p_fkv (:,ivr,:,issa)*norm&
         &-basis% p_fkv (:,jvr,:,issa)*dg
    If(calc_forces) Then
       basis% p_fdpv(:,ivr,:,issa)=basis% p_fdpv(:,ivr,:,issa)*norm&
            &-basis% p_fdpv(:,jvr,:,issa)*dg
       basis% p_fdmv(:,ivr,:,issa)=basis% p_fdmv(:,ivr,:,issa)*norm&
            &-basis% p_fdmv(:,jvr,:,issa)*dg
    End If
    !!
    Return
    !!
  End Subroutine orthogonalize_two_fvs
  !!***********************************************************************
  !!
  !! This subroutine checks the normalization of the
  !! local basis states.
  !!
  Subroutine bascheck(nsort,elements,basis,relativistic)
    !!
    Implicit None
    !!
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: relativistic
    !! number of different atomic sorts
    Integer(t_int),Intent(in)    :: nsort
    Character(2),Intent(in)      :: elements(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="bascheck"
    Logical(t_log) :: rel
    Integer(t_int) :: icr,issa,kap,j,l,nmain,icomp,etype,ivr,multo,nr
    Real(t_real)   :: norm,val
    Real(t_real),Pointer :: p_tr(:),p_texp(:)
    Type(T_Mesh),Pointer :: bmesh
    !!PARAMETERS
    Real(t_real),Parameter   :: CTINY=1.0e-8_t_real
    Real(t_real),Parameter   :: CTOO_LARGE=1.0e-5_t_real
    Real(t_real),Parameter   :: VTINY=1.0e-6_t_real
    Real(t_real),Parameter   :: VTOO_LARGE=1.0e-3_t_real
    !!
    Nullify(p_tr)
    Nullify(p_texp)
    !!
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    !!
    !!     ----------------------------------------------------
    !!     -------------- check core states -------------------
    !!     ----------------------------------------------------
    Do issa=1_t_int,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       nr=bmesh% nr
       Call palloc('p_tr'  ,p_tr  ,1_t_int,nr,nmodule,nsubfunc)
       Call palloc('p_texp',p_texp,1_t_int,nr,nmodule,nsubfunc)
       !!
       p_texp(1:nr)=bmesh% p_rmesh(1:nr)**(basis% p_fgam(issa))
       Do icr=1,basis% p_nRc(issa)
          nmain=basis% p_nnRc(icr,issa)
          kap=basis% p_kapRc(icr,issa)
          l=(Abs(2*kap+1)-1)/2
          j=2*Abs(kap)-1
          !!
          multo=coreMultIndex(icr,issa,basis)
          !!
          !!Write(outfile,"(i1,a1,i1,'/2',1x,i1)")&
          !!     & nmain,lqnchar(l),j,basis% ncomp
          !!
          
          !! check normalization
          !!
          norm=0.0_t_real
          Do icomp=1,basis% ncomp
             p_tr(1:nr)=(basis% p_fc(1:nr,icr,icomp,issa)*p_texp(1:nr))**2
             Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
             norm=norm+val
             !!Print *,'val=',val
          Enddo !! icomp
          norm=Sqrt(norm)
          !Print *,'norm=',norm
          !!
          If (Abs(norm-1.0_t_real) >CTINY)Then
             If(.Not.rel)Then
                Write(errbuf,121) icr,nmain,lqnchar(l),multo&
                     &,elements(issa),issa,norm-1.0_t_real
             Else
                Write(errbuf,1121) icr,nmain,lqnchar(l),multo,j&
                     &,elements(issa),issa,norm-1.0_t_real
             Endif
             etype=E_warning
             If (Abs(norm-1.0_t_real) > CTOO_LARGE) Then
                etype=E_WARNING
             Endif
             Call errmsg(nmodule,nsubfunc,etype,msg=errbuf)
          Endif
          !!
          !!
       Enddo !! ic
    Enddo !! issa
    Write(outfile,110)nsubfunc
    !!     ----------------------------------------------------
    !!     -------------- check valence states ----------------
    !!     ----------------------------------------------------
    Do issa=1_t_int,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       nr=bmesh% nr
       Call palloc('p_tr'  ,p_tr  ,1_t_int,nr,nmodule,nsubfunc)
       Call palloc('p_texp',p_texp,1_t_int,nr,nmodule,nsubfunc)
       !!
       p_texp(1:nr)=bmesh% p_rmesh(1:nr)**(basis% p_fgam(issa))
       Do ivr=1,basis% p_nRv(issa)
          nmain=basis% p_nnRv(ivr,issa)
          kap=basis% p_kapRv(ivr,issa)
          If(relativistic == SCALAR_RELATIVISTIC .And. .Not. newsrel .And. &
               &basis% p_tRv(ivr,issa) /= ORB_TYPE_SEMICORE) Cycle
          l=(Abs(2*kap+1)-1)/2
          j=2*Abs(kap)-1
          !!
          !!
          !Write(outfile,"(i1,a1,i1,'/2',1x,1x,i1)")&
          !     &nmain,lqnchar(l),j,basis% ncomp
          !!
          !! check normalization
          !Print *,'TODO: bascheck, including semi-core check'
          !!
          norm=0.0_t_real
          Do icomp=1,basis% ncomp
             p_tr(1:nr)=(basis% p_fv(1:nr,ivr,icomp,issa)*p_texp(1:nr))**2
             Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
             norm=norm+val
             !!Print *,'val=',val
          Enddo !! icomp
          norm=Sqrt(norm)
          !!Print *,'norm=',norm
          !!
          If (Abs(norm-1.0_t_real) >VTINY)Then
             If(.Not.rel)Then
                Write(errbuf,120) ivr,nmain,lqnchar(l)&
                     &,elements(issa),issa,norm-1.0_t_real
             Else
                Write(errbuf,1120) ivr,nmain,lqnchar(l),j&
                     &,elements(issa),issa,norm-1.0_t_real
             Endif
             etype=E_warning
             If (Abs(norm-1.0_t_real) > VTOO_LARGE) Then
                etype=E_warning
             Endif
             Call errmsg(nmodule,nsubfunc,etype,msg=errbuf)
          Endif
          !!
          !!
          !!
          !!
       Enddo ! ivr
    Enddo !! issa
    Write(outfile,150)nsubfunc
    !!
    Call pfree(p_texp)
    Call pfree(p_tr)
    !!
    !!
    Return
    !!
110 Format(a,': core    states are o.k.')
150 Format(a,': valence states are o.k.')
120 Format('''Norm of state ',i3,' (',i2,a1,') of atom ',a2,' sort ',i4&
         &,' is not unity : err=',e20.8e3,'''')
1120 Format('''Norm of state ',i3,' (',i2,a1,' ',i1,'/2) of atom ',a2&
          &,' sort ',i4,' is not unity : err=',e20.8e3,'''')
121 Format('''Norm of state ',i3,' (',i2,a1,i1,') of atom ',a2,' sort ',i4&
         &,' is not unity : err=',e20.8e3,'''')
1121 Format('''Norm of state ',i3,' (',i2,a1,i1,' ',i1,'/2) of atom ',a2&
          &,' sort ',i4,' is not unity : err=',e20.8e3,'''')
    !!
  End Subroutine bascheck
  !!
  !!***********************************************************************
  !!
  !!
  !!
  Subroutine printbasis(nsort,elements,basis,relativistic,calc_forces,mol)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Mol),Intent(in)       :: mol
    Character(2),Intent(in)      :: elements(:)
    Logical(t_log),Intent(in)    :: calc_forces
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: nsort
    !!LOCAL
!    Character(*),Parameter::nsubfunc="printbasis"
    Integer(t_int) :: issa,icomp
    Type(T_Mesh),Pointer         :: bmesh
    Type(T_Mesh),Pointer         :: atommesh
    Character(100)   :: str
    !!
    !!print wave-functions
    If(.Not. isoption('PLOT_BASIS')) Return
    !!core
    Do issa=1_t_int,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       atommesh => mol% p_atommeshs(issa)
       !!
       !!
       Write(str,"('+vatom.',i3.3)") issa
       Open(12,file=str,status='unknown')
       Write(12,100) Trim(FPLOMAINVERSION)
100    Format('# Version: ',a)
       Write(12,101) elements(issa)
101    Format('# element: ',a2)
       Call write_mesh_info(12_t_int,atommesh)
       Call write_mesh_function(12_t_int,atommesh,mol% p_vatom(:,0,issa)&
            &,"vatom1*r*sqrt(4*Pi)")
       !!
       !!
       Do icomp=1_t_int,basis% ncomp
          !!
          Call write_bas_comp_to_file('+fcor',elements(issa),issa,icomp&
               &,basis% p_nRc,basis% p_nnRc,basis% p_kapRc&
               &,basis% p_fgam(issa)&
               &,basis% p_fc(:,:,icomp,issa),bmesh,relativistic)
          Call write_bas_comp_to_file('+fkcor',elements(issa),issa,icomp&
               &,basis% p_nRc,basis% p_nnRc,basis% p_kapRc&
               &,basis% p_fgam(issa)&
               &,basis% p_fkc(:,:,icomp,issa),bmesh,relativistic)
          If(calc_forces) Then
             
             Call write_bas_comp_to_file('+fdpcor',elements(issa),issa,icomp&
                  &,basis% p_nRc,basis% p_nnRc,basis% p_kapRc&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fdpc(:,:,icomp,issa),bmesh,relativistic)
             
             Call write_bas_comp_to_file('+fdmcor',elements(issa),issa,icomp&
                  &,basis% p_nRc,basis% p_nnRc,basis% p_kapRc&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fdmc(:,:,icomp,issa),bmesh,relativistic)
          End If
          
       Enddo ! icomp
       !!valence
       Do icomp=1_t_int,basis% ncomp
          Call write_bas_comp_to_file('+fval',elements(issa),issa,icomp&
               &,basis% p_nRv,basis% p_nnRv,basis% p_kapRv&
               &,basis% p_fgam(issa)&
               &,basis% p_fv(:,:,icomp,issa),bmesh,relativistic)
          Call write_bas_comp_to_file('+fkval',elements(issa),issa,icomp&
               &,basis% p_nRv,basis% p_nnRv,basis% p_kapRv&
               &,basis% p_fgam(issa)&
               &,basis% p_fkv(:,:,icomp,issa),bmesh,relativistic)

          If(calc_forces) Then
             Call write_bas_comp_to_file('+fdpval',elements(issa),issa,icomp&
                  &,basis% p_nRv,basis% p_nnRv,basis% p_kapRv&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fdpv(:,:,icomp,issa),bmesh,relativistic)
             
             Call write_bas_comp_to_file('+fdmval',elements(issa),issa,icomp&
                  &,basis% p_nRv,basis% p_nnRv,basis% p_kapRv&
                  &,basis% p_fgam(issa)&
                  &,basis% p_fdmv(:,:,icomp,issa),bmesh,relativistic)
             
          End If
          
       Enddo
    Enddo
    !!     
    !!
  End Subroutine printbasis
  !!***********************************************************************
  !!***********************************************************************
  Subroutine dump_basis(basis,nsort,calc_forces,mode)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(inout)  :: basis
    Logical(t_log),Intent(in)    :: calc_forces
    Integer(t_int),Intent(in)    :: nsort
    Character(*),Intent(in)      :: mode
    !!LOCAL
!    Character(*),Parameter::nsubfunc="dump_basis"
    Integer(t_int) :: issa,icomp,ir,icr,ivr
    Type(T_Mesh),Pointer         :: bmesh
    !!
    !!
    If(Trim(mode) == 'write')Then
       Open(12,file=f_basis_dump,form='UNFORMATTED')
       Do issa=1,nsort
          bmesh => basis% p_basmeshs(issa)
          !! fgam dump
          Write(12) basis% p_fgam(issa)
          !! core dump
          Do icr=1,basis% p_nRc(issa)
             Do icomp=1,basis% ncomp
                Do ir=1,bmesh% nr
                   Write(12) basis% p_fc(ir,icr,icomp,issa)
                Enddo
                Do ir=1,bmesh% nr
                   Write(12) basis% p_fkc(ir,icr,icomp,issa)
                Enddo
                If(calc_forces) Then
                   Do ir=1,bmesh% nr
                      Write(12) basis% p_fdpc(ir,icr,icomp,issa)
                   Enddo
                   Do ir=1,bmesh% nr
                      Write(12) basis% p_fdmc(ir,icr,icomp,issa)
                   Enddo
                Endif
                
             Enddo
          Enddo
          !! valence dump
          Do ivr=1,basis% p_nRv(issa)
             Do icomp=1,basis% ncomp
                Do ir=1,bmesh% nr
                   Write(12) basis% p_fv(ir,ivr,icomp,issa)
                Enddo
                Do ir=1,bmesh% nr
                   Write(12) basis% p_fkv(ir,ivr,icomp,issa)
                Enddo
                If(calc_forces) Then
                   Do ir=1,bmesh% nr
                      Write(12) basis% p_fdpv(ir,ivr,icomp,issa)
                   Enddo
                   Do ir=1,bmesh% nr
                      Write(12) basis% p_fdmv(ir,ivr,icomp,issa)
                   Enddo
                Endif
                
             Enddo
          Enddo
          !!
       Enddo
    Else
       Open(12,file=f_basis_dump,status='old',form='UNFORMATTED')
       Do issa=1,nsort
          bmesh => basis% p_basmeshs(issa)
          !! fgam dump
          Read(12) basis% p_fgam(issa)
          !! core dump
          Do icr=1,basis% p_nRc(issa)
             Do icomp=1,basis% ncomp
                Do ir=1,bmesh% nr
                   Read(12) basis% p_fc(ir,icr,icomp,issa)
                Enddo
                Do ir=1,bmesh% nr
                   Read(12) basis% p_fkc(ir,icr,icomp,issa)
                Enddo
                If(calc_forces) Then
                   Do ir=1,bmesh% nr
                      Read(12) basis% p_fdpc(ir,icr,icomp,issa)
                   Enddo
                   Do ir=1,bmesh% nr
                      Read(12) basis% p_fdmc(ir,icr,icomp,issa)
                   Enddo
                Endif
                
             Enddo
          Enddo
          !! valence dump
          Do ivr=1,basis% p_nRv(issa)
             Do icomp=1,basis% ncomp
                Do ir=1,bmesh% nr
                   Read(12) basis% p_fv(ir,ivr,icomp,issa)
                Enddo
                Do ir=1,bmesh% nr
                   Read(12) basis% p_fkv(ir,ivr,icomp,issa)
                Enddo
                If(calc_forces) Then
                   Do ir=1,bmesh% nr
                      Read(12) basis% p_fdpv(ir,ivr,icomp,issa)
                   Enddo
                   Do ir=1,bmesh% nr
                      Read(12) basis% p_fdmv(ir,ivr,icomp,issa)
                   Enddo
                Endif
                
             Enddo
          Enddo
       Enddo
       Close(12)
    Endif
    !!
    Return
    !!
  End Subroutine dump_basis
  !!***********************************************************************
  Subroutine  write_bas_comp_to_file(fprefix,element,issa,icomp,nR,nnR,kapR&
       &,fgam,f,mesh,relativistic)
    !!
    Implicit None  
    !!IO
    Type(T_Mesh),Intent(in)      :: mesh
    Integer(t_int),Intent(in)    :: relativistic
    Character(*),Intent(in)      :: fprefix,element
    Integer(t_int),Intent(in)    :: issa,icomp,nR(:),nnR(:,:),kapR(:,:)
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: f(:,:)
    !!LOCAL
    Character(100) :: str
    Integer(t_int) :: ivr,kap,j,l
    !!
    !!
    Write(str,1)Trim(fprefix),issa,icomp
    !!
    Open(12,file=str,status='unknown')
    Write(12,100) Trim(FPLOMAINVERSION)
100 Format('# Version: ',a)
    Write(12,101) element
101 Format('# element: ',a2)
    Write(12,102) fgam
102 Format('# fgam: ',e30.15e3)
    Call write_mesh_info(12_t_int,mesh)
    !!
    If(nR(issa) == 0) Write(12,5)
    !!
    Do ivr=1,nR(issa)
       kap=kapR(ivr,issa)
       j=2*Abs(kap)-1
       l=(Abs(2*kap+1)-1)/2
       If(relativistic==FULL_RELATIVISTIC)Then
          Write(str,22)nnR(ivr,issa),lqnchar(l),j
       Else
          Write(str,2)nnR(ivr,issa),lqnchar(l)
       Endif

       Call write_mesh_function(12_t_int,mesh,f(:,ivr)*mesh% p_rmesh(:)**fgam&
            &,Trim(str))

    Enddo               ! i
    Close(12)
    !!
    Return
    !!
5   Format(' ')
1   Format(a,'.',i3.3,'.',i1.1)
2   Format('# state : ',i4,a1)
22  Format('# state : ',i4,a1,1x,i1,'/2')
!4   Format(2(e30.15e3,1x))
    !!
  End Subroutine write_bas_comp_to_file
  !!***********************************************************************
  !!
  !! find the minimun of core and valence functions
  !!
  Subroutine funcmin(structure,numerics,basis,relativistic)
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: relativistic
    Type(T_Structure), Intent(in):: structure
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Basis),Intent(inout)  :: basis
    !!LOCAL
!    Character(*),Parameter::nsubfunc="funcmin"
    Logical(t_log) :: rel,srel,khsrel
    Integer(t_int) :: icr,issa,ivr,nmain,kap,lc,lv,icomp,j,multo
    Integer(t_int) :: ncomp,nsort
    Real(t_real)   :: eps,rffcmax,rffvmax,fr,rffmax
    Type(T_Mesh),Pointer :: bmesh
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    srel=(relativistic == SCALAR_RELATIVISTIC)
    khsrel=(relativistic == SCALAR_RELATIVISTIC_KH)
    !!
    
    !!
    !!
    eps=numerics% cuttol_orbital
    !!
    nsort=structure% nsort
    
    ncomp=basis% ncomp
    !!
    issaloop: Do issa=1,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       basis% p_rfcmax(:,:,issa)=bmesh% rmax
       basis% p_rfvmax(:,:,issa)=bmesh% rmax
       !!
       icompc: Do icomp=1,ncomp
          Do icr=1,basis% p_nRc(issa)
             
             Call find_fcutoff(bmesh% nr,bmesh% p_rmesh,basis% p_fgam(issa)&
                  &,basis% p_fc(:,icr,icomp,issa),eps&
                  &,basis% p_rfcmax(icr,icomp,issa))
             
             If(basis% p_lRc(icr,issa) == 0 ) Then
                If(icomp>2)Then
                   basis% p_rfcmax(icr,icomp,issa)=0.0_t_real
                Endif
             Endif
          Enddo
       Enddo icompc
       icompv: Do icomp=1,ncomp
          Do ivr=1,basis% p_nRv(issa)

             Call find_fcutoff(bmesh% nr,bmesh% p_rmesh,basis% p_fgam(issa)&
                  &,basis% p_fv(:,ivr,icomp,issa),eps&
                  &,basis% p_rfvmax(ivr,icomp,issa))

             If(basis% p_lRv(ivr,issa) == 0 ) Then
                If(icomp>2)Then
                   basis% p_rfvmax(ivr,icomp,issa)=0.0_t_real
                Endif
             Endif
             If(icomp>2 .And. &
                  &(srel.And..Not.newsrel .And. &
                  &basis% p_tRv(ivr,issa) /= ORB_TYPE_SEMICORE) )Then
                basis% p_rfvmax(ivr,icomp,issa)=0.0_t_real
             Endif

          Enddo
       Enddo icompv
    Enddo issaloop
1   Format()
    Write(outfile,1)
    !!
    Write(outfile,200)
    If(rel)Then
       Write(outfile,1220)
    Else If(srel)Then
       Write(outfile,2220)
    Else If(khsrel)Then
       Write(outfile,2221)
    Else
       Write(outfile,220)
    Endif
    basis% p_rtfcmax(:,:)=0.0_t_real
    basis% p_rtfvmax(:,:)=0.0_t_real
    !!
    Do issa=1,nsort
       !! core
       Do icomp=1,ncomp
          basis% p_rtfcmax(icomp,issa)=Maxval(&
               &basis% p_rfcmax(1:basis% p_nRc(issa),icomp,issa))
       Enddo
       basis% p_rtfcmax(0,issa)=Maxval(basis% p_rtfcmax(1:ncomp,issa))
       !!
       If(basis% p_nRc(issa) == 0 )Then
          basis% p_rtfcmax(0:ncomp,issa)=0.0_t_real
       Endif
       !! valence
       Do icomp=1,ncomp
          basis% p_rtfvmax(icomp,issa)=Maxval(&
               &basis% p_rfvmax(1:basis% p_nRv(issa),icomp,issa))
       Enddo
       basis% p_rtfvmax(0,issa)=Maxval(basis% p_rtfvmax(1:ncomp,issa))
       !!
    Enddo
    !!

    rffcmax=Maxval(basis% p_rtfcmax(0,1:nsort))
    rffvmax=Maxval(basis% p_rtfvmax(0,1:nsort))
    rffmax=Max(rffcmax,rffvmax)
    !!
    Do issa=1,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       fr=100.0_t_real/bmesh% rmax
       !!
       Do icr=1,basis% p_nRc(issa)
          nmain=basis% p_nnRc(icr,issa)
          kap=basis% p_kapRc(icr,issa)
          lc=(Abs(2*kap+1)-1)/2
          j=2*Abs(kap)-1
          multo=coreMultIndex(icr,issa,basis)
          If(rel)Then
             Write(outfile,1310)structure% p_elements(issa),nmain&
                  &,lqnchar(lc),multo,j&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)*fr&
                  &,basis% p_rfcmax(icr,SMALL_COMP1,issa)&
                  &,basis% p_rfcmax(icr,SMALL_COMP1,issa)*fr&
                  &,"  Core ",issa
          Elseif(srel)Then
             Write(outfile,11310)structure% p_elements(issa),nmain&
                  &,lqnchar(lc),multo&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)&
                  &,basis% p_rfcmax(icr,SMALL_COMP1,issa)&
                  &,basis% p_rfcmax(icr,LARGE_COMP1+2,issa)&
                  &,basis% p_rfcmax(icr,SMALL_COMP1+2,issa)&
                  &,"  Core ",issa
          Elseif(khsrel)Then
             Write(outfile,11311)structure% p_elements(issa),nmain&
                  &,lqnchar(lc),multo&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)&
                  &,basis% p_rfcmax(icr,SMALL_COMP1,issa)&
                  &,basis% p_rfcmax(icr,LARGE_COMP2,issa)&
                  &,"  Core ",issa
          Else
             Write(outfile,1210)structure% p_elements(issa),nmain&
                  &,lqnchar(lc),multo&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)&
                  &,basis% p_rfcmax(icr,LARGE_COMP1,issa)*fr&
                  &,"  Core ",issa
          Endif
       Enddo ! icr
       !! sort totals
       Write(outfile,230)
       If(rel)Then
          Write(outfile,2310)structure% p_elements(issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)*fr&
               &,basis% p_rtfcmax(SMALL_COMP1,issa)&
               &,basis% p_rtfcmax(SMALL_COMP1,issa)*fr&
               &,"  Core ",issa
       Elseif(srel)Then
          Write(outfile,21310)structure% p_elements(issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)&
               &,basis% p_rtfcmax(SMALL_COMP1,issa)&
               &,basis% p_rtfcmax(LARGE_COMP1+2,issa)&
               &,basis% p_rtfcmax(SMALL_COMP1+2,issa)&
               &,"  Core ",issa
       Elseif(khsrel)Then
          Write(outfile,21311)structure% p_elements(issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)&
               &,basis% p_rtfcmax(SMALL_COMP1,issa)&
               &,basis% p_rtfcmax(LARGE_COMP2,issa)&
               &,"  Core ",issa
       Else
          Write(outfile,21210)structure% p_elements(issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)&
               &,basis% p_rtfcmax(LARGE_COMP1,issa)*fr&
               &,"  Core ",issa
       Endif
       Write(outfile,230)
       !! valence
       Do ivr=1,basis% p_nRv(issa)
          nmain=basis% p_nnRv(ivr,issa)
          kap=basis% p_kapRv(ivr,issa)
          lv=(Abs(2*kap+1)-1)/2
          j=2*Abs(kap)-1
          If(rel) Then
             Write(outfile,310)structure% p_elements(issa),nmain&
                  &,lqnchar(lv),j&
                  &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)&
                  &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)*fr&
                  &,basis% p_rfvmax(ivr,SMALL_COMP1,issa)&
                  &,basis% p_rfvmax(ivr,SMALL_COMP1,issa)*fr&
                  &,"Valence",issa
          Elseif(srel)Then
             If(basis% ncomp>2)Then
                Write(outfile,51310)structure% p_elements(issa)&
                     &,nmain,lqnchar(lv)&
                     &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)&
                     &,basis% p_rfvmax(ivr,SMALL_COMP1,issa)&
                     &,basis% p_rfvmax(ivr,LARGE_COMP1+2,issa)&
                     &,basis% p_rfvmax(ivr,SMALL_COMP1+2,issa)&
                     &,"Valence",issa
             Else
                Write(outfile,31310)structure% p_elements(issa)&
                     &,nmain,lqnchar(lv)&
                     &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)&
                     &,basis% p_rfvmax(ivr,SMALL_COMP1,issa)&
                     &,"Valence",issa
             Endif
          Elseif(khsrel)Then
             Write(outfile,51311)structure% p_elements(issa)&
                  &,nmain,lqnchar(lv)&
                  &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)&
                  &,basis% p_rfvmax(ivr,SMALL_COMP1,issa)&
                  &,basis% p_rfvmax(ivr,LARGE_COMP2,issa)&
                  &,"Valence",issa
          Else
             Write(outfile,210)structure% p_elements(issa),nmain&
                  &,lqnchar(lv)&
                  &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)&
                  &,basis% p_rfvmax(ivr,LARGE_COMP1,issa)*fr&
                  &,"Valence",issa  
          Endif
       Enddo ! ivr
       !! sort totals
       Write(outfile,230)
       If(rel) Then
          Write(outfile,22310)structure% p_elements(issa)&
               &,basis% p_rtfvmax(LARGE_COMP1,issa)&
               &,basis% p_rtfvmax(LARGE_COMP1,issa)*fr&
               &,basis% p_rtfvmax(SMALL_COMP1,issa)&
               &,basis% p_rtfvmax(SMALL_COMP1,issa)*fr&
               &,"Valence",issa
       Elseif(srel)Then
          If(basis% ncomp>2)Then
             Write(outfile,21310)structure% p_elements(issa)&
                  &,basis% p_rtfvmax(LARGE_COMP1,issa)&
                  &,basis% p_rtfvmax(SMALL_COMP1,issa)&
                  &,basis% p_rtfvmax(LARGE_COMP1+2,issa)&
                  &,basis% p_rtfvmax(SMALL_COMP1+2,issa)&
                  &,"Valence",issa
          Else
             Write(outfile,41310)structure% p_elements(issa)&
                  &,basis% p_rtfvmax(LARGE_COMP1,issa)&
                  &,basis% p_rtfvmax(SMALL_COMP1,issa)&
                  &,"Valence",issa
          Endif
       Elseif(khsrel)Then
          Write(outfile,21311)structure% p_elements(issa)&
               &,basis% p_rtfvmax(LARGE_COMP1,issa)&
               &,basis% p_rtfvmax(SMALL_COMP1,issa)&
               &,basis% p_rtfvmax(LARGE_COMP2,issa)&
               &,"Valence",issa
       Else
          Write(outfile,2210)structure% p_elements(issa)&
               &,basis% p_rtfvmax(LARGE_COMP1,issa)&
               &,basis% p_rtfvmax(LARGE_COMP1,issa)*fr&
               &,"Valence",issa  
       Endif
       Write(outfile,230)
    Enddo ! issa
    Write(outfile,230)
    !!
    Write(outfile,10) rffcmax
    Write(outfile,11) rffvmax
    Write(outfile,12) rffmax
    Write(outfile,230)    
    !!
    !!
    Return
    !!
10  Format('| max extension of core    states=',f15.8,t72,'|')
11  Format('| max extension of valence states=',f15.8,t72,'|')
12  Format('| max extension of state         =',f15.8,t72,'|')
200 Format(72("-")/"|",t20, &
         & "MINIMAL RADIUS FOR THE LOCAL BASIS FUNCTIONS",t72,"|")
1210 Format("|",t4,a2,t13,i1,a1,i1,t25,f14.4,t46,f7.2,'%',t56,a,t66,i3&
         &,t72,"|")
21210 Format("|",t4,a2,t25,f14.4,t46,f7.2,'%',t56,a,t66,i3&
         &,t72,"|")
210 Format("|",t4,a2,t13,i1,a1,t25,f14.4,t46,f7.2,'%',t56,a,t66,i3&
         &,t72,"|")
2210 Format("|",t4,a2,t25,f14.4,t46,f7.2,'%',t56,a,t66,i3&
         &,t72,"|")
11310 Format("|",t4,a2,t8,i1,a1,i1,1x,t19,f9.4,t28,f9.4&
         &,t38,f9.4,t47,f9.4,t59,a7,t67,i4,t72,"|")
11311 Format("|",t4,a2,t8,i1,a1,i1,1x,t19,f9.4,t28,f9.4&
         &,t47,f9.4,t59,a7,t67,i4,t72,"|")
51310 Format("|",t4,a2,t8,i1,a1,t19,f9.4,t28,f9.4&
         &,t38,f9.4,t47,f9.4,t59,a7,t67,i4,t72,"|")
51311 Format("|",t4,a2,t8,i1,a1,t19,f9.4,t28,f9.4&
         &,t47,f9.4,t59,a7,t67,i4,t72,"|")
21310 Format("|",t4,a2,t19,f9.4,t28,f9.4&
         &,t38,f9.4,t47,f9.4,t59,a7,t67,i4,t72,"|")
21311 Format("|",t4,a2,t19,f9.4,t28,f9.4&
         &,t47,f9.4,t59,a7,t67,i4,t72,"|")
41310 Format("|",t4,a2,t19,f9.4,t28,f9.4&
         &,t59,a7,t67,i4,t72,"|")
31310 Format("|",t4,a2,t8,i1,a1,t19,f9.4,t28,f9.4&
         &,t59,a7,t67,i4,t72,"|")
1310 Format("|",t4,a2,t8,i1,a1,i1,1x,i1,'/2',t19,f9.4,t28,'=',f7.2,'%'&
         &,t38,f9.4,t47,'=',f7.2,'%',t59,a7,t67,i4,t72,"|")
2310 Format("|",t4,a2,t19,f9.4,t28,'=',f7.2,'%'&
         &,t38,f9.4,t47,'=',f7.2,'%',t59,a7,t67,i4,t72,"|")
310 Format("|",t4,a2,t8,i1,a1,1x,i1,'/2',t19,f9.4,t28,'=',f7.2,'%'&
         &,t38,f9.4,t47,'=',f7.2,'%',t59,a7,t67,i4,t72,"|")
22310 Format("|",t4,a2,t19,f9.4,t28,'=',f7.2,'%'&
         &,t38,f9.4,t47,'=',f7.2,'%',t59,a7,t67,i4,t72,"|")
220 Format(72("-")/"|",t3,"ATOM",t10,"ELECTRON",t32, &
         & "RMIN",t46,"percent",t56,"TYPE",t66,"SORT",t72,"|"/72("-"))
2220 Format(72("-")/"|",t3,"ATOM",t10,"ELECTRON"&
          &,t22,"RMIN_LA",t31,"RMIN_SA",t41,"RMIN_LD",t50,"RMIN_SD"&
          &,t59,"TYPE",t67,"SORT",t72,"|"/72("-"))
2221 Format(72("-")/"|",t3,"ATOM",t10,"ELECTRON"&
          &,t22,"RMIN_LA",t31,"RMIN_SA",t50,"RMIN_SD"&
          &,t59,"TYPE",t67,"SORT",t72,"|"/72("-"))
1220 Format(72("-")/"|",t3,"ATOM",t10,"ELECTRON",t21,"RMIN(LARGE COMP.)"&
          &,t40,"RMIN(SMALL COMP.)"&
          &,t61,"TYPE",t67,"SORT",t72,"|"/72("-"))
230 Format(72("-"))
    !!
  End Subroutine funcmin
  !!***********************************************************************
  Subroutine find_fcutoff(nr,rmesh,fgam,f,eps,rfmax)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nr
    Real(t_real),Intent(in)      :: fgam,eps
    Real(t_real),Intent(in)      :: rmesh(nr),f(:)
    Real(t_real),Intent(out)     :: rfmax
    !!LOCAL
!    Character(*),Parameter::nsubfunc="find_fcutoff"
    Integer(t_int) :: ir
    Real(t_real)   :: r0,r1,ff0,ff1,texp,r
    !!
    !!
    ir=nr
    r1=rmesh(ir)
    texp=r1**fgam
    ff1=Abs(f(ir)*texp/r1)
    !!
    rfmax=r1
    Do ir=nr-1,2,-1
       r0=rmesh(ir)
       texp=r0**fgam
       ff0=Abs(f(ir)*texp/r0)
       If(ff0 > eps) Then
          !Print *,r,ff
          !Print *,r1,ff1
          !! linear interpolation
          r=Min(Max(r0,r0-(ff0-eps)*(r1-r0)/(ff1-ff0)),r1)
          !Print *,r0,r,r1
          !Print *,ff0,(ff1-ff0)/(r1-r0)*(r-r0)+ff0-eps,ff1
          !Print *
          rfmax=r
          If(ir >= nr-2 )Then
             rfmax=rmesh(nr)
          Endif
          Exit
       Endif
       ff1=ff0
       r1=r0
    Enddo
    !!
    Return
    !!
  End Subroutine find_fcutoff
  !!***********************************************************************
  !! print_basis
  Subroutine print_basis(structure,natom,basis,relativistic)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)        :: relativistic
    Type(T_Structure),Intent(in)     :: structure
    Type(T_Basis),Intent(in)         :: basis
    Real(t_real),Intent(in)      :: natom(structure% nsort)
    !!LOCAL
!    Character(*),Parameter::nsubfunc="print_basis"
    Integer(t_int) :: issa,issaIO,ic,iv,kap,l,j,m,ms,jj,ll,icomp
    Integer(t_int) :: mu,k,ml,jc,multo
    Real(t_real)   :: Q
    Logical(t_log) :: prot
    Character(3)   :: rangle(4)
    Character(1000):: buf
    Character(7)   :: dum(2)
    Character(40)  :: dummy
    Type(T_Basdef),Pointer :: bd
    !!
    !!
    prot=isoption('PROT_PRINT_BASIS')
    !!
    If ( .Not. prot  ) Return
    !!
1   Format(72('='))
    Write(outfile,1)
2   Format('NONRELATIVISTIC BASIS DEFINITION'&
         &,/,'(for magnetic relativistic case, the sorts below ',/&
         &,' belong to the reduced symmetry)')
    Write(outfile,2)
3   Format('sort El.  Z         core : semicore / valence ')
    Write(outfile,3)
10  Format()
    
    issaloopIO: Do issa=1_t_int,structure % nsort
       issaIO=structure% p_isrtTOIOisrt(issa)

       bd => basis% p_bdef(issaIO)

       Write(outfile,11,advance='no')issa,structure% p_elements(issa)&
            &,natom(issa)
       !! core states
       Do ic=1,bd% nchi
          multo=1
          Do jc=bd% nchi+1,bd% nc
             If( (bd% p_nnc(jc) == bd% p_nnc(ic)) .And. &
                  &(bd% p_lc(jc) == bd% p_lc(ic)))Then
                multo=multo+1
             Endif
          Enddo
          If(multo==1)Then
             Write(outfile,12,advance='no') bd% p_nnc(ic),lqnchar(bd% p_lc(ic))
          Else
             Write(outfile,312,advance='no') &
                  &multchars(multo:multo),bd% p_nnc(ic),lqnchar(bd% p_lc(ic))
          Endif
       End Do
       Write(outfile,112,advance='no')
       
       !! semi core states
       Do iv=1,bd% nv
          If(bd% p_tv(iv)==ORB_TYPE_SEMICORE)Then
             Write(outfile,12,advance='no') bd% p_nnv(iv),lqnchar(bd% p_lv(iv))
          Endif
       End Do
       Write(outfile,113,advance='no')
       
       
       !! valence states
       Do iv=1,bd% nv
          If(bd% p_tv(iv)==ORB_TYPE_VALENCE)Then
             Write(outfile,12,advance='no') bd% p_nnv(iv),lqnchar(bd% p_lv(iv))
          Endif
       End Do
       Write(outfile,10)
       !Print *,'hier1'
!!$       !! core confinement
!!$       Do ic=1,bd% nc
!!$          If(basis % p_IOcnfc(ic,issaIO)==CORE_CONF_INF)Then
!!$             Write(outfile,166) bd% p_nnc(ic),lqnchar(bd% p_lc(ic))&
!!$                  &,basis % p_IOrcinf(ic,issaIO)
!!$          Endif
!!$       Enddo
!!$166    Format(t20,i1,a1,': infinite confinement, rinf=',g10.5)
       !! Q
       Do ic=1,bd% nchi
          multo=1
          k=1
          Write(buf(k:),55)'('
          k=k+1
          Write(buf(k:),56)bd% p_Qionc(ic)
          k=Len_trim(buf)
          Do jc=ic+1,bd% nc
             If( (bd% p_nnc(jc) == bd% p_nnc(ic)) .And. &
                  &(bd% p_lc(jc) == bd% p_lc(ic)))Then
                multo=multo+1
                Write(buf(k:),55)','
                k=k+1
                Write(buf(k:),56)bd% p_Qionc(jc)
                k=Len_trim(buf)
             Endif
          Enddo
          Write(buf(k:),55)')'

          If(multo>1)Then
             Write(outfile,57) ic,multchars(multo:multo)&
                  &,bd% p_nnc(ic),lqnchar(bd% p_lc(ic)),Trim(buf)
          Endif
       Enddo
       Do iv=1,bd% nv
          Q=bd% p_Qionv(iv)
          If(Abs(Q)>1.0e-6_t_real)Then
             Write(outfile,266) iv,bd% p_nnv(iv),lqnchar(bd% p_lv(iv)),Q
          Endif
       Enddo
55     Format(a)
56     Format(g12.5)
57     Format(t20,'c',i4,'(',a1,i1,a1,'): Qion=',a)
266    Format(t20,'v',i4,'(',i1,a1,'): Qion=',g12.5)
       !!
    Enddo issaloopIO
    Write(outfile,10)
    Write(outfile,15)
    !!
11  Format(i4,1x,a2,1x,f10.5,'  ')
12  Format(i1,a1)
312 Format(a1,i1,a1)
112 Format(' : ')
113 Format(' / ')
15  Format(72('='))       
!101 Format('(')
!102 Format(')')
    !!
    !!------------------------------------------------------------------------
    !! relativistic basis 
    relif:If(relativistic==FULL_RELATIVISTIC ) Then
       
       
       rangle(1)='\  '
       rangle(2)=' \ '
       rangle(3)=' / '
       rangle(4)='/  '
       !!     
       dum(1)='  g(r) '
       dum(2)='i f(r) '
       !!     
       Write(outfile,200)
       Write(outfile,210)
       Write(outfile,200)
       !!
       Write(outfile,201)
       Write(outfile,220)
       Write(outfile,201)
       llopp: Do l=0,basis% lmx
          jjloop: Do jj=Max(2-l,1_t_int),2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             muloop: Do m=1,j+1
                mu=2*(m-1)-j
                Write(outfile,202)
                icomploop: Do icomp=1,2
                   k=kap*(3-2*icomp)
                   ll=(Abs(2*k+1)-1)/2
                   msloop: Do ms=1,2
                      ml=(mu-(3-2*ms))/2
                      Write(dummy,231) dum(icomp)&
                              &,cgoOFKapMu(ms,k,mu),ll,ml& 
                              &,rangle((icomp-1)*2+ms)
                      If ( Abs(ml)>ll .Or. Abs(mu)>2*Abs(k)-1) &
                           & Write(dummy,230) rangle((icomp-1)*2+ms)
                      If ((ms==2) .And. (icomp==1)) Then
                         Write(outfile,252) lqnchar(l),j,kap,mu,dummy
                      Else
                         Write(outfile,253) dummy
                      Endif
                   Enddo msloop
                Enddo icomploop
                Write(outfile,202)
                Write(outfile,201)
             Enddo muloop
          End Do jjloop
       End Do llopp
       !!
200    Format(72('-'))
201    Format(72('-'))
202    Format('|',t72,'|')
       !!
210    Format(5('='),t15,' RELATIVISTIC CALCULATION: ATOMIC BASIS SET '&
            &,t67,6('='))
!212    Format('BASIS STATES OF ATOMIC SORT',i4,' (',a2,')')
       !!
220    Format('|',t6,'| l j   ; kap ;  mu  >')
       !!
230    Format('|  0',t28,a3)
231    Format('|',a7,' * ',f6.3,' * Y_{',i1,i2,'}',a3)
       !!
252    Format('|',t6,'| ',a1,1x,i1,'/2 ; ',sp,i2,'  ; ',i2,'/2 > =',t31,a30&
            &,t72,'|')
253    Format('|',t31,a30,t72,'|')
       
    Endif relif
    !!
    !!
    !!
    Return
  End Subroutine print_basis
  !!***********************************************************************
  !!
  Subroutine setup_relat_basis(structure,basis,relativistic,vca,natom)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: relativistic
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(inout)  :: basis
    Logical(t_log),Intent(in)    :: vca
    Real(t_real),Intent(in)      :: natom(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_relat_basis"
    Integer(t_int) :: ic,iv,issa,jj,j,l,kap,icr,ivr,lcm,lvm,jv,jvr,jcr,jc
    Integer(t_int) :: nmain,issaIO,ivat,iz,j0,nline,isort,issb,issbIO,iz0
    Type(T_Basdef),Pointer :: bd
    Type(T_Basdef) :: bdef
    Character(1000):: buf,word,fname
    Character(2)   :: ch2
    Real(t_real)   :: Qion,ion,ZZ,Spol
    Logical(t_log) :: err,rel,srel,exists,khsrel
    Integer(t_int),Pointer :: itmpv(:),itmpc(:)
    !Character(1000):: bas
    Character(5000)   :: basdef
    Character(6)   :: str
    Integer :: eof
    !!
    Nullify(itmpc,itmpv)
    !!
    Call init_basdef(bdef)
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    srel=(relativistic == SCALAR_RELATIVISTIC)
    khsrel=(relativistic == SCALAR_RELATIVISTIC_KH)
    !Print *,'START:',nsubfunc
    !!
    !!
    !!
    !!
    Call palloc_basdefs('basis% p_bdef',basis% p_bdef&
         &,structure% IOnsort,nmodule,nsubfunc)
    !! load default basis definitions
    Do issaIO=1,structure% IOnsort
       iz=getelementnumber(structure% p_IOelements(issaIO))
       Call get_basdef(iz,' ',basdef)
       !! find a related sort number ...
       issa=-1
       Do issb=1,structure% nsort
          issbIO=structure% p_isrtTOIOisrt(issb)
          If(issbIO==issaIO) Then
             issa=issb
          Endif
       Enddo
       If(issa<0)  Then
          Write(errbuf,"('''Weird data inconsistency, please report!''')")
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
       Endif
       !! ... in order to get the nuclear charge
       ZZ=natom(issa)
       basis% p_bdef(issaIO)% pexdef =get_def_pex(iz,vca,ZZ)
       basis% p_bdef(issaIO)% exn2def=get_def_exn2(iz,vca,ZZ)
       Call extract_IO_basis(basdef,basis% p_bdef(issaIO))
    Enddo
    !!
    !!
    Inquire(file='=.basdef',exist=exists)
    If(exists)Then
       
       If(vca)Then
          Call errmsg(nmodule,nsubfunc,E_warning&
               &,msg="('VCA: user defined basis: the interpolation will be done&
               & with the default basis of the neighbouring atoms  !')")
       Endif
       
       nline=0
       Open(12,file='=.basdef',status='old')
       readloop: Do 
          Read(12,10,iostat=eof) buf
          If(eof < 0 ) Exit readloop
          If( Len_trim(buf)==0) Cycle readloop
          nline=nline+1
          If(next_word(buf,word))Then
             Read(word,*) isort
          Endif
          If(isort <1 .Or. isort > structure% nsort) Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="(' in =.basdef wrong sort')")
          Endif
          
          !! find an according sort number ...
          issa=-1
          Do issb=1,structure% nsort
             issbIO=structure% p_isrtTOIOisrt(issb)
             If(issbIO==isort) Then
                issa=issb
             Endif
          Enddo
          If(issa<0)  Then
             Write(errbuf,"('''Weird data inconsistency, please report!''')")
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
          Endif
          !! ... in order to get the nuclear charge
          ZZ=natom(issa)
          iz=getelementnumber(structure% p_IOelements(isort))
          !!
          basis% p_bdef(isort)% pexdef =get_def_pex(iz,vca,ZZ)
          basis% p_bdef(isort)% exn2def=get_def_exn2(iz,vca,ZZ)
          !!
          basdef=Trim(buf)
          Call extract_IO_basis(basdef,basis% p_bdef(isort))
       Enddo readloop
       Close(12)
    Endif
10  Format(a)
    !!
    !!
    fname='=.basdef'
1   Format(72('-'),/,'Start: content of ',a,/,72('-'))
3   Format(72('-'),/,'End  : content of ',a,/,72('-'))
    !!
    Write(outfile,1) Trim(fname)
    Do issaIO=1,structure% IOnsort
       Call stringify_basdef(basis% p_bdef(issaIO),basdef)
       Write(outfile,"(i5,1x,a)")issaIO,Trim(basdef)
    Enddo
    Write(outfile,3)Trim(fname)


    !! VCA: interpolate basis sets as good as possible
    VCAissaIOloop:Do issaIO=1,structure% IOnsort
       iz=getelementnumber(structure% p_IOelements(issaIO))
       iz0=iz
       
       If(vca) Then
          
          !! find an according sort number ...
          issa=-1
          Do issb=1,structure% nsort
             issbIO=structure% p_isrtTOIOisrt(issb)
             If(issbIO==issaIO) Then
                issa=issb
             Endif
          Enddo
          If(issa<0)  Then
             Write(errbuf,"('''Weird data inconsistency, please report!''')")
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
          Endif
          !! ... in order to get the nuclear charge
          ZZ=natom(issa)
          ion=ZZ-Real(iz0,t_real)
          If(Abs(ion)<1.0e-5_t_real) Cycle VCAissaIOloop
          !! here we assmue that iz0-1<=ZZ<=iz0+1
          If(ZZ<iz0) Then
             iz=iz0-1
             Print *,'iz:',iz,ZZ,iz0
          Endif
          If(ZZ>iz0) Then
             iz=iz0+1
             Print *,'iz:',iz0,ZZ,iz
          Endif
          If(iz>MAX_ELEMENT) Then
             Write(errbuf,"('''Element number ',i5,' not implemented!''')")&
                  &iz
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
          Endif

          !! we need this here for VCA otherwise the interpolation is wrong
          !! if pex and exn2 are not defined in basdef
          Call init_basdef(bdef)
          bdef% pexdef =get_def_pex(iz,vca,ZZ)
          bdef% exn2def=get_def_exn2(iz,vca,ZZ)
          !!
          !!
          !! get neighbouring basis for interpolation purposes
          !!
          Call get_basdef(iz,' ',basdef)
          Call extract_IO_basis(basdef,bdef)

          
          Call merge_basdefs(basis% p_bdef(issaIO),bdef,Abs(ion))
          
          Call clean_basdef(bdef)
          
       Endif
       !!
    Enddo VCAissaIOloop
    !!
    !!
    !! general purpose arrays have the capital R (relativistic) in the name
    !! they describe the basis for non relativistic and for full
    !! relativistic calculations. 
    !!
    Call palloc('basis % p_nRc',basis % p_nRc &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_nRchi',basis % p_nRchi &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_nRv',basis % p_nRv &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_maxl',basis % p_maxl &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_nNLv',basis % p_nNLv &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    
    !! determine dimensions of arrays
    If(rel)Then
       basis% ncomp=2
    Else If(srel)Then
       basis% ncomp=4
    Else If(khsrel)Then
       basis% ncomp=3
    Else
       basis% ncomp=1
    Endif
    !!
    basis% nNLvmx=0_t_int
    basis% nRchimx=0_t_int
    basis% nRcmx=0_t_int
    basis% nRvmx=0_t_int
    basis% degRcmx=0_t_int
    basis% degRvmx=0_t_int
    lcm=0_t_int
    lvm=0_t_int
    basis% p_maxl(:)=0_t_int
    basis% p_nRchi(:)=0_t_int
    basis% p_nRc(:)=0_t_int
    issaloop0: Do issa=1_t_int,structure % nsort
       !!
       issaIO=structure% p_isrtTOIOisrt(issa)
       !!
       bd => basis% p_bdef(issaIO)
       
       !! 
       icr=0
       icloop0: Do ic=1,bd% nc
          nmain=bd% p_nnc(ic)
          l=bd% p_lc(ic)
          !! testing
          If(nmain-l-1<0)Then
             Write(errbuf,634)structure% p_IOelements(issaIO),issaIO&
                  &,nmain,lqnchar(l)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          If(l>3)Then
             Write(errbuf,636)structure% p_IOelements(issaIO),issaIO&
                  &,nmain,lqnchar(l)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          !!
          lcm=Max(lcm,l)
          basis% p_maxl(issa)=Max(basis% p_maxl(issa),l)
          If(rel)Then
             icr=icr+Min(2_t_int,l+1)
             basis% degRcmx=Max(basis% degRcmx,2*l+2)
          Else
             icr=icr+1
             basis% degRcmx=Max(basis% degRcmx,2*l+1)
          Endif
          If(ic==bd% nchi)Then
             basis% p_nRchi(issa)=icr
             basis% nRchimx=Max(basis% nRchimx,icr)
          Endif
       End Do icloop0
       basis% p_nRc(issa)=icr
       basis% nRcmx=Max(basis% nRcmx,icr)
       
      
       ivr=0_t_int
       ivloop0: Do iv=1,bd% nv
          !!
          nmain=bd% p_nnv(iv)
          l=bd% p_lv(iv)
          !! testing
          If(nmain-l-1<0)Then
             Write(errbuf,635)structure% p_IOelements(issaIO),issaIO&
                  &,nmain,lqnchar(l)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          !!
          lvm=Max(lvm,l)
          basis% p_maxl(issa)=Max(basis% p_maxl(issa),l)

          If(rel) Then
             ivr=ivr+Min(2_t_int,l+1)
             basis% degRvmx=Max(basis% degRvmx,2*l+2)
          Else
             ivr=ivr+1
             basis% degRvmx=Max(basis% degRvmx,2*l+1)
          Endif
          !!
       End Do ivloop0
       basis% p_nRv(issa)=ivr
       basis% nRvmx=Max(basis% nRvmx,ivr)
       basis% p_nNLv(issa)=bd% nv
       basis% nNLvmx=Max(basis% nNLvmx,bd% nv)
       
    End Do issaloop0
    !!

    !!
    basis% lvmx=lvm
    basis% lcmx=lcm
    basis% lmx=Max(basis% lvmx,basis% lcmx)
    !!

    !! allocate all arrays
    Call palloc('basis % p_lRc',basis % p_lRc &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_lRv',basis % p_lRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_lNLv',basis % p_lNLv &
         &,1_t_int,basis% nNLvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_tRv',basis % p_tRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_degRc',basis % p_degRc &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_degRv',basis % p_degRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_degNLv',basis % p_degNLv &
         &,1_t_int,basis% nNLvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_degNLSv',basis % p_degNLSv &
         &,1_t_int,basis% nNLvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_nnRc',basis % p_nnRc &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_nnRv',basis % p_nnRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_nnNLv',basis % p_nnNLv &
         &,1_t_int,basis% nNLvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_kapRc',basis % p_kapRc &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_kapRv',basis % p_kapRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_firstRv',basis % p_firstRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_firstNLv',basis % p_firstNLv &
         &,1_t_int,basis% nNLvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_pex',basis % p_pex &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_exn2',basis % p_exn2 &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)


    Call palloc('basis % p_pexRv',basis % p_pexRv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis % p_icrTOic',basis % p_icrTOic &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    Call palloc('basis % p_ivrTOiv',basis % p_ivrTOiv &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)


    !! initialize all relativistic organization arrays
    !! 
    issaloop1: Do issa=1_t_int,structure % nsort
       !!
       issaIO=structure% p_isrtTOIOisrt(issa)
       !!
       iz=getelementnumber(structure% p_IOelements(issaIO))
!!TODO: vatclasses
       bd =>basis% p_bdef(issaIO)
       !!
       !!
       basis% p_pex(issa)=bd% pexdef
       basis% p_exn2(issa)=bd% exn2def
       !!
  !     Print *,'confinement:',issa,basis% p_pex(issa),basis% p_exn2(issa)
       !! core case
       icr=0
       icloop1: Do ic=1,bd% nc
          l=bd% p_lc(ic)
          j0=Max(2-l,1_t_int)
          If(.Not.rel) j0=2
          jcloop1: Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             icr=icr+1
             !!map relativistic index to nonrel
             basis% p_icrTOic(icr,issa)=ic 
             If(rel)Then
                basis% p_degRc(icr,issa)=j+1
             Else
                basis% p_degRc(icr,issa)=2*l+1
             Endif
             basis% p_nnRc(icr,issa)=bd% p_nnc(ic)
             basis% p_lRc(icr,issa)=l
             basis% p_kapRc(icr,issa)=kap
          Enddo jcloop1
       End Do icloop1
       !!
       !! valence case
       ivr=0
       ivloop1: Do iv=1,bd% nv
          l=bd% p_lv(iv)
          j0=Max(2-l,1_t_int)
          If(.Not.rel) j0=2
          jvloop1: Do jj=j0,2
             kap=l-(jj-1)*(2*l+1)
             j=2*Abs(kap)-1
             ivr=ivr+1
             !!map relativistic index to nonrel
             basis% p_ivrTOiv(ivr,issa)=iv 
             If(rel) Then
                basis% p_degRv(ivr,issa)=j+1
                basis% p_degNLv(iv,issa)=2*(2*l+1)
             Else
                basis% p_degRv(ivr,issa)=2*l+1
                basis% p_degNLv(iv,issa)=2*l+1
             Endif
             basis% p_degNLSv(iv,issa)=2*l+1
             basis% p_nnRv(ivr,issa)=bd% p_nnv(iv)
             basis% p_nnNLv(iv,issa)=bd% p_nnv(iv)
             ch2(1:1)=bd% p_firstv(iv)(1:1)
             ch2(2:2)=bd% p_firstv(iv)(2:2)
             basis% p_firstRv(ivr,issa)=ch2
             basis% p_firstNLv(iv,issa)=ch2
             basis% p_pexRv(ivr,issa)=bd% p_Pexv(iv)
             basis% p_lRv(ivr,issa)=l
             basis% p_lNLv(iv,issa)=l
             basis% p_tRv(ivr,issa)=bd% p_tv(iv)
             basis% p_kapRv(ivr,issa)=kap
          Enddo jvloop1
       End Do ivloop1
       !!
    Enddo issaloop1
    !!
    If(.Not.rel)Then
       !! paranoia: better I need treatment, than the program 
       err=.False.
       Do issa=1,structure% nsort
          issaIO=structure% p_isrtTOIOisrt(issa)

          bd => basis% p_bdef(issaIO)

          If(basis% p_nRc(issa)/=bd% nc) err=.True.
          If(basis% p_nRchi(issa)/=bd% nchi) err=.True.
          Do icr=1,basis% p_nRc(issa)
             ic=icr
             If(basis% p_icrTOic(icr,issa)/=ic) err=.True.
             kap=basis% p_kapRc(icr,issa)
             l=(Abs(2*kap+1)-1)/2
             If(bd% p_lc(ic)/=l) err=.True.
             l=-kap-1
             If(bd% p_lc(ic)/=l) err=.True.
             If(basis% p_nnRc(icr,issa)/=bd% p_nnc(ic)) err=.True.
             If(basis% p_degRc(icr,issa)/=2*l+1) err=.True.
             If(basis% p_lRc(icr,issa)/=l) err=.True.
             If(basis% p_degRc(icr,issa)> basis% degRcmx) err=.True.
          Enddo
          If(basis% p_nRv(issa)/=bd% nv) err=.True.
          Do ivr=1,basis% p_nRv(issa)
             iv=ivr
             If(basis% p_ivrTOiv(ivr,issa)/=iv) err=.True.
             kap=basis% p_kapRv(ivr,issa)
             l=(Abs(2*kap+1)-1)/2
             If(bd% p_lv(iv)/=l) err=.True.
             l=-kap-1
             If(bd% p_lv(iv)/=l) err=.True.
             If(basis% p_nnRv(ivr,issa)/=bd% p_nnv(iv)) err=.True.
             If(basis% p_nnNLv(iv,issa)/=bd% p_nnv(iv)) err=.True.
             If(basis% p_lRv(ivr,issa)/=l) err=.True.
             If(basis% p_lNLv(iv,issa)/=l) err=.True.
             If(basis% p_degRv(ivr,issa)/=2*l+1) err=.True.
             If(basis% p_degRv(ivr,issa)> basis% degRvmx) err=.True.
          Enddo
       Enddo
       If(err)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="' Non relativistic index arrays are wrong, Oops!'")
       Endif
    Endif
    !!
    !!
    !!
    !! make mapping of orbitals into vatom-classes
    !!
    Call palloc('basis% p_nvat',basis% p_nvat,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
 
    Call palloc('itmpc',itmpc,1_t_int,basis% nRcmx,nmodule,nsubfunc)
    Call palloc('itmpv',itmpv,1_t_int,basis% nRvmx,nmodule,nsubfunc)
    basis% maxNvat=0
    Do issa=1,structure% nsort
       issaIO=structure% p_isrtTOIOisrt(issa)
       
       bd => basis% p_bdef(issaIO)

       itmpc(:)=0_t_int
       itmpv(:)=0_t_int
       ivat=0
       icloop01: Do icr=1,basis% p_nRc(issa)
          ic=basis% p_icrTOic(icr,issa)
          l=basis% p_lRc(icr,issa)
          nmain=basis % p_nnRc(icr,issa)
          Qion=bd% p_Qionc(ic)
          Spol=bd% p_Spolc(ic)
          !Write(outfile,999) issaIO,ic,nmain,lqnchar(l),Qion
          If(itmpc(icr)==0)Then
             ivat=ivat+1
             itmpc(icr)=ivat
             !Print *,'ivat=',ivat,itmpc(icr)
             Do jcr=1,basis% p_nRc(issa)
                jc=basis% p_icrTOic(jcr,issa)
                !Print *,'jcr=',jcr,itmpc(jcr),bd% p_Qionc(jc)
                If(Abs(bd% p_Qionc(jc)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolc(jc)-Spol) <1.0e-6_t_real)Then
                   itmpc(jcr)=ivat
                   !Print *,'itmpc(',jcr,')=',itmpc(jcr)
                Endif
             Enddo
             Do jvr=1,basis% p_nRv(issa)
                jv=basis% p_ivrTOiv(jvr,issa)
                !Print *,'jvr=',jvr,itmpv(jvr),bd% p_Qionv(jv)
                If(Abs(bd% p_Qionv(jv)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolv(jv)-Spol) <1.0e-6_t_real)Then
                   itmpv(jvr)=ivat
                   !Print *,'itmpv(',jvr,')=',itmpv(jvr)
                Endif
             Enddo
             Cycle icloop01
          Endif
       Enddo icloop01
       !!
       ivloop01: Do ivr=1,basis% p_nRv(issa)
          iv=basis% p_ivrTOiv(ivr,issa)
          l=basis% p_lRv(ivr,issa)
          nmain=basis % p_nnRv(ivr,issa)
          Qion=bd% p_Qionv(iv)
          Spol=bd% p_Spolv(iv)
          !Write(outfile,999) issaIO,iv,nmain,lqnchar(l),Qion
          If(itmpv(ivr)==0)Then
             ivat=ivat+1
             itmpv(ivr)=ivat
             !Print *,'ivat=',ivat,itmpv(ivr)
             Do jvr=1,basis% p_nRv(issa)
                jv=basis% p_ivrTOiv(jvr,issa)
                !Print *,'jvr=',jvr,itmpv(jvr),bd% p_Qionv(jv)
                If(Abs(bd% p_Qionv(jv)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolv(jv)-Spol) <1.0e-6_t_real)Then
                   !Print *,'itmpv(',jvr,')=',itmpv(jvr)
                   itmpv(jvr)=ivat
                Endif
             Enddo
             Cycle ivloop01
          Endif
       Enddo ivloop01
       basis% p_nvat(issa)=ivat
       basis% maxNvat=Max(basis% maxNvat,ivat)
       !Write(outfile,*) ivat
       !Write(outfile,*) itmpc(1:basis% nRcmx)
       !Write(outfile,*) itmpv(1:basis% nRvmx)
    Enddo
    !!
    Call pfree(itmpc)
    Call pfree(itmpv)
    !!
    Call palloc('basis% p_Qvat',basis% p_Qvat,1_t_int,basis% maxNvat&
         &,1_t_int,structure% nsort,nmodule,nsubfunc)
    Call palloc('basis% p_Svat',basis% p_Svat,1_t_int,basis% maxNvat&
         &,1_t_int,structure% nsort,nmodule,nsubfunc)
    Call palloc('basis% p_vatclsv',basis% p_vatclsv,1_t_int,basis% nRvmx&
         &,1_t_int,structure% nsort,nmodule,nsubfunc)
    Call palloc('basis% p_vatclsc',basis% p_vatclsc,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort,nmodule,nsubfunc)
    !!
    Do issa=1,structure% nsort
       issaIO=structure% p_isrtTOIOisrt(issa)

       bd => basis% p_bdef(issaIO)

       basis% p_vatclsv(:,issa)=0_t_int
       basis% p_vatclsc(:,issa)=0_t_int
       ivat=0
       icloop02: Do icr=1,basis% p_nRc(issa)
          ic=basis% p_icrTOic(icr,issa)
          l=basis% p_lRc(icr,issa)
          nmain=basis % p_nnRc(icr,issa)
          Qion=bd% p_Qionc(ic)
          Spol=bd% p_Spolc(ic)
          If(basis% p_vatclsc(icr,issa)==0)Then
             ivat=ivat+1
             basis% p_Qvat(ivat,issa)=Qion
             basis% p_Svat(ivat,issa)=Spol
             basis% p_vatclsc(icr,issa)=ivat
             Do jcr=1,basis% p_nRc(issa)
                jc=basis% p_icrTOic(jcr,issa)
                If(Abs(bd% p_Qionc(jc)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolc(jc)-Spol) <1.0e-6_t_real)Then
                   basis% p_vatclsc(jcr,issa)=ivat
                Endif
             Enddo
             Do jvr=1,basis% p_nRv(issa)
                jv=basis% p_ivrTOiv(jvr,issa)
                If(Abs(bd% p_Qionv(jv)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolv(jv)-Spol) <1.0e-6_t_real)Then
                   basis% p_vatclsv(jvr,issa)=ivat
                Endif
             Enddo
             Cycle icloop02
          Endif
       Enddo icloop02
       ivloop02: Do ivr=1,basis% p_nRv(issa)
          iv=basis% p_ivrTOiv(ivr,issa)
          l=basis% p_lRv(ivr,issa)
          nmain=basis % p_nnRv(ivr,issa)
          Qion=bd% p_Qionv(iv)
          Spol=bd% p_Spolv(iv)
          If(basis% p_vatclsv(ivr,issa)==0)Then
             ivat=ivat+1
             basis% p_Qvat(ivat,issa)=Qion
             basis% p_Svat(ivat,issa)=Spol
             basis% p_vatclsv(ivr,issa)=ivat
             Do jvr=1,basis% p_nRv(issa)
                jv=basis% p_ivrTOiv(jvr,issa)
                If(Abs(bd% p_Qionv(jv)-Qion) <1.0e-6_t_real.And.&
                     &Abs(bd% p_Spolv(jv)-Spol) <1.0e-6_t_real)Then
                   basis% p_vatclsv(jvr,issa)=ivat
                Endif
             Enddo
             Cycle ivloop02
          Endif
       Enddo ivloop02
    Enddo
    !!
    !! print result
    !!
    Write(outfile,987)
    Do issa=1,structure% nsort
       issaIO=structure% p_isrtTOIOisrt(issa)
       Do icr=1,basis% p_nRc(issa)
          l=basis% p_lRc(icr,issa)
          kap=basis% p_kapRc(icr,issa)
          j=2*Abs(kap)-1
          nmain=basis % p_nnRc(icr,issa)
          ivat=basis% p_vatclsc(icr,issa)
          If(rel)Then
             Write(str,173)nmain,lqnchar(l),j
          Else
             Write(str,172)nmain,lqnchar(l)
          Endif
          Write(outfile,988) issa,structure% p_elements(issa),icr&
               &,str,'core',ivat,basis % p_Qvat(ivat,issa)&
               &,basis % p_Svat(ivat,issa)&
               &,0.0_t_real,0.0_t_real,0.0_t_real
       Enddo
       Do ivr=1,basis% p_nRv(issa)
          l=basis% p_lRv(ivr,issa)
          nmain=basis % p_nnRv(ivr,issa)
          kap=basis% p_kapRv(ivr,issa)
          j=2*Abs(kap)-1
          ivat=basis% p_vatclsv(ivr,issa)
          If(rel)Then
             Write(str,173)nmain,lqnchar(l),j
          Else
             Write(str,172)nmain,lqnchar(l)
          Endif
          Write(outfile,988) issa,structure% p_elements(issa),ivr&
               &,str,'val ',ivat,basis % p_Qvat(ivat,issa)&
               &,basis % p_Svat(ivat,issa)&
               &,basis% p_exn2(issa),basis% p_pex(issa)&
               &,basis% p_pexRv(ivr,issa)
       Enddo
    Enddo
    Write(outfile,986)
    !!
172       Format(i1,a1)
173       Format(i1,a1,1x,i1,'/2')
!999 Format('sort: ',i5,' state: ',i5,' ',i1,a1,' Q=',f12.6)
987 Format(74('-'),/,74('-'),/,'         Vatom-classes and ionicities'&
         &,/,74('-'),/,'   sort     state           vat-class   Q      S   &
         &  N       p        px',/,74('-'))
988 Format(i5,' ',a2,' ',i5,' ',a6,2x,a4,1x,i5,' ',f8.4,1x,f8.4,1x,f5.2,2(1x,f8.4))
986 Format(74('-'))
    !!
    !!
    !!
    !!
    Call clean_basdef(bdef)
    !!
    !!
    Return
    !!
634 Format('''Atom ',a1,' sort(',i3,'): the core orbital ',i1,a1&
         &,' does not exist!''') 
635 Format('''Atom ',a1,' sort(',i3,'): the valence orbital ',i1,a1&
         &,' does not exist!''') 
636 Format('''Atom ',a1,' sort(',i3,'): orbital ',i1,a1&
         &,' not allowed in core!''') 
    !!
  End Subroutine setup_relat_basis

  !!***********************************************************************
  !!
  !! Calculates spin character of local basis states
  !! This is the expectation value of \beta\Sigma_z
  Subroutine spin_character(nsort,elements,basis,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(inout)  :: basis
    Integer(t_int),Intent(in)    :: nsort
    Character(*),Intent(in)      :: elements(:)
    Integer(t_int),Intent(in)    :: relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="spin_character"
    Logical(t_log) :: exists
    Integer(t_int) :: issa,ivr,icr,kap,l,j,mu,nmain,m,multo,nr
    Real(t_real)   :: val,alpha,sp
    Real(t_real),Pointer :: p_tr(:)
    Real(t_real),Pointer :: p_texp(:)
    Type(T_Mesh),Pointer :: bmesh
    !!
    Nullify(p_tr)
    Nullify(p_texp)
    !!
    If(relativistic /= FULL_RELATIVISTIC) Return
    !!
    !!
    !!     --------------------------------------------
    !!     ------------- PREPARATIONS -----------------
    !!     --------------------------------------------
    Write(outfile,1000)
    Write(outfile,1010)
    !!     --------------------------------------------
    !!     ----------- MATRIX ELEMENTS ----------------
    !!     --------------------------------------------
    sortloop: Do issa=1,nsort
       !!
       bmesh => basis% p_basmeshs(issa)
       !!
       nr=bmesh% nr
       !!
       Call palloc('p_tr'  ,p_tr  ,1_t_int,nr,nmodule,nsubfunc)
       Call palloc('p_texp',p_texp,1_t_int,nr,nmodule,nsubfunc)
       !!
       p_texp(1:nr)=bmesh% p_rmesh(1:nr)**(basis% p_fgam(issa))
       icrloop: Do icr=1,basis% p_nRc(issa)

          !!
          nmain=basis% p_nnRc(icr,issa)
          kap=basis% p_kapRc(icr,issa)
          j=2*Abs(kap)-1
          l=(Abs(2*kap+1)-1)/2
          !!
          Do m=1,basis% p_degRc(icr,issa)
             mu=-j+2*(m-1)
             !Print *,'m=',m,' mu=',mu
             !!
             sp=0.0_t_real
             !!
             alpha=c_mukk(mu,kap,kap,exists)
             If(exists)Then
                p_tr(1:nr)=&
                     &  basis% p_fc(1:nr,icr,LARGE_COMP1,issa)&
                     & *basis% p_fc(1:nr,icr,LARGE_COMP1,issa)&
                     & *p_texp(1:nr)**2
                Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
                sp=sp+val*alpha
             Endif
             !!
             !!
             !!
             alpha=c_mukk(mu,-kap,-kap,exists)
             If(exists)Then
                p_tr(1:nr)=&
                     &  basis% p_fc(1:nr,icr,SMALL_COMP1,issa)&
                     & *basis% p_fc(1:nr,icr,SMALL_COMP1,issa)&
                     & *p_texp(1:nr)**2
                Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
                !!
                sp=sp-val*alpha
             Endif
             !!
             !!
             !!
             !!
             multo=coreMultIndex(icr,issa,basis)
             !!
             Write(outfile,1101) elements(issa),nmain,lqnchar(l),multo,j,mu,sp
          Enddo ! m
       Enddo icrloop
       ivrloop: Do ivr=1,basis% p_nRv(issa)

          
          nmain=basis% p_nnRv(ivr,issa)
          kap=basis% p_kapRv(ivr,issa)
          j=2*Abs(kap)-1
          l=(Abs(2*kap+1)-1)/2
          !!
          Do m=1,basis% p_degRv(ivr,issa)
             mu=-j+2*(m-1)
             !Print *,'m=',m,' mu=',mu
             !!
             sp=0.0_t_real
             !!
             alpha=c_mukk(mu,kap,kap,exists)
             If(exists)Then
                p_tr(1:nr)=&
                     &  basis% p_fv(1:nr,ivr,LARGE_COMP1,issa)&
                     & *basis% p_fv(1:nr,ivr,LARGE_COMP1,issa)&
                     & *p_texp(1:nr)**2
                Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
                sp=sp+val*alpha
             Endif
             !!
             !!
             !!
             alpha=c_mukk(mu,-kap,-kap,exists)
             If(exists)Then
                p_tr(1:nr)=&
                     &  basis% p_fv(1:nr,ivr,SMALL_COMP1,issa)&
                     & *basis% p_fv(1:nr,ivr,SMALL_COMP1,issa)&
                     & *p_texp(1:nr)**2
                Call tjlag4int(nr,p_tr,bmesh% p_jmesh,bmesh% hx,val)
                !!
                sp=sp+val*alpha
             Endif
             !!
             !!
             !!
             Write(outfile,1100) elements(issa),nmain,lqnchar(l),j,mu,sp
          Enddo
       Enddo ivrloop 
    Enddo sortloop
    !!
    Write(outfile,1000)
    !!
    Call pfree(p_texp)
    Call pfree(p_tr)
    !!
    Return
    !!
1000 Format(72('-'))
1010 Format(t3,'ATOM',t9,'ELECTRON',t22,'J_z',t28,'<Sigma_z>')
1100 Format(72('-')/'|',t4,a2,t11,i1,a1,1x,i1,'/2',t20,i3,'/2',t29,f14.6&
          &,t72,'|')
1101 Format(72('-')/'|',t4,a2,t11,i1,a1,i1,1x,i1,'/2',t20,i3,'/2',t29,f14.6&
          &,t72,'|')
    !!
  End Subroutine spin_character
  !!***********************************************************************
  !! build transfomration matricees for the mapping of psedo non-relativistic
  !! into true relativistic matrices
  !!
  !! mapping if spin index ms=1,2 onto spin variable s=1,-1:
  !!                              s(ms)=3-2*ms
  !! back mapping
  !!                              ms(s)=(3-s)/2
  !! thus
  !!                              ms(-s)=3-ms
  !!
  !! mapping of linear index onto mu is as follows ( j=(2|k|-1)/2 )
  !!
  !! mu=-j,j ->  m=1,2|k|   2mu=2(m-|k|)-1
  !! 
  !! we use 2j and 2mu as variable !!!
  !!
  !! mu2(m)=2(m-|k|)-1
  !! m(mu2)=(mu2+1)/2+|k|
  !! m(-mu2)=2|k|+1-m=2j+2-m
  !!
  !!
  Subroutine  build_map_trans(basis,quant,relativistic)
    !!
    Implicit none
    !!IO
    Type(T_Basis),Intent(inout)  :: basis
    Type(T_Quant),Intent(in)     :: quant
    Integer(t_int),Intent(in)    :: relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="build_map_trans"
    !!
    Logical(t_log) :: rel
    Integer(t_int) :: l,kap,kp,ma,mmu,mub,ms,ml,lmax,s,fac,mmuneg,msneg
    Complex(t_real):: c,cd(2),T,Tp
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Pointer::  p_Uylm(:,:,:)
    !!
    lmax=basis% lmx
    Nullify(p_Uylm)
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    !!
    !Print *,'lmax=' ,lmax
    Call palloc('basis% p_relTOpseudo',basis% p_relTOpseudo &
         &,1_t_int,(2*lmax+3)& !! deg_{lmax+1}
         &,1_t_int,(2*lmax+2)& !! deg_j = 2( (lmax+1) -1/2)+1 
         &,1_t_int,2_t_int& !! pseudo-spin
         &,-(lmax+1),lmax+1& !! kap
         &,nmodule,nsubfunc)
    Call palloc('basis% p_relTOpseudoNoD',basis% p_relTOpseudoNoD &
         &,1_t_int,(2*lmax+3)& !! deg_{lmax+1}
         &,1_t_int,(2*lmax+2)& !! deg_j = 2( (lmax+1) -1/2)+1 
         &,1_t_int,2_t_int& !! pseudo-spin
         &,-(lmax+1),lmax+1& !! kap
         &,nmodule,nsubfunc)
    !!
    If(.Not.(rel)) Return
    !!
    Call palloc('p_Uylm',p_Uylm,1_t_int,2*lmax+3,1_t_int,2*lmax+3,0_t_int,lmax+1&
         &,nmodule,nsubfunc)
    !!
    Call  make_ylm_U_matrix(lmax+1,p_Uylm)
    !!
    !!
    !!
    !!
    !! 
    !!
    kaploop: Do kap=-lmax-1,lmax+1
       kp=Abs(kap)
       If(kap==0) Cycle kaploop
       l=(Abs(2*kap+1)-1)/2
       !!
       !Print *,' state=',lqnchar(l),2*kp-1,'/2'
       maloop: Do ma=1,2*l+1
          !Print *,' m_l=',ma-l-1
          mmubloop: Do mmu=1,2*kp !! j+1
             mub=2*(mmu-kp)-1
             !Print *,' m_l=',ma-l-1,'  mu',mub,'/2'
             msloop: Do ms=1,2
                !! one-bases m_l index
                !ml=(mmu-kp+ms-2)+l+1
                ml=(mub-1)/2+ms+l
                !!          Print *,' m_l='ma-l-1

                If(ml>=1 .And. ml<=2*l+1)Then
                   c=p_Uylm(ma,ml,l)*cgoOFKapMu(ms,kap,mub)
                Else
                   c=c0
                Endif
                !!
                !Print *,'index =',ma,mmu
                cd(ms)=c
             End Do msloop
             !! rotate spin (magnetic field) axis
             !!
             Do ms=1,2
                basis% p_relTOpseudo(ma,mmu,ms,kap)=&
                     &Dot_product(quant% D1o2(1:2,ms),cd(1:2))
                basis% p_relTOpseudoNoD(ma,mmu,ms,kap)=cd(ms)
             Enddo
             !!
          End Do mmubloop
       Enddo maloop
    Enddo kaploop
    !!
    !!
    !!
    Do kap=-lmax-1,lmax+1
       kp=Abs(kap)
       If(kap==0) Cycle
       l=(Abs(2*kap+1)-1)/2
       !!
       Do ma=1,2*l+1
          Do mmu=1,2*kp !! j+1
             mub=2*(mmu-kp)-1
             Do ms=1,2
                s=3-2*ms
                mmuneg=2*kp+1-mmu
                msneg=3-ms
                fac=(-1)**( (mub+1)/2 ) *kap/kp*s
                !!
                T=Conjg(basis% p_relTOpseudo(ma,mmu,ms,kap))
                Tp=basis% p_relTOpseudo(ma,mmuneg,msneg,kap)*fac
                If( Abs( T - Tp) >1.0e-12_t_real ) Then
                   Write(outfile,12) kap,ma-l-1,mub,3-2*ms,T,TP
12                 Format('kap =',i5,' ml=',i2,' mu=',i2,'/2 s=',i2,3x&
                        &,2('(',e20.5e3,',',e20.5e3,') '))
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="' Symmetries of relTOpseudo&
                        & are wong?, Oops!'")
                Endif
             End Do
          End Do
       End Do
    Enddo
    !!
    !!
!!$    Do l=0,2
!!$       Do ms=1,2
!!$          Do ma=1,2*l+1
!!$             Do kap=l,-l-1,-(2*l+1)
!!$                If(kap==0) Cycle
!!$                Do mmu=1,2*Abs(kap)
!!$                   Write(outfile,"(f9.5,'+i*',f9.5,' ')",advance='no')&
!!$                        &basis% p_relTOpseudo(ma,mmu,ms,kap)
!!$                Enddo
!!$             Enddo
!!$             Write(outfile,"()")
!!$          Enddo
!!$       Enddo
!!$    Enddo
!!$    Stop
    !!
    Call pfree(p_Uylm)
    !!
    Return
    !!
  End Subroutine build_map_trans
  !!***********************************************************************
  ! {{{ 

  Subroutine allocate_basis(structure,basis,calc_forces)
    !!
    Implicit None 
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(inout)  :: basis
    Logical(t_log),Intent(in)    :: calc_forces
    !!LOCAL
    Character(*),Parameter::nsubfunc="allocate_basis"
    !!
    !! allocate core and valence states

    Call palloc('basis% p_fgam',basis% p_fgam&
         &,1_t_int,structure% nsort,nmodule,nsubfunc)


    !! all core states
    Call palloc('basis% p_fc',basis% p_fc &
         &,1_t_int,basis% nrb&
         &,1_t_int,basis% nRcmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    If(calc_forces) Then
       Call palloc('basis% p_fdpc',basis% p_fdpc &
            &,1_t_int,basis% nrb&
            &,1_t_int,basis% nRcmx&
            &,1_t_int,basis% ncomp&
            &,1_t_int,structure% nsort&
            &,nmodule,nsubfunc)
       Call palloc('basis% p_fdmc',basis% p_fdmc &
            &,1_t_int,basis% nrb&
            &,1_t_int,basis% nRcmx&
            &,1_t_int,basis% ncomp&
            &,1_t_int,structure% nsort&
            &,nmodule,nsubfunc)
    Endif
    !! kinetic functions for all core states
    Call palloc('basis% p_fkc',basis% p_fkc &
         &,1_t_int,basis% nrb&
         &,1_t_int,basis% nRcmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis% p_fv',basis% p_fv &
         &,1_t_int,basis% nrb&
         &,1_t_int,basis% nRvmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    If(calc_forces) Then
       Call palloc('basis% p_fdpv',basis% p_fdpv &
            &,1_t_int,basis% nrb&
            &,1_t_int,basis% nRvmx&
            &,1_t_int,basis% ncomp&
            &,1_t_int,structure% nsort&
            &,nmodule,nsubfunc)
       Call palloc('basis% p_fdmv',basis% p_fdmv &
            &,1_t_int,basis% nrb&
            &,1_t_int,basis% nRvmx&
            &,1_t_int,basis% ncomp&
            &,1_t_int,structure% nsort&
            &,nmodule,nsubfunc)
    End If
    
    Call palloc('basis% p_fkv',basis% p_fkv &
         &,1_t_int,basis% nrb&
         &,1_t_int,basis% nRvmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)



!    Call mem_status()



    !! maximum radii
    Call palloc('basis% p_rfcmax',basis% p_rfcmax &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis% p_rfvmax',basis% p_rfvmax &
         &,1_t_int,basis% nRvmx&
         &,1_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis% p_rtfcmax',basis% p_rtfcmax &
         &,0_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('basis% p_rtfvmax',basis% p_rtfvmax &
         &,0_t_int,basis% ncomp&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    !!
    Return
    !!
  End Subroutine allocate_basis
  !!***********************************************************************
  !! make core occupation, this is fragil in the moment
  ! {{{ 

  Subroutine make_core_occupation(structure,potential&
       &,basis,mspin,czval,coreoccu,relativistic)
    !!
    Implicit None 
    !!IO
    Type(T_Structure),Intent(in)  :: structure
    Type(T_Basis),Intent(inout)   :: basis
    Type(T_Potential),Intent(in)  :: potential
    Type(T_Coreoccu),Intent(in)   :: coreoccu
    Integer(t_int),Intent(in)     :: relativistic
    Integer(t_int),Intent(in)      :: mspin
    Real(t_real) ,Intent(out)     :: czval
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_core_occupation"    
    Logical(t_log) :: rel,relmag
    Integer(t_int) :: isa,issa,ic,l,m,ms,i,n,ic0,icr,kap,j,deg
    Integer(t_int) :: issaIO,m0
    Real(t_real)  :: w(2),wo,con,spinfac!,ncc
    Real(t_real),Pointer :: p_IOcoreoccu(:,:,:)
    Type(T_Basdef),Pointer :: bd
    Integer(t_int) :: nspin
    !!
    Nullify(p_IOcoreoccu)
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    relmag=(rel .And. mspin == 2_t_int)
    nspin=mspin
    If(rel) nspin=1
    spinfac=2.0_t_real/mspin
    
    !! The Open core treatment will not really work for
    !! instance for 4f states, in general they are not bound by the
    !! spherically averaged crystal potential (atom/orbital calculation)
    !!
    !!
    !!
    !! only up to f-states may be cores states 
    !!         -> hard dimensions 7 ( non relativistic) and 8 (relativistic)
    !!
    Call palloc('basis% p_corecharge',basis% p_corecharge &
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('p_IOcoreoccu',p_IOcoreoccu &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% IOnsort&
         &,1_t_int,mspin&
         &,nmodule,nsubfunc)

    Call palloc('basis% p_Rcoreoccu',basis% p_Rcoreoccu &
         &,1_t_int,basis% nRcmx&
         &,1_t_int,structure% nsort&
         &,1_t_int,mspin&
         &,nmodule,nsubfunc)


    p_IOcoreoccu(:,:,:)=0.0_t_real
    basis% p_Rcoreoccu(:,:,:)=0.0_t_real
    !! 
    Do issa=1_t_int,structure % nsort
       issaIO=structure% p_isrtTOIOisrt(issa)
       
       bd => basis% p_bdef(issaIO)

       !! the true core orbitals get occupied the added ones not
       Do ic=1,bd% nchi
          l=bd% p_lc(ic)
          Do ms=1_t_int, mspin
             p_IOcoreoccu(ic,issaIO,ms)=2*l+1
          Enddo
       Enddo

!!$       !! 4f semi core occupation
!!$       Do ic=1,bd% nchi
!!$          l=bd% p_lc(ic)
!!$          nc=bd% p_nnc(ic)
!!$          If(l== 3_t_int .And. nc ==  4_t_int) Then
!!$             ! spherically averaged 4f-occupation
!!$             occ=Max(Min((potential % p_natom(issa)-57.0_t_real)&
!!$                  &,14.0_t_real),0.0_t_real)
!!$             If(mspin == 1)Then
!!$                p_IOcoreoccu(ic,issaIO,1)=occ/2.0_t_real
!!$             Else
!!$                p_IOcoreoccu(ic,issaIO,1)=Min(occ,7.0_t_real)
!!$                p_IOcoreoccu(ic,issaIO,2)=Max(occ-7.0_t_real,0.0_t_real)
!!$                
!!$             Endif
!!$          Endif
!!$       Enddo                   ! ic
       
    Enddo                     ! issa

    
    ! calculate core occupations
    countloop: Do i=1_t_int,coreoccu% count

       issaIO=coreoccu% p_sort(i)
       If(issaIO <1_t_int .Or. issaIO  > structure % IOnsort )Then
          Write(errbuf,"('''input sort index (',i5,') out of range!''')") &
               &issaIO
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg=Trim(errbuf))
       End If
       
       Read(coreoccu% p_state(i)(1_t_int:1_t_int),"(i1)") n
       l=lcharqn(coreoccu% p_state(i)(2_t_int:2_t_int))

       If(l>3)Then
          Write(errbuf,636)structure % p_IOelements(issaIO),issaIO,n&
               &,lqnchar(l)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
636    Format('''core occupation: input atom ',a1,' sort(',i3&
            &,'): orbital ',i1,a1&
            &,' not allowed as core!''') 

       bd => basis% p_bdef(issaIO)
          
       !! get index of core state, to be modified
       ic0=-1_t_int
       Do ic=1,bd% nchi
          If( (bd% p_lc(ic) == l) .And. (bd% p_nnc(ic) == n) )Then
             ic0=ic
             Exit
          Endif
       Enddo
       !!
       If(ic0<0_t_int)Then
          Write(errbuf,"('''state (',i2,a,') not defined for sort ',i5&
               &,' (',a2,')!'',/,'' will be ignored''')") &
               &n,lqnchar(l),issaIO &
               &,structure % p_IOelements(issaIO)
          Call errmsg(nmodule,nsubfunc,E_warning,msg=Trim(errbuf))
       Else
          !! make spherical average for now
          w(1)=0.0_t_real
          w(2)=0.0_t_real
          deg=2*l+1
          m0=(7-1)/2-l ! 7 because coreoccu% p_spinup is allocated for max f states
          !! ignore the invalid m in input
          w(1)=Sum(coreoccu% p_spinup(m0+1:m0+deg,i))
          w(2)=Sum(coreoccu% p_spindn(m0+1:m0+deg,i))
          If(mspin == 1_t_int) Then
             w(1)=Sum(w)*0.5_t_real
             w(2)=w(1)
          End If
          If(w(1)>deg+1.e-8_t_real )Then
             Write(errbuf,"('''state (',i2,a,') input sort ',i5,' (',a2&
                  &,') : occupation of spin up (',f10.5,') to large''')") &
                  &n,lqnchar(l),issaIO,structure % p_IOelements(issaIO)&
                  &,w(1)
             
             Call errmsg(nmodule,nsubfunc,E_fatalerror &
                  &,msg=Trim(errbuf))
             
          End If
          If(w(2)>deg+1.e-8_t_real )Then
             Write(errbuf,"('''state (',i2,a,') sort ',i5,' (',a2&
                  &,') : occupation of spin down (',f10.5,') to large''')") &
                  &n,lqnchar(l),issaIO,structure % p_IOelements(issaIO)&
                  &,w(2)
             
             Call errmsg(nmodule,nsubfunc,E_fatalerror &
                  &,msg=Trim(errbuf))
             
          End If
          
          
          Do ms=1_t_int,mspin
             p_IOcoreoccu(ic0,issaIO,ms)=w(ms)
          Enddo
       End If
       
    Enddo countloop
    !!
    !! organize relativistic occupation number arrays
    !!
    Do issa=1_t_int,structure% nsort
       !! write IOarray into normal array
       issaIO=structure% p_isrtTOIOisrt(issa)
       Do icr=1,basis% p_nRc(issa)
          !!
          l=basis% p_lRc(icr,issa)
          !! nonrelat state index
          ic=basis% p_icrTOic(icr,issa)
          !!
          Do ms=1,mspin
             basis% p_Rcoreoccu(icr,issa,ms)=p_IOcoreoccu(ic,issaIO,ms)
          Enddo
       Enddo
    Enddo
    !!
    !!     print core occupation numbers
    !!     
    !!
    If(rel.And..Not.relmag)Then
       Write(outfile,99)
       Do issa=1_t_int,structure % nsort
          Write(outfile,198)issa,structure % p_elements(issa)
          Do icr=1_t_int,basis % p_nRchi(issa)
             !! kappa qn
             kap=basis% p_kapRc(icr,issa)
             !! angular momentum
             l=basis% p_lRc(icr,issa)
             !! j momentum
             j=2*Abs(kap)-1
             !!
             deg=basis% p_degRc(icr,issa)
             !!
             w(1)=basis% p_Rcoreoccu(icr,issa,1)/Real(2*l+1)*deg
             wo=w(1)/deg
             If(wo-Int(wo,t_int) > 1.e-6_t_real) Write(outfile,13)
             !!
             !!
             If(w(1) /= j+1)Then
                Write(errbuf,567)
                Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
             Endif
             !!
             Write(outfile,12,advance='no')&
                  & icr,basis % p_nnRc(icr,issa),lqnchar(l),j,w(1)
             
             
             Do m=1,(7+1-deg)/2
                Write(outfile,14,advance='no')
             Enddo
             Do m=1_t_int,deg
                Write(outfile,15,advance='no')w(1)/deg
             Enddo                 ! m
             Write(outfile,1)
             
          Enddo                   ! icr
       Enddo                     ! issa
       Write(outfile,20)
       
    Else
       !!non-relativistic
       !!
       Write(outfile,99)
       Do issa=1_t_int,structure % nsort

       
          Write(outfile,98)issa,structure % p_elements(issa)
          Do icr=1,basis% p_nRchi(issa)
             If(basis% p_kapRc(icr,issa)>0) Cycle
             l=basis% p_lRc(icr,issa)
             deg=2*l+1
             w(1:2)=0.0_t_real
             Do ms=1_t_int,mspin
                w(ms)=basis% p_Rcoreoccu(icr,issa,ms)
                wo=w(ms)/deg
                If(wo-Int(wo,t_int) > 1.e-6_t_real) Write(outfile,13)
             Enddo                 ! ms
             !!
             !!
             Do ms=1_t_int,mspin
                If(w(ms) /= deg)Then
                   Write(errbuf,567)
                   Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
                Endif
             Enddo ! ms
             Write(outfile,22,advance='no')&
                  &icr,basis% p_nnRc(icr,issa),lqnchar(l),w(1_t_int)
             ms=1_t_int
             Do m=-3_t_int,-l-1_t_int
                Write(outfile,14,advance='no')
             Enddo
             Do m=1_t_int,deg
                wo=basis% p_Rcoreoccu(icr,issa,ms)/deg
                Write(outfile,15,advance='no')wo
             Enddo                 ! m
             Write(outfile,1)
             ms=mspin
             Write(outfile,23,advance='no')w(ms)
             Do m=-3_t_int,-l-1_t_int
                Write(outfile,14,advance='no')
             Enddo
             Do m=1_t_int,deg
                wo=basis% p_Rcoreoccu(icr,issa,ms)/deg
                Write(outfile,15,advance='no')wo
             Enddo                 ! m
             Write(outfile,1)
          Enddo                   ! icr
       Enddo                     ! issa
       Write(outfile,20)
    Endif

    !!
    czval=0.0_t_real-potential% molecule_ionicity
    Do isa=1_t_int,structure % nsite
       issa=structure % p_isrt(isa)

       con=structure% p_conao(issa)
       basis% p_corecharge(issa)=0.0_t_real

       Do icr=1,basis% p_nRchi(issa)
          If(basis% p_kapRc(icr,issa)>0) Cycle
          Do ms=1,mspin
             basis% p_corecharge(issa)=basis% p_corecharge(issa)&
                  &+basis% p_Rcoreoccu(icr,issa,ms)*spinfac
          Enddo
       Enddo

       czval=czval+(potential % p_natom(issa)-basis% p_corecharge(issa))*con
    Enddo
    If(czval < 0.0_t_real)Then
       Write(errbuf,"('''occupation of valence bands : ',f10.5&
            &,' is negative '',/,'' check core occupation (f-states) !''')") &
            &czval
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
    Endif
    
    Write(outfile,50) czval

    !!
    !!
    Call pfree(p_IOcoreoccu)
    !!
    !!
    Return
    !!
99  Format(/,72('-')/,'|',t5 &
         & ,'OCCUPATION NUMBERS OF CORE STATES FOR CRYSTAL CALCULATION',t72 &
         & ,'|'/,72('-'))
98  Format('Sort: ',i3,' Element: ',a2,/72('-')/ &
         & ,' State  Spin   Occu' &
         & ,'    M =  -3     -2     -1      0' &
         & ,'      1      2      3')
198 Format('Sort: ',i3,' Element: ',a2,/72('-')/ &
         & ,' State   J     Occu' &
         & ,'   J_z=  -7/2   -5/2   -3/2   -1/2   1/2    3/2    5/2    7/2')
    !!
1   Format() 
12  Format(i3,':',i1,a1,'  ',i1,'/2  ',f7.2,'   :')
13  Format('noninteger occupation detected')
14  Format(7(' '))
15  Format(f7.2) 
20  Format(72('-'))
22  Format(i3,':',i1,a1,'  up   ',f7.2,'   :')
23  Format('        down ',f7.2,'   :')
50  Format(/,'Number of occupied valence bands (averaged): ',f12.5,/,/)
567 Format("'open core shell detected !',/,' Please check input!'") 

    !!
    !!
  End Subroutine make_core_occupation
  !!***********************************************************************
  Integer(t_int) Function coreMultIndex(icr,issa,basis)
    !!
    Implicit None 
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: icr,issa
    !!LOCAL
    Integer(t_int) :: jcr
    !!
    !!
    coreMultIndex=0
    !!
    Do jcr=1,icr
       If( (basis% p_nnRc(jcr,issa)== basis% p_nnRc(icr,issa)) .And.&
            & (basis% p_lRc(jcr,issa)== basis% p_lRc(icr,issa)) .And. &
            & (basis% p_kapRc(jcr,issa) == basis% p_kapRc(icr,issa)))Then
          coreMultIndex=coreMultIndex+1
       Endif
    Enddo
    !!
    Return
    !!
  End Function coreMultIndex  
  !!***********************************************************************
  !! for VCA: try to merge basdefs as good as possible 
  !!          this is we take the basdef0 and interpolate with basdef,
  !!          if the same orbital is in both basdefs
  Subroutine merge_basdefs(bdef0,bdef,ion)
    !!
    Implicit None
    !!IO
    Type(T_Basdef),Intent(inout) :: bdef0,bdef
    Real(t_real),Intent(in)      :: ion
    !!LOCAL
!    Character(*),Parameter::nsubfunc="merge_basdefs"
    Integer(t_int) :: ic,nmain,l,jc,iv,jv
    Real(t_real)   :: Qion,Spol,Pex
    Character(5000) :: basdef
    !!
    !Call stringify_basdef(bdef0,basdef)
    !Write(outfile,"('bas0=[',a,']')") Trim(basdef)
    !!
    !Call stringify_basdef(bdef,basdef)
    !Write(outfile,"('bas =[',a,']')") Trim(basdef)
    !!
    !!
    bdef0% pexdef=bdef0% pexdef-(bdef0% pexdef-bdef% pexdef)*ion
    bdef0% exn2def=bdef0% exn2def-(bdef0% exn2def-bdef% exn2def)*ion
    !!
    Do ic=1,bdef0% nc
       nmain=bdef0% p_nnc(ic)
       l=bdef0% p_lc(ic)
       Qion=bdef0% p_Qionc(ic)
       Spol=bdef0% p_Spolc(ic)
       !Print "(i1,a1,' : ',f30.15)",nmain,lqnchar(l),Qion
       Do jc=1,bdef% nc
          If( nmain == bdef% p_nnc(jc)  .And. l == bdef% p_lc(jc)  ) Then
             !! same orbital
             bdef0% p_Qionc(ic)=Qion-(Qion-bdef% p_Qionc(jc))*ion
             bdef0% p_Spolc(ic)=Spol-(Spol-bdef% p_Spolc(jc))*ion
          Endif
       Enddo
    Enddo
    !!
    Do iv=1,bdef0% nv
       nmain=bdef0% p_nnv(iv)
       l=bdef0% p_lv(iv)
       Qion=bdef0% p_Qionv(iv)
       Spol=bdef0% p_Spolv(iv)
       Pex=bdef0% p_Pexv(iv)
       !Print "(i1,a1,' : ',f30.15)",nmain,lqnchar(l),Qion
       Do jv=1,bdef% nv
          If( nmain == bdef% p_nnv(jv)  .And. l == bdef% p_lv(jv)  ) Then
             !! same orbital
             bdef0% p_Qionv(iv)=Qion-(Qion-bdef% p_Qionv(jv))*ion
             bdef0% p_Spolv(iv)=Spol-(Spol-bdef% p_Spolv(jv))*ion
             bdef0% p_Pexv(iv)=Pex-(Pex-bdef% p_Pexv(jv))*ion
          Endif
       Enddo
    Enddo
    

    !!
    !!
    Call stringify_basdef(bdef0,basdef)
    !!
    !Write(outfile,"('vcabas=[',a,']')") Trim(basdef)
    !!
    Return
    !!
  End Subroutine merge_basdefs

  !!***********************************************************************
  
End Module orbitalbasis
