Module overlapdensity
  !!
  Use precisiondefs
  Use constants
  Use errors
  Use memory
  Use clock
  Use matserv
  Use tmesh
  Use mesh
  Use integrate1d
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="OVERLAPDENSITY"  
  !!
  Public :: calc_overlapdensity
  !!***********************************************************************
Contains 
  !!***********************************************************************
  !!  
  !! makes overlap density from core & valence functions with 1D-shaping
  !! TODO: shift isa loop  into rover, better spin loop
  Subroutine calc_overlapdensity(mspin,ms,nsite,nsort,isa,issa,isrt,conao&
       &,npairsmax,npairs,pairs,npshell,ninpshell,maxinpshell&
       &,pairabs&
       &,pmesh,nrp,basmeshs,nrbmax,imesh&
       &,ncomp,nvsmx,ncsmx&
       &,lmx,nvm,nv,lv,ncm,nc,lc,fv,fc,fgam&
       &,rtfcmax,rtfvmax,rfvmax,rfcmax&
       &,max_L,mlmp,nlmp,lmp,lcd&
       &,onv,onc&
       &,ovldns_neta,degree)
    !!
    Implicit None
    !!IO
    Type(T_Mesh), Intent(in)     :: pmesh
    Type(T_Mesh), Pointer        :: basmeshs(:)
    Type(T_Mesh), Intent(in)     :: imesh
    Integer(t_int),Intent(in)    :: mspin,nsite,nsort,isrt(nsite)
    Integer(t_int),Intent(in)    :: isa,issa,ms
    Integer(t_int),Intent(in)    :: npairsmax,npairs(nsite,nsite)
    Integer(t_int),Intent(in)    :: npshell(nsite,nsite)
    Integer(t_int),Intent(in)    :: ninpshell(:,:,:),maxinpshell
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite)
    Integer(t_int),Intent(in)    :: nrp,nrbmax
    Integer(t_int),Intent(in)    :: nvsmx,ncsmx,ncomp
    Integer(t_int),Intent(in)    :: nvm,nv(nsort),lv(nvm,nsort),lmx
    Integer(t_int),Intent(in)    :: ncm,nc(nsort),lc(ncm,nsort)
    Real(t_real),Intent(in)      :: fgam(nsort)
    Real(t_real),Intent(in)      :: fc(nrbmax,ncm,ncomp,nsort)
    Real(t_real),Intent(in)      :: fv(nrbmax,nvm,ncomp,nsort)
    Real(t_real),Intent(in)      :: conao(nsort)
    Integer(t_int),Intent(in)    :: ovldns_neta,degree
    Real(t_real),Intent(in)      :: rtfvmax(0:ncomp,nsort)
    Real(t_real),Intent(in)      :: rtfcmax(0:ncomp,nsort)
    Real(t_real),Intent(in)      :: rfvmax(nvm,ncomp,nsort)
    Real(t_real),Intent(in)      :: rfcmax(ncm,ncomp,nsort)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Real(t_real),Intent(in)      :: pairabs(:,:,:)
    Real(t_real),Intent(inout)   :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: onc(ncsmx,nvsmx,nsite,nsite,mspin,Max(1_t_int,npairsmax))
    Real(t_real),Intent(in)      :: onv(nvsmx,nvsmx,nsite,nsite,mspin,Max(1_t_int,npairsmax))
    !!LOCAL
    Character(*),Parameter::nsubfunc="overlapdensity"    
    Logical(t_log)       :: nocore,nocore0
    Integer(t_int) :: isb,j,ixa,lmxa,i,ia,la,ma,ind,i0v,i0c,issb,lmxb,mmll,mxcv
    Integer(t_int) :: dega,degb
    Integer(t_int) :: ib,lb,mb,iv,ic,nshell,n,ish,ne,nf,i1,ir,l,n1,jl0,n2
    Integer(t_int) :: m1,jl,ind2,il0,m2,il,ik,ixb,m0,i0,m,findexa,findexb
    Real(t_real)   :: rmax,ra,f,cb,rbig,s,fpa,fpb,fq,rffmax
    Real(t_real)   :: vv(-lmx:lmx,-lmx:lmx),va(-2*lmx:2*lmx)
    Real(t_real)   :: eu(3),pp,cp,sp,rb,wwb,za,zb,yp,arg
    Real(t_real)   :: aa,ff,wwbf,wwbfcv,wwbfvc,texp
    Integer(t_int) :: inva(nvsmx),mnva(nvsmx)
    Integer(t_int) :: invb(nvsmx),mnvb(nvsmx)
    Integer(t_int) :: inca(ncsmx),mnca(ncsmx)
    Integer(t_int) :: incb(ncsmx),mncb(ncsmx)
    Real(t_real)  :: fac(0:2_t_int*(Max(lmx,max_L)+1_t_int))
    Real(t_real)  :: flpv (nvsmx),flpc (ncsmx)
    Real(t_real)  :: flv  (nvsmx),flc  (ncsmx)
    Integer(t_int) :: ndif(nsite,Max(1_t_int,npairsmax))
    Real(t_real)  :: vp((max_L+1_t_int)**2_t_int,Max(1_t_int,maxinpshell))
    Real(t_real)  :: hza((12+1)*(12+2)/2) ! maximum L=12 implemented
    Real(t_real)  :: hzb((12+1)*(12+2)/2) ! maximum L=12 implemented
    Real(t_real)  :: ycc((max_L+1_t_int)**2_t_int)
    Real(t_real)  :: fcoefa(0:degree)
    Real(t_real)  :: fcoefb(0:degree)
    Type(T_Mesh), Pointer :: bmesha,bmeshb
    !!
    Real(t_real),Pointer :: flpv0(:,:)
    Real(t_real),Pointer :: flpc0(:,:)
    Real(t_real),Pointer :: onnvv(:,:,:)
    Real(t_real),Pointer :: onncv(:,:,:)
    Real(t_real),Pointer :: onnvc(:,:,:)
    Real(t_real),Pointer :: wd(:,:)
    Real(t_real),Pointer :: wu(:,:)
    Real(t_real),Pointer :: oh(:,:)
    Real(t_real),Pointer :: oh1(:,:)

    !! pointer, do not free !!!
    Real(t_real),Pointer   :: p_ab(:),p_wb(:)
    !Real(t_real),Parameter :: zero=0.0_t_real
    !!***********************************************************************
    !!     
    Nullify(flpv0)
    Nullify(flpc0)
    Nullify(onnvv)
    Nullify(onncv)
    Nullify(onnvc)
    Nullify(wd)
    Nullify(wu)
    Nullify(oh)
    Nullify(oh1)
    !!
    !Return
    !!
    Call palloc('flpv0',flpv0,1_t_int,nvsmx,1_t_int,imesh% nr,nmodule,nsubfunc)
    Call palloc('flpc0',flpc0,1_t_int,ncsmx,1_t_int,imesh% nr,nmodule,nsubfunc)
    Call palloc('onnvv',onnvv,1_t_int,nvsmx,1_t_int,nvsmx&
         &,1_t_int,Max(1_t_int,maxinpshell),nmodule,nsubfunc)
    Call palloc('onncv',onncv,1_t_int,ncsmx,1_t_int,nvsmx&
         &,1_t_int,Max(1_t_int,maxinpshell),nmodule,nsubfunc)
    Call palloc('onnvc',onnvc,1_t_int,nvsmx,1_t_int,ncsmx&
         &,1_t_int,Max(1_t_int,maxinpshell),nmodule,nsubfunc)
    Call palloc('wd',wd,1_t_int,imesh% nr,1_t_int,mlmp,nmodule,nsubfunc)
    !!
    mmll=Max(lmx,max_L)+1_t_int
    !!
    Call palloc('wu',wu,1_t_int,mmll*(4*mmll**2-1)/3&
         &,1_t_int,Max(1_t_int,maxinpshell),nmodule,nsubfunc)
    !!
    mxcv=Max(nvsmx,ncsmx)
    !!
    Call palloc('oh' ,oh ,1_t_int,mxcv,1_t_int,mxcv,nmodule,nsubfunc)
    Call palloc('oh1',oh1,1_t_int,mxcv,1_t_int,mxcv,nmodule,nsubfunc)
    !!



  

    !!
    !! basic preparations
    Call vecdifdens(nsite,isa,ndif,npairs,npairsmax,pairs)
    !!
    !!
    Call make_factorials(fac,2*mmll)
    !!
    !!
    bmesha => basmeshs(issa)
    !!
    rmax=imesh% rmax
    !!     radial core+valence functions at P times ra     
    lmxa=max_L
    Do ixa=1_t_int,imesh% nr
       ra=imesh% p_rmesh(ixa)
       
       findexa=mintpl_coefs(degree,bmesha,ra,fcoefa)
       
       i=0_t_int
       Do ia=1_t_int,nv(issa)
          la=lv(ia,issa)
          lmxa=Max(lmxa,la)
          f=0.0_t_real
          If(ra <= rfvmax(ia,1,issa)) Then
             
             f=Dot_product(fv(findexa:findexa+degree,ia,1,issa)&
                  &,fcoefa(0:degree))
          Endif

          Do ma=-la,la
             i=i+1_t_int
             flpv0(i,ixa)=f
             ind=la*(la+1_t_int)/2_t_int+1_t_int+Abs(ma)
             inva(i)=ind
             mnva(i)=ma
          Enddo                 ! ma
       Enddo                   ! ia
       i=0_t_int
       Do ia=1_t_int,nc(issa)
          la=lc(ia,issa)
          lmxa=Max(lmxa,la)
          f=0.0_t_real
          If(ra <= rfcmax(ia,1,issa))Then
             f=Dot_product(fc(findexa:findexa+degree,ia,1,issa)&
                  &,fcoefa(0:degree))
          Endif
          Do ma=-la,la
             i=i+1_t_int
             flpc0(i,ixa)=f
             ind=la*(la+1_t_int)/2_t_int+1_t_int+Abs(ma)
             inca(i)=ind
             mnca(i)=ma
          Enddo                 ! ma
       Enddo                   ! ia
    Enddo                     ! ixa
    !!     
    i0v=Sum(2*lv(1:nv(issa),issa)+1)
    i0c=Sum(2*lc(1:nc(issa),issa)+1)
    !!
    !!
    wd(:,:)=0.0_t_real
    !!
    Call get_gauss(ovldns_neta,p_ab,p_wb)
    !!
    isbloop: Do isb=1_t_int,nsite
       issb=isrt(isb)
       !!
       bmeshb => basmeshs(issb)
       !!
       rffmax=Max(rtfcmax(0,issb),rtfvmax(0,issb))
       !!
       cb=conao(issb)
       i=0_t_int
       lmxb=0_t_int
       Do ib=1_t_int,nv(issb)
          lb=lv(ib,issb)
          lmxb=Max(lmxb,lb)
          Do mb=-lb,lb
             i=i+1_t_int
             ind=lb*(lb+1_t_int)/2_t_int+1_t_int+Abs(mb)
             invb(i)=ind
             mnvb(i)=mb
          Enddo                 ! mb
       Enddo                   ! ib
       iv=i
       i=0_t_int
       Do ib=1_t_int,nc(issb)
          lb=lc(ib,issb)
          lmxb=Max(lmxb,lb)
          Do mb=-lb,lb
             i=i+1_t_int
             ind=lb*(lb+1_t_int)/2_t_int+1_t_int+Abs(mb)
             incb(i)=ind
             mncb(i)=mb
          Enddo                 ! mb
       Enddo                   ! ib
       ic=i
       !!        
       nshell=npshell(isb,isa)
       !!       For all shells
       n=0_t_int
       ishloop: Do ish=1_t_int,nshell
          nf=n+1_t_int
          ne=n+ninpshell(ish,isb,isa)
          n=ne
          rbig=pairabs(ish,isb,isa)
          s=(rmax+bmeshb% rmax)/rbig
          If(s < 1.0_t_real)  Cycle ishloop
          !! rotation matrices            
          !Call watch('OVDENS-occu',cpu_start)
          i1=0_t_int
          inshlloop: Do ir=nf,ne
             Call reuler(pairs(1:3,ir,isb,isa),eu)
             i1=i1+1_t_int
             Do l=0_t_int,Max(max_L,lmx)
                n1=l*(l*l*4_t_int-1_t_int)/3_t_int
                Call ytr(eu,0_t_int,fac,l,wu(n1+1_t_int:,i1),.False.)
             Enddo
             oh(:,:) =0.0_t_real
             oh1(:,:)=0.0_t_real
             j=0_t_int
             jl0=0_t_int
             Do ib=1_t_int,nv(issb)
                lb=lv(ib,issb)
                degb=2*lb+1
                n2=lb*(lb*lb*4_t_int-1_t_int)/3_t_int
                Do m1=1,degb
                   j=j+1_t_int
                   jl=jl0
                   Do mb=1,degb
                      jl=jl+1_t_int
                      ind2=degb*(mb-1_t_int)+m1+n2
                      i=0_t_int
                      il0=0_t_int
                      Do ia=1_t_int,nv(issa)
                         la=lv(ia,issa)
                         dega=2*la+1
                         n1=la*(la*la*4_t_int-1_t_int)/3_t_int
                         Do m2=1,dega
                            i=i+1_t_int
                            il=il0
                            Do ma=1,dega
                               il=il+1_t_int
                               ind=dega*(ma-1_t_int)+m2+n1
                               oh(i,j)=oh(i,j)+wu(ind,i1)&
                                    & *onv(il,jl,isa,isb,ms,ir)*wu(ind2,i1)
                            Enddo     ! ma
                         Enddo       ! m2
                         il0=il0+dega
                      Enddo         ! ia
                      i=0_t_int
                      il0=0_t_int
                      Do ia=1_t_int,nc(issa)
                         la=lc(ia,issa)
                         dega=2*la+1
                         n1=la*(la*la*4_t_int-1_t_int)/3_t_int
                         Do m2=1,dega
                            i=i+1_t_int
                            il=il0
                            Do ma=1,dega
                               il=il+1_t_int
                               ind=dega*(ma-1_t_int)+m2+n1
                               oh1(i,j)=oh1(i,j)+wu(ind,i1)&
                                    & *onc(il,jl,isa,isb,ms,ir)*wu(ind2,i1)
                            Enddo     ! ma
                         Enddo       ! m2
                         il0=il0+dega
                      Enddo         ! ia
                   Enddo           ! mb
                Enddo             ! m1             
                jl0=jl0+degb
             Enddo               ! ib             
             onnvv(1:i0v,1:iv,i1)=oh (1:i0v,1:iv)
             onncv(1:i0c,1:iv,i1)=oh1(1:i0c,1:iv)

             ik=ndif(isb,ir)
             oh(:,:)=0.0_t_real
             j=0_t_int
             jl0=0_t_int
             Do ib=1_t_int,nc(issb)
                lb=lc(ib,issb)
                degb=2*lb+1
                n2=lb*(lb*lb*4_t_int-1_t_int)/3_t_int
                Do m1=1,degb
                   j=j+1
                   jl=jl0
                   Do mb=1,degb
                      jl=jl+1_t_int
                      ind2=degb*(mb-1_t_int)+m1+n2
                      i=0_t_int
                      il0=0_t_int
                      Do ia=1_t_int,nv(issa)
                         la=lv(ia,issa)
                         dega=2*la+1
                         n1=la*(la*la*4_t_int-1_t_int)/3_t_int
                         Do m2=1,dega
                            i=i+1_t_int
                            il=il0
                            Do ma=1,dega
                               il=il+1_t_int
                               ind=dega*(ma-1_t_int)+m2+n1
                               oh(j,i)=oh(j,i)+wu(ind,i1)&
                                    & *onc(jl,il,isb,isa,ms,ik)*wu(ind2,i1)
                            Enddo     ! ma
                         Enddo       ! m2
                         il0=il0+dega
                      Enddo         ! ia
                   Enddo           ! mb
                Enddo             ! m1             
                jl0=jl0+degb
             Enddo               ! ib             
             onnvc(1:i0v,1:ic,i1)=Transpose(oh(1:ic,1:i0v))
             !Do i=1,i0v
             !   Do j=1,ic
             !      onnvc(i,j,i1)=oh(j,i)
             !   Enddo
             !Enddo
          Enddo inshlloop                 ! ir
          !Call watch('OVDENS-occu',cpu_stop)
          !!
          !! radial loop      ra
          !!
          ixaloop: Do ixa=1_t_int,imesh% nr
             ra=imesh% p_rmesh(ixa)
             !! radial core+valence functions at P times ra
             nocore0=ra > rtfcmax(0,issa)
             !! eta mesh
             vp(:,:)=0.0_t_real
             etaloop: Do ixb=1_t_int,ovldns_neta
                pp=(p_ab(ixb)+1.0_t_real)*pi2
                cp=Cos(pp)
                sp=Sin(pp)
                rb=Sqrt(ra**2_t_int+rbig**2_t_int-2.0_t_real*rbig*ra*cp)
                wwb=sp*p_wb(ixb)*pi2
                If(rb > rffmax) Cycle etaloop
                za=ra*cp
                zb=(za-rbig)/rb
                yp=ra*sp/rb
                arg=za/rbig
                If(arg > 1.0_t_real) Cycle etaloop
                Call yhzy(hza,cp,sp,lmxa,.True.)
                Call yhzy(hzb,zb,yp,lmxb,.True.)
                yp=ra*sp

                fpa=dprofile(ra,0.0_t_real,rbig/2.0_t_real)
                fpb=dprofile(rb,0.0_t_real,rbig/2.0_t_real)
                fq=1.0_t_real-fpa-fpb

                ff=1.0_t_real
                If(arg > 0.0_t_real) ff=fku(arg)
                wwbf=wwb*(fpa+fq*ff)
                !wwbf=wwb*ff
                ff=1.0_t_real
                If(arg > 0.0_t_real) ff=fku(arg**3)
                wwbfcv=wwb*ff
                ff=1.0_t_real
                If(arg > 0.0_t_real) ff=fku(1.0_t_real-(1.0_t_real-arg)**3)
                wwbfvc=wwb*ff
                nocore=rb > rtfcmax(0,issb)

                texp=rb**(fgam(issb)-1)
                findexb=mintpl_coefs(degree,bmeshb,rb,fcoefb)

                !! radial core+valence functions at isb
                i=0_t_int
                Do ib=1_t_int,nv(issb)
                   lb=lv(ib,issb)
                   f=0.0_t_real
                   If(rb <= rfvmax(ib,1,issb)) Then

                      f=Dot_product(fv(findexb:findexb+degree,ib,1,issb)&
                           &,fcoefb(0:degree))*texp

                   Endif
                   Do mb=1_t_int,lb+lb+1_t_int
                      i=i+1_t_int
                      flv(i)=f*hzb(invb(i))*cb
                   Enddo           ! mb
                Enddo             ! ib
                If(.Not. nocore)Then 
                   i=0_t_int
                   Do ib=1_t_int,nc(issb)
                      lb=lc(ib,issb)
                      f=0.0_t_real
                      If(rb <= rfcmax(ib,1,issb)) Then
                         f=Dot_product(fc(findexb:findexb+degree,ib,1,issb)&
                              &,fcoefb(0:degree))*texp
                      Endif
                      Do mb=1_t_int,lb+lb+1_t_int
                         i=i+1_t_int
                         flc(i)=f*hzb(incb(i))*cb
                      Enddo           ! mb
                   Enddo             ! ib
                Endif

                !!
                !! radial functions at A tiems Hza
                !!
                Do i=1_t_int,i0v
                   flpv(i)=flpv0(i,ixa)*hza(inva(i))
                Enddo             ! i
                If(.Not. nocore0)Then
                   Do i=1_t_int,i0c
                      flpc(i)=flpc0(i,ixa)*hza(inca(i))
                   Enddo             ! ib
                End If
                Do i=1_t_int,(max_L+1_t_int)**2_t_int
                   ycc(i)=hza(mPlm(i))*2.0_t_real
                Enddo
                !! For all vectors in Shell ish
                i1=0_t_int
                irloop: Do ir=nf,ne
                   i1=i1+1_t_int
                   !!               
                   vv(:,:)=0.0_t_real
                   Do j=1_t_int,iv
                      mb=mnvb(j)
                      aa=flv(j)
                      Do i=1_t_int,i0v
                         ma=mnva(i)
                         vv(ma,mb)=vv(ma,mb)+flpv(i)*onnvv(i,j,i1)*aa*wwbf
                      Enddo         ! i
                      If(.Not.nocore0) Then
                         Do i=1_t_int,i0c
                            ma=mnca(i)
                            vv(ma,mb)=vv(ma,mb)+flpc(i)*onncv(i,j,i1)*aa*wwbfcv
                         Enddo       ! i
                      Endif
                   Enddo           ! j
                   If(.Not.nocore) Then
                      Do j=1_t_int,ic
                         mb=mncb(j)
                         aa=flc(j)*wwbfvc
                         Do i=1_t_int,i0v
                            ma=mnva(i)
                            vv(ma,mb)=vv(ma,mb)+flpv(i)*onnvc(i,j,i1)*aa
                         Enddo         ! i
                      Enddo           ! j
                   Endif
                   !! times (MMM)
                   va(:)=0.0_t_real
                   Call dselect(lmx,vv,va)
                   Do l=0,max_L
                      m0=Min(2*lmx,l)
                      i0=l*(l+1)+1
                      i=i0-m0
                      Do m=-m0,m0
                         vp(i,i1)=vp(i,i1)+va(m)*ycc(i)
                         i=i+1
                      Enddo
                   Enddo
                Enddo irloop             ! ir 
             Enddo etaloop               ! ieta

             !!           final rotation
             i1=0_t_int
             Do ir=nf,ne
                i1=i1+1_t_int
                Do lb=0_t_int,max_L
                   n1=lb*(lb*lb*4_t_int-1_t_int)/3_t_int
                   n2=lb*lb+1_t_int
                   Call mmult(ycc(n2),wu(n1+1_t_int:,i1),vp(n2,i1),lb+lb+1_t_int)
                Enddo
                Do il=1_t_int,nlmp(isa)
                   jl=lmp(il,isa)
                   wd(ixa,il)=wd(ixa,il)+ycc(jl)
                Enddo
             Enddo               ! ir
             !!End r-mesh
          Enddo ixaloop                 ! ixa
       Enddo ishloop                   ! ish
    Enddo isbloop                     ! isb
    !! interpolation oszilation ovoiding
    !!
    !!
    !! reinterpol
    !!
    Do ir=1_t_int,pmesh% nr
       ra=pmesh% p_rmesh(ir)
       findexa=mintpl_coefs(degree,imesh,ra,fcoefa)
       texp=spow(ra,fgam(issa)+1.0_t_real)
       Do i=1_t_int,nlmp(isa)
          lcd(ir,i,isa,ms)=lcd(ir,i,isa,ms)+&
               &Dot_product(wd(findexa:findexa+degree,i),fcoefa(0:degree))*texp
       Enddo                   ! i
    Enddo                     ! ir
    !!
    Call pfree(oh)
    Call pfree(oh1)
    Call pfree(wu)
    Call pfree(wd)
    Call pfree(onnvv)
    Call pfree(onncv)
    Call pfree(onnvc)
    Call pfree(flpv0)
    Call pfree(flpc0)
    !!
    Return
    !!
  Contains
    Real(t_real) Function fku(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fku=(1.0_t_real-fk(fk(x)))
    End Function fku

    Real(t_real) Function fku3(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fku3=(1.0_t_real-fk(fk(fk(x))))
    End Function fku3

    Real(t_real) Function fk(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fk=1.0_t_real-(Cos(pi*x)+1.0_t_real)*0.5_t_real  
    End Function fk
    !!
  End Subroutine calc_overlapdensity
  !!***********************************************************************
  Subroutine  dselect(lmx,A,C)
    !!
    Use precisiondefs
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: lmx
    Real(t_real),Intent(in)      :: A(-lmx:lmx,-lmx:lmx)
    Real(t_real),Intent(inout)   :: C(-2*lmx:2*lmx)
    !!LOCAL
    Integer(t_int) :: m,ma
    !!
    M=0
    C(M)=C(M)+A(0,0)*0.5_t_real
    Do ma=1,lmx
       C(M)=C(M)+A(ma,ma)+A(-ma,-ma)
    Enddo
    !!
    Do M=1,lmx
       C(M)=C(M)+A(M,0)+A(0,M)
       !!
       C(-M)=C(-M)+A(-M,0)+A(0,-M)
    Enddo
    !!
    Do M=1,2*lmx
       Do ma=Max(1_t_int,M-lmx),Min(M-1,lmx)
          C(M)=C(M)+A(ma,M-ma)-A(-ma,ma-M)
          C(-M)=C(-M)+A(ma,ma-M)+A(-ma,M-ma)
       Enddo
       Do ma=1,lmx-M
          C(M)=C(M)+A(M+ma,ma)+A(-M-ma,-ma)+A(ma,M+ma)+A(-ma,-M-ma)
          C(-M)=C(-M)+A(-M-ma,ma)-A(M+ma,-ma)-A(-ma,M+ma)+A(ma,-M-ma)
       Enddo
    Enddo
    !!
    Return
    !!
  End Subroutine dselect
  !!***********************************************************************
  !!
  Subroutine mmult(va,wu,vb,n)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) ::  n
    Real(t_real),Intent(in)     :: vb(n),wu(n,n)
    Real(t_real),Intent(out)    :: va(n)
    !!LOCAL
    Integer(t_int) ::  i,j
    Real(t_real)  ::  a
    !!
    Do i=1_t_int,n
       a=0.0_t_real
       Do j=1_t_int,n
          a=a+wu(j,i)*vb(j)
       Enddo
       va(i)=a
    Enddo
    !!
  End Subroutine mmult
  !!
  !!***********************************************************************
  Subroutine vecdifdens(nsite,isa,ndif,npairs,npairsmax,pairs)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nsite,isa,npairsmax
    Integer(t_int),Intent(in)    :: npairs(nsite,nsite)
    Real(t_real),Intent(in)     :: pairs(:,:,:,:)
    Integer(t_int),Intent(out)   :: ndif(nsite,Max(1_t_int,npairsmax))
    !!LOCAL
    Character(*),Parameter::nsubfunc="vecdifdens"
    Integer(t_int) ::  i,j,m,isb
    Real(t_real)  ::  rr(3)
    !!
    ndif(:,:)=0_t_int
    Do isb=1_t_int,nsite
       Do i=1_t_int,npairs(isb,isa)
          rr(:)=pairs(:,i,isb,isa)
          j=0_t_int
          mloop: Do m=1_t_int,npairs(isa,isb)
             If(All(Abs(pairs(:,m,isa,isb)+rr) < 1.e-3_t_real)) Then
                j=m
                Exit mloop
             Endif
          Enddo mloop
          If(j == 0_t_int)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Lattice corrupted!'")
          End If
          ndif(isb,i)=j
       Enddo
    Enddo
    Return
  End Subroutine vecdifdens
  !!***********************************************************************
  !! a profile function
  Real(t_real) Function dprofile(r,rmin,rmax)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: r,rmin,rmax
    !!LOCAL
    Real(t_real)   :: x
    !!
    dprofile=0.0_t_real
    If(r>=rmax) Return
    If(r<=rmin)Then
       dprofile=1.0_t_real
       Return
    Endif
    !!
    x=(r-rmin)/(rmax-rmin)
    !! cosine profile
    dprofile=(1.0_t_real-fc(fc(x)))
    !!
  Contains
    Real(t_real) Function  fc(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in) :: x
      !!LOCAL
      fc=(1.0_t_real-Cos(pi*x))*0.5_t_real
    End Function fc
    !!
  End Function dprofile
  !!***********************************************************************

End Module overlapdensity

