!!other todos: FREL bandwise, symmetrization, spin-multiplicity, spin polarized
Module plotdens
  !!
  Use precisiondefs
  Use file_iounits
  Use constants
  Use errors
  Use matserv
  Use clock
  Use tio
  Use tsymmetry
  Use tmesh
  Use mesh
  Use trelativistic
  Use diracdefs
  Use tkohnsham
  Use tloi
  Use tlsdau
  Use topc
  Use lsdau
  Use kohnsham
  Use torbitalbasis
  Use netdensity
  Use string
  Use tbzone
  Use ioaux
  Use tgrid
  Use grid
  Use memory
  Use vecharm
  Use tdiracdefs
  Use tpotentialdensity
  Use occu
  Use xc
  Use pot
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="PLOTDENS"  
  !!
  Public :: plot_dens,plot_denspot,plot_dens_resolved
  !!
  !!***********************************************************************
  !!
  Character(*),Parameter  :: cpumesg_resolved='Resolved density output'
  Character(*),Parameter  :: cpumesg_resolvednet='Resolved density output: net&
       &'
  Character(*),Parameter  :: cpumesg_resolvedovl='Resolved density output: overlap'
  !!
  Character(10),Target  :: datadescriptor_SREL(8)
  Character(10),Target  :: datadescriptor_KHSREL(6)
  Character(10),Target  :: datadescriptor_NREL(2)
  Character(10),Target  :: datadescriptor_FREL(8)
  Data datadescriptor_NREL/&
       &'Re(g)     ',&
       &'Im(g)     '&
       &/
  Data datadescriptor_SREL/&
       &'Re(g)     ',&
       &'Im(g)     ',&
       &'Re(f)     ',&
       &'Im(f)     ',&
       &'Re(dg)    ',&
       &'Im(dg)    ',&
       &'Re(df)    ',&
       &'Im(df)    ' &
       &/
  Data datadescriptor_KHSREL/&
       &'Re(g)     ',&
       &'Im(g)     ',&
       &'Re(f)     ',&
       &'Im(f)     ',&
       &'Re(df)    ',&
       &'Im(df)    ' &
       &/
  Data datadescriptor_FREL/&
       &'Re(g_up)  ',&
       &'Im(g_up)  ',&
       &'Re(g_dn)  ',&
       &'Im(g_dn)  ',&
       &'Re(f_up)  ',&
       &'Im(f_up)  ',&
       &'Re(f_dn)  ',&
       &'Im(f_dn)  '&
       &/
  !!
  Type T_ResolvedDens
     !! N/S REL
     Real(t_real),Pointer :: p_nonvv(:,:,:,:)
     Real(t_real),Pointer :: p_noncv(:,:,:,:)
     Real(t_real),Pointer :: p_noncc(:,:,:,:)
     !! FREL
     Complex(t_real),Pointer:: p_roncc(:,:,:,:)
     Complex(t_real),Pointer:: p_ronvv(:,:,:,:)
     Complex(t_real),Pointer:: p_roncv(:,:,:,:)
     !! overlap density occupation numbers
     Real(t_real),Pointer :: p_onvv(:,:,:,:,:,:)
     Real(t_real),Pointer :: p_oncv(:,:,:,:,:,:)
     !! for net density LM expansion (which is faster)
     Integer(t_int) :: max_L
     Integer(t_int),Pointer :: p_nlmp(:),p_lmp(:,:)
     !! net density as LM expansion
     Real(t_real),Pointer   :: p_lcd(:,:,:,:)
     !! organization of overlap density
     Integer(t_int) :: Nmin,Nmax
     Integer(t_int),Pointer  :: p_siteOrder(:,:,:,:,:)
     !!
  End Type T_ResolvedDens
  !!
Contains
  
  !!***********************************************************************
  Subroutine init_resolveddens(rd)
    !!
    Implicit None
    !!IO
    Type(T_ResolvedDens),Intent(inout) :: rd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_resolveddens"
    !!
    Nullify(rd% p_roncc)
    Nullify(rd% p_ronvv)
    Nullify(rd% p_roncv)
    !!
    Nullify(rd% p_noncc)
    Nullify(rd% p_nonvv)
    Nullify(rd% p_noncv)
    !!
    Nullify(rd% p_onvv)
    Nullify(rd% p_oncv)
    !!
    Nullify(rd% p_nlmp)
    Nullify(rd% p_lmp)
    Nullify(rd% p_lcd)
    !!
    Nullify(rd% p_siteOrder)
    !!
    Return
    !!
  End Subroutine init_resolveddens
  !!***********************************************************************
  Subroutine cleanup_resolveddens(rd)
    !!
    Implicit None
    !!IO
    Type(T_ResolvedDens),Intent(inout) :: rd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_resolveddens"
    !!
    Call pfree(rd% p_roncc)
    Call pfree(rd% p_ronvv)
    Call pfree(rd% p_roncv)
    !!
    Call pfree(rd% p_noncc)
    Call pfree(rd% p_nonvv)
    Call pfree(rd% p_noncv)
    !!
    Call pfree(rd% p_onvv)
    Call pfree(rd% p_oncv)
    !!
    Call pfree(rd% p_nlmp)
    Call pfree(rd% p_lmp)
    Call pfree(rd% p_lcd)
    !!
    Call pfree(rd% p_siteOrder)
    !!
    !!
    Return
    !!
  End Subroutine cleanup_resolveddens
  !!***********************************************************************


  !!***********************************************************************
  Subroutine plot_denspot(symmetry,structure,lattice,mspin,potential,degree&
       &,gridplot,BEB)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Type(T_GridPlot),Intent(inout):: gridplot
    Logical(t_log),Intent(in)    :: BEB
    Integer(t_int),Intent(in)    :: degree
    Integer(t_int),Intent(in)    :: mspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="plot_denspot"
    Logical(t_log) :: alert,gga,gdens,gpot,stop_after,have_rhos
    Logical(t_log) :: gtotpot,gcoulpot,gxcpot,gewaldpot
    Integer(t_int) :: i,ir,imod
    Real(t_real)   :: rsh(3),rho(2),vvew(2),vex(2),vc(2),error,alerttol
    Real(t_real)   :: rho0(2)
    Character(200) :: dfname
    Character(200) :: pfname
    Character(200) :: pcfname
    Character(200) :: pxfname
    Character(200) :: pefname
    Type(T_Grid),pointer :: g
    Real(t_real),Pointer :: p_dlcd(:,:,:,:)
    Real(t_real),Pointer :: p_d2lcd(:,:,:,:)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    Real(t_real),Pointer :: p_rhos(:,:,:)
    Type(T_Mesh),Pointer  ::  p_rhomeshes(:)
    !!
    !!
    If( .Not. gridplot% active .Or. gridplot% ngrids<1 ) Return
    !!
    Nullify(p_dlcd,p_d2lcd,p_yb,p_rvylm,p_qvylm,p_rhomeshes,p_rhos)
    !!
    Write(outfile,1)
    !!
    Call prep_for_gga(structure,mspin,potential,p_yb,p_dlcd,p_d2lcd&
         &,p_rvylm,p_qvylm)
    !!
    have_rhos=.False.
    Call getDiracAtomicDensities(structure,have_rhos,p_rhos,p_rhomeshes)
    Call getFPLOAtomicDensities(mspin,structure,have_rhos,p_rhos,p_rhomeshes)
    !!
    !!
    stop_after=gridplot% stopafter.And.&
         &.Not.(Any(gridplot% p_grids(:)% nenergywindowresolved>0).Or.&
         &      has_k_resolved(gridplot))
    !!
    Do i=1,gridplot% ngrids
       !!
       g=>gridplot% p_grids(i)
       !!
       If(g% nq<1) Cycle
       !!
       gdens=Any(g% quantities==GRID_QUANTITY_TOTAL_DENSITY)
       gtotpot=Any(g% quantities==GRID_QUANTITY_TOTAL_POTENITAL)
       gcoulpot=Any(g% quantities==GRID_QUANTITY_COULOMB_POTENITAL)
       gxcpot=Any(g% quantities==GRID_QUANTITY_XC_POTENITAL)
       gewaldpot=Any(g% quantities==GRID_QUANTITY_EWALD_POTENITAL)
       gpot=(gtotpot.Or.gcoulpot.Or.gxcpot.Or.gewaldpot)
       stop_after=(stop_after .And. &
            &.Not. Any(g% quantities(:)==GRID_QUANTITY_SCF_WAVEFNCT))
       !!
       If(.Not. (gdens .Or. gpot )) cycle
       !!
       g% latticetype=GRID_DIRECT_LATTICE
       !!
       g% gridtype=GRID_TYPE_REGULAR
       If(Len_trim(g% pointfile)/=0) g% gridtype=GRID_TYPE_CUSTOM
       !!
       Call print_grid_props(outfile,g)
       Call make_grid(g,symmetry,structure)
       !!
       !!
       gga=((getxcversion() == XCVER_PBE96) .And. (gtotpot.Or.gxcpot))
       !!
       alert=.False.
       error=0.0_t_real
       alerttol=1.e-6_t_real
       !!
       imod=0
       !!
       If(gdens) Then
          Write(dfname,"('dens.',i3.3)")i
          Open(13,file='+grid_'//Trim(dfname))
          If(BEB)Then
             Call errmsg(nmodule,nsubfunc,E_warning&
                  &,msg="'Total (averaged) CPA density does not make much sense!'")
          Endif
       Endif
       !!
       If(gtotpot) Then
          Write(pfname,"('pot.',i3.3)")i
          Open(14,file='+grid_'//Trim(pfname))
       Endif
       If(gcoulpot) Then
          Write(pcfname,"('coulpot.',i3.3)")i
          Open(15,file='+grid_'//Trim(pcfname))
       Endif
       If(gxcpot) Then
          Write(pxfname,"('xcpot.',i3.3)")i
          Open(16,file='+grid_'//Trim(pxfname))
       Endif
       If(gewaldpot) Then
          Write(pefname,"('ewaldpot.',i3.3)")i
          Open(17,file='+grid_'//Trim(pefname))
       Endif
       !!
       pointloop: Do ir=1,g% npoints
          !!
          If(Nint(100.0_t_real*ir/g% npoints)>imod)Then
             Write(outfile,5) 100.0_t_real*ir/g% npoints
             imod=imod+10
          Endif
5         Format('calc at : ',f5.1,'% of point done')
          !!
          rsh(1:3)=g% points(1:3,ir)


          Call calc_dens_at_point(g% quantities,g% fixzero&
               &,rsh,rho,vvew,vex,vc,gga&
               &,alert,error,alerttol,p_yb,p_dlcd,p_d2lcd&
               &,p_rvylm,p_qvylm,symmetry,structure,lattice&
               &,potential,degree,mspin&
               &,have_rhos,p_rhomeshes,p_rhos,rho0)

          !!
          !!
          !!
          If(gdens)Then
             Call print_outputdata(13_t_int,g,ir,rsh,rho(1:2)-rho0(1:2)&
                  &,1.0_t_real)
          Endif
          If(gtotpot)Then
             Call print_outputdata(14_t_int,g,ir,rsh&
                  &,vvew(1:2)+vex(1:2)+vc(1:2),0.5_t_real)
          Endif
          If(gcoulpot)Then
             Call print_outputdata(15_t_int,g,ir,rsh&
                  &,vvew(1:2)+vc(1:2),0.5_t_real)
          Endif
          If(gxcpot)Then
             Call print_outputdata(16_t_int,g,ir,rsh&
                  &,vex(1:2),0.5_t_real)
          Endif
          If(gewaldpot)Then
             Call print_outputdata(17_t_int,g,ir,rsh&
                  &,vvew(1:2),0.5_t_real)
          Endif
          !!
       Enddo pointloop
       !!
       If(gdens) Close(13)
       If(gtotpot)  Close(14)
       If(gcoulpot)  Close(15)
       If(gxcpot)  Close(16)
       If(gewaldpot)  Close(17)
       !!
       If(gdens)Then
          Write(outfile,"('Density plot created in file ',a)")&
               &'+grid_'//Trim(dfname)
       Endif
       If(gtotpot)Then
          Write(outfile,"('Total Potential plot created in file ',a)")&
               &'+pot_'//Trim(pfname)
       Endif
       If(gcoulpot)Then
          Write(outfile,"('Coulomb Potential plot created in file ',a)")&
               &'+pot_'//Trim(pcfname)
       Endif
       If(gxcpot)Then
          Write(outfile,"('XC Potential plot created in file ',a)")&
               &'+pot_'//Trim(pxfname)
       Endif
       If(gewaldpot)Then
          Write(outfile,"('Ewald Potential plot created in file ',a)")&
               &'+pot_'//Trim(pefname)
       Endif
       !!
       If(g% createopendx)Then
          If(gdens)Then
             Call create_dx_descriptor(g,dfname)
          Endif
          If(gtotpot)Then
             Call create_dx_descriptor(g,pfname)
          Endif
          If(gcoulpot)Then
             Call create_dx_descriptor(g,pcfname)
          Endif
          If(gxcpot)Then
             Call create_dx_descriptor(g,pxfname)
          Endif
          If(gewaldpot)Then
             Call create_dx_descriptor(g,pefname)
          Endif
       Endif
       !!
       If(alert)Then
          
3         Format('''Numerical inaccuracy in vxc/exc &
               &calculation! '',/,'' maxerror=',e12.4e3,'''')
          Write(errbuf,3) error
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Endif
       !!
    Enddo
    !!
    !!
    !!
    Call pfree(p_dlcd)
    Call pfree(p_d2lcd)
    Call pfree(p_qvylm)
    Call pfree(p_rvylm)
    Call pfree(p_yb)
    Call pfree_meshs(p_rhomeshes)
    Call pfree(p_rhos)
    !!
    Write(outfile,1)
    !!
    If(stop_after) Then
       Call terminate(TERM_NORMAL,'due to user input.')
    Endif
    !!
    !!
    Return
    !!
1   Format(/,/,72('='),/,/)
  End Subroutine plot_denspot
  !!***********************************************************************
  Subroutine plot_dens(symmetry,structure,lattice,mspin,potential,degree,BEB&
       &,gridplot)
    !!
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Type(T_GridPlot),Intent(in)  :: gridplot
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: degree
    Logical(t_log),Intent(in)    :: BEB
    !!LOCAL
    Character(*),Parameter::nsubfunc="plot_dens"
    Logical(t_log) :: alert,gga
    Integer(t_int) :: index
    Integer(t_int) :: line,quantities(1)
    Real(t_real)   :: rsh(3),rho(2),ff(2),vvew(2),vex(2),vc(2),fspin
    Real(t_real)   :: error,alerttol
    Character(5000):: buffer,buf
    Integer        :: eof,err
    Logical(t_log) :: exist,header,stop_after,print_empty_space
    Logical(t_log) :: print_empty,print_index,print_spindown
    Logical(t_log) :: print_spinup,print_spin,print_total
    Logical(t_log) :: print_comments,print_point
    !!
    Real(t_real),Pointer :: p_dlcd(:,:,:,:)
    Real(t_real),Pointer :: p_d2lcd(:,:,:,:)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    !!
    !!
    !!
    Nullify(p_dlcd,p_d2lcd,p_yb,p_rvylm,p_qvylm)
    !!
    exist=gridplot% active !! avoid warnings
    
    Inquire(file=f_densgrid_in,exist=exist)
    If(exist)Then
       Write(outfile,"(/,/,72('='))")
       Write(errbuf,"('''Found file ',a,'.&
            & '',/,'' Will create density/potential plot!''')")&
            &Trim(f_densgrid_in)
       Call errmsg(nmodule,nsubfunc,E_information,msg=errbuf)
    Else
       return
    Endif
    !!
    If(BEB)Then
       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg="'Total (averaged) CPA density does not make much sense!'")
    Endif
    !!
    alert=.False.
    error=0.0_t_real
    alerttol=1.e-6_t_real
    !!
    Call prep_for_gga(structure,mspin,potential,p_yb,p_dlcd,p_d2lcd&
         &,p_rvylm,p_qvylm)
    !!
    !!
    !! default settings
    !!
    quantities(1)=GRID_QUANTITY_TOTAL_DENSITY
    print_comments=.False.
    print_point=.True.
    print_total=.True.
    print_spin=.True.
    print_spinup=.True.
    print_spindown=.True.
    print_index=.False.
    print_empty=.False.
    print_empty_space=.False.
    stop_after=.False.
    !!
    Open(12,file=f_densgrid_in)
    Open(13,file=f_densgrid_out)
    line=0_t_int
    index=0_t_int
    header=.True.
    readloop: Do 
       !! read line
       Read(12,"(a)",iostat=eof) buffer
       !! observe EOF
       If(eof < 0_t_int) Exit readloop
       line=line+1_t_int
       buf=Adjustl(buffer)
       !! read header and body  '#' lines 
       commentif: If(buf(1:1) == '#' )Then
          headerif: If(header)Then
             buf=Adjustl(buf(2:))
             If(buf(1:4) == "type" )Then
                buf=Adjustl(buf(5:))
                If(buf(1:1) == ":" )Then
                   buf=Adjustl(buf(2:))
                   If(buf(1:7) == "density" )Then
                      quantities(1)=GRID_QUANTITY_TOTAL_DENSITY
                   Else If(buf(1:9) == "potential" )Then
                      quantities(1)=GRID_QUANTITY_TOTAL_POTENITAL
                   Endif
                Endif
             Endif
             If(buf(1:6) == "output" )Then
                buf=Adjustl(buf(7:))
                If(buf(1:1) == ":" )Then
                   buf=Adjustl(buf(2:))
                   Do 
                      If(buf(1:8) == "comments" )Then
                         print_comments=.True.
                         buf=Adjustl(buf(9:))
                      Elseif(buf(1:11) == "emptylines " )Then
                         print_empty=.True.
                         buf=Adjustl(buf(12:))
                      Elseif(buf(1:21) == "emptylines_with_space" )Then
                         print_empty_space=.True.
                         buf=Adjustl(buf(22:))
                      Elseif(buf(1:4) == "stop" )Then
                         stop_after=.True.
                         buf=Adjustl(buf(5:))
                      Else
                         Exit
                      Endif
                   Enddo
                Endif
             Endif
             If(buf(1:4) == "data" )Then
                print_point=.False.
                print_total=.False.
                print_spin=.False.
                print_spinup=.False.
                print_spindown=.False.
                print_index=.False.
                buf=Adjustl(buf(5:))
                If(buf(1:1) == ":" )Then
                   buf=Adjustl(buf(2:))
                   Do 
                      If(buf(1:5) == "point" )Then
                         print_point=.True.
                         buf=Adjustl(buf(6:))
                      Elseif(buf(1:5) == "total" )Then
                         print_total=.True.
                         buf=Adjustl(buf(6:))
                      Elseif(buf(1:5) == "index" )Then
                         print_index=.True.
                         buf=Adjustl(buf(7:))
                      Elseif(buf(1:5) == "spin " )Then
                        print_spin=.True.
                         buf=Adjustl(buf(6:))
                      Elseif(buf(1:6) == "spinup" )Then
                         print_spinup=.True.
                         buf=Adjustl(buf(7:))
                      Elseif(buf(1:8) == "spindown" )Then
                         print_spindown=.True.
                         buf=Adjustl(buf(9:))
                      Else
                         Exit
                      Endif
                   Enddo
                Endif
             Endif
             If(print_comments)Write(13,'(a)')Trim(buffer)
             Cycle
          Else
             If(print_comments)Write(13,'(a)')Trim(buffer)
             Cycle
          Endif headerif
       End If commentif
       
       !! copy empty lines into output
       If(Len_trim(buffer) == 0 )Then
          If(print_empty_space)Then
             Write(13,"(' ')")
          else If(print_empty)Then
             Write(13,'()')
          Endif
          Cycle
       Endif
       If(header)Then
          
          If(print_comments)Then
             Write(13,"('#')",advance='no')
             If(print_index)Then
                Write(13,"(' index                         ')",advance='no')
             Endif
             If(print_point)Then
                Write(13,"(' x                             ',&
                     &     '  y                             ',&
                     &     '  z                             ')",advance='no')
             Endif
             If(print_total)Then
                Write(13,"(' total                         ')",advance='no')
             Endif
             If(print_spin)Then
                Write(13,"(' spin                          ')",advance='no')
             Endif
             If(print_spinup)Then
                Write(13,"(' spinup                        ')",advance='no')
             Endif
             If(print_spindown)Then
                Write(13,"(' spindown                       ')",advance='no')
             Endif
             Write(13,"()")
          Endif
       Endif
       header=.False.
       index=index+1_t_int
       Read(buffer,*,iostat=err) rsh(:)
       If(err>0)Then
          Write(errbuf,"('''File ''''',a,''''': Cannot read 3 &
               &real numbers in line ',i10,'!''')")&
               & Trim(f_densgrid_in),line
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif



       gga=((getxcversion() == XCVER_PBE96) .And.&
            & (quantities(1)==GRID_QUANTITY_TOTAL_POTENITAL))
       !!
       !!
       !!
       Call calc_dens_at_point(quantities,.False.,rsh,rho,vvew,vex,vc,gga&
            &,alert,error,alerttol,p_yb,p_dlcd,p_d2lcd&
            &,p_rvylm,p_qvylm,symmetry,structure,lattice&
            &,potential,degree,mspin)
       
       If(quantities(1)==GRID_QUANTITY_TOTAL_DENSITY)Then
          fspin=1.0_t_real
          ff(1:2)=rho(1:2)
       Else
          fspin=0.5_t_real
          ff(1:2)=vvew(1:2)+vex(1:2)+vc(1:2)
       Endif
       
       !!
       !!
       If(print_index)Then
          Write(13,"(1(i12,2x))",advance='no')index
       Endif
       If(print_point)Then
          Write(13,"(3(e30.15e3,2x))",advance='no')rsh(:)
       End If
       If(print_total)Then
          Write(13,"(1(e30.15e3,2x))",advance='no')(ff(1)+ff(2))*fspin
       Endif
       If(print_spin)Then
          Write(13,"(1(e30.15e3,2x))",advance='no')(ff(1)-ff(2))*fspin
       Endif
       If(print_spinup)Then
          Write(13,"(1(e30.15e3,2x))",advance='no')ff(1)
       Endif
       If(print_spindown)Then
          Write(13,"(1(e30.15e3,2x))",advance='no')ff(2)
       Endif
       Write(13,"()")
    Enddo readloop
    !!
    Close(12)
    Close(13)

    !!
    If(alert)Then
       
3      Format('''Numerical inaccuracy in vxc/exc &
            &calculation! '',/,'' maxerror=',e12.4e3,'''')
       Write(errbuf,3) error
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
    Endif
    !!
    If(quantities(1) == GRID_QUANTITY_TOTAL_POTENITAL)Then
       Write(outfile,"('Potential plot created in file ',a)")Trim(f_densgrid_out)
    Else
       Write(outfile,"('Density plot created in file ',a)")Trim(f_densgrid_out)
    Endif
    
    
    Write(outfile,"(72('='),/,/)")
    !!
    Call pfree(p_dlcd)
    Call pfree(p_d2lcd)
    Call pfree(p_qvylm)
    Call pfree(p_rvylm)
    Call pfree(p_yb)
    !!
    Write(outfile,"('old style =.densgrid: it is better to use the new grid&
         &plot submenu')")
    If(stop_after) Then
       
       Call terminate(TERM_NORMAL,'due to user input.')
    Endif
    !!
    !!
    Return
    !! 
    !!
    !!
  End Subroutine plot_dens
  !!***********************************************************************
  Subroutine print_outputdata(lun,gr,index,point,ff,fspin)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    Integer(t_int),Intent(in)    :: lun
    Integer(t_int),Intent(in)    :: index
    Real(t_real),Intent(in)      :: point(3),ff(2),fspin
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_outputdata"
    Real(t_real)   :: tot
    !!
    tot=(ff(1)+ff(2))*fspin
    !!
    If(Any(gr% outputdata==GRID_OUTPUTDATA_INDEX))Then
       Write(lun,"(1(i12,2x))",advance='no')index
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_POINTS))Then
       Write(lun,"(3(e30.15e3,2x))",advance='no')point(:)
    End If
    If(Any(gr% outputdata==GRID_OUTPUTDATA_TOTAL))Then
       Write(lun,"(1(e30.15e3,2x))",advance='no')tot
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPIN))Then
       Write(lun,"(1(e30.15e3,2x))",advance='no')(ff(1)-ff(2))*fspin
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPINUP))Then
       Write(lun,"(1(e30.15e3,2x))",advance='no')ff(1)
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPINDOWN))Then
       Write(lun,"(1(e30.15e3,2x))",advance='no')ff(2)
    Endif
    Write(lun,"()")
    
    !!
    !!
    Return
    !!
  End Subroutine print_outputdata
  !!***********************************************************************
  Subroutine print_outputdata_psi(lun,gr,psi,iband,index,ncomp,ms&
       &,rel,msig)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: gr
    Logical(t_log),Intent(in)    :: rel
    Integer(t_int),Intent(in)    :: lun,iband,ms,ncomp
    Integer(t_int),Intent(in)    :: index
    Complex(t_real),Intent(in)   :: msig(2,2)
    Complex(t_real),Intent(in)   :: psi(:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_outputdata_psi"
    Complex(t_real):: c
    Integer(t_int) :: icomp,ns,i,nns
    Real(t_real)   :: a,re,im,sofms,s,su,sd,f
    !!
    !!
    !!
    If(Any(gr% outputdata==GRID_OUTPUTDATA_INDEX))Then
       Write(lun,"(1(i12,2x))",advance='no')index
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_POINTS))Then
       Write(lun,"(3(e30.15e3,2x))",advance='no')gr% points(1:3,index)
    End If
    !!
    !!
    If(rel)Then
       a=0.0_t_real
       s=0.0_t_real
       Do icomp=1,ncomp
          f=(3.0_t_real-2.0_t_real*icomp)
          s=s+f*Real(Dot_product(psi(iband,1:2,icomp)&
               &,Matmul(msig(1:2,1:2),psi(iband,1:2,icomp))),t_real)
          a=a+Real(Dot_product(psi(iband,1:2,icomp)&
               &,psi(iband,1:2,icomp)),t_real)
       Enddo
    Else
       !! ms determines the the spinup,spindown and total spin density
       a=Sum(Abs(psi(iband,ms,1:ncomp))**2)
       sofms=3.0_t_real-2.0_t_real*ms
       s=a*sofms
    Endif
    !!
    su=(a+s)*0.5_t_real
    sd=(a-s)*0.5_t_real
    !!
    !!
    If(Any(gr% outputdata==GRID_OUTPUTDATA_TOTAL))Then
       Write(lun,"(2(e30.15e3,2x))",advance='no')a
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPIN))Then
       Write(lun,"(2(e30.15e3,2x))",advance='no')s
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPINUP))Then
       Write(lun,"(2(e30.15e3,2x))",advance='no')su
    Endif
    If(Any(gr% outputdata==GRID_OUTPUTDATA_SPINDOWN))Then
       Write(lun,"(2(e30.15e3,2x))",advance='no')sd
    Endif
    !!
    If(Any(gr% outputdata==GRID_OUTPUTDATA_Psi))Then
       Do icomp=1,ncomp
          nns=1
          If(rel)nns=2
          Do i=1,nns
             ns=ms
             If(rel) ns=i
             c=psi(iband,ns,icomp)
             re=Real(c,t_real)
             im=Aimag(c)
             If(Abs(im)<1.0e-15_t_real) im=0.0_t_real
             Write(lun,"(2(e30.15e3,2x))",advance='no')re,im
          Enddo
       Enddo
    Endif
    Write(lun,"()")
    !!
    !!
    Return
    !!
  End Subroutine print_outputdata_psi
  !!***********************************************************************
  Subroutine calc_dens_at_point(quantities,fixzero,rsh,rho,vvew,vex,vc,gga&
       &,alert,error,alerttol,p_yb,p_dlcd,p_d2lcd,p_rvylm&
       &,p_qvylm,symmetry,structure,lattice,potential,degree,mspin&
       &,Ihave_rhos,p_rhomeshes,p_rhos,rho0)
    !!
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Type(T_Mesh),Pointer,Optional      :: p_rhomeshes(:)
    Logical(t_log),Intent(in),Optional :: Ihave_rhos
    Logical(t_log),Intent(in)    :: fixzero
    Integer(t_int),Intent(in)    :: mspin
    Integer(t_int),Intent(in)    :: degree
    Integer(t_int),Intent(in)    :: quantities(:)
!!$    Logical(t_log),Intent(in)    :: BEB
    Logical(t_log),Intent(in)    :: gga
    Real(t_real),Intent(in)      :: rsh(3)
    Real(t_real),Intent(in),Optional      :: p_rhos(:,:,:)
    Real(t_real),Intent(in)      :: alerttol
    Logical(t_log),Intent(out)   :: alert
    Real(t_real),Intent(out)     :: error
    Real(t_real),Intent(out)     :: rho(2),vex(2),vc(2),vvew(2)
    Real(t_real),Intent(out),Optional    :: rho0(2)
    Real(t_real),Pointer :: p_dlcd(:,:,:,:)
    Real(t_real),Pointer :: p_d2lcd(:,:,:,:)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_rvylm(:,:)
    Real(t_real),Pointer :: p_qvylm(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_dens_at_point"
    Logical(t_log) :: have_rhos
    Integer(t_int) :: isa,issa,k,NR(2,3),i1,i2,i3,indexa,j,ilm,l,m,ms
    Integer(t_int) :: ii,jj,nn
    Real(t_real)   :: rmax,rmin,a(3),rg,ag,ap(3),rr(3),drr,dr2,f,df,d2f,d,s

    Real(t_real)   :: epsxc
    Real(t_real)   :: fdp,fpmpm,trho,tgrad(3)
    Real(t_real)   :: afdp,afpmpm,gfn(3),gfdp(3),gqv(5),aa
    Real(t_real)   :: grho(3,2),laprhos(2),ggrho(3,3,2),gradrhos(2),gradrho
    Real(t_real)   :: mixrhos(2),mixrho
    Real(t_real)   :: coefa(0:degree)
    Type(T_Mesh),Pointer :: pmesh
    !!
    have_rhos=(Present(Ihave_rhos).And.Present(p_rhomeshes).And.&
         &Present(p_rhos).And.Present(rho0))
    If(have_rhos) have_rhos=Ihave_rhos
    !!
    !! collect contributions
    If(have_rhos)  rho0(1:2)=0.0_t_real
    rho(1:2)=0.0_t_real
    grho(1:3,1:2)=0.0_t_real
    ggrho(1:3,1:3,1:2)=0.0_t_real
    laprhos(1:2)=0.0_t_real
    vex(1:2)=0.0_t_real
    vc(1:2)=0.0_t_real
    siteloop: Do isa=1,structure% nsite
       !If(isa /= 2_t_int)Cycle siteloop!!TODO: remove
       issa=structure% p_isrt(isa)
       !!
       !! do the superposition of neutral densities
    
       If(have_rhos.And.Any(quantities(:)==GRID_QUANTITY_TOTAL_DENSITY))Then
          
          pmesh => p_rhomeshes(issa)
          rmin=pmesh% rmin
          rmax=pmesh% rmax
          a(:)=rsh(:)-structure% p_tau(:,isa)
          !!
          Do k=1_t_int,3_t_int
             rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rmax/twopi
             ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
             
             NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
             NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
          Enddo
          !!
          If(structure% str_type == STR_TYPE_MOLECULE)Then
             NR(:,:)=0_t_int
          Endif
          !!
          i1loop1: Do i1=NR(1,1),NR(2,1)
             i2loop1: Do i2=NR(1,2),NR(2,2)
                i3loop1: Do i3=NR(1,3),NR(2,3)
                   ap(:)= symmetry% lattice_vectors(:,1)*i1&
                        &+symmetry% lattice_vectors(:,2)*i2&
                        &+symmetry% lattice_vectors(:,3)*i3
                   
                   rr(:)=a(:)-ap(:)
                   dr2=Dot_product(rr,rr)
                   drr=Sqrt(dr2)
                   !!
                   If(fixzero.And.drr<rmin) Then
                      rr(:)=rmin*(/1.1_t_real,-1.2_t_real,0.9_t_real/)
                      dr2=Dot_product(rr,rr)
                      drr=Sqrt(dr2)
                   Endif
                   !!
                   If(drr>rmax .Or. &
                        &(.Not.fixzero.And.drr <= Max(rmin,0.0_t_real))) &
                     &Cycle i3loop1
                   !!
                   indexa=mintpl_coefs(degree,pmesh,drr,coefa)
                   !!
                   d=Dot_product(&
                        &p_rhos(indexa:indexa+degree,issa,1)&
                        &,coefa(0:degree))/dr2&
                        &*structure % p_conao(issa)
                   s=Dot_product(&
                        &p_rhos(indexa:indexa+degree,issa,2)&
                        &,coefa(0:degree))/dr2&
                        &*structure % p_conao(issa)
                   rho0(1)=rho0(1)+(d+s)*0.5_t_real
                   rho0(2)=rho0(2)+(d+s)*0.5_t_real
                End Do i3loop1
             End Do i2loop1
          End Do i1loop1
          !!
       Endif
       !!
       !!
       !! do the real densities
       !!
       pmesh => potential% p_pmeshs(issa)
       !!
       rmin=pmesh% rmin
       rmax=pmesh% rmax
       !!
       
       a(:)=rsh(:)-structure% p_tau(:,isa)
       !!
       Do k=1_t_int,3_t_int
          rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rmax/twopi
          ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
          
          NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
          NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
       Enddo
       !!
       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Endif
       !!
!!$          Do k=1_t_int,3_t_int
!!$             Print *,'NR=[',NR(1,k),',',NR(2,k),']'
!!$          End Do
       !!
       i1loop: Do i1=NR(1,1),NR(2,1)
          i2loop: Do i2=NR(1,2),NR(2,2)
             i3loop: Do i3=NR(1,3),NR(2,3)
                ap(:)= symmetry% lattice_vectors(:,1)*i1&
                     &+symmetry% lattice_vectors(:,2)*i2&
                     &+symmetry% lattice_vectors(:,3)*i3
                
                rr(:)=a(:)-ap(:)
                dr2=Dot_product(rr,rr)
                drr=Sqrt(dr2)
                !!
                If(fixzero.And.drr<rmin) Then
                   rr(:)=rmin*(/1.1_t_real,-1.2_t_real,0.9_t_real/)
                   dr2=Dot_product(rr,rr)
                   drr=Sqrt(dr2)
                Endif
                !!
                If(drr>rmax .Or. &
                     &(.Not.fixzero.And.drr <= Max(rmin,0.0_t_real))) &
                     &Cycle i3loop
                !!
                Call ylms12(p_yb,rr(1),rr(2),rr(3),potential% max_L)
                !!
                If(gga)Then
                   Call calc_real_quad_harmonics_opt(potential% max_L&
                        &,p_yb,p_rvylm,p_qvylm)
                Endif
                !!
                indexa=mintpl_coefs(degree,pmesh,drr,coefa)
                
                jloop: Do j=1_t_int,potential% p_nlmp(isa)
                   ilm=potential% p_lmp(j,isa)
                   l=llilm(ilm)
                   m=mmilm(ilm)
                   !!
                   msloop: Do ms=1_t_int,mspin
                      !!
                      rho(ms)=rho(ms)+ &
                           &Dot_product(&
                           &potential% p_lcd(indexa:indexa+degree,j,isa,ms)&
                           &,coefa(0:degree))*p_yb(ilm)/dr2&
                           &*structure % p_conao(issa)
                      !!
                      afdp=0.0_t_real
                      afpmpm=0.0_t_real
                      gfn(:)=0.0_t_real
                      gfdp(:)=0.0_t_real
                      gqv(:)=0.0_t_real
                      ggaif: If(gga)Then
                         
                         f=Dot_product(&
                              &potential% p_lcd(indexa:indexa+degree,j,isa,ms)&
                              &,coefa(0:degree))/dr2
                         df=Dot_product(&
                              & p_dlcd(indexa:indexa+degree,j,isa,ms)&
                              &,coefa(0:degree))/drr
                         d2f=Dot_product(&
                              & p_d2lcd(indexa:indexa+degree,j,isa,ms)&
                              &,coefa(0:degree))
                         
                         !! gradient of density
                         fdp=df-(2.0_t_real+l)*f
                         
                         !!
                         fpmpm=d2f-(2.0_t_real*l+5.0_t_real)*df&
                              &+(l+4.0_t_real)*(l+2.0_t_real)*f
                         !!
                         afdp   =afdp   +fdp  *p_yb(ilm)
                         afpmpm =afpmpm +fpmpm*p_yb(ilm)
                         gfn(:) =gfn(:) +f   *p_rvylm(:,ilm)
                         gfdp(:)=gfdp(:)+fdp  *p_rvylm(:,ilm)
                         gqv(:) =gqv(:) +f   *p_qvylm(:,ilm)
                         !!
                      Endif ggaif
                      !! collect the separate sums
                      grho(:,ms)=grho(:,ms)+gfn(:)/drr+afdp*rr(:)/dr2
                      !!
                      Do jj=1,3
                         ggrho(jj,jj,ms)=ggrho(jj,jj,ms)+afdp/dr2
                         Do ii=1,3
                            ggrho(ii,jj,ms)=ggrho(ii,jj,ms)&
                                 &+afpmpm*rr(ii)*rr(jj)/dr2/dr2&
                                 &+(gfdp(ii)*rr(jj) +gfdp(jj)*rr(ii))&
                                 & /drr/dr2
                         Enddo
                      Enddo
                      !!
                      !!
                      nn=0
                      Do jj=1,3
                         Do ii=jj,3
                            nn=nn+1
                            If(nn==6) Then
                               aa=(-gqv(1)-gqv(4))/dr2
                            Else
                               aa=gqv(nn)/dr2
                            Endif
                            ggrho(ii,jj,ms)=ggrho(ii,jj,ms)+aa
                            If(ii /= jj) ggrho(jj,ii,ms)=ggrho(jj,ii,ms)+aa
                         Enddo
                      Enddo
                      !!
                      !!
                   Enddo msloop
                   !!
                   If(Any(quantities(:)==GRID_QUANTITY_TOTAL_POTENITAL).Or.&
                        &Any(quantities(:)==GRID_QUANTITY_COULOMB_POTENITAL))Then
                      !! local coulomb potential contribution
                      vc(1:2)=vc(1:2)+ &
                           &Dot_product(&
                           &potential% p_vc(indexa:indexa+degree,j,isa)&
                           &,coefa(0:degree))*p_yb(ilm)/drr&
                           &*structure % p_conao(issa)
                   Endif
                   !!
                   !!
                Enddo jloop
             End Do i3loop
          End Do i2loop
       End Do i1loop
    Enddo siteloop
    !!
    !!
    If(Any(quantities(:)==GRID_QUANTITY_TOTAL_POTENITAL).Or.&
         &Any(quantities(:)==GRID_QUANTITY_XC_POTENITAL))Then
       !! xc-potential
       !!
       If(getxcversion()== XCVER_PBE96)Then
          trho=rho(1)+rho(2)
          tgrad(1:3)=grho(1:3,1)+grho(1:3,2)
          gradrhos(1)=Sqrt(Sum(grho(1:3,1)**2))
          gradrhos(2)=Sqrt(Sum(grho(1:3,2)**2))
          gradrho=Sqrt(Sum(tgrad(1:3)**2))
          !!
          Do ms=1,mspin
             !!laprhos(ms)=Sum((/(ggrho(nn,nn,ms),nn=1,3)/))
             d=0.0_t_real
             Do nn=1,3
                d=d+ggrho(nn,nn,ms)
             Enddo
             laprhos(ms)=d
          Enddo
          !!
          mixrhos(1)=Dot_product(grho(1:3,1)&
               &,Matmul(ggrho(1:3,1:3,1),grho(1:3,1)))
          mixrhos(2)=Dot_product(grho(1:3,2)&
               &,Matmul(ggrho(1:3,1:3,2),grho(1:3,2)))
          mixrho=Dot_product(tgrad(1:3)&
               &,Matmul(ggrho(1:3,1:3,1)+ggrho(1:3,1:3,2),tgrad(1:3)))
       Endif
       !!
       Call adjustrho(rho,1.0_t_real,error,mspin,alerttol,alert)
       Call vxcrho(mspin,rho,gradrhos,gradrho,laprhos,mixrhos,mixrho&
            &,vex,epsxc,potential% xreduce)
       !!
    Endif
    !!
    If(Any(quantities(:)==GRID_QUANTITY_EWALD_POTENITAL))Then
       !!
       If(structure% str_type == STR_TYPE_CRYSTAL)Then
          vvew(1:2)=vew(rsh(:),potential% p_vft,lattice% nlattg,lattice% p_glat)
       Else If(structure% str_type == STR_TYPE_MOLECULE)Then
          vvew(1:2)=vew_sum(rsh(:),structure% nsite,structure% p_isrt&
               &,structure% p_tau,potential% p_pmeshs,potential% nrp,degree&
               &,potential% max_L,potential% mlmp,potential% p_nlmp&
               &,potential% p_lmp,potential% p_vew&
               &,potential% p_ac)
       Else
          vvew(1:2)=0.0_t_real
          Call terminate(TERM_CRASH,Trim(nsubfunc)//'unknown str_type!')
       Endif
    Endif
    !!
    !!
    !!
    If(mspin == 1)Then
       rho(2)=rho(1)*0.5_t_real
       rho(1)=rho(2)
       vex(2)=vex(1)
       vvew(2)=vvew(1)
       vc(2)=vc(1)
    Endif
    !!
    !!
    !!
    !!
    !!
    !!
    Return
    !! 
    !!
    !!
  End Subroutine calc_dens_at_point
  !!***********************************************************************
  Subroutine create_dx_descriptor(grid,text)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: grid
    Character(*),Intent(in)      :: text
    !!LOCAL
    Character(*),Parameter::nsubfunc="create_dx_descriptor"
    Character(500) :: field,structure,typ,position,seltxt
    Character(1),Pointer   :: cc(:)
    Integer(t_int) :: add,l,ib
    Include "densgrid_net.f90"
    Include "densgrid_cfg.f90"
    !!
    Nullify(cc)
    !!
    If(All(grid% outputdata/=GRID_OUTPUTDATA_POINTS))Then
       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg="'Output data do not include grid points. &
            &No possibility to',/&
            &,' create opendx output !'")
       Return
    Endif
    !!
    add=0
    If(grid% InclPeriodicPoints) add=1


    Open(13,file='grid_'//Trim(text)//'.general')
    Write(13,"('file = ',a)")'+grid_'//Trim(text)
    If(Len_trim(grid% pointfile)==0)Then
       Write(13,"('grid = ',2(i8,' x '),i8)")grid% subdivisions(3:1:-1)+add
    Else
       Write(13,"('points = ',i8)")grid% npoints
       
    Endif
    Write(13,"('format = ascii')")
    Write(13,"('interleaving = field')")
    If(Len_trim(grid% pointfile)==0) &
         &Write(13,"('majority = row')")
    !!
    field='field ='
    structure='structure ='
    typ='type ='
    position='dependency = '
    !!
    ib=0
    seltxt=''
    If(Any(grid% outputdata==GRID_OUTPUTDATA_INDEX))Then
       field=Trim(field)//' index, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' int, '
       position=Trim(position)//' positions,'
    End If
    If(Any(grid% outputdata==GRID_OUTPUTDATA_POINTS))Then
       field=Trim(field)//' locations, '
       structure=Trim(structure)//' 3-vector, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
    End If
    If(Any(grid% outputdata==GRID_OUTPUTDATA_TOTAL))Then
       field=Trim(field)//' total, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'

       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""total"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPIN))Then
       field=Trim(field)//' spin, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spin"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPINUP))Then
       field=Trim(field)//' spinup, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spinup"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPINDOWN))Then
       field=Trim(field)//' spindown, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spindown"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif


    l=Len_trim(field)
    If(field(l:l)==',')field(l:l)=' '
    Write(13,2)trim(field)

    l=Len_trim(structure)
    If(structure(l:l)==',')structure(l:l)=' '
    Write(13,2)trim(structure)
   

    l=Len_trim(typ)
    If(typ(l:l)==',')typ(l:l)=' '
    Write(13,2)trim(typ)

    l=Len_trim(position)
    If(position(l:l)==',')position(l:l)=' '
    Write(13,2)trim(position)
    !!
    Write(13,"('')")
    Write(13,"('end')")
    !!
    Close(13)
    !!
!!$    Call replace(cc,densgrid_net,"densgrid.general"&
!!$         &,'grid_'//text//cnum//'.general')
    Call write_codified('grid_'//Trim(text)//'.net',densgrid_net)
    !!

    Call replace(cc,densgrid_cfg,"densgrid.general"&
         &,'grid_'//Trim(text)//'.general')

    Call write_codified('grid_'//Trim(text)//'.cfg',scopy(cc))
    !!
    Call pfree(cc)
    !!
    Return
    !!
2   Format(a)
    !!
  End Subroutine create_dx_descriptor
  !!***********************************************************************
  Subroutine create_dx_descriptor_psi(grid,text,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(in)      :: grid
    Integer(t_int),Intent(in)    :: relativistic
    Character(*),Intent(in)      :: text
    !!LOCAL
    Character(*),Parameter::nsubfunc="create_dx_descriptor_psi"
    Character(500) :: field,structure,typ,position,seltxt
    Character(1),Pointer   :: cc(:)
    Integer(t_int) :: add,l,ib,i
    Character(10),Pointer :: datadescr(:)

    Include "densgrid_net.f90"
    Include "densgrid_cfg.f90"
    !!
    Nullify(cc)
    !!
    If(All(grid% outputdata/=GRID_OUTPUTDATA_POINTS))Then
       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg="'Output data do not include grid points. &
            &No possibility to',/&
            &,' create opendx output !'")
       Return
    Endif
    !!
    add=0
    If(grid% InclPeriodicPoints) add=1


    Open(13,file='grid_'//Trim(text)//'.general')
    Write(13,"('file = ',a)")'+grid_'//Trim(text)
    If(Len_trim(grid% pointfile)==0)Then
       Write(13,"('grid = ',2(i8,' x '),i8)")grid% subdivisions(3:1:-1)+add
    Else
       Write(13,"('points = ',i8)")grid% npoints
       
    Endif
    Write(13,"('format = ascii')")
    Write(13,"('interleaving = field')")
    If(Len_trim(grid% pointfile)==0) &
         &Write(13,"('majority = row')")
    !!
    field='field ='
    structure='structure ='
    typ='type ='
    position='dependency = '
    !!
    ib=0
    seltxt=''
    If(Any(grid% outputdata==GRID_OUTPUTDATA_INDEX))Then
       field=Trim(field)//' index, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' int, '
       position=Trim(position)//' positions,'
    End If
    If(Any(grid% outputdata==GRID_OUTPUTDATA_POINTS))Then
       field=Trim(field)//' locations, '
       structure=Trim(structure)//' 3-vector, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
    End If
    If(Any(grid% outputdata==GRID_OUTPUTDATA_TOTAL))Then
       field=Trim(field)//' total, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'

       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""total"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPIN))Then
       field=Trim(field)//' spin, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spin"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPINUP))Then
       field=Trim(field)//' spinup, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spinup"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif
    If(Any(grid% outputdata==GRID_OUTPUTDATA_SPINDOWN))Then
       field=Trim(field)//' spindown, '
       structure=Trim(structure)//' scalar, '
       typ=Trim(typ)//' double, '
       position=Trim(position)//' positions,'
       seltxt=Trim(seltxt)//"// option["//Trim(intToChar10(ib))&
            &//"]: name = ""spindown"", value = "//intToChar10(ib)//"\n"
       ib=ib+1
    Endif

    If(Any(grid% outputdata==GRID_OUTPUTDATA_PSI))Then
       If(relativistic==SCALAR_RELATIVISTIC)Then
          datadescr=>datadescriptor_SREL
       Else If(relativistic==NON_RELATIVISTIC)Then
          datadescr=>datadescriptor_NREL
       Else If(relativistic==SCALAR_RELATIVISTIC_KH)Then
          datadescr=>datadescriptor_KHSREL
       Else If(relativistic==FULL_RELATIVISTIC)Then
          datadescr=>datadescriptor_FREL
       Else
          datadescr=>datadescriptor_SREL !! aboid warning in gfortran
       Endif
       Do i=1,Size(datadescr,1)
          field=Trim(field)//' '//Trim(datadescr(i))//', '
          structure=Trim(structure)//' scalar, '
          typ=Trim(typ)//' double, '
          position=Trim(position)//' positions,'
          seltxt=Trim(seltxt)//'// option['//Trim(intToChar10(ib))&
               &//']: name = "'//Trim(datadescr(i))//'", value = '&
               &//intToChar10(ib)//'\n'
          ib=ib+1
       Enddo
    Endif


    l=Len_trim(field)
    If(field(l:l)==',')field(l:l)=' '
    Write(13,2)trim(field)

    l=Len_trim(structure)
    If(structure(l:l)==',')structure(l:l)=' '
    Write(13,2)trim(structure)
   

    l=Len_trim(typ)
    If(typ(l:l)==',')typ(l:l)=' '
    Write(13,2)trim(typ)

    l=Len_trim(position)
    If(position(l:l)==',')position(l:l)=' '
    Write(13,2)trim(position)
    !!
    Write(13,"('')")
    Write(13,"('end')")
    !!
    Close(13)
    !!
!!$    Call replace(cc,densgrid_net,"densgrid.general"&
!!$         &,'grid_'//text//cnum//'.general')
    Call write_codified('grid_'//Trim(text)//'.net',densgrid_net)
    !!

    Call replace(cc,densgrid_cfg,"densgrid.general"&
         &,'grid_'//Trim(text)//'.general')

    Call write_codified('grid_'//Trim(text)//'.cfg',scopy(cc))
    !!
    Call pfree(cc)
    !!
    Return
    !!
2   Format(a)
    !!
  End Subroutine create_dx_descriptor_psi
  !!***********************************************************************
  Subroutine getDiracAtomicDensities(structure,have_rhos,p_rhos,p_rhomeshes)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Mesh),Pointer  ::  p_rhomeshes(:)
    Logical(t_log),Intent(out)   :: have_rhos
    Real(t_real),Pointer :: p_rhos(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getDiracAtomicDensities"
    Integer(t_int) :: nrmax,issa,j,j2
    Integer :: eof
    Logical(t_log) :: exists
    Character(200) :: fname
    Character(2)   :: el
    Character(1000) :: buf
    !!
    !!
    have_rhos=.True.
    !!
    Call palloc_meshs('p_rhomeshes',p_rhomeshes,structure% nsort&
         &,nmodule,nsubfunc)
    
    !!
    nrmax=0
    Do issa=1,structure% nsort
       Write(fname,"('=.atrho.',i3.3)")issa
       Inquire(file=Trim(fname),exist=exists)
       If(.Not.exists) Then
          have_rhos=.False.
          Exit
       Endif
       Open(12,file=Trim(fname))
       el=''
       readloop: Do 
          Read(12,2,iostat=eof) buf
          If(eof /=0 ) Exit readloop
          j=Index(buf,'Version: ')
          If(j > 0) Then
             Cycle readloop
          Endif
          !!
          j=Index(buf,'element: ')
          If(j > 0) Then
             el=Trim(Adjustl(buf(j+8:)))
             !Print "('<',a,'>')",el
          Endif
          !!
          j=Index(buf,'<start mesh>')
          If(j > 0) Then
             Call read_mesh_info(12_t_int,p_rhomeshes(issa))
             Call setup_mesh(p_rhomeshes(issa))
             Exit readloop
          Endif
       Enddo readloop
       nrmax=Max(nrmax,p_rhomeshes(issa)% nr)
2      Format(a)
       Close(12)
       If(el/=structure% p_elements(issa))Then
          Write(outfile,45)issa,el,structure% p_elements(issa)
45        Format('found file =.atrho.',i3.3,', but for different element ('&
               &,a2,')?',/,'Need ',a,'!')
          have_rhos=.False.
          Exit
       Endif
       !Print *,p_rhomeshes(issa)% rmax,p_rhomeshes(issa)% nr,nrmax
    Enddo
    !!
    Call palloc('p_rhos',p_rhos,1_t_int,Max(1_t_int,nrmax)&
         &,1_t_int,structure% nsort&
         &,1_t_int,2_t_int,nmodule,nsubfunc)
    !!
    p_rhos=0.0_t_real
    !!
    If(have_rhos)Then
       Do issa=1,structure% nsort
          Write(fname,"('=.atrho.',i3.3)")issa
          Open(12,file=Trim(fname))
          readloop1: Do 
             Read(12,2,iostat=eof) buf
             If(eof /=0 ) Exit readloop1
             j=Index(buf,'<start function')
             If(j > 0) Then
                j=j+15
                j2=Index(buf(j:),'density')
                If(j2>0)Then
                   Call read_mesh_function(12_t_int,p_rhos(:,issa,1))
                Endif
                j2=Index(buf(j:),'spin density')
                If(j2>0 )Then
                   Call read_mesh_function(12_t_int,p_rhos(:,issa,2))
                Endif
                Cycle readloop1
             Endif
          End Do readloop1
          !!
          Close(12)
       End Do
       !!
       Write(outfile,"('found =.atrho files, will use them')")
       !!
    Endif

    !!
    Return
    !!
  End Subroutine getDiracAtomicDensities
  !!***********************************************************************
  Subroutine getFPLOAtomicDensities(mspin,structure,have_rhos,p_rhos,p_rhomeshes)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Mesh),Pointer  ::  p_rhomeshes(:)
    Integer(t_int),Intent(in)    :: mspin
    Logical(t_log),Intent(inout)   :: have_rhos
    Real(t_real),Pointer :: p_rhos(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="getFPLOAtomicDensities"
    Integer(t_int) :: nrmax,issa,j,j2,ir
    Integer :: eof
    Real(t_real)   :: d
    Logical(t_log) :: exists,have_frhos,have_some
    Character(200) :: fname
    Character(2)   :: el,els
    Character(1000) :: buf
    Real(t_real),Pointer :: rf(:)
    Type(T_Mesh) :: tmesh
    !!
    Nullify(rf)
    !!
    Call init_mesh(tmesh)
    !!
    have_frhos=.True.
    have_some=.False.
    !!

    Print *,'todo: sorts and sites'
    
    !!
    nrmax=0
    issaloop: Do issa=1,structure% nsort
       Write(fname,5)issa,1
       Inquire(file=Trim(fname),exist=exists)
       If(.Not.exists) Then
          have_frhos=.False.
          Exit issaloop
       Else
          If(issa==1)Then
             Call palloc_meshs('p_rhomeshes',p_rhomeshes,structure% nsort&
                  &,nmodule,nsubfunc)
          Endif
       Endif
       have_some=.True.
       Open(12,file=Trim(fname))
       el=''
       readloop: Do 
          Read(12,2,iostat=eof) buf
          If(eof /=0 ) Exit readloop
          j=Index(buf,'Version: ')
          If(j > 0) Then
             Cycle readloop
          Endif
          !!
          j=Index(buf,'element: ')
          If(j > 0) Then
             el=Trim(Adjustl(buf(j+8:)))
             !Print "('<',a,'>')",el
          Endif
          !!
          j=Index(buf,'<start mesh>')
          If(j > 0) Then
             Call read_mesh_info(12_t_int,p_rhomeshes(issa))
             Call setup_mesh(p_rhomeshes(issa))
             Exit readloop
          Endif
       Enddo readloop
       !Call write_mesh_info(outfile,p_rhomeshes(issa))
       nrmax=Max(nrmax,p_rhomeshes(issa)% nr)
2      Format(a)
       Close(12)
       If(el/=structure% p_elements(issa))Then
          Write(outfile,45)issa,el,structure% p_elements(issa)
45        Format('found file =.atdens.',i3.3,'.1, but for different element ('&
               &,a2,')?',/,'Need ',a,'!')
          Call terminate(TERM_ERROR,'inconsistent files found.')
       Endif
       !Print *,p_rhomeshes(issa)% rmax,p_rhomeshes(issa)% nr,nrmax
       !!
       !! now try to read the second mesh
       !!
       If(mspin==1) Cycle issaloop
       !!
       Write(fname,5)issa,2
       Inquire(file=Trim(fname),exist=exists)
       If(.Not.exists) Then
          Write(outfile,"('Cannot find =.atdens.',i3.3,'.2')")issa
          Call terminate(TERM_ERROR,'inconsistent files found.')
       Endif
       Open(12,file=Trim(fname))
       els=''
       readloop2: Do 
          Read(12,2,iostat=eof) buf
          If(eof /=0 ) Exit readloop2
          j=Index(buf,'Version: ')
          If(j > 0) Then
             Cycle readloop2
          Endif
          !!
          j=Index(buf,'element: ')
          If(j > 0) Then
             els=Trim(Adjustl(buf(j+8:)))
             !Print "('<',a,'>')",els
          Endif
          !!
          j=Index(buf,'<start mesh>')
          If(j > 0) Then
             Call read_mesh_info(12_t_int,tmesh)
             Call setup_mesh(tmesh)
             Exit readloop2
          Endif
       Enddo readloop2
       !!
       If(el/=els)Then
          Write(outfile,46)issa,els,el
46        Format('found file =.atdens.',i3.3,'.2, but for different element ('&
               &,a2,')?',/,'Need ',a,'!')
          have_frhos=.False.
          Call terminate(TERM_ERROR,'inconsistent files found.')
       Endif
       !!
       !Call write_mesh_info(outfile,tmesh)
       !!
       If(.Not.meshesEqual(tmesh,p_rhomeshes(issa)))Then
          Write(outfile,"('mesh in file =.atdens.',i3.3,'.1 ')")issa
          Call write_mesh_info(outfile,p_rhomeshes(issa))
          Write(outfile,"('mesh in file =.atdens.',i3.3,'.2 ')")issa
          Call write_mesh_info(outfile,tmesh)
          Write(outfile,"('the meshes obtained from the two files differ!')")
          Call terminate(TERM_ERROR,'inconsistent files found.')
       Endif
       !!
    Enddo issaloop
    !!
    If(have_frhos)Then
       !Print *,'alloc p_rhos'
       !!
       Call palloc('rf',rf,1_t_int,nrmax,nmodule,nsubfunc)
       !!
       Call palloc('p_rhos',p_rhos,1_t_int,Max(1_t_int,nrmax)&
            &,1_t_int,structure% nsort&
            &,1_t_int,2_t_int,nmodule,nsubfunc)
       !!
       p_rhos=0.0_t_real
       !!
       issareadloop:Do issa=1,structure% nsort
          !Print *,'issa=',issa
          Write(fname,5)issa,1
          Open(12,file=Trim(fname))
          readloop1: Do 
             Read(12,2,iostat=eof) buf
             If(eof /=0 ) Exit readloop1
             j=Index(buf,'<start function')
             If(j > 0) Then
                j=j+15
                j2=Index(buf(j:),'sqrt(4*pi)*r^2* l :    0  m:    0')
                If(j2>0)Then
                   Call read_mesh_function_rf(12_t_int,rf,p_rhos(:,issa,1))
                   p_rhos(:,issa,1)=p_rhos(:,issa,1)/fourpi
                   If(    p_rhomeshes(issa)% with_zero  .And. &
                        & Abs(rf(1)) > 1.0e-12_t_real)Then
                      p_rhos(1,issa,1)=0.0_t_real
                      p_rhos(2:nrmax,issa,1)=p_rhos(1:nrmax-1,issa,1)
                   Endif
                Endif
                Cycle readloop1
             Endif
          End Do readloop1
          Close(12)
          !!
          Write(fname,5)issa,2
          Inquire(file=Trim(fname),exist=exists)
          If(.Not. exists .Or. mspin==1) Then
             p_rhos(:,issa,2)=0.0_t_real
             Cycle issareadloop
          Endif
          !!
          !Print *,'issa=',issa
          Open(12,file=Trim(fname))
          readloop3: Do 
             Read(12,2,iostat=eof) buf
             If(eof /=0 ) Exit readloop3
             j=Index(buf,'<start function')
             If(j > 0) Then
                j=j+15
                j2=Index(buf(j:),'sqrt(4*pi)*r^2* l :    0  m:    0')
                If(j2>0)Then
                   Call read_mesh_function_rf(12_t_int,rf,p_rhos(:,issa,2))
                   p_rhos(:,issa,2)=p_rhos(:,issa,2)/fourpi
                   If(    p_rhomeshes(issa)% with_zero  .And. &
                        & Abs(rf(1)) > 1.0e-12_t_real)Then
                      p_rhos(1,issa,2)=0.0_t_real
                      p_rhos(2:nrmax,issa,2)=p_rhos(1:nrmax-1,issa,2)
                   Endif
                Endif
                Cycle readloop3
             Endif
          End Do readloop3
          Close(12)
          !!
          !!
          Do ir=1,p_rhomeshes(issa)% nr
             d=p_rhos(ir,issa,1)
             p_rhos(:,issa,1)=d+p_rhos(:,issa,2)
             p_rhos(:,issa,2)=d-p_rhos(:,issa,2)
          Enddo
          !!
       End Do issareadloop
       !!
       Write(outfile,"('found =.atdens files, will use them')")
       !!
    Endif
    !!
    If(have_some .And. .Not. have_frhos)Then
3      Format('''Found some of the =.atdens.site.spin files but not all! ''')
       Write(errbuf,3) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    If(have_rhos .And. have_frhos)Then
33     Format('''Found two different sets of atomic density files &
            &remove one of them! ''')
       Write(errbuf,33) 
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)

    Endif
    if(have_frhos) have_rhos=have_frhos
    !!
    Call cleanup_mesh(tmesh)
    !!
    Call pfree(rf)
    !!
    Return
    !!
5   Format('=.atdens.',i3.3,'.',i1)
  End Subroutine getFPLOAtomicDensities
  !!***********************************************************************
  Subroutine plot_dens_resolved(symmetry,structure,lattice,potential,basis&
       &,kohnsham,loi,lsdau,opc,bzone,degree,mspin,relativistic&
       &,quant,gridplot)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Lsdau),Intent(inout)  :: lsdau
    Type(T_OPC),Intent(inout)    :: opc
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_LOI),Intent(in)       :: loi
    Type(T_Quant),Intent(in)     :: quant
    Integer(t_int),Intent(in)    :: degree,mspin,relativistic
    Type(T_GridPlot),Intent(inout):: gridplot
    !!LOCAL
    Character(*),Parameter::nsubfunc="plot_dens_resolved"
    Logical(t_log) :: rel,stop_after
    Integer(t_int) :: imod,ig,lun,ir,ie,ik,iband,ms,nbanddim,nspin,ispin
    Integer(t_int) :: kmod,in,i,nns,ns,icomp
    Real(t_real)   :: rsh(3),rho(2),sigz(2,2),eminscf,emaxscf,scale
    Real(t_real)   :: re,im,emin,emax
    Complex(t_real):: msig(2,2),c
    Type(T_ResolvedDens) :: rd
    Character(200) :: dfname
    Type(T_Grid),Pointer :: g
    Type(T_EnergyWindowResolved),Pointer :: ew
    Type(T_Kresolved),Pointer :: kr
    Type(T_Kresolved),Target  :: krscf
    Real(t_real),Pointer :: p_ybkr(:)
    Complex(t_real),Pointer:: p_ss(:,:,:)
    !! we save all coefficients
    Complex(t_real),Pointer :: p_cvv(:,:,:)!! nvdim,nband,nspin
    Complex(t_real),Pointer :: p_ccv(:,:,:)!! ncdim,nband,nspin
    !! resulting list of band indices, either from p_bandindices or 
    !! from the energy window
    Integer(t_int) :: nbands(2) !! for each spin
    Integer(t_int),Pointer :: p_bands(:,:) !! nband,nspin
    !! we need separate spin arrays, for organizational reason
    Complex(t_real),Pointer:: p_psi(:,:,:,:) !! max(nbands),...
    !!                                          nspin(or 2 If rel)...
    !!                                          ncomp,T_Grid% npoints
    !!
    Character(10),Pointer :: datadescr(:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Data sigz(1,1)/1.0_t_real/
    Data sigz(1,2)/0.0_t_real/
    Data sigz(2,1)/0.0_t_real/
    Data sigz(2,2)/-1.0_t_real/
    !!
    If( .Not. gridplot% active .Or. gridplot% ngrids<1 ) Return
    !!
    msig=Matmul(quant% D1o2,Matmul(sigz,Transpose(Conjg(quant% D1o2))))
    !!
    Nullify(p_ybkr,p_ss,p_cvv,p_ccv,p_bands,p_psi)
    !!
    Call init_resolveddens(rd)
    Call init_kresolved(krscf)
    !!
    Call watch(cpumesg_resolved,cpu_start,reset=.True.)            
    !!
    Write(outfile,1)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    nspin=mspin
    If(rel) nspin=1
    !!
    !!
    scale=twopi/ symmetry% lattice_constants(1)
    eminscf=Floor  (Minval(kohnsham% p_ebkfsm-kohnsham% efermi)*hatoev)-1
    emaxscf=Ceiling(Maxval(kohnsham% p_ebkfsm-kohnsham% efermi)*hatoev)+1
    !!
    If(relativistic==SCALAR_RELATIVISTIC)Then
       datadescr=>datadescriptor_SREL
    Else If(relativistic==NON_RELATIVISTIC)Then
       datadescr=>datadescriptor_NREL
    Else If(relativistic==SCALAR_RELATIVISTIC_KH)Then
       datadescr=>datadescriptor_KHSREL
    Else If(relativistic==FULL_RELATIVISTIC)Then
       datadescr=>datadescriptor_FREL
    Else
       datadescr=>datadescriptor_SREL !! aboid warning in gfortran
    Endif
    !!
    stop_after=gridplot% stopafter
    !!
    gridloop: Do ig=1,gridplot% ngrids
       !!
       g=>gridplot% p_grids(ig)
       !!
       !!
       !!
       g% latticetype=GRID_DIRECT_LATTICE
       !!
       g% gridtype=GRID_TYPE_REGULAR
       If(Len_trim(g% pointfile)/=0) g% gridtype=GRID_TYPE_CUSTOM
       !
       g% fixzero=.True.
       g% fixzero=.False. !!todo
       !!
       Call print_grid_props(outfile,g)
       Call make_grid(g,symmetry,structure)
       !!
       !!
       !!
       !! 
       !! prepare site ordering scheme for overlap matrix
       Call prepSiteOrderArray(symmetry,structure,lattice,rd)
       !!
       ewinloop: Do ie=1,g% nenergywindowresolved
          !!
          ew => g% p_energywindowresolved(ie)
          !!
          If(ew% nsections<1) Cycle
          !!
          imod=0
          Write(dfname,"('resolveddens.er',i3.3,'.',a,'.g',i3.3)")ie&
               &,Trim(ew% name),ig
          lun=13
          Open(lun,file='+grid_'//Trim(dfname))
          !!
          !!
          !! prepare the occupation numbers
          !!
          Call prepare_resolved_occus(mspin,symmetry,structure,lattice,basis&
               &,kohnsham,bzone,rd,relativistic,ew)
          !!
          !!
          !! make net density as local  YLM expansion, this is faster than
          !! on the fly starting from orbtials
          !!
          !!
          Call prepare_net_density(mspin,structure,potential,basis&
               &,degree,quant,rd,relativistic)
          !!
          !!
          pointloop: Do ir=1,g% npoints
             If(Nint(100.0_t_real*ir/g% npoints)>imod)Then
                Write(outfile,5) 100.0_t_real*ir/g% npoints
                imod=imod+10
             Endif
             !!
             rsh(1:3)=g% points(1:3,ir)
             !!
             !!
             Call calc_resolveddens_at_point(g% fixzero&
                  &,rsh,rho,symmetry,structure&
                  &,basis,kohnsham,degree,mspin&
                  &,rd,rd% p_onvv,rd% p_oncv)
             
             !!
             Call print_outputdata(lun,g,ir,rsh,rho(1:2),1.0_t_real)
             !!
          End Do pointloop
          !!
          Close(lun)
          !!
          Write(outfile,"('Resolved density plot created in file ',a)")&
               &'+grid_'//Trim(dfname)
          !!
          If(g% createopendx)Then
             Call create_dx_descriptor(g,dfname)
          Endif
          !!
       Enddo ewinloop
       !!       
       !! 
       !! k-resolved densities/wave functions
       !!
       Call palloc('p_ss',p_ss,1_t_int,2_t_int,1_t_int,2*(basis% lmx+1)&
            &,-(basis% lmx+1),(basis% lmx+1),nmodule,nsubfunc)
       Call palloc('p_ybkr',p_ybkr,1_t_int,(basis% lmx+1)**2&
            &,nmodule,nsubfunc)
       Call palloc('p_cvv',p_cvv,1_t_int,kohnsham% nvdim&
            &,1_t_int,kohnsham% nband,1_t_int,nspin&
            &,nmodule,nsubfunc) 
       Call palloc('p_ccv',p_ccv,1_t_int,kohnsham% ncdim&
            &,1_t_int,kohnsham% nband,1_t_int,nspin&
            &,nmodule,nsubfunc)
       Call palloc('p_bands',p_bands,1_t_int,kohnsham% nband&
            &,1_t_int,nspin,nmodule,nsubfunc)
       !!
       kresloop: Do ik=1,g% nkresolved
          !!
          kr => g% p_kresolved(ik)
          !!
          If(kr% nbandindices<1.And. kr% emin>=kr% emax) Cycle
          !!
          !!
          !!
          imod=0
          !!
          !!
          Call calc_eigenvalues_at_kpoint(symmetry% lattice_constants&
               &,structure,lattice,mspin,kohnsham,bzone&
               &,basis,loi,lsdau,opc,relativistic,kr&
               &,nbands,p_bands,p_cvv,p_ccv)
          !!
          If(Any(nbands(1:nspin)==0))Then
             Write(errbuf,"('""No bands in energy window for one or &
                  &both spins""')")
             Call errmsg(nmodule,nsubfunc,E_information,msg=errbuf)
          Endif

          nbanddim=Max(1_t_int,Maxval(nbands(1:nspin)))
          !!
          !!
          ispin=nspin
          If(rel) ispin=2
          Call palloc('p_psi',p_psi,1_t_int,nbanddim,1_t_int,ispin&
               &,1_t_int,basis% ncomp,1_t_int,g% npoints,nmodule,nsubfunc)
          !!
          p_psi=c0
          !!
          rpointloop: Do ir=1,g% npoints
             If(Nint(100.0_t_real*ir/g% npoints)>imod)Then
                Write(outfile,5) 100.0_t_real*ir/g% npoints
                imod=imod+10
             Endif
             !!
             rsh(1:3)=g% points(1:3,ir)
             !!
             !!
             Call calc_psi_at_point(ir,rsh&
                  &,kr% kpoint*scale,nbands,p_cvv,p_ccv,p_psi&
                  &,mspin,symmetry,structure,basis&
                  &,kohnsham% nvdim,kohnsham% ncdim&
                  &,kohnsham% p_vblockstart,kohnsham% p_cblockstart&
                  &,g% fixzero,degree,relativistic,p_ybkr,p_ss)
             !!
             !!
             !!
          End Do rpointloop
          !!
          !!
          kmsloop: Do ms=1,nspin
             lun=13
             Do iband=1,nbands(ms)
                If(rel)Then
                   Write(dfname,"('kresolveddens.kr',i3.3,'.',a&
                        &,'.band',a,'.g',i3.3)")ik&
                        &,Trim(kr% name)&
                        &,Trim(intToChar10(p_bands(iband,ms)))&
                        &,ig
                Else
                   Write(dfname,"('kresolveddens.kr',i3.3,'.',a&
                        &,'.band',a,'.spin',a,'.g',i3.3)")ik&
                        &,Trim(kr% name)&
                        &,Trim(intToChar10(p_bands(iband,ms)))&
                        &,Trim(intToChar10(ms)),ig
                Endif
                Open(lun,file='+grid_'//Trim(dfname))
                Do ir=1,g% npoints
                   Call print_outputdata_psi(lun,g,p_psi(:,:,:,ir),iband,ir&
                        &,basis% ncomp,ms,rel,msig)
                Enddo
                Close(lun)
                
                Write(outfile,"('K-resolved density plot &
                     &created in file ',a)") '+grid_'//Trim(dfname)
                !!
                If(g% createopendx)Then
                   Call create_dx_descriptor_psi(g,dfname,relativistic)
                Endif
             Enddo
             !!
          Enddo kmsloop
          !!
          Call pfree(p_psi)
          !!
          !!
       Enddo kresloop
       !!
       !!
       scfwfnctif: If(Any(g% quantities(:)==GRID_QUANTITY_SCF_WAVEFNCT))Then
          !! calculate and write all SCF core-orthogonalized  wave functions
          emin=g% scfergwindow(1)
          emax=g% scfergwindow(2)
          If(emin>emax)Then
             emin=eminscf
             emax=emaxscf
          Endif
          emin=Max(eminscf,emin)
          emax=Min(emaxscf,emax)
          Write(outfile,"(/,'scf wave function energy window:')",advance='no')
          If(Abs(emin)<10000.0_t_real)Then
             Write(outfile,"(f12.5,' eV ')",advance='no') emin
          Else
             Write(outfile,"(e14.5e3,' eV ')",advance='no') emin
          Endif
          If(Abs(emax)<10000.0_t_real)Then
             Write(outfile,"(f12.5,' eV')",advance='no') emax
          Else
             Write(outfile,"(e14.5e3,' eV ')",advance='no') emax
          Endif
          Write(outfile,"(/)")
          
          !!
          Write(dfname,"('+gridscfpsi','.g',i3.3)")ig
          lun=13
          Open(lun,file=dfname)
          Write(lun,"('# scf wave functions, k,points (below) given in global&
               & cartesian units')")
          Write(lun,"('# nkp= ',i8)")bzone% nkp
          Write(lun,"('# max number of bands= ',i8)")kohnsham% nband
          Write(lun,"('# mspin= ',i8)")nspin
          !!
          Write(lun,"('# gridpoints (x,y,z) following in global &
               &cartesian units',/&
               &,'# ngridpoints= ',i8)")g% npoints
          Write(lun,"('# x                               y&
               &                                z')")
          Do ir=1,g% npoints
             Write(lun,"(3(e30.15e3,2x))")g% points(1:3,ir)
          Enddo

          kmod=0
          !!
          scfkresloop: Do ik=1,bzone% nkp
             !!
             If(Nint(100.0_t_real*ik/bzone% nkp)>kmod)Then
                Write(outfile,15) 100.0_t_real*ik/bzone% nkp
                kmod=kmod+10
             Endif
             !!
             kr => krscf
             kr% kpoint=bzone% p_bk(:,ik)/scale
             kr% emin=emin
             kr% emax=emax
             !!
             Write(lun,"('# ik    = ',i8,/,'# k     = ',3(e30.15e3,1x))")&
                  &ik,kr% kpoint*scale
             !!
             !!
             Do ms=1,nspin
                !!
                !!
                !!
                Call determineKresBands(kohnsham% nband,ms&
                     &,kohnsham% p_ebkfsm(:,ik,ms),kohnsham% efermi,kr&
                     &,nbands,p_bands)
                !!
                !!
                Do in=1,nbands(ms)
                   p_cvv(:,in,ms)=&
                        &kohnsham% p_ccvk(p_bands(in,ms),ik,:,ms)
                   p_ccv(:,in,ms)=&
                        &kohnsham% p_ccck(p_bands(in,ms),ik,:,ms)
                Enddo
                !!
                !Write(outfile,"('ms=',i2,1000(f12.5))") &
                ! &ms,(kohnsham% p_ebkfsm(:,ik,ms)-kohnsham% efermi)*hatoev
                !Write(outfile,"('nbands=',i8,' eint=[',f12.5,' ',f12.5,']')")&
                !     &nbands(ms),kr% emin,kr% emax
             Enddo
             !!
             !!
             !!
             !!
             imod=0
             !!
             !!
             !!
             nbanddim=Max(1_t_int,Maxval(nbands(1:nspin)))
             !!
             ispin=nspin
             If(rel) ispin=2
             Call palloc('p_psi',p_psi,1_t_int,nbanddim,1_t_int,ispin&
                  &,1_t_int,basis% ncomp,1_t_int,g% npoints,nmodule,nsubfunc)
             !!
             p_psi=c0
             !!
             scfrpointloop: Do ir=1,g% npoints
                ! If(Nint(100.0_t_real*ir/g% npoints)>imod)Then
                !   Write(outfile,5) 100.0_t_real*ir/g% npoints
                !   imod=imod+10
                ! Endif
                !!
                rsh(1:3)=g% points(1:3,ir)
                !!
                !!
                Call calc_psi_at_point(ir,rsh&
                     &,kr% kpoint*scale,nbands,p_cvv,p_ccv,p_psi&
                     &,mspin,symmetry,structure,basis&
                     &,kohnsham% nvdim,kohnsham% ncdim&
                     &,kohnsham% p_vblockstart,kohnsham% p_cblockstart&
                     &,g% fixzero,degree,relativistic,p_ybkr,p_ss)
                !!
                !!
                !!
             End Do scfrpointloop
             !!
             !!
             !!
             scfkmsloop: Do ms=1,nspin
                If(rel) Then
                   Write(lun,"('# spin  = ',i8&
                        &,'   note that spin is a wave functions &
                        &component in FREL')")ms
                Else
                   Write(lun,"('# spin  = ',i8)")ms
                Endif
                Write(lun,"('# Nbands= ',i8)")nbands(ms)
                scfbandloop:Do iband=1,nbands(ms)
                   Write(lun,"('# iband= ',i8)")iband
                   Write(lun,"('# ')",advance='no')
                   Do i=1,Size(datadescr,1)
                      Write(lun,2,advance='no')Trim(datadescr(i))
                   Enddo
                   Write(lun,3)
                   Do ir=1,g% npoints
                      Do icomp=1,basis% ncomp
                         nns=1
                         If(rel)nns=2
                         Do i=1,nns
                            ns=ms
                            If(rel) ns=i
                            c=p_psi(iband,ns,icomp,ir)
                            re=Real(c,t_real)
                            im=Aimag(c)
                            If(Abs(im)<1.0e-15_t_real) im=0.0_t_real
                            Write(lun,4,advance='no')re,im
                         Enddo
                      Enddo
                      Write(lun,3)
                   Enddo
                   !!
                Enddo scfbandloop
                !!
             Enddo scfkmsloop
             !!
             Call pfree(p_psi)
             !!
          Enddo scfkresloop
          !!
          Close(lun)
          Write(outfile,"('K-resolved density/wave functions on&
               & SCF k-mesh created in file ',a)") dfname 

          !!
       Endif scfwfnctif
       !!
       Call pfree(p_cvv)
       Call pfree(p_ccv)
       Call pfree(p_bands)
       !!
    Enddo gridloop
    !!
    Call pfree(p_ybkr)
    Call pfree(p_ss)
    !!
    Call cleanup_resolveddens(rd)
    Call cleanup_kresolved(krscf)
    !!
    !!
    Write(outfile,1)
    !!
    Call watch(cpumesg_resolved,cpu_stop,delete=.True.,Print=.True.)
    !!
    If(stop_after) Then
       Call terminate(TERM_NORMAL,'due to user input.')
    Endif
    !!
    !!
    Return
    !!
1   Format(/,/,72('='),/,/)
2   Format(a30,2x)
3   Format()
4   Format(2(e30.15e3,2x))
5   Format('calc at : ',f5.1,'% of grid points done')
15  Format('calc at : ',f5.1,'% of scf k-points done')

    !!
  End Subroutine plot_dens_resolved
  !!***********************************************************************
  Subroutine prepare_resolved_occus(mspin,symmetry,structure,lattice,basis&
       &,kohnsham,bzone,rd,relativistic,ew)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin,relativistic
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_EnergyWindowResolved),Pointer :: ew
    Type(T_ResolvedDens),Intent(inout) :: rd
    !!LOCAL
    Character(*),Parameter::nsubfunc="prepare_resolved_occus"
    Logical(t_log) :: rel
    Integer(t_int) :: isa,nband,lenchi,istchi,is,spin,ms,ns,ic,ik,in
    Real(t_real)   :: emin,emax,de,f,e,spinfac
    Real(t_real),Pointer :: p_erc(:,:,:)
    Real(t_real),Pointer :: p_erv(:,:,:)
    !!
    Nullify(p_erc,p_erv)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    nband=kohnsham% nband
    !!
    If(rel)Then
       !!
       Call palloc('rd% p_roncc',rd% p_roncc,1_t_int,kohnsham% ncsmx&
            &,1_t_int,kohnsham% ncsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       Call palloc('rd% p_ronvv',rd% p_ronvv,1_t_int,kohnsham% nvsmx&
            &,1_t_int,kohnsham% nvsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       Call palloc('rd% p_roncv',rd% p_roncv,1_t_int,kohnsham% ncsmx&
            &,1_t_int,kohnsham% nvsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       !!
    Else
       !!
       Call palloc('rd% p_nonvv',rd% p_nonvv,1_t_int,kohnsham% nvsmx&
            &,1_t_int,kohnsham% nvsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       Call palloc('rd% p_noncv',rd% p_noncv,1_t_int,kohnsham% ncsmx&
            &,1_t_int,kohnsham% nvsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       Call palloc('rd% p_noncc',rd% p_noncc,1_t_int,kohnsham% ncsmx&
            &,1_t_int,kohnsham% ncsmx,1_t_int,structure% nsite&
            &,1_t_int,mspin,nmodule,nsubfunc)
       !!
       !!
    Endif
    !!
    Call palloc('rd% p_onvv',rd% p_onvv,1_t_int,kohnsham% nvsmx&
         &,1_t_int,kohnsham% nvsmx&
         &,1_t_int,structure% nsite,1_t_int,structure% nsite&
         &,1_t_int,mspin,1_t_int,Max(1_t_int,lattice% npairsmax)&
         &,nmodule,nsubfunc)
    Call palloc('rd% p_oncv',rd% p_oncv,1_t_int,kohnsham% ncsmx&
         &,1_t_int,kohnsham% nvsmx&
         &,1_t_int,structure% nsite,1_t_int,structure% nsite&
         &,1_t_int,mspin,1_t_int,Max(1_t_int,lattice% npairsmax)&
         &,nmodule,nsubfunc)
    !!
    !!
    !! core energy screening
    !!
    Call palloc('p_erc',p_erc,1_t_int,kohnsham% nchismx&
         &,1_t_int,structure% nsite,1_t_int,mspin,nmodule,nsubfunc)
    !!
    p_erc=0.0_t_real
    !!
    Do isa=1,structure% nsite
       !!
       lenchi=kohnsham% p_chiblocklength(isa)
       istchi=kohnsham% p_chiblockstart(isa)-1
       !!
       Do is=1,ew% nsections
          emin=ew% p_emin(is)
          emax=ew% p_emax(is)
          de=ew% p_de(is)
          spin=ew% p_spin(is)
          !!
          !!
          Do ms=1,mspin
             !!
             ns=ms
             If(rel)ns=1
             !!
             f=0.0_t_real
             If(mspin==2)Then
                If(spin==ms.Or.spin==0) f=1.0_t_real
             Else
                If(spin==0) Then
                   f=1.0_t_real
                Else
                   f=0.5_t_real
                Endif
             Endif
             !!
             Do ic=1,lenchi
                e=(kohnsham% p_hc(istchi+ic,ns)-&
                     &(kohnsham% efermi-(3-2*ns)*kohnsham% Bfsm))*hatoev
                If(emin<=e .And. e<=emax)Then
                   p_erc(ic,isa,ms)=Max(p_erc(ic,isa,ms),f)
                Else If(emax<e .And. de>0.0_t_real)Then
                   p_erc(ic,isa,ms)=Max(p_erc(ic,isa,ms)&
                        &     ,Exp(-((e-emax)/de)**2)*f)
                Else If(e<emin .And. de>0.0_t_real)Then
                   p_erc(ic,isa,ms)=Max(p_erc(ic,isa,ms)&
                        &     ,Exp(-((emin-e)/de)**2)*f)
                Endif
             Enddo
          Enddo
       Enddo
    Enddo
    !!
    !! valence energy screening
    !!
    !!
    Call palloc('p_erv',p_erv,1_t_int,nband,1_t_int,bzone% nkp&
         &,1_t_int,mspin,nmodule,nsubfunc)

    !!
    p_erv=0.0_t_real
    !!
    Do ik=1,bzone% nkp
       Do in=1,nband

          Do is=1,ew% nsections
             emin=ew% p_emin(is)
             emax=ew% p_emax(is)
             de=ew% p_de(is)
             spin=ew% p_spin(is)
             !!
             Do ms=1,mspin
                !!
                ns=ms
                If(rel) ns=1
                !!
                f=0.0_t_real
                If(mspin==2)Then
                   If(spin==ms.Or.spin==0) f=1.0_t_real
                Else
                   If(spin==0) Then
                      f=1.0_t_real
                   Else
                      f=0.5_t_real
                   Endif
                Endif
                !!
                e=(kohnsham% p_ebkfsm(in,ik,ns)-kohnsham% efermi)*hatoev
                If(emin<=e .And. e<=emax)Then
                   p_erv(in,ik,ms)=Max(p_erv(in,ik,ms),f)
                Else If(emax<e.And. de>0.0_t_real)Then
                   p_erv(in,ik,ms)=Max(p_erv(in,ik,ms)&
                        &     ,Exp(-((e-emax)/de)**2)*f)
                Else If(e<emin.And. de>0.0_t_real)Then
                   p_erv(in,ik,ms)=Max(p_erv(in,ik,ms)&
                        &     ,Exp(-((emin-e)/de)**2)*f)
                Endif
             Enddo
          Enddo

       Enddo ! in
    Enddo ! ik
    !!
    !!
    Do ms=1,mspin
       ns=ms
       If(rel)ns=1
       Do isa=1,structure% nsite
          lenchi=kohnsham% p_chiblocklength(isa)
          p_erc(1:lenchi,isa,ms)=kohnsham% p_coccu(1:lenchi,isa,ns)&
               &*p_erc(1:lenchi,isa,ms)
       Enddo
    Enddo
    !!
    spinfac=2.0_t_real/mspin
    If(rel) spinfac=1.0_t_real
    !!
    Do ms=1,mspin
       ns=ms
       If(rel)ns=1
       Do ik=1,bzone% nkp
          p_erv(1:nband,ik,ms)=bzone% p_wbk(ik)*spinfac&
               &*p_erv(1:nband,ik,ms)
       Enddo
    Enddo
    !!
    Call make_netoccus(mspin,symmetry,structure,kohnsham,p_erv,p_erc&
         &,bzone% nkp,basis,relativistic,rd% p_noncc,rd% p_noncv,rd% p_nonvv&
         &,rd% p_roncc,rd% p_roncv,rd% p_ronvv,.True.)
    !!
    Call make_ovloccus(mspin,symmetry,structure,lattice,basis,bzone&
         &,kohnsham,p_erv,relativistic,rd% p_oncv,rd% p_onvv,.True.,.True.)
    !!
    !!
    !!
    Call pfree(p_erc)
    Call pfree(p_erv)
    !!
    Return
    !!
  End Subroutine prepare_resolved_occus

  !!***********************************************************************
  Subroutine prepare_net_density(mspin,structure,potential,basis,degree&
       &,quant,rd,relativistic)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: mspin,degree,relativistic
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Quant),Intent(in)     :: quant
    Type(T_Potential),Intent(in) :: potential
    Type(T_ResolvedDens),Intent(inout) :: rd
    !!LOCAL
    Character(*),Parameter::nsubfunc="prepare_net_density"
    Logical(t_log) :: rel
    Integer(t_int) :: mlmp,il,isa,ms
    Real(t_real),Pointer :: p_lcd(:,:,:,:)
    !!
    Nullify(p_lcd)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    rd% max_L=basis% lmx*2
    If(rel)Then
       !! Magnet FREL has small components with l=lmx+1, which
       !! give non-vanishing LM-densities with L_max=(lmx+1)*2.
       !! In nonmag FREL the term vanishes.
       rd% max_L=rd% max_L+2 
    Endif
    !!
    !!
    mlmp=(rd% max_L+1)**2
    Call palloc('rd% p_nlmp',rd% p_nlmp,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('rd% p_lmp',rd% p_lmp,1_t_int,mlmp,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Do isa=1,structure% nsite
       Do il=1,mlmp
          rd% p_lmp(il,isa)=il
       Enddo
       rd% p_nlmp(isa)=mlmp
    Enddo
    !!
    Call palloc('rd% p_lcd',rd% p_lcd,1_t_int,basis% nrb,1_t_int,mlmp&
         &,1_t_int,structure% nsite,1_t_int,mspin,nmodule,nsubfunc)
    Call palloc('p_lcd',p_lcd,1_t_int,basis% nrb,1_t_int,mlmp&
         &,1_t_int,structure% nsite,1_t_int,mspin,nmodule,nsubfunc)
    !!
    rd% p_lcd=0.0_t_real
    If(rel)Then
       Do ms=1,mspin
          p_lcd=0.0_t_real
          Call calc_net_density(mspin,structure,potential&
               &,rd% max_L,rd% p_nlmp,rd% p_lmp,basis&
               &,rd% p_nonvv,rd% p_noncv,rd% p_noncc&
               &,rd% p_ronvv(:,:,:,ms),rd% p_roncv(:,:,:,ms)&
               &,rd% p_roncc(:,:,:,ms)&
               &,quant,basis% p_basmeshs,p_lcd,degree&
               &,relativistic,printorbmom=.False.,useOVO=use_OVO)
          rd% p_lcd(:,:,:,ms)=p_lcd(:,:,:,ms)
       Enddo
    Else
       Call calc_net_density(mspin,structure,potential&
            &,rd% max_L,rd% p_nlmp,rd% p_lmp,basis&
            &,rd% p_nonvv,rd% p_noncv,rd% p_noncc&
            &,rd% p_ronvv(:,:,:,1),rd% p_roncv(:,:,:,1),rd% p_roncc(:,:,:,1)&
            &,quant,basis% p_basmeshs,rd% p_lcd,degree&
            &,relativistic,printorbmom=.False.,useOVO=use_OVO)
    Endif
    !!
    Call pfree(p_lcd)
    !!
    Return
    !!
  End Subroutine prepare_net_density
  !!***********************************************************************
  Subroutine calc_resolveddens_at_point(fixzero&
       &,rsh,rho,symmetry,structure&
       &,basis,kohnsham,degree,mspin&
       &,rd,onvv,oncv)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Logical(t_log),Intent(in)    :: fixzero
    Integer(t_int),Intent(in)    :: degree,mspin
    Real(t_real),Intent(in)      :: rsh(3)
    Real(t_real),Intent(in)      :: onvv(:,:,:,:,:,:)
    Real(t_real),Intent(in)      :: oncv(:,:,:,:,:,:)
    Type(T_ResolvedDens),Intent(in) :: rd
    Real(t_real),Intent(out)     :: rho(2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_resolveddens_at_point"
    Logical(t_log) :: nocore,debnoovl,debnonet
    Integer(t_int) :: isa,issa,NR(2,3),indexa,i3,i2,i1,iva,la
    Integer(t_int) :: il,m,nc,ms,ncomp,nv,lmax,k
    Integer(t_int) :: iblv,inblv,nblmax,nblv,iblva,iblvb,isb,irba,lena,lenb
    Integer(t_int) :: inblva,inblvb,iblc,inblc,nblc,inblca,iblca,lenc
    Integer(t_int) :: ica,j
    Real(t_real)   :: rmin,rmax,a(3),rg,ag,drr,dr2,rr(3),ap(3),texp,ff1
    Real(t_real)   :: oor
    Real(t_real)   :: coefa(0:degree)
    !Complex(t_real):: c
    Integer(t_int),Pointer :: p_siteblv(:),p_Rindexblv(:,:)
    Integer(t_int),Pointer :: p_siteblc(:),p_Rindexblc(:,:)
    Integer(t_int),Pointer :: p_lenv(:)
    Integer(t_int),Pointer :: p_lenc(:)
    Real(t_real),Pointer :: p_yb(:)
    Real(t_real),Pointer :: p_fblv(:)
    Real(t_real),Pointer :: p_fblc(:)
    Type(T_Mesh),Pointer :: bmesh
    !!
    Nullify(p_yb,p_lenv,p_lenc)
    Nullify(p_siteblv,p_Rindexblv,p_siteblc,p_Rindexblc,p_fblv,p_fblc)
    !!
    rho(1:2)=0.0_t_real
    !!
    debnoovl=debugfun('DENSITY','calc_density','NO_OVERDENS')
    debnonet=debugfun('DENSITY','calc_density','NO_NETDENS')
    !!
    ncomp=basis% ncomp
    !!
    lmax=basis% lmx
    !!
    nblmax=0
    Do isa=1,structure% nsite
       bmesh => basis% p_basmeshs(structure% p_isrt(isa))
       rmin=bmesh% rmin
       rmax=bmesh% rmax
       a(:)=rsh(:)-structure% p_tau(:,isa)
       !!
       Do k=1,3
          rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rmax/twopi
          ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
          
          NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
          NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
       Enddo
       !!
       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Endif
       !!
       Do i1=NR(1,1),NR(2,1)
          Do i2=NR(1,2),NR(2,2)
             Do i3=NR(1,3),NR(2,3)
                ap(:)= symmetry% lattice_vectors(:,1)*i1&
                     &+symmetry% lattice_vectors(:,2)*i2&
                     &+symmetry% lattice_vectors(:,3)*i3
                
                rr(:)=a(:)-ap(:)
                dr2=Dot_product(rr,rr)
                drr=Sqrt(dr2)
                !!
                If(fixzero.And.drr<rmin) Then
                   rr(:)=rmin*(/1.1_t_real,-1.2_t_real,0.9_t_real/)
                   dr2=Dot_product(rr,rr)
                   drr=Sqrt(dr2)
                Endif
                !!
                If(drr>rmax .Or. &
                     &(.Not.fixzero.And.drr <= Max(rmin,0.0_t_real))) &
                     &Cycle 
                nblmax=nblmax+1
             End Do
          End Do
       End Do
    End Do
    !!
    !!
    !!
    Call palloc('p_yb',p_yb,1_t_int,(Max(lmax,rd% max_L)+1)**2&
         &,nmodule,nsubfunc)
    !!
    !!
    Call palloc('p_lenv',p_lenv,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_lenc',p_lenc,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_siteblv',p_siteblv,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_siteblc',p_siteblc,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_Rindexblv',p_Rindexblv,1_t_int,3_t_int&
         &,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_Rindexblc',p_Rindexblc,1_t_int,3_t_int&
         &,1_t_int,nblmax,nmodule,nsubfunc)
    Call palloc('p_fblv',p_fblv,1_t_int,kohnsham% nvsmx*nblmax&
         &,nmodule,nsubfunc)
    Call palloc('p_fblc',p_fblc,1_t_int,kohnsham% ncsmx*nblmax&
         &,nmodule,nsubfunc)
    !!
    !!
    !!
    iblv=0
    iblc=0
    inblv=0
    inblc=0
    p_fblv=0.0_t_real
    p_fblc=0.0_t_real
    netsiteloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       !!
       lena=kohnsham% p_vblocklength(isa)
       lenb=kohnsham% p_vblocklength(isa)
       !!
       !!
       bmesh => basis% p_basmeshs(issa)
       rmin=bmesh% rmin
       rmax=bmesh% rmax
       a(:)=rsh(:)-structure% p_tau(:,isa)
       !!
       Do k=1,3
          rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rmax/twopi
          ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
          
          NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
          NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
       Enddo
       !!
       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Endif
       !!
       i1loop: Do i1=NR(1,1),NR(2,1)
          i2loop: Do i2=NR(1,2),NR(2,2)
             i3loop: Do i3=NR(1,3),NR(2,3)
                ap(:)= symmetry% lattice_vectors(:,1)*i1&
                     &+symmetry% lattice_vectors(:,2)*i2&
                     &+symmetry% lattice_vectors(:,3)*i3
                
                rr(:)=a(:)-ap(:)
                dr2=Dot_product(rr,rr)
                drr=Sqrt(dr2)
                !!
                If(fixzero.And.drr<rmin) Then
                   rr(:)=rmin*(/1.1_t_real,-1.2_t_real,0.9_t_real/)
                   dr2=Dot_product(rr,rr)
                   drr=Sqrt(dr2)
                Endif
                !!
                nocore=(drr > basis% p_rtfcmax(0,issa))
                !!
                If(drr>rmax .Or. (drr > basis% p_rtfvmax(0,issa)) .Or. &
                     &(.Not.fixzero.And.drr <= Max(rmin,0.0_t_real))) &
                     &Cycle i3loop
                !!
                indexa=mintpl_coefs(degree,bmesh,drr,coefa)
                !!
                Call ylms12(p_yb,rr(1),rr(2),rr(3),Max(lmax,rd% max_L))
                !!
   
                oor=1.0_t_real
                If(drr>0.0_t_real)oor=oor/drr
                texp=drr**basis% p_fgam(issa)*oor

                If(debnonet) Goto 342
                !! net from lcd
                Do j=1,rd% p_nlmp(isa)
                   il=rd% p_lmp(j,isa)
                   Do ms=1,mspin
                      rho(ms)=rho(ms)+ &
                           &Dot_product(&
                           &rd% p_lcd(indexa:indexa+degree,j,isa,ms)&
                           &,coefa(0:degree))*p_yb(il)*oor*oor
                   Enddo
                Enddo
342             Continue
                !!
                If(nocore) Goto 301
                nc=0
                Do ica=1,basis% p_nRc(issa)
                   la=basis% p_lRc(ica,issa)
                   !!
                   ff1=0.0_t_real
                   If(drr <= basis% p_rfcmax(ica,1,issa) )Then
                      ff1=Dot_product(basis% p_fc(indexa:indexa+degree&
                           &,ica,1,issa),coefa(0:degree))*texp
                   Endif
                   !!
                   il=la*la
                   Do m=1,2*la+1
                      il=il+1
                      nc=nc+1
                      p_fblc(inblc+nc)=ff1*p_yb(il)
                   Enddo
                Enddo
                iblc=iblc+1
                p_siteblc(iblc)=isa
                p_lenc(iblc)=nc
                p_Rindexblc(1:3,iblc)=(/i1,i2,i3/)
                inblc=inblc+nc
301             Continue
                nv=0
                Do iva=1,basis% p_nRv(issa)
                   la=basis% p_lRv(iva,issa)
                   !!
                   ff1=0.0_t_real
                   If(drr <= basis% p_rfvmax(iva,1,issa) )Then
                      ff1=Dot_product(basis% p_fv(indexa:indexa+degree&
                           &,iva,1,issa),coefa(0:degree))*texp
                   Endif
                   !!
                   il=la*la
                   Do m=1,2*la+1
                      il=il+1
                      nv=nv+1
                      p_fblv(inblv+nv)=ff1*p_yb(il)
                   Enddo
                   
                Enddo
                iblv=iblv+1
                p_siteblv(iblv)=isa
                p_lenv(iblv)=nv
                p_Rindexblv(1:3,iblv)=(/i1,i2,i3/)
                inblv=inblv+nv
                !!

             End Do i3loop
          End Do i2loop
       End Do i1loop
       !!
       !!
    End Do netsiteloop
    nblv=iblv
    nblc=iblc
    !!
    !Call watch(cpumesg_resolvedovl,cpu_start)
    !!
    If(debnoovl) Goto 5454
    !!
    Do ms=1,mspin
       !!
       inblva=0
       Do iblva=1,nblv
          isa=p_siteblv(iblva)
          lena=p_lenv(iblva)
          inblvb=0
          Do iblvb=1,nblv
             isb=p_siteblv(iblvb)
             lenb=p_lenv(iblvb)
             i1=-(p_Rindexblv(1,iblva)-p_Rindexblv(1,iblvb))
             i2=-(p_Rindexblv(2,iblva)-p_Rindexblv(2,iblvb))
             i3=-(p_Rindexblv(3,iblva)-p_Rindexblv(3,iblvb))
             If(   rd% Nmin<=i1 .And. i1 <=rd% Nmax.And.&
                  &rd% Nmin<=i2 .And. i2 <=rd% Nmax.And.&
                  &rd% Nmin<=i3 .And. i3 <=rd% Nmax)then
                irba=rd% p_siteOrder(isa,isb,i1,i2,i3)
             Else
                irba=0
             Endif
             If(irba/=0)Then
                !! f_a * n_{ab} * f_b^{+}
                rho(ms)=rho(ms)+Dot_product(&
                     &p_fblv(inblvb+1:inblvb+lenb)&
                     &,Matmul(p_fblv(inblva+1:inblva+lena)&
                     &,onvv(1:lena,1:lenb,isa,isb,ms,irba)))
             Endif
             inblvb=inblvb+lenb
          Enddo
          inblva=inblva+lena
       Enddo
       !!
       !!
       inblca=0
       Do iblca=1,nblc
          isa=p_siteblc(iblca)
          lenc=p_lenc(iblca)
          If(lenc==0)Cycle
          inblvb=0
          Do iblvb=1,nblv
             isb=p_siteblv(iblvb)
             lenb=p_lenv(iblvb)
             i1=-(p_Rindexblc(1,iblca)-p_Rindexblv(1,iblvb))
             i2=-(p_Rindexblc(2,iblca)-p_Rindexblv(2,iblvb))
             i3=-(p_Rindexblc(3,iblca)-p_Rindexblv(3,iblvb))
             If(   rd% Nmin<=i1 .And. i1 <=rd% Nmax.And.&
                  &rd% Nmin<=i2 .And. i2 <=rd% Nmax.And.&
                  &rd% Nmin<=i3 .And. i3 <=rd% Nmax)Then
                irba=rd% p_siteOrder(isa,isb,i1,i2,i3)
             Else
                irba=0
             Endif
             If(irba/=0)Then
                !! f_a * n_{ab} * f_b^{+}
                rho(ms)=rho(ms)+Real(Dot_product(&
                     &p_fblv(inblvb+1:inblvb+lenb)&
                     &,Matmul(p_fblc(inblca+1:inblca+lenc)&
                     &,oncv(1:lenc,1:lenb,isa,isb,ms,irba))),t_real)*2.0_t_real
             Endif
             inblvb=inblvb+lenb
          Enddo
          inblca=inblca+lenc
       Enddo
    Enddo ! ms
    !!
5454 Continue 
    !!
    !Call watch(cpumesg_resolvedovl,cpu_stop)
    !!
    Call pfree(p_yb)
    Call pfree(p_siteblv)
    Call pfree(p_Rindexblv)
    Call pfree(p_fblv)
    Call pfree(p_fblc)
    Call pfree(p_lenv)
    Call pfree(p_lenc)
    !!
    If(mspin==1)Then
       rho(2)=rho(1)*0.5_t_real
       rho(1)=rho(2)
    Endif
    Return
    !!
  End Subroutine calc_resolveddens_at_point
  !!***********************************************************************
  Subroutine prepSiteOrderArray(symmetry,structure,lattice,rd)
    !!
    !! for a given set of R-indices i1,i2,i3, which constitue a real lattice
    !! vector and two sites a and b find the index ir into p_pairs
    !! such that the corresponding P_ba=p_pairs(isb,isa) is
    !! P_ba=R+s_b-s_a
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_ResolvedDens),Intent(inout)   :: rd
    !!LOCAL
    Character(*),Parameter::nsubfunc="prepSiteOrderArray"
    Integer(t_int) :: isa,isb,ir,i1,i2,i3,nsite
    Real(t_real)   :: p(3)
    !!
    !!
    nsite=structure% nsite
    !!
    rd% Nmin=0
    rd% Nmax=0
    Do isa=1,nsite
       Do isb=1,nsite
          Do ir=1,lattice% p_npairs(isb,isa)
             p=lattice% p_pairs(:,ir,isb,isa)&
                  &-(structure% p_tau(:,isb)-structure% p_tau(:,isa))
             p=Matmul(p,symmetry% reciprocial_lattice_vectors)/twopi
             i1=Nint(p(1),t_int)
             i2=Nint(p(2),t_int)
             i3=Nint(p(3),t_int)
             rd% Nmin=Min(rd% Nmin,i1,i2,i3)
             rd% Nmax=Max(rd% Nmax,i1,i2,i3)
             !Print "(i5,i5,3f12.8)",isa,isb,p
          Enddo
       Enddo
    Enddo
    !!
    Call palloc('rd% p_siteOrder',rd% p_siteOrder&
         &,1_t_int,nsite,1_t_int,nsite,rd% Nmin,rd% Nmax&
         &,rd% Nmin,rd% Nmax,rd% Nmin,rd% Nmax,nmodule,nsubfunc)
    !!
    rd% p_siteOrder(1:nsite,1:nsite,rd% Nmin:rd% Nmax,rd% Nmin:rd% Nmax&
         &,rd% Nmin:rd% Nmax)=0_t_int
    Do isa=1,nsite
       Do isb=1,nsite
          Do ir=1,lattice% p_npairs(isb,isa)
             p=lattice% p_pairs(:,ir,isb,isa)&
                  &-(structure% p_tau(:,isb)-structure% p_tau(:,isa))
             p=Matmul(p,symmetry% reciprocial_lattice_vectors)/twopi
             i1=Nint(p(1),t_int)
             i2=Nint(p(2),t_int)
             i3=Nint(p(3),t_int)
             rd% p_siteOrder(isa,isb,i1,i2,i3)=ir
          Enddo
       Enddo
    Enddo
    !!
    Write(outfile,"('Min/Max lattice vector for pairs: ',i5,i5)")&
         &rd% Nmin,rd% Nmax
    !!
    Return
    !!
  End Subroutine prepSiteOrderArray
  !!***********************************************************************
  Logical(t_log) Function has_k_resolved(gridplot)
    !!
    Implicit None
    !!IO
    Type(T_GridPlot),Intent(in):: gridplot
    !!LOCAL
    !Character(*),Parameter::nsubfunc="has_k_resolved"
    Integer(t_int) :: ig,j
    Type(T_KResolved),Pointer::kr
    Type(T_Grid),Pointer :: g
    !!
    !!
    has_k_resolved=.True.
    !!
    If(All(gridplot% p_grids(:)% nkresolved==0).Or.gridplot% ngrids<1)Then
       has_k_resolved=.False.
       Return
    Else
       has_k_resolved=.False.
       Do ig=1,gridplot% ngrids 
          g=>gridplot% p_grids(ig)
          Do j=1,g% nkresolved
             kr=> g% p_kresolved(j)
             If(kr% nbandindices>0.Or.kr% emin< kr% emax)Then
                has_k_resolved=.True.
             Endif
          Enddo
       Enddo
    Endif
    !!
    Return
    !!
  End Function has_k_resolved
  !!***********************************************************************
  Subroutine calc_eigenvalues_at_kpoint(lattice_constants&
       &,structure,lattice,mspin,kohnsham,bzone&
       &,basis,loi,lsdau,opc,relativistic,kr,nbands,bands,cvv,ccv)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Lsdau),Intent(inout)  :: lsdau 
    Type(T_Bzone),Intent(in)     :: bzone 
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Type(T_OPC),Intent(inout)    :: opc
    Type(T_Loi),Intent(in)       :: loi
    Type(T_KResolved),Intent(inout) :: kr
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(in)      :: lattice_constants(3)
    Integer(t_int),Intent(out)   :: nbands(2)
    Integer(t_int),Intent(out)   :: bands(:,:)
    Complex(t_real),Intent(out)  :: cvv(:,:,:)
    Complex(t_real),Intent(out)  :: ccv(:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="calc_eigenvalues_at_kpoint"
    Logical(t_log) :: rel,relmag
    Integer(t_int) :: nspin,ms,ik,iv,in,iz
    Real(t_real)  :: qq(3),scale
    !!
    Complex(t_real):: ce
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Type(T_KS) :: ks
    !!***********************************************************************
    !!
    !!
    !!
    Call init_KS(ks,occu=.True.,savesp=.False.,lsdau_kintest=.False.&
         &,lsdau_energy=.False.,spinmat=.False.,need_sigma=.False.)
    !!
    !!
    !!
    !!
    !!
    !!
    !!
    nspin=mspin
    rel=(relativistic==FULL_RELATIVISTIC)
    relmag=(rel .And. mspin==2)
    if(relmag)nspin=1
    !!
    !!
    !!
    !!
    !!
    Call setup_KS(ks,mspin,bzone% nkp,structure,kohnsham,lsdau,opc&
         &,relativistic)
    !!
    !!
    scale=twopi/ lattice_constants(1_t_int)
    !!
    !!
    !!
    spinloop: Do ms=1,nspin
       !!
       !!
       qq(:)=kr% kpoint*scale
       
       Write(outfile,8800) ms,qq
8800   Format('spin= ',i2,/,'kpoint=',3(f15.8,2x))
       
       
       Call calc_at_kpoint(ks,ik,qq,ms,mspin&
            &,structure,lattice,basis,loi,kohnsham&
            &,lsdau,opc,relativistic)
       
       !!
       Call determineKresBands(kohnsham% nband,ms&
            &,ks% p_eb,kohnsham% efermi,kr,nbands,bands)
       !!
       Do in=1,nbands(ms)
          cvv(:,in,ms)=ks% p_hp(:,bands(in,ms))
       Enddo
       !!
       !!
       !!
       !!
       !!core coefficients
       Do in=1,nbands(ms)
          Do iz=1,kohnsham% ncdim
             ce=c0
             Do iv=1,kohnsham% nvdim
                ce=ce+ks% p_scrawv(iv,iz)*ks% p_hp(iv,bands(in,ms))
             Enddo ! iv
             ccv(iz,in,ms)=-ce
          Enddo !iz
       Enddo !in
       
       
    End Do spinloop                     ! ms
    !!
    !!
    !!
    !!
    Call clean_KS(ks,lsdau,opc)
    !!
    !!
    !!
    Return
    !!
    !!
  End Subroutine calc_eigenvalues_at_kpoint
  !!***********************************************************************
  Subroutine determineKresBands(nband,ms,eb,efermi,kr,nbands,bands)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    ::ms,nband
    Type(T_KResolved),Intent(inout) :: kr
    Real(t_real),Intent(in)      :: eb(:),efermi
    Integer(t_int),Intent(out)   :: nbands(2)
    Integer(t_int),Intent(out)   :: bands(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="determineKresBands"
    Integer(t_int) :: in,iz,n
    Real(t_real)   :: e
    !!
    If(kr% nbandindices>0)Then
       !!
       in=0
       !!
       Do n=1,kr% nbandindices
          If(kr% p_bandindices(n)>nband)Then
             !!
3            Format("'Band index ",i8," exceeds max number of bands ",i8,",',/&
                  &,' which is determined by the orbitals and might be&
                  & restricted by',/,' the ''Occupied bands'' option! '")
             Write(errbuf,3) kr% p_bandindices(n),nband
             Call errmsg(nmodule,nsubfunc,E_information,msg=errbuf)
          Else
             in=in+1
             bands(in,ms)=kr% p_bandindices(n)
          Endif
       Enddo
       !!
       nbands(ms)=in
       !!
    Else
       nbands(ms)=0
       Do iz=1,nband
          e=(eb(iz)-efermi)*hatoev
          If(kr% emin<=e .And. e<=kr% emax)Then
             nbands(ms)=nbands(ms)+1
             bands(nbands(ms),ms)=iz
          Endif
       Enddo
       !!
       !!
       !!
    Endif

    !!
    !!
    Return
    !!
  End Subroutine determineKresBands
  !!***********************************************************************
  Subroutine calc_psi_at_point(ir,rsh,kpoint,nbands,cvv,ccv,psi&
       &,mspin,symmetry,structure,basis&
       &,nvdim,ncdim,vblockstart,cblockstart&
       &,fixzero,degree,relativistic,p_yb,p_ss)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Logical(t_log),Intent(in)    :: fixzero
    Integer(t_int),Intent(in)    :: mspin,degree,relativistic,ir
    Integer(t_int),Intent(in)    :: nvdim,ncdim
    Integer(t_int),Intent(in)    :: vblockstart(:)
    Integer(t_int),Intent(in)    :: cblockstart(:)
    Integer(t_int),Intent(in)    :: nbands(2)
    Real(t_real),Intent(in)      :: kpoint(3)
    Complex(t_real),Intent(in)   :: cvv(:,:,:)
    Complex(t_real),Intent(in)   :: ccv(:,:,:)
    Real(t_real),Intent(inout)   :: p_yb(:)
    Complex(t_real),Intent(inout):: p_ss(2,2*(basis% lmx+1),-(basis% lmx+1):(basis% lmx+1))
    Complex(t_real),Intent(inout):: psi(:,:,:,:)
    Real(t_real),Intent(in)      :: rsh(3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_psi_at_point"
    Logical(t_log) :: nocore,rel,relmag
    Integer(t_int) :: isa,k,NR(2,3),i1,i2,i3,indexa,issa,iva,deg,il,kapa
    Integer(t_int) :: iz,istv,istc,in,ms,nspin,ncomp,icomp,ica,degja
    Real(t_real)   :: a(3),rg,ag,rmax,rr(3),ap(3),dr2,drr,rmin,oor
    Real(t_real)   :: z,y,phi,kp(3)
    Real(t_real)   :: coefa(0:degree)
    Complex(t_real):: phase,cff,texphase,cfac
    Complex(t_real),Pointer:: cfv(:,:),cfc(:,:)
    Complex(t_real),Pointer:: cfvr(:,:,:),cfcr(:,:,:)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Type(T_Mesh),Pointer :: bmesh
    !!
    Nullify(cfv,cfc,cfvr,cfcr)
    !!
    Call palloc('cfv',cfv,1_t_int,nvdim,1_t_int,basis% ncomp&
         &,nmodule,nsubfunc)
    Call palloc('cfc',cfc,1_t_int,ncdim,1_t_int,basis% ncomp&
         &,nmodule,nsubfunc)

    Call palloc('cfvr',cfvr,1_t_int,nvdim,1_t_int,2_t_int&
         &,1_t_int,basis% ncomp,nmodule,nsubfunc)
    Call palloc('cfcr',cfcr,1_t_int,ncdim,1_t_int,2_t_int&
         &,1_t_int,basis% ncomp,nmodule,nsubfunc)
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    relmag=(rel.And.(mspin==2))
    nspin=mspin
    If(rel)nspin=1
    ncomp=basis% ncomp
    !!
    !!
    kp=kpoint
    If(structure% str_type == STR_TYPE_MOLECULE)Then
       kp=0.0_t_real
    Endif
    !!
    !!
    cfv=c0
    cfc=c0
    cfvr=c0
    cfcr=c0
    !!
    isaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       bmesh => basis% p_basmeshs(issa)
       rmin=bmesh% rmin
       rmax=bmesh% rmax
       istv=vblockstart(isa)-1
       istc=cblockstart(isa)-1
       !!
       a(:)=rsh(:)-structure% p_tau(:,isa)
       !!
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Else
          Do k=1,3
             rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rmax/twopi
             ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
             
             NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
             NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
          Enddo
       Endif
       !!
       R1loop: Do i1=NR(1,1),NR(2,1)
          R2loop: Do i2=NR(1,2),NR(2,2)
             R3loop: Do i3=NR(1,3),NR(2,3)
                ap(:)= symmetry% lattice_vectors(:,1)*i1&
                     &+symmetry% lattice_vectors(:,2)*i2&
                     &+symmetry% lattice_vectors(:,3)*i3
                rr(:)=a(:)-ap(:)
                dr2=Dot_product(rr,rr)
                drr=Sqrt(dr2)
                If(fixzero.And.drr<rmin) Then
                   rr(:)=rmin*(/1.1_t_real,-1.2_t_real,0.9_t_real/)
                   dr2=Dot_product(rr,rr)
                   drr=Sqrt(dr2)
                Endif
                !!
                phase=Exp(ci*Dot_product(kp,ap+structure% p_tau(:,isa)))
                !!
                nocore=(drr > basis% p_rtfcmax(0,issa))
                !!
                If(drr>rmax .Or. (drr > basis% p_rtfvmax(0,issa)) .Or. &
                     &((.Not.fixzero).And.drr <= Max(rmin,0.0_t_real))) &
                     &Cycle R3loop
                !!
                indexa=mintpl_coefs(degree,bmesh,drr,coefa)
                !!
                If(.Not.rel)Then
                   Call ylms12(p_yb,rr(1),rr(2),rr(3),basis% lmx)
                Else
                   z=rr(3)/drr
                   y=Sqrt(Max(0.0_t_real,Min(1.0_t_real,1.0_t_real-z*z)))
                   If(y==0.0_t_real)Then
                      phi=0.0_t_real
                   Else
                      phi=Atan2(rr(2),rr(1))
                   Endif
                   Call sphsp(basis% lmx,p_ss,basis% lmx,y,z,phi)
                Endif
                !!
                oor=1.0_t_real
                If(drr>0.0_t_real)oor=1.0_t_real/drr
                texphase=drr**basis% p_fgam(issa)*oor*phase
                !!
                !!
                !!
                !!
                relif2: If(rel)Then
                   !!
                   If(.Not.nocore)Then
                      relcicomploop: Do icomp=1,ncomp
                         cfac=texphase
                         If(icomp==2) cfac=ci*texphase
                         iz=istc
                         relicaloop: Do ica=1,basis% p_nRc(issa)
                            kapa=basis% p_kapRc(ica,issa)
                            If(icomp==2) kapa=-kapa
                            degja=basis% p_degRc(ica,issa)
                            If(drr <= basis% p_rfcmax(ica,icomp,issa) )Then
                               cff=Dot_product(basis% p_fc(indexa:indexa&
                                    &+degree,ica,icomp,issa)&
                                    &,coefa(0:degree))*cfac
                               Do ms=1,2
                                  cfcr(iz+1:iz+degja,ms,icomp)&
                                       &=cfcr(iz+1:iz+degja,ms,icomp)&
                                       & +cff*p_ss(ms,1:degja,kapa)
                               Enddo
                            Endif
                            !!
                            iz=iz+degja
                            !!
                         Enddo relicaloop
                         !!
                      End Do relcicomploop
                   Endif
                   relvicomploop: Do icomp=1,ncomp
                      cfac=texphase
                      If(icomp==2) cfac=ci*texphase
                      iz=istv
                      relivaloop: Do iva=1,basis% p_nRv(issa)
                         kapa=basis% p_kapRv(iva,issa)
                         If(icomp==2) kapa=-kapa
                         degja=basis% p_degRv(iva,issa)
                         If(drr <= basis% p_rfvmax(iva,icomp,issa) )Then
                            cff=Dot_product(basis% p_fv(indexa:indexa&
                                 &+degree,iva,icomp,issa)&
                                 &,coefa(0:degree))*cfac
                            Do ms=1,2
                               cfvr(iz+1:iz+degja,ms,icomp)&
                                    &=cfvr(iz+1:iz+degja,ms,icomp)&
                                    & +cff*p_ss(ms,1:degja,kapa)
                            Enddo
                         Endif
                         !!
                         iz=iz+degja
                         !!
                      Enddo relivaloop
                      !!
                   End Do relvicomploop
                Else relif2
                   !!
                   If(.Not.nocore)Then
                      cicomploop: Do icomp=1,ncomp
                         iz=istc
                         Do ica=1,basis% p_nRc(issa)
                            il=basis% p_lRc(ica,issa)**2
                            deg=basis% p_degRc(ica,issa)
                            If(drr <= basis% p_rfcmax(ica,icomp,issa) )Then
                               cff=Dot_product(basis% p_fc(indexa:indexa&
                                    &+degree,ica,icomp,issa)&
                                    &,coefa(0:degree))*texphase
                               cfc(iz+1:iz+deg,icomp)=cfc(iz+1:iz+deg,icomp)&
                                    &+cff*p_yb(il+1:il+deg)
                            Endif
                            !!
                            iz=iz+deg
                            !!
                         Enddo
                         !!
                      Enddo cicomploop
                   Endif
                   vicomploop: Do icomp=1,ncomp
                      iz=istv
                      Do iva=1,basis% p_nRv(issa)
                         il=basis% p_lRv(iva,issa)**2
                         deg=basis% p_degRv(iva,issa)
                         If(drr <= basis% p_rfvmax(iva,icomp,issa) )Then
                            cff=Dot_product(basis% p_fv(indexa:indexa&
                                 &+degree,iva,icomp,issa)&
                                 &,coefa(0:degree))*texphase
                            cfv(iz+1:iz+deg,icomp)=cfv(iz+1:iz+deg,icomp)&
                                 &+cff*p_yb(il+1:il+deg)
                         Endif
                         !!
                         iz=iz+deg
                         !!
                      Enddo
                      !!
                   Enddo vicomploop
                Endif relif2
                !!
             End Do R3loop
          End Do R2loop
       End Do R1loop
       
    Enddo isaloop
    !!
    If(rel)Then
       Do icomp=1,ncomp
          Do ms=1,2
             Do in=1,nbands(1)
                psi(in,ms,icomp,ir)=& 
                     &psi(in,ms,icomp,ir)&
                     &+Sum(cfvr(1:nvdim,ms,icomp)*cvv(1:nvdim,in,1))&
                     &+Sum(cfcr(1:ncdim,ms,icomp)*ccv(1:ncdim,in,1))
             Enddo
          Enddo
       Enddo
       
    Else
       Do icomp=1,ncomp
          Do ms=1,nspin
             Do in=1,nbands(ms)
                psi(in,ms,icomp,ir)=&
                     &psi(in,ms,icomp,ir)&
                     &+Sum(cfv(1:nvdim,icomp)*cvv(1:nvdim,in,ms))&
                     &+Sum(cfc(1:ncdim,icomp)*ccv(1:ncdim,in,ms))
             Enddo
          Enddo
       Enddo
    Endif
    !!
    Call pfree(cfv)
    Call pfree(cfc)
    Call pfree(cfvr)
    Call pfree(cfcr)
    !!
    Return
    !!
  End Subroutine calc_psi_at_point
  !!***********************************************************************


  
End Module plotdens
