Module points
  !!
  Use precisiondefs
  Use errors
  Use constants
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="POINTS"  
  !!
  Public :: poi6i,poi12i,poi8i,poi24i1,poi24i2,poi240i,poi48i&
       &,hex2i,hex60i,hex61i,hex122i,hex121i,hex120i,hex24i
  !!***********************************************************************
Contains
   !!***********************************************************************
   !!
   !! irreducible spherical mesh for each pointgroup
   !!
   !!
   !!***********************************************************************
   !!  begin of cubic definitions
   !!***********************************************************************
   !!
   Subroutine poi6i(a,wf,ng,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  :: a(3,6),wf(6)
     !!LOCAL 
     Character(*),Parameter::nsubfunc="poi6i"
     Real(t_real),Parameter :: wpnt(4)=(/&
          &6.0_t_real,4.0_t_real,2.0_t_real,1.0_t_real/)
     Real(t_real) :: pnt(3,6)
     !!
     pnt=Reshape(source=(/ &
          &  1.0_t_real, 0.0_t_real, 0.0_t_real&
          &, 0.0_t_real, 0.0_t_real, 1.0_t_real&
          &, 0.0_t_real, 0.0_t_real,-1.0_t_real&
          &, 0.0_t_real, 1.0_t_real, 0.0_t_real&
          &, 0.0_t_real,-1.0_t_real, 0.0_t_real&
          &,-1.0_t_real, 0.0_t_real, 0.0_t_real&
          &/),shape=(/3,6/))
     !!
     Select Case(ipgr)
     Case (37_t_int,36_t_int,35_t_int,34_t_int,33_t_int)
        !! OH,Td,O,Th,T
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (16_t_int,15_t_int,14_t_int,12_t_int,11_t_int,10_t_int)
        !! D4h,D2d(b),D2d(a),D4,C4h,S4
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int:3_t_int)
        ng=ng+2_t_int
     Case (13_t_int,9_t_int)
        !! C4v,C4
        a(:,1_t_int:3_t_int)=pnt(:,1_t_int:3_t_int)
        wf(1_t_int:3_t_int) =wpnt((/2_t_int,4_t_int,4_t_int/))
        ng=ng+3_t_int
     Case (8_t_int,6_t_int,5_t_int,2_t_int)
        !! D2h,D2,C2h,Ci
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int/))
        wf(1_t_int:3_t_int) =wpnt(3_t_int)
        ng=ng+3_t_int
     Case (7_t_int)
        !! C2v
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,3_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/3_t_int,4_t_int,3_t_int,4_t_int/))
        ng=ng+4_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:5_t_int)=pnt(:,(/1_t_int,6_t_int,4_t_int,2_t_int,3_t_int/))
        wf(1_t_int:5_t_int) =wpnt((/4_t_int,4_t_int,3_t_int,4_t_int,4_t_int/))
        ng=ng+5_t_int
     Case (3_t_int)
        !! C2
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,5_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/3_t_int,3_t_int,4_t_int,4_t_int/))
        ng=ng+4_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,6_t_int,3_t_int,5_t_int/))
        wf(1_t_int:6_t_int) =wpnt(4_t_int)
        ng=ng+6_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     
     !!     
   End Subroutine poi6i
   !!***********************************************************************
   !!     
   Subroutine poi12i(a,wf,ng,ipgr)
     !!     
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  ::  a(3,12),wf(12)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi12i"
     !!
     Real(t_real),Parameter :: wpnt(5)=(/&
          &12.0_t_real,8.0_t_real,4.0_t_real,2.0_t_real,1.0_t_real/)
     Real(t_real) :: pnt(3,12)
     !!
     pnt=Reshape(source=(/ &
          &sq2o2,sq2o2,0.0_t_real& !1
          &,sq2o2,0.0_t_real,sq2o2&
          &,sq2o2,0.0_t_real,-sq2o2&
          &,0.0_t_real,sq2o2,sq2o2&
          &,0.0_t_real,sq2o2,-sq2o2& !5
          &,-sq2o2,sq2o2,0.0_t_real& !6
          &,-sq2o2,0.0_t_real,sq2o2& !7
          &,-sq2o2,0.0_t_real,-sq2o2& !8
          &,sq2o2,-sq2o2,0.0_t_real& !9
          &,0.0_t_real,-sq2o2,sq2o2& !10
          &,-sq2o2,-sq2o2,0.0_t_real& !11
          &,0.0_t_real,-sq2o2,-sq2o2&!12
          &/),shape=(/3,12/))
     
     Select Case(ipgr)
     Case (37_t_int,36_t_int,35_t_int,34_t_int,33_t_int)
        !! OH,Td,O
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (16_t_int,14_t_int,12_t_int,11_t_int)
        !! D4h,D2d(a),D4,C4h
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt((/3_t_int,2_t_int/))
        ng=ng+2_t_int
     Case (15_t_int,13_t_int,10_t_int,9_t_int)
        !! D2d(b),C4v,S4,C4
        a(:,1_t_int:3_t_int)=pnt(:,1_t_int:3_t_int)
        wf(1_t_int:3_t_int) =wpnt(3_t_int)
        ng=ng+3_t_int
     Case (8_t_int,6_t_int)
        !! D2h,D2
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int/))
        wf(1_t_int:3_t_int) =wpnt(3_t_int)
        ng=ng+3_t_int
     Case (7_t_int)
        !! C2v
        a(:,1_t_int:5_t_int)=pnt(:,1_t_int:5_t_int)
        wf(2_t_int:5_t_int) =wpnt(4_t_int)
        wf(1_t_int) =wpnt(3_t_int)
        ng=ng+5_t_int
     Case (5_t_int)
        !! C2h
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,3_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/3_t_int,4_t_int,3_t_int,4_t_int/))
        ng=ng+4_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:8_t_int)=pnt(:,(/1_t_int,6_t_int,4_t_int,5_t_int,2_t_int,7_t_int,3_t_int,8_t_int/))
        wf(1_t_int:4_t_int) =wpnt(4_t_int)
        wf(5_t_int:8_t_int) =wpnt(5_t_int)
        ng=ng+8_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,9_t_int,10_t_int,7_t_int/))
        wf(1_t_int:6_t_int) =wpnt(4_t_int)
        ng=ng+6_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,2_t_int,4_t_int,9_t_int&
             &,10_t_int,7_t_int,11_t_int,8_t_int,12_t_int,6_t_int&
             &,5_t_int,3_t_int/))
        wf(1_t_int:12_t_int) =wpnt(5_t_int)
        ng=ng+12_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     !!
   End Subroutine poi12i
   !!***********************************************************************
   !!
   Subroutine poi8i(a,wf,ng,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  :: a(3,8),wf(8)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi8i"
     Real(t_real) :: pnt(3,8)
     !!
     pnt=Reshape(source=(/ &
          &sq3o3,sq3o3,sq3o3& !1
          &,sq3o3,sq3o3,-sq3o3& !2
          &,-sq3o3,sq3o3,sq3o3& !3
          &,-sq3o3,sq3o3,-sq3o3&!4
          &,sq3o3,-sq3o3,sq3o3& !5
          &,-sq3o3,-sq3o3,sq3o3& !6
          &,-sq3o3,-sq3o3,-sq3o3& !7
          &,sq3o3,-sq3o3,-sq3o3& !8
          &/),shape=(/3,8/))
     
     !!
     
     Select Case(ipgr)
     Case (37_t_int,35_t_int,34_t_int,16_t_int,15_t_int,12_t_int,11_t_int,8_t_int)
        !! OH,O,Th,D4h,D2d(b),D4,C4h,D2h
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =8.0_t_real
        ng=ng+1_t_int
     Case (36_t_int,33_t_int,14_t_int,13_t_int,10_t_int,9_t_int,7_t_int,6_t_int,5_t_int)
        !! Td,T,D2d(a),C4v,S4,C4,C2v,D2,C2h
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =4.0_t_real
        ng=ng+2_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:4_t_int)=pnt(:,1_t_int:4_t_int)
        wf(1_t_int:4_t_int) =2.0_t_real
        ng=ng+4_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,6_t_int/))
        wf(1_t_int:4_t_int) =2.0_t_real
        ng=ng+4_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:8_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,6_t_int,7_t_int,8_t_int,4_t_int,2_t_int/))
        wf(1_t_int:8_t_int) =1.0_t_real
        ng=ng+8_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select

     !!     
   End Subroutine poi8i
   !!     
   !!***********************************************************************
   !!
   Subroutine poi24i1(a,wf,ng,x,y,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y
     Real(t_real),Intent(out)  :: a(3,24),wf(24)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi24i1"
     Real(t_real),Parameter :: wpnt(6)=(/&
          &24.0_t_real,12.0_t_real,16.0_t_real,8.0_t_real,4.0_t_real&
          &,2.0_t_real/)
     Real(t_real) :: pnt(3,24)
     !!
     pnt=Reshape(source=(/ &
          &  x, y, y &!1
          &, x, y,-y &!2
          &, y, y, x &!3
          &, y, y,-x &!4
          &, y, x, y &!5
          &, y, x,-y &!6
          &,-x, y, y &!7
          &,-x, y,-y &!8
          &,-y, y, x &!9
          &,-y, y,-x &!10
          &,-y, x, y &!11
          &,-y, x,-y &!12
          &, y,-x, y &!13
          &, y,-y, x &!14
          &, x,-y, y &!15
          &,-x,-y, y &!16
          &,-y,-y, x &!17
          &,-y,-x, y &!18
          &,-x,-y,-y &!19
          &,-y,-y,-x &!20
          &,-y,-x,-y &!21
          &, y,-x,-y &!22
          &, y,-y,-x &!23
          &, x,-y,-y &!24
          &/),shape=(/3,24/))
     !!
     Select Case(ipgr)
     Case (37_t_int,35_t_int,34_t_int)
        !! OH,O,Th
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (36_t_int,33_t_int)
        !! Td,T
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (16_t_int)
        !! D4h
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(3_t_int:4_t_int)
        ng=ng+2_t_int
     Case (15_t_int,12_t_int)
        !! D2d(b),D4
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,3_t_int,2_t_int/))
        wf(1_t_int:3_t_int) =wpnt(4_t_int)
        ng=ng+3_t_int
     Case (14_t_int,13_t_int)
        !! D2d(a),C4v
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,2_t_int,4_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/4_t_int,5_t_int,4_t_int,5_t_int/))
        ng=ng+4_t_int
     Case (11_t_int,8_t_int)
        !! C4h,D2h
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int/))
        wf(1_t_int:3_t_int) =wpnt(4_t_int)
        ng=ng+3_t_int
     Case (10_t_int,9_t_int,7_t_int,6_t_int,5_t_int)
        !! S4,C4,C2v,D2,C2h
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,3_t_int,2_t_int,4_t_int,5_t_int,6_t_int/))
        wf(1_t_int:6_t_int) =wpnt(5_t_int) 
        ng=ng+6_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,3_t_int,2_t_int,4_t_int&
             &,5_t_int,6_t_int,7_t_int,9_t_int,8_t_int,10_t_int&
             &,11_t_int,12_t_int/))
        wf(1_t_int:12_t_int) =wpnt(6_t_int) 
        ng=ng+12_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int&
             &, 13_t_int,14_t_int,15_t_int,11_t_int,9_t_int&
             &,7_t_int,16_t_int,17_t_int,18_t_int /))
        wf(1_t_int:12_t_int) =wpnt(6_t_int) 
        ng=ng+12_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:24_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int, 13_t_int&
             &,14_t_int,15_t_int,11_t_int,9_t_int,7_t_int,16_t_int&
             &,17_t_int,18_t_int &
             &,19_t_int,20_t_int,21_t_int,12_t_int,10_t_int,8_t_int&
             &,22_t_int,23_t_int,24_t_int,2_t_int,4_t_int,6_t_int/))
        wf(1_t_int:24_t_int) =1.0_t_real
        ng=ng+24_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select

     !!     
   End Subroutine poi24i1
   !!     
   !!***********************************************************************
   !!     
   Subroutine poi24i2(a,wf,ng,x,y,ipgr)
     !!     
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y
     Real(t_real),Intent(out)  :: a(3,24),wf(24)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi24i2"
     Real(t_real),parameter :: wpnt(6)=(/&
          & 24.0_t_real,12.0_t_real,16.0_t_real,8.0_t_real,4.0_t_real&
          &,2.0_t_real/)
     Real(t_real) :: pnt(3,24)
     !!
     pnt=Reshape(source=(/ &
          &  x, x, y & !1
          &, x, x,-y & !2
          &, x, y, x & !3
          &, x, y,-x & !4
          &, y, x, x & !5
          &, y, x,-x & !6
          &,-x, x, y & !7
          &,-x, x,-y & !8
          &,-x, y, x & !9
          &,-x, y,-x & !10
          &,-y, x, x & !11
          &,-y, x,-x & !12
          &, x,-x, y & !13 
          &, y,-x, x & !14
          &, x,-y, x & !15 
          &,-x,-x, y & !16
          &,-x,-y, x & !17
          &,-y,-x, x & !18
          &,-x,-x,-y & !19
          &,-x,-y,-x & !20
          &,-y,-x,-x & !21
          &, x,-x,-y & !22
          &, y,-x,-x & !23
          &, x,-y,-x & !24
          &/),shape=(/3,24/))
     !!
     Select Case(ipgr)
     Case (37_t_int,35_t_int,34_t_int)
        !! OH,O,Th
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (36_t_int,33_t_int)
        !! Td,T
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (16_t_int)
        !! D4h
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt((/4_t_int,3_t_int/)) 
        ng=ng+2_t_int
     Case (15_t_int,12_t_int)
        !! D2d(b),D4
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int/))
        wf(1_t_int:3_t_int) =wpnt(4_t_int) 
        ng=ng+3_t_int
     Case (14_t_int,13_t_int)
        !! D2d(a),C4v
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,2_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/5_t_int,4_t_int,4_t_int,5_t_int/)) 
        ng=ng+4_t_int
     Case (11_t_int,8_t_int)
        !! C4h,D2h
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int/))
        wf(1_t_int:3_t_int) =wpnt(4_t_int) 
        ng=ng+3_t_int
     Case (10_t_int,9_t_int,7_t_int,6_t_int,5_t_int)
        !! S4,C4,C2v,D2,C2h
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,2_t_int&
             &,5_t_int,6_t_int/))
        wf(1_t_int:6_t_int) =wpnt(5_t_int) 
        ng=ng+6_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,2_t_int&
             &,5_t_int,6_t_int,7_t_int,9_t_int,10_t_int,8_t_int&
             &,11_t_int,12_t_int/))
        wf(1_t_int:12_t_int) =wpnt(6_t_int) 
        ng=ng+12_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,13_t_int&
             &,14_t_int,15_t_int,7_t_int,11_t_int,9_t_int,16_t_int&
             &,17_t_int,18_t_int/))
        wf(1_t_int:12_t_int) =wpnt(6_t_int) 
        ng=ng+12_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:24_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,13_t_int&
             &,14_t_int,15_t_int,7_t_int,11_t_int,9_t_int,16_t_int&
             &,17_t_int,18_t_int&
             &        ,19_t_int,20_t_int,21_t_int,8_t_int,12_t_int&
             &,10_t_int,22_t_int,23_t_int,24_t_int,2_t_int,4_t_int,6_t_int/))
        wf(1_t_int:24_t_int) =1.0_t_real
        ng=ng+24_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select

     !!     
   End Subroutine poi24i2
   !!
   !!***********************************************************************
   !!
   Subroutine poi240i(a,wf,ng,x,y,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y
     Real(t_real),Intent(out)  :: a(3,24),wf(24)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi240i"
     Real(t_real),Parameter :: wpnt(6)=(/&
          &24.0_t_real,12.0_t_real,8.0_t_real,4.0_t_real,2.0_t_real&
          &,1.0_t_real/)
     Real(t_real) :: pnt(3,24)
     !!
     pnt=Reshape(source=(/ &
          & x,y,0.0_t_real& !1
          &,x,0.0_t_real,y& !2
          &,y,0.0_t_real,x& !3
          &,x,0.0_t_real,-y& !4
          &,y,0.0_t_real,-x&!5
          &,y,x,0.0_t_real & !6
          &,0.0_t_real,x,y& !7
          &,0.0_t_real,y,x& !8
          &,0.0_t_real,x,-y& !9
          &,0.0_t_real,y,-x& !10
          &,-x,y,0.0_t_real& !11  
          &,-y,x,0.0_t_real& !12 
          &,-x,0.0_t_real,y& !13 
          &,-y,0.0_t_real,x& !14 
          &,-x,0.0_t_real,-y&   !15
          &,-y,0.0_t_real,-x&   !16
          &,x,-y,0.0_t_real&  !17
          &,y,-x,0.0_t_real& !18 
          &,0.0_t_real,-x,y& !19
          &,0.0_t_real,-y,x& !20
          &,-x,-y,0.0_t_real&  !21
          &,-y,-x,0.0_t_real & !22
          &,0.0_t_real,-x,-y&  !23
          &,0.0_t_real,-y,-x&  !24
          &/),shape=(/3,24/))
     !!Print *,pnt

     Select Case(ipgr)
     Case (37_t_int,36_t_int,35_t_int)
        !! OH,Td,O
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (34_t_int,33_t_int)
        !! Th,T
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (16_t_int,14_t_int,12_t_int)
        !! D4h,D2d(a),D4
        a(:,1_t_int:3_t_int)=pnt(:,1_t_int:3_t_int)
        wf(1_t_int:3_t_int) =wpnt(3_t_int)
        ng=ng+3_t_int
     Case (15_t_int,13_t_int)
        !! D2d(b),C4v
        a(:,1_t_int:5_t_int)=pnt(:,1_t_int:5_t_int)
        wf(2_t_int:5_t_int) =wpnt(4_t_int)
        wf(1_t_int) =wpnt(3_t_int)
        ng=ng+5_t_int
     Case (11_t_int)
        !! C4h
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,2_t_int,3_t_int,6_t_int/))
        wf(1_t_int:4_t_int) =wpnt((/4_t_int,3_t_int,3_t_int,4_t_int/))
        ng=ng+4_t_int
     Case (10_t_int,9_t_int)
        !! S4,C4
        a(:,1_t_int:6_t_int)=pnt(:,1_t_int:6_t_int)
        wf(1_t_int:6_t_int) =wpnt(4_t_int)
        ng=ng+6_t_int
     Case (8_t_int,6_t_int)
        !! D2h,D2,D2
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,2_t_int,3_t_int,6_t_int,7_t_int,8_t_int/))
        wf(1_t_int:6_t_int) =wpnt(4_t_int)
        ng=ng+6_t_int
     Case (7_t_int)
        !! C2v
        a(:,1_t_int:10_t_int)=pnt(:,1_t_int:10_t_int)
        wf(1_t_int:10_t_int) =wpnt(5_t_int)
        wf(1_t_int) =wpnt(4_t_int)
        wf(6_t_int) =wpnt(4_t_int)
        ng=ng+10_t_int
     Case (5_t_int)
        !! C2h
        a(:,1_t_int:8_t_int)=pnt(:,(/1_t_int,2_t_int,3_t_int,6_t_int,7_t_int,8_t_int,4_t_int,5_t_int/))
        wf(2_t_int:8_t_int) =wpnt(5_t_int)
        wf(1_t_int)=wpnt(4_t_int)
        wf(4_t_int:6_t_int)=wpnt(4_t_int) 
        ng=ng+8_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:16_t_int)=pnt(:,(/1_t_int,6_t_int,7_t_int,8_t_int&
             &,11_t_int,12_t_int,9_t_int,10_t_int, 2_t_int,3_t_int&
             &,13_t_int,14_t_int,4_t_int,5_t_int,15_t_int,16_t_int/))
        wf(1_t_int:8_t_int)  =wpnt(5_t_int)
        wf(9_t_int:16_t_int) =wpnt(6_t_int)
        ng=ng+16_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,2_t_int,3_t_int,6_t_int&
             &,7_t_int,8_t_int,17_t_int,18_t_int,19_t_int,20_t_int&
             &,13_t_int,14_t_int/))
        wf(1_t_int:12_t_int) =wpnt(5_t_int)
        ng=ng+12_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:24_t_int)=pnt(:,(/1_t_int,2_t_int,3_t_int,6_t_int&
             &,7_t_int,8_t_int,17_t_int,18_t_int,19_t_int,20_t_int&
             &,13_t_int,14_t_int&
             &,21_t_int,15_t_int,16_t_int,22_t_int,23_t_int&
             &,24_t_int,11_t_int,12_t_int,9_t_int,10_t_int,4_t_int,5_t_int/))
        wf(1_t_int:24_t_int) =1.0_t_real
        ng=ng+24_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select

     !!     
   End Subroutine poi240i
   !!
   !!***********************************************************************
   !!
   Subroutine poi48i(a,wf,ng,x,y,z,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y,z
     Real(t_real),Intent(out)  :: a(3,48),wf(48)
     !!LOCAL
     Character(*),Parameter::nsubfunc="poi48i"
     Real(t_real),Parameter :: wpnt(7)=(/&
          &48.0_t_real,24.0_t_real,12.0_t_real,16.0_t_real,8.0_t_real&
          &,4.0_t_real,2.0_t_real/)
     Real(t_real) :: pnt(3,48)
     !!
     pnt=Reshape(source=(/ &
          &  x, y, z , x, y,-z , x, z, y , x, z,-y , y, z, x , y, z,-x & 
          &, y, x, z , z, x, y , z, y, x , y, x,-z & !10 -34
          &, z, x,-y & !11 -35
          &, z, y,-x & !12 -36
          &,-x, y, z & !13 -46
          &,-x, y,-z & !14 -28
          &,-x, z, y & !15 -47
          &,-x, z,-y & !16 -29
          &,-y, z, x & !17 -48
          &,-y, z,-x & !18 -30
          &,-y, x, z & !19 
          &,-z, x, y & !20
          &,-z, y, x & !21
          &,-y, x,-z & !22
          &,-z, x,-y & !23 -26
          &,-z, y,-x & !24 -27
          &, y,-x, z & !25 -22
          &, z,-x, y & !26
          &, z,-y, x & !27
          &, x,-y, z & !28
          &, x,-z, y & !29
          &, y,-z, x & !30
          &,-x,-y, z & !31 -2
          &,-x,-z, y & !32 -4
          &,-y,-z, x & !33 -6
          &,-y,-x, z & !34-10
          &,-z,-x, y & !35-11
          &,-z,-y, x & !36-12
          &,-x,-y,-z & !37
          &,-x,-z,-y & !38
          &,-y,-z,-x & !39
          &,-y,-x,-z ,-z,-x,-y ,-z,-y,-x , y,-x,-z & !43
          &, z,-x,-y , z,-y,-x , x,-y,-z , x,-z,-y , y,-z,-x & !48 
          &/),shape=(/3,48/))

     Select Case(ipgr)
     Case (37_t_int)
        !! OH
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (36_t_int,35_t_int)
        !! Td,O
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (34_t_int)
        !! Th
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (33_t_int)
        !! T
        a(:,1_t_int:4_t_int)=pnt(:,1_t_int:4_t_int)
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (16_t_int)
        !! D4h
        a(:,1_t_int:3_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int/))
        wf(1_t_int:3_t_int) =wpnt(4_t_int)
        ng=ng+3_t_int
     Case (15_t_int,14_t_int,13_t_int,12_t_int)
        !! D2d(b),D2d(a),C4v,D4
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,2_t_int,4_t_int,6_t_int/))
        wf(1_t_int:6_t_int) =wpnt(5_t_int)
        ng=ng+6_t_int
     Case (11_t_int,8_t_int)
        !! C4h,D2h
        a(:,1_t_int:6_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,7_t_int,8_t_int,9_t_int/))
        wf(1_t_int:6_t_int) =wpnt(5_t_int)
        ng=ng+6_t_int
     Case (10_t_int,9_t_int,7_t_int,6_t_int,5_t_int)
        !! S4,C4,C2v,D2,C2h
        a(:,1_t_int:12_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,2_t_int&
             &,4_t_int,6_t_int,7_t_int,8_t_int,9_t_int,10_t_int,11_t_int&
             &,12_t_int/))
        wf(1_t_int:12_t_int) =wpnt(6_t_int)
        ng=ng+12_t_int
     Case (4_t_int)
        !! Cs
        a(:,1_t_int:24_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,2_t_int&
             &,4_t_int,6_t_int,7_t_int,8_t_int,9_t_int,10_t_int,11_t_int&
             &,12_t_int&
             &,13_t_int,15_t_int,17_t_int,14_t_int,16_t_int,18_t_int&
             &,19_t_int,20_t_int,21_t_int,22_t_int,23_t_int,24_t_int/))
        wf(1_t_int:24_t_int) =wpnt(7_t_int)
        ng=ng+24_t_int
     Case (3_t_int,2_t_int)
        !! C2,Ci
        a(:,1_t_int:24_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,7_t_int&
             &,8_t_int,9_t_int,25_t_int,26_t_int,27_t_int,28_t_int&
             &,29_t_int,30_t_int&
             &,19_t_int,20_t_int,21_t_int,13_t_int,15_t_int,17_t_int&
             &,31_t_int,32_t_int,33_t_int,34_t_int,35_t_int,36_t_int/))
        wf(1_t_int:24_t_int) =wpnt(7_t_int)
        ng=ng+24_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:48_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,7_t_int&
             &,8_t_int,9_t_int,25_t_int,26_t_int,27_t_int,28_t_int&
             &,29_t_int,30_t_int&
             &,19_t_int,20_t_int,21_t_int,13_t_int,15_t_int,17_t_int&
             &,31_t_int,32_t_int,33_t_int,34_t_int,35_t_int,36_t_int&
             &,37_t_int,38_t_int,39_t_int,40_t_int,41_t_int,42_t_int&
             &,22_t_int,23_t_int,24_t_int,14_t_int,16_t_int,18_t_int&
             &,43_t_int,44_t_int,45_t_int,46_t_int,47_t_int,48_t_int&
             &,2_t_int,4_t_int,6_t_int,10_t_int,11_t_int,12_t_int/))
        wf(1_t_int:48_t_int) =1.0_t_real
        ng=ng+48_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select

     !!
   End Subroutine poi48i
   !!
   !!***********************************************************************
   !! begin of hexagonal definitions
   !!***********************************************************************
   Subroutine hex2i(a,wf,ng,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  :: a(3,2),wf(2)
     !!LOCAL 
     Character(*),Parameter::nsubfunc="hex2i"
     !!
     !Print *,nsubfunc,'ng=',ng
     Select Case(ipgr)
     Case (32_t_int,31_t_int,30_t_int,28_t_int,27_t_int,26_t_int&
          &,24_t_int,23_t_int,20_t_int,19_t_int,18_t_int)
        !! D6h,D3h(b),D3h(a),D6,C6h,C3h,D3d(b),D3d(a),D3(b),D3(a),C3i
        a(:,1_t_int)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
        wf(1_t_int) =2.0_t_real
        ng=ng+1_t_int
     Case (29_t_int,25_t_int,22_t_int,21_t_int,17_t_int,1_t_int)
        !! C6v,C6,C3v(b),C3v(a),C3,C1
        a(:,1_t_int)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
        a(:,2_t_int)=(/0.0_t_real,0.0_t_real,-1.0_t_real/)
        wf(1_t_int:2_t_int) =1.0_t_real
        ng=ng+2_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return

   End Subroutine hex2i
   !!
   !!***********************************************************************
   Subroutine hex60i(a,wf,ng,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  :: a(3,6),wf(6)
     !!LOCAL 
     Character(*),Parameter::nsubfunc="hex60i"
     !!
     !Print *,nsubfunc,'ng=',ng
     Select Case(ipgr)
     Case (32_t_int,31_t_int,29_t_int,28_t_int,27_t_int,25_t_int&
          &,24_t_int,23_t_int,22_t_int,20_t_int,18_t_int)
        !! D6h,D3h(b),C6v,D6,C6h,C6,D3d(b),D3d(a),C3v(b),D3(b),C3i
        a(:,1_t_int)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
        wf(1_t_int) =6.0_t_real
        ng=ng+1_t_int
     Case (30_t_int,26_t_int,21_t_int,19_t_int,17_t_int)
        !! D3h(a),C3h,C3v(b),D3(a),C3
        a(:,1_t_int)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
        a(:,2_t_int)=(/0.5_t_real,sq3o2,0.0_t_real/)
        wf(1_t_int:2_t_int) =3.0_t_real
        ng=ng+2_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
        a(:,2_t_int)=(/-1.0_t_real,0.0_t_real,0.0_t_real/)
        a(:,3_t_int)=(/0.5_t_real,sq3o2,0.0_t_real/)
        a(:,4_t_int)=(/0.5_t_real,-sq3o2,0.0_t_real/)
        a(:,5_t_int)=(/-0.5_t_real,sq3o2,0.0_t_real/)
        a(:,6_t_int)=(/-0.5_t_real,-sq3o2,0.0_t_real/)
        wf(1_t_int:6_t_int) =1.0_t_real
        ng=ng+6_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return

   End Subroutine hex60i
   !!
   !!***********************************************************************
   Subroutine hex61i(a,wf,ng,ipgr)
     !!
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(out)  :: a(3,6),wf(6)
     !!LOCAL
     Character(*),Parameter::nsubfunc="hex61i"
     !!
     !Print *,nsubfunc,'ng=',ng
     Select Case(ipgr)
     Case (32_t_int,30_t_int,29_t_int,28_t_int,27_t_int,25_t_int,24_t_int,23_t_int,21_t_int,19_t_int,18_t_int)
        !! D6h,D3h(a),C6v,D6,C6,C6,D3d(b),D3d(a),C3v(b),D3(a),C3i
        a(:,1_t_int)=(/sq3o2,0.5_t_real,0.0_t_real/)
        wf(1_t_int) =6.0_t_real
        ng=ng+1_t_int
     Case (31_t_int,22_t_int,20_t_int)
        !! D3h(b),C3v(b),D3(b)
        a(:,1_t_int)=(/sq3o2,0.5_t_real,0.0_t_real/)
        a(:,2_t_int)=(/sq3o2,-0.5_t_real,0.0_t_real/)
        wf(1_t_int:2_t_int) =3.0_t_real
        ng=ng+2_t_int
     Case (26_t_int,17_t_int)
        !! C3h,C3
        a(:,1_t_int)=(/sq3o2,0.5_t_real,0.0_t_real/)
        a(:,2_t_int)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
        wf(1_t_int:2_t_int) =3.0_t_real
        ng=ng+2_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int)=(/sq3o2,0.5_t_real,0.0_t_real/)
        a(:,2_t_int)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
        a(:,3_t_int)=(/-sq3o2,0.5_t_real,0.0_t_real/)
        a(:,4_t_int)=(/sq3o2,-0.5_t_real,0.0_t_real/)
        a(:,5_t_int)=(/0.0_t_real,-1.0_t_real,0.0_t_real/)
        a(:,6_t_int)=(/-sq3o2,-0.5_t_real,0.0_t_real/)
        wf(1_t_int:6_t_int) =1.0_t_real
        ng=ng+6_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return



   End Subroutine hex61i
   !!
   !!***********************************************************************
   !!
   Subroutine hex122i(a,wf,ng,x,y,z,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y,z
     Real(t_real),Intent(out)  :: a(3,12),wf(12)
     !!LOCAL
     Character(*),Parameter::nsubfunc="hex122i"

     Real(t_real) :: d
     Real(t_real),Parameter :: wpnt(3)=(/&
          &12.0_t_real,6.0_t_real,3.0_t_real/)
     Real(t_real) :: pnt(3,12)

     !Print *,nsubfunc,'ng=',ng
     !! 
     !!
     d=Sqrt(x**2_t_int+y**2_t_int)
     If(Abs(y/d-0.5_t_real) > 1.e-12_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'y-coordinate has wrong symmetry!'")
     Endif
     If(Abs(x/d-sq3o2) > 1.e-12_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'x-coordinate has wrong symmetry!'")
     Endif
     !!
     pnt=Reshape(source=(/ &
          &  x, y, z &
          &  ,x,-y, z &
          &  ,x, y,-z &
          &  ,0.0_t_real, d, z &
          &  ,x,-y,-z &
          &  ,0.0_t_real, d,-z &
          &  ,-x,y, z &
          &  ,-x,-y, z &
          &  ,0.0_t_real,-d, z &
          &  ,-x,y,-z &
          &  ,-x,-y,-z &
          &  ,0.0_t_real,-d,-z &
          &/),shape=(/3,12/))

     !!
     Select Case(ipgr)
     Case (32_t_int,30_t_int,28_t_int,27_t_int,24_t_int)
        !! D6h,D3h(a),D6,C6h,D3d(b)
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (31_t_int,20_t_int)
        !! D3h(b),D3(b)
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (29_t_int,25_t_int,23_t_int,21_t_int,19_t_int,18_t_int)
        !! C6v,C6,D3d(a),C3v(b),D3(a),C3i
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (26_t_int)
        !! C3h
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,4_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (22_t_int)
        !! C3v(a)
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,2_t_int,5_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (17_t_int)
        !! C3
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,6_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:12_t_int)=pnt(:,1_t_int:12_t_int)
        wf(1_t_int:12_t_int) =1.0_t_real
        ng=ng+12_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return


   End Subroutine hex122i
   !!
   !!***********************************************************************
   Subroutine hex121i(a,wf,ng,x,y,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y
     Real(t_real),Intent(out)  :: a(3,12),wf(12)
     !!LOCAL
     Character(*),Parameter::nsubfunc="hex121i"
     Real(t_real),Parameter :: wpnt(4)=(/&
          &12.0_t_real,6.0_t_real,3.0_t_real,1.0_t_real/)
     Real(t_real) :: pnt(3,12)

     !Print *,nsubfunc,'ng=',ng
     !! 
     pnt=Reshape(source=(/ &
          &  x, 0.0_t_real, y &
          & ,x*0.5_t_real, x*sq3o2, y &
          & ,x, 0.0_t_real,-y &
          & ,x*0.5_t_real, x*sq3o2,-y &
          & ,-x*0.5_t_real, x*sq3o2, y &
          & ,-x, 0.0_t_real, y &
          & ,-x*0.5_t_real,-x*sq3o2, y &
          & , x*0.5_t_real,-x*sq3o2, y &
          & ,-x*0.5_t_real, x*sq3o2,-y &
          & ,-x, 0.0_t_real,-y &
          & ,-x*0.5_t_real,-x*sq3o2,-y &
          & , x*0.5_t_real,-x*sq3o2,-y &
          &/),shape=(/3,12/))
     !!
     Select Case(ipgr)
     Case (32_t_int,31_t_int,28_t_int,27_t_int,23_t_int)
        !! D6h,D3h(b),D6,C6h,D3d(a)
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (30_t_int,26_t_int,19_t_int,18_t_int)
        !! D3h(a),C3h,D3(a),C3i
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (29_t_int,25_t_int,24_t_int,22_t_int,20_t_int)
        !! C6v,C6,D3d(b),C3v(b),D3(b)
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (21_t_int,17_t_int)
        !! C3v(a),C3
        a(:,1_t_int:4_t_int)=pnt(:,1_t_int:4_t_int)
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:12_t_int)=pnt(:,1_t_int:12_t_int)
        wf(1_t_int:12_t_int) =wpnt(4_t_int)
        ng=ng+12_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return

     !!
   End Subroutine hex121i
   !!
   !!***********************************************************************
   Subroutine hex24i(a,wf,ng,x,y,z,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y,z
     Real(t_real),Intent(out)  :: a(3,24),wf(24)
     !!LOCAL
     Character(*),Parameter::nsubfunc="hex24i"
     Real(t_real),Parameter :: wpnt(4)=(/&
          &24.0_t_real,12.0_t_real,6._t_real,3.0_t_real/)
     Real(t_real) :: pnt(3,24)
     !Print *,nsubfunc,'ng=',ng
     !! 
     pnt=Reshape(source=(/ &
          &  x, y, z &
          & ,x,-y, z &
          & , 0.5_t_real*x+sq3o2*y, sq3o2*x-0.5_t_real*y, z & !3
          & ,x, y,-z &
          & , 0.5_t_real*x-sq3o2*y, sq3o2*x+0.5_t_real*y, z &
          & ,-0.5_t_real*x+sq3o2*y, sq3o2*x+0.5_t_real*y, z & !6
          & , 0.5_t_real*x+sq3o2*y, sq3o2*x-0.5_t_real*y,-z &
          & ,x,-y,-z &
          & , 0.5_t_real*x-sq3o2*y, sq3o2*x+0.5_t_real*y,-z & !9
          & ,-0.5_t_real*x+sq3o2*y, sq3o2*x+0.5_t_real*y,-z & !10

          & ,-0.5_t_real*x-sq3o2*y, sq3o2*x-0.5_t_real*y, z & 
          & ,-x, y, z &
          & ,-x,-y, z &
          & ,-0.5_t_real*x-sq3o2*y,-sq3o2*x+0.5_t_real*y, z & 
          & ,-0.5_t_real*x+sq3o2*y,-sq3o2*x-0.5_t_real*y, z & 
          & , 0.5_t_real*x-sq3o2*y,-sq3o2*x-0.5_t_real*y, z & 
          & , 0.5_t_real*x+sq3o2*y,-sq3o2*x+0.5_t_real*y, z & 
          & ,-0.5_t_real*x-sq3o2*y, sq3o2*x-0.5_t_real*y,-z & 
          & ,-x, y,-z &
          & ,-x,-y,-z &
          & ,-0.5_t_real*x-sq3o2*y,-sq3o2*x+0.5_t_real*y,-z & 
          & ,-0.5_t_real*x+sq3o2*y,-sq3o2*x-0.5_t_real*y,-z & 
          & , 0.5_t_real*x-sq3o2*y,-sq3o2*x-0.5_t_real*y,-z & 
          & , 0.5_t_real*x+sq3o2*y,-sq3o2*x+0.5_t_real*y,-z & 

          &/),shape=(/3,24/))
     !!
     Select Case(ipgr)
     Case (32_t_int)
        !! D6h
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (31_t_int)
        !! D3h(b)
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (30_t_int,27_t_int)
        !! D3h(a),C6h
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (29_t_int,28_t_int,24_t_int,23_t_int)
        !! C6v,D6,D3d(b),D3d(a)
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,4_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (26_t_int)
        !! C3h
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,6_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (25_t_int,21_t_int,19_t_int,18_t_int)
        !! C6,C3v(a),D3(a),C3i
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,7_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (22_t_int,20_t_int)
        !! C3v(b),D3(b)
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,4_t_int,2_t_int,8_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (17_t_int)
        !! C3
        a(:,1_t_int:8_t_int)=pnt(:,(/1_t_int,3_t_int,5_t_int,6_t_int,4_t_int&
             &,7_t_int,9_t_int,10_t_int/))
        wf(1_t_int:8_t_int) =wpnt(4_t_int)
        ng=ng+8_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:24_t_int)=pnt(:,1_t_int:24_t_int)
        wf(1_t_int:24_t_int) =1.0_t_real
        ng=ng+24_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return

     !!
   End Subroutine hex24i
   !!
   !!***********************************************************************
   Subroutine hex120i(a,wf,ng,x,y,ipgr)
     !!
     Implicit None
     !!IO
     Integer(t_int),Intent(in) :: ipgr
     Integer(t_int),Intent(inout) :: ng
     Real(t_real),Intent(in)   :: x,y
     Real(t_real),Intent(out)  :: a(3,12),wf(12)
     !!LOCAL
     Character(*),Parameter::nsubfunc="hex120i"
     Real(t_real),Parameter :: wpnt(3)=(/&
          &12.0_t_real,6.0_t_real,3.0_t_real/)
     Real(t_real) :: pnt(3,12)
     !Print *,nsubfunc,'ng=',ng
     pnt=Reshape(source=(/ &
          &  x, y, 0.0_t_real & !1
          &, x,-y, 0.0_t_real & !2
          & , 0.5_t_real*x+sq3o2*y, sq3o2*x-0.5_t_real*y,0.0_t_real & !3
          & , 0.5_t_real*x-sq3o2*y, sq3o2*x+0.5_t_real*y,0.0_t_real & !4
          & ,-0.5_t_real*x+sq3o2*y, sq3o2*x+0.5_t_real*y,0.0_t_real & !5
          & ,-0.5_t_real*x-sq3o2*y, sq3o2*x-0.5_t_real*y,0.0_t_real & 
          & ,-x, y, 0.0_t_real & 
          & ,-x,-y, 0.0_t_real & 
          & ,-0.5_t_real*x-sq3o2*y,-sq3o2*x+0.5_t_real*y,0.0_t_real & 
          & ,-0.5_t_real*x+sq3o2*y,-sq3o2*x-0.5_t_real*y,0.0_t_real & 
          & , 0.5_t_real*x-sq3o2*y,-sq3o2*x-0.5_t_real*y,0.0_t_real & 
          & , 0.5_t_real*x+sq3o2*y,-sq3o2*x+0.5_t_real*y,0.0_t_real & 
          &/),shape=(/3,12/))

     !!     
     Select Case(ipgr)
     Case (32_t_int,29_t_int,28_t_int,24_t_int,23_t_int)
        !! D6h,C6v,D6,D3d(b),D3d(a)
        a(:,1_t_int)=pnt(:,1_t_int)
        wf(1_t_int) =wpnt(1_t_int)
        ng=ng+1_t_int
     Case (31_t_int,22_t_int,20_t_int)
        !! D3h(b),C3v(b),D3(b)
        a(:,1_t_int:2_t_int)=pnt(:,1_t_int:2_t_int)
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (30_t_int,27_t_int,25_t_int,21_t_int,19_t_int,18_t_int)
        !! D3h(a),C6h,C6,C3v(a),D3(a),C3i
        a(:,1_t_int:2_t_int)=pnt(:,(/1_t_int,3_t_int/))
        wf(1_t_int:2_t_int) =wpnt(2_t_int)
        ng=ng+2_t_int
     Case (26_t_int,17_t_int)
        !! C3h,C3
        a(:,1_t_int:4_t_int)=pnt(:,(/1_t_int,3_t_int,4_t_int,5_t_int/))
        wf(1_t_int:4_t_int) =wpnt(3_t_int)
        ng=ng+4_t_int
     Case (1_t_int)
        !! C1
        a(:,1_t_int:12_t_int)=pnt(:,1_t_int:12_t_int)
        wf(1_t_int:12_t_int) =1.0_t_real
        ng=ng+12_t_int
     Case default
        Write(errbuf,"('''pointgroup ',i4,' does not exist!''')") ipgr
        Call errmsg(nmodule,nsubfunc,E_fatalerror&
             &,msg=errbuf)
     End Select
     return

     !!
   End Subroutine hex120i
   !!***********************************************************************


  
 End Module points
