Module pot
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use constants
  Use matserv
  Use tmesh
  Use memory
  Use mesh
  Use tsymmetry
  Use finuc
  Use trelativistic
  Use vecharm
  Use clock
  Use xc
  Use tpotentialdensity
  Use integrate1d
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="POT"
  !!
  Public :: vcoulomb,vew,vew_sum,cpalmptst,HFonsite,prep_for_gga&
       &,dvcoulomb
  !!
  !!***********************************************************************
Contains
  !!***********************************************************************
  !! coulomb potential
  !!
  !! nsite: number of sites
  !! mlmp: max number of lm-components
  !! nlmp: number of lm-components  for each atom
  !! lmp: lm-components  for each atom
  !! mspin: number of spins
  !! lcp: local potential components
  !! nlattg: number of plane waves
  !! glatt:  plane waves
  !! vft:  (complex)  fourier potential
  !! vuc:  volume of unit cell
  !! lcd:  density
  !! energy: is it for an energy calculation ?
  !!***********************************************************************
  !!
  Subroutine vcoulomb(nsite,tau,vuc,isrt,conao&
       &,nlattg,glatt&
       &,mspin,degree,fgam,max_L,mlmp,nlmp,lmp,pmeshs,nrp,natom&
       &,lcd,ew_p_r,lcp,vft,ac,Nl,relativistic,nuc_type,lew)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer      :: pmeshs(:)
    Integer(t_int),Intent(in) :: relativistic,nuc_type
    Integer(t_int),Intent(in) :: nsite,mlmp,mspin,isrt(nsite),nlmp(nsite)
    Integer(t_int),Intent(in) :: max_L,nlattg,degree,nrp
    Integer(t_int),Intent(in) :: lmp(mlmp,nsite)
    Real(t_real),Intent(in)   :: glatt(3,nlattg)
    Real(t_real),Intent(in)   :: fgam(*)
    Real(t_real),Intent(in)  :: tau(3,nsite),vuc
    Real(t_real),Intent(in)  :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)  :: natom(*),conao(*)
    Real(t_real),Intent(in)  :: ew_p_r
    Complex(t_real),Intent(out) :: vft(nlattg)
    Real(t_real),Intent(out) :: lcp(nrp,mlmp,nsite)
    Real(t_real),Intent(out) :: ac(mlmp,nsite)
    Real(t_real),Intent(out) :: Nl(0:max_L)
    !!optionally the local part of the ewald potential (without tails)
    Real(t_real),Intent(out),Optional :: lew(nrp,mlmp,nsite)
    !!LOCAL
    Character(*),Parameter  :: cpumesg='Coulomb-potential'
    !!
    Call watch(cpumesg,cpu_start,reset=.True.)

    !! multipole momenta
    Call mpolm(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
         &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl)


    !! fourier-part of Ewald potential
    Call ewd(nsite,isrt,conao,tau,vuc,nlattg,glatt,mlmp,nlmp,lmp&
         &,max_L,ew_p_r,vft,ac,Nl)

    !! local part of Coulomb potential

    Call vloc(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp,pmeshs&
         &,nrp,natom,lcd,ew_p_r,ac,Nl,lcp,relativistic,nuc_type,lew)

    !!
    Call watch(cpumesg,cpu_stop,delete=.True.,Print=.True.)
    !!
  End Subroutine vcoulomb
  !!***********************************************************************
  Subroutine dvcoulomb(nsite,tau,vuc,isrt,conao,nlattg,glatt&
       &,mspin,degree,fgam,max_L,mlmp,nlmp,lmp,pmeshs,nrp,natom&
       &,lcd,ew_p_r,lcp,dvft,ac,Nl,dolew)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in) :: dolew
    Type(T_Mesh),Pointer      :: pmeshs(:)
    Integer(t_int),Intent(in) :: nsite,mlmp,mspin,isrt(nsite),nlmp(nsite)
    Integer(t_int),Intent(in) :: max_L,nlattg,degree,nrp
    Integer(t_int),Intent(in) :: lmp(mlmp,nsite)
    Real(t_real),Intent(in)   :: glatt(3,nlattg)
    Real(t_real),Intent(in)   :: fgam(*)
    Real(t_real),Intent(in)  :: tau(3,nsite),vuc
    Real(t_real),Intent(in)  :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)  :: natom(*),conao(*)
    Real(t_real),Intent(in)  :: ew_p_r
    Complex(t_real),Intent(out) :: dvft(nlattg,nsite)
    Real(t_real),Intent(out) :: lcp(nrp,2,mlmp,nsite)
    Real(t_real),Intent(out) :: ac(mlmp,nsite)
    Real(t_real),Intent(out) :: Nl(0:max_L)
    !!LOCAL
    Character(*),Parameter  :: cpumesg='gradient Coulomb-potential'
    !!
    Call watch(cpumesg,cpu_start,reset=.True.)

    !! multipole momenta
    Call mpolm(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
         &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl)


    !! fourier-part of Ewald potential
    Call dewd(nsite,isrt,conao,tau,vuc,nlattg,glatt,mlmp,nlmp,lmp&
         &,max_L,ew_p_r,dvft,ac,Nl)

    !! local part of Coulomb potential

    Call dvloc(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp,pmeshs&
         &,nrp,natom,lcd,ew_p_r,ac,Nl,lcp,dolew)

    !!
    Call watch(cpumesg,cpu_stop,delete=.True.,Print=.True.)
    !!
  End Subroutine dvcoulomb
  !!***********************************************************************
  !! calculates the Multipol momenta of the density lcd(r,l,isa,ms)
  !! ac: multipol momenta
  !! do it with the same method as in vloc in order to assure zero
  !! potential at rmax, which is of the essence for Etot(c/a) jumps
  !!
  Subroutine mpolm(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
       &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: nsite,mspin,degree,isrt(nsite)
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite),nrp
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: ew_p_r
    Real(t_real),Intent(in)      :: natom(*)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(out)     :: ac(mlmp,nsite),Nl(0:max_L)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="mpolm"
    Integer(t_int) :: ng,isa,issa,ilm,l,ir,ig,index,deg
    Real(t_real)   :: p,pp,exf,ex,Q,NN,ri,r0,rl,dQ,dNN,xg,rg,jg,rgl
    Real(t_real)   :: texpout,sd
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrp)
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    Nl(:)=0.0_t_real
    ac(:,:)=0.0_t_real
    !!
    ng=10
    p=ew_p_r
    pp=p*p
    !!
    Call get_gauss(ng,p_xx,p_ww)
    !!
    Do isa=1_t_int, nsite
       !!
       issa=isrt(isa)
       !!
       pmesh => pmeshs(issa)
       !!
       Do ilm=1,nlmp(isa)
          l=llilm(lmp(ilm,isa))
          
          
          
          exf=-1.0_t_real
          If(l==0) exf=2.0_t_real*fgam(issa)-2.0_t_real
          If(l==1) exf=fgam(issa)-1.0_t_real
          ex=-exf-1.0_t_real
          Do ir=2,pmesh% nr
             dens(ir)=Sum(lcd(ir,ilm,isa,1:mspin))&
                  &*spow(pmesh% p_rmesh(ir),ex)
          Enddo
          dens(1)=0.0_t_real


          Q=0.0_t_real
          NN=0.0_t_real
          Do ir=2,pmesh% nr
             ri=pmesh% p_rmesh(ir)
             r0=pmesh% p_rmesh(ir-1)
             rl=spowi(ri,l)
             
             
             dQ=0.0_t_real
             dNN=0.0_t_real
             Do ig=1,ng
                xg=(p_xx(ig)+1.0_t_real)*0.5_t_real !! xi \in [0,1]
                rg=(ri-r0)*xg+r0 !! r \in [r0,ri]
                jg=(ri-r0)*p_ww(ig)*0.5_t_real

                !!TODO: mesh transform
                
                rgl=spowi(rg,2*l+2)
                
                texpout=spow(rg,exf+l+1)

                index=rmintpl_coefs1(degree,2_t_int,deg,pmesh,rg,coef)
                sd=Dot_product(dens(index:index+deg),coef(0:deg))
                !!
                ex=Exp(-(pp*rg*rg))
                !!
                !!
                dQ=dQ+sd*texpout*jg
                dNN=dNN+ex*rgl*jg
                !!                
             Enddo
             !!
             Q=Q+dQ
             NN=NN+dNN
             !!
          Enddo
          !!
          Nl(l)=NN
          !!
          If(l==0)Then
             ac(ilm,isa)=(Q-natom(issa)/sq4pi)
          Else
             ac(ilm,isa)=Q
          Endif
          !!
          !!
          !!
          !!
       Enddo !! ilm
       !!
    Enddo !! isa
    !!
    If(.False.)Then
       Do isa=1,nsite
          Write(outfile,10)isa
          Do ilm=1_t_int,nlmp(isa)
             l=llilm(lmp(ilm,isa))
             sd=1.0_t_real
             if(l==0)sd=sq4pi
             Write(outfile,10)ilm,l,mmilm(lmp(ilm,isa))-l-1,ac(ilm,isa)*sd
          Enddo
       Enddo
    Endif
    !!
    Return
    !!
10  Format('ilm: ',i5,' l,m=',i3,',',i3,' Q=',g30.15e3)
    !!
  End Subroutine mpolm
  !!***********************************************************************
  Subroutine vloc(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
       &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl,lcp,relativistic,nuc_type,lew)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: relativistic,nuc_type
    Integer(t_int),Intent(in)    :: nsite,mspin,degree,isrt(nsite)
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite),nrp
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: ew_p_r
    Real(t_real),Intent(in)      :: natom(*)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(in)      :: ac(mlmp,nsite),Nl(0:max_L)
    Real(t_real),Intent(out)     :: lcp(nrp,mlmp,nsite)
    Real(t_real),Intent(out),Optional :: lew(nrp,mlmp,nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="vloc"
    Logical(t_log) :: dolew
    Integer(t_int) :: ir,ig,ilm,l,isa,issa,ng,index,deg
    Real(t_real)   :: summ,sumew,ri,rl,r0,d,dew,xg,rg,jg,rgl,sd,ex,exf
    Real(t_real)   :: sdew,sdew1,ds,d1,d1ew,p,pp,Z,norm,texpout,texpin
    Real(t_real)   :: y(nrp),y1(nrp)
    Real(t_real)   :: yew(nrp),y1ew(nrp)
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrp)
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    dolew=Present(lew)
    !!
    ng=10
    p=ew_p_r
    pp=p*p
    !!
    Call get_gauss(ng,p_xx,p_ww)
    !!
    lcp(:,:,:)=0.0_t_real
    If(dolew) Then
       lew(:,:,:)=0.0_t_real
    Endif
    !ehz0=0.0_t_real
    Do isa=1_t_int, nsite
       !!
       issa=isrt(isa)
       !!
       pmesh => pmeshs(issa)
       !!
       Z=natom(issa)
       !!
       Do ilm=1,nlmp(isa)
          l=llilm(lmp(ilm,isa))
          
          ds=fourpi/(2.0_t_real*Real(l,t_real)+1.0_t_real)
          
          
          exf=-1.0_t_real
          If(l==0) exf=2.0_t_real*fgam(issa)-2.0_t_real
          If(l==1) exf=fgam(issa)-1.0_t_real
          If(l==2) Then

             If(relativistic==FULL_RELATIVISTIC .Or.&
                  &relativistic==SCALAR_RELATIVISTIC)Then
                If(nuc_type /= FINUC_NONE)Then
                   exf=4.0_t_real-2.0_t_real
                Else
                   exf=fgam(issa)+Sqrt(fgam(issa)**2-1+4)-2.0_t_real
                Endif
             Else If(relativistic==NON_RELATIVISTIC)Then
                exf=4.0_t_real-2.0_t_real
             Else If(relativistic==SCALAR_RELATIVISTIC_KH)Then
                exf=2.0_t_real*Sqrt(fgam(issa)**2+2)-2.0_t_real 
             Else
                Call terminate(TERM_CRASH,Trim(nsubfunc)&
                     &//': unknown relativistic option!')
             Endif
             
             !todo finuc
             !todo make all the output debug option
             !KH exf=2.0_t_real*Sqrt(fgam(issa)**2+2)-2.0_t_real 



             !exf=4.0_t_real-2.0_t_real !! finuc NREL FREL SREL

          Endif
          ex=-exf-1.0_t_real
          Do ir=2,pmesh% nr
             dens(ir)=Sum(lcd(ir,ilm,isa,1:mspin))&
                  &*spow(pmesh% p_rmesh(ir),ex)
          Enddo
          dens(1)=0.0_t_real

          !! dens=lcd*r**(-exf-1)=n*r**(1-exf)
          !! l==0: dens=n*r**(1-(2*gam-2))
          !!       n=r**(2*gam-2)*P
          !!       dens=P*r
          !!


          summ=0.0_t_real
          sumew=0.0_t_real
          y(1)=summ
          yew(1)=sumew
          Do ir=2,pmesh% nr
             ri=pmesh% p_rmesh(ir)
             r0=pmesh% p_rmesh(ir-1)
             rl=spowi(ri,l)
             
             norm=1.0_t_real/Nl(l)
             
             d=0.0_t_real
             dew=0.0_t_real
             d1=0.0_t_real
             d1ew=0.0_t_real
             Do ig=1,ng
                xg=(p_xx(ig)+1.0_t_real)*0.5_t_real !! xi \in [0,1]
                rg=(ri-r0)*xg+r0 !! r \in [r0,ri]
                jg=(ri-r0)*p_ww(ig)*0.5_t_real
                !!TODO: mesh transform
                
                rgl=spowi(rg,l)
                
                texpout=spow(rg,exf+l+1)
                texpin=spow(rg,exf-l)

                index=rmintpl_coefs1(degree,2_t_int,deg,pmesh,rg,coef)
                sd=Dot_product(dens(index:index+deg),coef(0:deg))
      
                
                ex=Exp(-(pp*rg*rg))
                
                sdew=rgl*ex*rg*rg*ac(ilm,isa)*norm
                sdew1=ex*ac(ilm,isa)*norm
                
                d=d+sd*texpout*jg
                dew=dew+sdew*rgl*jg

                !! y1 <- lcd*r**(-1-l) = n*r**(1-l)
                
                d1=d1+sd*texpin*jg
                d1ew=d1ew+sdew1*rg*jg
                
             Enddo
             
             summ=summ+d
             sumew=sumew+dew
             y(ir)=(summ-sumew)/rl*ds
             yew(ir)=sumew/rl*ds
             
             y1(ir-1)=d1
             y1ew(ir-1)=d1ew
             
          Enddo
          !!
          !!
          summ=0.0_t_real
          sumew=0.0_t_real
          yew(pmesh% nr)=yew(pmesh% nr)/pmesh% p_rmesh(pmesh% nr)
          Do ir=pmesh% nr-1,1,-1
             ri=pmesh% p_rmesh(ir)
             rl=spowi(ri,l)
             
             
             summ=summ+y1(ir)
             sumew=sumew+y1ew(ir)
             
             y(ir)=y(ir)+(summ-sumew)*rl*ri*ds
             If(ir==1)Then
                yew(ir)=sumew*rl*ds
             Else
                yew(ir)=yew(ir)/ri+sumew*rl*ds
             Endif
             
          Enddo
          !!
          If(l==0) Then
             lcp(1:pmesh% nr,ilm,isa)=y(1:pmesh% nr)-Z*sq4pi
          Else
             lcp(1:pmesh% nr,ilm,isa)=y(1:pmesh% nr)
          Endif
          If(dolew)Then
             lew(1:pmesh% nr,ilm,isa)=yew(1:pmesh% nr)
          Endif
          !!
          !! exact Ewald outward
          !Do ir=1,pmesh% nr
          !   ri=pmesh% p_rmesh(ir)
          !   pr=p*ri
          !   Call intngauss(pr,l,sdew)
          !   yewex(ir)=(sdew)*p**(l+1)*ri*ds*ac(ilm,isa)
          !Enddo
          !! ewald Exact inwards integration
          !Call exactEwaldInward(pmesh% nr,pmesh% p_rmesh,pp,l,y1ewex)
          !If(dolew)Then
          !   y1ewex=y1ewex*ac(ilm,isa)/Nl(l)*ds
          !Endif
          
!!$          If(l==2) Then
!!$             Call writefunc(pmesh% nr,pmesh% p_rmesh,y1,17_t_int,'+vloc')
!!$             !Stop
!!$          Endif
          
          !!
          If(ilm== 1 )Then
             If(.Not. pmesh% with_zero) Stop 'mesh must be a with_zero-mesh'
             !TODO:             ehz0=ehz0+y1(1)*sq4pi*Z !! TODO: specail code to extract r-factor
          Endif
          !!
       Enddo !! ilm
       !!
    Enddo !! isa
    !!
    Return
    !!
  End Subroutine vloc
  !!***********************************************************************
  Subroutine dvloc(nsite,isrt,mspin,degree,fgam,max_L,mlmp,nlmp,lmp&
       &,pmeshs,nrp,natom,lcd,ew_p_r,ac,Nl,lcp,dolew)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: dolew
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: nsite,mspin,degree,isrt(nsite)
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite),nrp
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: ew_p_r
    Real(t_real),Intent(in)      :: natom(*)
    Real(t_real),Intent(in)      :: fgam(*)
    Real(t_real),Intent(in)      :: ac(mlmp,nsite),Nl(0:max_L)
    Real(t_real),Intent(out)     :: lcp(nrp,2,mlmp,nsite)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="dvloc"
    Integer(t_int) :: ir,ig,ilm,l,isa,issa,ng,index,deg
    Real(t_real)   :: summ,ri,rl,r0,d,xg,rg,jg,rgl,sd,ex,exf
    Real(t_real)   :: sdew,sdew1,d1,p,pp,Z,norm,texpout,texpin
    Real(t_real)   :: y(nrp),y1(nrp),y2(nrp)
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrp)
    Type(T_Mesh),Pointer :: pmesh
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    !!
    ng=10
    p=ew_p_r
    pp=p*p
    !!
    Call get_gauss(ng,p_xx,p_ww)
    !!
    lcp(:,:,:,:)=0.0_t_real
    !ehz0=0.0_t_real
    Do isa=1_t_int, nsite
       !!
       issa=isrt(isa)
       !!
       pmesh => pmeshs(issa)
       !!
       Z=natom(issa)
       !!
       Do ilm=1,nlmp(isa)
          l=llilm(lmp(ilm,isa))
          
          
          
          
          exf=-1.0_t_real
          If(l==0) exf=2.0_t_real*fgam(issa)-2.0_t_real
          If(l==1) exf=fgam(issa)-1.0_t_real
          ex=-exf-1.0_t_real
          Do ir=2,pmesh% nr
             dens(ir)=Sum(lcd(ir,ilm,isa,1:mspin))&
                  &*spow(pmesh% p_rmesh(ir),ex)
          Enddo
          dens(1)=0.0_t_real


          summ=0.0_t_real

          y(1)=summ
          Do ir=2,pmesh% nr
             ri=pmesh% p_rmesh(ir)
             r0=pmesh% p_rmesh(ir-1)
             rl=spowi(ri,l)
             
             norm=1.0_t_real/Nl(l)
             
             d=0.0_t_real
             d1=0.0_t_real
             Do ig=1,ng
                xg=(p_xx(ig)+1.0_t_real)*0.5_t_real !! xi \in [0,1]
                rg=(ri-r0)*xg+r0 !! r \in [r0,ri]
                jg=(ri-r0)*p_ww(ig)*0.5_t_real
                !!TODO: mesh transform
                
                rgl=spowi(rg,l)
                
                texpout=spow(rg,exf+l+1)
                texpin=spow(rg,exf-l)

                index=rmintpl_coefs1(degree,2_t_int,deg,pmesh,rg,coef)
                sd=Dot_product(dens(index:index+deg),coef(0:deg))
      
                
                
                ex=0.0_t_real
                sdew=0.0_t_real
                sdew1=0.0_t_real
                If(dolew) Then
                   ex=Exp(-(pp*rg*rg))
                   sdew=rgl*ex*rg*rg*ac(ilm,isa)*norm
                   sdew1=ex*ac(ilm,isa)*norm
                Endif
                
                d=d+(sd*texpout-sdew*rgl)*jg
                
                d1=d1+(sd*texpin-sdew1*rg)*jg
                
             Enddo
             
             summ=summ+d
             y(ir)=summ/rl*fourpi
             
             y1(ir-1)=d1
             
          Enddo
          !! y =4Pi/rl*int_0^r n~ r^(l+2)
          !! y1(ri)=int_ri^{ri+1} n~ r^(-l+1)
          !!
          summ=0.0_t_real
          y2(pmesh% nr)=0.0_t_real
          Do ir=pmesh% nr-1,1,-1
             ri=pmesh% p_rmesh(ir)
             rl=spowi(ri,l)
             
             summ=summ+y1(ir)
             
             y2(ir)=summ*rl*ri*fourpi
             
          Enddo
          !!
          If(l==0) Then
             lcp(1:pmesh% nr,2,ilm,isa)=-y(1:pmesh% nr)+Z*sq4pi
          Else
             lcp(1:pmesh% nr,2,ilm,isa)=-y(1:pmesh% nr)
          Endif
          lcp(1:pmesh% nr,1,ilm,isa)=y2(1:pmesh% nr)
          !!
          !!
          If(ilm== 1 )Then
             If(.Not. pmesh% with_zero) &
                  &Stop 'mesh must be a with_zero-mesh'
          Endif
          !!
       Enddo !! ilm
       !!
    Enddo !! isa
    !!
    Return
    !!
  End Subroutine dvloc
  !!***********************************************************************
#ifdef my_compile_unused
  Subroutine exactEwaldInward(nr,rmesh,pp,l,f)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nr,l
    Real(t_real),Intent(in)      :: pp
    Real(t_real),Intent(in)      :: rmesh(nr)
    Real(t_real),Intent(out)     :: f(nr)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="exactEwaldInward"
    Integer(t_int) :: ir
    Real(t_real)   :: ri,rr,exR,ex,sdew
    !!
    !!
    ri=rmesh(nr)
    rr=ri*ri
    exR=Exp(-(pp*rr))
    Do ir=1,nr
       ri=rmesh(ir)
       rr=ri*ri
       ex=Exp(-(pp*rr))
       sdew=(ex-exR)/pp*0.5_t_real
       f(ir)=sdew*spowi(ri,l+1)
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine exactEwaldInward
#endif
  !!***********************************************************************
  
  !!***********************************************************************
  !! I_l(r)=u^(-l-1) \int_0^u u^(l+2) g^{ew}_l(u) dr 
  !!
  !! g^{ew}_l(u)=u^l exp(- u^2)/ Nl(l)
  !! with Nl(l) being the exact normalization constant
  !! u^(l+1)*I_l(u) should be close to one when p is choosen correctly
  !!
#ifdef my_compile_unused
  Subroutine intngauss(u,l,res)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)      :: u
    Integer(t_int),Intent(in)    :: l
    Real(t_real),Intent(out)     :: res
    !!LOCAL
    !Character(*),Parameter::nsubfunc="intngauss"
    Integer(t_int) :: k,nn
    Real(t_real)   :: x,u2,ex,PP,dfac,leps!,d
    !!
    !Print *,'u=',u
    !Print *,'l=',l
    !!
    leps=Log(Epsilon(1.0_t_real))
    !!
    u2=u*u
    ex=Exp(-(u2))
    x=2.0_t_real*u2
    If(u>1.0_t_real)Then

       PP=1.0_t_real
       Do k=1,l
          PP=1.0_t_real+PP*x/(2*(l-k+1)+1)
       Enddo
       res=(erf(u)-2.0_t_real/sqpi*PP*u*ex)/u**(l+1)
    Else

       nn=Ceiling(leps/Log(x/(2*l+5))-1)
       PP=1.0_t_real
       Do k=nn,0,-1
          PP=1.0_t_real+PP*x/(2*(l+k)+5)
       Enddo
       
       x=2.0_t_real*u
       dfac=x
       Do k=1,l+1
          dfac=dfac*x/(2*k+1)
       Enddo

       res=1.0_t_real/sqpi*PP*dfac*ex
       
    Endif
    
    !!
    Return
    !!
  End Subroutine intngauss
#endif
  !!***********************************************************************
  !!
  !! the program fourier redistributes the density with Ewald method
  !!
  Subroutine ewd(nsite,isrt,conao,tau,vuc,nlattg,glatt,mlmp,nlmp,lmp&
       &,max_L,ew_p_r,vft,ac,Nl)
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nlattg,nsite,max_L,isrt(nsite),mlmp
    Integer(t_int),Intent(in) :: nlmp(nsite),lmp(mlmp,nsite)
    Real(t_real),Intent(in)  :: tau(3,nsite),glatt(3,nlattg),vuc
    Real(t_real),Intent(in)  :: ew_p_r
    Real(t_real),Intent(in)  :: conao(*)
    Real(t_real),Intent(in)  :: ac(mlmp,nsite),Nl(0:max_L)
    Complex(t_real),Intent(out) :: vft(nlattg)
    !!LOCAL
    Integer(t_int) :: isa,issa,ig,ilm,il,l
    Real(t_real)  :: p,pp,con,g2,g,d,ex0,ex,ff
    Real(t_real)  :: tt(3),gg(3)
!    Real(t_real)  :: rei(-max_L:max_L),imi(-max_L:max_L)
    Real(t_real)  :: ya((max_L+1_t_int)**2_t_int)
    Complex(t_real) :: cex
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    vft(:)=c0
    !!
    !!
    Do isa=1_t_int, nsite
       issa=isrt(isa)
       !p=ewcutoff/ratom(issa)
       !If(auto) p=ew_p_r
       p=ew_p_r
       pp=p*p
       con=conao(issa)
       tt(:)=tau(:,isa)
       ff=fourpi*sq4pi**3/vuc*con/(2.0_t_real*p)**3
       !! - Fourier transformed ewald potential
       Do ig=2_t_int,nlattg
          gg(:)=glatt(:,ig)
          g2=Sum(gg*gg)
          g=Sqrt(g2)
          d=Dot_product(tt,gg)
          cex=Exp(-ci*d)
          
          ex0=Exp(-(g2/4.0_t_real/pp))*ff/g2

          Call ylms12(ya,gg(1_t_int),gg(2_t_int),gg(3_t_int),max_L)

          Do ilm=1_t_int,nlmp(isa)
             il=lmp(ilm,isa)
             l=llilm(il)

             ex=ex0*(-g*0.5_t_real/pp)**l*ac(ilm,isa)*ya(il)/Nl(l)
             
             vft(ig)=vft(ig)+cex*ci**l*ex
          Enddo                 ! ilm
       Enddo                   ! ig
    Enddo                     ! isa
    !!
    vft(1_t_int)=c0
    !!
    Return
    !!
  End Subroutine ewd
  !!***********************************************************************
  !!
  !! the program fourier redistributes the density with Ewald method
  !! and returns the Ewald potential gradient expansion with respect to sites
  Subroutine dewd(nsite,isrt,conao,tau,vuc,nlattg,glatt,mlmp,nlmp,lmp&
       &,max_L,ew_p_r,dvft,ac,Nl)
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nlattg,nsite,max_L,isrt(nsite),mlmp
    Integer(t_int),Intent(in) :: nlmp(nsite),lmp(mlmp,nsite)
    Real(t_real),Intent(in)  :: tau(3,nsite),glatt(3,nlattg),vuc
    Real(t_real),Intent(in)  :: ew_p_r
    Real(t_real),Intent(in)  :: conao(*)
    Real(t_real),Intent(in)  :: ac(mlmp,nsite),Nl(0:max_L)
    Complex(t_real),Intent(out) :: dvft(nlattg,nsite)
    !!LOCAL
    Integer(t_int) :: isa,issa,ig,ilm,il,l
    Real(t_real)  :: p,pp,con,g2,g,d,ex0,ex,ff
    Real(t_real)  :: tt(3),gg(3)
!    Real(t_real)  :: rei(-max_L:max_L),imi(-max_L:max_L)
    Real(t_real)  :: ya((max_L+1_t_int)**2_t_int)
    Complex(t_real) :: cex
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    dvft(:,:)=c0
    !!
    !!
    Do isa=1_t_int, nsite
       issa=isrt(isa)
       !p=ewcutoff/ratom(issa)
       !If(auto) p=ew_p_r
       p=ew_p_r
       pp=p*p
       con=conao(issa)
       tt(:)=tau(:,isa)
       ff=fourpi*sq4pi**3/vuc*con/(2.0_t_real*p)**3
       !! - Fourier transformed ewald potential
       Do ig=2_t_int,nlattg
          gg(:)=glatt(:,ig)
          g2=Sum(gg*gg)
          g=Sqrt(g2)
          d=Dot_product(tt,gg)
          cex=Exp(-ci*d)
          
          ex0=Exp(-(g2/4.0_t_real/pp))*ff/g2

          Call ylms12(ya,gg(1_t_int),gg(2_t_int),gg(3_t_int),max_L)

          Do ilm=1_t_int,nlmp(isa)
             il=lmp(ilm,isa)
             l=llilm(il)

             ex=ex0*(-g*0.5_t_real/pp)**l*ac(ilm,isa)*ya(il)/Nl(l)
             
             dvft(ig,isa)=dvft(ig,isa)+cex*ci**l*ex
          Enddo                 ! ilm
       Enddo                   ! ig
    Enddo                     ! isa
    !!
    dvft(1_t_int,1:nsite)=c0
    !!
    Return
    !!
  End Subroutine dewd
  !!***********************************************************************
#ifdef my_compile_unused
  !!***********************************************************************
  !! normalization factors N_l for Gauss screening charge (Ewald procedure)
  !!
  !!
  !!  g_l= 1/N_l * r^l e^{-r*2 * p*2}
  !!
  !!  int_0^\infty g_l r^l r^2 dr =1
  !!
  !!  N_l = \frac{ \Gamma(\frac{2 l+3}{2})  } { 2 p^{2l+3}}
  !!
  !!  N_l = N_{l-1} * \frac{2 l+1}{ 2 p^2}
  !!
  !!
  !!  N_{-1} = \frac{ \sqrt{\Pi} } { 2 p }
  !!
  Subroutine EwGaussNl(p,max_L,Nl)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: max_L
    Real(t_real),Intent(in)      :: p
    Real(t_real),Intent(out)     :: Nl(0:max_L)
    !!LOCAL
    Integer(t_int) :: l
    Real(t_real)   :: ex,p2
    p2=p**2
    ex=sqpi*0.5_t_real/p
    Do l=0,max_L
       ex=ex*Real(2*l+1,t_real)/p2*0.5_t_real
       NL(l)=ex
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine EwGaussNl
#endif
  !!***********************************************************************
  !! ft-potential (Ewald procedure) at point(x,y,z)
  !!
  Real(t_real)  Function vew(r,vft,nlattg,glatt)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nlattg
    Real(t_real),Intent(in)      :: r(3)
    Complex(t_real),Intent(in)   :: vft(nlattg)
    Real(t_real),Intent(in)      :: glatt(3,nlattg)
    !!LOCAL
    Integer(t_int) :: ig
    Real(t_real)  :: g
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    vew=0.0_t_real
    !!
    Do ig=2_t_int,nlattg
       g=Dot_product(glatt(:,ig),r(:))
       vew=vew+Real(vft(ig)*Exp(ci*g),t_real)
    Enddo                     ! ig
    !!
  End Function vew
  !!***********************************************************************
  !! direct lattice sum of all ewald aux potentials, replaces
  !! fourier potential for molecules 
  !!
  Real(t_real) Function vew_sum(rsh,nsite,isrt,tau,pmeshs,nrp,degree&
       &,max_L,mlmp,nlmp,lmp,vew,ac)
    !!
    Implicit none  
    !!IO
    Type(T_Mesh),Pointer         :: pmeshs(:)
    Integer(t_int),Intent(in)    :: degree
    Integer(t_int),Intent(in)    :: nsite,isrt(nsite)
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite),nrp
    Real(t_real),Intent(in)      :: tau(3,nsite)
    Real(t_real),Intent(in)      :: vew(nrp,mlmp,nsite)
    Real(t_real),Intent(in)      :: ac(mlmp,nsite)
    !! the real space point at which to calculate the potenital
    Real(t_real),Intent(in)      :: rsh(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="vew_sum"
    Integer(t_int) :: isb,issb,indexb,in,j,l
    Real(t_real)   :: ydn(1:degree+1),coefb(0:degree)
    Real(t_real)   :: r(3),tt(3),vvew,rb2,rb,yb(169),dd!,p
    Type(T_Mesh),Pointer :: pmesh
    !!
    !!
    vvew=0.0_t_real
    isbloop: Do isb=1_t_int,nsite
       issb=isrt(isb)
       !!
       pmesh => pmeshs(issb)
       !!
       tt(:)=tau(:,isb)
       !!
       !!
       r(:)=rsh(:)-tt(:)
       rb2=Dot_product(r,r)
       rb=Sqrt(rb2)
       !!
       !!
       Call ylms12(yb,r(1),r(2),r(3),max_L)
       !!
       !!
       !!
       indexb=mintpl_coefs(degree,pmesh,rb,coefb)
       !!
       rltrhoif: If(rb <= pmesh% rmax)Then
          !!
          !!
          !!
          Do in=0_t_int,degree
             dd=0.0_t_real
             Do j=1_t_int,nlmp(isb)
                dd=dd+vew(indexb+in,j,isb)*yb(lmp(j,isb))
             Enddo
             ydn(in+1_t_int)=dd
          Enddo
          dd=Dot_product(ydn(1:1+degree),coefb(0:degree))
          vvew=vvew+dd

       Else 
          !! add up the multipole tails 
          dd=0.0_t_real
          Do j=1_t_int,nlmp(isb)
             l=llilm(lmp(j,isb))
             dd=dd+ac(j,isb)/(2.0_t_real*l+1.0_t_real)*fourpi&
                  &/rb**(l+1)*yb(lmp(j,isb))
          Enddo
          vvew=vvew+dd
          
          
          
       Endif rltrhoif

    End Do isbloop
    !!
    vew_sum=vvew
    !!
    Return
    !!
  End Function vew_sum
  !!***********************************************************************
  !!
  Subroutine cpalmptst(naps,nlmp,lmp,nqAtPos,qposToSite)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)  :: naps,nlmp(:),lmp(:,:)&
         & , nqAtPos(:),qposToSite(:,:)

    !!LOCAL
    Character(*),Parameter::nsubfunc="cpalmptst"
    Integer(t_int) :: isap,iqa,iqb,isa,isb,ja,jb,j
    !!
    Do isap=1_t_int,naps
       Do iqa=1_t_int,nqAtPos(isap)
          isa=qposToSite(isap,iqa)
          ja=nlmp(isa)
          Do iqb=1_t_int,nqAtPos(isap)
             isb=qposToSite(isap,iqb)
             jb=nlmp(isb)
             If(ja/=jb)Then
                Write(12,*)'CPALMPTST: isa ',isa,' isb ',isb,' nlmp(a) ' &
                     & ,ja,' <>  nlmp(b) ',jb
                Call terminate(TERM_CRASH,Trim(nsubfunc)//'CPALMPTST: ERROR!')
             Endif
             Do j=1_t_int,ja
                If(lmp(j,isa)/=lmp(j,isb))Then
                   Write(12,*)'CPALMPTST: isa ',isa,' isb ',isb,' lmp(',j &
                        & ,',a) ',lmp(j,isa),' <>  lmp(',j,'b) ',lmp(j,isb)
                   Call terminate(TERM_CRASH,Trim(nsubfunc)//'CPALMPTST: ERROR!')
                Endif
             Enddo               ! j
          Enddo                 ! iqb
       Enddo                   ! iqa
    Enddo                     ! isap
    !!
  End Subroutine cpalmptst
  !!***********************************************************************
  !! lcd(:,2,:,:) = r^2 d_l^{+} v_L(r)
  !! lcd(:,1,:,:) = r^2 d_l^{-} v_L(r)
  !! f1(1:3) = sqrt{4Pi/3} \int_0^{\infty} \tilde{n}_{1m}(x) dx
  !!        which is the lim_{r\rightarrow 0} of the l=1 part
  !!
  !!***********************************************************************
  Subroutine HFonsite(isa,nsite,fgam,mspin,degree,coulerg_nr,pmesh,nrp&
       &,max_L,mlmp,nlmp,lmp,ew_p_r,lcd,ac,Nl,f1,f1ew)
    !!
    Implicit None
    !!IO
    Type(T_Mesh),Intent(in)      :: pmesh
    Integer(t_int),Intent(in)    :: isa,nsite,mspin
    Integer(t_int),Intent(in)    :: degree,coulerg_nr,nrp
    Integer(t_int),Intent(in)    :: max_L,mlmp,nlmp(nsite),lmp(mlmp,nsite)
    Real(t_real),Intent(in)      :: fgam
    Real(t_real),Intent(in)      :: ew_p_r
    Real(t_real),Intent(in)      :: lcd(nrp,mlmp,nsite,mspin)
    Real(t_real),Intent(in)      :: ac(mlmp,nsite),Nl(0:max_L)

    Real(t_real),Intent(out)     :: f1(3),f1ew(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="HFonsite"
    Integer(t_int) :: nn,ilm,jj,l,ig,ir,k,index,deg
    Real(t_real)   :: al,y1,y1ew,xg,rg,wg,jg,texp,sd,sdew,ex,p,pp,norm
    Real(t_real)   :: coef(0:degree)
    Real(t_real)   :: dens(nrp)
    !! DONT FREE THEM
    Real(t_real),Pointer  :: p_xx(:),p_ww(:)
    !!
    !!
    !!
    nn=coulerg_nr
    Call get_gauss(nn,p_xx,p_ww)
    al=(1+2.0_t_real)/fgam
    !!
    f1(:)=0.0_t_real
    f1ew(:)=0.0_t_real
    !!
    p=ew_p_r
    pp=p*p
    !!
    Do ilm=1_t_int,nlmp(isa)
       jj=lmp(ilm,isa)
       l=llilm(jj)
       If(l /= 1 ) Cycle

       k=Mod(jj-1,3_t_int)+1

       ex=-fgam
       Do ir=2,pmesh% nr
          dens(ir)=Sum(lcd(ir,ilm,isa,1:mspin))&
               &*spow(pmesh% p_rmesh(ir),ex)
               
       Enddo
       dens(1)=0.0_t_real

       norm=1.0_t_real/Nl(l)


       y1=0.0_t_real
       y1ew=0.0_t_real
       Do ig=1,nn
          xg=(p_xx(ig)+1.0_t_real)*0.5_t_real
          rg=xg**al*pmesh% rmax
          wg=al*rg/xg
          jg=wg*p_ww(ig)*0.5_t_real

          texp=spow(rg,fgam-2)

          
          sdew=rg*Exp(-(pp*rg*rg))*ac(ilm,isa)*norm
          !!
          index=rmintpl_coefs(degree,2_t_int,deg,pmesh,rg,coef)
          sd=Dot_product(dens(index:index+deg),coef(0:deg))

          
          y1=y1+(sd*texp-sdew)*jg
          y1ew=y1ew+sdew*jg
          
       Enddo                 ! ir
       !!
       !!
       !!
       !!
       !! limiting r->0 value for the l=1 component force formula
       f1(k)=y1*sq4pi*sq3o3
       f1ew(k)=y1ew*sq4pi*sq3o3
       !!
    Enddo                   ! ilm
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine HFonsite
  !!***********************************************************************
  Subroutine prep_for_gga(structure,mspin,potential,p_yb,p_dlcd,p_d2lcd&
       &,p_rvylm,p_qvylm)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Pointer         :: p_yb(:)
    Real(t_real),Pointer         :: p_dlcd(:,:,:,:)
    Real(t_real),Pointer         :: p_d2lcd(:,:,:,:)
    Real(t_real),Pointer         :: p_rvylm(:,:)
    Real(t_real),Pointer         :: p_qvylm(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_for_gga"
    Integer(t_int) :: isa,issa,j,l,ms,ir
    Real(t_real)   :: ex
    Real(t_real),Pointer :: p_tmp(:)
    Type(T_Mesh),Pointer :: pmesha
    !!
    Nullify(p_tmp)
    !!
    !! no GGA -> need less stuff
    Call palloc('p_yb',p_yb,1_t_int,(potential% max_L+1)**2&
         &,nmodule,nsubfunc)
    !!
    If(getxcversion() == XCVER_PBE96 )Then
       !! GGA preparations
       !!
       Call palloc('p_dlcd',p_dlcd&
            &,1_t_int,potential% nrp&
            &,1_t_int,potential% mlmp&
            &,1_t_int,structure% nsite&
            &,1_t_int,mspin&
            &,nmodule,nsubfunc)
       Call palloc('p_d2lcd',p_d2lcd&
            &,1_t_int,potential% nrp&
            &,1_t_int,potential% mlmp&
            &,1_t_int,structure% nsite&
            &,1_t_int,mspin&
            &,nmodule,nsubfunc)
       !!
       !Call init_mesh(imesh)
       !!
       Call palloc('p_tmp',p_tmp,1_t_int,potential% nrp,nmodule,nsubfunc)
       !!
       !!
       Do isa=1,structure% nsite
          issa=structure% p_isrt(isa)
          pmesha    => potential% p_pmeshs(issa)
          
          
          !imesh% Type=LOGZERO_MESH
          !imesh% rmax=pmesha% rmax
          !imesh% rmin=pmesha% rmin
          !imesh% nr=1000
          !imesh% with_zero=.True.
          !Call setup_mesh(imesh)
          
          Do j=1_t_int,potential% p_nlmp(isa)
             !!
             l=llilm(potential% p_lmp(j,isa))
!!$             exf=-2.0_t_real
!!$             If(l==0) exf=2.0_t_real*fgam(issa)-2.0_t_real
!!$             If(l==1) exf=fgam(issa)-1.0_t_real
!!$             ex=-exf-2.0_t_real
             !!TODO: find out how to treat the derivatives
             ex=0.0_t_real
             !Print *,'ex=',ex,l,fgam(issa)
             !!
             Do ms=1,mspin
                Do ir=1,pmesha% nr
                   p_tmp(ir)=potential% p_lcd(ir,j,isa,ms)&
                        &*spow(pmesha% p_rmesh(ir),ex)
                Enddo
                Call dfspline(3_t_int,pmesha% nr,pmesha% p_rmesh,p_tmp&
                     &,p_dlcd(:,j,isa,ms),p_d2lcd(:,j,isa,ms))

                
!!$             Write(str,"('+rho_s',i3.3,'_j',i3.3,'_s',i1.1)")isa,j,ms
!!$             Call mreinter(8_t_int,pmesha,potential% p_lcd(:,j,isa,ms)&
!!$                  &,imesh,tmp)
!!$             Call writefunc(imesh% nr-1,imesh% p_rmesh(2:)&
!!$                  &,tmp(2:),12,Trim(str))
!!$
!!$             Write(str,"('+drho_s',i3.3,'_j',i3.3,'_s',i1.1)")isa,j,ms
!!$             Call mreinter(8_t_int,pmesha,p_dlcd(:,j,isa,ms)&
!!$                  &,imesh,tmp)
!!$             Call writefunc(imesh% nr-1,imesh% p_rmesh(2:)&
!!$                  &,tmp(2:),12,Trim(str))
!!$             Write(str,"('+d2rho_s',i3.3,'_j',i3.3,'_s',i1.1)")isa,j,ms
!!$             Call mreinter(8_t_int,pmesha,p_d2lcd(:,j,isa,ms)&
!!$                  &,imesh,tmp)
!!$             Call writefunc(imesh% nr-1,imesh% p_rmesh(2:)&
!!$                  &,tmp(2:),12,Trim(str))
                
                
             Enddo
          Enddo
          !!
          !Call cleanup_mesh(imesh)
          !!
       Enddo
       !!
       !Call cleanup_mesh(imesh)
       !!
       !!
       Call palloc('p_rvylm',p_rvylm,1_t_int,3_t_int&
            &,1_t_int,(potential% max_L+1)**2,nmodule,nsubfunc)
       Call palloc('p_qvylm',p_qvylm,1_t_int,5_t_int&
            &,1_t_int,(potential% max_L+1)**2,nmodule,nsubfunc)
       !!
       !!
       Call pfree(p_tmp)
    Endif
    !!
    !!
    !!
    Return
    !!
  End Subroutine prep_for_gga
  !!***********************************************************************
End Module pot
