!!START AUTOMATIC HEADER
    !!created by ./MMakefile, Sat May  1 21:03:00 CEST 2010
    !!using conf/gcc-ifort-Linux-x86_64 and templates/precisiondefstmpl 
!!END AUTOMATIC HEADER
!!***********************************************************************
Module PrecisionDefs
  !!@\authors{Klaus Koepernik}
  !!@\name{module}{PrecisionDefs}
  !!@\synopsis{
  !!@Use this module everywhere. 
  !!@  Define all variables as
  !!@  i.e. integer(t_int) , real(t_real), logical(t_log)
  !!@  Define all constants as
  !!@  i.e. 1_t_int, 3.14159_t_real.
  !!@  Exception: integer for internal calls like Allocate(...,STAT=icheck)
  !!@             where icheck should be standard Integer
  !!@}
  !!@\description{
  !!@Define global precision specifiers.
  !!@Set t_int,t_real and t_log to the appropriate type 
  !!@If changed here, all f90 code is changed as well.
  !!@Standard integer kinds:
  !!@ byte(8bit) :  range=1-2   : represents +-10^2 
  !!@ word(16bit):  range=3-4   : represents +-10^4
  !!@ long(32bit):  range=5-9   : represents +-10^9
  !!@ longlong(64bit):  range=10-18 : represents +-10^18
  !!@Standard real kinds: 
  !!@ here we do not specify the bit numbers, since this is machine dependend
  !!@ float:  intrinsic simple real type
  !!@ double:  intrinsic double precision real type
  !!@ longdouble:  intrinsic quad precision real type
  !!@}
  !!@\bugs{If a certain machine does not support all kinds defined here, the
  !!@ compiler will launch an error. Make corresponding corrections here and
  !!@ recompile the whole code. }
  !!@\seealso{}
  !!@\changes{}
  Implicit None 
  !!
  Private
  !!
  Public:: t_real,t_int,t_log
  !!***********************************************************************
  Integer,Private , Parameter :: &
       & byte_range    = 2 &
       &,word_range    = 4 &
       &,long_range    = 9 &
       &,longlong_range=18

  !!@\parameter{byte}{kind specifier for 1 byte integer}
  Integer, Parameter :: byte = Selected_int_kind(byte_range)
  !!@\parameter{word}{kind specifier for 2 byte integer}
  Integer, Parameter :: word = Selected_int_kind(word_range)
  !!@\parameter{long}{kind specifier for 4 byte integer}
  Integer, Parameter :: long = Selected_int_kind(long_range)
  !!@\parameter{longlong}{kind specifier for 8 byte integer}
  Integer, Parameter :: longlong = Selected_int_kind(longlong_range)




  !!@\parameter{float}{kind specifier for single precision}
  Integer, Parameter :: float=kind(1.0) 
  !!@\parameter{double}{kind specifier for duoble precision}
  !!  Integer, Parameter :: Double=Selected_real_kind(15,100)
  Integer, Parameter :: double=Kind(1.d0)
  !!@\parameter{longdouble}{kind specifier for quad precision}
  !!not supported on x86,vasf90-f88: Integer, Parameter :: longdouble=Kind(1.q0)


  !! THIS ARE THE GLOBAL TYPES, USE ONLY THIS AND MAKE GLOBAL CHANGES HERE
  !! PLUS NECCESSARY CORRECTIONS (NEW TYPES) ABOVE

  
  !!START AUTOMATIC DATA TYPES
  Integer,Parameter  :: t_log=long
  Integer,Parameter  :: t_int=longlong
  Integer,Parameter  :: t_real=double
  !!END AUTOMATIC DATA TYPES

  
End Module PrecisionDefs
!!***********************************************************************

