Module roots
  !!
  Use precisiondefs
  Use errors
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="ROOTS"  
  !!
  Public :: zroots,laguer
  !!***********************************************************************
Contains
  !!***********************************************************************
  !!***********************************************************************
  !! laguere method of finding a root of a polynomial
  Subroutine laguer(a,m,x,its)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: m
    Integer(t_int),Intent(inout) :: its
    Complex(t_real),Intent(in)   :: a(m+1)
    Complex(t_real),Intent(out)  :: x
    !!LOCAL
    Character(*),Parameter::nsubfunc="laguer"
    Integer(t_int) ::  iter,j
    Real(t_real)   :: abx,abp,abm,err
    Complex(t_real):: dx,x1,b,d,f,g,h,sq,gp,gm,g2
    Real(t_real),Parameter   :: epss=Epsilon(1.0_t_real)*200.0_t_real
    Integer(t_int),Parameter :: mr=8
    Integer(t_int),Parameter :: mt=10
    Integer(t_int),Parameter :: maxit=mt*mr
    Real(t_real)   :: frac(mr)
    !!
    Save frac
    Data frac /0.5_t_real,0.25_t_real,0.75_t_real,0.13_t_real,0.38_t_real&
         &,0.62_t_real,0.88_t_real,1.0_t_real/
    !!
!    Do j=1,m+1
!       Write(outfile,10)j-1,a(j)
!10     Format('a(',i3,')=(',e30.15e3,',',e30.15e3,')')
!    Enddo
!    Write(outfile,*)
    Do iter=1,maxit
       its=iter
       b=a(m+1)
       err=Abs(b)
       d=Cmplx(0.0_t_real,0.0_t_real,t_real)
       f=Cmplx(0.0_t_real,0.0_t_real,t_real)
       abx=Abs(x)
       Do j=m,1,-1
          f=x*f+d
          d=x*d+b
          b=x*b+a(j)
          err=Abs(b)+abx*err
       Enddo
       err=epss*err
       !Print "('x=',2(f30.15,1x),' b=',2(f30.15,1x))",x,b
       !Print *,'err ',err
       If(Abs(b)<=err) Then
          Return
       Else
          g=d/b
          g2=g*g
          h=g2-2.0_t_real*f/b
          sq=Sqrt(Real(m-1,t_real)*(Real(m*h,t_real)-g2))
          gp=g+sq
          gm=g-sq
          abp=Abs(gp)
          abm=Abs(gm)
          If(abp<abm) gp=gm
          If (Max(abp,abm)>0.0_t_real) Then
             dx=Real(m,t_real)/gp
          Else
             dx=Exp(Cmplx(Log(1.0_t_real+abx),Real(iter,t_real),t_real))
          Endif
       Endif
       x1=x-dx
       !Print *,'x,x1 ',x,x1
       If(x==x1)Return
       If (Mod(iter,MT)/=0) Then
          x=x1
       Else
          x=x-dx*frac(iter/MT)
       Endif
    Enddo
    !!
    !Print *,Sqrt()*2
    Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="' too many iterations in laguer',/,' Please report!'")
    !!
    Return
    !!
  End Subroutine laguer
  !!  (C) Copr. 1986-92 Numerical Recipes Software -0'.
  !!***********************************************************************
  Subroutine zroots(a,m,roots,polish)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: m
    Logical(t_log),Intent(in)    :: polish
    Complex(t_real),Intent(in)   :: a(m+1)
    Complex(t_real),Intent(out)  :: roots(m)
    !!LOCAL
    Character(*),Parameter::nsubfunc="zroots"
    Real(t_real),Parameter :: eps=Epsilon(1.0_t_real)*10.0_t_real
    Integer(t_int) :: i,j,jj,its
    Complex(t_real):: ad(m+1),x,b,c
    !!
    ad(1:m+1)=a(1:m+1)
    !!    
    Do j=m,1,-1
       x=Cmplx(0.0_t_real,0.0_t_real,t_real)
       !Print *,'search root j=',j
       Call laguer(ad,j,x,its)
       !Print *,'root j=',j,x
       If(Abs(Aimag(x))<=2.0_t_real*eps*Abs(Real(x,t_real))) &
            & x=Cmplx(Real(x,t_real),0.0_t_real,t_real)
       roots(j)=x
       b=ad(j+1)
       Do jj=j,1,-1
          c=ad(jj)
          ad(jj)=b
          b=x*b+c
       End Do
    End Do
    !!
    If (polish) Then
       Do j=1,m
          Call laguer(a,m,roots(j),its)
       End Do
    Endif
    !!
    Do j=2,m
       x=roots(j)
       Do i=j-1,1,-1
          If(Real(roots(i),t_real)<= Real(x,t_real)) Goto 10
          roots(i+1)=roots(i)
       End Do
       i=0
10     Continue
       roots(i+1)=x
    End Do
     !!
    Return
  End Subroutine zroots
  !!  (C) Copr. 1986-92 Numerical Recipes Software -0'.
  !!***********************************************************************
End Module roots
