Module shape
  !!
  Use precisiondefs
  Use errors
  Use constants
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="SHAPE"
  !!
  Public :: cheese_shape
  !!***********************************************************************
Contains
  !!***********************************************************************
  !! calculate the local voronoi like shapefunction and the 'cheese' factors
  !!
  Real(t_real) Function cheese_shape(rsh,isa,nsite,isrt,rcompsup&
       &,npshell,ninpshell,pairabs,pairs,rsite)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: isa,nsite,isrt(*)
    Integer(t_int),Intent(in)    :: npshell(nsite,nsite)
    Integer(t_int),Intent(in)    :: ninpshell(:,:,:)
    Real(t_real),Intent(in)      :: pairabs(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    Real(t_real),Intent(in)      :: rsite(*),rcompsup(*)
    Real(t_real),Intent(in)      :: rsh(3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="cheese_shape"
    Integer(t_int) :: issa,isb,issb,n,ne,nf,ish,insh
    Real(t_real)   :: fac0,fac1
    Real(t_real)   :: rnna,rnnb,ra,rmaxa,rmaxb,rmax2,fsum,fp,fq,f0
    Real(t_real)   :: rb
    Real(t_real)   :: r2(3)
    !!
    !!
    !!
    issa=isrt(isa)
    rnna=rsite(issa)*2.0_t_real
    rmaxa=rcompsup(issa)
    fac0=0.25_t_real
    fac1=(1.0_t_real-fac0)
    
    ra=Sqrt(Dot_product(rsh,rsh))
    !!
    !Print *,Trim(nsubfunc),'ra=',ra
    !!
!    If(stiffness==2)Then
       fsum=fsh_cos_stiff2(rsh,isa,nsite,isrt,rcompsup&
            &,npshell,ninpshell,pairabs,pairs)&
            &*dprofile(ra,0.0_t_real,rmaxa)
!    Endif
    !!
    f0=fsum
    !Print *,Trim(nsubfunc),'ra=',ra,' f0=',f0
    !!
    fp=dprofile(ra,rnna*fac0,rnna*fac1)
    fq=1.0_t_real-fp
    !!
    isbloop: Do isb=1,nsite
       issb=isrt(isb)
       rnnb=rsite(issb)*2.0_t_real
       rmaxb=rcompsup(issb)
       rmax2=rmaxa+rmaxb
       !!
       n=0_t_int
       ishloop: Do ish=1,npshell(isb,isa)
          nf=n+1_t_int
          ne=n+ninpshell(ish,isb,isa)
          n=ne
          If(pairabs(ish,isb,isa) > (rmax2)) Cycle ishloop
          inshloop: Do insh=nf,ne
             r2(:)=rsh(:)-pairs(:,insh,isb,isa)
             !!
             !!
             rb=Sqrt(Dot_product(r2,r2))
             If(rb <= rmaxb ) Then
                
                fsum=fsum+fsh_cos_stiff2(r2,isb,nsite,isrt,rcompsup&
                     &,npshell,ninpshell,pairabs,pairs)&
                     &*dprofile(rb,0.0_t_real,rmaxb)
                !!
                fq=fq-dprofile(rb,rnnb*fac0,rnnb*fac1)
             Endif
          Enddo inshloop
       End Do ishloop
    End Do isbloop
    !!
    If(f0 /= 0.0_t_real ) Then
       If(fsum==0.0_t_real)Then
          Write(errbuf,15) rsh,ra
15        Format("'shape normalizer is zero: r= ",f10.5,',',f10.5,',',f10.5&
               &," , |r|=",f10.5,"',/,' please report this error, &
               &including all input files!'")
          Call errmsg(nmodule,nsubfunc,E_fatalerror ,msg=errbuf)
       Endif
       f0=f0/fsum
    Endif
    If(ra==0.0_t_real)f0=1.0_t_real
    !!
    cheese_shape=fp+fq*f0
    !!
    !!
  End Function cheese_shape
  !!***********************************************************************
  !! calculate the local voronoi like raw shape function
  !! it is defined through a factor function build from all atoms, 
  !! whose spheres overlap the central sphere. It is continues
  !! with respect to shifts of atoms, even if a new sphere enters the
  !! defining set.
  !! 
  Real(t_real) Function fsh_cos_stiff2(rsh,isa,nsite,isrt,rcompsup&
       &,npshell,ninpshell,pairabs,pairs)
    !!
    Implicit none  
    !!IO
    Integer(t_int),Intent(in)    :: nsite,isa,isrt(*)
    Integer(t_int),Intent(in)    :: npshell(nsite,nsite)
    Integer(t_int),Intent(in)    :: ninpshell(:,:,:)
    Real(t_real),Intent(in)      :: rsh(3),rcompsup(*)
    Real(t_real),Intent(in)      :: pairabs(:,:,:)
    Real(t_real),Intent(in)      :: pairs(:,:,:,:)
    !!LOCAL
    Integer(t_int) :: isc,ish,insh,n,ne,nf
    Real(t_real)   :: f,pp(3),rbig,d0,d1,d,rp,rmaxa,rmaxc,rmax2
    !!
    !!
    !!TODO: reduce number of pair loops by predetermined radius condition
    !!
    rmaxa=rcompsup(isrt(isa))
    f=1.0_t_real
    iscloop1: Do isc=1,nsite
       rmaxc=rcompsup(isrt(isc))
       rmax2=rmaxa+rmaxc
       n=0_t_int
       ishloop: Do ish=1,npshell(isc,isa)
          nf=n+1_t_int
          ne=n+ninpshell(ish,isc,isa)
          n=ne
          rbig=pairabs(ish,isc,isa)
          If(rbig > (rmax2)) Cycle ishloop
          d0=Max(rbig-rmaxc,0.0_t_real)
          d1=Min(rbig,rmaxa)
          inshloop: Do insh=nf,ne
             pp(:)=pairs(:,insh,isc,isa)
             rp=Dot_product(rsh(:),pp(:))/rbig
             If(rp<=d0) Cycle inshloop !! factor is 1
             !!
             If(rp>=d1) Then !! factor is 0
                f=0.0_t_real
                Exit iscloop1
             Endif
             !!
             !!
             d=(rp-d0)/(d1-d0)
             !! stiffness 2
             f=f*(1.0_t_real-fc(fc(d)))
             !!
          Enddo inshloop
       End Do ishloop
       !!
    End Do iscloop1
    !!
    fsh_cos_stiff2=f
    !!
  Contains
    Real(t_real) Function  fc(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in) :: x
      !!LOCAL
      fc=(1.0_t_real-Cos(pi*x))*0.5_t_real
    End Function fc
    !!
  End Function fsh_cos_stiff2
  !!***********************************************************************
  !! a profile function
  Real(t_real) Function dprofile(r,rmin,rmax)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: r,rmin,rmax
    !!LOCAL
    Real(t_real)   :: x
    !!
    dprofile=0.0_t_real
    If(r>=rmax) Return
    If(r<=rmin)Then
       dprofile=1.0_t_real
       Return
    Endif
    !!
    x=(r-rmin)/(rmax-rmin)
    !! cosine profile
    dprofile=(1.0_t_real-fc(fc(x)))
    !!
  Contains
    Real(t_real) Function  fc(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in) :: x
      !!LOCAL
      fc=(1.0_t_real-Cos(pi*x))*0.5_t_real
    End Function fc
    !!
  End Function dprofile
  !!***********************************************************************
End Module shape
