Module sort
  !!
  Use precisiondefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="SORT"  
  !!
  Public :: ipiksrt1,iuniq1,dshel2,dshell,ishell,det_degbands
  !!***********************************************************************
contains
  !!***********************************************************************
  !***********************************************************************
  !  (c) copr. 1986-92 numerical recipes software -0'.
  !
  Subroutine ipiksrt1(iarr,n)
    !
    !***********************************************************************
    !
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: n
    Integer(t_int),Intent(inout) :: iarr(:)
    !!LOCAL
    Integer(t_int) :: j,ia,i
    !***********************************************************************
    !     
    Do j=2_t_int,n
       ia=iarr(j)
       Do i=j-1_t_int,1_t_int,-1_t_int
          If(iarr(i) <= ia) Goto 10
          iarr(i+1_t_int)=iarr(i)
       Enddo
       i=0_t_int
10     iarr(i+1_t_int)=ia
    Enddo
    !     
  End Subroutine ipiksrt1
  !***********************************************************************
  !***********************************************************************
  !     . iarr is sorted, uniq gives 1..nap non equal elements  
  Subroutine iuniq1(iarr,n,nap)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)   :: n
    Integer(t_int),Intent(out)  :: nap
    Integer(t_int),Intent(inout):: iarr(:)
    !!LOCAL
    Integer(t_int) :: ia,i,j
    !***********************************************************************
    !     
    ia=iarr(1_t_int)
    i=1_t_int
    Do j=2_t_int,n
       If (iarr(j) > ia) Then
          ia=iarr(j)
          i=i+1_t_int
          iarr(i)=iarr(j)
       Endif
    Enddo
    Do j=i+1_t_int,n
       iarr(j)=0_t_int
    End Do
    nap=i
    !     
  End Subroutine iuniq1
  !
  !***********************************************************************
  !***********************************************************************
  Subroutine dshel2(m,n,darray,iwk,tol0)
    !!- shell sort of a array of double precision vectors
    !!BEGIN REMARK
    !! corrected by Klaus Koepernik
    !! the comparison of double precision vectors has to include
    !! a tolerance
    !! the default tol=1.d-12 is set , if tol.eq.0.0
    !! if problems with sorting occur
    !! enhance the accuracy of the input data, or increase the tolerance
    !!END REMARK
    !! ----------------------------------------------------------------------
    !!i Inputs:
    !!i   m     :number of components in darray
    !!i   n     :number of elements in darray
    !!i   darray:array to be sorted
    !!i   tol   :tolerance
    !!o Outputs:
    !!o   iwk   :table of indices to array darray
    !! ----------------------------------------------------------------------
    Implicit None
    !!IO
    Integer(t_int),Intent(in)  :: m,n
    Real(t_real),Intent(in)   :: tol0
    Real(t_real),Intent(in)   :: darray(m,0:n-1)
    Integer(t_int),Intent(out) :: iwk(n)
    !!LOCAL 
    Integer(t_int) ::  lognb2,i,j,k,l,n2,nn,it,mm
    Real(t_real) :: dif,tol
    !!
    tol=tol0
    If(tol == 0.0_t_real) tol=1.e-12_t_real
    lognb2 = Int(Log(Real(n+1_t_int,t_real))*1.4426950_t_real)
    n2 = n
    Do i = 1_t_int, n
       iwk(i) = i-1_t_int
    Enddo

    nnloop: Do nn = 1_t_int, lognb2
       n2 = n2/2_t_int
       k = n - n2
       jloop: Do  j = 1_t_int, k
          i = j
3         Continue
          l = i + n2
          mmloop: Do  mm = 1_t_int, m
             dif=(darray(mm,iwk(l))- darray(mm,iwk(i)))
             If (dif > Abs(tol)) Exit mmloop
             If (Abs(dif) < Abs(tol)) Cycle mmloop
             
             it = iwk(i)
             iwk(i) = iwk(l)
             iwk(l) = it
             i = i - n2
             If (i >= 1_t_int) Goto 3
             Exit mmloop
             
          End Do mmloop
          
       Enddo jloop
       
    End Do nnloop
    Return
    
  End Subroutine dshel2
  !!***********************************************************************
  Subroutine dshell(m,n,darray,tol0)
    !!- shell sort of a array of double precision vectors
    !!BEGIN REMARK
    !! corrected by Klaus Koepernik
    !! the comparison of double precision vectors has to include
    !! a tolerance
    !! the default tol=1.d-12 is set , if tol.eq.0.0
    !! if problems with sorting occur
    !! enhance the accuracy of the input data, or increase the tolerance
    !!END REMARK
    !! ----------------------------------------------------------------------
    !!i Inputs:
    !!i   m     :number of components in darray
    !!i   n     :number of elements in darray
    !!io  darray:array to be sorted
    !!o Outputs:
    !!io  darray:array to be sorted
    !! ----------------------------------------------------------------------
    Implicit None
    !!IO
    Integer(t_int),Intent(in)  :: m,n
    Real(t_real),Intent(in)   :: tol0
    Real(t_real),Intent(inout)   :: darray(m,0:n-1)
    !!LOCAL
    Integer(t_int) :: lognb2,i,j,k,l,n2,nn,mm,mmm
    Real(t_real)  :: dt,dif,tol
    !!

    tol=tol0
    If(tol == 0.0_t_real) tol=1.e-12_t_real
    lognb2 = Int(Log(Real(n+1_t_int,t_real))*1.4426950_t_real)
    n2 = n
    nnlopp: Do nn = 1_t_int, lognb2
       n2 = n2/2_t_int
       k = n - n2
       jloop: Do  j = 1_t_int, k
          i = j - 1_t_int
3         Continue
          l = i + n2
          mmloop: Do  mm = 1_t_int, m
             dif=(darray(mm,l)- darray(mm,i))
             If (dif > Abs(tol)) Exit mmloop
             If (Abs(dif) < Abs(tol)) Cycle mmloop
             
             Do mmm = 1_t_int, m
                dt = darray(mmm,i)
                darray(mmm,i) = darray(mmm,l)
                darray(mmm,l) = dt
             Enddo
             i = i - n2
             If (i >= 0_t_int) Goto 3
             Exit mmloop
          Enddo mmloop
       Enddo jloop
    Enddo nnlopp
    !!
  End Subroutine dshell
  !!***********************************************************************
  
  Subroutine ishell(m,n,iarray)
    !!- shell sort of a array of integer vectors
    !! ----------------------------------------------------------------------
    !! Inputs:
    !!   m     :number of components in iarray
    !!   n     :number of elements in iarray
    !!   iarray:array to be sorted
    !! Outputs:
    !!   iarray:array to be sorted
    !! ----------------------------------------------------------------------
    Implicit None
    !!IO
    Integer(t_int) ,Intent(in) :: m,n
    Integer(t_int),Intent(out) :: iarray(m,0:n-1)
    !!LOCAL
    Integer(t_int) :: lognb2,i,j,k,l,n2,nn,it,mm,mmm
    !!

    lognb2 = Int(Log(Real(n+1_t_int,t_real))*1.4426950_t_real)
    n2 = n
    Do nn = 1_t_int, lognb2
       n2 = n2/2_t_int
       k = n - n2
       Do j = 1_t_int, k
          i = j - 1_t_int
3         Continue
          l = i + n2
          mmloop: Do mm = 1_t_int, m
             
             If (iarray(mm,l) > iarray(mm,i))  Exit mmloop
             If (iarray(mm,l) == iarray(mm,i)) Cycle mmloop
             
             Do mmm = 1_t_int, m
                it = iarray(mmm,i)
                iarray(mmm,i) = iarray(mmm,l)
                iarray(mmm,l) = it
             Enddo
             i = i - n2
             If (i >= 0_t_int) Goto 3
             Exit mmloop
          End Do mmloop
          
       End Do
       
    Enddo
    
  End Subroutine ishell
  !!***********************************************************************
  Subroutine det_degbands(nband,eb,ndegbands,degbands,tol)
    !! determine the indices of degenerate values in eb
    !! assuming that eb is sorted
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nband
    Integer(t_int),Intent(out)   :: ndegbands
    Integer(t_int),Intent(out)   :: degbands(:)
    Real(t_real),Intent(in)      :: tol
    Real(t_real),Intent(in)      :: eb(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="det_degbands"
    Integer(t_int) :: iband,in
    Real(t_real)   :: e,d
    !!
    !!
    e=eb(1)
    iband=1
    in=1
    ndegbands=1
    degbands(ndegbands)=1
    Do iband=1,nband
       d=Abs(e-eb(iband))
       If(Abs(e)>1.0_t_real)d=d/Abs(e)
       If(d<tol)Then
          in=iband
       Else
          ndegbands=ndegbands+1
          degbands(ndegbands)=iband
          e=eb(iband)
          in=iband
       Endif
    Enddo
    ndegbands=ndegbands+1
    degbands(ndegbands)=nband+1
    !!
    Return
    !!
  End Subroutine det_degbands
  !!***********************************************************************

End Module sort
