Module spline
  !!
  Use precisiondefs
  Use errors
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="SPLINE"  
  !!
  Public :: cspline,splint,equspline,equsplint,nanspline
  !!***********************************************************************

contains
  !!***********************************************************************
  !! spline including the not-a-not condition
  Subroutine nanspline(n,x,f,d2f,work,stype,df1,dfN)
    !!
    Use bandmatrix
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: stype
    Integer(t_int),Intent(in)    :: n
    Real(t_real),Intent(in)      :: x(n),f(n)
    Real(t_real),Intent(in),Optional :: df1,dfN
    Real(t_real),Intent(out)     :: work(n),d2f(n)
    !!LOCAL
    Character(*),Parameter::nsubfunc="nanspline"
    Integer(t_int) :: i
    Real(t_real)   :: p,ddy,dd,sig,sig2,sig3,sigNm1
    Real(t_real)   :: AA1,BB1,CC1,e1,c1,c2,a1
    Real(t_real)   :: AAN,BBN,CCN,eN,cNm1,cNm2,aN
    !!
    If(n<4) Call terminate(TERM_CRASH,'Spline: less than 4 points?!')
    !!
    c1=x(2)-x(1)
    c2=x(3)-x(2)
    sig2=c1/c2
    sig3=c2/(x(4)-x(3))
    cNm1=x(N)-x(N-1)
    cNm2=x(N-1)-x(N-2)
    sigNm1=cNm2/cNm1
    !!
    !If(stype==1_t_int)Then
    !! not-a-knot condition
    AA1=-1.0_t_real/c1
    CC1=-1.0_t_real/c2
    BB1=-AA1-CC1
    a1=0.0_t_real
    !! not-a-knot condition
    AAN=-1.0_t_real/cNm1
    CCN=-1.0_t_real/cNm2
    BBN=-AAN-CCN
    aN=0.0_t_real
    If(stype==2_t_int)Then
       !! natural spline
       AA1= 1.0_t_real
       BB1= 0.0_t_real
       CC1= 0.0_t_real
       a1=0.0_t_real
       !! natueral spline
       AAN= 1.0_t_real
       BBN= 0.0_t_real
       CCN= 0.0_t_real
       aN=0.0_t_real
    Endif
    If(Present(df1))Then
       !! left clamped spline 
       AA1= 2.0_t_real
       BB1= 1.0_t_real
       CC1= 0.0_t_real
       a1=6.0_t_real*( (f(2)-f(1))/c1-df1 )/c1
    Endif
    If(Present(dfN))Then
       !! right clamped spline 
       AAN= 2.0_t_real
       BBN= 1.0_t_real
       CCN= 0.0_t_real
       aN=6.0_t_real*( dfN-(f(n)-f(n-1))/cNm1 )/cNm1
    Endif
    !!
    !!
    AA1=1.0_t_real/AA1
    BB1=BB1*AA1
    CC1=CC1*AA1
    AAN=1.0_t_real/AAN
    BBN=BBN*AAN
    CCN=CCN*AAN
    !!
    e1=sig2*CC1/((BB1-2.0_t_real)*sig2-2.0_t_real)
    eN=CCN/(BBN-2.0_t_real-2.0_t_real*sigNm1)
    !!
    If(n==4)Then
       BBN=BBN-CCN*e1
       eN=CCN/(BBN-2.0_t_real-(2.0_t_real-e1)*sigNm1)
    Endif
    !!
    !!
    !!
    d2f(1)=0.0_t_real
    work(1)=0.0_t_real
    Do i=2,n-1
       ddy=6.0_t_real*((f(i+1)-f(i))/(x(i+1)-x(i))&
            &-(f(i)-f(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i))
       If(i==2)     ddy=ddy-AA1*a1
       If(i==N-1)   ddy=ddy-AAN*aN

       sig=(x(i)-x(i-1))/(x(i+1)-x(i))
       !!
       dd=0.0_t_real
       If(i==2)   dd=dd-BB1*sig2
       If(i==3)   dd=dd-e1*sig3
       If(i==N-2) dd=dd-eN
       If(i==N-1) dd=dd-BBN
       !!
       p=(2.0_t_real*(1.0_t_real+sig)+sig*d2f(i-1)+dd)
       d2f(i)=-1.0_t_real/p
       work(i)=(ddy-sig*work(i-1))/p
    Enddo
    !!
    !!
    d2f(n-1)=work(n-1)
    Do i=n-2,2,-1
       d2f(i)=work(i)+d2f(i)*d2f(i+1)
    Enddo

    !!
    !!
    d2f(N-1)=d2f(N-1)-eN*d2f(N-2)
    d2f(N)=aN*AAN-BBN*d2f(N-1)-CCN*d2f(N-2)
    !!
    d2f(2)=d2f(2)-e1*d2f(3)
    d2f(1)=a1*AA1-BB1*d2f(2)-CC1*d2f(3)
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine nanspline
  !!***********************************************************************
  !!   (C) Copr. 1986-92 Numerical Recipes Software -0'.
  Subroutine cspline(n,x,y,dy1,dyn,d2y,u,natural)
    !!
    Implicit None
    !!IO
    !! if natural is true then the spline condition is set to zero second
    !! derivative at the mesh end points (natural spline) if false
    !! the first derivatives at the mesh end points is taken from dy1 an dyn
    Logical(t_log),Intent(in)    :: natural
    !! number of points
    Integer(t_int),Intent(in)    :: n
    !! abszissa and function values
    Real(t_real),Intent(in)      :: x(n),y(n)
    !! the first derivatives of y at the first and last point
    Real(t_real),Intent(in)      :: dy1,dyn
    !! the array of second derivatives
    Real(t_real),Intent(out)     :: d2y(n)
    !! work array
    Real(t_real),Intent(inout)   :: u(n)
    !!LOCAL
    Integer(t_int) :: i,k
    Real(t_real)   :: p,qn,sig,un
    !!
    If (dy1 > 0.99e30_t_real .Or. natural) Then
       d2y(1)=0.0_t_real
       u(1)=0.0_t_real
    Else
       d2y(1)=-0.5_t_real
       u(1)=(3.0_t_real/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-dy1)
    Endif
    !!
    Do i=2_t_int,n-1_t_int
       sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
       p=sig*d2y(i-1)+2.0_t_real
       d2y(i)=(sig-1.0_t_real)/p
       u(i)=(6.0_t_real*((y(i+1)-y(i))/(x(i+1)-x(i))&
            &-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
    Enddo
    !!
    If (dyn > 0.99e30_t_real .Or. natural) Then
       qn=0.0_t_real
       un=0.0_t_real
    Else
       qn=0.5_t_real
       un=(3.0_t_real/(x(n)-x(n-1)))*(dyn-(y(n)-y(n-1))/(x(n)-x(n-1)))
    Endif
    d2y(n)=(un-qn*u(n-1))/(qn*d2y(n-1)+1.0_t_real)
    !!
    Do i=n-1_t_int,1_t_int,-1_t_int
       d2y(i)=d2y(i)*d2y(i+1)+u(i)
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine cspline

  !!***********************************************************************
  !! modified version for equidistant meshs
  Subroutine equspline(n,hx,y,dy1,dyn,d2y,u,natural)
    !!
    Implicit None
    !!IO
    !! if natural is true then the spline condition is set to zero second
    !! derivative at the mesh end points (natural spline) if false
    !! the first derivatives at the mesh end points is taken from dy1 an dyn
    Logical(t_log),Intent(in)    :: natural
    !! number of points
    Integer(t_int),Intent(in)    :: n
    !! abszissa and function values
    Real(t_real),Intent(in)      :: y(n)
    !! x step width
    Real(t_real),Intent(in)      ::  hx
    !! the first derivatives of y at the first and last point
    Real(t_real),Intent(in)      :: dy1,dyn
    !! the array of second derivatives
    Real(t_real),Intent(out)     :: d2y(n)
    !! work array
    Real(t_real),Intent(inout)   :: u(n)
    !!LOCAL
    Integer(t_int) :: i,k
    Real(t_real)   :: p,qn,un
    !!
    If (dy1 > 0.99e30_t_real .Or. natural) Then
       d2y(1)=0.0_t_real
       u(1)=0.0_t_real
    Else
       d2y(1)=-0.5_t_real
       u(1)=(3.0_t_real/hx)*((y(2)-y(1))/hx-dy1)
    Endif
    !!
    Do i=2_t_int,n-1_t_int
       p=d2y(i-1)+4.0_t_real
       d2y(i)=-1.0_t_real/p
       u(i)=(6.0_t_real*((y(i+1)-2.0_t_real*y(i)+y(i-1))/hx)/hx-u(i-1))/p
    Enddo
    !!
    If (dyn > 0.99e30_t_real .Or. natural) Then
       qn=0.0_t_real
       un=0.0_t_real
    Else
       qn=0.5_t_real
       un=(3.0_t_real/hx)*(dyn-(y(n)-y(n-1))/hx)
    Endif
    d2y(n)=(un-qn*u(n-1))/(qn*d2y(n-1)+1.0_t_real)
    !!
    Do i=n-1_t_int,1_t_int,-1_t_int
       d2y(i)=d2y(i)*d2y(i+1)+u(i)
    Enddo
    !!
    !!
    Return
    !!
  End Subroutine equspline
  !!***********************************************************************
  !! (C) Copr. 1986-92 Numerical Recipes Software -0'.
  Subroutine splint(n,x,y,d2y,xp,yp,dyp,d2yp)
    !!
    Implicit None  
    !!IO
    !! number of mesh points
    Integer(t_int),Intent(in)    :: n
    !! tables of abszissas, function values and second derivatives
    Real(t_real),Intent(in)      :: x(n),y(n),d2y(n)
    !! the x for which y(x) shall be returned
    Real(t_real),Intent(in)      :: xp
    !! the value y(x)
    Real(t_real),Intent(out)     :: yp,dyp,d2yp
    !!LOCAL
    Integer(t_int) :: k,khi,klo
    Real(t_real)   :: a,b,h
    !!
    !!TODO: make bisection outside problem
    !!TODO: make for equidistant
    !!TODO: manage outside mesh cases
    klo=1_t_int
    khi=n
    !!
    Do While(khi-klo > 1)
       k=(khi+klo)/2_t_int
       If(x(k) > xp)Then
          khi=k
       Else
          klo=k
       Endif
    End Do
    !!
    h=x(khi)-x(klo)
    !!
    If (h == 0.0_t_real) Stop 'bad x input in splint'
    !!
    a=(x(khi)-xp)/h
    !!
    b=(xp-x(klo))/h
    !!
    
    yp=a*y(klo)+b*y(khi)+((a**3-a)*d2y(klo)+(b**3-b)*d2y(khi))*(h**2)&
         &/6.0_t_real
    !!
    dyp=(y(khi)-y(klo))/h&
         &+(3.0_t_real*(b**2*d2y(khi)-a**2*d2y(klo))-(d2y(khi)-d2y(klo))&
         &)*h/6.0_t_real

    d2yp=b*d2y(khi)+a*d2y(klo)
    !!
    Return
    !!
  End Subroutine splint
  !!***********************************************************************
  !! (C) Copr. 1986-92 Numerical Recipes Software -0'.
  Subroutine equsplint(n,hx,x,y,d2y,xp,yp)
    !!
    Implicit None  
    !!IO
    !! number of mesh points
    Integer(t_int),Intent(in)    :: n
    !! x step width
    Real(t_real),Intent(in)      ::  hx
    !! tables of abszissas, function values and second derivatives
    Real(t_real),Intent(in)      :: x(n),y(n),d2y(n)
    !! the x for which y(x) shall be returned
    Real(t_real),Intent(in)      :: xp
    !! the value y(x)
    Real(t_real),Intent(out)     :: yp
    !!LOCAL
    Integer(t_int) :: k,khi,klo,index
    Real(t_real)   :: a,b,h
    !!
    !!TODO: make bisection outside problem
    !!TODO: make for equidistant
    !!TODO: manage outside mesh cases
    klo=1_t_int
    khi=n
    !!
    Do While(khi-klo > 1)
       k=(khi+klo)/2_t_int
       If(x(k) > xp)Then
          khi=k
       Else
          klo=k
       Endif
    End Do
    !!
    !!TODO: treat over and underflow
    !!
    index=Floor((xp-x(1_t_int))/hx)+1_t_int
    index=Max(1,Min(index,n-1))
    If(index/=klo)Then
       Print *,'x1   =',x(1)
       Print *,'xp   =',xp
       Print *,'xN   =',x(n)
       Print *,'klo  =',klo
       Print *,'index=',index
       stop 'klo'
    Endif
    !!
    h=hx
    !!
    If (h == 0.0_t_real) Stop 'bad x input in splint'
    !!
    a=(x(khi)-xp)/h
    !!
    b=(xp-x(klo))/h
    !!
    yp=a*y(klo)+b*y(khi)+((a**3-a)*d2y(klo)+(b**3-b)*d2y(khi))*(h**2)&
         &/6.0_t_real
    !!
    Return
    !!
  End Subroutine equsplint
  !!***********************************************************************

End Module spline
