!!***********************************************************************
Program spline_it
  !!
  Use precisiondefs
  Use memory
  Use file_iounits
  Use errors
  Use tio
  Use bspline
  Use ioadapter
  Use versioncontrol
  Use ioaux
  !!
  Implicit None
  !!
  !Integer,Parameter  :: long_range    = 9
  !Integer,Parameter  :: t_log=Selected_int_kind(long_range)
  !Integer,Parameter  :: t_int=Selected_int_kind(long_range)
  !Integer,Parameter  :: t_real=Kind(1.d0)
  !!
  Character(*),Parameter::nmodule="PROGRAM"    
  Character(*),Parameter::nsubfunc="main"    
  !!
  !!***********************************************************************
  !!
  Include '../mainversion.h' 
  Include '../subversion.h' 
  Include '../release.h' 
  !!
  APPLICATIONNAME="splineit"
  !!
  !!
  g_debug=.True.
  !!
  Open(errfile,file=f_error,status='unknown')
  Close(errfile,status='delete')
  !!
  Call sizetest
  If( debugfun(nmodule,nsubfunc,'mem_stat') ) Then
     Call init_mem_stat(.True.)
  Else
     Call init_mem_stat(.False.)
  Endif
  !!
  Call splineit()
  !!
Contains
  !!***********************************************************************
  Subroutine splineit
    !!
    Implicit None 
    !!
    !Integer,Parameter  :: long_range    = 9
    !Integer,Parameter  :: t_log=Selected_int_kind(long_range)
    !Integer,Parameter  :: t_int=Selected_int_kind(long_range)
    !Integer,Parameter  :: t_real=Kind(1.d0)
    !!
    !!IO
    !!LOCAL
    Character(*),Parameter::nsubfunc="splineit"
    Character(2000) :: fname,oname,buffer,meth,word,foutname
    Integer(t_int) :: nline,i,jderiv,p,k,j,ileft,nderiv,no
    Real(t_real)   :: dum,a,bsf,x,rmin,rmax,sf
    Real(t_real),Pointer :: r(:),f(:),work(:),knots(:),p_q(:),bwork(:)
    Real(t_real),Pointer :: BiAtX(:,:),derivleft(:),derivright(:),ro(:)
    Integer :: eof
    Logical(t_log) :: exists
    Character(5000):: buf
    !!
    Nullify(r,f,work,knots,p_q,bwork,BiAtX,derivleft,derivright,ro)
    !!
    Inquire(file='=.splineit',exist=exists)
    If(.Not.exists) Then
       Open(12,file='=.splineit')
       Write(12,*)'# data file name'
       Write(12,*)'e'
       Write(12,*)'# output file name'
       Write(12,*)'es'
       Write(12,*)'#'
       Write(12,*)'# nr(output) degree(odd) which-derivative'
       Write(12,*)'100  3  0'
       Write(12,*)'# method rmin rmax (inp | equi rmin rmax | file filename) '
       Write(12,*)'  equi     0            1'
       Close(12)
       Call terminate(TERM_NORMAL,'Spline: input file creation')
    Endif
    !!
    meth='inp'
    rmin=0.0_t_real
    rmax=1.0_t_real
    Open(12,file='=.splineit')
    Read(12,*)
    Read(12,*)fname
    Read(12,*)
    Read(12,*)oname
    Read(12,*)
    Read(12,*)
    Read(12,*)no, p, jderiv
    Read(12,fmt=*,iostat=eof)
    If(eof < 0 ) goto 1000
    Read(12,fmt="(a)",iostat=eof) buffer
    If(eof < 0 ) goto 1000
1000 Continue
    Close(12)
    !!
    If(.Not.next_word(buffer,word)) Then
       Print *,'set output mesh to auto'
    Else
       !Print *,'word<',Trim(word),'>'
       !Print *,'buffer<',Trim(buffer),'>'
       If(word(1:3)=='equ') Then
          meth='equ'
          If(.Not.next_word(buffer,word)) Call terminate(TERM_CRASH&
                  &,Trim(nsubfunc)//' missing argument rmin in output-mesh line')
          
          Read(word,*,iostat=eof)rmin
          If(eof /= 0 ) Call terminate(TERM_CRASH&
               &,Trim(nsubfunc)//' missing valid argument rmin in output-mesh line')
          If(.Not.next_word(buffer,word)) Call terminate(TERM_CRASH&
                  &,Trim(nsubfunc)//' missing argument rmax in output-mesh line')
          
          Read(word,*,iostat=eof)rmax
          If(eof /= 0 ) Call terminate(TERM_CRASH&
               &,Trim(nsubfunc)//' missing valid argument rmax in output-mesh line')
          
       Elseif(word(1:3)=='inp') Then
          meth='inp'
       Elseif(word(1:4)=='file') Then
          meth='file'
          !Print *,'word<',Trim(word),'>'
          !Print *,'buffer<',Trim(buffer),'>'
          If(.Not.next_word(buffer,word)) Call terminate(TERM_CRASH&
               &,Trim(nsubfunc)//' missing filename argument in&
               & output-mesh line')
          foutname=Trim(word)
       Else
          Print "(/,' Unknwon output method ',a, '!',/&
               &,' Possible value for line 9:',/&
               &,' * equidistant mesh:',/&
               &,'      equ r0 r1',/&
               &,'   where r0 is minimal and r1 is maximal mesh value',/&
               &,' * input mesh:',/&
               &,'      inp',/&
               &,' * mesh from file:',/&
               &,'      file filename',/&
               &,'   where filename is the filename containing the mesh',/&
               &)",Trim(word)
       Endif
    Endif
    !!
    Print *,'input file=',Trim(fname)
    Print *,'outfile =',Trim(oname)
    Print *,'meth =',Trim(meth)
    If(meth=='equ')Then
       Print *,'rmin =',rmin
       Print *,'rmax =',rmax
    Elseif(meth=='inp')Then
       Print *,'rmin =',rmin
       Print *,'rmax =',rmax
    Elseif(meth=='file')Then
       Print *,'output mesh file =',trim(foutname)
    Endif
    !!
    Open(12,file=Trim(fname))
    nline=0
    countloop: Do
       Read(12,12,iostat=eof) buf
       If(eof < 0 ) Exit countloop
       If( Len_trim(buf)==0) Cycle countloop
       word=Adjustl(buf)
       If( word(1:1)=='#') Cycle countloop
       nline=nline+1
    Enddo countloop
    !!
    Print *,'nline=',nline
    !!
    Call palloc('r',r,1_t_int,nline,nmodule,nsubfunc)
    Call palloc('f',f,1_t_int,nline,nmodule,nsubfunc)
    Rewind(12)
    i=0
    countloop1: Do
       Read(12,12,iostat=eof) buf
       If(eof < 0 ) Exit countloop1
       If( Len_trim(buf)==0) Cycle countloop1
       word=Adjustl(buf)
       If( word(1:1)=='#') Cycle countloop1
       i=i+1
       Read(buf,fmt=*) r(i),f(i)
    Enddo countloop1
    Close(12)
    !!
    If(Trim(meth)=='inp')Then
       rmin=r(1)
       rmax=r(nline)
    Else If(Trim(meth)=='file')Then
       Open(12,file=Trim(foutname))
       no=0
       ocountloop: Do
          Read(12,fmt=*,iostat=eof) dum
          If(eof < 0 ) Exit ocountloop
          no=no+1
       Enddo ocountloop
       Call palloc('ro',ro,1_t_int,no,nmodule,nsubfunc)
       Rewind(12)
       Do i=1,no
          Read(12,fmt=*) ro(i)
       Enddo
       Close(12)
    Endif
    !!
    If(meth /='file')Then
       Call palloc('ro',ro,1_t_int,no,nmodule,nsubfunc)
       Do i=1,no
          ro(i)=(rmax-rmin)*Real(i-1,t_real)/(no-1)+rmin
       Enddo
    End If
    !!


    !Allocate(work(nline))
    Call palloc("work",work,1_t_int,nline,nmodule,nsubfunc)




    !!
    !!

        
    k=p+1
    Call palloc("knots",knots,1_t_int,nline+k,nmodule,nsubfunc)
    Call palloc("p_q",p_q,1_t_int,(2_t_int*k-1_t_int)*nline,nmodule,nsubfunc)

    Call set_knots(nline,k,r,knots)

    Call bsplint(r,knots,nline,k,p_q,work)
    !!
    work=f
    Call banslv(p_q,2*k-1_t_int,nline,p,p,work)
    !!
goto 100
    Open(12,file='+bspbas')
    Do j=1,nline
       !!
       work=0.0_t_real
       work(j)=1.0_t_real
       Do i=1,no
          a=ro(i)
          
          sf=bvalue(knots,work,nline,k,a,0_t_int)
          Write(12,10) a,sf

       Enddo
       Write(12,*)
    Enddo
    Do j=1,nline+k
       Write(12,"(f10.5,2x,f10.5)")knots(j),0.0_t_real
       Write(12,"(f10.5,2x,f10.5)")knots(j),1.0_t_real
       Write(12,*)
    Enddo
    Do j=1,nline
       Write(12,"(f10.5,2x,f10.5)")r(j),0.0_t_real
       Write(12,"(f10.5,2x,f10.5)")r(j),0.5_t_real
       Write(12,*)
    Enddo
    Close(12)
    !!
100 continue
    !! determine the coefficients on the endpoints, for extrapolation
    !!
    nderiv=k
    Call palloc("derivleft",derivleft,0_t_int,nderiv-1,nmodule,nsubfunc)
    Call palloc("derivright",derivright,0_t_int,nderiv-1,nmodule,nsubfunc)
    Call palloc("bwork",bwork,1_t_int,((k+1)*(k+2))/2,nmodule,nsubfunc)
    Call palloc("BiAtX",BiAtX,1_t_int,k,1_t_int,nderiv,nmodule,nsubfunc)
    !!
    x=knots(k)
    ileft=k
    Call bsp_bspvd(nline,knots,k,nderiv,x,ileft,nderiv,BiAtX,bwork,.False.)
    derivleft(0:nderiv-1)=Matmul(work(ileft-k+1:ileft),BiAtX(1:k,1:nderiv))
    !!
    x=knots(nline+1)
    ileft=nline
    Call bsp_bspvd(nline,knots,k,nderiv,x,ileft,nderiv,BiAtX,bwork,.False.)
    derivright(0:nderiv-1)=Matmul(work(ileft-k+1:ileft),BiAtX(1:k,1:nderiv))
    !!
    !!
    !Print *,derivright(1:p)
    !!
    Open(12,file=Trim(oname))
    Do i=1,no
       a=ro(i)
       If(a<knots(k))Then
          x=a-knots(k)
          bsf=derivleft(p)
          Do j=p-1,jderiv,-1
             bsf=x*bsf/Real(j+1-jderiv,t_real)+derivleft(j)
          Enddo
       Else If(a>=knots(nline+1))Then
          x=a-knots(nline+1)
          bsf=derivright(p)
          Do j=p-1,jderiv,-1
             bsf=x*bsf/Real(j+1-jderiv,t_real)+derivright(j)
          Enddo
       Else
          bsf=bvalue(knots,work,nline,k,a,jderiv)
       Endif
       Write(12,10) a,bsf
    Enddo
    Close(12)
    !!
    !! 
    !!
    !!
    Call pfree(p_q)
    Call pfree(knots)
    Call pfree(work)
    Call pfree(bwork)
    Call pfree(derivleft)
    Call pfree(derivright)
    Call pfree(BiAtX)
    Call pfree(r)
    Call pfree(ro)
    Call pfree(f)
    !!
    !Call terminate(TERM_NORMAL,'Spline')
    !!
    Return
    !!
10  Format(5(e30.15e3,2x))
12  Format(a)
    !!
  End Subroutine splineit
  !!***********************************************************************
  !! setup natural parametrization (knots)
  Subroutine set_knots(n,k,x,t)
    !!
    Implicit None  
    !!IO
    !! order
    Integer(t_int),Intent(in)    :: k
    !! No. of mesh points
    Integer(t_int),Intent(in)    :: n
    !! mesh points
    Real(t_real),Intent(in)      :: x(n)
    !! knots
    Real(t_real),Intent(out)     :: t(n+k)
    !!LOCAL
    Character(*),Parameter::nsubfunc="set_knots"
    Integer(t_int) :: j
    !!
    If(n<k)Then
       Call terminate(TERM_CRASH&
            &,Trim(nsubfunc)//': n must be larger than k!')
    Endif
    If(Mod(k,2_t_int)==0_t_int)Then
       t(1:k)=x(1)
       t(k+1:n)=x(k+1-k/2:n-k/2)
       t(n+1:n+k)=x(n)
    Else
       t(1:k)=x(1)
       Do j=1,n-k
          t(k+j)=0.5_t_real*(x((k-1)/2+j)+x((k+1)/2+j))
       Enddo
       t(n+1:n+k)=x(n)
    Endif
    !!
  End Subroutine set_knots
  !!***********************************************************************

End Program spline_it
