!!----DEBUG options----------------------------|
!! SUBFUNC          | MODES                    |
!!---------------------------------------------|
!! setup_stm        | write_back_tip_functions |
!! setup_stm        | reportMeshInfo           |
!! make_stm_lattice | prot                     |
!! stm_loi_int0     | prot                     |
!!---------------------------------------------|
!todo: separate the individual tip orbital contributions
!todo: remove kinetic part !!!!
!! check, if hamiltonian correct
!!todo: core-ortho ? (should be very small)
Module stm
  !!
  Use precisiondefs
  Use file_iounits
  Use memory
  Use errors
  Use constants
  Use clock
  Use constants
  Use optionstest
  Use tsymmetry
  Use tbzone
  Use bzone
  Use tkohnsham
  Use tpotentialdensity
  Use tstm
  Use ioaux
  Use atomdefs
  Use loi0
  Use matserv
  Use finuc
  Use tmol
  Use torbitalbasis
  Use tnumerics
  Use tloi
  Use tmesh
  Use prnt
  Use mesh
  Use trelativistic
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="STM"  
  !!
  Public :: setup_stm,stm_loi_int0,stm_lomatrix,stm_write_weight_header&
       &,stm_calc_weights,stm_calc_dos,stm_make_map&
       &,tip_mat_analyse
  !!***********************************************************************
  Character(*),Parameter   :: mesg='STM'
  !!
Contains  
  !!***********************************************************************
  Subroutine setup_stm(symmetry,structure,potential,stm,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_STM),Intent(inout)    :: stm
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="setup_stm"
    Integer(t_int) :: nmap,ix,iy,ir,ivr
    Real(t_real)   :: rx,ry
    !!
    !!
    !!
    Call read_stm_input(symmetry,structure,stm)
    If(.Not.stm% active) Return
    !!
    If(relativistic==FULL_RELATIVISTIC)&
         &Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="' relativistic version of STM not implemented!'")
    !!
    !! read vatom and basis from file
    !!
    Call read_vatom(stm)
    Call read_fval(stm)
    Call read_fkval(stm)
    !!
    !!
    !! setup map mesh
    !!
    !!
    stm% mapN=(stm% mapNx+1)*(stm% mapNy+1)
    Call palloc('stm% p_map',stm% p_map,1_t_int,3_t_int&
         &,1_t_int,stm% mapN,nmodule,nsubfunc)
    Call palloc('stm% p_mapxy',stm% p_mapxy,1_t_int,2_t_int&
         &,1_t_int,stm% mapN,nmodule,nsubfunc)
    !!
    nmap=0
    Do ix=1,stm% mapNx+1
       rx=(ix-1.0_t_real)/(stm% mapNx)
       Do iy=1,stm% mapNy+1
          ry=(iy-1.0_t_real)/(stm% mapNy)
          nmap=nmap+1
          stm% p_mapxy(1,nmap)=rx
          stm% p_mapxy(2,nmap)=ry
          stm% p_map(:,nmap)=&
               & rx*symmetry% lattice_vectors(:,1)&
               &+ry*symmetry% lattice_vectors(:,2)&
               &+(/0.0_t_real,0.0_t_real,stm% tau(3)/)
          !Print "('xy : ',2(f30.15,1x))",stm% p_mapxy(:,nmap)
          !Print "('xyz: ',3(f30.15,1x))",stm% p_map(:,nmap)
          
       Enddo
    Enddo
    !!
    !!
    Call make_stm_lattice(symmetry,structure,potential,stm,stm% tau)
    !!
    !!
    If(debugfun(nmodule,nsubfunc,'reportMeshInfo'))Then
       !!
       Call write_mesh_info(outfile,stm% atommesh)
       Call write_mesh_info(outfile,stm% basmesh)
       !!
    Endif
    !!
    If(debugfun(nmodule,nsubfunc,'write_back_tip_functions'))Then
       !!
       Open(13,file='tip_vatom')
       Do ir=1,stm% atommesh% nr
          Write(13,*)stm% atommesh% p_rmesh(ir),stm% p_vatom(ir)
       Enddo
       Close(13)
       !!
       !!
       Open(13,file='tip_fv')
       Do ivr=1,stm% nRv
          Do ir=1,stm% basmesh% nr
             Write(13,*)stm% basmesh% p_rmesh(ir),stm% p_fv(ir,ivr)&
                  &*stm% basmesh% p_rmesh(ir)**stm% fgam
          Enddo
          Write(13,*)
       Enddo
       Close(13)
       !!
       Open(13,file='tip_fkv')
       Do ivr=1,stm% nRv
          Do ir=1,stm% basmesh% nr
             Write(13,*)stm% basmesh% p_rmesh(ir),stm% p_fkv(ir,ivr)&
                  &*stm% basmesh% p_rmesh(ir)**stm% fgam
          Enddo
          Write(13,*)
       Enddo
       Close(13)
    Endif
    !!
    !!
    Return
    !!
    !!
    !!
    !!
  End Subroutine setup_stm
  !!***********************************************************************
  Logical(t_log) Function readCoorddinate(buffer,symmetry,structure,xyz)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Character(*),Intent(inout)   :: buffer
    Real(t_real),Intent(out)     :: xyz(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="readCoorddinate"
    Logical(t_log) :: relative(3)
    Character(Len(buffer))   :: word
    Character(2)   :: el
    Real(t_real)   :: lxyz(3),d
    Integer(t_int) :: ixyz,j,lengthunits(3), expect,sites(3)
    Integer :: err
    Integer(t_int),Parameter :: end=0
    Integer(t_int),Parameter :: start=1
    Integer(t_int),Parameter :: unit=3
    Integer(t_int),Parameter :: element=4
    Integer(t_int),Parameter :: openparen=5
    Integer(t_int),Parameter :: site=6
    Integer(t_int),Parameter :: closeparen=7
    Integer(t_int),Parameter :: plus=8
    Integer(t_int),Parameter :: anumber=9
    !!
    !!
    xyz=0.0_t_real
    readCoorddinate=.True.
    !!
    word=''
    relative=.False.
    lengthunits=0
    sites=0
    !!
    ixyz=0
    expect=start
    Do While(expect/=End)
       !Print *,'WHILE:',expect
       If(expect==start)Then
          ixyz=ixyz+1
          !Print *,'START ixyz=',ixyz,readCoorddinate
          If(Len_trim(word)==0)Then
             If(ixyz<=3)Then
                If(.Not.next_word(buffer,word)) readCoorddinate=.False.
             Else
                expect=End
                Cycle
             Endif
          Endif
          !Print *,'START:',Trim(word)
          Read(word,fmt=*,iostat=err)lxyz(ixyz)
          If(err==0)Then
             !Print *,'absolute'
             word=''
             expect=unit
          Else
             !Print *,'complex',readCoorddinate
             j=Scan(word,'aA',.False.)
             If(j==0) Then
                !Print *,'NO numberunit',readCoorddinate,Trim(word)
                expect=element
             Else
                !Print *,'must be numberunit',readCoorddinate,word(1:j-1)
                Read(word(1:j-1),fmt=*,iostat=err)lxyz(ixyz)
                If(err==0) Then 
                   word=word(j:)
                   expect=unit
                Else
                   !Print *,'must be El(...)'
                   expect=element
                Endif
             Endif
          Endif
       Elseif(expect==anumber)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'ANUMBER:',Trim(word)
          Read(word,fmt=*,iostat=err)lxyz(ixyz)
          If(err/=0) Then
             j=Scan(word,'aA',.False.)
             If(j==0)Then
                readCoorddinate=.False.
                expect=End
             Else
                Read(word(1:j-1),fmt=*,iostat=err)lxyz(ixyz)
                If(err/=0)Then
                   readCoorddinate=.False.
                   expect=End
                Else
                   word=word(j:)
                   expect=unit
                Endif
             Endif
          Else
             !Print *,'anumber=',ixyz,lxyz(ixyz)
             expect=unit
             word=''
          Endif
       Elseif(expect==unit)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) Then
                If(ixyz<3)Then
                   readCoorddinate=.False.
                Endif
                expect=End
                Cycle
             Endif
          Endif
          !Print *,'UNIT:',Trim(word)
          Call capitalize(word)
          If(word=='Ab')Then
             lengthunits(ixyz)=SYM_LENGTH_BOHRRADII
             !Print *,'lengthunit=',ixyz,lengthunits(ixyz)
             word=''
          Elseif(word=='A'.Or.word=='Ang')Then
             lengthunits(ixyz)=SYM_LENGTH_ANGSTROEM
             !Print *,'lengthunit=',ixyz,lengthunits(ixyz)
             word=''
          Endif
          expect=start
       Elseif(expect==element)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'ELEMENT:',Trim(word),readCoorddinate
          j=Index(word,'(')
          If(j==0)Then
             !Print *,'must be single element',Trim(word)
             If((Len_trim(word)<1.Or.Len_trim(word)>2))Then
                readCoorddinate=.False.
                expect=End
             Else
                el=Trim(word)
                Call capitalize(el)
                If(.Not.isElement(el)) Then
                   readCoorddinate=.False.
                   expect=End
                Else
                   word=''
                   expect=openparen
                Endif
             Endif
          Else
             !Print *,'must be element(...',Trim(word)
             el=word(1:j-1)
             Call capitalize(el)
             If(.Not.isElement(el)) Then
                readCoorddinate=.False.
                expect=End
             Else
                word=word(j+1:)
                expect=site
             Endif
          Endif
       Elseif(expect==openparen)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'OPENPAREN:',Trim(word)
          If(word(1:1)=='(')Then
             word=word(2:)
             expect=site
          Else
             readCoorddinate=.False.
             expect=End
          Endif
       Elseif(expect==site)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'SITE:',Trim(word)
          j=Index(word,')')
          If(j==0)Then
             Read(word,fmt=*,iostat=err)sites(ixyz)
             !Print *,'must be just site',err
             If(err/=0) Then 
                readCoorddinate=.False.
                expect=End
             Else
                word=''
                expect=closeparen
                !Print *,'expect=',expect
             Endif
          Else
             !Print *,'must be site)'
             Read(word(1:j-1),fmt=*,iostat=err)sites(ixyz)
             If(err/=0) Then 
                readCoorddinate=.False.
                expect=End
             Else
                word=word(j:)
                expect=closeparen
             Endif
          Endif
       Elseif(expect==closeparen)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'CLOSEPAREN:',Trim(word)
          If(word(1:1)==')')Then
             word=word(2:)
             expect=plus
          Else
             readCoorddinate=.False.
             expect=End
          Endif
       Elseif(expect==plus)Then
          If(Len_trim(word)==0)Then
             If(.Not.next_word(buffer,word)) readCoorddinate=.False.
          Endif
          !Print *,'PLUS:',Trim(word)
          If(word(1:1)=='+')Then
             word=word(2:)
             expect=anumber
          Else
             readCoorddinate=.False.
             expect=End
          Endif
       Endif
    Enddo
    !!
    !!
    Do ixyz=1,3
       If(sites(ixyz)==0)Then
          relative(ixyz)=(lengthunits(ixyz)==0)
       Else
          If(lengthunits(ixyz)==0) lengthunits(ixyz)=SYM_LENGTH_BOHRRADII
       Endif
       !Print "('ixyz=',i3,' rel=',l1,' unit=',i1,' site=',i3)"&
       !     &,ixyz,relative(ixyz),lengthunits(ixyz),sites(ixyz)
    Enddo
    Do ixyz=1,3
       If(relative(ixyz))Then
          !Print *,'ixyz is relative, xyz=',xyz
          !Print *,'lxyz=',lxyz(ixyz)
          !Print *,'lat=',symmetry% lattice_vectors(:,ixyz)
          xyz=xyz+symmetry% lattice_vectors(:,ixyz)*lxyz(ixyz)
          !Print *,'ixyz is relative, xyz=',xyz
       Endif
    Enddo
    !!
    Do ixyz=1,3
       If(sites(ixyz)==0)Then 
          If(.Not.relative(ixyz))Then
             d=lxyz(ixyz)
             If(lengthunits(ixyz)==SYM_LENGTH_ANGSTROEM)Then
                d=d/brtoan
             Endif
             xyz(ixyz)=d
          Endif
       Else
          If(sites(ixyz)<1.Or.sites(ixyz)>structure% nsite)Then
             readCoorddinate=.False.
          Else
             d=lxyz(ixyz)
             If(lengthunits(ixyz)==SYM_LENGTH_ANGSTROEM)Then
                d=d/brtoan
             Endif
             xyz(ixyz)=structure% p_tau(ixyz,sites(ixyz))+d
          Endif
       Endif
    Enddo
    !!
    Return
    !!
  End Function readCoorddinate
  !!***********************************************************************
  Subroutine read_vatom(stm)
    !!
    Implicit None  
    !!IO
    Type(T_Stm),Intent(inout)    :: stm
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_vatom"
    Integer(t_int) :: lun,j
    Integer :: eof
    Character(5000)   :: buffer
    Character(5000)    :: ele
    !!
    !!
    !!
    lun=12_t_int
    !!
    !!
    Open(lun,file=Trim(stm% vatomfile),status='old')
    !!
    !!
    readloop: Do 
       Read(lun,3,iostat=eof) buffer
       If(eof /=0 ) Exit readloop
       !!
       If(Index(buffer,'Version: ') > 0) Cycle readloop
       !!
       j=Index(buffer,'element: ')
       If(j > 0) Then
          j=j+8
          Read(buffer(j:),*)ele
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start mesh>')
       If(j > 0) Then
          Call read_mesh_info(lun,stm% atommesh)
          Call setup_mesh(stm% atommesh)
          Call palloc('stm% p_vatom',stm% p_vatom,1_t_int,stm% atommesh% nr&
               &,nmodule,nsubfunc)
          stm% p_vatom(:)=0.0_t_real
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start function')
       If(j > 0) Then
          j=j+15
          !!
          Call read_mesh_function(lun,stm% p_vatom)
          !!
          Cycle readloop
       Endif
       !!
       !!
    Enddo readloop
    !!
    !!
    Close(lun)
    !!
    !!
    If(Trim(ele) /= Trim(stm% element) ) Then
       Write(errbuf,99)Trim(stm% vatomfile)
99     Format("'File ",a," belongs to another atom!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_vatom
  !!***********************************************************************
  Subroutine read_fval(stm)
    !!
    Implicit None  
    !!IO
    Type(T_Stm),Intent(inout)    :: stm
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_fval"
    Integer(t_int) :: lun,j,nmain,l,ivr
    Integer :: eof
    Logical(t_log),Pointer :: p_have(:)
    Character(5000)   :: buffer
    Character(5000)   :: ele
    Character(100)    :: ch
    Character(1)      :: c
    !!
    !!
    Nullify(p_have)
    !!
    Call palloc('p_have',p_have,1_t_int,stm% nRv,nmodule,nsubfunc)
    !!
    p_have=.False.
    !!
    lun=12_t_int
    !!
    !!
    Open(lun,file=Trim(stm% fvalfile),status='old')
    !!
    !!
    readloop: Do 
       Read(lun,3,iostat=eof) buffer
       If(eof /=0 ) Exit readloop
       !!
       If(Index(buffer,'Version: ') > 0) Cycle readloop
       !!
       j=Index(buffer,'element: ')
       If(j > 0) Then
          j=j+8
          Read(buffer(j:),*)ele
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'fgam: ')
       If(j > 0) Then
          j=j+Index(buffer(j:),':')+0
          Read(buffer(j:),*)stm% fgam
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start mesh>')
       If(j > 0) Then
          Call read_mesh_info(lun,stm% basmesh)
          Call setup_mesh(stm% basmesh)
          Call palloc('stm% p_fv',stm% p_fv,1_t_int,stm% basmesh% nr&
               &,1_t_int,stm% nRv,nmodule,nsubfunc)
          stm% p_fv=0.0_t_real
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start function')
       If(j > 0) Then
          j=j+Index(buffer(j:),'# state :')
          j=j+Index(buffer(j:),':')
          !!
          ch=Adjustl(buffer(j:))
          j=Index(ch,'>')
          ch(j:j)=' '
          Read(ch,"(i1,a1)")nmain,c
          l=lcharqn(c)
          Do ivr=1,stm% nRv
             If(stm% p_nnRv(ivr)==nmain .And.stm% p_lRv(ivr)==l)Then
                !Print "('<',i1,a1,'>')",nmain,lqnchar(l)
                p_have(ivr)=.True.
                Call read_mesh_function(lun,stm% p_fv(:,ivr))
                stm% p_fv(2:,ivr)=stm% p_fv(2:,ivr)&
                     &/stm% basmesh% p_rmesh(2:)**stm% fgam
             Endif
          Enddo
          Cycle readloop
       Endif
       !!
       !!
    Enddo readloop
    !!
    !!
    Close(lun)
    !!
    !!
    If(Any(p_have(:).Eqv..False.))Then
       Write(errbuf,88)Trim(stm% fvalfile)
88     Format("'File ",a,": missing some states!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    Call pfree(p_have)
    !!
    !!
    If(Trim(ele) /= Trim(stm% element) ) Then
       Write(errbuf,99)Trim(stm% fvalfile)
99     Format("'File ",a," belongs to another atom!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    !!
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_fval
  !!***********************************************************************
  Subroutine read_fkval(stm)
    !!
    Implicit None  
    !!IO
    Type(T_Stm),Intent(inout)    :: stm
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_fkval"
    Integer(t_int) :: lun,j,nmain,l,ivr
    Integer :: eof
    Logical(t_log),Pointer :: p_have(:)
    Character(5000)   :: buffer
    Character(5000)   :: ele
    Character(100)    :: ch
    Character(1)      :: c
    !!
    !!
    Nullify(p_have)
    !!
    Call palloc('p_have',p_have,1_t_int,stm% nRv,nmodule,nsubfunc)
    !!
    p_have=.False.
    !!
    lun=12_t_int
    !!
    !!
    Open(lun,file=Trim(stm% fkvalfile),status='old')
    !!
    !!
    readloop: Do 
       Read(lun,3,iostat=eof) buffer
       If(eof /=0 ) Exit readloop
       !!
       If(Index(buffer,'Version: ') > 0) Cycle readloop
       !!
       j=Index(buffer,'element: ')
       If(j > 0) Then
          j=j+8
          Read(buffer(j:),*)ele
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start mesh>')
       If(j > 0) Then
          Call read_mesh_info(lun,stm% basmesh)
          Call setup_mesh(stm% basmesh)
          Call palloc('stm% p_fkv',stm% p_fkv,1_t_int,stm% basmesh% nr&
               &,1_t_int,stm% nRv,nmodule,nsubfunc)
          stm% p_fkv=0.0_t_real
          Cycle readloop
       Endif
       !!
       j=Index(buffer,'<start function')
       If(j > 0) Then
          j=j+Index(buffer(j:),'# state :')
          j=j+Index(buffer(j:),':')
          !!
          ch=Adjustl(buffer(j:))
          j=Index(ch,'>')
          ch(j:j)=' '
          Read(ch,"(i1,a1)")nmain,c
          l=lcharqn(c)
          Do ivr=1,stm% nRv
             If(stm% p_nnRv(ivr)==nmain .And.stm% p_lRv(ivr)==l)Then
                !Print "('<',i1,a1,'>')",nmain,lqnchar(l)
                p_have(ivr)=.True.
                Call read_mesh_function(lun,stm% p_fkv(:,ivr))
                stm% p_fkv(2:,ivr)=stm% p_fkv(2:,ivr)&
                     &/stm% basmesh% p_rmesh(2:)**stm% fgam
             Endif
          Enddo
          Cycle readloop
       Endif
       !!
       !!
    Enddo readloop
    !!
    !!
    Close(lun)
    !!
    !!
    If(Any(p_have(:).Eqv..False.))Then
       Write(errbuf,88)Trim(stm% fkvalfile)
88     Format("'File ",a,": missing some states!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    Call pfree(p_have)
    !!
    !!
    If(Trim(ele) /= Trim(stm% element) ) Then
       Write(errbuf,99)Trim(stm% fkvalfile)
99     Format("'File ",a," belongs to another atom!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    !!
    Return
    !!
3   Format(a)
    !!
  End Subroutine read_fkval
  !!***********************************************************************
  Subroutine read_stm_input(symmetry,structure,stm)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_STM),Intent(inout)    :: stm
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_stm_input"
    Integer(t_int) :: i,ivr,idum
    Integer :: eof
    Logical(t_log) :: exists
    Character(1)   :: c
    Character(1000) :: buffer,word,cbuffer
    Character(*),Parameter :: filename='=.stm'
    !!
    Inquire(file=Trim(filename),exist=exists)
    If(.Not. exists)Then
       stm% active=.False.
       Return
    Endif
    !!
    Open(12,file=Trim(filename))
    Read(12,*) !! comment
    Read(12,*) stm% active
    Read(12,*) !! comment
    Read(12,"(a)") buffer
    !Print "('<',a,'>')",Trim(buffer)
    If(.Not.next_word(buffer,word)) Goto 888
    If(Len_trim(word)<1 .Or. Len_trim(word)>2) Goto 888
    stm% element=Trim(word)
    If(.Not.readCoorddinate(buffer,symmetry,structure,stm% tau)) Goto 888
    Read(12,*) !! comment
    Read(12,*) stm% vatomfile,stm% fvalfile,stm% fkvalfile
    Read(12,*) !! comment
    Read(12,"(a)") buffer
    !!Print "('<',a,'>')",Trim(buffer)
    cbuffer=buffer
    stm% nRv=0
    Do While(next_word(buffer,word))
       stm% nRv=stm% nRv+1
    Enddo
    Call palloc('stm% p_nnRv',stm% p_nnRv,1_t_int,stm% nRv&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_lRv',stm% p_lRv,1_t_int,stm% nRv&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_ev',stm% p_ev,1_t_int,stm% nRv&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_delta',stm% p_delta,1_t_int,stm% nRv&
         &,nmodule,nsubfunc)
    !!
    i=0
    Do While(next_word(cbuffer,word))
       i=i+1
       Read(word,"(i1,a1)")stm% p_nnRv(i),c
       stm% p_lRv(i)=lcharqn(c)
    Enddo
    !! 
    stm% natom=Real(getelementnumber(stm% element),t_real)
    !! 
    stm% lmx=Maxval(stm% p_lRv)
    !!
    Read(12,*) !! orbital energies
    Read(12,"(a)") buffer
    !!Print "('<',a,'>')",Trim(buffer)
    
    i=0
    Do While(next_word(buffer,word))
       i=i+1
       Read(word,*)stm% p_ev(i)
    Enddo
    Read(12,*) !! orbital deltas
    Read(12,"(a)") buffer
    !!Print "('<',a,'>')",Trim(buffer)
    
    i=0
    Do While(next_word(buffer,word))
       i=i+1
       Read(word,*)stm% p_delta(i)
    Enddo
    !!
    Read(12,*) !! comment
    Read(12,*) stm% v0,stm% v1,stm% nev
    !!Print *,stm% v0,stm% v1,stm% nev
    Read(12,*) !! comment
    Read(12,*)stm% mapNx,stm% mapNy
    Read(12,*) !! comment
    Read(12,"(a)") buffer
    stm% N_sort_nl=0
    cbuffer=buffer
    Do While(next_word(buffer,word))
       stm% N_sort_nl=stm% N_sort_nl+1   
    Enddo
    Call palloc('stm% p_sort_nl',stm% p_sort_nl,1_t_int,stm% N_sort_nl&
         &,nmodule,nsubfunc)
    !!
    i=0
    Do While(next_word(cbuffer,word))
       Read(word,*,iostat=eof)idum
       i=i+1
       If(eof/=0)Then
          Write(errbuf,199)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
199       Format("' resolved line should contain sort numbers'")
       Endif
       stm% p_sort_nl(i)=idum
    Enddo
    !!
    Close(12)
    !!
    !!
    If(stm% active)Then
       Write(outfile,2) mesg
       Write(outfile,"(a,': tip atom ',a2,' Z=',f10.3,' at ',3(f13.5,1x))") &
            &mesg,stm% element,stm% natom,stm% tau
       Write(outfile,"(a,': read data from the files: ',a,', ',a,' and ',a)") &
            &mesg,Trim(stm% vatomfile),Trim(stm% fvalfile)&
            &,Trim(stm% fkvalfile)
       Write(outfile,2) mesg
       Write(outfile,"(a,' output on ',i5,' points in interval [',f12.5,','&
            &,f12.5,'] ',/,a)") &
            &mesg,stm% nev,stm% v0,stm% v1,mesg
       Do ivr=1,stm% nRv
          Write(outfile,"(a,' ',i1,a1,' E=',f30.15,' delta=',f30.15)")&
               &mesg,stm% p_nnRv(ivr),lqnchar(stm% p_lRv(ivr)),stm% p_ev(ivr)&
               &,stm% p_delta(ivr)
       Enddo
       Write(outfile,"(a,' map at mesh: ',i5,' x ',i5)")mesg,stm% mapNx,stm% mapNy
       Write(outfile,2) mesg
       Write(outfile,2) mesg
       Write(outfile,"(a,' resolved data for sorts: ')",advance='no')mesg
       Do i=1,stm% N_sort_nl
          Write(outfile,"(1x,i4)",advance='no')stm% p_sort_nl(i)
       Enddo
       Write(outfile,*)
       Write(outfile,2) mesg
    Else
       Return
    Endif
    !!
    !!
    Return
    !!
2   Format(a,':')
    !!
    !! failure of element position line
888 Continue
    !!
    Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="' line should contain El xyz xyz xyz!',/&
         &,' where xyz= [r] number or ',/&
         &,'       xyz= El(site) + number [A/ab]',/&
         &,' [] arguments are optional',/&
         &,'    r means in relative unit cell coordinates',/&
         &,'    A=Ang, ab=Bohr radii (default)'&
         &")
    !!
  End Subroutine read_stm_input
  !!***********************************************************************
  Subroutine make_stm_lattice(symmetry,structure,potential,stm,stmtau)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Type(T_STM),Intent(inout)    :: stm
    Real(t_real),Intent(in)      :: stmtau(3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_stm_lattice"
    Integer(t_int) :: dim,isa,issa,k,NR(2,3),i,i1,i2,i3
    Real(t_real)   :: tau(3),a(3),rcut,rg,ag,ap(3),pts(3),apts2
    Integer(t_int),Pointer :: itmp(:)
    Real(t_real),Pointer :: tmp(:,:,:)
    !!
    Nullify(tmp,itmp)
    !!
    stm% maxradius=Min(stm% atommesh% rmax,stm% basmesh% rmax)
    !!
    !! determine the geometric relations
    !! forall sites s find vectors P_ts=t-R-s, which point from the site
    !! to the tip
    !!
    !! 
    dim=0
    Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       tau=structure% p_tau(:,isa)
       a=stmtau-tau
       rcut= stm% maxradius+potential% p_pmeshs(issa)% rmax
       Do k=1,3
          rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rcut/twopi
          ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
          
          NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
          NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
       Enddo
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Endif
       !!
       dim=Max(dim,(NR(2,1)-NR(1,1)+1)*(NR(2,2)-NR(1,2)+1)*(NR(2,3)-NR(1,3)+1))
    Enddo
    !!
    Call palloc('tmp',tmp,1_t_int,3_t_int,1_t_int,dim,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('itmp',itmp,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!
    stm% npairmax=0
    Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       tau=structure% p_tau(:,isa)
       a=stmtau-tau
       rcut= stm% maxradius+potential% p_pmeshs(issa)% rmax
       Do k=1,3
          rg=Sqrt(Sum(symmetry% reciprocial_lattice_vectors(:,k)**2))*rcut/twopi
          ag=Dot_product(a,symmetry% reciprocial_lattice_vectors(:,k))/twopi
          
          NR(1,k)=Floor(Min(ag-rg,ag+rg))-1
          NR(2,k)=Ceiling(Max(ag-rg,ag+rg))+1
       Enddo
       If(structure% str_type == STR_TYPE_MOLECULE)Then
          NR(:,:)=0_t_int
       Endif
       !!
       !!
       i=0
       i1loop1: Do i1=NR(1,1),NR(2,1)
          i2loop1: Do i2=NR(1,2),NR(2,2)
             i3loop1: Do i3=NR(1,3),NR(2,3)
                ap(:)= symmetry% lattice_vectors(:,1)*i1&
                     &+symmetry% lattice_vectors(:,2)*i2&
                     &+symmetry% lattice_vectors(:,3)*i3
                
                pts(:)=a(:)-ap(:)
                apts2=Sum(pts**2)
                If(apts2>rcut**2) Cycle
                i=i+1
                tmp(:,i,isa)=pts(:)
             End Do i3loop1
          End Do i2loop1
       End Do i1loop1
       !!
       itmp(isa)=i
       stm% npairmax=Max(stm% npairmax,i)
       !!
       !!
    Enddo
    !!
    Call palloc('stm% p_npairs',stm% p_npairs,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_pairs',stm% p_pairs,1_t_int,3_t_int&
         &,1_t_int,stm% npairmax,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_pairabs',stm% p_pairabs&
         &,1_t_int,stm% npairmax,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_ninpshell',stm% p_ninpshell&
         &,1_t_int,stm% npairmax,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    Do isa=1,structure% nsite
       stm% p_npairs(isa)=itmp(isa)
       Do i=1,stm% p_npairs(isa)
          stm% p_pairs(:,i,isa)=tmp(:,i,isa)
          stm% p_pairabs(i,isa)=Sqrt(Sum(tmp(:,i,isa)**2))
          stm% p_ninpshell(i,isa)=1_t_int
       Enddo
    Enddo
    !!
    If(debugfun(nmodule,nsubfunc,'prot'))Then
       Do isa=1,structure% nsite
          Write(outfile,"(a,': ',i4,' pairs to site',i4)")&
               &mesg,stm% p_npairs(isa),isa
          Do i=1,stm% p_npairs(isa)
             Write(outfile,"(a,': pair(',i3,')=',3f18.10,' |p|=',f18.10)")&
                  &mesg,i,stm% p_pairs(:,i,isa),stm% p_pairabs(i,isa)
          Enddo
       Enddo
    Endif
    !!
    Call pfree(tmp)
    Call pfree(itmp)
    !!
    Return
    !!
  End Subroutine make_stm_lattice
  !!***********************************************************************
  Subroutine stm_loi_int0(structure,basis,mol,numerics&
       &,natom,nuc_type,stm&
       &,vblocklength,cblocklength)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Mol),Intent(in)       :: mol
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_STM),Intent(inout),Target    :: stm
    Integer(t_int),Intent(in)    :: nuc_type
    Integer(t_int),Intent(in)    :: vblocklength(:),cblocklength(:)
    Real(t_real),Intent(in)      :: natom(*)
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_loi_int0"
    Integer(t_int) :: isb,issb,wudim,hzlmx,ncomp
    Real(t_real),Pointer :: p_wu    (:)
    Real(t_real),Pointer :: p_fac   (:)
    !Character(*),Parameter :: cpumesg_tci='Tip two-center integrals'
    !!
    Integer(t_int) :: itcivc,itci,ir,icomp,lmx,ivr,nva,nvb,ncb,n,ne,nf,itcivv
    Integer(t_int) :: ish,ivv,ivc,irp
    Real(t_real)   :: rffmax,ZZa,rnuca,ZZb,rnucb
    Type(T_Mesh),Pointer :: bmesha,bmeshb
    Type(T_Mesh),Pointer :: atommesha,atommeshb
    !!
    If(.Not.stm% active) return
    ncomp=basis% ncomp
    !!
    !!Print *,nsubfunc//'start'
    lmx=Max(basis% lmx,stm% lmx)
    !!
    Nullify(p_fac)
    Nullify(p_wu)
    !!
    !! everything, which is not crystal potential related
    !!
    !Call watch(cpumesg_tci,cpu_start,reset=.True.)
    !Call watch(cpumesg_tci,cpu_stop)
    !!
    Call palloc("stm% p_tcivvindex",stm% p_tcivvindex&
         &,1_t_int,stm% npairmax,1_t_int,structure% nsite,nmodule,nsubfunc)
    Call palloc("stm% p_tcivcindex",stm% p_tcivcindex&
         &,1_t_int,stm% npairmax,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!

    stm% tcivvdim=0_t_int
    stm% tcivcdim=0_t_int 
    itcivv=0_t_int
    itcivc=0_t_int
    !!
    stm% nvdim=0
    Do ivr=1,stm% nRv
       stm% nvdim=stm% nvdim+2*stm% p_lRv(ivr)+1
    Enddo
    !!
    nva=stm% nvdim
    !!Print "('stm% nvdim=',i5)",stm% nvdim
    !! two center
    Do isb=1,structure% nsite
       !!
       !!
       nvb=vblocklength(isb)
       ncb=cblocklength(isb)
       !!
       !!Print "('nvb,ncb=',i5,i5)",nvb,ncb

       !! valence-valence two center integrals
       !!
       ivv=nva*nvb
       ivc=nva*ncb
       !!
       stm% tcivvdim=stm% tcivvdim+ivv*stm% p_npairs(isb)
       stm% tcivcdim=stm% tcivcdim+ivc*stm% p_npairs(isb)
       !!
       !!
       !!
       n=0_t_int
       ishloop: Do ish=1,stm% p_npairs(isb)
          nf=n+1
          ne=n+stm% p_ninpshell(ish,isb)
          n=ne
          !!
          Do ir=nf,ne
             !!
             stm% p_tcivvindex(ir,isb)=itcivv
             stm% p_tcivcindex(ir,isb)=itcivc
             itcivv=itcivv+ivv
             itcivc=itcivc+ivc
          End Do
       End Do ishloop
    Enddo                  ! isb
    !!
    !!Print *,'hier2',stm% tcivvdim,stm% tcivcdim

    !! initialize LOIs
    Call palloc("stm% p_tci_vv0",stm% p_tci_vv0&
         &,1_t_int,stm% tcivvdim,nmodule,nsubfunc)
    Call palloc("stm% p_tci_vHv0",stm% p_tci_vHv0&
         &,1_t_int,stm% tcivvdim,nmodule,nsubfunc)
    Call palloc("stm% p_tci_vc0",stm% p_tci_vc0&
         &,1_t_int,stm% tcivcdim,nmodule,nsubfunc)
    Call palloc("stm% p_tci_vHc0",stm% p_tci_vHc0&
         &,1_t_int,stm% tcivcdim,nmodule,nsubfunc)
    !!
    stm% p_tci_vv0 (:) =0.0_t_real
    stm% p_tci_vHv0(:) =0.0_t_real
    stm% p_tci_vc0 (:) =0.0_t_real
    stm% p_tci_vHc0 (:) =0.0_t_real
    !!
    !! for tci_gkp we need at least lmx==3
    !! in the moment it may be 4 at maximum, yhzy... must be
    !! generalized in tci_gkp, if more is needed
    hzlmx=Max(3_t_int,lmx)
    !! valence/core Wigner D-functions (rotation matrices)
    !! for calc_tcis
    wudim=(lmx+1)*(4*(lmx+1)**2-1)/3
    Call palloc('p_wu',p_wu,1_t_int,wudim,nmodule,nsubfunc) 
    !!
    !! factorials for the ytr calls to calculate the Wigner D-functions
    !!
    !!
    Call palloc('p_fac',p_fac ,0_t_int,2*lmx+1 ,nmodule,nsubfunc) 
    !!
    Call make_factorials(p_fac,2*lmx+1)
    !!
    !! a is tip atom
    !!
    !!
    ZZa=stm% natom
    !!
    rnuca=nuc_max_rad(nuc_type,ZZa)
    !!
    bmesha    => stm% basmesh
    atommesha => stm% atommesh
    !!
    !!
    !!
    !! calculation of two-center integrals
    isbloop: Do isb=1_t_int,structure% nsite
       !!

       issb=structure% p_isrt(isb)
       !!
       ZZb=natom(issb)
       !!
       rnucb=nuc_max_rad(nuc_type,ZZb)
       !!
       bmeshb    => basis% p_basmeshs(issb)
       atommeshb => mol% p_atommeshs(issb)
       !!
       !Call watch(cpumesg_tci,cpu_start)
       !!
       rffmax=Max(stm% maxradius&
            &,basis% p_rtfcmax(0,issb),basis% p_rtfvmax(0,issb))
       
       !!
       icomp=1
       Call calc_tcis(&
            !&structure% nsite,&
            &1_t_int,isb&
            &,stm% p_npairs(isb)& !! no shells, each vec is a shell
            &,stm% p_pairs(:,:,isb)&
            &,stm% p_pairabs(:,isb)&
            &,stm% p_ninpshell(:,isb)& !! all are ones
            &,atommesha,atommeshb,bmesha,bmeshb,basis% nrb&
            &,stm% p_vatom(:)&
            &,mol% p_vatom(:,0,issb)*0& 
            &,rffmax&
            &,stm% nRv&
            &,basis% p_nRv(issb)&
            &,basis% p_nRc(issb),basis% nRvmx,basis% nRcmx&
            &,hzlmx,lmx&
            &,stm% p_lRv(:)&
            &,basis% p_lRv(:,issb)&
            &,basis% p_lRc(:,issb)&
            &,stm% p_fv(:,:)&
            &,basis% p_fv (:,:,icomp,issb)&
            &,stm%  p_fkv(:,:)*0&
            &,basis% p_fkv(:,:,icomp,issb)*0&
            &,basis% p_fc (:,:,icomp,issb)&
            &,basis% p_fkc(:,:,icomp,issb)*0&
            &,stm% fgam&
            &,basis% p_fgam(issb)&
            &,ZZa,ZZb,rnuca,rnucb,nuc_type&
            &,numerics% degree,lmx,p_fac,p_wu&
            &,stm% p_tci_vHv0,stm% p_tci_vv0&
            &,stm% p_tci_vHc0,stm% p_tci_vc0&
            &,stm% p_tcivvindex(:,isb)&
            &,stm% p_tcivcindex(:,isb)&
            &,prot=.False.)
       !!
       !Call watch(cpumesg_tci,cpu_read,Stop=.True.,Print=.True.)
       !!
       !!
    End Do isbloop
    !!
    !!
    Call pfree(p_fac)
    Call pfree(p_wu)
    !!
    !Call watch(cpumesg_tci,cpu_read,delete=.True.,Print=.True.,mesg='Total ')
    !!
    If(debugfun(nmodule,nsubfunc,'prot'))Then
       Do isb=1,structure% nsite
          nvb=vblocklength(isb)
          !!
          Do irp=1,stm% p_npairs(isb)
             Write(outfile,"('p(',i5,')=',3f30.15,' |p|=',f30.15)") &
                  &irp,stm% p_pairs(:,irp,isb)&
                  &,Sqrt(Sum(stm% p_pairs(:,irp,isb)**2))
             itci=stm% p_tcivvindex(irp,isb)
             Write(outfile,10) isb,irp
10           Format(' siteB=',i5,' pair=',i5)
             
             Call printmatsb('Hvv=',"(e30.15e3,1x)"&
                  &,"('                 0             ')"&
                  &,stm% p_tci_vHv0(itci+1:),nvb,nva,1.0e-8_t_real,trans=.True.)
             !!
          Enddo
       Enddo
    Endif
    !!
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine stm_loi_int0
  !!***********************************************************************
  Subroutine stm_lomatrix(nsite,stm,qq,vblockstart,vblockend)
    !!
    !! Fourier lattice transform
    !!
    Implicit None
    !!IO
    Type(T_STM),Intent(inout)    :: stm
    Integer(t_int),Intent(in)    :: nsite
    Integer(t_int),Intent(in)    :: vblockstart(:)
    Integer(t_int),Intent(in)    :: vblockend(:)
    Real(t_real),Intent(in)      :: qq(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="stm_lomatrix"
    Integer(t_int) :: isb,istb,ienb,ir,itci,i,j
    Complex(t_real):: eikr
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    If(.Not.stm% active) Return
    !!
    !!
    stm% p_Sts=c0
    stm% p_Vts=c0
    !!
    Do isb=1,nsite
       istb=vblockstart(isb)
       ienb=vblockend(isb)
       
       Do ir=1,stm% p_npairs(isb)

          eikr=Exp(-ci*Dot_product(qq,stm% p_pairs(:,ir,isb)))
          itci=stm% p_tcivvindex(ir,isb)
          
          Do i=1,stm% nvdim
             Do j=istb,ienb
                itci=itci+1
                stm% p_Sts(i,j)=stm% p_Sts(i,j)+stm% p_tci_vv0(itci)*eikr
                stm% p_Vts(i,j)=stm% p_Vts(i,j)+stm% p_tci_vHv0(itci)*eikr
             Enddo
          Enddo
          
       Enddo
    Enddo
    !!
    !!
    !!
    Return
    !!
  End Subroutine stm_lomatrix
  !!***********************************************************************
  Subroutine stm_write_weight_header(unit,stm)
    !!
    Implicit None
    !!IO
    Type(T_STM),Intent(in)       :: stm
    Integer(t_int),Intent(in)    :: unit
    !!LOCAL
    !Character(*),Parameter::nsubfunc="stm_write_weight_header"
    Integer(t_int) :: ivr,nmain,l,deg,m
    Character(1)   :: chl
    !!
    If(.Not.stm% active) Return
    !!
    Do ivr=1,stm% nRv
       nmain=stm% p_nnRv(ivr)
       l=stm% p_lRv(ivr)
       chl=lqnchar(l)
       deg=2*l+1
       Do m=1,deg
          Write(unit,23,advance='no')stm% element,0,nmain,chl,m-l-1
          Write(unit,25,advance='no')stm% element,0,nmain,chl,m-l-1
       Enddo
    Enddo
    !!
    Write(unit,24)
    !!
    Return
    !!
23  Format('S',a2,'(',i3.3,')',i1,a1,SP,i2,'     ')
25  Format('V',a2,'(',i3.3,')',i1,a1,SP,i2,'     ')
24  Format()
    !!
  End Subroutine stm_write_weight_header
  !!***********************************************************************
  Subroutine stm_calc_weights(unit,stm,ZZ,eb,io,iu,kpnt)
    !!
    Implicit None
    !!IO
    Type(T_STM),Intent(in)       :: stm
    Integer(t_int),Intent(in)    :: unit,io,iu
    Complex(t_real),Intent(in)   :: ZZ(:,:)
    Real(t_real),Intent(in)      :: eb(:),kpnt
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_calc_weights"
    Integer(t_int) :: in,inn,iw,lnband,nw,nvdim,iz
    Real(t_real),Pointer      :: p_weights(:,:)
    Complex(t_real):: caa
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    If(.Not.stm% active) Return
    !!
    Nullify(p_weights)
    !!
    lnband=iu-io+1
    !!
    nw=stm% nvdim*2
    nvdim=Size(ZZ,1)
    !!
    Call palloc('p_weights',p_weights,1_t_int,nw,1_t_int,lnband&
         &,nmodule,nsubfunc)
    !!
    p_weights(:,:)=0.0_t_real
    Do in=io,iu
       inn=in-io+1
       Do iw=1,stm% nvdim
          caa=c0
          Do iz=1,nvdim
             caa=caa+stm% p_Sts(iw,iz)*ZZ(iz,in)
          Enddo
          p_weights(2*iw-1,inn)=Abs(caa)**2
          caa=c0
          Do iz=1,nvdim
             caa=caa+stm% p_Vts(iw,iz)*ZZ(iz,in)
          Enddo
          p_weights(2*iw,inn)=Abs(caa)**2*hatoev**2 !! in eV^2
       Enddo
    End Do
    !!
    !!
    !!
    Do in=io,iu
       inn=in-io+1
       Write(unit,30,advance='no')kpnt,eb(in)
       !!
       Do iw=1,nw
          Write(unit,31,advance='no')p_weights(iw,inn)
       Enddo       ! mv
       Write(unit,24)
       !!
    Enddo ! iband
    !!
    Call pfree(p_weights)
    !!
    !!
    Return
    !!
24  Format()
30  Format(f16.7,1x,f16.7)
31  Format(1x,e16.7e3)
    !!
  End Subroutine stm_calc_weights
  !!***********************************************************************
  Subroutine stm_calc_dIdVMatEl(structure,basis,stm,Vts,ZZ,ik,ms,nvdim&
       &,vblockstart,vblockend)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Basis),Intent(in)     :: basis
    Type(T_STM),Intent(in)       :: stm
    Complex(t_real),Intent(in)   :: Vts(:,:),ZZ(:,:)
    Integer(t_int),Intent(in)    :: nvdim,ik,ms,vblockstart(:),vblockend(:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_calc_dIdVMatEl"
    Integer(t_int) :: iz,ib,isa,issa,ivw,degw,mw,iw,ist,ied,ivr,deg,inl,issb
    Real(t_real)   :: d
    Complex(t_real):: cc,ccs
    Complex(t_real),Pointer:: p_cc(:)
    !!
    Nullify(p_cc)
    !!
    !!
    Call palloc('p_cc',p_cc,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    iw=0
    Do ivw=1,stm% nRv
       degw=2*stm% p_lRv(ivw)+1
       Do mw=1,degw
          iw=iw+1
          !!
          Do ib=1,nvdim
             p_cc(:)=Vts(iw,:)*ZZ(:,ib)
             cc=Sum(p_cc)
             Do isa=1,structure% nsite
                issa=structure% p_isrt(isa)
                ist=vblockstart(isa)
                ied=vblockend(isa)
                ccs=Conjg(Sum(p_cc(ist:ied)))
                stm% p_akdIdV(ib,ik,ivw,issa,ms)=&
                     &stm% p_akdIdV(ib,ik,ivw,issa,ms)&
                     &+Real(ccs*cc,t_real)
                Do inl=1,stm% N_sort_nl
                   issb=stm% p_sort_nl(inl)
                   If(issb==issa)Then
                      iz=ist-1
                      Do ivr=1,basis% p_nRv(issa)
                         deg=basis% p_degRv(ivr,issa)
                         d=Real(Sum(p_cc(iz+1:iz+deg))*Conjg(cc),t_real)
                         !d=Sum(Abs(p_cc(iz+1:iz+deg))**2)
                         iz=iz+deg
                         stm% p_akdIdVnl(ib,ik,ivw,inl,ivr,ms)=&
                              &stm% p_akdIdVnl(ib,ik,ivw,inl,ivr,ms)&
                              &+d
                      Enddo
                   Endif
                Enddo !inl
                !!
             End Do ! isa
          End Do ! ib
       End Do ! mw
    End Do ! ivw
    !! 
    Call pfree(p_cc)
    !!
    Return
    !!
  End Subroutine stm_calc_dIdVMatEl 
  !!***********************************************************************
  Subroutine stm_calc_dIdV(mspin,structure,stm,kohnsham,basis,bzone,Nop)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_STM),Intent(in)       :: stm
    Integer(t_int),Intent(in)    :: mspin,Nop
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_calc_dIdV"
    Integer(t_int) :: nptdos,ms,ivw,issa,ie,inl,ivr
    Character(100) :: str
    Real(t_real)   :: v0,v1,spinfac,beta,ef,dv,e,d,v
    Real(t_real),Pointer :: dIdV(:)
    Real(t_real),Pointer :: dIdVtot(:)
    Real(t_real),Pointer :: tmp(:)
    Real(t_real),Pointer :: agrs(:,:)
    !!
    v0=stm% v0/hatoev
    v1=stm% v1/hatoev
    nptdos=stm% nev
    !!
    spinfac=2.0_t_real/mspin
    !!
    !!     
    Nullify(dIdV,dIdVtot,tmp,agrs)
    !!
    !!
    !!
    beta=1.0_t_real/(bzone% kTcur/(eVtokT*hatoev))
    !!
    !!
    !!
    !!
    !!     
    Call palloc('dIdV',dIdV,1_t_int,nptdos,nmodule,nsubfunc)
    Call palloc('dIdVtot',dIdVtot,1_t_int,nptdos,nmodule,nsubfunc)
    Call palloc('tmp',tmp,1_t_int,nptdos,nmodule,nsubfunc)
    Call palloc('agrs',agrs,1_t_int,kohnsham% nvdim,1_t_int,bzone% nkp&
         &,nmodule,nsubfunc)
    !!
    !!
    spinloop: Do ms=1,mspin
       ef=kohnsham% efermi
       
       
       dIdVtot(:)=0.0_t_real
       issaloop: Do issa=1,structure% nsort
          Do ivw=1,stm% nRv
             agrs=0.0_t_real
             e=(ef-stm% p_ev(ivw))
             d=stm% p_delta(ivw)/hatoev/Pi/(e**2+stm% p_delta(ivw)/hatoev**2)
             If(bzone% gammapoint)Then
                agrs(:,1)=agrs(:,1)+d*stm% p_akdIdV(:,1,ivw,issa,ms)
             Else
                agrs(:,:)=agrs(:,:)+d*stm% p_akdIdV(:,:,ivw,issa,ms)
             Endif
             !!
             dIdV(:)=0.0_t_real
             tmp(:)=0.0_t_real
             !!
             If(bzone% gammapoint)Then
                Call gamdos(dIdV(:),tmp(:)&
                     &,kohnsham% p_ebkfsm(:,1,ms),v1+ef,v0+ef,kohnsham% nvdim&
                     &,nptdos&
                     &,agrs,spinfac,0_t_int,beta)
             Else
                Call tetdos(dIdV(:),tmp(:)&
                     &,kohnsham% p_ebkfsm(:,:,ms),v1+ef,v0+ef,bzone% p_tet&
                     &,kohnsham% nvdim,bzone% nkp&
                     &,nptdos,bzone% ntet,bzone% ntetsum&
                     &,agrs,spinfac,0_t_int)
             Endif
             dIdV=dIdV/Nop
             !!
             dIdVtot=dIdVtot+dIdV
             Write(str,"('+dIdV.sort',i3.3,'.spin',i1,'.tipstate',i1,a1)")&
                  &issa,ms,stm% p_nnRv(ivw),lqnchar(stm% p_lRv(ivw))
             Open(12,file=Trim(str))
             dv = (v1-v0)/(nptdos-1)
             Do ie=1,nptdos
                v=Real(ie-1,t_real)*dv+v0
22              Format(2(e30.15e3,2x))
                Write(12,22)v*hatoev,dIdV(ie)
             Enddo
             Close(12)
             !!
          Enddo
       Enddo issaloop
       !!
       Write(str,"('+dIdV.total.spin',i1)") ms
       Open(12,file=Trim(str))
       dv = (v1-v0)/(nptdos-1)
       Do ie=1,nptdos
          v=Real(ie-1,t_real)*dv+v0
          Write(12,22)v*hatoev,dIdVtot(ie)
       Enddo
       Close(12)
       !!
       inlloop: Do inl=1,stm% N_sort_nl
          issa=stm% p_sort_nl(inl)
          ivrloop: Do ivr=1,basis%  p_nRv(issa)
             Do ivw=1,stm% nRv
                agrs=0.0_t_real
                e=(ef-stm% p_ev(ivw))
                d=stm% p_delta(ivw)/hatoev/Pi/(e**2+stm% p_delta(ivw)/hatoev**2)
                If(bzone% gammapoint)Then
                   agrs(:,1)=agrs(:,1)+d*stm% p_akdIdVnl(:,1,ivw,inl,ivr,ms)
                Else
                   agrs(:,:)=agrs(:,:)+d*stm% p_akdIdVnl(:,:,ivw,inl,ivr,ms)
                Endif
                Print *,inl,ivr,Sum(stm% p_akdIdVnl(:,:,:,inl,ivr,ms))
                !!
                dIdV(:)=0.0_t_real
                tmp(:)=0.0_t_real
                !!
                If(bzone% gammapoint)Then
                   Call gamdos(dIdV(:),tmp(:)&
                        &,kohnsham% p_ebkfsm(:,1,ms),v1+ef,v0+ef,kohnsham% nvdim&
                        &,nptdos&
                        &,agrs,spinfac,0_t_int,beta)
                Else
                   Call tetdos(dIdV(:),tmp(:)&
                        &,kohnsham% p_ebkfsm(:,:,ms),v1+ef,v0+ef,bzone% p_tet&
                        &,kohnsham% nvdim,bzone% nkp&
                        &,nptdos,bzone% ntet,bzone% ntetsum&
                        &,agrs,spinfac,0_t_int)
                Endif
                dIdV=dIdV/Nop
                !!
                Write(str,"('+dIdV.sort',i3.3,'.spin',i1,'.nl',i3.3&
                     &,'.tipstate',i1,a1)")&
                     &issa,ms,ivr,stm% p_nnRv(ivw),lqnchar(stm% p_lRv(ivw))
                Open(12,file=Trim(str))
                Write(12,"('# nl=',i1,a1)")&
                     &basis% p_nnRv(ivr,issa),lqnchar(basis% p_lRv(ivr,issa))
                dv = (v1-v0)/(nptdos-1)
                Do ie=1,nptdos
                   v=Real(ie-1,t_real)*dv+v0
                   Write(12,22)v*hatoev,dIdV(ie)
                Enddo
                Close(12)
                !!
             Enddo
          Enddo ivrloop
       Enddo inlloop
       !!
       !!
    Enddo spinloop
    !!
    !!
    !!
    !!
    Call pfree(dIdV)
    Call pfree(dIdVtot)
    Call pfree(tmp)
    Call pfree(agrs)
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine stm_calc_dIdV
  !!***********************************************************************
  Subroutine stm_calc_dIdV1(mspin,structure,stm,kohnsham,basis,bzone&
       &,dIdV,dIdVnl,Nop)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Bzone),Intent(in)     :: bzone
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_STM),Intent(in)       :: stm
    Integer(t_int),Intent(in)    :: mspin,Nop
    Real(t_real),Intent(inout)   :: dIdV(:,:,:)
    Real(t_real),Intent(inout)   :: dIdVnl(:,:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_calc_dIdV1"
    Integer(t_int) :: nptdos,ms,ivw,issa,ie,inl,ivr
    Real(t_real)   :: v0,v1,spinfac,beta,ef,dv,e,d
    Real(t_real),Pointer :: tmp(:)
    Real(t_real),Pointer :: agrs(:,:)
    !!
    v0=stm% v0/hatoev
    v1=stm% v1/hatoev
    nptdos=stm% nev
    dv = (v1-v0)/(nptdos-1)
    !!
    Call palloc('stm% p_voltage',stm% p_voltage,1_t_int,stm% nev&
         &,nmodule,nsubfunc)
    Do ie=1,stm% nev
       stm% p_voltage(ie)=(Real(ie-1,t_real)*dv+v0)*hatoev
    Enddo
    !!
    spinfac=2.0_t_real/mspin
    !!
    !!     
    Nullify(tmp,agrs)
    !!
    !!
    !!
    beta=1.0_t_real/(bzone% kTcur/(eVtokT*hatoev))
    !!
    !!
    !!
    !!
    !!     
    Call palloc('tmp',tmp,1_t_int,nptdos,nmodule,nsubfunc)
    Call palloc('agrs',agrs,1_t_int,kohnsham% nvdim,1_t_int,bzone% nkp&
         &,nmodule,nsubfunc)
    !!
    !!
    spinloop: Do ms=1,mspin
       ef=kohnsham% efermi

       !!
       Do issa=1,structure% nsort
          agrs=0.0_t_real
          Do ivw=1,stm% nRv
             e=(ef-stm% p_ev(ivw))
             d=stm% p_delta(ivw)/hatoev/Pi/(e**2+stm% p_delta(ivw)/hatoev**2)
             If(bzone% gammapoint)Then
                agrs(:,1)=agrs(:,1)+d*stm% p_akdIdV(:,1,ivw,issa,ms)
             Else
                agrs(:,:)=agrs(:,:)+d*stm% p_akdIdV(:,:,ivw,issa,ms)
             Endif
          Enddo
          !!
          If(bzone% gammapoint)Then
             Call gamdos(dIdV(:,issa,ms),tmp(:)&
                  &,kohnsham% p_ebkfsm(:,1,ms),v1+ef,v0+ef,kohnsham% nvdim&
                  &,nptdos&
                  &,agrs,spinfac,0_t_int,beta)
          Else
             Call tetdos(dIdV(:,issa,ms),tmp(:)&
                  &,kohnsham% p_ebkfsm(:,:,ms),v1+ef,v0+ef,bzone% p_tet&
                  &,kohnsham% nvdim,bzone% nkp&
                  &,nptdos,bzone% ntet,bzone% ntetsum&
                  &,agrs,spinfac,0_t_int)
          Endif
          dIdV(:,issa,ms)=dIdV(:,issa,ms)/Nop
          dIdV(:,structure% nsort+1,ms)=dIdV(:,structure% nsort+1,ms)&
               &+dIdV(:,issa,ms)
       Enddo ! issa
       !!
       !!
       inlloop: Do inl=1,stm% N_sort_nl
          issa=stm% p_sort_nl(inl)
          ivrloop: Do ivr=1,basis%  p_nRv(issa)
             agrs=0.0_t_real
             Do ivw=1,stm% nRv
                e=(ef-stm% p_ev(ivw))
                d=stm% p_delta(ivw)/hatoev/Pi/(e**2+stm% p_delta(ivw)/hatoev**2)
                If(bzone% gammapoint)Then
                   agrs(:,1)=agrs(:,1)+d*stm% p_akdIdVnl(:,1,ivw,inl,ivr,ms)
                Else
                   agrs(:,:)=agrs(:,:)+d*stm% p_akdIdVnl(:,:,ivw,inl,ivr,ms)
                Endif
             Enddo
             !!
             If(bzone% gammapoint)Then
                Call gamdos(dIdVnl(:,inl,ivr,ms),tmp(:)&
                     &,kohnsham% p_ebkfsm(:,1,ms),v1+ef,v0+ef,kohnsham% nvdim&
                     &,nptdos&
                     &,agrs,spinfac,0_t_int,beta)
             Else
                Call tetdos(dIdVnl(:,inl,ivr,ms),tmp(:)&
                     &,kohnsham% p_ebkfsm(:,:,ms),v1+ef,v0+ef,bzone% p_tet&
                     &,kohnsham% nvdim,bzone% nkp&
                     &,nptdos,bzone% ntet,bzone% ntetsum&
                     &,agrs,spinfac,0_t_int)
             Endif
             dIdVnl(:,inl,ivr,ms)=dIdVnl(:,inl,ivr,ms)/Nop
          Enddo ivrloop ! issa
          !!
       End Do inlloop
       !!
    Enddo spinloop
    !!
    !!
    !!
    !!
    Call pfree(tmp)
    Call pfree(agrs)
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine stm_calc_dIdV1
  !!***********************************************************************
  Subroutine stm_calc_dos(mspin,nspin,symmetry,structure,stm,kohnsham&
       &,basis,bzone)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_STM),Intent(inout)    :: stm
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Basis),Intent(in)     :: basis
    Type(T_BZone),Intent(in)     :: bzone
    Integer(t_int),Intent(in)    :: mspin,nspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_calc_dos"
    Integer(t_int) :: nvdim,nkp,ms,ik,subdiv,Nop,jops
    Logical(t_log) :: fbz
    Real(t_real)   :: qq(3),gk(3)
    Complex(t_real),Pointer:: p_Cg(:,:)
    Character(*),Parameter  :: cpumesg_stmdos='STM DOS'
    !!
    If(.Not.stm% active) Return
    !!
    Nullify(p_Cg)
    !!
    nvdim=kohnsham% nvdim
    nkp=bzone% nkp
    If(nvdim/=kohnsham% nband)Then
       Write(errbuf,99)
99     Format("'STM module cannot deal with the number-of-bands option!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    If(.Not.stm% active)Return
    !!
    fbz=isoption('FULLBZ')
    If(fbz)Then
       Nop=1_t_int
    Else
       Nop=symmetry% Rnumops
    Endif
    !!
    Call watch(cpumesg_stmdos,cpu_start,reset=.True.)
    !!
    Call palloc('stm% p_Sts',stm% p_Sts,1_t_int,stm% nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_Vts',stm% p_Vts,1_t_int,stm% nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_akdIdV',stm% p_akdIdV,1_t_int,nvdim,1_t_int,nkp&
         &,1_t_int,stm% nRv,1_t_int,structure% nsort,1_t_int,mspin&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_akdIdVnl',stm% p_akdIdVnl,1_t_int,nvdim,1_t_int,nkp&
         &,1_t_int,stm% nRv,1_t_int,stm% N_sort_nl&
         &,1_t_int,basis% nRvmx,1_t_int,mspin&
         &,nmodule,nsubfunc)
    Call palloc('p_Cg',p_Cg,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    stm% p_akdIdV=0.0_t_real
    stm% p_akdIdVnl=0.0_t_real
    !!
    !!
    !!
    subdiv=(Max(1_t_int,nkp/(100/10)));
    !!
    stmspinloop: Do ms=1,nspin
       !!
       !!
       !!
       !!
       stmkloop: Do ik=1,nkp
          qq(:)=bzone% p_bk(:,ik)
          !! 
          If ( Mod(ik,subdiv) ==0 .Or. ik==1 .Or.ik==nkp)Then
             Write(outfile,"('STM: ',f6.2,'% done')") &
                  &100.0_t_real*ik/nkp
          Endif
          !!
          jopsloop: Do jops=1,Nop
             !!
             !! create C^{alpha*k}_{s\mu,n}
             If(fbz) Then 
                p_Cg=Transpose(kohnsham% p_ccvk(:,ik,:,ms))
                gk=qq
             Else
                gk(:)=Matmul(qq,symmetry% Ropmat(:,:,jops))
                Call create_C_alpha_K(symmetry,structure,basis&
                     &,kohnsham,jops,Transpose(kohnsham% p_ccvk(:,ik,:,ms))&
                     &,p_Cg)
             Endif
             !!
             Call stm_lomatrix(structure% nsite,stm,gk&
                  &,kohnsham% p_vblockstart,kohnsham% p_vblockend)
             !!
             !!
             !!
             Call stm_calc_dIdVMatEl(structure,basis,stm,stm% p_Vts&
                  &,p_Cg,ik,ms,nvdim&
                  &,kohnsham% p_vblockstart,kohnsham% p_vblockend)
             !!
          End Do jopsloop
          !!
       End Do stmkloop
    End Do stmspinloop
    !!
    Call stm_calc_dIdV(mspin,structure,stm,kohnsham,basis,bzone,Nop)
    !!
    !!
    !!
    Call pfree(stm% p_Sts)
    Call pfree(stm% p_Vts)
    Call pfree(stm% p_akdIdV)
    Call pfree(stm% p_akdIdVnl)
    Call pfree(p_Cg)
    !!
    Call watch(cpumesg_stmdos,cpu_stop,delete=.True.,Print=.True.)
    !!
    Return
    !!
  End Subroutine stm_calc_dos
  !!***********************************************************************
  Subroutine stm_make_map(mspin,nspin,symmetry,structure,potential&
       &,basis,mol,kohnsham,numerics,stm,bzone)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_STM),Intent(inout)    :: stm
    Type(T_Mol),Intent(in)       :: mol
    Type(T_Numerics),Intent(in)  :: numerics
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Potential),Intent(in) :: potential
    Type(T_BZone),Intent(in)     :: bzone
    Integer(t_int),Intent(in)    :: mspin,nspin
    !!LOCAL
    Character(*),Parameter::nsubfunc="stm_make_map"
    Integer(t_int) :: nvdim,nkp,ms,ik,subdiv,imap,ie,jops,Nop,nsort,issa
    Integer(t_int) :: inl,ivr
    Logical(t_log) :: fbz
    Real(t_real)   :: qq(3),d,gk(3)
    Character(200) :: fname
    Real(t_real),Pointer :: dIdV(:,:,:,:)
    Real(t_real),Pointer :: dIdVnl(:,:,:,:,:)
    Complex(t_real),Pointer:: p_Cg(:,:)
    Character(*),Parameter  :: cpumesg_stmmap='STM MAP'
    !!
    If(.Not.stm% active) Return
    !!
    Nullify(dIdV,dIdVnl,p_Cg)
    !!
    nsort=structure% nsort
    nvdim=kohnsham% nvdim
    nkp=bzone% nkp
    If(nvdim/=kohnsham% nband)Then
       Write(errbuf,99)
99     Format("'STM module cannot deal with the number-of-bands option!'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Endif
    !!
    If(.Not.stm% active)Return
    !!
    fbz=isoption('FULLBZ')
    If(fbz)Then
       Nop=1_t_int
    Else
       Nop=symmetry% Rnumops
    Endif
    !!
    Call watch(cpumesg_stmmap,cpu_start,reset=.True.)
    !!
    Call palloc('stm% p_Sts',stm% p_Sts,1_t_int,stm% nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_Vts',stm% p_Vts,1_t_int,stm% nvdim,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_akdIdV',stm% p_akdIdV,1_t_int,nvdim,1_t_int,nkp&
         &,1_t_int,stm% nRv,1_t_int,nsort,1_t_int,mspin&
         &,nmodule,nsubfunc)
    Call palloc('stm% p_akdIdVnl',stm% p_akdIdVnl,1_t_int,nvdim,1_t_int,nkp&
         &,1_t_int,stm% nRv,1_t_int,stm% N_sort_nl&
         &,1_t_int,basis% nRvmx,1_t_int,mspin&
         &,nmodule,nsubfunc)
    Call palloc('dIdV',dIdV,1_t_int,stm% nev,1_t_int,nsort+1&
         &,1_t_int,mspin,1_t_int,stm% mapN,nmodule,nsubfunc)
    Call palloc('dIdVnl',dIdVnl,1_t_int,stm% nev,1_t_int,stm% N_sort_nl&
         &,1_t_int,basis% nRvmx&
         &,1_t_int,mspin,1_t_int,stm% mapN,nmodule,nsubfunc)
    Call palloc('p_Cg',p_Cg,1_t_int,nvdim,1_t_int,nvdim,nmodule,nsubfunc)
    !!
    !!
    subdiv=(Max(1_t_int,stm% mapN/(100/10)));
    !! 
    dIdV=0.0_t_real
    imaploop: Do imap=1,stm% mapN
       !!
       If ( Mod(imap,subdiv) ==0 .Or. imap==1 .Or.imap==stm% mapN)Then
          Write(outfile,"('STM map: ',f6.2,'% done')") &
               &100.0_t_real*imap/stm% mapN
       Endif
       !!
       !!
       Call make_stm_lattice(symmetry,structure,potential,stm&
            &,stm% p_map(:,imap))
       !!
       Call stm_loi_int0(structure,basis,mol,numerics&
            &,potential% p_natom,potential% nuc_type,stm&
            &,kohnsham% p_vblocklength,kohnsham% p_cblocklength)
       !!
       !!
       !!
       stm% p_akdIdV=0.0_t_real
       stm% p_akdIdVnl=0.0_t_real
       !!
       !!
       !!
       stmspinloop: Do ms=1,nspin
          !!
          !!
          !!
          stmkloop: Do ik=1,nkp
             qq(:)=bzone% p_bk(:,ik)
             !!
             !!
             jopsloop: Do jops=1,Nop
                !!
                !!
                !!
                !! create C^{alpha*k}_{s\mu,n}
                If(fbz) Then 
                   p_Cg=Transpose(kohnsham% p_ccvk(:,ik,:,ms))
                   gk=qq
                Else
                   gk(:)=Matmul(qq,symmetry% Ropmat(:,:,jops))
                   Call create_C_alpha_K(symmetry,structure,basis&
                        &,kohnsham,jops,Transpose(kohnsham% p_ccvk(:,ik,:,ms))&
                        &,p_Cg)
                Endif
                !!
                !!
                Call stm_lomatrix(structure% nsite,stm,gk&
                     &,kohnsham% p_vblockstart,kohnsham% p_vblockend)
                !!
                !!
                Call stm_calc_dIdVMatEl(structure,basis,stm,stm% p_Vts&
                     &,p_Cg,ik,ms,nvdim&
                     &,kohnsham% p_vblockstart,kohnsham% p_vblockend)
                !!
             Enddo jopsloop
             !!
          End Do stmkloop
          !!
          !!
       End Do stmspinloop
       !!
       Call stm_calc_dIdV1(mspin,structure,stm,kohnsham,basis,bzone&
            &,dIdV(:,:,:,imap),dIdVnl(:,:,:,:,imap),Nop)
       !!
       !!
    Enddo imaploop
    !!
    !!
    !! write total
    Open(17,file='+map')
    Do imap=1,stm% mapN
       Write(17,"('#xy : ',e30.15e3,1x,e30.15e3)")stm% p_mapxy(:,imap)
       Write(17,"('#xyz: ',e30.15e3,1x,e30.15e3,1x,e30.15e3)")stm% p_map(:,imap)
       Do ie=1,stm% nev
          d=dIdV(ie,nsort+1,1,imap)
          If(mspin==2)d=d+dIdV(ie,nsort+1,2,imap)
          Write(17,22)stm% p_voltage(ie),d
       Enddo
    Enddo
    Write(17,"('# number of tip positions: ',i5)")stm% mapN
    Close(17)
    !!
    !!write sort resolved
    Do issa=1,nsort
       Write(fname,"('+map.sort',i3.3)")issa
       Open(17,file=Trim(fname))
       Write(17,"('# number of tip positions: ',i5)")stm% mapN
       Do imap=1,stm% mapN
          Write(17,"('#xy : ',e30.15e3,1x,e30.15e3)")stm% p_mapxy(:,imap)
          Write(17,"('#xyz: ',e30.15e3,1x,e30.15e3,1x,e30.15e3)")stm% p_map(:,imap)
          Do ie=1,stm% nev
             d=dIdV(ie,issa,1,imap)
             If(mspin==2)d=d+dIdV(ie,issa,2,imap)
             Write(17,22)stm% p_voltage(ie),d
          Enddo
       Enddo
       Close(17)
    Enddo
    !!
    !!write sort nl resolved
    Do inl=1,stm% N_sort_nl
       issa=stm% p_sort_nl(inl)
       Do ivr=1,basis% p_nRv(issa)
          Write(fname,"('+map.sort',i3.3,'.nl',i3.3)")issa,ivr
          Open(17,file=Trim(fname))
          Write(17,"('# nl : ',i1,a1)")basis% p_nnRv(ivr,issa)&
               &,lqnchar(basis% p_lRv(ivr,issa))
          Write(17,"('# number of tip positions: ',i5)")stm% mapN
          Do imap=1,stm% mapN
             Write(17,"('#xy : ',e30.15e3,1x,e30.15e3)")stm% p_mapxy(:,imap)
             Write(17,"('#xyz: ',e30.15e3,1x,e30.15e3,1x,e30.15e3)")stm% p_map(:,imap)
             Do ie=1,stm% nev
                d=dIdVnl(ie,inl,ivr,1,imap)
                If(mspin==2)d=d+dIdVnl(ie,inl,ivr,2,imap)
                Write(17,22)stm% p_voltage(ie),d
             Enddo
          Enddo
          Close(17)
       Enddo
    Enddo
    !!
    !!
    Call pfree(stm% p_Sts)
    Call pfree(stm% p_Vts)
    Call pfree(stm% p_akdIdV)
    Call pfree(stm% p_akdIdVnl)
    Call pfree(stm% p_voltage)
    Call pfree(dIdV)
    Call pfree(dIdVnl)
    Call pfree(p_Cg)
    !!
    Call watch(cpumesg_stmmap,cpu_stop,delete=.True.,Print=.True.)
    !!
    Return
    !!
    !!
    !!
    !!
    Return
    !!
22  Format(2(e30.15e3,2x))
    !!
  End Subroutine stm_make_map
  !!***********************************************************************
  Subroutine create_C_alpha_K(symmetry,structure,basis,kohnsham&
       &,jops,Ck,Cgk)
    !!
    !! no exp(i k*s) phase factor is not returned, since it should drop
    !! out, of all cases considered so far
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Integer(t_int),Intent(in)    :: jops
    Complex(t_real),Intent(in)   :: Ck(:,:)
    Complex(t_real),Intent(out)  :: Cgk(:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="create_C_alpha_K"
    Integer(t_int) :: nvdim,ia0,la,dega,iva,iga0,isa,istga,ista,isgma
    Integer(t_int) :: issa
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !!
    !!
    !!
    !!
    nvdim=kohnsham% nvdim
    !!
    !!
    !!
    !Write(outfile,"(a)")Trim((symmetry% Ropsymbol(jops)))
    !!
    !!
    !!
    Cgk=c0
    isaloop: Do isa=1,structure% nsite
       issa=structure% p_isrt(isa)
       isgma=structure% p_equivsitesrev(jops,isa)
       ista=kohnsham% p_vblockstart(isa)-1
       istga=kohnsham% p_vblockstart(isgma)-1
          
       ia0=ista
       iga0=istga
       ivaloop: Do iva=1,basis% p_nRv(issa)
          dega=basis% p_degRv(iva,issa)
          la=basis% p_lRv(iva,issa)
          Cgk(ia0+1:ia0+dega,1:nvdim)=Cgk(ia0+1:ia0+dega,1:nvdim)&
               &+Matmul(symmetry% Rwu_ks(1:dega,1:dega,la,jops)&
               &,Ck(iga0+1:iga0+dega,1:nvdim))
          !!
          ia0=ia0+dega
          iga0=iga0+dega
       End Do ivaloop
    Enddo isaloop
    !!
    !!
    Return
    !!
  End Subroutine create_C_alpha_K
  !!***********************************************************************
  Subroutine tip_mat_analyse(symmetry,structure,basis,kohnsham&
       &,potential,mol,stm,numerics)
    !!
    Implicit None
    !!IO
    Type(T_STM),Intent(inout)    :: stm
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Mol),Intent(in)       :: mol
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Kohnsham),Intent(in)  :: kohnsham
    Type(T_Potential),Intent(in) :: potential
    Type(T_Numerics),Intent(in)  :: numerics
    !!LOCAL
    !Character(*),Parameter::nsubfunc="tip_mat_analyse"
    Logical(t_log) :: exists
    Integer(t_int) :: nvdim,Nx,ix,isb,irp,itci,issb,ivrb,degb,mb
    Integer(t_int) :: tnmain,tl,iw,ivw,degw,mw,nmain,l,irpmin,irpmin2,irestrict
    Real(t_real)   :: tau(3),x,pp,ppmin,ppmin2,direction(3),tolrestrict
    Real(t_real),Parameter :: tol=1.0e-6_t_real
    !!
    If(.Not.stm% active) Return
    !!
    Inquire(file='=.tip_mat_analyse',exist=exists)
    If(.Not. exists)Then
       Return
    Endif
    !!
    Open(12,file='=.tip_mat_analyse')
    Read(12,*)
    Read(12,*) isb
    Read(12,*)
    Read(12,*) tnmain , tl 
    Read(12,*)
    Read(12,*) nmain , l
    Read(12,*)
    Read(12,*) Nx
    Read(12,*)
    Read(12,*) direction 
    Read(12,*)
    Read(12,*) irestrict,tolrestrict
    !!
    !direction=Matmul(symmetry% lattice_vectors,direction)
    !!
    nvdim=kohnsham% nvdim
    !!
    !!
    !!
    !!
    Open(12,file='pmin')
    Open(13,file='pmin2')
    issb=structure% p_isrt(isb)
    Do ix=1,Nx
       x=(ix-1.0_t_real)/(Nx-1.0_t_real)
       tau=x*direction+stm% tau
       Print "('tau=',3(f30.15,1x))",tau
       Call make_stm_lattice(symmetry,structure,potential,stm,tau)
       !!
       irpmin=-1
       ppmin=1.0e30_t_real
       Do irp=1,stm% p_npairs(isb)
          pp=Sqrt(Sum(stm% p_pairs(:,irp,isb)**2))
          Write(outfile,"('p(',i5,')=',3f30.15,' |p|=',f30.15)") &
               &irp,stm% p_pairs(:,irp,isb)&
               &,pp
          If(Abs(stm% p_pairs(irestrict,irp,isb)) >tolrestrict)Cycle
          If(pp<ppmin+tol)Then
             ppmin=pp
             irpmin=irp
          Endif
       End Do
       !Print *,'ppmin=',irpmin,ppmin
       irpmin2=-1
       ppmin2=1.0e30_t_real
       Do irp=1,stm% p_npairs(isb)
          If(irp==irpmin) Cycle
          If(Abs(stm% p_pairs(irestrict,irp,isb)) >tolrestrict)Cycle
          pp=Sqrt(Sum(stm% p_pairs(:,irp,isb)**2))
          If(pp<ppmin2+tol)Then
             ppmin2=pp
             irpmin2=irp
          Endif
       End Do
       !Print *,'ppmin2=',irpmin2,ppmin2
       
       Call stm_loi_int0(structure,basis,mol,numerics&
            &,potential% p_natom,potential% nuc_type,stm&
            &,kohnsham% p_vblocklength,kohnsham% p_cblocklength)
       !!
       Do irp=1,stm% p_npairs(isb)
          pp=Sqrt(Sum(stm% p_pairs(:,irp,isb)**2))
          If(.Not.(irp==irpmin.Or.irp==irpmin2)) Cycle
          Write(outfile,"('p(',i5,')=',3f30.15,' |p|=',f30.15)") &
               &irp,stm% p_pairs(:,irp,isb)&
               &,pp
          !          Write(outfile,10) isb,irp
          !10        Format(' siteB=',i5,' pair=',i5)
          
          itci=stm% p_tcivvindex(irp,isb)
          !!
          If(irp==irpmin)  Write(12,"(2(e30.15e3,1x))",advance='no')x,pp
          If(irp==irpmin2) Write(13,"(2(e30.15e3,1x))",advance='no')x,pp
          iw=0
          Do ivw=1,stm% nRv
             degw=2*stm% p_lRv(ivw)+1
             Do mw=1,degw
                iw=iw+1
                !!
                Do ivrb=1,basis% p_nRv(issb)
                   degb=basis% p_degRv(ivrb,issb)
                   If((stm% p_nnRv(ivw)==tnmain.And.stm% p_lRv(ivw)==tl).And.&
                        &(basis% p_nnRv(ivrb,issb)==nmain.And.&
                        & basis% p_lRv (ivrb,issb)==l ))Then
                      Do mb=1,degb
                         If(irp==irpmin) Write(12,"(1x,e30.15e3)",advance='no')&
                              &stm% p_tci_vHv0(itci+mb)
                         If(irp==irpmin2) Write(13,"(1x,e30.15e3)",advance='no')&
                              &stm% p_tci_vHv0(itci+mb)
                      Enddo
                   Endif
                   itci=itci+degb
                End Do
             End Do
          End Do ! ivw
          If(irp==irpmin) Write(12,*)
          If(irp==irpmin2) Write(13,*)
       End Do ! irp
       !!
    Enddo ! ix
    Close(12)
    Close(13)
    !!
    !!
    Stop
    !!
    Return
    !!
  End Subroutine tip_mat_analyse
  !!***********************************************************************


End Module stm
