  !!----DEBUG options-----------------|
  !! SUBFUNC    | MODES               |
  !!----------------------------------|
  !!----------------------------------|
Module surfaceintegration
  !!
  Use precisiondefs
  Use errors
  Use file_iounits
  Use constants
  Use integrate1d
  Use nsphintirred
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="SURFACEINTEGRATION"  
  !!
  Public :: sph_zyl_mesh
  !!***********************************************************************
  !! meshes for three center integration
!!$  Integer(t_int) :: nsp1,nsp2,nsp3,nsp4,nsp5,nsp6
!!$  Real(t_real),Target :: wg1(8),ag1(3,8),wg2(18),ag2(3,18) ,wg3(26),ag3(3,26)&
!!$       &,wg4(50),ag4(3,50) ,  wg5(116),ag5(3,116)  
!!$  Real(t_real),Pointer ::  wg6(:),ag6(:,:)
  
Contains
!!$  !!***********************************************************************
!!$  !!
!!$  !! get surface integration points and weights for the not rather
!!$  !! accurate three center integration
!!$  !!
!!$  !! !! wellcome to the weakest part of FPLO !!
!!$  !!
!!$  Subroutine get_thci_surf(r,p_ag,p_wg,nga)
!!$    Implicit None 
!!$    !!IO
!!$    Real(t_real),Intent(in)     :: r
!!$    Integer(t_int),Intent(out)   :: nga
!!$    Real(t_real),Pointer        :: p_ag(:,:)
!!$    Real(t_real),Pointer        :: p_wg(:)
!!$    !!LOCAL
!!$    !!     
!!$    If(r < 5.0e-2_t_real) Then
!!$       nga=nsp1
!!$       p_ag=>ag1
!!$       p_wg=>wg1
!!$       Return
!!$    Else If(r < 1.0e-1_t_real) Then
!!$       nga=nsp2
!!$       p_ag=>ag2
!!$       p_wg=>wg2
!!$       Return
!!$    Else If(r < 5.0e-1_t_real) Then
!!$       nga=nsp3
!!$       p_ag=>ag3
!!$       p_wg=>wg3
!!$       Return
!!$    Else If(r < 1.0e+0_t_real) Then
!!$       nga=nsp4
!!$       p_ag=>ag4
!!$       p_wg=>wg4
!!$       Return
!!$    Else If(r < 2.5e+0_t_real) Then
!!$       nga=nsp5
!!$       p_ag=>ag5
!!$       p_wg=>wg5
!!$       Return
!!$    Endif
!!$    nga=nsp6
!!$    p_ag=>ag6
!!$    p_wg=>wg6
!!$    Return  
!!$  End Subroutine get_thci_surf
  !!***********************************************************************

!!$  !!***********************************************************************
!!$  Subroutine setup_sphere_integration1(hex)
!!$    !!
!!$    Implicit None 
!!$    !!IO
!!$    Logical(t_log),Intent(in)    :: hex
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="setup_sphere_integration1"    
!!$    Integer(t_int) :: pgrpnum
!!$    Integer(t_int) :: nv
!!$    Logical(t_log) :: reducephi,prot,lhex
!!$    !!
!!$    lhex=hex
!!$    !! prepare meshes for three center integration
!!$    If(debugfun(nmodule,nsubfunc,'FORCE_HEX'))Then
!!$       lhex=.True.
!!$    Endif
!!$    If(debugfun(nmodule,nsubfunc,'FORCE_CUB'))Then
!!$       lhex=.False.
!!$    Endif
!!$
!!$    pgrpnum=1_t_int
!!$    !!
!!$    nv=11_t_int !! default maximum mesh
!!$    reducephi=.True.
!!$1    Inquire(file='=.nvgauss',exist=exnvg)
!!$1    If (exnvg) Then
!!$1       Open(12,file='=.nvgauss')
!!$1       Read(12,*) nv,lhex,reducephi
!!$1    Endif
!!$    !!
!!$    prot=debugfun(nmodule,nsubfunc,'')
!!$    If(prot .And. lhex)Write(outfile,*) 'use hexagonal meshes'
!!$    If(prot .And. .Not. lhex)Write(outfile,*) 'use cubic meshes'
!!$    !!
!!$    Call gsphirred( 1_t_int,ag1,wg1,nsp1,lhex,pgrpnum,prot)
!!$    If(prot) Write(outfile,*) 'SURF1:',nsp1,' points'
!!$    Call gsphirred( 2_t_int,ag2,wg2,nsp2,lhex,pgrpnum,prot)
!!$    If(prot) Write(outfile,*) 'SURF2:',nsp2,' points'
!!$    Call gsphirred( 3_t_int,ag3,wg3,nsp3,lhex,pgrpnum,prot)
!!$    If(prot) Write(outfile,*) 'SURF3:',nsp3,' points'
!!$    Call gsphirred( 5_t_int,ag4,wg4,nsp4,lhex,pgrpnum,prot)
!!$    If(prot) Write(outfile,*) 'SURF4:',nsp4,' points'
!!$    Call gsphirred( 8_t_int,ag5,wg5,nsp5,lhex,pgrpnum,prot)
!!$    If(prot) Write(outfile,*) 'SURF5:',nsp5,' points'
!!$    If (nv >= 100_t_int) Then
!!$       nv=nv-100_t_int
!!$       Call sph_zyl_mesh(nv,ag6,wg6,nsp6,reducephi)
!!$    Else
!!$       !! this is rubbish, in the moment 1000 is enoughm but its
!!$       !! not general, as palloc suggests
!!$       Call palloc('ag6',ag6 ,1_t_int,3_t_int ,1_t_int,1000_t_int &
!!$            &,nmodule,nsubfunc)
!!$       Call palloc('wg6',wg6 ,1_t_int,1000_t_int ,nmodule,nsubfunc)
!!$       Call gsphirred(nv,ag6,wg6,nsp6,lhex,pgrpnum,prot)
!!$    Endif
!!$    If(prot) Write(outfile,*) 'SURF6:',nsp6,' points'
!!$    !If(prot) Call terminate(TERM_NORMAL,Trim(nsubfunc)//' test mode')
!!$  End Subroutine setup_sphere_integration1
  !!***********************************************************************
  Subroutine  sph_zyl_mesh(nv,ag,wg,nsp,reducephi)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: nv !! maximum L to be integrated is 2*nv+1
    Integer(t_int),Intent(out)   :: nsp
    Real(t_real),Pointer         :: ag(:,:),wg(:)
    Logical(t_log),Intent(in)    :: reducephi
    !!LOCAL
    Character(*),Parameter::nsubfunc="sph_zyl_mesh"    
    Integer(t_int) :: nit,nit2,nw,ir,iphi
    !Logical(t_log) :: exnvg
    Real(t_real) :: x,y,z,rjac,ww,dp,p,qq
    !! DONT FREE THEM !
    Real(t_real),Pointer   :: p_cc(:),p_xx(:)
    !!
    nit=nv*2_t_int+1_t_int    
    !!
    nsp=0_t_int
    !!allocate output arrays
    Call palloc('ag',ag ,1_t_int,3_t_int ,1_t_int,nit**2_t_int &
         &,nmodule,nsubfunc)
    Call palloc('wg',wg ,1_t_int,nit**2_t_int ,nmodule,nsubfunc)
    !!
    !! doit
    nw=0_t_int
    Call get_gauss(nit,p_xx,p_cc)
    Do ir=1,nit
       z=p_xx(ir)
       rjac=Sqrt(1.0_t_real-z*z)
       nit2=nit
       !! this is modification to avoid to many points
       If (reducephi .And. rjac < 0.25_t_real) nit2=nv+1 
       dp=twopi/Real(nit2,t_real)
       ww=p_cc(ir)*dp
       Do iphi=1,nit2
          nw=nw+1_t_int
          p=Real(iphi,t_real)*dp
          x=rjac*Cos(p)
          y=rjac*Sin(p)
          qq=Sqrt(x*x+y*y+z*z)
          ag(1,nw)=x/qq
          ag(2,nw)=y/qq
          ag(3,nw)=z/qq
          wg(nw)=ww
       Enddo                    ! if
    Enddo                     ! ir
    nsp=nw
    
    !!
  End Subroutine sph_zyl_mesh
  !!***********************************************************************
End Module surfaceintegration
