!!----DEBUG options-----------------|
!! SYMMETRY:relsymreduce            |
!! SYMMETRY:prep_double_group       |
!! SYMMETRY:prep_double_group:info  |
!!----------------------------------|
! TODO: make clean def_cpa_indexarrays
! TODO: symmetrization for relativistic variant
Module symmetry
  !!
  Use precisiondefs
  Use memory
  Use diracdefs
  Use file_iounits
  Use errors
  Use constants
  Use optionstest
  Use clock
  Use trelativistic
  Use matserv
  Use tsymmetry
  Use ioaux
  Use sort
  !!
  Implicit None 
  !!
  Private
  Public :: getSpacegroupName &
       &,getCrystallSystem &
       &,getCrystallSystemName &
       &,crystal_structure_setup,print_crystal_structure&
       &,make_lattice,def_cpa_indexarrays,def_pairs,prep_equivsites&
       &,relsymreduce,rotloc!,sym_analysis,tracesym
  !!***********************************************************************
  Character(*),Parameter::nmodule="SYMMETRY"    
  !!
  Real(t_real),Parameter :: etol=1.0e-5_t_real !! tolerance for symmetry equivalence
  !!
  !!***********************************************************************
Contains
  !!***********************************************************************
  
  Character(8) Function getSpacegroupName(spgrnum)
    !!
    implicit none 
    !!IO
    Integer(t_int),Intent(in) :: spgrnum
    !!LOCAL
    Character(*),Parameter::nsubfunc="getSpacegroupName"
    Character(8) :: buf
    Integer(t_int) :: err
    !!
    External cgetspacegroupname
    !!
    Call cgetspacegroupname(buf,spgrnum,err)
    Call cerrmsg(nmodule,nsubfunc,err)
    getSpacegroupName=buf
    return
  End Function getSpacegroupName
  
  !!***********************************************************************
  
  Character(12) Function getCrystallSystemName(num)
    !!
    implicit none 
    !!IO
    Integer(t_int),Intent(in) :: num
    !!LOCAL
    Character(*),Parameter::nsubfunc="getCrystallSystemName"
    Character(12) :: buf
    Integer(t_int) :: err
    !!
    External cgetcrystallsystemname
    !!
    Call cgetcrystallsystemname(buf,num,err)
    Call cerrmsg(nmodule,nsubfunc,err)
    getCrystallSystemName=buf
    return
  End Function getCrystallSystemName
  
  !!***********************************************************************
  Integer(t_int)  Function getCrystallSystem(spgrnum)
    !!
    implicit none 
    !!IO
    Integer(t_int),Intent(in)    :: spgrnum
    !!LOCAL
    Character(*),Parameter::nsubfunc="getCrystallSystem"
    Integer(t_int) :: c
    Integer(t_int) :: err
    External cgetspacegroupcrystallsystem
    Call cgetspacegroupcrystallsystem(c,spgrnum,err)
    Call cerrmsg(nmodule,nsubfunc,err)
    getCrystallSystem=c
    Return
  End Function getCrystallSystem


  !***********************************************************************
  !! IMPORTANT: this routine must be called with the structure arrays in the 
  !! non-relativistic setup 
  Subroutine def_cpa_cryststruct(structure,tol)
    !
    Implicit None
    !!IO 
    Type(T_Structure),Intent(inout) :: structure 
    Real(t_real) ,Intent(in)  :: tol
    !!LOCAL
    Character(*),Parameter::nsubfunc="def_cpa_cryststruct"
    Integer(t_int) :: ikaps,isa,isb,issaIO,ib,a,b
    Integer(t_int) :: wi(structure % nsite)
    Integer(t_int) :: ind(structure % nsite)
    Logical(t_log) :: random
    !!
    Call palloc('structure % p_kaps',structure % p_kaps&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('structure % p_IOconao',structure % p_IOconao&
         &,1_t_int,structure% IOnsort&
         &,nmodule,nsubfunc)

    !!
    structure % p_kaps=0_t_int
    ikaps=0_t_int
    Do isa=1_t_int,structure % nsite
       issaIO=structure % p_IOisrt(isa)
       If(structure % p_kaps(isa)==0_t_int)Then
          ikaps=ikaps+1_t_int
          structure % p_IOconao(issaIO)=1.0_t_real
          structure % p_kaps(isa)=isa
          
          Do isb=isa+1_t_int,structure % nsite
             
             If(Sum((structure % p_tau(:,isb)-&
                  &structure % p_tau(:,isa))**2_t_int)<tol)Then
                structure % p_kaps(isb)=isa
                structure % p_IOconao(structure % p_IOisrt(isb))=0.0_t_real
             Endif
          Enddo
       Endif
    Enddo
    !Do isa=1,structure % nsite
    !   Print *,isa,structure % p_kaps(isa)
    !Enddo
    structure % naps=ikaps
    !test of block structure
    Do isa=1_t_int,structure % nsite
       ib=0_t_int
       Do isb=1_t_int,structure % nsite
          If(structure % p_kaps(isa)==structure % p_kaps(isb)) ib=ib+1_t_int
       Enddo
       wi(isa)=ib
       ind(isa)=isa
    Enddo

    Do isa=1_t_int,structure % nsite-1_t_int
       Do isb=isa+1_t_int,structure % nsite
          a=wi(ind(isa))
          b=wi(ind(isb))
          If(a<b) Then
             a=ind(isb)
             ind(isb)=ind(isa)
             ind(isa)=a
          Else If(a==b)Then
             a=structure % p_kaps(ind(isa))
             b=structure % p_kaps(ind(isb))
             If(a>b) Then
                a=ind(isb)
                ind(isb)=ind(isa)
                ind(isa)=a
             Else If(a==b)Then
                a=structure % p_IOisrt(ind(isa))
                b=structure % p_IOisrt(ind(isb))
                If(a>b) Then
                   a=ind(isb)
                   ind(isb)=ind(isa)
                   ind(isa)=a
                Endif
             Endif
          Endif
       Enddo
    Enddo
    !     

    structure % p_IOisrt(:)=structure % p_IOisrt(ind(:))
    structure % p_tau(:,:)=structure % p_tau(:,ind(:))

    !
    structure % p_kaps=0_t_int
    ikaps=0_t_int
    structure% naps_a=0_t_int
    Do isa=1_t_int,structure % nsite
       issaIO=structure % p_IOisrt(isa) 
       If(structure % p_kaps(isa)==0_t_int)Then
          ikaps=ikaps+1_t_int
          structure % p_IOconao(issaIO)=1.0_t_real
          structure % p_kaps(isa)=isa
          random=.False.
          Do isb=isa+1_t_int,structure % nsite
             If(Sum((structure % p_tau(:,isb)&
                  &-structure % p_tau(:,isa))**2_t_int)<tol)Then
                structure % p_kaps(isb)=isa
                structure % p_IOconao(structure % p_IOisrt(isb))=0.0_t_real
                random=.True.
             Endif
          Enddo
          If(random.Or.Trim(structure% p_IOelements(issaIO)) /="E")Then
             structure% naps_a=structure% naps_a+1_t_int
          Endif
       Endif
    Enddo
    !      do isa=1,nsite
    !        print *,isa,kaps(isa)
    !      enddo
    structure % naps=ikaps
    !
  End Subroutine Def_cpa_cryststruct
  !
  !***********************************************************************
  !!***********************************************************************
  ! set up crystal structure of compound using symmetry definition
  !
  ! crystal_structure_setup
  ! {{{ 

  Subroutine crystal_structure_setup(symmetry,structure,complain_subgroups&
       &,save_syminfo)
    !!
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(inout)  :: symmetry
    Type(T_Structure),Intent(inout) :: structure
    Logical(t_log),Intent(in)    :: complain_subgroups,save_syminfo
    !!LOCAL
    Character(*),Parameter::nsubfunc="crystal_structure_setup"
    Logical(t_log) :: exists
    Integer(t_int) :: err,i,isa,isb,issaIO,j,jops
    Real(t_real)   :: tol,ss(3),dd(3),v(3),ccell(3,3),det,origin(1:3)
    Character(1)   :: msg

    !!
    External ccalcsymmetry,creportcalcsym,ccalcunitcell,csyminit&
         &,cgetreportbuffer,csymexit,cconvertvecrtgtostg,cgetsites

    !! setup crystal structure from structure definitions
    
    If(structure % lengthunit==SYM_LENGTH_ANGSTROEM) Then
       symmetry % lattice_constants = symmetry % lattice_constants/brtoan
    End If

    
    Call csyminit

    Inquire(file='=.shiftorigin',exist=exists)
    If(exists)Then
       Open(12,file='=.shiftorigin')
       Read(12,*)origin(1:3)
       Close(12)
    Endif
    

    Call ccalcsymmetry(symmetry % spgrpnum,structure% str_type&
         &,symmetry % lattice_constants &
         &,symmetry % axis_angles&
         &,symmetry% nugen,symmetry% iugen&
         &,symmetry % lattice_vectors &
         &,symmetry % reciprocial_lattice_vectors&
         &,ccell&
         &,structure % vuc &
         &,symmetry % pgrpnum,symmetry % pgr &
         &,symmetry % HEX &
         &,symmetry % RTG &
         &,symmetry %lattice &
         &,symmetry % rcentering &
         &,symmetry % gcentering &
         &,symmetry % IOnumops &
         &,symmetry % IOopmat &
         &,symmetry % IOoptau &
         &,symmetry % opgen &
         &,symmetry % opeuler &
         &,symmetry % opeulerinv &
         &,symmetry % IOopind &
         &,symmetry % inversion &
         &,symmetry % IOopsymbol &
         &,err,complain_subgroups)
    Call cerrmsg(nmodule,nsubfunc,err)
    
    Call prep_double_group(symmetry% IOopsymbol,symmetry% IOnumops,symmetry% opeulerinv&
         &,symmetry% opeuler,symmetry% IOopind)

    symmetry % reciprocial_lattice_vectors &
         & =symmetry % reciprocial_lattice_vectors*twopi

    If(symmetry% RTG) Then
       ccell = symmetry% lattice_vectors
    Endif

    If(structure% str_type == STR_TYPE_MOLECULE) Then
       symmetry % convreciprocial_lattice_vectors=0.0_t_real
       Do i=1,3
          symmetry % convreciprocial_lattice_vectors(i,i)=1.0_t_real
       Enddo
    Else
       Call dinv33(ccell,1_t_int&
            &,symmetry % convreciprocial_lattice_vectors,det)
    Endif

    symmetry % convreciprocial_lattice_vectors &
         & =symmetry % convreciprocial_lattice_vectors*twopi


    symmetry% reduced_sym=.False.

    
    Call creportcalcsym
    i=0_t_int
    If(save_syminfo)Then
       Open(13,file='+symminfo')
    Else
       Open(13,file='+symmetry')
    Endif
    Call cgetreportbuffer(msg,i)
    Do While(Ichar(msg)/=0)
       If(Ichar(msg)==13)Then
          Write(13,"()")
       Else
          Write(13     ,"(a1)",advance='no')msg
       Endif
       i=i+1_t_int
       Call cgetreportbuffer(msg,i)
    End Do
    Write(13,"()")
    Close(13)


    If(structure% str_type == STR_TYPE_MOLECULE) Then
       !! if molecule type the wyckoffs are in absolute units
       !! and need to be scaled according to the settings
       If(structure % lengthunit==SYM_LENGTH_ANGSTROEM) Then
          Do issaIO=1_t_int,structure% IOnsort
             structure% p_IOwtau(:,issaIO)=&
                  & structure% p_IOwtau(:,issaIO) /brtoan
          End Do
       End If
       !! if molecule the non-symmorphic part should be ignored
       symmetry % IOoptau(1:3,1:symmetry% IOnumops)=0.0_t_real
    End If
    
    
    tol=1.0e-4_t_real
    Call ccalcunitcell(structure % IOnsort,structure % p_IOwtau&
         &,structure % nsite,Trim(STR_TYPES(structure% str_type)),tol,err)
    Call cerrmsg(nmodule,nsubfunc,err)

    Call palloc('structure % p_tau',structure % p_tau&
         &,1_t_int,3_t_int &
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)

    Call palloc('structure % p_IOisrt',structure % p_IOisrt&
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)

    
    Call cgetsites(structure % nsite,structure % p_tau,structure % p_IOisrt)
    

    !! manipulate origin: debug option
    Do issaIO=1,structure% IOnsort
       structure% p_IOwtau(:,issaIO)=structure% p_IOwtau(:,issaIO)-origin(:)
    Enddo
    origin=Matmul(ccell,origin(:))
    Do isa=1_t_int,structure% nsite
       structure% p_tau(:,isa)=structure% p_tau(:,isa)-origin(:)
    Enddo
    Do jops=1,symmetry% IOnumops
       symmetry% IOoptau(1:3,jops)=symmetry% IOoptau(1:3,jops)&
            &-origin+Matmul(origin,symmetry% IOopmat(:,:,jops))
    Enddo
    

    !!CPA: check if two positions are equivalent by translation
    !!CPA: if so, unify site vectors
    If(structure% str_type == STR_TYPE_CRYSTAL) Then
       Do isa=1_t_int,structure% nsite
          Do isb=isa+1_t_int,structure% nsite
             ss(:)=structure% p_tau(:,isb)-structure% p_tau(:,isa)
             dd(1)=Sum(ss(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
             dd(2)=Sum(ss(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
             dd(3)=Sum(ss(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
             If( vecIsOnGrid(3_t_int,dd,etol) )Then
                structure% p_tau(:,isb)=structure% p_tau(:,isa)
             Endif
          Enddo
       Enddo
    Endif
    
    !!define CPA  crystal structure
    
    tol=1.e-7_t_real
    Call def_cpa_cryststruct(structure,tol)
    
    
    If(structure% str_type == STR_TYPE_MOLECULE) Then
       structure % vuc=1.0_t_real
    Endif

    !!work around for CPA
    !! START NEW: EMPTY SPHERE
    structure % arws=(structure % vuc/structure % naps_a&
         &/fourpi*3.0_t_real)**c1o3
    !! OLD: EMPTY SPHERE
    !! END NEW: EMPTY SPHERE



    Call csymexit
    !!
    !!
    Call palloc('structure % p_IOmult',structure % p_IOmult&
         &,1_t_int,structure% IOnsort ,nmodule,nsubfunc)
    !!
    Do issaIO=1,structure% IOnsort
       structure% p_IOmult(issaIO)=0_t_int
       Do isa=1,structure% nsite
          If(structure% p_IOisrt(isa) == issaIO )Then
             structure% p_IOmult(issaIO)=structure% p_IOmult(issaIO)+1_t_int
          Endif
       Enddo
    Enddo
    !!

    !!
    If(.True.)Then
       Open(89,file='=.str_template',status='unknown')
       Write(89,700)
       Write(89,701)symmetry % spgrpnum
       Do j=1,3
          If(structure % lengthunit==SYM_LENGTH_ANGSTROEM) Then
             Write(89,702) (symmetry% lattice_vectors(i,j)*brtoan,i=1,3)
          Else
             Write(89,702) (symmetry% lattice_vectors(i,j),i=1,3)
          Endif
       Enddo
       Write(89,701)structure % nsite
       !Write(89,704).True.
       Write(89,704).false.
       Do isa=1_t_int,structure % nsite
          issaIO=structure % p_IOisrt(isa)
          !v(:)=Matmul(structure % p_tau(:,isa)&
          !     &,symmetry% reciprocial_lattice_vectors(:,:))/twopi
          !Print *,symmetry% lattice
          !If(symmetry% lattice(1:2) == 'FC')Then
          !   v(:)=Matmul(v(:),Bfcc)
          !   !Print *,Bfcc(1,:)
          !   !Print *,Bfcc(2,:)
          !   !Print *,Bfcc(3,:)
          !Endif
          !!
          v(:)=structure % p_tau(:,isa)
          If(structure % lengthunit==SYM_LENGTH_ANGSTROEM) Then
             v(:)=structure % p_tau(:,isa)*brtoan
          Endif
          !!
          Write(89,703) structure % p_IOelements(issaIO),issaIO,v(:)
       Enddo
       Write(89,704).True.
       Write(89,1701) 1_t_int,0_t_int,0_t_int
       Write(89,1701) 0_t_int,1_t_int,0_t_int
       Write(89,1701) 0_t_int,0_t_int,1_t_int
       Close(89)
700    Format('1.e-6')
701    Format(i4)
1701   Format(3(i4,2x))
702    Format(3(f30.15,1x))
703    Format(a2,1x,i4,1x,3(f30.15,1x))
704    Format(l2)
    Endif
    !!
    !!
    Return
    !!
  End Subroutine crystal_structure_setup
  !!***********************************************************************
  Subroutine print_crystal_structure(symmetry,structure,lattice_constants_S&
       &,axis_angles_S,IOwtau_S)
    !!
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(in)     :: symmetry
    Type(T_Structure),Intent(in)    :: structure
    Character(*),Intent(in)         :: IOwtau_S(:,:)
    Character(*),Intent(in)         :: axis_angles_S(3)
    Character(*),Intent(in)         :: lattice_constants_S(3)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="print_crystal_structure"
    Integer(t_int) :: i,isa,issaIO
    Real(t_real)   :: dum(3)
    Character(200) :: fmt
    Integer        :: eof
    Character(5000):: buffer
    !!
    
    Write(outfile,"(72('='))") 
    Write(outfile,"(22x,'CALCULATION OF CRYSTALL STRUCTURE',/)") 
    Write(outfile,"(72('-'))") 
    Write(outfile,"(32x,'INPUT DATA',/)") 
    Write(outfile,"(72('-'))") 
    !! length units
    Write(outfile,"('Unit of length    : ',a)") &
         &Trim(SYM_LENGTH_UNITS(structure % lengthunit))
    Write(outfile,"('Spacegroup        : ',i4,' - ',a)") symmetry % spgrpnum &
         &,Trim(getSpacegroupName(symmetry % spgrpnum))


    i=Max(Int(Max(&
         &        Maxval(Len_trim(lattice_constants_S(1:3)))&
         &       ,Maxval(Len_trim(axis_angles_S(1:3)))&
         &        ),t_int),10_t_int)+1_t_int

    Write(fmt,"('(''Lattice constants : '',a,',i3,'x,a,',i3,'x,a)')")&
         & i-Len_trim(lattice_constants_S(1_t_int:2_t_int))

    Write(outfile,fmt) &
         & Trim(lattice_constants_S(1))&
         &,Trim(lattice_constants_S(2))&
         &,Trim(lattice_constants_S(3))
          
    Write(fmt,"('(''Axis angles       : '',a,',i3,'x,a,',i3,'x,a)')")&
         & i-Len_trim(axis_angles_S(1_t_int:2_t_int))

    Write(outfile,fmt) &
         & Trim(axis_angles_S(1))&
         &,Trim(axis_angles_S(2))&
         &,Trim(axis_angles_S(3))
          

    
    
    
    !! Wyckoff positions
    
    Write(outfile,"(/,15x,'Wyckoff positions',/)") 
    Write(outfile,"('Number of Wyckoff positions : ',i4)") structure % IOnsort
    
    i=Max(Int(Maxval(Len_trim(IOwtau_S(:,1_t_int:structure % IOnsort))),t_int)&
         &,10_t_int)&
         &+1_t_int
    Write(fmt,"('(''No.   Element   X'',',i3,'x,''Y'',',i3,'x,''Z'')')")&
         &i-1_t_int,i-1_t_int
    Write(outfile,fmt)
    Do issaIO=1_t_int,structure % IOnsort
       Write(fmt,"('(i4,4x,a2,6x,a,',i3,'x,a,',i3,'x,a)')")&
            & i-Len_trim(IOwtau_S(1_t_int:2_t_int,issaIO))
       
       Write(outfile,fmt)issaIO,structure % p_IOelements(issaIO)&
            &,Trim(IOwtau_S(1_t_int,issaIO))&
            &,Trim(IOwtau_S(2_t_int,issaIO))&
            &,Trim(IOwtau_S(3_t_int,issaIO))
    End Do
    Write(outfile,"()")
    Write(outfile,"(72('-'))") 

    
    !! symmetry

    Write(outfile,"(72('-'))") 
    Write(outfile,"(28x,'SYMMETRY CREATION',/)")
    Write(outfile,"(72('-'))") 
    Open(12,file='+symmetry',status='old',iostat=eof)
    If(eof == 0_t_int )Then
       Do 
          Read(12,"(a)",iostat=eof)buffer
          If(eof < 0_t_int) Exit
          Write(outfile,"(a)")Trim(buffer)
       Enddo
       Close(12)
    Endif
    
    

    Write(outfile,"(72('-'))")
    Write(outfile,"(30x,'UNIT CELL CREATION',/)")
    Write(outfile,"(72('-'))")
    !! Wyckoff positions 
    Write(outfile,"(/,15x,'Wyckoff positions')") 
    
    If(symmetry % RTG)Then
       Write(outfile,"(/,'RTG as simple bravais lattice',/)") 
    End If
    
    Write(outfile,"('Number of Wyckoff positions : ',i4)") structure % IOnsort
    i=23_t_int
    Write(fmt,"('(''No.   Element    X'',',i3,'x,''Y'',',i3,'x,''Z'',',i3,'x,''Concentration'')')")i-1_t_int,i-1_t_int,i-1_t_int
    Write(outfile,fmt)
    Do issaIO=1_t_int,structure % IOnsort
       Write(outfile,"(i4,4x,a2,2x,3(f22.15,1x),f22.15)") issaIO&
            &,structure % p_IOelements(issaIO) &
            &,structure % p_IOwtau(:,issaIO),structure % p_IOconao(issaIO)
    End Do
    Write(outfile,"()")
    
    
    
    If(symmetry % RTG)Then
       Write(outfile,"(/,'RTG as nonsimple STG-bravais lattice',/)") 
       Write(outfile,"('Number of Wyckoff positions : ',i4)")structure % IOnsort
       i=23_t_int
       Write(fmt,"('(''No.   Element    X'',',i3,'x,''Y'',',i3,'x,''Z'',',i3,'x,''Concentration'')')")&
            & i-1_t_int&
            &,i-1_t_int&
            &,i-1_t_int
       Write(outfile,fmt)
       
       Do issaIO=1_t_int,structure % IOnsort
          Call cconvertvecrtgtostg(dum,structure % p_IOwtau(1_t_int,issaIO))
          Write(outfile,"(i4,4x,a2,2x,3(f22.15,1x),f22.15)")issaIO &
               &,structure % p_IOelements(issaIO) ,dum&
               &,structure % p_IOconao(issaIO)
       End Do
       
       
    End If !! RTG
    
    !! sites
    Write(outfile,"(/,17x,'Atom sites',/)") 
    Write(outfile,"('Number of sites : ',i4)") structure % nsite
    Write(outfile,"('Number of real sites : ',i4)") structure % naps
    Write(outfile,"('Number of nonempty real sites : ',i4)") &
         &structure % naps_a
    Write(outfile,"('No.  Element WPS CPA-Block    X                      Y                      Z')")
    Do isa=1_t_int,structure % nsite
       Write(outfile,"(i4,3x,a2,2x,i4,3x,i4,3x,3(f22.15,1x))") &
            & isa&
            &,structure % p_IOelements( structure % p_IOisrt(isa))&
            &,structure % p_IOisrt(isa),structure % p_kaps(isa)&
            &,structure % p_tau(:,isa)
    End Do
    
    Write(outfile,"(/,'Structure type: ',a,/)") &
         &Trim(STR_TYPES(structure% str_type))

    If(structure% str_type == STR_TYPE_CRYSTAL) Then
       Write(outfile,"(/,'Averaged WS radius : ',f10.5,' Bohr radii')") &
            &structure % arws
       Write(outfile,"('Vuc                : ',f30.15,' Bohr radii',/)") &
            &structure % vuc
    Endif


    Write(outfile,"(/,72('-'),/)")    
    Write(outfile,"(72('='))")     

  
    return
  End Subroutine print_crystal_structure

  ! }}}
  !!***********************************************************************
  Subroutine relsymreduce(symmetry,structure,mspin,quant_axis,relativistic&
       &,direction_invariant)
    !!
    Implicit None  
    !!IO
    Type(T_Symmetry),Intent(inout)  :: symmetry
    Type(T_Structure),Intent(inout) :: structure
    Logical(t_log),Intent(in)       :: direction_invariant
    Integer(t_int),Intent(in)       :: relativistic,mspin
    Real(t_real),Intent(in)         :: quant_axis(3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="relsymreduce"
    Logical(t_log) :: parallel,found,equiv
    Logical(t_log) :: debfull,rel,relmag
    Integer(t_int) :: isa,isb,isc,i,j,jops,kinv,issa,iops,kops,issaIO
    Integer(t_int) :: antiparallel,timerev
    Real(t_real)   :: v(3),d,s2(3),r(3),eu(3),fac(0:3),b(3,3),t(3),op(3,3)
    Real(t_real),Pointer :: Rwtau(:,:)
    Logical(t_log),Pointer :: lsite(:)
    !!
    rel=(relativistic == FULL_RELATIVISTIC)
    relmag=(rel .And. mspin==2)
    !!
!    If(.True. .Or. .Not. relmag)Then !! reduce never
    If(.Not. relmag)Then !! reduce only magnetic relativistic case
!!TODO
!    If(.Not. rel)Then !! reduce both relativistic cases 
       !!
       symmetry% reduced_sym=.False.
       If(symmetry% nugen>0) symmetry% reduced_sym=.True.    
       !!
       symmetry% Rnumops=symmetry% IOnumops
       symmetry% Ropind=symmetry% IOopind
       symmetry% Roptimerev=0_t_int
       symmetry% Ropmat=symmetry% IOopmat
       symmetry% Roptau=symmetry% IOoptau
       symmetry% Ropsymbol=symmetry% IOopsymbol
       !!
       structure% nsort=structure% IOnsort
       !!
       Call palloc('structure % p_equivsites',structure % p_equivsites&
            &,1_t_int,48_t_int&
            &,1_t_int,structure% nsite&
            &,nmodule,nsubfunc)

       Call palloc('structure % p_equivsitesrev',structure % p_equivsitesrev&
         &,1_t_int,48_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)


       Call palloc('structure % p_isrt',structure % p_isrt&
            &,1_t_int,structure% nsite ,nmodule,nsubfunc)
       Call palloc('structure % p_mult',structure % p_mult&
            &,1_t_int,structure% nsort ,nmodule,nsubfunc)

       !!
       Do isa=1,structure% nsite
          Do jops=1,symmetry% IOnumops
             structure % p_equivsites(jops,isa)=&
                  &structure % p_IOequivsites(jops,isa)
             structure % p_equivsitesrev(jops,isa)=&
                  &structure % p_IOequivsitesrev(jops,isa)
          End Do
          structure % p_isrt(isa)=structure % p_IOisrt(isa)
       Enddo

       !!

       Call palloc('structure % p_isrtTOIOisrt',structure % p_isrtTOIOisrt&
            &,1_t_int,structure% nsort &
            &,nmodule,nsubfunc)
       !!
       Call palloc('structure % p_elements',structure % p_elements&
            &,1_t_int,structure% nsort &
            &,nmodule,nsubfunc)
       !!
       Call palloc('structure % p_conao',structure % p_conao&
            &,1_t_int,structure% nsort&
            &,nmodule,nsubfunc)
       !!
       Do issa=1,structure% nsort
          issaIO=issa
          structure % p_isrtTOIOisrt(issa)=issaIO
          structure % p_mult(issa)=structure % p_IOmult(issaIO)
          structure % p_elements(issa)=structure % p_IOelements(issaIO)
          structure % p_conao(issa)=structure % p_IOconao(issaIO)
       Enddo
       !!
       !!
       Call rel_prep_wu_ks(symmetry,rel)
       !!
       Return
    End If
    !! 
    !!  relativistic treatment
    !! 
    symmetry% reduced_sym=.True.
    !!
    Nullify(lsite)
    Nullify(Rwtau)
    !!
    Call make_factorials(fac,3_t_int)
    !!
    debfull=debugfun(nmodule,nsubfunc,'')
    !!
    !!
    Write(outfile,1000)
    !!
    If(debfull)Then
       Write(outfile,200)
       oploop: Do jops=1,symmetry% IOnumops
          Write(outfile,1) symmetry% IOopsymbol(jops)
          !!
          Do i=1,3
             Write(outfile,3,advance='no') symmetry% IOopmat(1,i,jops)
             Do j=2,3
                Write(outfile,31,advance='no') symmetry% IOopmat(j,i,jops)
             Enddo
             If(i==1)Then
                Write(outfile,5) symmetry% IOoptau(1:3,jops)
             Else
                Write(outfile,4) 
             Endif
          End Do
          !!
          Do isa=1_t_int,structure% nsite
             isc=structure% p_IOequivsites(jops,isa) ! g isa
             Write(outfile,2) isa,isc
          End Do
       End Do oploop
    Endif
    !!
    !! reduce operations
    !!
    symmetry% Rnumops=0_t_int
    !!
    !!
    Write(outfile,201)
    oploop2: Do jops=1,symmetry% IOnumops
       v(:)=Matmul(quant_axis,symmetry% IOopmat(:,:,jops))
       d=Dot_product(v(:),quant_axis(:))&
            &/Sqrt(Dot_product(v(:),v(:)))&
            &/Sqrt(Dot_product(quant_axis(:),quant_axis(:)))
       parallel=(Abs(d-1.0_t_real) < 1.e-5_t_real)
       If((Abs(d+1.0_t_real) < 1.e-5_t_real))Then
          antiparallel=1_t_int
       Else
          antiparallel=0_t_int
       Endif
       If(parallel.Or.( (antiparallel==1).And..Not.direction_invariant))Then

          symmetry% Rnumops=symmetry% Rnumops+1_t_int
          symmetry% Ropmat(:,:,symmetry% Rnumops)=symmetry% IOopmat(:,:,jops)
          symmetry% Roptau(:,symmetry% Rnumops)=symmetry% IOoptau(:,jops)
          symmetry% Ropind(symmetry% Rnumops)=symmetry% IOopind(jops)
          symmetry% Ropsymbol(symmetry% Rnumops)=symmetry% IOopsymbol(jops)
          kinv=symmetry% opeulerinv(symmetry% IOopind(jops))
          timerev=Ieor(antiparallel,kinv)
          symmetry% Roptimerev(symmetry% Rnumops)=timerev

          If(symmetry% Roptimerev(symmetry% Rnumops) == 1)Then
             Write(outfile,81) symmetry% Rnumops,jops,symmetry% IOopsymbol(jops)
          Else
             Write(outfile,8) symmetry% Rnumops,jops,symmetry% IOopsymbol(jops)
          Endif
          Do i=1,3
             Do j=1,3
                Write(outfile,31,advance='no') symmetry% IOopmat(j,i,jops)
             Enddo
             If(i==1)Then
                Write(outfile,5) symmetry% IOoptau(1:3,jops)
             Else
                Write(outfile,4) 
             Endif
          End Do
          !!
          !! TODO: test symmetry% Ropmat=op(symmetry% Ropind(Ropgen))
          !!
          Write(outfile,6) quant_axis
          Write(outfile,7) v
          
       Endif
       
    End Do oploop2
    !!
    !! reduce sites, and sorts
    !!  g s+t =s'\R  ts=ts'
    !!
    Call palloc('structure % p_equivsites',structure % p_equivsites&
         &,1_t_int,48_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('structure % p_equivsitesrev',structure % p_equivsitesrev&
         &,1_t_int,48_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)


    !!
    !!
    If(debfull)Write(outfile,202)
    oploop3: Do jops=1,symmetry% Rnumops
       If(debfull)Write(outfile,51) symmetry% Ropsymbol(jops)
       !!
       Do i=1,3
          If(debfull)Write(outfile,53,advance='no') symmetry% IOopmat(1,i,jops)
          Do j=2,3
             If(debfull)Write(outfile,31,advance='no') symmetry% IOopmat(j,i,jops)
          Enddo
          If(i==1)Then
             If(debfull)Write(outfile,5) symmetry% IOoptau(1:3,jops)
          Else
             If(debfull)Write(outfile,4) 
          Endif
       End Do
       !!
       isaloop: Do isa=1_t_int,structure% nsite
          !!
          s2=Matmul(structure% p_tau(:,isa),symmetry% Ropmat(:,:,jops))&
               &+symmetry% Roptau(:,jops)
          
          isc=0_t_int
          isbloop: Do isb=1_t_int,structure% nsite
             
             v(:)=s2(:)-structure% p_tau(:,isb)
             
             !! v=GSa-Sb==R .and. ta==tb
             If(structure% str_type == STR_TYPE_MOLECULE)Then
                equiv=All(Abs(v(1:3))<etol)
             Else
                r(1)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
                r(2)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
                r(3)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
                equiv=vecIsOnGrid(3_t_int,r,etol)
             Endif
             
             If(equiv .And.(structure% p_IOisrt(isa) == &
                  & structure% p_IOisrt(isb))) Then
                isc=isb
                Exit isbloop
             Endif
          Enddo isbloop
          !!
          If(isc ==  0_t_int) &
               & Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Symmetry is incorrect!'")
          !!
          structure% p_equivsites(jops,isa)=isc
          structure% p_equivsitesrev(jops,isc)=isa

          kinv=symmetry% opeulerinv(symmetry% Ropind(jops))
          eu=symmetry% opeuler(:,symmetry% Ropind(jops))
          
          !!
          Call ytr(eu,kinv,fac,1_t_int,b,.False.)
          Call paranoia(symmetry% Ropmat(:,:,jops),b,jops,1.e-14_t_real)
          !!
          If(debfull)Write(outfile,52) isa,isc
          !!
       End Do isaloop
    End Do oploop3
    !!
    !! 
    !! test group property of reduced operations
    ioploop: Do iops=1,symmetry% Rnumops
       joploop: Do jops=1,symmetry% Rnumops
          !! gs=so+t
          !! gi*gj*s=s*oj*oi+tj*oi+ti={oj*oi|tj*oi+ti}
          op(:,:)=Matmul(symmetry% Ropmat(:,:,jops),symmetry% Ropmat(:,:,iops))
          t(:)=Matmul(symmetry% Roptau(:,jops),symmetry% Ropmat(:,:,iops))&
               &+symmetry% Roptau(:,iops)
          !!
          found=.False.
          koploop: Do kops=1,symmetry% Rnumops

             v(:)=t(:)-symmetry% Roptau(:,kops)
             If(structure% str_type == STR_TYPE_MOLECULE)Then
                equiv=All(Abs(v(1:3))<etol)
             Else
                r(1)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
                r(2)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
                r(3)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
                equiv=vecIsOnGrid(3_t_int,r,etol)
             Endif

             If(All(Abs(op(:,:)-symmetry% Ropmat(:,:,kops))<1.0e-4_t_real)&
                  &.And. equiv)Then
                found=.True.
                If(debfull)Then
                   Write(outfile,35) Trim(symmetry% Ropsymbol(iops))&
                        &,Trim(symmetry% Ropsymbol(jops))&
                        &,Trim(symmetry% Ropsymbol(kops))
                Endif

                Exit koploop
             Endif
          End Do koploop
          If(.Not. found)Then
             Write(outfile,34) symmetry% Ropsymbol(iops)&
                  &,symmetry% Ropsymbol(jops)
             Call terminate(TERM_CRASH,'Reduced operations do not form a group ?!')
          Endif
          !!
       End Do joploop
    End Do ioploop
    !!
    !!
    !! define reduced sorts
    !!
    Call palloc('structure % p_isrt',structure % p_isrt&
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)
    !!
    structure% nsort=0_t_int
    structure% p_isrt(:)=0_t_int
    !!
    Call palloc('Rwtau',Rwtau,1_t_int,3_t_int,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)

    Write(outfile,203)
    Do isa=1,structure% nsite
       Write(outfile,9,advance='no') isa
       If(structure% p_isrt(isa)==0_t_int)Then
          structure% nsort=structure% nsort+1_t_int
          structure% p_isrt(isa)=structure% nsort
          Rwtau(:,structure% nsort)=structure% p_tau(:,isa)
       Endif
       Do jops=1,symmetry% Rnumops
          isc=structure% p_equivsites(jops,isa)
          structure% p_isrt(isc)=structure% p_isrt(isa)
          Write(outfile,10,advance='no') isc
       Enddo
       Write(outfile,4)
    Enddo
    !!
    !!
    Call palloc('structure % p_isrtTOIOisrt',structure % p_isrtTOIOisrt&
         &,1_t_int,structure% nsort &
         &,nmodule,nsubfunc)

    Call palloc('structure % p_elements',structure % p_elements&
         &,1_t_int,structure% nsort &
         &,nmodule,nsubfunc)
    !!
    Call palloc('structure % p_conao',structure % p_conao&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)
    !!
    !!
    structure% p_isrtTOIOisrt(:)=0_t_int
    Do isa=1,structure% nsite
       issaIO=structure% p_IOisrt(isa)
       issa=structure% p_isrt(isa)
       structure% p_isrtTOIOisrt(issa)=issaIO
    Enddo
    !!
    Do issa=1,structure% nsort
       issaIO=structure% p_isrtTOIOisrt(issa)
       structure% p_elements(issa)=structure% p_IOelements(issaIO)
       structure% p_conao(issa)=structure% p_IOconao(issaIO)
    Enddo
    

    !! check: mapping of rel sorts onto nonrel sorts 
    !! 
    Do isa=1,structure% nsite
       issaIO=structure% p_IOisrt(isa)
       issa=structure% p_isrt(isa)
       If(structure% p_isrtTOIOisrt(issa) /= issaIO)Then
          Call terminate(TERM_CRASH,' relativistic onto non relativistic '&
               &//'sort mapping failed !?')
       Endif
       If(Trim(structure% p_IOelements(issaIO)) /= &
            & Trim(structure% p_elements(issa)))Then
          Call terminate(TERM_CRASH,' relativistic onto non relativistic '&
               &//'sort mapping failed !?')          
       Endif
    End Do
    !!
    !! check if some isrt ==0, map rel sorts onto nonrel sorts 
    Do issa=1,structure% nsort
       If(structure% p_isrt(issa) == 0_t_int)Then
          Call terminate(TERM_CRASH,' relativistic sort mapping failed !?')
       Endif
       If(structure% p_isrtTOIOisrt(issa) == 0_t_int)Then
          Call terminate(TERM_CRASH,' relativistic sort mapping failed !?')
       Endif
    Enddo


    !! test group property and completeness of reduced sites/sorts
    !!
    Call palloc('lsite',lsite,1_t_int,structure% nsite,nmodule,nsubfunc)
    !!
    lsite(:)=.False.
    !!
    roploop: Do jops=1,symmetry% Rnumops
       rsortloop: Do issa=1,structure% nsort
          !!
          s2=Matmul(Rwtau(:,issa),symmetry% Ropmat(:,:,jops))&
               &+symmetry% Roptau(:,jops)
          !!
          found=.False.
          siteloop: Do isa=1,structure% nsite
             v(:)=s2(:)-structure% p_tau(:,isa)
             If(structure% str_type == STR_TYPE_MOLECULE)Then
                equiv=All(Abs(v(1:3))<etol)
             Else
                r(1)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
                r(2)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
                r(3)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
                equiv=vecIsOnGrid(3_t_int,r,etol)
             Endif
             
             If(equiv .And.(structure% p_isrt(isa) == issa)) Then
                found=.True. !! s2 found in sites
                lsite(isa)=.True. !! remember that isa was found
                Exit siteloop
             Endif
          End Do siteloop
          If(.Not. found)Then
             Call terminate(TERM_CRASH,'Sites are not consistent with '&
                  &//'reduced summetry ?!')
          Endif
          !!
       Enddo rsortloop
    Enddo roploop
    !! all sites reproduced from reduced Wyckoff positions?
    Do isa=1,structure% nsite
       If(.Not.lsite(isa))Then
          Call terminate(TERM_CRASH,'Sites are not consistent with '&
               &//'reduced summetry ?!')
       Endif
    Enddo
    !!
    Call pfree(lsite)
    Call pfree(Rwtau)
    !!
    Call palloc('structure % p_mult',structure % p_mult&
         &,1_t_int,structure% nsort ,nmodule,nsubfunc)
    !!
    Do issa=1,structure% nsort
       structure% p_mult(issa)=0_t_int
       Do isa=1,structure% nsite
          If(structure% p_isrt(isa) == issa )Then
             structure% p_mult(issa)=structure% p_mult(issa)+1_t_int
          Endif
       Enddo
    Enddo
    !! some output
    !!
    Write(outfile,111) 
    Do isa=1,structure% nsite
       issaIO=structure% p_IOisrt(isa)
       issa=structure% p_isrt(isa)
       Write(outfile,11) isa,structure% p_IOelements(issaIO),issaIO&
            &,structure% p_IOmult(issaIO)&
            &,structure% p_elements(issa),issa,structure% p_mult(issa)
    End Do
    Write(outfile,112) 
    !!
    Do issa=1,structure% nsort
       issaIO=structure% p_isrtTOIOisrt(issa)
       Write(outfile,12) structure% p_elements(issa),issa&
            &,structure% p_IOelements(issaIO),issaIO
    End Do
    !!
    Call rel_prep_wu_ks(symmetry,.True.)
    !!
1   Format('NONREL: ',a15) 
51  Format('REL   : ',a15) 
8   Format('n_rel=',i2,10x,' n_nonrel=',i2,' : ',a15) 
81  Format('n_rel=',i2,10x,' n_nonrel=',i2,' : ',a15,' time reversal') 
2   Format('NONREL: ',10x,'isa= ',i4,' G isa= ',i4) 
52  Format('REL   : ',10x,'isa= ',i4,' G isa= ',i4) 
3   Format('NONREL: ',f10.4,' ')
53  Format('REL   : ',f10.4,' ')
31  Format(f10.4,' ')
4   Format() 
5   Format(' : ',3(f10.4,1x))
6   Format(10x,'axis   : ',3(f10.4,1x))
7   Format(10x,'G axis : ',3(f10.4,1x))
9   Format('site: ',i4,' equivalent sites : ')
10  Format(1x,i4)
111 Format('|',50('-'),'|'&
         &,/,'|',t4,'     RELATIVISTIC SYMMETRY REDUCTION',t52,'|'&
         &,/,'|',50('-'),'|'&
         &,/,'|',t4,'site   El sort  mult       El sort mult',t52,'|'&
         &,/&
         &,'|',t4,'       non-rel             relativistic',t52,'|'&
         &,/,'|',50('-'),'|'&
         &)
112 Format('|',50('-'),'|')
11  Format('|',t4,i4,3x,a2,'(',i4,') ',i4,'  ->   ',a2,'(',i4,') ',i4,t52,'|')
12  Format(a2,'(',i4,') -> old sort ',a2,'(',i4,')')
35  Format(a22,' * ',a22,' = ',a22) 
34  Format(a,'*',a,' does not give an operation of the reduced set!') 
200 Format('NONREL: ',' Symmetry operations') 
201 Format('   Symmetry Reduction') 
202 Format('REL   : ',' Reduced symmetry operations') 
203 Format(' Reduced site classes ') 
1000 Format(72('='),/,'START:',t20,'RELATIVISTIC SYMMETRY REDUCTION') 
1001 Format('END:',t20,'RELATIVISTIC SYMMETRY REDUCTION',/,72('=')) 
    !!
    Write(outfile,1001)
    !!
    !!
    Return
    !!
  End Subroutine relsymreduce
  
  !!***********************************************************************
  !!***********************************************************************
  !! 
  !! the program make lattice direct or reciprocal space vectors
  !!
  Subroutine make_lattice(lattice_vectors,Gmax,nlattr,p_rlat)
    Implicit None
    !!IO
    Real(t_real),Pointer      :: p_rlat(:,:)
    Integer(t_int),Intent(out) :: nlattr
    Real(t_real),Intent(in)    :: Gmax
    Real(t_real),Intent(in)   :: lattice_vectors(3,3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="make_lattice"
    Logical(t_log) :: check
    Integer      :: icheck
    Integer(t_int):: ni,nj,nk,m,maxinvshl,Nijk(3),novshl
    Real(t_real) :: v(3),vv,tol,vmx
    Integer(t_int),Allocatable :: iid(:)
    Integer(t_int) :: i,j,k,l
    Real(t_real)  :: rec(3,3),det,vm
    Integer(t_int),Pointer ::  p_ninvshl(:)
    Real(t_real),Pointer      :: p_vabs(:)
    Nullify(p_ninvshl,p_vabs)
    
    tol=1.e-8_t_real
    check=isoption('PROT_MAKELATTICE')
    !!
    If(Abs(gmax)<1.0e-6_t_real)Then
       nlattr=1
       Call palloc('p_rlat',p_rlat,1_t_int,3_t_int,1_t_int,nlattr &
            &,nmodule,nsubfunc)
       p_rlat(:,1)=0.0_t_real
       Return
    Endif
    !!
    Call dinv33(lattice_vectors,1_t_int,rec,det)
    
    Do i=1,3
       Nijk(i)=Int(Gmax/(Dot_product(lattice_vectors(:,i),rec(:,i))&
            &/Sqrt(Dot_product(rec(:,i),rec(:,i)))),t_int)+2_t_int
       Write(outfile,200)Trim(nsubfunc),i,Nijk(i)
200    Format(a,': N',i1,'',i10)
    Enddo
    !!
    ni=Nijk(1)
    nj=Nijk(2)
    nk=Nijk(3)
    !!
    vmx=Gmax**2
    !!
    vm=0.0_t_real
    nlattr=0_t_int
    Do i=-ni,ni
       Do j=-nj,nj
          Do k=-nk,nk

             v(:)=  Real(i,t_real)*lattice_vectors(:,1)&
                  &+Real(j,t_real)*lattice_vectors(:,2)&
                  &+Real(k,t_real)*lattice_vectors(:,3)
             
             !!
             vv=Dot_product(v,v)
             !!
             !Print *,'nlattr=',Sqrt(vv),nlattr
             If(vv <= vmx) Then
                vm=Max(vm,vv)
                !Print *,'vm=',Sqrt(vm),Sqrt(vv),nlattr
                nlattr=nlattr+1_t_int
             Endif
          Enddo
       Enddo
    Enddo
    !!
    ! avoid non coincidence of final dimension m  and nlattr via precision loss
    vmx=vm+tol
    
    !! end condition
    !! ***********************




    Call palloc('p_rlat',p_rlat&
         &,1_t_int,3_t_int &
         &,1_t_int,nlattr &
         &,nmodule,nsubfunc)

    Call palloc('p_ninvshl',p_ninvshl&
         &,1_t_int,nlattr &
         &,nmodule,nsubfunc)

    Call palloc('p_vabs',p_vabs&
         &,1_t_int,nlattr &
         &,nmodule,nsubfunc)


    ! tmporary
    Allocate(iid(nlattr),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for iid !'")
    Endif
    
    
    l=0_t_int
    m=0_t_int
    Do i=-ni,ni
       Do j=-nj,nj
          Do k=-nk,nk
             v(:)=  Real(i,t_real)*lattice_vectors(:,1_t_int)&
                  &+Real(j,t_real)*lattice_vectors(:,2_t_int)&
                  &+Real(k,t_real)*lattice_vectors(:,3_t_int)

             vv=Dot_product(v,v)
             l=l+1_t_int
             If(vv <= vmx) Then
                m=m+1_t_int
                iid(m)=l
                p_vabs(m)=Sqrt(vv)
                p_rlat(:,m)=v
             Endif
          Enddo
       Enddo
    Enddo


    Call vshell(iid,p_ninvshl,novshl,nlattr,p_vabs,p_rlat,maxinvshl)
    If(Allocated(iid))Deallocate(iid)


    i=0_t_int
    Do k=1_t_int,novshl
       i=i+p_ninvshl(k)
       p_vabs(k)=Sqrt(Sum(p_rlat(:,i)**2_t_int))
    Enddo
    
    

    If (check) Then
       Write(outfile,100) 
    Endif
    i=0_t_int
    Do k=1_t_int,novshl
       j=p_ninvshl(k)
       If (check) Then
          Write(outfile,101) k,j
          Write(outfile,102) 
       Endif
       Do l=1_t_int,j
          v=p_rlat(:,i+l)
          vv=Sqrt(Sum(v**2_t_int))
          If (check) Then
             Write(outfile,103) l,v,vv
          Endif
          If(Abs(vv-p_vabs(k))>1.0e-9_t_real) Then
             !Write(outfile,104) l,v,vv,p_vabs(k)
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Corrupted shells, please report bug !'")
          Endif
       Enddo                 ! l
       i=i+j
    Enddo                   ! k
    !!
    Call pfree(p_ninvshl)
    Call pfree(p_vabs)
    Return
    !!
100 Format('make_lattice: check output of vector shells ')
101 Format(' no of shell ',i5,'  no in shell ',i5)
102 Format('no        vx              vy                vz  ', &
         & '             |v| ')
103 Format(i2,1x,4(f15.6,2x))
!104 Format(i2,1x,3(f15.6,2x),2(e30.15e3,1x))
    !!
  End Subroutine make_lattice

  !!***********************************************************************
  !**********************************************************************
  !     find index array ind such that a(ind(n+1)) >= a(ind(n))
  !     see w. press et al, numerical recipes, cambridge 1986
  !**********************************************************************  
#ifdef my_compile_unused
  Subroutine vindex(n,a,ind)
    !**********************************************************************
    !
    Implicit None 
    !!IO
    Integer(t_int),Intent(in):: n
    Integer(t_int),Intent(out):: ind(n)
    Real(t_real),Intent(in) :: a(n)
    !!LOCAL
    Integer(t_int) :: i,j,l,ir,indxt
    Real(t_real)  :: q
    !**********************************************************************
    !
    Do i=1_t_int,n
       ind(i)=i
    End Do
    
    l=n/2_t_int+1_t_int
    ir=n
    Do 
       If(l > 1_t_int) Then
          l=l-1_t_int
          indxt=ind(l)
          q=a(indxt)
       Else
          indxt=ind(ir)
          q=a(indxt)
          ind(ir)=ind(1_t_int)
          ir=ir-1_t_int
          If(ir <=  1_t_int) Then
             ind(1_t_int)=indxt
             Return
          Endif
       Endif
       
       i=l
       j=l+l
       
       
       Do While (j <= ir) 
          If(j < ir) Then
             If(a(ind(j)) < a(ind(j+1_t_int))) j=j+1_t_int
          Endif
          If(q < a(ind(j))) Then
             ind(i)=ind(j)
             i=j
             j=j+j
          Else
             j=ir+1_t_int
          Endif
       End Do
       
       ind(i)=indxt
    End Do
    
  End Subroutine vindex
#endif
  !!***********************************************************************
  !! the program sorts lattice vectors on shells
  Subroutine vshell(iid,ninvshl,novshl,nlattr,vabs,vlatt,maxinvshl)
    ! nlattr : number of vectors
    ! vabs   : lengths of vectors 
    ! ninvshl: number of vectors in shell
    ! novshl : number of shells
    ! ..................................................................
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: nlattr
    Integer(t_int),Intent(inout) :: iid(nlattr)
    Integer(t_int),Intent(out)   :: novshl,ninvshl(nlattr),maxinvshl
    Real(t_real),Intent(inout)  :: vabs(nlattr),vlatt(3,nlattr)
    !!LOCAL
    Character(*),Parameter::nsubfunc="vshell"
    Integer(t_int) :: i,j,m,k
    Real(t_real) :: a,v
    !
    

    ! first find |v|=0 vector
    Do i=1_t_int,nlattr
       v= Sum(vlatt(:,i)**2_t_int)
       If(v < 1.e-5_t_real) Then 
          
          m=iid(1_t_int)
          iid(1_t_int)=iid(i)
          iid(i)=m
          a=vabs(1_t_int)
          vabs(1_t_int)=0.0_t_real
          vabs(i)=a
          Do j=1_t_int,3_t_int
             a=vlatt(j,1_t_int)
             vlatt(j,1_t_int)=0.0_t_real
             vlatt(j,i)=a
          Enddo
          Exit
       Endif
    Enddo

    
    ! - find for each v  a (-v)
    
    Do i=2_t_int,nlattr-1_t_int,2_t_int
       kloop: Do k=2_t_int,nlattr
          
          v=Sum((vlatt(:,i)+vlatt(:,k))**2_t_int)
          If(v < 1.e-5_t_real) Then
             
             m=iid(i+1_t_int)
             iid(i+1_t_int)=iid(k)
             iid(k)=m
             a=vabs(i+1_t_int)
             vabs(i+1_t_int)=vabs(k)
             vabs(k)=a
             Do j=1_t_int,3_t_int
                a=vlatt(j,i+1_t_int)
                vlatt(j,i+1_t_int)=vlatt(j,k)
                vlatt(j,k)=a
             Enddo
             Exit kloop
          Endif
       Enddo kloop
    Enddo
    !check

    If (Abs(vabs(1_t_int)) > 1.e-5_t_real) Then
       Write(outfile,*) vabs(1_t_int),(vlatt(j,1_t_int),j=1_t_int,3_t_int)
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="' VSHELL: vabs(1) <> 0.0 '")
    Endif
    Do i=2_t_int,nlattr-1_t_int,2_t_int
       If (Abs(vabs(i)-vabs(i+1_t_int)) > 1.e-5_t_real) Then
          Write(outfile,*) i,vabs(i),(vlatt(j,i),j=1_t_int,3_t_int)
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="' VSHELL: no negative r-vector '")
       Endif
    Enddo
    !     -   sort vectors,inverse vector pairs with respect to length
    Do i=2_t_int,nlattr-1_t_int,2_t_int
       v=1.0e30_t_real
       m=i
       Do k=i,nlattr-1_t_int,2_t_int
          v=Min(v,vabs(k))
          !If(v == vabs(k)) m=k
          If(Abs(v-vabs(k))<1.0e-11_t_real) m=k
       Enddo
       !Print *,'sort: ',i,m,vabs(i),vabs(m)
       a=vabs(i)
       vabs(i)=vabs(m)
       vabs(m)=a
       
       a=vabs(i+1_t_int)
       vabs(i+1_t_int)=vabs(m+1_t_int)
       vabs(m+1_t_int)=a
       
       k=iid(i)
       iid(i)=iid(m)
       iid(m)=k
       
       k=iid(i+1_t_int)
       iid(i+1_t_int)=iid(m+1_t_int)
       iid(m+1_t_int)=k
       
       Do j=1_t_int,3_t_int
          a=vlatt(j,i)
          vlatt(j,i)=vlatt(j,m)
          vlatt(j,m)=a
       Enddo
       
       Do j=1_t_int,3_t_int
          a=vlatt(j,i+1_t_int)
          vlatt(j,i+1_t_int)=vlatt(j,m+1_t_int)
          vlatt(j,m+1_t_int)=a
       Enddo
    Enddo

!!NOTUSED    open(15)
!!NOTUSED    Do i=1_t_int,nlattr
!!NOTUSED       Write(15,"(i6,1x,4(e30.15e3,2x))")i,vlatt(1_t_int,i),vlatt(2_t_int,i),vlatt(3_t_int,i)&
!!NOTUSED            &,vabs(i)
!!NOTUSED    Enddo
!!NOTUSED    Close(15)    
    ! classify vector into shells
    novshl=1_t_int
    ninvshl(1_t_int)=1_t_int
    maxinvshl=1_t_int
    If(nlattr>1)Then
       i=2_t_int
       novshl=2_t_int
       loop30: Do
          k=i+1_t_int
          If(k > nlattr) Exit loop30
          ninvshl(novshl)=1_t_int
          Do While(Abs(vabs(k)-vabs(k-1_t_int)) <= 1.e-10_t_real) 
             ninvshl(novshl)=ninvshl(novshl)+1_t_int
             maxinvshl=Max(ninvshl(novshl),maxinvshl)
             k=k+1_t_int
             If(k > nlattr) Exit loop30
          End Do
          novshl=novshl+1_t_int
          i=k
       Enddo loop30
    Endif
    !!
    Return
    !!
  End Subroutine vshell
  !!***********************************************************************
  !***********************************************************************
  Subroutine def_cpa_indexarrays(sym,str,invprt,relativistic)
    !
    !***********************************************************************
    !
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(inout)   :: sym
    Type(T_Structure),Intent(inout)  :: str
    Integer(t_int),Intent(in)    :: relativistic
    Logical(t_log),Intent(out)  :: invprt
    !!LOCAL
    Character(*),Parameter::nsubfunc="def_cpa_indexarrays"
    Integer(t_int) :: isa,isb,iq,issa,k,im,isap,isa0,j,i,issb,issbIO
    Real(t_real) :: dr,s,BG(3,3),det,t(3),w(3)
    Real(t_real),Parameter :: epsshl=1.e-4_t_real
    Real(t_real),Parameter :: epscon=1.e-6_t_real
    Integer(t_int),Pointer   :: posToSort(:,:)
    Logical(t_log) :: gotit
    !***********************************************************************
    !

    Call palloc('str% p_posToSiteRep',str % p_posToSiteRep&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_siteToPos',str % p_siteToPos&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_qposToSite',str % p_qposToSite&
         &,1_t_int,str% nsite&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_nqAtPos',str % p_nqAtPos&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_isst',str % p_isst&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_icis',str % p_icis&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    Call palloc('str% p_icisr',str % p_icisr&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)

    !
    invprt=.False.
    Do isa=1_t_int,str % nsite
       j=0_t_int
       Do isb=1_t_int,str % nsite
          If(str % p_kaps(isa) ==  str % p_kaps(isb)) j=j+1_t_int
       Enddo
       invprt=invprt.Or.(j == 1_t_int)
    Enddo
    invprt=invprt.And.(str % naps < str % nsite)
    
    !
    str % p_posToSiteRep=str % p_kaps

    !!
    Call ipiksrt1(str % p_posToSiteRep,str % nsite)
    ! TODO: check the str % naps== naps below
    Call iuniq1(str % p_posToSiteRep,str % nsite,str % Naps)

    !
    ! check wether kaps == kaps if |S-S|=0 and kaps /= kasp if |S-S| /= 0
    !
    Do isa=1_t_int,str % nsite
       Do isb=isa,str % nsite
          dr=Sqrt(Sum((str % p_tau(:,isa)-str % p_tau(:,isb))**2_t_int))
          If (str % p_kaps(isa) == str % p_kaps(isb) &
               &.And. dr > epsshl) Then
101          Format('''Inconsistency in p_kaps, atoms ',i4,' and ',i4, &
                  & ' do not have the same tau!'',/,''kaps(',i4,')=',i4, &
                  & ' kaps(',i4,')=',i4,'''')
             Write(errbuf,101)isa,isb,isa&
                  &,str % p_kaps(isa),isb,str % p_kaps(isb)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          If (str % p_kaps(isa) /= str % p_kaps(isb) &
               &.And.dr < epsshl) Then
             Write(outfile,102)nsubfunc,isa,isb,isa,str % p_kaps(isa)&
                  &,isb,str % p_kaps(isb)
             Call terminate(TERM_CRASH)
          Endif
       Enddo
    Enddo
    !

    Nullify(posToSort)
    Call palloc('posToSort',posToSort&
         &,1_t_int,str% nsite&
         &,1_t_int,str% nsite&
         &,nmodule,nsubfunc)
    !

    Do isap=1_t_int,str % naps
       iq=0_t_int
       isa=str % p_posToSiteRep(isap)
       Do isb=1_t_int,str % nsite
          If(str % p_kaps(isb) == isa) Then
             str % p_siteToPos(isb)=isap
             iq=iq+1_t_int
             posToSort(isap,iq)=str % p_isrt(isb)
             str % p_nqAtPos(isap)=iq
          Endif
       Enddo
    Enddo
    !
    str% maxq=0_t_int
    Do isap=1_t_int,str % naps
       str% maxq=Max(str% maxq,str % p_nqAtPos(isap))
       Do iq=1_t_int,str % p_nqAtPos(isap)
          im=0_t_int
          Do isb=1_t_int,str % nsite
             If (str % p_isrt(isb) == posToSort(isap,iq) .And. &
                  &str % p_kaps(isb) == str % p_posToSiteRep(isap)) Then
                im=im+1_t_int
                str % p_qposToSite(isap,iq)=isb
                If (im > 1_t_int) Then 
                   str % p_qposToSite(isap,iq)=0_t_int 
                   Write(outfile,104)nsubfunc,isb,str % p_posToSiteRep(isap)&
                        &,posToSort(isap,iq)
                   Call terminate(TERM_CRASH)
                Endif
             Endif
          End Do
          If (im == 0_t_int) Then
             str % p_qposToSite(isap,iq)=0_t_int 
             Call terminate(TERM_CRASH,Trim(nsubfunc)//'im ==0')
          Endif
       Enddo
    Enddo
    !
    Write(outfile,99)str % Naps,str % maxq
    Do isap=1_t_int,str % naps
       Write(outfile,"(i4,9x,3(f8.4),5x,i4,5x,' isrt : ')"&
            &,advance='no')str % p_posToSiteRep(isap)&
            &,str % p_tau(1,str % p_posToSiteRep(isap))&
            &,str % p_tau(2,str % p_posToSiteRep(isap))&
            &,str % p_tau(3,str % p_posToSiteRep(isap))&
            & ,str % p_nqAtPos(isap)
       Do iq=1_t_int,str % p_nqAtPos(isap)
          Write(outfile,"(i4,1x)",advance='no')posToSort(isap,iq)
       Enddo
       Write(outfile,"()")
       Write(outfile,"(52x,'isa  : ')",advance='no')
       Do iq=1_t_int,str % p_nqAtPos(isap)
          Write(outfile,"(i4,1x)",advance='no')str % p_qposToSite(isap,iq)
       Enddo
       Write(outfile,"()")
    Enddo
    Write(outfile,"(72('-')/)")
    !
    Do isap=1_t_int,str % naps
       Do iq=1_t_int,str % p_nqAtPos(isap)
          k=str % p_qposToSite(isap,iq)
          If(str % p_siteToPos(k) /= isap) Then
             Write(outfile,300)nsubfunc,isap,iq,str % p_siteToPos(k),k
             call terminate(TERM_CRASH)
          Endif
       Enddo
    Enddo
    !!
    !isst maps isrt onto isa
    !!
    If(relativistic /= FULL_RELATIVISTIC) Then
       !! this is not suited for full relativistic stuff
       !!
       !! when using forces, it might be desirable to
       !! use the sites, which are actually equivalent to wyckoffs as
       !! wyckoff representatives
       
       If(sym% RTG .Or. str% str_type == STR_TYPE_MOLECULE) Then
          
          BG=0.0_t_real
          Do i=1,3
             BG(i,i)=1.0_t_real
          Enddo
       Else
          Call dinv33(Transpose(sym% rcentering/12.0_t_real),1_t_int,BG,det)
       Endif
       !!
       !!
       Do issb=1,str% nsort
          issbIO=str% p_isrtTOIOisrt(issb)
          gotit=.False.
          Do isa=1,str% nsite
             If(str% p_isrt(isa) /= issb)Cycle
             
             If(str% str_type == STR_TYPE_MOLECULE) Then
                t=str% p_tau(:,isa)
                w=str% p_IOwtau(:,issbIO)
                If(All(Abs(t-w)<1.0e-4_t_real))Then
                   !Write(outfile,"('Wyck(',a,')=',3(f17.5))")&
                   !     & Trim(intToChar10(issb))&
                   !     &,str% p_IOwtau(:,issbIO)
                   !Write(outfile,"('Site(',a,')=',3(f17.5))")&
                   !     & Trim(intToChar10(isa)),str% p_tau(:,isa)
                   gotit=.True.
                   str% p_isst(issb)=isa
                Endif
             Else
                t=Matmul(str% p_tau(:,isa)&
                     &,sym% reciprocial_lattice_vectors(:,:))/twopi
                w=Matmul(str% p_IOwtau(:,issbIO),BG)
                !Write(*,"(' W',i3,'=',3(f17.5))")issb,str% p_IOwtau(:&
                !     &,issbIO)
                !Write(*,"('xW',i3,'=',3(f17.5))")issb,w
                
                !Write(*,"(' S',i3,'=',3(f17.5))")isa,str% p_tau(:&
                !     &,isa)
                !Write(*,"('xS',i3,'=',3(f17.5))")isa,t
                
                If(vecIsOnGrid(3_t_int,t-w,1.0e-4_t_real))Then
                   !Write(outfile,"('Wyck(',a,')=',3(f17.5))")&
                   !     & Trim(intToChar10(issb))&
                   !     &,str% p_IOwtau(:,issbIO)
                   !Write(outfile,"('Site(',a,')=',3(f17.5))")&
                   !     & Trim(intToChar10(isa)),str% p_tau(:,isa)
                   gotit=.True.
                   str% p_isst(issb)=isa
                Endif
             Endif
          Enddo
          If(.Not.gotit)Then
             Write(outfile,"('Wyck(',a,')=',3(f17.5))")&
                  & Trim(intToChar10(issb))&
                  &,str% p_IOwtau(:,issbIO)
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Could not find Wyckoff position corresponding&
                  & to site!'")
          Endif
       Enddo
    Else !! FULL_RELATIVISTIC    
       !! TODO: not fit for forces !!
       Do issa=1_t_int,str % nsort
          Do isa=1_t_int,str % nsite
             k=str % p_isrt(isa)
             If(k == issa)Then
                str % p_isst(issa)=isa
             Endif
          Enddo
       Enddo
    Endif
    !
    Write(outfile,402)
    Do issa=1_t_int,str % nsort
       isa=str % p_isst(issa)
       If(issa /= str % p_isrt(isa))Then
          Write(outfile,400)nsubfunc,issa,isa,str % p_isrt(isa)
          Call terminate(TERM_CRASH)
       Endif
       If(isa /= str % p_isst(issa))Then
          Write(outfile,404)nsubfunc,issa,isa,str % p_isst(issa)
          Call terminate(TERM_CRASH)
       Endif
       Write(outfile,401)issa,isa
    Enddo
    Write(outfile,403)
    !maps isort onto real sites
    !     used in vatcr
    !     ncis=number of crystllographic inequivalent sites, supposing no CPA
    !     icis(1..ncis) representant of the above sites 
    !     icisr(1..nsort) maps isort onto representant
    str % ncis=0_t_int
    Do issa=1_t_int,str % nsort

       isa=str % p_isst(issa)
       isap=str % p_siteToPos(isa)
       k=posToSort(isap,1_t_int)
       If(k == issa)Then
          str % ncis=str % ncis+1_t_int
          str % p_icis(str % ncis)=issa
          Do iq=1_t_int,str % p_nqAtPos(isap)
             str % p_icisr(posToSort(isap,iq))=issa
          Enddo                 !iq
       Endif
    Enddo                     ! issa
    
    Write(outfile,501) str % ncis
    Do j=1_t_int,str % ncis
       Write(outfile,500) j,str % p_icis(j)
    Enddo
    Write(outfile,502) str % nsort
    Do issa=1_t_int,str % nsort
       Write(outfile,503) issa,str % p_icisr(issa)
    Enddo
    Write(outfile,403)
    
    !
    Do isap=1_t_int,str % naps
       s=0.0_t_real
       Do isa=1_t_int,str % nsite
          If (str % p_kaps(isa) == str % p_posToSiteRep(isap)) &
               & s=s+str % p_conao(str % p_isrt(isa))
          If (str % p_conao(str % p_isrt(isa)) <= 0.0_t_real) Then
             Write(errbuf,"('''Concentration of sort ',i4&
                  &,' <=  zero  : ',e12.3e2,'''')") str % p_isrt(isa)&
                  &,str % p_conao(str % p_isrt(isa))
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          If (str % p_conao(str % p_isrt(isa)) > 1.0_t_real) Then
             Write(errbuf,"('''Concentration of isrt ',i4&
                  &,' >= 1 : ',e12.3e2,'''')") str % p_isrt(isa)&
                  &,str % p_conao(str % p_isrt(isa))
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
       Enddo
       If (Abs(s-1.0_t_real) > epscon) Then
          Write(errbuf,"('''More than atom sit at site : ',i4,'! &
               &Check Wyckoff positions.''')")str % p_posToSiteRep(isap)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Write(errbuf,"('''Concentrations at site : ',i4,' do not add&
               & to unity !''')")str % p_posToSiteRep(isap)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
    Enddo
    !
    !! check block structure 
    isa0=0_t_int
    Do isap=1,str% naps
       Do iq=1,str% p_nqAtPos(isap)
          isa=str% p_qposToSite(isap,iq)
          If(isa /= isa0+1_t_int)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'corrupted block structure?'")
          Endif
          isa0=isa
       End Do !iq
    End Do
    !!
    Call pfree(posToSort)
    !!
    Return
    !!    
99  Format(72('-')/&
         &,'Number of diferent sites (Naps) : ',i5/ &
         &,'maximum atoms at one site       : ',i5,/, &
         & 'pointer ---    tau(1..3,1..nsite)    --- atoms')
102 Format(a,': inconsistency in kaps, atoms ',i4,' and ',i4, &
         & 'have the same tau!'/,'kaps(',i4,')=',i4, &
         & ' kaps(',i4,')=',i4)
104 Format(a,': the atom ',i4,' at site : ',i4, &
         & ' is multiply occupied by the same type ',i4,' !')
300 Format(a,': real site ',i3,' local sort ',i3,' siteToPos ' &
         & ,i3,' site ',i3,/,' ERROR: siteToPos should be real site !')
400 Format(a,': sort ',i4,' maped onto site ',i4 &
         & ,' which is of sort ',i4)
401 Format('sort ',i4,' site ',i4)
402 Format(' ------- mapping of sort onto site ------------')
403 Format(' ----------------------------------------------')
404 Format(a,': sort ',i4,' maped onto site ',i4 &
         & ,' but function isstf  ',i4)
500 Format('icis  ',i4,' representing sort ',i4)
501 Format(' ------- classification of sorts   ------------' &
         & ,/,'number of inequivalent sites ',i4)
503 Format('sort ',i4,' representing sort ',i4)
502 Format(/,'number of inequivalent atoms ',i4)
    !
  End Subroutine def_cpa_indexarrays
  !
  !!***********************************************************************
  Subroutine calcpairs(structure,lattice_vectors&
       &,reciprocial_lattice_vectors,lattice)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(inout):: lattice
    Real(t_real),Intent(in)      :: lattice_vectors(3,3)
    Real(t_real),Intent(in)      :: reciprocial_lattice_vectors(3,3)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calcpairs"
    !!
    Integer(t_int) :: NRm(3),NRp(3),j,isa,isb,n,i1,i2,i3
    Real(t_real)   :: ag,rg,rcut2,pabs2,ra
    Real(t_real)   :: ta(3),tb(3),a(3),g(3),p(3)
    !!
    rcut2=lattice% rcut**2
    !!
    lattice% npairsmax=0
    !!
    isaloop1: Do isa=1,structure% nsite
       ta=structure% p_tau(:,isa)
       isbloop1: Do isb=1,structure% nsite
          tb=structure% p_tau(:,isb)
          
          a=ta-tb
          Do j=1,3
             g=reciprocial_lattice_vectors(:,j)/twopi
             rg=lattice% rcut*Sqrt(Dot_product(g,g))
             ag=Dot_product(a,g)
             NRm(j)=Floor  (Min(-ag-rg,-ag+rg))-1
             NRp(j)=Ceiling(Max(-ag-rg,-ag+rg))+1
          Enddo
          If(structure% str_type == STR_TYPE_MOLECULE)Then
             NRm(:)=0_t_int
             NRp(:)=0_t_int
          Endif
          n=0
          i1loop1: Do i1=NRm(1),NRp(1)
             Do i2=NRm(2),NRp(2)
                Do i3=NRm(3),NRp(3)
                   
                   p=   a &
                        &+lattice_vectors(:,1)*i1&
                        &+lattice_vectors(:,2)*i2&
                        &+lattice_vectors(:,3)*i3
                   
                   pabs2=Dot_product(p,p)
                   If(pabs2 <= rcut2 .And. pabs2>=1.0e-6_t_real)Then
                      n=n+1
                   Endif

                End Do
             End Do
          End Do i1loop1
          lattice% npairsmax=Max(lattice% npairsmax,n)

       End Do isbloop1
    Enddo isaloop1
    Write(outfile,903) lattice% npairsmax
    !!
    Call palloc('lattice% p_pairs',lattice% p_pairs&
         &,1_t_int,3_t_int &
         &,1_t_int,Max(1_t_int,lattice% npairsmax) &
         &,1_t_int,structure% nsite &
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)
    !!
    isaloop2: Do isa=1,structure% nsite
       ta=structure% p_tau(:,isa)
       ra=1.0e30_t_real
       If(structure% str_type==STR_TYPE_MOLECULE)Then
          If(structure% nsite>1)Then
             isb=isa+1
             If(isb > structure% nsite) isb=1
             p=structure% p_tau(:,isb)-ta
             ra=Sqrt(Sum(p**2))
          Endif
       Endif
       isbloop2: Do isb=1,structure% nsite
          tb=structure% p_tau(:,isb)
          
          a=ta-tb
          Do j=1,3
             g=reciprocial_lattice_vectors(:,j)/twopi
             rg=lattice% rcut*Sqrt(Dot_product(g,g))
             ag=Dot_product(a,g)
             NRm(j)=Floor  (Min(-ag-rg,-ag+rg))-1
             NRp(j)=Ceiling(Max(-ag-rg,-ag+rg))+1
          Enddo
          If(structure% str_type == STR_TYPE_MOLECULE)Then
             NRm(:)=0_t_int
             NRp(:)=0_t_int
          Endif
          n=0
          i1loop2: Do i1=NRm(1),NRp(1)
             Do i2=NRm(2),NRp(2)
                Do i3=NRm(3),NRp(3)
                   
                   p=   a &
                        &+lattice_vectors(:,1)*i1&
                        &+lattice_vectors(:,2)*i2&
                        &+lattice_vectors(:,3)*i3
                   
                   pabs2=Dot_product(p,p)
                   If(pabs2 <= rcut2 .And. pabs2>=1.0e-6_t_real)Then
                      n=n+1
                      lattice% p_pairs(:,n,isa,isb)=p
                      ra=Min(ra,Sqrt(pabs2))
                   Endif

                End Do
             End Do
          End Do i1loop2
          lattice% p_npairs(isa,isb)=n
       End Do isbloop2
       lattice% p_rsite(structure% p_isrt(isa)) = ra*0.5_t_real
    Enddo isaloop2
    
    !!
    Return
    !!
903 Format("Maximum of       pairs:",i4)
    !!
  End Subroutine calcpairs
  !!***********************************************************************
  
  Subroutine defpair(nsite,npairsmax,npairs,pairs,npshell,npshellmax)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nsite,npairsmax
    Integer(t_int),Intent(in) :: npairs(nsite,nsite)
    Integer(t_int),Intent(out) :: npshell(nsite,nsite),npshellmax
    Real(t_real),Intent(inout)  :: pairs(3,Max(npairsmax,1_t_int),nsite,nsite)
    !!LOCAL
    Character(*),Parameter::nsubfunc="defpair"
    Integer(t_int) :: isa,isb,ip,n,iwk(Max(1_t_int,npairsmax)),i
    Real(t_real)  :: p,psh,apair(Max(1_t_int,npairsmax))
    !!
    npshellmax=0_t_int
    Do isa = 1_t_int, nsite
       Do isb = 1_t_int, nsite
          n=npairs(isa,isb)
          Do ip=1_t_int,n
             apair(ip)=Sqrt(Sum(pairs(:,ip,isa,isb)**2_t_int))
          Enddo
          Call dshel2(1_t_int,n,apair,iwk,1.e-12_t_real)
          
          pairs(:,1_t_int:n,isa,isb)=pairs(:,iwk(1_t_int:n)+1_t_int,isa,isb)
          
          i=0_t_int
          psh=1.0e30_t_real
          Do ip=1_t_int,n
             p=Sqrt(Sum(pairs(:,ip,isa,isb)**2_t_int))
             If(Abs(p-psh) > 1.e-10_t_real) Then
                i=i+1_t_int
                psh=p
             Endif
          Enddo
          npshell(isa,isb)=i
          npshellmax=Max(npshellmax,i)
       Enddo ! isb
    Enddo ! isa
    Do isa=1_t_int,nsite
       Do isb=1_t_int,nsite
          If(npshell(isb,isa) /= npshell(isa,isb) ) Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Shells are not symmetric: please report this!'")
          Endif
       End Do
    End Do
    
    Return
  End Subroutine defpair
  !!***********************************************************************
  Subroutine defpshl(nsite,npairs,pairs,ninpshell,pairabs,maxinpshell,npshell)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nsite,npshell(nsite,nsite)
    Integer(t_int),Intent(in) :: npairs(nsite,nsite)
    Integer(t_int),Intent(out):: ninpshell(:,:,:)
    Integer(t_int),Intent(out):: maxinpshell
    Real(t_real),Intent(inout)  :: pairs(:,:,:,:)
    Real(t_real),Intent(out) :: pairabs(:,:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="defpshl"    
    Integer(t_int) :: isa,isb,i,ip,k,n,j,ne,nf
    Real(t_real)  :: psh,p
    Real(t_real),Allocatable  :: test(:,:),test2(:,:)
    Integer(t_int),Allocatable :: iwk(:)
    Integer :: icheck
    !!
    maxinpshell=0_t_int
    Do isa = 1_t_int, nsite
       Do isb = 1_t_int, nsite
          n=npairs(isa,isb)
          psh=11110.0_t_real
          k=0_t_int
          i=0
          Do ip=1_t_int,n
             p=Sqrt(Sum(pairs(:,ip,isa,isb)**2_t_int))
             If(Abs(p-psh) > 1.e-10_t_real) Then
                i=1_t_int
                k=k+1_t_int
                psh=p
             Else
                i=i+1_t_int
             Endif
             maxinpshell=Max(maxinpshell,i)
             ninpshell(k,isa,isb)=i
             pairabs(k,isa,isb)=p
          Enddo
       Enddo ! isb
    Enddo ! isa
!!NOTUSED    old and wrong histery    
!!NOTUSED    If(maxinpshell > 48_t_int)Then
!!NOTUSED       Call errmsg(nmodule,nsubfunc,E_fatalerror &
!!NOTUSED            &,msg="' No more than 48 equivalent ',/&
!!NOTUSED            &,' neighbours per shell expected! ',/&
!!NOTUSED            &,' decrease two-center cutoff.',/&
!!NOTUSED            &,' I hope this will not be a problem !'")
!!NOTUSED    Endif


    Allocate(test(3_t_int,maxinpshell),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for test !'")
    Endif
    Allocate(test2(3_t_int,maxinpshell),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for test2 !'")
    Endif
    Allocate(iwk(maxinpshell),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for iwk !'")
    Endif

    Do isa=1_t_int,nsite
       Do isb=1_t_int,nsite
          n=0_t_int
          Do i=1_t_int,npshell(isa,isb)
             nf=n+1_t_int
             ne=n+ninpshell(i,isa,isb)
             n=ne
             p=pairabs(i,isa,isb)

             test(:,1_t_int:ninpshell(i,isa,isb))=pairs(:,nf:ne,isa,isb)
             test2(:,1_t_int:ninpshell(i,isa,isb))=pairs(:,nf:ne,isa,isb)
             Call dshell(3_t_int,ninpshell(i,isa,isb),test2,1.0e-12_t_real)
             pairs(:,nf:ne,isa,isb)=test2(:,1_t_int:ninpshell(i,isa,isb))
             iwk(1_t_int:ninpshell(i,isa,isb))=1_t_int
             
             Do k=1_t_int,ninpshell(i,isa,isb)
                Do j=1_t_int,ninpshell(i,isa,isb)
                   If(All(Abs(test(:,j)-pairs(:,nf+k-1_t_int,isa,isb))&
                        &<1.0e-11_t_real)) Then
                      If(iwk(j) == 0_t_int)Then
                         Call errmsg(nmodule,nsubfunc,E_fatalerror&
                              &,msg="'after in-shell sort: double occurence!'&
                              &,/,' Please contact programmers!'")
                         
                      Endif
                      iwk(j)=0_t_int
                   Endif
                End Do
             End Do
             Do j=1_t_int,ninpshell(i,isa,isb)
                If(iwk(j)/=0_t_int)Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'after in-shell sort: zero occurence!'&
                        &,/,' Please contact programmers!'")
                   
                Endif
             Enddo
             
          Enddo
       Enddo
    Enddo
    
    If(Allocated(test))Deallocate(test)
    If(Allocated(test2))Deallocate(test2)
    If(Allocated(iwk))Deallocate(iwk)
    

    Return
    !!     
    !!
  End Subroutine defpshl
  !!***********************************************************************
  Subroutine strprn(nsite,nsort,elements,isrt&
       &,npairs,pairs,npshell,ninpshell,pairabs,rsite,ratom,maxinpshell)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nsite,isrt(nsite),nsort
    Character(2),Intent(in)   :: elements(*)
    Integer(t_int),Intent(in) :: maxinpshell
    Integer(t_int),Intent(in) :: npairs(nsite,nsite)
    Integer(t_int),Intent(in) :: npshell(nsite,nsite)
    Integer(t_int),Intent(in) :: ninpshell(:,:,:)
    Real(t_real),Intent(in)   :: pairs(:,:,:,:),pairabs(:,:,:)
    Real(t_real),Intent(in)   :: rsite(nsort)
    Real(t_real),Intent(in)   :: ratom(nsort)
    
    !!LOCAL
    Character(*),Parameter::nsubfunc="strprn"
    Logical(t_log) :: check
    Integer(t_int) :: isa,isb,i,k,issa,issb,ne,nf,nn
    Real(t_real)  :: p,pp
    !!
    check=isoption('PROT_STRUCTURE_PRNT')
    !!      
    Write(outfile,"()")
    Write(outfile,900)
    Write(outfile,901)
    
    Do isa=1_t_int,nsite
       issa=isrt(isa)
       Write(outfile,902)elements(issa),isa,rsite(issa),ratom(issa)&
            &,Sum(npshell(isa,1_t_int:nsite))&
            &,Sum(npairs(isa,1_t_int:nsite))

    Enddo
    Write(outfile,903)
900 Format(77("-")/10("="),t33,"STRUCTURE DATA",t68,10("="))
901 Format(77("-") /"|",t3,"ATOM",t8,"SITE",t13," R_NN/2  ",t24,'nonempty'&
         &,t42,"SHELLS",t61,"NEIGHBOURS",t77,"|")
902 Format(77("-")/"|",t4,a,t8,i3,t13,f10.4,t24,f10.4&
         &,t41,i4,t60,i4,t65,t71,t77,"|")
903 Format(77("-"))
    
    
    Write(outfile,904)
    Write(outfile,905)
    Do isa=1_t_int,nsite
       issa=isrt(isa)
       Do isb=isa,nsite
          issb=isrt(isb)
          
          
          If(npairs(isa,isb) > 0_t_int) Then
             Write(outfile,906)elements(issa),isa,elements(issb),isb&
                  &,npairs(isa,isb)&
                  &,npshell(isa,isb)&
                  &,pairabs(1,isa,isb)&
                  &,pairabs(npshell(isa,isb),isa,isb)
          Else
             Write(outfile,906)elements(issa),isa,elements(issb),isb&
                  &,0_t_int&
                  &,0_t_int &
                  &,0.0_t_real,0.0_t_real
          Endif
       Enddo
    Enddo
    Write(outfile,9041)
9041 Format(79("-"))
904 Format(79("-")&
         &,/,10("="),t24,"TABLE OF NEIGHBOURS",t70,10("="))
905 Format(79("-") /"|",t3,"EL",t6,"SITE",t12,"EL",t15,"SITE", &
         &     t26,"PAIRS",t49,"SHELLS",t64,"RMIN",t73,"RMAX",t79,"|")
906 Format(79("-") /"|",t3,a2,t6,i3,t12,a2,t15,i3&
         &,t19,t24,i4,t29,t34&
         &,t42,t47,i4,t52,t57&
         &,t62,f8.4,t71,f8.4,t79,"|")
    !!
    
    If (check) Then
       !!       npair and pair
       Do isa=1_t_int,nsite
          Do isb=1_t_int,nsite
             Write(outfile,100)
             Write(outfile,101) elements(isrt(isa)),isa&
                  &,elements(isrt(isb)),isb&
                  &,npairs(isa,isb)
             Write(outfile,102)
             
             Do i=1_t_int,npairs(isa,isb)
                p=Sqrt(Sum(pairs(:,i,isa,isb)**2_t_int))
                Write(outfile,103) i,pairs(:,i,isa,isb),p
                
             Enddo
          Enddo
       Enddo
    Endif
    !!       npshell,ninpshell,pairabs
    If (check) Then
       Write(outfile,106)
       Write(outfile,761)maxinpshell
761    Format('Maximum vectors in one shell: ',i3)
    Endif
    Do isa=1_t_int,nsite
       Do isb=1_t_int,nsite
          If (check) Then
             Write(outfile,104)
             Write(outfile,101) elements(isrt(isa)),isa&
                  &,elements(isrt(isb)),isb&
                  &,npshell(isa,isb)
          Endif
          nn=0_t_int
          Do i=1_t_int,npshell(isa,isb)
             nf=nn+1_t_int
             ne=nn+ninpshell(i,isa,isb)
             nn=ne
             p=pairabs(i,isa,isb)
             If (check) Then
                Write(outfile,105) i,ninpshell(i,isa,isb)
                Write(outfile,102) 
             Endif
             Do k=nf,ne
                pp=Sqrt(Sum(pairs(:,k,isa,isb)**2_t_int))
                If (check) Then
                   Write(outfile,1103) k,pairs(:,k,isa,isb),p,pp
                Endif
                If(Abs(p-pp)>1.0e-10_t_real) Then
                   Call errmsg(nmodule,nsubfunc,E_fatalerror&
                        &,msg="'Corrupted pairs, please report bug !'")
                Endif
             Enddo
          Enddo
       Enddo
    Enddo

100 Format('site_A site_B        number of pairs')
101 Format(a2,1x,i3,1x,a2,1x,i3,6x,i4)
102 Format('type          no        px            py&
         &            pz           |p|')
103 Format(t12,i4,1x,4(f12.6,2x))
1103 Format(t12,i4,1x,5(f12.6,2x))
104 Format('site_A site_B       number of pair shells')
105 Format(' no of shell ',i2,'  no in shell ',i2)
106 Format(/,/)
    Return
  End Subroutine strprn
  !!***********************************************************************
  Subroutine def_pairs(symmetric,structure,lattice)
    !! 
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(in)   :: symmetric
    Type(T_Structure),Intent(in)  :: structure
    Type(T_Lattice),Intent(inout) :: lattice
    !!LOCAL
    Character(*),Parameter::nsubfunc="def_pairs"
    Integer(t_int) :: isa,isb,isbp,issa,issb,isap
    Real(t_real)   :: ra
    !!
    Call palloc('lattice% p_rsite',lattice% p_rsite&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('lattice% p_ratom',lattice% p_ratom&
         &,1_t_int,structure% nsort&
         &,nmodule,nsubfunc)

    Call palloc('lattice% p_npairs',lattice% p_npairs&
         &,1_t_int,structure% nsite&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    
    Call calcpairs(structure,symmetric% lattice_vectors&
         &,symmetric% reciprocial_lattice_vectors,lattice)

    
    Call palloc('lattice% p_npshell',lattice% p_npshell&
         &,1_t_int,structure% nsite &
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)

    

    Call defpair(structure % nsite,lattice % npairsmax,lattice % p_npairs&
         &,lattice % p_pairs,lattice % p_npshell,lattice % npshellmax)
    


    Call palloc('lattice% p_pairabs',lattice% p_pairabs&
         &,1_t_int,Max(1_t_int,lattice% npshellmax) &
         &,1_t_int,structure% nsite &
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)



    Call palloc('lattice% p_ninpshell',lattice% p_ninpshell&
         &,1_t_int,Max(1_t_int,lattice% npshellmax) &
         &,1_t_int,structure% nsite &
         &,1_t_int,structure% nsite &
         &,nmodule,nsubfunc)


    
    Call defpshl(structure % nsite,lattice % p_npairs&
         &,lattice % p_pairs,lattice % p_ninpshell,lattice % p_pairabs&
         &,lattice % maxinpshell,lattice % p_npshell)
    
    
    !!START NEW : EMPTY SITES
    Do isa=1,structure% nsite
       isap=structure% p_siteToPos(isa)
       issa=structure% p_isrt(isa)
       ra=1e30_t_real
       If(structure% p_nqAtPos(isap) >1 .Or. &
            &Trim(structure% p_elements(issa)) /= "E" )Then
          Do isbp=1,structure% naps
             isb=structure% p_posToSiteRep(isbp)
             issb=structure% p_isrt(isb)
             If(structure% p_nqAtPos(isbp) >1 .Or. &
                  &Trim(structure% p_elements(issb)) /= "E" )then
                If(lattice% p_npairs(isb,isa)>0)Then
                   ra=Min(ra,lattice% p_pairabs(1_t_int,isb,isa)*0.5_t_real)
                Endif
             Endif
          Enddo
       Else
          ra=lattice% p_rsite(issa)
       Endif
       If(ra>1e29_t_real)Then
          ra=lattice% p_rsite(issa)
       Endif
       lattice% p_ratom(issa)=ra
    Enddo
    !!END NEW : EMPTY SITES


    If(structure% str_type ==  STR_TYPE_MOLECULE)Then
       If(structure% naps == 1) Then
          Do isa=1,structure% nsite
             issa=structure% p_isrt(isa)
             lattice% p_rsite(issa)=100.0_t_real
             lattice% p_ratom(issa)=100.0_t_real
          Enddo
       Endif
    Endif


    Call strprn(structure % nsite,structure % nsort &
         &,structure % p_elements,structure % p_isrt&
         &,lattice % p_npairs,lattice % p_pairs&
         &,lattice % p_npshell,lattice % p_ninpshell&
         &,lattice % p_pairabs,lattice % p_rsite,lattice % p_ratom&
         &,lattice % maxinpshell&
         &)
    !!
  End Subroutine def_pairs
  !!***********************************************************************
  !!***********************************************************************
  !!
  !!   Symmetrizationmatrices for Hamiltonian, Greenian on-site
  !!   parts
  !!
!!$  Subroutine tracesym(naps,nsite,isrt,tau,qposToSite,posToSiteRep,nqAtPos&
!!$       &,maxq,nv,lv &
!!$       &,numops,opmat,optau,opind,opeulerinv,opeuler,rlatt,lattr,nvsmx&
!!$       &,vblocklindex,ning,trace,relativistic)
!!$    Implicit None
!!$    !!IO
!!$    Integer(t_int),Intent(in) :: relativistic
!!$    Integer(t_int),Intent(in) :: naps,nsite,numops,lattr,nvsmx
!!$    Integer(t_int),Intent(in) :: isrt(nsite),qposToSite(nsite,nsite),posToSiteRep(nsite)
!!$    Integer(t_int),Intent(in) :: nqAtPos(nsite),nv(:),lv(:,:)
!!$    Integer(t_int),Intent(in) :: vblocklindex(2,nvsmx,nsite),maxq
!!$    Real(t_real),Intent(in)  :: opmat(3,3,48),optau(3,48),opeuler(3,64)
!!$    Integer(t_int),Intent(in) :: opind(48),opeulerinv(64)
!!$    Real(t_real),Intent(in)  :: tau(3,nsite),rlatt(3,*)
!!$    Integer(t_int),Intent(out):: ning(naps,naps)
!!$    Real(t_real),Intent(out) :: trace(naps,naps,maxq*maxq,nvsmx&
!!$         &,ORBITAL_MAX_MAT_DIM,nvsmx,ORBITAL_MAX_MAT_DIM)
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="tracesym"
!!$    Real(t_real) :: wu(ORBITAL_MAX_MAT_DIM*ORBITAL_MAX_MAT_DIM)
!!$    Real(t_real) :: u(0:ORBITAL_MAX_L,ORBITAL_MAX_MAT_DIM,ORBITAL_MAX_MAT_DIM)
!!$    Integer(t_int):: isap,isbp,iscp,isc,isa,isb,ir,iqaleft,iqaright,isa1,isa2,iqn,is1l,is2l&
!!$         & ,is1r,is2r,isum,i,j,ih(48),issll,isslr,issrl,issrr,nvll,nvlr&
!!$         & ,nvrl,nvrr,my,ny,irl,irr,lrl,lrr,mrl,mrr,mll,mlr,ii,mm&
!!$         & ,mm0,jops,kinv,ing(48,naps,naps)
!!$    Real(t_real) :: eu(3),fac(0:ORBITAL_MAX_MAT_DIM),norm,xx(3)
!!$    Logical(t_log) :: tst
!!$    !!
!!$    If(relativistic == FULL_RELATIVISTIC ) Then
!!$       !Call errmsg(nmodule,nsubfunc,E_warning,msg="' Make relativistic!'")
!!$       Return
!!$    Endif
!!$    !!
!!$    Call make_factorials(fac,ORBITAL_MAX_MAT_DIM)
!!$
!!$    
!!$    Call watch(nsubfunc,cpu_start,reset=.True.)     
!!$    
!!$    tst=isoption('TEST_SYMMETRIZATION')
!!$    
!!$    ning(:,:)=0_t_int
!!$    trace=0.0_t_real
!!$
!!$    
!!$    norm=1.0_t_real/Real(numops,t_real)
!!$    !!     forall S'
!!$    !!     isa1=(nsite), isap=(naps) ,  isa=(naps,iq=1)
!!$    Do isbp=1_t_int,naps
!!$       isb=posToSiteRep(isbp)
!!$       !!       forall G
!!$       Do jops=1_t_int,numops
!!$          !!         US'
!!$          xx(:)=Matmul(tau(:,isb),opmat(:,:,jops))+optau(:,jops)
!!$          !!         which S=US'
!!$          isap=0_t_int
!!$          iploop1: Do iscp=1_t_int,naps
!!$             isc=posToSiteRep(iscp)
!!$             Do ir=1_t_int,lattr
!!$                If(All(Abs(xx(:)-tau(:,isc)-rlatt(:,ir))< 1.e-3_t_real)) Then
!!$                   isap=iscp
!!$                   isa=isc
!!$                   ning(isap,isbp)=ning(isap,isbp)+1_t_int
!!$                   If(ning(isap,isbp) > 48_t_int) Then
!!$                      Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$                           &,msg="more than 48 operations !'")
!!$                   Endif
!!$                   ing(ning(isap,isbp),isap,isbp)=jops
!!$                   If(isrt(isb) /= isrt(isa))Then
!!$                      Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$                           &,msg="'Atoms not sorted for CPA!'")
!!$                   Endif
!!$                   Exit iploop1
!!$                Endif
!!$             Enddo               ! ir
!!$          Enddo iploop1    
!!$
!!$          If(isap == 0_t_int) Then
!!$             Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$                  &,msg="'Symmetry is not true!'")
!!$          Endif
!!$          !!         isa1=S isa2=S',  S'=U-1 S
!!$          If(nqAtPos(isap) /= nqAtPos(isbp))Then
!!$             Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$                  &,msg="'nqAtPos  /=  nqAtPos for symmetry equivalent sites!'")
!!$          Endif
!!$          Do iqaleft=1_t_int,nqAtPos(isap)
!!$             isa1=qposToSite(isap,iqaleft)
!!$             isa2=qposToSite(isbp,iqaleft)
!!$             If(isrt(isa1) /= isrt(isa2)) Then
!!$                Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!$                     &,msg="'Atoms not sorted for CPA!'")
!!$             Endif
!!$          Enddo                 ! iqaleft
!!$          !!         rotations matrices
!!$          kinv=opeulerinv(opind(jops))
!!$          eu=opeuler(:,opind(jops))
!!$          
!!$          Call ytr(eu,kinv,fac,1_t_int,wu,.false.)
!!$          Call paranoia(opmat(:,:,jops),wu,jops,1.e-14_t_real)
!!$
!!$          Do lrr=0_t_int,ORBITAL_MAX_L
!!$             Call ytr(eu,kinv,fac,lrr,wu,.false.)
!!$             Do mrr=1_t_int,lrr+lrr+1_t_int
!!$                Do mll=1_t_int,lrr+lrr+1_t_int
!!$                   ii=(mrr-1_t_int)*(2_t_int*lrr+1_t_int)+mll
!!$!!                   u(lrr,mll,mrr)=cimprov(wu(ii))
!!$                   u(lrr,mll,mrr)=wu(ii)
!!$                Enddo             ! mll
!!$             Enddo               ! mrr
!!$          Enddo                 ! lrr
!!$          !!
!!$          iqn=0_t_int
!!$          Do iqaleft=1_t_int,nqAtPos(isap)
!!$             is1l=qposToSite(isap,iqaleft)
!!$             is2l=qposToSite(isbp,iqaleft)
!!$             Do iqaright=1_t_int,nqAtPos(isap)
!!$                is1r=qposToSite(isap,iqaright)
!!$                is2r=qposToSite(isbp,iqaright)
!!$                iqn=iqn+1_t_int
!!$                !!             
!!$                issll=isrt(is1l)
!!$                nvll=nv(issll)   
!!$                isslr=isrt(is1r)
!!$                nvlr=nv(isslr)   
!!$                issrl=isrt(is2l)
!!$                nvrl=nv(issrl)   
!!$                issrr=isrt(is2r)
!!$                nvrr=nv(issrr)   
!!$                If(issll /= issrl) Then
!!$                   Call terminate(TERM_CRASH,Trim(nsubfunc)//'ll /= rl')
!!$                Endif
!!$                If(isslr /= issrr) Then
!!$                   Call terminate(TERM_CRASH,Trim(nsubfunc)//'lr /= rr')
!!$                Endif
!!$                If(nvll /= nvrl) Then
!!$                   Call terminate(TERM_CRASH,Trim(nsubfunc)//'nlr /= nrr')
!!$                Endif
!!$                If(nvlr /= nvrr) Then
!!$                   Call terminate(TERM_CRASH,Trim(nsubfunc)//'nlr /= nrr')
!!$                Endif
!!$                !!             rechter block
!!$                my=0_t_int
!!$                mm0=0_t_int
!!$                Do irl=1_t_int,nvrl
!!$                   lrl=lv(irl,issrl)
!!$                   Do mrl=1_t_int,lrl+lrl+1_t_int
!!$                      my=my+1_t_int
!!$                      ny=0_t_int
!!$                      Do irr=1_t_int,nvrr
!!$                         lrr=lv(irr,issrr)
!!$                         Do mrr=1_t_int,lrr+lrr+1_t_int
!!$                            ny=ny+1_t_int
!!$                            !!                     linker block
!!$                            mm=mm0
!!$                            Do mll=1_t_int,lrl+lrl+1_t_int
!!$                               mm=mm+1_t_int
!!$                               If(vblocklindex(1,my,is2l)+mll /= mm)Then
!!$                                  Call terminate(TERM_CRASH,Trim(nsubfunc)//'lindex error')
!!$                               Endif
!!$                               If(vblocklindex(2,my,is2l) /= lrl)Then
!!$                                  Call terminate(TERM_CRASH,Trim(nsubfunc)//'lmy error')
!!$                               Endif
!!$                               Do mlr=1_t_int,lrr+lrr+1_t_int
!!$                                  trace(isap,isbp,iqn,my,mll,ny,mlr)=&
!!$                                       &  trace(isap,isbp,iqn,my,mll,ny,mlr)+u(lrl,mll,mrl)*u(lrr,mlr,mrr)*norm
!!$                               Enddo   ! mlr
!!$                            Enddo     ! mll
!!$                         Enddo       ! mrr
!!$                      Enddo         ! irr
!!$                   Enddo           ! mrl
!!$                   mm0=mm0+2*lrl+1
!!$                Enddo             ! irl
!!$                
!!$                
!!$             Enddo               ! iqaright
!!$          Enddo                 ! iqaleft
!!$          
!!$          
!!$       Enddo                   ! jops
!!$    Enddo                     ! isbp
!!$    !!     
!!$    Do isap=1_t_int,naps
!!$       Do isbp=1_t_int,naps
!!$          Do iqn=1_t_int,maxq*maxq
!!$             Do my=1_t_int,nvsmx
!!$                Do ny=1_t_int,nvsmx
!!$                   Do mll=1_t_int,ORBITAL_MAX_MAT_DIM
!!$                      Do mlr=1_t_int,ORBITAL_MAX_MAT_DIM
!!$                         trace(isap,isbp,iqn,my,mll,ny,mlr)=&
!!$!!                              & cimprov(trace(isap,isbp,iqn,my,mll,ny,mlr))
!!$                              & trace(isap,isbp,iqn,my,mll,ny,mlr)
!!$                      Enddo         ! mlr
!!$                   Enddo           ! mll
!!$                Enddo             ! ny
!!$             Enddo               ! my
!!$          Enddo                 ! iqn
!!$       Enddo                   ! isbp
!!$    Enddo                     ! isap
!!$    !!     . test of arrays
!!$    !!     
!!$    Do isap=1_t_int,naps
!!$       isum=0_t_int
!!$       i=0_t_int
!!$       Do isbp=1_t_int,naps
!!$          isum=isum+ning(isap,isbp)
!!$          Do j=1_t_int,ning(isap,isbp)
!!$             i=i+1_t_int
!!$             ih(i)=ing(j,isap,isbp)
!!$          Enddo
!!$       End Do
!!$       Call ipiksrt1(ih,numops)
!!$       Do i=1_t_int,numops
!!$          If (ih(i) /= i) Then
!!$             Write(outfile,15) isap,i
!!$             Call terminate(TERM_CRASH,Trim(nsubfunc))
!!$          Endif
!!$       Enddo
!!$       If (isum /= numops) Then
!!$          Write(outfile,12)isap,isum,numops
!!$          Call terminate(TERM_CRASH,Trim(nsubfunc))
!!$       Endif
!!$    End Do
!!$    !!
!!$    If (tst) Then
!!$       Write(outfile,10)
!!$       Do isap=1_t_int,naps
!!$          Write(outfile,11)isap,(isbp,ning(isap,isbp),isbp=1_t_int,naps)
!!$       Enddo
!!$       Write(outfile,13)
!!$       Do isap=1_t_int,naps
!!$          Do isbp=1_t_int,naps
!!$             Write(outfile,14)isap,isbp,(ing(i,isap,isbp),i=1_t_int,ning(isap,isbp))
!!$          End Do
!!$       Enddo
!!$    Endif
!!$    !!
!!$    Call watch(nsubfunc,cpu_stop,delete=.True.,Print=.True.)  
!!$    !!
!!$    
!!$    !!
!!$10  Format('isap     isbp     operations')
!!$11  Format(10(i4,4x))
!!$12  Format('TRACESYM: isap ',i4,' : number of operations ',i4,&
!!$         & '  /=  numax ',i4)
!!$13  Format('isap    isbp   type of operation')
!!$14  Format(50(i3,1x))
!!$15  Format('TRACESYM: isap ',i4,' : operation ',i4,' is missed !')
!!$
!!$
!!$
!!$    !!
!!$  End Subroutine tracesym
  !!***********************************************************************
  !! prepare an array equivsites(ig,isa) which
  !! gives for a site isa and an operation ig the site g *isa
  !!
  Subroutine prep_equivsites(symmetry,structure)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(inout)  :: symmetry
    Type(T_Structure),Intent(inout) :: structure 
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_equivsites"
    Logical(t_log) :: equiv
    Integer(t_int) :: isa,jops,isc,isb,kinv,i,j
    Real(t_real)   :: s2(3),eu(3),b(3,3),fac(0:ORBITAL_MAX_MAT_DIM),r(3),v(3)
    !!
    Call make_factorials(fac,ORBITAL_MAX_MAT_DIM)

    !!
    Call palloc('structure % p_IOequivsites',structure % p_IOequivsites&
         &,1_t_int,48_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    Call palloc('structure % p_IOequivsitesrev',structure % p_IOequivsitesrev&
         &,1_t_int,48_t_int&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    siteloop: Do isa=1_t_int,structure% nsite
       !!
       !!Print "('isa=',i5,' tau=',t30,3(f15.7,1x))",isa,structure% p_tau(:,isa)
       !!
       oploop: Do jops=1_t_int,symmetry% IOnumops
          !!
          !!Print *,'OP=',jops,Trim(symmetry% IOopsymbol(jops))
          !!
          !! s2=g*s=op_alpha*s+op_tau (opmat is defined as the inverse of 
          !!                          op_alpha)
          s2=Matmul(structure% p_tau(:,isa),symmetry% IOopmat(:,:,jops))&
               &+symmetry% IOoptau(:,jops)
          !!
          !!Print "('OP*tau=',t30,3(f15.7,1x))",s2
          !!
          isc=0_t_int
          isbloop: Do isb=1_t_int,structure% nsite
             !!
             !!Print "('isb=',i5,' tau=',t30,3(f15.7,1x))",isb,structure% p_tau(:,isb)
             !!
             !! v=Gsa-sb
             v(:)=s2(:)-structure% p_tau(:,isb)
             !!
             !!Print "('Gsa-sb=',t30,3(f15.7,1x))",v
             !!
             !! v==R .and. ta==tb ?
             If(structure% str_type == STR_TYPE_MOLECULE)Then
                equiv=All(Abs(v(1:3))<etol)
             Else
                r(1)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
                r(2)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
                r(3)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
                equiv=vecIsOnGrid(3_t_int,r,etol)
             Endif
             !!
             !!Print "('Gsa-sb=',t30,3(f15.7,1x))",r
             !!
             If(equiv .And.(structure% p_IOisrt(isa) == &
                  & structure% p_IOisrt(isb))) Then
                !!Print "(' equivalent sites: ',i5,i5)",isa,isb
                isc=isb
                Exit isbloop
             Endif
          Enddo isbloop
          
          If(isc ==  0_t_int) &
               & Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Symmetry is incorrect!'")
          
          structure% p_IOequivsites(jops,isa)=isc
          structure% p_IOequivsitesrev(jops,isc)=isa

          kinv=symmetry% opeulerinv(symmetry% IOopind(jops))
          eu=symmetry% opeuler(:,symmetry% IOopind(jops))
          
          
          Call ytr(eu,kinv,fac,1_t_int,b,.False.)
          Call paranoia(symmetry% IOopmat(:,:,jops),b,jops,1.e-14_t_real)


       Enddo oploop                   ! jops
       !!
    Enddo siteloop

    !! paranoid control loop
    siteloop2: Do isa=1_t_int,structure% nsite
       !!
       !!Print "('isa=',i5,' tau=',3(f15.7,1x))",isa,structure% p_tau(:,isa)
       !!
       oploop2: Do jops=1_t_int,symmetry% IOnumops
          !!
          !!Print *,'OP=',Trim(symmetry% IOopsymbol(jops))
          !!
          !! s2=g^(+)*s
          s2=Matmul(symmetry% IOopmat(:,:,jops)&
               &,   structure% p_tau(:,isa)-symmetry% IOoptau(:,jops)) 
          !!
          !!Print "('OP*tau=',3(f15.7,1x))",s2
          !!
          isc=0_t_int
          isbloop2: Do isb=1_t_int,structure% nsite
             !! v=Gsa-sb
             v(:)=s2(:)-structure% p_tau(:,isb)
             !!
             !!Print "('Gsa-sb=',i5,' tau=',3(f15.7,1x))",isb,v
             !!
             !! v==R .and. ta==tb ?
             If(structure% str_type == STR_TYPE_MOLECULE)Then
                equiv=All(Abs(v(1:3))<etol)
             Else
                r(1)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,1))/twopi
                r(2)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,2))/twopi
                r(3)=Sum(v(:)*symmetry% reciprocial_lattice_vectors(:,3))/twopi
                equiv=vecIsOnGrid(3_t_int,r,etol)
             Endif
             
             If(equiv .And.(structure% p_IOisrt(isa) == &
                  & structure% p_IOisrt(isb))) Then
                isc=isb
                !!
                !!Print "('Gsa-sb is equivalence isa=',i5,' equivsite=',i5)"&
                !!&,isc,structure% p_IOequivsitesrev(jops,isa)
                !!
                Exit isbloop2
             Endif
          Enddo isbloop2
          
          If(isc ==  0_t_int) &
               & Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Symmetry is incorrect!'")

          If(isc /= structure% p_IOequivsitesrev(jops,isa) ) &
               & Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'equivsitesrev is incorrect!'")
          

       Enddo oploop2                   ! jops
       !!
    Enddo siteloop2
    !!
    !!
    If(outlevel >= L_many ) Then
       !!
       Write(outfile,300)
       Write(outfile,200)
       poploop: Do jops=1,symmetry% IOnumops
          Write(outfile,1) symmetry% IOopsymbol(jops)
          !!
          Do i=1,3
             Write(outfile,3,advance='no') symmetry% IOopmat(1,i,jops)
             Do j=2,3
                Write(outfile,31,advance='no') symmetry% IOopmat(j,i,jops)
             Enddo
             If(i==1)Then
                Write(outfile,5) symmetry% IOoptau(1:3,jops)
             Else
                Write(outfile,4) 
             Endif
          End Do
          !!
          Do isa=1_t_int,structure% nsite
             isc=structure% p_IOequivsites(jops,isa) ! g isa
             Write(outfile,2) isa,isc
          End Do
       End Do poploop
       Write(outfile,300)
       !!
    Endif
    !!
    !!
    Call prep_wu_ks(symmetry,symmetry% IOnumops,symmetry% IOopind&
         &,ORBITAL_MAX_L,symmetry% NRwu_ks)
    !!
    Return
    !!
1   Format('NONREL: ',a15) 
2   Format('NONREL: ',10x,'isa= ',i4,' G isa= ',i4) 
3   Format('NONREL: ',f10.4,' ')
4   Format() 
5   Format(' : ',3(f10.4,1x))
31  Format(f10.4,' ')
200 Format(&
         & 'NONREL: ',' TABLE OF SYMMETRY OPERATIONS ON SITES',/&
         &,'NONREL: ','   isa is site number, ',/&
         &,'NONREL: ','   G*isa is site obtained from isa by operation G',/&
         &,'NONREL: ','  (non-relativistic input symmetries)'&
         &) 
300 Format('NONREL: ',60('='))
    !!
  End Subroutine prep_equivsites
  !!***********************************************************************
  !! prepare non relativistic Wigner D-functions for the crystal
  !! synmmetry group 
  Subroutine prep_wu_ks(symmetry,numops,opind,lmax,wu_ks)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(inout)  :: symmetry
    Integer(t_int),Intent(in)    :: lmax
    Integer(t_int),Intent(in)    :: numops
    Integer(t_int),Intent(in)    :: opind(48)
    Real(t_real),Intent(out)     :: wu_ks(2*lmax+1,2*lmax+1,0:lmax,48)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="prep_wu_ks"
    Integer(t_int) :: jops,kinv,l,l2,m1,m2
    Real(t_real)   :: eu(3),u((2*lmax+1)**2)
    Real(t_real) :: fac(0_t_int:2*lmax+1)
    !!
    Call make_factorials(fac,2*lmax+1)
    !!
    !! prepare rotation matrices for the kohn-sham objects
    
    Do jops=1,numops
       kinv=symmetry% opeulerinv(opind(jops))
       eu=symmetry% opeuler(:,opind(jops))

       Do l=0,lmax
          Call ytr(eu,kinv,fac,l,u,.False.)
          l2=2*l+1
          Do m1=1,l2
             Do m2=1,l2
                wu_ks(m1,m2,l,jops)=u(m1+l2*(m2-1))
             End Do             
          End Do
          
       End Do ! l
    End Do ! jops
    
    !!
    Return
    !!
  End Subroutine prep_wu_ks
  !!***********************************************************************
  !! prepare Ylm Wigner D-functions for the relativistically reduced crystal
  !! symmetry group 
  Subroutine rel_prep_wu_ks(symmetry,rel)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: rel
    Type(T_Symmetry),Intent(inout)  :: symmetry
    !!LOCAL
    Character(*),Parameter::nsubfunc="rel_prep_wu_ks"
    Integer(t_int) :: KMAX,JMAX,jops,kinv,kap,j,l,ma,mb,i,mub,deg
    Real(t_real)   :: eu(3),finv,s
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Real(t_real),Pointer   :: p_fac(:)
    Complex(t_real),Pointer:: p_u(:)
    !!
    Nullify(p_fac,p_u)
    !! nrel symmetries in relat site labeling
    !!
    Call prep_wu_ks(symmetry,symmetry% Rnumops,symmetry% Ropind&
         &,ORBITAL_MAX_L+1,symmetry% Rwu_ks)
    !!
    If(.Not.rel) Then
       !!
       Call palloc('symmetry% p_Rwu_tjnl_ks',symmetry% p_Rwu_tjnl_ks&
            &,1_t_int,ORBITAL_MAX_MAT_DIM+2&
            &,1_t_int,ORBITAL_MAX_MAT_DIM+2&
            &,0_t_int,ORBITAL_MAX_L+1&
            &,1_t_int,48_t_int&
            &,nmodule,nsubfunc)
       !!
       Do l=0,ORBITAL_MAX_L+1
          deg=2*l+1
          Do jops=1,symmetry% Rnumops
             symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)=&
                  &symmetry% Rwu_ks(1:deg,1:deg,l,jops)
          Enddo
       Enddo
       Return
    Endif
    !!
    !! full-relat symmetries
    !!
    KMAX=Ubound(symmetry% Rwu_tj_ks,3)
    JMAX=2*KMAX-1
    !!
    Call palloc('p_fac',p_fac,0_t_int,JMAX,nmodule,nsubfunc)
    Call palloc('p_u',p_u,1_t_int,(JMAX+1)**2,nmodule,nsubfunc)
    !!
    Call make_factorials(p_fac,JMAX)
    !!
    Call palloc('symmetry% p_Rwu_tjnl_ks',symmetry% p_Rwu_tjnl_ks&
         &,1_t_int,2*ORBITAL_MAX_MAT_DIM&
         &,1_t_int,2*ORBITAL_MAX_MAT_DIM&
         &,0_t_int,ORBITAL_MAX_L&
         &,1_t_int,48_t_int&
         &,nmodule,nsubfunc)
    symmetry% p_Rwu_tjnl_ks=c0
    !!
    Do jops=1,symmetry% Rnumops
       !Write(outfile,"(a)") symmetry% Ropsymbol(jops)
       kinv=symmetry% opeulerinv(symmetry% Ropind(jops))
       eu=symmetry% opeuler(:,symmetry% Ropind(jops))
       !Call write_euler(eu)
       
       Do kap=-KMAX,KMAX
          If(kap==0) Cycle
          j=2*Abs(kap)-1
          l=(Abs(2*kap+1)-1)/2
          finv=1.0_t_real
          If(kinv==1) finv=(-1.0_t_real)**l

          Call wignerDJ(j,eu,p_u(:),p_fac,.True.)
          Do ma=1,j+1
             Do mb=1,j+1
                i=mb
                s=1.0_t_real
                If(symmetry% Roptimerev(jops)==1) Then 
                   i=j+1-(mb-1) !! index of negative mu
                   mub=2*(mb-1)-j !! mu of index mb
                   s=(-1.0_t_real)**((mub+1)/2)*Sign(1_t_int,kap)
                Endif
                
                symmetry% Rwu_tj_ks(ma,mb,kap,jops)=p_u(ma+(j+1)*(i-1))*s*finv
             Enddo
          Enddo
          !!
       End Do
       !!
       Do l=0,KMAX-1
          deg=2*l
          symmetry% p_Rwu_tjnl_ks(1:deg,1:deg,l,jops)=&
               &symmetry% Rwu_tj_ks(1:deg,1:deg,l,jops)
          ma=deg
          deg=deg+2
          symmetry% p_Rwu_tjnl_ks(ma+1:ma+deg,ma+1:ma+deg,l,jops)=&
               &symmetry% Rwu_tj_ks(1:deg,1:deg,-l-1,jops)
       Enddo
       !!
    Enddo
    !!
    Call pfree(p_fac)
    Call pfree(p_u)
    !!
    !!
    Return
    !!
  End Subroutine rel_prep_wu_ks
  !!***********************************************************************
  !! analyse the symmetry properties of important objects
  !! like the density matrices and local densities of states
  !!TODO: relativistic case: two modes, bandanisotropy and full anisotropy
!!!  Subroutine sym_analysis(symmetry,structure,relativistic)
!!!    !!
!!!    Implicit None 
!!!    !!IO
!!!    Type(T_Symmetry),Intent(inout)  :: symmetry
!!!    Type(T_Structure),Intent(in)    :: structure
!!!    Integer(t_int),Intent(in)       :: relativistic
!!!    !!LOCAL
!!!    Character(*),Parameter::nsubfunc="sym_analysis"
!!!    Integer(t_int) :: l,l2,isa,jops,isc,m,n,p,q,ig,hits,i,j,npar,nij,issa
!!!    Integer        :: icheck
!!!    Real(t_real)   :: norm,tol,tolproj,tolvec
!!!    Real(t_real),Pointer :: bb(:,:,:,:,:)
!!!    Real(t_real),Pointer :: cc(:,:)
!!!    Real(t_real),Pointer :: vec(:)
!!!    !!
!!!    Nullify(bb)
!!!    Nullify(vec)
!!!    Nullify(cc)
!!!    !!
!!!    If(relativistic == FULL_RELATIVISTIC)Then
!!!       !Call errmsg(nmodule,nsubfunc,E_warning&
!!!       !     &,msg="' Not yet full relativistic!'")
!!!       Return
!!!    Endif
!!!
!!!    Call palloc('symmetry% p_sdcmp_ij',symmetry% p_sdcmp_ij&
!!!         &,1_t_int,Int(ORBITAL_MAX_MAT_DIM,t_int)&
!!!         &,1_t_int,Int(ORBITAL_MAX_MAT_DIM,t_int)&
!!!         &,1_t_int,structure% nsite&
!!!         &,0_t_int,Int(ORBITAL_MAX_L,t_int)&
!!!         &,nmodule,nsubfunc)
!!!
!!!    Call palloc('bb',bb&
!!!         &,1_t_int,ORBITAL_MAX_MAT_DIM&
!!!         &,1_t_int,ORBITAL_MAX_MAT_DIM&
!!!         &,1_t_int,ORBITAL_MAX_MAT_DIM&
!!!         &,1_t_int,ORBITAL_MAX_MAT_DIM&
!!!         &,1_t_int,structure% nsite,nmodule,nsubfunc)
!!!    
!!!    Open(12,file='+symanalysis')
!!!    tol=1.e-6_t_real
!!!    tolproj=1.e-8_t_real
!!!    tolvec=1.e-6_t_real
!!!    !!
!!!    !!
!!!    !! onsite density matrix blocks, ldos
!!!    !! space sub group
!!!    !! 
!!!    !! A     =B        A    
!!!    !!  s               s
!!!    !!         m,n      
!!!    !!  m,n    p,q      p,q
!!!
!!!    Write(12,"(30('-'),'SYMMETRY ANALYSIS',30('-'))")
!!!    Write(12,"(20x,'onsite density matrix (local DOS)')")  
!!!    Write(12,"('within standard coordinate system, without &
!!!         &bandplot rotation of ',/,'quantization axis',/)")  
!!!
!!!    lloop: Do l=0_t_int,ORBITAL_MAX_L
!!!       l2=2_t_int*l+1_t_int
!!!       
!!!       nij=0_t_int
!!!       isaloop: Do isa=1_t_int,structure% nsite
!!!          If(l==0_t_int)Then
!!!             Write(12,"('space group operations reproducing site '&
!!!                  &,i5,2x,a2)")&
!!!                  &  isa,structure% p_elements(structure% p_isrt(isa))
!!!          End If
!!!          
!!!          ig=0_t_int
!!!          bb(:,:,:,:,1_t_int)=0.0_t_real
!!!          
!!!          oploop: Do jops=1,symmetry% Rnumops
!!!             isc=structure% p_equivsites(jops,isa) ! g isa
!!!             !!
!!!             !! A     = D^(*) A    D        
!!!             !!  s             gs                
!!!             !!  m,n     p,m   p,q  q,n
!!!             If(isc==isa)Then
!!!                If(l==0_t_int)Then
!!!                   If(Mod(ig,4_t_int)== 0_t_int ) &
!!!                        & Write(12,"('     ')",advance='no')
!!!                End If
!!!                
!!!                
!!!                ig=ig+1_t_int
!!!                
!!!                If(l==0_t_int)Then
!!!                   Write(12,"(a15)",advance='no') symmetry% Ropsymbol(jops)
!!!                   If(Mod(ig,4_t_int)== 0_t_int ) Then
!!!                      Write(12,"()")
!!!                   Endif
!!!                End If
!!!                
!!!                
!!!                !! 
!!!                !! B      =Sum   D^(+)   D
!!!                !!   m,n    g     p,m     q,n
!!!                !!   p,q
!!!                Do m=1_t_int,l2
!!!                   Do n=1_t_int,l2
!!!                      Do p=1_t_int,l2
!!!                         Do q=1_t_int,l2
!!!                            bb(m,n,p,q,1_t_int)=bb(m,n,p,q,1_t_int)&
!!!                                 &+symmetry% NRwu_ks(p,m,l,jops)&
!!!                                 &*symmetry% NRwu_ks(q,n,l,jops)
!!!                         End Do
!!!                      End Do
!!!                   End Do
!!!                End Do
!!!
!!!             End If
!!!
!!!
!!!          End Do oploop
!!!          If(l==0_t_int)Then
!!!             If(Mod(ig,4_t_int)/= 0_t_int ) Write(12,"()")
!!!          End If
!!!          
!!!
!!!
!!!          norm=1.0_t_real/Real(ig,t_real)
!!!
!!!
!!!          Do m=1_t_int,l2
!!!             Do n=1_t_int,l2
!!!                If(All(Abs(bb(m,n,1_t_int:l2,1_t_int:l2,1_t_int))<tol))Then
!!!                   symmetry% p_sdcmp_ij(m,n,isa,l)=0_t_int
!!!                Else
!!!                   nij=nij+1_t_int
!!!                   symmetry% p_sdcmp_ij(m,n,isa,l)=nij
!!!                End If
!!!             Enddo
!!!          Enddo
!!!
!!!
!!!          Write(12,"('matrix structure site ',i5,2x,a2,' l= ',i1)")  &
!!!               &  isa,structure% p_elements(structure% p_isrt(isa)),l
!!!          Do m=1_t_int,l2
!!!             Write(12,"('     ')",advance='no')
!!!             Do n=1_t_int,l2
!!!                If(symmetry% p_sdcmp_ij(m,n,isa,l) /= 0_t_int)Then
!!!                   Write(12,"('X',1x)",advance='no')
!!!                Else
!!!                   Write(12,"('0',1x)",advance='no')
!!!                Endif
!!!             Enddo
!!!             Write(12,"()")
!!!          Enddo
!!!
!!!          Write(12,"(2x,'symmetry conditions l=',i1)")  l
!!!          Do m=1_t_int,l2
!!!             Do n=1_t_int,l2
!!!                hits=-1_t_int
!!!                Do p=1_t_int,l2
!!!                   Do q=1_t_int,l2
!!!                      If(Abs(bb(m,n,p,q,1_t_int))>tol &
!!!                           &.And.  symmetry% p_sdcmp_ij(p,q,isa,l) &
!!!                           &/=0_t_int ) Then
!!!                         If(hits < 0_t_int)Then
!!!                            Write(12,"(5x,'A(',i1,',',i1,')=')",advance='no')m,n
!!!                            hits=0_t_int
!!!                         Else
!!!                            If(hits==0_t_int)Then
!!!                               Write(12,"(5x,'       ')",advance='no') 
!!!                            Endif
!!!
!!!                         End If
!!!
!!!                         Write(12,"(sp,f8.5,ss,' A(',i1,',',i1,')')"&
!!!                              &,advance='no') bb(m,n,p,q,1_t_int)*norm,p,q
!!!
!!!                         hits=hits+1_t_int
!!!
!!!                         If(Mod(hits,4_t_int)== 0_t_int ) Then
!!!                            Write(12,"()")
!!!                            hits=0_t_int
!!!                         End If
!!!                      Endif
!!!                   Enddo
!!!                Enddo
!!!                If(hits>0_t_int)Then
!!!                   Write(12,"()")
!!!                Endif
!!!             Enddo
!!!          Enddo
!!!
!!!
!!!       Enddo isaloop
!!!       symmetry% sdcmp_nij(l)=nij
!!!
!!!    End Do lloop
!!!    
!!!
!!!    !! now full space group
!!!    !! space sub group
!!!    !! 
!!!    !! A     =B        A    
!!!    !!  s      s,gs     gs
!!!    !!         m,n      
!!!    !!  m,n    p,q      p,q
!!!!!$    Write(outfile,100)nsubfunc
!!!!!$100 Format(a,': before full space group')
!!!
!!!    m=Maxval(symmetry% sdcmp_nij(0_t_int:ORBITAL_MAX_L))
!!!    Call palloc('symmetry% p_sdcmp_vec',symmetry% p_sdcmp_vec&
!!!         &,1_t_int,m&
!!!         &,1_t_int,m&
!!!         &,0_t_int,Int(ORBITAL_MAX_L,t_int)&
!!!         &,nmodule,nsubfunc)
!!!    
!!!    Call palloc('symmetry% p_sdcmp_npar_s',symmetry% p_sdcmp_npar_s&
!!!         &,0_t_int,Int(ORBITAL_MAX_L,t_int)&
!!!         &,1_t_int,m &
!!!         &,1_t_int,structure% nsort&
!!!         &,nmodule,nsubfunc)
!!!
!!!    symmetry% p_sdcmp_npar_s(&
!!!         &0_t_int:ORBITAL_MAX_L&
!!!         &,1_t_int:m&
!!!         &,1_t_int:structure% nsort)=0_t_int
!!!
!!!    lloop2: Do l=0_t_int,ORBITAL_MAX_L
!!!       l2=2_t_int*l+1_t_int
!!!       
!!!!!$       Write(outfile,101)nsubfunc,l
!!!!!$101    Format(a,': llop2 l=',i3)
!!!       
!!!       norm=1.0_t_real/Real(symmetry% Rnumops,t_real)
!!!       
!!!       nij=symmetry% sdcmp_nij(l)
!!!       
!!!    
!!!       Call palloc('vec',vec,1_t_int,nij,nmodule,nsubfunc)   
!!!       Call palloc('cc ',cc ,1_t_int,nij,1_t_int,nij,nmodule,nsubfunc)
!!!!!$       Call palloc('zz ',zz ,1_t_int,nij,1_t_int,nij,nmodule,nsubfunc)
!!!!!$       Call palloc('eig ',eig,1_t_int,nij,nmodule,nsubfunc)
!!!       cc(1:nij,1:nij)=0.0_t_real
!!!       
!!!       
!!!       isaloop2: Do isa=1_t_int,structure% nsite
!!!          Write(12,"(/,'Full symmetry, site ',i5,2x,a2,' l=',i1)") isa &
!!!               &,structure% p_elements(structure% p_isrt(isa)),l
!!!!!$          Write(outfile,102)nsubfunc,l,isa
!!!!!$102       Format(a,': isaloop2 l=',i3,' isa=',i5)
!!!          
!!!          bb(1:ORBITAL_MAX_MAT_DIM,1:ORBITAL_MAX_MAT_DIM&
!!!               &,1:ORBITAL_MAX_MAT_DIM,1:ORBITAL_MAX_MAT_DIM&
!!!               &,1_t_int:structure% nsite)=0.0_t_real
!!!          oploop2: Do jops=1,symmetry% Rnumops
!!!             isc=structure% p_equivsites(jops,isa) ! g isa
!!!             !!
!!!             !! A     = D^(*) A    D        
!!!             !!  s             gs                
!!!             !!  m,n     p,m   p,q  q,n
!!!             
!!!             
!!!             
!!!             !! 
!!!             !! B      =Sum   D^(+)   D
!!!             !!   m,n    g     p,m     q,n
!!!             !!   p,q
!!!
!!!             Do m=1_t_int,l2
!!!                Do n=1_t_int,l2
!!!                   Do p=1_t_int,l2
!!!                      Do q=1_t_int,l2
!!!                         bb(m,n,p,q,isc)=bb(m,n,p,q,isc)&
!!!                              &+symmetry% NRwu_ks(p,m,l,jops)&
!!!                              &*symmetry% NRwu_ks(q,n,l,jops)*norm
!!!
!!!                      End Do
!!!                   End Do
!!!                End Do
!!!             End Do
!!!
!!!
!!!
!!!
!!!          End Do oploop2
!!!
!!!
!!!
!!!          Do m=1_t_int,l2
!!!             Do n=1_t_int,l2
!!!                hits=-1_t_int
!!!                i=symmetry% p_sdcmp_ij(m,n,isa,l)
!!!                Do isc=1_t_int,structure% nsite
!!!                   Do p=1_t_int,l2
!!!                      Do q=1_t_int,l2
!!!                         j=symmetry% p_sdcmp_ij(p,q,isc,l)
!!!                         If(Abs(bb(m,n,p,q,isc))>tol .And. j /= 0_t_int ) Then
!!!                            cc(i,j)=bb(m,n,p,q,isc)
!!!                            If(hits < 0_t_int)Then
!!!                               Write(12,"(5x,'A(',i1,',',i1,',',i4,')=')"&
!!!                                    &,advance='no')m,n,isa
!!!                               hits=0_t_int
!!!                            Else
!!!                               If(hits==0_t_int)Then
!!!                                  Write(12,"(5x,'            ')",advance='no') 
!!!                               Endif
!!!
!!!                            End If
!!!
!!!                            Write(12,"(sp,f8.5,ss,' A(',i1,',',i1,',',i4,')')"&
!!!                                 &,advance='no') bb(m,n,p,q,isc),p,q,isc
!!!
!!!                            hits=hits+1_t_int
!!!
!!!                            If(Mod(hits,3_t_int)== 0_t_int ) Then
!!!                               Write(12,"()")
!!!                               hits=0_t_int
!!!                            End If
!!!                         Endif
!!!                      Enddo
!!!                   Enddo
!!!                End Do ! isc
!!!                If(hits>0_t_int)Then
!!!                   Write(12,"()")
!!!                Endif
!!!
!!!             Enddo
!!!          Enddo
!!!
!!!
!!!       Enddo isaloop2
!!!
!!!
!!!       !! make eigen decomposition of cc
!!!       !! determine symmetry parameter
!!!
!!!!!$       Do i=1,nij
!!!!!$          Do j=i+1,nij
!!!!!$             If(Abs(cc(i,j)-cc(j,i))>1.0e-10_t_real)Then
!!!!!$                Print *,'no hermes ',i,j,cc(i,j),cc(j,i)
!!!!!$                Stop
!!!!!$             Endif
!!!!!$          Enddo
!!!!!$       Enddo
!!!
!!!!!$       Print *,'nij=',nij
!!!!!$       
!!!!!$       Call rdiagno_noO(nij,cc,zz,eig)
!!!!!$
!!!!!$       Do i=1,nij
!!!!!$          Print *,'eig=',i,eig(i)
!!!!!$       Enddo
!!!
!!!
!!!       npar=0_t_int
!!!       testvecloop: Do i=1_t_int,nij
!!!!!$          Write(outfile,103)nsubfunc,l,Real(i,t_real)/nij,nij
!!!!!$103       Format(a,': testvecloop l=',i3,' i/nij=',f6.2,' nij=',i7)
!!!          !! candidate is i-th column
!!!          symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l)=cc(1_t_int:nij,i)
!!!          !! orthogonalize
!!!          Do j=1,npar
!!!             symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l)&
!!!                  &=symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l)&
!!!                  &-Dot_product(cc(1_t_int:nij,i)&
!!!                  &,symmetry% p_sdcmp_vec(1_t_int:nij,j,l))&
!!!                  &/Dot_product(symmetry% p_sdcmp_vec(1_t_int:nij,j,l)&
!!!                  &,symmetry% p_sdcmp_vec(1_t_int:nij,j,l))&
!!!                  &*symmetry% p_sdcmp_vec(1_t_int:nij,j,l)
!!!          Enddo ! j
!!!          !! some tests
!!!          If(Any(Abs(Matmul(cc(1_t_int:nij,1_t_int:nij)&
!!!               &           ,symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l))&
!!!               &-symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l))>tolproj))&
!!!               &Then
!!!             Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!!                  &,msg="'cc violates projection property !'")
!!!          Endif
!!!          
!!!          !! is it /= 0 ?
!!!          If(Any(Abs(symmetry% p_sdcmp_vec(1_t_int:nij,npar+1_t_int,l))&
!!!               &>tolvec))Then
!!!             npar=npar+1_t_int
!!!          Endif
!!!          
!!!       Enddo testvecloop
!!!
!!!       symmetry% sdcmp_npar(l)=npar
!!!       
!!!       Do j=1,npar
!!!          symmetry% p_sdcmp_vec(1_t_int:nij,j,l)&
!!!               &=symmetry% p_sdcmp_vec(1_t_int:nij,j,l)&
!!!               &/Maxval(symmetry% p_sdcmp_vec(1_t_int:nij,j,l))
!!!       Enddo
!!!
!!!       !! make coefficients of single combinations unity
!!!       Do i=1_t_int,nij
!!!          m=0_t_int
!!!          Do j=1,npar
!!!             If(Abs(symmetry% p_sdcmp_vec(i,j,l))>tolvec)Then
!!!                m=m+1_t_int
!!!                n=j
!!!             Endif
!!!          Enddo
!!!          If(m == 1_t_int)Then
!!!             symmetry% p_sdcmp_vec(1_t_int:nij,n,l)&
!!!                  &=symmetry% p_sdcmp_vec(1_t_int:nij,n,l)&
!!!                  &/symmetry% p_sdcmp_vec(i,n,l)
!!!          Endif
!!!       End Do
!!!
!!!       !! which vectors contribute to the sort issa
!!!
!!!
!!!       isaloop3: Do isa=1,structure% nsite
!!!          issa=structure% p_isrt(isa)
!!!          
!!!          Do m=1_t_int,l2
!!!             Do n=1_t_int,l2
!!!                i=symmetry% p_sdcmp_ij(m,n,isa,l)
!!!                If(i /= 0_t_int)Then
!!!                   Do j=1_t_int,npar
!!!                      If(Abs(symmetry% p_sdcmp_vec(i,j,l))>tolvec)Then
!!!                         symmetry% p_sdcmp_npar_s(l,j,issa)=1_t_int
!!!                      Endif
!!!                   End Do !j
!!!                Endif ! i/= 0
!!!             End Do !n
!!!          End Do !m
!!!       End Do isaloop3
!!!       
!!!       !! print symmetry decomposition
!!!       
!!!       isaloop4: Do isa=1,structure% nsite
!!!          issa=structure% p_isrt(isa)
!!!          Write(12,"('Parameter decomposition, site',i5,2x,a2,' l=',i1)")isa &
!!!               &,structure% p_elements(issa),l
!!!          Do m=1_t_int,l2
!!!             Do n=1_t_int,l2
!!!                i=symmetry% p_sdcmp_ij(m,n,isa,l)
!!!                If(i /= 0_t_int)Then
!!!                   Write(12,"('     A(',i1,',',i1,',',i5,')=')",advance='no')&
!!!                        &m,n,isa
!!!                   hits=0_t_int
!!!                   Do j=1_t_int,npar
!!!                      If(Abs(symmetry% p_sdcmp_vec(i,j,l))>tolvec)Then
!!!                         If(hits == 0_t_int)Then
!!!                            Write(12,"(e30.14e3,'*C(',i5,')')"&
!!!                                 &,advance='no')symmetry% p_sdcmp_vec(i,j,l),j
!!!                         Else
!!!                            Write(12,"(17x,'+',e30.14e3,'*C(',i5,')')"&
!!!                                 &,advance='no')symmetry% p_sdcmp_vec(i,j,l),j
!!!                         End If
!!!                         hits=hits+1_t_int
!!!                         Write(12,"()")
!!!                         If(symmetry% p_sdcmp_npar_s(l,j,issa) == 0_t_int)Then
!!!                            Call errmsg(nmodule,nsubfunc,E_fatalerror&
!!!                                 &,msg="'parameter sort mapping wrong!'")
!!!
!!!                         Endif
!!!                      Endif
!!!                   End Do !j
!!!                Endif ! i/= 0
!!!             End Do !n
!!!          End Do !m
!!!       End Do isaloop4
!!!       
!!!       !!
!!!       Call pfree(vec)   
!!!       Call pfree(cc)
!!!!!$       Call pfree(zz)
!!!!!$       Call pfree(eig)
!!!       !!
!!!    End Do lloop2
!!!    Write(12,"(77('-'))")    
!!!    Close(12)
!!!    !!
!!!    Call pfree(bb)
!!!    !!
!!!  End Subroutine sym_analysis
  !!***********************************************************************
  !!
  !!   Rotation of local site density or potential
  !!
  Subroutine rotloc(mspin,ms,vs,mlmp,nlmp,lmp,nr&
       &,nsite,equivsites &
       &,numops,opeuler,opeulerinv,opind &
       &,max_L)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: mspin,nsite,max_L
    Integer(t_int),Intent(in) :: equivsites(48,nsite)
    Integer(t_int),Intent(in) :: mlmp,lmp(mlmp,nsite),nlmp(nsite)
    Integer(t_int),Intent(in) :: numops,opind(48),opeulerinv(64)
    Real(t_real),Intent(in)  :: opeuler(3,64)
    Integer(t_int),Intent(in) :: nr,ms
    Real(t_real),Intent(inout)  :: vs(nr,mlmp,nsite,mspin)
    !!LOCAL
    Character(*),Parameter::nsubfunc="rotloc"
    Real(t_real) :: fac(0_t_int:Max(3_t_int,2_t_int*max_L+1_t_int))
    Real(t_real) :: wu((2_t_int*max_L+1_t_int)**2_t_int)
    Real(t_real) :: norm,eu(3)
    Real(t_real),Pointer :: vh(:,:,:)
    Integer(t_int) :: isa,isb,jops,kinv,ja,ilm,jb
    !!
    Nullify(vh)
    !!
    Call palloc('vh',vh,1_t_int,nr,1_t_int,mlmp,1_t_int,nsite,nmodule,nsubfunc)
    !!
    Call make_factorials(fac,Max(3_t_int,2_t_int*max_L+1_t_int))

    !!
    
    norm=1.0_t_real/Real(numops,t_real)
    
    
    vh(:,:,:)=0.0_t_real
    
    siteloop: Do isa=1_t_int,nsite
       
       oploop: Do jops=1_t_int,numops
          isb=equivsites(jops,isa)

          kinv=opeulerinv(opind(jops))
          eu=opeuler(:,opind(jops))
          
          Do ja=1_t_int,nlmp(isa)
             
             ilm=lmp(ja,isa)
             
             Call ytr(eu,kinv,fac,llilm(ilm),wu,.False.)
             Do jb=1_t_int,nlmp(isb)
                If(llilm(lmp(jb,isb)) == llilm(ilm)) Then
                   vh(1:nr,ja,isa)=vh(1:nr,ja,isa)+vs(1:nr,jb,isb,ms)&
                        &*wu((mmilm(ilm)-1)*(2*llilm(lmp(jb,isb))+1)&
                        &+mmilm(lmp(jb,isb)))*norm
                End If
             Enddo             ! jb
          Enddo               ! ja
          
       Enddo oploop
       
       
    Enddo siteloop
    
    vs(1:nr,1:mlmp,1:nsite,ms)=vh(1:nr,1:mlmp,1:nsite)
    !!
    Call pfree(vh)
    !!
    Return
    !!
  End Subroutine rotloc
  !!***********************************************************************
  Subroutine prep_double_group(opsymbol,numops,opeulerinv,opeuler,opind)
    !!
    !! prepare consistent euler angles for relativistic double groups
    !! note that operations C^m_n with m>n/2 are treates as C^(m-n)_n
    !!
    Implicit None
    !!IO
    Character(*),Intent(in)      :: opsymbol(:)
    Integer(t_int),Intent(in)    :: numops
    Integer(t_int),Intent(in)    :: opeulerinv(:),opind(:)
    Real(t_real),Intent(inout)   :: opeuler(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_double_group"
    Logical(t_log) :: produceinverseeuler,prot,shortprot
    Integer(t_int) :: jops,kinv,bared,iloop
    Real(t_real)   :: c,ns(3),sa,n(3),D1(3,3),s,phi,eu(3),cb,sb,su,di,D1n(3,3)
    Complex(t_real):: D1o2(2,2),un(2,2)
    Real(t_real),Pointer   :: p_fac(:)
    Character(3),Save    :: strbaredinv(-1:1,0:1)!! first index is bared
    Data strbaredinv(-1,0:1)  /'  _','_I*'/
    Data strbaredinv( 1,0:1)  /'   ',' I*'/
    Real(t_real),Parameter :: tol=1.0e-4_t_real
    !Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    !Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Integer(t_int),Parameter  :: in(3)=(/3_t_int,1_t_int,2_t_int/)
    !!
    Nullify(p_fac)
    !!
    produceinverseeuler=.True.
    prot=debugfun(nmodule,nsubfunc,'')
    shortprot=debugfun(nmodule,nsubfunc,'info')
    !!
    !!
    Call palloc('p_fac',p_fac,0_t_int,3_t_int,nmodule,nsubfunc)
    Call make_factorials(p_fac,3_t_int)
    !!
    If(shortprot.Or.prot)Then
       Write(outfile,1)
    Endif
    !!
    ilooploop: Do iloop=1,2
       jopsloop: Do jops=1,numops
          If(prot)Then
             Write(outfile,85) Trim(opsymbol(jops))
          Endif
          
          kinv=opeulerinv(opind(jops))
          eu=opeuler(:,opind(jops))
          
          !! the l=1 rep
          Call ytr(eu,kinv,p_fac,1_t_int,D1,.False.)
          
          
          !! print D^R, which is the cartesian rep, which is obtained
          !! from the real harmonics rep by permutation: Y_{1m} has m=y,z,x
          If(prot)Then
             Write(outfile,"(/,'D^R=',/,3(f12.5,1x))")D1(in(1),in(:))
             Write(outfile,"(3(f12.5,1x))")D1(in(2),in(:))
             Write(outfile,"(3(f12.5,1x),/)")D1(in(3),in(:))
          Endif
          
          
          !! return wigner of (eu), but eu=eu^{-1} in opeuler
          Call wignerDJ(1_t_int,eu,D1o2,p_fac,.True.)
          
          
          
          !todo: time reversal factor, do this before mag-reduction, best done in c-code
          
          If(prot)Then
             Write(outfile,"(/,'D1/2=',/,2('(',f12.5,',',f12.5,')'))")D1o2(1,:)
             Write(outfile,"(2('(',f12.5,',',f12.5,')'),/)")D1o2(2,:)
          Endif
          
          
          c=Real((D1o2(1,1)+D1o2(2,2)),t_real)*0.5_t_real
          ns(1)=Real(ci*(D1o2(1,2)+D1o2(2,1)),t_real)
          ns(2)=Real((D1o2(1,2)-D1o2(2,1)),t_real)
          ns(3)=Real(ci*(D1o2(2,2)-D1o2(1,1)),t_real)
          ns=ns*0.5_t_real
          If(Abs(Abs(c)-1.0_t_real)>tol)Then
             sa=Sqrt(Sum(ns*ns))
             n=ns/sa
          Else
             n=(/0.0_t_real,0.0_t_real,1.0_t_real/)
          Endif
          s=1.0_t_real
          If(Abs(n(1))<tol)Then
             !!n=(0,ny,nz)
             If(Abs(n(2))<tol)Then
                !! n=(0,ny,0)
                If(n(3)<-tol) s=-1.0_t_real
                sa=ns(3)/n(3)
             Else
                If(n(2)<-tol) s=-1.0_t_real
                sa=ns(2)/n(2)
             Endif
          Else
             If(n(1)<-tol) s=-1.0_t_real
             sa=ns(1)/n(1)
          Endif
          If(All(Abs(Abs(n(:))-sq3o3)<tol))Then
             If(s*n(3)<-tol) s=-s
          Endif
          n=n*s
          sa=sa*s
          
          
          phi=Atan2(sa,c)*2.0_t_real
          If(phi<-tol)Then
             phi=fourpi+phi
          Endif
          
          bared=1
          If(phi>=twopi-tol)Then
             bared=-1
             phi=phi-twopi
          Endif
          
          If(phi>=pi+tol)Then
             phi=phi-twopi
             bared=-bared
          Endif
          
          !! make the unbared operation
          un=D1o2*bared
          
          !! determine proper euler angles in extended domain
          su=Sqrt(Max(0.0_t_real,Real(un(1,1)*un(2,2),t_real)))
          di=Sqrt(Max(0.0_t_real,Real(-un(1,2)*un(2,1),t_real)))
          eu(2)=2.0_t_real*Atan2(di,su)
          If(Abs(eu(2))<tol)Then
             sb=Sin(eu(2)*0.5_t_real)*2.0_t_real
             su=2.0_t_real*Real(Log(un(1,1))/ci,t_real)
             di=-su
          Else If(Abs(eu(2)-pi)<tol)Then
             eu(1)=0.0_t_real
             di=2.0_t_real*Real(Log(un(1,2))/ci,t_real)
             su=-di
          Else
             cb=Cos(eu(2)*0.5_t_real)*2.0_t_real
             sb=Sin(eu(2)*0.5_t_real)*2.0_t_real
             su=2.0_t_real*Real(Log(un(1,1)/cb)/ci,t_real)
             di=2.0_t_real*Real(Log(un(1,2)/sb)/ci,t_real)
          Endif
          eu(1)=(su+di)*0.5_t_real
          Do While(eu(1)<0.0_t_real)
             eu(1)=eu(1)+twopi
          Enddo
          Do While(eu(1)>=twopi)
             eu(1)=eu(1)-twopi
          Enddo
          eu(3)=su-eu(1)
          Do While(eu(3)<0.0_t_real)
             eu(3)=eu(3)+fourpi
          Enddo
          Do While(eu(3)>=fourpi)
             eu(3)=eu(3)-fourpi
          Enddo
          
          
          If(produceinverseeuler)Then
             su=eu(1)
             eu(1)=-eu(3)
             eu(2)=-eu(2)
             eu(3)=-su
             If(eu(2)<0.0_t_real)Then
                eu(2)=twopi+eu(2)  !![0,2pi]
                eu(3)=twopi+eu(3)  !!(0,2pi]
             Endif
             If(eu(1)==fourpi)eu(1)=0.0_t_real!![0,4pi)
             If(eu(1)>=twopi)Then
                eu(1)=eu(1)-twopi!![0,2pi)
                eu(3)=eu(3)+twopi!!(0,4pi]
             Endif
             If(eu(3)==fourpi)eu(3)=0.0_t_real!![0,4pi)
          Endif
          
          If(prot)Then
             Write(outfile,"(' c=',f12.5,' s=',f12.5,2x)") c,sa
             Call write_euler(eu)
             Write(outfile,"(/,'D1/2_unbared=',/,2('(',f12.5,',',f12.5,')'))")un(1,:)
             Write(outfile,"(2('(',f12.5,',',f12.5,')'),/)")un(2,:)
          Endif
          
          If(prot.Or.shortprot)Then
2            Format('Operation ',i3,' was    : ',a,'C(',f12.5,'&
                  &,[',3(f12.5,1x),'])')
3            Format('Operation ',i3,' now is : ',a,'C(',f12.5,'&
                  &,[',3(f12.5,1x),'])')
             If(iloop==1)Then
                Write(outfile,2) jops,strbaredinv(bared,kinv)&
                     &,phi*180.0_t_real/pi,n(1:3)
             Else
                Write(outfile,3) jops,strbaredinv(bared,kinv)&
                     &,phi*180.0_t_real/pi,n(1:3)
             Endif
          Endif
          
          Call wignerDJ(1_t_int,eu,un,p_fac,produceinverseeuler)
          ! the original Euler engles can contain a \bar{E}=-1=D(2pi,\vec{n})
          ! which we try to remove with the new angles
          ! for paranoia check we have to put it back
          If(Any(Abs(un*bared-D1o2)>1.0e-6_t_real))Then
             
             Write(outfile,"(/,'D1/2_original=',/,2('(',f12.5,',',f12.5,')'))")D1o2(1,:)
             Write(outfile,"(2('(',f12.5,',',f12.5,')'),/)")D1o2(2,:)
             
             Write(outfile,"(/,'D1/2_corrected=',/,2('(',f12.5,',',f12.5,')'))")&
                  &un(1,:)*bared
             Write(outfile,"(2('(',f12.5,',',f12.5,')'),/)")un(2,:)*bared
             
             Write(errbuf,66)
66           Format("'The double group euler angles were determined wrongly&
                  &, ask programmer!'")
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
          Endif
          
          
          
          !! ytr(..false) returns inverse D^l
          Call ytr(eu,kinv,p_fac,1_t_int,D1n,.Not.produceinverseeuler)
          If(Any(Abs(D1n-D1)>1.0e-6_t_real))Then
             
             Write(outfile,"(/,'D^R_orig=',/,3(f12.5,1x))")D1(in(1),in(:))
             Write(outfile,"(3(f12.5,1x))")D1(in(2),in(:))
             Write(outfile,"(3(f12.5,1x),/)")D1(in(3),in(:))
             
             Write(outfile,"(/,'D^R_correted=',/,3(f12.5,1x))")D1n(in(1),in(:))
             Write(outfile,"(3(f12.5,1x))")D1n(in(2),in(:))
             Write(outfile,"(3(f12.5,1x),/)")D1n(in(3),in(:))
             
             
             
             Write(errbuf,66)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
          Endif
          !!
          if(iloop==1) opeuler(:,opind(jops))=eu
          !!
       Enddo jopsloop
    Enddo ilooploop
    !!
    Call pfree(p_fac)
    !!
    !!
    !!
    Return
    !!
1   Format(72('-'),/,"  Determine consistent Euler angles for double groups",/&
         &,72('-'),/,"  _ before operations means bared (R(phi+2Pi)) operation")
85  Format('Probing operation ',a,30('-'),/)
    !!
    !!
  End Subroutine prep_double_group
  !!***********************************************************************

  
End Module symmetry

