Module symtest
  !!
  Use precisiondefs
  Use matserv
  Use errors
  Use constants
  Use diracdefs
  Use file_iounits
  Use memory
  Use forces
  Use prnt
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="SYMTEST"  
  !!
  Public :: symtests,mag_rel_tests
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine rotxyz(r,wu,ro)
    !!
    Implicit None
    !!IO
    Real(t_real),Intent(in)     ::  r(3),wu(9)
    Real(t_real),Intent(out)    ::  ro(3)
    !!LOCAL
    Real(t_real)  :: dd
    !!
    dd=Sqrt(r(1_t_int)**2_t_int+r(2_t_int)**2_t_int+r(3_t_int)**2_t_int)
    !!
    ro(1_t_int)=(wu((3_t_int-1_t_int)*3_t_int+3_t_int)*r(1_t_int)&
         &      +wu((1_t_int-1_t_int)*3_t_int+3_t_int)*r(2_t_int) &
         &      +wu((2_t_int-1_t_int)*3_t_int+3_t_int)*r(3_t_int))/dd
    ro(2_t_int)=(wu((3_t_int-1_t_int)*3_t_int+1_t_int)*r(1_t_int)&
         &      +wu((1_t_int-1_t_int)*3_t_int+1_t_int)*r(2_t_int)&
         &      +wu((2_t_int-1_t_int)*3_t_int+1_t_int)*r(3_t_int))/dd
    ro(3_t_int)=(wu((3_t_int-1_t_int)*3_t_int+2_t_int)*r(1_t_int)&
         &      +wu((1_t_int-1_t_int)*3_t_int+2_t_int)*r(2_t_int)&
         &      +wu((2_t_int-1_t_int)*3_t_int+2_t_int)*r(3_t_int))/dd
    !!
  End Subroutine rotxyz

  !!***********************************************************************
  !! The wigner D-functions for integer l, for the spherical harmonics
  !! D^l_{m,m^\prime}=<lm|\hat{D}|lm^\prime>
  !!
  !! the matrix index runs from -m to m
  !!
  Subroutine  wignerDL(l,eulang,D,fac)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: l
    Real(t_real),Intent(in)      :: eulang(3),fac(0:2*l)
    Complex(t_real),Intent(out)  :: D(-l:l,-l:l)
    !!LOCAL
    Integer(t_int) :: m,mp
    Real(t_real)   :: cb,sb
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    cb=Cos(eulang(2)*0.5_t_real)
    sb=Sin(eulang(2)*0.5_t_real)
    !!
    Do m=-l,l
       Do mp=-l,l
          D(m,mp)=Exp(-ci*m*eulang(1))*dlmm (l,m,mp,sb,cb,fac)*Exp(-ci*mp*eulang(3))
       Enddo
    Enddo
    !!
  End Subroutine wignerDL


  !!***********************************************************************
  !! returns the dl,mm'(beta) functions
  !! for integer l
  Real(t_real) Function dlmm(l,m,mp,sb,cb,fac)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: l,m,mp
    Real(t_real),Intent(in)      :: cb,sb,fac(0:2*l)
    !!LOCAL
    Integer(t_int) :: k,k1,k2
    Real(t_real)   :: ck,sk,f,d
    !!
    d=0.0_t_real
    Do k=Max(m+mp,0),Min(l+m,l+mp)
       f=Real((-1_t_int)**k,t_real)/(fac(k)*fac(l+m-k)*fac(l+mp-k)*fac(k-m-mp))

       k1=2_t_int*k-m-mp
       k2=2_t_int*l-k1

       If(k1 == 0_t_int)Then
          ck=1.0_t_real
       Else
          ck=cb**k1
       Endif
       If(k2 == 0_t_int)Then
          sk=1.0_t_real
       Else
          sk=sb**k2
       Endif
       d=d+f*ck*sk
    Enddo
    dlmm=d*Real((-1_t_int)**(l+m),t_real)&
         &*Sqrt(fac(l+m)*fac(l-m)*fac(l+mp)*fac(l-mp))
    !!    
    Return
    !!
  End Function dlmm
  !!***********************************************************************
  Subroutine symtests()
    !!
    Implicit None  
    !!IO
    !!LOCAL
    !!
    !!TODO
    Character(*),Parameter::nsubfunc="symtests"
    Integer(t_int) :: i,j,l,n1,maxl,m,mp,JJ,kap,mu,ind,n,nu,LL,sss,ttt,rrr,su
    Integer(t_int) :: kk,iii,kapq,ml,nl,deg,il,jl,ilm,is
    Real(t_real)   :: f(0:3),E(3,3),eu(3),nx(3),nz(3),ny(3),r(3),v(3),dum,s
    Real(t_real)   :: MM(3,3),fac(0:100),ro(3),wu(1000),theta,phi,thetap,phip
    Real(t_real)   :: Ep(3,3),rp(3),ylm(1000),ylmp(1000),vp(3),d,B(3),BP(3)
    Complex(t_real):: cy(1000),cyp(1000),cs(4),cdlmm(1000),cd(4),cd2(2,2)
    Complex(t_real):: cdjmm(1000),sigx(2,2),sigy(2,2),sigz(2,2),ze(2,2)
    Complex(t_real):: ze1(2,2),ze2(2,2),ss(4,1000),ssp(4,1000),cc,cc2
    Complex(t_real):: ssq(4,1000),sspq(4,1000),cH(20,2,20,2),cUHU(20,2,20,2)
    Complex(t_real):: psi(4,2,1000),psip(4,2,1000)
    Real(t_real)   :: gk,gkq,fk,fkq,hk,hkq,hl,dhl
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    Complex(t_real),Pointer::  p_Uylm(:,:,:)
    Real(t_real),Pointer :: rvylm(:,:,:)
    Real(t_real),Pointer :: rvylmp(:,:,:)
    !!
    Nullify(p_Uylm)
    Nullify(rvylm)
    Nullify(rvylmp)
    !!
    fac(0)=1.0_t_real
    Do m=1,100
       fac(m)=fac(m-1)*m
    Enddo
    !!

    !!
    E(:,1)=(/1.0_t_real,0.0_t_real,0.0_t_real/)
    E(:,2)=(/0.0_t_real,1.0_t_real,0.0_t_real/)
    E(:,3)=(/0.0_t_real,0.0_t_real,1.0_t_real/)
    !!
    Do i=1,3
       Print 1,'E',i,E(:,i)
    Enddo
    !!
    eu(1)=30.0_t_real
    nz(:)=(/-9.0_t_real,13.0_t_real,-5.0_t_real/)
    nz(:)=nz(:)/Sqrt(Sum(nz(:)**2))
    !!    nx(:)=(/Cos(pi/180.0_t_real*al),Sin(pi/180.0_t_real*al),0.0_t_real/)
    nx(:)=(/-2.0_t_real,7.0_t_real,-5.0_t_real/)
    nx(:)=nx(:)/Sqrt(Sum(nx(:)**2))
    nx(:)=nx(:)-nz(:)*Sum(nx(:)*nz(:))
    nx(:)=nx(:)/Sqrt(Sum(nx(:)**2))
    Call cross(nz,nx,ny)
    !!
    Print *
    Print 1,'n',1,nx(:)
    Print 1,'n',2,ny(:)
    Print 1,'n',3,nz(:)
    !!
    theta=20.0_t_real/180.0_t_real*Pi
    phi  =70.0_t_real/180.0_t_real*Pi
    r(:)=(/Sin(theta)*Cos(phi),Sin(theta)*Sin(phi),Cos(theta)/)
    !!
    !! (r_j * e_j)_i = E^T r
    v(:)=Matmul(E(:,:),r(:))
    Print *
    Print 2,'r_'   ,r(:)
    Print 2,'r_e_' ,v(:)
    !!
    !!
    Call reuler2(nz,nx,eu)
    Print *
    Print 2,'al'   ,eu(1)/Pi*180
    Print 2,'be'   ,eu(2)/Pi*180
    Print 2,'ga'   ,eu(3)/Pi*180
    !!
    Call m_euler(MM,eu)
    Print *
    Do i=1,3
       Print 1,'M',i,MM(:,i)
    Enddo    
    !!
    !! transform axis vectors and real vector
    !! De_i =Sum_j  e_j M_ji         D E^T =E^T M
    !! r_i' =Sum_j  r_j M_ji         \vec{r}=E^T r=E^{'T} r^{'}
    !!                               r^{'T}=r^{T} M
    Do i=1,3
       Ep(:,i)=Matmul(E(:,:),MM(:,i))
       rp(i)=Dot_product(r(:),MM(:,i))
    Enddo
    Print *
    Do i=1,3
       Print 1,'E''',i,Ep(:,i)
    Enddo
    !!
    Do i=1,3
       !! (r_j' * e_j')_i
       vp(i)=Sum(rp(:)*Ep(i,:))
    Enddo
    Print *
    Print 2,'r_'''     ,rp(:)
    Print 2,'r_''e_''' ,vp(:)
    !!
    If(Any(Abs(Ep(:,3)-nz(:))>1.e-8_t_real))Then
       Stop 'Ep3/=nz'
    Endif
    If(Any(Abs(Ep(:,2)-ny(:))>1.e-8_t_real))Then
       Stop 'Ep3/=nz'
    Endif
    If(Any(Abs(Ep(:,1)-nx(:))>1.e-8_t_real))Then
       Stop 'Ep3/=nz'
    Endif
    If(Any(Abs(v(:)-vp(:))>1.e-8_t_real))Then
       Stop 'v/=v'''
    Endif
    !!
    !!
    !!
    maxl=13
    Call cxylm(maxl,cy,theta,phi)
    Call ylms13(ylm,r(1),r(2),r(3),maxl)
    l=13
    !Do m=-l,l
    !   Print 3,'cY',l,m,cy(l*(l+1)+1+m)
    ! Enddo
    !!
    Print *
    !!
    Do m=-l,l
       If(m<0)Then
          s=sq2o2*Aimag((-1)**m*cy(l*(l+1)+1+Abs(m))-cy(l*(l+1)+1-Abs(m)))
       Else If(m==0)Then
          s=Real(cy(l*(l+1)+1+0),t_real)
       Else
          s=sq2o2*Real((-1)**m*cy(l*(l+1)+1+Abs(m))+cy(l*(l+1)+1-Abs(m))&
               &,t_real)
       Endif
       If(Abs(s-ylm(l*(l+1)+1+m))>1.e-12_t_real)Then
          Print 3,'rY',l,m,s,ylm(l*(l+1)+1+m),s-ylm(l*(l+1)+1+m)
          Stop 'cY->rY'
       Endif
3      Format(a,i2,',',i3,'=',3(e12.5e2,2x))
    Enddo
    !!
    !!
    
    !!
    Do m=-l,l
       Do mp=-l,l
          If(Abs(dlmm (l,m,mp,Sin(eu(2)),Cos(eu(2)),fac)-dlmm1(l,m,mp,Sin(eu(2))&
               &,Cos(eu(2)),fac))>1.e-12_t_real)Then
             Stop 'dlmm'
          Endif
       Enddo
    Enddo
    !!
    Call  wignerDL(l,eu,cdlmm,fac)
    Call  ytr(eu,0,fac,l,wu,.True.)
    !!
!!$    Do m=1,2*l+1
!!$       Do mp=1,2*l+1
!!$          Write(6,5,advance='no')cdlmm(m+(2*l+1)*(mp-1))
!!$       Enddo
!!$       Write(6,"()")
!!$    Enddo
5   Format(4('(',e12.5e2,',',e12.5e2,')  '))
    !!
    
    !! determine polar coordinates of r' in new system
    Call get_spherical_coordinates(rp,thetap,phip,dum)

    !!
    v(:)=(/Sin(thetap)*Cos(phip),Sin(thetap)*Sin(phip),Cos(thetap)/)
    If(Any(Abs(rp(:)-v(:))>1.e-8_t_real))Then
       Print *,' rp(theta,phi) /= rp ?'
       Stop 'rp(thetat,phi)'
    Endif
    
    !! Ylm(r')
    Call cxylm(maxl,cyp,thetap,phip)
    Call ylms13(ylmp,rp(1),rp(2),rp(3),maxl)
    !!
    Do mp=-l,l
       cd(1)=Cmplx(0.0_t_real,0.0_t_real,t_real)
       d=0.0_t_real
       Do m=-l,l
          cd(1)=cd(1)+cy(l*(l+1)+1+m)*cdlmm(m+l+1+(2*l+1)*(mp+l+1-1))
          d=d+ylm(l*(l+1)+1+m)*wu(m+l+1+(2*l+1)*(mp+l+1-1))
       Enddo
       If(Abs(cd(1)-cyp(l*(l+1)+1+mp))>1.0e-8_t_real)Then
          Print *,'mp=',mp, cyp(l*(l+1)+1+mp)-cd(1)
          Stop 'cY(r'')/=cY(r)'
       Endif
       If(Abs(d-ylmp(l*(l+1)+1+mp))>1.0e-8_t_real)Then
          Print *,'mp=',mp, ylmp(l*(l+1)+1+mp),d
          Stop 'rY(r'')/=rY(r)'
       Endif

    Enddo
    !!
    !!
    !! spin transform
    !!
    !!
    JJ=2*1/2
    Call wignerD2(eu,cd2)
    Call wignerDJ(JJ,eu,cdjmm,fac,.False.)
    Do m=0,JJ
       Do mp=0,JJ
          If(Abs(cd2(JJ-m+1,JJ-mp+1)-cdjmm(m+1+(JJ+1)*(mp+1-1)))&
               &>1.e-12_t_real)Then
             Write(6,51,advance='no')-JJ+2*m,-JJ+2*mp&
                  &,cd2(JJ-m+1,JJ-mp+1)-cdjmm(m+1+(JJ+1)*(mp+1-1))
             Stop 'djmm/=d2'
          Endif
       Enddo
       !Write(6,"()")
    Enddo
51  Format(i3,'/2',i3,'/2',1x,4('(',e12.5e2,',',e12.5e2,')  '))
    !!
    !!
    !! Pauli matricess
    !!
    sigx(1,:)=(/c0,c1/)
    sigx(2,:)=(/c1,c0/)
    sigy(1,:)=(/c0,-ci/)
    sigy(2,:)=(/ci,c0/)
    sigz(1,:)=(/c1,c0/)
    sigz(2,:)=(/c0,-c1/)
    !!
    B(:)=(/17.0_t_real,-4.0_t_real,-6.1_t_real/)
    !! B=nz
    !!
    !! <t\Sigma | sig.B |s\Sigma >  
    ze(:,:)=sigx(:,:)*B(1)+sigy(:,:)*B(2)+sigz(:,:)*B(3)
    !!
    Do m=1,2
       Do mp=1,2
          Write(6,5,advance='no')ze(m,mp)
       Enddo
       Write(6,"()")
    Enddo
    !!
    !! B in \Sigma^\prime is BM
    Do i=1,3
       BP(i)=Dot_product(B(:),MM(:,i))
    Enddo
    !!
    !! <t\Sigma| (M sig).BM |s\Sigma>  
    !!
    !! transform sigma
    !! sig_i' =Sum_j M_ji sig_j
    ze1(:,:)=c0
    Do i=1,3
       ze1(:,:)=ze1(:,:)&
            &+(sigx(:,:)*MM(1,i)+sigy(:,:)*MM(2,i)+sigz(:,:)*MM(3,i))*BP(i)
    Enddo
    !!
    Do m=1,2
       Do mp=1,2
          If(Abs(ze1(m,mp)-ze(m,mp))>1.0e-12_t_real)Then
             Write(6,5,advance='no')ze1(m,mp)-ze(m,mp)
             Stop 'ze1/=ze'
          Endif

       Enddo
       !Write(6,"()")
    Enddo
    !!
    !! B in \Sigma^\prime is B*M
    !! D sig.B.M D^+  
    ze2=c0
    ze2=ze2+Matmul(Matmul(cd2(:,:),sigx(:,:)),Conjg(Transpose(cd2(:,:))))*BP(1)
    ze2=ze2+Matmul(Matmul(cd2(:,:),sigy(:,:)),Conjg(Transpose(cd2(:,:))))*BP(2)
    ze2=ze2+Matmul(Matmul(cd2(:,:),sigz(:,:)),Conjg(Transpose(cd2(:,:))))*BP(3)
    Do m=1,2
       Do mp=1,2
          If(Abs(ze2(m,mp)-ze(m,mp))>1.0e-12_t_real)Then
             Write(6,5,advance='no')ze2(m,mp)-ze(m,mp)
             Stop 'ze2/=ze'
          Endif
       Enddo
       !Write(6,"()")
    Enddo
    !!--------------------------------------------------
    Print *,'Real Vector spherical harmonics'
    !!
    maxl=12
    !!
    Call palloc('rvylm',rvylm,1_t_int,3_t_int,1_t_int,2_t_int&
         &,1_t_int,(maxl+1)**2,nmodule,nsubfunc)
    Call palloc('rvylmp',rvylmp,1_t_int,3_t_int,1_t_int,2_t_int&
         &,1_t_int,(maxl+1)**2,nmodule,nsubfunc)
    !!
    Call calc_real_vec_Ylm(r,maxl,rvylm)
    Call calc_real_vec_Ylm(rp,maxl,rvylmp)
    !!
    !!
    !!
    Do l=0,maxl
       !!
       deg=2*l+1
       !!
       Call  ytr(eu,0,fac,l,wu,.True.)
       !!
       Print 2,"er"     ,-r(:)/Sqrt(Dot_product(r,r))/sq4pi
       Print 2,"er_'"     ,-rp(:)/Sqrt(Dot_product(rp,rp))/sq4pi
       Print 2,"M"     ,MM(1,:)
       Print 2,"M"     ,MM(2,:)
       Print 2,"M"     ,MM(3,:)
       !!
       Do is=-1,1,2
          If(l+is>=0)Then
             Do m=-l,l
                ilm=l*(l+1)+1
                !! transform R(rR)^T= [ R(r)^T D^l ] M
                !!
                B=0.0_t_real
                Do n=-l,l
                   B=B+rvylm(1:3,(is+3)/2,ilm+n)*wu(n+l+1+deg*(m+l+1-1))
                Enddo
                B=Matmul(B,MM)
                !!
                !!
                Write(outfile,89) m,ilm+m,l+is,l,rvylm(:,(is+3)/2,ilm+m)&
                     &,rvylmp(:,(is+3)/2,ilm+m)
                !Print *,B(1:3)
                If(Any(Abs(B-rvylmp(:,(is+3)/2,ilm+m))>1.0e-12_t_real)) Then
                   Print *,B(1:3)
                   Stop 'rotated real vector harmonics wrong'
                Endif
             Enddo
          Endif
          Print *
       Enddo
    Enddo
    !!
89  Format('m=',i3,' ilm=',i5,' R^',i2.2,'_',i2.2,'=',3(f10.5,2x)&
         &,'  R(rR)=',3(f10.5,2x))
    !!
    !!
    !!
    !!
    !!
    Print *,'FINISH: real vector spherical harmonics'
    !Stop 'FINISH: real vector spherical harmonics'
    !!--------------------------------------------------
    !! spherical spinors
    Print *,'spherical spinors'
    !!
    maxl=12
    l=5
    !!
    kap=-1
    JJ=2*Abs(kap)-1
    LL=(Abs(2*kap+1)-1)/2


    Do m=0,JJ
       mu=-JJ+2*m
       
       Call spherical_spinor(kap,mu,cy,ss(1:2,m+1))
       Call spherical_spinor(-kap,mu,cy,ss(3:4,m+1))
      
       Call spherical_spinor(kap,mu,cyp,ssp(1:2,m+1))
       Call spherical_spinor(-kap,mu,cyp,ssp(3:4,m+1))
    Enddo
    !! 
    Do m=0,JJ
       mu=-JJ+2*m
       Write(6,61)JJ,mu,ss(1:4,m+1)
    Enddo
61  Format(i3,'/2',i3,'/2',1x,4(t12,'(',e12.5e2,',',e12.5e2,')',/))
         
    
    !!
    Call wignerDJ(JJ,eu,cdjmm,fac,.False.)
    !!
    Do mp=0,JJ
       ind=(JJ+1)*(mp+1-1)
       Do i=1,4
          !! chi(r) * D^j
          cd(i)=Sum(ss(i,1:JJ+1)*cdjmm(1+ind:JJ+1+ind))
       Enddo
       !! (D^1/2)^+ chi(r) D^j == chi ( r R )
       cs(1:2)=Matmul(Transpose(Conjg(cd2(:,:))),cd(1:2))
       cs(3:4)=Matmul(Transpose(Conjg(cd2(:,:))),cd(3:4))
       If(Any(Abs(cs(1:2)-ssp(1:2,mp+1))>1.0e-12_t_real))Then
          mu=-JJ+2*mp
          Write(6,61)JJ,mu,cs(1)-ssp(1,mp+1),cs(2)-ssp(2,mp+1)
          Stop 'shpspin(a r)/=shpspin'
       Endif
       If(Any(Abs(cs(3:4)-ssp(3:4,mp+1))>1.0e-12_t_real))Then
          mu=-JJ+2*mp
          Write(6,61)JJ,mu,cs(3)-ssp(3,mp+1),cs(4)-ssp(4,mp+1)
          Stop 'shpspin(a r)/=shpspin'
       Endif
    Enddo
    !!
    !!
    !! weird
    kap=-3
    kapq=-kap-1
    JJ=2*Abs(kap)-1
    LL=(Abs(2*kap+1)-1)/2
    deg=2*LL+1

    Call wignerD2(eu,cd2)
    Call wignerDL(LL,eu,cdlmm,fac)

    gk=1.23_t_real
    gkq=-9.75_t_real
    !gk=1.0_t_real
    !gkq=1.0_t_real

    fk=6.39_t_real
    fkq=2.823_t_real
    !fk=0.0_t_real
    !fkq=0.0_t_real
    Do m=0,JJ
       mu=-JJ+2*m
       
       !! unrotated
       Call spherical_spinor(kap,mu,cy,ss(1:2,m+1))
       Call spherical_spinor(-kap,mu,cy,ss(3:4,m+1))
       Call spherical_spinor(kapq,mu,cy,ssq(1:2,m+1))
       Call spherical_spinor(-kapq,mu,cy,ssq(3:4,m+1))


      

       !! rotated
       Call spherical_spinor(kap,mu,cyp,ssp(1:2,m+1))
       Call spherical_spinor(-kap,mu,cyp,ssp(3:4,m+1))
       Call spherical_spinor(kapq,mu,cyp,sspq(1:2,m+1))
       Call spherical_spinor(-kapq,mu,cyp,sspq(3:4,m+1))

       ssp(1:2,m+1)=Matmul(cd2(:,:),ssp(1:2,m+1))
       ssp(3:4,m+1)=Matmul(cd2(:,:),ssp(3:4,m+1))
       sspq(1:2,m+1)=Matmul(cd2(:,:),sspq(1:2,m+1))
       sspq(3:4,m+1)=Matmul(cd2(:,:),sspq(3:4,m+1))


       Print *,'mu=',mu,'/2'
       Print *,'chi',ss(1:2,m+1)
       Print *,'chiq',ssq(1:2,m+1)
    Enddo


    !!
    psi(:,:,:)=c0
    psip(:,:,:)=c0
    Do ml=1,2*LL+1
       Do sss=1,2
          
          Do m=0,JJ
             mu=-JJ+2*m

             If((mu-3+2*sss)/2 == ml-LL-1 )Then
                Print *,'ml=',ml,'s=',sss,'mu=',mu,cgoOFKapMu(sss,kap,mu),cgoOFKapMu(sss,kapq,mu)
                psi(1:2,sss,ml)=psi(1:2,sss,ml)&
                     &+gk *ss (1:2,m+1)*cgoOFKapMu(sss,kap,mu)&
                     &+gkq*ssq(1:2,m+1)*cgoOFKapMu(sss,kapq,mu)
                Print *,'psi=',psi(1:2,sss,ml)
                psi(3:4,sss,ml)=psi(3:4,sss,ml)&
                     &+gk *ss (3:4,m+1)*cgoOFKapMu(sss,kap,mu)&
                     &+gkq*ssq(3:4,m+1)*cgoOFKapMu(sss,kapq,mu)

                psip(1:2,sss,ml)=psip(1:2,sss,ml)&
                     &+gk *ssp (1:2,m+1)*cgoOFKapMu(sss,kap,mu)&
                     &+gkq*sspq(1:2,m+1)*cgoOFKapMu(sss,kapq,mu)
                psip(3:4,sss,ml)=psip(3:4,sss,ml)&
                     &+gk *ssp (3:4,m+1)*cgoOFKapMu(sss,kap,mu)&
                     &+gkq*sspq(3:4,m+1)*cgoOFKapMu(sss,kapq,mu)

                Print *,'psip=',psip(1:2,sss,ml)

             Endif
          Enddo
       Enddo
    Enddo
    !!
    !!
    Do ml=1,2*LL+1
       Do sss=1,2
          Print *,'ml=',ml,'s=',sss,'mu=',mu

          cs(1:4)=c0
          Do ttt=1,2
             Do nl=1,2*LL+1
             Print *,'nm=',nl,ml,cdlmm(nl+(2*LL+1)*(ml-1))

                cs(1:4)=cs(1:4)+psi(1:4,ttt,nl)*cd2(ttt,sss)*&
                     & cdlmm(nl+(2*LL+1)*(ml-1))
             Enddo
          Enddo

          Print *,'cs=',cs(1:2)
          Print *,'psip=',psip(1:2,sss,ml)

          If(Any(Abs(cs(1:4)-psip(1:4,sss,ml))>1.0e-12_t_real))Then
             Write(6,161)ml,sss,cs(1:4)-psip(1:4,sss,ml)
161          Format(i3,' ',i3,' ',1x,4(t12,'(',e12.5e2,',',e12.5e2,')',/))

             Print *,'weird wrong'
             Stop 'weird wrong'
          Endif
          
       Enddo
    Enddo
    !!
    Print *,'after weird'
    !!
    hk=1.0_t_real
    hkq=1.0340_t_real
    !!
    hl=hk*Real(LL+1,t_real)/deg+hkq*Real(LL,t_real)/deg
    dhl=hk-hkq
    !!
    cH=c0
    Do nl=1,2*LL+1
       Do sss=1,2
          Do ml=1,2*LL+1
             Do ttt=1,2
                mu=2*(nl-LL-1)+(3-2*sss)
                If( Abs(mu)<=JJ  .And. (ml -LL-1 == nl-LL-1+(ttt-sss)) )Then
                   cH(nl,sss,ml,ttt)=cH(nl,sss,ml,ttt)&
                        &+cgoOFKapMu(sss,kap,mu)*hk*cgoOFKapMu(ttt,kap,mu)&
                        &+cgoOFKapMu(sss,kapq,mu)*hkq*cgoOFKapMu(ttt,kapq,mu)
                Endif
             Enddo
          Enddo
       Enddo
    Enddo
    !!
    Do sss=1,2
       Do nl=1,2*LL+1
          Do ttt=1,2
             Do ml=1,2*LL+1
                Write(outfile,123,advance='no') Real(cH(nl,sss,ml,ttt),t_real)
             Enddo
             Write(outfile,124,advance='no')
          Enddo
          Write(outfile,*)
       Enddo
       Write(outfile,125)
    Enddo
123 Format(f10.5,' ')
124 Format('  |  ')
125 Format(72('-'))
    !! direct matrix elements
    Do nl=1,2*LL+1
       Do sss=1,2
          su=3-2*sss
          Do ml=1,2*LL+1
             Do ttt=1,2
                mu=2*(nl-LL-1)+su
                
                cc=c0
                If( sss == ttt .And. nl==ml )Then
                   cc=hl+dhl*su*(nl-LL-1)/deg
                Endif
                !Print *,nl,sss,ml,ttt,(ml-LL-1==nl-LL-1+su)&
                !     &,Abs(2*(nl-LL-1)+su) <= 2*LL-1,nl-LL-1==su*LL


                If( su == -(3-2*ttt) .And. (ml-LL-1==nl-LL-1+su) &
                     &.And. Abs(2*(nl-LL-1)+su) <= 2*LL-1 )Then
                   cc=dhl*Sqrt(Real(deg**2-(2*(nl-LL-1)+su)**2,t_real))/deg/2
                Endif
                
                If(Abs(cc-cH(nl,sss,ml,ttt))>1.0e-12_t_real)Then
                   Write(6,261)nl,sss,ml,ttt,cc,cH(nl,sss,ml,ttt)
261                Format(4(i3,' '),1x,2(t22,'(',e12.5e2,',',e12.5e2,')',/))
                   
                   Print *,'cH direct wrong'
                   stop 'cH direct wrong'
                Endif
                
             Enddo
          Enddo
       Enddo
    Enddo
    !!
    !!
    !! transformation properties
    !!
    Do nl=1,deg
       Do sss=1,2
          Do ml=1,deg
             Do ttt=1,2
                cc2=c0
                Do il=1,deg
                   Do jl=1,deg
                      cc=Dot_product(cd2(1:2,sss),Matmul(cH(il,1:2,jl,1:2),cd2(1:2,ttt)))
                      cc2=cc2+Conjg(cdlmm(il+(2*LL+1)*(nl-1))) * cc * cdlmm(jl+(2*LL+1)*(ml-1))
                   Enddo
                Enddo

                If(Abs(cc2-cH(nl,sss,ml,ttt))>1.0e-12_t_real)Then
                   Write(6,261)nl,sss,ml,ttt,cc2,cH(nl,sss,ml,ttt)
                   
                   Print *,'cH transform wrong'
                   stop 'cH transform wrong'
                Endif
             End Do
          End Do
       End Do
    End Do
    !!
    !!
    !!
    !!
    Call palloc('p_Uylm',p_Uylm&
         &,1_t_int,2*LL+1&
         &,1_t_int,2*LL+1&
         &,0_t_int,LL&
         &,nmodule,nsubfunc)
    !!
    Call  make_ylm_U_matrix(LL,p_Uylm)
    !!
    !!
    Do sss=1,2
       Do ttt=1,2
          cUHU(1:deg,sss,1:deg,ttt)=Matmul(p_Uylm(1:deg,1:deg,LL)&
               &,Matmul(cH(1:deg,sss,1:deg,ttt),Transpose(Conjg(p_Uylm(1:deg,1:deg,LL)))))
       Enddo
    Enddo
    !!
    Print *,'UHU+'
    Do sss=1,2
       Do nl=1,2*LL+1
          Do ttt=1,2
             Do ml=1,2*LL+1
                Write(outfile,123,advance='no') Real(cUHU(nl,sss,ml,ttt),t_real)
             Enddo
             Write(outfile,124,advance='no')
          Enddo
          Write(outfile,*)
       Enddo
       Write(outfile,125)
    Enddo
    Print *,'imag UHU+'
    Do sss=1,2
       Do nl=1,2*LL+1
          Do ttt=1,2
             Do ml=1,2*LL+1
                Write(outfile,123,advance='no') Aimag(cUHU(nl,sss,ml,ttt))
             Enddo
             Write(outfile,124,advance='no')
          Enddo
          Write(outfile,*)
       Enddo
       Write(outfile,125)
    Enddo
    !!
    !!
    If(Any(Abs(cUHU(1:deg,1,1:deg,1)-Conjg(cUHU(1:deg,2,1:deg,2)))>1.0e-12_t_real))Then
       !Write(6,161)ml,sss,cs(1:4)-psip(1:4,sss,ml)
       
       Print *,'H11=H22*'
       Stop 'H11=H22*'
    Endif
    If(Any(Abs(cUHU(1:deg,1,1:deg,2)+Transpose(cUHU(1:deg,1,1:deg,2)))>1.0e-12_t_real))Then
       !Write(6,161)ml,sss,cs(1:4)-psip(1:4,sss,ml)
       
       Print *,'H12+H12T=0'
       Stop 'H12+H12T=0'
    Endif
    !!
    Print *,'after weird Hamiltonian'
    !!

    Print *,'D formulas'
    Do kap=-4,4
       If(kap==0 ) Cycle
       JJ=2*Abs(kap)-1
       LL=(Abs(2*kap+1)-1)/2
       Call wignerD2(eu,cd2)
       Call wignerDL(LL,eu,cdlmm,fac)
       Call wignerDJ(JJ,eu,cdjmm,fac,.False.)
       !!
       Do n=0,JJ
          nu=-JJ+2*n
          Do m=0,JJ
             mu=-JJ+2*m
             cc=c0
             Do sss=1,2
                i=(nu-(3-2*sss))/2
                Do ttt=1,2
                   j=(mu-(3-2*ttt))/2
                   If( (Abs(i)<=LL) .And. (Abs(j)<=LL)) Then
                      !Print *,nu,mu,sss,ttt,i,j
                      !Print *,cgoOFKapMu(sss,kap,nu)
                      !Print *,cgoOFKapMu(ttt,kap,mu)
                      !Print *,(i+LL+1+(2*LL+1)*(j+LL+1-1)),cdlmm(i+LL+1+(2*LL+1)*(j+LL+1-1))
                      cc=cc+cd2(sss,ttt)*cgoOFKapMu(sss,kap,nu)&
                           &*cdlmm(i+LL+1+(2*LL+1)*(j+LL+1-1))&
                           &*cgoOFKapMu(ttt,kap,mu)
                   Endif
                Enddo ! ttt
             Enddo ! sss
             ind=(JJ+1)*(m+1-1)
             
             If(Abs(cc-cdjmm(n+1+ind))>1.0e-12_t_real)Then
                Write(6,62)JJ,nu,mu,cc,cdjmm(n+1+ind)
                Stop 'Dj/=DcDlc'
             Endif
62           Format(i3,'/2',i3,'/2',i3,'/2',1x,2(t12,'(',e12.5e2,',',e12.5e2,')',/))
             
          Enddo ! m
       Enddo ! n



       !!
       !Print *,'hier',kap
       Do n=0,JJ
          nu=-JJ+2*n
          Do m=0,JJ
             mu=-JJ+2*m
             Do sss=1,2
                cc=c0
                Do ttt=1,2
                   i=(nu-(3-2*ttt))/2
                   j=(mu-(3-2*sss))/2
                   If( (Abs(i)<=LL) .And. (Abs(j)<=LL)) Then
                      cc=cc+cd2(ttt,sss)*cgoOFKapMu(ttt,kap,nu)&
                           &*cdlmm(i+LL+1+(2*LL+1)*(j+LL+1-1))
                   Endif
                Enddo ! ttt
                ind=(JJ+1)*(m+1-1)
                cc2=cdjmm(n+1+ind)*cgoOFKapMu(sss,kap,mu)
                If(Abs(cc-cc2)>1.0e-12_t_real)Then
                   Write(6,62)JJ,nu,mu,cc,cc2
                   Stop 'Djc/=DlDsc'
                Endif
             Enddo ! sss
          Enddo ! m
       Enddo ! n




       Do n=0,JJ
          nu=-JJ+2*n
          Do m=0,JJ
             mu=-JJ+2*m
             Do rrr=1,2
                cc=c0
                cc2=c0
                
                Do sss=1,2
                   i=(nu-(3-2*sss))/2
                   Do ttt=1,2
                      j=(mu-(3-2*ttt))/2
                      If( (Abs(i)<=LL) .And. (Abs(j)<=LL)) Then
                         dum=   cgoOFKapMu(ttt,kap,mu)&
                              &*cgoOFKapMu(rrr,kap,mu)&
                              &*cgoOFKapMu(sss,kap,nu)
                         
                         cc=cc+cd2(sss,ttt)&
                              &*cdlmm(i+LL+1+(2*LL+1)*(j+LL+1-1))&
                              &*dum
                      Endif
                   Enddo ! ttt
                Enddo ! sss
                
                
                Do sss=1,2
                   i=(nu-(3-2*sss))/2
                   j=(mu-(3-2*rrr))/2
                   If( (Abs(i)<=LL) .And. (Abs(j)<=LL)) Then
                      cc2=cc2+cd2(sss,rrr)*cdlmm(i+LL+1+(2*LL+1)*(j+LL+1-1))&
                           &*cgoOFKapMu(sss,kap,nu)
                   Endif
                Enddo
                
                
                If(Abs(cc-cc2)>1.0e-12_t_real)Then
                   Write(6,62)JJ,nu,mu,cc,cc2
                   Stop 'Dj/=DcDlc'
                Endif

             Enddo ! rrr
             


             

          Enddo ! m
       Enddo ! n


    Enddo ! kap
    !!
    !!
    
!!$    !! prepare ytr input, max l=3 (f-states)
!!$    fac(0_t_int)=1.0_t_real
!!$    Do j=1_t_int,7_t_int
!!$       fac(j)=Real(j,t_real)*fac(j-1_t_int)
!!$    Enddo
!!$    !! calc rotation matrices
!!$    l=1
!!$    n1=l*(l*l*4_t_int-1_t_int)/3_t_int
!!$    Call ytr(al,be,ga,0_t_int,fac,l,wu(n1+1_t_int),.False.)
!!$    
!!$    !! controll output
!!$    l=1_t_int
!!$    n1=l*(l*l*4_t_int-1_t_int)/3_t_int
!!$    Write(outfile,400)nz(:)
!!$400 Format('z-Vector: ',3(f16.7,2x))
!!$402 Format('x-Vector: ',3(f16.7,2x))
!!$401 Format('becomes : ',3(f16.7,2x))
!!$    r(:)=nz(:)
!!$    Call rotxyz(r,wu(n1+1_t_int),ro)
!!$    Write(outfile,401)ro(1_t_int),ro(2_t_int),ro(3_t_int)
!!$    !!     
!!$    Write(outfile,402)nx(:)
!!$    r(:)=nx(:)
!!$
!!$    Call rotxyz(r,wu(n1+1_t_int),ro)
!!$    Write(outfile,401)ro(1_t_int),ro(2_t_int),ro(3_t_int)

    !!
    Call pfree(rvylm)
    Call pfree(rvylmp)
    !!
1   Format(a,i1,'=',3(f10.5,2x))
2   Format(a,'=',3(f10.5,2x))
    Call terminate(TERM_FINISH,Trim(nsubfunc)//': symtest!')
    !!
    !!
  End Subroutine symtests
  !!***********************************************************************
  Integer(t_int) Function  delta(i,j)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)    :: i,j
    !!LOCAL
    !!
    !!
    delta=0_t_int
    If(i==j)delta=1_t_int
    !!
    Return
  End Function delta
  !!***********************************************************************
  Subroutine  mag_rel_tests()
    !!
    Implicit none  
    !!IO
    !!LOCAL
    Character(*),Parameter::nsubfunc="mag_rel_tests"
    Integer(t_int) :: kap,j,l,mua,mub,ma,mb,degj,maxl,ind,i
    Real(t_real)   :: s,s1,theta,phi,thetap,phip,fac(0:100)
    Real(t_real)   :: tol,eu(3),nx(3),nz(3),MM3(3,3),r(3),rp(3),dum
    Real(t_real),Pointer   :: Dti(:,:)
    Real(t_real),Pointer   :: MM(:,:)
    Complex(t_real),Pointer   :: cMM(:,:)
    Complex(t_real),Pointer   :: cdJ(:,:)
    Complex(t_real):: cy(1000),ss(4,1000),ss1(4,1000),ss2(4,1000),sigma(2,2,3)
    Complex(t_real):: cyp(1000),ssp(4,1000)
    Complex(t_real):: cdjmm(1000),cd2(2,2)!,cdlmm(1000)
    Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
    Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
    !!
    maxl=12
    Call make_factorials(fac,2*maxl+1)
    !!
    Nullify(Dti)
    Nullify(cdJ)
    Nullify(MM)
    Nullify(cMM)
    !!
    !! Pauli matricess
    !!
    sigma(1,:,1)=(/c0,c1/)
    sigma(2,:,1)=(/c1,c0/)
    sigma(1,:,2)=(/c0,-ci/)
    sigma(2,:,2)=(/ci,c0/)
    sigma(1,:,3)=(/c1,c0/)
    sigma(2,:,3)=(/c0,-c1/)
    !!
    tol=1.0e-10_t_real
    !! time reversal
    !!
    Do kap=-4,4,1
       If(kap==0) Cycle
       j=(2*Abs(kap)-1)
       l=(Abs(2*kap+1)-1)/2
       
       degj=j+1
       Print *,'degj=',degj
       Call palloc('dti',Dti,1_t_int,degj,1_t_int,degj,nmodule,nsubfunc)
       Call palloc('MM',MM,1_t_int,degj,1_t_int,degj,nmodule,nsubfunc)
       MM=0.0_t_real
       Do ma=1,j+1
          MM(ma,ma)=1.0_t_real
       Enddo
       
       Do ma=1,j+1
          mua=2*ma-j-2
          Do mb=1,j+1
             mub=2*mb-j-2
             !!
             !Print *,'mua=',mua
             !Print *,'mub=',mub
             !Print *,'delta=',delta(mua,-mub)
             s=(-1.0_t_real)**((mub+1)/2)*Sign(1_t_int,kap)
             !s1=(-1.0_t_real)**(l+(j-mub)/2)
             Dti(ma,mb)=delta(mua,-mub)*s
             !If(s1 /= s) Then
             !   Print *,'j+l-mu=',s1,s
             !   Stop's ?'
             !Endif
          Enddo
       End Do
       !!
       !Call printmat1('Dtime',"(f10.5,2x)","('      0     ')",Dti,degj,degj,tol)
       MM=Dti+Transpose(Dti)
       If(Any(Abs(MM)>tol))Then
          Call printmat1('MM',"(f10.5,2x)","('      0     ')",MM,degj,degj,tol)
          Stop 'Dti*Dti /= -1'
       Endif
       !!
       !!
       !!
       theta=20.0_t_real/180.0_t_real*Pi
       phi  =70.0_t_real/180.0_t_real*Pi
       r(:)=(/Sin(theta)*Cos(phi),Sin(theta)*Sin(phi),Cos(theta)/)
       !!
       Call cxylm(maxl,cy,theta,phi)

       !!
       Do ma=0,j
          mua=2*ma-j
          Call spherical_spinor(kap,mua,cy,ss(1:2,ma+1))
          !!
          ss1(1:2,ma+1)=-ci*Matmul(sigma(1:2,1:2,2),Conjg(ss(1:2,ma+1)))

          !!
       Enddo
       !!
       ss2(1:2,1:degj)=Matmul(ss(1:2,1:degj),Dti(1:degj,1:degj))
       !!
       !Call cprintmat1('ss1',"(2(f10.5,2x))","('      0           0     ')"&
       !     &,ss1(1:2,1:degj),2,degj,tol)
       !Call cprintmat1('ss2',"(2(f10.5,2x))","('      0           0     ')"&
       !     &,ss2(1:2,1:degj),2,degj,tol)

       If(Any(Abs(ss2(1:2,1:degj)-ss1(1:2,1:degj))>tol))Then
          Call cprintmat1('ss',"(2(f10.5,2x))","('      0           0     ')"&
               &,ss2(1:2,1:degj),2,degj,tol)
       Endif
       !!
       !!
       !!
       nz(:)=(/-9.0_t_real,13.0_t_real,-5.0_t_real/)
       nx(:)=(/-2.0_t_real,7.0_t_real,-5.0_t_real/)
       nz(:)=nz(:)/Sqrt(Sum(nz(:)**2))
       nx(:)=nx(:)/Sqrt(Sum(nx(:)**2))
       nx(:)=nx(:)-nz(:)*Sum(nx(:)*nz(:))
       nx(:)=nx(:)/Sqrt(Sum(nx(:)**2))
       !!
       Call reuler2(nz,nx,eu)
       !!
       Call m_euler(MM3,eu)
       !!
       rp=Matmul(r,MM3)
       Call get_spherical_coordinates(rp,thetap,phip,dum)
       !!
       Call cxylm(maxl,cyp,thetap,phip)
       !!
       Call wignerD2(eu,cd2)
       !!
       Do ma=0,j
          mua=2*ma-j
          Call spherical_spinor(kap,mua,cyp,ssp(1:2,ma+1))
          !!
          ssp(1:2,ma+1)=Matmul(cd2,ssp(1:2,ma+1))
       Enddo

       !Call wignerDL(l,eu,cdlmm,fac)
       Call palloc('cdJ',cdJ,0_t_int,j,0_t_int,j,nmodule,nsubfunc)
       
       Call wignerDJ(j,eu,cdJ,fac,.False.)

           !!
       Do ma=0,j
          ind=(j+1)*(ma+1-1)
          Do i=1,2
             !! chi(r) * D^j
             ss2(i,ma+1)=Sum(ss(i,1:j+1)*cdJ(0:j,ma))
          Enddo
       Enddo
       !!

       If(Any(Abs(ss2(1:2,1:degj)-ssp(1:2,1:degj))>tol))Then
          Call cprintmat1('ssp',"(2(f10.5,2x))","('      0           0     ')"&
               &,ssp(1:2,1:degj),2,degj,tol)
          Call cprintmat1('ss2',"(2(f10.5,2x))","('      0           0     ')"&
               &,ss2(1:2,1:degj),2,degj,tol)
          Stop 'D1/2 chi(r D) /= chi(r) D^j' 
       Endif
       !!
       Call palloc('cMM',cMM,1_t_int,degj,1_t_int,degj,nmodule,nsubfunc)
       !!
       cMM=Matmul(Dti(1:degj,1:degj),cdJ(0:j,0:j))&
            &-Matmul(Conjg(cdJ(0:j,0:j)),Dti(1:degj,1:degj))
       !!
       If(Any(Abs(cMM)>tol))Then
          Call cprintmat1('cMM',"(2(f10.5,2x))","('      0                 ')"&
               &,cMM,degj,degj,tol)
          Stop 'Dti*Dti /= -1'
       Endif

       !!
    Enddo
    !!
    Call pfree(cdJ)
    Call pfree(MM)
    Call pfree(cMM)
    Call pfree(Dti)
    !!
    Call terminate(TERM_FINISH,Trim(nsubfunc)//': mag_rel_tests!')
    !!
  End Subroutine mag_rel_tests
    !!***********************************************************************
End Module symtest
