Module tatom
  !!
  Use precisiondefs
  Use memory
  Use titeration
  Use tmesh
  Use finuc
  !!
  Implicit none  
  !!
  Private
  !!
  Public :: T_Atom,init_tatom,clean_tatom,copy_atom
  !!
  Character(*),Parameter::nmodule="TATOM"
  !!
  !!***********************************************************************
  !!
  !!
  !!
  !! the data type for the atom solutions
  Type T_Atom
     !! 
     !! ---------------------------------------------------------
     !! input data
     !! ---------------------------------------------------------
     !!
     !! number of spin directions
     Integer(t_int) :: mspin
     !! element
     Character(2)   :: element
     !! nuclear charge, ionocity
     Real(t_real)   :: Z,ionicity,Magmom
     !! for use in fplo
     Logical(t_log) :: vca
     !! type of finite nucleus
     Integer(t_int) :: nuc_type
     !!
     Integer(t_int) :: relativistic
     !! use sonfining potential
     Logical(t_log) :: compr
     !! compression parameter and exponent
     Real(t_real)   :: r0,exn0
     !! wave function tolerance
     Real(t_real)   :: wftol
     !! 
     Real(t_real)   :: xreduce
     !!
     Logical(t_log) :: force_pos_spin
     !! don't calculated states where the occus are zero
     !! if this option is not set, then there is a test of the
     !! occus and energies with respect to the aufbau principle
     Logical(t_log) :: noUnOccupied
     Logical(t_log) :: sic,sic4f
     Logical(t_log) :: standalone
     !! extract a leading power of r ?
     Logical(t_log) :: gam_factor
     !!
     !!
     !! options
     !! output is numbered with iterationstep or overwrite each time 
     Logical(t_log) :: overwriteout
     !! output of orbitals,density  and potential
     Logical(t_log) :: basout,rhoout,potout
     !!
     !!
     Type(T_mesh) :: mesh
     !!
     Type(T_iteration)  :: iteration_control
     !!
     !! define basis and non-rel occus
     Character(200)   :: basis_def
     Character(200)   :: noccus_def
     Character(200)   :: magorder_def
     !! ---------------------------------------------------------
     !! internal data
     !! ---------------------------------------------------------
     !!
     !!
     !! non relativistic
     !!
     !! number of wave function componenet
     !!nonrel    : 2 (function and derivative)
     !!khsrel    : 3 (large and small and small delta)
     !!rel nonpol: 2 (large and small)
     !!rel pol   : 4 (large and small for kapa, large and small for \bar{kapa})
     Integer(t_int) :: ncomp
     !! number of states
     Integer(t_int) :: nv
     !! main quantum numbers
     Integer(t_int),Pointer :: p_nnv(:)! nv
     !! angular momentum
     Integer(t_int),Pointer :: p_lv(:) ! nv
     !! magnetic order  for vatom.f90
     Integer(t_int),Pointer :: p_magorder(:) ! nv
     !! non relativistic occupation numbers
     Real(t_real),Pointer   :: p_nroccu(:,:) ! nv,2
     !! maximum orbital momentum
     Integer(t_int) :: lmax
     !!
     Logical(t_log) :: nonspheredens !! shall nonspherical density be 
     !! calculated and writen to file
     !! p_gsum(l,ms,iv) is occupation factor of orbital iv,ms
     !! for density 2*l,0-component
     !! 
     Real(t_real),Pointer   :: p_gsum(:,:,:) ! 0:lmax,2,nv
     !!
     !! non relativistic
     !!
     !! number of relativistic radial orbitals
     Integer(t_int)         :: nRv
     !! max radial degeneracy of relativistic orbitals
     Integer(t_int)         :: degRvmx
     !! radial degeneracy of relativistic orbitals
     Integer(t_int),Pointer :: p_degRv(:) ! nRv
     !! main qn.  of relativistic orbitals
     Integer(t_int),Pointer :: p_nnRv(:) ! nRv
     !! angular momentum of first large component of relativistic orbitals
     Integer(t_int),Pointer :: p_lRv(:) ! nRv
     !! kappa qn.  of relativistic orbitals
     Integer(t_int),Pointer :: p_kapRv(:) ! nRv
     !! mu qn.  of relativistic orbitals
     Integer(t_int),Pointer :: p_muRv(:,:) ! degRvmx,nRv
     !! map relativistic onto non relativistic index
     Integer(t_int),Pointer :: p_ivrTOiv(:) ! nRv
     !! relativistic occupation numbers of distinct radial subshell
     !! that is j-shells for non magnetic
     !! and  kapa,mu-shells for magnetic
     Real(t_real),Pointer   :: p_roccu(:) ! nRv

     !! table of energies and fit parameters, for helping solver
     Real(t_real),Pointer  :: p_etab(:,:)  ! nRv,2
     Real(t_real),Pointer  :: p_vartab(:,:,:)  ! 2,nRv,2
     !! the eigen energies
     Real(t_real),Pointer  :: p_ev(:,:)  ! nRv,2
     !! the eigen solutions
     Real(t_real),Pointer  :: p_fv(:,:,:,:)  ! nr,nRv,ncomp,2
     !! the eigen solutions exponent
     Real(t_real)  :: fgamv
     !! spherical orbital density for sic
     Real(t_real),Pointer  :: p_orbdens(:,:,:)  ! nr,nRv,2
     !!
     !! density and potentials
     !!
     !!  4pi*r**2*density,4pi*r**2*spin_density
     Real(t_real),Pointer  :: p_rho(:),p_sp(:) 
     !!  4pi*r**2*d/dr density,4pi*r**2*d/dr spin_density
     Real(t_real),Pointer  :: p_drho(:),p_dsp(:) 
     Real(t_real),Pointer  :: p_d2rho(:),p_d2sp(:) 
     !!  r**potential,r**magnetic_field
     Real(t_real),Pointer  :: p_vtr(:),p_btr(:) 
     !! p_rhol(:,l) is the 2*l,0-component of the nonspherical density
     Real(t_real),Pointer  :: p_rhol(:,:)! nr,0:lmax
     !! p_spl(:,l) is the 2*l,0-component of the nonspherical spin density
     Real(t_real),Pointer  :: p_spl(:,:)! nr,0:lmax
     !!
     !!total energy
     Real(t_real)   :: Etot
     !!
     !! temporary aux arrays
     !!
     Real(t_real),Pointer  :: p_vhar(:)
     Real(t_real),Pointer  :: p_vxc (:)
     !!
  End Type T_Atom
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine  init_tatom(at)
    !!
    Implicit none  
    !!IO
    Type(T_atom),Intent(inout)        :: at
    !!LOCAL
    !!
    at% standalone=.True.
    at% gam_factor=.False.
    at% nv=0_t_int
    at% ionicity=0.0_t_real
    at% Magmom=0.0_t_real
    at% nonspheredens=.False.
    at% nuc_type=FINUC_NONE
    at% xreduce=1.0_t_real
    Call init_mesh(at% mesh)
    Call init_iterationcontrol(at% iteration_control)
    Nullify(at% p_nnv)
    Nullify(at% p_lv)
    Nullify(at% p_magorder)
    Nullify(at% p_nroccu)
    Nullify(at% p_gsum)
    Nullify(at% p_rho)
    Nullify(at% p_drho)
    Nullify(at% p_d2rho)
    Nullify(at% p_vtr)
    Nullify(at% p_rhol)
    Nullify(at% p_sp)
    Nullify(at% p_dsp)
    Nullify(at% p_d2sp)
    Nullify(at% p_btr)
    Nullify(at% p_spl)
    Nullify(at% p_roccu)
    Nullify(at% p_degRv)
    Nullify(at% p_nnRv)
    Nullify(at% p_lRv)
    Nullify(at% p_kapRv)
    Nullify(at% p_muRv)
    Nullify(at% p_ivrTOiv)
    Nullify(at% p_etab)
    Nullify(at% p_vartab)
    Nullify(at% p_ev)
    Nullify(at% p_fv)
    Nullify(at% p_orbdens)
    Nullify(at% p_vhar)
    Nullify(at% p_vxc)
    !!
    !!
  End Subroutine init_tatom
  !!***********************************************************************
  Subroutine  clean_tatom(at)
    !!
    Implicit none  
    !!IO
    Type(T_atom),Intent(inout)        :: at
    !!LOCAL
    !!
    Call cleanup_mesh(at% mesh)
    at% nv=0_t_int
    call pfree(at% p_nnv)
    call pfree(at% p_lv)
    call pfree(at% p_magorder)
    call pfree(at% p_nroccu)
    Call pfree(at% p_gsum)
    Call pfree(at% p_rho)
    Call pfree(at% p_drho)
    Call pfree(at% p_d2rho)
    Call pfree(at% p_rhol)
    call pfree(at% p_sp)
    call pfree(at% p_dsp)
    call pfree(at% p_d2sp)
    call pfree(at% p_spl)
    call pfree(at% p_roccu)
    call pfree(at% p_degRv)
    call pfree(at% p_nnRv)
    call pfree(at% p_lRv)
    call pfree(at% p_kapRv)
    call pfree(at% p_muRv)
    call pfree(at% p_ivrTOiv)
    call pfree(at% p_etab)
    Call pfree(at% p_vartab)
    Call pfree(at% p_ev)
    Call pfree(at% p_fv)
    Call pfree(at% p_orbdens)
    Call pfree(at% p_vhar)
    Call pfree(at% p_vxc)
    Call pfree(at% p_vtr)
    Call pfree(at% p_btr)
    at% vca=.False.
    !!
  End Subroutine clean_tatom
  !!***********************************************************************
  Subroutine copy_atom(from,to)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(in)      :: from
    Type(T_Atom),Intent(inout)   :: to
    !!LOCAL
    Character(*),Parameter::nsubfunc="copy_atom"
    Integer(t_int) :: l1,u1,l2,u2,l3,u3,l4,u4
    !!
    to% mspin=from% mspin
    to% element=from% element
    to% Z=from% Z
    to% ionicity=from% ionicity
    to% Magmom=from% Magmom
    to% vca=from% vca
    to% nuc_type=from% nuc_type
    to% relativistic=from% relativistic
    to% compr=from% compr
    to% r0=from% r0
    to% exn0=from% exn0
    to% wftol=from% wftol
    to% xreduce=from% xreduce
    to% force_pos_spin=from% force_pos_spin
    to% noUnOccupied=from% noUnOccupied
    to% sic=from% sic
    to% sic4f=from% sic4f
    to% standalone=from% standalone
    to% gam_factor=from% gam_factor
    to% overwriteout=from% overwriteout
    to% basout=from% basout
    to% rhoout=from% rhoout
    to% potout=from% potout
    !!
    Call copy_mesh(from% mesh,to% mesh)
    !!
    !!
    to% iteration_control=from% iteration_control
    to% basis_def=from% basis_def
    to% noccus_def=from% noccus_def
    to% magorder_def=from% magorder_def
    to% ncomp=from% ncomp
    to% nv=from% nv
    !!
    l1=Lbound(from% p_nnv,1)
    u1=Ubound(from% p_nnv,1)
    Call palloc('to% p_nnv',to% p_nnv,l1,u1,nmodule,nsubfunc)
    l1=Lbound(from% p_lv,1)
    u1=Ubound(from% p_lv,1)
    Call palloc('to% p_lv',to% p_lv,l1,u1,nmodule,nsubfunc)
    l1=Lbound(from% p_magorder,1)
    u1=Ubound(from% p_magorder,1)
    Call palloc('to% p_magorder',to% p_magorder,l1,u1,nmodule,nsubfunc)
    l1=Lbound(from% p_nroccu,1)
    u1=Ubound(from% p_nroccu,1)
    l2=Lbound(from% p_nroccu,2)
    u2=Ubound(from% p_nroccu,2)
    Call palloc('to% p_nroccu',to% p_nroccu,l1,u1,l2,u2,nmodule,nsubfunc)
    to% p_nnv(:)=from% p_nnv(:)
    to% p_lv(:)=from% p_lv(:)
    to% p_magorder(:)=from% p_magorder(:)
    to% p_nroccu(:,:)=from% p_nroccu(:,:)
    !!
    to% lmax=from% lmax
    to% nonspheredens=from% nonspheredens
    !!
    If(Associated(from% p_gsum))Then
       l1=Lbound(from% p_gsum,1)
       u1=Ubound(from% p_gsum,1)
       l2=Lbound(from% p_gsum,2)
       u2=Ubound(from% p_gsum,2)
       l3=Lbound(from% p_gsum,3)
       u3=Ubound(from% p_gsum,3)
       Call palloc('to% p_gsum',to% p_gsum,l1,u1,l2,u2,l3,u3,nmodule,nsubfunc)
       to% p_gsum=from% p_gsum
    Endif
    !!
    to% nRv=from% nRv
    to% degRvmx=from% degRvmx
    !!
    l1=Lbound(from% p_degRv,1)
    u1=Ubound(from% p_degRv,1)
    Call palloc('to% p_degRv',to% p_degRv,l1,u1,nmodule,nsubfunc)
    to% p_degRv=from% p_degRv
    !!
    l1=Lbound(from% p_nnRv,1)
    u1=Ubound(from% p_nnRv,1)
    Call palloc('to% p_nnRv',to% p_nnRv,l1,u1,nmodule,nsubfunc)
    to% p_nnRv=from% p_nnRv
    !!
    l1=Lbound(from% p_lRv,1)
    u1=Ubound(from% p_lRv,1)
    Call palloc('to% p_lRv',to% p_lRv,l1,u1,nmodule,nsubfunc)
    to% p_lRv=from% p_lRv
    !!
    l1=Lbound(from% p_kapRv,1)
    u1=Ubound(from% p_kapRv,1)
    Call palloc('to% p_kapRv',to% p_kapRv,l1,u1,nmodule,nsubfunc)
    to% p_kapRv=from% p_kapRv
    !!
    l1=Lbound(from% p_muRv,1)
    u1=Ubound(from% p_muRv,1)
    l2=Lbound(from% p_muRv,2)
    u2=Ubound(from% p_muRv,2)
    Call palloc('to% p_muRv',to% p_muRv,l1,u1,l2,u2,nmodule,nsubfunc)
    to% p_muRv=from% p_muRv
    !!
    l1=Lbound(from% p_ivrToiv,1)
    u1=Ubound(from% p_ivrToiv,1)
    Call palloc('to% p_ivrToiv',to% p_ivrToiv,l1,u1,nmodule,nsubfunc)
    to% p_ivrToiv=from% p_ivrToiv
    !!
    l1=Lbound(from% p_roccu,1)
    u1=Ubound(from% p_roccu,1)
    Call palloc('to% p_roccu',to% p_roccu,l1,u1,nmodule,nsubfunc)
    to% p_roccu=from% p_roccu
    !!
    l1=Lbound(from% p_etab,1)
    u1=Ubound(from% p_etab,1)
    l2=Lbound(from% p_etab,2)
    u2=Ubound(from% p_etab,2)
    Call palloc('to% p_etab',to% p_etab,l1,u1,l2,u2,nmodule,nsubfunc)
    to% p_etab=from% p_etab
    !!
    l1=Lbound(from% p_vartab,1)
    u1=Ubound(from% p_vartab,1)
    l2=Lbound(from% p_vartab,2)
    u2=Ubound(from% p_vartab,2)
    l3=Lbound(from% p_vartab,3)
    u3=Ubound(from% p_vartab,3)
    Call palloc('to% p_vartab',to% p_vartab,l1,u1,l2,u2,l3,u3,nmodule,nsubfunc)
    to% p_vartab=from% p_vartab
    !!
    l1=Lbound(from% p_ev,1)
    u1=Ubound(from% p_ev,1)
    l2=Lbound(from% p_ev,2)
    u2=Ubound(from% p_ev,2)
    Call palloc('to% p_ev',to% p_ev,l1,u1,l2,u2,nmodule,nsubfunc)
    to% p_ev=from% p_ev
    !!
    !!
    l1=Lbound(from% p_fv,1)
    u1=Ubound(from% p_fv,1)
    l2=Lbound(from% p_fv,2)
    u2=Ubound(from% p_fv,2)
    l3=Lbound(from% p_fv,3)
    u3=Ubound(from% p_fv,3)
    l4=Lbound(from% p_fv,4)
    u4=Ubound(from% p_fv,4)
    Call palloc('to% p_fv',to% p_fv,l1,u1,l2,u2,l3,u3,l4,u4,nmodule,nsubfunc)
    to% p_fv=from% p_fv
    !!
    to% fgamv=from% fgamv
    !!
    l1=Lbound(from% p_orbdens,1)
    u1=Ubound(from% p_orbdens,1)
    l2=Lbound(from% p_orbdens,2)
    u2=Ubound(from% p_orbdens,2)
    l3=Lbound(from% p_orbdens,3)
    u3=Ubound(from% p_orbdens,3)
    Call palloc('to% p_orbdens',to% p_orbdens,l1,u1,l2,u2,l3,u3,nmodule,nsubfunc)
    to% p_orbdens=from% p_orbdens
    !!
    !!
    l1=Lbound(from% p_rho,1)
    u1=Ubound(from% p_rho,1)
    Call palloc('to% p_rho',to% p_rho,l1,u1,nmodule,nsubfunc)
    to% p_rho=from% p_rho
    !!
    l1=Lbound(from% p_sp,1)
    u1=Ubound(from% p_sp,1)
    Call palloc('to% p_sp',to% p_sp,l1,u1,nmodule,nsubfunc)
    to% p_sp=from% p_sp
    !!
    l1=Lbound(from% p_drho,1)
    u1=Ubound(from% p_drho,1)
    Call palloc('to% p_drho',to% p_drho,l1,u1,nmodule,nsubfunc)
    to% p_drho=from% p_drho
    !!
    l1=Lbound(from% p_dsp,1)
    u1=Ubound(from% p_dsp,1)
    Call palloc('to% p_dsp',to% p_dsp,l1,u1,nmodule,nsubfunc)
    to% p_dsp=from% p_dsp
    !!
    l1=Lbound(from% p_d2rho,1)
    u1=Ubound(from% p_d2rho,1)
    Call palloc('to% p_d2rho',to% p_d2rho,l1,u1,nmodule,nsubfunc)
    to% p_d2rho=from% p_d2rho
    !!
    l1=Lbound(from% p_d2sp,1)
    u1=Ubound(from% p_d2sp,1)
    Call palloc('to% p_d2sp',to% p_d2sp,l1,u1,nmodule,nsubfunc)
    to% p_d2sp=from% p_d2sp
    !!
    l1=Lbound(from% p_vtr,1)
    u1=Ubound(from% p_vtr,1)
    Call palloc('to% p_vtr',to% p_vtr,l1,u1,nmodule,nsubfunc)
    to% p_vtr=from% p_vtr
    !!
    l1=Lbound(from% p_btr,1)
    u1=Ubound(from% p_btr,1)
    Call palloc('to% p_btr',to% p_btr,l1,u1,nmodule,nsubfunc)
    to% p_btr=from% p_btr
    !!
    !!
    If(Associated(from% p_rhol))Then
       l1=Lbound(from% p_rhol,1)
       u1=Ubound(from% p_rhol,1)
       l2=Lbound(from% p_rhol,2)
       u2=Ubound(from% p_rhol,2)
       Call palloc('to% p_rhol',to% p_rhol,l1,u1,l2,u2,nmodule,nsubfunc)
       to% p_rhol=from% p_rhol
    Endif
    !!
    If(Associated(from% p_spl))Then
       l1=Lbound(from% p_spl,1)
       u1=Ubound(from% p_spl,1)
       l2=Lbound(from% p_spl,2)
       u2=Ubound(from% p_spl,2)
       Call palloc('to% p_spl',to% p_spl,l1,u1,l2,u2,nmodule,nsubfunc)
       to% p_spl=from% p_spl
    Endif
    !!
    to% Etot=from% Etot
    !!
    l1=Lbound(from% p_vhar,1)
    u1=Ubound(from% p_vhar,1)
    Call palloc('to% p_vhar',to% p_vhar,l1,u1,nmodule,nsubfunc)
    to% p_vhar=from% p_vhar
    !!
    l1=Lbound(from% p_vxc,1)
    u1=Ubound(from% p_vxc,1)
    Call palloc('to% p_vxc',to% p_vxc,l1,u1,nmodule,nsubfunc)
    to% p_vxc=from% p_vxc
    !!
    Return
    !!
  End Subroutine copy_atom
  !!***********************************************************************

End Module tatom

