Module tbandplot
  !!
  Use precisiondefs
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TBANDPLOT"  
  !!
  Public :: T_Bandplot,init_bandplot,T_LDOS,init_ldos,T_WeightRot&
       &,init_WeightRot,clean_WeightRot,FRELPROJECTION_TYPES&
       &,FRELPROJECTION_JMU,FRELPROJECTION_LMS
!!$       &,ERG_UNIT_EV,ERG_UNIT_HA,ERG_UNIT_RY,ERG_UNITS
  !!***********************************************************************
  Integer(t_int),Parameter :: FRELPROJECTION_JMU=1
  Integer(t_int),Parameter :: FRELPROJECTION_LMS=2
  Character(3)   :: FRELPROJECTION_TYPES(2)
  Data FRELPROJECTION_TYPES/'jmu','lms'/
  !!***********************************************************************
!!$  Integer(t_int),Parameter :: ERG_UNIT_EV   =1_t_int
!!$  Integer(t_int),Parameter :: ERG_UNIT_HA   =2_t_int
!!$  Integer(t_int),Parameter :: ERG_UNIT_RY   =3_t_int
!!$  !!
!!$  Character(3),Parameter :: ERG_UNITS(3)=(/ &
!!$       & "eV " &
!!$       &,"Ha "&
!!$       &,"Ry "&
!!$       &/)
  !!
  Type T_Bandplot
     Logical(t_log) :: plotbands,read_sympoints,plotidos,plotndos
     Logical(t_log) :: plotkspintextures
     !! if this is true, the output in +band and +bweights contains
     !! only the bands
     !!     lpo - part_occu_offset(1) ... hpo + part_occu_offset(2)
     !! where lpo is the lowest and hpo is the highest partially occupied
     !! band
     Logical(t_log) :: output_partoccu_bands
     Integer(t_int) :: part_occu_offset(2)
     Integer(t_int) :: ndiv
     Integer(t_int) :: nptdos
     !! energy window for DOS output and for band[weight]s output (if required)
     Real(t_real)   :: emin,emax
     !! band indices of bands which fall into (partially) into the energy window
     Integer(t_int) :: highest_in_window
     Integer(t_int) :: lowest_in_window
     !! here we require the output of the band[weight]s to be restricted 
     !! to the energy window, given above
     Logical(t_log) :: output_bands_in_window
     Logical(t_log) :: bandweights,bweight_rot
     Real(t_real)   :: z_axis(3)
     Real(t_real)   :: x_axis(3)
     !! FREL projection
     !Integer(t_int) :: frelprojection
     !! output coefficient file +coeff
     Logical(t_log) :: coeffout
     !! special points
     Integer(t_int) :: npoints
     Character(20),Pointer :: p_labels(:)
     Real(t_real),Pointer  :: p_points(:,:)
     !! k-points path along special points
     Integer(t_int)        :: nkpnts
     Real(t_real),Pointer  :: p_kpnts(:,:) ! 4,nkpnts
     !! for local dos creation

     !!
     !! suffix for +band and +bweights file
     Character(100)   :: suffix 
     !!
  End Type T_Bandplot
  !!***********************************************************************
  Type T_LDOS
     !! shall we calculate the ldos at all?
     Logical(t_log) :: plotldos
     !! number of sites for which to calculate the ldos
     Integer(t_int) :: nldos
     !! sites for which to calculate the ldos
     Integer(t_int),Pointer :: p_ldsites(:) ! 1:nldos
     !! p_siteld(isa) gives ldos-definition number of site isa
     !! zero means that site isa is not in the list for which
     !! to calculate ldos
     Integer(t_int),Pointer :: p_siteld(:)  ! 1:nsite
     !!
  End Type T_LDOS

  !!***********************************************************************
  Type T_WeightRot
     !! N/SREL lm-based axes rotation matrices
     Real(t_real),Pointer  :: p_wu(:,:)
     !! FREL jm-based axes rotation matrices
     Complex(t_real),Pointer:: p_cdjmm(:,:)
     !! FREL jm-to-lms transformation including rotation of axes
     Complex(t_real),Pointer   :: p_relToNrel(:,:,:,:)! ml,mj,ms,kappa
     !! compact FREL njm-to-lms transformation including rotation of axes
     Complex(t_real),Pointer   :: p_relToNrelwhole(:,:,:)! mjmj,mlms,l
     !! where mlms runs over mlms=-lup, -ldn, ...,lup, ldn
     !! unified axes rotation matrices (slight overhead, but unified)
     Complex(t_real),Pointer   :: p_rotwhole(:,:,:)! mjmj,ml,l
     !! combined symmetry and local transformed axes operations for ldos
     Complex(t_real),Pointer   :: p_wu_ks(:,:,:,:)
     !!
  End Type T_WeightRot
  !!***********************************************************************
  
Contains  
  Subroutine init_bandplot(bp)
    Implicit None 
    !!IO
    Type(T_Bandplot),Intent(inout) :: bp
    !!LOCAL
    Nullify(bp% p_labels)
    Nullify(bp% p_points)
    Nullify(bp% p_kpnts)
    bp% part_occu_offset(1)=0
    bp% part_occu_offset(2)=0
    bp% output_partoccu_bands=.False.
    bp% plotndos=.False.
    bp% output_bands_in_window=.False.
    bp% suffix=''
    !bp% frelprojection=FRELPROJECTION_JMU
    bp% coeffout=.False.
    bp% plotkspintextures=.False.
  End Subroutine init_bandplot
  !!***********************************************************************
  Subroutine init_ldos(ld)
    Implicit None 
    !!IO
    Type(T_LDOS),Intent(inout) :: ld
    !!LOCAL
    ld% plotldos=.False.
    ld% nldos=0_t_int
    Nullify(ld% p_ldsites)
    Nullify(ld% p_siteld)
    !!
  End Subroutine init_ldos
  !!***********************************************************************
  Subroutine init_WeightRot(wr)
    Implicit None 
    !!IO
    Type(T_WeightRot),Intent(inout) :: wr
    !!LOCAL
    Nullify(wr% p_wu)
    Nullify(wr% p_cdjmm)
    Nullify(wr% p_relToNrel)
    Nullify(wr% p_relToNrelwhole)
    Nullify(wr% p_rotwhole)
    Nullify(wr% p_wu_ks)
    !!
  End Subroutine init_WeightRot
  !!***********************************************************************
  Subroutine clean_WeightRot(wr)
    Implicit None 
    !!IO
    Type(T_WeightRot),Intent(inout) :: wr
    !!LOCAL
    Call pfree(wr% p_wu)
    Call pfree(wr% p_cdjmm)
    Call pfree(wr% p_relToNrel)
    Call pfree(wr% p_relToNrelwhole)
    Call pfree(wr% p_rotwhole)
    Call pfree(wr% p_wu_ks)
    !!
  End Subroutine clean_WeightRot
  !!***********************************************************************
  

End Module tbandplot
