Module tbasdef
  !!
  Use precisiondefs
  Use constants
  Use errors
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TBASDEF"  
  !!
  Public :: init_basdef,clean_basdef,T_Basdef&
       &,palloc_basdefs,pfree_basdefs
  !!***********************************************************************
  Type T_Basdef
     Integer(t_int) :: nchi,nc
     Integer(t_int),Pointer :: p_lc(:),p_nnc(:)
     Real(t_real),Pointer   :: p_Qionc(:),p_Spolc(:)
     !! valence
     Integer(t_int) :: nv
     Integer(t_int),Pointer :: p_lv(:),p_nnv(:),p_tv(:)
     Real(t_real),Pointer   :: p_Qionv(:),p_Spolv(:),p_Pexv(:)
     Character(4),Pointer   :: p_firstv(:)
     !!
     Real(t_real)   :: pexdef,exn2def
  End Type T_Basdef
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine init_basdef(bd)
    !!
    Implicit None
    !!IO
    Type(T_Basdef),Intent(inout) :: bd
    !!LOCAL
!    Character(*),Parameter::nsubfunc="init_basdef"
    !!
    Nullify(bd% p_lc)
    Nullify(bd% p_nnc)
    Nullify(bd% p_Qionc)
    Nullify(bd% p_Spolc)
    Nullify(bd% p_lv)
    Nullify(bd% p_nnv)
    Nullify(bd% p_tv)
    Nullify(bd% p_firstv)
    Nullify(bd% p_Qionv)
    Nullify(bd% p_Spolv)
    Nullify(bd% p_Pexv)
    !!
    bd% nchi=0
    bd% nc=0
    bd% nv=0
    !!
    Return
    !!
  End Subroutine init_basdef
  !!***********************************************************************
  Subroutine clean_basdef(bd)
    !!
    Implicit None
    !!IO
    Type(T_Basdef),Intent(inout) :: bd
    !!LOCAL
!    Character(*),Parameter::nsubfunc="clean_basdef"
    !!
    Call pfree(bd% p_lc)
    Call pfree(bd% p_nnc)
    !!
    bd% nchi=0
    bd% nc=0
    bd% nv=0
    !!
    Return
    !!
  End Subroutine clean_basdef
  !!***********************************************************************
  Subroutine palloc_basdefs(name,bdefs,N,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N
    Type(T_Basdef),Pointer       :: bdefs(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
!    Character(*),Parameter::nsubfunc="palloc_basdefs"
    Integer(t_int) :: i
    Integer :: icheck
    !!
    !!
    Call pfree_basdefs(bdefs)
    !!
    Allocate(bdefs(1:N),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    Do i=1,N
       Call init_basdef(bdefs(i))
    Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_basdefs
  !!***********************************************************************
  Subroutine pfree_basdefs(bdefs)
    Implicit None 
    !!IO
    Type(T_Basdef),Pointer       :: bdefs(:)
    !!LOCAL
    Integer(t_int) :: i
    External crealmemaddress
    !!
    !!
    If(Associated(bdefs))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       Do i=1,Size(bdefs)
          Call clean_basdef(bdefs(i))
       Enddo
       !!
       Deallocate(bdefs)
       Nullify(bdefs)
    Endif
    !!
  End Subroutine pfree_basdefs
  !!***********************************************************************

End Module tbasdef

