Module tbzone
  !!
  Use precisiondefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TBZONE"  
  !!
  Public :: T_Bzone,init_bzone&
       &,BZ_TETRAHEDRON_METHOD,BZ_METHFESSEL_PAXTON_METHOD,BZ_METHODS
  !!***********************************************************************
  Integer(t_int),Parameter :: BZ_TETRAHEDRON_METHOD       =1_t_int
  Integer(t_int),Parameter :: BZ_METHFESSEL_PAXTON_METHOD =2_t_int
  !!***********************************************************************
  Character(24),Parameter :: BZ_METHODS(2)=(/ &
       & "tetrahedron method      " &
       &,"Methfessel-Paxton method"&
       &/)
!!***********************************************************************

  Type T_Bzone
     Integer(t_int) :: mp_norder
     Logical(t_log) :: metal,gammapoint,symeqkpoint_fullbz,dospecial
     Logical(t_log) :: ensure_negkpoint !! make sure -k is explicitly contained
     Integer(t_int) :: method
     Real(t_real)   :: mp_width
     Integer(t_int) :: nkxyz(3) 
     Integer(t_int) :: nkp,ntet
     Integer(t_int) :: highest_fully_occupied
     Integer(t_int) :: lowest_fully_unoccupied
     Integer(t_int) :: kcut(3,4,6) !! micor cell tetra subdivision
     Real(t_real)   :: ntetsum
     Real(t_real)   :: kTmin !! temperature for gamma point calculation in Kelvin
     Real(t_real)   :: dkT   !! temperature for gamma point calculation in Kelvin
     Real(t_real)   :: kTmax !! temperature for gamma point calculation in Kelvin
     Real(t_real)   :: kTcur !! temperature for gamma point calculation in Kelvin
     !! tet(0,:) contains multiplicity of tetrhedron
     !! tet(0,:)/ntetsum is weight of tet, since all tets are of the same
     !! volume, we get the volume weight
     Integer(t_int),Pointer :: p_tet(:,:) ! 0:5,ntet
     !! map k-mesh indices ik1,ik2,ik3 onto irreducible point ikp
     Integer(t_int),Pointer :: p_ijk_to_ikp(:) ! continous index
     !! with ibk(i,j,k)=i+((j-1)+(k-1)*nkxyz(2))*nkxyz(1)
     !!
     !! Map irreducible point ikp onto k-mesh indices ik1,ik2,ik3 of 
     !! irreducible point
     !! ik1=p_ikp_to_ijk(1,ikp)
     !! ik2=p_ikp_to_ijk(2,ikp)
     !! ik3=p_ikp_to_ijk(3,ikp)
     !! multiplicity of ikp=p_ikp_to_ijk(4,ikp)
     Integer(t_int),Pointer :: p_ikp_to_ijk(:,:) ! 4,nkp
     Real(t_real),Pointer   :: p_bk(:,:)  ! 3,nkp
     Real(t_real),Pointer   :: p_wbk(:)   ! 3,nkp
     Integer(t_int),Pointer :: p_irredindex(:) ! nkp
  End Type T_Bzone
  !!***********************************************************************
Contains

  Subroutine init_bzone(bz)
    Implicit None 
    !!IO
    Type(T_Bzone),Intent(inout) :: bz
    !!LOCAL
    bz% gammapoint=.False.
    bz% kTmax=10.0e-3_t_real
    bz% symeqkpoint_fullbz=.true.
    bz% dospecial=.False.
    bz% ensure_negkpoint=.False.
    Nullify(bz% p_tet)
    Nullify(bz% p_bk)
    Nullify(bz% p_wbk)
    Nullify(bz% p_ijk_to_ikp)
    Nullify(bz% p_ikp_to_ijk)
    Nullify(bz% p_irredindex)
  End Subroutine init_bzone
  !!***********************************************************************

End Module tbzone
