Module tdiracdefs
  !!
  Use precisiondefs
  !!
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TDIRACDEFS"  
  !!
  Public :: T_Diracdefs,init_diracdefs,T_Quant,init_quant
       !&,REL_SPIN_CHAR_V,REL_SPIN_CHAR_CSCV,REL_SPIN_CHAR_FULL,SPIN_POP_TYPES
  !!***********************************************************************
  !! The spin character of the population analysis may be calculated within 
  !! several approximations.
  !!
  !!    with BSvv==<v|\beta\Sigma_z|v>
  !!    and  BScv==<c|\beta\Sigma_z|v>
  !!    and  BScc==diag <c|\beta\Sigma_z|c>
  !!
  !! 1) take the expression
  !!           nvv*(Svv-Svc*Scv)*diag(BSvv)
  !!    Approximation is crude.
  Integer(t_int),Parameter :: REL_SPIN_CHAR_V   =1_t_int
  !! 2) take the expression  
  !!           nvv*(BSvv-Svc*BScc*Scv)
  !!    Approximation is the diagonal BScc
  !!    and  BScv~ \approx BScc *Scv
  Integer(t_int),Parameter :: REL_SPIN_CHAR_CSCV=2_t_int
  !! 3) use  exact valence expression: 
  !!             nvv*(BSvv-Svc*BScv-BSvc*Scv+Scv*BScc*Scv)
  !!    The only approximation is the use of the diagonal core-core matrix
  !!    elements in BScc instead taking the full (non diagonal) matrix
  Integer(t_int),Parameter :: REL_SPIN_CHAR_FULL=3_t_int
  !!
  !Character(38),Parameter :: SPIN_POP_TYPES(3)=(/ &
  !     & "crude approximation (cheap)           " &
  !     &,"medium approximation (less expensive) "&
  !     &,"exact expression (expensive)          "&
  !     &/)
  !!
  !!
  !!
  !! This type controls some aspect of the full relativistic calculation
  Type T_Diracdefs
     !! which type of approximation for the population analysis
     Integer(t_int) :: spin_pop_type_not_used
  End Type T_Diracdefs
  !!
  !!
  !!
  !!
  !! The quantization axis of the magnetic dirac case
  Type T_Quant
     !! quantization axis
     Real(t_real)   :: axis(3)
     !! euler angles, for the rotation, which makes axis the new z-axis
     Real(t_real)   :: eulang(3)
     !! rotation matrix M belonging to the euler angles
     !! a vector b with coordinates b_i in the current system 
     !! has coordinates  b'_i=b_j*M_ji in the rotated system
     !! (the axis coordinates transform as  axis'=(0,0,1)|axis| )
     Real(t_real)           :: MM(3,3)
!!$     !! transposed of Ylm-rotation matrices (real harmonics)
!!$     Real(t_real),Pointer   :: p_DRealYlmT(:,:)
     !! spin-1/2 Wigner-D function for these Euler angles
     Complex(t_real):: D1o2(2,2)
     !!
     Complex(t_real),Pointer:: p_chiSigmaRchi(:,:,:,:)
     !! <chi|\vec_{\Sigma}|chi>
     Complex(t_real),Pointer:: p_chiVSigmachi(:,:,:,:,:)
     Complex(t_real),Pointer:: p_chiLRchi(:,:,:,:)
     Complex(t_real),Pointer:: p_chipSLRchi(:,:,:,:,:)
  End Type T_QUANT
  !!***********************************************************************
Contains
  !!
  !!***********************************************************************
  Subroutine  init_diracdefs(d)
    !!
    Implicit none  
    !!IO
    Type(T_Diracdefs),Intent(inout)          :: d
    !!LOCAL
    !!
    d% spin_pop_type_not_used=REL_SPIN_CHAR_FULL
    !!
    Return
    !!
  End Subroutine init_diracdefs
  !!***********************************************************************
  Subroutine  init_quant(q)
    !!
    Implicit none  
    !!IO
    Type(T_Quant),Intent(inout)            :: q
    !!LOCAL
    !!
!!$    Nullify(q% p_DRealYlmT)
    Nullify(q% p_chiSigmaRchi)
    Nullify(q% p_chiVSigmachi)
    Nullify(q% p_chiLRchi)
    Nullify(q% p_chipSLRchi)
    !!
    q% axis(1)=0.0_t_real
    q% axis(2)=0.0_t_real
    q% axis(3)=1.0_t_real
    !!
    q% D1o2(1,1)=Cmplx(1.0_t_real,0.0_t_real,t_real)
    q% D1o2(2,2)=Cmplx(1.0_t_real,0.0_t_real,t_real)
    q% D1o2(1,2)=Cmplx(0.0_t_real,0.0_t_real,t_real)
    q% D1o2(2,1)=Cmplx(0.0_t_real,0.0_t_real,t_real)
    !!
    Return
    !!
  End Subroutine init_quant
  !!***********************************************************************
  !!
End Module tdiracdefs

