Module tests
  !!
  Use precisiondefs
  Use constants
  Use file_iounits
  Use errors
  Use tsymmetry
  Use symmetry
  Use tpotentialdensity
  Use torbitalbasis
  Use optionstest
  Use tmesh
  Use mesh
  Use clock
  Use nsphintirred
  Use matserv
  Use integrate1d
  Use memory
  Use trelativistic
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TESTS"  
  !!
  Public ::  dotests
  !!***********************************************************************
contains
  !!***********************************************************************
  !! do paranoid tests
  Subroutine dotests(symmetry,structure,lattice,potential,basis,tol&
       &,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Lattice),Intent(in)   :: lattice
    Integer(t_int),Intent(in)    :: relativistic
    Real(t_real),Intent(in)      :: tol
    !!LOCAL
    !!
    If(relativistic==FULL_RELATIVISTIC)Then
       Return
    Endif
    !!
    If(.Not. isoption('NO_SYMMETRYTEST'))Then
       Call symtst(symmetry,structure,lattice &
            &,basis% p_maxl,potential % max_LIO,potential % max_L&
            &,potential % mlmp,potential % p_nlmp&
            &,potential % p_lmp,tol,relativistic)
    Endif
    !!
    !!
    !!
  End Subroutine dotests
  !!***********************************************************************
  !! tests symmetry treatment
  !! 1. irreducible part integration
  !! 2. rotation
  !! 3. 1.+2.
  !! 4. detlrot
  !!
  Subroutine symtst(symmetry,structure,lattice&
       &,maxl,max_LIO,max_L,mlmp,nlmp,lmp,tol,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in) :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in) :: lattice
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in) :: max_L,max_LIO,mlmp,nlmp(*),lmp(mlmp,*)
    Integer(t_int),Intent(in)    :: maxl(:) !! max l of orbitals
    Real(t_real),Intent(in) :: tol
    !!LOCAL
    Character(*),Parameter::nsubfunc="symtst"
    Integer(t_int) :: nvg
    Logical(t_log) :: OK
    Integer,Parameter:: nnnp=20
    Integer(t_int):: nnn
    Real(t_real) :: v(nnnp,mlmp,structure % nsite)
    Real(t_real) :: vi(nnnp,mlmp,structure % nsite)
    Real(t_real) :: vf(nnnp,mlmp,structure % nsite)
    Real(t_real) :: vfi(nnnp,mlmp,structure % nsite)
    Real(t_real) :: vv(nnnp,mlmp,structure % nsite)
    !!DEC compiler workaround
    nnn=nnnp
    !!
    OK=.True.
    Call watch(nsubfunc,cpu_start,reset=.True.)     
    
    nvg=20

    Call symtst1(symmetry % HEX,structure,lattice,nnn,maxl&
         &,max_LIO,max_L,mlmp,nlmp,lmp,nvg,tol,OK,relativistic)


    Call symtst2(symmetry % HEX,symmetry % pgrpnum&
         &,structure,lattice,nnn,v,vf&
         &,max_L ,mlmp,nlmp,lmp,nvg,.False.)

    !!rotate v
    vv=v
    Call rotloc(1_t_int,1_t_int,vv,mlmp,nlmp,lmp,nnn&
       &,structure % nsite,structure % p_equivsites&
       &,symmetry % IOnumops&
       &,symmetry % opeuler,symmetry % opeulerinv,symmetry % IOopind &
       &,max_L)


    Call vvvcmp(nnn,mlmp,structure % nsite,v,vv,OK,tol,.True.)
    
    !!rotate vf
    vv=vf
    Call rotloc(1_t_int,1_t_int,vv,mlmp,nlmp,lmp,nnn&
       &,structure % nsite,structure % p_equivsites&
       &,symmetry % IOnumops&
       &,symmetry % opeuler,symmetry % opeulerinv,symmetry % IOopind &
       &,max_L)

    Call vvvcmp(nnn,mlmp,structure % nsite,vf,vv,OK,tol,.True.)

    If(symmetry% reduced_sym) goto 1000 !! on reduced sym skip next test
    

    !!irreduciblemo integration + rotation
    Call symtst2(symmetry % HEX,symmetry % pgrpnum&
         &,structure,lattice,nnn,vi,vfi&
         &,max_L ,mlmp,nlmp,lmp,nvg,.true.)

    !!rotate vi
    vv=vi
    Call rotloc(1_t_int,1_t_int,vv,mlmp,nlmp,lmp,nnn&
       &,structure % nsite,structure % p_equivsites&
       &,symmetry % IOnumops&
       &,symmetry % opeuler,symmetry % opeulerinv,symmetry % IOopind &
       &,max_L)

    Call vvvcmp(nnn,mlmp,structure % nsite,v,vv,OK,tol,.False.)
    !!rotate vif
    vv=vfi
    Call rotloc(1_t_int,1_t_int,vv,mlmp,nlmp,lmp,nnn&
       &,structure % nsite,structure % p_equivsites&
       &,symmetry % IOnumops&
       &,symmetry % opeuler,symmetry % opeulerinv,symmetry % IOopind &
       &,max_L)

    Call vvvcmp(nnn,mlmp,structure % nsite,vf,vv,OK,tol,.False.)
    !!
1000 Continue
    !!
    Call watch(nsubfunc,cpu_stop,delete=.True.,Print=.True.)
    If(.Not.OK) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Symmetry tests are not ok!'&
            &,/,' Checkout symmetry, symtest-tolerances, or report the failure!'")
    Endif
    !!      
    !!
  End Subroutine symtst
  !!
  !!***********************************************************************
  !!
  Subroutine symtst2(HEX,pgrpnum,structure,lattice&
       &,nnn,v,vf,max_L,mlmp,nlmp,lmp ,nvg,sym)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Logical(t_log),Intent(in)    :: HEX
    Integer(t_int),Intent(in) :: pgrpnum
    Integer(t_int),Intent(in) :: max_L ! max angular momentum for potential
    Integer(t_int),Intent(in) :: nnn ! number of test mesh points
    Integer(t_int),Intent(in) :: nvg ! angular mesh type, should be 20 
    Integer(t_int),Intent(in) :: mlmp,nlmp(*),lmp(mlmp,*)
    Logical(t_log),Intent(in) :: sym
    Real(t_real),Intent(out)  :: v(nnn,mlmp,structure % nsite)
    Real(t_real),Intent(out)  :: vf(nnn,mlmp,structure % nsite)
    !!LOCAL
    Type(T_Mesh) :: lmesh
    Integer(t_int) :: isa,issa,nga,il,j,insh,issb,ne,nf,ish,n,isb,ig,ir
    Real(t_real) :: rnn,rbb,rnnb,d1,ff,rsh(3),ww,d,ri
    Real(t_real) :: ag(3,2000)
    Real(t_real) :: wg(2000)
    Real(t_real) :: ya((max_L+1_t_int)**2_t_int)
    !!
    Call init_mesh(lmesh)
    !!     radial mesh points
    lmesh % rmax=0.0_t_real
    Do isa=1_t_int,structure % nsite
       lmesh % rmax=Max(lmesh % rmax,lattice% p_ratom(structure % p_isrt(isa)))
    Enddo
    lmesh % rmin=1.e-4_t_real
    lmesh % rmax=lmesh % rmax*4.5_t_real
    lmesh % nr=nnn
    lmesh% type=LIN_MESH
    lmesh% with_zero=.False.
    Call setup_mesh(lmesh)

    v(:,:,:)=0.0_t_real
    vf(:,:,:)=0.0_t_real

    If(sym)Then
       Call gsphirred(nvg,ag,wg,nga,HEX,pgrpnum,.False.)
    Else
       Call gsphirred(nvg,ag,wg,nga,HEX,1_t_int,.False.)
    Endif

    Do isa=1_t_int,structure % nsite
       !! radial mesh
       issa=structure % p_isrt(isa)
       rnn=lattice % p_ratom(issa)
       Do ir=1_t_int,nnn
          ri=lmesh % p_rmesh(ir)
          d=Exp(-((ri/rnn)**2_t_int))*Real(issa,t_real)
          !! angular mesh
          Do ig=1_t_int,nga
             ww=wg(ig)
             rsh(:)=ri*ag(:,ig)
             ff=ww!!fsh(rsh,isa,structure,lattice,shape)*ww
             !! onsite contributions
             Call ylms12(ya,rsh(1),rsh(2),rsh(3),max_L)
             !! neighbour contributions
             d1=0.0_t_real
             isbloop: Do isb=1_t_int,structure % nsite
                n=0_t_int
                ishloop: Do ish=1_t_int,lattice % p_npshell(isb,isa)
                   nf=n+1_t_int
                   ne=n+lattice % p_ninpshell(ish,isb,isa)
                   n=ne
                   issb=structure % p_isrt(isb)
                   rnnb=lattice % p_ratom(issb)
                   Do insh=nf,ne
                      rbb=Sum((rsh(:)-lattice % p_pairs(:,insh,isb,isa))**2_t_int) 
                      d1=d1+Exp(-rbb/rnnb**2_t_int)*Real(issb,t_real)
                   Enddo               ! insh
                End Do ishloop
             End Do isbloop

             Do j=1_t_int,nlmp(isa)
                il=lmp(j,isa)
                v(ir,j,isa)=v(ir,j,isa)+(d+d1)*ya(il)*ww
                vf(ir,j,isa)=vf(ir,j,isa)+(d+d1)*ya(il)*ff
             enddo               ! j
             !!END angular integral           
          Enddo                 ! ig
          !!END r-mesh, nsite
       Enddo                   ! ir
    Enddo                     ! isa
    !!
    Call cleanup_mesh(lmesh)
    !!
  End Subroutine symtst2
  !!***********************************************************************
  Subroutine symtst1(HEX,structure,lattice,nnn&
       &,maxl,max_LIO,max_L,mlmp,nlmp,lmp,nvg,tol,OK,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Logical(t_log),Intent(inout) :: OK
    Logical(t_log),Intent(in)    :: HEX
    Integer(t_int),Intent(in)    :: relativistic
    Integer(t_int),Intent(in) :: nnn ! number of test mesh points
    Integer(t_int),Intent(in) :: max_L ! max angular momentum for potential
    Integer(t_int),Intent(in) :: max_LIO! same from user input
    !! ... this might be different, since we adjust individual 
    !! ... atoms max L to max L of the net density
    Integer(t_int),Intent(in) :: nvg ! angular mesh type, should be 13 
    Integer(t_int),Intent(in) :: maxl(:) !! max l of orbitals
    Integer(t_int),Intent(in) :: mlmp,nlmp(*),lmp(mlmp,*)
    Real(t_real),Intent(in)  :: tol
    !!LOCAL
    Type(T_Mesh) :: lmesh
    Integer(t_int):: isa,issa,ir,lmlmp,nga,ig,j,nl,isb,n,nf,ne,ish,issb,insh
    Integer(t_int) :: glmax,l
    Real(t_real) :: rnn,ri,d,ww,rsh(3),d1,ff,rnnb,rbb
    Real(t_real) :: ag(3,2000)
    Real(t_real) :: wg(2000)
    Real(t_real) :: maxerr,minerr
    Logical(t_log) :: wrong,tomany,misanna
    !! automatic arrays
    Real(t_real) :: ya((max_L+1_t_int)**2_t_int)
    Real(t_real) :: v(nnn,(max_L+1_t_int)**2_t_int,structure % nsite)
    Real(t_real) :: vf(nnn,(max_L+1_t_int)**2_t_int,structure % nsite)
    !!
    Call init_mesh(lmesh)
    !!     radial mesh points
    lmesh % rmax=0.0_t_real
    Do isa=1_t_int,structure % nsite
       lmesh % rmax=Max(lmesh % rmax,lattice% p_ratom(structure % p_isrt(isa)))
    Enddo
    lmesh % rmin=1.e-4_t_real
    lmesh % rmax=lmesh % rmax*4.5_t_real
    lmesh % nr=nnn
    lmesh % type=LIN_MESH
    lmesh % with_zero=.False.
    !!
    Call setup_mesh(lmesh)
    !!
    Call gsphirred(nvg,ag,wg,nga,HEX,1_t_int,.False.)
    !!
    lmlmp=(max_L+1_t_int)**2_t_int
    v(:,:,:)=0.0_t_real
    vf(:,:,:)=0.0_t_real
    Do isa=1_t_int,structure % nsite
       !! radial mesh
       issa=structure % p_isrt(isa)
       rnn=lattice % p_ratom(issa)
       Do ir=1_t_int,nnn
          ri=lmesh % p_rmesh(ir)
          d=Exp(-((ri/rnn)**2_t_int))*Real(issa,t_real)**1.7_t_real
          !! angular mesh
          Do ig=1_t_int,nga
             ww=wg(ig)
             rsh(:)=ri*ag(:,ig)
             ff=ww!!fsh(rsh,isa,structure,lattice,shape)
             !! onsite contributions
             Call ylms12(ya,rsh(1),rsh(2),rsh(3),max_L)
             
             !! neighbour contributions
             d1=0.0_t_real
             isbloop: Do isb=1_t_int,structure % nsite
                n=0_t_int
                ishloop: Do ish=1_t_int,lattice % p_npshell(isb,isa)
                   nf=n+1_t_int
                   ne=n+lattice % p_ninpshell(ish,isb,isa)
                   n=ne
                   issb=structure % p_isrt(isb)
                   rnnb=lattice % p_ratom(issb)
                   Do insh=nf,ne
                      rbb=Sum((rsh(:)-lattice % p_pairs(:,insh,isb,isa))**2_t_int) 
                      d1=d1+Exp(-rbb/rnnb**2_t_int)*Real(issb,t_real)**1.7_t_real
                   Enddo               ! insh
                End Do ishloop
             End Do isbloop
             
             Do j=1_t_int,(max_L+1_t_int)**2_t_int
                v(ir,j,isa)=v(ir,j,isa)+(d+d1)**1.1_t_real*ya(j)*ww
                vf(ir,j,isa)=vf(ir,j,isa)+(d+d1)**1.1_t_real*ya(j)*ff
             Enddo               ! j
             !!END angular integral           
          Enddo                 ! ig
          !!END r-mesh
       Enddo                   ! ir
    Enddo                     ! isa
    !!
    !!     
    !!search nonzero components 
    wrong=.False.
    misanna=.False.
    maxerr=0.0_t_real
    minerr=1.0e30_t_real
    tomany=.false.
    Do isa=1_t_int,structure % nsite
       issa=structure% p_isrt(isa)
       If(relativistic ==FULL_RELATIVISTIC)Then
          glmax=Max(max_LIO,(maxl(issa)+0)*2)
          !! todo: test wheter +1 in relat makes a difference for the
          !! force correction term adn energies
          !! todo: change the +0 into +1 in fplostartup too!!!
       Else
          glmax=Max(max_LIO,maxl(issa)*2)
       Endif
       !!
       nl=0_t_int
       Do j=1_t_int,lmlmp
          l=llilm(j)
          If(l>glmax) Cycle
          d=0.0_t_real
          d1=0.0_t_real
          Do ir=1_t_int,nnn
             ri=lmesh % p_rmesh(ir)
             d=d+(v(ir,j,isa))**2_t_int
             d1=d1+(vf(ir,j,isa))**2_t_int
          Enddo
          d=Sqrt(d/nnn)
          d1=Sqrt(d1/nnn)
          If(Abs(d) < tol .And. Abs(d1) < tol .And. &
               & Any(lmp(1_t_int:nlmp(isa),isa) == j))Then
             Write(outfile,21)isa,llilm(j),mmilm(j),d,d1
             minerr=Min(minerr,Abs(d))
             minerr=Min(minerr,Abs(d1))
             tomany=.true.
          Endif
          If(((Abs(d)>=tol).Or.(Abs(d1)>=tol)).And. &
               & (All(lmp(1_t_int:nlmp(isa),isa) /= j)))Then
             maxerr=Max(maxerr,Abs(d))
             maxerr=Max(maxerr,Abs(d1))
             misanna=.True.
             Write(outfile,15)isa,llilm(j),mmilm(j),d,d1
             wrong=.True.
          Endif
          !!         
          If(d > tol .Or. d1 > tol) Then
             nl=nl+1_t_int
          Endif
       Enddo                   ! j
       If(nl /= nlmp(isa))Then
          Write(outfile,14)nl,isa,nlmp(isa)
          wrong=.True.
       Endif
    Enddo                     ! isa
    !!
    If(tomany)Then
       Write(outfile,"('Found less L-components than given analytically!'&
            &,/,'This indicates, that the symmetry is higher &
            &than specified,'&
            &,/,'or that symtest-tolerance is to large!')")
       Write(outfile,"('min(tests)=',e13.3e3,' < tol=',e13.3e3)")minerr,tol
    Endif
    If(misanna)Then
       Write(outfile,"('Found more L-components than given analytically!'&
            &,/,'This indicates, that the symtest-tolerance is to small,'&
            &,/,'or that the Wyckoff positions are inaccurate,'&
            &,/,'or that a serious problem occured!')")
       Write(outfile,"('max error=',e13.3e3,' > tol=',e13.3e3)")maxerr,tol
    Endif

    If(wrong) Then
       Write(outfile,16)
    Else
       Write(outfile,17)
    Endif
    OK=(OK.And.(.Not.wrong))
    !!
21  Format('site ',i4,': can not find L-component l=',i4,' m=',i4 &
         & ,' as given analytically',/ &
         & ,'test1 ',e12.5e2,'  test2 ',e12.5e2)
14  Format('number of L-components (',i4,') for site ',i3,' is '&
         & ,'differing from analytic result (',i3,')')
15  Format('site ',i4,': nonzero L-component detected : l=',i4,' m=',i4 &
         & ,' , not given analytically',/ &
         & ,'d ',e10.2e2,'df ',e10.2e2)
16  Format('SYMTST: The L-test is not OK. check if symtest-tolerances are to small'&
         &,/,' or if symmetry is incorrect or lower than nesseccary!'&
         &,/,' The symmetry may be incorrect if i.e. in hexagonal lattices'&
         &,/,' the Wyckoff positions are given as 0.333 instead of 1/3!'&
         &,/,' If the symmetry is to low and this is desired'&
         &,/,' use option ''NO_SYMMETRYTEST'' !' &
         &,/,' If other problems, contact programmers !')
17  Format('SYMTST: The L-test is  OK.')

    !!
    Call cleanup_mesh(lmesh)
    !!
  End Subroutine symtst1
  !!***********************************************************************
  !!
  Subroutine  vvvcmp(nnn,mlmp,nsite,v,vv,OK,tol,lv)
        !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: nnn,mlmp,nsite
    Real(t_real),Intent(in)  :: v(nnn,mlmp,nsite),vv(nnn,mlmp,nsite),tol
    Logical(t_log),Intent(in)       :: lv
    Logical(t_log),Intent(inout)    :: OK
    !!LOCAL
    Integer(t_int) :: isa,j,ir
    Logical(t_log) :: wrong
    Real(t_real)   :: maxerr,d
    !!
    maxerr=0.0_t_real
    wrong=.False.
    Do isa=1_t_int,nsite
       Do j=1_t_int,mlmp
          Do ir=1_t_int,nnn
             d=Abs(v(ir,j,isa)-vv(ir,j,isa))
             If(d > tol)Then
                maxerr=Max(maxerr,d)
                If(lv)Then
                   Write(outfile,12)isa,ir,j,v(ir,j,isa),vv(ir,j,isa) &
                        & ,v(ir,j,isa)-vv(ir,j,isa)
                Else
                   Write(outfile,112)isa,ir,j,v(ir,j,isa),vv(ir,j,isa) &
                        & ,v(ir,j,isa)-vv(ir,j,isa)
                Endif
                wrong=.True.
             Endif
          Enddo
       Enddo
    Enddo
    If(wrong)Then
       Write(outfile,"('max(tests)=',e13.3e3,' > tol=',e13.3e3)")maxerr,tol
       If(lv) Then
          Write(outfile,16)
       Else
          Write(outfile,116)
       End If
    Else
       If(lv)Then
          Write(outfile,17)
       Else
          Write(outfile,117)
       Endif
    Endif
    OK=(OK.And.(.Not.wrong))
    !!
12  Format(' site ',i3,': ir ',i3,' nL ',i3,' v ',e13.3e3,'<> rot(v) ' &
         & ,e13.3e3,' delta',e13.3e3)
16  Format('SYMTST: The rot(v)-test is not OK.  ' &
         & ,/,' It means, that the application of the space'&
         & ,' group symmetry to a fully '&
         & ,/,' symmetrical function does not'&
         & ,' give the same result!'&
         & ,/,' May be the the symtest-tolerance is to small',/ &
         & ,/,' If not, please report this to the programmers !')
17  Format('SYMTST: The rot(v)-test is  OK.')


112 Format(' site ',i3,' ir ',i3,' nL ',i3,' vi ',e13.3e3&
         & ,'<> rot(vi) ',e13.3e3,' delta ',e13.3e3)
116  Format('SYMTST: The rot(vi)-test is not OK. ' &
          & ,/,' It means, that the application of the space&
          & group symmetry to the irreducible'&
          &,/,' integrated  function does not give the same &
          &as the full integration!'&
          & ,/,' May be the the symtest-tolerance is to small' &
          & ,/,' If not, please report this to the programmers !')
117  Format('SYMTST: The rot(vi)-test is  OK.')
    !!
  End Subroutine vvvcmp
  !!***********************************************************************
  
End Module tests
