Module tforces
  !!
  Use precisiondefs
  Use titeration
  Use errors
  Use memory
  !!
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TFORCES"  
  !!
  Public :: T_Forces,T_SymTrans&
       &,FORCE_MODE_NONE,FORCE_MODE_GEO_OPT,FORCE_MODE_SINGLE_FSTEP&
       &,FORCE_MODES,FORCE_MODE_MAPPING&
       &,palloc_symtrans,pfree_symtrans,init_forces,clean_forces
  !!***********************************************************************
  Integer(t_int),Parameter   :: FORCE_MODE_NONE           =1_t_int
  Integer(t_int),Parameter   :: FORCE_MODE_GEO_OPT        =2_t_int
  Integer(t_int),Parameter   :: FORCE_MODE_SINGLE_FSTEP   =3_t_int
  !!
  Character(26)              :: FORCE_MODES(3)=(/&
       & "No forces                 "&
       &,"Site geometry optimization"&
       &,"Single force step         "&
       &/)
  !!
  Integer(t_int),Parameter   :: FORCE_MODE_MAPPING(3)=(/&
       & FORCE_MODE_NONE,FORCE_MODE_GEO_OPT,FORCE_MODE_SINGLE_FSTEP&
       &/)
  !!
  !!***********************************************************************
  Type T_SymTrans
     !! analyse the possible free parameter/translations
     !! Pop is the symmetry projector
     Real(t_real)   :: Pop(3,3)
     Real(t_real)   :: Qop(3,3)
     !! translation:there are nparal allowed and nperp forbidden 
     !!     directions for a translational invariance of the energy in
     !!     the current symmetry
     !! free parameter: there are nparal free parameter for the site
     !!     and nperp fixed parameter
     Integer(t_int) :: nperp,nparal
     !! the corresponding directions (only nperp/nparal used)
     Real(t_real)   ::  perp(3,3),paral(3,3)
     !! the site, which fixes the sites with respect to the allowed
     !! global shifts
     Integer(t_int) :: issashift
     !! Pop*p_tau(isashift)
     Real(t_real)   :: Pshift(3)
     !! the fixed(perp) part of the site (for free parameters)
     Real(t_real)   :: Qtau(3)
     !! for free-parameter analysis: is this site in the list of 
     !! sites to be varied
     Logical(t_log) :: do_vary
  End Type T_SymTrans
  !!***********************************************************************
  !!
  Type T_Forces
     !! forces mode
     Integer(t_int) :: force_mode
     Logical(t_log) :: force_in_each_step
     !!
     Logical(t_log) :: approx_nodndL_ewald
     !! Hellman Feynman
     Real(t_real),Pointer   :: p_hfforces(:,:) ! 3,nsite 
     !! core pulay
     Real(t_real),Pointer   :: p_cplforces(:,:) ! 3,nsite
     !! full pulay
     Real(t_real),Pointer   :: p_plforces(:,:) ! 3,nsite
     Real(t_real),Pointer   :: p_dndvforces(:,:) ! 3,nsite
     !! total force
     Real(t_real),Pointer   :: p_forces(:,:) ! 3,nsite
     !!
     Integer(t_int) :: nsitehistory
     Real(t_real),Pointer :: p_sitehistory(:,:,:) !3,nsite,count
     !! which sites to vary, empty means all
     Integer(t_int) :: ngeomopt_sites
     Integer(t_int),Pointer :: p_geomopt_sites(:) ! ngeomopt_sites
     !!
     Type(T_iteration)  :: forceiteration_control
     !!
     !!
     !! this defines the free shifts of the sites, which leaves
     !! the total energy invariant
     Type(T_SymTrans) :: symtrans
     !! this defines the free parameters of all sites, determined
     !! due to symmetry
     Type(T_SymTrans),Pointer :: p_sitefreeparam(:)
     !! the number of symmetry induced free parameter minus number
     !! of parameters reduced by translational reduction
     Integer(t_int) :: Ndegfree
     !!
  End Type T_Forces
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine init_forces(fo)
    Implicit None 
    !!IO
    Type(T_Forces),Intent(inout) :: fo
    !!LOCAL
    fo% ngeomopt_sites=0
    Nullify(fo% p_geomopt_sites)
    !!
    Nullify(fo% p_sitefreeparam)
    !!
    Nullify(fo% p_hfforces)
    Nullify(fo% p_cplforces)
    Nullify(fo% p_plforces)
    Nullify(fo% p_dndvforces)
    Nullify(fo% p_forces)
    Nullify(fo% p_sitehistory)
    fo% nsitehistory=0_t_int
    fo% force_mode=FORCE_MODE_NONE
    fo% force_in_each_step=.False.
    fo% approx_nodndL_ewald=.False.
    !!
  End Subroutine init_forces
  !!***********************************************************************
  Subroutine clean_forces(fo)
    Implicit None 
    !!IO
    Type(T_Forces),Intent(inout) :: fo
    !!LOCAL
    fo% ngeomopt_sites=0
    Call pfree(fo% p_geomopt_sites)
    !!
    Call pfree_symtrans(fo% p_sitefreeparam)
    !!
    Call pfree(fo% p_hfforces)
    Call pfree(fo% p_cplforces)
    Call pfree(fo% p_plforces)
    Call pfree(fo% p_dndvforces)
    Call pfree(fo% p_forces)
    Call pfree(fo% p_sitehistory)
    fo% nsitehistory=0_t_int
    !!
  End Subroutine clean_forces
  !!***********************************************************************
  Subroutine palloc_symtrans(name,st,N,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N
    Type(T_SymTrans),Pointer     :: st(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    !Character(*),Parameter::nsubfunc="palloc_symtrans"
    !Integer(t_int) :: i
    Integer :: icheck
    !!
    !!
    Call pfree_symtrans(st)
    !!
    Allocate(st(1:N),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    !Do i=1,N
    !   Call init_mesh(meshs(i))
    !Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_symtrans
  !!***********************************************************************
  Subroutine pfree_symtrans(st)
    Implicit None 
    !!IO
    Type(T_SymTrans),Pointer :: st(:)
    !!LOCAL
    !Integer(t_int) :: i
    !External crealmemaddress
    !!
    !!
    If(Associated(st))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       !Do i=1,Size(st)
       !   Call cleanup_mesh(st(i))
       !enddo
       !!
       Deallocate(st)
       Nullify(st)
    Endif
    !!
  End Subroutine pfree_symtrans
  !!***********************************************************************
  
End Module tforces
