Module tgrid
  !!
  Use precisiondefs
  Use memory
  Use errors
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TGRID"  
  !!
  Public :: T_Grid,T_GridPlot,T_EnergyWindowResolved&
       &,init_gridplot,cleanup_gridplot&
       &,init_grid,cleanup_grid,palloc_grids,pfree_grids &
       &,palloc_energywindowresolved&
       &,GRID_TYPE_REGULAR,GRID_TYPE_CUSTOM,GRID_TYPES &
       &,GRID_BASIS_PRIMITIVE,GRID_BASIS_CONVENTIONAL,GRID_BASIS_TYPES&
       &,GRID_BASIS_TYPES_HKEY &
       &,GRID_DIRECT_LATTICE,GRID_RECIP_LATTICE,GRID_LATTICE_TYPES&
       &,GRID_BASIS_CARTESIAN&
       &,MAP_GRID_BASIS_TYPES&
       &,gridquantities,GRID_QUANTITY_NONE,GRID_QUANTITY_TOTAL_DENSITY&
       &,GRID_QUANTITY_TOTAL_POTENITAL,GRID_QUANTITY_SCF_WAVEFNCT&
       &,GRID_QUANTITY_COULOMB_POTENITAL,GRID_QUANTITY_XC_POTENITAL&
       &,GRID_QUANTITY_EWALD_POTENITAL&
       &,GRID_QUANTITIES_HKEY&
       &,gridoutputdata,GRID_OUTPUTDATA_NONE,GRID_OUTPUTDATA_INDEX&
       &,GRID_OUTPUTDATA_POINTS,GRID_OUTPUTDATA_TOTAL&
       &,GRID_OUTPUTDATA_SPIN,GRID_OUTPUTDATA_SPINUP,GRID_OUTPUTDATA_SPINDOWN&
       &,GRID_OUTPUTDATA_PSI&
       &,GRID_OUTPUTDATA,GRID_OUTPUTDATA_HKEY&
       &,T_KResolved,palloc_kresolved,outputdataIsValid&
       &,init_kresolved,cleanup_kresolved
  !!***********************************************************************
  !! definition of molecular orbital type
  Type T_GridPlot
     !!
     Logical(t_log) :: active
     Logical(t_log) :: stopafter
     !!
     Integer(t_int) :: ngrids
     !!
     Type(T_grid),Pointer :: p_grids(:)
     !!
  End Type T_GridPlot
  !!***********************************************************************
  Type T_EnergyWindowResolved
     Integer(t_int) :: nsections
     Character(100) :: name
     Real(t_real),Pointer :: p_emin(:)
     Real(t_real),Pointer :: p_emax(:)
     Real(t_real),Pointer :: p_de(:)
     Integer(t_int),Pointer :: p_spin(:)
  End Type T_EnergyWindowResolved
  !!***********************************************************************
  Type T_KResolved
     Character(500) :: name
     Real(t_real)   :: kpoint(3)
     !! user input: which bands
     Integer(t_int) :: nbandindices
     Integer(t_int),Pointer :: p_bandindices(:)!nbands or 0
     Real(t_real)   :: emin
     Real(t_real)   :: emax
  End Type T_KResolved
  !!***********************************************************************
  Type T_Grid
     !!
     Integer(t_int) :: gridtype
     Integer(t_int) :: basis
     Integer(t_int) :: latticetype
     Real(t_real)   :: directions(3,3)
     Real(t_real)   :: origin(3)
     Integer(t_int) :: subdivisions(3)
     Logical(t_log) :: InclPeriodicPoints
     Integer(t_int) :: nq
     Integer(t_int),Pointer :: quantities(:)
     Integer(t_int) :: no
     Integer(t_int),Pointer :: outputdata(:)
     Real(t_real)   :: scfergwindow(2)
     Character(200) :: pointfile
     Logical(t_log) :: createopendx
     Logical(t_log) :: fixzero
     !!
     Real(t_real)   :: delta(3,3) !! the resulting grid unit directions
     !! to be multipplied by i1,i2,i3 to build the grid
     Real(t_real)   :: o(3) !! the resulting origin
     Integer(t_int) :: n123(3) !! resulting  mesh point numbers
     Real(t_real)   :: ibox(3,3) !! inverse of delta*n123, for box testing
     Integer(t_int) :: npoints
     Real(t_real),Pointer :: points(:,:) !! 3,npoints
     Integer(t_int) :: nenergywindowresolved
     Type(T_EnergyWindowResolved),Pointer :: p_energywindowresolved(:)
     Integer(t_int) :: nkresolved
     Type(T_KResolved),Pointer :: p_kresolved(:)
  End Type T_Grid
  !!***********************************************************************
  !! grid types
  Integer(t_int),Parameter :: GRID_TYPE_REGULAR=1_t_int
  Integer(t_int),Parameter :: GRID_TYPE_CUSTOM =2_t_int
  Character(7)   :: GRID_TYPES(1:2)
  !!
  Data GRID_TYPES(1:2) /'regular','custom '/
  !!
  !!
  Integer(t_int),Parameter :: GRID_DIRECT_LATTICE=1_t_int
  Integer(t_int),Parameter :: GRID_RECIP_LATTICE=2_t_int
  Character(11)   :: GRID_LATTICE_TYPES(1:2)
  !!
  Data GRID_LATTICE_TYPES(1:2) /'direct     ','reciprocial'/
  !!
  !! regular grid basis types
  Integer(t_int),Parameter :: GRID_BASIS_PRIMITIVE   =1_t_int
  Integer(t_int),Parameter :: GRID_BASIS_CONVENTIONAL=2_t_int
  Integer(t_int),Parameter :: GRID_BASIS_CARTESIAN   =3_t_int
  !!
  Integer(t_int) :: MAP_GRID_BASIS_TYPES(3)
  Data MAP_GRID_BASIS_TYPES/&
       &GRID_BASIS_PRIMITIVE&
       &,GRID_BASIS_CONVENTIONAL&
       &,GRID_BASIS_CARTESIAN /
  !!
  Character(12)              :: GRID_BASIS_TYPES(3)
  Data GRID_BASIS_TYPES(1:3) /&
       & 'primitive   '&
       &,'conventional'&
       &,'cartesian   '/
  Character(15)              :: GRID_BASIS_TYPES_HKEY(3)
  Data GRID_BASIS_TYPES_HKEY(1:3) /&
       & '@(p)rimitive   '&
       &,'@(c)onventional'&
       &,'c@(a)rtesian   '/
  !!
  !! grid quantities
  Integer(t_int),Parameter :: GRID_QUANTITY_NONE         =0_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_TOTAL_DENSITY      =1_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_TOTAL_POTENITAL    =2_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_SCF_WAVEFNCT       =3_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_COULOMB_POTENITAL  =4_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_XC_POTENITAL       =5_t_int
  Integer(t_int),Parameter :: GRID_QUANTITY_EWALD_POTENITAL    =6_t_int
  Character(17)            :: GRID_QUANTITIES(6)
  Data GRID_QUANTITIES(1:6) /&
       & 'Density          '&
       &,'Potential        '&
       &,'SCF Wavefnct     '&
       &,'Coulomb Potential'&
       &,'XC Potential     '&
       &,'Ewald Potential  '&
       &/
  Character(23)              :: GRID_QUANTITIES_HKEY(6)
  Data GRID_QUANTITIES_HKEY(1:6) /&
       & 'Total @(D)ensity       '&
       &,'Total @(P)otential     '&
       &,'SCF   @(W)ave functions'&
       &,'@(C)oulomb Potential   '&
       &,'XC Po@(t)ential        '&
       &,'@(E)wald Potential     '&
       &/
  !! grid outpudata
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_NONE     =0_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_INDEX    =1_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_POINTS   =2_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_TOTAL    =3_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_SPIN     =4_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_SPINUP   =5_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_SPINDOWN =6_t_int
  Integer(t_int),Parameter :: GRID_OUTPUTDATA_PSI      =7_t_int
  Character(12)              :: GRID_OUTPUTDATA(7)
  Data GRID_OUTPUTDATA(1:7) /&
       & 'Index       '&
       &,'Grid points '&
       &,'Total       '&
       &,'Spin        '&
       &,'Spinup      '&
       &,'Spindown    '&
       &,'Psi         '&
       &/
  Character(18)              :: GRID_OUTPUTDATA_HKEY(7)
  Data GRID_OUTPUTDATA_HKEY(1:7) /&
       & '@(I)\tIndex       '&
       &,'@(P)\tGrid points '&
       &,'@(T)\tTotal       '&
       &,'@(S)\tSpin        '&
       &,'@(U)\tSpinup      '&
       &,'@(D)\tSpindown    '&
       &,'@(C)\tComponents  '&
       &/
  
Contains
  !!***********************************************************************
  Subroutine init_grid(gr)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(inout)   :: gr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_grid"
    !!
    !!
    gr% gridtype=GRID_TYPE_REGULAR
    gr% basis = GRID_BASIS_PRIMITIVE
    gr% latticetype = GRID_DIRECT_LATTICE
    gr% origin(:) = (/ 0.0_t_real , 0.0_t_real , 0.0_t_real /)
    gr% directions(:,1) = (/ 1.0_t_real , 0.0_t_real , 0.0_t_real /)
    gr% directions(:,2) = (/ 0.0_t_real , 1.0_t_real , 0.0_t_real /)
    gr% directions(:,3) = (/ 0.0_t_real , 0.0_t_real , 1.0_t_real /)
    gr% subdivisions=(/ 1_t_int , 1_t_int , 1_t_int /)
    gr% InclPeriodicPoints=.True.
    gr% createopendx=.True.
    gr% fixzero=.False.
    gr% npoints=0
    gr% pointfile=""
    gr% scfergwindow(:)=(/1.0_t_real,-1.0_t_real/)
    Nullify(gr% points)
    gr% nq=0
    Nullify(gr% quantities)
    gr% no=0
    Nullify(gr% outputdata)
    Nullify(gr% p_energywindowresolved)
    gr% nenergywindowresolved=0
    Nullify(gr% p_kresolved)
    gr% nkresolved=0
    !!
    Return
    !!
  End Subroutine init_grid
  !!***********************************************************************
  Subroutine cleanup_grid(gr)
    !!
    Implicit None
    !!IO
    Type(T_Grid),Intent(inout)   :: gr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_grid"
    !!
    !!
    Call pfree(gr% points)
    Call pfree(gr% quantities)
    Call pfree(gr% outputdata)
    Call pfree_energywindowresolved(gr% p_energywindowresolved)
    gr% nenergywindowresolved=0
    Call pfree_kresolved(gr% p_kresolved)
    gr% nkresolved=0
    !!
    Return
    !!
  End Subroutine cleanup_grid
  !!***********************************************************************
  Subroutine palloc_grids(name,grids,N,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N
    Type(T_Grid),Pointer         :: grids(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    !Character(*),Parameter::nsubfunc="palloc_grids"
    Integer(t_int) :: i
    Integer :: icheck
    !!
    If(N<=0) Return
    !!
    Call pfree_grids(grids)
    !!
    Allocate(grids(1:N),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    Do i=1,N
       Call init_grid(grids(i))
    Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_grids
  !!***********************************************************************
  Subroutine pfree_grids(grids)
    Implicit None 
    !!IO
    Type(T_Grid),Pointer         :: grids(:)
    !!LOCAL
    Integer(t_int) :: i
    External crealmemaddress
    !!
    !!
    If(Associated(grids))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       Do i=1,Size(grids)
          Call cleanup_grid(grids(i))
       Enddo
       !!
       Deallocate(grids)
       Nullify(grids)
    Endif
    !!
  End Subroutine pfree_grids
  !!***********************************************************************
  Subroutine init_gridplot(gp)
    !!
    Implicit None
    !!IO
    Type(T_GridPlot),Intent(inout) :: gp
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_gridplot"
    !!
    gp% active=.False.
    gp% stopafter=.False.
    gp% ngrids=0
    Nullify(gp% p_grids)
    !!
    !!
    Return
    !!
  End Subroutine init_gridplot
  !!***********************************************************************
  Subroutine cleanup_gridplot(gp)
    !!
    Implicit None
    !!IO
    Type(T_GridPlot),Intent(inout) :: gp
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_gridplot"
    !!
    !!
    Call pfree_grids(gp% p_grids)
    !!
    Return
    !!
  End Subroutine cleanup_gridplot
  !!***********************************************************************
  Character(100) Function gridquantities(i)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: i
    !!LOCAL
    !Character(*),Parameter::nsubfunc="gridquantities"
    !!
    !!
    If (   ( i < GRID_QUANTITY_TOTAL_DENSITY ) .Or. &
         & ( i > GRID_QUANTITY_EWALD_POTENITAL )    ) Then
       gridquantities="unknown quantity"
    Else
       gridquantities=GRID_QUANTITIES(i)
    Endif
    !!
    Return
    !!
  End Function gridquantities
  !!***********************************************************************
  Character(100) Function gridoutputdata(i)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: i
    !!LOCAL
    !Character(*),Parameter::nsubfunc="gridoutputdata"
    !!
    !!
    If (outputdataIsValid(i)) Then
       gridoutputdata=GRID_OUTPUTDATA(i)
    Else
       gridoutputdata="unknown outputdata"
    Endif
    !!
    Return
    !!
  End Function gridoutputdata
  !!***********************************************************************
  Logical(t_log) Function outputdataIsValid(i)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: i
    !!LOCAL
    !Character(*),Parameter::nsubfunc="outputdataIsValid"
    !!
    !!
    If (   ( i < GRID_OUTPUTDATA_INDEX ) .Or. &
         & ( i > GRID_OUTPUTDATA_PSI )    ) Then
       outputdataIsValid=.False.
    Else
       outputdataIsValid=.true.
    Endif
    !!
    Return
    !!
  End Function outputdataIsValid
  !!***********************************************************************
  Subroutine init_energywindowresolved(ew)
    !!
    Implicit None
    !!IO
    Type(T_EnergyWindowResolved),Intent(inout)   :: ew
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_energywindowresolved"
    !!
    Nullify(ew% p_emin)
    Nullify(ew% p_emax)
    Nullify(ew% p_de)
    Nullify(ew% p_spin)
    ew% nsections=0
    ew% name="erdens"
    !!
    !!
    Return
    !!
  End Subroutine init_energywindowresolved
  !!***********************************************************************
  Subroutine cleanup_energywindowresolved(ew)
    !!
    Implicit None
    !!IO
    Type(T_EnergyWindowResolved),Intent(inout)   :: ew
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_energywindowresolved"
    !!
    Call pfree(ew% p_emin)
    Call pfree(ew% p_emax)
    Call pfree(ew% p_de)
    Call pfree(ew% p_spin)
    ew% nsections=0
    !!
    !!
    Return
    !!
  End Subroutine cleanup_energywindowresolved
  !!***********************************************************************
  Subroutine palloc_energywindowresolved(name,ews,N,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N
    Type(T_EnergyWindowResolved),Pointer :: ews(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    !Character(*),Parameter::nsubfunc="palloc_energywindowresolved"
    Integer(t_int) :: i
    Integer :: icheck
    !!
    !!
    Call pfree_energywindowresolved(ews)
    !!
    Allocate(ews(1:N),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    Do i=1,N
       Call init_energywindowresolved(ews(i))
    Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_energywindowresolved
  !!***********************************************************************
  Subroutine pfree_energywindowresolved(ews)
    Implicit None 
    !!IO
    Type(T_EnergyWindowResolved),Pointer :: ews(:)
    !!LOCAL
    Integer(t_int) :: i
    External crealmemaddress
    !!
    !!
    If(Associated(ews))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       Do i=1,Size(ews)
          Call cleanup_energywindowresolved(ews(i))
       Enddo
       !!
       Deallocate(ews)
       Nullify(ews)
    Endif
    !!
  End Subroutine pfree_energywindowresolved
  !!***********************************************************************
  Subroutine init_kresolved(kr)
    !!
    Implicit None
    !!IO
    Type(T_KResolved),Intent(inout)   :: kr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_kresolved"
    !!
    Nullify(kr% p_bandindices)
    kr% nbandindices=0_t_int
    kr% name="k-dens"
    kr% kpoint=(/0.0_t_real,0.0_t_real,0.0_t_real/)
    kr% emin=1.0_t_real
    kr% emax=-1.0_t_real
    !!
    !!
    Return
    !!
  End Subroutine init_kresolved
  !!***********************************************************************
  Subroutine cleanup_kresolved(kr)
    !!
    Implicit None
    !!IO
    Type(T_KResolved),Intent(inout)   :: kr
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_kresolved"
    !!
    Call pfree(kr% p_bandindices)
    !!
    !!
    Return
    !!
  End Subroutine cleanup_kresolved
  !!***********************************************************************
  Subroutine palloc_kresolved(name,krs,N,nmod,nsub)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: N
    Type(T_Kresolved),Pointer :: krs(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    !Character(*),Parameter::nsubfunc="palloc_kresolved"
    Integer(t_int) :: i
    Integer :: icheck
    !!
    !!
    Call pfree_kresolved(krs)
    !!
    Allocate(krs(1:N),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    Do i=1,N
       Call init_kresolved(krs(i))
    Enddo
    !!
    !If(STATISTICS)Then
    !   Call crealmemaddress(ra,add)
    !   Call register_mem_stat(add,nmod,nsub,name,TYPE_T_REAL&
    !        &,size10,size11&
    !        &)
    !Endif
    !!
    !!
    Return
    !!
  End Subroutine palloc_kresolved
  !!***********************************************************************
  Subroutine pfree_kresolved(krs)
    Implicit None 
    !!IO
    Type(T_Kresolved),Pointer :: krs(:)
    !!LOCAL
    Integer(t_int) :: i
    External crealmemaddress
    !!
    !!
    If(Associated(krs))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       Do i=1,Size(krs)
          Call cleanup_kresolved(krs(i))
       Enddo
       !!
       Deallocate(krs)
       Nullify(krs)
    Endif
    !!
  End Subroutine pfree_kresolved
  !!***********************************************************************

End Module tgrid
