Module titeration
  !!
  Use precisiondefs
  Use memory
  !!
  Implicit None 
  !!
  Private 
  !!
  Public  :: T_iteration,init_iterationcontrol,clean_iterationcontrol&
       &,IT_ENTRY_INIT,IT_ENTRY_PREPARE,IT_ENTRY_STARTDATA&
       &,IT_ENTRY_LOOP,IT_ENTRY_EXIT,IT_ENTRY_BEST&
       !! short description of all schemes and entries
       &,ITERATION_SCHEMES,ITERATION_ENTRIES,FORCE_IT_SCHEME_MAPPING&
       !! schemes
       &,IT_SCHEME_SIMPLEMIX,IT_SCHEME_ITERAT,IT_SCHEME_LCITERAT&
       &,IT_SCHEME_NITERAT&
       &,IT_SCHEME_BROYDEN,IT_SCHEME_ITSOL,IT_SCHEME_MINITERAT&
       &,IT_SCHEME_MAPPING&
       &,CONVCOND_ITERAT,CONVCOND_ANY,CONVCOND_BOTH,CONVCOND_LETSSTOP&
       &,ITERATION_CONVCONDITIONS,IterationHandler
  !! type declaration 
  Type T_iteration
     Character(100):: name
     Integer(t_int):: scheme
     Integer(t_int):: vecdim
     Integer(t_int):: maxdim
     Real(T_real)  :: tolerance
     Real(T_real)  :: mixing
     Real(T_real)  :: progress
     Real(T_real)  :: deviation
     Real(T_real)  :: alpha1
     Real(T_real)  :: alpha2
     Integer(t_int):: steps
     Integer(t_int):: step
     Integer(t_int) :: convcond
     Logical(t_log):: reset
     Logical(t_log):: letsstop !! program requires an iteration stop
     !! testing/development
     Integer(t_int):: nprojector
     Real(t_real),Pointer :: projector(:,:)
     !!
     !! operational variables
     !!
     Logical(t_log) :: prot,notermination,stepsexceeded
     Real(t_real)   :: erg
     !! linear vector containing all data which must be iteratet (densit)
     Real(t_real),Pointer :: vector(:) !! 1:vecdim
     !older gfortran dont like this Procedure(IterationHandler),Nopass,Pointer:: handler
     !!
  End Type T_iteration
  !!***********************************************************************
  !! constants decl
  !!***********************************************************************
  !! iteration stop conditions
  !!    stop if any of both 
  Integer(t_int),Parameter   :: CONVCOND_ANY      =1_t_int
  !!    stop if iterat converged
  Integer(t_int),Parameter   :: CONVCOND_ITERAT   =2_t_int
  !!    stop if both say stop
  Integer(t_int),Parameter   :: CONVCOND_BOTH     =3_t_int
  !!    stop if program requires it
  Integer(t_int),Parameter   :: CONVCOND_LETSSTOP =4_t_int
  Character(50)              :: ITERATION_CONVCONDITIONS(4)=(/&
       & "Any                                               "&
       &,"Iterat                                            "&
       &,"Both                                              "&
       &,"Program                                           "&
       &/)

  !! handlerentry types
  Integer(t_int),Parameter   :: IT_ENTRY_INIT     =1_t_int
  Integer(t_int),Parameter   :: IT_ENTRY_PREPARE  =2_t_int
  Integer(t_int),Parameter   :: IT_ENTRY_STARTDATA=3_t_int
  Integer(t_int),Parameter   :: IT_ENTRY_LOOP     =4_t_int
  Integer(t_int),Parameter   :: IT_ENTRY_EXIT     =5_t_int
  Integer(t_int),Parameter   :: IT_ENTRY_BEST     =6_t_int
  !! iteration scheme descriptions
  Character(11)              :: ITERATION_ENTRIES(6)
  Data ITERATION_ENTRIES&
       &/"init      "&
       &,"prepare   "&
       &,"start data"&
       &,"loop      "&
       &,"exit      "&
       &,"best      "&
       &/
  !!
  !!***********************************************************************
  !! iteration scheme types
  Integer(t_int),Parameter   :: IT_SCHEME_SIMPLEMIX=1_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_ITERAT   =2_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_BROYDEN  =3_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_LCITERAT =4_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_ITSOL    =5_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_MINITERAT=6_t_int
  Integer(t_int),Parameter   :: IT_SCHEME_NITERAT  =7_t_int
  !! iteration scheme descriptions
  Character(48)              :: ITERATION_SCHEMES(7)
  Integer(t_int),Parameter   :: IT_SCHEME_MAPPING(3)=(/&
       & IT_SCHEME_SIMPLEMIX,IT_SCHEME_ITERAT,IT_SCHEME_LCITERAT&
       !&,IT_SCHEME_MINITERAT&
!       &,IT_SCHEME_NITERAT&
       &/)
  Integer(t_int),Parameter   :: FORCE_IT_SCHEME_MAPPING(1)=(/&
       &IT_SCHEME_MINITERAT&
       &/)
  !!
  Data ITERATION_SCHEMES&
       &/"SimpleMix: Pratt's scheme                       "&
       &,"Iterat   : Hyperspace interpolation             "&
       &,"Broyden                                         "&
       &,"LCIterat : Iterat with linear progress control  "&
       &,"Itsol    : Iterat with complete progress control"&
       &,"MinIterat: Find minimum with iterat             "&
       &,"NIterat  : New version                          "&
       &/
  !!***********************************************************************
  Abstract Interface 
     !! this is the interface for the callback functions like sccfplo
     Subroutine IterationHandler(Data,Entry)
       !!
       Use precisiondefs
       Import :: T_iteration
       !!
       Implicit None 
       !!IO
       Integer(t_int),Intent(in)      ::Entry
       Type(T_iteration),Intent(inout)::Data
       !!LOCAL
     End Subroutine IterationHandler
  End Interface
  !!***********************************************************************
Contains  
  Subroutine init_iterationcontrol(ic)
    Implicit None 
    !!IO
    Type(T_iteration),Intent(inout) :: ic
    !!LOCAL
    !! avoid warnings
    ic% vecdim=0_t_int
    ic% nprojector=0_t_int
    Nullify(ic% projector)
    Nullify(ic% vector)
    !!
  End Subroutine init_iterationcontrol
  !!***********************************************************************
  Subroutine clean_iterationcontrol(ic)
    Implicit None 
    !!IO
    Type(T_iteration),Intent(inout) :: ic
    !!LOCAL
    !! avoid warnings
    ic% vecdim=0_t_int
    ic% nprojector=0_t_int
    Call pfree(ic% projector)
    Call pfree(ic% vector)
    !!
  End Subroutine clean_iterationcontrol
    !!***********************************************************************


End Module titeration

