Module tloi
  !!
  Use precisiondefs
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TLOI"  
  !!
  Public :: T_Loi,init_loi,cleanup_loi
  !!
  !!***********************************************************************
  !! v= valence, c= core, H=Hamiltonian
  Type  T_Loi
     !! ocidim: dimension of one-center arrays
     !! ocivcdim: dimension of one-center valence-core arrays
     !! tcivvdim: dimension of two-center arrays
     !! tcivcdim: dimension of two-center valence-core arrays
     Integer(t_int)         :: ocivvdim,ocivcdim,tcivvdim,tcivcdim
     !! core-core Hamiltonian
     Integer(t_int) :: ociccdim
     Integer(t_int),Pointer :: p_ociccindex(:)      !! nsite
     Integer(t_int),Pointer :: p_ocivvindex(:)      !! nsite
     Integer(t_int),Pointer :: p_ocivcindex(:)    !! nsite
     Integer(t_int),Pointer :: p_tcivvindex(:,:,:)  !! npairsmax,nsite,nsite
     Integer(t_int),Pointer :: p_tcivcindex(:,:,:)!! npairsmax,nsite,nsite
     !! cc-OCI
     Real(t_real),Pointer   :: p_oci_cc0(:)     !! ociccdim
     Real(t_real),Pointer   :: p_oci_cHc0(:)    !! ociccdim
     Real(t_real),Pointer   :: p_oci_cHc(:,:)   !! ociccdim,mspin
     !! vv-OCI
     Real(t_real),Pointer   :: p_oci_vv0(:)     !! ocivvdim
     Real(t_real),Pointer   :: p_oci_vHv0(:)    !! ocivvdim
     Real(t_real),Pointer   :: p_oci_vHv(:,:)   !! ocivvdim,mspin
     !! vc-OCI
     Real(t_real),Pointer   :: p_oci_vc0(:)     !! ocivcdim
     Real(t_real),Pointer   :: p_oci_vHc0(:)    !! ocivcdim
     Real(t_real),Pointer   :: p_oci_vHc(:,:)    !! ocivcdim,mspin
     !!
     !! vv-TCI
     Real(t_real),Pointer   :: p_tci_vv0(:)     !! tcivvdim
     Real(t_real),Pointer   :: p_tci_vHv0(:)    !! tcivvdim
     Real(t_real),Pointer   :: p_tci_vHv(:,:)   !! tcivvdim,mspin
     !! vc-TCI
     Real(t_real),Pointer   :: p_tci_vc0(:)     !! tcivcdim
     Real(t_real),Pointer   :: p_tci_vHc0(:)    !! tcivcdim
     Real(t_real),Pointer   :: p_tci_vHc(:,:)   !! tcivcdim,mspin
     !! relativistic arrays --------------------------------------------
     !! relativistic spin character ( <|\beta\Sigma_z|>):  labeled [vc]bS[vc]
     !! cc-OCI
     Complex(t_real),Pointer:: p_roci_cHc(:)    !! ociccdim
     Complex(t_real),Pointer:: p_cbSc0(:,:,:)   !! ncsmx,ncsmx,nsite
     !! vv-OCI
     Complex(t_real),Pointer:: p_roci_vHv(:)    !! ocivvdim
     Complex(t_real),Pointer:: p_vbSv0(:,:,:)   !! nvsmx,nvsmx,nsite
     !! vc-OCI
     Complex(t_real),Pointer:: p_roci_vHc(:)    !! ocivcdim
     Complex(t_real),Pointer:: p_vbSc0(:,:,:)   !! nvsmx,ncsmx,nsite
     !! vv-TCI
     Complex(t_real),Pointer:: p_rtci_vHv(:)    !! tcivvdim
     Complex(t_real),Pointer:: p_rtci_vHv0(:)   !! tcivvdim
     Complex(t_real),Pointer:: p_rtci_vv0(:)    !! tcivvdim
     Complex(t_real),Pointer:: p_rtci_vbSv0(:)  !! tcivvdim
     !! vc-TCI
     Complex(t_real),Pointer:: p_rtci_vc0(:)    !! tcivcdim
     Complex(t_real),Pointer:: p_rtci_vHc0(:)   !! tcivcdim
     Complex(t_real),Pointer:: p_rtci_vHc(:)    !! tcivcdim
     Complex(t_real),Pointer:: p_rtci_vbSc0(:)  !! tcivcdim
     !!
     !! the complete spin vector in global coordinates
     !! cc-OCI
     Complex(t_real),Pointer:: p_cbVSc0(:,:,:,:)   !! ncsmx,ncsmx,nsite,3
     !! vv-OCI
     Complex(t_real),Pointer:: p_vbVSv0(:,:,:,:)   !! nvsmx,nvsmx,nsite,3
     !! vc-OCI
     Complex(t_real),Pointer:: p_vbVSc0(:,:,:,:)   !! nvsmx,ncsmx,nsite,3
     !! vv-TCI
     Complex(t_real),Pointer:: p_rtci_vbVSv0(:,:)  !! tcivvdim,3
     !! vv-TCI
     Complex(t_real),Pointer:: p_rtci_vbVSc0(:,:)  !! tcivcdim,3
     !!
     !! core-valence matrix elements after core-contractio
     !! NREL
     Real(t_real),Pointer   :: p_oci_vchi(:,:)
     Real(t_real),Pointer   :: p_tci_vchi(:,:)
     Real(t_real),Pointer   :: p_oci_vHchi(:,:)
     Real(t_real),Pointer   :: p_tci_vHchi(:,:)
     Real(t_real),Pointer   :: p_oci_vcraw(:,:)
     Real(t_real),Pointer   :: p_tci_vcraw(:,:)
     !!REL
     Complex(t_real),Pointer:: p_roci_vchi(:)
     Complex(t_real),Pointer:: p_rtci_vchi(:)
     Complex(t_real),Pointer:: p_roci_vHchi(:)
     Complex(t_real),Pointer:: p_rtci_vHchi(:)
     Complex(t_real),Pointer:: p_rtci_vbSchi(:)
     Complex(t_real),Pointer:: p_vbSchi(:,:,:)
     Complex(t_real),Pointer:: p_chibSchi(:,:,:)
     Complex(t_real),Pointer:: p_roci_vcraw(:)
     Complex(t_real),Pointer:: p_rtci_vcraw(:)
     Real(t_real),Pointer   :: p_cspchar(:)
     !!
     Complex(t_real),Pointer:: p_rtci_vbVSchi(:,:)
     Complex(t_real),Pointer:: p_vbVSchi(:,:,:,:)
     Complex(t_real),Pointer:: p_chibVSchi(:,:,:,:)
     !!
     !! force matrix elements
     !! non-rel
     Logical(t_log) :: gloi0_calced
     !! cc-OCI
     Real(t_real),Pointer :: p_oci_gcc0 (:,:)    !! 3,ociccdim
     Real(t_real),Pointer :: p_oci_gcHc0(:,:)    !! 3,ociccdim
     Real(t_real),Pointer :: p_oci_gcHc (:,:,:)  !! 3,ociccdim,mspin
     !! vv-OCI
     Real(t_real),Pointer :: p_oci_gvv0 (:,:)    !! 3,ocivvdim
     Real(t_real),Pointer :: p_oci_gvHv0(:,:)    !! 3,ocivvdim
     Real(t_real),Pointer :: p_oci_gvHv (:,:,:)  !! 3,ocivvdim,mspin
     !! vc-OCI
     Real(t_real),Pointer :: p_oci_gvc0 (:,:)    !! 3,ocivcdim
     Real(t_real),Pointer :: p_oci_gvHc0(:,:)    !! 3,ocivcdim
     Real(t_real),Pointer :: p_oci_gvHcr0(:,:)   !! 3,ocivcdim
     Real(t_real),Pointer :: p_oci_gvHc (:,:,:)  !! 3,ocivcdim,mspin
     Real(t_real),Pointer :: p_oci_gvHcr(:,:,:)  !! 3,ocivcdim,mspin
     !!
     !! vv-TCI
     Real(t_real),Pointer :: p_tci_gvv0 (:,:)    !! 3,tcivvdim
     Real(t_real),Pointer :: p_tci_gvHv0(:,:)    !! 3,tcivvdim
     Real(t_real),Pointer :: p_tci_gvHv (:,:,:)  !! 3,tcivvdim,mspin
     !! vc-TCI
     Real(t_real),Pointer :: p_tci_gvc0 (:,:)    !! 3,tcivcdim
     Real(t_real),Pointer :: p_tci_gvHc0(:,:)    !! 3,tcivcdim
     Real(t_real),Pointer :: p_tci_gvHcr0(:,:)    !! 3,tcivcdim
     Real(t_real),Pointer :: p_tci_gvHc (:,:,:)  !! 3,tcivcdim
     Real(t_real),Pointer :: p_tci_gvHcr(:,:,:)  !! 3,tcivcdim
  End Type T_Loi
  !!***********************************************************************
Contains

  Subroutine init_loi(loi)
    Implicit None 
    !!IO
    Type(T_Loi),Intent(inout)    :: loi
    !!LOCAL
    !!
    loi% gloi0_calced=.False.
    !!
    Nullify(loi% p_ociccindex)
    Nullify(loi% p_ocivvindex)
    Nullify(loi% p_ocivcindex)
    Nullify(loi% p_tcivvindex)
    Nullify(loi% p_tcivcindex)
    Nullify(loi% p_oci_cc0)
    Nullify(loi% p_oci_cHc0)
    Nullify(loi% p_roci_cHc)
    Nullify(loi% p_oci_cHc)
    Nullify(loi% p_cbSc0)
    Nullify(loi% p_vbSv0)
    Nullify(loi% p_oci_vHv0)
    Nullify(loi% p_oci_vHv)
    Nullify(loi% p_roci_vHv)
    Nullify(loi% p_oci_vv0)
    Nullify(loi% p_oci_vc0)
    Nullify(loi% p_oci_vHc0)
    Nullify(loi% p_oci_vHc)
    Nullify(loi% p_roci_vHc)
    Nullify(loi% p_vbSc0)
    !!
    Nullify(loi% p_cbVSc0)
    Nullify(loi% p_vbVSv0)
    Nullify(loi% p_vbVSc0)
    Nullify(loi% p_rtci_vbVSv0)
    Nullify(loi% p_rtci_vbVSc0)
    !!
    Nullify(loi% p_oci_vchi)
    Nullify(loi% p_tci_vchi)
    Nullify(loi% p_oci_vHchi)
    Nullify(loi% p_tci_vHchi)
    Nullify(loi% p_roci_vchi)
    Nullify(loi% p_rtci_vchi)
    Nullify(loi% p_roci_vHchi)
    Nullify(loi% p_rtci_vHchi)
    Nullify(loi% p_rtci_vbSchi)
    Nullify(loi% p_vbSchi)
    Nullify(loi% p_chibSchi)
    Nullify(loi% p_rtci_vbVSchi)
    Nullify(loi% p_vbVSchi)
    Nullify(loi% p_chibVSchi)
    Nullify(loi% p_oci_vcraw)
    Nullify(loi% p_tci_vcraw)
    Nullify(loi% p_roci_vcraw)
    Nullify(loi% p_rtci_vcraw)
    Nullify(loi% p_cspchar)


    Nullify(loi% p_tci_vHv0)
    Nullify(loi% p_tci_vHv)
    Nullify(loi% p_tci_vv0)
    Nullify(loi% p_rtci_vHv)
    Nullify(loi% p_rtci_vv0)
    Nullify(loi% p_rtci_vc0)
    Nullify(loi% p_rtci_vHc0)
    Nullify(loi% p_rtci_vHc)
    Nullify(loi% p_rtci_vHv0)
    Nullify(loi% p_rtci_vbSv0)
    Nullify(loi% p_tci_vc0)
    Nullify(loi% p_tci_vHc0)
    Nullify(loi% p_tci_vHc)
    Nullify(loi% p_rtci_vbSc0)
    !! forces
    Nullify(loi% p_oci_gcc0)
    Nullify(loi% p_oci_gcHc0)
    Nullify(loi% p_oci_gcHc)
    !!
    Nullify(loi% p_oci_gvv0)
    Nullify(loi% p_oci_gvHv0)
    Nullify(loi% p_oci_gvHv)
    !!
    Nullify(loi% p_oci_gvc0)
    Nullify(loi% p_oci_gvHc0)
    Nullify(loi% p_oci_gvHcr0)
    Nullify(loi% p_oci_gvHc)
    Nullify(loi% p_oci_gvHcr)
    !!
    Nullify(loi% p_tci_gvv0)
    Nullify(loi% p_tci_gvHv0)
    Nullify(loi% p_tci_gvHv)
    !!
    Nullify(loi% p_tci_gvc0)
    Nullify(loi% p_tci_gvHc0)
    Nullify(loi% p_tci_gvHcr0)
    Nullify(loi% p_tci_gvHc)
    Nullify(loi% p_tci_gvHcr)
    !!
    !!
  End Subroutine init_loi
  !!***********************************************************************
  Subroutine cleanup_loi(loi)
    Implicit None 
    !!IO
    Type(T_Loi),Intent(inout) :: loi
    !!LOCAL
    !!
    Call pfree(loi% p_oci_cHc)
    Call pfree(loi% p_oci_vHv)
    Call pfree(loi% p_oci_vHc)
    Call pfree(loi% p_tci_vHv)
    Call pfree(loi% p_tci_vHc)
    !!
    Call pfree(loi% p_roci_cHc)
    Call pfree(loi% p_roci_vHv)
    Call pfree(loi% p_roci_vHc)
    Call pfree(loi% p_rtci_vHv)
    Call pfree(loi% p_rtci_vHc)
    !! forces
    Call pfree(loi% p_oci_gcHc)
    Call pfree(loi% p_oci_gvHv)
    Call pfree(loi% p_oci_gvHc)
    Call pfree(loi% p_oci_gvHcr)
    Call pfree(loi% p_tci_gvHv)
    Call pfree(loi% p_tci_gvHc)
    Call pfree(loi% p_tci_gvHcr)
    Call pfree(loi% p_tci_gvHv)
    !!
    Call pfree(loi% p_roci_vchi)
    Call pfree(loi% p_rtci_vchi)
    Call pfree(loi% p_roci_vHchi)
    Call pfree(loi% p_rtci_vHchi)
    Call pfree(loi% p_rtci_vbSchi)
    Call pfree(loi% p_vbSchi)
    Call pfree(loi% p_chibSchi)
    Call pfree(loi% p_rtci_vbVSchi)
    Call pfree(loi% p_vbVSchi)
    Call pfree(loi% p_chibVSchi)
    Call pfree(loi% p_cspchar)
    Call pfree(loi% p_roci_vcraw)
    Call pfree(loi% p_rtci_vcraw)
    Call pfree(loi% p_oci_vchi)
    Call pfree(loi% p_tci_vchi)
    Call pfree(loi% p_oci_vHchi)
    Call pfree(loi% p_tci_vHchi)
    Call pfree(loi% p_oci_vcraw)
    Call pfree(loi% p_tci_vcraw)
    !!
    !!
  End Subroutine cleanup_loi
  !!***********************************************************************

End Module tloi
