Module tlsdau
  !!
  Use precisiondefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TLSDAU"  
  !!
  Public :: T_Lsdau,init_lsdau,LSDAU_PROJECTIONS,LSDAU_FUNCTIONALS&
       &,LSDAU_AMF,LSDAU_AL&
       &,LSDAU_ORTHOGONAL,LSDAU_GROSS&
       &,LSDAU_HYBRIDIZED_CORE,LSDAU_HYBRIDIZED_NO_CORE&
       &,LSDAU_DMAT_MODES,LSDAU_DMAT_MODE_SC,LSDAU_DMAT_MODE_FIXED&
       &,LSDAU_FIXEDPOT
  !!***********************************************************************
  !! around mean field double counting term
  Integer(t_int),Parameter :: LSDAU_AMF=1_t_int
  !! Anissimov double counting term
  Integer(t_int),Parameter :: LSDAU_AL =2_t_int
  !!
  Integer(t_int),Parameter :: LSDAU_FIXEDPOT =3_t_int
  !! orthogonal projection technique
  Integer(t_int),Parameter :: LSDAU_ORTHOGONAL        =1_t_int
  !! formally basis independent projection technique without core orthogonal.
  Integer(t_int),Parameter :: LSDAU_HYBRIDIZED_NO_CORE=2_t_int
  !! formally basis independent projection technique with some core in the
  !! valence overlap matrix
  Integer(t_int),Parameter :: LSDAU_HYBRIDIZED_CORE   =3_t_int
  !! gross occu projection
  Integer(t_int),Parameter :: LSDAU_GROSS             =4_t_int
  !!
  !!
  Integer(t_int),Parameter :: LSDAU_DMAT_MODE_SC    =1_t_int
  Integer(t_int),Parameter :: LSDAU_DMAT_MODE_FIXED =2_t_int
  !!
  Character(16)   :: LSDAU_DMAT_MODES(2)=(/&
       & "self consistent "&
       &,"fixed           "&
       &/)
  !!
  Character(33)   :: LSDAU_FUNCTIONALS(3)=(/&
       & "LSDA+U Around mean field (AMF/OP)"&
       &,"LSDA+U Atomic limit      (AL)    "&
       &,"Fixed Potential, Lagrange        "&
       &/)
  Character(25)   :: LSDAU_PROJECTIONS(4)=(/&
       & "orthogonal               "&
       &,"hybridized               "&
       &,"hybridized with some core"&
       &,"gross population         "&
       &/)
  !!
  !! old numbers ( igor definition )
  !! 0 hybridized   AMF   = (LSDAU_AMF,LSDAU_HYBRIDIZED)
  !! 1 orthogonal   AMF   = (LSDAU_AMF,LSDAU_ORTHOGONAL)
  !! 2 hybridized   AL    = (LSDAU_AL ,LSDAU_HYBRIDIZED)
  !! 3 orthogonal   AL    = (LSDAU_AL ,LSDAU_ORTHOGONAL)
  !!
  !!***********************************************************************
  !!
  !! organization of LSDA+U
  !!
  !! there are iudef=1:nudef sort-state pairs of U definitions
  !! 
  Type T_Lsdau
     !! make LSDA+U
     Logical(t_log)        :: lsdau
     !! number of LSDA+U sort-state pair definitions
     Integer(t_int)        :: nudef
     !! number of LSDA+U sites-state definitions
     Integer(t_int)        :: nusite
     !! maximum dimension of one U-block (2 max(l)+1)
     Integer(t_int)        :: umaxdim
     !! lat maximum dimension of one U-block (2 max(l)+1)
     Integer(t_int)        :: rumaxdim
     !! maximum angular momentum
     Integer(t_int)        :: lmx
     !! dimension of density matrix block, to be added to iteration vector
     Integer(t_int)        :: n_u_it      
     !! version of double counting term
     Integer(t_int)        :: functional
     !! version of projection
     Integer(t_int)        :: projection
     !! max deviation for internal occuaption matrix iteration
     Real(t_real)          :: omat_dev
     !! use fixed occupation number matrix
     Integer(t_int)        :: dmat_mode
     !! slater-integrals*fourpi/(2l+1) for l=2*n, n=0,1,2,3 
     Real(t_real),Pointer  :: p_u_f(:,:) ! 0:3,nudef
     !! U,J 
     Real(t_real),Pointer  :: p_u_j(:,:) ! 1:2,nudef
     !! gives sort of atom, which has an U corrected state
     Integer(t_int),Pointer:: p_isrtu(:) ! nudef
     !! state-string of U corrected states at isrtu(iudef)
     Character(2),Pointer  :: p_state(:) ! nudef
     !! p_udefs(i) points into p_isrtu,p_state,...
     Integer(t_int),Pointer:: p_udefs(:) ! nusite
     !! site index of definition udef
     Integer(t_int),Pointer:: p_isiteu(:)! nusite
     !! sort index of definition udef
     Integer(t_int),Pointer:: p_isortu(:)! nusite
     !! gives l of udef
     Integer(t_int),Pointer:: p_lvsu(:)   ! nusite
     !! gives deg of udef, FREL: deg=2*(2*l+1), not-FREL: deg=(2*l+1)
     Integer(t_int),Pointer:: p_degvsu(:)   ! nusite
     !! gives ivr index of U state
     Integer(t_int),Pointer:: p_nvsu(:)   ! nusite
     !! gives readable label of U state
     Character(2),Pointer  :: p_labelsu(:)   ! nusite
     !! start index of LSDA+U block in kohn sham matrices
     Integer(t_int),Pointer:: p_ubi1(:)  ! nusite
     !! end index of LSDA+U block in kohn sham matrices
     Integer(t_int),Pointer:: p_ubi2(:)  ! nusite
     !! relat ljmu -> lms
     Complex(t_real),Pointer:: p_T(:,:,:) ! rumaxdim,rumaxdim,nusite
     !! Wigner D functions for real Ylm corresponding to spin quantization axis
     Real(t_real),Pointer :: p_Dlquant(:,:,:)! umaxdim,umaxdim,nusite
     Real(t_real)   :: axis(3) !! save axis for output
     !!
     Complex(t_real),Pointer :: p_dmat(:,:,:,:) ! rumaxdim,rumaxdim,nusite,mspin
     !! U-energy and LSDA+U double counting term
     Real(t_real)   :: eeU,eeD
     !! aux arrays
     Complex(t_real),Pointer  :: p_cvv(:,:,:,:,:)!umaxdim,nusite,nband,nkp,mspin
     Complex(t_real),Pointer  :: p_Scvv(:,:,:,:,:)!umaxdim,nusite,nband,nkp,mspin
     Complex(t_real),Pointer  :: p_vmat(:,:,:,:) !umaxdim,umaxdim,nusite,mspin
     Complex(t_real),Pointer  :: p_hpu(:,:) ! nvdim,nvdim
     Real(t_real),Pointer     :: p_hh(:,:,:) !
     Complex(t_real),Pointer  :: p_odmat(:,:,:,:) !umaxdim,umaxdim,nusite,mspin


  End Type T_Lsdau
  !!***********************************************************************
contains
  Subroutine init_lsdau(lu)
    Implicit None 
    !!IO
    Type(T_Lsdau),Intent(inout) :: lu
    !!LOCAL
    !!
    lu% lsdau=.False.
    lu% nudef=0_t_int
    lu% nusite=0_t_int
    lu% functional=LSDAU_AMF
    lu% projection=LSDAU_ORTHOGONAL
    lu% umaxdim=0_t_int
    lu% rumaxdim=0_t_int
    lu% omat_dev=1.0_t_real
    lu% dmat_mode=LSDAU_DMAT_MODE_SC
    lu% n_u_it=0_t_int
    Nullify(lu% p_u_f)
    Nullify(lu% p_u_j)
    Nullify(lu% p_isrtu)
    Nullify(lu% p_state)
    Nullify(lu% p_udefs)
    Nullify(lu% p_isiteu)
    Nullify(lu% p_isortu)
    Nullify(lu% p_degvsu)
    Nullify(lu% p_nvsu)
    Nullify(lu% p_labelsu)
    Nullify(lu% p_ubi1)
    Nullify(lu% p_ubi2)
    Nullify(lu% p_T)
    Nullify(lu% p_Dlquant)
    Nullify(lu% p_dmat)
    Nullify(lu% p_cvv)
    Nullify(lu% p_Scvv)
    Nullify(lu% p_vmat)
    Nullify(lu% p_hpu)
    Nullify(lu% p_hh)
    Nullify(lu% p_odmat)
    !!
  End Subroutine init_lsdau
  !!***********************************************************************

End Module tlsdau
