Module tmol
  !!
  Use precisiondefs
  Use memory
  Use errors
  Use tmesh
  Use mesh
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TMOL"  
  !!
  Public :: T_Mol,init_mol,clean_mol
  !!***********************************************************************
  Type T_Mol
     !!
     Logical(t_log) :: irredmesh
     !! use tail compression in vatom to get a non-root behaviour at rmax
     Logical(t_log) :: compressed_vatom
     !! number of radial mesh points
     Integer(t_int) :: rad_nr
     !! start and end radius for linear interpolation of angular mesh sizes
     Real(t_real)   :: r_angmesh_1,r_angmesh_max
     !! start and end type for linear interpolation of angular mesh sizes
     Integer(t_int) :: t_angmesh_1,t_angmesh_max
     !! maximum number of mesh points
     Integer(t_int) :: maxpnts
     !! max number of radial mesh points for atommesh
     Integer(t_int) :: nra
     !!
     Integer(t_int),Pointer :: p_nm(:)   !! nsite
     !! grid points
     Real(t_real),Pointer :: p_rm(:,:,:) !! 3,maxpnts,nsite
     !! integration weights times shape function on grid
     Real(t_real),Pointer :: p_wm(:,:)   !! maxpnts,nsite
     !! index into radial grid
     Integer(t_int),Pointer :: p_rindxm(:,:)   !! maxpnts,nsite
     !! potential on grid
     Real(t_real),Pointer :: p_vm2(:,:,:)   !! mspin,maxpnts,nsite
     !! free NSP atom potential
     Real(t_real),Pointer :: p_vatom(:,:,:) !! atommesh% nrm,0:basis% maxNvat,nsort
     Real(t_real),Pointer :: p_excatom(:,:)   !! atommesh% nrm,nsort
     Real(t_real),Pointer :: p_vxcatom(:,:,:)   !! atommesh% nrm,2,nsort



     !! sphere weights on grid
     Real(t_real),Pointer :: p_wgm(:,:)   !! maxpnts,nsite

     !! radial meshes for the atomic solver
     Type(T_Mesh),Pointer   :: p_atommeshs(:) !! nsort 
     

     !!
  End Type T_Mol
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine  init_mol(m)
    !!
    Implicit none  
    !!IO
    Type(T_Mol),Intent(inout)     :: m
    !!LOCAL
    !!
    !!
    m% irredmesh=.True.
    m% compressed_vatom=.True.
    m% rad_nr=80_t_int
    m% r_angmesh_1=0.05_t_real
    m% t_angmesh_1=1_t_int
    m% r_angmesh_max=3.0_t_real
    m% t_angmesh_max=15_t_int

    !!
    Nullify(m% p_atommeshs)
    Nullify(m% p_nm)
    Nullify(m% p_wm)
    Nullify(m% p_rindxm)
    Nullify(m% p_vm2)
    Nullify(m% p_excatom)
    Nullify(m% p_vxcatom)
    Nullify(m% p_vatom)
    Nullify(m% p_rm)
    Nullify(m% p_wgm)
    m% irredmesh=.True.
    !!
  End Subroutine init_mol
  !!***********************************************************************
  Subroutine  clean_mol(m)
    !!
    Implicit none  
    !!IO
    Type(T_Mol),Intent(inout)     :: m
    !!LOCAL
    !!
    Call pfree(m% p_nm)
    Call pfree(m% p_wm)
    Call pfree(m% p_rindxm)
    Call pfree(m% p_vm2)
    Call pfree(m% p_excatom)
    Call pfree(m% p_vxcatom)
    Call pfree(m% p_vatom)
    Call pfree(m% p_rm)
    Call pfree_meshs(m% p_atommeshs)
    Nullify(m% p_wgm)
    !!
  End Subroutine clean_mol
  !!***********************************************************************
  
End Module tmol
