Module topc
  !!
  Use precisiondefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TOPC"  
  !!
  Public :: init_opc,T_OPC&
       &,OPC_ORTHOGONAL&
       &,OPC_FUNC_DEFAULT,OPC_FUNC_DEFAULT_NO_SPIN&
       &,OPC_FUNCTIONALS,OPC_PROJECTIONS&
       &,OPC_FUNC_ESCHRIG,OPC_FUNC_ESCHRIG_NO_SPIN
  !!***********************************************************************
  !! default functional
  Integer(t_int),Parameter :: OPC_FUNC_DEFAULT         =1_t_int
  Integer(t_int),Parameter :: OPC_FUNC_DEFAULT_NO_SPIN =2_t_int
  Integer(t_int),Parameter :: OPC_FUNC_ESCHRIG         =3_t_int
  Integer(t_int),Parameter :: OPC_FUNC_ESCHRIG_NO_SPIN =4_t_int
  !! orthogonal projection technique
  Integer(t_int),Parameter :: OPC_ORTHOGONAL        =1_t_int
  Integer(t_int),Parameter :: OPC_HYBRIDIZED        =2_t_int
  !!
  Character(69)   :: OPC_FUNCTIONALS(4)=(/&
       & "Spin dependend (Nordstroem, Brooks, Johansson, JPhysC 4, 3261 (1992))"&
       &,"Spin independend (Eriksson, Brooks, Johansson, PRB 41, R7311 (1990)) "&
       &,"Eschrig - spin dependend                                             "&
       &,"Eschrig - spin independend                                           "&
       &/)
  Character(25)   :: OPC_PROJECTIONS(2)=(/&
       & "orthogonal               "&
       &,"hybridized no core       "&
       &/)
  !!
  Type T_OPC
     !! make OPC
     Logical(t_log)        :: make_opc
     !! number of OPC sort-state pair definitions in input (non relat sort
     !! and state)
     Integer(t_int)        :: IOnodef
     !! gives sort of atom, which has an OPC corrected state (input sort !!!)
     Integer(t_int),Pointer:: p_IOisrto(:) ! IOnodef
     !! state-string of OPC corrected states at isrto(iodef)
     Character(2),Pointer  :: p_IOstate(:) ! IOnodef

     !! version of projection
     Integer(t_int)        :: projection
     !! version of OPC functional
     Integer(t_int)        :: functional
     !! some functionals are spin independend
     Logical(t_log) :: spin_independend

     !! number of OPC sort-state pair definitions in relativistic reduced
     !! symmetry (relat sorts), but non-relat states
     Integer(t_int)        :: nodef
     !! gives relat sort of atom, which has an OPC corrected state 
     Integer(t_int),Pointer:: p_isrto(:) ! nodef
     !! state-string of OPC corrected states at sort isrto(iodef)
     Character(2),Pointer  :: p_state(:) ! nodef
!!$     !! map relat sort-state paris onto input sort-state pairs
!!$     Integer(t_int),Pointer:: p_odefTOIOodef(:) ! nodef


     !! number of OPC sites-state definitions
     Integer(t_int)        :: nosite
     !! maximum angular momentum
     Integer(t_int)        :: lmx
     !! maximum dimension of one OPC-block (2 max(l)+1)*2
     Integer(t_int)        :: omaxdim
     !! gives number of OPC corrected states at isrto(iodef) 
     !!      !! NON-RELATIVISTIC basis counting !!
     Integer(t_int),Pointer:: p_nvo(:)   ! nodef
     !! gives l of OPC corrected states at isrto(iodef)
     Integer(t_int),Pointer:: p_lvo(:)   ! nodef
     !! p_odefs(i) points into p_IOisrto,p_IOnvo,...
     Integer(t_int),Pointer:: p_odefs(:) ! nosite
     !! site index of definition odef
     Integer(t_int),Pointer:: p_isiteo(:)! nosite
     !! start index of OPC block in kohn sham matrices
     Integer(t_int),Pointer:: p_obi1(:)  ! nosite
     !! end index of OPC block in kohn sham matrices
     Integer(t_int),Pointer:: p_obi2(:)  ! nosite
     !! the spin dependend Racah B-coefficient $B_{\vec{s}l\sigma}$ for every 
     !! sort-state pair (relat sorts)
     !! in the elements p_Bracah(:,:,2...) are additional coefficients
     !! which may result from the functional derivatives
     Real(t_real),Pointer :: p_Bracah(:,:,:) !! 2,nodef,2
     !! the spin dependend orbital moment $M_{\vec{s}l\sigma}$ for every 
     !! sort-state pair (relat sorts)
     Real(t_real),Pointer :: p_Morb(:,:) !! 2,nodef
     Real(t_real),Pointer :: p_occu(:,:) !! 2,nodef
     Complex(t_real),Pointer :: p_MSmat(:,:,:,:) !! 2,omaxdim,omaxdim,nodef
     Complex(t_real),Pointer :: p_MLmat(:,:,:,:) !! 2,omaxdim,omaxdim,nodef
     !!temporary aux array for the coefficient sub-blocks
     Complex(t_real),Pointer :: p_cvv(:,:,:,:) !! omaxdim,nosite,nband,nkp
  End Type T_OPC
  
Contains
  !!***********************************************************************
  Subroutine init_opc(opc)
    Implicit None 
    !!IO
    Type(T_OPC),Intent(inout) :: opc
    !!LOCAL
    !!
    opc% make_opc=.False.
    opc% IOnodef=0_t_int
    opc% nodef=0_t_int
    opc% projection=OPC_ORTHOGONAL
    opc% functional=OPC_FUNC_DEFAULT
    !!
    Nullify(opc% p_IOisrto)
    Nullify(opc% p_IOstate)
    Nullify(opc% p_isrto)
    Nullify(opc% p_state)
!!$    Nullify(opc% p_odefTOIOodef)
    Nullify(opc% p_nvo)
    Nullify(opc% p_lvo)
    Nullify(opc% p_odefs)
    Nullify(opc% p_isiteo)
    Nullify(opc% p_obi1)
    Nullify(opc% p_obi2)
    Nullify(opc% p_Bracah)
    Nullify(opc% p_Morb)
    Nullify(opc% p_occu)
    Nullify(opc% p_MLmat)
    Nullify(opc% p_MSmat)
    Nullify(opc% p_cvv)
    !!
    Return
    !!
  End Subroutine init_opc
  !!***********************************************************************
End Module topc
