!!todo: with larger subspaces (tol artificial large)
!!todo: protect against larger odd subspaces
Module topoins
  !!
  Use precisiondefs
  Use file_iounits
  Use errors
  Use memory
  Use tsymmetry
  Use ttopoins
  Use tlsdau
  Use tio
  Use ioadapter
  Use torbitalbasis
  Use clock
  Use diagonalize
  Use topc
  Use dump
  Use ioaux
  Use prnt
  Use tloi
  Use kohnsham
  Use matserv, Only: cmattest
  Use trelativistic
  Use constants
  Use tkohnsham
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TOPOINS"  
  !!
  Public :: setup_topoins,calc_Z2_invariant,setup_gamma&
       &,read_old_topoins_file
  !!***********************************************************************
  Complex(t_real),Parameter:: c0=(0.0_t_real,0.0_t_real)
  Complex(t_real),Parameter:: c1=(1.0_t_real,0.0_t_real)
  Complex(t_real),Parameter:: ci=(0.0_t_real,1.0_t_real)
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine setup_topoins(symmetry,ti,mspin,relativistic)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_TopoIns),Intent(inout):: ti
    Integer(t_int),Intent(in)    :: mspin,relativistic
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setup_topoins"
    Logical(t_log) :: ret
    Integer(t_int) :: jops
    Character(*),Parameter:: mesg='TI'
    Real(t_real),Parameter :: tol=1.0e-6_t_real
    Real(t_real)   :: ident(3,3)
    Data ident/1.0_t_real,0.0_t_real,0.0_t_real&
         &,0.0_t_real,1.0_t_real,0.0_t_real&
         &,0.0_t_real,0.0_t_real,1.0_t_real/
    !!
    !!
    Call read_old_topoins_file(ti,protect=.True.)
    !!
    !!
    If(.Not. ti% active) Then
       ti% readytoproceed=.False.
       Return
    Endif
    !!
    Write(outfile,1) mesg
    Write(outfile,1) mesg
    !!
    ret=.False.
    !!
    If(relativistic/=FULL_RELATIVISTIC)Then
       Write(outfile,"(a,': You need a full relativistic calculation to &
            &calculate the Z2 invariant.')") mesg
       ret=.True.
    Endif
    !!
    If(mspin==2)Then
       Write(outfile,"(a,': You need time reversal symmetry (no magnetism)',/&
            &,a,': to calculate the Z2 invariant. ')") mesg,mesg
       ret=.True.
    Endif
    !!
    !! check if inversion is an operation
    ti% jopsinv=-1
    Do jops=1,symmetry% Rnumops
       If(All(Abs(ident+symmetry% Ropmat(:,:,jops))<tol))Then
          ti% jopsinv=jops
          Exit
       Endif
    Enddo
    !!
    !!
    If(ti% jopsinv==-1)Then
       ti% hasZ2=.False.
       Write(outfile,"(a,': Z2 invariant cannot easily be calculated ',/&
            &,a,': in systems without inversion symmetry.')")mesg,mesg
       ret=.True.
    Else 
       ti% hasZ2=.True.
       Write(outfile,1) mesg
       Write(outfile,"(a,': System can have Z2 invariant.')")mesg
       Write(outfile,1) mesg
       !!
       !!
!!$       If(Any(Abs(symmetry% Roptau(:,ti% jopsinv))>tol))Then
!!$          Write(outfile,"(a,': Current implementation requests &
!!$               &inversion at origin.')")mesg
!!$          ret=.True.
!!$       Endif
       !!
    Endif
    !!
    !!
    !!
    !!
    If(ret)Then
       Write(outfile,1) mesg
       Return
    Endif
    !!
    ti% readytoproceed=.True.
    !!
    Call setup_gamma(symmetry,ti,mesg)
    !!
    Write(outfile,1) mesg
    !!
    !!
    Return
    !!
1   Format(a,': ')
    !!
  End Subroutine setup_topoins
  !!***********************************************************************
  Subroutine read_old_topoins_file(ti,protect)
    !!
    Implicit None
    !!IO
    Type(T_TopoIns),Intent(inout):: ti
    Logical(t_log),Intent(in)    :: protect
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_old_topoins_file"
    Logical(t_log) :: exists
    !!
    !!
    Inquire(file='=.ti',exist=exists)
    If(.Not.exists) Return
    !!
    If(protect)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'''=.ti'' found, this is old style ... ',/&
            &,' use the ''topological insulator'' submenu in fedit instead!',/&
            &,' Remove ''=.ti'' to re-run.'")
    Endif
    !!
    Open(12,file='=.ti')
    Read(12,*)
    Read(12,*)ti% active
    Close(12)
    !!
!!$    Call initparser
!!$    Call readfile(f_fplo_in)
!!$    Call setParserVarValue('topological_insulator.active'&
!!$         &,ti% active)
!!$    Call writeParsFile(f_fplo_in)
!!$    Call cleanreadfile
    !!
    Return
    !!
  End Subroutine read_old_topoins_file
  !!***********************************************************************
  Subroutine setup_gamma(symmetry,ti,mesg)
    !!
    !!  calculate the time reversal invariant points
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_TopoIns),Intent(inout):: ti
    Character(*),Intent(in)      :: mesg
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setup_gamma"
    Integer(t_int) :: i,i1,i2,i3
    !!
    !!
    !!
    Write(outfile,1) mesg
    Do i=1,3
       Write(outfile,"(a,': G',i1,'=',3(f20.12,1x))") mesg,i&
            &,symmetry% reciprocial_lattice_vectors(:,i)
    Enddo
    Write(outfile,1) mesg
    !!
    !!
    ti% mapnu(:,:)=0
    i=0
    Do i3=0,1
       Do i2=0,1
          Do i1=0,1
             i=i+1
             Write(ti% lgammas(i),"(i1,i1,i1)")i1,i2,i3
             ti% gammas(:,i)=&
                  &(symmetry% reciprocial_lattice_vectors(:,1)*i1&
                  &+symmetry% reciprocial_lattice_vectors(:,2)*i2&
                  &+symmetry% reciprocial_lattice_vectors(:,3)*i3)*0.5_t_real
             ti% mapnu(i,0)=1
             If(i1==1) ti% mapnu(i,1)=1
             If(i2==1) ti% mapnu(i,2)=1
             If(i3==1) ti% mapnu(i,3)=1
          enddo
       Enddo
    Enddo
    !!
    Do i=1,3
       Write(outfile,10) mesg,i,ti% mapnu(:,i)
    Enddo
10  Format(a,': iGamma for nu',i1,': ',8i2)
    !!
    Do i=1,8
       Write(outfile,"(a,': Gamma[',a3,']=2pi/a*[',3(f20.12,1x),']')")&
            &mesg,ti% lgammas(i),ti% gammas(:,i)&
            &*symmetry% lattice_constants(1)/twopi
    Enddo
    !!
    !!
    Return
    !!
1   Format(a,': ')
    !!
  End Subroutine setup_gamma
  !!***********************************************************************
  Subroutine calc_Z2_invariant(mspin,symmetry,structure,lattice,basis,loi&
       &,kohnsham,lsdau,opc,ti,relativistic,atstart)
    !!
    Implicit None
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_TopoIns),Intent(inout):: ti
    Type(T_Kohnsham),Intent(inout) :: kohnsham
    Type(T_Lsdau),Intent(inout)  :: lsdau 
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Loi),Intent(inout)    :: loi
    Type(T_OPC),Intent(inout)    :: opc
    Logical(t_log),Intent(in)    :: atstart !! are we calling from before
    !!                                         the first diagonalization?
    Integer(t_int),Intent(in)    :: mspin,relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_Z2_invariant"
    Logical(t_log) :: rel,prot,dumpreadd,dumpwritee,docalc,success
    Logical(t_log) :: err,toterr
    Integer(t_int) :: ik,ib,iz,iz1,nvdim
    Integer(t_int) :: ib1,ib2,ibb,edim
    Integer(t_int) :: iminus,iplus,ifermi,deltai(8),nui(0:3)
    Logical(t_log) :: badfermi
    Character(7)   :: snui
    Real(t_real)   :: qq(3),d,e,maxerr,error
    Complex(t_real):: cc
    Real(t_real),Pointer :: p_einv(:),p_egamma(:,:)
    Integer(t_int),Pointer :: p_Igamma(:,:)
    Complex(t_real),Pointer:: p_Utheta(:,:)
    Complex(t_real),Pointer:: p_Uinv(:,:),p_A(:,:),p_cc(:,:)
    Type(T_KS) :: ks
    Character(*),Parameter:: mesg='TI'
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    Real(t_real),Parameter   :: etol=1.0e-8_t_real
    Character(*),Parameter  :: cpumesg_ti='Z2 invariant'
    Character(1) :: cparity(-1:1)
    Data cparity(-1:1)/'O',' ','E'/
    !!
    If(.Not. (ti% hasZ2 .And. ti% readytoproceed)) Return
    !!
    Call watch(cpumesg_ti,cpu_start,reset=.True.)            
    !!
    prot=.false.
    Nullify(p_Utheta,p_Uinv,p_A,p_cc,p_einv,p_egamma,p_Igamma)
    !!
    If(relativistic/=FULL_RELATIVISTIC) Return
    !!
    Write(outfile,"(a,': ')")mesg
    Write(outfile,"(a,': ')")mesg
    !!
    dumpreadd=dumpread(DUMP_TI)
    If(atstart.And..Not.dumpreadd) Return
    dumpwritee=dumpwrite(DUMP_TI)
    If(dumpreadd)Then
       Open(12,file=Trim(gdump(DUMP_TI)% fname),status='old'&
            &,form='UNFORMATTED')
       gdump(DUMP_TI)% doread=.False.
    Elseif(dumpwritee)Then
       Open(12,file=Trim(gdump(DUMP_TI)% fname),form='UNFORMATTED')
    Endif
    !!
    Write(outfile,"(a,': ')")mesg
    !!
    nvdim=kohnsham% nvdim
    !!
    !!
    Call init_KS(ks,occu=.False.&
         &,savesp=.True.&
         &,lsdau_kintest=.False.,lsdau_energy=.False.,spinmat=.True.&
         &,need_sigma=.False.)
    !!
    !!
    rel=(relativistic==FULL_RELATIVISTIC)
    !!
    !!
    Call setup_KS(ks,mspin,8_t_int,structure,kohnsham,lsdau,opc&
         &,relativistic)
    !!
    Call palloc('p_egamma',p_egamma,1_t_int,8_t_int,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    Call palloc('p_Igamma',p_Igamma,1_t_int,8_t_int,1_t_int,nvdim&
         &,nmodule,nsubfunc)
    !!
    ikloop: Do ik=1,8
       qq(:)=ti% gammas(:,ik)
       !!
       Write(outfile,"(a,': Gamma[',a3,']=2pi/a*[',3(f20.12,1x),']')")&
            &mesg,ti% lgammas(ik),qq*symmetry% lattice_constants(1)/twopi
       !!
       docalc=.True.
       If(dumpreadd)Then
          docalc=.Not.dumpti(kohnsham,ks,'read')
       Endif
       If(docalc)Then
          If(atstart) Exit ikloop
          Call calc_at_kpoint(ks,ik,qq,1_t_int,mspin&
               &,structure,lattice,basis,loi,kohnsham&
               &,lsdau,opc,relativistic)
       Endif
       If(dumpwritee)Then
          success=dumpti(kohnsham,ks,'write')
       Endif
          
       !!
       !! rescale to Fermi level and eV
       If(rel)Then
          ks% p_eb=(ks% p_eb-kohnsham% efermi)*hatoev
!!$       Else
!!$          sofms=3.0_t_real-2.0_t_real*ms
!!$          ks% p_eb=(ks% p_eb+sofms*kohnsham% Bfsm-kohnsham% efermi)*hatoev
       Endif
       !!
       !!
       
       !Do ib=1,nvdim
       !   Write(outfile,"(f20.12,1x)",advance='no')ks% p_eb(ib)
       !   Write(outfile,*)
       !Enddo
       !!
       !! form S*C for later use
       ks% p_sp2(1:nvdim,1:nvdim)=Matmul(ks% p_sp2(1:nvdim,1:nvdim)&
            &,ks% p_hp(1:nvdim,1:nvdim))
       !!
       !!
       !!
       ib=1
       e=ks% p_eb(ib)
       ib1=ib
       ib2=ib
       !!
       bandloop: Do ib=2,nvdim
          d=Abs(e-ks% p_eb(ib))
          If(Abs(e)>1.0_t_real)Then
             d=d/Abs(e)
          Endif
          If(d<etol)Then
             ib2=ib
             If(ib/=nvdim) Cycle
          Endif
          !!
          edim=ib2-ib1+1
          If(Mod(edim,2_t_int)/=0)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror,&
                  &msg="'degenerate sub-spaces are not even dimensional?'")
          Endif
          !!
          Call palloc('p_Utheta',p_Utheta,1_t_int,edim,1_t_int,edim&
               &,nmodule,nsubfunc)
          Call palloc('p_Uinv',p_Uinv,1_t_int,edim,1_t_int,edim&
               &,nmodule,nsubfunc)
          Call palloc('p_A',p_A,1_t_int,edim,1_t_int,edim&
               &,nmodule,nsubfunc)
          Call palloc('p_cc',p_cc,1_t_int,edim,1_t_int,edim&
               &,nmodule,nsubfunc)
          Call palloc('p_einv',p_einv,1_t_int,edim,nmodule,nsubfunc)


          Call calc_time_reversal_rep(structure% nsite,structure% p_isrt&
               &,structure% p_tau,qq,nvdim,ib1,ib2,kohnsham% p_vblockstart&
               &,basis,ks% p_hp,ks% p_sp2,p_Utheta,.False.)
          Call calc_inversion_rep(structure% nsite,structure% p_isrt&
               &,structure% p_tau,structure% p_equivsites(ti% jopsinv,:)&
               &,symmetry% Roptau(:,ti% jopsinv)&
               &,qq,nvdim,ib1,ib2&
               &,kohnsham% p_vblockstart,basis,ks% p_hp,ks% p_sp2,p_Uinv&
               &,.False.)
          !!
          !!
          !!
          !!
          !!
          p_A=p_Uinv
          Call cdiagno_noO(edim,p_A,p_einv)
          !!
          !!
          iminus=0
          Do iz=edim,1,-1
             If(p_einv(iz)<-0.9)Then
                iminus=iz
                Exit
             End If
          Enddo
          iplus=edim+1
          Do iz=1,edim
             If(p_einv(iz)>=0.9)Then
                iplus=iz
                Exit
             End If
          Enddo
          !!
          !!

          Do iz=1,iminus,2
             Call blockdiagno_timereversal(p_Utheta,edim,iz,iminus,p_A&
                  &,.False.)
          Enddo
          Do iz=iplus,edim,2
             Call blockdiagno_timereversal(p_Utheta,edim,iz,edim,p_A&
                  &,.False.)
          Enddo
          !!
          toterr=.False.
          !!check unitarity
          p_cc=Matmul(Transpose(Conjg(p_A)),p_A)
          err=.Not.cmattest(p_cc,c1,c0,tol,maxerr)
          toterr=(toterr.Or.err)
          If(err)Then
             Call printmat("D^{+}*D=","('(',f8.4,',',f8.4,')')"&
                  &,"('                   ')",p_cc,edim,edim,tol)
             Write(outfile,"('max error of D^{+}*D=1: ',e30.15e3)")maxerr
          Endif
          !! check Utheta*D^{*}=D*U0
          maxerr=0.0_t_real
          err=.False.
          Do iz1=1,edim
             Do iz=1,edim
                cc=Dot_product(p_A(:,iz1),Matmul(p_Utheta(:,:)&
                     &,Conjg(p_A(:,iz))))
                If(Mod(iz1,2_t_int)==0 .And. iz==iz1-1)cc=cc-c1
                If(Mod(iz1,2_t_int)==1 .And. iz==iz1+1)cc=cc+c1
                error=Abs(cc)
                If(error>tol)Then
                   maxerr=Max(maxerr,error)
                   err=.True.
                Endif
             Enddo
          Enddo
          toterr=(toterr.Or.err)
          If(err)Then
             Call printmat("D^{+}*Utheta*D^{*}="&
                  &,"('(',f8.4,',',f8.4,')')"&
                  &,"('                   ')"&
                  &,Matmul(Transpose(Conjg(p_A)),Matmul(p_Utheta,Conjg(p_A)))&
                  &,edim,edim,tol)
             Write(outfile,"('max error of D^{+}*Utheta*D^{*}=U0: ',e30.15e3)")maxerr
          Endif
          !!
          !!check Uinv*D=D*(+/-1) and paired subspaces
          !!
          maxerr=0.0_t_real
          err=.False.
          Do iz1=1,edim
             Do iz=1,edim
                cc=Dot_product(p_A(:,iz1),Matmul(p_Uinv,p_A(:,iz)))
                If(iz1==iz)Then
                   If(iz<=iminus) cc=cc+c1
                   If(iplus<=iz) cc=cc-c1
                Endif
                error=Abs(cc)
                If(error>tol)Then
                   maxerr=Max(maxerr,error)
                   err=.True.
                Endif
             Enddo
          EndDo
          toterr=(toterr.Or.err)
          If(err)Then
             Call printmat("D^{+}*Uinv*D=","('(',f8.4,',',f8.4,')')"&
                  &,"('                   ')"&
                  &,Matmul(Transpose(Conjg(p_A)),Matmul(p_Uinv,p_A))&
                  &,edim,edim,tol)
             Write(outfile,"('max error of Uinv*D^{*}=D*(+/-1): ',e30.15e3)")maxerr
          Endif
          !!
          If(toterr)Then
             Call errmsg(nmodule,nsubfunc,E_warning,&
                  &msg="'inaccuracies in parity analysis !?!'")
          Endif
          !!
          If(iminus>0.And.Mod(iminus,2_t_int)/=0)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror,&
                  &msg="'negative parity subspace has odd dimensions'")
          Endif
          If(iplus<=edim.And.Mod(iplus,2_t_int)/=1)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror,&
                  &msg="'positive parity subspace has odd dimensions'")
          Endif
          !!
          !!
          Do iz=1,iminus
             ibb=iz+ib1-1
             !Print "(i5,f30.15,i5,f5.1)",ibb,ks% p_eb(ibb),-1,p_einv(iz)+1
             p_egamma(ik,ibb)=ks% p_eb(ibb)
             p_Igamma(ik,ibb)=-1
          Enddo
          Do iz=iplus,edim
             ibb=iz+ib1-1
             p_egamma(ik,ibb)=ks% p_eb(ibb)
             p_Igamma(ik,ibb)=1
             !Print "(i5,f30.15,i5,f5.1)",ibb,ks% p_eb(ibb),+1,p_einv(iz)-1
          Enddo
          !!
          e=ks% p_eb(ib)
          ib1=ib
          ib2=ib
          !!
       Enddo bandloop
       !!
       !!
    enddo ikloop 
    !!
    If(dumpreadd .Or. dumpwritee)Then
       Close(12)
    Endif
    !!
    Call clean_KS(ks,lsdau,opc)
    !!
    ifermi=nvdim
    Do iz=1,nvdim
       If(All(p_egamma(1:8,iz)>0.0_t_real))Then
          ifermi=iz
          Exit
       Endif
    Enddo
    !!
    Write(outfile,"(a,': ')")mesg
    Write(outfile,"(a,': ',153('-'))")mesg
    Write(outfile,"(a,':        ')",advance='no')mesg
    Do ik=1,8
       Write(outfile,"('   Gamma[',a3,']     ')",advance='no')ti% lgammas(ik)
    Enddo
    Write(outfile,*)
    Write(outfile,"(a,':  band     ')",advance='no')mesg
    Do ik=1,8
       Write(outfile,"('Energy    I delta ')",advance='no')
    Enddo
    Write(outfile,*)
    !!
    Write(outfile,"(a,': ',153('-'))")mesg
    deltai(1:8)=1
    badfermi=.False.
    Do iz=1,nvdim
       Write(outfile,"(a,': ',i5,' : ')",advance='no')mesg,iz
       Do ik=1,8
          If(Mod(iz,2_t_int)==1)Then
             deltai(ik)=deltai(ik)*p_Igamma(ik,iz)
          Endif
          Write(outfile,"(f11.5,1x,a1,1x,i2,2x)",advance='no')&
               &p_egamma(ik,iz),cparity(p_Igamma(ik,iz)),deltai(ik)
       Enddo
       Write(outfile,*)
       If(Mod(iz,2_t_int)==0)Then
          Call calc_ti_signature(deltai,ti% mapnu,nui,snui)
          Write(outfile,"(a,':       ',a)")mesg,snui
       Endif
       !!
       If(.Not.(All(p_egamma(1:8,iz)>0.0_t_real).Or.&
            &All(p_egamma(1:8,iz)<0.0_t_real)))Then
          Call errmsg(nmodule,nsubfunc,E_warning,&
               &msg="'TI: Is it really an insulator?'")
          badfermi=.True.
       Endif
       If(ifermi-1==iz)Then
          Write(outfile,"(a,': ',153('-'))")mesg
          Write(outfile,"(a,': delta_i:')",advance='no')mesg
          Call calc_ti_signature(deltai,ti% mapnu,nui,snui)
          Do ik=1,8
             Write(outfile,"(14x,i2,2x)",advance='no') deltai(ik)
          Enddo
          Write(outfile,*)
          Write(outfile,"(a,': Z2-invariants: ',a,1x)",advance='no')mesg,snui
          If(nui(0)==1)Then
             Write(outfile,"('-> strong TI')") 
          Else If(Any(nui(1:3)==1))Then
             Write(outfile,"('-> weak TI')") 
          Else
             Write(outfile,*)
          Endif
          Write(outfile,"(a,': ',153('-'))")mesg
          
       Endif
       !!
       If(iz>ifermi+8) Exit
    Enddo
    !!
    Write(outfile,"(a,': ',153('-'))")mesg
    !!
    !!
    !!
    Call pfree(p_Utheta)
    Call pfree(p_Uinv)
    Call pfree(p_A)
    Call pfree(p_cc)
    Call pfree(p_einv)
    Call pfree(p_egamma)
    Call pfree(p_Igamma)
    !!
    Write(outfile,"(a,': ')")mesg
    !!
    Call watch(cpumesg_ti,cpu_stop,delete=.True.,Print=.True.)         
    !!
    Return
    !!
  End Subroutine calc_Z2_invariant
  !!***********************************************************************
  Subroutine calc_ti_signature(deltai,mapnu,nui,txt)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: deltai(8),mapnu(8,0:3)
    Integer(t_int),Intent(out)   :: nui(0:3)
    Character(7),Intent(out)     :: txt
    !!LOCAL
    !Character(*),Parameter::nsubfunc="calc_ti_signature"
    Integer(t_int) :: ik,i
    !!
    !!
    nui=1
    Do ik=1,8
       Do i=0,3
          If(mapnu(ik,i)==1)Then
             nui(i)=nui(i)*deltai(ik)
          Endif
       Enddo
    Enddo
    Do i=0,3
       If(nui(i)<0)Then
          nui(i)=1
       Else
          nui(i)=0
       Endif
    Enddo
    Write(txt,"(i1,';(',3i1,')')")nui(0:3)
    !!
    Return
    !!
  End Subroutine calc_ti_signature
  !!***********************************************************************
  Subroutine blockdiagno_timereversal(Utheta,edim,D1,D2,p_D,prot)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: edim,D1,D2
    Complex(t_real),Intent(inout):: p_D(edim,edim)
    Complex(t_real),Intent(in)   :: Utheta(edim,edim)
    !!LOCAL
    Character(*),Parameter::nsubfunc="blockdiagno_timereversal"
    Integer(t_int) :: dim,j,k
    Complex(t_real):: DU(edim,D2-D1+3)
    Complex(t_real):: S(D2-D1+1,D2-D1+1)
    Real(t_real)   :: e(D2-D1+1)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    !!
    dim=D2-D1+1
    If(Mod(dim,2_t_int)/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'dimension not even!'")
       
    Endif
    !!
    If(prot)Then
       Print *,'D1,D2,dim=',D1,D2,dim
       !!
       Call printmat("D=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')",p_D(:,D1:D2),edim,dim,tol)
    Endif
    !!
    DU(:,1)=(p_D(:,D1)&
         &-Matmul(Utheta(:,:),Conjg(p_D(:,D1))))*sq2o2
    DU(:,2)=(p_D(:,D1)&
         &+Matmul(Utheta(:,:),Conjg(p_D(:,D1))))*sq2o2
    !!
    DU(:,3:dim+2)=p_D(:,D1:D2)
    !!
    !!
    If(prot)Then
       Call printmat("DU=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')",DU(:,:),edim,dim+2,tol)
    Endif
    !!
    Do j=3,dim+2
       DU(:,j)=DU(:,j)-DU(:,1)*Dot_product(DU(:,1),DU(:,j))&
            &-DU(:,2)*Dot_product(DU(:,2),DU(:,j))
    Enddo
    !!
    If(prot)Then
       Call printmat("DUortho=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')",DU(:,:),edim,dim+2,tol)
    Endif
    !!
    Do j=3,dim+2
       Do k=3,dim+2
          S(j-2,k-2)=Dot_product(DU(:,j),DU(:,k))
       Enddo
    Enddo
    !!
    Call cdiagno_noO(dim,S,e)
    !!
    If(prot)Then
       Do j=1,2
          Print *,'a0=',e(j)
       Enddo
       Do j=3,dim
          Print *,'a1=',e(j)
       Enddo
    Endif
    !!
    p_D(:,D1:D1+1)=DU(:,1:2)
    Do k=3,dim
       p_D(:,k+D1-1)=Matmul(DU(:,3:dim+2),S(:,k))/Sqrt(e(k))
    Enddo
    !!
    !!
    !!
    Return
    !!
  End Subroutine blockdiagno_timereversal
  !!***********************************************************************
  Subroutine calc_time_reversal_rep(nsite,isrt,tau,kpoint,nvdim,ib1,ib2&
       &,vblockstart,basis,zz,StC,U,prot)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: nsite,isrt(:)
    Integer(t_int),Intent(in)    :: nvdim,ib1,ib2,vblockstart(:)
    Real(t_real),Intent(in)      :: kpoint(3),tau(:,:)
    Complex(t_real),Intent(in)   :: zz(:,:),StC(:,:)
    Complex(t_real),Intent(out)  :: U(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_time_reversal_rep"
    Logical(t_log) :: err,toterr
    Integer(t_int) :: isa,issa,iz,ivr,kap,deg,m,ib,mu2,edim,m1
    Real(t_real)   :: gs
    Complex(t_real):: sphase,cc
    Complex(t_real),Pointer:: p_gC(:,:),p_cc(:,:)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    !!
    Nullify(p_gC,p_cc)
    !!
    edim=ib2-ib1+1
    Call palloc('p_gC',p_gC,1_t_int,nvdim,1_t_int,edim,nmodule,nsubfunc)
    Call palloc('p_cc',p_cc,1_t_int,edim,1_t_int,edim,nmodule,nsubfunc)
    !!
    p_gC=c0
    isaloop: Do isa=1,nsite
       issa=isrt(isa)
       gs=Dot_product(tau(:,isa),kpoint)
       sphase=Exp(-ci*gs*2.0_t_real)
       iz=vblockstart(isa)-1
       ivrloop: Do ivr=1,basis% p_nRv(issa)
          kap=basis% p_kapRv(ivr,issa)
          deg=basis% p_degRv(ivr,issa)
          Do m=1,deg
             mu2=2*m-deg-1
             m1=deg+1-m
             cc=sphase*(-1.0_t_real)**((-mu2+1)/2)*Sign(1_t_int,kap)
             p_gC(iz+m,1:edim)=Conjg(zz(iz+m1,ib1:ib2))*cc
          Enddo
          iz=iz+deg
       Enddo ivrloop
    Enddo isaloop
    !!
    Do iz=ib1,ib2
       Do ib=1,edim
          U(iz-ib1+1,ib)=Dot_product(StC(1:nvdim,iz),p_gC(1:nvdim,ib))
       Enddo
    Enddo
    !!
    If(prot)Then
       Call printmat("Utheta=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')",U,edim,edim,1.0e-10_t_real)
    Endif
    !!
    toterr=.False.
    !! check Conjg(U)*U=-1
    p_cc=Matmul(Conjg(U),U)
    err=.Not.cmattest(p_cc,-c1,c0,tol,gs)
    toterr=(toterr.Or.err)
    If(err)Then
       Call printmat("U^{*}*U=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')"&
            &,p_cc,tol)
       Write(outfile,"('max error of U^{*}*U=-1: ',e30.15e3)")gs
    Endif
    !!  check unitarity
    p_cc=Matmul(Transpose(Conjg(U)),U)
    err=.Not.cmattest(p_cc,c1,c0,tol,gs)
    toterr=(toterr.Or.err)
    If(err)Then
       Call printmat("U^{+}*U=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')"&
            &,p_cc,edim,edim,tol)
       Write(outfile,"('max error of U^{+}*U=1: ',e30.15e3)")gs
    Endif
    !!
    If(toterr)Then
       Call errmsg(nmodule,nsubfunc,E_warning,&
            &msg="'inaccuracies in timerev operation !?!'")
    Endif
    !!
    !!
    Call pfree(p_gC)
    Call pfree(p_cc)
    !!
    Return
    !!
  End Subroutine calc_time_reversal_rep
  !!***********************************************************************
  Subroutine calc_inversion_rep(nsite,isrt,tau,equivsites,optau&
       &,kpoint,nvdim,ib1,ib2,vblockstart,basis,zz,StC,U,prot)
    !!
    Implicit None
    !!IO
    Type(T_Basis),Intent(in)     :: basis
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: nsite,isrt(:),equivsites(:)
    Integer(t_int),Intent(in)    :: nvdim,ib1,ib2,vblockstart(:)
    Real(t_real),Intent(in)      :: kpoint(3),tau(:,:),optau(:)
    Complex(t_real),Intent(in)   :: zz(:,:),StC(:,:)
    Complex(t_real),Intent(out)  :: U(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_inversion_rep"
    Logical(t_log) :: err,toterr
    Integer(t_int) :: edim,isa,issa,isag,iz0,izg,ivr,deg,ib,iz
    Real(t_real)   :: gs
    Complex(t_real):: sphase,cc
    Complex(t_real),Pointer:: p_gC(:,:),p_cc(:,:)
    Real(t_real),Parameter :: tol=1.0e-10_t_real
    !!
    Nullify(p_gC,p_cc)
    !!
    edim=ib2-ib1+1
    Call palloc('p_gC',p_gC,1_t_int,nvdim,1_t_int,edim,nmodule,nsubfunc)
    Call palloc('p_cc',p_cc,1_t_int,edim,1_t_int,edim,nmodule,nsubfunc)
    !!
    p_gC=c0
    isaloop: Do isa=1,nsite
       isag=equivsites(isa)
       issa=isrt(isa)
       gs=Dot_product(tau(:,isa)-optau(:)*0.5_t_real,kpoint)
       sphase=Exp(-ci*gs*2.0_t_real)
       iz0=vblockstart(isa)-1
       izg=vblockstart(isag)-1
       ivrloop: Do ivr=1,basis% p_nRv(issa)
          deg=basis% p_degRv(ivr,issa)
          cc=(-1.0_t_real)**basis% p_lRv(ivr,issa)*sphase
          Do ib=1,edim
             p_gC(iz0+1:iz0+deg,ib)=cc*zz(izg+1:izg+deg,ib1-1+ib)
          Enddo
          iz0=iz0+deg
          izg=izg+deg
       Enddo ivrloop
    Enddo isaloop
    !!
    !!
    Do iz=ib1,ib2
       Do ib=1,edim
          U(iz-ib1+1,ib)=Dot_product(StC(1:nvdim,iz),p_gC(1:nvdim,ib))
       Enddo
    Enddo
    !!
    If(prot)Then
       Call printmat("Uinv=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')",U,edim,edim,1.0e-10_t_real)
    Endif
    !!
    toterr=.False.
    !!  check unitarity
    p_cc=Matmul(Transpose(Conjg(U)),U)
    err=.Not.cmattest(p_cc,c1,c0,tol,gs)
    toterr=(toterr.Or.err)
    If(err)Then
       Call printmat("I^{+}*I=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')"&
            &,p_cc,edim,edim,tol)
       Write(outfile,"('max error of I^{+}*I=1: ',e30.15e3)")gs
    Endif
    !!  U*U=1
    p_cc=Matmul(U,U)
    err=.Not.cmattest(p_cc,c1,c0,tol,gs)
    toterr=(toterr.Or.err)
    If(err)Then
       Call printmat("I*I=","('(',f8.4,',',f8.4,')')"&
            &,"('                   ')"&
            &,p_cc,edim,edim,tol)
       Write(outfile,"('max error of I*I=1: ',e30.15e3)")gs
    Endif
    !!
    If(toterr)Then
       Call errmsg(nmodule,nsubfunc,E_warning,&
            &msg="'inaccuracies in inversion operation !?!'")
    Endif
    !!
    !!
    !!
    Call pfree(p_gC)
    Call pfree(p_cc)
    !!
    Return
    !!
  End Subroutine calc_inversion_rep
  !!***********************************************************************
    Logical(t_log) Function dumpti(kohnsham,ks,mode)
    !!
    Implicit None
    !!IO
    Type(T_KS),Intent(inout)     :: ks
    Type(T_Kohnsham),Intent(inout):: kohnsham
    Character(*),Intent(in)      :: mode
    !!LOCAL
    Character(*),Parameter::nsubfunc="dumpti"
    Integer(t_int) :: nvdim
    Integer ::err
    Character(30)  :: str30
    !!
    dumpti=.True.
    !!
    If(Trim(mode) == 'write')Then
       nvdim=Size(ks% p_eb,1)

       Write(12) '#nvdim                       =',nvdim
       Write(12) '#kohnsham% efermi            =',kohnsham%  efermi
       Write(12) '#kohnsham% Bfsm              =',kohnsham%  Bfsm
       Write(12) '#ks% p_eb                    =',ks%  p_eb
       Write(12) '#ks% p_zz                    =',ks%  p_hp
       Write(12) '#ks% p_sp2                   =',ks%  p_sp2

    Else
       !!
       Read(12,iostat=err) str30,nvdim
       !!
       If(err/=0.Or.&
            &Size(ks%  p_eb,1)/=nvdim.Or.&
            &Size(ks%  p_hp,1)/=nvdim.Or.&
            &Size(ks%  p_hp,2)/=nvdim.Or.&
            &Size(ks%  p_sp2,1)/=nvdim.Or.&
            &Size(ks%  p_sp2,2)/=nvdim&
            ) &
            &dumpti=.False.
       If(dumpti) Then
          Read(12,iostat=err) str30,kohnsham%  efermi
          Read(12,iostat=err) str30,kohnsham%  Bfsm
          Read(12,iostat=err) str30,ks%  p_eb
          Read(12,iostat=err) str30,ks%  p_hp
          Read(12,iostat=err) str30,ks%  p_sp2
       Endif
       If(err/=0) dumpti=.False.
       If(.Not.dumpti)Then
          Write(errbuf,34) Trim(gdump(DUMP_TI)% fname)
          Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Endif
    End If
    !!
    !!
    Return
    !!
34  Format("'data on dump file ",a," seem to belong to different settings.',/&
         &,' I will ignore the file''s content and proceed with normal &
         &calculation!'")
    !!
  End Function dumpti
  !!***********************************************************************

End Module topoins
