Module toptics
  !!
  Use precisiondefs
  Use errors
  Use memory
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TOPTICS"  
  !!
  Public :: T_Optics,init_optics,clean_optics
  !!***********************************************************************
  Type T_Optics
     !! is the module active
     Logical(t_log) :: active
     Logical(t_log) :: jointdos!! use linear approximation for theta function
     !!                  if .false. the full linear tetra method including
     !!                  delta and theta function is used
     Logical(t_log) :: bandoutput !! write data for analysis
     Logical(t_log) :: stopafter !! stop after optics is completed
     Integer(t_int) :: Ne !! numberof energy points
     Real(t_real)   :: emin,emax !! interval 
     Integer(t_int) :: Ni     !! number of nonzero components
     Integer(t_int) :: Ninoneq   !! number of independent nonzero components
     Integer(t_int) :: iof(9) !! combined index of nonzero component
     !!                       iof(i)=I+3*(J-1), where I,J are indices of eps
     Integer(t_int) :: Njofi(9) !! number of contribs to nonzero component 
     Integer(t_int) :: jofi(9,9) !! contribs to nonzero component
     Real(t_real)   :: aofji(9,9)!! the coefficients of the nonzero linear 
     !!                             combinations
     Integer(t_int) :: equiv(9) !! which components are equal are antiequal
     !!                            (negative of each other)
     !Integer(t_int) :: Nsymproj
     !Real(t_real)   :: Vsymproj(9,9)
     !! momentum matrix elements
     Complex(t_real),Pointer:: p_pmat(:,:,:,:,:)
  End Type T_Optics
  !!***********************************************************************
Contains
  !!***********************************************************************
  Subroutine init_optics(op)
    !!
    Implicit None
    !!IO
    Type(T_Optics),Intent(inout) :: op
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_optics"
    !!
    op% active=.False.
    op% bandoutput=.False.
    op% jointdos=.False.
    op% stopafter=.False.
    op% Ne=100
    op% emin=0.0_t_real
    op% emax=10.0_t_real
    !!
    Nullify(op% p_pmat)
    !!
    !!
    Return
    !!
  End Subroutine init_optics
  !!***********************************************************************
  Subroutine clean_optics(op)
    !!
    Implicit None
    !!IO
    Type(T_Optics),Intent(inout) :: op
    !!LOCAL
    !Character(*),Parameter::nsubfunc="clean_optics"
    !!
    !!
    Call pfree(op% p_pmat)
    !!
    !!
    Return
    !!
  End Subroutine clean_optics
  !!***********************************************************************


End Module toptics
