Module torbitalbasis
  !!
  Use precisiondefs
  Use errors
  Use memory
  Use tmesh
  Use tbasdef
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TORBITALBASIS"  
  !!
  Public :: T_Basis,T_SIC,init_basis,clean_basis,init_sic,clean_sic&
       !&,free_basis_IO_arrays&
       !&,copy_basis_IO_arrays&
       !&,alloc_basis_IO_core_arrays&
       !&,alloc_basis_IO_valence_arrays&
       &,ORB_TYPE_CORE,ORB_TYPE_SEMICORE,ORB_TYPE_VALENCE&
       &,ORB_TYPE_POLARIZATION,CORE_CONF_NO,CORE_CONF_INF,use_OVO
  !!***********************************************************************
  !! Use scalar-relativistic O*V*O-term ( O=L*S ) to correct divergent
  !! l=2 potentials from products of two p1/2 functions
  Logical(t_log) :: use_OVO=.True.
  !!***********************************************************************
  Type T_Basis
     !! max number of radial mesh points (for allocation purposes)
     Integer(t_int) :: nrb
     !! number of components of the radial functions
     !! non-relativistic         : ncomp=1
     !! scalar-relativistic      : ncomp=4
     !! KH scalar-relativistic   : ncomp=3
     !! full-relativistic        : ncomp=2
     Integer(t_int) :: ncomp
     !!
     !!
     !!
!!$     !! confined core-orbitals for open-core treatment
!!$     !! we use infinte potential barrier at some finite radius
!!$     !! IOrcinf(ic,issaIO)   ic=index of core orbital of atom-sort issaIO
!!$     !!                       
!!$     !! if IOcnfc(ic,issaIO) equals CORE_CONF_INF
!!$     Integer(t_int),Pointer :: p_IOcnfc(:,:) ! IOncm,nsort
!!$     Real(t_real),Pointer   :: p_IOrcinf(:,:) ! IOncm,nsort
     !!
     Type(T_Basdef),Pointer    :: p_bdef(:) !! IOnsort
     !!
     Logical(t_log) :: gam_factor !! gamma-prefactor used?
     !!
     !! lowest valence level
     Real(t_real)   :: valence_erg_bottom
     !! core states
     Real(t_real),Pointer  :: p_fc  (:,:,:,:)  ! nrb,nRcmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fdpc(:,:,:,:)  ! nrb,nRcmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fdmc(:,:,:,:)  ! nrb,nRcmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fkc(:,:,:,:)  ! nrb,nRcmx,ncomp,nsort
     !! valence states
     Real(t_real),Pointer  :: p_fv   (:,:,:,:)  ! nrb,nRvmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fdpv (:,:,:,:)  ! nrb,nRvmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fdmv (:,:,:,:)  ! nrb,nRvmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fkv(:,:,:,:)  ! nrb,nRvmx,ncomp,nsort
     Real(t_real),Pointer  :: p_fgam(:)    ! nRvmx,nsort
     !! maximum extension of individual states
     Real(t_real),Pointer  :: p_rfcmax(:,:,:)  ! nRcmx,ncomp,nsort
     Real(t_real),Pointer  :: p_rfvmax(:,:,:)  ! nRvmx,ncomp,nsort
     !! maximum extension of all states
     Real(t_real),Pointer  :: p_rtfcmax(:,:)  ! 0:ncomp*sccmpfac,nsort
     Real(t_real),Pointer  :: p_rtfvmax(:,:)  ! 0:ncomp         ,nsort
     !! non-relativistic core occupation
     Real(t_real),Pointer  :: p_corecharge(:) ! nsort
     !! maximum angular momentum per atom
     Integer(t_int)        ::  lvmx,lcmx,lmx

     !!
     !! relativistic organization
     !!
     !! naming convention in the following variable names: 
     !!  c =core
     !!  v =valence
     !!  R =relativistic-related
     !!  m at the end = max
     !!
     !!
     !! max number of relativistic orbitals per atom with different
     !! true core, all core, all valence
     Integer(t_int)        :: nRchimx,nRcmx,nRvmx
     !! maximum degeneracy per radial function
     Integer(t_int)        :: degRcmx,degRvmx
     !! number of relativistic orbitals per atom with different
     !! radial functions (like p_nc,p_nv)
     !! true core
     Integer(t_int),Pointer :: p_nRchi(:) ! nsort 
     !! all core=true core + added core functions
     Integer(t_int),Pointer :: p_nRc(:) ! nsort
     !! all valence
     Integer(t_int),Pointer :: p_nRv(:) ! nsort
     !! angular momentum of first large component
     Integer(t_int),Pointer :: p_lRc(:,:) ! nRcmx,nsort
     Integer(t_int),Pointer :: p_lRv(:,:) ! nRvmx,nsort
     Integer(t_int),Pointer :: p_maxl(:)  ! nsort, max l atom resolved
     !! type of valence function (for consistent SREL approximation)
     Integer(t_int),Pointer :: p_tRv(:,:) ! nRvmx,nsort
     !! p_degRc(icr,issa) = degeneracy of core state icr of atom issa
     Integer(t_int),Pointer :: p_degRc(:,:) ! nRcmx,nsort
     !! p_degRv(ivr,issa) = degeneracy of valence state ivr of atom issa
     Integer(t_int),Pointer :: p_degRv(:,:) ! nRvmx,nsort
     !! p_nnRc(icr,issa) = main-qu.-number of core state icr of atom issa
     Integer(t_int),Pointer :: p_nnRc(:,:) ! nRcmx,nsort
     !! p_nnRv(ivr,issa) = main-qu.-number of valence state ivr of atom issa
     Integer(t_int),Pointer :: p_nnRv(:,:) ! nRvmx,nsort
     !! p_kapRc(icr,issa) = kappa-number of core state icr of atom issa
     Integer(t_int),Pointer :: p_kapRc(:,:) ! nRcmx,nsort
     !! p_kapRv(ivr,issa) = kappa-number of valence state ivr of atom issa
     Integer(t_int),Pointer :: p_kapRv(:,:) ! nRvmx,nsort
     !! p_icrTOic(icr,issa) = ic, which is the core orbital number of the 
     !! corresponding nonrelativistic state of atom issa
     Integer(t_int),Pointer :: p_icrTOic(:,:) ! nRcmx,nsort
     !! p_ivrTOiv(ivr,issa) = iv, which is the valence orbital number of the 
     !! corresponding nonrelativistic state of atom issa
     Integer(t_int),Pointer :: p_ivrTOiv(:,:) ! nRvmx,nsort
     !! if double,triple orbitals, first(:,:) has the name of the main orbital 
     Character(2),Pointer   :: p_firstRv(:,:) ! nRvmx,nsort
     Real(t_real),Pointer   :: p_pex(:) ! nsort
     Real(t_real),Pointer   :: p_exn2(:) ! nsort
     Real(t_real),Pointer   :: p_pexRv(:,:) ! nRvmx,nsort
     !!
     !! for orbital iv and sort issa ivat=p_vatclsv(iv,issa)
     !! points into vatom-class of this atom with p_Qvat(ivat,issa) being
     !! the ionicity of the potential
     Integer(t_int),Pointer :: p_vatclsv(:,:) ! nRvmx,nsort
     !! same for core
     Integer(t_int),Pointer :: p_vatclsc(:,:) ! nRcmx,nsort
     !! maximum number of different atom potentials per sort
     Integer(t_int) :: maxNvat
     !! number of different atom potentials per sort
     Integer(t_int),Pointer :: p_nvat(:) ! nsort
     !!  ionicity of ivat's and issa's potential is p_Qvat(ivat,issa)
     Real(t_real),Pointer :: p_Qvat(:,:) ! maxNvat,nsort
     !!  spin-potential of ivat's and issa's potential is p_Svat(ivat,issa)
     Real(t_real),Pointer :: p_Svat(:,:) ! maxNvat,nsort

     !!
     !! For generalization we define arrays, which go through the
     !! basis states in nl-shells, not nlj-shells.
     !! For non-FREL calculations this is identical to the p_..Rv arrays
     !! for FREL calculations it is a different organization, where
     !! the degeneracy is 2*(2*l+1) 
     !!
     !! max number of valence states per atom
     !! only nl classification considered here
     Integer(t_int) :: nNLvmx
     Integer(t_int),Pointer :: p_nNLv(:)    !! nsort
     Integer(t_int),Pointer :: p_nnNLv(:,:) !! nNLvmxL,nsort
     Integer(t_int),Pointer :: p_lNLv(:,:)  !! nNLvmxL,nsort
     !! the degeneracy of each spin part of the nl shell (2*l+1)
     Integer(t_int),Pointer :: p_degNLSv(:,:)!! nNLvmxL,nsort
     !! the degeneracy of the whole nl shell (rel: 2*(2*l+1),else:  2*l+1)
     Integer(t_int),Pointer :: p_degNLv(:,:)!! nNLvmxL,nsort
     !! if double,triple orbitals, first(:,:) has the name of the main orbital 
     Character(2),Pointer   :: p_firstNLv(:,:) ! nNLvmx,nsort

     !!
     !! organization of relativistic basis mapping
     !!
     !! transformation matrix of pseudo into relat matrices
     !! including quant-axis
     Complex(t_real),Pointer:: p_relTOpseudo(:,:,:,:) 
     !! excluding quant-axis
     Complex(t_real),Pointer:: p_relTOpseudoNoD(:,:,:,:) 
     !                      (2*lmx+3),(2*lmx+2),2,-lmx-1:lmx+1
     !!
     !! this is organized such that it refers to nl-shell occupation even in FREL
     !! in fullrel the two nlj-shells   icr and icr+1 both
     !! have p_Rcoreoccu(icr or icr+1,sort,spin)=N_s=(N+s*M)/2
     !! this is ugly, but helps opencore-treatment in molcore
     !! Note: this also means that sum(p_Rcoreoccu(:,sort,spin)) does
     !! not give the core occuption !!!!
     Real(t_real),Pointer :: p_Rcoreoccu(:,:,:)  ! nRcmx,nsort,mspin
     !! the radial meshes for the atom's  basis function storage
     Type(T_Mesh),Pointer  :: p_basmeshs(:) ! nsort
     !!
  End Type T_Basis
  !!
  !!
  Type T_SIC
     Real(t_real),Pointer  :: p_esic(:,:) ! nsort,2
  End Type T_SIC
  !!
  Integer(t_int),Parameter :: ORB_TYPE_CORE          = 1_t_int
  Integer(t_int),Parameter :: ORB_TYPE_SEMICORE      = 2_t_int
  Integer(t_int),Parameter :: ORB_TYPE_VALENCE       = 3_t_int
  Integer(t_int),Parameter :: ORB_TYPE_POLARIZATION  = 4_t_int
  !!
  Integer(t_int),Parameter :: CORE_CONF_NO           = 1_t_int
  Integer(t_int),Parameter :: CORE_CONF_INF          = 2_t_int

  !!
  !!***********************************************************************
contains
  !!***********************************************************************
  Subroutine init_basis(ba)
    Implicit None 
    !!IO
    Type(T_Basis),Intent(inout) :: ba
    !!LOCAL
!    Character(*),Parameter::nsubfunc="init_basis"
    !!
    Nullify(ba% p_bdef)
    !!
    !! basis input arrays
!!$    Nullify(ba% p_IOrcinf)
!!$    Nullify(ba% p_IOcnfc)
    !!
    Nullify(ba% p_fgam)
    !!
    Nullify(ba% p_fc)
    Nullify(ba% p_fdpc)
    Nullify(ba% p_fdmc)
    Nullify(ba% p_fkc)
    Nullify(ba% p_fv)
    Nullify(ba% p_fdpv)
    Nullify(ba% p_fdmv)
    Nullify(ba% p_fkv)
    Nullify(ba% p_rfcmax)
    Nullify(ba% p_rfvmax)
    Nullify(ba% p_rtfcmax)
    Nullify(ba% p_rtfvmax)
    Nullify(ba% p_corecharge)
    Nullify(ba% p_nRchi)
    Nullify(ba% p_nRc)
    Nullify(ba% p_nRv)
    Nullify(ba% p_nNLv)
    Nullify(ba% p_lRc)
    Nullify(ba% p_lRv)
    Nullify(ba% p_lNLv)
    Nullify(ba% p_maxl)
    Nullify(ba% p_tRv)
    Nullify(ba% p_degRc)
    Nullify(ba% p_degRv)
    Nullify(ba% p_degNLv)
    Nullify(ba% p_degNLSv)
    Nullify(ba% p_nnRc)
    Nullify(ba% p_nnRv)
    Nullify(ba% p_nnNLv)
    Nullify(ba% p_kapRc)
    Nullify(ba% p_kapRv)
    Nullify(ba% p_icrTOic)
    Nullify(ba% p_ivrTOiv)
    Nullify(ba% p_firstRv)
    Nullify(ba% p_firstNLv)
    Nullify(ba% p_pex)
    Nullify(ba% p_exn2)
    Nullify(ba% p_pexRv)
    Nullify(ba% p_vatclsv)
    Nullify(ba% p_vatclsc)
    Nullify(ba% p_nvat)
    Nullify(ba% p_Qvat)
    Nullify(ba% p_Svat)
    !!
    Nullify(ba% p_relTOpseudo)
    Nullify(ba% p_relTOpseudoNoD)
    Nullify(ba% p_Rcoreoccu)
    Nullify(ba% p_basmeshs)
    !!
    !!
  End Subroutine init_basis
  !!***********************************************************************
  Subroutine clean_basis(ba)
    Implicit None 
    !!IO
    Type(T_Basis),Intent(inout) :: ba
    !!LOCAL
!    Character(*),Parameter::nsubfunc="clean_basis"
    !!
    Call pfree_basdefs(ba% p_bdef)
    !!
    !! basis input arrays
!!$    Call pfree(ba% p_IOrcinf)
!!$    Call pfree(ba% p_IOcnfc)
    !!
    Call pfree(ba% p_fgam)
    !!
    Call pfree(ba% p_fc)
    Call pfree(ba% p_fdpc)
    Call pfree(ba% p_fdmc)
    Call pfree(ba% p_fkc)
    Call pfree(ba% p_fv)
    Call pfree(ba% p_fdpv)
    Call pfree(ba% p_fdmv)
    Call pfree(ba% p_fkv)
    Call pfree(ba% p_rfcmax)
    Call pfree(ba% p_rfvmax)
    Call pfree(ba% p_rtfcmax)
    Call pfree(ba% p_rtfvmax)
    Call pfree(ba% p_corecharge)
    Call pfree(ba% p_nRchi)
    Call pfree(ba% p_nRc)
    Call pfree(ba% p_nRv)
    Call pfree(ba% p_nNLv)
    Call pfree(ba% p_lRc)
    Call pfree(ba% p_lRv)
    Call pfree(ba% p_lNLv)
    Call pfree(ba% p_maxl)
    Call pfree(ba% p_tRv)
    Call pfree(ba% p_degRc)
    Call pfree(ba% p_degRv)
    Call pfree(ba% p_degNLv)
    Call pfree(ba% p_degNLSv)
    Call pfree(ba% p_nnRc)
    Call pfree(ba% p_nnRv)
    Call pfree(ba% p_nnNLv)
    Call pfree(ba% p_kapRc)
    Call pfree(ba% p_kapRv)
    Call pfree(ba% p_icrTOic)
    Call pfree(ba% p_ivrTOiv)
    Call pfree(ba% p_firstRv)
    Call pfree(ba% p_firstNLv)
    Call pfree(ba% p_pex)
    Call pfree(ba% p_exn2)
    Call pfree(ba% p_pexRv)
    Call pfree(ba% p_vatclsv)
    Call pfree(ba% p_vatclsc)
    Call pfree(ba% p_nvat)
    Call pfree(ba% p_Qvat)
    Call pfree(ba% p_Svat)
    !!
    Call pfree(ba% p_relTOpseudo)
    Call pfree(ba% p_relTOpseudoNoD)
    Call pfree(ba% p_Rcoreoccu)
    Call pfree_meshs(ba% p_basmeshs)
    !!
    !!
  End Subroutine clean_basis
  !!***********************************************************************
  Subroutine init_sic(si)
    Implicit None 
    !!IO
    Type(T_SIC),Intent(inout) :: si
    !!LOCAL
    Nullify(si% p_esic)
  End Subroutine init_sic
  !!***********************************************************************
  Subroutine clean_sic(si)
    Implicit None 
    !!IO
    Type(T_SIC),Intent(inout) :: si
    !!LOCAL
    Call pfree(si% p_esic)
  End Subroutine clean_sic
!!$  !!***********************************************************************
!!$  !! often needed, to clean up allocated arrays of the input basis 
!!$  !! definition in io routines
!!$  Subroutine free_basis_IO_arrays(basis)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Basis),Intent(inout) :: basis
!!$    !!LOCAL
!!$    !!
!!$    Call pfree(basis% p_IOnc1)
!!$    Call pfree(basis% p_IOnnc1)
!!$    Call pfree(basis% p_IOcnfc)
!!$    Call pfree(basis% p_IOrcinf)
!!$    Call pfree(basis% p_IOlc1)
!!$    Call pfree(basis% p_IOnv)
!!$    Call pfree(basis% p_IOnnv)
!!$    Call pfree(basis% p_IOlv)
!!$    Call pfree(basis% p_IOtv)
!!$    Call pfree(basis% p_IOQion)
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine free_basis_IO_arrays
!!$  !!***********************************************************************
!!$  !! often needed, to copy the input basis definition arrays
!!$  !! in IO routines
!!$  Subroutine copy_basis_IO_arrays(dest,src,issaIO)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Basis),Intent(inout)  :: dest
!!$    Type(T_Basis),Intent(in)     :: src
!!$    Integer(t_int),Intent(in)    :: issaIO
!!$    !!LOCAL
!!$    Integer(t_int) :: nc,nv
!!$    !!
!!$    !!
!!$    nc=src% p_IOnc1(issaIO)
!!$    nv=src% p_IOnv(issaIO)
!!$    !!
!!$    dest% p_IOnc1(issaIO)=nc
!!$    dest% p_IOnv(issaIO)=nv
!!$    !!
!!$    dest% p_IOlc1   (1_t_int:nc,issaIO)=src% p_IOlc1   (1_t_int:nc,issaIO)
!!$    dest% p_IOlv   (1_t_int:nv,issaIO)=src% p_IOlv   (1_t_int:nv,issaIO)
!!$    dest% p_IOtv   (1_t_int:nv,issaIO)=src% p_IOtv   (1_t_int:nv,issaIO)
!!$    dest% p_IOnnc1  (1_t_int:nc,issaIO)=src% p_IOnnc1  (1_t_int:nc,issaIO)
!!$    dest% p_IOcnfc (1_t_int:nc,issaIO)=src% p_IOcnfc (1_t_int:nc,issaIO)
!!$    dest% p_IOrcinf(1_t_int:nc,issaIO)=src% p_IOrcinf(1_t_int:nc,issaIO)
!!$    dest% p_IOnnv  (1_t_int:nv,issaIO)=src% p_IOnnv  (1_t_int:nv,issaIO)
!!$    dest% p_IOQion (1_t_int:nv,issaIO)=src% p_IOQion (1_t_int:nv,issaIO)
!!$    !!
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine copy_basis_IO_arrays
!!$  !!***********************************************************************
!!$  !! for IO routines
!!$  !! allocate the core state-definition arrays (some of them)
!!$  Subroutine alloc_basis_IO_core_arrays(basis,nsort,IOncm)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Basis),Intent(inout)  :: basis
!!$    Integer(t_int),Intent(in)    :: nsort,IOncm
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="alloc_basis_IO_core_arrays"
!!$    !!
!!$    Call palloc('basis % p_IOnc1',basis % p_IOnc1 &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOnnc1',basis % p_IOnnc1 &
!!$         &,1_t_int,IOncm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOcnfc',basis % p_IOcnfc &
!!$         &,1_t_int,IOncm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOrcinf',basis % p_IOrcinf &
!!$         &,1_t_int,IOncm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOlc1',basis % p_IOlc1 &
!!$         &,1_t_int,IOncm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine alloc_basis_IO_core_arrays
!!$  !!***********************************************************************
!!$  !! for IO routines
!!$  !! allocate the valence state-definition arrays (some of them)
!!$  Subroutine alloc_basis_IO_valence_arrays(basis,nsort,IOnvm)
!!$    !!
!!$    Implicit None
!!$    !!IO
!!$    Type(T_Basis),Intent(inout)  :: basis
!!$    Integer(t_int),Intent(in)    :: nsort,IOnvm
!!$    !!LOCAL
!!$    Character(*),Parameter::nsubfunc="alloc_basis_IO_valence_arrays"
!!$    !!
!!$    Call palloc('basis % p_IOnv',basis % p_IOnv &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOnnv',basis % p_IOnnv &
!!$         &,1_t_int,IOnvm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOlv',basis % p_IOlv &
!!$         &,1_t_int,IOnvm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    Call palloc('basis % p_IOtv',basis % p_IOtv &
!!$         &,1_t_int,IOnvm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    !!
!!$    Call palloc('basis % p_IOQion',basis % p_IOQion &
!!$         &,1_t_int,IOnvm &
!!$         &,1_t_int,nsort &
!!$         &,nmodule,nsubfunc)
!!$    !!
!!$    !!
!!$    Return
!!$    !!
!!$  End Subroutine alloc_basis_IO_valence_arrays
  !!***********************************************************************
  


End Module torbitalbasis
