Module tpotentialdensity
  !!
  Use precisiondefs
  Use tmesh
  Use finuc
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TPOTENTIALDENSITY"  
  !!
  Public :: T_Potential,init_potential,CHARGEMODE_NONE,CHARGEMODE_JELLIUM&
       &,CHARGEMODE_MOL_ION,CHARGEMODE_TYPE&
       &,SPHER_NETVV,SPHER_NETVVALL,SPHER_NETALL,spheroptions&
       &,spheroptiontest
  !!***********************************************************************
  !! how to charge the system 
  !! 'none': mean only nuclear charges and corresponding electrons
  !!         of neutral atoms are present.
  !! 'jellium': adds a homogenous background charge (for 3d crystals).
  !!         An equal number of electrons is added to the system.
  !!         molecule_ionicity contains the amount of corresponding surplus
  !!         charges, A positive number means less electrons than nuclear 
  !!         charges.
  Integer(t_int),Parameter :: CHARGEMODE_NONE   =1
  Integer(t_int),Parameter :: CHARGEMODE_JELLIUM=2
  Integer(t_int),Parameter :: CHARGEMODE_MOL_ION=3
  Character(18)            :: CHARGEMODE_TYPE(3)
  !!
  Data CHARGEMODE_TYPE&
       &/"None              "&
       &,"Jellium           "&
       &,"Molecular ionicity"&
       &/
  !!
  Integer(t_int),Parameter :: SPHER_NETVV   =1_t_int
  Integer(t_int),Parameter :: SPHER_NETVVALL=2_t_int
  Integer(t_int),Parameter :: SPHER_NETALL  =3_t_int
  Character(8)   :: spheroptions(3)
  Data spheroptions/&
       &'netvv   ',&
       &'netvvall',&
       &'netall  ' / 
  !!
  !!***********************************************************************
  Type T_Potential
     !!
     !! how to charge the system 
     Integer(t_int) :: chargemode
     !!
     Logical(t_log) :: make_vca !! the user can switch vca on or off
     !!
     Integer(t_int) :: nvca !! number of input VCA definitions
     Integer(t_int),Pointer :: p_vcasorts(:) !! IOsorts of input VCA definitions
     Real(t_real),Pointer  ::  p_vcaZ(:)     !! nuclear charges of input VCA definitions
     !! nuclear charge
     Real(t_real),Pointer  :: p_natom(:) !! nsort
     !! averaged nuclear charges (CPA)
     Real(t_real),Pointer  :: p_natomav(:) !! nsite
     !! nuclear model
     Integer(t_int) :: nuc_type
     !! the user request this maximum angular momentum in 
     !! potential/density expansion
     Integer(t_int) :: max_LIO
     !! maximum angular momentum in potential/density expansion
     !! this may differ from max_LIO due to basis angular momenta
     Integer(t_int)     :: max_L
     !! max_over_sites number of angular momentum components in potential
     !! and density
     Integer(t_int)     :: mlmp
     !! number of ang mom comp. per site  
     Integer(t_int),Pointer :: p_nlmp(:)    ! nsite
     !! array of components for every site1 
     Integer(t_int),Pointer :: p_lmp(:,:)   ! mlmp,nsite
     Real(t_real),Pointer :: p_lcd(:,:,:,:) ! nrp,mlmp,nsite,mspin
     !! potential arrays, these are NOT permanentely allocated
     !! fourier potential for ewald construction
     Complex(t_real),Pointer :: p_vft(:) ! nlattg
     !! r* [local multipol-neutral coulomb potential contributions]
     Real(t_real),Pointer :: p_vc(:,:,:) ! nrp,mlmp,nsite 
     !! multipoles
     Real(t_real),Pointer :: p_ac(:,:) ! mlmp,nsite 
     !! numerical normalization factor for gauss compensation charges
     Real(t_real),Pointer :: p_Nl(:) !! 0:max_L
     !! local multipol compensation charge-potential contributions
     !! beware that unlike the p_vc, p_vew is NOT multiplied with r
!!!OLD     !! be aware that the L=00 component has a finite value at r=0
!!!OLD     !! v{L=00,r=0,site=isa} = 4 * p * p_ac(1,isa) * sqrt{4*Pi}
!!!OLD     !! where p is the Ewald parameter .
!!!OLD     !! The potential at \vec{r}=0 then is 
!!!OLD     !!            v{L=00,r=0,site=isa}*Y_{00} = 4 * p * p_ac(1,isa)
     Real(t_real),Pointer :: p_vew(:,:,:) ! nrp,mlmp,nsite 
     !! total nuclear charge including CPA average
     Real(t_real)         :: tot_nuc_charge
     !! spin dependend charge contained in density
     Real(t_real)         :: dens_charge(2)
     Real(t_real),Pointer :: p_charges(:,:) ! nsite,2
!     !! screening parameter in Ewald-Fuchs method
!     Real(t_real)         :: ewcutoff
     !! radial screening length
     Real(t_real)         :: ew_p_r
     !! reciprocial screening factor
     Real(t_real)         :: ewxc_g
     !! if true, automatic determination of Ewald screening
!     Logical(t_log)       :: auto_ew_screen
     !! total ionicity for molecule calculations
     Real(t_real)   :: molecule_ionicity
     !! max number of mesh points for the p_pmeshs
     Integer(t_int) :: nrp
     !!  mesh for all potentials
     Type(T_Mesh),Pointer   :: p_pmeshs(:) !! nsort
     !! reduce exchange part, xreduce=1 is full ex-field strength, xreduce=0
     !! is like non-magnetic
     Real(t_real)   :: xreduce
     !! number of sort-orbital pairs, for which to restrict density to spherical part
     Integer(t_int) :: nspherdens
     Integer(t_int),Pointer :: p_spherrestrictsort(:)
     Integer(t_int),Pointer :: p_spherrestrictivr(:)
     !! options for the spherical restriction
     Integer(t_int) :: spheroption
     !!
  End Type T_Potential
  !!***********************************************************************
    
Contains
  Subroutine init_potential(po)
    Implicit None 
    !!IO
    Type(T_Potential),Intent(inout) :: po
    !!LOCAL
    Nullify(po% p_natom)
    Nullify(po% p_natomav)
    Nullify(po% p_nlmp)
    Nullify(po% p_lmp)
    Nullify(po% p_lcd)
    Nullify(po% p_vft)
    Nullify(po% p_vc)
    Nullify(po% p_vew)
    Nullify(po% p_ac)
    Nullify(po% p_Nl)
    Nullify(po% p_charges)
    Nullify(po% p_pmeshs)
    po% make_vca=.False.
    po% nuc_type=FINUC_NONE
    po% chargemode=CHARGEMODE_NONE
    po% xreduce=1.0_t_real
    po% molecule_ionicity=0.0_t_real
    Nullify(po% p_vcasorts)
    Nullify(po% p_vcaZ)
    po% nvca=0_t_int
    po% nspherdens=0_t_int
    Nullify(po% p_spherrestrictsort,po% p_spherrestrictivr)
    po % spheroption=SPHER_NETALL
    !!
  End Subroutine init_potential
  !!***********************************************************************
  Logical(t_log) Function spheroptiontest(potential,sort,iva,ivb)
    !!
    Implicit None
    !!IO
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: sort,iva,ivb
    !!LOCAL
    !Character(*),Parameter::nsubfunc="spheroptiontest"
    Integer(t_int) :: ire
    !!
    !!
    spheroptiontest=.False.
    !!

    Do ire=1,potential% nspherdens
       If(.Not.(potential% p_spherrestrictsort(ire)==sort)) Cycle

       If((potential% spheroption == SPHER_NETALL).And.&
            &(potential% p_spherrestrictivr(ire) ==iva ).And. (ivb==0)) Then
          spheroptiontest=.True.
          Return
       Endif

       If((  (potential% spheroption == SPHER_NETALL).Or.&
            &(potential% spheroption == SPHER_NETVVALL)).And.&
            &(((potential% p_spherrestrictivr(ire) ==iva ).Or.&
            &(potential% p_spherrestrictivr(ire) ==ivb )).And.(iva>1.and.ivb>1)) ) Then
          spheroptiontest=.True.
          Return
       Endif

       If((potential% spheroption == SPHER_NETVV).And.&
            &((potential% p_spherrestrictivr(ire) ==iva ).And.&
            &(potential% p_spherrestrictivr(ire) ==ivb )) ) Then
          spheroptiontest=.True.
          Return
       Endif
       
    Enddo
    !!
    Return
    !!
  End Function spheroptiontest
  !!***********************************************************************


End Module tpotentialdensity
