Module tsymmetry
  !!
  Use precisiondefs
  Use constants
  !!
  Implicit None 
  !!
  Private
  Public :: init_lattice,init_structure,init_symmetry &
       &,SYM_LENGTH_BOHRRADII &
       &,SYM_LENGTH_ANGSTROEM &
       &,SYM_LENGTH_UNITS &
       &,STR_TYPE_CRYSTAL&
       &,STR_TYPE_MOLECULE&
       &,STR_TYPES&
       &,T_Symmetry &
       &,T_Structure &
       &,T_Lattice
  !!***********************************************************************
  Character(*),Parameter::nmodule="TSYMMETRY"    
  !!
  Type T_Symmetry
     Integer(t_int):: spgrpnum 
     Integer(t_int):: pgrpnum 
     Real(t_real) :: lattice_constants(3)
     Real(t_real) :: axis_angles(3)
     Integer(t_int) :: nugen
     Integer(t_int) :: iugen(48)
     Character(10):: pgr
     Character(10):: lattice
     Integer(t_int) :: rcentering(3,3)
     Integer(t_int) :: gcentering(3,3)
     Logical(t_log):: HEX,RTG
     Real(t_real) :: lattice_vectors(3,3)
     Real(t_real) :: reciprocial_lattice_vectors(3,3)
     Real(t_real) :: convreciprocial_lattice_vectors(3,3)
     Integer(t_int):: IOnumops
     Integer(t_int):: IOopind(48)
     Real(t_real) :: IOopmat(3,3,48)
     Real(t_real) :: IOoptau(3,48)
     Real(t_real) :: opgen(3,3,64)
     Real(t_real) :: opeuler(3,64)
     Real(t_real) :: NRwu_ks(ORBITAL_MAX_MAT_DIM,ORBITAL_MAX_MAT_DIM&
          &,0:ORBITAL_MAX_L,48)
     Integer(t_int):: opeulerinv(64)
     Integer(t_int):: inversion
     Character(40) :: IOopsymbol(48)
!!$     !! symmetry decomposition of onsite matrix elements (LSAD+U)
!!$     !! nonzero indicees in onsite matrix (0:2l+1,0:2l+1,site,l)
!!$     Integer(t_int),Pointer :: p_sdcmp_ij(:,:,:,:) 
!!$     !!         dimensions: ORBITAL_MAX_MAT_DIM,ORBITAL_MAX_MAT_DIM,nsite,0:ORBITAL_MAX_L
!!$     !! number of nonzero matrix elements among all sites for l
!!$     Integer(t_int)         :: sdcmp_nij(0:ORBITAL_MAX_L) 
!!$     !! number of parameters  among all sites for l
!!$     Integer(t_int)         :: sdcmp_npar(0:ORBITAL_MAX_L) 
!!$     !! eigen vectors of symmetry projector
!!$     Real(t_real),Pointer   :: p_sdcmp_vec(:,:,:) ! max(sdcmp_nij),max(sdcmp_nij),0:ORBITAL_MAX_L
!!$     !! does  vector j contribute to l and sort
!!$     Integer(t_int),Pointer :: p_sdcmp_npar_s(:,:,:) ! 0:ORBITAL_MAX_L,max(sdcmp_nij),nsort
     !! 
     !! in the relativistic case, we treat the calculation within the non
     !! relativistic setup for the lattice, but we need to reduce
     !! the symmetry (operations,Wyckoff-sorts)
     !! 
     Logical(t_log) :: reduced_sym !! true if crystal symmetry is reduced
     Integer(t_int):: Rnumops        !! number of reduced operations 
     Integer(t_int):: Ropind(48)     !! index of the 1:Rnumops operations 
     Integer(t_int):: Roptimerev(48) !! this operation has to be combined 
     !!                                 with time reversl asymmetry in order
     !!                                 to retain the magnetic direction in 
     !!                                 colinear calculations
     !!                                 into generator table
     Real(t_real)  :: Ropmat(3,3,48) !! the operations ((im)proper rotations)
     Real(t_real)  :: Roptau(3,48)   !! the operations (translational part)
     Character(40) :: Ropsymbol(48)  !! the operation symbol
     !! non-rel transformation matrices in the relat site labeling
     Real(t_real)  :: Rwu_ks(2*ORBITAL_MAX_L+3,2*ORBITAL_MAX_L+3&
          &,0:ORBITAL_MAX_L+1,48)
     !! relativisitic transformation matrices, including 
     !! time reversal symmetries D=D^j*D^{\theta}
     Complex(t_real)  :: Rwu_tj_ks(2*ORBITAL_MAX_L+2,2*ORBITAL_MAX_L+2&
          &,-(ORBITAL_MAX_L+1):(ORBITAL_MAX_L+1),48)
     !! relativisitic transformation matrices, including 
     !! time reversal symmetries D=D^j*D^{\theta} but transforming
     !! the whole relativistic nl-space (including l-1/2 and l+1/2)
     !!
     !!    
     !!   / D^(l-1/2)    0      \
     !!   \   0       D^(l+1/2) /
     !!
     Complex(t_real),Pointer  :: p_Rwu_tjnl_ks(:,:,:,:)
     !!
  End Type T_Symmetry
  !
  Type T_Lattice
     Integer(t_int)         :: nlattg
     Real(t_real),Pointer  :: p_glat(:,:)
     !! half distance of next neighbour site empty or not
     Real(t_real),Pointer  :: p_rsite(:) ! nsort
     !! half distance to non empty next neighbour site
     Real(t_real),Pointer  :: p_ratom(:) ! nsort


     !!
     Real(t_real)   :: rcut !! cut off for overlap and hamilton integrals
     !!                             crystal potential


     !! npair(isa,isb) is number of neigbourshells around isb pointing to isa 
     Integer(t_int)         :: npairsmax        ! maximum of all npairs
     Integer(t_int),Pointer :: p_npairs(:,:)    ! nsite,nsite
     !! number of neighbour shells of equal vector length for pair isa, isb
     Integer(t_int),Pointer :: p_npshell(:,:)   ! nsite,nsite
     Integer(t_int)         :: npshellmax       ! max of all npshell
     !! max number of sites in one shell
     Integer(t_int)         :: maxinpshell
     !! number of vectors in one shell
     Integer(t_int),Pointer :: p_ninpshell(:,:,:)! npshellmax,nsite,nsite
     !! neighbour vector pointing from an isb to an isa site
     Real(t_real),Pointer  :: p_pairs(:,:,:,:)   ! 3,npairsmax,nsite,nsite
     !! distance of shell
     Real(t_real),Pointer  :: p_pairabs(:,:,:)   ! npshellmax,nsite,nsite
  End Type T_Lattice
  
  Type T_Structure
     !! which type of calculation
     Integer(t_int) :: str_type
     !! number of Wyckoff positions
     Integer(t_int) :: IOnsort
     !! lengthunit of Wyckoff positions
     Integer(t_int) :: lengthunit
     !! Wyckoff positions
     Real(t_real),Pointer  :: p_IOwtau(:,:) ! 3,IOnsort
     Character(2),Pointer  :: p_IOelements(:) ! IOnsort
     !!  number of sites in unit cell, including CPA degeneracy
     !!  different sites may sit on the same position
     Integer(t_int) :: nsite
     !!  number of real sites (positions), which are not empty sites
     !!  CPA: single empty sites excluded
     !!  CPA: alloyed empty sites are counted
     Integer(t_int) :: naps_a
     !!  number of real sites (positions), without CPA degeneracy
     !!  naps=N(umber of) A(tom) P(osition)S 
     Integer(t_int) :: naps
     !! number of crystllographic inequivalent sites, supposed no CPA
     Integer(t_int) :: ncis
     !! max number of atoms at one real site
     Integer(t_int) :: maxq
     !! mapping of site onto sort index, sorts count Wyckoff positions
     Integer(t_int),Pointer :: p_IOisrt(:) ! nsite 
     !! multiplicity of input sorts
     Integer(t_int),Pointer :: p_IOmult(:) ! IOnsort 
     !! give for site and operation the transformed site g*site
     !! p_IOequivsites(jops,isa)=isb with S(isb)=G(jops) S(isa)
     Integer(t_int),Pointer :: p_IOequivsites(:,:) ! 48,nsite
     !! give for site and operation the transformed site g^(+)*site
     Integer(t_int),Pointer :: p_IOequivsitesrev(:,:) ! 48,nsite
     !!CPA: all atoms on the same real site have the same kaps, which equals the
     !!CPA: site index of the first atom at this site
     !!CPA: after the determination of kaps, the arrays p_IOisrt and p_IOwtau
     !!     are resorted
     !!CPA: to create a proper block structure
     Integer(t_int),Pointer :: p_kaps(:)  ! nsite
     !!CPA: every position isap is mapped onto one of the sites, which sit there
     Integer(t_int),Pointer :: p_posToSiteRep(:)  ! nsite
     !!CPA: every site may uniquely be mapped onto one position
     Integer(t_int),Pointer :: p_siteToPos(:) ! nsite
     !!CPA: p_nqAtPos(isap) = number of sites at position isap
     Integer(t_int),Pointer :: p_nqAtPos(:) ! nsite
     !!CPA: the atom iq at position isap has site index qposToSite(isap,iq)
     !!CPA: the index iq runs from 1 to p_nqAtPos(isap)
     Integer(t_int),Pointer :: p_qposToSite(:,:) ! nsite,nsite
     !!CPA: 
     Integer(t_int),Pointer :: p_isst(:)  ! nsite
     !!CPA: 
     Integer(t_int),Pointer :: p_icis(:)  ! nsite
     !!CPA: 
     Integer(t_int),Pointer :: p_icisr(:) ! nsite
     !!  tau(3,nsite): sites in unit cell, including CPA degeneracy
     !!  unit is Bohr radii
     Real(t_real),Pointer  :: p_tau(:,:) ! 3,nsite
     !!  every sort has a concentration
     Real(t_real),Pointer  :: p_IOconao(:) ! IOnsort
     Real(t_real),Pointer  :: p_conao(:)   ! nsort
     !! Wigner Seitz radius
     Real(t_real) :: arws
     !! Volume of unit cell
     Real(t_real) :: vuc
     !! 
     !! in the relativistic case, we treat the calculation within the non
     !! relativistic setup for the lattice, but we need to reduce
     !! the symmetry (operations,Wyckoff-sorts)
     !! 
     Integer(t_int) :: nsort !! number of sorts Wyckoff positions in the
     !!                         reduced syummetry
     !! mapping of site onto reduced sort index, sorts count reduced 
     !! Wyckoff positions
     Integer(t_int),Pointer :: p_isrt(:) ! nsite
     !! multiplicity of relat sorts
     Integer(t_int),Pointer :: p_mult(:) ! nsort 
     !! map relativistic sorts onto non-relativistic sorts
     Integer(t_int),Pointer :: p_isrtTOIOisrt(:) ! nsort
     !! give for site and operation the transformed site g*site
     !! p_equivsites(jops,isa)=isb with S(isb)=G(jops) S(isa)
     Integer(t_int),Pointer :: p_equivsites(:,:) ! 48,nsite
     !! give for site and operation the transformed site g^(+)*site
     Integer(t_int),Pointer :: p_equivsitesrev(:,:) ! 48,nsite
     !! Wyckoff positions
     Character(2),Pointer  :: p_elements(:) ! nsort
     !! radius of compact support
     Real(t_real),Pointer  :: p_rcompsup(:) ! nsort
     !! radius defining the compact support
     Real(t_real),Pointer  :: p_rcut(:) ! nsort
  End Type T_Structure
  !!***********************************************************************
  !! constants
  Integer(t_int),Parameter :: SYM_LENGTH_BOHRRADII=1_t_int
  Integer(t_int),Parameter :: SYM_LENGTH_ANGSTROEM=2_t_int
  !! these constants are also defined in the c-code, they
  !! should not differ from there
  Integer(t_int),Parameter :: STR_TYPE_CRYSTAL  =1_t_int
  Integer(t_int),Parameter :: STR_TYPE_MOLECULE =2_t_int
  !!***********************************************************************
  !! 
  Character(10),Parameter :: SYM_LENGTH_UNITS(2)=(/"Bohr radii","Angstroem "/)
  Character(10)           :: STR_TYPES(2)
  Data STR_TYPES&
       &/"Crystal   "&
       &,"Molecule  "&
       &/
  !!
  !!***********************************************************************
contains
    Subroutine init_lattice(la)
    Implicit None 
    !!IO
    Type(T_Lattice),Intent(inout) :: la
    !!LOCAL
    Nullify(la% p_glat)
    Nullify(la% p_rsite)
    Nullify(la% p_ratom)
    Nullify(la% p_npairs)
    Nullify(la% p_pairs)
    Nullify(la% p_npshell)
    Nullify(la% p_ninpshell)
    Nullify(la% p_pairabs)
  End Subroutine init_lattice
  !!***********************************************************************
  Subroutine init_structure(st)
    Implicit None 
    !!IO
    Type(T_Structure),Intent(inout) :: st
    !!LOCAL
    st% str_type=STR_TYPE_CRYSTAL
    Nullify(st% p_IOwtau)
    Nullify(st% p_IOelements)
    Nullify(st% p_IOisrt)
    Nullify(st% p_IOmult)
    Nullify(st% p_kaps)
    Nullify(st% p_posToSiteRep)
    Nullify(st% p_siteToPos)
    Nullify(st% p_nqAtPos)
    Nullify(st% p_qposToSite)
    Nullify(st% p_isst)
    Nullify(st% p_icis)
    Nullify(st% p_icisr)
    Nullify(st% p_tau)
    Nullify(st% p_IOconao)
    Nullify(st% p_IOequivsites)
    Nullify(st% p_IOequivsitesrev)
    !! reduced symmetry
    Nullify(st% p_conao)
    Nullify(st% p_isrt)
    Nullify(st% p_mult)
    Nullify(st% p_isrtTOIOisrt)
    Nullify(st% p_equivsites)
    Nullify(st% p_equivsitesrev)
    Nullify(st% p_elements)
    Nullify(st% p_rcompsup)
    Nullify(st% p_rcut)
    
    !!
  End Subroutine init_structure
  !!***********************************************************************
  Subroutine init_symmetry(sy)
    Implicit None 
    !!IO
    Type(T_Symmetry),Intent(inout) :: sy 
    !!LOCAL
    !! avoid warnings
    sy% spgrpnum=1_t_int
    sy% nugen=0_t_int
    sy% iugen(:)=0_t_int
    Nullify(sy% p_Rwu_tjnl_ks)
!!$    Nullify(sy% p_sdcmp_vec)
!!$    Nullify(sy% p_sdcmp_npar_s)
  End Subroutine init_symmetry
  !!***********************************************************************
  
  
  

End Module tsymmetry

