Module twandesc
  !!
  Use precisiondefs
  Use errors
  Use file_iounits
  Use memory
  Use tgrid
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="tWANDESC"  
  !!
  Public :: T_WanDescriptor,T_WanCollection&
       &,init_wancol,cleanup_wancol&
       &,palloc_wandesc,pfree_wandesc,realloc_wandesc_contribs&
       &,compare_wandesc,compare_wandesc_sites,wandesc_maxorbvdim
  !!
  !!***********************************************************************
  !!
  Type T_WanDescriptor
     !! describe the semi automatic wannier descriptions
     Logical(t_log) :: active
     Character(17)  :: name
     Integer(t_int) :: lband(2),uband(2)
     Real(t_real)   :: emin(2),emax(2),del(2),deu(2),center(3)
     Integer(t_int) :: neigen
     Integer(t_int),Pointer :: p_ieigen(:)
     Integer(t_int) :: ncontrib
     Integer(t_int) :: equivdesc(48)
     Integer(t_int) :: nsymequiv
     Integer(t_int) :: symequiv(48)
     Integer(t_int) :: nsubspaces
     Integer(t_int),Pointer :: p_site(:),p_equivbeta(:,:),p_subspace(:,:)
     Real(t_real),Pointer   :: p_difvec(:,:)
     Complex(t_real),Pointer:: p_Bloc(:,:,:)
  End Type T_WanDescriptor
  !!
  Type T_WanCollection
     !! collect wandescriptors and map on individual wandefs
     Logical(t_log) :: write_WF_coeff_stats,write_Ham_t_stats,have_wforigin
     Real(t_real)   :: ham_rcut
     Real(t_real)   :: WF_ham_threshold,WF_coeff_threshold
     Real(t_real)   :: slab_direction(3)
     Logical(t_log) :: slab_reduce
     Real(t_real)   :: AA(3,3),GG(3,3)
     !! 
     Type(T_Grid)   :: wfgrid,kgrid
     !!
     Integer(t_int) :: nwdesc
     Type(T_WanDescriptor),Pointer :: p_wdesc(:)
     !!
     Integer(t_int) :: nwan
     Character(17),Pointer :: p_name(:)
     Integer(t_int),Pointer :: p_inddesc(:)
     Integer(t_int) :: nwcenteridesc
     Integer(t_int),Pointer :: p_wcenteridesc(:)
     !!
     Integer(t_int) :: nswfpairsmax,nwfpairsmax
     Integer(t_int),Pointer :: p_nswfpairs(:,:)
     Integer(t_int),Pointer :: p_nwfpairs(:,:)
     Real(t_real),Pointer :: p_swfpairs(:,:,:,:)
     Real(t_real),Pointer :: p_wfpairs(:,:,:,:)
  End Type T_WanCollection
  !!
Contains
  !!
  !!***********************************************************************
  Subroutine init_wancol(wc)
    !!
    Implicit None
    !!IO
    Type(T_WanCollection),Intent(inout) :: wc
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_wancol"
    !!
    Nullify(wc% p_wdesc)
    Nullify(wc% p_name)
    Nullify(wc% p_inddesc)
    Nullify(wc% p_wcenteridesc)
    wc% nwdesc=0
    wc% have_wforigin=.False.
    wc% write_Ham_t_stats=.False.
    wc% write_WF_coeff_stats=.False.
    wc% slab_direction=0.0_t_real
    wc% slab_reduce=.False.
    !!
    Call init_grid(wc% wfgrid)
    Call init_grid(wc% kgrid)
    !!
    wc% nswfpairsmax=0_t_int
    wc% nwfpairsmax=0_t_int
    Nullify(wc% p_nswfpairs)
    Nullify(wc% p_nwfpairs)
    Nullify(wc% p_swfpairs)
    Nullify(wc% p_wfpairs)
    Return
    !!
  End Subroutine init_wancol
  !!***********************************************************************
  Subroutine cleanup_wancol(wc)
    !!
    Implicit None
    !!IO
    Type(T_WanCollection),Intent(inout) :: wc
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_wancol"
    !!
    !!
    wc% nwdesc=0
    Call pfree_wandesc(wc% p_wdesc)
    Call pfree(wc% p_name)
    Call pfree(wc% p_inddesc)
    Call pfree(wc% p_nswfpairs)
    Call pfree(wc% p_nwfpairs)
    Call pfree(wc% p_swfpairs)
    Call pfree(wc% p_wfpairs)
    Call pfree(wc% p_wcenteridesc)
    !!
    Call cleanup_grid(wc% wfgrid)
    Call cleanup_grid(wc% kgrid)
    !!
    Return
    !!
  End Subroutine cleanup_wancol
  !!***********************************************************************
  Subroutine palloc_wandesc(name,wd,size,nmod,nsub)
    Implicit None 
    !!IO
    Integer(t_int),Intent(in)    :: size
    Type(T_WanDescriptor),Pointer:: wd(:)
    Character(*),Intent(in)      :: name,nmod,nsub
    !!LOCAL
    Integer :: icheck
    Integer(t_int) :: i
    !!
    !!
    Call pfree_wandesc(wd)
    !!
    Allocate(wd(1:size),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmod,nsub,E_fatalerror&
            &,msg="'Could not allocate memory for "//Trim(name)//" !'")
    Endif
    !!
    !!
    Do i=1,size
       Call init_wandesc(wd(i))
    Enddo
    !!
  End Subroutine palloc_wandesc
  !!***********************************************************************
  Subroutine pfree_wandesc(wd)
    Implicit None 
    !!IO
    Type(T_WanDescriptor),Pointer:: wd(:)
    !!LOCAL
    Integer(t_int) :: i
    !!
    !!
    If(Associated(wd))Then
       !!
       !If(STATISTICS)Then
       !   Call crealmemaddress(ra,add)
       !   Call free_chunk(add)
       !End If
       !!
       Do i=1,Size(wd)
          Call cleanup_wandesc(wd(i))
       Enddo
       !!
       Deallocate(wd)
       Nullify(wd)
    Endif
    !!
  End Subroutine pfree_wandesc
  !!***********************************************************************
  Subroutine init_wandesc(wd)
    !!
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout):: wd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_wandesc"
    !!
    !!
    wd% active=.True.
    wd% name=""
    wd% lband=0
    wd% uband=0
    wd% ncontrib=0
    wd% center=0.0_t_real
    wd% equivdesc(:)=0_t_int
    wd% neigen=0
    wd% nsubspaces=0
    Nullify(wd% p_subspace)
    Nullify(wd% p_ieigen)
    Nullify(wd% p_site)
    Nullify(wd% p_equivbeta)
    Nullify(wd% p_difvec)
    Nullify(wd% p_Bloc)
    !!
    Return
    !!
  End Subroutine init_wandesc
  !!***********************************************************************
  Subroutine cleanup_wandesc(wd)
    !!
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout):: wd
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_wandesc"
    !!
    !!
    wd% active=.True.
    wd% lband=0
    wd% uband=0
    wd% ncontrib=0
    wd% nsubspaces=0
    Call pfree(wd% p_subspace)
    Call pfree(wd% p_ieigen)
    Call pfree(wd% p_site)
    Call pfree(wd% p_equivbeta)
    Call pfree(wd% p_difvec)
    Call pfree(wd% p_Bloc)
    !!
    Return
    !!
  End Subroutine cleanup_wandesc
  !!***********************************************************************
  Subroutine realloc_wandesc_contribs(wd,newsize)
    !!
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout):: wd
    Integer(t_int),Intent(in)    :: newsize
    !!LOCAL
    Character(*),Parameter::nsubfunc="realloc_wandesc_contribs"
    !!
    If(wd% ncontrib>=newsize) Return
    Call realloc('wd% p_site',wd% p_site,1_t_int,newsize,nmodule,nsubfunc)
    Call realloc('wd% p_difvec',wd% p_difvec,1_t_int,3_t_int,1_t_int,newsize&
         &,nmodule,nsubfunc)
    Call realloc('wa% p_equivbeta',wd% p_equivbeta,1_t_int,newsize&
         &,1_t_int,48_t_int,nmodule,nsubfunc)
    !!
    wd% ncontrib=newsize
    !!
    Return
    !!
  End Subroutine realloc_wandesc_contribs
  !!***********************************************************************
  Logical(t_log) Function compare_wandesc(wa,wb,etol)
    !!compare all essential information of two wandesc to map symmetry related
    !!descriptions, site patterns arenot compared
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout) :: wa,wb
    Real(t_real),Intent(in)      :: etol
    !!LOCAL
    !Character(*),Parameter::nsubfunc="compare_wandesc"
    !!
    !!
    !!
    !!
    compare_wandesc=&
         &(All(Abs(wa% emin(:)-wb% emin(:))<etol)).And.&
         &(All(Abs(wa% emax(:)-wb% emax(:))<etol)).And.&
         &(All(Abs(wa% del(:)-wb% del(:))<etol)).And.&
         &(All(Abs(wa% deu(:)-wb% deu(:))<etol)).And.&
         &(All(wa% lband(:)==wb% lband(:))).And.&
         &(All(wa% uband(:)==wb% uband(:))).And.&
         &(wa% ncontrib==wb% ncontrib)
    !!
    !!
    Return
    !!
  End Function compare_wandesc
  !!***********************************************************************
  Logical(t_log) Function compare_wandesc_sites(wa,wga,jops&
       &,Ropmat,equivsites,vtol,prot)
    !!compare site pattern of first wandesc with rotated second wandesc
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout) :: wa,wga
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: jops,equivsites(:,:)
    Real(t_real),Intent(in)      :: vtol,Ropmat(:,:,:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="compare_wandesc_sites"
    Integer(t_int) :: isa,igsa,abeta,ngbeta,jbeta,jsa,igbeta
    Real(t_real)   :: del(3),gdel(3)
    !!
    !!
    compare_wandesc_sites=.True.
    !!
    !!
    abetaloop: Do abeta=1,wa% ncontrib
       isa=wa% p_site(abeta)
       igsa=equivsites(jops,isa)
       del=wa% p_difvec(:,abeta)
       gdel=Matmul(del,Ropmat(:,:,jops))
       If(prot) Write(outfile,"(t5,'site ',i5,' difvec ',3(f20.15,1x))")isa,del
       If(prot) Write(outfile,"(t5,'gsite',i5,'gdifvec ',3(f20.15,1x))")igsa,gdel
       igbeta=0_t_int
       ngbeta=0_t_int
       jbetaloop: Do jbeta=1,wga% ncontrib
          jsa=wga% p_site(jbeta)
          If(jsa/=igsa .Or. &
               &Any(Abs(wga% p_difvec(:,jbeta)-gdel)>vtol))&
               & Cycle jbetaloop
          igbeta=jbeta
          ngbeta=ngbeta+1
       Enddo jbetaloop
       If(ngbeta==1) Then
          !!
          If(prot) Write(outfile,"(t5,' equiv contrib is ',i5)")igbeta
          wa% p_equivbeta(abeta,jops)=igbeta
       Else
          compare_wandesc_sites=.False.
          If(prot) Write(outfile,"(t5,' no equiv site pattern')")
          return
       Endif
    Enddo abetaloop
    !!
    !!
    Return
    !!
  End Function compare_wandesc_sites
  !!***********************************************************************
  Integer(t_int) Function wandesc_maxorbvdim(wd,vblocklength)
    !!
    Implicit None
    !!IO
    Type(T_WanDescriptor),Intent(inout) :: wd
    Integer(t_int),Intent(in)    :: vblocklength(:)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="wandesc_maxorbvdim"
    Integer(t_int) :: beta
    !!
    wandesc_maxorbvdim=0
    !!
    Do beta=1,wd% ncontrib
       wandesc_maxorbvdim=wandesc_maxorbvdim+vblocklength(wd% p_site(beta))
    Enddo
    !!
    Return
    !!
  End Function wandesc_maxorbvdim
  !!***********************************************************************

End Module twandesc

