Module twannier
  !!
  Use precisiondefs
  Use memory
  Use tgrid
  Use ioaux, Only: T_OrbitalDescriptor,pfree_orbitaldescriptor
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="TWANNIER"  
  !!
  Public :: T_Wannier,init_wannier,cleanup_wannier&
       &,T_WanDefs,init_wandefs,cleanup_wandefs
  !!***********************************************************************
  !! definition of molecular orbital type
  Type T_WanDefs
     Integer(t_int)          :: nwan
     Integer(t_int)          :: ninwanmax
     Integer(t_int),Pointer  :: p_ninwan(:)     !! nwan
     Integer(t_int),Pointer  :: p_site(:,:)     !! ninwanmax,nwan
     Real(t_real),Pointer    :: p_difvec(:,:,:) !! 3,ninwanmax,nwan
     Real(t_real),Pointer    :: p_center(:,:)   !! 3,nwan
     Real(t_real),Pointer    :: p_ewind(:,:,:)  !! 3,nwan,mspin
     Integer(t_int),Pointer  :: p_lband(:,:)    !! 2,nwan
     Integer(t_int),Pointer  :: p_uband(:,:)    !! 2,nwan
     Real(t_real),Pointer    :: p_axes(:,:,:,:) !! 3,2,ninwanmax,nwan
     Type(T_OrbitalDescriptor),Pointer :: p_od(:,:)      !! ninwanmax,nwan
     Integer(t_int),Pointer  :: p_ivr(:,:)      !! ninwanmax,nwan
     Complex(t_real),Pointer :: p_fac(:,:)      !! ninwanmax,nwan
     Character(17),Pointer   :: p_name(:)       !! nwan
     !! number of equivalent MOs
     Integer(t_int),Pointer :: p_Neq(:,:)!! nwan,Rnumops
     !! MO number of i-th equivalent MOs is jwan=p_eqMO(i,iwan,jops)
     Integer(t_int),Pointer :: p_eqMO(:,:,:)!! nwan,nwan,Rnumops
     !! symmetry rep mats
     Complex(t_real),Pointer :: p_DMO(:,:,:)    !! nwan,nwan,Rnumops
     !! for a given iwan and jops  p_equivMOrep(jops,iwan) is an MO
     !! such that we have g_{jops} center_{iwan}=center_{jwan}
     !! since we did not sort input, there mightbe many such MOs
     !! however, we need only one to extract the trans-laws
     Integer(t_int),Pointer  :: p_equivMOrep(:,:)!! Rnumops,nwan
     !! we need our own lattice stuff for the Wanniers, i.e. for 
     !! symmetry reasons
     !! here between WF and sites
     Integer(t_int) :: nswfpairsmax
     Integer(t_int),Pointer :: p_nswfpairs(:,:) !! nsite,nwan
     Real(t_real),Pointer   :: p_swfpairs(:,:,:,:) !! 3,nsite,nwan,nswfpairsmax
     !! here between WFs, for Hamiltonian
     Real(t_real)   :: ham_rcut 
     !! threshold for printing the Hamiltonian elements and for the fit
     Real(t_real)   :: WF_ham_threshold 
     Integer(t_int) :: nwfpairsmax
     Integer(t_int),Pointer :: p_nwfpairs(:,:) !! nsite,nwan
     Real(t_real),Pointer   :: p_wfpairs(:,:,:,:) !! 3,nwan,nwan,nswfpairsmax
     !! threshold for printing the WF-coefficients
     Real(t_real)   :: WF_coeff_threshold 
     !! shall the WF coefficient statistics be written to file?
     Logical(t_log) :: write_WF_coeff_stats 
     !! shall the Hopping matrix elements statistics be written to file?
     Logical(t_log) :: write_Ham_t_stats 
     !! if periodic slab, avoid unnecessary k-points in vacuum direction, by
     !! defining the slab direction and the slab middle, the whole slab must
     !! be properly contained in a single c-period in [-0.5,0.5] around the middle
     Real(t_real)   :: slab_direction(3)
     Logical(t_log) :: slab_reduce
     Real(t_real)   :: AA(3,3),GG(3,3)
     !! combination matrix
     !!
     Complex(t_real),Pointer:: p_Dloc(:,:,:) !!maxedeg,ninwanmax,nwan
     !!
  End Type T_WanDefs
  !!***********************************************************************
  !! type to manage wannier function business
  Type T_Wannier
     !! shall any wannier business take place?
     Logical(t_log) :: do_wanniers
     !! initiate the wannier-data output, in order to do it faster
     !! after re-run?
     Logical(t_log) :: do_wannier_from_file
     !!
     Type(T_WanDefs) :: wdef
     !! output grid of real space Wannier functions (for pictures)
     Logical(t_log) :: have_wforigin
     Type(T_Grid)    :: wfgrid
     !! output grid in reciprocial space for Hamiltonian
     Type(T_Grid)    :: kgrid
     !!
  End Type T_Wannier
  !!***********************************************************************
  !!
  Complex(t_real),Parameter ::  c0=(0.0_t_real,0.0_t_real)
  Complex(t_real),Parameter ::  c1=(1.0_t_real,0.0_t_real)
  Complex(t_real),Parameter ::  ci=(0.0_t_real,1.0_t_real)
  !!
Contains
  !!***********************************************************************
  Subroutine init_wannier(wan)
    !!
    Implicit None
    !!IO
    Type(T_Wannier),Intent(inout) :: wan
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_wannier"
    !!
    !!
    wan% do_wanniers=.False.
    wan% do_wannier_from_file=.False.
    wan% have_wforigin=.False.
    !!
    Call init_wandefs(wan% wdef)
    !!
    Call init_grid(wan% wfgrid)
    Call init_grid(wan% kgrid)
    !!
    Return
    !!
  End Subroutine init_wannier
  !!***********************************************************************
  Subroutine cleanup_wannier(wan)
    !!
    Implicit None
    !!IO
    Type(T_Wannier),Intent(inout) :: wan
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_wannier"
    !!
    !!
    Call cleanup_wandefs(wan% wdef)
    !!
    Call cleanup_grid(wan% wfgrid)
    Call cleanup_grid(wan% kgrid)
    Return
    !!
  End Subroutine cleanup_wannier
  !!***********************************************************************
  Subroutine init_wandefs(md)
    !!
    Implicit None
    !!IO
    Type(T_WanDefs),Intent(inout) :: md
    !!LOCAL
    !Character(*),Parameter::nsubfunc="init_wandefs"
    !!
    !!
    md% nwan=0
    md% slab_direction=0.0_t_real
    md% slab_reduce=.False.
    Nullify(md% p_ninwan)
    Nullify(md% p_name)
    Nullify(md% p_DMO)
    Nullify(md% p_equivMOrep)
    Nullify(md% p_nswfpairs)
    Nullify(md% p_swfpairs)
    Nullify(md% p_nwfpairs)
    Nullify(md% p_wfpairs)
    Nullify(md% p_site)
    Nullify(md% p_difvec)
    Nullify(md% p_center)
    Nullify(md% p_ewind)
    Nullify(md% p_lband)
    Nullify(md% p_uband)
    Nullify(md% p_axes)
    Nullify(md% p_od)
    Nullify(md% p_fac)
    Nullify(md% p_Dloc)
    Nullify(md% p_Neq)
    Nullify(md% p_eqMO)
    !!
    Return
    !!
  End Subroutine init_wandefs
  !!***********************************************************************
  Subroutine cleanup_wandefs(md)
    !!
    Implicit None
    !!IO
    Type(T_WanDefs),Intent(inout) :: md
    !!LOCAL
    !Character(*),Parameter::nsubfunc="cleanup_wandefs"
    !!
    !!
    md% nwan=0
    md% slab_direction=0.0_t_real
    md% slab_reduce=.False.
    Call pfree(md% p_ninwan)
    Call pfree(md% p_name)
    Call pfree(md% p_DMO)
    Call pfree(md% p_equivMOrep)
    Call pfree(md% p_nswfpairs)
    Call pfree(md% p_swfpairs)
    Call pfree(md% p_nwfpairs)
    Call pfree(md% p_wfpairs)
    Call pfree(md% p_site)
    Call pfree(md% p_difvec)
    Call pfree(md% p_center)
    Call pfree(md% p_ewind)
    Call pfree(md% p_lband)
    Call pfree(md% p_uband)
    Call pfree(md% p_axes)
    Call pfree_orbitaldescriptor(md% p_od)
    Call pfree(md% p_fac)
    Call pfree(md% p_Dloc)
    Call pfree(md% p_Neq)
    Call pfree(md% p_eqMO)
    !!
    Return
    !!
  End Subroutine cleanup_wandefs
  !!***********************************************************************

End Module twannier
