!!TODO: check normalization and fac by counting degeneracies!
Module unfold
  !!
  Use precisiondefs
  Use memory
  Use errors
  Use constants
  Use tunfold
  Use tsymmetry
  Use clock
  Use tpotentialdensity
  Use ioaux
  Use file_iounits, Only : outfile
  Use tbandplot
  Use bandplot
  Use torbitalbasis
  Use atomdefs
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="UNFOLD"  
  !!
  Public :: setupUnfold,write_unfold_weight_header
  !!***********************************************************************
Contains 
  !!***********************************************************************
  Subroutine setupUnfold(structure,potential,basis,unfold)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Potential),Intent(in) :: potential
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Unfold),Intent(inout) :: unfold
    !!LOCAL
    Character(*),Parameter::nsubfunc="setupUnfold"
    Character(*),Parameter :: fname='=.unfold'
    Logical(t_log) :: exists,wrong
    Character(2)   :: el
    Integer(t_int) :: i,isa,idef,issa,issb,ivr
    Real(t_real)   :: Z
    Real(t_real),Parameter :: Ztol=1.0e-6_t_real
    !!
    !!
    !!
    Inquire(file=fname,exist=exists)
    If(.Not.exists) Return
    !!
    Write(outfile,"('UNFOLD:',72('-'))")
    !!
    unfold% active=.True.
    !!
    Call read_unfold_file(fname,unfold,readdata=.False.)
    !!
    Call palloc('unfold% p_NCsite',unfold% p_NCsite,1_t_int,unfold% ndefs&
         &,nmodule,nsubfunc)
    Call palloc('unfold% p_count',unfold% p_count,1_t_int,unfold% ndefs&
         &,nmodule,nsubfunc)
    Call palloc('unfold% p_SCsites',unfold% p_SCsites,1_t_int&
         &,unfold% maxSCsites,1_t_int,unfold% ndefs,nmodule,nsubfunc)
    !!
    !!
    !!
    Call read_unfold_file(fname,unfold,readdata=.True.)
    !!
    !!
    Write(outfile,"('UNFOLD: ')")
    Do idef=1,unfold% ndefs
       !!
       Do i=1,unfold% p_count(idef)
          isa=unfold% p_SCsites(i,idef)
          If(isa<1 .Or. isa> structure% nsite)Then
             Write(errbuf,661) idef,isa
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
       Enddo
       !!
       isa=unfold% p_SCsites(1,idef)
       issa=structure% p_isrt(isa)
       el=structure% p_elements(issa)
       Z=potential% p_natom(issa)
       !!
       Write(outfile,"('UNFOLD: SCsites(',a2,1x,i3,'):')",advance='no')&
            &el,unfold% p_NCsite(idef)
       !!
       Do i=1,unfold% p_count(idef)
          Write(outfile,"(a2,'(',a,')')",advance='no') &
               &structure% p_elements(structure% p_isrt(&
               &                unfold% p_SCsites(i,idef)))&
               &,Trim(intToChar10(unfold% p_SCsites(i,idef)))
          If(i<unfold% p_count(idef))&
               &Write(outfile,"(', ')",advance='no')
       Enddo
       Write(outfile,*)
       !!
       Do i=2,unfold% p_count(idef)
          issb=structure% p_isrt(unfold% p_SCsites(i,idef))
          !!
          wrong=(basis% p_nRv(issa)/=basis% p_nRv(issb))
          If(.Not.wrong)Then
             Do ivr=1,basis% p_nRv(issa)
                If(basis% p_nnRv(ivr,issa) /= basis% p_nnRv(ivr,issb) .Or.&
                     & basis% p_kapRv(ivr,issa) /= basis% p_kapRv(ivr,issb)&
                     & )Then
                   wrong=.True.
                   Exit
                Endif
             Enddo
          Endif
          !!
          If(wrong)Then
             Write(errbuf,662) idef
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          !!
          If(el/=structure% p_elements(issb))Then
             Write(errbuf,77) 
             Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
          Else
             !!
             If(potential% make_vca.And.Abs(Z-potential% p_natom(issb))>Ztol)Then
                Write(errbuf,78) 
                Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
             Endif
          Endif
       Enddo
    Enddo
    Write(outfile,"('UNFOLD: ')")
    Write(outfile,"('UNFOLD:',72('-'))")
    !!
    !!
    Call count_weights(structure,basis,unfold)
    !!
    Return
    !!
77  Format("'Atoms in super cell site list are not identical. Is that what you&
         & want?'") 
78  Format("'Atoms in super cell site list are different due to VCA.&
         & Is that what you want?'") 
662 Format("'Unfolding definition ",i3,": atoms in super cell site list &
         &do not',/,' have the same basis, only partial unfolding possible!',/&
         &,' Put the offending atom into a separate unfolding definition.'") 
661 Format("'Unfolding definition ",i3,": site number ",i5&
         &," does not exist in the current structure.'") 
    !!
  End Subroutine setupUnfold
  !!***********************************************************************
  Subroutine read_unfold_file(fname,unfold,readdata)
    !!
    Implicit None
    !!IO
    Type(T_Unfold),Intent(inout) :: unfold
    Character(*),Intent(in)      :: fname
    Logical(t_log),Intent(in)    :: readdata
    !!LOCAL
    Character(*),Parameter::nsubfunc="read_unfold_file"
    Integer(t_int) :: nline,i,count,idef
    Integer :: eof
    Character(5000):: buf,word
    !!
    Open(12,file=fname,status='old')
    !! count defs and maxSCsites
    !!
    nline=0
    idef=0
    If(.Not.readdata)Then
       unfold% maxSCsites=0
    Endif
    readloop0: Do 
       Read(12,12,iostat=eof) buf
       If(eof < 0 ) Exit readloop0
       nline=nline+1
       Call replace_tabs_by_blanks(buf)
       If( Len_trim(buf)==0) Cycle readloop0
       buf=Adjustl(buf)
       If( buf(1:1)=='#') Cycle readloop0
       !!
       idef=idef+1
       !!
       !! NCsite
       If(.Not.next_word(buf,word)) Then
          Write(errbuf,66) nline,Trim(buf)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       Read(word,*,iostat=eof) i
       If(readdata) unfold% p_NCsite(idef)=i
       If(eof/=0)Then
          Write(errbuf,66) nline,Trim(word)
          Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Endif
       !! SCsites
       count=0
       Do While(next_word(buf,word))
          Read(word,*,iostat=eof)i
          If(eof/=0)Then
             Write(errbuf,661) nline,Trim(word)
             Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
          Endif
          count=count+1
          If(readdata) unfold% p_SCsites(count,idef)=i
       Enddo
       If(.Not.readdata)Then
          unfold% maxSCsites=Max(unfold% maxSCsites,count)
       Else
          unfold% p_count(idef)=count
       Endif
    End Do readloop0
    !!
    if(.not.readdata) unfold% ndefs=idef
    !!
    Close(12)
    !!
    !!
    Return
    !!
12  Format(a)
66  Format("'Line ",i4," ''",a,"'':Expecting a normal cell site number'")
661 Format("'Line ",i4," ''",a,"'':Expecting a super cell site number'")
    !!
  End Subroutine read_unfold_file
  !!***********************************************************************
  Subroutine count_weights(structure,basis,unfold)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Unfold),Intent(inout)    :: unfold
    Type(T_Basis),Intent(in)     :: basis
    !!LOCAL
    !Character(*),Parameter::nsubfunc="count_weights"
    Integer(t_int) :: udef,isa,issa,iv,deg
    !!
    !!
    unfold% nw=0
    Do udef=1,unfold% ndefs
       isa=unfold% p_SCsites(1,udef)
       issa=structure% p_isrt(isa)
       Do iv=1_t_int,basis% p_nRv(issa)
          deg=basis% p_degRv(iv,issa)
          unfold% nw=unfold% nw+deg
       End Do
    End Do
    !!
    !!
    Return
    !!
  End Subroutine count_weights
  !!***********************************************************************
  Subroutine write_unfold_weight_header(unit,structure,basis,unfold&
       &,rel,lmsresolved)
    !!
    Implicit None
    !!IO
    Type(T_Structure),Intent(in) :: structure
    Type(T_Unfold),Intent(in)    :: unfold
    Type(T_Basis),Intent(in)     :: basis
    Integer(t_int),Intent(in)    :: unit
    Logical(t_log),Intent(in)    :: rel,lmsresolved
    !!LOCAL
    !Character(*),Parameter::nsubfunc="write_unfold_weight_header"
    Integer(t_int) :: isa,issa,ivr,deg,j,mv,mu,nmain,l,udef,isNC,kap,ns
    Character(1)   :: chl
    Character(2)   :: updn(2)
    Data updn(1)/'up'/
    Data updn(2)/'dn'/
    !!
    udefloop: Do udef=1,unfold% ndefs
       isNC=unfold% p_NCsite(udef)
       isa=unfold% p_SCsites(1,udef)
       issa=structure% p_isrt(isa)
       
       lmsresolvedif: If(.Not.lmsresolved)Then
          ivrloop: Do ivr=1_t_int,basis% p_nRv(issa)
             deg=basis% p_degRv(ivr,issa)
             l=basis% p_lRv(ivr,issa)
             chl=lqnchar(l)
             j=2*Abs(basis% p_kapRv(ivr,issa))-1
             Do mv=1_t_int,deg
                mu=-j+2*(mv-1)
                nmain=basis% p_nnRv(ivr,issa)
                !!
                If(rel) Then
                   Write(unit,123,advance='no')&
                        &structure% p_elements(issa),isNC&
                        &,nmain,chl,j,mu
                Else
                   Write(unit,23&
                        &,advance='no')&
                        &structure% p_elements(issa),isNC&
                        &,nmain,chl,mv-l-1_t_int
                Endif
                !!
             Enddo
          Enddo ivrloop
       Else lmsresolvedif
          ivrlooplms: Do ivr=1,basis% p_nRv(issa)
             kap=basis% p_kapRv(ivr,issa)
             If(kap>0)Cycle 
             l=(Abs(2*kap+1)-1)/2
             deg=2*l+1
             chl=lqnchar(l)
             nmain=basis% p_nnRv(ivr,issa)
             Do mv=1,deg
                !!
                Do ns=1,2
                   Write(unit,235&
                        &,advance='no')&
                        &structure% p_elements(issa),isNC&
                        &,nmain,chl,mv-l-1,updn(ns)
                Enddo
                !!
             Enddo
          Enddo ivrlooplms
          
       Endif lmsresolvedif
    Enddo udefloop
    !!
    Write(unit,24)
    !!
    Return
    !!
123 Format(a2,'(',i3.3,')',i1,a1,i1,'/2',SP,i2,'/2 ')
23  Format(a2,'(',i3.3,')',i1,a1,SP,i2,'      ')
235 Format(a2,'(',i3.3,')',i1,a1,SP,i2,a2,'    ')
24  Format()
    !!
  End Subroutine write_unfold_weight_header
  !!***********************************************************************

End Module unfold
