Module vatom
  !!
  Use precisiondefs
  Use constants
  Use errors
  Use memory
  Use file_iounits
  Use tsymmetry
  Use torbitalbasis
  Use tmesh
  Use tmol
  Use dump
  Use ioaux
  Use tatom
  Use diracdata
  Use diracpotorb
  Use diracio
  Use trelativistic
  Use finuc
  Use titeration
  Use iteration
  Use diracdriver
  Use mesh
  Use tpotentialdensity
  Use xc
  Use basdef
  Use atomdefs
  !!
  Implicit None  
  !!
  Private
  !!
  Character(*),Parameter::nmodule="VATOM"  
  !!
  Public :: prep_vatom
  !!
Contains
  !!***********************************************************************
  Subroutine prep_vatom(structure,basis,potential,mol,relativistic)
    !!
    Implicit None  
    !!IO
    Type(T_Structure),Intent(in) :: structure  
    Type(T_Potential),Intent(in) :: potential  
    Type(T_Basis),Intent(in)     :: basis
    Type(T_Mol),Intent(inout)    :: mol
    Integer(t_int),Intent(in)    :: relativistic
    !!LOCAL
    Character(*),Parameter::nsubfunc="prep_vatom"
    Logical(t_log) :: prot,Zeq0
    Integer(t_int) :: issa,ir,ivat,iZ
    Real(t_real)   :: r,v,ZZ,QQ,dZ,f,atrmax,ppp,Spol!,charge,n,p
    Real(t_real),Pointer :: p_exc(:)
    Real(t_real),Pointer :: p_vxc(:,:)
    Type(T_Mesh),Pointer :: atommesh
    Type(T_Atom),Target  :: atomsp
    Type(T_Atom),Pointer :: patom
    Character(100)   :: str
    !!
    Call init_tatom(atomsp)
    !!
    prot=(outlevel>=L_many)
    !!
    Nullify(p_exc)
    Nullify(p_vxc)
    !!
    Write(outfile,"('Preparing basis potentials ...',/&
         &,'(Wanna see details ... set verbosity level to second highest.)')")
    !!
    Call palloc('mol% p_vatom',mol% p_vatom,1_t_int,mol% nra&
         &,0_t_int,basis% maxNvat,1_t_int,structure% nsort, nmodule,nsubfunc)
    Call palloc('mol% p_excatom',mol% p_excatom,1_t_int,mol% nra&
         &,1_t_int,structure% nsort, nmodule,nsubfunc)
    Call palloc('mol% p_vxcatom',mol% p_vxcatom,1_t_int,mol% nra&
         &,1_t_int,2_t_int&
         &,1_t_int,structure% nsort, nmodule,nsubfunc)
    !!
    !!
    If( dumpread(DUMP_VATOM)  )Then 
       !!
       gdump(DUMP_VATOM)% doread=.False.
       !!
       Open(12,file=Trim(gdump(DUMP_VATOM)% fname))
       Do issa=1,structure% nsort
          atommesh => mol% p_atommeshs(issa)

          ivat=0
          Read(12,*)
          Do ir=1,atommesh% nr
             Read(12,*)r,v,mol% p_excatom(ir,issa)&
                  &,mol% p_vxcatom(ir,1,issa),mol% p_vxcatom(ir,2,issa)
             mol% p_vatom(ir,ivat,issa)=v*sq4pi
          Enddo
          Read(12,*)
          Do ivat=1,basis% maxNvat
             Read(12,*)
             Do ir=1,atommesh% nr
                Read(12,*)r,v
                mol% p_vatom(ir,ivat,issa)=v*sq4pi
             Enddo
             Read(12,*)
          Enddo
          !Enddo
       Enddo
       Close(12)
       !!
       Write(outfile,"('... Done')")
       !!
       Return
    End If
    !!........................................................................
    !!
    !!
    Do issa=1_t_int,structure% nsort
       !!
       Write(str,"('vatom',a)")intToChar10(issa)
       atommesh => mol% p_atommeshs(issa)
       !!
       iZ=getelementnumber(structure % p_elements(issa))
       ZZ=potential% p_natom(issa)
       atrmax=get_atrmax(iZ,1.0e-2_t_real,potential% make_vca,ZZ)
       !!
       !!
       If(prot.And.potential% make_vca) Then
          Write(outfile,"('VCA: ',a2,'(',i4,')',' Z=',f12.5&
               &,' ion=',f12.5)")&
               &structure % p_elements(issa),issa,ZZ,ZZ-iZ
       Endif
       !!
       mol% p_vatom(:,0:,issa)=0.0_t_real
       mol% p_excatom(:,issa)=0.0_t_real
       mol% p_vxcatom(:,:,issa)=0.0_t_real
       !!
       Zeq0=.False.
       If(ZZ<1.0e-5_t_real) Then 
          Zeq0=.True.
          !Call errmsg(nmodule,nsubfunc,E_fatalerror&
          !     &,msg="'Need empty site?  Call programmer, to remind him!'")
       Endif
       !!
       !!
       !!
       If(Zeq0)Goto 33
       !!
       Call init_tatom(g_atomat)
       !! default init input data
       Call diracinit(g_atomat)
       
       g_atomat% standalone=.False.
       !! set specifics
       g_atomat% mesh% nr=1000!atommesh% nr
       g_atomat% mesh% rmin=atommesh% rmin
       g_atomat% mesh% rmax=atrmax
          
       g_atomat% noUnOccupied=.True.
       g_atomat% force_pos_spin=.True.
       g_atomat% wftol=1.e-6_t_real
          
       !!
       g_atomat% gam_factor=.False.
       g_atomat% gam_factor=.True.!!needed for GGA
       !!
       g_atomat% mspin=1_t_int
       If(relativistic==NON_RELATIVISTIC)Then
          g_atomat% relativistic=NON_RELATIVISTIC
       Else If(relativistic==SCALAR_RELATIVISTIC_KH)Then
          g_atomat% relativistic=SCALAR_RELATIVISTIC_KH
       Else
          g_atomat% relativistic=FULL_RELATIVISTIC
       Endif
       !!
       g_atomat% compr=mol% compressed_vatom
       g_atomat% r0=0.8_t_real*atrmax
       g_atomat% exn0=18.0_t_real             
       g_atomat% vca=potential% make_vca
       g_atomat% xreduce=potential% xreduce
       g_atomat% Z=ZZ
       g_atomat% nuc_type=potential% nuc_type
       g_atomat% element=structure% p_elements(issa)
       g_atomat% ionicity=0.0_t_real
       !!
       Call set_def_atom_basis_and_occus(g_atomat)
       Call updateRawOccus(g_atomat)
       !!
       g_atomat% iteration_control% scheme   =IT_SCHEME_LCITERAT
       g_atomat% iteration_control% maxdim   =5_t_int
       g_atomat% iteration_control% tolerance=1.e-6_t_real
       g_atomat% iteration_control% mixing   = 0.2_t_real
       g_atomat% iteration_control% progress = 0.1_t_real
       g_atomat% iteration_control% steps    = 200_t_int
       !!
       !! setup allocation
       Call diracsetup(g_atomat)
       !! do calculation
       Call iterate(sccatom,str,.False.,prot)
       
       !! reinterpolation and storage of potential 
       !!
       ivat=0
       Call palloc('p_exc',p_exc,1_t_int,g_atomat% mesh% nr,nmodule,nsubfunc)
       Call palloc('p_vxc',p_vxc,1_t_int,g_atomat% mesh% nr&
            &,1_t_int,2_t_int,nmodule,nsubfunc)
       Call mol_exc_density(g_atomat,p_exc,p_vxc,potential% xreduce)
       !!
       Call mreinter(8_t_int,g_atomat% mesh,p_exc&
            &,atommesh,mol% p_excatom(:,issa))
       Call mreinter(8_t_int,g_atomat% mesh,p_vxc(:,1)&
            &,atommesh,mol% p_vxcatom(:,1,issa))
       Call mreinter(8_t_int,g_atomat% mesh,p_vxc(:,2)&
            &,atommesh,mol% p_vxcatom(:,2,issa))
       !!
       Call pfree(p_exc)
       Call pfree(p_vxc)
       !!
       !Call writefunc(g_atomat% mesh% nr,g_atomat% mesh% p_rmesh,g_atomat% p_vtr,12,'+vtr')
       !Stop
       
       Call mreinter(8_t_int,g_atomat% mesh,g_atomat% p_vtr&
            &,atommesh,mol% p_vatom(:,ivat,issa))
       !!
       mol% p_vatom(:,ivat,issa)=mol% p_vatom(:,ivat,issa)*sq4pi
       !!
       Do ir=1,atommesh% nr
          r=atommesh% p_rmesh(ir)
          f=fku((r/atommesh% rmax))
          mol% p_excatom(ir,issa)=mol% p_excatom(ir,issa)*f
          mol% p_vxcatom(ir,1,issa)=mol% p_vxcatom(ir,1,issa)*f
          mol% p_vxcatom(ir,2,issa)=mol% p_vxcatom(ir,2,issa)*f
          mol% p_vatom(ir,ivat,issa)=mol% p_vatom(ir,ivat,issa)*f
       Enddo
       !!
       !!
       Call copy_atom(g_atomat,atomsp)
       atomsp% mspin=2
       Call updateRawOccus(atomsp)
       Call atominit(atomsp,str,prot)
       Call atom_transfer_fv(g_atomat,atomsp)
       !!
       33 continue
       !!
       ivatloop: Do ivat=1,basis% p_Nvat(issa)
          !!
          QQ=basis% p_Qvat(ivat,issa)
          Spol=basis% p_Svat(ivat,issa)
          !!
          dZ=Max(0.0_t_real,(QQ-ZZ))
          If(prot)Write(outfile,877) Trim(str),ivat,dZ,QQ,Spol
877       Format(a,': ivat=',i3,' dZ=',f10.5,' QQ=',f10.5,' Spol=',f10.5)
          !!
          If(dZ>0.0_t_real)Then
             !! make modified slater type orbitals for over-ioniced setting
             !!
             ppp=Min(1.0_t_real&
                  &,Max(0.2_t_real&
                  &,(ZZ-4.0_t_real)*0.8_t_real+0.2_t_real))
             !!
             Do ir=1,atommesh% nr
                r=atommesh% p_rmesh(ir)
                mol% p_vatom(ir,ivat,issa)=&
                     &-(ZZ+dZ*(1.0_t_real-Exp(-r/ppp)))*sq4pi
             Enddo
             If(Zeq0)Then
                Do ir=1,atommesh% nr
                   r=atommesh% p_rmesh(ir)
                   mol% p_vatom(ir,ivat,issa)=-dZ*sq4pi*r
                Enddo
             Endif
             Cycle ivatloop
          Endif
          !!
          If(Zeq0) Cycle ivatloop
          !!
          patom=>g_atomat
          If(Abs(Spol)>1.0e-6_t_real)Then
             patom=>atomsp
          Endif
          !!
          patom% ionicity=Max(QQ,0.0_t_real)
          patom% Magmom=0.0_t_real
          !!
          If(Abs(Spol)>1.0e-6_t_real)Then
             patom% Magmom=Spol
          Endif
          !!
          Call updateRawOccus(patom)
          Call make_occus(patom,str,prot)
          !!
          !! make new densities with QQ electrons taken out
          !!
          Call atcalc_rho(patom,str,prot)
          !!
          !! calculate potential
          Call potat(patom,1_t_int,1_t_int,str,.False.,potential% xreduce)!prot)
          !!
          !! reinterpolation and storage of potential 
          !!
          !!
          !!
          If(Spol>1.0e-6_t_real)Then
             Call mreinter(8_t_int,patom% mesh&
                  &,patom% p_vtr+patom% p_btr&
                  &,atommesh,mol% p_vatom(:,ivat,issa))
          Else If(Spol<-1.0e-6_t_real)Then
             Call mreinter(8_t_int,patom% mesh&
                  &,patom% p_vtr-patom% p_btr&
                  &,atommesh,mol% p_vatom(:,ivat,issa))
          Else
             Call mreinter(8_t_int,patom% mesh,patom% p_vtr&
               &,atommesh,mol% p_vatom(:,ivat,issa))
          Endif
          !!
          mol% p_vatom(:,ivat,issa)=mol% p_vatom(:,ivat,issa)*sq4pi
          !!
          If(QQ<0.0_t_real) Then
             ppp=Min(1.0_t_real&
                  &,Max(0.2_t_real&
                  &,(ZZ-4.0_t_real)*0.8_t_real+0.2_t_real))
             Do ir=1,atommesh% nr
                r=atommesh% p_rmesh(ir)
                mol% p_vatom(ir,ivat,issa)=mol% p_vatom(ir,ivat,issa)&
                     &+(QQ*(1.0_t_real-Exp(-r/ppp)))*sq4pi
             Enddo
          Endif
          !!
       Enddo ivatloop
       !!
       !!
       !! cleanup allocation          
       Call clean_tatom(g_atomat)
       !!
    Enddo
    !!
    !!
    If( dumpwrite(DUMP_VATOM)  )Then 
       Open(12,file=Trim(gdump(DUMP_VATOM)% fname))
       Do issa=1,structure% nsort
          atommesh => mol% p_atommeshs(issa)
          ivat=0
          Write(12,35)issa,ivat
          Do ir=1,atommesh% nr
             r=atommesh% p_rmesh(ir)
             v=mol% p_vatom(ir,ivat,issa)/sq4pi
             Write(12,34)r,v,mol% p_excatom(ir,issa)&
                  &,mol% p_vxcatom(ir,1,issa),mol% p_vxcatom(ir,2,issa)
          Enddo
          Write(12,2)
          Do ivat=1,basis% maxNvat
             Write(12,36)issa,ivat
             Do ir=1,atommesh% nr
                r=atommesh% p_rmesh(ir)
                v=mol% p_vatom(ir,ivat,issa)/sq4pi
                Write(12,34)r,v
             Enddo
             Write(12,2)
          Enddo
       Enddo
       Close(12)
    Endif
    !!
2   Format(' ')
34  Format(10(e39.15e3,2x))
!35  Format('# sort=',i5,' ms=',i2)
35  Format('# sort=',i5,' ivat=',i5,' and exc-atom')
36  Format('# sort=',i5,' ivat=',i5)
    !!
    Write(outfile,"('... Done')")
    !!
    Call clean_tatom(atomsp)
    !!
    Return
    !!
  Contains
    !!
    Real(t_real) Function fku(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fku=(1.0_t_real-fk(fk(x)))
    End Function fku
    !!
    Real(t_real) Function fku3(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fku3=(1.0_t_real-fk(fk(fk(x))))
    End Function fku3
    !!
    Real(t_real) Function fk(x)
      Implicit None 
      !!IO
      Real(t_real),Intent(in)     :: x
      !!LOCAL
      fk=1.0_t_real-(Cos(pi*x)+1.0_t_real)*0.5_t_real  
    End Function fk
    !!
  End Subroutine prep_vatom
  !!***********************************************************************
  !! xc-potential + energy
  !!
  Subroutine mol_exc_density(atom,exc,vxc,xreduce)
    !!
    Implicit None
    !!IO
    Type(T_Atom),Intent(in)      :: atom
    Real(t_real),Intent(in)      :: xreduce
    Real(t_real),Intent(out)     :: exc(:)
    Real(t_real),Intent(out)     :: vxc(:,:)
    !!LOCAL
    Character(*),Parameter::nsubfunc="mol_exc_density"
    Logical(t_log) :: alert
    Integer(t_int):: i,mspin
!    Real(t_real) :: cp
 !   Real(t_real):: ee(mesh% nr)
    Real(t_real)   :: rhos(2),r,r2,vex(2),eps,tol,error
    Real(t_real)   :: r0,rho,dums(2),gradrho,laprhos(2),mixrho

    mspin=1
    alert=.False.
    error=0.0_t_real
    tol=1.e-6_t_real
    
    r0=50.0_t_real*atom% mesh% p_rmesh(2) !!TODO: check this out !!


    
    exc(:)=0.0_t_real
    vxc(:,:)=0.0_t_real
    !!
    Do i=1_t_int,atom% mesh% nr
       r=atom% mesh% p_rmesh(i)
       r2=r**2_t_int
       If(i==1)Then
          !! avoid division by zero
          r2=1.0_t_real
       Endif



       rhos(1)=atom% p_rho(i)/r2/fourpi
       rhos(2)=0.0_t_real

       gradrho=atom% p_drho(i)/r2/fourpi

       laprhos(1)=atom% p_d2rho(i)/r2/fourpi
       laprhos(2)=0.0_t_real
       !!
       mixrho=(laprhos(1))*gradrho**2
       !!
       rho=rhos(1)
       !!
       If(i==1)Then
          !! put some value, to get through
          rhos(1:2)=0.0_t_real
          laprhos(1:2)=0.0_t_real
          mixrho=0.0_t_real
          gradrho=0.0_t_real
       Else
          laprhos(1)=laprhos(1)+2.0_t_real/r*gradrho
       Endif
       !!
       !!
       gradrho=Abs(gradrho)

       !!

       !!       xc-potential
       Call adjustrho(rhos,1.0_t_real,error,mspin,tol,alert)
       Call vxcrho(mspin,rhos,dums,gradrho,laprhos,dums,mixrho&
            &,vex,eps,xreduce)
       
       If(i>1)Then
          exc(i)=eps*r*sq4pi
          vxc(i,1)=vex(1)*r*sq4pi
          vxc(i,2)=vex(2)*r*sq4pi
       Endif
    Enddo! i
    !!
    !!
    !!
    If(alert)Then
       Write(errbuf,15) error
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=errbuf)
    Endif
    
    !!
    Return
    !!
15  Format('''Numerical inaccuracy in vxc/exc '&
         &//'calculation! '',/,'' maxerror=',e12.4e3,'''')
    !!
  End Subroutine mol_exc_density
  !!***********************************************************************
End Module vatom


