Module vlm
  !!
  Use precisiondefs
  Use memory
  Use errors
  Use clock
  Use tsymmetry
  Use tmesh
  Use mesh
  Use interpolation
  Use surfaceintegration
  Use tpotentialdensity
  Use matserv
  Use nsphintirred
  Use tio
  Use file_iounits
  Use constants
  Use pot
  !!
  Implicit None 
  !!
  Private
  !!
  Character(*),Parameter::nmodule="VLM"  
  !!
  Public :: calc_vlm
  !!
  !!***********************************************************************
contains 
  !!***********************************************************************
  !!
  Subroutine calc_vlm(mspin,symmetry,structure,lattice,potential,degree)
    !!
    Implicit None  
    !!IO
    Type(T_Symmetry),Intent(in)  :: symmetry
    Type(T_Structure),Intent(in) :: structure
    Type(T_Lattice),Intent(in)   :: lattice
    Type(T_Potential),Intent(in) :: potential
    Integer(t_int),Intent(in)    :: mspin,degree
    !!LOCAL
    Character(*),Parameter::nsubfunc="calc_vlm"
    Character(*),Parameter::cpumesg='V_lm calculation'

    Type(T_Mesh)   :: mesh
    Logical(t_log) :: exists
    Character(5000)   :: buf
    Integer(t_int) :: isap,isa,issa,nvg,ir,nga,ig,indexa,in,indexb,isb
    Integer(t_int) :: ish,insh,n,ne,nf,issb,isa2,ms,j,iq,l,m,issa2,ilm,nlma
    Real(t_real)   :: ri,w,rsh(3),rbb,rb,r2(3),d,vhoff,vvew!,rnnb,fq,Z
    Real(t_real)   :: ag(3,5000),vhar
    Real(t_real)   :: wg(5000),tt(3)
    Real(t_real)   :: coefa(0:degree)
    Real(t_real)   :: coefb(0:degree)
    Real(t_real),Pointer :: vh(:,:,:)
    Real(t_real),Pointer :: vhloc(:,:),vhonsite(:),ydn(:),ya(:),yb(:)
    Type(T_Mesh),Pointer :: pmesha,pmeshb
    !!
    !!
    !!
    Inquire(file=f_vlm,exist=exists)
    If(.Not. exists)Return
    !!
    Write(outfile,1)
    Write(errbuf,22)Trim(f_vlm)
22  Format('"File ''',a,''' found, will calculate&
         & lm-expansion of total",/," crystal potential around all sites!",/')
    Call errmsg(nmodule,nsubfunc,E_information,msg=Trim(errbuf))
    !!
    Write(errbuf,221)
221 Format('"Vlm mode currently switched off!",/')
    Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=Trim(errbuf))
    Return !!TODO:
    !!
    Call watch(cpumesg,cpu_start,reset=.True.)
    !!
    Nullify(vhloc)
    Nullify(vhonsite)
    Nullify(ydn)
    Nullify(ya)
    Nullify(yb)
    Nullify(vh)
    Call init_mesh(mesh)
    !!
    !!
    nvg=20
    !!
    !! Take full mesh just in case. Could be done better, I know. 
    Call gsphirred(nvg&
         &,ag&
         &,wg&
         &,nga,symmetry% HEX,1_t_int,.False.)
    !!
    Call palloc('vhloc',vhloc&
         &,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    !!
    Call palloc('vhonsite',vhonsite&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    Call palloc('ydn',ydn&
         &,1_t_int,degree+1_t_int&
         &,nmodule,nsubfunc)
    !!
    Call palloc('ya',ya&
         &,1_t_int,(12_t_int+1_t_int)**2&
         &,nmodule,nsubfunc)
    !!
    Call palloc('yb',yb&
         &,1_t_int,(12_t_int+1_t_int)**2&
         &,nmodule,nsubfunc)
    !!
    Call palloc('vh',vh&
         &,1_t_int,potential% nrp&
         &,1_t_int,potential% mlmp&
         &,1_t_int,structure% nsite&
         &,nmodule,nsubfunc)
    !!
    vh(:,:,:)=0.0_t_real
    !!
    isaploop: Do isap=1,structure% naps
       isa=structure% p_posToSiteRep(isap)
       issa=structure% p_isrt(isa)
       !!
       pmesha => potential% p_pmeshs(issa)
       !!
       !!
       !!
       mesh% Type=POW_MESH
       mesh% with_zero=.False.
       mesh% nr=50_t_int
       mesh% rmin=pmesha% rmin !! actually not needed for POW_MESH
       mesh% rmax=1.0_t_real! pmesha% rmax
       mesh% alpha=3.0_t_real
       !!
       Call setup_mesh(mesh)
       !!
       tt(:)=structure % p_tau(:,isa)
       !!
       rloop: Do ir=1,mesh% nr
          !!
          If(Mod(ir,(mesh% nr/10))==0)Then
             Write(outfile,5)isa,100.0_t_real*ir/mesh% nr
5            Format('site: ',i5,' ',f5.1,'% done')
          Endif
          !!
          ri=mesh% p_rmesh(ir)
          !!
          !! collect local hartree potential
          indexa=mintpl_coefs(degree,pmesha,ri,coefa)
          !!
          Do iq=1_t_int,structure% p_nqAtPos(isap)
             isa2=structure% p_qposToSite(isap,iq)
             issa2=structure% p_isrt(isa2)
             nlma=potential% p_nlmp(isa2)
             
             Do ms=1_t_int,mspin
                !!TODO: density for xc
             Enddo
             !!
             Do j=1_t_int,nlma
                vhloc(j,iq)=Dot_product(&
                     &potential% p_vc(indexa:indexa+degree,j,isa2)&
                     &,coefa(0:degree)) 
             Enddo ! j
          Enddo ! iq
          !!
          !!
          !! angular integration mesh
          igloop: Do ig=1,nga
             rsh(:)=ri*ag(:,ig)
             w=wg(ig)

             !! ewald contribution
             If(structure% str_type == STR_TYPE_CRYSTAL)Then
                vvew=vew(rsh(:)+tt(:),potential% p_vft&
                     &,lattice% nlattg,lattice% p_glat)
             Else If(structure% str_type == STR_TYPE_MOLECULE)Then
                vvew=vew_sum(rsh(:)+tt(:),structure% nsite,structure% p_isrt&
                     &,structure% p_tau,potential% p_pmeshs,potential% nrp,degree&
                     &,potential% max_L,potential% mlmp,potential% p_nlmp&
                     &,potential% p_lmp,potential% p_vew&
                     &,potential% p_ac)
             Else
                Call terminate(TERM_CRASH,Trim(nsubfunc)//'unknown str_type!')
             Endif

             Call yl12(ya,rsh(1),rsh(2),rsh(3)) 
             !! onsite Hartree contributions
             Do iq=1,structure% p_nqAtPos(isap)
                isa2=structure% p_qposToSite(isap,iq)
                nlma=potential% p_nlmp(isa2)
                Do ms=1_t_int,mspin
                   !!TODO: density
                Enddo
                vhonsite(iq)=Sum(vhloc(1:nlma,iq)&
                     &*ya(potential% p_lmp(1:nlma,isa2)))
             Enddo ! iq
             !!
             !!           neighbour contributions
             !!
             vhoff=0.0_t_real
             !!
             !!
             !fq=1.0_t_real
             isbloop: Do isb=1_t_int,structure% nsite
                issb=structure% p_isrt(isb)
                !!
                pmeshb => potential% p_pmeshs(issb)
                !!
                !rnnb=lattice% p_rsite(issb)*1.5
                !!
                n=0_t_int
                ishloop: Do ish=1_t_int,lattice% p_npshell(isb,isa)
                   nf=n+1_t_int
                   ne=n+lattice% p_ninpshell(ish,isb,isa)
                   n=ne
                   If(lattice% p_pairabs(ish,isb,isa) > &
                        &(mesh% rmax+pmeshb% rmax)) Cycle ishloop
                   inshloop: Do insh=nf,ne
                      r2(:)=rsh(:)-lattice% p_pairs(:,insh,isb,isa)
                      rbb=Dot_product(r2,r2)
                      rb=Sqrt(rbb)
                      If(rb > pmeshb% rmax .Or. rb ==0.0_t_real) Cycle inshloop
                      !!
                      Call yl12(yb,r2(1),r2(2),r2(3))
                      !!
                      !! TODO: index underflow protection
                      indexb=mintpl_coefs(degree,pmeshb,rb,coefb)
                      If(indexb>0_t_int)Then
                         Do ms=1_t_int,mspin
                            !!TODO: density
                         Enddo ! ms
                         !!
                         Do in=0_t_int,degree
                            d=0.0_t_real
                            Do j=1_t_int,potential% p_nlmp(isb)
                               d=d+potential% p_vc(indexb+in,j,isb)&
                                    &*yb(potential% p_lmp(j,isb))
                            Enddo
                            ydn(in+1_t_int)=d
                         Enddo
                         vhoff=vhoff&
                              &+Dot_product(ydn(1:1+degree),coefb(0:degree))/rb
                      End If
                      !!
                   Enddo inshloop
                Enddo ishloop
             Enddo isbloop
             !!
             !! collect result
             !!
             Do iq=1_t_int,structure% p_nqAtPos(isap)
                isa2=structure% p_qposToSite(isap,iq)
                !!
                Do ms=1_t_int,mspin
                   !!TODO: density
                Enddo             ! ms
                !!   specific onsite + [CPA-avergaed] offsite+ Ewald
                vhar=vhonsite(iq)/ri*1+vhoff*1+vvew*1
                !!
                Do j=1_t_int,potential% p_nlmp(isa2)
                   vh(ir,j,isa2)=vh(ir,j,isa2)+&
                        & (vhar*ya(potential% p_lmp(j,isa2))*w)*ri
                Enddo           ! j
                !!
             Enddo               ! iq
             !!
             !!
          Enddo igloop
          !!
          !!
       Enddo rloop
       !!
       Call cleanup_mesh(mesh)
       !!
    Enddo isaploop
    !!
    !! storage on file
    !!
    Do isa=1_t_int,structure% nsite
       issa=structure% p_isrt(isa)
       pmesha => potential% p_pmeshs(issa)
       !!
       !!
       !!
       mesh% Type=POW_MESH
       mesh% with_zero=.False.
       mesh% nr=50_t_int
       mesh% rmin=pmesha% rmin !! actually not needed for POW_MESH
       mesh% rmax=1.0_t_real! pmesha% rmax
       mesh% alpha=3.0_t_real
       !!
       Call setup_mesh(mesh)
       !!
       Write(buf,130)isa
       Open(12,file=Trim(buf),status='unknown')
       Write(12,140) isa,structure% p_elements(issa),mesh% nr
       Do j=1_t_int,potential% p_nlmp(isa)
          ilm=potential% p_lmp(j,isa)
          l=llilm(ilm)
          If(l /= 2) Cycle !!TODO
          m=mmilm(ilm)-l-1
          Write(12,20)l,m
          Do ir=1_t_int,mesh% nr
             ri=mesh% p_rmesh(ir)
             If(Abs(ri)<1.0e-9_t_real) Cycle !!TODO
             d=vh(ir,j,isa)/ri
!!$             d=vh(ir,j,isa)/ri*Sqrt(15.0_t_real/fourpi)&
!!$                  &*hatoev/brtoan**2/10.0_t_real
!!$             If(m==0)d=d*Sqrt(4.0_t_real/3.0_t_real)
!!$             !Print *,hatoev/brtoan**2
             Write(12,10)ri,d/ri**2 !! TODO
          Enddo               ! ir
          Write(12,2)
       Enddo                 ! j
       Close(12)
       !!
       Call cleanup_mesh(mesh)
       !!
    Enddo                   ! isa
    !!
!!$    Do ms=1_t_int,mspin
!!$       Do isa=1_t_int,structure% nsite
!!$          issa=structure% p_isrt(isa)
!!$          Write(buf,30)isa,ms
!!$          Open(12,file=Trim(buf),status='unknown')
!!$          Write(12,40) isa,structure% p_elements(issa),ms,mesh% nr
!!$          Do j=1_t_int,potential% p_nlmp(isa)
!!$             ilm=potential% p_lmp(j,isa)
!!$             l=llilm(ilm)
!!$             If(l /= 2) Cycle !!TODO
!!$             m=mmilm(ilm)-l-1
!!$             Write(12,20)l,m
!!$             Do ir=1_t_int,mesh% nr
!!$                ri=mesh% p_rmesh(ir)
!!$                If(Abs(ri)<1.0e-9_t_real) Cycle !!TODO
!!$                d=vh(ir,j,isa)/ri
!!$                Write(12,10)ri,d/ri**2 !! TODO
!!$             Enddo               ! ir
!!$             Write(12,2)
!!$          Enddo                 ! j
!!$          Close(12)
!!$       Enddo                   ! isa
!!$    Enddo                     ! ms
    !!
    !!
    Call cleanup_mesh(mesh)
    Call pfree(vhloc)
    Call pfree(vhonsite)
    Call pfree(ydn)
    Call pfree(ya)
    Call pfree(yb)
    Call pfree(vh)
    !!
    Call watch(cpumesg,cpu_stop,delete=.True.,Print=.True.)
    !!
    If(structure% nsite /= structure% naps)Then
567    Format('"The lm-expansion of the crystal potential may be doubtful",/,"&
            & for CPA cases, since it contains an conditionally averaged&
            & term!!!"')
       Write(errbuf,567)
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
       Call terminate(TERM_NORMAL,Trim(nsubfunc)//': VLM and CPA warning!')
    Endif
    !!
    Call terminate(TERM_NORMAL,Trim(nsubfunc)//': VLM calculated!')
    !!
    Return
    !!
1   Format()
2   Format(' ')
10  Format(10(e30.15e3,2x))
20  Format('# l,m=',i3,',',i3)
!30  Format('+vlm.',i3.3,'.',i1)
130 Format('+vharlm.',i3.3)
140 Format('# site:',i5,' element:',a2&
         &,/,'# mesh: r(i)=x(i)^3, x_i equidistant, i=1,',i4)
!40  Format('# site:',i5,' element:',a2,' spin: ',i1&
!         &,/,'# mesh: r(i)=x(i)^3, x_i equidistant, i=1,',i4)
    !!
    !!
  End Subroutine calc_vlm
  !!***********************************************************************
End Module vlm
