!!todo: make second order for vbh and pz
!!todo: profiling
Module xc
  !!
  Use precisiondefs
  Use file_iounits
  Use constants
  Use errors
  Use matserv
  !!
  Implicit None
  !!
  Private
  !!
  Public :: &
       !! global variables
       & XC_VERSIONS& !! description strings
       &,MAP_XC_VERSIONS& !! mapping of numbers into array indices
       &,setxcversion &
       &,getxcversion &
       !! subroutines
       &,vxcrho,adjustrho,getXCVersionIndex,setxcrelat,getxcrelat&
       &,kxcrho,derivxcrho&
       !! constants
       &,xcver_xonly,xcver_vbarthhedin,XCVER_PERDEWZUNGER&
       &,xcver_perdewwang92,XCVER_PBE96
  !!
  Character(*),Parameter::nmodule="XC"
  !!***********************************************************************
  !! constants
  Integer(t_int),Parameter :: XCVER_XONLY             = 1_t_int
  Integer(t_int),Parameter :: XCVER_VBARTHHEDIN       = 2_t_int
  Integer(t_int),Parameter :: XCVER_PERDEWZUNGER      = 8_t_int
  Integer(t_int),Parameter :: XCVER_PERDEWWANG92      = 9_t_int
  Integer(t_int),Parameter :: XCVER_PBE96             =10_t_int
  !!
  Integer(t_int),Parameter :: xcversionlength         =37_t_int
  Character(xcversionlength)   :: XC_VERSIONS(5)
  Integer(t_int)        :: MAP_XC_VERSIONS(5)
  !! translation tables
  Data XC_VERSIONS/&
       & "Exchange only                (LSDA)  "&
       &,"von Barth Hedin              (LSDA)  "&
       &,"Perdew and Zunger            (LSDA)  "&
       &,"Perdew Wang 92               (LSDA)  "&
       &,"Perdew Burke Ernzerhof 96    (GGA)   "&
       &/
  Data MAP_XC_VERSIONS/&
       & xcver_xonly &
       &,xcver_vbarthhedin &
       &,XCVER_PERDEWZUNGER &
       &,xcver_perdewwang92 &
       &,XCVER_PBE96 &
       &/
  !! global version variable
  Integer(t_int),Private   :: xcversion=XCVER_PERDEWZUNGER
  !! xcrelat: correction factors for relativistic calculations
  !! MacDonald and Vosko 1979
  !! = .true. if dirac equation AND C=137...._t_real
  !! = .false. if schroedinger OR C>>137._t_real
  !! not applicable to Perdew Wang 92
  Logical(t_log),Private         :: xcrelat  =.False. !! blah
  !!
  !!***********************************************************************
Contains
  !!***********************************************************************
  !!info
  !!info calculate the exchange potential and energy of density rho
  !!info
  !!info if mspin.eq.1 then total density and potential are in rho(1)
  !!info and vex(1)
  !!info
  Subroutine vxcrho(mspin,rho,gradrhos,gradrho,laprhos,mixrhos,mixrho&
       &,vex,eps,xreduce)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) :: rho(2)
    Real(t_real),Intent(in) :: gradrhos(2),gradrho,laprhos(2),mixrhos(2)
    Real(t_real),Intent(in) :: mixrho
    Real(t_real),Intent(in) :: xreduce !!  reduce xc-field B -> xreduce*B
    Real(t_real),Intent(out):: vex(2),eps
    !!LOCAL
    Character(*),Parameter::nsubfunc="vxcrho"
    Logical(t_log) :: dored
    Real(t_real)   :: dum,dum2(2),dum22(2,2),vex0(2),eps0,trho,yred
    Real(t_real)   :: dum22o1(2,2),dum22o2(2,2),dum22o3(2,2)
    Real(t_real)   :: dum1,dum3
    Real(t_real)   :: gradrhos0(2),laprhos0(2),mixrhos0(2)
    !!
    dored=((Abs(xreduce-1.0_t_real)>1.0e-6_t_real).and.(mspin==2))
    yred=1.0_t_real-xreduce
    !!
    If(rho(1_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    If(rho(2_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    !!
    Select Case (Int(xcversion))
    Case (Int(XCVER_PERDEWZUNGER))
       !!
       Call vlsdapz(rho(1),rho(2),vex(1),vex(2),eps,mspin)
       !!
    Case (Int(xcver_perdewwang92))
       !!
       If(dored)Then
          trho=(rho(1)+rho(2))*0.5_t_real
          Call klsdapw92(trho,trho,dum1,dum,dum3,dum22&
               &,vex0(1),vex0(2),eps0,mspin,second_derive=.False.)
          Call klsdapw92(rho(1),rho(2),dum1,dum,dum3,dum22&
               &,vex(1),vex(2),eps,mspin,second_derive=.False.)
          vex(:)=xreduce*vex(:)+yred*vex0(:)
          eps   =xreduce*eps   +yred*eps0
       Else
          Call klsdapw92(rho(1),rho(2),dum1,dum,dum3,dum22&
               &,vex(1),vex(2),eps,mspin,second_derive=.False.)
       Endif
       !!
    Case (Int(XCVER_PBE96))
       !!
       If(dored)Then
          trho=(rho(1)+rho(2))*0.5_t_real
          gradrhos0=gradrho*0.5_t_real
          laprhos0=Sum(laprhos(1:2))*0.5_t_real
          mixrhos0=mixrho/8.0_t_real
          Call kggapbe96(trho,trho,gradrhos0,gradrho,laprhos0&
               &,mixrhos0,mixrho,dum22,dum22o1,dum2,dum22o2,dum22o3,dum&
               &,vex0(1),vex0(2),eps0,mspin,second_derive=.False.)
          Call kggapbe96(rho(1),rho(2),gradrhos,gradrho,laprhos&
               &,mixrhos,mixrho,dum22,dum22o1,dum2,dum22o2,dum22o3,dum&
               &,vex(1),vex(2),eps,mspin,second_derive=.False.)
          vex(:)=xreduce*vex(:)+yred*vex0(:)
          eps   =xreduce*eps   +yred*eps0
       Else
          Call kggapbe96(rho(1),rho(2),gradrhos,gradrho,laprhos&
               &,mixrhos,mixrho,dum22,dum22o1,dum2,dum22o2,dum22o3,dum&
               &,vex(1),vex(2),eps,mspin,second_derive=.False.)
       Endif
       !!
    Case (Int(xcver_vbarthhedin))
       !!
       Call vlsdabh(rho(1),rho(2),vex(1),vex(2),eps,mspin)
       !!
    Case (Int(xcver_xonly))
       !!
       Call kxonly(rho(1),rho(2),dum1,dum,dum3,dum22&
            &,vex(1),vex(2),eps,mspin,second_derive=.False.)
       !!
    Case default
       !!
       Write(errbuf,"('''Xc-version ',i4,' is not supported!''')")xcversion
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
       !!
    End Select
    !!
  End Subroutine vxcrho
  !!
  !!***********************************************************************
  !!info
  !!info calculate the exchange kernals, potentials and energy of density rho
  !!info
  !!info if mspin.eq.1 then total density is in rho(1)
  !!info and  vex(1)=vex(2)
  !!info
  Subroutine kxcrho(mspin,rho,K,Q,P,vex,eps,xreduce)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) :: rho(2)
    Real(t_real),Intent(in) :: xreduce !!  reduce xc-field B -> xreduce*B
    Real(t_real),Intent(out):: vex(2),eps,K,Q,P
    !!LOCAL
    Character(*),Parameter::nsubfunc="kxcrho"
    Logical(t_log) :: dored
    Real(t_real)   :: dvex(2,2),trho,K0,Q0,P0,yred,vex0(2),eps0

    !!
    dored=((Abs(xreduce-1.0_t_real)>1.0e-6_t_real).and.(mspin==2))
    yred=1.0_t_real-xreduce
    !!
    If(rho(1_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    If(rho(2_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    !!
    If(mspin==2 .And. Abs(rho(1)-rho(2))==Abs(rho(1)+rho(2)) ) Then
!    If(mspin==2 .And. Abs(rho(1)-rho(2))==Abs(rho(1)+rho(2)).And.&
!         &rho(1)/=0.0_t_real.And.rho(2)/=0.0_t_real ) Then
!todo: only if both /= 0
!       Write(outfile,"('rho=',2(e30.15e3,1x))")rho(1)-rho(2),rho(1)+rho(2)
       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg="' fully polarized density -> kernel diverges !'")
    Endif
    
    Select Case (Int(xcversion))
       
    Case (Int(xcver_perdewwang92))
       !!
       If(dored)Then
          trho=(rho(1)+rho(2))*0.5_t_real
          Call klsdapw92(trho,trho,K0,Q0,P0,dvex&
               &,vex0(1),vex0(2),eps0,mspin,second_derive=.True.)
          Call klsdapw92(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
               &,second_derive=.True.)
          K=xreduce*K+yred*K0
          Q=xreduce*Q+yred*Q0
          P=xreduce*P+yred*P0
          vex(:)=xreduce*vex(:)+yred*vex0(:)
          eps   =xreduce*eps   +yred*eps0

       Else
          Call klsdapw92(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
               &,second_derive=.True.)
       Endif
       !!
    Case (Int(xcver_xonly))
       !!
       Call kxonly(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
            &,second_derive=.True.)
       !!
    Case default
       !!
       Write(errbuf,"('''Xc-version ',i4,' is not supported!''')")xcversion
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
       !!
    End Select
    !!
  End Subroutine kxcrho
  !!***********************************************************************
  !!info similar to kxcrho but output for dV/dn*(grad n)
  !!info calculate the exchange kernals, potentials and energy of density rho
  !!info
  !!info if mspin.eq.1 then total density is in rho(1)
  !!info and  vex(1)=vex(2)
  !!info and  dvex(1,1)=P dvex(else,else)=0
  Subroutine derivxcrho(mspin,rho,gradrhos,gradrho,laprhos,mixrhos,mixrho&
       &,dvex,dvexdabsgrads,dvexdabsgrad,dvexdlaprho&
       &,dvexdmixrhos,dvexdmixrho,vex,eps,xreduce)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) :: rho(2)
    Real(t_real),Intent(in) :: gradrhos(2),gradrho,laprhos(2),mixrhos(2)
    Real(t_real),Intent(in) :: mixrho
    Real(t_real),Intent(in) :: xreduce !!  reduce xc-field B -> xreduce*B
    Real(t_real),Intent(out):: vex(2),dvex(2,2),eps
    Real(t_real),Intent(out):: dvexdabsgrads(2,2),dvexdlaprho(2,2)
    Real(t_real),Intent(out):: dvexdmixrhos(2,2),dvexdabsgrad(2)
    Real(t_real),Intent(out):: dvexdmixrho
    !!LOCAL
    Character(*),Parameter::nsubfunc="derivxcrho"
    Logical(t_log) :: dored
    Real(t_real)   :: K,Q,P,yred,trho,dvex0(2,2),vex0(2),eps0,gradrhos0(2)
    Real(t_real)   :: laprhos0(2),mixrhos0(2),dvexdabsgrads0(2,2)
    Real(t_real)   :: dvexdabsgrad0(2),dvexdlaprho0(2,2),dvexdmixrhos0(2,2)
    Real(t_real)   :: dvexdmixrho0
    !!
    dored=((Abs(xreduce-1.0_t_real)>1.0e-6_t_real).and.(mspin==2))
    yred=1.0_t_real-xreduce
    dvex(1:2,1:2)=0.0_t_real
    dvexdabsgrads=0.0_t_real
    dvexdabsgrad=0.0_t_real
    dvexdlaprho=0.0_t_real
    dvexdmixrhos=0.0_t_real
    dvexdmixrho=0.0_t_real
    !!
    If(rho(1_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    If(rho(2_t_int)<0.0_t_real) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'density negativ!'")
    Endif
    !!
!!$    If(mspin==2 .And. Abs(rho(1)-rho(2))==Abs(rho(1)+rho(2)) ) Then
!!$       rho(2)=0.0_t_real
!!$       rho(1)=1.0e-15_t_real
!!$       Write(outfile,"('rho=',2(e30.15e3,1x))")rho(1:2)
!!$       Call errmsg(nmodule,nsubfunc,E_warning&
!!$            &,msg="' fully polarized density -> kernel diverges !'")
!!$    Endif
    
    Select Case (Int(xcversion))
       
    Case (Int(xcver_perdewwang92))
       !!
       If(dored)Then
          trho=(rho(1)+rho(2))*0.5_t_real
          Call klsdapw92(trho,trho,K,Q,P,dvex0&
               &,vex0(1),vex0(2),eps0,mspin,second_derive=.True.)
          Call klsdapw92(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
               &,second_derive=.True.)
          dvex(:,:)=xreduce*dvex(:,:)+yred*dvex0(:,:)
          vex(:)   =xreduce*vex(:)   +yred*vex0(:)
          eps      =xreduce*eps      +yred*eps0
       Else
          Call klsdapw92(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
               &,second_derive=.True.)
       Endif
       !!
    Case (Int(XCVER_PBE96))
       !!
       If(dored)Then
          trho=(rho(1)+rho(2))*0.5_t_real
          gradrhos0=gradrho*0.5_t_real
          laprhos0=Sum(laprhos(1:2))*0.5_t_real
          mixrhos0=mixrho/8.0_t_real
          Call kggapbe96(trho,trho,gradrhos0,gradrho,laprhos0&
               &,mixrhos0,mixrho&
               &,dvex0,dvexdabsgrads0,dvexdabsgrad0,dvexdlaprho0&
               &,dvexdmixrhos0,dvexdmixrho0&
               &,vex0(1),vex0(2),eps0,mspin,second_derive=.True.)
          Call kggapbe96(rho(1),rho(2),gradrhos,gradrho,laprhos&
               &,mixrhos,mixrho&
               &,dvex,dvexdabsgrads,dvexdabsgrad,dvexdlaprho&
               &,dvexdmixrhos,dvexdmixrho&
               &,vex(1),vex(2),eps,mspin,second_derive=.True.)
          dvex(:,:)=xreduce*dvex(:,:)+yred*dvex0(:,:)
          dvexdabsgrads(:,:)=xreduce*dvexdabsgrads(:,:)+yred*dvexdabsgrads0(:,:)
          dvexdabsgrad(:)=xreduce*dvexdabsgrad(:)+yred*dvexdabsgrad0(:)
          dvexdlaprho(:,:)=xreduce*dvexdlaprho(:,:)+yred*dvexdlaprho0(:,:)
          dvexdmixrhos(:,:)=xreduce*dvexdmixrhos(:,:)+yred*dvexdmixrhos0(:,:)
          dvexdmixrho=xreduce*dvexdmixrho+yred*dvexdmixrho0
          vex(:)=xreduce*vex(:)+yred*vex0(:)
          eps   =xreduce*eps   +yred*eps0

       Else
          Call kggapbe96(rho(1),rho(2),gradrhos,gradrho,laprhos&
               &,mixrhos,mixrho&
               &,dvex,dvexdabsgrads,dvexdabsgrad,dvexdlaprho&
               &,dvexdmixrhos,dvexdmixrho&
               &,vex(1),vex(2),eps,mspin,second_derive=.True.)
       Endif
       !!
    Case (Int(xcver_xonly))
       !!
       Call kxonly(rho(1),rho(2),K,Q,P,dvex,vex(1),vex(2),eps,mspin&
            &,second_derive=.True.)
       !!
    Case default
       !!
       Write(errbuf,"('''Xc-version ',i4,' is not supported!''')")xcversion
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
       !!
    End Select
    !!
    !!
    Return
    !!
  End Subroutine derivxcrho
  !!
  !!***********************************************************************
  !!
  !!     adjust negative densities
  !!     this may happen while the iteration process is still incomplete
  !!     especially if CPA or enpty spheres are involved
  !!     if |rho*factor| is larger than tol, set error and set alertflag
  !!
  !!
  Subroutine  adjustrho(rho,factor,maxerror,mspin,tol,alert)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in)   :: mspin
    Logical(t_log),Intent(out)  ::  alert
    Real(t_real),Intent(in)   :: factor,tol
    Real(t_real),Intent(inout):: rho(2),maxerror
    !!LOCAL
    !Character(*),Parameter::nsubfunc="adjustrho"
    !!
    If(rho(1_t_int)<0.0_t_real) Then
       maxerror=Max(maxerror,Abs(rho(1_t_int)*factor))
       If(Abs(factor*rho(1_t_int))>tol)Then
          alert=.True.
       Endif
       rho(1_t_int)=0.0_t_real
    Endif
    If(mspin == 2_t_int)Then
       If(rho(2_t_int)<0.0_t_real) Then
          maxerror=Max(maxerror,Abs(rho(2_t_int)*factor))
          If(Abs(factor*rho(2_t_int))>tol)Then
             alert=.True.
          Endif
          rho(2_t_int)=0.0_t_real
       Endif
    Endif
    !!
  End Subroutine adjustrho
  !!*********************************************************************
  !!     v.Barth-Hedin 1972 
  !!
  !!     vlsdabh (rho  (r), rho    (r))
  !!               up        down
  !!
  !!     exchange and correlation potential of a homogeneous electron
  !!     liquid at density rho
  !!
  !! based on code by Lutz Steinbeck
  !! corrections by Michael Kuzmin
  !!*********************************************************************
  Subroutine vlsdabh(rhoup,rhodown,vup,vdn,eps,mspin)
    !!
    Implicit None
    !!
    !!IO
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) :: rhoup,rhodown
    Real(t_real),Intent(out):: vup,vdn,eps
    !!LOCAL
    Character(*),Parameter::nsubfunc="vlsdabh"
    Real(t_real)   :: z,z2,z4,fz,dfz
    Real(t_real)   :: rho,rho1o3,rhoup1o3,rhodn1o3,rs,ex,kf,dum
    Real(t_real)   :: vc,bc,dum1,dum2,dum3,dum4
    Real(t_real)   :: epscf,vcf,epscp,vcp,x
    Real(t_real),Parameter :: c1o4pio3p1o3& ! 1/(4pi/3)**(1/3)
         &=0.6203504908994000166680068120477781673507_t_real
    Real(t_real),Parameter :: c3pip2p1o3&  !! (3*pi**2)**(1/3)
         &=3.0936677262801359309689452072223869579540_t_real
    Real(t_real),Parameter :: c6pip2p1o3&  !! (6*pi**2)**(1/3)
         &=3.897777089720753958963470917794_t_real

    Real(t_real),Parameter ::c0 =  0.0252_t_real
    Real(t_real),Parameter ::r0 = 30.0000_t_real
    Real(t_real),Parameter ::c1 =  0.0127_t_real
    Real(t_real),Parameter ::r1 = 75.0000_t_real

    !!
    !!***********************************************************************

    If(xcrelat)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Option xcrelat not longer supported!'")

    !!     total density n
    rho=rhoup+rhodown
    vup=0.0_t_real
    vdn=0.0_t_real
    eps=0.0_t_real
    If(rho<=1.0e-15_t_real) Return
    rho1o3=rho**c1o3
    If(mspin==2)Then
       rhoup1o3=rhoup**c1o3
       rhodn1o3=rhodown**c1o3
    Endif
    rs=c1o4pio3p1o3 /rho1o3 !! 1/n=(4 pi/3 rs^3)


    If(mspin == 2_t_int) Then

       Call spin_f_function(rhoup,rhodown,rho,rhoup1o3,rhodn1o3,rho1o3&
            &,z,z2,z4,fz,dfz,.False.,dum,.False.,dum1,dum2,dum3)

       kf=c6pip2p1o3*rhoup1o3
       Call exchange_pw92_pbe96(kf,rhoup,dum,dum,dum&
            &,ex,vup,dum1,dum2,dum3,dum4,.False.,second_derive=.False.)
       eps=ex*rhoup/rho
       kf=c6pip2p1o3*rhodn1o3
       Call exchange_pw92_pbe96(kf,rhodown,dum,dum,dum&
            &,ex,vdn,dum1,dum2,dum3,dum4,.False.,second_derive=.False.)
       eps=eps+ex*rhodown/rho

       x=rs/r0
       vcp=slog(1.0_t_real/x)
       epscp=-c0*((1.0_t_real+x**3_t_int)*vcp-x*x+x*0.5_t_real-c1o3)
       vcp=-c0*vcp

       x=rs/r1
       vcf=slog(1.0_t_real/x) 
       epscf=-c1*((1.0_t_real+x**3_t_int)*vcf-x*x+x*0.5_t_real-c1o3)-epscp
       vcf=-c1*vcf -vcp

       bc=epscf*dfz
       vc=vcp+vcf*fz-bc*z
       vup=vup+vc +bc
       vdn=vdn+vc -bc
       
       eps=eps+epscp+epscf*fz
       Return
    Else

       kf=c3pip2p1o3*rho1o3
       Call exchange_pw92_pbe96(kf,rho,dum,dum,dum&
            &,eps,vup,dum1,dum2,dum3,dum4,.False.,second_derive=.false.)

       x=rs/r0
       vcp=slog(1.0_t_real/x)
       eps=eps-c0*((1.0_t_real+x**3)*vcp-x*x+x*0.5_t_real-c1o3)
       vup=vup-c0*vcp
       vdn=vup
       return

    Endif
    !!
    Return
    !!
  End Subroutine vlsdabh
  !!*********************************************************************
  !! LSDA: Perdew-Zunger (Ceperley-Alder) 1981 :
  !!
  !!     vlsdapz (rho  (r), rho    (r))
  !!               up        down
  !!
  !!     exchange and correlation potential of a homogeneous electron
  !!     liquid at density rho
  !!
  !! based on code by Lutz Steinbeck
  !! corrections by Michael Kuzmin
  !!
  !!*********************************************************************
  Subroutine vlsdapz(rhoup,rhodown,vup,vdn,eps,mspin)
    !!
    Implicit None
    !!
    !!IO
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in)  :: rhoup,rhodown
    Real(t_real),Intent(out) :: vup,vdn,eps
    !!LOCAL
    Character(*),Parameter::nsubfunc="vlsdapz"
    Real(t_real)   :: z,z2,z4,fz,dfz
    Real(t_real) :: rho,rho1o3,rhoup1o3,rhodn1o3
    Real(t_real)   :: rs,kf,dum,vc,bc,ex,srs,dum1,dum2,dum3,dum4
    Real(t_real)   :: b2,lrs,vcp,vcf,epscp,epscf,p2,b3,p1
    !! Perdew Zunger 1981
    Real(t_real),Parameter ::gamp_pz=-0.1423_t_real
    Real(t_real),Parameter ::b1p_pz= 1.0529_t_real
    Real(t_real),Parameter ::b2p_pz= 0.3334_t_real
    Real(t_real),Parameter ::aap_pz= 0.0311_t_real
    Real(t_real),Parameter ::bbp_pz=-0.0480_t_real
    Real(t_real),Parameter ::ccp_pz= 0.0020_t_real
    Real(t_real),Parameter ::ddp_pz=-0.0116_t_real
    !!
    Real(t_real),Parameter ::gamf_pz=-0.0843_t_real
    Real(t_real),Parameter ::b1f_pz= 1.3981_t_real
    Real(t_real),Parameter ::b2f_pz= 0.2611_t_real
    ! TODO: check parameter aaf
    Real(t_real),Parameter ::aaf_pz= 0.01555_t_real
    Real(t_real),Parameter ::bbf_pz=-0.0269_t_real
    Real(t_real),Parameter ::ccf_pz= 0.0007_t_real
    Real(t_real),Parameter ::ddf_pz=-0.0048_t_real
    Real(t_real),Parameter :: c6pip2p1o3&  !! (6*pi**2)**(1/3)
         &=3.897777089720753958963470917794_t_real
    Real(t_real),Parameter :: c1o4pio3p1o3& ! 1/(4pi/3)**(1/3)
         &=0.6203504908994000166680068120477781673507_t_real
    Real(t_real),Parameter :: c3pip2p1o3&  !! (3*pi**2)**(1/3)
         &=3.0936677262801359309689452072223869579540_t_real
    Real(t_real),Parameter :: c7o6&
         &=1.1666666666666666666666666666666666666666_t_real
    !!
    !!***********************************************************************
    !!     total density n

    If(xcrelat)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Option xcrelat not longer supported!'")

    rho=rhoup+rhodown
    vup=0.0_t_real
    vdn=0.0_t_real
    eps=0.0_t_real
    If(rho<=0.0_t_real) Return
    rho1o3=rho**c1o3
    If(mspin==2)Then
       rhoup1o3=rhoup**c1o3
       rhodn1o3=rhodown**c1o3
    Endif

    rs=c1o4pio3p1o3 /rho1o3 !! 1/n=(4 pi/3 rs^3)



    If(mspin == 2_t_int) Then

       Call spin_f_function(rhoup,rhodown,rho,rhoup1o3,rhodn1o3,rho1o3&
            &,z,z2,z4,fz,dfz,.False.,dum,.False.,dum1,dum2,dum3)
       
       kf=c6pip2p1o3*rhoup1o3
       Call exchange_pw92_pbe96(kf,rhoup,dum,dum,dum&
            &,ex,vup,dum1,dum2,dum3,dum4,.False.,second_derive=.False.)
       eps=ex*rhoup/rho
       kf=c6pip2p1o3*rhodn1o3
       Call exchange_pw92_pbe96(kf,rhodown,dum,dum,dum&
            &,ex,vdn,dum1,dum2,dum3,dum4,.False.,second_derive=.False.)
       eps=eps+ex*rhodown/rho
       




       If (rs>=1.0_t_real) Then
          srs=Sqrt(rs)
          p1=b1p_pz*srs
          b3=b2p_pz*rs
          p2=1.0_t_real+b3+p1
          epscp =gamp_pz/p2
          vcp   =epscp*(1.0_t_real+c7o6*p1+c4o3*b3)/p2
          p1=b1f_pz*srs
          b3=b2f_pz*rs
          p2=1.0_t_real+b3+p1
          epscf =gamf_pz/p2
          vcf   =epscf*(1.0_t_real+c7o6*p1+c4o3*b3)/p2
          epscf=epscf-epscp


          bc=epscf*dfz
          vc=(vcf-vcp)*fz+vcp-bc*z
          vdn=vdn+vc-bc
          vup=vup+vc+bc

          eps=eps+epscp +epscf*fz
          Return

       Else
          lrs=Log(rs)
          b2    =aap_pz*lrs+bbp_pz
          epscp =(ccp_pz*lrs+ddp_pz)*rs
          epscf =(ccf_pz*lrs+ddf_pz)*rs-epscp
          b3    =aaf_pz*lrs+bbf_pz-b2
          bc=(epscf+b3)*dfz

          vcp   =c1o3*(2.0_t_real*epscp-rs*ccp_pz-aap_pz)
          vcf   =c1o3*(2.0_t_real*epscf+rs*(ccp_pz-ccf_pz)+aap_pz-aaf_pz)


          vc=(b3+vcf)*fz+vcp+b2-bc*z
          vdn=vdn+vc-bc
          vup=vup+vc+bc

          eps=eps+epscp+b2 +(epscf+b3)*fz
          Return

       Endif
       
       
    Else
       
       
       kf=c3pip2p1o3*rho1o3
       Call exchange_pw92_pbe96(kf,rho,dum,dum,dum&
            &,eps,vup,dum1,dum2,dum3,dum4,.False.,second_derive=.False.)

       If (rs>=1.0_t_real) Then
          p1=b1p_pz*Sqrt(rs)
          b3=b2p_pz*rs
          p2=1.0_t_real+b3+p1
          epscp =gamp_pz/p2
          vup   =vup+epscp*(1.0_t_real+c7o6*p1+c4o3*b3)/p2
          eps   =eps+epscp
          vdn=vup
          Return
       Else
          lrs=Log(rs)
          p1     =(ccp_pz*lrs+ddp_pz)*rs
          epscp  =aap_pz*lrs+bbp_pz
          vup    =vup+epscp+c1o3*(2.0_t_real*p1-rs*ccp_pz-aap_pz)
          eps    =eps+epscp+p1
          vdn=vup
          Return
       Endif

    End If

  End Subroutine vlsdapz
  !!***********************************************************************
  !!          Exc of Perdew & Wang, Phys.Rev.B 45, 13244 (1992)
  !!*******************************************************************
  Subroutine klsdapw92(rhoup,rhodown,K,Q,P,dvex,vup,vdn,eps,mspin,second_derive)
    !!
    !!
    !! Note that K,Q,P diverge for zeta=+/-rho.
    !! In order to get through, when used in products with the gradients
    !! of the density we set the divergent terms to zero, since they
    !! get screened by the vanishing gradient.
    !! The gradient of the densities vanishes since densities are 
    !! non-negative and should be differentiable away frm the nucleus.
    !!
    !!
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) ::rhoup,rhodown
    Real(t_real),Intent(out):: K,Q,P,vup,vdn,eps,dvex(2,2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="klsdapw92"
    Real(t_real) ::  z,z2,z4,fz,dfz,d2fz
    Real(t_real) ::  rho,rho1o3,rhoup1o3,rhodn1o3
    Real(t_real)   :: Kec,Qec,Pec,PKx,Qx
    Real(t_real)   :: rs,kf
    Real(t_real)   :: ec,decr,decz,d2ecr,d2ecrz,d2eczz,vc
    Real(t_real)   :: dum,ex,dum1,dum2,dum3
    Real(t_real),Parameter :: c6pip2p1o3&  !! (6*pi**2)**(1/3)
         &=3.897777089720753958963470917794_t_real
    Real(t_real),Parameter :: c1o4pio3p1o3& ! 1/(4pi/3)**(1/3)
         &=0.6203504908994000166680068120477781673507_t_real
    Real(t_real),Parameter :: c3pip2p1o3&  !! (3*pi**2)**(1/3)
         &=3.0936677262801359309689452072223869579540_t_real
    !!
    If(xcrelat)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Option xcrelat not longer supported!'")
    
    vup = 0.0_t_real
    vdn = 0.0_t_real
    eps = 0.0_t_real
    dvex=0.0_t_real
    P = 0.0_t_real
    Q = 0.0_t_real
    K = 0.0_t_real
    rho = rhoup + rhodown
    If (rho<=0.0_t_real) Return
    rho1o3=rho**c1o3
    If(mspin==2)Then
       rhoup1o3=rhoup**c1o3
       rhodn1o3=rhodown**c1o3
    Endif
    rs =c1o4pio3p1o3/rho1o3
    

    If (mspin==1_t_int) Then


       kf=c3pip2p1o3*rho1o3
       Call exchange_pw92_pbe96(kf,rho,dum,dum,dum&
            &,eps,vup,dvex(1,1),dum1,dum2,dum3,.False.,second_derive)

       Call corel_pw92(mspin,rs,z,z2,z4,fz,dfz,d2fz&
            &,ec,decr,decz,d2ecr,d2ecrz,d2eczz,vc,second_derive)

       eps = eps+ec
       vup = vup+vc
       vdn=vup
       
       !!
       If(second_derive)Then
          PKx=dvex(1,1)
          K=d2eczz/rho
          P=(d2ecr-decr)/rho  + PKx
          !!
          dvex(1,1)=P !! there should be *2 in PKx
          !!             ,but we use only the total density
          !! in dV=dV(s)=sum_t \int dvex(s,t)*dn(t)
          !!    dV      =sum_t \int dvex(s)*dn(t)*delta(s,t) <-- dvex diagonal
          !!                                                     in LSDA
          !!    dV      =\int dvex(1)*dn/2  <-- n(s)=n/2
          !!    dV      =\int P*2*dn/2  <-- n(s)=n/2
          !!    dV      =\int P*dn  <-- n(s)=n/2
          !! and that is why we do NOT use the correct formula dvex=2*P
          !! For a similar reason Kec drops out in potential expansions
       Endif
       !!
    Else

       Call spin_f_function(rhoup,rhodown,rho,rhoup1o3,rhodn1o3,rho1o3&
            &,z,z2,z4,fz,dfz,second_derive,d2fz,.False.,dum1,dum2,dum3)
       

       kf=c6pip2p1o3*rhoup1o3
       Call exchange_pw92_pbe96(kf,rhoup,dum,dum,dum&
            &,ex,vup,dvex(1,1),dum1,dum2,dum3,.False.,second_derive)
       eps=ex*rhoup/rho

       kf=c6pip2p1o3*rhodn1o3
       Call exchange_pw92_pbe96(kf,rhodown,dum,dum,dum&
            &,ex,vdn,dvex(2,2),dum1,dum2,dum3,.False.,second_derive)
       eps=eps+ex*rhodown/rho

       Call corel_pw92(mspin,rs,z,z2,z4,fz,dfz,d2fz&
            &,ec,decr,decz,d2ecr,d2ecrz,d2eczz,vc,second_derive)
          


       vup = vup + vc +decz
       vdn = vdn + vc -decz

       eps = eps + ec

       
       !!
       !!
       If(second_derive)Then
          !!
          PKx=0.25_t_real*(dvex(1,1)+dvex(2,2))
          Qx =0.25_t_real*(dvex(1,1)-dvex(2,2))
          !!
          Kec=d2eczz/rho
          Qec=-d2ecrz/rho-z*Kec
          Pec=(d2ecr-decr)/rho-2.0_t_real*Qec*z-z2*Kec
          !!
          dvex(1,1)=dvex(1,1)+Pec+2.0_t_real*Qec+Kec
          dvex(1,2)=Pec-Kec
          dvex(2,1)=dvex(1,2)
          dvex(2,2)=dvex(2,2)+Pec-2.0_t_real*Qec+Kec
          !!
          K   = Kec + PKx
          Q   = Qec + Qx
          P   = Pec + PKx
       !!
       Endif
       !!
    Endif
    !!
    Return
    !!
  End Subroutine klsdapw92
  !!*********************************************************************
  !!
  !!     exchange  potential of a homogeneous electron liquid at density rho
  !!
  !!
  !!*********************************************************************
  Subroutine kxonly(rhoup,rhodown,K,Q,P,dvex,vup,vdn,eps,mspin,second_derive)
    !!
    !! Note that K,Q,P diverge for zeta=+/-rho.
    !! In order to get through, when used in products with the gradients
    !! of the density we set the divergent terms to zero, since they
    !! get screened by the vanishing gradient.
    !! The gradient of the densities vanishes since densities are 
    !! non-negative and should be differentiable away frm the nucleus.
    !!
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Integer(t_int),Intent(in) :: mspin
    Real(t_real),Intent(in) ::rhoup,rhodown
    Real(t_real),Intent(out):: K,Q,P,vup,vdn,eps,dvex(2,2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="kxonly"
    Real(t_real) ::  rho,rho1o3,rhoup1o3,rhodn1o3,kf,ex,dum
    Real(t_real) :: dum1,dum2,dum3
    Real(t_real),Parameter :: c6pip2p1o3&  !! (6*pi**2)**(1/3)
         &=3.897777089720753958963470917794_t_real
    Real(t_real),Parameter :: c3pip2p1o3&  !! (3*pi**2)**(1/3)
         &=3.0936677262801359309689452072223869579540_t_real
    !!
    !!
    If(xcrelat)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Option xcrelat not longer supported!'")

    
    vup = 0.0_t_real
    vdn = 0.0_t_real
    dvex=0.0_t_real
    eps = 0.0_t_real
    K = 0.0_t_real
    Q = 0.0_t_real
    P = 0.0_t_real
    rho = rhoup + rhodown
    If (rho<=0.0_t_real) Return
    rho1o3=rho**c1o3

    
    If (mspin==1_t_int) Then

       kf=c3pip2p1o3*rho1o3
       Call exchange_pw92_pbe96(kf,rho,dum,dum,dum&
            &,eps,vup,dvex(1,1),dum1,dum2,dum3,.False.,second_derive)
       
       vdn=vup
       
       If(second_derive)Then
          P=dvex(1,1)
          K=P
       Endif
       !!
    Else
       rhoup1o3=rhoup**c1o3
       rhodn1o3=rhodown**c1o3

       kf=c6pip2p1o3*rhoup1o3
       Call exchange_pw92_pbe96(kf,rhoup,dum,dum,dum&
            &,ex,vup,dvex(1,1),dum1,dum2,dum3,.False.,second_derive)
       eps=ex*rhoup/rho

       kf=c6pip2p1o3*rhodn1o3
       Call exchange_pw92_pbe96(kf,rhodown,dum,dum,dum&
            &,ex,vdn,dvex(2,2),dum1,dum2,dum3,.False.,second_derive)
       eps=eps+ex*rhodown/rho


       If(second_derive)Then
          !!
          K  =0.25_t_real*(dvex(1,1)+dvex(2,2))
          Q  =0.25_t_real*(dvex(1,1)-dvex(2,2))
          P  =K
          !!
          !!
          !!
       Endif
       !!
    Endif
    !!
    Return
    !!
  End Subroutine kxonly
  !!*******************************************************************
  !! Vxc of Perdew Burke Ernzerhof, Phys.Rev.Lett 77, 3865 (1996)
  !!*******************************************************************
  Subroutine kggapbe96(rhoup,rhodn,gradrhos,gradrho,laprhos&
       &,mixrhos,mixrho,dvex,dvexdabsgrads,dvexdabsgrad,dvexdlaprho&
       &,dvexdmixrhos,dvexdmixrho,vup,vdn,eps,mspin,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in) :: second_derive
    Integer(t_int),Intent(in) :: mspin
    !! gradrho must be the gradient of the total density if mspin==1
    !! sum(laprhos) must be the laplacian of the total density if mspin==1
    !! mixrho must be the miixed derivative of the total density if mspin==1
    !! the other gradient fields are not used if mspin==1
    Real(t_real),Intent(in)   :: rhoup,rhodn,gradrhos(2),gradrho
    Real(t_real),Intent (in)  :: laprhos(2),mixrhos(2),mixrho

    Real(t_real),Intent(out)  :: vup,vdn,eps,dvex(2,2),dvexdabsgrads(2,2)
    Real(t_real),Intent(out)  :: dvexdabsgrad(2)
    Real(t_real),Intent(out)  :: dvexdlaprho(2,2),dvexdmixrhos(2,2)
    Real(t_real),Intent(out)  :: dvexdmixrho
    !!LOCAL
    Character(*),Parameter::nsubfunc="kggapbe96"
    Real(t_real) ::  z,fz,dfz,d2fz,eclsda,rs
    Real(t_real) ::  rho,decz,z4
    Real(t_real)   :: phi,kf,ks,t,ex,laprho,bc
    Real(t_real)   :: Hc,decr,Hct,t2,dphiophi,Hctt,t4
    Real(t_real)   :: rho1o3,Hcttt
    Real(t_real)   :: dmixrho,dlaprho,grz,vc
    Real(t_real)   :: ppc,d2ecr,d2ecrz,d2eczz,kkc,qqc
    Real(t_real)   :: Hca,dAr,dAz,ksrho,dArr,Hcat,Hcaa,dArz,z2
    Real(t_real)   :: Hcatt,Hcaat,d2phiophi,ttc,qhctt,ahctt,ahcat,ahct,qhcttz
    Real(t_real)   :: Hz,Hzz,Haz,Htz,a,dAzz,Httz,Hatz,Htzz,dgrzrho,ahct1
    Real(t_real)   :: rhoup1o3,rhodn1o3,rrc,qqcgga,ppcgga,kkcgga,ahcr
    Real(t_real)   :: ahcz,ksrho2,ksrho4,vclsda,k4httz
    Real(t_real),Parameter :: tol=1.0e-25_t_real
    !!
    Real(t_real),Parameter :: c6pip2p1o3&  !! (6*pi**2)**(1/3)
         &=3.897777089720753958963470917794_t_real
    !Real(t_real),Parameter :: c4o9&
    !     &=0.4444444444444444444444444444444444444444_t_real
    Real(t_real),Parameter :: c1o7&
         &=0.1428571428571428571428571428571428571428_t_real
    Real(t_real),Parameter :: c7o3&
         &=2.3333333333333333333333333333333333333333_t_real
    Real(t_real),Parameter :: c3pip2p1o3&  !! (3*pi**2)**(1/3)
         &=3.0936677262801359309689452072223869579540_t_real
    Real(t_real),Parameter :: sq4opitc3pip2p1o3&
         &=1.9846863952198559283386322800901461686623_t_real
    Real(t_real),Parameter :: c1o4pio3p1o3& ! 1/(4pi/3)**(1/3)
         &=0.6203504908994000166680068120477781673507_t_real
    Real(t_real),Parameter :: c7o6&
         &=1.1666666666666666666666666666666666666666_t_real
    !!
    If(xcrelat)&
         & Call errmsg(nmodule,nsubfunc,E_fatalerror&
         &,msg="'Option xcrelat not longer supported!'")
    !!
    vup = 0.0_t_real
    vdn = 0.0_t_real
    eps = 0.0_t_real
    If(second_derive)Then
       dvex=0.0_t_real
       dvexdabsgrads=0.0_t_real
       dvexdabsgrad=0.0_t_real
       dvexdlaprho=0.0_t_real
       dvexdmixrhos=0.0_t_real
       dvexdmixrho=0.0_t_real
    Endif
    rho = rhoup + rhodn
    If (rho<=1.0e-25_t_real) Return
    rho1o3=rho**c1o3
    If(mspin==2)Then
       rhoup1o3=rhoup**c1o3
       rhodn1o3=rhodn**c1o3
    Endif
    rs =c1o4pio3p1o3/rho1o3
    
    laprho=laprhos(1)+laprhos(2)


    If (mspin==1_t_int) Then

       
       !! exchange
       kf=c3pip2p1o3*rho1o3
       Call exchange_pw92_pbe96(kf,rho,gradrho,laprho,mixrho&
            &,eps,vup&
            &,dvex(1,1),dvexdabsgrad(1),dvexdlaprho(1,1),dvexdmixrho&
            &,.True.,second_derive)
       
       !! correlation

       Call corel_pw92(mspin,rs,z,z2,z4,fz,dfz,d2fz&
            &,eclsda,decr,decz,d2ecr,d2ecrz,d2eczz,vclsda,second_derive)


       ks=sq4opitc3pip2p1o3*Sqrt(rho1o3)
       ksrho=1.0_t_real/(2.0_t_real*ks*rho)
       ksrho2=ksrho*ksrho
       ksrho4=ksrho2*ksrho2
       t=gradrho*ksrho
       t2=t*t
       t4=t2*t2

       Call corel_func_pbe96(eclsda,decr,decz,d2ecr,d2ecrz,d2eczz&
            &,1.0_t_real,dphiophi,d2phiophi&
            &,t2,t4&
            &,Hc,Hct,Hca,Hctt,Hcat,Hcaa,Hcttt,Hcatt,Hcaat&
            &,dAr,dAz,dArr,dArz,dAzz,second_derive)


       eps = eps+eclsda+Hc

       dlaprho=-rho*ksrho2*Hct
       dmixrho=-rho*ksrho4*Hctt

       ahct1=(Hct+t2*Hctt)
       ahcr=Hct-Hcat*dAr

       vup = vup+ vclsda + Hc -Hca*dAr +t2*(c7o6*ahct1-ahcr)&
            &+laprho*dlaprho +mixrho*dmixrho
       
       vdn = vup
       
       If(second_derive)Then

          ppc=(d2ecr-decr)/rho
          
          ahct=c7o6*(Hct+ahct1)
          ahctt=c7o6*(4.0_t_real*Hctt+t2*Hcttt)
          qhctt=(Hctt-ahctt-Hcatt*dAr)
          ahcat=(2.0_t_real*Hcat+t2*Hcatt)*dAr
          
          dvex(1,1)=dvex(1,1)+ppc&
               &+(&
               &(-Hca+t2*Hcat)*(dAr-dArr)&
               &  +(Hcaa-t2*Hcaat)*dAr**2&
               &  +c7o6*t2*(Hct-ahct-ahcat+t2*qhctt)&
               
               &)/rho&
               
               &-laprho*ksrho2*(ahcr-ahct)&
               &-mixrho*ksrho4*qhctt
          
          
          
          dvexdabsgrad(1)=dvexdabsgrad(1)&
               &+ksrho2*(ahcr+c1o7*ahct+t2*ahctt+ahcat)&
               &+laprho*dmixrho&
               &-mixrho*rho*ksrho2*ksrho4*Hcttt
          
          
          dvexdlaprho(1,1)=dvexdlaprho(1,1)+dlaprho !!*grad laprho
          
          
          dvexdmixrho=dvexdmixrho+dmixrho
          !!
       Endif
       !!
    Else
       
       !! exchange

       kf=c6pip2p1o3*rhoup1o3
       Call exchange_pw92_pbe96(kf,rhoup,gradrhos(1),laprhos(1),mixrhos(1)&
            &,ex,vup&
            &,dvex(1,1),dvexdabsgrads(1,1),dvexdlaprho(1,1),dvexdmixrhos(1,1)&
            &,.True.,second_derive)
       eps = eps+ex*rhoup/rho

       kf=c6pip2p1o3*rhodn1o3
       Call exchange_pw92_pbe96(kf,rhodn,gradrhos(2),laprhos(2),mixrhos(2)&
            &,ex,vdn&
            &,dvex(2,2),dvexdabsgrads(2,2),dvexdlaprho(2,2),dvexdmixrhos(2,2)&
            &,.True.,second_derive)
       eps = eps+ex*rhodn/rho


       !! correlation

       Call spin_f_function(rhoup,rhodn,rho,rhoup1o3,rhodn1o3,rho1o3&
            &,z,z2,z4,fz,dfz,second_derive,d2fz,.True.&
            &,phi,dphiophi,d2phiophi)

       Call corel_pw92(mspin,rs,z,z2,z4,fz,dfz,d2fz&
            &,eclsda,decr,decz,d2ecr,d2ecrz,d2eczz,vclsda,second_derive)


       ks=sq4opitc3pip2p1o3*Sqrt(rho1o3)
       ksrho=1.0_t_real/(2.0_t_real*ks*phi*rho)
       ksrho2=ksrho*ksrho
       ksrho4=ksrho2*ksrho2
       t=gradrho*ksrho
       t2=t*t
       t4=t2*t2
       
       Call corel_func_pbe96(eclsda,decr,decz,d2ecr,d2ecrz,d2eczz&
            &,phi,dphiophi,d2phiophi&
            &,t2,t4&
            &,Hc,Hct,Hca,Hctt,Hcat,Hcaa,Hcttt,Hcatt,Hcaat&
            &,dAr,dAz,dArr,dArz,dAzz,second_derive)

       ahcr=Hct-Hcat*dAr    
       ahct1=(Hct+t2*Hctt)
       ahcz=(3.0_t_real*Hc-t2*Hct)
       
       Hz    =dphiophi*ahcz+Hca*dAz
       Htz=dphiophi*(2.0_t_real*Hct-ahct1)+Hcat*dAz


       eps = eps+eclsda+Hc


       !!
       
       grz=(gradrhos(1)**2-gradrhos(2)**2-z*gradrho**2)

       !!

       dlaprho=-rho*ksrho2*Hct
       dmixrho=-rho*ksrho4*Hctt
       dgrzrho=-ksrho2*Htz


       
       vc= vclsda +Hc-Hca*dAr+t2*(c7o6*ahct1-ahcr) -z*Hz &
            &+laprho*dlaprho +mixrho*dmixrho +grz*dgrzrho 
            
       !!
       bc=decz+Hz
       !!
       vup = vup +vc+bc
       vdn = vdn +vc-bc
       !!

       If(second_derive .And. rho>tol)Then


          !!lsda
          kkc=d2eczz/rho
          qqc=-d2ecrz/rho-z*kkc
          ppc=-(decr-d2ecr)/rho-2.0_t_real*qqc*z-z2*kkc
          
          !! gga

          Haz=dphiophi*(3.0_t_real*Hca-t2*Hcat)+Hcaa*dAz
          Httz=dphiophi*(-Hctt-t2*Hcttt)+Hcatt*dAz
          Hatz=dphiophi*(Hcat-t2*Hcatt)+Hcaat*dAz
          Hzz=(d2phiophi*ahcz&
               &+dphiophi*(3.0_t_real*Hz-t2*Htz)&
               &+Haz*dAz+Hca*dAzz)
          
          Htzz=d2phiophi*(2.0_t_real*Hct-ahct1)&
               &+dphiophi*(Htz-t2*Httz)&
               &+Hatz*dAz+Hcat*dAzz
          

          k4httz=-ksrho4*Httz


          ahct=c7o6*(Hct+ahct1)
          ahctt=c7o6*(4.0_t_real*Hctt+t2*Hcttt)
          ahcat=(2.0_t_real*Hcat+t2*Hcatt)*dAr
          qhctt=(Hctt-ahctt-Hcatt*dAr)
          qhcttz=(c7o6*t2*Httz+Hatz*dAr+Hcat*dArz)

          ttc=&
               &+laprho*dgrzrho&
               &+mixrho*k4httz&
               &-(grz*ksrho2*Htzz-t2*qhcttz)/rho


          rrc=-c7o3*t2*Htz/rho-ttc
          kkcgga=Hzz/rho
          qqcgga=ttc*0.5_t_real-(Haz*dAr+Hca*dArz)/rho-z*kkcgga
          ppcgga=-2.0_t_real*qqcgga*z-z2*kkcgga&
               &+(&
               &  +(-Hca+t2*Hcat)*(dAr-dArr)&
               &  +(Hcaa-t2*Hcaat)*dAr**2&
               &  +t2*c7o6*(Hct-ahct-ahcat+t2*qhctt)&
               &)/rho&

               &-laprho*ksrho2*(ahcr-ahct)&
               &-mixrho*ksrho4*qhctt&
               &+grz*ksrho2*(c7o3*Htz+qhcttz)/rho

          qqc=qqc+qqcgga
          ppc=ppc+ppcgga
          kkc=kkc+kkcgga
          
          
          dvex(1,1)=dvex(1,1)+ppc+2.0_t_real*qqc+kkc 
          dvex(2,2)=dvex(2,2)+ppc-2.0_t_real*qqc+kkc 
          dvex(2,1)=dvex(2,1)+ppc-kkc-rrc
          dvex(1,2)=dvex(1,2)+ppc-kkc+rrc


          dvexdlaprho=dvexdlaprho+dlaprho
          
          dvexdmixrho=dmixrho

          a=2.0_t_real*dgrzrho
          dvexdabsgrads(1:2,1)=dvexdabsgrads(1:2,1)+a
          dvexdabsgrads(1:2,2)=dvexdabsgrads(1:2,2)-a



          a=&
               &ksrho2*(ahcr+c1o7*ahct+t2*ahctt+ahcat) &
               &+laprho*dmixrho &
               &-mixrho*rho*ksrho2*ksrho4*Hcttt &
               &+grz*k4httz
               
               
          dvexdabsgrad(1)=a-(1.0_t_real+z)*dgrzrho
          dvexdabsgrad(2)=a+(1.0_t_real-z)*dgrzrho 

          
       Endif


    Endif
    !!
    Return
    !!
  End Subroutine kggapbe96
  !!***********************************************************************
  !!
  !! get xc description string index, check for existence
  !! must be a separate assignement, otherwise IO clash on error
  Subroutine getXCVersionIndex(index,nvers)
    !!
    Implicit None
    !!IO
    Integer(t_int),Intent(in),Optional  :: nvers ! version number of xc version
    Integer(t_int),Intent(out)          :: index ! index into desciptors
    !!LOCAL
    Character(*),Parameter::nsubfunc="getxcversionindex"
    Integer(t_int) :: i,ivers
    !! 
    !! set to global
    ivers=xcversion
    If(Present(nvers)) ivers=nvers !! or take argument
    
    !!
    index=-1_t_int
    Do i=1_t_int,Size(MAP_XC_VERSIONS)
       If(MAP_XC_VERSIONS(i)==ivers) Then
          index=i
          Exit
       End If
    Enddo
    If(index<0_t_int)Then
       Write(errbuf,"('''XC-version ',i4,' is not supported!''')")ivers
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg=Trim(errbuf))
    End If
    Return
  End Subroutine getXCVersionIndex
  !!
  !!***********************************************************************
  
  !!***********************************************************************
  Subroutine setxcrelat(rel)
    !!
    Implicit None 
    !!IO
    Logical(t_log),Intent(in) :: rel
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setxcrelat"
    !!
    xcrelat=rel
    Return
  End Subroutine setxcrelat
  
  !!***********************************************************************
  Logical(t_log) Function getxcrelat()
    !!
    Implicit None 
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="getxcrelat"
    !!
    getxcrelat=xcrelat
    Return
  End Function  getxcrelat
  
  !!***********************************************************************
  Subroutine setxcversion(nvers)
    !!
    Implicit None 
    !!IO
    Integer(t_int),Intent(in) :: nvers
    !!LOCAL
    !Character(*),Parameter::nsubfunc="setxcversion"
    Integer(t_int) ::i
    !!
    Call getXCVersionIndex(i,nvers) !! check setting
    xcversion=nvers
    Return
  End Subroutine setxcversion
  !!***********************************************************************
  Integer(t_int) Function getxcversion()
    !!
    Implicit None 
    !!IO
    !!LOCAL
    !Character(*),Parameter::nsubfunc="getxcversion"
    !!
    getxcversion=xcversion
    Return
  End Function getxcversion

  !!***********************************************************************
  Subroutine spin_f_function(rhoup,rhodn,rho,rhoup1o3,rhodn1o3,rho1o3&
       &,z,z2,z4,fz,dfz,second_derive,d2fz,gga,phi,dphiophi,d2phiophi)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive,gga
    Real(t_real),Intent(in)      :: rhoup,rhodn,rho,rhoup1o3,rhodn1o3,rho1o3
    Real(t_real),Intent(out)     :: z,z2,z4,fz,dfz,d2fz,phi,dphiophi,d2phiophi
    !!LOCAL
    !Character(*),Parameter::nsubfunc="spin_f_function"
    Real(t_real)   :: zup,zdn,o,fu1,fd1,fu4,fd4,x
    Real(t_real),Parameter :: tol=1.0e-25_t_real
    Real(t_real),Parameter :: c4o9o2p4o3m2=0.85496046708068280883_t_real
    Real(t_real),Parameter :: c2p1o3& ! 2**(1/3)
         &=1.2599210498948731647672106072782283505702_t_real
    Real(t_real),Parameter :: h=1.0e-5_t_real
    Real(t_real),Parameter :: ho=1.0e5_t_real
    !! h^(2/3)
    !-7:Real(t_real),Parameter :: h2=0.00002154434690031883721759293566519_t_real
    Real(t_real),Parameter :: h2=0.00046415888336127788924100763509194_t_real
    Real(t_real),Parameter :: c1 = 1.872427983539094650205761316872_t_real
    Real(t_real),Parameter :: c2 =-1.872427983539094650205761316872_t_real
    Real(t_real),Parameter :: c22=-3.744855967078189300411522633744_t_real
    Real(t_real),Parameter :: c3 = 1.604938271604938271604938271604_t_real
    Real(t_real),Parameter :: c33= 4.814814814814814814814814814814_t_real
    Real(t_real),Parameter :: c36= 9.629629629629629629629629629628_t_real
    Real(t_real),Parameter :: c4 =-0.748971193415637860082304526748_t_real
    Real(t_real),Parameter :: c44=-2.995884773662551440329218106992_t_real
    Real(t_real),Parameter :: c412=-8.987654320987654320987654320976_t_real
    Real(t_real),Parameter :: c5 = 0.144032921810699588477366255144_t_real
    Real(t_real),Parameter :: c55= 0.720164609053497942386831275720_t_real
    Real(t_real),Parameter :: c520=2.880658436213991769547325102880_t_real
    Real(t_real),Parameter :: c2o9=0.222222222222222222222222222222_t_real
    Real(t_real),Parameter :: c2o3&
         &=0.6666666666666666666666666666666666666666_t_real
    Real(t_real),Parameter :: c4o3o2p4o3m2&
         & =2.5648814012420484263459441643670244073078_t_real
    Real(t_real),Parameter :: c1o2p4o3m2& ! 1/(2**(4/3)-2)
         &=1.9236610509315363197594581232752683054808_t_real
    !!
    zup=Min(Max(rhoup/rho,0.0_t_real),1.0_t_real)
    zdn=Min(Max(rhodn/rho,0.0_t_real),1.0_t_real)
    !!
    z=zup-zdn
    !!
    z2=z*z
    z4=z2*z2
    !!
    zup=2.0_t_real*zup
    zdn=2.0_t_real*zdn
    !!
    o=c2p1o3/rho1o3
    fu1=rhoup1o3*o
    fd1=rhodn1o3*o
    !!
    fu4=zup*fu1
    fd4=zdn*fd1
    fz=c1o2p4o3m2*(fu4+fd4-2.0_t_real) !! zup**c4o3
    dfz=c4o3o2p4o3m2*(fu1-fd1)                 !! zup**c1o3
    !!
    If(second_derive)Then
       !! second derivative lsda
       !!
       If(zup<tol)Then
          d2fz=c4o9o2p4o3m2*(fd1/zdn)
       Else If(zdn<tol)Then
          d2fz=c4o9o2p4o3m2*(fu1/zup)
       Else
          d2fz=c4o9o2p4o3m2*(fu1/zup+fd1/zdn)  !! zup**(-c2o3)
       Endif
    Endif
    !!
    If(gga)Then
       
       If(zup<h)Then
          x=zup*ho
          phi=     x*(c1+  x*(c2+x*(c3+x*(c4+x*c5))))*h2
          dphiophi=  (c1+  x*(c22+x*(c33+x*(c44+x*c55))))*h2*ho
          If(second_derive)Then
             d2phiophi=(c22   +x*(c36+x*(c412+x*c520)))*h2*ho*ho
          Endif
       Else
          phi=fu1**2          !! zup**(c2o3)
          dphiophi=c2o3/fu1   !! zup**(-c1o3)
          If(second_derive)Then
             d2phiophi=-c2o9/fu4
          Endif
       Endif
       If(zdn<h)Then
          x=zdn*ho
          phi=phi+ x*(c1+  x*(c2+x*(c3+x*(c4+x*c5))))*h2
          dphiophi=dphiophi-(c1+x*(c22   +x*(c33+x*(c44+x*c55))))*h2*ho
          If(second_derive)Then
             d2phiophi=d2phiophi+(c22+x*(c36+x*(c412+x*c520)))*h2*ho*ho
          Endif
       Else
          phi=phi+fd1**2              !! zup**(c2o3)
          dphiophi=dphiophi-c2o3/fd1  !! zup**(-c1o3)
          If(second_derive)Then
             d2phiophi=d2phiophi-c2o9/fd4
          Endif
       Endif
       phi=phi*0.5_t_real
       dphiophi=dphiophi*0.5_t_real/phi
       d2phiophi=d2phiophi*0.5_t_real/phi-dphiophi*dphiophi
       !!
       !!
       !!
    Endif
    !!
    Return
    !!
  End Subroutine spin_f_function
  !!***********************************************************************
  Subroutine corel_func_pbe96(ec,decr,decz,d2ecr,d2ecrz,d2eczz&
       &,phi,dphiophi,d2phiophi&
       &,t2,t4&
       &,Hc,Hct,Hca,Hctt,Hcat,Hcaa,Hcttt,Hcatt,Hcaat&
       &,dAr,dAz,dArr,dArz,dAzz,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Real(t_real),Intent(in)      :: ec   !! LSDA correlation energy
    Real(t_real),Intent(in)      :: decr !! rs*d/drs ec
    Real(t_real),Intent(in)      :: decz !! d/zeta ec
    Real(t_real),Intent(in)      :: d2ecr!! rs*d/drs decr
    Real(t_real),Intent(in)      :: d2ecrz   !! rs*d/drs decz
    Real(t_real),Intent(in)      :: d2eczz   !! d/zeta decz
    Real(t_real),Intent(in)      :: phi      !! phi(zeta)
    Real(t_real),Intent(in)      :: dphiophi !! d/dzeta phi(zeta)
    Real(t_real),Intent(in)      :: d2phiophi !! phi''/phi-(phi')**2
    Real(t_real),Intent(in)      :: t2   !! reduced gradient^2
    Real(t_real),Intent(in)      :: t4   !! reduced gradient^4
    Real(t_real),Intent(out)     :: Hc   !! correlation enhancement factor
    Real(t_real),Intent(out)     :: Hct  !! 1/t d/dt Hc
    Real(t_real),Intent(out)     :: Hca  !! d/dA Hc
    Real(t_real),Intent(out)     :: Hctt !! 1/t d/dt Hct
    Real(t_real),Intent(out)     :: Hcat !! d/dA Hct
    Real(t_real),Intent(out)     :: Hcaa !! d/dA Hca
    Real(t_real),Intent(out)     :: Hcttt!! 1/t d/dt Hctt
    Real(t_real),Intent(out)     :: Hcatt!! d/dA Hctt
    Real(t_real),Intent(out)     :: Hcaat !! 1/t d/dt Hcaa
    Real(t_real),Intent(out)     :: dAr  !! rs*d/drs A/3
    Real(t_real),Intent(out)     :: dAz  !! d/zeta A
    Real(t_real),Intent(out)     :: dArr !! rs*d/drs dAr
    Real(t_real),Intent(out)     :: dArz !! d/zeta dAr
    Real(t_real),Intent(out)     :: dAzz !! d/zeta dAz
    !!LOCAL
    !Character(*),Parameter::nsubfunc="corel_func_pbe96"
    Real(t_real)   :: gp,bp,e,ee1,A,A2,x,fH,x2,bpy2,d,y,dfH,htg,htg2,dd
    Real(t_real)   :: d2fH,hag,dphi3,bpy,aa,gpo,d3fH,ddd,dez
    !!
    Real(t_real),Parameter :: beta=0.066725_t_real
    Real(t_real),Parameter :: gamma=0.031091_t_real
    Real(t_real),Parameter :: bg=beta/gamma
    !!
    gp=gamma*phi**3
    gpo=1.0_t_real/gp
    bp=bg*gp
    dphi3=3.0_t_real*dphiophi
    !!
    !!
    ee1=sexp(-ec*gpo)
    e=ee1+1.0_t_real
    !!
    A=bg/ee1
    !!
    x=A*t2
    x2=x*x
    !!
    A2=A*A/bp*e
    dAr=A2*decr    
    dez=(decz-ec*dphi3)

    dAz=A2*dez
       
    
    Call fhs(x,x2,fH,dfH,d2fH,d3fH)
    
    d=fH+dfH*x
    
    Hc=gp*slog(bg*t2*fH)
    
    y=1.0_t_real/(1.0_t_real+bg*t2*fH)

    bpy=bp*y
    bpy2=2.0_t_real*bpy
    dd=2.0_t_real*bpy2*(2.0_t_real*dfH+d2fH*x)

    Hca=bpy*t4*dfH
    
    Hct=bpy2*d
    
    hag=Hca*gpo
    htg=Hct*gpo
    htg2=htg*Hct

    Hctt=-htg2 +dd*A
    
    Hcat=0.5_t_real*dd*t2-Hct*hag

    If(second_derive)Then

       ddd=2.0_t_real*bpy2*d3fH

       aa=(e+1.0_t_real)/ee1*gpo
       
       dArr=A2*(aa*decr**2 +d2ecr)
       

       
       dArz=dAz*aa*decr-dphi3*dAr+A2*d2ecrz
       
       dAzz=dAz*(aa*dez-dphi3)&
            &+A2*(d2eczz-3.0_t_real*d2phiophi*ec-dphi3*decz)
       
       
       Hcaa=-Hca*hag+bpy*t4*t2*d2fH
       

       Hcttt=-htg*(3*Hctt+htg2)+2.0_t_real*A*A*ddd

       Hcatt=-2.0_t_real*htg*Hcat-hag*(Hctt+htg2)+dd+x*ddd

       Hcaat=-2.0_t_real*hag*Hcat-htg*(Hcaa+hag*Hca)+t4*ddd*0.5_t_real

    Endif


    !!
    Return
    !!
  Contains
    Subroutine fhs(x,x2,fH,dfH,d2fH,d3fH)
      !!
      Implicit None
      !!IO
      Real(t_real),Intent(in)      :: x,x2
      Real(t_real),Intent(out)     :: fH,dfH,d2fH,d3fH
      !!LOCAL
      !Character(*),Parameter::nsubfunc="fhs"
      Real(t_real)   :: a,y,y2
      !!
      a=1.0_t_real+x
      y=1.0_t_real/(a+x2)
      y2=y*y
      !!
      fH=  a*y
      !! first deriv
      dfH=-x*(fH+y)*y
      !!second deriv
      d2fH=2.0_t_real*(-1.0_t_real+(2.0_t_real+a)*x2)*y2*y
      !! (3.0_t_real*d2fH+x*d3fH)
      d3fH=6.0_t_real*(-1.0_t_real+x2*(6.0_t_real+4.0_t_real*x))*y2*y2
      !!
      Return
      !!
    End Subroutine fhs
    !!
  End Subroutine corel_func_pbe96
  !!***********************************************************************
  Subroutine corel_pw92_ec0(rs,srs,ec0,dec0,d2ec0,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Real(t_real),Intent(in)      :: rs   !! wigner seitz radius
    Real(t_real),Intent(in)      :: srs  !! sqrt(rs)
    Real(t_real),Intent(out)     :: ec0  !! ec(zeta=0)
    Real(t_real),Intent(out)     :: dec0 !! 1/rs*d/rs ec(zeta=0)
    Real(t_real),Intent(out)     :: d2ec0!! 1/rs*d/rs dec0
    !!LOCAL
    !Character(*),Parameter::nsubfunc="corel_pw92_ec0"
    Real(t_real)   :: ldenom,denom,den2,q0,Q1p,Q2p
    !Real(t_real),Parameter ::beta1_ec0_pw92   =7.5957_t_real
    !Real(t_real),Parameter ::beta2_ec0_pw92   =3.5876_t_real
    !Real(t_real),Parameter ::beta3_ec0_pw92   =1.6382_t_real
    !Real(t_real),Parameter ::beta4_ec0_pw92   =0.49294_t_real
    !Real(t_real),Parameter ::A_ec0_pw92       =0.031091_t_real
    !Real(t_real),Parameter ::alpha1_ec0_pw92  =0.21370_t_real
    Real(t_real),Parameter ::c2tA_ec0_pw92       =0.062182_t_real
    Real(t_real),Parameter ::c2tAtbeta1_ec0_pw92 =0.4723158174_t_real
    Real(t_real),Parameter ::cAtbeta1_ec0_pw92   =0.2361579087_t_real
    Real(t_real),Parameter ::c2tAtbeta2_ec0_pw92 =0.2230841432_t_real
    Real(t_real),Parameter ::c2tAtbeta3_ec0_pw92 =0.1018665524_t_real
    Real(t_real),Parameter ::c3tAtbeta3_ec0_pw92 =0.1527998286_t_real
    Real(t_real),Parameter ::c2tAtbeta4_ec0_pw92 =0.03065199508_t_real
    Real(t_real),Parameter ::c4tAtbeta4_ec0_pw92 =0.06130399016_t_real
    Real(t_real),Parameter ::c2tAtalpha1_ec0_pw92=0.0132882934_t_real
    !!
    Real(t_real),Parameter ::c4tAtalpha1_ec0_pw92=0.0265765868_t_real
    Real(t_real),Parameter ::cAo2tbeta1_ec0_pw92 =0.11807895435_t_real
    Real(t_real),Parameter ::c9o2tAtbeta3_ec0_pw92 =0.22919974290_t_real
    Real(t_real),Parameter ::c8tAtbeta4_ec0_pw92 =0.12260798032_t_real
    !!
    denom = (c2tAtbeta1_ec0_pw92 +c2tAtbeta3_ec0_pw92 *rs)*srs &
         & +(c2tAtbeta2_ec0_pw92 +c2tAtbeta4_ec0_pw92 *rs)*rs
    !!
    den2=1.0_t_real/(denom**2+denom)
    !!
    ldenom=slog(1.0_t_real/denom)
    !!
    q0    =-c2tA_ec0_pw92-c2tAtalpha1_ec0_pw92*rs
    !!
    ec0= q0*ldenom
    !!
    Q1p=   (cAtbeta1_ec0_pw92 +c3tAtbeta3_ec0_pw92*rs)*srs&
         &+(c2tAtbeta2_ec0_pw92+c4tAtbeta4_ec0_pw92*rs)*rs
    
    !!
    dec0=-c2tAtalpha1_ec0_pw92*rs*ldenom-q0*Q1p*den2
    
    If(second_derive)Then
       
       Q2p=   (cAo2tbeta1_ec0_pw92+c9o2tAtbeta3_ec0_pw92*rs)*srs&
            &+(c2tAtbeta2_ec0_pw92+c8tAtbeta4_ec0_pw92*rs)*rs
       
       d2ec0=&
            &(-c2tAtalpha1_ec0_pw92*rs*ldenom&
            &+c4tAtalpha1_ec0_pw92*rs*Q1p*den2&
            &-q0*Q2p*den2&
            &+q0*Q1p**2*(2.0_t_real*denom+1.0_t_real)*den2**2)
    Endif

    !!
    Return
    !!
  End Subroutine corel_pw92_ec0
  !!***********************************************************************
  Subroutine corel_pw92_ec1(rs,srs,ec1,dec1,d2ec1,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Real(t_real),Intent(in)      :: rs   !! wigner seitz radius
    Real(t_real),Intent(in)      :: srs  !! sqrt(rs)
    Real(t_real),Intent(out)     :: ec1  !! ec(zeta=1)
    Real(t_real),Intent(out)     :: dec1 !! 1/rs*d/rs ec(zeta=1)
    Real(t_real),Intent(out)     :: d2ec1!! 1/rs*d/rs dec1
    !!LOCAL
    !Character(*),Parameter::nsubfunc="corel_pw92_ec1"
    Real(t_real)   :: ldenom,denom,den2,q0,Q1p,Q2p
    !Real(t_real),Parameter ::beta1_ec1_pw92   =14.1189_t_real
    !Real(t_real),Parameter ::beta2_ec1_pw92   =6.1977_t_real
    !Real(t_real),Parameter ::beta3_ec1_pw92   =3.3662_t_real
    !Real(t_real),Parameter ::beta4_ec1_pw92   =0.62517_t_real
    !Real(t_real),Parameter ::A_ec1_pw92       =0.015545_t_real
    !Real(t_real),Parameter ::alpha1_ec1_pw92  =0.20548_t_real
    Real(t_real),Parameter ::c2tA_ec1_pw92       =0.03109_t_real
    Real(t_real),Parameter ::c2tAtbeta1_ec1_pw92 =0.438956601_t_real
    Real(t_real),Parameter ::cAtbeta1_ec1_pw92   =0.2194783005_t_real
    Real(t_real),Parameter ::c2tAtbeta2_ec1_pw92 =0.192686493_t_real
    Real(t_real),Parameter ::c2tAtbeta3_ec1_pw92 =0.104655158_t_real
    Real(t_real),Parameter ::c3tAtbeta3_ec1_pw92 =0.156982737_t_real
    Real(t_real),Parameter ::c2tAtbeta4_ec1_pw92 =0.0194365353_t_real
    Real(t_real),Parameter ::c4tAtbeta4_ec1_pw92 =0.0388730706_t_real
    Real(t_real),Parameter ::c2tAtalpha1_ec1_pw92=0.0063883732_t_real
    !!
    Real(t_real),Parameter ::cAo2tbeta1_ec1_pw92 =0.10973915025_t_real
    Real(t_real),Parameter ::c9o2tAtbeta3_ec1_pw92 =0.2354741055_t_real
    Real(t_real),Parameter ::c8tAtbeta4_ec1_pw92 =0.0777461412_t_real
    Real(t_real),Parameter ::c4tAtalpha1_ec1_pw92=0.0127767464_t_real
    !!
    denom = (c2tAtbeta1_ec1_pw92 +c2tAtbeta3_ec1_pw92 *rs)*srs &
         & +(c2tAtbeta2_ec1_pw92 +c2tAtbeta4_ec1_pw92 *rs)*rs
    !!
    den2=1.0_t_real/(denom**2+denom)
    !!
    ldenom=slog(1.0_t_real/denom)
    !!
    q0    =-c2tA_ec1_pw92-c2tAtalpha1_ec1_pw92*rs
    !!
    ec1= q0*ldenom
    !!
    Q1p=   (cAtbeta1_ec1_pw92 +c3tAtbeta3_ec1_pw92*rs)*srs&
         &+(c2tAtbeta2_ec1_pw92+c4tAtbeta4_ec1_pw92*rs)*rs
    
    !!
    dec1 = -c2tAtalpha1_ec1_pw92*rs*ldenom - q0*Q1p*den2
    
    If(second_derive)Then
       
       Q2p=   (cAo2tbeta1_ec1_pw92+c9o2tAtbeta3_ec1_pw92*rs)*srs&
            &+(c2tAtbeta2_ec1_pw92+c8tAtbeta4_ec1_pw92*rs)*rs
       
       d2ec1=&
            &(-c2tAtalpha1_ec1_pw92*rs*ldenom&
            &+c4tAtalpha1_ec1_pw92*rs*Q1p*den2&
            &-q0*Q2p*den2&
            &+q0*Q1p**2*(2.0_t_real*denom+1.0_t_real)*den2**2)
    Else
       d2ec1=0.0_t_real
    Endif

    !!
    Return
    !!
  End Subroutine corel_pw92_ec1
  !!***********************************************************************
  Subroutine corel_pw92_ac(rs,srs,ac,dac,d2ac,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Real(t_real),Intent(in)      :: rs   !! wigner seitz radius
    Real(t_real),Intent(in)      :: srs  !! sqrt(rs)
    Real(t_real),Intent(out)     :: ac   !! spin stiffness ac
    Real(t_real),Intent(out)     :: dac  !! 1/rs*d/rs ac
    Real(t_real),Intent(out)     :: d2ac !! 1/rs*d/rs dac
    !!LOCAL
    !Character(*),Parameter::nsubfunc="corel_pw92_ac"
    Real(t_real)   :: ldenom,denom,den2,q0,Q1p,Q2p
    !Real(t_real),Parameter ::beta1_ac_pw92   =10.357_t_real
    !Real(t_real),Parameter ::beta2_ac_pw92   =3.6231_t_real
    !Real(t_real),Parameter ::beta3_ac_pw92   =0.88026_t_real
    !Real(t_real),Parameter ::beta4_ac_pw92   =0.49671_t_real
    !Real(t_real),Parameter ::A_ac_pw92       =0.016887_t_real
    !Real(t_real),Parameter ::alpha1_ac_pw92  =0.11125_t_real
    Real(t_real),Parameter ::c2tA_ac_pw92       =0.033774_t_real
    Real(t_real),Parameter ::cAtbeta1_ac_pw92   =0.174898659_t_real
    Real(t_real),Parameter ::c2tAtbeta1_ac_pw92 =0.349797318_t_real
    Real(t_real),Parameter ::c2tAtbeta2_ac_pw92 =0.1223665794_t_real
    Real(t_real),Parameter ::c2tAtbeta3_ac_pw92 =0.02972990124_t_real
    Real(t_real),Parameter ::c3tAtbeta3_ac_pw92 =0.04459485186_t_real
    Real(t_real),Parameter ::c2tAtbeta4_ac_pw92 =0.01677588354_t_real
    Real(t_real),Parameter ::c4tAtbeta4_ac_pw92 =0.03355176708_t_real
    Real(t_real),Parameter ::c2tAtalpha1_ac_pw92=0.0037573575_t_real
    !!
    Real(t_real),Parameter ::cAo2tbeta1_ac_pw92   =0.0874493295_t_real
    Real(t_real),Parameter ::c9o2tAtbeta3_ac_pw92 =0.06689227779_t_real
    Real(t_real),Parameter ::c8tAtbeta4_ac_pw92 =0.06710353416_t_real
    Real(t_real),Parameter ::c4tAtalpha1_ac_pw92=0.0075147150_t_real
    !!
    denom = (c2tAtbeta1_ac_pw92 +c2tAtbeta3_ac_pw92 *rs)*srs &
         & +(c2tAtbeta2_ac_pw92 +c2tAtbeta4_ac_pw92 *rs)*rs
    
    den2=1.0_t_real/(denom**2+denom)
    
    ldenom=slog(1.0_t_real/denom)
    q0    =-c2tA_ac_pw92-c2tAtalpha1_ac_pw92*rs
    ac= -q0*ldenom
    
    Q1p=    (cAtbeta1_ac_pw92  +c3tAtbeta3_ac_pw92*rs)*srs &
         & +(c2tAtbeta2_ac_pw92+c4tAtbeta4_ac_pw92*rs)*rs
    
    
    
    dac = c2tAtalpha1_ac_pw92*rs*ldenom + q0*Q1p*den2
    
    !!
    If(second_derive)Then

       
       Q2p=    (cAo2tbeta1_ac_pw92  +c9o2tAtbeta3_ac_pw92*rs)*srs &
            & +(c2tAtbeta2_ac_pw92+c8tAtbeta4_ac_pw92*rs)*rs
       
       !!
       d2ac=&
            & + c2tAtalpha1_ac_pw92*rs*ldenom&
            & - c4tAtalpha1_ac_pw92*rs*Q1p*den2&
            & + q0*Q2p*den2&
            & - q0*Q1p**2*(2.0_t_real*denom+1.0_t_real)*den2**2
       
    Endif
    !!
    Return
    !!
  End Subroutine corel_pw92_ac
  !!***********************************************************************
  !! Perdew & Wang, Phys.Rev.B 45, 13244 (1992)
  !!
  Subroutine corel_pw92(mspin,rs,z,z2,z4,fz,dfz,d2fz&
       &,ec,decr,decz,d2ecr,d2ecrz,d2eczz,vc,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: second_derive
    Integer(t_int),Intent(in)    :: mspin
    Real(t_real),Intent(in)      :: z    !! zeta
    Real(t_real),Intent(in)      :: z2   !! zeta**2
    Real(t_real),Intent(in)      :: z4   !! zeta**4
    Real(t_real),Intent(in)      :: fz   !! f(zeta)
    Real(t_real),Intent(in)      :: dfz  !! d/d zeta f(zeta)
    Real(t_real),Intent(in)      :: d2fz !! d/d zeta dfz
    Real(t_real),Intent(in)      :: rs   !! wigner seitz radius
    Real(t_real),Intent(out)     :: ec   !! energy density
    Real(t_real),Intent(out)     :: vc   !! potential
    Real(t_real),Intent(out)     :: decr !! 1/3 1/rs d/d rs ec
    Real(t_real),Intent(out)     :: decz !!  d/d zeta ec
    Real(t_real),Intent(out)     :: d2ecr!! 1/9 1/rs d/d rs dec
    Real(t_real),Intent(out)     :: d2eczz !!  d/d zeta decz
    Real(t_real),Intent(out)     :: d2ecrz !! 1/3 1/rs d/d zeta decz
    !!LOCAL
    !Character(*),Parameter::nsubfunc="corel_pw92"
    Real(t_real)   :: srs,ac,dac,d2ac,ec1,dec1,d2ec1,ec0,dec0,d2ec0
    Real(t_real)   :: q0,q1,dq0,dq1
    Real(t_real),Parameter ::fd2_pw & ! f''(0)=(4/3)/(2**4/3-2.0)/3*2
         & = 1.7099209341613656175639627762446829382052_t_real
    Real(t_real),Parameter :: c1o9=0.1111111111111111111111111_t_real
    Real(t_real),Parameter :: c1o3=0.3333333333333333333333333_t_real
    !!
    srs=Sqrt(rs)
    !!
    If(mspin==1_t_int)Then
       Call corel_pw92_ec0(rs,srs,ec0,dec0,d2ec0,second_derive)
       decr=c1o3*dec0
       d2ecr=c1o9*d2ec0
       vc =ec0-decr
       ec =ec0
       If(second_derive) Then
          Call corel_pw92_ac (rs,srs,ac ,dac ,d2ac ,.False.)
          d2eczz = ac
       Endif
    Else
       Call corel_pw92_ec0(rs,srs,ec0,dec0,d2ec0,second_derive)
       Call corel_pw92_ec1(rs,srs,ec1,dec1,d2ec1,second_derive)
       Call corel_pw92_ac (rs,srs,ac ,dac ,d2ac ,second_derive)

       q0=ec1-ec0-ac/fd2_pw
       q1=q0*z4+ac/fd2_pw

       dq0=dec1-dec0-dac/fd2_pw
       dq1=dq0*z4+dac/fd2_pw

       ec = ec0+q1*fz


       decr=c1o3*(dec0+dq1*fz)


       decz = 4.0_t_real*fz*z2*z*q0+dfz*q1

       If(second_derive) Then

          d2eczz = 4.0_t_real*q0*z2*(3.0_t_real*fz+2.0_t_real*dfz*z)+d2fz*q1
          
          d2ecrz = c1o3*(4.0_t_real*fz*z2*z*dq0+dfz*dq1)
          
          d2ecr=c1o9*(d2ec0+(d2ec1-d2ec0)*z4*fz+d2ac*(1.0_t_real-z4)*fz&
               &/fd2_pw)
       Endif
       

       vc =ec-decr-decz*z
       !! bc=decz

    Endif
    !!
    Return
    !!
  End Subroutine corel_pw92
  !!***********************************************************************
  Subroutine exchange_pw92_pbe96(kf,rho,gradrho,laprho,mixrho&
       &,ex,vx,dvex,dvexdabsgrad,dvexdlaprho,dvexdmixrho&
       &,ggapbe,second_derive)
    !!
    Implicit None
    !!IO
    Logical(t_log),Intent(in)    :: ggapbe,second_derive
    Real(t_real),Intent(in)      :: kf !! mspin==1: (3*pi**2*rho)**(1/3)
    !!                                    mspin==2: (6*pi**2*rhoud)**(1/3)
    Real(t_real),Intent(in)      :: rho   !! the total density or rhoup/dn
    Real(t_real),Intent(in)      :: gradrho !! |grad rho|
    Real(t_real),Intent(in)      :: laprho  !! laplacian rho
    Real(t_real),Intent(in)      :: mixrho  !! (grad rho)(grad grad rho)(grad hro)
    Real(t_real),Intent(out)     :: ex,vx,dvex,dvexdabsgrad
    Real(t_real),Intent(out)     :: dvexdlaprho,dvexdmixrho
    !!LOCAL
    !Character(*),Parameter::nsubfunc="exchange_pw92_pbe96"
    Real(t_real)   :: kfrho,kfrho2,kfrho3,s,s2,x,x2,Fx,dFxos,ddFxos,dddFxos
    Real(t_real)   :: dlaprho,dmixrho,s2ddF,vxlsda,aF,s2dddF
    !Real(t_real),Parameter :: c4o9&
    !     &=0.4444444444444444444444444444444444444444_t_real
    Real(t_real),Parameter :: c3o4pi=0.238732414637843003653325645059_t_real
    Real(t_real),Parameter :: c3o8pi&
         &=0.1193662073189215018275231679277156536130_t_real
    Real(t_real),Parameter :: kappa_pbe96=0.804_t_real
    Real(t_real),Parameter :: mu_pbe96   =0.21951_t_real
    Real(t_real),Parameter :: mk=mu_pbe96/kappa_pbe96
    Real(t_real),Parameter :: mkt4=4.0_t_real*mk
    Real(t_real),Parameter :: mkt6=6.0_t_real*mk
    Real(t_real),Parameter :: mu_pbe96t2 =2.0_t_real*mu_pbe96
    !!
    ex=0.0_t_real
    vx=0.0_t_real
    dvex=0.0_t_real
    dvexdabsgrad=0.0_t_real
    dvexdlaprho=0.0_t_real
    dvexdmixrho=0.0_t_real
    !!
    If(rho<=0.0_t_real) Return
    !!
    ex=-c3o4pi*kf
    vx=c4o3*ex
    vxlsda=vx
    !!
    If(ggapbe)Then
       kfrho=1.0_t_real/(2.0_t_real*kf*rho)
       kfrho2=kfrho*kfrho
       kfrho3=kfrho2*kfrho
       s=gradrho*kfrho
       s2=s*s
       !!
       x=1.0_t_real/(1.0_t_real+mk*s2)
       x2=x*x
       Fx=1.0_t_real+kappa_pbe96-kappa_pbe96*x
       !! 1/s d/ds Fx(s)
       dFxos = mu_pbe96t2*x2
       !! d/ds (1/s d/ds Fx(s))
       ddFxos=-mkt4*dFxos*x
       !!
       !!
       dmixrho=c3o8pi*kfrho3*ddFxos
       dlaprho=c3o8pi*kfrho*dFxos
       !!
       ex=ex*Fx
       !!
       s2ddF=s2*ddFxos
       !!
       vx= vx*(Fx+s2*s2ddF) +laprho*dlaprho +mixrho*dmixrho
       !!
       If(second_derive.And. rho>1.0e-25_t_real)Then
          !!
          dddFxos=-mkt6*ddFxos*x
          !!
          s2dddF=s2*dddFxos
          aF=4.0_t_real*(dFxos+4.0_t_real*s2ddF+s2*s2dddF)
          !!
          dvex=(&
               !!
               & c1o3*vxlsda*(Fx+s2*(s2ddF-aF))&
               !!
               &-(&
               &  laprho*(dFxos+s2ddF)&
               &  +mixrho*kfrho2*(3.0_t_real*ddFxos+s2dddF)&
               & )/twopi*kfrho&
               !!
               &)/rho
          !!
          dvexdabsgrad=(& !!*grad |gradn|
               !!
               &vxlsda*aF*0.25_t_real*kfrho2&
               !!
               &+laprho*dmixrho&
               !!
               &+c3o8pi*mixrho*kfrho3*kfrho2*dddFxos&
               &) !! exchange
          !!
          dvexdlaprho=dlaprho
          dvexdmixrho=dmixrho !! exchange !!*grad mixrho
          !!
       Endif
       !!
    Else
       If(second_derive.And. rho>1.0e-25_t_real)Then
          !!lsda second
          dvex=c1o3*vxlsda/rho
       Endif
    Endif
    !!
    Return
    !!
  End Subroutine exchange_pw92_pbe96
  !!***********************************************************************


End Module xc
