//todo: first analyse the stuff /read +plasmon then ask for the palsmon frequencies
//todo: after determining the axes name components appropriately numnum for general
// axes or xyzxyz for special axes.
//todo: other optical functions -> need diagonalization
//todo: if magnetic relativistic the tensor symmetry is more complicated
// and cannot bededuced from the labels?
/*************************************************************************
 * @(#)OpticsControl.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   22 May 2012
 *************************************************************************
 */

#include "Except.hh"
#include "Assorted.hh"
#include "MyDebug.hh"
#include "MyMath.hh"
#include "SqMatrix.hh"
#include "constants.h"
#include "OpticsControl.hh"
#include "ValueArray.hh"
#include "PtrArray.hh"


static const char *updn[]={"up","down","total"};
static const char *axlbl[3]={"1st","2nd","3rd"};
static const  char comp[]={'x','y','z'};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OpticsControl::askForIntraData()
{

  if(notinteractive){

    for(long spin=-1;++spin<maxspinplas;){
      for(long k=-1;++k<3;){
	if(!haveomega[spin].at(k)){
	  omega[spin][k]=plasmon[spin].at(k);
	}
	haveomega[spin][k]=true;
      }
    }
    

    for(long spin=-1;++spin<maxspinplas;){
      printIntraBandSettings(spin);
    }

    return;
  }

  bool needinput=false;
  for(long spin=-1;++spin<maxspinplas;){
    for(long k=-1;++k<3;){
      needinput|=(!haveomega[spin].at(k));
    }
  }
  for(long k=-1;++k<3;){
    needinput|=(!havegamma[k]);
  }
  if(nointra) {
    needinput=false;
    for(long k=-1;++k<3;){
      havegamma[k]=true;
      for(long spin=-1;++spin<maxspinplas;){
	haveomega[spin][k]=true;
      }
    }
  }
  
  if(needinput){
    for(long spin=-1;++spin<maxspinplas;){
      printIntraBandSettings(spin);
      for(long k=-1;++k<3;){
	String line;
	while(!haveomega[spin].at(k)){
	  printf("for %s axis: plasmon freq [%g]: ",axlbl[k],plasmon[spin].at(k));
	  getLine(stdin,line);line.stripBlanks(true);
	  if(line.length()!=0){
	    try{
	      omega[spin][k]=line.convertToFortranReal();
	      haveomega[spin][k]=true;
	    }catch(Except &ex){};
	  }else{
	    omega[spin][k]=plasmon[spin].at(k);
	    haveomega[spin][k]=true;
	  }
	}
	while(!havegamma[k]){
	  printf("for %s axis: Gamma      [%g]: ",axlbl[k],gamma[k]);
	  getLine(stdin,line);line.stripBlanks(true);
	  if(line.length()!=0){
	    try{
	      gamma[k]=line.convertToFortranReal();
	      havegamma[k]=true;
	    }catch(Except &ex){};
	  }else{
	    havegamma[k]=true;
	  }
	}
	printf("\n");
      }
    }    
  }    
  for(long spin=-1;++spin<maxspinplas;){
    printIntraBandSettings(spin);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OpticsControl::printIntraBandSettings(long spin)
{
  
#define PRNT(c)					\
  printf(",   [");				\
  for(long j=-1;++j<3;){			\
    printf(" %12.8f",axis[spin](j,c));		\
  }						\
  printf("]\n");

  printf("\t\tINTRA BAND CONTRIBUTIONS\n\t\t\tSPIN %s\n",updn[spin]);
  printf("axis     Plasmon[eV]      Gamma[eV]    \t\t\tprincipal axis\n");
  
  for(long k=-1;++k<3;){
    printf("%s:     ",axlbl[k]);
    if(haveomega[spin].at(k)){
      printf( "%-15.8f",omega[spin].at(k));
    }else{
      printf(" %-15.8f",plasmon[spin].at(k));
    }
    printf(" %-15.8f",gamma[k]);
    
    PRNT(k);
  }
  printf("\n");
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OpticsControl::readInFile()
{
  FILE *FH=openforread(pinfile);

  //read labels;
  //pinterdata.clear();
  String line;
  long spin=0;
  while(getLine(FH,line)){
    line.stripBlanks(true);
    if(line[0]=='#'){
      long i0;
      if((i0=line.find("spin="))<0){
	line=line.subString(line.reject("#")).stripBlanks(true);
	//printf("<%s>\n",line());
	StringList sl=line.split(" \t");
	for (long i=1;i<sl.size();i++) {
	  setLabel(i-1,sl[i]);
	}
      }else{
	String s=line.subString(i0+5).stripBlanks();
	spin=s.convertToLong()-1;
      }
      continue;
    }
  }
  fclose(FH);
  maxspin=spin+1;
  

  //read data
  spin=-1;
  long n=-1;
  FH=openforread(pinfile);
  while(getLine(FH,line)){
    line.stripBlanks(true);
    if(line[0]=='#'){
      if(line.find("spin=")>=0) spin++;
      continue;
    }

    n++;
    if (((n) % ptakeevery) !=0 ) { continue; }
    StringList sl=line.split(" \t");
    if(sl.size()<1) {continue;}
    ValueArray<Cmplx> *r=new ValueArray<Cmplx>();
    pdata[spin].append(r);
    r->append(sl[0].convertToFortranReal());
    for (long j=1;j<sl.size();j++) {
      r->append(Cmplx(0.,sl[j].convertToFortranReal()));
    }
  }
  fclose(FH);



  spin=0;
  if(pdata[spin].size()<1) {
    throw Except(Sprintf("In file '%s': no data available?\n",pinfile()));
  }

  pinterdata.resize(MyMath::min(pinterdata.size(),pdata[spin].at(0)->size()-1),true);

  if(pdata[spin].at(0)->size()-1>pinterdata.size()){
    throw Except(Sprintf("In file '%s': number of data columns different "
			 "from number of header labels?\n",pinfile()));
  }

  

  isdiagonal=true;
  ValueArray<bool> iexists(3,false);
  for(int i=-1;++i<pinterdata.size();){
    String s=pinterdata.at(i).label;
    long i0=s.find("(+");
    if(i0>=0)pinterdata[i].isnonessential=true;
    i0=s.find("(");
    if(i0>=0){
      s=s.subString(0,s.find("(")-1);
    }
    if(s.length()<2)
      throw Except(Sprintf("In file '%s': header labels are wrong?\n",pinfile()));
    s=s.subString(s.length()-2,s.length()-1);
    if(s[0]=='x') pinterdata[i].i=0;
    if(s[0]=='y') pinterdata[i].i=1;
    if(s[0]=='z') pinterdata[i].i=2;
    if(s[1]=='x') pinterdata[i].j=0;
    if(s[1]=='y') pinterdata[i].j=1;
    if(s[1]=='z') pinterdata[i].j=2;
    for(int k=-1;++k<3;){
      if(pinterdata[i].i==k||pinterdata[i].j==k) iexists[k]=true;
    }
    if(pinterdata.at(i).i!=pinterdata.at(i).j){
      isdiagonal=false;
    }
  }


  // read plasmon information
  FH=openforread("+plasmon");
  spin=0;
  while(getLine(FH,line)){
    line.stripBlanks(true);
    long i0;
    if((i0=line.find("spin="))>=0){
      String s=line.subString(i0+5).stripBlanks();
      if(s=="total") spin=2;
      else spin=s.convertToLong()-1;
    }
    if((i0=line.find("1st"))>=0){
      String s=line.subString(line.find("axis")+4).stripBlanks();
      StringList sl=s.split(" \t");
      axis[spin](0,0)=sl[0].convertToReal();
      axis[spin](1,0)=sl[1].convertToReal();
      axis[spin](2,0)=sl[2].convertToReal();
    }
    if((i0=line.find("2nd"))>=0){
      String s=line.subString(line.find("axis")+4).stripBlanks();
      StringList sl=s.split(" \t");
      axis[spin](0,1)=sl[0].convertToReal();
      axis[spin](1,1)=sl[1].convertToReal();
      axis[spin](2,1)=sl[2].convertToReal();
    }
    if((i0=line.find("3rd"))>=0){
      String s=line.subString(line.find("axis")+4).stripBlanks();
      StringList sl=s.split(" \t");
      axis[spin](0,2)=sl[0].convertToReal();
      axis[spin](1,2)=sl[1].convertToReal();
      axis[spin](2,2)=sl[2].convertToReal();
    }
    if((i0=line.find("w_p"))>=0){
      String w=line.subString(line.find("w_p")+3).replace("eV","").stripBlanks();
      StringList sl=w.split(" \t");
      plasmon[spin]<<sl[0].convertToReal()<<sl[1].convertToReal()<<sl[2].convertToReal();
    }
  }
  maxspinplas=spin+1;
  fclose(FH);
  
  //todo: analyse the axis structure and compare to pinterdata
  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OpticsControl::writeOpticalFunctions()
{
    
  for(long spin=-1;++spin<maxspinplas;){

    String s=poutsuffix.stripBlanks(true);
    String scom="# total";
    if(s.length()>0)s="_"+s;
    if(maxspin>1 && spin<2){
      s.sprintf("_spin%ld%s",spin+1,s());
      scom.sprintf("# spin=%ld",spin+1);
    // }else{
    //   s.sprintf("_total%s",s());
    }
    
    FILE* FHR=openforwrite(Sprintf("re_eps%s",s()));
    FILE* FHI=openforwrite(Sprintf("im_eps%s",s()));
    FILE* FHL=openforwrite(Sprintf("loss%s",s()));
    FILE* FHRS=openforwrite(Sprintf("re_sigma%s",s()));
    
    
    //headers
    fprintf(FHR ,"%s\n# e[eV]",scom());
    fprintf(FHI ,"%s\n# e[eV]",scom());
    fprintf(FHL ,"%s\n# e[eV]",scom());
    fprintf(FHRS,"%s\n# e[eV]",scom());
    String sp;
    if(maxspin>1){
      if(spin==0) sp="$^$arrowup$.";
      else if(spin==1) sp="$^$arrowdown$.";
    }
    for (long kk=-1;++kk<pinterdata.size();) {
      const Data &dd=pinterdata.at(kk);
      if(dd.isnonessential) continue;
      fprintf(FHR ," Re($~e%s$_%c%c$.)",sp(),comp[dd.i],comp[dd.j]);
      fprintf(FHI ," Im($~e%s$_%c%c$.)",sp(),comp[dd.i],comp[dd.j]);
      fprintf(FHL ," L%s$_%c%c$.",sp(),comp[dd.i],comp[dd.j]);
      fprintf(FHRS," Re($~s%s$_%c%c$.)",sp(),comp[dd.i],comp[dd.j]);
    }
    fprintf(FHR,"\n");
    fprintf(FHI,"\n");
    fprintf(FHL,"\n");
    fprintf(FHRS,"\n");
    
    

    //write data
    // here we assume that all pdata[spin] are haveing the same length
    long Nd=pdata[0][0]->size()-1;
    for (long i=0;i<pdata[0].size();i++) {
      double w=pdata[0][i]->at(0).real();
      fprintf(FHR,"%g",w);
      fprintf(FHI,"%g",w);
      fprintf(FHL,"%g",w);
      fprintf(FHRS,"%g",w);
      

      SqMatrix<Cmplx> ep(3);
      // set(spin<2)/add(spin==2) inter-band spin contributions
      int ms0=spin-1,ms1=spin+1; if(spin==2){ms0=-1;ms1=2;}
      for(int ms=ms0;++ms<ms1;){
	const ValueArray<Cmplx> *r=pdata[ms][i];
	for (long kk=0;kk<Nd;kk++){
	  const Data &dd=pinterdata.at(kk);
	  Cmplx inter=r->at(kk+1);
	  if(!nointer){
	    ep(dd.i,dd.j)+=inter;
	  }
	}
      }
      // add intraband tensor
      if(!nointra){
	for (long ii=-1;++ii<3;){
	  for (long jj=-1;++jj<3;){
	    for (long k=-1;++k<3;){
	      ep(ii,jj)+=axis[spin](ii,k)
		*intraBandContribution(omega[spin].at(k),gamma[k],w)
		*axis[spin](jj,k);
	    }
	  }
	}
      }
      // add diagonal unit matrix
      ep+=SqMatrix<Cmplx>(3).diag(Cmplx::c1);
      

      SqMatrix<double> Lo=-imag(ep.inverse());
      
      for (long kk=0;kk<Nd;kk++){
	const Data &dd=pinterdata.at(kk);
	if(dd.isnonessential) continue;
	Cmplx eps=ep(dd.i,dd.j);
	fprintf(FHR," %g",eps.real());
	fprintf(FHI," %g",eps.imag());
	fprintf(FHL," %g",Lo(dd.i,dd.j));
	fprintf(FHRS," %g",w*eps.imag()*constant_resigtoinvOhmcm);
      }
      fprintf(FHR,"\n");
      fprintf(FHI,"\n");
      fprintf(FHL,"\n");
      fprintf(FHRS,"\n");
    }
    fclose(FHR);
    fclose(FHI);
    fclose(FHL);
    fclose(FHRS);
  }  
  printf("Note, that sigma has units of 1/(Ohm*cm)\n");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Cmplx OpticsControl::intraBandContribution(double omegaP,double gamma,double w)
{
  if (w==0) {
    w=1.0e-10;
  }
  double R=omegaP*omegaP/(w*w+gamma*gamma);
  
  return Cmplx(-R,R*gamma/w);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OpticsControl::calcOpticsFunctions()
{
  
  for(long spin=-1;++spin<maxspin;){
    PtrArray<ValueArray<Cmplx> > &pd=pdata[spin];
    
    long Nd=pd.size();
    //  printf("DATA.size=%ld\n",Nd);
    if(Nd<2){
      throw Except("not enough data available?\n");
    }
    long N=pd[0]->size();
    //printf("DATA components=%ld\n",N-1);
    
    long subdiv=(MyMath::max<long>(1,Nd/(100/20)));
    
    for (long I=0;I<Nd;I++) {
      
      if( (I%subdiv)==0 || I==0 || I==Nd){
	printf("done=%g%%\n",I*100./Nd);
      }
      double w=pd[I]->at(0).real();
      
      
      for (long k=1;k<N;k++){
	
	double sum=0.;
	ValueArray<double> x;
	ValueArray<double> y;
	for (long j=0;j<=I-1;j++){
	  double xx=pd[j]->at(0).real();
	  x.append(xx);
	  y.append(pd[j]->at(k).imag()*xx/(xx*xx-w*w));
	}
	MyMath::runningSum(&x,&y);
	if(y.size()) sum+=y.last()*2/Pi;
	
	
	
	x.clear();
	y.clear();
	for (long  j=I+1;j<Nd;j++){
	  double xx=pd[j]->at(0).real();
	  x.append(xx);
	  y.append(pd[j]->at(k).imag()*xx/(xx*xx-w*w));
	}
	MyMath::runningSum(&x,&y);
	if(y.size()) sum+=y.last()*2/Pi;
	
	bool correct=true;
	if (correct && I!=0 && I!=Nd-1){
	  double x0=pd[I-1]->at(0).real();
	  double x1=w;
	  double x2=pd[I+1]->at(0).real();
	  
	  double y0=pd[I-1]->at(k).imag();
	  double y1=pd[I  ]->at(k).imag();
	  double y2=pd[I+1]->at(k).imag();
	  
	  double D10=x1-x0;
	  double D20=x2-x0;
	  double D21=x2-x1;
	  double S10=x1+x0;
	  double S20=x2+x0;
	  double S21=x2+x1;
	  double L2110=log(S21/S10);
	  double J0=x1*S21*L2110/(D10*D20)-S21/(2*D10)-0.5;
	  double J1=log(D21/D10)*0.5+(-S10*S21*L2110+D20*S20)/(2*D10*D21);
	  double J2=x1*S10*L2110/(D20*D21)-S10/(2*D21)+0.5;
	  double cor=(J0*y0+J1*y1+J2*y2);
	  sum+=cor*2/Pi;
	} 
	
	
	(*pd[I])[k].real()=sum;
      }
    }
  }
}


/*EOF*/ 
