/* @(#)OpticsControl.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OPTICSCONTROL_H
#define _OPTICSCONTROL_H 1

#include "FString.hh"
#include "MyMath.hh"
#include "PtrArray.hh"
#include "ValueArray.hh"
#include "SqMatrix.hh"
#include "Cmplx.hh"

class OpticsControl{
 private:
  class Data{
  public:
    Data():label("??"),isnonessential(false){};
    String label;
    long i,j; // index into eps tensor
    bool isnonessential; // copy of another component
  };

 public:


  OpticsControl():pinfile("+imeps"),poutsuffix(""),ptakeevery(1),
		  nointer(false),nointra(false),isdiagonal(false),
		  notinteractive(false){
    for(long spin=-1;++spin<3;){
      axis[spin].resize(3);
      omega[spin].resize(3); omega[spin]=0.;
      haveomega[spin].resize(3);haveomega[spin]=false;
    }
    gamma[0]=gamma[1]=gamma[2]=0.1;
    havegamma[0]=havegamma[1]=havegamma[2]=false;
  };
  ~OpticsControl(){};



  const String &inFile()const {return pinfile;};
  void setInfile(const String &s){pinfile=s;};

  const String &outSuffix()const {return poutsuffix;};
  void setOutSuffix(const String &s){poutsuffix=s;};

  long skip()const{return ptakeevery;};
  void setSkip(long s){ptakeevery=MyMath::max<long>(1,s);};

  void setNoInter(bool n){nointer=n;}
  void setNoIntra(bool n){nointra=n;}
  void setNotInteractive(bool n){notinteractive=n;}
  
  void setGamma(long index,double g){
    gamma[index]=g;
    havegamma[index]=true;
  }
  void setOmega(long index,long spin,double o){
    omega[spin][index]=o;
    haveomega[spin][index]=true;
  }
  void setLabel(long index,const String &l){
    if(index>=pinterdata.size())
      pinterdata.resize(index+1);
    pinterdata[index].label=l;
  }

  void readInFile();
  void calcOpticsFunctions();
  void writeOpticalFunctions();
  void askForIntraData();

  void printIntraBandSettings(long spin);

  
 private:
  Cmplx intraBandContribution(double omegaP,double gamma,double w);

 private:
  String pinfile;
  String poutsuffix;
  ValueArray<Data> pinterdata;
  long ptakeevery;
  long maxspin;
  PtrArray<ValueArray<Cmplx> > pdata[2]; // [spin], spin=0(up),1(dn)
  bool nointer,nointra,isdiagonal,notinteractive;
  long maxspinplas;
  SqMatrix<double> axis[3]; // [spin] spin=0(up),1(dn),2(total), columns are axis-vectors
  ValueArray<double> plasmon[3]; // [spin], from file +plasmon
  ValueArray<double> omega[3]; // the user accepted/defined plasmon energies
  ValueArray<bool> haveomega[3];
  double gamma[3];
  bool havegamma[3];

};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OPTICSCONTROL_H */

