/************************************************************************
 *
 * graphic package for FPLO
 * (C) Klaus Koepernik April '98
 *
 * **********************************************************************
 */
#ifndef _SYMMETRY_H_
#define _SYMMETRY_H_




struct tscbuffer;

/* crystal systems */
#define SYM_CUBIC        0
#define SYM_HEXAGONAL    1
#define SYM_TETRAGONAL   2
#define SYM_TRIGONAL     3
#define SYM_ORTHORHOMBIC 4
#define SYM_MONOCLINIC   5
#define SYM_TRICLINIC    6
#define SYM_CRYSTALUNKNOWN 7


/* bravais types */
#define SYM_SIMPLE        0
#define SYM_BODYCENTERED  1
#define SYM_FACECENTERED  2
#define SYM_ABASECENTERED 3
#define SYM_CBASECENTERED 4
#define SYM_RHOMBOHEDRAL  5
#define SYM_BBASECENTERED 6
#define SYM_BRAVAISUNKNOWN 7

#define MAXPOINTGRP 37

#define SYM_OH  37

extern const char *operationsymbols[];


/* symmetry definition */
typedef struct tssymmetry{
  long num;			/* space group number */
  long npgr;			/* point group number */
  long crystal;			/* crystalsystem */
  long str_type;		/* crystalsystem */
  long bravais;			/* type of bravais lattice */
  long inversion;		/* has inversion ? */
  long nonsym;        		/* is nonsymmorphic ? */
  long hex;			/* hexagonal subgroup */
  long RTG;			/* is rhombohedric */
  long ngen;			/* number of generators */
  long ng;			/* number of operations */
  long itable[64];		/* indices into operation table */
  long igen[48];	         	/* indices into operation table */
  long    eulerinv[64];          /* inversion of operation */
  double v[3*64];		/* nonsym translations */
  double opmat[9*64];		/* operations */
  double eulerangles[3*64];     /* euler angles of operations */
  double scal[3];		/* scaling of lattice constants */
  double alat[3];		/* lattice constants */
  double angle[3];		/* axis angles */
  double usheer[9];		/* unit vectros along axis */
  double cell[9];		/* unit cell vectors */
  double rcell[9];		/* reciprocial unit cell vectors */
  double ccell[9];		/* conventional direct unit cell vecs */
  double crcell[9];		/* conventional reciprocial unit cell vecs */
  double vuc;                   /* volumn of cell */
  char group[9];		/* space group name Herman Mauguin */
  long schoenflies;		/* space group number of Schoenflies */
  char lattice[9];		/* lattice symbol */
  long nugen;			/* user defined set of generators */
  long iugen[48];		/* ... */
}tsSYMMETRY;



#ifdef	__cplusplus
extern "C" {
#endif

  extern void copySymmetry(const tsSYMMETRY *s1,tsSYMMETRY *s2);
  extern void reportcalcsym(struct tscbuffer *b,tsSYMMETRY *sym);
  extern long calcsym(tsSYMMETRY *sym,long num,const char *buf
		     ,double alat[3],double angle[3],long complain_subgroups
		      ,long str_type);
  extern const long   *bravais_centering(long brav);
  extern const char   *bravaistype(long brav);
  extern const char   *crystalsystem(long crys);
  extern long get_crystal_system_from_num(long num);
  extern long get_bravais_from_num(long num);
  extern long get_pointgroup_from_num(long num);
  extern long numberofcrystalsystem();
  extern long numberofbravaistypes();
  extern long numberofbravaistypesofcryssystem(long csystem);
  extern long bravaistypesofcryssystem(long csystem,long index);
  extern const char   *crystalsystems_longname(long crys);
  extern const char   *bravaistypes_longname(long bravasi);
  extern const char   *pointgroup(long pgr);
  extern void converttosimple(const double *cell,double *scell,long bravais);
  extern void rothextocart(double *gout,const double *gin,long hex);
  extern void getrotations(double *gen);

  extern void transatob(double *b,const double *a,long hex); 
  extern const double *transbtoa(double *b,const double *a,long hex); 

  extern long numberofspacegroupsof_c_b(long crystal,long btype);
  extern long spacegroupsof_c_b(long crystal,long btype,long i);  

  extern long operationindex(double *u,long hex);
  extern const char *operationXYZ(double *u);
  extern const char *operationsymbol(long ig);
/*   extern void getpointgroupmasks(long *pg); */
/*   extern long getMaxPointGroupForLattice(long crystal,long bravais); */
/*   extern void getanalysishelper(long c4[64],long s4[64],long c2[64],long s2[64] */
/* 				,long c3[64],long s3[64] */
/* 				,long c6[64],long s6[64] */
/* 				,long opi[64] */
/* 				,long ope[64] */
/* 				,long mm[64],long rm[64] */
/* 				,double u1[3*17] */
/* 				,long *xaxis,long *yaxis,long *zaxis */
/* 				,long *c2x,long *c2y,long *c2z */
/* 				,long *sx,long *sy */
/* 				,long *c14y,long *inv */
/* 				,long *c13z */
/* 				,long *c14z */
/* 				,long *s14z */
/* 				); */
  extern const char *getSpacegroupDescriptor(const char *name,long *num);
  extern const char *spcgrp_name(const char* sginfo);
  extern const char *spcgrp_crystallsystem(const char* sginfo) ;
  extern void ConvertVecRTGtoSTG(double *result,const double *v);
  extern long get_gentau_from_sginfo(const char *sginfo,long igen
				     ,long *j
				     ,long iv[3]
				     );
  extern long get_pgrnum_from_sginfo(const char *sginfo);
  extern long get_bravais_from_sginfo(const char *sginfo);
  extern long get_spacegroup_of_from_pointgroup(long pgrnum);

  extern long get_group_operations(long num,tsSYMMETRY *sym);
  

#ifdef	__cplusplus
}
#endif


#endif /* _SYMMETRY_H_ */
/* EOF */
