/* @(#)allocate.c
 */


#include <stdlib.h>
#include <stdio.h>
#include "heapcheck.h"
#include "allocate.h"
#include "merror.h"


static long Debug=0;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void *reallocate(void *pv,long size,const char *name,const char *section)
{
  void *p=NULL;
  if((p=realloc(pv,size))==NULL){
    (void)sprintf(errormsg
	    ,"error(%s): Could not reallocate memory (%s)!"
	    ,section,name);
    p=NULL;
  }else{

    if(Debug){
      (void)fprintf(stderr,"REALLOCATE %s=%p\n",name,p);
      (void)printf("REALLOCATE %s=%p\n",name,p);
    }
#ifdef EXTENDED_HEAP_TEST
    if(extended_heap_test){
      if(p!=pv){
	REMOVEFROMHEAP(pv);
	ADDTOHEAP(p);
      }
    }
#endif

  }
  return p;
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void*  allocate(long size,const char *name,const char *section)
{
  void *p=NULL;
  if( (p=malloc(size))==NULL ){ 
    (void)sprintf(errormsg
	    ,"error(%s): Could not allocate memory (%s)!"
	    ,section,name);
    p=NULL;
  }else{
#ifdef  EXTENDED_HEAP_TEST
    if(extended_heap_test){
      ADDTOHEAP(p);
    }
#endif

    if(Debug){
      (void)fprintf(stderr,"ALLOCATE %s=%p\n",name,p);
      (void)printf("ALLOCATE %s=%p\n",name,p);
    }
  }
  return p;
}
    

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FREEE(void *p,const char *name)
{
  if(p!=NULL){
    if(Debug){
      (void)printf("FREE %s %p\n",name,p);
      (void)fprintf(stderr,"FREE %s %p\n",name,p);
    }
#ifdef EXTENDED_HEAP_TEST
    if(extended_heap_test){
      REMOVEFROMHEAP(p);
    }
#endif    
    free(p);
  }
}
