#include <stdlib.h> 
#include <string.h> 
#include <stdarg.h> 
#include "allocate.h" 
#include "cbuffer.h"
#include "merror.h" 



/* the macro version  calls j twice !! */
static long LMAX(long i,long j)
{
  return ( ((i)>j?i:j) );
}


/* forward prototypes */
static long CBAdjust(tsCBUFFER *b,long len);


/* ***********************************************************************
 * initialize buffer 
 * ***********************************************************************
 */
void CBInit(tsCBUFFER *b,long minl,long incr)
{
  b->buflen=0;
  b->buffer=NULL;
  b->tail=NULL;
  b->minlen=LMAX(labs(minl),100);
  b->incr=LMAX(labs(incr),100);
}
/* ***********************************************************************
 * start buffering 
 * ***********************************************************************
 */

long CBStart(tsCBUFFER *b)
{
  const char *subfunc="CBStart";
  if( b->buffer != NULL){
    (void)sprintf(errorbuffer,"error(%s): called without initialization!\n"
	    " Must be initialited with CBInit or reset with CBEnd\n"
	    ,subfunc);
    return 0;
  }
  b->buflen=b->minlen;
  if(( b->buffer 
       =(char*)allocate(sizeof(char)*b->buflen,"buffer",subfunc))==NULL) {
    (void)sprintf(errorbuffer,"%s\n",errormsg);
    return 0; 
  }
  *(b->buffer)='\0';
  b->tail=b->buffer;
  return 1;
}

/* ***********************************************************************
 * cleanup 
 * ***********************************************************************
 */
void CBEnd(tsCBUFFER *b){
  FREEE(b->buffer,"b->buffer");
  b->buffer=NULL;
  b->tail=NULL;
  b->buflen=0;
}

/* ***********************************************************************
 * resize buffer to keep at least len more characters
 * ***********************************************************************
 */
static long CBAdjust(tsCBUFFER *b,long len)
{
  const char *subfunc="CBAdjust";
  long newlen,oldlen,leng;
  if( b ==NULL ) {
    (void)sprintf(errorbuffer,"error(%s): buffer==NULL!",subfunc);
    return 0;
  }
  oldlen=b->buflen;
  leng=b->tail-b->buffer;
  newlen=leng+1+len+1;
  while(newlen>b->buflen){
    b->buflen+=b->incr;
  }
  if(oldlen != b->buflen){
    if((b->buffer
	=(char*)reallocate(b->buffer,sizeof(char)*b->buflen
			   ,"cbbuffer",subfunc))==NULL){
      (void)sprintf(errorbuffer,"%s\n",errormsg);
      return 0;
    }
    
    b->tail=b->buffer+leng;
  }
  return 1;
}

/* ***********************************************************************
 * save print into buffer (at end) 
 * ***********************************************************************
 */
long CBAppend(tsCBUFFER *b,long exlen,const char *fmt,...)
{ 
  long len; 
  va_list ap;
  
  if(b!=NULL){
    len=strlen(fmt); 
    len+=exlen; 
    if(!CBAdjust(b,len)) return 0;

    va_start(ap, fmt);
    (void)vsprintf(b->tail, fmt, ap);
    va_end(ap);
    b->tail+=strlen(b->tail); 
  }
  return 1;
} 

/* ***********************************************************************
 * print buffer into FH 
 * ***********************************************************************
 */
void CBPrint(tsCBUFFER *b,FILE *FH)
{
  if(FH &&   b!=NULL  && b->buffer!=NULL ) {
    (void)fprintf(FH,"%s",b->buffer);
  }
}

/* ***********************************************************************
 * return pointer to buffer 
 * ***********************************************************************
 */
const char * CBGetBuffer(tsCBUFFER *b)
{
  if(  b!=NULL  && b->buffer!=NULL ) {
    return b->buffer;
  }
  return NULL;
}

