/* @(#)tools.c
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "tools.h"
#include "merror.h"
#include "allocate.h"


#ifndef RSIGN
#define RSIGN(x,y) ( (x)* ((y)>=0. ? 1.:-1.) )
#endif

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long ROUND(double x)
{
  return ((x)>0. ?((long)(x+.5)) :((long)(x-.5)));
}

/* ***********************************************************************
 * make normed vector
 * ***********************************************************************
 */
double norm(double *v,long n)
{
  double d=0.;
  long i;
  
  
  d=sqrt(ddotn(v,v,3));
  if(d == 0.) return d;
  for(i=0;i<n;i++){
    v[i]/=d;
  }
  return d;
}


/* ***********************************************************************
 * scalar product
 * ***********************************************************************
 */

double ddotn(const double *x,const double *y,long n)
{
  long i;
  double d=0.;
  for(i=0;i<n;i++){
    d+=x[i]*y[i];
  }
  return d;
}/* ***********************************************************************
 * scalar product
 * ***********************************************************************
 */

long idotn(const long *x,const long *y,long n)
{
  long i;
  long d=0;
  for(i=0;i<n;i++){
    d+=x[i]*y[i];
  }
  return d;
}


/* ***********************************************************************
 * given x,y,? and a radius
 * calc x,y,z at sphere of radius or x,y,0 if ouside sphere
 * and normalyze
 * ***********************************************************************
 */
void XYRtoXYZ(double *vec,double rad)
{

  vec[2]=0.;
  vec[2]=ddotn(vec,vec,3);
  if(vec[2]>=rad*rad)
    vec[2]=0.;
  else
    vec[2]=sqrt(rad*rad-vec[2]);
  (void)norm(vec,3);
    
}


/* ***********************************************************************
 * v=v+a*v1,return v
 * ***********************************************************************
 */
double *vplus(double *v,const double *v1,double a,long n)
{
  long i;
  for(i=0;i<n;i++){
    v[i]+=a*v1[i];
  }
  return v;
}


/* ***********************************************************************
 * v=v2-v1,return |v|
 * ***********************************************************************
 */
double vdiff(const double *v1,const double *v2,double *v,long n)
{
  long i;
  double d,r;
  r=0.;
  for(i=0;i<n;i++){
    v[i]=d=v2[i]-v1[i];
    r+=d*d;
  }

  return sqrt(r);

}



/* ***********************************************************************
 * initialize double array
 * ***********************************************************************
 */
void dinit(double *a,double x,long n)
{
  long i;
  for(i=0;i<n;i++){
    a[i]=x;
  }
}
/* ***********************************************************************
 * initialize long array
 * ***********************************************************************
 */
void linit(long *a,long x,long n)
{
  long i;
  for(i=0;i<n;i++){
    a[i]=x;
  }
}
/* ***********************************************************************
 * scale double array
 * ***********************************************************************
 */
void dscale(double *a,double x,long n)
{
  long i;
  for(i=0;i<n;i++){
    a[i]*=x;
  }
}
void iscal(long *a,long x,long n)
{
  long i;
  for(i=0;i<n;i++){
    a[i]*=x;
  }
}

/* ***********************************************************************
 * 3x3 matrix A  if opt=1 : A^-1^T
 *                      2 : A^-1
 * det  : determinate
 * ***********************************************************************
 */
long dinv33(const double *aa,long opt,double *iaa,double *det)
{
  long i,j;
  double d;
  cross(&aa[3],&aa[6],&iaa[0]);
  cross(&aa[6],&aa[0],&iaa[3]);
  cross(&aa[0],&aa[3],&iaa[6]);
  *det=aa[0]*iaa[0]+aa[1]*iaa[1]+aa[2]*iaa[2];

  if(*det == 0.){
    return 0;
  }
  if( opt % 2 == 0) {
    for(i=0;i<3;i++){
      for(j=i+1;j<3;j++){
	d=iaa[i+j*3];
	iaa[i+j*3]=iaa[j+i*3];
	iaa[j+i*3]=d;
      }
    }
  }
  for(i=0;i<9;i++){
    iaa[i]/=*det;
  }
  return 1;
}
/* ***********************************************************************
 * transpose matrix[i,j]=a[i+dim*j]
 * ***********************************************************************
 */
void transpose(double *a,long dim)
{
  long i,j;
  double d=0.;
  for(i=0;i<dim;i++){
    for(j=i+1;j<dim;j++){
      d=a[i+dim*j];
      a[i+dim*j]=a[j+dim*i];
      a[j+dim*i]=d;
    }
  }
}




/* ***********************************************************************
 * cross prodduct
 * ***********************************************************************
 */
void cross(const double *x1,const double *x2,double *cr12)
{
  cr12[0] = x1[1]*x2[2] - x1[2]*x2[1];
  cr12[1] = x1[2]*x2[0] - x1[0]*x2[2];
  cr12[2] = x1[0]*x2[1] - x1[1]*x2[0];
}
void icross(const long *x1,const long *x2,long *cr12)
{
  cr12[0] = x1[1]*x2[2] - x1[2]*x2[1];
  cr12[1] = x1[2]*x2[0] - x1[0]*x2[2];
  cr12[2] = x1[0]*x2[1] - x1[1]*x2[0];
}




/* ***********************************************************************
 * v0[i]=v1[i]+v2[j]*g[j][i]
 * g[9] is linear array , g[row,column]=g[row+column*3]
 * ***********************************************************************
 */
void vec1(double *v0,const double *v1,const double *v2,const double *g)
{
  long i,j;
  for(i=0;i<3;i++){
    v0[i]=v1[i];
    for(j=0;j<3;j++){
      v0[i]+=v2[j]*g[j+i*3];
    }
  }
}


/* ***********************************************************************
 * res[i]=A[i][j]*v[j]
 * A[9] is linear array , g[row,column]=g[row+column*n]
 * ***********************************************************************
 */
double *AtimesV(double *res,const double *A,const double *v,long n)
{
  long i,j;
  double d;
  for(i=0;i<n;i++){
    d=0.;
    for(j=0;j<n;j++){
      d+=A[i+j*n]*v[j];
    }
    res[i]=d;
  }
  return res;
}

/* ***********************************************************************
 * res[i]=v[j]*A[j][i]
 * A[9] is linear array , g[row,column]=g[row+column*n]
 * ***********************************************************************
 */
double *VtimesA(double *res,const double *v,const double *A,long n)
{
  long i,j;
  double d;
  for(i=0;i<n;i++){
    d=0.;
    for(j=0;j<n;j++){
      d+=v[j]*A[j+i*n];
    }
    res[i]=d;
  }
  return res;
}

/* ***********************************************************************
 * v0[i]=v2[j]*g[j][i]
 * g[9] is linear array , g[row,column]=g[row+column*n]
 * ***********************************************************************
 */
void vec2(double *v0,const double *v2,const double *g,long n)
{
  long i,j;
  for(i=0;i<n;i++){
    v0[i]=0.;
    for(j=0;j<n;j++){
      v0[i]+=v2[j]*g[j+i*n];
    }
  }
}




/* ***********************************************************************
 *  matrix product c=a*b 
 *  with fortran storing convention 
 *  means a[i,j]=a[i+n*j] 
 *
 * ***********************************************************************
 */
void matmult(double *cc,const double *aa,const double *bb,const long dim)
{
  long i,j,k;
  double d;
  for (i=0;i<dim;i++){
    for ( j=0;j<dim;j++){
      d=0.;
      for ( k=0;k<dim;k++){
        d+=aa[i+dim*k]*bb[k+dim*j];
      }
      cc[i+dim*j]=d;
    }
  }
}
/* ***********************************************************************
 *  matrix product c=a*b 
 *  with fortran storing convention 
 *  means a[i,j]=a[i+n*j] 
 *
 * ***********************************************************************
 */
void imatmult(long *cc,const long *aa,const long *bb,const long dim)
{
  long i,j,k;
  long d;
  for (i=0;i<dim;i++){
    for ( j=0;j<dim;j++){
      d=0;
      for ( k=0;k<dim;k++){
        d+=aa[i+dim*k]*bb[k+dim*j];
      }
      cc[i+dim*j]=d;
    }
  }
}


/* ***********************************************************************
 * rotation matrix which puts system X,Y,Z
 * into X',Y',Z' so that v becomes z-axis
 *
 * aa[i,k]*v[k] = (0,0,1) with v[0]^2+v[1]^2+v[2]^2=1 
 *
 * The invariant vector is the cross prouct of Z and v
 *
 * storage:  aa[i,k]=aa[i+3*k]
 * 
 * 
 * ***********************************************************************
 */
void rtomat(double *v,double *aa)
{
  double s,tol,s2;
  
 
  tol=1.e-4;
  s=sqrt(v[0]*v[0]+v[1]*v[1]);
  if(fabs(s)<tol){
    dinit(aa,0.,9);
    aa[0+3*0]=1.;
    aa[1+3*1]=1.;
    aa[2+3*2]=1.;
    return;
  }
  s2=s*s;
  aa[0+3*0]=(v[1]*v[1]+v[0]*v[0]*v[2])/s2;
  aa[0+3*1]=(-v[0]*v[1]+v[0]*v[1]*v[2])/s2;
  aa[0+3*2]=-v[0];
  aa[1+3*0]=(-v[0]*v[1]+v[0]*v[1]*v[2])/s2;
  aa[1+3*1]=(v[0]*v[0]+v[1]*v[1]*v[2])/s2;
  aa[1+3*2]=-v[1];
  aa[2+3*0]=v[0];
  aa[2+3*1]=v[1];
  aa[2+3*2]=v[2];
  
  
}

/* ***********************************************************************
 * spat product
 * ***********************************************************************
 */
double spat(const double *a1,const double *a2,const double *a3)
{
  double p[3];
  
  cross(a2,a3,p);
  return  ddotn(a1,p,3);
}

long ispat(const long *a1,const long *a2,const long *a3)
{
  long p[3];
  
  icross(a2,a3,p);
  return  idotn(a1,p,3);
}



#define BUFFER_MAX_STRING 5000 
/* ***********************************************************************
 * search token in file 
 * ***********************************************************************
 */
long idebfun(FILE *FH,const char *name,long over,char *line)
{
  long found=0;
  long i;
  char buf[BUFFER_MAX_STRING+1];
  char *cdum;
  
  rewind(FH);
  while(fgets(buf,BUFFER_MAX_STRING,FH)!=NULL){
    chop(buf);

    if( strstr(buf,name)!=NULL || name[0]=='\0'){
      
      if(over == 0){
	(void)strncpy0(line,buf,BUFFER_MAX_STRING);

      }else{
	for (i=1;i<over;i++){
	  cdum=fgets(buf,BUFFER_MAX_STRING,FH);
	}
      }

      found=1;
      break;
    }
  }
  cdum=cdum;
  return found;
}


/* ***********************************************************************
 * decompose d into m/n upto n=maxn with tolerance tol
 * ***********************************************************************
 */
void ratio(double d,long *m,long *n,long maxn,double tol)
{
  long i,s;
  double da=fabs(d);
  s=(long)RSIGN(1.,d);
  if(da<=tol){
    *m=0;
    *n=1;
    return;
  }
  *n=*m=0; 
  for(i=1;i<=maxn;i++){
    *m=ROUND(da*i);
    
    if(fabs(((double)*m)/((double)i)-da)<tol){
      *n=i;
      break;
    }
  }
  *m*=s;
}
/* ***********************************************************************
 * covert double to ratio
 * d=m/n
 * ***********************************************************************
 */

void doubletoratio(char *r,double d,long maxn,double tol)
{
  long m,n;
  ratio(d,&m,&n,maxn,tol);
  
  (void)sprintf(r,"%ld/%ld",m,n);
  if(n == 0) 
    (void)sprintf(r,"%-15.5f",d);
  else if(n == 1)  (void)sprintf(r,"%ld",m);
}

/* ***********************************************************************
 * interchange two vectors
 * ***********************************************************************
 */
void dswap(double *a,double *b,long n)
{
  long i;
  double d;
  for(i=0;i<n;i++){
    d=b[i];
    b[i]=a[i];
    a[i]=d;
  }
}
void iswap(long *a,long *b,long n)
{
  long i;
  long d;
  for(i=0;i<n;i++){
    d=b[i];
    b[i]=a[i];
    a[i]=d;
  }
}

/* ***********************************************************************
 * shell sort of a array of double precision vectors
 *BEGIN REMARK
 *info corrected by Klaus Koepernik
 *info the comparison of double precision vectors has to include
 *info a tolerance
 *info this is dafault tol=1.d-12, if tol.eq.0.d0
 *info if problems with sorting appear
 *info enhance the accuracy of the input data, or increase the tolerance
 *END REMARK
 * ----------------------------------------------------------------------
 *i Inputs:
 *i   m     :number of components in darray
 *i   n     :number of elements in darray
 *i   darray:array to be sorted
 *i   tol   :tolerance
 *o Outputs:
 *o   iwk   :table of indices to array darray
 * ----------------------------------------------------------------------
 *
 * ***********************************************************************
 */

void dsort(long m,long n,const double *darray,long *iwk,double tol)
{

  /*
      double precision darray(m,0:n-1),tol
  */

  long lognb2,i,j,k,l,n2,nn,it,mm;
  double dif;

  if(tol == 0.0) tol=1.e-12;
  lognb2 = (long)(log((double)(n+1))*1.4426950);
  n2 = n;
  for( i = 0;i<n;i++){
    iwk[i] = i;
  }
  for( nn = 0;nn<lognb2;nn++){
    n2 /= 2;
    k = n - n2;
    for( j = 0;j<k;j++){
      i = j;
    L3:;
      l = i + n2;
      for( mm = 0;mm<m;mm++){
	dif=darray[mm+m*iwk[l]]- darray[mm+m*iwk[i]];
	if (dif > fabs(tol)) goto L11;
	if (fabs(dif)<fabs(tol)) goto L15;

            
	it = iwk[i];
	iwk[i] = iwk[l];
	iwk[l] = it;
	i -= n2;
	if (i >= 0) goto L3;
	goto L11;
      L15:;
      }
    L11:;
    }
     
  }

  return ;
}



/* ***********************************************************************
 * test if ptr is a double
 * ret: 0 failure
 *      1 d contains double(ptr)
 * ***********************************************************************
 */

long isdouble(double  *d,char *ptr)
{
  char *err;
  if(ptr==NULL)return 0;
  *d=strtod(ptr,&err);
  return (*ptr!='\0' && *err==0);
}


/* ***********************************************************************
 * test if ptr is a rational
 * ret: 0 failure
 *      1 d contains double(ptr)
 * ***********************************************************************
 */

long isratio(double  *d,char *ptr)
{
  char *buf;
  char *p;
  long ret;
  double d1,d2;
  buf=(char*)malloc(sizeof(char)*strlen(ptr)*2);
  if(ptr==NULL)return 0;

  if((p=strstr(ptr,"/"))!=NULL){
    (void)strncpy(buf,ptr,p-ptr);
    buf[p-ptr]='\0';
    ret=isdouble(&d1,buf);
    (void)strcpy(buf,p+1);
    ret&=isdouble(&d2,buf);
    *d=d1/d2;
  }else{
    ret=isdouble(d,ptr);
  }
  free(buf);
  return ret;
}

/* ***********************************************************************
 *- shell sort of a array of integer vectors using index array
 * ----------------------------------------------------------------------
 *i Inputs:
 *i   m     :number of components in iarray
 *i   n     :number of elements in iarray
 *i   iarray:array to be sorted
 *o Outputs:
 *o   iwk: sorted index into array 
 * ----------------------------------------------------------------------
 *
 * ***********************************************************************
 */
void iwshell(long m,long n,const long *iarray,long *iwk)
{
  
  
  long lognb2,i,j,k,l,n2,nn,it,mm,id;

  lognb2 = (long)(log((double)(n+1))*1.4426950);
  n2 = n;
  for( i = 0;i<n;i++) iwk[i] = i;
  

  for( nn = 0;nn< lognb2;nn++){
    n2 /= 2;
    k = n - n2;
    for( j = 0;j<k;j++){
      i = j ;
    L3:;
      l = i + n2;
      for( mm = 0;mm<m;mm++){

	id=(iarray[mm+iwk[l]*m] - iarray[mm+m*iwk[i]]) ;
	if (id>0) goto L11;
	if (id==0) goto L15;



	it = iwk[i];
	iwk[i] = iwk[l];
	iwk[l] = it;
	
	i -=  n2;
	if (i >= 0) goto L3;
	goto L11;
      L15:;
	
      }
    L11:;
    }
  }

  
}

/* ***********************************************************************
 *- shell sort of a array of integer vectors
 * ----------------------------------------------------------------------
 *i Inputs:
 *i   m     :number of components in iarray
 *i   n     :number of elements in iarray
 *i   iarray:array to be sorted
 *o Outputs:
 *o   iarray:array to be sorted
 * ----------------------------------------------------------------------
 *
 * ***********************************************************************
 */
void ishell(long m,long n,long *iarray)
{
  
  
  long lognb2,i,j,k,l,n2,nn,it,mm,mmm,id;

  lognb2 = (long)(log((double)(n+1))*1.4426950);
  n2 = n;
  for( nn = 0;nn< lognb2;nn++){
    n2 /= 2;
    k = n - n2;
    for( j = 0;j<k;j++){
      i = j ;
    L3:;
      l = i + n2;
      for( mm = 0;mm<m;mm++){

	id=(iarray[mm+l*m] - iarray[mm+m*i]) ;
	if (id<0) goto L16;
	if (id==0) goto L15;
	if (id>0) goto L11;
	
      L16:;
	for( mmm = 0;mmm<m;mmm++){
	  it = iarray[mmm+m*i];
	  iarray[mmm+m*i] = iarray[mmm+m*l];
	  iarray[mmm+m*l] = it;
	}
	i -=  n2;
	if (i >= 0) goto L3;
	goto L11;
      L15:;
	
      }
    L11:;
    }
  }

  
}


/* ***********************************************************************
 * piksrt of a long array, adapted form numerical recipies
 * ***********************************************************************
 */
void lpiksrt(long *iarr,long n)
{
  long j,ia,i;
  for(j=1;j<n;j++){
    ia=iarr[j];
    for(i=j-1;i>-1;i--){
      if(iarr[i] <= ia) goto l10;
      iarr[i+1]=iarr[i];
    }
    i=-1;
  l10: iarr[i+1]=ia;
  }
}

/* ***********************************************************************
 * iarr is sorted, uniq gives 1..nap non equal elements
 * ***********************************************************************
 */

void luniq(long *iarr,long n,long *nap)
{
  long ia,i,j;
  if(n==0){ *nap=0; return;}
  ia=iarr[0];
  i=0;
  for(j=1;j<n;j++){
    if (iarr[j] > ia ){
      ia=iarr[j];
      i++;
      iarr[i]=iarr[j];
    }
  }
  for(j=i+1;j<n;j++){
    iarr[j]=0;
  }
  *nap=i+1;

}     


/* ***********************************************************************
 * strncpy with terminating zero
 * ***********************************************************************
 */
char* strncpy0(char *dest,const char *src,size_t count)
{
  char *tmp;
  tmp=strncpy(dest,src,count);
  dest[count]='\0';
  return tmp;
}

/* ***********************************************************************
 * strncat with terminating zero
 * ***********************************************************************
 */
char* strncat0(char *dest,const char *src,size_t count)
{
  char *tmp;
  tmp=strncat(dest,src,count);
  dest[count]='\0';
  return tmp;
}

/* ***********************************************************************
 * remove newline 
 * ***********************************************************************
 */
void chop(char *p)
{
  long l;
  if(p==NULL)return;
  l=strlen(p)-1;
  if(l<0)return;
  if(*(p+l)=='\n') *(p+l)='\0';
}

/* ***********************************************************************
 *
 * ret: 0 if pathname does not exist
 * ret: 1 if pathname does exist and is a regular file
 * ret: 2 if pathname does exist and is dir
 * ret: 3 if pathname does exist and something else
 * ret: 4 is symlink , switched off, since not used and not as portable
 * ***********************************************************************
 */
long pathExists(const char *pathname)
{
  struct stat buf;
  if(stat(pathname, &buf)!=0)
    return 0;
  
  if (S_ISDIR(buf.st_mode) )return 2;
  if (S_ISREG(buf.st_mode) )return 1;
/*   if (S_ISLNK(buf.st_mode) )return 4; */
  return 3;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void removetrailingspace(char *s)
{
  char *p=s+strlen(s);

  while(1){
    if(*p==' ' || *p=='\t' ){
      *p='\0';
    }else if(*p!='\0') break;
    
    if(p==s)break;
    p--;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long getcwdir(char **cwd)
{

  size_t size=200;
  *cwd=NULL;
  
 next:;
  size+=200;
  if(((*cwd)=reallocate(*cwd,sizeof(char)*(size+1),"cwd","getcwdir"))==NULL){
    (void)sprintf(errorbuffer,"%s\n",errormsg);
    return 0;
  }
  
  if(getcwd((*cwd),size)==NULL){
    (void)printf("to small: %ld\n",(long)size);
    goto next;
  }
  
  return 1;
}

/* ***********************************************************************
 * return i0 and i1 such that txt[i0] is the first nad txt[i1] is the
 * last non whitespace character of txt
 * ***********************************************************************
 */
void lenTextStripWhitespace(const char *txt,long *i0,long *i1)
{
  long i,n;
  n=strlen(txt);
  for(i=n-1;i>-1;i--){	
    if((*(txt+i) !=' ') &&  *(txt+i)!='\t' && *(txt+i) !='\0') break;
  }	
  
  *i1=i;
  for(i=0;i<*i1;i++){
    if(*(txt+i) != ' ' && *(txt+i) != '\t' && *(txt+i) !='\0')break;
  }
  *i0=i;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void splitpath(char *dir,char *file,const char *path)
{
  char *p;
  (void)strcpy(dir,path);
  p=dir+strlen(dir);
  file[0]='\0';

  while(p!=dir){
    if(*p != '/') {
      p--;
    }else{
      *p='\0';
      strcpy(file,p+1);
      break;
    }
  }
  if(p==dir){
    if(path[0]!='/'){
      (void)strcpy(dir,".");
      (void)strcpy(file,path);
    }else{
      dir[0]='\0';
      (void)strcpy(file,path+1);
    }
  }
  return ;
}

/* ***********************************************************************
 * return char* to position of delimit in src but ignore protected 
 * delimiter ('\'+delimit)
 * ***********************************************************************
 */
char *strposprotect(const char *src,const char delimit)
{
  char *ret;
  char tmp[2];
  tmp[0]=delimit;
  tmp[1]='\0';
  ret=strstr(src,tmp);
  if(ret!=NULL){
    if(ret != src){
      if (*(ret-1) == '\\')
	ret=strposprotect(ret+1,delimit);
    }
  }
  return ret;
}

/* ***********************************************************************
 *
 * create tmp name   file will be allocated, free it afterwards 
 * ***********************************************************************
 */
/* long aloctmpfile(char **file,char *emsg) */
/* { */
/*   char *p; */
/*   if((p=tmpnam(NULL))==NULL){ */
/*     sprintf(emsg,"internal(aloctmpfile): can't create temporary filename !\n"); */
/*     return 0; */
/*   } */
/*   if((*file=(char*)allocate(sizeof(char)*(strlen(p)+1),"file","aloctmpfile")) */
/*      ==NULL){ */
/*     strcpy(emsg,errormsg); */
/*     return 0; */
/*   } */
/*   (void)strcpy(*file,p); */
/*   return 1; */
/* } */
/************************************************************************ 
 * compare if str2 is in str1
 * if casesensitive=1 casesensitive
 * return= 1 if coincide
 *       = 0 if not
 ************************************************************************ 
 */



char *strstri(const char* haystack,const char *needle,long casesensitive)
{
  long lh,ln;
  char *ret=NULL;
  char *ph,*pn,*p;
  const char *po;
  if(haystack == NULL || needle == NULL ) return ret;
  lh=strlen(haystack);
  ln=strlen(needle);

  

  if(!casesensitive){
    po=haystack;
    ph=(char*)allocate(sizeof(char)*(lh+1),"ph","strstri");
    pn=(char*)allocate(sizeof(char)*(ln+1),"pn","strstri");

    p=ph; while( (*p++=tolower(*haystack++)) != '\0' ) ;
    p=pn; while( (*p++=tolower(*needle++)) != '\0') ;
    

    ret=strstr(ph,pn);   

    if(ret!=NULL){
      ret=(char*)po+(ret-ph);
    }
    
    FREEE(ph,"p1");ph=NULL;
    FREEE(pn,"pn");pn=NULL;

  }else{
    ret=strstr(haystack,needle);
  }
  return ret;
}

/* ***********************************************************************
 * how many tokens in line
 * ***********************************************************************
 */

long ntoken(const char *line)
{
  const char *pch;
  long n;
  char *str;

  str=(char*)allocate(sizeof(char)*(strlen(line)+1),"str","ntoken");
  
  pch=line+strspn(line," \t");
  n=0;
  while(sscanf(pch,"%s",str)!=EOF){
    n++;
    pch+=strcspn(pch," \t");
    pch+=strspn(pch," \t");
  };
  FREEE(str,"str");str=NULL;
  return n;
}

/* ***********************************************************************
 * scan words separated by whitespace 
 * ***********************************************************************
 */



char *sscan(char **src)
{
  char *end;
  char *tmp=*src;
  char *ret;


  /* return null  */
  if(*src==NULL){
    /* errinwin("internal(sscan): argument is NULL!",""); */
    printf("internal(sscan): argument is NULL!");
    fprintf(stderr,"internal(sscan): argument is NULL!");
    exit(1);
  }
    



  /* search first non whitespace token */
  ret=*src;
  while( (*ret)!='\0' && (*ret == ' ' || *ret == '\t'))
    ret++;
  


  tmp=ret;
  /* next whitespace */
  while( (*tmp)!='\0' && (*tmp == ' ' || *tmp == '\t'))
    tmp++;
  /* one after word */
  end=tmp;
  while( (*end)!='\0' && *end != ' ' && *end != '\t')
    end++;

  
  *src=end;

  if(**src != '\0')
    {
      **src='\0';
      (*src)++;
    }
  if(*tmp=='\0') ret=tmp;
  return ret;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
/* const char *mytmpname() */
/* { */
/*   return tmpnam(NULL); */
/* } */
/*EOF*/
