/* @(#)Tester.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _TESTER_HH
#define _TESTER_HH 1

#include <QThread>
#include <QMutex>
#include <QWaitCondition>
#include <stdio.h>
#include "MyMath.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class Tester:public QThread{
  Q_OBJECT;
public:
  Tester(QObject *parent):QThread(parent),restart(false),pabort(false){};
  ~Tester(){ abort(); };

  void workOnIt(){
    QMutexLocker locker(&mutex);

    if (!isRunning()) {
      start(LowestPriority);
    } else {
      restart=true; //flag a abort + restart
      condition.wakeOne(); // wake it up
    }
  };

public slots:
  void interrupt(){
    abort();
    emit progress(0.);
    emit aborted();
  };
signals:
  void resultReady();
  void aborted();
  void progress(double);
protected:
  virtual void run(){
    printf("Tester:run\n");

    forever{
      printf("Tester:restart\n");
      mutex.lock();
      //read class data
      mutex.unlock();
      long N=100000;
      long subdiv=(MyMath::max<long>(1,N/(100/5)));

      for(long i=0;i<N;i++){
	printf("Tester:i=%ld\n",i);
	if(restart) break;
	if(pabort) return;
	if((i % subdiv)==0){
	  //myDebug()<<"progress"<<1.*i/N;
	  emit progress(1.*i/N);
	}
      }

      if(!restart) {
	emit progress(0.);
	emit resultReady();
      }

      mutex.lock();
      if(!restart) condition.wait(&mutex);
      restart = false;
      mutex.unlock();

    }

  };
private:
  void abort(){
    printf("Tester::abort()\n");
    mutex.lock();
    pabort=true; // request abort
    condition.wakeOne(); // make sure the thread is running to process
    mutex.unlock();
    
    wait(); // wait for run() to return
  };
protected:
private:
  QMutex mutex;
  QWaitCondition condition;
  bool restart;
  bool pabort;
};


#endif /* _TESTER_HH */

