//todo: syncronize all dialogs also with fileNew
//todo: char[] buffers
//todo: =.bands3 slow after box move
//todo: remaingin dialogs
//todo: modified and changed logics

//todo: line stipple depends on width?

/*************************************************************************
 * @(#)graphview.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Oct 2003
 *************************************************************************
 */

#include <QClipboard>
#include <QMessageBox>
#include <QResizeEvent>
#include <QMouseEvent>
#include <QPaintEvent>
#include <QMenu>
#include "ParameterDocument.hh"
#include "DataFileDialog.hh"
#include "Config.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Page.hh"
#include "AntiAliasing.hh"
#include "XFPStatusBar.hh"
#include "ParamDriver.hh"
#include "MainWindow.hh"
#include "Graph.hh"
#include "GraphAxis.hh"
#include "Graphs.hh"
#include "QtPaintEngine.hh"
#include "Assorted.hh"
#include "IOControl.hh"
#include "graphview.hh"
#include "MenuData.hh"
Q_DECLARE_METATYPE(MenuData)
#include "defs.hh"
#include "Except.hh"
#include "MyMath.hh"
#include "FileLoader.hh"
#include "Xfp.hh"
#include "Shape.hh"
#include "FunctionGroup.hh"
#include "LegendBox.hh"
#include "TextBox.hh"
#include "ViewBox.hh"
#include "World.hh"
#include "WorldDlg.hh"
#include "GraphDlg.hh"
#include "ViewBoxDlg.hh"
#include "LegendBoxDlg.hh"
#include "TextBoxDlg.hh"
#include "ShapeDlg.hh"
#include "AxesDlg.hh"
#include "TicMarksDlg.hh"
#include "TransformDlg.hh"
#include "SetDlg.hh"
#include "Point.hh"

#include "iofunc.hh"
#include "MyDebug.hh"





// class Action: public QAction
// {
// public:
//   Action(QWidget *parent):QAction(parent){
//     myDebug()<<"Action"<<this<<"lives";
//   }
//   Action(const QString &text,QWidget *parent):QAction(text,parent){
//     myDebug()<<"Action"<<this<<"lives";
//   }
//   ~Action(){
//     myDebug()<<"Action"<<this<<"dies";
//   }
// };





/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphView::GraphView( MainWindow * parent, Qt::WFlags f)
  : QWidget(parent,f),pstatusbar(0),ppage(new Page())
  ,p_framewidth(1),pdevicesize(400,400)
  ,p_picx0(0),p_picy0(0),p_sizeisdirty(true)
  ,p_offset(Point(p_picx0+p_framewidth*3,p_picy0+p_framewidth*3))
  ,pgobject(0)
  ,p_editpum(0)
  ,p_datapum(0)
  ,piofc(new IOControl())
  ,piobc(new IOControl())
  ,piobwc(new IOControl())
  ,pioopt(new IOControl())
  ,pioak(new IOControl())
  ,pparameterdocument(new ParameterDocument(this))
  ,worlddlg(0)
  ,graphdlg(0)
  ,viewboxdlg(0)
  ,legendboxdlg(0)
  ,textboxdlg(0)
  ,shapedlg(0)
  ,axesdlg(0)
  ,ticmarksdlg(0)
  ,transformdlg(0)
  ,setdlg(0)
  ,pdragdescr(new DragDescriptor(5.))
  ,pfileloader(0),pautoscalefuncmode(false)
{
  //  NSF("GraphView");
  
  //QPR("START");
  
  //setBackgroundMode(NoBackground);
  //setBackgroundMode(Qt::PaletteBackground);


  //setPaletteBackgroundColor (  QColor(0,255,0) );
  setMouseTracking( TRUE );
  buffer=new QImage(lRound(pdevicesize.w),lRound(pdevicesize.h),
		    QImage::Format_RGB32);
  
  p_crosscursor.setShape(Qt::CrossCursor);
  p_arrowcursor.setShape(Qt::ArrowCursor);
  p_sizeallcursor.setShape(Qt::SizeAllCursor); 
  p_sizebdiagcursor.setShape(Qt::SizeBDiagCursor); 
  p_pointinghandcursor.setShape(Qt::PointingHandCursor); 
  p_sizefdiagcursor.setShape(Qt::SizeFDiagCursor); 
      
  setCursor(p_arrowcursor);



  
  

  piobc->setFileName("+band");
  piobwc->setFileName("+bweights");
  pioopt->setFileName("+optbands");
  pioak->setFileName("+akbl");

  lastimportdir=QDir::current();
  myDebug()<<"lastimportdir"<<lastimportdir.path();

  setContextMenuPolicy(Qt::DefaultContextMenu);

  parent->installEventFilter(this);

  {
    QObject *obj=this;
    while(obj){
      if(obj->inherits("QMainWindow")){
	pstatusbar= dynamic_cast<XFPStatusBar*>(((QMainWindow*)obj)->statusBar());
      }
      obj=obj->parent();
    }
  }

  pstatusbar->setCurrentGraph(ppage->graphs()->currentGraph());
    
  //QPR("END");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::setup()
{
  p_editpum=buildMainMenu("Edit");
  emit managePopupMenu(QString("edit"),p_editpum);

  p_datapum=new QMenu("&Data",this);
  p_datapum->setEnabled(false);
  emit managePopupMenu(QString("data"),p_datapum);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphView::~GraphView()
{
  NSF("~GraphView");

  //QPR("START");

  if(buffer) delete buffer; buffer=0;

  if(ppage) delete ppage; ppage=0;
  if(pdragdescr) delete pdragdescr;pdragdescr=0;

  if(!worlddlg.isNull()) delete worlddlg;worlddlg=0;
  if(!graphdlg.isNull()) delete graphdlg;graphdlg=0;
  if(!viewboxdlg.isNull()) delete viewboxdlg;viewboxdlg=0;
  if(!legendboxdlg.isNull()) delete legendboxdlg;legendboxdlg=0;
  if(!textboxdlg.isNull()) delete textboxdlg;textboxdlg=0;
  if(!shapedlg.isNull()) delete shapedlg;shapedlg=0;
  if(!axesdlg.isNull()) delete axesdlg;axesdlg=0;
  if(!ticmarksdlg.isNull()) delete ticmarksdlg;ticmarksdlg=0;
  if(!transformdlg.isNull()) delete transformdlg;transformdlg=0;
  if(!setdlg.isNull()) delete setdlg;setdlg=0;

  if(piofc)delete piofc;piofc=0;
  if(piobc)delete piobc;piobc=0;
  if(piobwc)delete piobwc;piobwc=0;
  if(pioopt)delete pioopt;pioopt=0;
  if(pioak)delete pioak;pioak=0;



  if(p_editpum) delete p_editpum;
  if(p_datapum) delete p_datapum;
  if(pfileloader) delete pfileloader;pfileloader=0;
  if(pparameterdocument) delete pparameterdocument;pparameterdocument=0;
  //QPR("END");
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::calcPicSize()
{
  //NSF("calcPicSize");
  pdevicesize=Size(width()-2*p_framewidth*3,height()-2*p_framewidth*3);
  
  
  if(ppage){
    double ratio=pdevicesize.h/pdevicesize.w;

    double w=ppage->paper().paperWidth();
    double h=ppage->paper().paperHeight();;
    
    double paperratio=h/w;
    if(ratio>paperratio){
      pdevicesize.h=MyMath::min<long>(MyMath::max<long>(lRound(pdevicesize.w*paperratio),0),pdevicesize.h);
      p_picx0=0;
      p_picy0=MyMath::max<long>(lRound((height()-2*p_framewidth*3-pdevicesize.h)/2.),0);

      p_picx0=0;
      p_picy0=0;
      
    }else{
      pdevicesize.w=MyMath::min<long>(MyMath::max<long>(lRound(pdevicesize.h/paperratio),0),pdevicesize.w);
      p_picx0=MyMath::max<long>(lRound((width()-2*p_framewidth*3-pdevicesize.w)/2.),0);
      p_picy0=0;

      p_picx0=0;
      p_picy0=0;

    }
    p_offset.x=p_picx0+p_framewidth*3;
    p_offset.y=p_picy0+p_framewidth*3;
  }
  
  pdragdescr->devicesize=pdevicesize;
}

 /* ***********************************************************************
 * paint event, copy bakup pixmap into window
 * ***********************************************************************
 */
void GraphView::paintEvent(QPaintEvent*event)
{
  NSF("paintEvent");
  //myDebug() << nsubfunc;

  QWidget::paintEvent( event );

  initSceene(0L);





  if(1){
    QPainter wp(this);
    qDrawWinPanel(&wp
		  ,p_picx0,p_picy0
		  , pdevicesize.w+2*p_framewidth*3,pdevicesize.h+2*p_framewidth*3
		  , palette(), true
		  , 0 ) ;
  }

  
  // draw drag object
  QImage tempbuffer(*buffer);
  QtPaintEngine *pe=new QtPaintEngine(&tempbuffer,antiAliasing.isOn());
  pe->init(ppage->paper());
  
  if(ppage->graphs()->currentGraph())
    ppage->graphs()->currentGraph()->drawCurrentMarkers(pe);

  if(pgobject && pdragdescr->drawdragmarker ){
    ppage->plotDragObject(pe,pgobject);
  }

  delete (pe);
  QPainter wp(this);
  wp.drawImage(p_offset.x,p_offset.y,tempbuffer);

}

/* ***********************************************************************
 * resiye backup pixmap and redraw sceene if window size changed
 * ***********************************************************************
 */
void GraphView::resizeEvent(QResizeEvent * e)
{

  QWidget::resizeEvent(e);
  updatePageSize();

}/* ***********************************************************************
  *
  * ***********************************************************************
  */
void GraphView::updatePageSize()
{
  p_sizeisdirty=true;
  calcPicSize();

  if(pdragdescr->drawdragmarker){
    stopDragging(false);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::initSceene(long force)
{
  //  NSF("initSceene");
  //  QPR("");
  
  if(p_sizeisdirty){
    if(buffer) delete buffer;
    buffer=new QImage(lRound(pdevicesize.w),lRound(pdevicesize.h),
		       QImage::Format_RGB32);
    buffer->fill(0xffffffff);
  }

  if(p_sizeisdirty || force){
    drawIt();
  }
  if(p_sizeisdirty){
    p_sizeisdirty=false;
  }
  return;

}



/* ***********************************************************************
 * draw the sceene
 * ***********************************************************************
 */

void GraphView::drawIt()
{
  NSF("drawIt");
  //myDebug() << nsubfunc;
  if(!ppage)return; 
  if( buffer->isNull()) return;
  

  buffer->fill(0xffffffff);
  
  QtPaintEngine *pe=new QtPaintEngine(buffer,antiAliasing.isOn());
  pe->init(ppage->paper());
  try{
    ppage->plot(pe);
  }catch(Except &ex){
    QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			 ,QMessageBox::Ok | QMessageBox::Default,0,0);

  }
  delete (pe);
  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::mousePressEvent(QMouseEvent *me)
{
  NSF("GraphView::mousePressEvent");
  //myDebug() << nsubfunc;


  if(pdragdescr->dragging){
    pdragdescr->dragging=false;
    setCursor(p_crosscursor);
  }
  
  if(me->button()== Qt::LeftButton){
    if(pautoscalefuncmode){
      autoScaleFromFunc(me->pos());
      return;
    }
    Point pm=Point(me->pos());
    if(pgobject){
      pdragdescr->mousepoint=pm-p_offset;
      if(pdragdescr->drawdragmarker &&
	 pgobject->insideRegion(*pdragdescr,true)
	 ==GraphObject::GORNone){
	stopDragging();
	return;
      }else{
	startDragObject( Point(me->pos()) );
	return;
      }
    }
    {
      //Point pmrel=limit(pm-Point(1,1)-p_offset,pdragdescr->devicesize);
      Point pmrel=pm-0*Point(1,1)-p_offset;
      if(pmrel.insideRect(Point(),pdragdescr->devicesize)){
	Graph *gr=ppage->graphs()->currentGraph();
	if(gr){
	  Point w=gr->world()->dctowc(pmrel/pdragdescr->devicesize);
	  onSaveCurrentCoordinate(w);
	}
      }
    }
  }  
  
//   if(me->button()== Qt::LeftButton){
//     if(pgobject){
//       // stop waiting for drag actions
//       stopDragging();
//       return;
//     }
//   }
  if(me->button()== Qt::RightButton){
    
    if(pgobject){
      // stop waiting for drag actions
      stopDragging();
      return;
    }
    contextMenu(me,false);
    return;
  } // right button

  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::autoScaleFromFunc(const QPoint &pos)
{
  QList<Graph*> graphlist;
  QHash<Graph*,QList<GraphObject*> >graphhash;
  GraphObject *dbclickgo;
  
  getObjectList(pos,graphlist,graphhash,&dbclickgo);
  
  if(!(dbclickgo && 
       dbclickgo->objectType()==GraphObject::GOTFunctionOrGroup)) return;
  Graph *gr=dbclickgo->graph();
  gr->autoScaleFunc((FGObject*)dbclickgo);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  setMouseHints(mapFromGlobal(QCursor::pos()));
  refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::mouseMoveEvent(QMouseEvent *me)
{
  NSF("mouseMoveEvent");
  //myDebug()<<nsubfunc;
  
  if( buffer->isNull()) return;


  Point pm=Point(me->pos());

  
  Point pmrel=limit(pm-0*Point(1,1)-p_offset,pdragdescr->devicesize);
  Graph *cgr=ppage->graphs()->currentGraph();
  if(cgr){
    Point w=cgr->world()->dctowc(pmrel/pdragdescr->devicesize);
    
    pstatusbar->setCurrentCoordinates(w);
  }

  setMouseHints(me->pos());
  

  if(me->buttons()== Qt::LeftButton){
      
    if(pdragdescr->dragging && pgobject){
      Point pmrel=limit(pm-p_offset,pdragdescr->devicesize);
      pdragdescr->mousepoint=pmrel;
      pgobject->updateDrag(*pdragdescr);
      repaint();
    }
    return;
  }
  
  if(me->buttons()== Qt::NoButton){
    if(pgobject){
      dragStateOnMove(pm);
    }
  }
  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::setMouseHints(const QPoint &pos)
{
  QList<Graph*> graphlist;
  QHash<Graph*,QList<GraphObject*> >graphhash;
  GraphObject *dbclickgo;
  getObjectList(pos,graphlist,graphhash,&dbclickgo);
  QString lefthint="";
  QString righthint;
  QString dbclckhint;
  bool ondragobject=false;
  if(pgobject){
    DragDescriptor dd=*pdragdescr;
    dd.mousepoint=Point(pos)-p_offset;
    ondragobject=(pgobject->insideRegion(dd,false)!=GraphObject::GORNone);
  }
  if(dbclickgo && ( 
		   (dbclickgo->objectType()==GraphObject::GOTTextBox)||
		   (dbclickgo->objectType()==GraphObject::GOTLegendBox)||
		   (dbclickgo->objectType()==GraphObject::GOTShape)||
		   (dbclickgo->objectType()==GraphObject::GOTFunctionOrGroup)
		   )){
    if(ondragobject){
      if(pautoscalefuncmode){
	if(dbclickgo &&
	   (dbclickgo->objectType()==GraphObject::GOTFunctionOrGroup)){
	  dbclckhint="";
	}else{
	  dbclckhint="Props";
	}
      }else{
	dbclckhint="Cancel";
      }
    }else{
      dbclckhint="Props";
    }
    
  }else{
    dbclckhint="toggle<br/>current graph";
  }
  if(pgobject){
    if(ondragobject){
      lefthint="drag";
    }else{
      lefthint="Cancel";
    }
    righthint="Cancel";
  }else{
    righthint="Context";
  }
  if(pautoscalefuncmode){ 
    if(dbclickgo &&(dbclickgo->objectType()==GraphObject::GOTFunctionOrGroup)){
      lefthint="autoscale<br/>func";
    }else{
      lefthint="";
    }
  }

  if(pdragdescr->dragging){
    lefthint="";
    dbclckhint="";
  }
  pstatusbar->setHint("left",lefthint);
  pstatusbar->setHint("leftdbl",dbclckhint);
  pstatusbar->setHint("right",righthint);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::mouseReleaseEvent(QMouseEvent *)
{
  //NSF("GraphView::mouseReleaseEvent");
  //myDebug() << nsubfunc;

  if(pdragdescr->dragging){
    pdragdescr->dragging=false;
    finishDragObject();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::mouseDoubleClickEvent(QMouseEvent *me)
{
  //  NSF("GraphView::mouseDoubleClickEvent");
  //  myDebug() << nsubfunc;
  if(me->button()!= Qt::LeftButton) return;
  if(pgobject && ! pautoscalefuncmode){
    // stop waiting for drag actions
    stopDragging();
    return;
  }

  contextMenu(me,true);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startDragObject(const Point &pmi)
{
  //NSF("startDragObject");
  if(pgobject && !pdragdescr->dragging){
    pdragdescr->mousepoint=pmi-p_offset;
    pdragdescr->dragging=true;
    pdragdescr->drawdragmarker=true;
    pstatusbar->setHint("left","");
    pstatusbar->setHint("leftdbl","");
    pgobject->initDrag(*pdragdescr);
    repaint();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::dragStateOnMove(const Point &pm)
{
  NSF("dragStateOnMove");
  //  myDebug()<<nsubfunc<<pautoscalefuncmode;
//   if(pautoscalefuncmode){
//     setCursor(p_crosscursor);
//     //    return;
//   }
  if(!pgobject) return ;
  
  pdragdescr->mousepoint=pm-p_offset;
  GraphObject::GraphObjectRegion region
    =pgobject->insideRegion(*pdragdescr,true);
  
  //myDebug()<<region;
  if(region==GraphObject::GORBody){
    if(pgobject->objectType()==GraphObject::GOTWorld){
      setCursor(p_crosscursor);
    }else{
      setCursor(p_pointinghandcursor);
    }
  }else if(
	   (region==GraphObject::GORUL) ||
	   (region==GraphObject::GORUR) ||
	   (region==GraphObject::GORLL) ||
	   (region==GraphObject::GORLR) ){
    setCursor(p_sizeallcursor);
  }else{
    setCursor(p_crosscursor);
  }
  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::refresh(bool graphmarkeronly)
{
  //  NSF("refresh");
  pstatusbar->setCurrentGraph(ppage->graphs()->currentGraph());

  initSceene(!graphmarkeronly);
  repaint();

}
// /* ***********************************************************************
//  *
//  * ***********************************************************************
//  */
// void GraphView::reloadDialogs()
// {
//   if(!textboxdlg.isNull())textboxdlg->reloadDialog();
//   if(!shapedlg.isNull())shapedlg->reloadDialog();
//   if(!legendboxdlg.isNull())legendboxdlg->reloadDialog();
//   if(!viewboxdlg.isNull())viewboxdlg->reloadDialog();
//   if(!worlddlg.isNull())worlddlg->reloadDialog();
//   if(!graphdlg.isNull())graphdlg->reloadDialog();
//   if(!ticmarksdlg.isNull())ticmarksdlg->reloadDialog();
//   if(!setdlg.isNull())setdlg->reloadDialog();
//   //todo others
// }
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::setDialogsCurrentGraph(bool updateGraphDlg,bool reset)
{
  emit reloadWorldStack();
  
  Graph *gr=ppage->graphs()->currentGraph();
  if(reset)gr=0;
  if(gr){

    if(!viewboxdlg.isNull()) viewboxdlg->setData(gr->viewBox());
    if(!worlddlg.isNull())   worlddlg->setData(gr->world());
    if(!legendboxdlg.isNull()) legendboxdlg->setData(gr->legendBox());
    
    if(!textboxdlg.isNull()) textboxdlg->setData(gr->textBoxes(),0);
    if(!shapedlg.isNull()) shapedlg->setData(gr->shapes(),0);

    if(!axesdlg.isNull()) axesdlg->setData(gr->axes());
    
    if(!ticmarksdlg.isNull()) ticmarksdlg->setData(gr,gr->ticMarks());
    
    if(!setdlg.isNull()) setdlg->setData(ppage,0);

    if(updateGraphDlg && !graphdlg.isNull()) 
      graphdlg->setData(ppage->graphs(),gr); 

  }else{
    
    if(!viewboxdlg.isNull()) viewboxdlg->setData(0);
    if(!worlddlg.isNull()) worlddlg->setData(0);
    if(!legendboxdlg.isNull()) legendboxdlg->setData(0);

    if(!textboxdlg.isNull()) textboxdlg->setData(0,0);
    if(!shapedlg.isNull()) shapedlg->setData(0,0);
    if(!axesdlg.isNull()) axesdlg->setData(ppage->graphs()->currentGraph()->axes());

    if(!ticmarksdlg.isNull()) ticmarksdlg->setData(0,0);
    
    if(!setdlg.isNull()) setdlg->setData(ppage,0);
    
  }
}

/* ***********************************************************************
 * return 1 on clean state
 * ***********************************************************************
 */
long GraphView::onExit()
{
  if(!ppage->isdirty1) return 1;
  switch( QMessageBox::warning(this,"Warning"
			       ,"Document changed!"
			       ,"&Save changes"
			       ,"&Forget changes"
			       ,"Cancel"
			       ,0
			       ,2 )) {
  case 0:
    if(save(0)) return 1; else return 0;
    break;
  case 1:
    return 1;
    break;
  case 2:
    {
      return 0;
      break;
    }
  }

  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importData()
{
  //NSF("importData");
  if(!ppage) return ;
  
  
  piofc->setDataType(Function::XY);
  QString filename;
  long ret=QDialog::Accepted;
  DataFileDialog *dlg=new DataFileDialog(DataFileDialog::XNY
					 ,this,"Load XNY data"
					 ,piofc,ppage
					 ,lastimportdir
					 ," All ( * ) ;; "
					 " Fplo output ( +* ) ;; "
					 " Fplo input ( =* ) ") ;
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
    lastimportdir=dlg->directory();
  }
  delete(dlg);

  if(!filename.isNull() && ret==QDialog::Accepted){
    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();
    Graph *gr=graphs->getExistingGraphOrNew1(piofc->targetGraphId());
    graphs->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(graphs->currentGraph());
    if(piofc->dataType()==Function::GRIDX_Y_NZ){
      try{
	readGridFuncs(gr,filename,piofc);
      }catch(Except &ex){
	QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			     ,QMessageBox::Ok | QMessageBox::Default,0,0);
      }
    }else{//  if(datatype==DATATYPE_XNY){
      
      try{
	readXYFuncs(gr,filename,piofc);
      }catch(Except &ex){
	QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			     ,QMessageBox::Ok | QMessageBox::Default,0,0);
      }
    }
    if(cgr!=gr){
      setDialogsCurrentGraph();
    }else{
      if(!setdlg.isNull()) setdlg->setData(ppage,0);
    }
    if(!worlddlg.isNull() && gr==graphs->currentGraph()){
      gr->world()->setUpdateRequest(GraphObject::GOCXY);
      worlddlg->updateRequest(gr->world());
    }
    refresh();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void readAkbl(Graph *gr,const QString &filename,IOControl *pioak)
{
  IOControl iofc;
  iofc.setAutoScale(pioak->autoScale());
  iofc.setDataType(Function::GRIDX_Y_NZ);
  iofc.setAutoColor(pioak->autoColor());
  iofc.setAutoSymbol(false);
  iofc.setClearGroup(true);
  iofc.setTargetGroupId(-1);
  
  FunctionGroup  *group=readGridFuncs(gr,filename,&iofc);
  
  double ddum=0;
  readFploPointData(gr,filename,group,0L,ddum,ddum);
  
  
  autoBPAdjustment(gr,pioak);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::import(bool dorefresh,const QString &filename,
		       Xfp::FileType type,
			long &funcindex,
			long targetgraphid,long targetgroupid)
{
  //do not call from withing the menues and such

//   myDebug()<<filename;
//   myDebug()<<type;
  if(filename.isNull())return ;
  
  //todo iofc iobc and correct errors if loading fails, like missing haeder
  // if no band file
  Graphs *graphs=ppage->graphs();
  Graph *cgr=graphs->currentGraph();

  Graph *gra=graphs->getExistingGraphOrNew1(targetgraphid);

  if(type==Xfp::FTXYNZ){
    try{
      piofc->setTargetGroupId(targetgroupid);
      piofc->setTargetGraphId(targetgraphid);
      readGridFuncs(gra,filename,piofc);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTXNY){
    try{
      piofc->setTargetGroupId(targetgroupid);
      piofc->setTargetGraphId(targetgraphid);
      piofc->setAutoColorIndex(funcindex);
      readXYFuncs(gra,filename,piofc);
      funcindex=piofc->autoColorIndex();
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTXYNW){
    piofc->setTargetGroupId(targetgroupid);
    piofc->setDataType(Function::XYNW);
    try{
      piofc->setAutoColorIndex(funcindex);
      readXYFuncs(gra,filename,piofc);
      funcindex=piofc->autoColorIndex();
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTBand){
    piobc->setTargetGraphId(targetgraphid);
    try{
      readBandData(gra,filename,piobc);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTAkbl){
    pioak->setTargetGraphId(targetgraphid);
    try{
      readAkbl(gra,filename,pioak);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTBandWeight){
    piobwc->setTargetGraphId(targetgraphid);
    piobwc->setDataType(Function::XYNW);

    try{
      IOFunc io;
      connect(&io,SIGNAL(progress(int)),pstatusbar,SLOT(setProgressInt(int)));
      connect(&io,SIGNAL(statusMesg(const String &)),pstatusbar,
	      SLOT(setMessage(const String &)));
      io.readBandWeightData(gra,filename,piobwc);
      
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }else if(type==Xfp::FTBandMatel){
    pioopt->setTargetGraphId(targetgraphid);
    pioopt->setDataType(Function::XYMatel);
    try{
      IOFunc io;
      connect(&io,SIGNAL(progress(int)),pstatusbar,SLOT(setProgressInt(int)));
      connect(&io,SIGNAL(statusMesg(const String &)),pstatusbar,
	      SLOT(setMessage(const String &)));
      io.readBandWeightData(gra,filename,pioopt);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
  }  
  if(cgr!=gra){
    setDialogsCurrentGraph();
  }

  if(!worlddlg.isNull()){
    gra->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gra->world());
  }
  if(dorefresh)
    refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String GraphView::playParamDocument(bool showMessages,bool debugparser)
{
  String ret="";
  ParamDriver driver;
  driver.setTraceParsing(debugparser);
  driver.setTraceScanning(false);
  driver.setPage(this->page(),this);
  Graphs *graphs=ppage->graphs();
  Graph *cgr=graphs->currentGraph();
  try{
    driver.parse(pparameterdocument->textDocument()->toPlainText(),true);
  }catch(Except &ex){
    ret=ex.msg();
    if(showMessages)
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
  }
  if(driver.needRefresh()){
    refreshPage();
  }
  pstatusbar->setCurrentGraph(graphs->currentGraph());
  if(cgr!=graphs->currentGraph()){
    setDialogsCurrentGraph();
  }
  return ret;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::playParamDocument(const QString &fname,bool debugparser)
{
  ParamDriver driver;
  driver.setTraceParsing(debugparser);
  driver.setTraceScanning(false);
  driver.setPage(this->page(),this);
  Graphs *graphs=ppage->graphs();
  Graph *cgr=graphs->currentGraph();
  try{
    driver.parse(fname);
  }catch(Except &ex){
    QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			 ,QMessageBox::Ok | QMessageBox::Default,0,0);
  }
  if(driver.needRefresh()){
    refreshPage();
  }
  pstatusbar->setCurrentGraph(graphs->currentGraph());
  if(cgr!=graphs->currentGraph()){
    setDialogsCurrentGraph();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importBandData()
{
  //NSF("importBandData");
  if(!ppage) return ;
  
  
  QString filename;
  long ret=QDialog::Accepted;
  DataFileDialog *dlg=new DataFileDialog(DataFileDialog::BAND
					 ,this,"Load Band data"
					 ,piobc,ppage
					 ,lastimportdir
					 ," Fplo band data ( +*band* ) ;; "
					 " All ( * ) ;; ");
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
    lastimportdir=dlg->directory();
  }
  delete(dlg);
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();
    Graph *gr=graphs->getExistingGraphOrNew1(piobc->targetGraphId());
    graphs->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(graphs->currentGraph());

    try{
      readBandData(gr,filename,piobc);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }

    if(cgr!=gr){
      setDialogsCurrentGraph();
    }else{
      if(!setdlg.isNull()) setdlg->setData(ppage,0);
    }
    if(!worlddlg.isNull()){
      gr->world()->setUpdateRequest(GraphObject::GOCXY);
      worlddlg->updateRequest(gr->world());
    }
    if(!ticmarksdlg.isNull()){
      if(piobc->adjustXTics()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"adjustxtics");
      }
      if(piobc->EFTic()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"eftic");
      }
      ticmarksdlg->updateRequest(gr,gr->ticMarks(),"grouptics");
      
    }
    if(!textboxdlg.isNull()){
      TextBox *tb;
      if(piobc->hideXLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTXAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCActive);
	textboxdlg->updateRequest(tb);
      }
      if(piobc->defaultYLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTYAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCText);
	textboxdlg->updateRequest(tb);
      }
    }
    refresh();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importBandWeightData()
{
  //NSF("importBandData");
  if(!ppage) return ;
  
  piobwc->setDataType(Function::XYNW);
  
  QString filename;
  long ret=QDialog::Accepted;
  DataFileDialog *dlg=new DataFileDialog(DataFileDialog::BAND
					 ,this,"Load Band Weights Data"
					 ,piobwc,ppage
					 ,lastimportdir
					 ," Fplo band weight data ( +*bw* ) ;; "
					 " All ( * ) ;; ");
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
    lastimportdir=dlg->directory();
  }
  delete(dlg);
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();
    Graph *gr=graphs->getExistingGraphOrNew1(piobwc->targetGraphId());
    graphs->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(graphs->currentGraph());

    try{
      IOFunc io;
      connect(&io,SIGNAL(progress(int)),pstatusbar,SLOT(setProgressInt(int)));
      connect(&io,SIGNAL(statusMesg(const String &)),pstatusbar,
	      SLOT(setMessage(const String &)));
      io.readBandWeightData(gr,filename,piobwc);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
    if(cgr!=gr){
      setDialogsCurrentGraph();
    }else{
      if(!setdlg.isNull()) setdlg->setData(ppage,0);
    }

    if(!worlddlg.isNull()){
      gr->world()->setUpdateRequest(GraphObject::GOCXY);
      worlddlg->updateRequest(gr->world());
    }
    if(!ticmarksdlg.isNull()){
      if(piobwc->adjustXTics()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"adjustxtics");
      }
      if(piobwc->EFTic()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"eftic");
      }
      ticmarksdlg->updateRequest(gr,gr->ticMarks(),"grouptics");
      
    }
    if(!textboxdlg.isNull()){
      TextBox *tb;
      if(piobwc->hideXLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTXAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCActive);
	textboxdlg->updateRequest(tb);
      }
      if(piobwc->defaultYLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTYAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCText);
	textboxdlg->updateRequest(tb);
      }
    }
    refresh();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importBandMatelData()
{
  //NSF("importBandMatelData");
  if(!ppage) return ;

  pioopt->setDataType(Function::XYMatel);
  
  
  QString filename;
  long ret=QDialog::Accepted;
  DataFileDialog *dlg=new DataFileDialog(DataFileDialog::BAND
					 ,this,"Load Band "
					 "Matrix element Data"
					 ,pioopt,ppage
					 ,lastimportdir
					 ," Fplo band matel data ( +*optband* ) ;; "
					 " All ( * ) ;; ");
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
    lastimportdir=dlg->directory();
  }
  delete(dlg);
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();
    Graph *gr=graphs->getExistingGraphOrNew1(pioopt->targetGraphId());
    graphs->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(graphs->currentGraph());

    try{
      IOFunc io;
      connect(&io,SIGNAL(progress(int)),pstatusbar,SLOT(setProgressInt(int)));
      connect(&io,SIGNAL(statusMesg(const String &)),pstatusbar,
	      SLOT(setMessage(const String &)));
      io.readBandWeightData(gr,filename,pioopt);

    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
    if(cgr!=gr){
      setDialogsCurrentGraph();
    }else{
      if(!setdlg.isNull()) setdlg->setData(ppage,0);
    }

    if(!worlddlg.isNull()){
      gr->world()->setUpdateRequest(GraphObject::GOCXY);
      worlddlg->updateRequest(gr->world());
    }
    if(!ticmarksdlg.isNull()){
      if(pioopt->adjustXTics()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"adjustxtics");
      }
      if(pioopt->EFTic()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"eftic");
      }
      ticmarksdlg->updateRequest(gr,gr->ticMarks(),"grouptics");
      
    }
    if(!textboxdlg.isNull()){
      TextBox *tb;
      if(pioopt->hideXLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTXAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCActive);
	textboxdlg->updateRequest(tb);
      }
      if(pioopt->defaultYLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTYAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCText);
	textboxdlg->updateRequest(tb);
      }
    }
    refresh();
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importAkblData()
{
  //NSF("importAkblData");
  if(!ppage) return ;
  
  
  QString filename;
  long ret=QDialog::Accepted;
  DataFileDialog *dlg=new DataFileDialog(DataFileDialog::BAND
					 ,this
					 ,"Load Bloch spectral density data"
					 ,pioak,ppage,
					 lastimportdir
					 ," Fplo bloch sepctral density"
					 " data ( +akbl* ) ;; "
					 " All ( * ) ;; ");
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
    lastimportdir=dlg->directory();
  }
  delete(dlg);
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();
    Graph *gr=graphs->getExistingGraphOrNew1(pioak->targetGraphId());
    graphs->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(graphs->currentGraph());

    readAkbl(gr,filename,pioak);
    if(cgr!=gr){
      setDialogsCurrentGraph();
    }else{
      if(!setdlg.isNull()) setdlg->setData(ppage,0);
    }

    if(!worlddlg.isNull()){
      gr->world()->setUpdateRequest(GraphObject::GOCXY);
      worlddlg->updateRequest(gr->world());
    }
    if(!ticmarksdlg.isNull()){
      if(pioak->adjustXTics()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"adjustxtics");
      }
      if(pioak->EFTic()){
	ticmarksdlg->updateRequest(gr,gr->ticMarks(),"eftic");
      }
      ticmarksdlg->updateRequest(gr,gr->ticMarks(),"grouptics");
      
    }
    if(!textboxdlg.isNull()){
      TextBox *tb;
      if(pioak->hideXLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTXAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCActive);
	textboxdlg->updateRequest(tb);
      }
      if(pioak->defaultYLabel()){
	tb=gr->textBoxes()->at(TextBox::TBTYAxisLabel);
	tb->setUpdateRequest(GraphObject::GOCText);
	textboxdlg->updateRequest(tb);
      }
    }
    refresh();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::autoScaleAll()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr) return;
  gr->autoScaleAll(Xfp::Graph_X);
  gr->autoScaleAll(Xfp::Graph_Y);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::autoScaleX()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  gr->autoScaleAll(Xfp::Graph_X,true);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::autoScaleY()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  gr->autoScaleAll(Xfp::Graph_Y,true);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::autoScaleFunc()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  pautoscalefuncmode=true;
  pgobject=gr->world();
  pgobject->setUpdateRequest(GraphObject::GOCXY);
  dragStateOnMove(mapFromGlobal(QCursor::pos()));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::zoomAll()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  pgobject=gr->world();
  pgobject->setUpdateRequest(GraphObject::GOCXY);
  dragStateOnMove(mapFromGlobal(QCursor::pos()));

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::zoomX()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  pgobject=gr->world();
  pgobject->setUpdateRequest(GraphObject::GOCX);
  dragStateOnMove(mapFromGlobal(QCursor::pos()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::zoomY()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  pgobject=gr->world();
  pgobject->setUpdateRequest(GraphObject::GOCY);
  dragStateOnMove(mapFromGlobal(QCursor::pos()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::zoomIn()
{
  //NSF("zoomIn");
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  gr->world()->zoom(Xfp::Graph_X,-0.1);
  gr->world()->zoom(Xfp::Graph_Y,-0.1);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::zoomOut()
{
  //NSF("zoomOut");
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();  
  if(!gr)return;
  
  gr->world()->zoom(Xfp::Graph_X, 0.1);
  gr->world()->zoom(Xfp::Graph_Y, 0.1);
  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::ruler()
{
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  pgobject=gr->world();
  pgobject->setUpdateRequest(GraphObject::GOCMeasureXY);
  dragStateOnMove(mapFromGlobal(QCursor::pos()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onWorldChanged()
{
  //NSF("worldChanged");
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;

  if(!worlddlg.isNull()){
    gr->world()->setUpdateRequest(GraphObject::GOCXY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::moveRight(){moveWorld(Xfp::Graph_X,0.1);}
void GraphView::moveUp(){moveWorld(Xfp::Graph_Y,0.1);}
void GraphView::moveDown(){moveWorld(Xfp::Graph_Y,-0.1);}
void GraphView::moveLeft(){moveWorld(Xfp::Graph_X,-0.1);}

void GraphView::moveWorld(Xfp::XYAxisType xy,double shift)
{
  //NSF("moveLeft");
  if(!ppage)return;
  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;

  gr->world()->shiftXY(xy,shift);
  if(!worlddlg.isNull()){
    if(xy==Xfp::Graph_X)
      gr->world()->setUpdateRequest(GraphObject::GOCX);
    else
      gr->world()->setUpdateRequest(GraphObject::GOCY);
    worlddlg->updateRequest(gr->world());
  }
  refresh();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startWorldDlg(GraphObject *go)
{
  if(!worlddlg.isNull()) {
    worlddlg->activateWindow();
    worlddlg->raise();
    return;
  }
  worlddlg=new WorldDlg(0,this,"WorldDlg");
  connect(worlddlg,SIGNAL(acceptRequest(GraphObject *,bool))
	  ,this,SLOT(onDlgAccept(GraphObject *,bool)));
  //connect(worlddlg,SIGNAL(endRequest()),this,SLOT(onWorldDlgEnd()));

  worlddlg->setData(go);
  worlddlg->show();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startGraphDlg(Graph *gr)
{
  if(!graphdlg.isNull()) {
    graphdlg->activateWindow();
    graphdlg->raise();
    return;
  }
  graphdlg=new GraphDlg(0,this,"GraphDlg");
  connect(graphdlg,SIGNAL(acceptRequest(Graph *,bool))
	  ,this,SLOT(onGraphDlgAccept(Graph *,bool)));

  graphdlg->setData(ppage->graphs(),gr);
  graphdlg->show();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startViewBoxDlg(GraphObject *go)
{
  if(!viewboxdlg.isNull()) {
    viewboxdlg->activateWindow();
    viewboxdlg->raise();
    return;
  }
  viewboxdlg=new ViewBoxDlg(0,this,"ViewBoxDlg");
  connect(viewboxdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	  ,this,SLOT(onDlgAccept(GraphObject *,bool)));

  viewboxdlg->setData(go);
  viewboxdlg->show();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startLegendBoxDlg(GraphObject *go)
{
  if(!legendboxdlg.isNull()) {
    legendboxdlg->activateWindow();
    legendboxdlg->raise();
    legendboxdlg->setData(go);
    return;
  }
  legendboxdlg=new LegendBoxDlg(0,this,"LegendBoxDlg");
  connect(legendboxdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	  ,this,SLOT(onDlgAccept(GraphObject *,bool)));

  legendboxdlg->setData(go);
  legendboxdlg->show();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startTextBoxDlg(GraphObject *go)
{
  if(!textboxdlg.isNull()) {
    textboxdlg->activateWindow();
    textboxdlg->raise();
  }else{
    textboxdlg=new TextBoxDlg(0,this,"TextBoxDlg");
    connect(textboxdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    textboxdlg->show();
  }
  Graph *gr=0;
  if(!go){
    gr=ppage->graphs()->currentGraph();
  }else{
    gr=go->graph();
  }

  if(gr){
    textboxdlg->setData(gr->textBoxes(),(TextBox*)go);
  }else{
    textboxdlg->setData(0,0);
  }
    
  
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startShapeDlg(GraphObject *go)
{
  if(!shapedlg.isNull()) {
    shapedlg->activateWindow();
    shapedlg->raise();
  }else{
    shapedlg=new ShapeDlg(0,this,"ShapeDlg");
    connect(shapedlg,SIGNAL(acceptRequest(GraphObject *,bool))
	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    shapedlg->show();
  }

  Graph *gr=0;
  if(!go){
    gr=ppage->graphs()->currentGraph();
  }else{
    gr=go->graph();
  }
  
  if(gr){
    shapedlg->setData(gr->shapes(),(Shape*)go);
  }else{
    shapedlg->setData(0,0);
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startAxesDlg(GraphObject *go)
{
  if(!axesdlg.isNull()) {
    axesdlg->activateWindow();
    axesdlg->raise();
  }else{
    axesdlg=new AxesDlg(0,this,"AxesDlg");
    connect(axesdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    axesdlg->show();
  }

  Graph *gr=0;
  if(!go){
    gr=ppage->graphs()->currentGraph();
  }else{
    gr=go->graph();
  }
  
  if(gr){
    axesdlg->setData(gr->axes());
  }else{
    axesdlg->setData(0);
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startTicMarksDlg(GraphObject *)
{
  if(!ticmarksdlg.isNull()) {
    ticmarksdlg->activateWindow();
    ticmarksdlg->raise();
  }else{
    ticmarksdlg=new TicMarksDlg(0,this,"TicMarksDlg");
    connect(ticmarksdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    ticmarksdlg->show();
  }
  if(ppage->graphs()->currentGraph())
    ticmarksdlg->setData(ppage->graphs()->currentGraph(),
			 ppage->graphs()->currentGraph()->ticMarks());
  else
    ticmarksdlg->setData(0,0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startTransformDlg(GraphObject *)
{
  if(!transformdlg.isNull()) {
    transformdlg->activateWindow();
    transformdlg->raise();
  }else{
    transformdlg=new TransformDlg(0,this,"TransformDlg");
    connect(transformdlg,SIGNAL(acceptRequest(GraphObject *,bool))
 	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    transformdlg->show();
  }

  transformdlg->setData(this,pparameterdocument);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::startSetDlg(FGObject *fgo)
{
  if(!setdlg.isNull()) {
    setdlg->activateWindow();
    setdlg->raise();
  }else{
    setdlg=new SetDlg(0,this,"SetDlg");
    connect(setdlg,SIGNAL(acceptRequest(GraphObject *,bool))
	    ,this,SLOT(onDlgAccept(GraphObject *,bool)));
    setdlg->show();
  }
  setdlg->setData(ppage,fgo);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onDlgAccept(GraphObject *go,bool forceChanged)
{
//   myDebug()<<"sender"<<sender();
//   myDebug()<<"forceChanged"<<forceChanged;
//   myDebug()<<"go"<<go;
//   if(go)myDebug()<<"go->changed"<<go->changed();
  if(!go)go=ppage->graphs()->currentGraph();
  if((go && go->changed()) ||forceChanged) {
    refresh();
    if(go) go->setChanged(false);
    if(sender()==transformdlg){
      setDialogsCurrentGraph(true);
      transformdlg->activateWindow();
      transformdlg->raise();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onGraphDlgAccept(Graph *gr,bool currentChanged)
{
  //myDebug()<<gr<<currentChanged<<ppage->graphs()->currentGraph();
  if(ppage->graphs()->currentGraph()==0 && gr){
    //myDebug()<<"here 1";
    // previous was no graph, now new graph
    ppage->graphs()->setCurrentGraph(gr);
    pstatusbar->setCurrentGraph(ppage->graphs()->currentGraph());
    setDialogsCurrentGraph(false);

  }
  if(currentChanged){
    pstatusbar->setCurrentGraph(ppage->graphs()->currentGraph());
    setDialogsCurrentGraph(false);
  }

  refresh();
  if(!viewboxdlg.isNull()) {
    if(ppage->graphs()->currentGraph()){
      ViewBox *vb=ppage->graphs()->currentGraph()->viewBox();
      vb->setUpdateRequest(GraphObject::GOCSize);
      viewboxdlg->updateRequest(vb);
    }
  }

  if(ppage->graphs()->currentGraph()){
    ppage->graphs()->currentGraph()->setChanged(false);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::buildViewMenu(QMenu *inmenu,GraphObject *go
			      ,bool creatsubmenu,bool proponly)
{
  QMenu *submenu=inmenu;
  if(creatsubmenu) submenu=inmenu->addMenu("&View");
  QAction *a;
  if(!proponly){
    a=submenu->addAction("&Move/Resize",this,SLOT(popups()));
    
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::ViewBox
						     ,MenuData::Move)));
  }
  
  a=submenu->addAction((proponly?"&View":"&Properties"),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::ViewBox
						   ,MenuData::Prop)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::buildLegendMenu(QMenu *inmenu,GraphObject *go
				,bool creatsubmenu,bool proponly)
{
  QMenu *submenu=inmenu;
  if(creatsubmenu) submenu=inmenu->addMenu("&Legend");
  QAction *a;

  if(!creatsubmenu && !proponly){
    a=submenu->addAction("Legend");
    a->setEnabled(false);
    QFont font=a->font();
    font.setWeight(QFont::Black);
    a->setFont(font);
    submenu->addSeparator();
  }
  if(!proponly){
    a=submenu->addAction("&Move",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::LegendBox
						     ,MenuData::Move)));
    a=submenu->addAction("&Hide",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::LegendBox
					    ,MenuData::Hide)));
  }
  a=submenu->addAction((proponly?"&Legend":"&Properties"),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::LegendBox
						   ,MenuData::Prop)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::buildShapeMenu(QMenu *inmenu,GraphObject *go
			       ,bool creatsubmenu,bool incontext)
{
  QMenu *submenu=inmenu;
  String name="Sh&ape";
  String subname="Shape";
  if(go){
    Shape *sh=(Shape*)go;
    name=sh->guiName();
    subname=sh->guiName();
  }
  QAction *a;
  if(creatsubmenu){
    submenu=inmenu->addMenu(name());
  }else if(incontext){
    a=submenu->addAction(subname());
    a->setEnabled(false);
    QFont font=a->font();
    font.setWeight(QFont::Black);
    a->setFont(font);
    submenu->addSeparator();
  }
    
  if(incontext){
    a=submenu->addAction("&Move/Resize",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::Shape
						     ,MenuData::Move)));
    a=submenu->addAction("&Delete",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::Shape
						     ,MenuData::Delete)));
  }
  a=submenu->addAction((!incontext?(name+"s")():"&Properties"),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::Shape
						   ,MenuData::Prop)));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::buildTextBoxMenu(QMenu *inmenu,GraphObject *go)
{
  QMenu *submenu=inmenu->addMenu("&TextBoxes");
  QAction *a=submenu->addAction("&New",this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::TextBox
						   ,MenuData::New)));

  Graph *gr=ppage->graphs()->currentGraph();
  if(!gr)return;
  submenu->addSeparator();
  TextBox *tb=gr->textBoxes()->at(TextBox::TBTTitle);

  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::Title
						   ,MenuData::Prop)));
  tb=gr->textBoxes()->at(TextBox::TBTSubTitle);
  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::SubTitle
						   ,MenuData::Prop)));
  tb=gr->textBoxes()->at(TextBox::TBTXAxisLabel);
  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::XAxisLabel
						   ,MenuData::Prop)));
  tb=gr->textBoxes()->at(TextBox::TBTYAxisLabel);
  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::YAxisLabel
						   ,MenuData::Prop)));
  tb=gr->textBoxes()->at(TextBox::TBTOppositeXAxisLabel);
  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::OppXAxisLabel
						   ,MenuData::Prop)));
  tb=gr->textBoxes()->at(TextBox::TBTOppositeYAxisLabel);
  a=submenu->addAction(tb->guiName(true)(),this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::OppYAxisLabel
						   ,MenuData::Prop)));
  a=submenu->addAction("&Others",this,SLOT(popups()));
  a->setData(QVariant::fromValue(MenuData(go,0,MenuData::TextBox
						   ,MenuData::Prop)));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::buildWorldMenu(QMenu *inmenu,GraphObject *go,bool mouseactions)
{
  QMenu *submenu=inmenu->addMenu("&World");
  QAction *a;
  if(mouseactions){
    a=submenu->addAction(QIcon(":Icons/zoomall"),"Zoom &All"
			 ,this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						     ,MenuData::ZoomXY)));
    a=submenu->addAction(QIcon(":Icons/zoomx"),"Zoom &X-only"
			 ,this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						     ,MenuData::ZoomX)));
    a=submenu->addAction(QIcon(":Icons/zoomy"),"Zoom &Y-only"
			 ,this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						     ,MenuData::ZoomY)));
  }
   a=submenu->addAction(QIcon(":Icons/autoscaleall"),"A&utoscale All"
			,this,SLOT(popups()));
   a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						     ,MenuData::AutoScaleXY)));
   a=submenu->addAction(QIcon(":Icons/autoscalex"),"Auto&scale X"
			,this,SLOT(popups()));
   a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						    ,MenuData::AutoScaleX)));
   a=submenu->addAction(QIcon(":Icons/autoscaley"),"Autosca&le Y"
			,this,SLOT(popups()));
   a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						    ,MenuData::AutoScaleY)));
   a=submenu->addAction("&Properties",this,SLOT(popups()));
   a->setData(QVariant::fromValue(MenuData(go,0,MenuData::World
						    ,MenuData::Prop)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::contextMenu(QMouseEvent *me,bool doubleclicked)
{
  
  QList<Graph*> graphlist;
  QHash<Graph*,QList<GraphObject*> >graphhash;
  GraphObject *dbclickgo;

  getObjectList(me->pos(),graphlist,graphhash,&dbclickgo);

  
  if(doubleclicked){

    
    

//     if(dbclickgo->objectType()==GraphObject::GOTViewBox)
//     if(dbclickgo->objectType()==GraphObject::GOTWorld)
//     if(dbclickgo->objectType()==GraphObject::GOTTicMarks)

    
    if(dbclickgo && dbclickgo->objectType()==GraphObject::GOTTextBox){
      if(pgobject && pautoscalefuncmode){
	stopDragging();
      }
      startTextBoxDlg(dbclickgo);
      goto cleanup;
    }else if(dbclickgo && dbclickgo->objectType()==GraphObject::GOTLegendBox){
      if(pgobject && pautoscalefuncmode){
	stopDragging();
      }
      startLegendBoxDlg(dbclickgo);
      goto cleanup;
    }else if(dbclickgo && dbclickgo->objectType()==GraphObject::GOTShape){
      if(pgobject && pautoscalefuncmode){
	stopDragging();
      }
      startShapeDlg(dbclickgo);
      goto cleanup;
    }else if(dbclickgo && dbclickgo->objectType()==
	     GraphObject::GOTFunctionOrGroup){
      if(!pautoscalefuncmode)
// 	autoScaleFromFunc(me->pos());
//       else
	startSetDlg((FGObject*)dbclickgo);
      goto cleanup;
      
    }

    if(pgobject && pautoscalefuncmode){
      stopDragging();
    }
    changeCurrentGraph(graphlist);  

  cleanup:
    return;
  }

  QMenu *menu=0;

  if(!graphlist.size()){
    menu=buildMainMenu("Context");
  }else{
    menu=new QMenu(this);

    QHash<Graph*,QMenu*> menus;
    if(graphlist.size()>1){
      foreach(Graph *g,graphlist){
 	QString name=QString("%1 %2").arg(g->guiName(true)())
 	  .arg((g==ppage->graphs()->currentGraph()?"[current]":""));
	menus[g]=menu->addMenu(name);
      }
    }
    
    QAction *a;
    for(long ig=0;ig<graphlist.size();ig++){
      Graph *g=graphlist[ig];
      foreach(GraphObject *go,graphhash[g]){
	QMenu *m=(graphlist.size()>1?menus[g]:menu);
	if( (go->objectType()==GraphObject::GOTViewBox) ){
	  buildViewMenu(m,go,true,false);
	}else if( (go->objectType()==GraphObject::GOTWorld) ){
	  buildWorldMenu(m,go,true);
	}else if((go->objectType()==GraphObject::GOTLegendBox) ){
	  buildLegendMenu(m,go,(graphhash[g].size()>1),false);
	}else if((go->objectType()==GraphObject::GOTFunctionOrGroup) ){
	  FGObject *fgo=(FGObject*)(go);
	  QMenu *submenu=m;
	  QString name="Set";
	  if(fgo->type()==FGObject::FGOTGroup)name="Group";
	  a=submenu->addAction(name,this,SLOT(popups()));
	  a->setData(QVariant::fromValue(MenuData(fgo,0,MenuData::FunctionOrGroup
						  ,MenuData::Prop)));
	}else if((go->objectType()==GraphObject::GOTTextBox) ){
	  TextBox *tb=(TextBox*)(go);
	  unsigned long sublevel=tb->type()+MenuData::Title;
	  QString move="&Move";
	  if(tb->pos()->restriction()){
	    move="&Offset";
	  }
	  QMenu *submenu=m;
	  if((graphhash[g].size()>1)){
	    submenu=menu->addMenu(tb->guiName(true)());
	  }else{
	    a=submenu->addAction(tb->guiName(true).removeFromSet("&")());
	    a->setEnabled(false);
	    QFont font=a->font();
	    font.setWeight(QFont::Black);
	    a->setFont(font);
	    submenu->addSeparator();
	  }
	  
	  
	  a=submenu->addAction(move,this,SLOT(popups()));
	  a->setData(QVariant::fromValue(MenuData(tb,0,sublevel
						  ,MenuData::Move)));
	  a=submenu->addAction("&Hide",this,SLOT(popups()));
	  a->setData(QVariant::fromValue(MenuData(tb,0,MenuData::TextBox
						  ,MenuData::Hide)));
	  
	  if(tb->type()==TextBox::TBTGeneral){
	    a=submenu->addAction("&Delete",this,SLOT(popups()));
	    a->setData(QVariant::fromValue(MenuData(tb,0,sublevel
						    ,MenuData::Delete)));
	    a=submenu->addAction("&Copy",this,SLOT(popups()));
	    a->setData(QVariant::fromValue(MenuData(tb,0,sublevel
						    ,MenuData::Copy)));
	  }
	  a=submenu->addAction("&Properties",this,SLOT(popups()));
	  a->setData(QVariant::fromValue(MenuData(tb,0,sublevel
						  ,MenuData::Prop)));
	}else if((go->objectType()==GraphObject::GOTShape) ){
	  buildShapeMenu(m,go,(graphhash[g].size()>1),true);
	}
	
      }//for all objects under the mouse
    }//for all different graphs with objects under the mouse
    
    
  }// more than world object

  //QAction *ar=
  menu->exec(me->globalPos());
//   if(ar)
//     myDebug("action=%x",ar->data().toInt());
//   myDebug()<<"menu "<<menu<<"dies";
  delete menu;


  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::getObjectList(const QPoint &me,QList<Graph*> &graphlist,
			      QHash<Graph*,QList<GraphObject*> > &graphhash,
			      GraphObject **dbclickgo)
{
  graphlist.clear();
  graphhash.clear();

  pdragdescr->mousepoint=Point(me)-p_offset;
  PtrArray<GraphObject> gobjects(false);
  ppage->getObjectsAt(gobjects,*pdragdescr);
  if(gobjects.size()==0){ return; }
  

  bool haveothers=false;
  Graph *gr=0;
  for(long i=0;i<gobjects.size();i++){
    GraphObject *go=gobjects[i];
    if(go->objectType()!=GraphObject::GOTWorld &&
       go->objectType()!=GraphObject::GOTViewBox){
      // if other then view and world under cursor
      // display the others only
      haveothers=true;
    }
    Graph *g=go->graph();;
    if(!gr){ 
      gr=g;
      graphlist.append(g);
    }else if(gr!=g){ 
      gr=g;
      graphlist.append(g);
    };
    graphhash[g].append(go);
  }

  bool firstsolidobject=false;
  for(long ig=graphlist.size()-1;ig>=0;ig--){
    Graph *graph=graphlist.at(ig);
    bool onlyworld=true;
    foreach(GraphObject *go,graphhash[graph]){
      if(go->objectType()!=GraphObject::GOTWorld){
	onlyworld=false;
	break;
      }
    }
    if(onlyworld){
      graphlist.takeAt(ig);
      continue;
    }
    if(haveothers){
      for(long i=graphhash[graph].size()-1;i>=0;i--){
	GraphObject *go=graphhash[graph][i];
	if( (go->objectType()==GraphObject::GOTWorld)||
	    (go->objectType()==GraphObject::GOTViewBox) ){
	  graphhash[graph].removeAt(i);
	}
      }
    }
    
    if(graphhash[graph].empty()){
      graphlist.takeAt(ig);
      continue;
    }
    if(!firstsolidobject){
      for(long i=graphhash[graph].size()-1;i>=0;i--){
	GraphObject *go=graphhash[graph][i];
	if( (go->objectType()!=GraphObject::GOTFunctionOrGroup) ){
	  firstsolidobject=true;
	}
      }
    }
     
    if(firstsolidobject){
      for(long i=graphhash[graph].size()-1;i>=0;i--){
	GraphObject *go=graphhash[graph][i];
	if( (go->objectType()==GraphObject::GOTFunctionOrGroup) ){
	  graphhash[graph].removeAt(i);
	}
      }
    }
  

  
    
    if(graphhash[graph].empty()){
      graphlist.takeAt(ig);
      continue;
    }

  }


  if(0){
    for(long ig=graphlist.size()-1;ig>=0;ig--){
      Graph *graph=graphlist.at(ig);
      myDebug()<<graph->guiName();
      foreach(GraphObject *go,graphhash[graph]){
	myDebug()<<"\t"<<go->printS()<<go;
      }
    }
    myDebug()<<"";
  }  

  (*dbclickgo)=0;
  if(graphlist.size()>0){
    Graph *g=graphlist[graphlist.size()-1];
    if(graphhash[g].size()>0){
      (*dbclickgo)=graphhash[g][graphhash[g].size()-1];
    }
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QMenu *GraphView::buildMainMenu(const QString &context)
{
  //myDebug()<<"buildMainMenu"<<context;
  QMenu *menu=new QMenu("&Edit",this);
  QAction *a;
  QMenu *m;
  bool proponly=true;
  bool creatsubmenu=false;
  ViewBox *vb=0;
  World *world=0;
  LegendBox *lb=0;
  Graph *cgr=ppage->graphs()->currentGraph();
  if(context=="Context"){
    m=menu->addMenu("&File");
    a=m->addAction("&Save",this, SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Save,0)));
    a=m->addAction("Save &As",this,SLOT(popups())); 
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::SaveAs,0)));
    a=m->addAction("&Open",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Open,0)));
    menu->addSeparator();
    creatsubmenu=true;
    proponly=false;
    vb=cgr->viewBox();
    lb=cgr->legendBox();
    world=cgr->world();
  }

  if(context=="Edit" || context=="Context"){
    
    buildViewMenu(menu,vb,creatsubmenu,proponly);
    buildWorldMenu(menu,world,!proponly);
    buildTextBoxMenu(menu,0);
    buildLegendMenu(menu,lb,creatsubmenu,proponly);
    buildShapeMenu(menu,0,false,false);

    
    a=menu->addAction("&Graph",this,SLOT(popups())); 
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Graph
						     ,MenuData::Prop)));

    a=menu->addAction("A&xes",this,SLOT(popups())); 
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Axes
						     ,MenuData::Prop)));

    a=menu->addAction("&Sets",this,SLOT(popups())); 
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Sets
					    ,MenuData::Appearance)));
    
    a=menu->addAction("Tics/&Marks",this,SLOT(popups()));
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::TicMarks
						     ,0)));

    a=menu->addAction("Script/T&ransformations",this,SLOT(popups()));
    a->setShortcut(Qt::CTRL+Qt::Key_E);
    a->setData(QVariant::fromValue(MenuData(0,0,MenuData::Transform
						     ,0)));
  }    
  return menu;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::popups()
{
  
  NSF("popups");
  
  //myDebug()<<nsubfunc;
  
  QAction *a=dynamic_cast<QAction*>(sender());
  //myDebug()<<a;
  if(!a) return;
  MenuData md=a->data().value<MenuData>();
  GraphObject *go=md.graphObject();
  long sublevel=md.subLevel();
  long subsublevel=md.subSubLevel();
  

  if(!ppage) return;

  Graph *gr1=ppage->graphs()->currentGraph();
  //if(!gr1)return;
  
  if(subsublevel==MenuData::Prop){
    switch (sublevel) {
    case MenuData::ViewBox:
      startViewBoxDlg((go?go:(gr1?gr1->viewBox():0)));
      break;
    case MenuData::World:
      startWorldDlg((go?go:(gr1?gr1->world():0)));
      break;
    case MenuData::Graph:
      startGraphDlg(gr1);
      break;
    case MenuData::Axes:
      startAxesDlg(gr1);
      break;
    case MenuData::LegendBox:
      startLegendBoxDlg((go?go:(gr1?gr1->legendBox():0)));
      break;
    case MenuData::FunctionOrGroup:
      startSetDlg((FGObject*)go);
      break;
    case MenuData::Title:
    case MenuData::SubTitle:
    case MenuData::XAxisLabel:
    case MenuData::YAxisLabel:
    case MenuData::OppXAxisLabel:
    case MenuData::OppYAxisLabel:
    case MenuData::TextBox:
      startTextBoxDlg((go?go:(gr1?
			      gr1->textBoxes()
			      ->textBoxByType((TextBox::TextBoxType)
					      (sublevel-MenuData::Title)):0)));
      break;
    case MenuData::Shape:
      startShapeDlg(go);
      break;
    }
  }else if(subsublevel==MenuData::ZoomXY){
    if(go){
      pgobject=go;
      pgobject->setUpdateRequest(GraphObject::GOCXY);
      dragStateOnMove(mapFromGlobal(QCursor::pos()));
    }
  }else if(subsublevel==MenuData::ZoomX){
    if(go){
      pgobject=go;
      pgobject->setUpdateRequest(GraphObject::GOCX);
      dragStateOnMove(mapFromGlobal(QCursor::pos()));
    }
  }else if(subsublevel==MenuData::ZoomY){
    if(go){
      pgobject=go;
      pgobject->setUpdateRequest(GraphObject::GOCY);
      dragStateOnMove(mapFromGlobal(QCursor::pos()));
    }
  }else if(subsublevel==MenuData::Hide){
    //myDebug()<<"subsublevel Hide"<<sublevel;
    if(sublevel==MenuData::LegendBox && go){
      ((LegendBox*)go)->setActive(false);
      refresh();
    }else if(sublevel==MenuData::TextBox && go){
      //myDebug()<<"hiding";
      ((TextBox*)go)->setActive(false);
      refresh();
    }    
  }else if(subsublevel==MenuData::Move){
    if(go){
      pgobject=go;
      pdragdescr->drawdragmarker=true;
      //      pdragdescr->dragging=true;
      pgobject->initDrag(*pdragdescr);
      repaint();
      dragStateOnMove(mapFromGlobal(QCursor::pos()));
    }
  }else if(subsublevel==MenuData::Copy){
    if(sublevel==MenuData::TextBox && go) {
      TextBox *tb=(TextBox*)go;
      if(!textboxdlg.isNull()){
	if(go->graph() && go->graph()==gr1){
	  textboxdlg->copyTextBox(tb);
	}else{
	  go->graph()->textBoxes()->copyTextBox(tb);
	}
      }else{
	if(go->graph()){
	  go->graph()->textBoxes()->copyTextBox(tb);
	}else{
	  gr1->textBoxes()->copyTextBox(tb);
	}
      }
      refresh();
    }
  }else if(subsublevel==MenuData::Delete){
    if(sublevel==MenuData::TextBox && go) {
      TextBox *tb=(TextBox*)go;
      if(!textboxdlg.isNull()){
	if(go->graph() && go->graph()==gr1){
	  textboxdlg->deleteTextBox(tb);
	}else{
	  go->graph()->textBoxes()->deleteTextBox(tb);
	}
      }else{
	if(go->graph()){
	  go->graph()->textBoxes()->deleteTextBox(tb);
	}else{
	  gr1->textBoxes()->deleteTextBox(tb);
	}
      }
      refresh();
    }else if(sublevel==MenuData::Shape && go) {
      Shape *sh=(Shape*)go;
      if(!shapedlg.isNull()){
	if(go->graph() && go->graph()==gr1){
	  shapedlg->deleteShape(sh);
	}else{
	  go->graph()->shapes()->deleteShape(sh);
	}
      }else{
	if(go->graph()){
	  go->graph()->shapes()->deleteShape(sh);
	}else{
	  gr1->shapes()->deleteShape(sh);
	}
      }
      refresh();
    }
  }else if(subsublevel==MenuData::New){
    if(sublevel==MenuData::TextBox) {
      if(gr1){
	TextBox *tb=gr1->textBoxes()->appendTextBox();
	if(!textboxdlg.isNull()) startTextBoxDlg(tb);
	refresh();
      }
    } 
  }else if(subsublevel==MenuData::AutoScaleXY){
    if(sublevel==MenuData::World){
      autoScaleAll();
      return;
    }
  }else if(subsublevel==MenuData::AutoScaleX){
    if(sublevel==MenuData::World){
      autoScaleX();
      return;
    }
  }else if(subsublevel==MenuData::AutoScaleY){
    if(sublevel==MenuData::World){
      autoScaleY();
      return;
    }
  }else if(subsublevel==MenuData::Appearance){
    switch(sublevel){
    case MenuData::Sets:
      {
	startSetDlg(0);
	break;
      }
    }
  }else{
    switch (sublevel) {
    case MenuData::TicMarks:
      {
	startTicMarksDlg(0);
	break;
      }
    case MenuData::Transform:
      {
	startTransformDlg(0);
	break;
      }
    case MenuData::SaveAs:
      save(1);
      break;
    case MenuData::Save:
      save(0);
      break;
    case MenuData::Open:
      load();
      break;
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::finishDragObject()
{
  if(!ppage) return;
  if(!pgobject) return;
  
  pgobject->setFromDrag(*pdragdescr);
  
  
  if(pgobject->objectType()==GraphObject::GOTWorld){
    pdragdescr->drawdragmarker=false;
    if(pgobject->updateRequest()==GraphObject::GOCMeasureXY){
      Point w0=pdragdescr->w0;
      Point w1=pdragdescr->w1;
      Point dw=w1-w0;
      String s=Sprintf(
		       "x0= %g\ny0= %g\n"
		       "x1= %g\ny1= %g\n"
		       "dx= %g\ndy= %g\n",
	     w0.x,w0.y,
	     w1.x,w1.y,
	     dw.x,dw.y
		       );
      QApplication::clipboard()->setText(s());
      emit measurementText(s());
    }else{
      if(!worlddlg.isNull()) worlddlg->updateRequest(pgobject);
    }
  }
  if(pgobject->objectType()==GraphObject::GOTViewBox){
    if(!viewboxdlg.isNull()) viewboxdlg->updateRequest(pgobject);
  }
  if(pgobject->objectType()==GraphObject::GOTLegendBox){
    if(!legendboxdlg.isNull()) legendboxdlg->updateRequest(pgobject);
  }
  if(pgobject->objectType()==GraphObject::GOTTextBox){
    if(!textboxdlg.isNull()) textboxdlg->updateRequest((TextBox*)pgobject);
  }
  if(pgobject->objectType()==GraphObject::GOTShape){
    if(!shapedlg.isNull()) shapedlg->updateRequest((Shape*)pgobject);
  }
  refresh();
  setMouseHints(mapFromGlobal(QCursor::pos()));
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphView::eventFilter(QObject *object, QEvent *event)
{
  if (object == parent() && event->type() == QEvent::KeyPress) {
    QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
    if (keyEvent->key() == Qt::Key_Escape) {
      if(pdragdescr->drawdragmarker)
	stopDragging();
      
      return true;
    }else{
      if(pgobject && pdragdescr->drawdragmarker){
	switch (keyEvent->key()){
	case Qt::Key_Up: 
// 	  myDebug()<<"UP";
// 	  pdragdescr->mousepoint=limit(pdragdescr->mousepoint-Point(0,1)
// 				       ,pdragdescr->devicesize);
	  //todo: the logic of stored points violates cursor movement
// 	  pgobject->updateDrag(pgraph->viewBox(),*pdragdescr);
// 	  repaint();

	default:
	  return false;
	}
      }
    }
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::stopDragging(bool forecerepaint)
{
  pautoscalefuncmode=false;
  pdragdescr->dragging=false;
  pdragdescr->drawdragmarker=false;
  if(forecerepaint) repaint();
  pgobject=0;
  setCursor(p_arrowcursor);

  setMouseHints(mapFromGlobal(QCursor::pos()));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::changeCurrentGraph(const QList<Graph*> &graphlist)
{

  Graph *cgr=ppage->graphs()->currentGraph();

  if(graphlist.size()==0){
    long j=ppage->graphs()->indexOf(cgr);
    j=(j<0?0:(j+1)% ppage->graphs()->size());
    ppage->graphs()->setCurrentGraph(ppage->graphs()->at(j));
  }else{
    long j=graphlist.indexOf(cgr);
    j=(j<0?0:(j+1)% graphlist.size());
    ppage->graphs()->setCurrentGraph(graphlist.at(j));
  }

  if(cgr!=ppage->graphs()->currentGraph()){
    setDialogsCurrentGraph();
    refresh(true);
  }
  pstatusbar->setCurrentGraph(ppage->graphs()->currentGraph());


  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void GraphView::toggleAntiA(bool a)
{
  antiAliasing.setAntiAliasing(a);
  antiAliasing.writeSettings();
  refresh();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::printPS()
{
  try{
    ppage->printPS();
  }catch(Except &ex){
    QMessageBox::warning(this,"Warning",ex.msg(),QMessageBox::Ok);
  }
  pstatusbar->setMessage(QString("printed %1")
			 .arg(ppage->printFileName()()),1000);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::killAll(bool dorefresh)
{
  ppage->reset();
  setDialogsCurrentGraph(true,true);
  if(dorefresh) refresh();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::refreshPage()
{
  updatePageSize();
  setDialogsCurrentGraph(true);
  refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::saveWorld()
{
  ppage->pushCurrentWorld();
  emit reloadWorldStack();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onSaveCurrentCoordinate(const Point &w)
{
  
  if(!ppage)return;

  if(!ppage->mouseHookBuffer().empty()){

    Graphs *graphs=ppage->graphs();
    Graph *cgr=graphs->currentGraph();

    FILE *FO;
    FO=fopen("","");
    try{
      FO=openforwrite("+currentpoint");
    }catch(...){return;}
    fprintf(FO,"%30.15g %30.15g \n",w.x,w.y);
    emit measurementText(Sprintf("x=%30.15g\ny=%30.15g \n",w.x,w.y)());
    fclose(FO);
    
    ppage->setCursor(w);
    
    ParamDriver driver;
    driver.setTraceParsing(false);
    driver.setTraceScanning(false);
    driver.setPage(ppage,this);
    try{
      driver.parse(ppage->mouseHookBuffer(),true);
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
    }
    refreshPage();
    
    pstatusbar->setCurrentGraph(graphs->currentGraph());
    if(cgr!=graphs->currentGraph()){
      setDialogsCurrentGraph();
    }
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::importSomething()
{
  //myDebug()<<"start GraphView::importSomething()";
  emit iAmBusy(true);
  pfileloader=new FileLoader(this);
  connect(pfileloader,SIGNAL(resultReady()),this,SLOT(onFileLoaderReady()));
  connect(pfileloader,SIGNAL(progress(double)),
	  pstatusbar,SLOT(setProgress(double)));
  connect(pfileloader,SIGNAL(aborted()),this,SLOT(onFileLoaderAborted()));
  connect(pstatusbar,SIGNAL(interruptRequest()),
	  pfileloader,SLOT(interrupt()));

  pfileloader->workOnIt();

  // myDebug()<<"end GraphView::importSomething()";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onFileLoaderReady()
{
  printf("job done!\n");
  emit iAmBusy(false);
  if(pfileloader) delete pfileloader;pfileloader=0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphView::onFileLoaderAborted()
{
  printf("job aborted!\n");
  emit iAmBusy(false);
  if(pfileloader) delete pfileloader;pfileloader=0;
}

/*EOF*/ 
