/* @(#)graphview.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef _GRAPHVIEW_H
#define _GRAPHVIEW_H 1


#include <stdio.h>
#include <QDir>
#include <QPointer>
#include <QWidget>
#include "Point.hh"
#include "Graph.hh"
#include "Xfp.hh"
#include "FString.hh"


class QPoint;
class IOControl;
class DlgBase;
class DragDescriptor;
class FGObject;
class GraphObject;
class LegendBox;
class TextBox;
class SetDlg;
class TextBoxDlg;
class ShapeDlg;
class TicMarksDlg;
class TransformDlg;
class GraphDlg;
class AxesDlg;
class ViewBox;
class World;
class QMenu;
class Page;
class MainWindow;
class XFPStatusBar;
class FileLoader;
class ParameterDocument;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class GraphView: public QWidget 
{
  Q_OBJECT
  public:
  GraphView( MainWindow * parent = 0, Qt::WFlags f = 0);
  virtual ~GraphView();
/*   void setGraph(Graph *pg){pgraph=pg;}; */
/*   struct Graph *graph(){return pgraph;}; */
  struct Page *page(){return ppage;};
  virtual long save(long ){return 1;};
  virtual long print(long ){return 1;};
  virtual void load(){};
  virtual void loadOld(){};
  virtual void setup();
  virtual long onExit();
  virtual void refresh(bool graphmarkeronly=false);
  //  virtual void reloadDialogs();
  virtual void loadFile(const QString &filename,bool oldformat)=0;
  void printPS();

  void import(bool dorefresh,const QString &filename,Xfp::FileType type,
	       long &funcindex,
	       long targetgraphid=Graph::CurrentGraph,long targetgroupid=-1);
  void playParamDocument(const QString &fname,bool debugparser=false);

  void killAll(bool dorefresh=true);
  void refreshPage();

  void saveWorld();
  void updatePageSize();


  ParameterDocument *parameterDocument(){return pparameterdocument;};

signals:
  void managePopupMenu(const QString &hint, QMenu *pum);
  void reloadWorldStack();
  void measurementText(const QString&);
  void iAmBusy(bool);

public slots: 
 String playParamDocument(bool showMessages=true,bool debugparser=false);
 void importData();
 void importBandData(); 
 void importBandWeightData(); 
 void importBandMatelData();
 void importAkblData(); 
 void importSomething();
 void autoScaleAll();
 void autoScaleX();
 void autoScaleY();
 void autoScaleFunc();
 void zoomAll();
 void zoomX();
 void zoomY();
 void zoomIn();
 void zoomOut();
 void ruler();
 void moveLeft();
 void moveRight();
 void moveUp();
 void moveDown();
 void toggleAntiA(bool);
 void onWorldChanged();
 void onFileLoaderReady();
 void onFileLoaderAborted();
protected: 
  virtual void paintEvent(QPaintEvent*);
  virtual void resizeEvent(QResizeEvent * e);
  virtual void initSceene(long force);
  virtual void drawIt();
  virtual void mouseReleaseEvent(QMouseEvent *me);
  virtual void mouseDoubleClickEvent(QMouseEvent *me);
  virtual void mouseMoveEvent(QMouseEvent *me);
  virtual void mousePressEvent(QMouseEvent *me);
  bool eventFilter(QObject *object, QEvent *event);


 private slots:
   void popups();

   void onDlgAccept(GraphObject *,bool);
   void onGraphDlgAccept(Graph *,bool);
   void startWorldDlg(GraphObject *go);
   void startGraphDlg(Graph *gr);
   void startViewBoxDlg(GraphObject *go);
   void startLegendBoxDlg(GraphObject *go);
   void startTextBoxDlg(GraphObject *go);
   void startShapeDlg(GraphObject *go);
   void startAxesDlg(GraphObject *go);
   void startTicMarksDlg(GraphObject *go);
   void startSetDlg(FGObject *fgo);
   void startTransformDlg(GraphObject *go);


 private:
   QMenu *buildMainMenu(const QString &context);
   void dragStateOnMove(const QPoint &po){
     dragStateOnMove(Point(po.x(),po.y()));};
  void dragStateOnMove(const Point &pm);
  void startDragObject(const Point &pm);
  void finishDragObject();
  void stopDragging(bool repaint=true);
  void calcPicSize();
  void contextMenu(QMouseEvent *me,bool doubleclicked);
  void changeCurrentGraph(const QList<Graph*> &graphlist);

  void buildViewMenu(QMenu *inmenu,GraphObject *,bool creatsubmenu
		     ,bool proponly);
  void buildLegendMenu(QMenu *inmenu,GraphObject *go
		       ,bool creatsubmenu,bool proponly);
  void buildShapeMenu(QMenu *inmenu,GraphObject *go
		      ,bool creatsubmenu,bool incontext);
  void buildTextBoxMenu(QMenu *inmenu,GraphObject *go);
  void buildWorldMenu(QMenu *inmenu,GraphObject *go,bool mouseactions);
  void moveWorld(Xfp::XYAxisType xy,double shift);

  void setDialogsCurrentGraph(bool updateGraphDlg=true,bool reset=false);
  void onSaveCurrentCoordinate(const Point &w);
  void getObjectList(const QPoint &pos,QList<Graph*> &graphlist,
		     QHash<Graph*,QList<GraphObject*> > &graphhash,
		     GraphObject **dblickgo);
  void setMouseHints(const QPoint &pos);
  void autoScaleFromFunc(const QPoint &pos);
  
 protected:
  XFPStatusBar *pstatusbar;
  Page *ppage;
 private:
  /** backup buffer */
  QImage *buffer;
  long p_framewidth;
  Size pdevicesize;
  long p_picx0,p_picy0;
  QCursor p_crosscursor;
  QCursor p_arrowcursor;
  QCursor p_sizeallcursor;
  QCursor p_pointinghandcursor;
  QCursor p_sizebdiagcursor;
  QCursor p_sizefdiagcursor;
  QCursor p_prevcursor;
  bool p_sizeisdirty;
  Point p_offset;
  GraphObject *pgobject;
  QMenu *p_editpum;
 protected:
  QMenu *p_datapum;
 private:
  IOControl *piofc;
  IOControl *piobc;
  IOControl *piobwc;
  IOControl *pioopt;
  IOControl *pioak;
  QDir lastimportdir;
  ParameterDocument *pparameterdocument;
  
  QPointer<DlgBase> worlddlg;
  QPointer<GraphDlg> graphdlg; //todo: dlgbase ?
  QPointer<DlgBase> viewboxdlg;
  QPointer<DlgBase> legendboxdlg;
  QPointer<TextBoxDlg> textboxdlg; //todo: dlgbase ?
  QPointer<ShapeDlg> shapedlg; //todo: dlgbase ?
  QPointer<AxesDlg> axesdlg;//todo: dlgbase ?
  QPointer<TicMarksDlg> ticmarksdlg; //todo: dlgbase ?
  QPointer<TransformDlg> transformdlg;
  QPointer<SetDlg> setdlg; //todo: dlgbase ?

  DragDescriptor *pdragdescr;
  


  FileLoader *pfileloader;
  bool pautoscalefuncmode;
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _GRAPHVIEW_H */

