/*************************************************************************
 * @(#)iofunc.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Nov 2003
 *************************************************************************
 */

#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <math.h>
#include "iofunc.hh"
#include "Attributes.hh"
#include "GraphObject.hh"
#include "FString.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "Page.hh"
#include "allocread.h"
#include "Graph.hh"
#include "OptMatelDefinition.hh"
#include "defs.hh"
#include "World.hh"
#include "IOControl.hh"
#include "StringPtr.hh"
#include "FString.hh"
#include "Except.hh"
#include "Assorted.hh"
#include "TextBox.hh"
#include "TicMarks.hh"
#include "MyMath.hh"
#include "IOControl.hh"
#include "WeightLabels.hh"
#include "WeightSelection.hh"
#ifndef _DARWIN_COMPAT
#include <values.h>
#else
#include <float.h>
#endif
#include <limits.h>








/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Graph;

typedef long (*DATAPARSER_CALLBACK)(long dptype,const void *id,void *data
				    ,String &emsg); 
 

static long parse_datafile(FILE *FH,const String &fname,String &emsg
			   ,DATAPARSER_CALLBACK callback,void *cbdata);


/* ***********************************************************************
 *
 * ***********************************************************************
 */
#define DATAPARSE_INTERNAL_ERROR      0
#define DATAPARSE_LEX_ERROR          -1
#define DATAPARSE_CALLBACK_ERROR     -2
#define DATAPARSE_SUCCES              1

#define DP_INIT                1
#define DP_EXIT                2
#define DP_CLEAN               3
#define DP_COMMENT             4
#define DP_INSETCOMMENT        5
#define DP_NEWBLOCK            6
#define DP_LINEEND             7
#define DP_NUM                 8
#define DP_BLOCKEND            9


/* ***********************************************************************
 *
 * ***********************************************************************
 */

/* ***********************************************************************
 *
 * ***********************************************************************
 */

#define DATALEXER_ERROR        0
#define DATALEXER_NUM_ERROR    1
#define DATALEXER_EOF          2
#define DATALEXER_CHAR         3
#define DATALEXER_COMMENT      4
#define DATALEXER_NUM          5
#define DATALEXER_NEWLINE      6
#define DATALEXER_WHITESPACE   7
#define DATALEXER_NAN          8
#define DATALEXER_INF          9
#define DATALEXER_UNDEF        10

#define DATALEXREAD(buf,max_size,FH,emsg)\
if ( ((result=fread(buf,1,max_size, FH))==0)  && ferror(FH)  ){ \
  sprintf(emsg,"Input in data scanner failed\n");\
  return DATALEXER_ERROR;}\
data->buffer[result]='\0';\
data->p=data->buffer;\
data->readbytes+=result;



/* **************************************** */
#define BUFFER_SIZE 1024
typedef struct tslexdata{
  char buffer[BUFFER_SIZE+1];
  char *p;
  unsigned long readbytes;
}tsLEXDATA;
/* **************************************** */
typedef union tutoken{
  char id[10000+1];
  double dnum;
}tuTOKEN;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
#define DECPOINT 0x00000001
#define NUMBER   0x00000002
#define EXPONENT 0x00000004
#define END      0x00000008
static long datalexer(FILE *FH,tsLEXDATA *data
	       ,tuTOKEN *token,long *colnr,long *linenr,char *emsg)
{
  /* NSF("datalexer"); */
  long i;
  long result=0;
  long expect,decp,number;
  double d;
  char *err;
  int ret=DATALEXER_NUM;

  if(data->p[0]=='\0'){
    DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
    if(result==0) return DATALEXER_EOF;
  }


  
  if(data->p[0]=='+'){
    /* ******************** +  ***************** */
    i=0;
    token->id[i++]='+';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) goto unexpendnumerror;
    }
    decp=number=0;
    expect=NUMBER | DECPOINT;
    goto expnumber;
  }else if(data->p[0]=='-'){
    /* ******************** -  ***************** */
    i=0;
    token->id[i++]='-';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) goto unexpendnumerror;
    }
    decp=number=0;
    expect=NUMBER | DECPOINT;
    goto expnumber;
  }else if(data->p[0]=='.'){
    /* ******************** -  ***************** */
    i=0;
    token->id[i++]='.';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    decp=1;
    number=0;
    expect=NUMBER;
    goto expnumber;
  }else if(data->p[0]=='n' || data->p[0]=='N'){ 
    i=0;
    token->id[i++]='N';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    if(!(data->p[0]=='a' || data->p[0]=='A')) goto internalerror;
    
    token->id[i++]='a';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    if(!(data->p[0]=='n' || data->p[0]=='N')) goto internalerror;
    
    token->id[i++]='N';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    ret=DATALEXER_NAN;
    goto conver;
    
  }else if(strncasecmp(data->p,"infinity",8)==0){
    i=8;
    strcpy(token->id,"infinity");
    data->p+=i; (*colnr)+=i;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    ret=DATALEXER_INF;
    goto conver;
    
  }else if(strncasecmp(data->p,"undef",5)==0){
    i=5;
    strcpy(token->id,"infinity");
    data->p+=i; (*colnr)+=i;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    ret=DATALEXER_UNDEF;
    goto conver;
    
  }else if(strncasecmp(data->p,"inf",3)==0){
    i=3;
    strcpy(token->id,"inf");
    data->p+=i; (*colnr)+=i;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0)goto unexpendnumerror;
    }
    ret=DATALEXER_INF;
    goto conver;
    
  }else if(isdigit(data->p[0])){ 
    /* ******************** integer  ***************** */
    i=0;
    expect=NUMBER;
    decp=number=0;
    goto expnumber;
  }else if(data->p[0]==' ' || data->p[0]=='\t'){
    /* ******************** whitespace  ***************** */
    data->p++;
    (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) return DATALEXER_WHITESPACE;
    }
    while(data->p[0]==' ' || data->p[0]=='\t'){
      data->p++;
      (*colnr)++;
      if(data->p[0]=='\0'){
	DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	if(result==0) return DATALEXER_WHITESPACE;
      }
    }
    return DATALEXER_WHITESPACE;
  }else if(data->p[0]=='\n'){
    /* ******************** newline  ***************** */
    data->p++;
    (*linenr)++;
    (*colnr)=0;
    return DATALEXER_NEWLINE;
  }else if(data->p[0]=='#'){
    /* ******************** comment  ***************** */
    data->p++;
    (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) return DATALEXER_COMMENT;
    }
    i=0;
    while(data->p[0]!='\n'){
      token->id[i++]=data->p[0];
      data->p++;
      (*colnr)++;
      if(data->p[0]=='\0'){
	token->id[i]='\0';
	DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	if(result==0) return DATALEXER_COMMENT;
      }
    }
    token->id[i]='\0';
    if(data->p[0]=='\n'){
      (*linenr)++;
      (*colnr)=0;
      data->p++;
    }
    return DATALEXER_COMMENT;
  }

  goto internalerror;
  


 expnumber:
  if(strncasecmp(data->p,"infinity",8)==0){
    strcpy(token->id+i,"infinity");
    i=8;
    data->p+=i; (*colnr)+=i;
    ret=DATALEXER_INF;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) goto conver;
    }
    goto conver;
  }else if(strncasecmp(data->p,"inf",3)==0){
    strcpy(token->id+i,"inf");
    i=3;
    data->p+=i; (*colnr)+=i;
    ret=DATALEXER_INF;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) goto conver;
    }
    goto conver;
  }else if((expect & NUMBER)==NUMBER && isdigit(data->p[0])){ 
    /* ******************** integer  ***************** */
    token->id[i++]=data->p[0];
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0) goto conver;
    }
    while( isdigit(data->p[0])){ 
      token->id[i++]=data->p[0];
      data->p++;(*colnr)++;
      if(data->p[0]=='\0'){
	DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	if(result==0) goto conver;
      }
    } 
    token->id[i]='\0'; 
    if(decp){
      expect=EXPONENT;
    }else{
      expect=DECPOINT|EXPONENT;
    }
    number=1;
    if(data->p[0]==' ' || data->p[0]=='\t'|| data->p[0]=='\n'){
      goto conver;
    }

    goto expnumber;
  }else if( (expect & DECPOINT)==DECPOINT && !decp && data->p[0]=='.' ){ 
    token->id[i++]='.';
    data->p++; (*colnr)++;
    if(data->p[0]=='\0'){
      DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
      if(result==0){
	if(!number) goto unexpendnumerror; else goto conver; 
      }
    }
    if(number && (data->p[0]==' ' || data->p[0]=='\t'|| data->p[0]=='\n')){
      goto conver;
    };

    decp=1;
    if(!number){
      expect=NUMBER;
    }else{
      expect=NUMBER | EXPONENT;
    }
    goto expnumber;
  }else if( (expect & EXPONENT)==EXPONENT ){ 
    if( data->p[0]=='e' ||  data->p[0]=='E' || 
	data->p[0]=='d' || data->p[0]=='D' ){ 
      
      token->id[i++]='E';
      data->p++; (*colnr)++;
      if(data->p[0]=='\0'){
	DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	if(result==0) goto unexpendnumerror;
      }
      if(data->p[0]=='+'){
	token->id[i++]='+';
	data->p++; (*colnr)++;
	if(data->p[0]=='\0'){
	  DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	  if(result==0) goto unexpendnumerror;
	}
      }else if(data->p[0]=='-'){
	token->id[i++]='-';
	data->p++; (*colnr)++;
	if(data->p[0]=='\0'){
	  DATALEXREAD(data->buffer,BUFFER_SIZE,FH,emsg);
	  if(result==0) goto unexpendnumerror;
  	}
      }
      expect=NUMBER | END;
      goto expnumber;
    }else if(number && (data->p[0]==' ' || data->p[0]=='\t'
			|| data->p[0]=='\n')){
      goto conver;
    }else{
      goto numcharerror;
    }
  }else{
    goto numcharerror;
  }

  goto internalerror;

  
 conver:
  if(ret==DATALEXER_INF){
    if(token->id[0]=='-') token->dnum=-DBL_MAX;
    else token->dnum=DBL_MAX;
    return DATALEXER_INF;
  }
  if(ret==DATALEXER_UNDEF){
    if(token->id[0]=='-') token->dnum=0;
    else token->dnum=0;
    return DATALEXER_UNDEF;
  }
  token->id[i]='\0';
  d=strtod(token->id,&err);
  if (*err!='\0' ) {
    sprintf(emsg,"Number conversion error?!");
    return DATALEXER_ERROR;
  }
  if(ret==DATALEXER_NUM){
    token->dnum=d;
    return DATALEXER_NAN;
  }else{
    token->dnum=0.;
    return DATALEXER_NUM;
  }
  

  
 numcharerror:
  sprintf(emsg,"Unexpected character in number."); 
  return  DATALEXER_ERROR;
  
 unexpendnumerror:
  sprintf(emsg,"Unexpected end of input while scanning a number.");
  token->id[i]='\0';
  return DATALEXER_ERROR;
  
 internalerror:
  sprintf(emsg,"Internal error in data scanner.");
  return DATALEXER_ERROR;;
  
}


#undef BUFFER_SIZE
#undef DATALEXREAD

#define NOTIFY								\
  if(currentsize!=data.readbytes){					\
    if(( data.readbytes / subdiv)>currentper){				\
    }									\
    currentper=data.readbytes / subdiv;					\
    currentsize=data.readbytes;						\
  }									
//was in the if (data.readbytes // ...:: printf("NOTIFY %g \n",1.*data.readbytes/totalsize);	

/* ***********************************************************************
 * data file parser
 * ***********************************************************************
 */
static long parse_datafile(FILE *FH,const String &fname,String &emsg
		    ,DATAPARSER_CALLBACK callback,void *cbdata)
{
  NSF("parse_datafile");
  long colnr;
  long ret;
  long linenr=0;
  tsLEXDATA data;
  char ebuf[5000+1];
  tuTOKEN token;
  struct stat st;
  long totalsize=0;
  unsigned long currentsize=0;
  unsigned long subdiv;
  unsigned long currentper=0;

  colnr=1;
  linenr=1;
  data.buffer[0]='\0';
  data.p=data.buffer;
  data.readbytes=0;


  if(FH==NULL){
    emsg.sprintf("internal(%s): File handle is NULL?!\n",nsubfunc);
    return DATAPARSE_INTERNAL_ERROR;
  }
  if(callback==NULL){
    emsg.sprintf("internal(%s): callback handler is NULL?!\n",nsubfunc);
    return DATAPARSE_INTERNAL_ERROR;
  }

  fstat(fileno(FH),&st);
  totalsize=st.st_size;
  subdiv=(MyMath::max<long>(1,totalsize/(100/5)));
  //printf("subdiv=%ld\n",subdiv);
  
  if(!callback(DP_INIT,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;

  

 expectall:
  if((ret=datalexer(FH,&data,&token,&colnr,&linenr,ebuf))==DATALEXER_ERROR) 
    goto lexerror; 


  NOTIFY;

  if(ret==DATALEXER_EOF) goto succes;

  if(ret==DATALEXER_WHITESPACE){
    goto expectall;
  }else if(ret==DATALEXER_NEWLINE){
    goto expectall;
  }else if(ret==DATALEXER_COMMENT){
    if(!callback(DP_COMMENT,token.id,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectall;
  }else if(ret==DATALEXER_NUM){
    if(!callback(DP_NEWBLOCK,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else if(ret==DATALEXER_NAN|| ret==DATALEXER_INF|| ret==DATALEXER_UNDEF){
    if(!callback(DP_NEWBLOCK,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else{
    goto error;
  }


 expectline:  
  if((ret=datalexer(FH,&data,&token,&colnr,&linenr,ebuf))==DATALEXER_ERROR) 
    goto lexerror; 
  
  NOTIFY;


  if(ret==DATALEXER_NUM){
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else if(ret==DATALEXER_NAN||ret==DATALEXER_INF||ret==DATALEXER_UNDEF){
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else if( (ret==DATALEXER_NEWLINE) || (ret==DATALEXER_COMMENT)||(ret==DATALEXER_EOF) ){
    if(!callback(DP_LINEEND,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    if( (ret==DATALEXER_EOF) ){
      if(!callback(DP_BLOCKEND,token.id,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
      goto succes;
    }else{
      goto expectdataorws;
    }
  }else if(ret==DATALEXER_WHITESPACE){
    goto expectline;
  }else{
    goto error;
  }

 expectdataorws:
  if((ret=datalexer(FH,&data,&token,&colnr,&linenr,ebuf))==DATALEXER_ERROR) 
    goto lexerror; 

  NOTIFY;


  if(ret==DATALEXER_NUM){
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else if(ret==DATALEXER_NAN||ret==DATALEXER_INF||ret==DATALEXER_UNDEF){
    if(!callback(DP_NUM,&token.dnum,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectline;
  }else if(ret==DATALEXER_COMMENT){
    if(!callback(DP_INSETCOMMENT,token.id,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectdataorws;
  }else if(ret==DATALEXER_WHITESPACE){
    goto expectdataorws;
  }else if(ret==DATALEXER_NEWLINE){
    if(!callback(DP_BLOCKEND,token.id,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto expectall;
  }else if(ret==DATALEXER_EOF){
    if(!callback(DP_BLOCKEND,token.id,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
    goto succes;
  }else{
    /* CPRL(ret); */
    goto error;
  }
  


  goto error;


 succes:
  if(!callback(DP_CLEAN,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
  if(!callback(DP_EXIT,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
  return DATAPARSE_SUCCES;

 error:
  if(!callback(DP_CLEAN,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
  emsg.sprintf("Unexpected input near character %ld in line %ld "
	       "\nin file \"%s\"",MyMath::max<long>(colnr-1,0),linenr,fname());
  return DATAPARSE_INTERNAL_ERROR;
  
 lexerror:
  if(!callback(DP_CLEAN,NULL,cbdata,emsg)) return DATAPARSE_CALLBACK_ERROR;
  emsg.sprintf("%s: %s  near character %ld in line %ld "
	       "\nin file \"%s\"",nsubfunc,ebuf,MyMath::max<long>(0,colnr-1),linenr,fname());
  return DATAPARSE_LEX_ERROR;

}


#undef NOTIFY











/* ***********************************************************************
 *
 * ***********************************************************************
 */
#define NOTHING  0
#define XCOLUMN  1
#define YCOLUMN  2
#define ZCOLUMN  3

typedef struct tsxnyscan{
  String filename;
  long nblock;
  long nline;
  long ncolumn;
  long ncolumns;
  long nfuncs;
  long warning;
  long expect;
  double x;
  WeightLabels *wl;
  String lastcomment;
  FunctionGroup *group;
  Page *page;
  bool onlyonecolumn;
}tsXNYSCAN;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static String stripNewFPLOComment(const String &inp)
{
  String s=inp;
  
  //myDebug("[%s]\n",s());
  s.stripBlanks();
  int j0=s.find("<start");
  int j1=s.find(">");
  if(j0>=0 && j1==s.length()-1){
    int j=s.find(":");
    s=s.subString(j+1);
    //myDebug("[%s]\n",s());
    s=s.subString(0,s.length()-2).stripBlanks();
    //myDebug("[%s]\n",s());
  }

  return s;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long XYCallback(long dptype,const void *id,void *data,String &emsg)
{
  NSF("XYCallback");
  tsXNYSCAN *sc=(tsXNYSCAN*)data;
  const char *pid=(const char *)id;
  const double *pd=(const double*)id;
  double y;
  long nf,na;
  Function *func;
  String txt;




  if(data==NULL){
    emsg.sprintf("internal(%s): callback data is NULL?!\n",nsubfunc);
    return 0;
  }

  switch(dptype) {
  case DP_INIT:
    {
      sc->lastcomment="";
      sc->nblock=0;
      sc->nline=0;
      sc->ncolumn=0;
      sc->ncolumns=0;
      sc->nfuncs=sc->group->funcs()->size();
      sc->warning=0;      
      break;
    }
  case DP_EXIT:
    {
      break;
    }
  case DP_CLEAN:
    {
      sc->lastcomment="";
      break;
    }
  case DP_COMMENT:
    {
      sc->lastcomment=pid;
      break;
    }
  case DP_INSETCOMMENT:
    {
      /* set_string(&sc->lastcomment,pid); */
      break;
    }
  case DP_NEWBLOCK:
    {
      sc->nblock++;
      sc->nline=0;
      sc->ncolumn=0;
      sc->ncolumns=0;
      sc->nfuncs=sc->group->funcs()->size();
      sc->onlyonecolumn=true;
      break;
    }
  case DP_BLOCKEND:
    {
      //printf("ncolumns %ld\n",sc->ncolumns);
      //printf("nfuncs %ld\n",sc->group->funcs()->size());
      //printf("onlyonecolumn %d\n",sc->onlyonecolumn);
      if(sc->onlyonecolumn){
	func=sc->group->funcs()->at(sc->nfuncs);
	for(int i=0;i<func->length();i++){
	  func->setData(1,i,func->data(0,i));
	}
      }else{
	long nf=sc->group->funcs()->size();
	long n0=nf-sc->ncolumns+1;

	StringList sl=StringPtr::toStringList(sc->lastcomment,false);
	
	if(sl.size()==nf-n0+1){
	  for(long i=n0-1;++i<nf;){
	    func=sc->group->funcs()->at(i);
	    if(sc->ncolumns>2){
	      func->attribs()
		->setLegend(Sprintf("%s",sl[i-n0+1]()),true);
	    }else{
	      func->attribs()
		->setLegend(Sprintf("%s",sl[i-n0+1]()),true);
	    }
	  }
	}
      }
      sc->lastcomment="";
      break;
    }
  case DP_LINEEND:
    {
      sc->nline++;
      sc->ncolumns=sc->ncolumn;
      sc->ncolumn=0;
      break;
    }
  case DP_NUM:
    {

      nf=sc->nfuncs+sc->ncolumn-1;
      
      if(sc->ncolumn==0){
	sc->x=*pd;
	y=0.;
	nf++;
      }else{
	y=*pd;
	sc->onlyonecolumn=false;
      }

      sc->group->funcs()->assureSize(nf+1);


      func=sc->group->funcs()->at(nf);
      func->setActive(true);
      na=func->length();
      if(sc->ncolumn==1) na--;

      if(sc->nline ==0 ){
	if(sc->ncolumn>1 || sc->lastcomment.length()<2){
	  if(sc->lastcomment.length()>1){
	    txt.sprintf("%s [Y-column %ld]",sc->lastcomment(),sc->ncolumn);
	  }else{
	    txt.sprintf("[block %ld] [Y-column %ld]",sc->nblock,sc->ncolumn);
	  }
	  func->attribs()->setLegend(txt,true);
	}else{
	  func->attribs()->setLegend(stripNewFPLOComment(sc->lastcomment),true);
	}
	func->setComment(Sprintf("%s:%s",sc->filename(),sc->lastcomment()));
      }

      func->setData(0,na,sc->x);
      func->setData(1,na,y);

      sc->ncolumn++;
      break;
    }
  default:
    {
      emsg.sprintf("internal(%s): unknwon callback call type?!\n",nsubfunc);
      return 0;

    }
  }

  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long XYNWCallback(long dptype,const void *id,void *data,String &emsg)
{
  NSF("XYNWCallback");
  tsXNYSCAN *sc=(tsXNYSCAN*)data;
  const char *pid=(const char *)id;
  const double *pd=(const double*)id;
  double y;
  long nf,na;
  Function *func;
  String txt;




  if(data==NULL){
    emsg.sprintf("internal(%s): callback data is NULL?!\n",nsubfunc);
    return 0;
  }

  switch(dptype) {
  case DP_INIT:
    {
      sc->lastcomment="";
      sc->nblock=0;
      //printf("DP_INIT %ld\n",sc->nblock);
      sc->nline=0;
      sc->ncolumn=0;
      sc->ncolumns=0;
      sc->nfuncs=sc->group->funcs()->size();
      sc->warning=0;      
      break;
    }
  case DP_EXIT:
    {
      break;
    }
  case DP_CLEAN:
    {
      sc->lastcomment="";
      break;
    }
  case DP_COMMENT:
    {
      sc->lastcomment=pid;
      break;
    }
  case DP_INSETCOMMENT:
    {
      /* set_string(&sc->lastcomment,pid); */
      break;
    }
  case DP_NEWBLOCK:
    {
      sc->nblock++;
      //printf("DP_NEWBLOCK %ld\n",sc->nblock);
      sc->nline=0;
      sc->ncolumn=0;
      sc->ncolumns=0;
      sc->nfuncs=sc->group->funcs()->size();
      sc->wl=sc->page->weightLabelsCollector()->demandNewAuto();
      break;
    }
  case DP_BLOCKEND:
    {
      //printf("DP_BLOCKEND %ld\n",sc->nblock);
      long nff=sc->group->funcs()->size();
      if(nff>0){
	long nw=sc->ncolumns-2;
	if(nw>0){
	  StringList sl=StringPtr::toStringList(sc->lastcomment,false);

	  if(sl.size()==nw+2){
	    sc->group->funcs()->at(nff-1)
	      ->attribs()->setLegend(Sprintf("%s",sl[1]()),true);
	    for(long iw=-1;++iw<nw;){
	      myDebug()<<iw;
	      sc->wl->setLabel(iw,sl[iw+2]);
	    }
	  }
	}
      }
      sc->lastcomment="";
      break;
    }
  case DP_LINEEND:
    {
      sc->nline++;
      sc->ncolumns=sc->ncolumn;
      sc->ncolumn=0;
      break;
    }
  case DP_NUM:
    {

      //printf("DP_NUM %ld\n",sc->nblock);
      
      nf=sc->nfuncs;
      
      if(sc->ncolumn==0){
	sc->x=*pd;
	y=0.;
      }else{
	y=*pd;
      }

      if(nf+1>sc->group->funcs()->size()){
	sc->group->funcs()->assureSize(nf+1);
	sc->group->funcs()->assureSize(nf+1);
	sc->group->funcs()->at(nf)->setWeightLabels(sc->wl);
      }

      long iwl=sc->ncolumn-2;
      if(iwl>=0){
	sc->wl->setLabel(iwl,Sprintf("W %ld",iwl+1));
      }

      func=sc->group->funcs()->at(nf);
      func->setDataType(Function::XYNW);
      func->setActive(true);
      //func->attribs()->symbol().setType(Xfp::STYCircle);
      na=func->length();
      if(sc->ncolumn>=1) na--;

      if(sc->nline ==0 ){
	if(sc->ncolumn==1 || sc->lastcomment.length()<2){
	  if(sc->lastcomment.length()>1){
	    txt.sprintf("%s ",sc->lastcomment(),sc->ncolumn);
	  }else{
	    txt.sprintf("[block %ld] ",sc->nblock,sc->ncolumn);
	  }
	  func->attribs()->setLegend(txt,true);
	}else{
	  func->attribs()->setLegend(sc->lastcomment,true);
	}
	func->setComment(Sprintf("%s:%s",sc->filename(),
				 func->attribs()->legend()()));
      }

      if(sc->ncolumn==0) 
	func->setData(0,na,sc->x);
      if(sc->ncolumn>=1)
	func->setData(sc->ncolumn,na,y);

      sc->ncolumn++;
      break;
    }
  default:
    {
      emsg.sprintf("internal(%s): unknwon callback call type?!\n",nsubfunc);
      return 0;

    }
  }

  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long GridCallback(long dptype,const void *id,void *data,String &emsg)
{
  NSF("GridCallback");
  tsXNYSCAN *sc=(tsXNYSCAN*)data;
  const char *pid=(const char *)id;
  const double *pd=(const double*)id;
  long nf;
  Function *func;




  if(data==NULL){
    emsg.sprintf("internal(%s): callback data is NULL?!\n",nsubfunc);
    return 0;
  }

  switch(dptype) {
  case DP_INIT:
    {
      sc->lastcomment="";
      sc->nblock=0;
      sc->nline=0;
      sc->ncolumn=0;
      sc->expect=XCOLUMN;
      sc->nfuncs=sc->group->funcs()->size();
      sc->warning=0;
      break;
    }
  case DP_EXIT:
    {
      if(sc->expect!=XCOLUMN){
	emsg.sprintf("error(%s): expect a sequence of three blocks: x, y and z!\n",nsubfunc);
	return 0;
      }
      break;
    }
  case DP_CLEAN:
    {
      sc->lastcomment="";
      break;
    }
  case DP_COMMENT:
    {
      /*p=pid+strspn(pid," \t");
      if(p[0]=='x' && (*(p+strspn(p+1," \t")+1)=='\0') ){
      	sc->expect=XCOLUMN;
      	sc->nx=0;
      	sc->xarray=NULL;
	}*/
      sc->lastcomment=pid;
      break;
    }
  case DP_INSETCOMMENT:
    {
      /* set_string(&sc->lastcomment,pid); */
      break;
    }
  case DP_NEWBLOCK:
    {
      sc->nblock++;
      sc->nline=0;
      sc->ncolumn=0;
      if(sc->expect==XCOLUMN){
	sc->group->funcs()->assureSize(sc->nfuncs+1);
	sc->group->funcs()->at(sc->group->funcs()->size()-1)
	  ->setDataType(Function::GRIDX_Y_NZ);
	sc->nfuncs=sc->group->funcs()->size();
      }else if(sc->expect==YCOLUMN){
      }else if(sc->expect==ZCOLUMN){
	nf=sc->nfuncs-1;
	func=sc->group->funcs()->at(nf);
	func->adjustComp(1,func->xdata().size()*func->ydata().size(),true);
      }
      break;
    }
  case DP_BLOCKEND:
    {
      if(sc->expect==XCOLUMN){
	sc->expect=YCOLUMN;
      }else if(sc->expect==YCOLUMN){
	sc->expect=ZCOLUMN;
      }else if(sc->expect==ZCOLUMN){
	sc->expect=XCOLUMN;
	nf=sc->nfuncs-1;
	func=sc->group->funcs()->at(nf);
	if(sc->nline<func->xdata().size()*func->ydata().size()){
	  emsg.sprintf("warning(%s): less z-data than defined by x and y data size!\n",nsubfunc);
	  sc->warning=1;
	}
      }
      sc->lastcomment="";
      break;
    }
  case DP_LINEEND:
    {
      sc->nline++;
      sc->ncolumn=0;
      break;
    }
  case DP_NUM:
    {
      nf=sc->nfuncs-1;
      func=sc->group->funcs()->at(nf);
      func->setActive(true);
      if(sc->expect==XCOLUMN){
	if(sc->ncolumn==0){
	  func->setDataXY(Xfp::Graph_X,sc->nline,*pd);
	}	
      }else if(sc->expect==YCOLUMN){
	if(sc->ncolumn==0){
	  func->setDataXY(Xfp::Graph_Y,sc->nline,*pd);
	}	
      }else if(sc->expect==ZCOLUMN){
	if(sc->nline<func->xdata().size()*func->ydata().size()){
	  func->setData(sc->ncolumn,sc->nline,*pd);
	}else{
	  emsg.sprintf("warning(%s): more z-data than defined by x and y data size!\n",nsubfunc);
	  sc->warning=1;
	}
      }
      sc->ncolumn++;
      break;
    }
  default:
    {
      emsg.sprintf("internal(%s): unknwon callback call type?!\n",nsubfunc);
      return 0;

    }
  }

  return 1;
}

/* ***********************************************************************
 * 
 * ***********************************************************************
 */
void readXYFuncs(Graph *graph,const String &ffname,IOControl *iofc)
{
  /* NSF("readXYFuncs"); */
  FILE *FHI=NULL;
  tsXNYSCAN xnyscan;
  FunctionGroup *group;
  Function *func;
  long nf=0,i;
  String mesg,fname=ffname;
  if(graph==NULL) return;
  
  
  /* select proper group to read into */
  group=graph->groups()->getExistingGroupOrNew(iofc->targetGroupId());
  group->setActive(true);
  
  
  /* adjust group */
  if(iofc->clearGroup()){
    group->funcs()->reset();
  }
  /* remember current func number for autocoloring, ... */
  nf=group->funcs()->size();
  
  try{
    if(fname=="-"){
      fname="stdin";
      FHI=stdin;
    }else FHI=openforread(fname);
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }
  xnyscan.filename=fname;
  xnyscan.group=group;
  xnyscan.page=group->graph()->page();
  if(iofc->dataType()==Function::XY){
    if(parse_datafile(FHI,fname,mesg,XYCallback,&xnyscan)!=DATAPARSE_SUCCES)
      goto error;
  }else if(iofc->dataType()==Function::XYNW){
    if(parse_datafile(FHI,fname,mesg,XYNWCallback,&xnyscan)!=DATAPARSE_SUCCES)
      goto error;
  }
  
  
  
 clean:
  if(FHI!=NULL && FHI!=stdin){ fclose(FHI);FHI=NULL; }
  
  if((iofc->autoScale() & IOControl::X)==IOControl::X){
    graph->autoScaleAll(Xfp::Graph_X); 
  }
  if((iofc->autoScale() & IOControl::Y)==IOControl::Y){
    graph->autoScaleAll(Xfp::Graph_Y); 
  }
  
  for(i=nf;i<group->funcs()->size();i++){
    func=group->funcs()->at(i);
    /* setwidth_linestyle(&func->attribs()->linestyle,1); */
    /* setstyle_linestyle(&func->attribs()->linestyle,i% (PSMaxLineStyles-1)+1); */
    QRgb cc=Xfp::rgbColor(iofc->autoColorIndex() % (Xfp::maxRgbColors()-1));
    if(iofc->autoColor()){
      func->attribs()->lineStyle().setColor(cc);
      iofc->setAutoColorIndex((iofc->autoColorIndex()+1 )%
			      (Xfp::maxRgbColors()-1));

    }
    
    
    if(func->length()==1 || iofc->autoSymbol()){
      func->attribs()->symbolStyle().
	setType((Xfp::SymbolTypes)(i % (Xfp::maxSymbols()-2)+1));
      func->attribs()->symbolStyle().setSize(12.);
    }
  
    if(func->dataType()==Function::XYNW){
//       myDebug()<<"func"<<func;
//       wds->print();
      WeightLabels *wl=func->weightLabels();
      WeightSelections *wss=func->weightSelections();
      for(long i=0;i<wl->size();i++){
	QRgb cc=Xfp::rgbColor((i % (Xfp::maxRgbColors()-2))+1);
	bool ac=false;
	if(i==nf) ac=true;
	(*wss)[i]->setUp(wl->at(i),ac,cc);
	//wss->append(new WeightSelection(wl->at(i),ac,cc));
      }
      wl->analyse();
    }
  
  }
  if(!mesg.empty())
    throw Except(mesg);
  return;
 error:
  /* printf("%s\n",emsg); */
  goto clean;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *readGridFuncs(Graph *graph,const String &fname
			     ,const IOControl *iofc)
{
  NSF("readGridFuncs");
  FILE *FHI=NULL;
  tsXNYSCAN xnyscan;
  String mesg;
  long nf,i;
  Function *func;
  FunctionGroup *group=0;
  if(graph==NULL) return group;


  /* select proper group to read into */
  group=graph->groups()->getExistingGroupOrNew(iofc->targetGroupId());
  group->setActive(true);

  if(iofc->clearGroup()){
    group->funcs()->reset();
  }
  /* remember current func number for autocoloring, ... */
  nf=group->funcs()->size();


  try{
    FHI=openforread(fname);
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }
  xnyscan.group=group;
  xnyscan.page=graph->page();
  if(parse_datafile(FHI,fname,mesg,GridCallback,&xnyscan)!=DATAPARSE_SUCCES)
    goto error;


  if(xnyscan.warning){
    goto warning;
  }
  
  
  if(group->funcs()->size()==nf){
    mesg.sprintf("%s: file '%s' seems to be empty or corrupted, "
		 "or I failed?!",nsubfunc,fname());
    goto warning;
  }

  
  
 clean:
  if(FHI!=NULL){ fclose(FHI);FHI=NULL; }

  if((iofc->autoScale() & IOControl::X)==IOControl::X){
    graph->autoScaleAll(Xfp::Graph_X); 
  }
  if((iofc->autoScale() & IOControl::Y)==IOControl::Y){
    graph->autoScaleAll(Xfp::Graph_Y); 
  }
  /* TODO */
  for(i=nf;i<group->funcs()->size();i++){
    func=group->funcs()->at(i);
    /* setwidth_linestyle(&func->attrib.linestyle,1); */
    /* setstyle_linestyle(&func->attrib.linestyle,i% (PSMaxLineStyles-1)+1); */
    if(iofc->autoColor()){
      func->attribs()->lineStyle().
	setColor(Xfp::rgbColor(i % (Xfp::maxRgbColors()-1)));

    }
    
    
    /*if(func->Narray==1 || iofc->autoSymbol()){
      settype_symbol(&func->attribs()->symbol,i % (PSMaxSymbols-2)+1);
      setsize_symbol(&func->attribs()->symbol,12.);
      }*/
    
  }

  if(!mesg.empty())
    throw Except(mesg);
  return group;

 error:
  /* printf("%s\n",emsg); */
  goto clean;
 warning:
  /* printf("%s\n",emsg); */
  goto clean;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void readBandData(Graph *graph,const String &ffname
		  ,IOControl *iobc)
{
  NSF("readBandData");
  FILE *FHI=NULL;
  String txt,ctxt,fname=ffname;
  String mesg;
  long mspin,nband,ik,in,nread,idum,nk,ms,io,iu,lower,start;
  long scale=0;
  double ef,dk,e,dkmin,dkmax,ddk;
  char *pbuffer=NULL;
  FunctionGroup *group[2];
  long lenpbuffer=0;
  const char *spintxt[2]={
    "Spin $arrowup",
    "Spin $arrowdown"
  };

  Function *func;


  bool hasNaN=false;


  if(graph==NULL) return;


  
  

  try{
    if(ffname=="-"){
      FHI=stdin;
      fname="stdin";
    }else FHI=openforread(fname);
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }
  
  
  
  if(!(allocreadbuf(FHI,&pbuffer,&lenpbuffer))){
    (void)mesg.sprintf("%s: Can not read file \'%s\' : skip!\n"
		       ,nsubfunc,fname());
    goto error;
  }
  nread=sscanf(pbuffer,"%*s%ld%lg%ld%ld%ld%ld%ld%ld"
	       ,&idum,&ef,&nk,&idum,&nband,&mspin,&io,&iu);

  if(nread==8){
    lower=io;
    //    upper=iu;
  }else{
    lower=1;
    //    upper=nband;
  }
  /* CPRL(lower); */
  /* CPRL(upper); */
  
  
  /* adjust groups */
  for(int ms=0;ms<mspin;ms++){
    if(mspin==2){
      txt.sprintf("\'%s\': %s",fname(),spintxt[ms]);
      ctxt.sprintf("\'%s\': Spin %ld",fname(),ms+1);
    }else{
      txt.sprintf("\'%s\' ",fname());
      ctxt=txt;
    }
    
    group[ms]=graph->groups()->append();
    group[ms]->funcs()->setSize(nband);
    group[ms]->setComment(ctxt);
    group[ms]->attribs()->setLegend(txt,true);
    group[ms]->setUseAttribs(true);
    group[ms]->setActive(true);
    if(ms==1){
      group[ms]->attribs()->lineStyle().setStyle(Xfp::LSTDash);
    }
    if(iobc->autoColor()){
      group[ms]->attribs()
	->lineStyle().setColor(Xfp::rgbColor(iobc->autoColorIndex() % 
					     (Xfp::maxRgbColors()-1)));
    }
  }
  
  
  iobc->setAutoColorIndex((iobc->autoColorIndex()+1 )%(Xfp::maxRgbColors()-1));
  //myDebug()<<"iobc->autoColorIndex()"<<iobc->autoColorIndex();
    
  /* force allocation of at least nk data points */
  for(in=0;in<nband;in++){
    func=group[0]->funcs()->at(in);
    func->adjustComp(2,nk,true);
    txt.sprintf("nband=%ld spin=%ld",lower+in,1);
    func->attribs()->setLegend(txt,true);
    func->setComment(Sprintf("\'%s\':%s",fname(),txt()));
    func->setActive(true);
    if(mspin==2){
      func=group[1]->funcs()->at(in);
      func->adjustComp(2,nk,true);
      txt.sprintf("nband=%ld spin=%ld",lower+in,2);
      func->attribs()->setLegend(txt,true);
      func->setComment(Sprintf("\'%s\':%s",fname(),txt()));
      func->setActive(true);
      func->adjustComp(2,nk,true);
    }
  };

  dkmin=0;
  dkmax=0;
  start=1;
  for(ik=0;ik<nk;ik++){
    (void)allocreadbuf(FHI,&pbuffer,&lenpbuffer);
    for(ms=0;ms<mspin;ms++){
      (void)fscanf(FHI,"%lf",&dk);
      if((dk!=dk)) { hasNaN=true; dk=0; }
      if(start){
	start=0;
	dkmin=dk;
	dkmax=dk;
      }else{
	dkmin=MyMath::min<double>(dkmin,dk);
	dkmax=MyMath::max<double>(dkmax,dk);
      }
      for(in=0;in<nband;in++){
	(void)fscanf(FHI,"%lf",&e);
	if((e!=e)) { hasNaN=true; e=0; }
	group[ms]->funcs()->at(in)->setData(0,ik,dk);
	group[ms]->funcs()->at(in)->setData(1,ik,e);
      }
      (void)allocreadbuf(FHI,&pbuffer,&lenpbuffer);
    } /* ms */
  } /* ik */
  if( FHI != NULL && FHI!=stdin ) {(void)fclose(FHI);FHI=NULL;}  


  ddk=(dkmax-dkmin);
  if(fabs(ddk)>0)ddk=1./ddk;


  if(scale){
    for(ik=0;ik<nk;ik++){
      for(ms=0;ms<mspin;ms++){
	for(in=0;in<nband;in++){
	  func=group[ms]->funcs()->at(in);
	  dk=func->data(0,ik);
	  dk=(dk-dkmin)*ddk;
	  func->setData(0,ik,dk);//dataRef(0,ik)=dk;
	}
      }
    }
  }

  try{
    readFploPointData(graph,fname,group[0],scale,dkmin,ddk);
    if(mspin==2){
      readFploPointData(graph,fname,group[1],scale,dkmin,ddk);
      group[1]->userTics()->setDrawTics(false);
    }
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }


  autoBPAdjustment(graph,iobc);
  
  
 clean:
  free(pbuffer);

  if(FHI!=NULL && FHI!=stdin){ fclose(FHI);FHI=NULL; }
  
  if((iobc->autoScale() & IOControl::X)==IOControl::X){
    graph->autoScaleAll(Xfp::Graph_X); 
  }
  if((iobc->autoScale() & IOControl::Y)==IOControl::Y){
    graph->autoScaleAll(Xfp::Graph_Y); 
    graph->world()->setW01(Xfp::Graph_Y,-12,10);
  }

  if(!mesg.empty())
    throw Except(mesg);

  if(hasNaN) throw Except(Sprintf("File \"%s\": some numbers are NaN!",
				  fname())());

  return;


 error:
  goto clean;
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void IOFunc::readBandWeightData(Graph *graph,const String &ffname
				,IOControl *iobwc)
{
  NSF("readBandWeightData");
  FILE *FHI=NULL;
  String txt,ctxt;
  String mesg,fname=ffname;
  char *txtst,*end;
  long nread,nk,nband,mspin,io,iu,nsite,nvdim,lower;
  long index,norbital,in,l,i,ik,ms,scale=0,lmax=0;
  bool first=true;
  double ef,ddum=0;
  WeightLabels *wls;
  FunctionGroup *group[2];
  WeightSelections wss;
  const char *p;
  char *pbuffer=NULL;
  char *pbufferheader=NULL;
  long lenpbuffer=0;
  long lenpbufferheader=0;
  Function *func;
  unsigned long subdiv;
  long countMax;
  double *data=0;

  if(graph==NULL) return ;



  bool hasNaN=false;

  try{
    if(fname=="-"){
      fname="stdin";
      FHI=stdin;
    }else FHI=openforread(fname);
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }




  
  
  if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ){
    (void)mesg.sprintf("%s: Can not read file \'%s\' : skip!\n"
		       ,nsubfunc,fname());
    goto error;
  }
  nread=sscanf(pbuffer,"%*s%ld%lg%ld%ld%ld%ld%ld%ld"
	       ,&nsite,&ef,&nk,&nvdim,&nband,&mspin,&io,&iu);


  if(nread==8){
    lower=io;
    //upper=iu;
  }else{
    lower=1;
    //upper=nband;
  }


  /* adjust groups */
  for(int ms=0;ms<mspin;ms++){
    group[ms]=graph->groups()->append();
  }
  
  group[0]->funcs()->setSize(nband);

  //printf("%s graph=%p\n",nsubfunc,graph);
  //printf("%s graph->page()=%p\n",nsubfunc,graph->page());

  wls=graph->page()->weightLabelsCollector()->demandNewAuto();
  if(mspin==2){
    txt.sprintf("\'%s\': Spin $arrowup",fname());
    ctxt.sprintf("\'%s\': Spin 1",fname());
  }else{
    txt.sprintf("\'%s\' ",fname());
    ctxt=txt;
  }

  group[0]->setComment(ctxt);
  group[0]->attribs()->setLegend(txt,false);
  group[0]->setUseAttribs(true);
  group[0]->setActive(true);


  if(mspin==2){
    group[1]->funcs()->setSize(nband);
    txt.sprintf("\'%s\': Spin $arrowdown",fname());
    ctxt.sprintf("\'%s\': Spin 2",fname());
    group[1]->setComment(ctxt);
    group[1]->attribs()->setLegend(txt,false);
    group[1]->attribs()->lineStyle().setStyle(Xfp::LSTDash);
    group[1]->setUseAttribs(true);
    group[1]->setActive(true);
  }


  //iobwc->setAutoColorIndex((iobwc->autoColorIndex()+1 )%(Xfp::maxRgbColors()-1));

  /* get the number of orbitals from reading two more lines */

  (void)allocreadbuf(FHI,&pbufferheader,&lenpbufferheader);
  // we take the first data line to determine the number of weights
  (void)allocreadbuf(FHI,&pbuffer,&lenpbuffer);



  p=pbuffer;
  l=1;
  norbital=-2;
  while(l){
    //printf("%ld <%s>\n",lmax,p);
    long l0=strspn(p," \t");
    p=p+l0;
    l=strcspn(p," \t");
    if(l>0){
      norbital=norbital+1;
      p=p+l;
    }
    if(norbital==1){
      lmax=l+l0;
    }

  }

  //printf("know norbital=%ld lmax=%ld\n",norbital,lmax);



  /* get the labels */
  if(norbital==1){
    txtst=new char[strlen(pbufferheader)+1];
  }else{
    txtst=new char[lmax+1];
  }
  for(i=0;i<norbital;i++){
    QRgb cc=Xfp::rgbColor((i % (Xfp::maxRgbColors()-2))+1);
    index=42+i*lmax;
    if(norbital==1){
      (void)strcpy(txtst,pbufferheader+index);
    }else{
      (void)strncpy(txtst,pbufferheader+index,lmax);
      txtst[lmax]='\0';
    }
    String s=String(txtst).chop().stripBlanks();
    //myDebug("<%s>\n",s());
    wls->append(s);
    bool ac=(i==0);
    WeightSelection *ws=new WeightSelection(wls->at(i));
    ws->setActive(ac);
    ws->setColor(cc);
    wss.append(ws);
  }
  wls->analyse();

  delete [] txtst;

  if(iobwc->dataType()==Function::XYMatel){
    OptMatelDefinition *matel=new OptMatelDefinition();
    group[0]->setOptMatelDefinition(matel);
    if(mspin==2)group[1]->setOptMatelDefinition(matel);
  }

  group[0]->weightSelections()->copySelections(&wss);
  if(mspin==2){
    group[1]->weightSelections()->copySelections(&wss);
  }
  
  /* force allocation of at least nk data points */
  for(in=0;in<nband;in++){
    Function *func=group[0]->funcs()->at(in);
    func->adjustComp(norbital+2,nk,true);
    //myDebug()<<"func "<<func->length()<<func->components();
    func->setDataType(iobwc->dataType());
    func->setWeightLabels(wls);
    func->weightSelections()->copySelections(&wss);
    {
      txt.sprintf("nband=%ld spin=%ld",lower+in,1);
      func->setComment(Sprintf("\'%s\':%s",fname(),txt()));
      func->attribs()->setLegend(txt,true);
      func->setActive(true);
    }

    if(mspin==2){
      Function *func=group[1]->funcs()->at(in);
      func->adjustComp(norbital+2,nk,true);
      func->setDataType(iobwc->dataType());
      func->setWeightLabels(wls);
      func->weightSelections()->copySelections(&wss);
      {
	txt.sprintf("nband=%ld spin=%ld",lower+in,2);
	func->setComment(Sprintf("\'%s\':%s",fname(),txt()));
	func->attribs()->setLegend(txt,true);
	func->setActive(true);
      }
    }
  };


  /* read the shit */
  subdiv=(MyMath::max<long>(1,nk/(100/5)));
  countMax=MyMath::max<long>(1,nk/subdiv);

#ifdef QXFPL
  emit statusMesg(Sprintf("Reading %s...",fname())());
  emit progress(0);
#endif

  myDebug()<<"first="<<first;
  p=pbuffer;
  
  data=new double[norbital+2];
  for(ik=0;ik<nk;ik++){
    if ( ik%subdiv ==0 ){
#ifdef QXFPL
      emit progress((int)(ik/subdiv*(100./countMax)));
#endif
    }
    for(ms=0;ms<mspin;ms++){
      for(in=0;in<nband;in++){
	func=group[ms]->funcs()->at(in);
	for(i=0;i<norbital+2;i++){
	  if(first){
	    data[i]=strtod(p,&end);
	    p=end;
	  }else{
	    (void)fscanf(FHI,"%lf",data+i);
	  }
	  if((data[i]!=data[i])) {
	    hasNaN=true;
	    data[i]=0;
	  }
	}
	func->setDataComponents(ik,data,norbital+2);
	

	first=false;
      }
    }
  }
  delete [] data;
#ifdef QXFPL
  emit progress(100);
  emit progress(0);
  emit statusMesg("");
#endif

  
  if( FHI != NULL && FHI!=stdin ) {(void)fclose(FHI);FHI=NULL;}  
  
  try{
    readFploPointData(graph,fname,group[0],scale,ddum,ddum);
    if(mspin==2){
      readFploPointData(graph,fname,group[1],scale,ddum,ddum);
      group[1]->userTics()->setDrawTics(false);
    }
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }

  autoBPAdjustment(graph,iobwc);


 clean:
  free(pbuffer);
  free(pbufferheader);

  if(FHI!=NULL && FHI!=stdin){ fclose(FHI);FHI=NULL; }

  if((iobwc->autoScale() & IOControl::X)==IOControl::X){
    graph->autoScaleAll(Xfp::Graph_X); 
  }
  if((iobwc->autoScale() & IOControl::Y)==IOControl::Y){
    graph->autoScaleAll(Xfp::Graph_Y); 
    graph->world()->setW01(Xfp::Graph_Y,-12,10);
  }

  if(!mesg.empty())
    throw Except(mesg);

  
  if(hasNaN) throw Except(Sprintf("File \"%s\": some numbers are NaN!",
				  fname())());

  return;


 error:
  goto clean;

}

/* ***********************************************************************
 * ng = group index
 * ***********************************************************************
 */
void readFploPointData(Graph *graph,const String &fname,FunctionGroup *group
		       ,long scale,double dkmin,double ddk)
{
  NSF("readFploPointData");
  FILE *FHI=NULL;
  String line;
  String txt;
  String dir;
  String file;
  String mesg;
  long nlabel,ilabel;
  double dk,e;

  if(graph==NULL) return;

  /* x axis labels */
  /* extract path of file */
  splitPath(dir,file,fname);
   
  

  line.sprintf("%s/+points",dir());
  if((FHI=fopen(line(),"r"))==NULL ){ 
    throw Except(Sprintf("%s: File \'%s\' does not exist.\n" 
			" This file usually is created by fplo.\n" 
			,nsubfunc,line() )); 
  }

  (void)fscanf(FHI,"%*s%ld\n",&nlabel); 
  /* printf("igroup=%ld\n",igroup); */
  /* TODO: avoid overwriting of user tics */
  group->userTics()->resize(nlabel,true);
  group->userTics()->setDrawTics(true);

  
  for(ilabel=0;ilabel<nlabel;ilabel++){ 
    getLine(FHI,line);
    
    long i0=line.find("'");
    long i1=line.find("'",true);
    txt=line.subString(i0+1,i1-1);
    
    getLine(FHI,line);
    (void)sscanf(line(),"%lg%lg",&dk,&e); 
    
    getLine(FHI,line);
    getLine(FHI,line);
    
    group->userTics()->tic(ilabel)->setType(Xfp::Graph_X);
    group->userTics()->tic(ilabel)->setKind(Xfp::TKIMajor);
    if(scale){
      dk=(dk-dkmin)*ddk;
    }
    group->userTics()->tic(ilabel)->setPos(dk);
    group->userTics()->tic(ilabel)->setTicSide(Xfp::TSNormal);
    group->userTics()->tic(ilabel)->setTicLabelSide(Xfp::TSNormal);
    group->userTics()->tic(ilabel)->setTicLength(1.);
    group->userTics()->tic(ilabel)->setLabel(txt);
    

	  
  } /* for all labels */
  
  if( FHI != NULL ) {(void)fclose(FHI);FHI=NULL;}  



  return;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long autoBPAdjustment(Graph *graph,IOControl *iobc)
{
  /* NSF("autoBPAdjustment"); */
  long itic,gotsome;
  TicMarks *tm;

  if(graph==NULL) return 0;
  
  tm=graph->ticMarks();

  if(tm==NULL) return 0;

  if(iobc->adjustXTics()){
    tm->setTicSide(Xfp::Graph_X,Xfp::TSNoSide); 
    tm->setTicLabelSide(Xfp::Graph_X,Xfp::TSNoSide); 
  }
  
  if(iobc->defaultYLabel()){
    graph->textBoxes()->textBoxByType(TextBox::TBTYAxisLabel)
      ->setText("Energy $~e$_n$.(k) [eV]");
  }
  
  if(iobc->hideXLabel()){
    graph->textBoxes()->textBoxByType(TextBox::TBTXAxisLabel)
      ->setActive(false);
  }
  
  if(iobc->EFTic()){
    /* fermi energy */
    
    gotsome=0;
    for(itic=0;itic<tm->userTics()->size();itic++){
      /* CPRL(itic); */
      if( tm->userTics()->tic(itic)->type()==Xfp::Graph_Y &&
	  tm->userTics()->tic(itic)->kind()==Xfp::TKIMajor && 
	  fabs(tm->userTics()->tic(itic)->pos()-0.)<1.e-10 ){
	gotsome=1;
	/* CPRL(gotsome); */
      }
    }
    
    if(!gotsome){
      tm->userTics()->resize(tm->userTics()->size()+1);
      itic=tm->userTics()->size()-1;
      tm->userTics()->tic(itic)->setType(Xfp::Graph_Y);
      tm->userTics()->tic(itic)->setKind(Xfp::TKIMajor);
      tm->userTics()->tic(itic)->setPos(0.);
      tm->userTics()->tic(itic)->setTicSide(Xfp::TSOpposite);
      tm->userTics()->tic(itic)->setTicLabelSide(Xfp::TSOpposite);
      tm->userTics()->tic(itic)->setTicLength(1.0);
      tm->userTics()->tic(itic)->setLabel("$~e$_F$.");
      tm->userTics()->setDrawTics(true);
    }else{
      //      iobc->setEFTic(false);
    }
  }
  return 1;
}
/* EOF */
