/*************************************************************************
 * @(#)plotview.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Oct 2003
 *************************************************************************
 */

#include <QMessageBox>
#include <QMenu>
#include <QFileDialog>
#include <QPainter>

#include "plotview.hh"
#include "MainWindow.hh"
#include "XFPStatusBar.hh"
#include "defs.hh"
#include "Graph.hh"
#include "Page.hh"
#include "PrintDlg.hh"
#include "ReadOldGraph.hh"
#include "Except.hh"
#include <math.h>
#include "iofunc.hh"
#include "Config.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
PlotView::PlotView( MainWindow *parent, Qt::WFlags f)
  : GraphView(parent,f),p_knowfname(false),p_filename("")
{
  //NSF("PlotView");
  

  //QPR("START");

  

 
  //QPR("END");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PlotView::~PlotView()
{
  NSF("~PlotView");

  //QPR("START");
  

  //QPR("END");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long PlotView::save(long saveas)
{
  //   NSF("save");
  long ret;
  long saved=0;
  if(!ppage)return 0;
  
  
  ret=QDialog::Accepted;
  if( !p_knowfname || saveas){
    
    QFileDialog *dlg=new QFileDialog(this,"Save graph","."
				     "Graph command files ( *.xfp ) ;; "
				     "All ( * )");

    dlg->setAcceptMode(QFileDialog::AcceptSave);
    dlg->setFileMode(QFileDialog::AnyFile);
#ifdef Q_OS_MAC
    dlg->setOptions(QFileDialog::DontUseNativeDialog);
#endif
    dlg->setDirectory(QDir::currentPath()+"/");

    
    if(p_knowfname && !p_filename.isEmpty()){
      dlg->selectFile(p_filename);
    }else{
      dlg->selectFile("graph.xfp");
    }
    
    if((ret=dlg->exec())==QDialog::Accepted){
      if(dlg->selectedFiles().size()>0)
	p_filename=dlg->selectedFiles().at(0);
    }
    delete (dlg);
  }     
  if(!p_filename.isNull() && ret==QDialog::Accepted){
    try{
      ppage->write(String(p_filename));
    }catch(Except &ex){
      QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default,0,0);
      
    }
    pstatusbar->setDocument(p_filename);
    saved=1;
    p_knowfname=true;
  }
  
  return saved;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PlotView::load()
{
  long ret;
  if(!ppage)return;
  
  QString filename;
  ret=QDialog::Accepted;
  QFileDialog *dlg=new QFileDialog(this, "Load graph",".",
				   "Graph command files ( *.xfp ) ;; "
				   "All ( * )") ;
  dlg->setAcceptMode(QFileDialog::AcceptOpen);
  dlg->setFileMode(QFileDialog::ExistingFile);
#ifdef Q_OS_MAC
  dlg->setOptions(QFileDialog::DontUseNativeDialog);
#endif
  dlg->setDirectory(QDir::currentPath()+"/");
  
  if(p_knowfname && !p_filename.isEmpty()){
    dlg->selectFile(p_filename);
  }
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
  }
  delete (dlg);
  
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    loadFile(filename,false);
  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PlotView::loadOld()
{
  long ret;
  if(!ppage)return;
  
  QString filename;
  ret=QDialog::Accepted;
  QFileDialog *dlg=new QFileDialog(this, "Load old graph",".",
				   "Old Graph files ( =.* ) ;; "
				   "All ( * )") ;
  dlg->setAcceptMode(QFileDialog::AcceptOpen);
  dlg->setFileMode(QFileDialog::ExistingFile);
#ifdef Q_OS_MAC
  dlg->setOptions(QFileDialog::DontUseNativeDialog);
#endif
  dlg->setDirectory(QDir::currentPath()+"/");
  
  if(p_knowfname && !p_filename.isEmpty()){
    dlg->selectFile(p_filename);
  }
  if((ret=dlg->exec())==QDialog::Accepted){
    filename=dlg->selectedFiles().at(0);
  }
  delete (dlg);
  
  
  if(!filename.isNull() && ret==QDialog::Accepted){
    loadFile(filename,true);
    p_filename=filename;
    p_knowfname=true;
  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PlotView::setup()
{

  GraphView::setup();
  
  p_datapum->setEnabled(true);
  
  p_datapum->addAction("&Import general",this,SLOT(importData()));
 
  p_datapum->addAction("Import &band data",this,SLOT(importBandData()));
  
  p_datapum->addAction("Import band &weight data",this
			 ,SLOT(importBandWeightData()));

  p_datapum->addAction("Import &matel band data",this
			 ,SLOT(importBandMatelData()));

  p_datapum->addAction("Import Bloch &spectral density data"
 			 ,this,SLOT(importAkblData()));

//   p_datapum->addAction("Import something"
// 		       ,this,SLOT(importSomething()));
  
  emit managePopupMenu(QString("data"),p_datapum);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long PlotView::print(long)
{

  if(!ppage)return 0;

  QString file=QDir::currentPath()+QString("/graph.eps");
  if(ppage->knowPrintFileName()){
    file=ppage->printFileName()();
  }else{
    if(p_knowfname){
      QFileInfo fi(p_filename);
      file=fi.fileName();
      if(file.startsWith("=.")){
	file=file.replace(0,2,"");
      }
      if(file.contains(".")){
	file=fi.baseName();
      }
      file=fi.path()+"/"+file+".eps";
    }
  }
  
  PrintDlg *dlg=new PrintDlg(this,file);
  int ret=dlg->exec();


  if (ret==QDialog::Accepted){
    file=dlg->fileName();
    if(!file.isEmpty()){


      bool doit=true;
      if(QFileInfo(file).exists()){
	QString text="File "+file+" exists!\nOverwrite?" ;
	doit=(QMessageBox::Yes==
	 QMessageBox::warning(qApp->activeWindow(),"Save file", text,
			      QMessageBox::Yes,QMessageBox::No ));
	
      }
      
      if(doit){
	ppage->setPrintFileName(String(file));
	printPS();
      }
    }
    
  }
  delete (dlg);
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void PlotView::loadFile(const QString &filename,bool oldformat)
{
  killAll(false);
  try{
    printf("filename <%s>\n",String(filename)());
    if(oldformat){
      readOldGraph(ppage,String(filename));
    }else{
      playParamDocument(filename,false);
    }
  }catch(Except &ex){

    QMessageBox::warning(this,IDS_APP_NAME,ex.msg()
			 ,QMessageBox::Ok | QMessageBox::Default,0,0);
    return;
  }

  refreshPage();
  
  pstatusbar->setDocument(filename);

  p_filename=filename;
  p_knowfname=true;
  
}

/*EOF*/ 
