/*************************************************************************
 * @(#)ComboBoxDelegate.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   11 May 2011
 *************************************************************************
 */

#include <QComboBox>
#include "ComboBoxDelegate.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ComboBoxDelegate::ComboBoxDelegate(QObject *parent
				   ,const QMap<int,QString> &map)
  : QStyledItemDelegate(parent)
{
  pdata=map;
  int length=0;
  foreach(int key,pdata.keys()){
    if(pdata[key].length()>length){
      length=pdata[key].length();
      plongest=pdata[key];
    }
  }
  if(length<1)
    plongest="X";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *ComboBoxDelegate::createEditor(QWidget *parent
					,const QStyleOptionViewItem &option,
					const QModelIndex &index) const
{
  Q_UNUSED(option);
  Q_UNUSED(index);

  QComboBox *editor = new QComboBox(parent);
  QList<int> keys=pdata.keys();
  for(int i=0;i<keys.size();i++){
    int key=keys.at(i);
    editor->insertItem(i,pdata[key],key);
  } 
  return editor;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboBoxDelegate::setEditorData(QWidget *editor
				     ,const QModelIndex &index) const
{
  int value=index.model()->data(index, Qt::EditRole).toInt();

  QComboBox *cb = static_cast<QComboBox*>(editor);
  cb->setCurrentIndex(value);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboBoxDelegate::
updateEditorGeometry(QWidget *editor
		     ,const QStyleOptionViewItem &option
		     ,const QModelIndex &index) const
{
  Q_UNUSED(index);
  editor->setGeometry(option.rect);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboBoxDelegate::setModelData(QWidget *editor
				    ,QAbstractItemModel *model,
				    const QModelIndex &index) const
{
  QComboBox *cb = static_cast<QComboBox*>(editor);
  int value=cb->currentIndex();

  model->setData(index,value,Qt::EditRole);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QSize ComboBoxDelegate::sizeHint(const QStyleOptionViewItem &option,
				 const QModelIndex &index) const
{
  Q_UNUSED(option);
  Q_UNUSED(index);
  //myDebug() << "ComboBoxDelegate::sizeHint"<<plongest;
  
  QComboBox *box=new QComboBox();
  box->insertItem(0,plongest,0);
  QSize size=box->minimumSizeHint();
  delete box;
  return size;
  
  
}



/*EOF*/ 
