/*************************************************************************
 * @(#)DataFileDialog.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 May 2011
 *************************************************************************
 */

#include <QLabel>
#include <QLayout>
#include <QGridLayout>
#include "DataFileDialog.hh"
#include "Graph.hh"
#include "IOControl.hh"
#include "XNYDlgControl.hh"
#include "BandDlgControl.hh"
#include "defs.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
DataFileDialog::DataFileDialog(Type type,QWidget *parent,const QString &caption
			       ,IOControl *iofc,const Page *page
			       ,const QDir &dir,const QString &filter)
  :QFileDialog(parent,caption,".",filter),xnyctrl(0),bandctrl(0),ptype(type)
{
  setWindowIcon(QIcon(":Icons/logo32")  );
  setFileMode(QFileDialog::ExistingFile);
  setAcceptMode(QFileDialog::AcceptOpen);
  setDirectory(dir);

  QGridLayout *glay=dynamic_cast<QGridLayout*>(layout());
  
  if(glay){
    if(ptype==XNY){
      xnyctrl=new XNYDlgControl(this);
      xnyctrl->setData(iofc,page);
      glay->addWidget(xnyctrl,glay->rowCount(),0 ,1,glay->columnCount());
    }else if(ptype==BAND){
      bandctrl=new BandDlgControl(this);
      bandctrl->setData(iofc,page);
      glay->addWidget(bandctrl,glay->rowCount(),0 ,1,glay->columnCount());
    }
    QString s(iofc->fileName()());
    if(!s.isEmpty()){
      selectFile(s);
    }
  }else{
    myDebug()<<"unkonwn QFileDialog layout, please report to developper!";
    exit(1);
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
DataFileDialog::~DataFileDialog()
{
  if(xnyctrl) delete xnyctrl;xnyctrl=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DataFileDialog::accept()
{
  QFileDialog::accept();

  if(ptype == XNY){
    if(xnyctrl){
      IOControl *iofc=xnyctrl->getData();
      if(selectedFiles().size()>0)
	iofc->setFileName(String(selectedFiles().at(0)));
    }
  }else if(ptype == BAND){
    if(bandctrl){
      IOControl *iobc=bandctrl->getData();
      if(selectedFiles().size()>0)
	iobc->setFileName(String(selectedFiles().at(0)));
    }
  }
  return ;
}

/*EOF*/ 
