/*************************************************************************
 * @(#)GraphDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 Nov 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "Graph.hh"
#include "Graphs.hh"
#include "ToolButtons.hh"
#include "GraphPropWdg.hh"
#include "GraphDlg.hh"
#include "GraphsModel.hh"
#include "MyMath.hh"
#include "Page.hh"
#include "ReselectEvent.hh"
#include "Config.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphDlg::GraphDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),t_graphs(0),t_graph(0),grsmodel(0),
    mute_currentIndexChanged(false)
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);
  
  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);
  
  connect(this,SIGNAL(endRequest()),this,SLOT(deleteLater()));

  if(wProps) delete wProps;
  wProps=new GraphPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);

  connect(wProps,SIGNAL(toolTriggered(const QString &)),
	  this,SLOT(onToolTriggered(const QString &)));
  
  grsmodel = new GraphsModel(this);
  lvGraphs->setModel(grsmodel);

  ptoolbuttons.createToolButtonsActions(lvGraphs,"Graph"
		     ,tbInsert,tbDelete,tbUp,tbDown);


  ptoolbuttons.connectSlots(this,
			    SLOT(slotInsertGraph()),
			    SLOT(slotDeleteGraph()),
			    SLOT(slotMoveGraph()),
			    SLOT(slotMoveGraph()));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphDlg::~GraphDlg()
{
  if(grsmodel) delete grsmodel;grsmodel=0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphDlg::dataAcceptable()
{
  QString s,txt="";

  wProps->check(txt);

  
  

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    return false;
  }

  return true;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::apply( )
{
  //myDebug()<<"apply";
  
  wProps->applyData(t_graph);


  grsmodel->resetItem(t_graph);
  emit acceptRequest(0,false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::reloadDialog()
{
  // discard all data and reload;
  if(!t_graphs) return;
  
  QModelIndex index=lvGraphs->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row();
  }

  grsmodel->reset(t_graphs);
  
  row=MyMath::max<long>(0,MyMath::min<long>(row,grsmodel->rowCount()-1));

  QModelIndex ci=grsmodel->index(row,0,QModelIndex());
  lvGraphs->scrollTo(ci);
  lvGraphs->setCurrentIndex(ci);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::setData(Graphs *graphs,Graph *gr)
{
  // sh==0 means the call from menu others where there are no others
  
  //bool newgraphs=false;
  if(graphs){
    if(t_graphs!=graphs){
      if(t_graphs==0){
	//newgraphs=true;
	t_graphs=graphs;
	grsmodel->reset(graphs);
      }else{
	//newgraphs=true;
	t_graphs=graphs;
	grsmodel->reset(graphs);
	
	if(t_graph){
	  // previous selected textbox
	  if(protectChanges(false)) return;
	}
	// no previous selected textbox
	Graph *first=0;
	if(graphs->size()>0){
	  first=graphs->at(0);
	}
	t_graph=first;
	lvGraphs->setCurrentIndex(grsmodel->graphIndex(first));
	return ;
      }
    }
  }else{
    return;
  }

  grsmodel->resetItem(gr);

  

  if(!t_graph){
    // no previous textbox;
    if(!gr){
      //new textbox does not exists
      Graph *first=0;
      if(graphs->size()>0){
	first=graphs->at(0);
      }
      t_graph=first;
    }else{
      t_graph=gr;
    }
    lvGraphs->setCurrentIndex(grsmodel->graphIndex(t_graph));
    return;
  }else{
    //previous textbox
    if(gr){
      //new textbox exists
      if((t_graph!=gr)){
	if(protectChanges(false)) return;
	t_graph=gr;
	lvGraphs->setCurrentIndex(grsmodel->graphIndex(t_graph));
      }
    }else{
      //new textbox does not exist
      return;
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphDlg::protectChanges(bool withCancel)
{
  if(wProps->isDirty()){
    QString s="The content of the dialog changed.\n"
      "(Use the dialog's apply button for"
      " confirming changes.)";
    int ret;
    if(withCancel)
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard,QMessageBox::Cancel);
    else{
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard);
    }
    if(ret==QMessageBox::Apply){
      if(dataAcceptable()){
	apply();
      }else{
 	return true;
      }
    }else if(ret==QMessageBox::Cancel){
      return true;
    }else if(ret==QMessageBox::Discard){
      wProps->setDirty(false);
    }
  }
  return false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphDlg::event(QEvent *event)
{
  //  NSF("GraphDlg::event");
  if(event->type()==ReselectEvent::ReselectEventType){
    if(ReselectEvent *re=dynamic_cast<ReselectEvent*>(event)){
      mute_currentIndexChanged=true;
      lvGraphs->setCurrentIndex(re->index());
      mute_currentIndexChanged=false;
    }
    return true;
  }
  return BrainDialog::event(event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::slotInsertGraph()
{
  //  NSF("GraphDlg::slotInsertGraph()");
  //myDebug()<< nsubfunc;
  if(protectChanges())
    return;


  QModelIndex index=lvGraphs->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
  }
  grsmodel->insertRow(row);
  QModelIndex ci=grsmodel->index(row,0,QModelIndex());
  lvGraphs->scrollTo(ci);
  lvGraphs->setCurrentIndex(ci);
  emit acceptRequest(grsmodel->currentGraph(lvGraphs->currentIndex())
		     ,true);
  wProps->setEnabled(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::slotDeleteGraph()
{
  if(QMessageBox::warning(this,IDS_APP_NAME,"Really delete",
			  QMessageBox::Ok,QMessageBox::Cancel)
     !=QMessageBox::Ok) return;

  //NSF("GraphDlg::slotDeleteGraph"); 
  int numrows=grsmodel->rowCount();
  if(numrows<=0) return;
  QModelIndex currentindex=lvGraphs->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a graph first."
			   , 1, 0,0 );
    return;
  }
  wProps->setDirty(false);
  
  int row=currentindex.row();
  
  Graph *cgr=t_graphs->currentGraph();
  grsmodel->removeRow(row);
  
  if(numrows!=grsmodel->rowCount())
    lvGraphs
      ->setCurrentIndex(grsmodel->
			index(MyMath::max<long>(0,MyMath::min<long>(row,numrows-2)),0,QModelIndex()));
  
  emit acceptRequest(grsmodel->currentGraph(lvGraphs->currentIndex())
		     ,cgr!=t_graphs->currentGraph());

  if(grsmodel->rowCount()==0){
    wProps->setDirty(false);
    wProps->setEnabled(false);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::slotMoveGraph()
{
  //NSF("GraphDlg::slotMoveGraph"); 
  QModelIndex currentindex=lvGraphs->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a shape first."
			   , 1, 0,0 );
    return;
  }
  int row=currentindex.row();
  int by=-1;
  if(sender()==ptoolbuttons.pmoveDownAction) by=1;
  if(grsmodel->moveRow(row,by)){
    lvGraphs->setCurrentIndex(grsmodel->index(row+by,0,QModelIndex()));
  }
  emit acceptRequest(grsmodel->currentGraph(lvGraphs->currentIndex())
		     ,true);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::on_lvGraphs_currentItemChanged(const QModelIndex &current
					      ,const QModelIndex &previous)
{
  if(mute_currentIndexChanged)return;

  if(protectChanges()){
    QEvent *event=new ReselectEvent(ReselectEvent::ReselectEventType,previous);
    QCoreApplication::postEvent(this,event);
    return;
  }
    
  setAddDeleteEnabed(current);
  if(!current.isValid()){
    t_graph=0;
    wProps->setData(0);
  }else{
    Graph *gr=grsmodel->currentGraph(current);
    if(gr){
      t_graph=gr;
      wProps->setData(gr);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::setAddDeleteEnabed(const QModelIndex &current)
{
  if(!current.isValid()){
    //tbInsert->setEnabled(false);
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
    return;
  }
  Graph *gr=grsmodel->currentGraph(current);
  if(!gr){
    tbInsert->setEnabled(false);
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
  }else{
    tbInsert->setEnabled(true);
    tbDelete->setEnabled(true);
    if(current.row()==0)
      tbUp->setEnabled(false);
    else
      tbUp->setEnabled(true);
    if(current.row()<grsmodel->rowCount()-1)
      tbDown->setEnabled(true);
    else
      tbDown->setEnabled(false);
    
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphDlg::onToolTriggered(const QString &mode)
{
  if(mode=="scale all"){
    emit acceptRequest(0,false);
  }else if ("scale view"){
    emit acceptRequest(0,false);
  }else{
    myDebug()<< "GraphDlg::onToolTriggered: unknown mode";
  }
}

/*EOF*/ 
