/* @(#)GraphDlg.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _GRAPHDLG_H
#define _GRAPHDLG_H 1


#include <QCloseEvent>
#include "ToolButtons.hh"
#include "ui_GraphDlg.h"

class Graph;
class Graphs;
class GraphsModel;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class GraphDlg: public BrainDialog, protected Ui::GraphDlg
{
  Q_OBJECT
public:
  GraphDlg(QWidget *parent,QWidget *base,const char *name);
  ~GraphDlg();

  void setData(Graphs *graphs=0,Graph *gr=0);
  void reloadDialog(); 

  void updateRequest(Graph *);
  void deleteGraph(Graph *);

signals:
  void acceptRequest(Graph *,bool forceChanged);
  void endRequest(); 

public slots:
  virtual void reject(){ emit endRequest();};
protected:
  virtual void closeEvent( QCloseEvent * e ) {e->accept(); emit endRequest();};
  virtual bool event(QEvent *event);

private slots:
  void on_pbApply_clicked(){if(dataAcceptable()){apply();}};
  void on_pbCancel_clicked(){ reject(); };
  void on_pbClose_clicked(){
    if(dataAcceptable()){apply(); emit endRequest();return;}}; 
  void slotInsertGraph();
  void slotDeleteGraph();
  void slotMoveGraph();
  void on_lvGraphs_currentItemChanged(const QModelIndex &
				      ,const QModelIndex &);
  void onContentChanged(bool changed){pbApply->setEnabled(changed);}

  void onToolTriggered(const QString &);

private:
   bool dataAcceptable();
   void apply();
   bool protectChanges(bool withCancel=true);
   void setAddDeleteEnabed(const QModelIndex &);

private:
   Graphs *t_graphs;
   Graph *t_graph;
   GraphsModel *grsmodel;
   bool mute_currentIndexChanged;


  ToolButtons ptoolbuttons;

};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _GRAPHDLG_H */

