/*************************************************************************
 * @(#)GraphPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   25 Nov 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QMenu>
#include "Except.hh"
#include "GraphPropWdg.hh"
#include "Graph.hh"
#include "ViewBox.hh"
#include "Valid.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphPropWdg::GraphPropWdg(QWidget *parent)
  :PropWdgBase(parent)
{
  setupUi(this);

  connectChanged(this);

  new RatValidator(this,leScale,"scale",1.0e-10,1.e10);

  QMenu *m=new QMenu(tbScale);

  cscaleallaction=m->addAction("Scale &whole graph");
  cscaleviewaction=m->addAction("Scale &view frame");

  connect(cscaleallaction,SIGNAL(triggered())
	  ,this,SLOT(onCScaleActionsTriggered()));;
  connect(cscaleviewaction,SIGNAL(triggered())
	  ,this,SLOT(onCScaleActionsTriggered()));;


  tbScale->setMenu(m);
  tbScale->setDefaultAction(cscaleallaction);

  setDirty(false);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphPropWdg::~GraphPropWdg()
{
  if(cscaleallaction)  delete cscaleallaction;
  if(cscaleviewaction)  delete cscaleviewaction;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphPropWdg::check(QString &msg)
{
  ((RatValidator*)leScale->validator())->check(msg);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphPropWdg::dataAcceptable()
{
  QString txt="";
  check(txt);
  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    return false;
  }
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphPropWdg::setData(void *go)
{
  Graph *gr=0;
  //todo: if(!go) PropWdgBase::reset(this);
  if(go) gr=dynamic_cast<Graph*>((Graph *)go);
  t_graph=gr;
  if(!t_graph)return;

  cbActive->setChecked(t_graph->active());

  lGraph->setText(t_graph->guiName(true)());
  dspbxPointScale->setValue(gr->allOverPointScale());
  dspbxLineWidthScale->setValue(gr->allOverLineWidthScale());
  leScale->setText("1");

  //t_graph->setChanged(false);
  setDirty(false);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphPropWdg::applyData(void *go)
{
  if(!go) return;
  Graph *gr=0;
  if(go) gr=dynamic_cast<Graph*>((Graph *)go);
  if(!gr)return;

  t_graph->setActive(cbActive->isChecked());
  t_graph->setAllOverPointScale(dspbxPointScale->value());
  t_graph->setAllOverLineWidthScale(dspbxLineWidthScale->value());

  setDirty(false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphPropWdg::updateRequest(void *)
{
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphPropWdg::onCScaleActionsTriggered()
{
  //myDebug() << sender();
  if(!dataAcceptable()) return;
  double scale;
  try{
    scale=String(leScale->text()).convertToFortranReal();
  }catch(Except &e){scale=1.;};

  tbScale->setDefaultAction((QAction*)sender());
  if(sender()==cscaleallaction){
    ViewBox *vb=t_graph->viewBox();
    Size size=vb->size();
    vb->setSize(size*scale);
    dspbxPointScale->setValue(dspbxPointScale->value()*scale);
    t_graph->setAllOverPointScale(dspbxPointScale->value());
    emit toolTriggered("scale all");
  }else if(sender()==cscaleviewaction){
    ViewBox *vb=t_graph->viewBox();
    Size size=vb->size();
    vb->setSize(size*scale);
    emit toolTriggered("scale view");
  }
}


/*EOF*/ 
