/*************************************************************************
 * @(#)GraphsModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   25 Nov 2011
 *************************************************************************
 */

#include "Page.hh"
#include "Graph.hh"
#include "Graphs.hh"
#include "GraphsModel.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphsModel::GraphsModel(QObject * parent)
  : QAbstractListModel(parent),pgraphs(0)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphsModel::reset(Graphs *graphs)
{
  if(graphs) pgraphs=graphs;
  QAbstractListModel::reset();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant GraphsModel::headerData(int section, Qt::Orientation orientation
				    , int role) const
{
  Q_UNUSED(section);
  if (role != Qt::DisplayRole)
    return QVariant();
  if (orientation == Qt::Horizontal) {
    return "Header";
  }
  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int GraphsModel::rowCount(const QModelIndex & parent) const
{
  Q_UNUSED(parent);
  if(!pgraphs) return 0;
  return pgraphs->size();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant GraphsModel::data(const QModelIndex &index,int role)const
{
  if(!pgraphs) return QVariant();
  if (!index.isValid()) return QVariant();
  int row=index.row();
  //int col=0;
  if (row >= pgraphs->size() || row < 0) return QVariant();

  const Graph *gr=pgraphs->at(row);
  
  if (role == Qt::DisplayRole) {
    
    return gr->guiName()();
  }

  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphsModel::insertRow(int &row,Graph *gr)
{
  if(!pgraphs) return false;
  if(row<0){
    row=pgraphs->size();
  }
  beginInsertRows(QModelIndex(),row,row);
  pgraphs->insertGraph(row,gr);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphsModel::insertRow(int &row)
{
  if(!pgraphs) return false;
  if(row<0){
    row=pgraphs->size();
  }
  beginInsertRows(QModelIndex(),row,row);
  pgraphs->insertGraph(row);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphsModel::removeRow(int row)
{
  if(!pgraphs) return;
  if(row<0 || row>=pgraphs->size()) return;
  
  
  beginRemoveRows(QModelIndex(), row, row);
  pgraphs->deleteGraph(row);
  endRemoveRows();
  return;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool GraphsModel::moveRow(int row,int by)
{
  if(!pgraphs) return false;
  if(!index(row+by,0,QModelIndex()).isValid()) return false;

  Graph *pfrom=pgraphs->at(row);
  Graph *pto=pgraphs->at(row+by);
  if(!pfrom) return false;
  if(!pto) return false;

  
  beginRemoveRows(QModelIndex(), row, row);
  Graph *p=pgraphs->takeAt(row);
  endRemoveRows();
  
  int r=MyMath::max<long>(0,row+by);
  insertRow(r,p);

  return true;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Graph *GraphsModel::currentGraph(const QModelIndex &index)const
{
  if(!pgraphs) return 0;
  if(!index.isValid())return 0;
  return pgraphs->at(index.row());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QModelIndex GraphsModel::graphIndex(Graph *gr)const
{
  if(!gr) return this->index(0,0,QModelIndex());
  return this->index(pgraphs->indexOf(gr),0,QModelIndex());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphsModel::resetItem(Graph *gr)
{
  if(!gr) return;
  QModelIndex index=this->index(pgraphs->indexOf(gr),0,QModelIndex());
  emit dataChanged(index,index);
}



/*EOF*/ 



/*EOF*/ 
