/*************************************************************************
 * @(#)LegendBoxPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Apr 2011
 *************************************************************************
 */

#include <QColor>
#include "Attributes.hh"
#include "LegendBox.hh"
#include "LegendBoxDlg.hh"
#include "LegendBoxPropWdg.hh"
#include "Valid.hh"
#include "Config.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
LegendBoxDlg::LegendBoxDlg(QWidget *parent,QWidget *base,const char *name)
  : DlgBase(parent,base,name)
{
  setWindowTitle(IDS_APP_NAME + ": LegendBox settings");

  if(wProps) delete wProps;
  wProps=new LegendBoxPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
LegendBoxPropWdg::LegendBoxPropWdg(QWidget *parent)
  :PropWdgBase(parent)
{
  //  myDebug()<<"hier";
  setupUi(this);

  QString s="Text props ";
  new RealValidator(this,lSubsc->buddy(),s+lSubsc->text(),0.,100.);
  new RealValidator(this,lLineSpacing->buddy(),s+lLineSpacing->text(),1.,-1.);

  new RealValidator(this,lSymbolSpacing->buddy(),s+lSymbolSpacing->text(),1.,-1.);
  new RealValidator(this,lSymbolWidth->buddy(),s+lSymbolWidth->text(),1.,-1.);

  new RealValidator(this,lBorder->buddy(),"Border "+lBorder->text(),1.,-1.);
  
  s="Position ";
  new RealValidator(this,lXPos->buddy(),s+lXPos->text(),1.,-1.);
  new RealValidator(this,lYPos->buddy(),s+lYPos->text(),1.,-1.);
  s="Box origin ";
  new RealValidator(this,lXOrigin->buddy(),s+lXOrigin->text(),0.,1.);
  new RealValidator(this,lYOrigin->buddy(),s+lYOrigin->text(),0.,1.);

  connectChanged(this);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
LegendBoxPropWdg::~LegendBoxPropWdg()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBoxPropWdg::check(QString &msg)
{
  ((IntValidator*)subscLE->validator())->check(msg);
  ((IntValidator*)leLineSpacing->validator())->check(msg);

  ((IntValidator*)leSymbolSpacing->validator())->check(msg);
  ((IntValidator*)leSymbolWidth->validator())->check(msg);

  ((IntValidator*)leBorder->validator())->check(msg);
  ((IntValidator*)leXPos->validator())->check(msg);
  ((IntValidator*)leYPos->validator())->check(msg);
  ((IntValidator*)leXOrigin->validator())->check(msg);
  ((IntValidator*)leYOrigin->validator())->check(msg);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBoxPropWdg::setData( void *go )
{
  //myDebug()<<"LegendBoxPropWdg::setData"<<go;
  LegendBox *lb=0;
  if(!go) PropWdgBase::reset(this);
  if(go) lb=dynamic_cast<LegendBox*>((GraphObject *)go);
  if(!lb) return;
  plb=lb;

  QWidget *fw=this->focusWidget();
  if(fw)
    fw->setFocus();
  else
    dspbxFsz->setFocus();

  mutecontentchanged=true;

  cbActive->setChecked(lb->active());

  dspbxFsz->setValue(lb->textStyle()->size());
  subscLE->setText(QString().setNum(lb->textStyle()->subScriptScale(),'g',10));  
  tbColor->setColor(lb->textStyle()->color());


  leLineSpacing->setText(QString().setNum(lb->lineSpacing()));

  leSymbolSpacing->setText(QString().setNum(lb->symbolSpacing()));
  leSymbolWidth->setText(QString().setNum(lb->symbolWidth()));

  cbDrawFrame->setChecked(lb->drawFrame());

  cmbLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }
  cmbLineStyle->setCurrentIndex(lb->rim()->style());  
  spbxLineWidth->setValue(lb->rim()->width());
  tbRimColor->setColor(lb->rim()->color());


  cbFillFrame->setChecked(lb->fill()->fill());
  tbFillColor->setColor(lb->fill()->color());
  

  leBorder->setText(QString().setNum(lb->border(),'g',10));  


  leXPos->setText(QString().setNum(lb->pos().x,'g',10));  
  leYPos->setText(QString().setNum(lb->pos().y,'g',10));  
  leXPos->setCursorPosition(0);
  leYPos->setCursorPosition(0);

  leXOrigin->setText(QString().setNum(lb->origin().x,'g',10));  
  leYOrigin->setText(QString().setNum(lb->origin().y,'g',10));  

  lb->setChanged(false);

  mutecontentchanged=false;
  setDirty(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBoxPropWdg::applyData(void *go)
{
  //NSF("applydata");
  LegendBox *lb=0;
  if(go) lb=dynamic_cast<LegendBox*>((GraphObject *)go);
  if(!lb) return;


  lb->setActive(cbActive->isChecked());

  ;
  QString t;
  lb->textStyle()->setSize(dspbxFsz->value());

  t=subscLE->text(); IntValidator::stripAllWhiteSpace(t);
  lb->textStyle()->setSubScriptScale(t.toDouble());

  lb->textStyle()->setColor(tbColor->color());

  t=leLineSpacing->text();IntValidator::stripAllWhiteSpace(t);
  lb->setLineSpacing(t.toDouble());

  t=leSymbolSpacing->text();IntValidator::stripAllWhiteSpace(t);
  lb->setSymbolSpacing(t.toDouble());
  t=leSymbolWidth->text();IntValidator::stripAllWhiteSpace(t);
  lb->setSymbolWidth(t.toDouble());

  lb->setDrawFrame(cbDrawFrame->isChecked());


  
  lb->rim()->setStyle((Xfp::LineStyles)cmbLineStyle->currentIndex());
  lb->rim()->setWidth(spbxLineWidth->value());
  lb->rim()->setColor(tbRimColor->color());

  
  
  lb->fill()->setFill(cbFillFrame->isChecked());
  lb->fill()->setColor(tbFillColor->color());

  t=leBorder->text() ; IntValidator::stripAllWhiteSpace(t);
  lb->setBorder(t.toDouble());

  

 


  t=leXPos->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYPos->text();IntValidator::stripAllWhiteSpace(t2);
  lb->setPos(Point(t.toDouble(),t2.toDouble()));

  t=leXOrigin->text();IntValidator::stripAllWhiteSpace(t);
  t2=leYOrigin->text();IntValidator::stripAllWhiteSpace(t2);
  lb->setOrigin(Point(t.toDouble(),t2.toDouble()));
  
  

  setDirty(false);
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBoxPropWdg::updateRequest(void *go)
{
  LegendBox *lb=0;
  if(go) lb=dynamic_cast<LegendBox*>((GraphObject *)go);
  if(!lb) return;
  if(plb->graph()!=lb->graph()) return;
  bool di=isDirty();
  if(lb->updateRequest() & GraphObject::GOCPosition){
    leXPos->setText(QString().setNum(lb->pos().x,'g',10));  
    leYPos->setText(QString().setNum(lb->pos().y,'g',10));  
    leXPos->setCursorPosition(0);
    leYPos->setCursorPosition(0);

  }
  if(!di) setDirty(false);

}

/*EOF*/ 
