/*************************************************************************
 * @(#)PageSetupDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   14 Oct 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "PageSetupDlg.hh"
#include "Page.hh"
#include "Paper.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PageSetupDlg::PageSetupDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);

  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);

  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PageSetupDlg::~PageSetupDlg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::onDlgEnd()
{
  //myDebug()<<"PageSetupDlg::onDlgEnd"<<this;
  deleteLater();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_pbApply_clicked()
{
  //myDebug() <<"PageSetupDlg::on_pbApply_clicked";
  if(dataAcceptable()){
    apply();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_pbClose_clicked()
{
  //myDebug() <<"PageSetupDlg::on_pbClose_clicked";
  if(dataAcceptable()){
    apply();
    emit endRequest();
    return;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool PageSetupDlg::dataAcceptable()
{
  QString s,txt="";

  //wProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    return false;
  }

  
  return true;
 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::apply()
{
  Paper oldpaper=ppage->paper();
  ppaper.setCustomWidth(pwidth);
  ppaper.setCustomHeight(pheight);

  ppage->paper()=ppaper;
  ppage->refreshPaperSize(oldpaper,false);

  emit acceptRequest();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::setData(Page *p)
{
  ppage=p;
  ppaper=ppage->paper();
  mute=true;
  cmbSize->clear();
  int i=0;
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Custom),Paper::Custom);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A1),Paper::A1);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A2),Paper::A2);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A3),Paper::A3);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A4),Paper::A4);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A5),Paper::A5);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::A6),Paper::A6);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Letter),Paper::Letter);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::LetterSmall),Paper::LetterSmall);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Legal),Paper::Legal);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Tabloid),Paper::Tabloid);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Folio),Paper::Folio);
  cmbSize->insertItem(i++,Paper::paperSizeName(Paper::Quarto),Paper::Quarto);

  cmbOrientation->clear();
  i=0;
  cmbOrientation->insertItem(i++,Paper::orientationTypeName(Paper::Portrait),
			     Paper::Portrait);
  cmbOrientation->insertItem(i++,Paper::orientationTypeName(Paper::Landscape),
			     Paper::Landscape);

  mute=false;
  cmbSize->setCurrentIndex(cmbSize->findData(ppaper.paperSize()));
  cmbOrientation->setCurrentIndex(cmbOrientation->findData(ppaper.orientation()));
  pwidth=ppaper.customWidth();
  pheight=ppaper.customHeight();
  spbxWidth->setValue(ppaper.paperWidth());
  spbxHeight->setValue(ppaper.paperHeight());

  if(ppaper.paperSize()==Paper::Custom){
    cmbOrientation->setEnabled(false);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_cmbSize_currentIndexChanged(int current)
{
  if(mute)return;
  Paper::Size ps=(Paper::Size)cmbSize->itemData(current).toInt();
  ppaper.setPaperSize(ps);
  if(ps==Paper::Custom){
    spbxWidth->setEnabled(true);
    spbxHeight->setEnabled(true);
    spbxWidth->setValue(pwidth);
    spbxHeight->setValue(pheight);
    cmbOrientation->setEnabled(false);
  }else{
    spbxWidth->setEnabled(false);
    spbxHeight->setEnabled(false);
    spbxWidth->setValue(ppaper.paperWidth());
    spbxHeight->setValue(ppaper.paperHeight());
    cmbOrientation->setEnabled(true);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_cmbOrientation_currentIndexChanged(int current)
{
  if(mute)return;
  Paper::Orientation po=(Paper::Orientation)cmbOrientation->itemData(current).toInt();
  ppaper.setOrientation(po);
  if(ppaper.paperSize()==Paper::Custom){
    spbxWidth->setValue(pwidth);
    spbxHeight->setValue(pheight);
  }else{
    spbxWidth->setValue(ppaper.paperWidth());
    spbxHeight->setValue(ppaper.paperHeight());
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_spbxWidth_valueChanged(int v)
{
  if(spbxWidth->isEnabled()){
    pwidth=v;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PageSetupDlg::on_spbxHeight_valueChanged(int v)
{
  if(spbxHeight->isEnabled()){
    pheight=v;
  }
}


/*EOF*/ 
