/*************************************************************************
 * @(#)PropWdgBase.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   29 Apr 2011
 *************************************************************************
 */

#include <QLineEdit>
#include <QCheckBox>
#include <QComboBox>
#include <QGroupBox>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include "ColorButton.hh"
#include "PropWdgBase.hh"
#include "defs.hh"

#define DOIT(x)							\
  printf("PropWdgBase: need to implement a method %s\n",#x);	\
      exit(1);\


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropWdgBase::connectChanged(QObject *obj,QList<QObject*> exceptlist)
{
  NSF("PropWdgBase::connectChanged(QObject *obj)");

  pconnected=true;
  
  bool stop=false;
  //myDebug()<<obj->metaObject()->className();
  foreach(QObject *o,obj->children()){
    if(o->isWidgetType()){
      QString isA(o->metaObject()->className());
      //myDebug()<<"\t"<<isA;
      if(exceptlist.contains(o)) continue;
      if(isA=="QLabel") continue;
      if(isA=="QMenu") continue;
      if(isA=="QScrollBar") continue;
      if(isA=="QTabBar") continue;
      if(isA=="QHeaderView") continue;
      if(isA=="QTableView") continue;
      if(isA=="QTableCornerButton") continue;

      if(isA=="QLineEdit"){
 	connect(o,SIGNAL(textChanged(const QString &))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QRadioButton"){
 	connect(o,SIGNAL(toggled(bool))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QListWidget"){ 
 	connect(o,SIGNAL(itemChanged(QListWidgetItem*))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="WeightSelListView"){ 
 	connect(o,SIGNAL(myDataChanged())
 		,this,SLOT(onContentChanged()));
      }else if(isA=="ColorButton"){ 
 	connect(o,SIGNAL(colorChanged(QRgb))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QToolButton"){
  	connect(o,SIGNAL(clicked())
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QComboBox"){
  	connect(o,SIGNAL(currentIndexChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QSpinBox"){
  	connect(o,SIGNAL(valueChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QDoubleSpinBox"){
  	connect(o,SIGNAL(valueChanged(double))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QCheckBox"){
  	connect(o,SIGNAL(stateChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QGroupBox"){
  	connect(o,SIGNAL(toggled(bool))
  		,this,SLOT(onContentChanged()));
      }else if(!(isA=="QTabWidget"||
 		 isA=="QFrame"||
 		 isA=="QWidget"||
 		 isA=="Oxygen::TransitionWidget"||
 		 isA=="Oxygen::SunkenFrameShadow"||
 		 isA=="UserTicWdg"||
 		 isA=="QStackedWidget" )){
	stop=true;
	myDebug()<<nsubfunc<<isA<<"not implemented";
      }
    }
    if(o->children().size()){
      connectChanged(o,exceptlist);
    }
  }

  if(stop){
    //    exit(1);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropWdgBase::reset(QObject *obj)
{
  NSF("PropWdgBase::reset(QObject *obj)");

  bool stop=false;
  foreach(QObject *o,obj->children()){
    if(o->isWidgetType()){
      QString isA(o->metaObject()->className());

      if(isA=="QLabel") continue;
      if(isA=="QMenu") continue;
      if(isA=="QScrollBar") continue;
      if(isA=="QTabBar") continue;
      if(isA=="QHeaderView") continue;
      if(isA=="QTableView") continue;
      if(isA=="QTableCornerButton") continue;

      if(isA=="QLineEdit"){
	((QLineEdit*)o)->setText("");
//       }else if(isA=="WeightSelListView"){ 
//  	connect(o,SIGNAL(myDataChanged())
//  		,this,SLOT(onContentChanged()));
      }else if(isA=="ColorButton"){ 
	((ColorButton*)o)->setColor(0x000000);
      }else if(isA=="QComboBox"){
	((QComboBox*)o)->setCurrentIndex(0);
      }else if(isA=="QSpinBox"){
	((QSpinBox*)o)->setValue(0);
      }else if(isA=="QDoubleSpinBox"){
	((QSpinBox*)o)->setValue(0);
      }else if(isA=="QCheckBox"){
	((QCheckBox*)o)->setChecked(false);
      }else if(isA=="QGroupBox"){
	((QGroupBox*)o)->setChecked(false);
      }else if(!(isA=="QTabWidget"||
		 isA=="QFrame"||
  		 isA=="QWidget"||
  		 isA=="Oxygen::TransitionWidget"||
  		 isA=="Oxygen::SunkenFrameShadow"||
  		 isA=="UserTicWdg"||
  		 isA=="QToolButton"||
  		 isA=="QListWidget"||
  		 isA=="QRadioButton"||
  		 isA=="QStackedWidget" )){
 	stop=true;
 	myDebug()<<nsubfunc<<isA<<"not implemented";
      }
    }
    if(o->children().size()){
      reset(o);
    }
  }
  if(stop){
    //    exit(1);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void PropWdgBase::check(QString &)
{
  DOIT(check);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropWdgBase::setData(void *)
{
  DOIT(setData);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropWdgBase::applyData(void *)
{
  DOIT(applyData);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropWdgBase::updateRequest(void *)
{
  DOIT(updateRequest);
}



/*EOF*/ 
