//todo: add delete witg report to ticmarks
//todo: reload logic
/*************************************************************************
 * @(#)SetDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 May 2011
 *************************************************************************
 */

#include <QMenu>
#include <QMessageBox>
#include "ReselectEvent.hh"
#include "WeightSetDataEvent.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "Graph.hh"
#include "Graphs.hh"
#include "Page.hh"
#include "SetDlg.hh"
#include "SetPropWdg.hh"
#include "SetTreeModel.hh"
#include "SetTreeSelectionModel.hh"
#include "Attributes.hh"
#include "FString.hh"
#include "ToolButtons.hh"
#include "Config.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SetDlg::SetDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),stmodel(0),t_graph(0),t_page(0),
    mute_currentIndexChanged(false),pfirstcall(true),
    ptoolbuttons(new ToolButtons())
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);

  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);


  
  if(wSetProps) delete wSetProps;
  wSetProps=new SetPropWdg(this);
  propVerticalLayout->insertWidget(1,wSetProps,10);

  stmodel=new SetTreeModel(this);
  stselmodel=new SetTreeSelectionModel(stmodel);
  tvSets->setModel(stmodel);
  tvSets->setSelectionModel(stselmodel);

  QMenu *m=new QMenu(tbTools);
  QAction *a,*ad;
  toolactions.append(ad=a=m->addAction("Set different &colors"));
  connect(a,SIGNAL(triggered())
	  ,this,SLOT(onToolsActionsTriggered()));;
  toolactions.append(a=m->addAction("&Pack sets/groups"));
  connect(a,SIGNAL(triggered())
	  ,this,SLOT(onToolsActionsTriggered()));;


  tbTools->setMenu(m);
  //foreach(QAction *a,toolactions){
  tbTools->setDefaultAction(ad);
  //}

  ptoolbuttons->createToolButtonsActions(tvSets,"set(s)/group(s)"
					 ,0,tbDelete,tbUp,tbDown);
  tbInsert->hide();
  ptoolbuttons->connectSlots(this,0,
			     SLOT(slotDeleteSetGroupSelection()),
			     SLOT(slotMoveSetGroupSelection()),
			     SLOT(slotMoveSetGroupSelection()));

  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
SetDlg::~SetDlg()
{
  if(stmodel) delete stmodel;stmodel=0;
  if(ptoolbuttons) delete ptoolbuttons;ptoolbuttons=0;

  qDeleteAll(toolactions);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void on_pbHelp_clicked()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SetDlg::dataAcceptable()
{
  QString s,txt="";

  wSetProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  //myDebug() << "END BondsDlg::dataAcceptable";

  return true;
 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::apply(bool considercurrent)
{
  //myDebug()<<"";
  QList<FGObject *> fgolist;
  foreach(QModelIndex index,tvSets->selectionModel()->selectedIndexes()){
    //myDebug()<<index;
    fgolist.append(stmodel->getFGObject(index));
  }
  if(considercurrent){
    QModelIndex cindex=tvSets->currentIndex();
    if(cindex.isValid()){
      //myDebug()<<cindex;
      FGObject *fgo=stmodel->getFGObject(cindex);
      if(!fgolist.contains(fgo)){
	fgolist.append(fgo);
      }
    }
  }
  wSetProps->applyData(fgolist);
  foreach(QModelIndex index,tvSets->selectionModel()->selectedIndexes()){
    //myDebug()<<index;
    tvSets->update(index);
  }
  if(considercurrent){
    QModelIndex cindex=tvSets->currentIndex();
    if(cindex.isValid()){
      tvSets->update(cindex);
    }
  }
  emit acceptRequest(t_graph,false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QList<FGObject*> SetDlg::getSelectedFGObjects(bool considercurrent)
{
  //myDebug()<<"";
  QList<FGObject *> fgolist;
  foreach(QModelIndex index,tvSets->selectionModel()->selectedIndexes()){
    //myDebug()<<index;
    fgolist.append(stmodel->getFGObject(index));
  }
  if(considercurrent){
    QModelIndex cindex=tvSets->currentIndex();
    if(cindex.isValid()){
      //myDebug()<<cindex;
      FGObject *fgo=stmodel->getFGObject(cindex);
      if(!fgolist.contains(fgo)){
	fgolist.append(fgo);
      }
    }
  }
  return fgolist;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::onContentChanged(bool )
{
  //pbApply->setEnabled(changed);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::reloadDialog()
{
  stmodel->resetData(t_graph);
  // todo: avoid warning on change
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::setData(Page *page,FGObject *fgo)
{
  //  myDebug()<<"SetDlg:: fgo"<<fgo;


  if(wSetProps->isDirty()){
    if(protectChanges(false)) return;
  }

  t_page=page;
  pfirstcall=(t_graph!=page->graphs()->currentGraph());
  if(!fgo){
    t_graph=page->graphs()->currentGraph();
  }else{
    t_graph=fgo->graph();
  }

  enableToolButtons();

  QModelIndex cindex=tvSets->currentIndex();
  stmodel->resetData(t_graph);
  if(!pfirstcall) {
    if(fgo){
      selectFGObject(fgo);
    }else{
      selectFGObject(stmodel->getFGObject(cindex));
    }
    return;
  }
  wSetProps->setDirty(false); 
  if(t_graph->groups()->size()==0){
    lSet->setText(t_graph->guiName(true)());
    propVerticalLayout->setEnabled(false);
    wSetProps->setData(0,false);
  }else{
    selectFGObject(fgo);
  }
  pfirstcall=false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::selectFGObject(FGObject *fgo)
{
  if(!fgo){
    tvSets->setCurrentIndex(stmodel->index(0,0,QModelIndex()));
  }else{
    if(fgo->type()==FGObject::FGOTGroup){
      tvSets->setCurrentIndex(stmodel->index(fgo,QModelIndex()));
    }else{
      FunctionGroup *fg=((Function*)fgo)->group();
      QModelIndex parent=stmodel->index(fg,QModelIndex());
      tvSets->setCurrentIndex(stmodel->index(fgo,parent));
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::on_tvSets_myCurrentChanged(const QModelIndex &current,
					const QModelIndex &previous)
{
  if(!t_graph) return;
  
//   tvSets->selectionModel()->select(current,QItemSelectionModel::Select|
// 				   QItemSelectionModel::Current);

  if(mute_currentIndexChanged)return;
  if(protectChanges()){
    QEvent *event=new ReselectEvent(ReselectEvent::ReselectEventType,previous);
    QCoreApplication::postEvent(this,event);
    return;
  }
  FGObject *fgo=stmodel->getFGObject(current);
  if(!fgo) return;

  if(fgo->type()==FGObject::FGOTGroup){
    lSet->setText(QString("%1,%2")
		  .arg(t_graph->guiName(true)())
		  .arg(fgo->guiName(true)())
		  );
    lSetComment->setText(fgo->comment()());
    lType->setText("");

    
  }else if(fgo->type()==FGObject::FGOTFunction){
    
    Function *fu=(Function*)fgo;
    FunctionGroup *fg=fu->group();
    lSet->setText(QString("%1,%2/%3")
		  .arg(t_graph->guiName(true)())
		  .arg(fg->guiName(true)())
		  .arg(fu->guiName(true)())
		  );
    
    lSetComment->setText(fgo->comment()());
    lType->setText(
		   Sprintf("%s, length %ld",
			   Function::dataTypeGuiName(fu->dataType()),
			   fu->length()
			   )());
    //todo: if grid
  }

  QEvent *event=new 
    WeightSetDataEvent(WeightSetDataEvent::WeightSetDataEventType,fgo);
  QCoreApplication::postEvent(this,event);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SetDlg::event(QEvent *event)
{
  //  NSF("SetDlg::event");
  if(event->type()==ReselectEvent::ReselectEventType){
    if(ReselectEvent *re=dynamic_cast<ReselectEvent*>(event)){
      mute_currentIndexChanged=true;
      tvSets->setCurrentIndex(re->index());
      mute_currentIndexChanged=false;
    }
    return true;
  }else if(event->type()==WeightSetDataEvent::WeightSetDataEventType){
    if(WeightSetDataEvent *we=dynamic_cast<WeightSetDataEvent*>(event)){
      bool weightsallowed=true;
      bool start=true;
      WeightLabels *wls=0;
      //myDebug()<<"";
      foreach(QModelIndex index,tvSets->selectionModel()->selectedIndexes()){
	FGObject *fgo1=stmodel->getFGObject(index);
	if(start){
	  wls=fgo1->weightLabels();
	  start=false;
	  //myDebug()<<"index"<<index<<weightsallowed;
	}else{
	  if(wls!=fgo1->weightLabels()){
	    weightsallowed=false;
	    //myDebug()<<"index"<<index<<weightsallowed<<fgo1->weightLabels();
	    //break;
	  }
	}
      }
      wSetProps->setData(we->fgo(),weightsallowed);
    }
    return true;
  }
  return BrainDialog::event(event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SetDlg::protectChanges(bool withCancel)
{
  if(wSetProps->isDirty()){
    QString s="The content of the dialog changed.\n"
      "(Use the dialog's apply button for"
      " confirming changes.)";
    int ret;
    if(withCancel)
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard,QMessageBox::Cancel);
    else{
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard);
    }
    if(ret==QMessageBox::Apply){
      if(dataAcceptable()){
	apply(false);
      }else{
 	return true;
      }
    }else if(ret==QMessageBox::Cancel){
      return true;
    }else if(ret==QMessageBox::Discard){
      wSetProps->setDirty(false);
    }
  }
  return false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::onToolsActionsTriggered()
{
  tbTools->setDefaultAction((QAction*)sender());

  if(sender()==toolactions.at(0)){
    //myDebug()<<sender();
    QModelIndex cindex=tvSets->currentIndex();
    bool saveddirty=wSetProps->isDirty();
    long i=0;
    foreach(QModelIndex index,tvSets->selectionModel()->selectedIndexes()){
      FGObject *fgo1=stmodel->getFGObject(index);
      QRgb cc=Xfp::rgbColor(i % (Xfp::maxRgbColors()-1));
      fgo1->attribs()->lineStyle().setColor(cc);
      if(cindex.isValid()){
	//myDebug()<<cindex;
	FGObject *fgo=stmodel->getFGObject(cindex);
	if(fgo==fgo1){
	  wSetProps->setSelectedProps("linecolor");
	}
      }
      i++;
    }
    wSetProps->setDirty(saveddirty);
    emit acceptRequest(t_graph,false);

  }else if(sender()==toolactions.at(1)){
    stmodel->packIDs();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static bool milLessThan(const QModelIndex &m1,const QModelIndex &m2)
{
  
  if(!m1.parent().isValid() && !m2.parent().isValid()){
    // both m1 and m2 are groups or sets
    return m1.row()<m2.row();
  }
  if(!m1.parent().isValid() && m2.parent().isValid()){
    // m1 is group and m2 is set
    return (m1.row()<m2.parent().row());
  }
  if(m1.parent().isValid() && !m2.parent().isValid()){
    // m1 is set and m2 is group
    return m1.parent().row()<m2.row();
  }
  if( m1.parent().isValid() &&  m2.parent().isValid()){
    if(m1.parent()==m2.parent()){
      return m1.row()<m2.row();
    }else
      return m1.parent().row()<m2.parent().row();
  }

  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static bool milLessThanDelete(const QModelIndex &m1,const QModelIndex &m2)
{
  
  if(!m1.parent().isValid() && !m2.parent().isValid()){
    // both m1 and m2 are groups or sets
    return m1.row()<m2.row();
  }
  if(!m1.parent().isValid() && m2.parent().isValid()){
    // m1 is group and m2 is set
    return true;
    //return (m1.row()<m2.parent().row());
  }
  if(m1.parent().isValid() && !m2.parent().isValid()){
    // m1 is set and m2 is group
    return false;
    //return m1.parent().row()<m2.row();
  }
  if( m1.parent().isValid() &&  m2.parent().isValid()){
    if(m1.parent()==m2.parent()){
      return m1.row()<m2.row();
    }else
      return m1.parent().row()<m2.parent().row();
  }

  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::slotMoveSetGroupSelection()
{
  //myDebug()<<"slotMoveSetGroupSelection";
  
  long by=-1;
  if(sender()==ptoolbuttons->pmoveDownAction)by=1;
  
  QModelIndexList mil=tvSets->selectionModel()->selectedIndexes();
  qStableSort(mil.begin(),mil.end(),milLessThan);
  if(mil.size()<=0) return;

  //todo: anable buttons depending on selection
  
  long gcount=0,fcount=0;
  foreach(QModelIndex index,mil){
    if(index.parent()==QModelIndex()){
      gcount++;
    }else{
      fcount++;
    }
  }
  
  if(fcount*gcount>0){
    QMessageBox::warning(this,IDS_APP_NAME,"Cannot move groups and "
			 "sets at the same time.", 1, 0,0 ) ;
    return;
  }


  QModelIndex firstgroupindex
    =stmodel->index(t_graph->groups()->at(0),QModelIndex());
  QModelIndex lastgroupindex
    =stmodel->index(t_graph->groups()->last(),QModelIndex());
  
 

  if(by<0){
    if((mil.first().row()==0 &&
	mil.first().parent()==firstgroupindex)||
       mil.first()==firstgroupindex) return;
    QModelIndex lastparent;
    long offset=0;
    for(long i=0;i<mil.size();i++){
      QModelIndex &index=mil[i];
      QModelIndex parent=index.parent();
      if(lastparent!=parent){
	offset=0;
	lastparent=parent;
      }
      int row=index.row()+offset;
      if((mil[i]=stmodel->moveRow(parent,row,by))!=QModelIndex() && row==0)
	  offset--;
    }  
    
    tvSets->setCurrentIndex(stmodel->index(mil.first().row(),
					   0,mil.first().parent()));
  }else{
    if((mil.last().row()==stmodel->rowCount(lastgroupindex)-1 &&
	mil.last().parent()==lastgroupindex)||
       mil.last()==lastgroupindex) return;

    for(long i=mil.size()-1;i>=0;i--){
      QModelIndex &index=mil[i];
      QModelIndex parent=index.parent();
      int row=index.row();
      bool atend=(row==stmodel->rowCount(parent)-1);
      mil[i]=stmodel->moveRow(parent,row,by);
      if(atend){
	for(int j=i+1;j<mil.size();j++){
	  if(mil[j].parent()!=mil[i].parent()) break;
	  mil[j]=stmodel->index(mil[j].row()+1,0,mil[j].parent());
	}
      }
    }  
    tvSets->setCurrentIndex(stmodel->index(mil.first().row(),
					   0,mil.first().parent()));
  }

  foreach(QModelIndex index,mil){
    tvSets->selectionModel()
      ->select(stmodel->index(index.row(),0,index.parent())
	       ,QItemSelectionModel::Select);
  }
  emit acceptRequest(t_graph,false);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SetDlg::slotDeleteSetGroupSelection()
{

  if(QMessageBox::warning(this,IDS_APP_NAME,"Really delete",
			  QMessageBox::Ok,QMessageBox::Cancel)
     !=QMessageBox::Ok) return;

  //  myDebug()<<"slotDeleteSetGroupSelection";
  QModelIndexList mil=tvSets->selectionModel()->selectedIndexes();
  qStableSort(mil.begin(),mil.end(),milLessThanDelete);
  

  if(mil.size()<=0) return;






    
  for(long i=mil.size()-1;i>=0;i--){
    QModelIndex index=mil.at(i);
    QModelIndex parent=index.parent();
    int row=index.row();
    FGObject *fgo=stmodel->getFGObject(index);
    stmodel->removeRow(parent,row);
    
  }


  QModelIndex cpindex;
  QModelIndex cindex;
  
  for(long i=mil.size()-1;i>=0;i--){
    QModelIndex index=mil.at(i);
    QModelIndex parent=index.parent();
    FGObject *pfgo=stmodel->getFGObject(parent);
    QModelIndex in=stmodel->index(pfgo,QModelIndex());
    if(in.isValid()){
      cpindex=in;
      break;
    }
  }
  if(!cpindex.isValid()){
    cindex=stmodel->index(stmodel->rowCount(QModelIndex())-1,0,QModelIndex());
  }else{
    if(stmodel->rowCount(cpindex)>0){
      long ndel=0;
      for(long i=mil.size()-1;i>=0;i--){
	if(stmodel
	   ->index(stmodel->getFGObject(mil.at(i).parent()),QModelIndex())!=cpindex) 
	  continue;
	ndel++;
      }
      
      for(long i=mil.size()-1;i>=0;i--){
	QModelIndex index=mil.at(i);
	QModelIndex parent=index.parent();
	QModelIndex in=stmodel->index(stmodel->getFGObject(parent),QModelIndex());
	if(in!=cpindex) continue;
	long row=index.row()-ndel;
	cindex=stmodel
	  ->index(MyMath::min<long>(row+1,stmodel->rowCount(parent)-1),0,cpindex);
	break;
      }
    }    
  }
  if(!cindex.isValid()) cindex=cpindex;




  if(cindex.isValid()){
    tvSets->setCurrentIndex(cindex);
  }

  enableToolButtons();
  
  
  emit acceptRequest(t_graph,false);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SetDlg::enableToolButtons()
{
  if(t_graph->groups()->size()<1){
    tbDelete->setEnabled(false);
  }else{
    tbDelete->setEnabled(true);
  }
  if(t_graph->groups()->size()<2){
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
  }else{
    tbUp->setEnabled(true);
    tbDown->setEnabled(true);
  }


}
/*EOF*/ 
