/* @(#)SetDlg.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SETDLG_H
#define _SETDLG_H 1

#include <QCloseEvent>
#include "Graph.hh"
#include "BrainDialog.hh"
#include "ui_SetDlg.h"

class QAction;
class Graph;
class Page;
class FGObject;
class SetTreeModel;
class SetTreeSelectionModel;
class ToolButtons;


class SetDlg: public BrainDialog, protected Ui::SetDlg
{
  Q_OBJECT
public:
  SetDlg(QWidget *parent,QWidget *base,const char *name);
  ~SetDlg();

  void setData(Page *,FGObject *);
  void reloadDialog();

signals:
  void acceptRequest(GraphObject *,bool forceChanged);
  void endRequest();

public slots:
  virtual void reject(){ emit endRequest();};
protected:
  virtual void closeEvent( QCloseEvent * e ) {e->accept(); emit endRequest();};
  virtual bool event(QEvent *event);

private slots:
  void on_pbApply_clicked(){ if(dataAcceptable()){apply();}};
  void on_pbCancel_clicked(){ reject(); };
  void on_pbClose_clicked(){
    if(dataAcceptable()){apply(); emit endRequest();return;}};
  void onDlgEnd(){deleteLater();};
  virtual void on_pbHelp_clicked(){};
  void on_tvSets_myCurrentChanged(const QModelIndex &,const QModelIndex &);
  void onContentChanged(bool changed);
  void onToolsActionsTriggered();

  QList<FGObject*> getSelectedFGObjects(bool considercurrent);
  void triggerUpdate(){emit acceptRequest(t_graph,false);};
  void slotMoveSetGroupSelection();
  void slotDeleteSetGroupSelection();

private:
  bool protectChanges(bool withCancel=true);
  bool dataAcceptable();
  void apply(bool considercurrent=true);
  void selectFGObject(FGObject *fgo);
  void enableToolButtons();

private:
  SetTreeModel *stmodel;
  SetTreeSelectionModel *stselmodel;
  Graph *t_graph;
  Page *t_page;
  bool mute_currentIndexChanged;
  QList<QAction*> toolactions;
  bool pfirstcall;
  ToolButtons *ptoolbuttons;
  friend class SetPropWdg;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SETDLG_H */

