/*************************************************************************
 * @(#)ShapeDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   23 Nov 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QMenu>
#include <QSignalMapper>
#include "Shape.hh"
#include "Graph.hh"
#include "ShapeDlg.hh"
#include "ShapePropWdg.hh"
#include "ShapesModel.hh"
#include "ToolButtons.hh"
#include "MyMath.hh"
#include "MyDebug.hh"
#include "ReselectEvent.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ShapeDlg::ShapeDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),t_sh(0),t_shs(0),shsmodel(0),
    insertMenu(0),signalMapper(0),
    mute_currentIndexChanged(false)
{
  //myDebug()<<"hier ShapeDlg"<<this;
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);
  
  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);
  
  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));
  
  if(wProps) delete wProps;
  wProps=new ShapePropWdg(this);
  verticalLayout->insertWidget(1,wProps,10);
  
  shsmodel = new ShapesModel(this);
  lvShapes->setModel(shsmodel);
  
  
  ptoolbuttons.createToolButtonsActions(lvShapes,"Shape"
					,tbInsert,tbDelete,tbUp,tbDown);

  signalMapper = new QSignalMapper(this);
  connect(signalMapper,SIGNAL(mapped(int)),
	  this,SLOT(slotInsertShape(int)));

  QMenu *m=insertMenu=new QMenu(tbInsert);
  QAction *a;
  a=m->addAction("&Line");  
  signalMapper->setMapping(a,Xfp::SHTLine);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  a=m->addAction("&Ellipse");  
  signalMapper->setMapping(a,Xfp::SHTEllipse);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  tbInsert->setMenu(insertMenu);

  ptoolbuttons.connectSlots(this,
			    SLOT(slotInsertTriggerd()),
			    SLOT(slotDeleteShape()),
			    SLOT(slotMoveShape()),
			    SLOT(slotMoveShape()));



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ShapeDlg::~ShapeDlg()
{
  //myDebug()<<"hier ~ShapeDlg"<<this;
  if(shsmodel) delete shsmodel;shsmodel=0;

  if(signalMapper) delete signalMapper;
  if(insertMenu) delete insertMenu; insertMenu=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::on_pbApply_clicked()
{
  //myDebug() <<"ShapeDlg::on_pbApply_clicked";
  if(dataAcceptable()){
    apply();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::on_pbClose_clicked()
{
  //myDebug() <<"ShapeDlg::on_pbClose_clicked";
  if(dataAcceptable()){
    apply();
    emit endRequest();
    return;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapeDlg::dataAcceptable()
{
  QString s,txt="";

  wProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    return false;
  }


  return true;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::apply()
{
  wProps->applyData(t_sh);

  shsmodel->resetItem(t_sh);
  emit acceptRequest(t_sh,false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::onDlgEnd()
{
  //myDebug()<<"ShapeDlg::onDlgEnd"<<this;
  deleteLater();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::onContentChanged(bool changed)
{
  pbApply->setEnabled(changed);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::updateRequest(Shape *sh)
{
  if(!sh) return;
  if(t_sh==sh){
    wProps->updateRequest(sh);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::reloadDialog()
{
  // discard all data and reload;
  if(!t_shs) return;
  
  QModelIndex index=lvShapes->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row();
  }

  shsmodel->reset(t_shs);
  
  row=MyMath::max<long>(0,MyMath::min<long>(row,shsmodel->rowCount()-1));

  QModelIndex ci=shsmodel->index(row,0,QModelIndex());
  lvShapes->scrollTo(ci);
  lvShapes->setCurrentIndex(ci);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::setData(Shapes *shs,Shape *sh)
{
  // sh==0 means the call from menu others where there are no others

  if(wProps->isDirty()){
    if(protectChanges(false)) return;
  }

  //myDebug()<<shs<<sh;
  //bool newshs=false;
  if(shs){
    if(t_shs!=shs){
      if(t_shs==0){
	//newshs=true;
	t_shs=shs;
	shsmodel->reset(shs);
      }else{
	//newshs=true;
	t_shs=shs;
	shsmodel->reset(shs);
	
	if(t_sh){
	  // previous selected textbox
	  //	  if(protectChanges(false)) return;
	}
	// no previous selected textbox
	t_sh=0;
	if(sh){
	  t_sh=sh;
	}else{
	  if(shs->size()>0){
	    t_sh=shs->at(0);
	  }
	}
	lvShapes->setCurrentIndex(shsmodel->shapeIndex(t_sh));
	return ;
      }
    }
  }else{
    t_shs=0;
    t_sh=0;
    shsmodel->reset(0);
    wProps->setData(0);
    //myDebug()<<"here";
    setAddDeleteEnabed(QModelIndex());
    tbInsert->setEnabled(false);
    return;
  }
  tbInsert->setEnabled(true);
  shsmodel->resetItem(sh);

  

  if(!t_sh){
    // no previous textbox;
    if(!sh){
      //new textbox does not exists
      Shape *first=0;
      if(shs->size()>0){
	first=shs->at(0);
      }
      t_sh=first;
    }else{
      t_sh=sh;
    }
    lvShapes->setCurrentIndex(shsmodel->shapeIndex(t_sh));
    return;
  }else{
    //previous textbox
    if(sh){
      //new textbox exists
      if((t_sh!=sh)){
	//	if(protectChanges(false)) return;
	t_sh=sh;
	lvShapes->setCurrentIndex(shsmodel->shapeIndex(t_sh));
      }
    }else{
      //new textbox does not exist
      return;
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapeDlg::protectChanges(bool withCancel)
{
  if(wProps->isDirty()){
    QString s="The content of the dialog changed.\n"
      "(Use the dialog's apply button for"
      " confirming changes.)";
    int ret;
    if(withCancel)
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard,QMessageBox::Cancel);
    else{
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard);
    }
    if(ret==QMessageBox::Apply){
      if(dataAcceptable()){
	apply();
      }else{
 	return true;
      }
    }else if(ret==QMessageBox::Cancel){
      return true;
    }else if(ret==QMessageBox::Discard){
      wProps->setDirty(false);
    }
  }
  return false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::deleteShape(Shape *deletedsh)
{
  if(!deletedsh) return;
  if(t_sh==deletedsh){
    slotDeleteShape(false);
  }else{
    long deleteindex=shsmodel->shapeIndex(deletedsh).row();
    shsmodel->removeRow(deleteindex);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::slotInsertTriggerd()
{
  //NSF("Shape::slotInsertTriggerd()");
  //myDebug() <<nsubfunc;

  QPoint pos;
  pos=tbInsert->mapToGlobal(QPoint(0,tbInsert->height()));
  //    pos=QCursor::pos();

  tbInsert->menu()->popup(pos);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::slotInsertShape(int m)
{
  //  NSF("ShapeDlg::slotInsertShape()");
  //myDebug()<< nsubfunc;
  if(protectChanges())
    return;

  Xfp::ShapeType mode=(Xfp::ShapeType)m;

  QModelIndex index=lvShapes->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
  }
  shsmodel->insertRow(row,mode);
  QModelIndex ci=shsmodel->index(row,0,QModelIndex());
  lvShapes->scrollTo(ci);
  lvShapes->setCurrentIndex(ci);
  emit acceptRequest(shsmodel->currentShape(lvShapes->currentIndex())
		     ,true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::slotDeleteShape(bool warn)
{
  if(warn){
    if(QMessageBox::warning(this,IDS_APP_NAME,"Really delete",
			    QMessageBox::Ok,QMessageBox::Cancel)
       !=QMessageBox::Ok) return;
  }

  //NSF("ShapeDlgs::slotDeleteShape"); 
  int numrows=shsmodel->rowCount();
  if(numrows<=0) return;
  QModelIndex currentindex=lvShapes->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a shape first."
			   , 1, 0,0 );
    return;
  }
  wProps->setDirty(false);
  
  int row=currentindex.row();
  
  shsmodel->removeRow(row);
  
  if(numrows!=shsmodel->rowCount())
    lvShapes
      ->setCurrentIndex(shsmodel->
			index(MyMath::max<long>(0,MyMath::min<long>(row,numrows-2)),0,QModelIndex()));
  
  emit acceptRequest(shsmodel->currentShape(lvShapes->currentIndex())
		     ,true);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::slotMoveShape()
{
  //NSF("ShapeDlg::slotMoveShape"); Q_UNUSED(nsubfunc);
  QModelIndex currentindex=lvShapes->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a shape first."
			   , 1, 0,0 );
    return;
  }
  int row=currentindex.row();
  int by=-1;
  if(sender()==ptoolbuttons.pmoveDownAction) by=1;
  if(shsmodel->moveRow(row,by)){
    lvShapes->setCurrentIndex(shsmodel->index(row+by,0,QModelIndex()));
  }

  emit acceptRequest(shsmodel->currentShape(lvShapes->currentIndex())
		     ,true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapeDlg::event(QEvent *event)
{
  //  NSF("ShapeDlg::event");
  if(event->type()==ReselectEvent::ReselectEventType){
    if(ReselectEvent *re=dynamic_cast<ReselectEvent*>(event)){
      mute_currentIndexChanged=true;
      lvShapes->setCurrentIndex(re->index());
      mute_currentIndexChanged=false;
    }
    return true;
  }
  return BrainDialog::event(event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::setAddDeleteEnabed(const QModelIndex &current)
{
  if(!current.isValid()){
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
    return;
  }
  Shape *sh=shsmodel->currentShape(current);
  if(!sh){
    tbInsert->setEnabled(false);
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
  }else{
    tbInsert->setEnabled(true);
    tbDelete->setEnabled(true);
    if(current.row()==0)
      tbUp->setEnabled(false);
    else
      tbUp->setEnabled(true);
    if(current.row()<shsmodel->rowCount()-1)
      tbDown->setEnabled(true);
    else
      tbDown->setEnabled(false);
    
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapeDlg::on_lvShapes_currentItemChanged(const QModelIndex &current
					      ,const QModelIndex &previous)
{
  if(mute_currentIndexChanged)return;

  if(protectChanges()){
    QEvent *event=new ReselectEvent(ReselectEvent::ReselectEventType,previous);
    QCoreApplication::postEvent(this,event);
    return;
  }
    

  if(t_shs)
    lGraph->setText(t_shs->graph()->guiName(true)());
  
  setAddDeleteEnabed(current);
  if(!current.isValid()){
    t_sh=0;
    wProps->setData(0);
  }else{
    Shape *sh=shsmodel->currentShape(current);
    if(sh){
      t_sh=sh;
      wProps->setData(sh);
    }
  }
}


/*EOF*/ 
