/*************************************************************************
 * @(#)ShapePropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   23 Nov 2011
 *************************************************************************
 */

#include "ShapePropWdg.hh"

#include "Attributes.hh"
#include "Shape.hh"
#include "Graph.hh"
#include "LineShape.hh"
#include "EllipseShape.hh"
#include "ShapePropWdg.hh"
#include "Valid.hh"
#include "defs.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ShapePropWdg::ShapePropWdg(QWidget *parent)
  :PropWdgBase(parent),mute_coordsystem(false)
{
  setupUi(this);

  QString s="Line Start ";
  new RealValidator(this,lXStart->buddy(),s+lXStart->text(),1.,-1.);
  new RealValidator(this,lYStart->buddy(),s+lYStart->text(),1.,-1.);
  s="Line End ";
  new RealValidator(this,lXEnd->buddy(),s+lXEnd->text(),1.,-1.);
  new RealValidator(this,lYEnd->buddy(),s+lYEnd->text(),1.,-1.);

  s="Ellipse center ";
  new RealValidator(this,lXCenter->buddy(),s+lXCenter->text(),1.,-1.);
  new RealValidator(this,lYCenter->buddy(),s+lYCenter->text(),1.,-1.);
  s="Ellipse angle ";
  new RealValidator(this,lAngle->buddy(),s+lAngle->text(),1.,-1.);
  s="Ellipse radius ";
  new RealValidator(this,lWRadius->buddy(),s+lWRadius->text(),1.,-1.);
  new RealValidator(this,lHRadius->buddy(),s+lHRadius->text(),1.,-1.);

  
  connectChanged(this);

  tabProps->setEnabled(true);
  tabPosition->setEnabled(true);
  lineTabWidget->setEnabled(true);
  ellipseTabWidget->setEnabled(true);
  

  cmbLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  cmbEllLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbEllLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  mute_coordsystem=true;
  cmbCoordSystem->clear();
  for(int i=0;i<Xfp::maxCoordinateSystems();i++){
    cmbCoordSystem
      ->insertItem(i,Xfp::coordinateSystemGuiName((Xfp::CoordinateSystem)i),i);
  }
  mute_coordsystem=false;

  mute_coordsystem=true;
  cmbEllCoordSystem->clear();
  for(int i=0;i<Xfp::maxCoordinateSystems();i++){
    cmbEllCoordSystem
      ->insertItem(i,Xfp::coordinateSystemGuiName((Xfp::CoordinateSystem)i),i);
  }
  mute_coordsystem=false;

  cmbHeadPosition->clear();
  for(int i=0;i<Xfp::maxArrowPositions();i++){
    cmbHeadPosition
      ->insertItem(i,Xfp::arrowPositionGuiName((Xfp::ArrowPosition)i),i);
  }

  cmbCapPosition->clear();
  for(int i=0;i<Xfp::maxArrowPositions();i++){
    cmbCapPosition
      ->insertItem(i,Xfp::arrowPositionGuiName((Xfp::ArrowPosition)i),i);
  }

  
  cmbHeadStyle->clear();
  for(int i=0;i<Xfp::maxArrowStyles();i++){
    cmbHeadStyle
      ->insertItem(i,Xfp::arrowStyle(i).guiname,Xfp::arrowStyle(i).type);
  }

  setDirty(false);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ShapePropWdg::~ShapePropWdg()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::check(QString &msg)
{
  if(!p_sh) return;
  if(p_sh->type()==Xfp::SHTLine){
    ((IntValidator*)leXStart->validator())->check(msg);
    ((IntValidator*)leYStart->validator())->check(msg);
    ((IntValidator*)leXEnd->validator())->check(msg);
    ((IntValidator*)leYEnd->validator())->check(msg);
  }else if(p_sh->type()==Xfp::SHTEllipse){
    ((IntValidator*)leAngle->validator())->check(msg);
    ((IntValidator*)leXCenter->validator())->check(msg);
    ((IntValidator*)leYCenter->validator())->check(msg);
    ((IntValidator*)leWRadius->validator())->check(msg);
    ((IntValidator*)leHRadius->validator())->check(msg);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::setData(void *go)
{
  //myDebug()<<"setData";
  mutecontentchanged=true;
  Shape *sh=0;
  if(!go) PropWdgBase::reset(this);
  if(go) sh=dynamic_cast<Shape*>((GraphObject *)go);
  p_sh=sh;

  QWidget *fw=this->focusWidget();

  QWidget *cw=stckwdgProps->currentWidget();
  
  if(!sh){
    pageLine->setEnabled(false);
    pageEllipse->setEnabled(false);
    lName->setEnabled(false);
    leName->setEnabled(false);
    cbActive->setEnabled(false);
    return;
  }else{
    pageLine->setEnabled(sh->active());
    pageEllipse->setEnabled(sh->active());
    lName->setEnabled(sh->active());
    leName->setEnabled(sh->active());
    cbActive->setEnabled(true);
  }
  


  cbActive->setChecked(sh->active());
  leName->setText(sh->name()());

  if(sh->type()==Xfp::SHTLine){
    stckwdgProps->setCurrentWidget(pageLine);
    pageEllipse->setEnabled(false);
    LineShape *lsh=(LineShape*)sh;
    cmbLineStyle->setCurrentIndex(lsh->lineStyle()->style());
    spbxLineWidth->setValue(lsh->lineStyle()->width());
    tbLineColor->setColor(lsh->lineStyle()->color());

    cbArrowHeadFillColorExtra->setChecked(lsh->extraFillColor());
    cbArrowHeadFillColor->setColor(lsh->fill()->color());

    dspbxSize->setValue(lsh->arrowHeadSize());
    dspbxSharpness->setValue(lsh->sharpness());

    cmbHeadPosition->setCurrentIndex(lsh->arrowPosition());
    cmbCapPosition->setCurrentIndex(lsh->capPosition());

    cmbHeadStyle->setCurrentIndex(cmbHeadStyle->findData(lsh->arrowStyle()));

    mute_coordsystem=true;
    cmbCoordSystem->setCurrentIndex(lsh->coordinateSystem());
    oldcoordsys=lsh->coordinateSystem();
    mute_coordsystem=false;
    
    setPosition(lsh->posStart(),lsh->posEnd());
  }else if(sh->type()==Xfp::SHTEllipse){
    stckwdgProps->setCurrentWidget(pageEllipse);
    pageLine->setEnabled(false);

    EllipseShape *esh=(EllipseShape*)sh;

    cmbEllLineStyle->setCurrentIndex(esh->lineStyle()->style());
    spbxEllLineWidth->setValue(esh->lineStyle()->width());
    tbEllLineColor->setColor(esh->lineStyle()->color());

    leAngle->setText(QString().setNum(esh->angle(),'g',10));

    cbEllFillColorExtra->setChecked(esh->extraFillColor());
    cbEllFillColor->setColor(esh->fill()->color());
    gbEllFill->setChecked(esh->fill()->fill());

    mute_coordsystem=true;
    cmbEllCoordSystem->setCurrentIndex(esh->coordinateSystem());
    oldellcoordsys=esh->coordinateSystem();
    mute_coordsystem=false;

    cbIsEllipse->setChecked(!esh->isCircle());

    setCenter(esh->center());
    setRadii(esh->radii());
  }

  if(stckwdgProps->currentWidget()==cw && fw){
    fw->setFocus();
//   }else{
//     leName->setFocus();
  }

    
  sh->setChanged(false);

  mutecontentchanged=false;
  setDirty(false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::applyData(void *go)
{
  //NSF("applydata");
  if(!go) return;
  Shape *sh=0;
  if(go) sh=dynamic_cast<Shape*>((GraphObject *)go);

  
  sh->setActive(cbActive->isChecked());
  sh->setName(String(leName->text()));


  if(sh->type()==Xfp::SHTLine){
    LineShape *lsh=(LineShape*)sh;

    lsh->lineStyle()->setStyle((Xfp::LineStyles)cmbLineStyle->currentIndex());
    lsh->lineStyle()->setWidth(spbxLineWidth->value());
    lsh->lineStyle()->setColor(tbLineColor->color());
    lsh->setExtraFillColor(cbArrowHeadFillColorExtra->isChecked());
    lsh->fill()->setColor(cbArrowHeadFillColor->color());
    lsh->setArrowHeadSize(dspbxSize->value());
    lsh->setSharpness(dspbxSharpness->value());
    lsh
      ->setCapPosition((Xfp::ArrowPosition)cmbCapPosition->currentIndex());

    lsh
      ->setArrowPosition((Xfp::ArrowPosition)cmbHeadPosition->currentIndex());

    lsh->setArrowStyle((Xfp::ArrowStyle)cmbHeadStyle
		       ->itemData(cmbHeadStyle->currentIndex()).toInt());

    lsh
      ->setCoordinateSystem((Xfp::CoordinateSystem)cmbCoordSystem
			    ->currentIndex());


    Point s,e;
    getPosition(s,e);
    lsh->setPosStart(s);
    lsh->setPosEnd(e);

    
  }else if(sh->type()==Xfp::SHTEllipse){
    EllipseShape *esh=(EllipseShape*)sh;

    esh->lineStyle()->setStyle((Xfp::LineStyles)cmbEllLineStyle->currentIndex());
    esh->lineStyle()->setWidth(spbxEllLineWidth->value());
    esh->lineStyle()->setColor(tbEllLineColor->color());

    esh->setExtraFillColor(cbEllFillColorExtra->isChecked());
    esh->fill()->setColor(cbEllFillColor->color());
    esh->fill()->setFill(gbEllFill->isChecked());

    esh
      ->setCoordinateSystem((Xfp::CoordinateSystem)cmbEllCoordSystem
			    ->currentIndex());

    bool ok;
    QString t=leAngle->text();IntValidator::stripAllWhiteSpace(t);
    esh->setAngle(t.toDouble(&ok));

    esh->setIsCircle(!cbIsEllipse->isChecked());

    Point c;
    getCenter(c);
    esh->setCenter(c);
    Size r;
    getRadii(r);
    esh->setRadii(r);
  }

  setDirty(false);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::updateRequest(void *go)
{
  Shape *sh=0;
  if(go) sh=dynamic_cast<Shape*>((GraphObject *)go);
  
  if(!sh) return;
   bool di=isDirty();
   if(sh->type()==Xfp::SHTLine){
     LineShape *lsh=(LineShape*)sh;
     if(sh->updateRequest() & GraphObject::GOCPosition ||
	sh->updateRequest() & GraphObject::GOCSize){
       setPosition(lsh->posStart(),lsh->posEnd());
     }
   }else if(sh->type()==Xfp::SHTEllipse){
     EllipseShape *esh=(EllipseShape*)sh;
     if(sh->updateRequest() & GraphObject::GOCPosition){
       setCenter(esh->center());
     }else if(sh->updateRequest() & GraphObject::GOCSize){
       setRadii(esh->radii());
     }
   }

  if(!di) setDirty(false);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::on_cmbHeadPosition_currentIndexChanged(int index)
{
  //myDebug()<<index;
  bool on=true;
  if(index==Xfp::ARPNone){
    on=false;
  }
  lHeadStyle->setEnabled(on);
  cmbHeadStyle->setEnabled(on);
  lSize->setEnabled(on);
  dspbxSize->setEnabled(on);
  lSharpness->setEnabled(on);
  dspbxSharpness->setEnabled(on);
  gbHeadFill->setEnabled(on);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::on_cmbCoordSystem_currentIndexChanged(int index)
{
  if(mute_coordsystem)return;

  Point s,e;
  getPosition(s,e);


  Xfp::CoordinateSystem mode=(Xfp::CoordinateSystem)index;

  if(p_sh->type()!=Xfp::SHTLine) return;
  LineShape *lsh=(LineShape*)p_sh;
  if(oldcoordsys != mode){
    s=lsh->dcToPos(lsh->posToDC(s,oldcoordsys),mode);
    e=lsh->dcToPos(lsh->posToDC(e,oldcoordsys),mode);
    setPosition(s,e);
  }
  oldcoordsys=mode;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::on_cmbEllCoordSystem_currentIndexChanged(int index)
{
  if(mute_coordsystem)return;

  Point c;
  Size r;
  getCenter(c);
  getRadii(r);


  Xfp::CoordinateSystem mode=(Xfp::CoordinateSystem)index;

  if(p_sh->type()!=Xfp::SHTEllipse) return;
  EllipseShape *esh=(EllipseShape*)p_sh;
  if(oldellcoordsys != mode){
    c=esh->dcToPos(esh->posToDC(c,oldellcoordsys),mode);
    r=abs(esh->dcToSize(esh->sizeToDC(r,oldellcoordsys),mode));
    setCenter(c);
    setRadii(r);
  }
  oldellcoordsys=mode;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::getPosition(Point &s,Point &e)
{
  bool ok;
  QString t=leXStart->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYStart->text();IntValidator::stripAllWhiteSpace(t2);

  s=Point(t.toDouble(&ok),t2.toDouble(&ok));

  t=leXEnd->text();IntValidator::stripAllWhiteSpace(t);
  t2=leYEnd->text();IntValidator::stripAllWhiteSpace(t2);

  e=Point(t.toDouble(&ok),t2.toDouble(&ok));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::setPosition(const Point &s,const Point &e)
{
  leXStart->setText(QString().setNum(s.x));
  leYStart->setText(QString().setNum(s.y));
  
  leXEnd->setText(QString().setNum(e.x));
  leYEnd->setText(QString().setNum(e.y));
  leXStart->setCursorPosition(0);
  leYStart->setCursorPosition(0);
  leXEnd->setCursorPosition(0);
  leYEnd->setCursorPosition(0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::setCenter(const Point &c)
{
  leXCenter->setText(QString().setNum(c.x));
  leYCenter->setText(QString().setNum(c.y));
  
  leXCenter->setCursorPosition(0);
  leYCenter->setCursorPosition(0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::getCenter(Point &c)
{
  bool ok;
  QString t=leXCenter->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYCenter->text();IntValidator::stripAllWhiteSpace(t2);

  c=Point(t.toDouble(&ok),t2.toDouble(&ok));
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::setRadii(const Size &r)
{
  leWRadius->setText(QString().setNum(r.w));
  leHRadius->setText(QString().setNum(r.h));
  
  leWRadius->setCursorPosition(0);
  leHRadius->setCursorPosition(0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapePropWdg::getRadii(Size &r)
{
  bool ok;
  QString t=leWRadius->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leHRadius->text();IntValidator::stripAllWhiteSpace(t2);

  r=abs(Size(t.toDouble(&ok),t2.toDouble(&ok)));
  
}

/*EOF*/ 
