/*************************************************************************
 * @(#)ShapesModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   28 Apr 2011
 *************************************************************************
 */

#include "Shape.hh"
#include "ShapesModel.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ShapesModel::ShapesModel(QObject * parent)
  : QAbstractListModel(parent),pshapes(0)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapesModel::reset(Shapes *shs)
{
  pshapes=shs;
  QAbstractListModel::reset();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant ShapesModel::headerData(int section, Qt::Orientation orientation
				    , int role) const
{
  Q_UNUSED(section);
  if (role != Qt::DisplayRole)
    return QVariant();
  if (orientation == Qt::Horizontal) {
    return "Header";
  }
  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int ShapesModel::rowCount(const QModelIndex & parent) const
{
  Q_UNUSED(parent);
  if(!pshapes) return 0;
  return pshapes->size();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant ShapesModel::data(const QModelIndex &index,int role)const
{
  if(!pshapes) return QVariant();
  if (!index.isValid()) return QVariant();
  int row=index.row();
  //int col=0;
  if (row >= pshapes->size() || row < 0) return QVariant();

  const Shape *sh=pshapes->at(row);
  
  if (role == Qt::DisplayRole) {
    
    return sh->guiName()();
  }

  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapesModel::insertRow(int &row,Shape *sh)
{
  if(!pshapes) return false;
  if(row<0){
    row=pshapes->size();
  }
  beginInsertRows(QModelIndex(),row,row);
  pshapes->insertShape(row,sh);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapesModel::insertRow(int &row,Xfp::ShapeType mode)
{
  if(!pshapes) return false;
  if(row<0){
    row=pshapes->size();
  }
  beginInsertRows(QModelIndex(),row,row);
  pshapes->insertShape(row,mode);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapesModel::removeRow(int row)
{
  if(!pshapes) return;
  if(row<0 || row>=pshapes->size()) return;
  
  //Shape *sh=(*pshapes)[row];
  
  beginRemoveRows(QModelIndex(), row, row);
  pshapes->deleteShape(row);
  endRemoveRows();
  return;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ShapesModel::moveRow(int row,int by)
{
  if(!pshapes) return false;
  if(!index(row+by,0,QModelIndex()).isValid()) return false;

  Shape *pfrom=pshapes->at(row);
  Shape *pto=pshapes->at(row+by);
  if(!pfrom) return false;
  if(!pto) return false;
  //   if(pfrom->type()!=Shape::TBTGeneral) return false;
  //   if(pto->type()!=Shape::TBTGeneral) return false;

  
  beginRemoveRows(QModelIndex(), row, row);
  Shape *p=pshapes->takeAt(row);
  endRemoveRows();
  
  int r=MyMath::max<long>(0,row+by);
  insertRow(r,p);

  return true;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Shape *ShapesModel::currentShape(const QModelIndex &index)const
{
  if(!pshapes) return 0;
  if(!index.isValid())return 0;
  return pshapes->at(index.row());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QModelIndex ShapesModel::shapeIndex(Shape *sh)const
{
  if(!sh) return this->index(0,0,QModelIndex());
  return this->index(pshapes->indexOf(sh),0,QModelIndex());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ShapesModel::resetItem(Shape *sh)
{
  if(!sh) return;
  QModelIndex index=this->index(pshapes->indexOf(sh),0,QModelIndex());
  emit dataChanged(index,index);
}



/*EOF*/ 
