/*************************************************************************
 * @(#)TextBoxDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Apr 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "ToolButtons.hh"
#include "TextBox.hh"
#include "HelpDlg.hh"
#include "TextBoxesModel.hh"
#include "TextBoxDlg.hh"
#include "TextBoxPropWdg.hh"
#include "ReselectEvent.hh"
#include "Config.hh"
#include "MyMath.hh"
#include "defs.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxDlg::TextBoxDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),t_tb(0),t_tbs(0),tbsmodel(0)
  ,mute_currentIndexChanged(false),ptoolbuttons(new ToolButtons())
{
  //myDebug()<<"hier TextBoxDlg"<<this;
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);

  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);

  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));

  if(wProps) delete wProps;
  wProps=new TextBoxPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);

  tbsmodel = new TextBoxesModel(this);
  lvTextBoxes->setModel(tbsmodel);


  ptoolbuttons->createToolButtonsActions(lvTextBoxes,"TextBox"
		     ,tbInsert,tbDelete,tbUp,tbDown);

  ptoolbuttons->connectSlots(this,SLOT(slotInsertTextBox()),
			    SLOT(slotDeleteTextBox()),
			    SLOT(slotMoveTextBox()),
			    SLOT(slotMoveTextBox()));



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxDlg::~TextBoxDlg()
{
  //myDebug()<<"hier ~TextBoxDlg"<<this;
  if(tbsmodel) delete tbsmodel;tbsmodel=0;
  if(ptoolbuttons) delete ptoolbuttons;ptoolbuttons=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","Text-Formating");
  h->show();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::on_pbApply_clicked()
{
  //myDebug() <<"TextBoxDlg::on_pbApply_clicked";
  if(dataAcceptable()){
    apply();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::on_pbClose_clicked()
{
  //myDebug() <<"TextBoxDlg::on_pbClose_clicked";
  if(dataAcceptable()){
    apply();
    emit endRequest();
    return;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBoxDlg::dataAcceptable()
{
  QString s,txt="";

  wProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  //myDebug() << "END BondsDlg::dataAcceptable";

  return true;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::apply()
{
  wProps->applyData(t_tb);

  tbsmodel->resetItem(t_tb);
  emit acceptRequest(t_tb,false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::onDlgEnd()
{
  //myDebug()<<"TextBoxDlg::onDlgEnd"<<this;
  deleteLater();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::setData(TextBoxes *tbs,TextBox *tb)
{
  // tb==0 means the call from menu others where there are no others
  //myDebug()<<tbs<<tb;
  //bool newtbs=false;

  if( (t_tb && tb && (t_tb!=tb)) ){
    if(protectChanges(false)) return;
  }



  if(tbs){
    if(t_tbs!=tbs){
      if(t_tbs==0){
	//newtbs=true;
	t_tbs=tbs;
	tbsmodel->reset(tbs);
      }else{
	//newtbs=true;
	t_tbs=tbs;
	tbsmodel->reset(tbs);

	if(tb!=0){
	  t_tb=tb;
	  lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(t_tb));
	  return;
	  
	  
	}else{
	  if(t_tb){
	    // previous selected textbox
	    TextBox::TextBoxType type=t_tb->type();
	    t_tb=tbs->textBoxByType(type);
	    lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(t_tb));
	    return;
	  }else{
	    // no previous selected textbox
	    TextBox *title=tbs->textBoxByType(TextBox::TBTTitle);
	    t_tb=title;
	    lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(title));
	    return ;
	  }
	}
      }
    }
  }else{
    t_tbs=tbs;
    t_tb=tb;
    tbsmodel->reset(0);
    wProps->setData(0);
    //myDebug()<<"here";
    setAddDeleteEnabed(QModelIndex());
    return;
  }

  tbsmodel->resetItem(tb);

  

  if(!t_tb){
    // no previous textbox;
    if(!tb){
      //new textbox does not exists
      TextBox *title=tbs->textBoxByType(TextBox::TBTTitle);
      lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(title));
    }else{
      t_tb=tb;
      lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(t_tb));
    }
    return;
  }else{
    //previous textbox
    if(tb){
      //new textbox exists
      if((t_tb!=tb)){
	t_tb=tb;
	lvTextBoxes->setCurrentIndex(tbsmodel->textBoxIndex(t_tb));
      }
    }else{
      //new textbox does not exist
      return;
    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::reloadDialog()
{
  // discard all data and reload;
  if(!t_tbs) return;
  
  QModelIndex index=lvTextBoxes->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row();
  }

  tbsmodel->reset(t_tbs);

  row=MyMath::max<long>(0,MyMath::min<long>(row,tbsmodel->rowCount()-1));

  QModelIndex ci=tbsmodel->index(row,0,QModelIndex());
  lvTextBoxes->scrollTo(ci);
  lvTextBoxes->setCurrentIndex(ci);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::updateRequest(TextBox *tb)
{
  if(!tb) return;
  if(t_tb==tb){
    wProps->updateRequest(tb);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::deleteTextBox(TextBox *deletedtb)
{
  if(!t_tbs) return;
  if(!deletedtb) return;
  if(t_tb==deletedtb){
    slotDeleteTextBox(false);      
  }else{
    long deleteindex=tbsmodel->textBoxIndex(deletedtb).row();
    tbsmodel->removeRow(deleteindex);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::copyTextBox(TextBox *copyfrom)
{
  if(!t_tbs) return;
  if(!copyfrom) return;

  int row=0;
  tbsmodel->insertRow(row,copyfrom->clone());

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::slotInsertTextBox()
{
  NSF("TextBoxDlg::slotInsertTextBox()");
  //myDebug()<< nsubfunc;
  if(!t_tbs) return;
  if(protectChanges())
    return;

  QModelIndex index=lvTextBoxes->currentIndex();
  int row=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
  }
  TextBox *tbb=0;
  if(row-1>=TextBox::TBTGeneral){
    tbb=tbsmodel->currentTextBox(index)->clone();
  }


  tbsmodel->insertRow(row,tbb);
  QModelIndex ci=tbsmodel->index(row,0,QModelIndex());
  lvTextBoxes->scrollTo(ci);
  lvTextBoxes->setCurrentIndex(ci);
  emit acceptRequest(tbsmodel->currentTextBox(lvTextBoxes->currentIndex())
		     ,true);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::slotDeleteTextBox(bool warn)
{
  if(warn){
    if(QMessageBox::warning(this,IDS_APP_NAME,"Really delete",
			    QMessageBox::Ok,QMessageBox::Cancel)
       !=QMessageBox::Ok) return;
  }

  NSF("TextBoxDlgs::slotDeleteTextBox"); 
  int numrows=tbsmodel->rowCount();
  if(numrows<=0) return;
  QModelIndex currentindex=lvTextBoxes->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a textbox first."
			   , 1, 0,0 );
    return;
  }
  wProps->setDirty(false);
  
  int row=currentindex.row();
  
  tbsmodel->removeRow(row);
  
  if(numrows!=tbsmodel->rowCount())
    lvTextBoxes->setCurrentIndex(tbsmodel->
				 index(MyMath::min<long>(row,numrows-2),0,QModelIndex()));
  
  emit acceptRequest(tbsmodel->currentTextBox(lvTextBoxes->currentIndex())
		     ,true);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::slotMoveTextBox()
{
  NSF("TextBoxDlg::slotMoveTextBox"); Q_UNUSED(nsubfunc);
  QModelIndex currentindex=lvTextBoxes->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a textbox first."
			   , 1, 0,0 );
    return;
  }
  int row=currentindex.row();
  int by=-1;
  if(sender()==ptoolbuttons->pmoveDownAction) by=1;
  if(tbsmodel->moveRow(row,by)){
    lvTextBoxes->setCurrentIndex(tbsmodel->index(row+by,0,QModelIndex()));
  }
  emit acceptRequest(tbsmodel->currentTextBox(lvTextBoxes->currentIndex())
		     ,true);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::setAddDeleteEnabed(const QModelIndex &current)
{
  if(!current.isValid()){
    tbInsert->setEnabled(false);
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
    return;
  }
  TextBox *tb=tbsmodel->currentTextBox(current);
  if(!tb){
    tbInsert->setEnabled(false);
    tbDelete->setEnabled(false);
    tbUp->setEnabled(false);
    tbDown->setEnabled(false);
  }else{
    if(tb->type()!=TextBox::TBTGeneral){
      tbInsert->setEnabled(true);
      tbDelete->setEnabled(false);
      tbUp->setEnabled(false);
      tbDown->setEnabled(false);
    }else{
      tbInsert->setEnabled(true);
      tbDelete->setEnabled(true);
      if(current.row()==TextBox::TBTGeneral)
	tbUp->setEnabled(false);
      else
	tbUp->setEnabled(true);
      if(current.row()<tbsmodel->rowCount()-1)
	tbDown->setEnabled(true);
      else
	tbDown->setEnabled(false);
    }
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::on_lvTextBoxes_currentItemChanged(const QModelIndex &current
						   ,const QModelIndex &previous)
{
  if(mute_currentIndexChanged)return;

  if(protectChanges()){
    QEvent *event=new ReselectEvent(ReselectEvent::ReselectEventType,previous);
    QCoreApplication::postEvent(this,event);
    return;
  }
    
  setAddDeleteEnabed(current);
  if(!current.isValid()){
    wProps->setData(0);
  }else{
    TextBox *tb=tbsmodel->currentTextBox(current);
    if(tb){
      t_tb=tb;
      wProps->setData(tb);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxDlg::onContentChanged(bool changed)
{
  pbApply->setEnabled(changed);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBoxDlg::event(QEvent *event)
{
  //  NSF("TextBoxDlg::event");
  if(event->type()==ReselectEvent::ReselectEventType){
    if(ReselectEvent *re=dynamic_cast<ReselectEvent*>(event)){
      mute_currentIndexChanged=true;
      lvTextBoxes->setCurrentIndex(re->index());
      mute_currentIndexChanged=false;
    }
    return true;
  }
  return BrainDialog::event(event);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBoxDlg::protectChanges(bool withCancel)
{
  if(wProps->isDirty()){
    QString s="The content of the dialog changed.\n"
      "(Use the dialog's apply button for"
      " confirming changes.)";
    int ret;
    if(withCancel)
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard,QMessageBox::Cancel);
    else{
      ret=QMessageBox::warning(this,"Content changed",s,QMessageBox::Apply
			       ,QMessageBox::Discard);
    }
    if(ret==QMessageBox::Apply){
      if(dataAcceptable()){
	apply();
      }else{
 	return true;
      }
    }else if(ret==QMessageBox::Cancel){
      return true;
    }else if(ret==QMessageBox::Discard){
      wProps->setDirty(false);
    }
  }
  return false;

}

/*EOF*/ 
