/*************************************************************************
 * @(#)TextBoxPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   28 Apr 2011
 *************************************************************************
 */

#include <QColor>
#include "Attributes.hh"
#include "Position.hh"
#include "TextBox.hh"
#include "TextBoxPropWdg.hh"
#include "Valid.hh"
#include "defs.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxPropWdg::TextBoxPropWdg(QWidget *parent)
  :PropWdgBase(parent),p_tb(0),mute_coordsystem(false)
{
  setupUi(this);

  QString s="Text properties ";
  new IntValidator(this,lFsz->buddy(),s+lFsz->text(),1,1000);
  new RealValidator(this,lSubsc->buddy(),s+lSubsc->text(),0.,100.);
  s="border ";
  new RealValidator(this,lBorder->buddy(),s+lBorder->text(),0.,100.);
  s="Position ";
  new RealValidator(this,lXPos->buddy(),s+lXPos->text(),1.,-1.);
  new RealValidator(this,lYPos->buddy(),s+lYPos->text(),1.,-1.);
  s="Box origin ";
  new RealValidator(this,lXOrigin->buddy(),s+lXOrigin->text(),0.,1.);
  new RealValidator(this,lYOrigin->buddy(),s+lYOrigin->text(),0.,1.);

  new RealValidator(this,lOffset->buddy(),lOffset->text(),0.,1000.);

  new RealValidator(this,lAngle->buddy(),lAngle->text(),-360.,360.);

  for(int i=1;i<Position::numOfOffsetDescriptors();i++){
    lvOffsetSelection->addItem(Position::offsetGuiDescription(i));
    QListWidgetItem *item=lvOffsetSelection->item(i-1);
    item->setCheckState(Qt::Unchecked);
  }

  mute_coordsystem=true;
  cmbPosCoordSystem->clear();
  for(int i=0;i<Xfp::maxCoordinateSystems();i++){
    cmbPosCoordSystem
      ->insertItem(i,Xfp::coordinateSystemGuiName((Xfp::CoordinateSystem)i),i);
  }
  mute_coordsystem=false;
  
  connectChanged(this);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxPropWdg::~TextBoxPropWdg()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::check(QString &msg)
{
  ((IntValidator*)leFsz->validator())->check(msg);
  ((IntValidator*)leSubsc->validator())->check(msg);
  ((IntValidator*)leBorder->validator())->check(msg);
  ((IntValidator*)leXPos->validator())->check(msg);
  ((IntValidator*)leYPos->validator())->check(msg);
  ((IntValidator*)leXOrigin->validator())->check(msg);
  ((IntValidator*)leYOrigin->validator())->check(msg);
  ((IntValidator*)leOffset->validator())->check(msg);
  ((IntValidator*)leAngle->validator())->check(msg);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::setData(void *go)
{
  mutecontentchanged=true;
  TextBox *tb=0;
  if(!go) PropWdgBase::reset(this);
  if(go) tb=dynamic_cast<TextBox*>((GraphObject *)go);
  if(!tb) return;
  p_tb=tb;
  
  QWidget *fw=this->focusWidget();
  if(fw)
    fw->setFocus();
  else
    leText->setFocus();
  


  //myDebug()<<"HIER TextBoxPropWdg::setData"<<tb;

  cbActive->setChecked(tb->active());

  leText->setText(tb->text()());
  leText->setCursorPosition(0);

  leFsz->setText(QString().setNum(tb->textStyle()->size()));
  leFsz->setCursorPosition(0);

  leSubsc->setText(QString().setNum(tb->textStyle()->subScriptScale(),'g',10));
  leSubsc->setCursorPosition(0);
  tbColor->setColor(tb->textStyle()->color());

  cbDrawFrame->setChecked(tb->boxIsActive());
  cmbLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }
  cmbLineStyle->setCurrentIndex(tb->rim()->style());  
  spbxLineWidth->setValue(tb->rim()->width());
  tbRimColor->setColor(tb->rim()->color());
  leBorder->setText(QString().setNum(tb->border(),'g',10));  
  leBorder->setCursorPosition(0);
  
  grbFillFrame->setChecked(tb->fill()->fill());
  tbFillColor->setColor(tb->fill()->color());

  mute_coordsystem=true;
  cmbPosCoordSystem->setCurrentIndex(tb->pos()->coordinateSystem());
  oldcoordsys=tb->pos()->coordinateSystem();
  mute_coordsystem=false;

  bool res=tb->pos()->restriction();
  frRestriction->setEnabled(res);
  //myDebug() << "res"<<res;
  if(res){
    rbRestricted->setChecked(true);
    rbFree->setChecked(false);
  }else{
    rbRestricted->setChecked(false);
    rbFree->setChecked(true);
  }
  setPosition(tb->pos()->pos());

  leOffset->setText(QString().setNum(tb->pos()->offset(),'g',10));  
  leOffset->setCursorPosition(0);


  leXOrigin->setText(QString().setNum(tb->pos()->origin().x,'g',10));  
  leYOrigin->setText(QString().setNum(tb->pos()->origin().y,'g',10));  
  leXOrigin->setCursorPosition(0);
  leYOrigin->setCursorPosition(0);


  leAngle->setText(QString().setNum(tb->pos()->angle(),'g',10));  
  leAngle->setCursorPosition(0);

  long ot=tb->pos()->offsettype();
  

  if(ot==Position::OSTPosX)
    rbPosXOffset->setChecked(true);
  else if(ot==Position::OSTNegX) 
    rbNegXOffset->setChecked(true);
  else if(ot==Position::OSTPosY) 
    rbPosYOffset->setChecked(true);
  else if(ot==Position::OSTNegY) 
    rbNegYOffset->setChecked(true);
  else
    rbPosXOffset->setChecked(true);

  
  for(long i=0;i<lvOffsetSelection->count();i++){
    lvOffsetSelection->item(i)->setCheckState(Qt::Unchecked);
  }
  for(long i=0;i<tb->pos()->numAddTypes();i++){
    long index=Position::indexOfOffsetDescriptionBtype(tb->pos()->addType(i));
    lvOffsetSelection->item(index-1)->setCheckState(Qt::Checked);
  }

  tb->setChanged(false);

  mutecontentchanged=false;
  setDirty(false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::applyData(void *go)
{
  //NSF("applydata");
  bool ok;
  TextBox *tb=0;
  if(go) tb=dynamic_cast<TextBox*>((GraphObject *)go);

  if(!tb) return;
  
  tb->setActive(cbActive->isChecked());
  tb->setText(String(leText->text()));

  QString t=leFsz->text();IntValidator::stripAllWhiteSpace(t);
  tb->textStyle()->setSize(t.toLong(&ok));

  t=leSubsc->text();IntValidator::stripAllWhiteSpace(t);
  tb->textStyle()->setSubScriptScale(t.toDouble());
  tb->textStyle()->setColor(tbColor->color());

  tb->setBoxIsActive(cbDrawFrame->isChecked());

  tb->rim()->setStyle((Xfp::LineStyles)cmbLineStyle->currentIndex());
  tb->rim()->setWidth(spbxLineWidth->value());
  tb->rim()->setColor(tbRimColor->color());
  t=leBorder->text();IntValidator::stripAllWhiteSpace(t);
  tb->setBorder(t.toDouble());
  tb->fill()->setFill(grbFillFrame->isChecked());
  tb->fill()->setColor(tbFillColor->color());

  tb->pos()->setCoordinateSystem((Xfp::CoordinateSystem)cmbPosCoordSystem
				 ->currentIndex());


  
  Point s;
  double o;
  getPosition(s,o);
  tb->pos()->setPos(s);
  tb->pos()->setOffset(o);

  t=leXOrigin->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYOrigin->text();IntValidator::stripAllWhiteSpace(t2);
  tb->pos()->setOrigin(Point(t.toDouble(),t2.toDouble()));
  

  
  t=leAngle->text();IntValidator::stripAllWhiteSpace(t);
  tb->pos()->setAngle(t.toDouble());
  
  Position::OffsetType ot=Position::OSTNone;
  if(rbPosXOffset->isChecked())ot=Position::OSTPosX;
  if(rbNegXOffset->isChecked())ot=Position::OSTNegX;
  if(rbPosYOffset->isChecked())ot=Position::OSTPosY;
  if(rbNegYOffset->isChecked())ot=Position::OSTNegY;

  tb->pos()->setOffsetType(ot);
  
  ValueArray<long> addtypes;
  for(long i=0;i<lvOffsetSelection->count();i++){
    if(lvOffsetSelection->item(i)->checkState()==Qt::Checked){
      addtypes.append(Position::offsetTypeDescription(i+1)); 
    }
  }
  tb->pos()->setAddTypes(addtypes);

  tb->pos()->setRestriction(rbRestricted->isChecked());

  setDirty(false);
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::updateRequest(void *go)
{
  TextBox *tb=0;
  if(go) tb=dynamic_cast<TextBox*>((GraphObject *)go);

  if(!tb) return;
  bool di=isDirty();
  if(tb->updateRequest() & GraphObject::GOCActive){
    cbActive->setChecked(tb->active());
  }
  if(tb->updateRequest() & GraphObject::GOCText){
    leText->setText(tb->text()());
    leText->setCursorPosition(0);
  }
  if(tb->updateRequest() & GraphObject::GOCOffset){
    leOffset->setText(QString().setNum(tb->pos()->offset(),'g',10));  
    leOffset->setCursorPosition(0);
  }
  if(tb->updateRequest() & GraphObject::GOCPosition){
    setPosition(tb->pos()->pos());
  }
  if(!di) setDirty(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::placementChanged()
{
  frRestriction->setEnabled(rbRestricted->isChecked());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::on_cmbPosCoordSystem_currentIndexChanged(int index)
{
  if(mute_coordsystem)return;
  if(!p_tb)return;

  Point s;
  double o;
  getPosition(s,o);


  Xfp::CoordinateSystem mode=(Xfp::CoordinateSystem)index;

  TextBox *tb=(TextBox*)p_tb;
  if(oldcoordsys != mode){
    s=tb->pos()->dcToPos(tb->graph(),
			 tb->pos()->posToDC(tb->graph(),s,oldcoordsys)
			 ,mode);
    setPosition(s);

//     todo: change offset depending on which restriction it is
//       make sure setFromDrag works
//     myDebug()<<o<<String("s");
//     if(tb->pos()->restriction()){
//       Point pp0;
//       Point pp;
//       if( (tb->pos()->offsettype() & Position::OSTX)  ){
// 	pp=Point(o,0.);
//       }else if( (tb->pos()->offsettype() & Position::OSTY)  ){
// 	pp=Point(0.,o);
//       }
//       myDebug() << "pp"<<pp;
//       myDebug() << "pp0"<<pp0;
//       pp0=tb->dcToPos(tb->posToDC(pp0,oldcoordsys),mode);
//       pp=tb->dcToPos(tb->posToDC(pp,oldcoordsys),mode);
//       myDebug()<<"p  changed"<<pp;
//       myDebug()<<"p0 changed"<<pp0;
//       if( (tb->pos()->offsettype() & Position::OSTX)  ){
// 	o=fabs(pp.x-pp0.x);
//       }else{
// 	o=fabs(pp.y-pp0.y);
//       }
//       myDebug()<<"o"<<o;
//     }
      
    leOffset->setText(QString().setNum(o,'g',10));  
    leOffset->setCursorPosition(0);

  }
  oldcoordsys=mode;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::getPosition(Point &s,double &o)
{
  bool ok;
  
  QString t=leXPos->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYPos->text();IntValidator::stripAllWhiteSpace(t2);

  s=Point(t.toDouble(&ok),t2.toDouble(&ok));

  t=leOffset->text();IntValidator::stripAllWhiteSpace(t);
  o=t.toDouble();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxPropWdg::setPosition(const Point &s)
{
  leXPos->setText(QString().setNum(s.x,'g',10));  
  leYPos->setText(QString().setNum(s.y,'g',10));  

  leXPos->setCursorPosition(0);
  leYPos->setCursorPosition(0);


}

/*EOF*/ 
