/*************************************************************************
 * @(#)TextBoxesModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   28 Apr 2011
 *************************************************************************
 */

#include "TextBox.hh"
#include "TextBoxesModel.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxesModel::TextBoxesModel(QObject * parent)
  : QAbstractListModel(parent),ptextboxes(0)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxesModel::reset(TextBoxes *tbs)
{
  ptextboxes=tbs;
  QAbstractListModel::reset();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant TextBoxesModel::headerData(int section, Qt::Orientation orientation
				    , int role) const
{
  Q_UNUSED(section);
  if (role != Qt::DisplayRole)
    return QVariant();
  if (orientation == Qt::Horizontal) {
    return "Header";
  }
  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int TextBoxesModel::rowCount(const QModelIndex & parent) const
{
  Q_UNUSED(parent);
  if(!ptextboxes) return 0;
  return ptextboxes->size();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant TextBoxesModel::data(const QModelIndex &index,int role)const
{
  if(!ptextboxes) return QVariant();
  if (!index.isValid()) return QVariant();
  int row=index.row();
  //int col=0;
  if (row >= ptextboxes->size() || row < 0) return QVariant();

  const TextBox *tb=ptextboxes->at(row);
  
  if (role == Qt::DisplayRole) {
    return tb->guiName()();
  }

  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBoxesModel::insertRow(int &row,TextBox *tb)
{
  if(!ptextboxes) return false;
  if(row<TextBox::TBTGeneral){
    row=ptextboxes->size();
  }
  beginInsertRows(QModelIndex(),row,row);
  ptextboxes->insertTextBox(row,tb);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxesModel::removeRow(int row)
{
  if(!ptextboxes) return;
  if(row<0 || row>=ptextboxes->size()) return;
  
  TextBox *p=ptextboxes->at(row);
  if(p->type()!=TextBox::TBTGeneral) return;
  
  beginRemoveRows(QModelIndex(), row, row);
  ptextboxes->deleteTextBox(row);
  endRemoveRows();
  return;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBoxesModel::moveRow(int row,int by)
{
  if(!ptextboxes) return false;
  if(!index(row+by,0,QModelIndex()).isValid()) return false;

  TextBox *pfrom=ptextboxes->at(row);
  TextBox *pto=ptextboxes->at(row+by);
  if(!pfrom) return false;
  if(!pto) return false;
  if(pfrom->type()!=TextBox::TBTGeneral) return false;
  if(pto->type()!=TextBox::TBTGeneral) return false;

  
  beginRemoveRows(QModelIndex(), row, row);
  TextBox *p=ptextboxes->takeAt(row);
  endRemoveRows();

  int r=MyMath::max<long>(0,row+by);
  insertRow(r,p);

  return true;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBox *TextBoxesModel::currentTextBox(const QModelIndex &index)const
{
  if(!ptextboxes) return 0;
  if(!index.isValid())return 0;
  return ptextboxes->at(index.row());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QModelIndex TextBoxesModel::textBoxIndex(TextBox *tb)const
{
  if(!tb) return this->index(0,0,QModelIndex());
  return this->index(ptextboxes->indexOf(tb),0,QModelIndex());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxesModel::resetItem(TextBox *tb)
{
  if(!tb) return;
  QModelIndex index=this->index(ptextboxes->indexOf(tb),0,QModelIndex());
  emit dataChanged(index,index);
}



/*EOF*/ 
