/*************************************************************************
 * @(#)TicMarksDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 May 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "TicMarks.hh"
#include "TicMarksDlg.hh"
#include "TicMarksPropWdg.hh"
#include "MyDebug.hh"
#include "FString.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarksDlg::TicMarksDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),t_tms(0)
{
  //myDebug()<<"hier TicMarksDlg"<<this;
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);

  pbClose->setToolTip(IDS_CLOSE_TIP);
  pbApply->setToolTip(IDS_APPLY_TIP);
  pbCancel->setToolTip(IDS_CANCEL_TIP);

  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));

  if(wProps) delete wProps;
  wProps=new TicMarksPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarksDlg::~TicMarksDlg()
{
  //myDebug()<<"hier ~TicMarksDlg"<<this;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::reloadDialog()
{
  setData(t_graph); 
  update();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::updateRequest(Graph *graph,TicMarks *tms,const QString &requ)
{
  ((TicMarksPropWdg*)wProps)->updateRequest(graph,tms,requ);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::onDlgEnd()
{
  //myDebug()<<"TicMarksDlg::onDlgEnd"<<this;
  deleteLater();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::on_pbApply_clicked()
{
  //myDebug() <<"TicMarksDlg::on_pbApply_clicked";
  if(dataAcceptable()){
    apply();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::on_pbClose_clicked()
{
  //myDebug() <<"TicMarksDlg::on_pbClose_clicked";
  if(dataAcceptable()){
    apply();
    emit endRequest();
    return;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TicMarksDlg::dataAcceptable()
{
  QString s,txt="";

  wProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  //myDebug() << "END BondsDlg::dataAcceptable";

  return true;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::apply()
{
  wProps->applyData(t_tms);

//   tbsmodel->resetItem(t_tb);
  emit acceptRequest(t_tms,false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksDlg::setData(Graph *graph,TicMarks *tms)
{
  t_graph=graph;
  if(tms)
    t_tms=tms;
  ((TicMarksPropWdg*)wProps)->setData(t_graph,t_tms);
}



/*EOF*/ 
