/*************************************************************************
 * @(#)TicMarksPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 May 2011
 *************************************************************************
 */

#include <QColor>
#include "Attributes.hh"
#include "TicMarksPropWdg.hh"
#include "TicMarks.hh"
#include "Valid.hh"
#include "Xfp.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarksPropWdg::TicMarksPropWdg(QWidget *parent)
  :PropWdgBase(parent)
{
  setupUi(this);


  QString s="Major X tics ";
  new RealValidator(this,lXMajorSpacing->buddy(),s+lXMajorSpacing->text()
		    ,0.,1e100);
  new RealValidator(this,lXMajorLength->buddy(),s+lXMajorLength->text()
		    ,0.,10); 
  s="Minor X tics ";
  new RealValidator(this,lXMinorLength->buddy(),s+lXMinorLength->text()
		    ,0.,10); 
  s="X tics ";
  new RealValidator(this,lXLabelOffset->buddy(),s+lXLabelOffset->text(),0.,1000.); 

  s="Major Y tics ";
  new RealValidator(this,lYMajorSpacing->buddy(),s+lYMajorSpacing->text()
		    ,0.,1.0e100); 
  new RealValidator(this,lYMajorLength->buddy(),s+lYMajorLength->text()
		    ,0.,10); 
  s="Minor Y tics ";
  new RealValidator(this,lYMinorLength->buddy(),s+lYMinorLength->text()
		    ,0.,10); 

  s="Y tics ";
  new RealValidator(this,lYLabelOffset->buddy(),s+lYLabelOffset->text(),0.,1000.); 

  s="Label ";
  new IntValidator(this,lXFontSize->buddy(),s+lXFontSize->text(),1,1000); 
  new RealValidator(this,lXSubSc->buddy(),s+lXSubSc->text(),0.,100.);

  s="Label ";
  new IntValidator(this,lYFontSize->buddy(),s+lYFontSize->text(),1,1000); 
  new RealValidator(this,lYSubSc->buddy(),s+lYSubSc->text(),0.,100.);

  cmbXSite->clear();
  for(int i=0;i<Xfp::ticMaxSides();i++){
    cmbXSite->insertItem(i,Xfp::ticSideGuiName(i),i);
  }
  cmbXLabelSite->clear();
  for(int i=0;i<Xfp::ticMaxSides();i++){
    cmbXLabelSite->insertItem(i,Xfp::ticSideGuiName(i),i);
  }

  cmbYSite->clear();
  for(int i=0;i<Xfp::ticMaxSides();i++){
    cmbYSite->insertItem(i,Xfp::ticSideGuiName(i),i);
  }
  cmbYLabelSite->clear();
  for(int i=0;i<Xfp::ticMaxSides();i++){
    cmbYLabelSite->insertItem(i,Xfp::ticSideGuiName(i),i);
  }

  cmbXMajorLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbXMajorLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  cmbYMajorLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbYMajorLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  cmbXMinorLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbXMinorLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  cmbYMinorLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbYMinorLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }

  
  connectChanged(this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarksPropWdg::~TicMarksPropWdg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksPropWdg::check(QString &msg)
{
  ((IntValidator*)leXMajorSpacing->validator())->check(msg);
  ((IntValidator*)leXMajorLength->validator())->check(msg); 
  ((IntValidator*)leXMinorLength->validator())->check(msg); 
  ((IntValidator*)leXLabelOffset->validator())->check(msg); 


  ((IntValidator*)leYMajorSpacing->validator())->check(msg);
  ((IntValidator*)leYMajorLength->validator())->check(msg); 
  ((IntValidator*)leYMinorLength->validator())->check(msg); 
  ((IntValidator*)leYLabelOffset->validator())->check(msg); 

  ((IntValidator*)leXFontSize->validator())->check(msg); 
  ((IntValidator*)leYFontSize->validator())->check(msg); 
  ((IntValidator*)leXSubSc->validator())->check(msg); 
  ((IntValidator*)leYSubSc->validator())->check(msg); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void TicMarksPropWdg::updateRequest(Graph *gr,TicMarks *tms,const QString &requ)
{
  bool di=isDirty();
  if(requ=="adjustxtics"){
    cmbXSite->setCurrentIndex(tms->ticSide(Xfp::Graph_X));  
  }
  //todo
  
  wUserTics->updateRequest(gr,requ);
  if(!di) setDirty(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksPropWdg::applyData(void *go)
{
  //NSF("applydata");
  bool ok;
  TicMarks *tms=0;
  if(go) tms=dynamic_cast<TicMarks*>((GraphObject *)go);
  if(!tms) return;

  tms->setAutoTics(Xfp::Graph_X,cbXAutoTics->isChecked()); 
  QString t=leXMajorSpacing->text();IntValidator::stripAllWhiteSpace(t);
  tms->setMajorTics(Xfp::Graph_X,t.toDouble(&ok));

  t=leXMajorLength->text();IntValidator::stripAllWhiteSpace(t);
  tms->setTicLength(Xfp::Graph_X,Xfp::TKIMajor,t.toDouble(&ok));
  tms->tics()->setDrawTics(Xfp::Graph_X,Xfp::TKIMajor
			   ,grbxMajorXTics->isChecked()); 


  tms->setMinorTics(Xfp::Graph_X,spbxXMinorSubdiv->value());

  t=leXMinorLength->text(); IntValidator::stripAllWhiteSpace(t);
  tms->setTicLength(Xfp::Graph_X,Xfp::TKIMinor,t.toDouble(&ok));

  tms->tics()->setDrawTics(Xfp::Graph_X,Xfp::TKIMinor
			   ,grbxMinorXTics->isChecked());


  tms->setTicSide(Xfp::Graph_X,(Xfp::TicSide)cmbXSite->currentIndex()); 
  tms->setTicLabelSide(Xfp::Graph_X,(Xfp::TicSide)cmbXLabelSite->currentIndex()); 

  t=leXLabelOffset->text() ;IntValidator::stripAllWhiteSpace(t);
  tms->setLabelOffset(Xfp::Graph_X,t.toDouble(&ok));



  tms->setAutoTics(Xfp::Graph_Y,cbYAutoTics->isChecked()); 
  t=leYMajorSpacing->text();IntValidator::stripAllWhiteSpace(t);
		    
  tms->setMajorTics(Xfp::Graph_Y,t.toDouble(&ok));
  t=leYMajorLength->text();IntValidator::stripAllWhiteSpace(t);
		    
  tms->setTicLength(Xfp::Graph_Y,Xfp::TKIMajor,t.toDouble(&ok));
  tms->setSeparateYTicLength(Xfp::TKIMajor,cbYMajorLength->isChecked());

  tms->tics()->setDrawTics(Xfp::Graph_Y,Xfp::TKIMajor
			   ,grbxMajorYTics->isChecked()); 
  tms->setMinorTics(Xfp::Graph_Y,spbxYMinorSubdiv->value());
  t=leYMinorLength->text();IntValidator::stripAllWhiteSpace(t);		    
  tms->setTicLength(Xfp::Graph_Y,Xfp::TKIMinor,t.toDouble(&ok));
  tms->setSeparateYTicLength(Xfp::TKIMinor,cbYMinorLength->isChecked());

  tms->tics()->setDrawTics(Xfp::Graph_Y,Xfp::TKIMinor
			   ,grbxMinorYTics->isChecked());
  tms->setTicSide(Xfp::Graph_Y,(Xfp::TicSide)cmbYSite->currentIndex()); 
  tms->setTicLabelSide(Xfp::Graph_Y,(Xfp::TicSide)cmbYLabelSite->currentIndex()); 
  t=leYLabelOffset->text();IntValidator::stripAllWhiteSpace(t);
  tms->setLabelOffset(Xfp::Graph_Y,t.toDouble(&ok));


  tms->setDecimals(Xfp::Graph_X,spbxXDec->value());
  tms->setDecimals(Xfp::Graph_Y,spbxYDec->value());

  t=leXFontSize->text();IntValidator::stripAllWhiteSpace(t);
  tms->textStyle(Xfp::Graph_X)->setSize(t.toLong(&ok));

  t=leYFontSize->text();IntValidator::stripAllWhiteSpace(t);
  tms->textStyle(Xfp::Graph_Y)->setSize(t.toLong(&ok));
  t=leXSubSc->text();IntValidator::stripAllWhiteSpace(t);
  tms->textStyle(Xfp::Graph_X)->setSubScriptScale(t.toDouble(&ok));
  t=leYSubSc->text();IntValidator::stripAllWhiteSpace(t);		      
  tms->textStyle(Xfp::Graph_Y)->setSubScriptScale(t.toDouble(&ok));
  tms->textStyle(Xfp::Graph_X)->setColor(cbXColor->color()); 
  tms->textStyle(Xfp::Graph_Y)->setColor(cbYColor->color()); 

  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->
    setStyle((Xfp::LineStyles)cmbXMajorLineStyle->currentIndex());
  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->
    setWidth(spbxXMajorLineWidth->value());
  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->
    setColor(cbXMajorLineColor->color());


  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->
    setStyle((Xfp::LineStyles)cmbYMajorLineStyle->currentIndex());
  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->
    setWidth(spbxYMajorLineWidth->value());
  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->
    setColor(cbYMajorLineColor->color());
  setDirty(false);


  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->
    setStyle((Xfp::LineStyles)cmbXMinorLineStyle->currentIndex());
  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->
    setWidth(spbxXMinorLineWidth->value());
  tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->
    setColor(cbXMinorLineColor->color());


  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->
    setStyle((Xfp::LineStyles)cmbYMinorLineStyle->currentIndex());
  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->
    setWidth(spbxYMinorLineWidth->value());
  tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->
    setColor(cbYMinorLineColor->color());
  setDirty(false);

  wUserTics->applydata();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarksPropWdg::setData(Graph *gr,GraphObject *go)
{
  TicMarks *tms=0;
  if(!gr) {PropWdgBase::reset(this);return;}
  if(go) tms=dynamic_cast<TicMarks*>(go);
  if(!tms) return;


  cbXAutoTics->setChecked(tms->autoTics(Xfp::Graph_X)); 
  leXMajorSpacing->setText(QString().setNum(tms->majorTics(Xfp::Graph_X)));
  leXMajorLength->setText(QString().setNum(tms->ticLength(Xfp::Graph_X
							  ,Xfp::TKIMajor)));
  
  grbxMajorXTics->setChecked(tms->tics()->
			     drawTics(Xfp::Graph_X,Xfp::TKIMajor));


  spbxXMinorSubdiv->setValue(tms->minorTics(Xfp::Graph_X)); 
  leXMinorLength->setText(QString().setNum(tms->ticLength(Xfp::Graph_X
							  ,Xfp::TKIMinor)));

  
  grbxMinorXTics->setChecked(tms->tics()->drawTics(Xfp::Graph_X
						     ,Xfp::TKIMinor));

  cmbXSite->setCurrentIndex(tms->ticSide(Xfp::Graph_X));  
  cmbXLabelSite->setCurrentIndex(tms->ticLabelSide(Xfp::Graph_X));  
  leXLabelOffset->setText(QString().setNum(tms->labelOffset(Xfp::Graph_X)));


  cbYAutoTics->setChecked(tms->autoTics(Xfp::Graph_Y));
  leYMajorSpacing->setText(QString().setNum(tms->majorTics(Xfp::Graph_Y)));
  leYMajorLength->setText(QString().setNum(tms->ticLength(Xfp::Graph_Y
							  ,Xfp::TKIMajor)));
  cbYMajorLength->setChecked(tms->separateYTicLength(Xfp::TKIMajor));
  grbxMajorYTics->setChecked(tms->tics()->drawTics(Xfp::Graph_Y
						     ,Xfp::TKIMajor));

  spbxYMinorSubdiv->setValue(tms->minorTics(Xfp::Graph_Y)); 
  leYMinorLength->setText(QString().setNum(tms->ticLength(Xfp::Graph_Y
							  ,Xfp::TKIMinor)));  
  cbYMinorLength->setChecked(tms->separateYTicLength(Xfp::TKIMinor));
  grbxMinorYTics->setChecked(tms->tics()->drawTics(Xfp::Graph_Y
						     ,Xfp::TKIMinor));
  cmbYSite->setCurrentIndex(tms->ticSide(Xfp::Graph_Y));  
  cmbYLabelSite->setCurrentIndex(tms->ticLabelSide(Xfp::Graph_Y));  
  leYLabelOffset->setText(QString().setNum(tms->labelOffset(Xfp::Graph_Y)));

  spbxXDec->setValue(tms->decimals(Xfp::Graph_X)); 
  spbxYDec->setValue(tms->decimals(Xfp::Graph_Y)); 

  
  leXFontSize->setText(QString().setNum(tms->textStyle(Xfp::Graph_X)->size())); 
  leXSubSc->setText(QString().setNum(tms->textStyle(Xfp::Graph_X)->subScriptScale()
				    ,'g',10));   
  cbXColor->setColor(tms->textStyle(Xfp::Graph_X)->color());

  leYFontSize->setText(QString().setNum(tms->textStyle(Xfp::Graph_Y)->size())); 
  leYSubSc->setText(QString().setNum(tms->textStyle(Xfp::Graph_Y)->subScriptScale()
				    ,'g',10));   
  cbYColor->setColor(tms->textStyle(Xfp::Graph_Y)->color());

  cmbXMajorLineStyle->
    setCurrentIndex(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->style());  
  spbxXMajorLineWidth->
    setValue(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->width());
  cbXMajorLineColor->
    setColor(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMajor)->color());

  cmbYMajorLineStyle->
    setCurrentIndex(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->style());  
  spbxYMajorLineWidth->
    setValue(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->width());
  cbYMajorLineColor->
    setColor(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMajor)->color());

  cmbXMinorLineStyle->
    setCurrentIndex(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->style());  
  spbxXMinorLineWidth->
    setValue(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->width());
  cbXMinorLineColor->
    setColor(tms->lineStyle(Xfp::Graph_X,Xfp::TKIMinor)->color());

  cmbYMinorLineStyle->
    setCurrentIndex(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->style());  
  spbxYMinorLineWidth->
    setValue(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->width());
  cbYMinorLineColor->
    setColor(tms->lineStyle(Xfp::Graph_Y,Xfp::TKIMinor)->color());

  
  wUserTics->setData(gr);

  tms->setChanged(false);
  setDirty(false);
}

/*EOF*/ 
