/*************************************************************************
 * @(#)TransformDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   09 Sep 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QMenuBar>
#include <QFileDialog>
#include <QMenu>
#include <QAction>
#include "Except.hh"
#include "FileActions.hh"
#include "Page.hh"
#include "ViewBox.hh"
#include "LegendBox.hh"
#include "World.hh"
#include "TextBox.hh"
#include "Shape.hh"
#include "HelpDlg.hh"
#include "Assorted.hh"
#include "ParameterDocument.hh"
#include "TransformDlg.hh"
#include "PtrArray.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Graphs.hh"
#include "InsertMenuData.hh"
#include "graphview.hh"
#include "Config.hh"
#include "SearchWidget.hh"
#include "MyDebug.hh"



Q_DECLARE_METATYPE(InsertMenuData)

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TransformDlg::TransformDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),
  pfileactions(new FileActions(this,"commands.cmd")),menubar(0),insertmenu(0)
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME);

  pbApply->setToolTip(IDS_APPLY_TIP);



  connect(this,SIGNAL(endRequest()),this,SLOT(onDlgEnd()));

  pteCommandsPallette=pteCommands->palette();
  pteCommands->setCursorWidth(2);


  wSearch->setTextWidget(pteCommands);


  menubar=new QMenuBar(this);
  QMenu *m=menubar->addMenu("&File");
  QAction *a=m->addAction("&Load",this,SLOT(slotLoad()),QKeySequence(Qt::CTRL+Qt::Key_O));
  a=m->addAction("&Save",this,SLOT(slotSave()),QKeySequence(Qt::CTRL+Qt::Key_S));
  a->setData(1);
  a=m->addAction("Save As",this,SLOT(slotSave()));
  a->setData(2);
  a=m->addAction("&Apply",this,SLOT(on_pbApply_clicked()));
  a=m->addAction("&Close/Apply",this,SLOT(slotClose()));
  a=m->addAction("Close",this,SLOT(slotCancel()),
		 QKeySequence(Qt::CTRL+Qt::Key_W));
  m=menubar->addMenu("&Edit");
  a=m->addAction("&Find Ctrl+F",wSearch,SLOT(startSearch()));
  a=m->addAction("&Undo",pteCommands,SLOT(undo()),QKeySequence(Qt::CTRL+Qt::Key_Z));
  
  insertmenu=menubar->addMenu("&Insert");
  m=menubar->addMenu("&Help");
  m->addAction("&Help",this,SLOT(help()),QKeySequence(Qt::Key_F1));

  connect(insertmenu,SIGNAL(aboutToShow()),this,SLOT(onInsertMenuAboutToShow()));

  mainLayout->setMenuBar(menubar);

  pteCommands->setAcceptRichText(false);
  //myDebug()<<"NEW";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TransformDlg::~TransformDlg()
{
  //  myDebug() << "TransformDlg::~TransformDlg";

  if(menubar) delete menubar;

  //  myDebug() << "end TransformDlg::~TransformDlg";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TransformDlg::dataAcceptable()
{
  QString s,txt="";

  //wSetProps->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME 
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  //myDebug() << "END BondsDlg::dataAcceptable";

  return true;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::apply()
{
  String ret=pgv->playParamDocument(false);
  //myDebug()<<"ret="<<ret;
  if(!ret.empty()){
    QMessageBox::warning(this,IDS_APP_NAME,ret()
			 ,QMessageBox::Ok | QMessageBox::Default,0,0);

    StringList sl=ret.split(":");
    //myDebug()<<"sl="<<sl;
    if(sl.size()>0){
      try{
	sl=sl[1].split(".");
	long line=-1;
	long col=-1;
	line=sl[0].convertToLong();
	if(sl[1].find("-")>=0){
	  sl=sl[1].split("-");
	  col=sl[0].convertToLong();
	}else{
	  col=sl[1].convertToLong();
	}
	QTextCursor c=pteCommands->textCursor();
	c.movePosition(QTextCursor::Start,QTextCursor::MoveAnchor,1);
	c.movePosition(QTextCursor::Down,
		       QTextCursor::MoveAnchor,line-1);
	c.movePosition(QTextCursor::Right,
		       QTextCursor::MoveAnchor,col-1);
	pteCommands->setTextCursor(c);
	pteCommands->setFocus();
	
      }catch(...){
	myDebug()<<"failure"<<sl;
      }
    }
  }
  
  emit acceptRequest(0,true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::setData(GraphView *gv,ParameterDocument *docu)
{
  pdocument=docu;
  pgv=gv;
  if(!pdocument->paramFile().isEmpty())
    pfileactions->setFileName(pdocument->paramFile(),true);
  pteCommands->setDocument(docu->textDocument());
  on_pteCommands_textChanged();
  pteCommands->setFocus();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::slotSave()
{
  QAction *a=qobject_cast<QAction*>(sender());
  if(!a)return;
  int imd=a->data().toInt();

  QString filename=pfileactions->save("Save commands",".",
				      "Commands ( *.cmd ) ;; All ( * )",(imd==2));
  
  
  if(!filename.isEmpty()){
    pdocument->saveDocument(filename);
    on_pteCommands_textChanged();
    on_pteCommands_cursorPositionChanged();
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::slotLoad()
{
  
  QString filename=pfileactions->load("Load commands",".",
			      "Commands ( *.cmd ) ;; All ( * )");
  
  
  if(!filename.isEmpty()){
    pdocument->loadDocument(filename);
    on_pteCommands_textChanged();

  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::on_pteCommands_textChanged()
{
  //myDebug()<<"on_pteCommands_textChanged";
  if(pdocument->textDocument()->isModified()){
    QPalette p=pteCommandsPallette;
    p.setColor(QPalette::Base,QColor(255, 210, 210));
    pteCommands->setPalette(p);
  }else{
    pteCommands->setPalette(pteCommandsPallette);
  }

  //  pteCommands->setTextBackgroundColor(0xff0000);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::on_pteCommands_cursorPositionChanged()
{
  //  myDebug()<<"TransformDlg::on_pteCommands_cursorPositionChanged";
  QTextCursor c=pteCommands->textCursor();
  //myDebug() << c.blockNumber()<<c.columnNumber();
  lPosition->setText( QString("<html><head/><body><p>%1: <span style=\" "
			      "font-weight:600;\">Line:</span> %2 <span"
			      " style=\" font-weight:600;\">Col:</span> "
			      "%3</p></body></html>")
		      .arg(pfileactions->fileBaseName())
		      .arg(c.blockNumber()+1).arg(c.columnNumber()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::onInsertMenuAboutToShow()
{
  insertmenu->clear();
  Graphs *graphs=pgv->page()->graphs();
  QMenu *m=insertmenu;
  Graph *gr=graphs->currentGraph();
  if(graphs->size()>1){
    for(int ig=0;ig<graphs->size();ig++){
      Graph *gr=graphs->at(ig);
      if(gr==graphs->currentGraph()) {
	m=insertmenu->addMenu(QString("graph %1 [current]")
			   .arg(gr->guiName()()));
      }else{
	m=insertmenu->addMenu(QString("graph %1").arg(gr->guiName()()));
      }
      //      insertmenu->addMenu(m);
      addContextGraphActions(gr,m);
    }
  }else{
    addContextGraphActions(gr,m);
  }
  
  

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::help()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","Scripting");
  h->show();
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::addContextGraphActions(Graph *gr,QMenu *m)
{
  const char *slot=SLOT(onInsertMenu());
  m->addAction("&View size/pos",this,slot)->
    setData(QVariant::
	    fromValue(InsertMenuData(InsertMenuData::View_Size_Pos,
				     gr->viewBox())));
  m->addAction("&View properties",this,slot)->
    setData(QVariant::fromValue(InsertMenuData(InsertMenuData::View_Props,
					       gr->viewBox())));

  m->addAction("&Legend",this,slot)->
    setData(QVariant::
	    fromValue(InsertMenuData(InsertMenuData::Legend,
				     gr->legendBox())));

  m->addAction("&World",this,slot)->
    setData(QVariant::
	    fromValue(InsertMenuData(InsertMenuData::World,
				     gr->world())));


  QMenu *ms=m->addMenu("&TextBoxes");
  for(int it=0;it<gr->textBoxes()->size();it++){
    TextBox *tb=gr->textBoxes()->at(it);
    ms->addAction(tb->guiName(true)(),this,slot)->
      setData(QVariant::fromValue(InsertMenuData(InsertMenuData::TextBox,
						 tb)));
  }  

  ms=m->addMenu("&Shapes");
  for(int is=0;is<gr->shapes()->size();is++){
    Shape *sh=gr->shapes()->at(is);
    ms->addAction(sh->guiName(true)(),this,slot)->
      setData(QVariant::fromValue(InsertMenuData(InsertMenuData::Shape,
						 sh)));
  }  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TransformDlg::onInsertMenu()
{
  QAction *a=qobject_cast<QAction*>(sender());
  if(!a)return;
  InsertMenuData imd=a->data().value<InsertMenuData>();
  GraphObject *go=imd.graphobject;
  if(!go) return;

  pteCommands->insertPlainText(go->toCommands(imd.type)());
  
}
/*EOF*/ 
