/* @(#)TransformDlg.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _TRANSFORMDLG_H
#define _TRANSFORMDLG_H 1

#include <QCloseEvent>
#include "BrainDialog.hh"
#include "ui_TransformDlg.h"

class QMenu;
class QMenuBar;
class GraphObject;
class GraphView;
class Graph;
class FileActions;
class ParameterDocument;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class TransformDlg: public BrainDialog, protected Ui::TransformDlg
{
  Q_OBJECT
public:
  TransformDlg(QWidget *parent,QWidget *base,const char *name);
  ~TransformDlg();

  void setData(GraphView *gv,ParameterDocument *docu);

signals:
  void acceptRequest(GraphObject *,bool forceChanged);
  void endRequest();

public slots:
  virtual void reject(){ };//block dialog behaviour
protected:
  virtual void closeEvent( QCloseEvent * e ) {e->accept(); emit endRequest();};
private slots:
  void on_pbApply_clicked(){ if(dataAcceptable()){apply();}};
  void slotCancel(){ emit endRequest(); };
  void slotLoad();
  void help();
  void slotClose(){
    if(dataAcceptable()){apply(); emit endRequest();return;}};
  void onDlgEnd(){deleteLater();};
  void on_pteCommands_textChanged();
  void on_pteCommands_cursorPositionChanged();
  void onInsertMenu();
  void onInsertMenuAboutToShow();
  void slotSave();
private:
  bool dataAcceptable();
  void apply();
  void addContextGraphActions(Graph *gr,QMenu *m);
 private:
  GraphView *pgv;
  ParameterDocument *pdocument;
  QPalette pteCommandsPallette;
  FileActions *pfileactions;
  QMenuBar *menubar;
  QMenu *insertmenu;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _TRANSFORMDLG_H */

