/*************************************************************************
 * @(#)UserTicModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 May 2011
 *************************************************************************
 */

#include "TicMarks.hh"
#include "UserTicModel.hh"
//#include "debug.h"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
UserTicModel::UserTicModel(QObject * parent)
  : QAbstractTableModel(parent),ptics(0)
{
  pheader=QString("Axis\nType\nPosition\nLabel\nTic Side\nL. Side\nLength").split("\n");
  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
UserTicModel::~UserTicModel()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicModel::resetData(Tics *tics)
{
  ptics=tics;
  reset();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant UserTicModel::headerData(int section, Qt::Orientation orientation
				  , int role) const
{
  if (role != Qt::DisplayRole)
    return QVariant();
  if (orientation == Qt::Horizontal) {
    return pheader.at(section);
  }
  return QVariant();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int UserTicModel::rowCount(const QModelIndex & parent) const
{
  Q_UNUSED(parent);
  if(!ptics) return 0;
  return ptics->size();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int UserTicModel::columnCount(const QModelIndex &parent) const
{
  Q_UNUSED(parent);
  return pheader.size();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Qt::ItemFlags UserTicModel::flags(const QModelIndex &index) const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;

  int row=index.row();
  int col=index.column();

  if (row >= ptics->size() || row < 0) return QAbstractTableModel::flags(index);
  if (col >= columnCount() || col < 0) return QAbstractTableModel::flags(index);


  //const Tic *t=ptics->tic(row);
  return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant UserTicModel::data(const QModelIndex &index, int role) const
{
  if (!ptics) return QVariant();
  if (!index.isValid()) return QVariant();


  int row=index.row();
  int col=index.column();


  if (row >= ptics->size() || row < 0) return QVariant();
  if (col >= columnCount() || col < 0) return QVariant();

  const Tic *t=ptics->tic(row);

  if (role == Qt::DisplayRole) {
    switch(col){
    case AxisType:
      return (t->type()==Xfp::Graph_X?"X":"Y");
    case Kind:
      return (t->kind()==Xfp::TKIMajor?"Major":"Minor");
    case Position:
      return QString("%1").arg(t->pos());
    case Label:
      return t->label()();
    case TicSide:
      return Xfp::ticSideGuiName(t->ticSide());
    case TicLabelSide:
      return Xfp::ticSideGuiName(t->ticLabelSide());
    case TicLength:
      return QString("%1").arg(t->ticLength());
    default:
      return QVariant();
    }
  }else if(role == Qt::EditRole) {
    switch(col){
    case AxisType:
      return t->type();
    case Kind:
      return t->kind();
    case Position:
      return QString("%1").arg(t->pos());
    case Label:
      return t->label()();
    case TicSide:
      return t->ticSide();
    case TicLabelSide:
      return t->ticLabelSide();
    case TicLength:
      return QString("%1").arg(t->ticLength());
    default:
      return QVariant();
    }
    
  }else  if( role == Qt::CheckStateRole ){
    return QVariant();
  }
  return QVariant();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool UserTicModel::setData(const QModelIndex &index, const QVariant &value
			   , int role)
{
  //NSF("SpspModel::setData"); 
  //  myDebug() << nsubfunc;

  if(!ptics) return false;
  if (!index.isValid()) return false;

  int row = index.row();
  int col = index.column();
  Tic *t=ptics->tic(row);
  if ( role == Qt::EditRole) {
    switch(col){
    case AxisType:
      t->setType((Xfp::XYAxisType)value.toInt());
      break;
    case Kind:
      t->setKind((Xfp::TicKind)value.toInt());
      break;
    case Position:
      t->setPos(value.toDouble());
      //todo: validation
      break;
    case Label:
      t->setLabel(String(value.toString()));
      break;
    case TicSide:
      t->setTicSide((Xfp::TicSide)value.toInt());
      break;
    case TicLabelSide:
      t->setTicLabelSide((Xfp::TicSide)value.toInt());
      break;
    case TicLength:
      t->setTicLength(value.toDouble());
      //todo: validation
      break;
    default:
      return false;
    }

    emit(dataChanged(index, index));

    return true;

  }
  return false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool UserTicModel::insertRow(int &row,Tic *t)
{
  if(!ptics) return false;
  if(!t){
    if(row-1>=0 && row-1<ptics->size())
      t=ptics->tic(row-1);
  }
  beginInsertRows(QModelIndex(),row,row);
  if(t)
    ptics->insert(row,new Tic(*t));
  else 
    ptics->insert(row);
 endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicModel::removeRow(int row)
{
  if(!ptics) return;
  if(row<0 || row>=ptics->size()) return;
  
  beginRemoveRows(QModelIndex(), row, row);
  ptics->deleteTic(row);
  endRemoveRows();
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool UserTicModel::moveRow(int row,int by)
{
  if(!ptics) return false;
  if(!index(row+by,0,QModelIndex()).isValid()) return false;

  Tic *tfrom=ptics->tic(row);
  Tic *tto=ptics->tic(row+by);
  if(!tfrom) return false;
  if(!tto) return false;

  
  beginRemoveRows(QModelIndex(), row, row);
  Tic *t=ptics->takeAt(row);
  endRemoveRows();

  int r=MyMath::max<long>(0,row+by);
  insertRow(r,t);

  return true;

}


/*EOF*/ 
