//todo: we need reference to where the group tics come from
//todo: and then we have to keep track of their destruction and movement
//todo: especially when set dialog changes stuff

/*************************************************************************
 * @(#)UserTicWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 May 2011
 *************************************************************************
 */

#include <QColor>
#include <QMessageBox>
#include "Attributes.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "ComboBoxDelegate.hh"
#include "Graph.hh"
#include "TicMarks.hh"
#include "ToolButtons.hh"
#include "UserTicWdg.hh"
#include "defs.hh"
#include "MyDebug.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
UserTicWdg::UserTicWdg(QWidget *parent)
  :QWidget(parent),umodel(0),axisdelegate(0),kinddelegate(0),sidedelegate(0)
  ,putics(0)
{
  setupUi(this);

  umodel=new UserTicModel(this);
  tvUserTics->setModel(umodel);

  QMap<int,QString> axismap;
  axismap[Xfp::Graph_X]="X";
  axismap[Xfp::Graph_Y]="Y";
  QMap<int,QString> kindmap;
  kindmap[Xfp::TKIMajor]="Major";
  kindmap[Xfp::TKIMinor]="Minor";
  QMap<int,QString> sidemap;
  for(int i=0;i<Xfp::ticMaxSides();i++){
    sidemap[Xfp::ticSideType(i)]=Xfp::ticSideGuiName(i);
  }

  axisdelegate=new ComboBoxDelegate(this,axismap);
  kinddelegate=new ComboBoxDelegate(this,kindmap);
  sidedelegate=new ComboBoxDelegate(this,sidemap);

  connect(umodel,SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&))
	  ,this,SLOT(onModelDataChanged(const QModelIndex&
					    ,const QModelIndex&)));

  
  tvUserTics->setItemDelegateForColumn(UserTicModel::AxisType,axisdelegate);
  tvUserTics->setItemDelegateForColumn(UserTicModel::Kind,kinddelegate);
  tvUserTics->setItemDelegateForColumn(UserTicModel::TicSide,sidedelegate);
  tvUserTics->setItemDelegateForColumn(UserTicModel::TicLabelSide,sidedelegate);


  ptoolbuttons.createToolButtonsActions(tvUserTics,"Tic"
					,tbInsert,tbDelete,tbUp,tbDown);

  ptoolbuttons.connectSlots(this,
			    SLOT(slotInsertTic()),
			    SLOT(slotDeleteTic()),
			    SLOT(slotMoveTic()),
			    SLOT(slotMoveTic()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
UserTicWdg::~UserTicWdg()
{
  if(umodel) delete umodel; umodel=0;
  if(axisdelegate) delete axisdelegate;axisdelegate=0;
  if(kinddelegate) delete kinddelegate;kinddelegate=0;
  if(sidedelegate) delete sidedelegate;sidedelegate=0;

  if(putics) delete putics; 
  for(int i=0;i<pgtics.size();i++){
    if(pgtics[i].second) delete pgtics[i].second;
  }
  pgtics.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::setData(Graph *gr)
{
  pgraph=gr;
  if(putics)delete putics;putics=0;
  //putics=new Tics(*(pgraph->ticMarks()->userTics()));
  putics=new Tics();
  *putics=*(pgraph->ticMarks()->userTics());
  putics->setChanged(false);
  
  for(int i=0;i<pgtics.size();i++){
    if(pgtics[i].second) delete pgtics[i].second;
  }
  pgtics.clear();
  
  cmbGroup->clear();
  cmbGroup->insertItem(0,QString("Global"),0);
  for(int i=0;i<pgraph->groups()->size();i++){
    pgtics.append(qMakePair(pgraph->groups()->at(i),
			    new Tics(*(pgraph->groups()->at(i)->userTics()))));
    pgtics.last().second->setChanged(false);
    cmbGroup->insertItem(i+1,QString("Group %1").arg(i+1),i+1);
  }
  cmbGroup->setCurrentIndex(0);  

  //cbDrawUserTics->setChecked(putics->drawTics(Xfp::Graph_X,Xfp::TKIMajor));
  tvUserTics->setCurrentIndex(umodel->index(0,0,QModelIndex()));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::updateRequest(Graph *gr,const QString &requ)
{
  // in the moment only new groups are added
  if(requ=="eftic"){
    if(putics)delete putics;putics=0;
    putics=new Tics(*(pgraph->ticMarks()->userTics()));
    if(cmbGroup->currentIndex()==0){
      umodel->resetData(putics);
      cbDrawUserTics->setChecked(putics->drawTics(Xfp::Graph_X,Xfp::TKIMajor));
      adjustTvUserTics();
    }
  }else if(requ=="grouptics"){
    if(pgraph!=gr){
      myDebug() <<"local copy of graph different from updateRequest(Graph*) argument ... need to implement multiple graphs";
      exit(1);
    }
    
    long cindex=cmbGroup->currentIndex();
    bool needrefresh=false;
    for(int ig=0;ig<pgraph->groups()->size();ig++){
      bool haveit=false;
      FunctionGroup *fg=pgraph->groups()->at(ig);
      for(long i=0;i<pgtics.size();i++){
	if(pgtics[i].first ==fg){
	  haveit=true;
	  break;
	}
      }
      if(!haveit){
	// new group, which we do not have
	cmbGroup->insertItem(ig+1,QString("Group %1").arg(ig+1),ig+1);
	pgtics.append(qMakePair(fg,new Tics(*(fg->userTics()))));
	if(cindex-1==ig)
	  needrefresh=true;
      }
    }  
    if(needrefresh){
      Tics *tcs=pgtics[cindex-1].second;
      umodel->resetData(tcs);
      cbDrawUserTics->setChecked(tcs->drawTics(Xfp::Graph_X,Xfp::TKIMajor));
      adjustTvUserTics();
    }
  }else{
    return;
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::applydata()
{
  //myDebug()<<"UserTicWdg::applydata()";
  *(pgraph->ticMarks()->userTics())=*putics;
  putics->setChanged(false);
  
  for(int i=0;i<pgtics.size();i++){
    //myDebug() << "pgtics i"<<i;
    int ig=pgraph->groups()->indexOf(pgtics[i].first);
    Tics *tcs=pgtics[i].second;
    if(ig>=0 && tcs && pgraph->groups()->at(ig)->userTics()){
      //myDebug() << *tcs;
      *(pgraph->groups()->at(ig)->userTics())=*(tcs);
      tcs->setChanged(false);
      //myDebug() << "pgraph->groups()->at(ig)->userTics()"
      //       <<*pgraph->groups()->at(ig)->userTics();

    }
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::on_cmbGroup_currentIndexChanged(int current)
{
  //NSF("UserTicWdg::on_cmbGroup_currentIndexChanged(int)");
  //myDebug()<<nsubfunc;

  if(current<0) return;
  
  Tics *tics=0;
  if(current==0){
    tics=putics;
  }else{
    if(pgtics.at(current-1).second){
      tics=pgtics[current-1].second;
    }
  }
  if(tics){
    umodel->resetData(tics);
    cbDrawUserTics->setChecked(tics->drawTics(Xfp::Graph_X,Xfp::TKIMajor));
  }
  adjustTvUserTics();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::on_cbDrawUserTics_toggled(bool b)
{
  int ind=cmbGroup->currentIndex();
  if(!ind){
    putics->setDrawTics(b);
  }else{
    pgtics[ind-1].second->setDrawTics(b);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::adjustTvUserTics()
{
  int rows=umodel->rowCount();
  int cols=umodel->columnCount();
  for(int irow=0;irow<rows;irow++)
    tvUserTics->resizeRowToContents(irow);
  for(int icol=0;icol<cols;icol++){
    tvUserTics->resizeColumnToContents(icol);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::onModelDataChanged(const QModelIndex&
					     ,const QModelIndex&)
{
  //myDebug()<<"UserTicWdg::onModelDataChanged";
  adjustTvUserTics();
  emit dataChanged();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::slotInsertTic()
{
  //NSF("UserTicWdg::slotInsertTic()");
  //myDebug()<< nsubfunc;

  QModelIndex index=tvUserTics->currentIndex();
  int row=0;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
    col=index.column();
  }
  umodel->insertRow(row);
  QModelIndex ci=umodel->index(row,col,QModelIndex());
  tvUserTics->scrollTo(ci);
  tvUserTics->setCurrentIndex(ci);
  adjustTvUserTics();
  emit dataChanged();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void UserTicWdg::slotDeleteTic()
{
  //NSF("UserTicWdg::slotDeleteTic"); 
  //  myDebug() << nsubfunc ;

  int numrows=umodel->rowCount();

  if(numrows<=0) return;

  QModelIndex currentindex=tvUserTics->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a tic first."
			   , 1, 0,0 );
    return;
  }

  int row=currentindex.row();
//  int col=currentindex.column();


  umodel->removeRow(row);
  if(numrows!=umodel->rowCount())
    tvUserTics->setCurrentIndex(umodel->
				index(MyMath::min<long>(row,numrows-2),0,QModelIndex()));

  adjustTvUserTics();

  emit dataChanged();

  return;

}

/* ***********************************************************************
*
* ***********************************************************************
*/
void UserTicWdg::slotMoveTic()
{
  NSF("UserTicWdg::slotMoveTic"); Q_UNUSED(nsubfunc);

  QModelIndex currentindex=tvUserTics->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a tic first."
			   , 1, 0,0 );
    return;
  }
  int row=currentindex.row();
  int col=currentindex.column();
  int by=-1;
  if(sender()==ptoolbuttons.pmoveDownAction) by=1;

  if(umodel->moveRow(row,by)){
    tvUserTics->setCurrentIndex(umodel->index(row+by,col,QModelIndex()));
    adjustTvUserTics();
  }
  emit dataChanged();
}


/*EOF*/ 
