/*************************************************************************
 * @(#)ViewBoxPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Apr 2011
 *************************************************************************
 */

#include <QColor>
#include "Attributes.hh"
#include "ViewBox.hh"
#include "ViewBoxDlg.hh"
#include "ViewBoxPropWdg.hh"
#include "Valid.hh"
#include "Config.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ViewBoxDlg::ViewBoxDlg(QWidget *parent,QWidget *base,const char *name)
  : DlgBase(parent,base,name)
{
  setWindowTitle(IDS_APP_NAME + ": ViewBox settings");

  if(wProps) delete wProps;
  wProps=new ViewBoxPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ViewBoxPropWdg::ViewBoxPropWdg(QWidget *parent)
  :PropWdgBase(parent)
{
  //  myDebug()<<"hier";
  setupUi(this);

  new RealValidator(this,leXPos,"Position X",1.,-1.);
  new RealValidator(this,leYPos,"Position Y",1.,-1.);

  new RealValidator(this,leWidth,"Width",1.,-1.);
  new RealValidator(this,leHeight,"Height",1.,-1.);

  connectChanged(this);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ViewBoxPropWdg::~ViewBoxPropWdg()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBoxPropWdg::check(QString &msg)
{
  ((IntValidator*)leXPos->validator())->check(msg);
  ((IntValidator*)leYPos->validator())->check(msg);
  ((IntValidator*)leWidth->validator())->check(msg);
  ((IntValidator*)leHeight->validator())->check(msg);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBoxPropWdg::setData( void *go )
{
  //myDebug()<<"ViewBoxPropWdg::setData"<<wd;
  ViewBox *vb=0;
  if(!go) PropWdgBase::reset(this);
  if(go) vb=dynamic_cast<ViewBox*>((GraphObject*)go);
  if(!vb) return;

  pvb=vb;

  mutecontentchanged=true;

  leXPos->setText(QString().setNum(vb->pos().x,'g',10));  
  leYPos->setText(QString().setNum(vb->pos().y,'g',10));  

  leWidth->setText(QString().setNum(vb->size().w,'g',10));  
  leHeight->setText(QString().setNum(vb->size().h,'g',10));  



  tbRimColor->setColor(vb->rim()->color());
  tbFillColor->setColor(vb->fill()->color());

  cbDrawFrame->setChecked(vb->drawFrame());
  cbFillFrame->setChecked(vb->fill()->fill());

  spbxLineWidth->setValue(vb->rim()->width());

  cmbLineStyle->clear();
  for(int i=0;i<Xfp::maxLineStyles();i++){
    cmbLineStyle->insertItem(i,Xfp::lineStyleGuiName((Xfp::LineStyles)i),i);
  }
  cmbLineStyle->setCurrentIndex(vb->rim()->style());  


  vb->setChanged(false);

  mutecontentchanged=false;
  setDirty(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBoxPropWdg::applyData(void *go)
{
  //NSF("applydata");
  ViewBox *vb=0;
  if(go) vb=dynamic_cast<ViewBox*>((GraphObject *)go);
  if(!vb) return;

  
  vb->rim()->setStyle((Xfp::LineStyles)cmbLineStyle->currentIndex());
  vb->rim()->setWidth(spbxLineWidth->value());
  vb->rim()->setColor(tbRimColor->color());

  
  vb->fill()->setFill(cbFillFrame->isChecked());
  vb->fill()->setColor(tbFillColor->color());

  QString t=leXPos->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leYPos->text();IntValidator::stripAllWhiteSpace(t2);
		   
  vb->setPos(Point(t.toDouble(),t2.toDouble()));

  t=leWidth->text();IntValidator::stripAllWhiteSpace(t);
  t2=leHeight->text();IntValidator::stripAllWhiteSpace(t2);
  vb->setSize(Size(t.toDouble(),t2.toDouble()));

  vb->setDrawFrame(cbDrawFrame->isChecked());
  
  
  setDirty(false);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBoxPropWdg::updateRequest(void *go)
{
  
  ViewBox *vb=0;
  if(go) vb=dynamic_cast<ViewBox*>((GraphObject *)go);
  if(!vb) return;
  if(pvb->graph()!=vb->graph()) return;
  bool di=isDirty();

  if(vb->updateRequest() & GraphObject::GOCPosition){
    leXPos->setText(QString().setNum(vb->pos().x,'g',10));  
    leYPos->setText(QString().setNum(vb->pos().y,'g',10));  
  }
  if(vb->updateRequest() & GraphObject::GOCSize){
    leWidth->setText(QString().setNum(vb->size().w,'g',10));  
    leHeight->setText(QString().setNum(vb->size().h,'g',10));  
  }
  
  if(!di) setDirty(false);

}

/*EOF*/ 
