/*************************************************************************
 * @(#)WeightSelModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   31 May 2011
 *************************************************************************
 */

#include "WeightSelModel.hh"
#include "WeightSelection.hh"
#include "MyMath.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
WeightSelModel::WeightSelModel(QObject * parent)
  : QAbstractListModel(parent),pwss(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelModel::reset(WeightSelections *wss)
{

  if(pwss)delete pwss;pwss=0;
  if(wss){
    pwss=new WeightSelections();
    pwss->copySelections(wss);
  }
  
  if(pwss) QAbstractListModel::reset();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant WeightSelModel::headerData(int section,Qt::Orientation orientation
		    ,int role)const
{
  Q_UNUSED(section);
  if (role != Qt::DisplayRole)
    return QVariant();
  if (orientation == Qt::Horizontal) {
    return "Header";
  }
  return QVariant();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
int WeightSelModel::rowCount(const QModelIndex & parent) const
{
  Q_UNUSED(parent);
  if(!pwss) return 0;
  return pwss->size();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Qt::ItemFlags WeightSelModel::flags(const QModelIndex &index)const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;
  return QAbstractListModel::flags(index) | Qt::ItemIsUserCheckable;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant WeightSelModel::
data(const QModelIndex &index,int role)const
{
  if(!pwss) return QVariant();
  if (!index.isValid()) return QVariant();
  int row=index.row();
  //int col=0;
  if (row >= pwss->size() || row < 0) return QVariant();

  const WeightSelection *ws=pwss->at(row);
  if (role == Qt::DisplayRole) {
    if(ws->weightLabel()){
      return ws->weightLabel()->label()();
    }else{
      return QVariant();
    }
  }else if (role == Qt::UserRole) {
    return QVariant::fromValue((void*)ws);
  }else if(role == Qt::CheckStateRole){
    return (ws->active()?Qt::Checked:Qt::Unchecked);
  }

  return QVariant();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool WeightSelModel::setData(const QModelIndex &index,const QVariant &value,
			     int role)
{
  Q_UNUSED(value);
  if(!pwss) return false;
  if (!index.isValid()) return false;
  int row=index.row();
  //int col=0;
  if (row >= pwss->size() || row < 0) return false;

  WeightSelection *ws=(*pwss)[row];
  if(role == Qt::CheckStateRole){
    if(ws){
      ws->setActive(value.toInt()==Qt::Checked);
      emit weightItemSelected(ws);
    
    
      emit(dataChanged(index, index));
      return true;
    }
    
    
  }
  return false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool WeightSelModel::insertRow(int position,WeightSelection *ws)
{
  beginInsertRows(QModelIndex(),position,position);
  pwss->insertNewWeightSelection(position,ws);
  endInsertRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
WeightSelection *WeightSelModel::removeRow(int position)
{
  WeightSelection *ws;
  beginRemoveRows(QModelIndex(), position, position);
  ws=pwss->removeAt(position);
  endRemoveRows();
  return ws;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool WeightSelModel::moveRow(int row,int by)
{
  if(!index(row+by,0,QModelIndex()).isValid()) return false;
  //myDebug() << "here valid";

 

  WeightSelection *ws=removeRow(row);
  insertRow(MyMath::max<long>(0,row+by),ws);

  return true;
 
}


/*EOF*/ 
