/*************************************************************************
 * @(#)WorldPropWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Apr 2011
 *************************************************************************
 */

#include "WorldDlg.hh"
#include "WorldPropWdg.hh"
#include "World.hh"
#include "Graph.hh"
#include "FString.hh"
#include "Valid.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
WorldDlg::WorldDlg(QWidget *parent,QWidget *base,const char *name)
  : DlgBase(parent,base,name)
{
  //myDebug()<<"hier WorldDlg"<<this;

  setWindowTitle(IDS_APP_NAME + ": World settings");

  if(wProps) delete wProps;
  wProps=new WorldPropWdg(this);
  verticalLayout->insertWidget(0,wProps,10);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
WorldPropWdg::WorldPropWdg(QWidget *parent)
  :PropWdgBase(parent)
{
  //  myDebug()<<"hier";
  setupUi(this);

  new RealValidator(this,leXMin,"X min",1.,-1.);
  new RealValidator(this,leXMax,"X max",1.,-1.);

  new RealValidator(this,leYMin,"Y min",1.,-1.);
  new RealValidator(this,leYMax,"Y max",1.,-1.);

  new RealValidator(this,leXOffset,"Offset x",1.,-1.);
  new RealValidator(this,leYOffset,"Offset y",1.,-1.);

  connectChanged(this);

  leXMin->setFocus();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
WorldPropWdg::~WorldPropWdg()
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::check(QString &msg)
{
  ((IntValidator*)leXMin->validator())->check(msg);
  ((IntValidator*)leXMax->validator())->check(msg);
  ((IntValidator*)leXOffset->validator())->check(msg);

  ((IntValidator*)leYMin->validator())->check(msg);
  ((IntValidator*)leYMax->validator())->check(msg);
  ((IntValidator*)leYOffset->validator())->check(msg);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::setData(void *go )
{
  //myDebug()<<"WorldPropWdg::setData"<<wd;
  World *wd=0;
  if(!go) PropWdgBase::reset(this);
  if(go) wd=dynamic_cast<World*>((GraphObject *)go);
  if(!wd) return;

  mutecontentchanged=true;

  setX(wd);
  setY(wd);

  leXOffset
    ->setText(QString().setNum(wd->autoScaleOffset(Xfp::Graph_X),'g',10));
  leYOffset
    ->setText(QString().setNum(wd->autoScaleOffset(Xfp::Graph_Y),'g',10));


  wd->setChanged(false);

  mutecontentchanged=false;
  setDirty(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::applyData(void *go)
{
  //NSF("applydata");
  World *wd=0;
  if(go) wd=dynamic_cast<World*>((GraphObject *)go);

  if(!wd) return;
  bool adjusted0,adjusted1;
  
  QString t=leXMin->text();IntValidator::stripAllWhiteSpace(t);
  QString t2=leXMax->text();IntValidator::stripAllWhiteSpace(t2);
  wd->setW01(Xfp::Graph_X,t.toDouble(),t2.toDouble(),adjusted0,adjusted1);
  setX(wd,adjusted0,adjusted1);

  t=leYMin->text();IntValidator::stripAllWhiteSpace(t);
  t2=leYMax->text();IntValidator::stripAllWhiteSpace(t2);
  wd->setW01(Xfp::Graph_Y,t.toDouble(),t2.toDouble(),adjusted0,adjusted1);
  setY(wd,adjusted0,adjusted1);


  t=leXOffset->text();IntValidator::stripAllWhiteSpace(t);
  wd->setAutoScaleOffset(Xfp::Graph_X,t.toDouble());
  t=leYOffset->text();IntValidator::stripAllWhiteSpace(t);
  wd->setAutoScaleOffset(Xfp::Graph_Y,t.toDouble());
  
  setDirty(false);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::updateRequest(void *go)
{
  World *wd=0;
  if(go) wd=dynamic_cast<World*>((GraphObject *)go);
  if(!wd) return;
  bool di=isDirty();
  
  if(wd->updateRequest() & GraphObject::GOCX){
    setX(wd);
  }
  if(wd->updateRequest() & GraphObject::GOCY){
    setY(wd);
  }
  if(!di) setDirty(false);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::setX(const World *wd,bool set0,bool set1)
{
  if(set0){
    leXMin->setText(QString().setNum(wd->w0(Xfp::Graph_X),'g',10));  
    leXMin->setCursorPosition(0);
  }
  if(set1){
    leXMax->setText(QString().setNum(wd->w1(Xfp::Graph_X),'g',10));  
    leXMax->setCursorPosition(0);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WorldPropWdg::setY(const World *wd,bool set0,bool set1)
{
  if(set0){
    leYMin->setText(QString().setNum(wd->w0(Xfp::Graph_Y),'g',10));  
    leYMin->setCursorPosition(0);
  }
  if(set1){
    leYMax->setText(QString().setNum(wd->w1(Xfp::Graph_Y),'g',10));  
    leYMax->setCursorPosition(0);
  }

}

/*EOF*/ 
