/*************************************************************************
 * @(#)XNYDlgControl.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 May 2011
 *************************************************************************
 */

#include <QColor>
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "Graph.hh"
#include "Graphs.hh"
#include "Page.hh"
#include "IOControl.hh"
#include "XNYDlgControl.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
XNYDlgControl::XNYDlgControl(QWidget *parent)
  :QWidget(parent),piofc(0)
{
  setupUi(this);

  cmbAutoScale->clear();
  for(int i=0;i<IOControl::autoScaleMaxTypes();i++){
    cmbAutoScale->insertItem(i, IOControl::autoScaleGuiName(i) ,i);
  }

  cmbDataType->clear();
  for(int i=0;i<Function::numOfDataTypes();i++){
    cmbDataType->insertItem(i,Function::dataTypeGuiName((Function::DataType)i),i);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XNYDlgControl::on_cmbGraph_currentIndexChanged(int cindex)
{
  long ID=cmbGraph->itemData(cindex).toInt();
  myDebug()<<"ID"<<ID;
  cmbGroup->clear();
  if(ID==Graph::NewGraph || cindex<0){
    cmbGroup->insertItem( 0,"New group" ,-1 );
    cmbGroup->setCurrentIndex(0);
    return ;
  }
  // current graph
  const Graph *gr=ppage->graphs()->currentGraph();
  myDebug()<<"current"<<gr->guiName(true);
  if(ID!=Graph::CurrentGraph){
    gr=((Page*)ppage)->graphs()->getExistingGraphOrNew1(ID);
  }
  myDebug()<<"gr"<<gr->guiName(true);

  long ii=0;
  cmbGroup->insertItem( ii++,"New group" ,-1 );
  if(gr){
    myDebug()<<"gr->groups()->size()"<<gr->groups()->size();
    for(int i=0;i<gr->groups()->size();i++){
      const FunctionGroup *fg=gr->groups()->at(i);
      cmbGroup->insertItem( ii++,fg->guiName(true)(),(int)fg->ID());
    }
  }
  int index=cmbGroup->findData((int)piofc->targetGroupId());
  if(index<0) index=0;

  cmbGroup->setCurrentIndex(index);

  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
XNYDlgControl::~XNYDlgControl()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XNYDlgControl::setData(IOControl *iofc,const Page *page)
{
  ppage=page;
  piofc=iofc;

  cmbGraph->clear();
  int ii=0;
  if(ppage->graphs()->size()>0)
    cmbGraph->insertItem( ii++,Graph::graphIDGuiName(Graph::CurrentGraph)
			  ,Graph::CurrentGraph);
  cmbGraph->insertItem( ii++,Graph::graphIDGuiName(Graph::NewGraph)
			,Graph::NewGraph);
  for(int i=0;i<ppage->graphs()->size();i++){
    const Graph *gr=ppage->graphs()->at(i);
    cmbGraph->insertItem( ii++,gr->guiName(true)() ,(int)gr->ID());
  }
  
  int index=cmbGraph->findData((int)iofc->targetGraphId());

  if(index<0) index=0;
  //if(ppage->graphs()->size()==0) index=1;
  cmbGraph->setCurrentIndex(index);

  cmbAutoScale->setCurrentIndex(iofc->autoScale());
  cmbDataType->setCurrentIndex(iofc->dataType());
  
  cbClearGroup->setChecked(iofc->clearGroup());
  cbAutoColor->setChecked(iofc->autoColor());
  cbAutoSymbol->setChecked(iofc->autoSymbol());

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
IOControl *XNYDlgControl::getData(  )
{
  if(!piofc) return 0;
  piofc->setAutoScale((IOControl::AutoScale)cmbAutoScale->currentIndex());

  piofc->setDataType((Function::DataType)cmbDataType->currentIndex());
  
  piofc->setClearGroup(cbClearGroup->isChecked());
  piofc->setAutoColor(cbAutoColor->isChecked());
  piofc->setAutoSymbol(cbAutoSymbol->isChecked());

  piofc->setTargetGroupId(cmbGroup->itemData(cmbGroup->currentIndex()).toInt());

  
  piofc->setTargetGraphId(cmbGraph->itemData(cmbGraph->currentIndex()).toInt());

  return piofc;
}


/*EOF*/ 
