/* @(#)Attributes.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ATTRIBUTES_H
#define _ATTRIBUTES_H 1

#include "FString.hh"
#include "MyQRgb.hh"
#include "Xfp.hh"


/* ******************************
 * font style 
 * ******************************
 */

class FontStyle{
 public:
  FontStyle():pchanged(false),psize(12.),psubscriptscale(0.75),pcolor(0x00){};

  void reset(){
    psize=12.;
    psubscriptscale=0.75;
    pcolor=0x0;
    pchanged=false;
  };

  void setSize(double size){pchanged|=(psize!=size);psize=size;};
  double size()const{return psize;};

  double subScriptScale()const{return psubscriptscale;};
  void setSubScriptScale(double s){pchanged|=(psubscriptscale!=s);
    psubscriptscale=s;};

  QRgb color()const{return pcolor;};
  void setColor(QRgb c){pchanged|=(pcolor!=c);pcolor=c;};

  String asCommands(const String &prefix)const{
    return Sprintf("%s size %g\n"
		   "%s subscriptscale %g\n"
		   "%s color 0x%x\n"
		   ,prefix(),psize
		   ,prefix(),psubscriptscale
		   ,prefix(),pcolor
		   );};

  bool changed()const{return pchanged;};
  void setChanged(bool c){pchanged=c;};

 private:
  bool pchanged;
  double psize;
  double psubscriptscale;
  QRgb pcolor;
};

/* ******************************
 * line style 
 * ******************************
 */
class LineStyle{
 public:
  LineStyle():pchanged(false),pstyle(Xfp::LSTSolid),pwidth(1),pcolor(0x00){};
  
  void reset(){
    pstyle=Xfp::LSTSolid;
    pwidth=1;
    pcolor=0x0;
    pchanged=false;
  };
  
  Xfp::LineStyles style()const{return pstyle;};
  void setStyle(Xfp::LineStyles s1){
    int ss=s1;
    while(ss<0) ss+=Xfp::maxLineStyles();
    ss=ss%(Xfp::maxLineStyles());
    pchanged|=(pstyle!=ss);pstyle=(Xfp::LineStyles)ss;}
  
  QRgb color()const{return pcolor;};
  void setColor(QRgb c){pchanged|=(pcolor!=c);pcolor=c;};
  
  double width()const{return pwidth;};
  void setWidth(double w){pchanged|=(pwidth!=w);pwidth=w;};

  String asCommands(const String &prefix,bool pextralinecolor=true)const{
    if(pextralinecolor){
      return Sprintf("%s style %s\n%s width %g\n%s color 0x%x\n"
		     ,prefix(),Xfp::lineStyleParserName(pstyle)
		     ,prefix(),pwidth,prefix(),pcolor);
    }else{
      return Sprintf("%s style %s\n%s width %g\n"
		     ,prefix(),Xfp::lineStyleParserName(pstyle),prefix(),pwidth);
      
    }
  };
  
  bool changed()const{return pchanged;};
  void setChanged(bool c){pchanged=c;};
  
 private:
  bool pchanged;
  Xfp::LineStyles pstyle;
  double pwidth;
  QRgb pcolor;
};


/* ******************************
 * fill style 
 * ******************************
 */

class FillStyle{
 public:
  FillStyle():pchanged(false),pfill(false),pcolor(0xffffff){};

  void reset(){
    pfill=false;
    pcolor=0xffffff;
    pchanged=false;
  };

  bool fill()const{return pfill;};
  void setFill(bool f){pchanged|=(pfill!=f);pfill=f;};

  QRgb color()const{return pcolor;};
  void setColor(QRgb c){pchanged|=(pcolor!=c);pcolor=c;};

  String asCommands(const String &prefix,bool pextrafillcolor=true)const{
    if(pextrafillcolor){
      return Sprintf("%s %s\n%s color 0x%x\n"
		     ,prefix(),Xfp::OnOff(pfill),prefix(),pcolor);
    }else{
      return Sprintf("%s %s\n",prefix(),Xfp::OnOff(pfill));
    }
  };

  bool changed()const{return pchanged;};
  void setChanged(bool c){pchanged=c;};

 private:
  bool pchanged;
  bool pfill;
  QRgb pcolor;
};

/* ******************************
 * symbol style 
 * ******************************
 */

class SymbolStyle{
 public:
  SymbolStyle():pchanged(false),ptype(Xfp::STYNone),psize(12.)
    ,pextralinecolor(false),pextrafillcolor(false),pchar('A'){
  };

  void reset(){
    ptype=Xfp::STYNone;
    psize=12.;
    plinestyle.reset();
    pfillstyle.reset();
    pextralinecolor=false;
    pextrafillcolor=false;
    pchar='A';
    pchanged=false;
  };

  Xfp::SymbolTypes type()const{return ptype;};
  void setType(Xfp::SymbolTypes t1){
    int tt=t1;
    while(tt<0) tt+=Xfp::maxSymbols();
    tt=tt%(Xfp::maxSymbols());
    pchanged|=(ptype!=tt);ptype=(Xfp::SymbolTypes)tt;
  };

  void setSize(double size){pchanged|=(psize!=size);psize=size;};
  double size()const{return psize;};

  const LineStyle &lineStyle()const{return plinestyle;};
  LineStyle &lineStyle(){return plinestyle;};
  const struct FillStyle &fillStyle()const{return pfillstyle;};
  struct FillStyle &fillStyle(){return pfillstyle;};

  bool extraLineColor()const{return pextralinecolor;};
  void setExtraLineColor(bool e){pchanged|=(pextralinecolor!=e);
    pextralinecolor=e;};
  
  void setExtraFillColor(bool e){pchanged|=(pextrafillcolor!=e);
    pextrafillcolor=e;};
  bool extraFillColor()const{return pextrafillcolor;};

  char userchar()const{return pchar;};
  void setUserChar(char c){pchanged|=(pchar!=c);pchar=c;};

  String asCommands(const String &tab)const;

  bool changed()const{return pchanged|plinestyle.changed()|
      pfillstyle.changed();};
  void setChanged(bool c){
    pchanged=c;
    plinestyle.setChanged(c);
    pfillstyle.setChanged(c);
  };


 private:
  bool pchanged;
  Xfp::SymbolTypes ptype;
  double psize;
  LineStyle plinestyle;
  FillStyle pfillstyle;
  bool pextralinecolor;		/* if true, the plinestyle.color will be used 
				   for the symbol lines */
  bool pextrafillcolor;		/* if true, the fs.color will be used 
				   for the symbol fill */
  char  pchar;
};

/* ***********************************************************************
 *  attribute class
 * ***********************************************************************
 */
class FunctionAttribs{
 public:
  FunctionAttribs():pchanged(false),plegend(""),
    pinterpolationdepth(0),pzmax(1),pusesecondderivative(false),pzpower(1),
    puselegend(true){
  }

  QRgb symbolLineColor()const{
    return (psymbolstyle.extraLineColor()?
	    psymbolstyle.lineStyle().color():
	    plinestyle.color());
  };

  QRgb symbolFillColor()const{
    return (psymbolstyle.extraFillColor()?
	    psymbolstyle.fillStyle().color():
	    plinestyle.color());
  };
  
  inline const LineStyle &lineStyle()const{return plinestyle;};
  inline LineStyle &lineStyle(){return plinestyle;};

  inline const SymbolStyle &symbolStyle()const{return psymbolstyle;};
  inline SymbolStyle &symbolStyle(){return psymbolstyle;};


  inline const String &legend()const{return plegend;};
  inline void setLegend(const String &l){pchanged|=(plegend!=l);plegend=l;};
  inline void setLegend(const String &l,bool u){
    setLegend(l);setUseLegend(u);};

  inline bool useLegend()const{return puselegend;};
  inline void setUseLegend(bool u){pchanged|=(puselegend!=u);puselegend=u;};

  inline double zmax()const{return pzmax;};
  inline void setZMax(double z){pchanged|=(pzmax!=z);pzmax=z;};

  inline long zPower()const{return pzpower;};
  inline void setZPower(long p){pchanged|=(pzpower!=p);pzpower=p;};

  inline bool useSecondDerivative()const{return pusesecondderivative;};
  inline void setUseSecondDerivative(bool s){
    pchanged|=(pusesecondderivative!=s);pusesecondderivative=s;};

  inline long interpolationDepth()const{return pinterpolationdepth;};
  inline void setInterpolationDepth(long d){
    pchanged|=(pinterpolationdepth!=d);pinterpolationdepth=d;};

  

  String asCommands(const String &tab)const;

  bool changed()const{
    return pchanged|psymbolstyle.changed()|plinestyle.changed();
  };
  void setChanged(bool c){
    pchanged=c;
    psymbolstyle.setChanged(c);
    plinestyle.setChanged(c);
  };

 private:
  bool pchanged;
  LineStyle plinestyle;
  SymbolStyle psymbolstyle;
  String plegend;
  long pinterpolationdepth;//todo: this is temporary fix
  double pzmax;//todo: this is temporary fix
  bool pusesecondderivative;//todo: this is temporary fix
  long pzpower;//todo: this is temporary fix
  bool puselegend;
};

#ifdef	__cplusplus
extern "C" {
#endif


#ifdef	__cplusplus
}
#endif


#endif /* _ATTRIBUTES_H */

