/* @(#)BSpline.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _BSPLINE_H
#define _BSPLINE_H 1


#include "ValueArray.hh"

class String;

class BSpline{
 public:
  BSpline(long degree,const ValueArray<double> &x);
  BSpline(long degree,const double *x,long Nx);
  ~BSpline(){};

  void splint(const ValueArray<double> &y);
  void splint(const double *y);

  void spline(const ValueArray<double> &xout,
	      ValueArray<double> &yout,long jderiv);
  void spline(const ValueArray<double> &xout,
	      double *yout,long jderiv);

  double value(double at,long jderiv){return bvalue(pbcoeff,at,jderiv);};
  
  void writeBasisFunctions(const String &,
				  const ValueArray<double> &xout);

 private:
  void setKnots();
  void bsplint();
  void bsplvb(long jhigh,long index,double at, long left,
	      ValueArray<double> &biatx );
  bool banfac(ValueArray<double> &w,long N,
	      long  nbandl,long  nbandu);
  void banslv(ValueArray<double> &w,long N,
	      long nbandl,long nbandu,ValueArray<double> &b);
  double bvalue(const ValueArray<double> &bcoef,double at,long jderiv);
  int interv(double at,long &left);


 private:
  long pdegree,porder;       // porder ->k pdegree->p
  ValueArray<double> px;     // real point
  ValueArray<double> pknots; // knot sequence
  //  array of size  (2*k-1)*n , containing the triangular factoriz-
  //       ation of the coefficient matrix of the linear system for the b-
  //       coefficients of the spline interpolant.
  //       the b-coeffs for the interpolant of an additional data set can
  //       be obtained without going through all the calculations in this
  //       routine, simply by loading  y  into  bcoef  and then execut-
  //       ing the    call banslv ( q, 2*k-1, n, k-1, k-1, bcoef )
  ValueArray<double> pq;     // band matrix
  // the b-coefficients of the interpolant, of length  n.
  ValueArray<double> pbcoeff;
  // saved state for bsplvb;
  long pj,pilo;
  ValueArray<double> pdeltal;
  ValueArray<double> pdeltar;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _BSPLINE_H */

