/* @(#)EllipseShape.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ELLIPSESHAPE_H
#define _ELLIPSESHAPE_H 1

#include "Attributes.hh"
#include "Shape.hh"

class LineStyle;
class FillStyle;

class EllipseShape: public Shape
{
 public:
  EllipseShape(Graph *parent,long index);
  virtual ~EllipseShape();

  virtual void reset();

  const Point &center()const{return pcenter;};
  void setCenter(const Point &c){ pchanged|=(pcenter!=c);pcenter=c;};

  const Size &radii()const{return pradii;};
  void setRadii(const Size &r){ pchanged|=(pradii!=r);pradii=abs(r);};

  double angle()const{return pangle;};
  void setAngle(double a){pchanged|=(pangle!=a);pangle=a;};

  Point centerToDC(bool dragging=false)const{
    if(dragging) return posToDC(pcenter,pcoordsystem)+pobjectpos;
    else return posToDC(pcenter,pcoordsystem);
  }
  Size radiiToDC(bool dragging=false)const{
    if(dragging) return abs(abs(sizeToDC(pradii,pcoordsystem))+pdeltasize);
    else return abs(sizeToDC(pradii,pcoordsystem));
  }

  LineStyle *lineStyle(){return plinestyle;};
  const LineStyle *lineStyle()const{return plinestyle;};

  FillStyle *fill(){return pfill;};
  const FillStyle *fill()const{return pfill;};

  QRgb fillColor()const{
    return (pextrafillcolor?pfill->color():plinestyle->color());
  };

  void setExtraFillColor(bool e){pchanged|=(pextrafillcolor!=e);
    pextrafillcolor=e;};
  bool extraFillColor()const{return pextrafillcolor;};


  bool isCircle()const{return pisCircle;};
  void setIsCircle(bool i){pchanged|=(pisCircle!=i);pisCircle=i;};

  virtual String asCommands(const String &tab)const;
  

  bool changed()const;
  void setChanged(bool c);

  
  virtual GraphObjectRegion insideRegion(const DragDescriptor &dd,bool update);
  virtual void initDrag(const DragDescriptor &dd);
  virtual void updateDrag(const DragDescriptor &dd);
  virtual void setFromDrag(DragDescriptor &dd);

 private:
  double pangle;
  Point pcenter; 
  Size pradii;
  LineStyle *plinestyle;
  FillStyle *pfill;
  bool pextrafillcolor;
  bool pisCircle;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _ELLIPSESHAPE_H */

