/*************************************************************************
 * @(#)FGObject.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   19 Jul 2012
 *************************************************************************
 */

#include "FGObject.hh"
#include "Attributes.hh"
#include "WeightSelection.hh"
#include "OptMatelDefinition.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FGObject::FGObject(Type t,long uniqueid)
  :GraphObject(0),
   pattribs(new FunctionAttribs()),
   pactive(true),
   ptype(t),puniqueid(uniqueid),
   pweightselections(new WeightSelections()),
   poptmateldefinition(0)

{
  setObjectType(GraphObject::GOTFunctionOrGroup);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FGObject::~FGObject()
{
  //myDebug()<<"FGObject dies"<<this;
  if(pattribs) delete pattribs;pattribs=0;

  
  if(pweightselections) delete pweightselections;pweightselections=0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String FGObject::guiName(bool longname)const
{
  return Sprintf("%s%ld",(ptype==FGOTFunction?(longname?"Set":"s"):
			  (longname?"Group":"gr")),puniqueid+1);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FGObject::changed()const
{
  bool c=pchanged||pattribs->changed()||pweightselections->changed();
  
  if(poptmateldefinition) c|=poptmateldefinition->changed();
  return c;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FGObject::setChanged(bool c)
{
  pchanged=c;
  pattribs->setChanged(c);
  pweightselections->setChanged(c);
  if(poptmateldefinition) poptmateldefinition->setChanged(c);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FGObject::internalInitializeWeightSelections()
{
  //  myDebug() << "FGObject::internalInitializeWeightSelections()"
  //	   <<weightLabels();
  if(weightLabels()){
    // TODO: remove && ! pweightselections->isInitialized()???
    //printf("weightlabels %p\n",weightLabels());
    pweightselections->updateFrom(weightLabels());
  }
}



/*EOF*/ 
