/* @(#)FGObject.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _FGOBJECT_HH
#define _FGOBJECT_HH 1

#include "GraphObject.hh"

class FunctionGroups;
class FunctionAttribs;
class WeightLabels;
class WeightSelections;
class OptMatelDefinition;

/* ***********************************************************************
 * base class for groups (class FunctionGroup) und sets (class Function)
 * ***********************************************************************
 */
class FGObject:public GraphObject{
 public:
  enum DataType{XY=0,XYNW=1,GRIDX_Y_NZ=2,XYMatel=3};
  enum Type{FGOTGroup=0,FGOTFunction=1};

  FGObject(Type t,long uniqueid);
  virtual ~FGObject();
  FGObject();//to avoid calling it we do not implement it

  Type type()const{return ptype;};

  long ID()const{return puniqueid;};

  String guiName(bool longname=false)const;

  const FunctionAttribs *attribs()const {return pattribs;};
  FunctionAttribs *attribs(){return pattribs;};

  bool active()const{return pactive;};
  void setActive(bool a){pchanged|=(pactive!=a);pactive=a;};

  inline const String &comment()const{return pcomment;};
  inline void setComment(const String &s){pchanged|=(pcomment!=s);
    pcomment=s;};

  WeightLabels *weightLabels(){ 
    return (WeightLabels *)weightLabelsInternalConst(); };
  const WeightLabels *weightLabels()const{return weightLabelsInternalConst();};


  WeightSelections *weightSelections(){internalInitializeWeightSelections();
    return pweightselections;};

  OptMatelDefinition *optMatelDefinition(){return poptmateldefinition;};
  void setOptMatelDefinition(OptMatelDefinition *o){poptmateldefinition=o;};

  bool changed()const;
  void setChanged(bool);


 protected:
  virtual const WeightLabels *weightLabelsInternalConst()const=0;
  void internalInitializeWeightSelections();

 protected:
  FunctionAttribs *pattribs;
  bool pactive; 
  String pcomment;
 private:
  Type ptype;
 protected:
  long puniqueid;
  WeightSelections *pweightselections;
  OptMatelDefinition *poptmateldefinition;
  friend class FunctionGroups;
};


#endif /* _FGOBJECT_HH */

