/*************************************************************************
 * @(#)Function.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Apr 2011
 *************************************************************************
 */

#include <stdlib.h>
#include "Attributes.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "BSpline.hh"
#include "FunctionGroup.hh"
#include "Graph.hh"
#include "GraphAxis.hh"
#include "OptMatelDefinition.hh"
#include "Page.hh"
#include "Range.hh"
#include "MyMath.hh"
#include "defs.hh"
#include "FString.hh"
#include "WeightLabels.hh"
#include "WeightSelection.hh"



const Function::tsDataType Function::pdatatypes[]={
  {Function::XY,"XY"}
  ,{Function::XYNW,"XYNW"}
  ,{Function::GRIDX_Y_NZ,"Grid X:Y:NZ"}
  ,{Function::XYMatel,"XYMatel"}
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const long Function::pdatamaxtypes=sizeof(Function::pdatatypes)
  /sizeof(Function::tsDataType);

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function::DataType Function::dataTypeFromString(const  char *s)
{
  long i;
  for(i=0;i<pdatamaxtypes;i++){
    if(strcmp(pdatatypes[i].guiname,s)==0){
      return pdatatypes[i].type;
    }
  }
  return Function::XY;
}




/* ***********************************************************************
 * 
 * ***********************************************************************
 */
Function::Function()
  :FGObject(FGObject::FGOTFunction,-1),
   pweightlabels(0),
   pgroup(0),
   pweightlabelreference(-1),
   pcurrentinterpolationdepth(0),
   pcurrentinterpolcomp(0),
   pcurrentusesecondderivative(false)
{
  pdatatype=XY;  
}
/* ***********************************************************************
 * 
 * ***********************************************************************
 */
Function::Function(FunctionGroup *fg,long uniqueid)
  :FGObject(FGObject::FGOTFunction,uniqueid),
   pweightlabels(0),
   pgroup(fg),
   pweightlabelreference(-1),
   pcurrentinterpolationdepth(0),
   pcurrentinterpolcomp(0),
   pcurrentusesecondderivative(false)
{
  NSF("Function::Function");

  
  pdatatype=XY;  
  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function::~Function()
{
//   myDebug()<<"Function dies"<<this;
//   myDebug()<<"Function group"<<pgroup;
//   myDebug()<<"Function graph"<<pgroup->parent();
//   myDebug()<<"Function page"<<pgroup->parent()->parent();
//   myDebug()<<"Function wlc"<<pgroup->parent()->parent()->weightLabelsCollector();

  WeightLabelsCollector*wlc=pgroup->graph()->page()->
    weightLabelsCollector();
  if(wlc)
    wlc->remove(pweightlabels);

  clean(); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::reParent(FunctionGroup *p)
{
  pgroup=p;
  pgraph=pgroup->graph();
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::copyFrom(const Function *src)
{
  //FGObject:
  pchanged=true;
  pactive=true;
  pcomment=src->pcomment;
  pweightselections->copySelections(src->pweightselections);
  poptmateldefinition=src->poptmateldefinition;
  //todo if option width args  pattribs=src->pattribs

  //Function:
  pdatatype=src->pdatatype;

  parray=src->parray;
  pxarray=src->pxarray;
  pyarray=src->pyarray;
  pweightlabels=src->pweightlabels;
  pweightlabelreference=src->pweightlabelreference; //todo: check logic
  prange=src->prange;
  pcurrentinterpolationdepth=0;
  pcurrentinterpolcomp=0;
  pcurrentusesecondderivative=false;


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setWeightLabels(WeightLabels *wl)
{
  wl->increaseUsage();
  pweightlabels=wl;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setWeightLabelByReference()
{
  //myDebug()<<"\nsetWeightLabelByReference";
  
  WeightLabelsCollector *wlc=pgroup->
    graph()->page()->weightLabelsCollector();
  
  //myDebug()<<"wlc"<<wlc;
  //myDebug()<<"pweightlabelreference"<<pweightlabelreference;
  //myDebug()<<"pweightlabels"<<pweightlabels;
  
  pweightselections->setDirty(true);
  pgroup->pweightselections->setDirty(true);

  if(pweightlabelreference>=0){
    WeightLabels *wls=wlc->getByUniqueIndex(pweightlabelreference);
    //myDebug()<<"wls"<<wls;
    if(wls){
      if(pweightlabels && pweightlabels!=wls){
	//myDebug()<<"remove pweightlabels"<<pweightlabels;
	pweightlabels->decreaseUsage();
      }
      setWeightLabels(wls);
      pweightlabelreference=-1;
      
    }
  }

  return;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */

const WeightLabels *Function::weightLabelsInternalConst()const
{ 
  if(pdatatype==XYNW && pweightlabels){
    return pweightlabels;
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setUpDefaultWeightLabels()
{
  if(pdatatype==XYNW){
    if(!pweightlabels){
      WeightLabelsCollector *wlc=pgroup->graph()->page()->
	weightLabelsCollector();

      long nw=components()-2;
      
      WeightLabels *wls=wlc->demandNewAuto();
      
      //myDebug()<<"setUp"<<nw<<wls;
      for(long ic=0;ic<nw;ic++){
	wls->append(Sprintf("www %ld",ic+1));
      }
      setWeightLabels(wls);
    }
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setData(long ic,long i,double d)
{
  //NSF("setData_func");
  //myDebug()<<nsubfunc;

  if(pdatatype==XY || 
     pdatatype==XYNW || 
     pdatatype==XYMatel || 
     pdatatype==GRIDX_Y_NZ){
    parray.adjustLength(i+1);
    parray.adjustComponents(ic+1);
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

    (*parray[ic])[i]=d;
    return ;
  }
  
  return ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::shiftOrigin(Xfp::XYAxisType xy,double o)
{
  if(pdatatype==XY || 
     pdatatype==XYNW || 
     pdatatype==XYMatel ){
    if(xy==Xfp::Graph_X){
      for(long i=0;i<parray[0]->size();i++){
	(*parray[0])[i]-=o;
      }
    }else{
      for(long i=0;i<parray[1]->size();i++){
	(*parray[1])[i]-=o;
      }
    }
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

  }
  pchanged=true;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::scaleData(Xfp::XYAxisType xy,double s)
{
  if(pdatatype==XY || 
     pdatatype==XYNW || 
     pdatatype==XYMatel ){
    if(xy==Xfp::Graph_X){
      for(long i=0;i<parray[0]->size();i++){
	(*parray[0])[i]*=s;
      }
    }else{
      for(long i=0;i<parray[1]->size();i++){
	(*parray[1])[i]*=s;
      }
    }
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

  }
  pchanged=true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setData(const ValueArray<double> &x,const ValueArray<double> &y)
{
  if(pdatatype==XY){
    parray.adjustLength(x.size());
    parray.adjustComponents(2);
    //myDebug()<< parray.length();
    (*parray[0])=x;
    (*parray[1])=y;
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

    //print();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::appendXYData(Xfp::XYAxisType xy,const ValueArray<double> &x)
{
  if(pdatatype==GRIDX_Y_NZ){
    if(xy==Xfp::Graph_X){
      pxarray.append(x);
    }else{
      pyarray.append(x);
    }
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setDataComponents(long i,double *data,long ndata)
{
  if(pdatatype==XY || 
     pdatatype==XYNW || 
     pdatatype==XYMatel || 
     pdatatype==GRIDX_Y_NZ){
    parray.adjustLength(i+1);
    parray.adjustComponents(ndata);
    for(long ic=0;ic<ndata;ic++){
      (*parray[ic])[i]=data[ic];
    }
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

    return ;
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::clean()
{
  NSF("clean_func"); 
  
  parray.adjustComponents(0,true);
  parray.adjustLength(0,true);
  prange.invalidate();
  pcurrentinterpolationdepth=0;
  pcurrentinterpolcomp=0;
  pcurrentusesecondderivative=false;


  adjustXYLength(Xfp::Graph_X,0,true); 
  adjustXYLength(Xfp::Graph_Y,0,true); 
 
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::setDataXY(Xfp::XYAxisType xy,long i,double d)
{
  /* NSF("setData_func"); */

  if(pdatatype==GRIDX_Y_NZ){
    if(xy==Xfp::Graph_X){
      adjustXYLength(Xfp::Graph_X,i+1);
      pxarray[i]=d;
    }else{
      adjustXYLength(Xfp::Graph_Y,i+1);
      pyarray[i]=d;
    }
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

  }
   
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::adjustXYLength(Xfp::XYAxisType xy,long size,bool shrink)
{
  NSF("adjust_xyarray_func");

  if(xy==Xfp::Graph_X){
    pxarray.resize(size,shrink);
  }else{
    pyarray.resize(size,shrink);
  }
  pcurrentinterpolationdepth=0;
  pcurrentinterpolcomp=0;
  pcurrentusesecondderivative=false;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Function::gridIsEquidistant(double tol)const
{
  if(pdatatype!=GRIDX_Y_NZ) return false;
  if(pxarray.size()<1)  return false;
  if(pyarray.size()<1)  return false;
  
  if(pxarray.size()>2){
    double x0=pxarray.at(0);
    double x1=pxarray.at(1);
    double h=x1-x0;
    x0=x1;
    long N=pxarray.size();
    for(long i=2;i<N;i++){
      x1=pxarray.at(i);
      double dx=x1-x0;
      if(fabs(dx-h)>h*tol) {
	return false;
      }
      x0=x1;
    }
  }

  if(pyarray.size()>2){
    double y0=pyarray.at(0);
    double y1=pyarray.at(1);
    double h=y1-y0;
    y0=y1;
    long N=pyarray.size();
    for(long i=2;i<N;i++){
      y1=pyarray.at(i);
      double dy=y1-y0;
      if(fabs(dy-h)>h*tol) {
	return false;
      }
      y0=y1;
    }
  }

  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::calcRange(Range &range,long restrictto
			 ,double omin,double omax)
{
  
  range=calcFuncRange();

  if(restrictto<Xfp::Graph_X) return;
    
  

  if(pdatatype != GRIDX_Y_NZ){
    if(parray.components()<2) return;
    if(parray.length()==0) return;


    Xfp::XYAxisType othertype=Xfp::otherAxisType(restrictto);


    range.setNew(othertype);

    for(long i=0;i<parray.length();i++){
      if((omin<=parray[restrictto]->at(i)) && 
	 (parray[restrictto]->at(i)<=omax)){
	range.extend(othertype,parray[othertype]->at(i));
      }
    }
  }
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Range Function::calcFuncRange()
{
  if(prange.isValid()) return prange;
  

  prange=Range();
  
  if( pdatatype == GRIDX_Y_NZ){
    if(pxarray.size()==0) return prange;
    if(pyarray.size()==0) return prange;
    
    for(long i=0;i<pxarray.size();i++){
      prange.extend(Xfp::Graph_X,pxarray.at(i));
    }
    
    for(long i=0;i<pyarray.size();i++){
      prange.extend(Xfp::Graph_Y,pyarray.at(i));
    }
  }else{
    
    if(parray.components()<2) return prange;
    if(parray.length()==0) return prange;
    
    for(long xy=Xfp::Graph_X;xy<=Xfp::Graph_Y;xy++){
      for(long i=0;i<parray.length();i++){
	prange.extend((Xfp::XYAxisType)xy,parray[xy]->at(i));
      }
    }
  }
  return prange;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Function::isInRange(const Point &w0,const Point &w1)
{
  calcFuncRange();

  double xmin=prange.min(Xfp::Graph_X);
  double xmax=prange.max(Xfp::Graph_X);
  double ymin=prange.min(Xfp::Graph_Y);
  double ymax=prange.max(Xfp::Graph_Y);

  if(xmin>w1.x) return false;
  if(xmax<w0.x) return false;
  if(ymin>w1.y) return false;
  if(ymax<w0.y) return false;
  
  return true;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Function::isCloseToPoint(const Point &w,const Size &sensitivespot,
			      double &dist,const GraphAxes *axes)
{

  // input w is in current scale (lin/log)
  
  if(pdatatype!=Function::GRIDX_Y_NZ){
    if( (!pactive) || (parray.length()==0) || (parray.components()<2)) 
      return false;

    double r=sensitivespot.w/sensitivespot.h;


    if(parray.length()==1){
      Point pi(parray[0]->at(0),parray[1]->at(0));
      if(!axes->allow(pi)) return false;
      pi=axes->toScaled(pi);
      dist=norm((w-pi)*Size(1,r));
      return (dist<=sensitivespot.w);
    }


    double sw=sensitivespot.w;
    Point ww=w*Size(1,r);
    for(long i=0;i<parray.length()-1;i++){
      Point wi(parray[0]->at(i),parray[1]->at(i));
      if(!axes->allow(wi)) continue;
      wi=axes->toScaled(wi);
      Point pi=Point(wi.x,wi.y*r);
      wi=Point(parray[0]->at(i+1),parray[1]->at(i+1));
      if(!axes->allow(wi)) continue;
      wi=axes->toScaled(wi);
      Point pip1=Point(wi.x,wi.y*r);
      
      Point nc=(pip1-pi)*0.5;
      Point pw=ww-pi-nc;
      double lc=norm(nc);
      nc/=lc;
      dist=fabs(pw*nc.ortho());
      if( (dist<sw) && (fabs(pw*nc)<lc+sw) ) {
	return true;
      }
    }
    return false;
  }else{

    calcFuncRange();
    
    if(axes->axisScaling(Xfp::Graph_X)==Xfp::GASLogarithmic){
      if(w.x<axes->toScaled(Xfp::Graph_X,prange.minpos(Xfp::Graph_X))) return false;
      if(w.x>axes->toScaled(Xfp::Graph_X,prange.maxpos(Xfp::Graph_X))) return false;
    }else{
      if(w.x<prange.min(Xfp::Graph_X)) return false;
      if(w.x>prange.max(Xfp::Graph_X)) return false;
    }

    if(axes->axisScaling(Xfp::Graph_Y)==Xfp::GASLogarithmic){
      if(w.y<axes->toScaled(Xfp::Graph_Y,prange.minpos(Xfp::Graph_Y))) return false;
      if(w.y>axes->toScaled(Xfp::Graph_Y,prange.maxpos(Xfp::Graph_Y))) return false;
    }else{
      if(w.y<prange.min(Xfp::Graph_Y)) return false;
      if(w.y>prange.max(Xfp::Graph_Y)) return false;
    }

    return true;
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::print()const
{
  long i,ix,iy;

  if(pdatatype==XY||pdatatype==XYNW||pdatatype==XYMatel){
    printf("%s type\n",pdatatypes[pdatatype].guiname);
    for(i=0;i<parray[0]->size();i++){
      printf("%ld: ",i);
      for(unsigned int ic=0;ic<parray.components();ic++){
	printf(" %f",parray[ic]->at(i));
      }
      printf("\n");
    }
  }else if(pdatatype==GRIDX_Y_NZ){

    printf("GRID type\n");

    long nx=pxarray.size();
    long ny=pyarray.size();
    if(nx>0){
      printf("x=");
      long nwords=6;
      bool havenl=false;
      for(long ix=0;ix<nx;ix++){
	printf(" %.15g",pxarray.at(ix));
	havenl=false;
	if((nwords==1 &&ix<nx-1) || (ix%nwords==nwords-1 && ix<nx-1) ){
	  printf("\n");
	  havenl=true;
	}
      }
      if(!havenl){
	printf("\n");
      }
    }

    if(ny>0){
      printf("y=");
      long nwords=6;
      bool havenl=false;
      for(long iy=0;iy<ny;iy++){
	printf(" %.15g",pyarray.at(iy));
	havenl=false;
	if((nwords==1 &&iy<ny-1) || (iy%nwords==nwords-1 && iy<ny-1) ){
	  printf("\n");
	  havenl=true;
	}
      }
      if(!havenl){
	printf("\n");
      }
    }
    if(pxarray.size()==0) return ;
    if(pyarray.size()==0) return ;
    
    printf("z=");
    for(ix=0;ix<pxarray.size();ix++){
      for(iy=0;iy<pyarray.size();iy++){
// 	printf("%f",pxarray.at(ix));
// 	printf(" %f",pyarray.at(iy));

	for(unsigned int ic=0;ic<parray.components();ic++){
	  printf(" %f",parray[ic]->at(ix*pyarray.size()+iy));
	}
	printf("\n");
      }
    }
  }
  return ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::writeAsCommands(FILE *FHO,String tab)
{
  //FunctionAttribs *at=func->attribs();

  fprintf(FHO,
	  "%swith s%ld\n"
	  "%s  %s \n"
	  "%s  comment \"%s\"\n"
	  "%s"
	  "%s"
	  ,tab(),ID()+1,tab(),Xfp::OnOff(pactive)
	  ,tab(),pcomment()
	  ,pattribs->asCommands(tab+"  ")()
	  ,pweightselections->asCommands(tab+"  ",weightLabels())()
	  );
  if(pweightlabels){
    fprintf(FHO,
	    "%s  weightlabel reference%ld\n"
	    ,tab(),pweightlabels->uniqueIndex()+1
	    );
  }
  fprintf(FHO,
	  "%s  data %s\n"
	  ,tab(),dataTypeGuiName(dataType())
	  );
  if(pdatatype==XY || pdatatype == XYNW || pdatatype == XYMatel){
    if(parray.length()>0){
      long n=parray[0]->size();
      long nc=parray.components();
      for(long i=0;i<n;i++){
	fprintf(FHO,"%s ",tab());
	for(long ic=0;ic<nc;ic++){
	  fprintf(FHO," %.15g",parray[ic]->at(i));
	}
	fprintf(FHO,"\n");
      }
    }
    fprintf(FHO,"%s  end data\n",tab());
  }else{
    long nx=pxarray.size();
    long ny=pyarray.size();
    long nc=parray.components();
    fprintf(FHO,"%s  xdata\n"
	    "%s  # the x data , each number is an x, "
	    "even when multiple data on the same line\n",tab(),tab());
    if(nx>0){
      fprintf(FHO,"%s ",tab());
      long nwords=6;
      bool havenl=false;
      for(long ix=0;ix<nx;ix++){
	fprintf(FHO," %.15g",pxarray.at(ix));
	havenl=false;
	if((nwords==1 &&ix<nx-1) || (ix%nwords==nwords-1 && ix<nx-1) ){
	  fprintf(FHO,"\n%s ",tab());
	  havenl=true;
	}
      }
      if(!havenl){
	fprintf(FHO,"\n");
      }
    }
    fprintf(FHO,"%s  end xdata\n"
	    "%s  ydata\n"
	    "%s  # the y data , each number is an y, "
	    "even when multiple data on the same line\n",tab(),tab(),tab());
    if(ny>0){
      fprintf(FHO,"%s ",tab());
      long nwords=6;
      bool havenl=false;
      for(long iy=0;iy<ny;iy++){
	fprintf(FHO," %.15g",pyarray.at(iy));
	havenl=false;
	if((nwords==1 &&iy<ny-1) || (iy%nwords==nwords-1 && iy<ny-1) ){
	  fprintf(FHO,"\n%s ",tab());
	  havenl=true;
	}
      }
      if(!havenl){
	fprintf(FHO,"\n");
      }
    }
    fprintf(FHO,"%s  end ydata\n"
	    "%s  zdata\n"
	    "%s  # each line contains the z data for one x,y-point\n"
	    "%s  # x is the outer loop, y the inner\n"
	    "%s  # multiple columns, means mutiple z values for each point\n"
	    ,tab(),tab(),tab(),tab(),tab());
    long ii=0;
    for(long ix=0;ix<nx;ix++){
      for(long iy=0;iy<ny;iy++){
	fprintf(FHO,"%s ",tab());
	for(unsigned int ic=0;ic<nc;ic++){
	  fprintf(FHO," %.15g",parray[ic]->at(ii));
	}
	fprintf(FHO,"\n");
	ii++;
      }
    }
    fprintf(FHO,"%s  end zdata\n%s  end data\n",tab(),tab());
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::convolute(double width)
{
  if(pdatatype!=XY) return;
  if(parray.components()<2) return;
  if(parray.length()==0) return;

  long N=parray.length();
  ValueArray<double> h;
  h.resize(N,false);  
  for(long i=0;i<N;i++){
    double xi=parray[0]->at(i);
    for(long j=0;j<N;j++){
      double xj=parray[0]->at(j);
      double yj=parray[1]->at(j);
      double x=xi-xj;
      double ex=exp(-(x/width)*(x/width))/sqrt(Pi)/width;
      h[j]=ex*yj;
    }
    MyMath::runningSum(parray[0],&h);

    (*parray[1])[i]=h[N-1];
  }
  prange.invalidate();
  pcurrentinterpolationdepth=0;
  pcurrentinterpolcomp=0;
  pcurrentusesecondderivative=false;

  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::integrate()
{
  if(pdatatype!=XY) return;
  if(parray.components()<2) return;
  if(parray.length()==0) return;

  MyMath::runningSum(parray[0],parray[1]);
  prange.invalidate();
  pcurrentinterpolationdepth=0;
  pcurrentinterpolcomp=0;
  pcurrentusesecondderivative=false;

  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Function::calcMoment(long order,double xmin,double xmax) const
{
  if(pdatatype!=XY || (parray.components()<2) || (parray.length()==0)){
    return 0.;
  }

  long N=parray.length();
  ValueArray<double> hn;
  ValueArray<double> h;
  hn.resize(N,false);  
  h.resize(N,false);  
  for(long i=0;i<N;i++){
    double x=parray[0]->at(i);
    double y;
    y=parray[1]->at(i);
    if(x<xmin || xmax<x) y=0.;
    hn[i]=y;
    h[i]=pow(x,order)*y;
  }
  
  MyMath::runningSum(parray[0],&hn);
  MyMath::runningSum(parray[0],&h);
  return h[N-1]/hn[N-1];
  

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::interpolateGridFunction(long icomp,long depth,
				       bool usesecondderivative)
{
  if(pdatatype!=GRIDX_Y_NZ) return;
  if(depth<=0) {
    pcurrentinterpolationdepth=depth;
    pcurrentinterpolcomp=icomp;
    pcurrentusesecondderivative=usesecondderivative;
  }
  
  //myDebug()<<"pcurrentinterpolationdepth"<<pcurrentinterpolationdepth<<depth;
  if(pcurrentinterpolationdepth==depth && 
     pcurrentusesecondderivative==usesecondderivative &&
     pcurrentinterpolcomp==icomp ) return;

  

  long Nx=pxarray.size();
  long Ny=pyarray.size();
  if(!Ny) return;
  
  const double *fy=pyarray.dataRef();
  const double *fz=parray[icomp]->dataRef();
  long nsuby=depth+1;
  long Nout=nsuby*(Ny-1)+1;
 
  pxarrayinterpol=pxarray;
  pyarrayinterpol.resize(Nout,true);
  parrayinterpol.adjustComponents(1,true);
  parrayinterpol.adjustLength(Nx*Nout,true);

  //myDebug()<<"start interpolating";

  //interpolate data to get a finer mesh
  
  for(long i=0;i<Ny;i++){
    double x0=fy[i];
    if(i==Ny-1){
      pyarrayinterpol[(Ny-1)*nsuby]=x0;
      break;
    }
    double dx=(fy[i+1]-x0)/nsuby;
    for(long k=0;k<nsuby;k++){
      pyarrayinterpol[i*nsuby+k]=x0;
      x0+=dx;
    }
  }
  
  ValueArray<double> *fzinter=parrayinterpol[0];
  BSpline bsp(4,fy,Ny);
  for(long ix=0;ix<Nx;ix++){
    bsp.splint(fz+ix*Ny);
    for(int j=0;j<Nout;j++){
      double f=bsp.value(pyarrayinterpol[j],0);
      if(usesecondderivative){
	double df2=bsp.value(pyarrayinterpol[j],2);
	(*fzinter)[ix*Nout+j]=f*MyMath::max<double>(0,-df2);
      }else{
	(*fzinter)[ix*Nout+j]=f;
      }
    }
    
  }
  //myDebug()<<"done interpolating";
  pcurrentinterpolationdepth=depth;
  pcurrentinterpolcomp=icomp;
  pcurrentusesecondderivative=usesecondderivative;
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::bspline(long order,long ideriv)
{
  
  bspline(order,ideriv,*(parray[0]));
  prange.invalidate();

  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Function::bspline(long order,long ideriv,const ValueArray<double> &xmesh)
{
  //myDebug()<<order<<ideriv;
  long Nx=parray.length();
  const double *fx=parray[0]->constDataRef();
  double *fy=parray[1]->dataRef();

  BSpline bsp(order,fx,Nx);
  bsp.splint(fy);
  long Nout=xmesh.size();
  //myDebug()<<"xmesh"<<xmesh;

  adjustLength(Nout,true);
  
  for(int j=0;j<Nout;j++){
    double f=bsp.value(xmesh.at(j),ideriv);
    (*parray[0])[j]=xmesh.at(j);
    (*parray[1])[j]=f;
    //printf("%30.15f %30.15f \n",(*parray[0])[j],(*parray[1])[j]);
  }
}

/*EOF*/ 
