//todo: find all places, where func is altered to invalidate Ranges

/* @(#)Function.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _MYFUNCTION_H
#define _MYFUNCTION_H 1

#include "FString.hh"
#include "PtrArray.hh"
#include "FGObject.hh"
#include "Range.hh"
#include "ValueArray.hh"
#include "Xfp.hh"

class FunctionGroup;
class Point;
class Graph;
class GraphAxes;
class Size;
class FGObject;
class WeightLabels;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class  NDoubleArrays{
 public:
  NDoubleArrays():plength(0){};
 ~NDoubleArrays(){
   if(!pdata.size())return;
   for(long i=pdata.size()-1;i>=0;i--){
     ValueArray<double> *p=pdata.takeAt(i);
     if(p)
       delete p;
   }
 };
 NDoubleArrays &operator=(const NDoubleArrays &other){
   plength=other.plength;
   pdata.clear();
   pdata.resize(other.pdata.size());
   for(long i=0;i<pdata.size();i++){
     //pdata[i]->resize(plength,true);
     *(pdata[i])=*(other.pdata.at(i));
   }
   return *this;
 };
 long components()const{return pdata.size();};
 long length()const{return plength;};
 inline const ValueArray<double> *operator[](long ic)const{return pdata.at(ic);};
 inline  ValueArray<double> *operator[](long ic){return pdata[ic];};
 void adjustComponents(long newcomponents,bool shrink=false){
   long currentcomponents=pdata.size();
   if(newcomponents>currentcomponents || shrink){
     pdata.resize(newcomponents,shrink);
   }
   if(newcomponents>currentcomponents){
     for(long i=currentcomponents;i<newcomponents;i++){
       pdata[i]->resize(plength,true);
     }
   }
 };

 void adjustLength(long newlength,bool shrink=false){
   if(!shrink && newlength<=plength) return;
   for(long i=0;i<pdata.size();i++){
     pdata[i]->resize(newlength,shrink);
   }
   plength=newlength;
 };

 private:
 long plength;
 PtrArray<ValueArray<double> > pdata;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Function:public FGObject{
 public:
  
  Function();
  Function(FunctionGroup *fg,long uniqueid);
  virtual ~Function();

  FunctionGroup *group()const{return pgroup;};
  void reParent(FunctionGroup *p);

  void copyFrom(const Function *other);

  void adjustXYLength(Xfp::XYAxisType xy,long size,bool shrink=false); 

  inline void adjustComp(unsigned long components,bool shrink){
    if(components==0){
      parray.adjustLength(0,shrink);
    }
    parray.adjustComponents(components,shrink);
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;
  }
  inline void adjustComp(unsigned long components,
			 unsigned long length,bool shrink){
    parray.adjustLength(length,shrink);
    parray.adjustComponents(components,shrink);
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;

  };
  inline void adjustLength(unsigned long length,bool shrink=false){
    parray.adjustLength(length,shrink);
    prange.invalidate();
    pcurrentinterpolationdepth=0;
    pcurrentinterpolcomp=0;
    pcurrentusesecondderivative=false;
  };

  void invalidateRange(){prange.invalidate();}

  void setData(long ic,long i,double d);
  void setDataComponents(long i,double *data,long ndata);
  void setDataXY(Xfp::XYAxisType xy,long i,double d);

  void setData(const ValueArray<double> &x,const ValueArray<double> &y);
  void appendXYData(Xfp::XYAxisType xy,const ValueArray<double> &x);

  void shiftOrigin(Xfp::XYAxisType xy,double o);
  void scaleData(Xfp::XYAxisType xy,double s);

  inline long components()const{return parray.components();};
  inline long length()const{return parray.length();};
  inline const ValueArray<double> *data(long icomp)const{
    return parray[icomp];};
  inline double data(long icomp,long i)const{return parray[icomp]->at(i);};

  inline const ValueArray<double> &xdata()const{return pxarray;};
  inline const ValueArray<double> &ydata()const{return pyarray;};

  inline const ValueArray<double> &xdatainterpolated()const{
    if(pcurrentinterpolationdepth==0)
      return pxarray;
    else
      return pxarrayinterpol;
  };
  inline const ValueArray<double> &ydatainterpolated()const{
    if(pcurrentinterpolationdepth==0)
      return pyarray;
    else
      return pyarrayinterpol;
  };
  inline const double *datainterpolated(long icomp)const{
    if(pcurrentinterpolationdepth==0)
      return parray[icomp]->constDataRef();
    else
      return parrayinterpol[0]->constDataRef();
  };
  
  bool gridIsEquidistant(double tol=1.0e-6)const;

  void interpolateGridFunction(long icomp,long depth,
			       bool usesecondderivative);

  void setUpDefaultWeightLabels();

  void setWeightLabels(WeightLabels *wl);
  void setWeightLabelReference(long r){pweightlabelreference=r;};
  void setWeightLabelByReference();

  DataType dataType()const{return pdatatype;};
  void setDataType(DataType dt){pchanged|=(pdatatype!=dt);pdatatype=dt;};
    
  void print()const;
  void calcRange(Range &range,long restrictto,double omin,double omax);

  Range calcFuncRange();

  bool isInRange(const Point &w0,const Point &w1);

  bool isCloseToPoint(const Point &w,const Size &sensitivespot,
		      double &dist,const GraphAxes *axes);

  void writeAsCommands(FILE *FHO,String tab);

  void convolute(double width);
  double calcMoment(long order,double xmin,double xmax)const;
  void integrate();
  void bspline(long order,long ideriv);
  void bspline(long order,long ideriv,const ValueArray<double> &xmesh);

 protected:

  const WeightLabels *weightLabelsInternalConst()const;



 private:
  struct tsDataType{
    DataType type;
    const char *guiname;
  };

 public:
  static DataType dataTypeFromString(const  char *s);
  inline static const char *dataTypeGuiName(DataType dt){
    return pdatatypes[dt].guiname;};
  static long numOfDataTypes(){return pdatamaxtypes;};
  void clean();
  

 private:
  DataType pdatatype;	 	/* which data type in array */  
  
  /* The function is an array of Ncomp arrays each of length Narray */
  //  double **Array;                  /* data array of type 'datatype' */
  NDoubleArrays parray;
  /* for grid data we need separate x and y data
     the z data are arranged continously in Array with x running first
     : z(x0,y0),z(x0,y1),z(x0,y2),...,z(x1,y0),z(x1,y2),...
   */
  ValueArray<double> pxarray;
  ValueArray<double> pyarray;

  // interpolated data;

  NDoubleArrays parrayinterpol;
  ValueArray<double> pxarrayinterpol;
  ValueArray<double> pyarrayinterpol;
  
 private:
  WeightLabels *pweightlabels; /* if the functions have weights this is
				  the list of labels, which may be shared with 
				  other funcs and groups */

  static const tsDataType pdatatypes[];
  static const long pdatamaxtypes;

  FunctionGroup *pgroup;

  long pweightlabelreference;
  
  long pcurrentinterpolationdepth;
  long pcurrentinterpolcomp;
  bool pcurrentusesecondderivative;

  Range prange;

  friend class Functions;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _MYFUNCTION_H */

