/*************************************************************************
 * @(#)FunctionGroup.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Apr 2011
 *************************************************************************
 */

#include "Attributes.hh"
#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "MyProgressDialog.hh"
#include "TicMarks.hh"
#include "WeightSelection.hh"
#include "defs.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup::FunctionGroup(Graph *parent,long uniqueid)
  :FGObject(FGObject::FGOTGroup,uniqueid),
   puseattribs(false),
   pfuncs(new Functions(this)),
   puserticmarks(new Tics())

{
  puserticmarks->setDrawTics(false);
  //todo: putin constructor
  setGraph(parent);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup::~FunctionGroup()
{
//   printf("FunctionGroup dying %p\n",this);
//   printf("FunctionGroup dying graph=%p\n",pgraph);

  if(puserticmarks) delete puserticmarks; puserticmarks=0;
  if(pfuncs) delete pfuncs; pfuncs=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FunctionGroup::changed()const
{
  bool c=FGObject::changed();
  if(puserticmarks) c|=puserticmarks->changed();
  c|=pfuncs->changed();
  
  return c;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroup::setChanged(bool c)
{
  if(puserticmarks) puserticmarks->setChanged(c);
  pfuncs->setChanged(c);
  
  pchanged=c;
  FGObject::setChanged(c);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroup::writeAsCommands(FILE *FHO,String tab,
				    MyProgressDialog &prgdlg)
{
  fprintf(FHO,"%swith gr%ld\n"
	  "%s  %s \n"
	  "%s  comment \"%s\"\n"
	  "%s  use attributes %s\n"
	  "%s"
	  "%s"
	  "%s"
	  ,tab(),ID()+1,tab(),Xfp::OnOff(pactive)
	  ,tab(),pcomment()
	  ,tab(),Xfp::OnOff(puseattribs)
	  ,pattribs->asCommands(tab+"  ")()
	  ,puserticmarks->asCommands(tab+"  ")()
	  ,pweightselections->asCommands(tab+"  ",weightLabels())()

	  );
  
  tab+="  ";
  
  for(long ifu=0;ifu<pfuncs->size();ifu++){
    prgdlg.increaseCounter();
    Function *func=pfuncs->at(ifu);
    func->writeAsCommands(FHO,tab);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const WeightLabels *FunctionGroup::weightLabelsInternalConst()const
{
  // return weightlable, if shared by all functions, otherwise 0
  // groups can only be weightlabel manipulators, if all funcs
  // have the same wightlabel
  WeightLabels *ret=0;

  bool start=true;
  for(long ifu=0;ifu<pfuncs->size();ifu++){
    Function *func=(Function *)pfuncs->at(ifu);
    WeightLabels *wls=func->weightLabels();
    if(func->dataType()==Function::XYNW && wls ){
      if(start){
	ret=wls;
	start=false;
      }else{
	if(wls!=ret) {
	  ret=0;
	  break;
	}
      }
    }else{
      return 0;
    }
  }
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FunctionGroup::allFuncsHaveSaveNumberOfComponents(long &ncomp)const
{
  ncomp=-1;
  for(long i=0;i<pfuncs->size();i++){
    if(ncomp<0)ncomp=pfuncs->at(i)->components();
    if(ncomp!=pfuncs->at(i)->components())return false;
  }
  return true;
}



/*EOF*/ 
