/* @(#)FunctionGroup.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _FUNCTIONGROUP_H
#define _FUNCTIONGROUP_H 1

#include <stdio.h>
#include "FString.hh"
#include "FGObject.hh"
#include "PtrArray.hh"
#include "ValueArray.hh"

class Tics;
class FunctionGroups;
class Graph;
class MyProgressDialog;
class Functions;

/* ******************************
 * the group class  
 * ******************************
 */

class FunctionGroup: public FGObject{
 public:

  FunctionGroup(Graph *parent,long uniqueid);
  virtual ~FunctionGroup();


  Functions *funcs(){return pfuncs;};
  const Functions *funcs()const {return pfuncs;};

  
  void reParentFuncs(){ 
    for(int i=0;i<pfuncs->size();i++) 
      pfuncs->at(i)->reParent(this); 
  };


  bool hasFuncOfType(FGObject::DataType t)const{
    for(int i=0;i<pfuncs->size();i++){
      if(pfuncs->at(i)->dataType()==t)
	return true;
    }
    return false;
  }


  bool allFuncsAre(FGObject::DataType t)const{
    for(int i=0;i<pfuncs->size();i++){
      if(pfuncs->at(i)->dataType()!=t)
	return false;
    }
    return true;
  }


  bool allFuncsHaveSaveNumberOfComponents(long &ncomp)const;

  Tics *userTics(){return puserticmarks;};
  const Tics *userTics()const{return puserticmarks;};


  bool useAttribs()const{return puseattribs;};
  void setUseAttribs(bool a){pchanged|=(puseattribs!=a);puseattribs=a;};

  void writeAsCommands(FILE *FHO,String tab,MyProgressDialog &prgdlg);

  void invalidateRange(){if(pfuncs) pfuncs->invalidateRange();}

  bool changed()const;
  void setChanged(bool);


 protected:
  const WeightLabels *weightLabelsInternalConst()const;
  
  
 private:
  bool puseattribs ;		/* shall this attribs overwrite the
				   single func attribs?
				 */
  Functions *pfuncs;     	/* functions */
  Tics *puserticmarks;          /* irregular user defined tics for this group*/


  friend class Function;
  friend class FunctionGroups;

};




#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _FUNCTIONGROUP_H */

