/*************************************************************************
 * @(#)FunctionGroups.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   19 Jul 2012
 *************************************************************************
 */

#include "GraphObject.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "Graph.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroups::FunctionGroups(Graph *parent)
  :pgraph(parent),pcurrentgroupindex(-1)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroups::~FunctionGroups()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroups::reset()
{
  pgroups.clear(); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *FunctionGroups::takeAt(long i)
{
  pgraph->setChanged(true);
  return pgroups.takeAt(i);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *FunctionGroups::take(FunctionGroup *g)
{
  pgraph->setChanged(true);
  return pgroups.take(g);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *FunctionGroups::getById(long uniqueid,bool append)
{
  // get an existing graph by id or create a new one with id
  for(int i=0;i<pgroups.size();i++){
    if(pgroups.at(i)->puniqueid==uniqueid){
      return pgroups[i];
    }
  }
  if(!append) return 0;
  
  FunctionGroup *fg=new FunctionGroup(pgraph,uniqueid);
  fg->setChanged(true);
  pgroups.append(fg);
  
  return fg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroups::insert(long position,FunctionGroup *fg)
{
  if(!fg){
    printf("FunctionGroups::insert: fg=0!?\n");
    exit(1);
  }
  fg->setGraph(pgraph);
  fg->reParentFuncs();
  fg->setChanged(true);
  pgroups.insert(position,fg);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *FunctionGroups::insert(long i)
{
  FunctionGroup *fg=new FunctionGroup(pgraph,makeUniqueId());
  fg->setChanged(true);
  pgroups.insert(i,fg);
  return fg;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FunctionGroup *FunctionGroups::getExistingGroupOrNew(long id)
{
  if(id<0){
    append();
    return pgroups[pgroups.size()-1];
  }
  return getById(id);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FunctionGroups::changed()const
{
  for(int i=0;i<pgroups.size();i++){
    if(pgroups.at(i) && pgroups.at(i)->changed())
      return true;
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroups::setChanged(bool c)
{
  for(int i=0;i<pgroups.size();i++){
    if(pgroups[i])
      pgroups[i]->setChanged(c);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroups::invalidateRange()
{
  for(int i=0;i<pgroups.size();i++){
    if(pgroups[i])
      pgroups[i]->invalidateRange();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FunctionGroups::makeUniqueId()
{
  long ind=0;
  while (true){
  again:;
    for(long i=0;i<size();i++){
      if(pgroups.at(i)->puniqueid==ind){
	ind++;
	goto again;
      }
    }
    break;
  }
  return ind;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FunctionGroups::packIDs()
{
  for(long i=0;i<pgroups.size();i++){
    FunctionGroup *fg=pgroups[i];
    fg->puniqueid=i;
    for(long j=0;j<fg->funcs()->size();j++){
      fg->funcs()->at(j)->puniqueid=j;
    }
    
  }

}

/*EOF*/ 
