/*************************************************************************
 * @(#)Functions.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   19 Jul 2012
 *************************************************************************
 */

#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Functions::Functions(FunctionGroup *parent)
  :pgroup(parent)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Functions::changed()const
{
  for(int i=0;i<pfuncs.size();i++){
    if(pfuncs.at(i) && pfuncs.at(i)->changed())
      return true;
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::setChanged(bool c)
{
  for(int i=0;i<pfuncs.size();i++){
    if(pfuncs[i])
      pfuncs[i]->setChanged(c);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::invalidateRange()
{
  for(int i=0;i<pfuncs.size();i++){
    if(pfuncs[i])
      pfuncs[i]->invalidateRange();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::setSize(long size)
{
  long N=pfuncs.size();
  pfuncs.resize(size,true);
  pgroup->setChanged(true);

  for(int i=N;i<pfuncs.size();i++){
    pfuncs[i]->reParent(pgroup);
    assureUniqueId(pfuncs[i]);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::assureSize(long size)
{
  long N=pfuncs.size();
  if(size<N)return;
  setSize(size);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function *Functions::insert(long i)
{
  Function *fu=new Function(pgroup,makeUniqueId());
  pgroup->setChanged(true);
  fu->setGraph(pgroup->graph());
  pfuncs.insert(i,fu);
  return fu;
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::insert(long position,Function *fu)
{
  if(!fu){
    printf("Functions::insert: fu=0!?\n");
    exit(1);
  }
  fu->reParent(pgroup);
  assureUniqueId(fu);
  pgroup->setChanged(true);
  pfuncs.insert(position,fu);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function *Functions::takeAt(long i)
{
  pgroup->setChanged(true);
  return pfuncs.takeAt(i);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function *Functions::take(Function *f)
{
  pgroup->setChanged(true);
  return pfuncs.take(f);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Functions::makeUniqueId()
{
  long maxid=-1;
  for(int i=0;i<pfuncs.size();i++){
    maxid=MyMath::max<long>(maxid,pfuncs.at(i)->ID());
  }
  
  return maxid+1;
  
//   long ind=0;
//   while (true){
//   again:;
//     for(long i=0;i<size();i++){
//       if(pfuncs.at(i)->puniqueid==ind){
// 	ind++;
// 	goto again;
//       }
//     }
//     break;
//   }
//   return ind;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Function *Functions::getById(long uniqueid,bool append)
{
  // get an existing func by id or create a new one with id
  for(int i=0;i<pfuncs.size();i++){
    if(pfuncs.at(i)->puniqueid==uniqueid){
      return pfuncs[i];
    }
  }
  if(!append) return 0;
  
  Function *fu=new Function(pgroup,uniqueid);
  pgroup->setChanged(true);
  fu->setGraph(pgroup->graph());
  pfuncs.append(fu);
  
  return fu;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Functions::assureUniqueId(Function *fu)
{
  bool alright=true;
  if(fu->ID()>=0){
    for(int i=0;i<pfuncs.size();i++){
      if(pfuncs.at(i)->ID()==fu->ID() && pfuncs.at(i)!=fu){
	alright=false;
	break;
      }
    }
    if(alright) return;
  }
  long maxid=-1;
  for(int i=0;i<pfuncs.size();i++){
    maxid=MyMath::max<long>(maxid,pfuncs.at(i)->ID());
  }
  
  fu->puniqueid=maxid+1;
}

/*EOF*/ 
