/* @(#)Graph.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _GRAPH_H
#define _GRAPH_H 1

#include "GraphObject.hh"
#include "Point.hh"
#include "PaintEngine.hh"
#include "PtrArray.hh"
#include "Xfp.hh"
//#include "debug.h"

class GraphAxes;
class Function;
class FunctionGroups;
class FunctionAttribs;
class LegendBox;
class TextBox;
class TextBoxes;
class Tics;
class FGObject;
class OptMatelDefinition;
class Shape;
class Stripes;
class Shapes;
class TicMarks;
class ViewBox;
class WeightSelections;
class World;
class Page;

/* ******************************
 * the graph class  
 * ******************************
 */

class Graph:public GraphObject{
 public:

  enum GraphType{NewGraph=-3,CurrentGraph=-2,UnknownGraph=-1};

  static const char *graphIDGuiName(GraphType id){
    if(id==NewGraph) return pgraphids[0].guiname;
    if(id==CurrentGraph) return pgraphids[1].guiname;
    return "Graph ";
  };


  Graph();
  Graph(Page *parent,long uniqueid);
  ~Graph();

  Page *page(){return ppage;};
  void reParent(Page *p){ppage=p;};

  long ID()const{return puniqueid;};

  

  FunctionGroups *groups(){return pgroups;};
  const FunctionGroups *groups()const {return pgroups;};





  String guiName(bool longname=false)const;
    
  bool active(){return pactive;};
  void setActive(bool a){pchanged|=(pactive!=a);pactive=a;};


  TextBoxes *textBoxes(){return ptextboxes;};
  const TextBoxes *textBoxes()const{return ptextboxes;};

  Shapes *shapes(){return pshapes;};
  const Shapes *shapes()const{return pshapes;};

  LegendBox *legendBox(){return plegendbox;};
  const LegendBox *legendBox()const{return plegendbox;};

  GraphAxes *axes(){return paxes;};
  const GraphAxes *axes()const {return paxes;};
  
  TicMarks *ticMarks(){return pticmarks;};
  const TicMarks *ticMarks()const{return pticmarks;};

  World *world(){return pworld;};
  const World *world()const{return pworld;};

  ViewBox *viewBox(){return pviewbox;};
  const ViewBox *viewBox()const{return pviewbox;};

  double allOverPointScale()const{return palloverpointscale;};
  void setAllOverPointScale(double s){
    pchanged|=(palloverpointscale!=s);palloverpointscale=s;};

  double allOverLineWidthScale()const{return palloverlinewidthscale;};
  void setAllOverLineWidthScale(double s){
    pchanged|=(palloverlinewidthscale!=s);palloverlinewidthscale=s;};

  bool changed()const;
  void setChanged(bool );
 
  void plotGraph(PaintEngine *pengine);
  void plotGraphLegend(PaintEngine *pengine);
  
  


  void autoScaleAll(Xfp::XYAxisType xy,bool restricted=false); 
  void autoScaleFunc(FGObject *fgo); 

  //experimental
  void getObjectsAt(PtrArray<GraphObject> &objects,const DragDescriptor &dd,
		    bool update);

  void plotDragObject(PaintEngine *pe,GraphObject *go);

  void drawCurrentMarkers(PaintEngine *pe,bool dragging=false);

  
  void changeScaling(Xfp::XYAxisType xy,Xfp::GraphAxisScaling oldsc,
		     Xfp::GraphAxisScaling newsc);
  
 private:
  enum ViewBoxPlotMode{VBPMFill=0x1,VBPMFrame=0x2};


  void plotInit(PaintEngine *pe);
  void additionalOffset(PaintEngine *pe,TextBox *tb);
  void formText(PaintEngine *pe
		,const double xx0,const double yy0,const String &txt
		,double const angle
		,const double fontsize,Xfp::TextOrientation just
		,const double factor
		,bool show);
  void plot_drag_world(PaintEngine *pe,GraphObject *object);
  void plotTextBox(PaintEngine *pe,TextBox *tb);
  void plot_drag_textbox(PaintEngine *pe,GraphObject *object);
  void plot_drag_shape(PaintEngine *pe,GraphObject *object);
  void plot_legend(PaintEngine *pe,LegendBox *lb);
  void plot_drag_legendbox(PaintEngine *pe,GraphObject *go);       
  void plot_viewbox(PaintEngine *pe,const ViewBox *vb,ViewBoxPlotMode mode);
  void plot_drag_viewbox(PaintEngine *pe,GraphObject *go);
  void plotshapes(PaintEngine *pe);
  void plotShape(PaintEngine *pe,const Shape *sh);

  void plotticmarks_internal(PaintEngine *pe
			     ,const Tics *
			     ,Xfp::XYAxisType xy
			     ,Xfp::TicSide ticside
			     ,bool show
			     );
  void plotticsandmarks_internal(PaintEngine *pe
				 ,Xfp::XYAxisType xy,Xfp::TicSide ticside
				 ,bool show);
  void plotticsandmarks(PaintEngine *pe);
  void plotfuncs(PaintEngine *pe);
  void plotdensfunc(Function *func,const FunctionAttribs *attrib
		    ,PaintEngine *pe);
  void plotxyfunc(Function *func,const FunctionAttribs *attrib
		  ,PaintEngine *pe,bool nowSymbols);
  void plotxynwfunc(Function *func,const FunctionAttribs *attrib,
		    const WeightSelections *wss,
		    long iweightsel,PaintEngine *pe);
  void plotmatelfunc(const Function *func0,const Function *func1,long icomp,
		     const OptMatelDefinition *mateldef,
		     const FunctionAttribs *attrib,
		     Stripes &stripes);
  void plotstripes(const Stripes &stripes,PaintEngine *pe);
  void plottextboxes(PaintEngine *pe);
  //  void plotaxes(PaintEngine *pe);


  void produceGridFunccoordinates(ValueArray<double> &xx,
				  const double *data,long Ndata,
				  long &imin,long &imax,
				  Xfp::XYAxisType xy);

  void legDetTextSize(PaintEngine *pe,const Point &q0,const String &text,
		   const LegendBox *lb);
  void legDrawEntry(PaintEngine *pe,const FunctionAttribs *at,
		    double legsymwidth,bool adjusttosymbols,
		    const String &text,LegendBox *lb,bool widthLine);
  

  
 private:
  /* ---- view area ---- */
  ViewBox *pviewbox;
  /* ---- world coordinates ---- */
  World *pworld;
  /* ---- legendbox ---- */
  LegendBox *plegendbox;
  /* ---- tic marks ---- */
  TicMarks *pticmarks;
  /* ---- text boxes ---- */
  TextBoxes *ptextboxes;
  /* ---- function groups ---- */
  FunctionGroups *pgroups;
  /* ---- Shapes ---- */
  Shapes *pshapes;
  /* ---- axes  ---- */
  GraphAxes *paxes;

  bool pactive;
  double palloverpointscale;
  double palloverlinewidthscale;
 private:
  //internal state variables
  bool pxopossitetic;		/* are oposite site x tics drawn */
  bool pxnormsitetic; 		/* are normal site x tics drawn */
  bool pyopossitetic;		/* are oposite site y tics drawn */
  bool pynormsitetic; 		/* are normal site y tics drawn */

  Page *ppage;
  long puniqueid;
 private:
  struct tsGraphID{
    GraphType type;
    const  char *guiname;
  };
  static const tsGraphID pgraphids[];
  static const long pmaxgraphid;

  friend class Graphs;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _GRAPH_H */

