/* @(#)GraphAxis.hh
 * author: magru@unas
 */

#ifndef _GRAPHAXIS_HH
#define _GRAPHAXIS_HH 1

#include <limits> 
#include "GraphObject.hh"
#include "PtrArray.hh"
#include "Graph.hh"
#include "Xfp.hh"
#include "Point.hh"

class GraphAxes;

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class GraphAxis: public GraphObject
{
public:

  // ----------------------------------------------------------------
  GraphAxis():GraphObject(0),paxisscaling(Xfp::GASLinear){}

  // ----------------------------------------------------------------
  virtual ~GraphAxis(){}

  // ----------------------------------------------------------------
  virtual void reset(){};

  // ----------------------------------------------------------------
  inline Xfp::GraphAxisScaling axisScaling()const{return paxisscaling;};


  // ----------------------------------------------------------------
  virtual bool changed()const;
  virtual void setChanged(bool c);

protected:
private:
  // ----------------------------------------------------------------
  void setScaling(Xfp::GraphAxisScaling sc)
  { pchanged|=(paxisscaling!=sc);paxisscaling=sc; }
private:
  Xfp::GraphAxisScaling paxisscaling;
  friend class GraphAxes;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class GraphAxes: public GraphObject
{
public:
  GraphAxes(Graph *);
  virtual ~GraphAxes(){}

  void reset();

  // ----------------------------------------------------------------
  inline const GraphAxis *at(long i)const{return paxes.at(i);}

  // ----------------------------------------------------------------
  inline Xfp::GraphAxisScaling axisScaling(Xfp::XYAxisType xy)const{
    return paxes.at(xy)->axisScaling();
  }
  // ----------------------------------------------------------------
  inline void setScaling(Xfp::XYAxisType xy,Xfp::GraphAxisScaling sc)
  {
    if((paxes[xy]->axisScaling()!=sc)) 
      graph()->changeScaling(xy,paxes[xy]->axisScaling(),sc);
    paxes[xy]->setScaling(sc);
  }


  // ----------------------------------------------------------------
  inline bool allow(const Point &p)const{
    if(paxes.at(Xfp::Graph_X)->axisScaling()==Xfp::GASLogarithmic){
      if(p.x<=0.0) return false;
    }
    if(paxes.at(Xfp::Graph_Y)->axisScaling()==Xfp::GASLogarithmic){
      if(p.y<=0.0) return false;
    }
    return true;
    
  }
  
  // ----------------------------------------------------------------
  inline Point fromScaled(const Point &p)const{
    Point r=p;
    r.x=fromScaled(Xfp::Graph_X,p.x);
    r.y=fromScaled(Xfp::Graph_Y,p.y);
    return r;
  }

  // ----------------------------------------------------------------
  inline Size fromScaled(const Size &s)const{
    Size r;
    r.w=sizeFromScaled(s.w,paxes.at(Xfp::Graph_X)->axisScaling());
    r.h=sizeFromScaled(s.h,paxes.at(Xfp::Graph_Y)->axisScaling());
    return r;
  }

  // ----------------------------------------------------------------
  inline Point toScaled(const Point &p)const{
    Point r;
    r.x=toScaled(Xfp::Graph_X,p.x);
    r.y=toScaled(Xfp::Graph_Y,p.y);
    return r;
  }
  // ----------------------------------------------------------------
  inline Size toScaled(const Size &s)const{
    Size r;
    r.w=sizeToScaled(s.w,paxes.at(Xfp::Graph_X)->axisScaling());
    r.h=sizeToScaled(s.h,paxes.at(Xfp::Graph_Y)->axisScaling());
    return r;
  }

  // ----------------------------------------------------------------
  inline double fromScaled(Xfp::XYAxisType xy,double x)const{
    return fromScaled(x,paxes.at(xy)->axisScaling());
  }


  // ----------------------------------------------------------------
  inline double toScaled(Xfp::XYAxisType xy,double x)const{
    return toScaled(x,paxes.at(xy)->axisScaling());
  }

  // ----------------------------------------------------------------
  inline double adjustUnscaled(Xfp::XYAxisType xy,double x)const{
    bool adjusted;
    return adjustUnscaled(xy,x,adjusted);
  }
  inline double adjustUnscaled(Xfp::XYAxisType xy,double x,bool &adjusted)const{
    return adjustUnscaled(x,paxes.at(xy)->axisScaling(),adjusted);
  }
  

  // ----------------------------------------------------------------
  static inline double fromScaled(double x,Xfp::GraphAxisScaling sc){
    if(sc==Xfp::GASLogarithmic){
      return pow(10.,x);
    }else{
      return x;
    }
  }

  // ----------------------------------------------------------------
  static inline double toScaled(double x,Xfp::GraphAxisScaling sc){
    if(sc==Xfp::GASLogarithmic){
      if(x<=0.) {
	return log10(std::numeric_limits<double>::min()*1e2);
      }else{
	return log10(x);
      }
    }else{
      return x;
    }
  }
  
  // ----------------------------------------------------------------
  static inline double sizeFromScaled(double x,Xfp::GraphAxisScaling sc){
    if(sc==Xfp::GASLogarithmic){
      //return pow(10.,x+log10(1.))-1; //not needed but for logic
      return pow(10.,x)-1;
    }else{
      return x;
    }
  }

  // ----------------------------------------------------------------
  static inline double sizeToScaled(double x,Xfp::GraphAxisScaling sc){
    if(sc==Xfp::GASLogarithmic){
      //return log10(1.+x)-log10(1.); //not needed but for logic
      return log10(1.+x);
    }else{
      return x;
    }
  }
  
  // ----------------------------------------------------------------
  static inline double adjustUnscaled(double x,Xfp::GraphAxisScaling sc,bool &adjusted){
    adjusted=false;
    if(sc==Xfp::GASLogarithmic){
      if(x<=0.) {
	adjusted=true;
	return std::numeric_limits<double>::min()*1e2;
      }else{
	return x;
      }
    }else{
      return x;
    }
  }
  

  String asCommands(const String &prefix)const{
    return Sprintf("%s# axis settings\n"
		   "%sx axis scaling %s\n"
		   "%sy axis scaling %s\n"
		   ,prefix()
		   ,prefix(),Xfp::graphAxisScalingTypeGuiName(axisScaling(Xfp::Graph_X))
		   ,prefix(),Xfp::graphAxisScalingTypeGuiName(axisScaling(Xfp::Graph_Y))
		   );
  }

  bool changed()const;
  void setChanged(bool c);

protected:
private:
  bool pchanged;
  PtrArray<GraphAxis> paxes;
 
};

#endif /* _GRAPHAXIS_HH */

