/* @(#)GraphObject.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _GRAPHOBJECT_H
#define _GRAPHOBJECT_H 1

#include "FString.hh"
#include "InsertMenuData.hh"
#include "Point.hh"

class Graph;
template <class T>
class PtrArray;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class DragDescriptor{
 public:
  DragDescriptor(double s):dragging(false),drawdragmarker(false)
    ,sensitivityAroundPoint(s){};
  bool dragging;
  bool drawdragmarker;
  Point mousepoint;
  Size devicesize;
  double sensitivityAroundPoint;
  Point w0,w1;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class GraphObject{
 public:
  enum GraphObjectCommand{
    GOCNone      =0x00,
    GOCX         =0x01,
    GOCY         =0x02,
    GOCXY        =GOCX | GOCY,
    GOCPosition  =0x04,
    GOCSize      =0x08,
    GOCPosAndSize=GOCPosition | GOCSize,
    GOCOffset    =0x10,
    GOCActive    =0x1000,
    GOCText      =0x2000,
    GOCMeasureXY =0x3000|GOCXY
  };
  enum GraphObjectRegion{
    GORNone=0,
    GORUL=1,
    GORUR=2,
    GORLL=3,
    GORLR=4,
    GORBody=5
  };
  enum GraphObjectType{
    GOTNone=0,
    GOTWorld=1,
    GOTViewBox=2,
    GOTLegendBox=3,
    GOTTextBox=4,
    GOTTicMarks=5,
    GOTShape=6,
    GOTFunctionOrGroup=7,
    GOTGraph=8
  };

  GraphObject(Graph *parent):
    pchanged(false),pupdaterequest(GOCNone),pupdaterequestnumber(0),
    pgraph(parent),pobjecttype(GOTNone){};
  virtual ~GraphObject(){};

  virtual GraphObjectRegion insideRegion(const DragDescriptor &,
					 bool /*update*/){
    return GraphObject::GORNone;};
  virtual void initDrag(const DragDescriptor &){};
  virtual void updateDrag(const DragDescriptor &){};
  virtual void setFromDrag(DragDescriptor&){};

  GraphObjectType objectType()const{return pobjecttype;};
  void setObjectType(GraphObjectType t){pobjecttype=t;};


  const Point &mousePos()const{return pmousepos;};
  const Point &objectPos()const{return pobjectpos;};

  GraphObjectCommand updateRequest()const{return pupdaterequest;};
  void setUpdateRequest(GraphObjectCommand re){pupdaterequest=re;};

  Graph *graph(){return pgraph;}
  const Graph *graph()const {return pgraph;}
  void setGraph(Graph *gr){pgraph=gr;};

  String printS()const;

  virtual String asCommands(const String &)const{return "";};
  virtual String toCommands(InsertMenuData::Type,const String &tab="")const{
    return asCommands(tab); };

  virtual bool changed()const{return pchanged;};
  virtual void setChanged(bool c){pchanged=c;};
 protected:
  bool pchanged;
  Point pmousepos;
  Point pobjectpos;
  Size  pdeltasize;
  GraphObjectCommand pupdaterequest;
  long pupdaterequestnumber;
  Graph *pgraph;
 private:
  GraphObjectType pobjecttype;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _GRAPHOBJECT_H */

