/* @(#)IOControl.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _IOCONTROL_H
#define _IOCONTROL_H 1

#include "FGObject.hh" 
#include "Graph.hh" 
#include "Xfp.hh" 
#include "FString.hh" 

class IOControl
{

 public:
  enum AutoScale{None=0,X=1,Y=2,Both=X|Y};


  IOControl()
    :pautoscale(IOControl::Both),pdatatype(FGObject::XY),pcleargroup(false),
    pautocolor(true),pautosymbol(false),ptargetgraphid(Graph::CurrentGraph),
    ptargetgroupid(-1),
    pdefaultylabel(true),
    padjustxtics(true),
    phidexlabel(true),
    peftic(true),
    pautocolorindex(Xfp::Black) {};

  FGObject::DataType dataType()const{return pdatatype;};
  void setDataType(FGObject::DataType dt){pdatatype=dt;};

  long targetGroupId()const{return ptargetgroupid;};
  void setTargetGroupId(long t){ptargetgroupid=t;};

  long targetGraphId()const{return ptargetgraphid;};
  void setTargetGraphId(long t){ptargetgraphid=t;};

  bool autoColor()const{return pautocolor;};
  void setAutoColor(bool a){pautocolor=a;};

  bool clearGroup()const{return pcleargroup;};
  void setClearGroup(bool a){pcleargroup=a;};

  bool autoSymbol()const{return pautosymbol;};
  void setAutoSymbol(bool a){pautosymbol=a;};
    
  AutoScale autoScale()const{return pautoscale;};
  void setAutoScale(AutoScale a){pautoscale=a;};

  bool defaultYLabel()const{return pdefaultylabel;};
  void setDefaultYLabel(bool a){pdefaultylabel=a;};

  bool adjustXTics()const{return padjustxtics;};
  void setAdjustXTics(bool a){padjustxtics=a;};

  bool hideXLabel()const{return phidexlabel;};
  void setHideXLabel(bool a){phidexlabel=a;};

  bool EFTic()const{return peftic;};
  void setEFTic(bool a){peftic=a;};

  long autoColorIndex()const{return pautocolorindex;};
  void setAutoColorIndex(long a){pautocolorindex=a;};

  const String &fileName()const{return pfname;};
  void setFileName(const String &f){pfname=f;};

  static long autoScaleMaxTypes(){return pautoscalemaxtypes;};
  static const char *autoScaleGuiName(long i){
    return pautoscaletypes[i].guiname;};


 private:
  AutoScale pautoscale;
  FGObject::DataType pdatatype;
  bool pcleargroup;
  bool pautocolor;
  bool pautosymbol;
  long ptargetgraphid;
  long ptargetgroupid;
  bool pdefaultylabel;
  bool padjustxtics;
  bool phidexlabel;
  bool peftic;
  long pautocolorindex;

  String pfname;

 private:
  struct tsAutoScaleType{
    long type;
    const  char *guiname;
  };

  static const tsAutoScaleType pautoscaletypes[];
  static const long pautoscalemaxtypes;


};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _IOCONTROL_H */

