/*************************************************************************
 * @(#)LegendBox.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2011
 *************************************************************************
 */

#include "Attributes.hh"
#include "Graph.hh"
#include "ViewBox.hh"
#include "LegendBox.hh"

LegendBox::LegendBox(Graph *parent)
  :GraphObject(parent),pactive(true),ppos(Point(0.98,0.02))
  ,porigin(Point(1.,0.)),pdrawframe(true),pborder(0.8)
  ,prim(new LineStyle()),pfill(new FillStyle())
  ,psymbolwidth(3.),psymbolspacing(.5),plinespacing(0.2)
  ,ptextstyle(new FontStyle())
{
  setObjectType(GraphObject::GOTLegendBox);
  pfill->setFill(true);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
LegendBox::~LegendBox()
{
  if(prim) delete prim;prim=0;
  if(pfill) delete pfill;pfill=0;
  if(ptextstyle) delete ptextstyle;ptextstyle=0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBox::reset()
{
  pactive=true;
  ppos=Point(0.98,0.02);
  psize=0;
  porigin=Point(1.,0.);
  pdrawframe=true;
  pborder=0.8;
  prim->reset();
  pfill->reset();
  psymbolwidth=3.;
  psymbolspacing=.5;
  plinespacing=0.2;
  ptextstyle->reset();
  pfill->setFill(true);
  setChanged(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool LegendBox::changed()const
{
  return pchanged|prim->changed()|pfill->changed()|ptextstyle->changed();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBox::setChanged(bool c)
{
  pchanged=c;
  prim->setChanged(c);
  pfill->setChanged(c);
  ptextstyle->setChanged(c);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphObject::GraphObjectRegion 
LegendBox::insideRegion(const DragDescriptor &dd,bool update)
{
  
  if(!pactive) return GraphObject::GORNone;
  
  const ViewBox *vb=pgraph->viewBox();
  
  Point p=dd.mousepoint-vb->vctodc(ppos)*dd.devicesize;
  Point p0=-porigin*psize;
  Point po=p;

  if(po.insideRect(p0,psize)){
    if(update) pupdaterequest=GOCPosition;
    return GraphObject::GORBody;
  }
  return GraphObject::GORNone;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBox::initDrag(const DragDescriptor &dd)
{
  const ViewBox *vb=pgraph->viewBox();
  pmousepos=dd.mousepoint;
  pobjectpos=vb->vctodc(ppos)*dd.devicesize;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBox::updateDrag(const DragDescriptor &dd)
{
  pobjectpos+=dd.mousepoint-pmousepos;
  pmousepos=dd.mousepoint;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LegendBox::setFromDrag(DragDescriptor &dd)
{
  const ViewBox *vb=pgraph->viewBox();
  setPos(vb->dctovc(pobjectpos/dd.devicesize));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

String LegendBox::asCommands(const String &tab)const
{
  return Sprintf("%s# legend box\n"
		 "%slegend %s\n"
		 "%s"
		 "%s  legend line spacing %g\n"
		 "%s  legend symbol marker spacing %g\n"
		 "%s  legend symbol marker width %g\n"
		 "%s  legend frame %s\n"
		 "%s"
		 "%s  legend frame border spacing %g\n"
		 "%s"
		 "%s  legend position %g , %g\n"
		 "%s  legend origin %g , %g\n"
		 ,tab(),tab(),Xfp::OnOff(pactive)
		 ,ptextstyle->asCommands(tab+"  legend font")()
		 ,tab(),plinespacing,tab(),psymbolspacing,tab(),psymbolwidth
		 ,tab(),Xfp::OnOff(pdrawframe)
		 ,prim->asCommands(tab+"  legend frame rim")()
		 ,tab(),pborder
		 ,pfill->asCommands(tab+"  legend frame fill")()
		 ,tab(),ppos.x,ppos.y
		 ,tab(),porigin.x,porigin.y
		 );
  
}

/*EOF*/ 
