/* @(#)LegendBox.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _LEGENDBOX_H
#define _LEGENDBOX_H 1

#include "GraphObject.hh"
#include "Point.hh"

class LineStyle;
class FillStyle;
class FontStyle;

class LegendBox: public GraphObject
{
 public:
  LegendBox(Graph *parent);
  ~LegendBox();

  void reset();
  
  double border()const{return pborder;};
  void setBorder(double b){pchanged|=(pborder!=b);pborder=b;};

  double symbolWidth()const{return psymbolwidth;};
  void setSymbolWidth(double b){pchanged|=(psymbolwidth!=b);psymbolwidth=b;};

  double symbolSpacing()const{return psymbolspacing;};
  void setSymbolSpacing(double b){pchanged|=(psymbolspacing!=b);psymbolspacing=b;};

  double lineSpacing()const{return plinespacing;};
  void setLineSpacing(double b){pchanged|=(plinespacing!=b);plinespacing=b;};
  
  bool drawFrame()const{return pdrawframe;};
  void setDrawFrame(bool a){pchanged|=(pdrawframe!=a);pdrawframe=a;};

  LineStyle *rim(){return prim;};
  const LineStyle *rim()const{return prim;};

  FillStyle *fill(){return pfill;};
  const FillStyle *fill()const{return pfill;};

  FontStyle *textStyle(){return ptextstyle;};
  const FontStyle *textStyle()const{return ptextstyle;};

  bool active()const{return pactive;};
  void setActive(bool a){pchanged|=(pactive!=a);pactive=a;};

  Point &pos(){return ppos;};
  const Point &pos()const{return ppos;};
  void setPos(const Point &p){pchanged|=(ppos!=p);ppos=p;};

  const Size &size()const{return psize;};
  void setSize(const Size &s){pchanged|=(psize!=s);psize=s;};

  Point &origin(){return porigin;};
  const Point &origin()const {return porigin;};
  void setOrigin(const Point &p){pchanged|=(porigin!=p);porigin=p;};


  virtual GraphObjectRegion insideRegion(const DragDescriptor &dd,bool update);
  virtual void initDrag(const DragDescriptor &dd);
  virtual void updateDrag(const DragDescriptor &dd);
  virtual void setFromDrag(DragDescriptor &dd);

  String asCommands(const String &tab)const;


  bool changed()const;
  void setChanged(bool);

 private:
  bool pactive;			/* inactive legendbox is invisible */
  Point ppos; 	         	/* the position of the box origin 
				   as fraction of the view area */
  Size  psize;			/* only used in interactive mode
				   the postscript device is write only
				   thus we never get the box size in print mode
				 */
  /* --- origin of text box --- */
  Point porigin;            /* the origin of the box in fractions of the size 
				the ul corner is origin 0,0
				the lr corner is origin 1,1
				the origin of the box is the placed at pos with respect to view
			   */
  /* --- frame --- */
  bool pdrawframe;		/* draw a frame around hte box */
  double pborder;  		/* border between legendbox and content
				   as fraction of fontheight
				 */
  LineStyle *prim;		/* the rim style  */
  FillStyle *pfill;		/* the fill style */
  /* --- symbol --- */
  double psymbolwidth;		/* width of marker(linesegment+symbol) as fraction of text height */
  double psymbolspacing;	/* spacing between  symbols and text as fraction of text height */

  /* --- text --- */
  double plinespacing;		/* spacing between entries as fraction 
				   of fontsize */
  FontStyle *ptextstyle;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _LEGENDBOX_H */

