/* @(#)LineShape.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _LINE_H
#define _LINE_H 1


#include "Attributes.hh"
#include "Shape.hh"

class LineStyle;
class FillStyle;

class LineShape: public Shape
{
 public:
  LineShape(Graph *parent,long index);
  virtual ~LineShape();

  virtual void reset();

  const Point &posStart()const{return pposstart;};
  void setPosStart(const Point &p){ pchanged|=(pposstart!=p);pposstart=p;};

  const Point &posEnd()const{return pposend;};
  void setPosEnd(const Point &p){ pchanged|=(pposend!=p);pposend=p;};
  


  // scaled world to dc
  Point startToDC(bool dragging=false)const{
    if(dragging) return posToDC(pposstart,pcoordsystem)+pobjectpos;
    else return posToDC(pposstart,pcoordsystem);
  };
  Point endToDC(bool dragging=false)const{ 
    if(dragging) return posToDC(pposend,pcoordsystem)+pdeltasize;
    else return posToDC(pposend,pcoordsystem);
  };

  LineStyle *lineStyle(){return plinestyle;};
  const LineStyle *lineStyle()const{return plinestyle;};

  FillStyle *fill(){return pfill;};
  const FillStyle *fill()const{return pfill;};

  QRgb arrowHeadFillColor()const{
    return (pextrafillcolor?pfill->color():plinestyle->color());
  };

  void setExtraFillColor(bool e){pchanged|=(pextrafillcolor!=e);
    pextrafillcolor=e;};
  bool extraFillColor()const{return pextrafillcolor;};


  Xfp::ArrowPosition capPosition()const{return pcapposition;};
  void setCapPosition(Xfp::ArrowPosition p1){
    int pp=p1;
    while(pp<0) pp+=Xfp::maxArrowPositions();
    pp=pp%(Xfp::maxArrowPositions());
    pchanged|=(pcapposition!=pp);pcapposition=(Xfp::ArrowPosition)pp;};

  Xfp::ArrowPosition arrowPosition()const{return parrowposition;};
  void setArrowPosition(Xfp::ArrowPosition p1){
    int pp=p1;
    while(pp<0) pp+=Xfp::maxArrowPositions();
    pp=pp%(Xfp::maxArrowPositions());
    pchanged|=(parrowposition!=pp);parrowposition=(Xfp::ArrowPosition)pp;};

  Xfp::ArrowStyle arrowStyle()const{return parrowstyle;};
  void setArrowStyle(Xfp::ArrowStyle p1){
    int pp=p1;
    while(pp<0) pp+=Xfp::maxArrowStyles();
    pp=pp%(Xfp::maxArrowStyles());
    pchanged|=(parrowstyle!=pp);parrowstyle=(Xfp::ArrowStyle)pp;};

  double sharpness()const{return psharpness;};
  void setSharpness(double s){pchanged|=(psharpness!=s);psharpness=s;};


  
  double arrowHeadSize()const{return parrowheadsize;};
  void setArrowHeadSize(double s){pchanged|=(parrowheadsize!=s);parrowheadsize=s;};

  virtual String asCommands(const String &tab)const;
  

  bool changed()const;
  void setChanged(bool c);

  
  virtual GraphObjectRegion insideRegion(const DragDescriptor &dd,bool update);
  virtual void initDrag(const DragDescriptor &dd);
  virtual void updateDrag(const DragDescriptor &dd);
  virtual void setFromDrag(DragDescriptor &dd);

 private:
  Point pposstart;
  Point pposend;
  LineStyle *plinestyle;
  FillStyle *pfill;
  bool pextrafillcolor;
  Xfp::ArrowPosition parrowposition;
  Xfp::ArrowPosition pcapposition;
  Xfp::ArrowStyle parrowstyle;
  double parrowheadsize;
  double psharpness;
  
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _LINE_H */

