/*************************************************************************
 * @(#)PSPaintEngine.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 May 2011
 *************************************************************************
 */

#include <stdlib.h>
#include "Attributes.hh"
#include "PSPaintEngine.hh"
#include "MyMath.hh"
#include "LineShape.hh"
#include "EllipseShape.hh"
#include "Xfp.hh"
#include "Assorted.hh"
#include "Sysinfo.hh"
#include "defs.hh"

#define mRed(x) (((x&0xff0000)>>16)/255.)
#define mGreen(x) (((x&0xff00)>>8)/255.)
#define mBlue(x) (((x&0xff))/255.)

const char *PSPaintEngine::psfonts[]={"Helvetica-ISOLatin1",
				      "Helvetica-Oblique-ISOLatin1",
				      "Symbol"};

const char *PSPaintEngine::poperations[]={"add","sub","mul","div","max","min"};

const char *PSPaintEngine::pvariables[]={
  "FploLegendBoxWidth",
  "FploLegendBoxHeight",
  "FploLegendTextMaxHeight",
  "FploLegendTextMaxWidth",
  "FploSymbolSize",
  "FploMaxSymbolSize",
  "FploLegendSymbolOffset",
  "FploXNormalTicLabelHeight",
  "FploYNormalTicLabelWidth",
  "FploXOpositeTicLabelHeight",
  "FploYOpositeTicLabelWidth",
  "FploTextAscent",
  "FploTextDescent",
  "FploTextHeight",
  "FploTextWidth",
  "FploTextBoxHeight",
  "FploTextBoxWidth",
  "FploSubtitleHeight",
  "FploXOppositeXALabelHeight"
};

const long PSPaintEngine::pmaxvariables
=sizeof(PSPaintEngine::pvariables)/sizeof(char*);

const String PSPaintEngine::pmaindict="\
/N {def} bind def\n\
/B {bind def} bind N\n\
/i {index} B\n\
/m {moveto} B\n\
/rm {rmoveto} B\n\
/l {lineto} B\n\
/rl {rlineto} B\n\
/n {newpath} B\n\
/c {closepath} B\n\
/v {gsave} B\n\
/r {grestore} B\n\
/s {stroke} B\n\
/g {setgray} B\n\
/q {setrgbcolor} B\n\
/f {fill} B\n\
/x {exch} B\n\
/w {setlinewidth} B\n\
/SLC {setlinecap} B\n\
/stringcat {\n\
1 i load\n\
dup length\n\
dup\n\
3 i length add\n\
4 i x string N  % redefine /a length \n\
3 i load  0  4 3 roll putinterval\n\
3 2 roll load x 3 2 roll putinterval\n\
}B\n\
/cvas {% array string cvas --\n\
dup 0 string N\n\
dup dup 500 string cvs N\n\
dup (=[) stringcat\n\
x { 500 string cvs 1 i x stringcat 0 i ( ) stringcat } forall\n\
0 i (]) stringcat\n\
} B\n\
/showStack {\n\
(----------------------START stack) =\n\
pstack\n\
(----------------------END stack) =\n\
} B\n\
/RJ {\n\
 stringwidth neg x neg x\n\
 rm\n\
} B\n\
/CS {\n\
stringwidth\n\
2 div neg x 2 div neg x\n\
rm\n\
} B\n\
/max { 2 copy ge{pop}{x pop}ifelse} B\n\
/min { 2 copy ge{x pop}{ pop}ifelse} B\n\
/SL {\n\
  { x stringwidth pop  max  }repeat\n\
} B\n\
1   setlinecap\n\
\n\
	mark             \n\
	/ISOLatin1Encoding \n\
	  8#000 1 8#054 {StandardEncoding exch get} for \n\
	  /minus \n\
	  8#056 1 8#217 {StandardEncoding exch get} for \n\
	  /dotlessi \n\
	  8#301 1 8#317 {StandardEncoding exch get} for \n\
	  /space /exclamdown /cent /sterling /currency /yen\
 /brokenbar /section \n\
	  /dieresis /copyright /ordfeminine /guillemotleft \
/logicalnot /hyphen \n\
	  /registered /macron /degree /plusminus /twosuperior\
 /threesuperior /acute \n\
	  /mu /paragraph /periodcentered /cedilla /onesuperior\
 /ordmasculine \n\
	  /guillemotright /onequarter /onehalf /threequarters \
/questiondown /Agrave \n\
	  /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE \
/Ccedilla /Egrave /Eacute\n\
	  /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex\
 /Idieresis /Eth /Ntilde\n\
	  /Ograve /Oacute /Ocircumflex /Otilde /Odieresis \
/multiply /Oslash /Ugrave \n\
	  /Uacute /Ucircumflex /Udieresis /Yacute /Thorn \
/germandbls /agrave /aacute \n\
	  /acircumflex /atilde /adieresis /aring /ae /ccedilla\
 /egrave /eacute \n\
	  /ecircumflex /edieresis /igrave /iacute /icircumflex \
/idieresis /eth /ntilde\n\
	  /ograve /oacute /ocircumflex /otilde /odieresis \
/divide /oslash /ugrave \n\
	  /uacute /ucircumflex /udieresis /yacute /thorn \
/ydieresis \n\
	  /ISOLatin1Encoding where not {256 array astore def}\
 if \n\
	 cleartomark \n\
 \n\
	/makeISOEncoded \n\
	{ findfont /curfont exch def \n\
	  /newfont curfont maxlength dict def  \n\
	  /ISOLatin1 (-ISOLatin1) def\n\
	  /curfontname curfont /FontName get dup length \
string cvs def \n\
	  /newfontname curfontname length ISOLatin1 length\
 add string \n\
    	 dup 0                  curfontname putinterval \n\
    	 dup curfontname length ISOLatin1   putinterval \n\
	  def \n\
	  curfont   \n\
	  { exch dup /FID ne  \n\
    	{ dup /Encoding eq  \n\
    	  { exch pop ISOLatin1Encoding exch }  \n\
    	  if  \n\
    	  dup /FontName eq  \n\
    	  { exch pop newfontname exch }  \n\
    	  if  \n\
    	  exch newfont 3 1 roll put  \n\
    	}  \n\
     	{ pop pop }  \n\
    	ifelse  \n\
	  }  \n\
	  forall \n\
	  newfontname newfont definefont \n\
	} def \n\
 \n\
	/Times-Roman makeISOEncoded pop \n\
	/Times-Bold makeISOEncoded pop \n\
	/Times-Italic makeISOEncoded pop \n\
	/Times-BoldItalic makeISOEncoded pop \n\
	/Helvetica makeISOEncoded pop \n\
	/Helvetica-Bold makeISOEncoded pop \n\
	/Helvetica-Oblique makeISOEncoded pop \n\
	/Helvetica-BoldOblique makeISOEncoded pop \n\
<<\n\
/Helvetica-ISOLatin1 [ 0.953  0.285 ]\n\
/Helvetica-Bold-ISOLatin1 [ 0.979  0.307 ]\n\
/Helvetica-Oblique-ISOLatin1 [ 0.979  0.284 ]\n\
/Helvetica-BoldOblique-ISOLatin1 [ 0.953  0.309 ]\n\
/Times-Roman-ISOLatin1 [ 0.924  0.281 ]\n\
/Times-Bold-ISOLatin1 [ 0.96  0.341 ]\n\
/Times-Italic-ISOLatin1 [ 0.924  0.27 ]\n\
/Times-BoldItalic-ISOLatin1 [ 0.964  0.324 ]\n\
/Symbol [ 1.01  0.293 ]\n\
>> /FontMetricsDict x def\n\n\
";

const String PSPaintEngine::pgraphdict="\
/FFS { dup 3 1 roll x findfont x scalefont }B\n\
/FS  { x  dup 3 2 roll  FFS setfont x \n\
FontMetricsDict x get aload pop\n\
x 2 i  mul /ascent x N x mul  /descent x N\n\
} B\n\
/printDebug { \n\
% usage: /symbol printDebug ---\n\
dup /tmpprintDebug 0 N /tmpprintDebug 20 string def tmpprintDebug cvs   \n\
	0 1 0 q /Helvetica-ISOLatin1  20 FS show  (=) show\n\
	load tmpprintDebug cvs \n\
 	show\n\
} B\n\
/textbox { v currentpoint translate 0 0 1 q dup stringwidth pop 0 descent neg 3 2\n\
 roll ascent descent add  rectxywh r }B\n\
/fontmetrics {\n\
currentfont dup /FontBBox get aload pop\n\
4 i /FontMatrix get transform /ascent x N pop\n\
3 2 roll /FontMatrix get transform neg /descent  x N pop\n\
} B\n\
/maxascent  { dup load currentpoint x pop ascent  add max N}B\n\
/maxdescent { dup load currentpoint x pop descent sub neg max N}B\n\
%%  string name addmvsw : add stringwidth of string to name and move\n\
/addmvsw { x stringwidth pop dup 0 rmoveto 1 i load add N}B\n\
%%  factor string name facaddmvsw : add factor*stringwidth of string to name and move\n\
/facaddmvsw  { x stringwidth pop 3 2 roll mul dup 0 rmoveto 1 i load add  N}B\n\
";

const String PSPaintEngine::pprimitivesdict="\
/rectxywh {4 2 roll n m 1 i 0 rl 0 x rl neg 0 rl c s}B\n\
/fillrectxywh {4 2 roll n m 1 i 0 rl 0 x rl neg 0 rl c v q f r {s} {} ifelse}B\n\
/rect {2 i sub x 3 i sub x rectxywh}B\n\
/fillrect {2 i sub x 3 i sub x fillrectxywh}B\
/cliprectxywh {4 2 roll n m 1 i 0 rl 0 x rl neg 0 rl c clip}B\n\
/cliprect {2 i sub x 3 i sub x rectxywh}B\n\
/ellipse{\n\
% create elliptical path, fill or stroke comes separatly\n\
% rotationangle x y w h ellipse\n\
/savematrix matrix currentmatrix N\n\
n 5 2 roll translate neg rotate scale\n\
0 0 1 0 360 arc c \n\
savematrix setmatrix\n\
}B\n\
";

const String PSPaintEngine::psymboldict="\
/SymbolStar{3 1 roll translate 1 1 4 {pop n dup neg 0 m\n\
 dup 0 l c s 45 rotate} for pop}B\n\
/SymbolSquare{3 1 roll translate neg dup neg dup dup neg 5 4 roll {fillrect}{rect}ifelse}B\n\
/SymbolCircle{3 1 roll translate n 0 x 0 x 0 360 arc \n			\
c {v q f r}{} ifelse {s}{}ifelse }B\n\
/SymbolX{3 1 roll translate 45 rotate n dup neg 0 m dup 0 \n\
l dup neg 0 x m 0 x l c {s}{}ifelse}B\n\
/SymbolDiamond{3 1 roll translate 45 rotate 2 sqrt div neg dup neg dup \n\
dup neg 5 4 roll {fillrect}{rect}ifelse}B\n\
/SymbolTriangle{n dup 3 sqrt div 1 i \n\
neg 1 i neg m 1 i 1 i neg l 2 mul x pop 0 x \n\
l c {v q f r}{}ifelse{s}{}ifelse}B\n\
/SymbolTriangleUp{3 1 roll translate SymbolTriangle}B\n\
/SymbolTriangleLeft{3 1 roll translate 90 rotate SymbolTriangle}B\n\
/SymbolTriangleDown{3 1 roll translate 180 rotate SymbolTriangle}B\n\
/SymbolTriangleRight{3 1 roll translate -90 rotate SymbolTriangle}B\n\
/SymbolPlus{3 1 roll translate n dup neg 0 m dup 0 l \n\
dup neg 0 x m 0 x l c {s}{}ifelse}B\n\
";


/* ***********************************************************************
 *
 * ***********************************************************************
 */
PSPaintEngine::PSPaintEngine(const String &filename)
  :PaintEngine(),pfilename(filename),psbufferlen(1000)
{
  pispsengine=true;

  psbuffer=new char[psbufferlen];
  psbuffer[0]='\0';
  pstail=psbuffer;
  

  pscreator="xfplo";
  pstitle="xfplo output";
  psnumpages=1;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PSPaintEngine::~PSPaintEngine()
{
  restore();  
  
  /* end picture */
  restore();
  //  addToBuffer("showpage\n");

  /* print ps_buffer into file */
  FILE *FH;
  FH=openforwrite(pfilename());
  {
    (void)fprintf(FH,
		  "\
%%!PS-Adobe-3.0  EPSF-3.0\n\
%%%%LanguageLevel: 2\n\
%%%%Creator: %s\n\
%%%%Title: %s\n\
%%%%CreationDate: %s\n\
%%%%BoundingBox: %ld %ld %ld %ld\n\
%%%%EndComments\n\
%%%%BeginProlog\n",
		  pscreator(),pstitle(),Sysinfo::getDateAndTime()()
		  ,psbx0,psby0,psbx1,psby1
		  );
    
    for(int i=0;i<psdicts.size();i++){
      fprintf(FH,"%s",psdicts.at(i)());
    }
    fprintf(FH,"%%%%EndProlog\n%%%%BeginSetup\n%%%%EndSetup\nFPLOdict begin\n");
    fprintf(FH,"%s",psbuffer);
    fprintf(FH,"end %% use FPLOdict\n%%%%Trailer\n"
	    "%%%%DocumentFonts: \n%%%%EOF\n");

  }
  fclose(FH);

  /* cleanup */
  if(psbuffer) delete [] psbuffer;psbuffer=0;
}


/* ***********************************************************************
 * adjust psbuffer length
 * ***********************************************************************
 */
void PSPaintEngine::adjustBuffer(long minlen)
{
  const double BUFFERLENINCR=1.2;
  long oldlen=psbufferlen;
  long leng=pstail-psbuffer;
  long newlen=leng+1+minlen+1;
  while(newlen>psbufferlen){
    psbufferlen=MyMath::max<long>(psbufferlen+10,dRound(psbufferlen*BUFFERLENINCR));
  }
  if(oldlen!=psbufferlen){
    char *temp=new char[psbufferlen];
    strcpy(temp,psbuffer);
    char *s=psbuffer;
    psbuffer=temp;
    delete [] s;
    pstail=psbuffer+leng;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::addToBuffer(const char *text)
{ 
  long len=strlen(text); 
  adjustBuffer(len); 
  (void)strcpy(pstail,text);
  pstail+=strlen(pstail); 
}






/* ***********************************************************************
 * set bounding box
 * ***********************************************************************
 */

void PSPaintEngine::setBoundingbox(long x0,long y0,long x1,long y1)
{ 
  psbx0=x0;
  psby0=y0;
  psbx1=x1;
  psby1=y1;
  if(ppaper.orientation()==Paper::Landscape){ 
    psbx0=x0;
    psby0=y0;
    psbx1=x1;
    psby1=y1;
  } 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void PSPaintEngine::page(long /*pagenumber*/)
{
  //  addToBuffer(Sprintf("%%%%Page: %d %d\n",pagenumber,pagenumber));
  save();
  addToBuffer(Sprintf("%f %f scale\n",psscale,psscale));

  //  if(peps){
    if( ppaper.orientation()==Paper::Portrait){
      addToBuffer(Sprintf("0 0 translate\n0 rotate\n0 %f translate\n"
			  ,ppaper.paperHeight()/psscale));
    }else{
      addToBuffer(Sprintf("0 0 translate\n0 rotate\n0 %f translate\n"
			  ,ppaper.paperHeight()/psscale));
    }
    //  }
  addToBuffer("0 0 translate\n");
}

/* ***********************************************************************
 * set dashes
 * ***********************************************************************
 */
void PSPaintEngine::setDash(Xfp::LineStyles style,double width)
{
  long i;
  double d;

  pscurrentlinestyle=style;
  addToBuffer(" ["); 
  const long *dashes=Xfp::lineStyleDashes(style);
  for(i=0;i<dashes[0];i++){
    if(dashes[i+1]==0)break;
    d=dashes[i+1]*width;
    addToBuffer(Sprintf(" %g",d*1)); 
  }
  addToBuffer("] 0 setdash\n");
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::init(const Paper &paper)
{
  bool border=false;
  ppaper=paper;
  psdicts.append("/FPLOdict 72 dict def FPLOdict begin\n");
  psdicts.append(pmaindict);
  psdicts.append(pprimitivesdict);
  psdicts.append(psymboldict);
  psdicts.append(pgraphdict);


  for(int i=0;i<pmaxvariables;i++){
    psdicts.append(Sprintf("0 /%s x N\n",pvariables[i]));
  }
  psdicts.append("end % FPLOdict\n");

  //psscale=0.25;
  psscale=1;
  double paperwidth=ppaper.paperWidth(); 
  double paperheight=ppaper.paperHeight();
  

  double margleft=0.;
  double margbottom=0.;
  double margright=0.;

  double margtop=0.;


  setBoundingbox(lRound(margleft),lRound(margbottom)
		 ,lRound(paperwidth-margright),lRound(paperheight-margtop)); 

  /* paint page 1*/
  page(1);

  pswidth=paperwidth/psscale-0; //todo:
  psheight=paperheight/psscale-0; //todo:
  if(border){ 
    save(); 

    setPen(0x00,Xfp::LSTSolid,1); 
    drawRectFromValue(0.,PaintEngine::RelativeX,
		      0.,PaintEngine::RelativeY,
		      1./2,PaintEngine::RelativeX,
		      1./2,PaintEngine::RelativeY,
		      0x0,false,PaintEngine::ULLR); 

    restore();
  }
  /* picture */
  save();
  setPen(0x00,Xfp::LSTSolid,1);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::defineValue(Variable var,Variable factor,double val)
{
  argvalue(factor,val,1);
  addToBuffer(Sprintf("/%s x N\n",pvariables[var]));  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::defineValue(Variable var,double val,ValueType relarg)
{
  argvalue(val,relarg,1);
  addToBuffer(Sprintf("/%s x N\n",pvariables[var]));  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::argvalue(Variable factor,double val,long sign)
{
  addToBuffer(Sprintf(" %s %g mul ",pvariables[factor],sign*val));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::argvalue(double val,ValueType relarg,long sign)
{
  if(relarg==RelativeX){
    addToBuffer(Sprintf(" %g ",sign*(val*pswidth)));
  }else if(relarg==RelativeY){
    addToBuffer(Sprintf(" %g ",sign*(val*psheight)));
  }else if(relarg==Absolute){
    addToBuffer(Sprintf(" %g ",sign*val));
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double PSPaintEngine::getValue(Variable v,ValueType relarg)
{
  // for debug purpose
  addToBuffer("%DEBUG\n");
  addToBuffer(Sprintf("(%s) =\n",pvariables[v]));
  if(relarg==RelativeX){
    addToBuffer(Sprintf("%s %g mul",pvariables[v],1/pswidth));
  }else if(relarg==RelativeY){
    addToBuffer(Sprintf("%s %g mul",pvariables[v],1/psheight));
  }else if(relarg==Absolute){
    addToBuffer(Sprintf("%s ",pvariables[v]));
  }

  addToBuffer(Sprintf(" =\n"));
  addToBuffer("%ENDDEBUG\n");
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::save()
{
  addToBuffer("v\n");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::restore()
{
  addToBuffer("r\n");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::rotate(double angle)
{
  addToBuffer(Sprintf("%g  rotate\n",angle));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::translate(double x,ValueType relargx,
			      double y,ValueType relargy)
{
  argvalue(x,relargx,1);
  argvalue(y,relargy,-1);
  addToBuffer(" translate\n");

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::translate(double x,ValueType relargx,
			      Variable factory,double y)
{
  argvalue(x,relargx,1);
  argvalue(factory,y,-1);
  addToBuffer(" translate\n");

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::translate(Variable factorx,double x,
			      double y,ValueType relargy)
{
  argvalue(factorx,x,1);
  argvalue(y,relargy,-1);
  addToBuffer(" translate\n");

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::translate(Variable factorx,double x,
			      Variable factory,double y)
{
  argvalue(factorx,x,1);
  argvalue(factory,y,-1);
  addToBuffer(" translate\n");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::moveTo(double x,ValueType relargx,
			   double y,ValueType relargy)
{
  addToBuffer("n ");
  argvalue(x,relargx,1);
  argvalue(y,relargy,-1);
  addToBuffer(" m\n");  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::setPen(QRgb color,Xfp::LineStyles linestyle,
			   double linewidth,Xfp::LineCapStyle capstyle)
{
  addToBuffer(Sprintf("%.4f %.4f %.4f q\n",mRed(color),mGreen(color),
		      mBlue(color)));
  double llinewidth=linewidth*palloverlinewidthPtscale*palloverPtscale/psscale;
  setDash(linestyle,llinewidth);

  addToBuffer(Sprintf("0 SLC %.4f w\n%ld SLC\n",llinewidth,capstyle));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawComment(const String &c)
{
  addToBuffer(Sprintf("%%%%%%%% %s\n",c()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void PSPaintEngine::
// drawRectFromValue(Variable fac1,double val1,
// 		  Variable fac2,double val2,
// 		  Variable fac3,double val3,
// 		  Variable fac4,double val4,
// 		  QRgb fc,bool fill,RectType recttype)
// {
//   const char *recttypes[2]={"","xywh"};
  
//   if(fill){
//     addToBuffer(Sprintf("%s %g %g %g "
// 			,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
// 			,mRed(fc),mGreen(fc),mBlue(fc))); 
//   }
  
//   argvalue(fac1,val1,1);
//   argvalue(fac2,val2,-1);
//   argvalue(fac3,val3,1);
//   argvalue(fac4,val4,-1);
  
//   if(fill){
//     addToBuffer(Sprintf(" fillrect%s\n",recttypes[recttype]));
//   }else{
//     addToBuffer(Sprintf(" rect%s\n",recttypes[recttype]));
//   }
  
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::
drawRectFromValue(double val1,ValueType relarg1,
		  double val2,ValueType relarg2,
		  Variable fac3,double val3,
		  Variable fac4,double val4,
		  QRgb fc,bool fill,RectType recttype)
{
  const char *recttypes[2]={"","xywh"};
  
  if(fill){
    addToBuffer(Sprintf("%s %g %g %g "
			,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
			,mRed(fc),mGreen(fc),mBlue(fc))); 
  }
  
  argvalue(val1,relarg1,1);
  argvalue(val2,relarg2,-1);
  argvalue(fac3,val3,1);
  argvalue(fac4,val4,-1);
  
  if(fill){
    addToBuffer(Sprintf(" fillrect%s\n",recttypes[recttype]));
  }else{
    addToBuffer(Sprintf(" rect%s\n",recttypes[recttype]));
  }
  

  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::
drawRectFromValue(double val1,ValueType relarg1,
		  double val2,ValueType relarg2,
		  double val3,ValueType relarg3,
		  double val4,ValueType relarg4,
		  QRgb fc,bool fill,RectType recttype)
{
  const char *recttypes[2]={"","xywh"};
  
  if(fill){
    addToBuffer(Sprintf("%s %g %g %g "
			,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
			,mRed(fc),mGreen(fc),mBlue(fc))); 
  }
  
  argvalue(val1,relarg1,1);
  argvalue(val2,relarg2,-1);
  argvalue(val3,relarg3,1);
  argvalue(val4,relarg4,-1);
  
  if(fill){
    addToBuffer(Sprintf(" fillrect%s\n",recttypes[recttype]));
  }else{
    addToBuffer(Sprintf(" rect%s\n",recttypes[recttype]));
  }
  

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawLineFromValue(double val1 ,ValueType relarg1,
				      double val2 ,ValueType relarg2,
				      double val3 ,ValueType relarg3,
				      double val4 ,ValueType relarg4,
				      RectType recttype)
{
  const char *recttypes[2]={"l","rlineto"};

  
  addToBuffer(" n ");
  argvalue(val1,relarg1,1);
  argvalue(val2,relarg2,-1);
  addToBuffer(" m ");
  argvalue(val3,relarg3,1);
  argvalue(val4,relarg4,-1);
  addToBuffer(Sprintf(" %s s ",recttypes[recttype]));
  
  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawLineFromValue(double val1 ,ValueType relarg1,
				      double val2 ,ValueType relarg2,
				      Variable fac3,double val3,
				      double val4 ,ValueType relarg4,
				      RectType recttype)
{
  const char *recttypes[2]={"l","rlineto"};

  
  
  addToBuffer(" n ");
  argvalue(val1,relarg1,1);
  argvalue(val2,relarg2,-1);
  addToBuffer(" m ");
  argvalue(fac3,val3,1);
  argvalue(val4,relarg4,-1);
  addToBuffer(Sprintf(" %s s \n",recttypes[recttype]));

  
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawLineRel(double x0,double y0,double x1,double y1
				,RectType recttype)
{
  const char *recttypes[2]={"l","rlineto"};

  
  addToBuffer(Sprintf("n %g %g m %g %g %s s\n" 
		      ,(x0*pswidth)
		      ,-(y0*psheight)
		      ,(x1*pswidth)
		      ,-(y1*psheight),recttypes[recttype]));
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawArrow(double x0,double y0,double x1,double y1
			      ,const LineShape *lsh)
{
  NSF("drawQTLineRel");

  setPen(lsh->lineStyle()->color(),
	 lsh->lineStyle()->style(),
	 lsh->lineStyle()->width());

  Point s((x0*pswidth),(y0*psheight));
  Point e((x1*pswidth),(y1*psheight));

  double w=lsh->arrowHeadSize();
  w/=psscale/palloverPtscale;

  double h=w/sqrt(4.*lsh->sharpness()*lsh->sharpness()-1.);
  Point dd=e-s;
  double ldd=norm(dd);
  dd=dd/ldd;
  Point ort=dd.ortho();
  Point ss=s;
  Point ee=e;

  
  if(lsh->arrowStyle()==Xfp::ARSTClosed){
    if(lsh->arrowPosition() & Xfp::ARPStart){
      ss=s+(dd*w);
    }
    if(lsh->arrowPosition() & Xfp::ARPEnd){
      ee=e-(dd*w);
    }
  }
  
  addToBuffer(Sprintf("%g %g m %g %g l s\n",ss.x,-ss.y,ee.x,-ee.y));



  if(lsh->arrowPosition() & Xfp::ARPStart){
    Point *points=new Point[3];
    points[0]=s+dd*w+ort*h;
    points[1]=s;
    points[2]=s+dd*w-ort*h;
    if(lsh->arrowStyle()==Xfp::ARSTClosed){
      QRgb color=lsh->arrowHeadFillColor();
      addToBuffer(Sprintf("n %g %g m %g %g l %g %g l c v %g %g %g q f r ",
			  points[0].x,-points[0].y,
			  points[1].x,-points[1].y,
			  points[2].x,-points[2].y
			  ,mRed(color),mGreen(color),mBlue(color)));
      setPen(lsh->lineStyle()->color(),
	     lsh->lineStyle()->style(),
	     lsh->lineStyle()->width());
      addToBuffer(" s\n");
    }else{
      addToBuffer(Sprintf("n %g %g m %g %g l %g %g l s\n",
			  points[0].x,-points[0].y,
			  points[1].x,-points[1].y,
			  points[2].x,-points[2].y));
    }
    delete [] points;
  }
  if(lsh->arrowPosition() & Xfp::ARPEnd){
    Point *points=new Point[3];
    points[0]=e-dd*w+ort*h;
    points[1]=e;
    points[2]=e-dd*w-ort*h;
    if(lsh->arrowStyle()==Xfp::ARSTClosed){
      QRgb color=lsh->arrowHeadFillColor();
      addToBuffer(Sprintf("n %g %g m %g %g l %g %g l c v %g %g %g q f r ",
			  points[0].x,-points[0].y,
			  points[1].x,-points[1].y,
			  points[2].x,-points[2].y
			  ,mRed(color),mGreen(color),mBlue(color)));
      setPen(lsh->lineStyle()->color(),
	     lsh->lineStyle()->style(),
	     lsh->lineStyle()->width());
      addToBuffer(" s\n");
    }else{
      addToBuffer(Sprintf("n %g %g m %g %g l %g %g l s\n",
			  points[0].x,-points[0].y,
			  points[1].x,-points[1].y,
			  points[2].x,-points[2].y));
    }
    delete [] points;
  }

  if( (lsh->capPosition() & Xfp::ARPEnd)&&
     !(lsh->arrowPosition() & Xfp::ARPEnd)){
    setPen(lsh->lineStyle()->color(),
	   lsh->lineStyle()->style(),
	   lsh->lineStyle()->width(),Xfp::LCSRound);
    Point c=(ss+ee)*0.5; 
    addToBuffer(Sprintf("%g %g m %g %g l s\n",c.x,-c.y,ee.x,-ee.y));

  }
  if( (lsh->capPosition() & Xfp::ARPStart)&&
     !(lsh->arrowPosition() & Xfp::ARPStart)){
    setPen(lsh->lineStyle()->color(),
	   lsh->lineStyle()->style(),
	   lsh->lineStyle()->width(),Xfp::LCSRound);
    Point c=(ss+ee)*0.5; 
    addToBuffer(Sprintf("%g %g m %g %g l s\n",c.x,-c.y,ss.x,-ss.y));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawEllipse(const EllipseShape *esh)
{
  save();


  Size r=esh->radiiToDC()*Size(pswidth,psheight);
  if(esh->isCircle())r.h=r.w;
  
  Point c=esh->centerToDC()*Size(pswidth,-psheight);
  
  
  
  addToBuffer(Sprintf("%g %g %g %g %g ellipse ",
		      esh->angle(),c.x,c.y,r.w,r.h));
  if(esh->fill()->fill()){
    addToBuffer(Sprintf(" v %g %g %g q f r ",
			mRed(esh->fillColor()),
			mGreen(esh->fillColor()),
			mBlue(esh->fillColor())));
  }
  setPen(esh->lineStyle()->color(),
	 esh->lineStyle()->style(),
	 esh->lineStyle()->width());
  addToBuffer(Sprintf(" s\n"));
  restore();

}

/* ***********************************************************************
 * encaps line drawing for PS device
 * ***********************************************************************
 */
void PSPaintEngine::drawPolyLineRel(const ValueArray<Point> &points)
{
  double x0,y0,x1,y1;
  if(!points.size()) return;

  addToBuffer("n ");  

  x0=points.at(0).x*pswidth;
  y0=points.at(0).y*psheight;

  addToBuffer(Sprintf("%g %g m\n" ,x0,-y0)); 
  
  for(long i=1;i<points.size();i++){

    x1=points.at(i).x*pswidth;
    y1=points.at(i).y*psheight;
    
    addToBuffer(Sprintf("%g %g l " ,x1,-y1)); 

    if(i%5==0){
      addToBuffer("\n");  
    }
    x0=x1;
    y0=y1;
  }
  
  addToBuffer("  s\n");  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawPolyLineRel(const ValueArray<Point> &points,
				    const ValueArray<double> &radii,
				    const ValueArray<bool> &plotsymbol,
				    QRgb fc,double rmin,double rmax,
				    const SymbolStyle &ss,
				    Xfp::WeightsStyle ws)
{
  if(!points.size()) return;
  addToBuffer(Sprintf("v %.4f %.4f %.4f q \n",mRed(fc),mGreen(fc),mBlue(fc)));
  for(long i=0;i<points.size()-1;i++){
    //printf("i=%ld N=%ld\n",i,points.size());
    double r0=radii.at(i);
    r0/=psscale/palloverPtscale;
    r0*=rmax;
    if(r0<0.0)r0=0.0;
    double x0=points.at(i).x*pswidth;
    double y0=points.at(i).y*psheight;
    double r1=radii.at(i+1);
    r1/=psscale/palloverPtscale;
    r1*=rmax;
    if(r1<0.0)r1=0.0;
    if(r0<rmin && r1<rmin) continue;
    double x1=points.at(i+1).x*pswidth;
    double y1=points.at(i+1).y*psheight;
    double d10x=x1-x0;
    double d10y=y1-y0;
    double d10sq=d10x*d10x+d10y*d10y;
    double d10=sqrt(d10sq);
    //todo: if d is very small

    if(ws==Xfp::WESConnected){
      addToBuffer(Sprintf("n %g %g m %g %g l  %g %g l %g %g l c f \n" ,
			  x0-d10y/d10*r0,
			  -(y0+d10x/d10*r0) ,
			  x1-d10y/d10*r1,
			  -(y1+d10x/d10*r1), 
			  x1+d10y/d10*r1,
			  -(y1-d10x/d10*r1), 
			  x0+d10y/d10*r0,
			  -(y0-d10x/d10*r0)
			  )); 
      
      addToBuffer(Sprintf("n %g %g %g 0 360 arc c f\n",x0,-y0,r0));
      if(i==points.size()-2){
	addToBuffer(Sprintf("n %g %g %g 0 360 arc c f\n",x1,-y1,r1));
      }
    }else if(ws==Xfp::WESDots){
      if(plotsymbol.at(i)){
	addToBuffer(Sprintf("n %g %g %g 0 360 arc c f\n",x0,-y0,r0));
      }
      if(i==points.size()-2 && plotsymbol.at(i+1)){
	addToBuffer(Sprintf("n %g %g %g 0 360 arc c f\n",x1,-y1,r1));
      }
    }else if(ws==Xfp::WESIndividual){
      if(( ss.type()!=Xfp::STYNone)){
	if(plotsymbol.at(i)){
	  double lww=r0*Xfp::symbolBaseScale(ss.type());
	  drawSymbol(x0,y0,lww,ss,ss.fillStyle().color(),
		     ss.fillStyle().color());
	}
	if(i==points.size()-2 && plotsymbol.at(i+1)){
	  double lww=r1*Xfp::symbolBaseScale(ss.type());
	  drawSymbol(x1,y1,lww,ss,ss.fillStyle().color(),
		     ss.fillStyle().color());
	}
      }

    }
  }

  addToBuffer("r\n");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawSymbol(double ix,double iy,double lww,
			       const SymbolStyle &ss,
			       QRgb symlincol,QRgb symfilcol)
{
  addToBuffer("v "); 
  //myDebug()<<"ss.lineStyle().style()"<<ss.lineStyle().style();
  setPen(symlincol
	 ,ss.lineStyle().style()
	 ,ss.lineStyle().width()
	 );
  addToBuffer(Sprintf(" %s "
		      ,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
		      )); 
  
  if(Xfp::symbolFillable(ss.type())){
    if(ss.fillStyle().fill()){
      addToBuffer(Sprintf(" %g %g %g true "
			  ,mRed(symfilcol),
			  mGreen(symfilcol),
			  mBlue(symfilcol)));
    }else{
      addToBuffer("false ");
    }
  }
  addToBuffer(Sprintf(" %g %g %g %s r\n",ix,-iy,lww
		      ,Xfp::symbolPSProcedure(ss.type())));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool PSPaintEngine::symbolSetup(const SymbolStyle *symbolStyle,double altsize,
				double &lw)
{
  double w;
  Xfp::SymbolTypes type;

  
  if( !symbolStyle) return false;
  type=symbolStyle->type();
  if( type==Xfp::STYNone) return false;
  if( type==Xfp::STYChar) {
    printf("symbol char not implemented in ps interface!\n");
    exit(1);
    return false;
  }

  /* **** symbol size in font size, be aware of allover scaling **** */
  w=symbolStyle->size();
  if(altsize>0.){
    w=altsize;
  }
  w/=psscale/palloverPtscale;

  /* symbol base width in formulas below is 2*w */
  w*=0.5;

  lw=(w*Xfp::symbolBaseScale(type));
  
  defineValue(SymbolSize,(double)(2.*w),Absolute);


  return true;
}

/* ***********************************************************************
 * encaps symbol drawing for PS device
 * ***********************************************************************
 */

void PSPaintEngine::
drawSymbolFromValue(Variable fac1,double val1,
		    double val2,ValueType relarg2,
		    const FunctionAttribs *at,double altsize)
{
  double lw;
  
  QRgb color;
  bool fill;

  if(!at) return;
  const SymbolStyle &symbol=at->symbolStyle();
  if(!symbolSetup(&symbol,altsize,lw)) return;

  color=at->symbolFillColor();
  fill=symbol.fillStyle().fill();
  

  addToBuffer("v "); 
  setPen(at->symbolLineColor()
	 ,symbol.lineStyle().style()
	 ,symbol.lineStyle().width()
	 );
  addToBuffer(Sprintf(" %s "
		      ,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
		      )); 

  if(Xfp::symbolFillable(symbol.type())){
    if(fill){
      addToBuffer(Sprintf(" %g %g %g true "
			  ,mRed(color),mGreen(color),mBlue(color)));
    }else{
      addToBuffer("false ");
    }
  }
  argvalue(fac1,val1,1);
  argvalue(val2,relarg2,-1);
  
  addToBuffer(Sprintf(" %g %s r\n",lw,Xfp::symbolPSProcedure(symbol.type())));
  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawPolySymbolRel(const ValueArray<Point> &points
				      ,const FunctionAttribs *at)
{
  double lw;
  double x,y;
  QRgb color;
  bool  fill;


 
  if(!points.size()) return;
  if(!at)return;
  const SymbolStyle &symbol=at->symbolStyle();
  if(!symbolSetup(&symbol,-1.,lw)) return;


  color=at->symbolFillColor();
  fill=symbol.fillStyle().fill();
  

  addToBuffer("v\n"); 
  setPen(at->symbolLineColor()
	 ,symbol.lineStyle().style()
	 ,symbol.lineStyle().width()
	 );
    
  for(long i=0;i<points.size();i++){
    
    x=points.at(i).x*pswidth;
    y=points.at(i).y*psheight;


    addToBuffer("v ");
    addToBuffer(Sprintf(" %s "
			,(pscurrentlinestyle==Xfp::LSTNone?"false":"true")
			)); 
    
    if(Xfp::symbolFillable(symbol.type())){
      if(fill){
	addToBuffer(Sprintf(" %g %g %g true "
			    ,mRed(color),mGreen(color),mBlue(color)));
      }else{
	addToBuffer(" false ");
      }
    }
    addToBuffer(Sprintf(" %g %g %g %s r\n",x,-y,lw
			,Xfp::symbolPSProcedure(symbol.type())));
    

  } /* forall points */
  
  addToBuffer("r\n");  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::flushMove(double x,double y,long font,double fontsize)
{

  if(y==0. && x==0.){
    addToBuffer("0 currentpoint x pop neg rm \n");
  }else if(y==0. && x!=0.){
    addToBuffer(Sprintf(
			"/%s %g FS  %g (%s) stringwidth pop mul 0 rm\n"
			,psFont(font)
			,fontsize/psscale*palloverPtscale,x,"X"));
  }else{ 
    addToBuffer(Sprintf("/%s %g FS 0 ascent %g mul neg rm\n"
			,psFont(font)
			,fontsize/psscale*palloverPtscale,y));
  } 
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::arithValue(Variable var,
			       Variable fac2 ,double val2,
			       Variable fac3 ,double val3,
			       Operation op)
{
  
  argvalue(fac2,val2,1);
  argvalue(fac3,val3,1);



  addToBuffer(Sprintf(" %s /%s x N\n" ,poperations[op] ,pvariables[var]));  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::arithValue(Variable var,
			       Variable fac2 ,double val2,
			       double val3 ,ValueType relarg3,
			       Operation op)
{
  
  argvalue(fac2,val2,1);
  argvalue(val3,relarg3,1);



  addToBuffer(Sprintf(" %s /%s x N\n" ,poperations[op] ,pvariables[var]));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::setClippingRectFromValue(double val1 ,ValueType relarg1,
					     double val2 ,ValueType relarg2,
					     double val3 ,ValueType relarg3,
					     double val4 ,ValueType relarg4,
					     RectType recttype)
{
  
  const char *recttypes[2]={"","xywh"};
  

  
  argvalue(val1,relarg1,1);
  argvalue(val2,relarg2,-1);
  argvalue(val3,relarg3,1);
  argvalue(val4,relarg4,-1);
  
  addToBuffer(Sprintf(" cliprect%s\n",recttypes[recttype]));
  
  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::setClipping(bool c)
{
  if(c) 
    save();
  else
    restore();
      
}

/* ***********************************************************************
 * encapsulate the initialization of text plotting
 * ***********************************************************************
 */
void PSPaintEngine::textInit()
{

  addToBuffer(Sprintf("/%s 0 N\n","FploTextWidth"));
  addToBuffer(Sprintf("/%s 0 N\n","FploTextHeight"));
  addToBuffer(Sprintf("/%s 0 N\n","FploTextDescent"));
  addToBuffer(Sprintf("/%s 0 N\n","FploTextAscent"));

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::textClean()
{ 
  addToBuffer(" FploTextAscent FploTextDescent add /FploTextHeight x N\n");

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::initCalcTextWidth()
{
  addToBuffer("0 0 m\n");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::metricMove(double x,double y,long font,double fontsize)
{

  if(y==0. && x==0.){
    addToBuffer("0 currentpoint x pop neg rm \n");
  }else if(y==0. && x!=0.){
    addToBuffer(Sprintf(
			"/%s %g FS  %g (%s) /FploTextWidth facaddmvsw\n"
			,psFont(font)
			,fontsize/psscale*palloverPtscale,x,"X"));

  }else{ 
    addToBuffer(Sprintf("/%s %g FS 0 ascent %g mul neg rm\n"
			,psFont(font)
			,fontsize/psscale*palloverPtscale,y));
  } 
  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::adjustText(Xfp::TextOrientation just)
{

  
  if(just & Xfp::TOCenter){ 
    addToBuffer("FploTextWidth 2 div neg 0 translate\n");
  }else if(just & Xfp::TORight){ 
    addToBuffer("FploTextWidth  neg 0 translate\n");
  } 
  if(just & Xfp::TOTop){ 
    addToBuffer("0 FploTextAscent neg  translate\n");
  }else if(just & Xfp::TOBottom){ 
    addToBuffer("0 FploTextDescent     translate\n");
  }else if(just & Xfp::TOVCenter){ 
    addToBuffer("0 FploTextAscent FploTextDescent sub 2 div neg  translate\n");
  } 
  
  
  
  addToBuffer("0 0 m\n");

#ifdef TEXTBOX_DEBUG   
  save();
  setPen(PSRGBColors[PS_GREEN],PS_SOLIDLINE,1);
  drawRectFromValue(0,0.,Absolute
		      ,"FploTextDescent",1.,Absolute
		      ,"FploTextWidth",1.,Absolute
		      ,"FploTextAscent",-1.0,Absolute
		      ,0,0,GRAPH_RECT_ULLR); 

  restore();
#endif


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void escapePS(char *buffer,const char *text)
{
  char *p=buffer;
  long i;
  i=0;
  while(text[i] !='\0'){
    if(text[i]=='(' || text[i]==')' || text[i]=='\\'){
      *p++='\\';
      *p++=text[i];
    }else{
      *p++=text[i];
    }
    i++;
  }
  *p='\0';
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::flushText(const char *text,long font,double fontsize)
{
  char buffer[30000];

  escapePS(buffer,text);  
  

  addToBuffer(Sprintf("/%s   %g FS (%s) show \n" 
		      ,psFont(font)
		      ,fontsize/psscale*palloverPtscale,buffer));
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::metricText(const char *text,long font,double fontsize)
{
  char buffer[30000];

  escapePS(buffer,text);  

  addToBuffer(Sprintf(
		      "/%s %g FS /FploTextAscent maxascent "
		      "/FploTextDescent maxdescent\n"
		      "(%s)"
#ifdef TEXTBOX_DEBUG_TEXTBOX
		      " textbox "
#endif
#ifdef TEXTBOX_DEBUG_SHOW
		      "                   v dup 1 0 1 q show r" 
#endif
		      " /FploTextWidth addmvsw\n "
#ifdef TEXTBOX_DEBUG_BOXES
		      " v 2 w 1 0 0 q 0 FploTextDescent neg  FploTextWidth FploTextAscent  FploTextDescent add rectxywh r " 
#endif

		      ,psFont(font)
		      ,fontsize/psscale*palloverPtscale,buffer));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PixMap *PSPaintEngine::createPixMap(int /*width*/,int /*height*/)
{
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PSPaintEngine::drawPixMap(PixMap *,double /*val1*/ ,ValueType /*relarg1*/,
			       double /*val2*/ ,ValueType /*relarg2*/,
			       double /*val3*/ ,ValueType /*relarg3*/,
			       double /*val4*/ ,ValueType /*relarg4*/,
			       RectType /*recttype*/)
{
}



/*EOF*/ 
