/* @(#)PSPaintEngine.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PSPAINTENGINE_H
#define _PSPAINTENGINE_H 1

#include "PaintEngine.hh"
#include "Xfp.hh" 
#include "FString.hh" 

class SymbolStyle;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class PSPaintEngine:public PaintEngine{
 public:
  PSPaintEngine(const String &filename);
  ~PSPaintEngine();

  virtual void init(const Paper&);

  virtual void defineValue(Variable var,double val,ValueType relarg);
  virtual void defineValue(Variable var,Variable factor,double val);
  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  Variable fac3 ,double val3,
			  Operation op);
  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  double val3 ,ValueType relarg3,
			  Operation op);
  virtual double getValue(Variable,ValueType);

  virtual void save();
  virtual void restore();
  virtual void translate(double x,ValueType relargx,
			 double y,ValueType relargy);
  virtual void translate(double x,ValueType relargx,
			 Variable factory,double y);
  virtual void translate(Variable factorx,double x,
			 double y,ValueType relargy);
  virtual void translate(Variable factorx,double x,
			 Variable factory,double y);
  virtual void rotate(double angle);
  virtual void moveTo(double x,ValueType relargx,
 		      double y,ValueType relargy);
  virtual void setPen(QRgb,Xfp::LineStyles,double,
		      Xfp::LineCapStyle cs=Xfp::LCSButt);

  virtual void drawComment(const String &);
  
/*   virtual void drawRectFromValue(Variable fac1,double val1, */
/* 				 Variable fac2,double val2, */
/* 				 Variable fac3,double val3, */
/* 				 Variable fac4,double val4, */
/* 				 QRgb fc,bool fill,RectType recttype); */
  virtual void drawRectFromValue(double val1,ValueType relarg1,
				 double val2,ValueType relarg2,
				 Variable fac3,double val3,
				 Variable fac4,double val4,
				 QRgb fc,bool fill,RectType recttype);
  virtual void drawRectFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 QRgb fc,bool fill,RectType recttype);


  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 RectType recttype);
  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 Variable fac3,double val3,
				 double val4 ,ValueType relarg4,
				 RectType recttype);


  virtual void drawSymbolFromValue(Variable fac1,double val1,
				   double val2,ValueType relarg2,
				   const FunctionAttribs *at,double altsize);


  virtual void drawPolyLineRel(const ValueArray<Point> &points);
  virtual void drawPolyLineRel(const ValueArray<Point> &,
			       const ValueArray<double> &,
			       const ValueArray<bool> &,QRgb,
			       double min,double max,
			       const SymbolStyle &ss,
			       Xfp::WeightsStyle ws);
  virtual void drawPolySymbolRel(const ValueArray<Point> &points
				 ,const FunctionAttribs *at);

  virtual void drawLineRel(double x0,double y0,double x1,double y1
			   ,RectType recttype);

  virtual void drawArrow(double x0,double y0,double x1,double y1
			 ,const LineShape *);
  virtual void drawEllipse(const EllipseShape *);

  virtual PixMap *createPixMap(int width,int height);
  virtual void drawPixMap(PixMap *,double val1 ,ValueType relarg1,
			  double val2 ,ValueType relarg2,
			  double val3 ,ValueType relarg3,
			  double val4 ,ValueType relarg4,RectType recttype);

  virtual void setClippingRectFromValue(double val1 ,ValueType relarg1,
					double val2 ,ValueType relarg2,
					double val3 ,ValueType relarg3,
					double val4 ,ValueType relarg4,
					RectType recttype);
  virtual void setClipping(bool);

  //text
  virtual void textInit();
  virtual void textClean();
  virtual void initCalcTextWidth();
  virtual void metricText(const char *text,long font,double fontsize);
  virtual void flushText(const char *text,long font,double fontsize);
  virtual void metricMove(double x,double y,long font,double fontsize);
  virtual void flushMove(double x,double y,long font,double fontsize);
  virtual void adjustText(Xfp::TextOrientation just);


 private:
  void argvalue(Variable factor,double val,long sign);
  void argvalue(double val,ValueType relarg,long sign);
  bool symbolSetup(const SymbolStyle *symbolStyle,double altsize,double &lw);

  void adjustBuffer(long minlen);
  void addToBuffer(const String &s){addToBuffer(s());};
  void addToBuffer(const char *text);
  void setBoundingbox(long x0,long y0,long x1,long y1);
  void page(long pagenumber);
  void setDash(Xfp::LineStyles style,double width);
  const char *psFont(int i)const{return psfonts[i];};
  void drawSymbol(double ix,double iy,double lww,const SymbolStyle &ss,
		  QRgb symlincol,QRgb symfilcol);  
  
 private:
  String pfilename;
  static const char *pvariables[];
  static const long pmaxvariables;
  static const char *poperations[];
  static const String pmaindict;
  static const String pprimitivesdict;
  static const String psymboldict;
  static const String pgraphdict;
  static const char *psfonts[];

  char *psbuffer;
  long  psbufferlen;
  char *pstail;
  String pscreator;
  String pstitle;
  long psnumpages;
  long psbx0,psby0,psbx1,psby1;
  double psscale;
  double pswidth,psheight;
  Xfp::LineStyles pscurrentlinestyle;
  ValueArray<String> psdicts;
};
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PSPAINTENGINE_H */

