/*************************************************************************
 * @(#)Page.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   16 Jun 2011
 *************************************************************************
 */

#include <stdio.h>
#include <QPrinter>
#include <QPainter>
#include "Assorted.hh"
#include "Except.hh"
#include "Page.hh"
#include "Attributes.hh"
#include "MyProgressDialog.hh"
#include "Graph.hh"
#include "GraphAxis.hh"
#include "Graphs.hh"
#include "Shape.hh"
#include "FGObject.hh"
#include "Function.hh"
#include "Functions.hh"
#include "FunctionGroup.hh"
#include "FunctionGroups.hh"
#include "TextBox.hh"
#include "LegendBox.hh"
#include "TicMarks.hh"
#include "PSPaintEngine.hh"
#include "World.hh"
#include "ViewBox.hh"
#include "WeightLabels.hh"
#include "WeightSelection.hh"
#include "MyDebug.hh"
#include "Config.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
Page::Page()
  :pgraphs(new Graphs(this)),
   pweightLabelsCollector(new WeightLabelsCollector()),
   p_knowprintname(false),p_printfilename("")
{
  
  isdirty1=false;

  ppaper.setPaperSize(Paper::A4);
  ppaper.setOrientation(Paper::Landscape);
  //  orientation=Paper::Portrait;
  pgraphs->reset();



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Page::~Page()
{
  //printf("Page dying %p\n",this);
  if (pgraphs) delete pgraphs; pgraphs=0;
  if (pweightLabelsCollector){
    delete pweightLabelsCollector; pweightLabelsCollector=0;}
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::reset(bool createOneGraph)
{
  ppaper.setPaperSize(Paper::A4);
  ppaper.setOrientation(Paper::Landscape);
  pmousebuffer="";

  pgraphs->reset(createOneGraph);

  isdirty1=false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::plotDragObject(PaintEngine *pe,GraphObject *go)
{
  if(!go->graph()){
    myDebug()<<"go->graph()==0 ?";
    return;
  };
  go->graph()->plotDragObject(pe,go);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::getObjectsAt(PtrArray<GraphObject> &objects,const DragDescriptor &dd,
			GraphObject::GraphObjectType got)
{
  for(int ig=0;ig<pgraphs->size();ig++){
    pgraphs->at(ig)->getObjectsAt(objects,dd,got);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::plot(PaintEngine *pe)
{
  for(int i=0;i<pgraphs->size();i++){
    pgraphs->at(i)->plotGraph(pe);
  }
  for(int i=0;i<pgraphs->size();i++){
    pgraphs->at(i)->plotGraphLegend(pe);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::write(const String &fname)
{
  isdirty1=false;
  FILE *FHO=NULL;

  FHO=openforwrite(fname);
  fprintf(FHO,"Xfpl file version \"%s\"\n",VERSION());
  fprintf(FHO,"kill all\n");
  fprintf(FHO,"%s",ppaper.asCommands("")());

  long maxprogress=0;
  for(long ig=0;ig<pgraphs->size();ig++){
    for(long igr=0;igr<pgraphs->at(ig)->groups()->size();igr++){
      maxprogress+=pgraphs->at(ig)->groups()->at(igr)->funcs()->size();
    }
  }
  MyProgressDialog prgdlg(Sprintf("Writing %s...",fname()),maxprogress);


  String tab="";
  for(long ig=0;ig<pgraphs->size();ig++){
    tab="  ";
    Graph *g=pgraphs->at(ig);
    fprintf(FHO,"with g%ld\n  %s\n",g->ID()+1,Xfp::OnOff(g->active()));
    fprintf(FHO,"%s graph point size scale %g\n",tab(),g->allOverPointScale());
    fprintf(FHO,"%s graph line width scale %g\n",tab(),g->allOverLineWidthScale());
    fprintf(FHO,"%s",g->viewBox()->asCommands(tab)());
    fprintf(FHO,"%s",g->world()->asCommands(tab)());
    fprintf(FHO,"%s",g->axes()->asCommands(tab)());
    fprintf(FHO,"%s",g->legendBox()->asCommands(tab)());
    fprintf(FHO,"%s",g->textBoxes()->asCommands(tab)());
    fprintf(FHO,"%s",g->ticMarks()->asCommands(tab)());
    fprintf(FHO,"%s",g->shapes()->asCommands(tab)());
    //todo weight
    
    for(long igr=0;igr<g->groups()->size();igr++){
      FunctionGroup *group=g->groups()->at(igr);
      group->writeAsCommands(FHO,tab,prgdlg);
    }

  }
  
  pweightLabelsCollector->writeAsCommand(FHO,"");

  fclose(FHO);
  p_knowprintname=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::printPS()
{
  try{
    PSPaintEngine *pe=new PSPaintEngine(p_printfilename);
    pe->init(ppaper);
    plot(pe);
    delete (pe);

    if(0){
      // Setup printer
      QPrinter printer(QPrinter::HighResolution);
      printer.setOutputFormat(QPrinter::PostScriptFormat);
      printer.setOutputFileName(p_printfilename());
      printer.setFullPage(true);
      //printer.setPageSize(QPrinter::A4);
      printer.setPageSize(QPrinter::Custom);
      printer.setOrientation(QPrinter::Landscape);
      printer.setPaperSize(QSizeF(ppaper.paperWidth(),ppaper.paperHeight())
 			   ,QPrinter::Point);
      printer.setPageMargins(0,0,0,0,QPrinter::Millimeter);
      
      // Get the painting context from the printer
      QPainter painter;
      if( !painter.begin(&printer)){
	throw Except("");}; // Destructing painter writes the pdf file
      
      // Set the scale, so we can draw in millimeter
      double scale = printer.width()/ppaper.paperWidth();
      painter.setTransform(QTransform(scale,0,0, 0,scale,0, 0,0,1.0));
      
      // Draw the image
//       QRectF src(0,0,image.width(),image.height());
//       QRectF dst(imageOffsetX,imageOffsetY,imageWidth,imageHeight);
//       painter.drawImage(dst,image,src);
      
      // Draw the markings
      QPen pen;
      double m_lineWidth=0.05;
      pen.setWidthF(m_lineWidth);
      pen.setColor(Qt::red);
      QVector<qreal> dashes;
      dashes << 10/m_lineWidth << 5/m_lineWidth;
      pen.setDashPattern(dashes);
      painter.setPen(pen);
      int m_pdfMargin=10;
      int productionWidth=100;
      int productionHeight=200;
      painter.drawRect(QRectF(m_pdfMargin-m_lineWidth/2,
			      m_pdfMargin-m_lineWidth/2,
			      productionWidth+m_lineWidth,
			      productionHeight+m_lineWidth));
    }
    
    
  }catch(Except &ex){
    throw Except(Sprintf("Could not print to file '%s'!\n%s"
			 ,p_printfilename(),ex.msg()));
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::updateWeightLabelReferences()
{
  for(long ig=0;ig<pgraphs->size();ig++){
    Graph *g=pgraphs->at(ig);
    for(long igr=0;igr<g->groups()->size();igr++){
      FunctionGroup *group=g->groups()->at(igr);
      for(long ifu=0;ifu<group->funcs()->size();ifu++){
	Function *func=group->funcs()->at(ifu);
	func->setWeightLabelByReference();
	
      }
    }
  }
  pweightLabelsCollector->collectGarbage();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::normalizeWeightSelections()
// Make sure that no function has invalid (dangling) weight selections, which
// do not refer to anything.
// And make sure that the plot order of weights is reflected in the data 
// structures.
{
  for(long ig=0;ig<pgraphs->size();ig++){
    Graph *g=pgraphs->at(ig);
    for(long igr=0;igr<g->groups()->size();igr++){
      FunctionGroup *group=g->groups()->at(igr);
      WeightSelections *wss=group->weightSelections();
      if(wss) wss->removeUndefined();
      if(wss) wss->resortAccordingToPlotOrder();
      for(long ifu=0;ifu<group->funcs()->size();ifu++){
	Function *func=group->funcs()->at(ifu);
	WeightSelections *wss=func->weightSelections();
	if(wss) wss->removeUndefined();
	if(wss) wss->resortAccordingToPlotOrder();
	
      }
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::import(const String &/*filename*/,Xfp::FileType /*type*/,
		  long /*targetgraph*/,long /*targetgroup*/)
{}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Page::pushCurrentWorld()
{
  Graph *gr=pgraphs->currentGraph();
  if(!gr) return;
  World *w=gr->world();
  pstack.append(WorldWindow(w->w0(),w->w1(),gr->ID()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Page::refreshPaperSize(const Paper &oldpaper,bool )
{
  for(long ig=0;ig<pgraphs->size();ig++){
    Graph *g=pgraphs->at(ig);
    Point pos=g->viewBox()->pos();
    Size size=g->viewBox()->size();
    double w=ppaper.paperWidth();
    double h=ppaper.paperHeight();
    double ow=oldpaper.paperWidth();
    double oh=oldpaper.paperHeight();
    g->viewBox()->setPos(Point(pos.x*ow/w,pos.y*oh/h));
    g->viewBox()->setSize(Size(size.w*ow/w,size.h*oh/h));
  }  
}

/*EOF*/ 
