/* @(#)Page.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PAGE_H
#define _PAGE_H 1

#include "Xfp.hh"
#include "PaintEngine.hh"
#include "GraphObject.hh"
#include "WeightLabels.hh"
#include "Paper.hh"
#include "PtrArray.hh"
#include "ValueArray.hh" 

class PaintEngine;
class GraphObject;
class Graphs;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class WorldWindow{
 public:
  WorldWindow(){};
  WorldWindow(const Point &w0,const Point &w1,long graphid)
    :ppw0(w0),ppw1(w1),pgraphid(graphid){};

    inline long graphId()const{return pgraphid;};

    inline const Point &w0()const{return ppw0;};
    inline const Point &w1()const{return ppw1;};

 private:
    Point ppw0;
    Point ppw1;
    long pgraphid;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Page{
 public:
  Page();
  ~Page();


  Graphs *graphs(){return pgraphs;};
  const Graphs *graphs()const {return pgraphs;};
  

  
  void reset(bool createOneGraph=true);
  const Paper &paper()const{return ppaper;};
  Paper &paper(){return ppaper;};
  void refreshPaperSize(const Paper &oldpaper,bool );

  void write(const String &fname);

  WeightLabelsCollector *weightLabelsCollector(){return pweightLabelsCollector;};
  void updateWeightLabelReferences();
  void normalizeWeightSelections();
  
  void setCursor(const Point &c){pcursor=c;};
  const Point &cursor()const {return pcursor;};
 
  void printPS();
  const String printFileName()const{return p_printfilename;};
  bool knowPrintFileName()const{return p_knowprintname;};
  void setPrintFileName(const String &fn){
    p_printfilename=fn;
    p_knowprintname=true;
  };

  void pushCurrentWorld();
  
  // return the hook for mouse clicked
  // to do something with the local coordinates
  const String &mouseHookBuffer()const {return pmousebuffer;};
  void setMouseHookBuffer(const String *b){pmousebuffer=*b;};
  
  const ValueArray<WorldWindow> &worldStack()const{return pstack;};
  void clearWorldStack(){pstack.clear();};

  void import(const String &filename,Xfp::FileType type,
	      long targetgraph,long targetgroup);

  void plot(PaintEngine *pe);
  void plotDragObject(PaintEngine *pe,GraphObject *go);

  void getObjectsAt(PtrArray<GraphObject> &objects,const DragDescriptor &dd,
		    GraphObject::GraphObjectType got=GraphObject::GOTNone);


 public:
  bool isdirty1;


 private:
  
  Graphs *pgraphs;
  WeightLabelsCollector *pweightLabelsCollector;
  //
  bool p_knowprintname;
  String p_printfilename;
  Paper ppaper;

  ValueArray<WorldWindow> pstack;
  
  String pmousebuffer;
  Point pcursor;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PAGE_H */

