
/* @(#)PaintEngine.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PAINTENGINE_H
#define _PAINTENGINE_H 1

#include "MyQRgb.hh" 
#include "Point.hh" 
#include "Paper.hh" 
#include "Xfp.hh" 
#include "ValueArray.hh" 

class FunctionAttribs;
class SymbolStyle;
class LineShape;
class EllipseShape;



class PixMap{
 public:
  // the user must take care of deletion of the image data object
  PixMap(){};
  virtual ~PixMap(){};
  virtual void setPixel(int,int,QRgb)=0;
 private:
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class PaintEngine{
 public:


  enum ValueType{Absolute=0,RelativeX=1,RelativeY=2};
  enum RectType{ULLR=0,ULWH=1};

  enum Variable{
    LegendBoxWidth=0,
    LegendBoxHeight=1,
    LegendTextMaxHeight=2,
    LegendTextMaxWidth=3,
    SymbolSize=4,
    MaxSymbolSize=5,
    LegendSymbolOffset=6,
    XNormalTicLabelHeight=7,
    YNormalTicLabelWidth=8,
    XOpositeTicLabelHeight=9,
    YOpositeTicLabelWidth=10,
    TextAscent=11,
    TextDescent=12,
    TextHeight=13,
    TextWidth=14,
    TextBoxHeight=15,
    TextBoxWidth=16,
    SubtitleHeight=17,
    XOppositeXALabelHeight=18,
  };
  enum Operation{
    Plus= 0,
    Minus=1,
    Mult= 2,
    Div=  3,
    Max=  4,
    Min=  5
  };


  PaintEngine();
  virtual ~PaintEngine();

  bool isPSEngine(){return pispsengine;};

  const Paper &paper()const{return ppaper;};

  virtual void init(const Paper &)=0;

  void setAllOverPointScale(double s){palloverPtscale=s;};
  void setAllOverLineWidthScale(double s){palloverlinewidthPtscale=s;};

  virtual void defineValue(Variable var,double val,ValueType relarg)=0;
  virtual void defineValue(Variable var,Variable factor,double val)=0;
  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  Variable fac3 ,double val3,
			  Operation op)=0;
  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  double val3 ,ValueType relarg3,
			  Operation op)=0;
  virtual double getValue(Variable var,ValueType relarg)=0;

  virtual void save()=0;
  virtual void restore()=0;

  virtual void translate(double x,ValueType relargx,
			 double y,ValueType relargy)=0;
  virtual void translate(double x,ValueType relargx,
			 Variable factory,double y)=0;
  virtual void translate(Variable factorx,double x,
			 double y,ValueType relargy)=0;
  virtual void translate(Variable factorx,double x,
			 Variable factory,double y)=0;

  virtual void rotate(double angle)=0;

  virtual void moveTo(double x,ValueType relargx,
 		      double y,ValueType relargy)=0;
  void moveTo(double x,double y){moveTo(x,Absolute,y,Absolute);};

  virtual void setPen(QRgb,Xfp::LineStyles,double,
		      Xfp::LineCapStyle cs=Xfp::LCSButt)=0;

  virtual void drawComment(const String &)=0;

/*   virtual void drawRectFromValue(Variable fac1,double val1, */
/* 				 Variable fac2,double val2, */
/* 				 Variable fac3,double val3, */
/* 				 Variable fac4,double val4, */
/* 				 QRgb fc,bool fill,RectType recttype)=0; */
  virtual void drawRectFromValue(double val1,ValueType relarg1,
				 double val2,ValueType relarg2,
				 Variable fac3,double val3,
				 Variable fac4,double val4,
				 QRgb fc,bool fill,RectType recttype)=0;
  virtual void drawRectFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 QRgb fc,bool fill,RectType recttype)=0;
  void drawRectFromValue(double val1,double val2,double val3,double val4,
			 QRgb fc,bool fill,RectType recttype){
    drawRectFromValue(val1,Absolute,val2,Absolute,val3,Absolute,val4,Absolute
		      ,fc,fill,recttype);
  };


  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 RectType recttype)=0;
  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 Variable fac3,double val3,
				 double val4 ,ValueType relarg4,
				 RectType recttype)=0;
  void drawLineFromValue(double val1,double val2,double val3,double val4,
			 RectType recttype){
    drawLineFromValue(val1,Absolute,val2,Absolute,val3,Absolute,val4,Absolute
		      ,recttype);};


  virtual void drawSymbolFromValue(Variable fac1,double val1,
				   double val2,ValueType relarg2,
				   const FunctionAttribs *at,double altsize)=0;
  virtual void drawPolyLineRel(const ValueArray<Point> &points)=0;
  virtual void drawPolyLineRel(const ValueArray<Point> &points,
			       const ValueArray<double> &radii,
			       const ValueArray<bool> &plotsymbool,
			       QRgb fc,
			       double min,double max,
			       const SymbolStyle &ss,
			       Xfp::WeightsStyle ws)=0;
  virtual void drawPolySymbolRel(const ValueArray<Point> &points
				 ,const FunctionAttribs *at)=0;
  virtual void drawLineRel(double x0,double y0,double x1,double y1
			   ,RectType recttype)=0;

  virtual void drawArrow(double x0,double y0,double x1,double y1,
			 const LineShape *)=0;
  virtual void drawEllipse(const EllipseShape *)=0;

  virtual PixMap *createPixMap(int width,int height)=0;
  virtual void drawPixMap(PixMap *,double val1 ,ValueType relarg1,
			  double val2 ,ValueType relarg2,
			  double val3 ,ValueType relarg3,
			  double val4 ,ValueType relarg4,RectType recttype)=0;

  virtual void setClippingRectFromValue(double val1 ,ValueType relarg1,
					double val2 ,ValueType relarg2,
					double val3 ,ValueType relarg3,
					double val4 ,ValueType relarg4,
					RectType recttype)=0;


  virtual void setClipping(bool)=0;
  
  //text
  virtual void textInit()=0;
  virtual void textClean()=0;
  virtual void initCalcTextWidth()=0;
  virtual void metricText(const char *text,long font,double fontsize)=0;
  virtual void flushText(const char *text,long font,double fontsize)=0;
  virtual void metricMove(double x,double y,long font,double fontsize)=0;
  virtual void flushMove(double x,double y,long font,double fontsize)=0;
  virtual void adjustText(Xfp::TextOrientation just)=0;


  //specialized methods for on screen display
  virtual void drawSpringFromValue(double,ValueType,
				   double,ValueType,
				   double,ValueType,
				   double,ValueType,
				   RectType){};
  virtual void setDrag(bool ){};

  virtual Size deviceSize(){return Size();};

  void setDebugMode(const String &m=""){pdebugmode=m;};

 protected:
  bool pispsengine;
  Paper ppaper;
  String pdebugmode;
  double palloverPtscale;
  double palloverlinewidthPtscale;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PAINTENGINE_H */

