/* @(#)Paper.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PAPER_H
#define _PAPER_H 1

#include "FString.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Paper{
 public:
  Paper():pchanged(false),ppapersize(A4),porientation(Landscape),
    pcustomwidth(500),pcustomheight(500){};

  enum Orientation{Portrait=0,Landscape=1};
  enum Size{ Custom=-1,Letter=0,LetterSmall=1,Legal=2,Statement=3,Tabloid=4,
	     Ledger=5,
	     Folio=6,Quarto=7,_7x9=8,_9x11=9,_9x12=10,_10x13=11,_10x14=12,
	     Executive=13,A0=14,A1=15,A2=16,A3=17,A4=18,A5=19,A6=20,A7=21,
	     A8=22,A9=23,A10=24,B0=25,B1=26,B2=27,B3=28,B4=29,B5=30,B6=31,
	     B7=32,B8=33,B9=34,B10=35,ISOB0=36,ISOB1=37,ISOB2=38,ISOB3=39,
	     ISOB4=40,ISOB5=41,ISOB6=42,ISOB7=43,ISOB8=44,ISOB9=45,ISOB10=46,
	     C0=47,C1=48,C2=49,C3=50,C4=51,C5=52,C6=53,C7=54,DL=55
};

  Paper &operator=(const Paper &other){
    pchanged=other.pchanged;
    ppapersize=other.ppapersize;
    porientation=other.porientation;
    pcustomwidth=other.pcustomwidth;
    pcustomheight=other.pcustomheight;
    return *this;
  };

  Size paperSize()const{return ppapersize;};
  void setPaperSize(Size s){pchanged|=(ppapersize!=s);ppapersize=s;};

  Orientation orientation()const{return porientation;};
  void setOrientation(Orientation o){pchanged|=(porientation!=o);porientation=o;};

  long customWidth()const{return pcustomwidth;};
  void setCustomWidth(long w){pchanged|=(pcustomwidth!=w);pcustomwidth=w;};
  long customHeight()const{return pcustomheight;};
  void setCustomHeight(long h){pchanged|=(pcustomheight!=h);pcustomheight=h;};
  
  static const char *paperSizeName(Size i){
    if(i==Custom)return "Custom";
    return pspapersizes[i];
  };

  static const char *orientationTypeName(Orientation i){
    return psorienttypes[i];};

  long paperWidth()const{
    if(ppapersize==Custom) return pcustomwidth;
    if(porientation==Paper::Portrait)
      return pspaperwidth[ppapersize];
    else
      return pspaperheight[ppapersize];
  };

  long paperHeight()const{
    if(ppapersize==Custom) return pcustomheight;
    if(porientation==Paper::Portrait)
      return pspaperheight[ppapersize];
    else
      return pspaperwidth[ppapersize];
  };

  String asCommands(const String &tab)const;

  
  bool changed()const;
  void setChanged(bool c);
    
 private:
  bool pchanged;
  Size ppapersize;
  Orientation porientation;
  long pcustomwidth;
  long pcustomheight;
  static const char *pspapersizes[];
  static const char *psorienttypes[];
  static const long pspaperwidth[];
  static const long pspaperheight[];

  
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PAPER_H */

