/*************************************************************************
 * @(#)ParameterList.cpp
 * author: magru@unas
 * date:   08 Jan 2013
 *************************************************************************
 */

#include <QApplication>
#include <QStringList>
#include "ParameterList.hh"
#include "Except.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ParameterList::scanCommandLine()
{
  
  const QStringList &arg=qApp->arguments();
  int len=arg.size();
  
  for(int i=1;i<len;i++){
    if(arg[i]=="-a"){
      i++;
      //myDebug()<<"i="<<i<<len;
      if(i>=len)throw Except("option -a needs argument of the structure: name:value");
      pdata.append(ParameterListEntry(ParameterList::Parameter,arg[i]));
    }
  }
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String ParameterList::getValueAsString(const String &name)
{
  long len=pdata.size();
  for(long i=0;i<len;i++){
    const ParameterListEntry &e=pdata.at(i);
    if(e.type()==Parameter){
      if(e.name()==name){
	return e.value();
      }
    }
  }
  throw Except(Sprintf("parameter with name '%s' not found!",name())());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double ParameterList::getValue(const String &name)
{
  long len=pdata.size();
  for(long i=0;i<len;i++){
    const ParameterListEntry &e=pdata.at(i);
    if(e.type()==Parameter){
      if(e.name()==name){
	try{
	  return e.value().convertToReal();
	}catch(Except &ex){
	  throw Except(Sprintf("parameter name '%s': %s!",name(),ex.msg()));
	}
      }
    }
  }
  throw Except(Sprintf("parameter with name '%s' not found!",name())());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ParameterList::
ParameterListEntry::ParameterListEntry(ParameterListType t,const String &pair)
  :ptype(t)
{
  if(ptype==Parameter){
    
    //myDebug()<<"pair="<<pair;
    long index=pair.find(":");
    //myDebug()<<index;
    if(index<1){
      throw Except("option -a needs argument of the structure: name:value");
    }
    pname=pair.subString(0,index-1);
    pvalue=pair.subString(index+1);
    //myDebug()<<pname;
    //myDebug()<<pvalue;
  }

}



/*EOF*/ 
