/*************************************************************************
 * @(#)Point.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   28 Apr 2011
 *************************************************************************
 */

#include "Point.hh"
#include "MyMath.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point::Point(const Size &size):x(size.w),y(size.h)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
#ifdef QXFPL
#include <QPoint>
Point::Point(const QPoint &point):x(point.x()),y(point.y()){}
#endif

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point limit(const Point &a,const Size &s)
{
  return Point(MyMath::min<double>(MyMath::max<double>(a.x,0),s.w),
	       MyMath::min<double>(MyMath::max<double>(a.y,0),s.h));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double norm(const Point &a)
{
  return sqrt(a.x*a.x+a.y*a.y);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Point::insideRect(const Point &p0,const Size &s)const
{
  return ( (x>=p0.x) && (x<=p0.x+s.w) && 
	   (y>=p0.y) && (y<=p0.y+s.h) );

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Point::atPoint(const Point &p0,double sensitivewidth)const
{
  return (fabs(x-p0.x)<=sensitivewidth) && (fabs(y-p0.y)<=sensitivewidth);
}

bool Point::atPoint(const Point &p0,const Size &sensitivespot)const
{
  return (fabs(x-p0.x)<=sensitivespot.w) && (fabs(y-p0.y)<=sensitivespot.h);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const Point &Point::operator+=(const Size &other)
{
  x+=other.w;
  y+=other.h;
  return *this;
}

const Point &Point::operator-=(const Size &other)
{
  x-=other.w;
  y-=other.h;
  return *this;
}


const Point &Point::operator*=(const Size &other)
{
  x*=other.w;
  y*=other.h;
  return *this;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator+(const Point &a,const Point &b){
  return Point(a.x+b.x,a.y+b.y);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator*(const Point &a,const Size &b)
{
  return Point(a.x*b.w,a.y*b.h);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
double operator*(const Point &a,const Point &b)
{
  return a.x*b.x+a.y*b.y;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator/(const Point &a,const Size &b)
{
  return Point(a.x/b.w,a.y/b.h);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator+(const Point &a,const Size &b)
{
  return Point(a.x+b.w,a.y+b.h);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator-(const Point &a,const Point &b)
{
  return Point(a.x-b.x,a.y-b.y);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point operator-(const Point &a,const Size &b)
{
  return Point(a.x-b.w,a.y-b.h);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size operator*(const Size &a,const Size &b)
{
  return Size(a.w*b.w,a.h*b.h);
}

Size operator*(const Size &a,double b)
{
  return Size(a.w*b,a.h*b);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size operator+(const Size &a,const Size &b)
{
  return Size(a.w+b.w,a.h+b.h);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size operator/(const Size &a,const Size &b)
{
  return Size(a.w/b.w,a.h/b.h);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size abs(const Size &s)
{
  return Size(fabs(s.w),fabs(s.h));
}

/*EOF*/ 
