/* @(#)Point.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _POINT_H
#define _POINT_H 1

#include "MyMath.hh" 
#include "Xfp.hh" 



class Size;
#ifdef QXFPL
class QPoint;
#endif
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Point{
 public:
  Point():x(0),y(0){};
  Point(double xx,double yy):x(xx),y(yy){};
  Point(const Size &point);
#ifdef QXFPL
  Point(const QPoint &point);
#endif
  const Point &operator=(double l){
    x=y=l;
    return *this;
  };
  const Point &operator+=(const Point &other){
    x+=other.x;
    y+=other.y;
    return *this;
  };
  double operator[](long i)const {if(i==Xfp::Graph_X) return x;else return y;}
  double &operator[](long i){if(i==Xfp::Graph_X) return x;else return y;}
  const Point &operator+=(const Size &other);
  const Point &operator-=(const Size &other);
  const Point &operator*=(const Size &other);
  const Point &operator/=(double d){ x/=d; y/=d; return *this; };
  Point operator-()const{return Point(-x,-y);};
  bool operator!=(const Point &other){
    return (other.x!=x)||(other.y!=y);
  };
  Point rotate(double ang){
    double angle=ang/180.*Pi;
    double cs=cos(angle);
    double sn=sin(angle);
    return Point(cs*x-sn*y,cs*y+sn*x);
  };

  Point ortho()const{return Point(-y,x);};
  Point &normalize(){ double d=sqrt(x*x+y*y);x/=d;y/=d;return *this;};
  Point normalized()const{ double d=sqrt(x*x+y*y);return Point(x/d,y/d);};

  bool insideRect(const Point &p0,const Size &s)const;
  bool atPoint(const Point &p0,double sensitivewidth)const;
  bool atPoint(const Point &p0,const Size &sensitivespot)const;
  
  double x,y;
};




/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Size{
 public:
  Size():w(0),h(0){};
  Size(double ww,double hh):w(ww),h(hh){};
  Size(const Point &point):w(point.x),h(point.y){};
  Size(double d):w(d),h(d){};
  const Size &operator=(double l){
    w=h=l;
    return *this;
  };
  Size operator-()const{return Size(-w,-h);};
  bool operator!=(const Size &other){
    return (other.w!=w)||(other.h!=h);
  };
  const Size &operator+=(const Size &other){
    w+=other.w;
    h+=other.h;
    return *this;
  };
  const Size &operator-=(const Size &other){
    w-=other.w;
    h-=other.h;
    return *this;
  };
  Size toWidth()const{return Size(w,0);};
  Size toHeight()const{return Size(0,h);};
  bool isPositive()const{return w>=0 && h>=0;};
  double w,h;
};


  
/* ***********************************************************************
 *
 * ***********************************************************************
 */

Point operator+(const Point &a,const Point &b);
Point operator-(const Point &a,const Point &b);
Point operator-(const Point &a,const Size &b);
double operator*(const Point &a,const Point &b);
Point operator*(const Point &a,const Size &b);
Point operator/(const Point &a,const Size &b);
Point operator+(const Point &a,const Size &b);
Size operator+(const Size &a,const Size &b);
Size operator*(const Size &a,const Size &b);
//Size operator*(const Size &a,double b);
Size operator/(const Size &a,const Size &b);

Point limit(const Point &a,const Size &s);
double norm(const Point &a);
Size abs(const Size &s);


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _POINT_H */

