/* @(#)Range.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _RANGE_H
#define _RANGE_H 1

#include <limits> 
#include "Xfp.hh"
#include "MyMath.hh"

class Range{
 public:
 Range():pvalid(false){
    setNew(Xfp::Graph_X);
    setNew(Xfp::Graph_Y);
    pmin[Xfp::Graph_X]=0.;
    pmin[Xfp::Graph_Y]=0.;
    pmax[Xfp::Graph_X]=0.;
    pmax[Xfp::Graph_Y]=0.;
    pminpos[Xfp::Graph_X]=0;
    pminpos[Xfp::Graph_Y]=0;
    pmaxpos[Xfp::Graph_X]=0;
    pmaxpos[Xfp::Graph_Y]=0;
  };
  inline double min(Xfp::XYAxisType xy)const{return pmin[xy];};
  inline double max(Xfp::XYAxisType xy)const{return pmax[xy];};
  inline double minpos(Xfp::XYAxisType xy)const{return pminpos[xy];};
  inline double maxpos(Xfp::XYAxisType xy)const{return pmaxpos[xy];};

  inline void setNew(Xfp::XYAxisType xy){
    pisnew[xy]=true;
    pisnewpos[xy]=true;
  }

  inline void extend(Xfp::XYAxisType xy,double x){
    if(pisnew[xy]){
      pmin[xy]=x;
      pmax[xy]=x;
      pisnew[xy]=false;
    }else{
      pmin[xy]=MyMath::min<double>(pmin[xy],x);
      pmax[xy]=MyMath::max<double>(pmax[xy],x);
    }
    if(x<=0.) return;
    if(pisnewpos[xy]){
      pminpos[xy]=x;
      pmaxpos[xy]=x;
      pisnewpos[xy]=false;
    }else{
      pminpos[xy]=MyMath::min<double>(pminpos[xy],x);
      pmaxpos[xy]=MyMath::max<double>(pmaxpos[xy],x);
    }
  }

  inline void invalidate(){pvalid=false;};
  inline void setValidity(bool v){pvalid=v;};
  inline bool isValid(){return pvalid;};

  void extend(const Range &other,Xfp::XYAxisType xy){
    pmin[xy]=MyMath::min(pmin[xy],other.pmin[xy]);
    pmax[xy]=MyMath::max(pmax[xy],other.pmax[xy]);
    pminpos[xy]=MyMath::min(pminpos[xy],other.pminpos[xy]);
    pmaxpos[xy]=MyMath::max(pmaxpos[xy],other.pmaxpos[xy]);
  }

 private:
  bool pvalid;
  bool pisnew[2],pisnewpos[2]; // for extend
  // general range
  double pmin[2],pmax[2];
  // positive range
  double pminpos[2],pmaxpos[2];
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _RANGE_H */

