/*************************************************************************
 * @(#)Shape.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   22 Nov 2011
 *************************************************************************
 */

#include "Shape.hh"
#include "LineShape.hh"
#include "EllipseShape.hh"
#include "ViewBox.hh"
#include "Graph.hh"
#include "World.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Shape::Shape(Graph *parent,long index)
  :GraphObject(parent),ptype(Xfp::SHTNone),pname(""),pactive(true),
   puniqueindex(index),
   pcoordsystem(Xfp::CSYView)
{
  setObjectType(GOTShape);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Shape::~Shape()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Shape::changed()const
{
  return pchanged;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Shape::setChanged(bool c)
{
  pchanged=c;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Shape::guiName(bool inmenu)const
{
  if(!inmenu){
    if(pname.length()>0){
      return Sprintf("%s%ld[%s]",
		     Xfp::shapeTypeGuiName(ptype),
		     puniqueindex+1,pname()
		     ).removeFromSet("&");
	
    }else{
      return Sprintf("%s%ld",
		     Xfp::shapeTypeGuiName(ptype),
		     puniqueindex+1
		     ).removeFromSet("&");
    }
  }else{
    return Sprintf("%s%ld[%s]",
		   Xfp::shapeTypeGuiName(ptype),
		   puniqueindex+1,pname()
		   );
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point Shape::posToDC(const Point &p,Xfp::CoordinateSystem currentmode)const
{
  if(currentmode==Xfp::CSYView){
    return pgraph->viewBox()->vctodc(p);
  }else{
    return pgraph->world()->wctodc(p);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Point Shape::dcToPos(const Point &p,Xfp::CoordinateSystem currentmode)const
{
  if(currentmode==Xfp::CSYView){
    return pgraph->viewBox()->dctovc(p);
  }else{
    return pgraph->world()->dctowc(p);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size Shape::sizeToDC(const Size &s,Xfp::CoordinateSystem currentmode)const
{
  if(currentmode==Xfp::CSYView){
    return pgraph->viewBox()->vctodc(s);
  }else{
    return pgraph->world()->wctodc(s);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Size Shape::dcToSize(const Size &s,Xfp::CoordinateSystem currentmode)const
{
  if(currentmode==Xfp::CSYView){
    return pgraph->viewBox()->dctovc(s);
  }else{
    return pgraph->world()->dctowc(s*Size(1,-1));
  }

}












/* ***********************************************************************
 *
 * ***********************************************************************
 */
Shapes::Shapes(Graph *parent)
  :GraphObject(parent),pchanged(false)
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */


Shapes::~Shapes()
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Shapes::changed()const
{
  bool c=pchanged;
  for(int i=0;i<pshapes.size();i++){
    c|=pshapes.at(i)->changed();
  }
  return c;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Shapes::setChanged(bool c)
{
  pchanged=c;
  for(int i=0;i<pshapes.size();i++){
    pshapes[i]->setChanged(c);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Shapes::reset()
{
  pshapes.resize(0,true);

  setChanged(false);
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Shape *Shapes::getByIndex(long index,Xfp::ShapeType mode)
{
  for(int i=0;i<size();i++){
    if(at(i)->ptype==mode && at(i)->puniqueindex==index){
      return at(i);
    }
  }
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Shapes::insertShape(long i,Shape *sh)
{
  if(!sh){
    printf("Shapes::insertShape: sh=0!?\n");
    exit(1);
  }
  pshapes.insert(i,sh);
 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Shapes::insertShape(long i,Xfp::ShapeType mode)
{
  Shape *sh;
  if(mode==Xfp::SHTLine){
    sh=new LineShape(pgraph,makeUniqueIndex(mode));
  }else if(mode==Xfp::SHTEllipse){
    sh=new EllipseShape(pgraph,makeUniqueIndex(mode));
  }
  
  pshapes.insert(i,sh);
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Shapes::makeUniqueIndex(Xfp::ShapeType mode)
{
  long ind=0;
  while (true){
  again:;
    for(long i=0;i<size();i++){
      if(at(i)->ptype!=mode) continue;
      if(at(i)->puniqueindex==ind){
	ind++;
	goto again;
      }
    }
    break;
  }
  return ind;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Shapes::asCommands(const String &tab)const
{
  String s=Sprintf("%s# shapes settings\n",tab());
  
  for(long it=0;it<pshapes.size();it++){
    const Shape *sh=pshapes.at(it);
    s+=Sprintf("%s",sh->asCommands(tab)());
  }

  return s;
}

/*EOF*/ 
