/* @(#)Shape.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SHAPE_H
#define _SHAPE_H 1

#include "GraphObject.hh"
#include "PtrArray.hh" 
#include "Xfp.hh" 
#include "InsertMenuData.hh"


class Shape: public GraphObject
{
 public:

  Shape():GraphObject(0),ptype(Xfp::SHTNone),pname(""),pactive(true),
    puniqueindex(-1),pcoordsystem(Xfp::CSYView){};
    Shape(Graph *parent,long index);
  virtual ~Shape();

  virtual void reset(){};

  bool active()const{return pactive;};
  void setActive(bool a){pchanged|=(pactive!=a);pactive=a;};

  const String &name()const{return pname;};
  void setName(const String &i){pname=i;};
  
  Xfp::ShapeType type()const{return ptype;};

  virtual String guiName(bool inmenu=false)const;

  Xfp::CoordinateSystem coordinateSystem()const{return pcoordsystem;};
  void setCoordinateSystem(Xfp::CoordinateSystem c){
    pchanged|=(pcoordsystem!=c);pcoordsystem=c;
  };



  Point posToDC(const Point &p,Xfp::CoordinateSystem currentmode)const;
  Point dcToPos(const Point &p,Xfp::CoordinateSystem currentmode)const;

  Size sizeToDC(const Size &p,Xfp::CoordinateSystem currentmode)const;
  Size dcToSize(const Size &p,Xfp::CoordinateSystem currentmode)const;



  virtual bool changed()const;
  virtual void setChanged(bool c);

 protected:
  Xfp::ShapeType ptype;
  String pname;
  bool pactive;
  long puniqueindex;
  Xfp::CoordinateSystem pcoordsystem;
 private:

  friend class Shapes;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */


class Shapes: public GraphObject
{
 public:
  Shapes(Graph *);
  virtual ~Shapes();
  
  inline long size()const{return pshapes.size();};
  inline Shape *at(long i){return pshapes[i];};
  inline const Shape *at(long i)const{return pshapes.at(i);};

  Shape *getByIndex(long i,Xfp::ShapeType mode);

  inline long indexOf(Shape *sh){return pshapes.indexOf(sh);};
  inline Shape *takeAt(long i){return pshapes.takeAt(i);};

  inline void deleteShape(Shape *sh){pshapes.remove(sh);};
  inline void deleteShape(long i){pshapes.removeAt(i);};

  void insertShape(long i,Shape *sh);
  void insertShape(long i,Xfp::ShapeType mode);

  void reset();


  String asCommands(const String &tab)const;

  bool changed()const;
  void setChanged(bool c);

 private:
  long makeUniqueIndex(Xfp::ShapeType mode);

 private:
  bool pchanged;
  PtrArray<Shape> pshapes;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SHAPE_H */

