/*************************************************************************
 * @(#)TextBox.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 Apr 2011
 *************************************************************************
 */

#include "Attributes.hh"
#include "Graph.hh"
#include "TextBox.hh"
#include "Position.hh"
#include "defs.hh"


const char *TextBox::pguinames[]={
  "&Title",
  "&Subtitle",
  "&X-axis label",
  "&Y-axis label",
  "Opposite X-ax&is label",
  "Opposite Y-axis &label",
  "&TextBox[%s]"
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBox::TextBox(Graph *parent,TextBoxes *textboxes,
		 TextBoxType t,const String &s)
  :GraphObject(parent),ptextboxes(textboxes),ptype(t),pactive(true),
   pboxisactive(true),prim(0),pfill(0),pborder(0.2),ptextstyle(0),
   ptext(s),ppos(0)
{
  setObjectType(GraphObject::GOTTextBox);
  ppos=new Position();
  prim=new LineStyle();
  pfill=new FillStyle();
  ptextstyle=new FontStyle();
  pfill->setFill(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBox::TextBox(const TextBox &other)
  :GraphObject(other.pgraph),ptextboxes(other.ptextboxes),ptype(other.ptype),
   pactive(other.pactive),
   pboxisactive(other.pboxisactive),pborder(other.pborder),
   ptext(other.ptext)
{
  setObjectType(GraphObject::GOTTextBox);
  ppos=new Position();*ppos=*(other.ppos);
  prim=new LineStyle();*prim=*(other.prim);
  pfill=new FillStyle();*pfill=*(other.pfill);
  ptextstyle=new FontStyle();*ptextstyle=*(other.ptextstyle);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBox::~TextBox()
{
  if(ppos) delete ppos;ppos=0;
  if(prim) delete prim;prim=0;
  if(pfill) delete pfill;pfill=0;
  if(ptextstyle) delete ptextstyle;ptextstyle=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBox::reset()
{
  pactive=true;
  pboxisactive=true;
  prim->reset();
  pfill->reset();
  pfill->setFill(true);
  pborder=0.2;
  ptextstyle->reset();
  ptext="";
  ppos->reset();
  psize=0;
  setChanged(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const String TextBox::guiName(bool inmenu)const
{
  if(ptype!=TBTGeneral){
    if(!inmenu)
      return String(pguinames[ptype]).removeFromSet("&");
    else
      return String(pguinames[ptype]);
  }else{
    if(!inmenu)
      return Sprintf(pguinames[ptype],text().subString(0,6)())
	.removeFromSet("&");
    else
      return Sprintf(pguinames[ptype],text().subString(0,6)());
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const String TextBox::paramGuiName(long index)const
{
  if(ptype!=TBTGeneral){
    return String(pguinames[ptype]).removeFromSet("& -").lowerCase();
  }else{
    return Sprintf("textbox%ld",index-TBTGeneral+1);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TextBox::changed()const
{
  return pchanged|prim->changed()|pfill->changed()|ptextstyle->changed()
    |ppos->changed();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBox::setChanged(bool c)
{
  pchanged=c;
  prim->setChanged(c);
  pfill->setChanged(c);
  ptextstyle->setChanged(c);
  ppos->setChanged(c);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphObject::GraphObjectRegion 
TextBox::insideRegion(const DragDescriptor &dd,bool update)
{
  /* NSF("inside_region_textbox"); */

  if(!pactive) return GraphObject::GORNone;

  Point p=dd.mousepoint-ppos->absposToDC(pgraph)*dd.devicesize;
  Point p0=-ppos->origin()*psize;
  Point po=p.rotate(ppos->angle());

  if(po.insideRect(p0,psize)){
    if(ppos->restriction()){
      if(update) pupdaterequest=GOCOffset;
    }else{
      if(update) pupdaterequest=GOCPosition;
    }
    return GraphObject::GORBody;
  }
  
  return GraphObject::GORNone;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBox::initDrag(const DragDescriptor &dd)
{
  pmousepos=dd.mousepoint;
  pobjectpos=ppos->absposToDC(pgraph)*dd.devicesize;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBox::updateDrag(const DragDescriptor &dd)
{


  Point pd=dd.mousepoint-pmousepos;
  
  if(ppos->restriction()){
    Point p=ppos->pos0ToDC(pgraph)*dd.devicesize;
    
    Point pi=pobjectpos+pd-p;
    
    if(ppos->offsettype() & Position::OSTY){
      if(pi.y*ppos->offsetsign() >=0){
	pobjectpos.y+=pd.y;
	pmousepos.y=dd.mousepoint.y;
      }else{
	pobjectpos.y+=pd.y-pi.y;
	pmousepos.y=dd.mousepoint.y;
      }
    }
    
    if((ppos->offsettype() & Position::OSTX)){
      if(pi.x*ppos->offsetsign() >=0){
	pobjectpos.x+=pd.x;
	pmousepos.x=dd.mousepoint.x;
      }else{
	pobjectpos.x+=pd.x-pi.x;
	pmousepos.x=dd.mousepoint.x;
      }
    }
  }else{
    pobjectpos+=pd;
    pmousepos=dd.mousepoint;
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBox::setFromDrag(DragDescriptor &dd)
{
 


  if(ppos->restriction()){
    Point v=ppos->dcToPos(pgraph,pobjectpos/dd.devicesize,Xfp::CSYView);
    if( (ppos->offsettype() & Position::OSTX)  ){
      ppos->setOffset(MyMath::max<double>(0.,ppos->offset()
			   +(v-ppos->absposToVC(pgraph)).x
			   *ppos->offsetsign()));
    }else if( (ppos->offsettype() & Position::OSTY)  ){
      ppos->setOffset(MyMath::max<double>(0.,ppos->offset()
			   +(v-ppos->absposToVC(pgraph)).y
			   *ppos->offsetsign()));
    }
  }else{
    Point v=ppos->dcToPos(pgraph,pobjectpos/dd.devicesize,
			  ppos->coordinateSystem());
    ppos->setPos(v);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String TextBox::asCommands(const String &tab)const
{
  String s;
  long index=ptextboxes->indexOf(this);
  String name=paramGuiName(index);
  s+=Sprintf("%s%s %s\n",tab(),name(),Xfp::OnOff(active()));
  name=tab+"  "+name;
  s+=Sprintf("%s \"%s\"\n",name(),text()());
  s+=textStyle()->asCommands(name+" font");
  String pref=name+" frame";
  s+=Sprintf("%s %s\n",pref(),Xfp::OnOff(boxIsActive()));
  s+=rim()->asCommands(pref+" rim");
  s+=fill()->asCommands(pref+" fill");
  s+=Sprintf("%s border spacing %g\n",pref(),border());
  s+=pos()->asCommands(name);
  
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TextBoxes::TextBoxes(Graph *parent)
  :pchanged(false),pgraph(parent)
{
  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTTitle));

  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTSubTitle));

  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTXAxisLabel));
  
  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTYAxisLabel));

  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTOppositeXAxisLabel));

  ptextboxes.append(new TextBox(pgraph,this,TextBox::TBTOppositeYAxisLabel));

  setSpecialBoxes();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxes::setSpecialBoxes()
{
  TextBox *tb;
  {
    tb=ptextboxes[0];
    tb->setText("Compound X$_2$.YZ$^$~m$_0$.");
    Position *ps=tb->pos();
    ps->setPos(Point(0.5,0.));
    ps->setOrigin(Point(0.5,1.));
    
    tb->rim()->setColor(0xffffff);
    tb->textStyle()->setSize(20.);
    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTNegY);
    ps->setOffset(0.03);
    
    ps->appendAddType(Position::XOposSiteTicLabelHeight);
    ps->appendAddType(Position::XOposSiteTicLabelOffset);
    ps->appendAddType(Position::SubTitleOffset);
    ps->appendAddType(Position::SubTitleHeight);
  }
  
  {
    tb=ptextboxes[1];
    tb->setText("Subtitle");

    tb->setActive(false);
    Position *ps=tb->pos();
    ps->setPos(Point(0.5,0.));
    ps->setOrigin(Point(0.5,1.));
    
    tb->rim()->setColor(0xffffff);

    tb->textStyle()->setSize(16.);
    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTNegY);
    ps->setOffset(0.01);
    
    ps->appendAddType(Position::XOposSiteTicLabelHeight);
    ps->appendAddType(Position::XOposSiteTicLabelOffset);
  }
  {
    tb=ptextboxes[2];
    tb->setText("X-axis label");
    Position *ps=tb->pos();
    ps->setPos(Point(0.5,1.));
    ps->setOrigin(Point(0.5,0.0));
    tb->rim()->setColor(0xffffff);


    tb->textStyle()->setSize(16.);

    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTPosY);
    ps->setOffset(0.03);
    
    ps->appendAddType(Position::XNormSiteTicLabelHeight);
    ps->appendAddType(Position::XNormSiteTicLabelOffset);
  }
  {
    tb=ptextboxes[3];
    tb->setText("Y-axis label");
    
    Position *ps=tb->pos();
    ps->setPos(Point(0.,0.5));
    ps->setOrigin(Point(0.5,1.0));
    

    tb->rim()->setColor(0xffffff);

    tb->textStyle()->setSize(16.);

    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTNegX);
    ps->setOffset(0.03);
    
    ps->appendAddType(Position::YNormSiteTicLabelWidth);
    ps->appendAddType(Position::YNormSiteTicLabelOffset);
    
    ps->setAngle(90.);
  }
  {
    tb=ptextboxes[4];
    tb->setText("Opposite X-axis label");

    tb->setActive(false);
    Position *ps=tb->pos();
    ps->setPos(Point(0.,0.));
    ps->setOrigin(Point(0.,1.0));

    tb->rim()->setColor(0xffffff);

    tb->textStyle()->setSize(16.);

    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTNegY);
    ps->setOffset(0.01);
    
    ps->appendAddType(Position::XOposSiteTicLabelHeight);
    ps->appendAddType(Position::XOposSiteTicLabelOffset);
  }
  {
    tb=ptextboxes[5];
    tb->setText("Opposite Y-axis label");

    tb->setActive(false);
    Position *ps=tb->pos();
    ps->setPos(Point(1.,0.5));
    ps->setOrigin(Point(0.5,1.0));
    
    tb->rim()->setColor(0xffffff);

    tb->textStyle()->setSize(16.);

    
    ps->setRestriction(true);
    ps->setOffsetType(Position::OSTPosX);
    ps->setOffset(0.03);
    
    ps->appendAddType(Position::YOposSiteTicLabelWidth);
    ps->appendAddType(Position::YOposSiteTicLabelOffset);
    
    ps->setAngle(-90.);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TextBoxes::reset()
{
  adjustSizeToIndex(TextBox::TBTGeneral,true);  
  for(long i=0;i<TextBox::TBTGeneral;i++){
    ptextboxes[i]->reset();
  }
  setSpecialBoxes();
  setChanged(false);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String TextBoxes::asCommands(const String &tab)const
{
  String s=Sprintf("%s# text boxes settings\n",tab());
  
  for(long it=0;it<size();it++){
    s+=Sprintf("%s",at(it)->asCommands(tab)());

  }

  return s;
}


/*EOF*/ 
