/*************************************************************************
 * @(#)TicMarks.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2011
 *************************************************************************
 */

#include "Attributes.hh"
#include "Graph.hh"
#include "GraphAxis.hh"
#include "TicMarks.hh"
#include "World.hh"
#include "Xfp.hh"
#include "MyMath.hh"
#include "defs.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Tic::Tic()
  :pchanged(false)
  ,ptype(Xfp::Graph_X),pkind(Xfp::TKIMajor),ppos(0.)
  ,plinestyle(new LineStyle())
  ,pticside(Xfp::TSNormal)
  ,pticlabelside(Xfp::TSNormal)
  ,pticlength(0.02)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Tic::Tic(const Tic &other)
  :pchanged(other.pchanged)
  ,ptype(other.ptype),pkind(other.pkind),ppos(other.ppos)
  ,plabel(other.plabel)
  ,plinestyle(0)
  ,pticside(other.pticside)
  ,pticlabelside(other.pticlabelside)
  ,pticlength(other.pticlength)
{
  setLineStyle(other.plinestyle);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Tic::~Tic()
{
  if(plinestyle) delete plinestyle; plinestyle=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Tic::changed()const
{
  return pchanged|plinestyle->changed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Tic::setChanged(bool c)
{
  pchanged=c;
  plinestyle->setChanged(c);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Tic::setLineStyle(const LineStyle *ls)
{
  if(!plinestyle) plinestyle=new LineStyle();
  *plinestyle=*ls;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Tic::asCommands(const String &tab,long index)const
{
  String s=Sprintf("%sitic%ld type %s %s\n"
		   "%sitic%ld position %.15g \n"
		   "%sitic%ld label \"%s\" \n"
		   "%sitic%ld tic side %s \n"
		   "%sitic%ld label side %s \n"
		   "%sitic%ld length %g \n"
		   ,tab(),index,(ptype==Xfp::Graph_X?"x":"y")
		   ,(pkind==Xfp::TKIMajor?"major":"minor")
		   ,tab(),index,ppos
		   ,tab(),index,plabel()
		   ,tab(),index,Xfp::ticSideGuiName(pticside)
		   ,tab(),index,Xfp::ticSideGuiName(pticlabelside)
		   ,tab(),index,pticlength
		   )
    +lineStyle()->asCommands(Sprintf("%sitic%ld line",tab(),index))
    ;


  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Tics::Tics()
  :pchanged(false)
{
  setDrawTics(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Tics::Tics(const Tics &other)
{
  pdrawtics[0][0]=other.pdrawtics[0][0];
  pdrawtics[0][1]=other.pdrawtics[0][1];
  pdrawtics[1][0]=other.pdrawtics[1][0];
  pdrawtics[1][1]=other.pdrawtics[1][1];

  
  ptics.resize(other.size());
  for(int i=0;i<other.size();i++){
    ptics[i]=new Tic(*(other.tic(i)));
  }

  pchanged=other.pchanged;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const Tics &Tics::operator=(const Tics &other)
{
  pdrawtics[0][0]=other.pdrawtics[0][0];
  pdrawtics[0][1]=other.pdrawtics[0][1];
  pdrawtics[1][0]=other.pdrawtics[1][0];
  pdrawtics[1][1]=other.pdrawtics[1][1];
  

  ptics.resize(other.size(),true);
  for(int i=0;i<other.size();i++){
    ptics[i]=new Tic(*(other.tic(i)));
  }

  pchanged=other.pchanged;

  return *this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Tics::setChanged(bool c)
{
  for(int i=0;i<ptics.size();i++)
    ptics[i]->setChanged(c);

  pchanged=c;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Tics::changed()const
{
  bool c=pchanged;
  for(int i=0;i<ptics.size();i++)
    c|=ptics.at(i)->changed();
  return c;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Tics::reset()
{
  setDrawTics(true);
  ptics.resize(0,true);
  setChanged(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Tics::setDrawTics(bool u)
{
  setDrawTics(Xfp::Graph_X,Xfp::TKIMajor,u);
  setDrawTics(Xfp::Graph_X,Xfp::TKIMinor,u);
  setDrawTics(Xfp::Graph_Y,Xfp::TKIMajor,u);
  setDrawTics(Xfp::Graph_Y,Xfp::TKIMinor,u);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Tics::asCommands(const String &tab)const
{
  String s=Sprintf("%sirregular tics %s\n"
		   ,tab(),Xfp::OnOff(pdrawtics[Xfp::Graph_X][Xfp::TKIMajor])
		   );
  for(long it=0;it<ptics.size();it++){
    s+=ptics.at(it)->asCommands(tab,it+1);
  }
  
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarks::TicMarks(Graph *parent)
  :GraphObject(parent),ptics(new Tics())
  ,pusertics(new Tics())

{
  ptextstyle[Xfp::Graph_X]=new FontStyle();
  ptextstyle[Xfp::Graph_Y]=new FontStyle();
  setObjectType(GraphObject::GOTTicMarks);

  plinestyle[0][0]=new LineStyle();
  plinestyle[0][1]=new LineStyle();
  plinestyle[1][0]=new LineStyle();
  plinestyle[1][1]=new LineStyle();

  ptic_length[Xfp::Graph_X][Xfp::TKIMajor]=0.02;
  ptic_length[Xfp::Graph_X][Xfp::TKIMinor]=0.01;
  pticside[Xfp::Graph_X]=Xfp::TSNormal; 
  pticlabelside[Xfp::Graph_X]=Xfp::TSNormal; 

  pmajortics[Xfp::Graph_X]=1.; 
  pminortics[Xfp::Graph_X]=2; 
  pautotics[Xfp::Graph_X]=true; 
  plabeloffset[Xfp::Graph_X]=0.03;

  pseparateyticlength[Xfp::TKIMajor]=false;
  pseparateyticlength[Xfp::TKIMinor]=false;


  pdecimals[Xfp::Graph_X]=-1;
  pdecimals[Xfp::Graph_Y]=-1;
  ptextstyle[Xfp::Graph_X]->setSize(16.);
  ptextstyle[Xfp::Graph_Y]->setSize(16.);


  ptic_length[Xfp::Graph_Y][Xfp::TKIMajor]=0.02;
  ptic_length[Xfp::Graph_Y][Xfp::TKIMinor]=0.01;
  pticside[Xfp::Graph_Y]=Xfp::TSNormal; 
  pticlabelside[Xfp::Graph_Y]=Xfp::TSNormal; 

  pmajortics[Xfp::Graph_Y]=1.; 
  pminortics[Xfp::Graph_Y]=2; 

  pautotics[Xfp::Graph_Y]=true; 
  plabeloffset[Xfp::Graph_Y]=0.03;



  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
TicMarks::~TicMarks()
{
  if(ptics) delete ptics;
  if(pusertics) delete pusertics;
  if(ptextstyle[0]) delete ptextstyle[0];ptextstyle[0]=0;
  if(ptextstyle[1]) delete ptextstyle[1];ptextstyle[1]=0;
  if(plinestyle[0][0]) delete plinestyle[0][0];plinestyle[0][0]=0;
  if(plinestyle[0][1]) delete plinestyle[0][1];plinestyle[0][1]=0;
  if(plinestyle[1][0]) delete plinestyle[1][0];plinestyle[1][0]=0;
  if(plinestyle[1][1]) delete plinestyle[1][1];plinestyle[1][1]=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarks::setChanged(bool c)
{
  ptics->setChanged(c);
  ptextstyle[0]->setChanged(c);
  ptextstyle[1]->setChanged(c);
  plinestyle[0][0]->setChanged(c);
  plinestyle[0][1]->setChanged(c);
  plinestyle[1][0]->setChanged(c);
  plinestyle[1][1]->setChanged(c);
  pusertics->setChanged(c);
  pchanged=c;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool TicMarks::changed()const
{
  return pchanged|ptics->changed()|ptextstyle[0]->changed()|
    ptextstyle[1]->changed()|
    plinestyle[0][0]->changed()|plinestyle[1][0]->changed()|
    plinestyle[0][1]->changed()|plinestyle[1][1]->changed()|
    pusertics->changed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void TicMarks::reset()
{

  ptics->resize(0,true);
  ptextstyle[0]->reset();
  ptextstyle[1]->reset();
  pusertics->resize(0,true);

  plinestyle[0][0]->reset();
  plinestyle[0][1]->reset();
  plinestyle[1][0]->reset();
  plinestyle[1][1]->reset();

  ptic_length[Xfp::Graph_X][Xfp::TKIMajor]=0.02;
  ptic_length[Xfp::Graph_X][Xfp::TKIMinor]=0.01;
  pticside[Xfp::Graph_X]=Xfp::TSNormal; 
  pticlabelside[Xfp::Graph_X]=Xfp::TSNormal; 

  pseparateyticlength[Xfp::TKIMajor]=false;
  pseparateyticlength[Xfp::TKIMinor]=false;

  pmajortics[Xfp::Graph_X]=1.; 
  pminortics[Xfp::Graph_X]=2; 
  pautotics[Xfp::Graph_X]=true; 
  plabeloffset[Xfp::Graph_X]=0.03;



  pdecimals[Xfp::Graph_X]=-1;
  pdecimals[Xfp::Graph_Y]=-1;
  ptextstyle[Xfp::Graph_X]->setSize(16.);
  ptextstyle[Xfp::Graph_Y]->setSize(16.);


  ptic_length[Xfp::Graph_Y][Xfp::TKIMajor]=0.02;
  ptic_length[Xfp::Graph_Y][Xfp::TKIMinor]=0.01;
  pticside[Xfp::Graph_Y]=Xfp::TSNormal; 
  pticlabelside[Xfp::Graph_Y]=Xfp::TSNormal; 

  pmajortics[Xfp::Graph_Y]=1.; 
  pminortics[Xfp::Graph_Y]=2; 

  pautotics[Xfp::Graph_Y]=true; 
  plabeloffset[Xfp::Graph_Y]=0.03;


  setChanged(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

static void least_digit_number_in_interval(double x0,double x1,
					   double &lxb0)
{
  lxb0=x0;

      
  double m0,m1;
  long e0,e1;
  MyMath::splitbaseN(pow(10.,x0),10.,m0,e0);
  MyMath::splitbaseN(pow(10.,x1),10.,m1,e1);
  
  long nu=0;
  if(ceil(m0)>floor(m1*pow(10.,e1-e0))){
    nu=MyMath::max(lRound(ceil(-log10(m1*pow(10.,e1-e0)-m0))),0L);
  }
  long dte=ceil(m0*pow(10.,nu));
  lxb0=log10(dte)+e0-nu;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarks::calcAutoTicSpacing(double lx0,double lx1,double &mjtics,
				  long &mntics,long &mjexfac,double &lxb0,
				  Xfp::GraphAxisScaling sc)
{
  // input: dx = interval length
  // ouput: mjtics  = major tic spacing
  //        mntics  = minor tic subdivision
  //        mjexfac = ??
  NSF("calcautoticspacing_graph");

  double dx=lx1-lx0;

  //  myDebug()<<"\n"<<nsubfunc;

  if(sc==Xfp::GASLogarithmic){
    
    // myDebug()<<"x0="<<lx0<<pow(10.,lx0);
    // myDebug()<<"x1="<<lx1<<pow(10.,lx1);


    const double lxb0s[]={0.,log10(2.),log10(5.),log10(4.)};
    const double intervall[3]={1.,log10(2.),log10(5.)};
    const long intervallmn[3]={9,2,4};
    long ktyp=0;
    long n;
    long m=1000;
    long intvtyp=0;
    long xb0typ=0;

    bool tosmall=(lx1-lx0)<log10(2.);
    
    if(tosmall){

      least_digit_number_in_interval(lx0,lx1,lxb0);
      
      mjexfac=0;
      mjtics=2.;
      mntics=1;
      
      return;
    }


    for(int ix=-1;++ix<(int)(sizeof(lxb0s)/sizeof(double));){
      double lxb0=lxb0s[ix];
      long nb0=lRound(ceil(lx0-lxb0));
      if(lxb0+nb0>lx1)nb0--;

      for(int i=-1;++i<2;){
	double logl=intervall[i];
	long k=0;
	while((n=lRound(floor(dx/(logl+k))))>0){
	  //if(logl+k<lxb0+nb0-lx0){ k++; continue; }
	  
	  if(n>0 &&  n<20){
	    /* best is 5 subdivisions */
	    if(labs(5-n)<m){
	      m=labs(5-n);
	      ktyp=k;
	      intvtyp=i;
	      xb0typ=ix;
	    }
	  }
	  
	  k++;
	}
	if(m<3){break;}
	n=lRound(floor(dx/(logl+ktyp)));
      }
    }
    if(m==1000){
      printf("%s: log auto-subdivision failed! Shame on the author.\n",nsubfunc);
      //      exit(1);
    }

    lxb0=lxb0s[xb0typ];
    long nb0=lRound(ceil(lx0-lxb0));
    if(lxb0+nb0>lx1)nb0--;
    double logl=intervall[intvtyp];
    n=lRound(floor(dx/(logl+ktyp)));
    
    lxb0=lxb0+nb0;
    mjexfac=0;
    mjtics=logl+ktyp;
    mntics=intervallmn[intvtyp];
    
    return;
  }
    

  long exponent,facttyp=0,intvtyp=0;
  double mantisse;
  const double intervall[3]={1.,2.,5.};
  const long intervallmin[3]={5,4,5};


  if(dx==0.){
    mjtics=1;
    mntics=1;
    mjexfac=0;
    return ;
  }
  
  
  /*   printf("split10: %f\n",dx);  */
  long m=1000;
  MyMath::splitbaseN(dx,10.,mantisse,exponent);

  
  for(int i=-1;++i<3;){ /* for all subintervall lengths (1,2,5) */
    for(int j=-1;++j<3;){ /* for all factors 1,10,100 */
      
      long n=(long)(mantisse/intervall[i]*pow(10.,(double)j));
      
      /* no more than 20 and not less than 1 */
      if(n>0 &&  n<20){
	/* best is 5 subdivisions */
	if(labs(5-n)<m){
	  
	  m=labs(5-n);
	  intvtyp=i;
	  facttyp=j;
	}
      }
    }
  }
  /* some problems */
  if(m==1000){
    printf("%s: auto-subdivision failed! Shame on the author.\n",nsubfunc);
    exit(1);
  }
  mjexfac=exponent-facttyp;

  mjtics=intervall[intvtyp]*pow(10.,(double)mjexfac);
  
  mntics=intervallmin[intvtyp];

  lxb0=((long)(lx0/mjtics))*mjtics;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TicMarks::calcTicMarks(Graph *graph,double widthOverHeight)
{
  NSF("calcticmarks_graph");
  double mjtics,mantisse;
  long mjexfac,exponent,mntics;
  char frm[5000]; 
  String txt; 

  


  ptics->resize(0,true);
  long maxcount=0;
  long tcount=0;

  double tol=1.e-5;




  Xfp::XYAxisType xy=Xfp::Graph_X;
  for(int whichxy=-1;++whichxy<2;){

    //myDebug()<<"\n\nxy="<<xy;
    
    bool logscale=(graph->axes()->axisScaling(xy)==Xfp::GASLogarithmic);
    
    double x0=graph->world()->w0(xy);
    double x1=graph->world()->w1(xy);
    double lx0=graph->axes()->toScaled(xy,x0);
    double lx1=graph->axes()->toScaled(xy,x1);
    double lDX=lx1-lx0;
    double lxb0;
    //need unscaled world
    if(!pautotics[xy]) {
      mjtics=pmajortics[xy];
      mntics=pminortics[xy];
      if(logscale){
	//myDebug()<<"lx0="<<pow(10.,lx0);
	//myDebug()<<"lx1="<<pow(10.,lx1);
	//myDebug()<<"f="<<mjtics;
	if(mjtics<1.01) mjtics=2.;
	mjtics=log10(mjtics);
	long m=0;
	long xb0typ;
	const double lxb0s[]={0.,log10(2.),log10(5.),log10(4.)
			      ,log10(3.),log10(6.),
			      log10(8.),log10(9.),log10(7.)
	};
	for(int ix=-1;++ix<(int)(sizeof(lxb0s)/sizeof(double));){
	  lxb0=lxb0s[ix];
	  long nb0=lRound(ceil(lx0-lxb0));
	  if(lxb0+nb0>lx1)nb0--;
	  //myDebug()<<"ix="<<ix<<" xb0="<<pow(10.,lxb0+nb0);
	  long n0=0;
	  while(lxb0+nb0+mjtics*n0<lx0)n0++;
	  if(lxb0+nb0+mjtics*n0>lx1) continue;

	  //myDebug()<<"left="<<n0<<" xb00="<<pow(10.,lxb0+nb0+mjtics*n0);

	  long n1=n0;
	  while(lxb0+nb0+mjtics*n1<lx1)n1++;

	  //myDebug()<<"right="<<n1<<" xb01="<<pow(10.,lxb0+nb0+mjtics*n1);
	  
	  long N=n1-n0;
	  if(N>m){
	    m=N;
	    xb0typ=ix;
	    //myDebug()<<"N="<<N<<" xb0="<<pow(10.,lxb0s[xb0typ]+nb0);
	  }
	  if(m>0) break;
	}
	if(m==0){
	  least_digit_number_in_interval(lx0,lx1,lxb0);
	  //myDebug()<<"least="<<lxb0;
	  mntics=1;
	  mjexfac=0;
	}else{
	  lxb0=lxb0s[xb0typ];
	  long nb0=lRound(ceil(lx0-lxb0));
	  if(lxb0+nb0>lx1)nb0--;
	  lxb0=lxb0+nb0;
	}
      }
    }else{
      calcAutoTicSpacing(lx0,lx1,mjtics,mntics,mjexfac,lxb0,
			 graph->axes()->axisScaling(xy));
    }
    
    
    /* start point */
    double a0=lxb0; 

    //myDebug()<<"a0="<<a0<<pow(10.,a0);
    //myDebug()<<"mjtics="<<mjtics<<pow(10.,mjtics);
    //myDebug()<<"mntics="<<mntics;
    // myDebug()<<"mjexfac="<<mjexfac;

    
    
    /* major tics */

    

    long i0=lRound(floor((lx0-a0)/mjtics))-1;
    long i1=lRound(ceil((lx1-a0)/mjtics))+1;
    

    maxcount+=(i1-i0+1);
    ptics->resize(maxcount);
    ValueArray<double> lmjpos;

    
    for(long i=i0;i<=i1;i++){
      double a=a0+mjtics*i;
      if(a<lx0-tol*lDX || a>lx1+tol*lDX) continue;
      lmjpos.append(a);
      if(logscale) a=pow(10.,a);
      ptics->tic(tcount)->setType(xy);
      ptics->tic(tcount)->setKind(Xfp::TKIMajor);
      ptics->tic(tcount)->setPos(a);
      ptics->tic(tcount)->setTicSide(pticside[xy]);
      ptics->tic(tcount)->setTicLabelSide(pticlabelside[xy]);
      double length=ptic_length[xy][Xfp::TKIMajor];
      if(xy==Xfp::Graph_Y && !pseparateyticlength[Xfp::TKIMajor]){
	length=ptic_length[Xfp::Graph_X][Xfp::TKIMajor]/widthOverHeight;
      }
      ptics->tic(tcount)->setTicLength(length);
      
      if(!pautotics[xy]){
	MyMath::splitbaseN(mjtics,10.,mantisse,exponent);
	//printf("%g=%g E %ld\n",mjtics,mantisse,exponent);
	mjexfac=exponent;
      }
      if(logscale){
	txt.sprintf("%g",a).stripBlanks();
      }else{
	long dec=pdecimals[xy];
	//printf("mjexfac=%ld\n fmt=%s\n",mjexfac,frm);
	if(abs(mjexfac)>6){
	  (void)sprintf(frm,"%%-#30.%ldf",(dec<0?1L:dec));
	  txt.sprintf(frm,a/pow(10.,mjexfac)).stripBlanks();
	  txt+=Sprintf("E%+ld",mjexfac);
	}else{
	  if(dec<0) dec=MyMath::max<long>(0,1-mjexfac);
	  (void)sprintf(frm,"%%-#30.%ldf",dec);
	  txt.sprintf(frm,a).stripBlanks();
	}
	if(dec==0){
	  long l=txt.length();
	  if(txt[l-1]=='.'){
	    txt.chop(".");
	  }
	}
      }
      ptics->tic(tcount)->setLabel(txt);
      ptics->tic(tcount)->setLineStyle(this->lineStyle(xy,Xfp::TKIMajor));
      
      
      tcount++;
      
    } /* for the tics */
    
    /* minor */
    ValueArray<double> mnpos;

    double mu=mjtics/mntics;
    if(logscale){
      mu=(pow(10.,mjtics)-1.)/mntics;
    }
    for(long i=i0;i<=i1;i++){
      double ai=a0+mjtics*i;
      for(long j=-1;++j<=mntics;){
	double a,la;
	if(logscale){
	  a=pow(10.,ai)*(1.+mu*j);
	  la=log10(a);
	}else{
	  la=a=ai+mu*j;
	}
	if(la<lx0-tol*lDX || la>lx1+tol*lDX) continue;
	if(!lmjpos.contains(la,lDX*1.0e-4)){
	  mnpos.append(a);
	}
      }
    }

    maxcount+=mnpos.size();
    ptics->resize(maxcount);
    for(long i=0;i<mnpos.size();i++){
      double a=mnpos.at(i);

      ptics->tic(tcount)->setType(xy);
      ptics->tic(tcount)->setKind(Xfp::TKIMinor);
      ptics->tic(tcount)->setPos(a);
      ptics->tic(tcount)->setTicSide(pticside[xy]);
      double length=ptic_length[xy][Xfp::TKIMinor];
      if(xy==Xfp::Graph_Y && !pseparateyticlength[Xfp::TKIMinor]){
	length=ptic_length[Xfp::Graph_X][Xfp::TKIMinor]/widthOverHeight;
      }
      ptics->tic(tcount)->setTicLength(length);

      ptics->tic(tcount)->setLineStyle(this->lineStyle(xy,Xfp::TKIMinor));

      
      tcount++;
    } /* for the tics */
    
    xy=Xfp::Graph_Y;
  } /* for which */
  
  ptics->resize(tcount,true);
  
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String TicMarks::asCommands(const String &tab)const
{
  return Sprintf(
		 "%s#regular tics\n"
 		 "%sx auto tics %s\n"
 		 "%sx tic side %s\n"
 		 "%sx major tics %s\n"
 		 "%sx major tic spacing %.15g\n"
 		 "%sx major tic length %g\n"
 		 "%s"
 		 "%sx minor tics %s\n"
 		 "%sx minor tic subdiv %ld\n"
 		 "%sx minor tic length %g\n"
 		 "%s"
 		 "%sx tic label side %s\n"
 		 "%sx tic label offset %g\n"
 		 "%sx tic label decimals %ld\n"
 		 "%s"

 		 "%sy auto tics %s\n"
 		 "%sy tic side %s\n"
 		 "%sy major tics %s\n"
 		 "%sy major tic spacing %.15g\n"
 		 "%sy major separate tic length %s\n"
 		 "%sy major tic length %g\n"
 		 "%s"
 		 "%sy minor tics %s\n"
 		 "%sy minor tic subdiv %ld\n"
 		 "%sy minor separate tic length %s\n"
 		 "%sy minor tic length %g\n"
		 "%s"
 		 "%sy tic label side %s\n"
 		 "%sy tic label offset %g\n"
 		 "%sy tic label decimals %ld\n"
 		 "%s"
 		 "%s#graph global irregular tics\n"
 		 "%s"
		 ,tab()
 		 ,tab(),Xfp::OnOff(pautotics[Xfp::Graph_X])
 		 ,tab(),Xfp::ticSideGuiName(pticside[Xfp::Graph_X])
 		 ,tab(),Xfp::OnOff(ptics->drawTics(Xfp::Graph_X,Xfp::TKIMajor))
 		 ,tab(),pmajortics[Xfp::Graph_X]
 		 ,tab(),ptic_length[Xfp::Graph_X][Xfp::TKIMajor]
 		 ,plinestyle[Xfp::Graph_X][Xfp::TKIMajor]
 		 ->asCommands(Sprintf("%sx major tic line",tab()))()
 		 ,tab(),Xfp::OnOff(ptics->drawTics(Xfp::Graph_X,Xfp::TKIMinor))
 		 ,tab(),pminortics[Xfp::Graph_X]
 		 ,tab(),ptic_length[Xfp::Graph_X][Xfp::TKIMinor]
 		 ,plinestyle[Xfp::Graph_X][Xfp::TKIMinor]
 		 ->asCommands(Sprintf("%sx minor tic line",tab()))()
 		 ,tab(),Xfp::ticSideGuiName(pticlabelside[Xfp::Graph_X])
 		 ,tab(),plabeloffset[Xfp::Graph_X]
 		 ,tab(),pdecimals[Xfp::Graph_X]
 		 ,ptextstyle[Xfp::Graph_X]
 		 ->asCommands(Sprintf("%sx tic label font",tab()))()
 
 		 ,tab(),Xfp::OnOff(pautotics[Xfp::Graph_Y])
 		 ,tab(),Xfp::ticSideGuiName(pticside[Xfp::Graph_Y])
 		 ,tab(),Xfp::OnOff(ptics->drawTics(Xfp::Graph_Y,Xfp::TKIMajor))
 		 ,tab(),pmajortics[Xfp::Graph_Y]
		 ,tab(),Xfp::OnOff(pseparateyticlength[Xfp::TKIMajor])
 		 ,tab(),ptic_length[Xfp::Graph_Y][Xfp::TKIMajor]
 		 ,plinestyle[Xfp::Graph_Y][Xfp::TKIMajor]
 		 ->asCommands(Sprintf("%sy major tic line",tab()))()
 		 ,tab(),Xfp::OnOff(ptics->drawTics(Xfp::Graph_Y,Xfp::TKIMinor))
 		 ,tab(),pminortics[Xfp::Graph_Y]
		 ,tab(),Xfp::OnOff(pseparateyticlength[Xfp::TKIMinor])
 		 ,tab(),ptic_length[Xfp::Graph_Y][Xfp::TKIMinor]
 		 ,plinestyle[Xfp::Graph_Y][Xfp::TKIMinor]
 		 ->asCommands(Sprintf("%sy minor tic line",tab()))()
 		 ,tab(),Xfp::ticSideGuiName(pticlabelside[Xfp::Graph_Y])
 		 ,tab(),plabeloffset[Xfp::Graph_Y]
 		 ,tab(),pdecimals[Xfp::Graph_Y]
 		 ,ptextstyle[Xfp::Graph_Y]
 		 ->asCommands(Sprintf("%sy tic label font",tab()))()
		 ,tab(),pusertics->asCommands(tab)()
		 );




}

/*EOF*/ 
