/* @(#)TicMarks.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _USERTICKMARKS_H
#define _USERTICKMARKS_H 1

#include "GraphObject.hh"
#include "Xfp.hh"
#include "PtrArray.hh"
#include "FString.hh"

class FontStyle;
class Graph;
class LineStyle;

/* ******************************
 * the tic class  
 * ******************************
 */

class Tic
{
 public:
  Tic();
  Tic(const Tic &other);
  const Tic &operator=(const Tic &other);
  ~Tic();

  Xfp::XYAxisType type()const{return ptype;};
  void setType(Xfp::XYAxisType t){pchanged|=(ptype!=t);ptype=t;};
  
  Xfp::TicKind  kind()const{return pkind;};
  void setKind(Xfp::TicKind k){pchanged|=(pkind!=k);pkind=k;};

  double pos()const{return ppos;};
  void setPos(double p){pchanged|=(ppos!=p);ppos=p;};

  const String &label()const{return plabel;};
  void setLabel(const String &l){pchanged|=(plabel!=l);plabel=l;};

  Xfp::TicSide ticSide()const{return pticside;};
  void setTicSide(Xfp::TicSide s){pchanged|=(pticside!=s);pticside=s;};

  Xfp::TicSide ticLabelSide()const{return pticlabelside;};
  void setTicLabelSide(Xfp::TicSide s){pchanged|=(pticlabelside!=s);
    pticlabelside=s;};

  double ticLength()const{return pticlength;};
  void setTicLength(double l){pchanged|=(pticlength!=l);pticlength=l;};

  LineStyle *lineStyle(){return plinestyle;};
  const LineStyle *lineStyle()const{return plinestyle;};

  void setLineStyle(const LineStyle *ls);
  


  String asCommands(const String &tab,long index)const;

  bool changed()const;
  void setChanged(bool c);
  
 private:
  bool pchanged;
  Xfp::XYAxisType ptype;	/* x or y tics */
  Xfp::TicKind  pkind;		/* major or minor tic, major may have labels */
  double ppos;			/* at which world coordinate position */
  String plabel;  
  LineStyle *plinestyle;        /* individual linestyle  */
  Xfp::TicSide pticside;        /* which site of the graph shall it be */
  Xfp::TicSide pticlabelside;   /* which site of the graph shall it be */
  double pticlength;		/* length in ratio to frame */
};


/* ******************************
 * the tics class  
 * ******************************
 */

class Tics
{
 public:
  Tics();
  Tics(const Tics &other);
  const Tics &operator=(const Tics &other);

  void reset();

  bool drawTics(Xfp::XYAxisType xy,Xfp::TicKind mjmn)const{
    return pdrawtics[xy][mjmn];};
  
  void setDrawTics(Xfp::XYAxisType xy,Xfp::TicKind mjmn,bool d){
    pchanged|=(pdrawtics[xy][mjmn]!=d);pdrawtics[xy][mjmn]=d;};

  void setDrawTics(bool u);



  long size()const{return ptics.size();};
  void resize(long newsize,bool shrink=false){pchanged|=true;
    ptics.resize(newsize,shrink);};

  void deleteTic(long i){ptics.removeAt(i);};
  void insert(long index,Tic *t=0){
    if(t)
      ptics.insert(index,t);
    else
      ptics.insert(index,new Tic());
  };
  Tic *takeAt(long i){return ptics.takeAt(i);};

  Tic *tic(long i){return ptics[i];};
  const Tic *tic(long i)const{return ptics.at(i);};

  String asCommands(const String &tab)const;

  bool changed()const;
  void setChanged(bool );

  
 private:
  bool pchanged;
  bool pdrawtics[2][2];         /* generally switch on or of printing of 
				   user tics for this group, 
				   this overwrites individual drawtics 
				   property */
  
  PtrArray<Tic> ptics;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class TicMarks:public GraphObject
{
 public:
  TicMarks(Graph *parent);
  TicMarks(const TicMarks &other);
  const TicMarks &operator=(const TicMarks &other);
  ~TicMarks();
  
  void reset();

  FontStyle *textStyle(Xfp::XYAxisType xy){return ptextstyle[xy];};
  const FontStyle *textStyle(Xfp::XYAxisType xy)const{return ptextstyle[xy];};

  LineStyle *lineStyle(Xfp::XYAxisType xy,Xfp::TicKind k){
    return plinestyle[xy][k];};
  const LineStyle *lineStyle(Xfp::XYAxisType xy,Xfp::TicKind k)const{
    return plinestyle[xy][k];};

  Tics *tics(){return ptics;};
  const Tics *tics()const{return ptics;};
  Tics *userTics(){return pusertics;};
  const Tics *userTics()const{return pusertics;};

  double labelOffset(Xfp::XYAxisType t)const{return plabeloffset[t];};
  void setLabelOffset(Xfp::XYAxisType t,double d){
    pchanged|=(plabeloffset[t]!=d);plabeloffset[t]=d;};

  bool autoTics(Xfp::XYAxisType t)const{return pautotics[t];};
  void setAutoTics(Xfp::XYAxisType t,bool a){
    pchanged|=(pautotics[t]!=a);pautotics[t]=a;};

  double majorTics(Xfp::XYAxisType t)const{return pmajortics[t];};
  void setMajorTics(Xfp::XYAxisType t,double d){
    pchanged|=(pmajortics[t]!=d);pmajortics[t]=d;};

  long minorTics(Xfp::XYAxisType t)const{return pminortics[t];};
  void setMinorTics(Xfp::XYAxisType t,long m){
    pchanged|=(pminortics[t]!=m);pminortics[t]=m;};

  long ticSide(Xfp::XYAxisType t)const{return pticside[t];};
  void setTicSide(Xfp::XYAxisType t,Xfp::TicSide s){
    pchanged|=(pticside[t]!=s);pticside[t]=s;};

  long ticLabelSide(Xfp::XYAxisType t)const{return pticlabelside[t];};
  void setTicLabelSide(Xfp::XYAxisType t,Xfp::TicSide s){
    pchanged|=(pticlabelside[t]!=s);pticlabelside[t]=s;};

  long decimals(Xfp::XYAxisType t)const{return pdecimals[t];};
  void setDecimals(Xfp::XYAxisType t,long d){
    pchanged|=(pdecimals[t]!=d);pdecimals[t]=d;};

  double ticLength(Xfp::XYAxisType t,Xfp::TicKind k)const{
    return ptic_length[t][k];};
  void setTicLength(Xfp::XYAxisType t,Xfp::TicKind k,double d){
    pchanged|=(ptic_length[t][k]!=d);ptic_length[t][k]=d;};
  
  bool separateYTicLength(Xfp::TicKind k)const{
    return pseparateyticlength[k];
  };
  void setSeparateYTicLength(Xfp::TicKind k,bool b){
    pchanged|=(pseparateyticlength[k]!=b);pseparateyticlength[k]=b;
  };
					     

  virtual GraphObjectRegion insideRegion(const DragDescriptor &){
    return GraphObject::GORBody;};



  void calcTicMarks(Graph *graph,double widthOverHeight);

  String asCommands(const String &tab)const;

  bool changed()const;
  void setChanged(bool );

 private:
  static void calcAutoTicSpacing(double x0,double x1,double &mjtics
				 ,long &mntics,long &mjexfac,double &lxb0,
				 Xfp::GraphAxisScaling sc);
  
 private:
  /* regular tics */
  bool pautotics[2];		/* calc auto tics for x and y */
  double pmajortics[2];
  long   pminortics[2];
  double plabeloffset[2];
  Xfp::TicSide pticside[2];
  Xfp::TicSide pticlabelside[2];
  double ptic_length[2][2];
  bool pseparateyticlength[2];
  Tics *ptics;

  /* tic label props */
  long pdecimals[2];
  FontStyle *ptextstyle[2];
  LineStyle *plinestyle[2][2];	/* global x and y tic line style */


  /* user tics */
  Tics *pusertics;

};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _USERTICKMARKS_H */

