/*************************************************************************
 * @(#)ViewBox.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Apr 2011
 *************************************************************************
 */

#include "Attributes.hh"
#include "ViewBox.hh"
#include "defs.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ViewBox::ViewBox(Graph *parent)
  :GraphObject(parent)
  ,ppos(Point(0.120 * 1.5,0.166))
  ,psize(Size((1.-0.120)-ppos.x,(1.-0.166)-ppos.y))
  ,pdrawframe(true)
  ,prim(new LineStyle())
  ,pfill(new FillStyle())
{
  setObjectType(GOTViewBox);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ViewBox::~ViewBox()
{
  if(prim) delete prim;prim=0;
  if(pfill) delete pfill;pfill=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::reset()
{

  ppos=Point(0.120 * 1.5,0.166);
  psize=Size((1.-0.120)-ppos.x,(1.-0.166)-ppos.y);
  pdrawframe=true;
  prim->reset();
  pfill->reset();
  setChanged(false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ViewBox::changed()const
{
  return pchanged|prim->changed()|pfill->changed();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::setChanged(bool c){
  pchanged=c;
  prim->setChanged(c);
  pfill->setChanged(c);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::update()
{
  if(psize.w < 0.){
    ppos.x+=psize.w;
    psize.w=-psize.w;
  }
  if(psize.h<0.){
    ppos.y+=psize.h;
    psize.h=-psize.h;
  }
  if(psize.w==0)psize.w=0.001;
  if(psize.h==0)psize.h=0.001;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphObject::GraphObjectRegion 
ViewBox::insideRegion(const DragDescriptor &dd,bool update)
{

  Size si=psize*dd.devicesize;
  
  Point pul=ppos*dd.devicesize;

  if(update)pupdaterequest=GOCPosAndSize;
  
  if( dd.mousepoint.atPoint(pul,dd.sensitivityAroundPoint) ){
    return GORUL;
  }
  if( dd.mousepoint.atPoint(pul+Size(si.w,0),dd.sensitivityAroundPoint) ){
    return GORUR;
  }
  if( dd.mousepoint.atPoint(pul+Size(0,si.h),dd.sensitivityAroundPoint) ){
    return GORLL;
  }
  if( dd.mousepoint.atPoint(pul+si,dd.sensitivityAroundPoint) ){
    return GORLR;
  }
  if( dd.mousepoint.insideRect(pul,si)){
    if(update) pupdaterequest=GOCPosition;
    return GORBody;
  }
  if(update) pupdaterequest=GOCNone;
  return GORNone;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::initDrag(const DragDescriptor &dd)
{
  NSF("ViewBox::initDrag");

  GraphObjectRegion region=insideRegion(dd,true);

  if(region==GORNone)return;

  pmousepos=dd.mousepoint;
  
  if(region==GORUL){
    pupdaterequestnumber=0;
  }else if(region==GORUR){
    pupdaterequestnumber=1;
  }else if(region==GORLL){
    pupdaterequestnumber=2;
  }else if(region==GORLR){
    pupdaterequestnumber=3;
  }
  
  pobjectpos=0;
  pdeltasize=0;
  


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::updateDrag(const DragDescriptor &dd)
{
  //NSF("ViewBox::updateDrag");
  if(pupdaterequest == GOCPosition){
    pobjectpos+=(dd.mousepoint-pmousepos)/dd.devicesize;
  }else if((pupdaterequest & GOCSize) == GOCSize){
    if(pupdaterequestnumber==0){
      Point d=(dd.mousepoint-pmousepos)/dd.devicesize;
      pobjectpos+=d;
      pdeltasize-=d;
    }else if(pupdaterequestnumber==1){
      Point d=(dd.mousepoint-pmousepos)/dd.devicesize;
      pobjectpos+=Point(0,d.y);
      pdeltasize+=Point(d.x,-d.y);
    }else if(pupdaterequestnumber==2){
      Point d=(dd.mousepoint-pmousepos)/dd.devicesize;
      pobjectpos+=Point(d.x,0);
      pdeltasize+=Point(-d.x,d.y);
    }else if(pupdaterequestnumber==3){
      pdeltasize+=(dd.mousepoint-pmousepos)/dd.devicesize;
    }
  }
  pmousepos=dd.mousepoint;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ViewBox::setFromDrag(DragDescriptor &/*dd*/)
{
  //NSF("ViewBox::setFromDrag");

  if(pupdaterequest & GOCSize){
    setSize((psize+pdeltasize));
  }
  
  if(pupdaterequest & GOCPosition){
    setPos((ppos+pobjectpos));
  }
  pobjectpos=0;
  pdeltasize=0;

  update();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String ViewBox::asCommands(const String &tab)const
{
  return Sprintf("%s# view settings\n"
		 "%s %s"
		 "%s %s"
		 ,tab()
		 ,tab(),toCommands(InsertMenuData::View_Size_Pos)()
		 ,tab(),toCommands(InsertMenuData::View_Props,tab)()
		 );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String ViewBox::toCommands(InsertMenuData::Type request,const String &tab)const
{
  if(request==InsertMenuData::View_Size_Pos){
    return Sprintf("view %g,%g,%g,%g\n",ppos.x,ppos.y,psize.w,psize.h);
  }else if(request==InsertMenuData::View_Props){
    return Sprintf("view frame %s\n%s%s",Xfp::OnOff(pdrawframe)
		   ,prim->asCommands(tab+" view frame rim")()
		   ,pfill->asCommands(tab+" view frame fill")());
  }
  return "";
}


/*EOF*/ 
