/* @(#)ViewBox.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _VIEWBOX_H
#define _VIEWBOX_H 1

#include "GraphObject.hh"
#include "Point.hh"
#include "InsertMenuData.hh"



class Graph;
class LineStyle;
class FillStyle;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ViewBox: public GraphObject
{
 public:
  ViewBox(Graph *parent);
  ~ViewBox();

  void reset();
  
  LineStyle *rim(){return prim;};
  const LineStyle *rim()const{return prim;};

  FillStyle *fill(){return pfill;};
  const FillStyle *fill()const{return pfill;};

  Size size(bool dragging=false)const{
    if(dragging){
      return psize+pdeltasize;
    }else{
      return psize;
    }
  };
  void setSize(const Size &s){
    pchanged|=(psize!=s);psize=s;};

  Point pos(bool dragging=false)const{
    if(dragging)
      return ppos+pobjectpos;
    else
      return ppos;
  };
  void setPos(const Point &p){
    pchanged|=(ppos!=p);ppos=p;};

  bool drawFrame()const{return pdrawframe;};
  void setDrawFrame(bool a){
    pchanged|=(pdrawframe!=a);pdrawframe=a;};

  virtual GraphObjectRegion insideRegion(const DragDescriptor &dd,bool update);
  virtual void initDrag(const DragDescriptor &dd);
  virtual void updateDrag(const DragDescriptor &dd);
  virtual void setFromDrag(DragDescriptor &dd);
  
  // view coordinates to relative device coordinates
  Point vctodc(const Point &p)const{return p*psize+ppos;};
  Size  vctodc(const Size &p)const{return p*psize;};

  Point dctovc(const Point &p)const{return (p-ppos)/psize;};
  Size  dctovc(const Size &s)const{return s/psize;};

  String asCommands(const String &tab)const;

  String toCommands(InsertMenuData::Type request,const String &tab="")const;


  bool changed()const;
  void setChanged(bool c);

 private:
  void update();

 private:
  Point ppos;			/* pos of the upper left corner of the box
				   as fraction of the paper area */
  Size  psize;			/* the width and height of the view area
				   as fraction of the paper area
				 */
  /* --- frame box --- */
  bool pdrawframe; 
  LineStyle *prim;		/* the rim style  */
  FillStyle *pfill;		/* the fill style */

  
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _VIEWBOX_H */

