/* @(#)WeightLabels.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _WEIGHTLABELS_H
#define _WEIGHTLABELS_H 1

#include "FString.hh" 
#include "ValueArray.hh" 
#include "PtrArray.hh" 


class  WeightLabelsCollector;
class WeightLabels;

/* ***********************************************************************
 * a weight label
 * ***********************************************************************
 */
class WeightLabel{
 public:
  WeightLabel():plabel("weight"){};
  WeightLabel(const String &s):plabel(s){};

  inline const String &label()const{return plabel;};
  inline void setLabel(const String &s){plabel=s;};

 private:
  String plabel;
};

/* ***********************************************************************
 * the weightlabels of a certain function or set of functions
 * rudimentary garbage collection implemented
 * ***********************************************************************
 */
class WeightLabels{
 private: // only friends should create this
  WeightLabels(long ui):preferencecount(0),puniqueindex(ui){};
 public:
  ~WeightLabels();

  void analyse();

  bool inUse(){return (preferencecount>0);};
  void increaseUsage(){preferencecount++;};
  bool decreaseUsage(){preferencecount--;return (preferencecount<=0);};
  void append(const String &s){pwls.append(new WeightLabel(s));};
  long uniqueIndex()const{return puniqueindex;};
  long size()const{return pwls.size();};
  const WeightLabel *at(long i)const{return pwls.at(i);};
  void setLabel(int i,const String &s){
    if(i>=pwls.size()){
      pwls.resize(i+1);
      pwls[i]->setLabel(s);
    }else
      pwls[i]->setLabel(s);
  };
  void setLabels(const ValueArray<String> &ss){
    pwls.clear();
    for(long i=0;i<ss.size();i++){
      pwls.append(new WeightLabel(ss.at(i)));
    }
  };
  long indexOf(const WeightLabel *wl)const{
    return pwls.indexOf(wl);
  }

 private:
  long preferencecount;
  long puniqueindex;
  PtrArray<WeightLabel> pwls;
  friend class  WeightLabelsCollector;
};

/* ***********************************************************************
 * a colelction of globally available weigtlabels
 * ***********************************************************************
 */
class WeightLabelsCollector{
 public:
  WeightLabelsCollector();
  ~WeightLabelsCollector();
  
  WeightLabels *demandNewAuto();
  WeightLabels *demandNew(long definiteUniqueIndex);
  void remove(WeightLabels *);


  WeightLabels *getByUniqueIndex(long uniqueindex){
    for(long i=0;i<pwls.size();i++){
      if(pwls.at(i)->uniqueIndex()== uniqueindex)
	return pwls[i];
    }
    return 0;
  };

  void writeAsCommand(FILE *FHO,const String &tab);

  void collectGarbage();
  
 private:
  long pminuniqueindex;
  long plastuniqueindex;
  PtrArray<WeightLabels> pwls; 
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _WEIGHTLABELS_H */

