/*************************************************************************
 * @(#)WeightSelection.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Oct 2011
 *************************************************************************
 */

#include "WeightSelection.hh"
#include "MyMath.hh"
#include "Xfp.hh"
#include "MyDebug.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelections::updateFrom(WeightLabels *wl)
{
  if(!wl) return;

  if(!pdirty) return;

  //myDebug() << "WeightSelections::updateFrom wl="<<wl;

  long nl=wl->size();
  long ns=pdata.size();
  long n0=MyMath::min<long>(nl,ns);
  
  resize(nl,true);

  for(long i=0;i<n0;i++){
    pdata[i]->setWeightLabel(wl->at(i));
  }
  for(long i=n0;i<nl;i++){
    //myDebug()<<"updateFrom i="<<i;
    QRgb cc=Xfp::rgbColor((i % (Xfp::maxRgbColors()-2))+1);
    pdata[i]->setWeightLabel(wl->at(i));
    pdata[i]->setActive(false);
    pdata[i]->setColor(cc);
  }

  pdirty=false;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelections::resize(long newsize,bool shrink)
{
  long oldsize=pdata.size();
  long maxplotorder=-1;
  for(long i=0;i<oldsize;i++){
    maxplotorder=MyMath::max<long>(maxplotorder,pdata.at(i)->pplotorder);
  }
  pdata.resize(newsize,shrink);
  if(pdata.size()>oldsize){
    maxplotorder++;
    maxplotorder=MyMath::max<long>(maxplotorder,1);
    for(long i=oldsize;i<pdata.size();i++){
      pdata[i]->pplotorder=maxplotorder++;
    }
  }
  //myDebug()<<"resize oldsize="<<oldsize;
  //myDebug()<<"resize newsize="<<pdata.size();
  for(long i=0;i<pdata.size();i++){
    //myDebug()<<"resize plotorder="<<i<<pdata.at(i)->pplotorder;
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelections::append(WeightSelection *wd){
  pchanged=true;
  long oldsize=pdata.size();
  long maxplotorder=-1;
  for(long i=0;i<oldsize;i++){
    maxplotorder=MyMath::max<long>(maxplotorder,pdata.at(i)->pplotorder);
  }
  pdata.append(wd);
  if(pdata.size()>oldsize){
    maxplotorder++;
    maxplotorder=MyMath::max<long>(maxplotorder,1);
    for(long i=oldsize;i<pdata.size();i++){
      pdata[i]->pplotorder=maxplotorder++;
    }
  }
  //myDebug()<<"append oldsize="<<oldsize;
  //myDebug()<<"append newsize="<<pdata.size();
  for(long i=0;i<pdata.size();i++){
    //myDebug()<<"append plotorder="<<i<<pdata.at(i)->pplotorder;
    
  }
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String WeightSelections::asCommands(const String &tab,const WeightLabels *wls)const
{
  String s=Sprintf(
		   "%s  weight min %g\n"
		   "%s  weight max %g\n"
		   "%s  weight factor %g\n"
		   "%s  weight style %s\n"
		   ,tab(),pminwidth
		   ,tab(),pmaxwidth
		   ,tab(),pfactor
		   ,tab(),Xfp::weightsStylesGuiName(pstyle)
		   );	     
  
  long ns=pdata.size();
  for(long i=0;i<ns;i++){
    const WeightSelection *ws=pdata.at(i);
    long iweightdata=wls->indexOf(ws->weightLabel())+1;
    s+=Sprintf(
	       "%s  w%ld %s\n"
	       "%s  w%ld plot order %ld\n"
	       "%s  w%ld skip %ld\n"
	       "%s  w%ld color 0x%x\n"
	       "%s  w%ld symbol style %s\n"
	       "%s  w%ld symbol fill %s\n"
	       "%s  w%ld symbol line width %d\n"
	       ,tab(),iweightdata,Xfp::OnOff(ws->active())
	       ,tab(),iweightdata,i+1
	       ,tab(),iweightdata,ws->skip()
	       ,tab(),iweightdata,ws->color()
	       ,tab(),iweightdata,Xfp::symbolName(ws->type())
	       ,tab(),iweightdata,Xfp::OnOff(ws->fill())
	       ,tab(),iweightdata,ws->lineWidth()
	       );
  }
  return s;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelections::removeUndefined()
{
  for(long i=pdata.size()-1;i>=0;i--){
    if(!pdata.at(i)->weightLabel()){
      pdata.removeAt(i);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void WeightSelections::resortAccordingToPlotOrder()
{
  long ns=pdata.size();
  long pos=1;
  while(pos<ns){
    long apos=pdata.at(pos)->pplotorder;
    long aposm1=pdata.at(pos-1)->pplotorder;
//     myDebug()<<"pos="<<pos<<apos<<aposm1;

    if(apos>=aposm1){
      pos++;
    }else{
      WeightSelection *ws=pdata[pos];
      pdata[pos]=pdata[pos-1];
      pdata[pos-1]=ws;
      if(pos>1){
	pos--;
      }else{
	pos++;
      }
    }
  }

}


/*EOF*/ 
