/* @(#)WeightSelection.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _WEIGHTSELECTION_H
#define _WEIGHTSELECTION_H 1

#include "Attributes.hh"
#include "MyQRgb.hh" 
#include "PtrArray.hh" 
#include "WeightLabels.hh" 
#include "Xfp.hh" 




class WeightSelection{
 public:


  WeightSelection():pchanged(false),pwl(0),pactive(false),pplotorder(-1)
    ,pskip(0){
    setType(Xfp::STYSquare);
    setFill(true);
    setColor(0x00);
    psymbolstyle.setChanged(false);
  };
  WeightSelection(const WeightLabel *wl):pchanged(false),pwl(wl),
    pactive(false),pplotorder(-1),pskip(0){
    setType(Xfp::STYSquare);
    setFill(true);
    setColor(0x00);
    psymbolstyle.setChanged(false);
  };
  WeightSelection(const WeightSelection &other):pchanged(other.pchanged),
    pwl(other.pwl),
    pactive(other.pactive),
    psymbolstyle(other.psymbolstyle)
    ,pplotorder(other.pplotorder),pskip(other.pskip){};
  ~WeightSelection(){};

  void setUp(const WeightLabel *wl,bool a=true,QRgb c=0xff0000){
    pwl=wl;
    pactive=a;
    setColor(c);
  };

  QRgb color()const{return psymbolstyle.fillStyle().color();};
  void setColor(QRgb c){ psymbolstyle.fillStyle().setColor(c); };

  bool fill()const{ return psymbolstyle.fillStyle().fill(); };
  void setFill(bool f){ psymbolstyle.fillStyle().setFill(f); };

  long lineWidth()const{return psymbolstyle.lineStyle().width();};
  void setLineWidth(long w){psymbolstyle.lineStyle().setWidth(w);};
  
  Xfp::SymbolTypes type()const{return psymbolstyle.type();};
  void setType(Xfp::SymbolTypes s){psymbolstyle.setType(s);};

  const WeightLabel *weightLabel()const{return pwl;};
  void setWeightLabel(const WeightLabel *wl){pwl=wl;};

  inline const SymbolStyle &symbolStyle()const{return psymbolstyle;};

  long skip()const{return pskip;};
  void setSkip(long s){pchanged|=(pskip!=s);pskip=s;};

  bool active()const{return pactive;};
  void setActive(bool a){pchanged|=(pactive!=a);pactive=a;};


  void setPlotOrder(long po){pplotorder=po;};

  bool changed()const{return pchanged|psymbolstyle.changed();};
  void setChanged(bool c){pchanged=c;psymbolstyle.setChanged(c);};

 private:
  bool pchanged;
  const WeightLabel *pwl;
  bool pactive;
  SymbolStyle psymbolstyle;
  long pplotorder;
  long pskip;
  friend class WeightSelections;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class WeightSelections{
 public:
  WeightSelections():pchanged(false),pdirty(true),
    pfactor(1.0),pminwidth(0.2),pmaxwidth(6),
    pdisplayweightlegend(true),
    pstyle(Xfp::WESDots){};
  ~WeightSelections(){};

 
  // operator=() forbidden by empty definition ->linkage error if used
  WeightSelections &operator=(const WeightSelections &);


  void updateFrom(WeightLabels *weightLabels);

  void copySelections(const WeightSelections *other){
    pdata.clear();
    for(int i=0;i<other->pdata.size();i++){
      pdata.append(new WeightSelection(*(other->pdata.at(i))));
    }
  };

  String asCommands(const String &tab,const WeightLabels *)const;

  void setDirty(bool d){pdirty=d;};

  void clear(){pdata.clear();};

  WeightSelection *operator[](long i){return pdata[i];};
  const WeightSelection *at(long i)const{return pdata.at(i);};

  void resize(long newsize,bool shrink=false);



  void append(WeightSelection *wd);
  WeightSelection *last(){return pdata[pdata.size()-1];};
  long size()const{return pdata.size();};


  void removeUndefined();
  void resortAccordingToPlotOrder();

  Xfp::WeightsStyle style()const{return pstyle;};
  void setStyle(Xfp::WeightsStyle s){
    int ss=s;
    while(ss<0) ss+=Xfp::maxWeightsStyles();
    ss=ss%(Xfp::maxWeightsStyles());
    pchanged|=(pstyle!=ss);pstyle=(Xfp::WeightsStyle)ss;
  };

  double minWidth()const{return pminwidth;};
  void setMinWidth(double w){pchanged|=(pminwidth!=w);pminwidth=w;};

  double maxWidth()const{return pmaxwidth;};
  void setMaxWidth(double w){pchanged|=(pmaxwidth!=w);pmaxwidth=w;};

  double factor()const{return pfactor;};
  void setFactor(double f){pchanged|=(pfactor!=f);pfactor=f;};


  bool displayWeightLegend()const{return pdisplayweightlegend;};
  void setDisplayWeightLegend(bool a){pchanged|=(pdisplayweightlegend!=a);
    pdisplayweightlegend=a;
  };

  long indexOf(const WeightLabel *wd){
    for(int i=0;i<pdata.size();i++){
      if(pdata.at(i)->weightLabel()==wd)
	return i;
    }
    return -1;
  }

  long indexOf(const String &nm){
    for(int i=0;i<pdata.size();i++){
      const WeightLabel *wd=pdata.at(i)->weightLabel();
      if(wd){
	if(wd->label()==nm)
	return i;
      }
    }
    return -1;
  }


  WeightSelection *removeAt(int position){return pdata.takeAt(position);
    pchanged=true;
  };


  void insertNewWeightSelection(int position,WeightSelection *ws){
    pdata.insert(position,ws);
    pchanged=true;
  };


  bool changed()const{
    bool c=pchanged;
    for(int i=0;i<pdata.size();i++){
      c|=pdata.at(i)->changed();
    }
    return c;
  };
  void setChanged(bool c){
    pchanged=c;
    for(int i=0;i<pdata.size();i++){
      pdata[i]->setChanged(c);
    }
  }

 private:
  bool pchanged;
  bool pdirty;
  PtrArray<WeightSelection> pdata;
  double pfactor;
  double pminwidth;
  double pmaxwidth;
  bool pdisplayweightlegend;
  Xfp::WeightsStyle pstyle;
};
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _WEIGHTSELECTION_H */

