/*************************************************************************
 * @(#)World.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2011
 *************************************************************************
 */

#include <math.h>
#include "ViewBox.hh"
#include "World.hh"
#include "Graph.hh"
#include "GraphAxis.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::setW01(Xfp::XYAxisType t,double w0,double w1,
		   bool &adjusted0,bool &adjusted1)
{
  w0=pgraph->axes()->adjustUnscaled(t,w0,adjusted0);
  w1=pgraph->axes()->adjustUnscaled(t,w1,adjusted1);
  pchanged|=(ppw0[t]!=w0);ppw0[t]=w0;
  pchanged|=(ppw1[t]!=w1);ppw1[t]=w1;
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::setW0(Xfp::XYAxisType t,double w0)
{
  w0=pgraph->axes()->adjustUnscaled(t,w0);
  pchanged|=(ppw0[t]!=w0);ppw0[t]=w0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::setW1(Xfp::XYAxisType t,double w1)
{
  w1=pgraph->axes()->adjustUnscaled(t,w1);
  pchanged|=(ppw1[t]!=w1);ppw1[t]=w1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long World::checkWorld()
{
  double x;
  x=ppw1.x-ppw0.x;
  if(x < 0.){ x=ppw1.x; ppw1.x=ppw0.x; ppw0.x=x; }
  if(x == 0.){
    x=MyMath::max<double>(fabs(ppw0.x),1.);
    ppw0.x=pgraph->axes()->adjustUnscaled(Xfp::Graph_X,ppw0.x-x*.05);
    ppw1.x=pgraph->axes()->adjustUnscaled(Xfp::Graph_X,ppw1.x+x*.05);
  } 

  x=ppw1.y-ppw0.y;
  if(x < 0.){ x=ppw1.y; ppw1.y=ppw0.y; ppw0.y=x; }
  if(x == 0.){
    x=MyMath::max<double>(fabs(ppw0.y),1);
    ppw0.y=pgraph->axes()->adjustUnscaled(Xfp::Graph_Y,ppw0.y-x*.05);
    ppw1.y=pgraph->axes()->adjustUnscaled(Xfp::Graph_Y,ppw1.y+x*.05);
  }
  
  return 1;
}


/* ***********************************************************************
 * world to relative device coordinates
 * ***********************************************************************
 */
Point World::wctodc(double x,double y)const
{
  const ViewBox *vb=pgraph->viewBox();
  x=pgraph->axes()->toScaled(Xfp::Graph_X,x);
  y=pgraph->axes()->toScaled(Xfp::Graph_Y,y);
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return Point(x-w0.x,w1.y-y)*vb->size()/Size(w1-w0)+vb->pos();
}

Point World::wctodc(const Point &vv)const
{
  const ViewBox *vb=pgraph->viewBox();
  Point p=pgraph->axes()->toScaled(vv);
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return Point(p.x-w0.x,w1.y-p.y)*vb->size()/Size(w1-w0)+vb->pos();
}

Size World::wctodc(const Size &sin)const
{
  Size s=pgraph->axes()->toScaled(sin);
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return Size(s.w,-s.h)*pgraph->viewBox()->size()/Size(w1-w0);
}



double World::wctodc(double xory,Xfp::XYAxisType xy)const
{
  const ViewBox *vb=pgraph->viewBox();
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  if(xy==Xfp::Graph_X){
    xory=pgraph->axes()->toScaled(Xfp::Graph_X,xory);
    return vb->size().w*(xory-w0.x)/(w1.x-w0.x)+vb->pos().x;
  }else{
    xory=pgraph->axes()->toScaled(Xfp::Graph_Y,xory);
    return vb->size().h*(w1.y-xory)/(w1.y-w0.y)+vb->pos().y;
  }
}


double World::xwctoxdc(double x)
{
  const ViewBox *vb=pgraph->viewBox();
  x=pgraph->axes()->toScaled(Xfp::Graph_X,x);
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return vb->size().w*(x-w0.x)/(w1.x-w0.x)+vb->pos().x;
}



double World::ywctoydc(double y)
{
  const ViewBox *vb=pgraph->viewBox();
  y=pgraph->axes()->toScaled(Xfp::Graph_Y,y);
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return vb->size().h*(w1.y-y)/(w1.y-w0.y)+vb->pos().y;
}
/* ***********************************************************************
 * device coordinates to world coordinates
 * ***********************************************************************
 */
Point World::dctowc(const Point &v)const
{
  return pgraph->axes()->fromScaled(dctowcs(v));
}
Size World::dctowc(const Size &v)const
{
  return pgraph->axes()->fromScaled(dctowcs(v));
}

/* ***********************************************************************
 * device coordinates to scaled (lin/log) world coordinates
 * ***********************************************************************
 */
Point World::dctowcs(const Point &v)const
{
  const ViewBox *vb=pgraph->viewBox();
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  Point p=((v-vb->pos()-Point(0,vb->size().h))*Size(w1-w0)/vb->size())
    *Size(1,-1)+w0;
  return p;
}
Size World::dctowcs(const Size &v)const
{
  Point w0=pgraph->axes()->toScaled(ppw0);
  Point w1=pgraph->axes()->toScaled(ppw1);
  return (v*Size(w1-w0)/pgraph->viewBox()->size())*Size(1,-1);
}

/* ***********************************************************************
 * zoom positive is zoom out zoom negative is zoom in
 * ***********************************************************************
 */
long World::zoom(Xfp::XYAxisType xy,double zoom)
{
  /* NSF("zoom_graph"); */
  double d,a,s,l;

  
  if(zoom==0) return 1;
  a=fabs(zoom);
  s=zoom/a;
  
  
  if(xy ==Xfp::Graph_X){
    double w0=pgraph->axes()->toScaled(xy,ppw0.x);
    double w1=pgraph->axes()->toScaled(xy,ppw1.x);
    d=w1-w0;
    l=zoom*(1+s)/2. +zoom/(2.*a+1.)*(1-s)/2.; 
    
    
    w0-=l*d;
    w1+=l*d;
    ppw0.x=pgraph->axes()->fromScaled(xy,w0);
    ppw1.x=pgraph->axes()->fromScaled(xy,w1);
  }else{
    double w0=pgraph->axes()->toScaled(xy,ppw0.y);
    double w1=pgraph->axes()->toScaled(xy,ppw1.y);
    d=w1-w0;
    l=zoom*(1+s)/2. +zoom/(2.*a+1.)*(1-s)/2.; 
    
    
    w0-=l*d;
    w1+=l*d;
    ppw0.y=pgraph->axes()->fromScaled(xy,w0);
    ppw1.y=pgraph->axes()->fromScaled(xy,w1);
  }
  return 1;

}

/* ***********************************************************************
 * shift in x or y direction
 * ***********************************************************************
 */
long World::shiftXY(Xfp::XYAxisType xy,double shift)
{
  /* NSF("shiftXY"); */

  if(shift==0) return 1;
  double a=fabs(shift);
  double s=shift/a;
  
  if(xy ==Xfp::Graph_X){
    double w0=pgraph->axes()->toScaled(xy,ppw0.x);
    double w1=pgraph->axes()->toScaled(xy,ppw1.x);
    
    double d=w1-w0;
    
    w0+=s*a*d;
    w1+=s*a*d;
    ppw0.x=pgraph->axes()->fromScaled(xy,w0);
    ppw1.x=pgraph->axes()->fromScaled(xy,w1);
  }else{
    double w0=pgraph->axes()->toScaled(xy,ppw0.y);
    double w1=pgraph->axes()->toScaled(xy,ppw1.y);
    double d=w1-w0;
    
    w0+=s*a*d;
    w1+=s*a*d;
    ppw0.y=pgraph->axes()->fromScaled(xy,w0);
    ppw1.y=pgraph->axes()->fromScaled(xy,w1);
    
  }
  return 1;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::initDrag(const DragDescriptor &dd)
{
  pmousepos=dd.mousepoint;
  pobjectpos=dd.mousepoint;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::updateDrag(const DragDescriptor &dd)
{
  pmousepos=dd.mousepoint;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void World::setFromDrag(DragDescriptor &dd)
{
  

  if(  pmousepos.x==pobjectpos.x  || 
       pmousepos.y==pobjectpos.y   ) return;
  
  
  Point w0=dctowc(pmousepos/dd.devicesize);
  Point w1=dctowc(pobjectpos/dd.devicesize);
  if(w0.x>w1.x){ double d=w0.x; w0.x=w1.x; w1.x=d; }
  if(w0.y>w1.y){ double d=w0.y; w0.y=w1.y; w1.y=d; }
  
  if((pupdaterequest & GraphObject::GOCMeasureXY) ==GraphObject::GOCMeasureXY ){
    dd.w0=w0;
    dd.w1=w1;
    return;
  }

  if( pupdaterequest & GraphObject::GOCX){
    setW01(Xfp::Graph_X,w0.x,w1.x);
  }
  if( pupdaterequest & GraphObject::GOCY){
    setW01(Xfp::Graph_Y,w0.y,w1.y);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String World::asCommands(const String &tab)const
{
  return Sprintf("%s# world settings\n%sworld x %.15g , %.15g\n"
		 "%sworld y %.15g , %.15g\n"
		 "%sautoscale offset %g , %g\n"
		 ,tab(),tab()
		 ,ppw0.x,ppw1.x,tab()
		 ,ppw0.y,ppw1.y,tab()
		 ,pautoscale_offset[Xfp::Graph_X]
		 ,pautoscale_offset[Xfp::Graph_Y]
		 );
}


/*EOF*/ 
