/* @(#)World.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _WORLD_H
#define _WORLD_H 1

#include "GraphObject.hh"
#include "Point.hh"
#include "Xfp.hh"

class Graph;
class ViewBox;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class World: public GraphObject
{
 public:
  World(Graph *parent):GraphObject(parent){
    setObjectType(GraphObject::GOTWorld);
    ppw1.x=1.;
    ppw1.y=1.;
    
    pautoscale_offset[Xfp::Graph_X]=0.0;
    pautoscale_offset[Xfp::Graph_Y]=0.0;
    pchanged=false;
  };
  ~World(){};
  

  double w0(Xfp::XYAxisType t)const{
    if(t==Xfp::Graph_X)
      return ppw0.x;
    else
      return ppw0.y;
  }; 
  double w1(Xfp::XYAxisType t)const{
    if(t==Xfp::Graph_X)
      return ppw1.x;
    else
      return ppw1.y;
  };
  const Point &w0()const{return ppw0;};
  const Point &w1()const{return ppw1;};
  double autoScaleOffset(Xfp::XYAxisType t)const{return pautoscale_offset[t];};
  void setAutoScaleOffset(Xfp::XYAxisType t,double o){
    pchanged|=(pautoscale_offset[t]!=o);pautoscale_offset[t]=o;};

  inline void setW01(Xfp::XYAxisType t,double w0,double w1)
  {
    bool adjusted0,adjusted1;
    setW01(t,w0,w1,adjusted0,adjusted1);
  }
  void setW01(Xfp::XYAxisType t,double w0,double w1,
	       bool &adjusted0,bool &adjusted1);
  void setW0(Xfp::XYAxisType t,double w0);
  void setW1(Xfp::XYAxisType t,double w1);

  void setW0(const Point &w){
    pchanged|=(ppw0.x!=w.x);ppw0.x=w.x;
    pchanged|=(ppw0.y!=w.y);ppw0.y=w.y;
  }
  void setW1(const Point &w){
    pchanged|=(ppw1.x!=w.x);ppw1.x=w.x;
    pchanged|=(ppw1.y!=w.y);ppw1.y=w.y;
  }






  // world coords to device coords
  double xwctoxdc(double x);
  double ywctoydc(double y);
  double wctodc(double xory,Xfp::XYAxisType xy)const;
  Point wctodc(double x,double y)const;
  Point wctodc(const Point &v)const;
  Size  wctodc(const Size &p)const;
  
  // device coords to world coords
  Point dctowc(const Point &v)const;
  Size dctowc(const Size &v)const;
  // device coords to world scaled coords
  Point dctowcs(const Point &v)const;
  Size dctowcs(const Size &v)const;

  long zoom(Xfp::XYAxisType xy,double zoom);
  long shiftXY(Xfp::XYAxisType xy,double zoom);

  virtual GraphObjectRegion insideRegion(const DragDescriptor &,bool){
    return GraphObject::GORBody;};
  virtual void initDrag(const DragDescriptor &dd);
  virtual void updateDrag(const DragDescriptor &dd);
  virtual void setFromDrag(DragDescriptor &dd);

  long checkWorld();

  String asCommands(const String &tab)const;
  
  bool changed()const{return pchanged;};
  void setChanged(bool c){pchanged=c;};

 private:
  Point ppw0;
  Point ppw1;

  /* autoscaling */
  double pautoscale_offset[2];	/* how much offset shall be between the 
				   minimum and maximum autoscaled data
				   point and the frame
				*/
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _WORLD_H */

