/* @(#)Xfp.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ENUMS_H
#define _ENUMS_H 1

#include <string.h>
#include "MyQRgb.hh"

class Xfp
{
 public:
  enum FileType{FTXNY=0,FTXYNW=1,FTBand=2,FTBandWeight=3,FTXYNZ=4,
		FTGraph=5,FTUnknown=6,FTGraphOld=7,FTAkbl=8,FTParam=9
		,FTBandMatel=10
  };
  enum TicKind{TKIMajor=0,TKIMinor=1};
  enum TicSide{TSNoSide=0x0,TSNormal=0x1,TSOpposite=0x2
	       ,TSBoth=TSNormal|TSOpposite};
  enum XYAxisType{Graph_X=0,Graph_Y=1};
  enum GraphAxisScaling{GASLinear=0,GASLogarithmic=1};
  enum LineStyles{LSTNone=0,LSTSolid=1,
		  LSTDash=2,LSTDot=3,
		  LSTDashDot=4,
		  LSTDashDotDot=5,
		  LSTDotDashDash=6,
		  LSTLongDash=7,
		  LSTLongDashDot=8,
		  LSTLongDashDotDot=9,
		  LSTDotLongDashLongDash=10,
  };
  enum SymbolTypes{STYNone=0,
		   STYCircle=1,
		   STYSquare=2,
		   STYDiamond=3,
		   STYTriangleUp=4,
		   STYTriangleLeft=5,
		   STYTriangleDown=6,
		   STYTriangleRight=7,
		   STYPlus=8,
		   STYX=9,
		   STYStar=10,
		   STYChar=11};
  enum WeightsStyle{WESDots=0,WESConnected=1,WESIndividual=2};

  enum LineCapStyle{LCSButt=0,LCSRound=1,LCSSquare=2};

  enum ShapeType{SHTNone=0,SHTLine=1,SHTEllipse=2};

  enum ArrowPosition{ARPNone=0,ARPEnd=1,ARPStart=2,ARPBoth=3};
  enum ArrowStyle{ARSTOpen=0,ARSTClosed=1};


  enum CoordinateSystem{CSYView=0,CSYWorld=1};

  enum Fonts{HelveticaIsolatin1=0,HelveticaObliqueIsolatin1=1,Symbol=2};
  enum SpecialCharacter{
    LeftArrow ='\xac',
    UpArrow   ='\xad',
    RightArrow='\xae',
    DownArrow ='\xaf',
    Angstroem ='\xc5',
    Infinity  ='\xa5'
  };
  enum TextOrientation{
    TOLeft  =0x01,
    TORight =0x02,
    TOCenter=0x04,
    //    TOHAlign(x)  ((x)& 0x0f )
    TOVCenter=0x10,
    TOTop    =0x20,
    TOBottom =0x40,
    TOTopLeft=TOLeft|TOTop,
    TOTopCenter=TOCenter|TOTop,
    TOBottomCenter=TOCenter|TOBottom,
    TOVCenterRight=TORight|TOVCenter
    //#define PS_VALIGN(x)  ((x)& 0xf0 )
  };

  enum RGBColors{
    Black    =0,
    Red      =1,
    Green    =2,
    Blue     =3,
    Yellow   =4,
    Magenta  =5,
    Gray     =6,
    Orange   =7,
    Turquoise=8,
    Brown    =9,
    Indigo   =10,
    Maroon   =11,
    Violet   =12,
    White    =13
  };
  struct tsArrowStyles{
    ArrowStyle type;
    const  char *guiname;
  };

  static long ticMaxSides(){return pticmaxsides;};
  static const char *ticSideGuiName(long i){return pticsides[i].guiname;};
  static TicSide ticSideType(long i){return pticsides[i].type;};
  static TicSide getTicSiteFromString(const  char *s){
    for(long i=0;i<pticmaxsides;i++){
      if(strcmp(pticsides[i].guiname,s)==0){
	return pticsides[i].type;
      }
    }
    return TSNormal;
  };

  inline static XYAxisType otherAxisType(int t){
    return (XYAxisType)(Graph_Y-t);};

  static const char *shapeTypeGuiName(long i){return pshapetypes[i].guiname;};
  static long maxShapeTypes(){return pmaxshapetypes;};

  static const char *graphAxisScalingTypeGuiName(long i){
    return pgraphaxisscalingtypes[i].guiname;};
  static long maxGraphAxisScalingTypes(){return pmaxgraphaxisscalingtypes;};

  static const char *coordinateSystemGuiName(long i){
    return pcoordsystems[i].guiname;};
  static long maxCoordinateSystems(){return pmaxcoordinatesystems;};

  static const char *arrowPositionGuiName(long i){
    return parrowpositions[i].guiname;};
  static long maxArrowPositions(){return pmaxarrowpositions;};

  static const tsArrowStyles &arrowStyle(long s){
    for(int i=0;i<pmaxarrowstyles;i++){
      if(parrowstyles[i].type==s)
	return parrowstyles[i];
    }
    return parrowstyles[0];
  };
  static long maxArrowStyles(){return pmaxarrowstyles;};


  static long maxLineStyles(){return pmaxlinestyles;};
  static LineStyles getLineStyleFromString(const  char *s){
    int i;
    for(i=0;i<pmaxlinestyles;i++){
      if(strcmp(plinestyles[i].name,s)==0){
	return (LineStyles)i;
      }
    }
    return LSTSolid;
  };
  static const char *lineStyleGuiName(LineStyles l){
    return plinestyles[l].name;
  }
  static const char *lineStyleParserName(LineStyles l){
    return plinestyles[l].parsername;
  }
  static const long *lineStyleDashes(LineStyles l){
    return plinestyles[l].dashes;};
  
  static long maxSymbols(){return pmaxsymbols;};
  static const char *symbolName(Xfp::SymbolTypes t){
    return psymbols[t].name;}; 
  static double symbolBaseScale(Xfp::SymbolTypes t){
    return psymbols[t].basescale;};
 static bool symbolFillable(Xfp::SymbolTypes t){
    return psymbols[t].fillable;};
  static const char *symbolPSProcedure(Xfp::SymbolTypes t){
    return psymbols[t].proc;};
  static SymbolTypes getSymbolTypeFromString(const char *s){
    long i;
    for(i=0;i<pmaxsymbols;i++){
      if(strcmp(psymbols[i].name,s)==0){
	return (SymbolTypes)i;
      }
    }
    return STYNone;
  }

  static long maxWeightsStyles(){return pmaxweightsstyles;};
  static const char *weightsStylesGuiName(WeightsStyle s);

  static long maxRgbColors(){return prgbmaxcolors;};
  static QRgb rgbColor(int i){
    return prgbcolors[i];
  };

  static const double pointsToInch;
  static const double inchToCM;


  static const char* OnOff(bool o){return (o?"on":"off");};

 private:
  struct tsTicSides{
    TicSide type;
    const  char *guiname;
  };
  struct tsLineStyles{
    const char *name;
    const char *parsername;
    long dashes[7];
  };
  struct tsSymbols{
    const char *name;
    const char *proc;
    double basescale;
    bool fillable;
  };
  struct tsShapeTypes{
    ShapeType type;
    const  char *guiname;
  };
  struct tsGraphAxisScalingTypes{
    GraphAxisScaling type;
    const  char *guiname;
  };
  struct tsArrowPositions{
    ArrowPosition type;
    const  char *guiname;
  };

  struct tsCoordinateSystem{
    CoordinateSystem type;
    const  char *guiname;
  };

  static const tsTicSides pticsides[];
  static const long pticmaxsides;

  static const tsLineStyles plinestyles[];
  static const long pmaxlinestyles;
  static const long pmaxlinestylesdashes=6;

  static const tsSymbols psymbols[];
  static const long pmaxsymbols;

  static const QRgb  prgbcolors[];
  static const long  prgbmaxcolors;

  static const char *pstyles[];
  static const long pmaxweightsstyles;

  static const tsShapeTypes pshapetypes[];
  static const long pmaxshapetypes;

  static const tsGraphAxisScalingTypes pgraphaxisscalingtypes[];
  static const long pmaxgraphaxisscalingtypes;

  static const tsArrowPositions parrowpositions[];
  static const long pmaxarrowpositions;

  static const tsArrowStyles parrowstyles[];
  static const long pmaxarrowstyles;

  static const tsCoordinateSystem pcoordsystems[];
  static const long pmaxcoordinatesystems;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _ENUMS_H */

