/* @(#)MyDebug.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#include <stdarg.h>
#include "MyDebug.hh"
#include "FString.hh"
#include "ValueArray.hh"
#include "Vec3.hh"
#include "Cmplx.hh"
#include "Parser.hh"
#ifdef FMIN
#include "Variables.hh"
#endif
#ifdef QXFPL
#include "Point.hh"
#include "TicMarks.hh"
#endif
#if (defined QXFSF || defined QXFPL)
#include "Parser.hh"
#include <QString>
#endif

#ifdef QXFSF
#include "Plane.hh"
#include "Polyhedron.hh"
using namespace XFPLO;
#endif


MyDebug myDebug(const char *fmt,...)
{
  MyDebug dbg(stdout);  
  va_list ap;
  va_start(ap, fmt);
  vfprintf(dbg.FH, fmt, ap);
  va_end(ap);
  return dbg;
}


#ifdef FMIN
MyDebug operator<<(MyDebug dbg,const Value &v)
{
  dbg<<v.printS();
  return dbg;
}
#endif

MyDebug operator<<(MyDebug dbg,const Vec3 &v)
{
  dbg<<v.printS();
  return dbg;
}

#if (defined QXFPL)
MyDebug operator<<(MyDebug dbg,const Point &p)
{
  dbg<<" ( "<< p.x<<" , "<<p.y<<" ) ";
  return dbg;
}
#endif
MyDebug operator<<(MyDebug dbg,const Matrix3x3 &m)
{
  dbg<<" [\n"<< m.printS("%12.5g"," ")() <<" ] ";

  return dbg;
}



MyDebug operator<<(MyDebug dbg,const String &s)
{
  dbg<<s();
  return dbg;
}

MyDebug operator<<(MyDebug dbg,const Cmplx &d)
{
  fprintf(dbg.FH,"(%g,%g) ",d.real(),d.imag());
  return dbg;
}

MyDebug operator<<(MyDebug dbg,const StringList &s)
{
  fprintf(dbg.FH,"[");
  bool nl=false;
  long nlbreak=4;
  for(long i=0;i<s.size();i++){
    fprintf(dbg.FH,"\"%s\"",s[i]());
    if(i<s.size()-1)fprintf(dbg.FH,",");
    nl=false;
    if(((i+1) % nlbreak)==0 && i<s.size()-1){
      fprintf(dbg.FH,"\n");
      nl=true;
    }
  }
  if(!nl) fprintf(dbg.FH,"] ");
  return dbg;
}

MyDebug operator<<(MyDebug dbg,const ValueArray<double> &va)
{
  fprintf(dbg.FH,"[");
  bool nl=false;
  long nlbreak=4;
  for(long i=0;i<va.size();i++){
    fprintf(dbg.FH,"%g ",va.at(i));
    nl=false;
    //fprintf(dbg.FH,"%ld %ld %ld\n",i,nlbreak,i%nlbreak);
    if(((i+1) % nlbreak)==0 && i<va.size()-1){
      fprintf(dbg.FH,"\n");
      nl=true;
    }
  }
  if(!nl) fprintf(dbg.FH,"] ");
  return dbg;
}


#if (defined QXFSF || defined QXFPL)

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg,const QString &d)
{
  dbg<<d.toAscii().constData();
  return dbg;
}


#endif

#ifdef QXFPL

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Tics &p)
{
  for(int i=0;i<p.size();i++){
    const Tic *t=p.tic(i);
    dbg << " tic("<<i<<")=\n"
	<< "    drawtics ="<<p.drawTics(Xfp::Graph_X,Xfp::TKIMajor)
	<<" "<<p.drawTics(Xfp::Graph_X,Xfp::TKIMinor)
	<<" "<<p.drawTics(Xfp::Graph_Y,Xfp::TKIMajor)
	<<" "<<p.drawTics(Xfp::Graph_Y,Xfp::TKIMinor)<<"\n"
	<< *t;
    dbg << "    changed="<<p.changed()<<"\n";
    
  }
  return dbg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Tic &t)
{
  dbg << "        label="<<t.label()<<"\n";
  dbg << "        changed="<<t.changed()<<"\n";
  return dbg;
}

#endif

#ifdef QXFSF

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const Plane &p)
{
  dbg << " n=[ "<< p.n().printS() <<" ] d="<<p.d()<<"\n";
  
  return dbg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg, const PolyhedronProperties &p)
{
   dbg << "calculated="<<p.calculated()<<"\n";
   dbg << "visible="<<p.isVisible()<<"\n";
   dbg << "facecolor="<<Sprintf("0x%x",p.faceMaterial().color())<<"\n";
   dbg << "drawfaces="<<p.drawFaces()<<"\n";
   dbg << "drawnormals="<<p.drawNormals()<<"\n";
   dbg << "drawrim="<<p.drawRim()<<"\n";
   dbg << "rimcolor="<<Sprintf("0x%x",p.rimMaterial().color())<<"\n";
   dbg << "rimradius="<<p.rimRadius().S<<"\n";
   dbg << "rimradiusunits="<<p.rimRadiusUnits()<<"\n";
   dbg << "disprimradius="<<p.dispRimRadius()<<"\n";
    
  return dbg;
}



#endif
/*EOF*/
