/*************************************************************************
 * @(#)MyMath.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Mar 2011
 *************************************************************************
 */

#include <stdlib.h>
#include "MyMath.hh"
#include "MyDebug.hh"



/* ***********************************************************************
 * does line c1-c2 cross d1-d2  in 2D ?
 * point on line is r=c1+x*(c2-c1)
 * ***********************************************************************
 */
long lineCross2(double *c1,double *c2,double *c3,double *c4,double tol)
{
  // c1+x*(c2-c1)=c3+y*(c4-c3)
  // x*(c2-c1)+y*(c3-c4)=c3-c1
  double a00=c2[0]-c1[0];
  double a10=c2[1]-c1[1];
  double a01=-c4[0]+c3[0];
  double a11=-c4[1]+c3[1];

  double b0=c3[0]-c1[0];
  double b1=c3[1]-c1[1];

  double det=a00*a11-a01*a10;
  if(fabs(det) <tol) return 0; // parallel
  
  double l0=(a11*b0-a01*b1)/det;
  double l1=(-a10*b0+a00*b1)/det;
  return ((l0>0. && l0<1.) && (l1>0. && l1<1.));
  
}

/* ***********************************************************************
 *
 *    i,j: integer
 *    gcd: largest common divisor
 *    if(i == 0 && j==0) gcd=0 by definition, to allow iterative usage
 *
 * ***********************************************************************
 */

long MyMath::gcd(long i,long j)
{
  if(i==0 && j==0) {
    return 0;
  }
  if(labs(i)<labs(j)){
    long l=i;i=j;j=l;
  }
 
  while(j != 0) {
    long l=j;
    j=i%j;
    i=l;
  }
  return i;
  
}


long MyMath::gcd(long i,long j,long k)
{
  return gcd(i,gcd(j,k));
}

long MyMath::gcd(long i,long j,long k,long l)
{
  return gcd(gcd(i,l),gcd(j,k));
}


/* ***********************************************************************
 * simple integration where y is replaced by the running sum
 * ***********************************************************************
 */

void MyMath::runningSum(const ValueArray<double> *x,
			ValueArray<double> *y)
{
  // running integral int_{x0}^{xi} y(x) 
  if(x->size()!=y->size()) return;
  if(x->size()<1) return;
  double sum=0.;
  double y0=y->at(0);
  (*y)[0]=sum;
  for(long i=1;i<x->size();i++){
    double y1=(*y)[i];
    sum=sum+(y1+y0)*0.5*(x->at(i)-x->at(i-1));
    (*y)[i]=sum;
    y0=y1;
  }
  
}


/* ***********************************************************************
 * 
 * ***********************************************************************
 */
void MyMath::splitbaseN(double d,double base,double &mm,long &ee)
{
  // input: 
  //     real d
  //     real base
  // output:
  //     m,e: d=m*base^e such that |m| \in [1,10)
  if(d==0.){ mm=0.; ee=0; return;}
  
  ee=floor(log(fabs(d))/log(base));
  mm=d/pow(base,(double)ee);
  if(mm>=base){ mm/=10.; ee+=1; }
  if(mm<1){ mm*=10.; ee-=1; }
}


/*EOF*/ 
