/* @(#)MyMath.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _MYMATH_H
#define _MYMATH_H 1


#include <stdlib.h>
#include <math.h>
#include "ValueArray.hh"


#ifndef COMPARE
#define COMPARE(a,b) ((a)>(b) ? 1 : (((a)==(b))? 0 : -1))
#endif


const double Pi = 3.1415926535897932384626433832795028841968;
const double sqPi = 1.7724538509055160272981674833411451827974;
const double twopi  = 6.2831853071795864769252867665590057683936;
const double fourpi =12.5663706143591729538505735331180115367872;
const double sq3o2 =0.8660254037844386467637231707529361834714;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double *dinit(double *a,double a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long *linit(long *a,long a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline int *iinit(int *a,int a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double *ainit(double *a,double a0,long n)
{
  for(long i=0;i<n;i++) a[i]=a0;
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double dSign(double x,double y)
{
  return ( fabs(x)* ((y)>=0. ? 1.:-1.) );
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline double dRound(double x)
{
  return ((x)>0. ?( floor(x+.5) ) :(ceil(x-.5)));
}

inline long lRound(double x)
{
  return ((x)>0. ?((long)(x+.5)) :((long)(x-.5)));
}

inline unsigned long ulRound(double x)
{
  return ((x)>0. ?((unsigned long)(x+.5)) :((unsigned long)(x-.5)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline long lSign(long x,long y)
{
  return ( labs(x)* ((y)>=0 ? 1:-1) );
}

namespace MyMath{

  template<class T>
  inline T min(T x,const T &y){
    return (x<y?x:y);
  }

  template<class T>
  inline T min3(T x,const T &y,const T &z)
  {
    x=( x<y ? x : y  );
    return (x<z?x:z);
  }

  template<class T>
  inline T max(T x,const T &y){
    return (x>y?x:y);
  }
  template<class T>
  inline T max3(T x,const T &y,const T &z)
  {
    x=( x>y ? x : y  );
    return (x>z?x:z);
  }

  template<class T>
  T minVal(const ValueArray<T> &v){
    if(v.size()<1) return T(0);
    
    T m=v.at(0);;
    for(long i=0;++i<v.size();){
      m=MyMath::min<T>(m,v.at(i));
    }
    return m;
  }
  template<class T>
  T maxVal(const ValueArray<T> &v){
    if(v.size()<1) return T(0);
    
    T m=v.at(0);;
    for(long i=0;++i<v.size();){
      m=MyMath::max<T>(m,v.at(i));
    }
    return m;
  }
  
  
  template <class T>
  T ror(T x,int N,unsigned int moves)
  {
    // rotate the first N bits right by moves bits
    // example:  ror(00001100,4,1)=00000110
    // example:  ror(00001100,4,2)=00000011
    // example:  ror(00001100,4,3)=00001001
    // example:  ror(00001100,4,4)=00001100
    
    T mask=(1<<N)-1;
    
    return (x>>moves) | ((x<<(N-moves))&mask);
    
  }
  
  // template <class T>
  // int countBits(T x,int N)
  // {
  //   int count=0;
  //   for(int c=-1;++c<N;){
  //     count+=(x&1);
  //     x>>=1;
  //   }
  //   return count;
  // }

  template <class T>
  int bitCount(T x)
  {
    T c;
    for (c = 0; x; c++){
      x &= x - 1; 
    }
    return c;
  }
  
  template <class T>
  inline void swap(T &a,T &b){
    T c=a;
    a=b;
    b=c;
  }
  
  template <class T>
  inline void swap3(T &a,T &b,T &c){
    T t=a;
    a=b;
    b=c;
    c=t;
  }

  template <class T>
  inline void swap4(T &a,T &b,T &c,T &d){
    T t=a;
    a=b;
    b=c;
    c=d;
    d=t;
  }
  
  template <class T>
  inline T Abs(const T &a){
    return abs(a);
  }
  template <>
  inline long Abs(const long &a){
    return labs(a);
  }
  template <>
  inline float Abs(const float &a){
    return fabsf(a);
  }
  template <>
  inline double Abs(const double &a){
    return fabs(a);
  }

  // we need this for generic algos
  template <class T>
  inline double toDouble(const T &a){
    return a;
  }

  long gcd(long i,long j);
  long gcd(long i,long j,long k);
  long gcd(long i,long j,long k,long l);
  void runningSum(const ValueArray<double> *x,
		  ValueArray<double> *y);


  void splitbaseN(double d,double base,double &mm,long &ee);

  template <class T>
  class Factorial{
  public:
    Factorial(){}
    
    T operator()(long i)
    {
      long N=pfac.size();
      if(i<N) return pfac.at(i);
      pfac.resize(i+1);
      if(!N) pfac[N++]=T(1);
      for(long k=N;k<=i;k++) pfac[k]=pfac[k-1]*k;
      return pfac.at(i);
    }    

  private:
    ValueArray<T> pfac;
    
  };

  typedef Factorial<double> dFactorial;


}

#ifdef	__cplusplus
extern "C" {
#endif

extern long lineCross2(double *c1,double *c2,double *c3,double *c4,double tol);


#ifdef	__cplusplus
}
#endif


#endif /* _MYMATH_H */

