/*************************************************************************
 * @(#)Parser.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   08 Oct 2008
 *************************************************************************
 */

#include <string.h>


#include "tparser.h"
#include "Assorted.hh"
#include "merror.h"
#include "ioparser.h"
#include "Parser.hh"
#include "Except.hh"



#define PNC "%s::%s: "
#define PNS ,classname(),subfunc
#define THR(x) throw Except(classname+"::"+subfunc+": "+x)
#define THR2(x1,x2) throw Except(Sprintf(PNC x1 PNS ,x2))
#define THR3(x1,x2,x3) throw Except(Sprintf(PNC x1 PNS ,x2,x3))
#define THR4(x1,x2,x3,x4) throw Except(Sprintf(PNC x1 PNS ,x2,x3,x4))
#define MSG THR(errorbuffer)

// {{{Parser

const char *msg_uninitialized=PNC" object \"%s\" was not initialized correctly!";
const char *msg_valuenull=PNC"variable \"%s\" has invalid pointer value=NULL! ";
const char *msg_arprank=PNC" array \"%s\" multidimensional arrays not"
  " yet implemented!";
const char *msg_notscalar=PNC" object \"%s\" is not a scalar!";
const char *msg_notintscalar=PNC" object \"%s\" is not a scalar of integer type!";
const char *msg_notrealscalar=PNC" object \"%s\" is not a scalar of real type!";
const char *msg_notintarray=PNC" object \"%s\" is not an array of integer type!";
const char *msg_notrealarray=PNC" object \"%s\" is not an array of real type!";
const char *msg_notstringarray=PNC" object \"%s\" is not an array of string type!";
const char *msg_notarray=PNC" object \"%s\" is not a 1D array!";
const char *msg_notdeclared=PNC" variable \"%s\" not declared!";

#define NODE_NULL_ERROR \
  throw  Except(Sprintf(msg_uninitialized PNS,fullName()()))

#define VALUE_NULL_ERROR \
  throw  Except(Sprintf(msg_valuenull PNS,fullName()()))


#define ARP_RANK_ERROR \
  throw  Except(Sprintf(msg_arprank PNS,fullName()()))


#define NOT_SCALAR_ERROR \
  throw  Except(Sprintf(msg_notscalar PNS,fullName()()))

#define NOT_ARRAY_ERROR \
  throw  Except(Sprintf(msg_notarray PNS,fullName()()))

#define NOT_DECLARED_ERROR(x)					\
  throw  Except(Sprintf(msg_notdeclared PNS,x()))

#define NOT_AN_INTEGRAL_SCALAR\
  throw  Except(Sprintf(msg_notintscalar PNS,fullName()()))

#define NOT_AN_INTEGRAL_ARRAY\
  throw  Except(Sprintf(msg_notintarray PNS,fullName()()))

#define NOT_AN_REAL_SCALAR\
  throw  Except(Sprintf(msg_notrealscalar PNS,fullName()()))

#define NOT_AN_REAL_ARRAY \
  throw  Except(Sprintf(msg_notrealarray PNS,fullName()()))

#define NOT_A_STRING_ARRAY \
  throw  Except(Sprintf(msg_notstringarray PNS,fullName()()))

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Parser::Parser(long as)
  :classname("Parser"),padding(0),aschar(as),pfilename("")
{
  
  const char *subfunc="Parser(long)";
  pc=new PARSER_CONTROLL;
  if(!init_parser_control(pc,padding,aschar)) MSG;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Parser::Parser()
  :classname("Parser"),padding(0),aschar(1),pfilename("")
{
  
  const char *subfunc="Parser()";
  pc=new PARSER_CONTROLL;
  if(!init_parser_control(pc,padding,aschar)) MSG;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

Parser::~Parser()
{

  cleanup();
  delete pc; pc=NULL;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Parser::finalCleanUp()
{
  // clean up internal bison/yacc/lex stuff
  
  finalcleanup();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Parser::cleanup()
{
  const char *subfunc="cleanup()";
  if(!clean_vartable(&pc->table)) MSG;
  pfilename="";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Parser::parseFile(String const &filename)
{
  const char *subfunc="parseFile(String const&)";

  cleanup();
  
  if(!init_parser_control(pc,padding,aschar)) MSG;

  if(pathExistence(filename)==Directory){
    THR3(" File:'%s':\n%s\n",filename()," is a directory");
  }
  

  if(!parsefile(&yyin,filename(),pc) ) 
    THR3(" file:'%s':\n%s\n",filename(),errorbuffer);
  
  pfilename=filename;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Parser::writeFile(FILE *fo)
{
  const char *subfunc="writeFile(FILE*)";

  if(!print_vartab(fo,&pc->table)) MSG;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Parser::writeFile(const String &filename)
{
  FILE *FH;
  try{
    FH=openforwrite(filename);
  }catch(Except e){
    throw Except(String().sprintf("Cannot write file"
				  " \'%s\':\n %s\n!"
				  ,filename(),e.msg()));
  }
  
  try{
    this->writeFile(FH);
  }catch(Except e){
    fclose(FH);
    throw Except(String().sprintf("Cannot write file"
				  " \'%s\':\n %s\n!"
				  ,filename(),e.msg()));
  }
  fclose(FH);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Parser::operator()() const
{
  //NSF("operator()()");
 

  return Dobj(&pc->table,VTYPE_ROOT,"","",this);
}

// }}}


// {{{Dobj

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Dobj Dobj::operator[](long in) const
{
  const char *subfunc="operator[](long index)";

  
  if(node==0) NODE_NULL_ERROR;


  String ss=fullName()+"["+in+"]";
  

  if(structuretype==VTYPE_STRUCTARRAY){
    tsSTRUCTARRAY *sap;
    sap=SAP(node);
    
    if(in >= sap->Size)
      THR2(" struct array \"%s\" index out of bounds!",ss());
    
    return Dobj(sap->array[in],in,VTYPE_ROOT,ss,"",pparser);
    
  }else if(structuretype==VTYPE_ARRAY){
    return Dobj(node,in,VTYPE_SCALAR | VTYPE_ARRAY,ss,"",pparser);
  }else
    THR2(" object \"%s\" did not expect []!",ss());
  
  
  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Dobj::structureType()const
{
  const char *subfunc="structureType()";
  if(structuretype==VTYPE_SCALAR)
    return "scalar";
  else if(structuretype==VTYPE_ARRAY)
    return "array";
  else if(structuretype==(VTYPE_SCALAR | VTYPE_ARRAY))
    return "array element reference";
  else if(structuretype==VTYPE_STRUCT)
    return "struct";
  else if(structuretype==VTYPE_STRUCTARRAY)
    return "structarray";
  else if(structuretype==VTYPE_ROOT)
    return "table/structarray template/structarray root";
  else if(structuretype==VTYPE_SECTION)
    return "section";
  else if(structuretype==VTYPE_SECTIONEND)
    return "sectionend";
  else if(structuretype==VTYPE_PLACEMENT)
    return "placement";
  else if(structuretype==VTYPE_COMMENT)
    return "comment";
  else
    THR3(" variable %s: unkown structure type %ld!",fullName()(),structuretype);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Dobj Dobj::operator()(String const &n) const
{
  const char *subfunc="operator()(String const &)";
  VARTABLE *var;


  if(strstr(n(),"[")!=NULL)
    THR2(" variable %s: compound names not implemented!",n());
  
  if(strstr(n(),".")!=NULL){
    StringList sl=n.breakup(".");
    Dobj d=(*this)(sl[0]);
    for(long i=1;i<sl.size();i++)
      d=d(sl[i]);
    return d;
  }
  
  String fn=fullName();

  
  if( structuretype!=VTYPE_STRUCT && structuretype!=VTYPE_ROOT )
    THR3(" argument=\"%s\", object \"%s\" is not a struct or root!"
	 ,n(),fn());

  VARTABLE *tab=node;
  if(structuretype == VTYPE_STRUCT  ){
    tab=STP(node);
  }

  String ss=n;
  if(!fn.empty()) ss=fn+"."+n;
  

  if((var=lookup_list(tab,n(),NULL
		      ,VTYPE_STRUCT|VTYPE_STRUCTARRAY|VTYPE_ARRAY|VTYPE_SCALAR
		      ,0L))==NULL)
    NOT_DECLARED_ERROR(ss);
  
  if(var->structuretype==VTYPE_STRUCT)
    return Dobj(var,var->structuretype,fn,n,pparser);
  else
    return Dobj(var,var->structuretype,fn,n,pparser);
    
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList Dobj::list() const
{
  const char *subfunc="operator()()";
  char c[2]="";
  
  StringList sl;
  if(structuretype == VTYPE_SCALAR ||
     structuretype == (VTYPE_SCALAR | VTYPE_ARRAY)){
    sl.append((*this)());
  }else if(structuretype == VTYPE_ARRAY){
    tsARRAY7 *arp;
    arp=ARP(node);
    if(arp->rank!=1) ARP_RANK_ERROR;
    
    
    for(long k=0;k<arp->Size[1];k++){
      
      if( arp->scalartype==VTYPE_LOGICAL|| arp->scalartype==VTYPE_CHAR ){
	c[0]=((char*)arp->array)[k];
	sl.append(c);
      }else{
	const char *v=((char**)(arp->array))[k];
	if(v==0) THR2(" array \"%s\" array element pointer == 0!",fullName()());
	sl.append(v);
      }
    }
  }else{
    NOT_ARRAY_ERROR;
  }   

  return sl;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Dobj::operator()() const
{
  const char *subfunc="operator()()";

  char c[2]="";
  
  if(structuretype == VTYPE_SCALAR){
    tsSCALAR *scp=SCP(node);
    if(scp->value==NULL) VALUE_NULL_ERROR;
    if( scp->scalartype==VTYPE_LOGICAL|| scp->scalartype==VTYPE_CHAR ){
      c[0]=scp->value[0];
      return c;
    }else
      return scp->value;
  }else if(structuretype == (VTYPE_SCALAR | VTYPE_ARRAY)){
    tsARRAY7 *arp;
    arp=ARP(node);
    
    if(arp->rank!=1) ARP_RANK_ERROR;

    if(pindex >= arp->Size[1])
      THR2(" array \"%s\" index out of bounds!",fullName()());

    
    if( arp->scalartype==VTYPE_LOGICAL|| arp->scalartype==VTYPE_CHAR ){
      c[0]=((char*)arp->array)[pindex];
      return c;
    }else{
      const char *v=((char**)(arp->array))[pindex];
      if(v==0) THR2(" array \"%s\" array element pointer == 0!",fullName()());
      return v;
    }
    


    
  }else{
    NOT_SCALAR_ERROR;
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::valueL() const
{
  const char *subfunc="valueL()";

  char c[2]="";
  
  if(structuretype == VTYPE_SCALAR){
    tsSCALAR *scp=SCP(node);
    if(scp->value==NULL) VALUE_NULL_ERROR;
    if( scp->scalartype==VTYPE_LOGICAL ){
      if(node->aschar){
	if(scp->value[0]=='t' || scp->value[0]=='T')
	  return 1;
	else
	  return 0;
      }else{
	return (long)*((FPLOLOGICAL*)(scp->value));
      }
    }else if( scp->scalartype==VTYPE_INT ){
      if(node->aschar){
	const char *v=((char*)(scp->value));
	if(v==0) 
	  THR2(" array \"%s\" array element pointer == 0!",fullName()());
	return String(v).convertToLong();
      }else{
	return (long)*((FPLOINTEGER*)(scp->value));
      }

    }else if( scp->scalartype==VTYPE_CHAR ){
      return scp->value[0];
    }else
      NOT_AN_INTEGRAL_SCALAR;
  }else if(structuretype == (VTYPE_SCALAR | VTYPE_ARRAY)){
    tsARRAY7 *arp;
    arp=ARP(node);
    
    if(arp->rank!=1) ARP_RANK_ERROR;

    if(pindex >= arp->Size[1])
      THR2(" array \"%s\" index out of bounds!",fullName()());

    
    if( arp->scalartype==VTYPE_LOGICAL ){
      if(node->aschar){
	if(((char*)arp->array)[pindex]=='t' || ((char*)arp->array)[pindex]=='T')
	  return 1;
	else
	  return 0;
      }else{
	return (long)(((FPLOLOGICAL*)arp->array)[pindex]);
      }
    }else if( arp->scalartype==VTYPE_INT ){
      if(node->aschar){
	const char *v=((char**)(arp->array))[pindex];
	if(v==0) 
	  THR2(" array \"%s\" array element pointer == 0!",fullName()());
	return String(v).convertToLong();
      }else{
	return (long)(((FPLOINTEGER*)arp->array)[pindex]);
      }
    }else if( arp->scalartype==VTYPE_CHAR ){
      c[0]=((char*)arp->array)[pindex];
      return c[0];
    }else{
      NOT_AN_INTEGRAL_ARRAY;
    }
    


    
  }else{
    NOT_SCALAR_ERROR;
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Dobj::valueD() const
{
  const char *subfunc="valueD()";

  
  if(structuretype == VTYPE_SCALAR){
    tsSCALAR *scp=SCP(node);
    if(scp->value==NULL) VALUE_NULL_ERROR;
    if( scp->scalartype==VTYPE_REAL ){
      if(node->aschar){
	const char *v=((char*)(scp->value));
	if(v==0) 
	  THR2(" array \"%s\" array element pointer == 0!",fullName()());
	return (double) String(v).convertToFortranReal();
      }else{
	return (double)*((FPLODOUBLEPRECISION*)(scp->value));
      }
      
    }else
      NOT_AN_REAL_SCALAR;
  }else if(structuretype == (VTYPE_SCALAR | VTYPE_ARRAY)){
    tsARRAY7 *arp;
    arp=ARP(node);
    
    if(arp->rank!=1) ARP_RANK_ERROR;
    
    if(pindex >= arp->Size[1])
      THR2(" array \"%s\" index out of bounds!",fullName()());
    
    
    if( arp->scalartype==VTYPE_REAL ){
      if(node->aschar){
	const char *v=((char**)(arp->array))[pindex];
	if(v==0) 
	  THR2(" array \"%s\" array element pointer == 0!",fullName()());
	return String(v).convertToFortranReal();
      }else{
	return (double)(((FPLODOUBLEPRECISION*)arp->array)[pindex]);
      }
    }else{
      NOT_AN_REAL_ARRAY;
    }
    


    
  }else{
    NOT_SCALAR_ERROR;
  }
  
}

/* ***********************************************************************
 * here we use the convention data[i,j]=data[i+size_i*j] (i,j zero based)
 * ***********************************************************************
 */
void Dobj::getArray(long sz1,long sz2,double *data) const
{
  const char *subfunc="getArray(long sz1,long sz2,double *data)";

  if(node==0) NODE_NULL_ERROR;
  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp=ARP(node);

    if( arp->rank!=2) 
      THR2(" array \"%s\" dos not habe 2 dimensions!",fullName()());;
    
    if( arp->scalartype!=VTYPE_REAL ) NOT_AN_REAL_ARRAY;

    long size1=arp->Size[1];
    long size2=arp->Size[2];
    if( sz1 < size1 || sz2 < size2)
      THR4(" array \"%s\" larger than %ld x %ld!",fullName()(),sz1,sz2);

    for(long j=0;j<size2;j++){
      for(long i=0;i<size1;i++){
	long ind=i+j*size1;
	if(node->aschar){
	  data[i+j*sz1]=String(((char**)(arp->array))[ind])
	    .convertToFortranReal();
	}else{
	  data[i+j*sz1]=(double)(((FPLODOUBLEPRECISION*)arp->array)[ind]);
	}
      }
    }
  }else
    THR2(" object \"%s\" is not an array!",fullName()());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::getArray(long sz,double *data) const
{
  const char *subfunc="getArray(long size,double *data)";

  if(node==0) NODE_NULL_ERROR;
  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp=ARP(node);

    if( arp->rank!=1) ARP_RANK_ERROR;
    if( arp->scalartype!=VTYPE_REAL ) NOT_AN_REAL_ARRAY;
    if( sz < arp->Size[1])
      THR3(" array \"%s\" has more elements than %ld!",fullName()(),sz);

    for(long i=0;i<arp->Size[1];i++){
      if(node->aschar){
	data[i]=String(((char**)(arp->array))[i]).convertToFortranReal();
      }else{
	data[i]=(double)(((FPLODOUBLEPRECISION*)arp->array)[i]);
      }
    }
  }else
    THR2(" object \"%s\" is not an array!",fullName()());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::getArray(long sz,String *data) const
{
  const char *subfunc="getArray(long size,String *data)";

  if(node==0) NODE_NULL_ERROR;
  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp=ARP(node);

    if( arp->rank!=1) ARP_RANK_ERROR;
    if( arp->scalartype!=VTYPE_REAL ) NOT_AN_REAL_ARRAY;
    if( sz < arp->Size[1])
      THR3(" array \"%s\" has more elements than %ld!",fullName()(),sz);

    for(long i=0;i<arp->Size[1];i++){
      if(node->aschar){
	data[i]=((char**)(arp->array))[i];
      }else{
	//	HIERL(i);
	//	HIERD((double)(((FPLODOUBLEPRECISION*)arp->array)[i]));
	data[i]=Sprintf("%-30.15g"
			,(double)(((FPLODOUBLEPRECISION*)arp->array)[i]))
	  .removeTrailingBlanks();
	//	HIER(data[i]());
      }
    }
  }else
    THR2(" object \"%s\" is not an array!",fullName()());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::getStringArray(long sz,String *data) const
{
  const char *subfunc="getStringArray(long size,String *data)";

  if(node==0) NODE_NULL_ERROR;
  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp=ARP(node);

    if( arp->rank!=1) ARP_RANK_ERROR;
    if( arp->scalartype!=VTYPE_STRING ) NOT_A_STRING_ARRAY;
    
    if( sz < arp->Size[1])
      THR3(" array \"%s\" has more elements than %ld!",fullName()(),sz);

    for(long i=0;i<arp->Size[1];i++){
      data[i]=((char**)(arp->array))[i];
    }
  }else
    THR2(" object \"%s\" is not an array!",fullName()());
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::getArray(long sz,long *data) const
{
  const char *subfunc="getArray(long size,long *data)";

  if(node==0) NODE_NULL_ERROR;
  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp=ARP(node);

    if( arp->rank!=1) ARP_RANK_ERROR;
    if( arp->scalartype!=VTYPE_INT ) NOT_AN_INTEGRAL_ARRAY;
    if( sz < arp->Size[1])
      THR3(" array \"%s\" has more elements than %ld!",fullName()(),sz);

    for(long i=0;i<arp->Size[1];i++){
      if(node->aschar){
	data[i]=String(((char**)(arp->array))[i]).convertToLong();
      }else{
	data[i]=(long)(((FPLOINTEGER*)arp->array)[i]);
      }
    }
  }else
    THR2(" object \"%s\" is not an array!",fullName()());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::size(long i) const 
{
  const char *subfunc="size(long i)";


  if(node==0) NODE_NULL_ERROR;

  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp;
    arp=ARP(node);
    
    if(i<1 || i>7) 
      THR3("array variable '%s': dimension index i=%ld out of bound",fullName()(),i);

    return  arp->Size[i];
    
  }else if(structuretype==VTYPE_STRUCTARRAY){
    tsSTRUCTARRAY *sap;
    sap=SAP(node);

    if(i<1 || i>1) 
      THR3("struct array variable '%s': dimension index i=%ld out of bound",fullName()(),i);
    
    return  sap->Size;
  }else
    THR2(" object \"%s\" is not an (struct) array!",fullName()());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::getSizes(long s[8]) const 
{
  for(long i=1;i<=7;i++)s[i]=size(i);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::size() const 
{
  const char *subfunc="size()";


  if(node==0) NODE_NULL_ERROR;

  if(structuretype==VTYPE_ARRAY){
    tsARRAY7 *arp;
    arp=ARP(node);
    
    if(arp->rank!=1) ARP_RANK_ERROR;

    return  arp->Size[1];
  }else if(structuretype==VTYPE_STRUCTARRAY){
    tsSTRUCTARRAY *sap;
    sap=SAP(node);
    
    return  sap->Size;
  }else
    THR2(" object \"%s\" is not an (struct) array!",fullName()());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Dobj &Dobj::operator=(String const &s)
{
  const char *subfunc="operator=(String const &)";

  if(node==0) NODE_NULL_ERROR;
  

  if(structuretype==VTYPE_SCALAR){
    if( SCP(node)->scalartype==VTYPE_LOGICAL ){
      String v="f";
      if(s[0]=='t' || s[0]=='+') v="t";  
      if(!write_into_scalar(node,v(),WRITE_COPY)) MSG;
    }else
      if(!write_into_scalar(node,s(),WRITE_COPY)) MSG;
  }else if(structuretype==(VTYPE_SCALAR|VTYPE_ARRAY)){
    long indx[8];
    indx[1]=pindex+1;
    if(!write_into_array(node,indx,s(),WRITE_COPY)) MSG;
  }else{
    NOT_SCALAR_ERROR;
  }
  return *this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isArray() const
{

  return (structuretype==VTYPE_ARRAY);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isStructArray() const
{

  return (structuretype==VTYPE_STRUCTARRAY);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isStruct() const
{
  return (structuretype==VTYPE_STRUCT);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isFlag() const
{
  return (scalarType() == VTYPE_FLAG);
}

long Dobj::isInt() const
{
  return (scalarType() == VTYPE_INT);
}

long Dobj::isReal() const
{
  return (scalarType() == VTYPE_REAL);
}

long Dobj::isLogical() const
{
  return (scalarType() == VTYPE_LOGICAL);
}


long Dobj::isChar() const
{
  return (scalarType() == VTYPE_CHAR);
}

long Dobj::isString() const
{
  return (scalarType() == VTYPE_STRING);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::scalarType() const
{
  const char *subfunc="scalarType(long)";
  if(node==0) NODE_NULL_ERROR;
  
  switch ( structuretype )
    {
    case VTYPE_SCALAR:
      return (SCP(node)->scalartype);
    case VTYPE_ARRAY:
      return (ARP(node)->scalartype);
    case (VTYPE_SCALAR | VTYPE_ARRAY):
      return (ARP(node)->scalartype);
    case  VTYPE_STRUCT:
    case  VTYPE_STRUCTARRAY:
    case  VTYPE_ROOT:
      return 0;
    default:
      THR3("variable '%s' unimplemented structuretype %ld",fullName()()
	   ,structuretype);
    }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Dobj::isScalar()const
{
  return structuretype==VTYPE_SCALAR;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Dobj::rank()const
{
  const char *subfunc="rank()";
  if(node==0) NODE_NULL_ERROR;
  
  if(structuretype == VTYPE_ARRAY)
    return (ARP(node)->rank);
  
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isVariableSize()const
{
  const char *subfunc="isVariableSize()";
  if(node==0) NODE_NULL_ERROR;
  
  if(structuretype == VTYPE_ARRAY)
    return (ARP(node)->variable);
  else if(structuretype == VTYPE_STRUCTARRAY)
    return (SAP(node)->variable);
  
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::stringLength()const
{
  const char *subfunc="stringLength(long)";
  if(node==0) NODE_NULL_ERROR;
  
  switch ( structuretype )
    {
    case VTYPE_SCALAR:
      return (SCP(node)->strlength);
    case VTYPE_ARRAY:
      return (ARP(node)->strlength);
    case (VTYPE_SCALAR | VTYPE_ARRAY):
      return (ARP(node)->strlength);
    case  VTYPE_STRUCT:
    case  VTYPE_STRUCTARRAY:
    case  VTYPE_ROOT:
      return 0;
    default:
      THR3("variable '%s' unimplemented structuretype %ld",fullName()()
	   ,structuretype);
    }
  
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isVariableString()const
{
  const char *subfunc="isVariableString(long)";
  if(node==0) NODE_NULL_ERROR;
  
  switch ( structuretype )
    {
    case VTYPE_SCALAR:
      return (SCP(node)->strvariable);
    case VTYPE_ARRAY:
      return (ARP(node)->strvariable);
    case (VTYPE_SCALAR | VTYPE_ARRAY):
      return (ARP(node)->strvariable);
    case  VTYPE_STRUCT:
    case  VTYPE_STRUCTARRAY:
    case  VTYPE_ROOT:
      return 0;
    default:
      THR3("variable '%s' unimplemented structuretype %ld",fullName()()
	   ,structuretype);
    }
  
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::setDefaultValue(String const &v)
{
  const char *subfunc="setDefaultValue(long)";
  if(node==0) NODE_NULL_ERROR;


  if(structuretype == VTYPE_SCALAR){
    if(!set_defaultvalue_scalar(node,(char*)v())) MSG;
  }else if(structuretype == VTYPE_ARRAY){
    if(!set_defaultvalue_array(node,(char*)v())) MSG;
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::sapTemplate() const
{
  const char *subfunc="sapTemplate()";
  if(node==0) NODE_NULL_ERROR;

  if(structuretype!=VTYPE_STRUCTARRAY)
    THR2("object '%s': internal error: not a struct array ",fullName()());

  VARTABLE *n=SAP(node)->typetemplate;

  if(n==0)
    THR2("object '%s': internal error: typetemplate=0 ",fullName()());

  return Dobj(n,VTYPE_ROOT,"template("+fullName()+")","",pparser);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::sap(String const &n) const
{
  //const char *subfunc="sap(String const &)";
  
  long ok=1;
  Dobj d;
  try{ d=(*this)(n); }catch (...){ok=0;}
  
  if(!ok  ||  !d.isStructArray() ){
    throw Except(Sprintf("struct array variable '%s' not declared"
				  ,n()));
  }
  
  return d;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::arp(String const &n) const
{
  //const char *subfunc="arp(String const &)";
  
  long ok=1;
  Dobj d;
  try{ d=(*this)(n); }catch (...){ok=0;}
  
  if(!ok  ||  !d.isArray() ){
    throw Except(Sprintf("array variable '%s' not declared"
				  ,n()));
  }
  
  return d;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::scp(String const &n) const
{
  //const char *subfunc="scp(String const &)";
  
  long ok=1;
  Dobj d;
  try{ d=(*this)(n); }catch (...){ok=0;}
  
  if(!ok  ||  !d.isScalar() ){
    throw Except(Sprintf("scalar variable '%s' not declared"
				  ,n()));
  }
  
  return d;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::stp(String const &n) const
{
  //const char *subfunc="stp(String const &)";
  
  long ok=1;
  Dobj d;
  try{ d=(*this)(n); }catch (...){ok=0;}
  
  if(!ok  ||  !d.isStruct() ){
    throw Except(Sprintf("struct variable '%s' not declared"
				  ,n()));
  }
  
  return d;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::flagArp(String const &n) const
{
  //const char *subfunc="flagArp(String const &)";
  
  long ok=1;
  Dobj d;
  try{ 
    d=(*this)(n); 
  }catch (Except &e){ ok=0;}
  
  if(!ok  ||  !d.isArray() || !d.isFlag() ){
    throw Except(Sprintf("flag array variable '%s' not declared"
				  ,n()));
  }
  
  return d;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::first()
{
  //const char *subfunc="first()";
  
  if(node==NULL) return Dobj(node,0L,parentname,"",pparser);

  String pa=parentname;
  if(structuretype==VTYPE_STRUCT) pa=fullName();

  VARTABLE *var=0;
  if(structuretype==VTYPE_STRUCT)
    var=STP(node)->next;
  else if(structuretype==VTYPE_ROOT)
    var=node->next;
  
  if(var==NULL) return Dobj(var,0L,pa,"",pparser);

  String nn="";
  if(var->name) nn=var->name;

    
    

  return Dobj(var,var->structuretype,pa,nn,pparser);
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Dobj Dobj::next()
{
  //const char *subfunc="next()";
  
  if(node==NULL) return Dobj(node,0L,parentname,"",pparser);

  VARTABLE *var=node->next;
  
  if(var==NULL) return Dobj(var,0L,parentname,"",pparser);

  String nn="";
  if(var->name) nn=var->name;

  return Dobj(var,var->structuretype,parentname,nn,pparser);


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

String Dobj::fullName()const
{
  String ss;
  if(parentname.empty())
    ss=name;
  else{
    if(name.empty())
      ss=parentname;
    else
      ss=parentname+"."+name;
  }

  return ss;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Dobj::createTemplateFromDefValue(String const &n) 
{
  const char *subfunc="createTemplateFromDefValue(long)";


  VARTABLE* var=sapTemplate().node;

  if(!create_template_from_defaultvalue(var,n()))
    THR3("variable '%s': %s",fullName()(),errorbuffer);


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

long Dobj::hasIDRef() const
{
  const char *subfunc="hasIDRef";
  if(node==0) NODE_NULL_ERROR;

  if(structuretype == VTYPE_ARRAY)
    return (ARP(node)->idref[1] != NULL);


  if(structuretype == VTYPE_STRUCTARRAY)
    return (SAP(node)->idref != NULL);

  
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::hasIDRef(long i) const
{
  const char *subfunc="hasIDRef";
  if(node==0) NODE_NULL_ERROR;

  if(structuretype == VTYPE_ARRAY && i>=1 && i<=7)
    return (ARP(node)->idref[i] != NULL);

  if(structuretype == VTYPE_STRUCTARRAY && i==1)
    return (SAP(node)->idref != NULL);
  
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

String Dobj::IDRef(long i) const
{
  const char *subfunc="IDRef";
  String ss("");

  if(node==0) NODE_NULL_ERROR;

  if(structuretype == VTYPE_ARRAY && i>=1 && i<=7 ){
    if (ARP(node)->idref[i] != NULL)
      ss=ARP(node)->idref[i];
  }else if(structuretype == VTYPE_STRUCTARRAY && i==1 ){
    if (SAP(node)->idref != NULL)
      ss=SAP(node)->idref;
  }
  
  return ss;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

String Dobj::IDRef() const
{
  const char *subfunc="IDRef";
  String ss("");

  if(node==0) NODE_NULL_ERROR;

  if(structuretype == VTYPE_ARRAY){
    if (ARP(node)->idref[1] != NULL)
      ss=ARP(node)->idref[1];
  }else if(structuretype == VTYPE_STRUCTARRAY){
    if (SAP(node)->idref != NULL)
      ss=SAP(node)->idref;
  }
  
  return ss;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long Dobj::isDim() const 
{
  const char *subfunc="isDim";

  if(node==0) NODE_NULL_ERROR;

  if(structuretype == VTYPE_SCALAR){
    return SCP(node)->isdim;
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Dobj::resizeSap(long newsize) 
{
  const char *subfunc="resizeSap()";
  
  if(node==0) NODE_NULL_ERROR;
  
  if(structuretype == VTYPE_STRUCTARRAY){
    if(!resize_structarray(node,newsize))
      THR2("struct array '%s': resizing failed",fullName()());
  }else
    THR2("variable '%s': is not a struct array",fullName()());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Dobj::resizeArp(long newsizes[8]) 
{
  const char *subfunc="resizeArp()";
  
  if(node==0) NODE_NULL_ERROR;
  
  if(structuretype == VTYPE_ARRAY){
    if(!resize_vtarray(node,newsizes))
      THR2(" array '%s': resizing failed",fullName()());
  }else
    THR2("variable '%s': is not an array",fullName()());
  
}

// }}}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg,const Dobj &d)
{
  dbg << "parentName=" << d.parentName()<<"\n";
  dbg << "fullName=" << d.fullName()<<"\n";
  if(d.isScalar())dbg << "scalar" << "\n";
  if(d.isArray())dbg << "array" << "\n";
  if(d.isStructArray())dbg << "struct[]" << "\n";
  if(d.isStruct())dbg << "struct" << "\n";

  if(d.isInt()) dbg << "integer" << "\n";
  if(d.isReal()) dbg << "real" << "\n";
  if(d.isLogical()) dbg << "logical" << "\n";
  if(d.isFlag()) dbg << "flag" << "\n";
  if(d.isChar()) dbg << "char" << "\n";
  if(d.isString()) dbg << "char[]" << "\n";

  return dbg;
}

/*EOF*/ 
