/*************************************************************************
 * @(#)ValueArray.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 Dec 2011
 *************************************************************************
 */
#include <stdlib.h>
//#include "math.h"
#include "MyMath.hh"
#include "ValueArray.hh"
#include "MyDebug.hh"
#include "FString.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double norm(const ValueArray<double> &va)
{
  double d=0;
  for(int i=0;i<va.size();i++){
    d+=va.at(i)*va.at(i);
  }
  return sqrt(d);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<double> abs(const ValueArray<double> &v)
{
  ValueArray<double> res;
  res.resize(v.size());
  for(long i=0;i<v.size();i++){
    res[i]=fabs(v.at(i));
  }
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<double> max(const ValueArray<double> &va,const ValueArray<double> &vb)
{
  if(va.size()!=vb.size()){
    printf("ValueArray::max(a,b) different length\n");
    exit(1);
  }
  ValueArray<double> res;
  res.resize(va.size());
  for(long i=0;i<va.size();i++){
    res[i]=MyMath::max<double>(va.at(i),vb.at(i));
  }
  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<double> pow(const ValueArray<double> &va,double exp)
{
  ValueArray<double> res;
  res.resize(va.size());
  for(long i=0;i<va.size();i++){
    res[i]=pow(va.at(i),exp);
  }
  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ValueArray<double> sqrt(const ValueArray<double> &va)
{
  ValueArray<double> res;
  res.resize(va.size());
  for(long i=0;i<va.size();i++){
    res[i]=sqrt(va.at(i));
  }
  return res;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
void ValueArray<T>::dump()
{
  myDebug()<<"ValueArray<"<<typeid(T).name()<<">: pinternalsize="
	   <<pinternalsize;
  for(int i=0;i<pinternalsize;i++){
    myDebug()<<"ValueArray<"<<typeid(T).name()<<">: ["<<i<<"]="<<array[i];
  }
}


/*EOF*/ 
