/* @(#)ValueArray.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _VALUEARRAY_H
#define _VALUEARRAY_H 1

#include <stdio.h> 
#include <string.h> 
#include <stdlib.h> 
#include <math.h> 
#include <typeinfo>
#ifdef MY_DEBUGGING
#include "MyDebug.hh"
#endif


/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
class ValueArray
{
 public:
  ValueArray():psize(0),pinternalsize(0),array(0),pgrowthfactor(1.2){};
  ValueArray(long size,const T &tinit):psize(0),pinternalsize(0),array(0),
				       pgrowthfactor(1.2){
    resize(size);*this=tinit;
  };
  ValueArray(const T *p,long count)
    :psize(count),pinternalsize(count),array(new T[count]),pgrowthfactor(1.2){
    for(int i=0;i<psize;i++) array[i]=p[i];
  }
  ValueArray(const ValueArray &other){
    psize=other.psize;
    pinternalsize=psize;
    pgrowthfactor=other.pgrowthfactor;
    array=new T[psize];
    for(int i=0;i<psize;i++) array[i]=other.array[i];
  };
  ValueArray &operator=(const ValueArray &other){
    if(array) delete [] array;
    psize=other.psize;
    pinternalsize=psize;
    pgrowthfactor=other.pgrowthfactor;
    array=new T[psize];
    for(int i=0;i<psize;i++) array[i]=other.array[i];
    return *this;
  };

  ~ValueArray(){clear();};

  void setGrowthFactor(double f){pgrowthfactor=f;}

  void clear(){
    if(array) delete [] array;
    array=0;
    psize=0;
    pinternalsize=0;
  };
  inline long size() const{return psize;};

  ValueArray &operator<<(const T &v){
    this->append(v);
    return *this;
  }

  ValueArray &operator<<(const ValueArray &v){
    this->append(v);
    return *this;
  }

  bool operator!=(const ValueArray &b)const{
    if(psize!=b.psize)return true;
    for(int i=0;i<psize;i++){
      if((b.indexOf(array[i])<0)){
	return true;
      }
    }
    for(int i=0;i<b.psize;i++){
      if((indexOf(b.at(i))<0)){
	return true;
      }
    }
    return false;
  };

  T sum()const{
    T res=0;
    for(int i=0;i<psize;i++){
      res+=array[i];
    }
    return res;
  };

  T prod()const{
    T res=1;
    for(int i=0;i<psize;i++){
      res*=array[i];
    }
    return res;
  };

  ValueArray<T> &operator+=(const ValueArray &vb){
#ifdef MY_DEBUGGING
    if(psize!=vb.psize){
      printf("ValueArray::operator+=() differebnt length\n");
      exit(1);
    }
#endif
    for(int i=0;i<psize;i++){
      array[i]+=vb.array[i];
    }
    return *this;
  }

  ValueArray<T> &operator-(){
    for(int i=0;i<psize;i++){
      array[i]=-array[i];
    }
    return *this;
  }

  ValueArray<T> &operator/=(double s){
    for(int i=0;i<psize;i++){
      array[i]/=s;
    }
    return *this;
  }

  long indexOf(const T &t)const{
    for(int i=0;i<psize;i++){
      if(array[i]==t)
	return i;
    }
    return -1;
  };

  // find data, which have an operator==()
  // by specifing a few keys in oeprator==() and
  // return the full record (to simulate dictionaries)
  T *find(const T &t)const{
    for(int i=0;i<psize;i++){
      if(array[i]==t)
	return &(array[i]);
    }
    return 0;
  };

  const T &last()const{
#ifdef MY_DEBUGGING
    if(psize<1){
      printf("ValueArray: no elements\n");
      exit(1);
    }
#endif
    return array[psize-1];
  }
  T &last(){
#ifdef MY_DEBUGGING
    if(psize<1){
      printf("ValueArray: no elements\n");
      exit(1);
    }
#endif
    return array[psize-1];
  }

  T pop(){
#ifdef MY_DEBUGGING
    if(psize<1){
      printf("ValueArray: no elements\n");
      exit(1);
    }
#endif
    T d=array[psize-1];
    resize(psize-1);
    return d;
  }

  inline const T &at(long index)const{
#ifdef MY_DEBUGGING
    if(index<0 || index>=psize){
      printf("ValueArray::at: index out of bounds\n");
      exit(1);
    }
#endif
    return array[index];
  };

  inline const T *constDataRef()const{
    return array;
  };
  inline  T *dataRef(){
    return array;
  };

  bool contains(const T&v)const{
    for(long i=0;i<psize;i++){
      if(array[i]==v){
	return true;
      }
    }
    return false;
  };

  bool contains(const T&v,double tol)const{
    for(long i=0;i<psize;i++){
      if(fabs(array[i]-v)<tol){
	return true;
      }
    }
    return false;
  };

  inline T &operator[](long index){
#ifdef MY_DEBUGGING
    if(index<0 || index>=psize){
      printf("ValueArray::operator[]: index out of bounds\n");
      exit(1);
    }
#endif
    return array[index];
  };


  void resize(long newsize,bool shrink=false){
    if(shrink){
      if(newsize==pinternalsize){
	psize=newsize;
	return;
      }
      if(newsize<pinternalsize){
	pinternalsize=newsize;
      }else{
	long l=(long)(pinternalsize*pgrowthfactor);
	pinternalsize=(newsize>l?newsize:l);
      }
    }else{
      if(newsize<=pinternalsize) {
	psize=newsize;
	return;
      }
      long l=(long)(pinternalsize*pgrowthfactor);
      //l=(pinternalsize+10>l?pinternalsize+10:l);
      pinternalsize=(newsize>l?newsize:l);
    }
    
    
    
    T *temp=new T[pinternalsize];
    for(int i=0;i<(newsize<psize?newsize:psize);i++) temp[i]=array[i];
    T *c=array;
    array=temp;
    
    delete [] c;
    psize=newsize;

    //dump();
    
    return;
  };

  void setSize(long newsize){
    psize=newsize;
    pinternalsize=psize;
    
    if(array) delete [] array;
    
    array=new T[pinternalsize];
    
    return;
  };

  ValueArray &operator=(const T&v){
    if(!array) return *this;
    for(long i=0;i<psize;i++){
      array[i]=v;
    }
    return *this;
  }

  ValueArray &operator=(const T *v){
    if(!array) return *this;
    for(long i=0;i<psize;i++){
      array[i]=v[i];
    }
    return *this;
  }


  void append(const ValueArray<T>&a){
    
    long n=psize;
    long na=a.size();
    resize(n+na,false);
    for(long i=0;i<na;i++){
      array[n+i]=a.array[i];
    }
  }

  inline void append3(const T *a){
    long n=psize;
    resize(n+3,false);
    array[n]=a[0];
    array[n+1]=a[1];
    array[n+2]=a[2];
  }

  void append(const T &l){
    long i=psize;
    resize(i+1,false);
    array[i]=l;
    psize=i+1;
  };

  void replace(long i,const T &l){
    array[i]=l;
  }

  void prepend(const T &l){
    resize(psize+1,false);
    for(long i=psize-1;i>0;i--){
      array[i]=array[i-1];
    }
    array[0]=l;
  };

  void insert(long index,const T &l){
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::insert index="<<index<<"psize="<<psize
	     <<"pinternalsize="<<pinternalsize;
#endif
    index=(index<0?0:(index>=psize?psize:index));
    psize++;
    pinternalsize++;
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::insert newindex="<<index<<"newpsize="<<psize
	     <<"new pinternalsize="<<pinternalsize;
#endif
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::insert allocate temp";
#endif
    T *temp=new T[pinternalsize];
    for(long i=0; i<index; i++) {
#ifdef MY_DEBUGGING
      myDebug()<<"ValueArray::insert temp["<<i<<"]=array["<<i<<"]";
#endif
      temp[i] = array[i];
    }
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::insert temp["<<index<<"]=l"<<(void*)&l;
#endif
    temp[index]=l;
    for(long i=index+1; i<psize; i++) {
#ifdef MY_DEBUGGING
      myDebug()<<"ValueArray::insert temp["<<i<<"]=array["<<i-1<<"]";
#endif
      temp[i] = array[i-1];
    }
    swapint(temp);
    if(temp) delete [] temp;
  };

  T takeAt(long j){
    T tb=array[j];
    removeAtInternal(j);
    return tb;
  }
  
  void removeAt(long j){removeAtInternal(j);};
  void remove(const T &l){
    long j=indexOf(l);
    if(j>=0) removeAtInternal(j);
  };
  void removeLast(){if(psize) psize--;};



  void createMesh(const T &x0,const T &x1,long N){
    if(N<1) return;
    resize(N,true);
    for(int i=0;i<N;i++){
      array[i]=(x1-x0)*i/(N-1)+x0;
    }
  }

  void qSort(){
    qsort(array,psize,sizeof(T),qSortCmpT);
  }

  typedef int (*sortCallBackType)(const T *p1, const T *p2) ;


  void qSort(sortCallBackType sortFunc){
    qsort(array,psize,sizeof(T),(int (*)(const void *p1, const void *p2) )sortFunc);
  }

  void unify(){//T &padding){
    qsort(array,psize,sizeof(T),qSortCmpT);
    {
      if(psize==0)return;
      T ia=array[0];
      long i=0;
      for(long j=1;j<psize;j++){
	if (array[j] > ia ){
	  ia=array[j];
	  i++;
	  array[i]=array[j];
	}
      }
      // the remaining elements are undefined, 
      // for(long j=i+1;j<psize;j++){
      //  	array[j]=padding;
      // }
      psize=i+1;
    }
  }

  void swap(long i,long j){
#ifdef MY_DEBUGGING
    if((i<0||i>=psize )||(j<0||j>=psize)){
      printf("ValueArray:swap: index out of bounds\n");
      exit(1);
    }
#endif
    T d=array[i];array[i]=array[j];array[j]=d;
  }

  void dump();

 private:
  void removeAtInternal(long j){
#ifdef MY_DEBUGGING
    if(j<0||j>=psize){
      printf("ValueArray:removeAtInternal: index out of bounds\n");
      exit(1);
    }
#endif
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::removeAtInternal l"<<j<<"psize="<<psize;
#endif
    
    psize--;
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::removeAtInternal new psize="<<psize;
#endif
#ifdef MY_DEBUGGING
    myDebug()<<"ValueArray::removeAtInternal allocate temp pinternalsize="<<pinternalsize;
#endif
    T *temp=new T[pinternalsize];
    
    for(long i=0;i<j;i++) {
#ifdef MY_DEBUGGING
      myDebug()<<"ValueArray::removeAtInternal temp["<<i<<"]=array["<<i<<"]";
#endif
      temp[i]=array[i];
    }
    for(long i=j;i<psize;i++) {
#ifdef MY_DEBUGGING
      myDebug()<<"ValueArray::removeAtInternal temp["<<i<<"]=array["<<i+1<<"]";
#endif
      temp[i]=array[i+1];
    }
    
    swapint(temp);
    if(temp) delete [] temp;
  
  }

  void swapint(T *&temp){
    T *c=array;
    array=temp;
    temp=c;
  }


  static int qSortCmpT(const void *p1, const void *p2){
    if(*((T*)p1)<*((T*)p2))return -1;
    if(*((T*)p1)>*((T*)p2))return 1;
    //if(*p1<*p2)return -1;
    //if(*p1>*p2)return 1;
    return 0;
  }
 private:
  long psize;
  long pinternalsize;
  T *array;
  double pgrowthfactor;
};


double norm(const ValueArray<double> &va);






template <class T>
ValueArray<T> operator+(const ValueArray<T> &va,const ValueArray<T> &vb){
#ifdef MY_DEBUGGING
  if(va.size()!=vb.size()){
    printf("ValueArray::operator+() different length\n");
    exit(1);
  }
#endif
  ValueArray<T> res;
  res.resize(va.size());
  for(int i=0;i<va.size();i++){
    res[i]=va.at(i)+vb.at(i);
  }
  return res;
}

template <class T>
ValueArray<T> operator-(const ValueArray<T> &va,const ValueArray<T> &vb){
#ifdef MY_DEBUGGING
  if(va.size()!=vb.size()){
    printf("ValueArray::operator-() different length\n");
    exit(1);
  }
#endif
  ValueArray<T> res;
  res.resize(va.size());
  for(int i=0;i<va.size();i++){
    res[i]=va.at(i)-vb.at(i);
  }
  return res;
}

template <class T>
ValueArray<T> operator/(const ValueArray<T> &va,double s){
  ValueArray<T> res;
  res.resize(va.size());
  for(int i=0;i<va.size();i++){
    res[i]=va.at(i)/s;
  }
  return res;
}

template <class T>
//scaling product
ValueArray<T> operator*(const ValueArray<T> &va,const T &s)
{
  ValueArray<T> res;
  res.resize(va.size());
  for(long i=0;i<va.size();i++){
    res[i]=va.at(i)*s;
  }
  return res;
}

template <class T>
//elemet wise
ValueArray<T> operator/(const ValueArray<T> &va,const ValueArray<T> &vb)
{
#ifdef MY_DEBUGGING
  if(va.size()!=vb.size()){
    printf("ValueArray::operator/() different length\n");
    exit(1);
  }
#endif
  ValueArray<double> res;
  res.resize(va.size());
  for(long i=0;i<va.size();i++){
    res[i]=va.at(i)/vb.at(i);
  }
  return res;
}

template <class T>
//scalar product
T operator*(const ValueArray<T> &va,const ValueArray<T> &vb)
{
#ifdef MY_DEBUGGING
  if(va.size()!=vb.size()){
    printf("ValueArray::operator*() different length\n");
    exit(1);
  }
#endif

  T res=0;
  for(long i=0;i<va.size();i++){
    res+=va.at(i)*vb.at(i);
  }
  return res;
}
template <class T,class T1>
//scalar product
T operator*(const ValueArray<T> &va,const ValueArray<T1> &vb)
{
#ifdef MY_DEBUGGING
  if(va.size()!=vb.size()){
    printf("ValueArray::operator*() different length\n");
    exit(1);
  }
#endif

  T res=0;
  for(long i=0;i<va.size();i++){
    res+=va.at(i)*vb.at(i);
  }
  return res;
}

//element wise
ValueArray<double> abs(const ValueArray<double> &v);
//element wise
ValueArray<double> max(const ValueArray<double> &va,const ValueArray<double> &vb);
//element wise
ValueArray<double> pow(const ValueArray<double> &va,double exp);
//element wise
ValueArray<double> sqrt(const ValueArray<double> &va);



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _VALUEARRAY_H */

