/*************************************************************************
 * @(#)SearchWidget.cpp
 * author: magru@bono
 * date:   31 Mar 2014
 *************************************************************************
 */

#include <QDebug>
#include <QEvent>
#include <QTextCursor>
#include <QTextEdit>
#include <QKeyEvent>
#include "SearchWidget.hh"
#include "FString.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SearchWidget::SearchWidget(QWidget *parent)
:QWidget(parent),pinsearch(false),
 psearchcursor(new QTextCursor),
 pshortcut(Qt::Key_F),pshortcutmod(Qt::ControlModifier),ptextwidget(0),
  pmute(false)
{
  
  setupUi(this);
  palertpalette=pnormalpalette=cmbSearch->palette();
  palertpalette.setColor(QPalette::Base,QColor(255,00, 210));
  hide();
  
  //qDebug()<<"Constructor"<<pmute;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SearchWidget::~SearchWidget()
{
  if(psearchcursor) delete psearchcursor;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::setTextWidget(QWidget *tw)
{
  if(tw){
    ptextwidget=tw;
    tw->installEventFilter(this);
    cmbSearch->installEventFilter(this);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::startSearch()
{
  show();
  
  //qDebug()<<"startSearch"<<pmute;
  
  pinsearch=true;
  poldsearchstring=psearchstring;
  psearchstring="";
  *psearchcursor=textCursor();
  //qDebug()<<psearchcursor->position()<<psearchcursor->anchor();
  cmbSearch->clearEditText();
  cmbSearch->setFocus();
  phadsearchhit=false;
  pnewstring=false;
  pwraparound=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::stopSearching()
{
  pinsearch=false;
  onSearchSucces(true);
  registerSearchString(psearchstring);
  poldsearchstring=psearchstring;
  hide();
  ptextwidget->setFocus();
  //qDebug()<<"poldsearchstring"<<poldsearchstring;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::onSearchSucces(bool s)
{
  phadsearchhit=s;
  if(s){
    cmbSearch->setPalette(pnormalpalette);
  }else{
    cmbSearch->setPalette(palertpalette);
  }
  
  if(!s){
    QTextCursor cur(textCursor());
    cur.clearSelection();
    setTextCursor(cur);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::on_cmbSearch_editTextChanged()
{
  if(pmute) return;
  pwraparound=0;
  pnewstring=true;
  psearchstring=cmbSearch->currentText();
  
  //qDebug()<<"phadsearchhit"<<phadsearchhit;
  if(phadsearchhit) setTextCursor(*psearchcursor);
  
  QTextCursor cur=textCursor();
  bool found=find(psearchstring);
  if(found) {
    if(textCursor().anchor()==cur.anchor()) 
      *psearchcursor=cur;
    else{
      cur.setPosition(textCursor().anchor());
      *psearchcursor=cur;
    }


  }else{

    
    pwraparound=1;
    *psearchcursor=textCursor();
  }

  
  onSearchSucces(found);

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QTextCursor SearchWidget::textCursor()const
{
  QTextEdit *te;
  if( (te=dynamic_cast<QTextEdit*>(ptextwidget)) ){
    //qDebug()<<"textCursor"<<te->textCursor().position();
    return te->textCursor();
  }else{
    qDebug()<<"SearchWidget::textCursor(): cannot find type of "
      "SearchWidget's parent class";
    exit(0);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::setTextCursor(QTextCursor &cur)
{
  QTextEdit *te;
  if( (te=dynamic_cast<QTextEdit*>(ptextwidget)) ){
    te->setTextCursor(cur);
    //qDebug()<<"setTextCursor"<<cur.position();
  }else{
    qDebug()<<"SearchWidget::setTextCursor(): cannot find type of "
      "SearchWidget's parent class";
    exit(0);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SearchWidget::find(const QString &content,bool back)
{
  QTextEdit *te;
  if( (te=dynamic_cast<QTextEdit*>(ptextwidget)) ){
    if(back){
      return te->find(content,QTextDocument::FindBackward);
    }else{
      return te->find(content);
    }
  }else{
    qDebug()<<"SearchWidget::setTextCursor(): cannot find type of "
      "SearchWidget's parent class";
    exit(0);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::onSearchNext(bool back)
{
  if(!pnewstring){
    //qDebug()<<"here"<<poldsearchstring;
    psearchstring=poldsearchstring;
    pmute=true;
    cmbSearch->setEditText(psearchstring);
    pmute=false;
    pnewstring=true;
  }
  if(pwraparound==1 && psearchstring.length()>0){
    QTextCursor cur=textCursor();
    if(back){
      cur.movePosition(QTextCursor::End);
    }else{
      cur.setPosition(0);
    }
    setTextCursor(cur);
    pwraparound=2;
  }
  bool found=find(psearchstring,back);
  if(found) {
    QTextCursor cur=textCursor();
    cur.setPosition(textCursor().anchor());
    *psearchcursor=cur;


  }else{
    if(pwraparound==2){
      QTextCursor cur=textCursor();
      cur.setPosition(psearchcursor->anchor());
      setTextCursor(cur);
      pwraparound=1;
    }else{
      pwraparound=1;
      *psearchcursor=textCursor();
    }
  }
  onSearchSucces(found);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SearchWidget::registerSearchString(const QString &s)
{
  if(s.isEmpty()) return;
  for(long i=-1;++i<cmbSearch->count();){
    if(cmbSearch->itemText(i)==s){
      return;
    }
  }
  cmbSearch->insertItem(0,s);
	
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool SearchWidget::eventFilter(QObject *obj, QEvent *event)
{
  //qDebug()<<"pinsearch"<<pinsearch;
  if(pinsearch){
    //qDebug()<<event<<event->type();
    if(event->type()==QEvent::KeyPress){
      QKeyEvent *ke=static_cast<QKeyEvent*>(event);
      //qDebug()<<Sprintf("inside k=%c s=%c",ke->key(),pshortcut)()
      //<<ke->modifiers()<<pshortcutmod;
      if((ke->key()==pshortcut && ((ke->modifiers() & pshortcutmod))) || 
	 ke->key()==Qt::Key_Enter || ke->key()==Qt::Key_Return
	 ){
	cmbSearch->setFocus();
	
	// definitely take this string
	registerSearchString(cmbSearch->currentText());

	onSearchNext();	
	ke->accept();
	return true;
      }else if(ke->key()==Qt::Key_Escape){
	stopSearching();
	return true;
      }
      
      
    }
  }else{
    //qDebug()<<event<<event->type();
    if(event->type()==QEvent::KeyPress){
      QKeyEvent *ke=static_cast<QKeyEvent*>(event);
      //qDebug()<<event<<event->type();
      //qDebug()<<Sprintf("start k=%c s=%c",ke->key(),pshortcut)()<<ke->modifiers()<<pshortcutmod;
      if(ke->key()==pshortcut && ((ke->modifiers() & pshortcutmod))){
	//qDebug()<<"here";
	startSearch();
      }
    }
  }
  return QObject::eventFilter(obj, event);
}


/*EOF*/ 
