/*************************************************************************
 * @(#)ToolButtons.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   29 Mar 2011
 *************************************************************************
 */

#include <QToolButton>
#include "ToolButtons.hh"
#include "ShortCuts.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ToolButtons::ToolButtons()
  :pinsertAction(0),pdeleteAction(0),pmoveUpAction(0),
   pmoveDownAction(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ToolButtons::~ToolButtons(){
  if(pinsertAction) delete pinsertAction;
  if(pdeleteAction) delete pdeleteAction;
  if(pmoveUpAction) delete pmoveUpAction;
  if(pmoveDownAction) delete pmoveDownAction;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ToolButtons::connectSlots(QWidget *pparent,
			       const char *insertSlot,const char*deleteSlot,
			       const char*upSlot,const char*downSlot)
{
  if(pinsertAction){
    QObject::connect(pinsertAction,SIGNAL(triggered())
		     ,pparent,insertSlot);
  }
  if(pdeleteAction){
    QObject::connect(pdeleteAction,SIGNAL(triggered())
		     ,pparent,deleteSlot);
  }
  if(pmoveUpAction){
    QObject::connect(pmoveUpAction,SIGNAL(triggered())
	    ,pparent,upSlot);
  }
  if(pmoveDownAction){
    QObject::connect(pmoveDownAction,SIGNAL(triggered())
	    ,pparent,downSlot);
  }


  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void ToolButtons::createToolButtonsActions(QWidget *ptable
 			,const QString &what
			,QToolButton *tbInsert
			,QToolButton *tbDelete
			,QToolButton *tbUp
			,QToolButton *tbDown
			)
{
  
  if(tbInsert){
    pinsertAction=new QAction(ptable);
    ShortCuts::setShortCut(ShortCuts::addItemSC,pinsertAction);
    pinsertAction->setShortcutContext(Qt::WidgetShortcut);
    pinsertAction->setToolTip("Insert a "+what+" (Ctrl+I,Ins)");
    ptable->addAction(pinsertAction);

    pinsertAction->setIcon(QIcon(":/Icons/add"));
    
    tbInsert->setDefaultAction(pinsertAction);

  }

  if(tbDelete){
    pdeleteAction=new QAction(ptable);
    ShortCuts::setShortCut(ShortCuts::deleteItemSC,pdeleteAction);
    pdeleteAction->setShortcutContext(Qt::WidgetShortcut);
    pdeleteAction->setToolTip("Delete a "+what+" (Ctrl+D,Del)");
    ptable->addAction(pdeleteAction);
    
    pdeleteAction->setIcon(QIcon(":/Icons/delete"));
    
    tbDelete->setDefaultAction(pdeleteAction);
  }  

  if(tbUp){
    pmoveUpAction=new QAction(ptable);
    ShortCuts::setShortCut(ShortCuts::moveItemUpSC,pmoveUpAction);
    pmoveUpAction->setShortcutContext(Qt::WidgetShortcut);
    pmoveUpAction->setToolTip("Move "+what+" one up"+" (Ctrl+Up)");
    ptable->addAction(pmoveUpAction);
    
    pmoveUpAction->setIcon(QIcon(":/Icons/arrow_up"));

    tbUp->setDefaultAction(pmoveUpAction);
  }
  
  if(tbDown){
    pmoveDownAction=new QAction(ptable);
    ShortCuts::setShortCut(ShortCuts::moveItemDownSC,pmoveDownAction);
    pmoveDownAction->setShortcutContext(Qt::WidgetShortcut);
    pmoveDownAction->setToolTip("Move "+what+" one down"+" (Ctrl+Up)");
    ptable->addAction(pmoveDownAction);
    
    pmoveDownAction->setIcon(QIcon(":/Icons/arrow_down")); 
  

    tbDown->setDefaultAction(pmoveDownAction);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void createTableActions(QWidget *parent
 			,const QString &what
			,QAction **insertAction
			,QAction **deleteAction
			,QAction **moveUpAction
			,QAction **moveDownAction
			,QToolButton *tbInsert
			,QToolButton *tbDelete
			,QToolButton *tbUp
			,QToolButton *tbDown
			)
{
  if(insertAction){
    *insertAction=new QAction(parent);
    ShortCuts::setShortCut(ShortCuts::addItemSC,*insertAction);
    (*insertAction)->setShortcutContext(Qt::WidgetShortcut);
    (*insertAction)->setToolTip("Insert a "+what+" (Ctrl+I,Ins)");
    parent->addAction(*insertAction);

    (*insertAction)->setIcon(QIcon(":/Icons/add"));
  }

  *deleteAction=new QAction(parent);
  ShortCuts::setShortCut(ShortCuts::deleteItemSC,*deleteAction);
  (*deleteAction)->setShortcutContext(Qt::WidgetShortcut);
  (*deleteAction)->setToolTip("Delete a "+what+" (Ctrl+D,Del)");
  parent->addAction(*deleteAction);

  (*deleteAction)->setIcon(QIcon(":/Icons/delete"));

  

  if(moveUpAction){
    *moveUpAction=new QAction(parent);
    ShortCuts::setShortCut(ShortCuts::moveItemUpSC,*moveUpAction);
    (*moveUpAction)->setShortcutContext(Qt::WidgetShortcut);
    (*moveUpAction)->setToolTip("Move "+what+" one up"+" (Ctrl+Up)");
    parent->addAction(*moveUpAction);
    
    (*moveUpAction)->setIcon(QIcon(":/Icons/arrow_up"));
  }
  
  if(moveDownAction){
    *moveDownAction=new QAction(parent);
    ShortCuts::setShortCut(ShortCuts::moveItemDownSC,*moveDownAction);
    (*moveDownAction)->setShortcutContext(Qt::WidgetShortcut);
    (*moveDownAction)->setToolTip("Move "+what+" one down"+" (Ctrl+Up)");
    parent->addAction(*moveDownAction);
    
    (*moveDownAction)->setIcon(QIcon(":/Icons/arrow_down")); 
  }
  if(insertAction){
    tbInsert->setDefaultAction(*insertAction);
  }
  tbDelete->setDefaultAction(*deleteAction);
  if(moveUpAction)
    tbUp->setDefaultAction(*moveUpAction);
  if(moveDownAction)
    tbDown->setDefaultAction(*moveDownAction);

}



/*EOF*/ 
