#include <stdlib.h>
#include <stdio.h>
#include <QValidator>
#include <QWidget>
#include <QRegExp>
#include <QLineEdit>
#ifdef QXFSF
#include "PropertyTable.hh"
#endif
#include "Valid.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"

#define INVALID    0
#define VALID      1
#define ACCEPTABLE 2

using namespace XFPLO;

#ifdef QXFSF

/* ***********************************************************************
 * element validator 
 * ***********************************************************************
 */
ElementValidator::ElementValidator( QObject * parent )
  : QValidator( parent ),p_widget(0),p_valuename("")
{
  NSF("ElementValidator::ElementValidator");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ElementValidator::ElementValidator( QObject * parent,QWidget *w 
				    ,const QString &valuename)
  :  QValidator( parent ),p_widget(0)
{
  //NSF("ElementValidator");
  p_valuename=valuename;
  p_valuename=p_valuename.replace( QRegExp("&"), "" );

  if(w){
    p_widget=w;
    QString cn=w->metaObject()->className();
    if(cn=="QLineEdit"){
      ((QLineEdit*)w)->setValidator(this);
    }else{
      p_widget=0;
      printf("Unimplemented widget class (%s) for ElementValidator!\n"
	     ,cn.toAscii().constData());
      fprintf(stderr,"Unimplemented widget class (%s) for ElementValidator!\n"
	      ,cn.toAscii().constData());
      exit(1);
    }
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QValidator::State ElementValidator::validateIt( QString & input) const
{
  NSF("ElementValidator::validate");
  //  myDebug() << nsubfunc;
  QValidator::State ret;
  
  QString s=input; IntValidator::stripAllWhiteSpace(s);
  
  s=s.toLower();
  if(s.length()>=1) s[0]=s[0].toUpper();
  //  myDebug() <<"s" <<s;

  input=s;

  if(AtomPropertyTable::allowedElements.contains(s)) {
    ret=QValidator::Acceptable;
    goto ende;
  }

  if(s.length()>0 && !s.constData()->isLetter()) {
    ret=QValidator::Invalid;
    goto ende;
  }

  if(s.length()<2) {
    ret=QValidator::Intermediate;
    goto ende;
  }

 ende:
  //  myDebug() << "ret" << ret;

  return ret;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long ElementValidator::check(QString &txt)
{
  //  NSF("check");
  QString s="";
  if(!p_widget)return 0;
  //CPRP(p_widget);
  QString cn=p_widget->metaObject()->className();

  if(cn=="QLineEdit"){
    s=((QLineEdit*)p_widget)->text();
  }else{
    return 0;
  }
  
  if( validateIt(s)!=   QValidator::Acceptable){
    txt+=QString("\n%1 must be en element!\n").arg(p_valuename);
  }
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long ElementValidator::check(QString &txt, QString &value)
{
  //  NSF("check");
  
  if( validateIt(value)!=   QValidator::Acceptable){
    txt+=QString("\n%1 '%2' must be en element!\n").arg(p_valuename)
      .arg(value);
    return 0;
  }
  return 1;
}

#endif









/* ***********************************************************************
 *  double  validator 
 * ***********************************************************************
 */
RealValidator::RealValidator( QObject * parent )
  : IntValidator( parent )
{
  //  printf("RealValidator::RealValidator\n");
  setRange(1.,-1.);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
RealValidator::RealValidator( QObject * parent,QWidget *w 
				,const QString &valuename
				,double b,double t)
  :IntValidator( parent,w,valuename,0,0)
{

  setRange(b,t);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

RealValidator::~RealValidator()
{
  //  printf("RealValidator::~RealValidator\n");
  // nothing
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString RealValidator::warning()
{
  QString s;
  if(p_bottom<p_top){
    s=QString("a floating point number in [%2,%3]")
      .arg(p_bottom).arg(p_top);
  }else{
    s=QString("a floating point number");
  }
  return s;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RealValidator::setRange( double bottom, double top )
{
  //printf("RealValidator::setRange %g %g\n",p_bottom,p_top);
  p_bottom = bottom;
  p_top = top;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QValidator::State RealValidator::validateIt(const QString & input) const
{
  int i;
  QValidator::State ret;
  QString s;
  
  //  printf("RealValidator::validate b:%f t:%f\n",b,t);
  if(p_bottom<p_top && p_bottom>=0.)
    i=isReal(input);
  else
    i=isSReal(input);
  switch ( i ){
  case INVALID: {ret = QValidator::Invalid;break;}
  case VALID: {ret = QValidator::Intermediate;break;}
  case ACCEPTABLE: 
    {
      s=input; stripAllWhiteSpace(s);
      bool ok;
      double n=s.toDouble(&ok);
      if(ok){
        if(p_bottom<p_top){
          if( n<p_bottom || n> p_top){
            ret = QValidator::Intermediate;break;
          }else{
            ret = QValidator::Acceptable;break;
          }
        }else{
          ret = QValidator::Acceptable;break;
        }
      }else{
        ret = QValidator::Invalid;break;
      }
    }
  default: ret=QValidator::Invalid;
  }
    
  return ret;  

}

/* ***********************************************************************
 * test space unsigned real double space
 * ***********************************************************************
 */

long  RealValidator::isReal(const QString & s) const
{
  long ret;
    QRegExp rega1( QString::fromLatin1("^ *[+]? *[0-9]+ *\\.? *$") );
    QRegExp rega1E( QString::fromLatin1("^ *[+]? *[0-9]+ *\\.? *[eE] *[+-]? *[0-9]+ *$") );
    QRegExp rega2( QString::fromLatin1("^ *[+]? *[0-9]* *\\.? *[0-9]+ *$") );
    QRegExp rega2E( QString::fromLatin1("^ *[+]? *[0-9]* *\\.? *[0-9]+ *[eE] *[+-]? *[0-9]+ *$") );
  //  QRegExp empty( QString::fromLatin1("^ *[+]? *\\.?$") );
  QRegExp val1 ( QString::fromLatin1("^.*$") );
  if ( rega1.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else if ( rega1E.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( rega2.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( rega2E.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
    //  }else   if ( empty.indexIn( s ) >= 0 ){
    //    ret= VALID;
  }else   if ( val1.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    ret=INVALID;
  }

  return ret;
}

/* ***********************************************************************
 * test space signed real double space
 * ***********************************************************************
 */

long  RealValidator::isSReal(const QString & s) const
{
  long ret;
  QRegExp rega1( QString::fromLatin1("^ *[+-]? *[0-9]+ *\\.? *$") );
  QRegExp rega1E( QString::fromLatin1("^ *[+-]? *[0-9]+ *\\.? *[eE] *[+-]? *[0-9]+ *$") );
  QRegExp rega2( QString::fromLatin1("^ *[+-]? *[0-9]* *\\.? *[0-9]+ *$") );
  QRegExp rega2E( QString::fromLatin1("^ *[+-]? *[0-9]* *\\.? *[0-9]+ *[eE] *[+-]? *[0-9]+ *$") );
  //  QRegExp empty( QString::fromLatin1("^ *[+-]? *\\.?$") );
  QRegExp val1 ( QString::fromLatin1("^.*$") );
  if ( rega1.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else if ( rega1E.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( rega2.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( rega2E.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
    //  }else   if ( empty.indexIn( s ) >= 0 ){
    //    ret= VALID;
  }else   if ( val1.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    ret=INVALID;
  }

  return ret;
}


















/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IntValidator::stripAllWhiteSpace(QString &s)
{
  for(int j=(int)s.length()-1;j>-1;j--){
    if(s[j] == ' ') s.remove(j,1);
  }
}
/* ***********************************************************************
 * integer validator 
 * ***********************************************************************
 */
IntValidator::IntValidator( QObject * parent )
  : QValidator( parent ),p_widget(0),p_valuename("")
{
  //  printf("IntValidator::IntValidator\n");
  setRange(1,-1);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
IntValidator::IntValidator( QObject * parent,QWidget *w 
			    ,const QString &valuename
			    ,long b,long t)
  :  QValidator( parent ),p_widget(0)
{
  //NSF("IntValidator");
  p_valuename=valuename;
  p_valuename=p_valuename.replace( QRegExp("&"), "" );

  if(w){
    p_widget=w;
    QString cn=w->metaObject()->className();
    if(cn=="QLineEdit"){
      ((QLineEdit*)w)->setValidator(this);
    }else{
      p_widget=0;
      printf("Unimplemented widget class (%s) for IntValidator!\n"
	     ,cn.toAscii().constData());
      fprintf(stderr,"Unimplemented widget class (%s) for IntValidator!\n"
	      ,cn.toAscii().constData());
      exit(1);
    }
  }
  setRange(b,t);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
IntValidator::~IntValidator()
{
  //  printf("IntValidator::~IntValidator\n");
    // nothing
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString IntValidator::warning()
{
  QString s;
  if(p_bottom<p_top){
     s=QString("an integer number in [%2,%3]").arg(p_bottom).arg(p_top);
  }else{
    s=QString("an integer number");
  }
  return s;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IntValidator::check(QString &txt)
{
  //  NSF("check");
  QString s="";
  if(!p_widget)return 0;
  //CPRP(p_widget);
  QString cn=p_widget->metaObject()->className();

  if(cn=="QLineEdit"){
    s=((QLineEdit*)p_widget)->text();
  }else{
    return 0;
  }
  
  if( validateIt(s )!=   QValidator::Acceptable){
    txt+=QString("\n%1 must be %2!\n").arg(p_valuename).arg(warning());
  }
  return 1;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long IntValidator::check(QString &txt,const QString &value)
{
  //  NSF("check");

  if( validateIt(value )!=   QValidator::Acceptable){
    txt+=QString("\n%1 '%2' must be %3!\n").arg(p_valuename)
      .arg(value).arg(warning());
    return 0;
  }
  return 1;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QValidator::State IntValidator::validateIt( const QString & input ) const
{
  int i;
  QValidator::State ret;
  QString s;
  if(p_bottom<p_top && p_bottom>-1)
    i=isInt(input);
  else
    i=isSInt(input);
  switch ( i ){
  case INVALID: {ret = QValidator::Invalid;break;}
  case VALID: {ret = QValidator::Intermediate;break;}
  case ACCEPTABLE: 
    {
      s=input; stripAllWhiteSpace(s);
      bool ok;
      long int n=s.toLong(&ok);
      if(ok){
        if(p_bottom<p_top){
          if( n<p_bottom || n> p_top){
            ret = QValidator::Intermediate;break;
          }else{
            ret = QValidator::Acceptable;break;
          }
        }else{
          ret = QValidator::Acceptable;break;
        }
      }else{
        ret = QValidator::Invalid;break;
      }
    }
  default: ret=QValidator::Invalid;
  }
    
  return ret;  

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IntValidator::setRange( long bottom, long top )
{
  //printf("IntValidator::setRange\n");
  p_bottom = bottom;
  p_top = top;
}

/* ***********************************************************************
 * test space unsigned integer space
 * ***********************************************************************
 */
long  IntValidator::isInt(const QString & s) const
{
  long ret;
  QRegExp reg( QString::fromLatin1("^ *[+]? *[0-9]+ *$") );
  QRegExp empty( QString::fromLatin1("^ *[+]? *$") );
  if ( reg.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( empty.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    ret=INVALID;
  }

  return ret;
}
/* ***********************************************************************
 * test space signed integer space
 * ***********************************************************************
 */

long  IntValidator::isSInt(const QString & s) const
{
  long ret;
  QRegExp reg( QString::fromLatin1("^ *[-+]? *[0-9]+ *$") );
  QRegExp empty( QString::fromLatin1("^ *[-+]? *$") );
  if ( reg.indexIn( s ) >= 0 ){
    ret= ACCEPTABLE;
  }else   if ( empty.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    ret=INVALID;
  }

  return ret;
}

























/* ***********************************************************************
 * rational numbers
 * ***********************************************************************
 */

RatValidator::RatValidator( QObject * parent )
  : RealValidator( parent )
{
  //    printf("RatValidator::RatValidator %f %f \n",bottom(),top());
    setRange(1.,-1.);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
RatValidator::RatValidator( QObject * parent,QWidget *w 
			    ,const QString &valuename
			    ,double b,double t)
  :RealValidator( parent,w,valuename,b,t )
{
  //setRange(b,t);
}/* ***********************************************************************
 *
 * ***********************************************************************
 */
RatValidator::~RatValidator()
{
  //  printf("RatValidator::~RatValidator \n");
    // nothing
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString RatValidator::warning()
{
  QString s;
  if(p_bottom<p_top){
    s=QString("a fractional in [%2,%3]").arg(p_bottom).arg(p_top);
  }else{
    s=QString("a fractional");
  }
  return s;
}
/* ***********************************************************************
 * test space unsigned ratio (int/int)  space
 * ***********************************************************************
 */

long  RatValidator::isRatio(const QString & s) const
{
  long ret;
  QRegExp rega1( QString::fromLatin1("^ *[0-9]+ */ *[0-9]+ *$") );
  QRegExp regv1( QString::fromLatin1("^ *[0-9]* */? *[0-9]* *$") );
  QRegExp val1 ( QString::fromLatin1("^.*$") );

  if ( rega1.indexIn( s ) >= 0 ){
    //    printf("isRAT ACCEPT :%d:%s\n",i,s.latin1());
    ret= ACCEPTABLE;
  }else   if ( regv1.indexIn( s ) >= 0 ){
    //    printf("isRAT VALID  :%d:%s\n",i,s.latin1());
    ret= VALID;
  }else   if ( val1.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    //    printf("isRAT INVALID:%d:%s\n",i,s.latin1());
    ret=INVALID;
  }

  return ret;
}
/* ***********************************************************************
 * test space signed ratio (int/int)  space
 * ***********************************************************************
 */

long  RatValidator::isSRatio(const QString & s) const
{
  long ret;
  QRegExp rega1( QString::fromLatin1("^ *-? *[0-9]+ */ *[0-9]+ *$") );
  QRegExp regv1( QString::fromLatin1("^ *-? *[0-9]* */? *[0-9]* *$") );
  QRegExp val1 ( QString::fromLatin1("^.*$") );

  //  printf("isSRAT :%s:\n",s.latin1());
  if ( rega1.indexIn( s ) >= 0 ){
    //    printf("isSRAT ACCEPT :%d:%s\n",i,s.latin1());
    ret= ACCEPTABLE;
  }else   if ( regv1.indexIn( s ) >= 0 ){
    //    printf("isSRAT VALID  :%d:%s\n",i,s.latin1());
    ret= VALID;
  }else   if ( val1.indexIn( s ) >= 0 ){
    ret= VALID;
  }else{
    //    printf("isSRAT INVALID:%d:%s\n",i,s.latin1());
    ret=INVALID;
  }
  //  myDebug() << "ret" << ret;
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

QValidator::State RatValidator::validateIt( const QString & input ) const
{
  int i;
  QValidator::State ret;
  QString s;
  //double b=bottom();
  //double t=top();
  //  printf("b:%f t:%f\n",b,t);
  // first real, including ints
  
  ret=RealValidator::validateIt(input);

  // true rationals
  if(ret == QValidator::Acceptable)
    return ret;

  //  printf("RatValidator::validate b:%f t:%f\n",b,t);  

  if(p_bottom<p_top && p_bottom>=0.)
    i=isRatio(input);
  else
    i=isSRatio(input);
  switch ( i ){
  case INVALID: {ret = QValidator::Invalid;break;}
  case VALID: {ret = QValidator::Intermediate;break;}
  case ACCEPTABLE: 
    {
      
      s=input; stripAllWhiteSpace(s);
      i=s.indexOf('/',0,Qt::CaseInsensitive);
      QString t1=s.left(i);
      QString t2=s;
      t2.remove(0,i+1);

      bool ok1,ok2;
      double n1=t1.toDouble(&ok1);
      double n2=t2.toDouble(&ok2);

      double n=n1/n2;
      //      printf("RatValidator::validate s:%s=%f/%f=%f \n",s.latin1(),n1,n2,n);  
      if(ok1 && ok2){
        if(n2 == 0.) {
          ret = QValidator::Intermediate;break;
        }else
          if(p_bottom<p_top){
            if( n<p_bottom || n> p_top){
              ret = QValidator::Intermediate;break;
            }else{
              ret = QValidator::Acceptable;break;
            }
          }else{
            ret = QValidator::Acceptable;break;
          }
      }else{
        ret = QValidator::Invalid;break;
      }
      
    }
  default: ret=QValidator::Invalid;
  }

  return ret;  

}

