/* ***********************************************************************
 *
 * graphic package for FPLO
 * (C) Klaus Koepernik April '98
 *
 * **********************************************************************
 */
#ifndef _VALID_H_
#define _VALID_H_



#include <QValidator> 


class QLineEdit;

#ifdef QXFSF

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class  ElementValidator: public QValidator
{
  //  Q_OBJECT
public:
    ElementValidator( QObject * parent);
    ElementValidator( QObject * parent,QWidget *w
		      ,const QString &valuename );
    ~ElementValidator(){};

    QValidator::State validate( QString &input, int &value ) const{
      return validateIt(input);Q_UNUSED(value);};
    QValidator::State validateIt(QString &) const;

    long check(QString &);
    long check(QString &,QString &value);



private:

protected:
    QWidget *p_widget;
    QString p_valuename;
};

#endif
/**
 * This is an integer validator. It allows to check for integers
 * within a range.
 */

class  IntValidator: public QValidator
{
    Q_OBJECT
public:
    IntValidator( QObject * parent);
    IntValidator( QObject * parent,QWidget *w
		  ,const QString &valuename,long b,long t );
    virtual ~IntValidator();

    QValidator::State validate( QString &input, int &value ) const{
      return validateIt(input);Q_UNUSED(value);};
    virtual QValidator::State validateIt( const QString &) const;

    long check(QString &);
    long check(QString &,const QString &value);

    virtual void setRange( long bottom, long top );
    virtual QString warning();

    long  isInt(const QString & ) const;
    long  isSInt(const QString & ) const;

    long bottom() const { return p_bottom; }
    long top() const { return p_top; }


    static void stripAllWhiteSpace(QString &s);
private:
    long p_bottom, p_top;
protected:
    QWidget *p_widget;
    QString p_valuename;
};

/* ***********************************************************************
 * real validator
 * ***********************************************************************
 */

class  RealValidator: public IntValidator
{
    Q_OBJECT
public:
    RealValidator( QObject * parent);
    RealValidator( QObject * parent,QWidget *w
		    ,const QString &valuename,double b,double t );
    virtual ~RealValidator();

    QValidator::State validate( QString &input, int &value ) const{
      return validateIt(input);Q_UNUSED(value);};
    virtual QValidator::State validateIt(const  QString &) const;

    virtual QString warning();

    virtual void setRange( long bottom, long top ){
      p_bottom=(double)bottom;
      p_top=(double)top;
    };
    virtual void setRange( double bottom, double top );

    long  isReal(const QString & ) const; 
    long  isSReal(const QString & ) const; 

    double bottom() const { return p_bottom; }
    double top() const { return p_top; }
protected:
    double p_bottom, p_top;
};

/* ***********************************************************************
 * ratio validator
 * ***********************************************************************
 */

class  RatValidator: public RealValidator
{
    Q_OBJECT
public:
    RatValidator( QObject * parent );
    RatValidator( QObject * parent,QWidget *w
		  ,const QString &valuename,double b,double t );
    ~RatValidator();

    QValidator::State validate( QString &input, int &value ) const{
      return validateIt(input);Q_UNUSED(value);};
    virtual QValidator::State validateIt(const  QString &) const;

    virtual QString warning();
    
    long  isRatio(const QString & ) const;
    long  isSRatio(const QString & ) const;

};


#endif /* _VALID_H_ */
/* EOF */
