/* @(#)ParamDriver.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PARAMDRIVER_H
#define _PARAMDRIVER_H 1

#include <string>
#include "ParamParser.tab.hh"
#include "ParserDecl.hh"
#include "Xfp.hh"
#include "SetDescriptor.hh"
#include "TempSetDescriptor.hh"

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

class Page;
class Graph;
class FGObject;
class Function;
class FontStyle;
class FunctionAttribs;
class FunctionGroup;
class GraphView;
class WeightSelections;
class WeightSelection;
class LineShape;
class EllipseShape;
class TextBox;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ParamDriver{
 public:
  ParamDriver();
  virtual ~ParamDriver();

  enum TieIns{TIE_XDATA_Begin=0,TIE_SIDEID=1,TIE_XDATA=2,TIE_YDATA=3,
	      TIE_SYMBOLSTYLE=4,TIE_LINESTYLE=5,TIE_ARROWPOSTYPE=6,
	      TIE_COORDINATESYSTEM=7,
	      TIE_RESTRICTIONTYPE=8,
	      TIE_GRAPHAXISSCALING=9,
	      TIE_DATATYPE=10 // this serves as last marker,do add before this
	      //                and rename
	      };

  // Handling the scanner.
  void scan_begin ();
  void scan_end ();

  bool eof(){return peof;}
  void setEof(){peof=true;}

  std::string &file(){return pfilename;};

  bool traceScanning()const{return ptracescanning;};
  void setTraceScanning(bool t){ptracescanning=t;};

  bool traceParsing()const{return ptraceparsing;};
  void setTraceParsing(bool t){ptraceparsing=t;};

  SetDescriptor *currentSetDescriptor(){return &psetd;};
  void setCurrentWeight(const String *w);

  void resizeGraphView();
  void setPage(Page *p,GraphView *g){ppage=p;pgv=g;};
  // Run the parser.  Return 0 on success.
  int parse(const char *bufferorfilename,bool frombuffer=false);
  int parse(const String &bufferorfilename,bool frombuffer=false){
    return parse(bufferorfilename(),frombuffer);
  };

  bool tieIn(TieIns t)const{return ptieins.at(t);};
  void setTieIn(TieIns t,bool e){ptieins[t]=e;};

  // Error handling.
  void error (const PP::location& l, const std::string& m)const;
  void error (const std::string& m)const;

  void setLocation(const PP::location& l){ploc=l;};

  
  Page *currentPage(){return ppage;};


  void checkCurrentWeight(const SetDescriptor *,WeightSelections *);
  void checkTextBox(long tbt,bool allownew);

  void readFile(const String *file,Xfp::FileType ftype,
		const SetDescriptor *sd=0);
  void convolute(double d);
  void newGraph();
  void switchOnOff(bool onoff);
  void toggle();


  inline void setCurrentFontStyle(FontStyle *fs){pcurrentfontstyle=fs;};
  inline FontStyle *currentFontStyle(){return pcurrentfontstyle;};
  inline void setCurrentLineStyle(LineStyle *ls,const char *from=""){
    pcurrentlinestyle=ls;
    pfrom=from;};
  inline LineStyle *currentLineStyle(){return pcurrentlinestyle;};
  inline void setCurrentFillStyle(FillStyle *ls,const char *from=""){
    pcurrentfillstyle=ls;pfrom=from;};
  inline FillStyle *currentFillStyle(){return pcurrentfillstyle;};
  inline bool comesFrom(const char *from)const{return pfrom==from;};
  inline bool context(const char *context)const{return pcontext==context;};
  inline void setContext(const char *context){pcontext=context;};
  inline void setCurrentSymbolStyle(SymbolStyle *ls){pcurrentsymbolstyle=ls;};
  inline SymbolStyle *currentSymbolStyle(){return pcurrentsymbolstyle;};

  inline void setCurrentLineShape(LineShape *lsh){pcurrentlineshape=lsh;};
  inline LineShape *currentLineShape(){return pcurrentlineshape;};

  inline void setCurrentEllipseShape(EllipseShape *esh){
    pcurrentellipseshape=esh;};
  inline EllipseShape *currentEllipseShape(){return pcurrentellipseshape;};

  inline void setCurrentTextBox(TextBox *tb){
    pcurrenttextbox=tb;};
  inline TextBox *currentTextBox(){return pcurrenttextbox;};




  Tics *getUserTics();



  Function *currentFunc(){return pcurrentfunc;};


    

  bool needRefresh()const{return pneedrefresh;};

  void resetPage();

  


  void setVariable(const String *,const TempSetDescriptor &tempsd);
  void copyFromSetDescriptor(TempSetDescriptor *tempsd);
  TempSetDescriptor *cloneVariable(const String *v);
  const TempSetDescriptor *getVariable(const String *v)const;
  double getVariableValue(const String *v);
  void setFromTemp(const TempSetDescriptor *tempsd);

  //void appendWeight(const SetDescriptor *sd,const std::string *name);

  

  Graph *getGraph(const SetDescriptor *sd=0,bool append=false);
  FunctionGroup *getGroup(const SetDescriptor *sd=0,bool append=false);
  Function *getSet(const SetDescriptor *sd=0,bool append=false,
		   bool complain=true);
  FGObject *getFGObject();
  FunctionAttribs *getFuncAttribs();
  WeightSelections *getWeightSelections(const SetDescriptor *sd=0); 
  WeightSelection *getWeightSelection(const SetDescriptor *sd=0
				      ,bool append=false); 

  LineShape *getLineShape(long index,bool append=false);
  EllipseShape *getEllipseShape(long index,bool append=false);

  void copySet(bool move=false);


  void pushCurrentSetDescriptor(){
    psdstack.append(psetd);
  }
  void popCurrentSetDescriptor(){
    psetd=psdstack.at(psdstack.size()-1);
    psdstack.resize(psdstack.size()-1);
  }

 public:
  int result;
 private:
  GraphView *pgv;
  Page *ppage;
  
  bool ptracescanning;
  bool ptraceparsing;
  ValueArray<Pair<String, TempSetDescriptor> > pvariables;
  std::string pfilename;
  void *pscanner;
  PP::location ploc;
  SetDescriptor psetd;
  friend class PP::ParamParser;
  FontStyle *pcurrentfontstyle;
  LineStyle *pcurrentlinestyle;
  FillStyle *pcurrentfillstyle;
  String pfrom;
  String pcontext;
  SymbolStyle *pcurrentsymbolstyle;
  LineShape *pcurrentlineshape;
  EllipseShape *pcurrentellipseshape;
  TextBox *pcurrenttextbox;
  Function *pcurrentfunc;
  ValueArray<SetDescriptor> psdstack;
  ValueArray<bool> ptieins;
  bool pneedrefresh;
  long pfuncindex;
  bool pkilled;
  // for scanning strings
  YY_BUFFER_STATE lexstate;
  bool pfrombuffer;
  const char *pbuffer;
  bool peof;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PARAMDRIVER_H */

