/*************************************************************************
 * @(#)TempSetDescriptor.cpp
 * author: magru@unas
 * date:   08 Jan 2014
 *************************************************************************
 */

#include "TempSetDescriptor.hh"
#include "ParserFunctions.hh"
#include "FString.hh"
#include "MyMath.hh"
#include "Except.hh"

#define FUNC_ERRORS				\
  if((fdef->name==ParserFunctions::Sqrt) && a<0){	\
    return "sqrt with negative argument";	\
  }									\
  if((fdef->name==ParserFunctions::Asin || fdef->name==ParserFunctions::Acos) &&	\
     (a<-1 || a>1) ){							\
    return "asin/acos with invalid argument |x|>1";			\
  }									\
  if((fdef->name==ParserFunctions::Log) && (a<=0) ){				\
    return "log with non-positive argument";				\
  }									\
  if((fdef->name==ParserFunctions::Log10) && (a<=0) ){				\
    return Sprintf("log10 with non-positive argument: %g",a);		\
  }									\
  if((fdef->name==ParserFunctions::Cot) && (a==0) ){				\
    return "cot with argument zero";					\
  }

#define FUNC_ERRORS2							\
  if((fdef->name==ParserFunctions::Div) && (b==0) ){			\
    return "division by zero";						\
  }									\
  if(fdef->name==ParserFunctions::Pow){					\
    if(a<0 && (((int)b)!=b)){						\
    return "negative number raised to a non integer power";		\
    }									\
  }
/* ***********************************************************************
 *
 * ***********************************************************************
 */
String TempSetDescriptor::func(const struct ParserFunctions::tsfuncdef *fdef)
{
  if(ptype==Value){
    double a=pvalue;
    FUNC_ERRORS;
    pvalue=(fdef->fp1)(a);
  }else if(ptype==Vector){
    for(int i=0;i<pxdata.size();i++){
      double a=pxdata.at(i);
      FUNC_ERRORS;
      pxdata[i]=(fdef->fp1)(a);
    }
  }else if(ptype==VectorSet){
    for(int j=0;j<pgrdata.size();j++){
      ValueArray<double> &dd=pgrdata[j];
      for(int i=0;i<dd.size();i++){
	double a=dd.at(i);
	FUNC_ERRORS;
	dd[i]=(fdef->fp1)(a);
      }
    }
  }else{
    return Sprintf("TempSetDescriptor::func: type not yet implemented!");
  }
  
  return "";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
String TempSetDescriptor::func(const struct ParserFunctions::tsfuncdef *fdef,
			       const TempSetDescriptor &other)
{
  if(ptype==Value){
    if(other.ptype==Value){
      double a=pvalue;
      double b=other.pvalue;
      FUNC_ERRORS2;
      pvalue=(fdef->fp2)(a,b);
    }else if(other.ptype==Vector){
      ptype=Vector;
      pxdata.resize(other.pxdata.size(),true);
      for(int i=0;i<pxdata.size();i++){
	double a=pvalue;
	double b=other.pxdata.at(i);
	FUNC_ERRORS2;
	pxdata[i]=(fdef->fp2)(a,b);
      }
    }else if(other.ptype==VectorSet){
      ptype=VectorSet;
      long N=other.pgrdata.size();
      pgrdata.resize(N,true);
      for(long j=-1;++j<N;){
	ValueArray<double> &dd=pgrdata[j];
	const ValueArray<double> &ddo=other.pgrdata.at(j);
	dd.resize(ddo.size(),true);
	for(int i=0;i<ddo.size();i++){
	  double a=pvalue;
	  double b=ddo.at(i);
	  FUNC_ERRORS2;
	  dd[i]=(fdef->fp2)(a,b);
	}
      }
    }else{
      return "TempSetDescriptor::func: type not yet implemented!";
    }
  }else if(ptype==Vector){
    if(other.ptype==Value){
      for(int i=0;i<pxdata.size();i++){
	double a=pxdata.at(i);
	double b=other.pvalue;
	FUNC_ERRORS2;
	pxdata[i]=(fdef->fp2)(a,b);
      }
    }else if(other.ptype==Vector){
      if(pxdata.size()!=other.pxdata.size()){
	return Sprintf("cannot apply function %s to vectors of different length",
		       fdef->guiName);
      }
      for(int i=0;i<pxdata.size();i++){
	double a=pxdata.at(i);
	double b=other.pxdata.at(i);
	FUNC_ERRORS2;
	pxdata[i]=(fdef->fp2)(a,b);
      }
    }else if(other.ptype==VectorSet){
      ptype=VectorSet;
      long N=other.pgrdata.size();
      pgrdata.resize(N,true);
      for(long j=-1;++j<N;){
	ValueArray<double> &dd=pgrdata[j];
	const ValueArray<double> &ddo=other.pgrdata.at(j);
	dd.resize(ddo.size(),true);
	
	if(pxdata.size()!=ddo.size()){
	  return Sprintf("cannot apply function %s to vectors of different length",
			 fdef->guiName);
	}
	for(int i=0;i<pxdata.size();i++){
	  double a=pxdata.at(i);
	  double b=ddo.at(i);
	  FUNC_ERRORS2;
	  dd[i]=(fdef->fp2)(a,b);
	}
      }

    }else{
      return "TempSetDescriptor::func: type not yet implemented!";
    }
  }else if(ptype==VectorSet){
    if(other.ptype==Value){
      long N=pgrdata.size();
      for(long j=-1;++j<N;){
	ValueArray<double> &dd=pgrdata[j];
	for(int i=0;i<dd.size();i++){
	  double a=dd.at(i);
	  double b=other.pvalue;
	  FUNC_ERRORS2;
	  dd[i]=(fdef->fp2)(a,b);
	}
      }
    }else if(other.ptype==Vector){
      long N=pgrdata.size();
      for(long j=-1;++j<N;){
	ValueArray<double> &dd=pgrdata[j];
	if(other.pxdata.size()!=dd.size()){
	  return Sprintf("cannot apply function %s to vectors of different length",
			 fdef->guiName);
	}
	for(int i=0;i<dd.size();i++){
	  double a=dd.at(i);
	  double b=other.pxdata.at(i);
	  FUNC_ERRORS2;
	  dd[i]=(fdef->fp2)(a,b);
	}
      }
    }else if(other.ptype==VectorSet){
      long N=pgrdata.size();
      if(N!=other.pgrdata.size()){
	return Sprintf("cannot apply function %s to groups of different size",
		       fdef->guiName);
      }
      for(long j=-1;++j<N;){
	ValueArray<double> &dd=pgrdata[j];
	const ValueArray<double> &ddo=other.pgrdata.at(j);
	if(ddo.size()!=dd.size()){
	  return Sprintf("cannot apply function %s to vectors of different length",
			 fdef->guiName);
	}
	for(int i=0;i<dd.size();i++){
	  double a=dd.at(i);
	  double b=ddo.at(i);
	  FUNC_ERRORS2;
	  dd[i]=(fdef->fp2)(a,b);
	}
      }
      
    }else{
      return "TempSetDescriptor::func: type not yet implemented 4!";
    }
  }else{
    return "TempSetDescriptor::func: type not yet implemented!";
  }
  
  return "";
}



#undef FUNC_ERRORS
#undef FUNC_ERRORS2


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double TempSetDescriptor::minval()
{
  if(ptype==Value){
    return pvalue;
  }else if(ptype==Vector){
    return MyMath::minVal(pxdata);
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double TempSetDescriptor::maxval()
{
  if(ptype==Value){
    return pvalue;
  }else if(ptype==Vector){
    return MyMath::maxVal(pxdata);
  }
  return 0;
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TempSetDescriptor::setMesh(double x0,double x1,long N)
{
  pxdata.createMesh(MyMath::min<double>(x0,x1),MyMath::max<double>(x0,x1),N);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TempSetDescriptor::print(const char *txt)const
{
  if(ptype==Value){
    printf("%s=%g\n",txt,pvalue);
  }else if(ptype==Vector){
    for(int i=0;i<pxdata.size();i++){
      printf("%s[%d]=%g\n",txt,i,pxdata.at(i));
    }
  }else{
    printf("TempSetDescriptor::print: type not yet implemented!\n");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MyDebug operator<<(MyDebug dbg,const TempSetDescriptor &v)
{
  if(v.type()==TempSetDescriptor::Value){
    fprintf(dbg.FH,"tempsd=%g",v.value());
  }else if(v.type()==TempSetDescriptor::Vector){
    dbg<<"tempsd="<<v.xdata();
  }else{
    throw Except("TempSetDescriptor::func: type not yet implemented!");
  }
  return dbg;

}


/*EOF*/ 
