/* @(#)TempSetDescriptor.hh
 * author: magru@unas
 */

#ifndef _TEMPSETDESCRIPTOR_HH
#define _TEMPSETDESCRIPTOR_HH 1

#include "MyDebug.hh"
#include "ValueArray.hh"
#include "ParserFunctions.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class TempSetDescriptor
{
public:
  enum DataType{Value=1,Vector=2,VectorSet=3};
  
  TempSetDescriptor():ptype(Value){};
  TempSetDescriptor(double d):ptype(Value),pvalue(d){};
  TempSetDescriptor(DataType t):ptype(t){};
  TempSetDescriptor(const TempSetDescriptor & other):ptype(other.ptype){
    if(ptype==Value){
      pvalue=other.pvalue;
    }else if(ptype==Vector){
      pxdata=other.pxdata;
    }else{
      pgrdata=other.pgrdata;
    }
  }
  ~TempSetDescriptor(){}
  
  TempSetDescriptor &operator=(double d){
    if(ptype==Value){
      pvalue=d;
    }
    return *this;
  }
  TempSetDescriptor &operator=(const TempSetDescriptor & other){
    ptype=other.ptype;
    if(ptype==Value){
      pvalue=other.pvalue;
    }else if(ptype==Vector){
      pxdata=other.pxdata;
    }else{
      pgrdata=other.pgrdata;
    }
    return *this;
  }
  
  inline DataType type()const{return ptype;};
  
  inline double value()const{return pvalue;};
  
  double minval();
  double maxval();

  inline const ValueArray<double> &xdata()const{return pxdata;};
  inline ValueArray<double> &xdata(){return pxdata;};
  
  inline const ValueArray<ValueArray<double> > &grdata()const{return pgrdata;};
  inline  ValueArray<ValueArray<double> > &grdata(){return pgrdata;};
  
  void print(const char *txt)const;
  
  void setMesh(double x0,double x1,long N);
  
  String func(const struct ParserFunctions::tsfuncdef *fdef);
  String func(const struct ParserFunctions::tsfuncdef *fdef,
	      const TempSetDescriptor &other);
  
private:
  DataType ptype;
  ValueArray<double> pxdata;
  ValueArray<ValueArray<double> > pgrdata;
  double pvalue;
  
};

MyDebug operator<<(MyDebug dbg,const TempSetDescriptor &v);


#endif /* _TEMPSETDESCRIPTOR_HH */

