/*************************************************************************
 * @(#)GraphToolBox.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 May 2011
 *************************************************************************
 */

#include <QColor>
#include "AntiAliasing.hh"
#include "Graph.hh"
#include "Graphs.hh"
#include "GraphToolBox.hh"
#include "Page.hh"
#include "World.hh"
#include "graphview.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphToolBox::GraphToolBox(QWidget *parent)
  :QDockWidget(parent),p_gv(0),mutecmbWorldStack(false)
{
  setupUi(this);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
GraphToolBox::~GraphToolBox()
{

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphToolBox::setGraphView(GraphView *gv)
{
  p_gv=gv;
  if(p_gv){
    connect(tbAA,SIGNAL(clicked()),p_gv,SLOT(autoScaleAll()));
    tbAA->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_A));
    connect(tbAX,SIGNAL(clicked()),p_gv,SLOT(autoScaleX()));
    tbAX->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_X));
    connect(tbAY,SIGNAL(clicked()),p_gv,SLOT(autoScaleY()));
    tbAY->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Y));
    connect(tbAf,SIGNAL(clicked()),p_gv,SLOT(autoScaleFunc()));
    connect(tbZoomIn,SIGNAL(clicked()),p_gv,SLOT(zoomIn()));
    tbZoomIn->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Plus));
    connect(tbZoomOut,SIGNAL(clicked()),p_gv,SLOT(zoomOut()));
    tbZoomOut->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Minus));
    connect(tbZoomAll,SIGNAL(clicked()),p_gv,SLOT(zoomAll()));
    connect(tbMoveLeft,SIGNAL(clicked()),p_gv,SLOT(moveLeft()));
    tbMoveLeft->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Left));
    connect(tbMoveRight,SIGNAL(clicked()),p_gv,SLOT(moveRight()));
    tbMoveRight->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Right));
    connect(tbMoveUp,SIGNAL(clicked()),p_gv,SLOT(moveUp()));
    tbMoveUp->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Up));
    connect(tbMoveDown,SIGNAL(clicked()),p_gv,SLOT(moveDown()));
    tbMoveDown->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Down));
    connect(tbZoomX,SIGNAL(clicked()),p_gv,SLOT(zoomX()));
    connect(tbZoomY,SIGNAL(clicked()),p_gv,SLOT(zoomY()));
    connect(tbRuler,SIGNAL(clicked()),p_gv,SLOT(ruler()));
    connect(tbSaveWorld,SIGNAL(clicked()),this,SLOT(saveWorld()));
    connect(tbClearWorldStack,SIGNAL(clicked()),this,SLOT(clearWorldStack()));
    connect(tbAntiA,SIGNAL(toggled(bool)),p_gv,SLOT(toggleAntiA(bool)));
    connect(tbPlay,SIGNAL(clicked()),p_gv,SLOT(playParamDocument()));
    tbAntiA->setChecked(antiAliasing.isOn());
    
    connect(p_gv,SIGNAL(reloadWorldStack()),
	    this,SLOT(onReloadWorldStack()));

    connect(this,SIGNAL(worldChanged()),p_gv,SLOT(onWorldChanged()));

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void  GraphToolBox::saveWorld()
{
  //myDebug()<<"GraphToolBox::saveWorld";
  if(!p_gv) return;
  p_gv->saveWorld();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphToolBox::onReloadWorldStack()
{
  Page *page=p_gv->page();
  const ValueArray<WorldWindow> &ww=page->worldStack();
  //  long ngr=page->currentGraphNumber();
  mutecmbWorldStack=true;
  cmbWorldStack->clear();
  for(int i=0;i<ww.size();i++){
    cmbWorldStack->insertItem(i,Sprintf("%d: G%ld",i+1,
					ww.at(i).graphId()+1)(),i);
  }
  mutecmbWorldStack=false;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphToolBox::on_cmbWorldStack_activated(int current)
{
  if(mutecmbWorldStack)return;
  if(current<0)return;
  //myDebug() << "GraphToolBox::on_cmbWorldStack_activated(int current)";
  Page *page=p_gv->page();
  const ValueArray<WorldWindow> &wws=page->worldStack();
  const WorldWindow &ww=wws.at(current);
  Graph *gr=page->graphs()->currentGraph();
  if(gr){
    gr->world()->setW0(ww.w0());
    gr->world()->setW1(ww.w1());
    emit worldChanged();
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphToolBox::clearWorldStack()
{
  if(!p_gv) return;
  Page *page=p_gv->page();
  page->clearWorldStack();
  onReloadWorldStack();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void GraphToolBox::setMessageText(const QString& s)
{
  pltxedMeasure->setPlainText(s);
}

/*EOF*/ 
