/*************************************************************************
 * @(#)MainWindow.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 May 2011
 *************************************************************************
 */

#include <QMenuBar>
#include <QMessageBox>
#include <QToolBar>
#include <QKeySequence>
#include "Page.hh"
#include "Graph.hh"
#include "allocread.h"
#include "Assorted.hh"
#include "AfterShowEvent.hh"
#include "Except.hh"
#include "Parser.hh"
#include "MainWindow.hh"
#include "GraphToolBox.hh"
#include "ParameterDocument.hh"
#include "HelpDlg.hh"
#include "XFPStatusBar.hh"
#include "Settings.hh"
#include "graphview.hh"
#include "PageSetupDlg.hh"
#include "ObserveView.hh"
#include "plotview.hh"
#include "Config.hh"
#include "defs.hh"
#include "ParameterList.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
MainWindow::MainWindow(QWidget *parent,Qt::WindowFlags)
  :QMainWindow(parent),p_gv(0),pov(new ObserveView(this)),
   datamenu(0),editmenu(0),modemenu(0),pageSetupDlg(0)
{
  setWindowTitle(IDS_APP_NAME);
  setWindowIcon( QIcon(":Icons/logo32")  );

  resize(864, 697);

  graphtoolbox=new GraphToolBox(this);
  graphtoolbox->setObjectName("GraphToolBox");
  graphtoolbox->setFloating(false);
  graphtoolbox->setAllowedAreas(Qt::LeftDockWidgetArea);
  addDockWidget(Qt::LeftDockWidgetArea, graphtoolbox);

  filemenu=menuBar()->addMenu("&File");

  QAction *fileNewAction=new QAction(QIcon(":/Icons/filenew"),"&New",this);
  fileNewAction->setStatusTip(tr("New graph"));
  connect(fileNewAction,SIGNAL(triggered()),this,SLOT(fileNew()));

  QAction *fileOpenAction=new QAction(QIcon(":/Icons/fileopen"),"&Open",this);
  fileOpenAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_O));
  fileOpenAction->setStatusTip(tr("Open a file"));
  connect(fileOpenAction,SIGNAL(triggered()),this,SLOT(fileOpen()));


  QAction *fileSaveAction=new QAction(QIcon(":/Icons/filesave"),"&Save",this);
  fileSaveAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_S));
  fileSaveAction->setStatusTip(tr("Save to file"));
  connect(fileSaveAction,SIGNAL(triggered()),this,SLOT(fileSave()));

  QAction *fileSaveAsAction=new QAction(QIcon(":/Icons/filesaveas"),"Save&As",this);
  fileSaveAsAction->setStatusTip(tr("Save as new file"));
  connect(fileSaveAsAction,SIGNAL(triggered()),this,SLOT(fileSaveAs()));

  fileQuitAction=new QAction(QIcon(":/Icons/exit"),"&Quit",this);
  fileQuitAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Q));
  fileQuitAction->setStatusTip(tr("Close application"));
  connect(fileQuitAction,SIGNAL(triggered()),this,SLOT(fileQuit()));

  QAction *filePrintAction=new QAction(QIcon(":/Icons/print"),"&Print",this);
  filePrintAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_P));
  filePrintAction->setStatusTip(tr("Close application"));
  connect(filePrintAction,SIGNAL(triggered()),this,SLOT(filePrint()));
  
  filemenu->addAction(fileNewAction);
  filemenu->addAction(fileOpenAction);
  filemenu->addAction(fileSaveAction);
  filemenu->addAction(fileSaveAsAction);
  filemenu->addSeparator();
  filemenu->addAction(filePrintAction);
  filemenu->addSeparator();
  filemenu->addAction(fileQuitAction);

  QAction *a;

  modemenu=menuBar()->addMenu("&Mode");

  a=new QAction("&Plot",this);
  a->setStatusTip(tr("Normal plot mode"));
  a->setData(ObserveView::PlotMode);
  connect(a,SIGNAL(triggered()),this,SLOT(plotMode()));
  modemenu->addAction(a);

  QMenu *m;
  m=modemenu->addMenu("&Observe");
  a=new QAction("&Orbitals",this);
  a->setStatusTip(tr("Orbitals"));
  a->setData(ObserveView::Orbitals);
  connect(a,SIGNAL(triggered()),this,SLOT(plotMode()));
  m->addAction(a);

  a=new QAction("&Iteration progress",this);
  a->setStatusTip(tr("iteration progress"));
  a->setData(ObserveView::Iteration);
  connect(a,SIGNAL(triggered()),this,SLOT(plotMode()));
  m->addAction(a);

  
  QMenu *settingsmenu=menuBar()->addMenu("&Settings");
  a=new QAction("&Page setup",this);
  connect(a,SIGNAL(triggered()),this,SLOT(pageSetup()));
  settingsmenu->addAction(a);
  
  

  menuBar()->addSeparator();

  helpmenu=menuBar()->addMenu("&Help");



  QAction *helpAboutAction=new QAction("&About",this);
  connect(helpAboutAction,SIGNAL(triggered()),this,SLOT(about()));

  QAction *helpAboutQtAction=new QAction("About &Qt",this);
  connect(helpAboutQtAction,SIGNAL(triggered()),qApp,SLOT(aboutQt()));

  helpmenu->addAction("&Help",this,SLOT(help()),QKeySequence(Qt::Key_F1));
  helpmenu->addAction(helpAboutAction);
  helpmenu->addAction(helpAboutQtAction);


  pstatusbar=new XFPStatusBar(this);
  setStatusBar(pstatusbar);
  

  fileToolBar = addToolBar("File toolbar");
  fileToolBar->addAction(fileNewAction);
  fileToolBar->addAction(fileOpenAction);
  fileToolBar->addAction(fileSaveAction);
  fileToolBar->addAction(filePrintAction);
  fileToolBar->addAction(fileQuitAction);

  




  centralwidget = new QWidget(this);
  centralwidget->setObjectName("centralwidget");
  this->setCentralWidget(centralwidget);
  
	
  p_gv=new PlotView(this); 
  p_gv->setMinimumSize(QSize(105,150)); 
  connect(p_gv,SIGNAL(managePopupMenu(const QString &, QMenu*))
	  ,this,SLOT(manageForeignPopups(const QString&, QMenu*)));
  connect(p_gv,SIGNAL(iAmBusy(bool)),this,SLOT(onBusyGraphView(bool)));
  graphtoolbox->setGraphView(p_gv);
  p_gv->setup();
  p_gv->show();
  connect(p_gv,SIGNAL(measurementText(const QString&)),
	  graphtoolbox,SLOT(setMessageText(const QString&)));

  Settings set(QString("mainwindow ")+"graph");
  resize(set.getSize("geometry/size",QSize(825, 560)));
  move(set.getPoint("geometry/pos",QPoint(100,100)));
  

  specialize(ObserveView::PlotMode);
  


  QEvent *event=new AfterShowEvent(AfterShowEvent::AfterShowEventType,"");
  QCoreApplication::postEvent(this,event);



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MainWindow::~MainWindow()
{
  Settings set(QString("mainwindow ")+"graph");
  set.set("geometry/size",size());
  set.set("geometry/pos",pos());

 if(p_gv) delete p_gv;p_gv=0;
 if(pov) { delete pov; pov=0;}

 if(!pageSetupDlg.isNull()) delete pageSetupDlg;pageSetupDlg=0;
 if(graphtoolbox) delete graphtoolbox; graphtoolbox=0;
 if(pstatusbar) delete pstatusbar;pstatusbar=0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::fileNew()
{
  if(p_gv && p_gv->onExit() && p_gv->page()){
    p_gv->killAll();
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::fileOpen()
{
  if(p_gv){
    p_gv->load();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::fileSave()
{
  if(p_gv){
    p_gv->save(0);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::fileSaveAs()
{
  if(p_gv){
    p_gv->save(1);
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::fileQuit()
{
  if(p_gv && p_gv->onExit()) {
    close();
    qApp->closeAllWindows();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::filePrint()
{
  if(p_gv){
    p_gv->print(1L);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::pageSetup()
{
  if(!pageSetupDlg.isNull()) {
    pageSetupDlg->activateWindow();
    pageSetupDlg->raise();
  }else{
    pageSetupDlg=new PageSetupDlg(0,this,"PageSetupDlg");
    connect(pageSetupDlg,SIGNAL(acceptRequest())
	    ,this,SLOT(onPageSetupDlgAccept()));
    pageSetupDlg->show();
  }
  pageSetupDlg->setData(p_gv->page());


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::onPageSetupDlgAccept()
{
  //  myDebug()<<"MainWindow::onPageSetupDlgAccept";
  p_gv->updatePageSize();
  p_gv->refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::help()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","Contents");
  h->show();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::about()
{
  QMessageBox::about (this,this->windowTitle(),IDS_APP_ABOUT);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::plotMode()
{
  ObserveView::Mode what=(ObserveView::Mode)(((QAction*)sender())->data().toInt());
  specialize(what);  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::specialize(ObserveView::Mode what)
{
  // take out p_gv for reinsertion
  if(centralwidget->layout()){
    int index=centralwidget->layout()->indexOf(p_gv);
    //myDebug() << "centralwidget->layout()->indexOf(p_gv)="<<index;
    if(index>=0) centralwidget->layout()->takeAt(index);
  }

  // take out all stuff implemented by pov
  String pobserveoutfile=pov->outFile();
  pov->cleanUp();
  
 if(centralwidget->layout()) delete centralwidget->layout();
  
  if(what==ObserveView::PlotMode){
    
    QGridLayout *gl=new QGridLayout(centralwidget);
 
    gl->addWidget(p_gv, 0, 0, 1, 1);
    gl->setColumnStretch (1,0);
    
    setWindowTitle(IDS_APP_NAME+QString("(Plot mode)"));
  }else{
    
    //QGridLayout *gl=new QGridLayout(centralwidget);
    pov->setOutfile(pobserveoutfile);
    pov->setObjectName("ObserveView");
    pov->setUp(centralwidget,p_gv,what);
    
    setWindowTitle(IDS_APP_NAME+QString("(Observe mode)"));
  }
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::manageForeignPopups(const QString &mode,QMenu *menu)
{
  //myDebug() << "manageForeignPopups";
  QMenu *before=modemenu;;
  if(mode=="data"){
    if(editmenu)
      before=editmenu;
    datamenu=menu;
  }else if(mode=="edit"){
    editmenu=menu;
  }
  //myDebug()<<"MainWindow::manageForeignPopups"<<mode;
  menuBar()->insertMenu(before->menuAction(),menu);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::onCmdArguments()
{
  // sort out non-file arguments and execute them;
  // delegate the rest to a file reader

  const QStringList &arg=qApp->arguments();
  int len=arg.size();
  bool gottype=false;
  Xfp::FileType type=Xfp::FTXNY;
  QString filename="";
  QString cdir=".";
  long ifilename=0;
  bool debugparser=false,cwd=false;
  long funcindex=0;
  //  bool observiter=false;
  for(int i=1;i<len;i++){
    //myDebug()<<"type"<<type;
    if(arg[i]=="-xny"){
      gottype=true;
      type=Xfp::FTXNY;
    }else if(arg[i]=="-cwd"){
      cwd=true;
    }else if(arg[i]=="-xynz"){
      gottype=true;
      type=Xfp::FTXYNZ;
    }else if(arg[i]=="-pp"){
      debugparser=true;
    }else if(arg[i]=="-p"){
      type=Xfp::FTParam;
      gottype=true;
    }else if(arg[i]=="-xynw"){
      type=Xfp::FTXYNW;
      gottype=true;
    }else if(arg[i]=="-band"){ 
      type=Xfp::FTBand;
      gottype=true;
    }else if(arg[i]=="-bandweight"){
      type=Xfp::FTBandWeight;
      gottype=true;
    }else if(arg[i]=="-bandmatel"){
      type=Xfp::FTBandMatel;
      gottype=true;
    }else if(arg[i]=="-akbl"){
      type=Xfp::FTAkbl;
      gottype=true;
    }else if(arg[i]=="-oi"){
      //observiter=true;
      specialize(ObserveView::Iteration);
    }else if(arg[i]=="-a"){
      i++;
    }else{
      filename=arg[i];
      if(!filename.isEmpty()){
	//myDebug() << filename;
	if((++ifilename)==1){
	  QFileInfo fi(filename);
	  cdir=fi.absolutePath();
	  if(cwd)QDir::setCurrent(cdir);
	}
	if(p_gv){

	  
	  if(!gottype) type=guessFileType(filename);
	  //	  myDebug()<< "type"<< type;
	  if(type==Xfp::FTGraph){
	    p_gv->loadFile(filename,false);
	  }else if(type==Xfp::FTParam){
	    p_gv->parameterDocument()->loadDocument(filename);
	    p_gv->playParamDocument(filename,debugparser);
	  }else if(type==Xfp::FTGraphOld){
	    p_gv->loadFile(filename,true);
	  }else{
	    //myDebug()<< "before";
	    p_gv->import(false,filename,type,funcindex);
	    //myDebug()<< "after";
	  }
	}
      }
    }
    //todo: make multiple options
  }
  p_gv->refresh();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Xfp::FileType MainWindow::guessFileType(const QString &filename)
{
  NSF("MainWindow::guessFileType");
  //myDebug()<< "guessFileType";
  String mesg;
  String fname(filename);
  FILE *FHI=NULL;
  char *pbuffer=NULL;
  long lenpbuffer=0;
  Xfp::FileType ret=Xfp::FTXNY;
  bool foundexplicit=false;
  bool isband=true;
  bool isbandweight=true;
  bool isakbl=true;
  bool isxfpl=false;
  if(1){
    Parser par(1);
    try{
      par.parseFile(fname());
      String s;
      try{
	s=par()("filetype.application")();
	if(s=="xfpl"){
	  return Xfp::FTGraphOld;
	}
      }catch(Except &e){};

    }catch(Except &ex){
      //fall through
    }
  }

  try{
    FHI=openforread(fname);
  }catch(Except &ex){
    mesg=ex.msg();
    goto error;
  }

  

 
  //line 1
  if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ){
    (void)mesg.sprintf("%s: Can not read file \'%s\'!\n"
		       ,nsubfunc,fname());
    goto error;
  }

  if(String(pbuffer).lowerCase().subString(0,16)=="xfpl file version"){
    isxfpl=true;
    goto clean;
    
  }


  if(pbuffer[0]!='#'){
    isbandweight=false;
    isband=false;
    isakbl=false;
  }
  if(String(pbuffer).subString(0,7)!="# nsite="){
    isakbl=false;
  }

  //myDebug()<<"Line 1"<<pbuffer<<isband<<isbandweight<<isakbl;

  //line 2
  if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
  

  if(isbandweight && 
     String(pbuffer).subString(0,26)=="#       ik           e(k,n)"){
    ret=Xfp::FTBandWeight;
    foundexplicit=true;
    goto clean;
  }

  if(isakbl && 
     String(pbuffer).subString(0,18)=="# av. fermi energy="){
    ret=Xfp::FTAkbl;
    foundexplicit=true;
    goto clean;
  }

  if(pbuffer[0]!='#') isband=false;
  //myDebug()<<"Line 2"<<pbuffer[0]<<isband;
  

  //line 3
  if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
  if(pbuffer[0]=='#') isband=false;
  //myDebug()<<"Line 3"<<pbuffer[0]<<isband;
  
  //line 4 
  if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
  if(pbuffer[0]=='#') {
    //myDebug()<<"Line 4"<<pbuffer[0]<<isband;
    //line 4 #
    //line 5 _
    if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
    if(pbuffer[0]=='#') isband=false;    
    //line 6 #
    if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
    if(pbuffer[0]!='#') isband=false;
  }else{
    //myDebug()<<"Line 4"<<pbuffer[0]<<isband;
    //line 4 _
    //line 5 #
    if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
    if(pbuffer[0]!='#') isband=false;
    //myDebug()<<"Line 5"<<pbuffer[0]<<isband;
    //line 6 #
    if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
    if(pbuffer[0]=='#') isband=false;
    //myDebug()<<"Line 6"<<pbuffer[0]<<isband;
    //line 7 _
    if  ( !(allocreadbuf(FHI,&pbuffer,&lenpbuffer)) ) goto clean;
    if(pbuffer[0]=='#') isband=false;    
    //myDebug()<<"Line 7"<<pbuffer[0]<<isband;
  }
  if(isband) {
    ret=Xfp::FTBand;
    foundexplicit=true;
  }


 clean:
  free(pbuffer);

  if(FHI!=NULL){ fclose(FHI);FHI=NULL; }

  //if(!mesg.empty())
  //  throw Except(mesg);
  if(isxfpl) {
    ret=Xfp::FTGraph;
    foundexplicit=true;
  }
  
  if(!foundexplicit){
    if(filename.endsWith(".cmd")){
      ret=Xfp::FTParam;
    }
    if(filename.endsWith(".xfp")){
      ret=Xfp::FTGraph;
    }
  }
  return ret;


 error:
  goto clean;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool MainWindow::event(QEvent *event)
{
  if(event->type()==AfterShowEvent::AfterShowEventType){
    if(/*AfterShowEvent *ae=*/dynamic_cast<AfterShowEvent*>(event)){
      try{ 
	onCmdArguments();
      }catch(Except &ex){
	myDebug()<<ex.msg();
	exit(1);
      }
    }
    return true;
  }
  return QWidget::event(event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MainWindow::onBusyGraphView(bool busy)
{
  //  myDebug()<<"MainWindow::onBusyGraphView()";
  if(busy){
    p_gv->setEnabled(false);
    graphtoolbox->setEnabled(false);
  }else{
    p_gv->setEnabled(true);
    graphtoolbox->setEnabled(true);
  }

  foreach(QObject *o,menuBar()->children()){
    QMenu *m;
    if((m=qobject_cast<QMenu*>(o))){
      if(m!=filemenu){
 	m->setEnabled(!busy);
      }
    }
  }

  foreach(QAction *a,filemenu->actions()){
    if(a!=fileQuitAction){
      a->setEnabled(!busy);
    }
  }

}

/*EOF*/ 
