/* @(#)ObserveView.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OBSERVEVIEW_H
#define _OBSERVEVIEW_H 1

#include <QObject>
#include "OutGrep.hh"

class GraphView;
class MainWindow;
class QComboBox;
class QCheckBox;
class QDoubleSpinBox;
class QGridLayout;
class QPlainTextEdit;
class QSplitter;
class QLabel;
class QPushButton;
class QTreeWidget;
class QTreeWidgetItem;
class QLineEdit;
class QTimer;
class QSpinBox;

class ObserveView: public QObject
{
  Q_OBJECT
 public: 
  enum Mode{PlotMode=0,Orbitals=1,Iteration=2};

  ObserveView(MainWindow *parent);
  ~ObserveView();
  GraphView *graphView(){return p_gv;};
  void setUp(QWidget *parentwidget,GraphView *gv,Mode what);
  void cleanUp();

  void setOutfile(const String &fn){pobserveoutfile=fn;};
  const String &outFile()const {return pobserveoutfile;};

 private slots:
  void onTrwOrbitalsCurrentItemChanged(QTreeWidgetItem *,QTreeWidgetItem *);
  void onPbBrowseClicked();
  void iterationMode();
  void orbitalMode();
  void onPbUpdateClicked();  
  void onUpdateTimer();
  void onSpbxIter0ValueChanged(int);
  void onSpbxAutoUpdateTimeValueChanged(int);
  void onCbAutoUpdateStatechanged(int);
  void onDspbxRmaxValueChanged(double );  

 private:
  void occupyCmbSite();
  
 private:
  GraphView *p_gv;
  QWidget *parentwidget;
  // widgets
  QTreeWidget *trwOrbitals;
  QLabel *lSelect;
  QComboBox *cmbMode;
  QLabel *lOutputFile;
  QLineEdit *leOutFile;
  QPushButton *pbBrowse;
  QLabel *lSite;
  QComboBox *cmbSite;
  QPushButton *pbUpdate;
  QTimer *updateTimer;
  QLabel *lIter0;
  QSpinBox *spbxIter0;
  QCheckBox *cbAutoUpdate;
  QSpinBox *spbxAutoUpdateTime;
  QPlainTextEdit *plaintextedit;
  QSplitter *splitter;
  QDoubleSpinBox *dspbxRmax;
  QLabel *lRmax;
  double rmax;
  Mode pwhat;
  OutGrep poutgreper;
  String pobserveoutfile;
  bool muteIterationMode;
  int pautoupdatetime;
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OBSERVEVIEW_H */

