/*************************************************************************
 * @(#)QtPaintEngine.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 May 2011
 *************************************************************************
 */

#include <QPainter>
#include "Attributes.hh"
#include "Paper.hh"
#include "QtPaintEngine.hh"
#include "LineShape.hh"
#include "EllipseShape.hh"
#include "MyMath.hh"
#include "defs.hh"
#include "MyDebug.hh"



#define TEXTBOX_DEBUG  1
#undef  TEXTBOX_DEBUG  

#define TEXTBOX_DEBUG_SHOW  1
#undef  TEXTBOX_DEBUG_SHOW  

#define TEXTBOX_DEBUG_TEXTBOX  1
#undef  TEXTBOX_DEBUG_TEXTBOX  

class FontInfo{
public:
  const char *name;
  bool italic;
};

static const Qt::PenCapStyle qtcapstyles[]={
  Qt::FlatCap,
  Qt::RoundCap,
  Qt::SquareCap
};

static const FontInfo qt_fonts[]={
  {"Helvetica",false},
  {"Helvetica",true},
  {"Symbol",false},
  //"-adobe-helvetica-medium-r-normal--20-140-100-100-p-100-iso8859-1"
  //,"-adobe-symbol-medium-r-normal--20-140-100-100-p-107-adobe-fontspecific"
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
QtPaintEngine::QtPaintEngine(QPaintDevice *pd,bool antialiasing)
  :PaintEngine(),wp(new QPainter()),pwidth(pd->width()),pheight(pd->height()),
   pdrag(false)
{
  defineValue(PaintEngine::LegendBoxWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::LegendBoxHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::LegendTextMaxHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::LegendTextMaxWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::SymbolSize,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::MaxSymbolSize,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::LegendSymbolOffset,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::XNormalTicLabelHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::YNormalTicLabelWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::XOpositeTicLabelHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::YOpositeTicLabelWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextAscent,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextDescent,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextBoxHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::TextBoxWidth,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::SubtitleHeight,0.,PaintEngine::Absolute); 
  defineValue(PaintEngine::XOppositeXALabelHeight,0.,PaintEngine::Absolute); 

  pwidthMM=pd->widthMM();

  wp->begin(pd);
  wp->save();
  if(antialiasing)
    wp->setRenderHint(QPainter::Antialiasing);
  pantialiasing=antialiasing;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QtPaintEngine::~QtPaintEngine()
{
  qDeleteAll(pfonts);
  wp->restore();
  wp->end();
  if(wp) delete wp;wp=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::init(const Paper &paper)
{
  ppaper=paper;

  wp->translate(0,0);
  pangles.append(0.);
  pcurrentangle=0.;
  pcangle=cos(pcurrentangle/180.*Pi);
  psangle=sin(pcurrentangle/180.*Pi);
  
  double w=ppaper.paperWidth();
  //  double h=ppaper.paperHeight();
  

  pscreenratio=( (1.0*pwidthMM /( w*Xfp::pointsToInch*Xfp::inchToCM*10)) );

  plinewidthratio=(1.0*pwidth/w);
  

  /*

    how many px is one Pt?

    physical_screen_width=pwidthMM mm
    physical_screen_width=paperwidth_in_Pt Pt
    physical_screen_width=pwidth px

    (Pt/mm)=(Pt/in*in/cm*10)

    the factor
    1 Pt = pwidthMM/(paperwidth*(Pt/mm)) mm
    has to be used for scaling qt-things, which are given in Pt

    the factor 
    1 Pt = pwidth/paperwidth_in_Pt px
    has to be used for scaling qt-things, which are given in px,
    in order to obtain the size in Pt
    
    linewidth=x*Pt
    linewidth=x*pwidth/(paperwidth_in_Pt) px
    linewidth=x*pscreenratio*(pwidth/pwidthMM) px
    
  */

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

double QtPaintEngine::argvalue(Variable fac,double val )
{
  NSF("argvalue");

  if(pvalues.contains(fac)) 
    return pvalues[fac]*val;
  else{
      printf("%s:internal variable v%d not defined!\n",nsubfunc,fac);
      exit(1);
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

double QtPaintEngine::argvalue(double val,ValueType relarg)
{
  NSF("argvalue");
  if(relarg==RelativeX){
    return val*pwidth;
  }else if(relarg==RelativeY){
    return val*pheight;
  }else if(relarg==Absolute){
    return val;
  }else
    return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double QtPaintEngine::getValue(Variable var,ValueType relarg)
{
  NSF("getQTValue");

  if(!pvalues.contains(var)){
    printf("%s:internal variable v%d not defined!\n",nsubfunc,var);
    exit(1);
    return 0;
  }
  if(relarg==Absolute){
    return pvalues[var];
  }else if(relarg==RelativeX){
    return pvalues[var]/pwidth;
  }else if(relarg==RelativeY){
    return pvalues[var]/pheight;
  }
  return 0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::arithValue(Variable var,
			       Variable fac2 ,double val2,
			       Variable fac3 ,double val3,
			       Operation op)
{
  NSF("arithQTValueValue");

  if(!pvalues.contains(var)){
    pvalues[var]=0;
    printf("%s:internal variable v%d not defined!\n",nsubfunc,var);
    exit(1);
  }


  double v2=argvalue(fac2,val2);
  double v3=argvalue(fac3,val3);


  if(op==Plus) {
    pvalues[var]=v2+v3; 
  }else if(op==Minus) {
    pvalues[var]=v2-v3; 
  }else if(op==Mult) {
    pvalues[var]=v2*v3; 
  }else if(op==Div) {
    pvalues[var]=v2/v3; 
  }else if(op==Max) {
    pvalues[var]=MyMath::max<double>(v2,v3); 
  }else if(op==Min) {
    pvalues[var]=MyMath::min<double>(v2,v3); 
  }  

  //printf("%s(...)=%g\n",op,pvalues[var]);
  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::arithValue(Variable var,
			       Variable fac2 ,double val2,
			       double val3 ,ValueType relarg3,
			       Operation op)
{
  NSF("arithQTValueValue");

  if(!pvalues.contains(var)){
    pvalues[var]=0;
    printf("%s:internal variable v%d not defined!\n",nsubfunc,var);
    exit(1);
  }


  double v2=argvalue(fac2,val2);
  double v3=argvalue(val3,relarg3);

  if(op==Plus) {
    pvalues[var]=v2+v3; 
  }else if(op==Minus) {
    pvalues[var]=v2-v3; 
  }else if(op==Mult) {
    pvalues[var]=v2*v3; 
  }else if(op==Div) {
    pvalues[var]=v2/v3; 
  }else if(op==Max) {
    pvalues[var]=MyMath::max<double>(v2,v3); 
  }else if(op==Min) {
    pvalues[var]=MyMath::min<double>(v2,v3); 
  }  

  //printf("%s(...)=%g\n",op,pvalues[var]);
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::prepAAOddset(double &aadx,double &aady)
{
  int width=wp->pen().width();
  aadx=0;
  aady=0;
  //if(pantialiasing && width%2==1){
  if(pantialiasing){
    if(width%2==1){
      aadx=pcangle-psangle;
      aady=pcangle+psangle;
    }else{
      aadx=(pcangle-psangle)*0.5;
      aady=(pcangle+psangle)*0.5;
    }
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void QtPaintEngine::drawRectFromValue(Variable fac1,double val1,
// 				      Variable fac2,double val2,
// 				      Variable fac3,double val3,
// 				      Variable fac4,double val4,
// 				      QRgb fc,bool fill,RectType recttype)
// {
//   //NSF("drawQTRectFromValue");
//   return;
//   double x0=argvalue(fac1,val1);
//   double y0=argvalue(fac2,val2);
//   double x1=argvalue(fac3,val3);
//   double y1=argvalue(fac4,val4);
//   if(recttype==ULWH){
//     x1+=x0;
//     y1+=y0;
//   }


//   double w=x1-x0;
//   double h=y1-y0;
  
//   if(fill){
//     wp->fillRect(QRectF(x0,y0,w,h),QColor(fc));
//   }
//   wp->drawRect(QRectF(x0,y0,w,h));

  
  
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawRectFromValue(double val1,ValueType relarg1,
				      double val2,ValueType relarg2,
				      Variable fac3,double val3,
				      Variable fac4,double val4,
				      QRgb fc,bool fill,RectType recttype)
{
  //NSF("drawQTRectFromValue");
  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);
  double x1=argvalue(fac3,val3);
  double y1=argvalue(fac4,val4);
  if(recttype==ULWH){
    x1+=x0;
    y1+=y0;
  }


  double w=x1-x0;
  double h=y1-y0;
  
  double aadx,aady;
  prepAAOddset(aadx,aady);
  if(fill){
    wp->fillRect(QRectF(x0+aadx,y0+aady,w,h),QColor(fc));
  }
  wp->drawRect(QRectF(x0+aadx,y0+aady,w,h));

  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawRectFromValue(double val1 ,ValueType relarg1,
				      double val2 ,ValueType relarg2,
				      double val3 ,ValueType relarg3,
				      double val4 ,ValueType relarg4,
				      QRgb fc,bool fill,RectType recttype)
{
  NSF("drawQTRectFromValue");

  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);
  double x1=argvalue(val3,relarg3);
  double y1=argvalue(val4,relarg4);
  if(recttype==ULWH){
    x1+=x0;
    y1+=y0;
  }


  double w=x1-x0;
  double h=y1-y0;

  double aadx,aady;
  prepAAOddset(aadx,aady);
  if(fill){
    wp->fillRect(QRectF(x0+aadx,y0+aady,w,h),QColor(fc));
  }
  wp->drawRect(QRectF(x0+aadx,y0+aady,w,h));

  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawLineFromValue(double val1 ,ValueType relarg1,
				      double val2 ,ValueType relarg2,
				      double val3 ,ValueType relarg3,
				      double val4 ,ValueType relarg4,
				      RectType recttype)
{
  double aadx,aady;
  prepAAOddset(aadx,aady);

  double x1,y1;
  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);

  if(recttype==ULWH){
    x1=(argvalue(val3,relarg3)+x0);
    y1=(argvalue(val4,relarg4)+y0);
  }else{
    x1=(argvalue(val3,relarg3));
    y1=(argvalue(val4,relarg4));
  }
  wp->drawLine(QLineF(x0+aadx,y0+aady,x1+aadx,y1+aady));
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawLineFromValue(double val1 ,ValueType relarg1,
				      double val2 ,ValueType relarg2,
				      Variable fac3,double val3,
				      double val4 ,ValueType relarg4,
				      RectType recttype)
{
  double aadx,aady;
  prepAAOddset(aadx,aady);


  double x1,y1;
  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);

  if(recttype==ULWH){
    x1=(argvalue(fac3,val3)+x0);
    y1=(argvalue(val4,relarg4)+y0);
  }else{
    x1=(argvalue(fac3,val3));
    y1=(argvalue(val4,relarg4));
  }
  wp->drawLine(QLineF(x0+aadx,y0+aady,x1+aadx,y1+aady));
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawLineRel(double x0,double y0,double x1,double y1
				,RectType recttype)
{
  NSF("drawQTLineRel");
  double aadx,aady;
  prepAAOddset(aadx,aady);

  if(recttype==ULWH){
    double lx0=(x0*pwidth);
    double ly0=(y0*pheight);
    double lx1=((x1+x0)*pwidth);
    double ly1=((y1+y0)*pheight);
    wp->drawLine(QLineF(lx0+aadx,ly0+aady,lx1+aadx,ly1+aady));
  }else{
    double lx0=(x0*pwidth);
    double ly0=(y0*pheight);
    double lx1=(x1*pwidth);
    double ly1=(y1*pheight);
    
    wp->drawLine(QLineF(lx0+aadx,ly0+aady,lx1+aadx,ly1+aady));
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawArrow(double x0,double y0,double x1,double y1,
			      const LineShape *lsh)
{
  NSF("drawQTLineRel");
  double aadx,aady;
  prepAAOddset(aadx,aady);

  setPen(lsh->lineStyle()->color(),
	 lsh->lineStyle()->style(),
	 lsh->lineStyle()->width());

  QPointF s((x0*pwidth)+aadx,(y0*pheight)+aady);
  QPointF e((x1*pwidth)+aadx,(y1*pheight)+aady);

  double w=lsh->arrowHeadSize();
  w*=plinewidthratio*palloverPtscale;

  double h=w/sqrt(4.*lsh->sharpness()*lsh->sharpness()-1.);
  QPointF dd=e-s;
  double ldd=sqrt(dd.x()*dd.x()+dd.y()*dd.y());
  dd/=ldd;
  QPointF ort=QPointF(-dd.y(),dd.x());
  QPointF ss=s;
  QPointF ee=e;

  
  if(lsh->arrowStyle()==Xfp::ARSTClosed){
    if(lsh->arrowPosition() & Xfp::ARPStart){
      ss=s+w*dd;
    }
    if(lsh->arrowPosition() & Xfp::ARPEnd){
      ee=e-w*dd;
    }
  }
  
  wp->drawLine(ss,ee);


  if(lsh->arrowPosition() & Xfp::ARPStart){
    QPointF *points=new QPointF[3];
    points[0]=s+w*dd+h*ort;
    points[1]=s;
    points[2]=s+w*dd-h*ort;
    if(lsh->arrowStyle()==Xfp::ARSTClosed){
      wp->setBrush(QColor(lsh->arrowHeadFillColor()));
      wp->drawPolygon(points,3);
    }else{
      wp->drawPolyline(points,3);
    }
    delete [] points;
  }
  if(lsh->arrowPosition() & Xfp::ARPEnd){
    
    QPointF *points=new QPointF[3];
    points[0]=e-w*dd+h*ort;
    points[1]=e;
    points[2]=e-w*dd-h*ort;
    if(lsh->arrowStyle()==Xfp::ARSTClosed){
      wp->setBrush(QColor(lsh->arrowHeadFillColor()));
      wp->drawPolygon(points,3);
    }else{
      wp->drawPolyline(points,3);
    }
    delete [] points;
  }
  if( (lsh->capPosition() & Xfp::ARPEnd)&&
     !(lsh->arrowPosition() & Xfp::ARPEnd)){
    setPen(lsh->lineStyle()->color(),
	   lsh->lineStyle()->style(),
	   lsh->lineStyle()->width(),Xfp::LCSRound);
    wp->drawLine(0.5*(ss+ee),ee);
  }
  if( (lsh->capPosition() & Xfp::ARPStart)&&
     !(lsh->arrowPosition() & Xfp::ARPStart)){
    setPen(lsh->lineStyle()->color(),
	   lsh->lineStyle()->style(),
	   lsh->lineStyle()->width(),Xfp::LCSRound);
    wp->drawLine(0.5*(ss+ee),ss);
  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawEllipse(const EllipseShape *esh)
{
  //myDebug()<<"ere";
  double aadx,aady;
  prepAAOddset(aadx,aady);

  save();

  setPen(esh->lineStyle()->color(),
	 esh->lineStyle()->style(),
	 esh->lineStyle()->width());

  Size r=esh->radiiToDC(pdrag)*Size(pwidth,pheight);
  if(esh->isCircle())r.h=r.w;
  
  Point c=esh->centerToDC(pdrag);
  wp->translate(c.x*pwidth+aadx,c.y*pheight+aady);
  
  wp->rotate(esh->angle());
  if(!pdrag){
    if(esh->fill()->fill()){
      wp->setBrush(QColor(esh->fillColor()));
    }else{
      wp->setBrush(Qt::NoBrush);
    }
    wp->drawEllipse(QPointF(),r.w,r.h);
  }else{
    QRgb c(0xff0000);
    setPen(c,Xfp::LSTSolid,1);
    wp->setBrush(Qt::NoBrush);
    wp->drawEllipse(QPointF(),r.w,r.h);
    wp->drawRect(-r.w,-r.h,2*r.w,2*r.h);
    
    save();
    double width=0.005*pwidth;
    wp->translate(-r.w,-r.h);
    wp->fillRect(-width,-width,2*width,2*width,c);
    wp->translate(2*r.w,0);
    wp->fillRect(-width,-width,2*width,2*width,c);
    wp->translate(0,2*r.h);
    wp->fillRect(-width,-width,2*width,2*width,c);
    wp->translate(-2*r.w,0);
    wp->fillRect(-width,-width,2*width,2*width,c);
    restore();

  }
  restore();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawPolyLineRel(const ValueArray<Point> &points)
{
  if(!points.size()) return;

  double aadx,aady;
  prepAAOddset(aadx,aady);

  QVector<QPointF> pts;
  for(long i=0;i< points.size() ;i++){
    pts.append(QPointF(points.at(i).x*pwidth+aadx,points.at(i).y*pheight+aady));
  }
  //todo: set mitter temporarily
  wp->drawPolyline(pts);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawPolyLineRel(const ValueArray<Point> &points,
				    const ValueArray<double> &radii,
				    const ValueArray<bool> &plotsymbols,
				    QRgb fc,double rmin,double rmax,
				    const SymbolStyle &ss,
				    Xfp::WeightsStyle ws)
{
  if(!points.size()) return;

  save();
  setPen(fc,Xfp::LSTNone,1);
  wp->setBrush(QColor(fc));
  double aadx,aady;
  prepAAOddset(aadx,aady);

  QPointF pts[4];
  for(long i=0;i<points.size()-1;i++){
    //printf("i=%ld N=%ld\n",i,points.size());
    double r0=radii.at(i);
//     r0*=Xfp::pointsToInch*Xfp::inchToCM*10/pwidthMM*pwidth;
//     r0*=pscreenratio;
    r0*=plinewidthratio*palloverPtscale;
    r0*=rmax;
    if(r0<0.0)r0=0.0;
    double x0=points.at(i).x*pwidth;
    double y0=points.at(i).y*pheight;
    double r1=radii.at(i+1);
//     r1*=Xfp::pointsToInch*Xfp::inchToCM*10/pwidthMM*pwidth;
//     r1*=pscreenratio;
    r1*=plinewidthratio*palloverPtscale;
    r1*=rmax;
    if(r1<0.0)r1=0.0;
    if(r0<rmin && r1<rmin) continue;
    double x1=points.at(i+1).x*pwidth;
    double y1=points.at(i+1).y*pheight;
    double d10x=x1-x0;
    double d10y=y1-y0;
    double d10sq=d10x*d10x+d10y*d10y;
    double d10=sqrt(d10sq);
    //todo: if d is very small
    //printf("r0/r1/c=%g %g %g\n",r0,r1,c);

//todo   max of defineValue(SymbolSize,(double)2.*w,Absolute);

    if(ws==Xfp::WESConnected){
      pts[0].rx()=x0-d10y/d10*r0+aadx;
      pts[0].ry()=y0+d10x/d10*r0+aady;
      pts[3].rx()=x0+d10y/d10*r0+aadx;
      pts[3].ry()=y0-d10x/d10*r0+aady;
      
      
      pts[1].rx()=x1-d10y/d10*r1+aadx;
      pts[1].ry()=y1+d10x/d10*r1+aady;
      pts[2].rx()=x1+d10y/d10*r1+aadx;
      pts[2].ry()=y1-d10x/d10*r1+aady;
      
      wp->drawPolygon(pts,4);
      wp->drawChord(QRectF(x0-r0+aadx,y0-r0+aady,2*r0,2*r0),0,5760);
      if(i==points.size()-2){
	wp->drawChord(QRectF(x1-r1+aadx,y1-r1+aady,2*r1,2*r1),0,5760);
      }
    }else if(ws==Xfp::WESDots){
      if(plotsymbols.at(i)){
	wp->drawChord(QRectF(x0-r0+aadx,y0-r0+aady,2*r0,2*r0),0,5760);
      }
      if((i==points.size()-2) && plotsymbols.at(i+1)){
	wp->drawChord(QRectF(x1-r1+aadx,y1-r1+aady,2*r1,2*r1),0,5760);
      }
    }else if(ws==Xfp::WESIndividual){
      if(( ss.type()!=Xfp::STYNone)){
	if(plotsymbols.at(i)){
	  double lww=r0*Xfp::symbolBaseScale(ss.type());
	  //todo: defineValue(SymbolSize,(double)2.*w,Absolute);
	  
	  drawSymbol(x0,y0,ss.type(),lww,ss,ss.fillStyle().color(),
		     ss.fillStyle().color());
	}
	if((i==points.size()-2) && plotsymbols.at(i+1)){
	  double lww=r1*Xfp::symbolBaseScale(ss.type());
	  drawSymbol(x1,y1,ss.type(),lww,ss,ss.fillStyle().color(),
		     ss.fillStyle().color());
	}
      }

    }
  }
  //printf("done\n");

  restore();
}

/* ***********************************************************************
 * setup symbol shapes prepare as much as possible
 * ***********************************************************************
 */
void QtPaintEngine::drawSymbolSetup(const SymbolStyle &symbol,double altsize
				    ,double &lww)
{
  //NSF("drawQTSymbolSetup");
  double w=symbol.size();
  if(altsize > 0.){
    w=altsize;
  }

  // ****** make size in font units ***********
  //  w*=Xfp::pointsToInch*Xfp::inchToCM*10/pwidthMM*pwidth;
  // scale to physical screen size
  //  w*=pscreenratio;
  w*=plinewidthratio*palloverPtscale;
  // symbol base width in formulas below is 2*w
  w*=0.5;
  
  
  lww=w*Xfp::symbolBaseScale(symbol.type());
  
  
  defineValue(SymbolSize,(double)2.*w,Absolute);
  
  
}

/* ***********************************************************************
 * simple drawing primitives for the symbol ploting, need setup before
 * ***********************************************************************
 */
void QtPaintEngine::
drawSymbol(double ix,double iy,Xfp::SymbolTypes type
	   ,double lww,
	   const SymbolStyle &ss,
	   QRgb symlincol,QRgb symfilcol)
{
  //NSF("drawQTSymbol");
  wp->save();
  
  setPen(symlincol
	 ,ss.lineStyle().style()
	 ,ss.lineStyle().width());
  
  QRgb color=symfilcol;
  bool  fill=ss.fillStyle().fill();
  
  QColor c(color);

  double aadx,aady;
  prepAAOddset(aadx,aady);
  wp->translate(ix+aadx,iy+aady);

  if(!fill)wp->setBrush(Qt::NoBrush);
  switch(type){
  case Xfp::STYSquare:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      points.append(QPointF(-lww,-lww));
      points.append(QPointF(lww,-lww));
      points.append(QPointF(lww,lww));
      points.append(QPointF(-lww,lww));
      wp->drawPolygon(points) ;
      break;
    }
  case Xfp::STYCircle:
    {
      if(fill){
	wp->setBrush(c);
	wp->drawChord(QRectF(-lww,-lww,2*lww,2*lww),0,5760 );
      }else{
	wp->drawArc(QRectF(-lww,-lww,2*lww,2*lww),0,5760);
      }
      break;
    }
  case Xfp::STYDiamond:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      points.append(QPointF(lww,0));
      points.append(QPointF(0,lww));
      points.append(QPointF(-lww,0));
      points.append(QPointF(0,-lww));
      wp->drawPolygon (points ) ;
      break;
    }
  case Xfp::STYTriangleUp:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      double l3=lww*0.57735026919;
      points.append(QPointF(-lww,l3));
      points.append(QPointF(lww,l3));
      points.append(QPointF(0,-2*l3));
      wp->drawPolygon (points) ;
      break;
    }
  case Xfp::STYTriangleLeft:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      double l3=lww*0.57735026919;
      points.append(QPointF(l3,-lww));
      points.append(QPointF(l3,lww));
      points.append(QPointF(-2*l3,0));
      wp->drawPolygon (points ) ;
      break;
    }
  case Xfp::STYTriangleDown:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      double l3=lww*0.57735026919;
      points.append(QPointF(-lww,-l3));
      points.append(QPointF(lww,-l3));
      points.append(QPointF(0,2*l3));
      wp->drawPolygon (points ) ;
      break;
    }
  case Xfp::STYTriangleRight:
    {
      if(fill)wp->setBrush(c);
      QVector<QPointF> points;
      double l3=lww*0.57735026919;
      points.append(QPointF(-l3,-lww));
      points.append(QPointF(-l3,lww));
      points.append(QPointF(2*l3,0));
      wp->drawPolygon (points ) ;
      break;
    }
  case Xfp::STYPlus:
    {
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      wp->drawLine(QLineF(0,-lww,0,lww) ) ;
      break;
    }
  case Xfp::STYX: 
    {
      wp->rotate(45);	  
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      wp->drawLine(QLineF(0,-lww,0,lww) ) ;
      break;
    }
  case Xfp::STYStar: 
    {
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      wp->rotate(45);
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      wp->rotate(90);
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      wp->rotate(135);
      wp->drawLine(QLineF(-lww,0,lww,0) ) ;
      break;
    }
  case Xfp::STYNone:
    break;
  case Xfp::STYChar:
    printf("symbol char not yet implmented in qt!\n");
    exit(1);
    break;
  }
  wp->restore();
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawSymbolFromValue(Variable fac1,double val1,
					double val2,ValueType relarg2,
					const FunctionAttribs *at,double altsize)
{
  double lww,ix,iy;
  
  if( at->symbolStyle().type()==Xfp::STYNone) return;

  drawSymbolSetup(at->symbolStyle(),altsize,lww);
  
  ix=(argvalue(fac1,val1));
  iy=(argvalue(val2,relarg2));
  
  drawSymbol(ix,iy,at->symbolStyle().type(),lww,at->symbolStyle(),
	     at->symbolLineColor(),at->symbolFillColor());
  

}

/* ***********************************************************************
 * draw symbols for a set of points
 * ***********************************************************************
 */
void QtPaintEngine::drawPolySymbolRel(const ValueArray<Point> &points
				      ,const FunctionAttribs *at)
{
  //NSF("drawQTPolySymbolRel");
  if(!points.size()) return;

  double lww,ix,iy;

  if( at->symbolStyle().type()==Xfp::STYNone) return;
  
  drawSymbolSetup(at->symbolStyle(),-1.,lww);

  const SymbolStyle &symbol=at->symbolStyle();

  for(long i=0;i< points.size();i++){

    ix=(points.at(i).x*(pwidth));
    iy=(points.at(i).y*(pheight));

    drawSymbol(ix,iy,symbol.type(),lww,at->symbolStyle(),
	       at->symbolLineColor(),at->symbolFillColor());

  }

}

/* ***********************************************************************
 * draw encaps func
 * ***********************************************************************
 */

void QtPaintEngine::setPen(QRgb color,Xfp::LineStyles style,double width
			   ,Xfp::LineCapStyle capstyle)
{

  Qt::PenStyle s[]={Qt::NoPen,Qt::SolidLine,Qt::DashLine,Qt::DotLine
		    ,Qt::DashDotLine,Qt::DashDotDotLine};
  double linewidth=width*plinewidthratio*palloverPtscale
    *palloverlinewidthPtscale;
  double nlinewidth=linewidth;
  if(!pantialiasing) nlinewidth=dRound(nlinewidth);

 
  QPen pen(QColor(color),nlinewidth,s[style],
	   qtcapstyles[capstyle],Qt::MiterJoin);
  if(style!= Xfp::LSTSolid){
    //The dash pattern is specified in units of the pens width
    double factor=linewidth/MyMath::max<double>(1,nlinewidth);
    const long *ld=Xfp::lineStyleDashes(style);
    QVector<qreal> dashes;
    for(int i=0;i<ld[0];i++){
      dashes << ld[i+1]*factor;
    }
    pen.setDashPattern(dashes);
  }

  wp->setPen(pen);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::setClippingRectFromValue(double val1 ,ValueType relarg1,
					     double val2 ,ValueType relarg2,
					     double val3 ,ValueType relarg3,
					     double val4 ,ValueType relarg4,
					     RectType recttype)
{
//   double aadx,aady;
//   prepAAOddset(aadx,aady);

  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);
  double x1=argvalue(val3,relarg3);
  double y1=argvalue(val4,relarg4);
  if(recttype==ULWH){
    x1+=x0;
    y1+=y0;
  }


  double w=x1-x0;
  double h=y1-y0;
  
  wp->setClipRect(QRectF(x0+1,y0+1,w,h));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::setClipping(bool c)
{
  wp->setClipping(c);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::textInit()
{
  ptextascent=0;
  ptextdescent=0;
  defineValue(TextWidth,0.,Absolute);
  defineValue(TextHeight,0.,Absolute);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::textClean()
{ 
  defineValue(TextHeight,ptextascent+ptextdescent+1,Absolute);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::fontSelection(long font,double fontsize)
{
  NSF("fontSelection");
  double fsz=fontsize*pscreenratio*palloverPtscale;
  QString s=QString("%1:%2").arg(font).arg(fsz);
  
  if(!pfonts.contains(s)){
    QFont *fnt=new QFont(qt_fonts[font].name,fsz,QFont::Normal,
			 qt_fonts[font].italic);
    fnt->setPointSizeF(fsz); 
    wp->setFont(*fnt); 
    pfonts[s]=fnt;
  }else{
    wp->setFont(*(pfonts[s])); 
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::metricText(const char *text,long font,double fontsize)
{
  //NSF("metricQTText");
  
  fontSelection(font,fontsize);
  
  double w=wp->fontMetrics().width(text);
  
#ifdef TEXTBOX_DEBUG_SHOW
  wp->save();
  setPen(0xff00ff,Xfp::LSTSolid,1);
  wp->drawText(pcurrentpoint,QString(text)); 
  wp->restore();
#endif
  
  QRectF r=wp->fontMetrics().boundingRect(text);
  
#ifdef TEXTBOX_DEBUG_TEXTBOX
  wp->save();
  setPen(0xff,Xfp::LSTSolid,1);
  wp->drawRect( QRectF(r.topLeft()+pcurrentpoint,r.size()));
  wp->restore();
#endif
  
  double a=wp->fontMetrics().ascent();
  double d=wp->fontMetrics().descent();
  a=MyMath::max<double>(-r.top(),a);
  d=MyMath::max<double>(r.bottom(),d);

  
  arithValue(TextWidth,TextWidth,1.,(double)w ,Absolute ,Plus);

  pcurrentpoint+=QPointF(w,0);
  ptextascent=MyMath::max<double>(ptextascent,a-pcurrentpoint.y()+1);
  
  ptextdescent=MyMath::max<double>(ptextdescent,d+pcurrentpoint.y());
  
#ifdef TEXTBOX_DEBUG_BOXES
  {
    wp->save();
    setQTPen(device,PSRGBColors[PS_RED],PS_SOLIDLINE,1);
    double w=getQTValue(wp,"FploTextWidth",Absolute);
    QRectF r(0 ,wp->textdescent ,w
	    ,-(wp->textascent+wp->textdescent+1));
    wp->drawRect(r);
    wp->restore();
  }
#endif



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::adjustText(Xfp::TextOrientation just)
{
  //NSF("adjustQTText");

  QPointF p(0,0);
  double w=getValue(TextWidth,Absolute);
  if(just & Xfp::TOCenter){
    p.setX((-w*0.5));
  }else if(just & Xfp::TORight){
    p.setX((-w));
  }
  if(just & Xfp::TOTop){
    p.setY(ptextascent);
  }
  if(just & Xfp::TOBottom){
    p.setY(-ptextdescent);
  }
  if(just & Xfp::TOVCenter){
    p.setY((ptextascent-ptextdescent)*0.5);
  }
  
 
  wp->translate(p);
  moveToInt(0,0);
  


#ifdef TEXTBOX_DEBUG  
  wp->save();
  setPen(0x00ff00,Xfp::LSTSolid,1);
  QRectF r(0,ptextdescent,w,-(ptextascent+ptextdescent+1));
  wp->drawRect( r);
  wp->restore();
#endif
  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::flushMove(double x,double y,long font,double fontsize)
{

  fontSelection(font,fontsize);


  double a=wp->fontMetrics().ascent();
  double cw=wp->fontMetrics().width("X");;

  double lx=x*cw;
  double ly=y*a;
  
  

  if(ly==0 && x==0){//TODO: make another function which shifts the baseline
    QPointF p(lx,ly);
    p+=pcurrentpos;
    p.setY(0);
    moveToInt(p);
  }else if(ly==0){
    //arithValue(TextWidth,TextWidth,1.,(double)lx ,Absolute ,Plus);
    QPointF p(lx,ly);
    p+=pcurrentpos;
    p.setY(0);
    moveToInt(p);
  }else{
    moveToInt(QPointF(lx,ly) + pcurrentpos );
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::metricMove(double x,double y,long font,double fontsize)
{

  fontSelection(font,fontsize);


  double a=wp->fontMetrics().ascent();
  double cw=wp->fontMetrics().width("X");;

  double lx=x*cw;
  long ly=y*a;


  if(ly==0 && x==0){//TODO: make another function which shifts the baseline
    pcurrentpoint+=QPointF(lx,ly);
    pcurrentpoint.setY(0);
  }else if(ly==0){
    arithValue(TextWidth,TextWidth,1.,(double)lx ,Absolute ,Plus);
    pcurrentpoint+=QPointF(lx,ly);
    pcurrentpoint.setY(0);

  }else{
    pcurrentpoint+=QPointF(lx,ly);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::flushText(const char *text,long font,double fontsize)
{
  
  fontSelection(font,fontsize);
  
  

  double w=wp->fontMetrics().width(text);
  wp->drawText(pcurrentpos,QString(text)); 
  
  moveToInt(QPointF(w,0) + pcurrentpos );

  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawSpringFromValue(double val1,ValueType relarg1,
					double val2,ValueType relarg2,
					double val3,ValueType relarg3,
					double val4,ValueType relarg4,
					RectType recttype)
{
  //NSF("drawQTSpringFromValue");


  double lx1,ly1;
  double lx0=argvalue(val1,relarg1);
  double ly0=argvalue(val2,relarg2);

  if(recttype==ULWH){
    lx1=(argvalue(val3,relarg3)+lx0);
    ly1=(argvalue(val4,relarg4)+ly0);
  }else{
    lx1=(argvalue(val3,relarg3));
    ly1=(argvalue(val4,relarg4));
  }
  double aadx,aady;
  prepAAOddset(aadx,aady);

  double wx=lx1-lx0;
  double wy=ly1-ly0;
  if(wx==0. && wy==0.) return;
  double s=sqrt(wx*wx+wy*wy);
  double w=5;
  double dx=wy/s*w;
  double dy=-wx/s*w;
  long n=lRound(MyMath::max<double>(2,MyMath::max<double>(fabs(wx),fabs(wy))/w));
  lx0+=aadx;
  ly0+=aady;
  double ix0=lx0;
  double iy0=ly0;
  //n=3;
  double ix,iy;
  for(long i=1;i<=n;i++){
    ix=wx*(i-0.5)/(n-1)+lx0;
    iy=wy*(i-0.5)/(n-1)+ly0;
    if(i==1 ){
      wp->drawLine(QLineF( (ix0),(iy0),(ix+dx),(iy+dy)));
    }else if( i==n){
      wp->drawLine(QLineF( (ix0-dx),(iy0-dy),(lx1),(ly1)));
    }else{
      wp->drawLine(QLineF( (ix0-dx),(iy0-dy),(ix+dx),(iy+dy)));
    }
    if(i>0 && i<n ){
      wp->drawLine(QLineF( (ix-dx),(iy-dy),(ix+dx),(iy+dy)));
    }
    ix0=ix;
    iy0=iy;
  }
  //wp->drawLine( (lx0),(ly0),(lx1),(ly1));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PixMap *QtPaintEngine::createPixMap(int width,int height)
{
  return new QtPixMap(new QImage(width,height,QImage::Format_RGB32));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void QtPaintEngine::drawPixMap(PixMap *pixmap,double val1 ,ValueType relarg1,
			       double val2 ,ValueType relarg2,
			       double val3 ,ValueType relarg3,
			       double val4 ,ValueType relarg4,
			       RectType recttype)
{
  if(!pixmap) return;
  double x0=argvalue(val1,relarg1);
  double y0=argvalue(val2,relarg2);
  double x1=argvalue(val3,relarg3);
  double y1=argvalue(val4,relarg4);
  if(recttype==ULWH){
    x1+=x0;
    y1+=y0;
  }
  double aadx,aady;
  prepAAOddset(aadx,aady);

  QImage *im=((QtPixMap*)pixmap)->qtImage();

  //im->fill(Qt::red);
  //myDebug()<<im->rect();

  QRectF target(x0+aadx,y1+aady,(x1-x0),(y0-y1));
  //myDebug()<<target;
  wp->drawImage(target,*im,im->rect());
  
  //  delete pixmap;
}


/*EOF*/ 
