/* @(#)QtPaintEngine.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _QTPAINTENGINE_H
#define _QTPAINTENGINE_H 1

#include <QHash>
#include <QImage>
#include <QPainter>
#include <QPointF>
#include <QFont>
#include <QVector>
#include <QDebug>
#include "PaintEngine.hh" 


class SymbolStyle;


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class QtPixMap:public PixMap{
 public:
  // the user must take care of deletion of the image data object
  QtPixMap():image(0){};
  QtPixMap(QImage *im):image(im){};
  virtual ~QtPixMap(){if(image) delete image;image=0;
    qDebug()<<"deleted QImage";
  };
  virtual void setPixel(int x,int y,QRgb c){image->setPixel(x,y,c);};
  QImage *qtImage(){return image;};
 private:
  QImage *image;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class QtPaintEngine:public PaintEngine{
 public:
  QtPaintEngine(QPaintDevice *,bool antialiasing);
  ~QtPaintEngine();

  virtual void init(const Paper &);

  virtual void defineValue(Variable var,double val,ValueType relarg){
    pvalues[var]=argvalue(val,relarg);};
  virtual void defineValue(Variable var,Variable factor,double val){
    pvalues[var]=argvalue(factor,val);};

  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  Variable fac3 ,double val3,
			  Operation op);
  virtual void arithValue(Variable var,
			  Variable fac2 ,double val2,
			  double val3 ,ValueType relarg3,
			  Operation op);
  virtual double getValue(Variable var,ValueType relarg);

  virtual void save(){pangles.append(pcurrentangle);wp->save();};
  virtual void restore(){pcurrentangle=pangles.pop();
    pcangle=cos(pcurrentangle/180.*Pi);
    psangle=sin(pcurrentangle/180.*Pi);
    wp->restore();};
  
  virtual void translate(double x,ValueType relargx,
			 double y,ValueType relargy){
    wp->translate(argvalue(x,relargx),argvalue(y,relargy));};
  virtual void translate(double x,ValueType relargx,
			 Variable factory,double y){
    wp->translate(argvalue(x,relargx),argvalue(factory,y));};
  virtual void translate(Variable factorx,double x,
			 double y,ValueType relargy){
    wp->translate(argvalue(factorx,x),argvalue(y,relargy));};
  virtual void translate(Variable factorx,double x,
			 Variable factory,double y){
    wp->translate(argvalue(factorx,x),argvalue(factory,y));};
  
  virtual void rotate(double angle){pcurrentangle+=angle;
    pcangle=cos(pcurrentangle/180.*Pi);
    psangle=sin(pcurrentangle/180.*Pi);
    wp->rotate(-angle);};

  virtual void setPen(QRgb,Xfp::LineStyles,double,
		      Xfp::LineCapStyle cs=Xfp::LCSButt);
  virtual void moveTo(double x,ValueType relargx,
 		      double y,ValueType relargy){
    moveToInt(argvalue(x,relargx),argvalue(y,relargy));}

  virtual void drawComment(const String &){};

/*   virtual void drawRectFromValue(Variable fac1,double val1, */
/* 				 Variable fac2,double val2, */
/* 				 Variable fac3,double val3, */
/* 				 Variable fac4,double val4, */
/* 				 QRgb fc,bool fill,RectType recttype); */
  virtual void drawRectFromValue(double val1,ValueType relarg1,
				 double val2,ValueType relarg2,
				 Variable fac3,double val3,
				 Variable fac4,double val4,
				 QRgb fc,bool fill,RectType recttype);
  virtual void drawRectFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 QRgb fc,bool fill,RectType recttype);

  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 double val3 ,ValueType relarg3,
				 double val4 ,ValueType relarg4,
				 RectType recttype);
  virtual void drawLineFromValue(double val1 ,ValueType relarg1,
				 double val2 ,ValueType relarg2,
				 Variable fac3,double val3,
				 double val4 ,ValueType relarg4,
				 RectType recttype);

  virtual void drawSymbolFromValue(Variable fac1,double val1,
				   double val2,ValueType relarg2,
				   const FunctionAttribs *at,double altsize);
  virtual void drawPolyLineRel(const ValueArray<Point> &points);
  virtual void drawPolyLineRel(const ValueArray<Point> &points,
			       const ValueArray<double> &radii,
			       const ValueArray<bool> &plotsymbols,QRgb fc,
			       double min,double max,
			       const SymbolStyle &ss,
			       Xfp::WeightsStyle ws);
  virtual void drawPolySymbolRel(const ValueArray<Point> &points
				 ,const FunctionAttribs *at);

  virtual void drawLineRel(double x0,double y0,double x1,double y1
			   ,RectType recttype);

  virtual void drawArrow(double x0,double y0,double x1,double y1
			 ,const LineShape*);
  virtual void drawEllipse(const EllipseShape *);

  virtual PixMap *createPixMap(int width,int height);
  virtual void drawPixMap(PixMap *,double val1 ,ValueType relarg1,
			  double val2 ,ValueType relarg2,
			  double val3 ,ValueType relarg3,
			  double val4 ,ValueType relarg4,RectType recttype);
  
  virtual void setClippingRectFromValue(double val1 ,ValueType relarg1,
					double val2 ,ValueType relarg2,
					double val3 ,ValueType relarg3,
					double val4 ,ValueType relarg4,
					RectType recttype);  
  virtual void setClipping(bool);

  // text 
  virtual void textInit();
  virtual void textClean();
  virtual void initCalcTextWidth(){pcurrentpoint=QPointF(0,0);moveToInt(0,0);};
  virtual void metricText(const char *text,long font,double fontsize);
  virtual void flushText(const char *text,long font,double fontsize);
  virtual void metricMove(double x,double y,long font,double fontsize);
  virtual void flushMove(double x,double y,long font,double fontsize);
  virtual void adjustText(Xfp::TextOrientation just);


  virtual void setDrag(bool d){pdrag=d;};
  
  //sepcial functions for this engine
  void translate(double x,double y){wp->translate(x,y);};
  void drawSpringFromValue(double val1,ValueType relarg1,
			   double val2,ValueType relarg2,
			   double val3,ValueType relarg3,
			   double val4,ValueType relarg4,
			   RectType recttype);

  virtual Size deviceSize(){return Size(pwidth,pheight);};


 private:
  double argvalue(Variable factor,double val);
  double argvalue(double val,ValueType relarg);
  void fontSelection(long font,double fontsize);
  void moveToInt(const QPointF &p){pcurrentpos=p;};
  void moveToInt(double x,double y){pcurrentpos=QPointF(x,y);};
  void drawSymbolSetup(const SymbolStyle &symbol,double altsize,double &lww);
  void drawSymbol(double ix,double iy,Xfp::SymbolTypes type
		  ,double lww
		  ,const SymbolStyle &ss
		  ,QRgb symlincol,QRgb symfilcol);
  void prepAAOddset(double &aadx,double &aady);

 private:
  QPainter *wp;
  QHash<int,double> pvalues;
  QHash<QString,QFont *>  pfonts;

  long pwidth;
  long pheight;
  int pwidthMM;
  double pscreenratio,plinewidthratio;
  double ptextascent,ptextdescent; /* the metrics of the text (different fonts,
				    sub and super scripts) */
  QPointF pcurrentpoint;
  QPointF pcurrentpos;

  bool pantialiasing;
  ValueArray<double> pangles;
  double pcurrentangle;
  double pcangle,psangle;

  bool pdrag;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _QTPAINTENGINE_H */

