/*************************************************************************
 * @(#)XFPStatusBar.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 May 2011
 *************************************************************************
 */

#include <QLabel>
#include <QTimer>
#include <QProgressBar>
#include <QToolButton>
#include "XFPStatusBar.hh"
#include "ButtonStatusWidget.hh"
#include "Point.hh"
#include "Graph.hh"
#include "FString.hh"
#include "defs.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
XFPStatusBar::XFPStatusBar(QWidget *parent)
  :QStatusBar(parent)
  ,indicator(new QLabel(this))
  ,grindicator(new QLabel(this))
  ,bsw(new ButtonStatusWidget(this))
  ,msglabel(new QLabel(this))
  ,labeltimer(new QTimer(this))
  ,progbar(0),tbInterupt(0)
  ,pdocumentname("")
{
  progbar=new QProgressBar(this);
  progbar->hide();
  
  grindicator->setFrameShadow(QFrame::Sunken);
  grindicator->setFrameShape(QFrame::Panel);


  addPermanentWidget(grindicator,0);
  addPermanentWidget(indicator,0);
  //  indicator->setFrameShape(QFrame::StyledPanel);
  //indicator->setFrameShadow(QFrame::Sunken);
  //indicator->setLineWidth(1);

  
  msglabel->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Preferred);
  addPermanentWidget(msglabel,2);
  if(0){
    msglabel->setFrameShape(QFrame::Panel);
    msglabel->setFrameShadow(QFrame::Sunken);
    msglabel->setLineWidth(1);
  }

  if(1){
    tbInterupt=new QToolButton(this);
    tbInterupt->hide();
    tbInterupt->setFocusPolicy(Qt::NoFocus);
#ifndef QT_NO_TOOLTIP
    tbInterupt->setToolTip("Interupt calculation/printing...");
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_STATUSTIP
    tbInterupt->setStatusTip("Interupt calculation/printing...");
#endif // QT_NO_STATUSTIP
    QIcon icon;
    icon.addPixmap(QPixmap(":/Icons/stop"));
    tbInterupt->setIcon(icon);
    tbInterupt->setText("");
    addPermanentWidget(tbInterupt,0);
  }
  addPermanentWidget(progbar,2);

  addPermanentWidget(bsw,0);

  connect(this,SIGNAL(messageChanged(const QString&))
	  ,this,SLOT(beware(const QString &)));

  connect(labeltimer, SIGNAL(timeout()), this, SLOT(labelTimer()));

  connect(tbInterupt,SIGNAL(clicked()),this,SIGNAL(interruptRequest()));
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
XFPStatusBar::~XFPStatusBar()
{
  if(indicator) delete indicator; indicator=0;
  if(grindicator) delete grindicator; grindicator=0;
  if(bsw) delete bsw; bsw=0;
  if(msglabel) delete msglabel; msglabel=0;
  if(labeltimer) delete labeltimer; labeltimer=0;
  if(progbar) delete progbar; progbar=0;
  if(tbInterupt) delete tbInterupt; tbInterupt=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::beware(const QString &msg)
{
  if(msg.isEmpty()){
    if(msglabel){
      msglabel->setText(pdocumentname);
    }
  }else{
    if(msglabel) msglabel->setText(msg);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::labelTimer()
{
  labeltimer->stop();
  clearMessage();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setDocument(const QString &name)
{
  pdocumentname=name;
  if(msglabel) msglabel->setText(name);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setMessage(const QString &msg,int msec)
{
  NSF("Status::setMessage");
  if(msg.isEmpty()){
    labeltimer->start(1000);
  }else{
    labeltimer->stop();
    showMessage(msg,msec);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setCurrentCoordinates(const Point &c)
{
  String sx,sy;
  sx.sprintf("%30.6g",c.x).stripBlanks();
  sy.sprintf("%30.6g",c.y).stripBlanks();
  indicator->setText(Sprintf("x,y=[ %s , %s ]",sx(),sy())());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setHint(const QString &what,const QString &s)
{
  bsw->setHint(what,s);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setCurrentGraph(Graph *gr)
{
  if(gr){
    grindicator->setText(gr->guiName(true)());
  }else{
    grindicator->setText("");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void XFPStatusBar::setProgress(double percent)
{
  NSF("XFPStatusBar::setProgress");
  if(percent<=0.0){
    progbar->setValue(0); 
    progbar->hide();
    tbInterupt->hide();
    return;
  }else{

    progbar->show();
    tbInterupt->show();


    progbar->setValue(MyMath::max<double>(0,MyMath::min<double>(100,dRound(100*percent))));
    
  }
  
}

/*EOF*/ 
