/*************************************************************************
 * @(#)MainIoTest.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Apr 2011
 *************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include "FString.hh"
#include "Except.hh" 
#include "Parser.hh" 


/* ***********************************************************************
 *
 * ***********************************************************************
 */
int main( int argc, char ** argv )
{
  StringList args;
  for(int i=0;i<argc;i++){
    args.append(argv[i]);
  }
  String mode;
  String fname;
  String value;
  String var;
  for(int index=0;index<argc;index++){
    if(args[index]=="-f"){
      if(index+1<args.size()){
	fname=args[index+1];
      }else{
	printf("option -f needs a file name\n");
	exit(1);
      }
    }else if(args[index]=="-value"){
      if(index+1<args.size()){
	value=args[index+1];
      }else{
	printf("option -value needs an argument\n");
	exit(1);
      }
    }else if(args[index]=="-mode"){
      if(index+1<args.size()){
	mode=args[index+1];
      }else{
	printf("option -mode needs an argument\n");
	exit(1);
      }
    }else if(args[index]=="-var"){
      if(index+1<args.size()){
	var=args[index+1];
      }else{
	printf("option -var needs an argument\n");
	exit(1);
      }
    }
  }
  if(fname.empty()){
    printf("need option -f file_name\n");
    exit(1);
  }
  if(mode.empty()){
    printf("need option -mode mode\n");
    exit(1);
  }
  
  if(!(mode=="SET_SCALAR" || mode=="GET_SCALAR")){
    printf("allowed modes: SET_SCALAR GET_SCALAR\n");
    exit(1);
  }
  
  if(mode=="SET_SCALAR"){
    if(var.empty()){
      printf("need option -var var_name\n");
      exit(1);
    }else{
      if(value.empty()){
	printf("need option -value value\n");
	exit(1);
      }
    }
  }
  if(mode=="GET_SCALAR"){
    if(var.empty()){
      printf("need option -var var_name\n");
      exit(1);
    }    
  }
  //printf("file \"%s\"\n",fname());
  //printf("value \"%s\"\n",value());
  //printf("var \"%s\"\n",var());
  
  Parser par(1);
  try{
    par.parseFile(fname());
  }catch(Except e){
    printf("%s",Sprintf("Cannot read file \"%s\"\n(%s)\n"
			,fname(),e.msg())());
    exit(1);
  }

  StringList sl=var.split(".");


  try{
    Dobj d=par();
    for(int i=0;i<sl.size();i++){
      StringList sa=sl[i].breakup("[");
      if(sa.size()==1){
	d=d(sl[i]);
      }else if(sa.size()==2){
	d=d(sa[0]);

	StringList sa2=sa[1].breakup("]");
	if(sa2.size()!=2){
	  throw Except("array element syntax invalid");
	}
	int index=sa2[0].convertToLong();
	if(index>d.size()-1){
	  throw Except("[struct]array index out of bound");
	}
	d=d[index];
	
      }else{
	throw Except("array element syntax invalid");
      }
    }
    if(mode=="SET_SCALAR"){
      long l;
      if(d.isInt()){
	value.convertToLong();
      }else if(d.isReal()){
	value.convertToFortranReal();
      }else if(d.isReal()){
      }else if(d.isLogical()){
	if(value.toLower()=="t"||value.toLower()=="f"){
	}else if(value.toLower()=="+"){
	}else if(value.toLower()=="-"){
	}else if(String::islong(&l,value())){
	  if(!l)
	    value="f";
	  else
	    value="t";
	}else{
	  throw Except("value is not a logical");
	}
      }else if(d.isChar()){
	if(value.length()>1){
	  throw Except("value is not a character");
	}
      }else if(d.isString()){
	if(!d.isVariableString()){
	  if(value.length()>d.stringLength()){
	    throw Except("value is too long");
	  }
	}
      }
      d=value;
    }else if(mode=="GET_SCALAR"){
      if(d.isChar()){
	printf("%s='%s'",d.fullName()(),d()());
      }else if(d.isString()){
	printf("%s=\"%s\"",d.fullName()(),d()());
      }else{
	printf("%s=%s",d.fullName()(),d()());
      }
    }
  }catch(Except e){
    printf("%s\n",e.msg());
    exit(1);
  };
  if(mode=="SET_SCALAR"){
    
    try{
      par.writeFile(stdout);
    }catch(Except e){
      printf("%s\n",e.msg());
      exit(1);
    };
  }  
  par.cleanup();

  exit(0);
  
}


/*EOF*/ 
