/*************************************************************************
 * @(#)Array.cpp
 * author: magru@bono
 * date:   18 Jun 2013
 *************************************************************************
 */

#include "Array.hh"
#include "Array_cpp.hh"
#include "Except.hh"


namespace XFPLO{


/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <>
void BaseArray<Cmplx>::printValue(const Cmplx& c)const
{
  printf("(%12.5g,%12.5g)",c.real(),c.imag());
}

template <>
void BaseArray<double>::printValue(const double &d)const
{
  printf("%12.5g",d);
}

template <>
void BaseArray<long>::printValue(const long &l)const
{
  printf("%12ld",l);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
PathArray::PathArray(long size)
  :BaseArray<long>(ValueArray<long>()<<2)
{
  BaseArray<long>::pclassname="PathArray";
  //printf("%s lives %p\n",pclassname(),this);
  setSize(size);
  for(long i=-1;++i<size;){
    pdata[i*2+0]=i;
    pdata[i*2+1]=i+1;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PathArray::~PathArray()
{
  //printf("%s dies %p\n",pclassname(),this);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshArray::MeshArray()
  :BaseArray<long>()
{
  pclassname="MeshArray";
  //printf("%s lives %p\n",pclassname(),this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshArray::~MeshArray()
{
  //printf("%s dies %p\n",pclassname(),this);
  for(int i=-1;++i<arrays.size();){
    destroy(const_cast<PathArray*>(arrays[i]));
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshArray::addArray(const PathArray *ba,bool take_over)
{
  if(ba->rank()!=1)
    throw Except("MeshArray::addArray(): argument rank!=1");
  
  if(ba->shape(0)!=2){
    throw Except("MeshArray::addArray(): all arrays need to have shape 2.");
  }

  if(arrays.size()==0){
    setShape(ValueArray<long>()<<ba->shape(0));
  }


  if(arrays.size()>2)
    throw Except("MeshArray::addArray(): not more than 3 arrays allowed.");
  
  if(!take_over) const_cast<PathArray*>(ba)->increaseUse();

  //myDebug()<<"ba->size()="<<ba->size();
  if(arrays.size()==0){
    pnitems=ba->size();
    pdata=ba->valueArray();
  }else{
    long oldsize=pnitems;
    long oldshape=pshape[0];
    pshape[0]*=2;
    long newshape=pshape[0];
    pdatasize*=2;
    setSize(oldsize*ba->size());
    long *d=pdata.dataRef();
    long Nx=d[oldsize*oldshape-1]+1;
    //myDebug()<<"Nx="<<Nx;
    for(long i=oldsize;--i>-1;){
      for(long c=-1;++c<oldshape;){
	d[i*newshape+c]=d[i*oldshape+c];
	d[i*newshape+c+oldshape]=d[i*newshape+c]+Nx;
      }
    }

    
    for(long j=0;++j<ba->size();){
      for(long i=-1;++i<oldsize;){
	for(long c=-1;++c<oldshape;){
	  d[(i+j*oldsize)*newshape+c]=d[(i+(j-1)*oldsize)*newshape+c]+Nx;
	  d[(i+j*oldsize)*newshape+c+oldshape]=d[(i+(j-1)*oldsize)*newshape+c]+2*Nx;
	}
      }
    }
    
    

  }

  arrays<<ba;



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshArray::debug()const
{

  BaseArray<long>::debug();

  printf("dimension=%ld Consists off:\n",arrays.size());

  for(int i=-1;++i<arrays.size();){
    arrays.at(i)->debug();
  }

  

}

}

/*EOF*/ 
