/* @(#)Array_cpp.hh
 * author: magru@unas
 */


/* ***********************************************************************
 * implementation of template 
 * ***********************************************************************
 */

#include "Except.hh"
#include "MyDebug.hh"

namespace XFPLO{

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
BaseArray<T>::BaseArray()
  :pclassname("BaseArray"),ptype(XFPLO::TypeInfo<T>::type),prank(0),
   pnitems(0),pusecounter(1),pdatasize(0)
{
  //printf("%s lives %p used=%ld\n",pclassname(),this,used());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
BaseArray<T>::BaseArray(const ValueArray<long> &shape)
  :pclassname("BaseArray"),ptype(XFPLO::TypeInfo<T>::type),prank(0),
   pnitems(0),pusecounter(1)
{
  //printf("%s lives %p used=%ld\n",pclassname(),this,used());
  setShape(shape);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
BaseArray<T>::~BaseArray()
{
  //printf("BaseArray dies %p\n",this);
  cleanData();
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
void BaseArray<T>::debug()const
{
  printf("%s: type=%s rank=%ld shape=[",pclassname(),XFPLO::TypeInfo<T>::name,prank);
  for(long i=-1;++i<prank;){
    printf("%ld",pshape.at(i));
    if(i<prank-1) printf(" , ");
  }
  printf("] items=%ld\n",pnitems);
  printf("data=[");
  long d0=0,d1=0;
  if(prank>0)d0=pshape.at(0);
  if(prank>1)d1=pshape.at(1);
  long ind=0;
  long nskip=100;
  long nn=0;
  for(long i=-1;++i<pnitems;){
    if(i>=nskip && (i<pnitems-3)){
      if(i==nskip) printf("\n\t...\n");
      ind+=pdatasize;
      continue;
    }
    switch(prank)
      {
      case 0:
	printValue(pdata.at(ind++));
	nn++;
	if(i<pnitems-1) printf(" , ");
	break;
      case 1:
	printf("\t[");
	for(long j=-1;++j<d0;){
	  printValue(pdata.at(ind++));
	  if(j<d0-1) printf(" , ");
	}
	if(i<pnitems-1) printf("],");
	else printf("]");
 	nn+=d0;
	break;
      case 2:
	printf("\n\t[");
	for(long j=-1;++j<d0;){
	  if(j==0)
	    printf("[");
	  else
	    printf("\t [");
	  for(long k=-1;++k<d1;){
	    printValue(pdata.at(ind++));
	    if(k<d1-1) printf(" , ");
	  }
	  if(j<d0-1) printf("],\n");
	  else printf("]");
	}
	if(i<pnitems-1) printf("],");
	else printf("]");
      }
    if(((prank>0 && nn>0) || nn>5) && i!=pnitems-1){
      printf("\n");
      nn=0;
    }
  }
  printf("]\n");
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
RegularArray<T>::RegularArray(long shape,
			      const ValueArray<T> &origin,
			      const ValueArray<T> &delta,
			      long size)
  :BaseArray<T>(ValueArray<long>()<<shape)
{
  BaseArray<T>::pclassname="RegularArray";
  //  printf("%s lives %p\n",BaseArray<T>::pclassname(),this);
  
  if(delta.size()<shape){
    throw Except("RegularArray: size of deltas< size of shapes");
  }

  if(origin.size()<shape){
    throw Except("RegularArray: size of origin< size of shapes");
  }
  
  porigin=origin;
  pdelta=delta;
  BaseArray<T>::setSize(size);
  if(size>0){
    // inline T point(long index,long comp)const{
    //    return porigin.at(comp)+pdelta.at(comp)*index;}
    T *d,*d1;
    for(long c=-1;++c<shape;){
      d=BaseArray<T>::pdata.dataRef()+c;
      *d=porigin.at(c);
      const T &v=pdelta.at(c);
      for(long i=0;++i<size;){
	d1=d+shape;
	*d1=(*d)+v;
	d=d1;
      }
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
RegularArray<T>::~RegularArray()
{
  //  printf("%s dies %p\n",BaseArray<T>::pclassname(),this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
void RegularArray<T>::debug()const
{
  BaseArray<T>::debug();
  long d0=BaseArray<T>::pshape.at(0);
  
  printf("origin= [");
  for(long i=-1;++i<d0;){
    BaseArray<T>::printValue(porigin.at(i));
    if(i<d0-1) printf(" , ");
  }
  printf("]\n");

  printf("delta=  [");
   for(long i=-1;++i<d0;){
     BaseArray<T>::printValue(pdelta.at(i));
     if(i<d0-1) printf(" , ");
   }
   printf("]\n");

  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
ProductArray<T>::ProductArray()
  :BaseArray<T>()
{
  BaseArray<T>::pclassname="ProductArray";
  //  printf("%s lives %p\n",BaseArray<T>::pclassname(),this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
ProductArray<T>::~ProductArray()
{
  //  printf("%s dies %p\n",BaseArray<T>::pclassname(),this);

  for(int i=-1;++i<arrays.size();){
    BaseArray<T>::destroy(const_cast<BaseArray<T>*>(arrays[i]) );
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
void ProductArray<T>::addArray(const BaseArray<T> *ba,bool take_over)
{
  if(ba->rank()!=1)
    throw Except("ProductArray::addArray(): argument rank!=1");

  if(arrays.size()==0){
    BaseArray<T>::setShape(ValueArray<long>()<<ba->shape(0));
  }
  long shape=BaseArray<T>::shape(0);

    
  if(arrays.size()>2)
    throw Except("ProductArray::addArray(): not more than 3 arrays allowed.");
  if(arrays.size()>0 && (shape != ba->shape(0)))
    throw Except("ProductArray::addArray(): all arrays need to have the same shape.");
  

  // myDebug()<<"arrays.size()="<<arrays.size();
  // myDebug()<<"ba->shape(0)="<<ba->shape(0);
  // myDebug()<<"type="<<dynamic_cast<MeshArray*>(this);
  if((arrays.size()>0) && arrays.size()>shape-1)
    throw Except("ProductArray::addArray(): cannot add "
		 "more arrays than embedding dimensions.");

  // myDebug()<<"rank="<<BaseArray<T>::prank;
  // myDebug()<<"shape="<<BaseArray<T>::pshape;

  if(!take_over) const_cast<BaseArray<T>*>(ba)->increaseUse();
  if(arrays.size()==0){
    BaseArray<T>::pnitems=ba->size();
    BaseArray<T>::pdata=ba->valueArray();
  }else{
    long oldsize=BaseArray<T>::pnitems;
    BaseArray<T>::pnitems*=ba->size();
    BaseArray<T>::pdata.resize(BaseArray<T>::pnitems*shape);
    const T *db=ba->data();
    T* d=BaseArray<T>::pdata.dataRef();
    for(long i=0;++i<ba->size();){
      for(long j=-1;++j<oldsize;){
	for(long c=-1;++c<shape;){
	  d[(i*oldsize+j)*shape+c]=d[j*shape+c]+db[i*shape+c];
	}
      }
    }
    for(long j=-1;++j<oldsize;){
      for(long c=-1;++c<shape;){
	d[j*shape+c]=d[j*shape+c]+db[c];
      }
    }
    
  }
  
  arrays<<ba;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class T>
void ProductArray<T>::debug()const
{

  BaseArray<T>::debug();

  printf("dimension=%ld Consists off:\n",arrays.size());

  for(int i=-1;++i<arrays.size();){
    arrays.at(i)->debug();
  }

  

}



}
