/* @(#)Atom.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _CLUSTERATOM_H
#define _CLUSTERATOM_H 1

#include "Symmetry.hh"
#include "PropertyTable.hh"
#include "Xf.hh"

namespace XFPLO{
 
  class Atom
  {
  public:
    Atom(){}
    Atom(const Atom &o){ *this=o; };
    
    Atom(long ind,const Site &s,long sit)
      : el(s.el()),index(ind),type(s.type()),sort(s.sort()),site(sit)
      ,tauD(s.tauD()),property(s.property())
      ,hidden(false),visible(s.visible())
      ,lengthunit(s.lengthUnit()){property.atomicRadii().setNN(s.nndist()*0.5);};
    
    Atom(long ind,const Site &s,long sit,const Vec3 t)
      : el(s.el()),index(ind),type(s.type()),sort(s.sort()),site(sit)
      ,tauD(t),property(s.property()),hidden(false)
      ,visible(s.visible())
      ,lengthunit(s.lengthUnit()){
      property.atomicRadii().setNN(s.nndist()*0.5);};
    
    
    
    const Atom &operator=(const Atom &o){
      el=o.el; index=o.index; type=o.type; sort=o.sort; site=o.site;
      tauD=o.tauD;property=o.property;
      hidden=o.hidden;
      visible=o.visible;
      lengthunit=o.lengthunit;
      return *this; }
    
    bool operator==(const Atom &o){
      if( site != o.site) return false;
      return (tauD.eq(o.tauD,1.0e-4));
    }
    
    inline double atomicRadius() const{
      return property.atomicRadii().current(lengthunit);
    }
    
    inline double NN() const{
      return property.atomicRadii().NN();
    }
    
    
    void print() const;
    String el;
    long index;
    long type; // user defined type (nuclear charge by default)
    long sort; // zero based
    long site; // zero based
    Vec3 tauD;
    AtomProperty property;
    bool hidden;
    bool visible;
    Xf::LengthUnits   lengthunit;
    
  };
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  inline MyDebug operator<<(MyDebug dbg, const Atom &at)
  {
    dbg << "el="<<at.el
	<<"index="<<at.index
	<<"type="<<at.type
	<<"sort="<<at.sort
	<<"site="<<at.site
	<<"tau="<<at.tauD
	<<"hidden="<<at.hidden
	<<"visible"<<at.visible;
    return dbg;
  }
}
#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _CLUSTERATOM_H */

