/*************************************************************************
 * @(#)AtomFindMarker.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 Feb 2011
 *************************************************************************
 */

#include "AtomFindMarker.hh"
#include "Cluster.hh"
#include "MyDebug.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomFindMarker::AtomFindMarker()
  :QObject()
{
  connect(&pmarkproperties,SIGNAL(propertiesChanged())
	  ,this,SLOT(onPropertiesChanged()));
}

 
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AtomFindMarker::mark(Cluster *cluster,Xf::FindAtomModes mode,long number)
{
  //myDebug()<<"AtomFindMarker::mark";

  if(!cluster)return false;

  //myDebug()<<"AtomFindMarker::mark clearing";
  emit markAtomsRequest(false,true);patomlist.clear();
  
  bool found=false;
  for(long i=0;i<cluster->numAtoms();i++){
    const Atom& at=cluster->atom(i);
    if(mode==Xf::FindSite && number==at.site+1){
      patomlist.append(at);
      found=true;
    }else if(mode==Xf::FindSort && number==at.sort+1){
      patomlist.append(at);
      found=true;
    }else if(mode==Xf::FindType && number==at.type){
      patomlist.append(at);
      found=true;
    }else if(mode==Xf::FindClusterNumber && number==i+1){
      patomlist.append(at);
      found=true;
    }
  }
  
  if(found) {
    
    if(0){
      myDebug() << "FindAtom: number="<< number;
      print();
    }
    //myDebug()<<"AtomFindMarker::mark";

    emit markAtomsRequest(true,true);
  }
  
  return found;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomFindMarker::print()
{
  foreach(Atom at,patomlist){
    at.print();
  }
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomFindMarker::onPropertiesChanged()
{
  //myDebug() << pmarkproperties.isSpecular();
  //myDebug()<<"AtomFindMarker::onPropertiesChanged";
  emit markAtomsRequest(true,true);

}

/*EOF*/ 
