/*************************************************************************
 * @(#)AutoColor.cpp
 * author: magru@unas
 * date:   09 Jul 2013
 *************************************************************************
 */

#include "Field.hh"
#include "AutoColor.hh"
#include "IsoSurface.hh"
#include "Array_cpp.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AutoColor::AutoColor()
  :Module("AutoColor"),pfield(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AutoColor::~AutoColor()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AutoColor::setInput(const Module *input)
{
  if(input->classname()=="Field"){
    pfield=const_cast<Field*>(dynamic_cast<const Field*>(input));
  }else if(input->classname()=="IsoSurface"){
    pfield=const_cast<IsoSurface*>(dynamic_cast<const IsoSurface*>(input))
      ->outputField();
  }else{
    throw Except("AutoColor::setInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AutoColor::removeInput(const Module *input)
{
  if(input->classname()=="Field"){
    if(pfield==input) pfield=0;
  }else if(input->classname()=="IsoSurface"){
    pfield=0;
  }else{
    throw Except("AutoColor::removeInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AutoColor::cascadingUpdate(const String &prefix)
{
  if(Module::debugmode) printf("%s%s::cascadingUpdate\n",prefix(),id()());
  Module::cascadingUpdate(prefix+"  ");
  showDirt(prefix,Sprintf("%s must react to",id()()));
  bool ret=false;
  if(pisdirty || dirtyInput("")){
    sendDirt("changed");
    calcAutoColor();
    ret=true;
  }
  resetDirt();
  if(Module::debugmode) printf("%sret=%d\n",prefix(),ret);
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AutoColor::calcAutoColor()
{
  //myDebug()<<id()<<"field="<<(void*)pfield;
  if(!pfield) return;

  const BaseArrayPointer &bap=pfield->data();
  //myDebug()<<id()<<"datatype="<<bap.dataType();

  switch(bap.dataType()){
  case TIlong:
    calcAutoColor_internal<long>();
    break;
  case TIdouble:
    calcAutoColor_internal<double>();
    break;
  case TICmplx:
    calcAutoColor_internal<Cmplx>();
    break;
  case TIunkown:
    break;
  }

}

#include "AutoColor_internal_cpp.hh"

/*EOF*/ 
