/*************************************************************************
 * @(#)AutoColor_internal_cpp.hh
 * author: magru@unas
 * date:   24 Jun 2013
 *************************************************************************
 */

#include "AutoColor.hh"
#include "ColorModel.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

template <class X>
void AutoColor::calcAutoColor_internal()
{
  bool prot=false;
  if(prot) printf("=======================================================\n"
		  "%s::calcAutoColor_internal<%s>()\n",id()(),TypeInfo<X>::name);
  if(!pfield) return;  

  const BaseArray<X> *bdi=pfield->data().to<X>();
  if(!bdi) return;
  if(bdi->datasize()!=1)
    throw Except("AutoColor for non scalar data not yet implemented");
  
  const X *ddat=bdi->data();

  ColorModel *colormodel=new ColorModel();//todo: make input and track changes
  colormodel->setMap();
  
  DataTypes datatype=bdi->dataType();
  long N=bdi->size();
  // long rank=bdi->rank();
  // long datasize=bdi->datasize();
  
  //todo: vector and tensor data
  //todo: make specialized cmplx version, chose phase are real or imag or abs

  pfield->setupColors(ValueArray<long>()<<4);
  BaseArray<double> *bacol=pfield->colors();
  bacol->setSize(N);

  double *dcol=bacol->valueArray().dataRef();

  const DataStatistics *dstat=pfield->dataStatistics();
  const ValueArray<X> &dmin=dstat->data<X>()->minValues;
  const ValueArray<X> &dmax=dstat->data<X>()->maxValues;

  //todo: vector and tensor data
  double datmin;
  double datmax;

  double delta;

  if(datatype==TICmplx){
    datmin=MyMath::toDouble(MyMath::Abs(dmin.at(0)));
    datmax=MyMath::toDouble(MyMath::Abs(dmax.at(0)));
  }else{
    datmin=MyMath::toDouble(dmin.at(0));
    datmax=MyMath::toDouble(dmax.at(0));
  }
  // datmin=-twopi/5.;
  // datmax=twopi/5.;
  delta=datmax-datmin;
  //if(delta!=0) delta=1./delta;

  if(prot) myDebug()<< "Field data in ["<<datmin<<","<<datmax<<"] delta="<<delta;
  
  for(long i=-1;++i<N;){
    
    double d;
    
    if(datatype==TICmplx){
      d=MyMath::toDouble(MyMath::Abs(ddat[i]));
    }else{
      d=MyMath::toDouble(ddat[i]);
    }

    double h=0.5;
    if(fabs(delta)>1.0e-12)  h=(d-datmin)/delta;
    //myDebug()<<d;
    double dfcol[4];
    colormodel->getColorArrayFromIndex(h,dfcol);
    //todo: alpha=isocolor.alpha()/255.;

    memcpy(dcol+4*i,dfcol,sizeof(double)*3);
    dcol[4*i+3]=1.;//todo fucos(alpha);
    
    
  }

  //todo: allow to take color componenets fron one several data and alpha from another

  if(prot) printf(" =======================================================\n");
}

/*EOF*/
