/* @(#)Axes.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _AXES_H
#define _AXES_H 1

#include "MyQRgb.hh"
#include <stdio.h>
#include "SReal.hh"
#include "Xf.hh"
#include "MACROS.hh"
#include "IDCache.hh"
#include "PtrArray.hh"
#include "Vec3.hh"

class Parser;

namespace XFPLO{
  class Symmetry;

  class Axess;


/* ***********************************************************************
 *
 * ***********************************************************************
 */


class Axes
{
public:

  Axes();
  Axes(Xf::AxesType t/*=Xf::ATCartesian*/);
  Axes(const Axes &o);
  const Axes &operator=(const Axes &o);
  Axes *clone(){return new Axes(*this);};
  ~Axes(){};
    

  
  
  void setDefaultLabel(){
    if(ptype!=Xf::ATGeneral){
      setLabel(0,Xf::axesName(ptype,0));
      setLabel(1,Xf::axesName(ptype,1));
      setLabel(2,Xf::axesName(ptype,2)); 
    }
    setFontShift(0,SReal3(-0.25,-0.5,1.5));
    setFontShift(1,SReal3(-0.25,-0.5,1.5));
    setFontShift(2,SReal3(-0.25,-0.5,1.5));
    setFontScale(SReal("0.1"));
    setFontColor(0x00);
  }
  void setDefaultOrigin(){
    setAxisOrigin(0,SReal("0"));
    setAxisOrigin(1,SReal("0"));
    setAxisOrigin(2,SReal("0"));
    if(ptype==Xf::ATConventional)
      setOriginUnits(Xf::SUConvCell);
    else if(ptype==Xf::ATPrimitive)
      setOriginUnits(Xf::SUPrimCell);
    else
      setOriginUnits(Xf::SUWorld);
  }
  void setDefaultAxes(){
    if(ptype==Xf::ATConventional || ptype == Xf::ATPrimitive ){
      setAxisLength(0,SReal("1"));
      setAxisLength(1,SReal("1"));
      setAxisLength(2,SReal("1"));
      setAxesLengthUnits(Xf::SUWorld);
    }else{
      setAxisLength(0,SReal("0.55"));
      setAxisLength(1,SReal("0.55"));
      setAxisLength(2,SReal("0.55"));
      setAxesLengthUnits(Xf::SUBoundary);
    }
    
    setColor(defaultColor());

  };
  void setDefaultRod(){
    setRodRadius(SReal("0.01"));
    setRodRadiusUnits(Xf::SUBoundary);
  };
  void setDefaultTip(){
    setTipRadius(SReal("2"));
    setTipRadiusUnits(Xf::SURodRadius);
    setTipLength(SReal("2.5"));
    setTipLengthUnits(Xf::SUTipRadius);
    setHasTip(true);
  };







  bool changed()const{return pchanged;};
  void setChanged(bool c){pchanged=c;pcalced&=!pchanged;};



  void calculateAxes(Xf::LatticeType latticetype
		     ,double alloverscale
		     ,const Symmetry *sym);

  
  // from here return  dependend/calculated data for display
  Vec3 dispVector(int i)const{return pvectors(i);};
  Vec3 dispOrigin()const{return porigin;};
  double dispRodRadius()const{return prr;};
  double dispTipRadius()const{return ptr;};
  double dispTipLength()const{return ptl;};
  bool show()const{return pshow && pvisible;};

  void print();

private:
  PROPERTY_CHANGED_CALCED(type,axesType,AxesType,Xf::AxesType);
  CPROPERTY_CHANGED(id,id,ID,String);
  CPROPERTY_CHANGED(name,name,Name,String);
  bool pchanged;
  PROPERTY(calced,calculated,Calculated,bool);
  PROPERTY(visible,isVisible,Visible,bool);
  CVPROPERTY_CHANGED_CALCED(rgb,3,color,Color,QRgb);
  CVPROPERTY_CHANGED_CALCED(defaultrgb,3,defaultColor,DefaultColor,QRgb);
  CPROPERTY_CHANGED_CALCED(rodradius,rodRadius,RodRadius,SReal);
  PROPERTY_CHANGED_CALCED(rodradiusunits,rodRadiusUnits,RodRadiusUnits,Xf::SceeneUnits);
  CPROPERTY_CHANGED_CALCED(tipradius,tipRadius,TipRadius,SReal);
  PROPERTY_CHANGED_CALCED(tipradiusunits,tipRadiusUnits,TipRadiusUnits,Xf::SceeneUnits);
  CPROPERTY_CHANGED_CALCED(tiplength,tipLength,TipLength,SReal);
  PROPERTY_CHANGED_CALCED(tiplengthunits,tipLengthUnits,TipLengthUnits,Xf::SceeneUnits);
  PROPERTY_CHANGED_CALCED(hastip,hasTip,HasTip,bool);
  CVPROPERTY_CHANGED_CALCED(axeslength,3,axisLength,AxisLength,SReal);
  PROPERTY_CHANGED_CALCED(axeslengthunits,axesLengthUnits,AxesLengthUnits,Xf::SceeneUnits);
  CVPROPERTY_CHANGED_CALCED(axesorigin,3,axisOrigin,AxisOrigin,SReal);
  PROPERTY_CHANGED_CALCED(originunits,originUnits,OriginUnits,Xf::SceeneUnits);
  CVPROPERTY_CHANGED_CALCED(label,3,label,Label,String);
  CVPROPERTY_CHANGED_CALCED(fontshift,3,fontShift,FontShift,SReal3);
  CPROPERTY_CHANGED_CALCED(fontscale,fontScale,FontScale,SReal);
  CPROPERTY_CHANGED_CALCED(fontcolor,fontColor,FontColor,QRgb);
  // these are dependend data
  Matrix3x3 pvectors;
  Vec3 porigin;
  double prr,ptr,ptl;
  bool pshow;
  friend class Axess;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Axess{
public:
  
  Axess():pchanged(false),pcalced(false){setDefault();}
  Axess(const Axess &);
  const Axess &operator=(const Axess &other)
  {
    //myDebug()<<"Axess operator="<<(void*)this;
    clear();
    paxes=other.paxes;
    pidcache=other.pidcache;
    pnamecache=other.pnamecache;
    pchanged=other.pchanged;
    pcalced=other.pcalced;
    return *this;
  }

  ~Axess(){clear();}

  void clear(){pidcache.clear();pnamecache.clear();paxes.clear();}
  
  inline int size()const{return paxes.size();};
  inline int indexOf(const Axes*ax)const{return paxes.indexOf(ax);};
  inline const Axes *axes(int i)const{return paxes.at(i);};
  inline Axes *axes(int i){return paxes[i];};
  Axes *findByType(Xf::AxesType type);
  Axes *findByID(const String &);



  void insert(int at,Axes *a,bool createuniquename);


  void setDefault(){
    //myDebug()<<"Axess"<<(void*)this;
    insert(0,new Axes(Xf::ATCartesian),false);
    insert(1,new Axes(Xf::ATConventional),false);
    insert(2,new Axes(Xf::ATPrimitive),false);
    // for(long i=-1;++i<paxes.size();)
    //   myDebug()<<(void*)paxes.at(i);
  }
  

  // ----------------------------------------------------------------
  bool changed()const
  {
    bool c=pchanged; 
    for(long i=-1;++i<paxes.size();) c|=paxes.at(i)->changed();
    return c;
  }
  
  // ----------------------------------------------------------------
  void setChanged(bool c)
  {
    pchanged=c;
    for(long i=-1;++i<paxes.size();) paxes[i]->setChanged(c);
  }

  // ----------------------------------------------------------------
  bool calculated()
  {
    bool c=pcalced;
    for(long i=-1;++i<paxes.size();) c&=paxes[i]->calculated();
    return c;
  }

  // ----------------------------------------------------------------
  void setCalculated(bool s)
  {
    pcalced=s;
    for(long i=-1;++i<paxes.size();) paxes[i]->setCalculated(s);
  }

  // ----------------------------------------------------------------
  void calculateAxes(Xf::LatticeType latticetype
		     ,double alloverscale
		     ,const Symmetry *sym)
  {
    if(calculated())return;
    setCalculated(false); //propagate the changes trough all properties
    for(long i=-1;++i<paxes.size();)
      paxes[i]->calculateAxes(latticetype,alloverscale,sym);
    pchanged=false;
    pcalced=true;

  }

  // ----------------------------------------------------------------
  void readAxesPTSectionFromFile(Parser *par);

  void axesPTSectionToFile(FILE *FH);

private:
  bool pchanged,pcalced;
  PtrArray<Axes> paxes;
  IDCache pidcache;
  IDCache pnamecache;
};

}
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _AXES_H */

