/*************************************************************************
 * @(#)BWFromCoeffs.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Nov 2010
 *************************************************************************
 */

#include "BWFromCoeffs.hh"
#include "Except.hh"
#include "Assorted.hh"
#include "Exception.hh"
#include "MyDebug.hh"

using namespace XFPLO;
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWFromCoeffs::save(const QString &fname)
{
  //myDebug() << "save to "<<fname;

  FILE *FH;
  try{
    FH=openforwrite(String(fname));
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }
//   if(puseinfplo)
//     fprintf(FH,"\nuse_in_fplo\n");
//   else
//     fprintf(FH,"\nX use_in_fplo\n");
  fprintf(FH,"\ncoefficients %s\n",pcurrentcoeffsfile());
  fprintf(FH,"bweights %s\n\n",pcurrentbwfile());
    
  pmolorbdefs.writeToFile(FH);
  fclose(FH);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWFromCoeffs::load(const QString &fname)
{
  //myDebug() << "load from "<<fname;
  FILE *FH;
  try{
    FH=openforread(String(fname));
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }
  StringList output;
  String warn=pmolorbdefs.readFromFile(FH,pcurrentcoeffsfile,pcurrentbwfile,
				       false,output);
  //,puseinfplo);
  fclose(FH);

  if(!warn.empty()){
    throw Exception(warn,Exception::warning);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BWFromCoeffs::isBWDefFile(const QString &fileName)
{
  //myDebug() << "BWFromCoeffs::isBWDefFile"<<fileName;
  bool isdeffile=false;

  FILE *FH;
  try{
    FH=openforread(String(fileName));
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }
  String line;
  bool isempty=true;
  long secondguess=0;
  while (getLine(FH,line)){
    line=line.chop("\n").stripBlanks().lowerCase();
    if(line.subString(0,4)=="bwdef"){
      isdeffile=true;
      break;
    }
    if(line.subString(0,11)=="coefficients"){
      secondguess++;
    } 
    if(line.subString(0,7)=="bweights"){
      secondguess++;
    }
   //myDebug() << line() << line.empty();
    if(!line.empty()){
      //myDebug() << "{"<<line()<<"}";
      isempty=false;
    }
  }
  //myDebug() << "isempty" << isempty;
  //myDebug()<<"secondguess="<<secondguess;
  fclose(FH);

  return (isdeffile||isempty||(secondguess==2));
}


/*EOF*/ 
