/* @(#)BWFromCoeffs.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _BWFROMCOEFFS_H
#define _BWFROMCOEFFS_H 1

#include "FString.hh"
#include "MolecularOrbital.hh"
#include "CoeffBasisInfo.hh"
#include "Coeffs.hh"

namespace XFPLO{

class BWFromCoeffs
{
 public:
  BWFromCoeffs()
    :pcurrentbwfile("+bweights_bwdef")
    ,pcurrentcoeffsfile("+coeff")
    {};
  ~BWFromCoeffs(){
    pmolorbdefs.clean();
    pcoeffs.clean();
    pcbi.clean();
  };

  void save(const QString &fname);
  void load(const QString &fname);
  
  static bool isBWDefFile(const QString &fileName);

  inline MolecularOrbitals *molOrbDefs(){return &pmolorbdefs;};
  inline Coeffs *coefficientData(){return &pcoeffs;};
  inline const CoeffBasisInfo *coeffBasisInfo()const{return &pcbi;};

  inline void readCoeffsFromFile(const ::String &fileName,bool forceload){
    pcoeffs.readFromFile(fileName,pcbi,forceload);
    //todo    checkConsistency(fileName);
  };
  inline bool coeffsAreReady()const{return pcoeffs.isReady();}; 

  inline void setGlobalSpinZaxis(const Vec3 &n){
    pmolorbdefs.setGlobalSpinZaxis(n);
  }
  const String &currentCoeffFile()const{return pcurrentcoeffsfile;}; 
  void setCurrentCoeffFile(const String & f){pcurrentcoeffsfile=f;};

  const String &currentBWFile()const{return pcurrentbwfile;}; 
  void setCurrentBWFile(const String & f){pcurrentbwfile=f;};


  inline void makeBandWeights(const String &bwfname){ 
    pcoeffs.makeBandWeights(bwfname,pmolorbdefs,pcbi); 
  }; 

 private:
  MolecularOrbitals pmolorbdefs;
  Coeffs pcoeffs;
  CoeffBasisInfo pcbi;
  
  String pcurrentbwfile;
  String pcurrentcoeffsfile;

};
}
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _BWFROMCOEFFS_H */

