/*************************************************************************
 * @(#)BWeights.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   14 Aug 2010
 *************************************************************************
 */

#include <QProgressDialog>
#include <stdio.h>
#include <QString>
#include "BWeights.hh"
#include "Except.hh"
#include "FString.hh"
#include "Assorted.hh"
#include "RegularMesh.hh"
#include "Exception.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
BWeights::BWeights()
  :pilower(0),piupper(0),pnband(0),pnspin(0),pnkp(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BWeights::~BWeights()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWeights::load(const char *fname,RegularMesh *pmesh,long b0,long b1)
{
  FILE *FH;

  try{
    FH=openforread(fname);
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }

  long nsite;
  long nweights;
  double ef;

  int err=fscanf(FH,"%*s%ld %lg %ld %ld %ld %ld %ld %ld\n",&nsite,&ef,&pnkp
		 ,&nweights,&pnband,&pnspin,&pilower,&piupper)-8;
  pilower--;
  piupper--;
  if(err==-2){
    /* old file assumed */
    pilower=0;
    piupper=pnband-1;
    err=0;
  }
  
  if(err){
    throw Exception(Sprintf("Header of file '%s' corrupted?",fname));
  }

  String header;
  getLine(FH,header);
  //printf("%s\n",header());
  nweights=0;
  while( 1 ){
    nweights++;
    int i=42+(nweights-1)*17;
    if(i+16>header.length()) break;
    plabels.append(header.subString(i,i+16).removeTrailingBlanks()());
  }
  nweights=plabels.size();

//   foreach (QString s,plabels){
//     (void)printf("state_to_plot : <%s>\n",s.toAscii().constData());
//   }
//   (void)printf("nweights=%ld\n",nweights);


  // we reserve 0 for the gradient norm and use the rest for weights
  QList<MeshData*> &meshdata=pmesh->meshData();
  for(int i=0;i<meshdata.size();i++){
    meshdata[i]->addExtraDataArray(nweights+1);
  }

  long subdiv=(MyMath::max<long>(1,pnkp/(100/5)));
  long countMax=MyMath::max<long>(1,pnkp/subdiv);
  QProgressDialog progress(QString("Reading %1...").arg(fname)
			   , "OK", 0, countMax, 0);
  progress.setWindowModality(Qt::ApplicationModal);

  long bdim=b1-b0+1;

  for(long iki=0;iki<pnkp;iki++){
    if ( (iki)%(MyMath::max<long>(1,pnkp/(100/5))) ==0 ){
      progress.setValue((int)(iki/subdiv));
    }
    long ll=pmesh->RequivI_Rot(iki);
    long ik=pmesh->IequivR(ll);
    for(long ms=0;ms<pnspin;ms++){
      for(long ib=pilower;ib<=piupper;ib++){
	if( ib>=b0 && ib<=b1) {
	  String sdum;
	  getLine(FH,sdum);
	  sdum.stripBlanks();
	  StringList sl=sdum.split(" \t");
	  //printf("%s %s\n",sl[2](),sl[3]());
	  if(sl.size()-2!=nweights)
	    throw Exception(Sprintf("Different number of labels and weight \n"
					      "columns in file '%s'?",fname));
	  double w;
	  for(int i=2;i<sl.size();i++){
	    sscanf(sl[i](),"%lg",&w);
	    //printf("%g ",w);
	    pmesh->setMeshExtraData((ib-b0+bdim*ms),ik,i-1,w);
	  }
	  //printf("\n");

	}
      }
    }
  }
  progress.setValue(countMax)      ;

  
  fclose(FH);

}

/*EOF*/ 
