
//todo: understand findBondDefBounds

//todo: does permits need bin argument ->search
//todo: there is this problem with  checkSites 0,000968768 s 
//todo: Symmetry::calcSymFromGroupInput calcAutoRadii 0,033185645 s 
//todo: Symmetry::calcSymFromGroupInput determineShells

//todo: for 7000 sites per unit cell

/*************************************************************************
 * @(#)BondDefs.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   29 Mar 2011
 *************************************************************************
 */

#include "BondDefs.hh"
#include "BBins.hh"
#include "Parser.hh"
#include "MachineTime.hh"
#include "Except.hh"
#include "Symmetry.hh"
#include "Config.hh"
#include "MyDebug.hh"


ValueArray<Pair<BondDefGroupEnum::CutOffMode,const char *> > 
BondDefGroupEnum::pcutoffmodename=
  (ValueArray<Pair<BondDefGroupEnum::CutOffMode,const char *> >()
   <<Pair<BondDefGroupEnum::CutOffMode,const char *>(BondDefGroupEnum::Boundary,"In Boundary")
   <<Pair<BondDefGroupEnum::CutOffMode,const char *>(BondDefGroupEnum::AddMissingAtoms,"Add Missing")
   <<Pair<BondDefGroupEnum::CutOffMode,const char *>(BondDefGroupEnum::RemoveIncomplete,"Del Incomplete")
   <<Pair<BondDefGroupEnum::CutOffMode,const char *>(BondDefGroupEnum::RemoveLoner,"Del Loners")
   );



PolyhedronProperties BondDefSortProperties::defaultPolyProps;
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefSortProperties::setUpDefaultPolyProps()
{
  defaultPolyProps.setVisible(false);
  defaultPolyProps.setDrawRim(true);
  defaultPolyProps.setDrawFaces(true);
  defaultPolyProps.setFaceMaterial(Material(0xaa9f9f9f,0x323232,50.));
  defaultPolyProps.setRimMaterial(Material(0xaa9f9f9f,50.));
  defaultPolyProps.setRimRadius(0.03);
  defaultPolyProps.setRimRadiusUnits(Xf::SUAngstroem);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondDefs::BondDefs()
  :pchanged(false)
{
  NSF("BondDefs::BondDefs"); 
  //myDebug() << nsubfunc;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::setDefaultFromSymmetry(const Symmetry *sym)
{
  NSF("BondDefGroup::setDefaultFromSymmetry");
  MachineTime time;
  long prot=0;
  if(prot) myDebug() << nsubfunc;

  setBinTol(0.1);
  setTopLevelGrouping(BBins::BTLElement);


  for(int sort=0;sort<sym->nsort();sort++){
    long psite=sym->siteBySort(sort);
    long type=sym->wyckoffPosition(sort).type();
    if(prot) myDebug() << "sort=" << sort << sym->wyckoffPosition(sort).el();
    BBins bins;
    bins.calculateBins(BBins::BCWyck,topLevelGrouping()
		       ,sort,type,psite,sym->pairs(),binTol());
    Bin *bin=bins.rootBin();
    if(bin->size()>0){
      const Bin *elbin=bin->bin(0);
      for(int i=0;i<elbin->size();i++){
	const Bin *wybin=elbin->bin(i);

	BondDef bdef=BondDef(sort,wybin,&bins);
	if(prot) myDebug() << "add this one";
	add(bdef);
	if(prot) myDebug() << "bdef     " << bdef;

      }
    }
    setAtomCutOffMode(sort,BondDefGroupEnum::Boundary);
  }

  time.print(String(nsubfunc)+" all");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefs::cleanBonds(const Symmetry *sym)
{
  NSF("BondDefs::cleanBonds");
  if(!sym) return;
  
  bool prot=false;
  // todo: use BondDef::remove?
  
  if(prot) myDebug()<<"\n"<<nsubfunc
		    <<"------------------------------begin----------------------------\n"
	     ;

  const ValueArray<PtrArray<PairVec> > &pairs=sym->pairs();

  for(long ibdg=-1;++ibdg<pbonddefgroups.size();){
    if(prot) myDebug()<<"cleaning BondDefGroup"<<ibdg;
    BondDefGroup *bdg=pbonddefgroups.group(ibdg);
    if(prot) myDebug()<<"old"<<*bdg;
    ValueArray<long> iwk(bdg->size(),1);
    for(long ibd=-1;++ibd<bdg->size();){
      BondDef *bd=bdg->bond(ibd);
      long psite=sym->siteBySort(bd->sorta());
      if(psite<0){
	iwk[ibd]=0;
	continue;
      }
      const PtrArray<PairVec> &pairs=sym->pairs().at(psite);
      if(prot) myDebug()<<"old bond"<<ibd<<*bd<<psite+1;
      double pmin=-1;
      double pmax=-2;
      for(long ip=-1;++ip<pairs.size();){
	const PairVec &pv=*(pairs.at(ip));
	if(pv.sort!=bd->sortb()) continue;
	if(prot) myDebug()<<pv;
	if(pmin<0)pmin=pv.pabs;
	else pmin=MyMath::min<double>(pmin,pv.pabs);
	pmax=MyMath::max<double>(pmax,pv.pabs);
      }
      if(prot) myDebug()<<"pmin,pmax="<<pmin<<pmax;
      if(pmin<=pmax){
	if(prot) myDebug()<<"valid bound for pairs, what to do?";
	continue;
	// now be canbe not at all in [pmin,pmax] or partially
	if(bd->pabsmax()<=pmin-1.0e-6){
	  iwk[ibd]=0;
	}else if(bd->pabsmin()>=pmax+1.0e-6){
	  iwk[ibd]=0;
	}else{
	  //bd->ppabsmin=MyMath::max<double>(bd->ppabsmin,pmin);
	  bd->setPabsmax(MyMath::min<double>(bd->pabsmax(),pmax));
	}
	
      }else{
	if(prot) myDebug()<<"there is no pairs for this bonddef: remove the bonddef";
	iwk[ibd]=0;
      }
    }
    if(prot) myDebug()<<"iwk="<<iwk;
    // delete all for which iwk[...]==0
    long i0=0;
    long N=iwk.size();
    for(long i1=-1;++i1<N;){
      if(iwk[i1]){
	if(i0!=i1){
	  *(bdg->bond(i0))=*(bdg->bond(i1));
	}
	i0++;
      }
    }
    bdg->resize(i0,true);

    //todo: delete old crap bdg->resizeA1Properties(sym->nsort(),true);

    if(prot) myDebug()<<"new"<<*bdg;
 
  }
  if(prot) myDebug()<<"\n"<<nsubfunc
		    <<"------------------------------end------------------------------\n";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefs::PTSectionToFile(FILE *FH,const SReal &ballscale,
			       const SReal &stickballratio)const
{

  fprintf(FH,"struct{\n");
  fprintf(FH,"       real ballscale;real stickballratio;\n");
  BondDefGroups::PTHeaderToFile(FH,"       ",";}bondDefs={\n");
  fprintf(FH,"       %s,%s,\n",
	  ballscale.S(),
	  stickballratio.S());
  pbonddefgroups.PTBodyToFile(FH,"       ","\n");
  fprintf(FH,"};\n\n\n");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefs::readPTSectionFromFile(const Dobj &root)
{
  clear();
  pbonddefgroups.readPTSectionFromFile(root("BondDefGroups"));
  setChanged(false);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BondDef::findBondDefBounds(const BBins *bins)
{
  NSF("BondDef::findBondDefBounds");
  //  myDebug() << nsubfunc;
  long prot=0;
  if(prot) myDebug() << nsubfunc;
  // if we have a bdef and a Bins structure
  // look up which are the closest bins of the same type as bdef
  // in order to extract bounds for the look up of neighbours
  // we assume that bdef.psorta or bdef.psortb == bins.psort
  long bsort=bins->sort();
  // one of the psorts must equal bsort
  MASSERT("Ooops ",((psorta == bsort) || (psortb == bsort)));
  // the other sort must be different from bsort (unles both psorts are equal)
  long othersort=psorta;
  if(othersort == bsort){ othersort=psortb;}
      
  double pmin=0.0; // naturally
  double pmax=1.0e10; // we might not find one
  bool havemax=false; // was the last bin of another sort (proper pmax defined)
  double pmaxinlist=0.0;
  bool startmax=false;
  bool stopmin=false;
  bool stop=false;
  Bin *rootbin=bins->rootBin();
  for(int i=0;i<rootbin->size();i++){
    const Bin *bin=rootbin->bin(i);
    //todo: make generic
    if(bin->binType()==Bin::BTElement){
      if(prot) myDebug("\telement bin[%d]=%s",i,bin->toString()());
      for(int is=0;is<bin->size();is++){
	const Bin *wybin=bin->bin(is);
	pmaxinlist=MyMath::max<double>(pmaxinlist,wybin->pabs());
	if(prot){
	  myDebug() << "\t\tis=" << is << (*wybin)
		    << "bdef.permits? " << (*this);
	}
	if(!startmax && permits(bsort,wybin->sort(),wybin->pabs(),1.0e-9)){
	  if(prot) myDebug() << "\t\tthats me="<<pmin<<pmax;
	  startmax=true;
	  stopmin=true;
	  continue;
	}else if(startmax && wybin->sort()==othersort){
	  pmax=wybin->pabs();
	  if(prot) myDebug() << "\t\tthats next sort="<<pmin<<pmax;
	  havemax=true;
	  stop=true;
	  break;
	}else if(!stopmin && wybin->sort() ==othersort){
	  //pmin=MyMath::min<double>(pmin,wybin->pabs());
	  //pmin=MyMath::min<double>(pmin,wybin->pabs());
	  pmin=wybin->pabs();
	  if(prot) myDebug() << "\t\tthats earliest="<<pmin<<pmax;
	}
      }
    }else if(bin->binType()==Bin::BTWhatEver){
      if(prot) myDebug() << "\twhatever=" << i << (*bin);
      for(int eb=0;eb<bin->size();eb++){
	const Bin *elbin=bin->bin(eb);
	if(prot) myDebug() << "\teb=" << eb << (*elbin);
	for(int is=0;is<elbin->size();is++){
	  const Bin *wybin=elbin->bin(is);
	  //	  if(wybin->sort()!=othersort) continue;
	  pmaxinlist=MyMath::max<double>(pmaxinlist,wybin->pabs());
	  if(prot){
	    myDebug() << "\t\tis=" << is << (*wybin);
	    myDebug() << "bdef.permits? " << (*this);
	  }
	  if(!startmax && permits(bsort,wybin->sort(),wybin->pabs(),1.0e-9)){
	    if(prot) myDebug() << "\t\tthats me=";
	    startmax=true;
	    stopmin=true;
	    continue;
	  }else if(startmax && wybin->sort()==othersort){
	    if(prot) myDebug() << "\t\tthats next=";
	    pmax=wybin->pabs();
	    havemax=true;
	    stop=true;
	    break;
	  }else if(!stopmin && wybin->sort() ==othersort){
	    if(prot) myDebug() << "\t\tthats earliest=";
	    //pmin=MyMath::min<double>(pmin,wybin->pabs());
	    pmin=wybin->pabs();
	  }
	}
	if(stop) break;
      }
      
      
    }
    if(stop) break;

  }
  if(prot){
    myDebug("pmin      =%30.15g",pmin);
    myDebug("ppabsmin  =%30.15g",ppabsmin);
    myDebug("pmax      =%30.15g",pmax);
    myDebug("havemax=%d\nppabsmax  =%30.15g",havemax,ppabsmax);
    myDebug("pmaxinlist=%30.15g",pmaxinlist);
    //myDebug() << "pmax\\ "<<ppabsmin+(pmax-ppabsmin)*0.90;
  }
  if(!havemax)pmax=pmaxinlist*1.5;
  if(pmin<1.0e-4){
    ppabsmin=0.;
  }else{
    ppabsmin=ppabsmin+(pmin-ppabsmin)*0.50;
  }
  ppabsmax=ppabsmax+(pmax-ppabsmax)*0.50;
  if(prot){
    myDebug("ppabsmin  =%30.15g",ppabsmin);
    myDebug("ppabsmax  =%30.15g",ppabsmax);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDef::PTHeaderToFile(FILE *FH,const char *prefix
				   ,const char *tail)
{

  fprintf(FH,
	  "%1$sstruct{\n"
	  "%1$s       int sorta;\n"
	  "%1$s       int sortb;\n"
	  "%1$s       real pabsmin;\n"
	  "%1$s       real pabsmax;\n"
	  ,prefix);
  fprintf(FH,"%s}BondDef%s",prefix,tail);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDef::PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const
{
  fprintf(FH,"%s{%ld,%ld,%30.15g,%30.15g}%s"
	  ,prefix,psorta,psortb,ppabsmin,ppabsmax,tail);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDef::readPTSectionFromFile(const Dobj &root)
{
  psorta=root("sorta").valueL();
  psortb=root("sortb").valueL();
  ppabsmin=root("pabsmin").valueD();
  ppabsmax=root("pabsmax").valueD();
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefSortProperties::
PTHeaderToFile(FILE *FH,const char *prefix,const char *tail)
{
  fprintf(FH,
	  "%1$sstruct{\n"
	  "%1$s       int cutoffmode;\n"
	  ,prefix);
  PolyhedronProperties::PTHeaderToFile(FH,Sprintf("%s       ",prefix)(),";\n");
  fprintf(FH,"%s}BondDefSortProperties%s",prefix,tail);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefSortProperties::
PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const
{
  fprintf(FH,"%s{%d,\n",prefix,patomcutmode);
  ppolyprops.PTBodyToFile(FH,Sprintf("%s  ",prefix)(),"\n");
  fprintf(FH,"%s}%s",prefix,tail);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefSortProperties::readPTSectionFromFile(const Dobj &root)
{
  String s=(*(root.parser()))()("version.mainversion")();
  double mainvers;
  try{
    mainvers=s.convertToFortranReal();
  }catch(Except){
    mainvers=9.07;
  }
  //myDebug() << mainvers;
  patomcutmode=(BondDefGroupEnum::CutOffMode)root("cutoffmode").valueL();
  // older input
  try{
    ppolyprops.setVisible(root("drawpolygon").valueL());
  }catch(Except){
  }
  QRgb c(0xaa9f9f9f);
  QRgb specular(0xff323232);
  try{
    c=root("polycolor")().convertToULong();
    if(mainvers<=9.071){
      if(qAlpha(c)/255.==0){
	c|=0xaa000000;
      }
    }
  }catch(Except){};
  ppolyprops.setFaceMaterial(Material(c,specular,50.));
  

  // new full poly props
  try{
    ppolyprops.readPTSectionFromFile(root);
    if(mainvers<=9.071){
      //myDebug() << "set missing new";
      ppolyprops.setDrawFaces(true);
      ppolyprops.setDrawRim(false);
      ppolyprops.setRimRadius(0.03);
      ppolyprops.setRimRadiusUnits(Xf::SUAngstroem);
    }
  }catch(Except){}

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondDefGroup::BondDefGroup(QRgb col)
  :pactive(true),pcolor(col)
			    //,radius(0.2)
  ,pname("??"),pvisible(true),pbintol(0.1),
   ptoplevelgrouping(BBins::BTLElement),pchanged(false)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::add(const BondDef &b)
{
  NSF("BondDefGroup::add");
  //  myDebug() <<nsubfunc;
  long prot=0;
  if(prot) myDebug() << nsubfunc;

  double eps=1.0e-3;

  if(prot) myDebug() << pbonddeflist.size(); 

  long found=0;
  for(int i=0;i<pbonddeflist.size();i++){
    BondDef *bi=pbonddeflist[i];
    if(prot) myDebug() << "list["<<i<<"] " << (*bi);

    if( b.eqSorts(*bi) ){

      if(prot) myDebug() << "b       " << b;

      if ( ( b.pabsmax()>=bi->pabsmin()-eps ) &&
	   ( bi->pabsmax()+eps>= b.pabsmin())){
	bi->setPabsmin(MyMath::min<double>(bi->pabsmin(), b.pabsmin()));
	bi->setPabsmax(MyMath::max<double>(bi->pabsmax(), b.pabsmax()));
	found=1;

	if(prot) myDebug() << "mod list["<<i<<"] " << (*bi);
	pchanged=true;
      }
      
    }
  }
  if(!found){
    pbonddeflist.append(b.clone());

    pchanged=true;
    
    if(prot) myDebug() << "new list[i] " << (*pbonddeflist.last());

  }else{
    fuse();
  }
  //  myDebug() <<nsubfunc<<"end";
  return;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::remove(const BondDef &bdef,const BBins *bins)
{
  NSF("BondDefGroup::remove");

  long prot=0;
  if(prot) myDebug() << nsubfunc;
  if(prot) myDebug() << nsubfunc<< "arg_bdef " << bdef;



  long n=pbonddeflist.size();
  if(prot) myDebug() << "pbonddeflist " << pbonddeflist;
  

  int i=0;
  while(i<n){
    if(prot) myDebug();
    if(prot) myDebug() << "list["<<i<<"] " << (*pbonddeflist.at(i));

    if( !bdef.eqSorts(*pbonddeflist.at(i)) ){i++;
      if(prot) myDebug()<<"not equal sorts: continue loop";
      continue;
    }      // not equal sorts
    
    if((bdef.pabsmax() >= pbonddeflist.at(i)->pabsmax() )&& 
	     (bdef.pabsmin() <= pbonddeflist.at(i)->pabsmin() )){
      //
      //bdef              :     |xxxxxxxxxxxx|
      //pbonddeflist[i]   :       |--------|
      //newpbonddeflist[i]:     |
      pbonddeflist[i]->setPabsmin(bdef.pabsmin());
      pbonddeflist[i]->setPabsmax(bdef.pabsmin());
      if(prot) myDebug() << "remove whole";
      pchanged=true;
  }else if((bdef.pabsmax() >= pbonddeflist.at(i)->pabsmin() )&& 
       (bdef.pabsmin() <= pbonddeflist.at(i)->pabsmin() )){
      //
      //bdef               :   |xxxx|
      //pbonddeflist[i]    :      |---------| 
      //newpbonddeflist[i] :        |-------| 
      pbonddeflist[i]->setPabsmin(bdef.pabsmax());
      if(prot) myDebug() << "remove lower end";
      pchanged=true;
    }else if((bdef.pabsmax() >= pbonddeflist.at(i)->pabsmax() )&& 
	     (bdef.pabsmin() <= pbonddeflist.at(i)->pabsmax() )){
      //
      //bdef               :              |xxxx|
      //pbonddeflist[i]    :      |---------|
      //newpbonddeflist[i] :      |-------|
      pbonddeflist[i]->setPabsmax(bdef.pabsmin());
      if(prot) myDebug() << "remove upper end";
      pchanged=true;
    }else if((bdef.pabsmax() <= pbonddeflist.at(i)->pabsmax() )&& 
	     (bdef.pabsmin() >= pbonddeflist.at(i)->pabsmin() )){
      //
      //bdef               :         |xxxx|
      //pbonddeflist[i]    :      |---------|
      //newpbonddeflist[i] :      |--|    |-|
      
      BondDef *b=bdef.clone();
      b->setPabsmin(bdef.pabsmax());
      b->setPabsmax(pbonddeflist[i]->pabsmax());
      pbonddeflist.insert(i+1,b);
      
      pbonddeflist[i]->setPabsmax(bdef.pabsmin());
      n++;
      pchanged=true;
      
      
      if(prot) myDebug() << "remove middle";
    }else{
      if(prot) myDebug()<<"nothing done";
    }
    
    i++;
  } // all bonddef

  if(prot){
    myDebug();
    for(long i=0;i<pbonddeflist.size();i++){
      myDebug() << "pbonddeflist["<<i<<"] " << (*pbonddeflist.at(i));
    }
    myDebug();
  }

  // remove zero length
  // only touch the current sort-pair
  double tol=1.0e-4;
  n=pbonddeflist.size();
  for(int i=n-1;i>=0;i--){
    if( bdef.eqSorts(*pbonddeflist.at(i)) ){
      if(fabs(pbonddeflist.at(i)->pabsmax()-pbonddeflist.at(i)->pabsmin())<tol){
	removeAt(i);
      }
    }
  }
  //remove  empty slots
  long bsort=bins->sort();
  MASSERT("Ooops ",((bdef.sorta() == bsort) || (bdef.sortb() == bsort)));
  //long othersort=bdef.psorta;
  //if(othersort == bsort){ othersort=bdef.psortb;}
  
  n=pbonddeflist.size();
  for(int i=n-1;i>=0;i--){
    if( ! pbonddeflist.at(i)->eqSorts(bdef) ) continue;
    long got_one=0;
    Bin *rootbin=bins->rootBin();
    //todo: BTWhatEver and others
    for(int eb=0;eb<rootbin->size();eb++){
      const Bin *elbin=rootbin->bin(eb);
      if(elbin->binType()==Bin::BTElement){
	for(int is=0;is<elbin->size();is++){
	  const Bin *wybin=elbin->bin(is);
	  if(pbonddeflist.at(i)->permits(bsort,wybin->sort()
					 ,wybin->pabs(),1.0e-3)){
	    if(prot) myDebug() << "got one " << (*pbonddeflist.at(i));
	    got_one=1;
	  }
	} //is
      }else{
	//todo other types of bins structures
	got_one=1;
      }
    } // eb
    if(!got_one){
      removeAt(i);
    }
  }
  // join what is joinable
  fuse();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::fuse()
{
  NSF("BondDefGroup::fuse");

  double eps=1.0e-3;
  long prot=0;
  if(prot) myDebug() << nsubfunc;
  
  for(int i=pbonddeflist.size()-1;i>=0;i--){
    const BondDef *bi=pbonddeflist.at(i);
    for(int j=i-1;j>=0;j--){
      const BondDef *bj=pbonddeflist.at(j);
      if( bi->eqSorts(*bj) ){
	if( (bj->pabsmax()>=bi->pabsmin()-eps) &&
	    (bj->pabsmin()<=bi->pabsmax()+eps) ) {
	  if(prot) {
	    myDebug() << "bond list["<<i<<"] " << (*bi);
	    myDebug() << "and  list["<<j<<"] " << (*bj)
		     << " overlap";
	  }
	  pbonddeflist[j]->setPabsmin(MyMath::min<double>(bi->pabsmin(), bj->pabsmin()));
	  pbonddeflist[j]->setPabsmax(MyMath::max<double>(bi->pabsmax(), bj->pabsmax()));
	  if(prot) myDebug() << "remove "<<i<<" ! ";
	  removeAt(i);
	  if(prot) myDebug() << "removed "<<i<<" ! ";
	  break;
	}
      }
    }
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::setFromBinSelection(const BBins *bbins,const Bin *bin,
				       bool checked)
{
  NSF("BondDefGroup::setFromBinSelection");
  long prot=0;
  if(prot) myDebug() << nsubfunc;
  if(prot) myDebug() << (checked?"add ":"remove ")<< (*bin);
  if(prot) myDebug() << "BBins::binTypeName"<<BBins::binTypeName.at(bin->binType()).v2;
  if(prot) myDebug() << "from"<<*this;
  
  setChanged(true);
  
  //todo: make recursive
  if(bin->binType()==Bin::BTWhatEver){
    for(int eb=0;eb<bin->size();eb++){
      const Bin *elbin=bin->bin(eb);
      for(int is=0;is<elbin->size();is++){
	const Bin *wybin=elbin->bin(is);
	BondDef bdef=BondDef(bbins->sort(),wybin,bbins);
	
	if(prot) myDebug() << "bdef"<<bdef;
	if(checked){
	  add(bdef);
	}else{
	  remove(bdef,bbins);
	}
      }
    }
    
    
  }else if(bin->binType()==Bin::BTElement){
    //myDebug()<<"bin->binType()==Bin::BTElement";
    //myDebug()<<"bin->size()="<<bin->size();
    for(int is=0;is<bin->size();is++){
      const Bin *wybin=bin->bin(is);
      //myDebug()<<"is="<<is<<"wybin="<<(*wybin);
      BondDef bdef=BondDef(bbins->sort(),wybin,bbins);
      //myDebug()<<"bdef="<<bdef;
      //myDebug()<<"bdef after findBondDefBounds="<<bdef;
      
      if(checked){
	add(bdef);
      }else{
	remove(bdef,bbins);
      }
      
    }
  }else if(bin->binType()==Bin::BTWyck){
    const Bin *wybin=bin;
    
    BondDef bdef=BondDef(bbins->sort(),wybin,bbins);
    
    if(checked){
      add(bdef);
    }else{
      remove(bdef,bbins);
    }
    
  }else{
    //todo other types
  }
  
  if(prot) myDebug()<<*this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::PTHeaderToFile(FILE *FH,const char *prefix
				   ,const char *tail)
{
  String s=String(prefix)+"       ";
  const char *ps=s();

  fprintf(FH,
	  "%1$sstruct{\n"
	  "%1$s       logical active;\n"
	  "%1$s       char[*] name;\n"
	  "%1$s       char[*] bondcolor;\n"
	  "%1$s       logical bondisvisible;\n"
	  "%1$s       real bintol;\n"
	  "%1$s       struct{int type;char[*] description;}toplevelgrouping;\n"
	  ,prefix);
  BondDef::PTHeaderToFile(FH,ps,"[*];\n");
  BondDefSortProperties::PTHeaderToFile(FH,ps,"[*];\n");
  fprintf(FH,"%s}BondDefGroup%s",prefix,tail);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::
PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const
{
  String s=String(prefix)+"       ";
  const char *ps=s();
  fprintf(FH,"%s{%c,\"%s\",%s,%c,%g,{%d,\"%s\"},{\n"
	  ,prefix,(pactive?'t':'f'),String(pname)(),Sprintf("\"%x\"",pcolor)()
	  ,(pvisible?'t':'f'),pbintol,ptoplevelgrouping,
	  BBins::binsTopLevelGroupingName.at(ptoplevelgrouping).v2);
  for(int i=0;i<pbonddeflist.size();i++){
    pbonddeflist.at(i)->
      PTBodyToFile(FH,ps,(i<pbonddeflist.size()-1?",\n":"\n"));
  }
  fprintf(FH,"%1$s},\n%1$s{\n",prefix);
  for(int i=0;i<pa1properties.size();i++){
    pa1properties.at(i).
      PTBodyToFile(FH,ps,(i<pa1properties.size()-1?",\n":"\n"));
  }
  fprintf(FH,"%s}}%s",prefix,tail);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroup::readPTSectionFromFile(const Dobj &root)
{

  pname=root("name")()();
  pcolor=root("bondcolor")().convertToULong();
  pvisible=root("bondisvisible").valueL();
  try{
    pactive=root("active").valueL();
  }catch(...){}
  pbintol=root("bintol").valueD();
  try{
    ptoplevelgrouping=(BBins::BinsTopLevelGrouping)root("toplevelgrouping.type").valueL();
  }catch(...){
    ptoplevelgrouping=BBins::BTLElement;
  }

  Dobj dbda=root("BondDef");
  for(int ibd=0;ibd<dbda.size();ibd++){
    Dobj dbdi=dbda[ibd];
    BondDef *bd=new BondDef();
    bd->readPTSectionFromFile(dbdi);
    pbonddeflist.append(bd);
  }
  Dobj dbda1;
  try{
    dbda1=root("BondDefGroupA1Properties");
  }catch(...){
    dbda1=root("BondDefSortProperties");
  }
  //todo: do not append, but resize first-> in general true
  for(int ip=0;ip<dbda1.size();ip++){
    Dobj dbdi=dbda1[ip];
    pa1properties.append(BondDefSortProperties());
    pa1properties.last().readPTSectionFromFile(dbdi);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroups::PTHeaderToFile(FILE *FH,const char *prefix
				   ,const char *tail)
{
  String s=String(prefix)+"       ";
  const char *ps=s();

  fprintf(FH,
	  "%1$sstruct{\n"
	  ,prefix);
  BondDefGroup::PTHeaderToFile(FH,ps,"[*];\n");
  fprintf(FH,"%s}BondDefGroups%s",prefix,tail);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroups::
PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const
{
  String s=String(prefix)+"       ";
  const char *ps=s();
  fprintf(FH,"%1$s{\n%1$s  {\n",prefix);
  for(int i=0;i<pbonddefgrouplist.size();i++){
    pbonddefgrouplist.at(i)->
      PTBodyToFile(FH,ps,(i<pbonddefgrouplist.size()-1?",\n":"\n"));
  }
  fprintf(FH,"%1$s  }\n%1$s}%2$s",prefix,tail);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondDefGroups::readPTSectionFromFile(const Dobj &root)
{

  Dobj dbdgsa=root("BondDefGroup");
  for(int ibdg=0;ibdg<dbdgsa.size();ibdg++){
    Dobj dbdgi=dbdgsa[ibdg];
    BondDefGroup *bdg=new BondDefGroup(0x9f9f9f);
    bdg->readPTSectionFromFile(dbdgi);
    pidcache.insert(bdg->name());
    pbonddefgrouplist.append(bdg);
  }
  
}



/*EOF*/ 
