//  todo: normalize plane directions: I think there is no logical algorithm

/*************************************************************************
 * @(#)Boundary.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   06 Jul 2009
 *************************************************************************
 */
// #include <math.h>
// #include <string.h>
// #include <stdio.h>
// #include <stdlib.h>

#include "fsymmetry.h"
#include "Vlevel.hh"
#include "Symmetry.hh"
#include "Parser.hh"
#include "Except.hh"
#include "Boundary.hh"
#include "Polyhedron.hh"
#include "MyMath.hh"
#include "MyDebug.hh"

using namespace XFPLO;

const Pair<String,String> Boundary::defaultBoxDistS=
    Pair<String,String>("0","1");

ValueArray<Face> Boundary::userFaces;


const char *Boundary::pcelltypename[]={
  "Cell/Box",
  "Wigner-Seitz cell",
  "Faces",
};

const char *Boundary::pcellbasistypename[]={
  "Primitive basis vectors",
  "Conventional basis vectors",
  "Cartesian basis vectors",
  "Cell/Box vectors"
};

const char *Boundary::pcenteringtypename[]={
  "None",
  "Centered",
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Triple{
public:
  Triple(long i1,long i2,long i3){
    long g=abs(MyMath::gcd(MyMath::gcd(i1,i2),i3)); 
    pi1=i1/g; pi2=i2/g; pi3=i3/g; };
  Vec3 v()const{return Vec3(pi1,pi2,pi3);};
  bool operator==(const Triple &other)const{
    return ((pi1==other.pi1) &&(pi2==other.pi2) &&(pi3==other.pi3));
  };
  bool operator<(const Triple &other)const{
    if(pi1<other.pi1) return true;
    if(pi1==other.pi1) 
      if(pi2<other.pi2) return true;
    if(pi1==other.pi1 && pi2==other.pi2) 
      if(pi3<other.pi3) return true;
    return false;
  };
  long pi1,pi2,pi3;
};

MyDebug operator<<(MyDebug dbg, const Triple &t)
{
  dbg << t.pi1<<t.pi2<<t.pi3<<")\n";
  return dbg;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::minl123(Matrix3x3 &aa,long i1,long i2,long i3,double &smin,
		       bool prot)
{
  
  if(prot) printf("\nminl123 i1,i2,i3=%ld %ld %ld\n",i1,i2,i3);

  double aa1=aa(i1)*aa(i1);
  double aa2=aa(i2)*aa(i2);
  double aa3=aa(i3)*aa(i3);
  double d12=aa(i1)*aa(i2);
  double d13=aa(i1)*aa(i3);
  double d23=aa(i2)*aa(i3);
  //

  double a11=aa2;
  double a12=d23;
  double a21=d23;
  double a22=aa3;
  //
  double det=a11*a22-a12*a21;
  double d=a11;
  a11=a22/det;
  a22=d/det;
  a12=-a12/det;
  a21=-a21/det;
  if(prot){
    printf("\nAA=\n%s\n",aa.transposed().printS("%12.5g")());
    printf("\nS=\n%s\n",(aa.transposed()*aa).printS("%12.5g")());
    printf("det=%g\n",det);
  }
  //
  double b1=-d12;
  double b2=-d13;
  //
  long n2=lRound(a11*b1+a12*b2);
  long n3=lRound(a21*b1+a22*b2);
  //
    
  if(prot)printf("n2=%ld n3=%ld\n",n2,n3);
      
  //
  smin=aa1+aa2+aa3;
  long i20=n2;
  long i30=n3;
  for(long ii2=n2-2;ii2<=n2+2;ii2++){
    for(long ii3=n3-2;ii3<=n3+2;ii3++){
      double s=aa1+aa2+aa3+2.*ii2*d12+2.*ii3*d13+
	2.*ii2*ii3*d23+(ii2*ii2)*aa2+(ii3*ii3)*aa3;
      if(smin>s){
	i20=ii2;
	i30=ii3;
	smin=s;
      }
    }
  }
  if(prot){
    printf("aa1=%g\n",aa1);
    printf("aa2=%g\n",aa2);
    printf("aa3=%g\n",aa3);
    printf("d12=%g\n",d12);
    
      
    printf("i2=%ld i3=%ld\n",i20,i30);
  }
  //
  double *data=aa.data();
  for(int i=0;i<3;i++)
    data[3*i1+i]+=data[3*i2+i]*i20+data[3*i3+i]*i30;

  if(prot){
    printf("\nAA=\n%s\n",aa.transposed().printS("%12.5g")());
    printf("\nS=\n%s\n",(aa.transposed()*aa).printS("%12.5g")());
    printf("smin=%g %g\n",smin,trace((aa.transposed()*aa)));
    
    printf("\nminl123 END\n");
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::cellReduce(Matrix3x3 &aa,double tol,bool prot)
{
  double s1;



  double aa1=aa(0)*aa(0);
  double aa2=aa(1)*aa(1);
  double aa3=aa(2)*aa(2);

  double smin=aa1+aa2+aa3;
  //
  if(prot){
    printf("smin0=%12.5g\n",smin);
    printf("minlat0 a1: %12.5g %12.5g %12.5g: %12.5g\n",aa(0)[0],aa(0)[1],aa(0)[2],sqrt(aa1));
    printf("minlat0 a2: %12.5g %12.5g %12.5g: %12.5g\n",aa(1)[0],aa(1)[1],aa(1)[2],sqrt(aa2));
    printf("minlat0 a3: %12.5g %12.5g %12.5g: %12.5g\n",aa(2)[0],aa(2)[1],aa(2)[2],sqrt(aa3));
  }
  //
  while(true){
    if(prot){
      printf("sminx=%12.5g\n",smin);
      printf("minlatx a1: %12.5g %12.5g %12.5g: %12.5g\n",aa(0)[0],aa(0)[1],aa(0)[2],sqrt(aa1));
      printf("minlatx a2: %12.5g %12.5g %12.5g: %12.5g\n",aa(1)[0],aa(1)[1],aa(1)[2],sqrt(aa2));
      printf("minlatx a3: %12.5g %12.5g %12.5g: %12.5g\n",aa(2)[0],aa(2)[1],aa(2)[2],sqrt(aa3));
    }
    aa1=aa(0)*aa(0);
    aa2=aa(1)*aa(1);
    aa3=aa(2)*aa(2);
    smin=aa1+aa2+aa3;
    //      print *,aa1,aa2,aa3
    if(aa1>=aa2 && aa1 >= aa3){
      minl123(aa,0,1,2,s1,prot);
      if(prot) printf("s1=%12.5g\n",s1);
    }else if(aa2>=aa1 && aa2>= aa3){
      minl123(aa,1,2,0,s1,prot);
      if(prot) printf("s2=%12.5g\n",s1);
    }else if(aa3>=aa1 && aa3 >= aa2){
      minl123(aa,2,0,1,s1,prot);
      if(prot) printf("s3=%12.5g\n",s1);
    }
    if(prot) printf("smin=%12.5g\n",smin);
    //     
    if(fabs(s1-smin)<tol) break;
  }
  

  aa1=aa(0)*aa(0);
  aa2=aa(1)*aa(1);
  aa3=aa(2)*aa(2);


  if(prot){
    printf("smin0=%12.5g\n",smin);
    printf("minlat1 a1: %12.5g %12.5g %12.5g: %12.5g\n",aa(0)[0],aa(0)[1],aa(0)[2],sqrt(aa1));
    printf("minlat1 a2: %12.5g %12.5g %12.5g: %12.5g\n",aa(1)[0],aa(1)[1],aa(1)[2],sqrt(aa2));
    printf("minlat1 a3: %12.5g %12.5g %12.5g: %12.5g\n",aa(2)[0],aa(2)[1],aa(2)[2],sqrt(aa3));
  }
  
  if(prot) (aa.transposed()*aa).print("S=");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::prepWSCellPlanes(const Matrix3x3 &primcell,ValueArray<Plane>&pla)
{
  // make cell as isotropic as possible
  Matrix3x3 pr=primcell;
  cellReduce(pr,1.0e-6,false);

  // make corners of (1,0,0)+perm "WS"-cell and determine the largest
  // needed parallelepiped to be sure that we have all planes
  
  Matrix3x3 rec=pr.inverseT();
  Matrix3x3 S=pr.transposed()*pr;
  Matrix3x3 Sinv=rec.transposed()*rec;
  double a0=S(0,0);
  double a1=S(1,1);
  double a2=S(2,2);
  
  ValueArray<Vec3>gs;
  ValueArray<Vec3>corners;
  gs.append(Vec3( a0, a1, a2));
  gs.append(Vec3(-a0, a1, a2));
  gs.append(Vec3( a0,-a1, a2));
  gs.append(Vec3(-a0,-a1, a2));
  gs.append(Vec3( a0, a1,-a2));
  gs.append(Vec3(-a0, a1,-a2));
  gs.append(Vec3( a0,-a1,-a2));
  gs.append(Vec3(-a0,-a1,-a2));

  corners.append(rec*Vec3( a0, a1, a2)*0.5);
  corners.append(rec*Vec3(-a0, a1, a2)*0.5);
  corners.append(rec*Vec3( a0,-a1, a2)*0.5);
  corners.append(rec*Vec3(-a0,-a1, a2)*0.5);
  corners.append(rec*Vec3( a0, a1,-a2)*0.5);
  corners.append(rec*Vec3(-a0, a1,-a2)*0.5);
  corners.append(rec*Vec3( a0,-a1,-a2)*0.5);
  corners.append(rec*Vec3(-a0,-a1,-a2)*0.5);


  double nx=0,ny=0,nz=0;
  Vec3 d=Sinv*Vec3(1,0,0);
  for(int i=-1;++i<gs.size();){
    nx=MyMath::max<double>(nx,gs.at(i)*d);
  }
 
  
  d=Sinv*Vec3(0,1,0);
  for(int i=-1;++i<gs.size();){
    ny=MyMath::max<double>(ny,gs.at(i)*d);
  }
  
  d=Sinv*Vec3(0,0,1);
  for(int i=-1;++i<gs.size();){
    nz=MyMath::max<double>(nz,gs.at(i)*d);
  }
  
  long inx;
  long iny;
  long inz;
  
  double tol=1.0e-4;
  inx=lRound(nx); if(fabs(inx-nx)>tol) inx=floor(nx);
  iny=lRound(ny); if(fabs(iny-ny)>tol) iny=floor(ny);
  inz=lRound(nz); if(fabs(inz-nz)>tol) inz=floor(nz);
  
  
  //printf("nx=%g\nny=%g\nnz=%g\n",nx,ny,nz);
  //printf("nx=%ld\nny=%ld\nnz=%ld\n",inx,iny,inz);
  
  for(long i1=-inx;i1<=inx;i1++){ 
    for(long i2=-iny;i2<=iny;i2++){ 
      for(long i3=-inz;i3<=inz;i3++){ 
	if( i1 || i2 || i3 ){
	  Vec3 R=pr*Vec3(i1,i2,i3);
	  pla.append(Plane(R,0.5));
	}
      }
    }
  } // i1 


  if(1){

    double tol=1.0e-8;
    // start with 8-corner cell
    long np=pla.size();
    for(int ip=np-1;ip>=0;ip--){
      const Plane &p=pla.at(ip);
      long nc=corners.size();
      bool onecorneroutside=false;
      for(int ic=0;ic<nc;ic++){
	if(p.d()<p.n()*corners.at(ic)+tol){
	  onecorneroutside=true;
	  break;
	}
      }
      if(!onecorneroutside){
	// discard plane
	/*Plane p=*/pla.takeAt(ip);
      }
    }
  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Boundary::Boundary()
  :pcelltype(Cell),pcellbasistype(Primitive),pcentered(None)
  ,plattice(Xf::LTDirect)
  ,poriginbasis(Cartesian)
  ,pradius(1.)
  ,ppolyhedron(new Polyhedron())
  ,pchanged(true),pcalced(false)
{
  poriginD=0.;
  poriginS[0]=poriginS[1]=poriginS[2]="0";
  porigincartesian=0.;
  pmid=0.;
  for(int i=0;i<9;i++) pboxS[i]="0";
  for(int i=0;i<3;i++) pboxS[4*i]="1";
  pboxD=Matrix3x3::unitMatrix();

  pboxdistS[0]=defaultBoxDistS;
  pboxdistS[1]=defaultBoxDistS;
  pboxdistS[2]=defaultBoxDistS;
  
  pboxdistD[0]=Pair<double,double>(0.,1.);
  pboxdistD[1]=Pair<double,double>(0.,1.);
  pboxdistD[2]=Pair<double,double>(0.,1.);
  


  setDefaultFaces("BOX",0,Conventional);
}

/* ***********************************************************************
 * copy the defining data, but not the calculated data
 * after this the pcalced should be false
 * ***********************************************************************
 */
const Boundary &Boundary::operator=(const Boundary &other)
{
  setCellType(other.pcelltype);
  setCellBasisType(other.pcellbasistype);
  setCentering(other.pcentered);
  setLattice(other.plattice);
  setOrigin(other.poriginS,other.poriginbasis);
  setBox(other.pboxS);
  setBoxDistances(other.pboxdistS);
  setFaces(other.pfaces);
  *(ppolyhedron)=*(other.ppolyhedron);
  pchanged=other.pchanged;
  pcalced=other.pcalced;

  return *this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Boundary::~Boundary()
{
  clean();
  if(ppolyhedron) delete ppolyhedron;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::clean()
{
  ppolyhedron->clear();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Boundary::changed()
{
  return pchanged|ppolyhedron->properties().changed();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setChanged(bool s)
{
  ppolyhedron->properties().setChanged(s);
  pchanged=s;
  pcalced=false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Boundary::calculated()
{
  return pcalced & ppolyhedron->properties().calculated();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setCalculated(bool s)
{
  pcalced=s;
  ppolyhedron->properties().setCalculated(s);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::copyPolyPropertiesFrom(const Boundary *other)
{
  ppolyhedron->properties()=other->ppolyhedron->properties();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
// long Boundary::typeCompressed() const
// {
//   return pcelltype+pcentered*0x100+plattice*0x10000;
  
// }

Boundary::CellType Boundary::cellTypefromCompressed(long t)
{
  long tt=(t & 0xff);
  if(tt==0) return Cell; 
  if(tt==1) return WignerSeitz; 
  if(tt==2) return Cell; 
  if(tt==3) return Cell; 
  return Cell;
}

Boundary::BasisType Boundary::cellBasisTypefromCompressed(long t)
{
  long tt=(t & 0xff);
  if(tt==0) return Primitive; 
  if(tt==2) return Conventional; 
  if(tt==3) return Cartesian; 
  return Conventional;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setCellType(CellType t)
{
  pchanged|=(pcelltype !=t);
  if(pchanged) pcalced=false;
  pcelltype=t;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setCellBasisType(BasisType t)
{
  pchanged|=(pcellbasistype !=t);
  if(pchanged) pcalced=false;
  pcellbasistype=t;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setLattice(Xf::LatticeType t)
{
  pchanged|=(plattice !=t);
  if(pchanged) pcalced=false;
  plattice=t;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setCentering(CenteringType t)
{
  pchanged|=(pcentered !=t);
  if(pchanged) pcalced=false;
  pcentered=t;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setBox(int i,int j,const String &v)
{
  
  double d=1;
  try{ d=String(v).convertToFortranReal(); }catch(Except e){};
  pchanged|=(fabs(pboxD(j)[i]-d)>1.0e-10);
  if(pchanged) pcalced=false;
  
  pboxD.data()[i+3*j]=d;
  pboxS[i+3*j]=v;
}


void Boundary::setBox(const String b[9])
{
  Vec3 v1(b+0);
  Vec3 v2(b+3);
  Vec3 v3(b+6);
  pchanged|=pboxD(0).neq(v1);
  pchanged|=pboxD(1).neq(v2);
  pchanged|=pboxD(2).neq(v3);
  if(pchanged) pcalced=false;


  
  pboxD=Matrix3x3(v1,v2,v3);

  for(int i=0;i<9;i++) pboxS[i]=b[i];
    
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setDefault(Xf::LatticeType lt,Xf::DisplayCellType dct)
{
  setLattice(lt);
  setCellBasisType(Boundary::Conventional);
  setOriginBasis(Boundary::Conventional);
  setCentering(Boundary::None);
  setCellType(Boundary::Cell);
  
  if(lt==Xf::LTReciprocial){
    if(dct==Xf::DCTMainBoundary){
      setCellType(Boundary::WignerSeitz);
      setCentering(Boundary::Centered);
    }
  }

  if(dct==Xf::DCTPrimitive)
    setCellBasisType(Boundary::Primitive);
    
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Face::Face(const String p[3],const String d)
{
  pS[0]=p[0];
  pS[1]=p[1];
  pS[2]=p[2];
  distS=d;
  //  myDebug("d=<%s>",d());
  distD=String(d).convertToFortranReal();
  //  myDebug() << "distD="<< distD;
  pD=Vec3(p);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::removeFace(int pos)
{
  if(pos<0 || pos>= pfaces.size()) return;
  pfaces.removeAt(pos);
  pchanged=true;
  if(pchanged) pcalced =false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setDefaultUserFaces()
{
  String faces[6][3]={ 
    {"-1","0","0"} ,
    {"1","0","0"} ,
    {"0","-1","0"} ,
    {"0","1","0"} ,
    {"0","0","-1"} ,
    {"0","0","1"} 
  };
  userFaces.clear();
  userFaces << Face(faces[0],String("0"));
  userFaces << Face(faces[1],String("1"));
  userFaces << Face(faces[2],String("0"));
  userFaces << Face(faces[3],String("1"));
  userFaces << Face(faces[4],String("0"));
  userFaces << Face(faces[5],String("1"));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setDefaultFaces(const String &mode,const Symmetry *sym
				 ,BasisType b)
{

  if(mode=="BOX" && sym==0){
    pfaces.clear();
    //  if(b==Conventional)
    String faces[6][3]={ 
      {"-1","0","0"} ,
      {"1","0","0"} ,
      {"0","-1","0"} ,
      {"0","1","0"} ,
      {"0","0","-1"} ,
      {"0","0","1"} 
    };
    pfaces << Face(faces[0],String("0"));
    pfaces << Face(faces[1],String("1"));
    pfaces << Face(faces[2],String("0"));
    pfaces << Face(faces[3],String("1"));
    pfaces << Face(faces[4],String("0"));
    pfaces << Face(faces[5],String("1"));
    
    pchanged=true;
    pcalced=false;
    return;
  }else if(mode=="CBOX"){
    String s[9]={ "1", "0", "0", "0", "1", "0", "0", "0", "1" };
    setBox(s);
    pchanged=true;
    pcalced=false;
    return;
  }else if(mode=="USERCELL"){
    pfaces=userFaces;
    pchanged=true;
    pcalced=false;
    return;
  }

  ValueArray<Plane> pla;
  
  Matrix3x3 primcell;
  Matrix3x3 otherconvcell;
  Matrix3x3 convcell;
  Matrix3x3 G;
  
  if( plattice == Xf::LTDirect ){
    primcell=sym->cell();
    convcell=sym->ccell();
    otherconvcell=sym->crcell();
  }else{
    primcell=sym->rcell();
    convcell=sym->crcell();
    otherconvcell=sym->ccell();
  }
//   convcell.print("conv");
//   primcell.print("prim");

  if(mode=="WS"){
  }else if(mode=="WSRSCONVRECCELL"){
    
    // take conventional cell of the other lattice
    // make it reciprocal and construct ws
    // see below
  }else if(mode=="HEX"){
    //    myDebug() << plattice;
    G=convcell;
    if(plattice==Xf::LTDirect || sym->bravais()==SYM_RHOMBOHEDRAL){
      pla<<    Plane(G*Vec3(-1    ,-1./2. , 0),1);
      pla<<    Plane(G*Vec3( 1    , 1./2. , 0),1);
      pla<<    Plane(G*Vec3( 1./2.,-1./2. , 0),1);
      pla<<    Plane(G*Vec3(-1./2., 1./2. , 0),1);
      pla<<    Plane(G*Vec3( 1./2., 1     , 0),1);
      pla<<    Plane(G*Vec3(-1./2.,-1     , 0),1);
      pla<<    Plane(G*Vec3( 0    ,0      ,-1),0);
      pla<<    Plane(G*Vec3( 0    ,0      , 1),1);
    }else{
      pla<<    Plane(G*Vec3( 1./2.  , 1./2. , 0),1);
      pla<<    Plane(G*Vec3(-1./2.  ,-1./2. , 0),1);
      pla<<    Plane(G*Vec3(-1./2.  , 1     , 0),1);
      pla<<    Plane(G*Vec3( 1./2.  ,-1     , 0),1);
      pla<<    Plane(G*Vec3( 1      ,-1./2. , 0),1);
      pla<<    Plane(G*Vec3(-1      , 1./2. , 0),1);
      pla<<    Plane(G*Vec3( 0      , 0     ,-1),0);
      pla<<    Plane(G*Vec3( 0      , 0     , 1),1);
    }
  }else if(mode=="BOX"){
    if(b==Conventional)
      G=convcell;
    else if(b==Primitive)
      G=primcell;
    else 
      G=Matrix3x3::unitMatrix();
    //     G=convcell;


    pla<< Plane(G*Vec3(-1, 0, 0),0);
    pla<< Plane(G*Vec3( 1, 0, 0),1);
    pla<< Plane(G*Vec3( 0,-1, 0),0);
    pla<< Plane(G*Vec3( 0, 1, 0),1);
    pla<< Plane(G*Vec3( 0, 0,-1),0);
    pla<< Plane(G*Vec3( 0, 0, 1),1);
  }else if(mode=="CONVCELL"){
    
    G=convcell;
    G=G.inverseT();
      

    for(int i=0;i<3;i++){
      double gg=G(i)*G(i);
      pla.append(Plane(-G(i)/gg,0));
      pla.append(Plane( G(i)/gg,1));
    }
    
  }else if(mode=="PRIMCELL"){
    
    G=primcell;
    G=G.inverseT();
      

    for(int i=0;i<3;i++){
      double gg=G(i)*G(i);
      pla.append(Plane(-G(i)/gg,0));
      pla.append(Plane( G(i)/gg,1));
    }

  }else if(mode=="CCONVCELL"|| mode== "CPRIMCELL"){
    Matrix3x3 P;
    if(mode=="CCONVCELL") 
      P=convcell;
    else
      P=primcell;


    bool indef;
    if(b==Conventional)
      G=convcell.inverse(indef);
    else if(b==Primitive)
      G=primcell.inverse(indef);
    else 
      G=Matrix3x3::unitMatrix();


    Matrix3x3 M=G*P;


    String s[9];
    for(int i=0;i<9;i++){
      s[i]=String::doubleToRatio(M.data()[i],20,1.e-5);    
    }
    setBox(s);
    pchanged=true;
    pcalced=false;
    return;
  }else{
    myDebug() << "unknown mode:"<<mode;
    exit(0);
  }
  
  double tol=1.0e-6;
  if((mode=="WS") || (mode=="WSRSCONVRECCELL")){

    Matrix3x3 gc=otherconvcell.inverseT();
    if(mode=="WS")
      gc=primcell;

    myDebug() << mode;
    myDebug() << otherconvcell;
    prepWSCellPlanes(gc,pla);
    ppolyhedron->makeCorners(pla,tol,Vlevel(Vlevel::Silent));

//     long oldsize=0;
//     long n=1;
//     QList<Triple> triples;
//     while(true){
//       pla.clear();
//       for(long i1=-n;i1<=n;i1++){ 
// 	for(long i2=-n;i2<=n;i2++){ 
// 	  for(long i3=-n;i3<=n;i3++){ 
// 	    if( i1 || i2 || i3 ){
// 	      const Triple &t=Triple(i1,i2,i3);
// 	      if(!triples.contains(t))
// 		triples.append(t);
// 	    }
// 	  }
// 	}
//       } // i1 
//       //qSort(triples.begin(),triples.end());
//       //myDebug()<<triples;
//       for(long i=0;i<triples.size();i++){
// 	Vec3 R=gc*triples.at(i).v();
// 	pla.append(Plane(R,0.5));
//       }
//       //myDebug() << "pla:"<<n<<pla.size();
//       //for(long i=0;i<pla.size();i++){
//       //myDebug() << i<<triples.at(i)<<pla.at(i);
//       //}
//       // make innermost corners and planes 
//       ppolyhedron->makeCorners(pla,tol,Vlevel(Vlevel::More));
    
//       //myDebug() << "planes"<<n<<ppolyhedron->planes().size();

//       if(oldsize==ppolyhedron->planes().size() || n>=2) break;
//       //todo: make universal
//       oldsize=ppolyhedron->planes().size();
//       n++;
//     }
    
  }else{
    // make innermost corners and planes 
    ppolyhedron->makeCorners(pla,tol,Vlevel(Vlevel::Silent));
  }

  const ValueArray<Plane> &planes=ppolyhedron->planes();
  pfaces.clear();
  for(int i=0;i<planes.size();i++){
    String s[3];
    const Vec3 &P=planes.at(i).P();
    
    if(b==Conventional)
      G=convcell.inverseT();
    else if(b==Primitive)
      G=primcell.inverseT();
    else 
      G=Matrix3x3::unitMatrix();

    Vec3 v=P*G;

    s[0]=String::doubleToRatio(v[0],20,1.e-5);
    s[1]=String::doubleToRatio(v[1],20,1.e-5);
    s[2]=String::doubleToRatio(v[2],20,1.e-5);
    String D=String::doubleToRatio(planes.at(i).D(),20,1.e-5);
    pfaces.append(Face(s,D));

  }
  //
  pchanged=true;
  pcalced=false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::insertFace(long i,const Face &face)
{
  if(i<0) return ;

  long where=i+1;
  if( i>= pfaces.size()) where=pfaces.size();
  
  //myDebug() << "insert " << where;
  pfaces.insert(where,face);

  pchanged=true;
  if(pchanged) pcalced =false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::modifyFace(int i,int which,const String &s)
{
  //  myDebug() << "Boundary::modifyFace" << i<<which<<s();
  if(i<0 || i>= pfaces.size()) return;
  
  if(which<3){
    pchanged |= (pfaces[i].pS[which] != s);
    pfaces[i].pS[which]=s;
    try{
      pfaces[i].pD[which]=String(s).convertToFortranReal();
    }catch(Except e){};
  }
  else if(which==3){
    pchanged |= (pfaces[i].distS != s);
    pfaces[i].distS=s;
    try{
      pfaces[i].distD=String(s).convertToFortranReal();
    }catch(Except e){};
  }

  if(pchanged) pcalced=false;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setFaces(const ValueArray<Face> &f)
{
  pchanged |= (pfaces.size() != f.size());
  for(int i=0;i<MyMath::min<long>(f.size(),pfaces.size());i++){
    pchanged |= (pfaces[i].pS[0] != f.at(i).pS[0]);
    pchanged |= (pfaces[i].pS[1] != f.at(i).pS[1]);
    pchanged |= (pfaces[i].pS[2] != f.at(i).pS[2]);

    pchanged |= (pfaces[i].distS != f.at(i).distS);
  }



  if(pchanged) pcalced=false;

  pfaces.clear();  
  for(int i=0;i<f.size();i++){
    pfaces.append(f.at(i));
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setBoxDistance(int i,int fs,const String &s)
{
  double d0=1;
  try{
    d0=String(s).convertToFortranReal();
  }catch(Except e){};

  if(fs==0){
    pchanged|=(fabs(pboxdistD[i].v1-d0)>1.0e-10);
    pboxdistD[i].v1=d0;
    pboxdistS[i].v1=s;
  }else{
    pchanged|=(fabs(pboxdistD[i].v2-d0)>1.0e-10);
    pboxdistD[i].v2=d0;
    pboxdistS[i].v2=s;
  }
  if(pchanged) pcalced=false;

  

}
void Boundary::setBoxDistances(const Pair<String,String> d[3])
{
  
  for(int i=0;i<3;i++){
    double d0=String(d[i].v1).convertToFortranReal();
    double d1=String(d[i].v2).convertToFortranReal();
  
    pchanged|=(fabs(pboxdistD[i].v1-d0)>1.0e-10);
    pchanged|=(fabs(pboxdistD[i].v2-d1)>1.0e-10);

    pboxdistD[i].v1=d0;
    pboxdistD[i].v2=d1;
  }
  
  if(pchanged) pcalced=false;

  pboxdistS[0]=d[0];
  pboxdistS[1]=d[1];
  pboxdistS[2]=d[2];
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::setOrigin(const String a[3],BasisType t)
{
  Vec3 pa(a);
  pchanged|=(poriginbasis !=t);
  pchanged|=poriginD.neq(pa);
  if(pchanged) pcalced=false;


  poriginbasis=t;

  poriginD=pa;
  poriginS[0]=a[0];
  poriginS[1]=a[1];
  poriginS[2]=a[2];
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Boundary::PTHeaderToFile(FILE *FH,const char *prefix
				      ,const char *tail)
{
  fprintf(FH,
	  "%1$sstruct{ \n"
	  "%1$s      struct {int type;char[*] description;} cell_type;\n"
	  "%1$s      struct {int type;char[*] description;} cell_basis_type;\n"
	  "%1$s      struct {int type;char[*] description;} cell_centering;\n"
	  "%1$s      struct {real shift[3];\n"
	  "%1$s              struct {int type;char[*] description;} basis;\n"
	  "%1$s      }origin;\n"
	  "%1$s      struct {\n"
	  "%1$s              struct {real V1[3];real V2[3];real V3[3];}vectors;\n"
	  "%1$s              struct {real d0;real d1;}distances[3];\n"
	  "%1$s      }box;\n"
	  "%1$s      struct {real P[3];real dist;}faces[*];\n"
	  "%1$s}boundary%2$s",prefix,tail);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::PTBodyToFile(FILE *FH,const char *prefix,const char *tail)
{
  //  NSF("Boundary::symPTBodyToFile");
  //  myDebug() << nsubfunc;

  String s=String(prefix)+"       ";
  const char *ps=s();
  fprintf(FH,"%s{\n",prefix);
  fprintf(FH,"%s{%d,\"%s\"},\n",ps,pcelltype,pcelltypename[pcelltype]);
  
  fprintf(FH,"%s{%d,\"%s\"},\n",ps,pcellbasistype
	  ,pcellbasistypename[pcellbasistype]);
  fprintf(FH,"%s{%d,\"%s\"},\n",ps,pcentered,pcenteringtypename[pcentered]);
  fprintf(FH,"%s{{%s,%s,%s},{%d,\"%s\"}},\n",ps
	  ,poriginS[0](),poriginS[1](),poriginS[2]()
	  ,poriginbasis,pcellbasistypename[poriginbasis]);
  fprintf(FH,"%s{{{%s,%s,%s},\n",ps,pboxS[0](),pboxS[1](),pboxS[2]());
  fprintf(FH,"%s{%s,%s,%s},\n",ps,pboxS[3](),pboxS[4](),pboxS[5]());
  fprintf(FH,"%s{%s,%s,%s}},\n",ps,pboxS[6](),pboxS[7](),pboxS[8]());
  fprintf(FH,
	  "%s{ {%s,%s},{%s,%s},{%s,%s} }\n"
	  "%s},\n"
	  ,ps
	  ,pboxdistS[0].v1() ,pboxdistS[0].v2()
	  ,pboxdistS[1].v1() ,pboxdistS[1].v2()
	  ,pboxdistS[2].v1() ,pboxdistS[2].v2()
	  ,ps
	  );

  fprintf(FH,"%s{\n",ps);
  for(int i=0;i<pfaces.size();i++){
    fprintf(FH,"%s{{%s,%s,%s},%s}"
	    ,ps
	    ,pfaces.at(i).pS[0]()
	    ,pfaces.at(i).pS[1]()
	    ,pfaces.at(i).pS[2]()
	    ,pfaces.at(i).distS() );

    if(i<pfaces.size()-1) fprintf(FH,",\n"); 
    else fprintf(FH,"}\n");
  }
  
  fprintf(FH,"%s}%s",prefix,tail);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::readPTSectionFromFile(const Dobj &root)
{
  try{
    CellType t_type=(CellType)root("cell_type.type").valueL();
    setCellType(t_type);
  }catch(Except e){};

  try{
    BasisType t_type=
      (BasisType)root("cell_basis_type.type").valueL();
    setCellBasisType(t_type);
  }catch(Except e){};

  try{
    CenteringType t_type=
      (CenteringType)root("cell_centering.type").valueL();
    setCentering(t_type);
  }catch(Except){};

  String t_dg[3];
  Boundary::BasisType t_shiftbasis=Conventional;
  try{
    t_dg[0]=t_dg[1]=t_dg[2]="0";
    root("origin.shift").getArray(3,t_dg);
  }catch(Except e){};

  try{
    t_shiftbasis=
      (Boundary::BasisType)(root("origin.basis.type").valueL());
  }catch(Except e){};
  setOrigin(t_dg,t_shiftbasis);

  String t_box[9]={"1","0","0","0","1","0","0","0","1"};
  try{
    root("box.vectors.V1").getArray(3,t_box+0);
    root("box.vectors.V2").getArray(3,t_box+3);
    root("box.vectors.V3").getArray(3,t_box+6);
  }catch(Except e){};
  setBox(t_box);

  Pair<String,String> t_boxdist[3]={
    defaultBoxDistS, defaultBoxDistS, defaultBoxDistS };
  try{
    for(int i=0;i<3;i++){
      t_boxdist[i].v1=root("box.distances")[i]("d0")();
      t_boxdist[i].v2=root("box.distances")[i]("d1")();
    }
  }catch(Except e){};

  setBoxDistances(t_boxdist);

  try{
    Dobj d=root("faces");
    int nfaces=d.size();
    pfaces.clear();
    for(int i=0;i<nfaces;i++){
      String P[3];
      d[i]("P").getArray(3,P);
      String dist=d[i]("dist")();
      insertFace(i,Face(P,dist));
    }
  }catch(Except e){};


}
/* ***********************************************************************
 * lattice indicess (hkl), which span a box, enclosing  a certain sphere 
 * of radius rmax centered at mid
 * ***********************************************************************
 */

void Boundary::hklEnclSphere(const Vec3 mid,double radius
			     ,const Matrix3x3 &rcell
			     ,long h1[2],long h2[2],long h3[2])
{
  //  NSF("Boundary::hklEnclSphere");

  double tol=1.0e-2;
  
  double ag=mid*rcell(0);
  double rg=radius*norm(rcell(0));



  h1[0]=(long)floor(ag-rg-tol);
  h1[1]=(long)ceil(ag+rg+tol);

  ag=mid*rcell(1);
  rg=radius*norm(rcell(1));



  h2[0]=(long)floor(ag-rg-tol);
  h2[1]=(long)ceil(ag+rg+tol);

  ag=mid*rcell(2);
  rg=radius*norm(rcell(2));


  h3[0]=(long)floor(ag-rg-tol);
  h3[1]=(long)ceil(ag+rg+tol);

}

/* ***********************************************************************
 * lattice indicess (hkl), which span a box, enclosing  a certain sphere 
 * of radius rmax centered at mid-tau limited by predefined boundaries
 * ***********************************************************************
 */

void Boundary::hklEnclSphere(const Vec3 &tau,
			     const Matrix3x3 &rcell,
			     long h1[2],long h2[2],long h3[2])const
{
  //  NSF("Boundary::hklEnclSphere");

  double tol=1.0e-2;

  Vec3 mid=pmid-tau;
  
  double ag=mid*rcell(0);
  double rg,rg0=pradius*norm(rcell(0));

  rg=MyMath::max(-rg0,pRADxGneg[0]); //rg=-rg0;
  h1[0]=(long)floor(ag+rg-tol);
  rg=MyMath::min(rg0,pRADxGpos[0]); //rg=rg0;
  h1[1]=(long)ceil(ag+rg+tol);

  ag=mid*rcell(1);
  rg=pradius*norm(rcell(1));



  rg=MyMath::max(-rg0,pRADxGneg[1]); //rg=-rg0;
  h2[0]=(long)floor(ag+rg-tol);
  rg=MyMath::min(rg0,pRADxGpos[1]); //rg=rg0;
  h2[1]=(long)ceil(ag+rg+tol);

  ag=mid*rcell(2);
  rg=pradius*norm(rcell(2));


  rg=MyMath::max(-rg0,pRADxGneg[2]); //rg=-rg0;
  h3[0]=(long)floor(ag+rg-tol);
  rg=MyMath::min(rg0,pRADxGpos[2]); //rg=rg0;
  h3[1]=(long)ceil(ag+rg+tol);

}


/* ***********************************************************************
 * check whether x is within boundary
 * ***********************************************************************
 */
bool Boundary::isInBoundary(double x,double y,double z,double tol) const
{
  const ValueArray<Plane> &planes=ppolyhedron->planes();
  for(long ip=0;ip<planes.size();ip++){
    const Vec3 &n=planes.at(ip).n();
    if(x*n[0]+y*n[1]+z*n[2]-planes.at(ip).d() >= tol){
      // we are out
      return false;
    }
  }
  return true;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Boundary::isInBoundary(const Vec3 &x,double tol)const
{
  const ValueArray<Plane> &planes=ppolyhedron->planes();
  bool inSphere=  norm(x-pmid)<=pradius+tol;
  for(long ip=0;ip<planes.size();ip++){
    if(x*planes.at(ip).n()-planes.at(ip).d() >= tol){
      // we are out
      return false;
    }
  }
  return inSphere;
}




/* ***********************************************************************
 * calc info about cell boundary
 * start with set np planes search innermost polyhedron
 * ***********************************************************************
 */
String Boundary::makeBoundaryCell(const Symmetry *sym,double tol,const Vlevel &vlevel)
{
  NSF("Boundary::makeBoundaryCell");

  
  //myDebug() << nsubfunc;

  ValueArray<Plane> pla;
  initCell(pla,sym);

  
//   for(int i=0;i<pla.size();i++)  
//     myDebug() << "plane=" << pla[i].P().printS()() << pla[i].D();

  // make innermost corners and planes 
  ppolyhedron->makeCorners(pla,tol,vlevel);
  

  
  // midpoint and circumsphere 
  if(sym->structureType() != Xf::STMolecule){
    calcCellSphere();
    calcCellExtend(sym->rcell());
  }
  //  myDebug() << "radius="<< pradius;

  pcalced=true;
  pchanged=false;

  return reportBoundary(vlevel);
}

/* ***********************************************************************
 * make init for creation of primitive or wigner seitz cell
 * ***********************************************************************
 */

void Boundary::initCell(ValueArray<Plane> &pla,const Symmetry *sym)
{
  //  NSF("Boundary::initCell");

  Matrix3x3 convcell;
  Matrix3x3 primcell;
  Matrix3x3 box;
  Matrix3x3 G;

  //myDebug() << nsubfunc;

  if(sym->structureType()==Xf::STMolecule){

    pradius=1.;
    pmid=Vec3();

    const ValueArray<Site> &sites=sym->sites();
    long nsite=sites.size();
    
    long isa=0;
    double r=sites.at(isa).nndist()*0.25;
    double xmin=sites.at(isa).tauD()[0]-r;
    double xmax=sites.at(isa).tauD()[0]+r;

    double ymin=sites.at(isa).tauD()[1];
    double ymax=sites.at(isa).tauD()[1];

    double zmin=sites.at(isa).tauD()[2];
    double zmax=sites.at(isa).tauD()[2];
    
    for(int isa=0;isa<nsite;isa++){
      double r=sites.at(isa).nndist()*0.25;
      xmin=MyMath::min<double>(xmin,sites.at(isa).tauD()[0]-r);
      xmax=MyMath::max<double>(xmax,sites.at(isa).tauD()[0]+r);

      ymin=MyMath::min<double>(ymin,sites.at(isa).tauD()[1]-r);
      ymax=MyMath::max<double>(ymax,sites.at(isa).tauD()[1]+r);

      zmin=MyMath::min<double>(zmin,sites.at(isa).tauD()[2]-r);
      zmax=MyMath::max<double>(zmax,sites.at(isa).tauD()[2]+r);

      pmid+=sites.at(isa).tauD();
    }
    if(nsite){
      pmid/=nsite;
      // enclosing radius 
      pradius=0;
      for(long isa=0;isa<nsite;isa++){
	double r=sites.at(isa).nndist()*0.25;
    	pradius=MyMath::max<double>(pradius,norm(pmid-sites.at(isa).tauD())+r);
      }
      
    }
    if(!pradius) pradius=1.;

    pla.append(Plane( Vec3( 1,0,0),xmax));
    pla.append(Plane( Vec3(-1,0,0),-xmin));

    pla.append(Plane( Vec3(0, 1,0),ymax));
    pla.append(Plane( Vec3(0,-1,0),-ymin));

    pla.append(Plane( Vec3(0,0, 1),zmax));
    pla.append(Plane( Vec3(0,0,-1),-zmin));
    


    return;
  }

  // crystal branch
  if( plattice == Xf::LTDirect ){
    //    g=sym->rcell();
    primcell=sym->cell();
    convcell=sym->ccell();
    box=sym->cell();
  }else{
    //g=sym->cell();
    primcell=sym->rcell();
    convcell=sym->crcell();
    box=sym->rcell();
  }


  //myDebug() << "pcelltype="<<pcelltype;

  Matrix3x3 g2=convcell;

  if(pcelltype == Cell) {

    switch (pcellbasistype) {
    case Cartesian: 
      g2=Matrix3x3::unitMatrix();
    case Conventional: 
      // create reciprocial of conventional cell 
      box=g2;
    case Primitive:
      
      box=box*pboxD;
      
      G=box.inverseT();
      
      
      
      if(pcentered == Centered ){
	double gg=G(0)*G(0);
	double d0=MyMath::min<double>(pboxdistD[0].v2,pboxdistD[0].v1);
	double d1=MyMath::max<double>(pboxdistD[0].v2,pboxdistD[0].v1);
	double d=(d1-d0)*0.5;
	pla.append(Plane( G(0)/gg,d));
	pla.append(Plane(-G(0)/gg,d));
	gg=G(1)*G(1);
	d0=MyMath::min<double>(pboxdistD[1].v2,pboxdistD[1].v1);
	d1=MyMath::max<double>(pboxdistD[1].v2,pboxdistD[1].v1);
	d=(d1-d0)*0.5;
	pla.append(Plane( G(1)/gg,d));
	pla.append(Plane(-G(1)/gg,d));
	gg=G(2)*G(2);
	d0=MyMath::min<double>(pboxdistD[2].v2,pboxdistD[2].v1);
	d1=MyMath::max<double>(pboxdistD[2].v2,pboxdistD[2].v1);
	d=(d1-d0)*0.5;
	pla.append(Plane( G(2)/gg,d));
	pla.append(Plane(-G(2)/gg,d));
      }else{
	for(int i=0;i<3;i++){
	  double gg=G(i)*G(i);
	  pla.append(Plane( G(i)/gg,pboxdistD[i].v2));
	  pla.append(Plane(-G(i)/gg,-pboxdistD[i].v1));
	}
      }
      
      break;
      
    default:;
      printf("initcell: the cell basis type flag is not implemented\n"); 
      exit(1);
    }
    
  }else if(pcelltype == WignerSeitz){
    
    prepWSCellPlanes(primcell,pla);


  }else if(pcelltype == Faces){

    G=Matrix3x3::unitMatrix();
    if(pcellbasistype==Conventional) G=convcell;
    if(pcellbasistype==Primitive)    G=primcell;
    

    for(int i=0;i<pfaces.size();i++){
//       myDebug() <<i
// 	       << pfaces.at(i).pS[0]()
// 	       << pfaces.at(i).pS[1]()
// 	       << pfaces.at(i).pS[2]()
// 	       << pfaces.at(i).distS();
      Vec3 R=G*pfaces.at(i).pD;
      pla.append(Plane(R,pfaces.at(i).distD));

    }


  }// pcelltype


  G=Matrix3x3::unitMatrix();
  if(poriginbasis == Conventional)    G=convcell;
  if(poriginbasis == Primitive)       G=primcell;
  if(poriginbasis == Box)             G=box;
  

  
  porigincartesian=G*poriginD;
  

  for(int i=0;i<pla.size();i++) {
    double d=porigincartesian*pla[i].n()/norm(pla[i].P());
    pla[i]=Plane(pla[i].P(),pla[i].D()+d);
  }
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Boundary::getBoxCell(Vec3 &origin,Matrix3x3 &cell,const Symmetry *sym)const
{
  //  NSF("Boundary::boxCell");

  Matrix3x3 convcell;
  Matrix3x3 primcell;

  //myDebug() << nsubfunc;

  
  // crystal branch
  if( plattice == Xf::LTDirect ){
    //    g=sym->rcell();
    primcell=sym->cell();
    convcell=sym->ccell();
    cell=sym->cell();
  }else{
    //g=sym->cell();
    primcell=sym->rcell();
    convcell=sym->crcell();
    cell=sym->rcell();
  }


  //myDebug() << "pcelltype="<<pcelltype;

  Matrix3x3 g2=convcell;
  Matrix3x3 box;

  origin=Vec3();

  if(pcelltype == Cell) {

    switch (pcellbasistype) {
    case Cartesian: 
      g2=Matrix3x3::unitMatrix();
    case Conventional: 
      // create reciprocial of conventional cell 
      cell=g2;
    case Primitive:
      
      box=cell;
      box=box*pboxD;
      cell=cell*pboxD;
      
      {
	double d0=MyMath::min<double>(pboxdistD[0].v2,pboxdistD[0].v1);
	double d1=MyMath::max<double>(pboxdistD[0].v2,pboxdistD[0].v1);
	double d=(d1-d0);
	if(pcentered != Centered ){
// 	  origin-=cell(0)*(d1+d0)*0.5;
// 	}else{
	  origin+=cell(0)*d0;
	}
	cell.scaleColumn(0,d);
	d0=MyMath::min<double>(pboxdistD[1].v2,pboxdistD[1].v1);
	d1=MyMath::max<double>(pboxdistD[1].v2,pboxdistD[1].v1);
	d=(d1-d0);
	if(pcentered != Centered ){
// 	  origin-=cell(1)*(d1+d0)*0.5;
// 	}else{
	  origin+=cell(1)*d0;
	}
	cell.scaleColumn(1,d);
	d0=MyMath::min<double>(pboxdistD[2].v2,pboxdistD[2].v1);
	d1=MyMath::max<double>(pboxdistD[2].v2,pboxdistD[2].v1);
	d=(d1-d0);
	if(pcentered != Centered ){
// 	  origin-=cell(2)*(d1+d0)*0.5;
// 	}else{
	  origin+=cell(2)*d0;
	}
	cell.scaleColumn(2,d);
      }    	
      origin=origin*pboxD;
      if(pcentered == Centered ){
	origin-=(cell(0)+cell(1)+cell(2))*0.5;
      }
      
      
      
      break;
      
    default:;
      printf("boxCell: the cell basis type flag is not implemented\n"); 
      exit(1);
    }
    
  }else{
    cell=Matrix3x3();
    return;
  }// pcelltype

  Matrix3x3 G=Matrix3x3::unitMatrix();
  if(poriginbasis == Conventional)    G=convcell;
  if(poriginbasis == Primitive)       G=primcell;
  if(poriginbasis == Box)             G=box;
  

  origin+=G*poriginD;
  
  
}



/* ***********************************************************************
 * calc midpoint of and radius including  cell
 * ***********************************************************************
 */

void Boundary::calcCellSphere()
{
  // midpoint 
  // for all planes 

  pradius=1.;
  
  pmid=Vec3();
  
  const ValueArray<Vec3> &pverts=ppolyhedron->vertices();

  for(long iv=0;iv<pverts.size();iv++) pmid+=pverts.at(iv);
  
  if(!pverts.size())return ;

  pmid/=pverts.size();

  // enclosing radius 
  pradius=0;
  for(long iv=0;iv<pverts.size();iv++)
    pradius=MyMath::max<double>(pradius,norm(pmid-pverts.at(iv)));
  
  if(!pradius) pradius=1.;
  return;
}

/* ***********************************************************************
 * calc min/max distance of polyhedron corners in the recip latt directions
 * ***********************************************************************
 */
void Boundary::calcCellExtend(const Matrix3x3 &rcell)
{
  const ValueArray<Vec3> &pverts=ppolyhedron->vertices();
  
  pRADxGneg=0.;
  pRADxGpos=0.;
  //myDebug()<<"pmid="<<pmid;
  //myDebug()<<"rcell="<<rcell;

  for(long iv=-1;++iv<pverts.size();){
    //myDebug()<<"pverts.at(iv)-mid="<<pverts.at(iv)-pmid;
    if(iv==0){
      double ag=(pverts.at(iv)-pmid)*rcell(0);
      //myDebug()<<"ag0              ="<<ag;
      pRADxGneg[0]=ag;
      pRADxGpos[0]=ag;
      ag=(pverts.at(iv)-pmid)*rcell(1);
      //myDebug()<<"ag1              ="<<ag;
      pRADxGneg[1]=ag;
      pRADxGpos[1]=ag;
      ag=(pverts.at(iv)-pmid)*rcell(2);
      //myDebug()<<"ag2              ="<<ag;
      pRADxGneg[2]=ag;
      pRADxGpos[2]=ag;
    }else{
      double ag=(pverts.at(iv)-pmid)*rcell(0);
      //myDebug()<<"ag0              ="<<ag;
      pRADxGneg[0]=MyMath::min<double>(pRADxGneg[0],ag);
      pRADxGpos[0]=MyMath::max<double>(pRADxGpos[0],ag);
      ag=(pverts.at(iv)-pmid)*rcell(1);
      //myDebug()<<"ag1              ="<<ag;
      pRADxGneg[1]=MyMath::min<double>(pRADxGneg[1],ag);
      pRADxGpos[1]=MyMath::max<double>(pRADxGpos[1],ag);
      ag=(pverts.at(iv)-pmid)*rcell(2);
      //myDebug()<<"ag2              ="<<ag;
      pRADxGneg[2]=MyMath::min<double>(pRADxGneg[2],ag);
      pRADxGpos[2]=MyMath::max<double>(pRADxGpos[2],ag);
    }
  }
  // myDebug()<<"pRADxGneg="<<pRADxGneg;
  // myDebug()<<"pRADxGpos="<<pRADxGpos;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String Boundary::reportBoundary(const Vlevel &vlevel)
{
  const ValueArray<Plane> &planes=ppolyhedron->planes();
  const ValueArray<ValueArray<long> > &polys=ppolyhedron->polyIndices();
  const ValueArray<Vec3> &pverts=ppolyhedron->vertices();
  
  const char *line70="----------------------------------------------"
    "------------------------\n";

  String s="\n";
  s+=line70;
  s+="                     Report Boundary Creation   \n";
  s+=line70;
  
  s+=Sprintf("Faces            : %ld\n",planes.size());
  s+="No.                X               Y               Z         "
    "   distance\n";
  for(long i=0;i<planes.size();i++){
    s+=Sprintf("%3ld :    %s % 15.5f\n",i+1
	       ,planes.at(i).n().printS("% 15.5f")(),planes.at(i).d());
  }

  if(vlevel >= Vlevel::More ){
    s+="Vertices : \n";
    for(long ip=0;ip<planes.size();ip++){
      s+=Sprintf("Plane %ld :\n",ip+1);
      for(long ic=0;ic<polys.at(ip).size();ic++){
	s+=Sprintf("\t%3ld: %s\n",ic+1
		   ,pverts.at(polys.at(ip).at(ic)).printS("% 15.5f")());
      }
    }
    
    

    s+=line70;
    s+="Vertex-Polygon Representation\n";
    s+=Sprintf("\nVertices : %ld\n",pverts.size());
    s+="No.                X               Y               Z         \n";
    for(long iv=0;iv<pverts.size();iv++){
      s+=Sprintf("%3ld :    %s \n"
		 ,iv+1,pverts.at(iv).printS("% 15.5f")());
      
    }
    

    s+=Sprintf("\nPolygons : %ld\n",planes.size());
    s+="No. corners     \n";
    for(long ip=0;ip<planes.size();ip++){
      String t=Sprintf("%3ld :",ip+1);
      for(long ic=0;ic<polys.at(ip).size();ic++){
	t+=Sprintf(" %3ld",polys.at(ip).at(ic));
      }
      s+=t+"\n";
    }
  }
  s+="\n";
  s+=line70;
  s+="\n";

  return s;
}
