/* @(#)Cluster.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef CLUSTER_H
#define CLUSTER_H

#include "MyQRgb.hh"
#include "FString.hh" 
#include "Atom.hh" 
#include "Bond.hh" 
#include "PtrArray.hh" 
#include "Polyhedron.hh" 

class Boundary;
class Status;


namespace XFPLO{
  class Symmetry;
  class BondDefs;



/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Cluster
{
public:
  Cluster();
  ~Cluster();

  bool calculated(){return pcalced;};
  void setCalculated(bool s){pcalced=s;};

  double radius()const{return pradius;};
  Vec3 midPoint()const{return pmid;};

  
  String create(const Symmetry *sym,const Boundary *bound
		 ,const BondDefs *bonddefs
		 ,Status *s);

  long numBonds()const {return pbonds.size();};
  const Bond &bond(long i) const {return pbonds.at(i);};

  long numPolys()const {return ppolyhedrons.size();};
  const AtomPolyhedron *polyhedron(long i) const {return ppolyhedrons.at(i);};
  
  long numAtoms()const {return patoms.size();};
  const Atom &atom(long i) const {return patoms.at(i);};

/*   Atom findBySiteIndex1(long isa){ */
/*     for(int i=0;i<patoms.size();i++){ */
/*       if(patoms[i].site==isa){ */
/* 	return patoms[i]; */
/*       } */
/*     } */
/*     return Atom(); */
/*   }; */

  

  long findByPosition(const Vec3 &p){
    long N=patoms.size();
    for(int i=0;i<N;i++){
      if(patoms[i].tauD.eq(p)){
	return i;
      }
    }
    return -1;
  };

  long findByPositionSite(const Vec3 &p,long site)const{
    long N=patoms.size();
    for(int i=0;i<N;i++){
      const Atom &at=patoms.at(i);
      if(at.site!=site)continue;
      if(at.tauD.eq(p)){
	return i;
      }
    }
    return -1;
  };

  long findByPosition(const Vec3 &p,long nn){
    for(int i=0;i<nn;i++){
      if(patoms[i].tauD.eq(p)){
	return i;
      }
    }
    return -1;
  };

  long findByPosition(const Vec3 &p,long n0,long nn){
    for(int i=n0;i<nn;i++){
      if(patoms[i].tauD.eq(p)){
	return i;
      }
    }
    return -1;
  };


private:

  void createBonds(const Symmetry *sym
		   ,const BondDefs *bonddefs
		   ,Status *status
		   ,Xf::LatticeType latticetype
		   ,double alloverscale);


  String reportCluster();

private:
  ValueArray<Atom> patoms; 
  //  long pnatom; /* the natom first atoms are in the boundary, the
  //		 rest is added due to bonds ...
  //	       */
  
  ValueArray<Bond> pbonds;
  PtrArray<AtomPolyhedron> ppolyhedrons;
  //
  Vec3 pmid;
  double pradius;
  //
  bool pcalced;
};

}
#endif // CLUSTER_H
