//todo: relat
//todo: rotation
//todo: combination
/*************************************************************************
 * @(#)Coeffs.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   30 Oct 2010
 *************************************************************************
 */
#include "Except.hh"
#include "Assorted.hh"
#include "Coeffs.hh"
#include "Vec3.hh"
#include "MyMath.hh"
#include "Exception.hh"
#include "MolecularOrbital.hh"
#include "CoeffBasisInfo.hh"
#include "Cmplx.hh"
#include "Interrupt.hh"
#include "MyDebug.hh"

using namespace XFPLO;
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Coeffs::Coeffs()
  :kpoints(0),ekb(0),zz(0),pisready(false)
{
  //  myDebug()<<"Coeffs::Coeffs";
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Coeffs::~Coeffs()
{
  clean();
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Coeffs::clean()
{
  if(zz) delete [] zz; zz=0;
  if(ekb) delete [] ekb; ekb=0;
  if(kpoints) delete [] kpoints; kpoints=0;
  pisready=false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Coeffs::isCoeffFile(const String &fileName)
{
  FILE *FH;
  try{
    FH=openforread(fileName());
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }
  String header;
  getLine(FH,header);

  bool iscoeff=false;
  header.stripBlanks(true);
  if(header.find("type")>=0){
    StringList sl=header.split(":");
    String s=sl[sl.size()-1].stripBlanks();
    if(s=="coeff") iscoeff=true;
    //printf("<%s>\n",s());
  }else{
    // for now we accept all files, which look like coeff files
    //    printf("%s\n",header());

    // doess it have the bweights/coeff header line
    long ldum;
    double ddum;
    int err=sscanf(header(),"# %ld %lg %ld %ld %ld %ld %ld %ld\n"
		   ,&ldum,&ddum,&ldum,&ldum,&ldum,&ldum,&ldum,&ldum)-8;
    //printf("err=%ld\n",err);
    
    if(!err){
      getLine(FH,header);
      header.stripBlanks();
      StringList sl=header.split(" \t");
//       printf("%s %s %s\n",header.subString(1,50)()
// 	     ,sl[1](),sl[2]());
      //header
      if(sl[1]=="band" && sl[2]=="index"){
	iscoeff=true;
      }
    }


  }

  fclose(FH);


  
  return iscoeff;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Coeffs::readFromFile(const String &fname,CoeffBasisInfo &cbi,bool forceload)
{

  if(pisready && fname==pcurrentfile && !forceload){
    return;
  }

  clean();
  

  FILE *FH;
  try{
    FH=openforread(fname);
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }

  int err;
  long ldum,nvdim;
  String header;
  getLine(FH,header);
  //  printf("%s\n",header());
  header.stripBlanks(true);
  if(header.find("type")>=0){
    getLine(FH,header);
    //printf("%s\n",header());
    err=fscanf(FH,"# %ld %lg %ld %ld %ld %ld %ld %ld\n",&ldum,&eff,&nkp
	       ,&nvdim,&nband,&nspin,&ilower,&iupper)-8;
  }else{
    err=sscanf(header(),"# %ld %lg %ld %ld %ld %ld %ld %ld\n"
	       ,&ldum,&eff,&nkp
	       ,&nvdim,&nband,&nspin,&ilower,&iupper)-8;
  }

  ilower--;
  iupper--;
  if(err){
    throw Exception(Sprintf("Header of file '%s' corrupted?",fname()));
  }
  //printf("nband %ld %ld\n",nband,iupper-ilower+1);

  getLine(FH,header);
  //printf("%s\n",header());
  nvdim=0;
  StringList labels;
  while( 1 ){
    nvdim++;
    int i=42+(nvdim-1)*34;
    if(i+16>header.length()) break;
    labels.append(header.subString(i,i+16).removeTrailingBlanks()());
  }
  nvdim=labels.size();
  //(void)printf("nvdim=%ld\n",nvdim);


  zz=new double[nvdim*nband*nkp*nspin*2];
  ekb=new double[nband*nkp*nspin];
  kpoints=new double[4*nkp];

  try{
    cbi.getCBIFromLabels(labels);
  }catch(Except ex){
    printf("%s\n",ex.msg());
  }

  if(nvdim!=cbi.nvdim())
    throw Exception(Sprintf("Data in '%s'  inconsistent!\n"
			    " nvdim %ld != labels.nvdim %ld"
			    ,fname()
			    ,nvdim,cbi.nvdim()
			    ));

  long subdiv=(MyMath::max<long>(1,nkp/(100/5)));
  long countMax=MyMath::max<long>(1,nkp/subdiv);

  
  bool stopit=false;
  for(long ik=0;ik<nkp;ik++){

    if ( (ik)%subdiv ==0 ){
      if(interrupt.requested()){
	stopit=true;
	break;
      }
      emit progress((int)(ik/subdiv*(100./countMax)));

    }

    for(long ms=0;ms<nspin;ms++){
      String sdum;
      if(!getLine(FH,sdum))
	throw Exception(Sprintf("File '%s' truncated?",fname()));
      sdum.stripBlanks(true);
      
      
      StringList sl=sdum.breakup("dk=");
      if(sl.size()!=2)
	throw Exception(Sprintf("File '%s' corrupted.",fname()));

      sscanf(sl[1](),"%lg",kpoints+4*ik+3);
      sl=sl[0].breakup("k=")[1].stripBlanks().split(" ");
      Vec3 k(sl);
      kpoints[4*ik+0]=k[0];
      kpoints[4*ik+1]=k[1];
      kpoints[4*ik+2]=k[2];
      
      //printf("k=(%s) dk=%12.8g\n",k.printS("%12.8g",",")(),kpoints[4*ik+3]);
      //zz(iz,ib,ik,ms)=zz(iz+nvdim*(ib+nband*(ik+nkp*ms))))
      
      for(long ib=0;ib<nband;ib++){
	String sdum;
	if(!getLine(FH,sdum))
	  throw Exception(Sprintf("File '%s' truncated?",fname()));

	sdum.stripBlanks(true);
	StringList sl=sdum.split(" \t");
	if(sl.size()-2!=nvdim*2)
	  throw Exception(Sprintf("Different number of labels and"
				  " coefficient \n"
				  "columns in file '%s'?",fname()));
	long idum;
	double r,i;
	//printf("size %ld\n",sl.size());
	sscanf(sl[0](),"%ld",&idum);
	sscanf(sl[1](),"%lg",ekb+ib+nband*(ik+nkp*ms));
	//printf("%12ld %12.8g ",idum,ekb[ib+nband*(ik+nkp*ms)]);
	long ind=2;
	long j=2*nvdim*(ib+nband*(ik+nkp*ms));
	for(int iz=0;iz<nvdim;iz++){
	  //try{
	  //printf("iz %ld\n",2*(iz-1)+2+1);
	  sscanf(sl[ind++](),"%lg",&r);
	  sscanf(sl[ind++](),"%lg",&i);
	  //}catch(Except ex) {
	  //throw Exception(QString().sprintf("%s",ex.msg()));  
	  // }
	  zz[j++]=r;
	  zz[j++]=i;
	  //printf("%12.8g %12.8g ",r,i);
	}
	//printf("\n");


      }
    }
  }
  
  fclose(FH);


  if(stopit){
    pisready=false;
    return ;
  }

  pcurrentfile=fname;
  pisready=true;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Coeffs::makeBandWeights(const String &wfname
			     ,MolecularOrbitals &mo
			     ,CoeffBasisInfo &cbi)
{
  // myDebug() <<"Coeffs::makeBandWeights";
  // myDebug() << wfname();

  StringList output;
  mo.setup(cbi,false,output);

  FILE *FH;
  try{
    FH=openforwrite(wfname());
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }



  mo.buildMatrix(Vec3(),cbi);

  double wtol=1.0e-8;
  long nvdim=cbi.nvdim();
  long numweights=mo.numWeights();
  Cmplx *coeffs=new Cmplx[nvdim];
  double *weights=new double[numweights*nband];
  double *wsum=new double[nband];


  
  // write header
  fprintf(FH,"# %5ld%12.7f %10ld %10ld %10ld %10ld %10ld %10ld\n"
	  ,cbi.nsite(),eff,nkp,numweights,nband,nspin,ilower+1,iupper+1);
  fprintf(FH,"#       ik           e(k,n)               ");
  for(long iw=0;iw<numweights;iw++) fprintf(FH,"%-17s",mo.name(iw)());
  fprintf(FH,"\n");

  // prepare status reporting
  long subdiv=(MyMath::max<long>(1,nkp/(100/5)));
  long countMax=MyMath::max<long>(1,nkp/subdiv);
  bool stopit=false;
  
  // k-loop
  for(long ik=0;ik<nkp;ik++){
    if ( (ik)%subdiv ==0 ){
      if(interrupt.requested()){
	stopit=true;
	break;
      }
      emit progress((int)(ik/subdiv*(100./countMax)));
    }
    
    //spin loop
    for(long ms=0;ms<nspin;ms++){


      mo.buildMatrix(Vec3(kpoints+4*ik),cbi);

            
      // make raw band weights
      
      long jkm=nband*(ik+nkp*ms);

      for(long ib=0;ib<nband;ib++){

	for(long i=0;i<nvdim;i++) coeffs[i]=0.;


	double dd=0.;
	long j=2*(nvdim*(ib+jkm));
	for(int iz=0;iz<nvdim;iz++){
	  Cmplx caa=Cmplx(zz[j],zz[j+1]);
	  j+=2;
	  // sum of netto weights
	  dd+=caa.normsq();
	  // netto weights
	  coeffs[iz]=coeffs[iz]+caa;
	}
	wsum[ib]=dd;
	
	//modify
	for(int iw=0;iw<numweights;iw++){
	  Cmplx caa=0.;
	  for(int iz=0;iz<nvdim;iz++){
	    // conjg(U[:,iw])*w[:,ib]
	    caa+=conjg(mo.U(iz,iw))*coeffs[iz];
	  }
	  weights[iw+numweights*ib]=caa.normsq();
	}

      }//ib

      // average over degenerate energies
      if(nband>0){      
	long in=0;
	double e=ekb[in+jkm];
	long in1=in;
	long in2=in;
	for(long in=1;in<nband;in++){
          double d=fabs(e-ekb[in+jkm]);
          if(fabs(e)>1.0){
	    d=d/fabs(e);
	  }
          if(d<wtol){
	    in2=in;
	  }else{
	    // take average of weights for degenerate bands
	    for(long iw=0;iw<numweights;iw++){
	      double cw=0.;
	      for(long ib=in1;ib<=in2;ib++){
		cw+=weights[iw+numweights*ib];
	      }
	      cw/=(in2-in1+1);
	      for(long ib=in1;ib<=in2;ib++){
                weights[iw+numweights*ib]=cw;
	      }
	    }
	    double w=0.;
	    for(long ib=in1;ib<=in2;ib++){
	      w+=wsum[ib];
	    }
	    w/=(in2-in1+1);
	    for(long ib=in1;ib<=in2;ib++){
	      wsum[ib]=w;
	    }
	    e=ekb[in+jkm];
	    in1=in;
	    in2=in;
	  }
	}//in

 	for(long iw=0;iw<numweights;iw++){
	  double cw=0.;
	  for(long ib=in1;ib<=in2;ib++){
	    cw+=weights[iw+numweights*ib];
	  }
	  cw/=(in2-in1+1);
	  for(long ib=in1;ib<=in2;ib++){
	    weights[iw+numweights*ib]=cw;
	  }
	}
	double w=0.;
	for(long ib=in1;ib<=in2;ib++){
	  w+=wsum[ib];
	}
	w/=(in2-in1+1);
	for(long ib=in1;ib<=in2;ib++){
	  wsum[ib]=w;
	}
      }//nband>0
      
      //write weights
      for(long in=0;in<nband;in++){

	
	fprintf(FH,"%16.7f %16.7f",kpoints[4*ik+3],ekb[in+jkm]);
	
	double dd=wsum[in];
	if(fabs(dd)<1.0e-6) dd=1.0;
 	for(long iw=0;iw<numweights;iw++){
	  fprintf(FH," %16.7f",weights[iw+numweights*in]/dd);
	}
	fprintf(FH,"\n");
	
      } //in

      
    }// ms
  }// ik
  delete [] weights;
  delete [] wsum;
  
  fclose(FH);

  if(stopit){
    return ;
  }

  // myDebug() << "done";
}
  
  
  
/*EOF*/ 
