/*************************************************************************
 * @(#)ColorMap.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 Jul 2012
 *************************************************************************
 */

#include "ColorMap.hh"
#include "MyMath.hh"
#include "Parser.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QColor ColorMap::colorFromIndex(double s)const
{
  //myDebug()<<printS();
  int N=pmap.size();
  
  if(N==0) return QColor();
  if(N==1) return QColor::fromHsv(pmap.at(0).rawHue(),
				  pmap.at(0).saturation(),
				  pmap.at(0).value());
  s=MyMath::min<double>(pmap.last().parameter(),MyMath::max<double>(pmap.at(0).parameter(),s));

  for(int k=1;k<N;k++){
    const ColorMapData &cd0=pmap.at(k-1);
    const ColorMapData &cd1=pmap.at(k);
    if(cd0.parameter()<=s && s<=cd1.parameter()){
      return ColorMapData::interpolateNodes(cd0,cd1,s,true);
    }
  }
  return QColor();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QColor ColorMap::ColorMapData
::interpolateNodes(const ColorMapData &c1,const ColorMapData &c2,
		   double ss,bool inspherical)
{
  double s=MyMath::max<double>(0.,MyMath::min<double>(1.,(ss-c1.ps)/(c2.ps-c1.ps)));
  //todo:
  if(inspherical){
    double tol=1.e-3;
    
    double phi1=c1.phue/180.*Pi;
    double phi2=c2.phue/180.*Pi;
    double r1=c1.psat/255.;
    double r2=c2.psat/255.;

    //myDebug()<<"phi12s"<<n1.rawHue()<<n2.rawHue()<<s;
    double x1=r1*cos(phi1),y1=r1*sin(phi1);
    double x2=r2*cos(phi2),y2=r2*sin(phi2);
    //myDebug()<<"v1="<<x1<<y1;
    //myDebug()<<"v2="<<x2<<y2;

    double x=(x2-x1)*s+x1;
    double y=(y2-y1)*s+y1;
    //myDebug()<<"v ="<<x<<y;
    double phi=atan2(y,x)/Pi*180;
    //myDebug()<<"phi0"<<phi;
    while(phi<phi1)phi+=360.;
    while(phi>phi2)phi-=360.;
    
    if(fabs(s)<tol && fabs(phi-phi2) <tol )phi=phi1;
    if(fabs(1.-s)<tol && fabs(phi-phi1) <tol )phi=phi2;
    

    double r=sqrt(x*x+y*y)*255.;
    //myDebug()<<"phi"<<phi<<"r"<<r;


    int rawhue=lRound(phi)%360;
    if(rawhue<0)rawhue+=360;
    int sat=MyMath::max<long>(0,MyMath::min<long>(255,lRound(r)));
    int val=MyMath::max<long>(0,MyMath::min<long>(255,lRound((c2.pval-c1.pval)*s+c1.pval)));
    //myDebug()<<"s="<<s<<"c="<<rawhue<<sat<<val;
    return QColor::fromHsv(rawhue,sat,val);
  }
  
  int rawhue=lRound((c2.rawHue()-c1.rawHue())*s+c1.rawHue());

  int sat=MyMath::max<long>(0,MyMath::min<long>(255,lRound((c2.psat-c1.psat)*s+c1.psat)));
  int val=MyMath::max<long>(0,MyMath::min<long>(255,lRound((c2.pval-c1.pval)*s+c1.pval)));
    
  return QColor::fromHsv(rawhue,sat,val);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String ColorMap::printS()const
{
  String s("ColorMap=[\n");
  for(int i=0;i<pmap.size();i++){
    const ColorMapData &d=pmap.at(i);
    s+=Sprintf("\ts=%g, hue=%d, sat=%d, val=%d%s\n",
	       d.parameter(),d.rawHue(),d.saturation(),
	       d.value(),
	       (i<pmap.size()-1?",":"")
	       );
  }
  s+=Sprintf("]\n");
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String ColorMap::PTHeaderToString()const
{
  
  return String("struct{real parameter;real hue;real sat;real val;}colormap[*]");

}
 /* ***********************************************************************
  *
  * ***********************************************************************
  */
String ColorMap::PTSectionToString()const
{
  String s("{\n");
  for(int i=0;i<pmap.size();i++){
    const ColorMapData &d=pmap.at(i);
    s+=Sprintf("{%g, %d, %d, %d}%s\n",
	       d.parameter(),d.rawHue(),d.saturation(),
	       d.value(),
	       (i<pmap.size()-1?",":"")
	       );
  }
  s+=Sprintf("}\n");
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorMap::readPTSectionFromFile(const Dobj &root)
{

  clear();

  Dobj da=root;
  for(int i=0;i<da.size();i++){
    Dobj dai=da[i];
    append(dai("parameter").valueD(),dai("hue").valueD()
		,dai("sat").valueD(),dai("val").valueD());
  }

}


/*EOF*/ 
