/* @(#)ColorMap.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _COLORMAP_HH
#define _COLORMAP_HH 1

#include "MyQRgb.hh"
#include <QList>
#include "FString.hh"


class Dobj;

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class ColorMap
{
private:
  class ColorMapData
  {
  public:
    ColorMapData(double ss,int h,int s,int v):ps(ss),phue(h),
					      psat(s),pval(v){};
    
    inline double parameter()const{return ps;};
    inline void setParameter(double s){ps=s;};
    inline int rawHue()const{return phue;};
    inline int saturation()const{return psat;};
    inline void setSaturation(int s){psat=s;};
    inline int value()const{return pval;};
    inline void setValue(int v){pval=v;};

    static QColor interpolateNodes(const ColorMapData &c1,
				   const ColorMapData &c2,
				   double s,bool inspherical);
    
  private:
    double ps;
    int phue;
    int psat;
    int pval;
  };

public:
  ColorMap(){};
  ~ColorMap(){};

  inline void clear(){pmap.clear();};

  inline int size()const{return pmap.size();};

  inline const ColorMapData &at(int i)const{return pmap.at(i);};
  inline ColorMapData &operator[](int i){return pmap[i];};

  inline void append(double s,int h,int sa,int v){
    pmap.append(ColorMapData(s,h,sa,v));
  };

  QColor colorFromIndex(double)const;

  String printS()const;
  String PTHeaderToString()const;
  String PTSectionToString()const;
  void readPTSectionFromFile(const Dobj &root);

private:
  QList<ColorMapData> pmap;
};


#endif /* _COLORMAP_HH */

