/*************************************************************************
 * @(#)Connection.cpp
 * author: magru@bono
 * date:   21 Jun 2013
 *************************************************************************
 */

#include "Position.hh"
#include "Except.hh"
#include "Connection.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Connections::Connections()
  :Module("Connections"),pusecounter(1),pdata(0),pfrompositions(false),
   ppositions(0),pconnectiontype(CTYCuboid)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Connections::setInput(const Module *input)
{
  if(input->classname()=="Positions"){
    ppositions=dynamic_cast<const Positions*>(input);
    pisdirty=true;
  }else{
    throw Except("Connections::setInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Connections::removeInput(const Module *input)
{
  //myDebug()<<pclassname<<"removeInput"<<(void*)input<<input->classname();
  // dynamic_cast no longer possible, since called from destructor
  if(input->classname()=="Positions"){
    if(ppositions==input) ppositions=0;
  }else{
    throw Except("Connections::removeInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Connections::setFromPositions(const Positions *pos)
{
  clearData();



  const ProductArray<double> *pa;
  if((pa=dynamic_cast<const ProductArray<double> *>(pos->array()))==0)
    throw Except("Connections::setFromPositions: positions are not a regular");

  long D=pa->dimensions();

  for(int i=-1;++i<D;){
    if((dynamic_cast<const RegularArray<double> *>(pa->array(i)))==0){
      throw Except("Connections::setFromPositions: positions are not regular");
    }
  }
  
  pdata=MeshArray::construct();

  for(int i=-1;++i<D;){
    const RegularArray<double> *ra
      =dynamic_cast<const RegularArray<double> *>(pa->array(i));
      
    //myDebug()<<ra->size();

    ((MeshArray*)pdata)->addArray(PathArray::construct(ra->size()-1));

  }

  pconnectiontype=CTYCuboid;

  //  myDebug()<<"created Connections";
  //pisdirty=true;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Connections::cascadingUpdate(const String &prefix)
{
  if(Module::debugmode) printf("%s%s::cascadingUpdate\n",prefix(),id()());
  Module::cascadingUpdate(prefix+"  ");
  showDirt(prefix,Sprintf("%s must react to",id()()));
  bool ret=false;
  if(dirtyInput("Positions")){
    if(pfrompositions){
      setFromPositions(ppositions);
    }
    sendDirt("changed");
    ret=true;
    resetDirt("Positions");
  }
  if(pisdirty){
    sendDirt("changed");
    ret=true;
  }
  resetDirt();
  if(Module::debugmode) printf("%sret=%d\n",prefix(),ret);
  return ret;
}




/*EOF*/ 
