/* @(#)Connection.hh
 * author: magru@bono
 */

#ifndef _CONNECTION_HH
#define _CONNECTION_HH 1

#include "Array.hh"
#include "Module.hh"
#include "MyDebug.hh"


namespace XFPLO{

class Positions;

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class Connections:public Module{
private:
  Connections();
  Connections(const Connections&);
  Connections &operator=(const Connections &);
  virtual ~Connections(){
    clearData();
  }
public: 

  enum ConnectionType{CTYSimplex=1,CTYCuboid=2};

  static Connections *construct(){
      return new Connections();
    }

  static void destroy(Connections *p){
    if((--(p->pusecounter))==0) delete p;
  }

  void setConnectionType(ConnectionType t){pconnectiontype=t;}
  ConnectionType connectionType()const{return pconnectiontype;}

  void setPositionDependency(bool d){pfrompositions=d;}


  void setupArray(const ValueArray<long> &shape){
    clearData();
    pfrompositions=false;
    pconnectiontype=CTYSimplex;
    pdata=BaseArray<long>::construct(shape);

  }


  BaseArray<long> *array(){return pdata;}
  const BaseArray<long> *array()const{return pdata;}


  virtual bool cascadingUpdate(const String &prefix);

private:

  void setFromPositions(const Positions *);

  inline void increaseUse(){pusecounter++;}
  void clearData(){
    if(pdata){
      pdata->destroy(pdata);
      pdata=0;
    }
  }

protected:
  virtual void setInput(const Module *input);
  virtual void removeInput(const Module *);
private:
  long pusecounter;
  BaseArray<long> *pdata;
  bool pfrompositions;
  const Positions *ppositions;
  ConnectionType pconnectiontype;
};

}
#endif /* _CONNECTION_HH */

