/*************************************************************************
 * @(#)DMDoc.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Mar 2010
 *************************************************************************
 */

#include <stdio.h>
#include <QMessageBox>
#include <QDir>
#include "DMDoc.hh"
#include "FString.hh"
#include "Assorted.hh"
#include "Parser.hh"
#include "Except.hh"
#include "MyMath.hh"
#include "Settings.hh"
#include "qt5compat.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
DMDoc::DMDoc(const String &mode)
  :pmode(mode),pfilename("=.dmat_init"),pfrel(false),
   pglobalspinzaxis(Vec3(0.,0.,1.))
{
  

  
  if(mode=="dmat"){
    pfilename="=.dmat_init";
  }else if(mode=="vmat"){
    pfilename="=.vmat";
  }else{
    QMessageBox::warning(0,"Reading..."
			 ,Sprintf("DMDoc:: unknown mode '%s'\n"
				  ,mode())()
			 ,QMessageBox::Yes );
    exit(1);
  }



}
 
/* ***********************************************************************
 *
 * ***********************************************************************
 */
DMDoc::~DMDoc()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool DMDoc::load()
{
  //myDebug() << "DMDoc::load()" ;

  FILE *FI;
  try{
    FI=openforread(pfilename);
  }catch(Except &ex){
    QMessageBox::warning(0,"Reading...",ex.msg(),QMessageBox::Yes );
    return false;
  }

  String s;
  int nline=0;
  while (getLine(FI,s)){
    nline++;
    
    s=s.stripBlanks(true);
    if(s.length()==0) continue; // skip empty lines
    

    // split into words
    StringList sl=s.split(" ");
    // skip comments
    if( sl[0]=="#"){ continue; }
    //myDebug() << s();
    
    // read element
    if(sl[0].length()==1)
      sl[0]+=" ";
    Occu occu;
    try{
      if(sl[0].length()!=2) throw Except();
      occu.setEl(sl[0]);
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading..."
			   ,Sprintf("Line %d: expect element as "
				    "1st word ",nline)()
			   ,QMessageBox::Yes );
      return false;
    }
    


    // read site
    try{
      occu.setSite(sl[1].convertToLong());
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading..."
			   ,Sprintf("Line %d: expect integer as "
				    "second word ",nline)()
			   ,QMessageBox::Yes );
      return false;
    }

    // read orbital
    String orbital;
    try{ 
      orbital=sl[2];
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading..."
			   ,Sprintf("Line %d: expect orbital as "
				    "third word ",nline)()
			   ,QMessageBox::Yes );
      return false;
      
    }
    try{
      long l;
      orbital.orbitalToInt(occu.pnmain,l);
      occu.setL(l);
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading..."
			   ,Sprintf("Line %d: "
				    "third word: %s",nline,ex.msg())()
			   ,QMessageBox::Yes );
      return false;
    }

    // read spin
    long ms;
    try{ 
      String sms=sl[3];
      if(sms=="up"){
	ms=0;
      }else if(sms=="down"){
	ms=1;
      }else if(sms.startsWith("mixed")){
	ms=2;
	setFREL(true);
      }else{
	throw Except("");
      }
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading..."
			   ,Sprintf("Line %d: expect up/down as "
				    "4th word ",nline)()
			   ,QMessageBox::Yes );
      return false;
      
    }

    long ind;
    if(poccus.contains(occu)){
      //myDebug() << "old"<<occu.isFREL();
      //ind=poccus.indexOf(occu);
      //poccus[ind].setFREL(occu.isFREL());
    }else{
      //myDebug() << "new"<<occu.isFREL();;
      poccus.append(occu);
      ind=poccus.size()-1;
    }
    Occu &o=poccus[ind];

    //myDebug() << o.printS()();
    
    
    // read 2l+1 lines real part
    for(int imre=0;imre<2;imre++){
      long m=0;
      while(1) {
	if(!getLine(FI,s)){
	  QMessageBox::warning(0,"Reading..."
			       ,Sprintf("Line %d: unexpected end "
					,nline+1)()
			       ,QMessageBox::Yes );
	  return false;
	}
	nline++;
	s=s.stripBlanks(true);
	if(s.length()==0) continue;
	
	// split into words
	StringList sl=s.split(" ");
	// skip comments
	if( sl[0]=="#"){ continue; }
	//myDebug() << "\t" << s();
	m++;
	// here we may read the value

	int si,sj; o.getSpinOffset(ms,si,sj);

	for(int m1=0;m1<o.pdegl;m1++){
	  try{
	    String s=sl[m1];
	    double d=s.convertToFortranReal();
	    if(imre==0){
	      o.sp(m-1+si,m1+sj)=Cmplx(d);
	      if(ms==2) o.sp(m1+sj,m-1+si)=Cmplx(d);
	    }else{
	      o.sp(m-1+si,m1+sj)=Cmplx(o.sp(m-1+si,m1+sj).real(),d);
	      if(ms==2) o.sp(m1+sj,m-1+si)=Cmplx(o.sp(m-1+si,m1+sj).real(),-d);
	    }
	  }catch(Except &ex){
	    QMessageBox::warning(0,"Reading..."
				 ,Sprintf("Line %d: expect %d real values "
					  ,nline,o.pdegl)()
				 ,QMessageBox::Yes );
	    return false;
	  }
	  // save value here
	}
	
	if(m>=o.pdegl){
	  break;
	}
      } // while(1)
      
    } // imre
    //poccus.append(occu);
    //myDebug();
    
  } // while getLine
  fclose(FI);

  setGlobalSpinZaxis(Vec3(0.,0.,1.));
      
  if(pfrel){
    try{
      QString pcdirtmp = QDir::currentPath() ;
      Parser inparser(1);
      String fname="=.in";
      inparser.parseFile(fname);
      Vec3 qaxis;
      inparser()("quantization_axis").getArray(3,qaxis());
      setGlobalSpinZaxis(qaxis);
      QDir::setCurrent(pcdirtmp) ;
      
    }catch(Except &ex){
      QMessageBox::warning(0,"Reading FPLO quantizationa axis",ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default);
      return false;
    }
  }
  
  return true;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool DMDoc::save()
{
  //myDebug() << "DMDoc::save()" ;

  FILE *FO;
  try{

    FO=openforwrite(pfilename);
  }catch(Except &ex){
    QMessageBox::warning(0,"Reading...",ex.msg(),QMessageBox::Yes );
    return false;
  }



  const char *txt[3]={"up","down","mixed(updown)"};

  for(int i=0;i<poccus.size();i++){
    
    for(int ms=0;ms<3;ms++){
      
      if(ms==3 && !isFREL()) break;
      
      fprintf(FO,"# Element site  state spin\n");
      Occu &oc=poccus[i];
      int si=0,sj=0; 
      if(ms==1){ // cannot use getSpinOffset, since we have default settings
	si=sj=oc.degL();
      }else if(ms==2){
	si=0;sj=oc.degL();
      }
      fprintf(FO,"%s %ld %s %s\n",oc.el()(),oc.site(),oc.orbital()(),txt[ms]);
      fprintf(FO,"# real part\n");
      int degl=oc.degL();
      for(int i=0;i<degl;i++){
	for(int j=0;j<degl;j++){
	  fprintf(FO,"%-17.10f",oc.sp(i+si,j+sj).real());
	}
	fprintf(FO,"\n");
      }
      fprintf(FO,"\n");
      fprintf(FO,"# imaginary part\n");
      for(int i=0;i<degl;i++){
	for(int j=0;j<degl;j++){
	  fprintf(FO,"%-17.10f",oc.sp(i+si,j+sj).imag());
	}
	fprintf(FO,"\n");
      }
      fprintf(FO,"\n");
      fprintf(FO,"\n");
    }
  }
  

  fclose(FO);
  

  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMDoc::saveAxes()
{
  Settings set("./dmatedit.ini","Data");
  set.beginWriteArray("Occus");
  for(int i=0;i<poccus.size();i++){
    set.setArrayIndex(i);
    set.set("occuName",poccus.at(i).name());
    set.set("currentAxes", (int)poccus.at(i).currentAxes());
    set.beginWriteArray("Axes");
    for(int j=0;j<poccus.at(i).localaxes.size();j++){
      set.setArrayIndex(j);
      set.set("xaxis",poccus.at(i).localaxes.at(j).xaxis);
      set.set("zaxis",poccus.at(i).localaxes.at(j).zaxis);
    }
    set.endArray();
   }
  set.endArray();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMDoc::loadAxes()
{
  Settings set("./dmatedit.ini","Data");
  int noccu=set.beginReadArray("Occus");
  for (int i = 0; i < noccu; ++i) {
    set.setArrayIndex(i);
    Occu oc1;
    oc1.fromName(String(set.getString("occuName")));
    int curax=set.getInt("currentAxes");
    int ioccu;
    if((ioccu=poccus.indexOf(oc1))>=0){
      Occu &oc=poccus[ioccu];
      oc.setCurrentAxes(curax);
      oc.localaxes.clear();
      int naxes=set.beginReadArray("Axes");
      for (int j = 0; j < naxes; ++j) {
	set.setArrayIndex(j);
	oc.localaxes.append(DMQuantAxis(
					set.getVec3("xaxis")
					,set.getVec3("zaxis")
					));
      }
      set.endArray();
     }
   }
  set.endArray();
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::calcTrans(bool needDj)
{
  if(currentaxes<0 || currentaxes>localaxes.size()-1){
    printf("Occu::calcTrans: currentaxes=%ld out of range\n",currentaxes);
    exit(1);
  }
  if(needDj){
    localaxes[currentaxes].calcTransLJ(pl);
  }else{
    localaxes[currentaxes].calcTransLL(pl);
  }
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::toRepresentation(YlmRepresentation rep,const Vec3 &spinzaxis,Props &pr)
{
  if(currentaxes<0 || currentaxes>localaxes.size()-1){
    printf("Occu::toRepresentation: currentaxes out of range\n");
    exit(1);
  }
  const SqMatrix<Cmplx> &ytr=localaxes[currentaxes].YTransMat();
  
  switch(rep){
  case RealYlm:{
    trsp=ytr.hermiteConjugated()*sp*ytr;
    break;
  }
  case ComplexYlm:{
    
    //first rotate the unrotate real rep
    //then make the rotated rep complex rep
    trsp=pU.hermiteConjugated()*ytr.hermiteConjugated()*sp*ytr*pU;
    
    break;
  }
  case SphericalSpinor:{
    // default form is the real Ylm representation of the occupation
    // number matrices in the spin frame of the global quantization axis
    // set in FPLO for the colinear FREL calculations
    
    // essemble the full spin occuaption matrix
    
    // first step: take us back to the spherical spinors
    // n_kamu=(Dq1/2+T)+ nB (Dq1/2+T)
    //second step: rotate into local axis
    // n_kamu'=Dj+ nm Dj
    
    //note, that for spherical spinors, we rotate the whole spinor
    
    SqMatrix<Cmplx> reltopsD=relTOpseudo(spinzaxis,pl)*ytr;
    trsp=reltopsD.hermiteConjugated()*sp*reltopsD;
    
    
    break;
  }
  }
  
  pcurrentylmrep=rep;
  analyseProps(pr); 
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::fromRepresentation(const Vec3 &spinzaxis,Props &pr)
{
  if(currentaxes<0 || currentaxes>localaxes.size()-1){
    printf("Occu::fromRepresentation: currentaxes out of range\n");
    exit(1);
  }
  
  analyseProps(pr);
  
  const SqMatrix<Cmplx> &ytr=localaxes[currentaxes].YTransMat();
  

  switch(pcurrentylmrep){
  case RealYlm:{
    sp=ytr*trsp*ytr.hermiteConjugated();
    break;
  }
  case ComplexYlm:{
    sp=ytr*pU*trsp*pU.hermiteConjugated()*ytr.hermiteConjugated();
    break;
  }
  case SphericalSpinor:{
    
    SqMatrix<Cmplx> reltopsD=relTOpseudo(spinzaxis,pl)*ytr;
    sp=reltopsD*trsp*reltopsD.hermiteConjugated();
    
    break;
    }
  }

  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::fromName(const String &nam)
{

  String n=nam;
  n.removeLeadingBlanks();

  pel=n.subString(0,1);
  n=n.subString(2).removeLeadingBlanks().removeSection(0,0);
  int i=n.find(")");
  
  try{
    psite=n.subString(0,i-1).convertToLong();
  }catch(Except &ex){
  }
  n.removeSection(0,i).removeLeadingBlanks();
  n.orbitalToInt(pnmain,pl);
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::flipSpin()
{

  //find spin axis
  //transform with D1o2=D^{1/2} to put local z into spin axis,
  //which removes any y-component
  //rotate spin around y-axis (D=i*<|sigma_y|>)
  //transform back -> this should not touch L


  SqMatrix<Cmplx> Ts=getSpinFlipMatrix(trsp,pcurrentylmrep);
  
  Ts=Ts*psigma[pcurrentylmrep][1]*Ts.hermiteConjugated();

  trsp=Ts*trsp*Ts.hermiteConjugated();
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Occu::flipLz()
{

  SqMatrix<Cmplx> Ts=getSpinFlipMatrix(trsp,pcurrentylmrep);
  
  Ts=(Ts*psigma[pcurrentylmrep][1]).conjugated()*(Ts*ptheta[pcurrentylmrep]).hermiteConjugated();
  
  trsp=Ts.hermiteConjugated()*trsp.conjugated()*Ts;

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SqMatrix<Cmplx> Occu::getSpinFlipMatrix(const SqMatrix<Cmplx> &a,
			     YlmRepresentation ylmrep) const
{
  // return the matrix, which transforms into a local ccordinate system,
  // in which the spin has zero y components.

  Vec3 Svec=SVec(a);
  if(norm(Svec)<1.0e-8){
    Svec=Vec3(0.,0.,1.);
  }
  
  SqMatrix<Cmplx> Ts(2*pdegl);

  switch (ylmrep){
  case RealYlm:
  case ComplexYlm:
    {
      SqMatrix<Cmplx> D1o2=wignerDChijm(Vec3::eulerAngles(Svec),1,false);
      for(int i=-1;++i<pdegl;){
	Ts(i,i)=D1o2(1,1);
	Ts(i+pdegl,i+pdegl)=D1o2(0,0);
	Ts(i,i+pdegl)=D1o2(1,0);
	Ts(i+pdegl,i)=D1o2(0,1);
      }
      break;
    }
  case SphericalSpinor:
    {
      SqMatrix<Cmplx> Dj=wignerDChijm(Vec3::eulerAngles(Svec),2*pl-1,false);
      Ts.copyBlock(0,0,Dj,0,0,pdegl-1,pdegl-1);
      Dj=wignerDChijm(Vec3::eulerAngles(Svec),pdegl,false);
      Ts.copyBlock(pdegl-1,pdegl-1,Dj,0,0,pdegl+1,pdegl+1);
      break;
    }
  }
  
  
  return Ts;
}

/*EOF*/ 
