/*************************************************************************
 * @(#)DataStatistics_cpp.hh
 * author: magru@unas
 * date:   24 Jun 2013
 *************************************************************************
 */

#include "DataStatistics.hh"
#include "Cmplx.hh"
#include "MyMath.hh"

using namespace XFPLO;

namespace MyMath{
  // ----------------------------------------------------------------
  // redeclare min and max for this local purpose to compare abs(Cmplx) instead
  // of Cmplx
  // ----------------------------------------------------------------
  template<>
  Cmplx min(Cmplx x,const Cmplx &y){
    return (x.abs()<y.abs()?x:y);
  }
  
  template<>
  Cmplx max(Cmplx x,const Cmplx &y){
    return (x.abs()>y.abs()?x:y);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class X>
void DataStatistics::calculate(const BaseArray<X> *ba)
{
  pdata.create<X>();

  if(!ba) return;
  long nitems=ba->items();
  long datasize=ba->datasize();


  //myDebug()<<TypeInfo<X>::name;
  //myDebug()<<"datasize="<<datasize;
  
  ValueArray<X> &min=pdata.to<X>()->minValues;
  ValueArray<X> &max=pdata.to<X>()->maxValues;
  min.resize(datasize);
  max.resize(datasize);

  min=X(0);
  max=X(0);

  if(nitems<1) return;
  
  for(long c=-1;++c<datasize;){
    X d=ba->point(0,c);
    min[c]=d;
    max[c]=d;
  }
  for(long i=0;++i<nitems;){
    for(long c=-1;++c<datasize;){
      X d=ba->point(i,c);
      min[c]=MyMath::min<X>(d,min[c]);
      max[c]=MyMath::max<X>(d,max[c]);
    }
  }
  
}



/*EOF*/ 
