//todo: oglview->invalidate()? use protected pointers?



/* @(#)DisplayCells.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _DISPLAYCELLS_H
#define _DISPLAYCELLS_H 1

#include "MyQRgb.hh"
#include  "SReal.hh"
#include  "MACROS.hh"
#include  "IDCache.hh"
#include  "Xf.hh"

class Boundary;
namespace XFPLO{
  class Symmetry;
}
class Vlevel;
class Dobj;

using namespace XFPLO;
/* ***********************************************************************
 *
 * ***********************************************************************
 */

class DisplayCell
{
 public:
  DisplayCell();
  DisplayCell(Xf::DisplayCellType type,Xf::LatticeType lt);
  DisplayCell(const DisplayCell &o);
  const DisplayCell &operator=(const DisplayCell &);
  ~DisplayCell();


  inline const Boundary *boundary()const{return pboundary;};
  inline Boundary *boundary(){return pboundary;};
  


  bool isVisible()const;
  void setVisible(bool v)const;

  bool changed()const;
  void setChanged(bool c);

  bool calculated();
  void setCalculated(bool s);

  String calculate(Xf::LatticeType latticetype
		 ,double alloverscale,const Symmetry *sym,double tol
		 ,const Vlevel &level);



  static void PTHeaderToFile(FILE *FH,const char *prefix,const char *tail);
  void PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const;
  void readPTSectionFromFile(const Dobj &root);
  
 private:
  PROPERTY_CHANGED_CALCED(type,type,Type,Xf::DisplayCellType);
  bool pchanged;
  bool pcalced;
  CPROPERTY(id,id,Id,String);
  CPROPERTY_CHANGED(name,name,Name,String);
  Boundary *pboundary;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class DisplayCells
{
 public:
  DisplayCells();
  DisplayCells(const DisplayCells &);
  const DisplayCells &operator=(const DisplayCells &other);
  ~DisplayCells();

  // special copy, which does not copy the invisible boundary data
  // for the main cells
  void copyFrom(const DisplayCells *);

  bool changed()const;
  
  void setChanged(bool c);

  bool calculated();
  void setCalculated(bool s);

  String  calculate(Xf::LatticeType latticetype
		     ,const Symmetry *sym,double tol
		     ,const Vlevel &level);

  inline int size()const{return pcells.size();};
  inline const QList<DisplayCell*> &cells()const{return pcells;};
  inline const DisplayCell *cell(int i)const{return pcells.at(i);};
  inline DisplayCell *cell(int i){return pcells.value(i);};
  inline int indexOf(DisplayCell*dc)const{return pcells.indexOf(dc);};

  DisplayCell *findByType(Xf::DisplayCellType type);
    
  void insert(int at,DisplayCell *c,bool createuniquename);
  void remove(DisplayCell *dc);

  void PTSectionToFile(FILE *FH)const;
  void readPTSectionFromFile(const Dobj &root);
  void setDefault(Xf::LatticeType lt);

 private:
  void clear();
  

 private:
  bool pchanged;
  bool pcalced;
  Xf::LatticeType platticetype;
  QList<DisplayCell*> pcells;
  IDCache pidcache;
  IDCache pnamecache;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _DISPLAYCELLS_H */

