/*************************************************************************
 * @(#)DocBase.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 Jul 2009
 *************************************************************************
 */
#include <QFileInfo>
#include "DocBase.hh"
#include "Parser.hh"
#include "Assorted.hh"
#include "Except.hh"
#include "Config.hh"
#include "Light.hh"
#include "Axes.hh"
#include "DisplayCells.hh"
#include "Fog.hh"
#include "Settings.hh"
#include "MyDebug.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
DocBase::DocBase(OGLView *oglview)
  :modified(false),paxes(new Axess()),pdisplaycells(new DisplayCells()),
   plights(new Lights),pfog(new Fog()),
   pfontres(100),poglview(oglview)
{
  //myDebug() <<"DocBase::DocBase";
  setCurrentDir();


  plights->setGlobalAmbientFactor(0.25);
  plights->setGlobalAmbient(0xffffffff);


  {
    LightThemes lt;
    *plights=lt.currentLights();
  }



  magfac=4;
  picquality=100;


  Settings set("mainwindow");
  magfac=set.getInt("printing/magnification_factor",magfac);
  picquality=set.getInt("printing/quality",picquality);
  setCanvasLocked(set.getBool("canvas/lock",canvas().locked()));



}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
DocBase::~DocBase()
{
  myDebug()<<"DocBase~DocBase";

  Settings set("mainwindow");
  
  set.set("printing/magnification_factor",magfac);
  set.set("printing/quality",picquality);
  set.set("canvas/lock",canvas().locked());

  delete paxes;
  delete pdisplaycells;
  delete pfog;
  delete plights;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Boundary *DocBase::bound() 
{
  return pdisplaycells->findByType(Xf::DCTMainBoundary)->boundary();
}
const Boundary *DocBase::bound() const 
{
  return (const Boundary *)(pdisplaycells->findByType(Xf::DCTMainBoundary)
			    ->boundary());
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString DocBase::absolutePath(const QString &file)
{
  saveCurrentDir();
  QString p;
  if(file.isEmpty())
    p=QFileInfo(pfilename).absolutePath();
  else
    p=QFileInfo(file).absolutePath();
  restoreCurrentDir();
  return p;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString DocBase::absoluteFilePath(const QString &file) 
{
  saveCurrentDir();
  QString p;
  if(file.isEmpty())
    p=QFileInfo(pfilename).absoluteFilePath();
  else
    p=QFileInfo(file).absoluteFilePath();
  restoreCurrentDir();
  return p;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString DocBase::fileName(const QString &file) 
{
  saveCurrentDir();
  QString p;
  if(file.isEmpty())
    p=QFileInfo(pfilename).fileName();
  else
    p=QFileInfo(file).fileName();
  restoreCurrentDir();
  return p;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void DocBase::readRelativisticMode()
{
  saveCurrentDir();
  Parser inparser(1);
  String fname="=.in";
  try{
    inparser.parseFile(fname);
    prelatmode=(Xf::RelativisticMode)inparser()("relativistic.type").valueL();
  }catch(Except &ex){
    prelatmode=Xf::ScalarRelativistic;
  }
  //myDebug()<<"prelatmode="<<prelatmode;
  restoreCurrentDir();
}
