/* @(#)DocBase.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef DOCBASE_H
#define DOCBASE_H

#include <QObject> 
#include "MyQRgb.hh"
#include <QDir> 

#include "Vec3.hh"
#include "Xf.hh"
#include "View.hh"
#include "CanvasSize.hh"


class Boundary;
class DisplayCells;
class Fog;
class OGLView;


namespace XFPLO{
  class Lights;
  class Axess;
}

using namespace XFPLO;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class DocBase: public QObject
{
Q_OBJECT
public:
  DocBase(OGLView *oglview);
  virtual ~DocBase();


  void setFileName(const QString & fn){pfilename=fn;};
  QString absolutePath(const QString &file="") ;
  QString absoluteFilePath(const QString &file="");
  QString fileName(const QString &file="");

  bool isNew() const{return pisnew;};
  void setNew(){pisnew=true;};
  void setOld(){pisnew=false;};

  bool isModified() const{return modified;};
  void setModified(bool m){modified=m;};
  void setModified(){setModified(true);};
  
  void readRelativisticMode();
  inline Xf::RelativisticMode relativisticMode()const {
    return prelatmode;};

  const CanvasSize &canvas() const {return pcanvas;};
  CanvasSize &canvas() {return pcanvas;};
  void setCanvasSize(const CanvasSize &c) {pcanvas=c;};
  void setCanvasLocked(bool l) {pcanvas.setLocked(l);};

  
  virtual Vec3 dataMidPoint()=0;
  virtual double  dataRadius() =0;

  virtual bool showLegend()=0;
  virtual double legendPositionY()=0;

  View &view() {return pview;};

  Fog *fog(){return pfog;};

  DisplayCells *displayCells() {return pdisplaycells;};
  Axess *axes() {return paxes;};

  Boundary *bound();
  const Boundary *bound()const;


  int fontRes()const{return pfontres;}

  Lights *lights() {return plights;};
  const Lights *lights()const{return plights;};


  // handle multiple mainwindow's currentdir
  const QString &currentDir(){ return pcurrentdir;};
  void setCurrentDir(const QString &cd=""){ 
    if(cd.isEmpty())
      pcurrentdir=QDir::currentPath();
    else
      pcurrentdir=cd;
  };

  void saveCurrentDir(){ pcdirtmp = QDir::currentPath() ; 
    QDir::setCurrent(pcurrentdir);};
  void restoreCurrentDir(){ QDir::setCurrent(pcdirtmp) ;};



public:





  int magfac;
  int picquality;

  bool allcalced;

protected:
  QString pfilename;
  bool modified;

  View pview;


  CanvasSize pcanvas;


  Axess *paxes;
  DisplayCells *pdisplaycells;

  Lights *plights;

  
  bool pisnew;

  Fog *pfog;

  int pfontres;

private:
  QString pcurrentdir,pcdirtmp;		/* since we can have multiple 
				   mainwindows, and currentdir
				   is qapplication-wide, we
				   need to keep track of the 
				   static currentdir
				 */
  Xf::RelativisticMode prelatmode;

protected:
  OGLView *poglview;

};

#endif // DOCBASE_H
