//todo: diagonalize the plasma tensor

/*************************************************************************
 * @(#)Erg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 Jun 2009
 *************************************************************************
 */

#include <QProgressDialog>
#include <QPushButton>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Boundary.hh"
#include "MeshSymmetry.hh"
#include "constants.h"
#include "MyMath.hh"
#include "Vlevel.hh"
#include "Assorted.hh"
#include "Exception.hh"
#include "OGLView.hh"
#include "Except.hh"

#include "Erg.hh"
#include "MeshData.hh"
#include "RegularMesh.hh"
#include "TriangleMesh.hh"
#include "BWeights.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"





/* ***********************************************************************
 *
 * ***********************************************************************
 */
Energies::Energies()
  :pemptylattice(false),pmeshchanged(true),pmeshcalculated(false)
  ,penergiesloaded(false)
  ,pusesymmetry(true),pnx0subdiv(12)
  ,pmanualsubdiv(false)
  ,pnx0subdivmanual(0)
  ,pny0subdivmanual(0)
  ,pnz0subdivmanual(0)
  ,puseboundarycell(false)
  ,pmesh(0),nbdim(0),ib0(0),ib1(-1)
  ,pbandfile("+band_kp")
  ,pbandweightsfile("+bweights_kp")
{
  pbands[0].reserve(0);
  pbands[1].reserve(0);
  nmin[0]=-1;
  nmin[1]=-1;
  nmax[0]=-2;
  nmax[1]=-2;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Energies::~Energies()
{
  if(pmesh) delete pmesh; pmesh=0;

  foreach(TriangleMesh *trm,ptriameshs){
    delete trm;
  }
  ptriameshs.clear();

  pbands[0].clear();
  pbands[1].clear();

  clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Energies::clear()
{
 

  nbdim=0;
  ib0=ib1=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Energies::setDefaultBands()
{
  pbands[0].resize(1);
  pbands[1].resize(1);
  
  pbands[0][0]=0;
  pbands[1][0]=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Energies::setUpMesh(OGLView *oglview,const MeshSymmetry &meshsym
			 ,const Vlevel &vlevel
			 ,bool usecell,const Vec3 &origin
			 ,const Matrix3x3 &cell)
{
  if(pmesh) {
    if(oglview) {
      oglview->invalidateMesh(pmesh);
    }

    delete pmesh; pmesh=0;
  }
  String msg1;
  pmesh=new RegularMesh("RegularMesh bands");
  //   pmesh->setOrigin(-0.5*(cell(0)+cell(1)+cell(2)));
  //pmesh->setSubdivisions(doc->tetra.nx,doc->tetra.ny,doc->tetra.nz);
  if(usecell){
//     myDebug()<< origin;//*meshsym.convRecCell();
//     myDebug()<< cell;//*meshsym.convRecCell();
    pmesh->setOrigin(origin);
    pmesh->setCell(cell);
    pmesh->setPeriodic(false);
  }else{
    pmesh->setOrigin(Vec3());
    pmesh->setCell(meshsym.recCell());
    pmesh->setPeriodic(true);
  
    msg1=pmesh
      ->setSymmetricMeshSubdivision(meshsym,pnx0subdiv,vlevel);
  }
  //hack
  if(pmanualsubdiv){
    if(pnx0subdivmanual>0) pmesh->setXSubdivision(pnx0subdivmanual);
    if(pny0subdivmanual>0) pmesh->setYSubdivision(pny0subdivmanual);
    if(pnz0subdivmanual>0) pmesh->setZSubdivision(pnz0subdivmanual);
  }

  try{
    pmesh->calcPoints(meshsym,pusesymmetry);
  }catch(Exception &ex){   
    pmeshcalculated=false;
    throw ex;
  };
  pmesh->calcMicroCells();
  //pmesh->printPoints();

  pmeshchanged=false;
  pmeshcalculated=true;
  penergiesloaded=false;

  if(!msg1.empty()) throw Exception(msg1,Exception::warning);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Energies::calculateIsoSurfaces(OGLView *oglview
				    ,const Boundary *bound,bool clip)
{
  
  foreach(TriangleMesh *trm,ptriameshs){
    if(trm) {
      if(oglview){
	oglview->invalidateMesh(trm);
	oglview->removeObject(trm->id());
      }
      delete trm;
    }
    
  }
  ptriameshs.clear();
  
  double ef=pef;
  if(!pemptylattice)
    ef=(pusedefaultEf ? 0. : ef);
  
  for(long ms=0;ms<pnspin;ms++){
    
    for(long jj=0;jj<pbands[ms].size();jj++){
      long ib=pbands[ms].at(jj);
      //myDebug() << "jj" << jj << "ib "<<ib;
      if(ib<ib0 || ib>ib1) continue;
      
      ptriameshs.
	append(new TriangleMesh(QString("TriangleMesh band %1 spin %2")
				.arg(ib).arg(ms)));
      
      QList<Plane> clippingplanes;
      //clippingplanes.append(Plane(Vec3(1,1,1),0.03));
      //clippingplanes.append(Plane(Vec3(0,-1,0),0.0));
      pmesh->createIsoSurface(ptriameshs.last(),bound
			      ,&clippingplanes
			      ,ef,(ib-ib0+nbdim*ms)
			      ,clip);
    }
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString Energies::checkIndices()
{
  long errocc=0;
  long nb;
  QString text="",range,indicees;
  
  indicees+="\n";
  range+="\n";
  
  //    myDebug() << nsubfunc << " checking indices";
  
  for(long ms=0;ms<pnspin;ms++){
    QVector<long> tmp;
    if(nmin[ms]<=nmax[ms]){
      range+=QString().sprintf("\tspin %ld: [%ld ... %ld]"
			  ,ms+1,nmin[ms]+1,nmax[ms]+1);
    }else{
      range+="none";
    }
    
    indicees+=QString().sprintf("\tspin %ld: index=",ms+1) ;
    
    for(long i=0;i<pbands[ms].size();i++){
      nb=pbands[ms].at(i);
      
      if(nb<nmin[ms] || nb>nmax[ms]){
	errocc+=1;
	if(errocc>1){
	  indicees+=",";
	}
	indicees+=QString().sprintf("%ld",nb+1) ;
      }else{
	tmp.append(nb);
      }
    }
    if(pnspin>1 && ms==0){ indicees+="\n"; }
    if(pnspin>1 && ms==0){ range+="\n"; }
    pbands[ms]=tmp;
    
  }
  indicees+="\n";
  range+="\n";
  
  
  if(errocc){
    text.sprintf("Given band(s) %sdo not cross the Fermi energy \n\n"
		 "Possible range:%s\n"
		 "This may dependend on the mesh quality\n"
		 ,indicees.toAscii().constData()
		 ,range.toAscii().constData()) ;
  }

  if(numBands()==0){
    text+="The number of bands in +band seems to be zero.\n"
      "You most likely have a gap.\n"
      "It migh be due to numerics.\n"
      "If you want a different Fermi level to play around\n"
      "and there are no bands, consider the k-export options\n"
      "in the options menu (uncheck the \"partially occupied\" flag)\n"
      "and re-export the k-mesh";

  }
  
  return text;
  
}


/* ***********************************************************************
 * load/make band energies
 * ***********************************************************************
 */
void Energies::load(const QString &file,const QString &cfile
		    ,const MeshSymmetry &meshsym
		    ,bool &cbandread
		    ,long icbandrestrict,long icbandmin,long icbandmax
		    ,const Vlevel &vlevel,bool excol)
{
  NSF("epsilon");

  int prot=(vlevel >= Vlevel::More );
  
  if(prot)
    printf("%s: START ======================================\n",nsubfunc);  

  clear();

  cbandread=false;
  if(! pemptylattice ){
    readbanderg(file.toAscii().constData()
		,cfile.toAscii().constData()
		,cbandread,icbandrestrict,icbandmin,icbandmax,excol) ;
    
  }else{
    /* epmty lattice calc */
    

    pmesh->clearMeshData();

    /* maximum nbmax bands starting from zero index */
    pnspin=1;
    long nbandwish=10;
    nbdim=nbandwish;

    pmesh->appendMeshData(nbdim*pnspin);

    nkfromfile=pmesh->numIrredRotPoints();
    


    const Matrix3x3 &rcell=meshsym.recCell();
    const Matrix3x3 &cell=meshsym.cell();

    double conv=constant_hatoev*twopi*twopi;
      
    if(meshsym.lengthUnit()== Xf::Angstroem)
      conv*=constant_aBtoAng*constant_aBtoAng;

    QList<double> Eb;
    for(long ipi=0;ipi<pmesh->numIrredRotPoints();ipi++){
      //
      Vec3 q=pmesh->irredRotPoint(ipi);

      Eb.clear();
      double qx=q*cell(0);
      double qy=q*cell(1);
      double qz=q*cell(2);
      
      long h0,h1,k0,k1,l0,l1;
      double rmax=0.1/1.5;	

      do {
	rmax*=1.5;
	h1=(long)ceil ( rmax*norm(cell(0))-qx);
	h0=(long)floor(-rmax*norm(cell(0))-qx);
	k1=(long)ceil ( rmax*norm(cell(1))-qy);
	k0=(long)floor(-rmax*norm(cell(1))-qy);
	l1=(long)ceil ( rmax*norm(cell(2))-qz);
	l0=(long)floor(-rmax*norm(cell(2))-qz);

	
      }while((h1-h0+1)*(k1-k0+1)*(l1-l0+1)<nbandwish);
      
      for(long h=h0;h<=h1;h++){
	for(long k=k0;k<=k1;k++){
	  for(long l=l0;l<=l1;l++){
	    
	    Vec3 v=q+rcell*Vec3(h,k,l);
	    Eb.append(v*v);
	  }
	}
      }//h
      qSort(Eb.begin(),Eb.end());

      long ipi0=pmesh->IequivR(pmesh->RequivI_Rot(ipi));
      for(long i=0;i<nbdim;i++){
	pmesh->setMeshData(i,ipi0,Eb[i]*conv*0.5);
      }
      
      
      
    } //ipi
    
    pmesh->fillIrredFromIrredRot();

    ib0=0;
    ib1=nbdim-1;
    

  } // empty lattice 

  if(prot){
    printf("%s: erg.ib0=%ld\n",nsubfunc,ib0); 
    printf("%s: erg.ib1=%ld\n",nsubfunc,ib1); 
    printf("%s: erg.nbdim=%ld\n",nsubfunc,nbdim); 
  }
  
  if((vlevel >= Vlevel::Info )){
    printf("%s: erg memory    : %f Meg\n",nsubfunc
	   ,sizeof(double)*nkfromfile*nbdim/1000000.);
  }




  if(prot)printf("%s: END   ======================================\n",nsubfunc);  


}



/* ***********************************************************************
 * det the bands crossing ef
 * ***********************************************************************
 */

void Energies::calcRange(const Vlevel &vlevel,bool takeAll)
{
  NSF("calcefrange");

  double ef=pef;
  if(!pemptylattice)
    ef=(pusedefaultEf? 0. : pef);

  long lnmin[2];
  long lnmax[2];
  lnmin[0]=nmin[0];
  lnmin[1]=nmin[1];

  lnmax[0]=nmax[0];
  lnmax[1]=nmax[1];
  
  int prot=(vlevel >= Vlevel::More );

  if(prot)
    printf("%s: START ======================================\n",nsubfunc);  

  if(vlevel >= Vlevel::All ){
    printf("%s: ib0= %ld\n",nsubfunc,ib0);  
    printf("%s: ib1= %ld\n",nsubfunc,ib1);  
  }

  for(int ms=0;ms<pnspin;ms++){
    nmax[ms]=0;
    nmin[ms]=ib1;
    
    for(long ib=ib0;ib<=ib1;ib++){
      if(vlevel >= Vlevel::All ) printf("%s: ib=%ld\n",nsubfunc,ib);   
      
      double emin,emax;
      pmesh->meshDataMinMax((ib-ib0+nbdim*ms),emin,emax);

      if(vlevel >= Vlevel::All ){
	printf("%s: emin=%g  emax=%g  ef=%g\n",nsubfunc,emin,emax,ef);  
	printf("%s: ef=%g\n",nsubfunc,ef);   
      }
      if(emin <ef && emax>= ef){

	nmax[ms]=MyMath::max<long>(nmax[ms],ib);
	nmin[ms]=MyMath::min<long>(nmin[ms],ib);
	if(vlevel >= Vlevel::All )
	  printf("%s: nmax=%ld nmin=%ld\n",nsubfunc,nmin[ms],nmax[ms]);   
	
      }
    } /* ib */
    if(prot){
      printf("%s: minimum band number %ld\n",nsubfunc,nmin[ms]);  
      printf("%s: maximum band number %ld\n",nsubfunc,nmax[ms]);  
    }
  }

  if(prot)
    printf("%s: END   ======================================\n",nsubfunc);  

  prangechanged|=(lnmin[0]!=nmin[0]);
  prangechanged|=(lnmin[1]!=nmin[1]);
  prangechanged|=(lnmax[0]!=nmax[0]);
  prangechanged|=(lnmax[1]!=nmax[1]);


  if(takeAll){
    for(int ms=0;ms<pnspin;ms++){
      pbands[ms].clear();
      for(long i=nmin[ms];i<=nmax[ms];i++){
	pbands[ms].append(i);      
      }
    }
  }

}



/* ***********************************************************************
 * read band eergies from file
 * ***********************************************************************
 */
void Energies::readbanderg(const char *fname,const char *cname
			   ,bool &cbandread,long icbandrestrict
			   ,long icbandmin,long icbandmax,bool excol)
{
  //NSF("readbanderg");
  long n=0;
  FILE *FH;

  cbandread=false;

  pmesh->clearMeshData();
  
  /* band number is zero for secure use after,  if failure */
  nbdim=0;
  long b0=0;
  long b1=-1;
  long bdim=0;
  long err,idum,nkip,Nb,io,iu;

  try{
    FH=openforread(fname);
  }catch(Except ex) {
    throw Exception(ex.msg());
  }
  


  {
    /* header of band file fname */
    err=fscanf(FH," # %ld %lg %ld %ld %ld %ld %ld %ld\n",&idum
	       ,&efermi,&nkip,&idum,&Nb,&pnspin,&io,&iu)-8; 

    if( pmesh->numIrredRotPoints() != nkip){
      String text;
      text.sprintf("The number of k-points %ld in file \'%s'\n"
		   "does not match the actual settings: \n"
		   "(irreducible) points : %ld!\n\n"
		   "Did you export the k-points?\n"
		   "Did you save and load the settings?\n"
		   "Did you let fplo recalculate the file '+band'?"
		   ,nkip
		   ,fname
		   ,pmesh->numIrredRotPoints());
      throw Exception(text);
    }


    io--;
    iu--;
    if(err==-2){
      /* old +band file assumed */
      io=0;
      iu=Nb-1;
      err=0;
    }
    n+=err;
    /* check tetra against fname info */

    /* no error */
    if(!n){

      /* printf("pnspin %ld\n",erg.pnspin);   */
      b0=io;
      b1=iu;
      if(icbandrestrict){
	b0=MyMath::max<long>(b0,icbandmin);
	b1=MyMath::min<long>(b1,icbandmax);
      }
      bdim=b1-b0+1;
      
      
      
      pmesh->appendMeshData(bdim*pnspin);


      
      
      nkfromfile=nkip;

      long subdiv=(MyMath::max<long>(1,nkip/(100/5)));
      long countMax=MyMath::max<long>(1,nkip/subdiv);

      QProgressDialog progress(QString("Reading %1...").arg(fname)
			       , "OK", 0, countMax, 0);
      progress.setWindowModality(Qt::ApplicationModal);
//       QPushButton pb("OK",&progress);
//       progress.setCancelButton(&pb);
//       QObject::connect(&pb,SIGNAL(clicked()),&progress,SLOT(hide()));

//      printf("k%% = %ld %ld\n",subdiv,countMax);

      for(long iki=0;iki<nkip;iki++){
	if ( (iki)%(MyMath::max<long>(1,nkip/(100/5))) ==0 ){
	  //printf("k%% =%10.2f %ld %ld\n",100.*iki/(nkip*1.),iki,nkip);
	  //	  printf("k%% = %ld %ld %ld\n",iki,(long)(iki/subdiv),countMax);
	  progress.setValue((int)(iki/subdiv));

	  //	  if (progress.wasCanceled()) break;
	}
	long ik=pmesh->IequivR(pmesh->RequivI_Rot(iki));

	long c;
	while ((c=fgetc(FH)) != '\n' && (c != EOF)) {};/* skip line */
	for(int ms=0;ms<pnspin;ms++){

	  double q;
	  n+=fscanf(FH," %lg ",&q)-1; /* skip k-point coordinate */
	  
	  
	  for(long ib=io;ib<=iu;ib++){
	    n+=fscanf(FH," %lg ",&q)-1; 
	    if( ib>=b0 && ib<=b1) {
	      pmesh->setMeshData((ib-b0+bdim*ms),ik,q);
	    }
	  }
	  (void)fscanf(FH,"\n"); 
	}
      }

      progress.setValue(countMax)      ;

      if(n!=0)
	throw Exception(Sprintf("error reading file \'%s\'!",fname));
      
    }
    fclose(FH);    


  }

  if(  n  )
    throw Exception(Sprintf("Error reading file \'%s1\'!",fname));
  
  
  nbdim=bdim;
  ib0=b0;
  ib1=b1;

  if(excol){
    
    BWeights bweights;
    bweights.load(cname,pmesh,b0,b1);
    
    plabels=bweights.labels();


    if(1){
      if(bweights.ilower() != io) 
	throw Exception(
			Sprintf("The highest fully occupied band number %ld in "
				"file \'%s'\ndoes not match the settings %ld in "
				"file \'%s\'!\n" ,bweights.ilower()+1,cname,io+1,fname));
      if(bweights.iupper() != iu)
	throw Exception(
			Sprintf("The lowest fully unoccupied band number %ld in "
				"file \'%s'\ndoes not match the settings %ld in "
				"file \'%s\'!\n",bweights.iupper()+1,cname,iu+1,fname));
    }
    if(bweights.nband() != Nb)
      throw Exception(
		      Sprintf("The number of bands %ld in file \'%s'\n"
			      "does not match the settings %ld in file \'%s\'!\n"
			      ,bweights.nband(),cname,Nb,fname));
    if(bweights.nspin() != pnspin)
      throw Exception(
		      Sprintf("The number of spins %ld in file \'%s'\n"
			      "does not match the settings %ld in file \'%s\'!\n"
			      ,bweights.nspin(),cname,pnspin,fname));
    if(bweights.nkp() != nkfromfile)
      throw Exception(
		      Sprintf("The number of k-points %ld in file \'%s'\n"
			      "does not match the settings %ld in file \'%s\'!\n"
			      ,bweights.nkp(),cname,nkfromfile,fname));
    
  }
  

  pmesh->fillIrredFromIrredRot();


  cbandread=false;
  cbandread=true;


}

/* ***********************************************************************
 * make some calculations based on the tetras
 * ***********************************************************************
 */

QString Energies::analyseFermiSurface(
				double vuc,double tol
				,long lengthunit,Xf::RelativisticMode relatmode,
				const Vlevel &vlevel)

{
  NSF("Energies::analyseFermiSurface");
  //  myDebug() << nsubfunc;
  double ef=pef;
  if(!pemptylattice)
    ef=(pusedefaultEf?0.:pef);

  
  long prot=(vlevel >= Vlevel::More );

  QString rep;

  double twopi2=twopi*twopi;

  double fspin=(3.-pnspin);
  if(relatmode==Xf::FullRelativistic){
    fspin=1.;
  }
  

  
  if(prot)
    printf("%s: START ======================================\n",nsubfunc);  


  double conv=constant_hbar/(constant_me*constant_aBtoAng
		      *constant_angstroem
		      *constant_hatoev
		      );

  if(lengthunit == Xf::Angstroem ) {
    conv=conv/constant_aBtoAng; 
  }




  if(prot){
    printf("hbar  =%g\n",constant_hbar);
    printf("me    =%g\n",constant_me);
    printf("aB    =%g\n",constant_aBtoAng*constant_angstroem);
    printf("Ha/eV =%g\n",constant_hatoev);
    printf("conv: eV aB/hbar   =%g m/s\n",conv);
  }


  
    

  if(prot) printf("%s: erg.ndim %ld\n",nsubfunc,nbdim); 
  if(!nbdim) return "";
  

  double dosef[2],vFav[2],vF2av[2],totarea[2];
  Matrix3x3 vdv[2];

  for(int ms=0;ms<pnspin;ms++){
    
    dosef[ms]=0.;
    vFav[ms]=0.;
    vF2av[ms]=0.;
    /*  we have to meassure the cell size, since we are using larger volumes
	than the first Brillouin zone */
    double vol=0.; 
    
    
    for(long it=0;it<pmesh->NTetrahedra();it++){
      

      /* build the four corner vectors */ 
      const long *tetra=pmesh->tetrahedron(it);
      Vec3 qq0=pmesh->point(tetra[0])*twopi;/* put back the infamous factor 2*Pi */
      Matrix3x3 m;
      for(int i=1;i<4;i++)
	m.setColumn(i-1, pmesh->point(tetra[i])*twopi-qq0);
      double dV; bool indef;
      Matrix3x3 gg=m.inverseT(indef,dV);/* for gradient */
      
      vol+=fabs(dV)/6.0;
      
      /* for all bands */
      long iwk[6];
      double ei,ee[4];
      Vec3 pp[6];
      for(int jj=0;jj<pbands[ms].size();jj++){
	long ib=pbands[ms].at(jj);
	if(ib<ib0 || ib>ib1) break;
	long id=(ib-ib0+nbdim*ms);
	MeshData *md=pmesh->meshData().at(id);

	/* for all edges */
	int np=0;
	for(int i=0;i<4;i++){
	  long ki=tetra[i];
	  long iki=pmesh->IequivR(ki); 
	  ee[i]=ei=md->element(iki);
	  for(int j=i+1;j<4;j++){
	    long kj=tetra[j];
	    long ikj=pmesh->IequivR(kj); 
	    double ej=md->element(ikj);
	    double l=(ef-ei)/(ej-ei);
	    if( ( l >= 0.-tol ) && ( l <= 1.+tol) ){ 
	      pp[np]=pmesh->point(ki)*(1.-l)+pmesh->point(kj)*l;
	      iwk[np]=1;
	      np++;
	    }  /* l in I */
	  } /* j */
	}	/* i */
	
	/* gradient*/
	Vec3 grad;
	for(int i=0;i<3;i++) grad+=gg(i)*(ee[i+1]-ee[0]); 
	
	
	
	
	
	double absgrad=norm(grad);
	
	/* unify the set of  points */
	
	
	for(long i=0;i<np;i++)
	  for(long j=i+1;j<np;j++){
	    if(norm(pp[i]-pp[j])< tol)iwk[j]=0;
	  }
	    
	    
	
	long np0=0;
	Vec3 t[6];
	for(int i=0;i<np;i++){
	  if(iwk[i]){
	    t[np0]=pp[i];
	    np0++;
	  }
	}

	
	
	/* printf("np0 %d\n",np0);  */
	if(np0 == 4){
	  

  	  if(lineCross3(t[0],t[1],t[2],t[3],tol)){
	    Vec3 d=t[1];t[1]=t[2];t[2]=d;
  	  }
  	  if(lineCross3(t[0],t[3],t[1],t[2],tol)){
	    Vec3 d=t[0];t[0]=t[1];t[1]=d;
  	  }
	  
	  /* area of triangle */
	  Vec3 r1=t[0]-t[1];
	  double area=norm(r1.cross(t[0]-t[2]))*0.5*twopi2;
	  totarea[ms]+=area;
	  double cont=area/absgrad; 
	  dosef[ms]+=cont;
	  vFav[ms]+=absgrad*cont;
	  vF2av[ms]+=absgrad*absgrad*cont;
	  vdv[ms]+=grad.outer(grad)*cont;
	  
	  
	  
	  /* area of triangle */
	  r1=t[0]-t[2];
	  area=norm(r1.cross(t[0]-t[3]))*0.5*twopi2;
	  totarea[ms]+=area;
	  cont=area/absgrad; 
	  dosef[ms]+=cont;
	  vFav[ms]+=absgrad*cont;
	  vF2av[ms]+=absgrad*absgrad*cont;
	  vdv[ms]+=grad.outer(grad)*cont;
	  
	  
	  
	  
	}else if(np0 == 3){
	  
	  
	  /* area of triangle */
	  Vec3 r1=t[0]-t[1];
	  double area=norm(r1.cross(t[0]-t[2]))*0.5*twopi2;
	  totarea[ms]+=area;
	  double cont=area/absgrad; 
	  dosef[ms]+=cont;
	  vFav[ms]+=absgrad*cont;
	  vF2av[ms]+=absgrad*absgrad*cont;
	  vdv[ms]+=grad.outer(grad)*cont;
	  
	}else if( np0 >4){
	  exit(1);
	}
      } /* ib */
    } /* it */
    

    dosef[ms]*=fspin/vol;
    vFav [ms]*=fspin/vol;
    vF2av[ms]*=fspin/vol;
    vdv[ms]*=fspin/vol;
    
  } /* ms */


  conv=conv/1e5;
  double conv2=conv*conv;

  double tvFav=0.;
  double tvF2av=0.;
  double tdosef=0.;
  Matrix3x3  tvdv;
  if(pnspin>1){
    tdosef=dosef[0]+dosef[1];
    tvFav=(vFav[0]+vFav[1])/tdosef*conv;
    tvF2av=sqrt((vF2av[0]+vF2av[1])/tdosef)*conv;
    tvdv=(vdv[0]+vdv[1])/tdosef;
  }


  rep+="\n\n                     FERMI SURFACE PROPERTIES \n\n";

  for(int ms=0;ms<pnspin;ms++){
    
    vFav [ms]*=conv/dosef[ms];
    vF2av[ms]=sqrt(vF2av[ms]/dosef[ms])*conv;
    
    
    vdv[ms]/=dosef[ms];
      
    if(pbands[ms].size()>0){
      
      rep+=QString().sprintf("\nspin  = %d ---------"
			     "-------------------------------------\n",ms+1);
      
      rep+=QString().sprintf("DOS        : %f [states/Ha]\n\n"
			     ,dosef[ms]*constant_hatoev);
      rep+=QString().sprintf("<|vF|>     : %f [10^5 m/s]\n\n",vFav[ms]);
      rep+=QString().sprintf("sqrt(<vF2>): %f [10^5 m/s]\n\n",vF2av[ms]);
      rep+=QString().sprintf("<vov>      : %10.5f %10.5f %10.5f [10^5 m/s]^2\n"
			     "           : %10.5f %10.5f %10.5f \n"
			     "           : %10.5f %10.5f %10.5f \n\n"
	      ,vdv[ms](0,0)*conv2 ,vdv[ms](0,1)*conv2 ,vdv[ms](0,2)*conv2
              ,vdv[ms](1,0)*conv2 ,vdv[ms](1,1)*conv2 ,vdv[ms](1,2)*conv2
              ,vdv[ms](2,0)*conv2 ,vdv[ms](2,1)*conv2 ,vdv[ms](2,2)*conv2
			     );
      
      double fac=dosef[ms]*fourpi*constant_hatoev/vuc;
      if(lengthunit == Xf::Angstroem ) { fac=fac*constant_aBtoAng; }
      
      rep+=QString().sprintf("Plasma frequency from Lindhard epsilon"
			     " (intra band)\n"
			     "w_P        : %10.5f %10.5f %10.5f [eV]\n\n"
			     
			     ,sqrt(vdv[ms](0,0)*fac) ,sqrt(vdv[ms](1,1)*fac)
			     ,sqrt(vdv[ms](2,2)*fac) );
      
      rep+=QString().sprintf("sqrt(Trvov): %f [10^5 m/s]\n\n"
	     ,sqrt(vdv[ms](0,0)+vdv[ms](1,1)+vdv[ms](2,2))*conv);
    }
  }
  
  if(pnspin>1){


    rep+=QString().sprintf("\ntotal ------------------"
			   "--------------------------------\n");
    
    rep+=QString().sprintf("DOS        : %f [states/Ha]\n\n"
			   ,tdosef*constant_hatoev);
    rep+=QString().sprintf("<|vF|>     : %f [10^5 m/s]\n\n",tvFav );
    rep+=QString().sprintf("sqrt(<vF2>): %f [10^5 m/s]\n\n",tvF2av );
    rep+=QString().sprintf("<vov>      : %10.5f %10.5f %10.5f [10^5 m/s]^2\n"
	   "           : %10.5f %10.5f %10.5f \n"
	   "           : %10.5f %10.5f %10.5f \n\n"
	   
	   ,tvdv(0,0)*conv2 ,tvdv(0,1)*conv2 ,tvdv(0,2)*conv2
	   ,tvdv(1,0)*conv2 ,tvdv(1,1)*conv2 ,tvdv(1,2)*conv2
	   ,tvdv(2,0)*conv2 ,tvdv(2,1)*conv2 ,tvdv(2,2)*conv2 );
    
    double fac=tdosef*fourpi*constant_hatoev/vuc;
    if(lengthunit == Xf::Angstroem ) { fac=fac*constant_aBtoAng; }
    
    rep+=QString().sprintf("Plasma frequency from Lindhard epsilon (intra band)\n"
	   "w_P        : %10.5f %10.5f %10.5f [eV]\n\n"
	    ,sqrt(tvdv(0,0)*fac) ,sqrt(tvdv(1,1)*fac) ,sqrt(tvdv(2,2)*fac));
    
    rep+=QString().sprintf("sqrt(Trvov): %f [10^5 m/s]\n\n"
	   ,sqrt(tvdv(0,0)+tvdv(1,1)+tvdv(2,2))*conv);
  }

  rep+="\n\n";
    


  

  if(prot)printf("%s: END   ======================================\n",nsubfunc);  

  return rep;
}

/*EOF*/ 
