/* @(#)Erg.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ERG_H
#define _ERG_H 1

#include <QString> 
#include <QVector> 
#include "Xf.hh"

class MeshSymmetry;
class Boundary;
class Vlevel;
class OGLView;
class RegularMesh;
class TriangleMesh;
class Vec3;
class Matrix3x3;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Energies{
 public:
  Energies();
  ~Energies();

  void calcRange(const Vlevel &vlevel,bool takeAll=false);

  void load(const QString &file,const QString &cfile
	    ,const MeshSymmetry &sym
	    ,bool &cbandread
	    ,long icbandrestrict,long icbandmin,long icbandmax
	    ,const Vlevel &vlevel,bool excol);
  
  QString analyseFermiSurface(
			   double vuc,double tol
			   ,long lengthunit,Xf::RelativisticMode relatmode,
			   const Vlevel &vlevel);
  
  QString checkIndices();

  void setUpMesh(OGLView *oglview,const MeshSymmetry &sym
		 ,const Vlevel &vlevel
		 ,bool usecell,const Vec3 &origin,const Matrix3x3 &cell);
  void calculateIsoSurfaces(OGLView *oglview,const Boundary *bound,bool clip);
  

  void clear();

  inline long nspin()const {return pnspin;};
  inline long minBandAtEF(int ms)const {return nmin[ms];};
  inline long maxBandAtEF(int ms)const {return nmax[ms];};
  inline long minBand()const {return ib0;}; 
  inline long maxBand()const {return ib1;};
  inline long numBands()const {return nbdim;};
  inline bool emptyLattice()const{return pemptylattice;};
  inline void setEmptyLattice(bool e){pemptylattice=e;};
  inline double fermiLevel()const{return pef;};
  inline void setFermiLevel(double ef,bool usedefault)
    {pef=ef;pusedefaultEf=usedefault;};
  inline bool useDefaultEf()const {return pusedefaultEf;};
  inline void setRangeChanged(bool c){prangechanged=c;};
  inline bool rangeChanged()const {return prangechanged;};
  inline void setEfChanged(bool c){pefchanged=c;};
  inline bool efChanged()const {return pefchanged;};
  
  inline void setMeshDefChanged(bool c){pmeshchanged=c;
    pmeshcalculated=false;};
  inline bool meshDefChanged()const {return pmeshchanged;};
  inline void setMeshCalculated(bool c){pmeshcalculated=c;};
  inline bool meshCalculated()const {return pmeshcalculated;};
  inline void setXSubdiv(long n){pnx0subdiv=n;};
  inline long xSubdiv()const{return pnx0subdiv;};
  inline void setUseSymmetry(bool u){pusesymmetry=u;};
  inline bool useSymmetry()const{return pusesymmetry;};

  inline void setManualXSubdiv(long n){pnx0subdivmanual=n;};
  inline void setManualYSubdiv(long n){pny0subdivmanual=n;};
  inline void setManualZSubdiv(long n){pnz0subdivmanual=n;};
  inline void setManualSubDiv(long n){pmanualsubdiv=n;};
  inline long xSubdivManual()const{return pnx0subdivmanual;};
  inline long ySubdivManual()const{return pny0subdivmanual;};
  inline long zSubdivManual()const{return pnz0subdivmanual;};
  inline bool manualSubDiv()const{return pmanualsubdiv;};

  inline bool useBoundaryCell()const{return puseboundarycell;};
  inline void setUseBoundaryCell(bool u){puseboundarycell=u;};

  inline void setEnergiesLoaded(bool l){penergiesloaded=l;};
  inline bool energiesLoaded()const{return penergiesloaded;};


  void setDefaultBands();
  inline  void setBands(QVector<long> b[2]){pbands[0]=b[0];pbands[1]=b[1];};
  inline const QVector<long> &bands(int i)const {return pbands[i];};

  inline RegularMesh *mesh(){return pmesh;};
  inline QList<TriangleMesh*> &triameshs(){return ptriameshs;};


  const QString &bandFile()const{return pbandfile;};
  void setBandFile(const QString &b){ pbandfile=b;};

  const QString &bandWeightsFile()const{return pbandweightsfile;};
  void setBandWeightsFile(const QString &b){ pbandweightsfile=b;};

  const QStringList labels()const{return plabels;};
  
  QString label(int i)const{
    if(i>=0 && i<plabels.size())
      return plabels.at(i);
    else
      return QString("invalid");
  };

 private:
  void readbanderg(
		   const char *fname,const char *cname
		  ,bool &cbandread,long icbandrestrict
		  ,long icbandmin,long icbandmax,bool excol);
 private:
  bool pemptylattice;
  double pef;			/* Fermi level */
  bool pusedefaultEf;		/* use default Fermi level? */
  bool prangechanged;
  bool pefchanged;
  bool pmeshchanged;
  bool pmeshcalculated;
  bool penergiesloaded;
  bool pusesymmetry;
  long pnx0subdiv;		/* if symmetry is used, how many points
				   into the representative direction
				 */
  bool pmanualsubdiv;
  long pnx0subdivmanual;	/* manual overwrite used for slab-directions */
  long pny0subdivmanual;
  long pnz0subdivmanual;
  bool puseboundarycell;

  QVector<long> pbands[2];
    
  //dependends
  RegularMesh *pmesh;
  QList<TriangleMesh*> ptriameshs;
  long nbdim;			/* dimension-number of bands */
  long nkfromfile;			/* number of k-points in file +band */
  long ib0,ib1;			/* only band number ib0...ib1 are read 
				   from file
				*/
  
  double efermi;		/* fermi energy from file*/
  long nmin[2],nmax[2];   	/* lower and upper band index crossing 
				   fermi ernergy (from file) */
  long pnspin;			/* number of spin sorts, always =1 if FREL  */
  
  QString pbandfile;
  QString pbandweightsfile;
  QStringList plabels;

};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _ERG_H */

