  //todo: consider fplo nugen  input!!!
/***************************************************************************
                          fsdoc.cpp  -  description
                             -------------------
    begin                : Mon Mar 13 15:16:52 UTC 2000
    copyright            : (C) 2000 by Klaus Koepernik
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QApplication>
#include <QString>
#include <QMessageBox>
#include <math.h>
#include "Config.hh"
#include "Fog.hh"
#include "Symmetry.hh"
#include "MeshSymmetry.hh"
#include "Boundary.hh"
#include "FSDoc.hh"
#include "Vlevel.hh"
#include "Polyhedron.hh"
#include "Assorted.hh"
#include "ColorModel.hh"
#include "Spsp.hh"
#include "RegularMesh.hh"
#include "Light.hh"
#include "TriangleMesh.hh"
#include "DisplayCells.hh"
#include "Parser.hh" 
#include "Except.hh" 
#include "Sysinfo.hh"
#include "Axes.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSDoc::FSDoc(OGLView *oglview)
  :DocBase(oglview),xefsym(0),fploinsym(0),currentsym1(0),par(0)
{


  pview.setDefaultXYZRot();
  pview.setPerspective(false);
  pview.setTheta(10.);
  pview.setRadius(0.17);


  pdisplaycells->setDefault(Xf::LTReciprocial);
  xefsym=new  Symmetry();
  fploinsym=new  Symmetry();
  currentsym1=new  Symmetry();


  par=new Parser(1);

  specpoi=new SpecialSymPoints();
  defaultspecpoi=new SpecialSymPoints();

  newDoc();

  plights->setGlobalAmbientFactor(0.5);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSDoc::~FSDoc()
{
  //myDebug("FSDoc::~FSDoc");


  if(xefsym) delete xefsym;
  if(fploinsym) delete fploinsym;
  if(currentsym1) delete currentsym1;




  


  delete par;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSDoc::newDoc()
{
  // free structures


  erg.clear();
  specpoi->clear();
  specpoi->setVisible(false);
  defaultspecpoi->clear();
  defaultspecpoi->setVisible(false);
  






  iweight=1;

  




  erg.setFermiLevel(7,true);
  erg.setDefaultBands();
  
  
  
  // settings
  icbandrestrict=0;
  icbandmin=0;
  icbandmax=100000;
  write_tet_file=0;
  ufactor=6;
  ufactore=0;
  vunit=VUnits::METERperSEC;

  double fac=pow(10.,ufactor)*VUnits::velocityUnitFactor(vunit);  
  vmindef=1*fac;
  vmaxdef=2*fac;

  calc_only_part_occupied=1;
  part_occu_offset[0]=1;
  part_occu_offset[1]=1;
  vlevel=Vlevel::Info;
  tol=1.e-8;
  bclip1=true;
  
  


  setFileName("./Untitled");
  setNew();


  excol=0;

  vmin=10.;
  vmax=20.;
  vmine=0.;
  vmaxe=1.;
  vdefe=1;
  vdef=1;

  

  icut=0;
  ncut0[0]=0.;
  ncut0[1]=0.;
  ncut0[2]=1.;
  np=1;
  fro=-.2;
  to=.2;


  pstate.pgrid=pstate.pgridi=pstate.ptetra=pstate.pcubes=0;
  pstate.pfermi=1;

  pstate.pfill=1;
  pstate.pvelolegend=1;
  pstate.pvelolegunit=1;
  pstate.pcutcontour=1;
  pstate.pusenormals=1;
  pstate.pusesubdivs=0;

  legposx=0.1;
  legposy=0.15;
  
  pcanvas.setWidth(400);
  pcanvas.setHeight(400);


  surffname="surf.jpg";

  cutfname="cut.ps";
  fsfvfile="+fsfvel";
  fsfcfile="+fsfcvel";

  /* make flags*/
  resetflags();
  //myDebug() << "FSDoc::newDoc modified= " << modified;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Vec3 FSDoc::dataMidPoint() 
{
  const Vec3 &mp=bound()->midPoint();
  //myDebug() <<"mp="<<mp;
  return mp;
}

double  FSDoc::dataRadius()  
{
  Boundary *mb=bound();
  double radius=0;
  foreach(DisplayCell *dc,pdisplaycells->cells()){
    if(dc){
      Boundary *b=dc->boundary();
      if(b->polyhedron()->properties().isVisible()){
	radius=MyMath::max<double>(radius,b->radius()+norm(mb->midPoint()-b->midPoint()));
      }
    }
  }
  
  //myDebug() <<"radius"<<radius;
  return radius;
};



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSDoc::save(const QString &fname)
{
  
  try{
    saveData(fname);
  }catch(Except e){
    throw QString("%2").arg(e.msg());
  }
  modified=false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSDoc::load(const QString &filename)
{
  
  try{
    readData(filename );
  }catch (Except e){
    throw QString("%2 ").arg(e.msg());
  }


  setOld();
  /* reset flags */
  resetflags();
  
  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
/** notification about change of contents */
void FSDoc::refresh()
{
  //myDebug()<<"FSDoc::refresh " ;

  /* throw warning */
  if((! erg.emptyLattice()) && (! cbandread) && excol && (
					      isocolor.outerFromData() ||
					      isocolor.innerFromData() 
					      )){
    QMessageBox::warning(qApp->activeWindow(),"XfsDoc"
			 ,QString("File '%1' not found, colloring "
				  "option 'extern' leads to "
				  "unpredictable results!\n")
			 .arg(erg.bandWeightsFile())
			 ,QMessageBox::Yes);
  }





}

/* ***********************************************************************
 * reset flags
 * ***********************************************************************
 */

void  FSDoc::resetflags() 
{

  currentsym1->setCalculated(false);
  pdisplaycells->setCalculated(false);
  paxes->setCalculated(false);
  specpoi->setChanged(true);
  defaultspecpoi->setChanged(true);
  erg.setRangeChanged(true);
  erg.setEfChanged(true);
  erg.setMeshDefChanged(true);
  erg.setEnergiesLoaded(false);
  cbandread=false;
  triacalced = false;
  allcalced = false;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSDoc::loadEnergies(const QString &workdir,const QString &bandfile,
			 const QString &bandweightsfile)
{
  QString file=workdir + bandfile;
  QString cfile=workdir + bandweightsfile;
  
  meshsymmetry.setUpFromSymmetry(currentsym1);

  readRelativisticMode();
  

  erg.load(file,cfile,meshsymmetry,cbandread
	   ,icbandrestrict,icbandmin,icbandmax,vlevel,excol);
  
}






/* ***********************************************************************
 *
 * ***********************************************************************
 */

void FSDoc::readData(const QString &file )
{
  //  NSF("FSDoc::readData");

  String fname=file.toAscii().constData();
  
  
  if(pathExistence(fname()) == DoesNotExist)
    throw Except(String().sprintf("File \'%s\' does not exist!",fname()));
  
  //todo:: character
  par->parseFile(fname());
  
  String vm=(*par)()("version")("mainversion")();
  String vs=(*par)()("version")("subversion")();
  
  long t_emptylattice=(*par)()("empty_lattice").valueL();
  
  long t_write_tet_file=0;
  try{
    t_write_tet_file=(*par)()("write_tet_file").valueL(); 
  }catch(Except){};
  
  
  long t_calc_only_part_occupied=1;
  try{ t_calc_only_part_occupied=
      (*par)()("calc_only_partially_occupied").valueL();
  }catch(Except){};
  
  long t_part_occu_offset[2];
  t_part_occu_offset[0]=0;
  t_part_occu_offset[1]=0;
  
  try{
    t_part_occu_offset[0]=
      (*par)()("part_occu_offset")[0].valueL();
    t_part_occu_offset[1]=
      (*par)()("part_occu_offset")[1].valueL();
  }catch(Except){};
  
  
  long t_icbandrestrict=0;
  try{ 
    t_icbandrestrict=
      (*par)()("bandinput.restrict").valueL();
  }catch(Except){};

  
  
  long t_icbandmin=0;  
  long t_icbandmax=10000;  
  
  try{
    t_icbandmin=
      (*par)()("bandinput.bandmin").valueL();
    t_icbandmax=
      (*par)()("bandinput.bandmax").valueL();
  }catch(Except){};
  
  long t_ufactor=6;
  try{
    t_ufactor=(*par)()("velo_unit_factor").valueL();
  }catch(Except){};
  
  long t_ufactore=0;
  try{
    t_ufactore=(*par)()("excol_unit_factor").valueL();
  }catch(Except){};
  
  VUnits::VelUnits t_vunit=VUnits::METERperSEC; 
  try{
    t_vunit=(VUnits::VelUnits) (*par)()("velocity_unit.type").valueL();
  }catch(Except){};
  
  bool t_irred=false; 
  long t_nx=2; 
  t_irred=(*par)()("k_mesh.irreducible").valueL();
  t_nx=(*par)()("k_mesh.subdivision").valueL();

  bool t_manual=false; 
  try{
    t_manual=(*par)()("k_mesh.manualoverwrite").valueL();
  }catch(Except){};

  bool t_useboundarycell=false; 
  try{
    t_useboundarycell=(*par)()("k_mesh.useboundarycell").valueL();
  }catch(Except){};

  long t_nxm=0; 
  long t_nym=0; 
  long t_nzm=0; 
  try{
    t_nxm=(*par)()("k_mesh.xsubman").valueL();
  }catch(Except){};
  try{
    t_nym=(*par)()("k_mesh.ysubman").valueL();
  }catch(Except){};
  try{
    t_nzm=(*par)()("k_mesh.zsubman").valueL();
  }catch(Except){};

  
  // START old boundary reading for compatibility
  Boundary::CellType t_celltype=Boundary::Cell;
  Boundary::BasisType t_cellbasistype=Boundary::Primitive;
  try{
    long t_btype=Boundary::Primitive;
    t_btype=(*par)()("boundary_type.type").valueL();
    t_celltype=Boundary::cellTypefromCompressed(t_btype);
    t_cellbasistype=Boundary::cellBasisTypefromCompressed(t_btype);
  }catch(Except){};


  Boundary::BasisType t_shiftbasis=Boundary::Cartesian; 
  try{
    t_shiftbasis=(Boundary::BasisType)((*par)()("boundary_shift_system").valueL());
  }catch(Except){};
  
  String t_dg[3];
  t_dg[0]=t_dg[1]=t_dg[2]="0";
  try{
    (*par)()("cell_shift").getArray(3,t_dg);
  }catch(Except e){};
  
  String t_box[9]={"1","0","0","0","1","0","0","0","1"};
  try{
    (*par)()("boundary_box_vectors.V1").getArray(3,t_box+0);
    (*par)()("boundary_box_vectors.V2").getArray(3,t_box+3);
    (*par)()("boundary_box_vectors.V3").getArray(3,t_box+6);
  }catch(Except e){};


  Boundary::CenteringType t_centered=Boundary::Centered;
  try{
    t_centered=(Boundary::CenteringType)
      (*par)()("boundary_centering.type").valueL();
  }catch(Except){};
  // END old boundary
  
  /* START read single boundary */
  Boundary boundtmp;

  boundtmp.setCellType(t_celltype);
  boundtmp.setCellBasisType(t_cellbasistype);
  boundtmp.setLattice(Xf::LTReciprocial);
  boundtmp.setCentering(t_centered);
  boundtmp.setOrigin(t_dg,t_shiftbasis);
  boundtmp.setBox(t_box);

  bool havesingleboundary=false;
  try{
    boundtmp.readPTSectionFromFile((*par)()("boundary"));
    havesingleboundary=true;
  }catch(Except){};
  /* END read single boundary */

  
  /* display cells */
  DisplayCells displaycellstmp;
  displaycellstmp.setDefault(Xf::LTReciprocial);
  
  bool havedisplaycells=false;
  try{
    Dobj root=(*par)()("displayCells");
    displaycellstmp.readPTSectionFromFile(root);
    havedisplaycells=true;
  }catch(Except){}
  /* end display cells */
  

  Lights lighttmp;
  lighttmp.readPTSectionFromFile((*par)());
  *plights=lighttmp;


  // START old view settings
  try{
    double t_v0[9];
    (*par)()("view_axis").getArray(3,3,t_v0);
    pview.setZ(t_v0+6);
    pview.setUp(t_v0+3);
  }catch(Except e){};
  // END old view settings

  pview.readPTSectionFromFile(par);
  
  pcanvas.readCanvasPTSectionFromFile(par);

  
  bool t_clip=false;
  t_clip=(*par)()("clip_at_boundary").valueL();
  
  long t_pgrid=0; 
  t_pgrid=(*par)()("plot_grid").valueL();    
  
  long t_pgridi=0; 
  t_pgridi=(*par)()("plot_irred_grid").valueL();    
  
  long t_ptetra=0;
  t_ptetra=(*par)()("plot_tetras").valueL();    
  
  long t_pcube=0;
  t_pcube=(*par)()("plot_cubes").valueL();    
  
  long t_pfermi=1;
  t_pfermi=(*par)()("plot_fermi_surface").valueL();    
  
  //OLD axes
  bool t_paxesprim=false;
  try{
    t_paxesprim=(*par)()("plot_prim_axis").valueL();    
  }catch(Except){};
  
  bool t_paxesconv=false;
  try{
    t_paxesconv=(*par)()("plot_conv_axis").valueL();    
  }catch(Except){};
  
  bool t_paxescart=false;
  try{
    t_paxescart=(*par)()("plot_axis").valueL();    
  }catch(Except){};
  //END OLD axes





  paxes->findByType(Xf::ATCartesian)->setVisible(t_paxescart);
  paxes->findByType(Xf::ATConventional)->setVisible(t_paxesconv);
  paxes->findByType(Xf::ATPrimitive)->setVisible(t_paxesprim);

  paxes->readAxesPTSectionFromFile(par);  
  
  
  long t_pfill=1;
  try{
    t_pfill=(*par)()("plot_fill_triangles").valueL();    
  }catch(Except){};
  
  bool haveoldspecpoi=false;
  long t_pspec=0;
  try{
    t_pspec=(*par)()("plot_special_points").valueL();    
    haveoldspecpoi=true;
  }catch(Except){};
  
  long t_pvelolegend=1;
  try{
    t_pvelolegend=(*par)()("plot_velocity_legend").valueL();    
  }catch(Except){};
  
  long t_pvelolegunit=1;
  try{
    t_pvelolegunit=(*par)()("plot_velocity_legend_unit").valueL();    
  }catch(Except){};
  
  long t_pcutcontour=1;
  try{
    t_pcutcontour=(*par)()("plot_cut_contour").valueL();    
  }catch(Except){};

  long t_pusenormals=1;
  try{
    t_pusenormals=(*par)()("plot_use_normals").valueL();    
  }catch(Except){};

  long t_pusesubdivs=0;
  try{
    t_pusesubdivs=(*par)()("plot_use_subdivision").valueL();    
  }catch(Except){};
  
  
  QRgb t_ocol;
  QRgb t_icol;
  t_ocol=0xffff0000;
  t_icol=0xff4c4c4c;

  try{
    long t[3];
    (*par)()("surface_colors.outer_color").getArray(3,t);
    t_ocol=qRgba(t[0],t[1],t[2],255);
    (*par)()("surface_colors.inner_color").getArray(3,t);
    t_icol=qRgba(t[0],t[1],t[2],255);
  }catch(Except){
    Dobj c=(*par)()("surface_colors.outer_rgb");
    if(c.isInt()){
      t_ocol=c.valueL();
      t_ocol|=0xff000000;
    }else{
      t_ocol=c().convertToULong();
    }
    c=(*par)()("surface_colors.inner_rgb");
    if(c.isInt()){
      t_icol=(*par)()("surface_colors.inner_rgb").valueL();
      t_icol|=0xff000000;
    }else{
      t_icol=c().convertToULong();
    }
  }
  
  long t_ovelo=0; 
  long t_ivelo=0; 
  long t_excol=0; 
  t_ovelo=(*par)()("surface_colors.outer_velocity").valueL();    
  t_ivelo=(*par)()("surface_colors.inner_velocity").valueL();    
  t_excol=(*par)()("surface_colors.extern_coloring").valueL();    
  

  
  
  double t_vminmax[2];
  double t_vminmaxe[2];
  t_vminmax[0]=10.; t_vminmax[1]=20.; 
  t_vminmaxe[0]=0.; t_vminmaxe[1]=1.; 
  (*par)()("surface_colors.velocity_interval").getArray(2,t_vminmax);
  (*par)()("surface_colors.extern_color_interval").getArray(2,t_vminmaxe);
  
  long t_vdef=1; 
  long t_vdefe=1; 
  t_vdef=(*par)()("surface_colors.take_default_velocity_intervall")
    .valueL();    
  t_vdefe=(*par)()("surface_colors.take_default_ext_col_intervall")
    .valueL();    



  ColorMap t_cmap;
  try{
    Dobj root=(*par)()("surface_colors.colormap");
    t_cmap.readPTSectionFromFile(root);
  }catch(...){
    t_cmap.append(0,240,255,255);
    t_cmap.append(1,0,255,255);

  }

  
  long t_icut=0;       
  t_icut=(*par)()("surface_cuts.plot_cut").valueL();    
  
  double t_ncut[3];
  t_ncut[0]=t_ncut[1]=0.; t_ncut[2]=1.; 
  (*par)()("surface_cuts.plane").getArray(3,t_ncut);
  
  long t_np=1; 
  t_np=(*par)()("surface_cuts.number_of_cuts").valueL();
  
  double t_fro=-0.2; 
  double t_to=0.2; 
  t_fro=(*par)()("surface_cuts.distance_from").valueD();
  t_to=(*par)()("surface_cuts.distance_to").valueD();
  
  
  double t_ef=0.5; 
  t_ef=(*par)()("fermi_energy").valueD();
  
  long t_defef=1;
  t_defef=(*par)()("use_default_ef").valueL();
  

  QString t_bandfile="+band";
  try{
    t_bandfile=(*par)()("bandfile")()();
  }catch(...){};
  
  FSDoc::LegendControl t_legcontrol;
  try{
    Dobj d=(*par)()("legendcontrol");
    t_legcontrol.useminlabel=d("useminlabel").valueL();
    t_legcontrol.minlabel=d("minlabel")();
    t_legcontrol.usemaxlabel=d("usemaxlabel").valueL();
    t_legcontrol.maxlabel=d("maxlabel")();
    t_legcontrol.usetitle=d("usetitle").valueL();
    t_legcontrol.title=d("title")();
    t_legcontrol.showunits=d("showunits").valueL();
  }catch(...){};


  Vlevel::VerbosityLevel t_vlevel=Vlevel::More; 
  t_vlevel=(Vlevel::VerbosityLevel )
    ((*par)()("verbosity_level.type").valueL());
  
  double t_tol=1e-5; 
  t_tol=(*par)()("tolerance").valueD();
  
  Symmetry symtmp;
  symtmp.setStructureType(Xf::STCrystal);
  Symmetry::readSymmetryPTSectionFromFile(par,symtmp,false);

  
  MeshSymmetry meshsymtmp;
  MeshSymmetry::readMeshSymmetryPTSectionFromFile(par,meshsymtmp);
  
  

  Fog fogtmp;
  fogtmp.readPTSectionFromFile((*par)());

  bool havenewspecpoi=false;
  SpecialSymPoints specpoitmp;
  try{
    specpoitmp.readPTSectionFromFile((*par)());
    havenewspecpoi=true;
  }catch(Except &e){}
  
  
  QVector<long> t_bands[2];

  int done=0;
  Dobj d;
  try{
    d=(*par)()("band_indicesup");
  }catch(Except){
    // does not exist, isold file
    t_bands[0].append((*par)()("band_index").valueL());
    done=1;
  }
  if(!done){
    t_bands[0].resize(d.size());
    for(int i=0;i<t_bands[0].size();i++) t_bands[0][i]=i;
    d.getArray(d.size(),t_bands[0].data());
    
  }

  done=0;
  try{
    d=(*par)()("band_indicesdn");
  }catch(Except){
    done=1;
  }
  if(!done){
    t_bands[1].resize(d.size());
    for(int i=0;i<t_bands[1].size();i++) t_bands[1][i]=i;
    d.getArray(d.size(),t_bands[1].data());
    
  }
  
  QString t_weightsfile="+bweights";
  try{
    t_weightsfile=(*par)()("weightsfile")()();
  }catch(...){}
  try{
    t_weightsfile=(*par)()("bandweightsfile")()();
  }catch(...){}


  long t_iweight;
  try{
    t_iweight=(*par)()("iweight").valueL();
  }catch(Except){
    t_iweight=1;
  }

  

  iweight=t_iweight;
  erg.setBandWeightsFile(t_weightsfile);
  erg.setEmptyLattice(t_emptylattice);
  write_tet_file=t_write_tet_file;
  calc_only_part_occupied=t_calc_only_part_occupied;
  part_occu_offset[0]=t_part_occu_offset[0];
  part_occu_offset[1]=t_part_occu_offset[1];
  icbandrestrict=t_icbandrestrict;
  icbandmin=t_icbandmin;
  icbandmax=t_icbandmax;
  ufactor=t_ufactor;
  ufactore=t_ufactore;
  vunit=t_vunit;
  erg.setXSubdiv(t_nx);
  erg.setManualSubDiv(t_manual);
  erg.setManualXSubdiv(t_nxm);
  erg.setManualYSubdiv(t_nym);
  erg.setManualZSubdiv(t_nzm);
  erg.setUseSymmetry(t_irred);
  erg.setUseBoundaryCell(t_useboundarycell);



  *pdisplaycells=displaycellstmp;


  
  if(!havedisplaycells && havesingleboundary){
    PolyhedronProperties props=bound()->polyhedron()->properties();
    *(bound())=boundtmp;
    bound()->polyhedron()->properties()=props;
  }

  pdisplaycells->setCalculated(false);
  pdisplaycells->setChanged(false);

  //pdisplaycells->PTSectionToFile(stderr);


  *pfog=fogtmp;

  if(havenewspecpoi){
    *specpoi=specpoitmp;
  }else{
    if(haveoldspecpoi)
      specpoi->setVisible(t_pspec);
    specpoi->setUserDefined(false);
  }
  specpoi->setChanged(true);




  


  bclip1=t_clip;
  pstate.pgrid=t_pgrid;
  pstate.pgridi=t_pgridi;
  pstate.ptetra=t_ptetra;
  pstate.pcubes=t_pcube;
  pstate.pfermi=t_pfermi;

  pstate.pfill=t_pfill;
  pstate.pvelolegend=t_pvelolegend;
  pstate.pvelolegunit=t_pvelolegunit;
  pstate.pcutcontour=t_pcutcontour;
  pstate.pusenormals=t_pusenormals;
  pstate.pusesubdivs=t_pusesubdivs;


  


  isocolor.setOuterColor(t_ocol);
  isocolor.setInnerColor(t_icol);
  
  isocolor.setInnerFromData(t_ivelo);
  isocolor.setOuterFromData(t_ovelo);
  isocolor.colorModel()->setColorMap(t_cmap);
  excol=t_excol;
  vmin=t_vminmax[0];
  vmax=t_vminmax[1];
  vmine=t_vminmaxe[0];
  vmaxe=t_vminmaxe[1];
  vdef=t_vdef;
  vdefe=t_vdefe;
  icut=t_icut;
  ncut0[0]=t_ncut[0];
  ncut0[1]=t_ncut[1];
  ncut0[2]=t_ncut[2];
  np=t_np;
  fro=t_fro;
  to=t_to;
  
  erg.setFermiLevel(t_ef,t_defef);
  
  vlevel=t_vlevel;
  tol=t_tol;

  *xefsym=symtmp;
  
  meshsymmetry=meshsymtmp;
  
  erg.setBands(t_bands);
  erg.setBandFile(t_bandfile);
  

  legendcontrol=t_legcontrol;

}

/* ***********************************************************************
 * IO
 * ***********************************************************************
 */
 
void FSDoc::saveData(const QString &fname)
{
  FILE *FH;


//   INTEGER *t_bands[2],bsize[2];

  
//   t_bands[0]=NULL;
//   t_bands[1]=NULL;
//   bsize[0]=0;
//   bsize[1]=0;

  String filename(fname.toAscii().constData());

  /* write input file */
  FH=openforwrite(filename);
  fprintf(FH,"%s\n\n",
"########################################################################\n\
# this file is part of the FPLO bandstructure package\n\
# it was created automatically\n\
# please dont delete this file nor\n\
# edit it, unless you really know what you are doing\n\
# any changes added by you may be lost if this file is recreated\n\
# by FPLO (which is the normal behaviour )\n\
# so keep duplicates of your changes somewhere else\n\
#\n\
########################################################################");
  /* header with version  numbers*/
  fprintf(FH,"section header{\n\n");
  fprintf(FH,"  struct {char[*] application;char[*] description;} filetype\n"
	  "  ={\"xef\",\"FPLO-Graphics\"};\n\n");

  fprintf(FH,"  struct {char[*] mainversion;char[*] subversion;} version={");
  fprintf(FH,"\"%s\",\"%s\"};\n"
	  ,FPLOMAINVERSION()
	  ,FPLOSUBVERSION());
  fprintf(FH,"\n\n  struct {char[*] hostname;char[*] date;char[*] by;} last_changes\n    ={");
  
  fprintf(FH,"\"%s\",\"%s\",\"%s\"};\n"
	  ,Sysinfo::getHostname()(),Sysinfo::getDateAndTime()()
	  ,(EXECNAMESHORT+" ("+FPLOMAINVERSION+"-"+FPLORELEASE+")")());
  fprintf(FH,"};\n\n\n");
  
  /* section options*/
  fprintf(FH,"section options{\n");
  fprintf(FH,"    logical empty_lattice=%c;\n"
	  ,(erg.emptyLattice() != 0?'t':'f'));
  fprintf(FH,"    logical write_tet_file=%c;\n"
	  ,(write_tet_file != 0?'t':'f'));
  fprintf(FH,"    logical calc_only_partially_occupied=%c;\n"
	  ,(calc_only_part_occupied != 0?'t':'f'));
  fprintf(FH,"    int part_occu_offset[2]={%ld,%ld};\n"
	  ,part_occu_offset[0],part_occu_offset[1]);
  fprintf(FH,"    struct {int type;char[*] description;} verbosity_level\n");
  fprintf(FH,"    ={%d,\"%s\"};\n",vlevel.level(),vlevel.name());
  
  /* end section options */  
  fprintf(FH,"};\n\n\n");
  
  
  
  /* section input control*/
  fprintf(FH,"section input_control{\n");
  fprintf(FH,"    struct{logical restrict;int bandmin;int bandmax;}"
	  " bandinput={%c,%ld,%ld}; \n"
	  ,(icbandrestrict != 0 ?'t':'f'),icbandmin,icbandmax);
  /* end section input control  */  
  fprintf(FH,"};\n\n\n");
  
  
  /* section mesh_generation*/
  fprintf(FH,"section mesh_generation{\n");
  fprintf(FH,"    struct {logical irreducible;int subdivision;\n"
	  "               logical manualoverwrite; int xsubman;\n"
	  "               int ysubman;int zsubman;\n"
	  "               logical useboundarycell;\n"
	  "              } k_mesh\n");
  fprintf(FH,"    ={%c,%ld,%c,%ld,%ld,%ld,%c};\n",(erg.useSymmetry() ?'t':'f')
	  ,erg.xSubdiv()
	  ,(erg.manualSubDiv() ?'t':'f')
	  ,erg.xSubdivManual()
	  ,erg.ySubdivManual()
	  ,erg.zSubdivManual()
	  ,(erg.useBoundaryCell() ?'t':'f')
	  );
  /* end section mesh_generation */  
  fprintf(FH,"};\n\n\n");


  /* old boundary stuff */
  fprintf(FH,"# for downward compatibility only we write the old boundary stuff\n");
  
  bound()->PTHeaderToFile(FH,"","=");
  bound()->PTBodyToFile(FH,"",";\n\n");

  /* display cells */
  pdisplaycells->PTSectionToFile(FH);

  
  paxes->axesPTSectionToFile(FH);
  
   /* section plot*/
  fprintf(FH,"section plot{\n");
  

  fprintf(FH,"    logical clip_at_boundary=%c;\n",(bclip1 ?'t':'f'));
  
  
  fprintf(FH,"    logical plot_grid=%c;\n", (pstate.pgrid != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_irred_grid=%c;\n", (pstate.pgridi != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_tetras=%c;\n", (pstate.ptetra != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_cubes=%c;\n", (pstate.pcubes != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_fermi_surface=%c;\n", (pstate.pfermi != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_fill_triangles=%c;\n"
	  , (pstate.pfill != 0 ?'t':'f'));
  //for old
  fprintf(FH,"    logical plot_special_points=%c;\n"
	  , (specpoi->isVisible() != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_velocity_legend=%c;\n"
	  , (pstate.pvelolegend != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_velocity_legend_unit=%c;\n"
	  , (pstate.pvelolegunit != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_cut_contour=%c;\n"
	  , (pstate.pcutcontour != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_use_normals=%c;\n"
	  , (pstate.pusenormals != 0 ?'t':'f'));
  fprintf(FH,"    logical plot_use_subdivision=%c;\n"
	  , (pstate.pusesubdivs != 0 ?'t':'f'));

  fprintf(FH,"    struct {logical useminlabel;char[*] minlabel;\n"
	  "               logical usemaxlabel;char[*] maxlabel;\n"
	  "               logical usetitle;   char[*] title;\n"
	  "               logical showunits;\n"
	  "       }legendcontrol={%c,\"%s\",%c,\"%s\",%c,\"%s\",%c};\n"
	  ,(legendcontrol.useminlabel ?'t':'f')
	  ,legendcontrol.minlabel()
	  ,(legendcontrol.usemaxlabel ?'t':'f')
	  ,legendcontrol.maxlabel()
	  ,(legendcontrol.usetitle ?'t':'f')
	  ,legendcontrol.title()
	  ,(legendcontrol.showunits ?'t':'f')
	  );

  

  pview.PTSectionToFile(FH);
  pcanvas.canvasPTSectionToFile(FH);

  pfog->PTSectionToFile(FH);


  specpoi->PTSectionToFile(FH);
  
  fprintf(FH,"    struct {\n"
	  "      char[*] inner_rgb;\n"
	  "      char[*] outer_rgb;\n"
	  "      logical inner_velocity;\n"
	  "      logical outer_velocity;\n"
	  "      logical extern_coloring;\n"
	  "      real velocity_interval[2];\n"
	  "      real extern_color_interval[2];\n"
	  "      logical take_default_velocity_intervall;\n"
	  "      logical take_default_ext_col_intervall;\n"
	  "      %s;\n"
	  "    }surface_colors\n",
	  isocolor.colorModel()->colorMap().PTHeaderToString()()
	  );
  fprintf(FH,"    ={%s,%s\n"
	  "      ,%c,%c\n"
	  "      ,%c\n"
	  "      ,{%-39.15g,%-39.15g}\n"
	  "      ,{%-39.15g,%-39.15g}\n"
	  "      ,%c\n"
	  "      ,%c\n"
	  "      ,%s\n"
	  "    };\n"
	  ,Sprintf("\"%x\"",isocolor.innerColor())()
	  ,Sprintf("\"%x\"",isocolor.outerColor())()
	  ,(isocolor.innerFromData() != 0 ?'t':'f')
	  ,(isocolor.outerFromData() != 0 ?'t':'f')
	  ,(excol != 0 ?'t':'f')
	  ,vmin,vmax
	  ,vmine,vmaxe
	  ,(vdef != 0 ?'t':'f')
	  ,(vdefe != 0 ?'t':'f')
	  ,isocolor.colorModel()->colorMap().PTSectionToString()()
	  );


  
  fprintf(FH,"    struct {int type;char[*] description;} "
	  "velocity_unit\n");
  fprintf(FH,"    ={%ld,\"%s\"};\n",(long)vunit,VUnits::velocityUnit(vunit));
  fprintf(FH,"    int velo_unit_factor=%ld;\n", ufactor);
  fprintf(FH,"    int excol_unit_factor=%ld;\n", ufactore);
  
  fprintf(FH,"    struct {\n"
	  "      logical plot_cut;\n"
	  "      real plane[3];\n"
	  "      int number_of_cuts;\n"
	  "      real distance_from;\n"
	  "      real distance_to;\n"
	  "    }surface_cuts\n");
  fprintf(FH,"    ={%c\n"
	  "      ,{%-30.15g,%-30.15g,%-30.15g}\n"
	  "      ,%ld\n"
	  "      ,%-30.15g\n"
	  "      ,%-30.15g\n"
	  "    };\n"
	  ,(icut != 0 ?'t':'f') ,ncut0[0],ncut0[1],ncut0[2] ,np ,fro,to );
  
  
  /* end section plot  */  
  fprintf(FH,"};\n\n\n");
  
  plights->PTSectionToFile(FH);
  
  /* section fermi_suface*/
  fprintf(FH,"section fermi_suface{\n");
  fprintf(FH,"    real fermi_energy=%-30.15g;\n",erg.fermiLevel());
  fprintf(FH,"    logical use_default_ef=%c;\n"
	  ,(erg.useDefaultEf() != 0 ?'t':'f'));

  fprintf(FH,"    char[*] bandfile=\"%s\";\n",String(erg.bandFile())());
  fprintf(FH,"    char [*] bandweightsfile=\"%s\";\n",
	  String(erg.bandWeightsFile())());

  fprintf(FH,"    # this is obsolete\n    "
	  "int band_index=1;\n");
  
  fprintf(FH,"    # and this is new\n    int band_indicesup[*]={");
  for(int i=0;i<erg.bands(0).size();i++){ 
    fprintf(FH,"%ld",erg.bands(0).at(i)); 
    if(i<erg.bands(0).size()-1) fprintf(FH,","); 
  }
  fprintf(FH,"};\n"); 
  
  fprintf(FH,"    int band_indicesdn[*]={");
  for(int i=0;i<erg.bands(1).size();i++){ 
    fprintf(FH,"%ld",erg.bands(1).at(i)); 
    if(i<erg.bands(1).size()-1) fprintf(FH,","); 
  }
  fprintf(FH,"};\n"); 
  
  fprintf(FH,"# this is obsolete\n    int spin_sort=1;\n");
  /* end section fermi_suface  */  
  fprintf(FH,"};\n\n\n");
  
  /* section numerics*/
  fprintf(FH,"section numerics{\n");
  fprintf(FH,"    real tolerance=%-30.15g;\n",tol);
  /* end section numerics  */  
  fprintf(FH,"};\n\n\n");
  
  
  /* section temporary*/
  fprintf(FH,"section temporary{\n");
  fprintf(FH,"    int iweight=%ld;\n",iweight);
  meshsymmetry.meshSymmetryPTSectionToFile(FH);
  /* end section numerics  */  
  fprintf(FH,"};\n\n\n");
  
  
  
  /* section symmetry*/
  fprintf(FH,"# the following symmetry is used in empty lattice mode "
	  "only!\n\n");
  xefsym->symPTSectionToFile(FH,false);
  fclose(FH);
  
  return;
  
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSDoc::saveKPoints(const QString &workdir)
{
  //  NSF("writekpoints");

  String fname=(workdir+"=.kp").toAscii().constData();
  
  FILE *FH=openforwrite(fname());

  double a=currentsym1->latticeConstant(0);
  
  RegularMesh *mesh=erg.mesh();
  
  long nk=mesh->numIrredRotPoints();
  fprintf(FH,"%ld %c %ld %ld\n",nk
	  ,(calc_only_part_occupied ?'t':'f')
	  ,part_occu_offset[0]
	  ,part_occu_offset[1]
	  );
  for(long ipi=0;ipi<nk;ipi++){
    Vec3 Q=mesh->point(mesh->RequivI_Rot(ipi))*a;
    fprintf(FH,"%s\n" ,Q.printS("%-30.15g")());
  }
  fclose(FH);


  


//   if(write_tet_file){
//     fname=(workdir+"=.tet").toAscii().constData();
//     FH=openforwrite(fname());
    
//     long nk=tetra.nk;
//     fprintf(FH,"# k-mesh full BZ\n%ld\n",nk);
//     for(long ik=0;ik<nk;ik++){
//       Vec3 Q=tetra.Q[ik]*a;
//       fprintf(FH,"%s\n" ,Q.printS("%-30.15g")());
//     }
    
//     nk=(tetra.irred?tetra.nki:tetra.nk);
//     fprintf(FH,"# mapping irreducible onto full k-mesh \n%ld\n",nk);
//     for(long ik=0;ik<nk;ik++){
//       if( tetra.irred ) {
// 	fprintf(FH,"%-8ld\n" ,tetra.ieqi[ik]+1);
//       }else{
// 	fprintf(FH,"%-8ld\n" ,ik+1);
//       }
      
//     } /* for ik */
      
//     fprintf(FH,"# mapping full k-mesh onto irreducible \n%ld\n",tetra.nk);
      
//     for(long ik=0;ik<tetra.nk;ik++){
//       if( tetra.irred ) {
// 	fprintf(FH,"%-8ld\n" ,tetra.ieq[ik]+1);
//       }else{
// 	fprintf(FH,"%-8ld\n" ,ik+1);
//       }
//     }
//     fprintf(FH,"# tetrahedra in full BZ \n%ld\n",tetra.ntet);
//     for(long it=0;it<tetra.ntet;it++){
//       fprintf(FH,"%-8ld %-8ld %-8ld %-8ld\n" 
// 	      ,tetra.tet[0+4*it]+1
// 	      ,tetra.tet[1+4*it]+1
// 	      ,tetra.tet[2+4*it]+1
// 	      ,tetra.tet[3+4*it]+1
// 	      );
//     }
//     fclose(FH);
//   }
  
   
  
}
  

