 /***************************************************************************
                          fsdoc.h  -  description
                             -------------------
    begin                : Mon Mar 13 15:16:52 UTC 2000
    copyright            : (C) 2000 by Klaus Koepernik
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef FSDOC_H
#define FSDOC_H

// include files for QT
#include "DocBase.hh"
#include "Erg.hh"
#include "IsoColor.hh"
#include "FString.hh"
#include "Vunit.hh"
#include "Vlevel.hh"
#include "MeshSymmetry.hh"





class SpecialSymPoints;
class Parser;
class OGLView;

/**
  * the Document Class
  */

class FSDoc : public DocBase
{
  Q_OBJECT



    public:
    FSDoc(OGLView *oglview);
    ~FSDoc();

    class LegendControl{
    public:
      LegendControl():useminlabel(false),usemaxlabel(false),usetitle(false),
	minlabel("min"),maxlabel("max"),title(""),showunits(true){};

      bool useminlabel;
      bool usemaxlabel;
      bool usetitle;
      String minlabel;
      String maxlabel;
      String title;
      bool showunits;
    };



    void save(const QString &filename);
    void load(const QString &filename);


    // void loaded(){emit documentChanged();};
    void refresh();
    void resetflags();
    void saveKPoints(const QString &workdir);

    virtual bool showLegend(){ 
      return (pstate.pvelolegend && 
	      (isocolor.innerFromData()||isocolor.outerFromData())); };
    virtual double legendPositionY(){ return legposy; };



    virtual Vec3 dataMidPoint();
    virtual double  dataRadius();
      
    // energy band related
    void loadEnergies(const QString &workdir,const QString &bandfile,
		      const QString &bandweightsfile);


    Symmetry *xefsym;
    Symmetry *fploinsym;
    Symmetry *currentsym1;
    MeshSymmetry meshsymmetry;


    Energies erg;
    SpecialSymPoints *specpoi;
    SpecialSymPoints *defaultspecpoi;





    bool bclip1;
    Vlevel vlevel;


    double tol;
    long write_tet_file;
    long calc_only_part_occupied;
    long part_occu_offset[2];
    long ufactor;
    long ufactore;
    VUnits::VelUnits vunit;


    
    struct tspstate{
      long pgrid;
      long pgridi;
      long ptetra;
      long pcubes;
      long pfermi;
      long pfill;
      long pvelolegend;
      long pvelolegunit;
      long pcutcontour;
      long pusenormals;
      long pusesubdivs;
    };


    


    QString surffname;
    QString cutfname;
    QString fsfvfile;
    QString fsfcfile;

    struct tspstate pstate;
      


    IsoColor isocolor;
    long excol;
    long  vdef;
    long  vdefe;
    long  icbandmin;
    long  icbandmax;
    long  icbandrestrict;
    double vmin,vmax;
    double vmindef,vmaxdef;
    double vminedef,vmaxedef;
    double vmine,vmaxe;
    long iweight;


    double legposx,legposy;



    bool complainedWeightSym;
    
    long icut;
    double ncut0[3];
    double fro,to;
    long np;
      
    
  protected:

 private:
    void saveData(const QString &fname);
    void readData(const QString &fname);

    /* flags */
 public:    


    LegendControl legendcontrol;

    bool cbandread;
    bool triacalced;

    Parser *par;

 private:
    void newDoc();



};

#endif
