/*************************************************************************
 * @(#)Field.cpp
 * author: magru@unas
 * date:   17 Jun 2013
 *************************************************************************
 */

#include "Field.hh"
#include "DataStatistics_cpp.hh"

using namespace XFPLO;



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Field::calcStatistics(bool prot)
{
  if(prot) myDebug()<<"Field::calcStatistics()";
  switch(pdata.dataType()){
  case TIlong:
    pdatastatistics->calculate<long>(pdata.to<long>());
    break;
  case TIdouble:
    pdatastatistics->calculate<double>(pdata.to<double>());
    break;
  case TICmplx:
    pdatastatistics->calculate<Cmplx>(pdata.to<Cmplx>());
    break;
  case TIunkown:
    break;
  }
  pposstatistics->calculate<double>(ppositions->array());
  if(prot){
    myDebug()<< "Positions are enclosed between"
	     <<pposstatistics->data<double>()->minValues<<"and"
	     <<pposstatistics->data<double>()->maxValues;
    
    myDebug()<< "Data are enclosed between"
	     <<pdatastatistics->data<double>()->minValues<<"and"
	     <<pdatastatistics->data<double>()->maxValues;
  }  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Field::cascadingUpdate(const String &prefix)
{
  if(Module::debugmode) printf("%s%s::cascadingUpdate\n",prefix(),id()());
  Module::cascadingUpdate(prefix+"  ");
  showDirt(prefix,Sprintf("%s must react to",id()()));
  bool ret=false;
  //if(!ppositions) return;
  if(pisdirty || dirtyInput("Positions")){
    calcStatistics(Module::debugmode);
    sendDirt("Statistics changed");
    resetDirt("Positions");
    ret=true;
  }
  if(pisdirty || dirtyInput("Connections")){
    sendDirt("changed");
    resetDirt("Connections");
    ret=true;
  }
  if(pisdirty){
    sendDirt("changed");
    ret=true;
  }
  resetDirt();
  if(Module::debugmode) printf("%sret=%d\n",prefix(),ret);
  return ret;
}
