/* @(#)Field.hh
 * author: magru@unas
 */

#ifndef _FIELD_HH
#define _FIELD_HH 1

#include "Array.hh"
#include "Module.hh"
#include "MyDebug.hh"
#include "Position.hh"
#include "Connection.hh"
#include "DataStatistics.hh"

namespace XFPLO{

class BoundingBox;

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class Field: public Module
{
public:
  Field():Module("Field"),ppositions(Positions::construct()),
	  pconnections(Connections::construct()),//,neighbours(0)
	  pdatastatistics(new DataStatistics),
	  pposstatistics(new DataStatistics),pnormals(0),
	  pcolors(0)
  {
    Module::connect(ppositions,this);
    Module::connect(pconnections,this);
    Module::connect(ppositions,pconnections);
  }
  virtual ~Field(){
    if(ppositions) Positions::destroy(ppositions); ppositions=0;
    if(pconnections) Connections::destroy(pconnections); pconnections=0;
    //if(neighbours) delete [] neighbours ; neighbours=0;
    if(pdatastatistics) delete pdatastatistics;pdatastatistics=0;
    if(pposstatistics) delete pposstatistics;pposstatistics=0;
    deleteNormals();
    deleteData();
    deleteColors();
  }

  void setGridD(long vectordimension,
		const ValueArray<long> &gridsize, // Nx x Ny ( x Nz)
		const ValueArray<double> &origin, // one origin for all
		const ValueArray<ValueArray<double> > &deltas
		){
    ppositions->setGridD(vectordimension,gridsize,origin,deltas);
    pconnections->setPositionDependency(true);
  }

  template<class T>
  void setupData(const ValueArray<long> &shape){
    pdata.build<T>(shape);
  }

  BaseArrayPointer &data(){return pdata;}
  const BaseArrayPointer &data()const{return pdata;}

  inline BaseArray<double> *normals(){return pnormals;}
  inline const BaseArray<double> *normals()const{return pnormals;}


  inline BaseArray<double> *colors(){return pcolors;}
  inline const BaseArray<double> *colors()const{return pcolors;}
  
  void setupNormals(const ValueArray<long> &shape){
    deleteNormals();
    pnormals=BaseArray<double>::construct(shape);
  }
  void setupColors(const ValueArray<long> &shape){
    deleteColors();
    pcolors=BaseArray<double>::construct(shape);
  }

  Positions *positions(){return ppositions;}
  const Positions *positions()const{return ppositions;}
  
  Connections *connections(){return pconnections;}
  const Connections *connections()const{return pconnections;}

  const BoundingBox &boundingBox()const{return ppositions->boundingBox();}

  const DataStatistics *dataStatistics()const{return pdatastatistics;}

  virtual bool cascadingUpdate(const String &prefix);
  
  void calcStatistics(bool prot);
  
protected:
  void deleteData(){
    pdata.clean();
  }
  void deleteNormals(){
    if(pnormals){ pnormals->destroy(pnormals); pnormals=0; }
  }
  void deleteColors(){
    if(pcolors){ pcolors->destroy(pcolors); pcolors=0; }
  }
private:
  Positions *ppositions;
  Connections *pconnections;
  DataStatistics *pdatastatistics;
  DataStatistics *pposstatistics;
  BaseArray<double> *pnormals; //must have the same shape as ppositions
  BaseArray<double> *pcolors; 
  BaseArrayPointer pdata;
  // OGLMaterial pmatfront;//todo: introduce a simple material class and copy onto OGLMaterial if needed
  // OGLMaterial pmatback;

};



}
#endif /* _FIELD_HH */

