/* @(#)Fog.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _FOG_H
#define _FOG_H 1

#include <stdio.h>
#include "MyQRgb.hh"

class Dobj;

class Fog{
 public:

  enum FogMode{Exp=1,Gaussian=2,Linear=3};

  Fog():pchanged(false),pusefog(false),pdensity(0.04)
    ,pnear(0.25),pfar(1.3),pcolor(0xffffffff)
    ,pfogmode(Linear){};

  inline void setDefault(){
    pdensity=0.04;
    pnear=0.25;
    pfar=1.3;
    pcolor=0xffffffff;
    pfogmode=Linear;
  };

  inline FogMode fogMode()const {return pfogmode;};
  inline void setFogMode(FogMode u) {pfogmode=u;pchanged=true;};

  inline bool useFog()const {return pusefog;};
  inline void setUseFog(bool u) {pusefog=u;pchanged=true;};


  inline QRgb color()const {return pcolor;};
  inline void setColor(QRgb u) {pcolor=u;pchanged=true;};

  inline double density()const {return pdensity;};
  inline void setDensity(double d) {pdensity=d;pchanged=true;};

  inline double near()const {return pnear;};
  inline void setNear(double d) {pnear=d;pchanged=true;};

  inline double far()const {return pfar;};
  inline void setFar(double d) {pfar=d;pchanged=true;};

  inline bool changed()const{return pchanged;};
  inline void setChanged(bool c){pchanged=c;};

  static const char *fogModeName(FogMode m){return pfogmodenames[m];};

  void PTSectionToFile(FILE *FH);
  void readPTSectionFromFile(const Dobj &root);

 private:
  bool pchanged;
  bool pusefog;
  double pdensity;
  double pnear,pfar;
  QRgb pcolor;
  FogMode pfogmode;
  static const char *pfogmodenames[];
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _FOG_H */

