/*************************************************************************
 * @(#)Fplo.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Jul 2009
 *************************************************************************
 */

#include <QString> 
#include <QDir> 
#include <QMessageBox> 
#include <sys/utsname.h>
#include "Settings.hh"
#include "Parser.hh"
#include "Assorted.hh"
#include "Except.hh"
#include "Fplo.hh"
#include "qt5compat.hh"


QString Fplo::puname="";

const char *ExecPathes::pkeynames[]={
  "ENV","Dir","Exec"
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecPathes::ExecPathes()
{
  readPathSettings();
  if(pkeys.count()<1)
    setDefault();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecPathes::~ExecPathes()
{
  savePathSettings();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathes::setDefault()
{
  append(Directory,QString(getenv("HOME"))+"/FPLO/bin");
  append(EnvVar,"PATH");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathes::readPathSettings()
{
  clear();
  Settings set("fplo");
  int n=set.beginReadArray("ExecPathes");
  for (int i = 0; i < n; ++i) {
    set.setArrayIndex(i);
    PatternType key=(PatternType)set.getInt("type");
    QString v=set.getString("name");
    append(key,v);
    
  }
  set.endArray();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathes::savePathSettings()
{
  Settings set("fplo");
  set.beginWriteArray("ExecPathes");
  for(int i=0;i<pkeys.count();i++){
    set.setArrayIndex(i);
    set.set("type",pkeys.at(i));
    set.set("name",value(pkeys.at(i)));
  }
  set.endArray();


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QStringList ExecPathes::unfold()
{
  savePathSettings();
  
  QStringList ppathes;

  foreach(PatternType key,pkeys){
    if(key==ExecPathes::Directory){
      QString dir=value(key);
      int i;
      if( (i=ppathes.indexOf(dir))>=0){
	ppathes.removeAt(i);
      }
      ppathes.append(dir);
    }
    if(key==ExecPathes::Executable){
      QString dir=value(key);
      int i;
      if( (i=ppathes.indexOf(dir))>=0){
	ppathes.removeAt(i);
      }
      ppathes.append(dir);
    }
    if(key==ExecPathes::EnvVar){
      QStringList sl=QString(getenv(value(key)
				    .toAscii().constData())).split(":");
      sl.sort();
      foreach(const QString &dir,sl){
	if( !ppathes.contains(dir)){
	  ppathes.append(dir);
	}
      }
      
      
    }
  }



  return ppathes;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fplo::Fplo(const QString &infilename,const QString &outfile)
  :poutfile(outfile),pcheckbandplot(false),pcheckinitialssplit(false)
  ,pforcesymupdate(false),pmessage(""),ptermcode("")
{  
  detMachine();

  
  pcurrentWorkDir=QFileInfo(infilename).absolutePath();
  //myDebug()<<"pcurrentWorkDir="<<pcurrentWorkDir;


  if(pathExistence(String(infilename)()) == DoesNotExist){
    throw Sprintf("Cannot find '=.in' in current directory:\n  %s\n",
 		  String(pcurrentWorkDir)());
  }else{
    //    pcurrentWorkDir=QDir::currentPath();
  }

  try{
    readVersions(infilename,pmainversion,plastchanged);
  }catch(Except &e){};

  pprograms=getPrioritizedFeditList(pmainversion,plastchanged);


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fplo::~Fplo()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fplo::detMachine()
{
  if(puname.isEmpty()){
    // determine uname
//     { // older Qt-based version
//       QProcess *proc=new QProcess();
//       proc->start("uname",QStringList()<< "-m");
//       if(!proc->waitForFinished()){
// 	myDebug() << "cannot execute 'uname -m', please report this";
// 	exit(1);
//       }
//       puname=proc->readAll().trimmed();
//       delete proc;
//     }
//     { // does that compile on MAC?
//       struct utsname u;
//       ::uname(&u);
//       puname=u.machine;
//     }    
    {
      puname=(sizeof(long)==8?"x86_64":"i386");
    }
  }
  //myDebug()<<"puname"<<puname;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
QStringList Fplo::getFeditList(const QString &pattern)
{

  ppathes=pexecpathes.unfold();

  
//   foreach(QString s,ppathes){
//     myDebug() << "PATH="<<s;
//   }
  

  QStringList res;
  foreach( QString path, ppathes) {
    QFileInfo fi(path);
    if(fi.isExecutable()&&fi.isFile()){
      res.append(path);
    }else if(fi.isDir()){

      QDir d(path);
      QStringList l=d.entryList(QStringList(pattern)
				,QDir::Files | QDir::Readable | 
 			      QDir::Executable);
      foreach(QString s,l){
	res.append(path+"/"+s);
      }
    }
  }

  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QStringList Fplo::getPrioritizedFeditList(const QString &mainversion
					  ,const QString &lastchanged)
{
  QString ver=lastchanged;
  QString rel="";
  QString sys="";
  QString hov="";
  QString miv="";
  {
    QRegExp rx;
    rx.setPattern("^([^-]+)-(.*)$");
    
    long pos = rx.indexIn(ver);
    if (pos > -1) {
      ver=rx.cap(1);
      rel=rx.cap(2);
    }
    pos = rx.indexIn(rel);
    if (pos > -1) {
      rel=rx.cap(1);
      sys=rx.cap(2);
    }

    rx.setPattern("^(\\d+)\\.(\\d+)$");
    pos = rx.indexIn(ver);
    if (pos > -1) {
      hov=rx.cap(1);
      miv=rx.cap(2);
    }
    
  }
  //myDebug() << "ver" << ver << "rel"<< rel  << "sys"<< sys;
  //myDebug() << "uname"<< uname();
  //myDebug() << "hov,miv"<< hov<<miv;

  QStringList programs;
  QStringList p=getFeditList("*fplo*");
  QRegExp rx;
  if(!ver.isEmpty() && !rel.isEmpty() && ! (puname.isEmpty()) ){
    //myDebug() << " ver-rel-uname match";
    rx.setPattern(QString(".*fplo")+QString("%1-%2-%3")
		  .arg(ver).arg(rel).arg(puname)+".*");
    //myDebug() << "pattern" << rx.pattern();
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	if(!programs.contains(s))
	  programs.append(s);
      }
    }
  }
  if(!ver.isEmpty() && !rel.isEmpty() && ! sys.isEmpty() ){
    //myDebug() << " ver-rel-sys match";
    rx.setPattern(QString(".*fplo")+QString("%1-%2-%3")
		  .arg(ver).arg(rel).arg(sys)+".*");
    //myDebug() << "pattern" << rx.pattern();
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	if(!programs.contains(s))
	  programs.append(s);
      }
    }
  }
  if(!ver.isEmpty() && !rel.isEmpty() ){
    //myDebug() << " ver-rel match";
    rx.setPattern(QString(".*fplo")+QString("%1-%2")
		  .arg(ver).arg(rel)+".*");
    //myDebug() << "pattern" << rx.pattern();
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	if(!programs.contains(s))
	  programs.append(s);
      }
    }
  }
  if(!ver.isEmpty()  ){
    //myDebug() << " ver match";
    rx.setPattern(QString(".*fplo")+QString("%1").arg(ver)+".*");
    //myDebug() << "pattern" << rx.pattern();
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	if(!programs.contains(s))
	  programs.append(s);
      }
    }
  }

  if(!mainversion.isEmpty()  ){
    //myDebug() << " mainver match";
    rx.setPattern(QString(".*fplo")+QString("%1").arg(mainversion)+".*");
    //myDebug() << "pattern" << rx.pattern();
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	if(!programs.contains(s))
	  programs.append(s);
      }
    }
  }


  if(!hov.isEmpty()  ){
    //myDebug() << " hov match";
    rx.setPattern(QString(".*fplo")+QString("%1\\.").arg(hov)+".*");
    //myDebug() << "pattern" << rx.pattern();
    QStringList l;
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	l.append(s);
      }      
    }
    qStableSort(l.begin(),l.end());
    for(int i=l.size()-1;i>=0;i--){
      if(!programs.contains(l[i]))
	programs.append(l[i]);
    }
  }

  if(!hov.isEmpty()  ){
    //myDebug() << " hov match";
    rx.setPattern(QString(".*fplo")+QString("%1").arg(hov)+".*");
    //myDebug() << "pattern" << rx.pattern();
    QStringList l;
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	l.append(s);
      }      
    }
    qStableSort(l.begin(),l.end());
    for(int i=l.size()-1;i>=0;i--){
      if(!programs.contains(l[i]))
	programs.append(l[i]);
    }
  }


  {
    //myDebug() << " hov match";
    rx.setPattern(QString(".*fplo\\d+.*"));
    //myDebug() << "pattern" << rx.pattern();
    QStringList l;
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	l.append(s);
      }      
    }
    qStableSort(l.begin(),l.end());
    for(int i=l.size()-1;i>=0;i--){
      if(!programs.contains(l[i]))
	programs.append(l[i]);
    }
  }
  {
    //myDebug() << " hov match";
    rx.setPattern(QString(".*fplo.*"));
    //myDebug() << "pattern" << rx.pattern();
    QStringList l;
    foreach(QString s,p){
      if (rx.indexIn(s) > -1) {
	l.append(s);
      }      
    }
    qStableSort(l.begin(),l.end());
    for(int i=l.size()-1;i>=0;i--){
      if(!programs.contains(l[i]))
	programs.append(l[i]);
    }
  }

  return programs;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fplo::readVersions(const QString &file,QString &mainversion
			,QString &lastchanged)
{
  mainversion="";
  lastchanged="";
  
  Parser *par=new Parser(1);
  
  String fname=file.toAscii().constData();
  if(pathExistence(fname()) == DoesNotExist)
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  par->parseFile(fname());
  String vm,vs,lc;
  try{
    vm=(*par)()("version")("mainversion")();
    vs=(*par)()("version")("subversion")();
    lc=(*par)()("last_changes")("by")();
  }catch(Except e){};

  lastchanged=lc();
  QRegExp rx("^.*[(]([^()]*)[)].*$");
  int pos = rx.indexIn(lastchanged);
  if (pos > -1) lastchanged=rx.cap(1);


  delete par;
  mainversion=vm();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Fplo::prepareConditions()
{
  try{
    if(pcheckbandplot){
      if(!checkBandplot()) return false;
    }
    if(pcheckinitialssplit){
      if(!checkInitialssplit()) return false;
    }
    if(pforcesymupdate){
      forceSymUpdate();
    }
  }catch(Except e){
    QMessageBox::warning(0,"Checking bandplot flags"
			 ,QString("Some error occured in checking the fplo"
				  " input settings (bandplot, initial spin split,"
				  " symmetry update):\n %1\n ")
			 .arg(e.msg())
			 ,QMessageBox::Ok| QMessageBox::Default);
    return false;
  }

  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Fplo::forceSymUpdate()
{
  Parser *par=new Parser(1);

  QString file=pcurrentWorkDir+"/=.sym";
  
  String fname(file);
  if(pathExistence(fname()) == DoesNotExist){
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  }
  par->parseFile(fname());
  
  Dobj d=(*par)()("status");

  d="symmetry changed";
  
  par->writeFile(fname);


  delete par;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Fplo::checkBandplot()
{
  Parser *par=new Parser(1);

  QString file=pcurrentWorkDir+"/=.in";
  
  String fname(file);
  if(pathExistence(fname()) == DoesNotExist){
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  }
  par->parseFile(fname());
  
  String vm;
  bool bandplot;
  Dobj d;
  try{
    d=(*par)()("bandplot_control.bandplot");
    bandplot=d.valueL();
  }catch(Except e){
    throw Except(String().sprintf("Cannot read the bandplot flags "
				  "in file \'%s\':\n %s\n!"
				  ,fname(),e.msg()));
  }
  if(!bandplot){
    int ret=QMessageBox::warning(0,"Checking bandplot flags"
				 ,QString("It appears that the bandplot is not"
					  " switched on in\n %1.\n\n"
					  " Shall I do it?").arg(file)
				 ,QMessageBox::Yes | QMessageBox::Default
				 ,QMessageBox::No  
				 );
    if(ret==QMessageBox::Yes){
      //myDebug() << "yes";
      d="t";
      par->writeFile(fname);

    }else
      return false;

  }

  delete par;
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Fplo::checkInitialssplit()
{
  Parser *par=new Parser(1);

  QString file=pcurrentWorkDir+"/=.in";
  
  String fname=file.toAscii().constData();
  if(pathExistence(fname()) == DoesNotExist){
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  }
  par->parseFile(fname());
  
  String vm;
  bool initialssplit;
  Dobj d;
  try{
    d=(*par)()("spin.initial_spin_split");
    initialssplit=d.valueL();
  }catch(Except e){
    throw Except(String().sprintf("Cannot read the initial spin split flag "
				  "in file \'%s\':\n %s\n!"
				  ,fname(),e.msg()));
  }
  if(initialssplit){
    int ret=QMessageBox::warning(0,"Checking initial spin split flag."
				 ,QString("It appears that the initial "
					  "spin split is switched on"
					  " in\n %1.\n\n"
					  " Shall switch it off?").arg(file)
				 ,QMessageBox::Yes | QMessageBox::Default
				 ,QMessageBox::No  
				 );
    if(ret==QMessageBox::Yes){
      //myDebug() << "yes";
      d="f";
      par->writeFile(fname);

    }else
      return false;

  }

  delete par;
  return true;
}


//  long run(const QString &workdir);







/*EOF*/ 
