/* @(#)fplo.h
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _FPLO_H
#define _FPLO_H 1

#include <QStringList>
#include <QMap>


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ExecPathes{
 public:
  enum PatternType{EnvVar=0,Directory=1,Executable=2};
  
  ExecPathes();
  ~ExecPathes();


  QStringList unfold();

  void setDefault();
  
  void clear(){pkeys.clear();pvalues.clear();};

  void readPathSettings();
  void savePathSettings();

  void append(PatternType t,const QString &v){
    pkeys.append(t);
    pvalues.append(v);
  }
  
  QList<PatternType> keys()const{return pkeys;};
  QString value(PatternType key)const{
    int index=pkeys.indexOf(key);
    if(index>=0) 
      return pvalues.at(index);
    else 
      return "";
  };

  static const char *keyName(PatternType key){
    return pkeynames[key];
  }

 private:
  static const char *pkeynames[];
  QList<PatternType> pkeys;
  QList<QString> pvalues;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Fplo
{
public:
  Fplo(const QString &infilename,const QString &outfile);
  ~Fplo();

  
  const QString &message()const{return pmessage;};
  void setMessage(const QString &m){pmessage=m;};

  void setCheckBandPlot(bool c){pcheckbandplot=c;};
  void setForceSymUpdate(bool c){pforcesymupdate=c;};
  void setCheckInitialSSplit(bool c){pcheckinitialssplit=c;};
  void setForceTerminationCode(const QString &code){ptermcode=code;};
  const QString &terminationCode()const{return ptermcode;};

  long run(const QString &workdir);


  bool prepareConditions();
  
  const QString &mainVersion()const{return pmainversion;};
  const QString &lastChanged()const{return plastchanged;};
  const QString &currentWorkDir()const{return pcurrentWorkDir;};
  const QString &outFile()const{return poutfile;};
  const QStringList &programs()const{return pprograms;};

  const ExecPathes &execPathes()const{return pexecpathes;};
  void setExecPathes(const ExecPathes &ep){
    pexecpathes=ep;
    pprograms=getPrioritizedFeditList(pmainversion,plastchanged);
  };


private:

  void detMachine();
  QStringList getFeditList(const QString &pattern);
  QStringList getPrioritizedFeditList(const QString &mainversion
				      ,const QString &lastchanged);

  static void readVersions(const QString &file,QString &mainversion
			   ,QString &lastchangedby);
  bool checkBandplot();
  bool checkInitialssplit();
  void forceSymUpdate();

private:
  QStringList ppathes;
  QStringList pprograms;
  QString pcurrentWorkDir;
  QString pmainversion;
  QString plastchanged;
  QString poutfile;
  static QString puname;
  bool pcheckbandplot;
  bool pcheckinitialssplit;
  bool pforcesymupdate;
  QString pmessage;
  QString ptermcode;
  ExecPathes pexecpathes;
};






#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _FPLO_H */

