/*************************************************************************
 * @(#)IsoSurface.cpp
 * author: magru@unas
 * date:   24 Jun 2013
 *************************************************************************
 */

#include "Except.hh"
#include "Array.hh"
#include "Array_cpp.hh"
#include "IsoSurface.hh"
#include "Field.hh"
#include "AutoColor.hh"
#include "MyDebug.hh"






using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
IsoSurface::IsoSurface()
  :Module("IsoSurface"),pinputfield(0),poutputfield(new Field()),
   pusegradientasnewdata(false)
{
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
IsoSurface::~IsoSurface()
{
  delete poutputfield;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IsoSurface::setInput(const Module *input)
{
  if(input->classname()=="Field"){
    pinputfield=dynamic_cast<const Field*>(input);
  }else if(input->classname()=="AutoColor"){
    pinputfield=dynamic_cast<const AutoColor*>(input)->field();
  }else{
    throw Except("IsoSurface::setInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IsoSurface::removeInput(const Module *input)
{
  //myDebug()<<pclassname<<"removeInput"<<(void*)input<<input->classname();
  // dynamic_cast no longer possible, since called from destructor
  if(input->classname()=="Field"){
    pinputfield=0;
  }else if(input->classname()=="AutoColor"){
    pinputfield=0;
  }else{
    throw Except("IsoSurface::setInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool IsoSurface::cascadingUpdate(const String &prefix)
{
  if(Module::debugmode) printf("%s%s::cascadingUpdate\n",prefix(),id()());
  Module::cascadingUpdate(prefix+"  ");
  showDirt(prefix,Sprintf("%s must react to",id()()));
  bool ret=false;
  //printf("%spinputfield=%p\n",prefix(),pinputfield);
  if(pisdirty || dirtyInput("Field") || dirtyInput("AutoColor")){
    sendDirt("changed");
    calcIsoSurface();
    ret=true;
  }
  resetDirt();
  if(Module::debugmode) printf("%sret=%d\n",prefix(),ret);
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IsoSurface::calcIsoSurface()
{
  if(!pinputfield) return;
  
  const BaseArrayPointer &bap=pinputfield->data();

  

  //myDebug()<<"datatype="<<bap.dataType();

  switch(bap.dataType()){
  case TIlong:
    calcIsoSurface_internal<long>();
    break;
  case TIdouble:
    calcIsoSurface_internal<double>();
    break;
  case TICmplx:
    throw Except("iso surface of a complex function of position not implemented\n");
    //calcIsoSurface_internal<Cmplx>();
    break;
  case TIunkown:
    break;
  }

  
}

#include "IsoSurface_internal_cpp.hh"

/*EOF*/ 
