/*************************************************************************
 * @(#)IsoSurface_internal_cpp.hh
 * author: magru@unas
 * date:   24 Jun 2013
 *************************************************************************
 */

#include "Position.hh"
#include "Connection.hh"
#include "Assorted.hh"
#include "PatternCount.hh"
#include "Vec3.hh"


#define CROSS(vres,v1,v2)			\
  vres[0]=v1[1]*v2[2]-v1[2]*v2[1];		\
  vres[1]=v1[2]*v2[0]-v1[0]*v2[2];		\
  vres[2]=v1[0]*v2[1]-v1[1]*v2[0];
#define SCALAR(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])
#define DIFFVEC(vres,v1,v2)			\
  vres[0]=v1[0]-v2[0];				\
  vres[1]=v1[1]-v2[1];				\
  vres[2]=v1[2]-v2[2];
#define DEF_DIFFVEC(vres,v1,v2) double vres[3];DIFFVEC(vres,v1,v2);
/* ***********************************************************************
 *
 * ***********************************************************************
 */
#ifdef MY_DEBUGGING
static Vec3 gradient(double *d,double x,double y,double z,double &f)
{
  f=(d[0]*(1-x)*(1-y)+d[1]*x*(1-y)+d[2]*(1-x)*y+d[3]*x*y)*(1-z)+
    (d[4]*(1-x)*(1-y)+d[5]*x*(1-y)+d[6]*(1-x)*y+d[7]*x*y)*z;

  double fx=(-d[0]*(1-y)+d[1]*(1-y)-d[2]*y+d[3]*y)*(1-z)+
    (-d[4]*(1-y)+d[5]*(1-y)-d[6]*y+d[7]*y)*z;
  double fy=(-d[0]*(1-x)-d[1]*x+d[2]*(1-x)+d[3]*x)*(1-z)+
    (-d[4]*(1-x)-d[5]*x+d[6]*(1-x)+d[7]*x)*z;
  double fz=-(d[0]*(1-x)*(1-y)+d[1]*x*(1-y)+d[2]*(1-x)*y+d[3]*x*y)+
    (d[4]*(1-x)*(1-y)+d[5]*x*(1-y)+d[6]*(1-x)*y+d[7]*x*y);

  printf("f=%lg grad=[%lg,%lg,%lg]\n",f,fx,fy,fz);
  return Vec3(fx,fy,fz);
}
#endif


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void detPattern3DCub(unsigned char &x,int &s,int *crn,int mat[3])
{
  //return and s=-1 if some effective inversion was used (orientation)
  // modify x into standard patterns, return permutated corners in crn
  // and return 3x3 operation matrix in mat
  int co=MyMath::bitCount(x); 
  
  //invert bits, nothing to be done to crn
  if(co<4) { x^=0xff; co=8-co;}
  else if(co==4) x=MyMath::max<long>(x^0xff,x);
  co=8-co;
  
  if(co==0) return; //here count==0 is ready
  
  unsigned char x0=MyMath::bitCount(x& 0x55);
  unsigned char x1=MyMath::bitCount(x& 0xaa);
  unsigned char y0=MyMath::bitCount(x& 0x33);
  unsigned char y1=MyMath::bitCount(x& 0xcc);
  unsigned char z0=MyMath::bitCount(x& 0x0f);
  unsigned char z1=MyMath::bitCount(x& 0xf0);
#define xyztest()							\
  if(x0!=MyMath::bitCount(x& 0x55)){					\
    myDebug()<<"error in x0"<<x0<<MyMath::bitCount(x& 0x55);		\
    exit(1);								\
  }									\
  if(x1!=MyMath::bitCount(x& 0xaa)){					\
    myDebug()<<"error in x1"<<x1<<MyMath::bitCount(x& 0xaa);		\
    exit(1);								\
  }									\
  if(y0!=MyMath::bitCount(x& 0x33)){					\
    myDebug()<<"error in y0"<<y0<<MyMath::bitCount(x& 0x33);		\
    exit(1);								\
  }									\
  if(y1!=MyMath::bitCount(x& 0xcc)){					\
    myDebug()<<"error in y1"<<y1<<MyMath::bitCount(x& 0xcc);		\
    exit(1);								\
  }									\
  if(z0!=MyMath::bitCount(x& 0x0f)){					\
    myDebug()<<"error in z0"<<z0<<MyMath::bitCount(x& 0xf0);		\
    exit(1);								\
  }									\
  if(z1!=MyMath::bitCount(x& 0xf0)){					\
    myDebug()<<"error in z1"<<z1<<MyMath::bitCount(x& 0x0f);		\
    exit(1);								\
  }
  
  
  s=1;
  // make z0<z1, use s(z)
  if(z0>z1){
    //myDebug()<<"s(z)";//exit(1);
    //s(z)
    x=((x&0xf)<<4)| ((x&0xf0)>>4);
    s*=-1;
    MyMath::swap(z0,z1);
    MyMath::swap(crn[0],crn[4]);
    MyMath::swap(crn[1],crn[5]);
    MyMath::swap(crn[2],crn[6]);
    MyMath::swap(crn[3],crn[7]);
    mat[2]*=-1;
  }
  // make y0<y1, use s(y)
  if(y0>y1){
    //myDebug()<<"s(y)";//exit(1);
    //s(y)
    x=((x&0x33)<<2)| ((x&0xcc)>>2);
    s*=-1;
    MyMath::swap(y0,y1);
    MyMath::swap(crn[0],crn[2]);
    MyMath::swap(crn[1],crn[3]);
    MyMath::swap(crn[4],crn[6]);
    MyMath::swap(crn[5],crn[7]);
    mat[1]*=-1;
  }
  // make x0<x1, use s(x)
  if(x0>x1){
    //myDebug()<<"s(x)";//exit(1);
    //s(x)
    x=((x&0x55)<<1)| ((x&0xaa)>>1);
    s*=-1;
    MyMath::swap(x0,x1);
    MyMath::swap(crn[0],crn[1]);
    MyMath::swap(crn[2],crn[3]);
    MyMath::swap(crn[4],crn[5]);
    MyMath::swap(crn[6],crn[7]);
    mat[0]*=-1;
  }
  
  
  if(x==0xfe) return; //here count==1 343434is ready
  // shift largest x1 to z1 x<->z
  if(x1>z1){
    //myDebug()<<"x<->z";//exit(1);
    x=((x&0xa)<<3)| ((x&0x50)>>3)| (x&0xa5);
    s*=-1;
    MyMath::swap(x0,z0);	
    MyMath::swap(x1,z1);	
    MyMath::swap(crn[1],crn[4]);
    MyMath::swap(crn[3],crn[6]);
    MyMath::swap(mat[0],mat[2]);
  }
  // shift largest y1 to z1 y<->z
  if(y1>z1){
    //myDebug()<<"y<->z";//exit(1);
    x=((x&0xc)<<2)| ((x&0x30)>>2)|(x&0xc3);
    s*=-1;
    MyMath::swap(y0,z0);	
    MyMath::swap(y1,z1);	
    MyMath::swap(crn[2],crn[4]);
    MyMath::swap(crn[3],crn[5]);
    MyMath::swap(mat[1],mat[2]);
  }
  if(x==0xf8) return; //here count==3:232314 is ready
  if(x==0xf0) return; //here count==4:222204 is ready

  // shift largest x1 to y1 x<->y
  if(x1>y1){
    //myDebug()<<"x<->y";//exit(1);
    x=((x&0x22)<<1)| ((x&0x44)>>1)|(x&0x99);
    s*=-1;
    MyMath::swap(x0,y0);	
    MyMath::swap(x1,y1);
    MyMath::swap(crn[1],crn[2]);
    MyMath::swap(crn[5],crn[6]);
    MyMath::swap(mat[0],mat[1]);
  }
  xyztest();
  if(x==0xfc) return; //here count==2:332424 is ready
  if(x==0xe9) return; //here count==3:232323 is ready
  if(x==0xe8) return; //here count==4:131313 is ready
  if(x==0x96) return; //here count==4:222222 is ready
  
  if(co==3 || (co==4 && (x==0x99 || x==0xc3))){
    while(x!=0xda && x!=0xa5){
      //rotat C3
      //myDebug()<<"C3(111)";//exit(1);
      x=((x&0x2)<<1)|((x&0x4)<<2)|((x&0x8)<<3)| ((x&0x10)>>3)|((x&0x20)>>2)|
	((x&0x40)>>1)| (x&0x81) ;
      MyMath::swap3(crn[4],crn[2],crn[1]);
      MyMath::swap3(crn[6],crn[3],crn[5]);
      MyMath::swap3(mat[0],mat[2],mat[1]);
    }
  }
  if(x==0xa5) return; //here count==4:222222 a5 is ready
  if(x==0xda) return; //here count==3:232323 da is ready
  if(co==4 && (x==0xe4)){
    // s(x)
    //myDebug()<<"s(x)";//exit(1);
    x=((x&0x55)<<1)| ((x&0xaa)>>1);
    s*=-1;
    MyMath::swap(crn[0],crn[1]);
    MyMath::swap(crn[2],crn[3]);
    MyMath::swap(crn[4],crn[5]);
    MyMath::swap(crn[6],crn[7]);
    mat[0]*=-1;
  }
  if(x==0xd8) return; //here count==4:221313 d8 is ready
  unsigned char m=0x1;
  if(co==4) m=0x10;
  while((x&m)){
    //rotate C43
    //myDebug()<<"C43(z)";//exit(1);
    x=((x&0x11)<<1)| ((x&0x22)<<2)| ((x&0x44)>>2)| ((x&0x88)>>1);
    MyMath::swap4(crn[1],crn[0],crn[2],crn[3]);
    MyMath::swap4(crn[5],crn[4],crn[6],crn[7]);
    MyMath::swap(mat[0],mat[1]);
    mat[0]*=-1;
  }
  //here count==2:333324 f6 is ready
  //here count==2:333333 7e is ready
  //here count==4:222213 e1 is ready
  return;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void unifyData(ValueArray<double> &//vapo
	       ,ValueArray<long> &//vaco
	       ,ValueArray<double> &//vano
	       )
{
  return;
  // long napo=vapo.size();
  
  // double *dvapo=vapo.dataRef();
  // double *dvano=vano.dataRef();

  // for(long ip=-1;++ip<napo;){
  //   //Vec3 normal(dvano+3*ip);
  //   for(long jp=vapo.size();--jp>ip;){
  //     myDebug()<<ip<<jp;
  //     // if(pos==posp){
  //     // 	dvano+=dvano+3*j;
  //     // }
  //     // find(con with posj)->replace with posi;
  //   }
  //   // normal.normalize();
  //   // dvano+3*i=normal;
  // }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

template <class X>
static void precalcGradients(const BaseArray<double> *bapi,
			     const Connections *coi,const BaseArray<X> *bdi,
			     ValueArray<double> &vagri
			     )
{
  const BaseArray<long> *baci=coi->array();
  if(bdi->dataType()==TICmplx)
    throw Except("gradients for complex function not implemented\n");
  if(bdi->datasize()!=1)
    throw Except("gradients for non scalar functions not implemented");

  long embeddingdimension=bapi->shape(0);
  long consize=baci->size();
  long conshape=baci->shape(0);
  // dimensionality of mesh, cannot be infered from positions, unless it is
  // a productarray
  long conD=0;
  if(coi->connectionType()==Connections::CTYCuboid){
    conD=dRound(::pow(conshape,0.5));
  }else{
    throw Except("connectiontype not implemented");
  }

  //myDebug()<<"embeddingdimension="<<embeddingdimension;
  //myDebug()<<"connection size="<<consize;
  //myDebug()<<"connection shape="<<conshape;
  //myDebug()<<"conD="<<conD;

  if(conD>embeddingdimension)
    throw Except("Connection element has higher dimension than embedding space.");

  long Nsize=bapi->size();
  
  const X *ddat=bdi->data();
  const long *dcon=baci->data();
  const double *dpos=bapi->data();

  ValueArray<long> multi(Nsize,0L);
  vagri.resize(Nsize*3);
  vagri=0.;
  double *dni=vagri.dataRef();
  for(long i=-1;++i<consize;){//for all connections
    if(coi->connectionType()==Connections::CTYCuboid){
      if(conD==3){
	//embedding space must be 3D
	long icon=i*conshape;
	long indx0=dcon[icon+0];
	long indx1=dcon[icon+1];
	long indx2=dcon[icon+2];
	long indx3=dcon[icon+3];
	long indx4=dcon[icon+4];
	long indx5=dcon[icon+5];
	long indx6=dcon[icon+6];
	long indx7=dcon[icon+7];
	double d10=ddat[indx1]-ddat[indx0];
	double d20=ddat[indx2]-ddat[indx0];
	double d40=ddat[indx4]-ddat[indx0];
	double d31=ddat[indx3]-ddat[indx1];
	double d51=ddat[indx5]-ddat[indx1];
	double d32=ddat[indx3]-ddat[indx2];
	double d62=ddat[indx6]-ddat[indx2];
	double d73=ddat[indx7]-ddat[indx3];
	double d54=ddat[indx5]-ddat[indx4];
	double d64=ddat[indx6]-ddat[indx4];
	double d75=ddat[indx7]-ddat[indx5];
	double d76=ddat[indx7]-ddat[indx6];

	const double *dp0=dpos+3*indx0;
	const double *dp1=dpos+3*indx1;
	const double *dp2=dpos+3*indx2;
	const double *dp3=dpos+3*indx3;
	const double *dp4=dpos+3*indx4;
	const double *dp5=dpos+3*indx5;
	const double *dp6=dpos+3*indx6;
	const double *dp7=dpos+3*indx7;

	DEF_DIFFVEC(dp10,dp1,dp0);
	DEF_DIFFVEC(dp20,dp2,dp0);
	DEF_DIFFVEC(dp40,dp4,dp0);
	DEF_DIFFVEC(dp31,dp3,dp1);
	DEF_DIFFVEC(dp51,dp5,dp1);
	DEF_DIFFVEC(dp32,dp3,dp2);
	DEF_DIFFVEC(dp62,dp6,dp2);
	DEF_DIFFVEC(dp73,dp7,dp3);
	DEF_DIFFVEC(dp64,dp6,dp4);
	DEF_DIFFVEC(dp54,dp5,dp4);
	DEF_DIFFVEC(dp75,dp7,dp5);
	DEF_DIFFVEC(dp76,dp7,dp6);
	double vv0[3],vv1[3],vv2[3],det;
	//c0
#define d01  d10
#define d02  d20
#define d04  d40
#define d13  d31
#define d15  d51
#define d45  d54
#define d46  d64
#define d23  d32
#define d26  d62
#define d37  d73
#define d57  d75
#define d67  d76
#define dp01 dp10
#define dp02 dp20
#define dp04 dp40
#define dp13 dp31
#define dp15 dp51
#define dp45 dp54
#define dp46 dp64
#define dp23 dp32
#define dp26 dp62
#define dp37 dp73
#define dp57 dp75
#define dp67 dp76
#define GRADIENT(c0,cx,cy,cz)						\
	CROSS(vv0,dp##cy##c0,dp##cz##c0);				\
	CROSS(vv1,dp##cz##c0,dp##cx##c0);				\
	CROSS(vv2,dp##cx##c0,dp##cy##c0);				\
	det=SCALAR(vv0,dp##cx##c0);					\
	multi[indx##c0]++;						\
	dni[3*indx##c0  ]+=(d##cx##c0*vv0[0]+d##cy##c0*vv1[0]+d##cz##c0*vv2[0])/det; \
	dni[3*indx##c0+1]+=(d##cx##c0*vv0[1]+d##cy##c0*vv1[1]+d##cz##c0*vv2[1])/det; \
	dni[3*indx##c0+2]+=(d##cx##c0*vv0[2]+d##cy##c0*vv1[2]+d##cz##c0*vv2[2])/det;

	GRADIENT(0,1,2,4);
	GRADIENT(1,0,3,5);
	GRADIENT(2,3,0,6);
	GRADIENT(3,2,1,7);
	GRADIENT(4,5,6,0);
	GRADIENT(5,4,7,1);
	GRADIENT(6,7,4,2);
	GRADIENT(7,6,5,3);

#undef d01 
#undef d02 
#undef d04 
#undef d13 
#undef d15 
#undef d45 
#undef d46 
#undef d23 
#undef d26 
#undef d37 
#undef d57 
#undef d67 
#undef dp01
#undef dp02
#undef dp04
#undef dp13
#undef dp15
#undef dp45
#undef dp46
#undef dp23
#undef dp26
#undef dp37
#undef dp57
#undef dp67
#undef GRADIENT
      }else{
	throw Except("precalcGradients: conD not implemented");
      }
    }else{
      throw Except("precalcGradients: connectiontype not implemented");
    }
  }
  if(conD==3){
    dni=vagri.dataRef();
    for(long i=-1;++i<Nsize;){
      dni[0]/=multi[i];
      dni[1]/=multi[i];
      dni[2]/=multi[i];
      dni+=3;
    }
  }else{
    throw Except("precalcGradients: normalization: unkown conD");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
template <class X>
void IsoSurface::calcIsoSurface_internal()
{
  bool prot=false;
  if(prot)printf("=======================================================\n"
		 "%s::calcIsoSurface_internal<%s>()\n",id()(),TypeInfo<X>::name);
  
  if(!pinputfield) return;  
  if(!poutputfield) return;  


  const Positions *poi=pinputfield->positions();
  const Connections *coi=pinputfield->connections();
  const BaseArray<X> *bdi=pinputfield->data().to<X>();
  const BaseArray<long> *baci=coi->array();
  const BaseArray<double> *bapi=poi->array();
  if((!poi) || (!bdi) || (!coi)) return;
  if(bdi->dataType()==TICmplx)
    throw Except("iso surface of a complex function of position not implemented\n");
  if(bdi->datasize()!=1)
    throw Except("isosurface for non scalar functions not implemented");
  
  long embeddingdimension=poi->embeddingDimension();
  long consize=baci->size();
  long conshape=baci->shape(0);
  // dimensionality of mesh, cannot be infered from positions, unless it is
  // a productarray
  long conD=0;
  if(coi->connectionType()==Connections::CTYCuboid){
    conD=dRound(::pow(conshape,0.5));
  }else{
    throw Except("connectiontype not implemented");
  }


  X isovalue=pisovalue.to<X>();

  if(prot){
    myDebug()<<"embeddingdimension="<<embeddingdimension;
    myDebug()<<"connection size="<<consize;
    myDebug()<<"connection shape="<<conshape;
    myDebug()<<"isovalue="<<isovalue;
    myDebug()<<"conD="<<conD;
  }

  
  //todo: productarray of regular meshes and simplex is a special case
  //todo: special code for regular meshes and mesharray

  if(conD>embeddingdimension)
    throw Except("Connection element has higher dimension than embedding space.");



  Positions *poo=poutputfield->positions();
  Connections *coo=poutputfield->connections();
  poutputfield->setupData<double>(ValueArray<long>()<<1);
  BaseArray<double> *badao=poutputfield->data().to<double>();
  ValueArray<double> &vadao=badao->valueArray();


  BaseArray<double> *baclo=0;
  ValueArray<double> *vaclo=0;
  if(pinputfield->colors()){
    poutputfield->setupColors(ValueArray<long>()<<4);
    baclo=poutputfield->colors();
    vaclo=&(baclo->valueArray());
    vaclo->setGrowthFactor(1.5);
  }

  poutputfield->setupNormals(ValueArray<long>()<<embeddingdimension);
  BaseArray<double> *bano=poutputfield->normals();
  poo->setupArray(ValueArray<long>()<<embeddingdimension);
  ValueArray<double> &vapo=poo->array()->valueArray();
  ValueArray<double> &vano=bano->valueArray();
  // conD=4 -> 3d simplex: 4
  // conD=3 -> 2d simplex: 3
  // conD=2 -> 1d simplex: 2
  // conD=1 -> 0d simplex: conD
  coo->setupArray(ValueArray<long>()<<conD);
  ValueArray<long> &vaco=coo->array()->valueArray();

  vapo.setGrowthFactor(1.5);
  vano.setGrowthFactor(1.5);
  vadao.setGrowthFactor(1.5);
  vaco.setGrowthFactor(1.5);
  

  // embeddingdimension==3
  //     conD==3 -> 2d simplex
  //     conD==2 -> 1d simplex
  //     conD==1 -> points
  // embeddingdimension==2
  //     conD==2 -> 1d simplex
  //     conD==1 -> points
  // embeddingdimension==1
  //     conD==1 -> points
  
  // target has same embeddingdimension and conD-1 mesh dimension, which is reflected
  // in the resulting connection
  // result is always an irregular positions mesh and an irregular/explicit
  // simplex connections mesh
  

  
  const X *ddat=bdi->data();
  const long *dcon=baci->data();
  const double *dpos=bapi->data();

  // here we have avoided Cmplx and vector primary data hence X=double,long

  //  PatternCount patterncount(256);

  //We do not have neighbors for now and need unique gradients, so we
  //first calc them for all connections (waste) and then interpolate them.
  //This can be changed, if neighbors are available.

  ValueArray<double> vagri;// used as gradient input if no neighboors, silent otherwise
  bool neighbors=false;//todo
  double *dgri=0;
  if(!neighbors){
    //todo: make version for simplex and for no-neighbor but regular
    precalcGradients(bapi,coi,bdi,vagri);
    dgri=vagri.dataRef();
  }
  const double *dcli=0;
  if(vaclo) dcli=pinputfield->colors()->valueArray().constDataRef();

  double eps=1.0e-13;
  Vec3 gradient;
  long pp[4]={1,3,2,3};
  long pm[4]={0,2,0,1};
  long indexpo=0;
  double lam;
  ValueArray<int> edg(4,0);
  for(long i=-1;++i<consize;){
    //printf("connection %ld\n",i);
    if(coi->connectionType()==Connections::CTYCuboid){

      // for(long c=-1;++c < conshape;){
      // 	long index=dcon[i*conshape+c];
      // 	printf("%ld:[%g %g %g]\n",index,
      // 	       bapi->point(index,0),
      // 	       bapi->point(index,1),
      // 	       bapi->point(index,2));
      // }
      if(conD==2){
	// 2D->1D simplest algo
	long indx[4];
	indx[0]=dcon[i*conshape+0];
	indx[1]=dcon[i*conshape+1];
	indx[2]=dcon[i*conshape+2];
	indx[3]=dcon[i*conshape+3];
	edg[0]=0;edg[1]=1;edg[2]=2;edg[3]=3;
	double d[4];
	d[0]=ddat[indx[0]]-isovalue;
	d[1]=ddat[indx[1]]-isovalue;
	d[2]=ddat[indx[2]]-isovalue;
	d[3]=ddat[indx[3]]-isovalue;
	if(d[0]==0)d[0]=eps;
	if(d[1]==0)d[1]=eps;
	if(d[2]==0)d[2]=eps;
	if(d[3]==0)d[3]=eps;
	myDebug()<<"edge"<<indx[0]<<indx[1]<<":"<<d[0]<<d[1];
	myDebug()<<"edge"<<indx[2]<<indx[3]<<":"<<d[2]<<d[3];
	myDebug()<<"edge"<<indx[0]<<indx[2]<<":"<<d[0]<<d[2];
	myDebug()<<"edge"<<indx[1]<<indx[3]<<":"<<d[1]<<d[3];

	unsigned int pattern=0;
	if(d[0]*d[1]>0) pattern|=1;
	if(d[2]*d[3]>0) pattern|=2;
	if(d[0]*d[2]>0) pattern|=4;
	if(d[1]*d[3]>0) pattern|=8;
	  
	
	//todo: orientation, gradient, extra data, representation
	//todo: algo for regular positions and/or regular connections
	//todo: neighbors, periodic
	//todo: simplex
	//todo: 3d
	//todo: with neighbors, we can store vertices in a once-per-vertex
	//      fashion->needs recreation of neighbors
	//todo: clipping

	if(pattern==0xf) continue;

	printf("pattern=%8s=%x\n",toBin(pattern,true)(),pattern);


	if(pattern==6 || pattern==3){
	  edg.swap(2,3);
	  edg.swap(0,2);
	  edg.swap(1,3);
	  if(pattern==3)pattern=0xc;
	  else pattern=0xa;
	}else if(pattern==5){
	  // 1032
	  edg.swap(0,1);
	  edg.swap(2,3);
	  pattern=0xa;
	}else if(pattern==9){
	  // 2310=0123->3120->2130->2310
	  edg.swap(0,3);
	  edg.swap(0,2);
	  edg.swap(1,2);
	  pattern=0xa;
	}

	printf("new pattern=%8s=%x\n",toBin(pattern,true)(),pattern);

	long in0,in1;
	if(pattern==0xa){

#ifdef MY_DEBUGGING
#define GETLAMBDA(i0,f10)					\
	  lam=-d[i0]/f10;					\
	  if(1 || lam<0 || lam>1) myDebug()<<"ERRORLAM"<<lam;		\
	  lam=MyMath::min<double>(1.,MyMath::max<double>(0.,lam));	
#else
#define GETLAMBDA(i0,f10)						\
	  lam=MyMath::min<double>(1.,MyMath::max<double>(0.,-d[i0]/f10));
#endif

#define POINT(i0,i1)							\
	  {								\
	    double f10=d[i1]-d[i0];					\
	    GETLAMBDA(i0,f10);						\
	    								\
	    in0=indx[i0];						\
	    in1=indx[i1];						\
	    vapo.append(dpos[3*in0+0]+(dpos[3*in1+0]-dpos[3*in0+0])*lam); \
	    vapo.append(dpos[3*in0+1]+(dpos[3*in1+1]-dpos[3*in0+1])*lam); \
	    vapo.append(dpos[3*in0+2]+(dpos[3*in1+2]-dpos[3*in0+2])*lam); \
	  }
	  
	  //todo: gradient needs the x contribution and this is simpler in
	  //todo: Regular lattices
	  //todo: test warped meshes after adding x contribution

	  myDebug()<<"edg="<<edg[0]<<edg[1]<<edg[2]<<edg[3];
	  long i0=pm[edg.at(0)];
	  long i1=pp[edg.at(0)];
	  POINT(i0,i1);
	  vaco.append(indexpo);
	  indexpo++;

	  i0=pm[edg.at(2)];
	  i1=pp[edg.at(2)];
	  POINT(i0,i1);
	  vaco.append(indexpo);
	  indexpo++;



	  continue;
	}else if(pattern==0xc){
	  
	  myDebug()<<"edg="<<edg[0]<<edg[1]<<edg[2]<<edg[3];
	  
	  long i0=pm[edg.at(0)];
	  long i1=pp[edg.at(0)];
	  POINT(i0,i1);
	  vaco.append(indexpo);
	  indexpo++;

	  i0=pm[edg.at(1)];
	  i1=pp[edg.at(1)];
	  POINT(i0,i1);
	  vaco.append(indexpo);
	  indexpo++;


	  continue;
	}else if(!pattern){
	  POINT(0,1);
	  POINT(1,3);
	  POINT(3,2);
	  POINT(2,0);
	  double dm=(d[0]*d[3]-d[1]*d[2]);
	  double nom=d[3]-d[1]-d[2]+d[0];
	  if(nom!=0 && d[0]*dm/nom>0){
	    vaco.append(indexpo);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+3);
	  }else{
	    vaco.append(indexpo);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	  }

	  indexpo+=4;
	  
	  continue;
	}else
	  throw Except("unknow pattern");
	

      }else if(conD==3){
	// 3D->2D simplest algo

	//todo: extra data, representation
	//todo: algo for regular positions and/or regular connections
	//todo: neighbors, periodic
	//todo: simplex
	//todo: 3d
	//todo: with neighbors, we can store vertices in a once-per-vertex
	//      fashion->needs recreation of neighbors
	//todo: clipping
	
	long indx[8];
	int crn[8];
	double d[8];
	unsigned char pattern=0;
	unsigned mask=1;
	for(int c=-1;++c<8;) {
	  indx[c]=dcon[i*conshape+c];
	  crn[c]=c;	
	  d[c]=ddat[indx[c]]-isovalue;
	  if(d[c]==0)d[c]=eps;
	  if(d[c]>0) pattern|=mask;
	  mask=mask<<1;
	}
	
	
	if(pattern==0xff||pattern==0x00) continue;
	
	// for(int c=-1;++c<8;) 
	//   myDebug()<<"crn"<<crn[c]<<"indx="<<indx[crn[c]]<<":"<<d[crn[c]];
	// myDebug();

	int s;
	int mat[3]={1,2,3};
	detPattern3DCub(pattern,s,crn,mat);
	
	// for(int c=-1;++c<8;) 
	//   myDebug()<<"crn"<<crn[c]<<"indx="<<indx[crn[c]]<<":"<<d[crn[c]];
	// myDebug();

	//	patterncount[pattern]++;
	
	long in0,in1;
	if(0){
	  //exclude
	  //1
	  if(pattern==0xfe) continue;
	  //2
	  if(pattern==0xfc) continue;
	  if(pattern==0xf6) continue;
	  if(pattern==0x7e) continue;
	  //3
	  if(pattern==0xf8) continue;
	  if(pattern==0xda) continue;
	  if(pattern==0xe9) continue;
	  //4
	  if(pattern==0xf0) continue;
	  if(pattern==0xe8) continue;
	  if(pattern==0xd8) continue;
	  if(pattern==0xe1) continue;
	  if(pattern==0x96) continue;
	  if(pattern==0xa5) continue;
	}
	


#define DIF_F(c0,c1) double df##c0##c1=d[crn[c1]]-d[crn[c0]];

#define DELTA26 d[crn[2]]*d[crn[4]]-d[crn[0]]*d[crn[6]]
#define DELTA23 d[crn[2]]*d[crn[7]]-d[crn[3]]*d[crn[6]]
#define DELTA04 d[crn[0]]*d[crn[5]]-d[crn[1]]*d[crn[4]]
#define DELTA01 d[crn[0]]*d[crn[3]]-d[crn[1]]*d[crn[2]]
#define DELTA15 d[crn[1]]*d[crn[7]]-d[crn[3]]*d[crn[5]]
#define DELTA46 d[crn[4]]*d[crn[7]]-d[crn[5]]*d[crn[6]]
#define DEF_DELTA(c0,c1) double D##c0##c1=DELTA##c0##c1;
// define all D variables, some get replaced by equivalents
#define D51 D04
#define D02 (-D26)
#define D10 (-D04)
#define D40 (D26)
#define D20 (-D01)
#define D13 (-D01)
#define D31 (-D15)
#define D32 (D01)
#define D62 (-D23)
#define D37 (-D23)
#define D73 (D15)
#define D64 (-D26)
#define D45 (-D04)
#define D54 (-D46)
#define D67 (-D46)
#define D76 (D23)
#define D57 (-D15)
#define D75 (D46)


#define GRAD_INDEX01_0 1
#define GRAD_INDEX01_1 2
#define GRAD_INDEX01_2 0
#define GRAD_SIGN01_0  
#define GRAD_SIGN01_1  
#define GRAD_SIGN01_2  

#define GRAD_INDEX02_0 2
#define GRAD_INDEX02_1 0
#define GRAD_INDEX02_2 1
#define GRAD_SIGN02_0  
#define GRAD_SIGN02_1 
#define GRAD_SIGN02_2  

#define GRAD_INDEX04_0 0
#define GRAD_INDEX04_1 1
#define GRAD_INDEX04_2 2
#define GRAD_SIGN04_0  
#define GRAD_SIGN04_1  
#define GRAD_SIGN04_2  

#define GRAD_INDEX13_0 0 
#define GRAD_INDEX13_1 2
#define GRAD_INDEX13_2 1
#define GRAD_SIGN13_0  -
#define GRAD_SIGN13_1  
#define GRAD_SIGN13_2  

#define GRAD_INDEX15_0 1 
#define GRAD_INDEX15_1 0
#define GRAD_INDEX15_2 2
#define GRAD_SIGN15_0  
#define GRAD_SIGN15_1 - 
#define GRAD_SIGN15_2  

#define GRAD_INDEX23_0 2
#define GRAD_INDEX23_1 1
#define GRAD_INDEX23_2 0
#define GRAD_SIGN23_0  
#define GRAD_SIGN23_1 -
#define GRAD_SIGN23_2  

#define GRAD_INDEX26_0 1 
#define GRAD_INDEX26_1 0
#define GRAD_INDEX26_2 2
#define GRAD_SIGN26_0 - 
#define GRAD_SIGN26_1 
#define GRAD_SIGN26_2  

#define GRAD_INDEX37_0 0 
#define GRAD_INDEX37_1 1
#define GRAD_INDEX37_2 2
#define GRAD_SIGN37_0 - 
#define GRAD_SIGN37_1 -
#define GRAD_SIGN37_2  

#define GRAD_INDEX45_0 2
#define GRAD_INDEX45_1 1
#define GRAD_INDEX45_2 0
#define GRAD_SIGN45_0 -
#define GRAD_SIGN45_1 
#define GRAD_SIGN45_2  

#define GRAD_INDEX46_0 0 
#define GRAD_INDEX46_1 2
#define GRAD_INDEX46_2 1
#define GRAD_SIGN46_0  
#define GRAD_SIGN46_1 -
#define GRAD_SIGN46_2  

#define GRAD_INDEX57_0 2
#define GRAD_INDEX57_1 0
#define GRAD_INDEX57_2 1
#define GRAD_SIGN57_0 - 
#define GRAD_SIGN57_1 -
#define GRAD_SIGN57_2  

#define GRAD_INDEX67_0 1
#define GRAD_INDEX67_1 2
#define GRAD_INDEX67_2 0
#define GRAD_SIGN67_0 - 
#define GRAD_SIGN67_1 -
#define GRAD_SIGN67_2  

#define CHECK_GRAD(a,b,marker)			\
	if(norm(a-b)>1.0e-10){						\
	  myDebug()<<"GRAD ERROR:"<<marker<<" real gradient="<<a<<" gradient="<<b; \
	}								\
	if(fabs(f)>1.0e-10){						\
	  myDebug()<<"GRAD ERROR:"<<marker<<" f=="<<f<<" !=0";		\
	}
#ifdef MY_DEBUGGING
#define TEST_GRADIENT_INTER(c0,c1,tag)					\
	{								\
	  int xx[]={0,1,0,1,0,1,0,1};					\
	  int yy[]={0,0,1,1,0,0,1,1};					\
	  int zz[]={0,0,0,0,1,1,1,1};					\
	  printf("gr  =[%7g %7g %7g]\n",gradient[0],gradient[1],gradient[2]); \
	  double f;							\
	  Vec3 grad=gradient(d,						\
			     xx[crn[c0]]+(xx[crn[c1]]-xx[crn[c0]])*lam,	\
			     yy[crn[c0]]+(yy[crn[c1]]-yy[crn[c0]])*lam,	\
			     zz[crn[c0]]+(zz[crn[c1]]-zz[crn[c0]])*lam,f); \
	  CHECK_GRAD(grad,gradient,tag#c0#c1);				\
	}
#else
#define TEST_GRADIENT_INTER(c0,c1,tag)
#endif
	  
#define TESTPOINT2(c0,c1,s)				\
	gradient[GRAD_INDEX##c0##c1##_0]		\
	  =(s(D##c0##c1/df##c0##c1));			\
	gradient[GRAD_INDEX##c0##c1##_1]		\
	  =(s(-(D##c1##c0/df##c0##c1)));		\
	gradient[GRAD_INDEX##c0##c1##_2]		\
	  =(s(df##c0##c1));				\
	
#define POINT2(c0,c1,tag)						\
	DIF_F(c0,c1);							\
	GETLAMBDA(crn[c0],df##c0##c1);					\
	in0=indx[crn[c0]];						\
	in1=indx[crn[c1]];						\
	vapo.append(dpos[3*in0+0]+(dpos[3*in1+0]-dpos[3*in0+0])*lam);	\
	vapo.append(dpos[3*in0+1]+(dpos[3*in1+1]-dpos[3*in0+1])*lam);	\
	vapo.append(dpos[3*in0+2]+(dpos[3*in1+2]-dpos[3*in0+2])*lam);	\
	vadao.append(isovalue);						\
	if(dgri){	/*no neigbors, no average*/			\
	  gradient[0]=(dgri[3*in0+0]+(dgri[3*in1+0]-dgri[3*in0+0])*lam);	\
	  gradient[1]=(dgri[3*in0+1]+(dgri[3*in1+1]-dgri[3*in0+1])*lam);	\
	  gradient[2]=(dgri[3*in0+2]+(dgri[3*in1+2]-dgri[3*in0+2])*lam);	\
	}else{								\
	  gradient[abs(mat[GRAD_INDEX##c0##c1##_0])-1]			\
	    =lSign(1,mat[GRAD_INDEX##c0##c1##_0])			\
	    *(GRAD_SIGN##c0##c1##_0(-D##c0##c1/df##c0##c1));		\
	  gradient[abs(mat[GRAD_INDEX##c0##c1##_1])-1]			\
	    =lSign(1,mat[GRAD_INDEX##c0##c1##_1])			\
	    *(GRAD_SIGN##c0##c1##_1(D##c1##c0/df##c0##c1));		\
	  gradient[abs(mat[GRAD_INDEX##c0##c1##_2])-1]			\
	    =lSign(1,mat[GRAD_INDEX##c0##c1##_2])			\
	    *(GRAD_SIGN##c0##c1##_2(df##c0##c1));			\
	  TEST_GRADIENT_INTER(c0,c1,tag);				\
	  /*todo: properly averaged absgradient */			\
	  /*todo: we must take pos into account for normals!!!*/	\
	}								\
	/* todo: should we just use lam for gradients, wee need the position part*/ \
	vano.append3(gradient());					\
	if(dcli){							\
	  for(int icol=-1;++icol<4;){					\
	    vaclo->append(dcli[4*in0+icol]				\
			  +(dcli[4*in1+icol]-dcli[4*in0+icol])*lam);	\
	  }								\
	}
	
	

#define DIAGONAL_PATTERN(dm,id0,id1,iod0,iod1,bit)			\
	nom=d[crn[id0]]-d[crn[iod0]]-d[crn[iod1]]+d[crn[id1]];		\
	midpat|=(nom!=0 && d[crn[id0]]*dm/nom>0 ? bit : 0);
	
#define AVERAGE_DATA(va,I0,Ndatasize,Nav,Iavdata)			\
		{							\
		  const double *dva=(va)->constDataRef()+I0;		\
		  for(int c=-1;++c<Ndatasize;){				\
		    double d=0.;					\
		    for(int i=-1;++i<Nav;){				\
		      d+=dva[Ndatasize*Iavdata[i]+c];			\
		    }							\
		    (va)->append(d/Nav);				\
		  }							\
		}

	
#define PRINT_CORNERS							\
	printf("crn: %7d %7d %7d %7d %7d %7d %7d %7d\n",crn[0],crn[1],crn[2],crn[3], \
	       crn[4],crn[5],crn[6],crn[7]);				\
	printf("d:   %7g %7g %7g %7g %7g %7g %7g %7g\n",d[crn[0]],d[crn[1]],	\
	       d[crn[2]],d[crn[3]],					\
	       d[crn[4]],d[crn[5]],d[crn[6]],d[crn[7]]);		\
	printf("\n");




	

	if(pattern==0xfe){

	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
 	  DEF_DELTA(2,6);

	  POINT2(0,1,"fe");
	  POINT2(0,2,"fe");
	  POINT2(0,4,"fe");
	  
	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	  }else{
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	  }
	  indexpo+=3;
	}else if(pattern==0xfc){

	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
 	  DEF_DELTA(2,6);

	  
	  POINT2(0,2,"fc");
	  POINT2(0,4,"fc");
	  POINT2(1,5,"fc");
	  POINT2(1,3,"fc");

	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+0);
	  }else{
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+3);
	  }
	  indexpo+=4;

	}else if(pattern==0xf6){
	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(2,6);
	  POINT2(0,1,"f6");
	  POINT2(0,2,"f6");
	  POINT2(0,4,"f6");
	  POINT2(1,3,"f6");
	  POINT2(2,3,"f6");
	  POINT2(3,7,"f6");

	  double nom;
	  unsigned char midpat=0;
	  DIAGONAL_PATTERN(D01,0,3,1,2,1);
	  
	  if(midpat){
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+5);
	    }else{
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+2);
	    }
	  }else{
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	    }else{
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+4);
	    }
	  }
	    
	  indexpo+=6;
	}else if(pattern==0x7e){
	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(2,6);
	  DEF_DELTA(4,6);
	  POINT2(0,1,"7e");
	  POINT2(0,2,"7e");
	  POINT2(0,4,"7e");
	  POINT2(6,7,"7e");
	  POINT2(5,7,"7e");
	  POINT2(3,7,"7e");

	  if(d[crn[0]]*(d[0]+d[1]+d[2]+d[3]+d[4]+d[5]+d[6]+d[7])>0){
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	    }else{
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+1);
	    }
	  }else{
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	    }else{
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+5);
	    }
	  }
	  indexpo+=6;
	}else if(pattern==0xf8){
	  
	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
 	  DEF_DELTA(2,6);
 	  DEF_DELTA(2,3);
	  
	  POINT2(0,4,"f8");
	  POINT2(1,5,"f8");
	  POINT2(1,3,"f8");
	  POINT2(2,3,"f8");
	  POINT2(2,6,"f8");
	  
	  //todo: use more info to decide on the tria subdevision
	  //todo: maybe use even more to interpolate
	  //todo: if we haveneighbors, can we use them to make biquadratic
	  //todo: interpolation on the fly?
	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+4);
	  }else{
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+4);
	  }
	  indexpo+=5;
	}else if(pattern==0xda){
	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(2,6);
	  DEF_DELTA(4,6);
	  POINT2(0,4,"da");
	  POINT2(0,1,"da");
	  POINT2(2,6,"da");
	  POINT2(2,3,"da");
	  POINT2(4,5,"da");
	  POINT2(1,5,"da");
	  POINT2(5,7,"da");
	  double nom;
	  unsigned char midpat=0;
	  DIAGONAL_PATTERN(D04,0,5,1,4,1);
	  if(midpat){
	    
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+1);
	    }else{
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+1);
	    }

	  }else{
	    if(d[crn[0]]*s<0){
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+5);
	    }else{
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+5);
	    }
	  }
	  indexpo+=7;
	}else if(pattern==0xe9){

	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(2,6);

	  double nom;
	  unsigned char midpat=0;
	  DIAGONAL_PATTERN(D04,0,5,1,4,1);
	  DIAGONAL_PATTERN(D01,0,3,1,2,2);
	  DIAGONAL_PATTERN(-D26,0,6,4,2,4);
	  //myDebug()<<"midpat"<<midpat<<mat[0]<<mat[1]<<mat[2];
	  if(midpat==2){
	    MyMath::swap3(crn[1],crn[2],crn[4]);
	    MyMath::swap3(crn[5],crn[3],crn[6]);
	    MyMath::swap3(mat[0],mat[1],mat[2]);
	    MyMath::swap3(D01,D26,D04);
	    D01*=-1;D26*=-1;
	    midpat=1;
	  }else if(midpat==4){
	    MyMath::swap3(crn[4],crn[2],crn[1]);
	    MyMath::swap3(crn[6],crn[3],crn[5]);
	    MyMath::swap3(mat[0],mat[2],mat[1]);
	    MyMath::swap3(D04,D26,D01);
	    D04*=-1;D26*=-1;
	    midpat=1;
	  }else if(midpat==5){
	    MyMath::swap3(crn[1],crn[2],crn[4]);
	    MyMath::swap3(crn[5],crn[3],crn[6]);
	    MyMath::swap3(mat[0],mat[1],mat[2]);
	    MyMath::swap3(D01,D26,D04);
	    D01*=-1;D26*=-1;
	    midpat=6;
	  }else if(midpat==3){
	    MyMath::swap3(crn[4],crn[2],crn[1]);
	    MyMath::swap3(crn[6],crn[3],crn[5]);
	    MyMath::swap3(mat[0],mat[2],mat[1]);
	    MyMath::swap3(D04,D26,D01);
	    D04*=-1;D26*=-1;
	    midpat=6;
	  }
	  //myDebug()<<"midpat"<<midpat<<mat[0]<<mat[1]<<mat[2];
	  // if(d[crn[0]]*(d[0]+d[1]+d[2]+d[3]+d[4]+d[5]+d[6]+d[7])>0){
	  //   myDebug()<<"hole";
	  // }
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(4,6);
	  int i0=vapo.size();
	  int id0=vadao.size();
	  int ic0; if(dcli) ic0=vaclo->size();
	  POINT2(0,1,"e9");
	  POINT2(0,2,"e9");
	  POINT2(0,4,"e9");
	  POINT2(1,3,"e9");
	  POINT2(2,3,"e9");
	  POINT2(1,5,"e9");
	  POINT2(2,6,"e9");
	  POINT2(4,5,"e9");
	  POINT2(4,6,"e9");

	  
	  //myDebug()<<"midpat="<<midpat;
	  switch (midpat){
	  case 0:
	    {
	      if(d[crn[1]]*s<0){
		vaco.append(indexpo+1);
		vaco.append(indexpo+0);
		vaco.append(indexpo+2);
		vaco.append(indexpo+3);
		vaco.append(indexpo+4);
		vaco.append(indexpo+5);
		vaco.append(indexpo+5);
		vaco.append(indexpo+4);
		vaco.append(indexpo+7);
		vaco.append(indexpo+4);
		vaco.append(indexpo+6);
		vaco.append(indexpo+7);
		vaco.append(indexpo+6);
		vaco.append(indexpo+8);
		vaco.append(indexpo+7);
	      }else{
		vaco.append(indexpo+0);
		vaco.append(indexpo+1);
		vaco.append(indexpo+2);
		vaco.append(indexpo+4);
		vaco.append(indexpo+3);
		vaco.append(indexpo+5);
		vaco.append(indexpo+4);
		vaco.append(indexpo+5);
		vaco.append(indexpo+7);
		vaco.append(indexpo+6);
		vaco.append(indexpo+4);
		vaco.append(indexpo+7);
		vaco.append(indexpo+8);
		vaco.append(indexpo+6);
		vaco.append(indexpo+7);
	      }
	      break;
	    }
	  case 1:
	    {
	      
	      //add 14567 averaged as point 9
	      int iavdata[]={1,4,5,6,7};
	      AVERAGE_DATA(&vadao,id0,1,5,iavdata);
	      AVERAGE_DATA(&vapo,i0,3,5,iavdata);
	      AVERAGE_DATA(&vano,i0,3,5,iavdata);
	      if(dcli){
		AVERAGE_DATA(vaclo,ic0,4,5,iavdata);
	      }

	      
	      if(d[crn[1]]*s<0){
		vaco.append(indexpo+2);
		vaco.append(indexpo+1);
		vaco.append(indexpo+7);
		vaco.append(indexpo+7);
		vaco.append(indexpo+1);
		vaco.append(indexpo+9);
		vaco.append(indexpo+9);
		vaco.append(indexpo+6);
		vaco.append(indexpo+7);
		vaco.append(indexpo+7);
		vaco.append(indexpo+6);
		vaco.append(indexpo+8);
		vaco.append(indexpo+4);
		vaco.append(indexpo+6);
		vaco.append(indexpo+9);
		vaco.append(indexpo+9);
		vaco.append(indexpo+5);
		vaco.append(indexpo+4);
		vaco.append(indexpo+4);
		vaco.append(indexpo+5);
		vaco.append(indexpo+3);
		vaco.append(indexpo+5);
		vaco.append(indexpo+9);
		vaco.append(indexpo+1);
		vaco.append(indexpo+1);
		vaco.append(indexpo+0);
		vaco.append(indexpo+5);
		
	      }else{
		vaco.append(indexpo+1);
		vaco.append(indexpo+2);
		vaco.append(indexpo+7);
		vaco.append(indexpo+1);
		vaco.append(indexpo+7);
		vaco.append(indexpo+9);
		vaco.append(indexpo+6);
		vaco.append(indexpo+9);
		vaco.append(indexpo+7);
		vaco.append(indexpo+6);
		vaco.append(indexpo+7);
		vaco.append(indexpo+8);
		vaco.append(indexpo+6);
		vaco.append(indexpo+4);
		vaco.append(indexpo+9);
		vaco.append(indexpo+5);
		vaco.append(indexpo+9);
		vaco.append(indexpo+4);
		vaco.append(indexpo+5);
		vaco.append(indexpo+4);
		vaco.append(indexpo+3);
		vaco.append(indexpo+9);
		vaco.append(indexpo+5);
		vaco.append(indexpo+1);
		vaco.append(indexpo+0);
		vaco.append(indexpo+1);
		vaco.append(indexpo+5);
		
	      }
	      indexpo++;
	      break;
	    }
	  case 6:
	    {
	      

	      //add 023578 averaged as point 9
	      int iavdata[]={0,2,3,5,7,8};
	      AVERAGE_DATA(&vadao,id0,1,6,iavdata);
	      AVERAGE_DATA(&vapo,i0,3,6,iavdata);
	      AVERAGE_DATA(&vano,i0,3,6,iavdata);
	      if(dcli){
		AVERAGE_DATA(vaclo,ic0,4,6,iavdata);
	      }
	      
	      
	      if(d[crn[1]]*s<0){
		vaco.append(indexpo+1);
		vaco.append(indexpo+4);
		vaco.append(indexpo+6);
		vaco.append(indexpo+2);
		vaco.append(indexpo+8);
		vaco.append(indexpo+9);
		vaco.append(indexpo+9);
		vaco.append(indexpo+8);
		vaco.append(indexpo+7);
		vaco.append(indexpo+7);
		vaco.append(indexpo+5);
		vaco.append(indexpo+9);
		vaco.append(indexpo+9);
		vaco.append(indexpo+5);
		vaco.append(indexpo+3);
		vaco.append(indexpo+3);
		vaco.append(indexpo+0);
		vaco.append(indexpo+9);
		vaco.append(indexpo+9);
		vaco.append(indexpo+0);
		vaco.append(indexpo+2);
	      }else{
		vaco.append(indexpo+4);
		vaco.append(indexpo+1);
		vaco.append(indexpo+6);
		vaco.append(indexpo+8);
		vaco.append(indexpo+2);
		vaco.append(indexpo+9);
		vaco.append(indexpo+8);
		vaco.append(indexpo+9);
		vaco.append(indexpo+7);
		vaco.append(indexpo+5);
		vaco.append(indexpo+7);
		vaco.append(indexpo+9);
		vaco.append(indexpo+5);
		vaco.append(indexpo+9);
		vaco.append(indexpo+3);
		vaco.append(indexpo+0);
		vaco.append(indexpo+3);
		vaco.append(indexpo+9);
		vaco.append(indexpo+0);
		vaco.append(indexpo+9);
		vaco.append(indexpo+2);
	      }
	      indexpo++;
	      break;
	    }
	  case 7:
	    {
	      if(d[crn[1]]*s<0){
		vaco.append(indexpo+0);
		vaco.append(indexpo+5);
		vaco.append(indexpo+3);
		vaco.append(indexpo+1);
		vaco.append(indexpo+4);
		vaco.append(indexpo+6);
		vaco.append(indexpo+2);
		vaco.append(indexpo+8);
		vaco.append(indexpo+7);
	      }else{
		vaco.append(indexpo+5);
		vaco.append(indexpo+0);
		vaco.append(indexpo+3);
		vaco.append(indexpo+4);
		vaco.append(indexpo+1);
		vaco.append(indexpo+6);
		vaco.append(indexpo+8);
		vaco.append(indexpo+2);
		vaco.append(indexpo+7);
	      }
	      break;
	    }
	  default:
	    exit(1);
	  }
	  indexpo+=9;
	}else if(pattern==0xf0){

	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
 	  DEF_DELTA(2,6);

	  POINT2(0,4,"f0");
	  POINT2(1,5,"f0");
	  POINT2(3,7,"f0");
	  POINT2(2,6,"f0");

	  
	  
	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	  }else{
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+2);
	  }
	  indexpo+=4;
	}else if(pattern==0xe8){
	  //todo: can we achieve neighbors for the resulting irregular mesh?


 	  DEF_DELTA(0,1);
 	  DEF_DELTA(0,4);
 	  DEF_DELTA(1,5);
 	  DEF_DELTA(2,3);
 	  DEF_DELTA(2,6);
 	  DEF_DELTA(4,6);

	  POINT2(4,6,"e8");
	  POINT2(2,6,"e8");
	  POINT2(4,5,"e8");
	  POINT2(1,5,"e8");
	  POINT2(2,3,"e8");
	  POINT2(1,3,"e8");



	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+5);
	    vaco.append(indexpo+4);
	  }else{
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+5);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+4);
	  }
	  indexpo+=6;
	}else if(pattern==0xd8){
	  DEF_DELTA(0,1);
	  DEF_DELTA(0,4);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(2,6);
	  DEF_DELTA(4,6);

	  POINT2(0,4,"d8");
	  POINT2(2,6,"d8");
	  POINT2(4,5,"d8");
	  POINT2(5,7,"d8");
	  POINT2(2,3,"d8");
	  POINT2(1,3,"d8");

	  if(d[crn[0]]*s<0){
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+5);
	    vaco.append(indexpo+4);
	  }else{
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+0);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+2);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+1);
	    vaco.append(indexpo+4);
	    vaco.append(indexpo+5);
	    vaco.append(indexpo+3);
	    vaco.append(indexpo+4);
	  }
	  indexpo+=6;
	}else if(pattern==0xe1){
	  DEF_DELTA(0,4);
	  DEF_DELTA(2,6);
	  double nom;
	  unsigned char midpat=0;

	  DIAGONAL_PATTERN(D04,0,5,1,4,1);
	  DIAGONAL_PATTERN(-D26,0,6,4,2,2);
	  
	  //myDebug()<<"midpat="<<midpat;
	  if(midpat==2){
	    MyMath::swap(crn[1],crn[2]);
	    MyMath::swap(crn[5],crn[6]);
	    MyMath::swap(mat[0],mat[1]);
	    MyMath::swap(D04,D26);
	    D04*=-1;D26*=-1;
	    midpat=1;
	    s*=-1;
	  }
	  if(midpat==3){
	    MyMath::swap(crn[0],crn[4]);
	    MyMath::swap(crn[1],crn[5]);
	    MyMath::swap(crn[2],crn[6]);
	    MyMath::swap(crn[3],crn[7]);
	    mat[2]*=-1;
	    D04*=-1;D26*=-1;
	    s*=-1;
	    midpat=0;
	  }
	  DEF_DELTA(0,1);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,3);
	  DEF_DELTA(4,6);

	  int i0=vapo.size();
	  int id0=vadao.size();
	  int ic0; if(dcli) ic0=vaclo->size();
	  POINT2(0,1,"e1");
	  POINT2(0,2,"e1");
	  POINT2(0,4,"e1");
	  POINT2(2,6,"e1");
	  POINT2(3,7,"e1");
	  POINT2(1,5,"e1");
	  POINT2(4,6,"e1");
	  POINT2(4,5,"e1");

	  int iavdata[]={3,5};
	  AVERAGE_DATA(&vadao,id0,1,2,iavdata);
	  AVERAGE_DATA(&vapo,i0,3,2,iavdata);
	  AVERAGE_DATA(&vano,i0,3,2,iavdata);
	  if(dcli){
	    AVERAGE_DATA(vaclo,ic0,4,2,iavdata);
	  }
	      

	  
	  //myDebug()<<"midpat="<<midpat;
	  switch(midpat){
	  case 0:
	    //todo: hole?
	    if(d[crn[1]]*s<0){
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	    }else{
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+4);
	    }
	    break;
	  case 1:
	    if(d[crn[1]]*s<0){
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+7);
	    }else{
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	    }
	    break;
	  default:
	    exit(1);
	  }

	  indexpo+=9;
	}else if(pattern==0x96){
	  printf("pattern=%8s=%x inv=%d\n",toBin(pattern,true)(),pattern,s);	  
	  // double dm,nom;
	  // unsigned char midpat=0;

	  // OLD_DIAGONAL_PATTERN(0,3,1,2,1);
	  // OLD_DIAGONAL_PATTERN(0,6,4,2,2);
	  // OLD_DIAGONAL_PATTERN(0,5,1,4,4);
	  // //myDebug()<<"midpat"<<midpat;

	  // if(midpat==010){
	  //   MyMath::swap3(crn[1],crn[2],crn[4]);
	  //   MyMath::swap3(crn[3],crn[6],crn[5]);
	  // }

	  // indexpo+=0;
	}else if(pattern==0xa5){
	  DEF_DELTA(0,4);
	  DEF_DELTA(2,3);
	  double nom;
	  unsigned char midpat=0;
	  DIAGONAL_PATTERN(D04,0,5,1,4,1);
	  DIAGONAL_PATTERN(D23,2,7,3,6,2);

	  //myDebug()<<"midpat="<<midpat;

	  if((midpat & 0x2)==0x2){
	    MyMath::swap(crn[0],crn[1]);
	    MyMath::swap(crn[2],crn[3]);
	    MyMath::swap(crn[4],crn[5]);
	    MyMath::swap(crn[6],crn[7]);
	    mat[0]*=-1;D04*=-1;D23*=-1;
	    s*=-1;
	    midpat^=0x3;
	  }
	  //myDebug()<<"midpat="<<midpat;
	  
	  DEF_DELTA(0,1);
	  DEF_DELTA(1,5);
	  DEF_DELTA(2,6);
	  DEF_DELTA(4,6);

	  int i0=vapo.size();
	  int id0=vadao.size();
	  int ic0; if(dcli) ic0=vaclo->size();
	  POINT2(0,1,"a5");
	  POINT2(0,4,"a5");
	  POINT2(1,5,"a5");
	  POINT2(4,5,"a5");

	  POINT2(2,3,"a5");
	  POINT2(2,6,"a5");
	  POINT2(3,7,"a5");
	  POINT2(6,7,"a5");



	  if(!midpat){
	     if(d[crn[1]]*s<0){
	       vaco.append(indexpo+0);
	       vaco.append(indexpo+1);
	       vaco.append(indexpo+5);
	       vaco.append(indexpo+5);
	       vaco.append(indexpo+4);
	       vaco.append(indexpo+0);
	       vaco.append(indexpo+2);
	       vaco.append(indexpo+6);
	       vaco.append(indexpo+3);
	       vaco.append(indexpo+3);
	       vaco.append(indexpo+6);
	       vaco.append(indexpo+7);
	     }else{
	       vaco.append(indexpo+1);
	       vaco.append(indexpo+0);
	       vaco.append(indexpo+5);
	       vaco.append(indexpo+4);
	       vaco.append(indexpo+5);
	       vaco.append(indexpo+0);
	       vaco.append(indexpo+6);
	       vaco.append(indexpo+2);
	       vaco.append(indexpo+3);
	       vaco.append(indexpo+6);
	       vaco.append(indexpo+3);
	       vaco.append(indexpo+7);
	     }
	    indexpo+=8;

	  }else{
	    int iavdata[]={0,1,2,3,4,5,6,7};
	    AVERAGE_DATA(&vadao,id0,1,8,iavdata);
	    AVERAGE_DATA(&vapo,i0,3,8,iavdata);
	    AVERAGE_DATA(&vano,i0,3,8,iavdata);
	    if(dcli){
	      AVERAGE_DATA(vaclo,ic0,4,8,iavdata);
	    }
	    
	    
	    if(d[crn[1]]*s<0){

	      vaco.append(indexpo+0);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+8);
	      vaco.append(indexpo+4);
	      vaco.append(indexpo+0);

	      vaco.append(indexpo+2);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+7);

	      vaco.append(indexpo+7);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+8);
	      vaco.append(indexpo+3);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+8);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+4);
	    }else{

	      vaco.append(indexpo+2);
	      vaco.append(indexpo+0);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+4);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+0);

	      vaco.append(indexpo+6);
	      vaco.append(indexpo+2);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+6);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+7);

	      vaco.append(indexpo+3);
	      vaco.append(indexpo+7);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+3);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+5);
	      vaco.append(indexpo+1);
	      vaco.append(indexpo+8);

	      vaco.append(indexpo+5);
	      vaco.append(indexpo+8);
	      vaco.append(indexpo+4);
	    }
	    
	    indexpo+=9;
	  }


	  
	}else{
	  printf("pattern=%8s=%x inv=%d\n",toBin(pattern,true)(),pattern,s);
	  //throw Except("unknow pattern");
	}

	//printf("pattern=%8s=%x inv=%d\n",toBin(pattern,true)(),pattern,s);
	  

	
      }else
	throw Except("conD not implemented");
    }else{
      throw Except("connectiontype not implemented");
    }
    
  }

  // we do not have neighbors and want to have smoother surfaces
  // unify pos and con and average the normals/gradient-info
  //todo: this should be done with a more efficient algo < O(N^2)
  //  unifyData(vapo,vaco,vano);

  poo->array()->pnitems=vapo.size()/3;
  coo->array()->pnitems=vaco.size()/conD;
  bano->pnitems=vano.size()/3;
  badao->pnitems=vadao.size();
  if(baclo){
    baclo->pnitems=vaclo->size()/4;
  }


  // now calc normals from gradients
  if(conD==3){
    long N=vano.size()/3;
    double tol=1.0e-10;
    double *dno=vano.dataRef();
    double *ddao=vadao.dataRef();
    for(long i=-1;++i<N;){
      double d=sqrt(dno[0 ]*dno[0]+dno[1]*dno[1]+dno[2]*dno[2]);
      //   //myDebug()<<"d"<<i<<"="<<d<<Vec3(dni);
      if(fabs(d)>tol){
       	dno[0]/=d;
       	dno[1]/=d;
       	dno[2]/=d;
      }
      if(pusegradientasnewdata) *(ddao++)=d;
      dno+=3;
    }
    
  }else{
    throw Except("IsoSurface: normalization: unkown conD");
  }

  //myDebug()<<"badao=";
  // if(badao) myDebug()<<badao->size();
  //  if(badao) badao->debug();
  // if(bano) myDebug()<<bano->size();
  // if(bano) myDebug()<<poo->array()->size();
  //  myDebug()<<"poo=";
  //  poo->array()->debug();
  //myDebug()<<"coo=";
  //  coo->array()->debug();
  //  myDebug()<<"noo=";
  //  if(bano) bano->debug();
  // myDebug()<<"col="<<(void*)baclo;
  // if(baclo) baclo->debug();
  
  poutputfield->calcStatistics(false);
  
  poo->calculateBoundingBox();

  //myDebug()<<"vapo="<<vapo;
  // myDebug()<<"vagri="<<vagri.size()/3;
  // long N=vagri.size()/3;
  // for(long i=-1;++i<N;){
  //   myDebug()<<i<<Vec3(vagri.constDataRef()+3*i)<<Vec3(dpos+3*i);
  // }
  
  //exit(0);
  //  patterncount.print();
  if(prot) printf(" =======================================================\n");

  pisdirty=true;

#undef GETLAMBDA
#undef POINT
#undef DIF_F
#undef DELTA26
#undef DELTA23
#undef DELTA04
#undef DELTA01
#undef DELTA15
#undef DELTA46
#undef DEF_DELTA
#undef D51
#undef D02
#undef D10
#undef D40
#undef D20
#undef D13
#undef D31
#undef D32
#undef D62
#undef D37
#undef D73
#undef D64
#undef D45
#undef D54
#undef D67
#undef D76
#undef D57
#undef D75

#undef GRAD_INDEX01_0
#undef GRAD_INDEX01_1
#undef GRAD_INDEX01_2
#undef GRAD_SIGN01_0 
#undef GRAD_SIGN01_1 
#undef GRAD_SIGN01_2 

#undef GRAD_INDEX02_0
#undef GRAD_INDEX02_1
#undef GRAD_INDEX02_2
#undef GRAD_SIGN02_0 
#undef GRAD_SIGN02_1 
#undef GRAD_SIGN02_2 

#undef GRAD_INDEX04_0
#undef GRAD_INDEX04_1
#undef GRAD_INDEX04_2
#undef GRAD_SIGN04_0 
#undef GRAD_SIGN04_1 
#undef GRAD_SIGN04_2 

#undef GRAD_INDEX13_0 
#undef GRAD_INDEX13_1
#undef GRAD_INDEX13_2
#undef GRAD_SIGN13_0 
#undef GRAD_SIGN13_1 
#undef GRAD_SIGN13_2 

#undef GRAD_INDEX15_0 
#undef GRAD_INDEX15_1
#undef GRAD_INDEX15_2
#undef GRAD_SIGN15_0 
#undef GRAD_SIGN15_1 
#undef GRAD_SIGN15_2 

#undef GRAD_INDEX23_0
#undef GRAD_INDEX23_1
#undef GRAD_INDEX23_2
#undef GRAD_SIGN23_0 
#undef GRAD_SIGN23_1 
#undef GRAD_SIGN23_2 

#undef GRAD_INDEX26_0 
#undef GRAD_INDEX26_1
#undef GRAD_INDEX26_2
#undef GRAD_SIGN26_0 
#undef GRAD_SIGN26_1 
#undef GRAD_SIGN26_2 

#undef GRAD_INDEX37_0 
#undef GRAD_INDEX37_1
#undef GRAD_INDEX37_2
#undef GRAD_SIGN37_0 
#undef GRAD_SIGN37_1 
#undef GRAD_SIGN37_2 

#undef GRAD_INDEX45_0
#undef GRAD_INDEX45_1
#undef GRAD_INDEX45_2
#undef GRAD_SIGN45_0 
#undef GRAD_SIGN45_1 
#undef GRAD_SIGN45_2 

#undef GRAD_INDEX46_0 
#undef GRAD_INDEX46_1
#undef GRAD_INDEX46_2
#undef GRAD_SIGN46_0 
#undef GRAD_SIGN46_1 
#undef GRAD_SIGN46_2 

#undef GRAD_INDEX57_0
#undef GRAD_INDEX57_1
#undef GRAD_INDEX57_2
#undef GRAD_SIGN57_0 
#undef GRAD_SIGN57_1 
#undef GRAD_SIGN57_2 

#undef GRAD_INDEX67_0
#undef GRAD_INDEX67_1
#undef GRAD_INDEX67_2
#undef GRAD_SIGN67_0  
#undef GRAD_SIGN67_1 
#undef GRAD_SIGN67_2  


#undef CHECK_GRAD
#undef TEST_GRADIENT_INTER
#undef TESTPOINT2
#undef POINT2
#undef DIAGONAL_PATTERN
#undef AVERAGE_DATA
#undef PRINT_CORNERS
}


/*EOF*/ 
