/* @(#)Light.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _LIGHT_H
#define _LIGHT_H 1

#include "MyQRgb.hh"
#include "PtrArray.hh"
#include "Pair.hh"
#include "MyDebug.hh"
#include "Parser.hh"
#include "MACROS.hh"
#include "IDCache.hh"

namespace XFPLO{
  
  
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  class Lights{
  public:

    enum LightTypes{Directional=0,Positional=1,Spot=2};
    
    // ================================================================
    // ================================================================
    class Light{
    public:
      // ------------------------------------------------------------
      Light():ptype(Directional),pname("Light"),pactive(true),
	      pdiffuse(0xff000000),pambient(0xff000000),pspecular(0xff000000),
	      pposition(ValueArray<double>()<<0.<<0.<<1.<<1.),
	      pspotdirection(ValueArray<double>()<<0.<<0.<<-1.),
	      pspotexponent(0.),pspotcutoff(180.),pconstantattenuation(1.0),
	      plinearattenuation(0.),pquadraticattenuation(0.)
      { }
      // ------------------------------------------------------------
      Light(int type):pname("Light"),pactive(true),
	      pdiffuse(0xff000000),pambient(0xff000000),pspecular(0xff000000),
	      pposition(ValueArray<double>()<<0.<<0.<<1.<<1.),
	      pspotdirection(ValueArray<double>()<<0.<<0.<<-1.),
	      pspotexponent(0.),pspotcutoff(180.),pconstantattenuation(1.0),
	      plinearattenuation(0.),pquadraticattenuation(0.)
      { 
	ptype=(LightTypes)(type&0xf);
	switch(type){
	case Directional|0x10:
	default:
	  pname="side_dif_spec";
	  pdiffuse=0xff4c4c4c;
	  pspecular=0xffffffff;
	  pposition=(ValueArray<double>()<<1.<<1.<<0.<<1.);
	  break;
	case Directional|0x20:
	  pname="front_dif";
	  pdiffuse=0xffb2b2b2;
	  pposition=(ValueArray<double>()<<0.<<0.<<1.<<1.);
	  break;
	case Positional|0x10:
	  pname="pos_downright";
	  pdiffuse=0xff6b6b6b;
	  pspecular=0xff505050;
	  pposition=(ValueArray<double>()<<1.25<<-0.80<<1.<<1.);
	  pconstantattenuation=0.35;
	  break;
	case Positional|0x20:
	  pname="pos_center";
	  pdiffuse=0xff6b6b6b;
	  pspecular=0xff505050;
	  pposition=(ValueArray<double>()<<0.0<<0.00<<0.<<1.);
	  pconstantattenuation=0.45;
	  break;
	case Spot|0x10:
	  pname="spot_upleft";
	  pdiffuse=0xff696969;
	  pspecular=0xff8f8f8f;
	  pposition=(ValueArray<double>()<<-0.85<<1.05<<0.55<<1.);
	  pspotdirection=(ValueArray<double>()<<0.20<<-0.25<<-0.15);
	  pspotexponent=3;
	  pspotcutoff=90;
	  pconstantattenuation=0.55;
	  break;
	}
      }

      // ------------------------------------------------------------
      Light(LightTypes ty,const char *name,bool ac,const QRgb &d,const QRgb &a
	    ,const QRgb &s,
	    const ValueArray<double> &p,const ValueArray<double> &dir,
	    double ex,double cut,double atconst,double atlin,double atquad)
	:ptype(ty),pname(name),pactive(ac),pdiffuse(d),pambient(a),pspecular(s),
	 pposition(p),pspotdirection(dir),
	 pspotexponent(ex),pspotcutoff(cut),pconstantattenuation(atconst),
	 plinearattenuation(atlin),pquadraticattenuation(atquad)
      {}

      // ------------------------------------------------------------
      ~Light(){}

      // ------------------------------------------------------------
      Light *clone()const{return new Light(*this);}


      // ------------------------------------------------------------
      Vec3 realPosition()const;

      // ------------------------------------------------------------
      static String PTHeader();
      void PTSectionToFile(FILE *FH,bool cont)const;
      void readPTSectionFromFile(const Dobj &root);

    private:
      PROPERTY(type,type,Type,LightTypes);
      CPROPERTY(name,name,Name,String);
      PROPERTY(active,active,Active,bool);
      CPROPERTY(diffuse,diffuse,Diffuse,QRgb);
      CPROPERTY(ambient,ambient,Ambient,QRgb);
      CPROPERTY(specular,specular,Specular,QRgb);
      CMODPROPERTY(position,position,Position,ValueArray<double>);
      CMODPROPERTY(spotdirection,spotDirection,SpotDirection,ValueArray<double>);
      PROPERTY(spotexponent,spotExponent,SpotExponent,double);
      PROPERTY(spotcutoff,spotCutoff,SpotCutoff,double);
      PROPERTY(constantattenuation,constantAttenuation,ConstantAttenuation,double);
      PROPERTY(linearattenuation,linearAttenuation,LinearAttenuation,double);
      PROPERTY(quadraticattenuation,quadraticAttenuation,QuadraticAttenuation,double);
    };
    // ================================================================
    // ================================================================
    
    // ------------------------------------------------------------
    Lights(){ setDefault();}
    
    // ------------------------------------------------------------
    ~Lights(){}

    // ------------------------------------------------------------
    Lights *clone()const{return new Lights(*this);}
    
    // ------------------------------------------------------------
    inline long size()const{return plights.size();}
    inline  Light *operator[](long i){return plights[i];}
    inline const Light *at(long i)const{return plights.at(i);}
    inline const PtrArray<Light> &lights()const{return plights;}


    // ------------------------------------------------------------
    void clear(){plights.clear();}

    // ------------------------------------------------------------
    void insertLight(long i,Light *li);
    Light *removeLightAt(long pos);
    
    // ------------------------------------------------------------
    QRgb ambientLight()const{
      return scaleRgb(pglobalambient,pglobalambientfactor)|0xff000000;
    }
    
    // ------------------------------------------------------------
    void PTSectionToFile(FILE *FH)const;
    void readPTSectionFromFile(const Dobj &root);

    // ------------------------------------------------------------
    void setDefault(){
      clear();
      plights<<new Light(Directional|0x10)<<new Light(Directional|0x20);
      pglobalambientfactor=0.25;
      pglobalambient=0xffffffff;
      pbackgroundcolor=0xffffffff;
    }

  public:
    static ValueArray<Pair<LightTypes,const char *> > lighttypes;

  private:
    PtrArray<Light> plights;
    
    CPROPERTY(globalambient,globalAmbient,GlobalAmbient,QRgb);
    PROPERTY(globalambientfactor,globalAmbientFactor,GlobalAmbientFactor,double);
    CPROPERTY(backgroundcolor,backgroundColor,BackgroundColor,QRgb);
    PROPERTY(changed,changed,Changed,bool);
  };

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  class LightThemes{
  public:

    // ------------------------------------------------------------
    LightThemes():pcurrenttheme(0){
      loadSettings();
      if(pthemes.size()<1){
	pthemes<<makePair(String("default"),Lights());
      }
    }

    // ------------------------------------------------------------
    ~LightThemes(){}


    // ------------------------------------------------------------
    const Pair<String,Lights> &at(long i){return pthemes.at(i);}

    // ------------------------------------------------------------
    Pair<String,Lights> &operator[](long i){return pthemes[i];}

    // ------------------------------------------------------------
    const Lights &currentLights()const{return pthemes.at(pcurrenttheme).v2;}
    Lights &currentLights(){return pthemes[pcurrenttheme].v2;}

    // ------------------------------------------------------------
    void saveSettings();

    // ------------------------------------------------------------
    void insertNewAt(int i,const Pair<String,Lights> &theme)
    {
      if(i>=pthemes.size()){
	pthemes.append(theme);
      }else{
	pthemes.insert(i,theme);
      }
    }

    // ------------------------------------------------------------
    void removeAt(int i){
      if(i>=0 && i<pthemes.size())
	pthemes.removeAt(i);
    };

    // ------------------------------------------------------------
    const ValueArray<Pair<String,Lights> > &themesArray()const{return pthemes;}

    // ------------------------------------------------------------
    StringList themes()const
    {
      StringList sl;
      for(int i=-1;++i<pthemes.size();) sl<<pthemes.at(i).v1;
      return sl;
    }
  private:
    void loadSettings();
  private:
    ValueArray<Pair<String,Lights> > pthemes;
    PROPERTY(currenttheme,currentTheme,CurrentTheme,long);
  };
  
  
}

using namespace XFPLO;

inline MyDebug operator<<(MyDebug dbg, const Lights::Light &lt)
{
  dbg << "position"<<lt.position()<<"\n";
  return dbg;
}
inline MyDebug operator<<(MyDebug dbg, const Lights &lt)
{
  dbg << "lights\n"<<lt.lights();
  return dbg;
}
inline MyDebug operator<<(MyDebug dbg, const LightThemes &lt)
{
  dbg << "currenttheme="<<lt.currentTheme()<<"\n";
  dbg << "themes="<<lt.themes()<<"\n";
  dbg << "themes="<<lt.themesArray();
  return dbg;
}

#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _LIGHT_H */

