/*************************************************************************
 * @(#)MeshData.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 May 2010
 *************************************************************************
 */

#include "MeshData.hh"
#include "MyMath.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshData::MeshData()
  : parray(0),pgradients(0),psize(0)
{
}

MeshData::MeshData(long N)
  : parray(0),pgradients(0),psize(0)
{
  resizeData(N);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshData::~MeshData()
{
  if(parray) delete [] parray; parray=0;
  if(pgradients) delete [] pgradients; pgradients=0;

  foreach(double *d,pextradata){
    //myDebug() << "delete"<< d;
    if(d) delete [] d;
  }
  pextradata.clear();
  psize=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshData::resizeData(long N)
{
  if(parray) delete [] parray;

  psize=N;
  parray=new double[psize];

  if(pgradients) delete [] pgradients;
  pgradients=new double[3*psize];

  dinit(pgradients,0.,psize*3);

  foreach(double *d,pextradata){
    if(d){
      delete [] d;
    }
    d=new double[psize];
    dinit(d,0.,psize);
  }
  
}



;


/* ***********************************************************************
 * add extra data arrays such that the total number is at least: min
 * ***********************************************************************
 */
void MeshData::addExtraDataArray(long min)
{
  for(long i=MyMath::max<long>(0,pextradata.size()-1);i<min;i++){
    pextradata.append(new double[psize]);
    dinit(pextradata.last(),0.,psize);
    //myDebug() << "added data array"<<i<<pextradata.last();
  }
}

/*EOF*/ 
