/*************************************************************************
 * @(#)MeshSymmetry.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   16 Aug 2010
 *************************************************************************
 */

#include "Parser.hh"
#include "Except.hh"
#include "MeshSymmetry.hh"
#include "fsymmetry.h"
#include "Symmetry.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshSymmetry::MeshSymmetry()
  :phandmade(false),phex(false),prtg(false),pnumOps(1)
  ,plengthunit(Xf::BohrRadius),puserdefinedstuffchanged(false)
{
  puserbravais=SYM_SIMPLE;
  phexagonal=false;
  prhombohedral=false;
  pconvcell=Matrix3x3::unitMatrix();

  pcrcell=Matrix3x3::unitMatrix();
  prcell=Matrix3x3::unitMatrix();
  pcell=Matrix3x3::unitMatrix();
  pccell=Matrix3x3::unitMatrix();

  piTable[0]=0;
  pnumOps=1;
  plengthunit=Xf::BohrRadius;
  pusersymbols[0]="E";
  for(int it=0;it<1;it++){
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
	puseropmat[9*it+3*j+i]=0.;
      }
      puseropmat[9*it+3*i+i]=1.;
    }
  }

    
  pusernumops=1;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshSymmetry::~MeshSymmetry()
{
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshSymmetry::setUpFromSymmetry(const Symmetry *sym)
{
  if(phandmade) {
    pbravais=puserbravais;
    
    // conventional cell C
    pccell=pconvcell;
    Matrix3x3 B(bravais_centering(pbravais));
    B.transpose();
    B/=12.;
    myDebug() << "B"<<B.printS()();
    // bravais cell A=B*C
    pcell=pccell*B;
    // conventional reciprocial cell
    prcell=pcell.inverseT();
    /* convert  rec cell to rec conventional cell */

    pcrcell=Symmetry::convertToSimple(prcell,pbravais,Xf::LTReciprocial);
      
    myDebug() << "BA"<<pccell.printS()();
    myDebug() << "A "<<pcell.printS()();
    myDebug() << "G "<<prcell.printS()();
    myDebug() << "GC "<<pcrcell.printS()();
      
    phex=phexagonal;
    prtg=prhombohedral;
    for(int it=0;it<64;it++){
      piTable[it]=it;
    }
//     pnumOps=pusernumops;

//     for(int it=0;it<pnumOps;it++){
//       for(int i=0;i<3;i++){
// 	for(int j=0;j<3;j++){
// 	  popmat[9*it+3*j+i]=puseropmat[9*it+3*j+i];
// 	}
//       }
//     }

    if(pusernumops){
      double tol=1.0e-4;
      pnumOps=0;
      // add operations and unify set
      for(int jt=0;jt<pusernumops;jt++){
	bool haveit=false;
	Matrix3x3 mu(puseropmat+9*jt);
	//myDebug() << mu.printQ(QString("mjt=%1").arg(jt).toAscii().constData());
	
	for(int it=0;it<pnumOps;it++){
	  Matrix3x3 m(popmat+9*it);
	  if((m-mu).isZero(tol)){
	    //myDebug() << m.printQ(QString("haveit mit=%1").arg(it).toAscii().constData());

	    haveit=true;
	    break;
	  }
	}
	
	if(!haveit){
	  pnumOps++;
	  int it=pnumOps-1;
	  //myDebug() <<"current pnumOps="<<pnumOps<<"it="<<it;
	  memcpy(popmat+9*it,puseropmat+9*jt,sizeof(double)*9);
	}
      }
      // now try all products
      int itmax=-1;
      while(itmax!=pnumOps) {
	itmax=pnumOps;
	for(int it=0;it<itmax;it++){
	  Matrix3x3 mi(popmat+9*it);
	  for(int jt=0;jt<itmax;jt++){
	    Matrix3x3 mj(popmat+9*jt);
	    Matrix3x3 mij=mi*mj;
	    
	    bool haveit=false;
	    
	    for(int kt=0;kt<pnumOps;kt++){
	      Matrix3x3 m(popmat+9*kt);
	      if((m-mij).isZero(tol)){
		//myDebug() << m.printQ(QString("haveit mit=%1").arg(kt).toAscii().constData());
		
		haveit=true;
		break;
// 	      }else{
// 		myDebug() << m.printQ(QString("checking mit=%1").arg(kt).toAscii().constData());
	      }
	    }
	    if(!haveit){
	      pnumOps++;
	      int kt=pnumOps-1;
	      //myDebug() <<"current pnumOps="<<pnumOps<<"kt="<<kt;
	      //myDebug() << mij.printQ(QString("adding mit=%1").arg(kt).toAscii().constData());
	      memcpy(popmat+9*kt,mij(),sizeof(double)*9);
	    }
	    
	    
	  }
	}
	//myDebug() <<"itmax="<<itmax<<"pnumOps="<<pnumOps;
      }
      
      
    }else{
      int it=0;
      pnumOps=1;
      for(int i=0;i<3;i++){
	for(int j=0;j<3;j++){
	  popmat[9*it+3*j+i]=0.;
	}
	popmat[9*it+3*i+i]=1.;
      }
    }
    
//     myDebug() <<"pnumOps="<<pnumOps;
//     for(int it=0;it<pnumOps;it++){
//       Matrix3x3 m(popmat+9*it);
//       myDebug() << m.printQ(QString("op%1").arg(it).toAscii().constData());
//     }
    


    plengthunit=sym->lengthUnit();  

    return;
  }else{
    pbravais=sym->bravais();
    prtg=(sym->bravais() == SYM_RHOMBOHEDRAL &&
	  (sym->crystal() == SYM_TRIGONAL ));
    
    phex=(sym->crystal() == SYM_HEXAGONAL || 
	  sym->crystal() == SYM_TRIGONAL );
    
    pcell=sym->cell();
    prcell=sym->rcell();
    pcrcell=sym->crcell();
    pccell=sym->ccell();
    
    pnumOps=sym->numOps();
    memcpy(popmat,sym->opmat(),sizeof(double)*9*64);
    for(int it=0;it<64;it++){
      piTable[it]=sym->iTable(it);
    }
    plengthunit=sym->lengthUnit();  
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshSymmetry::meshSymmetryPTSectionToFile(FILE *FH)
{
  fprintf(FH,
	  "  struct { \n"
	  "          logical handmade;\n"
	  "          int bravaistype;\n"
	  "          logical hexagonal;\n"
	  "          logical rhombohedral;\n"
	  "          struct {real V1[3];real V2[3];real V3[3];}convcell;\n"
	  "          struct{char[*] symbol;real opmat[9];}ops[*];\n"
	  "        } meshsymmetry\n"
	  "       ={\n"
	  "         %c,%ld,%c,%c,\n"
	  "         {{%g,%g,%g},{%g,%g,%g},{%g,%g,%g}},\n"
	  ,(phandmade?'t':'f')
	  ,puserbravais
	  ,(phexagonal?'t':'f')
	  ,(prhombohedral?'t':'f')
	  ,pconvcell(0)[0],pconvcell(0)[1],pconvcell(0)[2]
	  ,pconvcell(1)[0],pconvcell(1)[1],pconvcell(1)[2]
	  ,pconvcell(2)[0],pconvcell(2)[1],pconvcell(2)[2]
	  );
  fprintf(FH,"        {");
  for(int ig=0;ig<pusernumops;ig++){
    fprintf(FH,"          {\"%s\",{%g,%g,%g,%g,%g,%g,%g,%g,%g}}"
	    ,pusersymbols[ig].toAscii().constData()
	    ,puseropmat[9*ig+0],puseropmat[9*ig+1],puseropmat[9*ig+2]
	    ,puseropmat[9*ig+3],puseropmat[9*ig+4],puseropmat[9*ig+5]
	    ,puseropmat[9*ig+6],puseropmat[9*ig+7],puseropmat[9*ig+8]
	    );
    if(ig<pusernumops-1)
      fprintf(FH,",");
    fprintf(FH,"\n");
    
  }
  fprintf(FH,"        }\n");

  fprintf(FH
	  ,"       };\n\n");
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshSymmetry::readMeshSymmetryPTSectionFromFile(Parser *par
						     ,MeshSymmetry &tmp)
{
  try{
    bool t_type=(*par)()("meshsymmetry.handmade").valueL();
    tmp.phandmade=t_type;
  }catch(Except e){};

  try{
    bool t_type=(*par)()("meshsymmetry.hexagonal").valueL();
    tmp.phexagonal=t_type;
  }catch(Except e){};

  try{
    bool t_type=(*par)()("meshsymmetry.rhombohedral").valueL();
    tmp.prhombohedral=t_type;
  }catch(Except e){};

  try{
    long t_br=(*par)()("meshsymmetry.bravaistype").valueL();
    tmp.puserbravais=t_br;
  }catch(Except e){};

  String t_box[9]={"1","0","0","0","1","0","0","0","1"};
  try{
    (*par)()("meshsymmetry.convcell.V1").getArray(3,t_box+0);
    (*par)()("meshsymmetry.convcell.V2").getArray(3,t_box+3);
    (*par)()("meshsymmetry.convcell.V3").getArray(3,t_box+6);
  }catch(Except e){};
  tmp.pconvcell.setColumn(0,Vec3(t_box+0));
  tmp.pconvcell.setColumn(1,Vec3(t_box+3));
  tmp.pconvcell.setColumn(2,Vec3(t_box+6));

  try{
    Dobj d=(*par)()("meshsymmetry.ops");
    int nops=d.size();
    for(int i=0;i<nops;i++){
      d[i]("opmat").getArray(9,tmp.puseropmat+9*i);
      tmp.pusersymbols[i]=d[i]("symbol")()();
    }
    tmp.pusernumops=nops;
  }catch(Except e){
    tmp.pusernumops=1;
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
	tmp.puseropmat[3*j+i]=0.;
      }
      tmp.puseropmat[3*i+i]=1.;
    }
    tmp.pusersymbols[0]="E";
  };

  tmp.puserdefinedstuffchanged=true;
}
    
/*EOF*/ 
    
