/* @(#)MeshSymmetry.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _MESHSYMMETRY_H
#define _MESHSYMMETRY_H 1


#include <stdio.h>
#include "Xf.hh"
#include "Vec3.hh"

namespace XFPLO{
  class Symmetry;
}
class Parser;

using namespace XFPLO;

class MeshSymmetry{
  // for unfolding we need hand made mesh symmetry
  // for normal mode symmetry from FPLO input
  // MeshSymmetry mediates
 public:
  MeshSymmetry();
  ~MeshSymmetry();
  
  void setUpFromSymmetry(const Symmetry *sym);

  long bravais()const{return pbravais;};
  bool isHexagonal()const{return phex;};
  bool isRTG()const{return prtg;};

  Xf::LengthUnits lengthUnit() const{return plengthunit;};

  const Matrix3x3 &convRecCell()const{return pcrcell;};
  const Matrix3x3 &recCell()const{return prcell;};
  const Matrix3x3 &cell()const{return pcell;};
  const Matrix3x3 &ccell()const{return pccell;};


  const long &numOps()const{return pnumOps;};
  const long &iTable(long ig)const{return piTable[ig];};
  const double *opmat()const{return popmat;};

  bool userDefinedSymmetry()const{return phandmade;};
  void setUserDefinedSymmetry(bool b){phandmade=b;};

  // user defined data 
  long userBravais()const{return puserbravais;};
  bool hexagonal()const{return phexagonal;};
  bool rhombohedral()const{return prhombohedral;};
  const Matrix3x3 &convCell()const{return pconvcell;};
  //const Matrix3x3 &centeringMatrix()const{return pB;};
  void setConvCell(const Matrix3x3 &m){pconvcell=m;};
  //void setCenteringMatrix(const Matrix3x3 &m){pB=m;};

  long userNumOps()const {return pusernumops;};
  void setUserNumOps(long nops) {pusernumops=nops;};

  const double *userOpmat(long ig)const{return puseropmat+9*ig;};
  void setUserOpmat(long ig,double *op){
    memcpy(puseropmat+ig*9,op,sizeof(double)*9);
  };

  const QString &userOpSymbol(long ig)const{return pusersymbols[ig];};
  void setUserOpSymbol(long ig,const QString &s){pusersymbols[ig]=s;};
  
  
  void setUserBravais(long b){puserbravais=b;};
  void setHexagonal(bool b){phexagonal=b;};
  void setRTG(bool b){prhombohedral=b;};

  void setUserDefinedStuffChanged(bool c){puserdefinedstuffchanged=c;};
  bool userDefinedStuffChanged()const {return puserdefinedstuffchanged;};

  void meshSymmetryPTSectionToFile(FILE *FH);
  static void readMeshSymmetryPTSectionFromFile(Parser *par,MeshSymmetry &tmp);

 private:
  bool phandmade;
  // current data
  bool phex;
  bool prtg;
  Matrix3x3 pcrcell;
  Matrix3x3 prcell;
  Matrix3x3 pcell;
  Matrix3x3 pccell;
  long pnumOps;
  long piTable[64];
  double popmat[9*64];
  Xf::LengthUnits   plengthunit;
  long pbravais;
  // user defined data
  long puserbravais;
  bool phexagonal;
  bool prhombohedral;
  Matrix3x3 pconvcell;
  long pusernumops;
  double puseropmat[9*64];
  QString pusersymbols[64];
  bool puserdefinedstuffchanged;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _MESHSYMMETRY_H */

