/*************************************************************************
 * @(#)Module.cpp
 * author: magru@unas
 * date:   11 Jun 2013
 *************************************************************************
 */

#include "Module.hh"
#include "ValueArray.hh"
#include "MyDebug.hh"

using namespace XFPLO;

IDCache Module::pidchache;

bool Module::debugmode=false;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Module::Module(const char *classname)
  :pclassname(classname),pisdirty(false),
   pid(pidchache.makeUniqueName(Sprintf("%s_",classname)))
{
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Module::~Module()
{
  //printf("%s dying ...\n",pclassname());
  for(int i=-1;++i<pinputs.size();){
    //printf("remove from input of %s\n",pinputs.at(i)->pclassname());
    pinputs[i].module->poutputs.remove(Data(this));
  }
  for(int i=-1;++i<poutputs.size();){
    //printf("remove %s from output of %s\n",pclassname(),poutputs[i].module->pclassname());
    poutputs[i].module->removeInput(this);
    poutputs[i].module->pinputs.remove(Data(this));
  }
  pidchache.remove(pid);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Module::connect(Module *input,Module *output)
{
  if(!input || ! output ) return;
  input->poutputs.append(output);
  output->pinputs.append(input);
  output->setInput(input);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Module::debug(const String &direction,const String &prefix)const
{
  printf("%sModule[%s]\n",prefix(),pid());
  if(direction=="in" || direction=="both"){
    if(pinputs.size()>0) printf("%sInputs:\n",prefix());
    for(int i=-1;++i<pinputs.size();){
      pinputs.at(i).module->debug("in",prefix+"  ");
    }
  }
  if(direction=="out" || direction=="both"){
    if(poutputs.size()>0) printf("%sOutputs:\n",prefix());
    for(int i=-1;++i<poutputs.size();){
      poutputs.at(i).module->debug("out",prefix+"  ");
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Module::cascadingUpdate(const String &prefix)
{
  NSF("Module::cascadingUpdate()");
  //myDebug()<<nsubfunc<<pclassname;
  bool changed=false;
  for(int i=-1;++i<pinputs.size();){
    changed|=pinputs[i].module->cascadingUpdate(prefix);
  }
  //if(!pisdirty) return;
  //myDebug()<<pclassname<<"does its own work";

  return changed;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Module::sendDirt(const String &whatchanged)
{
  //myDebug() <<"Module::sendDirt"<<pclassname<<whatchanged;
  String w=pclassname+" "+whatchanged;
  for(int i=-1;++i<poutputs.size();){
    Module *m=poutputs[i].module;
    ValueArray<Data> &minp=m->pinputs;
    for(int j=-1;++j<minp.size();){
      Data &d=minp[j];
      if(d.module==this){
	// myDebug()<<"found myself"<<d.module->pclassname
	// 	 << "in"<<m->pclassname;
	if(!d.changedinput.contains(w)){
	  d.changedinput.append(w);
	}
      }
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Module::dirtyInput(const String &whatsdirty)const
{
  //myDebug()<<"dirtyInput"<<whatsdirty;
  for(int i=-1;++i<pinputs.size();){
    const StringList &ci=pinputs.at(i).changedinput;
    //myDebug()<<i<<pinputs.at(i).module->pclassname<<ci;
    if(whatsdirty.empty()){
      if(ci.size()>0) return true;
    }else{
      for(int j=ci.size();--j>-1;){
	if(ci.at(j).startsWith(whatsdirty)) return true;
      }
    }
  }
  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Module::resetDirt(const String &what)
{
  bool all=what.empty();
  //myDebug()<<"resetDirt"<<what<<all;
  for(int i=-1;++i<pinputs.size();){
    //myDebug()<<"checking"<<pinputs[i].module->pclassname;
    StringList &ci=pinputs[i].changedinput;
    //myDebug()<<"ci"<<ci;
    if(all) 
      ci.clear();
    else{
      for(int j=ci.size();--j>-1;){
	//myDebug()<<"dirt: "<<j<<ci.at(j);
	if(ci.at(j).startsWith(what)){
	  //myDebug()<<"remove"<<ci.at(j);
	  ci.remove(j);
	}
      }
    }
  } 
  if(all) pisdirty=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Module::showDirt(const String &prefix,const String &context)
{
  if(!debugmode) return;
  printf("%s%s\n",prefix(),context());
  if(pisdirty) {
    printf("%s>>I am dirty\n",prefix());
  }
  for(int i=-1;++i<pinputs.size();){
    const Data &d=pinputs.at(i);
    if(d.changedinput.size()>0){
      printf("%s>>Dirt for %s: %s\n",prefix(),d.module->pid(),
	     d.changedinput.join(" , ")());
    }
  }

}

/*EOF*/ 
