/* @(#)Plane.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _PLANE_H
#define _PLANE_H 1

#include "Vec3.hh"

class Plane{
public:
  Plane():pP(1.,0.,0.),pD(1.) {
    pn=pP;pd=pD;
  }

  Plane(const Vec3 &v,double dd,bool isnormalized=false)
  :pP(v),pD(dd) {
    if(isnormalized){
      pn=pP;pd=pD;
    }else{
      double f=norm(v);pn=v/f;pd=dd*f;
    }
  };
  inline void setDistance(double d){ pd=d; pD=pd/norm(pP); };

  inline const Vec3 &P() const{return pP;};
  inline const Vec3 &n() const{return pn;};
  inline Vec3 &n(){return pn;};
  inline double D()const{return pD;};
  inline double d()const{return pd;};

  static bool govlap(Vec3 &p,const Plane &g1,const Plane &g2,const Plane &g3);

private:
  Vec3 pP;			/* plane equation P*r=D*|P|^2 or
				   normalized n*r=d
				   with n=P/|P| d=D*|P| */
  Vec3 pn;
  double pD;
  double pd;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _PLANE_H */

