/* @(#)PointerT.hh
 * author: magru@unas
 */

#ifndef _POINTERT_HH
#define _POINTERT_HH 1


#include "Types.hh"

namespace XFPLO{



/**
 * It holds a pointer to X<T>, where T is a type defined in Types.hh.
 * One can extract the explicit type via PointerT.to<T>().
 * 
 */

template <template<class> class X>
class PointerT{
private:  
  PointerT(const PointerT&);
  PointerT &operator=(const PointerT&);
public:
  PointerT():pdata(0){}
  ~PointerT(){clean();}

  void clean(){
    if(!pdata) return;
    switch(ptype){
    case TIlong:
      delete static_cast<X<long>*>(pdata);
      break;
    case TIdouble:
      delete static_cast<X<double>*>(pdata);
      break;
    case TICmplx:
      delete static_cast<X<Cmplx>*>(pdata);
      break;
    case TIunkown:
      break;
    }
    pdata=0;
  }  


  /** 
   * determine the data type, such that one can make a decision on the
   * use of to<X>()
   * @see to<X>()
   * @return the data type of the pointer held inside
   */
  DataTypes dataType()const{return ptype;}

  
  template <class T>
  void create(){
    clean();
    ptype=TypeInfo<T>::type;
    pdata=new X<T>();
  }
  
  template <class T>
  X<T> *to(){
    if(ptype!=TypeInfo<T>::type)
      throw Except(Sprintf("cannot cast PointerT to %s",TypeInfo<T>::name));
    return static_cast<X<T>*>(pdata);
    
  }
  
  template <class T>
  const X<T> *to()const{
    if(ptype!=TypeInfo<T>::type)
      throw Except(Sprintf("cannot cast PointerT to %s",TypeInfo<T>::name));
    return static_cast<const X<T>*>(pdata);
    
  }
  
private:
  DataTypes ptype;
  void *pdata;

};
}



#endif /* _POINTERT_HH */

