
//todo: unas:FPLO/FPLO9.02-36/XFSF4/CASES/Clatrate_BaGeGa =.xstr5 cu=00 0.2
//todo:play with bonds it is wrong

//todo: what happens if the center is coplanar with the orientation
/*************************************************************************
 * @(#)Polyhedron.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Mar 2011
 *************************************************************************
 */

#include "Parser.hh"
#include "Except.hh"
#include "constants.h"
#include "MyMath.hh"
#include "Polyhedron.hh"
#include "Plane.hh"
#include "Symmetry.hh"
#include "Vlevel.hh"
#include "MyDebug.hh"


using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PolyhedronProperties::PTHeaderToFile(FILE *FH,const char *prefix
					  ,const char *tail)
{
  fprintf(FH,
 	  "%1$sstruct{logical visible;logical drawfaces;char[*] facecolor;\n"
	  "%1$s       char [*] facespecular;real faceshininess;\n"
	  "%1$s       logical drawrim;char[*] rimcolor;real rimradius;\n"
	  "%1$s       struct  {int type;char[*] description;} rimradiusunit;\n"
	  "%1$s}PolyhedronProperties%2$s",prefix,tail);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PolyhedronProperties::PTBodyToFile(FILE *FH,const char *prefix
					 ,const char *tail)const
{
  
  fprintf(FH,"%s{%c,%c,%s,%s,%g,%c,%s,%s,{%d,\"%s\"}}%s"
	  ,prefix
	  ,(pvisible?'t':'f')
	  ,(pdrawfaces?'t':'f')
	  ,Sprintf("\"%x\"",pfacematerial.color())()
	  ,Sprintf("\"%x\"",pfacematerial.specularColor())()
	  ,pfacematerial.shininess()
	  ,(pdrawrim?'t':'f')
	  ,Sprintf("\"%x\"",primmaterial.color())()	  
	  ,primradius.S()
	  ,primradiusunits
	  ,Xf::sceeneUnitName(primradiusunits)
	  ,tail);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PolyhedronProperties::readPTSectionFromFile(const Dobj &root)
{
  NSF("PolyhedronProperties::readPTSectionFromFile");

  try{
    const Dobj &oroot=root("DCTProperties");
    // try to read old data
    try{
      // old format
      pvisible=oroot("visible").valueL();
    }catch(Except &e){};
    try{
      pdrawrim=oroot("drawrim").valueL();
    }catch(Except &e){};
    try{
      Dobj c=oroot("rimcolor");
      if(c.isInt()){
	primmaterial.setColor(c.valueL());
      }else{
	primmaterial.setColor(c().convertToULong());
      }
    }catch(Except &e){};
    try{
      primradius=oroot("rimradius")();
    }catch(Except &e){};
    try{
      primradiusunits=(Xf::SceeneUnits)oroot("rimradiusunit.type").valueL();
    }catch(Except &e){};
  }catch(Except &e){};
  

  // new data
  try{
    Dobj nroot=root("PolyhedronProperties");
    try{
      pdrawfaces=nroot("drawfaces").valueL();
    }catch(Except &e){pdrawfaces=false;};
    
    try{
      pvisible=nroot("visible").valueL();
    }catch(Except &e){
    };
    
    try{
      pdrawrim=nroot("drawrim").valueL();
    }catch(Except &e){};
    
    QRgb col(0xff7777ff);
    QRgb spec(0xff323232);
    double shine=50.;
    try{
      Dobj c=nroot("facecolor");
      if(c.isInt()){
	col=c.valueL();
      }else{
	col=c().convertToULong();
      }
      spec=nroot("facespecular")().convertToULong();
      shine=nroot("faceshininess").valueD();
    }catch(Except &e){};
    pfacematerial=Material(col,spec,shine);

    //myDebug("PFACECOLOR %x",pfacecolor);
    //myDebug("PDRAWFACES %d",pdrawfaces);
    
    try{
      Dobj c=nroot("rimcolor");
      if(c.isInt()){
	primmaterial.setColor(c.valueL());
      }else{
	primmaterial.setColor(c().convertToULong());
      }
    }catch(Except &e){};
    try{
      primradius=nroot("rimradius")();
    }catch(Except &e){};
    try{
      primradiusunits=(Xf::SceeneUnits)nroot("rimradiusunit.type").valueL();
    }catch(Except &e){};
  }catch(Except &e){};

  setChanged(false);
  setCalculated(false);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PolyhedronProperties::calculate(Xf::LatticeType latticetype
				     ,double alloverscale,
				     Xf::LengthUnits datalengthunit)
{
  NSF("PolyhedronProperties::calculate");
  //myDebug()<<nsubfunc<<alloverscale;
  if(calculated())return;

  double u=constant_aBtoAng;
  if(latticetype==Xf::LTReciprocial) u=1/u;

  prr=primradius.D;
  if(primradiusunits == Xf::SUBoundary){
    prr*=(alloverscale);
  }else if(primradiusunits == Xf::SUAngstroem){
    if(datalengthunit==Xf::BohrRadius)
      prr/=u;
  }else if(primradiusunits == Xf::SUBohrRadius){
    if(datalengthunit==Xf::Angstroem)
      prr*=u;
  }

  //myDebug()<<nsubfunc<<prr;

  pchanged=false;
  pcalced=true;

  //myDebug()<<*this;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Polyhedron::clear()
{
  pplanes.clear();
  pnormals.clear();
  pvertices.clear();
  ppolyindices.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String AtomPolyhedron::printS()
{
  String s;
  s.sprintf("AtomPolyhedron:\n"
	    "\tNo of faces=%d\n"
	    ,ppolyindices.size());
  for(int ifa=0;ifa<ppolyindices.size();ifa++){
    s+=Sprintf("\tface %d: normal %s\n",ifa
	       ,pnormals[ifa].printS("%12.5f")()
	       );
    for(int ip=0;ip<ppolyindices[ifa].size();ip++){
      int ind=ppolyindices[ifa][ip];
      s+=Sprintf("\t\t%d: %s\n",ip,pvertices[ind].printS()());
    }
    
  }
  return s;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPolyhedron::calculateFaces(const ValueArray<Vec3> &atompositions
				,double theta0)
{
  //todo: create unique id based on production -> OGLPoly.pid
  
  
  bool prot=false;
  long natoms=atompositions.size();

  // determine kind of a center and bounding radius as a scale

  //todo: use that for bounding and NFPlane
  pradius=0;
  for(int i=0;i<natoms;i++){
    pmid+=atompositions.at(i);
    if(prot)myDebug("pos %3d: %s",i,atompositions.at(i).printS("%12.5f")()) ;
  }
  pmid/=natoms;
  for(int i=0;i<natoms;i++){
    pradius=MyMath::max<double>(pradius,norm(atompositions.at(i)-pmid));
  }

  if(prot){
    myDebug("mid: %s",pmid.printS("%12.5f")());
    myDebug("rad: %12.5f",pradius);
  }




//   // create list of planes, which might form the convex hull
//   QList<Plane> pla;
//   for(int i=0;i<natoms;i++){
//     const Vec3 &v1=atompositions.at(i);
//     for(int j=i+1;j<natoms;j++){
//       const Vec3 &v2=atompositions.at(j);
//       for(int k=j+1;k<natoms;k++){
// 	const Vec3 &v3=atompositions.at(k);
	
// 	//myDebug() << "v1"<<v1;
// 	//myDebug() << "v2"<<v2;
// 	//myDebug() << "v3"<<v3;
// 	Vec3 nvec=((v3-v1).cross(v2-v1));
// 	//myDebug() << "n "<<nvec;
// 	double nabs=norm(nvec);
// 	if(nabs<1.0e-6*pradius)continue;
// 	nvec/=nabs;
// 	double d=v1*nvec;
	
	

	
// 	bool left=false;
// 	bool right=false;
// 	for(int l=0;l<natoms;l++){
// 	  const Vec3 &vl=atompositions.at(l);
// 	  if(vl*nvec>d+pradius*planartol) right=true;
// 	  if(vl*nvec<d-pradius*planartol) left=true;
// 	  if(left && right) break;
// 	}
// 	if(left && right) continue;

// 	// (v-mid)*n >=0
// 	if((v1-pmid)*nvec<0){
// 	  d=-d;
// 	  nvec=-nvec;
// 	}

// 	pla.append(Plane(nvec,d,true));
// 	//myDebug() << "ijk"<<i<<j<<k;
	
//       }
//     }
//   }


//   if(prot){
//     myDebug() << "hull normal count"<<pla.size();
//     foreach(const Plane &pl,pla){
//       myDebug("hull normal %8.5f %8.5f %8.5f d= %8.5f"
// 	     ,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
//     }
//   }


//   if(0){
//     // remove similar planes, todo: use an average 
//     double costheta0=cos(theta0*Pi/180.);
//     if(prot){
//       myDebug() <<"costheta0"<<costheta0;
//     }
    
//     //todo how to preserve the convex hull while simplifying
//     //todo: allow for single planes -> CuO4
//     for(int i=pla.size()-1;i>=0;i--){
//       Plane pi=pla.at(i);
//       for(int j=0;j<i;j++){
// 	Plane pj=pla.at(j);
// 	//myDebug() <<"ij"<<i<<j<<pi.n()*pj.n();
// 	//if(fabs(pi.n()*pj.n())>costheta0){
// 	if((pi.n()*pj.n())>costheta0){
// 	  if(norm(pi.n()*pi.d()-pj.n()*pj.d())<planartol*pradius){
// 	    //myDebug() << "remove"<<i;
// 	    pla.removeAt(i);
// 	    break;
// 	  }
// 	}
//       }
//     }
//   }
//   if(prot){
//     myDebug() << "plane count"<<pla.size();
//     foreach(const Plane &pl,pla){
//       myDebug("normal %8.5f %8.5f %8.5f d= %8.5f"
// 	     ,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
//     }
//   }
  


//   //todo: introduce thickness for 4-corner single face, depending 
//   // on average distortion
//   for(int ip=0;ip<pla.size();ip++){
//     int ncorners=0;
//     Plane &pl=pla[ip];
//     if(prot){
//       myDebug("plane[%3d]  %8.5f %8.5f %8.5f d  = %8.5f"
// 	     ,ip,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
//     }
//     double daverage=0;
//     // which atompositions are on this plane?
//     for(int i=0;i<natoms;i++){
//       const Vec3 &v=atompositions.at(i);
//       double d=pl.n()*v;
//       if(fabs(d-pl.d())<pradius*planartol){
// 	daverage+=d;
// 	ncorners++;
//       }
//     }
//     daverage/=ncorners;
//     pl.setDistance(daverage);
    
//     if(prot){
//       myDebug("plane[%3d]  %8.5f %8.5f %8.5f dav= %8.5f corners=%d"
// 	     ,ip,pl.n()[0],pl.n()[1],pl.n()[2],pl.d(),ncorners);
//     }
    
//   }



// //   if(prot){
// //     myDebug() <<"npla"<<pla.size();
// //     myDebug() <<"pla"<<pla;
// //   }

   
//   double tol=1.0e-6;
//   if(pla.size()==1){
//     clear();

//     makeConvexHull2D(pla,atompositions);
    
//   }else{
//     QList<Plane> pplanes;
//     makeCorners(pla,tol,Vlevel(Vlevel::Silent),pplanes,pvertices,ppolyindices);
    
//     pnormals.clear();
//     foreach(const Plane &p,pplanes){
//       pnormals.append(p.n());
//     }
//   }
//   //myDebug()<<"pnormals"<<pnormals;

  makeConvexHull3D(atompositions,theta0);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPolyhedron::makeConvexHull3D(const ValueArray<Vec3> &points
				      ,double theta0)
{
  // given a point set in points construct the convex hull
  NSF("AtomPolyhedron::makeConvexHull3D");
  

  bool prot=false;
  long npoints=points.size();
  if(prot) myDebug()<< nsubfunc;

  int redocount=0;
 redo: ;
  
  if(prot){
    for(int i=0;i<npoints;i++){
      if(prot)myDebug("pos %3d: %s",i,points.at(i).printS("%12.5f")()) ;
    }
    myDebug("mid: %s",pmid.printS("%12.5f")());
    myDebug("rad: %12.5f",pradius);
  }
  
  double tol=1.0e-5;
  // create list of planes, which might form the convex hull
  ValueArray<Plane> pla;
  for(int i=0;i<npoints;i++){
    const Vec3 &v1=points.at(i);
    for(int j=i+1;j<npoints;j++){
      const Vec3 &v2=points.at(j);
      for(int k=j+1;k<npoints;k++){
	const Vec3 &v3=points.at(k);

	Vec3 nvec=((v3-v1).cross(v2-v1));
	double nabs=norm(nvec);
	if(nabs<1.0e-6*pradius)continue;
	nvec/=nabs;
	double d=v1*nvec;

	
	bool left=false;
	bool right=false;
	for(int l=0;l<npoints;l++){
	  const Vec3 &vl=points.at(l);
	  if(vl*nvec>d+pradius*tol) right=true;
	  if(vl*nvec<d-pradius*tol) left=true;
	  if(left && right) break;
	}
	if(left && right) continue;

	// (v-mid)*n >=0
	if((v1-pmid)*nvec<0){
	  d=-d;
	  nvec=-nvec;
	}

	pla.append(Plane(nvec,d,true));


      }
    }
  }
  if(prot){
    myDebug() << "hull normal count"<<pla.size();
    for(int i=-1;++i<pla.size();){
      const Plane &pl=pla.at(i);
      myDebug("hull normal %8.5f %8.5f %8.5f d= %8.5f"
	     ,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
    }
  }


  // find all points which lay on the planes
  ValueArray<ValueArray<int> > corners;
  for(int i=0;i<pla.size();i++){
    const Plane &pi=pla.at(i);
    corners.append(ValueArray<int>());
    for(int j=0;j<npoints;j++){
      if(fabs(points.at(j)*pi.n()-pi.d())<pradius*tol){
	corners[i].append(j);
      }
    }
  }
  

  if(prot){
    myDebug() << "planes/corners";
    for(int i=0;i<pla.size();i++){
      const Plane &pl=pla.at(i);
      myDebug("hull normal %d: %8.5f %8.5f %8.5f d= %8.5f"
	     ,i,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
      for(int j=-1;++j<corners.at(i).size();){
	int k=corners.at(i).at(j);
	myDebug(" corner: %s",points.at(k).printS("%8.5f")());
	
      }
    }
  }

  PtrArray<ValueArray<int> >bundles;
  for(int i=0;i<pla.size();i++) {
    bundles.append(new ValueArray<int>());
    bundles.last()->append(i);
  }
  
  
  double costheta0=cos(theta0*Pi/180.);
  if(prot){
    myDebug() <<"costheta0"<<costheta0;
  }
  
  bool done=false;
  
  while(!done){
    done=true;
    for(int ib=bundles.size()-1;ib>=0;ib--){ 
      ValueArray<int> *bi=bundles[ib];
      bool foundone=false;
      for(int jb=bundles.size()-1;jb>ib;jb--){ 
	const ValueArray<int> *bj=bundles.at(jb);
	for(int i=0;i<bi->size();i++){
	  const Plane &pi=pla.at(bi->at(i));
	  for(int j=0;j<bj->size();j++){
	    const Plane &pj=pla.at(bj->at(j));
	    
	    if((pi.n()*pj.n())>costheta0
	       //&& (norm(pi.n()*pi.d()-pj.n()*pj.d())<planartol*radius)
	       ){
	      // bundles are the same
	      foundone=true;
	      done=false;
	      for(int ii=-1;++ii<bj->size();){
		int k=bj->at(ii);
		//myDebug() << "add k?";
		if(!bi->contains(k)){
		  //myDebug() << "add "<<k<<" to "<<ib;
		  bi->append(k);
		}
	      }
	      bundles.removeAt(jb);
	      break;
	    }
	  }//in bj
	  if(foundone){
	    break;
	  }
	}//in bi
      }// bj
    }//bi
  }
  
  if(prot){
    for(int i=-1;++i<bundles.size();){
      const ValueArray<int> *l=bundles.at(i);
      myDebug()<<"bundle";
      for(int ii=-1;++ii<l->size();){
	int k=l->at(ii);
	myDebug()<<k<<pla.at(k).n();
      }
    }
  }

  ValueArray<Plane> pla2;
  for(int ib=bundles.size()-1;ib>=0;ib--){ 
    ValueArray<int> *bi=bundles[ib];
    Vec3 n;
    for(int i=0;i<bi->size();i++){
      int ip=bi->at(i);
      const Plane &pi=pla.at(ip);
      n+=pi.n();
    }
    n.normalize();

    double d=0;
    int count=0;
    for(int i=0;i<bi->size();i++){
      int ip=bi->at(i);
      for(int ii=-1;++ii<corners.at(ip).size();){
	int k=corners.at(ip).at(ii);
	count++;
	d+=points.at(k)*n;
      }
    }
    d/=count;
    pla2.append(Plane(n,d,true));
  }

  
  if(prot){
    myDebug() << "simplified planes";
    for(int i=0;i<pla2.size();i++){
      const Plane &pl=pla2.at(i);
      myDebug("hull normal %d: %8.5f %8.5f %8.5f d= %8.5f"
	     ,i,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
    }
  }

  if(pla2.size()==1){

    clear();
    makeConvexHull2D(pla2,points);
    pplanes=pla2;
    
  }else if(pla2.size()==2 && fabs(pla2.at(0).n()*pla2.at(1).n())>1-tol){

    clear();
    pla2.removeAt(1);
    makeConvexHull2D(pla2,points);
    pplanes=pla2;
    
  }else{  
    tol=1.0e-6;
    makeCorners(pla2,tol,Vlevel(Vlevel::Silent));
    
    if(pvertices.size()==0 && ! redocount){
      theta0=0;
      redocount++;
      goto redo;
    }
  }

  //exit(1);
  if(prot) myDebug()<< nsubfunc<<"END";

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPolyhedron::makeConvexHull2D(const ValueArray<Plane> &pla
				  ,const ValueArray<Vec3> &atompositions)
{
  //todo: model after 3d
  NSF("AtomPolyhedron::makeConvexHull2D");
  bool prot=false;
  pnormals.clear();
  pvertices.clear();
  ppolyindices.clear();
  if(prot){
    myDebug() << nsubfunc;
    myDebug() << pla.size();
  }
  if(pla.size()!=1){
    myDebug()<< "makeConvexHull2D called with more than one plane";
    return;
  }
  //myDebug() << "hier 1";
  //create a local coordinate system
  Vec3 nz=Vec3(pla.at(0).n()).normalize();
  Vec3 nx;
  double dx=fabs(nz*Vec3(1,0,0));
  double dy=fabs(nz*Vec3(0,1,0));
  double dz=fabs(nz*Vec3(0,0,1));
  //myDebug() << "hier 2";
  if(dy<dx){
    if(dz<dy){
      nx=Vec3(0,0,1);
    }else{ 
      nx=Vec3(0,1,0);
    }
  }else{
    if(dz<dx){
      nx=Vec3(0,0,1);
    }else{
      nx=Vec3(1,0,0);
    }
  }
  
  //myDebug() << "hier 3";
  nx=nz.cross(nx).normalize();
  Vec3 ny=nz.cross(nx).normalize();
  
  if(prot){
    myDebug()<< "nx"<< nx;
    myDebug()<< "ny"<< ny;
    myDebug()<< "nz"<< nz;
    Matrix3x3 m(nx,ny,nz);
    myDebug() << "m="<<m.transposed()*m;
    myDebug() << "m="<<m*m.transposed();
  }

  // project the points onto the plane perp to nz
  ValueArray<Vec3> points;
  for(int i=0;i<atompositions.size();i++){
    Vec3 v=atompositions.at(i);
    v+=nz*(pla.at(0).d()-v*nz);
    points.append(v);
    //myDebug() << points.last()*nz;
  }

  long npoints=points.size();
  Vec3 mid;
  double radius2=0;
  for(int i=0;i<npoints;i++){
    mid+=points.at(i);
    if(prot)myDebug("point %3d: %s",i,points.at(i).printS("%12.5f")()) ;
  }
  mid/=npoints;
  for(int i=0;i<npoints;i++){
    radius2=MyMath::max<double>(radius2,norm(points.at(i)-mid));
  }
  if(prot){
    myDebug("mid: %s",mid.printS("%12.5f")());
    myDebug("rad: %12.5f",radius2);
  }
  // cretae list of lines, which might form the convex hull
  ValueArray<Plane> pla2;
  double planartol=1.0e-6;
  for(int i=0;i<npoints;i++){
    const Vec3 &v1=points.at(i);
    for(int j=i+1;j<npoints;j++){
      const Vec3 &v2=points.at(j);
      Vec3 nvec=((v2-v1).cross(nz));
      double nabs=norm(nvec);
      if(nabs<1.0e-6*radius2)continue;
      nvec/=nabs;
      double d=v1*nvec;
      bool left=false;
      bool right=false;
      for(int l=0;l<npoints;l++){
	const Vec3 &vl=points.at(l);
	if(vl*nvec>d+pradius*planartol) right=true;
	if(vl*nvec<d-pradius*planartol) left=true;
	if(left && right) break;
      }
      if(left && right) continue;

      // (v-mid)*n >=0
      if((v1-mid)*nvec<0){
	d=-d;
	nvec=-nvec;
      }
      pla2.append(Plane(nvec,d,true));

    }
  }
  
  if(prot){
    myDebug() << "hull normal count"<<pla2.size();
  }

  // remove similar planes, todo: use an average 
  double costheta0=cos(0.1*Pi/180.);
  if(prot){
    myDebug() <<"costheta0"<<costheta0;
  }
  for(int i=pla2.size()-1;i>=0;i--){
    Plane pi=pla2.at(i);
    for(int j=0;j<i;j++){
      Plane pj=pla2.at(j);
      //myDebug() <<"ij"<<i<<j<<pi.n()*pj.n();
      if(fabs(pi.n()*pj.n())>costheta0){
	if(norm(pi.n()*pi.d()-pj.n()*pj.d())<planartol*pradius){
	  //myDebug() << "remove"<<i;
	  pla2.removeAt(i);
	  break;
	}
      }
    }
  }

  
  if(prot){
    myDebug() << "plane count"<<pla2.size();
    for(int i=-1;++i<pla2.size();){
      const Plane &pl=pla2.at(i);
      myDebug("normal %8.5f %8.5f %8.5f d= %8.5f"
	     ,pl.n()[0],pl.n()[1],pl.n()[2],pl.d());
    }
  }

  // take all points on the convex hull
  ValueArray<int> lvert;
  ppolyindices.append(ValueArray<long>());
  for(int i=0;i<pla2.size();i++){
    const Plane &pi=pla2.at(i);
    for(int j=0;j<points.size();j++){
      const Vec3 &v=points.at(j);
      if(fabs(v*pi.n()-pi.d())<pradius*planartol){
	if(!lvert.contains(j)){
	  lvert.append(j);
	  ppolyindices[0].append(lvert.size()-1);
	}
      }
    }
  }  
  for(int j=0;j<lvert.size();j++){
    pvertices.append(points.at(lvert.at(j)));
  }

  normPoly(ppolyindices[0],pvertices,nz,pradius*planartol,false);

//   myDebug() << ppolyindices;
//   myDebug() << pvertices;

  pnormals.append(nz);

}

/* ***********************************************************************
 * sort a polygon of nc points (laying in plane r*plane=something)
 * so that the lines do not cross
 * ***********************************************************************
 */

void Polyhedron::normPoly(ValueArray<long> &vert,const ValueArray<Vec3> &pverts
			  ,const Vec3 &plane_n,double tol,long prot)
{
  long nc=vert.size();

  // local copy of plane -> normalized
  Vec3 v3=plane_n; v3.normalize();

  // midpoint 
  Vec3 mid;
  for(long ic=0;ic<nc;ic++) mid+=pverts.at(vert.at(ic));
  mid/=nc;

  if(nc>3){
    if(prot ){
      printf("normpoly:---------------------------\n"); 
      printf("plane : %s \n",v3.printS()());
      for(long ic=0;ic<nc;ic++){
	Vec3 v=pverts.at(vert.at(ic));
	printf("vertex %ld: %s\n",ic,v.printS()());
      }
    }
    
    
    
    
    
    // a nonzero vector in plane 
    Vec3 v1;
    for(long ic=0;ic<nc;ic++){
      v1=mid-pverts.at(vert.at(ic));
      if(norm(v1) > tol )  break;
    }
    v1.normalize();
    // third vector 
    Vec3 v2=v3.cross(v1);
    
    if(prot ){
      printf("projection vectors:\n"); 
      printf("v1 %f %f %f \n",v1[0],v1[1],v1[2]); 
      printf("v2 %f %f %f \n",v2[0],v2[1],v2[2]); 
      printf("p  %f %f %f \n",v3[0],v3[1],v3[2]); 
    }
    
    double *c=(double*)malloc(sizeof(double)*2*nc);
    // project points onto plane 
    if(prot ) printf("projected vertex"); 
    
    ValueArray<long> iwk;
    for(long ic=0;ic<nc;ic++){
      c[0+2*ic]=pverts.at(vert.at(ic))*v1;
      c[1+2*ic]=pverts.at(vert.at(ic))*v2;
      if(prot ) printf("c  %f %f  \n",c[0+2*ic],c[1+2*ic]); 
      
      iwk.append(ic);
    }
    
    
    long hit;
    do{
      hit=0;
      if(prot ){
	printf("start loop\n");   
	printf("poly "); for(long ic=0;ic<nc;ic++) printf("%ld ",iwk[ic]); 
	printf("\n"); 
      }
      
      for(long ic=0;ic<nc;ic++){
	long jc=(ic+1)%nc;
	for(long j=0;j<nc-3;j++){
	  long kc=(j+jc+1)%nc;
	  long lc=(kc+1)%nc;
	  if(prot )
	    printf("try %ld-%ld  ,  %ld-%ld\n",iwk[ic],iwk[jc],iwk[kc]
		   ,iwk[lc]); 
	  if(lineCross2(c+2*iwk[ic],c+2*iwk[jc],c+2*iwk[kc],c+2*iwk[lc],tol)){
	    hit=1;
	    if(prot )
	      printf("hit %ld-%ld , %ld-%ld\n",iwk[ic],iwk[jc],iwk[kc],iwk[lc]); 
	    
	    long n1=kc;
	    long n2=jc;
	    if(prot )
	      printf("exch %ld <-> %ld\n",n1,n2); 
	    
	    long l=iwk[n1];
	    iwk[n1]=iwk[n2];
	    iwk[n2]=l;
	    if(prot ){
	      printf("poly ");
	      for(long ic=0;ic<nc;ic++) printf("%ld ",iwk[ic]); printf("\n"); 
	    }
	    goto end;
	  }
	}
      }
    end:;
      if(prot ){
	printf("poly ");
	for(long ic=0;ic<nc;ic++) printf("%ld ",iwk[ic]); printf("\n"); 
      }
    }while(hit);
    
    free(c);
    
    ValueArray<long> l;
    for(long ic=0;ic<nc;ic++) l.append(vert[iwk[ic]]);
    vert=l;
  }
 
  // orientation
 
  int N=vert.size();
  if(N>=1){
    if(((pverts.at(vert.at(0))-mid).cross(pverts.at(vert.at(1))-mid))*v3<0.){
      for(int i=0;i<(N/2);i++){
	vert.swap(i,N-1-i);
      }
    }
  }

  return;
}


/* ***********************************************************************
 * given a set of planes, calculate innermost polyhedron
 * ***********************************************************************
 */
void Polyhedron::makeCorners(const ValueArray<Plane> &pla,double tol
			     ,const Vlevel &vlevel)


{
  NSF("Polyhedron::makeCorners");

  bool prot=(vlevel >= Vlevel::More );
  if(prot)myDebug() << nsubfunc;
  //myDebug() <<"npla"<<pla.size();
  //myDebug() <<"pla"<<pla;

  pplanes.clear();
  pvertices.clear();
  pnormals.clear();
  ppolyindices.clear();

  // search corners, which form the boundary of the innermost  polyhedron 
  long np=pla.size();
  ValueArray<Vec3> rwk;
  for(long i1=0;i1<np;i1++){
    for(long i2=i1+1;i2<np;i2++){
      for(long i3=i2+1;i3<np;i3++){
	// if they are not coplanar 
	double f=pla.at(i1).n()*pla.at(i2).n().cross(pla.at(i3).n());

	if(fabs(f)>tol){

	  // rwk[js] is common point 
	  Vec3 r;
	  if(!Plane::govlap(r,pla.at(i1),pla.at(i2),pla.at(i3))){
	    pla.at(i1).n().print("p1");
	    pla.at(i2).n().print("p2");
	    pla.at(i3).n().print("p3");
	    myDebug() << "vol=" <<f;
	    printf("%s: volume was != 0 but now it is 0 ?? Sorry",nsubfunc);
	    exit(2); //goto clean1;
	  }
	  rwk.append(r);
	  
	}

      }	// i3 
    } // i2 
  } // i1 
  // here rwk is the array of corners=points in common of three planes
  
  //myDebug() <<"rwk"<<rwk;

  // tol prevents the innermost corners from being sorted out 
  // by numerical noice (at least I hope so) 

  ValueArray<long> iwk(rwk.size(),1);
  
  for(long i=rwk.size()-1;i>=0;i--){
    if(!iwk.at(i))continue;
    // for all pla 
    //myDebug()<< i<<rwk[i];
    for(long i1=0;i1<np;i1++){
      //myDebug()<< "\t"<<i1<<pla[i1].n()<<rwk[i]*pla[i1].n()<<pla[i1].d();
      if( rwk[i]*pla.at(i1).n() > pla.at(i1).d()+tol) { 
	// if corner*n > d+tol for any plane, 
	// then it is not an inner point 
	//myDebug() << "remove"<<i;
	//rwk.removeAt(i);
	iwk[i]=0;
	break;
      }
    }
  }
  // delete all for which iwk[...]==0
  long i0=0;
  long N=iwk.size();
  for(long i1=-1;++i1<N;){
    if(iwk[i1]){
      if(i0!=i1){
	rwk[i0]=rwk[i1];
      }
      i0++;
      
    }
  }
  rwk.resize(i0,true);


  //myDebug() <<"rwk removed"<<rwk;

  // unify the set of inner points, there might be multiples 
  for(long i=rwk.size()-2;i>=0;i--){
    for(long j=rwk.size()-1;j>i;j--){
      if(norm(rwk[j]-rwk[i]) < tol) rwk.removeAt(j);
    }
  }
  // each plane should contain at least 3 inner corner points 

  //myDebug() <<"rwk unified"<<rwk;


  for(long i1=0;i1<np;i1++){ // for all planes

    // save corner points index for later use 
    ValueArray<long> vert;
    for(long i=0;i<rwk.size();i++)
      if(fabs((rwk[i]*pla.at(i1).n())-pla.at(i1).d())<tol) vert.append(i);
    
    // at least 3 corners at plane pla(,i1) -> avoid degenerate planes
    if (vert.size() >= 3){
      
      // copy innermost pla 
      pplanes.append(pla.at(i1));
      //insert vertices, avoid multiples
      for(long i=0;i<vert.size();i++){
	Vec3 v=rwk[vert[i]];
	long ind=-1;
	for(int iv=0;iv<pvertices.size();iv++)
	  if(norm(pvertices[iv]-v)<tol){ ind=iv; break; }
	if(ind<0){
	  pvertices.append(v);
	  ind=pvertices.size()-1;
	}
	vert[i]=ind;
      }

      
      normPoly(vert,pvertices,pla.at(i1).n(),tol,(vlevel >= Vlevel::All));


      ppolyindices.append(vert);


    }// more than 3 corners 
  } // all pla 



  

  // debug output
  if(prot){
    printf("%s: planes ------------------------------------------------\n"
	   ,nsubfunc);
    
    for(long ip=0;ip<pplanes.size();ip++){
      printf("%2ld : %s |plane| %-10.5f corners %ld\n"
	     ,ip+1,pplanes.at(ip).n().printS("% 10.5f")()
	     ,pplanes.at(ip).d(),ppolyindices.at(ip).size());
      }

    printf("%s: vertices-----------------------------------------------\n"
	   ,nsubfunc);
    for(long i=0;i<pvertices.size();i++){
      printf("%4ld : %s |vertex| % 10.5f\n"
	     ,i+1,pvertices[i].printS("% 10.5f")(),norm(pvertices[i]));
    }
    printf("%s: -------------------------------------------------------\n"
	   ,nsubfunc);
  }



  rwk.clear();
  
  for(long i=-1;++i<pplanes.size();){
    pnormals.append(pplanes.at(i).n());
  }

  pradius=0;
  for(int i=0;i<pvertices.size();i++){
    pmid+=pvertices.at(i);
  }
  pmid/=pvertices.size();
  for(int i=0;i<pvertices.size();i++){
    pradius=MyMath::max<double>(pradius,norm(pvertices.at(i)-pmid));
  }


  if(prot) myDebug() << nsubfunc << "END";

  return;
}

/*EOF*/ 


