/* @(#)Polyhedron.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _POLYHEDRON_H
#define _POLYHEDRON_H 1

#include "MyQRgb.hh"
#include "FString.hh"
#include "Plane.hh"
#include "ValueArray.hh"
#include "Material.hh"
#include "SReal.hh"
#include "MACROS.hh"
#include "Vec3.hh"
#include "Xf.hh"


class Vlevel;
class Dobj;

//todo: separate properties

namespace XFPLO{

class PolyhedronProperties{
public:
  PolyhedronProperties()
 :pvisible(true),pfacematerial(0x4C00FFFF,50),pdrawfaces(false)
 ,pdrawnormals(false),pdrawrim(true),primmaterial(0xdd00dd,50.)
 ,pchanged(false),pcalced(false)
  {
    setDefaultRim();
  };

  const  PolyhedronProperties& operator=(const PolyhedronProperties &o){
    setVisible(o.pvisible);
    setDrawFaces(o.pdrawfaces);
    setFaceMaterial(o.pfacematerial);
    setRimMaterial(o.primmaterial);
    setDrawRim(o.pdrawrim);
    setRimRadius(o.primradius);
    setRimRadiusUnits(o.primradiusunits);
    pchanged=o.pchanged;
    pcalced=o.pcalced;
    prr=o.prr;
    return *this;
  }

  inline bool calculated()const{return pcalced;};
  inline void setCalculated(bool c){pcalced=c;};

  inline bool changed()const{return pchanged;};
  inline void setChanged(bool c){pchanged=c;};



  
  inline bool drawFaces()const{return pdrawfaces;};
  inline void setDrawFaces(bool d){pchanged|=(pdrawfaces!=d);
    pdrawfaces=d; pcalced&=!pchanged;};

  inline bool drawNormals()const{return pdrawnormals;};
  inline void setDrawNormals(bool d){pchanged|=(pdrawnormals!=d);
    pdrawnormals=d; pcalced&=!pchanged;};

  inline bool drawRim()const{return pdrawrim;};
  inline void setDrawRim(bool d){pchanged|=(pdrawrim!=d);
    pdrawrim=d; pcalced&=!pchanged;};


  inline void setRimRadius(const SReal &r){pchanged|=(primradius!=r); 
    primradius=r; pcalced&=!pchanged; };
  inline const SReal &rimRadius()const{return primradius;};
  inline Xf::SceeneUnits rimRadiusUnits()const{return primradiusunits;};
  inline void setRimRadiusUnits(Xf::SceeneUnits u){
    pchanged|=(primradiusunits!=u); primradiusunits=u;pcalced&=!pchanged;};
  inline void setDefaultRim(){ setRimRadius(SReal("0.02")); 
    setRimRadiusUnits(Xf::SUBoundary); };

  void calculate(Xf::LatticeType latticetype,
		 double alloverscale,
		 Xf::LengthUnits datalengthunits);

  inline double dispRimRadius()const{return prr;};
  
  static void PTHeaderToFile(FILE *FH,const char *prefix,const char *tail);
  void PTBodyToFile(FILE *FH,const char *prefix,const char *tail)const;
  void readPTSectionFromFile(const Dobj &root);

 private:
  PROPERTY(visible,isVisible,Visible,bool);
  CPROPERTY_CHANGED_CALCED(facematerial,faceMaterial,FaceMaterial,Material);
  bool pdrawfaces;
  bool pdrawnormals;
  bool pdrawrim;
  CPROPERTY_CHANGED_CALCED(rimmaterial,rimMaterial,RimMaterial,Material);
  SReal primradius;
  Xf::SceeneUnits primradiusunits;
  double prr;
  bool pchanged;
  bool pcalced;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Polyhedron{
 public:

  Polyhedron():pisA("Polyhedron"){};

/*  const  Polyhedron& operator=(const Polyhedron &other){ */
/*    pplanes=other.pplanes; */
/*    pnormals=other.pnormals; */
/*    pvertices=other.pvertices; */
/*    ppolyindices=other.ppolyindices; */
/*    pisA=other.pisA; */
/*    pradius=other.pradius; */
/*    pmid=other.pmid; */
/*    pproperties=other.pproperties; */
/*    return *this; */
/*  } */

  void clear();

  inline bool isA(const String &i)const{return pisA==i;};

  inline double radius()const{return pradius;};
  inline const Vec3 &midPoint()const{return pmid;};

  inline const ValueArray<ValueArray<long> > &polyIndices()const{return ppolyindices;};
  inline const ValueArray<Vec3> &normals()const{return pnormals;};
  inline const ValueArray<Vec3> &vertices()const{return pvertices;};
  inline const ValueArray<Plane> &planes()const{return pplanes;};


  PolyhedronProperties &properties(){return pproperties;};
  const PolyhedronProperties &properties()const{return pproperties;};
  
  // given a set of planes in pla, calculate planes, faces, vertices
  // of the innermost convex polyhedron defined by all pla's
  void makeCorners(const ValueArray<Plane> &pla,double tol,const Vlevel &vlevel);

 protected:
  static void normPoly(ValueArray<long> &vert,const ValueArray<Vec3> &pverts
		       ,const Vec3 &plane_n,double tol,long prot);


 protected:
  ValueArray<Plane> pplanes;
  ValueArray<Vec3> pnormals;
  ValueArray<Vec3> pvertices;
  ValueArray<ValueArray<long> > ppolyindices;
  String pisA;
  double pradius;
  Vec3 pmid;

 private:
  PolyhedronProperties pproperties;
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class AtomPolyhedron: public Polyhedron{
 public:

  AtomPolyhedron():Polyhedron(),relatedsite(-1){pisA="AtomPolyhedron";};
    
  String printS();




  void calculateFaces(const ValueArray<Vec3> &atompositions,double theta0);
  

 private:

  void makeConvexHull2D(const ValueArray<Plane> &pla
			,const ValueArray<Vec3> &atompositions);
  void makeConvexHull3D(const ValueArray<Vec3> &points,double theta0);
  
 public:
  long relatedsite; //we reference the poly by a site in order to put it on all
  // the cluster atoms
  Vec3 position;

  



};

}
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _POLYHEDRON_H */

