/* @(#)Position.hh
 * author: magru@unas
 */

#ifndef _POSITION_HH
#define _POSITION_HH 1

#include "Array.hh"
#include "Module.hh"
#include "MyDebug.hh"

namespace XFPLO{

class BoundingBox;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Positions:public Module{

private:
  Positions();
  Positions(const Positions&);
  Positions &operator=(const Positions &);
  virtual ~Positions();
public: 

  //todo: we need setters for irregular, or product
  //todo: product can be all regular or a mix, which we treat as total irregular
  //todo: both long and double
  //todo: sharing
  //todo: dependency of connections

  static Positions *construct(){
      return new Positions();
    }

  static void destroy(Positions *p){
    if((--(p->pusecounter))==0) delete p;
  }
   

  // void setSize(long s){
  //   pdata->setSize(s);
  //   pisdirty=true;
  // }

  // all regular means Productarray of D regulars
  // shape means position is vector[shape]
  // dimension must be <=shape
  // 2D surface embedded in 3D is: shape=3, D=2
  // gridsize.size()==D

  void setGridD(long vectordimension,
		const ValueArray<long> &gridsize, // Nx x Ny ( x Nz)
		const ValueArray<double> &origin, // one origin for all
		const ValueArray<ValueArray<double> > &deltas
		);

  void setupArray(const ValueArray<long> &shape){
    clearData();
    pdata=BaseArray<double>::construct(shape);

  }

  inline long embeddingDimension()const{return pdata->shape(0);}


  inline BaseArray<double> *array(){return pdata;}
  inline const BaseArray<double> *array()const{return pdata;}

  inline const BoundingBox &boundingBox()const{return *pbbox;}

  void calculateBoundingBox();

  virtual bool cascadingUpdate(const String &prefix);
    

private:

  inline void increaseUse(){pusecounter++;}
  void clearData(){
    if(pdata){
      pdata->destroy(pdata);
      pdata=0;
    }
  }

private:
  long pusecounter;
  BaseArray<double> *pdata;
  BoundingBox *pbbox;
};



}
#endif /* _POSITION_HH */

