/*************************************************************************
 * @(#)PropertyTable.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   09 Jul 2009
 *************************************************************************
 */

#include "PropertyTable.hh"
#include "Settings.hh"

namespace XFPLO{
  AtomPropertyTables *globalproperties;

  ValueArray<Xf::PairIntCharP> AtomicRadii::radiinames=
    (ValueArray<Xf::PairIntCharP>()
     <<Xf::PairIntCharP(AtomicRadii::ARCovalent,"covalent")
     <<Xf::PairIntCharP(AtomicRadii::ARIonic,"ionic")
     <<Xf::PairIntCharP(AtomicRadii::ARvdW,"vdW")
     <<Xf::PairIntCharP(AtomicRadii::ARMetallic,"metallic")
     <<Xf::PairIntCharP(AtomicRadii::ARCrystal,"crystal")
     );

}


using namespace XFPLO;



const StringList AtomPropertyTable::allowedElements=
  (StringList() <<"E"<<"H"<<"He"<<"Li"<<"Be"<<"B"<<"C"<<"N"<<"O"
   <<"F"<<"Ne"<<"Na"<<"Mg"<<"Al"<<"Si"<<"P"<<"S"<<"Cl"<<"Ar"<<"K"
   <<"Ca"<<"Sc"<<"Ti"<<"V"<<"Cr"<<"Mn"<<"Fe"<<"Co"<<"Ni"<<"Cu"<<"Zn"
   <<"Ga"<<"Ge"<<"As"<<"Se"<<"Br"<<"Kr"<<"Rb"<<"Sr"<<"Y"<<"Zr"<<"Nb"
   <<"Mo"<<"Tc"<<"Ru"<<"Rh"<<"Pd"<<"Ag"<<"Cd"<<"In"<<"Sn"<<"Sb"<<"Te"
   <<"I"<<"Xe"<<"Cs"<<"Ba"<<"La"<<"Ce"<<"Pr"<<"Nd"<<"Pm"<<"Sm"<<"Eu"
   <<"Gd"<<"Tb"<<"Dy"<<"Ho"<<"Er"<<"Tm"<<"Yb"<<"Lu"<<"Hf"<<"Ta"<<"W"
   <<"Re"<<"Os"<<"Ir"<<"Pt"<<"Au"<<"Hg"<<"Tl"<<"Pb"<<"Bi"<<"Po"<<"At"
   <<"Rn"<<"Fr"<<"Ra"<<"Ac"<<"Th"<<"Pa"<<"U"<<"Np"<<"Pu"<<"Am"<<"Cm"
   <<"Bk"<<"Cf"<<"Es"<<"Fm"<<"Md"<<"No"<<"Lr"<<"Rf");

;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const AtomProperty &AtomPropertyTable::find(const String &e) const
{
  //   NSF("AtomPropertyTable::find");
  //   myDebug() << nsubfunc;

  
  long i=list.indexOf(makePair<String,AtomProperty>(e.capitalized(),AtomProperty()));
  
  if(i<0){
    return defaultProperty;
  }else{
    return list.at(i).v2;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomProperty &AtomPropertyTable::find(const String &e)
{
  long i=list.indexOf(makePair<String,AtomProperty>(e.capitalized(),AtomProperty()));
  if(i<0){
    return defaultProperty;
  }else{
    return list[i].v2;
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long AtomPropertyTable::indexOf(const String &e) const
{
  return list.indexOf(makePair<String,AtomProperty>(e.capitalized(),AtomProperty()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropertyTable::loadDefault()
{
  list.clear();

#define PP Pair<String,AtomProperty>  
  list.append(PP("E" ,AtomProperty(Material(0xfcf8b3,50.),AtomicRadii(1.,1.,1.,1.00,1.00))));
  list.append(PP("H" ,AtomProperty(Material(0xFFFFFF,50.),AtomicRadii(0.37,0.25,1.20,0.00,0.10))));
  list.append(PP("He",AtomProperty(Material(0xD9FFFF,50.),AtomicRadii(0.32,0.31,1.40,0.00,1.40))));
	           
  list.append(PP("Li",AtomProperty(Material(0xCC80FF,50.),AtomicRadii(1.34,0.76,1.82,1.52,0.90))));
  list.append(PP("Be",AtomProperty(Material(0xC2FF00,50.),AtomicRadii(0.90,0.35,0.00,1.12,0.41))));
  list.append(PP("B" ,AtomProperty(Material(0xFFB5B5,50.),AtomicRadii(0.82,0.23,0.00,0.98,0.25))));
  list.append(PP("C" ,AtomProperty(Material(0x909090,50.),AtomicRadii(0.77,0.19,1.70,0.86,0.29))));
  list.append(PP("N" ,AtomProperty(Material(0x3050F8,50.),AtomicRadii(0.75,0.13,1.55,0.53,0.30))));
  list.append(PP("O" ,AtomProperty(Material(0xFF0D0D,50.),AtomicRadii(0.73,1.40,1.52,0.00,1.21))));
  list.append(PP("F" ,AtomProperty(Material(0x90E050,50.),AtomicRadii(0.71,1.33,1.47,0.00,1.19))));
  list.append(PP("Ne",AtomProperty(Material(0xB3E3F5,50.),AtomicRadii(0.69,0.00,1.54,0.00,1.54))));
	           
  list.append(PP("Na",AtomProperty(Material(0xAB5CF2,50.),AtomicRadii(1.54,1.02,2.27,1.91,1.16))));
  list.append(PP("Mg",AtomProperty(Material(0x8AFF00,50.),AtomicRadii(1.30,0.72,1.73,1.60,0.86))));
	           
  list.append(PP("Al",AtomProperty(Material(0xBFA6A6,50.),AtomicRadii(1.18,0.53,0.00,1.43,0.53))));
  list.append(PP("Si",AtomProperty(Material(0xF0C8A0,50.),AtomicRadii(1.11,0.40,2.10,1.38,0.40))));
  list.append(PP("P" ,AtomProperty(Material(0xFF8000,50.),AtomicRadii(1.06,0.38,1.80,1.28,0.31))));
  list.append(PP("S" ,AtomProperty(Material(0xFFFF30,50.),AtomicRadii(1.02,0.37,1.80,1.27,0.43))));
  list.append(PP("Cl",AtomProperty(Material(0x1FF01F,50.),AtomicRadii(0.99,1.81,1.75,0.91,1.67))));
  list.append(PP("Ar",AtomProperty(Material(0x80D1E3,50.),AtomicRadii(0.97,0.00,1.88,0.00,1.88))));
	           
  list.append(PP("K" ,AtomProperty(Material(0x8F40D4,50.),AtomicRadii(1.96,1.38,2.75,2.35,1.52))));
  list.append(PP("Ca",AtomProperty(Material(0x3DFF00,50.),AtomicRadii(1.74,0.99,0.00,1.97,1.14))));
	           
  list.append(PP("Sc",AtomProperty(Material(0xE6E6E6,50.),AtomicRadii(1.44,0.75,0.00,1.64,0.89))));
  list.append(PP("Ti",AtomProperty(Material(0xBFC2C7,50.),AtomicRadii(1.36,0.60,0.00,1.47,0.75))));
  list.append(PP("V" ,AtomProperty(Material(0xA6A6AB,50.),AtomicRadii(1.25,0.59,0.00,1.34,0.68))));
  list.append(PP("Cr",AtomProperty(Material(0x8A99C7,50.),AtomicRadii(1.27,0.52,0.00,1.28,0.76))));
  list.append(PP("Mn",AtomProperty(Material(0x9C7AC7,50.),AtomicRadii(1.39,0.46,0.00,1.27,0.81))));
  list.append(PP("Fe",AtomProperty(Material(0xE06633,50.),AtomicRadii(1.25,0.65,0.00,1.26,0.69))));
  list.append(PP("Co",AtomProperty(Material(0xF090A0,50.),AtomicRadii(1.26,0.75,0.00,1.25,0.54))));
  list.append(PP("Ni",AtomProperty(Material(0x50D050,50.),AtomicRadii(1.21,0.69,1.63,1.24,0.70))));
  list.append(PP("Cu",AtomProperty(Material(0xC88033,50.),AtomicRadii(1.38,0.73,1.40,1.28,0.71))));
  list.append(PP("Zn",AtomProperty(Material(0x7D80B0,50.),AtomicRadii(1.31,0.74,1.39,1.34,0.74))));
	           
  list.append(PP("Ga",AtomProperty(Material(0xC28F8F,50.),AtomicRadii(1.26,0.62,1.87,1.40,0.76))));
  list.append(PP("Ge",AtomProperty(Material(0x668F8F,50.),AtomicRadii(1.22,0.53,0.00,1.44,0.53))));
  list.append(PP("As",AtomProperty(Material(0xBD80E3,50.),AtomicRadii(1.19,0.58,1.85,1.48,0.72))));
  list.append(PP("Se",AtomProperty(Material(0xFFA100,50.),AtomicRadii(1.16,0.50,1.90,1.40,0.56))));
  list.append(PP("Br",AtomProperty(Material(0xA62929,50.),AtomicRadii(1.14,1.96,1.85,1.17,1.82))));
  list.append(PP("Kr",AtomProperty(Material(0x5CB8D1,50.),AtomicRadii(1.10,0.00,2.02,0.00,2.02))));
	           
  list.append(PP("Rb",AtomProperty(Material(0x702EB0,50.),AtomicRadii(2.11,1.52,0.00,2.48,1.66))));
  list.append(PP("Sr",AtomProperty(Material(0x00FF00,50.),AtomicRadii(1.92,1.12,0.00,2.15,1.32))));
	           
  list.append(PP("Y" ,AtomProperty(Material(0x94FFFF,50.),AtomicRadii(1.62,0.90,0.00,1.80,1.04))));
  list.append(PP("Zr",AtomProperty(Material(0x94E0E0,50.),AtomicRadii(1.48,0.72,0.00,1.60,0.86))));
  list.append(PP("Nb",AtomProperty(Material(0x73C2C9,50.),AtomicRadii(1.37,0.68,0.00,1.46,0.78))));
  list.append(PP("Mo",AtomProperty(Material(0x54B5B5,50.),AtomicRadii(1.45,0.65,0.00,1.39,0.79))));
  list.append(PP("Tc",AtomProperty(Material(0x3B9E9E,50.),AtomicRadii(1.56,0.56,0.00,1.36,0.79))));
  list.append(PP("Ru",AtomProperty(Material(0x248F8F,50.),AtomicRadii(1.26,0.68,0.00,1.34,0.82))));
  list.append(PP("Rh",AtomProperty(Material(0x0A7D8C,50.),AtomicRadii(1.35,0.68,0.00,1.34,0.81))));
  list.append(PP("Pd",AtomProperty(Material(0x006985,50.),AtomicRadii(1.31,0.86,1.63,1.37,0.78))));
  list.append(PP("Ag",AtomProperty(Material(0xC0C0C0,50.),AtomicRadii(1.53,1.26,1.72,1.44,1.29))));
  list.append(PP("Cd",AtomProperty(Material(0xFFD98F,50.),AtomicRadii(1.48,0.97,1.58,1.51,0.92))));
	           
  list.append(PP("In",AtomProperty(Material(0xA67573,50.),AtomicRadii(1.44,0.80,1.93,1.58,0.94))));
  list.append(PP("Sn",AtomProperty(Material(0x668080,50.),AtomicRadii(1.41,0.69,2.17,1.63,0.69))));
  list.append(PP("Sb",AtomProperty(Material(0x9E63B5,50.),AtomicRadii(1.38,0.76,0.00,1.66,0.90))));
  list.append(PP("Te",AtomProperty(Material(0xD47A00,50.),AtomicRadii(1.35,0.97,2.06,1.60,1.11))));
  list.append(PP("I" ,AtomProperty(Material(0x940094,50.),AtomicRadii(1.33,2.20,1.98,1.39,2.06))));
  list.append(PP("Xe",AtomProperty(Material(0x429EB0,50.),AtomicRadii(1.30,0.00,2.16,0.00,0.62))));
	           
  list.append(PP("Cs",AtomProperty(Material(0x57178F,50.),AtomicRadii(2.25,1.67,0.00,2.65,1.81))));
  list.append(PP("Ba",AtomProperty(Material(0x00C900,50.),AtomicRadii(1.98,1.35,0.00,2.22,1.49))));
	           
  list.append(PP("La",AtomProperty(Material(0x70D4FF,50.),AtomicRadii(1.69,1.06,0.00,1.88,1.36))));
	           
  list.append(PP("Ce",AtomProperty(Material(0xFFFFC7,50.),AtomicRadii(1.65,1.03,0.00,1.83,1.15))));
  list.append(PP("Pr",AtomProperty(Material(0xD9FFC7,50.),AtomicRadii(1.65,1.01,0.00,1.83,1.32))));
  list.append(PP("Nd",AtomProperty(Material(0xC7FFC7,50.),AtomicRadii(1.64,0.99,0.00,1.82,1.30))));
  list.append(PP("Pm",AtomProperty(Material(0xA3FFC7,50.),AtomicRadii(1.63,0.98,0.00,1.81,1.28))));
  list.append(PP("Sm",AtomProperty(Material(0x8FFFC7,50.),AtomicRadii(1.62,0.96,0.00,1.80,1.10))));
  list.append(PP("Eu",AtomProperty(Material(0x61FFC7,50.),AtomicRadii(1.85,0.95,0.00,2.04,1.31))));
  list.append(PP("Gd",AtomProperty(Material(0x45FFC7,50.),AtomicRadii(1.61,0.94,0.00,1.80,1.08))));
  list.append(PP("Tb",AtomProperty(Material(0x30FFC7,50.),AtomicRadii(1.59,0.92,0.00,1.78,1.18))));
  list.append(PP("Dy",AtomProperty(Material(0x1FFFC7,50.),AtomicRadii(1.59,0.91,0.00,1.77,1.05))));
  list.append(PP("Ho",AtomProperty(Material(0x00FF9C,50.),AtomicRadii(1.58,0.90,0.00,1.77,1.04))));
  list.append(PP("Er",AtomProperty(Material(0x00E675,50.),AtomicRadii(1.57,0.88,0.00,1.76,1.03))));
  list.append(PP("Tm",AtomProperty(Material(0x00D452,50.),AtomicRadii(1.56,0.87,0.00,1.75,1.02))));
  list.append(PP("Yb",AtomProperty(Material(0x00BF38,50.),AtomicRadii(1.74,0.86,0.00,1.94,1.13))));
  list.append(PP("Lu",AtomProperty(Material(0x00AB24,50.),AtomicRadii(1.60,0.85,0.00,1.73,1.10))));
	           
  list.append(PP("Hf",AtomProperty(Material(0x4DC2FF,50.),AtomicRadii(1.50,0.71,0.00,1.59,0.85))));
  list.append(PP("Ta",AtomProperty(Material(0x4DA6FF,50.),AtomicRadii(1.38,0.64,0.00,1.46,0.78))));
  list.append(PP("W" ,AtomProperty(Material(0x2194D6,50.),AtomicRadii(1.46,0.62,0.00,1.39,0.74))));
  list.append(PP("Re",AtomProperty(Material(0x267DAB,50.),AtomicRadii(1.59,0.56,0.00,1.37,0.77))));
  list.append(PP("Os",AtomProperty(Material(0x266696,50.),AtomicRadii(1.28,0.63,0.00,1.35,0.77))));
  list.append(PP("Ir",AtomProperty(Material(0x175487,50.),AtomicRadii(1.37,0.62,0.00,1.36,0.77))));
  list.append(PP("Pt",AtomProperty(Material(0xD0D0E0,50.),AtomicRadii(1.28,0.62,1.75,1.39,0.74))));
  list.append(PP("Au",AtomProperty(Material(0xFFD123,50.),AtomicRadii(1.44,0.85,1.66,1.44,1.51))));
  list.append(PP("Hg",AtomProperty(Material(0xB8B8D0,50.),AtomicRadii(1.49,1.02,1.55,1.51,0.83))));
	           
  list.append(PP("Tl",AtomProperty(Material(0xA6544D,50.),AtomicRadii(1.48,1.50,1.96,1.60,1.03))));
  list.append(PP("Pb",AtomProperty(Material(0x575961,50.),AtomicRadii(1.47,1.19,2.02,1.70,1.49))));
  list.append(PP("Bi",AtomProperty(Material(0x9E4FB5,50.),AtomicRadii(1.46,1.03,0.00,1.78,1.17))));
  list.append(PP("Po",AtomProperty(Material(0xAB5C00,50.),AtomicRadii(1.46,2.30,0.00,0.00,1.08))));
  list.append(PP("At",AtomProperty(Material(0x754F45,50.),AtomicRadii(1.46,2.40,0.00,0.00,0.76))));
  list.append(PP("Rn",AtomProperty(Material(0x428296,50.),AtomicRadii(1.45,0.00,0.00,0.00,1.30))));
	           
  list.append(PP("Fr",AtomProperty(Material(0x420066,50.),AtomicRadii(1.50,1.80,0.00,0.00,1.94))));
  list.append(PP("Ra",AtomProperty(Material(0x007D00,50.),AtomicRadii(1.90,1.43,0.00,0.00,1.62))));
	           
  list.append(PP("Ac",AtomProperty(Material(0x70ABFA,50.),AtomicRadii(1.88,1.12,0.00,1.90,1.26))));
	           
  list.append(PP("Th",AtomProperty(Material(0x00BAFF,50.),AtomicRadii(1.79,0.97,0.00,1.80,1.19))));
  list.append(PP("Pa",AtomProperty(Material(0x00A1FF,50.),AtomicRadii(1.61,0.78,0.00,1.64,1.09))));
  list.append(PP("U" ,AtomProperty(Material(0x008FFF,50.),AtomicRadii(1.58,0.52,1.86,1.54,0.87))));
  list.append(PP("Np",AtomProperty(Material(0x0080FF,50.),AtomicRadii(1.55,0.75,0.00,1.55,1.55))));
  list.append(PP("Pu",AtomProperty(Material(0x006BFF,50.),AtomicRadii(1.53,0.89,0.00,1.59,1.00))));
  list.append(PP("Am",AtomProperty(Material(0x545CF2,50.),AtomicRadii(1.51,0.98,0.00,1.73,1.12))));
  list.append(PP("Cm",AtomProperty(Material(0x785CE3,50.),AtomicRadii(0.99,0.97,0.00,1.74,1.11))));
  list.append(PP("Bk",AtomProperty(Material(0x8A4FE3,50.),AtomicRadii(1.54,0.95,0.00,1.70,1.10))));
  list.append(PP("Cf",AtomProperty(Material(0xA136D4,50.),AtomicRadii(1.83,0.93,0.00,1.86,1.10))));
  list.append(PP("Es",AtomProperty(Material(0xB31FD4,50.),AtomicRadii(1.50,0.93,0.00,1.86,1.10))));
  list.append(PP("Fm",AtomProperty(Material(0xB31FBA,50.),AtomicRadii(1.50,0.92,0.00,1.94,1.10))));
  list.append(PP("Md",AtomProperty(Material(0xB30DA6,50.),AtomicRadii(1.50,0.92,0.00,1.94,1.10))));
  list.append(PP("No",AtomProperty(Material(0xBD0D87,50.),AtomicRadii(1.50,0.91,0.00,1.94,1.10))));
  list.append(PP("Lr",AtomProperty(Material(0xC70066,50.),AtomicRadii(1.50,0.90,0.00,1.71,1.10))));
  list.append(PP("Rf",AtomProperty(Material(0xCC0059,50.),AtomicRadii(1.50,0.90,0.00,0.00,1.10))));

#undef PP
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomPropertyTables::AtomPropertyTables()
{
  loadSettings();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomPropertyTables::~AtomPropertyTables()
{
  list.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomPropertyTables &AtomPropertyTables::operator=(const AtomPropertyTables &other)
{
  list=other.list;
  pcurrentindex=other.pcurrentindex;

  return *this;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList AtomPropertyTables::themes()const
{
  StringList sl;
  for(int i=0;i<list.size();i++){
    sl<<list.at(i)->theme();
  }
  return sl;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropertyTables::loadSettings()
{
  list.clear();
  Settings set(Settings::atomPropSettingsFile,"Atom Properties");
  long ntheme=set.beginReadArray("Theme");
  for(int ic=0;ic<ntheme;ic++){
    set.setArrayIndex(ic);
    AtomPropertyTable *pt=new AtomPropertyTable(set.getString("Name"));
    list.append(pt);
    set.beginReadArray("properties");
    for(int i=0;i<pt->size();i++){
      set.setArrayIndex(i);
      AtomProperty &p=pt->find(set.getString("El"));
      p.material().setColor(set.getQRgb("AtomColor"));
      p.material().setSpecularColor(set.getQRgb("SpecularColor"));
      p.material().setShininess(set.getDouble("Shininess"));
      p.atomicRadii().setCovalent(set.getDouble("RCovalent"));
      p.atomicRadii().setIonic(set.getDouble("RIonic"));
      p.atomicRadii().setvdW(set.getDouble("RvdW"));
      p.atomicRadii().setMetallic(set.getDouble("RMetallic"));
      p.atomicRadii().setCrystal(set.getDouble("RCrystall"));
      p.atomicRadii().setCurrentType((AtomicRadii::AtomicRadiusType)set
				     .getInt("Rcurrenttype"));
      //myDebug()<<"i="<<i<<p.element()<<set.getInt("AtomColor");
    }
    set.endArray();

  }
  set.endArray();

  pcurrentindex=set.getInt("currentTheme");
  if(pcurrentindex<0 || pcurrentindex>=list.size())pcurrentindex=0;
  
  if(list.size()==0){
    list.append(new AtomPropertyTable());
    pcurrentindex=0;
  }
  //todo load (last) default
  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void AtomPropertyTables::saveSettings()
{
  Settings set(Settings::atomPropSettingsFile,"Atom Properties");
  set.beginWriteArray("Theme");
  for(int ic=0;ic<list.size();ic++){
    set.setArrayIndex(ic);
    set.set("Name",list.at(ic)->theme());
    const AtomPropertyTable *pt=list.at(ic);
    set.beginWriteArray("properties");
    for(int i=0;i<pt->size();i++){
      const AtomProperty &p=pt->at(i);
      set.setArrayIndex(i);
      set.set("Z",i);
      set.set("El",(*pt).element(i));
      set.set("AtomColor",p.material().color());
      set.set("SpecularColor",p.material().specularColor());
      set.set("Shininess",p.material().shininess());
      set.set("RCovalent",p.atomicRadii().covalent());
      set.set("RIonic",p.atomicRadii().ionic());
      set.set("RvdW",p.atomicRadii().vdW());
      set.set("RMetallic",p.atomicRadii().metallic());
      set.set("RCrystall",p.atomicRadii().crystal());
      set.set("Rcurrenttype",p.atomicRadii().currentType());
    }
    set.endArray();
  }
  set.endArray();

  set.set("currentTheme",(int)pcurrentindex);



}


/*EOF*/ 
