//todo: todo:optimize tetra routine
//todo: gradients/normals of plane 
//todo: speedup
//todo: clipping with a corner or edge
//todo: refinement
//todo: plasma frequ

/*************************************************************************
 * @(#)RegularMesh.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 May 2010
 *************************************************************************
 */

#include <stdio.h>
#include <QSet>
#include "fsymmetry.h"
#include "constants.h"
#include "MyMath.hh"
#include "RegularMesh.hh"
#include "TriangleMesh.hh"
#include "MeshSymmetry.hh"
#include "Boundary.hh"
#include "Polyhedron.hh"
#include "Vlevel.hh"
#include "Exception.hh"
#include "MyDebug.hh"


//  ik  ieq  ieqi itmp    ieqir
//  1   1    1     1        1
//  2   2    2     2	    2
//  3   1    5     	    1
//  4   2       	    2
//  5   5          3	    3
//  6   1		    1
//  7   2		    2
//  8   5		    3
//  9   2		    2

#define CELLS \
  const int cubcells[]={					\
    6								\
    ,0,1,3,7 ,0,2,7,3 ,0,4,5,7 ,0,5,1,7 ,0,2,6,7 ,0,6,4,7	\
    ,1,3,2,6 ,1,2,0,6 ,1,5,7,6 ,1,7,3,6 ,1,0,4,6 ,1,4,5,6	\
    ,2,0,1,5 ,2,3,5,1 ,2,6,4,5 ,2,4,0,5 ,2,3,7,5 ,2,7,6,5	\
    ,3,2,0,4 ,3,0,1,4 ,3,7,6,4 ,2,3,6,4 ,3,1,5,4 ,7,3,5,4	\
  };

#define HEXCELLS						\
  const int hexcells[]={					\
    6								\
    ,0,1,2,4 ,1,6,4,5 ,1,2,4,6 ,1,3,2,7 ,1,6,5,7 ,1,2,6,7	\
    ,0,1,2,6 ,0,4,5,6 ,1,0,5,6 ,1,3,2,6 ,3,7,6,5 ,3,1,5,6	\
    ,0,1,2,5 ,2,4,5,6 ,0,2,4,5 ,1,3,2,5 ,7,2,5,6 ,2,3,7,5	\
    ,4,5,0,6 ,5,0,2,1 ,5,6,2,0 ,7,5,6,3 ,5,2,3,1 ,5,2,6,3	\
    ,4,5,2,6 ,4,0,2,1 ,4,5,1,2 ,7,5,6,2 ,3,7,2,1 ,5,7,1,2	\
    ,4,5,1,6 ,0,6,1,2 ,4,0,6,1 ,7,5,6,1 ,6,3,1,2 ,7,6,3,1	\
  };
	 



class Corners{
public:
  Corners(long nw):pnw(nw){
    p[0]=new double[6+nw];
    p[1]=new double[6+nw];
    p[2]=new double[6+nw];
    p[3]=new double[6+nw];
  };
  ~Corners(){
    delete [] p[0];
    delete [] p[1];
    delete [] p[2];
    delete [] p[3];
  };
  long pnw;
  double a[4]; // energy, data to define iso surface
  double *p[4]; // 0..2 point of corner, 3..5 gradient of energy, 6 |grad|,
  //            7.. other data
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
RegularMesh::RegularMesh(const QString &id)
  : porigin(Vec3())
  ,pcell(Matrix3x3::unitMatrix())
  ,prcell(Matrix3x3::unitMatrix())
  ,pNx(1),pNy(1),pNz(1),pperiodic(false),phexsubdivision(false)
  ,pirred(false)
  ,ppointecalced(false),pnpoints(0),pnpointsirred(0),pnpointsirredrot(0)
  ,ppoints(0)
  ,pmicrocellscalced(false)
  ,pnmicrocells(0),pmicrocells(0)
  ,pntetras(0),ptetras(0)
  ,ieq(0),ieqrot(0),ieqi(0),ieqirot(0),ieqir(0),ieqirrot(0),pid(id)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
RegularMesh::~RegularMesh()
{
  clearMeshData();
  
  if(ppoints) delete [] ppoints;ppoints=0;pnpoints=0;pnpointsirred=0;
  pnpointsirredrot=0;
  if(pmicrocells) delete [] pmicrocells;pmicrocells=0;pnmicrocells=0;
  if(ptetras) delete [] ptetras;ptetras=0;pntetras=0;
  if(ieq) delete [] ieq;ieq=0;
  if(ieqrot) delete [] ieqrot;ieqrot=0;
  if(ieqi) delete [] ieqi;ieqi=0;
  if(ieqirot) delete [] ieqirot;ieqirot=0;
  if(ieqir) delete [] ieqir;ieqir=0;
  if(ieqirrot) delete [] ieqirrot;ieqirrot=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::clearMeshData()
{
  foreach(MeshData *md,pmeshdata){
    delete md;
  }
  pmeshdata.clear();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setSubdivisions(long Nx,long Ny,long Nz)
{
  pNx=Nx;
  pNy=Ny;
  pNz=Nz;
  ppointecalced=false;
  pmicrocellscalced=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setXSubdivision(long Nx)
{
  pNx=Nx;
  ppointecalced=false;
  pmicrocellscalced=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setYSubdivision(long Ny)
{
  pNy=Ny;
  ppointecalced=false;
  pmicrocellscalced=false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setZSubdivision(long Nz)
{
  pNz=Nz;
  ppointecalced=false;
  pmicrocellscalced=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setOrigin(const Vec3 &o)
{
  porigin=o;
  ppointecalced=false;
  pmicrocellscalced=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setPeriodic(bool p)
{
  pperiodic=p;
  ppointecalced=false;
  pmicrocellscalced=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setCell(const Matrix3x3 &m)
{
  pcell=m;
  prcell=pcell.inverseT();
  ppointecalced=false;
  pmicrocellscalced=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::setCell(const Vec3 &c0,const Vec3 &c1,const Vec3 &c2)
{
  pcell=Matrix3x3(c0,c1,c2);
  prcell=pcell.inverseT();
  ppointecalced=false;
  pmicrocellscalced=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String RegularMesh::setSymmetricMeshSubdivision(const MeshSymmetry &meshsym
						 ,long nxyz
						 ,const Vlevel &vlevel)

{
  NSF("RegularMesh::createKPoints");
  //todo: real space

  bool prot=(vlevel >= Vlevel::More );
  if(prot) printf("%s: START ====================================\n"
 		  ,nsubfunc);

  long ndxyz[3];

  bool rtg=meshsym.isRTG();
  bool hex=meshsym.isHexagonal() && !meshsym.isRTG();
  


  pcell=meshsym.convRecCell();
  pprimcell=meshsym.recCell();
  ndxyz[0]=ndxyz[1]=ndxyz[2]=2;
  
  if(rtg) pcell=pprimcell;
  
  
  if(hex) {
    ndxyz[0]=ndxyz[1]=3;
    phexsubdivision=true;
  }

  

  if(prot){
     printf("%s: pcell t1=%f %f %f\n",nsubfunc
	    ,pcell()[0],pcell()[1],pcell()[2]);
     printf("%s: pcell t2=%f %f %f\n",nsubfunc
	    ,pcell()[3],pcell()[4],pcell()[5]);
     printf("%s: pcell t3=%f %f %f\n",nsubfunc
 	   ,pcell()[6],pcell()[7],pcell()[8]);
  }


  
  prcell=pcell.inverseT();
  prprimcell=pprimcell.inverseT();

  String msg;
  
  /* edge length */
  double abc[3];
  abc[0]=norm(pcell(0));
  abc[1]=norm(pcell(1));
  abc[2]=norm(pcell(2));
  
  if(prot) 
    printf("%s: START ====================================\n",nsubfunc);

  if(prot){
    printf("%s: basic subdivision   nx0=%ld\n\n",nsubfunc,nxyz);
    printf("%s: subdivision factors ndx=%ld\n",nsubfunc,ndxyz[0]);
    printf("%s: subdivision factors ndy=%ld\n",nsubfunc,ndxyz[1]);
    printf("%s: subdivision factors ndz=%ld\n",nsubfunc,ndxyz[2]);
    printf("%s: cell length,          a=%g\n",nsubfunc,abc[0]);
    printf("%s: cell length,          b=%g\n",nsubfunc,abc[1]);
    printf("%s: cell length,          c=%g\n",nsubfunc,abc[2]);
  }
  

  
  pNx=nxyz;
  if(pNx <1) {
    pNx=1;
    msg="Interval along x-axis <1! Will take 1!";
  }
  if(pNx % ndxyz[0] != 0) {
    pNx=(pNx/ndxyz[0]+1)*ndxyz[0];
    msg=Sprintf("Interval along x-axis must be multiple of %ld!\n"
	       "I correct it!",ndxyz[0]);
  }
  double da=abc[0]/(pNx);
  pNy=MyMath::max<long>((dRound(abc[1]/da/ndxyz[1]))*ndxyz[1],ndxyz[1]);
  pNz=MyMath::max<long>((dRound(abc[2]/da/ndxyz[2]))*ndxyz[2],ndxyz[2]);
  if(prot) printf("%s:                      da=%g\n\n",nsubfunc,da);



  if(prot){
    printf("%s: subdivisions         nx=%ld\n",nsubfunc,pNx);
    printf("%s: subdivisions         ny=%ld\n",nsubfunc,pNy);
    printf("%s: subdivisions         nz=%ld\n",nsubfunc,pNz);
  }
  
  if(prot)
    printf("%s: END   ====================================\n",nsubfunc);
  


  
  if(prot) printf("%s: END   ====================================\n",nsubfunc);

  return msg;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::calcPoints(const MeshSymmetry &meshsym,bool useRotSymmetry)
{
  NSF("RegularMesh::calcPoints");
  if(ppointecalced) return;
  
  if(ppoints) delete [] ppoints; ppoints=0;

  String message="Mesh subdivision incompatible with symmetry.\n"
    " Check (manual) subdivisions. Beware of symmetry restrictions.";
  
  long Nx=pNx+1;
  long Ny=pNy+1;
  long Nz=pNz+1;
  pnpoints=Nx*Ny*Nz;
  pnpointsirred=Nx*Ny*Nz;
  pnpointsirredrot=Nx*Ny*Nz;
  
  ppoints=new Vec3[pnpoints];

  pirred=false;
  
  long ip=0;
  for(long iz=0;iz<Nz;iz++){
    for(long iy=0;iy<Ny;iy++){
      for(long ix=0;ix<Nx;ix++){
	ppoints[ip++]=pcell(0)*ix/pNx+pcell(1)*iy/pNy+pcell(2)*iz/pNz
	  +porigin;
      }
    }
  }

  if(ieq) delete [] ieq; ieq=0;
  if(ieqrot) delete [] ieqrot; ieqrot=0;
  if(ieqi) delete [] ieqi; ieqi=0;
  if(ieqirot) delete [] ieqirot; ieqirot=0;
  if(ieqir) delete [] ieqir; ieqir=0;
  if(ieqirrot) delete [] ieqirrot; ieqirrot=0;
  ieq=new long[pnpoints];
  ieqrot=new long[pnpoints];
  ieqir=new long[pnpoints];
  ieqirrot=new long[pnpoints];

  if(!pperiodic){
    ieqi=new long[pnpointsirred];
    ieqirot=new long[pnpointsirredrot];
    for(long ip=0;ip<pnpoints;ip++){
      ieq[ip]=ip;
      ieqrot[ip]=ip;
      ieqi[ip]=ip;
      ieqirot[ip]=ip;
      ieqir[ip]=ip;
      ieqirrot[ip]=ip;
    }
  }else{
    //todo: make only when reduction required e.g periodic
    //todo: supply symmetry operations needed like rotations and cells
    
    long hex=meshsym.isHexagonal();
    pirred=useRotSymmetry && meshsym.numOps()>0;
    
    //myDebug() << "pirred"<<pirred;
    /* prepare rotations+projection onto cell vectors */
    long ng=1;
    Matrix3x3 Gen[64];
    if(pirred){
      double dum[9];
      for(long ig=0;ig<meshsym.numOps();ig++){
	long it=meshsym.iTable(ig);
	//myDebug() <<"opmat "<<ig<<Matrix3x3(meshsym.opmat()+9*it).printS()();
	Gen[ig]=Matrix3x3(transbtoa(dum,meshsym.opmat()+9*it,hex))*prprimcell;
      }
      ng=meshsym.numOps();
    }
    
    Matrix3x3 C=prcell*Matrix3x3::diagonal(pNx,pNy,pNz);
    
    double tol=1.0e-6;
    
    for(long ip=0;ip<pnpoints;ip++){

      long id=pnpoints+1;
      Vec3 rt=(ppoints[ip]-porigin)*prprimcell;
      
      while(rt[0]>=1.)rt[0]-=1.;
      while(rt[1]>=1.)rt[1]-=1.;
      while(rt[2]>=1.)rt[2]-=1.;
      if(rt[0]<0)rt[0]+=1.;
      if(rt[1]<0)rt[1]+=1.;
      if(rt[2]<0)rt[2]+=1.;

      rt=(pprimcell*rt)*C;
      
      long e0=lRound(rt[0]);
      long e1=lRound(rt[1]);
      long e2=lRound(rt[2]);
      if(fabs(e0-rt[0])>tol){
	throw(Exception(message,Exception::fatalerror));
	//myDebug(,rt[0],e0);
      }
      if(fabs(e1-rt[1])>tol){
	throw(Exception(message,Exception::fatalerror));
	//myDebug("mesh subdivision incompatible round(%g)!= %ld",rt[1],e1);
      }
      if(fabs(e2-rt[2])>tol){
	throw(Exception(message,Exception::fatalerror));
	//myDebug("mesh subdivision incompatible round(%g)!= %ld",rt[2],e2);
	//exit(1);
      }
      
      e0=e0%pNx;if(e0<0){e0+=pNx;};
      e1=e1%pNy;if(e1<0){e1+=pNy;};
      e2=e2%pNz;if(e2<0){e2+=pNz;};
      
      
      long ip0=(e0+Nx*(e1+Ny*e2));
      
      id=MyMath::min<long>(id,ip0);
      if(e0<0 || e1<0 || e2<0){
	myDebug() << "e1,e2,e3<0"<<e0<<e1<<e2;
	exit(1);
      }
      if(id==pnpoints+1 || id<0){
	printf("%s: no reduction found? Sorry!!\n",nsubfunc);
	exit(1);
      }
      ieq[ip]=id;
      
      // pointgroup reduction
      if(!pirred) continue;
      
      id=pnpoints+1;
      for(long ig=0;ig<ng;ig++){
	Vec3 rt=(ppoints[ip]-porigin)*Gen[ig];
	
	while(rt[0]>=1.)rt[0]-=1.;
	while(rt[1]>=1.)rt[1]-=1.;
	while(rt[2]>=1.)rt[2]-=1.;
	if(rt[0]<0)rt[0]+=1.;
	if(rt[1]<0)rt[1]+=1.;
	if(rt[2]<0)rt[2]+=1.;
	
	rt=(pprimcell*rt)*C;
	
	long e0=lRound(rt[0]);
	long e1=lRound(rt[1]);
	long e2=lRound(rt[2]);
	if(fabs(e0-rt[0])>tol){
	  throw(Exception(message,Exception::fatalerror));
	  //myDebug("mesh subdivision incompatible round(%g)!= %ld",rt[0],e0);
	  //exit(1);
	}
	if(fabs(e1-rt[1])>tol){
	  throw(Exception(message,Exception::fatalerror));
	  //myDebug("mesh subdivision incompatible round(%g)!= %ld",rt[1],e1);
	  //exit(1);
	}
	if(fabs(e2-rt[2])>tol){
	  throw(Exception(message,Exception::fatalerror));
	  //myDebug("mesh subdivision incompatible round(%g)!= %ld",rt[2],e2);
	  //exit(1);
	}
	
	e0=e0%pNx;if(e0<0){e0+=pNx;};
	e1=e1%pNy;if(e1<0){e1+=pNy;};
	e2=e2%pNz;if(e2<0){e2+=pNz;};
	
	
	long ip0=(e0+Nx*(e1+Ny*e2));
	
	id=MyMath::min<long>(id,ip0);
	if(e0<0 || e1<0 || e2<0){
	  myDebug() << "e1,e2,e3<0"<<e0<<e1<<e2;
	  exit(1);
	}
      } /* ig */
      if(id==pnpoints+1 || id<0){
	printf("%s: no reduction found? Sorry!!\n",nsubfunc);
	exit(1);
      }
      ieqrot[ip]=id;
      //myDebug("IEQ %ld %ld",ip,ieq[ip]);
    }
    
    // unwind graph
    for(long ip=0;ip<pnpoints;ip++){ // irred -> kpoint_reps
      //myDebug("ip %ld",ip);
      long ip0=ip;
      long jp=ip;
      long kp=ieq[jp];
      QSet<long> known;
      known.insert(jp);
      while(1){
	if(kp<ip0)ip0=kp;
	//myDebug("\tip0 %ld jp %ld -> ieq[jp] %ld",ip0,jp,kp);
	if(known.contains(kp)) break;
	jp=kp;
	kp=ieq[jp];
	known.insert(jp);
      }
      foreach (long jp, known){
	ieq[jp]=ip0;
      }
    }
    // count irred k-points 
    pnpointsirred=0;
    for(long ip=0;ip<pnpoints;ip++){
      if(ieq[ip] == ip ) pnpointsirred++;
    }

    // ieqi[ipi]=ip
    ieqi=new long[pnpointsirred];
    long ipi=0;
    for(long ip=0;ip<pnpoints;ip++){
      if(ieq[ip] == ip ) ieqi[ipi++]=ip; // irred -> kpoint_reps
    }
    //  temporary for mapping
    long *itmp=new long[pnpoints];
    linit(itmp,-1,pnpoints);
    
    for(long ipi=0;ipi<pnpointsirred;ipi++){
      //myDebug("IEQI[%ld]=%ld",ipi,ieqi[ipi]);
      itmp[ieqi[ipi]]=ipi;
      //myDebug("ITMP %ld %ld %ld\n",ipi,ieqi[ipi],itmp[ieqi[ipi]]);
    }
    
    // ieqir[ip]=ipi
    for(long ip=0;ip<pnpoints;ip++){
      if(itmp[ieq[ip]]<0){
	myDebug("ip %ld ieq[ip] %ld itmp[ieq[ip]] %ld"
	       ,ip,ieq[ip],itmp[ieq[ip]]);
	exit(1);
      }
      ieqir[ip]=itmp[ieq[ip]]; // kpoint -> irred_index
      //myDebug("KK %ld %ld %ld\n",ip,ieqir[ip],ieq[ip]);
    }
    
    delete [] itmp;

    if(!pirred){
      pnpointsirredrot=pnpointsirred;
      ieqirot=new long[pnpointsirredrot];
      for(long ipi=0;ipi<pnpointsirredrot;ipi++){
	ieqirot[ipi]=ieqi[ipi];
      }
      for(long ip=0;ip<pnpoints;ip++){
	ieqirrot[ip]=ieqir[ip];
      }
    }else{
      // unwind graph
      for(long ip=0;ip<pnpoints;ip++){ // irred -> kpoint_reps
	//myDebug("ip %ld",ip);
	long ip0=ip;
	long jp=ip;
	long kp=ieqrot[jp];
	QSet<long> known;
	known.insert(jp);
	while(1){
	  if(kp<ip0)ip0=kp;
	  //myDebug("\tip0 %ld jp %ld -> ieqrot[jp] %ld",ip0,jp,kp);
	  if(known.contains(kp)) break;
	  jp=kp;
	  kp=ieqrot[jp];
	  known.insert(jp);
	}
	foreach (long jp, known){
	  ieqrot[jp]=ip0;
	}
      }
      // count irred k-points 
      pnpointsirredrot=0;
      for(long ip=0;ip<pnpoints;ip++){
	if(ieqrot[ip] == ip ) pnpointsirredrot++;
      }
    
      // ieqi[ipi]=ip
      ieqirot=new long[pnpointsirredrot];
      ipi=0;
      for(long ip=0;ip<pnpoints;ip++){
	if(ieqrot[ip] == ip ) ieqirot[ipi++]=ip; // irred -> kpoint_reps
      }
      //  temporary for mapping
      itmp=new long[pnpoints];
      linit(itmp,-1,pnpoints);
      
      for(long ipi=0;ipi<pnpointsirredrot;ipi++){
	itmp[ieqirot[ipi]]=ipi;
      }
      
      // ieqir[ip]=ipi
      for(long ip=0;ip<pnpoints;ip++){
	if(itmp[ieqrot[ip]]<0){
	  myDebug("ip %ld ieqrot[ip] %ld itmp[ieqrot[ip]] %ld"
		 ,ip,ieqrot[ip],itmp[ieqrot[ip]]);
	  exit(1);
	}
	ieqirrot[ip]=itmp[ieqrot[ip]]; // kpoint -> irred_index
      }
      
      delete [] itmp;
    }
  }  
  myDebug() << "pnpoints "<<pnpoints;
  myDebug() << "pnpointsirred "<<pnpointsirred;
  myDebug() << "pnpointsirredrot "<<pnpointsirredrot;
  ppointecalced=true;
  pmicrocellscalced=false;
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::calcMicroCells()
{
  const long kcut[]={
    0,0,0, 1,0,0, 0,1,0, 1,1,0
    ,0,0,1, 1,0,1, 0,1,1, 1,1,1
  };

  if(!ppointecalced) return;
  if(pmicrocellscalced) return;
  
  if(pmicrocells) delete [] pmicrocells; pmicrocells=0;
  if(ptetras) delete [] ptetras; ptetras=0;
  
  long Nx=pNx+1;
  long Ny=pNy+1;
  //  long Nz=pNz+1;

  CELLS;
  HEXCELLS;

  const int *cells;
  if(phexsubdivision){
    cells=hexcells;
  }else{
    cells=cubcells;
  }

  pnmicrocells=pNx*pNy*pNz;
  pmicrocells=new long[8*pnmicrocells];
    
  pntetras=cells[0]*pnmicrocells;
  ptetras=new long[4*pntetras];


  long nc=0;
  long nt=0;
  for(long iz=0;iz<pNz;iz++){
    for(long iy=0;iy<pNy;iy++){
      for(long ix=0;ix<pNx;ix++){
	
	long nc8=nc*8;

	for(long ic=0;ic<8;ic++){
	  long k0=kcut[0+3*ic]+ix;
	  long k1=kcut[1+3*ic]+iy;
	  long k2=kcut[2+3*ic]+iz;
	  pmicrocells[ic+nc8]=k0+Nx*(k1+Ny*k2);
	}
	int ici;
 	if(phexsubdivision){
	  ici=(ix+3-iy%3)%3+(iz%2)*3;
 	}else{
	  ici=(ix+0)%2+(((iy+0)%2)<<1)+(((iz+0)%2)<<2);
	  if(ici&0x4)ici=(~ici)&0x7;
	  //if(ici!=3){ici=-1;}else{ici=0;}
 	}



	if(ici>=0){//todo remove
	  ici=cells[0]*ici;
	  for(int icell=ici;icell<ici+cells[0];icell++){
	    const int *cellp=cells+4*icell+1;
	    for(int ic=0;ic<4;ic++){
	      long ip=pmicrocells[cellp[ic]+nc8];
	      ptetras[ic+4*(nt)]=ip;
	    }
	    nt++;
	  }
	}

	nc++;

      }
    }
  }
  pntetras=nt;//todo remove


  pmicrocellscalced=true;
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::printData()
{

  printf("Number of irreducible mesh points in regular box: %ld\n"
	 ,pnpointsirred);
  for(long id=0;id<pmeshdata.size();id++){
    MeshData *md=pmeshdata[id];
    

    printf("MeshData %ld\n",id);
    for(long ip=0;ip<pnpoints;ip++){
      long ipi=ieqir[ip];
      double *grad=md->gradient(ipi);
      double g0=grad[0]; if(fabs(g0)<1.0e-6) g0=0;
      double g1=grad[1]; if(fabs(g1)<1.0e-6) g1=0;
      double g2=grad[2]; if(fabs(g2)<1.0e-6) g2=0;
      
      printf("%3ld,%3ld: r=%10g %10g %10g: e=%10g "
	     "grad=%10g %10g %10g |grad|=%10g\n",ip,ipi
	     ,ppoints[ip][0]
	     ,ppoints[ip][1]
	     ,ppoints[ip][2]
	     ,md->parray[ipi]
	     ,g0,g1,g2
	     ,md->pextradata[0][ipi]
	     );
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::meshDataMinMax(long idata,double &min,double &max)const
{
  if(idata<0 || idata>=pmeshdata.size() || (pnpointsirred<1)){
    min=max=0;
    return;
  }
  MeshData *md=pmeshdata[idata];
  min=max=md->parray[0];
  for(long ipi=0;ipi<pnpointsirred;ipi++){
    min=MyMath::min<double>(md->parray[ipi],min);
    max=MyMath::max<double>(md->parray[ipi],max);
  }  
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::printPoints()
{
  printf("Number of mesh points in regular box: %ld\n",pnpoints);
  for(long ip=0;ip<pnpoints;ip++){
    printf("%ld: %10g %10g %10g\n",ip
	   ,ppoints[ip][0]
	   ,ppoints[ip][1]
	   ,ppoints[ip][2]
	   );
  }
  printf("Number of irreducible mesh points in regular box: %ld\n"
	 ,pnpointsirred);
  for(long ipi=0;ipi<pnpointsirred;ipi++){
    printf("%ld: %10g %10g %10g\n",ipi
	   ,ppoints[ieqi[ipi]][0]
	   ,ppoints[ieqi[ipi]][1]
	   ,ppoints[ieqi[ipi]][2]
	   );
  }
}

/* ***********************************************************************
 * report creation process into cbuffer
 * ***********************************************************************
 */
QString RegularMesh::reportKPoints(const Vlevel &vlevel)
{

  QString s;
  if(vlevel>=Vlevel::Info){
    /* the lattice vectors */
    s+="                     GRID PRODUCTION\n"
      "lattice vectors\n";
    for(long i=0;i<3;i++){
      s+=QString().sprintf("             a%1ld  : %-10.5f %-10.5f %-10.5f\n"
			   ,i+1,pcell(i)[0],pcell(i)[1],pcell(i)[2]);
    }
    s+="reciprocial lattice vectors (a.g=1)\n";
    for(long i=0;i<3;i++){
      s+=QString()
	.sprintf("             g%1ld  : %-10.5f %-10.5f %-10.5f\n"
		 ,i+1,prcell(i)[0],prcell(i)[1],prcell(i)[2]);
    }
    s+="\n                     SUBDIVISIONS    \n"
      "the subdivisions are related to a cell\n"
      "giving the best choice of tetrahedrons\n";
    s+=QString().sprintf("\n  |g1|= %-10.5f |g2|=%-10.5f |g3|=%-10.5f\n"
			,norm(pcell(0)),norm(pcell(1)),norm(pcell(2)));
    s+=QString().sprintf("  nx  = %-10ld ny  =%-10ld nz  =%-10ld\n"
			 ,pNx,pNy,pNz);
  }



  if(vlevel>=Vlevel::More){
    s+="\n                    Grid Points\n"
      " No.       kx          ky          kz \n" ;
    for(int i=0;i<pnpoints;i++){
       s+=QString()
	 .sprintf(" %-6d    %s\n",i,ppoints[i].printS("%-10.5f")());
     }
     s+="\n";
  }
  if(vlevel>=Vlevel::Info){
    s+=QString().sprintf("\nNumber of grid-points: %ld \n\n",pnpoints);
  }

  if(pirred){
    if(vlevel>=Vlevel::More){
      s+="\n              Symmetry Irreducible Grid Points\n"
 	" No.       kx          ky          kz \n";
      for(int i=0;i<pnpoints;i++){
 	if(ieqrot[i] == i)
 	  s+=QString()
 	    .sprintf(" %-6d    %s\n",i,ppoints[i].printS("%-10.5f")());
      }
      s+="\n";
    }
     if(vlevel>=Vlevel::Info){
       s+=QString().sprintf("\nNumber of symmetry irreducible grid-points: %ld \n\n",pnpointsirredrot);
     }
  }




  if(pperiodic){
    if(vlevel>=Vlevel::More){
      s+="\n              Translationally Irreducible Grid Points\n"
 	" No.       kx          ky          kz \n";
      for(int i=0;i<pnpoints;i++){
 	if(ieq[i] == i)
 	  s+=QString()
 	    .sprintf(" %-6d    %s\n",i,ppoints[i].printS("%-10.5f")());
      }
      s+="\n";
    }
     if(vlevel>=Vlevel::Info){
       s+=QString().sprintf("\nNumber of translationally irreducible grid-points: %ld \n\n",pnpointsirred);
     }
  }



  if(vlevel>=Vlevel::Info){
     s+="                     TETRAHEDRON PRODUCTION\n";
     s+=QString().sprintf("TET: %ld  tetrahedra\n",pntetras);
  }
  if(vlevel>=Vlevel::More){
     s+=" TETIRR:  ITET          KP1    KP2    KP3    KP4\n";
     for(int i=0;i<pntetras;i++){
       s+=QString().sprintf("        %6d      %7ld%7ld%7ld%7ld\n",i
			    ,ptetras[0+4*i],ptetras[1+4*i]
			    ,ptetras[2+4*i],ptetras[3+4*i]);
     }
  }
 
  return s;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::fillIrredFromIrredRot()
{
  if(!pirred) return;

  for(long ipi=0;ipi<pnpointsirred;ipi++){
    // Itr->F->Irot ->F ->Itr
    long ip0=ieqir[ieqirot[ieqirrot[ieqi[ipi]]]];
    for(long i=0;i<pmeshdata.size();i++){
      pmeshdata[i]->parray[ipi]=pmeshdata[i]->parray[ip0];
      for(long iw=0;iw<pmeshdata[i]->pextradata.size();iw++){
	pmeshdata[i]->pextradata[iw][ipi]=pmeshdata[i]->pextradata[iw][ip0];
      }
    }
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::appendMeshData(long n)
{
  for(long i=0;i<n;i++){
    pmeshdata.append(new MeshData(pnpointsirred));
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::calculateGradient()
{
  NSF("RegularMesh::calculateGradient");
  myDebug() << nsubfunc;


  

  for(int i=0;i<pmeshdata.size();i++){
    pmeshdata[i]->addExtraDataArray(1); //make sure "gradient norm" can be put 
    //                                    into first extra data slot
  }

  int *mult=new int[pnpointsirred];
  iinit(mult,0,pnpointsirred);


  for(long nt=0;nt<pntetras;nt++){
    for(int ic=0;ic<4;ic++){
      mult[ieqir[ptetras[ic+nt*4]]]++;
    }
  }

  for(long nt=0;nt<pntetras;nt++){
    
    long nt4=nt*4;
      
    long ip=ptetras[0+nt4];
    long ipi0=ieqir[ip];
    const Vec3 &p0=ppoints[ip];
      
    Matrix3x3 m;
    for(int ic=1;ic<4;ic++)
      m.setColumn(ic-1,ppoints[ptetras[ic+nt4]]-p0);
      
    
    Matrix3x3 gg=m.inverseT(); /* for gradient */
    
   
    for(int id=0;id<pmeshdata.size();id++){
      MeshData *md=pmeshdata[id];

      double a0=md->parray[ipi0];

      Vec3 grad;
      for(int ic=1;ic<4;ic++){
	ip=ptetras[ic+nt4];
	long ipi=ieqir[ip];
	grad+=(md->parray[ipi]-a0)*gg(ic-1);
      }

      // the gradient belongs to the tetrahedron and we put it
      // on all corners of the tetra
      for(int ic=0;ic<4;ic++){
	ip=ptetras[ic+nt4];
	long ipi=ieqir[ip];
	md->addToGradient(ipi,grad());
	md->pextradata[0][ipi]+=norm(grad);
      }
    }
    
  } // all tetras
  
  for(int id=0;id<pmeshdata.size();id++){
    MeshData *md=pmeshdata[id];
    
//     myDebug()<<"meshdata "<<id;
    for(long ipi=0;ipi<pnpointsirred;ipi++){
      double m=mult[ipi];
      double *grad=md->gradient(ipi);
      grad[0]/=m;
      grad[1]/=m;
      grad[2]/=m;
      double n=md->pextradata[0][ipi]/m;
      md->pextradata[0][ipi]=n;
//       myDebug("%3ld: %2d %4g %4g %4g :  %10.3g %10.3g %10.3g |%10.3g|"
// 	     ,ipi,mult[ipi]
// 	     ,ppoints[ieqi[ipi]][0]
// 	     ,ppoints[ieqi[ipi]][1]
// 	     ,ppoints[ieqi[ipi]][2]
// 	     ,grad[0]
// 	     ,grad[1]
// 	     ,grad[2]
// 	     ,n
// 	     );
    }
  }
  
  delete [] mult;
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class cPlanes{
public:
  cPlanes(int n){psize=n;pdata=new double[4*n];};
  ~cPlanes(){delete [] pdata;};

  inline int size()const{return psize;};

  inline double *plane(int ip)const{return pdata+4*ip;};

  void setData(int ip,const Plane &plane){
    memcpy(pdata+4*ip,plane.n()(),sizeof(double)*3);
    pdata[4*ip+3]=plane.d();
  }

  inline bool isInBoundary(const double *x,double tol)const
  {
    for(long ip=0;ip<psize;ip++){
      double *nn=pdata+ip*4;
      if(
	 x[0]*nn[0]+x[1]*nn[1]+x[2]*nn[2]-nn[3] >= tol){
	// we are out
	return false;
      }
    }
    return true;
  };

private:
  int psize;
  double *pdata;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Tria{
public:
  Tria(){myDebug() << "class Tria: default constructor called?\n"; exit(1);};
  Tria(long nw):pndata(6+nw){
    data[0]=new double[pndata];
    data[1]=new double[pndata];
    data[2]=new double[pndata];
    ncon++;
  };
  Tria(const Tria &other){
    pndata=other.pndata;
    data[0]=new double[pndata];
    data[1]=new double[pndata];
    data[2]=new double[pndata];
    memcpy(data[0],other.data[0],sizeof(double)*(pndata));
    memcpy(data[1],other.data[1],sizeof(double)*(pndata));
    memcpy(data[2],other.data[2],sizeof(double)*(pndata));
    ncopcon++;
    
  };
  const Tria &operator=(const Tria &other){
    Tria temp(other);
    pndata=other.pndata;
    MyMath::swap(this->data[0],temp.data[0]);
    MyMath::swap(this->data[1],temp.data[1]);
    MyMath::swap(this->data[2],temp.data[2]);
    nopeq++;

    return *this;
  };
  ~Tria(){
    delete [] data[0];
    delete [] data[1];
    delete [] data[2];
  }
  int pndata;
  double *data[3];
  // data[i][0..2] = point
  // data[i][3..5] = gradient
  // data[i][6]    = data
  static long ncon,ncopcon,nopeq;
};


long Tria::ncon=0;
long Tria::ncopcon=0;
long Tria::nopeq=0;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static void clipTrias(const double *plane,QList<Tria*> &trias)
{
  //myDebug()<<"clipTrias";
  double di[3];
  double d=plane[3];
  
  double size=trias.size();
  for(int it=size-1;it>=0;it--){
    Tria *tr=trias[it];
    long nw=tr->pndata;
    di[0]=(tr->data[0][0]*plane[0]+
	   tr->data[0][1]*plane[1]+
	   tr->data[0][2]*plane[2]
	   )-d;
    di[1]=(tr->data[1][0]*plane[0]+
	   tr->data[1][1]*plane[1]+
	   tr->data[1][2]*plane[2]
	   )-d;
    di[2]=(tr->data[2][0]*plane[0]+
	   tr->data[2][1]*plane[1]+
	   tr->data[2][2]*plane[2]
	   )-d;

    int i0=0;
    int i1=1;
    int i2=2;
    if(di[i1]<di[i0]) MyMath::swap(i0,i1);
    if(di[i2]<di[i0]) MyMath::swap(i0,i2);
    if(di[i2]<di[i1]) MyMath::swap(i1,i2);

    
    if(di[i2]<=0){ // biggest is inside
      ;
    }else if(di[i0]>=0.){ // smallest is outside
      //continue;
      Tria *tr=trias.takeAt(it);
      if(tr) delete tr;
    }else if(di[i0]<0 && di[i1] >=0){
      //continue;

      double *d0=tr->data[i0];
      double *d1=tr->data[i1];
      double *d2=tr->data[i2];

      double lambda=(-di[i0])/(di[i1]-di[i0]);
      for(int k=0;k<nw;k++){
	d1[k]=d0[k]+(d1[k]-d0[k])*lambda;
      }
      
      lambda=(-di[i0])/(di[i2]-di[i0]);
      for(int k=0;k<nw;k++){
	d2[k]=d0[k]+(d2[k]-d0[k])*lambda;
      }

           
    }else if(di[i1]<0 && di[i2] >=0){
      //continue;
      Tria *tr2=new Tria(*tr);

      double *d1=tr->data[i1];
      double *d2=tr->data[i2];
      
      double *d21=tr2->data[i1];
      
      
      double lambda=(-di[i1])/(di[i2]-di[i1]);
      for(int k=0;k<nw;k++){
	d2[k]=d1[k]+(d2[k]-d1[k])*lambda;
	d21[k]=d2[k];
      }
      
      
      double *d20=tr2->data[i0];
      double *d22=tr2->data[i2];
      
      lambda=(-di[i0])/(di[i2]-di[i0]);
      for(int k=0;k<nw;k++){
	d22[k]=d20[k]+(d22[k]-d20[k])*lambda;
      }
      
      trias.append(tr2);
    }
    
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline void makeTetraTriangles(TriangleMesh *triamesh,long add
			       ,const cPlanes &planes
			       ,bool clip,Corners &corners)
{
  int ic0=0;
  int ic1=1;
  int ic2=2;
  int ic3=3;
  int p=1;
  if(corners.a[ic1]<corners.a[ic0]){int i=ic0;ic0=ic1;ic1=i;p=-p;}
  if(corners.a[ic2]<corners.a[ic0]){int i=ic0;ic0=ic2;ic2=i;p=-p;}
  if(corners.a[ic3]<corners.a[ic0]){int i=ic0;ic0=ic3;ic3=i;p=-p;}
  if(corners.a[ic2]<corners.a[ic1]){int i=ic1;ic1=ic2;ic2=i;p=-p;}
  if(corners.a[ic3]<corners.a[ic1]){int i=ic1;ic1=ic3;ic3=i;p=-p;}
  if(corners.a[ic3]<corners.a[ic2]){int i=ic2;ic2=ic3;ic3=i;p=-p;}

  double a0=corners.a[ic0];
  double a3=corners.a[ic3];

  if(a0>=0 || a3<=0) return;
  
  double a1=corners.a[ic1];
  double a2=corners.a[ic2];



  double *p0=corners.p[ic0];
  double *p1=corners.p[ic1];
  double *p2=corners.p[ic2];
  double *p3=corners.p[ic3];

  
  int i1=(3-p)/2,i2=(3+p)/2;
  
  
  
  

  QList<Tria*> trias;
  
  long nw=corners.pnw+6;

  

  if(a0<0 && a1>=0) {

    //todo pointer to tr.data
    
    Tria *tr=new Tria(corners.pnw);
    double lambda1=(-a0)/(a1-a0);
    double lambda2=(-a0)/(a2-a0);
    double lambda3=(-a0)/(a3-a0);
    for(int k=0;k<nw;k++){
      tr->data[0][k]=(p1[k]-p0[k])*lambda1+p0[k];
      tr->data[i1][k]=(p2[k]-p0[k])*lambda2+p0[k];
      tr->data[i2][k]=(p3[k]-p0[k])*lambda3+p0[k];
    }
    
    trias.append(tr);


  }else if(a1<0 && a2>=0){
    
    Tria *tr1=new Tria(corners.pnw);
    Tria *tr2=new Tria(corners.pnw);


    double lambda21=(-a1)/(a2-a1);
    double lambda31=(-a1)/(a3-a1);
    double lambda20=(-a0)/(a2-a0);
    double lambda30=(-a0)/(a3-a0);
    for(int k=0;k<nw;k++){
      tr1->data[ 0][k]=(p2[k]-p1[k])*lambda21+p1[k];
      tr1->data[i1][k]=(p2[k]-p0[k])*lambda20+p0[k];
      tr1->data[i2][k]=(p3[k]-p1[k])*lambda31+p1[k];
      tr2->data[ 0][k]=tr1->data[i2][k];
      tr2->data[i1][k]=tr1->data[i1][k];
      tr2->data[i2][k]=(p3[k]-p0[k])*lambda30+p0[k];
    }

    trias.append(tr1);
    trias.append(tr2);


  }else{
    
    
    Tria *tr=new Tria(corners.pnw);
    
    double lambda1=(-a3)/(a0-a3);
    double lambda2=(-a3)/(a1-a3);
    double lambda3=(-a3)/(a2-a3);
    for(int k=0;k<nw;k++){
      tr->data[ 0][k]=(p0[k]-p3[k])*lambda1+p3[k];
      tr->data[i1][k]=(p1[k]-p3[k])*lambda2+p3[k];
      tr->data[i2][k]=(p2[k]-p3[k])*lambda3+p3[k];
    }

    trias.append(tr);
    //todo: remove qlist or make at least shallow copies
  }

  if(clip){
    //todo: each plane can produce max 2 triangles
    //make everything in c types
    for(int ip=0;ip<planes.size();ip++){
      if(trias.size()==0) break;
      clipTrias(planes.plane(ip),trias);
    }
  }

  for(int i=0;i<trias.size();i++){
    GLuint *tria=triamesh->appendTriangle(add);
    for(int k=0;k<3;k++){
      double *da=trias[i]->data[k];
      tria[k]=triamesh->appendVertex(da,da+3,da+6,add);
    }
  }
  
  

  foreach(Tria *tr,trias){
    if(tr) delete tr;
  }

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RegularMesh::createIsoSurface(TriangleMesh *triamesh
				   ,const Boundary *boundary
				   ,const QList<Plane> *clippingplanes
				   ,double isolevel,long idata,bool clip)
{
  NSF("RegularMesh::createIsoSurface");
  myDebug() << nsubfunc << "TRIA";

  if(idata<0 || idata>=pmeshdata.size()) return;

  // we make low level copies to speed it up
  const ValueArray<Plane> &bdplanes=boundary->polyhedron()->planes();
  int np=bdplanes.size();
  if(clippingplanes) np+=clippingplanes->size();

  cPlanes planes(np);
  for(int ip=0;ip<np;ip++)
    planes.setData(ip,bdplanes.at(ip));
  if(clippingplanes)
    for(int ip=0;ip<clippingplanes->size();ip++)
      planes.setData(np+ip,clippingplanes->at(ip));
  //
  double tol=1.e-8;

//   double radtol=(boundary->radius()+tol)*(boundary->radius()+tol);
//   double mid[3];
//   mid[0]=boundary->midPoint()[0];
//   mid[1]=boundary->midPoint()[1];
//   mid[2]=boundary->midPoint()[2];
  //


  int kmin[3];
  int kmax[3];
  kmin[0]=kmax[0]=0;
  kmin[1]=kmax[1]=0;
  kmin[2]=kmax[2]=0;
  const ValueArray<Vec3> &bdvertices=boundary->polyhedron()->vertices();

  if(pperiodic){
    if(bdvertices.size()>0){
      for(int j=0;j<3;j++){
	double dmin;
	double dmax;
	dmin=(bdvertices.at(0)-porigin)*pcell(j);
	dmax=dmin;
	for(int iv=0;iv<bdvertices.size();iv++){
	  double d=(bdvertices.at(iv)-porigin)*prcell(j);
	  dmin=MyMath::min<double>(d,dmin);
	  dmax=MyMath::max<double>(d,dmax);
	}
	kmin[j]=(int)(floor(dmin));
	kmax[j]=MyMath::max<long>(kmin[j],(int)(ceil (dmax-1)));
      } // all tetcell reciprocials
    }
//     myDebug(
// 	   "DIR x %d %d\n"
// 	   "DIR y %d %d\n"
// 	   "DIR z %d %d\n"
// 	   ,kmin[0],kmax[0]
// 	   ,kmin[1],kmax[1]
// 	   ,kmin[2],kmax[2]
// 	   ); 
  }

  bool pclip=clip;

  if(clip){
    triamesh->setBoundingMidPoint(boundary->midPoint());
    triamesh->setBoundingRadius(boundary->radius());
  }else{
    Matrix3x3 cc=pcell*Matrix3x3::diagonal(
					   (kmin[0]+kmax[0]+1)
					   ,(kmin[1]+kmax[1]+1)
					   ,(kmin[2]+kmax[2]+1)
					   );
    triamesh->setBoundingMidPoint(porigin+0.5*(cc(0)+cc(1)+cc(2)));
    cc=pcell*Matrix3x3::diagonal(
				 (kmax[0]-kmin[0]+1)
				 ,(kmax[1]-kmin[1]+1)
				 ,(kmax[2]-kmin[2]+1)
				 );
    double rad=MyMath::max<double>(
	 MyMath::max<double>( norm(cc(0)+cc(1)+cc(2)) ,norm(cc(0)+cc(1)-cc(2)) )
	 ,MyMath::max<double>(norm(cc(0)-cc(1)+cc(2)) ,norm(-cc(0)+cc(1)+cc(2))) );
    triamesh->setBoundingRadius(0.5*rad);
  }

  long nweights=pmeshdata[idata]->pextradata.size();

  triamesh->setNumberOfData(nweights);

  Corners corners(nweights);



  bool *inboundary=new bool[pnpoints];

  MeshData *md=pmeshdata[idata];
  
  Tria::ncon=0;
  Tria::ncopcon=0;
  Tria::nopeq=0;

  long nt4;
  for(long kz=kmin[2];kz<=kmax[2];kz++){
    for(long ky=kmin[1];ky<=kmax[1];ky++){
      for(long kx=kmin[0];kx<=kmax[0];kx++){

	Vec3 v=pcell*Vec3(kx,ky,kz);
	//myDebug() << kx<<ky<<kz<<v.printS()();


	if(pclip){
	  for(long ip=0;ip<pnpoints;ip++){
	    inboundary[ip]=planes.isInBoundary((ppoints[ip]+v)(),tol);
	  }
	}


	for(long nt=0;nt<pntetras;nt++){
	  long add=lRound((pntetras-nt)*0.15)+1;
	  nt4=nt*4;
	  
	  if(!pclip||
	     inboundary[ptetras[0+nt4]]||
	     inboundary[ptetras[1+nt4]]||
	     inboundary[ptetras[2+nt4]]||
	     inboundary[ptetras[3+nt4]]
	     ){
	    for(int ic=0;ic<4;ic++){
	      long ip=ptetras[ic+nt4];
	      long ipi=ieqir[ip];
	      corners.a[ic]=md->parray[ipi]-isolevel;
	      corners.p[ic][0]=ppoints[ip][0]+v[0];
	      corners.p[ic][1]=ppoints[ip][1]+v[1];
	      corners.p[ic][2]=ppoints[ip][2]+v[2];
	      corners.p[ic][3]=md->gradient(ipi)[0];
	      corners.p[ic][4]=md->gradient(ipi)[1];
	      corners.p[ic][5]=md->gradient(ipi)[2];
	      for(long iw=0;iw<nweights;iw++){
		corners.p[ic][6+iw]=md->pextradata[iw][ipi];
	      }
	    
	    }
	    makeTetraTriangles(triamesh,add,planes,pclip,corners);
	  }
	}
      }
    }
  }

  printf("Tria()          :%ld\n",Tria::ncon);
  printf("Tria(Tria&)     :%ld\n",Tria::ncopcon);
  printf("operator=(Tria&):%ld\n",Tria::nopeq);

  delete [] inboundary;

  triamesh->normalizeNormals();
  
  myDebug() << "TRIA nVertices "<<triamesh->nVertices() << " ndata "
	   <<triamesh->numberOfData();
  if(0){
    for(long iv=0;iv<triamesh->nVertices();iv++){
      printf("vert %ld of %ld: %g %g %g\n",iv,triamesh->nVertices()
	     ,triamesh->vertex(iv)[0]
	     ,triamesh->vertex(iv)[1]
	     ,triamesh->vertex(iv)[2]
	     );
    }
    
    for(long it=0;it<triamesh->nTriangles();it++){
      printf("tria %ld of %ld: %d %d %d\n",it,triamesh->nTriangles()
	     ,triamesh->triangle(it)[0]
	     ,triamesh->triangle(it)[1]
	     ,triamesh->triangle(it)[2]
	     );
    }
  }
  
  triamesh->setReady(true);

}

/*EOF*/ 
