/*************************************************************************
 * @(#)STDoc.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 Jul 2009
 *************************************************************************
 */
#include "Vlevel.hh"
#include "STDoc.hh"
#include "Parser.hh" 
#include "MyMath.hh" 
#include "MyDebug.hh" 
#include "Symmetry.hh" 
#include "Cmplx.hh" 
#include "Fog.hh" 
#include "RegularMesh.hh" 
#include "TriangleMesh.hh" 
#include "Except.hh" 
#include "Assorted.hh" 
#include "Sysinfo.hh" 
#include "Config.hh" 
#include "Cluster.hh" 
#include "Light.hh" 
#include "DisplayCells.hh"
#include "BondDefs.hh" 
#include "Boundary.hh" 
#include "Axes.hh" 
#include "STM.hh" 
#include "qt5compat.hh"
#include "Array.hh"
#include "Array_cpp.hh"
#include "Field.hh"
#include "IsoSurface.hh"
#include "AutoColor.hh"
#include "RepAutoGlyph.hh"
#include "RepBoundingBox.hh"
#include "RepVolume.hh"
#include "Module.hh"


using namespace XFPLO;


typedef Pair<int,const char*> PairIntCharP ;
ValueArray<PairIntCharP>  STDoc::TempData::funcnames=
  (ValueArray<PairIntCharP>()
   <<PairIntCharP(STDoc::TempData::Sphere,"Sphere")
   <<PairIntCharP(STDoc::TempData::Px,"Px")
   <<PairIntCharP(STDoc::TempData::Dz2,"Dz2")
   <<PairIntCharP(STDoc::TempData::Inter,"Inter")
   <<PairIntCharP(STDoc::TempData::Wave,"Wave")
   );







/* ***********************************************************************
 *
 * ***********************************************************************
 */
STDoc::STDoc(OGLView *oglview)
  :DocBase(oglview),symmetry(0),cluster(0),bonddefs(0),stm(0),par(0)
   
{
  setFileName("./Untitled");


  //LightThemes *lightthemes=new LightThemes();
  //  *plights=lightthemes->currentLights();
  //delete lightthemes;




  // if(0){
  //   struct timespec res;
  //   clock_getres(CLOCK_MONOTONIC,&res);
  //   printf("sec=%ld nsec=%ld\n",res.tv_sec,res.tv_nsec);
  // }
  
  if(0){
    try{

      BaseArray<double> *bx=BaseArray<double>::construct(ValueArray<long>()<<3);
      RegularArray<double> *by=
	RegularArray<double>::construct(3,
					ValueArray<double>()<<3<<4<<5,
					ValueArray<double>()<<0<<1<<0,
					4);

      bx->setSize(3);
      
      double *d=bx->data();
      for(long i=-1;++i<bx->size();){
	for(long j=-1;++j<bx->shape(0);){
	  *(d++)=(j==0?i:0);
	}
      }
      
      ProductArray<double> *pd=ProductArray<double>::construct();
      pd->addArray(bx);
      pd->addArray(by);
      
      pd->debug();
      pd->destroy(pd);

       PathArray *pa=PathArray::construct(5);
       PathArray *pb=PathArray::construct(3);
       PathArray *pc=PathArray::construct(4);
       MeshArray *ma=MeshArray::construct();
       ma->addArray(pa);
       ma->addArray(pb);
       ma->addArray(pc);
       ma->debug();

       ma->destroy(ma);

    }catch(Except &ex){
      myDebug()<<ex.msg();
    }
    exit(1);
  }


  field=new Field();
  Module::showIDCache();
  field->debug("both");


  if(1){
    tmpfielddata.NN.resize(3);
    tmpfielddata.ori.resize(3);
    
    try{
      FILE *FH=openforread("=.t");
      tmpfielddata.cell.resize(3);
      tmpfielddata.cell[0].resize(3);
      tmpfielddata.cell[1].resize(3);
      tmpfielddata.cell[2].resize(3);
      double *d;
      d=tmpfielddata.cell[0].dataRef();
      int n=fscanf(FH,"%lg%lg%lg\n",d,d+1,d+2);
      d=tmpfielddata.cell[1].dataRef();
      n=fscanf(FH,"%lg%lg%lg\n",d,d+1,d+2);
      d=tmpfielddata.cell[2].dataRef();
      n=fscanf(FH,"%lg%lg%lg\n",d,d+1,d+2);
      n=fscanf(FH,"%lg%lg%lg\n",&tmpfielddata.ori[0],&tmpfielddata.ori[1],&tmpfielddata.ori[2]);
      n=fscanf(FH,"%ld%ld%ld\n",&tmpfielddata.NN[0],&tmpfielddata.NN[1],&tmpfielddata.NN[2]);
      n=fscanf(FH,"%lg",&tmpfielddata.isovalue);
      {int e;n=fscanf(FH,"%d",&e);tmpfielddata.func=(TempData::Funcs)e;}
      n=fscanf(FH,"%lg%lg%lg%lg%lg%lg%lg%lg",tmpfielddata.dd,tmpfielddata.dd+1,
	     tmpfielddata.dd+2,tmpfielddata.dd+3,tmpfielddata.dd+4,
	     tmpfielddata.dd+5,tmpfielddata.dd+6,tmpfielddata.dd+7);
      n=n;
      fclose(FH);
      
      setField();
    }catch(...){}

  }
  
  AutoColor *autocolor=new AutoColor();
  modules<<Pair<String,Module*>(String(autocolor->id()),autocolor);
  Module::connect(field,autocolor);

  isosurface1=new IsoSurface();
  isosurface2=new IsoSurface();
  modules<<Pair<String,Module*>(isosurface1->id(),isosurface1);
  modules<<Pair<String,Module*>(isosurface2->id(),isosurface2);
  Module::connect(autocolor,isosurface1);
  Module::connect(autocolor,isosurface2);
  isosurface1->setIsoValue(XFPLO::Value(tmpfielddata.isovalue));
  isosurface2->setIsoValue(XFPLO::Value(-tmpfielddata.isovalue));
  isosurface1->setUseGradientAsNewData(true);
  isosurface2->setUseGradientAsNewData(true);

  AutoColor *autocolor1=new AutoColor();
  modules<<Pair<String,Module*>(String(autocolor1->id()),autocolor1);
  Module::connect(isosurface1,autocolor1);

  AutoColor *autocolor2=new AutoColor();
  modules<<Pair<String,Module*>(String(autocolor2->id()),autocolor2);
  Module::connect(isosurface2,autocolor2);
  
  
  //delete isosurface1;

  {
    RepAutoGlyph *rep=new RepAutoGlyph(poglview);
    fieldposautoglyphrep=rep;
    rep->mark("Positions");
    rep->setVisible(false);
    Module::connect(field,rep);
    prepresentations.append(rep);
  }

  {
    RepAutoGlyph *rep=new RepAutoGlyph(poglview);
    fieldconautoglyphrep=rep;
    rep->mark("Connections");
    rep->setVisible(false);
    Module::connect(field,rep);
    prepresentations.append(rep);
  }

  {
    RepBoundingBox *rep=new RepBoundingBox(poglview);
    fieldbbrep=rep;
    rep->setVisible(false);
    Module::connect(field,rep);
    prepresentations.append(rep);
  }
  
  if(1){
    RepAutoGlyph *rep=new RepAutoGlyph(poglview);
    isoconrep=rep;
    rep->mark("Connections");
    rep->setVisible(false);
    rep->setColor(0xff);
    Module::connect(isosurface1,rep);
    prepresentations.append(rep);
  }
  {
    RepVolume *rep=new RepVolume(poglview);
    isovolrep=rep;
    rep->setVisible(true);
    rep->setShowNormals(false);
    rep->setColor(0xff0000);
    Module::connect(autocolor1,rep);
    prepresentations.append(rep);
  }
  {
    RepVolume *rep=new RepVolume(poglview);
    rep->setVisible(true);
    rep->setShowNormals(false);
    rep->setColor(0xff0000);
    Module::connect(autocolor2,rep);
    prepresentations.append(rep);
  }
  

  if(0){
    {
    myDebug()<<"hier BaseArray ---------------------------------------------";
    BaseArray<long> *bl=BaseArray<long>::construct(ValueArray<long>()<<2);
    BaseArray<double> *bd=BaseArray<double>::construct(ValueArray<long>()<<2);
    BaseArray<Cmplx> *bc=BaseArray<Cmplx>::construct(ValueArray<long>()<<2);
    bl->setSize(6);
    bd->setSize(6);
    bc->setSize(6);
    long *dl=bl->data();
    double *dd=bd->data();
    Cmplx *dc=bc->data();
    for(long i=-1;++i<bd->size();){
      for(long j=-1;++j<bd->shape(0);){
	*(dl++)=10*i+1*j;
	*(dd++)=0.00003141*i+0.1*j;
	*(dc++)=Cmplx(i,j);
      }
    }
    bl->debug();
    bd->debug();
    bc->debug();

    myDebug()<<"amargedon ---------------------------------------------";
    bl->destroy(bl);
    bc->destroy(bc);
    bd->destroy(bd);
    myDebug()<<"hier ProductArray ---------------------------------------------";

    try{
      BaseArray<double> *bx=BaseArray<double>::construct(ValueArray<long>()<<3);
      RegularArray<double> *by=
	RegularArray<double>::construct(3,
					ValueArray<double>()<<0<<0<<0,
					ValueArray<double>()<<0<<1<<0,
					4);
      RegularArray<double> *bz=
	RegularArray<double>::construct(3,
					ValueArray<double>()<<0<<0<<0,
					ValueArray<double>()<<0<<0<<1,
					5);
      bx->setSize(3);
      
      double *d=bx->data();
      for(long i=-1;++i<bx->size();){
	for(long j=-1;++j<bx->shape(0);){
	  *(d++)=(j==0?i:0);
	}
      }
      
      ProductArray<double> *pd=ProductArray<double>::construct();
      pd->addArray(bx);
      pd->addArray(by);
      pd->addArray(bz);
      
      pd->debug();
      
      myDebug()<<"ProductArray amargedon ---------------------------------------------";
      pd->destroy(pd);

      myDebug()<<"PathArray  ---------------------------------------------";
      
      // line segments
      PathArray *pa=PathArray::construct(2);
      PathArray *pb=PathArray::construct(1);
      PathArray *pc=PathArray::construct(1);
      MeshArray *ma=MeshArray::construct();
      ma->addArray(pa);
      ma->addArray(pb);
      ma->addArray(pc);

      ma->debug();

      myDebug()<<"PathArray amargedon ---------------------------------------------";
      ma->destroy(ma);
      
    }catch(Except &ex){
      myDebug()<<"ERROR:"<<ex.msg();
      exit(1);
    }
      
    }
    
  }


  //exit(1);


  setNew();

  mesh=0;
  triamesh=0;
  
  pview.setDefaultXYZRot();
  pview.setPerspective(false);
  pview.setTheta(10.);

  //myDebug() << "theta="<<view.theta();

  ballscale=SReal("1.0");
  stickballratio=SReal("0.2");


  symmetry=new Symmetry();
  cluster=new Cluster();
  bonddefs=new BondDefs();
  stm=new STM();

  pdisplaycells->setDefault(Xf::LTDirect);




  resetflags();

  pradius=1.;

  
  pcanvas.setWidth(400);
  pcanvas.setHeight(400);




  vlevel=Vlevel::Info;
  tol=1.e-8;


  strfname="str.jpg";


  par=new Parser(1);
}
/* ********************************************i***************************
 *
 * ***********************************************************************
 */
STDoc::~STDoc()
{

  if(mesh) delete mesh ; mesh=0;
  if(triamesh) delete triamesh ; triamesh=0;
  if(field) delete field ; field=0;

  for(int i=-1;++i<modules.size();){
    if(modules.at(i).v2){
      delete modules[i].v2;
    }
  }
  
  for(int i=-1;++i<prepresentations.size();){
    if(prepresentations.at(i)){
      delete prepresentations[i];
    }
  }
  delete par;
  if(symmetry) delete symmetry;
  if(cluster) delete cluster;
  if(bonddefs) delete bonddefs;
  if(stm) delete stm;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Vec3 STDoc::dataMidPoint()
{
  //  NSF("STDoc::dataMidPoint");
  //  myDebug() << nsubfunc;

  calculateBB();

  //  printf("pmid= %lg %lg %lg \n",pmid[0],pmid[1],pmid[2]  );
  return pmid;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void commonSphere(const Vec3 &O1,double R1
				       ,const Vec3 &O2,double R2
				       ,Vec3 &O,double &R)
{
  Vec3 D=O2-O1;
  double d=norm(O2-O1);


  if(d<1.0e-10) d=0;
  
  if( d+R2 <=R1){
    //myDebug()<<"d+R2<R1";
    R=R1;
    O=O1;
    return;
  }

  if( d+R1 <=R2){
    //myDebug()<<"d+R1<R2";
    R=R2;
    O=O2;
    return;
  }

  //myDebug()<<"real case";
  D/=d;
  
  R=(d+R2+R1)*0.5;
  O=O1+D*(d+R2-R1)*0.5;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STDoc::calculateBB()
{
  //NSF("STDoc::calculateBB");
  //myDebug() << nsubfunc;

  pradius=0.;

    
  pmid=bound()->midPoint();
  

  foreach(DisplayCell *dc,pdisplaycells->cells()){
    if(dc){
      Boundary *b=dc->boundary();
      if(b->polyhedron()->properties().isVisible()){
	pradius=MyMath::max<double>(pradius,b->radius()+norm(pmid-b->midPoint()));
      }
    }
  }
 

  if(cluster->radius()>1.0e-6){
    commonSphere( pmid,pradius
		  ,cluster->midPoint(),cluster->radius()
		  ,pmid,pradius);
  }


  if(pradius<1.e-6) pradius=1.;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
double STDoc::dataRadius() 
{
  //  NSF("STDoc::dataRadius");
  //  myDebug() << nsubfunc;
  calculateBB();
  //  printf("pradius= %lg \n",pradius  );
  return pradius;
}

/* ***********************************************************************
 * reset flags
 * ***********************************************************************
 */

void  STDoc::resetflags() 
{
  symmetry->setCalculated(false);
  bound()->setCalculated(false);
  pdisplaycells->setCalculated(false);
  paxes->setCalculated(false);
  //  cluster->setChanged(true);
  allcalced = false;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STDoc::save(const QString &fname)
{
  
  try{
    saveData(fname);
  }catch(Except e){
    throw QString("%2").arg(e.msg());
  }
  modified=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STDoc::saveData(const QString &fname)
{
  FILE *FH;
  String filename(fname.toAscii().constData());
  
  /* write input file */
  FH=openforwrite(filename);
  fprintf(FH,"%s\n\n",
"########################################################################\n\
# this file is part of the FPLO bandstructure package\n\
# it was created automatically\n\
# please dont edit it, unless you really know what you are doing\n\
#\n\
########################################################################");
  /* header with version  numbers*/
  fprintf(FH,"section header{\n\n");

  fprintf(FH,"  struct {char[*] application;char[*] description;} filetype\n"
	  "  ={\"xstr\",\"FPLO-Graphics\"};\n\n");


  fprintf(FH,"  struct {char[*] mainversion;char[*] subversion;} version={");
  fprintf(FH,"\"%s\",\"%s\"};\n"
	  ,FPLOMAINVERSION()
	  ,FPLOSUBVERSION());
  fprintf(FH,"\n\n  struct {char[*] hostname;char[*] date;char[*] by;} last_changes\n    ={");
  
  fprintf(FH,"\"%s\",\"%s\",\"%s\"};\n"
	  ,Sysinfo::getHostname()(),Sysinfo::getDateAndTime()()
	  ,(EXECNAMESHORT+" ("+FPLOMAINVERSION+"-"+FPLORELEASE+")")());
  fprintf(FH,"};\n\n\n");
  
  symmetry->symPTSectionToFile(FH,true);
  
  pview.PTSectionToFile(FH);
  
  pcanvas.canvasPTSectionToFile(FH);

  pfog->PTSectionToFile(FH);

  /* old boundary stuff */
  fprintf(FH,"# for downward compatibility only we write the old boundary stuff\n");
  
  bound()->PTHeaderToFile(FH,"","=");
  bound()->PTBodyToFile(FH,"",";\n\n");

 
  /* display cells */
  pdisplaycells->PTSectionToFile(FH);

  /* bonds and polys */
  bonddefs->PTSectionToFile(FH,ballscale,stickballratio);

  paxes->axesPTSectionToFile(FH);

  plights->PTSectionToFile(FH);
  
  fclose(FH);
  
  return;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STDoc::load(const QString &filename)
{
  
  try{
    readData(filename );
  }catch (Except e){
    throw QString("%2 ").arg(e.msg());
  }


  setOld();
  /* reset flags */
  resetflags();
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void STDoc::readData(const QString &file )
{
  //  NSF("FSDoc::readData");

  String fname=file.toAscii().constData();
  
  
  if(pathExistence(fname()) == DoesNotExist)
    throw Except(String().sprintf("File \'%s\' does not exist!",fname()));
  
  //todo:: character
  par->parseFile(fname());
  
  String vm=(*par)()("version")("mainversion")();
  String vs=(*par)()("version")("subversion")();
  
  /* START read single boundary */
  Boundary boundtmp;
  boundtmp.setLattice(Xf::LTDirect);

  bool havesingleboundary=false;
  try{
    boundtmp.readPTSectionFromFile((*par)()("boundary"));
    havesingleboundary=true;
  }catch(Except){};
  /* END read single boundary */

  /* display cells */
  DisplayCells displaycellstmp;
  displaycellstmp.setDefault(Xf::LTDirect);

  bool havedisplaycells=false;
  
  try{
    Dobj root=(*par)()("displayCells");
    displaycellstmp.readPTSectionFromFile(root);
    havedisplaycells=true;
  }catch(Except){}
  /* end display cells */


  /* bonddefs */
  BondDefs bonddefstmp;
  try{
    Dobj root=(*par)()("bondDefs");
    try{
      ballscale=root("ballscale")();
      stickballratio=root("stickballratio")();
    }catch(...){
    }
    bonddefstmp.readPTSectionFromFile(root);
  }catch(Except ){}
  /* bonddefs */


  Symmetry symtmp;
  Symmetry::readSymmetryPTSectionFromFile(par,symtmp,true);

  *pdisplaycells=displaycellstmp;


  if(!havedisplaycells && havesingleboundary){
    PolyhedronProperties props=bound()->polyhedron()->properties();
    *(bound())=boundtmp;
    bound()->polyhedron()->properties()=props;
  }

  pdisplaycells->setCalculated(false);
  pdisplaycells->setChanged(false);

  *bonddefs=bonddefstmp;
  bonddefs->setChanged(false);

  *symmetry=symtmp;

  pview.readPTSectionFromFile(par);
  
  pcanvas.readCanvasPTSectionFromFile(par);


  paxes->readAxesPTSectionFromFile(par);  


  Fog fogtmp;
  fogtmp.readPTSectionFromFile((*par)());
  *pfog=fogtmp;
  
  Lights lighttmp;
  lighttmp.readPTSectionFromFile((*par)());
  *plights=lighttmp;
  

  par->cleanup();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STDoc::setField()
{
  bool addcolor=false;
  try{

    ValueArray<ValueArray<double> > cell=tmpfielddata.cell;
    ValueArray<long> &NN=tmpfielddata.NN;
    
    if(1){
      cell[0]/=(NN[0]-1);
      cell[1]/=(NN[1]-1);
      cell[2]/=(NN[2]-1);
      field->setGridD(3,NN,tmpfielddata.ori,cell);
    }else{//warped mehs for testing
      field->positions()->setupArray(ValueArray<long>()<<3);
      field->connections()->setupArray(ValueArray<long>()<<8);
      field->connections()->setConnectionType(Connections::CTYCuboid);
      BaseArray<double> *bapo=field->positions()->array();
      BaseArray<long> *baco=field->connections()->array();
      
      double theta0=20./180.*Pi;

      bapo->setSize(NN.prod());
      baco->setSize((NN[0]-1)*(NN[1]-1)*(NN[2]));
      
      double *dpos=bapo->data();
      long *dcon=baco->data();
      long index=0;

      for(int ir=-1;++ir<NN[0];){
	double r=ir*cell[0][0]/(NN[0]-1)+0.1;
	for(int it=-1;++it<NN[1];){
	  double the=(Pi-2*theta0)*it/(NN[1]-1)+theta0;
	  //myDebug()<<it<<the;

	  for(int ip=-1;++ip<NN[2];){
	    double phi=ip*twopi/(NN[2]-0);
	    //myDebug()<<ip<<phi;

	    if((ir<NN[0]-1)&&(it<NN[1]-1)&&(ip<NN[2]-1)){
	      long ind=index/3;
	      long iz=ind%(NN[2]);
	      ind=(ind-iz)/(NN[2]);
	      long iy=ind%NN[1];
	      long ix=(ind-iy)/NN[1];
	      
	      dcon[0]=(iz  )+((iy  )+(ix  )*NN[1])*NN[2];
	      dcon[1]=(iz  )+((iy+1)+(ix  )*NN[1])*NN[2];
	      dcon[2]=(iz+1)+((iy  )+(ix  )*NN[1])*NN[2];
	      dcon[3]=(iz+1)+((iy+1)+(ix  )*NN[1])*NN[2];
	      dcon[4]=(iz  )+((iy  )+(ix+1)*NN[1])*NN[2];
	      dcon[5]=(iz  )+((iy+1)+(ix+1)*NN[1])*NN[2];
	      dcon[6]=(iz+1)+((iy  )+(ix+1)*NN[1])*NN[2];
	      dcon[7]=(iz+1)+((iy+1)+(ix+1)*NN[1])*NN[2];
	      dcon+=8;

	    }else if((ir<NN[0]-1)&&(it<NN[1]-1)&&(ip==(NN[2]-1))){
	      long ind=index/3;
	      long iz=ind%(NN[2]);
	      ind=(ind-iz)/(NN[2]);
	      long iy=ind%NN[1];
	      long ix=(ind-iy)/NN[1];

	      dcon[0]=(iz  )%NN[2]+((iy  )+(ix  )*NN[1])*NN[2];
	      dcon[1]=(iz  )%NN[2]+((iy+1)+(ix  )*NN[1])*NN[2];
	      dcon[2]=(iz+1)%NN[2]+((iy  )+(ix  )*NN[1])*NN[2];
	      dcon[3]=(iz+1)%NN[2]+((iy+1)+(ix  )*NN[1])*NN[2];
	      dcon[4]=(iz  )%NN[2]+((iy  )+(ix+1)*NN[1])*NN[2];
	      dcon[5]=(iz  )%NN[2]+((iy+1)+(ix+1)*NN[1])*NN[2];
	      dcon[6]=(iz+1)%NN[2]+((iy  )+(ix+1)*NN[1])*NN[2];
	      dcon[7]=(iz+1)%NN[2]+((iy+1)+(ix+1)*NN[1])*NN[2];
	      dcon+=8;

	    }


	    dpos[index++]=r*cos(phi)*sin(the);
	    dpos[index++]=r*sin(phi)*sin(the);
	    dpos[index++]=r*cos(the);

	    

	  }
	}
      }

      field->positions()->setDirty(true);
      
      //bapo->debug();
      baco->debug();
      //      exit(0);
    }

    
    //field->positions()->array()->debug();
    //field->connections()->array()->debug();
    field->setupData<double>(ValueArray<long>());
    if(addcolor) field->setupColors(ValueArray<long>()<<4);
    {
      double *dd=tmpfielddata.dd;
      ValueArray<long> &NN=tmpfielddata.NN;
      const ValueArray<double> &ori=tmpfielddata.ori;
      const Positions *pos=field->positions();
      const BaseArray<double> *ar=pos->array();
      BaseArray<double> *da=field->data().to<double>();
      BaseArray<double> *bacol=0;
      double *dcol=0;
      if(addcolor) {
	bacol=field->colors();
	bacol->setSize(ar->size());
	bacol->valueArray()=0.;
	dcol=bacol->data();
      }
      da->setSize(ar->size());
      double *d=da->data();

      DataStatistics dstat;
      dstat.calculate<double>(ar);
      const ValueArray<double> &dmin=dstat.data<double>()->minValues;
      const ValueArray<double> &dmax=dstat.data<double>()->maxValues;
      myDebug()<< "Positions are enclosed between"
	       <<dmin<<"and"<<dmax;
      myDebug()<<"ori="<<ori;


      for(long i=-1;++i<ar->size();){
	Vec3 r(ar->point(i,0),ar->point(i,1),ar->point(i,2));
	//Vec3 r(ar->point(i,0),ar->point(i,1),0);
	double rr=MyMath::max<double>(norm(r),1.0e-12);
	double x=r[0]/rr;
	double y=r[1]/rr;
	double xp=(x+y)*0.7071;
	//	double yp=(x-y)*0.7071;
	if(addcolor) {
	  dcol[4*i+3]=1.;
	}
	switch(tmpfielddata.func){
	case TempData::Sphere:
	  d[i]=rr/4;
	  if(addcolor) {
	    dcol[4*i+0]=y;
	    dcol[4*i+2]=1.0-y;
	  }
	  break;
	case TempData::Px:
	  d[i]=(x*exp(-0.2*rr))*4;
	  if(addcolor) {
	    dcol[4*i+0]=y;
	    dcol[4*i+2]=1.0-y;
	  }
	  break;
	case TempData::Dz2:
	  d[i]=((3*xp*xp-1.)*exp(-0.2*rr))*2;
	  if(addcolor) {
	    dcol[4*i+0]=y;
	    dcol[4*i+2]=1.0-y;
	  }
	  break;
	case TempData::Wave:
	  {
	    d[i]=cos((r[0]));
	    if(addcolor) {
	      dcol[4*i+0]=y;
	      dcol[4*i+2]=1.0-y;
	    }
	    myDebug()<<r[0]<<d[i];
	    break;
	  }
	case TempData::Inter:
	default:
	  double xx=(r[0]-ori.at(0))/(cell[0][0]*(NN[0]-1));
	  double yy=(r[1]-ori.at(1))/(cell[1][1]*(NN[1]-1));
	  double zz=(r[2]-ori.at(2))/(cell[2][2]*(NN[2]-1));
	  //myDebug()<<"data pp"<<i<<xx<<yy<<zz<<(cell[0][0])*(NN[0]-1);
	  d[i]=(dd[0]*(1.-xx)*(1.-yy)+dd[1]*xx*(1.-yy)
		+dd[2]*(1.-xx)*yy +dd[3]*xx*yy)*(1.-zz)
	    +(dd[4]*(1.-xx)*(1.-yy)+dd[5]*xx*(1.-yy)
	      +dd[6]*(1.-xx)*yy +dd[7]*xx*yy)*zz ;
	  if(addcolor) {
	    dcol[4*i+0]=yy;
	    dcol[4*i+2]=1.0-yy;
	  }
	  break;
	}
	//myDebug()<<"data"<<i<<d[i]<<dd[1];
	//myDebug()<<i<<d[i];
      }
      
    }
    //field->colors()->debug();
    // field->data().to<double>()->debug();
    // exit(0);
  }catch(Except &ex){
    myDebug()<<"ERROR:"<<ex.msg();
    exit(1);
  }
}
