/* @(#)STDoc.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef STDOC_H
#define STDOC_H

#include <time.h>
#include <QString>
#include "MyQRgb.hh"
#include "DocBase.hh"
#include "OGLMaterial.hh"
#include "Vlevel.hh"
#include "SReal.hh"
#include "Pair.hh"



class Parser;
class STM;
class RegularMesh;
class OGLView;
class TriangleMesh;
namespace XFPLO{ //todo: make full usage
  class BondDefs;
  class Symmetry;
  class Representation;
  class RepAutoGlyph;
  class RepVolume;
  class RepBoundingBox;
  class Field;
  class IsoSurface;
  class Module;
  class Cluster;
}


using namespace XFPLO;

class STDoc: public DocBase
{
Q_OBJECT

public:
  STDoc(OGLView *oglview);
  ~STDoc();

  


  void resetflags();


  virtual bool showLegend(){return false;};
  virtual double legendPositionY(){ return 0.; };

  void load(const QString &filename);
  void readData(const QString &file );

  void save(const QString &fname);  
  void saveData(const QString &fname);
    
  virtual Vec3 dataMidPoint();
  virtual double dataRadius() ;
  
  void setField();


 public:

  class TempData{
  public:
    enum Funcs{Sphere=0,Px=1,Dz2=2,Inter=3,Wave=4};
    ValueArray<long> NN;
    ValueArray<ValueArray<double> > cell;
    ValueArray<double>ori;
    double dd[8];
    double isovalue;
    Funcs func;
    static ValueArray<Pair<int,const char*> > funcnames;
  };
  TempData tmpfielddata;


  long midcalced;

  Symmetry *symmetry;
  Cluster *cluster;
  BondDefs *bonddefs;
  STM *stm;


  OGLMaterial oglmat;
  
  double tol;
  Vlevel vlevel;


  SReal ballscale;
  SReal stickballratio;

  QString strfname;

  RegularMesh *mesh;
  TriangleMesh *triamesh;


  XFPLO::RepAutoGlyph *fieldposautoglyphrep;
  XFPLO::RepAutoGlyph *fieldconautoglyphrep;
  XFPLO::RepBoundingBox *fieldbbrep;
  XFPLO::Field *field;
  XFPLO::IsoSurface *isosurface1;
  XFPLO::IsoSurface *isosurface2;
  XFPLO::RepAutoGlyph *isoconrep;
  XFPLO::RepVolume *isovolrep;
  ValueArray<Pair<String,XFPLO::Module*> > modules;

  ValueArray<XFPLO::Representation *> prepresentations;
  
 private:
  void calculateBB();

 private:
  Parser *par;

  Vec3 pmid;
  double pradius;
};

#endif // STDOC_H
