/*************************************************************************
 * @(#)STM.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 Aug 2011
 *************************************************************************
 */

#include <stdio.h>
#include "Assorted.hh"
#include "FString.hh"
#include "Except.hh"
#include "constants.h"
#include "Exception.hh"
#include "PropertyTable.hh"
#include "STM.hh"
#include "Symmetry.hh"
#include "Vec3.hh"
#include "Xf.hh"
#include "MyDebug.hh"


const char *generalerrormsg=" line should contain El xyz xyz xyz!\n"
  " where xyz= [r] number or \n"
  "       xyz= El(site) + number [A/ab]\n"
  " [] arguments are optional\n"
  "    r means in relative unit cell coordinates\n"
  "    A=Ang, ab=Bohr radii (default)\n";
 
/* ***********************************************************************
 *
 * ***********************************************************************
 */
STM::STM()
  :pel("??"),lxyz(Vec3()),tau(Vec3()),pdatalengthunit(Xf::BohrRadius)
  ,pvisible(true),pcalculated(false)
{
  relative[0]=relative[1]=relative[2]=false;
  sites[0]=sites[1]=sites[2]=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STM::readCoorddinate(const String &buffer)
{
  bool prot=false;
  const int end=0;
  const int start=1;
  const int unit=3;
  const int element=4;
  const int openparen=5;
  const int site=6;
  const int closeparen=7;
  const int plus=8;
  const int anumber=9;

  StringList words=buffer.split(" \t");
  long lastwords=words.size()-1;
  long iword=1;
  String word="";
  pel=words[0];
  if(!AtomPropertyTable::allowedElements.contains(pel()))
    throw Exception(generalerrormsg);
    
  lengthunits[0]=lengthunits[1]=lengthunits[2]=Xf::Unknown;
  lxyz=0;
  sites[0]=sites[1]=sites[2]=0;
  long ixyz=-1;
  long expect=start;
  while(expect!=end){
    if(prot) printf("WHILE:%ld\n",expect);
    if(expect==start){
      ixyz++;
      if(prot) printf("START ixyz=%ld\n",ixyz);
      if(!word.length()){
	if(ixyz<=2){
	  if(iword>lastwords) throw Exception(generalerrormsg);
	  word=words[iword++];
	}else{
	  expect=end;
	  continue;
	}
      }
      if(prot) printf("START: %s\n",word());
      bool err=false;
      try{
	lxyz[ixyz]=word.convertToFortranReal();
      }catch(Except &ex){
	err=true;
      }
      if(!err){
	if(prot) printf("got number\n");
	word="";
	expect=unit;
      }else{
	if(prot) printf("complex\n");
	long j=word.scan("aA");
	if(j==-1) {
	  if(prot) printf("NO numberunit %s\n",word());
	  expect=element;
	}else{
	  if(prot) printf("must be numberunit %s\n",word.subString(0,j-1)());

	  err=false;
	  try{
	    lxyz[ixyz]=word.subString(0,j-1).convertToFortranReal();
	  }catch(Except &ex){
	    err=true;
	  }
	  if(!err){ 
	    word=word.subString(j);
	    expect=unit;
	  }else{
	    if(prot) printf("must be El(...)\n");
	    expect=element;
	  }
	}
      }
    }else if(expect==anumber){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }
      if(prot) printf("ANUMBER: %s\n",word());
      bool err=false;
      try{
	lxyz[ixyz]=word.convertToFortranReal();
      }catch(Except &ex){
	err=true;
      }
      if(err){
	long j=word.scan("aA");
	if(j<0){
	  throw Exception(generalerrormsg);
	}else{
	  bool err=false;
	  try{
	    lxyz[ixyz]=word.subString(0,j-1).convertToFortranReal();
	  }catch(Except &ex){
	    err=true;
	  }
	  if(err){
	    throw Exception(generalerrormsg);
	  }
	  word=word.subString(j);
	  expect=unit;
	}
	
      }else{
	if(prot) printf("anumber[%ld]=%g\n",ixyz,lxyz[ixyz]);
	expect=unit;
	word="";
      }
    }else if(expect==unit){
      if(!word.length()){
	if(iword>lastwords) {
	  if(ixyz<2) throw Exception(generalerrormsg);
	  expect=end;
	  continue;
	}else{
	  word=words[iword++];
	}
      }
      if(prot) printf("UNIT:%s\n",word());
      word=word.capitalize();
      if(prot) printf("UNIT:%s\n",word());
      if(word=="Ab"){
	lengthunits[ixyz]=Xf::BohrRadius;
	if(prot) printf("lengthunit[%ld]=%d\n",ixyz,lengthunits[ixyz]);
	word="";
      }else if(word=="A" || word=="Ang"){
	lengthunits[ixyz]=Xf::Angstroem;
	if(prot) printf("lengthunit[%ld]=%d\n",ixyz,lengthunits[ixyz]);
	word="";
      }
      expect=start;
    }else if(expect==element){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }

      if(prot) printf("ELEMENT:%s\n",word());
      long j=word.find("(");
      if(j<0){
	if(prot) printf("must be single element %s\n",word());
	if(word.length()<1 || word.length()>2){
	  throw Exception(generalerrormsg);
	}else{
	  String el=word.capitalize();
	  if(!AtomPropertyTable::allowedElements.contains(el())){
	    throw Exception(generalerrormsg);
	  }else{
	    word="";
	    expect=openparen;
	  }
	}
      }else{
	if(prot) printf("must be element(... %s\n",word());
	String el=word.subString(0,j-1);
	el=el.capitalize();
	if(!AtomPropertyTable::allowedElements.contains(el())){
	  throw Exception(generalerrormsg);
	}else{
	  word=word.subString(j+1);
	  expect=site;
	}
      }
    }else if(expect==openparen){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }
      if(prot) printf("OPENPAREN:%s\n",word());
      if(word[0]=='('){
	word=word.subString(1);
	expect=site;
      }else{
	throw Exception(generalerrormsg);
      }
    }else if(expect==site){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }
      if(prot) printf("SITE:%s\n",word());
      long j=word.find(")");
      if(j<0){
	bool err=false;
	try{
	  sites[ixyz]=word.convertToLong();
	}catch(Except &ex){
	  err=true;
	}
	
	if(prot) printf("must be just site %d\n",err);
	if(err){ 
	  throw Exception(generalerrormsg);
	}
	word="";
	expect=closeparen;
	if(prot) printf("expect=%ld\n",expect);
      }else{
	if(prot) printf("must be site\n");
	bool err=false;
	try{
	  sites[ixyz]=word.subString(0,j-1).convertToLong();
	}catch(Except &ex){
	  err=true;
	}
	if(err){ 
	  throw Exception(generalerrormsg);
	}
	word=word.subString(j);
	expect=closeparen;
      }
    }else if(expect==closeparen){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }
      if(prot) printf("CLOSEPAREN:%s\n",word());
      if(word[0]==')'){
	word=word.subString(1);
	expect=plus;
      }else{
	throw Exception(generalerrormsg);
      }
    }else if(expect==plus){
      if(!word.length()){
	if(iword>lastwords) throw Exception(generalerrormsg);
	word=words[iword++];
      }
      if(prot) printf("PLUS: %s\n",word());
      if(word[0]=='+'){
	word=word.subString(1);
	expect=anumber;
      }else{
	throw Exception(generalerrormsg);
      }
	  
    }
  }

  for(int ixyz=0;ixyz<3;ixyz++){
    relative[ixyz]=false;
    if(!sites[ixyz]){
      relative[ixyz]=(lengthunits[ixyz]==Xf::Unknown);
    }else{
      if(lengthunits[ixyz]==Xf::Unknown) lengthunits[ixyz]=Xf::BohrRadius;
    }
    if(prot) printf("ixyz=%d rel=%d unit=%d site=%ld\n",
		    ixyz,relative[ixyz],lengthunits[ixyz],sites[ixyz]);
  }

//   for(int ixyz=0;ixyz<3;ixyz++){
//     if(!sites[ixyz]){
//       if(!relative[ixyz]){
// 	if(lengthunits[ixyz]==Xf::Angstroem){
// 	  lxyz[ixyz]/=constant_aBtoAng;
// 	}else{
// 	}
//       }
//     }else{
//       if(lengthunits[ixyz]==Xf::Angstroem){
// 	lxyz[ixyz]/=constant_aBtoAng;
//       }
//     }
//   }  

  if(prot) {
    for(int ixyz=0;ixyz<3;ixyz++){
      printf("ixyz=%d rel=%d site=%ld xyz=%g %s\n",
	     ixyz,relative[ixyz],sites[ixyz],lxyz[ixyz]
	     ,Xf::lengthUnitName(lengthunits[ixyz]));
    }
  }
  pcalculated=false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString STM::calculate(const Symmetry *sym)
{
  double lu=1;
  if(sym->dataLengthUnit()==Xf::Angstroem) lu=1/constant_aBtoAng;
  tau=0;
  for(int ixyz=0;ixyz<3;ixyz++){
    if(relative[ixyz]){
      tau+=sym->cell()(ixyz)*lxyz[ixyz];
    }
  }

  for(int ixyz=0;ixyz<3;ixyz++){
    double lul=1;
    if(lengthunits[ixyz]==Xf::Angstroem)lul=1/constant_aBtoAng;
    double unit=lul/lu;
    if(!sites[ixyz]){
      if(!relative[ixyz]){
	tau[ixyz]=lxyz[ixyz]*unit;
      }
    }else{
      if(sites[ixyz]<1 || sites[ixyz]>sym->nsite()){
	throw Exception(Sprintf("Site %ld in =.stm is not valid in\n"
				"current symmetry\n",sites[ixyz]));
      }else{
	//myDebug()<<sym->sites().at(sites[ixyz]-1).tauD;
	tau[ixyz]=sym->sites().at(sites[ixyz]-1).tauD()[ixyz]+lxyz[ixyz]*unit;
      }
    }
  }
  pdatalengthunit=sym->dataLengthUnit();
  
  pproperty=globalproperties->current()->find(pel);

  QString out=QString("STM atom %1 at tau=%2\n").arg(pel())
    .arg(tau.printS()());
  //myDebug()<<out;
  pcalculated=true;
  return out;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STM::readSTMFile(const QString &fileName)
{
  FILE *FH;
  try{
    FH=openforread(fileName);
  }catch(Except ex) {
    throw Exception(ex.msg());  
  }
  String header;
  for(int i=0;i<3;i++){
    getLine(FH,header);
  }
  getLine(FH,header);
  fclose(FH);

  header.stripBlanks(true);
  //  myDebug() << header;
  readCoorddinate(header);
  
}



/*EOF*/ 
