/* @(#)Settings.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SETTING_H
#define _SETTING_H 1

#include <QSettings>
#include <QSize>
#include "MyQRgb.hh"
#include <QPoint>
#include "Vec3.hh"
#include "FString.hh"

class Settings
{
 public:
  Settings(const QString &file,const QString &group)
    :psettingsfile(file){
    qsettings=new QSettings(psettingsfile,QSettings::IniFormat);
    qsettings->beginGroup(group);
  };
  Settings(const QString &group)
    :psettingsfile(mainSettingsFile){
    qsettings=new QSettings(psettingsfile,QSettings::IniFormat);
    qsettings->beginGroup(group);
  };
  ~Settings(){if(qsettings){qsettings->endGroup();delete qsettings;}};


  void beginWriteArray(const QString &name){
    qsettings->beginWriteArray(name);
  };
  int beginReadArray(const QString &name){
    return qsettings->beginReadArray(name);
  };
  void endArray(){qsettings->endArray();};

  void setArrayIndex(int i){qsettings->setArrayIndex(i);};

  void set(const QString &name,int value)const{
    qsettings->setValue(name,value);
  };
  void set(const QString &name,double value)const{
    qsettings->setValue(name,value);
  };
  void set(const QString &name,const QSize &value)const{
    qsettings->setValue(name,value);
  };
  void set(const QString &name,const QPoint &value)const{
    qsettings->setValue(name,value);
  };
  void set(const QString &name,const QString &value)const{
    qsettings->setValue(name,value);
  };
  void set(const QString &name,const String &value)const{
    qsettings->setValue(name,value());
  };

  void set(const QString &name,QRgb value)const{
    qsettings->setValue(name,Sprintf("%lu",value)());
  };

  void set(const QString &name,const Vec3 &value)const{
    qsettings->setValue(name+"/0",value[0]);
    qsettings->setValue(name+"/1",value[1]);
    qsettings->setValue(name+"/2",value[2]);
  };

  void set(const QString &name,const Matrix3x3 &value)const{
    qsettings->setValue(name+"/00",value(0,0));
    qsettings->setValue(name+"/10",value(1,0));
    qsettings->setValue(name+"/20",value(2,0));
    qsettings->setValue(name+"/01",value(0,1));
    qsettings->setValue(name+"/11",value(1,1));
    qsettings->setValue(name+"/21",value(2,1));
    qsettings->setValue(name+"/02",value(0,2));
    qsettings->setValue(name+"/12",value(1,2));
    qsettings->setValue(name+"/22",value(2,2));
  };

  template <class T>
  void set(const QString &name,const ValueArray<T> &v)const{
    qsettings->setValue(name+"/size",(int)v.size());
    for(long i=-1;++i<v.size();){
      qsettings->setValue(name+QString("/%1").arg(i),v.at(i));
    }
  };


  bool getBool(const QString &name,bool defaultvalue=true)const{
    return qsettings->value(name,defaultvalue).toBool();
  };
  QRgb getQRgb(const QString &name,bool defaultvalue=0xff0000)const{
    return qsettings->value(name,defaultvalue).toUInt();
  };
  double getDouble(const QString &name,double defaultvalue=0.)const{
    return qsettings->value(name,defaultvalue).toDouble();
  };
  int getInt(const QString &name,int defaultvalue=0)const{
    return qsettings->value(name,defaultvalue).toInt();
  };
  Vec3 getVec3(const QString &name,const Vec3 &defaultvalue=Vec3())const{
    Vec3 v;
    v[0]=qsettings->value(name+"/0",defaultvalue[0]).toDouble();
    v[1]=qsettings->value(name+"/1",defaultvalue[0]).toDouble();
    v[2]=qsettings->value(name+"/2",defaultvalue[0]).toDouble();
    return v;
  };
  Matrix3x3 getMatrix3x3(const QString &name,
			 const Matrix3x3 &defaultvalue=Matrix3x3())const{
    Matrix3x3 m;
    m(0,0)=qsettings->value(name+"/00",defaultvalue(0,0)).toDouble();
    m(1,0)=qsettings->value(name+"/10",defaultvalue(1,0)).toDouble();
    m(2,0)=qsettings->value(name+"/20",defaultvalue(2,0)).toDouble();
    m(0,1)=qsettings->value(name+"/01",defaultvalue(0,1)).toDouble();
    m(1,1)=qsettings->value(name+"/11",defaultvalue(1,1)).toDouble();
    m(2,1)=qsettings->value(name+"/21",defaultvalue(2,1)).toDouble();
    m(0,2)=qsettings->value(name+"/02",defaultvalue(0,2)).toDouble();
    m(1,2)=qsettings->value(name+"/12",defaultvalue(1,2)).toDouble();
    m(2,2)=qsettings->value(name+"/22",defaultvalue(2,2)).toDouble();
    return m;
  };
  template <class T>
  ValueArray<T> getValueArray(const QString &name)const{
    ValueArray<T> v;
    long N=qsettings->value(name+"/size").toInt();
    for(long i=-1;++i<N;){
      v<<qsettings->value(name+QString("/%1").arg(i)).value<T>();
    }
    return v;
  };
  QString getString(const QString &name,const QString &defaultvalue="")const{
    return qsettings->value(name,defaultvalue).toString();
  };
  QSize getSize(const QString &name,const QSize &defaultvalue=QSize(0,0))const{
    return qsettings->value(name,defaultvalue).toSize();
  };
  QPoint getPoint(const QString &name,const QPoint &defaultvalue=QPoint(0,0))const{
    return qsettings->value(name,defaultvalue).toPoint();
  };

  static QString mainSettingsFile;
  static QString atomPropSettingsFile;
  static QString lightSettingsFile;

  static void setMainSettingsFile(const QString &file){
    mainSettingsFile=file;
  };

 private:
  QString psettingsfile;
  QSettings *qsettings;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SETTING_H */

