//todo: preserve path in monoclinic
//todo: define path separately from points
/*************************************************************************
 * @(#)Spsp.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Jun 2009
 *************************************************************************
 */

#include <stdlib.h>
#include <math.h>
#include <QFile>
#include "fsymmetry.h"
#include "Parser.hh"
#include "Assorted.hh"
#include "Symmetry.hh"
#include "Plane.hh"
#include "Boundary.hh"
#include "MyMath.hh"
#include "Vlevel.hh"
#include "MyDebug.hh"
#include "Spsp.hh"
#include "qt5compat.hh"


const double sq3o3=0.57735026918962576450;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::exportToInFile(const QString &fnam
				      ,const Symmetry *sym)const
{
  double aa=sym->latticeConstant(0);
  

  String fname(fnam);
  backUp(fnam,String(fname+".bak_xfplo"));
  
  

  Parser *par=new Parser(1);
  
  if(pathExistence(fname()) == DoesNotExist){
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  }
  par->parseFile(fname());

  long mmax=10;

  Dobj d=(*par)()("special_sympoints");

  int n=0;
  for(int i=0;i<list.size();i++){
    if(list[i]->isVisible())n++;
  }

  d.resizeSap(n);
  int j=0;
  for(int i=0;i<list.size();i++){
    SpecialPoint *sp=list[i];
    if(!sp->isVisible())continue;
    Dobj dai=d[j];
    QString s=sp->name();
    if(sp->type()==Xf::SPTJump) s="<JUMP>";
    if(sp->type()==Xf::SPTBreak) s="<BREAK>";
    dai("label")=s;//.replace('~',"$~");
    dai("kpoint")[0]=String::doubleToRatio(sp->point()[0].D*aa,mmax,1.e-5);
    dai("kpoint")[1]=String::doubleToRatio(sp->point()[1].D*aa,mmax,1.e-5);
    dai("kpoint")[2]=String::doubleToRatio(sp->point()[2].D*aa,mmax,1.e-5);
    j++;
  }


  par->writeFile(fname);

  delete par;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::importFromInFile(const QString &fnam
					,const Symmetry *sym)
{
  
  String fname(fnam);
  Parser *par=new Parser(1);
  
  if(fname.empty()) return;
  
  if(pathExistence(fname()) == DoesNotExist){
    throw Except(String().sprintf("Error reading file \'%s\'!",fname()));
  }
  par->parseFile(fname());

  Dobj d=(*par)()("lattice_constants");
  double aa=d[0].valueD();
  if(fabs(aa-sym->latticeConstant(0))>1.0e-5*sym->latticeConstant(0)){
    throw Except(String().sprintf("First lattice constant in file"
				  " \'%s\'\n different from current!"
				  ,fname()));

  }

  Dobj da=(*par)()("special_sympoints");

  list.clear();
  pchanged=true;
  for(int i=0;i<da.size();i++){
    Dobj dai=da[i];
    SReal3 s3;
    String s=dai("label")();
    Xf::SpecialPointType ty=Xf::SPTPoint;
    if(s=="<JUMP>") ty=Xf::SPTJump;
    if(s=="<BREAK>") ty=Xf::SPTBreak;

    SpecialPoint *p;
    list.append(p=new SpecialPoint(ty));
    s3[0]=dai("kpoint")[0]();
    s3[1]=dai("kpoint")[1]();
    s3[2]=dai("kpoint")[2]();
    s3/=aa;
    p->setPoint(s3);
    p->setName(s());


  }
  delete par;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::PTSectionToFile(FILE *FH)const
{
  fprintf(FH,"struct{logical visible;logical userdefined;logical showpath;\n"
	  "      struct{logical visible;\n"
	  "             struct{int type;char[*] description;}PointType;\n"
	  "             char[*] name;real point[3];real shift[3];\n"
	  "             real pointradius;char[*] pointcolor;\n"
	  "             char[*] textcolor;real fontscale;\n"
	  "      }points[*];\n"
	  "      logical usecommonproperties;\n"
	  "      real pointradius;char[*] pointcolor;\n"
	  "      char[*] textcolor;real fontscale;\n"
	  "      char[*] pathcolor;real pathlinewidth;\n"
	  "   }SpecialSymPoints={\n");
  fprintf(FH,"%c,%c,%c\n"
	  ,(pvisible?'t':'f')
	  ,(puserdefined?'t':'f')
	  ,(pshowpath?'t':'f')
	  );
  fprintf(FH,",{\n");
  for(int i=0;i<list.size();i++){
    const SpecialPoint *p=list.at(i);
    fprintf(FH,"{%c,{%d,\"%s\"},\n"
	    "   \"%s\",{%s,%s,%s},{%s,%s,%s},\n"
	    "   %s,%s,%s,%s}%s"
	    ,(p->isVisible()?'t':'f')
	    ,p->type(),Xf::specialPointTypeName(p->type())
	    ,String(p->name())()
	    ,p->point()[0].S()
	    ,p->point()[1].S()
	    ,p->point()[2].S()
	    ,p->shift()[0].S()
	    ,p->shift()[1].S()
	    ,p->shift()[2].S()
	    ,p->pointRadius().S()
	    ,Sprintf("\"%x\"",p->pointColor())()
	    ,Sprintf("\"%x\"",p->textColor())()
	    ,p->fontScale().S()
	    ,(i<list.size()-1?",\n":"\n"));  
  }
  fprintf(FH,"}\n");
  fprintf(FH,",%c,%s,%s,%s,%s,%s,%g"
	  ,(pusecommonproperties?'t':'f')
	  ,ppointradius.S()
	  ,Sprintf("\"%x\"",ppointcolor)()
	  ,Sprintf("\"%x\"",ptextcolor)()
	  ,pfontscale.S()
	  ,Sprintf("\"%x\"",ppathcolor)()
	  ,ppathlinewidth
	  );
  fprintf(FH,"};\n");
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::readPTSectionFromFile(const Dobj &root)
{
  clear();
  Dobj d=root("SpecialSymPoints");
  pvisible=d("visible").valueL();
  puserdefined=d("userdefined").valueL();
  try{
    pshowpath=d("showpath").valueL();
  }catch(Except &ex){}
  
  Dobj da=d("points");
  for(int i=0;i<da.size();i++){
    Dobj dai=da[i];
    SpecialPoint *p;
    list.append(p=new SpecialPoint());
    p->setName(dai("name")()());
    try{
      p->setVisible(dai("visible").valueL());
    }catch(Except &ex){p->setVisible(true);}
    try{
      p->setType((Xf::SpecialPointType)dai("PointType.type").valueL());
    }catch(Except &ex){p->setType(Xf::SPTPoint);}

    SReal3 s3;
    s3[0]=dai("point")[0]();
    s3[1]=dai("point")[1]();
    s3[2]=dai("point")[2]();
    p->setPoint(s3);
    s3[0]=dai("shift")[0]();
    s3[1]=dai("shift")[1]();
    s3[2]=dai("shift")[2]();
    p->setShift(s3);
    SReal s;
    s=dai("pointradius")();
    p->setPointRadius(s);
    s=dai("fontscale")();
    p->setFontScale(s);
    Dobj c=dai("pointcolor");
    if(c.isInt()){
      p->setPointColor(c.valueL());
    }else{
      p->setPointColor(c().convertToULong());
    }
    c=dai("textcolor");
    if(c.isInt()){
      p->setTextColor(c.valueL());
    }else{
      p->setTextColor(c().convertToULong());
    }

  }
  SReal s;
  try{
    setUseCommonProperties(d("usecommonproperties").valueL());
    s=d("pointradius")();
    setPointRadius(s);
    s=d("fontscale")();
    setFontScale(s);
    Dobj c=d("pointcolor");
    if(c.isInt()){
      setPointColor(c.valueL());
    }else{
      setPointColor(c().convertToULong());
    }
    c=d("textcolor");
    if(c.isInt()){
      setTextColor(c.valueL());
    }else{
      setTextColor(c().convertToULong());
    }
  }catch(Except &ex){}

  try{
    Dobj c=d("pathcolor");
    if(c.isInt()){
      setPathColor(c.valueL());
    }else{
      setPathColor(c().convertToULong());
    }
  }catch(Except &ex){setPathColor(0xaa0000);}

  try{
    ppathlinewidth=d("pathlinewidth").valueD();
  }catch(Except &ex){ppathlinewidth=3;}
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void SpecialSymPoints::simpleFileRead()
// {
//   pchanged=true;
//   clear();
//   QFile data("spsp.dat");
//   if (data.open(QFile::ReadOnly)) {
//     QTextStream stream(&data);
//     do {
//       QString n;
//       Vec3 v;
//       Vec3 shift;
//       QRgb pcol;
//       QRgb tcol;
//       double prad;
//       double fscal;
//       stream >> n;
//       myDebug() <<n << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "name missing";
// 	break;
//       }
//       stream >> v[0]>>v[1]>>v[2];
//       myDebug() <<v << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "point missing";
// 	break;
//       }
//       stream >> shift[0]>>shift[1]>>shift[2];
//       myDebug() <<shift << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "shift missing";
// 	break;
//       }
//       stream >> pcol;
//       myDebug() <<pcol << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "point color missing";
// 	break;
//       }
//       stream >> prad;
//       myDebug() <<prad << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "point radius missing";
// 	break;
//       }
//       stream >> tcol;
//       myDebug() <<tcol << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "text color missing";
// 	break;
//       }
//       stream >> fscal;
//       myDebug() <<fscal << stream.atEnd();
//       if(stream.atEnd()) {
// 	myDebug() << "text font scale missing";
// 	break;
//       }


//       list.append(SpecialPoint());
//       list.last().setName(n);
//       list.last().setPoint(v*aa);
//       list.last().setShift(shift);
//       list.last().setPointColor(pcol);
//       list.last().setPointRadius(prad);
//       list.last().setTextColor(tcol);
//       list.last().setFontScale(fscal);
//       //      myDebug() <<list.size();
      
//     }while(1);
//     //    QString line;
//     //    do {
//     //      line = stream.readLine();
//     //    } while (!line.isNull());
    
//   }else{
//   }
//   //myDebug() <<list.size();
//   myDebug() << reportSpecPoints(Vlevel::Many);
// }


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::calcPoints(const Symmetry *sym,const Vlevel &vlevel)
{
  NSF("calc_SpecialPoints");
  long prot;
  prot=(vlevel >= Vlevel::More );
  if(prot)
    printf("%s: START ====================================\n",nsubfunc); 

  pchanged=true;
  clear();
  aa=1./sym->latticeConstant(0);



  if(sym->crystal() == SYM_CUBIC ) {
    if (sym->bravais() == SYM_FACECENTERED ){
      makeFCCSpecPoints(sym->crcell());
    }else if (sym->bravais() == SYM_BODYCENTERED ){
      makeBCCSpecPoints(sym->crcell());
    }else if (sym->bravais() == SYM_SIMPLE ){
      makeSCSpecPoints(sym->rcell());
    }
  }else if(sym->crystal() == SYM_TETRAGONAL ) {
    if (sym->bravais() == SYM_SIMPLE ){
      makeSTSpecPoints(sym->rcell());
    }else if (sym->bravais() == SYM_BODYCENTERED ){
      makeBCTSpecPoints(sym->crcell());
    }
  }else if( (sym->bravais() == SYM_SIMPLE ) && 
 	    ((sym->crystal() == SYM_HEXAGONAL)||
 	     (sym->crystal() == SYM_TRIGONAL))){
    makeSHSpecPoints(sym->rcell());
  }else if((sym->bravais() == SYM_RHOMBOHEDRAL)&&
 	   (sym->crystal() == SYM_TRIGONAL)){
    makeRTGSpecPoints(sym->crcell());

  }else if(sym->crystal() == SYM_ORTHORHOMBIC ) {
    if (sym->bravais() == SYM_SIMPLE ){
      makeSOSpecPoints(sym->rcell());
    }else if ( (sym->bravais() == SYM_CBASECENTERED )||
	       (sym->bravais() == SYM_ABASECENTERED )
  	       ) {
      makeABCCOSpecPoints(sym->crcell(),sym->bravais()); 
    }else if (sym->bravais() == SYM_BODYCENTERED ){
      makeBCOSpecPoints(sym->crcell());
    }else if (sym->bravais() == SYM_FACECENTERED ){
      makeFCOSpecPoints(sym->crcell());
    }
  }else if(sym->crystal() == SYM_MONOCLINIC ) {
//     if (sym->bravais() == SYM_SIMPLE ){
//       makeSMSpecPoints(sym->rcell());
//     }else if (sym->bravais() == SYM_CBASECENTERED ){
//       makeCFCMSpecPoints(sym->crcell(),sym->rcell());
//     }
  }else if(sym->crystal() == SYM_TRICLINIC ) {
  }


  
  if(prot)
    printf("%s: END   ====================================\n",nsubfunc); 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString SpecialSymPoints::reportSpecPoints(const Vlevel &vlevel)
{
  QString s;

  if(vlevel >= Vlevel::Info ){
    s+="\n\n                     SPECIAL SYMMETRY POINTS \n\n";
    for(int i=0;i<list.size();i++){
      SpecialPoint *sp=list[i];
      s+=QString().sprintf("point %3d: \"%-10s\" %-10g %-10g %-10g\n"
			   ,i+1,sp->name().toAscii().constData()
			   ,sp->point()[0].D /aa ,sp->point()[1].D /aa 
			   ,sp->point()[2].D /aa );

    }
    s+="\n\n";
  }
  return s;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeSCSpecPoints(const Matrix3x3 &rcell)
{

  Matrix3x3 Gc=rcell.transposed();

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
	      
	
  Vec3 k;
  
  /* X point*/
  k=Vec3(0,1./2,0)*Gc;
  list.append(new SpecialPoint("X" ,k));
      
  /* M point*/
  k=Vec3(0.5,0.5,0)*Gc;
  list.append(new SpecialPoint("M" ,k));
  
  /* R point*/
  k=Vec3(0.5,0.5,0.5)*Gc;
  list.append(new SpecialPoint("R" ,k));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeFCCSpecPoints(const Matrix3x3 &crcell)
{
  Matrix3x3 Gc=crcell.transposed();

	      
	
  Vec3 k;

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0,0,0,0.5,-0.5,1.));
  

  /* X point*/
  k=Vec3(0,0.5,0)*Gc;
  list.append(new SpecialPoint("X" ,k));

  /* L point*/
  k=Vec3(0.25,0.25,0.25)*Gc;
  list.append(new SpecialPoint("L" ,k));

  /* W point*/
  k=Vec3(0.25,0.5,0)*Gc;
  list.append(new SpecialPoint("W" ,k));


  /* K point*/
  k=Vec3(0.75/2,0.75/2,0)*Gc;
  list.append(new SpecialPoint("K" ,k));


  /* U point*/
  k=Vec3(0.125,0.5,0.125)*Gc;
  list.append(new SpecialPoint("U" ,k));







}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeBCCSpecPoints(const Matrix3x3 &crcell)
{


  Matrix3x3 Gc=crcell.transposed();

	      
	
  Vec3 k;

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0,0,0,0.5,-0.5,1.));
  

  /* N point*/
  k=Vec3(0.25,0.25,0)*Gc;
  list.append(new SpecialPoint("N" ,k));

  /* H point*/
  k=Vec3(0,0.5,0)*Gc;
  list.append(new SpecialPoint("H" ,k));
      

  /* P point*/
  k=Vec3(0.25,0.25,0.25)*Gc;
  list.append(new SpecialPoint("P" ,k));
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeSTSpecPoints(const Matrix3x3 &rcell)
{

  Matrix3x3 Gc=rcell.transposed();

	      
	
  Vec3 k;

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0,0,0,0.5,-0.5,1.));
  

  /* X point*/
  k=Vec3(0,0.5,0)*Gc;
  list.append(new SpecialPoint("X" ,k));


  /* M point*/
  k=Vec3(0.5,0.5,0)*Gc;
  list.append(new SpecialPoint("M" ,k));


  /* Z point*/
  k=Vec3(0,0,0.5)*Gc;
  list.append(new SpecialPoint("Z" ,k));


  /* R point*/
  k=Vec3(0,0.5,0.5)*Gc;
  list.append(new SpecialPoint("R" ,k));
  
      
  /* A point*/
  k=Vec3(0.5,0.5,0.5)*Gc;
  list.append(new SpecialPoint("A" ,k));
  
      
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeBCTSpecPoints(const Matrix3x3 &crcell)
{

  Matrix3x3 Gc=crcell.transposed();
  Matrix3x3 ScI=Gc*crcell;
  double a2=4/ScI(0,0);
  double c2=4/ScI(2,2);
	      

	
  Vec3 k;

  


  if(a2/c2 > 1.){
    
    double Sp=1./4+c2/a2/4;
    double Sm=1./4-c2/a2/4;

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0,0,0,0.5,-0.5,1.));
    
    /* X point*/
    k=Vec3(0.25,0.25,0)*Gc;
    list.append(new SpecialPoint("X" ,k));

    /* P point*/
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("P" ,k));

    /* N point*/
    k=Vec3(0.25,0,0.25)*Gc;
    list.append(new SpecialPoint("N" ,k));


    /* Z1 point*/
    k=Vec3(0,0,Sp)*Gc;
    list.append(new SpecialPoint("Z1" ,k));

    /* Z0 point*/
    k=Vec3(0.5,0,Sm)*Gc;
    list.append(new SpecialPoint("Z0" ,k));

    /* M point*/
    k=Vec3(0.5,0,0)*Gc;
    list.append(new SpecialPoint("M" ,k));

    
  }else{

    double S0=1./4+a2/c2/4;
    double S=1./4-a2/c2/4;

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0,0,0,0.5,-0.5,1.));
    
    /* X point*/
    k=Vec3(0.25,0.25,0)*Gc;
    list.append(new SpecialPoint("X" ,k));


    /* P point*/
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("P" ,k));

    /* M point*/
    k=Vec3(0,0,0.5)*Gc;
    list.append(new SpecialPoint("M" ,k));

    /* N point*/
    k=Vec3(0.25,0,0.25)*Gc;
    list.append(new SpecialPoint("N" ,k));

    /* S0 point*/
    k=Vec3(S0,0,0)*Gc;
    list.append(new SpecialPoint("S0" ,k));

    /* S point*/
    k=Vec3(S,0,0.5)*Gc;
    list.append(new SpecialPoint("S" ,k));

    /* G point*/
    k=Vec3(S,S,0.5)*Gc;
    list.append(new SpecialPoint("G" ,k));

    /* R point*/
    k=Vec3(S0,S,0)*Gc;
    list.append(new SpecialPoint("R" ,k));

    
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeSHSpecPoints(const Matrix3x3 &rcell)
{

  Matrix3x3 Gc=rcell.transposed();
  //  Matrix3x3 ScI=Gc*rcell;

  Vec3 k;


  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));


  /* K point*/
  k=Vec3(1./3,1./3,0)*Gc;
  list.append(new SpecialPoint("K",k));
  
  
  /* M point*/
  k=Vec3(1./2,0,0)*Gc;
  list.append(new SpecialPoint("M",k));
  
  /* A point*/
  k=Vec3(0,0,1./2)*Gc;
  list.append(new SpecialPoint("A",k));

  /* L point*/
  k=Vec3(1./2,0,1./2)*Gc;
  list.append(new SpecialPoint("L",k));
  
  /* H point*/
  k=Vec3(1./3,1./3,1./2)*Gc;
  list.append(new SpecialPoint("H",k));
  
  


  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::makeRTGSpecPoints(const Matrix3x3 crcell)
{

  Matrix3x3 Gc=crcell.transposed();
  Matrix3x3 ScI=Gc*crcell;
  double Ah2=4/ScI(0,0);
  double Ch2=9/ScI(2,2);
  double Ar2=Ah2/3+Ch2/9;

  double cosa=1-Ah2/Ar2/2;
  


  Vec3 k;

  //
 
  if(cosa<0.){
    

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

    /* T point*/
    k=Vec3(1./3,-1./3,1./6)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* L point*/
    k=Vec3(1./3,1./6,1./6)*Gc;
    list.append(new SpecialPoint("L" ,k));

    /* F point*/
    k=Vec3(1./2,0,0)*Gc;
    list.append(new SpecialPoint("F" ,k));

    /* R0 point*/
    double R0=(3*Ah2-4*Ch2)/Ah2/18;
    k=Vec3(2./3,1./3,R0)*Gc;
    list.append(new SpecialPoint("R0" ,k));

    /* P0 point*/
    double P0=(4*Ch2-3*Ah2)/Ah2/18;
    k=Vec3(1./3,-1./3,P0)*Gc;
    list.append(new SpecialPoint("P0" ,k));

    
    
    
   }else{

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));


    /* T point*/
    k=Vec3(0,0,1./2)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* L point*/
    k=Vec3(1./3,1./6,1./6)*Gc;
    list.append(new SpecialPoint("L" ,k));

    /* F point*/
    k=Vec3(1./6,-1./6,1./3)*Gc;
    list.append(new SpecialPoint("F" ,k));

    /* X point*/
    k=Vec3(1./3,1./6,-1./3)*Gc;
    list.append(new SpecialPoint("X" ,k));


    /* Q0 point*/
    double Q0x=1./3-Ah2/Ch2/4;
    double Q0y=-Ah2/Ch2/4;
    k=Vec3(Q0x,Q0y,1./3)*Gc;
    list.append(new SpecialPoint("Q0" ,k));

    /* V0 point*/
    double V0y=1./3-Ah2/Ch2/4;
    k=Vec3(1./3,V0y,-1./3)*Gc;
    list.append(new SpecialPoint("V0" ,k));

    /* S0 point*/
    double S0=1./3+Ah2/Ch2/4;
    k=Vec3(S0,0,0)*Gc;
    list.append(new SpecialPoint("S0" ,k));


    /* H0 point*/
    double H0=-Ah2/Ch2/2;
    k=Vec3(1./3,H0,1./6)*Gc;
    list.append(new SpecialPoint("H0" ,k));


    /* M2 point*/
    double M2x=1./6-Ah2/Ch2/4;
    double M2y=-1./6+Ah2/Ch2/4;
    k=Vec3(M2x,M2y,1./2)*Gc;
    list.append(new SpecialPoint("M2" ,k));

    /* M0 point*/
    double M0=1./6+Ah2/Ch2/4;
    k=Vec3(M0,-M0,1./6)*Gc;
    list.append(new SpecialPoint("M0" ,k));

    /* M4 point*/
    double M4=1./3-Ah2/Ch2/2;
    k=Vec3(M4,M4/2,1./2)*Gc;
    list.append(new SpecialPoint("M4" ,k));



   }
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */


void SpecialSymPoints::makeFCOSpecPoints(const Matrix3x3 crcell)
{
  
  Matrix3x3 Gc=crcell.transposed();
  Matrix3x3 ScI=Gc*crcell;
  double a2=ScI(0,0);
  double b2=ScI(1,1);
  double c2=ScI(2,2);


  const int xtype=1;
  const int ytype=2;
  const int ztype=3;
  const int allplanes=4;
  int type=0;

  double da=a2-b2-c2;
  double db=b2-a2-c2;
  double dc=c2-a2-b2;
  

  if(da >= 0){
    type=xtype;
  }else if(db >= 0){
    type=ytype;
  }else if(dc >=0){
    type=ztype;
  }else{
    type=allplanes;
  }

  // Y = (0 1/2 0 ) ~  (1/2 0 1/2 )
  // Z = (0 0  1/2) ~  (1/2 1/2 0 )
  // T = (0 1/2 1/2) ~  (1/2 0  0 )
  // L = (/14 1/4 1/4)
  // Sigma  = (u 0 0 ) ~ U=(u 1/2 1/2)
  // Delta  = (0 u 0 ) ~ P=(1/2 u 1/2)
  // Lambda = (0 0 u ) ~ Q=(1/2 1/2 u)
  // A      = (u 0 1/2 ) ~ C=(u 1/2 0)
  // B      = (0 u 1/2 ) ~ D=(1/2 u 0)
  // G      = (1/2 0 u ) ~ H=(0 1/2 u)


  Vec3 k;
  if(type==xtype){
    
    double u0=da/4/a2;
    double sig0=0.5-u0;
    double A0=-dc/4/a2;
    double C0=0.5-A0;

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

    /* L point */
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("L" ,k));


    /* Y  point*/
    k=Vec3(0,0.5,0)*Gc;
    list.append(new SpecialPoint("Y" ,k));

    /* Z  point*/
    k=Vec3(0,0,0.5)*Gc;
    list.append(new SpecialPoint("Z" ,k));

    /* T  point*/
    k=Vec3(0,0.5,0.5)*Gc;
    list.append(new SpecialPoint("T" ,k));


    /* U0  point*/
    k=Vec3(u0,0.5,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("U0" ,k));

    /* Sig0  point*/
    k=Vec3(sig0,0,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("$~S0" ,k));

    /* A0  point*/
    k=Vec3(A0,0,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("A0" ,k));

    /* C0  point*/
    k=Vec3(C0,0.5,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("C0" ,k));

  }else if(type==ytype){

    double p0=db/4/b2;
    double delta0=0.5-p0;

    double D0=-da/4/b2;
    double B0=0.5-D0;


    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

    /* L point */
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("L" ,k));

    /* Y  point*/
    k=Vec3(0.5,0,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("Y" ,k));

    /* Z  point*/
    k=Vec3(0,0,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("Z" ,k));

    /* T  point*/
    k=Vec3(0.5,0,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("T" ,k));

    /* B0  point*/
    k=Vec3(0,B0,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("B0" ,k));

    /* D0  point*/
    k=Vec3(0.5,D0,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("D0" ,k));

    /* Delta0  point*/
    k=Vec3(0,delta0,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("$~D0" ,k));

    /* P0  point*/
    k=Vec3(0.5,p0,0.5)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("P0" ,k));

  }else if(type==ztype){
    

    double q0=dc/4/c2;
    double lam0=0.5-q0;
    double G0=-da/4/c2;
    double H0=0.5-G0;

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

    /* L point */
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("L" ,k));

    /* Y  point*/
    k=Vec3(0,0.5,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("Y" ,k));

    /* Z  point*/
    k=Vec3(0.5,0.5,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("Z" ,k));

    /* T  point*/
    k=Vec3(0.5,0,0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("T" ,k));

    /* Q0  point*/
    k=Vec3(0.5,0.5,q0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("Q0" ,k));


    /* Lambda0  point*/
    k=Vec3(0,0,lam0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("$~L0" ,k));


    /* H0  point*/
    k=Vec3(0,0.5,H0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("H0" ,k));

    /* G0  point*/
    k=Vec3(0.5,0,G0)*Gc;
    //correctSMSpecPoint(pla,k);
    list.append(new SpecialPoint("G0" ,k));

  }else{

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

    /* L point */
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("L" ,k));

    /* Y  point*/
    k=Vec3(0,0.5,0)*Gc;
    list.append(new SpecialPoint("Y" ,k));

    /* Z  point*/
    k=Vec3(0,0,0.5)*Gc;
    list.append(new SpecialPoint("Z" ,k));

    /* T  point*/
    k=Vec3(0.5,0,0)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* D0  point*/
    double d0=-da/4/b2;
    k=Vec3(0.5,d0,0)*Gc;
    list.append(new SpecialPoint("D0" ,k));

    /* G0  point*/
    double g0=-da/4/c2;
    k=Vec3(0.5,0,g0)*Gc;
    list.append(new SpecialPoint("G0" ,k));

    /* A0  point*/
    double a0=-dc/4/a2;
    k=Vec3(a0,0,0.5)*Gc;
    list.append(new SpecialPoint("A0" ,k));

    /* B0  point*/
    double b0=-dc/4/b2;
    k=Vec3(0,b0,0.5)*Gc;
    list.append(new SpecialPoint("B0" ,k));

    /* H0  point*/
    double h0=-db/4/c2;
    k=Vec3(0,0.5,h0)*Gc;
    list.append(new SpecialPoint("H0" ,k));

    /* C0  point*/
    double c0=-db/4/a2;
    k=Vec3(c0,0.5,0)*Gc;
    list.append(new SpecialPoint("C0" ,k));

  }

  return;

  

  

 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */


void SpecialSymPoints::makeBCOSpecPoints(const Matrix3x3 &crcell)
{

  Matrix3x3 Gc=crcell.transposed();
  Matrix3x3 ScI=Gc*crcell;
  double a2=ScI(0,0);
  double b2=ScI(1,1);
  double c2=ScI(2,2);

  double da=(b2+c2)/4;
  double db=(a2+c2)/4;
  double dc=(a2+b2)/4;

  Vec3 k;
  
  // X=(0   1/2  0)  ~ (1/2 0 0) ~ (0 0 1/2)
  // S=(0   1/4 1/4)
  // R=(1/4  0  1/4)
  // T=(1/4 1/4  0 )
  // W=(1/4 1/4 1/4)
  // Sigma=(u 0 0) ~ F=(u 1/2 0) ~ F=(u 0 1/2)
  // Delta=(0 u 0) ~ U=(0 u 1/2) ~ U=(1/2 u 0)
  // Lamda=(0 0 u) ~ G=(0 1/2 u) ~ G=(1/2 0 u)
   
  if(a2<=b2 && a2<=c2){

    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
    /* X  point*/
    k=Vec3(0.5,0,0)*Gc;
    list.append(new SpecialPoint("X" ,k));

    /* T  point*/
    k=Vec3(0.25,0.25,0)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* S  point*/
    k=Vec3(0,0.25,0.25)*Gc;
    list.append(new SpecialPoint("S" ,k));

   /* R  point*/
    k=Vec3(0.25,0,0.25)*Gc;
    list.append(new SpecialPoint("R" ,k));

    /* W  point*/
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("W" ,k));

    /* G0  point*/
    double G0=(db-a2/2)/c2;
    k=Vec3(0.5,0,G0)*Gc;
    list.append(new SpecialPoint("G0" ,k));

    /* U0  point*/
    double U0=(dc-a2/2)/b2;
    k=Vec3(0.5,U0,0)*Gc;
    list.append(new SpecialPoint("U0" ,k));

    /* Y0  point*/
    double Y0=dc/b2;
    k=Vec3(0,Y0,0)*Gc;
    list.append(new SpecialPoint("Y0" ,k));

    /* Lambda0  point*/
    double lam0=db/c2;
    k=Vec3(0,0,lam0)*Gc;
    list.append(new SpecialPoint("$~L0" ,k));

    /* ?0  point*/
    double q0=dc/b2;
    double q0z=(da-dc)/c2;
    k=Vec3(0,q0,q0z)*Gc;
    list.append(new SpecialPoint("?0" ,k));

    /* ?1  point*/
    double q1z=db/c2;
    double q1y=(da-db)/b2;
    k=Vec3(0,q1y,q1z)*Gc;
    list.append(new SpecialPoint("?1" ,k));

  }else if(b2<=a2 && b2<=c2){
    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
    /* X  point*/
    k=Vec3(0,0.5,0)*Gc;
    list.append(new SpecialPoint("X" ,k));

    /* T  point*/
    k=Vec3(0.25,0.25,0)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* S  point*/
    k=Vec3(0,0.25,0.25)*Gc;
    list.append(new SpecialPoint("S" ,k));

   /* R  point*/
    k=Vec3(0.25,0,0.25)*Gc;
    list.append(new SpecialPoint("R" ,k));

    /* W  point*/
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("W" ,k));

    /* Sigma0  point*/
    double sig0=dc/a2;
    k=Vec3(sig0,0,0)*Gc;
    list.append(new SpecialPoint("$~S0" ,k));


    /* H0  point*/
    double H0=(db-dc)/c2;
    k=Vec3(sig0,0,H0)*Gc;
    list.append(new SpecialPoint("H0" ,k));

    /* Lambda0  point*/
    double lam0=da/c2;
    k=Vec3(0,0,lam0)*Gc;
    list.append(new SpecialPoint("$~L0" ,k));

    /* V0  point*/
    double V0=(db-da)/a2;
    k=Vec3(V0,0,lam0)*Gc;
    list.append(new SpecialPoint("V0" ,k));

    /* F0  point*/
    double F0=(dc-b2/2)/a2;
    k=Vec3(F0,0.5,0)*Gc;
    list.append(new SpecialPoint("F0" ,k));

    /* G0  point*/
    double G0=(da-b2/2)/c2;
    k=Vec3(0,0.5,G0)*Gc;
    list.append(new SpecialPoint("G0" ,k));

  }else if(c2<=a2 && c2<=b2){
    /* Gamma  point*/
    list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
    /* X  point*/
    k=Vec3(0,0,0.5)*Gc;
    list.append(new SpecialPoint("X" ,k));

    /* T  point*/
    k=Vec3(0.25,0.25,0)*Gc;
    list.append(new SpecialPoint("T" ,k));

    /* S  point*/
    k=Vec3(0,0.25,0.25)*Gc;
    list.append(new SpecialPoint("S" ,k));

   /* R  point*/
    k=Vec3(0.25,0,0.25)*Gc;
    list.append(new SpecialPoint("R" ,k));

    /* W  point*/
    k=Vec3(0.25,0.25,0.25)*Gc;
    list.append(new SpecialPoint("W" ,k));

    /* Sigma0  point*/
    double sig0=db/a2;
    k=Vec3(sig0,0,0)*Gc;
    list.append(new SpecialPoint("$~S0" ,k));

    /* F0  point*/
    double F0=(db-c2/2)/a2;
    k=Vec3(F0,0,0.5)*Gc;
    list.append(new SpecialPoint("F0" ,k));

    /* U0  point*/
    double U0=(da-c2/2)/b2;
    k=Vec3(0,U0,0.5)*Gc;
    list.append(new SpecialPoint("U0" ,k));

    /* Y0  point*/
    double Y0=da/b2;
    k=Vec3(0,Y0,0)*Gc;
    list.append(new SpecialPoint("Y0" ,k));

    /* L0  point*/
    double L0=(dc-db)/b2;
    k=Vec3(sig0,L0,0)*Gc;
    list.append(new SpecialPoint("L0" ,k));

    /* M0  point*/
    double M0=(dc-da)/a2;
    k=Vec3(M0,Y0,0)*Gc;
    list.append(new SpecialPoint("M0" ,k));
  }







  


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeABCCOSpecPoints(const Matrix3x3 &crcell,
					   long bravais)
{

  Matrix3x3 Gc=crcell.transposed();
  Matrix3x3 ScI=Gc*crcell;
  double a2=ScI(0,0);
  double b2=ScI(1,1);
  double c2=ScI(2,2);

  double da=(b2+c2)/4;
  double dc=(a2+b2)/4;

  Vec3 k;

  if(bravais==SYM_CBASECENTERED){


    // Y= (1/2 0 0) ~ (0 1/2 0)
    // T= (0 1/2 1/2) ~ (1/2 0 1/2)
    // Z= (0 0 1/2)
    // S= (1/4 1/4 0)
    // R= (1/4 1/4 1/2)
    // Sigma= (u 0 0) ~ C=(u 1/2 0)
    // Delta= (0 u 0) ~ F=(1/2 u 0)
    // A= (u 0 1/2) ~ E=(u 1/2 1/2)
    // B= (0 u 1/2) ~ G=(1/2 u 1/2)



    if(a2>=b2){

      /* Gamma  point*/
      list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
      
      /* Y  point*/
      k=Vec3(0,0.5,0)*Gc;
      list.append(new SpecialPoint("Y" ,k));
    
      /* T  point*/
      k=Vec3(0,0.5,0.5)*Gc;
      list.append(new SpecialPoint("T" ,k));

      /* Z  point*/
      k=Vec3(0,0,0.5)*Gc;
      list.append(new SpecialPoint("Z" ,k));

      /* S  point*/
      k=Vec3(0.25,0.25,0)*Gc;
      list.append(new SpecialPoint("S" ,k));
      
      /* R  point*/
      k=Vec3(0.25,0.25,0.5)*Gc;
      list.append(new SpecialPoint("R" ,k));

      /* Sigma0  point*/
      double sig0=dc/a2;
      k=Vec3(sig0,0,0)*Gc;
      list.append(new SpecialPoint("$~S0" ,k));

      /* A0  point*/
      k=Vec3(sig0,0,0.5)*Gc;
      list.append(new SpecialPoint("A0" ,k));

      /* C0  point*/
      double C0=(dc-b2/2)/a2;
      k=Vec3(C0,0.5,0)*Gc;
      list.append(new SpecialPoint("C0" ,k));

      /* E0  point*/
      k=Vec3(C0,0.5,0.5)*Gc;
      list.append(new SpecialPoint("E0" ,k));

    
    }else{

      /* Gamma  point*/
      list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
      
      /* Y  point*/
      k=Vec3(0.5,0,0)*Gc;
      list.append(new SpecialPoint("Y" ,k));
    
      /* T  point*/
      k=Vec3(0.5,0,0.5)*Gc;
      list.append(new SpecialPoint("T" ,k));

      /* Z  point*/
      k=Vec3(0,0,0.5)*Gc;
      list.append(new SpecialPoint("Z" ,k));

      /* S  point*/
      k=Vec3(0.25,0.25,0)*Gc;
      list.append(new SpecialPoint("S" ,k));
      
      /* R  point*/
      k=Vec3(0.25,0.25,0.5)*Gc;
      list.append(new SpecialPoint("R" ,k));

      /* Delta0  point*/
      double delta0=dc/b2;
      k=Vec3(0,delta0,0)*Gc;
      list.append(new SpecialPoint("$~D0" ,k));


      /* B0  point*/
      k=Vec3(0,delta0,0.5)*Gc;
      list.append(new SpecialPoint("B0" ,k));

      /* F0  point*/
      double F0=(dc-a2/2)/b2;
      k=Vec3(0.5,F0,0)*Gc;
      list.append(new SpecialPoint("F0" ,k));

      /* G0  point*/
      k=Vec3(0.5,F0,0.5)*Gc;
      list.append(new SpecialPoint("G0" ,k));
    }
  }else if(bravais==SYM_ABASECENTERED){

    myDebug()<<a2<<b2<<c2;
    if(b2>=c2){

      /* Gamma  point*/
      list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
      
      /* Y  point*/
      k=Vec3(0,0,-0.5)*Gc;
      list.append(new SpecialPoint("Y" ,k));
    
      /* T  point*/
      k=Vec3(0.5,0,-0.5)*Gc;
      list.append(new SpecialPoint("T" ,k));

      /* Z  point*/
      k=Vec3(0.5,0,0)*Gc;
      list.append(new SpecialPoint("Z" ,k));

      /* S  point*/
      k=Vec3(0,0.25,-0.25)*Gc;
      list.append(new SpecialPoint("S" ,k));
      
      /* R  point*/
      k=Vec3(0.5,0.25,-0.25)*Gc;
      list.append(new SpecialPoint("R" ,k));

      /* Delta0  point*/
      double delta0=da/b2;
      k=Vec3(0,delta0,0)*Gc;
      list.append(new SpecialPoint("$~D0" ,k));


      /* B0  point*/
      k=Vec3(0.5,delta0,0)*Gc;
      list.append(new SpecialPoint("B0" ,k));

      /* F0  point*/
      double F0=(da-c2/2)/b2;
      k=Vec3(0,F0,-0.5)*Gc;
      list.append(new SpecialPoint("F0" ,k));

      /* G0  point*/
      k=Vec3(0.5,F0,-0.5)*Gc;
      list.append(new SpecialPoint("G0" ,k));
    
    }else{
      /* Gamma  point*/
      list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
      
      /* Y  point*/
      k=Vec3(0,0.5,0)*Gc;
      list.append(new SpecialPoint("Y" ,k));
    
      /* T  point*/
      k=Vec3(0.5,0.5,0)*Gc;
      list.append(new SpecialPoint("T" ,k));

      /* Z  point*/
      k=Vec3(0.5,0,0)*Gc;
      list.append(new SpecialPoint("Z" ,k));

      /* S  point*/
      k=Vec3(0,0.25,-0.25)*Gc;
      list.append(new SpecialPoint("S" ,k));
      
      /* R  point*/
      k=Vec3(0.5,0.25,-0.25)*Gc;
      list.append(new SpecialPoint("R" ,k));

      /* Sigma0  point*/
      double sig0=da/c2;
      k=Vec3(0,0,-sig0)*Gc;
      list.append(new SpecialPoint("$~S0" ,k));

      /* A0  point*/
      k=Vec3(0.5,0,-sig0)*Gc;
      list.append(new SpecialPoint("A0" ,k));

      /* C0  point*/
      double C0=(da-b2/2)/c2;
      k=Vec3(0,0.5,-C0)*Gc;
      list.append(new SpecialPoint("C0" ,k));

      /* E0  point*/
      k=Vec3(0.5,0.5,-C0)*Gc;
      list.append(new SpecialPoint("E0" ,k));
    }
    
    
    
    
    
  }else{
    myDebug()<< "B-base centered not implemented";
    exit(0);
  }

  

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeSOSpecPoints(const Matrix3x3 &rcell)
{
  Vec3 k;
  Matrix3x3 G=rcell.transposed();


  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));

  /* X  point*/
  k=Vec3(0.5,0,0)*G;
  list.append(new SpecialPoint("X" ,k));


  /* S point*/
  k=Vec3(0.5,0.5,0)*G;
  list.append(new SpecialPoint("S" ,k));

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));


  /* Z  point*/
  k=Vec3(0,0,0.5)*G;
  list.append(new SpecialPoint("Z" ,k));

  
  /* U  point*/
  k=Vec3(0.5,0,0.5)*G;
  list.append(new SpecialPoint("U" ,k));
  
  /* R  point*/
  k=Vec3(0.5,0.5,0.5)*G;
  list.append(new SpecialPoint("R" ,k));

  /* Z  point*/
  k=Vec3(0,0,0.5)*G;
  list.append(new SpecialPoint("Z" ,k));

  /* T  point*/
  k=Vec3(0,0.5,0.5)*G;
  list.append(new SpecialPoint("T" ,k));

  /* Y  point*/
  k=Vec3(0,0.5,0)*G;
  list.append(new SpecialPoint("Y" ,k));

  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));


  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeSMSpecPoints(const Matrix3x3 &rcell)
{

  // take these points as basis to evaluate cell-specific points
  Vec3 k;
  
  Matrix3x3 G=rcell.transposed();

  ValueArray<Plane> pla;
  Boundary::prepWSCellPlanes(rcell,pla);
  
  
  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
  
  
  /* Y  point*/
  k=Vec3(0.5,0,0)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("Y" ,k));
  
  /* C  point*/
  k=Vec3(0.5,0.5,0)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("C" ,k));
  
  /* Z  point*/
  k=Vec3(0.,0.5,0.)*G;
  list.append(new SpecialPoint("Z" ,k));
  
  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
  
  /* B  point*/
  k=Vec3(0,0,0.5)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("B" ,k));
  
  
  /* D  point*/
  k=Vec3(0,0.5,0.5)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("D" ,k));
  
  /* A  point*/
  k=Vec3(-0.5,0,0.5)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("A" ,k));
  
  /* E  point*/
  k=Vec3(-0.5,0.5,0.5)*G;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("E" ,k));
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialSymPoints::correctSMSpecPoint(const ValueArray<Plane> &pla,Vec3 &k)
{
  //todo: correct with all planes
  double eps=1.001;

  bool changed=true;
  while(changed){
    changed=false;
    for(long i=-1;++i<pla.size();){
      const Plane &p=pla.at(i);
      if( ( k*p.n() ) > p.d()*eps ){
	//myDebug()<<"larger than "<<p.P();
	k=k-p.P();
	changed=true;
      }
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpecialSymPoints::makeCFCMSpecPoints(const Matrix3x3 &crcell,
					  const Matrix3x3 &rcell)
{

  Vec3 k;
  
  Matrix3x3 Gc=crcell.transposed();
  
  ValueArray<Plane> pla;
  Boundary::prepWSCellPlanes(rcell,pla);

  
  /* Gamma  point*/
  list.append(new SpecialPoint("$~G",0.,0.,0.,0.5,-0.5,1.));
      
    
  /* A  point*/
  k=Vec3(0,0,0.5)*Gc;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("A" ,k));

  /* M  point*/
  k=Vec3(0,0.5,0.5)*Gc;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("M" ,k));

  /* Y  point*/
  k=Vec3(0,0.5,0)*Gc;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("Y" ,k));


  /* V  point*/
  k=Vec3(0.25,0.25,0.)*Gc;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("V" ,k));

  /* L  point*/
  k=Vec3(0.25,0.25,0.5)*Gc;
  correctSMSpecPoint(pla,k);
  list.append(new SpecialPoint("L" ,k));

  return;
}

/*EOF*/ 
