/* @(#)Spsp.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SPSP_H
#define _SPSP_H 1


#include <QList> 
#include <QString> 
#include "MyQRgb.hh"
#include "SReal.hh"
#include "Xf.hh"
#include "Vec3.hh"


namespace XFPLO{
  class Symmetry;
}
class Dobj;
class Plane;

using namespace XFPLO;
/* ***********************************************************************
 *
 * ***********************************************************************
 */

class SpecialPoint{
 public:

  SpecialPoint()
    :pvisible(true),pname("$~G"),ppoint(0,0,0),pshifts(0.5,-0.5,0.25)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(Xf::SPTPoint),pchanged(false)
    {};
  SpecialPoint(Xf::SpecialPointType t)
    :pvisible(true),pname("$~G"),ppoint(0,0,0),pshifts(0.5,-0.5,0.25)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(t),pchanged(false)
    {};
  SpecialPoint(const QString &n,double q0,double q1,double q2)
    :pvisible(true),pname(n),ppoint(q0,q1,q2),pshifts(0.5,-0.5,0.25)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(Xf::SPTPoint),pchanged(false)
    {};
  SpecialPoint(const QString &n,const Vec3 &q)
    :pvisible(true),pname(n),ppoint(q[0],q[1],q[2]),pshifts(0.5,-0.5,0.25)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(Xf::SPTPoint),pchanged(false)
    {};
  SpecialPoint(const QString &n,double q0,double q1,double q2
		 ,double fs0 ,double fs1 ,double fs2 )
    :pvisible(true),pname(n),ppoint(q0,q1,q2),pshifts(fs0,fs1,fs2)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(Xf::SPTPoint),pchanged(false)
    {};
  SpecialPoint(const QString &n,const Vec3 &q
		 ,double fs0 ,double fs1 ,double fs2 )
    :pvisible(true),pname(n),ppoint(q[0],q[1],q[2]),pshifts(fs0,fs1,fs2)
    ,ppointradius(0.025),ppointcolor(0xff0000),ptextcolor(0x0000ff)
    ,pfontscale(0.1),ptype(Xf::SPTPoint),pchanged(false)
    {};
  SpecialPoint(const SpecialPoint &other)
    {
      pvisible=other.pvisible;
      pname=other.pname;
      ppoint=other.ppoint;
      pshifts=other.pshifts;
      ppointradius=other.ppointradius;
      ppointcolor=other.ppointcolor;
      ptextcolor=other.ptextcolor;
      pfontscale=other.pfontscale;
      ptype=other.ptype;
      pchanged=other.pchanged;
    };

  inline bool isVisible()const{return pvisible;};
  inline void setVisible(bool b){pchanged|=(pvisible!=b);pvisible=b;};

  inline Xf::SpecialPointType type()const{return ptype;};
  inline void setType(Xf::SpecialPointType t){pchanged|=(ptype!=t);ptype=t;};

  inline const SReal &pointRadius()const{return ppointradius;};
  inline void setPointRadius(const SReal &r){
    pchanged|=(ppointradius!=r);ppointradius=r;};

  inline QRgb pointColor()const{return ppointcolor;};
  inline void setPointColor(QRgb c){
    pchanged|=(ppointcolor!=c);ppointcolor=c;};

  inline QRgb textColor()const{return ptextcolor;};
  inline void setTextColor(QRgb c){
    pchanged|=(ptextcolor!=c);ptextcolor=c;};

  inline const SReal &fontScale()const{return pfontscale;};
  inline void setFontScale(const SReal & s){
    pchanged|=(pfontscale!=s);pfontscale=s;};

  inline QString name()const{return pname;};
  inline void setName(QString n){
    pchanged|=(pname!=n);pname=n;};

  inline const SReal3 &point()const{return ppoint;};
  inline void setPoint(const SReal3 &p){
    pchanged|=(ppoint!=p);ppoint=p;};

  inline const SReal3 &shift()const{return pshifts;};
  inline void setShift(const SReal3 &p){
    pchanged|=(pshifts!=p);pshifts=p;
  };

  inline bool changed()const{return pchanged;};
  inline void setChanged(bool c){pchanged=c;};

 private:
  bool pvisible;
  QString pname;			/*  name of SPP */
  SReal3 ppoint;			/* k-point in cartesian coordinates  */
  SReal3 pshifts;  		/* shifts in units of text dimensions
				   relative to k-point
				   
				   pos x-shift moves label's lower left 
				   corner to the right
				   
				   neg y-shift moves label's lower left 
				   corner down

				   pos z-shift moves label's towards 
				   viewer (in units of text height)
				*/
  SReal ppointradius;
  QRgb ppointcolor;
  QRgb ptextcolor;
  SReal pfontscale;
  Xf::SpecialPointType ptype;
  bool pchanged;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Vlevel;

class SpecialSymPoints
{
  
 public:
  SpecialSymPoints():puserdefined(false),pshowpath(true),pvisible(false)
    ,pusecommonproperties(true)
    ,ppointradius("0.025"),ppointcolor(0xff0000),ptextcolor(0xff)
    ,pfontscale("0.1"),ppathcolor(0xaa0000),ppathlinewidth(3)
    ,pchanged(false),pcalced(false){};
  ~SpecialSymPoints(){};

  const SpecialSymPoints &operator=(const SpecialSymPoints &other){
    aa=other.aa;
    puserdefined=other.puserdefined;
    pshowpath=other.pshowpath;
    ppathcolor=other.ppathcolor;
    pusecommonproperties=other.pusecommonproperties;
    ppointradius=other.ppointradius;
    ppointcolor=other.ppointcolor;
    ptextcolor=other.ptextcolor;
    pfontscale=other.pfontscale;
    ppathlinewidth=other.ppathlinewidth;
    pvisible=other.pvisible;
    clear();
    foreach(SpecialPoint *p,other.list){
      list.append(new SpecialPoint());
      (*(list.last()))=*p;
    }
    pchanged=other.pchanged;
    pcalced=other.pcalced;
    return *this;
  }

  void clear(){qDeleteAll(list);list.clear();pchanged=true;};

  inline const SReal &pointRadius()const{return ppointradius;};
  inline void setPointRadius(const SReal &r){
    pchanged|=(ppointradius!=r);ppointradius=r;};

  inline QRgb pointColor()const{return ppointcolor;};
  inline void setPointColor(QRgb c){
    pchanged|=(ppointcolor!=c);ppointcolor=c;};

  inline QRgb textColor()const{return ptextcolor;};
  inline void setTextColor(QRgb c){
    pchanged|=(ptextcolor!=c);ptextcolor=c;};

  inline QRgb pathColor()const{return ppathcolor;};
  inline void setPathColor(QRgb c){
    pchanged|=(ppathcolor!=c);ppathcolor=c;};

  inline const SReal &fontScale()const{return pfontscale;};
  inline void setFontScale(const SReal & s){
    pchanged|=(pfontscale!=s);pfontscale=s;};


  inline const double &pathLineWidth()const{return ppathlinewidth;};
  inline void setPathLineWidth(const double & s){
    pchanged|=(ppathlinewidth!=s);ppathlinewidth=s;};



  inline bool useCommonProperties()const{return pusecommonproperties;};
  inline void setUseCommonProperties(bool b){
    pchanged|=(pusecommonproperties!=b);pusecommonproperties=b;};

  inline bool isVisible()const{return pvisible;};
  inline void setVisible(bool b){
    //pchanged|=(pvisible!=b);
    pvisible=b;
  };

  inline bool userDefined()const{return puserdefined;};
  inline void setUserDefined(bool u){
    pchanged|=(puserdefined!=u);puserdefined=u;};

  inline bool showPath()const{return pshowpath;};
  inline void setShowPath(bool u){
    //pchanged|=(pshowpath!=u);
    pshowpath=u;
  };

  void calcPoints(const Symmetry *sym,const Vlevel &vlevel);


  int size()const{return list.size();};
  SpecialPoint *operator[](int i){return list[i];};
  const SpecialPoint *at(int i)const{return list.at(i);};
  void insertNewPoint(int position,Xf::SpecialPointType type,SpecialPoint *p){
    if(p==0){
      if(type==Xf::SPTPoint){
	if(list.size()==0||position==0){
	  list.insert(position,new SpecialPoint());
	}else{
	  if(list.at(position-1)->type()==Xf::SPTPoint)
	    list.insert(position,new SpecialPoint(*(list.at(position-1))));
	  else
	    list.insert(position,new SpecialPoint());
	}
      }else if(type==Xf::SPTBreak || type==Xf::SPTJump){
	list.insert(position,new SpecialPoint(type));
      }
    }else
      list.insert(position,p);

  };
  SpecialPoint *removePoint(int position){
    return list.takeAt(position);
  };

  inline bool changed()const{
    foreach(const SpecialPoint *p,list){
      if(p->changed())return true;
    }
    return pchanged;
  };
  inline void setChanged(bool c){
    for(int i=0;i<list.size();i++)
      list[i]->setChanged(c);
    pchanged=c;};

  QString reportSpecPoints(const Vlevel &vlevel);

  void PTSectionToFile(FILE *FH)const;
  void readPTSectionFromFile(const Dobj &root);

  void exportToInFile(const QString &fname,const Symmetry *sym)const;
  void importFromInFile(const QString &fname,const Symmetry *sym);
  
 private:
  void makeSCSpecPoints(const Matrix3x3 &rcell);
  void makeFCCSpecPoints(const Matrix3x3 &crcell);
  void makeBCCSpecPoints(const Matrix3x3 &crcell);
  void makeSTSpecPoints(const Matrix3x3 &rcell);
  void makeBCTSpecPoints(const Matrix3x3 &crcell);
  void makeSHSpecPoints(const Matrix3x3 &rcell);
  void makeRTGSpecPoints(const Matrix3x3 crcell);
  void makeFCOSpecPoints(const Matrix3x3 crcell);
  void makeBCOSpecPoints(const Matrix3x3 &crcell);
  void makeABCCOSpecPoints(const Matrix3x3 &crcell,long bravais);
  void makeSOSpecPoints(const Matrix3x3 &rcell);
  void makeSMSpecPoints(const Matrix3x3 &rcell);
  void makeCFCMSpecPoints(const Matrix3x3 &crcell,const Matrix3x3 &rcell);
  
  void correctSMSpecPoint(const ValueArray<Plane> &pla,Vec3 &k);
  //
  QList<SpecialPoint*> list;

  double aa;
  bool puserdefined;
  bool pshowpath;
  bool pvisible;
  bool pusecommonproperties;
  SReal ppointradius;
  QRgb ppointcolor;
  QRgb ptextcolor;
  SReal pfontscale;
  QRgb ppathcolor;
  double ppathlinewidth;
  bool pchanged;
  bool pcalced;
  
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SPSP_H */

