//todo: cpa, frel, site reorder in fplo

/* @(#)Symmetry.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef STRUCTURE_H
#define STRUCTURE_H

#include <stdio.h>
#include "MyQRgb.hh"
#include "Vec3.hh"
#include "PropertyTable.hh"
#include "Group.hh"
#include "MACROS.hh"
#include "FString.hh"
#include "Xf.hh"

class Parser;
//class SpaceGroupOperations;

namespace XFPLO{

class PropertyTable;



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class WyckPos{
 public:

  WyckPos(){}
  WyckPos(const String &e,long ty,const String t[3],const AtomProperty &p,bool vis)
    :pel(e),ptype(ty),ptauD(t),pproperty(p),pvisible(vis){
    ptauS[0]=t[0]; ptauS[1]=t[1]; ptauS[2]=t[2]; };

  WyckPos(const String &e,long ty,const Vec3 &p,const AtomProperty &pr,bool vis)
    :pel(e),ptype(ty),ptauD(p),pproperty(pr),pvisible(vis){
    ptauS[0]=Sprintf("%30.15f",p[0]).stripBlanks().chop("0");
    ptauS[1]=Sprintf("%30.15f",p[1]).stripBlanks().chop("0");
    ptauS[2]=Sprintf("%30.15f",p[2]).stripBlanks().chop("0");
  };

  WyckPos(const String &e,long ty,const StringList &tl,const AtomProperty &p,bool vis)
    :pel(e),ptype(ty),ptauD(Vec3(tl[0],tl[1],tl[2])),pproperty(p),pvisible(vis){ 
    ptauS[0]=tl[0]; ptauS[1]=tl[1]; ptauS[2]=tl[2]; };

  WyckPos(const String &e,const String &x,const String &y,const String &z
	   ,const AtomProperty &p,bool vis)
    :pel(e), ptauD(Vec3(x,y,z)),pproperty(p),pvisible(vis){
    ptauS[0]=x;ptauS[1]=y;ptauS[2]=z;
    setDefaultType();};
  
  inline void setDefaultType(){
    String e=pel.stripBlanks().capitalize();
    ptype=AtomPropertyTable::allowedElements.find(e());
  };


  bool operator!=(const WyckPos &other)const{
    return   ( (pel != other.pel) ||
	       (ptype != other.ptype) ||
	       (pproperty != other.pproperty) ||
	       (pvisible != other.pvisible) ||
	       (ptauS[0] != other.ptauS[0]) ||
	       (ptauS[1] != other.ptauS[1]) ||
	       (ptauS[2] != other.ptauS[2]) );
  }

  void print()const;
private:
  CPROPERTY(el,el,Element,String);
  PROPERTY(type,type,Type,long);// user defined type (nuclear charge by default)
  CVPROPERTY(tauS,3,tauS,TauS,String);
  CPROPERTY(tauD,tauD,TauD,Vec3);
  CMODPROPERTY(property,property,Property,AtomProperty);
  PROPERTY(visible,visible,Visible,bool);
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class Site{
 public:
  Site(){}
  Site(const String &e,long ty,long srt,const Vec3 t,Xf::LengthUnits lu,
       const AtomProperty &p,bool vis)
    :pel(e),ptype(ty),psort(srt),ptauD(t),pnndist(0.5)
    ,plengthunit(lu),pproperty(p),pvisible(vis){};
  String printS()const;
private:
  CPROPERTY(el,el,Element,String);
  PROPERTY(type,type,Type,long);// user defined type (nuclear charge by default)
  PROPERTY(sort,sort,Sort,long);
  CPROPERTY(tauD,tauD,TauD,Vec3);
  PROPERTY(nndist,nndist,NNDist,double);
  PROPERTY(lengthunit,lengthUnit,LengthUnit,Xf::LengthUnits);
  CMODPROPERTY(property,property,Property,AtomProperty);
  PROPERTY(visible,visible,Visible,bool);
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class PairVec{
public:
  PairVec()
    :el("??"),site(-1),sort(-1),type(-1),pabs(0.){};
  PairVec(const String e,long isa,long issa,long ty,const Vec3 &pd)
    :el(e),site(isa),sort(issa),type(ty),p(pd),pabs(norm(pd)){};
  String el;
  long site;
  long sort;
  long type;// user defined type (nuclear charge by default)
  Vec3 p;
  double pabs;			/* vector from reference site to site */
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
inline MyDebug operator<<(MyDebug dbg, const PairVec &p)
{
  dbg << Sprintf("[%s(S%ld,W%ld,T%ld): [%s], |%g|]",p.el(),p.site+1,p.sort+1,p.type,
		p.p.printS()(),p.pabs);
  
  return dbg;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Symmetry
{
public:
  Symmetry();
  Symmetry(const Symmetry &other);
  ~Symmetry();

  // must be the same as in ../SYM/symmetry.c
  
  const Symmetry &operator=(const Symmetry &other);

  bool changed(){return pchanged;};
  void setChanged(bool s){pchanged=s;pcalced=false;};

  bool calculated(){return pcalced;};
  void setCalculated(bool s){pcalced=s;};

  long spacegroup() const{return pspacegroup;};
  void setSpacegroup(long s);

  Xf::LengthUnits dataLengthUnit() const{return pdatalengthunit;};
  Xf::LengthUnits lengthUnit() const{return plengthunit;};
  void setLengthunit(Xf::LengthUnits s);

  void convertTo(const String &mode);
  void increaseCell(const Matrix3x3 &,bool toconventional=false,long prot=0);
  void insertVacuum(const Vec3 &,double amount,double z0);

  Xf::StructureType dataStructureType() const{return pdatastructuretype;};
  Xf::StructureType structureType() const{return pstructuretype;};
  void setStructureType(Xf::StructureType t);

  const ValueArray<int> &subGroupGenerators()const{return psubgroupgenerators;};
  void setSubGroupGenerators(ValueArray<int> l);

  const String &latticeConstantS(int i) const{return palatS[i];};
  double latticeConstant(int i) const{return palatD[i];};
  void setLatticeConstants(const String a[3]);
  void setLatticeConstants(const StringList &sl);
  void setLatticeConstants(const Vec3 &a);

  const String &axesAngleS(int i)const {return panglesS[i];};
  double axesAngle(int i)const {return panglesD[i];};
  void setAxesAngles(const String a[3]);
  void setAxesAngles(const StringList sl);
  void setAxesAngles(const Vec3 &a);

  void applyTheme(const AtomPropertyTable *ptable);

  const String formula()const{return pformula;};

  inline long  nsort()const {return pwyckpos.size();};
  inline long  nsite()const {return psites.size();};
  const WyckPos &wyckoffPosition(long i)const {return pwyckpos.at(i);};
  WyckPos &wyckoffPosition(long i){return pwyckpos[i];};
  ValueArray<WyckPos> &wyckoffPositions(){return pwyckpos;}
  const ValueArray<WyckPos> &wyckoffPositions()const {return pwyckpos;}
  void setWyckoffPosition(long i,const WyckPos &pos);
  void setWyckoffPositions(const ValueArray<WyckPos> &l);
  WyckPos removeWyckoffPosition(long pos);
  void insertWyckoffPosition(long pos,const WyckPos *wpos);


  static Matrix3x3 convertToSimple(const Matrix3x3 &cell,long bravais
				   ,Xf::LatticeType lattype);


  Matrix3x3 cell() const;
  Matrix3x3 ccell() const;
  Matrix3x3 rcell() const;
  Matrix3x3 crcell() const;
  long bravais() const;
  long crystal() const;
  double vuc() const;
  double *opmat() const;
  double *optau() const;
  long numOps() const;
  long iTable(long ig) const;

  inline long siteBySort(long sort)const{
    if(sort<0 || sort >=pwyckpos.size())return -1;
    return psitebysort.at(sort);};
  inline ValueArray<long> siteListBySort(long sort)const{
    if(sort<0 || sort >=pwyckpos.size())return ValueArray<long>();
    return psitelistbysort.at(sort);
  };
  inline long sortBySite(long site)const{
    if(site<0 || site >=psites.size())return -1;
    return psites.at(site).sort();};

  const ValueArray<Site> &sites() const{return psites;};
  const ValueArray<PtrArray<PairVec> > &pairs() const{return ppairs;};

  void readSymInfo(const String &fname);
  void calcSymFromGroupInput(String &out,bool calcAtoms=false
			     ,bool giveinfo=true,bool calcpairinfo=true);


  // void calcSymmetryOperations();

  // const SpaceGroupOperations *spaceGroupOperations()const 
  //   {return pspacegroupoperations;};
  // SpaceGroupOperations *spaceGroupOperations() 
  //   {return pspacegroupoperations;};

  void symPTSectionToFile(FILE *FH,bool full);
  static void readSymmetryPTSectionFromFile(Parser *par,Symmetry &tmp
					    ,bool full);

  static void getGroupOperations(long num,Xf::StructureType str_type
				 ,ValueArray<Operation> &ops);

  static const char *spacegroupName(long spacegroup) ;
  static const char *structureTypes(Xf::StructureType l) 
    {return pstructuretypes[l];};

 private:
  //  void reportWyckoffs();
  void calcAtomsFromGroup();
  String reportCalcStructure();
  long checkSites(String &msg);
  void calcAutoRadii();
  void determineShells();  
  void combiShells();  

 private:
  long          pspacegroup;
  Xf::LengthUnits   plengthunit;
  Xf::StructureType pstructuretype;
  String palatS[3];
  String panglesS[3];
  Vec3 palatD;
  Vec3 panglesD;
  ValueArray<int> psubgroupgenerators;

  Xf::LengthUnits   pdatalengthunit;
  Xf::StructureType pdatastructuretype;
  ValueArray<WyckPos> pwyckpos;
  ValueArray<Site> psites;
  ValueArray<ValueArray<long> > psitelistbysort;
  ValueArray<long> psitebysort;
  ValueArray<PtrArray<PairVec> > ppairs;
  String pformula;

  // SpaceGroupOperations *pspacegroupoperations;
  
  struct tssymmetry *sym;
  bool pchanged;
  bool pcalced;
  static const char *pstructuretypes[];
  
};

  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  inline MyDebug operator<<(MyDebug dbg, const Site &site)
  {
    return dbg << " site:"<<site.printS();
  }
}


#endif // STRUCTURE_H
