/*************************************************************************
 * @(#)TriangleMesh.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 May 2010
 *************************************************************************
 */

#include <stdlib.h>
#include "MyMath.hh"
#include "TriangleMesh.hh"
#include "OGL.hh"
#include "ColorModel.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
TriangleMesh::TriangleMesh(const String &id)
  :pmeshcalced(false),pnvertices(0),pnallocvertices(0),pndata(1),pvertices(0)
  ,pnormals(0),pdata(0),pcolors(0)
  ,pntriangles(0),pnalloctriangles(0),ptriangles(0)
  ,pid(id)
  ,pboundingradius(1)
{
  //allocate_vertices_internal(1);
  //allocate_triangles_internal(1);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
TriangleMesh::~TriangleMesh()
{
  if(pvertices) free(pvertices); pvertices=0;
  if(pnormals) free(pnormals); pnormals=0;
  if(pdata) free(pdata); pdata=0;
  if(pcolors) free(pcolors); pcolors=0;
  pnvertices=0;
  pnallocvertices=0;

  if(ptriangles) free(ptriangles); ptriangles=0;
  pntriangles=0;
  pnalloctriangles=0;

  pmeshcalced=false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TriangleMesh::allocate_vertices_internal(long add)
{
  if(!pvertices){
    pnallocvertices=add;
    pvertices=(GLdouble*)malloc(sizeof(GLdouble)*pnallocvertices*3);
    pnormals=(GLdouble*)malloc(sizeof(GLdouble)*pnallocvertices*3);
    pdata=(double*)malloc(sizeof(double)*pnallocvertices*pndata);
    pcolors=(GLfloat*)malloc(sizeof(GLfloat)*pnallocvertices*4);
  }else{
    pnallocvertices+=add;
    pvertices=(GLdouble*)realloc(pvertices,sizeof(GLdouble)*pnallocvertices*3);
    pnormals=(GLdouble*)realloc(pnormals,sizeof(GLdouble)*pnallocvertices*3);
    pdata=(double*)realloc(pdata,sizeof(double)*pnallocvertices*pndata);
    pcolors=(GLfloat*)realloc(pcolors,sizeof(GLfloat)*pnallocvertices*4);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TriangleMesh::allocate_triangles_internal(long add)
{
  if(!ptriangles){
    pnalloctriangles=add;
    ptriangles=(GLuint*)malloc(sizeof(GLuint)*pnalloctriangles*3);
  }else{
    pnalloctriangles+=add;
    ptriangles=(GLuint*)realloc(ptriangles,sizeof(GLuint)*pnalloctriangles*3);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void TriangleMesh::dataRange(long i,double &dmin,double &dmax)
{
  if(!pdata){
    dmin=0.;
    dmax=1.;
    return;
  }
  dmin=pdata[i];
  dmax=pdata[i];
  for(long iv=0;iv<pnvertices;iv++){
    dmin=MyMath::min<double>(pdata[pndata*iv+i],dmin);
    dmax=MyMath::max<double>(pdata[pndata*iv+i],dmax);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

static double fucos(double x)
{
  return (1.-cos(x*Pi))/2.;
}

void TriangleMesh::applyColor(long idata,const IsoColor &isocolor
			      ,double dmin,double dmax)
{
  NSF("TriangleMesh::applyColor");
  myDebug() << nsubfunc << idata;
  pisocolor=isocolor;

  myDebug() << nsubfunc<< "minmax" << dmin<<dmax;
  
  if(
     (pisocolor.outerFromData() ||
      pisocolor.innerFromData() ) && pdata
     ){
  
    QColor c;
    double d=(dmax-dmin);
    for(long iv=0;iv<pnvertices;iv++){
      double h=0.5;
      double value=pdata[pndata*iv+idata];
      if(d!=0)  h=(value-dmin)/d;
      c=isocolor.colorModel()->getColorFromIndex(h);

      double alpha=1;
      if(1){
	double h=0.5;
	if(d!=0)  h=(value-dmin)/d;
	c=isocolor.colorModel()->getColorFromIndex(h);
	  
	double d1=value;
	double hal=0.5;
	if(d!=0)  hal=(d1);
	hal=(hal-0)/1;
	alpha=MyMath::max<double>(0.,MyMath::min<double>(1.,hal));
	alpha=isocolor.alpha()/255.;
	//if(fabs(alpha)>0.001) myDebug() << alpha;
      }
      if(0){
	//hack
	double hal=0.5;
	
	double d3=0*pdata[pndata*iv+3];//bulk
	double d2=0*pdata[pndata*iv+2];//subsurf
	double d1=pdata[pndata*iv+1];//surf

	double d7=0*pdata[pndata*iv+7];
	
	//d1=d2=0; // 4Fe
	
	if(d!=0)  hal=(d1+d2+d3+d7);
	hal=(hal-dmin)/d;
	//if(h<0) myDebug() << "h"<<h;
	//
	
	
	alpha=MyMath::max<double>(0.,MyMath::min<double>(1.,hal));
	c.setRgb(0xffff0000);
	
	QColor c3=QColor(0xffff0000);
	QColor c2=QColor(0xffff0000);
	QColor c1=QColor(0xff00ff00);
	QColor c7=QColor(0xff00ff00);
	double f3=((d3));
	double f2=((d2));
	double f1=((d1));
	double f7=((d7));
	double dsum=((f1)+(f2)+f3+f7);
	if(fabs(dsum)<1.0e-6)dsum=1.;
	int hue=lRound((
			c1.hue()*(f1)
			+c2.hue()*(f2)
			+c3.hue()*(f3)
			+c7.hue()*(f7)
			)/dsum);
	//myDebug() << "hue55"<<c55.hue()<<"hue"<<hue<<d55<<dsum<<d55/dsum;
	
	c.setHsv(MyMath::max<long>(0,MyMath::min<long>(255,(int)(hue))),255,255);
	c.setRgb(0xffff0000);
      }
      pcolors[4*iv+0]=c.red()/255.; 
      pcolors[4*iv+1]=c.green()/255.;
      pcolors[4*iv+2]=c.blue()/255.;
      pcolors[4*iv+3]=fucos(alpha);
    }
  }
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void TriangleMesh::normalizeNormals()
{
  for(long iv=0;iv<pnvertices;iv++){
    GLdouble d=sqrt(pnormals[3*iv]*pnormals[3*iv]+
		    pnormals[3*iv+1]*pnormals[3*iv+1]+
		    pnormals[3*iv+2]*pnormals[3*iv+2]);
    if(d>0){
      pnormals[3*iv  ]/=d;
      pnormals[3*iv+1]/=d;
      pnormals[3*iv+2]/=d;
    }
  }
  
}


/*EOF*/ 
