/* @(#)TriangleMesh.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _TRIANGLEMESH_H
#define _TRIANGLEMESH_H 1

#include <QGLWidget>
#include "Vec3.hh"
#include "IsoColor.hh"


class TriangleMesh
{
 public:
  TriangleMesh(const String &id);
  ~TriangleMesh();

  const String &id()const {return pid;};

  inline void setReady(bool ready){pmeshcalced=ready;};
  inline bool meshIsReady()const{return pmeshcalced;};

  inline long nVertices()const {return pnvertices;};
  inline long nTriangles()const {return pntriangles;};
  inline long numberOfData()const {return pndata;};

  inline const GLfloat  *colors(long i)const {return  pcolors+8*i;};
  // inline const GLfloat  *diffuseColor(long i)const {return  pcolors+8*i;};
  // inline const GLfloat  *ambientColor(long i)const {return  pcolors+8*i+4;};
  inline const GLdouble *normal(long i)const {return pnormals+3*i;};
  inline const GLdouble *vertex(long i)const {return pvertices+3*i;};
  inline const GLuint *triangle(long i)const {return ptriangles+3*i;};
  inline GLuint *triangle(long i) {return ptriangles+3*i;};

  inline const IsoColor &isoColor()const{return pisocolor;};

  void applyColor(long i,const IsoColor &isocolor,double dmin,double dmax);

  void dataRange(long i,double &dmin,double &dmax);

  void normalizeNormals();

/*   inline long appendVertex(const Vec3 &v,long add){ */
/*     adjustVertices(pnvertices,add); */
/*     pvertices[3*pnvertices+0]=v[0]; */
/*     pvertices[3*pnvertices+1]=v[1]; */
/*     pvertices[3*pnvertices+2]=v[2]; */
/*     pnvertices++; */
/*     return pnvertices-1; */
/*   }; */

  inline long appendVertex(const double *v,const double *n
			   ,double *data,long add){
    adjustVertices(pnvertices,add);
    pvertices[3*pnvertices+0]=v[0];
    pvertices[3*pnvertices+1]=v[1];
    pvertices[3*pnvertices+2]=v[2];
    pnormals[3*pnvertices+0]=n[0];
    pnormals[3*pnvertices+1]=n[1];
    pnormals[3*pnvertices+2]=n[2];
    memcpy(pdata+pndata*pnvertices,data,sizeof(double)*pndata);
    pnvertices++;
    return pnvertices-1;
  };

  inline GLuint *appendTriangle(long add){
    adjustTriangles(pntriangles,add);
    pntriangles++;
    return  ptriangles+3*pntriangles-3;
  };

  inline void setNumberOfData(long ndata){
    pndata=ndata;
  };
  
  inline void setBoundingMidPoint(const Vec3 &m){pboundingmidpoint=m;};
  inline void setBoundingRadius(double r){pboundingradius=r;};
  inline const Vec3 &boundingMidPoint()const{return pboundingmidpoint;};
  inline double boundingRadius()const{return pboundingradius;};
  
 private:


  inline void adjustVertices(long index,long add){
    if(index<pnallocvertices) return;
    allocate_vertices_internal(add);
  };

  inline void adjustTriangles(long index,long add){
    if(index<pnalloctriangles) return;
    allocate_triangles_internal(add);
  };
  

  void allocate_vertices_internal(long add);
  void allocate_triangles_internal(long add);
    
 private:
  bool pmeshcalced;

  long pnvertices;
  long pnallocvertices;
  long pndata; // number of data items per vertex
  GLdouble *pvertices; 
  GLdouble *pnormals; 
  double   *pdata; 
  GLfloat  *pcolors; 

  long pntriangles;
  long pnalloctriangles;
  GLuint *ptriangles; 

  IsoColor pisocolor;
  String pid;

  Vec3 pboundingmidpoint;
  double pboundingradius;
  //friend class RegularMesh; //todo rmove
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _TRIANGLEMESH_H */

