/* @(#)Value.hh
 * author: magru@unas
 */

#ifndef _VALUE_HH
#define _VALUE_HH 1

#include "Types.hh"

namespace XFPLO{

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class Value{
public:
  Value():pdata(0){}

  template <class T>
  Value(const T& v):ptype(TypeInfo<T>::type),pdata(new T(v)){}

  Value &operator=(const Value &o){
    clean();
    ptype=o.ptype;
    pdata=o.clone();
    return *this;
  }

  ~Value(){clean();}

  DataTypes dataType()const{return ptype;}


  template <class T>
  T to()const{
    if(pdata){
      if(ptype!=TypeInfo<T>::type)
	throw Except(Sprintf("cannot cast PointerT to %s",TypeInfo<T>::name));
      return *(static_cast<T*>(pdata));
    }else{
      return T();
    }
  }

  void clean(){
    if(!pdata) return;
    switch(ptype){
    case TIlong:
      delete static_cast<long*>(pdata);
      break;
    case TIdouble:
      delete static_cast<double*>(pdata);
      break;
    case TICmplx:
      delete static_cast<Cmplx*>(pdata);
      break;
    case TIunkown:
      break;
    }
    pdata=0;
  }

private:
  void *clone()const{
    if(!pdata) return 0;
    switch(ptype){
    case TIlong:
      return new long(*(static_cast<long*>(pdata)));
    case TIdouble:
      return new double(*(static_cast<double*>(pdata)));
    case TICmplx:
      return new Cmplx(*(static_cast<Cmplx*>(pdata)));
    case TIunkown:
      return 0;
    }
    return 0;
  }

private:
  DataTypes ptype;
  void *pdata;
};

inline MyDebug operator<<(MyDebug dbg,const Value &v){
  switch(v.dataType()){
  case TIlong:
    dbg<<v.to<long>();
    break;
  case TIdouble:
    dbg<<v.to<double>();
    break;
  case TICmplx:
    dbg<<v.to<Cmplx>();
    break;
  case TIunkown:
    dbg<<"unkown type";
    break;
  }
  return dbg;
}

}
#endif /* _VALUE_HH */

