/* @(#)View.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

//todo: remove what is not needed anymore

#ifndef _VIEW_H
#define _VIEW_H 1


#include <QGLWidget>
#include "Vec3.hh"
#include "MACROS.hh"
#include "MyMath.hh"

class Parser;


class OGLMatrix4x4{
 public:
  OGLMatrix4x4(){
    for(int k=0;k<16;k++)pM[k]=0;
    pM[0]=pM[5]=pM[10]=pM[15]=1;
  };
  OGLMatrix4x4(const Vec3 x,const Vec3 y,const Vec3 z){
    pM[0]=x[0];  pM[4]=x[1];  pM[8]=x[2];  pM[12]=0.;
    pM[1]=y[0];  pM[5]=y[1];  pM[9]=y[2];  pM[13]=0.;
    pM[2]=z[0];  pM[6]=z[1];  pM[10]=z[2];  pM[14]=0.;
    pM[3]=0.;  pM[7]=0.;  pM[11]=0.;  pM[15]=1.;
  };
  const GLdouble *operator()()const {return pM;};

  Vec3 operator*(const Vec3 &v)const{
    Vec3 r;
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
	r[i]+=pM[4*j+i]*v[j];
      }
    }
    return r;
  }

  OGLMatrix4x4 transposed()const{
    OGLMatrix4x4 m;
    for(int i=0;i<4;i++){
      for(int j=0;j<4;j++){
	m.pM[4*i+j]=pM[4*j+i];
      }
    }
    return m;
  };
 private:
  GLdouble pM[16];
};

// control the view port

class View 
{
 public:
  View();
  ~View(){};



  // new stuff
  void print() const;

  void setZ(const  Vec3 &vv);
  void setUp(const  Vec3 &vv);

  inline void setTheta(const  double t){ptheta=t;ptantheta=tan(ptheta/180.*Pi);};
  
  inline double minNear() const {return 0.01;};

  inline const OGLMatrix4x4 matrix()const{
    return OGLMatrix4x4(px,pup,pz);
  };


  inline void pan(double amount) { pscale*=exp(amount); };

  inline void setXYView(){setZ(Vec3(0,0,1)); setUp(Vec3(0,1,0));};
  inline void setYXView(){setZ(Vec3(0,0,-1));setUp(Vec3(1,0,0));};
  inline void setXZView(){setZ(Vec3(0,-1,0));setUp(Vec3(0,0,1));};
  inline void setZXView(){setZ(Vec3(0,1,0)); setUp(Vec3(1,0,0));};
  inline void setYZView(){setZ(Vec3(1,0,0)); setUp(Vec3(0,0,1));};
  inline void setZYView(){setZ(Vec3(-1,0,0));setUp(Vec3(0,1,0));};

  void setDefaultXYZRot();

  void PTSectionToFile(FILE *FH);
  void readPTSectionFromFile(Parser *par);


  void saveCameraToSettings();
  void loadCameraFromSettings();

  void setGLViewTransformation(double near,double far);



  

 private:
  /* view coorinate system  
   * up = worldvector which forms the view up or y-axis
   * z = worldvector, which points out of the view plane
   * mid = worldvector, which points to the center of the physical scene
   * focus = worldvector, which points to the viewing-at point, which gets
   *         mapped onto the center of the scene
   * radius = world radius, which encloses the world scene and determines
   *          the zoom together with scale
   */
  ROCPROPERTY(x,x,Vec3);
  ROCPROPERTY(up,up,Vec3);
  ROCPROPERTY(z,z,Vec3);
  CPROPERTY(focus,focus,Focus,Vec3);
  ROCPROPERTY(f,focalDistance,double);
  ROCPROPERTY(shift,origin_cam_w,Vec3);
  PROPERTY(lrim,leftRim,LeftRim,double);
  PROPERTY(rrim,rightRim,RightRim,double);
  PROPERTY(trim,upperRim,UpperRim,double);
  PROPERTY(brim,lowerRim,LowerRim,double);
  PROPERTY(perspective,isPerspective,Perspective,bool);
  ROPROPERTY(tantheta,tanTheta,double);
  ROPROPERTY(theta,theta,double);
  ROPROPERTY(N,near,double);
  ROPROPERTY(F,far,double);
  ROPROPERTY(T,top,double);
  ROPROPERTY(lambda,lambda,double);
  PROPERTY(scale,scale,Scale,double);
  PROPERTY(radius,radius,Radius,double);
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _VIEW_H */

