/*************************************************************************
 * @(#)AnnotationDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Aug 2009
 *************************************************************************
 */

#include <QMessageBox>
#include <QMouseEvent>
#include <QAction>
#include "ToolButtons.hh"
#include "AnnotationDlg.hh"
#include "Boundary.hh"
#include "DocBase.hh"
#include "HelpDlg.hh"
#include "Axes.hh"
#include "DisplayCells.hh"
#include "DisplayCellWidget.hh"
#include "Config.hh"
#include "AxisWidget.hh"
#include "Polyhedron.hh"
#include "MWBase.hh"
#include "ReselectEvent.hh"
#include "AnnotationsTreeModel.hh"
#include "ToolButtons.hh"
#include "MyDebug.hh"




/* ***********************************************************************
 *
 * ***********************************************************************
 */
AnnotationDlg::AnnotationDlg(QWidget *parent,QWidget *base,const char *name)
  :ProtoDlg(parent,base,name),currentsheet(0)
  ,mute_currentIndexChanged(true),anmodel(0)
{
  setupUi(this);

  setUp(pbAccept,pbClose,pbCancel,pbHelp,"AnnotationsDialog",": Axis/Cell settings");

  //  setWindowTitle(IDS_APP_NAME + ": Axis/Cell settings");

  

  
  Sheet *currentsheet=new Sheet(this);
  sheetLayout->insertWidget(0,currentsheet);
  






  t_displaycells=new DisplayCells();
  t_axes=new Axess();
  

  ptoolbuttons=new ToolButtons();
  ptoolbuttons->createToolButtonsActions(trvAnnotations,"cell",
					tbAdd,tbDelete);



  ptoolbuttons->connectSlots(this,SLOT(onAddActionTriggered()),
			    SLOT(onDeleteActionTriggered()));


  anmodel=new AnnotationsTreeModel(this,t_displaycells,t_axes);
  trvAnnotations->setModel(anmodel);




}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
AnnotationDlg::~AnnotationDlg()
{
  if(currentsheet) delete currentsheet; 
  delete ptoolbuttons;
  delete t_axes;
  delete t_displaycells;
  if(anmodel) delete anmodel;
}






/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::refresh(Xf::StructureType datastructurestype
			    ,Xf::LatticeType datalatticetype)
{
  NSF("AnnotationDlg::refresh");
  //myDebug() << nsubfunc;
  pdatastructurestype=datastructurestype;
  pdatalatticetype=datalatticetype;

  QModelIndex cindex=trvAnnotations->currentIndex();
  QModelIndex sindex=cindex;
  
  anmodel->reload(pdatastructurestype,pdatalatticetype);
  trvAnnotations->expandToDepth(1);

  if(cindex.isValid() && cindex.parent().isValid() ){
    if( (!cindex.flags()) ){
      sindex=anmodel->index(0,0,cindex.parent());
    }
  }

  if(sindex==cindex){
    mute_currentIndexChanged=true;
    //myDebug() << "muted";
  }else{

    //myDebug() << "currentsheet"<<currentsheet<<"being deleted";
    if(currentsheet){
      delete currentsheet;currentsheet=0;
    }
  }
  
  trvAnnotations->setCurrentIndex(sindex);
  
  mute_currentIndexChanged=false;

  return;

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::setData(AnnotationMode mode
			    ,DocBase *doc,const Symmetry *sym
			    ,Xf::StructureType datastructurestype
			    ,Xf::LatticeType datalatticetype)
{
  NSF("AnnotationDlg::setData");
  //myDebug() << nsubfunc<<mode;

  t_sym=sym;
  pdatastructurestype=datastructurestype;
  pdatalatticetype=datalatticetype;
  p_doc=doc;



  *t_displaycells=*doc->displayCells();
  t_displaycells->setChanged(false);

  *t_axes=*doc->axes();
  t_axes->setChanged(false);




  mute_currentIndexChanged=true;


  // anmodel->reload(pdatastructurestype,pdatalatticetype);
  // trvAnnotations->expandToDepth(1);
  

  // if(currentsheet){
  //   // we reload data, old sheets are invalid
  //   delete currentsheet; currentsheet=0;
  // }


 

  QModelIndex sindex;
  if(mode==AMDisplayCells){
    sindex=anmodel->cellIndex(0);
  }else if(mode==AMAxes){
    sindex=anmodel->axisIndex(0);
  }
  
  
  mute_currentIndexChanged=false;
  
  trvAnnotations->setCurrentIndex(sindex);



  refresh(pdatastructurestype,pdatalatticetype);
  
  //myDebug() << nsubfunc<<"END";
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void AnnotationDlg::getData(DocBase *doc, long &chn )
{
  NSF("AnnotationDlg::getData"); 
  chn=0;

  
  doc->displayCells()->copyFrom(t_displaycells);
  chn|=doc->displayCells()->changed();
  t_displaycells->setChanged(false);
  
  *(doc->axes())=*t_axes;
  chn|=doc->axes()->changed();
  t_axes->setChanged(false);
  
  
  if(pdatastructurestype==Xf::STMolecule){
    t_axes->axes(1)->setChanged(false);
    t_axes->axes(2)->setChanged(false);
    
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AnnotationDlg::dataAcceptable()
{
  if(currentsheet){
    if(!currentsheet->commit()) return false;
  }
  return true;
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::onDeleteActionTriggered()
{
  if(!trvAnnotations->hasFocus())return;
  QModelIndex cindex=trvAnnotations->currentIndex();
  
  bool beingcell=(cindex.internalId()==AnnotationsTreeModel::RIDisplayCell);
  bool beingincells=beingcell || 
    (cindex.internalId()==AnnotationsTreeModel::RIDisplayCells  );

  if(beingincells){
    int i=-1;
    if(beingcell){
      i=cindex.row();
      
      if(i<Xf::DCTGeneral-1){
   	i=-1;
      }
      
    }
    if(i>=0){
      
      if(currentsheet) {
   	delete currentsheet; 
   	currentsheet=0;
      }
      anmodel->removeCell(i);
    }
    
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::onAddActionTriggered()
{
  if(!trvAnnotations->hasFocus())return;
  
  QModelIndex cindex=trvAnnotations->currentIndex();
  
  bool beingcell=(cindex.internalId()==AnnotationsTreeModel::RIDisplayCell);
  bool beingincells=beingcell || 
  (cindex.internalId()==AnnotationsTreeModel::RIDisplayCells);

  if(beingincells){

    long i=Xf::DCTPrimitive;
    
    if(cindex.internalId()==AnnotationsTreeModel::RIDisplayCell){
      i=MyMath::max<long>(Xf::DCTPrimitive,cindex.row()+1);
    }      
      
    
  
    
    DisplayCell *dc=new DisplayCell(Xf::DCTGeneral,Xf::LTDirect);
    Boundary *b=dc->boundary();
    b->polyhedron()->properties().setVisible(true);
    b->setCellType(Boundary::Cell);
    b->setCellBasisType(Boundary::Conventional);
    b->setCentering(Boundary::None);
    b->setLattice(pdatalatticetype);
    b->setOriginBasis(Boundary::Conventional);
    
    
    

    anmodel->insertRow(i,dc);

    trvAnnotations->setCurrentIndex(anmodel->cellIndex(i));
    
  }
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void AnnotationDlg::setAddDeleteEnabed()
{
  QModelIndex cindex=trvAnnotations->currentIndex();
  bool beingcell=(cindex.internalId()==AnnotationsTreeModel::RIDisplayCell);
  bool beingincells=beingcell || (cindex.internalId()==AnnotationsTreeModel::RIDisplayCells);

  bool iscrystal=(pdatastructurestype==Xf::STCrystal);
  
  
  if(beingcell&&t_displaycells->cell(cindex.row())->type()==Xf::DCTGeneral){
    tbDelete->setEnabled(true);
  }else{
    tbDelete->setEnabled(false);
  }
  
  if(beingincells){
    tbAdd->setEnabled(true&iscrystal);
  }else{
    tbAdd->setEnabled(false);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::setSheet(const QModelIndex &current)
{
  //MASSERT("Oops",current);
  //myDebug() << "AnnotationDlg::setSheet" << current<<whatIsMyMaster(current) 
  //<<whatAmI(current)<<itemData(current);

  if(!current.isValid()) return;

  //myDebug()<<"setSheet";
  
  if(currentsheet) {
    delete currentsheet; 
    currentsheet=0;
  }

  if(current.internalId() == AnnotationsTreeModel::RIAxis){
    AxisWidget *axw=qobject_cast<AxisWidget *>(currentsheet);
    if(!currentsheet){
      //myDebug() << "creating AxisWidget";
      currentsheet=axw=new AxisWidget(this);
      sheetLayout->insertWidget(0,currentsheet);
    }else{
      //myDebug() << "using AxisWidget";
    }
    Axes *ax=t_axes->axes(current.row());
    Axess *axess=p_doc->axes();
    axw->connect(anmodel,SIGNAL(checkStateChanged(const Axes *))
	    ,SLOT(onActiveToggled(const Axes *)));
    axw->setData(ax,axess,dynamic_cast<MWBase*>(mainWidget())->oglView());
    return ;
  }else if(current.internalId() == AnnotationsTreeModel::RIDisplayCell){
    DisplayCellWidget *dcw=qobject_cast<DisplayCellWidget *>(currentsheet);
    if(!currentsheet){
      currentsheet=dcw=new DisplayCellWidget(this);
      sheetLayout->insertWidget(0,currentsheet);
    }
    dcw->connect(anmodel,SIGNAL(checkStateChanged(const DisplayCell *))
	    ,SLOT(onActiveToggled(const DisplayCell *)));
    dcw->setData(t_displaycells->cell(current.row()),t_sym);
    return ;
    
  }

  if(!currentsheet){
    // fallback
    currentsheet=new Sheet(this);
    sheetLayout->insertWidget(0,currentsheet);
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::refreshAxesActive(Xf::AxesType axestype)
{

  int pos=t_axes->indexOf(t_axes->findByType(axestype));
  anmodel->setData(anmodel->axisIndex(pos),
		   p_doc->axes()->findByType(axestype)->isVisible(),
	  Qt::CheckStateRole);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::refreshDisplayCellsActive(Xf::DisplayCellType ctype,bool on)
{
  NSF("AnnotationDlg::refreshDisplayCellsActive");
  //myDebug()<<nsubfunc<<ctype<<on;

  int pos=t_displaycells->indexOf(t_displaycells->findByType(ctype));
  anmodel->setData(anmodel->cellIndex(pos),
		   on,Qt::CheckStateRole);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AnnotationDlg::event(QEvent *event)
{
  NSF("AnnotationDlg::event");
  //myDebug() <<nsubfunc ;
  if(event->type()==ReselectEvent::ReselectEventType){
    if(ReselectEvent *re=dynamic_cast<ReselectEvent*>(event)){
      mute_currentIndexChanged=true;
      //myDebug() <<nsubfunc<<re->item() ;
      trvAnnotations->setCurrentIndex(re->index());
      mute_currentIndexChanged=false;
    }
    return true;
  }
  return BrainDialog::event(event);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AnnotationDlg::on_trvAnnotations_myCurrentChanged(const QModelIndex &current,
						     const QModelIndex &previous)
{
  //myDebug()<<"on_trvAnnotations_myCurrentChanged";
  setAddDeleteEnabed();

  if(mute_currentIndexChanged) return;

  //myDebug()<<"notmuted";

  if(currentsheet) {
    //myDebug()<<"have currentsheet";
    //myDebug() <<nsubfunc << "commit";
    if(!currentsheet->commit()){
      QEvent *event=new ReselectEvent(ReselectEvent::ReselectEventType,previous);
      QCoreApplication::postEvent(this,event);
      return;
    }
  }
  
  setSheet(current);

}



/*EOF*/ 
