/*************************************************************************
 * @(#)AtomPropsDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Mar 2012
 *************************************************************************
 */

#include "AtomPropsDlg.hh"
#include "AtomPropsModel.hh"
#include "Delegates.hh"
#include "PropertyTable.hh"
#include "FString.hh"
#include "STDoc.hh"
#include "Symmetry.hh"
#include "MyDebug.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomPropsDlg::AtomPropsDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
  ,t_ptables(0),apmodel(0),mute_cmbThemeEdit(false),mute_cmbThemeIndex(false)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"AtomPropsDialog",": Atom properties");


  t_ptables=new AtomPropertyTables();
  apmodel = new AtomPropsModel(this,t_ptables->current(),0);
  tvProps->setModel(apmodel);
  tvProps->setColumnHidden (AtomPropsModel::Visible,true );

  colordelegate=new ColorDelegate();
  speccolordelegate=new ColorDelegate();
  radtypdelegate=new ComboDelegate(0,&AtomicRadii::radiinames);

  tvProps->setItemDelegateForColumn(AtomPropsModel::AtomColor,colordelegate);
  tvProps->setItemDelegateForColumn(AtomPropsModel::SpecularColor,speccolordelegate);
  tvProps->setItemDelegateForColumn(AtomPropsModel::RadiusType,radtypdelegate);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomPropsDlg::~AtomPropsDlg()
{
  if(t_ptables) delete t_ptables;
  if(apmodel) delete apmodel; apmodel=0;
  if(colordelegate) delete colordelegate;colordelegate=0;
  if(speccolordelegate) delete speccolordelegate;speccolordelegate=0;
  if(radtypdelegate) delete radtypdelegate;radtypdelegate=0;



}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::setData(STDoc *stdoc,const AtomPropertyTables *ptables)
{
  pstdoc=stdoc;
  


  *t_ptables=*ptables;
  //  t_ptable->setChanged(false);
  mute_cmbThemeIndex=true;
  mute_cmbThemeEdit=true;
  cmbTheme->clear();
  const StringList &themes=t_ptables->themes();
  for(int i=0;i<themes.size();i++){
    cmbTheme->insertItem(i,themes.at(i)());
  }
  mute_cmbThemeEdit=false;
  mute_cmbThemeIndex=false;
  //myDebug()<<"current"<<ptables->current()->theme()<<t_ptables->currentIndex();

  cmbTheme->setCurrentIndex(t_ptables->currentIndex());

  apmodel->reload(t_ptables->current());

  
  tvProps->setCurrentIndex(apmodel->index(0,0,QModelIndex()));

  adjustTvProps();
  //apmodel->setSymmetry(psym);

  setTbDeleteEnabled();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::getData(STDoc *,AtomPropertyTables *ptables)
{
  if(ptables){
    *ptables=*t_ptables;
    ptables->saveSettings();
  }
  
  adjustTvProps();

  //todo: take the colors over

  //  t_ptable->setChanged(false);
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::adjustTvProps()
{
  int rows=apmodel->rowCount();
  int cols=apmodel->columnCount();
  for(int irow=0;irow<rows;irow++)
    tvProps->resizeRowToContents(irow);
  for(int icol=0;icol<cols;icol++){
    tvProps->resizeColumnToContents(icol);
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_tvProps_clicked(const QModelIndex &index)
{
  if(!index.isValid()) return;
  
  if(!(index.flags()&Qt::ItemIsEnabled))return;
  if((index.column()==AtomPropsModel::AtomColor)||
     (index.column()==AtomPropsModel::SpecularColor)){
    tvProps->edit(index);
  }else{
    return;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_pbLoadDefault_clicked()
{
  t_ptables->current()->loadDefault();
  apmodel->reload(t_ptables->current());
  adjustTvProps();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_cmbTheme_currentIndexChanged(int index)
{
  if(mute_cmbThemeIndex) return;
  //  myDebug()<<index<<cmbTheme->currentText();
  t_ptables->setCurrentIndex(index);
  apmodel->reload(t_ptables->current());

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_cmbTheme_editTextChanged(const QString &text)
{
  if(mute_cmbThemeEdit) return;
  cmbTheme->setItemText(cmbTheme->currentIndex(),text);
  (*t_ptables)[cmbTheme->currentIndex()]->setTheme(text);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_tbAdd_clicked()
{
  //mute_cmbThemeEdit=true;
  int ind=cmbTheme->currentIndex();
  t_ptables->insertNewAt(ind+1,"new",t_ptables->at(ind));

  cmbTheme->insertItem(ind+1,"new");
  cmbTheme->setCurrentIndex(ind+1);

  setTbDeleteEnabled();

  //mute_cmbThemeEdit=false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_tbDelete_clicked()
{
  int ind=cmbTheme->currentIndex();
  if(ind>0){
    t_ptables->removeAt(ind);
    cmbTheme->removeItem(ind);
  }
  setTbDeleteEnabled();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::setTbDeleteEnabled()
{
  if(cmbTheme->count()>1)
    tbDelete->setEnabled(true);
  else
    tbDelete->setEnabled(false);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::on_pbApplyTheme_clicked()
{
  pstdoc->symmetry->applyTheme(t_ptables->current());
  emit talkBack(Accept);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomPropsDlg::selectElement(const String &element)
{
  int i=t_ptables->current()->indexOf(element);
  tvProps->setCurrentIndex(apmodel->index(i,0,QModelIndex()));
}

/*EOF*/ 
