/*************************************************************************
 * @(#)AtomSequenceDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   08 Sep 2009
 *************************************************************************
 */

#include <QMessageBox>
#include "AtomSequenceDlg.hh"
#include "Parser.hh"
#include "Except.hh"
#include "Symmetry.hh"
#include "Assorted.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"

using namespace XFPLO;
 
/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomSequenceDlg::AtomSequenceDlg(QWidget *parent
				 ,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);

  connect(leFile,SIGNAL(textChanged(QString))
	  ,this,SLOT(loadSequence(QString)));

  connect(spbxFrame,SIGNAL(valueChanged(int))
	  ,this,SLOT(onValueChanged(int)));
  
 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
AtomSequenceDlg::~AtomSequenceDlg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomSequenceDlg::accept()
{
  //int i;

  
  
//   if(txt.length()){
//     QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
//   }else{
  QDialog::accept();
//   }
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomSequenceDlg::loadSequence(QString)
{
  NSF("AtomSequenceDlg::loadSequence");
  myDebug() <<nsubfunc;

  ::Parser *par=new ::Parser(1);

  String fname=leFile->text().toAscii().constData();
  if(pathExistence(fname()) == DoesNotExist){
    QMessageBox::warning(this,"Open"
			 ,String().sprintf("File \'%s\' does not exist!"
					   ,fname())()
			 ,QMessageBox::Yes);
    return;
  }
  par->parseFile(fname());
  
  aseq.clear();
  try{

    Dobj dw=(*par)()("wyck_history");
    long nsort=dw.sapTemplate()("atoms").size();
    //     myDebug() << "nhistory="<<dw.size();
    //     myDebug() << "nsort   ="<<nsort;
    for(long ih=0;ih<dw.size();ih++){
      aseq.append(ValueArray<WyckPos>());
      Dobj d=dw[ih]("atoms");
      for(long isort=0;isort<nsort;isort++){
	Dobj da=d[isort];
	// 	myDebug() << da("el")()()
	// 		 << da("tau")[0]()()
	// 		 << da("tau")[1]()()
	// 		 << da("tau")[2]()();
	aseq.last().append(WyckPos(da("el")().removeTrailingBlanks()
				   ,da("tau")[0]()
				   ,da("tau")[1]()
				   ,da("tau")[2](),AtomProperty(),true));
	aseq.last().last().setProperty(globalproperties->current()
				       ->find(aseq.last().last().el()));
      }
    }
  }catch(Except e){
    QMessageBox::warning(this,"Open"
			 ,String().sprintf("Problems reading file \'%s\':"
					   "\n%s!"
					   ,fname(),e.msg())()
			 ,QMessageBox::Yes);
  };
  spbxFrame->setMaximum(aseq.size());
  spbxFrame->setValue(1);
  onValueChanged(1);
  
  delete par;

  
}
  
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomSequenceDlg::setData(Symmetry *sym)
{
  psym=sym;
  leFile->setText("=.site_history");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AtomSequenceDlg::onValueChanged(int i)
{
  NSF("AtomSequenceDlg::onValueChanged");
  myDebug() << nsubfunc << i;

  psym->setWyckoffPositions(aseq.at(i-1));
  emit atomSequenceChanged();
  
}
  
/*EOF*/ 
  
