/*************************************************************************
 * @(#)AxisEdit.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Mar 2010
 *************************************************************************
 */

#include <QMessageBox>
#include "AxisEdit.hh"
#include "Except.hh"
#include "DMConfig.hh"
#include "DMDoc.hh"
#include "Settings.hh"
#include "qt5compat.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AxisEdit::AxisEdit(QWidget *parent) 
  : QDialog(parent),currentioccu(-1),currentaxes(0)
{
  setupUi(this);

  tedPage->setReadOnly ( true ) ;


  connect(leX1,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leX2,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leX3,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));
  
  connect(leZ1,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leZ2,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leZ3,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  
  Settings set("AxisEdit");
  resize(set.getSize("Geometry/Size",QSize(360, 340)));
  move(set.getPoint("Geometry/Pos",QPoint(200, 200)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AxisEdit::~AxisEdit() 
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::saveMainSettings()
{
  Settings set("AxisEdit");

  set.set("Geometry/Size", size());
  set.set("Geometry/Pos", pos());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::on_pbQuit_clicked()
{
  //myDebug() << "AxisEdit::on_pbQuit_clicked";

  try{
    checkData();
  }catch(Except &ex){
    QMessageBox::warning(0,"Checking data"
			 ,ex.msg(),QMessageBox::Yes );
    
    return;
  }
  

  if(setLedPageText(true)){
    
    {// save axis back
      if(currentioccu>=0 && currentioccu < dmdoc->occus().size()) {
	Occu &oc=dmdoc->occus()[currentioccu];
	//myDebug() << oc.name()();
	DMQuantAxis ax(x,z);
	int ind;
	if((ind=oc.localaxes.indexOf(ax))>=0){
	  currentaxes=ind;
	}else{
	  //myDebug() << "new";
	  oc.localaxes.append(ax);
	  currentaxes=oc.localaxes.size()-1;
	}
      }
    }

    /*
      for(int i=0;i<dmdoc->occus().size();i++){
      const Occu &oc=dmdoc->occus().at(i);
      for(int j=0;j<oc.localaxes.size();j++){
      printf("%s: %d: x=%s z=%s\n"
      ,oc.name()(),j
      ,oc.localaxes.at(j).xaxis.printS()()
      ,oc.localaxes.at(j).zaxis.printS()()
      );
      
      }
      }
    */    

    saveMainSettings();
    QDialog::accept();
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::reject()
{
  //myDebug() << "AxisEdit::reject";
  saveMainSettings();
  QDialog::reject();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void AxisEdit::setData(DMDoc *dm,long ioccu,long iaxes)
{
  dmdoc=dm;
  if(ioccu<0 || ioccu> dmdoc->occus().size()-1) return ;
  currentioccu=ioccu;
  const Occu &oc=dmdoc->occus().at(currentioccu);
  if(iaxes<0 || iaxes> oc.localaxes.size()-1) return ;
  currentaxes=iaxes;
  
  pl=oc.l();


  const DMQuantAxis &ax=oc.localaxes.at(currentaxes);
  leX1->setText(QString("%1").arg(ax.xaxis[0]));
  leX2->setText(QString("%1").arg(ax.xaxis[1]));
  leX3->setText(QString("%1").arg(ax.xaxis[2]));

  leZ1->setText(QString("%1").arg(ax.zaxis[0]));
  leZ2->setText(QString("%1").arg(ax.zaxis[1]));
  leZ3->setText(QString("%1").arg(ax.zaxis[2]));


  cmbAxes->clear();

  int currentindex=-1;
  int ind=0;
  for(int i=0;i<dmdoc->occus().size();i++){
    const Occu &oc=dmdoc->occus().at(i);
    for(int j=0;j<oc.localaxes.size();j++){
      QList<QVariant> l;
      l<<i<<j;
      cmbAxes->addItem(
		       Sprintf("%s: %d: x=%s z=%s"
			      ,oc.name()(),j
			      ,oc.localaxes.at(j).xaxis.printS()()
			      ,oc.localaxes.at(j).zaxis.printS()()
			)()
		       ,l);
      if(i==currentioccu && j==currentaxes) currentindex=ind;
      ind++;
    }
  }
  //myDebug() << currentindex;
  cmbAxes->setCurrentIndex(currentindex);


  setLedPageText(true);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::getData()
{
  if(currentioccu<0 || currentioccu > dmdoc->occus().size()-1) return;
  Occu &oc=dmdoc->occus()[currentioccu];
  oc.setCurrentAxes(currentaxes);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::on_cmbAxes_currentIndexChanged(int index)
{
  if(index==-1) return;
  QList<QVariant> l=cmbAxes->itemData(index).toList();

  int io=l[0].toInt();
  int ja=l[1].toInt();
  if(io<0 || io> dmdoc->occus().size()-1) return;
  const Occu &oc=dmdoc->occus().at(io);
  if(ja<0 || ja > oc.localaxes.size()-1) return;

  Vec3 x=oc.localaxes.at(ja).xaxis;
  Vec3 z=oc.localaxes.at(ja).zaxis;
  
  //myDebug() << oc.name()() << x.printS()()<< z.printS()();
  

  leX1->setText(QString("%1").arg(x[0]));
  leX2->setText(QString("%1").arg(x[1]));
  leX3->setText(QString("%1").arg(x[2]));

  leZ1->setText(QString("%1").arg(z[0]));
  leZ2->setText(QString("%1").arg(z[1]));
  leZ3->setText(QString("%1").arg(z[2]));

  
  setLedPageText(false);  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AxisEdit::setLedPageText(bool betight)
{
  //myDebug() << "AxisEdit::setLedPageText";

  try{ checkData(); }catch(Except &ex){ return false;}
  
  tedPage->clear();
  
  StringList sl;
  try{
    SqMatrix<Cmplx> ytr;
    sl=DMQuantAxis::calcAxesTrans(x,z,pl,ytr);
  }catch(Except &ex){ 
    tedPage->setTextColor(Qt::red);
    tedPage->append(ex.msg());
    QTextCursor cursor=tedPage->textCursor();
    cursor.setPosition(0);
    tedPage->setTextCursor(cursor);
    
    if(betight){
      QMessageBox::warning(0,"Checking data"
			   ,ex.msg(),QMessageBox::Yes );
    }
    return false;
  }
  
  
  tedPage->setTextColor(Qt::red);
  tedPage->append(sl[1]());
  tedPage->setTextColor(Qt::black);
  tedPage->append(sl[0]());

  QTextCursor cursor=tedPage->textCursor();
  cursor.setPosition(0);
  tedPage->setTextCursor(cursor);

  return true;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisEdit::checkData()
{
  double d;
  String s;

  bool ok=true;
  String msg;
  
  s=leX1->text().toAscii().constData();
  try{ d=s.convertToFortranReal();  x[0]=d; }catch(Except &ex){ 
    msg+="X1 does not contain a valid real number\n";
    ok=false;
  };


  s=leX2->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); x[1]=d; }catch(Except &ex){ 
    msg+="X2 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leX3->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); x[2]=d;}catch(Except &ex){ 
    msg+="X3 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leZ1->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[0]=d;}catch(Except &ex){ 
    msg+="Z1 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leZ2->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[1]=d;}catch(Except &ex){ 
    msg+="Z2 does not contain a valid real number\n";
    ok=false;
  };


  s=leZ3->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[2]=d;}catch(Except &ex){ 
    msg+="Z3 does not contain a valid real number\n";
    ok=false;
  };
  

  if(!ok){
    throw Except(msg);
  }

  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void AxisEdit::onLedPageTextEdited(const QString &)
{
  setLedPageText(false);
}


/*EOF*/ 
