/*************************************************************************
 * @(#)AxisWidget.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Aug 2009
 *************************************************************************
 */

#include <QMenu> 
#include <QMessageBox> 
#include "AxisWidget.hh"
#include "Axes.hh"
#include "OGLView.hh"
#include "MyDebug.hh"
#include "Action.hh"
#include "Config.hh"
#include "Valid.hh"


#define ALL    0
#define LABELS 1
#define ORIGIN 2
#define AXES   3
#define ROD    4
#define TIP    5

/* ***********************************************************************
 *
 * ***********************************************************************
 */
AxisWidget::AxisWidget(QWidget *parent)
  :Sheet(parent),t_axes(0),paxes(0),poglview(0)
{
  setupUi(this);

  setObjectName("AxisWidget");

  QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
  sizePolicy1.setHorizontalStretch(3);
  sizePolicy1.setVerticalStretch(0);
  sizePolicy1.setHeightForWidth(sizePolicy().hasHeightForWidth());
  setSizePolicy(sizePolicy1);

  const char *text1="set for &all";
  tbXColor->addAction(new Action(text1,tbXColor,0));
  tbYColor->addAction(new Action(text1,tbYColor,0));
  tbZColor->addAction(new Action(text1,tbZColor,0));
  const char *text2="&set default";
  tbXColor->addAction(new Action(text2,tbXColor,1));
  tbYColor->addAction(new Action(text2,tbYColor,1));
  tbZColor->addAction(new Action(text2,tbZColor,1));

  const char *text3="set &default for all";
  tbXColor->addAction(new Action(text3,tbXColor,2));
  tbYColor->addAction(new Action(text3,tbYColor,2));
  tbZColor->addAction(new Action(text3,tbZColor,2));

  connect(tbXColor,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onColorActionTriggered(QAction *)));
  connect(tbYColor,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onColorActionTriggered(QAction *)));
  connect(tbZColor,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onColorActionTriggered(QAction *)));


  tbDefault->addAction(new Action("&all",tbDefault,ALL));
  tbDefault->addAction(new Action("&labels",tbDefault,LABELS));
  tbDefault->addAction(new Action("&origin",tbDefault,ORIGIN));
  tbDefault->addAction(new Action("a&xes",tbDefault,AXES));
  tbDefault->addAction(new Action("&rod",tbDefault,ROD));
  tbDefault->addAction(new Action("&tip",tbDefault,TIP));
  

  connect(tbDefault,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onDefaultActionTriggered(QAction *)));

  
  
  
  rodRHelper
    =new SUCH(cmbRodUnits,
	      ValueArray<SUCH::Type>() 
	      <<SUCH::Type(Xf::SUBoundary,
			   Xf::sceeneUnitName(Xf::SUBoundary))
	      <<SUCH::Type(Xf::SUWorld,
			   Xf::sceeneUnitName(Xf::SUWorld))
	      <<SUCH::Type(Xf::SUAngstroem,
			   Xf::sceeneUnitName(Xf::SUAngstroem))
	      <<SUCH::Type(Xf::SUBohrRadius,
			   Xf::sceeneUnitName(Xf::SUBohrRadius))
	      );
  
  tipRHelper=new SUCH(cmbTipRadiusUnits,
		      ValueArray<SUCH::Type>() 
		      <<SUCH::Type(Xf::SURodRadius,
				   Xf::sceeneUnitName(Xf::SURodRadius))
		      <<SUCH::Type(Xf::SUBoundary,
				   Xf::sceeneUnitName(Xf::SUBoundary))
		      <<SUCH::Type(Xf::SUWorld,
				   Xf::sceeneUnitName(Xf::SUWorld))
		      <<SUCH::Type(Xf::SUAngstroem,
				   Xf::sceeneUnitName(Xf::SUAngstroem))
		      <<SUCH::Type(Xf::SUBohrRadius,
				   Xf::sceeneUnitName(Xf::SUBohrRadius))
		      );
  tipLHelper=new SUCH(cmbTipLengthUnits,
		      ValueArray<SUCH::Type>() 
		      <<SUCH::Type(Xf::SUTipRadius,
				   Xf::sceeneUnitName(Xf::SUTipRadius))
		      <<SUCH::Type(Xf::SURodRadius,
				   Xf::sceeneUnitName(Xf::SURodRadius))
		      <<SUCH::Type(Xf::SUBoundary,
				   Xf::sceeneUnitName(Xf::SUBoundary))
		      <<SUCH::Type(Xf::SUWorld,
				   Xf::sceeneUnitName(Xf::SUWorld))
		      <<SUCH::Type(Xf::SUAngstroem,
				   Xf::sceeneUnitName(Xf::SUAngstroem))
		      <<SUCH::Type(Xf::SUBohrRadius,
				   Xf::sceeneUnitName(Xf::SUBohrRadius))
		      );

  axesLHelper=new SUCH(cmbLengthUnit,
		       ValueArray<SUCH::Type>() 
		       <<SUCH::Type(Xf::SUBoundary,
				    Xf::sceeneUnitName(Xf::SUBoundary))
		       <<SUCH::Type(Xf::SUWorld,
				    Xf::sceeneUnitName(Xf::SUWorld))
		       <<SUCH::Type(Xf::SUAngstroem,
				    Xf::sceeneUnitName(Xf::SUAngstroem))
		       <<SUCH::Type(Xf::SUBohrRadius,
				    Xf::sceeneUnitName(Xf::SUBohrRadius))
		       );
  originHelper=new SUCH(cmbOriginUnit,
			ValueArray<SUCH::Type>() 
			<<SUCH::Type(Xf::SUWorld,
				     Xf::sceeneUnitName(Xf::SUWorld))
			<<SUCH::Type(Xf::SUConvCell,
				     Xf::sceeneUnitName(Xf::SUConvCell))
			<<SUCH::Type(Xf::SUPrimCell,
				     Xf::sceeneUnitName(Xf::SUPrimCell))
			<<SUCH::Type(Xf::SUAngstroem,
				     Xf::sceeneUnitName(Xf::SUAngstroem))
			<<SUCH::Type(Xf::SUBohrRadius,
				     Xf::sceeneUnitName(Xf::SUBohrRadius))
			);
  
  
  rodRHelper->populate();
  tipRHelper->populate();
  tipLHelper->populate();
  axesLHelper->populate();
  originHelper->populate();






  new RatValidator(this,leXLength,"The x-axis length ",1.e-10,1.e10);
  new RatValidator(this,leYLength,"The y-axis length ",1.e-10,1.e10);
  new RatValidator(this,leZLength,"The Z-axis length ",1.e-10,1.e10);
  new RatValidator(this,leRodRadius,"The rod radius ",1.e-10,1.e10);
  new RatValidator(this,leTipRadius,"The tip radius ",1.e-10,1.e10);
  new RatValidator(this,leTipLength,"The tip length ",1.e-10,1.e10);

  new RatValidator(this,leXOrigin,"The origin x-component ",-1.e10,1.e10);
  new RatValidator(this,leYOrigin,"The origin y-component ",-1.e10,1.e10);
  new RatValidator(this,leZOrigin,"The origin z-component ",-1.e10,1.e10);

  connectChanged(this);
  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
AxisWidget::~AxisWidget()
{
  delete rodRHelper;
  delete tipRHelper;
  delete tipLHelper;
  delete axesLHelper;
  delete originHelper;

  if(pbMove->isChecked()){
    pbMove->setChecked(false);
    on_pbMove_clicked();
  }
  pbMove->setEnabled(t_axes->isVisible());

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::onDefaultActionTriggered(QAction *a)
{

  //QToolButton *tb=qobject_cast<QToolButton *>(a->parentWidget());
  if(a->data().toInt()==ALL){


    t_axes->setDefaultLabel();    
    setUpLabels();    

    t_axes->setDefaultRod();    
    setUpRod();

    t_axes->setDefaultTip();    
    setUpTip();
    
    
    t_axes->setDefaultAxes();    
    setUpAxes();
    

    t_axes->setDefaultOrigin();    
    setUpOrigin();
    
  }else if(a->data().toInt()==ORIGIN){
    t_axes->setDefaultOrigin();    
    setUpOrigin();
  }else if(a->data().toInt()==LABELS){
    t_axes->setDefaultLabel();    
    setUpLabels();    
    
  }else if(a->data().toInt()==AXES){
    t_axes->setDefaultAxes();    
    setUpAxes();
    
  }else if(a->data().toInt()==ROD){
    t_axes->setDefaultRod();    
    setUpRod();    
  }else if(a->data().toInt()==TIP){
    t_axes->setDefaultTip();    
    setUpTip();
    
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::onColorActionTriggered(QAction *a)
{

  ColorButton *cb=qobject_cast<ColorButton *>(a->parentWidget());
  if(a->data().toInt()==0){
    QRgb rgb=cb->color();
    tbXColor->setColor(rgb);
    tbYColor->setColor(rgb);
    tbZColor->setColor(rgb);
  }else if(a->data().toInt()==1){
    if(cb==tbXColor) tbXColor->setColor(t_axes->defaultColor(0));
    if(cb==tbYColor) tbYColor->setColor(t_axes->defaultColor(1));
    if(cb==tbZColor) tbZColor->setColor(t_axes->defaultColor(2));
    
  }else if(a->data().toInt()==2){
    tbXColor->setColor(t_axes->defaultColor(0));
    tbYColor->setColor(t_axes->defaultColor(1));
    tbZColor->setColor(t_axes->defaultColor(2));
    
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AxisWidget::commit()
{
  //myDebug() << "AxisWidget::commit";

  if(!t_axes) return true;
  if(!isValid()) return false;

  QString t;

  //t_axes->setVisible(cbActive->isChecked());


  t_axes->setColor(0,tbXColor->color());
  t_axes->setColor(1,tbYColor->color());
  t_axes->setColor(2,tbZColor->color());

  if(t_axes->axesType()==Xf::ATCartesian){
    t=leXLength->text(); IntValidator::stripAllWhiteSpace(t);
    t_axes->setAxisLength(0,t);
    
    t=leYLength->text(); IntValidator::stripAllWhiteSpace(t);
    t_axes->setAxisLength(1,t);
    
    t=leZLength->text(); IntValidator::stripAllWhiteSpace(t);
    t_axes->setAxisLength(2,t);
  }
  t=leXOrigin->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setAxisOrigin(0,t);
  
  t=leYOrigin->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setAxisOrigin(1,t);
  
  t=leZOrigin->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setAxisOrigin(2,t);

  t=leRodRadius->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setRodRadius(t);

  t=leTipRadius->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setTipRadius(t);

  t=leTipLength->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setTipLength(t);

  t=leLabelX->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setLabel(0,t);
  t=leLabelY->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setLabel(1,t);
  t=leLabelZ->text(); IntValidator::stripAllWhiteSpace(t);
  t_axes->setLabel(2,t);

  SReal3 rr=t_axes->fontShift(0);
  rr[2]=dspbxZOffsetX->value();
  t_axes->setFontShift(0,rr);

  rr=t_axes->fontShift(1);
  rr[2]=dspbxZOffsetY->value();
  t_axes->setFontShift(1,rr);

  rr=t_axes->fontShift(2);
  rr[2]=dspbxZOffsetZ->value();
  t_axes->setFontShift(2,rr);

  t_axes->setFontColor(cbFontColor->color());
  

  t_axes->setFontScale(dbspbxFontScale->value());

  t_axes->setRodRadiusUnits(rodRHelper->data());

  t_axes->setTipRadiusUnits(tipRHelper->data());

  t_axes->setTipLengthUnits(tipLHelper->data());
  
  if(t_axes->axesType()==Xf::ATCartesian){
    t_axes->setAxesLengthUnits(axesLHelper->data());
  }
  t_axes->setOriginUnits(originHelper->data());
  
  
  t_axes->setHasTip(grpbxTip->isChecked());

  //t_axes->print();
  

  //  myDebug() << "END AxisWidget::commit";

  Sheet::commit();
  return true;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setData(Axes *axes,Axess *axess,OGLView *oglview)
{
  paxes=axess;
  poglview=oglview;

  connect(poglview,SIGNAL(textShiftChanged(const String&,const Vec3&)),this
	  ,SLOT(onTextShiftChanged(const String&,const Vec3&)));

  //  myDebug() << "AxisWidget::setData";
  t_axes=axes;
  //    t_axes->print();
  //  cbActive->setChecked(t_axes->isVisible());

  

  setUpLabels();

  setUpAxes();

  setUpOrigin();
  
  setUpTip();

  setUpRod();



  
  if(t_axes->axesType()!=Xf::ATCartesian){
    cmbLengthUnit->hide();
    leXLength->hide();
    leYLength->hide();
    leZLength->hide();
    lLength->hide();
  }


    
  //  myDebug() << "END AxisWidget::setData";

  onActiveToggled(t_axes);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool AxisWidget::isValid()
{
  //  myDebug() << "AxisWidget::isValid";
  if(!t_axes) return true;

  QString s,txt="";

  if(t_axes->axesType()==Xf::ATCartesian){
    
    ((RatValidator*)leXLength->validator())->check(txt);
    ((RatValidator*)leYLength->validator())->check(txt);
    ((RatValidator*)leZLength->validator())->check(txt);
  }

  ((RatValidator*)leRodRadius->validator())->check(txt);
  ((RatValidator*)leTipRadius->validator())->check(txt);
  ((RatValidator*)leTipLength->validator())->check(txt);

  ((RatValidator*)leXOrigin->validator())->check(txt);
  ((RatValidator*)leYOrigin->validator())->check(txt);
  ((RatValidator*)leZOrigin->validator())->check(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;

    return false;
  }
  
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::onActiveToggled(const Axes *ax)
{
  if(!ax || (ax!=t_axes)) return;
  if(pbMove->isChecked()){
    pbMove->setChecked(false);
    on_pbMove_clicked();
  }
  pbMove->setEnabled(t_axes->isVisible());
  enableWidgets(t_axes->isVisible());
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::enableWidgets(bool on)
{
  grpbxLabel->setEnabled(on);
  grpbxAxis->setEnabled(on);
  grpbxRod->setEnabled(on);
  grpbxTip->setEnabled(on);
  tbDefault->setEnabled(on);
 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::on_pbMove_clicked()
{
  if(!poglview) return;

  if(pbMove->isChecked()){
    enableWidgets(false);
    poglview->setAllowedMoveObjects("OGLText",t_axes->id());
  }else{
    enableWidgets(true);
    poglview->setAllowedMoveObjects("","");
  }
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::onTextShiftChanged(const String &id,const Vec3 &v)
{
  StringList sl=id.split(" ");
  int ip=sl[2].convertToULong();
  myDebug() << ip<<v;
  

  SReal3 rr=t_axes->fontShift(ip);
  rr[0]=v[0];
  rr[1]=v[1];
  t_axes->setFontShift(ip,rr);
  Axes *ax=paxes->findByID(id);
  if(!ax) return;
  if(ax) ax->setFontShift(ip,rr);



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setUpLabels()
{
  leLabelX->setText(t_axes->label(0)());
  leLabelY->setText(t_axes->label(1)());
  leLabelZ->setText(t_axes->label(2)());
  
  dspbxZOffsetX->setValue(t_axes->fontShift(0)[2].D);
  dspbxZOffsetY->setValue(t_axes->fontShift(1)[2].D);
  dspbxZOffsetZ->setValue(t_axes->fontShift(2)[2].D);
  
  cbFontColor->setColor(t_axes->fontColor());
  
  dbspbxFontScale->setValue(t_axes->fontScale().D);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setUpRod()
{
  leRodRadius->setText(t_axes->rodRadius().S());
  rodRHelper->setIndexFromData(t_axes->rodRadiusUnits());

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setUpTip()
{
  leTipRadius->setText(t_axes->tipRadius().S());
  leTipLength->setText(t_axes->tipLength().S());
  tipRHelper->setIndexFromData(t_axes->tipRadiusUnits());
  tipLHelper->setIndexFromData(t_axes->tipLengthUnits());
  
  grpbxTip->setChecked(t_axes->hasTip());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setUpOrigin()
{
  leXOrigin->setText(t_axes->axisOrigin(0).S());
  leYOrigin->setText(t_axes->axisOrigin(1).S());
  leZOrigin->setText(t_axes->axisOrigin(2).S());
  originHelper->setIndexFromData(t_axes->originUnits());
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void AxisWidget::setUpAxes()
{
  tbXColor->setColor(t_axes->color(0));
  tbYColor->setColor(t_axes->color(1));
  tbZColor->setColor(t_axes->color(2));

  if(t_axes->axesType()==Xf::ATCartesian){
    leXLength->setText(t_axes->axisLength(0).S());
    leYLength->setText(t_axes->axisLength(1).S());
    leZLength->setText(t_axes->axisLength(2).S());
    axesLHelper->setIndexFromData(t_axes->axesLengthUnits());
  }  
}
/*EOF*/ 
