/*************************************************************************
 * @(#)BondPolyDlg.cpp
 * author: magru@unas
 * date:   19 Aug 2013
 *************************************************************************
 */

#include <QMessageBox>
#include <QDebug>
#include "BondPolyDlg.hh"
#include "BondModel.hh"
#include "HelpDlg.hh"
#include "MyDebug.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondPolyDlg::BondPolyDlg(QWidget *parent,const char *name,QVariant v)
  : QDialog(parent)
{
  setWindowModality(Qt::WindowModal);
  setObjectName(name);
  setupUi(this);
  setWindowTitle(IDS_APP_NAME + ": Bond/Poly propperties");

  ValueArray<String> icons;
  icons<<":/Icons/cutmodeb"
       <<":/Icons/cutmodem"
       <<":/Icons/cutmodei"
       <<":/Icons/cutmodel";

  cutoffmodeHelper=new CMCH(cmbAtomBondVisibility,
			    BondDefGroupEnum::cutoffModeNames(),icons);

  rimRHelper=new RUCH(cmbPolyRimUnits,
		      ValueArray<RUCH::Type>() 
		      <<RUCH::Type(Xf::SUBoundary,
				   Xf::sceeneUnitName(Xf::SUBoundary))
		      <<RUCH::Type(Xf::SUWorld,
				   Xf::sceeneUnitName(Xf::SUWorld))
		      <<RUCH::Type(Xf::SUAngstroem,
				   Xf::sceeneUnitName(Xf::SUAngstroem))
		      <<RUCH::Type(Xf::SUBohrRadius,
				   Xf::sceeneUnitName(Xf::SUBohrRadius))
		      );

  cutoffmodeHelper->populate();
  rimRHelper->populate();

  setData(v);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondPolyDlg::~BondPolyDlg()
{
  delete cutoffmodeHelper;
  delete rimRHelper;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondPolyDlg::setData(QVariant v)
{
  BondModel::PrimaryAtomData *data=fromQVariant<BondModel::PrimaryAtomData>(v);
  polyprops=*(data->polyprops);
  pmode=data->mode;
  delete data;

  gbPolygon->setChecked(polyprops.isVisible());
  gbPolyFaces->setChecked(polyprops.drawFaces());
  cbPolyFaceColor->setColor(polyprops.faceMaterial().color());
  tbFaceSpecular->setColor(polyprops.faceMaterial().specularColor());
  dspbxFaceShine->setValue(polyprops.faceMaterial().shininess());
  dspbxPolyFaceOpacity->setValue(qAlpha(polyprops.faceMaterial().color())/255.);
  gbPolyRim->setChecked(polyprops.drawRim());
  cbPolyRimColor->setColor(polyprops.rimMaterial().color());
  dspbxPolyRimRadius->setValue(polyprops.rimRadius().D);
  rimRHelper->setIndexFromData(polyprops.rimRadiusUnits());
  
  cutoffmodeHelper->setIndexFromData(pmode);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant BondPolyDlg::getData()
{
  BondModel::PrimaryAtomData *data=new BondModel::PrimaryAtomData(&polyprops,pmode);
  
  return toQVariant(data);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondPolyDlg::acceptData()
{
  pmode=cutoffmodeHelper->data();
  polyprops.setVisible(gbPolygon->isChecked());
  polyprops.setDrawFaces(gbPolyFaces->isChecked());
  QRgb c=cbPolyFaceColor->color();
  unsigned long a=((unsigned long)(dspbxPolyFaceOpacity->value()*255))
    *0x1000000;
  polyprops.setFaceMaterial(Material((c&0xffffff)|a,tbFaceSpecular->color(),
				     dspbxFaceShine->value()));
  polyprops.setDrawRim(gbPolyRim->isChecked());
  polyprops.setRimMaterial(Material(cbPolyRimColor->color()|0xff000000,50.));
  polyprops.setRimRadiusUnits(rimRHelper->data());
  polyprops.setRimRadius(dspbxPolyRimRadius->value());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondPolyDlg::accept()
{

  acceptData();

  QString txt="";

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondPolyDlg::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","BondPolyDialog");
  h->show();
  
}


/*EOF*/ 
