/* @(#)BondTreeView.hh
 * author: magru@unas
 */

#ifndef _BONDTREEVIEW_HH
#define _BONDTREEVIEW_HH 1

#include <QMessageBox>
#include <QTreeView>
#include <QHeaderView>
#include <QEvent>
#include <QKeyEvent>
#include <QMouseEvent>
#include "BondModel.hh"
#include "BondPolyDlg.hh"
#include "BBins.hh"
#include "Config.hh"

class BondModel;

namespace XFPLO{
  class Symmmetry;
}

using namespace XFPLO;
/* ***********************************************************************
 *
 * ***********************************************************************
 */



class BondTreeView: public QTreeView
{
  Q_OBJECT;
public:

  enum SetBondsActions{SetFirstShell=0,SetNone=1};

  // ----------------------------------------------------------------
  BondTreeView(QWidget *parent=0)
    :QTreeView(parent),pbinscentertype(BBins::BCWyck),bpdlg(0)
  {
    header()->setResizeMode(QHeaderView::ResizeToContents);
    bondmodel=new BondModel(this);
    setModel(bondmodel);
  }

  // ----------------------------------------------------------------
  virtual ~BondTreeView(){
    if(bondmodel) delete bondmodel;
    if(bpdlg) delete bpdlg;
  }

  // ----------------------------------------------------------------
  BondModel *model()const{return bondmodel;}

  // ----------------------------------------------------------------
  void resetModel(const Symmetry *sym,BondDefGroup *bdg)
  {
    bondmodel->resetModel(sym,bdg,pbinscentertype);
  }

  

  // ----------------------------------------------------------------
  int maxLevels()const 
  {
    return bondmodel->maxLevels();
  }

  // ----------------------------------------------------------------
  long currentSort()const{
    return bondmodel->currentPrimaryIndex(currentIndex());
  }

  // ----------------------------------------------------------------
  void setCurrentSort(long sort,bool keepselection=false)
  {
    if (sort<0 && bondmodel->rowCount(QModelIndex())>0){
      sort=0;
    }
    if(sort>=0){
      QModelIndexList mil=selectionModel()->selectedIndexes();
      clearSelection();
      QModelIndex index=bondmodel->index(MyMath::max<long>(sort,0),0,
					 QModelIndex());
      bool doit=true;
      if(keepselection && bpdlg && !mil.contains(index) && mil.size()>1){
	QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			       , "Atom is not part of your selection", 1, 0,0 );
	doit=false;
      }
      if(doit){
	setCurrentIndex(index);
      }
      if(keepselection && bpdlg && mil.size()>1){
	for(int i=-1;++i<mil.size();){
	  selectionModel()->select(mil.at(i),QItemSelectionModel::Select);
	}
      }
      scrollTo(index,QAbstractItemView::PositionAtCenter);
    }
  }

  // ----------------------------------------------------------------
  void setBonds(SetBondsActions action)
  {
    bondmodel->setPropagateChanges(false);
    saveState();
    for(int i=-1;++i<pselectedindices.size();){
      QModelIndex index=pselectedindices.at(i);
      if(index.isValid() && index.internalId()==BondModel::PrimaryAtom){
	if(action==SetFirstShell){
	  QModelIndex child=bondmodel->index(0,0,index);
	  if(child.isValid()){
	    bondmodel->setData(child,2,Qt::CheckStateRole);
	  }
	}else if(action==SetNone){
	  for(int j=-1;++j<bondmodel->rowCount(index);){
	    QModelIndex child=bondmodel->index(j,0,index);
	    if(child.isValid()){
	      bondmodel->setData(child,0,Qt::CheckStateRole);
	    }
	  }
	  
	}
      }
    }
    //  todo: if unset all just remove all bonddefs, which belong to that?
    bondmodel->setPropagateChanges(true);
    bondmodel->resetModel(0,0,pbinscentertype,true);

    clearSelection();
    restoreState(true,true);

  }

  // ----------------------------------------------------------------
  void saveState()
  {
    pcurrentsort=bondmodel->currentPrimaryIndex(currentIndex());;
    pselectedindices=selectionModel()->selectedIndexes();

    for(int i=-1;++i<bondmodel->rowCount(QModelIndex());){
      QModelIndex ind=bondmodel->index(i,0,QModelIndex());
      expandmap[ind]=isExpanded(ind);
    }


  }

  // ----------------------------------------------------------------
  void restoreState(bool selected,bool expanded)
  {
    if(selected){
      QModelIndex index=bondmodel->index(MyMath::max<long>(pcurrentsort,0),0,
					 QModelIndex());
      setCurrentIndex(index);

      for(int i=-1;++i<pselectedindices.size();){
	const QModelIndex &ind=pselectedindices.at(i);
	if(ind.internalId()==BondModel::PrimaryAtom){
	  selectionModel()->select(ind,QItemSelectionModel::Select);
	}
      }
  
    }
    if(expanded){

      for(int i=-1;++i<bondmodel->rowCount(QModelIndex());){
	QModelIndex ind=bondmodel->index(i,0,QModelIndex());
	if(expandmap[ind]){
	  setExpanded(ind,true);
	}else{
	  setExpanded(ind,false);
	}
      }
    }

  }
  
signals:
  // ----------------------------------------------------------------
  void acceptRequest();
								
protected slots:
  // ----------------------------------------------------------------
  void currentChanged(const QModelIndex &current,const QModelIndex &previous){
    //if(!current.isValid()) return;
    QTreeView::currentChanged(current,previous);
    //scrollTo(current);
    if(bpdlg){
      bpdlg->setData(bondmodel->data(current,Qt::EditRole));
    }
  }


protected:

  // ----------------------------------------------------------------
  virtual bool event(QEvent *e)
  {
    if(e->type()==QEvent::KeyPress){
      QKeyEvent *k=(QKeyEvent*)e;
      QString key=k->text();
      if((k->key()==Qt::Key_F2) || (k->key()==Qt::Key_Space) || 
	 (key[0].isLetterOrNumber())){
	QModelIndex ci=currentIndex();
	if(ci.isValid() && ci.internalId()==BondModel::PrimaryAtom){
	  editPrimaryAtom(ci);
	  return true;
	}
      }
    }
    return QTreeView::event(e);
  }

  // ----------------------------------------------------------------
   virtual void mouseDoubleClickEvent(QMouseEvent *m)
   {
     if(m->button()==Qt::LeftButton){
       QModelIndex ci=indexAt(m->pos());
       if(ci.isValid() && ci.internalId()==BondModel::PrimaryAtom){
   	editPrimaryAtom(ci);
   	return;
       }
     }
     QTreeView::mouseDoubleClickEvent(m);
   }


private:

  // ----------------------------------------------------------------
  void editPrimaryAtom(const QModelIndex &index)
  {
    if(!index.isValid()) return;
    bpdlg=new BondPolyDlg(this,"BondPolyDlg",
			  bondmodel->data(index,Qt::EditRole));
    connect(bpdlg,SIGNAL(haveData()),this,SLOT(onHaveData()));
    if(bpdlg->exec()==QDialog::Accepted){
      commitBondData();
    }
    delete bpdlg; bpdlg=0;
 
  }

  // ----------------------------------------------------------------
  void commitBondData()
  {
    if(bpdlg){
      QModelIndexList mil=selectionModel()->selectedIndexes();
      for(int i=0;i<mil.size();i++){
	QModelIndex index=mil.at(i);
	if(index.isValid() && index.internalId()==BondModel::PrimaryAtom){
	  bondmodel->setData(index,bpdlg->getData(),Qt::EditRole);
	}
      }
    }
  }

private slots:

  // ----------------------------------------------------------------
  void onHaveData()
  {
    commitBondData();
    emit acceptRequest();
  }
  


  
private:
  BondModel *bondmodel;
  BBins::BinsCenterType pbinscentertype;
  BondPolyDlg *bpdlg;
  //state 
  QModelIndexList pselectedindices;
  long pcurrentsort;
  QMap<QModelIndex,bool> expandmap;
};


#endif /* _BONDTREEVIEW_HH */

