//todos: add/remove incomplete...
//todos: make bins E,T,S specific
//todos: maximal fuse





/*************************************************************************
 * @(#)BondsDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 Jul 2009
 *************************************************************************
 */

#include <QMessageBox>
#include "Config.hh"
#include "MyDebug.hh"
#include "Valid.hh"
#include "BondsDlg.hh"
#include "BBins.hh"
#include "MyMath.hh"
#include "Except.hh"
#include "BondModel.hh"
#include "BondDefs.hh"
#include "IconCache.hh"
#include "ToolButtons.hh"
#include "BondGroupsModel.hh"
#include "Delegates.hh"
#include "HelpDlg.hh"
#include "STDoc.hh"
#include "Cluster.hh"
#include "ShortCuts.hh"
#include "ToolButtons.hh"




/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondsDlg::BondsDlg(QWidget *parent ,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
  ,grmodel(0)
  ,p_doc(0),t_bonddefs(0)
  ,pcurrentsort(-1)
  ,ptoolbuttons(new ToolButtons())
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"BondDialog",": Bond/Poly/Atom settings");


  t_bonddefs=new BondDefs();
  
  //  setWindowModality ( Qt::WindowModal);

  new RatValidator(this,leBallScale,"The sphere scale",-1.e10,1.e10);
  new RatValidator(this,leStickBallRatio,"The stick/sphere ratio",-1.e10,1.e10);



  QAction *a;
  for(int i=0;++i<6;){
    keyactions.append(a=new QAction(this));
    a->setShortcut(QString("Ctrl+%1").arg(i));
    a->setData(i);
    connect(a,SIGNAL(triggered()),this,SLOT(onKeys()));
  }
  
  addActions(keyactions);




  
  



  bondcolordelegate=new ColorDelegate();
  if(grmodel) {delete(grmodel); grmodel=0;}
  grmodel = new BondGroupsModel(this);
  tvGroup->setModel(grmodel);
  tvGroup->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
  tvGroup->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
  tvGroup->setItemDelegateForColumn(BondGroupsModel::BondColor,bondcolordelegate);

  for(int i=-1;++i<BBins::binsTopLevelGroupingName.size();){
    cbTopLevelGrouping->addItem(BBins::binsTopLevelGroupingName.at(i).v2
				,BBins::binsTopLevelGroupingName.at(i).v1);
  }


  ptoolbuttons->createToolButtonsActions(tvGroup,"group",
					 tbAddGroup,tbDeleteGroup,
					 tbMoveUpGroup,tbMoveDownGroup);

  ptoolbuttons->connectSlot(this,SLOT(slotTvGroupButtons()));


  tbSetBonds->addAction(a=new QAction("first &coordination shell",tbSetBonds));
  a->setData(BondTreeView::SetFirstShell);
  tbSetBonds->addAction(a=new QAction("&all off",tbSetBonds));
  a->setData(BondTreeView::SetNone);



}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
BondsDlg::~BondsDlg()
{
  trvBonds->resetModel(0,0);
  if(grmodel) delete grmodel;
  if(bondcolordelegate) delete bondcolordelegate;bondcolordelegate=0;
  if(t_bonddefs) delete t_bonddefs;
  
  if(ptoolbuttons) delete ptoolbuttons;ptoolbuttons=0;

  qDeleteAll(keyactions);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BondsDlg::dataAcceptable()
{ 
  //myDebug() << "BondsDlg::dataAcceptable";
  //int i;
  QString s,txt="";


  ((RatValidator*)leBallScale->validator())->check(txt);
  ((RatValidator*)leStickBallRatio->validator())->check(txt);

  


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  //myDebug() << "END BondsDlg::dataAcceptable";

  return true;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::setData(const STDoc *doc)
{
  NSF("BondsDlg::setData");

  p_doc=doc;

  t_ballscale=doc->ballscale;
  t_stickballratio=doc->stickballratio;


  *t_bonddefs=*doc->bonddefs;
  t_bonddefs->setChanged(false);


  leBallScale->setText(t_ballscale.S());
  leStickBallRatio->setText(t_stickballratio.S());




  grmodel->reload(t_bonddefs->bondDefGroups());
  tvGroup->setCurrentIndex(grmodel->index(0,0,QModelIndex()));
  
  adjustTvGroup();

  




}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::getData(STDoc *doc, long &chn )
{
  //NSF("BoundDlg::getData");
  //myDebug() << nsubfunc;
  
  
  QString t=leBallScale->text(); IntValidator::stripAllWhiteSpace(t);
  doc->ballscale=t;
  chn=(t_ballscale != doc->ballscale);
  t_ballscale=t;
  
  t=leStickBallRatio->text(); IntValidator::stripAllWhiteSpace(t);
  doc->stickballratio=t;
  chn|=(t_stickballratio != doc->stickballratio);  
  t_stickballratio=t;







  *doc->bonddefs=*t_bonddefs;
  chn|=t_bonddefs->changed();

  t_bonddefs->setChanged(false);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::adjustTvGroup()
{
  int rows=grmodel->rowCount();
  int cols=grmodel->columnCount();
  for(int irow=0;irow<rows;irow++)
    tvGroup->resizeRowToContents(irow);
  for(int icol=0;icol<cols;icol++){
    tvGroup->resizeColumnToContents(icol);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_tvGroup_clicked(const QModelIndex &index)
{
  if(!index.isValid()) return;
  
  if(!(index.flags()&Qt::ItemIsEnabled))return;
  if((index.column()==BondGroupsModel::BondColor)){
    tvGroup->edit(index);
  }else{
    return;
  }
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::selectSite(long site)
{
  //NSF("BondsDlg::selectSite");
  long sort=p_doc->symmetry->sortBySite(site);

  trvBonds->setFocus();
  trvBonds->setCurrentSort(sort,true);
  
  
}






/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::slotTvGroupButtons()
{
  NSF("BondsDlg::slotTvGroupButtons");
  //myDebug() << nsubfunc;

  if(sender()==ptoolbuttons->pinsertAction){

    QModelIndex index=tvGroup->currentIndex();
    int row=0;
    int col=0;
    if(index !=  QModelIndex()){
      row=index.row()+1;
      col=index.column();
    }

    grmodel->insertRow(row,new BondDefGroup(0x9f9f9f));
    tvGroup->scrollTo(grmodel->index(row,0,QModelIndex()));
    tvGroup->setCurrentIndex(grmodel->index(row,col,QModelIndex()));

    adjustTvGroup();


  }else if(sender()==ptoolbuttons->pdeleteAction){
    QModelIndex currentindex=tvGroup->currentIndex();
    if(currentindex ==  QModelIndex()){
      QMessageBox::warning ( this,IDS_APP_NAME, "Select a bondgroup first."
			     , 1, 0,0 );
      return;
    }

    int numrows=grmodel->rowCount();
    int row=currentindex.row();
    int col=currentindex.column();


    BondDefGroup *bdg=grmodel->removeRow(row);
    if(bdg) delete bdg;
    tvGroup
      ->setCurrentIndex(grmodel
			->index(MyMath::min<long>(row,numrows-2),col,QModelIndex()));

    if(grmodel->rowCount()<=0){
      trvBonds->resetModel(p_doc->symmetry,0);
      spbxLevels->setMaximum(trvBonds->maxLevels());
    }

    adjustTvGroup();

  }else if((sender()==ptoolbuttons->pmoveUpAction)||
	   (sender()==ptoolbuttons->pmoveDownAction)){

    QModelIndex currentindex=tvGroup->currentIndex();
    if(currentindex ==  QModelIndex()){
      QMessageBox::warning ( this,IDS_APP_NAME, "Select an bondgroup first.", 1, 0,0 );
      return;
    }
    
    int row=currentindex.row();
    int col=currentindex.column();
    int by=-1;
    if(sender()==ptoolbuttons->pmoveDownAction) by=1;

    
    if(grmodel->moveRow(row,by)){
      tvGroup->setCurrentIndex(grmodel->index(row+by,col,QModelIndex()));
    }
    adjustTvGroup();


  }
  return;


  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_tvGroup_myCurrentChanged(const QModelIndex &currentindex
					   ,const QModelIndex &/*previous*/)
{
  //myDebug()<<"on_tvGroup_myCurrentChanged";
  if(!t_bonddefs) return;
  BondDefGroup *bdg=grmodel->currentGroup(currentindex);
  if(!bdg) return;

  pcurrentsort=trvBonds->currentSort();


  dspbxBinTol->setValue(bdg->binTol());
  cbTopLevelGrouping->setCurrentIndex(bdg->topLevelGrouping());

  trvBonds->resetModel(p_doc->symmetry,bdg);
  spbxLevels->setMaximum(trvBonds->maxLevels());


  on_spbxLevels_valueChanged(spbxLevels->value());

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_spbxLevels_valueChanged(int level)
{
  // myDebug()<<"on_spbxLevels_valueChanged";
  // myDebug()<<"pcurrentsort="<<pcurrentsort;
  if(level-2 <0){
    trvBonds->collapseAll();
  }else{
    trvBonds->expandToDepth(level-2);
  }

  trvBonds->setCurrentSort(pcurrentsort);
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::updateBins()
{
  //myDebug() << "BondsDlg::updateBins";

  QModelIndex ci=trvBonds->currentIndex();
  if(!ci.isValid())return;

  BondDefGroup *bdg=grmodel->currentGroup(tvGroup->currentIndex());
  if(!bdg) return;



  trvBonds->saveState();
  trvBonds->resetModel(p_doc->symmetry,bdg);
  trvBonds->restoreState(true,true);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_cbTopLevelGrouping_currentIndexChanged(int cindex)
{
  //NSF("on_cbTopLevelGrouping_currentIndexChanged");
  //myDebug()<<nsubfunc;

  BBins::BinsTopLevelGrouping toplevelgrouping
    =((BBins::BinsTopLevelGrouping)
      cbTopLevelGrouping->itemData(cindex).toInt());
  
  BondDefGroup *bdg=grmodel->currentGroup(tvGroup->currentIndex());
  
  if(bdg) bdg->setTopLevelGrouping(toplevelgrouping);


  updateBins();

}






/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_dspbxBinTol_valueChanged(double v)
{
  //NSF("BondsDlg::on_dspbxBinTol_valueChanged");
  //myDebug() << nsubfunc << v;

  BondDefGroup *bdg=grmodel->currentGroup(tvGroup->currentIndex());

  if(bdg) bdg->setBinTol(v);

  updateBins();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::onKeys()
{
  QAction *a=0;
  if((a=dynamic_cast<QAction*>(sender()))!=0){
    int level=a->data().toInt();
    spbxLevels->setValue(level);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BondsDlg::on_tbSetBonds_triggered(QAction *a)
{
  BondTreeView::SetBondsActions action=(BondTreeView::SetBondsActions)a->data().toInt();
  trvBonds->setBonds(action);
      
  
}


/*EOF*/ 
