/*************************************************************************
 * @(#)BoundDlgWidget.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Mar 2011
 *************************************************************************
 */

#include <QMenu>
#include "BoundDlgWidget.hh"
#include "Valid.hh"
#include "BBoxModel.hh"
#include "BFacesModel.hh"
#include "Vec3.hh"
#include "Config.hh"
#include "Boundary.hh"
#include "ToolButtons.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"



static long indexFromCellBasisType[4]={1,0,2,3};

static Boundary::BasisType cellBasisTypeFromIndex[4]={
  Boundary::Conventional, Boundary::Primitive, Boundary::Cartesian,Boundary::Box };

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BoundDlgWidget::BoundDlgWidget(QWidget *parent)
  : QWidget(parent),bbmodel(0),bfmodel(0)
  ,xyzvalid(0)
{
  setupUi(this);

  t_bound=new Boundary();

  new RatValidator(this,leSx,"The x-component of the shift",-1.e10,1.e10);
  new RatValidator(this,leSy,"The y-component of the shift",-1.e10,1.e10);
  new RatValidator(this,leSz,"The z-component of the shift",-1.e10,1.e10);




  xyzvalid=new RatValidator(this,0,"x,y,z,d0 or d1: ",-1e10,1e10);

  
  cbCellBasis->clear();
  for(int i=0;i<3;i++){
    cbCellBasis->insertItem(i,QString()
		 .sprintf("%-12s"
	 ,Boundary::cellBasisTypeName(cellBasisTypeFromIndex[i]))
			    ,QVariant(cellBasisTypeFromIndex[i]));
  }

  setupCbOriginBasis(true);

  connect(rbCell,SIGNAL(toggled(bool)),this,SLOT(switchInterface(bool)));
  connect(rbWign,SIGNAL(toggled(bool)),this,SLOT(switchInterface(bool)));
  connect(rbFaces,SIGNAL(toggled(bool)),this,SLOT(switchInterface(bool)));


  ptoolbuttons.createToolButtonsActions(tblvFaces,"face"
					,tbInsert,tbDelete,tbUp,tbDown);
    
  ptoolbuttons.connectSlots(this,SLOT(slotTblvFacesInsertPressed()),
			    SLOT(slotTblvFacesDeletePressed()),
			    SLOT(slotTblvFacesMoveUpPressed()),
			    SLOT(slotTblvFacesMoveDownPressed()));




  QMenu *m=new QMenu(tbSetFaces);

  fdefaultaction=m->addAction("&Default Faces");  
  fhexaction=m->addAction("&Hexagon Faces");
  fwignaction=m->addAction("&Wigner Seitz Faces");
  fconvcellaction=m->addAction("&Conventional uc Faces");
  fprimcellaction=m->addAction("&Primitive uc Faces");
  fWSRSConvRECcellaction=m->addAction("&WS of RS Conv");
  fusercellaction=m->addAction("&User saved Faces");


  connect(fhexaction,SIGNAL(triggered()),this,SLOT(onFHexactionTriggered()));;
  connect(fwignaction,SIGNAL(triggered()),this,SLOT(onFWignactionTriggered()));;
  connect(fdefaultaction,SIGNAL(triggered())
	  ,this,SLOT(onFDefaultactionTriggered()));;

  connect(fconvcellaction,SIGNAL(triggered())
	  ,this,SLOT(onFConvcellactionTriggered()));;

  connect(fprimcellaction,SIGNAL(triggered())
	  ,this,SLOT(onFPrimcellactionTriggered()));;
  connect(fWSRSConvRECcellaction,SIGNAL(triggered())
	  ,this,SLOT(onFWSRSConvRECcellactionTriggered()));;
  connect(fusercellaction,SIGNAL(triggered())
	  ,this,SLOT(onFUsercellactionTriggered()));;

  tbSetFaces->setMenu(m);

  tbSetFaces->setDefaultAction(fhexaction);


  m=new QMenu(tbSetCellMatrix);

  cdefaultaction=m->addAction("&Default Matrix");  
  cconvcellaction=m->addAction("&Conventional uc Matrix");
  cprimcellaction=m->addAction("&Primitive uc Matrix");

  connect(cdefaultaction,SIGNAL(triggered())
	  ,this,SLOT(onCDefaultactionTriggered()));;

  connect(cconvcellaction,SIGNAL(triggered())
	  ,this,SLOT(onCConvcellactionTriggered()));;

  connect(cprimcellaction,SIGNAL(triggered())
	  ,this,SLOT(onCPrimcellactionTriggered()));;

  tbSetCellMatrix->setMenu(m);
  tbSetCellMatrix->setDefaultAction(cdefaultaction);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BoundDlgWidget::~BoundDlgWidget()
{
  if(t_bound) delete t_bound;
  if(bbmodel) delete bbmodel;
  if(bfmodel) delete bfmodel;
  if(xyzvalid)  delete xyzvalid;


  if(fdefaultaction) delete fdefaultaction;
  if(fhexaction) delete fhexaction;
  if(fwignaction) delete fwignaction;
  if(fprimcellaction) delete fprimcellaction;
  if(fWSRSConvRECcellaction) delete fWSRSConvRECcellaction;
  if(fusercellaction) delete fusercellaction;
  if(fconvcellaction) delete fconvcellaction;

  if(cdefaultaction)  delete cdefaultaction;
  if(cprimcellaction) delete cprimcellaction;
  if(cconvcellaction) delete cconvcellaction;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::switchInterface(bool)
{
  //myDebug() << "BoundDlgWidget::switchInterface";
  if(rbCell->isChecked()){
    cbCellBasis->setEnabled(true);
    tabwInterface->setCurrentIndex(0);
    tabwInterface->setTabEnabled(0,true);
    tabwInterface->setTabEnabled(1,false);
    tabwInterface->setEnabled(true);
    setupCbOriginBasis(true);
  }else if(rbWign->isChecked()|| rbFaces->isChecked()){
    tabwInterface->setTabEnabled(0,false);
    tabwInterface->setTabEnabled(1,true);
    tabwInterface->setCurrentIndex(1);
    setupCbOriginBasis(false);
    if(rbWign->isChecked()){
      cbCellBasis->setEnabled(false);
      tabwInterface->setEnabled(false);
    }else{
      cbCellBasis->setEnabled(true);
      tabwInterface->setEnabled(true);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::setupCbOriginBasis(bool withbox)
{
  int index=cbOriginBasis->currentIndex();
  int n=4;
  if(withbox) n=4;
  cbOriginBasis->clear();
  for(int i=0;i<n;i++){
    cbOriginBasis->insertItem(i,QString()
		 .sprintf("%-12s"
	 ,Boundary::cellBasisTypeName(cellBasisTypeFromIndex[i]))
			    ,QVariant(cellBasisTypeFromIndex[i]));
  }
  cbOriginBasis->setCurrentIndex(index);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString BoundDlgWidget::dataAcceptable()
{ 

  //myDebug() << "BoundDlgWidget::dataAcceptable";

  QString s,txt="";


  if( cbCellBasis->currentIndex()<0)
    txt+="\nCell Basis must be choosen!";

  if( cbOriginBasis->currentIndex()<0)
    txt+="\nOrigin Basis must be choosen!";

  if(!( rbWign->isChecked() || rbCell->isChecked() || 
	rbFaces->isChecked() ) )
    txt+="\nOne boundary type radio button must be checked!";

  if(rbWign->isChecked() && 
     cellBasisTypeFromIndex[cbOriginBasis->currentIndex()]==Boundary::Box
     ){
    txt+="\nBox/Cell origin basis and Wigner-Seitz cell are not compatible!";
   }
  if(rbFaces->isChecked() && 
     cellBasisTypeFromIndex[cbOriginBasis->currentIndex()]==Boundary::Box
     ){
    txt+="\nBox/Cell origin basis and user defined faces are not compatible!";
  }

  ((RatValidator*)leSx->validator())->check(txt);
  ((RatValidator*)leSy->validator())->check(txt);
  ((RatValidator*)leSz->validator())->check(txt);


  // validate the bboxmodel
  QString Vtxt="";
  String b[9];
  for(int row=0; row < 3; row++){
    QModelIndex index;
    long err=0;
    QString m;
    
    for(int c=0; c < 6; c++){ 
      if(c==3) continue;
      index=bbmodel->index(row,c,QModelIndex());
      QString s=bbmodel->data(index,Qt::DisplayRole).toString();
      err|=!xyzvalid->check(m,s);
      IntValidator::stripAllWhiteSpace(s);
      if(c<3) 
	b[c+3*row]=s.toAscii().constData();
    }
    if(err)
      Vtxt+=QString("V%1: ").arg(row+1)+m;
  }

  if(!Vtxt.length()){

    Matrix3x3 g(Vec3(b+0),Vec3(b+3),Vec3(b+6));
    bool indef;
    g.inverseT(indef);
    if(indef) Vtxt+="Box vectors define box with zero volume";

  }
  txt+=Vtxt;


  // validate the bboxmodel
  Vtxt="";
  for(int row=0; row < bfmodel->rowCount(QModelIndex()); row++){
    QModelIndex index;
    long err=0;
    QString m;
    
    for(int c=0; c < 5; c++){ 
      if(c==3) continue;
      index=bfmodel->index(row,c,QModelIndex());
      QString s=bfmodel->data(index,Qt::DisplayRole).toString();
      err|=!xyzvalid->check(m,s);
      IntValidator::stripAllWhiteSpace(s);
    }
    if(err)
      Vtxt+=QString("F%1: ").arg(row+1)+m;
  }
  txt+=Vtxt;



  //myDebug() << "END BoundDlgWidget::dataAcceptable";

  return txt;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::setData(const Boundary *bound,const Symmetry *sym
			     ,bool setfocus)
{
  //  NSF("BoundDlgWidget::setData");
  //  CPR("");

  t_sym=sym;

  *t_bound=*bound;
  t_bound->setChanged(false);





  if(bbmodel) {delete(bbmodel); bbmodel=0;}
  bbmodel = new BBoxModel(this,t_bound);
  tblvBox->setModel(bbmodel);
  tblvBox->setCurrentIndex(bbmodel->index(0,0,QModelIndex()));
  
  tblvBox->horizontalHeader()->setResizeMode(3,QHeaderView::Fixed);
  tblvBox->horizontalHeader()->resizeSection(3,20);


  if(bfmodel) {delete(bfmodel); bfmodel=0;}
  bfmodel = new BFacesModel(this,t_bound);
  tblvFaces->setModel(bfmodel);
  tblvFaces->setCurrentIndex(bfmodel->index(0,0,QModelIndex()));
  
  tblvFaces->horizontalHeader()->setResizeMode(3,QHeaderView::Fixed);
  tblvFaces->horizontalHeader()->resizeSection(3,20);

  


  cbCellBasis->setCurrentIndex(indexFromCellBasisType[
			       bound->cellBasisType()]);

  cbOriginBasis->setCurrentIndex(indexFromCellBasisType[
			       bound->originBasis()]);

    
  switch  (bound->cellType()){
  case Boundary::Cell: 
    if(setfocus) rbCell->setFocus();
    rbCell->setChecked(true);
    break;
  case Boundary::WignerSeitz: 
    if(setfocus) rbWign->setFocus();
    rbWign->setChecked(true);
    break;
  case Boundary::Faces: 
    if(setfocus) rbFaces->setFocus();
    rbFaces->setChecked(true);
    break;
  }

  cbCentered->setChecked(bound->centeringType()==Boundary::Centered);

  leSx->setText(bound->originS()[0]());
  leSy->setText(bound->originS()[1]());
  leSz->setText(bound->originS()[2]());


  //switchInterface(true);	
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::getData(Boundary *bound, bool &chn )
{
  //  NSF("BoundDlgWidget::getData");
  //  CPR("");


  Boundary::BasisType btype=(Boundary::BasisType)
    cbCellBasis->itemData(cbCellBasis->currentIndex(),Qt::UserRole).toInt();
  
  bound->setCellBasisType(btype);
  t_bound->setCellBasisType(btype);

  

  //
  if(rbCell->isChecked()) {
    bound->setCellType(Boundary::Cell);
    t_bound->setCellType(Boundary::Cell);
  }
  if(rbWign->isChecked()) {
    bound->setCellType(Boundary::WignerSeitz);
    t_bound->setCellType(Boundary::WignerSeitz);
  }
  if(rbFaces->isChecked()) {
    bound->setCellType(Boundary::Faces);
    t_bound->setCellType(Boundary::Faces);
  }


  if(cbCentered->isChecked()){
    bound->setCentering(Boundary::Centered);
    t_bound->setCentering(Boundary::Centered);
  }else{
    bound->setCentering(Boundary::None);
    t_bound->setCentering(Boundary::None);
  }



  String s[3];
  QString t=leSx->text(); IntValidator::stripAllWhiteSpace(t);
  s[0]=t.toAscii().constData();

  t=leSy->text(); IntValidator::stripAllWhiteSpace(t);
  s[1]=t.toAscii().constData();

  t=leSz->text(); IntValidator::stripAllWhiteSpace(t);
  s[2]=t.toAscii().constData();

  Boundary::BasisType os=(Boundary::BasisType)
    cbOriginBasis->itemData(cbOriginBasis->currentIndex(),Qt::UserRole).toInt();

  
  bound->setOrigin(s,os);
  t_bound->setOrigin(s,os);



  bound->setBox(t_bound->boxVecS(0));
  bound->setBoxDistances(t_bound->boxDistanceS(0));

  bound->setFaces(t_bound->faces());
  


  
  chn=t_bound->changed();
  t_bound->setChanged(false);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::on_pbResetD_clicked()
{
  bbmodel->reset("D");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::onCDefaultactionTriggered()
{
  tbSetCellMatrix->setDefaultAction(cdefaultaction);
  
  int index=cbCellBasis->currentIndex();
  if(index>=0) bbmodel->reset("CBOX",t_sym,cellBasisTypeFromIndex[index]);
}
void BoundDlgWidget::onCConvcellactionTriggered()
{
  tbSetCellMatrix->setDefaultAction(cconvcellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bbmodel->reset("CCONVCELL",t_sym,cellBasisTypeFromIndex[index]);
}

void BoundDlgWidget::onCPrimcellactionTriggered()
{
  tbSetCellMatrix->setDefaultAction(cprimcellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bbmodel->reset("CPRIMCELL",t_sym,cellBasisTypeFromIndex[index]);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::onFDefaultactionTriggered()
{
  tbSetFaces->setDefaultAction(fdefaultaction);
  
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("BOX",t_sym,cellBasisTypeFromIndex[index]);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::onFConvcellactionTriggered()
{
  tbSetFaces->setDefaultAction(fconvcellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("CONVCELL",t_sym,cellBasisTypeFromIndex[index]);
}

void BoundDlgWidget::onFPrimcellactionTriggered()
{
  tbSetFaces->setDefaultAction(fprimcellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("PRIMCELL",t_sym,cellBasisTypeFromIndex[index]);
}

void BoundDlgWidget::onFWSRSConvRECcellactionTriggered()
{
  tbSetFaces->setDefaultAction(fWSRSConvRECcellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("WSRSCONVRECCELL",t_sym,cellBasisTypeFromIndex[index]);
}

void BoundDlgWidget::onFUsercellactionTriggered()
{
  tbSetFaces->setDefaultAction(fusercellaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("USERCELL",t_sym,cellBasisTypeFromIndex[index]);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::onFHexactionTriggered()
{
  tbSetFaces->setDefaultAction(fhexaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("HEX",t_sym,cellBasisTypeFromIndex[index]);
}

void BoundDlgWidget::onFWignactionTriggered()
{
  tbSetFaces->setDefaultAction(fwignaction);
  int index=cbCellBasis->currentIndex();
  if(index>=0) bfmodel->reset("WS",t_sym,cellBasisTypeFromIndex[index]);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::slotTblvFacesInsertPressed()
{
  NSF("BoundDlgWidget::slotTblvFacesInsertPressed"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc ;
  QModelIndex index=tblvFaces->currentIndex();
  if(index !=  QModelIndex()){
    int row=index.row();
    //myDebug() << index << "row"<<row;
    const Face f=*bfmodel->getData(row);
    bfmodel->insertRows(row,1,QModelIndex(),f);
   }else{
    const Face f=*bfmodel->getData(0);
    bfmodel->insertRows(0,1,QModelIndex(),f);
  }
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::slotTblvFacesDeletePressed()
{
  NSF("BoundDlgWidget::slotTblvFacesDeletePressed"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc ;

  QItemSelectionModel *selectionModel = tblvFaces->selectionModel();
  QModelIndexList indexes = selectionModel->selectedIndexes();
  QModelIndex index;

  QList<long> l;
  foreach (index, indexes) {
    int row=index.row();
    //myDebug() << index << row;
    if(!l.contains(row))
      l.append(row);
  }
  qSort(l.begin(),l.end());

  for(int i=l.size()-1;i>=0;i--){
    bfmodel->removeRows(l[i],1,QModelIndex());
  }
  return;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::slotTblvFacesMoveUpPressed()
{
  NSF("BoundDlgWidget::slotTblvFacesMoveUpPressed"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc ;

  QItemSelectionModel *selectionModel = tblvFaces->selectionModel();
  QModelIndexList indexes = selectionModel->selectedIndexes();
  MASSERT("",indexes.size()==1);
  QModelIndex index=indexes[0];
  int row=index.row();
  int col=index.column();

  QModelIndex prev=bfmodel->index(row-1,0,QModelIndex());
  if(!prev.isValid()) return;

  //  myDebug() <<"row"<<row;
  
  const Face f=*bfmodel->getData(row-1);
  //  w.print();
  bfmodel->removeRows(row-1,1,QModelIndex());
  //  w.print();

  bfmodel->insertRows(row-1,1,QModelIndex(),f);


  tblvFaces->setCurrentIndex(bfmodel->index(row-1,col,QModelIndex()));
  

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::slotTblvFacesMoveDownPressed()
{
  NSF("BoundDlgWidget::slotTblvFacesMoveDownPressed"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc ;

  QItemSelectionModel *selectionModel = tblvFaces->selectionModel();
  QModelIndexList indexes = selectionModel->selectedIndexes();
  MASSERT("",indexes.size()==1);
  QModelIndex index=indexes[0];
  int row=index.row();
  QModelIndex next=bfmodel->index(row+1,0,QModelIndex());
  if(!next.isValid()) return;

  //  myDebug() <<"row"<<row;
  
  const Face f=*bfmodel->getData(row);
  //myDebug() << f.pD.printS()() << f.distD;
  bfmodel->removeRows(row,1,QModelIndex());
  //myDebug() << f.pD.printS()() << f.distD;

  bfmodel->insertRows(row,1,QModelIndex(),f);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BoundDlgWidget::on_pbSaveCurrentFaces_clicked()
{
  Boundary::userFaces=t_bound->faces();
}


/*EOF*/ 
