/*************************************************************************
 * @(#)BrainDialog.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Dec 2003
 *************************************************************************
 */

#include <QShowEvent>
#include <QApplication>
#include <QDesktopWidget>
//#include "config.h"
#include "BrainDialog.hh"
#include "FString.hh"
#include "MyMath.hh"
//#include "MyDebug.hh"
#include "Settings.hh"


QHash<QString,QRect*> BrainDialog::BrainDialog_Geometries;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
BrainDialog::BrainDialog(QWidget *parent,QWidget *base
			 ,const char *name, bool modal,bool stayontop
			 ,bool considerposition)
  :QDialog(parent,(stayontop?Qt::Widget:
		   (considerposition?Qt::Window:Qt::Dialog)
		   )|
	   Qt::WindowTitleHint|
	   Qt::WindowSystemMenuHint|Qt::WindowCloseButtonHint|
	   Qt::WindowContextHelpButtonHint),pmainWidget(base)
  ,pconsiderposition(considerposition)
  ,pshown(false)
{
  //NSF("BrainDialog::BrainDialog");
  //CPRS();
  setObjectName(name);
  setModal(modal);
  //printf("%x\n",windowFlags());
  //myDebug() << windowFlags();
} 

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BrainDialog::~BrainDialog()
{
  //NSF("~BrainDialog");
  //CPR("");
  //  myDebug() << "BrainDialog::~BrainDialog";
  saveGeometry();

  //  myDebug() << "end BrainDialog::~BrainDialog";

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BrainDialog::saveGeometry()
{
  QRect *r=new QRect(pos(),size());
  if( BrainDialog_Geometries.contains(objectName())){
    delete BrainDialog_Geometries.take(QString(objectName()));
  }
  BrainDialog_Geometries.insert(objectName(),r);

  
  Settings("BrainDialog").set((QString("%1/Geometry/Size").arg(objectName()))
		 ,size());
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BrainDialog::showEvent(QShowEvent *e)
{
  //NSF("showEvent");
  if(pshown) {
    QDialog::showEvent(e);
    return;
  }
  //CPR("");
  //myDebug("XXXXXXXXXXXX %s: %d\n",String(objectName())(),BrainDialog_Geometries.contains(objectName()));
  if( BrainDialog_Geometries.contains(objectName())){

    //TODO: check if dialog still visible due to change in 
    //      widget position
    resize(BrainDialog_Geometries.value(objectName())->size());
    if(pconsiderposition)
      move(BrainDialog_Geometries.value(objectName())->topLeft());

  }else{
    long xx,yy;
    {
      whereTOplace(40L,xx, yy);
      if(pconsiderposition)
	move(xx,yy);
      
      
      resize(Settings("BrainDialog").getSize(QString("%1/Geometry/Size")
				.arg(objectName()),size()));
    }
  }


  QDialog::showEvent(e);
  pshown=true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BrainDialog::whereTOplace(long dtop,long &xx,long &yy)
{
  if(pmainWidget==0){
    printf("need to call setMainWidget first: \n");
    exit(1);
  }
  //  NSF("whereTOplace");
  //  CPR("");
  QDesktopWidget *desk = QApplication::desktop();
  long dw=desk->width();                   
  long dh=desk->height();                  
  //long w=width();
  //long h=height();
  //  CPR("before this->frameGeometry");
  long w=frameGeometry().width();
  long h=frameGeometry().height();

  //long gx=qApp->pmainWidget()->x();
  //long gy=qApp->pmainWidget()->y();
  //long gw=qApp->pmainWidget()->width();
  //long gh=qApp->pmainWidget()->height();
  //  CPR("before pmainWidget->frameGeometry");
  QRect r=pmainWidget->frameGeometry();
  long gx=r.x();
  long gy=r.y();
  long gw=r.width();
  long gh=r.height();
#if (QTMAINVERSION >2 )  
  h=h+20;
#endif  
  long ol=MyMath::min<long>(MyMath::max<long>(w-gx,0),w);
  long ot=MyMath::min<long>(MyMath::max<long>(h-gy-dtop,0),h);
  long ob=MyMath::min<long>(MyMath::max<long>(h-(dh-dtop-gy-gh),0),h);
  long ori=MyMath::min<long>(MyMath::max<long>(w-(dw-gx-gw),0),w);
  long A[4];

  
  
  A[0]=ori*ob;
  A[1]=ol*ob;
  A[2]=ori*ot;
  A[3]=ol*ot;
  long imin=1;
  for(long i=0;i<4;i++){
    if(A[i]<A[imin]){
      imin=i;
    }
  }


  switch (imin){
  case 0: 
    { 
      xx=MyMath::min<long>(MyMath::max<long>(gx+gw,0),dw-w);
      yy=MyMath::min<long>(MyMath::max<long>(gy+gh,0),dh-h);
      break;
    }
  case 1: 
    { 
      xx=MyMath::min<long>(MyMath::max<long>(gx-w,0),dw-w);
      yy=MyMath::min<long>(MyMath::max<long>(gy+gh,0),dh-h);
      break;
    }
  case 2: 
    { 
      xx=MyMath::min<long>(MyMath::max<long>(gx+gw,0),dw-w);
      yy=MyMath::min<long>(MyMath::max<long>(gy-h+dtop,dtop),dh-h);
      break;
    }
  case 3: 
    { 
      xx=MyMath::min<long>(MyMath::max<long>(gx-w,0),dw-w);
      yy=MyMath::min<long>(MyMath::max<long>(gy-h+dtop,dtop),dh-h);
      break;
    }
  }
}  

/*EOF*/ 
