/*************************************************************************
 * @(#)CIFImportDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   08 Aug 2012
 *************************************************************************
 */

#include <QMessageBox>
#include <QFileDialog>
#include <QTextTable>
#include <QTextStream>
#include "STDoc.hh"
#include "Symmetry.hh"
#include "CIFImportDlg.hh"
#include "CIFDriver.hh"
#include "CIFData.hh"

#define workdir (t_stdoc->currentDir() +'/')

using namespace CIF;


static long border=0;
//static long indent=20;
static QString pstyle1="<p  %1 style=\" margin-top:0px; margin-bottom:0px; "
  "margin-left:0px; margin-right:0px; -qt-block-indent:0; "
  "text-indent:0px;\">%2</p>";
static QString pstyle="<p  %1 >%2</p>";
static QString spanfweight="<span style=\" font-weight:%1; "
  "color:%2;\">%3</span>";
static QString spancolor="<span style=\" color:%1;\">%2</span>";
static QString spanbuline="<span style=\" font-weight:600; "
  "text-decoration: underline;\">%1</span>";

static QString tableheader="<table cellpadding=\"2\" cellspacing=\"0\" "
  "border=\"%1\" %2><tbody>\n";
static QString tablefooter="</tbody></table>\n";

static QString row="<tr>%1</tr>\n";
static QString column="<td %1>%2</td>\n";
const char *grey="bgcolor=\"#f8f8ff\"";
const char *rose="bgcolor=\"#ffe0e0\"";

/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFImportDlg::CIFImportDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"CIFImportDlg",": Import CIF file");

//   tbsplitter->setStretchFactor(0,3);
//   tbsplitter->setStretchFactor(1,10);

  pmetadict=pcifdata.metaDict();

  //loadTwDictionary();
  cbShowFullContent->setChecked(false);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
CIFImportDlg::~CIFImportDlg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::setData(STDoc *stdoc,const QString &filename)
{
  t_stdoc=stdoc;
  if(!filename.isEmpty()){
    loadCFFile(filename);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::on_pbBrowseCIFFile_clicked()
{

  QString f0=workdir+"*.cif";
  
  QString fileName = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"CIF (*.cif);;All files (*)");

  loadCFFile(fileName);

  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CIFImportDlg::loadCFFile(const QString &fileName)
{
  if (fileName.isEmpty()) return;

  QFileInfo fi(fileName);
  String s(t_stdoc->currentDir());
  if(fi.path()==s.chop("/")()){
    leCIFFile->setText(fi.fileName());
  }else{
    leCIFFile->setText(fileName);
  }
  cmbDataBlock->clear();
  pcifdata.clear();
  try{
    CIFDriver cifdriver;
    cifdriver.parse(&pcifdata,String(fileName)());
    ValueArray<String> blocknames=pcifdata.dataBlockNames();
    
    for(long i=0;i<blocknames.size();i++){
      cmbDataBlock->insertItem(i,blocknames.at(i)());
    }
    
  }catch(Except &ex){
    pcifdata.clear();
    QMessageBox::warning(0,"Parsing CIF file"
			 ,ex.msg(),QMessageBox::Yes );
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::on_cmbDataBlock_currentIndexChanged(const QString &text)
{
  if( text.isEmpty()) return;
  //loadTwDictionary();
  loadTebrCIFContent(text);
  //  loadPlteCIFContent(text);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::loadTebrCIFContent(const QString &datablockname)
{
  //  return;
  const CIFData::DataBlock *datablock=0;
  try{
    datablock=pcifdata.dataBlock(String(datablockname));
  }catch(...){}
  if(!datablock) return;

  QCursor savecursor=this->cursor();
  this->setCursor(Qt::WaitCursor);

  tebrCIFContent->document()->clear();
  tebrCIFContent->setLineWrapMode(QTextEdit::NoWrap);


  const char *valuecolor="#0000ff";
  const char *tableheadercolor="#d0d0d0";


  
  QTextCursor cursor(tebrCIFContent->textCursor());

  QString html=pstyle.arg("").arg("&nbsp;")
    +pstyle.arg("").arg("&nbsp;")
    +pstyle.arg("align=\"center\"").arg(spanbuline.arg("Selected CIF content"))
    +pstyle.arg("").arg("&nbsp;")
    ;
  

  
  //--------------------------
  //checmical names and formulas
  //--------------------------
  

  html+=
    allOfSingleSection(datablock,CIFMetaDict::ChemicalName,valuecolor)+
    allOfSingleSection(datablock,CIFMetaDict::ChemicalFormula,valuecolor);


  //--------------------------
  //cell
  //--------------------------
  
  html+=cellSection(datablock,valuecolor);
  //--------------------------
  //symmetry information
  //--------------------------
  
  html+=symmetrySection(datablock,valuecolor,tableheadercolor);
  

  //--------------------------
  //atom table
  //--------------------------

  html+=atomTableSection(datablock,valuecolor,tableheadercolor);

  //--------------------------
  //show all
  //--------------------------

  if(cbShowFullContent->isChecked()){
    html+=allSection(datablock,valuecolor,tableheadercolor);
    //html+=pcifdata.printS().replace("\n","<br>",0,true)();
  }



  cursor.insertHtml(html);

  this->setCursor(savecursor);
  
  
  //end paragraph

  tebrCIFContent->scrollToAnchor("#SelectedCIFContent");
  if(false){
    //if(true){
    QFile file("t.html");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
      return;
    QTextStream out(&file);
    out<<tebrCIFContent->document()->toHtml();
    //out<<html;
    file.close();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::atomTableSection(const CIF::CIFData::DataBlock *datablock,
				    const char *valuecolor,
				    const char *tableheadercolor)
{

  QString html;
  String atomtabletag=pmetadict.value(CIFMetaDict::AtomSite);
  String caption=(String(atomtabletag).replace("_"," ",0,true)
		  .capitalize(true)+" ");
  String labeltag=pmetadict.value(CIFMetaDict::AtomSiteLabel);
  String typesymboltag=pmetadict.value(CIFMetaDict::AtomSiteTypeSymbol);
  String fractxtag=pmetadict.value(CIFMetaDict::AtomSiteFractX);
  String fractytag=pmetadict.value(CIFMetaDict::AtomSiteFractY);
  String fractztag=pmetadict.value(CIFMetaDict::AtomSiteFractZ);
  
  const CIF::CIFData::DataTable *datatable;
  try{
    datatable=datablock->atomTable(pmetadict);
  }catch(...){
    html=pstyle
      .arg("")
      .arg(spanfweight.arg(600)
	   .arg("#000000","loop_ with tag '")+
	   spanfweight.arg(600)
	   .arg("#ff0000",atomtabletag())+
	   spanfweight.arg(600)
	   .arg("#000000","' not found")
	   );
    
    return html;
  };
  
  bool stopit=false;
  String text1=Sprintf("loop_ %s does not contain tag(s): ",atomtabletag());
  if(!(datatable->contains(labeltag)|| 
       datatable->contains(typesymboltag))){
    html+=pstyle
      .arg("")
      .arg(spanfweight.arg(600)
	   .arg("#000000",text1())+
	   spanfweight.arg(600)
	   .arg("#ff0000",labeltag())+
	   spanfweight.arg(600)
	   .arg("#000000"," or ")+
	   spanfweight.arg(600)
	   .arg("#ff0000",typesymboltag())
	   );
    stopit=true;
  };
#define tagwarning(tag)				\
  if(!datatable->contains(tag)){		\
    html+=pstyle				\
      .arg("")				\
      .arg(spanfweight.arg(600)			\
	   .arg("#000000",text1())+		\
	   spanfweight.arg(600)			\
	   .arg("#ff0000",tag()));		\
    stopit=true;				\
  };
  tagwarning(fractxtag);
  tagwarning(fractytag);
  tagwarning(fractztag);
#undef tagwarning
  if(stopit) return html;
  
  
  bool haswycksymbol=datatable
    ->contains(pmetadict.value(CIFMetaDict::AtomSiteWyckoffSymbol));
  bool hasoccupancy=datatable
    ->contains(pmetadict.value(CIFMetaDict::AtomSiteOccupancy));
  bool hasmult=datatable
    ->contains(pmetadict.value(CIFMetaDict::AtomSiteSymmetryMultiplicity));
  
  
  
  
  html=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column.arg("",pstyle.arg("").arg(spanfweight.arg(600)
					    .arg("#000000",caption()))));
  
  
  QString tblhtml=tableheader.arg(border).arg("");
  QString headercols;
  
  
  QString headspan=spanfweight.arg(600).arg("#000000").arg("&nbsp;%1");
  
  QString firstspan=headspan.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");

  QString headcell=column.arg(QString("bgcolor=\"%1\"")
			      .arg(tableheadercolor),
			      pstyle.arg("")
			      .arg(headspan));
  
  headercols+=column.arg(QString("bgcolor=\"%1\"")
			 .arg(tableheadercolor),
			 pstyle.arg("")
			 .arg(firstspan
			      .arg(pmetadict
				   .subValue(CIFMetaDict::AtomSiteLabel,
					     CIFMetaDict::AtomSite)()))
			 );

  headercols+=headcell.arg(pmetadict
			   .subValue(CIFMetaDict::AtomSiteFractX,
				     CIFMetaDict::AtomSite)());
  
  headercols+=headcell.arg(pmetadict
			   .subValue(CIFMetaDict::AtomSiteFractY,
				     CIFMetaDict::AtomSite)());
  headercols+=headcell.arg(pmetadict
			   .subValue(CIFMetaDict::AtomSiteFractZ,
				     CIFMetaDict::AtomSite)());
  
  if(haswycksymbol){
    headercols+=headcell
      .arg(pmetadict.subValue(CIFMetaDict::AtomSiteWyckoffSymbol,
			      CIFMetaDict::AtomSite)());
  }
  if(hasoccupancy){
    headercols+=headcell
      .arg(pmetadict.subValue(CIFMetaDict::AtomSiteOccupancy,
			      CIFMetaDict::AtomSite)());
  }
  if(hasmult){
    headercols+=headcell
      .arg(pmetadict.subValue(CIFMetaDict::AtomSiteSymmetryMultiplicity,
			      CIFMetaDict::AtomSiteSymmetry)());
  }
  tblhtml+=row.arg(headercols);
  

  QString span=spancolor.arg(valuecolor).arg("&nbsp;%1");
  firstspan=span.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");
  QString cell=column.arg("%1",pstyle.arg("").arg(span.arg("%2")));

  for (long r=0; r < datatable->rows(); r++) {
    const char *cf=rose; if((r)%2==0){ cf=grey; }
    QString cols;
    cols+=column.arg(cf,pstyle
		     .arg("")
		     .arg(firstspan.arg(datatable->atomLabel(r,pmetadict)())));

    Vec3 pos=datatable->atomFracPosition(r,pmetadict)();
    cols+=cell.arg(cf).arg(String::doubleToString("%30.15f",pos[0],true)());
    cols+=cell.arg(cf).arg(String::doubleToString("%30.15f",pos[1],true)());
    cols+=cell.arg(cf).arg(String::doubleToString("%30.15f",pos[2],true)());


    if(haswycksymbol){
      cols+=cell.arg(cf).arg(datatable->atomWyckoffSymbol(r,pmetadict)());
    }
    if(hasoccupancy){
      String s="?";
      try {
	s=String
	  ::doubleToString("%10.15f",
			   datatable->atomOccupancy(r,pmetadict),true);
      }catch(...){};
      cols+=cell.arg(cf).arg(s());
    }
    if(hasmult){
      String s=Sprintf("%10ld",
		       datatable->atomSymmetryMultiplicity(r,pmetadict))
 	.stripBlanks();
      cols+=cell.arg(cf).arg(s());
     }
    
    tblhtml+=row.arg(cols);
 
  }
  tblhtml+=tablefooter;
  html+=row.arg(column.arg("",pstyle.arg("").arg(tblhtml)));
  return html+tablefooter;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::cellSection(const CIFData::DataBlock *datablock,
				  const char *valuecolor)
{

  const char *warncolor="#ff0000";

  String basename=pmetadict.value(CIFMetaDict::Cell);
  basename.replace("_"," ",0,true).capitalize(true);

  QString notfound=spanfweight.arg(600).arg(warncolor,"not found");
  QString slatcon=notfound;
  try{
    Vec3 v=datablock->latticeConstants(pmetadict);
    slatcon=spancolor.arg(valuecolor,v.printS("%15.8f","  ",true)());
  }catch(...){}
  QString saxisang=notfound;
  try{
    Vec3 v=datablock->axesAngles(pmetadict);
    saxisang=spancolor.arg(valuecolor,v.printS("%15.8f","  ",true)());
  }catch(...){}




  QString html=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column.arg("colspan=\"2\"",
		       pstyle.arg("").arg(spanfweight.arg(600)
					 .arg("#000000",basename()))));   
  QString tab="&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
  html+=row.arg(
		column.arg("width=\"30%\"",
			   pstyle.arg("").arg(tab+"Lattice parameter:&nbsp;"))+
		column.arg("", pstyle.arg("").arg(slatcon))
		);
  html+=row.arg(
		column.arg("",
			   pstyle.arg("").arg(tab+"Axis angles:&nbsp;"))+
		column.arg("", pstyle.arg("").arg(saxisang))
		);
  
  
  try{
    double vol=datablock->cellVolume(pmetadict);
    QString svol=spancolor.arg(valuecolor,
			       String::doubleToString("%15.8f",vol,true)());    
    html+=row.arg(
		  column.arg("",
			     pstyle.arg("").arg(tab+"Cell volume:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(svol))
		  );
  }catch(...){ }

  try{
    long N=datablock->cellFormulaUnits(pmetadict);
    QString s=spancolor.arg(valuecolor).arg(N);
    html+=row.arg(
		  column.arg("",
			     pstyle.arg("").arg(tab+"Formula units:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
   }catch(...){};


  try{
    String sspec=datablock->cellSpecialDetails(pmetadict);
    QString s=spancolor.arg(valuecolor).arg(sspec());
    html+=row.arg(
		  column.arg("",
			     pstyle.arg("").arg(tab+"Special details:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
   }catch(...){};


  return html+tablefooter;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::symmetrySection(const CIFData::DataBlock *datablock,
				      const char *valuecolor,
				      const char *tableheadercolor)
{
  


  QString html=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column.arg("colspan=\"2\"",
		       pstyle.arg("").arg(spanfweight.arg(600)
					 .arg("#000000","Symmetry"))));
  
  QString colformat="width=\"30%\"";
  QString tab="&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
  
  String crystalsystem=datablock->crystalSystem(pmetadict);
  if(!crystalsystem.empty()){
    QString s=spancolor.arg(valuecolor).arg(crystalsystem());
    html+=row.arg(
		  column.arg(colformat,
			     pstyle.arg("").arg(tab+"Crystal system:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
    colformat="";
  }

  long spgrnum=datablock->spaceGroupNumber(pmetadict);
  if(spgrnum>0){
    QString s=spancolor.arg(valuecolor).arg(spgrnum);
    html+=row.arg(
		  column.arg(colformat,
			     pstyle.arg("").arg(tab+"Number:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
    colformat="";
  }
  String spgrhall=datablock->spaceGroupNameHall(pmetadict);
  if(!spgrhall.empty()){
    QString s=spancolor.arg(valuecolor).arg(spgrhall());
    html+=row.arg(
		  column.arg(colformat,
			     pstyle.arg("").arg(tab+"Name-Hall:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
    colformat="";
  }
  String spgrhm=datablock->spaceGroupNameHM(pmetadict);
  if(!spgrhm.empty()){
    QString s=spancolor.arg(valuecolor).arg(spgrhm());
    html+=row.arg(
		  column.arg(colformat,
			     pstyle.arg("").arg(tab+"Name-H-M:&nbsp;"))+
		  column.arg("", pstyle.arg("").arg(s))
		  );
    colformat="";
  }

  ValueArray<Pair<long,String> > spgrgenxyz
    =datablock->spaceGroupGeneratorsXYZ(pmetadict);

  if(spgrgenxyz.size()){
    QString tblhtml=tableheader.arg(border).arg("");
    QString headercols;
    QString headspan=spanfweight.arg(600).arg("#000000").arg("&nbsp;%1");
    QString firstspan=headspan.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");
    long nrow=spgrgenxyz.size();
    QString color=QString("bgcolor=\"%1\"").arg(tableheadercolor);
    QString cell=column.arg(color,pstyle.arg("",headspan));
    headercols+=column
      .arg(color,
	   pstyle
	   .arg("",firstspan.arg("Id")));
    headercols+=cell.arg("xyz-symbol");
  
      
    
    tblhtml+=row.arg(headercols);
    
    QString span=spancolor.arg(valuecolor).arg("&nbsp;%1");
    firstspan=span.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");
    cell=column.arg("%1",pstyle.arg("",span.arg("%2")));
    
    for(long ir=0;ir<nrow;ir++){
      const char *cf=rose; if((ir)%2==0){ cf=grey; }
      QString cols;
      cols+=column.arg(cf,pstyle.arg("",firstspan.arg("%1")))
	.arg(spgrgenxyz.at(ir).v1);
      cols+=cell.arg(cf).arg(spgrgenxyz.at(ir).v2());
      
      tblhtml+=row.arg(cols);
      
    }
    tblhtml+=tablefooter;
    
    html+=row.arg(column.arg("colspan=\"2\"",tblhtml));
  }
  
  return html+tablefooter;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::allOfSingleSection(const CIFData::DataBlock *datablock,
					 CIFMetaDict::DictKey key,
					 const char *valuecolor)
{
  ValueArray<Pair<String,String> > names;
  String basename=pmetadict.value(key);
  try{
    names=datablock->allOfSingleCategoryAsString(basename);
  }catch(...){return "";}
  basename.replace("_"," ",0,true).capitalize(true);

  
  QString html=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column
	    .arg("colspan=\"2\"",
		 pstyle.arg("").arg(spanfweight.arg(600).arg("#000000",
							    basename()))));
  
  QString span=spancolor.arg(valuecolor,"&nbsp;%1");

  QString col=column
    .arg("width=\"30%\"",
	 pstyle.arg("")
	 .arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1:&nbsp;"))+
    column.arg("",pstyle.arg("").arg(span.arg("%2")));
    
  for(long i=0;i<names.size();i++){
    html+=row.arg(col.arg(names[i].v1()).arg(names[i].v2.stripBlanks(true)()));
  }
  return html+tablefooter;
  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::allSection(const CIFData::DataBlock *datablock,
				 const char *valuecolor,
				 const char *tableheadercolor)
{
  ValueArray<Pair<String,String> > names;
  String basename="";
  try{
    names=datablock->allOfSingleCategoryAsString(basename);
  }catch(...){return "";}
  basename.replace("_"," ",0,true).capitalize(true);

  QString html=pstyle.arg("").arg("&nbsp;")
    +pstyle.arg("").arg("&nbsp;")
    +pstyle.arg("align=\"center\"").arg(spanbuline.arg("Full CIF content"))
    +pstyle.arg("").arg("&nbsp;")
    ;

  
  return html+genericDataBlock(datablock,valuecolor,tableheadercolor);
    

  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::genericDataBlock(const CIFData::DataBlock *datablock,
				       const char *valuecolor,
				       const char *tableheadercolor)
{
  
  QString html=tableheader.arg(border).arg("width=\"100%\"");  

  QString cell1=column.arg("width=\"10%\"",
			   pstyle.arg("")
			   .arg(spanfweight
				.arg(600)
				.arg("#000000","%1")));
  QString cell2=column.arg("width=\"90%\"",
			   pstyle.arg("")
				.arg(spancolor.arg(valuecolor,"%2")));
  QString scell=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column.arg("",
		       pstyle.arg("")
		       .arg(spanfweight
			    .arg(600)
			    .arg("#000000","SaveFrame %1"))))+
    row.arg(column.arg("","%2"))+tablefooter;
  
  for(long i=0;i<datablock->numItems();i++){
    const CIFData::DataBlockItem *dbi=datablock->at(i);
    if(dbi->type()==CIF::Single){
      html+=row.arg(cell1.arg(dbi->nameLC()())+
		    cell2.arg(dbi->value().asString()
			      .stripBlanks(true)
			      .replace("\n","<br>",0,true)())
		    );
      
      
    }else if(dbi->type()==CIF::Table){
      html+=row.arg(column.arg("colspan=\"2\" width=\"100%\"",
			       genericTable(datablock->at(i)->table(),
					    valuecolor,tableheadercolor)));
    }else if(dbi->type()==CIF::SaveFrame){
      const CIFData::DataBlock *saveframe=datablock->at(i)->saveFrame();
      html+=row.arg(column.arg("bgcolor=\"#e0ffe0\" colspan=\"2\" "
			       "width=\"100%\"",
			       scell.arg(saveframe->nameLC()())
	.arg(genericDataBlock(saveframe,valuecolor,tableheadercolor))));

    }

  }
  return html+tablefooter;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString CIFImportDlg::genericTable(const CIFData::DataTable *datatable,
				   const char *valuecolor,
				   const char *tableheadercolor)
{
  
  String commoncat=datatable->commonCategory();
  long lcommoncat=commoncat.length();


  QString html=tableheader.arg(border).arg("width=\"100%\"")+
    row.arg(column.arg("",pstyle
		       .arg("")
		       .arg(spanfweight
			    .arg(600)
			    .arg("#000000",(commoncat.replace("_"," ",0,true)
					    .capitalize(true)+" ")()))));
  
  QString tblhtml=tableheader.arg(border).arg("");
  QString headercols;
  QString headspan=spanfweight.arg(600).arg("#000000").arg("&nbsp;%1");
  QString firstspan=headspan.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");

  long ncol=datatable->columns();
  long nrow=datatable->rows();
  QString color=QString("bgcolor=\"%1\"").arg(tableheadercolor);
  QString cell=column.arg(color,pstyle.arg("",headspan));
  
  for(long ic=0;ic<ncol;ic++){
    if(ic==0){
      headercols+=column
	.arg(color,
	     pstyle
	     .arg("",firstspan.arg(datatable->normalizedTag(ic)
				    .subString(lcommoncat)())));
    }else{
      headercols+=cell.arg(datatable->normalizedTag(ic)
			   .subString(lcommoncat)());
    }

  }
  tblhtml+=row.arg(headercols);
  

  QString span=spancolor.arg(valuecolor).arg("&nbsp;%1");
  firstspan=span.arg("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%1");
  cell=column.arg("%1",pstyle.arg("",span.arg("%2")));

  for(long ir=0;ir<nrow;ir++){
    const char *cf=rose; if((ir)%2==0){ cf=grey; }
    QString cols;
    for(long ic=0;ic<ncol;ic++){
      if(ic==0){
	cols+=column.arg(cf,pstyle.arg("",firstspan.arg("%1")))
	  .arg(datatable->value(ir,ic)
	       .asString().replace("\n","<br>",0,true)());
      }else{
	cols+=cell.arg(cf)
	  .arg(datatable->value(ir,ic)
	       .asString().replace("\n","<br>",0,true)());
      }
    }
    tblhtml+=row.arg(cols);

  }
  tblhtml+=tablefooter;
  html+=row.arg(column.arg("",pstyle.arg("").arg(tblhtml)));
  return html+tablefooter;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void CIFImportDlg::loadTwDictionary()
// {

//   const CIFData::DataBlock *datablock=0;
//   try{
//     datablock=pcifdata.dataBlock(String(cmbDataBlock->currentText()));
//   }catch(...){}


//   twDictionary->clear();
//   twDictionary->setRowCount(0);
//   twDictionary->setColumnCount(2);
//   QStringList headers;
//   headers<<"Content"<<"CIF-name";
//   twDictionary->setHorizontalHeaderLabels(headers);
//   for(long i=0;i<pmetadict.size();i++){
//     twDictionary->insertRow(i);
//     QTableWidgetItem *item;
//     twDictionary->setItem(i,0,item=new QTableWidgetItem());
//     item->setText(pmetadict.at(i).v1());
//     twDictionary->setItem(i,1,item=new QTableWidgetItem());
//     item->setText(pmetadict.at(i).v2());

//     if(datablock){
//       if(datablock->hasSingle(pmetadict.at(i).v2())){
// 	item->setForeground(QColor(0x00ff00));
//       }else{
// 	item->setForeground(QColor(0xff0000));
//       }
//     }else{
//     }
    

//   }
//   adjustTw(twDictionary);
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::adjustTw(QTableWidget *tw)
{
  int rows=tw->rowCount();
  int cols=tw->columnCount();
  //myDebug()<<"cols"<<cols;
  for(int irow=0;irow<rows;irow++)
    tw->resizeRowToContents(irow);
  for(int icol=cols-1;icol>-0;icol--){
    tw->resizeColumnToContents(icol);
  }
  tw->resizeColumnsToContents();
  tw->resizeRowsToContents();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::getData(bool &ok)
{
  // in the current logic this is the apply slot, ok=false if nothing worked
  ok=false;
  try{
//     String datablockname(cmbDataBlock->currentText());
//     if(datablockname.empty()) return;
//     const CIFData::DataBlock *datablock=pcifdata.dataBlock(datablockname);

//     myDebug()<<pmetadict.itemOf(Pair<String,String>("spacegroupnumber1")).v1;
//     myDebug()<<pmetadict.itemOf(Pair<String,String>("spacegroupnumber1")).v2;
//     long spgrnum=datablock->spaceGroupNumber(pmetadict);
//     Vec3 angles=datablock->axesAngles(pmetadict);
//     Vec3 latcon=datablock->latticeConstants(pmetadict);
//     const CIFData::DataTable &table=
//       datablock->atomTable(pmetadict);

//     QList<WyckPos> wpos;
//     for(long i=0;i<table.size();i++){
//       String el=table.atomLabel(i,pmetadict).subString(0,1);
//       wpos.
// 	append(WyckPos(el,
// 		       i,
// 		       table.atomPosition(i,pmetadict),
// 		       globalproperties
// 		       ->current()->find(el).atomColor()));
//       wpos.last().setDefaultType();
//     }


//     t_stdoc->symmetry->setSpacegroup(spgrnum);
//     t_stdoc->symmetry->setAxesAngles(angles);
//     t_stdoc->symmetry->setLatticeConstants(latcon);
//     t_stdoc->symmetry->setLengthunit(Xf::Angstroem);
//     t_stdoc->symmetry->setWyckoffPositions(wpos);
      

    ok=true;
    return;

    

  }catch(Except &ex){
    QMessageBox::warning(0,"Extracting CIF file information"
			 ,ex.msg(),QMessageBox::Yes );
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CIFImportDlg::on_cbShowFullContent_stateChanged(int)
{
  loadTebrCIFContent(cmbDataBlock->currentText());
  //  loadPlteCIFContent(cmbDataBlock->currentText());
}



/*EOF*/ 
