/*************************************************************************
 * @(#)ColorRampWidget.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   21 Jun 2012
 *************************************************************************
 */

#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QComboBox>
#include <QUndoStack>
#include "ColorRampWidget.hh"
#include "HsvSampler.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ColorRampWidget::ColorRampWidget(QWidget *parent)
  :QWidget(parent),pundostack(0)

{
  QGridLayout *gridLayout=new QGridLayout(this);

  
  int row=0;
  HsvSampler *s;
  QLabel *l;

  l=new QLabel("Gradient");
  samplers
    .append(s=new HsvSampler(this,HsvSampler::None,HsvSampler::Parameter));
  s->setNodeList(&nodelist);
  connect(s,SIGNAL(nodeChanged()),this,SLOT(onNodeChanged()));

  gridLayout->addWidget(l, 0, row  , 1, 1);
  gridLayout->addWidget(s, 1, row++, 1, 1);

  l=new QLabel("Hue-Sat");
  samplers
    .append(s=new HsvSampler(this,HsvSampler::Hue,HsvSampler::Saturation,true));
  s->setNodeList(&nodelist);
  connect(s,SIGNAL(nodeChanged()),this,SLOT(onNodeChanged()));
  gridLayout->addWidget(l, 0, row  , 1, 2);
  gridLayout->addWidget(s, 1, row  , 1, 2);row+=2; 


  l=new QLabel("Satur.");
  samplers
    .append(s=new HsvSampler(this,HsvSampler::Saturation,HsvSampler::Parameter));
  s->setNodeList(&nodelist);
  connect(s,SIGNAL(nodeChanged()),this,SLOT(onNodeChanged()));
  gridLayout->addWidget(l, 0, row  , 1, 1);
  gridLayout->addWidget(s, 1, row++, 1, 1);
  
  l=new QLabel("Value");
  samplers
    .append(s=new HsvSampler(this,HsvSampler::Value,HsvSampler::Parameter));
  s->setNodeList(&nodelist);
  connect(s,SIGNAL(nodeChanged()),this,SLOT(onNodeChanged()));
  gridLayout->addWidget(l, 0, row  , 1, 1);
  gridLayout->addWidget(s, 1, row++, 1, 1);
  


  
//   samplers
//     .append(s=new HsvSampler(this,HsvSampler::Hue,HsvSampler::Value,true));
//   s->setNodeList(&nodelist);
//   gridLayout->addWidget(s, 2, 3, 1, 2);
//   connect(s,SIGNAL(nodeChanged()),this,SLOT(onNodeChanged()));
  
  
  gridLayout->setRowStretch(0, 0);
  gridLayout->setRowStretch(1, 10);

  pundostack=new QUndoStack(this);

  QPushButton *pbUndo=new QPushButton(this);
  pbUndo->setText("Undo");
  gridLayout->addWidget(pbUndo, 3, 0  , 1, 1);
  connect(pbUndo,SIGNAL(clicked()),pundostack,SLOT(undo()));
  pbUndo->setAutoDefault(false);

  QPushButton *pbRedo=new QPushButton(this);
  pbRedo->setText("Redo");
  gridLayout->addWidget(pbRedo, 4, 0  , 1, 1);
  connect(pbRedo,SIGNAL(clicked()),pundostack,SLOT(redo()));
  pbRedo->setAutoDefault(false);

  cmbSetGradient=new QComboBox(this);
  gridLayout->addWidget(cmbSetGradient, 2, 0  , 1, 1);

  gridLayout->setRowStretch(2, 0);
  gridLayout->setRowStretch(3, 0);

  QComboBox *cmbSet=new QComboBox(this);
  gridLayout->addWidget(cmbSet, 2, 4  , 1, 1);

  gridLayout->setRowStretch(2, 0);
  gridLayout->setRowStretch(3, 0);


  foreach(HsvSampler *s,samplers){
    connect(s,SIGNAL(registerUndo()),this,SLOT(onRegisterUndo()));
  }

  cmbSetGradient->insertItem(0,"equal");

  cmbSet->insertItem(0,"zero");
  cmbSet->insertItem(1,"one");
  cmbSet->insertItem(2,"linear");
  cmbSet->insertItem(3,"S3");
  cmbSet->insertItem(4,"S5");
  cmbSet->insertItem(5,"S7");
  cmbSet->insertItem(6,"W3");
  cmbSet->insertItem(7,"W5");
  cmbSet->insertItem(8,"invert");
  
  connect(cmbSetGradient,SIGNAL(activated(const QString &)),
	  this,SLOT(onCmbSetCurrentIndexChanged(const QString &)));

  connect(cmbSet,SIGNAL(currentIndexChanged(const QString &)),
	  this,SLOT(onCmbSetCurrentIndexChanged(const QString &)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ColorRampWidget::~ColorRampWidget()
{
  //if(cmbSetGradient) delete cmbSetGradient;cmbSetGradient=0;
  qDeleteAll(samplers);
  samplers.clear();
  if(pundostack)  delete pundostack;pundostack=0;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorRampWidget::setColorMap(const ColorMap &cmap)
{
  //myDebug()<<"setColorMap"<<cmap.printS()();
  nodelist.clear();

  for(int i=0;i<cmap.size();i++){
    nodelist.append(new HsvSampler::Node(cmap.at(i).parameter(),
			     cmap.at(i).rawHue(),
			     cmap.at(i).saturation(),
			     cmap.at(i).value()));
  }
  foreach(HsvSampler *s,samplers){
    s->setNodeList(&nodelist);
  }
  poldcmap=cmap;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorRampWidget::getColorMap(ColorMap &cmap)
{
  cmap.clear();
  for(int i=0;i<nodelist.size();i++){
    const HsvSampler::Node *n=nodelist.at(i);
    cmap.append(n->parameter(),n->rawHue(),
		n->saturation(),n->value());
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ColorMap ColorRampWidget::returnColorMap()
{
  ColorMap cmap;
  for(int i=0;i<nodelist.size();i++){
    const HsvSampler::Node *n=nodelist.at(i);
    cmap.append(n->parameter(),n->rawHue(),
		n->saturation(),n->value());
  }
  return cmap;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorRampWidget::onNodeChanged()
{
  foreach(HsvSampler *s,samplers){
    s->update();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorRampWidget::onRegisterUndo()
{
  ColorMap m=returnColorMap();
  MoveNode *mn=new MoveNode(this,poldcmap,m);
  poldcmap=m;
  pundostack->push(mn);
 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorRampWidget::onCmbSetCurrentIndexChanged(const QString &text)
{
  myDebug()<<text;

  ColorMap m=returnColorMap();
  if(sender()==cmbSetGradient){
    for(int i=0;i<m.size();i++){
      double s=1.*i/(m.size()-1);
      m[i].setParameter(s);
    }
  }else{
    if(text=="invert"){
      for(int i=0;i<m.size();i++){
	int v=m.at(i).value();
	m[i].setValue(255-v);
	
      }
    }else{
      for(int i=0;i<m.size();i++){
	double s=m.at(i).parameter();
	if(text=="linear"){
	  m[i].setValue(lRound(s*255.));
	}else if(text=="one"){
	  m[i].setValue(255);
	}else if(text=="zero"){
	  m[i].setValue(255);
	}else if(text=="S3"){
	  m[i].setValue(lRound(0.5*(1+pow(2.*s-1.,3))*255.));
	}else if(text=="S5"){
	  m[i].setValue(lRound(0.5*(1+pow(2.*s-1.,5))*255.));
	}else if(text=="S7"){
	  m[i].setValue(lRound(0.5*(1+pow(2.*s-1.,7))*255.));
	}else if(text=="W3"){
	  double u=2.*s-1.;
	  m[i].setValue(lRound(0.5*(1+dSign(pow(fabs(u),1./3),u))*255.));
	}else if(text=="W5"){
	  double u=2.*s-1.;
	  m[i].setValue(lRound(0.5*(1+dSign(pow(fabs(u),1./5),u))*255.));
	}
      }
    }
  }
  MoveNode *mn=new MoveNode(this,poldcmap,m);
  setColorMap(m);
  pundostack->push(mn);
  
}


/*EOF*/ 
