/* @(#)ColorRampWidget.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _COLORRAMPWIDGET_HH
#define _COLORRAMPWIDGET_HH 1

#include <QWidget>
#include <QUndoCommand>
#include "ColorMap.hh"
#include "MyMath.hh"
#include "HsvSampler.hh"


class QComboBox;
class QUndoStack;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class ColorRampWidget:public QWidget
{
  Q_OBJECT;

  
public:
  ColorRampWidget(QWidget *parent=0);
  ~ColorRampWidget();
  void setColorMap(const ColorMap &);
  void getColorMap(ColorMap &);
  ColorMap returnColorMap();

  QUndoStack *undoStack(){return pundostack;};

private slots:
  void onNodeChanged();
  void onRegisterUndo();
  void onCmbSetCurrentIndexChanged(const QString &);
private:

  ///////////////////////////////////////////////////
  ///////////////////////////////////////////////////
  ///////////////////////////////////////////////////
  ///////////////////////////////////////////////////
  class MoveNode:  public QUndoCommand
  {
  public:
    MoveNode(ColorRampWidget *cmwidget,const ColorMap &oldcm,
	     const ColorMap &newcm)
      :pcmwidget(cmwidget),poldcmap(oldcm),pnewcmap(newcm){
      setText("move node");};
    virtual void undo(){pcmwidget->setColorMap(poldcmap);};
    virtual void redo(){pcmwidget->setColorMap(pnewcmap);};
  private:
    ColorRampWidget *pcmwidget;
    ColorMap poldcmap;
    ColorMap pnewcmap;
  };

private:

  QList<HsvSampler*> samplers;
  HsvSampler::NodeList nodelist;
  HsvSampler::Node *currentnode;
  QUndoStack *pundostack;
  ColorMap poldcmap;
  QComboBox *cmbSetGradient;

};


#endif /* _COLORRAMPWIDGET_HH */

