/*************************************************************************
 * @(#)ContribDlg.cpp
 * author: magru@unas
 * date:   29 Sep 2013
 *************************************************************************
 */

#include <QCompleter>
#include "ContribDlg.hh"
#include "Valid.hh"
#include "DataExchange.hh"
//#include "MyDebug.hh"
#include "SqMatrix.hh"
#include "DMQuantAxis.hh"
#include "Except.hh"
#include "Atom.hh"
#include "BWEdit.hh"
#include "BWEdit.hh"
#include "DocBase.hh"




/* ***********************************************************************
 *
 * ***********************************************************************
 */
ContribDlg::ContribDlg(QWidget *parent,QWidget *base,const char *name
		       ,int code,bool editexisting)
  : ProtoDlg(parent,base,name),pcode(code),piswaiting(false),
    orbcompleter(0),pmo(0),pmd(0),peditexisting(editexisting)
{
  setupUi(this);
  setUp(0,pbOK,pbCancel,pbHelp,"ContribDialog",": Pattern contribution dialog");

  tedPage->setReadOnly ( true ) ;
  new IntValidator(this,leSite,"The site",1,100000);

  new RealValidator(this,leFR,"The real part of factor",-1.e10,1.e10);
  new RealValidator(this,leFI,"The imaginary part of factor",-1.e10,1.e10);

  new RealValidator(this,lePx,"The x-component of the position",-1.e10,1.e10);
  new RealValidator(this,lePy,"The y-component of the position",-1.e10,1.e10);
  new RealValidator(this,lePz,"The z-component of the position",-1.e10,1.e10);

  new RealValidator(this,leXx,"The x-component of the x-axis",-1.e10,1.e10);
  new RealValidator(this,leXy,"The y-component of the x-axis",-1.e10,1.e10);
  new RealValidator(this,leXz,"The z-component of the x-axis",-1.e10,1.e10);
  new RealValidator(this,leZx,"The x-component of the z-axis",-1.e10,1.e10);
  new RealValidator(this,leZy,"The y-component of the z-axis",-1.e10,1.e10);
  new RealValidator(this,leZz,"The z-component of the z-axis",-1.e10,1.e10);

  connect(leXx,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leXy,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leXz,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leZx,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leZy,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  connect(leZz,SIGNAL(textEdited(const QString &))
	  ,this,SLOT(onLedPageTextEdited(const QString &)));

  vSpacerBottom->changeSize(0,0,QSizePolicy::Minimum,QSizePolicy::Minimum);
  if(pcode == 1){
    frameContrib->setVisible(false);
    frameAxes->setVisible(false);
    setWindowTitle("Molecular orbital");
    vSpacerBottom->changeSize(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);

  }else if(pcode == 2){
    frameName->setVisible(false);
    setWindowTitle("Molecular orbital contribution");
  }else if(pcode == 0){
    setWindowTitle("Atomic orbital");
    frameName->setVisible(false);
    framePosition->setVisible(false);
  }

  pbGetPosition->setVisible(true);
  leOrbital->setVisible(false);
  leSite->setVisible(false);

  adjustSize();



  connect(&dataExchange
	  ,SIGNAL(dataAvailable(QObject *,const QObject *,const QString&
				,const void *))
	  ,this,SLOT(onDataAvailable(QObject *,const QObject *,const QString&
				     ,const void*)));


  QObject *obj=parent;
  while(true){
    QString isA(obj->metaObject()->className());
    if(isA == "BWEdit") break;
    obj=obj->parent();
  }
  pfrel=(((BWEdit*)obj)->document()->relativisticMode()==Xf::FullRelativistic);

  makeCompleter();
   
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ContribDlg::~ContribDlg()
{
  if(orbcompleter) delete orbcompleter;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::setData(const CoeffBasisInfo *cbi,
			 MolecularOrbital *mo,
			 MolecularOrbitalData *md)
{
  pmo=0;
  pmd=0;
  //myDebug() << "code"<<pcode;
  if(pcode==0|| pcode==2){
    pmd=md;
  }else if(pcode==1){
    pmo=mo;
    leName->setText(mo->name()());
    leName->selectAll();
    return;
  }

  pcbi=cbi;
  //myDebug() << "pcbi"<<pcbi->isReady();
  leSite->setVisible(!pcbi->isReady());
  leOrbital->setVisible(!pcbi->isReady());
  //pbGetPosition->setVisible(!pcbi->isReady());
  cmbOrbital->setVisible(pcbi->isReady());
  cmbSite->setVisible(pcbi->isReady());
  


  if(pcbi->isReady()){
    lbSite->setBuddy(cmbSite);
    lbOrbital->setBuddy(cmbOrbital);
    for(long isa=0;isa<cbi->nsite();isa++){
      String s;
      s.sprintf("%2s(%03ld)",cbi->element(isa)(),isa+1);
      cmbSite->insertItem(isa,s(),QVariant((int)isa));
    }
    for(int i=0;i<cmbSite->count();i++){
      if(cmbSite->itemData(i,Qt::UserRole).toInt() == md->site()){
	cmbSite->setCurrentIndex(i);
	break;
      }
    }
    bool gotit=false;
    for(int i=0;i<cmbOrbital->count();i++){
      if(cmbOrbital->itemText(i) == QString(md->orbital()())){
	cmbOrbital->setCurrentIndex(i);
	gotit=true;
	break;
      }
    }
    if(!gotit){
      cmbOrbital->setCurrentIndex(0);    
    }
  }else{
    makeCompleter();
    lbSite->setBuddy(leSite);
    lbOrbital->setBuddy(leOrbital);
    leSite->setText(String(md->site()+1)());
    leOrbital->setText(md->orbital()());
  }


  


  leXx->setText(Sprintf("%30.15f",md->xaxis()[0]).stripBlanks().chop("0")());
  leXy->setText(Sprintf("%30.15f",md->xaxis()[1]).stripBlanks().chop("0")());
  leXz->setText(Sprintf("%30.15f",md->xaxis()[2]).stripBlanks().chop("0")());

  leZx->setText(Sprintf("%30.15f",md->zaxis()[0]).stripBlanks().chop("0")());
  leZy->setText(Sprintf("%30.15f",md->zaxis()[1]).stripBlanks().chop("0")());
  leZz->setText(Sprintf("%30.15f",md->zaxis()[2]).stripBlanks().chop("0")());

  setLedPageText(false);

  if(pcode==0) return;

  leFR->setText(Sprintf("%12.8f",md->factor().real()).stripBlanks().chop("0")());
  leFI->setText(Sprintf("%12.8f",md->factor().imag()).stripBlanks().chop("0")());

  lePx->setText(Sprintf("%30.15f",md->position()[0]).stripBlanks().chop("0")());
  lePy->setText(Sprintf("%30.15f",md->position()[1]).stripBlanks().chop("0")());
  lePz->setText(Sprintf("%30.15f",md->position()[2]).stripBlanks().chop("0")());



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ContribDlg::dataAcceptable()
{
  QString s,txt="";


  
  if(pcode==1){
    if(leName->text().length()>17){
      txt+="Weight labels cannot be longer than 17 characters";
    }
  }else{
    try{
      checkData();
    }catch(Except &ex){
      txt+=ex.msg();
    }

    if(!pcbi->isReady()){
      ((IntValidator*)leSite->validator())->check(txt);
      long nmain,l,j,m;
      String::StateLabels ret=
	String::isStateLabel(String(leOrbital->text()),nmain,l,j,m);
      if(ret==String::SL_invalid || 
	 (leOrbital->text().toLower()=="all" && pcode==2)||
	 (leOrbital->text().toLower()=="allnlm" && pcode==2)){
	txt+="Invalid orbital\n";
      }

    }

    if(pcode!=0){
      
      ((RealValidator*)leFR->validator())->check(txt);
      ((RealValidator*)leFI->validator())->check(txt);
      
      ((RealValidator*)lePx->validator())->check(txt);
      ((RealValidator*)lePy->validator())->check(txt);
      ((RealValidator*)lePz->validator())->check(txt);
    }
    ((RealValidator*)leXx->validator())->check(txt);
    ((RealValidator*)leXy->validator())->check(txt);
    ((RealValidator*)leXz->validator())->check(txt);

    ((RealValidator*)leZx->validator())->check(txt);
    ((RealValidator*)leZy->validator())->check(txt);
    ((RealValidator*)leZz->validator())->check(txt);
  }


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }

  if(pcode!=1)
    if(!setLedPageText(true)) return false;

  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int ContribDlg::getData(MolecularOrbital *&mo,MolecularOrbitalData *&md,
			bool &editexisting)
{
  editexisting=peditexisting;
  QString t;
  

  if(pcode==1){
    mo=pmo;
    mo->setName(leName->text());
    md=0;
    return pcode;
  } 
  
  mo=0;
  md=pmd;
  
  
  if(pcbi->isReady()){
    long i=cmbSite->currentIndex();
    md->setSite(cmbSite->itemData(i,Qt::UserRole).toInt());
    md->setOrbital(String(cmbOrbital->currentText()));
  }else{
    t=leSite->text(); IntValidator::stripAllWhiteSpace(t);
    md->setSite(t.toLong()-1);
    md->setOrbital(leOrbital->text());
  }
  
  
  
  
  t=leXx->text(); IntValidator::stripAllWhiteSpace(t);
  md->xaxis()[0]=t.toDouble();
  t=leXy->text(); IntValidator::stripAllWhiteSpace(t);
  md->xaxis()[1]=t.toDouble();
  t=leXz->text(); IntValidator::stripAllWhiteSpace(t);
  md->xaxis()[2]=t.toDouble();
  
  t=leZx->text(); IntValidator::stripAllWhiteSpace(t);
  md->zaxis()[0]=t.toDouble();
  t=leZy->text(); IntValidator::stripAllWhiteSpace(t);
  md->zaxis()[1]=t.toDouble();
  t=leZz->text(); IntValidator::stripAllWhiteSpace(t);
  md->zaxis()[2]=t.toDouble();
  
  if(pcode==0) return pcode;;
  
  QString tre=leFR->text(); IntValidator::stripAllWhiteSpace(tre);
  QString tim=leFI->text(); IntValidator::stripAllWhiteSpace(tim);
  
  md->setFactor(Cmplx(tre.toDouble(),tim.toDouble()));
  
  t=lePx->text(); IntValidator::stripAllWhiteSpace(t);
  md->position()[0]=t.toDouble();
  t=lePy->text(); IntValidator::stripAllWhiteSpace(t);
  md->position()[1]=t.toDouble();
  t=lePz->text(); IntValidator::stripAllWhiteSpace(t);
  md->position()[2]=t.toDouble();
  return pcode;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::on_cmbSite_currentIndexChanged(int siteindex)
{
  if(siteindex==-1) return;

  
  if(!pcbi)return;
  
  QString currentOrbital=cmbOrbital->currentText();
  
  long i=cmbSite->currentIndex();
  long isa=cmbSite->itemData(i,Qt::UserRole).toInt();
  
  
  QStringList sl;
  if(pcode != 2){
    sl.append("all");
    sl.append("allnlm");
  }
  //myDebug()<<"FREL"<<pcbi->FREL();
  if(isa>=0 && isa<pcbi->nsite()){
    if(pcbi->FREL()){
      for(long ivr=0;ivr<pcbi->nRv(isa);ivr++){
	long l=pcbi->lRv(ivr,isa);
	long j=pcbi->jRv(ivr,isa);
	long deg=2*l+1;
	if(j==deg){
	  if(pcode != 2){
	    sl.append(Sprintf("%1ld%1c"
			      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      )());
	  }
	  for(long mv=0;mv<deg;mv++){
	    sl.append(Sprintf("%1ld%1c%+2ld"
	 		      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      ,mv-l
			      )());
	  }
	  
	  long j=2*l-1;
	  long degj=j+1;
	  if(pcode != 2){
	    sl.append(Sprintf("%1ld%1c%1ld/2"
			      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      ,j
			      )());
	  }
	  for(long m=0;m<degj;m++){
	    sl.append(Sprintf("%1ld%1c%1ld/2%+2ld/2"
			      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      ,j
			      ,2*m-j
			      )());
	  }
	  j=2*l+1;
	  degj=j+1;
	  if(pcode != 2){
	    sl.append(Sprintf("%1ld%1c%1ld/2"
			      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      ,j
			      )());
	  }
	  for(long m=0;m<degj;m++){
	    sl.append(Sprintf("%1ld%1c%1ld/2%+2ld/2"
			      ,pcbi->nnRv(ivr,isa)
			      ,String::intToSpdf(l)
			      ,j
			      ,2*m-j
			      )());
	  }
	}
      }
    }else{
      for(long ivr=0;ivr<pcbi->nRv(isa);ivr++){
	long l=pcbi->lRv(ivr,isa);
	if(pcode != 2){
	  sl.append(Sprintf("%1ld%1c"
			    ,pcbi->nnRv(ivr,isa)
			    ,String::intToSpdf(l)
			    )());
	}
	for(long mv=0;mv<pcbi->degRv(ivr,isa);mv++){
	  sl.append(Sprintf("%1ld%1c%+2ld"
			    ,pcbi->nnRv(ivr,isa)
			    ,String::intToSpdf(l)
			    ,mv-l
			    )());
	}
      }
    }
  }
  
  cmbOrbital->clear();
  cmbOrbital->addItems(sl);
  
  cmbOrbital->setCurrentIndex(MyMath::max<long>(cmbOrbital->findText(currentOrbital),0));
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ContribDlg::setLedPageText(bool betight)
{
  //myDebug() << "ContribDlg::setLedPageText";
  
  try{ checkData(); }catch(Except &ex){ return false;}
  
  tedPage->clear();

  long pl=-1;
  StringList sl;
  try{
    SqMatrix<Cmplx> ytr;
    sl=DMQuantAxis::calcAxesTrans(x,z,pl,ytr);
  }catch(Except &ex){ 
    tedPage->setTextColor(Qt::red);
    tedPage->append(ex.msg());
    QTextCursor cursor=tedPage->textCursor();
    cursor.setPosition(0);
    tedPage->setTextCursor(cursor);
    
    if(betight){
      QMessageBox::warning(0,"Checking data"
			   ,ex.msg(),QMessageBox::Yes );
    }
    return false;
  }
  
  
  tedPage->setTextColor(Qt::red);
  tedPage->append(sl[1]());
  tedPage->setTextColor(Qt::black);
  tedPage->append(sl[0]());

  QTextCursor cursor=tedPage->textCursor();
  cursor.setPosition(0);
  tedPage->setTextCursor(cursor);

  return true;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::checkData()
{
  double d;
  String s;

  bool ok=true;
  String msg;

  if(pcode==2){
    String orb;
    if(pcbi->isReady()){
      orb=cmbOrbital->currentText();
    }else{
      orb=leOrbital->text();
    }
    long nmain,l,j,m;
    String::StateLabels ret=
      String::isStateLabel(orb,nmain,l,j,m);
    if(!(ret==String::SL_nlm || ret==String::SL_nljm)){
      msg+="Orbital for molecular pattern contributions must be fully qualified!\n";
      ok=false;
    }

 
  }
  

  
  s=leXx->text().toAscii().constData();
  try{ d=s.convertToFortranReal();  x[0]=d; }catch(Except &ex){ 
    msg+="X1 does not contain a valid real number\n";
    ok=false;
  };


  s=leXy->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); x[1]=d; }catch(Except &ex){ 
    msg+="X2 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leXz->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); x[2]=d;}catch(Except &ex){ 
    msg+="X3 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leZx->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[0]=d;}catch(Except &ex){ 
    msg+="Z1 does not contain a valid real number\n";
    ok=false;
  };
  

  s=leZy->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[1]=d;}catch(Except &ex){ 
    msg+="Z2 does not contain a valid real number\n";
    ok=false;
  };


  s=leZz->text().toAscii().constData();
  try{ d=s.convertToFortranReal(); z[2]=d;}catch(Except &ex){ 
    msg+="Z3 does not contain a valid real number\n";
    ok=false;
  };
  

  if(!ok){
    throw Except(msg);
  }

  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void ContribDlg::onLedPageTextEdited(const QString &)
{
  setLedPageText(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::on_pbGetAtom_clicked()
{
  //myDebug ()<<"ContribDlg::on_pbGetAtom_clicked";

  waitMode(true);


  dataExchange.registerListener(pbGetAtom,"atom data");



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::on_pbGetPosition_clicked()
{
  //myDebug ()<<"ContribDlg::on_pbGetPosition_clicked";

  waitMode(true);


  dataExchange.registerListener(pbGetPosition,"atom data");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::onDataAvailable(QObject *,const QObject *listener
				 ,const QString& // expectedData
				 ,const void* data)
{
  //myDebug() <<data;
  //QString *s=qobject_cast<QString *>(data);
  //if(s)
  //myDebug() << "onDataAvailable" << object << data;
  if(data){
    const Atom *at=(Atom *)data;
    double unit=1./constant_aBtoAng;
    if(at->lengthunit==Xf::BohrRadius) unit=1.;
    Vec3 p=at->tauD*unit;

    if(listener==pbGetAtom){
      long site=at->site;
      if(pcbi->isReady()){
	for(int i=0;i<cmbSite->count();i++){
	  if(cmbSite->itemData(i,Qt::UserRole).toInt() == site){
	    cmbSite->setCurrentIndex(i);
	    break;
	  }
	}
      }else{
	leSite->setText(String(site+1)());
      }

      lePx->setText(Sprintf("%30.15f",p[0]).stripBlanks().chop("0")());
      lePy->setText(Sprintf("%30.15f",p[1]).stripBlanks().chop("0")());
      lePz->setText(Sprintf("%30.15f",p[2]).stripBlanks().chop("0")());
      
    }else if(listener==pbGetPosition){
      lePx->setText(Sprintf("%30.15f",p[0]).stripBlanks().chop("0")());
      lePy->setText(Sprintf("%30.15f",p[1]).stripBlanks().chop("0")());
      lePz->setText(Sprintf("%30.15f",p[2]).stripBlanks().chop("0")());
    }
  }

  activateWindow();
  raise();
  //myDebug() << "onDataAvailable gotit" ;
  
  waitMode(false);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::waitMode(bool on)
{
  //myDebug() <<"ContribDlg::waitMode"<<on;

  if(!on){
    dataExchange.unRegisterListener(pbGetAtom);
    dataExchange.unRegisterListener(pbGetPosition);
  }
  foreach (QObject *o,children()){
    QWidget *w=qobject_cast<QWidget *>(o);
    if(w){
      w->setEnabled(!on);
    }
  }
  piswaiting=on;  
  
  if(on){
    lbInformation->
      setText("<font color='red'>Click on an atom or press escape!</font>");
    lbInformation->setEnabled(true);
  }else{
    lbInformation->setText("");
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ContribDlg::makeCompleter()
{
  QCompleter *c=leOrbital->completer();
  if (c)
    QObject::disconnect(c, 0, this, 0);
  QStringList list;
  if(pcode != 2){
    list<<"all"<<"allnlm";
  }
  for(int n=0;++n<8;){
    for(int l=-1;++l<MyMath::min(4,n);){
      int deg=2*l+1;
      if(pcode != 2){
	list<<Sprintf("%d%c",n,String::intToSpdf(l))();
      }
      for(int m=-l-1;++m<=l;){
	list<<Sprintf("%d%c%+d",n,String::intToSpdf(l),m)();
	}
      if(pfrel){
	for(int kap=l;kap>=-l-1;kap-=deg){
	  if(!kap) continue;
	  int j=2*abs(kap)-1;
	  if(pcode != 2){
	    list<<Sprintf("%d%c%d/2",n,String::intToSpdf(l),j)();
	  }
	  for(int mu=-j;mu<=j;mu+=2){
	    list<<Sprintf("%d%c%d/2%+d/2",n,String::intToSpdf(l),j,mu)();
	  }
	}
      }
    }
  }

  
  orbcompleter=new QCompleter(list,this);
  orbcompleter->setCaseSensitivity(Qt::CaseInsensitive);
  leOrbital->setCompleter(orbcompleter);
  leOrbital->setText(list.at(0));
  if(c) delete c;
}

/*EOF*/ 
