/*************************************************************************
 * @(#)CutDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox> 
#include <QCloseEvent> 
#include "CutDlg.hh"
#include "Valid.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
CutDlg::CutDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,0,"CutDialog",": Cut planes");



  new RealValidator(this,leX,"The x-component of plane vector",-1.e10,1.e10);
  new RealValidator(this,leY,"The y-component of plane vector",-1.e10,1.e10);
  new RealValidator(this,leZ,"The z-component of plane vector",-1.e10,1.e10);


  new IntValidator(this,leNp,"The number of cuts",1,10000);

  new RealValidator(this,leFr,"The from-value",-1.e10,1.e10);
  new RealValidator(this,leTo,"The to-value",-1.e10,1.e10);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CutDlg::setData( long icut, double ncut[3], long np, double fro, double to )
{
  t_icut=icut;
  cbCut->setChecked(icut);
  
  t_n0=QString().setNum(ncut[0],'g',10);
  leX->setText(t_n0);
  t_n1=QString().setNum(ncut[1],'g',10);
  leY->setText(t_n1);
  t_n2=QString().setNum(ncut[2],'g',10);
  leZ->setText(t_n2);
  t_np=np;
  leNp ->setText(QString().setNum(np));

  t_fr=QString().setNum(fro,'g',10);
  leFr->setText(t_fr);
  t_to=QString().setNum(to,'g',10);
  leTo->setText(t_to);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CutDlg::getData( long *icut, double ncut[3] , long *np, double *fro, double *to, long *chn )
{
  
  *icut=cbCut->isChecked();
  *chn = (  *icut != t_icut );
  t_icut=*icut;

  bool ok;
  QString t=leX->text();
  IntValidator::stripAllWhiteSpace(t);
  ncut[0]=t.toDouble(&ok);
  *chn |= (  t != t_n0 );
  t_n0=t;

  t=leY->text();
  IntValidator::stripAllWhiteSpace(t);
  ncut[1]=t.toDouble(&ok);
  *chn |= (  t != t_n1 );
  t_n1=t;
  t=leZ->text();
  IntValidator::stripAllWhiteSpace(t);
  ncut[2]=t.toDouble(&ok);
  *chn |= (  t != t_n2 );
  t_n2=t;

  t=leNp->text();
  IntValidator::stripAllWhiteSpace(t);
  *np=t.toLong(&ok);
  *chn |= (  *np != t_np );
  t_np=*np;
  t=leFr->text();
  IntValidator::stripAllWhiteSpace(t);
  *fro=t.toDouble(&ok);
  *chn |= (  t != t_fr );
  t_fr=t;
  
  t=leTo->text();
  IntValidator::stripAllWhiteSpace(t);
  *to=t.toDouble(&ok);
  *chn |= (  t != t_to );
  t_to=t;

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool CutDlg::dataAcceptable()
{
  //int i;

  QString s,txt="";

  ((IntValidator*)leX->validator())->check(txt);
  ((IntValidator*)leY->validator())->check(txt);
  ((IntValidator*)leZ->validator())->check(txt);
  ((IntValidator*)leNp->validator())->check(txt);
  ((IntValidator*)leFr->validator())->check(txt);
  ((IntValidator*)leTo->validator())->check(txt);


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
    return false;
  }else{
    return true;
  }


}



/*EOF*/ 
