/*************************************************************************
 * @(#)Delegates.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   17 Jul 2009
 *************************************************************************
 */


#include <QApplication>
#include <QDoubleSpinBox>
#include <QColorDialog>
#include <QDebug>
#include <QComboBox>
#include <QPainter>
#include <QEvent>
#include <QKeyEvent>
#include <QLineEdit>

#include "Delegates.hh"
#include "Valid.hh"
#include "ValueArray.hh"
#include "MyDebug.hh"
#include "BondPolyDlg.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
LineEditDelegate::LineEditDelegate(QObject *parent,QValidator *valid)
  : QItemDelegate(parent),pvalid(valid)
{
  NSF("LineEditDelegate::LineEditDelegate");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *LineEditDelegate::createEditor(QWidget *parent
					, const QStyleOptionViewItem &/*option*/,
					const QModelIndex &/*index*/) const
{
  QLineEdit *editor = new QLineEdit(parent);
  if(pvalid)
    editor->setValidator(pvalid);

  return editor;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LineEditDelegate::setEditorData(QWidget *editor
, const QModelIndex &index) const
{
  QString value = index.model()->data(index, Qt::EditRole).toString();
  
  QLineEdit *lineeditor = static_cast<QLineEdit*>(editor);
  lineeditor->setText(value);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LineEditDelegate::setModelData(QWidget *editor
				    , QAbstractItemModel *model,
				    const QModelIndex &index) const
{
  QLineEdit *lineeditor = static_cast<QLineEdit*>(editor);

  QString value = lineeditor->text();

  model->setData(index, value, Qt::EditRole);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LineEditDelegate::updateEditorGeometry(QWidget *editor,
		      const QStyleOptionViewItem &option
			 , const QModelIndex &/*index*/) const
{ 
  editor->setGeometry(option.rect);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool LineEditDelegate::eventFilter(QObject *object,QEvent *event)
{

  NSF("LineEditDelegate::eventFilter");
  //  myDebug() << nsubfunc << event->type();
  QWidget *editor = qobject_cast<QWidget*>(object);
  

  if (!editor) return false;
  
  if (event->type() == QEvent::KeyPress) {
    //myDebug() << nsubfunc << static_cast<QKeyEvent *>(event)->key()
    //	     << Qt::Key_Tab;

    long key=static_cast<QKeyEvent *>(event)->key();
    switch (key) {
    case Qt::Key_Tab:
    case Qt::Key_Backtab:
    case Qt::Key_Enter:
    case Qt::Key_Up:
    case Qt::Key_Down:
    case Qt::Key_Return:
#ifndef QT_NO_LINEEDIT
      //QLineEdit *e = qobject_cast<QLineEdit*>(editor);
      //myDebug() << "hier" << e;
      //myDebug() << "e->hasAcceptableInput()" << e->hasAcceptableInput();
      if (QLineEdit *e = qobject_cast<QLineEdit*>(editor)){
	if (!e->hasAcceptableInput()){
	  //myDebug() << "return false";
	  return true;//false;
	}
      }
      if(    
	 key == Qt::Key_Tab ||  
	 key == Qt::Key_Backtab ||
	 key == Qt::Key_Up ||
	 key == Qt::Key_Down ){
	emit commitData(editor);
       	emit closeEditor(editor, QAbstractItemDelegate::EditNextItem);
	return true;
      }
#endif // QT_NO_LINEEDIT
      QMetaObject::invokeMethod(this, "_q_commitDataAndCloseEditor",
				Qt::QueuedConnection, Q_ARG(QWidget*, editor));
      return false;
    }
  }
  
  return  QItemDelegate::eventFilter(object,event);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
ColorDelegate::ColorDelegate(QObject *parent)
  : QStyledItemDelegate(parent)
{
  //NSF("ColorDelegate::ColorDelegate");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *ColorDelegate::createEditor(QWidget *parent
				     ,const QStyleOptionViewItem &option,
				     const QModelIndex &index) const
{
  //  myDebug() <<"ColorDelegate::createEditor";
  Q_UNUSED(option);
  Q_UNUSED(index);
  QColorDialog *editor=new QColorDialog(parent);
  //ColorButton *editor=new ColorButton(parent);
  connect(editor,SIGNAL(finished(int)),this,SLOT(onColorChanged(int)));
  //editor->setMaximumSize(QSize(sizeHint(option,index).width(),20));
  //editor->setMaximumSize(QSize(20,20));
  //editor->resize(QSize(20,20));

  return editor;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorDelegate::onColorChanged(int state)
{
  Q_UNUSED(state);
  //myDebug("%x",color);
  QColorDialog *ceditor=qobject_cast<QColorDialog*>(sender());
  //ColorButton *ceditor=qobject_cast<ColorButton*>(sender());
  if(state==QDialog::Accepted)
    emit commitData(ceditor);
  emit closeEditor(ceditor);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorDelegate::
setEditorData(QWidget *editor,const QModelIndex &index) const
{
  QRgb color=index.model()->data(index, Qt::EditRole).toUInt();
  
  QColorDialog *ceditor=static_cast<QColorDialog*>(editor);
  //ColorButton *ceditor=static_cast<ColorButton*>(editor);
  ceditor->setCurrentColor(color);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorDelegate::setModelData(QWidget *editor,QAbstractItemModel *model,
				 const QModelIndex &index) const
{
  QColorDialog *ceditor=static_cast<QColorDialog*>(editor);
  //ColorButton *ceditor=static_cast<ColorButton*>(editor);

  QRgb color=ceditor->currentColor().rgb()&0x00ffffff;
  //  myDebug("setModelData %x",color);
  model->setData(index, color, Qt::EditRole);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorDelegate::updateEditorGeometry(QWidget *editor,
					 const QStyleOptionViewItem &option
					 ,const QModelIndex &index) const
{
  Q_UNUSED(index);
  editor->setGeometry(option.rect);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QSize ColorDelegate::sizeHint(const QStyleOptionViewItem &option,
				    const QModelIndex &index) const
{
  Q_UNUSED(option);
  Q_UNUSED(index);
  return QSize(22,22);
  return QStyledItemDelegate::sizeHint(option,index);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ColorDelegate::paint(QPainter *painter,const QStyleOptionViewItem &option
	   ,const QModelIndex &index)const
{
  Q_UNUSED(painter);
  Q_UNUSED(option);
  Q_UNUSED(index);
  
  if(!index.isValid())return;
  QRgb color=index.model()->data(index, Qt::EditRole).toUInt();

  if (option.state & QStyle::State_Selected)
    painter->fillRect(option.rect, option.palette.highlight());

  //  myDebug("option.state<%x>",option.state);
  
  if ( 
      (option.state & QStyle::State_Active)&&
      (option.state & QStyle::State_MouseOver)
       )
    painter->fillRect(option.rect, QBrush(option.palette.highlight()
					  .color().lighter(170)));

  bool high=false;
  QColor rim=Qt::darkGray;
  QColor rim2=Qt::gray;
  if (!(option.state & QStyle::State_Enabled)){
    rim=Qt::darkGray;
    color=0xC6C3BF;
  }else{
    if (option.state & QStyle::State_MouseOver){
      rim2=option.palette.highlight().color();
      high=true;
    }
  }

  QRect rect=option.rect;
  QRect innerrect=rect;
  QRect innerrect2=rect;

  painter->save();
  painter->setRenderHint(QPainter::Antialiasing, true);
  painter->setBrush(QBrush(color));
  if(high){
    painter->setPen(QPen(rim2));
    innerrect2.adjust(3,3,-3,-3);
    painter->drawRoundedRect(innerrect2,2,2);
  }
  painter->setPen(QPen(rim));
  innerrect.adjust(4,4,-4,-4);
//   int yOffset = (rect.height() - PaintingScaleFactor) / 2;
//   painter->translate(rect.x(), rect.y() + yOffset);
//   painter->scale(PaintingScaleFactor, PaintingScaleFactor);
  painter->drawRoundedRect(innerrect,2,2);
  painter->restore();
}











/* ***********************************************************************
 *
 * ***********************************************************************
 */
DoubleSpinBoxDelegate::DoubleSpinBoxDelegate(QObject *parent)
  : QStyledItemDelegate(parent),minimum(0.),maximum(1.),singlestep(0.1)
  ,decimals(1),gotsizehint(false),sizehint(QSize(12,12))
{
  NSF("DoubleSpinBoxDelegate::DoubleSpinBoxDelegate");
}
  
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *DoubleSpinBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,
			const QModelIndex &index) const
{
  Q_UNUSED(option);
  Q_UNUSED(index);

  QDoubleSpinBox *editor = new QDoubleSpinBox(parent);
  editor->setMinimum(minimum);
  editor->setMaximum(maximum);
  editor->setDecimals(decimals);
  editor->setSingleStep(singlestep);
  
  return editor;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DoubleSpinBoxDelegate::setEditorData(QWidget *editor
					,const QModelIndex &index) const
{
  double value=index.model()->data(index, Qt::EditRole).toDouble();

  QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
  spinBox->setValue(value);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DoubleSpinBoxDelegate::setModelData(QWidget *editor
				       ,QAbstractItemModel *model,
				       const QModelIndex &index) const
{
  QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
  spinBox->interpretText();
  double value=spinBox->value();

  model->setData(index,value,Qt::EditRole);

}
  
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DoubleSpinBoxDelegate::
updateEditorGeometry(QWidget *editor
		     ,const QStyleOptionViewItem &option
		     ,const QModelIndex &index) const
{
  Q_UNUSED(index);
  editor->setGeometry(option.rect);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QSize DoubleSpinBoxDelegate::sizeHint(const QStyleOptionViewItem &option,
				    const QModelIndex &index) const
{
  Q_UNUSED(option);
  Q_UNUSED(index);
  //myDebug() << "DoubleSpinBoxDelegate::sizeHint";
  if(!gotsizehint){
    gotsizehint=true;
    QDoubleSpinBox *box=new QDoubleSpinBox();
    box->setDecimals(decimals);
    box->setValue(1.);
    sizehint=box->sizeHint();
    delete box;
  }
  return sizehint;
  
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ComboDelegate::ComboDelegate(QObject *parent,
			     const ValueArray<Pair<int,const char *> > *content)
  : QStyledItemDelegate(parent),pcontent(content)
{
  NSF("ComboDelegate::ComboDelegate");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *ComboDelegate::createEditor(QWidget *parent
				     ,const QStyleOptionViewItem &option,
				     const QModelIndex &index) const
{
  //  myDebug() <<"ComboDelegate::createEditor";
  Q_UNUSED(option);
  Q_UNUSED(index);
  QComboBox *editor=new QComboBox(parent);
  for(int i=-1;++i<pcontent->size();){
    editor->insertItem(i,pcontent->at(i).v2,pcontent->at(i).v1);
  }
  //  connect(editor,SIGNAL(finished(int)),this,SLOT(onColorChanged(int)));

  return editor;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboDelegate::setEditorData(QWidget *editor,const QModelIndex &index) const
{
  int data=index.model()->data(index, Qt::EditRole).toInt();
  
  QComboBox *ceditor=static_cast<QComboBox*>(editor);
  ceditor->setCurrentIndex(pcontent->indexOf(makePair<int,const char*>(data,"")));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboDelegate::setModelData(QWidget *editor,QAbstractItemModel *model,
				 const QModelIndex &index) const
{
  QComboBox *ceditor=static_cast<QComboBox*>(editor);

  int cindex=ceditor->currentIndex();
  
  model->setData(index, ceditor->itemData(cindex).toInt(), Qt::EditRole);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ComboDelegate::updateEditorGeometry(QWidget *editor,
					 const QStyleOptionViewItem &option
					 ,const QModelIndex &index) const
{
  Q_UNUSED(index);
  editor->setGeometry(option.rect);

}






/*EOF*/ 
