/* @(#)Delegates.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _DELEGATES_H
#define _DELEGATES_H 1

#include <QStyledItemDelegate>
#include <QItemDelegate>
#include "Pair.hh"

template <class T>
class ValueArray;

class QLineEdit;
class QValidator;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class LineEditDelegate : public QItemDelegate
{
  Q_OBJECT

public:
  LineEditDelegate(QObject *parent = 0,QValidator *valid=0);
  
  QWidget *createEditor(QWidget *parent
			, const QStyleOptionViewItem &option,
			const QModelIndex &index) const;
  
  void setEditorData(QWidget *editor, const QModelIndex &index) const;
  void setModelData(QWidget *editor, QAbstractItemModel *model,
		    const QModelIndex &index) const;
  
  void updateEditorGeometry(QWidget *editor,
        const QStyleOptionViewItem &option, const QModelIndex &index) const;
protected :
  virtual bool eventFilter(QObject *editor,QEvent *event);

private:
  QValidator *pvalid;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ColorDelegate : public QStyledItemDelegate
{
  Q_OBJECT;
  
 public:
  ColorDelegate(QObject *parent = 0);
  
  QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
			const QModelIndex &index) const;
  
  void setEditorData(QWidget *editor, const QModelIndex &index) const;
  void setModelData(QWidget *editor, QAbstractItemModel *model,
		    const QModelIndex &index) const;
  
  void updateEditorGeometry(QWidget *editor,
			    const QStyleOptionViewItem &option, const QModelIndex &index) const;
  QSize sizeHint(const QStyleOptionViewItem &option,
		 const QModelIndex &index) const;
  virtual void	paint(QPainter *painter,const QStyleOptionViewItem &option
		      ,const QModelIndex &index)const;
private slots:
  void onColorChanged(int);
};
/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ComboDelegate : public QStyledItemDelegate
{
  Q_OBJECT;
  
 public:
  ComboDelegate(QObject *parent,
		const ValueArray<Pair<int,const char *> > *content);
  
  QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
			const QModelIndex &index) const;
  
  void setEditorData(QWidget *editor, const QModelIndex &index) const;
  void setModelData(QWidget *editor, QAbstractItemModel *model,
		    const QModelIndex &index) const;
  
  void updateEditorGeometry(QWidget *editor,
			    const QStyleOptionViewItem &option, const QModelIndex &index) const;
private slots:
  //  void onColorChanged(int);
  
private:
  const ValueArray<Pair<int,const char*> > *pcontent;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class DoubleSpinBoxDelegate : public QStyledItemDelegate
{
  Q_OBJECT;
  
 public:
  DoubleSpinBoxDelegate(QObject *parent = 0);
  
  QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
			const QModelIndex &index) const;
  
  void setEditorData(QWidget *editor, const QModelIndex &index) const;
  void setModelData(QWidget *editor, QAbstractItemModel *model,
		    const QModelIndex &index) const;
  
  void updateEditorGeometry(QWidget *editor,
			    const QStyleOptionViewItem &option, const QModelIndex &index) const;
  QSize sizeHint(const QStyleOptionViewItem &option,
			       const QModelIndex &index) const;

  inline void setMinimum(double m){minimum=m;};
  inline void setMaximum(double m){maximum=m;};
  inline void setSingleStep(double s){singlestep=s;};
  inline void setDecimals(int d){decimals=d;};

 private:
  double minimum;
  double maximum;
  double singlestep;
  int decimals;
  mutable bool gotsizehint;
  mutable QSize sizehint;
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _DELEGATES_H */

