/*************************************************************************
 * @(#)ExecPathDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   09 Feb 2012
 *************************************************************************
 */

#include <QMenu>
#include <QMessageBox>
#include <QPainter>
#include <QFileDialog>
#include <QLineEdit>
#include <QSignalMapper>
#include "ExecPathDlg.hh"
#include "MyMath.hh"
#include "Fplo.hh"
#include "HelpDlg.hh"
#include "MyDebug.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ButtonedLineEdit::ButtonedLineEdit(QWidget *parent)
  :QWidget(parent)
{
  horizontalLayout = new QHBoxLayout(this);
  horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
  horizontalLayout->setContentsMargins(0, 0, 0, 0);
  pb=new QPushButton(this);
  pb->setText("Browse");
  horizontalLayout->addWidget(pb);

  ple=new QLineEdit();
  horizontalLayout->addWidget(ple);

  ple->setFocus();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ButtonedLineEdit::~ButtonedLineEdit()
{
  if(pb) delete pb;
  if(ple) delete ple;
  if(horizontalLayout) delete horizontalLayout;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void ExecPathDelegate::paint(QPainter *painter,
			     const QStyleOptionViewItem &option,
			     const QModelIndex &index)const
{
  if (index.data().canConvert<QString>()) {
    QString v=index.data().toString();
    //if (option.state & QStyle::State_Selected)
    //painter->fillRect(option.rect, option.palette.highlight());
    
    //    QRgb color=Qt::red;
    
    if (option.state & QStyle::State_Selected)
      painter->fillRect(option.rect, option.palette.highlight());
    
    //  myDebug("option.state<%x>",option.state);
    
    if ( 
	(option.state & QStyle::State_Active)&&
	(option.state & QStyle::State_MouseOver)
	)
      painter->fillRect(option.rect, QBrush(option.palette.highlight()
					    .color().lighter(170)));
    
    bool high=false;
    QColor rim=Qt::darkGray;
    QColor rim2=Qt::gray;
    if (!(option.state & QStyle::State_Enabled)){
      rim=Qt::darkGray;
      //color=0xC6C3BF;
    }else{
      if (option.state & QStyle::State_MouseOver){
	rim2=option.palette.highlight().color();
	high=true;
      }
    }
    
    QFont fnt=qApp->font();
    QString txt="Browse";
    QRect txtrect=painter->boundingRect(0,0,0,0,Qt::AlignCenter,"XX"+txt+"XX");
    myDebug()<<txtrect.width();

    QRect rect=option.rect;
    rect.setWidth(txtrect.width());
    QRect innerrect=rect;
    QRect innerrect2=rect;
    int rad=4;
    
    QBrush bcolor=option.palette.button();
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing, true);
    painter->setBrush(bcolor);
    if(high){
      painter->setPen(QPen(rim2));
      innerrect2.adjust(2,2,-2,-2);
      painter->drawRoundedRect(innerrect2,rad,rad);
    }
    painter->setPen(QPen(rim));
    painter->setPen(QPen(bcolor.color()));
    innerrect.adjust(2,2,-3,-3);
    //   int yOffset = (rect.height() - PaintingScaleFactor) / 2;
    //   painter->translate(rect.x(), rect.y() + yOffset);
    //   painter->scale(PaintingScaleFactor, PaintingScaleFactor);
    painter->drawRoundedRect(innerrect,rad,rad);
    painter->setPen(QPen(Qt::black));
    painter->drawText(rect, Qt::AlignCenter,txt);
    painter->restore();
    
    //     starRating.paint(painter, option.rect, option.palette,
    // 		   StarRating::ReadOnly);
  } else {
    QStyledItemDelegate::paint(painter, option, index);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDelegate::updateEditorGeometry(QWidget *editor,
					    const QStyleOptionViewItem &option
					    ,const QModelIndex &index) const
{
  Q_UNUSED(index);
  editor->setGeometry(option.rect);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QSize ExecPathDelegate::sizeHint(const QStyleOptionViewItem &option,
				 const QModelIndex &index) const
{
  //  if (qVariantCanConvert<QString>(index.data())) {
//     QString v=index.data().toString();
//     return starRating.sizeHint();
//   } else {
    return QStyledItemDelegate::sizeHint(option, index);
    //}
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
				    const QModelIndex &index) const
{
  if (index.data().canConvert<QString>()) {
    ButtonedLineEdit *lineEditor = qobject_cast<ButtonedLineEdit *>(editor);
    model->setData(index, qVariantFromValue(lineEditor->text()));
  } else {
    QStyledItemDelegate::setModelData(editor, model, index);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDelegate::commitAndCloseEditor()
{
  ButtonedLineEdit *editor = qobject_cast<ButtonedLineEdit *>(sender());
  emit commitData(editor);
  emit closeEditor(editor);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDelegate::setEditorData(QWidget *editor,
				     const QModelIndex &index) const
{
  if (index.data().canConvert<QString>()) {
    QString v = index.data().value<QString>();
    ButtonedLineEdit *lineEditor = qobject_cast<ButtonedLineEdit *>(editor);
    lineEditor->setText(v);
  } else {
    QStyledItemDelegate::setEditorData(editor, index);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *ExecPathDelegate::createEditor(QWidget *parent,
					const QStyleOptionViewItem &option,
					const QModelIndex &index) const
  
{
  if (index.data().canConvert<QString>()) {
    ButtonedLineEdit *editor = new ButtonedLineEdit(parent);
    connect(editor, SIGNAL(editingFinished()),
	    this, SLOT(commitAndCloseEditor()));
    return editor;
  } else {
    return QStyledItemDelegate::createEditor(parent, option, index);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecPathDlg::ExecPathDlg(QWidget *parent,Fplo *fplo)
  :QDialog(parent),insertMenu(0),pfplo(fplo),delegate(0)
{
  setupUi(this);
  QStringList headers;
  headers<<"Type"<<"Value";
  twPatterns->setColumnCount(2);
  twPatterns->setHorizontalHeaderLabels(headers);

  ptoolbuttons.createToolButtonsActions(twPatterns,"high symmetry point"
					,tbInsert,tbDelete,tbUp,tbDown);

  ptoolbuttons.connectSlots(this,SLOT(onTbInsertClicked()),
			    SLOT(onTbDeleteClicked()),
			    SLOT(onTbMoveClicked()),
			    SLOT(onTbMoveClicked()));

  signalMapper = new QSignalMapper(this);
  connect(signalMapper,SIGNAL(mapped(int)),
	  this,SLOT(slotInsertPattern(int)));

  QMenu *m=insertMenu=new QMenu(tbInsert);
  QAction *a;
  a=m->addAction("Insert &Env var");  
  signalMapper->setMapping(a,ExecPathes::EnvVar);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  a=m->addAction("Insert &Directory");  
  signalMapper->setMapping(a,ExecPathes::Directory);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  a=m->addAction("Insert E&xecutable");  
  signalMapper->setMapping(a,ExecPathes::Executable);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  tbInsert->setMenu(insertMenu);

  delegate=new ExecPathDelegate();
  //  twPatterns->setItemDelegateForColumn(1,delegate);

  const ExecPathes &execpathes=pfplo->execPathes();
  int row=0;
  foreach(ExecPathes::PatternType key,execpathes.keys()){
    twPatterns->insertRow(row);
    QTableWidgetItem *item;
    twPatterns->setItem(row,0,item=new QTableWidgetItem());
    item->setText(ExecPathes::keyName(key));
    item->setData(Qt::UserRole,key);
    item->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
    twPatterns->setItem(row,1,item=new QTableWidgetItem());
    item->setText(execpathes.value(key));
    if(key!=ExecPathes::EnvVar)
      item->setIcon(QIcon(":/Icons/browsebutton"));
    //twPatterns->openPersistentEditor(item);
    
    row++;

  }
  adjustTwPatterns();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ExecPathDlg::~ExecPathDlg()
{
  if(signalMapper) delete signalMapper;
  if(insertMenu) delete insertMenu; insertMenu=0;
  if(delegate) delete delegate; delegate=0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","ExecPathDlg");
  h->show();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::slotInsertPattern(int m)
{
  NSF("ExecPathDlg::slotInsertPattern"); 

  ExecPathes::PatternType mode=(ExecPathes::PatternType)m;

  QModelIndex index=twPatterns->currentIndex();
  int row=0;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
    col=index.column();
  }
  
  twPatterns->insertRow(row);
  QTableWidgetItem *item;
  twPatterns->setItem(row,0,item=new QTableWidgetItem());
  item->setText(ExecPathes::keyName(mode));
  item->setData(Qt::UserRole,mode);
  item->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
  twPatterns->setItem(row,1,item=new QTableWidgetItem());
  item->setText("??");
  if(mode!=ExecPathes::EnvVar)
    item->setIcon(QIcon(":/Icons/browsebutton"));

  
  twPatterns->scrollToItem(twPatterns->item(row,0));
  twPatterns->setCurrentCell(row,col);
  adjustTwPatterns();
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::onTbInsertClicked()
{
  QPoint pos;
  pos=tbInsert->mapToGlobal(QPoint(0,tbInsert->height()));
  //    pos=QCursor::pos();

  tbInsert->menu()->popup(pos);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::onTbDeleteClicked()
{

  int numrows=twPatterns->rowCount();

  if(numrows<=0) return;

  QTableWidgetItem *currentitem=twPatterns->currentItem();
  if(!currentitem 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a row first."
			   , 1, 0,0 );
    return;
  }

  int row=twPatterns->currentRow();
  int col=twPatterns->currentColumn();


  twPatterns->removeRow(row);
  twPatterns->setCurrentItem(twPatterns->
			     item(MyMath::min<long>(row,numrows-2),col));
  adjustTwPatterns();


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::onTbMoveClicked()
{

  QTableWidgetItem *currentitem=twPatterns->currentItem();
  if(!currentitem){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a row first."
			   , 1, 0,0 );
    return;
  }

  int row=twPatterns->currentRow();
  int col=twPatterns->currentColumn();
  int by=-1;
  if(sender()==ptoolbuttons.pmoveDownAction) by=1;

  if(twPatterns->item(row+by,0)){
    QTableWidgetItem *item0=twPatterns->takeItem(row,0);
    QTableWidgetItem *item1=twPatterns->takeItem(row,1);
    twPatterns->removeRow(row);
    int newrow=MyMath::max<long>(0,row+by);
    twPatterns->insertRow(newrow);
    twPatterns->setItem(newrow,0,item0);
    twPatterns->setItem(newrow,1,item1);
    twPatterns->setCurrentCell(row+by,col);
    adjustTwPatterns();
  }


  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::adjustTwPatterns()
{
  int rows=twPatterns->rowCount();
  int cols=twPatterns->columnCount();
  for(int irow=0;irow<rows;irow++)
    twPatterns->resizeRowToContents(irow);
  for(int icol=0;icol<cols;icol++){
    twPatterns->resizeColumnToContents(icol);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::on_pbOK_clicked()
{
  ExecPathes execpathes;
  execpathes.clear();
  for(int row=0;row<twPatterns->rowCount();row++){
    QTableWidgetItem *item=twPatterns->item(row,0);
    ExecPathes::PatternType mode=
      (ExecPathes::PatternType)item->data(Qt::UserRole).toInt();
    item=twPatterns->item(row,1);
    execpathes.append(mode,item->text());
  }
  pfplo->setExecPathes(execpathes);
  
  QDialog::accept(); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ExecPathDlg::on_twPatterns_cellClicked(int row,int col)
{
  if(col==1){
    QTableWidgetItem *item=twPatterns->item(row,0);
    QTableWidgetItem *item1=twPatterns->item(row,1);
    ExecPathes::PatternType mode=
      (ExecPathes::PatternType)item->data(Qt::UserRole).toInt();

    if(mode==ExecPathes::Directory){
      QFileDialog *dlg=new QFileDialog(this,"select a directory",".");
      dlg->setAcceptMode(QFileDialog::AcceptOpen);
      dlg->setFileMode(QFileDialog::Directory);
      dlg->setOption(QFileDialog::ShowDirsOnly, true);
#ifdef Q_OS_MAC
      dlg->setOptions(QFileDialog::DontUseNativeDialog);
#endif
      dlg->setDirectory(item1->text());
      long ret;
      QString filename;
      if((ret=dlg->exec())==QDialog::Accepted){
	filename=dlg->selectedFiles().at(0);
      }
      delete (dlg);
      if(!filename.isNull() && ret==QDialog::Accepted){
	item1->setText(filename);
      }

    
      adjustTwPatterns();

    }else if(mode==ExecPathes::Executable){
      QFileDialog *dlg=new QFileDialog(this,"select an executable",".");
      dlg->setAcceptMode(QFileDialog::AcceptOpen);
      dlg->setFileMode(QFileDialog::ExistingFile);
#ifdef Q_OS_MAC
      dlg->setOptions(QFileDialog::DontUseNativeDialog);
#endif
      //dlg->setDirectory(item1->text());
      dlg->selectFile(item1->text());
      
      long ret;
      QString filename;
      if((ret=dlg->exec())==QDialog::Accepted){
	filename=dlg->selectedFiles().at(0);
      }
      delete (dlg);
      if(!filename.isNull() && ret==QDialog::Accepted){
	item1->setText(filename);
      }
      adjustTwPatterns();

    }
   
  }
}

/*EOF*/ 
