/*************************************************************************
 * @(#)FSSweepDialog.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   27 Oct 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "FSSweepDialog.hh"
#include "MyMath.hh"
#include "Erg.hh"
#include "Config.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSSweepDialog::FSSweepDialog(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
{
  setupUi(this);
  setUp(0,pbClose,0,0,"FSSweepDialog",": FS sweeper");
  
  cbShowAllSheets->hide();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSSweepDialog::~FSSweepDialog()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::saveData(const Energies &erg)
{
  pcurrentUseDefault=erg.useDefaultEf();
  pcurrentEF=erg.fermiLevel();  
  long ms=0;
  pcurrentbands[ms]=erg.bands(ms);
  if(erg.nspin()>1) {
    ms=1;
    pcurrentbands[ms]=erg.bands(ms);
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::restoreData(Energies &erg)
{
  erg.setFermiLevel(pcurrentEF,pcurrentUseDefault);
  erg.setBands(pcurrentbands);
  erg.setEfChanged(true);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_dspbxCurrent_valueChanged(double value)
{
  emit currentValueChanged(value);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_dspbxMin_valueChanged(double value)
{
  if(value>=dspbxMax->value()-1.0e-6){
    dspbxMin->
      setStyleSheet(QString::fromUtf8("background-color: rgb(255, 0, 0);"));
   
  }else{
    dspbxMin->setStyleSheet("");
  }
  dspbxMax->setStyleSheet("");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_dspbxMax_valueChanged(double value)
{
  if(value<=dspbxMin->value()+1.0e-6){
    dspbxMax->
      setStyleSheet(QString::fromUtf8("background-color: rgb(255, 0, 0);"));
   
  }else{
    dspbxMax->setStyleSheet("");
  }
  dspbxMin->setStyleSheet("");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_tbBack_clicked()
{
  dspbxCurrent->setValue(dspbxMin->value());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_tbForward_clicked()
{
  dspbxCurrent->setValue(dspbxMax->value());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_tbStep_clicked()
{
  double c=dspbxCurrent->value();
  double min=dspbxMin->value();
  double max=dspbxMax->value();
  double step=dspbxStep->value();
  if(c>=max){
    c=min;
  }else{
    c=MyMath::min<double>(max,c+step);
  }
  
  dspbxCurrent->setValue(c);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSSweepDialog::on_tbStart_clicked()
{
  
}

/*EOF*/ 
