/*************************************************************************
 * @(#)FSToolBox.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 Jul 2009
 *************************************************************************
 */

#include <QKeyEvent>
#include <QSignalMapper>
#include <QMenu>
#include "Symmetry.hh"
#include "FSToolBox.hh"
#include "FSDoc.hh"
#include "Spsp.hh"
#include "SpecialPointDlg.hh"
#include "Status.hh"
#include "FSW.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSToolBox::FSToolBox(FSW *parent,FSDoc *d)
  :ToolBox(parent,d),fsw(parent),doc(d)
{
  setupUi(this);


  prbPersp=rbPersp;
  prbOrtho=rbOrtho;
  ptbResetCamera=tbResetCamera;
  ptbCamSaveLoad=tbCamSaveLoad;
  ptbCart=tbCart;
  ptbConv=tbConv;
  ptbPrim=tbPrim;
  ptbXYView=tbXYView;
  ptbYXView=tbYXView;
  ptbXZView=tbXZView;
  ptbZXView=tbZXView;
  ptbYZView=tbYZView;
  ptbZYView=tbZYView;
  ptbXYZView=tbXYZView;
  ptbWireFrame=tbWireFrame;
  ptbBoundOnRotate=tbBoundOnRotate;
  ptbAA=tbAA;
  ptbBoundCell=tbBoundCell;
  ptbPrimCell=tbPrimCell;
  ptbConvCell=tbConvCell;
  pdspbxEye=dspbxEye;

  ToolBox::init();



}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSToolBox::~FSToolBox()
{
  //  myDebug() << "~FSToolBox";
  //  myDebug() << "tbResetCamera" <<  tbResetCamera;
}


















/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSToolBox::on_tbShowLegend_toggled(bool onoff)
{
//  myDebug("on_tbShowLegend_toggled");

  if(muted) return;
  if(!fsw) return;
  if(!doc) return;

  doc->pstate.pvelolegend=onoff;

  doc->setModified();
 
  fsw->updateData();
  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSToolBox::on_tbShowLegendUnits_toggled(bool onoff)
{
//  myDebug("on_tbShowLegendUnits_toggled");

  if(muted) return;
  if(!fsw) return;
  if(!doc) return;

  doc->pstate.pvelolegunit=onoff;

  doc->setModified();
  
  fsw->updateData();
  

}






/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSToolBox::on_tbSpecPoi_clicked()
{
//  myDebug("on_tbSpecPoi_clicked");

  if(!fsw) return;
  if(!doc) return;

  bool onoff=tbSpecPoi->isChecked();

  doc->specpoi->setVisible(onoff);
  fsw->oglView()->setVisible("special point",onoff);
  fsw->oglView()->setVisible("special point path",onoff && 
			     doc->specpoi->showPath());

  doc->setModified();

  if(fsw->specialpointdlg)
    fsw->specialpointdlg->refreshActive(onoff);

  if(!doc->allcalced) 
    fsw->updateData();
  else{
    fsw->oglView()->doRendering();
  }

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSToolBox::on_tbShowHideFermiSurface_toggled(bool onoff)
{
//  myDebug("on_tbShowHideFermiSurface_toggled");

  if(muted) return;
  if(!fsw) return;
  if(!doc) return;

  doc->pstate.pfermi=onoff;
  doc->setModified();

  
  fsw->updateData();
  
  
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSToolBox::updateToolBox()
{

  if(!fsw) return;
  if(!doc) return;

  ToolBox::updateToolBox();

  
  
  tbShowLegend->setChecked(doc->pstate.pvelolegend);
  tbShowLegendUnits->setChecked(doc->pstate.pvelolegunit);






  tbShowHideFermiSurface->setChecked(doc->pstate.pfermi);


  tbSpecPoi->setChecked(doc->specpoi->isVisible());
  

  setMuted(false);


  
  

}





/*EOF*/ 
