/*************************************************************************
 * @(#)FermiDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   22 May 2009
 *************************************************************************
 */

#include <QMessageBox>
#include "FermiDlg.hh"
#include "Valid.hh"
#include "Erg.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

FermiDlg::FermiDlg(QWidget *parent ,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  //  NSF("FermiDlg::FermiDlg");
  //  CPR("");

  setupUi(this);
  setWindowTitle(IDS_APP_NAME + ": Energy settings");

  //  setWindowModality ( Qt::WindowModal);


  //  new IntValidator(this,leBand,"The band index",1,1000000);

  new RealValidator(this,leFermi,"The Fermi energy",-1.e10,1.e10);

  t_bands[0].resize(0);
  t_bands[1].resize(0);

  cleanGroupBox(gbxUp);
  cleanGroupBox(gbxDown);

  //  leBand->selectAll();
  gbxUp->setFocus();


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::accept()
{
  //  myDebug(" accept");
  
  //int i;

  QString txt="";

  //myDebug(" t_efchn(%ld)\n",t_efchn);

  if(t_efchn){
    on_updatePB_clicked();
    return;
  }
  


  ((IntValidator*)leFermi->validator())->check(txt);

  {

    
    long ind=countCheckBoxes(gbxUp);
    
  
    if(t_erg->nspin()>1) 
       ind+=countCheckBoxes(gbxDown);
    
    if(!ind){

      bool nobands=(t_erg->numBands()==0);
      if(!countCheckBoxes(gbxUp) && !countCheckBoxes(gbxDown)&&nobands
	 ){
	txt+="\nThere are no bands crossing the Fermi level!\n"
	  "Do you have a gap?\n"
	  "It migh be due to numerics.\n"
	  "If you want a different Fermi level to play around\n"
	  "and there are no bands, consider the k-export options\n"
	  "in the options menu (uncheck the \"partially occupied\" flag)\n"
	  "and re-export the k-mesh";
      }else{
	txt+="\nOne band must be selected!";
      }
    }
  }  


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::on_cbDefEf_clicked()
{
  long ch=cbDefEf->isChecked();
  
  leFermi->setEnabled((!ch) | (!cbDefEf->isEnabled()));

  t_efchn=true; 
  //
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void FermiDlg::makeButtonList(QGroupBox *gbx
			      ,long nmin,long nmax
			      ,const QVector<long> &bands)
{

  

  QCheckBox* cbox;
  /* todo: set at least one */
  for(long i=nmin;i<=nmax;i++){
    QString s; 
    s.sprintf("CBOX%ld",i) ;
    cbox=new QCheckBox(  s ,gbx );
    cbox->setObjectName(s);
   //if(i==nmin){
    //  gbx->setBuddy(cbox);
    //}
    s.sprintf("band &%ld",i+1) ;
    cbox->setText( s );
    cbox->show();
    for(long j=0;j<bands.size();j++){
      if(i==bands.at(j)){
	cbox->setChecked(1);
      }
    }
    gbx->layout()->addWidget( cbox ); 
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::setData(Energies *erg,const Vlevel &vlevel)
{

  t_vlevel=vlevel;
  t_erg=erg;
  
  long ms=0;
  t_bands[ms]=erg->bands(ms);

  makeButtonList(gbxUp
		 ,erg->minBandAtEF(ms)
		 ,erg->maxBandAtEF(ms),t_bands[ms]);
  if(t_erg->nspin()>1) {
    ms=1;
    t_bands[ms]=erg->bands(ms);
    makeButtonList(gbxDown
		   ,erg->minBandAtEF(ms)
		   ,erg->maxBandAtEF(ms),t_bands[ms]);
  }  
  t_emptyl=erg->emptyLattice();
  t_ef=QString().setNum(erg->fermiLevel(),'g',15);
  leFermi->setText(t_ef);
  
  t_defef=erg->useDefaultEf();
  cbDefEf->setChecked(t_defef);
  if(t_emptyl){
    cbDefEf->setEnabled(false);
  }
  on_cbDefEf_clicked();
  t_efchn=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::readButtonList(QGroupBox *gbx,QVector<long> &bands)
{
  bool ok;
  bands.resize(0);
  
  QObjectList l=gbx->children();
  

  for (int i = 0; i < l.size(); ++i) {
    // for each found object...
    //((QButton*)obj)->setEnabled( FALSE );
    if(QString("QCheckBox") == l[i]->metaObject()->className() ){
      
      const QCheckBox *cbox=((QCheckBox*)l[i]);
      QString s=QString(cbox->objectName());
      if( s.startsWith("CBOX")) {
	if( cbox->isChecked() ){
	  long nb=s.mid(4,10000).toInt(&ok,10);
	  bands.append(nb);
	}
      }
    }
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::on_updatePB_clicked()
{

  bool ok;
  QString t=leFermi->text();
  IntValidator::stripAllWhiteSpace(t);
  double ef=t.toDouble(&ok);
  long defef=cbDefEf->isChecked();

  double ef1=t_erg->fermiLevel();
  bool defef1=t_erg->useDefaultEf();
  
  t_erg->setFermiLevel(ef,defef);
  t_erg->calcRange(t_vlevel);
  t_erg->setFermiLevel(ef1,defef1);

  QVector<long> tmp;


  long ms=0;
  
  readButtonList(gbxUp,tmp);

  cleanGroupBox(gbxUp);
  

  makeButtonList(gbxUp
		 ,t_erg->minBandAtEF(ms)
		 ,t_erg->maxBandAtEF(ms),tmp);
  gbxUp->updateGeometry();

  if(t_erg->nspin()>1) {
    ms=1;
    readButtonList(gbxDown,tmp);
    
    
    cleanGroupBox(gbxDown);
    
    makeButtonList(gbxDown
		   ,t_erg->minBandAtEF(ms)
		   ,t_erg->maxBandAtEF(ms),tmp);
    gbxDown->updateGeometry();
  }

  updateGeometry();

  
  QRect r=geometry();
  r.setHeight(minimumHeight());
  resize(r.width(),r.height());
  
  



  t_efchn=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::cleanGroupBox(QGroupBox *gbx)
{
  QObjectList l=gbx->children();
  for(int i=0;i<l.size();i++){
    //myDebug("%s:%s\n",l[i]->metaObject()->className(),l[i]->name());
    if(QString("QCheckBox") == l[i]->metaObject()->className() )
      delete l[i];
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FermiDlg::getData( Energies *erg )
{ 
  bool ok;
  QString t=leFermi->text();
  bool chn=erg->efChanged();
  chn |= ( t != t_ef ) ;
  IntValidator::stripAllWhiteSpace(t);

  bool defef=cbDefEf->isChecked();
  chn |= ( defef != t_defef);

  erg->setFermiLevel(t.toDouble(&ok),defef);
  //  t=leBand->text();
  //  IntValidator::stripAllWhiteSpace(t);
  //  iband=t.toLong(&ok);

  
  long ms=0;
  QVector<long> b[2];
  readButtonList(gbxUp,b[ms]);
   
  if (t_bands[ms].size() != b[ms].size()) {
    chn |= true;
  }else{
    for(long i=0;i<b[ms].size();i++){
      chn |= (b[ms].at(i)!= t_bands[ms].at(i));
    }
  }

  if(t_erg->nspin()>1) {
    ms=1;
    readButtonList(gbxDown,b[ms]);
    
    if (t_bands[ms].size() != b[ms].size()) {
      chn |= true;
    }else{
      for(long i=0;i<b[ms].size();i++){
	chn |= (b[ms].at(i) != t_bands[ms].at(i));
      }
    }
  }  

  erg->setBands(b);
 
  erg->setEfChanged(chn);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FermiDlg::countCheckBoxes(QGroupBox *gbx)
{
    QObjectList l = gbx->children( );
    long ind=0;
    
    for (int i = 0; i < l.size(); ++i) {
      // for each found object...
      //myDebug("%s:%s\n",l[i]->metaObject()->className(),l[i]->name());
      if(QString("QCheckBox") == l[i]->metaObject()->className() ){
	
	const QCheckBox *cbox=((QCheckBox*)l[i]);
	QString s=QString(cbox->objectName());
	if( s.startsWith("CBOX")) {
	  //myDebug("%d\n",cbox->isChecked());
 	  if( cbox->isChecked() ){
	    ind++;
	  }
	}
      }
    }
    

    return ind;
}

/*EOF*/ 
