/*************************************************************************
 * @(#)FilesDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Jun 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QFileDialog>
#include "FilesDlg.hh"
#include "Erg.hh"
#include "Config.hh"
#include "Valid.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FilesDlg::FilesDlg(QWidget *parent ,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{

  setupUi(this);
  setWindowTitle(IDS_APP_NAME + ": File settings");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FilesDlg::accept()
{
  //  myDebug(" accept");
  
  //int i;

  QString txt="";

  //myDebug(" t_efchn(%ld)\n",t_efchn);



  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FilesDlg::setData(Energies *erg,const QString &workdir)
{
  t_workdir=workdir;
  t_bandfile=erg->bandFile();
  leBandFile->setText(erg->bandFile());
  t_bandweightsfile=erg->bandWeightsFile();
  leBandWeightsFile->setText(erg->bandWeightsFile());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FilesDlg::getData( Energies *erg )
{ 
  QString t;
  bool chn=false;

  t=leBandFile->text();
  if(( t != t_bandfile)){
    chn |= true;
    IntValidator::stripAllWhiteSpace(t);
    erg->setBandFile(t);
    erg->setEnergiesLoaded(false);
  }


  t=leBandWeightsFile->text();
  if(( t != t_bandweightsfile)){
    chn |= true;
    IntValidator::stripAllWhiteSpace(t);
    erg->setBandWeightsFile(t);
    erg->setEnergiesLoaded(false);
  }

  erg->setEfChanged(chn);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FilesDlg::on_pbBrowseBandFile_clicked()
{
  QString f0=leBandFile->text();
  QString fn="";
#ifdef Q_OS_MAC
  fn = QFileDialog::getOpenFileName(this,"Band File",f0);
#else
  fn = QFileDialog::getOpenFileName(this,"Band File",f0
				    , " FPLO band files(+*band*) ;; All files(*)"
				    ,0,0);
#endif
  if ( !fn.isEmpty() ) {
    QFileInfo fi(fn);
    myDebug()<<t_workdir<<fi.absolutePath()+"/";
    if(t_workdir!=fi.absolutePath()+"/"){
      QMessageBox::critical(this,"Band file"
			    ,QString("The file %1 \n"
				     "is not int the current "
				     "working directory!").arg(fn)
			    ,QMessageBox::Ok);  
      return;

    }
    leBandFile->setText(fi.fileName());

  }


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FilesDlg::on_pbBrowseBandWeightsFile_clicked()
{
  QString f0=leBandWeightsFile->text();
  QString fn="";
#ifdef Q_OS_MAC
  fn = QFileDialog::getOpenFileName(this,"Band weight File",f0);
#else
  fn = QFileDialog::getOpenFileName(this,"Band weight File",f0
				    , " FPLO band weight files(+*bw*) ;; All files(*)"
				    ,0,0);
#endif
  if ( !fn.isEmpty() ) {
    QFileInfo fi(fn);
    myDebug()<<t_workdir<<fi.absolutePath()+"/";
    if(t_workdir!=fi.absolutePath()+"/"){
      QMessageBox::critical(this,"Band file"
			    ,QString("The file %1 \n"
				     "is not int the current "
				     "working directory!").arg(fn)
			    ,QMessageBox::Ok);  
      return;

    }
    leBandWeightsFile->setText(fi.fileName());

  }


}


/*EOF*/ 
