/*************************************************************************
 * @(#)FindAtomDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 Feb 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "AtomFindMarker.hh"
#include "FindAtomDlg.hh"
#include "Settings.hh"
#include "Config.hh"
#include "HelpDlg.hh"
#include "Atom.hh"
#include "Valid.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FindAtomDlg::FindAtomDlg(QWidget *parent,QWidget *base,const char *name
			 ,AtomFindMarker *atomFindMarker)
  : ProtoDlg(parent,base,name),rbbg(0),patomfindmarker(atomFindMarker),
    mute_apply(false)
{
  setupUi(this);
  setUp(0,pbClose,0,pbHelp,"FindAtomDialog",": Find atom");


  //new IntValidator(this,leSite,"Site/Sort/Cluster-sort number",1,100000);
  

  rbbg=new QButtonGroup(this);
  rbbg->setExclusive(true);
  rbbg->addButton(rbSite,Xf::FindSite);
  rbbg->addButton(rbWyckoff,Xf::FindSort);
  rbbg->addButton(rbType,Xf::FindType);
  rbbg->addButton(rbCluster,Xf::FindClusterNumber);


  connect(rbSite,SIGNAL(toggled(bool)),this,SLOT(onRbToggled(bool)));
  connect(rbWyckoff,SIGNAL(toggled(bool)),this,SLOT(onRbToggled(bool)));
  connect(rbType,SIGNAL(toggled(bool)),this,SLOT(onRbToggled(bool)));
  connect(rbCluster,SIGNAL(toggled(bool)),this,SLOT(onRbToggled(bool)));


  setMessage("");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FindAtomDlg::~FindAtomDlg()
{
  if(rbbg) delete rbbg;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::onRbToggled(bool on)
{
  // myDebug() << sender() << on;
  // myDebug()<<"FindAtomDlg::onRbToggled";

  if(on){
    ((QRadioButton *) sender())->
      setStyleSheet(QString::fromUtf8("QRadioButton:enabled { color: blue }"));
    apply();
  }else{
    ((QRadioButton *) sender())->
      setStyleSheet(QString::fromUtf8("QRadioButton:enabled { color: black }"));
  }


  // myDebug()<<"FindAtomDlg::onRbToggled after apply";


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::setMessage(const QString &msg)
{
  if(msg.length()>0){
    lbMsg->setText(msg);
  }else{
    lbMsg->setText("");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::setData()
{
  //myDebug() << "FindAtomDlg::setData";
  mute_apply=true;
  Settings set("structure");
  Xf::FindAtomModes mode=
    (Xf::FindAtomModes)set.getInt("FindAtom/mode",Xf::FindSort);
  long number=set.getInt("FindAtom/number",0);

  wAMP->setData(&(patomfindmarker->properties()));

  QRadioButton *rb=(QRadioButton*)rbbg->button(mode);
  if(rb){
    rb->setChecked(true);
  }

  spbxSite->setValue(number);
  mute_apply=false;
  apply();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::updateData(const XFPLO::Atom &at)
{
  //myDebug() << "FindAtomDlg::updateData";
  QRadioButton *rb=(QRadioButton*)rbbg->checkedButton();
  if(!rb) return;

  Xf::FindAtomModes mode=(Xf::FindAtomModes)rbbg->id(rb);

  int number;
  if(mode==Xf::FindSite){
    number=at.site;
  }else if(mode==Xf::FindSort){ 
    number=at.sort;
  }else if(mode==Xf::FindType){ 
    number=at.type;
 }else if(mode==Xf::FindClusterNumber){
    number=at.index;
  }else{
    return;
  }
  if(mode!=Xf::FindType) number++;

  spbxSite->setValue(number);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::getData(Xf::FindAtomModes &mode,long &number)
{
  //myDebug() << "FindAtomDlg::getData";
  number=spbxSite->value();

  QRadioButton *rb=(QRadioButton*)rbbg->checkedButton();
  if(rb){
    mode=(Xf::FindAtomModes)rbbg->id(rb);
  }else{
    mode=Xf::FindSite;
  }

  Settings set("structure");
  set.set("FindAtom/mode",mode);
  set.set("FindAtom/number",(int)number);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::apply()
{
  if(mute_apply) return;
  
  // myDebug() <<"FindAtomDlg::apply";
  
  //spbxSite->selectAll();
  
  onpbAcceptClicked();
  spbxSite->setFocus();
  // myDebug() <<"FindAtomDlg::apply after onpbAcceptClicked";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FindAtomDlg::on_spbxSite_valueChanged(int)
{
  //myDebug()<<"FindAtomDlg::on_spbxSite_valueChanged";
  apply();
}


/*EOF*/ 
