/*************************************************************************
 * @(#)FogDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Mar 2011
 *************************************************************************
 */

#include "Fog.hh"
#include "FogDlg.hh"
#include "HelpDlg.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FogDlg::FogDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"FogDialog",": Fog settings");

  t_fog=new Fog();

  cmbFogMode->insertItem(0,Fog::fogModeName(Fog::Linear),Fog::Linear);
  cmbFogMode->insertItem(1,Fog::fogModeName(Fog::Exp),Fog::Exp);
  cmbFogMode->insertItem(2,Fog::fogModeName(Fog::Gaussian),Fog::Gaussian);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FogDlg::~FogDlg()
{
  if(t_fog) delete t_fog;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FogDlg::setData(const Fog *fog)
{
  *t_fog=*fog;
  cbUseFog->setChecked(fog->useFog());
  dspbxDensity->setValue(fog->density());
  dspbxNear->setValue(fog->near());
  dspbxFar->setValue(fog->far());
  tbFogColor->setColor(fog->color());
  int i=cmbFogMode->findData(fog->fogMode());
  if(i<0){
    printf("unknown fogmode in FogDlg::setData");
    exit(1);
  }
  cmbFogMode->setCurrentIndex(i);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FogDlg::getData(Fog *fog)
{


  t_fog->setUseFog(cbUseFog->isChecked());
  t_fog->setDensity(dspbxDensity->value());
  t_fog->setNear(dspbxNear->value());
  t_fog->setFar(dspbxFar->value());
  t_fog->setColor(tbFogColor->color());

  int i=cmbFogMode->currentIndex();
  if(i!=-1){
    t_fog->setFogMode((Fog::FogMode) cmbFogMode->itemData(i).toInt());
  }

  *fog=*t_fog;


  t_fog->setChanged(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FogDlg::on_cmbFogMode_currentIndexChanged(int index)
{
  if(index<0) return;
  bool lin=false;
  bool den=true;
  //myDebug() <<index<<cmbFogMode->itemData(index).toInt();
  if(cmbFogMode->itemData(index).toInt()==Fog::Linear){
    lin=true;
    den=false;
  }
  dspbxNear->setEnabled(lin);
  dspbxFar->setEnabled(lin);
  lbNear->setEnabled(lin);
  lbFar->setEnabled(lin);
  lbDensity->setEnabled(den);
  dspbxDensity->setEnabled(den);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FogDlg::on_pbSetDefault_clicked()
{
  Fog f;
  f.setDefault();
  f.setChanged(true);
  setData(&f);
}

/*EOF*/ 
