/*************************************************************************
 * @(#)FploDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 Jul 2009
 *************************************************************************
 */

#include <stdlib.h> 
#include <QDir> 
#include <QMessageBox> 
#include <QTextStream> 
#include <QTimer> 
#include <QTimer> 
#include "Parser.hh"
#include "ExecPathDlg.hh"
#include "Assorted.hh"
#include "Except.hh"
#include "FploDlg.hh"
#include "Fplo.hh"


QMap<QString,QString> FploDlg::ppreviousprograms;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FploDlg::FploDlg(QWidget *parent,Fplo *fplo)
  :QDialog(parent),proc(0),ptimer(0)
  ,outFile(0),tsout(0),refreshtime(100),pfplo(fplo),pdidrun(false)
  ,pkilleractive(false)
{
  setupUi(this);

  leOutFile->setText(pfplo->outFile());

  proc=new QProcess(this);
  connect(proc,SIGNAL(started()),this,SLOT(onProcStarted()));
  connect(proc,SIGNAL(error(QProcess::ProcessError))
	  ,this,SLOT(onProcError(QProcess::ProcessError)));
  connect(proc,SIGNAL(finished(int,QProcess::ExitStatus))
	  ,this,SLOT(onProcFinished(int,QProcess::ExitStatus)));

  connect( proc, SIGNAL(readyReadStandardOutput()),
	   this, SLOT(onProcReadFromStdout()) );

  pbKill->setEnabled(false);
  leWorkDir->setText(pfplo->currentWorkDir());

  ptimer=new QTimer(this);
  connect(ptimer, SIGNAL(timeout()), this, SLOT(ptimerTimout()));


  leVersion->setText(fplo->mainVersion());
  leLastChangedBy->setText(pfplo->lastChanged());
  
  
  populateCmbFploExec();
  

  if(pfplo->message().isEmpty()){
    lMessage->hide();
  }else{
    lMessage->setText(pfplo->message());
    lMessage->show();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
FploDlg::~FploDlg()
{
  if(proc) {
    proc->kill();
    delete proc; proc=0;
  }
  if(tsout) { delete tsout; tsout=0;}

  if(outFile) {
    outFile->close();
    delete outFile;
    outFile=0;
  }
  if(ptimer) {
    ptimer->stop();
    delete ptimer;
    ptimer=0;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::populateCmbFploExec()
{
  cmbFploExec->clear();
  int i=0;
  foreach(QString s,pfplo->programs()){
    cmbFploExec->insertItem(i++,s);
  }
  QMap<QString,QString>::const_iterator it
    =ppreviousprograms.find(leWorkDir->text());
  if(it!=ppreviousprograms.end()){
    //myDebug()<<"have one"<<it.value();
    cmbFploExec->setCurrentIndex(cmbFploExec->findText(it.value()));
    
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::on_pbKill_clicked()
{
  //myDebug()<<"on_pbKill_clicked";
  if(proc){
    pkilleractive=true;
    proc->kill();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::on_pbRun_clicked()
{

  if(!pfplo->prepareConditions()) return;

  pbRun->setEnabled(false);
//   myDebug()<< "FploDlg::on_pbRun_clicked" ;
//   myDebug()<< pprogram ;
//   myDebug()<< pworkdir ;

  teOutput->clear();
  ptxtStdout="";

  QString outfile=leOutFile->text().trimmed();

  cbShowOutput->setEnabled(false);
  leOutFile->setEnabled(false);

  proc->setWorkingDirectory(pfplo->currentWorkDir());
  proc->setProcessChannelMode(QProcess::MergedChannels);

  if((!outfile.isEmpty()) ){
    outfile=pfplo->currentWorkDir()+"/"+outfile;
    proc->setStandardOutputFile(outfile,QIODevice::WriteOnly |
				QIODevice::Truncate);
    if( cbShowOutput->isChecked()){
      ptimer->start(10);
      outFile= new QFile(outfile);
    }

  }
  
  pprogram=cmbFploExec->currentText();

  ppreviousprograms.insert(leWorkDir->text(),pprogram);

  proc->start(pprogram, QStringList(""),QIODevice::ReadWrite);  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::onProcStarted()
{
  //  myDebug() << "started";
  teOutput->append("program: "+pprogram+"\n in \n"+pfplo->currentWorkDir()+"\n started"
		   +"\n-----------------------------------------------\n");
  pbKill->setEnabled(true);
  cbShowOutput->setEnabled(false);
  leOutFile->setEnabled(false);
  pbRun->setEnabled(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::onProcError(QProcess::ProcessError error)
{
  //  myDebug() << "Error in process" << error;
  readFromTsOut();
  if(tsout) delete tsout; tsout=0;
  if(outFile){
    outFile->close();
    delete outFile; outFile=0;
  }
  ptimer->stop();

  if(error==QProcess::FailedToStart){
    QMessageBox::warning(this,"Running","Could not run the process? \n "
			 "Is the path correct?"
			 ,QMessageBox::Ok| QMessageBox::Default);
  }
  pbKill->setEnabled(false);
  pbRun->setEnabled(true);
  leOutFile->setEnabled(true);
  cbShowOutput->setEnabled(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::onProcFinished(int exitCode,QProcess::ExitStatus exitStatus)
{
  Q_UNUSED(exitCode); Q_UNUSED(exitStatus);

  //  myDebug() << "finished" << exitCode;
  //  myDebug() << "finished" << exitStatus;

  readFromTsOut();
  if(tsout) delete tsout; tsout=0;
  if(outFile){
    outFile->close();
    delete outFile; outFile=0;
  }
  ptimer->stop();

  teOutput->append(QString("\n-----------------------------------------------\n")
		   +"program: "+pprogram+"\n in \n"+pfplo->currentWorkDir()+"\n finished\n"
		   +QString(" exit code=%1").arg(exitCode)
		   +QString("\n exit status=%1").arg(exitStatus)
		   );
  

  pbKill->setEnabled(false);
  pbRun->setEnabled(true);
  leOutFile->setEnabled(true);
  cbShowOutput->setEnabled(true);
  pdidrun=true;

  if(exitCode==0){
    if(!pfplo->terminationCode().isEmpty()){
      QStringList sl=teOutput->toPlainText().split("\n");
      bool foundit=false;
      foreach(const QString &s,sl){
	if(s.startsWith("TERMINATION:")){
	  if(s.contains(pfplo->terminationCode())){
	    foundit=true;
	    break;
	  }
	}
      }
      //myDebug()<<"foundit"<<foundit;
      //myDebug()<<"pkilleractive"<<pkilleractive;
      if(!foundit && !pkilleractive) on_pbRun_clicked();

    }
  }else{
    QMessageBox::information(this,"Running","Process finished!\n"
			     " check last output!"
			     ,QMessageBox::Ok| QMessageBox::Default);
  }

  pkilleractive=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::onProcReadFromStdout()
{
  //  myDebug() << "onProcReadFromStdout";
  //  myDebug() << cbShowOutput->isChecked();
  
  if( !cbShowOutput->isChecked()) return;

  while(1){
    char buf[1024];
    qint64 lineLength = proc->readLine(buf, sizeof(buf));
    if (lineLength ==0) {
      // done reading for now
      return;
    }
    if (lineLength == -1) {
      // error occured
      return ;
    }
    QString str(buf);
    
    long i=str.indexOf('\n');

    if(i==-1){
      ptxtStdout+=str;
    }else{
      ptxtStdout+=str.left(i);
      teOutput->append(ptxtStdout);
      
      str.remove(0,i+1);
      ptxtStdout="";
    }


  }
 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::accept()
{
  if (proc){
    if(proc->state()!=QProcess::NotRunning){
      long ret=QMessageBox::warning(this,"Running","Process is running, really abort?"
			   ,QMessageBox::Ok| QMessageBox::Default
			   , QMessageBox::Cancel| QMessageBox::Escape);
     
      
      if(ret == QMessageBox::Ok) {
	QDialog::accept();
      }
      return;
      
    }
  }
  QDialog::accept();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::ptimerTimout()
{
  //  myDebug() << "timeout";
  ptimer->stop();
  ptimer->start(refreshtime);
  if(outFile){
    if(!outFile->isOpen()){
      //      myDebug("%x\n",int(outFile->permissions()));
      if(!outFile->open(QIODevice::ReadOnly | QIODevice::Text)) return;
      //ptimer->stop();
      tsout=new QTextStream(outFile);
      
    }else{

      readFromTsOut();

    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::readFromTsOut()
{
  if(!tsout) return;
  while(1){
    QString str = tsout->readLine();
    if(str.isNull()){
      // done reading for now
      return;
    }
    teOutput->append(str);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FploDlg::on_pbManageExecs_clicked()
{
  ExecPathDlg *dlg=new ExecPathDlg(this,pfplo);
  
  int ret=dlg->exec();
  if(ret==QDialog::Accepted){
    populateCmbFploExec();
  }else{
  }
}

/*EOF*/ 
