/*************************************************************************
 * @(#)HelpBrowser.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 Mar 2011
 *************************************************************************
 */


#include <QApplication>
#include <QHelpEngineCore>
#include "HelpBrowser.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
HelpBrowser::HelpBrowser(QWidget *parent)
  : QTextBrowser(parent)
{
  
  QString collectionFile=IDS_APP_HELP_FULL_FILE;
  m_helpEngine = new QHelpEngineCore(collectionFile, this);
  if (!m_helpEngine->setupData()) {
    delete m_helpEngine;
    m_helpEngine = 0;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
HelpBrowser::~HelpBrowser()
{
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void HelpBrowser::showHelpForKeyword(const QString &id)
{
  if (m_helpEngine) {
    QMap<QString, QUrl> links = m_helpEngine->linksForIdentifier(id);
    if (links.count())
      setSource(links.constBegin().value());
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant HelpBrowser::loadResource(int type, const QUrl &name)
{
  QByteArray ba;
  if (type < 4 && m_helpEngine) {
    QUrl url(name);
    if (name.isRelative())            
      url = source().resolved(url);
    ba = m_helpEngine->fileData(url);        
  }
  return ba;
}




/*EOF*/ 
