/* @(#)HsvSampler.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _HSVSAMPLER_H
#define _HSVSAMPLER_H 1


#include <QWidget>
#include "MyMath.hh"

class ColorRampWidget;


/* ***********************************************************************
 *
 * ***********************************************************************
 */

class HsvSampler:public QWidget
{
  Q_OBJECT;
private:

  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////

  class Node{
  public:
    Node(double s,int h,int sa,int v):ps(s),psat(sa),pval(v),
				      highlight(false){
      phue=h%360;phueinterval=h/360;
      if(phue<0){ phue+=360;phueinterval--;}
    };
    
    double xx(HsvSampler *parent,bool raw=false)const;
    double yy(HsvSampler *parent,bool raw=false)const;
    void setPoint(HsvSampler *parent,QPoint pt,QPoint oldpt,
		  bool isnew);
    void paint(HsvSampler *parent,QPainter &,bool raw)const;

    inline double parameter()const{return ps;};
    inline void setParameter(double s){ps=s;};

    inline QColor color()const{
      int h=phue%360;if(h<0){ h+=360;}
      return QColor::fromHsv(h,psat,pval);
    };


    inline int rawHue()const{return phue+phueinterval*360;};
    inline int saturation()const{return psat;};
    inline int value()const{return pval;};

    bool underCursor(HsvSampler *parent,QPoint pt)const;

    QRectF boundingRect(HsvSampler *parent,const QPointF &pt)const;

    QPointF toDevice(HsvSampler *parent,bool raw=false)const;

    //static bool parameterLessThan(const Node *s1,const Node *s2){
    //  return s1->ps<s2->ps;
    //};

    static Node interpolateNodes(const Node &n1,const Node &n2,double s,
				 bool inspherical=false);
    
  private:
    double ps;
    int phue,psat,pval;
    int phueinterval;
  public:
    bool highlight;
  };

  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////
  class NodeList{
  public:
    ~NodeList(){
      clear();
    }
    inline void clear(){qDeleteAll(list); list.clear();};
    inline const Node *at(int i)const{return list.at(i);};
    inline int size()const{return list.size();};
    inline Node *operator[](int i){return list[i];};
    inline void append(Node *n){list.append(n);};
    inline void prepend(Node *n){list.prepend(n);};
    inline int indexOf(Node *n){return list.indexOf(n);};
    inline Node *last(){return list.last();};
    inline void insert(int before,Node *n){list.insert(before,n);};
    inline void removeOne(Node *n){list.removeOne(n);delete n;};
    void recalculateIminImax();

  private:
    QList<Node*> list;
    int imin,imax;
    //friend class Node;
    friend class HsvSampler;
    friend class Node;
  };


public:

  enum Modes{None=0x0,Hue=0x1,Saturation=0x2,Value=0x4,New=0x8,Parameter=0x10};


  HsvSampler(QWidget *parent,int xmode,int ymode,bool circular=false);
  virtual ~HsvSampler();


  void setNodeList(NodeList *nl);


signals:
  void nodeChanged();
  void registerUndo();

protected:
  virtual void paintEvent(QPaintEvent*);
  virtual void mouseMoveEvent(QMouseEvent *event);
  virtual void mousePressEvent(QMouseEvent *event);
  virtual void mouseReleaseEvent(QMouseEvent *event);
  virtual void leaveEvent(QEvent *event);
private:
  void snapNodes();


private:
  int pointradius,Rmin;
  QPoint delta,lastdpos;
  int pmode[2];
  QImage *background;
  bool needbackgroundupdate;
  NodeList *nodelist;
  Node *currentnode;
  bool pcircular;
  bool pinterpolinspherical;
  //  friend class Node;
  friend class ColorRampWidget;
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _HSVSAMPLER_H */

