/*************************************************************************
 * @(#)InpConDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox> 
#include "InpConDlg.hh"
#include "Config.hh"
#include "Valid.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */

InpConDlg::InpConDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME + ": Input Control Settings");


  //  setWindowModality ( Qt::WindowModal);
  

  new IntValidator(this,leBandmin,"The minimum band index",1,1000000);
  new IntValidator(this,leBandmax,"The maximum band index",1,1000000);
  

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void InpConDlg::setData( long restrict,long bandmin,long bandmax )
{
  t_restrict=restrict;
  cbRestrict->setChecked(restrict);
  
  t_bandmin=bandmin+1;
  leBandmin ->setText(QString().setNum(bandmin+1));
  
  t_bandmax=bandmax+1;
  leBandmax ->setText(QString().setNum(bandmax+1));
  
  if(t_restrict){
    leBandmin->setEnabled(true);
    leBandmax->setEnabled(true);
  }else{
    leBandmin->setEnabled(false);
    leBandmax->setEnabled(false);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void InpConDlg::getData(long &restrict,long &bandmin,long &bandmax,long &chn)
{
  bool ok;
  QString t=leBandmin->text();
  IntValidator::stripAllWhiteSpace(t);
  bandmin=t.toLong(&ok);
  chn = ( bandmin != t_bandmin ) ;
  bandmin--;

  t=leBandmax->text();
  IntValidator::stripAllWhiteSpace(t);
  bandmax=t.toLong(&ok);
  chn |= ( bandmax != t_bandmax ) ;
  bandmax--;


  restrict=cbRestrict->isChecked();
  chn |= ( restrict != t_restrict);
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */


void InpConDlg::accept()
{
  
  //int i;
  
  QString s,txt="";

  ((IntValidator*)leBandmin->validator())->check(txt);
  ((IntValidator*)leBandmax->validator())->check(txt);
  
  
  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }
  
}

/*EOF*/ 
