/*************************************************************************
 * @(#)KexpDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */


#include <QMessageBox> 
#include "KexpDlg.hh"
#include "Valid.hh"
#include "Config.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
KexpDlg::KexpDlg(QWidget *parent ,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + ": options");

  //  setWindowModality ( Qt::WindowModal);

  resize(1,1);
  
  
  new IntValidator(this,leLowerOffset,"The lower offset",-1000000,1000000);
  new IntValidator(this,leUpperOffset,"The upper offset",-1000000,1000000);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void KexpDlg::setData( long exptet,long part,const long *offset)
{
  t_exptet=exptet;
  cbTetFile->setChecked(exptet);

  leLowerOffset ->setText(QString().setNum(offset[0]));
  t_lowoff=offset[0];

  leUpperOffset ->setText(QString().setNum(offset[1]));
  t_upoff=offset[1];
  
  t_part=part;
  cbPart->setChecked(part);

  setEnabled();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void KexpDlg::getData( long *exptet,long * part,long *offset,long * chn )
{
  bool ok;
  QString t;
  
  t=leLowerOffset->text();
  IntValidator::stripAllWhiteSpace(t);
  offset[0]=t.toLong(&ok);
  *chn |= ( offset[0] != t_lowoff ) ;

  t=leUpperOffset->text();
  IntValidator::stripAllWhiteSpace(t);
  offset[1]=t.toLong(&ok);
  *chn |= ( offset[1] != t_upoff ) ;

  *part=cbPart->isChecked();
  *chn |= ( *part != t_part);

  *exptet=cbTetFile->isChecked();
  *chn |= ( *exptet != t_exptet);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void KexpDlg::setEnabled()
{
  long ch=cbPart->isChecked();
  
  leLowerOffset->setEnabled( ch );
  leUpperOffset->setEnabled( ch );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void KexpDlg::accept()
{
  //int i;
  QString s,txt="";

  ((IntValidator*)leLowerOffset->validator())->check(txt);
  ((IntValidator*)leUpperOffset->validator())->check(txt);


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }
  
}

/*EOF*/ 
