/*************************************************************************
 * @(#)LightDlg.cpp
 * author: magru@unas
 * date:   12 Jul 2013
 *************************************************************************
 */

#include <QMessageBox> 
#include <QSignalMapper> 
#include <QMenu> 
#include <QDebug> 
#include "LightDlg.hh"
#include "Config.hh"
#include "ThemeDlg.hh"
#include "OGLView.hh"
#include "Light.hh"
#include "LightsModel.hh"
#include "ToolButtons.hh"
#include "DocBase.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
LightDlg::LightDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
  ,t_lights(0),mute(false),mutetalkback(false),pcontinuous(false),
    insertMenu(0),signalMapper(0)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"LightDialog",": Lights");

  t_lights=new Lights();
  
  lightsmodel = new LightsModel(this);

  lvLights->setModel(lightsmodel);

  signalMapper = new QSignalMapper(this);
  connect(signalMapper,SIGNAL(mapped(int)),
	  this,SLOT(onAddSignalMapper(int)));

  QMenu *m=insertMenu=new QMenu(tbAdd);
  QAction *a;
  a=m->addAction("Insert &Directional side_dif_spec");  
  signalMapper->setMapping(a,Lights::Directional+0x10);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));

  a=m->addAction("Insert &Directional front_dif");  
  signalMapper->setMapping(a,Lights::Directional+0x20);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));

  a=m->addAction("Insert &Positional downright");  
  signalMapper->setMapping(a,Lights::Positional+0x10);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));

  a=m->addAction("Insert &Positional center");  
  signalMapper->setMapping(a,Lights::Positional+0x20);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));

  a=m->addAction("Insert &Spot upperleftside");  
  signalMapper->setMapping(a,Lights::Spot+0x10);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));

  tbAdd->setMenu(insertMenu);

  ptoolbuttons=new ToolButtons();
  ptoolbuttons->createToolButtonsActions(lvLights,"light",tbAdd,tbDelete,tbUp,tbDown);
  ptoolbuttons->connectSlot(this,SLOT(onLvLightstoolButtons()));

  typeHelper=new LTCH(cmbType,Lights::lighttypes);
  typeHelper->populate();

  connectChanged(this);
  connect(this,SIGNAL(contentChanged(QObject *)),
	  this,SLOT(onContentChanged(QObject *)));

  connect(lightsmodel,SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&)),
	  this,SLOT(onModeldataChanged(const QModelIndex&,const QModelIndex&)));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
LightDlg::~LightDlg()
{
  if(lightsmodel) delete lightsmodel; lightsmodel=0;
  poglview->setShowLightHelpers(false);
  delete typeHelper;
  if(signalMapper) delete signalMapper;
  if(insertMenu) delete insertMenu; insertMenu=0;
  if(ptoolbuttons) delete ptoolbuttons;ptoolbuttons=0;
  if(t_lights) delete t_lights;
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::setData(DocBase *doc,OGLView *oglview)
{
  poglview=oglview;
  pdoc=doc;
  (*t_lights)=*(doc->lights());
  cbShowLightHelpers->setChecked(oglview->showLightHelpers());


  setLights();
  setLightTypeEnabled();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::setLights()
{
  mutetalkback=true;
  cbBackground->setColor(t_lights->backgroundColor());
  cbGlobalAmbient->setColor(t_lights->globalAmbient());
  dspbxGlobalAmbientFactor->setValue(t_lights->globalAmbientFactor());

  mute=true;
  lightsmodel->reload(t_lights);
  mute=false;

  QModelIndex ci=lightsmodel->index(0,0,QModelIndex());
  lvLights->scrollTo(ci);
  lvLights->setCurrentIndex(ci);
  mutetalkback=false;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::getData(DocBase *doc)
{
  t_lights->setBackgroundColor(cbBackground->color());
  t_lights->setGlobalAmbient(cbGlobalAmbient->color());
  t_lights->setGlobalAmbientFactor(dspbxGlobalAmbientFactor->value());
  
  
  *(doc->lights())=(*t_lights);
  doc->lights()->setChanged(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::onAddSignalMapper(int mode)
{
  //myDebug()<<"tbAdd"<<mode;
  
  
  QModelIndex index=lvLights->currentIndex();
  int row=0;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
    col=index.column();
  }
  
  
  lightsmodel->insertRow(row,new Lights::Light(mode));
  lvLights->scrollTo(lightsmodel->index(row,0,QModelIndex()));
  lvLights->setCurrentIndex(lightsmodel->index(row,col,QModelIndex()));

  if(pcontinuous && !mutetalkback){
    emit talkBack(ProtoDlg::Accept);
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::onLvLightstoolButtons()
{
  
  if(sender()==ptoolbuttons->pinsertAction){

    if(lightsmodel->rowCount()>7){
      QMessageBox::warning ( this,IDS_APP_NAME, 
			     "OpenGL restriction: Only 8 lights possible."
			     , 1, 0,0 );
      return;
    }

    QPoint pos;
    pos=tbAdd->mapToGlobal(QPoint(0,tbAdd->height()));
    tbAdd->menu()->popup(pos);
    return;



  }else if(sender()==ptoolbuttons->pdeleteAction){
    //myDebug()<<"tbDelete";
      
    QModelIndex currentindex=lvLights->currentIndex();
    if(currentindex ==  QModelIndex()){
      QMessageBox::warning ( this,IDS_APP_NAME, "Select a light first."
			     , 1, 0,0 );
      return;
    }

    int numrows=lightsmodel->rowCount();
    int row=currentindex.row();
    int col=currentindex.column();


  
    Lights::Light *li=lightsmodel->removeRow(row);
    if(li) delete li;
    lvLights
      ->setCurrentIndex(lightsmodel
			->index(MyMath::min<long>(row,numrows-2),col,QModelIndex()));

  }else if((sender()==ptoolbuttons->pmoveUpAction)||
	   (sender()==ptoolbuttons->pmoveDownAction)){
    
    QModelIndex currentindex=lvLights->currentIndex();
    if(currentindex ==  QModelIndex()){
      QMessageBox::warning ( this,IDS_APP_NAME, "Select an light first.", 1, 0,0 );
      return;
    }

    int row=currentindex.row();
    int col=currentindex.column();
    int by=-1;
    if(sender()==ptoolbuttons->pmoveDownAction) by=1;

    if(lightsmodel->moveRow(row,by)){
      lvLights->setCurrentIndex(lightsmodel->index(row+by,col,QModelIndex()));
    }
  }

  if(pcontinuous && !mutetalkback){
    emit talkBack(ProtoDlg::Accept);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::on_lvLights_currentItemChanged(const QModelIndex &currentindex
					      ,const QModelIndex &/*previous*/)
{
  if(mute) return;
  
  if(currentindex ==  QModelIndex()) return;
  int row=currentindex.row();
  //  int col=currentindex.column();
  
  //cbActive->setChecked(t_lights->at(row)->active());

  mutetalkback=true;
  typeHelper->setIndexFromData(t_lights->at(row)->type());

  cbDiffuse->setColor(t_lights->at(row)->diffuse());
  cbAmbient->setColor(t_lights->at(row)->ambient());
  cbSpecular->setColor(t_lights->at(row)->specular());
  dspbxP1->setValue(t_lights->at(row)->position().at(0));
  dspbxP2->setValue(t_lights->at(row)->position().at(1));
  dspbxP3->setValue(t_lights->at(row)->position().at(2));
  dspbxP4->setValue(t_lights->at(row)->position().at(3));

  dspbxD1->setValue(t_lights->at(row)->spotDirection().at(0));
  dspbxD2->setValue(t_lights->at(row)->spotDirection().at(1));
  dspbxD3->setValue(t_lights->at(row)->spotDirection().at(2));
  dspbxExp->setValue(t_lights->at(row)->spotExponent());
  dspbxCut->setValue(t_lights->at(row)->spotCutoff());

  dspbxK0->setValue(t_lights->at(row)->constantAttenuation());
  dspbxK1->setValue(t_lights->at(row)->linearAttenuation());
  dspbxK2->setValue(t_lights->at(row)->quadraticAttenuation());
  mutetalkback=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::onContentChanged(QObject* sender)
{
  if(mute) return;
  if(!t_lights) return;

  if(sender==cbContinuousUpdate){
    pcontinuous=cbContinuousUpdate->isChecked();
    if(pcontinuous && !mutetalkback){
      emit talkBack(ProtoDlg::Accept);
    }
  }else if(sender==cbShowLightHelpers){
    poglview->setShowLightHelpers(cbShowLightHelpers->isChecked());
    if(pcontinuous && !mutetalkback){
      emit talkBack(ProtoDlg::Accept);
    }
  }else if(sender==dspbxGlobalAmbientFactor){
    t_lights->setGlobalAmbientFactor(dspbxGlobalAmbientFactor->value());
    if(pcontinuous && !mutetalkback){
      emit talkBack(ProtoDlg::Accept);
    }
  }else if(sender==cbBackground){
    t_lights->setBackgroundColor(cbBackground->color());
    if(pcontinuous && !mutetalkback){
      emit talkBack(ProtoDlg::Accept);
    }
  }else if(sender==cbGlobalAmbient){
    t_lights->setBackgroundColor(cbBackground->color());
    if(pcontinuous && !mutetalkback){
      emit talkBack(ProtoDlg::Accept);
    }
  }

  QModelIndex currentindex=lvLights->currentIndex();
  if(currentindex ==  QModelIndex()) return;
  int row=currentindex.row();

  //QString isA(sender->metaObject()->className());
  //myDebug()<<"isA="<<isA;

  if(sender==cbDiffuse){
    (*t_lights)[row]->setDiffuse(cbDiffuse->color());
    
  }else if(sender==cbAmbient){
    (*t_lights)[row]->setAmbient(cbAmbient->color());
    
  }else if(sender==cbSpecular){
    (*t_lights)[row]->setSpecular(cbSpecular->color());
    
  }else if(sender==dspbxP1){
    (*t_lights)[row]->position()[0]=dspbxP1->value();
    
  }else if(sender==dspbxP2){
    (*t_lights)[row]->position()[1]=dspbxP2->value();
    
  }else if(sender==dspbxP3){
    (*t_lights)[row]->position()[2]=dspbxP3->value();
    
  }else if(sender==dspbxP4){
    (*t_lights)[row]->position()[3]=dspbxP4->value();
    
  }else if(sender==dspbxD1){
    (*t_lights)[row]->spotDirection()[0]=dspbxD1->value();
    
  }else if(sender==dspbxD2){
    (*t_lights)[row]->spotDirection()[1]=dspbxD2->value();
    
  }else if(sender==dspbxD3){
    (*t_lights)[row]->spotDirection()[2]=dspbxD3->value();
    
  }else if(sender==dspbxExp){
    (*t_lights)[row]->setSpotExponent(dspbxExp->value());
    
  }else if(sender==dspbxCut){
    (*t_lights)[row]->setSpotCutoff(dspbxCut->value());
    
  }else if(sender==dspbxK0){
    (*t_lights)[row]->setConstantAttenuation(dspbxK0->value());
    
  }else if(sender==dspbxK1){
    (*t_lights)[row]->setLinearAttenuation(dspbxK1->value());
    
  }else if(sender==dspbxK2){
    (*t_lights)[row]->setQuadraticAttenuation(dspbxK2->value());
    
  }else if(sender==cmbType){
    (*t_lights)[row]->setType(typeHelper->data());
    setLightTypeEnabled();
  }
  if(pcontinuous && !mutetalkback){
    emit talkBack(ProtoDlg::Accept);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::onModeldataChanged(const QModelIndex&,const QModelIndex&)
{
  if(pcontinuous && !mutetalkback){
    emit talkBack(ProtoDlg::Accept);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::setLightTypeEnabled()
{
  QModelIndex currentindex=lvLights->currentIndex();
  if(currentindex ==  QModelIndex()) return;
  int row=currentindex.row();

  const Lights::Light *li=t_lights->at(row);

  if(li->type()==Lights::Directional){
    lPosition->setText("&Position from camera");
    dspbxP4->setEnabled(false);
    dspbxD1->setEnabled(false);
    dspbxD2->setEnabled(false);
    dspbxD3->setEnabled(false);
    dspbxExp->setEnabled(false);
    dspbxCut->setEnabled(false);
    dspbxK0->setEnabled(false);
    dspbxK1->setEnabled(false);
    dspbxK2->setEnabled(false);
  }else if(li->type()==Lights::Positional){
    lPosition->setText("&Position from focus");
    dspbxP4->setEnabled(true);
    dspbxD1->setEnabled(false);
    dspbxD2->setEnabled(false);
    dspbxD3->setEnabled(false);
    dspbxExp->setEnabled(false);
    dspbxCut->setEnabled(false);
    dspbxK0->setEnabled(true);
    dspbxK1->setEnabled(true);
    dspbxK2->setEnabled(true);
  }else if(li->type()==Lights::Spot){
    lPosition->setText("&Position from focus");
    dspbxP4->setEnabled(true);
    dspbxD1->setEnabled(true);
    dspbxD2->setEnabled(true);
    dspbxD3->setEnabled(true);
    dspbxExp->setEnabled(true);
    dspbxCut->setEnabled(true);
    dspbxK0->setEnabled(true);
    dspbxK1->setEnabled(true);
    dspbxK2->setEnabled(true);
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::on_tbDefault_clicked()
{


  t_lights->setDefault();

  setLights();
  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::on_tbSaveTheme_clicked()
{
  LightThemes *lightthemes=new LightThemes();
  ThemeDlg *dlg=new ThemeDlg(this,ThemeDlg::Save,lightthemes);
  int ret=dlg->exec();
  if(ret==QDialog::Accepted){
    lightthemes->currentLights()=(*t_lights);
    lightthemes->saveSettings();

  }
  delete lightthemes;
  delete dlg;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void LightDlg::on_tbLoadTheme_clicked()
{
  LightThemes *lightthemes=new LightThemes();
  ThemeDlg *dlg=new ThemeDlg(this,ThemeDlg::Load,lightthemes);
  int ret=dlg->exec();
  if(ret==QDialog::Accepted){
    (*t_lights)=lightthemes->currentLights();
    setLights();
    lightthemes->saveSettings();
  }
  delete dlg;
  delete lightthemes;
}

/*EOF*/ 
