//todo dont save when not complete
//todo check the =.dens file for consistency
//todo help+guide

/*************************************************************************
 * @(#)MapAtomsDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   10 Feb 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QFileDialog>
#include "Assorted.hh"
#include "Except.hh"
#include "MapAtomsDlg.hh"
#include "DataExchange.hh"
#include "STR.hh"
#include "STDoc.hh"
#include "MyMath.hh"
#include "Exception.hh"
#include "AtomFindMarker.hh"
#include "MWBase.hh"
#include "Cluster.hh"
#include "IconCache.hh"
#include "Config.hh"
#include "MyDebug.hh"




/* ***********************************************************************
 *
 * ***********************************************************************
 */
MapAtomsDlg::MapAtomsDlg(QWidget *parent ,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name,true),pchanged(false),pisnew(true)
  ,p_doc(0),pstr(0),poldstr(0)
{

  MWBase *mwbase=dynamic_cast<MWBase*>(base);
  if(mwbase){
    pstr=dynamic_cast<STR*>(mwbase->control());
  }

  setupUi(this);
  setUp(0,pbClose,0,0,"MapAtomsDialog",": Density Mapper");
  

  headers << "New sites" << "Old sites" <<"dens file" ;
  twMap->setColumnCount(headers.count());
  twMap->setHorizontalHeaderLabels(headers);


  connect(&dataExchange
	  ,SIGNAL(dataAvailable(QObject *,const QObject *,const QString&
				,const void *))
	  ,this,SLOT(onDataAvailable(QObject *,const QObject *,const QString&
				     ,const void*)));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MapAtomsDlg::~MapAtomsDlg()
{
  if(poldstr){
    poldstr->oglView()->atomFindMarker()->clear();
  }
  if(pstr){
    pstr->oglView()->atomFindMarker()->clear();
  }


  dataExchange.unRegisterListener(this);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool MapAtomsDlg::dataAcceptable()
{
  if(pchanged){
    raise();
    long ret=QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
				  ,"Leave density mapper without saving changes?"
				  ,QMessageBox::Yes
				  ,QMessageBox::Save
				  ,QMessageBox::No|QMessageBox::Default
				  );
    if(ret==QMessageBox::Yes){
      return true;
    }else if(ret==QMessageBox::Save){
      on_pbSave_clicked();
      return true;
    }else
      return false;
  }
  return true;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbLoadSiteList_clicked()
{
  if(!p_doc) return;

  if(!pisnew){
    long ret=QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
				  ,"Really want a fresh start?"
				  ,QMessageBox::Yes|QMessageBox::Default
				  ,QMessageBox::No
			     );
    if(ret==QMessageBox::No){
      return;
    }
  }
  
  pisnew=false;

  pmapdata.clear();
  for(long site=0;site<p_doc->symmetry->sites().size();site++){
    pmapdata.insert(site,MapData(-1,-1,p_doc->symmetry->sites().at(site).el()));
  }


  load_twMap();

  pchanged=true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbSave_clicked()
{
  QString filename="=.densmap";
  QString fn;

  QString workdir=".";
  if(pstr){
    workdir=pstr->document()->currentDir()+"/";
  }
  
  filename=workdir+filename;
  
#ifdef Q_OS_MAC
  fn = QFileDialog::getSaveFileName(this,"Save File",filename);
#else
  fn = QFileDialog::getSaveFileName(this,"Save File",filename
				    , " FPLO files(=.*) ;; All files(*)"
				    ,0,0);
#endif
  if(fn.isEmpty()) return;
  
  FILE *FH;
  try{
    FH=openforwrite(String(fn));
  }catch(Except ex) {
    QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			 ,QString("Cannot open %1 for write!").arg(fn)
			 ,QMessageBox::Ok|QMessageBox::Default
			 );
    return;
  }
  
  QList<long> keys=pmapdata.keys();
  for(long i=0;i<keys.size();i++){
    long site=keys.at(i);
    fprintf(FH,"%ld %2s %ld %ld %s\n",site+1,
	    pmapdata[site].el(),
	    pmapdata[site].site+1,
	    pmapdata[site].sort+1,String(pmapdata[site].file)());

  }
  fclose(FH);
  pchanged=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbLoad_clicked()
{
  QString filename="=.densmap";
  QString fn;

  QString workdir=".";
  if(pstr){
    workdir=pstr->document()->currentDir()+"/";
  }
  
  filename=workdir+filename;
  
#ifdef Q_OS_MAC
  fn = QFileDialog::getOpenFileName(this,"Open File",filename);
#else
  fn = QFileDialog::getOpenFileName(this,"Open File",filename
				    , " FPLO files(=.*) ;; All files(*)"
				    ,0,0);
#endif

  if(fn.isEmpty()) return;
  
  FILE *FH;
  try{
    FH=openforread(String(fn));
  }catch(Except ex) {
    QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			 ,QString("Cannot open %1 for read!").arg(fn)
			 ,QMessageBox::Ok|QMessageBox::Default
			 );
    return;
  }

  pmapdata.clear();
  for(long site=0;site<p_doc->symmetry->sites().size();site++){
    pmapdata.insert(site,MapData());
  }

  bool inconsistent=false;
  String line;
  while (getLine(FH,line)){
    line=line.chop("\n").stripBlanks();
    if(line[0]=='#') continue;
    if(line.empty()) continue;
    StringList sl=line.split(" \t");
    long site=-1,oldsite=-1,oldsort=-1;
    String oldel="??";
    QString file="";
    if(sl.size()==4 || sl.size()==5){
      long i,j;
      j=0;
      if(String::islong(&i,sl[j]())){
	site=i-1;
      }else
	inconsistent=true;
      j++;
      if(sl[j].length() <1 || sl[j].length()>2)
	inconsistent=true;
      oldel=sl[j]();
      j++;
      if(String::islong(&i,sl[j]())){
	oldsite=i-1;
      }else
	inconsistent=true;
      j++;
      if(String::islong(&i,sl[j]())){
	oldsort=i-1;
      }else
	inconsistent=true;
      j++;
      if(sl.size()==5)
	file=sl[j]();
    }else{
      inconsistent=true;
    }
    //myDebug() << "load "<<site<<oldsite<<oldel()<<inconsistent;
    pmapdata[site]=MapData(oldsite,oldsort,oldel,file);
  }
  

  fclose(FH);

  if(inconsistent){
    QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			 ,QString("Data in file \"%1\" seem to be corrupted!\n"
				  "Check resulting table, correct it,or reset.")
			 .arg(fn)
			 ,QMessageBox::Ok|QMessageBox::Default
			 );

  }

  load_twMap();

  pchanged=false;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void MapAtomsDlg::load_twMap()
{
  // load the table from pmapdata, which must be set up beforehand

  long currentrow=twMap->currentRow();

  twMap->clear();
  twMap->setRowCount(0);
  twMap->setHorizontalHeaderLabels(headers);

  twMap->setHorizontalHeaderLabels(QStringList());

  const ValueArray<Site> &sites=p_doc->symmetry->sites();

  //todo check consistency

  bool inconsistent=false;
  

  QList<long> keys=pmapdata.keys();
  long row=-1;
  for(long i=0;i<pmapdata.keys().size();i++){
    long site=keys.at(i);
    //myDebug() << site<<oldsite;


    if(site>=sites.size() || site<0){
      inconsistent=true;
    }else{

      
      row++;
      twMap->setRowCount(row+1);

      QTableWidgetItem *item;
      QTableWidgetItem *itemf;
      item=twMap->item(row,0);
      if(!item) twMap->setItem(row,0,item=new QTableWidgetItem(""));
      item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
      item->setData(Qt::UserRole,(int)site);
      
      item=twMap->item(row,1);
      if(!item) twMap->setItem(row,1,item=new QTableWidgetItem(""));
      item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
      item->setData(Qt::UserRole,(int)site);

      itemf=twMap->item(row,2);
      if(!itemf) twMap->setItem(row,2,itemf=new QTableWidgetItem(""));
      itemf->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
      itemf->setData(Qt::UserRole,(int)site);
      

      const Site &at=sites.at(site);

      
      
      
      twMap->item(row,0)
	->setText(Sprintf("%-2s (S%ld,W%ld,T%ld)",at.el()(),site+1,at.sort()+1
			  ,at.type())());
      twMap->item(row,0)->setIcon(iconcache.getAtomIcon(at.property().material().color()));

      
      //myDebug() << "site,oldsite"<<site<<oldsite;
      item=twMap->item(row,1);
      setOldSiteItemText(item,itemf,pmapdata[site]);
    }
  }
  //  myDebug() <<"hier";
  
  
  long r=MyMath::min<long>(MyMath::max<long>(0,currentrow),row);
  if(r<twMap->rowCount() && r>=0){
    twMap->setCurrentCell(r,1,QItemSelectionModel::Select);
    twMap->setRangeSelected(QTableWidgetSelectionRange(r,0,r,2),true);
  }

  int rows=twMap->rowCount();
  for(int irow=0;irow<rows;irow++)
    twMap->resizeRowToContents(irow);
  int cols=twMap->columnCount();
  for(int icol=0;icol<cols;icol++)
    twMap->resizeColumnToContents(icol);


  if(inconsistent){
    QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			 ,QString("Current data are inconsistent with "
				  "structure !")
			 ,QMessageBox::Ok|QMessageBox::Default
			 );

  }

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbOpenOldStructure_clicked()
{
  //  myDebug() <<"MapAtomsDlg::on_pbOpenOldStructure_clicked()";

  if(poldstr){
    long ret=QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
				  ,"You have already opened an old structure "
				  "view.\nProceed?"
				  ,QMessageBox::Yes
				  ,QMessageBox::No|QMessageBox::Default
				  );
    if(ret==QMessageBox::No) return;
  }

  if(pcurrentstrfile.isEmpty())
    pcurrentstrfile=pstr->document()->currentDir()+"/=.in";
  
    QString fn;
#ifdef Q_OS_MAC
    fn = QFileDialog::getOpenFileName(this,"Open File",pcurrentstrfile);
#else
    fn = QFileDialog::getOpenFileName(this,"Open File",pcurrentstrfile
				      , " FPLO files(=.*) ;; All files(*)"
				    ,0,0);
#endif
    if(fn.isEmpty()) return;
    pcurrentstrfile=fn;

  if(pstr){
    // create a new window for structure view

    if(poldstr){
      poldstr->oglView()->atomFindMarker()->clear();
    }

    dataExchange.registerListener(this,"structure loaded");


    QFileInfo fi(pcurrentstrfile);
    MWBase *mwbase=
      pstr->base()
      ->callActivityMenuEntry(Activity(Activity::Structure
				       ,fi.absolutePath()
				       ,fi.fileName()));
    
    if(mwbase){
      poldstr=dynamic_cast<STR*>(mwbase->control());
      
      QList<int> spare;
      spare
	<<Actions::Activities
	<<Actions::File_Save
	<<Actions::File_SaveAs
	<<Actions::File_Print
	<<Actions::File_Close
	<<Actions::Plot
	<<Actions::Input_BoundaryDlg
	<<Actions::Tools_Measure
	<<Actions::Tools_FindAtom
	<<Actions::Run
	<<Actions::Window
	<<Actions::Help
	;
      mwbase->setEnableAction_but(false,spare);
    }

  }
      
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbHide_toggled()
{
  //myDebug() <<"MapAtomsDlg::on_pbHide_toggled()" << pbHide->isChecked();
  if(pbHide->isChecked()){
    pbHide->setText("Show all");
  }else{
    pbHide->setText("Hide matched");
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::setData(AtomMarkerProperties &amp)
{
  if(pstr)
    p_doc=(STDoc*)pstr->document();

  on_pbLoadSiteList_clicked();


  wAMP->setData(&amp);

  twMap->setFocus();
  twMap->setCurrentCell(0,1,QItemSelectionModel::Select);
  twMap->setRangeSelected(QTableWidgetSelectionRange(0,0,0,2),true);
  
  dataExchange.registerListener(this,"atom data");


  pchanged=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_twMap_currentCellChanged(int row,int col)
{
  NSF("MapAtomsDlg::on_twMap_currentCellChanged(int row,int col)");
  if(!p_doc) return;
  //myDebug() << nsubfunc<< row << col;
  
  if(col==1){
    int site=twMap->item(row,col)->data(Qt::UserRole).toInt();
    if(pstr){
      pstr->oglView()
	->atomFindMarker()->mark(p_doc->cluster,Xf::FindSite,site+1);
    }      
    if(poldstr){
      long oldsite=pmapdata.value(site).site;
      //       if(oldsite>=0){
      poldstr->oglView()->atomFindMarker()
	->mark(((STDoc*)poldstr->document())->cluster
	      ,Xf::FindSite,oldsite+1);
      //       }else{
      // 	poldstr->oglView()->atomFindMarker().clear();
      //       }
      
    }      
  }
  
  if(col==0||col==2){
    twMap->setCurrentCell(row,1);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::selectSite(long site)
{
  for(int row=0;row<twMap->rowCount();row++){
    long isa=twMap->item(row,0)->data(Qt::UserRole).toInt();
    if(isa==site){
      twMap->setCurrentCell(row,1);
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::onDataAvailable(QObject *sender,const QObject *
				  ,const QString& expectedData
				  ,const void* data)
{
  if(expectedData=="atom data" && data){
    const Atom *at=(Atom *)data;
    if(at && poldstr && sender==poldstr){
      //myDebug() << sender<<at->site;
      QTableWidgetItem *item=twMap->item(twMap->currentRow(),1);
      QTableWidgetItem *itemf=twMap->item(twMap->currentRow(),2);
      if(item && itemf){
	long site=item->data(Qt::UserRole).toInt();
	pmapdata[site]=MapData(at->site,at->sort,at->el,
			       QFileInfo(pcurrentstrfile).
			       absolutePath()+"/=.dens");
	setOldSiteItemText(item,itemf,pmapdata[site]);
	pchanged=true;
      }
      
      /*
      STR *newoldstr=dynamic_cast<STR*>(sender);
      if(newoldstr && newoldstr!=poldstr){
	if(poldstr){
	  poldstr->oglView()->atomFindMarker().clear();
	}
	poldstr=newoldstr;
      }
      */

      if(poldstr){
	poldstr->oglView()->atomFindMarker()
	  ->mark(((STDoc*)poldstr->document())->cluster
		,Xf::FindSite,at->site+1);
      }
    }
  }

  if(poldstr && sender==poldstr && expectedData=="structure loaded" ){
    //myDebug() << "\n\n\n\n\nstructure loaded\n\n\n\n\n";
    load_twMap();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_tbClearOldSite_clicked()
{
  QTableWidgetItem *item=twMap->item(twMap->currentRow(),1);
  QTableWidgetItem *itemf=twMap->item(twMap->currentRow(),2);
  if(item && itemf){
    long site=item->data(Qt::UserRole).toInt();
    pmapdata[site]=MapData();
    setOldSiteItemText(item,itemf,pmapdata[site]);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::setOldSiteItemText(QTableWidgetItem *item,
				     QTableWidgetItem *itemf,
				     const MapData &md)
{
  if(!item) return;
  
  if(md.site<0){
    item->setText("");
    itemf->setText("");
  }else{
    String text=Sprintf("%-2s (S%ld,W%ld)",md.el(),md.site+1,md.sort+1);
//     if(pstr){
//       const QList<Site> &sites=((STDoc*)pstr->document())
// 	->symmetry->sites();
//       if(md.site>=0 && md.site<sites.size()){
// 	const Site &at=sites.at(md.site);
// 	item->setIcon(iconcache.getAtomIcon(at.atomColor));
//       }
//     }
    item->setText(text());
    itemf->setText(md.file);
    
  }

  int rows=twMap->rowCount();
  for(int irow=0;irow<rows;irow++)
    twMap->resizeRowToContents(irow);
  int cols=twMap->columnCount();
  for(int icol=0;icol<cols;icol++)
    twMap->resizeColumnToContents(icol);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MapAtomsDlg::on_pbCopyThisAtom_clicked()
{
  

  QTableWidgetItem *item=twMap->item(twMap->currentRow(),1);
  if(!item) return;

  long currentsite=item->data(Qt::UserRole).toInt();


  MapData md=pmapdata[currentsite];

  for(long i=0;i<pmapdata.keys().size();i++){
    long site=pmapdata.keys().at(i);
    if(pmapdata[site].el==md.el){
      pmapdata[site]=md;
    }
  }
  load_twMap();

  pchanged=true;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void MapAtomsDlg::enableParent(bool en)
// {
//   QObject *o=this;
//   while(o){
//     //myDebug() << "o" << o << "o.parent()"<< o->parent();
//     if(o  ){
//       MWBase *mw=qobject_cast<MWBase *>(o);
//       if(mw){
// 	mw->enable_but(en,QString("activities help").split(" "));
//       }
//     }
//     o=o->parent();
    
//   }
// }

/*EOF*/ 
