/*************************************************************************
 * @(#)MeshDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox>
#include "MeshDlg.hh"
#include "Config.hh"
#include "Valid.hh"
#include "HelpDlg.hh"
#include "Erg.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MeshDlg::MeshDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + ": Mesh generation");

  //  setWindowModality ( Qt::WindowModal);

  new IntValidator(this,leSubdiv,"The number of subdivisions",0,30000);

  leSubdiv->selectAll();
  leSubdiv->setFocus();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshDlg::accept()
{

  //int i;
  QString s,txt="";
  //QString apname=qApp->argv()[0];

  ((IntValidator*)leSubdiv->validator())->check(txt);


  {
    IntValidator iv(this,0,"Manual X subdivision",1,30000);
    s=leSubX->text().simplified();
    if(!s.isEmpty()){
      iv.check(txt,s);
    }
  }

  {
    IntValidator iv(this,0,"Manual Y subdivision",1,30000);
    s=leSubY->text().simplified();
    if(!s.isEmpty()){
      iv.check(txt,s);
    }
  }
  {
    IntValidator iv(this,0,"Manual Z subdivision",1,30000);
    s=leSubZ->text().simplified();
    if(!s.isEmpty()){
      iv.check(txt,s);
    }
  }


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshDlg::setData( const Energies *erg )
{
  t_nx=erg->xSubdiv();
  t_ir=erg->useSymmetry();
  t_useboundarycell=erg->useBoundaryCell();
  QString s;
  t_nmx=erg->xSubdivManual();
  t_nmy=erg->ySubdivManual();
  t_nmz=erg->zSubdivManual();
  t_manual=erg->manualSubDiv();
  leSubdiv->setText(QString().setNum(t_nx));

  
  cbUseBoundaryCell->setChecked(erg->useBoundaryCell());
    
  s="";
  if(t_nmx>0) s=QString().setNum(t_nmx);  
  leSubX->setText(s);

  s="";
  if(t_nmy>0) s=QString().setNum(t_nmy);  
  leSubY->setText(s);

  s="";
  if(t_nmz>0) s=QString().setNum(t_nmz);  
  leSubZ->setText(s);
    
  leSubdiv->setText(QString().setNum(t_nx));
  if(t_ir) cbIrred->setChecked(true);
  grpbxManual->setChecked(t_manual);

  //  on_grpbxManual_clicked();  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshDlg::getData( Energies *erg)
{
  bool ok;
  QString t=leSubdiv->text();
  IntValidator::stripAllWhiteSpace(t);
  erg->setXSubdiv(t.toLong(&ok));
  erg->setUseSymmetry(cbIrred->isChecked());


  t=leSubX->text().simplified();
  long n=0;
  if(!t.isEmpty()) n=t.toLong(&ok);
  erg->setManualXSubdiv(n);

  t=leSubY->text().simplified();
  n=0;
  if(!t.isEmpty()) n=t.toLong(&ok);
  erg->setManualYSubdiv(n);

  t=leSubZ->text().simplified();
  n=0;
  if(!t.isEmpty()) n=t.toLong(&ok);
  erg->setManualZSubdiv(n);
  
  erg->setUseBoundaryCell(cbUseBoundaryCell->isChecked());

  erg->setManualSubDiv(grpbxManual->isChecked());

  bool chn=erg->meshDefChanged();
  chn|=( erg->xSubdiv() != t_nx );
  chn|=( erg->xSubdivManual() != t_nmx );
  chn|=( erg->ySubdivManual() != t_nmy );
  chn|=( erg->zSubdivManual() != t_nmz );
  chn|=( erg->manualSubDiv() != t_manual );
  chn|=( erg->useBoundaryCell() != t_useboundarycell );
  chn|=( erg->useSymmetry() != t_ir );
  erg->setMeshDefChanged(chn);

}


// /* ***********************************************************************
//  *
//  * ***********************************************************************
//  */
// void MeshDlg::on_grpbxManual_clicked()
// {
//   long t_manual=grpbxManual->isChecked();

//   //grpbxAuto->setEnabled(t_manual);
//   grpbxManual->setEnabled(t_manual);

// }


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MeshDlg::slotHelp()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","MeshDialog");
  h->show();
  //  h->deleteLater();
  
}

/*EOF*/ 
