/*************************************************************************
 * @(#)OptDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox> 
#include "OptDlg.hh"
#include "Config.hh"
#include "Vlevel.hh"
#include "Valid.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OptDlg::OptDlg(QWidget *parent,QWidget *base,const char *name) 
  : BrainDialog(parent,base,name,false)
{
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + ": options");

  //  setWindowModality ( Qt::WindowModal);

  resize(1,1);

  cbxVlevel->clear();
  for(long i=0;i<Vlevel::noVerbosityLevels();i++){
    cbxVlevel->insertItem(i,Vlevel::verbosityLevelFromIndex(i).name());
  }

  
  new RealValidator(this,leTol,"The tolerance",1.e-30,1.e10);


}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void OptDlg::setData( const Vlevel &vlevel ,double tol)
{
  t_vlevel=vlevel;
  
  cbxVlevel->setCurrentIndex(vlevel.index());  

  t_tol=QString().setNum(tol,'g',15);
  leTol->setText(t_tol);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OptDlg::getData( Vlevel &vlevel, double &tol,long &chn )
{
  bool ok;
  //QString t=leSubdiv->text();
  //IntValidator::stripAllWhiteSpace(t);
  vlevel=Vlevel::verbosityLevelFromIndex(cbxVlevel->currentIndex());
  chn|=( vlevel != t_vlevel );


  QString t=leTol->text();
  chn |= ( t_tol != t );
  IntValidator::stripAllWhiteSpace(t);
  tol=t.toDouble(&ok);



}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OptDlg::accept()
{
  //int i;
  QString s,txt="";

  ((IntValidator*)leTol->validator())->check(txt);


  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }
  
}

/*EOF*/ 
