/*************************************************************************
 * @(#)PrntDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox> 
#include <QFileDialog> 
#include "PrntDlg.hh"
#include "Config.hh"
#include "DocBase.hh"
#include "HelpDlg.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PrntDlg::PrntDlg(QWidget *parent,const char *name)
  : QDialog(parent),pdoc(0)
{
  setObjectName(name);
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + ": Print");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrntDlg::setData( DocBase *doc,const QString &filename)
{
  pdoc=doc;
  MASSERT("",pdoc);
  t_filename=filename;


  pdoc->saveCurrentDir();
  QFileInfo fi(t_filename);
  bool isabs=fi.isAbsolute();
  QString ap=fi.absolutePath();
  QString fn=fi.fileName();
  pdoc->restoreCurrentDir();

  if(isabs){
    if(ap==pdoc->currentDir()){
      t_filename=fn;
    }
  }
  

  t_magfac=pdoc->magfac;
  leFileName->setText(t_filename);
  spbxMagFac->setValue(pdoc->magfac);

  t_quality=pdoc->picquality;
  spbxQuality->setValue(pdoc->picquality);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrntDlg::getData( DocBase *doc,QString &filename)
{
  filename=(leFileName->text()).trimmed();
  doc->magfac=spbxMagFac->value();
  doc->picquality=spbxQuality->value();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrntDlg::accept()
{
  QString txt="";

  
  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
  }else{
    QDialog::accept();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrntDlg::on_tbChooseFile_clicked()
{
  QString selectedfilter="All files";

  MASSERT("",pdoc);

  pdoc->saveCurrentDir();
  QFileInfo fi(leFileName->text().trimmed());
  QString p=fi.absoluteFilePath();
  pdoc->restoreCurrentDir();

  long ret=QDialog::Accepted;
  
  QFileDialog *dlg
    =new QFileDialog(this,"Save image",".",
		     "Images files (*.jpg *.png *.jpeg *.bmp *.ps)"
		     ";;All files (*)");
  dlg->setAcceptMode(QFileDialog::AcceptSave);
  dlg->setFileMode(QFileDialog::AnyFile);
#ifdef Q_OS_MAC
  dlg->setOptions(QFileDialog::DontUseNativeDialog | 
		  QFileDialog::DontConfirmOverwrite);
#endif
  dlg->setOptions(QFileDialog::DontConfirmOverwrite);
  dlg->setDirectory(QDir::currentPath()+"/");
  
  if(!p.isEmpty()){
    dlg->selectFile(p);
  }else{
    dlg->selectFile("str.jpg");
  }
  QString filename;
  if((ret=dlg->exec())==QDialog::Accepted){
    if(dlg->selectedFiles().size()>0)
      filename=dlg->selectedFiles().at(0);
  }
  delete (dlg);


  if(!filename.isEmpty()){
    pdoc->saveCurrentDir();
    QFileInfo fi(filename);
    QString ap=fi.absolutePath();
    QString fn=fi.fileName();
    pdoc->restoreCurrentDir();
    if(ap==pdoc->currentDir())
      leFileName->setText(fn);
    else
      leFileName->setText(filename);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PrntDlg::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","PrintingDialog");
  h->show();
  //  h->deleteLater();
  
}

/*EOF*/ 
