/*************************************************************************
 * @(#)Propview.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   07 Aug 2009
 *************************************************************************
 */

#include <QImage>
#include <QKeyEvent>
#include <QResource>
#include <QPainter>
#include <QUrl>
#include "Propview.hh"
#include "Cluster.hh"
#include "IconCache.hh"
#include "HelpDlg.hh"
#include "OGLView.hh"
#include "constants.h"
#include "MyMath.hh"
#include "STDoc.hh"
#include "Config.hh"
#include "MyDebug.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool PropViewFilter::eventFilter(QObject *object, QEvent *event)
{
  //NSF("bool Filter::eventFilter(QObject *object, QEvent *event)");
  if (event->type() == QEvent::KeyPress) {
    QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
    //myDebug() << nsubfunc << keyEvent->key()<<object->parent();
    PropView *pv = dynamic_cast<PropView *>(object->parent());
    if (keyEvent->key() == Qt::Key_M) {
      if(pv) pv->toggleVisiblity();
      return true;
    }else if(keyEvent->key() == Qt::Key_Escape) {
      if(pv) pv->fullReset();
      return true;
    }else
      return QObject::eventFilter(object,event);
  }
  return QObject::eventFilter(object,event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static int UNITS_CONV=3;
static int UNITS_PRIM=4;

const char *PropView::header=
  "    <table border=\"0\" cellspacing=\"5\" cellpadding=\"0\">\n"
  "<tr>\n"
  "<th>No</th>\n"
  "<th colspan=\"2\">Elm</th>\n"
  "<th>Site</th>\n"
  "<th>Wyck</th>\n"
  "<th colspan=\"8\">Pos</th>\n</tr>\n"
  "<tr><td colspan=\"13\" height=\"1\"><hr></td></tr>";

const char *PropView::footer="</table>\n"; 

const char *PropView::tablerow="<tr>\n"
  "<td>%1</td>\n" //No
  "<td> <img src=\"mydata://ball%2\"/></td>\n" //Ball
  "<td>%3</td>\n" //Elm
  "<td>%4</td>\n" //Site
  "<td>%5</td>\n" //Sort
  "<td>(</td>\n" //Pos
  "<td>%6</td>\n" //Pos
  "<td>,</td>\n" //Pos
  "<td>%7</td>\n" //Pos
  "<td>,</td>\n" //Pos
  "<td>%8</td>\n" //Pos
  "<td>)</td>\n" //Pos
  "<td>%9</td>\n" //unit
  "</tr>\n";

const char *PropView::tablerowdif="<tr>\n"
  "<td></td>\n" //No
  "<td></td>\n" //Elm
  "<td>dif</td>\n" //Ball
  "<td></td>\n" //Site
  "<td></td>\n" //Sort
  "<td>(</td>\n" //Pos
  "<td>%6</td>\n" //Pos
  "<td>,</td>\n" //Pos
  "<td>%7</td>\n" //Pos
  "<td>,</td>\n" //Pos
  "<td>%8</td>\n" //Pos
  "<td>)</td>\n" //Pos
  "<td>%9</td>\n" //unit
  "</tr>\n";

const char *PropView::distrow=
  "<tr>\n<th colspan=\"5\" align=\"left\">dist %1</th>\n"
  "<th>=</td>\n"
  "<th colspan=\"6\" align=\"left\">&nbsp;%2</td>\n"
  "<th align=\"left\">aB</td>\n"
  "</tr>\n"
  "<tr>\n<th colspan=\"5\"></th>\n"
  "<th>=</td>\n"
  "<th colspan=\"6\" align=\"left\">&nbsp;%3</td>\n"
  "<th align=\"left\">Ang</td>\n"
  "</tr>\n"; 
const char *PropView::anglerow=
  "<tr>\n<th colspan=\"5\" align=\"left\">angle 1-2-3</th>\n"
  "<th>=</td>\n"
  "<th colspan=\"7\" align=\"left\">&nbsp;%1</td>\n"
  "</tr>\n";

/* ***********************************************************************
 *
 * ***********************************************************************
 */

PropView::PropView(QWidget *parent,STDoc *d,OGLView *v)
  :QWidget(parent,Qt::Window),doc(d),oglview(v),filter(0)
{
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + ": Measure distances");

  connect(&ppicker,SIGNAL(atomPropPickerChanged()),
 	  this,SLOT(onAtomPropPickerChanged()));


  cmbUnits->clear();
  cmbUnits->insertItem(0,"Bohr radii",Xf::BohrRadius);
  cmbUnits->insertItem(1,"Angstroem" ,Xf::Angstroem);
  UNITS_CONV=MyMath::max<double>(Xf::BohrRadius,Xf::Angstroem)+1;
  UNITS_PRIM=UNITS_CONV+1;
  cmbUnits->insertItem(2,"Conventional cell" ,UNITS_CONV);
  cmbUnits->insertItem(3,"Primitive cell" ,UNITS_PRIM);

  textView->setFocus();

  PropViewFilter *filter=new PropViewFilter();
  cmbUnits->installEventFilter(filter);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
PropView::~PropView()
{
  if(filter) delete filter;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void PropView::keyPressEvent(QKeyEvent *e) 
// {
//   if(e->key() ==Qt::Key_Escape &&
//      e->modifiers()==Qt::NoModifier ){
//     myDebug() << "emit escapePressed";
//     e->accept();
//     QWidget::keyPressEvent(e);
//     emit escapePressed();
//     return;
//   }

//   if(e->key() ==Qt::Key_M &&
//      e->modifiers()==Qt::NoModifier ){
//     myDebug() << "emit mPressed";
//     e->accept();
//     QWidget::keyPressEvent(e);
//     emit mPressed();
//     return;
//   }

//   QWidget::keyPressEvent(e);
  
// }
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","AtomDistanceMeasureDialog");
  h->show();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::on_cmbUnits_currentIndexChanged(int index)
{
  Q_UNUSED(index);
  onAtomPropPickerChanged();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::reset()
{
  textView->clear();
  ppicker.reset();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::fullReset()
{
  oglview->setAtomPropPicker(0);
  reset();
  if(isVisible()) oglview->setAtomPropPicker(&ppicker);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::toggleVisiblity()
{
  if(isVisible()){
    oglview->setAtomPropPicker(0);
    reset();
    hide();
  }else{
    oglview->setAtomPropPicker(&ppicker);
    reset();
    show();
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::closeEvent(QCloseEvent *e)
{
  if(isVisible()){
    oglview->setAtomPropPicker(0);
    reset();
  }
  QWidget::closeEvent(e);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::loadIcon(QRgb rgb)
{
	
  QString url=QString("mydata://ball%1").arg(rgb);
  textView->document()->addResource(QTextDocument::ImageResource
				    ,QUrl(url)
				    , QVariant(iconcache.getAtomImage(rgb)));
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */

void PropView::applyUnits(Vec3 &v,QString &unit)
{
  MASSERT("Oops ",doc);
  const Symmetry *sym=doc->symmetry;
  long lu=cmbUnits->itemData(cmbUnits->currentIndex(),Qt::UserRole).toInt();

  if(lu==Xf::Angstroem){
    unit="Ang";
    if(sym->dataLengthUnit()==Xf::BohrRadius)
      v*=constant_aBtoAng;
  }else if(lu==Xf::BohrRadius){
    unit="aB";
    if(sym->dataLengthUnit()==Xf::Angstroem)
      v/=constant_aBtoAng;
  }else if(lu==UNITS_CONV){
    unit="Conv";
    if(sym->dataStructureType()==Xf::STCrystal){
      Matrix3x3 G=sym->ccell().inverseT();
      v=v*G;
    }else{
      if(sym->dataLengthUnit()==Xf::BohrRadius)
	unit="aB";
      else
	unit="Ang";
    }
  }else if(lu==UNITS_PRIM){
    unit="Prim";
    if(sym->dataStructureType()==Xf::STCrystal){
      Matrix3x3 G=sym->cell().inverseT();
      v=v*G;
    }else{
      if(sym->dataLengthUnit()==Xf::BohrRadius)
	unit="aB";
      else
	unit="Ang";
    }
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::onAtomPropPickerChanged()
{
  //  myDebug() << "PropView::onAtomPropPickerChanged";
  
  //myDebug() << ppicker.list();
  MASSERT("Oops ",doc);

  const Symmetry *sym=doc->symmetry;
  

  textView->clear();

  QString txt=header;
    
  double d1ab=0;
  double d1A=0;
  double d2ab=0;
  double d2A=0;
  double angle=0;
  Vec3 n1,v2,dv21;
  if(ppicker.size()>=2){
    Vec3 v1=doc->cluster->atom(ppicker.at(0)).tauD;
    v2=doc->cluster->atom(ppicker.at(1)).tauD;
    n1=(v1-v2);
    dv21=-n1;
    d1ab=d1A=norm(n1);
    n1/=d1A;
    if(sym->dataLengthUnit()==Xf::BohrRadius){
      d1A*=constant_aBtoAng;
    }else{
      d1ab/=constant_aBtoAng;
    }
  }
  QString sd1=QString(distrow).arg("1st 2nd").arg(d1ab).arg(d1A);
  
  if(ppicker.size()>=3){
    Vec3 v3=doc->cluster->atom(ppicker.at(2)).tauD;
    Vec3 n2=v3-v2;
    d2ab=d2A=norm(n2);
    n2/=d2A;
    if(sym->dataLengthUnit()==Xf::BohrRadius){
      d2A*=constant_aBtoAng;
    }else{
      d2ab/=constant_aBtoAng;
    }
    angle=acos(MyMath::max<double>(-1,MyMath::min<double>(n1*n2,1)))/Pi*180.;
  }
  QString sangle=QString(anglerow).arg(angle);
  
  QString sd2=QString(distrow).arg("2nd 3rd").arg(d2ab).arg(d2A);

  

  for(int i=0;i<ppicker.size();i++){

    const Atom &at=doc->cluster->atom(ppicker.at(i));
    Vec3 v=at.tauD;
    QString unit;
    StringList vC;
    applyUnits(v,unit);
    vC=v.toString(6);
    
    QString s=QString(tablerow).arg(at.index+1).arg(at.property.material().color())
      .arg(at.el()).arg(at.site+1)
      .arg(at.sort+1).arg(vC[0]()).arg(vC[1]()).arg(vC[2]())
      .arg(unit);
    txt+=s;
    loadIcon(at.property.material().color());
    
  }
  StringList dv21C;
  QString unit;
  if(ppicker.size() >=2){
    applyUnits(dv21,unit);
    dv21C=dv21.toString(6);
  }
  if(ppicker.size() ==2){
    QString s=QString(tablerowdif)
      .arg(dv21C[0]()).arg(dv21C[1]()).arg(dv21C[2]()).arg(unit);
    txt+=s;
  }
  if(ppicker.size() >=2) txt+=sd1;
  if(ppicker.size() ==3) txt+=sd2;
  if(ppicker.size() ==3) txt+=sangle;
  
  textView->setText(txt+footer);

  txt="";
  if(ppicker.size() >=2){
    const Atom &at=doc->cluster->atom(ppicker.at(1));
    txt+=QString("    site %1\n    difvec %2")
      .arg(at.site+1)
      .arg(dv21.printS("%30.15f"," ",true)());
    teCopy->append(txt);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::on_pbSave_clicked()
{
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void PropView::on_pbClear_clicked()
{
  teCopy->clear();
}


/*EOF*/



