/*************************************************************************
 * @(#)ProtoDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   07 Jan 2012
 *************************************************************************
 */

#include <QMessageBox>
#include "ProtoDlg.hh"
#include "HelpDlg.hh"
#include "Config.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
ProtoDlg::ProtoDlg(QWidget *parent ,QWidget *base,const char *name,
		   bool stayontop)
  : BrainDialog(parent,base,name,false,stayontop),mutecontentchanged(false),
    pconnected(false)
{
  
  connect(this,SIGNAL(talkBack(int)),
	  this,SLOT(onDlgEnd(int)));

  // connect(this,SIGNAL(contentChanged())
  // 	  ,this,SLOT(onContentChanged()));
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ProtoDlg::setUp(QPushButton *pbAccept,QPushButton *pbClose,
		     QPushButton *pbCancel,QPushButton *pbHelp,
		     const QString &helpchapter,
		     const QString &title
		     )
{
  if(pbAccept) {
    pbAccept->setToolTip(IDS_APPLY_TIP);
    connect(pbAccept,SIGNAL(clicked()),this,SLOT(onpbAcceptClicked()));
  }
  if(pbClose)  {
    pbClose->setToolTip(IDS_CLOSE_TIP);
    connect(pbClose,SIGNAL(clicked()),this,SLOT(onpbCloseClicked()));
  }
  if(pbCancel) {
    pbCancel->setToolTip(IDS_CANCEL_TIP);
    connect(pbCancel,SIGNAL(clicked()),this,SLOT(reject()));  
  }

  if(pbHelp) {
    connect(pbHelp,SIGNAL(clicked()),this,SLOT(onpbHelpClicked()));  
  }

  setWindowTitle(IDS_APP_NAME + title);

  setWindowIcon(QIcon(":/Icons/logo32"));

  phelpchapter=helpchapter;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ProtoDlg::~ProtoDlg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ProtoDlg::connectChanged(QObject *obj,QList<QObject*> exceptlist)
{
  NSF("ProtoDlg::connectChanged(QObject *obj)");

  pconnected=true;
  
  bool stop=false;
  //myDebug()<<obj->metaObject()->className();
  foreach(QObject *o,obj->children()){
    if(o->isWidgetType()){
      QString isA(o->metaObject()->className());
      //myDebug()<<"\t"<<isA;
      if(exceptlist.contains(o)) continue;
      if(isA=="QLabel") continue;
      if(isA=="QMenu") continue;
      if(isA=="QScrollBar") continue;
      if(isA=="QTabBar") continue;
      if(isA=="QHeaderView") continue;
      if(isA=="QTableView") continue;
      if(isA=="QTableCornerButton") continue;
      if(isA=="QPushButton") continue;

      if(isA=="QLineEdit"){
 	connect(o,SIGNAL(textChanged(const QString &))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QRadioButton"){
 	connect(o,SIGNAL(toggled(bool))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QListWidget"){ 
 	connect(o,SIGNAL(itemChanged(QListWidgetItem*))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="WeightSelListView"){ 
 	connect(o,SIGNAL(myDataChanged())
 		,this,SLOT(onContentChanged()));
      }else if(isA=="ColorButton"){ 
 	connect(o,SIGNAL(colorChanged(QRgb))
 		,this,SLOT(onContentChanged()));
      }else if(isA=="QToolButton"){
  	connect(o,SIGNAL(clicked())
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QComboBox"){
  	connect(o,SIGNAL(currentIndexChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QSpinBox"){
  	connect(o,SIGNAL(valueChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QDoubleSpinBox"){
  	connect(o,SIGNAL(valueChanged(double))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QCheckBox"){
  	connect(o,SIGNAL(stateChanged(int))
  		,this,SLOT(onContentChanged()));
      }else if(isA=="QGroupBox"){
  	connect(o,SIGNAL(toggled(bool))
  		,this,SLOT(onContentChanged()));
      }else if(!(isA=="QTabWidget"||
 		 isA=="QFrame"||
 		 isA=="QWidget"||
 		 isA=="Oxygen::TransitionWidget"||
 		 isA=="Oxygen::SunkenFrameShadow"||
 		 isA=="UserTicWdg"||
 		 isA=="MyListView"||
 		 isA=="QStackedWidget" )){
	stop=true;
	myDebug()<<nsubfunc<<isA<<"not implemented";
      }
    }
    if(o->children().size()){
      connectChanged(o,exceptlist);
    }
  }

  if(stop){
    //    exit(1);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ProtoDlg::dataAcceptable(const QString&txt)
{ 
  //myDebug() << "ProtoDlg::dataAcceptable"<<txt;
  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }
  return true;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ProtoDlg::onpbHelpClicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg",phelpchapter);
  h->show();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ProtoDlg::onpbAcceptClicked()
{

  //myDebug() <<"STR ProtoDlg::on_pbAccept_clicked";
  if(dataAcceptable()){
    //myDebug() <<"here";
    emit talkBack(Accept);
    afterAcceptAction();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ProtoDlg::onpbCloseClicked()
{
  //myDebug() <<"STR ProtoDlg::on_pbUpdate_clicked";
  if(dataAcceptable()){
    //myDebug() <<"here";
    emit talkBack(Accept);
    afterAcceptAction();
    emit talkBack(End);
    return;
  }
}



/*EOF*/ 
