/*************************************************************************
 * @(#)SettingsDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   11 Mar 2011
 *************************************************************************
 */

#include "SettingsDlg.hh"
#include "Settings.hh"
#include "SettingsExternalWidget.hh"
#include "SettingsGraphicsWidget.hh"
#include "SettingsOGLWidget.hh"
#include "Config.hh"
#include "HelpDlg.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SettingsDlg::SettingsDlg(QWidget *parent,QWidget *base,const char *name)
  : BrainDialog(parent,base,name,false),currentsheet(0)
{
  setupUi(this);
  setWindowTitle(IDS_APP_NAME  + ": Settings");

  Sheet *currentsheet=new Sheet(this);
  sheetLayout->insertWidget(0,currentsheet);


  
  pgroups.append(Group("Graphics",new SettingsGraphicsWidget(this)));
  pgroups.append(Group("External",new SettingsExternalWidget(this)));
  pgroups.append(Group("OpenGL",new SettingsOGLWidget(this)));


  for(int i=0;i<pgroups.size();i++){
    QListWidgetItem *item=new QListWidgetItem();
    item->setText(pgroups.at(i).name);
    item->setData(Qt::UserRole,i);
    lwGroup->insertItem(i,item);
    connect(pgroups[i].sheet,SIGNAL(hasChanged())
 	    ,this,SLOT(onSheetHasChanged()));
  }

  setButtonState(false);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SettingsDlg::~SettingsDlg()
{
  if(currentsheet) delete currentsheet;
  foreach(const Group &g,pgroups){
    //the active gets deleted by the parent the other not
    if (g.sheet) delete g.sheet;
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::setSheet(QListWidgetItem *current)
{
  NSF("SettingsDlg::setSheet");
  MASSERT("Oops",current);
  int row=current->data(Qt::UserRole).toInt();
  //myDebug() << nsubfunc<<row;
  
  if(currentsheet) {
    currentsheet->setVisible(false);
  }
  sheetLayout->insertWidget(0,currentsheet=pgroups[row].sheet);
  currentsheet->setVisible(true);
  

  return;

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::on_lwGroup_currentItemChanged(QListWidgetItem *current
					,QListWidgetItem *)
{
  NSF("SettingsDlg::on_lwGroup_currentItemChanged");
  //myDebug() << nsubfunc;
  setSheet(current);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::on_pbClose_clicked()
{
  NSF("SettingsDlg::on_pbClose_clicked");
  //myDebug() <<nsubfunc;
  
  on_pbSave_clicked();
  BrainDialog::accept();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::on_pbSave_clicked()
{
  NSF("SettingsDlg::on_pbSave_clicked");
  //myDebug() <<nsubfunc;
  
  for(int i=0;i<pgroups.size();i++){
    pgroups[i].sheet->commit();
  }
  setButtonState(false);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::onSheetHasChanged()
{
  NSF("SettingsDlg::onSheetHasChanged");
  //myDebug() <<nsubfunc;
  setButtonState(true);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::setButtonState(bool saved)
{
  pbSave->setEnabled(saved);
  pbClose->setText((saved?"Save and &close":"&Close"));
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SettingsDlg::on_pbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","PreferencesDialog");
  h->show();
  //  h->deleteLater();
  
}

/*EOF*/ 
