/*************************************************************************
 * @(#)SpecialPointDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   08 Apr 2011
 *************************************************************************
 */

#include <QMessageBox>
#include <QColorDialog> 
#include <QFileDialog>
#include <QSignalMapper>
#include <QMenu>
#include <QItemEditorFactory>
#include "MyMath.hh"
#include "SpecialPointDlg.hh"
#include "Spsp.hh"
//#include "ColorButton.h"
#include "SpspModel.hh"
#include "Config.hh"
#include "ToolButtons.hh"
#include "Delegates.hh"
#include "ShortCuts.hh"
#include "Symmetry.hh"
#include "RealDlg.hh"
#include "OGLView.hh"
#include "OGLUserData.hh"
#include "Vlevel.hh"
#include "Valid.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
SpecialPointDlg::SpecialPointDlg(OGLView *oglview,QWidget *parent
				 ,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
  ,pspsp(0),spspmodel(0),mutetableadjust(false),pwaitforpick("")
  ,poglview(oglview),insertMenu(0),pickMenu(0),signalMapper(0)
  ,phaschanged(false)
{
//   QItemEditorFactory *factory = new QItemEditorFactory;
//   QItemEditorCreatorBase *colorListCreator =
//     new QStandardItemEditorCreator<ColorButton>();
//   factory->registerEditor(QVariant::UInt, colorListCreator);
//   QItemEditorFactory::setDefaultFactory(factory);

  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"SpecialPointDialog",": High symmetry points");
  

  pspsp=new SpecialSymPoints();
  spspmodel = new SpspModel(this,pspsp);
  tvSpsp->setModel(spspmodel);
  colordelegate=new ColorDelegate();
//   pointradiusdelegate=new DoubleSpinBoxDelegate();
//   pointradiusdelegate->setMinimum(0.001);
//   pointradiusdelegate->setMaximum(10);
//   pointradiusdelegate->setDecimals(3);
//   pointradiusdelegate->setSingleStep(0.005);

//   fontscaledelegate=new DoubleSpinBoxDelegate();
//   fontscaledelegate->setMinimum(0.001);
//   fontscaledelegate->setMaximum(10);
//   fontscaledelegate->setDecimals(3);
//   fontscaledelegate->setSingleStep(0.005);

  tvSpsp->setItemDelegateForColumn(SpspModel::PointColor,colordelegate);
  tvSpsp->setItemDelegateForColumn(SpspModel::TextColor,colordelegate);
  //tvSpsp->setItemDelegateForColumn(SpspModel::PointRadius,pointradiusdelegate);
  //tvSpsp->setItemDelegateForColumn(SpspModel::FontScale,fontscaledelegate);

  ptoolbuttons.createToolButtonsActions(tvSpsp,"high symmetry point",
			   tbInsert,tbDelete,tbUp,tbDown);

  signalMapper = new QSignalMapper(this);
  connect(signalMapper,SIGNAL(mapped(int)),
	  this,SLOT(slotInsertPoint(int)));

  QMenu *m=insertMenu=new QMenu(tbInsert);
  QAction *a;
  a=m->addAction("Insert &Point");  
  signalMapper->setMapping(a,Xf::SPTPoint);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  a=m->addAction("Insert &Jump");  
  signalMapper->setMapping(a,Xf::SPTJump);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  a=m->addAction("Insert &Break");  
  signalMapper->setMapping(a,Xf::SPTBreak);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));
  tbInsert->setMenu(insertMenu);


  ptoolbuttons.connectSlots(this,SLOT(slotInsertTriggerd()),
			    SLOT(slotDeletePoint()),
			    SLOT(slotMovePoint()),SLOT(slotMovePoint()));
  



  connect(spspmodel,SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&))
	  ,this,SLOT(onSpspModelDataChanged(const QModelIndex&,const QModelIndex&)));

  tbSetForAll->addAction(a=new QAction("point color",tbSetForAll));
  a->setData(SpspModel::PointColor);
  tbSetForAll->addAction(a=new QAction("point size",tbSetForAll));
  a->setData(SpspModel::PointRadius);
  tbSetForAll->addAction(a=new QAction("text color",tbSetForAll));
  a->setData(SpspModel::TextColor);
  tbSetForAll->addAction(a=new QAction("font scale",tbSetForAll));
  a->setData(SpspModel::FontScale);

  connect(tbSetForAll,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onSetForAllActionTriggered(QAction *)));

  new RatValidator(this,lePointSize,"point size",0.0001,1e10);
  new RatValidator(this,leFontScale,"text scale",0.0001,1e10);

  connect(oglview,SIGNAL(objectClicked(const String &,const OGLUserData*)),
 	  this,SLOT(onObjectClicked(const String &,const OGLUserData*)));



  m=pickMenu=new QMenu(tbPick);
  a=m->addAction("&Current");   
  a->setData("Current");
  a=m->addAction("&New");  
  a->setData("New");

  connect(m,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onTbPickAction(QAction *)));
  

  
  connect(poglview,SIGNAL(textShiftChanged(const String&,const Vec3&)),this
	  ,SLOT(onTextShiftChanged(const String&,const Vec3&)));

  connectChanged(this,QList<QObject*> ());

  connect(this,SIGNAL(contentChanged(QObject *)),this,SLOT(onObjectChanged(QObject *)));
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
SpecialPointDlg::~SpecialPointDlg()
{
  closeColorButtons(SpspModel::PointColor);
  closeColorButtons(SpspModel::TextColor);

  if(signalMapper) delete signalMapper;
  if(insertMenu) delete insertMenu; insertMenu=0;
  if(pickMenu) delete pickMenu; pickMenu=0;


  if(colordelegate) delete colordelegate;colordelegate=0;
//   if(pointradiusdelegate) delete pointradiusdelegate; pointradiusdelegate=0;
//   if(fontscaledelegate) delete fontscaledelegate; fontscaledelegate=0;
  if(pspsp) delete pspsp; pspsp=0;
  if(spspmodel) delete spspmodel; spspmodel=0;


  poglview->showPotentialSpecPoints(false);
  pwaitforpick="";

  poglview->setAllowedMoveObjects("","");

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::slotInsertTriggerd()
{
  NSF("SpecialPointDlg::slotInsertTriggerd()");
  //myDebug() <<nsubfunc;

  QPoint pos;
  pos=tbInsert->mapToGlobal(QPoint(0,tbInsert->height()));
  //    pos=QCursor::pos();

  tbInsert->menu()->popup(pos);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::closeColorButtons(int column)
{
  return ;
  for(int i=0;i<spspmodel->rowCount();i++)
    tvSpsp->closePersistentEditor(spspmodel->
				  index(i,column ,QModelIndex()));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::openColorButtons(int column)
{
  return;
  for(int i=0;i<spspmodel->rowCount();i++)
    tvSpsp->openPersistentEditor(spspmodel->
				  index(i,column ,QModelIndex()));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SpecialPointDlg::dataAcceptable()
{ 
  //myDebug() << "SpecialPointDlg::dataAcceptable";
  QString txt="";//wBoundDlg->dataAcceptable();
 
  ((RatValidator*)lePointSize->validator())->check(txt);
  ((RatValidator*)leFontScale->validator())->check(txt);

  
  spspmodel->validateData(txt);

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , txt, 1, 0,0 ) ;
    //myDebug() << "returning";
    return false;
  }
  return true;

}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::setData(SpecialSymPoints *spsp
			      ,SpecialSymPoints *defaultspsp
			      ,const Symmetry *sym)
{
  t_defaultspsp=defaultspsp;
  psym=sym;
  spspmodel->setSymmetry(psym);
  //  mutetableadjust=true;
  t_spsp=spsp;
  *pspsp=*spsp;
  pspsp->setChanged(false);
  
  cbActive->setChecked(pspsp->isVisible());
  cbUserDefined->setChecked(pspsp->userDefined());
  cbShowPath->setChecked(pspsp->showPath());
  tbPathColor->setColor(pspsp->pathColor());
  dspbxPathLineWidth->setValue(pspsp->pathLineWidth());
  
  cbCommonProperties->setChecked(pspsp->useCommonProperties());
  tbPointColor->setColor(pspsp->pointColor());
  tbTextColor->setColor(pspsp->textColor());
  leFontScale->setText(pspsp->fontScale().S());
  lePointSize->setText(pspsp->pointRadius().S());

  //  on_cbCommonProperties_stateChanged(pspsp->useCommonProperties());
  
  modelReload();

  tvSpsp->setCurrentIndex(spspmodel->index(0,0,QModelIndex()));

  adjustTvSpsp();
  //mutetableadjust=false;
  phaschanged=false;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_cbCommonProperties_stateChanged(int state)
{
  spspmodel->setColumnsEnabled(!state);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::modelReload()
{
  spspmodel->reload();
  openColorButtons(SpspModel::PointColor);
  openColorButtons(SpspModel::TextColor);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::refreshActive(bool on)
{
  cbActive->setChecked(on);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::adjustTvSpsp()
{
  int rows=spspmodel->rowCount();
  int cols=spspmodel->columnCount();
  for(int irow=0;irow<rows;irow++)
    tvSpsp->resizeRowToContents(irow);
  for(int icol=0;icol<cols;icol++){
    tvSpsp->resizeColumnToContents(icol);
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::getData(SpecialSymPoints *spsp)
{
  pspsp->setVisible(cbActive->isChecked());

  pspsp->setUserDefined(cbUserDefined->isChecked());
  pspsp->setShowPath(cbShowPath->isChecked());

  pspsp->setPathLineWidth(dspbxPathLineWidth->value());

  pspsp->setUseCommonProperties(cbCommonProperties->isChecked());

  QString t=lePointSize->text();IntValidator::stripAllWhiteSpace(t);
  pspsp->setPointRadius(t);
  t=leFontScale->text();IntValidator::stripAllWhiteSpace(t);
  pspsp->setFontScale(t);

  pspsp->setPathColor(tbPathColor->color());
  pspsp->setPointColor(tbPointColor->color());
  pspsp->setTextColor(tbTextColor->color());

  
  if(spsp)
    *spsp=*pspsp;
  
  pspsp->setChanged(false);

  adjustTvSpsp();

  phaschanged=false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_cbShowPath_stateChanged(int state)
{
  //poglview->setPathVisible(state);
  pspsp->setShowPath(state);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_cbUserDefined_stateChanged(int state)
{
  enableWidgets(state);
}
void SpecialPointDlg::on_cbActive_stateChanged(int state)
{
  cbUserDefined->setEnabled(state);
  fPath->setEnabled(state);
  enableWidgets(cbUserDefined->isChecked() && cbActive->isChecked());
}
void SpecialPointDlg::enableWidgets(bool on)
{
  frTable->setEnabled(on);
  pbLoadDefault->setEnabled(on);
  pbMove->setEnabled(on);
  frCommonProps->setEnabled(on);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::onSpspModelDataChanged(const QModelIndex&
					     ,const QModelIndex&)
{
  myDebug()<<"SpecialPointDlg::onSpspModelDataChanged";
  //  if(mutetableadjust) return;
  phaschanged=true;
  adjustTvSpsp();
  //tvSpsp->resizeRowsToContents();
  //tvSpsp->resizeColumnsToContents();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_pbLoadDefault_clicked()
{
  //myDebug()<<"SpecialPointDlg::on_pbLoadDefault_clicked";
  if(!psym) return;
  pspsp->calcPoints(psym,Vlevel::Silent);
  modelReload();
  adjustTvSpsp();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::onSetForAllActionTriggered(QAction *a)
{
  SpspModel::Columns action=((SpspModel::Columns)a->data().toInt());
  QVariant value;
  if(action==SpspModel::PointColor){
//     QColor c=QColorDialog::getColor(0xff0000,this,"");
//     if(c.isValid())
//       value=c.rgb()&0x00ffffff;
    value=tbPointColor->color();
  }else if(action==SpspModel::PointRadius){
    //value=RealDlg::getReal(0.025,this,"point size");
    value=lePointSize->text();
  }else if(action==SpspModel::TextColor){
//     QColor c=QColorDialog::getColor(0x0000ff,this,"");
//     if(c.isValid())
//       value=c.rgb()&0x00ffffff;
    value=tbTextColor->color();
  }else if(action==SpspModel::FontScale){
    //value=RealDlg::getReal(0.1,this,"font scale");
    value=leFontScale->text();
  }
  //myDebug("%x",value.toUInt());
  //myDebug("%x",color);
  spspmodel->setDefaultForColumn(action,value);
//   if(
//      action==SpspModel::PointColor||
//      action==SpspModel::TextColor
//      ){
//     for(int i=0;i<spspmodel->rowCount();i++){
//       ColorButton *cbut=
// 	qobject_cast<ColorButton*>(tvSpsp->indexWidget(spspmodel->
// 						       index(i,action
// 							     ,QModelIndex())));
//       cbut->setColor(value.toUInt());
//     }
//   }
	

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::slotInsertPoint(int m)
{
  NSF("SpecialPointDlg::slotInsertPoint"); 

  Xf::SpecialPointType mode=(Xf::SpecialPointType)m;

  QModelIndex index=tvSpsp->currentIndex();
  int row=0;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
    col=index.column();
  }

  
  spspmodel->insertRow(row,mode);
  tvSpsp->scrollTo(spspmodel->index(row,0,QModelIndex()));
  tvSpsp->setCurrentIndex(spspmodel->index(row,col,QModelIndex()));
  adjustTvSpsp();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::slotDeletePoint()
{
  NSF("SpecialPointDlg::slotDeletePoint"); 
  //  myDebug() << nsubfunc ;

  int numrows=spspmodel->rowCount();

  if(numrows<=0) return;

  QModelIndex currentindex=tvSpsp->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select a point first."
			   , 1, 0,0 );
    return;
  }

  int row=currentindex.row();
  int col=currentindex.column();


  spspmodel->removeRow(row);
  tvSpsp->setCurrentIndex(spspmodel->
			  index(MyMath::min<long>(row,numrows-2),col,QModelIndex()));
  adjustTvSpsp();

  return;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::slotMovePoint()
{
  NSF("SpecialPointDlg::slotMovePoint"); Q_UNUSED(nsubfunc);

  QModelIndex currentindex=tvSpsp->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvSpsp->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select an atom first."
			   , 1, 0,0 );
    return;
  }

  int row=currentindex.row();
  int col=currentindex.column();
  int by=-1;
  if(sender()==ptoolbuttons.pmoveDownAction) by=1;

  if(spspmodel->moveRow(row,by)){
    tvSpsp->setCurrentIndex(spspmodel->index(row+by,col,QModelIndex()));
    adjustTvSpsp();
  }


  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_tvSpsp_clicked(const QModelIndex &index)
{
  //  myDebug()<<"SpecialPointDlg::on_tvSpsp_clicked";
  if(!index.isValid()) return;
  
  if(!(index.flags()&Qt::ItemIsEnabled))return;
  if((index.column()!=SpspModel::PointColor)&&
     (index.column()!=SpspModel::TextColor))return;
  //  return;
  //myDebug()<<"SpecialPointDlg::on_tvSpsp_clicked hier";
  tvSpsp->edit(index);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::onTbPickAction(QAction *a)
{
  //myDebug() <<"SpecialPointDlg::onTbPickAction"<<a->data();
  
  tbPick->setCheckable(true);
  tbPick->setChecked(true);
  pwaitforpick=a->data().toString();
  if(!pwaitforpick.isEmpty()){
    poglview->setMoveBlockingClick(true);
    poglview->showPotentialSpecPoints(true);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_tbPick_clicked()
{
  if(!tbPick->isCheckable()){
    QPoint pos;
    pos=tbPick->mapToGlobal(QPoint(0,tbPick->height()));
    pickMenu->popup(pos);
    return;
  }

  tbPick->setCheckable(false);
  tbPick->setChecked(false);
  poglview->setMoveBlockingClick(false);
  poglview->showPotentialSpecPoints(false);
  pwaitforpick="";
  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::onObjectClicked(const String &what
				      ,const OGLUserData*udata)
{
  NSF("SpecialPointDlg::onObjectClicked(const QString &)");
  if(pwaitforpick.isEmpty()) return;
  if(!what.startsWith("potential special point dots")) return;

  Vec3 v=udata->toVec3();
  myDebug() << nsubfunc << what<<v;
  poglview->showPotentialSpecPoints(false);

  tbPick->setCheckable(false);
  tbPick->setChecked(false);
  QModelIndex index=tvSpsp->currentIndex();
  int row=-1;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row();
    col=index.column();
  }
  if(pwaitforpick=="New" || row<0 || row>=spspmodel->rowCount()){
    //row=spspmodel->rowCount();
    row++;
    spspmodel->insertRow(row,Xf::SPTPoint,new SpecialPoint("new",v[0],v[1],v[2]));
    tvSpsp->scrollTo(spspmodel->index(row,0,QModelIndex()));
    tvSpsp->setCurrentIndex(spspmodel->index(row,col,QModelIndex()));
  }else if(pwaitforpick=="Current"){
    spspmodel->setPoint(row,v);
  }
  adjustTvSpsp();


  pwaitforpick="";

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_pbExport_clicked()
{
  QString f0=QDir::currentPath()+"/=.in";
  
  QString fileName = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"FPLO input (=.*);;All files (*)");
  try{
    if(cbUserDefined->isChecked())
      pspsp->exportToInFile(fileName,psym);
    else
      t_defaultspsp->exportToInFile(fileName,psym);
  }catch(Except &ex){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , ex.msg(), 1, 0,0 ) ;

  }

//   modelReload();
//   adjustTvSpsp();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_pbImport_clicked()
{
  QString f0=QDir::currentPath()+"/=.in";
  
  QString fileName = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"FPLO input (=.*);;All files (*)");
  try{
    pspsp->importFromInFile(fileName,psym);
  }catch(Except &ex){
    QMessageBox::warning ( this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
			   , ex.msg(), 1, 0,0 ) ;

  }

  modelReload();
  adjustTvSpsp();

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::onTextShiftChanged(const String &id,const Vec3 &v)
{
  int ip=QString(id()).remove("special point label").toInt();
  myDebug() << ip<<v;

  SReal3 s;
  s=pspsp->at(ip)->shift();
  s[0]=v[0];
  s[1]=v[1];
  (*t_spsp)[ip]->setShift(s);
  (*pspsp)[ip]->setShift(s);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpecialPointDlg::on_pbMove_clicked()
{
  if(pbMove->isChecked()){
    if(phaschanged){
      long ret=QMessageBox::warning(this,IDS_APP_NAME+ " " + FPLOMAINVERSION()
				    ,"You have to accept changes first?"
				    ,QMessageBox::Ok|QMessageBox::Default
				    ,QMessageBox::Cancel
				    );
      if(ret!=QMessageBox::Ok){
	pbMove->setChecked(false);
	return;
      }
      if(!dataAcceptable()) {
	pbMove->setChecked(false);
	return;
      }
      emit talkBack(Accept);
      
    }

    cbUserDefined->setEnabled(false);
    cbActive->setEnabled(false);
    fPath->setEnabled(false);
    frTable->setEnabled(false);
    pbAccept->setEnabled(false);
    pbClose->setEnabled(false);
    pbLoadDefault->setEnabled(false);
    pbExport->setEnabled(false);
    frCommonProps->setEnabled(false);
    poglview->setAllowedMoveObjects("OGLText","special point label");
  }else{
    cbUserDefined->setEnabled(true);
    cbActive->setEnabled(true);
    fPath->setEnabled(true);
    frTable->setEnabled(true);
    pbAccept->setEnabled(true);
    pbClose->setEnabled(true);
    pbLoadDefault->setEnabled(true);
    pbExport->setEnabled(true);
    frCommonProps->setEnabled(true);
    poglview->setAllowedMoveObjects("","");
  }
}


/*EOF*/ 
