/* @(#)SpecialPointDlg.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SPECIALPOINTDLG_H
#define _SPECIALPOINTDLG_H 1

#include "ui_SpecialPointDlg.h"
#include "ToolButtons.hh"
#include "Xf.hh"

class QAction;
class SpecialSymPoints;
class SpspModel;
class QSignalMapper;
class ColorDelegate;
class QModelIndex;
namespace XFPLO{
  class Symmetry;
}
class OGLView;
class OGLUserData;
class QMenu;
class Vec3;
class String;

using namespace XFPLO;

class SpecialPointDlg : public ProtoDlg, private Ui::SpecialPointDlg
{
  Q_OBJECT
public:
  SpecialPointDlg(OGLView *oglview,QWidget *parent,QWidget *base,const char *name);
  ~SpecialPointDlg();

  void setData(SpecialSymPoints *
	       ,SpecialSymPoints *defaultspsp,const Symmetry *sym);
  void getData(SpecialSymPoints *);

  void refreshActive(bool on);

private:
  bool dataAcceptable();
  void adjustTvSpsp();
  void enableWidgets(bool on);
  void closeColorButtons(int column);
  void openColorButtons(int column);
  void modelReload();

private slots:
  void onObjectChanged(QObject *){phaschanged=true;};
  void on_pbMove_clicked();
  void on_tbPick_clicked();
  void onTbPickAction(QAction *);
  void on_cbActive_stateChanged(int);
  void on_cbUserDefined_stateChanged(int);
  void on_pbLoadDefault_clicked();
  void on_cbShowPath_stateChanged(int state);
  void onSpspModelDataChanged(const QModelIndex&,const QModelIndex&);
  void onSetForAllActionTriggered(QAction *);
  void on_cbCommonProperties_stateChanged(int);
  void on_tvSpsp_clicked(const QModelIndex &index);
  void on_pbExport_clicked();
  void on_pbImport_clicked();
  void slotInsertPoint(int);
  void slotDeletePoint();
  void slotMovePoint();
  void slotInsertTriggerd();
  void onObjectClicked(const String &what,const OGLUserData*);
  void onTextShiftChanged(const String&,const Vec3&);

private:
  SpecialSymPoints *pspsp,*t_spsp,*t_defaultspsp;
  SpspModel *spspmodel;
  ColorDelegate *colordelegate;
  bool mutetableadjust;
  QString pwaitforpick;
  const Symmetry *psym;
  OGLView *poglview;
  QMenu *insertMenu,*pickMenu;
  QSignalMapper *signalMapper;
  bool phaschanged;
  ToolButtons ptoolbuttons;
};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SPECIALPOINTDLG_H */

