/*************************************************************************
 * @(#)SubGroupGenDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   19 Aug 2009
 *************************************************************************
 */

#include "SubGroupGenDlg.hh"


SubGroupGenDlg::SubGroupGenDlg(QWidget *parent)
  : QDialog(parent)
{
  setupUi(this);

  connect(lwOps,SIGNAL(itemChanged(QListWidgetItem *))
	  ,this,SLOT(onItemChanged(QListWidgetItem *)));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubGroupGenDlg::setData(const ValueArray<Operation> *ops
			     ,const ValueArray<int> subgroupgenerators)
{
  //myDebug() << "SubGroupGenDlg::setData";

  pops=ops;

  Qt::ItemFlags flags= Qt::ItemIsSelectable|
    Qt::ItemIsEnabled|
    Qt::ItemIsUserCheckable;
  
  QListWidgetItem *lwi;
  QString s="";
  for(int i=0;i<ops->size();i++){
    lwi=new QListWidgetItem( Sprintf("%- 12s: %s"
				     ,ops->at(i).CSSymbol()()
				     ,ops->at(i).XYZSymbol()())()
			     ,lwOps);
    lwi->setFlags(flags);
    lwi->setCheckState(Qt::Unchecked);
    lwi->setData(Qt::UserRole,ops->at(i).index());
    for(int j=0;j<subgroupgenerators.size();j++){
      if(subgroupgenerators.at(j)==ops->at(i).index()){
	lwi->setCheckState(Qt::Checked);
	if(s.isEmpty())
	  s+=ops->at(i).CSSymbol()();
	else
	  s+=QString(" , ")+ops->at(i).CSSymbol()();
	break;
      }
    }

    lwOps->insertItem(i,lwi);
  }
  if(s.isEmpty()) s="full group";
  leSelected->setText(s);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubGroupGenDlg::getData(ValueArray<int> &subgroupgenerators)
{
  subgroupgenerators.clear();
  for(int i=0;i<lwOps->count();i++){
    QListWidgetItem *lwi=lwOps->item(i);
    if(lwi->checkState()==Qt::Checked){
      int index=lwi->data(Qt::UserRole).toInt();
      if(!subgroupgenerators.contains(index))
	subgroupgenerators.append(index);
    }
  }

  //myDebug() << subgroupgenerators;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubGroupGenDlg::accept()
{
  QDialog::accept();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubGroupGenDlg::on_pbUnset_clicked()
{
  //myDebug() << "SubGroupGenDlg::on_pbUnset_clicked";
  for(int i=0;i<lwOps->count();i++){
    QListWidgetItem *lwi=lwOps->item(i);
    lwi->setCheckState(Qt::Unchecked);
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SubGroupGenDlg::onItemChanged(QListWidgetItem *)
{
  //myDebug() << "SubGroupGenDlg::onItemChanged";

  QString s="";
  for(int i=0;i<lwOps->count();i++){
    QListWidgetItem *lwi=lwOps->item(i);
    if(lwi->checkState()==Qt::Checked){
      if(s.isEmpty())
	s+=pops->at(i).CSSymbol()();
      else
	s+=QString(" , ")+pops->at(i).CSSymbol()();
    }
  }
  if(s.isEmpty()) s="full group";
  leSelected->setText(s);
  
}
/*EOF*/ 
