/*************************************************************************
 * @(#)SurColDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QMessageBox> 
#include <QCloseEvent> 
#include <math.h> 
#include <stdio.h> 
#include "SurColDlg.hh"
#include "ColorModelEditor.hh"
#include "ColorModel.hh"
#include "Valid.hh"
#include "Config.hh"
#include "Vunit.hh"
#include "MyDebug.hh"

#define validcolor(i) MyMath::min<long>(MyMath::max<long>((i),0),255)

/* ***********************************************************************
 *
 * ***********************************************************************
 */

SurColDlg::SurColDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name)
{
  //NSF("SurColDlg::SurColDlg");
  //CPR("");

  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,0,"SurColDialog",": Surface coloring");


  new RealValidator(this,leBlue,"blue boundary",-1.e10,1.e10);
  new RealValidator(this,leRed ,"red boundary",-1.e10,1.e10);
  new RealValidator(this,leBluee,"extern blue boundary",-1.e10,1.e10);
  new RealValidator(this,leRede,"extern red boundary",-1.e10,1.e10);


  new IntValidator(this,leFactor,"unit factor",-1000000,1000000);
  new IntValidator(this,leFactore,"extern unit factor",-1000000,1000000);


  cbxUnit->clear();
  for(long i=0;i<VUnits::numberOfUnits();i++){
    cbxUnit->insertItem(i,VUnits::velocityUnit(VUnits::vunitFromIndex(i)));
  }



  //leBlue->selectAll();
  //leBlue->selectAll();


  connect(cbExtCol,SIGNAL(clicked()),this,SLOT(setEnable()));

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::setData( const IsoColor &isocolor , long excol
			 , double vmin, double vmax , double vmine
			 , double vmaxe , long def, long defe 
			 , double *dmin, double *dmax , double *edmin
			 , double *edmax ,VUnits::VelUnits vunit,long ufactor
			 ,long ufactore
			 ,long iweight
			 ,const FSDoc::LegendControl &legendcontrol)
{
  t_isocolor=isocolor;
  t_isocolor.setChanged(false);
  t_colormap=t_isocolor.colorModel()->colorMap();
  t_def=def;
  t_defe=defe;
  t_excol=excol;
  t_iweight=iweight;
  t_legcontrol=legendcontrol;

  bOutCol->setColor(t_isocolor.outerColor());
  bInCol->setColor(t_isocolor.innerColor());
  dspbxOpacity->setValue(qAlpha(t_isocolor.outerColor())/255.);
  rbOutCol->setChecked(!t_isocolor.outerFromData());
  rbOutVel->setChecked(t_isocolor.outerFromData());

  rbInCol->setChecked(!t_isocolor.innerFromData());
  rbInVel->setChecked(t_isocolor.innerFromData());

  // this is a work around, since it seems Qstring.sprintf is buggy
  char buf[5000];
  sprintf(buf,"%g",vmin ); t_min .sprintf("%s",buf);
  sprintf(buf,"%g",vmax ); t_max .sprintf("%s",buf); 
  sprintf(buf,"%g",vmine); t_mine.sprintf("%s",buf);
  sprintf(buf,"%g",vmaxe); t_maxe.sprintf("%s",buf);
  //t_min=QString().setNum(vmin,'g',10);
  //t_max=QString().setNum(vmax,'g',10);
  //t_mine=QString().setNum(vmine,'g',10);
  //t_maxe=QString().setNum(vmaxe,'g',10);

  leBlue->setText(t_min);
  leBluee->setText(t_mine);
  leRed->setText(t_max);
  leRede->setText(t_maxe);
  
  spbxState->setValue(iweight);

  leFactor ->setText(QString().setNum(ufactor));
  t_ufactor=ufactor;

  leFactore ->setText(QString().setNum(ufactore));
  t_ufactore=ufactore;

  

  cbDef->setChecked(def);
  cbDefe->setChecked(defe);
  cbExtCol->setChecked(excol);
  cbShowUnits->setChecked(legendcontrol.showunits);
  cbMinLabel->setChecked(legendcontrol.useminlabel);
  leMinLabel->setText(legendcontrol.minlabel());
  cbMaxLabel->setChecked(legendcontrol.usemaxlabel);
  leMaxLabel->setText(legendcontrol.maxlabel());
  cbTitle->setChecked(legendcontrol.usetitle);
  leTitle->setText(legendcontrol.title());

  printf("dmin=%g dmax=%g \n",*dmin,*dmax);
  t_dmin=dmin;
  t_dmax=dmax;
  t_edmin=edmin;
  t_edmax=edmax;

  t_vunit=vunit;
  cbxUnit->setCurrentIndex(VUnits::indexFromVunit(vunit));


    
  setEnable();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::getData( IsoColor &isocolor , long &excol , double &min
			 , double &max, double &mine, double &maxe
			 , long &def, long &defe,VUnits::VelUnits &vunit
			 ,long &ufactor
			 ,long &ufactore 
			 ,long &iweight
			 ,FSDoc::LegendControl &legendcontrol
			 , long &chn, long &excolchn)
{
  chn=false;

  unsigned long a=((unsigned long)(dspbxOpacity->value()*255))*0x1000000;
  isocolor.setOuterColor((bOutCol->color()&0xffffff)|a);
  isocolor.setInnerColor(bInCol->color());
  isocolor.setOuterFromData(rbOutVel->isChecked());
  isocolor.setInnerFromData(rbInVel->isChecked());
  isocolor.colorModel()->colorMap()=t_colormap;
  isocolor.setChanged(true);//todo: change logic of colormap is awful
  


  def=cbDef->isChecked();
  chn |= ( def != t_def ) ;

  t_def=def;

  defe=cbDefe->isChecked();
  chn |= ( defe != t_defe ) ;

  t_defe=defe;

  excol=cbExtCol->isChecked();
  chn |= ( excol != t_excol ) ;
  excolchn = ( excol != t_excol ) ;

  t_excol=excol;

  legendcontrol.showunits=cbShowUnits->isChecked();
  chn |= ( legendcontrol.showunits != t_legcontrol.showunits ) ;
  excolchn = ( legendcontrol.showunits != t_legcontrol.showunits ) ;

  legendcontrol.useminlabel=cbMinLabel->isChecked();
  chn |= ( legendcontrol.useminlabel != t_legcontrol.useminlabel ) ;
  excolchn = ( legendcontrol.useminlabel != t_legcontrol.useminlabel ) ;

  legendcontrol.usemaxlabel=cbMaxLabel->isChecked();
  chn |= ( legendcontrol.usemaxlabel != t_legcontrol.usemaxlabel ) ;
  excolchn = ( legendcontrol.usemaxlabel != t_legcontrol.usemaxlabel ) ;

  legendcontrol.usetitle=cbTitle->isChecked();
  chn |= ( legendcontrol.usetitle != t_legcontrol.usetitle ) ;
  excolchn = ( legendcontrol.usetitle != t_legcontrol.usetitle ) ;

  legendcontrol.minlabel=leMinLabel->text();
  chn |= ( legendcontrol.minlabel != t_legcontrol.minlabel ) ;
  excolchn = ( legendcontrol.minlabel != t_legcontrol.minlabel ) ;

  legendcontrol.maxlabel=leMaxLabel->text();
  chn |= ( legendcontrol.maxlabel != t_legcontrol.maxlabel ) ;
  excolchn = ( legendcontrol.maxlabel != t_legcontrol.maxlabel ) ;

  legendcontrol.title=leTitle->text();
  chn |= ( legendcontrol.title != t_legcontrol.title ) ;
  excolchn = ( legendcontrol.title != t_legcontrol.title ) ;


  t_legcontrol=legendcontrol;

  bool ok;
  QString t=leBlue->text();
  chn |= ( t != t_min ) ;

  t_min=t;
  IntValidator::stripAllWhiteSpace(t);
  min=t.toDouble(&ok);

  t=leBluee->text();
  chn |= ( t != t_mine ) ;


  t_mine=t;
  IntValidator::stripAllWhiteSpace(t);
  mine=t.toDouble(&ok);

  t=leRed->text();
  chn |= ( t != t_max ) ;


  t_max=t;
  IntValidator::stripAllWhiteSpace(t);
  max=t.toDouble(&ok);

  t=leRede->text();
  chn |= ( t != t_maxe ) ;

  t_maxe=t;
  IntValidator::stripAllWhiteSpace(t);
  maxe=t.toDouble(&ok);


  
  
  long i=spbxState->value();
  excolchn = ( i != iweight ) ;
  chn |= ( i != iweight ) ;
  t_iweight=i;
  iweight=i;
  

  t=leFactor->text();
  IntValidator::stripAllWhiteSpace(t);
  ufactor=t.toLong(&ok);
  chn |= ( ufactor != t_ufactor ) ;

  t_ufactor=ufactor;


  t=leFactore->text();
  IntValidator::stripAllWhiteSpace(t);
  ufactore=t.toLong(&ok);
  chn |= ( ufactore != t_ufactore ) ;

  t_ufactore=ufactore;
    
  vunit=VUnits::vunitFromIndex(cbxUnit->currentIndex());
  chn|=( vunit != t_vunit );

  t_vunit=vunit;

  t_isocolor=isocolor;
  chn|= ( isocolor.changed() ) ;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SurColDlg::dataAcceptable()
{
  //int i;

  QString s,txt="";
  ((IntValidator*)leBlue->validator())->check(txt);
  ((IntValidator*)leBluee->validator())->check(txt);
  ((IntValidator*)leRed->validator())->check(txt);
  ((IntValidator*)leRede->validator())->check(txt);
  ((IntValidator*)leFactor->validator())->check(txt);
  ((IntValidator*)leFactore->validator())->check(txt);

  bool ok;
  double min,max,mine,maxe;
  QString t=leBlue->text();
  IntValidator::stripAllWhiteSpace(t);
  min=t.toDouble(&ok);

  t=leBluee->text();
  IntValidator::stripAllWhiteSpace(t);
  mine=t.toDouble(&ok);

  t=leRed->text();
  IntValidator::stripAllWhiteSpace(t);
  max=t.toDouble(&ok);

  t=leRede->text();
  IntValidator::stripAllWhiteSpace(t);
  maxe=t.toDouble(&ok);
  if(min>=max)
    txt+="\nBlue must be less than red!\n ";
  if(mine>=maxe)
    txt+="\nExtern blue must be less than extern red!\n ";
  

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
    return false;
  }else{
    return true;
  }
  return true;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::setToDefault()
{
  QString t;
  bool ok;

  t=leFactor->text();
  IntValidator::stripAllWhiteSpace(t);
  long ufac=t.toLong(&ok);

  double f=VUnits::
    velocityUnitFactor(VUnits::vunitFromIndex(cbxUnit->currentIndex()));
  double fac=pow(10.,-ufac)*f;
  t=QString().setNum(*t_dmin*fac,'g',10);
  leBlue->setText(t);
  t=QString().setNum(*t_dmax*fac,'g',10);
  leRed->setText(t);


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::setToDefaulte()
{
  QString t;
  bool ok;

  t=leFactore->text();
  IntValidator::stripAllWhiteSpace(t);
  long ufac=t.toLong(&ok);
  
  double fac=pow(10.,-ufac);

  t=QString().setNum(*t_edmin*fac,'g',10);
  leBluee->setText(t);
  t=QString().setNum(*t_edmax*fac,'g',10);
  leRede->setText(t);


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::setEnable()
{
  QString t;
  long c=cbExtCol->isChecked();
  
  
  cbDef->setEnabled(!c);
  pbDef->setEnabled(!c);
  leBlue->setEnabled(!c);
  leRed->setEnabled(!c);
  leFactor->setEnabled(!c);
  cbxUnit->setEnabled(!c);

  cbDefe->setEnabled(c);
  bDefe->setEnabled(c);
  leBluee->setEnabled(c);
  leRede->setEnabled(c);
  leFactore->setEnabled(c);

  spbxState->setEnabled(c);

  if(!c){
    // we have no extern coloring here, first group is active
    long ch=cbDef->isChecked();
    leBlue->setEnabled(!ch);
    leRed->setEnabled(!ch);

    
  }else{
    long ch=cbDefe->isChecked();

    leBluee->setEnabled(!ch);
    leRede->setEnabled(!ch);

  }



}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SurColDlg::on_pbColorModel_clicked()
{
  //myDebug()<<"here";
  ColorModelEditor *cme
    =new ColorModelEditor(this,this->mainWidget(),"ColorModelEditor");
  cme->setData(t_colormap);
  int ret=cme->exec();
  if(ret==QDialog::Accepted){
    cme->getData(t_colormap);
  }
  
}

/*EOF*/ 
