/*************************************************************************
 * @(#)SymDlg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QFileDialog> 
#include <QMessageBox> 
#include <QMenu> 
#include <QDebug> 
#include "ToolButtons.hh"
#include "Parser.hh"
#include "Assorted.hh"
#include "fsymmetry.h"
#include "MyMath.hh"
#include "SetTypeDlg.hh"
#include "SReal.hh"
#include "FploDlg.hh"
#include "SubGroupGenDlg.hh"
#include "Symmetry.hh"
#include "AtomPropsModel.hh"
#include "WyckPosModel.hh"
#include "IconCache.hh"
#include "Config.hh"
#include "Delegates.hh"
#include "HelpDlg.hh"
#include "Fplo.hh"
#include "Valid.hh"
#include "SymDlg.hh"
#include "ToolButtons.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
SymDlg::SymDlg(QWidget *parent,QWidget *base,const char *name)
  : ProtoDlg(parent,base,name),t_sym(0),psym(0),model(0),propmodel(0),
    mutespbxgroup(false),ptvWyckRowSizeKnown(false)
{
  setupUi(this);
  setUp(pbAccept,pbClose,pbCancel,pbHelp,"SymmetryDialog",
	": Symmetry settings (for empty lattice calc.)");


  //  setWindowModality ( Qt::WindowModal);

  t_sym=new Symmetry();

  t_currentrepindex=0;

  //  new IntValidator(this,leGroup,"The spacegroup number",1,230);

  new RealValidator(this,leAlat,"The first lattice constant",1.e-10,1.e10);
  new RealValidator(this,leBlat,"The second lattice constant",1.e-10,1.e10);
  new RealValidator(this,leClat,"The third lattice constant",1.e-10,1.e10);

  new RealValidator(this,leAlpha,"The first axis angle",1.e-10,179.999999);
  new RealValidator(this,leBeta ,"The second axis angle",1.e-10,179.999999);
  new RealValidator(this,leGamma,"The third axis angle",1.e-10,179.999999);





  spbxGroup->setFocus();


  for(int i=0;i<numberofcrystalsystem();i++){
    cmbCrysSystem->insertItem(i,QString()
			 .sprintf("%-12s",
				  crystalsystems_longname(i)));
  }


  ptoolbuttons=new ToolButtons();
  ptoolbuttons->createToolButtonsActions(tvWyck,"atom"
					,tbInsert,tbDelete,tbUp,tbDown);

  ptoolbuttons->connectSlots(this,SLOT(slotTvWyckInsertPressed()),
			     SLOT(slotTvWyckDeletePressed()),
			     SLOT(slotTvWyckMovePressed()),
			     SLOT(slotTvWyckMovePressed()));


  elvalid=new ElementValidator(this,0,"Ele.");
  typevalid=new IntValidator(this,0,"Type",0,1000000);
  xyzvalid=new RatValidator(this,0,"x,y,z",-1e10,1e10);

  //eldelegate=new LineEditDelegate(this,elvalid);
  //xyzdelegate=new LineEditDelegate(this,xyzvalid);

  //  tvWyck->setItemDelegateForColumn ( 0, eldelegate );
  //  tvWyck->setItemDelegateForColumn ( 1, xyzdelegate );
  //  tvWyck->setItemDelegateForColumn ( 2, xyzdelegate );
  //  tvWyck->setItemDelegateForColumn ( 3, xyzdelegate );

  model = new WyckPosModel(this,t_sym);
  propmodel = new AtomPropsModel(this,0,t_sym);
  tvWyck->setModel(model);
  tvWyckProps->setModel(propmodel);
  connect(tvWyck,SIGNAL(commitHappening(bool)),
	  model,SLOT(userCommitHappening(bool)));

  colordelegate=new ColorDelegate();
  speccolordelegate=new ColorDelegate();
  radtypdelegate=new ComboDelegate(0,&AtomicRadii::radiinames);
  radiusdelegate=new DoubleSpinBoxDelegate(0);
  radiusdelegate->setMinimum(0.01);
  radiusdelegate->setMaximum(1000000.);
  radiusdelegate->setSingleStep(0.2);
  radiusdelegate->setDecimals(2);

  shineninessdelegate=new DoubleSpinBoxDelegate(0);
  shineninessdelegate->setMinimum(0.0);
  shineninessdelegate->setMaximum(128.0);
  shineninessdelegate->setSingleStep(5);
  shineninessdelegate->setDecimals(0);



  tvWyckProps->setItemDelegateForColumn(AtomPropsModel::AtomColor,colordelegate);
  tvWyckProps->setItemDelegateForColumn(AtomPropsModel::SpecularColor,speccolordelegate);
  tvWyckProps->setItemDelegateForColumn(AtomPropsModel::RadiusType,radtypdelegate);
  tvWyckProps->setItemDelegateForColumn(AtomPropsModel::Radius,radiusdelegate);
  tvWyckProps->setItemDelegateForColumn(AtomPropsModel::AtomShininess,shineninessdelegate);

  QAction *a;
  tbSetDefault->addAction(a=new QAction("Type by &Z",tbSetDefault));
  a->setData(WyckPosModel::TypeByZ);
  tbSetDefault->addAction(a=new QAction("Type per &element",tbSetDefault));
  a->setData(WyckPosModel::TypePerElement);


  connect(tbSetDefault,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onDefaultActionTriggered(QAction *)));


  tbSetDefaultProps->addAction(a=new QAction("&from theme",tbSetDefaultProps));
  a->setData(AtomPropsModel::ColorByZ);
  themeMenu=new QMenu(0);
  a->setMenu(themeMenu);

  connect(themeMenu,SIGNAL(aboutToShow()),this,SLOT(onThemeMenuAboutToShow()));
  connect(themeMenu,SIGNAL(triggered(QAction *)),this
	  ,SLOT(onThemeMenuAction(QAction *)));

  pbManipulate->hide();
  wManipulate->hide();
  wManipulate->setEnabled(false);
  //pbManipulate->hide();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::onDefaultActionTriggered(QAction *a)
{
  WyckPosModel::SetDefaultsActions action
    =(WyckPosModel::SetDefaultsActions)a->data().toInt();
  if(action==WyckPosModel::TypeByZ){
    model->setDefaults(action,QMap<QString,int>());
  }else if(action==WyckPosModel::TypePerElement){
    SetTypeDlg *dlg=new SetTypeDlg(this,model->getElementTypeList());
    long ret=dlg->exec();
    if(ret){
      model->setDefaults(action,dlg->getData());
    }
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::onThemeMenuAboutToShow()
{
  StringList sl=globalproperties->themes();
  themeMenu->clear();
  for(int i=0;i<sl.size();i++){
    QAction *a=themeMenu->addAction(sl.at(i)());
    a->setData(sl.at(i)());
  }
  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::onThemeMenuAction(QAction *a)
{
  QString theme=a->data().toString();
  
  AtomPropsModel::SetDefaultsActions action=AtomPropsModel::ColorByZ;
  
  propmodel->setDefaults(action,theme);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::slotTvWyckInsertPressed()
{
  NSF("SymDlg::slotTvWyckInsertPressed"); Q_UNUSED(nsubfunc);

//   if(!tvWyck->hasFocus()){
//     QMessageBox::warning ( this,IDS_APP_NAME, "Select an atom first."
// 			   , 1, 0,0 );
//     return;
//   }



  QModelIndex index=tvWyck->currentIndex();
  int row=0;
  int col=0;
  if(index !=  QModelIndex()){
    row=index.row()+1;
    col=index.column();
  }

  
  model->insertRow(row);
  tvWyck->scrollTo(model->index(row,0,QModelIndex()));
  tvWyck->setCurrentIndex(model->index(row,col,QModelIndex()));
  propmodel->reload();
  adjustTvWyck();
  adjustTvProps();

  
    
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::slotTvWyckDeletePressed()
{
  NSF("SymDlg::slotTvWyckDeletePressed"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc ;

  QModelIndex currentindex=tvWyck->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvWyck->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select an atom first."
			   , 1, 0,0 );
    return;
  }

  int numrows=model->rowCount();
  int row=currentindex.row();
  int col=currentindex.column();


  model->removeRow(row);
  tvWyck->setCurrentIndex(model->index(MyMath::min<long>(row,numrows-2),col,QModelIndex()));
  propmodel->reload();
  adjustTvWyck();
  adjustTvProps();
  return;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::slotTvWyckMovePressed()
{
  NSF("SymDlg::slotTvWyckMovePressed"); Q_UNUSED(nsubfunc);

  QModelIndex currentindex=tvWyck->currentIndex();
  if(currentindex ==  QModelIndex() 
     //|| (!tvWyck->hasFocus())
     ){
    QMessageBox::warning ( this,IDS_APP_NAME, "Select an atom first."
			   , 1, 0,0 );
    return;
  }

  int row=currentindex.row();
  int col=currentindex.column();
  int by=-1;
  if(sender()==ptoolbuttons->pmoveDownAction) by=1;

  if(model->moveRow(row,by)){
    tvWyck->setCurrentIndex(model->index(row+by,col,QModelIndex()));
    adjustTvWyck();
  }

  propmodel->reload();
  adjustTvProps();

  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_cmbCrysSystem_currentIndexChanged(int csystem)
{
  //myDebug() <<"cs" <<csystem << "bt" << numberofbravaistypesofcryssystem(csystem);

  cmbBravais->clear();
  if(isCrystal()){
    
    for(int i=0;i<numberofbravaistypesofcryssystem(csystem);i++){
      int btype=bravaistypesofcryssystem(csystem,i);
      cmbBravais->insertItem(i,QString()
			     .sprintf("%-12s",
				      bravaistypes_longname(btype)),QVariant(btype));
    }
  }else{
    cmbGroup->clear();
    QList<int> l;
    for(int num=0;num<230;num++){
      if(get_crystal_system_from_num(num)==csystem){
	long pgr=get_pointgroup_from_num(num);
	if(l.indexOf(pgr)<0){
	  l.append(pgr);
	}
      }
    }
    for(int i=0;i<l.size();i++){
      cmbGroup->insertItem(i,QString().sprintf("%-12s",pointgroup(l[i]))
			   ,QVariant(l[i]));
    }
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_cmbBravais_currentIndexChanged(int index)
{
  //myDebug()<<"on_cmbBravais_currentIndexChanged";

  cmbGroup->clear();
  muteCmbRep=true;
  cmbRep->clear();

  if(isCrystal()){
    
    long csystem=cmbCrysSystem->currentIndex();
    long btype=cmbBravais->itemData(index,Qt::UserRole).toInt();
    //   myDebug() << "btype" << btype << "csystem" << csystem <<
    //     bravaistypes_longname(btype) << numberofspacegroupsof_c_b(csystem,btype);
    if(btype==SYM_RHOMBOHEDRAL){
      cmbRep->insertItem(0,"rhombohedral",QVariant(SYM_RHOMBOHEDRAL));
      cmbRep->insertItem(1,"hexagonal",QVariant(SYM_SIMPLE));
      cmbRep->setEnabled(true);
      //myDebug()<<"t_currentrepindex" << t_currentrepindex;
      cmbRep->setCurrentIndex(t_currentrepindex);
      muteCmbRep=false;
      on_cmbRep_currentIndexChanged(cmbRep->currentIndex());
    }else{
      cmbRep->setEnabled(false);
    }
    
    for(int i=0;i<numberofspacegroupsof_c_b(csystem,btype);i++){
      int num=spacegroupsof_c_b(csystem,btype,i);
      cmbGroup->insertItem(i,QString()
			   .sprintf("%-12s",
				    Symmetry::spacegroupName(num)),QVariant(num));
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_cmbGroup_currentIndexChanged(int index)
{

  long num=cmbGroup->itemData(index,Qt::UserRole).toInt();

  if(!isCrystal()){
    num=get_spacegroup_of_from_pointgroup(num);
    if(num<1) return;
    
  }
  //  myDebug() <<"SetText";
  if(!rbNumber->isChecked()){
    mutespbxgroup=true;
    spbxGroup->setValue(num);
    mutespbxgroup=false;
  }
  

  slotCrystalSystemEnableWidgets(num);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_cmbRep_currentIndexChanged(int index)
{
  //myDebug() << "on_cmbRep_currentIndexChanged";
  //myDebug() << "idnex" << index << (muteCmbRep?"mute":"active");
  if(index<0 || muteCmbRep) return;

  QString txt;
  if(!checkLatticeParameters(txt)){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 );
    muteCmbRep=true;
    cmbRep->setCurrentIndex(t_currentrepindex);
    muteCmbRep=false;
    return;
  }
  


  long reptype=cmbRep->itemData(index,Qt::UserRole).toInt();
  long currentreptype=cmbRep->itemData(t_currentrepindex,Qt::UserRole).toInt();
  //myDebug() << "current rep" << currentreptype << "set reptype" << reptype;
  
  if(reptype==SYM_RHOMBOHEDRAL){
    if(currentreptype==SYM_SIMPLE){
      convertLatticeParameters(currentreptype,reptype);
    }
  }else if(reptype==SYM_SIMPLE){
    if(currentreptype==SYM_RHOMBOHEDRAL){
      convertLatticeParameters(currentreptype,reptype);
    }
  }

  t_currentrepindex=index;

  slotCrystalSystemEnableWidgets(spbxGroup->value());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::convertLatticeParameters(long from,long to)
{
  //myDebug() << "convert"<<bravaistypes_longname(from) << " -> "
  //	   <<bravaistypes_longname(to);

  Matrix3x3 m(Vec3(1,0,1),Vec3(-1,1,1),Vec3(0,-1,1));
  
  if(from==SYM_RHOMBOHEDRAL && to ==SYM_SIMPLE){
    QString t=leAlpha->text(); IntValidator::stripAllWhiteSpace(t);
    SReal alpha=t;
    t=leAlat->text(); IntValidator::stripAllWhiteSpace(t);
    SReal A=t;

    //    myDebug() <<"A" << A.D << "alpha"<<alpha.D;

    double ca=cos(alpha.D*Pi/180.);
    double a=A.D*sqrt(2.*MyMath::min<double>(2,MyMath::max<double>(0,(1.-ca))));
    double c=A.D*sqrt(3.*MyMath::min<double>(3,MyMath::max<double>(0,(1.+2.*ca))));
    
    String sa=Sprintf("%30.15f",a).removeLeadingBlanks().chop("0");
    String sc=Sprintf("%30.15f",c).removeLeadingBlanks().chop("0");
    
    leAlat->setText(sa());
    leBlat->setText(sa());
    leClat->setText(sc());
    leAlpha->setText("90");
    leBeta->setText("90");
    leGamma->setText("120");

    for(int row=0; row < model->rowCount(); row++){
      QModelIndex index;
      SReal x,y,z;
      
      index=model->index(row,WyckPosModel::X,QModelIndex());
      x=model->data(index,Qt::DisplayRole).toString();
      index=model->index(row,WyckPosModel::Y,QModelIndex());
      y=model->data(index,Qt::DisplayRole).toString();
      index=model->index(row,WyckPosModel::Z,QModelIndex());
      z=model->data(index,Qt::DisplayRole).toString();
      //myDebug() << x.D<<y.D<<z.D;

      Vec3 v(x.D,y.D,z.D);
      v=v*m.inverse();
      //myDebug() << m.printS()()<<"\n"<< v.printS()();


      for(int i=0;i<3;i++){
	String s=Sprintf("%30.15f",v[i]).removeLeadingBlanks().chop("0");
	index=model->index(row,i+WyckPosModel::X,QModelIndex());
	model->setData(index,s(),Qt::DisplayRole|Qt::EditRole);
      }
      
    }

    
  }else{
    QString t=leAlat->text(); IntValidator::stripAllWhiteSpace(t);
    SReal a=t;
    t=leClat->text(); IntValidator::stripAllWhiteSpace(t);
    SReal c=t;
    
    double a2=a.D*a.D;
    double c2=c.D*c.D;

    double A2=a2/3.+c2/9.;
    double A=sqrt(A2);
    double alpha=acos(MyMath::min<double>(1,MyMath::max<double>(-1,(c2/9.-a2/6.)/(A2))))/Pi*180.;
    //myDebug() << "a" << a.D << "c" << c.D << "alpha" <<alpha;

    String sA=Sprintf("%30.15f",A).removeLeadingBlanks().chop("0");
    String salpha=Sprintf("%30.15f",alpha).removeLeadingBlanks().chop("0");

    leAlat->setText(sA());
    leBlat->setText(sA());
    leClat->setText(sA());

    leAlpha->setText(salpha());
    leBeta->setText(salpha());
    leGamma->setText(salpha());

    for(int row=0; row < model->rowCount(); row++){
      QModelIndex index;
      SReal x,y,z;
      
      index=model->index(row,WyckPosModel::X,QModelIndex());
      x=model->data(index,Qt::DisplayRole).toString();
      index=model->index(row,WyckPosModel::Y,QModelIndex());
      y=model->data(index,Qt::DisplayRole).toString();
      index=model->index(row,WyckPosModel::Z,QModelIndex());
      z=model->data(index,Qt::DisplayRole).toString();
      //myDebug() << x.D<<y.D<<z.D;

      Vec3 v(x.D,y.D,z.D);
      v=v*m;
      //myDebug() << m.printS()()<<"\n"<< v.printS()();


      for(int i=0;i<3;i++){
	String s=Sprintf("%30.15f",v[i]).removeLeadingBlanks().chop("0");
	index=model->index(row,i+WyckPosModel::X,QModelIndex());
	model->setData(index,s(),Qt::DisplayRole|Qt::EditRole);
      }
      
    }
  }
  leAlat->setCursorPosition(0);
  leBlat->setCursorPosition(0);
  leClat->setCursorPosition(0);
  leAlpha->setCursorPosition(0);
  leBeta->setCursorPosition(0);
  leGamma->setCursorPosition(0);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_rbNumber_toggled(bool onoff)
{
  //  myDebug() <<  onoff;

  cmbCrysSystem->setEnabled(!onoff && !t_silent);
  cmbBravais->setEnabled(!onoff && !t_silent);
  cmbGroup->setEnabled(!onoff && !t_silent);

  spbxGroup->setEnabled(onoff && !t_silent);


  if(onoff){
    lSpacegroup->setBuddy(spbxGroup);
  }else{
    lSpacegroup->setBuddy(cmbCrysSystem);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SymDlg::isCrystal()
{
  return (cmbStructureType->currentIndex()+1 == Xf::STCrystal);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_spbxGroup_valueChanged(int num)
{
  if(mutespbxgroup)return;
  //myDebug() <<"SymDlg::on_spbxGroup_valueChanged"<<num;
  long cs=get_crystal_system_from_num(num);
  long pgroup=get_pointgroup_from_num(num);

  //myDebug() << text << num << cs;
  cmbCrysSystem->setCurrentIndex(cs);

  long bt;
  if( isCrystal() ){
    bt=get_bravais_from_num(num);
    
    for(int i=0;i<numberofbravaistypesofcryssystem(cs);i++){
      int btype=bravaistypesofcryssystem(cs,i);
      if(btype==bt){
	cmbBravais->setCurrentIndex(i);
	
	for(int i=0;i<numberofspacegroupsof_c_b(cs,bt);i++){
	  if( num==spacegroupsof_c_b(cs,bt,i)){
	    cmbGroup->setCurrentIndex(i);
	  }
	}
	
	break;
      }
    }
  }else{

    QList<int> l;
    for(int j=0;j<230;j++){
      if(get_crystal_system_from_num(j)==cs){
	long pgr=get_pointgroup_from_num(j);
	if(l.indexOf(pgr)<0){
	  l.append(pgr);
	}
      }
    }

    for(int i=0;i<l.size();i++){
      if( pgroup==l[i]){
	cmbGroup->setCurrentIndex(i);
      }
    }
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::slotCrystalSystemSetRedundant()
{

  long num=cmbGroup->itemData(cmbGroup->currentIndex(),Qt::UserRole).toInt();

  if(!isCrystal() || t_silent) return;



  long cs=get_crystal_system_from_num(num);
  long bt=get_bravais_from_num(num);

  long currentreptype=
    cmbRep->itemData(t_currentrepindex,Qt::UserRole).toInt();


   switch (cs){
   case SYM_CUBIC:
     leBlat ->setText(leAlat->text());
     leClat ->setText(leAlat->text());
     leAlpha->setText("90");
     leBeta->setText("90");
     leGamma->setText("90");
     break;
   case SYM_TETRAGONAL:
     leBlat ->setText(leAlat->text());
     leAlpha->setText("90");
     leBeta->setText("90");
     leGamma->setText("90");
     break;
   case SYM_HEXAGONAL:
     leBlat ->setText(leAlat->text());
     leAlpha->setText("90");
     leBeta->setText("90");
     leGamma->setText("120");
     break;
   case SYM_TRIGONAL:
     if(bt==SYM_RHOMBOHEDRAL){
       if(currentreptype==SYM_RHOMBOHEDRAL){
	 leBlat ->setText(leAlat->text());
	 leClat ->setText(leAlat->text());
	 leBeta->setText(leAlpha->text());
	 leGamma->setText(leAlpha->text());
       }else{
	 leBlat ->setText(leAlat->text());
	 leAlpha->setText("90");
	 leBeta->setText("90");
	 leGamma->setText("120");
       }
     }else{
       leBlat ->setText(leAlat->text());
       leAlpha->setText("90");
       leBeta->setText("90");
       leGamma->setText("120");
     }
     break;
   case SYM_ORTHORHOMBIC:
     leAlpha->setText("90");
     leBeta->setText("90");
     leGamma->setText("90");
     break;
   case SYM_MONOCLINIC:
     leAlpha->setText("90");
     leGamma->setText("90");
     break;
   case SYM_TRICLINIC:
     break;
   default:;
   }
  leAlat->setCursorPosition(0);
  leBlat->setCursorPosition(0);
  leClat->setCursorPosition(0);
  leAlpha->setCursorPosition(0);
  leBeta->setCursorPosition(0);
  leGamma->setCursorPosition(0);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::slotCrystalSystemEnableWidgets(long num)
{
  long cs=get_crystal_system_from_num(num);
  long bt=get_bravais_from_num(num);

  Xf::StructureType stype=(Xf::StructureType) 
    (cmbStructureType->currentIndex()+1);

  bool on=(!t_silent && stype ==Xf::STCrystal);


  cmbRep->setEnabled((stype==Xf::STCrystal));
  
  Symmetry::getGroupOperations(num,stype,pops);

  leAlat ->setEnabled( on);

  long currentreptype=
    cmbRep->itemData(t_currentrepindex,Qt::UserRole).toInt();

  switch (cs){
  case SYM_CUBIC:
    lAlpha->setBuddy(leAlpha);
    leClat ->setEnabled( false);
    leBlat ->setEnabled( false);
    leAlpha->setEnabled( false);
    leBeta ->setEnabled( false);
    leGamma->setEnabled( false);
    break;
  case SYM_TETRAGONAL:
  case SYM_HEXAGONAL:
    lAlpha->setBuddy(leAlpha);
    leClat->setEnabled( on);
    leBlat->setEnabled( false);
    leAlpha->setEnabled( false);
    leBeta->setEnabled( false);
    leGamma->setEnabled( false);
    break;
  case SYM_TRIGONAL:
    lAlpha->setBuddy(leAlpha);
    if(bt==SYM_RHOMBOHEDRAL&& currentreptype == SYM_RHOMBOHEDRAL){
      leClat->setEnabled( false);
      leBlat->setEnabled( false);
      leAlpha->setEnabled( on);
      leBeta->setEnabled( false);
      leGamma->setEnabled( false);
    }else{
      leClat->setEnabled( on);
      leBlat->setEnabled( false);
      leAlpha->setEnabled( false);
      leBeta->setEnabled( false);
      leGamma->setEnabled( false);
    }
    break;
  case SYM_ORTHORHOMBIC:
    lAlpha->setBuddy(leAlpha);
    leClat->setEnabled( on);
    leBlat->setEnabled( on);
    leAlpha->setEnabled( false);
    leBeta->setEnabled( false);
    leGamma->setEnabled( false);
    break;
  case SYM_MONOCLINIC:
    lAlpha->setBuddy(leBeta);
    leClat->setEnabled( on);
    leBlat->setEnabled( on);
    leAlpha->setEnabled( false);
    leBeta->setEnabled(  on);
    leGamma->setEnabled( false);
    break;
  case SYM_TRICLINIC:
    lAlpha->setBuddy(leAlpha);
    leClat->setEnabled( on);
    leBlat->setEnabled( on);
    leAlpha->setEnabled( on );
    leBeta->setEnabled(  on);
    leGamma->setEnabled(  on);
    break;
  default:;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_cmbStructureType_currentIndexChanged(int stype)
{
  stype++;
  //  myDebug() <<stype << Symmetry::structureTypes((Xf::StructureType)stype);
  long sg=spbxGroup->value();

  if(stype==Xf::STCrystal){
    lSpacegroup->setText("Space grou&p");
    cmbBravais->show();
  }else{
    lSpacegroup->setText("Point grou&p");
    cmbBravais->clear();
    cmbBravais->hide();
  }
  on_cmbCrysSystem_currentIndexChanged(cmbCrysSystem->currentIndex());
  mutespbxgroup=true;
  spbxGroup->setValue(sg);
  mutespbxgroup=false;
  on_spbxGroup_valueChanged(sg);
  
  slotCrystalSystemEnableWidgets(spbxGroup->value());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SymDlg::~SymDlg()
{
  if(ptoolbuttons) delete ptoolbuttons;ptoolbuttons=0;
  //myDebug()<<"SymDlg dies"<<this;
  onClose();
  if(t_sym) delete t_sym; t_sym=0;
  if(model) {
    //myDebug() << "SymDlg::~SymDlg" << "remove model"; 
    delete model; model=0;
  }
  if(propmodel) {
    //myDebug() << "SymDlg::~SymDlg" << "remove model"; 
    delete propmodel; propmodel=0;
  }
  // if(eldelegate) delete eldelegate;eldelegate=0;
  // if(xyzdelegate) delete xyzdelegate;xyzdelegate=0;
  if(themeMenu) delete themeMenu; themeMenu=0;

  if(elvalid) delete elvalid;elvalid=0;
  if(xyzvalid) delete xyzvalid;xyzvalid=0;

  if(colordelegate) delete colordelegate;colordelegate=0;
  if(speccolordelegate) delete speccolordelegate;speccolordelegate=0;
  if(radtypdelegate) delete radtypdelegate;radtypdelegate=0;
  if(radiusdelegate) delete radiusdelegate;radiusdelegate=0;
  if(shineninessdelegate) delete shineninessdelegate;shineninessdelegate=0;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::setData( const Symmetry *sym ,SymDlgMode dlgmode)
{
  cmbRep->setCurrentIndex(0);


  pdlgmode=dlgmode;
  t_silent=(dlgmode==FermiSurfaceViewMode);
  bool enabled=!t_silent;
  
  psym=sym;
  *t_sym=*sym;
  t_sym->setChanged(false);

  //myDebug() << "set alat";
  leAlat->setText(sym->latticeConstantS(0)()) ;	
  leBlat->setText(sym->latticeConstantS(1)()) ;	
  leClat->setText(sym->latticeConstantS(2)()) ;	


  
  leAlpha->setText(sym->axesAngleS(0)()) ;	
  leBeta->setText(sym->axesAngleS(1)()) ;	
  leGamma->setText(sym->axesAngleS(2)()) ;	

  leAlat->setCursorPosition(0);
  leBlat->setCursorPosition(0);
  leClat->setCursorPosition(0);
  leAlpha->setCursorPosition(0);
  leBeta->setCursorPosition(0);
  leGamma->setCursorPosition(0);


  model->reload();
  propmodel->reload();
  tvWyck->setCurrentIndex(model->index(0,0,QModelIndex()));
  tvWyckProps->setCurrentIndex(propmodel->index(0,0,QModelIndex()));

  adjustTvProps();

  psubgroupgenerators=sym->subGroupGenerators();
  

  cmbStructureType->setCurrentIndex(sym->structureType()-1);

  //  printf("sym->spacegroup()=%ld\n",sym->spacegroup());
  mutespbxgroup=true;
  spbxGroup->setValue(sym->spacegroup());
  mutespbxgroup=false;
  //printf("spbxGroup->value()=%d\n",spbxGroup->value());

  on_spbxGroup_valueChanged(spbxGroup->value());  


  //rbNumber->setFocus();

  if(sym->lengthUnit()==Xf::Angstroem){
    //      rbAngstroem->setFocus();
      rbAngstroem->setChecked(true);
  }else if(sym->lengthUnit()==Xf::BohrRadius){
    //      rbBohr->setFocus();
    rbBohr->setChecked(true);
  }


  pbSubGroupGen->setEnabled(enabled);
  spbxGroup->setEnabled(enabled);
  leAlat->setEnabled(enabled);
  leBlat->setEnabled(enabled);
  leClat->setEnabled(enabled);
  leAlpha->setEnabled(enabled);
  leBeta->setEnabled(enabled);
  leGamma->setEnabled(enabled);
  rbAngstroem->setEnabled(enabled);
  rbBohr->setEnabled(enabled);
  rbNumber->setEnabled(enabled);
  rbSymbol->setEnabled(enabled);
  cmbCrysSystem->setEnabled(enabled);
  cmbBravais->setEnabled(enabled);
  cmbGroup->setEnabled(enabled);
  frameStructureType->setEnabled(enabled);
  tbInsert->setEnabled(enabled);
  tbDelete->setEnabled(enabled);
  tbUp->setEnabled(enabled);
  tbDown->setEnabled(enabled);
  leSubGroupGens->setEnabled(enabled);
  //lblWyck
  tvWyck->setEnabled(enabled);
  tbSetDefault->setEnabled(enabled);

  if(dlgmode==StructureMode){
    //pbApply->show();
    pbLoadFPLOInFile->hide();
    pbSaveFPLOInFile->show();
    pbSavePipe->show();
  }else if(dlgmode==FermiSurfaceEditMode){
    //pbApply->show();
    pbLoadFPLOInFile->show();
    pbSaveFPLOInFile->hide();
    pbSavePipe->hide();

    frameStructureType->hide();
    lblWyck->hide();
    tvWyck->hide();
    tabWidget->setEnabled(false);
    tbInsert->hide();
    tbDelete->hide();
    tbUp->hide();
    tbDown->hide();
  }

  if(dlgmode==FermiSurfaceViewMode){
    //pbApply->hide();
    pbLoadFPLOInFile->hide();
    pbSaveFPLOInFile->hide();
    pbSavePipe->hide();
    frameStructureType->hide();
    lblWyck->hide();
    tvWyck->hide();
    tabWidget->setEnabled(false);
    tbInsert->hide();
    tbDelete->hide();
    tbUp->hide();
    tbDown->hide();
  }

  

  rbNumber->setChecked(true);
  
  slotCrystalSystemEnableWidgets(sym->spacegroup());


  setSubGroupGenMarker();

  adjustTvWyck();

  cmbRep->setEnabled(enabled);
 

  on_rbNumber_toggled(rbNumber->isChecked());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::setSubGroupGenMarker()
{
  if(psubgroupgenerators.size()==0){
    leSubGroupGens->setText("full group");
    return;
  }
  
  QString s="";
  for(int i=0;i<psubgroupgenerators.size();i++){
    int found=-1;
    for(int j=0;j<pops.size();j++){
      if(pops.at(j).index()==psubgroupgenerators.at(i)){
	found=j;
	break;
      }
    }
    if(found>=0){
      if(s.isEmpty())
	s+=pops.at(found).CSSymbol()();
      else
	s+=QString(" , ")+pops.at(found).CSSymbol()();
    }else{
      if(s.isEmpty())
	s+="??";
      else
	s+=QString(" , ??");
    }
  }

  leSubGroupGens->setText(s);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::getData( Symmetry *sym, bool &chn )
{
  QString t;
  long num=spbxGroup->value();

  t_sym->setSpacegroup(num);
  sym->setSpacegroup(num);

  t_sym->setSubGroupGenerators(psubgroupgenerators);
  sym->setSubGroupGenerators(psubgroupgenerators);

 
  long reptype=
    cmbRep->itemData(cmbRep->currentIndex(),Qt::UserRole).toInt();

  if(reptype==SYM_SIMPLE && get_bravais_from_num(num) ==SYM_RHOMBOHEDRAL){
    convertLatticeParameters(reptype,SYM_RHOMBOHEDRAL);
  }

 
  String a[3];
  t=leAlat->text(); IntValidator::stripAllWhiteSpace(t);
  a[0]=t.toAscii().constData();

  t=leBlat->text(); IntValidator::stripAllWhiteSpace(t);
  a[1]=t.toAscii().constData();
  
  t=leClat->text(); IntValidator::stripAllWhiteSpace(t);
  a[2]=t.toAscii().constData();
  
  t_sym->setLatticeConstants(a);
  sym->setLatticeConstants(a);
  
  t=leAlpha->text(); IntValidator::stripAllWhiteSpace(t);
  a[0]=t.toAscii().constData();

  t=leBeta->text(); IntValidator::stripAllWhiteSpace(t);
  a[1]=t.toAscii().constData();

  t=leGamma->text(); IntValidator::stripAllWhiteSpace(t);
  a[2]=t.toAscii().constData();

  sym->setAxesAngles(a);
  t_sym->setAxesAngles(a);

  if(rbAngstroem->isChecked()){
      sym->setLengthunit(Xf::Angstroem);
      t_sym->setLengthunit(Xf::Angstroem);
  }
  if(rbBohr->isChecked()){
      sym->setLengthunit(Xf::BohrRadius);
      t_sym->setLengthunit(Xf::BohrRadius);
  }

  Xf::StructureType stype=(Xf::StructureType) 
    (cmbStructureType->currentIndex()+1);
  sym->setStructureType(stype);
  t_sym->setStructureType(stype);


  sym->setWyckoffPositions(t_sym->wyckoffPositions());



  chn=t_sym->changed();
  //  myDebug() << "chn" << chn;
  t_sym->setChanged(false);


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SymDlg::checkLatticeParameters(QString &txt)
{

  ((IntValidator*)leAlat->validator())->check(txt);
  ((IntValidator*)leBlat->validator())->check(txt);
  ((IntValidator*)leClat->validator())->check(txt);
  ((IntValidator*)leAlpha->validator())->check(txt);
  ((IntValidator*)leBeta->validator())->check(txt);
  ((IntValidator*)leGamma->validator())->check(txt);

  // validate the wyckposmodel
  //todo: belongs into the model?

  if(pdlgmode==StructureMode){
    for(int row=0; row < model->rowCount(); row++){
      QModelIndex index;
      long err=0;
      QString m;
      {
	index=model->index(row,WyckPosModel::El,QModelIndex());
	QString s=model->data(index,Qt::DisplayRole).toString();
	err=!elvalid->check(m,s);
	model->setData(index,s,Qt::DisplayRole|Qt::EditRole);
      }
      {
	index=model->index(row,WyckPosModel::Type,QModelIndex());
	QString s=model->data(index,Qt::DisplayRole).toString();
	err|=!typevalid->check(m,s);
	model->setData(index,s,Qt::DisplayRole|Qt::EditRole);
      }
      {
	index=model->index(row,WyckPosModel::X,QModelIndex());
	QString s=model->data(index,Qt::DisplayRole).toString();
	err|=!xyzvalid->check(m,s);
      }
      {
	index=model->index(row,WyckPosModel::Y,QModelIndex());
	QString s=model->data(index,Qt::DisplayRole).toString();
	err|=!xyzvalid->check(m,s);
      }
      {
	index=model->index(row,WyckPosModel::Z,QModelIndex());
	QString s=model->data(index,Qt::DisplayRole).toString();
	err|=!xyzvalid->check(m,s);
      }
      if(err)
	txt+=QString("Position %1:").arg(row+1)+m;
    }
  }    
  if(txt.length())return false;
  return true;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SymDlg::dataAcceptable()
{
  //int i;

//   pbOK->setFocus();
//   tvWyck->setFocus();


  slotCrystalSystemSetRedundant();

  QString s,txt="";


  checkLatticeParameters(txt);
  


  if(!( rbAngstroem->isChecked() || rbBohr->isChecked() ) )
    txt+="\nOne lenght unit radio button must be checked!";

  if(!( rbNumber->isChecked() || rbSymbol->isChecked() ) )
    txt+="\nOne group selector radio button must be checked!";



  
  if(psubgroupgenerators.size()>0){
    for(int i=0;i<psubgroupgenerators.size();i++){
      bool found=false;
      for(int j=0;j<pops.size();j++){
	if(pops.at(j).index()==psubgroupgenerators.at(i))
	  found=true;
      }
      if(!found){
	txt+=QString("Subgroup generator %1 does not belong to the group")
	  .arg(psubgroupgenerators.at(i));
      }
    }
  }

  if(txt.length()){
    QMessageBox::warning ( this,IDS_APP_NAME, txt, 1, 0,0 ) ;
    return false;
  }else{
    return  true;
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool SymDlg::commitData(Symmetry *sym)
{
  if(dataAcceptable()){
    bool chn;
    getData(sym,chn);
    return true;
  }else{
    return false;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SymDlg::onClose()
{
  if(pbManipulate->isChecked()){
    //myDebug()<<"open";
    resize(savedsize);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_pbSubGroupGen_clicked()
{
  //  myDebug() << "SymDlg::on_pbSubGroupGen_clicked";

  SubGroupGenDlg *dlg=new SubGroupGenDlg(this);


  dlg->setData(&pops,psubgroupgenerators);
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(psubgroupgenerators);
    setSubGroupGenMarker();    
  }
  delete dlg;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::adjustTvWyck()
{
  int rows=model->rowCount();
  // for(int irow=0;irow<rows;irow++)
  //   tvWyck->resizeRowToContents(irow);
  // //int cols=model->columnCount();
  // //for(int icol=0;icol<cols;icol++)
  // //tvWyck->resizeColumnToContents(icol);
  // tvWyck->resizeColumnToContents(WyckPosModel::El);
  // tvWyck->resizeColumnToContents(WyckPosModel::Type);
  //tvWyck->resizeRowsToContents();
  // tvWyck->resizeColumnsToContents();
  if (true && rows>0 && !ptvWyckRowSizeKnown){
    ptvWyckRowSizeKnown=true;
    // verticalHeader header
    tvWyck->resizeRowToContents(0);
    int size=tvWyck->verticalHeader()->sectionSize(0);
    tvWyck->verticalHeader()->setDefaultSectionSize(size);
    tvWyck->verticalHeader()->setResizeMode(QHeaderView::Fixed);

    tvWyckProps->verticalHeader()->setDefaultSectionSize(size);
    tvWyckProps->verticalHeader()->setResizeMode(QHeaderView::Fixed);


    // horizontal header
    tvWyck->resizeColumnToContents(WyckPosModel::El);
    size=tvWyck->horizontalHeader()->sectionSize(WyckPosModel::El);

    tvWyck->horizontalHeader()->setDefaultSectionSize(size);
    tvWyck->horizontalHeader()->setResizeMode(WyckPosModel::El,QHeaderView::Fixed);
    tvWyck->horizontalHeader()->setResizeMode(WyckPosModel::Type,QHeaderView::Fixed);
    tvWyck->horizontalHeader()->setResizeMode(WyckPosModel::X,
					      QHeaderView::ResizeToContents);
    tvWyck->horizontalHeader()->setResizeMode(WyckPosModel::Y,
					      QHeaderView::ResizeToContents);
    tvWyck->horizontalHeader()->setResizeMode(WyckPosModel::Z,
					      QHeaderView::ResizeToContents);
    tvWyckProps->horizontalHeader()->setDefaultSectionSize(size);
    tvWyckProps->horizontalHeader()->setResizeMode(QHeaderView::Fixed);
    tvWyckProps->horizontalHeader()->setResizeMode(AtomPropsModel::AtomShininess,
						   QHeaderView::ResizeToContents);
    tvWyckProps->horizontalHeader()->setResizeMode(AtomPropsModel::RadiusType,
						   QHeaderView::ResizeToContents);
    tvWyckProps->horizontalHeader()->setResizeMode(AtomPropsModel::Radius,
						   QHeaderView::ResizeToContents);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::adjustTvProps()
{
  // int rows=propmodel->rowCount();
  // int cols=propmodel->columnCount();
  // for(int irow=0;irow<rows;irow++)
  //   tvWyckProps->resizeRowToContents(irow);
  // for(int icol=0;icol<cols;icol++){
  //   tvWyckProps->resizeColumnToContents(icol);
  // }
  //tvWyckProps->resizeRowsToContents();
  // tvWyckProps->resizeColumnsToContents();
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_tvWyckProps_clicked(const QModelIndex &index)
{
  if(!index.isValid()) return;
  
  if(!(index.flags()&Qt::ItemIsEnabled))return;
  if((index.column()==AtomPropsModel::AtomColor)||
     (index.column()==AtomPropsModel::SpecularColor)){
    tvWyckProps->edit(index);
  }else{
    return;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void SymDlg::on_pbApply_clicked()
// {
//   NSF("SymDlg::on_pbApply_clicked()");
//   //myDebug() << nsubfunc;
//   if(dataAcceptable())
//     emit applyData(this,true);

//   setRepresentation();
// }
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::setRepresentation()
{
  if(cmbRep->currentIndex()<0)return;
  long reptype=
    cmbRep->itemData(cmbRep->currentIndex(),Qt::UserRole).toInt();

  long num=spbxGroup->value();

  if(reptype==SYM_SIMPLE && get_bravais_from_num(num) ==SYM_RHOMBOHEDRAL){
    convertLatticeParameters(SYM_RHOMBOHEDRAL,reptype);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_pbLoadFPLOInFile_clicked()
{
  NSF("SymDlg::on_pbLoadFPLOInFile_clicked()");
  //myDebug() << nsubfunc;
  emit loadFPLOInFile(this);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_pbSaveFPLOInFile_clicked()
{
  saveStuff("=.sym");
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_pbSavePipe_clicked()
{
  saveStuff("sympipe");
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::saveStuff(const QString &what)
{
  NSF("SymDlg::on_pbSaveFPLOInFile_clicked()");
  //myDebug() << nsubfunc;
  if(dataAcceptable()){

    bool chn=false;
    Symmetry tsym;
    getData(&tsym,chn);
    
    QString fname=what;
    QString title,filter;
    
    if(what=="=.sym"){
      title="Export symmetry";
      filter="Fplo input ( =.sym ) ;; All ( * )";
    }else{
      title="Export symmetry pipe info";
      filter="All ( * )";
    }
    long ret=QDialog::Accepted;

    QFileDialog *dlg=new QFileDialog(this,title,".",filter);
    dlg->setAcceptMode(QFileDialog::AcceptSave);
    dlg->setFileMode(QFileDialog::AnyFile);
#ifdef Q_OS_MAC
    dlg->setOptions(QFileDialog::DontUseNativeDialog | 
		    QFileDialog::DontConfirmOverwrite);
#endif
    dlg->setOptions(QFileDialog::DontConfirmOverwrite);
    dlg->setDirectory(QDir::currentPath()+"/");

    if(!fname.isEmpty()){
      dlg->selectFile(fname);
    }else{
      dlg->selectFile(what);
    }
    fname="";
    if((ret=dlg->exec())==QDialog::Accepted){
      if(dlg->selectedFiles().size()>0)
	fname=dlg->selectedFiles().at(0);
    }
    delete (dlg);
    if(!fname.isEmpty()){
      QFileInfo fi(fname);
      QString fp=fi.absolutePath();
      try{
	if(what=="=.sym"){
	  writeSymToFile(tsym,fi.absolutePath()+"/=.sym");
	}else{
	  writeSymPipeInfoToFile(tsym,fname);
	}
      }catch(Except &ex){
	QMessageBox::warning ( this,IDS_APP_NAME, ex.msg(), 1, 0,0 ) ;
      }
    }
    
  }

  setRepresentation();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::writeSymToFile(const Symmetry &tsym,const QString &fname)
{
  NSF("SymDlg::writeSymToFile");
  //myDebug()<<nsubfunc << fname;



  
  if(!QFileInfo(fname).exists()){
    FILE *FH;
    try{
      FH=openforwrite(String(fname));
    }catch(Except &ex){
      QMessageBox::warning(0,""
			   ,Sprintf("Cannot open file \"%s\" for write!%s\n"
				    ,String(fname)(),ex.msg())()
			   
			   ,QMessageBox::Ok| QMessageBox::Default);
    }
    
    // this is the best we can do, without knowing what fplo version is
    // installed and without forcing another Run-Dialog on the user
    fprintf(FH,
	   "section header{\n"
	   "    struct {char[*] mainversion;char[*] subversion;} version\n"
	   "    ={\"%s\",\"M-CPA\"};\n"
	   "    struct {char[*] hostname;char[*] date;char[*] by;} last_changes\n"
	   "    ={\"bono\",\"Thu Oct  6 13:59:43 2011\",\"fplo (%s-%s)\"};\n"
	   "  char[*] status=\"ready\";\n"
	   "  char[*] compound=\"Al example\";\n"
	   "};\n"
	   "section structure_definition{\n"
	   " struct {int type;char[*] description;} structure_type\n"
	   "  ={   1,\"Crystal\"};\n"
	   "  struct {int number;char[*] symbol;} spacegroup\n"
	   "  ={ 225,\"FM3M\"};\n"
	   "  int subgroupgenerators[*]\n"
	   "  ={\n"
	   "  };\n"
	   " struct {int type;char[*] description;} lengthunit\n"
	   "  ={   1,\"Bohr radii\"};\n"
	   "  real lattice_constants[3]={7.55,7.55,7.55  };\n"
	   "  real axis_angles[3]={90.,90.,90.  };\n"
	   "  # WARNING: max_L is obsolete now, since moved into =.in\n"
	   "  int max_L=  4;\n"
	   "  int nsort=  1;\n"
	   "  struct {char[2] element;real tau[3];} wyckoff_positions[nsort]\n"
	   "  ={\n"
	   "    {  \"Al\",{  0.,0.,0.  }}\n"
	   "  };\n"
	   "};\n"
	   "\n"
	    ,FPLOMAINVERSION(),FPLOMAINVERSION(),FPLORELEASE());
    
    fclose(FH);

  }

  Parser *symparser=new Parser(1);
  symparser->parseFile(fname);
  


  try{
    (*symparser)()("structure_type")("type")=tsym.structureType();
  }catch(Except){}



  (*symparser)()("spacegroup")("number")=tsym.spacegroup();
  (*symparser)()("lengthunit")("type")=tsym.lengthUnit();


  
  
  
  (*symparser)()("lattice_constants")[0]=tsym.latticeConstantS(0)();
  (*symparser)()("lattice_constants")[1]=tsym.latticeConstantS(1)();
  (*symparser)()("lattice_constants")[2]=tsym.latticeConstantS(2)();
  (*symparser)()("axis_angles")[0]=tsym.axesAngleS(0)();
  (*symparser)()("axis_angles")[1]=tsym.axesAngleS(1)();
  (*symparser)()("axis_angles")[2]=tsym.axesAngleS(2)();


  try{
    long n[8];
    n[1]=tsym.subGroupGenerators().size();
    Dobj da=(*symparser)()("subgroupgenerators");
    da.resizeArp(n);
    for(int i=0;i<n[1];i++){
      da[i]=tsym.subGroupGenerators().at(i);
    }
  }catch(Except e){};

  

  Dobj dw=(*symparser)()("wyckoff_positions");
  long nsort=tsym.wyckoffPositions().size();
  dw.resizeSap(nsort);
  (*symparser)()("nsort")=nsort;
  for(int issa=0;issa<nsort;issa++){
    Dobj d=dw[issa];
    d("element")=tsym.wyckoffPosition(issa).el();
    d("tau")[0]=tsym.wyckoffPosition(issa).tauS(0);
    d("tau")[1]=tsym.wyckoffPosition(issa).tauS(1);
    d("tau")[2]=tsym.wyckoffPosition(issa).tauS(2);
  }



  
  

  
  // write file back
  symparser->writeFile(fname);
  

  delete symparser;

  
  {
    // update the infile
    Fplo *fplo=0;
    try{
      fplo=new Fplo(fname,"out_update_sym");
    }catch(Except &ex){
      QMessageBox::warning(this,"Prepare for running fplo"
			   ,QString("%1\n")
			   .arg(ex.msg())
			   ,QMessageBox::Ok | QMessageBox::Default
			   );
    }
  
    fplo->setForceSymUpdate(true);
    fplo->setForceTerminationCode("Normal : File =.in updated!");
    fplo->setMessage("FPLO needs to update the new symmetry!\n"
		     "Please choose the executable and run it!");

    FploDlg *dlg=new FploDlg(this,fplo);
    dlg->exec();
    
    if(!dlg->didRun()){
      QMessageBox::warning(0,""
			 ,"You did not run FPLO, =.sym and =.in\n"
			   "will not be in sync!\n"
			   ,QMessageBox::Ok| QMessageBox::Default);

    }
    
    delete dlg;


  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::writeSymPipeInfoToFile(const Symmetry &tsym,const QString &fname)
{
  NSF("SymDlg::writeSymPipeInfoToFile");
  //myDebug()<<nsubfunc << fname;


  
  FILE *FH;
  try{
    FH=openforwrite(fname);
  }catch(Except ex){
    throw Except(String().sprintf("Cannot write file"
				  " \'%s\':\n %s\n!"
				  ,String(fname)(),ex.msg()));
  }
  
  fprintf(FH,"#enter symmetry menu\n@+@\n");
  fprintf(FH,"\n");
  fprintf(FH,"@s@\n@%ld@\n@x@\n",tsym.spacegroup());
  fprintf(FH,"\n");
  fprintf(FH,"@t@\n!%s!\n@x@\n",Symmetry::structureTypes(tsym.structureType()));
  fprintf(FH,"\n");
  fprintf(FH,"@u@\n!%s!\n@x@\n",Xf::lengthUnitName(tsym.lengthUnit()));
  fprintf(FH,"\n");
  fprintf(FH,"@l@%s %s %s\n",
	  tsym.latticeConstantS(0)(),
	  tsym.latticeConstantS(1)(),
	  tsym.latticeConstantS(2)()
	  );
  fprintf(FH,"\n");
  fprintf(FH,"@a@%s %s %s\n",
	  tsym.axesAngleS(0)(),
	  tsym.axesAngleS(1)(),
	  tsym.axesAngleS(2)()
	  );
  fprintf(FH,"\n");
  fprintf(FH,"@g@");
  for(int i=0;i<tsym.subGroupGenerators().size();i++){
    fprintf(FH," %d",tsym.subGroupGenerators().at(i));
    
  }
  fprintf(FH,"\n");
  long nsort=tsym.wyckoffPositions().size();
  fprintf(FH,"@n@%ld\n",nsort);
  for(int issa=0;issa<nsort;issa++){
    fprintf(FH,"@%d@%s@ %s %s %s\n",issa+1,
	    tsym.wyckoffPosition(issa).el()(),
	    tsym.wyckoffPosition(issa).tauS(0)(),
	    tsym.wyckoffPosition(issa).tauS(1)(),
	    tsym.wyckoffPosition(issa).tauS(2)()
	    );
  }
  fprintf(FH,"\n");
  fprintf(FH,"\n");
  fprintf(FH,"#update\n@+@\n");
  fprintf(FH,"#leave symmetry menu\n@x@\n");
  fprintf(FH,"\n");
  fprintf(FH,"\n");
  fprintf(FH,"#quit\n@q@\n");


  


  

  fclose(FH);
  

  
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::on_pbManipulate_toggled(bool state)
{
  QSize s=size();
  if(state){
    savedsize=s;
  }

  // save  geometry
  QRect oldGeo=geometry();
  QSize wMainOldSize=wMain->size();
  //

  
  //myDebug()<<"old wMain       geo"<<wMain->geometry().width();
  //myDebug()<<"old wManipulate geo"<<wManipulate->geometry().width();

  wManipulate->setVisible(state);
  wManipulate->setEnabled(state);

  qApp->processEvents();
  qApp->flush();
  int wMainNewWidth=wMain->size().width();
  int wManiNewWidth=wManipulate->size().width();
  //myDebug()<<"open wMain       geo"<<wMainNewWidth;
  //myDebug()<<"open wManipulate geo"<<wManiNewWidth;

  
  
  
   QRect r=oldGeo;
   int delta=(wMainOldSize.width()-wMainNewWidth);
   //myDebug()<<"delta="<<delta;
   r.setWidth(oldGeo.width()+delta);
   r.setHeight(oldGeo.height());
   setGeometry(r);
  
   wMain->resize(wMainOldSize);
   wManipulate->resize(QSize(wManiNewWidth,
			     wManipulate->geometry().size().height()));

   //myDebug()<<"new wMain       geo"<<wMain->geometry().width();
   //myDebug()<<"new wManipulate geo"<<wManipulate->geometry().width();

  
   if(state)
     wManipulate->onOpen();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::selectWyck(long sort)
{
  
  QModelIndex index=tvWyck->currentIndex();
  int row=sort;
  int col=0;
  if(index !=  QModelIndex()){
    col=index.column();
  }

  tvWyck->scrollTo(model->index(row,0,QModelIndex()));
  tvWyck->setCurrentIndex(model->index(row,col,QModelIndex()));

  {
    QModelIndex index=tvWyckProps->currentIndex();
    int row=sort;
    int col=0;
    if(index !=  QModelIndex()){
      col=index.column();
    }
    
    tvWyckProps->scrollTo(propmodel->index(row,0,QModelIndex()));
    tvWyckProps->setCurrentIndex(propmodel->index(row,col,QModelIndex()));
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::deleteWyckoffPosition(long sort)
{
  selectWyck(sort);
  slotTvWyckDeletePressed();
  onpbAcceptClicked();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::hideWyckoffPosition(long sort)
{
  QModelIndex index=propmodel->index(sort,AtomPropsModel::Visible,QModelIndex());
  if(!index.isValid()) return;
  propmodel->setData(index,false,Qt::CheckStateRole);
  onpbAcceptClicked();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymDlg::updateRequest(const String &what)
{
  if(what=="atomproperties"){
    t_sym->applyTheme(globalproperties->current());
    model->reload();
    propmodel->reload();
  }
}

/*EOF*/ 
