/* @(#)SymDlg.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SYMDLG_H
#define _SYMDLG_H 1

#include <QCloseEvent>
#include "ui_SymDlg.h"
#include "Group.hh"

class QAction;
class QMenu;
namespace XFPLO{
  class Symmetry;
}
class WyckPosModel;
class AtomPropsModel;
class ColorDelegate;
class DoubleSpinBoxDelegate;
class ComboDelegate;
//class LineEditDelegate;
class ElementValidator;
class IntValidator;
class RatValidator;
class ToolButtons;

using namespace XFPLO;

class SymDlg : public ProtoDlg, private Ui::SymDlg
{
  Q_OBJECT
    
    
public:

  enum SymDlgMode{StructureMode=1,FermiSurfaceViewMode=2,FermiSurfaceEditMode=3};

  SymDlg(QWidget *parent,QWidget *base,const char *name);
  ~SymDlg();
  void getData( Symmetry *sym, bool &chn );
  void setData( const Symmetry *symm ,SymDlgMode dlgmode);
  void selectWyck(long sort);  
  void deleteWyckoffPosition(long sort);
  void hideWyckoffPosition(long sort);
  void updateRequest(const String &);
 signals:
  void applyData(SymDlg *,bool);
  void loadFPLOInFile(SymDlg *);
  void saveFPLOInFile(SymDlg *);


private slots:

  void on_cmbCrysSystem_currentIndexChanged(int index);
  void on_cmbBravais_currentIndexChanged(int btype);
  void on_cmbGroup_currentIndexChanged(int index);
  void on_cmbRep_currentIndexChanged(int index);
  void on_rbNumber_toggled(bool onoff);
  void on_spbxGroup_valueChanged(int);
  void on_pbLoadFPLOInFile_clicked();
  void on_pbSaveFPLOInFile_clicked();
  void slotCrystalSystemEnableWidgets(long num);
  void on_cmbStructureType_currentIndexChanged(int stype);

  void slotTvWyckInsertPressed();
  void slotTvWyckDeletePressed();
  void slotTvWyckMovePressed();

  void on_tvWyckProps_clicked(const QModelIndex &index);
  
  void on_pbSubGroupGen_clicked();
  void on_pbSavePipe_clicked();

  void slotCrystalSystemSetRedundant();
  void onDefaultActionTriggered(QAction *a);

  void on_pbManipulate_toggled(bool);
  void onThemeMenuAboutToShow();
  void onThemeMenuAction(QAction *);
  
private:
  void onClose();
  bool isCrystal();
  void setSubGroupGenMarker();
  void convertLatticeParameters(long from,long to);
  bool checkLatticeParameters(QString &txt);
  void adjustTvWyck();
  void adjustTvProps();
  void setRepresentation();
  void writeSymToFile(const Symmetry &tsym,const QString &fname);
  void writeSymPipeInfoToFile(const Symmetry &tsym,const QString &fname);

  bool dataAcceptable();

  void afterAcceptAction(){setRepresentation();}

  bool commitData(Symmetry *sym);
  void saveStuff(const QString &what);
  
private:
  Symmetry *t_sym;
  const Symmetry *psym;
  ValueArray<Operation> pops;
  ValueArray<int> psubgroupgenerators;
  SymDlgMode pdlgmode;
  bool t_silent;
  WyckPosModel *model;
  AtomPropsModel *propmodel;
  // LineEditDelegate *xyzdelegate;
  // LineEditDelegate *eldelegate;
  ElementValidator *elvalid;
  IntValidator *typevalid;
  RatValidator *xyzvalid;
  long t_currentrepindex;
  bool muteCmbRep;
  QSize savedsize;
  ToolButtons *ptoolbuttons;
  QMenu *themeMenu;
  bool mutespbxgroup;
  ColorDelegate *colordelegate;
  ColorDelegate *speccolordelegate;
  ComboDelegate *radtypdelegate;
  DoubleSpinBoxDelegate *shineninessdelegate;
  DoubleSpinBoxDelegate *radiusdelegate;
  bool ptvWyckRowSizeKnown;
  friend class SymManipulateWdg;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SYMDLG_H */

