/*************************************************************************
 * @(#)SymManipulateWdg.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   07 Dec 2011
 *************************************************************************
 */

#include <QMessageBox>
#include "Except.hh"
#include "SymManipulateWdg.hh"
#include "Symmetry.hh"
#include "SymDlg.hh"
#include "Valid.hh"
#include "Config.hh"
#include "MyDebug.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SymManipulateWdg::SymManipulateWdg(QWidget *parent)
  :QWidget(parent)
{
  setupUi(this);
  xyzvalid=new IntValidator(this,0,"",-100000,100000);
  ratvalid=new RatValidator(this,0,"",-100000,100000);
  psymdlg=((SymDlg*)parent);
  leVacAmount->setText("8.0");
  leVacZ0->setText("0.0");
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
SymManipulateWdg::~SymManipulateWdg()
{

//   qDeleteAll(pundostack.begin(), pundostack.end()); 
  pundostack.clear();
  if(xyzvalid) delete xyzvalid;xyzvalid=0;
  if(ratvalid) delete ratvalid;ratvalid=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::onOpen()
{
  pbToP1->setFocus();
  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::communicateback(Symmetry&sym)
{
  sym.setChanged(true);
  psymdlg->setData(&sym,SymDlg::StructureMode);
  psymdlg->onpbAcceptClicked();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::pushUndo(Symmetry&sym)
{
  pundostack.append(sym);
  pbUndo->setEnabled(true);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SymManipulateWdg::getSymmetry(Symmetry &sym)
{
  if(psymdlg->commitData(&sym)){
    sym.setChanged(false);
    pushUndo(sym);
    return true; 
  }else{
    return false;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbUndo_clicked()
{
  if(pundostack.size()>0){
    Symmetry sym=pundostack.takeLast();
    psymdlg->setData(&sym,SymDlg::StructureMode);
    psymdlg->onpbAcceptClicked();
  }
  if(pundostack.size()>0){
    pbUndo->setEnabled(true);
  }else{
    pbUndo->setEnabled(false);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbToP1_clicked()
{
  myDebug()<< "todo: SymManipulateWdg::on_pbToP1_clicked";
  Symmetry sym;
  if(!getSymmetry(sym)) return;

  try{
    sym.convertTo("P1");
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbToConvCell_clicked()
{
  //myDebug()<< "SymManipulateWdg::on_pbToConvCell_clicked";
  Symmetry sym;
  if(!getSymmetry(sym)) return;

  try{
    sym.convertTo("ConvCell");
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbRemoveGenerators_clicked()
{
  //myDebug()<< "SymManipulateWdg::on_pbRemoveGenerators_clicked";

  Symmetry sym;
  if(!getSymmetry(sym)) return;

  try{
    sym.convertTo("RemoveGenerators");
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbDetermineSym_clicked()
{
  //myDebug() << "SymManipulateWdg::on_pbDetermineSym_clicked";
  
  Symmetry sym;
  if(!getSymmetry(sym)) return;

  try{
    sym.convertTo("DetermineSymmetry");
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbIncreaseCell_clicked()
{
  QString m,s,txt;
  Matrix3x3 incrmat;
  for(int i=0;i<3;i++){
    for(int j=0;j<3;j++){
      s=twLatIncr->item(i,j)->data(Qt::DisplayRole).toString();
      IntValidator::stripAllWhiteSpace(s);m="";
      long err=!xyzvalid->check(m,s);
      if(err)
	txt+=QString("Increas lattice(%1,%2): %3").arg(i+1).arg(j+1).arg(m);
      incrmat.data()[j+3*i]=s.toInt();
    }
  }


  if(txt.length()>0){
    QMessageBox::warning ( this,IDS_APP_NAME,txt, 1, 0,0 ) ;
    return;
  }



  Symmetry sym;
  if(!getSymmetry(sym)) return;

  // txt=Sprintf("det=%g nsite=%ld",incrmat.determinant(),sym.nsite())();
  // QMessageBox::warning ( this,IDS_APP_NAME,txt, 1, 0,0 ) ;

  try{
    sym.increaseCell(incrmat);
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SymManipulateWdg::on_pbInsertVac_clicked()
{
  QString m,s,txt;
  Vec3 vacaxis;
  for(int j=0;j<3;j++){
    s=twVacAxis->item(0,j)->data(Qt::DisplayRole).toString();
    IntValidator::stripAllWhiteSpace(s);m="";
    long err=!ratvalid->check(m,s);
    if(err)
      txt+=QString("Vacuum direction(%1): %2").arg(j+1).arg(m);
    String ss(s);
    vacaxis[j]=ss.convertToFortranReal();
    
  }


  s=leVacAmount->text();
  IntValidator::stripAllWhiteSpace(s);m="";
  long err=!ratvalid->check(m,s);
  if(err)
    txt+=QString("Vacuum amount: %1").arg(m);
  double amount=s.toDouble();

  s=leVacZ0->text();
  IntValidator::stripAllWhiteSpace(s);m="";
  err=!ratvalid->check(m,s);
  if(err)
    txt+=QString("Vacuum Z0: %1").arg(m);
  double z0=s.toDouble();

  //myDebug()<<"axis"<<vacaxis;
  //myDebug()<<"amount"<<amount;

  if(txt.length()>0){
    QMessageBox::warning ( this,IDS_APP_NAME,txt, 1, 0,0 ) ;
    return;
  }

  Symmetry sym;
  if(!getSymmetry(sym)) return;

  try{
    sym.insertVacuum(vacaxis,amount,z0);
  }catch(Except &ex){
    QMessageBox::warning(this,ex.title(),ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }
  communicateback(sym);

}

/*EOF*/ 
