/*************************************************************************
 * @(#)ThemeDlg.cpp
 * author: magru@unas
 * date:   15 Jul 2013
 *************************************************************************
 */

#include "ThemeDlg.hh"
#include "Light.hh"
#include "HelpDlg.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ThemeDlg::ThemeDlg(QWidget*parent,Mode mode,LightThemes *lightthemes)
  :QDialog(parent),pmode(mode),plightthemes(lightthemes),
   mute_cmbThemeEdit(false),mute_cmbThemeIndex(false)
{
  setupUi(this);
  if(pmode==Save){
    pbSave->setText("&Save as");
    tbAdd->show();
    cmbTheme->setEditable(true);
  }else if(pmode==Load){
    pbSave->setText("&Load");
    tbAdd->hide();
    cmbTheme->setEditable(false);
  }

  if(plightthemes){
    //myDebug()<<"ThemeDlg::ThemeDlg: plightthemes:\n"<<*plightthemes;
    mute_cmbThemeIndex=true;
    mute_cmbThemeEdit=true;
    cmbTheme->clear();
    const StringList &themes=plightthemes->themes();
    for(int i=0;i<themes.size();i++){
      cmbTheme->insertItem(i,themes.at(i)());
    }
    cmbTheme->setCurrentIndex(plightthemes->currentTheme());
    mute_cmbThemeEdit=false;
    mute_cmbThemeIndex=false;

  }
  
  setTbDeleteEnabled();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ThemeDlg::~ThemeDlg()
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void ThemeDlg::on_pbSave_clicked()
{
  //myDebug()<<"accept";
  accept();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::on_cmbTheme_editTextChanged(const QString &text)
{
  if(mute_cmbThemeEdit) return;
  cmbTheme->setItemText(cmbTheme->currentIndex(),text);
  if(plightthemes){
    (*plightthemes)[cmbTheme->currentIndex()].v1=String(text);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::on_cmbTheme_currentIndexChanged(int index)
{
  if(mute_cmbThemeIndex) return;
  //  myDebug()<<index<<cmbTheme->currentText();
  if(plightthemes){
    plightthemes->setCurrentTheme(index);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::on_tbAdd_clicked()
{
  int ind=cmbTheme->currentIndex();
  if(plightthemes){
    plightthemes->insertNewAt(ind+1,makePair(String("new"),plightthemes->at(ind).v2));
  }
  cmbTheme->insertItem(ind+1,"new");
  cmbTheme->setCurrentIndex(ind+1);

  setTbDeleteEnabled();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::on_tbDelete_clicked()
{
  int ind=cmbTheme->currentIndex();
  if(ind>0){
    if(plightthemes){
      plightthemes->removeAt(ind);
    }
    cmbTheme->removeItem(ind);
  }
  setTbDeleteEnabled();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::setTbDeleteEnabled()
{
  if(cmbTheme->count()>1)
    tbDelete->setEnabled(true);
  else
    tbDelete->setEnabled(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::accept()
{
  //myDebug()<<"ThemeDlg::accept()";
  if(pmode==Save){
    if(plightthemes){
      //myDebug()<<"t_lightthemes:\n"<<*plightthemes;
    }
  }else if(pmode==Load){
    if(plightthemes){
      //myDebug()<<"t_lightthemes:\n"<<*plightthemes;
    }
  }
 
  QDialog::accept();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ThemeDlg::on_tbHelp_clicked()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","ThemeDialog");
  h->show();
  
}
/*EOF*/ 
