/* @(#)ToolBox.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _TOOLBOX_H
#define _TOOLBOX_H 1

#include <QDockWidget>
#include <QMap>

class QRadioButton;
class QToolButton;
class QDoubleSpinBox;
class QAction;
class QMenu;

class ControlBase;
class DocBase;
class QSignalMapper;

class ToolBox : public QDockWidget
{
  Q_OBJECT
 public:
  ToolBox(ControlBase *parent,DocBase *d);
  virtual ~ToolBox();
  
  void init();

  virtual void updateToolBox();
  void updateAxesButtons();
  void updateCellButtons();

private slots:
  void setPerspective(bool );
  void onTbResetCameraClicked(int mode);
  void onTbCamSaveLoadCameraClicked(int mode);
  void onResetKeyAction();
  void onShowhideAxesAction();
  void onShowhideCellsAction();

  void ontbAxesToggled(bool onoff);

  void ondspbxEyeValueChanged(double);
  void ontbXZViewclicked();
  void ontbZXViewclicked();
  void ontbXYViewclicked();
  void ontbYXViewclicked();
  void ontbYZViewclicked();
  void ontbZYViewclicked();
  void ontbXYZViewclicked();

  void ontbDisplayCellsToggled(bool);

  void ontbWireFrameToggled(bool onoff);
  void onTbBoundOnRotateToggled(bool);
  void onTbAAClicked();

 protected:
  QRadioButton *prbPersp,*prbOrtho;
  QToolButton *ptbResetCamera;
  QToolButton *ptbCamSaveLoad;
  QToolButton *ptbCart,*ptbConv,*ptbPrim;
  QToolButton *ptbXYView;
  QToolButton *ptbYXView;
  QToolButton *ptbXZView;
  QToolButton *ptbZXView;
  QToolButton *ptbYZView;
  QToolButton *ptbZYView;
  QToolButton *ptbXYZView;
  QToolButton *ptbWireFrame;
  QToolButton *ptbBoundOnRotate;
  QToolButton *ptbAA;
  QToolButton *ptbBoundCell;
  QToolButton *ptbPrimCell;
  QToolButton *ptbConvCell;
  QDoubleSpinBox *pdspbxEye;
  bool muted;

 private:
  ControlBase *pparent;
  DocBase *pdoc;
  QAction *resetKeyAction;
  QAction *resetaction;
  QAction *centeraction;
  QAction *rescaleaction;
  QAction *camsaveaction;
  QAction *camloadaction;
  QSignalMapper *signalMapper;
  QSignalMapper *camSignalMapper;

  QAction *showhideAxesAction;
  QAction *showhideCellsAction;
  QMenu *showHideAxesMenu;
  QMenu *showHideCellsMenu;
  QMenu *resetViewMenu;
  QMenu *camSaveLoadMenu;


  QMap<QObject*,QAction*> cellMenuActions;
  QMap<QObject*,QAction*> axesMenuActions;
  
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _TOOLBOX_H */

