//todo: first look at space_group fpr HALL
//todo: fi not space_group look for symmetry 1)HALL2)ITnumber+
//todo: if symmetry_IT number or HM look for Symmetry Equiv Pos  _xyz
// and calculate form this, otherwise interprete from HM

//todo: spacegroup: loops, explicite operations
//todo: hydrogen
/* @(#)CIFData.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _CIFDATA_HH
#define _CIFDATA_HH 1

#include <stdlib.h>
#include "MyDebug.hh"
#include "FString.hh"
#include "Except.hh"
#include "Vec3.hh"
#include "ValueArray.hh"
#include "PtrArray.hh"
#include "Pair.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

namespace CIF{

  /* -------------------------------
   * some type specifiers to differentiat CIF data structures 
   * -------------------------------
   */

  enum DataType{None=0,Data=1,Single=2,SaveFrame=3,Table=4};

  /* -------------------------------
   * a dictionary to extract the relevcant data: in our case
   * the data describing the crystal
   * -------------------------------
   */

class CIFMetaDict{
public:
  enum DictKey{
    Single                        =0x00000, // mark single entries
    InTable                       =0x00001, // mark loop_-ed entries
    Category                      =0x00002, /* categories, which may have 
					     * several tags
					     */
    InTableCategory               =0x00003,
    TableMarker                   =0x00004,
    // category cell
    Cell                          =0x00012, 
    CellLength                    =0x01012,
    CellLengthA                   =0x02010,
    CellLengthB                   =0x03010,
    CellLengthC                   =0x04010,

    CellAngle                     =0x05012,
    CellAngleAlpha                =0x06010,
    CellAngleBeta                 =0x07010,
    CellAngleGamma                =0x08010,

    CellVolume                    =0x09010,
    CellFormulaUnitsZ             =0x0a010,
    CellSpecialDetails            =0x0b010,

    // category chemical
    Chemical                      =0x00022,
    ChemicalName                  =0x01022,
    // _chemical_name_common
    // _chemical_name_mineral
    // _chemical_name_structure_type
    // _chemical_name_systematic
    ChemicalFormula               =0x02022,
    

    // category space_group
    SpaceGroup                    =0x00032,
    SpaceGroupITNumber            =0x01030,
    SpaceGroupNameHMFull          =0x02030,
    SpaceGroupNameHMAlt           =0x03030,
    SpaceGroupNameHall            =0x04030,
    SpaceGroupCrystalSystem       =0x05030,
    SpaceGroupSymop               =0x06034,
    SpaceGroupSymopId             =0x07031,
    SpaceGroupSymopOperationXYZ   =0x08031,

 
    // category symmetry //old style
    Symmetry                      =0x00042,
    SymmetryIntTablesNumber       =0x01040,
    SymmetryCellSetting           =0x02040,
    SymmetrySpaceGroupNameHM      =0x03040,
    SymmetrySpaceGroupNameHall    =0x04040,
    SymmetryEquivPos              =0x05044,//table Marker
    SymmetryEquivPosSiteId        =0x06041,
    SymmetryEquivPosAsXyz         =0x07041,
    // table atom_site_, the first is the supposed table marker
    AtomSite                      =0x00054, //table marker
    AtomSiteLabel                 =0x01051,
    AtomSiteTypeSymbol            =0x02051,
    AtomSiteFractX                =0x03051,
    AtomSiteFractY                =0x04051,
    AtomSiteFractZ                =0x05051,
    AtomSiteOccupancy             =0x06051,
    AtomSiteWyckoffSymbol         =0x07051,
    AtomSiteSymmetry              =0x08053, //category in table
    AtomSiteSymmetryMultiplicity  =0x09051
 
  };
private:
  /* -------------------------------
     ------------------------------- */
  class DictEntry{
  public:
    DictEntry(){};
    DictEntry(DictKey key):pkey(key){};
    DictEntry(DictKey key,const String &cifname):pkey(key),pcifname(cifname){};

    // for finding records
    bool operator==(const DictEntry &other)const{
      return (pkey==other.pkey);
    };
      
  public:
    DictKey pkey;
    String pcifname;
  };
  /* -------------------------------
     ------------------------------- */
public:
  CIFMetaDict(){setDefault();};
  ~CIFMetaDict(){};
  void setDefault();

  String value(DictKey key)const;
  String subValue(DictKey key,DictKey categ)const;
  
private:
  ValueArray<DictEntry> pentries;
  friend MyDebug operator<<(MyDebug dbg,const CIFMetaDict::DictEntry &v);
};


  inline MyDebug operator<<(MyDebug dbg,const CIFMetaDict::DictEntry &){
    dbg<<"DictEntry";
    return dbg;
  }

  /* -------------------------------
   * The class storing the complete CIF file
   * -------------------------------
   */

class CIFData{
public:
  
  /* --------------------------
   * Storing a single value
   * --------------------------
   */
  class Value{
  public:
    enum ValueType{None=0,Character=1,Long=2,Double=3,
		   CharacterString=4};
    enum DelimiterType{Unquoted=1,SingleQuotes=2,DoubleQuotes=3,TextField=4};
    Value():ptype(None){};
    Value(char c):ptype(Character),pchar(c){};
    Value(long l):ptype(Long),plong(l){};
    Value(const String &d,const String &unc):ptype(Double),pstring(d),
					     puncertainty(unc){};
    Value(const String &s,DelimiterType delim)
      :ptype(CharacterString),pstring(s),pdelim(delim){};

    double D()const{
      if(ptype==Long){
	return (double)plong;
      }else if(ptype==CharacterString||ptype==Double){
	return pstring.convertToReal();
      }else {
	throw Except("value cannot be converted to double");
      }
    };
    double L()const{
      if(ptype!=Long)
	throw Except("value is not a long");
      return plong;
    };
    const String &S()const{
      if(ptype!=CharacterString)
	throw Except("value is not a characterstring");
      return pstring;
    };
      
    String asString()const;
    String printS()const;
    void print()const;
  private:
    ValueType ptype;
    char pchar;
    long plong;
    String pstring;
    String  puncertainty;
    DelimiterType pdelim;
  };

  /* --------------------------
   * a table aka loop_
   * --------------------------
   */
  
  class DataTable{
  public:
    DataTable():pcount(0){};

    inline long rows()const{return pvalues.size();};
    inline long columns()const{return ptags.size();};

    inline long indexOf(const String &tag)const{
	return ptagsnormalized.indexOf(tag);};

    inline bool contains(const String &tag)const{
      return ptagsnormalized.contains(tag);
    }
    inline bool containsCategory(const String &tag)const{
      long L=tag.length()-1;
      for(long i=0;i<ptagsnormalized.size();i++){
	if( (ptagsnormalized.at(i).subString(0,L)==tag)) return true;
      }
      return false;
    }

//     inline const String tag(long i)const{
//       return ptags.at(i)->lowerCase().replace(".","_",0L,true);};
    inline const String normalizedTag(long i)const{
      return ptagsnormalized.at(i);};


    inline const Value &value(long irow,long itag)const{
      return *(pvalues.at(irow)->at(itag)); };

    String commonCategory()const;

    String atomLabel(long i,const CIFMetaDict &valuedict)const;
    String atomWyckoffSymbol(long i,const CIFMetaDict &valuedict)const;
    long atomSymmetryMultiplicity(long i,const CIFMetaDict &valuedict)const;
    double atomOccupancy(long i,const CIFMetaDict &valuedict)const;
    Vec3 atomFracPosition(long i,const CIFMetaDict &valuedict)const;


    inline void appendTag(String *name){
      ptags.append(name);
      ptagsnormalized.append(String(*name)
			     .lowerCase().replace(".","_",0L,true));
    }; 
    void appendValue(Value *v);

    String printS()const;
    void print()const;

  private:
    long pcount;
    PtrArray<String> ptags;
    ValueArray<String> ptagsnormalized;
    PtrArray<PtrArray<Value> > pvalues;
  };

  /* --------------------------
   * an allowed component of a data block (aka data_)
   * this can be saveframes, single tag-value pairs or loops
   * --------------------------
   */
  class DataBlock;

  class DataBlockItem{
  public:
    DataBlockItem(); // disable default contructor
    DataBlockItem(DataType t):ptype(t),pname(0),pvalue(0),psaveframe(0){};
    //
    // production routines
    // a single tag-value pair
    static DataBlockItem *NewSaveFrame(String *name);
    // a saveframe
    static DataBlockItem *NewSingleTagValuePair(String *name,Value *v);
    // a table (loop_)
    static DataBlockItem *NewTable();
    // die
    ~DataBlockItem();

    inline DataType type()const{return ptype;};
    inline const String nameLC()const {
      return pname->lowerCase().replace(".","_",0,true);};
    inline const Value &value()const {return *pvalue;};
    inline const DataTable *table()const{return &ptable;};
    inline DataTable *table(){return &ptable;};
    inline DataBlock *saveFrame(){return psaveframe;};
    inline const DataBlock *saveFrame()const{return psaveframe;};

    String printS()const;
    void print()const;

  private:
    DataType ptype;
    String *pname;
    Value *pvalue;
    DataTable ptable;
    DataBlock *psaveframe;
    
  };

  /* --------------------------
   * the basic CIF record or data block aka data_, it is also used
   * to store saveframes
   * --------------------------
   */
  
  class DataBlock{
  public:
    DataBlock(); //disable default contructor
    DataBlock(DataType t,String *name):ptype(t),pname(name){};
    ~DataBlock(){if(pname)delete pname;};

    long numItems()const{return pdatablockitems.size();};
    const DataBlockItem *at(long i)const{return pdatablockitems.at(i);};

    inline void append(DataBlockItem *dbi){pdatablockitems.append(dbi);};


    inline const String nameLC()const {return pname->lowerCase();};

    const DataBlockItem *hasSingle1(const String &cifname)const;
    
    Vec3 latticeConstants(const CIFMetaDict &valuedict)const;
    Vec3 axesAngles(const CIFMetaDict &valuedict)const;
    double cellVolume(const CIFMetaDict &valuedict)const;
    long cellFormulaUnits(const CIFMetaDict &valuedict)const;
    String cellSpecialDetails(const CIFMetaDict &valuedict)const;

    String crystalSystem(const CIFMetaDict &valuedict)const;
    long spaceGroupNumber(const CIFMetaDict &valuedict)const;
    String spaceGroupNameHall(const CIFMetaDict &valuedict)const;
    String spaceGroupNameHM(const CIFMetaDict &valuedict)const;
    ValueArray<Pair<long,String> > 
    spaceGroupGeneratorsXYZ(const CIFMetaDict &valuedict)const;

    const DataTable *atomTable(const CIFMetaDict &valuedict)const;

    ValueArray<Pair<String,String> > 
    allOfSingleCategoryAsString(const String &cifcategory)const;

    ValueArray<Pair<String,String> > 
    allOfSingleCategoryAsString(const CIFMetaDict &valuedict,
				CIFMetaDict::DictKey key)const{
      return allOfSingleCategoryAsString(valuedict.value(key));
    };

    

    String printS()const;
    void print()const;

  private:
    DataType ptype;
    String *pname;
    PtrArray<DataBlockItem> pdatablockitems;
  };

  /* --------------------------
   * The CIF content, a collection of datablocks
   * --------------------------
   */

  CIFData();
  ~CIFData(){};


  void clear(){pdatablocks.clear();};


  ValueArray<String> dataBlockNames()const{
    ValueArray<String> n;
    for(long i=0;i<pdatablocks.size();i++){
      n<<pdatablocks.at(i)->nameLC();
    }
    return n;
  }

  const DataBlock *dataBlock(const String &datablockname)const{
    for(long i=0;i<pdatablocks.size();i++){
      if(pdatablocks.at(i)->nameLC()==datablockname){
	return pdatablocks.at(i);
      }
    }
     throw Except(Sprintf("no datablock with name \"%s\"",datablockname()));
  }
  
  DataBlock *appendNewDataBlock(String *name){
    pdatablocks.append(new DataBlock(Data,name));
    return pdatablocks.last();
  };

  CIFMetaDict metaDict(){return pvaluedict;};

  String printS()const;
  void print()const;

protected:
private:
  PtrArray<DataBlock> pdatablocks;
  CIFMetaDict pvaluedict;
};

}

inline MyDebug operator<<(MyDebug dbg,const CIF::CIFData::Value &s){
  fprintf(dbg.FH,"%s",s.printS()());
  return dbg;
}




#endif /* _CIFDATA_HH */

