/* @(#)CIFDriver.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _CIFDRIVER_HH
#define _CIFDRIVER_HH 1

#include <stdio.h>
#include <string>
#include "FString.hh"
#include "CIFData.hh"
#include "CIFParse.hpp"
#include "CIFParseDecl.hh"

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* ***********************************************************************
 *
 * ***********************************************************************
 */


class CIFDriver{
public:


  CIFDriver();
  ~CIFDriver();

  bool traceScanning()const{return ptracescanning;};
  void setTraceScanning(bool t){ptracescanning=t;};

  bool traceParsing()const{return ptraceparsing;};
  void setTraceParsing(bool t){ptraceparsing=t;};

  // Run the parser.  Return 0 on success.
  void parse(CIF::CIFData *cifdata,const char *bufferorfilename,
	    bool frombuffer=false);
  void parse(CIF::CIFData *cifdata,const String bufferorfilename,
	    bool frombuffer=false){
    parse(cifdata,bufferorfilename(),frombuffer);
  }

  //const String &errorMsg()const{return ciferrormsg;};

protected:
private:
  std::string &file(){return pfilename;};
  void scan_begin ();
  void scan_end ();
  // Error handling.
  void error(const CIF::location& l, const std::string& m);
  void error(const std::string& m);
private:
  bool ptracescanning;
  bool ptraceparsing;
  void *pscanner;
  FILE *pFH;
  const char *pbuffer;
  std::string pfilename;
  bool pfrombuffer;
  String ciferrormsg;
  YY_BUFFER_STATE lexstate;
  CIF::CIFData *pcifdata;
  CIF::CIFData::DataBlock *pdatablock;
  CIF::CIFData::DataBlock *psaveframe;
  CIF::CIFData::DataTable *ptable;
  friend class CIF::CIFParser;
  friend YY_DECL;
};


#endif /* _CIFDRIVER_HH */

