/*************************************************************************
 * @(#)IMatrix.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   11 Sep 2012
 *************************************************************************
 */

#include "IMatrix.hh"
#include "FMatrix.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
FMatrix IMatrix::diagInverseT(long rank)const{
  //invert diagonal up to rank and transpose
  FMatrix res(Td());
  long n=MyMath::min3<long>(rank,prows,pcols);
  for(long r=0;r<n;r++){
    res.p[r+pcols*r]=Fractional(1,p[r+prows*r]);
  }
  return res;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
IMatrix IMatrix::inversetV(long &idet)const
{
  // return inverse * |V|
  if(prows!=pcols){
    throw Except("IMatrix::inverse(): not square");
  }
  if(prows==3){
    IMatrix m(3,3);
    m.p[0]=p[4]*p[8]-p[5]*p[7];
    m.p[3]=p[5]*p[6]-p[3]*p[8];
    m.p[6]=p[3]*p[7]-p[4]*p[6];
    idet=m.p[0]*p[0]+m.p[3]*p[1]+m.p[6]*p[2];
    long i=1;
    if(idet<0){
      i=-1;
      idet*=i;
      m.p[0]*=i;
      m.p[3]*=i;
      m.p[6]*=i;
    }
    m.p[1]=i*(p[7]*p[2]-p[8]*p[1]);
    m.p[4]=i*(p[8]*p[0]-p[6]*p[2]);
    m.p[7]=i*(p[6]*p[1]-p[7]*p[0]);
    
    m.p[2]=i*(p[1]*p[5]-p[2]*p[4]);
    m.p[5]=i*(p[2]*p[3]-p[0]*p[5]);
    m.p[8]=i*(p[0]*p[4]-p[1]*p[3]);
    
    return m;
  
  }else{
    IMatrix m;
    throw Except("IMatrix::inverse(): not implemented");
    return m;
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
IMatrix operator*(const IMatrix &oi,const IMatrix &of){// mat product
  if(oi.pcols!=of.prows)
    throw Except("IMatrix:: operator*(IMatrix,IMatrix): "
		 "incompatible dimensions");
  IMatrix res(oi.prows,of.pcols);
  for(long r=0;r<oi.prows;r++){
    for(long c=0;c<of.pcols;c++){
      long d=0;
      for(long k=0;k<oi.pcols;k++){
	d+=of.p[k+of.prows*c]*oi.p[r+oi.prows*k];
      }
      res.p[r+oi.prows*c]=d;
    }
  }

  return res;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
Fractional IMatrix::dot(const FMatrix &other)const
{
  if(pcols!=1|| other.pcols!=1)
    throw Except("IMatrix::dot: dot product only for Nx1 matrices!");
  if(prows!=other.prows)
    throw Except("IMatrix::dot: incompatible dimensions!");
  
  Fractional d=0;
  for(long r=0;r<prows;r++){
    d+=other.p[r]*p[r];
  }
  return d;
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool IMatrix::operator==(const FMatrix &o)const
{
  if(prows!=o.prows || pcols!=o.pcols)
    return false;
  for(long i=0;i<pcols*prows;i++){
    if(o.p[i]!=p[i])return false;
  }
  return true;
}

/*EOF*/ 
