/*************************************************************************
 * @(#)SqMatrix.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Mar 2010
 *************************************************************************
 */

#include "SqMatrix.hh"
#include "ValueArray.hh"



/* ***********************************************************************
 *
 * ***********************************************************************
 */

void printSqMatrix(const SqMatrix<long> &m,const char*fmt)
{
  for(long i=0;i<m.size(1);i++){
    for(long j=0;j<m.size(2);j++){
      printf(fmt,m(i,j));
    }
    printf("\n");
  }
}

void printSqMatrix(const SqMatrix<int> &m,const char*fmt)
{
  for(long i=0;i<m.size(1);i++){
    for(long j=0;j<m.size(2);j++){
      printf(fmt,m(i,j));
    }
    printf("\n");
  }
}

void printSqMatrix(const SqMatrix<double> &m,const char*fmt,
		   const char*prefix)
{
  for(long i=0;i<m.size(1);i++){
    printf("%s ",prefix);
    for(long j=0;j<m.size(2);j++){
      printf(fmt,m(i,j));
    }
    printf("\n");
  }
}



void printSqMatrix(const SqMatrix<Cmplx> &m,const char*fmt)
{
  for(long i=0;i<m.size(1);i++){
    for(long j=0;j<m.size(2);j++){
      printf(fmt,m(i,j).real(),m(i,j).imag());
      
    }
    printf("\n");
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

ValueArray<double> operator*(const SqMatrix<double> &m,
			     const ValueArray<double> &v){
  if(m.size(2)!=v.size()){
    printf("SqMatrix::operator*(SqMatrix,ValueArray) different dimension\n");
    exit(1);
  }
  ValueArray<double> res;
  res.resize(m.size(1));
  for(int i=0;i<m.size(1);i++){
    double d=0.;
    for(int j=0;j<v.size();j++){
      d=d+m(i,j)*v.at(j);
    }
    res[i]=d;
  }
  return res;
}

ValueArray<double> operator*(const ValueArray<double> &v,
			     const SqMatrix<double> &m){
  if(m.size(1)!=v.size()){
    printf("SqMatrix::operator*(ValueArray,SqMatrix) different dimension\n");
    exit(1);
  }
  ValueArray<double> res;
  res.resize(m.size(2));
  for(int j=0;j<m.size(2);j++){
    double d=0.;
    for(int i=0;i<v.size();i++){
      d=d+v.at(i)*m(i,j);
    }
    res[j]=d;
  }
  return res;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
SqMatrix<double> imag(const SqMatrix<Cmplx> &o)
{
  SqMatrix<double> res(o.size(1),o.size(2));
  for(long i=0;i<o.size(1);i++){
    for(long j=0;j<o.size(2);j++){
      res(i,j)=o(i,j).imag();
    }
  }
  return res;
}
SqMatrix<double> real(const SqMatrix<Cmplx> &o)
{
  SqMatrix<double> res(o.size(1),o.size(2));
  for(long i=0;i<o.size(1);i++){
    for(long j=0;j<o.size(2);j++){
      res(i,j)=o(i,j).real();
    }
  }
  return res;
}



/*EOF*/ 
