/*************************************************************************
 * @(#)Vec3.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Jul 2009
 *************************************************************************
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Vec3.hh"
#include "MyMath.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Vec3::print(const char *name,const char *delim) const
{
  printf("%s=%g%s%g%s%g\n",name,p[0],delim,p[1],delim,p[2]);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
#if (defined QXFSF)

QString Vec3::printQ(const char *name,const char *delim) const
{
  return QString().sprintf("%s=%g%s%g%s%g",name,p[0],delim,p[1],delim,p[2]);
}
#endif
String Vec3::printS(const String &fmt,const String &delim,bool contract) const
{

  if(contract){
    
    return String::doubleToString(fmt(),p[0],contract)
      +delim+String::doubleToString(fmt(),p[1],contract)
      +delim+String::doubleToString(fmt(),p[2],contract);
    
    
  }else{
    return Sprintf((fmt+delim+fmt+delim+fmt)(),p[0],p[1],p[2]);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Vec3::Vec3(const String s[3])
{
  p[0]=String(s[0]).convertToFortranReal();
  p[1]=String(s[1]).convertToFortranReal();
  p[2]=String(s[2]).convertToFortranReal();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Vec3::Vec3(const StringList sl)
{
  if(sl.size()>=1) p[0]=String(sl[0]).convertToFortranReal();
  if(sl.size()>=2) p[1]=String(sl[1]).convertToFortranReal();
  if(sl.size()>=3) p[2]=String(sl[2]).convertToFortranReal();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

Vec3::Vec3(const String &x,const String &y,const String &z)
{
  p[0]=String(x).convertToFortranReal();
  p[1]=String(y).convertToFortranReal();
  p[2]=String(z).convertToFortranReal();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Vec3::lt(const Vec3 &other,double tol) const
{
  if( p[0]>other.p[0]+tol ) return false;
  if( p[0]<other.p[0]-tol ) return true;

  // p[0] equals
  if( p[1]>other.p[1]+tol ) return false;
  if( p[1]<other.p[1]-tol ) return true;

  // p[0] and p[1] equals
  
  if( p[2]>other.p[2]+tol ) return false;
  if( p[2]<other.p[2]-tol ) return true;
  
  // all equal
  return false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Vec3::neq(const Vec3 &other,double tol) const
{
  return (
	  (fabs(p[0]-other.p[0])>tol)||
	  (fabs(p[1]-other.p[1])>tol)||
	  (fabs(p[2]-other.p[2])>tol) );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool Vec3::eq(const Vec3 &other,double tol) const
{
  return (
	  (fabs(p[0]-other.p[0])<=tol)&&
	  (fabs(p[1]-other.p[1])<=tol)&&
	  (fabs(p[2]-other.p[2])<=tol) );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void Vec3::findPerpDirections(Vec3 &x,Vec3 &y)const
{
  
  Vec3 v1=Vec3(1,0,0).cross(*this);
  Vec3 v2=Vec3(0,1,0).cross(*this);
  Vec3 v3=Vec3(0,0,1).cross(*this);
  double d1=norm(v1);
  double d2=norm(v2);
  double d3=norm(v3);
  x=v1;
  if(d1>d2){
    if(d1<d3){
      x=v3;
    }
  }else{
    if(d2>d3){
      x=v2;
    }else{
      x=v3;
    }
  }
  x.normalize();
  y=(x.cross(*this)).normalize();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList Vec3::toString(long mmax) const
{
  StringList sl;
  sl.preallocate(3);
  
  sl.append(String::doubleToRatio(p[0],mmax,1.e-5));
  sl.append(String::doubleToRatio(p[1],mmax,1.e-5));
  sl.append(String::doubleToRatio(p[2],mmax,1.e-5));
  return sl;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
StringList Matrix3x3::toString(long mmax) const
{
  StringList sl;
  sl.preallocate(9);
  for(int i=0;i<9;i++){
    sl.append(String::doubleToRatio(p[i],mmax,1.e-5));
  }
  return sl;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Matrix3x3::print(const char *name,const char *delim) const
{
  int ln=strlen(name);
  printf("%*s=% -g%s% -g%s% -g\n"
	 "%*s % -g%s% -g%s% -g\n"
	 "%*s % -g%s% -g%s% -g\n"
	 ,ln,name
	 ,p[0],delim,p[3],delim,p[6]
	 ,ln," "
	 ,p[1],delim,p[4],delim,p[7]
	 ,ln," "
	 ,p[2],delim,p[5],delim,p[8]
	 );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
#if (defined QXFSF)
QString Matrix3x3::printQ(const char *name,const char *delim) const
{
  int ln=strlen(name);

  return QString().sprintf("%*s=% -g%s% -g%s% -g\n"
	 "%*s % -g%s% -g%s% -g\n"
	 "%*s % -g%s% -g%s% -g\n"
	 ,ln,name
	 ,p[0],delim,p[3],delim,p[6]
	 ,ln," "
	 ,p[1],delim,p[4],delim,p[7]
	 ,ln," "
	 ,p[2],delim,p[5],delim,p[8]
	 );
}
#endif

String Matrix3x3::printS(const String &fmt,const String &delim) const
{
  String f=fmt+delim+fmt+delim+fmt+"\n"+fmt+delim+fmt+delim+fmt+"\n"
    +fmt+delim+fmt+delim+fmt;
  return Sprintf(f()
		 ,p[0],p[3],p[6]
		 ,p[1],p[4],p[7]
		 ,p[2],p[5],p[8]
		 );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Matrix3x3::Matrix3x3(const Vec3 v0,const Vec3 v1,const Vec3 v2)
{
  memcpy(p  ,v0(),sizeof(double)*3);
  memcpy(p+3,v1(),sizeof(double)*3);
  memcpy(p+6,v2(),sizeof(double)*3); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Matrix3x3 Matrix3x3::inverseT(bool &indef,double &det) const
{
  
  Vec3 v0=Vec3(p+3).cross(p+6);
  Vec3 v1=Vec3(p+6).cross(p  );
  Vec3 v2=Vec3(p  ).cross(p+3);

  det=v0*Vec3(p);
  
  if(det == 0.) {
    indef=true;
    return Matrix3x3();
  }
  indef=false;
  
  
  return Matrix3x3(v0/det,v1/det,v2/det);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
// const Matrix3x3 &Matrix3x3::invertT(bool &indef)
// {
  
//   Vec3 v0=Vec3(p+3).cross(p+6);
//   Vec3 v1=Vec3(p+6).cross(p  );
//   Vec3 v2=Vec3(p  ).cross(p+3);

//   double det=v0*Vec3(p);
  
//   if(det == 0.) {
//     indef=true;
//     return *this;
//   }
//   indef=false;
  
//   memcpy(p  , (v0/det)(),sizeof(double)*3);
//   memcpy(p+3, (v1/det)(),sizeof(double)*3);
//   memcpy(p+6, (v2/det)(),sizeof(double)*3);
  
//   return *this;
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const Matrix3x3 &Matrix3x3::transpose()
{
  
  double d;
  d=p[3];p[3]=p[1];p[1]=d;
  d=p[6];p[6]=p[2];p[2]=d;
  d=p[5];p[5]=p[7];p[7]=d;
  return *this;
}

/* ***********************************************************************
 * does line c1-c2 cross d1-d2 ?
 * point on line is r=c1+x*(c2-c1)
 * ***********************************************************************
 */
bool lineCross3(const Vec3 &c1,const Vec3 &c2,const Vec3 &d1,const Vec3 &d2
		,double tol)
{
  // c1+x(c2-c1)=d1+y(d2-d1)
  // (c2-c1)x-(d2-d1)y=d1-c1
  Vec3 b=d1-c1;
  Vec3 r1=c2-c1;
  Vec3 r2=d1-d2;
  bool indef;
  Matrix3x3 m=Matrix3x3(r1,r2,r1.cross(r2)).inverseT(indef);
  if(indef) return false; // parallel lines are considered non crossing
  if(fabs(m(2)*b)>tol) return false; // lines do only cross in projection

  double x=m(0)*b;
  double y=m(1)*b;
  
  return (  0. < x && x < 1. && 0.< y && y < 1.);
}


/* ***********************************************************************
   !!***********************************************************************
   !! provides the euler angles, which turn the coordinat system
   !! such, that the vector (nz) is the new z-axis
   !! and that the vector (nx) is the new x-axis
   !! the x-axis is orthogonalized to the new z-axis
   !! so that input errors with respect to the orthogonality are
   !! removed in a defined way.
   !! The euler angles are defined as:
   !!  a rotation is described by rotating around z-axis about alpha
   !!  followed by rotation around new y-axis about beta
   !!  followed by rotation around new z-axis about gamma
   !!  positive angles mean clockwise rotation, when viewed into axis
   !!  direction. If the axis is an arrow, then positive rotation
   !!  is like turning a screw
   !!***********************************************************************
   * ***********************************************************************
 */
Vec3 Vec3::eulerAngles(const Vec3 &nz,const Vec3 &nx)
{
  Vec3 eu;

  
  double tol=5.e-15;

  // normalize z-axis
  Vec3 Z=nz;
  Z.normalize();

  eu[2]=0.;
  eu[1]=acos(MyMath::min<double>(MyMath::max<double>(-1.,Z[2]),1.));
  eu[0]=atan2(Z[1],Z[0]);

  // normalize x-axis
  //double rbig=norm(nx);
  Vec3 X=nx;
  X.normalize();

  // orthogonalize x_new to z_new
  double rbig=X*Z;
  X=(X-Z*rbig).normalize();

  //
  if (X*Z > tol){
    printf(": xz-orthogonality violated !");
    exit(1);
  }
  // intermediate x-axis is
  //     (cos(eu[0])cos(eu[1]),sin(eu[0])cos(eu[1]),-sin(eu[1]))
  // x_new scalar_product intermed x is cos(eu[2])
  eu[2]=acos(MyMath::max<double>(-1.0,MyMath::min<double>(1.0,cos(eu[1])*(cos(eu[0])*X[0]+sin(eu[0])*X[1])
			    -sin(eu[1])*X[2])));
  // x_new* y_intermed
  double s=-sin(eu[0])*X[0]+cos(eu[0])*X[1];
  if(s < 0.0)eu[2]=twopi-eu[2];

  return eu;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

Vec3 Vec3::eulerAngles(const Vec3 &nz)
{
  Vec3 eu;

  
  //  double tol=5.e-15;

  // normalize z-axis
  Vec3 Z=nz;
  Z.normalize();

  eu[2]=0.;
  eu[1]=acos(MyMath::min<double>(MyMath::max<double>(-1.,Z[2]),1.));
  eu[0]=atan2(Z[1],Z[0]);

  return eu;
}


/* ***********************************************************************
 *
 !! makes matrices from euler angles
 !! the returned matrix M_ik is defined as <e_i|\hat{D}|e_k>
 !!                     which is equivalent to e_i*e_k^\prime 
 !! and transforms the unit vectors of a system into the unit vectors of a 
 !! new system according to
 !!     e_k^\prime = e_i M_ik
 *
 * ***********************************************************************
 */
Matrix3x3 Vec3::eulerRMatrix()
{

  double sa=sin(p[0]);
  double sb=sin(p[1]);
  double sg=sin(p[2]);
  double ca=cos(p[0]);
  double cb=cos(p[1]);
  double cg=cos(p[2]);

  return Matrix3x3(
		   ca*cb*cg-sa*sg ,    sa*cb*cg+ca*sg,       -sb*cg,
		   -ca*cb*sg-sa*cg,   -sa*cb*sg+ca*cg,        sb*sg,
		   ca*sb          ,             sa*sb,           cb
		   );
      
}
