/* @(#)cio.c
 */
/* ***********************************************************************
 * wrapper for  f90 to C calls
 * dont use them in c code !!!
 * ***********************************************************************
 */

#include <stdlib.h>
#include <string.h>
#include "fortrantypes.h"
#include "cio.h"
#include "tparser.h"
#include "allocate.h"
#include "merror.h"
#include "ioparser.h"


/* global variable parser_control for the fortran interface */
PARSER_CONTROLL fortran_pc;

/* ***********************************************************************
 * allocate a c-strign and copy fortran character variable into this strng
 * ***********************************************************************
 */
char *f2cstrcpy(const char *src,long src_len)
{
  char *subfunc="f2cstrcpy";
  long l=src_len;
  char *buffer;
  
  while(--l>=0 && src[l]==' ');
  l++;
  if((buffer=allocate(sizeof(char)*(l+1),"f2cstrcpy",subfunc))==NULL)
    return NULL;
  (void)strncpy(buffer,src,l);
  buffer[l]='\0'; 
  return buffer;
}

/* ***********************************************************************
 * copy c-string src into fortran string dest of length dest_len
 * ***********************************************************************
 */
long c2fstrncpy(char *dest,const char *src,long dest_len,long abort)
{
  long i,l=strlen(src);

  if(dest_len<l){
    (void)sprintf(errorbuffer,"error(c2fstrncpy): string longer (%ld) than \n"
	    "fortran character variable (%ld)!",l,dest_len);
    if(abort){
      (void)fprintf(stderr,"fatal %s\n",errorbuffer);
      (void)fprintf(stdout,"fatal %s\n",errorbuffer);
      exit(2);
    }
    return 0;

  }
  for(i=0;i<dest_len;i++){
    if(i<l)
      dest[i]=src[i];
    else
      dest[i]=' ';
  }
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cvarexists)(const char *name,FPLOLOGICAL *exists
		       ,FPLOLOGICAL *err,char *errormsg
		       ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER err_len)
{
  char *p;
  tsNAMSCAN namescan;

  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;


  if(!getNameScan(&fortran_pc.table,p
		  ,(long) (VTYPE_ARRAY | VTYPE_SCALAR 
			   | VTYPE_STRUCT | VTYPE_STRUCTARRAY)
		  ,0L,&namescan,0L)){
    *exists=L_FALSE;
  }else{
    *exists=L_TRUE;
  }

  cleanup_namescan(&namescan);


  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetlogarray)(const char *name,FPLOLOGICAL *result,FPLOINTEGER *rank
			 ,const FPLOINTEGER *size
			 ,FPLOLOGICAL *err,char *errormsg
			 ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;

  if(!getValue(&fortran_pc.table,p,result,(long)(VTYPE_LOGICAL),0L
	       ,*rank,size,0L))goto error;

  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetrealarray)(const char *name,FPLODOUBLEPRECISION *result,FPLOINTEGER *rank
			  ,const FPLOINTEGER *size
			  ,FPLOLOGICAL *err,char *errormsg
			  ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;

  if(!getValue(&fortran_pc.table,p,result,(long)(VTYPE_REAL),0L
	       ,*rank,size,0L))goto error;

  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetintarray)(const char *name,FPLOINTEGER *result,FPLOINTEGER *rank
			 ,const FPLOINTEGER *size
			 ,FPLOLOGICAL *err,char *errormsg
			 ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;

  if(!getValue(&fortran_pc.table,p,result,(long)(VTYPE_INT),0L
	       ,*rank,size,0L))goto error;

  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetchararray)(const char *name,char *result,FPLOINTEGER *scalartype,
			  FPLOINTEGER *rank
			  ,const FPLOINTEGER *size
			  ,FPLOLOGICAL *err,char *errormsg
			  ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER result_len,
			  INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;
  
  if(!getValue(&fortran_pc.table,p,result,(long)(*scalartype),(long)result_len
	       ,*rank,size,0L))goto error;
  
  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(cgetrealchararray)(const char *name,char *result,FPLOINTEGER *rank
			      ,const FPLOINTEGER *size
			      ,FPLOLOGICAL *err,char *errormsg
			      ,INTERNAL_INTEGER name_len,
			      INTERNAL_INTEGER result_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;
  
  if(!getValue(&fortran_pc.table,p,result,(long)(VTYPE_REAL),(long)result_len
	       ,*rank,size,1L))goto error;
  
  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}

/* ***********************************************************************
 * an data array of up to 5 dimensions may have variable sizes,
 * if defined using structarrays
 * this procedure is called with an FPLOINTEGER array named sizearray
 * of rank=sarank  with sizes= sasize[1:sarank]
 *
 * the result arraysize contains the individual sizes for the last
 * array 'a' specified in the nested structure 's.s.a'
 * 
 * it can not be called for scalars and rank one arrays
 * ***********************************************************************
 * Example 1: struct {real a;}s; defines an scalar s.a of type real
 *
 * this throws an error message, to avoid confusions, defining sizearrays
 * for pure scalars !!!!!
 * ***********************************************************************
 * Example 2: struct {real a[*];}s; defines an array s.a of type real
 * with rank=1, 
 *      size={SIZE(s.a)};  
 *
 * do i=1,size[1]
 *   print *,a(i)
 * enddo
 *
 * this throws an error message, to avoid confusions, defining sizearrays
 * for nonexisting dimensions 
 * ***********************************************************************
 * Example 1: struct {real a;}s[3]; defines an array s.a of type real
 * with rank=1, 
 *      size={3};  
 *
 * do i=1,size[1]
 *   print *,a(i)
 * enddo
 *
 * this throws an error message, to avoid confusions, defining sizearrays
 * for nonexisting dimensions 
 * ***********************************************************************
 * Example 2: struct {real a[*];}s[3]; defines an array s.a of type real
 * with rank=2, 
 *      size={max(SIZE(s[].a)),3};
 * sizearray has sarank=1, 
 *               sasize={3};
 * sizearray={SIZE(s[1].a),SIZE(s[2].a),SIZE(s[3].a)};
 *
 * do j=1,size[2]
 *   do i=1,sizearray[j]
 *     print *,a(i,j)
 *   enddo
 * enddo
 *
 * ***********************************************************************
 * Example 3: struct {struct{int i;real a[*];}t[*];}s[3]; 
 *
 * defines an array s.t of type structarray
 * with rank=2, 
 *      size={max(SIZE(s[].t)),3}
 * sizearray('s.t') has sarank=1
 *               sasize={ 3 };
 * sizearray('s.t')={ SIZE(s[i].t),i=1,3 }
 *
 *
 * and defines an array s.t.i of type int
 * with rank=2, 
 *      size={max(SIZE(s[].t)),3}
 * sizearray('s.t.i') has sarank=1
 *               sasize={ 3 };
 * sizearray('s.t.i')={ SIZE(s[i].t),i=1,3 } , is in fact identical 
 * to sizearray('s.t')
 *
 *
 * do j=1,size[2]
 *   do i=1,sizearray('s.t')[j]
 *     print *,i(i,j)
 *   enddo
 * enddo
 *
 * and defines an array s.t.a of type real
 * with rank=3, 
 *      size={max(SIZE(s[].t[].a)),max(SIZE(s[].t)),3}
 * sizearray has sarank=2
 *               sasize={ max(SIZE(s[].t)) ,3 };
 * sizearray[:,1]={ SIZE(s[1].t[n].a),n=1,SIZE(s[1].t) }
 * sizearray[:,2]={ SIZE(s[2].t[n].a),n=1,SIZE(s[2].t) }
 * sizearray[:,3]={ SIZE(s[3].t[n].a),n=1,SIZE(s[3].t) }
 *
 * do k=1,size[3]
 *   do j=1,sizearray('s.t')[k]
 *     do i=1,sizearray('s.t.a')[j,k]
 *       print *,a(i,j,k)
 *     enddo
 *   enddo
 * enddo

 * ***********************************************************************
 */
void CFUNC(cgetsizearray)(const char *name,FPLOINTEGER *sarank,FPLOINTEGER sasize[7],
			  FPLOINTEGER *sizearray
			  ,FPLOLOGICAL *err,char *errormsg
			  ,INTERNAL_INTEGER name_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;
  
  if(!getSizeArray(&fortran_pc.table,p,*sarank,sasize,sizearray))goto error;
  

  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}
/* ***********************************************************************
 * an data array of up to 5 dimensions has
 * rank          dimensions
 * size[1:5]     maximum sizes for each dimension
 *
 * ***********************************************************************
 */
void CFUNC(cgetarrayrankandsize)(const char *name,FPLOINTEGER *rank,FPLOINTEGER size[7]
				 ,FPLOLOGICAL *err,char *errormsg
				 ,INTERNAL_INTEGER name_len,
				 INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(name,name_len))==NULL) goto error;
  
  if(!getDimensions(&fortran_pc.table,p,rank,size))goto error;
  
  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void CFUNC(csetvalue)(const char *var,const char *value
		      ,FPLOLOGICAL *err,char *errormsg
		      ,INTERNAL_INTEGER var_len,INTERNAL_INTEGER value_len
		      ,INTERNAL_INTEGER err_len)
{
  char *pvar=NULL;
  char *pvalue=NULL;
  *err=L_FALSE;
  if((pvar=f2cstrcpy(var,var_len))==NULL) goto error;

  if((pvalue=f2cstrcpy(value,value_len))==NULL) goto error;
  
  if(!setValue(&fortran_pc.table,pvar,pvalue))goto error;
  
  FREEE(pvar,"");pvar=NULL;
  FREEE(pvalue,"");pvalue=NULL;
  return;
 error:
  if(pvar){ FREEE(pvar,"");pvar=NULL; }
  if(pvalue){ FREEE(pvalue,"");pvalue=NULL; }
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return ;
  
}





/* ***********************************************************************
 * set a section name after which to stop
 * ***********************************************************************
 */

void CFUNC(caddexternalrealarray)(const char *aname,FPLODOUBLEPRECISION *array,
				  FPLOLOGICAL *err,char *errormsg
				  ,INTERNAL_INTEGER aname_len,
				  INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(aname,aname_len))==NULL) goto error; 


  if(strlen(p)>MAX_ID_LENGTH){
    (void)sprintf(errorbuffer,"error(caddexternalarray): array name to long!");
    goto error;
  }
  if(!AddExternalArrayAddress(&fortran_pc,p,array))
    goto error;
  
  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}
/* ***********************************************************************
 * set a section name after which to stop
 * ***********************************************************************
 */

void CFUNC(caddexternalintarray)(const char *aname,FPLOINTEGER *array,FPLOLOGICAL *err,
				 char *errormsg
				 ,INTERNAL_INTEGER aname_len,
				 INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(aname,aname_len))==NULL) goto error; 


  if(strlen(p)>MAX_ID_LENGTH){
    (void)sprintf(errorbuffer,"error(caddexternalarray): array name to long!");
    goto error;
  }
  if(!AddExternalArrayAddress(&fortran_pc,p,array))
    goto error;
  
  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}
/* ***********************************************************************
 * set a section name after which to stop
 * ***********************************************************************
 */

void CFUNC(csetparsestop)(const char *sname,FPLOLOGICAL *err,char *errormsg
			  ,INTERNAL_INTEGER sname_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(sname,sname_len))==NULL) goto error; 


  if(strlen(p)>MAX_ID_LENGTH){
    (void)sprintf(errorbuffer,"error(csetparsestop): stop section name to long!");
    goto error;
  }
  fortran_pc.stopaftersection=1;
  (void)strcpy(fortran_pc.stopsectionname,p);
  

  FREEE(p,"");p=NULL;
  return;
 error:
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cparsefile)(const char *fname,FPLOLOGICAL *err,char *errormsg
		       ,INTERNAL_INTEGER fname_len,INTERNAL_INTEGER err_len)
{
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(fname,fname_len))==NULL) goto error; 

  
  if(!parsefile(&yyin,p,&fortran_pc) ) goto error;
  FREEE(p,"");p=NULL;
  return;
 error: 
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cinitparser)(FPLOLOGICAL *err,char *errormsg,INTERNAL_INTEGER err_len)
{

  *err=L_FALSE;

  if(!init_parser_control(&fortran_pc,0,1))
    goto error;
  return;
 error:
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(ccleanparsefile)(FPLOLOGICAL *err,char *errormsg,
			    INTERNAL_INTEGER err_len)
{
  *err=L_FALSE;
  if(!clean_vartable(&fortran_pc.table))goto error;
  
  return;
  
 error:

  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cwriteparsefile)(const char *fname,FPLOLOGICAL *err,char *errormsg
			    ,INTERNAL_INTEGER fname_len,INTERNAL_INTEGER err_len)
{
  FILE *FO=NULL;
  char *p;
  *err=L_FALSE;
  if((p=f2cstrcpy(fname,fname_len))==NULL) goto error; 

  
  if((FO=fopen(p,"w"))==NULL){
    sprintf(errorbuffer,"Cannnt open '%s' for write.",p);
    goto error;
  }


  if(!print_vartab(FO,&fortran_pc.table))goto error;

  
  fclose(FO);
  FREEE(p,"");p=NULL;
  return;
  
 error:
  if(FO!=NULL) fclose(FO);
  FREEE(p,"");p=NULL;
  (void)c2fstrncpy(errormsg,errorbuffer,(long)err_len,1L);
  *err=L_TRUE;
  return;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */

void CFUNC(cfinalcleanup)(void)
{
  finalcleanup();
  return;
}
