/* @(#)ioyaccinternal_inc.c
 */

#include <string.h>
#include "allocate.h"

#ifndef _IOYACCINTERNAL_INC_H
#define _IOYACCINTERNAL_INC_H 1

/* ***********************************************************************
 * this file is included by io.y and contains functions used
 * internally by bison or which must be defined in io.y, due to bison
 * specific type declaration
 * ***********************************************************************
 */


/* better yyerror*/

#ifndef yyerror
#define yyerror(x)  yyerror1(linenr,columnr,x)
#endif

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void yyerror1 (long linenr,long columnr,char *s)  
{
  char errbuffer[5000];
  char buf[5000];
  char *p1,*p2;
  p1=s;
  p2=buf;

  while(*p1!='\0'){
    if(strstr(p1,"`$'")==p1){
      (void)strcat(p2,"EOF");
      p2+=3;
      p1+=3;
    }else if(strstr(p1,"`$undefined.'")==p1){
      (void)strcat(p2,"unknown input");
      p2+=13;
      p1+=13;
    }else{
      *p2++=*p1++;
    }
  }
  *p2='\0';
  (void)sprintf (errbuffer,"line %ld,near column %ld: %s:%s"
	   ,linenr,columnr,buf,errorbuffer);
  (void)strcpy(errorbuffer,errbuffer);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long checkarraytypes(VARTABLE *node ,CONSTANTS *cons )
{
  tsARRAY7 *arp=ARP(node);
  
  if(cons->type == VTYPE_POS_INF){
    if (arp->scalartype == VTYPE_REAL ){
      cons->type=VTYPE_REAL;
      if(!node->aschar ) cons->value.d=R_POS_INFINITY;
    }else if(arp->scalartype==VTYPE_INT)
      cons->type=VTYPE_INT;
  }else if(cons->type == VTYPE_NEG_INF){
    if (arp->scalartype == VTYPE_REAL ){
      cons->type=VTYPE_REAL;
      if(!node->aschar ) cons->value.d=R_NEG_INFINITY;
    }else if(arp->scalartype==VTYPE_INT)
      cons->type=VTYPE_INT;
  }
  
  
  if(arp->scalartype==VTYPE_REAL && cons->type==VTYPE_INT ){
    if(!node->aschar){
      cons->value.d=(FPLODOUBLEPRECISION)(cons->value.i);
    }
  }else if(arp->scalartype != cons->type){
    (void)sprintf(errorbuffer,"error: Variable %s "
		  "Constant type mismatch\n",node->name);
    return 0;
  }
  if(arp->scalartype == VTYPE_STRING &&
     teststringlength(cons->value.s,arp->strvariable,arp->strlength)<0){
    (void)sprintf(errorbuffer,"error: String %s: value longer than declared "
		  "string length!\n",node->name);
    return 0;
  }
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long checkscalartypes(VARTABLE *node,CONSTANTS *cons)
{
  tsSCALAR *scp=SCP(node);
  
  if(cons->type == VTYPE_POS_INF){
    if (scp->scalartype == VTYPE_REAL ){
      cons->type=VTYPE_REAL;
      if(!node->aschar ) cons->value.d=R_POS_INFINITY;
    }else if(scp->scalartype==VTYPE_INT)
      cons->type=VTYPE_INT;
  }else if(cons->type == VTYPE_NEG_INF){
    if (scp->scalartype == VTYPE_REAL ){
      cons->type=VTYPE_REAL;
      if(!node->aschar ) cons->value.d=R_NEG_INFINITY;
    }else if(scp->scalartype==VTYPE_INT)
      cons->type=VTYPE_INT;
  }
  
  
  if(scp->scalartype==VTYPE_REAL && cons->type==VTYPE_INT ){
    if(!node->aschar){
      cons->value.d=(FPLODOUBLEPRECISION)(cons->value.i);
    }
  }else if(scp->scalartype != cons->type){
    (void)sprintf(errorbuffer,"error: Variable %s "
		  "Constant type mismatch\n",node->name);
    return 0;
  }
  if(scp->scalartype == VTYPE_STRING &&
     teststringlength(cons->value.s,scp->strvariable,scp->strlength)<0){
    (void)sprintf(errorbuffer,"error: String %s: value longer than declared "
		  "string length!\n",node->name);
    return 0;
  }
  
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long preparedim(long i,DIMSPECS *index,DIMSPECS *dims)
{
  long l=0;

  if(index->idref[0]!=NULL){
    l=strlen(index->idref[0]);
  }

  if(l){
    if((dims->idref[i]=(char*)allocate(sizeof(char)*(l+1)+1
				       ,"dims->idref[i]","preparedim"))==NULL){
      (void)strcpy(errorbuffer,errormsg);
      return 0;
    }
    (void)strcpy(dims->idref[i],index->idref[0]);

  }  
  dims->value[i]=index->value[0];
  
  if(dims->rank !=i && dims->value[i] < 1){
    (void)sprintf(errorbuffer,"Array size of dimension %ld = 0!\n",i);
    return 0;
  }
  return 1;
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long initparser(STRUCTDATASTACK *std
		       ,PARSER_CONTROLL *pc
		       ,long *aschar
		       ,long *padding)
{
  if(!init_vartable(&pc->table,pc->aschar)){
    return 0;
  }

  if(!init_stdstack(std,pc->aschar)){
    return 0;
  }
  *aschar=pc->aschar;
  *padding=pc->padding;
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long writescalar(VARTABLE *node,CONSTANTS *cons)
{
  if(node->structuretype!=VTYPE_SCALAR)return 1;

  if(!checkscalartypes(node,cons)) return 0;
  
  
  if( !node->aschar &&node->data.scalar.scalartype == VTYPE_INT ){
    if(!write_into_scalar(node,&(cons->value.i),WRITE_COPY)){
      return 0;
    }
  }else if(!node->aschar && node->data.scalar.scalartype == VTYPE_REAL ){
    if(!write_into_scalar(node,&(cons->value.d),WRITE_COPY)){
      return 0;
    }    
  }else if(!node->aschar && node->data.scalar.scalartype == VTYPE_LOGICAL ){
    if(!write_into_scalar(node,&(cons->value.l),WRITE_COPY)){
      return 0;
    }    
  }else{
    if(node->data.scalar.scalartype &
       (VTYPE_STRING | VTYPE_FLAG | VTYPE_INT | VTYPE_REAL) ){
      if(!write_into_scalar(node,cons->value.s,WRITE_REF)){
	return 0;
      }
    }else if( node->data.scalar.scalartype == VTYPE_LOGICAL ){
      if(!write_into_scalar(node,&(cons->value.c),WRITE_COPY)){
	return 0;
      }
    }else if( node->data.scalar.scalartype == VTYPE_CHAR ){
      if(!write_into_scalar(node,&(cons->value.c),WRITE_COPY)){
	return 0;
      }
    }
  }
  
  
  return 1;

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long checkrank(VARTABLE *node,long rank) 
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;
  if((node->data.array7.rank)!=rank){
    (void)sprintf(errorbuffer,"Array %s: Data have wrong rank!"
		  ,node->name);
    return 0;
  }
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long checkunderflow(VARTABLE *node,long *indx,long padding,long dim)
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;
  if(indx[dim]<node->data.array7.Size[dim]){
    if(padding){
      (void)printf("Array %s: To few elements, padding with 0!\n"
	      ,node->name );
    }else{
      (void)sprintf(errorbuffer,"Array %s: To few elements!\n",node->name);
      return 0;
    }
  }
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long checklastindex(VARTABLE *node,long *indx,long dim)
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;

  if(indx[dim]>node->data.array7.Size[dim]){
    if(node->data.array7.variable){
      /* seems to be an obsolete hack : node->data.array7.size[dim]=indx[dim]; */
    }else{
      (void)sprintf(errorbuffer,"array %s: To many elements\n"
	      ,node->name);
      return 0;
    }
  }
  return 1;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long checkoverflow(VARTABLE *node,long *indx,long dim)
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;

  if(indx[dim]>node->data.array7.Size[dim]){
    (void)sprintf(errorbuffer,"array %s: To many elements\n"
	    ,node->name);
    return 0;
  }
  return 1;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

static long writearray(VARTABLE *node,long indx[8],CONSTANTS *cons)
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;
  
  if( !node->aschar &&node->data.array7.scalartype == VTYPE_INT ){
    if(!write_into_array(node,indx,&(cons->value.i),WRITE_COPY)){
      return 0;
    }
  }else if(!node->aschar && node->data.array7.scalartype == VTYPE_REAL ){
    if(!write_into_array(node,indx,&(cons->value.d),WRITE_COPY)){
      return 0;
    }    
  }else if(!node->aschar && node->data.array7.scalartype == VTYPE_LOGICAL ){
    if(!write_into_array(node,indx,&(cons->value.l),WRITE_COPY)){
      return 0;
    }    
  }else{
    if(node->data.array7.scalartype & 
       ( VTYPE_STRING | VTYPE_FLAG | VTYPE_INT | VTYPE_REAL)
       ){
      if(!write_into_array(node,indx,cons->value.s,WRITE_REF)){
	return 0;
      }
    }else if( node->data.array7.scalartype == VTYPE_LOGICAL ){
      if(!write_into_array(node,indx,&(cons->value.c),WRITE_COPY)){
	return 0;
      }
    }else if( node->data.array7.scalartype == VTYPE_CHAR ){
      if(!write_into_array(node,indx,&(cons->value.c),WRITE_COPY)){
	return 0;
      }
    }
  }
  return 1;
  
}

/* reset lex memory */
extern void lexreset(void); 

/* ***********************************************************************
 * to be called after parser exited, to clean up memory, does not cleanup
 * the data table itself, which must be cleared after its usage
 * ***********************************************************************
 */
static long exitparser(STRUCTDATASTACK *std,PARSER_CONTROLL *pc)
{
  pc=pc;
  lexreset();

  if(!clean_stdstack(std)){
    return 0;
  }

  return 1;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long checkunderoverflow(VARTABLE *node,long *indx,long padding,long dim)
{
  if(node->structuretype!=VTYPE_ARRAY)return 1;

  if(indx[dim]<node->data.array7.Size[dim]){
    if(padding){
      (void)printf("Array %s: To few elements, padding with 0!\n"
	      ,node->name );
    }else{
      (void)sprintf(errorbuffer,"Array %s: To few elements!\n",node->name);
      return 0;
    }
  }
  if(indx[dim]>node->data.array7.Size[dim]){
    (void)sprintf(errorbuffer,"array %s: To many elements\n"
	    ,node->name);
    return 0;
  }
  return 1;
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void init_dimspecs(DIMSPECS *dims)
{
  long i;
  dims->rank=0;
  dims->variable=0;
  for(i=0;i<8;i++) {
    dims->value[i]=0;
    dims->idref[i]=NULL;
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void clean_dimspecs(DIMSPECS *dims)
{
  long i;
  dims->rank=0;
  dims->variable=0;
  for(i=0;i<8;i++) {
    dims->value[i]=0;
    if(dims->idref[i]!=NULL){
      FREEE(dims->idref[i],"dims->idref[i]");  dims->idref[i]=NULL;
    }
  }
}

#endif /* _IOYACCINTERNAL_INC_H */

