/* @(#)table.c
 */


#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ioconfig.h"  
#include "table.h"
#include "tools.h"  
#include "fortrantypes.h"  
#include "helper.h"
#include "convtests.h"  
#include "merror.h"
#include "allocate.h"  


#define LMAX(x,y) ((x)>=(y)?(x):(y))

static long boundcheck=1;


static long disectDefVal(const char *defaultvalue,long *ndefval,char ***pds);
static VARTABLE *append_to_list(VARTABLE *list);
static long init_scalar_value(VARTABLE *node);
static long intern_reallocate_array(tsARRAY7 *arp,long oldsize,long newsize
				    ,long aschar,const char *name);
static long intern_reallocate_structarray(tsSTRUCTARRAY *sap
					  ,long oldsize
					  ,long newsize);
static long create_struct_from_template(VARTABLE *dest,VARTABLE *template
					,long allocarray,long reallocarray);

/* ***********************************************************************
 * initialize table of variables
 * ***********************************************************************
 */
long init_vartable(VARTABLE *table,long aschar)
{
  if(!myassert(table,"init_vartable","table"))
    return 0;

  table->structuretype=VTYPE_NONE;
  table->next=NULL;
  table->prev=NULL;
  table->parent=NULL;
  table->name=NULL;
  table->aschar=aschar;
  table->dataplacement=0;
  return 1;
}


/* ***********************************************************************
 * append a new node at end of list, initialize it
 * ***********************************************************************
 */
static VARTABLE *append_to_list(VARTABLE *list)
{
  char *subfunc="append_to_list";
  VARTABLE *node,*p;

  if(!myassert(list,subfunc,"list")) return NULL;
  if((node=allocate(sizeof(VARTABLE),"new variable",subfunc))==NULL)
    return NULL;
  

  if(!init_vartable(node,list->aschar))return NULL;
  
  p=list;
  while(p->next !=NULL ) p=p->next;
  p->next=node;
  node->prev=p;
  node->parent=list->parent;

  return node;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_scalar_to_list(VARTABLE *list,const char *name
				,long scalartype
				,long strlength
				,long strvariable
				,void *defaultvalue)
{  
  char *subfunc="append_scalar_to_list";
  VARTABLE * node;
  
  if(!myassert(list,subfunc,"list")) return NULL;
  
  
  if(lookup_list(list,name,NULL,VTYPE_SCALAR|VTYPE_ARRAY|VTYPE_STRUCT
		     |VTYPE_STRUCTARRAY ,0 )!=NULL){ 
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s declared twice !"
	    ,subfunc,name);
    return NULL;
  } 
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_SCALAR;  
  
  /* set node name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  (void)strcpy(node->name,name);
  
  
  node->data.scalar.scalartype=scalartype;
  node->data.scalar.strlength=strlength;
  node->data.scalar.strvariable=strvariable;
  node->data.scalar.value=NULL;
  node->data.scalar.isdim=0;
  node->data.scalar.defaultvalue=NULL;
  
  
  if(! set_defaultvalue_scalar(node,defaultvalue))return NULL;  
  
  if(!init_scalar_value(node) ) return NULL; 
 

  return node;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_struct_to_list(VARTABLE *list,char *name
				,VARTABLE *template
				,long allocarray)
{  
  char *subfunc="append_struct_to_list";
  VARTABLE * node;

  
  if(!myassert(list,subfunc,"list")) return NULL;
/*   if(applytemplate && !myassert(template,subfunc,"template")) return NULL; */
  
  
  if(lookup_list(list,name,NULL,VTYPE_SCALAR|VTYPE_ARRAY|VTYPE_STRUCT
		     |VTYPE_STRUCTARRAY ,0 )!=NULL){ 
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s declared twice !"
	    ,subfunc,name);
    return NULL;
  } 
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_STRUCT;  

  /* set node name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  (void)strcpy(node->name,name);


  
  
  node->data.list=NULL;
  if((node->data.list=allocate(sizeof(VARTABLE),name,subfunc))==NULL) 
    return NULL;
  
  if(!init_vartable(node->data.list,list->aschar))return NULL;
  
  if(template!=NULL){
    if(!create_struct_from_template(node->data.list,template,allocarray,0L))
      return NULL;
  }
  node->data.list->parent=node;
  
  return node;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_structarray_to_list(VARTABLE *list,char *name
					 ,long size
					 ,long variable
					 ,char *idref
					 ,VARTABLE *template
				         ,long allocarray)
{  
  char *subfunc="append_structarray_to_list";
  tsSTRUCTARRAY *sap;
  VARTABLE * node;

  /* printf("%s name=%s allocarray=%ld\n",subfunc,name,allocarray); */
  
  if(!myassert(list,subfunc,"list")) return NULL;
  /* if(applytemplate && !myassert(template,subfunc,"template")) return NULL; */
  
  
  if(lookup_list(list,name,NULL,VTYPE_SCALAR|VTYPE_ARRAY|VTYPE_STRUCT
		     |VTYPE_STRUCTARRAY ,0 )!=NULL){ 
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s declared twice !"
	    ,subfunc,name);
    return NULL;
  } 
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_STRUCTARRAY;  

  /* set  variable name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  (void)strcpy(node->name,name);


  sap=&node->data.structarray;


  sap->Size=0;
  sap->currentsize=0;
  sap->variable=0;
  sap->array=NULL;
  sap->idref=NULL;  
  sap->typetemplate=NULL;
  sap->typetemplateiscopy=(allocarray!=0);
  sap->allocated=0;  
  
  if(!allocarray){
    if((sap->typetemplate=allocate(sizeof(VARTABLE),name,subfunc))==NULL)
      return NULL;
    if(!init_vartable(sap->typetemplate,list->aschar))return NULL;
  }else{
    sap->typetemplate=template;
  }
  
  
  
  /* set up array specifics */
  sap->variable=variable;  
  if(size<0L){
    (void)sprintf(errorbuffer
	    ,"error(%s): structarray %s  size <%ld!",subfunc
	    ,node->name,0L);
    return NULL;
  }
  sap->Size=size;
  
  /*named index*/
  if(idref !=NULL ){ 
    if(sap->idref !=NULL ){ 
      FREEE(sap->idref,"sap->idref");sap->idref=NULL; 
    }
    if(( sap->idref=(char*)allocate(sizeof(char)*(strlen(idref)+1) 
				    ,"sap->idref",subfunc))==NULL)return NULL;
    (void)strcpy(sap->idref,idref); 
  }
  
  

  /* copy pattern */
  if(!allocarray){
    if(template!=NULL){
      if(!create_struct_from_template(sap->typetemplate,template,0L,0L))
	return NULL;
    }
  }


  
  if(allocarray){
    if(!allocate_structarray(node)) return 0;
  }
  
  return node;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_comment_to_list(VARTABLE *list,char *comment,long copy)
{
  char *subfunc="append_comment_to_list";
  VARTABLE * node;

  
  if(!myassert(list,subfunc,"list")) return NULL;
  
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_COMMENT;  
  
  
  if(copy==WRITE_COPY){
    if((node->name=(char*)allocate(sizeof(char)*(strlen(comment)+1)
				   ,comment,subfunc))==NULL) return NULL;
    (void)strcpy(node->name,comment);
  }else{
    node->name=comment;
  }
  
  

  return node;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_section_to_list(VARTABLE *list,const char *name)
{
  char *subfunc="append_section_to_list";
  VARTABLE * node;

  
  if(!myassert(list,subfunc,"list")) return NULL;

  
  if((lookup_list(list,name,NULL,VTYPE_SECTION,0))!=NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Section %s defined twice!"
	    ,subfunc,name);
    return NULL;
  }

  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_SECTION;  
  

  /* set node name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  (void)strcpy(node->name,name);
  
  

  return node;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_section_end_to_list(VARTABLE *list,const char *name)
{
  char *subfunc="append_section_end_to_list"; 
  VARTABLE * node;
  
  if(!myassert(list,subfunc,"list")) return NULL;

  
  
  if((lookup_list(list,name,NULL,VTYPE_SECTIONEND,0))!=NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Section end %s defined twice!"
	    ,subfunc,name);
    return NULL;
  }
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_SECTIONEND;  

  /* set node name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  
  (void)strcpy(node->name,name);
  
  return node;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE* append_placement_to_list(VARTABLE *list,VARTABLE *noderef)
{

  char *subfunc="append_placement_to_list"; 
  VARTABLE * node;

  
  if(!myassert(list,subfunc,"list")) return NULL;
  if(!myassert(noderef,subfunc,"noderef")) return NULL;

  
  
  if(noderef->name==NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Placement variable name is NULL!"
	    ,subfunc);
    return NULL;
  }
  
  if((lookup_list(list,NULL,noderef,VTYPE_PLACEMENT,0))!=NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s given a value twice!"
	    ,subfunc,noderef->name);
    return NULL;
  }
  if((node=append_to_list(list))==NULL) return NULL;
  
  node->structuretype=VTYPE_PLACEMENT;  
  node->data.noderef=noderef;  

 
  return node;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long clean_vartable(VARTABLE *list)
{
  char *subfunc="clean_vartable";
  VARTABLE *node,*t;
  
  long structuretype,ret=1;

  if(list==NULL) return 1;

  
  node=list; while(node->next!=NULL) node=node->next;
  
  
  while(node!=list){
#ifdef _DEBUG_     
    printf("information(clean_vartable): clean %p 0x%lx\n",node,node->structuretype); 
#endif 

    structuretype=node->structuretype;

    
    if( 
       (structuretype==VTYPE_PLACEMENT)||
       (structuretype==VTYPE_COMMENT)||
       (structuretype==VTYPE_SECTION)||
       (structuretype==VTYPE_SECTIONEND)
	){
      ;
    }else if(structuretype==VTYPE_SCALAR){
      if(!vartable_clean_scalar(node)) ret=0;
    }else if(structuretype==VTYPE_STRUCT){
      if(!clean_vartable(node->data.list)) ret=0;
      FREEE(node->data.list,node->name);  node->data.list=NULL;
    }else if(structuretype==VTYPE_STRUCTARRAY){
      if(!vartable_clean_structarray(node)) ret=0;
    }else if(structuretype==VTYPE_ARRAY){
      if(!vartable_clean_array(node)) ret=0;
    }else{
      (void)sprintf(errorbuffer
	      ,"error(%s): Variable has unknown structuretype 0x%lx!"
	      ,subfunc,node->structuretype);
      ret=0;
    }
    
    FREEE(node->name,"name"); node->name=NULL;
    
    t=node->prev;
    FREEE(node,"node");node=NULL;
    node=t;
  }
  list->next=NULL;

  return ret;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

VARTABLE *lookup_list(const VARTABLE * list
		      ,const char *name
		      ,const VARTABLE *noderef
		      ,const long structuretype
		      ,const long scalartype)
{
  char *subfunc="lookup_list";
  VARTABLE *node;
  subfunc=subfunc;

  if(!myassert(list,subfunc,"list")) return NULL;

  node=list->next;
  while(node!=NULL){
    /* TODO: put name compare here */
    if(!structuretype ||  structuretype & node->structuretype ){
      if(node->structuretype ==  VTYPE_SCALAR){
	/* printf("%s  structure 0x%lx scalartype 0x%lx\n",node->name  */
	/* 		 ,node->structuretype,node->data.scalar.scalartype);  */
	
	if(node->name!=NULL && name !=NULL
	   && (!scalartype || (node->data.scalar.scalartype & scalartype))
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_ARRAY){
	
	/*     printf("%s 0x%lx structure 0x%lx scalartype 0x%lx\n",node->name,node->type */
	/* 	   ,node->type& structure,node->type&scalartype); */
	
	if(node->name!=NULL && name !=NULL
	   && (!scalartype || (node->data.array7.scalartype & scalartype))
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_SECTION){
	/*     printf("%s 0x%lx structure 0x%lx scalartype 0x%lx\n",node->name,node->type */
	/* 	   ,node->type& structure,node->type&scalartype); */
	
	if(node->name!=NULL && name !=NULL
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_PLACEMENT){
	/*     printf("%s 0x%lx structure 0x%lx scalartype 0x%lx\n",node->name,node->type */
	/* 	   ,node->type& structure,node->type&scalartype); */
      	if(node->data.noderef!=NULL && noderef !=NULL
	   && node->data.noderef == noderef){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_SECTIONEND){
	/*     printf("%s 0x%lx structure 0x%lx scalartype 0x%lx\n",node->name,node->type */
	/* 	   ,node->type& structure,node->type&scalartype); */
	
	if(node->name!=NULL && name !=NULL
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_STRUCT){
	if(node->name!=NULL && name !=NULL
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }else if(node->structuretype ==  VTYPE_STRUCTARRAY){
	if(node->name!=NULL && name !=NULL
	   && strcmp(node->name,name)==0){
	  return node;
	}
      }
    
    }
    node=node->next;
  }
  return NULL;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE *append_array_to_list(VARTABLE *list,const char *name
			       ,long scalartype
			       ,long rank
			       ,long size[8]
			       ,char *idref[8]
			       ,long variable
			       ,long strlength,long strvariable
			       ,void *defaultvalue
			       ,void *externalarray
			       ,long allocarray
			       ,long reallocarray
			       )
{
  char *subfunc="append_array_to_list";
  VARTABLE * node;
  tsARRAY7 *arp;
  long i;

  if(!myassert(list,subfunc,"list")) return NULL;
  
  
  
  if(lookup_list(list,name,NULL,VTYPE_SCALAR | VTYPE_ARRAY|VTYPE_STRUCT
		     |VTYPE_STRUCTARRAY ,0)!=NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s declared twice !"
	    ,subfunc,name);
    return NULL;
  }
  
  
  if((node=append_to_list(list))==NULL) return NULL;
  node->structuretype=VTYPE_ARRAY;  
  
  /* set node name */
  if((node->name=allocate(sizeof(char)*(strlen(name)+1),name,subfunc))==NULL)
    return NULL;
  (void)strcpy(node->name,name);
  

  arp=&node->data.array7;
  
  arp->scalartype=scalartype;
  arp->strlength=strlength;
  arp->strvariable=strvariable;  
  arp->variable=variable;  
  arp->rank=rank;
  
  arp->allocated=0;
  for(i=0;i<8;i++) arp->Size[i]=0;/* was -1 ,shoudn't it be 0  */
  arp->currentsize=0;
  arp->lowdim=0;
  arp->external=0;
  arp->array=NULL;
  arp->defaultvalue=NULL;
  for(i=0;i<8;i++) arp->idref[i]=NULL;  

  


  
  
  if(externalarray!=NULL) {
    if( variable) {
      (void)sprintf(errorbuffer
		    ,"error(%s): array %s: scanning into external arrays not "
		    "possible\nfor arrays with variable last dimension!"
		    ,subfunc,name);
      return NULL;
    }
    if(list->aschar) {
      (void)sprintf(errorbuffer
		    ,"error(%s): internal error: array %s: scanning into "
		    "external arrays required,\n but character scanning switched on?!"
		    ,subfunc,name);
      return NULL;
    }
    
    if(! (scalartype & (VTYPE_REAL|VTYPE_INT))){
      (void)sprintf(errorbuffer,"error(%s): array %s: scanning into external arrays "
		    "only allowed for real and int arrays!",subfunc,name);
      return NULL;
    }
    arp->external=1;
    arp->array=externalarray;
  }    
  
  if(idref!=NULL){
    for(i=1;i<8;i++){ 
      if( idref[i]!=NULL){ 
	if(arp->idref[i]!=NULL){
	  FREEE(arp->idref[i],"arp->idref[i]");arp->idref[i]=NULL; 
	}
	if(( arp->idref[i]=(char*)allocate(sizeof(char)*(strlen(idref[i])+1) 
					   ,"arp->idref",subfunc))==NULL) 
	  return NULL;
	(void)strcpy(arp->idref[i],idref[i]); 
      }
    }
  }
  
#define TESTDIM(i,name,limit) \
    if(size[(i)]<limit){\
        (void)sprintf(errorbuffer\
	    ,"error(%s): array %s  dimension %ld: size < %ld!",subfunc\
	    ,(name),(i),(limit));\
         return NULL;\
    }
  
  
  TESTDIM(rank,name,0L); 
  arp->lowdim=1;
  switch ((int)rank){
  case 5:
    arp->lowdim*=size[4];
    arp->Size[5]=size[5];
    TESTDIM(4L,name,1L); 
  case 4:
    arp->lowdim*=size[3];
    arp->Size[4]=size[4];
    TESTDIM(3L,name,1L); 
  case 3:
    arp->lowdim*=size[2];
    arp->Size[3]=size[3];
    TESTDIM(2L,name,1L); 
  case 2:
    arp->lowdim*=size[1];
    arp->Size[2]=size[2];
    TESTDIM(1L,name,1L); 
  case 1:
    arp->Size[1]=size[1];
    break;
  default:
    (void)sprintf(errorbuffer
		  ,"error(%s): Array %s to many dimensions(%ld)!"
		  ,subfunc,name,rank);
    return NULL;
  }
  if(! set_defaultvalue_array(node,defaultvalue))return NULL;
  

  if(allocarray){
    if(! allocate_array(node,reallocarray )) return NULL;
  }
  
  return node;  
}

#undef TESTDIM














/* ***********************************************************************
 * type specific stuff
 * ***********************************************************************
 */





/* ***********************************************************************
 * tsSCALAR
 * ***********************************************************************
 */
long set_defaultvalue_scalar(VARTABLE* node,void *defaultvalue)
{
  char *subfunc="set_defaultvalue_scalar";
  long scalartype,l;
  FPLODOUBLEPRECISION d;
  FPLOINTEGER i;
  char *buf,*p;

  if(!myassert(node,subfunc,"node")) return 0;
  
  if(node->structuretype!=VTYPE_SCALAR){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s: not a scalar!"
		  ,subfunc,node->name);
    
    return 0;
  }


  scalartype=node->data.scalar.scalartype;
  
  FREEE(node->data.scalar.defaultvalue,"node->defaultvalue");
  node->data.scalar.defaultvalue=NULL;
  
  if(defaultvalue != NULL ){ 
    if(scalartype==VTYPE_REAL){
      if(node->aschar){
	if(( buf=allocate(sizeof(char)*(strlen(defaultvalue)+1) 
			  ,"buf",subfunc))==NULL) return 0;
	
	(void)strcpy(buf,defaultvalue);
	if(!convertfortranreal(buf,&d)){
	  (void)sprintf(errorbuffer,"error(%s): scalar %s: defaultvalue (%s) is "
		  "not a valid real!\n"
		  ,subfunc,node->name,(char*)defaultvalue);
	  FREEE(buf,"buf");buf=NULL;
	  return 0;
	}
	FREEE(buf,"buf");buf=NULL;
	
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		       ,"REAL scalar defaultvalue",subfunc))==NULL)
	  return 0;
	(void)strcpy( (char*)node->data.scalar.defaultvalue,defaultvalue); 
	
      }else{
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(FPLODOUBLEPRECISION) 
		       ,"REAL scalar defaultvalue",subfunc))==NULL)
	  return 0;
	*((FPLODOUBLEPRECISION*)node->data.scalar.defaultvalue)
	  =*((FPLODOUBLEPRECISION*)defaultvalue);
      }
      
    }else if(scalartype==VTYPE_INT){ 
      if(node->aschar){

	if(( buf=allocate(sizeof(char)*(strlen(defaultvalue)+1) 
			  ,"buf",subfunc))==NULL) return 0;
	(void)strcpy(buf,defaultvalue);
	removetrailingspace(buf);
	p=buf+strspn(buf," \t");


	if(!convertlong(p,&i)){
	  (void)sprintf(errorbuffer,"error(%s): scalar %s: defaultvalue (%s) is "
		  "not a valid int!\n"
		  ,subfunc,node->name,(char*)defaultvalue);
	  FREEE(buf,"buf");buf=NULL;
	  return 0;
	}
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(char)*(strlen(p)+1) 
		       ,"INT scalar defaultvalue",subfunc))==NULL){
	  FREEE(buf,"buf");buf=NULL;
	  return 0;
	}
	(void)strcpy( (char*)node->data.scalar.defaultvalue,p); 
	FREEE(buf,"buf");buf=NULL;

      }else{
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(FPLOINTEGER) 
		       ,"INT scalar defaultvalue",subfunc))==NULL)
	  return 0;	  
	*((FPLOINTEGER*)node->data.scalar.defaultvalue)
	  =*((FPLOINTEGER*)defaultvalue);
	
      }
      
      
    }else if(scalartype==VTYPE_LOGICAL){ 
      if(node->aschar){
	if(!( tolower(((char*)defaultvalue)[0])=='t' 
	      || tolower(((char*)defaultvalue)[0])=='f' )){
	  if(((char*)defaultvalue)[0]=='\0'){
	    (void)sprintf(errorbuffer,"error(%s): scalar %s defaultvalue () is "
		    "not a valid logical!\n"
		    ,subfunc,node->name);
	    return 0;
	  }else{
	    (void)sprintf(errorbuffer,"error(%s): scalar %s defaultvalue (%c) is "
		    "not a valid logical!\n"
		    ,subfunc,node->name,*((char*)defaultvalue));
	    return 0;
	  }
	  
	}
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(char) 
		       ,"LOGICAL scalar defaultvalue",subfunc))==NULL)
	  return 0;
	*((char*)node->data.scalar.defaultvalue)=tolower(((char*)defaultvalue)[0]); 
      }else{
	if(( node->data.scalar.defaultvalue
	     =allocate(sizeof(FPLOLOGICAL) 
		       ,"LOGICAL scalar defaultvalue",subfunc))==NULL)
	  return 0;
	*((FPLOLOGICAL*)node->data.scalar.defaultvalue)=*((FPLOLOGICAL*)defaultvalue); 
      }
    
    }else if(scalartype==VTYPE_CHAR){  
      if(( node->data.scalar.defaultvalue
	   =allocate(sizeof(char) 
		     ,"CHAR scalar defaultvalue",subfunc))==NULL)
	return 0;
      *((char*)node->data.scalar.defaultvalue)=((char*)defaultvalue)[0]; 


     }else if(scalartype==VTYPE_FLAG){ 
       if(!isinternflag(defaultvalue)){
	 (void)sprintf(errorbuffer,"error(%s): scalar %s defaultvalue (%s) is "
		 "not a valid internal flag representation !\n"
		 ,subfunc,node->name,(char*)defaultvalue);
	 return 0;
	 
       }
       if(( node->data.scalar.defaultvalue
	    =allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		      ,"FLAG scalar defaultvalue",subfunc))==NULL)
	 return 0;
       (void)strcpy( (char*)node->data.scalar.defaultvalue,defaultvalue); 
     }else if(scalartype==VTYPE_STRING){ 
       
       l=teststringlength(defaultvalue,node->data.scalar.strvariable
			  ,node->data.scalar.strlength); 

       
       if(l<0){ 
	 (void)sprintf(errorbuffer
		 ,"error(%s): Scalar %s: defaultvalue (%s) longer "
		 "than string length!"
		 ,subfunc,node->name,(char*)defaultvalue);
	 return 0;
       } 
       
       if(( node->data.scalar.defaultvalue
	    =allocate(sizeof(char)*(l+1) 
		      ,"STRING scalar defaultvalue",subfunc))==NULL)
	 return 0;
       
       (void)strncpy((char*)(node->data.scalar.defaultvalue),defaultvalue,l);  
       ((char*)(node->data.scalar.defaultvalue))[l]='\0';  
       

       
       
     }else{
       goto error;
     }
  }
  
  return 1;
 error:
  (void)sprintf(errorbuffer,"error(%s): Scalar %s: unknown type 0x%lx!\n"
	  ,subfunc,node->name,node->data.scalar.scalartype);
  return 0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static long init_scalar_value(VARTABLE *node)
{
  char *subfunc="init_scalar_value";
  long scalartype,l;
  const char *cdefreal="0.0";
  const char *cdefint="0";
  const char *cdeflog="f";
  const char *cdefchar="\0";
  const char *cdefstring="";
  const char *cdefflag="-NONE"; /* intern flag rep*/
  
  const char *p;

  if(!myassert(node,subfunc,"node")) return 0;
  
  scalartype=node->data.scalar.scalartype;
  
  if(node->data.scalar.value!=NULL){
    FREEE(node->data.scalar.value,node->name);node->data.scalar.value=NULL;
  }
  
  if(scalartype==VTYPE_INT){
   
    if(node->aschar){
      
      p=node->data.scalar.defaultvalue;
      if(p==NULL) p=cdefint;

      if((node->data.scalar.value=allocate(sizeof(char)*(strlen(p)+1),node->name
			      ,subfunc))==NULL) return 0;
      (void)strcpy((char*)(node->data.scalar.value),p);
    }else{   
      if((node->data.scalar.value=allocate(sizeof(FPLOINTEGER),node->name,subfunc))==NULL)
	return 0;
      if(node->data.scalar.defaultvalue==NULL) *((FPLOINTEGER*)(node->data.scalar.value))=0;
      else *((FPLOINTEGER*)(node->data.scalar.value))=*((FPLOINTEGER*)(node->data.scalar.defaultvalue));
    }
  }else if(scalartype==VTYPE_REAL){ 
    if(node->aschar){
      p=node->data.scalar.defaultvalue;
      if(p==NULL) p=cdefreal;
      
      if((node->data.scalar.value=allocate(sizeof(char)*(strlen(p)+1),node->name
			      ,subfunc))==NULL) return 0;
      (void)strcpy((char*)(node->data.scalar.value),p);
    }else{
      if((node->data.scalar.value=allocate(sizeof(FPLODOUBLEPRECISION),node->name
			      ,subfunc))==NULL) return 0;
      if(node->data.scalar.defaultvalue==NULL) 
	*((FPLODOUBLEPRECISION*)(node->data.scalar.value))=(FPLODOUBLEPRECISION)0.0;
      else 
	*((FPLODOUBLEPRECISION*)(node->data.scalar.value))
	  =*((FPLODOUBLEPRECISION*)(node->data.scalar.defaultvalue));

    }
  }else if(scalartype==VTYPE_LOGICAL){ 
    if(node->aschar){
      if((node->data.scalar.value=allocate(sizeof(char),node->name,subfunc))==NULL)
	return 0;
      p=node->data.scalar.defaultvalue;
      if(p==NULL) p=cdeflog;

      *((char*)(node->data.scalar.value))=p[0]; 
    }else{
      if((node->data.scalar.value=allocate(sizeof(FPLOLOGICAL),node->name,subfunc))==NULL)
	return 0;
      if(node->data.scalar.defaultvalue==NULL)
	*((FPLOLOGICAL*)node->data.scalar.value)=L_FALSE; 
      else
	*((FPLOLOGICAL*)node->data.scalar.value)=*((FPLOLOGICAL*)node->data.scalar.defaultvalue); 
    }
  }else if(scalartype==VTYPE_CHAR){ 

    if((node->data.scalar.value=allocate(sizeof(char),node->name,subfunc))==NULL)
      return 0;
    p=node->data.scalar.defaultvalue;
    if(p==NULL) p=cdefchar;

    *((char*)(node->data.scalar.value))=p[0]; 

  }else if(scalartype==VTYPE_STRING){ 
    p=node->data.scalar.defaultvalue;
    if(p==NULL) p=cdefstring;
    l=teststringlength(p,node->data.scalar.strvariable,node->data.scalar.strlength); 

    if(l<0){ 
      (void)sprintf(errorbuffer
	      ,"error(%s): Scalar %s: default value (%s) longer "
	      "than string length!"
	      ,subfunc,node->name,p);
      return 0;
    } 
    
    if((node->data.scalar.value=allocate(sizeof(char)*(strlen(p)+1),node->name
			    ,subfunc))==NULL)
      return 0;


    (void)strcpy((char*)node->data.scalar.value,p); 
    
  }else if(scalartype==VTYPE_FLAG){ 
    p=node->data.scalar.defaultvalue;
    if(p==NULL) p=cdefflag;
    
    
    if((node->data.scalar.value=allocate(sizeof(char)*(strlen(p)+1),node->name
			    ,subfunc))==NULL)
      return 0;
    (void)strcpy((char*)(node->data.scalar.value),p);
    
  }else{
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s: unknown type 0x%lx!"
	    ,subfunc,node->name,node->data.scalar.scalartype);
    return 0;
  }
  

  return 1;
}


/* ***********************************************************************
*
* ***********************************************************************
*/

long vartable_clean_scalar(VARTABLE *node)
{
  char *subfunc="vartable_clean_scalar";
  long scalartype,ret=1;
  



  if(node==NULL) return 1;


  scalartype=node->data.scalar.scalartype;

#ifdef _DEBUG_    
  printf("information(%s): clean %s 0x%lx\n",subfunc,node->name,node->data.scalar.scalartype);
#endif
  
  
  if(scalartype & (VTYPE_INT | VTYPE_REAL | VTYPE_CHAR | VTYPE_LOGICAL) ){
    FREEE(node->data.scalar.value,node->name);node->data.scalar.value=NULL;
    
  }else if(scalartype &  (VTYPE_FLAG|VTYPE_STRING) ){
    FREEE(node->data.scalar.value,node->name);node->data.scalar.value=NULL;
    
  }else{
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s: unknown type 0x%lx!"
	    ,subfunc,node->name,node->data.scalar.scalartype);
    ret=0;
  }
  FREEE(node->data.scalar.defaultvalue,"node->data.scalar.defaultvalue");node->data.scalar.defaultvalue=NULL;
  return ret;


}

/* ***********************************************************************
 * write value into scalar node->value, if aschar as char otherwise as value
 * if aschar=1, the correct type of the value must be assured by the caller
 * ***********************************************************************
 */

long write_into_scalar(VARTABLE *node,const void *value,long copy)
{
  long scalartype;
  long sl;
  char *subfunc="write_into_scalar";


  if(!myassert(node,subfunc,"node"))return 0;

  if(node->structuretype!=VTYPE_SCALAR){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s: not a scalar!"
		  ,subfunc,node->name);
    
    return 0;
  }
  
  scalartype=node->data.scalar.scalartype;
  

  if(node->data.scalar.value!=NULL){
    FREEE(node->data.scalar.value,node->name);node->data.scalar.value=NULL;
  }
  
  if(value==NULL){
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s: value=NULL!"
	    ,subfunc,node->name);
    
    return 0;
  }
  
  if(scalartype==VTYPE_INT){
   
    if(node->aschar){
      if(copy==WRITE_COPY){
	sl=strlen((char*)value);
	if((node->data.scalar.value=allocate(sizeof(char)*(sl+1),node->name
					     ,subfunc))==NULL)
	  return 0;
	(void)strcpy((char*)(node->data.scalar.value),value);
      }else{
	node->data.scalar.value=(char*)value; 
      }
    }else{   
      if((node->data.scalar.value=allocate(sizeof(FPLOINTEGER),node->name,subfunc))==NULL)
	return 0;
      *((FPLOINTEGER*)(node->data.scalar.value))=*((FPLOINTEGER*)value);
    }
  }else if(scalartype==VTYPE_REAL){ 
    if(node->aschar){
      if(copy==WRITE_COPY){
	sl=strlen((char*)value);
	if((node->data.scalar.value=allocate(sizeof(char)*(sl+1),node->name
					     ,subfunc))==NULL)
	  return 0;
	(void)strcpy((char*)(node->data.scalar.value),value);
      }else{
	node->data.scalar.value=(char*)value; 
      }
    }else{
      if((node->data.scalar.value=allocate(sizeof(FPLODOUBLEPRECISION),node->name,subfunc))==NULL)
	return 0;
      *(FPLODOUBLEPRECISION*)(node->data.scalar.value)=*((FPLODOUBLEPRECISION*)(value)); 

    }
  }else if(scalartype==VTYPE_LOGICAL){ 
    if(node->aschar){
      if((node->data.scalar.value=allocate(sizeof(char),node->name,subfunc))==NULL)
	return 0;
      *((char*)(node->data.scalar.value))=((char*)value)[0]; 
    }else{
      if((node->data.scalar.value=allocate(sizeof(FPLOLOGICAL),node->name,subfunc))==NULL)
	return 0;
      *((FPLOLOGICAL*)(node->data.scalar.value))=*((FPLOLOGICAL*)value); 
    }

  }else if(scalartype==VTYPE_CHAR){ 
    if((node->data.scalar.value=allocate(sizeof(char),node->name,subfunc))==NULL)
      return 0;
    *((char*)(node->data.scalar.value))=((char*)value)[0]; 

  }else if(scalartype==VTYPE_STRING){ 
    sl=teststringlength(value,node->data.scalar.strvariable,node->data.scalar.strlength); 
    
    if(sl<0){ 
      (void)sprintf(errorbuffer
	      ,"error(%s): Variable %s: String value longer "
	      "than string length!"
	      ,subfunc,node->name);
      return 0;
    }
    if(copy==WRITE_COPY){
      
      if((node->data.scalar.value=allocate(sizeof(char)*(sl+1),node->name
					   ,subfunc))==NULL)
	return 0;
      
      (void)strncpy((char*)(node->data.scalar.value),value,sl); 
      ((char*)(node->data.scalar.value))[sl]='\0'; 
    }else{

      node->data.scalar.value=(char*)value; 
    }
    
  }else if(scalartype==VTYPE_FLAG){ 
    if(copy==WRITE_COPY){
      sl=strlen((char*)value);
      if((node->data.scalar.value=allocate(sizeof(char)*(sl+1),node->name
					   ,subfunc))==NULL)
	return 0;
      (void)strcpy((char*)(node->data.scalar.value),value);
    }else{
      node->data.scalar.value=(char*)value; 
    }
  }else{
    (void)sprintf(errorbuffer
	    ,"error(%s): Variable %s: unknown type 0x%lx!"
	    ,subfunc,node->name,node->data.scalar.scalartype);
    return 0;
  }
  

  return 1;
}






/* ***********************************************************************
 * tsARRAY7
 * ***********************************************************************
 */

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long set_defaultvalue_array(VARTABLE *node,void *defaultvalue)
{
  char *subfunc="set_defaultvalue_array";
  long scalartype,l,j,n;
  FPLODOUBLEPRECISION d;
  tsARRAY7 *arp;
  FPLOINTEGER i;
  char c1[2];
  char **pds;
  if(!myassert(node,subfunc,"node")) return 0;
  
  if(node->structuretype!=VTYPE_ARRAY){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s: not an array!"
		  ,subfunc,node->name);
    
    return 0;
  }

  arp=&node->data.array7;

  scalartype=arp->scalartype;
  
  FREEE(arp->defaultvalue,"arp->defaultvalue");arp->defaultvalue=NULL;
  
  if(defaultvalue != NULL ){ 
    if(scalartype==VTYPE_REAL){
      if(node->aschar){
	if(!disectDefVal(defaultvalue,&n,&pds)) return 0;

	for(j=0;j<n;j++){

	  if(!convertfortranreal(pds[j+1],&d)){
	    (void)sprintf(errorbuffer,"error(%s): array %s: defaultvalue (%s) "
			  "contains an invalid real!\n"
			  ,subfunc,node->name,(char*)defaultvalue);
	    FREEE(pds[0],"pds[0]"); pds[0]=NULL;
	    FREEE(pds,"pds"); pds=NULL;
	    return 0;
	  }
	}
	FREEE(pds[0],"pds[0]"); pds[0]=NULL;
	FREEE(pds,"pds"); pds=NULL;
	
	if(( arp->defaultvalue
	     =allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		       ,"REAL array defaultvalue",subfunc))==NULL)
	  return 0;
	(void)strcpy( (char*)arp->defaultvalue,defaultvalue); 
	
      }else{
	if(( arp->defaultvalue
	     =allocate(sizeof(FPLODOUBLEPRECISION) 
		       ,"REAL array defaultvalue",subfunc))==NULL)
	  return 0;
	*((FPLODOUBLEPRECISION*)arp->defaultvalue)
	  =*((FPLODOUBLEPRECISION*)defaultvalue);
      }
      
    }else if(scalartype==VTYPE_INT){ 
      if(node->aschar){
	if(!disectDefVal(defaultvalue,&n,&pds)) return 0;

	for(j=0;j<n;j++){
	  if(!convertlong(pds[j+1],&i)){
	    (void)sprintf(errorbuffer,"error(%s): array %s: defaultvalue (%s) "
			  "contains an invalid int!\n"
			  ,subfunc,node->name,(char*)defaultvalue);
	    FREEE(pds[0],"pds[0]"); pds[0]=NULL;
	    FREEE(pds,"pds"); pds=NULL;
	    return 0;
	  }
	}
	FREEE(pds[0],"pds[0]"); pds[0]=NULL;
	FREEE(pds,"pds"); pds=NULL;
	if(( arp->defaultvalue
	     =allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		       ,"INT array defaultvalue",subfunc))==NULL)
	  return 0;
	(void)strcpy( (char*)arp->defaultvalue,defaultvalue); 
      }else{
	if(( arp->defaultvalue
	     =allocate(sizeof(FPLOINTEGER) 
		       ,"INT array defaultvalue",subfunc))==NULL)
	  return 0;	  
	*((FPLOINTEGER*)arp->defaultvalue)
	  =*((FPLOINTEGER*)defaultvalue);
	
      }
      
      
    }else if(scalartype==VTYPE_LOGICAL){ 
      if(node->aschar){
	if(!( tolower(((char*)defaultvalue)[0])=='t' 
	      || tolower(((char*)defaultvalue)[0])=='f' )){
	  c1[0]=((char*)defaultvalue)[0]; c1[1]='\0';
	  if(c1[0]=='\0') (void)strcpy(c1,"\0"); 
	    
	  (void)sprintf(errorbuffer,"error(%s): array %s: defaultvalue (%s) is "
		  "not a valid logical!\n"
		  ,subfunc,node->name,c1);
	  return 0;
	  
	}
	if(( arp->defaultvalue
	     =allocate(sizeof(char) 
		       ,"LOGICAL array defaultvalue",subfunc))==NULL)
	  return 0;
	*((char*)arp->defaultvalue)=tolower(((char*)defaultvalue)[0]); 
      }else{
	if(( arp->defaultvalue
	     =allocate(sizeof(FPLOLOGICAL) 
		       ,"LOGICAL array defaultvalue",subfunc))==NULL)
	  return 0;
	*((FPLOLOGICAL*)arp->defaultvalue)=*((FPLOLOGICAL*)defaultvalue); 
      }
    
    }else if(scalartype==VTYPE_CHAR){  
      if(( arp->defaultvalue
	   =allocate(sizeof(char) 
		     ,"CHAR array defaultvalue",subfunc))==NULL)
	return 0;
      *((char*)arp->defaultvalue)=((char*)defaultvalue)[0]; 


     }else if(scalartype==VTYPE_FLAG){ 
      if(!disectDefVal(defaultvalue,&n,&pds)) return 0;
      for(j=0;j<n;j++){
	if(!isinternflag(pds[j+1])){
	  (void)sprintf(errorbuffer,"error(%s): array %s: defaultvalue (%s) "
			"contains an invalid internal flag representation !\n"
			,subfunc,node->name,(char*)defaultvalue);
	  FREEE(pds[0],"pds[0]"); pds[0]=NULL;
	  FREEE(pds,"pds"); pds=NULL;
	  return 0;
	  
	}
      }      
      FREEE(pds[0],"pds[0]"); pds[0]=NULL;
      FREEE(pds,"pds"); pds=NULL;
       if(( arp->defaultvalue
	    =allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		      ,"FLAG array defaultvalue",subfunc))==NULL)
	 return 0;
	(void)strcpy( (char*)arp->defaultvalue,defaultvalue); 
     }else if(scalartype==VTYPE_STRING){ 
       l=teststringlength(defaultvalue,arp->strvariable,arp->strlength); 
       
       if(l<0){ 
	 (void)sprintf(errorbuffer
		 ,"error(%s): Array %s: defaultvalue (%s) longer "
		 "than string length!"
		 ,subfunc,node->name,(char*)defaultvalue);
	 return 0;
       } 
       
       if(( arp->defaultvalue
	    =allocate(sizeof(char)*(l+1) 
		      ,"STRING array defaultvalue",subfunc))==NULL)
	 return 0;
       (void)strncpy((char*)(arp->defaultvalue),defaultvalue,l); 
       ((char*)(arp->defaultvalue))[l]='\0'; 


       
       
     }else{
       goto error;
     }
  }
  
  return 1;
 error:
  (void)sprintf(errorbuffer,"error(%s): Array %s: unknown type 0x%lx!\n"
	  ,subfunc,node->name,arp->scalartype);
  return 0;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long allocate_array(VARTABLE *node,long reallocarray )
{
  char *subfunc="allocate_array";
  tsARRAY7 *arp;
  long newsize/* ,scalartype */;
  
  
  if(!myassert(node,subfunc,"node")) return 0;

  arp=&node->data.array7;
  if(arp->external) {
    arp->allocated=1;
    return 1;
  }
  
  if(arp->allocated){
    return 1;
/*     (void)sprintf(errorbuffer,"error(%s): array(%s): cannot call " */
/* 	    "allocate_array twice!\n" */
/* 	    ,subfunc,node->name); */
/*     return 0; */
  }

  arp->allocated=1;
  
  /* scalartype=arp->scalartype; */
  
  arp->currentsize=0;
  if(arp->variable){
    if(reallocarray)
      newsize=arp->Size[arp->rank];
    else
      newsize=INITIAL_ALLOC_SIZE;
  }else{
    newsize=arp->Size[arp->rank];
  }

  if(!intern_reallocate_array(arp,0L,newsize,node->aschar,node->name))return 0;


  return 1;  
}

/* ***********************************************************************
*
* ***********************************************************************
*/

long vartable_clean_array(VARTABLE *node)
{
  char *subfunc="vartable_clean_array";
  long scalartype,ret=1,i,ind;
  tsARRAY7 *arp;
    

  if(node==NULL) return 1;
  arp=&node->data.array7;


  scalartype=arp->scalartype;

#ifdef _DEBUG_    
  printf("information(%s): clean %s 0x%lx\n",subfunc,node->name,arp->scalartype);
#endif
  
  
  for(i=1;i<8;i++){  
    if(arp->idref[i]!=NULL){
      FREEE(arp->idref[i],"arp->idref[i]");arp->idref[i]=NULL; 
    }
  } 

  if(scalartype & (VTYPE_REAL | VTYPE_INT| VTYPE_LOGICAL | VTYPE_CHAR 
		   |VTYPE_FLAG|VTYPE_STRING) ){ 
    node->aschar=node->aschar && (scalartype & (VTYPE_REAL | VTYPE_INT) );
    node->aschar=node->aschar || (scalartype & (VTYPE_FLAG | VTYPE_STRING) );
    if(!node->aschar){
      if(!arp->external){
	FREEE(arp->array,node->name); arp->array=NULL;
      }
    }else{
      if(!arp->external){
	if(arp->array){
	  ind=LMAX(arp->Size[arp->rank],arp->currentsize); 
	  switch ((int)arp->rank){ 
	  case 5: 
	    ind=ind*arp->Size[4]; 
	  case 4: 
	    ind=ind*arp->Size[3]; 
	  case 3: 
	    ind=ind*arp->Size[2]; 
	  case 2: 
	    ind=ind*arp->Size[1]; 
	  case 1: 
	    break; 
	  default: 
	    (void)sprintf(errorbuffer
		    ,"error(%s): Array %s has to many dimensions(%ld)!"
		    ,subfunc,node->name,arp->rank);
	    return 0;
	  } 
	  
	  
	  for(i=0;i<ind;i++){  
	    FREEE(((char**)(arp->array))[i],((char**)(arp->array))[i]); 
	    ((char**)(arp->array))[i]=NULL;
	  } 
	  FREEE(arp->array,node->name); arp->array=NULL;
	  
	}
      }
    }    
  }else{
    (void)sprintf(errorbuffer
	    ,"error(%s): Array %s: unknown type 0x%lx!"
	    ,subfunc,node->name,arp->scalartype);
    ret=0;
  }
  FREEE(arp->defaultvalue,"arp->defaultvalue");arp->defaultvalue=NULL;
  return ret;


}

/* ***********************************************************************
 * reallocate array corresponding to sizes
 * ***********************************************************************
 */
static long intern_reallocate_array(tsARRAY7 *arp,long oldsize
				    ,long newsize,long aschar,const char *name)
{
  char *subfunc="intern_reallocate_array";
  long i,size,n;
  long oldcurrentsize;
  long scalartype;
  long l,update;
  const char *cdefreal="0.0";
  const char *cdefint="0";
  const char *cdeflog="f";
  const char *cdefchar="\0";
  const char *cdefstring="";
  const char *cdefflag="-NONE"; /* intern flag rep*/
  
  const char *pdef;
  char **pds;
    
  if(!myassert(arp,subfunc,"arp")) return 0;

  scalartype=arp->scalartype;
  
  
  
  oldcurrentsize=arp->currentsize;
  update=newsize > oldcurrentsize || (newsize==0 && oldcurrentsize == 0);
  if(update){  
    arp->currentsize=LMAX(newsize,oldcurrentsize+INITIAL_ALLOC_SIZE);
  }
  size=arp->currentsize*arp->lowdim;

  if(scalartype==VTYPE_REAL){
    
    if(aschar){
      if(update){      
	if((arp->array
	    =reallocate(arp->array,sizeof(char*)*(size)
			,"REAL ARRAY",subfunc))==NULL)
	  return 0;
      }

      pdef=arp->defaultvalue;
      if(pdef==NULL) pdef=cdefreal;
      

      if(!disectDefVal(pdef,&n,&pds)) return 0;

      
      
      for(i=arp->lowdim*oldsize;i<arp->lowdim*oldcurrentsize;i++){ 
	FREEE(((char**)(arp->array))[i],"((char**)(arp->array))[i]");
	((char**)(arp->array))[i]=NULL;
      }
      
      for(i=arp->lowdim*oldsize;i<size;i++){ 
	if((((char**)(arp->array))[i]
	    =allocate(sizeof(char)*(strlen(pds[i%n+1])+1)
		      ,"REAL ARRAY VALUE",subfunc))==NULL){
	  FREEE(pds[0],"pds[0]");pds[0]=NULL;
	  FREEE(pds,"pds");pds=NULL;
	  return 0;
	}
	(void)strcpy(((char**)(arp->array))[i],pds[i%n+1]); 
      } 
      FREEE(pds[0],"pds[0]");pds[0]=NULL;
      FREEE(pds,"pds");pds=NULL;
      
      
    }else{
      if(update){      
	if((arp->array
	    =(FPLODOUBLEPRECISION*)reallocate(arp->array
					  ,sizeof(FPLODOUBLEPRECISION)*(size)
					  ,"REAL VALUE",subfunc))==NULL)
	  return 0;
      }
      for(i=arp->lowdim*oldsize;i<size;i++){
	if(arp->defaultvalue==NULL)
	  ((FPLODOUBLEPRECISION*)arp->array)[i]=(FPLODOUBLEPRECISION)0.0;
	else
	  ((FPLODOUBLEPRECISION*)arp->array)[i]=
	    *((FPLODOUBLEPRECISION*)arp->defaultvalue);
      }
    }
  }else if(scalartype==VTYPE_INT){ 
    
    if(aschar){
      if(update){      
	if((arp->array
	    =reallocate(arp->array,sizeof(char*)*(size)
			,"INTEGER ARRAY",subfunc))==NULL)
	  return 0;
      }
      pdef=arp->defaultvalue;
      if(pdef==NULL) pdef=cdefint;
      
      if(!disectDefVal(pdef,&n,&pds)) return 0;

      for(i=arp->lowdim*oldsize;i<arp->lowdim*oldcurrentsize;i++){ 
	FREEE(((char**)(arp->array))[i],"((char**)(arp->array))[i]");
	((char**)(arp->array))[i]=NULL;
      }
      
      for(i=arp->lowdim*oldsize;i<size;i++){ 
	if((((char**)(arp->array))[i]
	    =allocate(sizeof(char)*(strlen(pds[i%n+1])+1)
		      ,"INTEGER ARRAY VALUE",subfunc))==NULL){
	  FREEE(pds[0],"pds[0]");pds[0]=NULL;
	  FREEE(pds,"pds");pds=NULL;
	  return 0;
	}
	(void)strcpy(((char**)(arp->array))[i],pds[i%n+1]); 
      } 
      FREEE(pds[0],"pds[0]");pds[0]=NULL;
      FREEE(pds,"pds");pds=NULL;
      
    }else{
      if(update){
	if((arp->array
	    =(FPLOINTEGER*)reallocate(arp->array,sizeof(FPLOINTEGER)*(size)
				  ,"INTEGER VALUE",subfunc))==NULL)
	  return 0;
      }
      for(i=arp->lowdim*oldsize;i<size;i++){
	if(arp->defaultvalue==NULL) ((FPLOINTEGER*)arp->array)[i]=0;
	else ((FPLOINTEGER*)arp->array)[i]= *((FPLOINTEGER*)arp->defaultvalue);
      }
    }
  }else if(scalartype==VTYPE_LOGICAL){ 
    
    if(aschar){
      if(update){      
	if((arp->array
	    =(char*)reallocate(arp->array,sizeof(char)*(size)
			       ,"LOGICAL VALUE",subfunc))==NULL)
	  return 0;
      }
      pdef=arp->defaultvalue;
      if(pdef==NULL) pdef=cdeflog;
      
      for(i=arp->lowdim*oldsize;i<size;i++){ 
	((char*)arp->array)[i]=pdef[0]; 
      } 
      
    }else{
      if(update){      
	if((arp->array
	    =(FPLOLOGICAL*)reallocate(arp->array,sizeof(FPLOLOGICAL)*(size)
				  ,"LOGICAL VALUE",subfunc))==NULL)
	  return 0;
      }
      for(i=arp->lowdim*oldsize;i<size;i++){ 
	if(arp->defaultvalue==NULL)
	  ((FPLOLOGICAL*)arp->array)[i]=L_FALSE; 
	else
	  ((FPLOLOGICAL*)arp->array)[i]=*((FPLOLOGICAL*)arp->defaultvalue); 
      } 
    }
  }else if(scalartype==VTYPE_CHAR){ 
    if(update){      
      if((arp->array
	  =(char*)reallocate(arp->array,sizeof(char)*(size)
			     ,"CHAR VALUE",subfunc))==NULL)
	return 0;
    }
    pdef=arp->defaultvalue;
    if(pdef==NULL) pdef=cdefchar;
    
    for(i=arp->lowdim*oldsize;i<size;i++){ 
      ((char*)arp->array)[i]=pdef[0]; 
      
      
    }
  }else if(scalartype==VTYPE_FLAG){ 
    if(update){      
      if((arp->array
	  =reallocate(arp->array,sizeof(char*)*(size)
		      ,"FLAG ARRAY",subfunc))==NULL)
	return 0;
    }      
    pdef=arp->defaultvalue;
    if(pdef==NULL) pdef=cdefflag;
    

    if(!disectDefVal(pdef,&n,&pds)) return 0;


    for(i=arp->lowdim*oldsize;i<arp->lowdim*oldcurrentsize;i++){ 
      FREEE(((char**)(arp->array))[i],"((char**)(arp->array))[i]");
      ((char**)(arp->array))[i]=NULL;
    }
    
    for(i=arp->lowdim*oldsize;i<size;i++){ 
      if((((char**)(arp->array))[i]
	  =allocate(sizeof(char)*(strlen(pds[i%n+1])+1)
		    ,"INTEGER ARRAY VALUE",subfunc))==NULL){
	FREEE(pds[0],"pds[0]");pds[0]=NULL;
	FREEE(pds,"pds");pds=NULL;
	return 0;
      }
     
      
      (void)strcpy(((char**)(arp->array))[i],pds[i%n+1]);  
      
    } 
    FREEE(pds[0],"pds[0]");pds[0]=NULL;
    FREEE(pds,"pds");pds=NULL;

  }else if(scalartype==VTYPE_STRING){ 
    if(update){      
      if((arp->array
	  =reallocate(arp->array,sizeof(char*)*(size)
		      ,"STRING ARRAY",subfunc))==NULL)
	return 0;
    }      
    pdef=arp->defaultvalue;
    if(pdef==NULL) pdef=cdefstring;
    
    l=teststringlength(pdef,arp->strvariable,arp->strlength); 
    
    if(l<0){ 
      (void)sprintf(errorbuffer
	      ,"error(%s): Array %s: default value (%s) longer "
		    "than string length!"
		    ,subfunc,name,pdef);
      return 0;
    } 
    for(i=arp->lowdim*oldsize;i<arp->lowdim*oldcurrentsize;i++){ 
      FREEE(((char**)(arp->array))[i],"((char**)(arp->array))[i]");
      ((char**)(arp->array))[i]=NULL;
    }
    
    for(i=arp->lowdim*oldsize;i<size;i++){ 
      if((((char**)(arp->array))[i]
	  =allocate(sizeof(char)*(l+1)
		    ,"STRING ARRAY VALUE",subfunc))==NULL)
	return 0;
      
      (void)strcpy(((char**)(arp->array))[i],pdef); 
      
    } 
    
  }else{
    goto error;
  }
  
  
  return 1;
 error:
  (void)sprintf(errorbuffer,"error(%s): Array %s: unknown type 0x%lx!\n"
	  ,subfunc,name,arp->scalartype);
  return 0;
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

#define TESTINDEX(i,limit) \
    if( indx[(i)]<(limit) || indx[(i)] > arp->Size[(i)]){\
      (void)sprintf(errorbuffer\
        ,"error(%s): array %s dimension %ld index=%ld out "\
        "of bound [%ld:%ld]!"\
      ,subfunc,node->name,(i),indx[(i)],(limit),arp->Size[(i)]);\
      return 0;\
    }

long write_into_array(VARTABLE *node,long indx[8],const void *value,long copy)
{
  char *subfunc="write_into_array";
  long ind;
  long rank;
  long maxindx;
  long scalartype;
  long l;
  tsARRAY7 *arp;
  
  if(!myassert(node,subfunc,"node")) return 0;
  arp=&node->data.array7;

  if(node->structuretype!=VTYPE_ARRAY){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s: not an array!"
		  ,subfunc,node->name);
    
    return 0;
  }
  


  
  if(!arp->allocated){
    if(! allocate_array(node,0L)) return 0;
/*     (void)sprintf(errorbuffer,"internalerror(%s): array(%s): cannot insert " */
/* 	    "element before allocation!\n" */
/* 	    ,subfunc,node->name); */
/*     return 0; */
  }


  scalartype=arp->scalartype;
  
  rank=arp->rank;
#ifdef _DEBUG_  
  printf("write_into_array: %s 0x%lx %ld\n",(char*)value,scalartype,rank);
#endif




  maxindx=indx[rank];

  if(boundcheck){
    switch ((int)rank){
    case 5:
      TESTINDEX(4L,1L);
    case 4:
      TESTINDEX(3L,1L);
    case 3:
      TESTINDEX(2L,1L);
    case 2:
      TESTINDEX(1L,1L);
    case 1:
      break;
    }
    if(arp->variable){
      if( maxindx<1 ){
	(void)sprintf(errorbuffer,"error(%s):Array %s variable dimension "
		"%ld index=%ld out of bound [%ld:]!",subfunc,node->name
		,rank,maxindx,1L);
	
	return 0;
      }
    }else{
      TESTINDEX(rank,1L);
    }
  }
  
  
  ind=maxindx-1;
  switch ((int)rank){
  case 5:
    ind=indx[4]-1+ind*arp->Size[4];
  case 4:
    ind=indx[3]-1+ind*arp->Size[3];
  case 3:
    ind=indx[2]-1+ind*arp->Size[2];
  case 2:
    ind=indx[1]-1+ind*arp->Size[1];
  case 1:
    break;
  default:
    (void)sprintf(errorbuffer,"error(%s):"
	    "Array %s to many dimensions(%ld)!",subfunc,node->name,rank);
    return 0;
  }

#ifdef _DEBUG_  
  fprintf(stderr,"ARRAY7 %s size %lx cusize %lx!\n",node->name,arp->size[arp->rank],arp->currentsize);
#endif




  if(arp->variable){
    
    if(!intern_reallocate_array(arp,arp->Size[arp->rank],maxindx,node->aschar,node->name)) 
      return 0;
          
    arp->Size[rank]=LMAX(maxindx,arp->Size[rank]);
  }
  
  if(value != NULL ){ /* value == NULL  for adjusting variable dimensions */  
    if(scalartype==VTYPE_REAL){
      if(node->aschar){
	if(copy==WRITE_COPY){
	  if(((char**)(arp->array))[ind]==NULL){ 
	    if((((char**)(arp->array))[ind]
		=allocate(sizeof(char)*(strlen(value)+1) 
			  ,"REAL ARRAY VALUE",subfunc))==NULL)
	      return 0;
	    
	  }else if( strlen(((char**)(arp->array))[ind])<strlen(value) ){ 
	    if((((char**)(arp->array))[ind]
		=reallocate(((char**)(arp->array))[ind]
			    ,sizeof(char)*(strlen(value)+1)
			    ,"REAL ARRAY VALUE",subfunc))==NULL)
	      return 0;
	  } 
	  (void)strcpy(((char**)(arp->array))[ind],value); 
	}else{
	  if(((char**)(arp->array))[ind]!=NULL){
	    FREEE(((char**)(arp->array))[ind],"((char**)(arp->array))[ind]");
	  }
	  ((char**)(arp->array))[ind]=(char*)value; 
	}
      }else{
	((FPLODOUBLEPRECISION*)arp->array)[ind]=*((FPLODOUBLEPRECISION*)value);
      }

    }else if(scalartype==VTYPE_INT){ 
      if(node->aschar){
       if(copy==WRITE_COPY){
	 if(((char**)(arp->array))[ind]==NULL){ 
	   if((((char**)(arp->array))[ind]
	       =allocate(sizeof(char)*(strlen(value)+1) 
			 ,"INTEGER ARRAY VALUE",subfunc))==NULL)
	     return 0;
	  
	 }else if( strlen(((char**)(arp->array))[ind])<strlen(value) ){ 
	   if((((char**)(arp->array))[ind]
	       =reallocate(((char**)(arp->array))[ind]
			   ,sizeof(char)*(strlen(value)+1)
			   ,"INTEGER ARRAY VALUE",subfunc))==NULL)
	     return 0;
	 } 
	 (void)strcpy(((char**)(arp->array))[ind],value); 
       }else{
	 if(((char**)(arp->array))[ind]!=NULL){
	   FREEE(((char**)(arp->array))[ind],"((char**)(arp->array))[ind]");
	 }
	 ((char**)(arp->array))[ind]=(char*)value; 

       }
      }else{
	((FPLOINTEGER*)arp->array)[ind]=*((FPLOINTEGER*)value);
      }

      
    }else if(scalartype==VTYPE_LOGICAL){ 
      if(node->aschar){
	((char*)arp->array)[ind]=((char*)value)[0]; 
      }else{
	((FPLOLOGICAL*)arp->array)[ind]=*((FPLOLOGICAL*)value); 
      }
    
    }else if(scalartype==VTYPE_CHAR){  

      ((char*)arp->array)[ind]=((char*)value)[0]; 

     }else if(scalartype==VTYPE_FLAG){ 
       if(copy==WRITE_COPY){
	 if(((char**)(arp->array))[ind]==NULL){ 
	   if((((char**)(arp->array))[ind]
	       =allocate(sizeof(char)*(strlen(value)+1) 
			 ,"FLAG ARRAY VALUE",subfunc))==NULL)
	     return 0;
	   
	 }else if( strlen(((char**)(arp->array))[ind])<strlen(value) ){ 
	   if((((char**)(arp->array))[ind]
	       =reallocate(((char**)(arp->array))[ind]
			   ,sizeof(char)*(strlen(value)+1)
			   ,"FLAG ARRAY VALUE",subfunc))==NULL)
	     return 0;
	 } 
	 
	 (void)strcpy(((char**)(arp->array))[ind],value); 
       }else{
	 if(((char**)(arp->array))[ind]!=NULL){
	   FREEE(((char**)(arp->array))[ind],"FLAG ARRAY VALUE");
	 }
	 ((char**)(arp->array))[ind]=(char*)value; 
       }
     }else if(scalartype==VTYPE_STRING){ 
       l=teststringlength(value,arp->strvariable,arp->strlength); 
       
       if(l<0){ 
	 (void)sprintf(errorbuffer
		 ,"error(%s): Array %s: String value longer "
		 "than string length!"
		 ,subfunc,node->name);
	 return 0;
       } 
       if(copy==WRITE_COPY){
	 
	 if(((char**)(arp->array))[ind]==NULL){ 
	   if((((char**)(arp->array))[ind]
	       =allocate(sizeof(char)*(l+1) 
			 ,"STRING ARRAY VALUE",subfunc))==NULL)
	     return 0;
	   
	 }else if( strlen(((char**)(arp->array))[ind])<(unsigned long)l ){ 
	   if((((char**)(arp->array))[ind]
	       =reallocate(((char**)(arp->array))[ind]
			   ,sizeof(char)*(l+1)
			   ,"STRING ARRAY VALUE",subfunc))==NULL)
	     return 0;
	 } 
	 
	 (void)strncpy(((char**)(arp->array))[ind],value,l); 
	 ((char**)(arp->array))[ind][l]='\0'; 
       }else{
	 if(((char**)(arp->array))[ind]!=NULL){
	   FREEE(((char**)(arp->array))[ind],"((char**)(arp->array))[ind]");
	 }
	 ((char**)(arp->array))[ind]=(char*)value; 
       }

    }else{
      goto error;
    }
  }
  
  return 1;
 error:
  (void)sprintf(errorbuffer,"error(%s): Array %s: unknown type 0x%lx!\n"
	  ,subfunc,node->name,arp->scalartype);
  return 0;
  
  
}

#undef TESTINDEX

/* ***********************************************************************
 * reallocate array corresponding to sizes
 * ***********************************************************************
 */
long resize_vtarray(VARTABLE *node,long sizes[8])
{
  char *subfunc="resize_vtarray";
  tsARRAY7 *arp;
  long i;
  long rank;
  long maxsize;
#ifdef _DEBUG_  
  long scalartype;
#endif  

  if(!myassert(node,subfunc,"node")) return 0;
  if(node->structuretype!=VTYPE_ARRAY) return 0;

  arp=&node->data.array7;
  
 
#ifdef _DEBUG_  
 scalartype=arp->scalartype;
#endif  
  rank=arp->rank;
  
  if(!arp->allocated){
    (void)sprintf(errorbuffer,"error(%s): array(%s): cannot call "
	    "%s before allocate_array!\n"
	    ,subfunc,node->name,subfunc);
    return 0;
  }


#ifdef _DEBUG_  
  printf("reallocate_array: %s 0x%lx %ld\n",(char*)value,scalartype,rank);
#endif



  for(i=1;i<rank;i++){

    if(arp->Size[i]!=sizes[i]){
      (void)sprintf(errorbuffer,"error(%s): array (%s): in the moment size update\n"
	      " is not implemented for lower dimensions of \n"
	      " multidimensional arrays!\n"
	      ,subfunc,node->name);
      return 0;
      
    }
  }



  
#define TESTSIZE(i,limit) \
    if( sizes[(i)]<(limit) ){\
      (void)sprintf(errorbuffer\
        ,"error(%s): array %s: dimension %ld: new size=%ld < %ld! "\
      ,subfunc,node->name,(i),sizes[(i)],(limit));\
      return 0;\
    }
  
  
  maxsize=sizes[rank];

  switch ((int)rank){
  case 5:
    TESTSIZE(4L,1L);
  case 4:
    TESTSIZE(3L,1L);
  case 3:
    TESTSIZE(2L,1L);
  case 2:
    TESTSIZE(1L,1L);
  case 1:
    break;
  }
  TESTSIZE(rank,0L);
  
  
  
  
#ifdef _DEBUG_  
  fprintf(stderr,"realo ARRAY7 %s size %lx cusize %lx!\n",node->name,arp->size[arp->rank],arp->currentsize);

#endif
  



  if(!intern_reallocate_array(arp,arp->Size[arp->rank],maxsize,node->aschar,node->name)) 
    return 0;
    
  
  arp->Size[rank]=maxsize;
  
  
  
  return 1;
  
}








/* ***********************************************************************
 * list (struct)
 * ***********************************************************************
 */


/* ***********************************************************************
 * take definition of from and create a equivalrent vartable in to
 * ***********************************************************************
 */
static long create_struct_from_template(VARTABLE *dest,VARTABLE *template
					,long allocarray,long reallocarray)
{
  char *subfunc="create_struct_from_template";
  VARTABLE *node;
  tsARRAY7 *arp;

  if(!myassert(dest,subfunc,"dest")) return 0;
  if(!myassert(template,subfunc,"template")) return 0;
  
  /* printf("%s\n",subfunc);  */
  
  if(!init_vartable(dest,template->aschar))return 0;
  
  /* use template as pattern */
  
  node=template->next;  
  while(node!=NULL){ /* for all src members */
    /* printf("%s copy %s\n",subfunc,node->name);  */

    if(node->structuretype==VTYPE_SCALAR){
/*       HIER("copy scalar"); */
/*       HIER(((SCALAR*)p->var)->name); */
      if((append_scalar_to_list(dest,node->name
				,node->data.scalar.scalartype
				,node->data.scalar.strlength
				,node->data.scalar.strvariable
				,node->data.scalar.defaultvalue
				))==NULL)
	return 0;
      
    }else if(node->structuretype==VTYPE_ARRAY){
/*       HIER("copy array"); */
/*       HIER(((ARRAY7*)p->var)->name); */

      arp=&node->data.array7;
      
      if(arp->rank!=1){
	(void)sprintf(errorbuffer,"Cannot have a multidimensional array "
		"as structarray member!");
	return 0;
	
      }
      if((append_array_to_list(dest,node->name
			       ,arp->scalartype
			       ,arp->rank
			       ,arp->Size
			       ,arp->idref
			       ,arp->variable
			       ,arp->strlength
			       ,arp->strvariable
			       ,arp->defaultvalue
			       ,NULL
			       ,allocarray
			       ,reallocarray
			       ))==NULL)
	return 0;
    }else if(node->structuretype==VTYPE_STRUCTARRAY){

      /* printf("%s     copy struct array %ld %p\n",subfunc,allocarray,node->data.structarray.template);  */
	  
      if((append_structarray_to_list(dest,node->name
					 ,node->data.structarray.Size
					 ,node->data.structarray.variable
				         ,node->data.structarray.idref
					 ,node->data.structarray.typetemplate
				         ,allocarray
					 ))==NULL)
	return 0;

    }else if(node->structuretype==VTYPE_STRUCT){

      if((append_struct_to_list(dest,node->name,node->data.list,allocarray))==NULL)
	return 0;

    }else{ 
      (void)sprintf(errorbuffer,"error(%s): type 0x%lx not allowed in structarray!\n"
	      ,subfunc,node->structuretype);
      return 0;
    } 
    
    node=node->next;
  }
  return 1;
  
}
#undef TESTSIZE

















/* ***********************************************************************
 * array of lists (structarray)
 * ***********************************************************************
 */

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long allocate_structarray(VARTABLE *node )
{
  char *subfunc="allocate_structarray";
  long newsize;  
  tsSTRUCTARRAY *sap;

  /* printf("%s name=%s\n",subfunc,node->name); */


  
  if(!myassert(node,subfunc,"node")) return 0;
  sap=&node->data.structarray;


  if(node->structuretype!=VTYPE_STRUCTARRAY){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s: not an structarray!"
		  ,subfunc,node->name);
    
    return 0;
  }

  if(sap->allocated){
    /* printf("%s %s\n",subfunc,"allready allocated"); */
    return 1;
/*     (void)sprintf(errorbuffer,"error(%s): structarray(%s): cannot call " */
/* 	    "allocate_structarray twice!\n" */
/* 	    ,subfunc,node->name); */
/*     return 0; */
  }
  sap->allocated=1;

  sap->currentsize=0;
  if(sap->variable){
    newsize=INITIAL_ALLOC_SIZE;
  }else{
    newsize=sap->Size;
  }
  
  if(!intern_reallocate_structarray(sap,0L,newsize))return 0;

  return 1;


}
/* ***********************************************************************
 * reallocate array corresponding to sizes
 * ***********************************************************************
 */
static long intern_reallocate_structarray(tsSTRUCTARRAY *sap
					  ,long oldsize
					  ,long newsize
					  )
{
  char *subfunc="intern_reallocate_structarray";
  long i,update;
  long oldcurrentsize;
 
  if(!myassert(sap,subfunc,"sap")) return 0;
  
  /* printf("%s oldsize=%ld newsize=%ld\n",subfunc,oldsize,newsize); */

  oldcurrentsize=sap->currentsize;

  update=newsize > oldcurrentsize || (newsize==0 && oldcurrentsize == 0);

  if(update){  
    sap->currentsize=LMAX(newsize,oldcurrentsize+INITIAL_ALLOC_SIZE);
  }
  
  if(update){  

    /* printf("%s reallocate new currentsize=%ld\n",subfunc,sap->currentsize); */
  
    if((sap->array
	=reallocate(sap->array,sizeof(VARTABLE**)*(sap->currentsize)
		    ,"STRUCTARRAY",subfunc))==NULL)
      return 0;
  }    
  for(i=oldsize;i<oldcurrentsize;i++){ 
    if(!clean_vartable(sap->array[i]))
      return 0;
    FREEE(sap->array[i],"sap->array[i]"); sap->array[i]=NULL;
  }
  
  for(i=oldsize;i<sap->currentsize;i++){ 
    if((sap->array[i]
	=allocate(sizeof(VARTABLE),"STRUCTARRAY VALUE",subfunc))==NULL)
      return 0;
    /* printf("%s copy template in %ld\n",subfunc,i);  */
    if(!create_struct_from_template(sap->array[i],sap->typetemplate,1L,1L))
      return 0;
  } 


  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long vartable_clean_structarray(VARTABLE *node)
{
  char *subfunc="vartable_clean_structarray"; 
  long ret=1,ind,i;
  tsSTRUCTARRAY *sap;
  subfunc=subfunc;
  if(node==NULL) return 1;
  sap=&node->data.structarray;
  

#ifdef _DEBUG_    
  printf("information(%s): clean structarray %s \n",subfunc,node->name);
#endif


  if(!sap->typetemplateiscopy){
    if(!clean_vartable(sap->typetemplate))
      ret=0;
    FREEE(sap->typetemplate,node->name);  
  }
  sap->typetemplate=NULL;
  


  if(sap->idref!=NULL){
    FREEE(sap->idref,"sap->idref");sap->idref=NULL; 
  }


  if(sap->array){
    ind=LMAX(sap->Size,sap->currentsize); 
    
    for(i=0;i<ind;i++){  
      if(!clean_vartable(sap->array[i]))
	ret=0;
      FREEE(sap->array[i],node->name); sap->array[i]=NULL;
    } 
  }
  
  FREEE(sap->array,node->name); sap->array=NULL;
  
  return ret;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
VARTABLE *insert_structelement_into_structarray(VARTABLE *node,long indx)
{
  char *subfunc="insert_struct_into_structarray";
  tsSTRUCTARRAY *sap;

  if(!myassert(node,subfunc,"node")) return 0;
  sap=&node->data.structarray;

  /* printf("%s name=%s indx=%ld\n",subfunc,node->name,indx); */

      

  if(!sap->allocated){
    if(! allocate_structarray(node)) return 0; 
/*     (void)sprintf(errorbuffer,"internalerror(%s): structarray(%s): cannot insert " */
/* 	    "element before allocation!\n" */
/* 	    ,subfunc,node->name); */
/*     return 0; */
  }


  if(boundcheck){
    
    if(sap->variable){
      if( indx<1 ){
	(void)sprintf(errorbuffer,"error(%s):Structarray %s: variable dimension "
		" index=%ld out of bound [%ld:]!",subfunc,node->name
		,indx,1L);
	
	return 0;
      }
    }else{
      if( indx<1L || indx > sap->Size){
	(void)sprintf(errorbuffer
		,"error(%s): structarray %s index=%ld out "
		"of bound [%ld:%ld]!"
		,subfunc,node->name,indx,1L,sap->Size);
	return 0;
      }
      
      
    }
  }
  
  

  if(sap->variable){
    
    if(!intern_reallocate_structarray(sap,sap->Size,indx))return 0;
    
    sap->Size=LMAX(indx,sap->Size);
  }
  
  return sap->array[indx-1];
  
}
/* ***********************************************************************
 * reallocate structarray corresponding to sizes
 * ***********************************************************************
 */
long resize_structarray(VARTABLE *node,long size)
{
  char *subfunc="resize_structarray";
  tsSTRUCTARRAY *sap;

  if(!myassert(node,subfunc,"node")) return 0;
  sap=&node->data.structarray;

  if(!sap->allocated){
    (void)sprintf(errorbuffer,"error(%s): structarray(%s): cannot call "
	    "%s before allocate_array!\n"
	    ,subfunc,node->name,subfunc);
    return 0;
  }

  if( size<0 ){
    (void)sprintf(errorbuffer
	    ,"error(%s): structarray %s:  new size=%ld < %ld! "
	    ,subfunc,node->name,size,0L);
    return 0;
  }
  

  if(!intern_reallocate_structarray(sap,sap->Size,size))return 0;
  
  sap->Size=size;
  
  return 1;
}

/* ***********************************************************************
 * disect the defaultvalues
 * ***********************************************************************
 */

static long disectDefVal(const char *defaultvalue,long *ndefval,char ***pds)
{
  char *subfunc="disectDefVal";
  long j;
  char *p0,*p,*buf;

  if(( buf=allocate(sizeof(char)*(strlen(defaultvalue)+1) 
		    ,"buf",subfunc))==NULL) return 0;

  (void)strcpy(buf,defaultvalue);
  removetrailingspace(buf);




  (*ndefval)=ntoken(buf);

  if(( (*pds)=allocate(sizeof(char*)*(*ndefval+1)
		       ,"buf",subfunc))==NULL) {
    FREEE(buf,"buf");
    return 0;
  }

  
  (*pds)[0]=buf;

  p0=buf;
  for(j=0;j<*ndefval;j++){
    p=sscan(&p0);
    (*pds)[j+1]=p;
  }
  
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
long create_template_from_defaultvalue(VARTABLE *typetemplate,const char *name)
{
  char *subfunc="create_template_from_defaultvalue";
  VARTABLE *node;
  char **pds;
  tsARRAY7 *arp;
  long n;
  
  if((node=lookup_list(typetemplate,name,NULL,VTYPE_ARRAY ,VTYPE_FLAG))==NULL){
    (void)sprintf(errorbuffer
		  ,"error(%s): Variable %s not declared as flag array !"
		  ,subfunc,name);
    return 0;
  }
  
  
  arp=&node->data.array7;
  
    
  if(arp->rank!=1){
    (void)sprintf(errorbuffer,"Cannot apply create_template_from_defaultvalue "
		  "to a multidimensional flag array !");
    return 0;
    
  }

  if(!disectDefVal(arp->defaultvalue,&n,&pds)) return 0;
  FREEE(pds[0],"pds[0]"); pds[0]=NULL;
  FREEE(pds,"pds"); pds=NULL;

  arp->Size[arp->rank]=n;
  if(!allocate_array(node,1L)) return 0; 

  
  return 1;
}
/* EOF */
