/* @(#)generatetau.c
 */


#include <string.h>
#include <math.h>
#include "merror.h"
#include "tools.h"
#include "fsymmetry.h"
#include "generatetau.h"


#define VEC(a,i) ((a)+3*(i))

/* ***********************************************************************
 * v0=v1+v2*g mod unit cell
 * ***********************************************************************
 */
#define VECVEC(v0,v1,v2,g,cell,rcell)		\
  vec1((v0),(v1),(v2),(g));			\
  (void)firstcube((v0),(cell),(rcell)); 


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static double DROUND(double x)
{
  return ((x)>0. ?( floor(x+.5) ) :(ceil(x-.5)));
}

/* ***********************************************************************
 * generate the atom positions, from space group symmetry
 * ***********************************************************************
 */
void generatetau_mol(const tsSYMMETRY *sym,long *nsite,long nsort
			,const double *tau,double *taus
			,long *isrt ,double tol)
{
  const char *nsubfunc="generatetau_mol";
  long is,ig,in,iin,go,i,iinv,it,k,ng;
  double ve[48][3],vv[3],ang[3];
  double gdum[9];

  
  ng=sym->ng;

  

  
  *nsite=0;
  /* for all sorts */
  for(is=0;is<nsort;is++){
    /* unity operation */
    ve[0][0]=tau[0+is*3];
    ve[0][1]=tau[1+is*3];
    ve[0][2]=tau[2+is*3];
    

    in=1;
    /* for all nontrivial operations */
    for(ig=0;ig<ng;ig++){
      it=sym->itable[ig];
      /* transformed vector */

      /* here we have to make sure that we use cartesian matrices
       since in molecules atoms are in absolute cooredinats */
      
      transbtoa(gdum,sym->opmat+9*it,sym->hex);

      vec2(vv,ve[0],gdum,3);

      go=0;
      /* is it present allready */
      for(iin=0;iin<in;iin++){
	ang[0]=ve[iin][0]-vv[0];
	ang[1]=ve[iin][1]-vv[1];
	ang[2]=ve[iin][2]-vv[2];
	

	if( sqrt(ddotn(ang,ang,3)) < tol ) {
	  /* it is */
	  go=1;
	  break;
	}
      }	/* iin */
      if( ! go){
	/* it is not add it to ve */
	ve[in][0]=vv[0];
	ve[in][1]=vv[1];
	ve[in][2]=vv[2];

	in++;
      }	/* go */
    } /* ig */


    /* make inversion symmetric vector  = -vector */
    for(i=0;i<in;i++){
      iinv=-1;
      if(sym->inversion){
	for(iin=0;iin<in;iin++){
	  ang[0]=ve[iin][0]+ve[i][0];
	  ang[1]=ve[iin][1]+ve[i][1];
	  ang[2]=ve[iin][2]+ve[i][2];
	  
	  if( sqrt(ddotn(ang,ang,3)) < tol ) {
	    iinv=iin;
	    break;
	  }
	} /* iin */
	if(i != iinv){
	  ve[iinv][0]=-ve[i][0];
	  ve[iinv][1]=-ve[i][1];
	  ve[iinv][2]=-ve[i][2];
	}
	if(iinv<0){
	  errmsg(nsubfunc,"Can not find the inversion related positions!"
		 ,SEVERE_ERROR);
	  return;
	} /* iinv failed */
      }	/* inversion */
    } /* i */
    if(in > 48){
      errmsg(nsubfunc,"More than 48 equivalent positions detected!"
	     ,SEVERE_ERROR);
      return;
    }
    
    /* define  the real vectors */
    for(iin=0;iin<in;iin++){
      dinit(taus+(*nsite+iin)*3,0.,3);
      for(k=0;k<3;k++){
	taus[k+(*nsite+iin)*3]+=ve[iin][k];
      }
      isrt[*nsite+iin]=is;
    }
    *nsite+=in;
  } /* is */
}


/* ***********************************************************************
 * generate the atom positions, from space group symmetry
 * ***********************************************************************
 */
void generatetau(const tsSYMMETRY *sym,long *nsite,long nsort
			,const double *tau,double *taus
			,long *isrt ,double tol)
{
  const char *nsubfunc="generatetau";
  long is,ig,in,iin,go,i,iinv,it,j,k,RTG,ng;
  double ve[48][3],vv[3],ang[3];
  double cell[9],rcell[9],det;
  const long *C;
  
  RTG=sym->RTG;
  ng=sym->ng;
  /* convert bravaistype lattice to form use by firstcell*/
  C=bravais_centering(sym->bravais);
  if(C==NULL){
    errmsg(nsubfunc,"Unknown bravais type!",SEVERE_ERROR);
    return;
  }
  for(i=0;i<9;i++){
    cell[i]=C[i]/12.;
  }

  transpose(cell,3);  
  if(! dinv33(cell,1,rcell,&det)){  
    errmsg("dinv33","Vanishing determinant of matrix!",SEVERE_ERROR);
    return ;
  }
  

  
  *nsite=0;
  /* for all sorts */
  for(is=0;is<nsort;is++){
    /* unity operation */
    ve[0][0]=tau[0+is*3];
    ve[0][1]=tau[1+is*3];
    ve[0][2]=tau[2+is*3];
    
    if(RTG) 
      /* vec1(ve[0],vzero3,tau+is*3,bravaislattice(SYM_RHOMBOHEDRAL));  */
      ConvertVecRTGtoSTG(ve[0],tau+is*3);

    in=1;
    /* for all nontrivial operations */
    for(ig=0;ig<ng;ig++){
      it=sym->itable[ig];
      /* transformed vector */
      VECVEC(vv,&(sym->v[it*3]),ve[0],sym->opmat+9*it,cell,rcell);
      if(erroroccured())return;
      go=0;
      /* is it present allready */
      for(iin=0;iin<in;iin++){
	ang[0]=ve[iin][0]-vv[0];
	ang[1]=ve[iin][1]-vv[1];
	ang[2]=ve[iin][2]-vv[2];
	
	if( firstcube(ang,cell,rcell) < tol ) {
	  /* it is */
	  go=1;
	  if(erroroccured())return;
	  break;
	}
	if(erroroccured())return;
      }	/* iin */
      if( ! go){
	/* it is not add it to ve */
	ve[in][0]=vv[0];
	ve[in][1]=vv[1];
	ve[in][2]=vv[2];

	in++;
      }	/* go */
    } /* ig */


    /* as short as possible */
    for(i=0;i<in;i++){
      (void)firstcube(&ve[i][0],cell,rcell);
    }
    if(erroroccured())return;
    /* make inversion symmetric vector  = -vector */
    for(i=0;i<in;i++){
      iinv=-1;
      if(sym->inversion){
	for(iin=0;iin<in;iin++){
	  ang[0]=ve[iin][0]+ve[i][0];
	  ang[1]=ve[iin][1]+ve[i][1];
	  ang[2]=ve[iin][2]+ve[i][2];
	  
	  if(firstcube(ang,cell,rcell) < tol ) {
	    iinv=iin;
	    if(erroroccured())return;
	    break;
	  }
	  if(erroroccured())return;
	} /* iin */
	if(i != iinv){
	  ve[iinv][0]=-ve[i][0];
	  ve[iinv][1]=-ve[i][1];
	  ve[iinv][2]=-ve[i][2];
	}
	if(iinv<0){
	  errmsg(nsubfunc,"Can not find the inversion related positions!"
		 ,SEVERE_ERROR);
	  return;
	} /* iinv failed */
      }	/* inversion */
    } /* i */
    if(in > 48){
      errmsg(nsubfunc,"More than 48 equivalent positions detected!"
	     ,SEVERE_ERROR);
      return;
    }
    
    /* define  the real vectors */
    for(iin=0;iin<in;iin++){
      dinit(taus+(*nsite+iin)*3,0.,3);
      for(k=0;k<3;k++){
	for(j=0;j<3;j++){
	  taus[k+(*nsite+iin)*3]+=ve[iin][j]
	    *sym->alat[j]
	    *sym->scal[j]
	    *sym->usheer[j+3*k];
	}
      }
      isrt[*nsite+iin]=is;
    }
    *nsite+=in;
  } /* is */
}

/* ***********************************************************************
 * p=p mod unit cell
 * cell[k+3*i] is i-th vector k-th cartesian component
 * rcell[k+3*i] is i-th reciprocial vector k-th cartesian component
 * return: |p|
 * ERRORS: sever_error: backfolding failed : unread
 * ***********************************************************************
 */
double firstcube(double *p,const double *cell,const double *rcell)
{
  const char *nsubfunc="firstcube";
  long i;
  double l,c[3],d;


  for(i=0;i<3;i++){
    l=ddotn(p,VEC(rcell,i),3);
    l=l-DROUND(l);
    if(l<-1.0){ 
      (void)sprintf(errormsg,"Backfolding of site into unit cell failed!\n"
		    "component %ld =%f  outsite cell:<-1\n",i,l);
      errmsg(nsubfunc,errormsg,SEVERE_ERROR);
    }else if(l>1.0) { 
      (void)sprintf(errormsg,"Backfolding of site into unit cell failed!\n"
		    "component %ld =%f  outsite cell:>1\n",i,l);
      errmsg(nsubfunc,errormsg,SEVERE_ERROR);
    }
    c[i]=l;
  }
  d=0.;
  for(i=0;i<3;i++){
    p[i]=VEC(cell,0)[i]*c[0]+VEC(cell,1)[i]*c[1]+VEC(cell,2)[i]*c[2];
    d+=p[i]*p[i];
  }
  return sqrt(d);
}
