#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cbuffer.h"
#include "merror.h"


/* buffer */
char errormsg[5000]="";
char errorbuffer[5000]="";


/* last severity level */

static long    slevel = 0;
static long    erocu  = 0;
static long    ehand  = 1;
static long initialized= 0;
static tsCBUFFER b;

#define ERR_CMASK     0x00ff
#define ERR_BMASK     0xff00

#define CATCH(i)      ( ( ((i) & ERR_CMASK) & ERR_CATCH ) )
#define BUFFERED(i)   ( ( ((i) & ERR_BMASK) & ERR_BUFFERED )  )
#define RESET(i)      ( ( ((i) & ERR_BMASK) & ERR_RESETBUFFER ) )


/*forwar prototype */
static void initerrorbuffer(void);

/* the macro version  calls j twice !! */
static long LMAX(long i,long j)
{
  return ( ((i)>j?i:j) );
}

/* the macro version  calls j twice !! */
static long LMIN(long i,long j)
{
  return ( ((i)<j?i:j) );
}


/* ***********************************************************************
 * handle  errors
 * ***********************************************************************
 */
void errmsg(const char *location,char *msg,long level)
{
  long i,len;
  char msgs[4][100]={
    "INFORMATION:",
    "WARNING:",
    "ERROR:",
    "FATAL ERROR:"
  };
  /* printf("errmsg %s %s \n",msg,location);   */
  i=LMIN(level,SEVERE_ERROR);
  i=(i<1?SEVERE_ERROR:i);
  erocu=LMAX( i>WARNING , erocu);
  slevel=LMAX(slevel,i);

  if(BUFFERED(ehand) && ! CATCH(ehand) ){
    initerrorbuffer();
    len=strlen(msgs[i-1])+strlen(msg)+strlen(location) ;
    if(!CBAppend(&b,len,"%s (%s): %s\n",msgs[i-1],location,msg)){
      (void)fprintf(stderr,"%s\n",errorbuffer);
      (void)printf("%s\n",errorbuffer);
      exit(1);
    }
  }else{
    (void)printf("%s (%s): %s\n",msgs[i-1],location,msg);
  }
  
  if(CATCH(ehand) && i>WARNING)
    exit(1);
}

/* ***********************************************************************
 * return severity level
 * reset internal severity
 * ***********************************************************************
 */
long severitylevel(void)
{
  long ret=slevel;
  slevel=erocu=0;
  return ret;
}
/* ***********************************************************************
 * return 1 if error occured, this is reset by severitylevel
 * ***********************************************************************
 */
long erroroccured(void)
{
  return erocu;
}
/* ***********************************************************************
 * set catch of errors, initialize ebuffer
 * ***********************************************************************
 */
void seterrorhandling(long e)
{
  ehand=e;

  if(RESET(e)){
    initerrorbuffer();
    
    CBEnd(&b);
    if(!CBStart(&b)){
      (void)fprintf(stderr,"%s\n",errorbuffer);
      (void)printf("%s\n",errorbuffer);
      exit(1);
    }
  }
}

/* ***********************************************************************
 * init error buffereing
 * ***********************************************************************
 */
static void initerrorbuffer(void)
{
  if(! initialized){
    initialized=1;
    CBInit(&b,5000,5000);

    if(!CBStart(&b)){
      (void)fprintf(stderr,"%s\n",errorbuffer);
      (void)printf("%s\n",errorbuffer);
      exit(1);
    }

  }
}

/* ***********************************************************************
 * flush error buffer into FILE *FH
 * ***********************************************************************
 */

void flusherror(FILE *FH)
{
  if(initialized){
    CBPrint(&b,FH);
  }
}



/* ***********************************************************************
 * return error buffer 
 * ***********************************************************************
 */
const char * errorgetbuffer(void)
{
  const char *ret=NULL;
  if(initialized){
    ret=CBGetBuffer(&b);
  }
  return ret;
}



/* EOF */
