/* @(#)AnnotationsTreeModel.hh
 * author: magru@unas
 */

#ifndef _ANNOTATIONSTREEMODEL_HH
#define _ANNOTATIONSTREEMODEL_HH 1

#include <QAbstractItemModel>
#include <QDebug>
#include <QIcon>
#include "FString.hh"
#include "DisplayCells.hh"
#include "Axes.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class AnnotationsTreeModel:public QAbstractItemModel
{
  Q_OBJECT;


public:

  enum RootItems{RIAxes=0,RILocalAxes=1,RIDisplayCells=2,RIDisplayCell=3,RIAxis=4};
  
  // ----------------------------------------------------------------
  AnnotationsTreeModel(QObject *parent,DisplayCells *displaycells,Axess *axes)
    :QAbstractItemModel(parent),pdisplaycells(displaycells),paxes(axes)
  {
  }
  
  // ----------------------------------------------------------------
  ~AnnotationsTreeModel()
  {
  }
  
  // ----------------------------------------------------------------
  QVariant headerData(int section, Qt::Orientation orientation, int role) const
  {
    if (role != Qt::DisplayRole)
      return QVariant();
    
    if (orientation == Qt::Horizontal) {
      return QVariant();
    }else{
      return section+1;
    }
    return QVariant();
  }

  // ----------------------------------------------------------------
  Qt::ItemFlags flags(const QModelIndex &index) const
  {
    if (!index.isValid()) return 0;
    
    int row=index.row();
    int col=index.column();
    

    if (row >= rowCount(index.parent())    || row < 0) return 0;
    if (col >= columnCount() || col < 0) return 0;
    
    //qDebug()<<index;
    bool iscrystal=(pdatastructurestype==Xf::STCrystal);
    
    if(index.internalId()==RIDisplayCells||
       index.internalId()==RIAxes||
       index.internalId()==RILocalAxes ){
      //myDebug()<<"roots";
      return Qt::ItemIsEnabled | Qt::ItemIsSelectable;

    }else if(index.internalId()==RIDisplayCell){
      
      //myDebug()<<pdisplaycells->cell(index.row())->type();
      Qt::ItemFlags flags=Qt::ItemIsEnabled | Qt::ItemIsSelectable |
	Qt::ItemIsUserCheckable;
      if(pdisplaycells->cell(index.row())->type()==Xf::DCTGeneral && iscrystal)
	return flags | Qt::ItemIsEditable;
      if(pdisplaycells->cell(index.row())->type()==Xf::DCTMainBoundary||iscrystal)
	return flags;

    }else if(index.internalId()==RIAxis){
      //myDebug()<<"RIAxis";
      Qt::ItemFlags flags=Qt::ItemIsEnabled | Qt::ItemIsSelectable |
	Qt::ItemIsUserCheckable;
      
      if(paxes->axes(index.row())->axesType()==Xf::ATGeneral)
	return flags|  Qt::ItemIsEditable;
      if(paxes->axes(index.row())->axesType()==Xf::ATCartesian||iscrystal)
	return flags;
      
    }
    return 0;
  }
  
  // ----------------------------------------------------------------
  QModelIndex index(int row, int column,
		    const QModelIndex &parent = QModelIndex()) const
  {
    if (column<0 || column>= columnCount()) return QModelIndex();
    if (row<0 || row>= rowCount(parent)) return QModelIndex();

    if (!hasIndex(row, column, parent)) return QModelIndex();

    QModelIndex rindex=QModelIndex();
    //qDebug()<<"parent="<<parent;
    if(!parent.isValid()){
      if(row==RIAxes) rindex=createIndex(row,0,RIAxes);
      else if(row==RILocalAxes) rindex=createIndex(row,0,RILocalAxes);
      else if(row==RIDisplayCells) rindex=createIndex(row,0,RIDisplayCells);
      else rindex=createIndex(row,column,-1);
    }else if(parent.internalId()==RIDisplayCells){
      rindex=createIndex(row,column,RIDisplayCell);
    }else if(parent.internalId()==RIAxes){
      rindex=createIndex(row,column,RIAxis);
    }
    //qDebug()<<"rindex="<<rindex;
    return rindex;
  }
  
  // ----------------------------------------------------------------
  QModelIndex parent(const QModelIndex &index=QModelIndex()) const
  {
    if (!index.isValid()) return QModelIndex();

    //qDebug()<<"index="<<index;
    if(index.internalId()==RIDisplayCell){
      QModelIndex parent=createIndex(RIDisplayCells,0,RIDisplayCells);
      return parent;
    }else if(index.internalId()==RIAxis){
      QModelIndex parent=createIndex(RIAxes,0,RIAxes);
      return parent;
    }
    return QModelIndex();
  }
  
  // ----------------------------------------------------------------
  int rowCount(const QModelIndex &parent=QModelIndex()) const
  {
    if(!parent.isValid()){
      return 3;
    }else if(parent.internalId()==RIDisplayCells){
      //qDebug()<<"parent="<<parent<<pdisplaycells->size();
      if(pdisplaycells) return pdisplaycells->size();
    }else if(parent.internalId()==RIAxes){
      //qDebug()<<"parent="<<parent<<pdisplaycells->size();
      if(paxes) return paxes->size();
    }else{
      return 0;
    }
    return 0;
  }

  // ----------------------------------------------------------------
  int columnCount(const QModelIndex &parent=QModelIndex()) const
  {
    Q_UNUSED(parent);
    return 1;
  }

  // ----------------------------------------------------------------
  QVariant data(const QModelIndex &index, int role) const
  {
    if (!index.isValid() || index.column() != 0) return QVariant();

    long row=index.row();
    
    if(index.internalId()==RIDisplayCells){
      if (role == Qt::DisplayRole) {
	return "Cells";
      }
    }else if(index.internalId()==RIAxes){
      if (role == Qt::DisplayRole) {
	return "Main Axes";
      }
    }else if(index.internalId()==RILocalAxes){
      if (role == Qt::DisplayRole) {
	return "Local Axes";
      }
    }else if(index.internalId()==RIDisplayCell){
      if (role == Qt::DisplayRole) {
	return pdisplaycells->cell(row)->name()();
      }else if (role == Qt::DecorationRole) {
	if(pdisplaycells->cell(row)->type()==Xf::DCTMainBoundary){
	  return QIcon(":/Icons/cellbound");
	}else if(pdisplaycells->cell(row)->type()==Xf::DCTConventional){
	  return QIcon(":/Icons/cellconv");
	}else if(pdisplaycells->cell(row)->type()==Xf::DCTPrimitive){
	  return QIcon(":/Icons/cellprim");
	}else{
	  return QIcon(":/Icons/cellgeneral");
	}
      }else if (role == Qt::CheckStateRole) {
	return MyMath::min<int>(2,2*pdisplaycells->cell(row)->isVisible());
      }
      
    }else if(index.internalId()==RIAxis){
      if (role == Qt::DisplayRole) {
	return paxes->axes(row)->name()();
      }else if (role == Qt::DecorationRole) {
	if(paxes->axes(row)->axesType()==Xf::ATConventional){
	  return QIcon(":/Icons/axesconv");
	}else if(paxes->axes(row)->axesType()==Xf::ATPrimitive){
	  return QIcon(":/Icons/axesprim");
	}else if(paxes->axes(row)->axesType()==Xf::ATCartesian){
	  return QIcon(":/Icons/axescart");
	}
      }else if (role == Qt::CheckStateRole) {
	return MyMath::min<int>(2,2*paxes->axes(row)->isVisible());
      }
      
    }else if(!parent().isValid()){
      if (role == Qt::DisplayRole) {
	return Sprintf("Root %d",index.row())();
      }

    }
    return QVariant();
  }

  // ----------------------------------------------------------------
  bool setData(const QModelIndex &index, const QVariant &value,
	       int role=Qt::EditRole)
  {
    
    if (!index.isValid()) return false;

    long row=index.row();

    if(role==Qt::EditRole){
      if(index.internalId()==RIDisplayCell){
	DisplayCell *dc=pdisplaycells->cell(row);
	if(dc->type()==Xf::DCTGeneral)
	  dc->setName(value.toString());
	emit(dataChanged(index, index));
      }
    }else if ( role == Qt::CheckStateRole) {
      if(index.internalId()==RIDisplayCell){
	DisplayCell *dc=pdisplaycells->cell(row);
	dc->setVisible(value.toInt());
	emit(checkStateChanged(dc));
	emit(dataChanged(index, index));
      }else if(index.internalId()==RIAxis){
	Axes *ax=paxes->axes(row);
	ax->setVisible(value.toInt());
	emit(checkStateChanged(ax));
	emit(dataChanged(index, index));
      }
    }

    return false;
  }

  // ----------------------------------------------------------------
  void reload(Xf::StructureType datastructurestype,
	      Xf::LatticeType datalatticetype)
  {

    pdatastructurestype=datastructurestype;
    pdatalatticetype=datalatticetype;
    beginResetModel();
    endResetModel();
  }

  // ----------------------------------------------------------------
  bool insertRow(int position,DisplayCell *dc)
  {
    beginInsertRows(index(RIDisplayCells,0,QModelIndex()),position,position);
    //psym->insertWyckoffPosition(position,w);
    pdisplaycells->insert(position,dc,true);
    endInsertRows();
    return true;
  }

  // ----------------------------------------------------------------
  void removeCell(int position)
  {
    myDebug()<<"position"<<position;
    beginRemoveRows(index(RIDisplayCells,0,QModelIndex()),position,position);
    pdisplaycells->remove(pdisplaycells->cell(position));
    endRemoveRows();
  }


  // ----------------------------------------------------------------
  QModelIndex cellIndex(int pos)
  {
    return index(pos,0,index(AnnotationsTreeModel::RIDisplayCells,0,QModelIndex()));
  }

  // ----------------------------------------------------------------
  QModelIndex axisIndex(int pos)
  {
    return index(pos,0,index(AnnotationsTreeModel::RIAxes,0,QModelIndex()));
  }


signals:
  void checkStateChanged(const Axes *);
  void checkStateChanged(const DisplayCell *);

private:
  DisplayCells *pdisplaycells;
  Axess *paxes;
  Xf::StructureType pdatastructurestype;
  Xf::LatticeType pdatalatticetype;
  
};

#endif /* _ANNOTATIONSTREEMODEL_HH */

