/* @(#)BBoxModel.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _BBOXMODEL_H
#define _BBOXMODEL_H 1

#include <QAbstractTableModel>
#include "Boundary.hh"
#include "BBoxModel.hh"
#include "Boundary.hh"
#include "MyDebug.hh"
#include "Symmetry.hh"
#include "qt5compat.hh"



using namespace XFPLO;

class BBoxModel: public QAbstractTableModel
{
public:

  // ----------------------------------------------------------------
  BBoxModel( QObject * parent = 0,Boundary *bound=0)
  : QAbstractTableModel(parent),pbound(bound)
  {
    pheader<<"x";
    pheader<<"y";
    pheader<<"z";
    pheader<<"";
    pheader<<"d0";
    pheader<<"d1";
  }

  // ----------------------------------------------------------------
  ~BBoxModel()
  {
  }
  
  // ----------------------------------------------------------------
  int rowCount(const QModelIndex & parent ) const
  {
    Q_UNUSED(parent);
    return 3;
  }

  // ----------------------------------------------------------------
  int columnCount(const QModelIndex &parent) const
  {
    Q_UNUSED(parent);
    return pheader.size();
  }
  
  // ----------------------------------------------------------------
  QVariant headerData(int section, Qt::Orientation orientation, int role) const
  {
    if (role != Qt::DisplayRole)
      return QVariant();
    if (orientation == Qt::Horizontal) {
      return pheader.at(section);
    }else{
    return Sprintf("V%ld",section+1)();
    }
    return QVariant();
  }

  // ----------------------------------------------------------------
  Qt::ItemFlags flags(const QModelIndex &index) const
  {
    if (!index.isValid())
      return Qt::ItemIsEnabled;
    
    if(index.column()==3){
      return  Qt::NoItemFlags;
    }else
      return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
  }

  // ----------------------------------------------------------------
  QVariant data(const QModelIndex &index, int role) const
  {
    
    if (!pbound) return QVariant();
    if (!index.isValid()) return QVariant();
    
    
    
    if (index.row() > 3 || index.row() < 0) return QVariant();
    
    
    int c=index.column();
    
    if (role == Qt::DisplayRole || role == Qt::EditRole) {
      //const WyckPos &pos= psym->wyckoffPosition(index.row());
      
      if (c < 3)
	return pbound->boxVecS(index.row())[c]();
      else if( c==4) return pbound->boxDistanceS(index.row())->v1(); 
      else if( c==5) return pbound->boxDistanceS(index.row())->v2(); 
    }
    return QVariant();
    
  }

  // ----------------------------------------------------------------
  bool setData(const QModelIndex &index, const QVariant &value, int role)
  {
    NSF("BBoxModel::setData"); Q_UNUSED(nsubfunc);
    //  myDebug() << nsubfunc;
    
    //  myDebug("pbound(%p)\n",pbound);
    
    if(!pbound) return false;
    
    
    
    if (index.isValid() && role == Qt::EditRole) {
      int row = index.row();
      
      int c=index.column();
      
      if (c < 3){
	String s=value.toString().toAscii().constData();
	pbound->setBox(c,row,s);
      }else if( c==4 || c==5){
	String s=value.toString().toAscii().constData();
	pbound->setBoxDistance(row,c-4,s);
      }else
	return false;
      
      //psym->setWyckoffPosition(row,pos);
      emit(dataChanged(index, index));
      
      return true;
    }
    
    return false;
  }

  // ----------------------------------------------------------------
  void reset(const String &mode,const Symmetry *sym=0
	     ,Boundary::BasisType b=Boundary::Conventional)
  {
    //myDebug() << "BoundDlg::on_pbResetUnitMat_clicked";
    if(mode=="D"){
      Pair<String,String> dist[3]={
	Boundary::defaultBoxDistS,
	Boundary::defaultBoxDistS,
	Boundary::defaultBoxDistS
      };
      pbound->setBoxDistances(dist);
      emit dataChanged(index(0,4,QModelIndex())
		       ,index(3,5,QModelIndex()));
    }else{
      pbound->setDefaultFaces(mode,sym,b);
      emit dataChanged(index(0,0,QModelIndex())
		       ,index(3,3,QModelIndex()));
    }
    
  }
  

private:
  Boundary *pbound;
  QStringList pheader;


};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _BBOXMODEL_H */

