/*************************************************************************
 * @(#)BFacesModel.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   31 Jul 2009
 *************************************************************************
 */

#include "BFacesModel.hh"
#include "Boundary.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BFacesModel::BFacesModel( QObject * parent ,Boundary *bound)
  : QAbstractTableModel(parent),pbound(bound)
{
  NSF("BFacesModel::BFacesModel");   Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc;
  MASSERT("parameter Boundary *bound",bound);
  //  myDebug("bound(%p)\n",bound);
  //  myDebug("pbound(%p)\n",pbound);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BFacesModel::~BFacesModel()
{
  NSF("BFacesModel::~BFacesModel"); Q_UNUSED(nsubfunc);
  //  myDebug() << nsubfunc;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int BFacesModel::rowCount(const QModelIndex & parent ) const
{
  Q_UNUSED(parent);
  if(!pbound) return 0;
  return pbound->faces().size();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int BFacesModel::columnCount(const QModelIndex & parent ) const
{
  Q_UNUSED(parent);
  return 5;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant BFacesModel::data(const QModelIndex &index, int role) const
{
  
  if (!pbound) return QVariant();
  if (!index.isValid()) return QVariant();
  
  

  if (index.row() >= pbound->faces().size() || index.row() < 0) 
    return QVariant();


  int c=index.column();

  if (role == Qt::DisplayRole || role == Qt::EditRole) {
    const Face &face= pbound->faces().at(index.row());
    
    if (c < 3)
      return face.pS[c]();
    else if( c==4) return face.distS(); 
  }
  return QVariant();
  
}



 /* ***********************************************************************
 *
 * ***********************************************************************
 */
QVariant BFacesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
  if (role != Qt::DisplayRole)
    return QVariant();
  
  if (orientation == Qt::Horizontal) {
    switch (section) {
    case 0:
      return "x";
      
    case 1:
      return "y";
    case 2:
      return "z";
    case 4:
      return "d0";
      
    default:
      return QVariant();
    }
  }else{
    return Sprintf("F%ld",section+1)();
  }
  return QVariant();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Qt::ItemFlags BFacesModel::flags(const QModelIndex &index) const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;
 
  if(index.column()==3){
    return  Qt::NoItemFlags;
  }else
    return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BFacesModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
  NSF("BFacesModel::setData"); Q_UNUSED(nsubfunc);
  //myDebug() << nsubfunc;
  
  //  myDebug("pbound(%p)\n",pbound);

  if(!pbound) return false;

  

  if (index.isValid() && role == Qt::EditRole) {
    int row = index.row();
    
    int c=index.column();
    
    if (c < 3 || c==4){
      String s=value.toString().toAscii().constData();
      pbound->modifyFace(row,(c<3?c:c-1),s);
    }else
      return false;
    
    emit(dataChanged(index, index));
    
    return true;
  }
  
  return false;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BFacesModel::reset(const String &mode,const Symmetry *sym
			,Boundary::BasisType b)
{
  //myDebug() << "BoundDlg::on_pbResetUnitMat_clicked";
  beginResetModel();
  pbound->setDefaultFaces(mode,sym,b);
  endResetModel();
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BFacesModel::removeRows(int position,int rows
			     ,const QModelIndex &index)
{
  Q_UNUSED(index);
  beginRemoveRows(QModelIndex(), position, position+rows-1);
  
  for (int row=0; row < rows; ++row) {
    //myDebug() << index << position;
    pbound->removeFace(position);
  }
  
  endRemoveRows();
  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BFacesModel::insertRows(int position,int rows
			     ,const QModelIndex &index,const Face &f)
{
  NSF("BFacesModel::insertRows"); Q_UNUSED(nsubfunc); Q_UNUSED(index);

  beginInsertRows(QModelIndex(), position, position+rows-1);
  
  for (int row=0; row < rows; row++) {
    //    myDebug() << nsubfunc << position ;
    pbound->insertFace(position,f);
  }
  
  endInsertRows();
  return true;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const Face *BFacesModel::getData(int row) const
{
  
  if (!pbound) return 0;
  if (row >= pbound->faces().size() || row < 0) return 0;
  
  return &pbound->faces().at(row);
    
}


/*EOF*/ 

 


/*EOF*/ 
