/* @(#)BondGroupsModel.hh
 * author: magru@unas
 */

#ifndef _BONDGROUPSMODEL_HH
#define _BONDGROUPSMODEL_HH 1

#include <QAbstractTableModel>
#include "BondDefs.hh"

using namespace XFPLO;

class BondGroupsModel: public QAbstractTableModel
{
public:

  enum Columns{Active=0,Name=1,BondColor=2,Visible=3};

  // ----------------------------------------------------------------
  BondGroupsModel(QObject * parent=0)
    : QAbstractTableModel(parent),pbdgs(0)
  {
    
    pheader<<"Act";
    pheader<<"Name";
    pheader<<"Color";
    pheader<<"BondVis";
  }

  // ----------------------------------------------------------------
  ~BondGroupsModel()
  {
  }

  // ----------------------------------------------------------------
  int rowCount(const QModelIndex & parent=QModelIndex()) const
  {
    Q_UNUSED(parent);
    if(pbdgs) return pbdgs->size();
    return 0;
  }

  // ----------------------------------------------------------------
  int columnCount(const QModelIndex &parent=QModelIndex()) const
  {
    Q_UNUSED(parent);
    return pheader.size();
  }

  // ----------------------------------------------------------------
  QVariant headerData(int section, Qt::Orientation orientation, int role) const
  {
    if (role != Qt::DisplayRole)
      return QVariant();
    if (orientation == Qt::Horizontal) {
      return pheader.at(section);
    // }else{
    //   return section+1;
    }
    return QVariant();
  }

  // ----------------------------------------------------------------
  void reload(BondDefGroups *bdgs=0)
  {
    beginResetModel();
    if(bdgs) pbdgs=bdgs;
    endResetModel();
  }

 // ----------------------------------------------------------------
  Qt::ItemFlags flags(const QModelIndex &index) const
  {
    if (!index.isValid()) return Qt::ItemIsEnabled;
    
    int row=index.row();
    int col=index.column();
    
    if (row >= rowCount()    || row < 0) return QAbstractTableModel::flags(index);
    if (col >= columnCount() || col < 0) return QAbstractTableModel::flags(index);
    
    
    switch(col){
    case Active:
    case Visible:
      return QAbstractTableModel::flags(index)  | Qt::ItemIsUserCheckable;
    case Name:
      return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
    case BondColor:
      return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
    }
    
    return QAbstractTableModel::flags(index);
  }
  // ----------------------------------------------------------------
  QVariant data(const QModelIndex &index, int role) const
  {
    NSF("SpspModel::data");
    if (!index.isValid()) return QVariant();

    int row=index.row();
    int col=index.column();
    
    if (row >= rowCount()    || row < 0) return QVariant();
    if (col >= columnCount() || col < 0) return QVariant();

    if (role == Qt::DisplayRole || role == Qt::EditRole) {
      switch(col){
      case Name:
	return pbdgs->group(row)->name()();
	break;
      case BondColor:
	return pbdgs->group(row)->color();
	break;
      default:
	return QVariant();
      }
    }else if (role == Qt::CheckStateRole) {
      if(col==Active){
	return (pbdgs->group(row)->active()?2:0);
      }else if(col==Visible){
	return (pbdgs->group(row)->visible()?2:0);
      }
    }

    return QVariant();

  }

  // ----------------------------------------------------------------
  bool setData(const QModelIndex &index, const QVariant &value
	       , int role=Qt::EditRole)
  {
    NSF("SpspModel::setData"); 
    //  myDebug() << nsubfunc;
    
    if (!index.isValid()) return false;
    
    int row = index.row();
    int col = index.column();
  
    BondDefGroup *bdg=0;
    if(pbdgs) bdg=pbdgs->group(row);

    if(!bdg) return false;
    if ( role == Qt::EditRole) {
      
      //      QModelIndex index1;
      switch(col)
	{
	case Name:
	  bdg->setName(value.toString());
	  break;
	case BondColor:
	  bdg->setColor(value.toUInt());
	  break;
	default:
	  return false;
	}
      emit(dataChanged(index, index));
      
      return true;
    }else if ( role == Qt::CheckStateRole) {
      if(col==Active){
	bdg->setActive(value.toInt());
     	emit(dataChanged(index, index));
      }else if(col==Visible){
	bdg->setVisible(value.toInt());
     	emit(dataChanged(index, index));
      }
    }
    
    return false;
    
  }

  // ----------------------------------------------------------------
  bool insertRow(int position,BondDefGroup *bdg=0,bool createuniquename=true)
  {
    beginInsertRows(QModelIndex(),position,position);
    pbdgs->insert(position,bdg,createuniquename);
    endInsertRows();
    return true;
  }

  // ----------------------------------------------------------------
  BondDefGroup *removeRow(int position)
  {
    
    beginRemoveRows(QModelIndex(), position, position);
    BondDefGroup *bdg=pbdgs->removeAt(position);
    endRemoveRows();
    return bdg;
  }

  // ----------------------------------------------------------------
  bool moveRow(int row,int by)
  {
    
    if(!index(row+by,0,QModelIndex()).isValid()) return false;
    
    insertRow(MyMath::max<long>(0,row+by),removeRow(row),false);
    
    return true;
  }

  // ----------------------------------------------------------------
  BondDefGroup *currentGroup(const QModelIndex&currentindex)
  {
    if(currentindex ==  QModelIndex()) return 0;
    return pbdgs->group(currentindex.row());
  }

private:
  BondDefGroups *pbdgs;
  QStringList pheader;

};

#endif /* _BONDGROUPSMODEL_HH */

