/* @(#)LightsModel.hh
 * author: magru@unas
 */

#ifndef _LIGHTSMODEL_HH
#define _LIGHTSMODEL_HH 1

#include <QAbstractTableModel>
#include "MyMath.hh"
#include "Light.hh"


using namespace XFPLO;

/* ***********************************************************************
 * all in the header, since we need it only once or twice
 * ***********************************************************************
 */


class LightsModel: public QAbstractTableModel
{
public:
  // ----------------------------------------------------------------
  LightsModel( QObject * parent = 0) : QAbstractTableModel(parent),pligths(0)
  {
    pheader<<"Light";
  }
  // ----------------------------------------------------------------
  ~LightsModel()
  {
  }

  // ----------------------------------------------------------------
  void reload(Lights *lights)
  {
    beginResetModel();
    pligths=lights; 
    endResetModel();
  }
  
  // ----------------------------------------------------------------
  int rowCount(const QModelIndex & parent = QModelIndex() ) const
  {
    Q_UNUSED(parent);
    if(!pligths) return 0;
    return pligths->size();
  }
  // ----------------------------------------------------------------
  int columnCount(const QModelIndex &parent = QModelIndex()) const
  {
    Q_UNUSED(parent);
    return pheader.size();
  }
  // ----------------------------------------------------------------
  QVariant headerData(int section, Qt::Orientation orientation, int role) const
  {
    if (role != Qt::DisplayRole)
      return QVariant();
    
    if (orientation == Qt::Horizontal) {
      return pheader.at(section);
    }else{
      return section+1;
    }
    return QVariant();
  }

  // ----------------------------------------------------------------
  Qt::ItemFlags flags(const QModelIndex &index) const
  {
    if (!index.isValid())
      return Qt::ItemIsEnabled;
    
    return //Qt::ItemIsEnabled | Qt::ItemIsSelectable;
      QAbstractTableModel::flags(index)| Qt::ItemIsEditable| Qt::ItemIsUserCheckable;
  }

  // ----------------------------------------------------------------
  QVariant data(const QModelIndex &index, int role) const
  {
    
    if (!pligths) return QVariant();
    if (!index.isValid()) return QVariant();
    
    long col=index.column();
    long row=index.row();
    
    if (col >= columnCount() || col < 0) return QVariant();
    if (row >= rowCount() || row < 0) return QVariant();
    
    
    
    if (role == Qt::DisplayRole) {
      //const Lights::Light &li= pligths->at(row);
      return pligths->at(row)->name()();
    }else if (role == Qt::CheckStateRole) {
      return MyMath::min(2,2*pligths->at(row)->active());
    }else if (role == Qt::EditRole) {
      return (*pligths)[row]->name()();
    }
    return QVariant();
    
  }
  
  // ----------------------------------------------------------------
  bool setData(const QModelIndex &index, const QVariant &value
	       , int role=Qt::EditRole)
  {
    
    if(!pligths) return false;
    if (!index.isValid()) return false;
    
    int row = index.row();
    //int col = index.column();
    
    Lights::Light *li= (*pligths)[row];
    
    if ( role == Qt::CheckStateRole) {
      li->setActive(value.toInt());
      emit(dataChanged(index, index));
    }else if ( role == Qt::EditRole) {
      li->setName(value.toString());
      emit(dataChanged(index, index));
      return true;
    }
    
    return false;
    
  }  

  // ----------------------------------------------------------------
  bool insertRow(int position,Lights::Light *li=0)
  {
    beginInsertRows(QModelIndex(),position,position);
    pligths->insertLight(position,li);
    endInsertRows();
    return true;
  }

  // ----------------------------------------------------------------
  Lights::Light *removeRow(int position)
  {
    
    beginRemoveRows(QModelIndex(), position, position);
    Lights::Light *li=pligths->removeLightAt(position);
    endRemoveRows();
    return li;
  }
  
  
  // ----------------------------------------------------------------
  bool moveRow(int row,int by)
  {
    
    if(!index(row+by,0,QModelIndex()).isValid()) return false;
    
    insertRow(MyMath::max<long>(0,row+by),removeRow(row));
    
    return true;
  }
  
private:
  Lights *pligths;
  QStringList pheader;
  
};

#endif /* _LIGHTSMODEL_HH */

