/* @(#)SpspModel.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _SPSPMODEL_H
#define _SPSPMODEL_H 1

#include <QAbstractTableModel>
#include <QStringList>
#include "Xf.hh"

class SpecialSymPoints;
class RatValidator;
class SpecialPoint;
namespace XFPLO{
  class Symmetry;
}
class Vec3;

using namespace XFPLO;
class SpspModel: public QAbstractTableModel
{
public:
  enum Columns{Active=0,Name=1,PointPosX=2,PointPosY=3,PointPosZ=4
	       ,TextDepth=5
	       ,PointColor=6,PointRadius=7,TextColor=8,FontScale=9};
  SpspModel( QObject * parent,SpecialSymPoints *spsp);
  ~SpspModel();

  void setSymmetry(const Symmetry *sym){psym=sym;};
  void reload(){beginResetModel();endResetModel();};
  void validateData(QString &txt);

  int rowCount(const QModelIndex & parent=QModelIndex()) const;
  int columnCount(const QModelIndex &parent=QModelIndex()) const;
  QVariant data(const QModelIndex &index, int role) const;
  QVariant headerData(int section, Qt::Orientation orientation, int role) const;
  Qt::ItemFlags flags(const QModelIndex &index) const;
  bool setData(const QModelIndex &index, const QVariant &value
	       , int role=Qt::EditRole);

  void setDefaultForColumn(Columns action,const QVariant &value);
  void setColumnsEnabled(bool);
  bool insertRow(int position,Xf::SpecialPointType type,SpecialPoint *p=0);
  SpecialPoint *removeRow(int position);
  bool moveRow(int row,int by);
  void setPoint(int row,const Vec3 &v);


  
private:
  SpecialSymPoints *pspsp;
  QStringList pheader;
  RatValidator *xyzvalid;
  RatValidator *sfxyzvalid;
  RatValidator *psizevalid;
  RatValidator *fscalevalid;
  QList<bool> columnEnabled;
  const Symmetry *psym;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _SPSPMODEL_H */

