/*************************************************************************
 * @(#)OGL.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2010
 *************************************************************************
 */

#include "MyDebug.hh"
#include "OGL.hh"
#include "Settings.hh"
#include "MyDebug.hh"





/* ***********************************************************************
 *
 * ***********************************************************************
 */

void OGLIssueVertexArray(GLuint pdispListName
			 ,const GLdouble *pvertices
			 ,const GLdouble *pnormals
			 ,const GLfloat *pcolors
			 ,const GLuint *pindices
			 ,long ndefs
			 ,const struct VertexArrayDef *defs 
			 )
{
  NSF("OGLIssueVertexArray");
  
  
  if(ogl.hasVertexArrays()){
    
    if(pnormals) glEnableClientState(GL_NORMAL_ARRAY);
    if(pcolors) glEnableClientState(GL_COLOR_ARRAY);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    glVertexPointer(3,GL_DOUBLE,0,pvertices);
    if(pnormals) glNormalPointer(GL_DOUBLE,0,pnormals);
    if(pcolors) glColorPointer(4,GL_FLOAT,0,pcolors);
  }      
  
  if(pdispListName){
    glNewList(pdispListName,GL_COMPILE);
  }
  
  for(int id=0;id<ndefs;id++){
    if(ogl.hasVertexArrays()){
      if(defs[id].useindex){
	glDrawElements(defs[id].mode,defs[id].count,GL_UNSIGNED_INT
		       ,pindices+defs[id].start);
      }else{
	glDrawArrays(defs[id].mode,defs[id].start,defs[id].count);
      }
      
    }else{
      if(defs[id].useindex){
	
	glBegin(defs[id].mode);
	for(int i=0;i<defs[id].count;i++){
	  int ind=pindices[i+defs[id].start];
	  if(pnormals) glNormal3dv(pnormals+3*ind);
	  if(pcolors)  glColor4fv(pcolors+4*ind);
	  glVertex3dv(pvertices+3*ind);
	}
	glEnd();
      }else{
	glBegin(defs[id].mode);
	for(int i=0;i<defs[id].count;i++){
	  int ind=i+defs[id].start;
	  if(pnormals) glNormal3dv(pnormals+3*ind);
	  if(pcolors)  glColor4fv(pcolors+4*ind);
	  glVertex3dv(pvertices+3*ind);
	}
	glEnd();
      }
    }
    
  }

  if(pdispListName){
    glEndList();
  }
  
  if(ogl.hasVertexArrays()){
    glDisableClientState(GL_VERTEX_ARRAY);
    if(pnormals) glDisableClientState(GL_NORMAL_ARRAY);
    if(pcolors) glDisableClientState(GL_COLOR_ARRAY);
  }
  
  
  ogl.reportGLError(nsubfunc);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void OGLIssueVertexArray2(const GLdouble *pvertices
			 ,const GLdouble *pnormals
			 ,const GLfloat *pcolors
			 ,const GLuint *pindices
			 ,long ndefs
			 ,const struct VertexArrayDef *defs 
			 ,GLenum faces
			 )
{
  NSF("OGLIssueVertexArray1");
  
  
  
  
  for(int id=0;id<ndefs;id++){
    
    if(defs[id].useindex){
      
      glBegin(defs[id].mode);
      for(int i=0;i<defs[id].count;i++){
	int ind=pindices[i+defs[id].start];
	if(pnormals) glNormal3dv(pnormals+3*ind);
	if(pcolors) {
	  glMaterialfv(faces,GL_AMBIENT_AND_DIFFUSE,pcolors+4*ind);
	}
	glVertex3dv(pvertices+3*ind);
      }
      glEnd();
    }else{
      glBegin(defs[id].mode);
      for(int i=0;i<defs[id].count;i++){
	int ind=i+defs[id].start;
	if(pnormals) glNormal3dv(pnormals+3*ind);
	if(pcolors){
	  glMaterialfv(faces,GL_AMBIENT_AND_DIFFUSE,pcolors+4*ind);
	}
	glVertex3dv(pvertices+3*ind);
      }
      glEnd();
    }    
    
  }

  
  ogl.reportGLError(nsubfunc);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGL::init(bool fullinit)
{
  //myDebug()<<"OGL::init()";
  Settings set("application");
  pallowvertexarrays=set.getBool("openGL/allowVertexArrays",true);
  pallowdisplaylists=set.getBool("openGL/allowDisplayLists",true);
  pdiagnostics=set.getBool("openGL/allowDiagnostics",false);
  pmultisample=set.getBool("openGL/allowMultiSample",true);
  //myDebug()<<"pmultisample="<<pmultisample;

  if(!fullinit) return;
  phasvertexarrays=queryExtension("GL_EXT_vertex_array");
  printf("openGL: Has vertex arrays: %s\n",(phasvertexarrays?"yes":"no"));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGL::setAllowVertexArrays(bool a)
{
  pallowvertexarrays=a;
  Settings set("application");
  set.set("openGL/allowVertexArrays",a);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGL::setAllowDisplayLists(bool a)
{
  pallowdisplaylists=a;
  Settings set("application");
  set.set("openGL/allowDisplayLists",a);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGL::setAllowDiagnostics(bool a)
{
  pdiagnostics=a;
  Settings set("application");
  set.set("openGL/allowDiagnostics",a);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGL::setAllowMultiSample(bool a)
{
  pmultisample=a;
  Settings set("application");
  set.set("openGL/allowMultiSample",a);
}

/*EOF*/ 
