/* @(#)OGL.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGL_H
#define _OGL_H 1


#include "MyQRgb.hh"
#include <QGLWidget>
#ifdef _DARWIN_COMPAT
#include <OpenGL/glu.h>
#else
#include <GL/glu.h>
#endif
#include <stdio.h>
#include "MyMath.hh"
#include "Vec3.hh"



typedef struct VertexArrayDef{
  GLenum mode;
  GLsizei count;
  long start;
  bool useindex;
}tsVertexArrayDef;




class OGLMagnifier
{
 public:
  OGLMagnifier():ix(0),iy(0),factor(1){};
  OGLMagnifier(int x,int y,int f):ix(x),iy(y),factor(f){};
  int ix,iy;
  int factor;
};

inline GLfloat *GLfloat4FromQRgb(GLfloat *c,const QRgb &rgb){
  c[0]=qRed(rgb)/255.;
  c[1]=qGreen(rgb)/255.;
  c[2]=qBlue(rgb)/255.;
  c[3]=qAlpha(rgb)/255.;
  return c;
};

template <class T>
inline GLfloat *GLfloat4FromValueArray(GLfloat *c,const ValueArray<T> &v){
  c[0]=v.at(0);
  c[1]=v.at(1);
  c[2]=v.at(2);
  c[3]=v.at(3);
  return c;
};


inline QRgb QRgbFromGLfloat4(const GLfloat *c){
  QRgb u;
  u=ulRound(c[0]*255)<<16;
  u+=ulRound(c[1]*255)<<8;
  u+=ulRound(c[2]*255);
  u+=ulRound(c[3]*255)<<24;

  return u;
};



void OGLIssueVertexArray(GLuint pdispListName
			 ,const GLdouble *pvertices
			 ,const GLdouble *pnormals
			 ,const GLfloat *pcolors
			 ,const GLuint *pindices
			 ,long ndefs
			 ,const struct VertexArrayDef *defs 
			 );


void OGLIssueVertexArray2(const GLdouble *pvertices
			 ,const GLdouble *pnormals
			 ,const GLfloat *pcolors
			 ,const GLuint *pindices
			 ,long ndefs
			 ,const struct VertexArrayDef *defs 
			 ,GLenum faces
			 );

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGL{
 public:
  OGL():pallowvertexarrays(true),pallowdisplaylists(true),
	pdiagnostics(false),pmultisample(true){};


  bool allowVertexArrays()const {return pallowvertexarrays;};
  void setAllowVertexArrays(bool a);

  bool allowDisplayLists()const {return pallowdisplaylists;};
  void setAllowDisplayLists(bool a);

  bool allowDiagnostics()const {return pdiagnostics;};
  void setAllowDiagnostics(bool a);

  bool allowMultiSample()const {return pmultisample;};
  void setAllowMultiSample(bool a);


  void init(bool fullinit);

  bool hasVertexArrays()const{return phasvertexarrays && pallowvertexarrays;};

  GLboolean queryExtension(const char *extName){
    const char *p = (const char *) glGetString(GL_EXTENSIONS);
    const char *end = p + strlen(p);
    while (p < end) {
      size_t n = strcspn(p, " ");
      if ((strlen(extName)==n) && (strncmp(extName,p,n)==0)) {
	return GL_TRUE;
      }
      p += (n + 1);
    }
    return GL_FALSE;
  };

  void reportGLError(const char * nsubfunc,const char *text=""){
    if(!pdiagnostics) return;
    GLenum glerr;
    if( (glerr =glGetError()) != GL_NO_ERROR){
      printf("GL ERROR: %d %s:%s:%s",glerr,nsubfunc,text,
	     (char*)gluErrorString(glerr));
      exit(1);
    }
  };

 private:
  bool pallowvertexarrays;
  bool pallowdisplaylists;
  bool phasvertexarrays;
  bool pdiagnostics;
  bool pmultisample;

};

extern OGL ogl;

#ifdef	__cplusplus
extern "C" {
#endif


  
  
#ifdef	__cplusplus
}
#endif


#endif /* _OGL_H */

