/*************************************************************************
 * @(#)OGLAutoGlyph.cpp
 * author: magru@unas
 * date:   10 Jun 2013
 *************************************************************************
 */

#include "OGLMaterial.hh"
#include "OGLAutoGlyph.hh"
#include "OGLView.hh"
#include "Field.hh"
#include "Position.hh"
#include "Except.hh"
#include "RepAutoGlyph.hh"
#include "BoundingBox.hh"
#include "MyDebug.hh"


using namespace XFPLO;

bool OGLAutoGlyph::pusedisplaylists=true;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheAutoGlyphItem::
OGLDataCacheAutoGlyphItem(const RepAutoGlyph *rep)
  :OGLDataCacheItem("OGLDataCacheAutoGlyphItem")
  ,prep(rep)
{
  NSF("OGLDataCacheAutoGlyphItem::OGLDataCacheAutoGlyphItem");
  
  String s=Sprintf("OGLDataCacheAutoGlyphItem:rep(%p)"
		   ,prep
		   );
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheAutoGlyphItem::printSpecQ()const
{
  NSF("OGLDataCacheAutoGlyphItem::printSpecQ");
  QString s;
  
  s.sprintf("OGLDataCacheAutoGlyphItem:   rep=%p\n",prep);
  s+=OGLDataCacheItem::printSpecQ();
  return s;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheAutoGlyphItem::construct(bool transparent)
{
  NSF("OGLDataCacheAutoGlyphItem::construct");

  const Field *field=prep->field();
  

  if(!field) return;

  const String &what=prep->marked();

  myDebug()<<nsubfunc<<(void*)field<<what;

  if(what=="Positions") {

    const Positions *pos=field->positions();
    const BaseArray<double> *ar=pos->array();

  
    pndefs=1;
    pdefs=new struct VertexArrayDef[pndefs];
    
    
    pverticessize=ar->size();
    pvertices=new GLdouble[pverticessize*3];
    //myDebug()<<"pverticessize"<<pverticessize;

    long d=ar->shape(0);
    
    if(d>3)
      throw Except(
		   Sprintf("OGLAutoGlyph:: do not know how to "
			   "represent %ld-dimensional positions",d));

    
    //ar->debug();
    
    long nvert=0;
    for(int ip=0;ip<pverticessize;ip++){

      for(int c=-1;++c<d;)
	pvertices[nvert*3+c]=ar->point(ip,c);
      for(int c=d-1;++c<3;)
	pvertices[nvert*3+c]=0.;
      //myDebug()<<ip<<pvertices[nvert*3+0]<<pvertices[nvert*3+1]<<pvertices[nvert*3+2];
      nvert++;
    }
    
    
    pdefs[0].mode=GL_POINTS;
    pdefs[0].count=pverticessize;
    pdefs[0].start=0;
    pdefs[0].useindex=false;
   
  }else if(what=="Connections") {
    const BaseArray<double> *ar=field->positions()->array();
    if(!ar) return;
    const BaseArray<long> *co=field->connections()->array();
    if(!co) return;

    pndefs=1;
    pdefs=new struct VertexArrayDef[pndefs];


    pverticessize=ar->size();
    pvertices=new GLdouble[pverticessize*3];

    long d=ar->shape(0);

    if(d>3)
      throw Except(
		   Sprintf("OGLAutoGlyph:: do not know how to "
			   "represent %ld-dimensional positions",d));

    long nvert=0;
    for(int ip=0;ip<pverticessize;ip++){

      for(int c=-1;++c<d;)
	pvertices[nvert*3+c]=ar->point(ip,c);
      for(int c=d-1;++c<3;)
	pvertices[nvert*3+c]=0.;
      // myDebug()<<"vertex"<<ip
      // 	       <<pvertices[nvert*3+0]<<pvertices[nvert*3+1]<<pvertices[nvert*3+2];
      nvert++;
    }
    
    long nind=0;

    

    if(field->connections()->connectionType()==Connections::CTYCuboid){

      long shape=co->shape(0);
      //myDebug()<<"shape="<<shape;
      long D=dRound(::log(shape)/log(2.));

      pindicessize=co->size()*D*shape;
      pindices=new GLuint[pindicessize];
      
      //myDebug()<<"pindicessize="<<pindicessize;
      //myDebug()<<"D="<<D;
      for(int ic=0;ic<co->size();ic++){

	//todo: this leads to doubled lines, but then we can only use
	// regular MeshArray if it is one, which makes code more complicated
	if(D==1){
	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,1);
	}else if(D==2){
	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,1);

	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,2);

	  pindices[nind++]=co->point(ic,2);
	  pindices[nind++]=co->point(ic,3);

	  pindices[nind++]=co->point(ic,1);
	  pindices[nind++]=co->point(ic,3);

	}else if(D==3){
	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,1);

	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,2);

	  pindices[nind++]=co->point(ic,2);
	  pindices[nind++]=co->point(ic,3);

	  pindices[nind++]=co->point(ic,1);
	  pindices[nind++]=co->point(ic,3);

	  pindices[nind++]=co->point(ic,4);
	  pindices[nind++]=co->point(ic,5);

	  pindices[nind++]=co->point(ic,4);
	  pindices[nind++]=co->point(ic,6);

	  pindices[nind++]=co->point(ic,6);
	  pindices[nind++]=co->point(ic,7);

	  pindices[nind++]=co->point(ic,5);
	  pindices[nind++]=co->point(ic,7);

	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,4);

	  pindices[nind++]=co->point(ic,1);
	  pindices[nind++]=co->point(ic,5);

	  pindices[nind++]=co->point(ic,2);
	  pindices[nind++]=co->point(ic,6);

	  pindices[nind++]=co->point(ic,3);
	  pindices[nind++]=co->point(ic,7);
	}
	
	

	//myDebug()<<"nind="<<nind;
	
      }
    }else if(field->connections()->connectionType()==Connections::CTYSimplex){
      
      long shape=co->shape(0);
      myDebug()<<"shape="<<shape;
      
      long D=shape-1;
      
      pindicessize=co->size()*(D+1)*D;
      pindices=new GLuint[pindicessize];
      
      myDebug()<<"pindicessize="<<pindicessize;
      myDebug()<<"D="<<D;
      for(int ic=0;ic<co->size();ic++){
	
	//todo: this leads to doubled lines, but then we can only use
	// regular MeshArray if it is one, which makes code more complicated
	if(D==1){
	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,1);
	  //myDebug()<<"index"<<(long)pindices[nind-2]<<(long)pindices[nind-1];
	}else if(D==2){
	  pindices[nind++]=co->point(ic,0);
	  pindices[nind++]=co->point(ic,1);
	  //myDebug()<<"index"<<(long)pindices[nind-2]<<(long)pindices[nind-1];
	  pindices[nind++]=co->point(ic,1);
	  pindices[nind++]=co->point(ic,2);
	  //myDebug()<<"index"<<(long)pindices[nind-2]<<(long)pindices[nind-1];
	  pindices[nind++]=co->point(ic,2);
	  pindices[nind++]=co->point(ic,0);
	  //myDebug()<<"index"<<(long)pindices[nind-2]<<(long)pindices[nind-1];
      // 	  pindices[nind++]=co->point(ic,0);
      // 	  pindices[nind++]=co->point(ic,1);
	  
      // 	  pindices[nind++]=co->point(ic,0);
      // 	  pindices[nind++]=co->point(ic,2);
	  
      // 	  pindices[nind++]=co->point(ic,2);
      // 	  pindices[nind++]=co->point(ic,3);
	  
      // 	  pindices[nind++]=co->point(ic,1);
      // 	  pindices[nind++]=co->point(ic,3);
	  
      // 	}else if(D==3){
      // 	  pindices[nind++]=co->point(ic,0);
      // 	  pindices[nind++]=co->point(ic,1);
	  
      // 	  pindices[nind++]=co->point(ic,0);
      // 	  pindices[nind++]=co->point(ic,2);
	  
      // 	  pindices[nind++]=co->point(ic,2);
      // 	  pindices[nind++]=co->point(ic,3);
	  
      // 	  pindices[nind++]=co->point(ic,1);
      // 	  pindices[nind++]=co->point(ic,3);
	  
      // 	  pindices[nind++]=co->point(ic,4);
      // 	  pindices[nind++]=co->point(ic,5);
	  
      // 	  pindices[nind++]=co->point(ic,4);
      // 	  pindices[nind++]=co->point(ic,6);
	  
      // 	  pindices[nind++]=co->point(ic,6);
      // 	  pindices[nind++]=co->point(ic,7);
	  
      // 	  pindices[nind++]=co->point(ic,5);
      // 	  pindices[nind++]=co->point(ic,7);
	  
      // 	  pindices[nind++]=co->point(ic,0);
      // 	  pindices[nind++]=co->point(ic,4);
	  
      // 	  pindices[nind++]=co->point(ic,1);
      // 	  pindices[nind++]=co->point(ic,5);
	  
      // 	  pindices[nind++]=co->point(ic,2);
      // 	  pindices[nind++]=co->point(ic,6);
	  
      // 	  pindices[nind++]=co->point(ic,3);
      // 	  pindices[nind++]=co->point(ic,7);
       	}else
	  throw Except("OGLAutoGlyph: Unkown Simplex dimension");
	
	
      }
	
      myDebug()<<"nind="<<nind;
	
      
    }else
      throw Except("OGLAutoGlyph: Unkown connection type");
    

    pdefs[0].mode=GL_LINES;
    pdefs[0].count=nind;
    pdefs[0].start=0;
    pdefs[0].useindex=true;
    

  }else{
    throw Except(String("OGLAutoGlyph: do not know how to represent ")+what);
    return;
  }

  if(OGLMeshPoints::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  
  if(pdispListName){
    
    OGLIssueVertexArray(pdispListName
   			,pvertices,pnormals,0,pindices,pndefs,pdefs);
    
    
    
    cleanInternal();
    
    
  }
  
  
  ogl.reportGLError(nsubfunc);

  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLAutoGlyph::OGLAutoGlyph(const RepAutoGlyph *rep)
  :OGLObject("OGLAutoGlyph")
  ,prep(rep)
  ,cachedData(0)
{
  setVisible(rep->isVisible());
  setID(rep->id());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLAutoGlyph::~OGLAutoGlyph()
{
  NSF("OGLMeshPoints::~OGLMeshPoints");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLAutoGlyph::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLAutoGlyph::construct(OGLView *oglview)
{
  NSF("OGLAutoGlyph::construct");
  //myDebug() << nsubfunc;

  
  OGLDataCacheAutoGlyphItem *data=new OGLDataCacheAutoGlyphItem(prep);
  
  
  if(data!=(cachedData=oglview->ogldatacache->append(data,false,true)))
    delete data;
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLAutoGlyph::paint(OGLView *
			 ,const OGLMagnifier &,bool reduced
			 ,bool badTransparency
			 )const
{
  NSF("OGLAutoGlyph::paint");
  Q_UNUSED(reduced);
  Q_UNUSED(badTransparency);
  //myDebug() << nsubfunc;


  //   if(type==OGLAutoGlyph::Full)
  //     meshp->material().setDiffuse(0xffff0000);
  //   else
  //     meshp->material().setDiffuse(0xff0000ff);
  
  
  //  pmaterial.issue();
  if(prep->marked()=="Positions") glPointSize(4.);
  glDisable(GL_LIGHTING);
  GLfloat rgb[4]; 
  glColor4fv(GLfloat4FromQRgb(rgb,pmaterial.diffuse()));
  if(cachedData) cachedData->paint(ptransparent);
  glEnable(GL_LIGHTING);
  
  
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLAutoGlyph::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLAutoGlyph::NFPlane";


  const BoundingBox &bbox=prep->boundingBox();
  
  //myDebug() << cell.printS()();
  
  double p0=bbox.origin()*n;
  const Matrix3x3 &cell=bbox.cell();
  
  near=-p0;
  far=-p0;
  
  double c0=cell(0)*n;
  double c1=cell(1)*n;
  double c2=cell(2)*n;
  
  

  
  double d=c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  
  d=c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1+c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1+c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c2+c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c0+c1+c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  
  //myDebug() << near<<far;  
  
  return true;
}



    



/*EOF*/ 
