/*************************************************************************
 * @(#)OGLBSPTree.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Apr 2011
 *************************************************************************
 */

#include <stdio.h>
#include "OGLBSPTree.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"
#include "ValueArray.hh"
#include "MyDebug.hh"

//long OGLBSPTree::countfrontrender=0;
//long OGLBSPTree::countbackrender=0;
//long OGLBSPTree::countsplitenter=0;
//long OGLBSPTree::countprim=0;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPolygon::calcEpsilon()
{
  if(pnvertices==3){
    peps=0;
    peps=MyMath::max<double>(peps,fabs(pvertices3[0]-pvertices3[0]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[3]-pvertices3[0]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[6]-pvertices3[0]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[1]-pvertices3[1]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[4]-pvertices3[1]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[7]-pvertices3[1]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[2]-pvertices3[2]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[5]-pvertices3[2]));
    peps=MyMath::max<double>(peps,fabs(pvertices3[8]-pvertices3[2]));
    
    peps*=1.0e-6;
    return;
  }
  
  
  peps=0;
  for (short i=-1;++i<pnvertices;){
    peps=MyMath::max<double>(peps,fabs(pvertices[3*i]-pvertices[0]));
    peps=MyMath::max<double>(peps,fabs(pvertices[3*i+1]-pvertices[1]));
    peps=MyMath::max<double>(peps,fabs(pvertices[3*i+2]-pvertices[2]));
  }
  peps*=1.0e-6;
  // myDebug() << pclass << "peps="<<peps;
  // printf("v0=%12.5f %12.5f %12.5f\n",vert[0],vert[1],vert[2]);
  // printf("v1=%12.5f %12.5f %12.5f\n",vert[3],vert[4],vert[5]);
  // printf("v2=%12.5f %12.5f %12.5f\n",vert[6],vert[7],vert[8]);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLPolygon::PolygonClassification 
OGLPolygon::split3(const GLdouble *part,OGLPolygon **front,
		   OGLPolygon **back)const
{
  //myDebug()<<"OGLPolygon::split";
  //OGLBSPTree::countsplitenter++;
  //todo: parallel planes
  // determine sides and correct for degeneracy
  GLdouble sides[3];
  GLdouble eps=peps;
  sides[0]=part[0]*pvertices3[0]+part[1]*pvertices3[1]
    +part[2]*pvertices3[2]-part[3];
  sides[1]=part[0]*pvertices3[3]+part[1]*pvertices3[4]
    +part[2]*pvertices3[5]-part[3];
  sides[2]=part[0]*pvertices3[6]+part[1]*pvertices3[7]
    +part[2]*pvertices3[8]-part[3];
  char wsides0=(sides[0]>eps?1:(sides[0]<-eps?-1:0));
  char wsides1=(sides[1]>eps?1:(sides[1]<-eps?-1:0));
  char wsides2=(sides[2]>eps?1:(sides[2]<-eps?-1:0));
  char sum=wsides0+wsides1+wsides2;
  char sumabs=abs(wsides0)+abs(wsides1)+abs(wsides2);


  if(sum<-1  || (sum==-1 && sumabs==1)) return OGLPolygon::InBackOf;
  else if(sum>1 || (sum==1 && sumabs==1)) return OGLPolygon::InFrontOf;
  else if(sumabs==0) return OGLPolygon::CoIncident;
  
  
  short i0;
  GLdouble *vfront,*vback,*nfront,*nback;
  GLfloat *cdfront,*cdback;
  if(((sumabs==3) && (sum==-1))||(sumabs==2)){
    if(wsides0>0){
      i0=0;
    }else if(wsides1>0){
      i0=1;
    }else if(wsides2>0){
      i0=2;
    }
    *front=new OGLPolygon(3,phasnormals,phascolors,pfaces);
    *back=new OGLPolygon(4,phasnormals,phascolors,pfaces);
    vfront=(*front)->pvertices3;
    vback=(*back)->pvertices;
    if(phasnormals){
      nfront=(*front)->pnormals3;
      nback=(*back)->pnormals;
    }
    if(phascolors){
      cdfront=(*front)->pcolors3;
      cdback=(*back)->pcolors;
    }
    
  }else{
    if(wsides0<0){
      i0=0;
    }else if(wsides1<0){
      i0=1;
    }else if(wsides2<0){
      i0=2;
    }
    *front=new OGLPolygon(4,phasnormals,phascolors,pfaces);
    *back=new OGLPolygon(3,phasnormals,phascolors,pfaces);
    vback=(*front)->pvertices;
    vfront=(*back)->pvertices3;
    if(phasnormals){
      nback=(*front)->pnormals;
      nfront=(*back)->pnormals3;
    }
    if(phascolors){
      cdback=(*front)->pcolors;
      cdfront=(*back)->pcolors3;
    }
  }
  
  {
    const GLdouble *ptA=pvertices3+3*i0;
    const GLdouble *pnA;
    const GLfloat *pcdA;
    memcpy(vfront,ptA,sizeof(GLdouble)*3);
    vfront+=3;
    if(phasnormals){
      pnA=pnormals3+3*i0;
      memcpy(nfront,pnA,sizeof(GLdouble)*3);
      nfront+=3;
    }
    if(phascolors){
      pcdA=pcolors3+4*i0;
      memcpy(cdfront,pcdA,sizeof(GLfloat)*4);
      cdfront+=4;
    }
    short i1=((i0+1)%3);
    const GLdouble *ptB=pvertices3+3*i1;
    const GLdouble *pnB;
    const GLfloat *pcdB;
    GLdouble x=sides[i1]-sides[i0];
    if(x!=0) x=MyMath::min<double>(MyMath::max<double>(-0.1,-sides[i0]/x),1.1);
    vfront[0]=vback[0]=ptA[0]+(ptB[0]-ptA[0])*x;
    vfront[1]=vback[1]=ptA[1]+(ptB[1]-ptA[1])*x;
    vfront[2]=vback[2]=ptA[2]+(ptB[2]-ptA[2])*x;
    vfront+=3;
    vback+=3;
    if(phasnormals){
      pnB=pnormals3+3*i1;
      nfront[0]=nback[0]=pnA[0]+(pnB[0]-pnA[0])*x;
      nfront[1]=nback[1]=pnA[1]+(pnB[1]-pnA[1])*x;
      nfront[2]=nback[2]=pnA[2]+(pnB[2]-pnA[2])*x;
      nfront+=3;
      nback+=3;
    }
    if(phascolors){
      pcdB=pcolors3+4*i1;
      for(short i=-1;++i<4;) cdfront[i]=cdback[i]=pcdA[i]+(pcdB[i]-pcdA[i])*x;
      cdfront+=4;
      cdback+=4;
    }
    memcpy(vback,ptB,sizeof(GLdouble)*3);
    vback+=3;
    if(phasnormals){
      memcpy(nback,pnB,sizeof(GLdouble)*3);
      nback+=3;
    }
    if(phascolors){
      memcpy(cdback,pcdB,sizeof(GLfloat)*4);
      cdback+=4;
    }
    i1=((i0+2)%3);
    ptB=pvertices3+3*i1;
    memcpy(vback,ptB,sizeof(GLdouble)*3);
    vback+=3;
    if(phasnormals){
      pnB=pnormals3+3*i1;
      memcpy(nback,pnB,sizeof(GLdouble)*3);
      nback+=3;
    }
    if(phascolors){
      pcdB=pcolors3+4*i1;
      memcpy(cdback,pcdB,sizeof(GLfloat)*4);
      cdback+=4;
    }
    x=sides[i1]-sides[i0];
    if(x!=0) x=MyMath::min<double>(MyMath::max<double>(-0.1,-sides[i0]/x),1.1);
    vfront[0]=vback[0]=ptA[0]+(ptB[0]-ptA[0])*x;
    vfront[1]=vback[1]=ptA[1]+(ptB[1]-ptA[1])*x;
    vfront[2]=vback[2]=ptA[2]+(ptB[2]-ptA[2])*x;
    if(phasnormals){
      nfront[0]=nback[0]=pnA[0]+(pnB[0]-pnA[0])*x;
      nfront[1]=nback[1]=pnA[1]+(pnB[1]-pnA[1])*x;
      nfront[2]=nback[2]=pnA[2]+(pnB[2]-pnA[2])*x;
    }
    if(phascolors) 
      for(short i=-1;++i<4;) cdfront[i]=cdback[i]=pcdA[i]+(pcdB[i]-pcdA[i])*x;
    
    
    (*front)->calcEpsilon();
    (*back)->calcEpsilon();
    (*front)->setMaterial(pmatfront,pmatback);
    (*back)->setMaterial(pmatfront,pmatback);
    memcpy((*front)->pplane,pplane,sizeof(GLdouble)*4);
    memcpy((*back)->pplane,pplane,sizeof(GLdouble)*4);
    return OGLPolygon::Spanning;
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLPolygon::PolygonClassification 
OGLPolygon::splitgeneral(const GLdouble *part,OGLPolygon **front
			 ,OGLPolygon **back)const
{

  //todo: parallel planes
  // determine sides and correct for degeneracy
  //  myDebug()<<pnvertices;
  GLdouble *sides=new GLdouble[pnvertices];
  short *wsides=new short[pnvertices];
  short sum=0;
  short sumabs=0;
  double eps=peps;
 cutsloop:;
  for (short i=-1;++i<pnvertices;){
    sides[i]=part[0]*pvertices[3*i]+part[1]*pvertices[3*i+1]
      +part[2]*pvertices[3*i+2]-part[3];
    wsides[i]=(sides[i]>eps?1:(sides[i]<-eps?-1:0));
    sum+=wsides[i];
    sumabs+=abs(wsides[i]);
  }
  // myDebug()<< "start wsides";
  //for (short i=-1;++i<pnvertices;){
  //   printf("cuts i %d %4d %g\n",i,wsides[i],sides[i]);
  // }

  if(sum==-pnvertices){
    //early return
    //myDebug() << "early return InBackOf";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::InBackOf;
  }else if(sum==pnvertices){
    //early return
    
    //myDebug() << "early return InFrontOf";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::InFrontOf;
  }else if(sumabs==0){
    //early return
    //myDebug() << "early return CoIncident";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::CoIncident;
  }

  short igotsome=0;
  bool gotsome;
  do{
    igotsome++;
    //myDebug() <<"igotsome"<<igotsome;
    gotsome=false;
    for (short i=-1;++i<pnvertices;){
      if(wsides[i]==0){
	gotsome=true;
	short wplus=wsides[(i+1)%pnvertices];
	short wminus=wsides[(i-1+pnvertices)%pnvertices];
	short ww=wplus*wminus;
	if(ww<0){
	  // neighbours on different sides, either choice is good
	  wsides[i]=-1;
	}else if(ww>0){
	  // neighbours on same side, convex polynom is on one side only
	  wsides[i]=wplus;
	}else{ // ww==0
	  if(wminus!=0){
	    // left neighbour on a side right point on 0:
	    // move the current point towards left point
	    wsides[i]=wminus;
	    //wsides[(i+1)%pnvertices]=wminus;
	  }else if(wplus!=0){
	    // right neighbour on a side left point on 0:
	    // move the current point towards right point
	    ///	    wsides[(i-1+pnvertices)%pnvertices]=wplus;
	    wsides[i]=wplus;
	  }else{
	    // wminus==w==wplus==0
	    // is solved in one the next loops, since not all wsides are zero
	  }
	}
      }
    }  
  }while(gotsome);

  sum=0;
  sumabs=0;
  short cuts=0;
  for (short i=-1;++i<pnvertices;){
    sum+=wsides[i];
    sumabs+=abs(wsides[i]);
    cuts+=abs(wsides[i]+wsides[(i+1)%pnvertices]);
  }
  if(sum==-pnvertices){
    //early return
    //myDebug() << "second early return InBackOf";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::InBackOf;
  }else if(sum==pnvertices){
    //early return
    
    //myDebug() << "second early return InFrontOf";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::InFrontOf;
  }else if(sumabs==0){
    //early return
    //myDebug() << "second early return CoIncident";
    delete [] sides;
    delete [] wsides;
    return OGLPolygon::CoIncident;
  }

  if(2*pnvertices-cuts!=4){
    myDebug()<< "cuts"<< 2*pnvertices-cuts;
    for (short i=-1;++i<pnvertices;){
      myDebug("cuts i %d %4d %g",i,wsides[i],sides[i]);
      // todo: count cuts and throw if !=2
    }
    eps=eps*10.;
    goto cutsloop;
    exit(1);
  }

  
  int maxdim=6*pnvertices;
  GLdouble *outpts=new GLdouble[maxdim];
  GLdouble *inpts=new GLdouble[maxdim];
  GLdouble *poutpts=outpts;
  GLdouble *pinpts=inpts;
  GLdouble *noutpts=0,*npoutpts,*ninpts=0,*npinpts;
  GLfloat *cdoutpts=0,*cdpoutpts,*cdinpts=0,*cdpinpts;
  if(phasnormals){
    noutpts=new GLdouble[maxdim];
    npoutpts=noutpts;
    ninpts=new GLdouble[maxdim];
    npinpts=ninpts;
  }
  if(phascolors){
    cdoutpts=new GLfloat[pnvertices*16];
    cdpoutpts=cdoutpts;
    cdinpts=new GLfloat[pnvertices*16];
    cdpinpts=cdinpts;
  }
  
  const GLdouble *pnA,*ptA=pvertices+3*pnvertices-3;
  const GLfloat *pcdA;
  if(phasnormals) pnA=pnormals+3*pnvertices-3;
  if(phascolors) pcdA=pcolors+4*pnvertices-4;
  GLdouble sideA=sides[pnvertices-1];
  short wsideA=wsides[pnvertices-1];
  for (short i=-1;++i<pnvertices;){
    const GLdouble *pnB,*ptB=pvertices+3*i;
    const GLfloat *pcdB;
    if(phasnormals)pnB=pnormals+3*i;
    if(phascolors)pcdB=pcolors+4*i;
    GLdouble sideB=sides[i];
    short wsideB=wsides[i];
    if(wsideB==-wsideA){
      //todo: remove degenerate
      //
      // compute the intersection point of the line
      // from point A to point B with the partition
      // plane. This is a simple ray-plane intersection.
      // A+(B-A)*x=d
      // (A*n)+((B*n)-(A*n))*x=d
      // (A*n-d)+((B*n-d)-(A*n-d))*x=0
      // sideA+(sideB-sideA)*x=0
      GLdouble b=sideB-sideA;
      GLdouble x;
      if(b==0){ //todo: toleance ?
	// there is nothing we can do, than to assume sideA==0
	x=0;
      }else{
	x=MyMath::min<double>(MyMath::max<double>(-0.1,-sideA/b),1.1);
      }
      poutpts[0]=pinpts[0]=ptA[0]+(ptB[0]-ptA[0])*x;
      poutpts[1]=pinpts[1]=ptA[1]+(ptB[1]-ptA[1])*x;
      poutpts[2]=pinpts[2]=ptA[2]+(ptB[2]-ptA[2])*x;
      poutpts+=3;
      pinpts+=3;

      if(phasnormals){
	npoutpts[0]=npinpts[0]=pnA[0]+(pnB[0]-pnA[0])*x;
	npoutpts[1]=npinpts[1]=pnA[1]+(pnB[1]-pnA[1])*x;
	npoutpts[2]=npinpts[2]=pnA[2]+(pnB[2]-pnA[2])*x;
	npoutpts+=3;
	npinpts+=3;
      }

      if(phascolors){
	for(short i=-1;++i<4;)
	  cdpoutpts[i]=cdpinpts[i]=pcdA[i]+(pcdB[i]-pcdA[i])*x;
	cdpoutpts+=4;
	cdpinpts+=4;
      }

    }
    if(wsideB>0){
      memcpy(poutpts,ptB,sizeof(GLdouble)*3);	
      poutpts+=3;
      if(phasnormals){
	memcpy(npoutpts,pnB,sizeof(GLdouble)*3);	
	npoutpts+=3;
      }
      if(phascolors){
	memcpy(cdpoutpts,pcdB,sizeof(GLfloat)*4);
	cdpoutpts+=4;
      }
    }else if(wsideB<0){
      memcpy(pinpts,ptB,sizeof(GLdouble)*3);
      pinpts+=3;
      if(phasnormals){
	memcpy(npinpts,pnB,sizeof(GLdouble)*3);
	npinpts+=3;
      }
      if(phascolors){
	memcpy(cdpinpts,pcdB,sizeof(GLfloat)*4);
	cdpinpts+=4;
      }
    }
    ptA=ptB;
    pnA=pnB;
    pcdA=pcdB;
    sideA=sideB;
    wsideA=wsideB;
  }
  *front=new OGLPolygon((poutpts-outpts)/3,outpts,false,noutpts,cdoutpts,pfaces);
  *back=new OGLPolygon((pinpts-inpts)/3,inpts,false,ninpts,cdinpts,pfaces);
  (*front)->setMaterial(pmatfront,pmatback);
  (*back)->setMaterial(pmatfront,pmatback);
  memcpy((*front)->pplane,pplane,sizeof(GLdouble)*4);
  memcpy((*back)->pplane,pplane,sizeof(GLdouble)*4);
  
  
  delete [] outpts;
  delete [] inpts;
  delete [] sides;
  delete [] wsides;
  if(phasnormals){
    delete [] noutpts;
    delete [] ninpts;
  }
  if(phascolors){
    delete [] cdoutpts;
    delete [] cdinpts;
  }
  //myDebug()<<"no early return Spanning";
  return OGLPolygon::Spanning;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLBSPTree::OGLBSPTree()
  :pseeded(false),front(0),back(0)
{
  //myDebug() << "OGLBSPTree "<< this << " start living";
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLBSPTree::~OGLBSPTree()
{
  //myDebug() << "OGLBSPTree "<< this << " dying #polys"<<polys.size();
  clear();
  //myDebug() << "OGLBSPTree "<< this << " DONE dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::clear()
{
  //myDebug()<< "OGLBSPTree::clear"<<this;
  //qDeleteAll(polys.begin(),polys.end());
  for(long i=0;i<polys.size();i++) delete polys[i];
  polys.clear();
  if(front)delete front; front=0;
  if(back)delete back; back=0;
  pseeded=false;
  //myDebug()<< "OGLBSPTree::clear END"<<this;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::seedTree(const GLdouble *plane)
{
  //myDebug()<< "OGLBSPTree::seedTree"<<this;
  clear();
  memcpy(partitionplane,plane,sizeof(GLdouble)*4); 
  // printf("partitionplane n=[%12.5f , %12.5f , %12.5f] d=%12.5f\n",
  // 	 partitionplane[0],partitionplane[1],partitionplane[2],partitionplane[3]);
  pseeded=true;
  //myDebug()<< "OGLBSPTree::seedTree END"<<this;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
static OGLPolygon *makeSeedPoly(const GLdouble *seedplanes)
{
  GLdouble pvertices[12];


  double d=seedplanes[3];
  double d1=MyMath::max<double>(d,0.5);
  Vec3 n(seedplanes);
  Vec3 v1,v2;
  n.findPerpDirections(v1,v2);
  v1*=d1;
  v2*=d1;
  Vec3 v=v1+v2+n*d;
  pvertices[0]=v[0];
  pvertices[1]=v[1];
  pvertices[2]=v[2];
  v=-v1+v2+n*d;
  pvertices[3]=v[0];
  pvertices[4]=v[1];
  pvertices[5]=v[2];
  v=-v1-v2+n*d;
  pvertices[6]=v[0];
  pvertices[7]=v[1];
  pvertices[8]=v[2];
  v=v1-v2+n*d;
  pvertices[9]=v[0];
  pvertices[10]=v[1];
  pvertices[11]=v[2];
  
  OGLPolygon *poly=new OGLPolygon(4,pvertices,false,
   				  0,0,GL_FRONT_AND_BACK);
  poly->setPlane(n,d);
  poly->setMaterial(new OGLMaterial(),new OGLMaterial());
  return poly;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::initialSeed(const ValueArray<GLdouble> &seedplane)
{
  if(seedplane.size()!=4) return;
  //myDebug()<<"OGLBSPTree::initialSeed"<<pseeded<<seedplane.size();
  // create a few leaves of the tree seeding with the planes, this
  // is for tree balancing
  if(pseeded) return;
  
  //printf("seedplane n=[%12.5f , %12.5f , %12.5f] d=%12.5f\n",
  //seedplane.at(0),seedplane.at(1),seedplane.at(2),seedplane.at(3));

  seedTree(seedplane.constDataRef());

  //polys.append(makeSeedPoly(seedplane.constDataRef()));
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::initialSeed(const ValueArray<ValueArray<GLdouble> > &seedplanes)
{
  if(seedplanes.size()==0)return;
  if(pseeded) return;
  //myDebug()<<"OGLBSPTree::initialSeed"<<seedplanes.size();

  OGLBSPTree *current=this;

  long side=0;
  for(int i=-1;++i<seedplanes.size();){
    //myDebug()<<"addToTree() with seeding";
    current->initialSeed(seedplanes.at(i));
    if(side==0){
      current=current->front=new OGLBSPTree();
      side=1;
    }else{
      current=current->back=new OGLBSPTree();
      side=0;
    }
  }

  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void OGLBSPTree::addToTree(OGLPolygon *poly)
{
  NSF("OGLBSPTree::addToTree(OGLPolygon *poly)");
  
  if(!pseeded){
    seedTree(poly->pplane);
    polys.append(poly);
    return;
  }
  // if(polys.size()<1){
  //   seedTree(poly->pplane);
  //   polys.append(poly);
  //   return;
  // }
  
  OGLPolygon   *frontPiece=0;
  OGLPolygon   *backPiece=0;
  OGLPolygon::PolygonClassification result
    =poly->split(partitionplane,&frontPiece,&backPiece);
  
  switch(result)
    {
    case OGLPolygon::CoIncident:
      polys.append(poly);
      return;
    case OGLPolygon::InBackOf:
      if(!back) back=new OGLBSPTree();
      back->addToTree(poly);
      return;
    case OGLPolygon::InFrontOf:
      if(!front) front=new OGLBSPTree();
      front->addToTree(poly);
      return;
    case OGLPolygon::Spanning:
      if(!back) back=new OGLBSPTree();
      back->addToTree(backPiece);
      if(!front) front=new OGLBSPTree();
      front->addToTree(frontPiece);
      delete poly;
      return;
    }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::backToFrontRender(const Vec3 &eye,const Vec3 &z,bool persp)
{
  //myDebug()<<"OGLBSPTree::backToFrontRender"<<polys.size();
  // camera is at eye and we are looking down the direction z

  //  (plane.normal*(-z))*(plane.normal*eye-plane.dist) <0 draw front first
  GLdouble side;
  if(persp) 
    side=partitionplane[0]*eye[0]+partitionplane[1]*eye[1]
      +partitionplane[2]*eye[2]-partitionplane[3];
  else
    side=-(partitionplane[0]*z[0]+partitionplane[1]*z[1]+partitionplane[2]*z[2]);
  
//   myDebug() << "n"<<Vec3(n);
//   myDebug() << "z"<<z;
//   myDebug() << "side"<<side;
//   myDebug() << "persp"<<persp;
  if(side>=0){
    //myDebug() << "positive side";
    if(back){
      //countbackrender++;
      back->backToFrontRender(eye,z,persp);
    }
    drawPolygonList();
    if(front){
      //countfrontrender++;
      front->backToFrontRender(eye,z,persp);
    }
  }else{
    //myDebug() << "negative side";
    if(front){
      //countfrontrender++;
      front->backToFrontRender(eye,z,persp);
    }
    drawPolygonList();
    if(back){
      //countbackrender++;
      back->backToFrontRender(eye,z,persp);
    }
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLBSPTree::drawPolygonList()
{
  //  myDebug() << "OGLBSPTree::drawPolygonList GL_";
  for(long i=0;i<polys.size();i++){
    //myDebug()<<++count;
    polys[i]->paint();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPolygon::paint()const
{
  //if(pmat) pmat->issue(false);
  pmatfront->issue(true);
  pmatback->issue(true);

  //OGLBSPTree::countprim++;
  
  //myDebug()<<"pmatfront"<<pmatfront.printS();

  if(pnvertices==3){
    glBegin(GL_TRIANGLES);
    if(phasnormals){
      if(phascolors){
	glNormal3dv(pnormals3 +0);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+0);
	glVertex3dv(pvertices3+0);
	glNormal3dv(pnormals3 +3);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+4);
	glVertex3dv(pvertices3+3);
	glNormal3dv(pnormals3 +6);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+8);
	glVertex3dv(pvertices3+6);
      }else{
	glNormal3dv(pnormals3 +0);
	glVertex3dv(pvertices3+0);
	glNormal3dv(pnormals3 +3);
	glVertex3dv(pvertices3+3);
	glNormal3dv(pnormals3 +6);
	glVertex3dv(pvertices3+6);
      }
    }else {
      if(phascolors){
	glNormal3dv(pplane);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+0);
	glVertex3dv(pvertices3+0);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+4);
	glVertex3dv(pvertices3+3);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors3+8);
	glVertex3dv(pvertices3+6);
      }else{
	glNormal3dv(pplane);
	glVertex3dv(pvertices3+0);
	glVertex3dv(pvertices3+3);
	glVertex3dv(pvertices3+6);
      }
    }
    glEnd();
  }else if(pnvertices==4){
    glBegin(GL_QUADS);
    if(phasnormals){
      if(phascolors){
	glNormal3dv(pnormals +0);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+0);
	glVertex3dv(pvertices+0);
	glNormal3dv(pnormals +3);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+4);
	glVertex3dv(pvertices+3);
	glNormal3dv(pnormals +6);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+8);
	glVertex3dv(pvertices+6);
	glNormal3dv(pnormals +9);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+12);
	glVertex3dv(pvertices+9);
      }else{
	glNormal3dv(pnormals +0);
	glVertex3dv(pvertices+0);
	glNormal3dv(pnormals +3);
	glVertex3dv(pvertices+3);
	glNormal3dv(pnormals +6);
	glVertex3dv(pvertices+6);
	glNormal3dv(pnormals +9);
	glVertex3dv(pvertices+9);
      }
    }else{
      if(phascolors){
	glNormal3dv(pplane);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+0);
	glVertex3dv(pvertices+0);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+4);
	glVertex3dv(pvertices+3);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+8);
	glVertex3dv(pvertices+6);
	glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+12);
	glVertex3dv(pvertices+9);
      }else{
	glNormal3dv(pplane);
	glVertex3dv(pvertices+0);
	glVertex3dv(pvertices+3);
	glVertex3dv(pvertices+6);
	glVertex3dv(pvertices+9);
      }
    }
    glEnd();
  }else{
    glBegin(GL_POLYGON);
    if(!phasnormals) glNormal3dv(pplane);
    int l=0,lc=0;
    if(phasnormals){
      if(phascolors){
	for(int iv=0;iv<pnvertices;iv++){
	  glNormal3dv(pnormals+l);
	  glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+lc);
	  glVertex3dv(pvertices+l);
	  l+=3;
	  lc+=4;
	}
      }else{
	for(int iv=0;iv<pnvertices;iv++){
	  glNormal3dv(pnormals+l);
	  glVertex3dv(pvertices+l);
	  l+=3;
	}
      }
    }else{
      if(phascolors){
	for(int iv=0;iv<pnvertices;iv++){
	  glMaterialfv(pfaces,GL_AMBIENT_AND_DIFFUSE,pcolors+lc);
	  glVertex3dv(pvertices+l);
	  l+=3;
	  lc+=4;
	}
      }else{
	for(int iv=0;iv<pnvertices;iv++){
	  glVertex3dv(pvertices+l);
	  l+=3;
	}
      }
    }
    glEnd();
  }

  
}

/*EOF*/ 
