/* @(#)OGLBSPTree.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLBSPTREE_H
#define _OGLBSPTREE_H 1

//#include <QGLWidget>
#ifdef _DARWIN_COMPAT
#include <OpenGL/glu.h>
#else
#include <GL/glu.h>
#endif
#include "Vec3.hh"
#include "OGLMaterial.hh"
#include "MyDebug.hh"

template <class T>
class ValueArray;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLPolygon{
public:
  enum PolygonClassification{CoIncident=0,InBackOf=1,InFrontOf=2
			     ,Spanning=3};

  OGLPolygon():pnvertices(0),phasnormals(false),phascolors(false){
    pvertices=0;pnormals=0,pcolors=0;
    //myDebug()<<"OGLPolygon"<<(void*)this;
  };
 
  OGLPolygon(int num,bool initnormals,bool initcolors,GLenum faces)
    :pnvertices(num),phasnormals(initnormals),phascolors(initcolors),pfaces(faces){
    //myDebug()<<"OGLPolygon"<<(void*)this;
    //myDebug()<<"phasnormals="<<phasnormals;
    if(num>3) {
      pvertices=new GLdouble[num*3];
      if(initnormals) pnormals=new GLdouble[num*3];
      else pnormals=0;
      
      if(initcolors) pcolors=new GLfloat[num*4];
      else pcolors=0;
    }
    //myDebug()<<"OGLPolygon(num)"<<this<<pnvertices<<pnormals;
  };
  
  OGLPolygon(int num,const GLdouble *vertices,
	     bool initializeplanefromvertices,
	     const GLdouble *normals,const GLfloat *colors,GLenum faces)
    :pnvertices(num),phasnormals(normals!=0),phascolors(colors!=0),pfaces(faces){
    //myDebug()<<"OGLPolygon"<<(void*)this;
    if(num>3) {
      pvertices=new GLdouble[num*3];
      memcpy(pvertices,vertices,sizeof(GLdouble)*pnvertices*3);
      if(normals){
	pnormals=new GLdouble[num*3];
	memcpy(pnormals,normals,sizeof(GLdouble)*pnvertices*3);
      }else pnormals=0;
      if(colors){
	pcolors=new GLfloat[num*4];
	memcpy(pcolors,colors,sizeof(GLfloat)*pnvertices*4);
      }else pcolors=0;
    }else{
      memcpy(pvertices3,vertices,sizeof(GLdouble)*9);
      if(normals) memcpy(pnormals3,normals,sizeof(GLdouble)*9);
      if(colors) memcpy(pcolors3,colors,sizeof(GLfloat)*24);
    }
    calcEpsilon();
    if(initializeplanefromvertices) setPlaneFromVertices(vertices);
    //myDebug()<<"OGLPolygon(num,vertices,...)"<<this<<pnvertices<<pnormals;
  };

  ~OGLPolygon(){
    if(pnvertices>3) {
      //myDebug()<<"OGLPolygon dying"<<(void*)this;
      //myDebug()<<"~OGLPolygon"<<this<<pnvertices<<pnormals;
      if(pvertices) delete [] pvertices;
      if(pnormals) delete [] pnormals;
      if(pcolors) delete [] pcolors;
      //todo: is it always set to zero
    }
  };

  // inline const OGLPlane &plane()const{return pplane;};
  // inline void setPlane(const Vec3 &v,double dist){pplane.setPlane(v,dist);};
  // inline void setPlane(const OGLPlane &p){ pplane=p;};


  inline void setPlane(const Vec3 &v,GLdouble d){ 
    pplane[0]=v[0];
    pplane[1]=v[1];
    pplane[2]=v[2];
    pplane[3]=d;
  };
  
  inline void setMaterial(const OGLMaterial *matfront,const OGLMaterial *matback){
    pmatfront=matfront; pmatback=matback; };

  inline  void setVertex(int i,double x,double y,double z){
    if(pnvertices>3){
      pvertices[3*i+0]=x;;
      pvertices[3*i+1]=y;;
      pvertices[3*i+2]=z;;
    }else{
      pvertices3[3*i+0]=x;;
      pvertices3[3*i+1]=y;;
      pvertices3[3*i+2]=z;;
    }
  }
  
  void paint()const;

  inline OGLPolygon::PolygonClassification split(const GLdouble *part
					       ,OGLPolygon **front
					       ,OGLPolygon **back)const{
    if(pnvertices==3)
      return split3(part,front,back);
    else
      return splitgeneral(part,front,back);
  }
  
  void calcEpsilon();

private:
  OGLPolygon::PolygonClassification split3(const GLdouble *part
					,OGLPolygon **front
					,OGLPolygon **back)const;

  OGLPolygon::PolygonClassification splitgeneral(const GLdouble *part
					,OGLPolygon **front
					,OGLPolygon **back)const;

  inline void setPlaneFromVertices(const GLdouble *v){ 
    const GLdouble *v0=v;
    const GLdouble *v1=v+3;
    const GLdouble *v2=v+6;
    //printf("loadfile(\"Maxima/vecops.mac\");\n");
    //printf("v0:[%f,%f,%f];\n",v0[0],v0[1],v0[2]);
    //printf("v1:[%f,%f,%f];\n",v1[0],v1[1],v1[2]);
    //printf("v2:[%f,%f,%f];\n",v2[0],v2[1],v2[2]);
    
    pplane[0]=(v1[1]-v0[1])*(v2[2]-v0[2])-(v1[2]-v0[2])*(v2[1]-v0[1]);
    pplane[1]=(v1[2]-v0[2])*(v2[0]-v0[0])-(v1[0]-v0[0])*(v2[2]-v0[2]);
    pplane[2]=(v1[0]-v0[0])*(v2[1]-v0[1])-(v1[1]-v0[1])*(v2[0]-v0[0]);
    //printf("nn:[%f,%f,%f]\n",pplane[0],pplane[1],pplane[2]);
    GLdouble an=sqrt(pplane[0]*pplane[0]+pplane[1]*pplane[1]+pplane[2]*pplane[2]);
    //printf("an:%f\n",an);
    if(an!=0.){pplane[0]/=an;pplane[1]/=an;pplane[2]/=an;}
    pplane[3]=pplane[0]*v0[0]+pplane[1]*v0[1]+pplane[2]*v0[2];
    //printf("n:[%f,%f,%f], dist=%f\n",pplane[0],pplane[1],pplane[2],pplane[3]);

  }
protected:
  GLdouble pplane[4];
  const OGLMaterial *pmatfront;
  const OGLMaterial *pmatback;
  GLdouble peps; // tolerance for point to being considered on a plane
private:
  int pnvertices;
  bool phasnormals,phascolors;
  union {
    GLdouble *pvertices;
    GLdouble pvertices3[9];
  };
  union {
    GLdouble *pnormals;
    GLdouble pnormals3[9];
  };
  union {
    GLfloat *pcolors;
    GLfloat pcolors3[24];
  };
  GLenum pfaces;

  friend class OGLBSPTree;

};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLBSPTree{
 public:
  OGLBSPTree();
  ~OGLBSPTree();

  void clear();

  void initialSeed(const ValueArray<ValueArray<GLdouble> > &seedplanes);;

  void addToTree(OGLPolygon *polygon);

  void backToFrontRender(const Vec3 &eye,const Vec3 &z,bool persp);


 private:
  void seedTree(const GLdouble *);
  void drawPolygonList();
  void initialSeed(const ValueArray<GLdouble> &seedplane);
 private:
  GLdouble partitionplane[4];
  ValueArray<const OGLPolygon*> polys;
  bool pseeded;
  OGLBSPTree *front;
  OGLBSPTree *back;
public: //todo: remove
  //static long countsplitenter;
  //static long countfrontrender;
  //static long countbackrender;
  //static long countprim;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLBSPTREE_H */

