/* @(#)OGLDataCache.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLDATACACHE_H
#define _OGLDATACACHE_H 1

#include <QGLWidget>
#include <QString>
#include "OGL.hh"

class RegularMesh;
class TriangleMesh;
class ColorModel;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheItem{
 public:

  enum RodEnding{Open=0,Flat=1,Round=2};

  OGLDataCacheItem(const QString &isa);

  bool operator==(const OGLDataCacheItem &other);

  inline const QString &objectType()const {return pisa;};
  inline bool isA(const QString &a)const{return (a==pisa);};
  inline const char *id()const{return pid;}
  
  virtual ~OGLDataCacheItem();
  void cleanInternal();

  inline int imageWidth()const{return pimageNx;};
  inline int imageHeight()const{return pimageNy;};

  inline bool isConstructed()const{return pconstructed;};
  inline int used()const {return pnused;};
  inline void setUnUsed(){pnused=0;};
  inline void increaseUsed(){pnused++;};

  virtual QString printSpecQ()const;
  
  virtual void construct(bool /*transparent*/){pconstructed=true;};
  virtual void paint(bool transparent)  const;

  // make a cylindrical object with or without lids
  void makeRodMesh(long &nvert,long &idef,long &nn0,
		   long pnphi,
		   double radius,double L,
		   double z0,const Matrix3x3 &rot,
		   RodEnding left,RodEnding right,
		   bool initsize=false);


  
 protected:
  QString pisa;
  char *pid;
  int pnused;
  bool pconstructed;
  GLuint pdispListName;
  long pverticessize; 
  long pindicessize; 
  GLdouble *pnormals; 
  GLdouble *pvertices; 
  GLfloat  *pcolors; 
  GLuint *pindices; 
  long pndefs;
  struct VertexArrayDef *pdefs;
  bool pcull;
  int pimageNx,pimageNy;
  GLubyte *pimage;

  
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */

class OGLDataCache{
 public:
  OGLDataCache(){};
  virtual ~OGLDataCache();

  void markAsOld();
  void cleanOldCacheData();

  void cleanCache(const QString &what="");

  OGLDataCacheItem *append(OGLDataCacheItem *d,bool transparent=false,
			   bool forceconstruct=false);
  void sortTransparent(const Vec3 &z);
  // remove all cache objects which refer to this mesh;
  void invalidateMesh(const RegularMesh *mesh);
  void invalidateMesh(const TriangleMesh *mesh);
  void invalidateLegend(const ColorModel *cmod);


 private:
  int indexOf(const OGLDataCacheItem *d)const;
 private:
  QList<OGLDataCacheItem*> plist;

};



#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLDATACACHE_H */

