/*************************************************************************
 * @(#)OGLLegend.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   20 May 2010
 *************************************************************************
 */

#include "OGLView.hh"
#include "OGLLegend.hh"
#include "ColorModel.hh"
#include "MyDebug.hh"

bool OGLLegend::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheLegendItem::OGLDataCacheLegendItem(double width,double height
					       ,OGLLegend::OGLLegendType type
					       ,const ColorModel *pcm)
  :OGLDataCacheItem("OGLDataCacheLegendItem")
  ,ptype(type),pwidth(width),pheight(height),pcolormodel(pcm)
{
  NSF("OGLDataCacheLegendItem::OGLDataCacheLegendItem specified construtor");
  //myDebug() << nsubfunc << this;

  
  if(
     ptype==OGLLegend::Horizontal||
     ptype==OGLLegend::Vertical
     ){
    String s=Sprintf("OGLDataCacheLegendItem:type(%d)width(%30.6f)"
		"height(%30.6f)"
		,ptype,pwidth,pheight
		)();
    if(pid) delete [] pid;
    pid=new char [s.length()+1];
    strcpy(pid,s());
  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
  //myDebug() << nsubfunc << printSpecQ();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheLegendItem::printSpecQ()const
{
  NSF("OGLDataCacheLegendItem::printSpecQ");
  QString s;
  if(ptype==OGLLegend::Horizontal){
    
    s.sprintf("OGLDataCacheLegendItem:   type=Horizontal\n"
	      "                          width=%g\n"
	      "                          height=%g\n"
	      ,pwidth,pheight);
    s+=OGLDataCacheItem::printSpecQ();
  }else if(ptype==OGLLegend::Vertical){
    
    s.sprintf("OGLDataCacheLegendItem:   type=Vertical\n"
	      "                          width=%g\n"
	      "                          height=%g\n"
	      ,pwidth,pheight);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheLegendItem::construct(bool transparent)
{
  NSF("OGLDataCacheLegendItem::construct");
  //myDebug() << nsubfunc <<"TODO";


  int ncolor=250;
  
  pverticessize=2*(ncolor+1);
  pvertices=new GLdouble[pverticessize*3];
  pcolors=new GLfloat[pverticessize*4];
  int nvert=0;
  pndefs=1;
  pdefs=new struct VertexArrayDef[pndefs];

  QColor c;
  for(int i=0;i<=ncolor;i++){
    double h=(i)/1./ncolor;
    c=pcolormodel->getColorFromIndex(h);
    //myDebug()<<i<<h<<c;
    double x=i*2*pwidth/(ncolor);
    double y=0;
    pvertices[nvert*3+0]=x;
    pvertices[nvert*3+1]=y+pheight*2;
    pvertices[nvert*3+2]=0;
    pcolors[nvert*4+0]=c.red()/255.;
    pcolors[nvert*4+1]=c.green()/255.;
    pcolors[nvert*4+2]=c.blue()/255.;
    pcolors[nvert*4+3]=1.;
    nvert++;
    pvertices[nvert*3+0]=x;
    pvertices[nvert*3+1]=y;
    pvertices[nvert*3+2]=0;
    pcolors[nvert*4+0]=c.red()/255.;
    pcolors[nvert*4+1]=c.green()/255.;
    pcolors[nvert*4+2]=c.blue()/255.;
    pcolors[nvert*4+3]=1;
    nvert++;
  }

  pdefs[0].mode=GL_QUAD_STRIP;
  pdefs[0].count=2*(ncolor+1);
  pdefs[0].start=0;
  pdefs[0].useindex=false;
  
  if(OGLLegend::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  if(pdispListName){
   
    OGLIssueVertexArray(pdispListName
			,pvertices,0,pcolors,pindices,pndefs,pdefs);
    
    
    
    cleanInternal();

    
  }


  ogl.reportGLError(nsubfunc);

  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLLegend::OGLLegend(double width,double height,double ratio
		     ,double x0,double y0
		     ,OGLLegendType type,const ColorModel *pcm)
  :OGLObject("OGLLegend")
  ,ptype(type),pwidth(width),pheight(height),pratio(ratio),px0(x0),py0(y0)
  ,cachedData(0),pcolormodel(pcm)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLLegend::~OGLLegend()
{
  NSF("OGLLegend::~OGLLegend");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLLegend::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLLegend::construct(OGLView *oglview)
{
  NSF("OGLLegend::construct");
  //myDebug() << nsubfunc << pmesh;


  OGLDataCacheLegendItem *data
    =new OGLDataCacheLegendItem(pwidth,pheight,ptype,pcolormodel);
  
  if(data!=(cachedData=oglview->ogldatacache->append(data)))
    delete data;
 
        

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLLegend::paint(OGLView *
		      ,const OGLMagnifier &mag,bool reduced
		      ,bool// badTransparency
		      )const
{
  NSF("OGLLegend::paint");
  Q_UNUSED(reduced);
  //myDebug() << nsubfunc;

//   OGLDataCacheLegendItem *data
//     =new OGLDataCacheLegendItem(pwidth,pheight,ptype);


  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT,GL_DIFFUSE);

  pmaterial.issue();

  glDisable(GL_LIGHTING);


  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

  // we have relative to screen placement not 3D
  double s=mag.factor;
  glScaled(s,s,1);
  glTranslated(1-1/s-mag.ix*2/s,1-1/s-mag.iy*2/s,0);

  //   double ratio=1;
  //   if(pratio>1.0e-4){
  //ratio=pratio*pwidth*oglview->width()/(pheight*oglview->height());
  //}
  glTranslated(2*px0-1,2*py0-1,0);
  //  glScaled(1,ratio,1);

  glDisable(GL_DEPTH_TEST);

  if(cachedData)
    cachedData->paint(ptransparent);

  glEnable(GL_DEPTH_TEST);


  glMatrixMode(GL_PROJECTION);
  glPopMatrix();  
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();  

  glEnable(GL_LIGHTING);

  glDisable(GL_COLOR_MATERIAL);



  //delete data;

  return;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLLegend::NFPlane(const Vec3 &n,double &near,double &far)
{
  Q_UNUSED(far);
  Q_UNUSED(near);
  Q_UNUSED(n);
  //myDebug() << "OGLLegend::NFPlane";

  return false;
}
/*EOF*/ 
