/* @(#)OGLLight.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLLIGHT_H
#define _OGLLIGHT_H 1

#include <QGLWidget>
#include "OGL.hh"
#include "Light.hh"

namespace XFPLO{
  
  /* ***********************************************************************
   *
   * ***********************************************************************
   */
  class OGLLights{
  public:
    
    OGLLights(){}
    
    
    
    
    // ----------------------------------------------------------------
    static void issueGlobalAmbient(const QRgb &c){
      GLfloat rgb[4]; GLfloat4FromQRgb(rgb,c);
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, rgb);
    }
    
    // ----------------------------------------------------------------
    static void issueLights(const Lights *lights){
      glEnable(GL_LIGHTING);
      glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
      //glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
      
      for(int i=0;i<8;i++){
	GLenum gll=getGLlight(i);
	glDisable(gll);
      }
      
      for(int i=-1;++i<lights->size();){
	const Lights::Light *li=lights->at(i);
	if(!li->active()) continue;
	GLenum gll=getGLlight(i);
	glEnable(gll);
	GLfloat rgb[4];
	glLightfv(gll,GL_DIFFUSE,GLfloat4FromQRgb(rgb,li->diffuse()));
	glLightfv(gll,GL_AMBIENT,GLfloat4FromQRgb(rgb,li->ambient()));
	glLightfv(gll,GL_SPECULAR,GLfloat4FromQRgb(rgb,li->specular()));
	if(li->type()==Lights::Positional){
	  glLightf(gll,GL_SPOT_EXPONENT,0.);
	  glLightf(gll,GL_SPOT_CUTOFF,180.);
	  glLightf(gll,GL_CONSTANT_ATTENUATION,li->constantAttenuation());
	  glLightf(gll,GL_LINEAR_ATTENUATION,li->linearAttenuation());
	  glLightf(gll,GL_QUADRATIC_ATTENUATION,li->quadraticAttenuation());
	}else if(li->type()==Lights::Spot){
	  glLightf(gll,GL_SPOT_CUTOFF,li->spotCutoff());
	  glLightf(gll,GL_SPOT_EXPONENT,li->spotExponent());
	  glLightf(gll,GL_CONSTANT_ATTENUATION,li->constantAttenuation());
	  glLightf(gll,GL_LINEAR_ATTENUATION,li->linearAttenuation());
	  glLightf(gll,GL_QUADRATIC_ATTENUATION,li->quadraticAttenuation());
	}else{
	  glLightf(gll,GL_SPOT_EXPONENT,0.);
	  glLightf(gll,GL_SPOT_CUTOFF,180.);
	  glLightf(gll,GL_CONSTANT_ATTENUATION,1.);
	  glLightf(gll,GL_LINEAR_ATTENUATION,0.);
	  glLightf(gll,GL_QUADRATIC_ATTENUATION,0.);
	}
      }
    }
    
    // ----------------------------------------------------------------
    static void placeCameraLights(const Lights *lights,double posScale,
				  const Vec3 &focalpoint){
      for(int i=-1;++i<lights->size();){
	const Lights::Light *li=lights->at(i);
	if(!li->active()) continue;
	GLenum gll=getGLlight(i);
	
	Vec3 pos=li->realPosition()*posScale;
	if(li->type()!=Lights::Directional){
	  pos+=focalpoint;
	}
	// myDebug()<<"posScale="<<posScale;
	// myDebug()<<"focalpoint="<<focalpoint;
	// myDebug()<<"position="<<pos;
	GLfloat lightPosition[4];
	lightPosition[0]=pos[0];
	lightPosition[1]=pos[1];
	lightPosition[2]=pos[2];
	if(li->type()==Lights::Directional){
	  lightPosition[3]=0.;
	}else{
	  lightPosition[3]=1.;
	}
	// myDebug()<<"lightPosition="<<lightPosition[0]<<lightPosition[1]<<
	//   lightPosition[2]<<lightPosition[3];
	

	glLightfv(gll, GL_POSITION, lightPosition);

	if(li->type()==Lights::Spot){
	  const double *d=li->spotDirection().constDataRef();
	  lightPosition[0]=((GLfloat)(d[0]*posScale));
	  lightPosition[1]=((GLfloat)(d[1]*posScale));
	  lightPosition[2]=((GLfloat)(d[2]*posScale));
	  
	  glLightfv(gll, GL_SPOT_DIRECTION, lightPosition);
	}
	
      }
      
    }
    // ----------------------------------------------------------------
    
  private:
    
    static GLenum getGLlight(int i){
      if(i>=8){
	myDebug() << "light index too large\n";
	exit(1);
      }
      switch(i)
	{
	case 0:
	  return GL_LIGHT0;
	case 1:
	  return GL_LIGHT1;
	case 2:
	  return GL_LIGHT2;
	case 3:
	  return GL_LIGHT3;
	case 4:
	  return GL_LIGHT4;
	case 5:
	  return GL_LIGHT5;
	case 6:
	  return GL_LIGHT6;
	default:
	  return GL_LIGHT7;
	}
    }
    
    
  };
  
}  

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLLIGHT_H */

