/*************************************************************************
 * @(#)OGLLightHelper.cpp
 * author: magru@unas
 * date:   17 Jul 2013
 *************************************************************************
 */

#include "OGLLightHelper.hh"
#include "View.hh"

OGLLightHelper::OGLLightHelper(const Lights::Light *light,const View *view)
  :OGLObject("OGLLightHelper"),pview(view),plight(*light),pvertices(0),pdefs(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLLightHelper::~OGLLightHelper()
{
  if(pdefs) delete pdefs;
  if(pvertices) delete pvertices;
  pndefs=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void OGLLightHelper::construct()
{
  long Nphi=6;

  pverticessize=2; 
  if(plight.type()==Lights::Spot){
    pverticessize+=(Nphi+2)*3;
  }else{
    pverticessize+=6*3;
  }
  pvertices=new GLdouble[3*pverticessize];
  pndefs=2;
  pdefs=new struct VertexArrayDef[pndefs];
  long il=0;
  pdefs[il].mode=GL_LINES;
  pdefs[il].count=4;
  pdefs[il].start=0;
  pdefs[il].useindex=false;

  double scale=pview->radius();
  Vec3 focus_c=pview->matrix()*pview->focus();
  Vec3 v0(0,0,0);
  Vec3 vp;

  Vec3 nz;

  // we are in camera coordinates (so are light positions.directions)
  v0=focus_c;
  vp=plight.realPosition()*scale+focus_c;


  nz=v0-vp;
  
  

  if(plight.type()==Lights::Spot){
    Vec3 nn(plight.spotDirection().constDataRef());
    double lam=MyMath::max<double>(scale,nn*nz/(nn*nn));
    v0=vp+nn*lam;
    nz=v0-vp;
  }


  Vec3 vf=pview->matrix().transposed()*v0;
  Vec3 vl=pview->matrix().transposed()*vp;
  


  // direcition indicator
  long j=0;
  pvertices[3*j+0]=vf[0];
  pvertices[3*j+1]=vf[1];
  pvertices[3*j+2]=vf[2];
  j++;

  pvertices[3*j+0]=vl[0];
  pvertices[3*j+1]=vl[1];
  pvertices[3*j+2]=vl[2];
  j++;


  if(plight.type()==Lights::Spot){
    // cone
    double r=0.1;
    double alpha=plight.spotCutoff()/180.*Pi;
    double ca=cos(alpha);
    double sa=sin(alpha);
    
    il=1;
    pdefs[il].mode=GL_TRIANGLE_FAN;
    pdefs[il].count=Nphi+2;
    pdefs[il].start=j;
    pdefs[il].useindex=false;
    
    Matrix3x3 U;
    if(norm(nz)<1.0e-10){
      U=Matrix3x3::unitMatrix();
    }else{
      U=Vec3::eulerAngles(nz).eulerRMatrix();
    }
    
    Vec3 v;
    v=vl;
    pvertices[3*j+0]=v[0];
    pvertices[3*j+1]=v[1];
    pvertices[3*j+2]=v[2];
    j++;
    
    
    for(int i=-1;++i<Nphi+1;){
      Vec3 v(sa*cos(-twopi/(Nphi)*(i-1)),sa*sin(-twopi/(Nphi)*(i-1)),ca);
      v*=r*scale;
      v=U*v+vp;
      v=pview->matrix().transposed()*v;
      pvertices[3*j+0]=v[0];
      pvertices[3*j+1]=v[1];
      pvertices[3*j+2]=v[2];
      j++;
    }
  }else{
    double r=0.03;
    il=1;
    pdefs[il].mode=GL_LINES;
    pdefs[il].count=6;
    pdefs[il].start=j;
    pdefs[il].useindex=false;

    for(int i=-1;++i<3;){
      Vec3 v(cos(-twopi/6.*(i-1)),sin(-twopi/6.*(i-1)),0.);
      v=pview->matrix().transposed()*(v*scale*r);
      pvertices[3*j+0]=vl[0]+v[0];
      pvertices[3*j+1]=vl[1]+v[1];
      pvertices[3*j+2]=vl[2]+v[2];
      j++;
      pvertices[3*j+0]=vl[0]-v[0];
      pvertices[3*j+1]=vl[1]-v[1];
      pvertices[3*j+2]=vl[2]-v[2];
      j++;
    }    
  }    
    
  

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLLightHelper::paint(OGLView *,const OGLMagnifier &/*mag*/,bool /*reduced*/
			   ,bool //badTransparency
			   )const
{
  //  myDebug()<<"here";


  const_cast<OGLLightHelper*>(this)->construct();
  
  


  glPushMatrix();
  // glMatrixMode(GL_MODELVIEW);
  // glLoadIdentity();

  // glMultMatrixd(pview->matrix()());
  // glTranslated(-pview->origin_cam_w()[0],-pview->origin_cam_w()[1],
  // 	       -pview->origin_cam_w()[2]);

  GLfloat lw;
  glGetFloatv(GL_LINE_WIDTH,&lw);
  glLineWidth(2);
  GLfloat rgb[4]; 
  GLfloat4FromQRgb(rgb,pmaterial.diffuse()&0xffffff);
  glColor4fv(rgb);
  //glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,GLfloat4FromQRgb(rgb,0xffffffff));
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  //glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  glDisable(GL_LIGHTING);

  OGLIssueVertexArray(0,pvertices,0,0,0,pndefs,pdefs);
  
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  glEnable(GL_LIGHTING);
  glLineWidth(lw);
  glPopMatrix();  



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLLightHelper::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "NFPlane " << id();




  double scale=pview->radius();
  Vec3 focus_c=pview->matrix()*pview->focus();

  Vec3 v0=focus_c;
  Vec3 vp=plight.realPosition()*scale+focus_c;
  Vec3 nz=v0-vp;
  
  

  if(plight.type()==Lights::Spot){
    Vec3 nn(plight.spotDirection().constDataRef());
    double lam=MyMath::max<double>(scale,nn*nz/(nn*nn));
    v0=vp+nn*lam;
  }


  Vec3 vf=pview->matrix().transposed()*v0;
  Vec3 vl=pview->matrix().transposed()*vp;


  double d;
  d=vf*n;
  near=d;
  far=d;

  d=vl*n;
  near=MyMath::max<double>(near,d);
  far=MyMath::min<double>(far,d);

  near=-near;
  far=-far;


  return true;
}



/*EOF*/ 
