/*************************************************************************
 * @(#)OGLLine.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   15 Apr 2011
 *************************************************************************
 */

#include "OGLLine.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLLine::OGLLine(const ValueArray<ValueArray<Vec3> > &lines,double linewidth)
  :OGLObject("OGLLine"),pvertices(0),pdefs(0),plinewidth(linewidth)
{
  pverticessize=0; 
  for(int il=0;il<lines.size();il++){
    pverticessize+=lines.at(il).size();
  }
  pvertices=new GLdouble[3*pverticessize];

  pndefs=lines.size();
  pdefs=new struct VertexArrayDef[pndefs];
  int j=0;
  for(int il=0;il<lines.size();il++){
    const ValueArray<Vec3> &line=lines.at(il);
    pdefs[il].mode=GL_LINE_STRIP;
    pdefs[il].count=line.size();
    pdefs[il].start=j;
    pdefs[il].useindex=false;
    for(int i=0;i<pdefs[il].count;i++){
      pvertices[3*j+0]=line.at(i)[0];
      pvertices[3*j+1]=line.at(i)[1];
      pvertices[3*j+2]=line.at(i)[2];
      j++;
    }
  }  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLLine::~OGLLine()
{
  NSF("OGLLine::~OGLLine");
  //myDebug() << nsubfunc<< "I am dying";
  if(pvertices) delete pvertices; pvertices=0;
  if(pdefs) delete pdefs; pdefs=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLLine::paint(OGLView *,const OGLMagnifier &mag,bool reduced
		    ,bool //badTransparency
		    )const
{
  Q_UNUSED(reduced);
  NSF("OGLLine::paint");
  //myDebug() << nsubfunc;



  //  glTranslated(pposition[0],pposition[1],pposition[2]);

//   myDebug() << ptransparent;
//   myDebug() << badTransparency;
  //pmaterial.issue(ptransparent && ! badTransparency);
  //if(cachedData)cachedData->paint(ptransparent && ! badTransparency);

  //pmaterial.issue(false);
  //myDebug()<<pmaterial;
  //  if(cachedData)cachedData->paint(ptransparent);


  GLfloat rgb[4]; 
  GLfloat4FromQRgb(rgb,pmaterial.diffuse()&0xffffff);
  glPushMatrix();
  GLfloat lw;
  glGetFloatv(GL_LINE_WIDTH,&lw);
  glLineWidth(plinewidth*mag.factor);
  glColor4fv(rgb);
//   glLineStipple(1, 0xfff0);
//   glEnable(GL_LINE_STIPPLE);
  glDisable(GL_LIGHTING);
  GLuint pdispListName=0;
  OGLIssueVertexArray(pdispListName
		      ,pvertices
		      ,0
		      ,0
		      ,0
		      ,pndefs
		      ,pdefs 
		      );
  glEnable(GL_LIGHTING);
//   glDisable(GL_LINE_STIPPLE);
  glLineWidth(lw);
  glPopMatrix();  
  

  
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLLine::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "NFPlane " << id();


  double a=pposition*n;
  for(int i=0;i<pverticessize;i++){
    double d=Vec3(pvertices+3*i)*n+a;
    if(i==0){
      near=d;
      far=d;
    }else{
      near=MyMath::max<double>(near,d);
      far=MyMath::min<double>(far,d);
    }
  }
  near=-near;
  far=-far;


  return true;
}



/*EOF*/ 
