/* @(#)OGLMaterial.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLMATERIAL_H
#define _OGLMATERIAL_H 1

#include <QGLWidget>
#include "FString.hh"
#include "MyMath.hh"
#include "MyDebug.hh"
#include "OGL.hh"

class OGLMaterial{
  
  

 public:
  OGLMaterial()
    { 
      pface=GL_FRONT;
      setAmbient(0xff000000);
      setDiffuse(0xffcc0000);
      setBase(0xffcc0000);
      setSpecular(0xffffffff);
      setEmission(0xff000000);
      setShininess(50.0);
      //setOpacity(1.0);
    };

  OGLMaterial(const OGLMaterial &other){
    pface=other.pface;
    setAmbient(other.ambient());
    setDiffuse(other.diffuse());
    setBase(other.base());
    setSpecular(other.specular());
    setEmission(other.emission());
    setShininess(other.shininess());
    //setOpacity(other.opacity());
  }


  const OGLMaterial &operator=(const OGLMaterial &other){
    pface=other.pface;
    setAmbient(other.ambient());
    setDiffuse(other.diffuse());
    setBase(other.base());
    setSpecular(other.specular());
    setEmission(other.emission());
    setShininess(other.shininess());
    //setOpacity(other.opacity());
    return *this;
  };
  
  void issue(bool applyOpacity=false) const{
    if(!applyOpacity){
      GLfloat rgb[4]; 
      memcpy(rgb,pdiffuse,sizeof(GLfloat)*3);rgb[3]=1;
      glMaterialfv(pface,GL_DIFFUSE,rgb);
    }else{
      glMaterialfv(pface,GL_DIFFUSE,pdiffuse);
    }
    glMaterialfv(pface,GL_AMBIENT,pambient);
    glMaterialfv(pface,GL_EMISSION,pemission);
    glMaterialf(pface,GL_SHININESS,pshininess);
    glMaterialfv(pface,GL_SPECULAR,pspecular);
    
  };

  inline void setAmbient(QRgb rgb){ 
    GLfloat4FromQRgb(pambient,(rgb&0xffffff) | 0xff000000); }
  inline void setAmbient(const GLfloat *rgba){ 
    memcpy(pambient,rgba,sizeof(GLfloat)*3);  
  }
  inline void setDiffuse(QRgb rgb){ GLfloat4FromQRgb(pdiffuse,rgb);  }
  inline void setDiffuse(const GLfloat *rgba,bool setalpha=true){ 
    if(setalpha)
      memcpy(pdiffuse,rgba,sizeof(GLfloat)*4);  
    else{
      memcpy(pdiffuse,rgba,sizeof(GLfloat)*3);  
    }
  }
  inline void setBase(QRgb rgb){ GLfloat4FromQRgb(pbase,rgb&0xffffff); }
  inline void setSpecular(QRgb rgb){ 
    GLfloat4FromQRgb(pspecular,(rgb&0xffffff)|0xff000000); }
  inline void setEmission(QRgb rgb){ 
    GLfloat4FromQRgb(pemission,(rgb&0xffffff)|0xff000000); }
  inline void setShininess(double shine){pshininess=shine;};
  inline void setFace(GLenum face){pface=face;};
  //inline void setOpacity(double opacity){popacity=opacity;};


  inline GLenum face() const{ return pface; };
  inline QRgb ambient() const{ return QRgbFromGLfloat4(pambient); };
  inline QRgb diffuse() const{ return QRgbFromGLfloat4(pdiffuse); };
  inline QRgb base() const{ return QRgbFromGLfloat4(pbase); };
  inline QRgb specular() const{ return QRgbFromGLfloat4(pspecular); };
  inline QRgb emission() const{ return QRgbFromGLfloat4(pemission); };
  inline double shininess() const{ return pshininess; };
  //inline double opacity() const{ return popacity; };

  String printS()const;

 private:
  GLenum  pface;
  GLfloat pambient[4]; 
  GLfloat pdiffuse[4]; 
  GLfloat pbase[4]; 
  GLfloat pspecular[4]; 
  GLfloat pemission[4]; 
  GLfloat pshininess;
  //double  popacity;
};

MyDebug operator<<(MyDebug dbg, const OGLMaterial &m);

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLMATERIAL_H */

