/*************************************************************************
 * @(#)OGLMeshCells.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 May 2010
 *************************************************************************
 */

#include "OGLMeshCells.hh"
#include "RegularMesh.hh"
#include "OGL.hh"
#include "OGLView.hh"
#include "MyDebug.hh"

bool OGLMeshCells::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheMeshCellsItem::
OGLDataCacheMeshCellsItem(OGLMeshCells::OGLMeshCellsType type
			   ,const RegularMesh *mesh)
  :OGLDataCacheItem("OGLDataCacheMeshCellsItem")
  ,ptype(type),pmesh(mesh)
{
  NSF("OGLDataCacheMeshCellsItem::OGLDataCacheMeshCellsItem");
  
  if(
     ptype==OGLMeshCells::Microcells||
     ptype==OGLMeshCells::Simplices
     ){
    String s=Sprintf("OGLDataCacheMeshCellsItem:type(%d)mesh(%p)"
		,ptype,pmesh
		);
    if(pid) delete [] pid;
    pid=new char [s.length()+1];
    strcpy(pid,s());

  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheMeshCellsItem::printSpecQ()const
{
  NSF("OGLDataCacheMeshCellsItem::printSpecQ");
  QString s;
  if( ptype==OGLMeshCells::Microcells ){
    
    s.sprintf("OGLDataCacheMeshCellsItem:   type=Microcells\n"
	      "                              mesh=%p\n",pmesh);
    s+=OGLDataCacheItem::printSpecQ();
  }else if( ptype==OGLMeshCells::Simplices ){
    
    s.sprintf("OGLDataCacheMeshCellsItem:   type=Simplices\n"
	      "                              mesh=%p\n",pmesh);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheMeshCellsItem::construct(bool transparent)
{
  NSF("OGLDataCacheMeshCellsItem::construct");
  //myDebug() << nsubfunc;


  pverticessize=pmesh->numPoints();
  pvertices=new GLdouble[pverticessize*3];
  
  long nvert=0;
  for(int ip=0;ip<pverticessize;ip++){
    const Vec3 &v=pmesh->point(ip);
    
    pvertices[nvert*3+0]=v[0];
    pvertices[nvert*3+1]=v[1];
    pvertices[nvert*3+2]=v[2];
    nvert++;
  }
  
  pndefs=1;
  pdefs=new struct VertexArrayDef[pndefs];

  long nind=0;
  
  if(ptype==OGLMeshCells::Microcells){
    
    
    pindicessize=pmesh->NMicroCells()*12*2;
    pindices=new GLuint[pindicessize];
    
    for(int ic=0;ic<pmesh->NMicroCells();ic++){
      const long *cell=pmesh->microCell(ic);
      
      pindices[nind++]=cell[0];
      pindices[nind++]=cell[1];
      
      pindices[nind++]=cell[0];
      pindices[nind++]=cell[2];
      
      pindices[nind++]=cell[3];
      pindices[nind++]=cell[1];
      
      pindices[nind++]=cell[3];
      pindices[nind++]=cell[2];
      
      pindices[nind++]=cell[4];
      pindices[nind++]=cell[5];
      
      pindices[nind++]=cell[4];
      pindices[nind++]=cell[6];
      
      pindices[nind++]=cell[7];
      pindices[nind++]=cell[5];
      
      pindices[nind++]=cell[7];
      pindices[nind++]=cell[6];
      
      pindices[nind++]=cell[0];
      pindices[nind++]=cell[4];
      
      pindices[nind++]=cell[1];
      pindices[nind++]=cell[5];
      
      pindices[nind++]=cell[2];
      pindices[nind++]=cell[6];
      
      pindices[nind++]=cell[3];
      pindices[nind++]=cell[7];
      
      
      //if(ic>=0) break;
    }
  }else{
    
    
    pindicessize=pmesh->NTetrahedra()*6*2;
    pindices=new GLuint[pindicessize];
    
    
    nind=0;
    for(int it=0;it<pmesh->NTetrahedra();it++){
      const long *tet=pmesh->tetrahedron(it);
      
      pindices[nind++]=tet[0];
      pindices[nind++]=tet[1];
      
      pindices[nind++]=tet[0];
      pindices[nind++]=tet[2];
      
      pindices[nind++]=tet[1];
      pindices[nind++]=tet[2];
      
      pindices[nind++]=tet[3];
      pindices[nind++]=tet[0];
      
      pindices[nind++]=tet[3];
      pindices[nind++]=tet[1];
      
      pindices[nind++]=tet[3];
      pindices[nind++]=tet[2];
      
      
      
    }
  }

  //myDebug() << pindicessize << nind;
  
  pdefs[0].mode=GL_LINES;
  pdefs[0].count=nind;
  pdefs[0].start=0;
  pdefs[0].useindex=true;
  
  if(OGLMeshCells::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  
  if(pdispListName){
    
    OGLIssueVertexArray(pdispListName
 			,pvertices,pnormals,0,pindices,pndefs,pdefs);
    
    
    
    cleanInternal();
    
    
  }
  
  
  ogl.reportGLError(nsubfunc);
  
  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLMeshCells::OGLMeshCells(OGLMeshCellsType type,const RegularMesh *mesh)
  :OGLObject("OGLMeshCells")
  ,ptype(type),pmesh(mesh)
  ,cachedData(0)
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLMeshCells::~OGLMeshCells()
{
  NSF("OGLMeshCells::~OGLMeshCells");
  //myDebug() << nsubfunc<< "I am dying";
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshCells::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshCells::construct(OGLView *oglview)
{
  NSF("OGLMeshCells::construct");
  //myDebug() << nsubfunc;


  
  if(!pmesh->microCellsAreReady())return;
  
  OGLDataCacheMeshCellsItem *data
    =new OGLDataCacheMeshCellsItem(ptype,pmesh);
  
  if(data!=(cachedData=oglview->ogldatacache->append(data)))
    delete data;
      

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshCells::paint(OGLView *
			 ,const OGLMagnifier &,bool reduced
			 ,bool// badTransparency
			 )const
{
  NSF("OGLMeshCells::paint");
  Q_UNUSED(reduced);
  //myDebug() << nsubfunc;

  //if(!pmesh->microCellsAreReady())return;

  

  
  
  //  pmaterial.issue();
  //glPointSize(4.);
  glDisable(GL_LIGHTING);
  GLfloat rgb[4]; 
  glColor4fv(GLfloat4FromQRgb(rgb,pmaterial.diffuse()));
  if(cachedData)cachedData->paint(ptransparent);
  glEnable(GL_LIGHTING);
  
  
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLMeshCells::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLMeshCells::NFPlane";

  const Matrix3x3 &cell=pmesh->cell();

  //myDebug() << cell.printS()();

   double p0=pmesh->origin()*n;

   near=-p0;
   far=-p0;
   
   double c0=cell(0)*n;
   double c1=cell(1)*n;
   double c2=cell(2)*n;
   

   double d=c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));


   d=c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1+c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1+c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c2+c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c0+c1+c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

  
   //myDebug() << near<<far;  

   return true;
}



/*EOF*/ 
