/*************************************************************************
 * @(#)OGLMeshPoints.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 May 2010
 *************************************************************************
 */

#include "OGLMeshPoints.hh"
#include "RegularMesh.hh"
#include "OGLView.hh"
#include "OGL.hh"
#include "MyDebug.hh"

bool OGLMeshPoints::pusedisplaylists=true;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheMeshPointsItem::
OGLDataCacheMeshPointsItem(OGLMeshPoints::OGLMeshPointsType type
			   ,const RegularMesh *mesh)
  :OGLDataCacheItem("OGLDataCacheMeshPointsItem")
  ,ptype(type),pmesh(mesh)
{
  NSF("OGLDataCacheMeshPointsItem::OGLDataCacheMeshPointsItem");
  
  if(
     ptype==OGLMeshPoints::Full||
     ptype==OGLMeshPoints::Irreducible
     ){
    String s=Sprintf("OGLDataCacheMeshPointsItem:type(%d)mesh(%p)"
		,ptype,pmesh
		);
    if(pid) delete [] pid;
    pid=new char [s.length()+1];
    strcpy(pid,s());

  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheMeshPointsItem::printSpecQ()const
{
  NSF("OGLDataCacheMeshPointsItem::printSpecQ");
  QString s;
  if(ptype==OGLMeshPoints::Full){
    
    s.sprintf("OGLDataCacheMeshPointsItem:   type=Full\n"
	      "                              mesh=%p\n",pmesh);
    s+=OGLDataCacheItem::printSpecQ();
  }else if(ptype==OGLMeshPoints::Irreducible){
    
    s.sprintf("OGLDataCacheMeshPointsItem:   type=Irreducible\n"
	      "                              mesh=%p\n",pmesh);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheMeshPointsItem::construct(bool transparent)
{
  NSF("OGLDataCacheMeshPointsItem::construct");
  //myDebug() << nsubfunc;


    pndefs=1;
    pdefs=new struct VertexArrayDef[pndefs];

  if(ptype==OGLMeshPoints::Full){
    pverticessize=pmesh->numPoints();
    pvertices=new GLdouble[pverticessize*3];
    
    
    long nvert=0;
    for(int ip=0;ip<pmesh->numPoints();ip++){
      const Vec3 v=pmesh->point(ip);
      
      pvertices[nvert*3+0]=v[0];
      pvertices[nvert*3+1]=v[1];
      pvertices[nvert*3+2]=v[2];
      nvert++;
    }
  }else if(ptype==OGLMeshPoints::Irreducible){
    pverticessize=pmesh->numIrredRotPoints();
    pvertices=new GLdouble[pverticessize*3];
    
    
    
    long nvert=0;
    for(int ipi=0;ipi<pmesh->numIrredRotPoints();ipi++){
      const Vec3 v=pmesh->irredRotPoint(ipi);
      
      pvertices[nvert*3+0]=v[0];
      pvertices[nvert*3+1]=v[1];
      pvertices[nvert*3+2]=v[2];
      nvert++;
    }
  }

  pdefs[0].mode=GL_POINTS;
  pdefs[0].count=pverticessize;
  pdefs[0].start=0;
  pdefs[0].useindex=false;

  if(OGLMeshPoints::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }

  if(pdispListName){
   
    OGLIssueVertexArray(pdispListName
			,pvertices,pnormals,0,pindices,pndefs,pdefs);
 
    
    
    cleanInternal();

    
  }


  ogl.reportGLError(nsubfunc);

  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLMeshPoints::OGLMeshPoints(OGLMeshPointsType type,const RegularMesh *mesh)
  :OGLObject("OGLMeshPoints")
  ,ptype(type),pmesh(mesh)
  ,cachedData(0)
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLMeshPoints::~OGLMeshPoints()
{
  NSF("OGLMeshPoints::~OGLMeshPoints");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshPoints::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshPoints::construct(OGLView *oglview)
{
  NSF("OGLMeshPoints::construct");
  //myDebug() << nsubfunc;

  
  if(!pmesh->meshIsReady())return;
  
  OGLDataCacheMeshPointsItem *data
    =new OGLDataCacheMeshPointsItem(ptype,pmesh);
  
  if(data!=(cachedData=oglview->ogldatacache->append(data)))
    delete data;
 
        

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLMeshPoints::paint(OGLView *
			  ,const OGLMagnifier &,bool reduced
			  ,bool// badTransparency
			  )const
{
  NSF("OGLMeshPoints::paint");
  Q_UNUSED(reduced);
  //myDebug() << nsubfunc;


  
  
  //  pmaterial.issue();
  if(ptype==Full)
    glPointSize(4.);
  else
    glPointSize(6.);
  glDisable(GL_LIGHTING);
  GLfloat rgb[4]; 
  glColor4fv(GLfloat4FromQRgb(rgb,pmaterial.diffuse()));
  if(cachedData)
    cachedData->paint(ptransparent);
  glEnable(GL_LIGHTING);
  
  

  
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLMeshPoints::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLMeshPoints::NFPlane";

  const Matrix3x3 &cell=pmesh->cell();

  //myDebug() << cell.printS()();

   double p0=pmesh->origin()*n;

   near=-p0;
   far=-p0;
   
   double c0=cell(0)*n;
   double c1=cell(1)*n;
   double c2=cell(2)*n;
   

   double d=c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));


   d=c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1+c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c1+c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c2+c0;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

   d=c0+c1+c2;
   near=MyMath::min<double>(near,-(p0+d));
   far=MyMath::max<double>(far,-(p0+d));

  
   //myDebug() << near<<far;  

   return true;
}


/*EOF*/ 
