/*************************************************************************
 * @(#)OGLObject.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2010
 *************************************************************************
 */

#include <QGLWidget>
#include <QImage>
#include "OGLObject.hh"
#include "OGLTransparentCollector.hh"
#include "OGLUserData.hh"
#include "OGLView.hh"
#include "MyMath.hh"
#include "View.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLObject::~OGLObject()
{
  if(puserdata) delete puserdata; puserdata=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLObjects::OGLObjects(OGLView *ov)
  :poglview(ov)
{
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLObjects::~OGLObjects()
{
  NSF("OGLObjects::~OGLObjects");
  //myDebug() << nsubfunc;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::append(OGLObject *o)
{
  //NSF("OGLObjects::append");
  pobjects.append(o);
  o->construct(poglview);
//   myDebug() << nsubfunc
// 	   <<pobjects.last()->type()
// 	   <<pobjects.last()->id()
//     ;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::clear(const String &id)
{
  NSF("OGLObjects::clear");
  //myDebug() << nsubfunc << id;
  for(int i=pobjects.size()-1;i>=0;i--){
    bool del=false;
//     if(pobjects.at(i)->type()=="OGLTriangleMesh"){
//       myDebug() << nsubfunc << i << pobjects.at(i)->id();
//     };
    if((id.length()>0)){
      if(pobjects.at(i)->id().startsWith(id)){
	del=true;
      }
    }else{
      del=true;
    }
    if(del){
      //myDebug() << "clean "<<i << pobjects.at(i)->type() << pobjects.at(i)->id();
      if(pobjects[i]) delete pobjects[i];
      pobjects.removeAt(i);
    }
  }
  //pobjects.clear();
  //myDebug() << nsubfunc << " done"; 

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::collectTransparentObjects(bool drawReduced,
			   const ValueArray<ValueArray<GLdouble> > &seedplanes)
{
  NSF("OGLObjects::collectTransparentObjects");
  // myDebug() << nsubfunc<<"needReConstruct()"<<poglview->ptpCollector->needReConstruct()
  // 	    <<"seedplanes"<<(void*)seedplanes;
  
  if(!poglview->ptpCollector->needReConstruct()) return;
  
  poglview->ptpCollector->reset(seedplanes);

  for(int i=0;i<pobjects.size();i++){
    OGLObject *o=pobjects[i];
    o->protectCachedData();
    if( (o->isVisible()) && 
	(!o->isHidden()) && 
	(!o->isOntop()) && 
	(o->isTransparent())){
      //myDebug() << o->id();
      o->collectTransparent(poglview->ptpCollector,drawReduced);
    }
  }

  poglview->ptpCollector->resetNeedReConstruct();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::paint(const Vec3 &eye,const Vec3 &z
		       ,bool drawReduced,bool badTransparency)const
{
  NSF("OGLObjects::paint");
  //myDebug() << nsubfunc;
  for(int i=0;i<pobjects.size();i++){
    OGLObject *o=pobjects.at(i);
    o->protectCachedData();
    if( 
       (!o) ||
       (!o->isVisible())||
       (o->isHidden())||
       (o->isOntop()) ||
       (o->isTransparent())
       ) continue;
    // here we have a visible non-hidden 3D object 
    // and (reduced-drawing or non-transparent object)

    glPushMatrix();  
    o->paint(poglview,pmag,drawReduced,badTransparency);
    glPopMatrix();  
  }

  
  if(!badTransparency){
    poglview->ptpCollector->paint(eye,z,poglview->view().isPerspective());
  }else{
    // draw transparent objects without primitive sorting
    
    for(int i=0;i<pobjects.size();i++){
      OGLObject *o=pobjects.at(i);
      if( 
	 (!o) ||
	 (!o->isVisible())||
	 (o->isHidden())||
	 (o->isOntop()) ||
	 (!o->isTransparent())
	 ) continue;
      // here we have a visible non-hidden 3D object 
      // and (reduced-drawing or non-transparent object)
      
      glPushMatrix();  
      o->paint(poglview,pmag,drawReduced,badTransparency);
      glPopMatrix();  
    }
  }


  for(int i=0;i<pobjects.size();i++){
    OGLObject *o=pobjects.at(i);
    if(
       (!o) ||
       (!o->isVisible())||
       (o->isHidden())||
       (!o->isOntop()) ||
       (o->isTransparent()) 
       )continue;
    // here we have the non-transparent ontop-objects 
    glPushMatrix();  
    o->paint(poglview,pmag,drawReduced,badTransparency);
    glPopMatrix();  
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::overpaint(QImage *image)const
{
  NSF("OGLObjects::overpaint");
  //myDebug() << nsubfunc;
  for(int i=0;i<pobjects.size();i++){
    if(!pobjects.at(i)) continue;
    if(!pobjects.at(i)->isVisible())continue;
    if(pobjects.at(i)->isHidden())continue;
    if(pobjects.at(i)->isOntop()) continue;
    //myDebug() << "paint "<<pobjects.at(i);
    glPushMatrix();  
    pobjects.at(i)->overpaint(image);
    glPopMatrix();  
  }
  for(int i=0;i<pobjects.size();i++){
    if(!pobjects.at(i)) continue;
    if(!pobjects.at(i)->isVisible())continue;
    if(pobjects.at(i)->isHidden())continue;
    if(!pobjects.at(i)->isOntop()) continue;
    //myDebug() << "paint "<<pobjects.at(i);
    glPushMatrix();  
    pobjects.at(i)->overpaint(image);
    glPopMatrix();  
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
int OGLObjects::indexOf(const String &id,bool exactMatch)
{
  for(int i=0;i<pobjects.size();i++){
    if(exactMatch){
      if(pobjects.at(i)->id()==id) return i;
    }else{
      if(pobjects.at(i)->id().startsWith(id)) return i;
    }
    
  }
  
  return -1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLObjects::setVisible(const String &id,bool v)
{
  bool trans=false;
  for(int i=0;i<pobjects.size();i++){
    if(pobjects.at(i)->id().startsWith(id)) {
      pobjects[i]->setVisible(v);
      trans|=pobjects[i]->isTransparent();
    }
  }
  return trans;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLObject *OGLObjects::operator[](int i)
{
  if(i>=0 && i< pobjects.size())
    return pobjects[i];
  else
    return 0;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLObjects::NFPlane(const Vec3 &n,double &near,double &far)
{
  NSF("OGLObjects::nearPlane");
  //  myDebug() << nsubfunc;
  for(int i=0;i<pobjects.size();i++){
    OGLObject *obj=pobjects.at(i);
    if(obj->isVisible()){
      double dn,df;
      if(obj->NFPlane(n,dn,df)){
	//myDebug() << pobjects.at(i)->id()<<dn<<df;
	if(i==0) {
	  near=dn;
	  far=df;
	}else{
	  near=MyMath::min<double>(near,dn);
	  far =MyMath::max<double>(far ,df);
	} 
      }
    } 
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLObject *OGLObjects::pickObject(const View &view,double x,double y)
{
  
  //myDebug() << "x,y" <<  x<<y;

  int ind=-1;
  double zmax=0.;
  for(int i=0;i<pobjects.size();i++){
    OGLObject *obj=pobjects.at(i);
    if(obj->isVisible()&&obj->pickable()){
      double z;
      if(obj->pick(view,x,y,z)){
	//myDebug() << "picked" << obj->id() << z;
	if(ind<0){
	  zmax=z;
	  ind=i;
	}else{
	  if(z>zmax){
	    zmax=z;
	    ind=i;
	  }
	}
      }
    }
  }
  if(ind>=0){
    return pobjects.at(ind);
  }else
    return 0;
}


/*EOF*/ 
