/* @(#)OGLObject.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLOBJECT_H
#define _OGLOBJECT_H 1

#include "OGL.hh"
#include "Vec3.hh"
#include "ValueArray.hh"


class View;
class QImage;
class OGLView;
class OGLTransparentCollector;
class OGLUserData;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLObject
{
public:
  OGLObject(const String &isa="OGLObject",const String &id="")
    :pid(id),pisa(isa),puserdata(0),pvisible(true),phidden(false)
    ,ptransparent(false),pontop(false),ppickable(false){};
  virtual ~OGLObject();
  

  inline void setID(const String &i){pid=i;};

  inline const String &type()const {return pisa;};
  inline const String &id()const {return pid;};

  inline bool isA(const String &a){return (a==pisa);};

  inline void setVisible(bool v){pvisible=v;};
  inline bool isVisible()const{return pvisible;};

  inline void setHidden(bool v){phidden=v;};
  inline bool isHidden()const{return phidden;};

  inline void setOntop(bool o){pontop=o;};
  inline bool isOntop()const{return pontop;};

  inline void setTransparent(bool transp){ptransparent=transp;};
  inline bool isTransparent()const{return ptransparent;};

  virtual void construct(OGLView *oglview)=0;
  virtual void collectTransparent(OGLTransparentCollector *,bool ){}; //todo pure
  virtual void protectCachedData()=0;
  virtual bool pick(const View &v,double x,double y,double &z)=0;

  virtual bool NFPlane(const Vec3 &n,double &near,double &far)=0;

  virtual void paint(OGLView *oglview,const OGLMagnifier &mag
		     ,bool reduced,bool badTransparency)const=0;

  virtual void shiftInViewCoords(double,double){};

  virtual void overpaint(QImage *)const{};

  //  {Q_UNUSED(reduced);Q_UNUSED(mag);};

  void setUserData(OGLUserData *d){puserdata=d;};
  OGLUserData *userData(){return puserdata;};

  void setPickable(bool p){ppickable=p;};
  bool pickable(){return ppickable;};

private:
  String pid;
  String pisa;
  OGLUserData *puserdata;
protected:
  bool pvisible; // visible means physically present in the scene or not
  bool phidden; // hidden objects are there but invisible
  // they can be picked
  bool ptransparent;
  bool pontop;
  bool ppickable;
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLObjects
{
public:
  OGLObjects(OGLView *ov);
  virtual ~OGLObjects();

  void clear(const String &id="");

  void paint(const Vec3 &eye,const Vec3 &z
	     ,bool reduced,bool badTransparency)const;
  void overpaint(QImage *image)const;
  
  int indexOf(const String &id,bool exactMatch=true);

  virtual void NFPlane(const Vec3 &n,double &near,double &far);

  long size()const{return pobjects.size();};

  bool setVisible(const String &id,bool v); //return if transparent objects

  void setMagnifier(const OGLMagnifier &mag){pmag=mag;};

  void collectTransparentObjects(bool drawReduced,
				 const ValueArray<ValueArray<GLdouble> > &seedplanes);

  OGLObject *operator[](int i);

  void append(OGLObject *o);

  OGLObject *pickObject(const View &v,double x,double y);

protected:
private:
  ValueArray<OGLObject*> pobjects;
  OGLMagnifier pmag;
  OGLView *poglview;
};


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLOBJECT_H */

