/*************************************************************************
 * @(#)OGLPoly.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2010
 *************************************************************************
 */

#include <QGLWidget>
#include "OGLView.hh"
#include "OGLDataCache.hh"
#include "OGLTransparentCollector.hh"
#include "OGLBSPTree.hh"
#include "OGLPoly.hh"
#include "Polyhedron.hh"

using namespace XFPLO;

bool OGLPoly::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCachePolyItem::
OGLDataCachePolyItem(const Polyhedron *poly)
  :OGLDataCacheItem("OGLDataCachePolyItem")
  ,polyhedron(new Polyhedron())
{
  NSF("OGLDataCachePolyItem::OGLDataCachePolyItem");
  
  (*polyhedron)=(*poly);

  pcull=false;

  String s=Sprintf("OGLDataCachePolyItem:faces(%d)\n"
		   ,polyhedron->polyIndices().size() );

  for(int ifa=0;ifa<polyhedron->polyIndices().size();ifa++){
    s+=Sprintf("face[%d]\n",ifa)();
    for(int i=0;i<polyhedron->polyIndices().at(ifa).size();i++){
      int ind=polyhedron->polyIndices().at(ifa).at(i);
      s+=Sprintf("v[%d](%20.9f %20.9f %20.9f)\n",i
		 ,polyhedron->vertices().at(ind)[0]
		 ,polyhedron->vertices().at(ind)[1]
		 ,polyhedron->vertices().at(ind)[2])();
    }
  }
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCachePolyItem::~OGLDataCachePolyItem()
{
  delete polyhedron;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCachePolyItem::printSpecQ()const
{
  NSF("OGLPoly::OGLDataCachePolyItem::printSpecQ");
  //myDebug() << nsubfunc<<pid<<ptype;
  QString s;
      
  s.sprintf("OGLDataCachePolyItem:   nfaces(%ld)\n"
	    ,polyhedron->polyIndices().size()
	    );
  return s;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCachePolyItem::construct(bool transparent)
{
  NSF("OGLDataCachePolyItem::construct");
  //myDebug() << nsubfunc<<pid;
  
  pverticessize=0;
  pndefs=polyhedron->polyIndices().size();
  for(int ifa=0;ifa<pndefs;ifa++){
    pverticessize+=polyhedron->polyIndices().at(ifa).size();
  }
  
  
  pvertices=new GLdouble[pverticessize*3];
  pnormals =new GLdouble[pverticessize*3];
  pdefs=new struct VertexArrayDef[pndefs];
  int nvert=0;
  int nn0=0;
  int idef=0;
  
  for(int ifa=0;ifa<pndefs;ifa++){
    int nfa=polyhedron->polyIndices().at(ifa).size();
    for(int ip=0;ip<nfa;ip++){
      
      int ind=polyhedron->polyIndices().at(ifa).at(ip);
      pnormals[nvert*3+0]=polyhedron->normals().at(ifa)[0];
      pnormals[nvert*3+1]=polyhedron->normals().at(ifa)[1];
      pnormals[nvert*3+2]=polyhedron->normals().at(ifa)[2];
      pvertices[nvert*3+0]=polyhedron->vertices().at(ind)[0];
      pvertices[nvert*3+1]=polyhedron->vertices().at(ind)[1];
      pvertices[nvert*3+2]=polyhedron->vertices().at(ind)[2];
      
      nvert++;
    }
    
    pdefs[idef].mode=GL_POLYGON;
    pdefs[idef].count=nfa;
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=nfa;
  }


  
  
  
  if(OGLPoly::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  if(pdispListName){
   
    OGLIssueVertexArray(pdispListName
			,pvertices,pnormals,0,pindices,pndefs,pdefs);
  }
  
  //if(pdispListName && !transparent){
  if(pdispListName){
    cleanInternal();
  }
  
  
  ogl.reportGLError(nsubfunc);
  
  
  
  //myDebug() << nsubfunc;
  //  myDebug() << nsubfunc  << printSpecQ();
  //myDebug() << "pdispListName="<<pdispListName;

  OGLDataCacheItem::construct(transparent);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCachePolyItem::
collectTransparent(OGLTransparentCollector *tpCollector
		   ,const Vec3 &pos
		   ,const OGLMaterial &mat)
{
  NSF("OGLDataCachePolyItem::collectTransparent");
  //myDebug() << nsubfunc<<this;

  bool old=false;
  //old=true;
  if(old){
    GLdouble *pvert=new GLdouble[pverticessize*3];
    
    for(int i=0;i<pverticessize;i++){
      pvert[3*i+0]=pvertices[3*i+0]+pos[0];
      pvert[3*i+1]=pvertices[3*i+1]+pos[1];
      pvert[3*i+2]=pvertices[3*i+2]+pos[2];
    }

    //myDebug()<<nsubfunc<<"mat"<<mat; 
    tpCollector->addObject(pvert
			   ,pnormals
			   ,0
			   ,GL_FRONT_AND_BACK
			   ,mat
			   ,mat
			   ,pindices
			   ,pndefs,pdefs);
    
    delete []  pvert;
  }else{
    OGLPolygon *pol;
    int nfa=polyhedron->polyIndices().size();
    for(int ifa=0;ifa<nfa;ifa++){
      int nvert=polyhedron->polyIndices().at(ifa).size();
      pol=new OGLPolygon(nvert,false,false,GL_FRONT);
      pol->setMaterial(&mat,&mat);
      pol->setPlane(polyhedron->planes().at(ifa).n(),
		    polyhedron->planes().at(ifa).d()
		    +pos*polyhedron->planes().at(ifa).n());
      for(int ip=0;ip<nvert;ip++){
	int ind=polyhedron->polyIndices().at(ifa).at(ip);
	const Vec3 &v=polyhedron->vertices().at(ind);
 	pol->setVertex(ip,pos[0]+v[0],pos[1]+v[1],pos[2]+v[2]);
	// 	//todo: make polygons in polyhedra
      }
      pol->calcEpsilon();
      
      tpCollector->addToTree(pol);
    }
    
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLPoly::OGLPoly(const Vec3 &position,const Polyhedron *poly)
  :OGLObject("OGLPoly")
  ,pposition(position)
  ,polyhedron(new Polyhedron())
  ,cachedData(0)
{
  (*polyhedron)=(*poly);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLPoly::~OGLPoly()
{
  NSF("OGLPoly::~OGLPoly");
  //myDebug() << nsubfunc<< "I am dying";
  delete polyhedron;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPoly::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPoly::paint(OGLView *,const OGLMagnifier &,bool reduced
		    ,bool //badTransparency
		    )const
{
  Q_UNUSED(reduced);
  NSF("OGLPoly::paint");
  //myDebug() << nsubfunc;



  glTranslated(pposition[0],pposition[1],pposition[2]);

//   myDebug() << ptransparent;
//   myDebug() << badTransparency;
  //pmaterial.issue(ptransparent && ! badTransparency);
  //if(cachedData)cachedData->paint(ptransparent && ! badTransparency);

  pmaterial.issue(ptransparent);
  if(cachedData)cachedData->paint(ptransparent);



  
  
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPoly::construct(OGLView *oglview)
{
  NSF("OGLPoly::construct");
  //myDebug() << nsubfunc;

  OGLDataCachePolyItem *data
    =new OGLDataCachePolyItem(polyhedron);
  
  if(data!=(cachedData=oglview->ogldatacache->append(data,ptransparent))) 
    delete data;


  
  return;

//todo: pcull?
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLPoly::collectTransparent(OGLTransparentCollector *tpCollector
				 ,bool reduced)
{
  NSF("OGLPoly::collectTransparent");Q_UNUSED(reduced);
  //myDebug() << nsubfunc;
  if(!ptransparent) return;
  
  
  ((OGLDataCachePolyItem*)cachedData)->
    collectTransparent(tpCollector,pposition,pmaterial);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLPoly::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "NFPlane " << id();

  double a=pposition*n;
  for(int i=0;i<polyhedron->vertices().size();i++){
    double d=polyhedron->vertices().at(i)*n+a;
    if(i==0){
      near=d;
      far=d;
    }else{
      near=MyMath::max<double>(near,d);
      far=MyMath::min<double>(far,d);
    }
  }
  near=-near;
  far=-far;


  return true;
}


/*EOF*/ 
