/* @(#)OGLPrimitives.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLPRIMITIVES_H
#define _OGLPRIMITIVES_H 1


#include "OGLMaterial.hh"
#include "OGL.hh"
#include "MyDebug.hh"

class OGLPrimitives{
 public:
  OGLPrimitives(GLenum m,int size=0);
  ~OGLPrimitives();

  

  static bool lessThan(const OGLPrimitives *a
				      ,const OGLPrimitives *b){
    return a->pz<b->pz;
  };

  //todo: vertex arrays
  inline void paint(){
    if(pvertices){
      issueMaterial();
      //GLfloat rgb[4];
      //glMaterialfv(GL_FRONT,GL_SPECULAR,GLfloat4FromQRgb(rgb,0xff000000));
      //glMaterialfv(GL_BACK,GL_SPECULAR,GLfloat4FromQRgb(rgb,0xff000000));

      if(pcolors){
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(pcolormode,GL_DIFFUSE);
      }
      
      glBegin(pmode);
      for(int i=0;i<pnsize;i++){
	if(pnormals) glNormal3dv(pnormals+3*i);
	if(pcolors)  glColor4fv(pcolors+4*i);
	glVertex3dv(pvertices+3*i);
      }
      glEnd();
      if(pcolors)
	glDisable(GL_COLOR_MATERIAL);
    }

  };

 private:

  
  inline void setZ(const double *z){
    
    if(pnsize==0){
      myDebug()<<"OGLPrimitives::setZ: weird, pnsize==0?";
      exit(1);
    }
    
    pz=0;
    for(int i=0;i<pnsize;i++){
      pz+=z[0]*pvertices[3*i+0]
	+z[1]*pvertices[3*i+1]
	+z[2]*pvertices[3*i+2];
    }
    
    pz=(pz/pnsize);
    
  };

  inline void clean(){
    if(pvertices) delete [] pvertices;
    if(pnormals ) delete [] pnormals;
    if(pcolors)   delete [] pcolors;
    pvertices=0;
    pnormals=0;
    pcolors=0;
  };
  

  inline void setMaterial(GLenum colormode,const OGLMaterial &front
			  ,const OGLMaterial &back){
    pcolormode=colormode;
    pmaterialfront=front;
    pmaterialback=back;
  };

  inline void issueMaterial(){
    pmaterialfront.issue(true);
    pmaterialback.issue(true);
  };
  
  inline void setVertices(const GLdouble *vertices,const GLuint *indices){
    if(pvertices) delete [] pvertices;
    if(vertices){
      pvertices=new GLdouble[pnsize*3];
      for(int i=0;i<pnsize;i++){
	memcpy(pvertices+3*i,vertices+3*indices[i],sizeof(GLdouble)*3);
      }
    }else{
      pvertices=0;
    }
  };

  inline void setVerticesNoIndex(const GLdouble *vertices,int start){
    if(pvertices) delete [] pvertices;
    if(vertices){
      pvertices=new GLdouble[pnsize*3];
      memcpy(pvertices,vertices+3*start,sizeof(GLdouble)*3*pnsize);
    }else{
      pvertices=0;
    }
  };

  inline void setNormals(const GLdouble *normals,const GLuint *indices){
    if(pnormals) delete [] pnormals;
    if(normals){
      pnormals=new GLdouble[pnsize*3];
      for(int i=0;i<pnsize;i++){
	memcpy(pnormals+3*i,normals+3*indices[i],sizeof(GLdouble)*3);
      }
    }else{
      pnormals=0;
    }
  };
  inline void setNormalsNoIndex(const GLdouble *normals,int start){
    if(pnormals) delete [] pnormals;
    if(normals){
      pnormals=new GLdouble[pnsize*3];
      memcpy(pnormals,normals+3*start,sizeof(GLdouble)*3*pnsize);
    }else{
      pnormals=0;
    }
  };
  inline void setColors(const GLfloat *colors,const GLuint *indices){
    if(pcolors) delete [] pcolors;
    if(colors){
      pcolors=new GLfloat[pnsize*4];
      for(int i=0;i<pnsize;i++){
	memcpy(pcolors+4*i,colors+4*indices[i],sizeof(GLfloat)*4);
      }
    }else{
      pcolors=0;
    }
  };
  inline void setColorsNoIndex(const GLfloat *colors,int start){
    if(pcolors) delete [] pcolors;
    if(colors){
      pcolors=new GLfloat[pnsize*4];
      memcpy(pcolors,colors+4*start,sizeof(GLfloat)*4*pnsize);
    }else{
      pcolors=0;
    }
  };
 private:
  GLenum pmode,pcolormode;
  int pnsize;
  GLdouble *pvertices;
  GLdouble *pnormals;
  GLfloat *pcolors;
  OGLMaterial pmaterialfront,pmaterialback;
  double pz;
  friend class OGLTransparentCollector;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLPRIMITIVES_H */

