//todo: perpendicular orientation
/*************************************************************************
 * @(#)OGLRotationSymbol.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   18 May 2012
 *************************************************************************
 */

#include <QGLWidget>
#include "OGLRotationSymbol.hh"
#include "OGLView.hh"
#include "OGLRotationSymbol.hh"
#include "MyDebug.hh"


bool OGLRotationSymbol::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheRotationSymbolItem
::OGLDataCacheRotationSymbolItem(
				 OGLRotationSymbol::OGLRotationSymbolType type
				 ,double length,long order,long mtau0,
				 double mu0,double dmu,
				 double radius,double invradius,
				 const Vec3 &axis,
				 double symbolthickness
				 ,double symbolwidth
				 ,long np
				 ,bool showbottom
				 ,bool showtop
				 )
  :OGLDataCacheItem("OGLDataCacheRotationSymbolItem")
  ,ptype(type),plength(length)
  ,pradius(radius),pinvradius(invradius),paxis(axis.normalized())
  ,psymbolthickness(symbolthickness)
  ,psymbolwidth(symbolwidth)
  ,porder(order),pmtau0(mtau0%order),pmu0(mu0),pdmu(dmu)
  ,pshowbottom(showbottom)
  ,pshowtop(showtop)
  ,pnphi(np)
  ,mu3(0.3)
  ,muy3(0.7)
{
  NSF("OGLDataCacheRotationSymbolItem::OGLDataCacheRotationSymbolItem");

  String s;  

  if(ptype == OGLRotationSymbol::Rotation ||
     ptype == OGLRotationSymbol::MirrorRotation||
     ptype == OGLRotationSymbol::AxisWithCenter||
     ptype == OGLRotationSymbol::MirrorRotationSpecifier||
     ptype == OGLRotationSymbol::AxisWithCenterSpecifier ){
    s=Sprintf("OGLDataCacheRotationSymbolItem:type(%d)length(%30.6f)"
	      ",radius(%30.6f),invradius(%30.6f)"
	      ",symbolthickness(%30.6f),symbolwidth(%30.6f),np(%ld)"
	      ",order(%ld),mtau0(%ld),mu0(%g),dmu(%g),"
	      "showbottom(%d),showtop(%d)",
	      ptype,plength,pradius,pinvradius,
	      psymbolthickness,psymbolwidth,pnphi,
	      porder,pmtau0,pmu0,pdmu,pshowbottom,pshowtop);
      
  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());

}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheRotationSymbolItem::printSpecQ()const
{
  NSF("OGLRotationSymbol::OGLDataCacheRotationSymbolItem::printSpecQ");
  //myDebug() << nsubfunc<<pid<<ptype;
  QString s;
  if(ptype == OGLRotationSymbol::Rotation ||
     ptype == OGLRotationSymbol::MirrorRotation ||
     ptype == OGLRotationSymbol::AxisWithCenter||
     ptype == OGLRotationSymbol::MirrorRotationSpecifier||
     ptype == OGLRotationSymbol::AxisWithCenterSpecifier
     ){
    
    s.sprintf("OGLDataCacheRotationSymbolItem:   type=Rotation\n"
	      "               nphi=%ld\n"
	      "               length=%g\n"
	      "               radius=%g\n"
	      "               symbolthickness=%g\n"
	      "               symbolwidth=%g\n"
	      "               order=%ld\n"
	      "               mtau0=%ld\n"
	      "               showbottom=%d\n"
	      "               showtop=%d\n"
	      ,pnphi,plength,pradius,psymbolthickness,psymbolwidth
	      ,porder,pmtau0,pshowbottom,pshowtop
	      );
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheRotationSymbolItem::construct(bool transparent)
{
  NSF("OGLDataCacheRotationSymbolItem::construct");
  //printf("%s\n",nsubfunc);


  long nvert=0;
  long nn0=0;
  long idef=0;
  double L=plength;

  Matrix3x3 meu=Matrix3x3::unitMatrix();
  if(porder>1) meu=Vec3::eulerAngles(paxis).eulerRMatrix().transposed();
  Matrix3x3 coordsys[3];
  coordsys[0]=meu*Matrix3x3(0,1,0,0,0,1,1,0,0);
  coordsys[1]=meu*Matrix3x3(0,0,1,1,0,0,0,1,0);
  coordsys[2]=meu*Matrix3x3(1,0,0,0,1,0,0,0,1);
  
  
  
  long nmarker=0;
  if(pshowbottom)nmarker++;
  if(pshowtop)nmarker++;
  
  long lidsubdiv=24/porder;
  
  long NsubC2=8;
  
  bool left=(pmtau0>(porder/2) && pmtau0);
  bool right=(pmtau0<=(porder/2) && pmtau0);
  
  long nnonsym=0;
  long legskip=MyMath::min<long>(pmtau0,porder-pmtau0);

  QList<double> mulist;
  QList<double> invlist;
  
  if(OGLRotationSymbol::
     equalBaseType(ptype,OGLRotationSymbol::MirrorRotation)){
    if(porder==1){
      invlist.append(0);
    }else{
      //invariant point marker for mirrorrotations or axiswithcenter
      long kmin=0,kmax=0;
      if(pdmu>0){
	kmin=floor(-pmu0/pdmu)-1;
	kmax=ceil((1.-pmu0)/pdmu)+1;
      }else{
	kmin=floor((1-pmu0)/pdmu)-1;
	kmax=ceil(-pmu0/pdmu)+1;
      }
      
      for(long k=kmin;k<=kmax;k++){
	double mu=k*pdmu+pmu0;
	if(0<mu && mu<1) mulist.append(L*mu);
	mu=mu+pdmu*0.5*(pmtau0?1:0);
	if(0<mu && mu<1) invlist.append(L*mu);
	
      }
      if(ptype==OGLRotationSymbol::MirrorRotation){
	if(porder%2==0){
	  invlist.clear();
	}else{
	  invlist=mulist;
	  //mulist.clear();
	}
      }else{
	if(porder%2==0){
	  //mulist.clear();
	  //invlist.clear();
	}
      }
    }
    // now determine mesh sizes for the inversion point(s) and
    // rotoinversion centers
    for(long k=0;k<invlist.size();k++){
      //inversion symbol
      for(long kk=0;kk<3;kk++){
	makeRodMesh(nvert,idef,nn0,pnphi*2,pradius,L,L*0.5,
		    Matrix3x3::unitMatrix(),Round,Round,true);
      }
    }
    //rotoinversion point
    for(long k=0;k<mulist.size();k++){
      makeRodMesh(nvert,idef,nn0,pnphi*2,pradius,L,L*0.5,
		  Matrix3x3::unitMatrix(),Flat,Flat,true);
    }
  }      

  //allocate the RotationSymbol data
  if(ptype==OGLRotationSymbol::AxisWithCenterSpecifier && porder>1){
    pverticessize=nmarker*(porder*(1+lidsubdiv)+2);
    pndefs=nmarker;
  }else if(ptype==OGLRotationSymbol::MirrorRotationSpecifier){
    if(porder==4){
      long nquads=0;
      if(lidsubdiv>1)nquads=1;
      pverticessize+=nmarker*(2*3+nquads*2*lidsubdiv);
      pndefs+=nmarker*(1+nquads);
    }else{
      pverticessize+=nmarker*(porder/2);
      pndefs+=nmarker;
    }
  }else{

    // the main rod
    if(porder>1){
      makeRodMesh(nvert,idef,nn0,pnphi,pradius,L,L*0.5,Matrix3x3::unitMatrix(),
		  (pshowbottom?Open:Flat),(pshowtop?Open:Flat),true);
    }
    
    
    if(porder==2){
      pverticessize+=nmarker*(3*4+2*2*(NsubC2-1)*(NsubC2>2?1:0)+2*2*(NsubC2+1));
      pndefs+=nmarker*(1+(NsubC2>2?1:0)*2+2);
      if(right){
	pverticessize+=nmarker*(4*5*2);
	pndefs+=nmarker*1;
      }
    }else if(porder!=1){
      if(left||right) {
	nnonsym=4*porder/legskip;
      }
      pverticessize+=nmarker*(porder+4*(porder+nnonsym));
      pndefs+=nmarker*2;
      if(OGLRotationSymbol::
	 equalBaseType(ptype,OGLRotationSymbol::Rotation)){
	pverticessize+=nmarker*porder;
	pndefs+=nmarker;
      }else if(OGLRotationSymbol::
	       equalBaseType(ptype,OGLRotationSymbol::MirrorRotation)){
	if( porder%2==1 || 
	    (ptype & OGLRotationSymbol::AxisWithCenterFlag && porder>1) ){
	  pverticessize+=nmarker*(2*porder*(1+lidsubdiv)+2);
	  pndefs+=nmarker;
	}else{
	  if(porder==4){
	    pverticessize+=nmarker*(2*(3+lidsubdiv));
	    pndefs+=nmarker*2;
	  }else{
	    pverticessize+=nmarker*(3*porder/2);
	    pndefs+=nmarker;
	  }
	}
	
      

      }else{
	myDebug()<<nsubfunc<<"what is this 1";
	exit(1);
      }
    }
  }

  pvertices=new GLdouble[pverticessize*3];
  pnormals =new GLdouble[pverticessize*3];
  pdefs=new struct VertexArrayDef[pndefs];
  
  
  
  if(ptype==OGLRotationSymbol::MirrorRotationSpecifier){
    if(pshowbottom)
      constructMarkerSpecifier(nvert,idef,nn0,false,L,lidsubdiv);
    if(pshowtop)
      constructMarkerSpecifier(nvert,idef,nn0,true,L,lidsubdiv);
  }else if(ptype==OGLRotationSymbol::AxisWithCenterSpecifier && porder>1){

    if(porder==2){
      //todo 
      // 	if(pshowbottom)
      // 	  constructMarkerC2(nvert,idef,nn0,right,false,L,NsubC2);
      // 	if(pshowtop)
      // 	  constructMarkerC2(nvert,idef,nn0,right,true,L,NsubC2);
    }else{
      if(pshowbottom)
	constructMarkerSpecifier(nvert,idef,nn0,false,L,lidsubdiv);
      if(pshowtop)
	constructMarkerSpecifier(nvert,idef,nn0,true,L,lidsubdiv);
    }
    

  }else{
    
    // the main rod
    if(porder>1){
      makeRodMesh(nvert,idef,nn0,pnphi,pradius,L,L*0.5,Matrix3x3::unitMatrix(),
		  (pshowbottom?Open:Flat),(pshowtop?Open:Flat));
    }

    if(porder==2){
      if(pshowbottom)
	constructMarkerC2(nvert,idef,nn0,right,false,L,NsubC2);
      if(pshowtop)
	constructMarkerC2(nvert,idef,nn0,right,true,L,NsubC2);
    }else if(porder!=1){
      if(pshowbottom)
	constructMarker(nvert,idef,nn0,right,left,legskip,nnonsym,false,L,
			lidsubdiv);
      if(pshowtop)
	constructMarker(nvert,idef,nn0,right,left,legskip,nnonsym,true,L,
			lidsubdiv);
    }
    
    

    
  }


  if(OGLRotationSymbol::
     equalBaseType(ptype,OGLRotationSymbol::MirrorRotation)){
    // the invariant points for rotoinversion
    
    
    for(int k=0;k<invlist.size();k++){
	//inversion symbol
	for(long kk=0;kk<3;kk++){
	  makeRodMesh(nvert,idef,nn0,pnphi*2,pinvradius,
		      pinvradius*24,invlist.at(k),
		      coordsys[kk],Round,Round);
	}
    }
    for(long k=0;k<mulist.size();k++){
      //rotoinversion point
      makeRodMesh(nvert,idef,nn0,pnphi*2,3*pradius,
		  pradius*0.4,mulist.at(k),Matrix3x3::unitMatrix(),
		  Flat,Flat);
    }
    
  }


  

  if(OGLRotationSymbol::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  if(pdispListName){
   
    OGLIssueVertexArray(pdispListName
			,pvertices,pnormals,0,pindices,pndefs,pdefs);
 
    
    
    cleanInternal();

    
  }
  
  
  
  ogl.reportGLError(nsubfunc);
  
  
  
  //myDebug() << nsubfunc;
  //  myDebug() << nsubfunc  << printSpecQ();
  //myDebug() << "pdispListName="<<pdispListName;

  OGLDataCacheItem::construct(transparent);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheRotationSymbolItem::
constructMarkerC2(long &nvert,long &idef,long &nn0,bool right,
		  bool top,double L,long NsubC2)
{

  double height0=0;
  double height1=-psymbolthickness;
  double inv=1.;
  if(top){
    inv=-1.;
    height0=L;
    height1=L+psymbolthickness;
  }
  
  double rscale=psymbolwidth*2;
  double h=rscale/5;

  double leglength=psymbolwidth*sqrt(3)*mu3;
  double legwidth=(1.-muy3)*psymbolwidth*0.5;

  double R=rscale*rscale/(8*h)+h*0.5;
  double alpha0=atan2(rscale,2*(R-h));

  double alpha=alpha0*2/NsubC2;

  double *xx=new double[NsubC2+1];
  double *yy=new double[NsubC2+1];

  for(long i=0;i<=NsubC2;i++){
    xx[i]=R*sin(inv*alpha*i-inv*alpha0);
    yy[i]=R*cos(inv*alpha*i-inv*alpha0)+h-R;
  }


  for(int k=0;k<2;k++){
    double h0=height0;
    double sig=1;
    if(k==1){
      h0=height1;
      sig=-1;
    }

    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[0];
    pvertices[nvert*3+1]=yy[0];
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[1];
    pvertices[nvert*3+1]=-sig*yy[1];
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[1];
    pvertices[nvert*3+1]=sig*yy[1];
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    
    
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[NsubC2];
    pvertices[nvert*3+1]=yy[NsubC2];
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[NsubC2-1];
    pvertices[nvert*3+1]=sig*yy[NsubC2-1];
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=sig*inv;
    pvertices[nvert*3+0]=xx[NsubC2-1];
    pvertices[nvert*3+1]=-sig*yy[NsubC2-1];
    pvertices[nvert*3+2]=h0;
    nvert++;
  }
  
  pdefs[idef].mode=GL_TRIANGLES;
  pdefs[idef].count=12;
  pdefs[idef].start=nn0;
  pdefs[idef].useindex=false;
  
  idef++;
  nn0+=12;

  if(NsubC2>2){
    for(long k=0;k<2;k++){
      double h0=height0;
      double sig=1;
      if(k==1){
	h0=height1;
	sig=-1;
      }
      for(long i=1;i<NsubC2-1;i++){
	
	pnormals[nvert*3+0]=0;
	pnormals[nvert*3+1]=0;
	pnormals[nvert*3+2]=sig*inv;
	pvertices[nvert*3+0]=xx[i];
	pvertices[nvert*3+1]=sig*yy[i];
	pvertices[nvert*3+2]=h0;
	nvert++;
	
	pnormals[nvert*3+0]=0;
	pnormals[nvert*3+1]=0;
	pnormals[nvert*3+2]=sig*inv;
	pvertices[nvert*3+0]=xx[i];
	pvertices[nvert*3+1]=-sig*yy[i];
	pvertices[nvert*3+2]=h0;
	nvert++;
      }
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig*inv;
      pvertices[nvert*3+0]=xx[NsubC2-1];
      pvertices[nvert*3+1]=sig*yy[NsubC2-1];
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig*inv;
      pvertices[nvert*3+0]=xx[NsubC2-1];
      pvertices[nvert*3+1]=-sig*yy[NsubC2-1];
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      
      
      pdefs[idef].mode=GL_QUAD_STRIP;
      pdefs[idef].count=2*(NsubC2-1);
      pdefs[idef].start=nn0;
      pdefs[idef].useindex=false;
      
      idef++;
      nn0+=2*(NsubC2-1);
    }    
  }

  for(long k=0;k<2;k++){
    double sig=1;
    double h0=height0;
    double h1=height1;
    if(k==1){
      sig=-1;
      h0=height1;
      h1=height0;
    }
    for(long i=0;i<=NsubC2;i++){
      double nx=xx[i]/R;
      double ny=yy[i]/R-h/R+1;
      pnormals[nvert*3+0]=nx;
      pnormals[nvert*3+1]=sig*ny;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=xx[i];
      pvertices[nvert*3+1]=sig*yy[i];
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=nx;
      pnormals[nvert*3+1]=sig*ny;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=xx[i];
      pvertices[nvert*3+1]=sig*yy[i];
      pvertices[nvert*3+2]=h0;
      nvert++;
    }
    pdefs[idef].mode=GL_QUAD_STRIP;
    pdefs[idef].count=2*(NsubC2+1);
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=2*(NsubC2+1);
  }

  //legs
  if(right){
    double cal=(R-h)/R;
    double sal=rscale/R*0.5;
    
    double XNp1=R*sal+leglength*cal;
    double YNp1=-leglength*sal;
    
    double Xq=XNp1-sal*legwidth;
    double Yq=YNp1-cal*legwidth;

    double kap=legwidth/(sal*(R*sal-xx[NsubC2-1])+cal*yy[NsubC2-1]);

    double Xx=R*sal*(1-kap)+xx[NsubC2-1]*kap;
    double Yx=-yy[NsubC2-1]*kap;

    double h0=height0;
    double h1=height1;
    if(top){
      h0=height1;
      h1=height0;
    }

    for(long k=0;k<2;k++){
      
      if(k==1){
	cal=-cal;
	sal=-sal;
	XNp1=-XNp1;
	YNp1=-YNp1;
	Xx=-Xx;
	Yx=-Yx;
	Xq=-Xq;
	Yq=-Yq;
      }
      
      pnormals[nvert*3+0]=sal;
      pnormals[nvert*3+1]=-cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=R*sal;
      pvertices[nvert*3+1]=0;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=sal;
      pnormals[nvert*3+1]=-cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=R*sal;
      pvertices[nvert*3+1]=0;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=sal;
      pnormals[nvert*3+1]=-cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=sal;
      pnormals[nvert*3+1]=-cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      //next
      pnormals[nvert*3+0]=cal;
      pnormals[nvert*3+1]=sal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=cal;
      pnormals[nvert*3+1]=sal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=cal;
      pnormals[nvert*3+1]=sal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=cal;
      pnormals[nvert*3+1]=sal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      //next
      pnormals[nvert*3+0]=-sal;
      pnormals[nvert*3+1]=cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=-sal;
      pnormals[nvert*3+1]=cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=-sal;
      pnormals[nvert*3+1]=cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xx;
      pvertices[nvert*3+1]=-Yx;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=-sal;
      pnormals[nvert*3+1]=cal;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=Xx;
      pvertices[nvert*3+1]=-Yx;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      //next
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=1;
      pvertices[nvert*3+0]=R*sal;
      pvertices[nvert*3+1]=0;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=1;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=1;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=1;
      pvertices[nvert*3+0]=Xx;
      pvertices[nvert*3+1]=-Yx;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      
      //next
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-1;
      pvertices[nvert*3+0]=R*sal;
      pvertices[nvert*3+1]=0;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-1;
      pvertices[nvert*3+0]=Xx;
      pvertices[nvert*3+1]=-Yx;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-1;
      pvertices[nvert*3+0]=Xq;
      pvertices[nvert*3+1]=-Yq;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-1;
      pvertices[nvert*3+0]=XNp1;
      pvertices[nvert*3+1]=-YNp1;
      pvertices[nvert*3+2]=h1;
      nvert++;
    }
    
    pdefs[idef].mode=GL_QUADS;
    pdefs[idef].count=4*5*2;
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=4*5*2;

  }


  delete [] xx;
  delete [] yy;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheRotationSymbolItem::
constructMarker(long &nvert,long &idef,long &nn0,
		bool right,bool left,long legskip,long nnonsym,
		bool top,double L,long lidsubdiv)
{
  NSF("constructMarker");
  double height0=0;
  double height1=-psymbolthickness;
  double inv=1.;
  if(top){
    height0=L;
    height1=L+psymbolthickness;
    inv=-1.;
  }
  
  double alpha=2.*Pi/porder;
  double onepcal=1.+cos(alpha);
  double onemcal=1.-cos(alpha);
  double rscale=psymbolwidth*(1.+sqrt(0.5))/(1.+sqrt(onepcal));

  if(porder==6) rscale*=1.2;
  
  // |(r1+r0)/2*(1-muy)|=const
  double muy=1.-psymbolwidth/rscale*sqrt(0.5)/sqrt(onepcal)*(1-muy3);
  
  // |(r1-r0)*mu|=const
  double mu=psymbolwidth/rscale*sqrt(3.)/sqrt(2.*onemcal)*mu3;

  
  
  double a1=(muy-1.)*onepcal/(1-cos(2.*alpha));
  double a0=(muy+1.-2*muy*cos(alpha))*onepcal/(1-cos(2.*alpha));
  
  double alpha0=0;
  if(porder==3) alpha0=inv*alpha/2;
  if(porder==4) alpha0=inv*alpha/2;
  if(porder==6) alpha0=inv*alpha/2;
  
  for(int i=0;i<porder;i++){
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=inv;
    
    double c=cos(inv*i*alpha+alpha0);
    double s=sin(inv*i*alpha+alpha0);
    pvertices[nvert*3+0]=rscale*c;
    pvertices[nvert*3+1]=rscale*s;
    pvertices[nvert*3+2]=height0;
    nvert++;
  }
  
  pdefs[idef].mode=GL_POLYGON;
  pdefs[idef].count=porder;
  pdefs[idef].start=nn0;
  pdefs[idef].useindex=false;
  
  idef++;
  nn0+=porder;

  if(OGLRotationSymbol::equalBaseType(ptype,OGLRotationSymbol::MirrorRotation)){

    if( porder%2==1 || (ptype & OGLRotationSymbol::AxisWithCenterFlag) ){
      double dotsize=0.3;
      for(int i=0;i<=porder;i++){
	double angle=-inv*i*alpha+alpha0;
	double x0=cos(angle)*rscale;
	double y0=sin(angle)*rscale;
	double x1=cos(angle-inv*alpha)*rscale;
	double y1=sin(angle-inv*alpha)*rscale;
	double da=-inv*alpha/(lidsubdiv+1);
	double dx=(x1-x0)/(lidsubdiv+1);
	double dy=(y1-y0)/(lidsubdiv+1);
	long dim=lidsubdiv;
	if(i==porder) dim=0;
	for(int j=0;j<=dim;j++){
	  
	  double c=cos(angle);
	  double s=sin(angle);
	  
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=rscale*c*dotsize;
	  pvertices[nvert*3+1]=rscale*s*dotsize;
	  pvertices[nvert*3+2]=height1;
	  nvert++;
	  
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=x0;
	  pvertices[nvert*3+1]=y0;
	  pvertices[nvert*3+2]=height1;
	  nvert++;
	  
	  angle+=da;
	  x0+=dx;
	  y0+=dy;
	}
      }
      
      pdefs[idef].mode=GL_QUAD_STRIP;
      pdefs[idef].count=2*porder*(1+lidsubdiv)+2;
      pdefs[idef].start=nn0;
      pdefs[idef].useindex=false;
      
      idef++;
      nn0+=2*porder*(1+lidsubdiv)+2;
    }else{
      if(porder==4){
	double co[4],si[4];
	
	for(int i=0;i<porder;i++){
	  co[i]=cos(-inv*i*alpha+alpha0);
	  si[i]=sin(-inv*i*alpha+alpha0);
	}
	Vec3 r0(rscale*co[0],rscale*si[0],height1);
	Vec3 r1(rscale*co[1],rscale*si[1],height1);
	Vec3 r2(rscale*co[2],rscale*si[2],height1);
	Vec3 r3(rscale*co[3],rscale*si[3],height1);
	Vec3 rc=(r0+r2)*0.5;

	double d=norm(rc-r0);
	double lambda=0.2;
	double mu=(lambda*lambda-1)/lambda*0.5;
	double R=sqrt(1+mu*mu)*d;
	
	Vec3 rho=rc+mu*(r1-rc);

	double phi0=atan2((r0[1]-rho[1])/R,(r0[0]-rho[0])/R);
	double dphi=(atan2((r2[1]-rho[1])/R,(r2[0]-rho[0])/R)-phi0)
	  /(1+lidsubdiv);
	


	// fan center
	pnormals[nvert*3+0]=0;
	pnormals[nvert*3+1]=0;
	pnormals[nvert*3+2]=-inv;
	  
	pvertices[nvert*3+0]=r1[0];
	pvertices[nvert*3+1]=r1[1];
	pvertices[nvert*3+2]=r1[2];
	nvert++;
	
	long N=1+lidsubdiv;

	double ph=phi0+dphi*N;
	for(int i=N;i>=0;i--){
	  Vec3 ri=rho+R*Vec3(cos(ph),sin(ph),0.0);
	  
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=ri[0];
	  pvertices[nvert*3+1]=ri[1];
	  pvertices[nvert*3+2]=ri[2];
	  nvert++;
	  ph-=dphi;
	}
	
    
	pdefs[idef].mode=GL_TRIANGLE_FAN;
	pdefs[idef].count=3+lidsubdiv;
	pdefs[idef].start=nn0;
	pdefs[idef].useindex=false;
	
	idef++;
	nn0+=3+lidsubdiv;


	// fan center
	pnormals[nvert*3+0]=0;
	pnormals[nvert*3+1]=0;
	pnormals[nvert*3+2]=-inv;
	  
	pvertices[nvert*3+0]=r3[0];
	pvertices[nvert*3+1]=r3[1];
	pvertices[nvert*3+2]=r3[2];
	nvert++;
	

	ph=phi0+dphi*N;
	for(int i=N;i>=0;i--){
	  Vec3 ri=2*rc-rho-R*Vec3(cos(ph),sin(ph),0.0);
	  
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=ri[0];
	  pvertices[nvert*3+1]=ri[1];
	  pvertices[nvert*3+2]=ri[2];
	  nvert++;
	  ph-=dphi;
	}
	
    
	pdefs[idef].mode=GL_TRIANGLE_FAN;
	pdefs[idef].count=3+lidsubdiv;
	pdefs[idef].start=nn0;
	pdefs[idef].useindex=false;
	
	idef++;
	nn0+=3+lidsubdiv;


      }else{
	double co[porder+1];
	double si[porder+1];
	for(int i=0;i<=porder;i++){
	  co[i]=cos(-inv*i*alpha+alpha0);
	  si[i]=sin(-inv*i*alpha+alpha0);
	}
	
	for(int i=0;i<porder/2;i++){
	  for(int k=0;k<3;k++){
	    pnormals[nvert*3+0]=0;
	    pnormals[nvert*3+1]=0;
	    pnormals[nvert*3+2]=-inv;
	    
	    pvertices[nvert*3+0]=rscale*co[2*i+k];
	    pvertices[nvert*3+1]=rscale*si[2*i+k];
	    pvertices[nvert*3+2]=height1;
	    nvert++;
	  }
	  
	}
	
	
	pdefs[idef].mode=GL_TRIANGLES;
	pdefs[idef].count=3*porder/2;
	pdefs[idef].start=nn0;
	pdefs[idef].useindex=false;
	
	idef++;
	nn0+=3*porder/2;
	
      }
    }
  }else if(ptype==OGLRotationSymbol::Rotation){

    for(int i=0;i<porder;i++){
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      double c=cos(-inv*i*alpha+alpha0);
      double s=sin(-inv*i*alpha+alpha0);
      pvertices[nvert*3+0]=rscale*c;
      pvertices[nvert*3+1]=rscale*s;
      pvertices[nvert*3+2]=height1;
      nvert++;
    }
    
    
    pdefs[idef].mode=GL_POLYGON;
    pdefs[idef].count=porder;
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=porder;
  }else{
    myDebug()<<nsubfunc<<"what is this2";
    exit(1);
  }
    
  for(int j=0;j<porder;j++){
    
    long i=j;
    long i1=i+1;
    long sig=inv;
    double h0=height0;
    double h1=height1;
    if(right){
      i=porder-j;
      i1=i-1;
      sig=-inv;
      h0=height1;
      h1=height0;
    }
    
    double c0=cos(inv*i*alpha+alpha0);
    double s0=sin(inv*i*alpha+alpha0);
    
    double c1=cos(inv*i1*alpha+alpha0);
    double s1=sin(inv*i1*alpha+alpha0);
    
    double cn=cos((inv*i+sig*0.5)*alpha+alpha0);
    double sn=sin((inv*i+sig*0.5)*alpha+alpha0);
    
    double C0=c0;
    double S0=s0;
    if((left || right) && ((j%legskip)==0)){
      C0=(c0-(c1-c0)*mu);
      S0=(s0-(s1-s0)*mu);
    }
    
    pnormals[nvert*3+0]=cn;
    pnormals[nvert*3+1]=sn;
    pnormals[nvert*3+2]=0;
    pvertices[nvert*3+0]=rscale*C0;
    pvertices[nvert*3+1]=rscale*S0;
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    
    pnormals[nvert*3+0]=cn;
    pnormals[nvert*3+1]=sn;
    pnormals[nvert*3+2]=0;
    pvertices[nvert*3+0]=rscale*C0;
    pvertices[nvert*3+1]=rscale*S0;
    pvertices[nvert*3+2]=h1;
    nvert++;
    
    pnormals[nvert*3+0]=cn;
    pnormals[nvert*3+1]=sn;
    pnormals[nvert*3+2]=0;
    pvertices[nvert*3+0]=rscale*c1;
    pvertices[nvert*3+1]=rscale*s1;
    pvertices[nvert*3+2]=h1;
    nvert++;
    
    pnormals[nvert*3+0]=cn;
    pnormals[nvert*3+1]=sn;
    pnormals[nvert*3+2]=0;
    pvertices[nvert*3+0]=rscale*c1;
    pvertices[nvert*3+1]=rscale*s1;
    pvertices[nvert*3+2]=h0;
    nvert++;
    
    
    if((left || right) && ((j%legskip)==0) ){
      double cx=(a0*c0+a1*c1);
      double sx=(a0*s0+a1*s1);
      double cout=((c0+c1)/2*muy-(c1-c0)*(0.5+mu));
      double sout=((s0+s1)/2*muy-(s1-s0)*(0.5+mu));
      
      
      pnormals[nvert*3+0]=-cn;
      pnormals[nvert*3+1]=-sn;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cx;
      pvertices[nvert*3+1]=rscale*sx;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      pnormals[nvert*3+0]=-cn;
      pnormals[nvert*3+1]=-sn;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cx;
      pvertices[nvert*3+1]=rscale*sx;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=-cn;
      pnormals[nvert*3+1]=-sn;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=-cn;
      pnormals[nvert*3+1]=-sn;
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      
      // next  
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-sig;
      pvertices[nvert*3+0]=rscale*c0;
      pvertices[nvert*3+1]=rscale*s0;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-sig;
      pvertices[nvert*3+0]=rscale*C0;
      pvertices[nvert*3+1]=rscale*S0;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-sig;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-sig;
      pvertices[nvert*3+0]=rscale*cx;
      pvertices[nvert*3+1]=rscale*sx;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
      
      // next 
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      pvertices[nvert*3+0]=rscale*cx;
      pvertices[nvert*3+1]=rscale*sx;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      pvertices[nvert*3+0]=rscale*C0;
      pvertices[nvert*3+1]=rscale*S0;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      pvertices[nvert*3+0]=rscale*c0;
      pvertices[nvert*3+1]=rscale*s0;
      pvertices[nvert*3+2]=h0;
      nvert++;
      
      
      // next 
      Vec3 n=Vec3(c0-c1,s0-s1,0).normalize();
      pnormals[nvert*3+0]=n[0];
      pnormals[nvert*3+1]=n[1];
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*C0;
      pvertices[nvert*3+1]=rscale*S0;
      pvertices[nvert*3+2]=h1;
      nvert++;
      pnormals[nvert*3+0]=n[0];
      pnormals[nvert*3+1]=n[1];
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*C0;
      pvertices[nvert*3+1]=rscale*S0;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=n[0];
      pnormals[nvert*3+1]=n[1];
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h0;
      nvert++;
      pnormals[nvert*3+0]=n[0];
      pnormals[nvert*3+1]=n[1];
      pnormals[nvert*3+2]=0;
      pvertices[nvert*3+0]=rscale*cout;
      pvertices[nvert*3+1]=rscale*sout;
      pvertices[nvert*3+2]=h1;
      nvert++;
      
    }
    
  }
  
  pdefs[idef].mode=GL_QUADS;
  pdefs[idef].count=4*(porder+nnonsym);
  pdefs[idef].start=nn0;
  pdefs[idef].useindex=false;
  
  idef++;
  nn0+=4*(porder+nnonsym);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheRotationSymbolItem::
constructMarkerSpecifier(long &nvert,long &idef,long &nn0,
			 bool top,double L,long lidsubdiv)
{
  double height1=-psymbolthickness;
  double inv=1.;
  if(top){
    height1=L+psymbolthickness;
    inv=-1.;
   }
  
  double alpha=2.*Pi/porder;
  double onepcal=1.+cos(alpha);
  double rscale=psymbolwidth*(1.+sqrt(0.5))/(1.+sqrt(onepcal));

   if(porder==6) rscale*=1.2;
  
  
  double alpha0=0;
  if(porder==3) alpha0=inv*alpha/2;
  if(porder==4) alpha0=inv*alpha/2;
  if(porder==6) alpha0=inv*alpha/2;
  

  if(ptype==OGLRotationSymbol::AxisWithCenterSpecifier){
    double dotsize=0.3;
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=-inv;
	
    pvertices[nvert*3+0]=0;
    pvertices[nvert*3+1]=0;
    pvertices[nvert*3+2]=height1;
    nvert++;
    
    for(int i=0;i<=porder;i++){
      double angle=-inv*i*alpha+alpha0;
      double da=-inv*alpha/(lidsubdiv+1);
      long dim=lidsubdiv;
      if(i==porder) dim=0;
      for(int j=0;j<=dim;j++){
	
 	double c=cos(angle);
	double s=sin(angle);

 	pnormals[nvert*3+0]=0;
 	pnormals[nvert*3+1]=0;
 	pnormals[nvert*3+2]=-inv;
	
 	pvertices[nvert*3+0]=rscale*c*dotsize;
 	pvertices[nvert*3+1]=rscale*s*dotsize;
 	pvertices[nvert*3+2]=height1;
 	nvert++;

	angle+=da;
      }
     }


    pdefs[idef].mode=GL_TRIANGLE_FAN;
    pdefs[idef].count=(porder*(1+lidsubdiv)+2);
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=(porder*(1+lidsubdiv)+2);
    
    
  }else if(ptype==OGLRotationSymbol::MirrorRotationSpecifier){
    if(porder==4){
      double co[4],si[4];
      for(int i=0;i<porder;i++){
	co[i]=cos(-inv*i*alpha+alpha0);
	si[i]=sin(-inv*i*alpha+alpha0);
      }
      Vec3 r0(rscale*co[0],rscale*si[0],height1);
      Vec3 r1(rscale*co[1],rscale*si[1],height1);
      Vec3 r2(rscale*co[2],rscale*si[2],height1);
      Vec3 r3(rscale*co[3],rscale*si[3],height1);
      Vec3 rc=(r0+r2)*0.5;
      
      double d=norm(rc-r0);
      double lambda=0.2;
      double mu=(lambda*lambda-1)/lambda*0.5;
      double R=sqrt(1+mu*mu)*d;
      
      Vec3 rho=rc+mu*(r1-rc);
      
      double phi0=atan2((r0[1]-rho[1])/R,(r0[0]-rho[0])/R);
      double dphi=(atan2((r2[1]-rho[1])/R,(r2[0]-rho[0])/R)-phi0)
	/(1+lidsubdiv);
      
      long N=1+lidsubdiv;

      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=r2[0];
      pvertices[nvert*3+1]=r2[1];
      pvertices[nvert*3+2]=r2[2];
      nvert++;
      
      Vec3 ri=2*rc-rho-R*Vec3(cos(phi0+dphi),sin(phi0+dphi),0.0);
  
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=ri[0];
      pvertices[nvert*3+1]=ri[1];
      pvertices[nvert*3+2]=ri[2];
      nvert++;
      
      ri=rho+R*Vec3(cos(phi0+dphi*N-dphi),sin(phi0+dphi*N-dphi),0.0);
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=ri[0];
      pvertices[nvert*3+1]=ri[1];
      pvertices[nvert*3+2]=ri[2];
      nvert++;
      
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=r0[0];
      pvertices[nvert*3+1]=r0[1];
      pvertices[nvert*3+2]=r0[2];
      nvert++;
      
      
      ri=rho+R*Vec3(cos(phi0+dphi),sin(phi0+dphi),0.0);
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=ri[0];
      pvertices[nvert*3+1]=ri[1];
      pvertices[nvert*3+2]=ri[2];
      nvert++;

      ri=2*rc-rho-R*Vec3(cos(phi0+dphi*(N-1)),sin(phi0+dphi*(N-1)),0.0);
  
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=-inv;
      
      pvertices[nvert*3+0]=ri[0];
      pvertices[nvert*3+1]=ri[1];
      pvertices[nvert*3+2]=ri[2];
      nvert++;

      pdefs[idef].mode=GL_TRIANGLES;
      pdefs[idef].count=2*3;
      pdefs[idef].start=nn0;
      pdefs[idef].useindex=false;
      
      idef++;
      nn0+=2*3;

      if(lidsubdiv>1){
	for(int i=1;i<=lidsubdiv;i++){
	  ri=2*rc-rho-R*Vec3(cos(phi0+dphi*(N-i)),sin(phi0+dphi*(N-i)),0.0);
	  
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=ri[0];
	  pvertices[nvert*3+1]=ri[1];
	  pvertices[nvert*3+2]=ri[2];
	  nvert++;
	  
	  ri=rho+R*Vec3(cos(phi0+dphi*i),sin(phi0+dphi*i),0.0);
	  pnormals[nvert*3+0]=0;
	  pnormals[nvert*3+1]=0;
	  pnormals[nvert*3+2]=-inv;
	  
	  pvertices[nvert*3+0]=ri[0];
	  pvertices[nvert*3+1]=ri[1];
	  pvertices[nvert*3+2]=ri[2];
	  nvert++;
	  
	}
	
	pdefs[idef].mode=GL_QUAD_STRIP;
	pdefs[idef].count=lidsubdiv*2;
	pdefs[idef].start=nn0;
	pdefs[idef].useindex=false;
	
	idef++;
	nn0+=lidsubdiv*2;
      }
      
    }else{
      for(int i=0;i<porder/2;i++){
	pnormals[nvert*3+0]=0;
	pnormals[nvert*3+1]=0;
	pnormals[nvert*3+2]=-inv;
	
	double c=cos(-inv*i*alpha*2+alpha0);
	double s=sin(-inv*i*alpha*2+alpha0);
	pvertices[nvert*3+0]=rscale*c;
	pvertices[nvert*3+1]=rscale*s;
	pvertices[nvert*3+2]=height1;
	nvert++;
      }
      
      
      pdefs[idef].mode=GL_POLYGON;
      pdefs[idef].count=porder/2;
      pdefs[idef].start=nn0;
      pdefs[idef].useindex=false;
      
      idef++;
      nn0+=porder/2;
    }      
  }
  
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLRotationSymbol::OGLRotationSymbol(OGLRotationSymbolType type,
				     const Vec3 &pos0,const Vec3 &pos1,
				     long order,long mtau0,
				     double mu0,double dmu,
				     double radius,double invradius,
				     long np,
				     double symbolthickness,
				     double symbolwidth,
				     bool showbottom,
				     bool showtop
				     )
  :OGLObject("OGLRotationSymbol")
  ,ptype(type)
  ,pnphi(np)
  ,pradius(radius),pinvradius(invradius)
  ,psymbolthickness(symbolthickness)
  ,psymbolwidth(symbolwidth)
  ,porder(order)
  ,pmtau0(mtau0%order),pmu0(mu0),pdmu(dmu)
  ,pshowbottom(showbottom)
  ,pshowtop(showtop)
  ,pposition0(pos0)
  ,pposition1(pos1)
  ,cachedDataNormal(0)
  ,cachedDataNormalSpecifier(0)
{
  plength=norm(pos1-pos0);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLRotationSymbol::~OGLRotationSymbol()
{
  NSF("OGLRotationSymbol::~OGLRotationSymbol");
  //myDebug() << nsubfunc<< "I am dying";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLRotationSymbol::protectCachedData()
{
  if(cachedDataNormal)
    cachedDataNormal->increaseUsed();
  if(cachedDataNormalSpecifier)
    cachedDataNormalSpecifier->increaseUsed();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLRotationSymbol::paint(OGLView *,const OGLMagnifier &,bool /*reduced*/
		    ,bool// badTransparency
		    )const
{
  NSF("OGLRotationSymbol::paint");
  //myDebug() << nsubfunc;



  glTranslated(pposition0[0],pposition0[1],pposition0[2]);
  if(porder>1){
    Vec3 eu=Vec3::eulerAngles(pposition1-pposition0);
    //myDebug() << "eu="<<eu[0]<<eu[1]<<eu[2];
    glRotated(eu[0]*180./Pi,0.,0.,1.);
    glRotated(eu[1]*180./Pi,0.,1.,0.);
  }
  if(cachedDataNormal){
    pmaterial.issue();
    cachedDataNormal->paint(ptransparent);
  }

  if(cachedDataNormalSpecifier){
    pspecifiermaterial.issue();
    cachedDataNormalSpecifier->paint(ptransparent);
  }
  

  
  
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLRotationSymbol::construct(OGLView *oglview)
{
  NSF("OGLRotationSymbol::construct");
  //  myDebug() << nsubfunc<<this;

  OGLDataCacheRotationSymbolItem *data
    =new OGLDataCacheRotationSymbolItem(ptype,plength,porder,pmtau0,
					pmu0,pdmu,pradius,pinvradius,
					(pposition1-pposition0),
					psymbolthickness,psymbolwidth,
					pnphi,pshowbottom,pshowtop);
  
  if(data!=(cachedDataNormal=oglview->ogldatacache->append(data))) 
    delete data;
  


  if(OGLRotationSymbol::equalBaseType(ptype,MirrorRotation)&&porder>1){
    if(porder%2==1 || (ptype & AxisWithCenterFlag)){
      OGLDataCacheRotationSymbolItem *data
	=new OGLDataCacheRotationSymbolItem(AxisWithCenterSpecifier,
					    plength,porder,pmtau0,
					    pmu0,pdmu,pradius,pinvradius,
					    (pposition1-pposition0),
					    psymbolthickness,
					    psymbolwidth,
					    pnphi,pshowbottom,pshowtop);
      
      if(data!=(cachedDataNormalSpecifier=oglview->ogldatacache->append(data))) 
	delete data;
    }else{
      OGLDataCacheRotationSymbolItem *data
	=new OGLDataCacheRotationSymbolItem(MirrorRotationSpecifier,
					    plength,porder,pmtau0,
					    pmu0,pdmu,pradius,pinvradius,
					    (pposition1-pposition0),
					    psymbolthickness,
					    psymbolwidth,
					    pnphi,pshowbottom,pshowtop);
      
      if(data!=(cachedDataNormalSpecifier=oglview->ogldatacache->append(data))) 
	delete data;
    }
  }
  

  
      

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLRotationSymbol::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "NFPlane " << id();
  if(OGLRotationSymbol::equalBaseType(ptype,MirrorRotation)&&porder==1){
    near=-pinvradius;
    far= pinvradius;
    return true;
  }

  double p0=pposition0*n;
  double p1=pposition1*n;
  if(p0<p1){ double d=p0;p0=p1;p1=d;}

  double b=(p1-p0)/plength;

  p0+=psymbolthickness;
  p1-=psymbolthickness;

  
  b=sqrt(1.-b*b);
  double a;
  a=MyMath::max<double>(pradius,psymbolwidth)*b;




  near=-MyMath::max<double>(MyMath::max<double>(p0+a,p0-a),MyMath::max<double>(p1+a,p1-a));

  far=-MyMath::min<double>(MyMath::min<double>(p0+a,p0-a),MyMath::min<double>(p1+a,p1-a));

  return true;
}



/*EOF*/ 
