/* @(#)OGLRotationSymbol.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLROTATIONSYMBOL_H
#define _OGLROTATIONSYMBOL_H 1

#include <QGLWidget>
#include "OGLDataCache.hh"
#include "OGLObject.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"



class OGLRotationSymbol: public OGLObject
{
public:
  enum OGLRotationSymbolType{Rotation=0x1,MirrorRotation=0x2,
			     AxisWithCenterFlag=0x10,
			     AxisWithCenter=(MirrorRotation|
					     AxisWithCenterFlag),
			     MirrorRotationSpecifier=0x100,
			     AxisWithCenterSpecifier=0x200};
  
  OGLRotationSymbol(OGLRotationSymbolType type,const Vec3 &pos0,
		    const Vec3 &pos1,long order,long mtau0,
		    double mu0,double dmu,
		    double radius=1.,double invradius=1,
		    long np=6,
		    double symbolthickness=0.1,
		    double symbolwidth=0.2,
		    bool showbottom=true,
		    bool showtop=true
	  );
  virtual ~OGLRotationSymbol();

  bool NFPlane(const Vec3 &n,double &near,double &far);

  
  void paint(OGLView *oglview,const OGLMagnifier &mag
	     ,bool reduced,bool badTransparency)const;
  void construct(OGLView *oglview);
  void protectCachedData();

  static bool equalBaseType(OGLRotationSymbolType a,
			    OGLRotationSymbolType b){
    if( (0xf & a)==(0xf & b)) return true;
    return false;
  }
  
  OGLMaterial &material(){return pmaterial;};
  OGLMaterial &specifierMaterial(){return pspecifiermaterial;};

  bool pick(const View &/*v*/,double /*x*/,double /*y*/,double &z)
  {z=0.;return false;};

  static void setUseDisplayLists(bool u){pusedisplaylists=u;};
  static bool useDisplayLists() {return pusedisplaylists;};


protected:

private:
  OGLRotationSymbolType ptype;
  long pnphi;
  double plength;
  double pradius,pinvradius;
  double psymbolthickness;
  double psymbolwidth;
  long porder;
  long pmtau0;
  double pmu0,pdmu;
  bool pshowbottom;
  bool pshowtop;
  static bool pusedisplaylists;
  Vec3 pposition0,pposition1;
  OGLMaterial pmaterial;
  OGLMaterial pspecifiermaterial;
  OGLDataCacheItem *cachedDataNormal;
  OGLDataCacheItem *cachedDataNormalSpecifier;

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheRotationSymbolItem: public OGLDataCacheItem
{
 public:
  OGLDataCacheRotationSymbolItem(OGLRotationSymbol::OGLRotationSymbolType type,
				 double length,
				 long order,long mtau0,
				 double mu0,double dmu,
				 double radius,double invradius,
				 const Vec3 &axis,
				 double symbolthickness,
				 double symbolwidth,
				 long nphi,
				 bool showbottom,
				 bool showtop
				 );
  
  QString printSpecQ()const;
  
  void construct(bool transparent);

private:
  void constructMarker(long &nvert,long &idef,long &nn0,bool right,bool left,
		       long legskip,long nnonsym,
		       bool top,double L,long lidsubdiv);
  void constructMarkerSpecifier(long &nvert,long &idef,long &nn0,
				bool top,double L,long lidsubdiv);
  void constructMarkerC2(long &nvert,long &idef,long &nn0,bool right,
			 bool top,double L,long NsubC2);
  
 private:
  OGLRotationSymbol::OGLRotationSymbolType ptype;
  double plength;
  double pradius,pinvradius;
  Vec3 paxis;
  double psymbolthickness;
  double psymbolwidth;
  long porder,pmtau0;
  double pmu0,pdmu;
  bool pshowbottom;
  bool pshowtop;
  long pnphi;
  double mu3,muy3;
    
};
#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _OGLROTATIONSYMBOL_H */

