/*************************************************************************
 * @(#)OGLSphere.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   05 Apr 2010
 *************************************************************************
 */

#include <QGLWidget>
#include "OGLDataCache.hh"
#include "OGLTransparentCollector.hh"
#include "OGLSphere.hh"
#include "OGLView.hh"
#include "OGL.hh"
#include "View.hh"
#include "MyMath.hh"


bool OGLSphere::pusedisplaylists=true;

OGLSphereDataCache OGLDataCacheSphereItem::oglspheredatacache;



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLSphereData::construct()
{
  NSF("OGLSphereData::construct");
  //myDebug()<<nsubfunc<<ptype<<pnphi_or_icodepth<<this;
  if(ptype==OGLSphere::Earth){
    
     
    double *phis=new double[2*pnphi_or_icodepth];
    
    { //prepare the cos(phi),sin(phi) array
      double cp=1.;
      double sp=0.;
      double c=cos(2.*Pi/pnphi_or_icodepth);
      double s=sin(2.*Pi/pnphi_or_icodepth);
      phis[0]=cp; 
      phis[1]=sp;
      for(int j=1;j<pnphi_or_icodepth;j++){
	double d=cp;
	cp=cp*c-sp*s;
	sp=sp*c+s*d;
	phis[2*j]=cp;
	phis[2*j+1]=sp;
      }
    }
    
    
    
    //allocate the sphere data
    pindicessize=2*(1+(pntheta-1)*(pnphi_or_icodepth+1));
    pverticessize=(2+(pntheta-1)*pnphi_or_icodepth);
    pindices=new GLuint[pindicessize];
    pnormals=new GLdouble[pverticessize*3];

    // calculate the spehre data
    int nvert=0;
    int nind=0;
    pnormals[nvert*3+0]=0.;
    pnormals[nvert*3+1]=0.;
    pnormals[nvert*3+2]=1.;
    pindices[nind]=0;
    nvert++;
    nind++;
    double c=cos(Pi/pntheta);
    double s=sin(Pi/pntheta);
    
    double ct=1.;
    double st=0.;
    for( int i=1;i<pntheta;i++){
      double d=ct;
      ct=ct*c-st*s;
      st=st*c+s*d;
      for(int j=0;j<pnphi_or_icodepth;j++){
	pnormals[nvert*3+0]=st*phis[2*j];
	pnormals[nvert*3+1]=st*phis[2*j+1];
	pnormals[nvert*3+2]=ct;
	if(i>=2){
	  pindices[nind++]=1+(i-2)*pnphi_or_icodepth+j;
	}
	pindices[nind++]=1+(i-1)*pnphi_or_icodepth+j;
	nvert++;
      }
      if(i>=2){
	pindices[nind++]=1+(i-2)*pnphi_or_icodepth;
      }
      pindices[nind++]=1+(i-1)*pnphi_or_icodepth;
    }
    pnormals[nvert*3+0]=0.;
    pnormals[nvert*3+1]=0.;
    pnormals[nvert*3+2]=-1.;
    nvert++;
    // lower lit
    pindices[nind++]=nvert-1;
    
    for(int j=pnphi_or_icodepth-1;j>=0;j--){
      pindices[nind++]=1+(pntheta-2)*pnphi_or_icodepth+j;
    }
    pindices[nind++]=(pntheta-1)*pnphi_or_icodepth;
    
    
    
    
    
    pndefs=pntheta;
    pdefs=new struct VertexArrayDef[pndefs];
    
    pdefs[0].mode=GL_TRIANGLE_FAN;
    pdefs[0].count=pnphi_or_icodepth+2;
    pdefs[0].start=0;
    pdefs[0].useindex=true;

    for(int i=1;i<pntheta-1;i++){
      pdefs[i].mode=GL_QUAD_STRIP;
      pdefs[i].count=2*(pnphi_or_icodepth+1);
      pdefs[i].start=1+(2*i-1)*(pnphi_or_icodepth+1);
      pdefs[i].useindex=true;
    }
    pdefs[pntheta-1].mode=GL_TRIANGLE_FAN;
    pdefs[pntheta-1].count=pnphi_or_icodepth+2;
    pdefs[pntheta-1].start=1+(2*pntheta-3)*(pnphi_or_icodepth+1);
    pdefs[pntheta-1].useindex=true;
    
    
    delete [] phis;
    
    
    

  
  }else{//Icosahedron
    static const double X=(sqrt(5.)-1)/sqrt(10.-2.*sqrt(5));
    static const double Z=2./sqrt(10.-2.*sqrt(5));
    
    
    static GLuint icoindex[]={
      0,1,4, 1,0,6, 3,2,5, 2,3,7,
      4,5,10, 5,4,8, 6,7,9, 7,6,11,
      1,9,8, 3,8,9, 0,10,11, 2,11,10,
      1,6,9, 4,1,8, 0,4,10, 6,0,11,
      7,3,9, 3,5,8, 5,2,10, 2,7,11 };
    
    static GLdouble icovert[]={
      -X,0.,Z, X,0.,Z, -X,0.,-Z, X,0.,-Z,
      0.,Z,X, 0.,Z,-X, 0.,-Z,X, 0.,-Z,-X,
      Z,X,0., Z,-X,0., -Z,X,0., -Z,-X,0. };
    
    long d4=1; for(int i=1;i<=pnphi_or_icodepth;i++){d4*=4;}//d4=4^icodepth
    
    int nfaces0=sizeof(icoindex)/sizeof(GLuint)/3;
    int nvertices0=sizeof(icovert)/sizeof(GLdouble)/3;
    
    // D  F        V
    // 0  F0       V0
    // 1  4*F0     V0+ 3/2*F0
    // 2  4^2*F0   V0+ 3/2*F0 +3/2*4*F0
    // 3  4^3*F0   V0+ 3/2*F0 +3/2*4*F0 +3/2*4^2*F0
    //
    //
    //    F0*4^D   V0+3/2*F0*sum_0^{D-1} 4^n = V0+3/2*F0*(4^D-1)/3
    
    
    
    int nfaces=(d4*nfaces0);
    int nvertices=(nvertices0+nfaces0/2*(d4-1));
    
    pindicessize=nfaces*3;
    pverticessize=nvertices;
    pindices=new GLuint[pindicessize];
    pnormals=new GLdouble[pverticessize*3];
    
    //     myDebug() << "nfaces0="<<nfaces0;
    //     myDebug() << "nvertices0="<<nvertices0;
    //myDebug() << "nvertices="<<nvertices;
    
    memcpy(pindices,icoindex,sizeof(icoindex));
    memcpy(pnormals,icovert,sizeof(icovert));
    
    int nfac=nfaces0;
    int nverts=nvertices0;
    double tol=1.0e-5;
    
    
    for(int id=1;id<=pnphi_or_icodepth;id++){
      
      int nf=0;
      for(int iface=0;iface<nfac;iface++){
	
	int i0=pindices[iface*3+0];
	int i1=pindices[iface*3+1];
	int i2=pindices[iface*3+2];
	double *v0=pnormals+i0*3;
	double *v1=pnormals+i1*3;
	double *v2=pnormals+i2*3;
	
	double d;
	double n[3];
	d=0.;for(int k=0;k<3;k++) { n[k]=v0[k]+v1[k]; d+=n[k]*n[k]; }
	d=sqrt(d);
	for(int k=0;k<3;k++) n[k]/=d;
	
	
	int i01,i12,i20;
	int nd;
	nd=0;
	for(int j=nverts-1;j>=0;j--){
	  double *v=pnormals+j*3;
	  if( (v[0]-n[0])*(v[0]-n[0])+
	      (v[1]-n[1])*(v[1]-n[1])+
	      (v[2]-n[2])*(v[2]-n[2]) < tol*tol){
	    nd=1;
	    i01=j;
	    break;
	  }
	}
	if(!nd){
	  memcpy(pnormals+nverts*3,n,sizeof(n));
	  i01=nverts;
	  nverts++;
	}
	
	
	d=0.;for(int k=0;k<3;k++) { n[k]=v1[k]+v2[k]; d+=n[k]*n[k]; }
	d=sqrt(d);
	for(int k=0;k<3;k++) n[k]/=d;
	
	nd=0;
	for(int j=nverts-1;j>=0;j--){
	  double *v=pnormals+j*3;
	  if( (v[0]-n[0])*(v[0]-n[0])+
	      (v[1]-n[1])*(v[1]-n[1])+
	      (v[2]-n[2])*(v[2]-n[2]) < tol*tol){
	    nd=1;
	    i12=j;
	    break;
	  }
	}
	if(!nd){
	  memcpy(pnormals+nverts*3,n,sizeof(n));
	  i12=nverts;
	  nverts++;
	}
	
	d=0.;for(int k=0;k<3;k++) { n[k]=v0[k]+v2[k]; d+=n[k]*n[k]; }
	d=sqrt(d);
	for(int k=0;k<3;k++) n[k]/=d;
	
	
	nd=0;
	for(int j=nverts-1;j>=0;j--){
	  double *v=pnormals+j*3;
	  if( (v[0]-n[0])*(v[0]-n[0])+
	      (v[1]-n[1])*(v[1]-n[1])+
	      (v[2]-n[2])*(v[2]-n[2]) < tol*tol){
	    nd=1;
	    i20=j;
	    break;
	  }
	}
	
	if(!nd){
	  memcpy(pnormals+nverts*3,n,sizeof(n));
	  i20=nverts;
	  nverts++;
	}
	
	pindices[iface*3+1]=i01;
	pindices[iface*3+2]=i20;
	
	pindices[(nfac+nf)*3+0]=i01;
	pindices[(nfac+nf)*3+1]=i1;
	pindices[(nfac+nf)*3+2]=i12;
	nf++;
	
	pindices[(nfac+nf)*3+0]=i12;
	pindices[(nfac+nf)*3+1]=i2;
	pindices[(nfac+nf)*3+2]=i20;
	nf++;
	
	pindices[(nfac+nf)*3+0]=i01;
	pindices[(nfac+nf)*3+1]=i12;
	pindices[(nfac+nf)*3+2]=i20;
	nf++;
      }
      nfac+=nf;
      
      
      
    }


    
    
    //myDebug() << "nfac="<<nfac << nfaces;
    
    pndefs=1;
    pdefs=new struct VertexArrayDef[pndefs];
    
    pdefs[0].mode=GL_TRIANGLES;
    pdefs[0].count=nfac*3;
    pdefs[0].start=0;
    pdefs[0].useindex=true;
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheSphereItem::OGLDataCacheSphereItem(double radius
					       ,OGLSphere::OGLSphereType type
					       ,long np_or_depth
					       ,long nt
					       )
  :OGLDataCacheItem("OGLDataCacheSphereItem")
  ,ptype(type),pradius(radius)
  ,pnphi_or_icodepth(np_or_depth),pntheta(nt)
{
  NSF("OGLDataCacheSphereItem::OGLDataCacheSphereItem specified construtor");
  //myDebug() << nsubfunc << this;

  //pcull=false;
  
  String s;
  if(ptype==OGLSphere::Earth){
    s=Sprintf("OGLDataCacheSphereItem:type(%d)radius(%30.6f)"
		"np(%ld)nt(%ld)"
		,ptype,pradius,pnphi_or_icodepth,pntheta
		);
  }else{
    s=Sprintf("OGLDataCacheSphereItem:type(%d)radius(%30.6f)"
		"np(%ld)"
		,ptype,pradius,pnphi_or_icodepth
		);
  }
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());

  if(ptype==OGLSphere::Icosahedron){
    if(pnphi_or_icodepth>4){
      myDebug() << "Warning icosahedron subdivision very large: "<<
	pnphi_or_icodepth;
    }
  }
  //myDebug() << nsubfunc << printSpecQ();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheSphereItem::printSpecQ()const
{
  QString s;
  if(ptype==OGLSphere::Earth){
    s.sprintf("OGLDataCacheSphereItem: type=Earth\n"
	      "               nphi=%ld\n"
	      "               ntheta=%ld\n"
	      ,pnphi_or_icodepth,pntheta);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    s.sprintf("OGLDataCacheSphereItem: type=Icosahedron\n"
	      "               icodepth=%ld\n"
	      ,pnphi_or_icodepth);
    s+=OGLDataCacheItem::printSpecQ();
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheSphereItem::construct(bool transparent)
{
  NSF("OGLDataCacheSphereItem::construct");
  //myDebug() << nsubfunc<<this << transparent;
  
  
  QString osd_id;
  if(ptype==OGLSphere::Icosahedron){
    osd_id=QString().sprintf("Icosahedron %ld",pnphi_or_icodepth);
  }else{
    osd_id=QString().sprintf("Earth %ld %ld",pnphi_or_icodepth,pntheta);
  }
  OGLSphereData *posd=OGLDataCacheSphereItem::oglspheredatacache.find(osd_id);
  if(posd == 0){
    if(ptype==OGLSphere::Icosahedron){
      posd=OGLDataCacheSphereItem::
	oglspheredatacache.insert(osd_id
				  ,new OGLSphereData(OGLSphere::Icosahedron
						     ,pnphi_or_icodepth));
    }else{
      posd=OGLDataCacheSphereItem::
	oglspheredatacache.insert(osd_id
				  ,new OGLSphereData(OGLSphere::Earth
						     ,pnphi_or_icodepth
						     ,pntheta));
    }
    posd->construct();
    //myDebug() << "new posd"<<osd_id<<posd;
  }else{
    //myDebug() << "old posd"<<osd_id<<posd;
  }
  
  
  
  pndefs=posd->pndefs;
  pdefs=new struct VertexArrayDef[pndefs];
  for(int i=0;i<pndefs;i++){
    pdefs[i]=posd->pdefs[i];
  }
  
  pverticessize=posd->pverticessize;
  pindicessize=posd->pindicessize;
  
  pindices=new GLuint[pindicessize];
  pvertices=new GLdouble[pverticessize*3];
  pnormals=new GLdouble[pverticessize*3];
  
  memcpy(pindices,posd->pindices,sizeof(GLuint)*pindicessize);
  memcpy(pnormals,posd->pnormals,sizeof(GLdouble)*pverticessize*3);
  
  for(int i=0;i<pverticessize*3;i++){
    pvertices[i]=pnormals[i]*pradius;
  }
  
  



  if(OGLSphere::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  
  if(pdispListName){
    
    OGLIssueVertexArray(pdispListName
			,pvertices,pnormals,0,pindices,pndefs,pdefs);
    
    
  }

  if(pdispListName && !transparent){
    cleanInternal();
  }
  ogl.reportGLError(nsubfunc);
  
  
  //  myDebug() << nsubfunc  << printSpecQ();
  //  myDebug() << "pdispListName="<<pdispListName;

  OGLDataCacheItem::construct(transparent);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheSphereItem::
collectTransparent(OGLTransparentCollector *tpCollector
		   ,const Vec3 &pos
		   ,const OGLMaterial &mat)
{
  NSF("OGLDataCacheSphereItem::collectTransparent");
  //myDebug() << nsubfunc<<this;


  
  QString osd_id;
  if(ptype==OGLSphere::Icosahedron){
    osd_id=QString().sprintf("Icosahedron %ld",pnphi_or_icodepth);
  }else{
    osd_id=QString().sprintf("Earth %ld %ld",pnphi_or_icodepth,pntheta);
  }
  OGLSphereData *posd=OGLDataCacheSphereItem::oglspheredatacache.find(osd_id);
  if(posd == 0){
    if(ptype==OGLSphere::Icosahedron){
      posd=OGLDataCacheSphereItem::
	oglspheredatacache.insert(osd_id
				  ,new OGLSphereData(OGLSphere::Icosahedron
						     ,pnphi_or_icodepth));
    }else{
      posd=OGLDataCacheSphereItem::
	oglspheredatacache.insert(osd_id
				  ,new OGLSphereData(OGLSphere::Earth
						     ,pnphi_or_icodepth
						     ,pntheta));
    }
    posd->construct();
    //myDebug() << "new posd"<<osd_id<<posd;
  }else{
    //myDebug() << "old posd"<<osd_id<<posd;
  }
  
  
  
  pndefs=posd->pndefs;
  pdefs=new struct VertexArrayDef[pndefs];
  for(int i=0;i<pndefs;i++){
    pdefs[i]=posd->pdefs[i];
  }
  
  pverticessize=posd->pverticessize;
  pindicessize=posd->pindicessize;
  
  pindices=new GLuint[pindicessize];
  pvertices=new GLdouble[pverticessize*3];
  pnormals=new GLdouble[pverticessize*3];
  
  memcpy(pindices,posd->pindices,sizeof(GLuint)*pindicessize);
  memcpy(pnormals,posd->pnormals,sizeof(GLdouble)*pverticessize*3);
  
  for(int i=0;i<pverticessize;i++){
    pvertices[3*i+0]=pnormals[3*i+0]*pradius+pos[0];
    pvertices[3*i+1]=pnormals[3*i+1]*pradius+pos[1];
    pvertices[3*i+2]=pnormals[3*i+2]*pradius+pos[2];
  }

  tpCollector->addObject(pvertices
			 ,pnormals
			 ,0
			 ,GL_FRONT
			 ,mat
			 ,mat
			 ,pindices
			 ,pndefs,pdefs);
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLSphere::OGLSphere(const Vec3 &pos,double radius
		     ,OGLSphereType type,long np_or_depth,long nt)
  :OGLObject("OGLSphere"),ptype(type),pntheta(nt)
  ,pnphi_or_icodepth(np_or_depth)
  ,pradius(radius),pposition(pos)
  ,cachedDataNormal(0)
  ,cachedDataReduced(0)
{
  pmaterial.setFace(GL_FRONT_AND_BACK);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLSphere::~OGLSphere()
{
  NSF("OGLSphere::~OGLSphere");
  //myDebug() << nsubfunc<< "I am dying";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLSphere::protectCachedData()
{
  if(cachedDataReduced)
    cachedDataReduced->increaseUsed();
  if(cachedDataNormal)
    cachedDataNormal->increaseUsed();
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLSphere::paint(OGLView *,const OGLMagnifier &,bool reduced
		      ,bool badTransparency)const
{
  NSF("OGLSphere::paint");
  //myDebug() << nsubfunc;

  

  glTranslated(pposition[0],pposition[1],pposition[2]);
  
  pmaterial.issue(ptransparent && ! badTransparency);
  //  myDebug() << 

  if(reduced){
    cachedDataReduced->paint(ptransparent && ! badTransparency);
  }else{
    cachedDataNormal->paint(ptransparent && ! badTransparency);
  }


  return;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLSphere::construct(OGLView *oglview)
{
  NSF("OGLSphere::construct");
  //myDebug() << nsubfunc<<pposition;

  
  OGLDataCacheSphereItem *data
    =new OGLDataCacheSphereItem(pradius,ptype,pnphi_or_icodepth,pntheta);


  if(data!=(cachedDataNormal=oglview->ogldatacache->append(data,ptransparent)))
    delete data;
  
  
  // make reduced model just in case the user has slow hardware
  npreduced=8;
  ntreduced=5;
  if(ptype==Icosahedron) npreduced=1;

  
  data=new OGLDataCacheSphereItem(pradius,ptype,npreduced,ntreduced);

  if(data!=(cachedDataReduced=oglview->ogldatacache->append(data,ptransparent))) 
    delete data;


  //myDebug() << nsubfunc << "END";
  
  
  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLSphere::collectTransparent(OGLTransparentCollector *tpCollector
				   ,bool reduced)
{
  NSF("OGLSphere::collectTransparent");
  //myDebug() << nsubfunc;
  if(!ptransparent) return;
  
  
  if(reduced){
    ((OGLDataCacheSphereItem*)cachedDataReduced)->
      collectTransparent(tpCollector,pposition,pmaterial);
  }else{
    ((OGLDataCacheSphereItem*)cachedDataNormal)->
      collectTransparent(tpCollector,pposition,pmaterial);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLSphere::NFPlane(const Vec3 &n,double &near,double &far)
{

  double p0=pposition*n;
  
  

  near=-MyMath::max<double>(p0+pradius,p0-pradius);
  far=-MyMath::min<double>(p0+pradius,p0-pradius);

  return true;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLSphere::pick(const View &v,double x,double y,double &z)
{
  double a,b;
  if(v.isPerspective()){
    Vec3 dir=v.x()*x+v.up()*y-v.z()*v.near();
    Vec3 rf=pposition-v.origin_cam_w();
    
    double dd=dir*dir;
    a=dir*rf/dd;
    b=(pradius*pradius-rf*rf)/dd;
  }else{
    Vec3 dir=-v.z();
    Vec3 rf=pposition-v.origin_cam_w()-(v.x()*x+v.up()*y);

    double dd=dir*dir;
    a=dir*rf/dd;
    b=(pradius*pradius-rf*rf)/dd;
  }
  

  double det=b+a*a;
  if(det>=0){
    z=-(a-sqrt(det));
    return true;
  }
  
  //myDebug()<<dir.printQ();

  return false;
}

/*EOF*/ 
