/* @(#)OGLSphere.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */


#include <QGLWidget>
#include "OGLObject.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"

#ifndef _OGLSPHERE_H
#define _OGLSPHERE_H 1



/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLSphere: public OGLObject
{
public:

  enum OGLSphereType{Earth=1,Icosahedron=2};
  
  OGLSphere(const Vec3 &pos,double radius
	    ,OGLSphereType type,long np_or_depth,long nt=4);
  virtual ~OGLSphere();

  bool NFPlane(const Vec3 &n,double &near,double &far);
  
  void paint(OGLView *oglview,const OGLMagnifier &mag
	     ,bool reduced,bool badTransparency)const;
  void construct(OGLView *oglview);
  void protectCachedData();
  void collectTransparent(OGLTransparentCollector *,bool );
  
  const Vec3 &position()const{return pposition;};

  OGLMaterial &material(){return pmaterial;};
  const OGLMaterial &material()const{return pmaterial;};

  bool pick(const View &v,double x,double y,double &z);

  static void setUseDisplayLists(bool u){pusedisplaylists=u;};
  static bool useDisplayLists() {return pusedisplaylists;};

protected:


private:
  OGLSphereType ptype;
  long pntheta,pnphi_or_icodepth;
  static bool pusedisplaylists;
  int npreduced,ntreduced;
  double pradius;
  Vec3 pposition;
  OGLMaterial pmaterial;
  OGLDataCacheItem *cachedDataNormal;
  OGLDataCacheItem *cachedDataReduced;
 
 
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLSphereData{
 public:
  OGLSphereData(OGLSphere::OGLSphereType t,long depth,long ntheta=1) 
    :ptype(t),pnphi_or_icodepth(depth),pntheta(ntheta)
    ,pnormals(0),pindices(0),pdefs(0){
    //printf("create OGLSphereData %p\n",this);
  };
  ~OGLSphereData(){
    //printf("deleting OGLSphereData %p\n",this);
    if(pnormals) delete [] pnormals;
    if(pindices) delete [] pindices;
    if(pdefs) delete [] pdefs;
  }
  void construct();
 public:
  OGLSphere::OGLSphereType ptype;
  long pnphi_or_icodepth,pntheta;
  QString pid;
  long pverticessize; 
  long pindicessize; 
  GLdouble *pnormals; 
  GLuint *pindices; 
  long pndefs;
  struct VertexArrayDef *pdefs;

    
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLSphereDataCache{
 public:
  OGLSphereDataCache(){};
  ~OGLSphereDataCache(){
    foreach (OGLSphereData *osd,oglspheredatamap){
      delete osd;
    }
  };
  
  OGLSphereData *insert(const QString &id,OGLSphereData *osd){
    QMap<QString,OGLSphereData*>::iterator iosd
      =oglspheredatamap.insert(id,osd);
    return iosd.value();
  }

  OGLSphereData *find(const QString &id){
    QMap<QString,OGLSphereData*>::iterator iosd
      =oglspheredatamap.find(id);
    if(iosd==oglspheredatamap.end())
      return 0;
    else
      return iosd.value();
  };

 private:
  QMap<QString,OGLSphereData*> oglspheredatamap;
  
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheSphereItem: public OGLDataCacheItem
{
 public: 
  OGLDataCacheSphereItem(double radius
			 ,OGLSphere::OGLSphereType type
			 ,long np_or_depth,long nt
			 );
  
  
  QString printSpecQ()const;
  
  void construct(bool transparent);
  void collectTransparent(OGLTransparentCollector *
			  ,const Vec3 &pos
			  ,const OGLMaterial &mat);

 private:
  OGLSphere::OGLSphereType ptype;
  double pradius;
  long pnphi_or_icodepth,pntheta;

  static OGLSphereDataCache oglspheredatacache;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLSPHERE_H */

