/*************************************************************************
 * @(#)OGLText.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   20 May 2010
 *************************************************************************
 */

#include <QPainter>
#include "OGLView.hh"
#include "OGLText.hh"
#include "View.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"


bool OGLText::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheTextItem::OGLDataCacheTextItem(const QString &s,int height
					   ,int fontres,const Vec3 &fs
					   ,QRgb textcolor
					   ,bool antialias
					   ,OGLText::OGLTextType type)
  :OGLDataCacheItem("OGLDataCacheTextItem")
  ,ptype(type),ptext(s),pheight(height),pfontres(fontres)
  ,pfshift(fs),pantialias(antialias),ptextcolor(textcolor)
{
  NSF("OGLDataCacheTextItem::OGLDataCacheTextItem specified construtor");
  //myDebug() << nsubfunc << this;

  pcull=ptype&OGLText::CameraFacing;

  String ss;
  if(
     (ptype&OGLText::CoMoving)||
     (ptype&OGLText::CameraFacing)
     ){
    ss=Sprintf("OGLDataCacheTextItem:type(%d)text(%s)"
		"fontres(%d)fshift(%30.6f,%30.6f,%30.6f)antialias(%d)"
		,ptype,ptext.toAscii().constData(),pfontres
		,fs[0],fs[1],fs[2],pantialias
		);
  }else if(
     ptype==OGLText::TwoD
     ){
    ss=Sprintf("OGLDataCacheTextItem:type(%d)text(%s)"
		"height(%d)textcolor(%x)fshift(%30.6f,%30.6f,%30.6f)"
		"antialias(%d)"
		,ptype,ptext.toAscii().constData(),pheight,ptextcolor
		,fs[0],fs[1],fs[2],pantialias
		);
  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
  if(pid) delete [] pid;
  pid=new char [ss.length()+1];
  strcpy(pid,ss());

  //myDebug() << nsubfunc << printSpecQ();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheTextItem::printSpecQ()const
{
  NSF("OGLDataCacheTextItem::printSpecQ");
  QString s;
  if(
     
     (ptype&OGLText::CoMoving)||
     (ptype&OGLText::CameraFacing)
     ){    
    s.sprintf("OGLDataCacheTextItem:   type=%s%s\n"
	      "                          text=%s\n"
	      "                          fontres=%d\n"
	      "                          fshift=%g %g %g\n"
	      "                          antialias=%d\n"
	      ,(ptype&OGLText::CoMoving?"CoMoving":"")
	      ,(ptype&OGLText::CameraFacing?"CameraFacing":"")
	      ,ptext.toAscii().constData()
	      ,pfontres,pfshift[0],pfshift[1],pfshift[2],pantialias);
    s+=OGLDataCacheItem::printSpecQ();
  }else if(
     	   (ptype==OGLText::TwoD)
	   ){    
    s.sprintf("OGLDataCacheTextItem:   type=TwoD\n"
	      "                          text=%s\n"
	      "                          height=%d\n"
	      "                          image=%dx%d\n"
	      "                          textcolor=%x\n"
	      "                          fshift=%g %g %g\n"
	      "                          antialias=%d\n"
	      ,ptext.toAscii().constData(),pheight,pimageNx,pimageNy
	      ,ptextcolor,pfshift[0],pfshift[1],pfshift[2],pantialias);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

static QImage *QStringToQImage(const QString &text
			       ,int height
			       ,int fontres,int &Nx,int &Ny
			       ,bool antialias,QRgb fillrgb,QRgb penrgb)
{
  QFont *font=0;
  QString label=text.trimmed();
  QString fontname="Helvetica";
  if(label[0]=='$' && label[1]=='~'){
    label=label.mid(2,-1);
    fontname="Symbol";
  }
  //myDebug("LABEL <%s>",label.toAscii().constData());
  int fres;
  QRect rect;
  if(fontres==0){
    fres=lRound(height*0.6);
  }else{
    fres=fontres;
  }
  font=new QFont(fontname, fres , QFont::Normal, false );
  //myDebug() << QFontInfo(*font).family();
  font->setStyleStrategy((antialias?QFont::PreferAntialias:
			  QFont::NoAntialias));
  QFontMetrics fm(*font);
  rect=fm.boundingRect(label);
  
  //myDebug() << "rect" << rect;

//  todo: find out how to obtaine proper fontsize and then scale properly
//     todo: scaleing when W>H W<H
//     todo: print

  QImage *qi=0;
  
  for(int i=0;i<2;i++){
    Nx=rect.width();
    Ny=rect.height();
    if(qi) delete qi;
    qi=new QImage(Nx,Ny,QImage::Format_ARGB32);
    qi->fill(fillrgb);
    QPainter *wp=new QPainter();
    if(!wp->begin(qi)){
      printf("cannot initialize painter?\nabort\n");
      exit(0);
    }
    wp->setRenderHint(QPainter::HighQualityAntialiasing,antialias);
    wp->setRenderHint(QPainter::TextAntialiasing,antialias);
    wp->setPen(QPen(QColor(penrgb),1,Qt::SolidLine));
    wp->setFont ( *font );
    wp->drawText( 0,0,Nx,Ny,Qt::AlignCenter,label ,&rect);
//     QRect innerrect2=rect;
//     innerrect2.adjust(1,1,-1,-1);
//     wp->drawRect(innerrect2);
    //myDebug() << "rect" << rect;
    wp->end();
    delete wp;
  }
  


  delete font;
  return qi;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static int expandPoly(int Nx,int Ny,unsigned char c
		      ,unsigned char *check,  long po[4],int vert)
{
  int ix0=po[0] % (Nx+1);
  int ix1=po[1] % (Nx+1);
  int iy0=(po[0]-ix0)/(Nx+1);
  int iy1=(po[3]-ix0)/(Nx+1);
  if(!vert){
    if(ix1>=Nx) return 0;

    for(int j=iy0;j<iy1;j++)
      if(check[ix1+j*Nx]!=c) return 0;
      
    
    po[1]++;
    po[2]++;
    for(int j=iy0;j<iy1;j++) check[ix1+j*Nx]=0xff;
    return 1;
    
  }else{
  if( iy1>=Ny) return 0;

    for(int i=ix0;i<ix1;i++) 
      if(check[i+iy1*Nx]!=c) return 0;

    po[2]+=Nx+1;
    po[3]+=Nx+1;
    for(int i=ix0;i<ix1;i++) check[i+iy1*Nx]=0xff;
    return 1;
    
  }
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheTextItem::construct(bool transparent)
{
  NSF("OGLDataCacheTextItem::construct");
  //myDebug() << nsubfunc;
  
  if(ptype==OGLText::TwoD){

    //myDebug() << nsubfunc << pheight;

    QImage *qi=QStringToQImage(ptext,pheight,pfontres,pimageNx,pimageNy,pantialias
			       ,0x00000000,ptextcolor);
    //myDebug("color %x",ptextcolor);
    //myDebug() << "pimageNx,pimageNy"<<pimageNx << pimageNy;
    
    QImage qii=qi->scaledToHeight(pheight/**mag.factor*/,Qt::SmoothTransformation);

    
    pimageNx=qii.width();
    pimageNy=qii.height();
    //myDebug() << pimageNx << pimageNy;


    pimage=new GLubyte[pimageNy*pimageNx*4];
    for(int j=0;j<pimageNy;j++){
      for(int i=0;i<pimageNx;i++){
 	QRgb rgb=qii.pixel(i,j);
 	int ind=4*(i+pimageNx*(pimageNy-1-j));
 	pimage[ind+0]=(GLubyte)qRed(rgb);
 	pimage[ind+1]=(GLubyte)qGreen(rgb);
 	pimage[ind+2]=(GLubyte)qBlue(rgb);
 	pimage[ind+3]=(GLubyte)qAlpha(rgb);
      }
    }
    
    delete qi;


    ogl.reportGLError(nsubfunc);
    
    //myDebug() << printSpecQ();
    
  }else{
    
    pndefs=1;
    pdefs=new struct VertexArrayDef[pndefs];
    
    int Nx,Ny;
    QImage *qi=QStringToQImage(ptext,0,pfontres,Nx,Ny,pantialias
			       ,0xffffffff,0xff000000);
    
    pverticessize=((Nx+1)*(Ny+1));
    pvertices=new GLdouble[pverticessize*3];
    //pcolors=new GLfloat[pverticessize*4];
    
    
    unsigned char *check=new unsigned char[Ny*Nx];
    for (int iy=0; iy<Ny; iy++) {
      int iyy=iy*Nx;
      for (int ix=0; ix<Nx; ix++) {
	if(!qi->valid(ix,iy)){
	  printf("image position invalid?\nabort\n");
	  exit(0);
	}
	QRgb ind=qi->pixel(ix,iy);
	//      if(ix==0 || ix==Nx-1 || iy==0 || iy==Ny-1) ind=0;
	if(ind!=0xffffffff){
	  check[ix+iyy]=qGray(ind);
	}else{
	  check[ix+iyy]=0xff;
	}
      }
    }
    
    long po[4];
    int nvert=0;
    for (int iy=0; iy<Ny; iy++) {
      for (int ix=0; ix<Nx; ix++) {
	unsigned char c=check[ix+iy*Nx];
	//      printf(" %02x",c);
	if(c!=0xff){
	  po[0]=ix+(Nx+1)*iy;
	  po[1]=ix+1+(Nx+1)*iy;
	  po[2]=ix+1+(Nx+1)*(iy+1);
	  po[3]=ix+(Nx+1)*(iy+1);
	  //printf("ix(%d) iy(%d)\n",ix,iy);
	  int ret=1;
	  while(ret){
	    ret=expandPoly(Nx,Ny,c,check,po,0);
	    ret|=expandPoly(Nx,Ny,c,check,po,1);
	  }
	  for(int k=0;k<4;k++){
	    int px=po[3-k]%(Nx+1);
	    int py=po[3-k]/(Nx+1);
	    pvertices[nvert*3+0]=1.0*(px)/pfontres;
	    pvertices[nvert*3+1]=1.0*(Ny-py)/pfontres;
	    pvertices[nvert*3+2]=pfshift[2];
	    // 	  pcolors[nvert*4+0]=c/255.;
	    // 	  pcolors[nvert*4+1]=c/255.;
	    // 	  pcolors[nvert*4+2]=c/255.;
	    // 	  pcolors[nvert*4+3]=c/255.;
	    nvert++;
	  }
	  
	}
      }
      //    printf("\n");
    }
    delete [] check;
    //myDebug() << "setting W/H"<<Nx<<Ny;
    pboxheight=1.*Ny/pfontres;
    pboxwidth=1.*Nx/pfontres;

    
    pverticessize=nvert;
    GLdouble *tmp=new GLdouble[nvert*3];
    for(long i=0;i<nvert*3;i++){
      tmp[i]=pvertices[i];
    }
    delete [] pvertices;
    pvertices=tmp;
    
    // {
    //     GLfloat *tmp=new GLfloat[nvert*4];
    //     for(long i=0;i<nvert*4;i++){
    //       tmp[i]=pcolors[i];
    //     }
    //     delete [] pcolors;
    //     pcolors=tmp;
    //   }
    
    pcolors=0;
    
    delete qi;
    
    pdefs[0].mode=GL_QUADS;
    pdefs[0].count=nvert;
    pdefs[0].start=0;
    pdefs[0].useindex=false;
    
    if(OGLText::useDisplayLists()){
      pdispListName=glGenLists(1);
    }else{
      pdispListName=0;
    }
    if(pdispListName){
      
      OGLIssueVertexArray(pdispListName
			  ,pvertices,0,0,pindices,pndefs,pdefs);
      
    }
    
    
    ogl.reportGLError(nsubfunc);
    
    //myDebug() << nsubfunc  ;
    //myDebug() << printSpecQ();
    
    if(pdispListName) cleanInternal();

  }
    
  OGLDataCacheItem::construct(transparent);
    
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLText::OGLText(const QString &s,const Vec3 position,double scale,double scale2
		 ,int fontres,const Vec3 &fs,bool antialias
		 ,OGLTextType type)
  :OGLObject("OGLText")
  ,ptype(type),ptext(s),pfontres(fontres),pfshift(fs),pposition(position)
  ,pscale(scale),pscale2(scale2),pantialias(antialias)
  ,cachedData(0)
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLText::~OGLText()
{
  NSF("OGLText::~OGLText");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLText::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLText::paint(OGLView *oglview
		    ,const OGLMagnifier &mag,bool reduced,bool // badTransparency
		    )const
{
  NSF("OGLText::paint");
  Q_UNUSED(reduced);
  //myDebug() << nsubfunc ;

  if(ptype == OGLText::TwoD){

    if(mag.factor==1){
      //int height=lRound(oglview->height()*pscale);
      //      long fres=0;
      //      if(pfontres>0)
	//fres=pfontres;

      
      
      
      if(!cachedData){
	myDebug() << nsubfunc << "My data are zero?";
	exit(1);
      };
      OGLDataCacheTextItem *item
 	=dynamic_cast<OGLDataCacheTextItem*>(cachedData);
      if(!item){
 	myDebug() << nsubfunc << "My data are of the wrong type?";
 	exit(1);
      };
      
      glDisable(GL_LIGHTING);
      glDisable(GL_DEPTH_TEST);
      
      glMatrixMode(GL_PROJECTION);
      glPushMatrix();
      glLoadIdentity();
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      glLoadIdentity();
      
      
      double fx=pfshift[0]*item->imageWidth()*2./oglview->width();
      double fy=pfshift[1]*item->imageHeight()*2./oglview->height();
      
      double px=pposition[0]*2.-1;
      double py=pposition[1]*2.-1;
      
      double dx=MyMath::max<double>(0.,px+fx+item->imageWidth()*2./oglview->width()-1);
      
      glRasterPos2d(MyMath::max<double>(-1.,px+fx-dx),MyMath::max<double>(-1.,py+fy));
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

      cachedData->paint(ptransparent);
      
      glDisable(GL_BLEND);
      
      glEnable(GL_DEPTH_TEST);
      glMatrixMode(GL_PROJECTION);
      glPopMatrix();  
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();  
      
      glEnable(GL_LIGHTING);
      
    }
  }else{
    
    
    
    glDisable(GL_COLOR_MATERIAL);
    
    
    glPushMatrix();
    
    
    glTranslated(pposition[0],pposition[1],pposition[2]);
    glScaled(pscale,pscale,pscale);
    if(ptype&OGLText::CameraFacing){
      glMultMatrixd(oglview->view().matrix().transposed()());
    }
    glTranslated(pfshift[0]/pscale2,pfshift[1]/pscale2,0);
    
    
    
    glShadeModel(GL_FLAT);
    glDisable(GL_LIGHTING);
    GLfloat rgb[4]; 
    glColor4fv(GLfloat4FromQRgb(rgb,pmaterial.diffuse()));

    
    if(cachedData)cachedData->paint(ptransparent);
    
    glPopMatrix();  
    
    glEnable(GL_LIGHTING);
    glShadeModel(GL_SMOOTH);
    
  }

  return;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLText::construct(OGLView *oglview)
{
  NSF("OGLText::construct");
  //myDebug() << nsubfunc;

  int fres=0;
  int height=0;
  if(ptype == OGLText::TwoD){
    height=lRound(oglview->height()*pscale);
    if(pfontres>0)
      fres=pfontres;
    //myDebug() << "oglview height "<<oglview->height() << "height"<<height;
  }else{
    fres=pfontres;
  }

  OGLDataCacheTextItem *data
    =new OGLDataCacheTextItem(ptext,height,fres,pfshift,pmaterial.diffuse()
			      ,pantialias,ptype);
  
  if(data!=(cachedData=oglview->ogldatacache->append(data))) 
    delete data;

 

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLText::overpaint(QImage *image)const
{
  if(ptype != OGLText::TwoD) return;

  int fres=0;
  int height=lRound(image->height()*pscale);
  if(pfontres>0)
    fres=pfontres;


  int pimageNx,pimageNy;
  QImage *qi=QStringToQImage(ptext,height,fres,pimageNx,pimageNy,pantialias
			     ,0x00000000,pmaterial.diffuse());
  

  QRect rect=qi->rect();
  double qiscale=1.*height/rect.height();

  double Nx=pimageNx*qiscale;
  double Ny=pimageNy*qiscale;
  
  double fx=pfshift[0]*Nx*2./image->width();
  double fy=pfshift[1]*Ny*2./image->height();
  
  double px=pposition[0]*2.-1;
  double py=pposition[1]*2.-1;
  
  double dx=MyMath::max<double>(0.,px+fx+Nx*2./image->width()-1);

  double x=(MyMath::max<double>(-1.,px+fx-dx)+1)*0.5*image->width();
  double y=(1-MyMath::max<double>(-1.,py+fy))*0.5*image->height()-Ny;
  
  QRectF target(x,y,qiscale*rect.width(),qiscale*rect.height());



  QPainter *wp=new QPainter();
  if(!wp->begin(image)){
    printf("cannot initialize painter on image?\nabort\n");
    exit(0);
  }
  wp->drawImage(target,*qi,rect);
  wp->end();
  
  delete wp;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLText::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLText::NFPlane"<<pfshift[2];


  double p0=(pposition)*n+pfshift[2]*pscale;

  //myDebug()<<-p0;
  near=-p0;
  far=-p0;

  if(ptype == OGLText::TwoD)
    return false;
  else
    return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLText::pick(const View &v,double x,double y,double &z)
{
  if(ptype!=CoMovingCameraFacing) return false;

  double W=((OGLDataCacheTextItem*)cachedData)->boxWidth();
  double H=((OGLDataCacheTextItem*)cachedData)->boxHeight();

  double a,mu,la;
  
  if(v.isPerspective()){
    Vec3 dir=v.x()*x+v.up()*y-v.z()*v.near();
    Vec3 rf=pposition-v.origin_cam_w();
    
    Matrix3x3 M(W*pscale*v.x(),H*pscale*v.up(),-dir);
    Vec3 b=-(rf+pscale*(pfshift[0]/pscale2*v.x()+pfshift[1]/pscale2*v.up()+pfshift[2]*v.z()));

    b=M.inverse()*b;
    la=b[0];
    mu=b[1];
    z=-b[2];
  }else{
    Vec3 rf=pposition-v.origin_cam_w()-(v.x()*x+v.up()*y);
    mu=-(rf*v.up()+pscale/pscale2*pfshift[1])/H/pscale;
    la=-(rf*v.x()+pscale/pscale2*pfshift[0])/W/pscale;
    a=rf*(-v.z());
    z=-a+pfshift[2]*pscale;
  }
  if(0<=la && la<=1 && 0<=mu && mu<=1){
    return true;
  }


  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLText::shiftInViewCoords(double dvx,double dvy)
{
  NSF("OGLText::shiftInViewCoords");
  //  myDebug() << nsubfunc;
  // dvx and dvy are in view coordinates and hence refer to view.x()
  // and view.up()

  
  pfshift[0]+=dvx/pscale*pscale2;
  pfshift[1]+=dvy/pscale*pscale2;
}



/*EOF*/ 
