/* @(#)OGLText.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */


#include <QGLWidget>
#include "OGLDataCache.hh"
#include "OGLObject.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"

#ifndef _OGLTEXT_H
#define _OGLTEXT_H 1




/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLText: public OGLObject
{
public:

  enum OGLTextType{CoMoving=1,CameraFacing=2,CoMovingCameraFacing=3,TwoD=4};
  
  OGLText(const QString &s,const Vec3 position,double scale,double scale2
	  ,int fontres,const Vec3 &fs
	  ,bool antialias
	  ,OGLTextType type);
  virtual ~OGLText();

  bool NFPlane(const Vec3 &n,double &near,double &far);
  
  void paint(OGLView *oglview,const OGLMagnifier &mag
	     ,bool reduced,bool badTransparency)const;
  void construct(OGLView *oglview);
  void overpaint(QImage *image)const;
  void protectCachedData();

  void shiftInViewCoords(double,double);
  const Vec3 &getShift()const{return pfshift;};

  OGLMaterial &material(){return pmaterial;};

  bool pick(const View &,double ,double ,double &z);
  
  static void setUseDisplayLists(bool u){pusedisplaylists=u;};
  static bool useDisplayLists() {return pusedisplaylists;};

protected:


private:
  OGLTextType ptype;
  QString ptext;
  int pfontres;
  Vec3 pfshift;
  Vec3 pposition;
  double pscale,pscale2;
  static bool pusedisplaylists;
  OGLMaterial pmaterial;
  bool pantialias;
  OGLDataCacheItem *cachedData;
 
};


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheTextItem: public OGLDataCacheItem
{
 public: 
  OGLDataCacheTextItem(const QString &s,int height
		       ,int fontres,const Vec3 &fs
		       ,QRgb textcolor
		       ,bool antialias
		       ,OGLText::OGLTextType type
		       );
  
/*   ~OGLDataCacheTextItem(){}; */
  
  double boxWidth()const{return pboxwidth;};
  double boxHeight()const{return pboxheight;};
  
  QString printSpecQ()const;
  
  void construct(bool transparent);

 private:
  OGLText::OGLTextType ptype;
  QString ptext;
  int pheight;
  int pfontres;
  Vec3 pfshift;
  bool pantialias;
  QRgb ptextcolor;
  double pboxwidth,pboxheight;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGTELXT_H */

