/*************************************************************************
 * @(#)OGLTransparentCollector.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   13 Aug 2010
 *************************************************************************
 */

#include "OGLTransparentCollector.hh"
#include "OGLMaterial.hh"
#include "OGLBSPTree.hh"
#include "Plane.hh"
#include "ValueArray.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLTransparentCollector::OGLTransparentCollector()
  :pneedreconstruct(false),bsptree(new OGLBSPTree())
{
  //  myDebug() << "TREEROOT"<<bsptree;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::reset(const ValueArray<ValueArray<GLdouble> > &seedplanes)
{
  NSF("OGLTransparentCollector::reset()");
  //myDebug() << nsubfunc<<"seedplanes"<<seedplanes.size();

  clean();

  
  bsptree->initialSeed(seedplanes);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::addObject(
					const GLdouble *pvertices
					,const GLdouble *pnormals
					,const GLfloat *pcolors
					,GLenum colormode
					,const OGLMaterial &front
					,const OGLMaterial &back
					,const GLuint *pindices
					,long ndefs
					,const struct VertexArrayDef *defs 
					)
{
  NSF("OGLTransparentCollector::addObject()");
  //myDebug() << nsubfunc;

  OGLPrimitives *prim;  
  for(int id=0;id<ndefs;id++){
    if(defs[id].mode==GL_TRIANGLES){
      if(defs[id].useindex){
	for(int i=0;i<defs[id].count/3;i++){
	  int ind=i*3+defs[id].start;

	  pdata.append(prim=new OGLPrimitives(GL_TRIANGLES));
	  
	  prim->setMaterial(colormode,front,back);
	  prim->setVertices(pvertices,pindices+ind);
 	  prim->setNormals(pnormals,pindices+ind);
 	  prim->setColors(pcolors,pindices+ind);
	}

      }else{
	myDebug() << "use no index not implemented yet\n";
	exit(1);
      }
    }else if(defs[id].mode==GL_TRIANGLE_FAN){
      if(defs[id].useindex){
	int N=defs[id].count-1;
	GLuint indices[3];
	indices[0]=pindices[0+defs[id].start];
	for(int i=1;i<N;i++){
	  indices[1]=pindices[i+defs[id].start];
	  indices[2]=pindices[i+1+defs[id].start];
	  
	  pdata.append(prim=new OGLPrimitives(GL_TRIANGLES));
	  prim->setMaterial(colormode,front,back);
	  prim->setVertices(pvertices,indices);
	  prim->setNormals(pnormals,indices);
	  prim->setColors(pcolors,indices);
	}
	
      }else{
	myDebug() << "use no index not implemented yet\n";
	exit(1);
      }

    }else if(defs[id].mode==GL_QUAD_STRIP){
      if(defs[id].useindex){
	int N=defs[id].count/2-1;
	GLuint indices[4];
	for(int i=0;i<N;i++){
	  indices[0]=pindices[i*2+0+defs[id].start];
	  indices[1]=pindices[i*2+1+defs[id].start];
	  indices[2]=pindices[i*2+3+defs[id].start];
	  indices[3]=pindices[i*2+2+defs[id].start];
	  
	  pdata.append(prim=new OGLPrimitives(GL_QUADS));
	  prim->setMaterial(colormode,front,back);
	  prim->setVertices(pvertices,indices);
	  prim->setNormals(pnormals,indices);
	  prim->setColors(pcolors,indices);
	  
	}

      }else{
	myDebug() << "use no index not implemented yet\n";
	exit(1);
      }
    }else if(defs[id].mode==GL_POLYGON){
      if(!defs[id].useindex){
	//	int N=defs[id].count;
	//for(int i=0;i<N;i++){
	  
	pdata.append(prim=new OGLPrimitives(GL_POLYGON,defs[id].count));
	prim->setMaterial(colormode,front,back);
	prim->setVerticesNoIndex(pvertices,defs[id].start);
	prim->setNormalsNoIndex(pnormals,defs[id].start);
	prim->setColorsNoIndex(pcolors,defs[id].start);
	
	//	}

      }else{
	myDebug() << "use of index not implemented yet\n";
	exit(1);
      }
    }else{
      myDebug() << nsubfunc << "GL mode"<<defs[id].mode<<"not implemented yet\n";
      exit(1);
    }
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::addObjecttoTree(const GLdouble *pvertices
					      ,const GLdouble *pnormals
					      ,const GLfloat *pcolors
					      ,GLenum faces
					      ,const OGLMaterial *front
					      ,const OGLMaterial *back
					      ,const GLuint *pindices
					      ,long ndefs
					      ,const struct VertexArrayDef *defs)
{
  NSF("OGLTransparentCollector::addObjecttoTree");
  //OGLBSPTree::countsplitenter=0;

  //myDebug()<<"defs[0].count="<<defs[0].count/3;

  for(int id=0;id<ndefs;id++){
    if(defs[id].mode==GL_TRIANGLES){
      if(defs[id].useindex){
	for(int i=0;i<defs[id].count/3;i++){
	  //int ind=i*3+defs[id].start;

	  
	  myDebug() << nsubfunc << "GL_TRIANGLES, !usindex not implemented yet\n";
	  exit(1);
	  
	  // OGLPolygon *pol=new OGLPolygon(3,pvertices,pindices+3*ind,pnormals+3*ind);
	  // pol->setMaterial(front,back);
	  // bsptree->addToTree(pol);
	}
      }else{
	//
	// N=n*m+r //like m small
	//
	// do k=0,n-1
	//   do j=0,m-1
	//      i=k+j*n
	// j=0: [0,n-1]
	// j=1: [n,2*n-1]
	// j=2: [2*n,3*n-1]
	// j=m-1: [(m-1)*n,m*n-1)]
	//
	// do j=0,r
	//    i=n*m+j
	//for(int i=0;i<defs[id].count/3;i++){
	long N=defs[id].count/3;
	
	long m=10;//N/100;
	long r=(N%m);
	long n=N/m;
	//myDebug()<<"N="<<N;
	//myDebug()<<"m="<<m;
	//myDebug()<<"n="<<n;
	//myDebug()<<"r="<<r;
	//if(n*m+r!=N){myDebug()<<"mod error";exit(1);}
	for(int k=-1;++k<n;){
	  for(int j=-1;++j<m;){
	    long i=k+j*n;
	    int ind=i*3+defs[id].start;
	    OGLPolygon *pol=new OGLPolygon(3,pvertices+ind*3,true,
					   (pnormals?pnormals+ind*3:0),
					   (pcolors?pcolors+ind*4:0),faces);
	    pol->setMaterial(front,back);
	    bsptree->addToTree(pol);
	    
	  }
	}
	for(int k=-1;++k<r;){
	  long i=k+m*n;
	  int ind=i*3+defs[id].start;
	  OGLPolygon *pol=new OGLPolygon(3,pvertices+ind*3,true,
					 (pnormals?pnormals+ind*3:0),
					 (pcolors?pcolors+ind*4:0),faces);
	  pol->setMaterial(front,back);
	  bsptree->addToTree(pol);
	  
	}
      }

    }else{ 
      myDebug() << nsubfunc << "GL mode"<<defs[id].mode<<"not implemented yet\n";
      exit(1);
    }//if defs.mode

    
  }//for all defs
  //myDebug()<<"OGLBSPTree::countsplitenter="<<OGLBSPTree::countsplitenter;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLTransparentCollector::~OGLTransparentCollector()
{
  NSF("OGLTransparentCollector::~OGLTransparentCollector");
  //myDebug() << nsubfunc << "I die";
  if(bsptree) delete bsptree; bsptree=0;
  clean();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::clean()
{
  //myDebug()<< "OGLTransparentCollector::clean";
  qDeleteAll(pdata.begin(), pdata.end());
  if(bsptree) bsptree->clear();
  pdata.clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::paint(const Vec3 &eye,const Vec3 &z,bool persp)
{
  NSF("void OGLTransparentCollector::paint");
  //myDebug() << nsubfunc;
  glEnable(GL_BLEND);
  glDepthMask(GL_FALSE);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //  //glBlendFunc(GL_ONE, GL_ONE);
  //  //glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
  for (int i = 0; i < pdata.size(); ++i) {
    pdata[i]->paint();
  }  
  //myDebug() << "OGLTransparentCollector::paint GL_";

  //OGLBSPTree::countfrontrender=0;
  //OGLBSPTree::countbackrender=0;
  //OGLBSPTree::countprim=0;
  bsptree->backToFrontRender(eye,z,persp);
  glDepthMask(GL_TRUE);
  glDisable(GL_BLEND);
  //myDebug()<<"OGLBSPTree::countfrontrender"<<OGLBSPTree::countfrontrender;
  //myDebug()<<"OGLBSPTree::countbackrender"<<OGLBSPTree::countbackrender;
  //myDebug()<<"OGLBSPTree::countprim"<<OGLBSPTree::countprim;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTransparentCollector::sort(const Vec3 &z)
{
  //myDebug()<< "OGLTransparentCollector::sort prepare";
  for (int i = 0; i < pdata.size(); ++i) {
    pdata[i]->setZ(z());
  }
  //myDebug()<< "OGLTransparentCollector::sort qsort";
  qSort(pdata.begin(),pdata.end(),OGLPrimitives::lessThan);
  //myDebug()<< "OGLTransparentCollector::sort done";

}


/*EOF*/ 
