/*************************************************************************
 * @(#)OGLTriangleMesh.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   04 May 2010
 *************************************************************************
 */

#include "OGLTransparentCollector.hh"
#include "OGLTriangleMesh.hh"
#include "OGLView.hh"
#include "TriangleMesh.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheTriangleMeshItem::
OGLDataCacheTriangleMeshItem(OGLTriangleMesh::OGLTriangleMeshType type
			     ,const TriangleMesh *mesh
			     ,const OGLMaterial &front
			     ,const OGLMaterial &back
			     )
  :OGLDataCacheItem("OGLDataCacheTriangleMeshItem")
  ,ptype(type),pmesh(mesh),pmaterialfront(front),pmaterialback(back)
{
  NSF("OGLDataCacheTriangleMeshItem::OGLDataCacheTriangleMeshItem");
  pdispListName=0;
  pcull=false;
  
  if(ptype==OGLTriangleMesh::Regular){
    String s=Sprintf("OGLDataCacheTriangleMeshItem:type(%d)mesh(%p)"
		,ptype,pmesh
		);
    if(pid) delete [] pid;
    pid=new char [s.length()+1];
    strcpy(pid,s());

  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheTriangleMeshItem::printSpecQ()const
{
  NSF("OGLDataCacheTriangleMeshItem::printSpecQ");
  QString s;
  if(ptype==OGLTriangleMesh::Regular){
    
    s.sprintf("OGLDataCacheTriangleMeshItem:   type=Regular\n"
	      "                              mesh=%p\n",pmesh);
    s+=OGLDataCacheItem::printSpecQ();
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheTriangleMeshItem::construct(bool transparent)
{
  NSF("OGLDataCacheTriangleMeshItem::construct");
  //myDebug() << nsubfunc << transparent;

  pndefs=1;
  pdefs=new struct VertexArrayDef[pndefs];

  

  pdefs[0].mode=GL_TRIANGLES;
  pdefs[0].count=pmesh->nTriangles()*3;
  pdefs[0].start=0;
  pdefs[0].useindex=false;

  
  


  ogl.reportGLError(nsubfunc);

  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheTriangleMeshItem::
collectTransparent(OGLTransparentCollector *tpCollector
		   ,const OGLMaterial &matfront,const OGLMaterial &matback)
{
  NSF("OGLDataCachePolyItem::collectTransparent");
  //myDebug() << nsubfunc<<this;
  
  bool datacolor=( pmesh->isoColor().outerFromData()||
		   pmesh->isoColor().innerFromData() );

  GLenum faces;
  if(pmesh->isoColor().outerFromData() && 
     pmesh->isoColor().innerFromData()){
    faces=GL_FRONT_AND_BACK;
  }else if(pmesh->isoColor().outerFromData()){
    faces=GL_FRONT;
  }else if(pmesh->isoColor().innerFromData()){
    faces=GL_BACK;
  }

  tpCollector->addObjecttoTree(pmesh->vertex(0),
			       pmesh->normal(0),
			       (datacolor?pmesh->colors(0):0),
			       faces,
			       &matfront,
			       &matback,
			       pmesh->triangle(0),
			       pndefs,pdefs);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheTriangleMeshItem::paint(bool transparent)  const
{
  NSF("OGLDataCacheTriangleMeshItem::paint");
  //myDebug() << nsubfunc;


  if(!pmesh->meshIsReady())return;

  if(pcull) {
    glEnable(GL_CULL_FACE);
  }else{
    glDisable(GL_CULL_FACE);
  }

  if(transparent){
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  }

  
  bool datacolor=( pmesh->isoColor().outerFromData()||
		   pmesh->isoColor().innerFromData() );
  
  GLenum faces;
  if(pmesh->isoColor().outerFromData() && 
     pmesh->isoColor().innerFromData()){
    faces=GL_FRONT_AND_BACK;
  }else if(pmesh->isoColor().outerFromData()){
    faces=GL_FRONT;
  }else if(pmesh->isoColor().innerFromData()){
    faces=GL_BACK;
  }
  
  OGLIssueVertexArray2(pmesh->vertex(0)
		      ,pmesh->normal(0)
		      ,(datacolor?pmesh->colors(0):0)
		      ,pmesh->triangle(0)
		      ,pndefs,pdefs
		      ,faces
		      );
  
  
  
  if(transparent){
    glDepthMask(GL_TRUE);
    glDisable(GL_BLEND);
  }
  
  glDisable(GL_CULL_FACE);


 ogl.reportGLError(nsubfunc);
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLTriangleMesh::OGLTriangleMesh(OGLTriangleMeshType type
				 ,const TriangleMesh *mesh,double linewidth)
  :OGLObject("OGLTriangleMesh")
  ,ptype(type),pmesh(mesh),plinewidth(linewidth)
  ,cachedData(0)
{
  pmaterialfront.setFace(GL_FRONT);
  pmaterialback.setFace(GL_BACK);
  //myDebug() << "OGLTriangleMesh::OGLTriangleMesh "<< pmesh;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLTriangleMesh::~OGLTriangleMesh()
{
  NSF("OGLTriangleMesh::~OGLTriangleMesh");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTriangleMesh::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTriangleMesh::construct(OGLView *oglview)
{
  NSF("OGLTriangleMesh::construct");
  //myDebug() << nsubfunc << pmesh << ptransparent;

  
  if(!pmesh->meshIsReady())return;
  //myDebug()<<"meshIsReady";

  OGLDataCacheTriangleMeshItem *data
    =new OGLDataCacheTriangleMeshItem(ptype,pmesh,materialFront(),materialBack());

  if(data!=(cachedData=oglview->ogldatacache->append(data,ptransparent)))
    delete data;
 
        

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTriangleMesh::paint(OGLView *
			    ,const OGLMagnifier &,bool reduced
			    ,bool// badTransparency
			    )const
{
  NSF("OGLTriangleMesh::paint");
  Q_UNUSED(reduced);
  //myDebug() << nsubfunc;

  //bool materialcolor=false;
  
  
  //myDebug() << nsubfunc<<" ptransparent="<<ptransparent;
  //exit(1);
  
  //  glPointSize(4.);
  //glDisable(GL_LIGHTING);
  // bool datacolor=(pmesh->isoColor().outerFromData() || 
  // 		  pmesh->isoColor().innerFromData());
  // if(materialcolor){
  //   if(datacolor){
  //     glEnable(GL_COLOR_MATERIAL);
  //   }
    
  //   if(pmesh->isoColor().outerFromData() && 
  //      pmesh->isoColor().innerFromData()){
  //     glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
  //   }else if(pmesh->isoColor().outerFromData()){
  //     glColorMaterial(GL_FRONT,GL_DIFFUSE);
  //   }else if(pmesh->isoColor().innerFromData()){
  //     glColorMaterial(GL_BACK,GL_DIFFUSE);
  //   }
  // }

  pmaterialfront.issue(ptransparent);
  pmaterialback.issue(ptransparent);

  //myDebug()<<"pmaterialfront"<<pmaterialfront.printS();


  if(plinewidth>0){
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
    glLineWidth(plinewidth);
  }else{
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  }

  
  if(cachedData)
    cachedData->paint(ptransparent);
  else{
    myDebug() << "cachedData is zero ?";
  }
  
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  
  // if(materialcolor){
  //   if(datacolor){
  //     glDisable(GL_COLOR_MATERIAL);
  //   }
  // }
  //glEnable(GL_LIGHTING);
  
  

  
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTriangleMesh::collectTransparent(OGLTransparentCollector *tpCollector
					 ,bool reduced)
{
  NSF("OGLTransparentCollector::collectTransparent");Q_UNUSED(reduced);
  //myDebug() << nsubfunc;
  if(!ptransparent) return;
  
  
  ((OGLDataCacheTriangleMeshItem*)cachedData)->
    collectTransparent(tpCollector,pmaterialfront,pmaterialback);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLTriangleMesh::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLTriangleMesh::NFPlane" ;



  //myDebug() << pmesh->boundingMidPoint().printS()()<<pmesh->boundingRadius();

  double p0=pmesh->boundingMidPoint()*n;
  double radius=pmesh->boundingRadius();

  near=-MyMath::max<double>(p0+radius,p0-radius);
  far=-MyMath::min<double>(p0+radius,p0-radius);
  //myDebug() << far<<near;
  
  return true;
}



/*EOF*/ 
