/* @(#)OGLTriangleMesh.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
 
#ifndef _OGLTRIANGLEMESH_H
#define _OGLTRIANGLEMESH_H 1


#include <QGLWidget>
#include "OGLDataCache.hh"
#include "OGLObject.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"

class TriangleMesh;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLTriangleMesh: public OGLObject
{
public:
  enum OGLTriangleMeshType{Regular=1};
  
  OGLTriangleMesh(OGLTriangleMeshType type,const TriangleMesh *mesh,double linewidth);
  virtual ~OGLTriangleMesh();

  bool NFPlane(const Vec3 &n,double &near,double &far);

  
  void paint(OGLView *oglview,const OGLMagnifier &mag
	     ,bool reduced,bool badTransparency)const;
  void construct(OGLView *oglview);
  void protectCachedData();
  void collectTransparent(OGLTransparentCollector *tpCollector,bool reduced);

  OGLMaterial &materialFront(){return pmaterialfront;};
  OGLMaterial &materialBack(){return pmaterialback;};

  bool pick(const View &/*v*/,double /*x*/,double /*y*/,double &z)
  {z=0.;return false;};
  

protected:


private:
  OGLTriangleMeshType ptype;
  const TriangleMesh *pmesh;
  OGLMaterial pmaterialfront;
  OGLMaterial pmaterialback;
  double plinewidth;
  OGLDataCacheItem *cachedData;

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheTriangleMeshItem: public OGLDataCacheItem
{
 public:
  OGLDataCacheTriangleMeshItem(OGLTriangleMesh::OGLTriangleMeshType type
			       ,const TriangleMesh *mesh
			       ,const OGLMaterial &front
			       ,const OGLMaterial &back
			       );
  
  QString printSpecQ()const;
  
  void paint(bool transparent)  const;
  void construct(bool transparent);
  void collectTransparent(OGLTransparentCollector *
			  ,const OGLMaterial &matfront,const OGLMaterial &matback);

  
 private:
  OGLTriangleMesh::OGLTriangleMeshType ptype;
  const TriangleMesh *pmesh;
  OGLMaterial pmaterialfront;
  OGLMaterial pmaterialback;

  friend class OGLDataCache;

};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLTRIANGLEMESH_H */

