/*************************************************************************
 * @(#)OGLTube.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   26 Apr 2010
 *************************************************************************
 */

#include <QGLWidget>
#include "OGLView.hh"
#include "OGLTube.hh"
#include "MyDebug.hh"


bool OGLTube::pusedisplaylists=true;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheTubeItem::OGLDataCacheTubeItem(OGLTube::OGLTubeType type
				  ,double length,double radius
					   ,double tiplenght
					   ,double tipradius
					   ,long np)
  :OGLDataCacheItem("OGLDataCacheTubeItem")
  ,ptype(type),plength(length)
  ,pradius(radius)
  ,ptiplength(tiplenght)
  ,ptipradius(tipradius)
  ,pnphi(np)
{
  NSF("OGLDataCacheTubeItem::OGLDataCacheTubeItem");

  String s;  
  if(ptype & OGLTube::Cylinder){
    if(ptype & OGLTube::RightArrow){
      
      s=Sprintf("OGLDataCacheTubeItem:type(%d)length(%30.6f),radius(%30.6f)"
		  ",tiplenght(%30.6f),tipradius(%30.6f),np(%ld)"
		  ",leftcap(%d)"
		  ,ptype,plength,pradius,ptiplength,ptipradius,pnphi
		  ,(ptype & OGLTube::LeftCap)
		  );
      
    }else{
      s=Sprintf("OGLDataCacheTubeItem:type(%d)length(%30.6f),radius(%30.6f)"
		  "np(%ld),leftcap(%d),rightcap(%d)"
		  ,ptype,plength,pradius,pnphi
		  ,(ptype & OGLTube::LeftCap)
		  ,(ptype & OGLTube::RightCap)
		  );
    }
  }else{
    myDebug() << nsubfunc << " type ??";
    exit(0);
  }
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());

}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheTubeItem::printSpecQ()const
{
  NSF("OGLTube::OGLDataCacheTubeItem::printSpecQ");
  //myDebug() << nsubfunc<<pid<<ptype;
  QString s;
  if(ptype & OGLTube::Cylinder){
    if(ptype & OGLTube::RightArrow){
      
      s.sprintf("OGLDataCacheTubeItem:   type=Arrow\n"
		"               nphi=%ld\n"
		"               length=%g\n"
		"               radius=%g\n"
		"               tiplenght=%g\n"
		"               tipradius=%g\n"
		"               leftcap=%d\n"
		,pnphi,plength,pradius,ptiplength,ptipradius
		,(ptype & OGLTube::LeftCap)
		);
      s+=OGLDataCacheItem::printSpecQ();
    }else {
      
      s.sprintf("OGLDataCacheTubeItem:   type=Cylinder\n"
		"               nphi=%ld\n"
		"               length=%g\n"
		"               radius=%g\n"
		"               leftcap=%d\n"
		"               rightcap=%d\n"
		,pnphi,plength,pradius
		,(ptype & OGLTube::LeftCap)
		,(ptype & OGLTube::RightCap)
		);
      s+=OGLDataCacheItem::printSpecQ();
    }      
  }else{
    myDebug() << nsubfunc << "Unkown ytype";
    exit(1);
  }
  return s;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheTubeItem::construct(bool transparent)
{
  NSF("OGLDataCacheTubeItem::construct");
  //myDebug() << nsubfunc;
  
  long nvert=0;
  long nn0=0;
  long idef=0;

  
  double *phis=new double[2*pnphi];
  
  { //prepare the cos(phi),sin(phi) array
    double cp=1.;
    double sp=0.;
    double c=cos(2.*Pi/pnphi);
    double s=sin(2.*Pi/pnphi);
    phis[0]=cp; 
    phis[1]=sp;
    for(int j=1;j<pnphi;j++){
      double d=cp;
      cp=cp*c-sp*s;
      sp=sp*c+s*d;
      phis[2*j]=cp;
      phis[2*j+1]=sp;
    }
  }
  
  
  bool arrow=(ptype & OGLTube::RightArrow);
  bool leftcap=(ptype & OGLTube::LeftCap);
  bool rightcap=(ptype & OGLTube::RightCap) && (!arrow);

  
  //allocate the tube data

  // arrow head flat part, 2*(pnphi+1) vetices, 1 def
  // arrow head tip, pnphi+2 vertices, 1 def

  double L=(arrow?plength-ptiplength:plength);
  makeRodMesh(nvert,idef,nn0,pnphi,pradius,L,L*0.5,Matrix3x3::unitMatrix(),
	      (leftcap?Round:Flat),(rightcap?Round:(arrow?Open:Flat))
	      ,true);

  if(arrow){
    pverticessize+=3*pnphi+4;
    pndefs+=2;
  }    
  
  pvertices=new GLdouble[pverticessize*3];
  pnormals =new GLdouble[pverticessize*3];
  pdefs=new struct VertexArrayDef[pndefs];
  
  if(arrow){
    // arrow head flat part, 2*(pnphi+1) vetices, 1 def
    double L=plength-ptiplength;

    double sig=1;
    if(ptipradius>pradius){
      sig=-1;
    }
    
    for(int j=pnphi;j>=0;j--){
      
      long jj=(j+1)%pnphi;
      
      double c=phis[2*jj];
      double s=phis[2*jj+1];
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      
      pvertices[nvert*3+0]=pradius*c;
      pvertices[nvert*3+1]=pradius*s;
      pvertices[nvert*3+2]=L;
      
      nvert++;
      
      
      pnormals[nvert*3+0]=0;
      pnormals[nvert*3+1]=0;
      pnormals[nvert*3+2]=sig;
      
      pvertices[nvert*3+0]=ptipradius*c;
      pvertices[nvert*3+1]=ptipradius*s;
      pvertices[nvert*3+2]=L;
      
      nvert++;
      
    }
    pdefs[idef].mode=GL_QUAD_STRIP;
    pdefs[idef].count=2*(pnphi+1);
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    
    idef++;
    nn0+=2*(pnphi+1);

    // arrow head tip, pnphi+2 vertices, 1 def
    pnormals[nvert*3+0]=0;
    pnormals[nvert*3+1]=0;
    pnormals[nvert*3+2]=1;
    
    pvertices[nvert*3+0]=0.;
    pvertices[nvert*3+1]=0.;
    pvertices[nvert*3+2]=plength;
    nvert++;
    
    for(int j=0;j<=pnphi;j++){
      long jj=(j+1)%pnphi;
      
      double c=phis[2*jj];
      double s=phis[2*jj+1];
      
      Vec3 n(ptiplength*c,ptiplength*s,0*ptipradius);
      n.normalize();
      
      pnormals[nvert*3+0]=n[0];
      pnormals[nvert*3+1]=n[1];
      pnormals[nvert*3+2]=n[2];
      pvertices[nvert*3+0]=ptipradius*c;
      pvertices[nvert*3+1]=ptipradius*s;
      pvertices[nvert*3+2]=L;
      nvert++;
      
    }
    
    pdefs[idef].mode=GL_TRIANGLE_FAN;
    pdefs[idef].count=(pnphi+2);
    pdefs[idef].start=nn0;
    pdefs[idef].useindex=false;
    idef++;
    nn0+=pnphi+2;
  }
  


  

  L=(arrow?plength-ptiplength:plength);
  makeRodMesh(nvert,idef,nn0,pnphi,pradius,L,L*0.5,Matrix3x3::unitMatrix(),
	      (leftcap?Round:Flat),(rightcap?Round:(arrow?Open:Flat)));

  



  


  delete [] phis;
  

  
  
  if(OGLTube::useDisplayLists()){
    pdispListName=glGenLists(1);
  }else{
    pdispListName=0;
  }
  if(pdispListName){
   
    OGLIssueVertexArray(pdispListName
			,pvertices,pnormals,0,pindices,pndefs,pdefs);
 
    
    
    cleanInternal();

    
  }
  
  
  
  ogl.reportGLError(nsubfunc);
  
  
  
  //myDebug() << nsubfunc;
  //  myDebug() << nsubfunc  << printSpecQ();
  //myDebug() << "pdispListName="<<pdispListName;

  OGLDataCacheItem::construct(transparent);

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLTube::OGLTube(OGLTubeType type,const Vec3 &pos0,const Vec3 &pos1
		 ,double radius
		 ,long np
		 ,double tiplenght
		 ,double tipradius
		 )
  :OGLObject("OGLTube")
  ,ptype(type)
  ,pnphi(np)
  ,pradius(radius)
  ,ptiplength(tiplenght)
  ,ptipradius(tipradius)
  ,pposition0(pos0)
  ,pposition1(pos1)
  ,cachedDataNormal(0)
  ,cachedDataReduced(0)
{
  plength=norm(pos1-pos0);
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLTube::~OGLTube()
{
  NSF("OGLTube::~OGLTube");
  //myDebug() << nsubfunc<< "I am dying";
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTube::protectCachedData()
{
  if(cachedDataReduced)
    cachedDataReduced->increaseUsed();
  if(cachedDataNormal)
    cachedDataNormal->increaseUsed();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTube::paint(OGLView *,const OGLMagnifier &,bool reduced
		    ,bool// badTransparency
		    )const
{
  NSF("OGLTube::paint");
  //myDebug() << nsubfunc;

//   int np=pnphi;
//   if(reduced){
//     np=npreduced;
//   }


  glTranslated(pposition0[0],pposition0[1],pposition0[2]);
  Vec3 eu=Vec3::eulerAngles(pposition1-pposition0);
  //myDebug() << "eu="<<eu[0]<<eu[1]<<eu[2];
  glRotated(eu[0]*180./Pi,0.,0.,1.);
  glRotated(eu[1]*180./Pi,0.,1.,0.);
  pmaterial.issue();

  
  if(reduced){
    if(cachedDataReduced)cachedDataReduced->paint(ptransparent);
  }else{
    if(cachedDataNormal)cachedDataNormal->paint(ptransparent);
  }

  
  
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLTube::construct(OGLView *oglview)
{
  NSF("OGLTube::construct");
  //  myDebug() << nsubfunc<<this;

  OGLDataCacheTubeItem *data
    =new OGLDataCacheTubeItem(ptype,plength,pradius
			      ,ptiplength
			      ,ptipradius
			      ,pnphi);
  
  if(data!=(cachedDataNormal=oglview->ogldatacache->append(data))) 
    delete data;
  

  

  // make reduced model just in case the user has slow hardware
  npreduced=4;

  data=new OGLDataCacheTubeItem(ptype,plength,pradius
				,ptiplength
				,ptipradius
				,npreduced);
  if(data!=(cachedDataReduced=oglview->ogldatacache->append(data))) 
    delete data;
  
      

  return;
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLTube::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "NFPlane " << id();

  double p0=pposition0*n;
  double p1=pposition1*n;
  double b=(p1-p0)/plength;


  if(ptype & OGLTube::LeftCap){
    p0-=pradius*b;
  }
  if(ptype & OGLTube::RightCap){
    p1+=pradius*b;
  }

  b=sqrt(1.-b*b);
  double a;
  if(ptype & OGLTube::RightArrow){
    a=MyMath::max<double>(ptipradius,pradius)*b;
  }else{
    a=pradius*b;
  }




  near=-MyMath::max<double>(MyMath::max<double>(p0+a,p0-a),MyMath::max<double>(p1+a,p1-a));

  far=-MyMath::min<double>(MyMath::min<double>(p0+a,p0-a),MyMath::min<double>(p1+a,p1-a));

  //myDebug()<<"OGLTube"<<near<<far;

  return true;
}


/*EOF*/ 
