/* @(#)OGLTube.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLTUBE_H
#define _OGLTUBE_H 1

#include <QGLWidget>
#include "OGLDataCache.hh"
#include "OGLObject.hh"
#include "OGLMaterial.hh"
#include "Vec3.hh"



class OGLTube: public OGLObject
{
public:
  enum OGLTubeType{Cylinder=1,RightArrow=0x10,LeftCap=0x20,RightCap=0x40
		   ,CylinderLeftCap=Cylinder|LeftCap
		   ,CylinderRightCap=Cylinder|RightCap
		   ,CylinderCap=Cylinder|RightCap|LeftCap
		   ,Arrow=Cylinder|RightArrow
		   ,ArrowLeftCap=Cylinder|RightArrow|LeftCap
  };
  
  OGLTube(OGLTubeType type,const Vec3 &pos0,const Vec3 &pos1
	  ,double radius=1.
	  ,long np=6
	  ,double tiplenght=0.1
	  ,double tipradius=0.2
	  );
  virtual ~OGLTube();

  bool NFPlane(const Vec3 &n,double &near,double &far);

  
  void paint(OGLView *oglview,const OGLMagnifier &mag
	     ,bool reduced,bool badTransparency)const;
  void construct(OGLView *oglview);
  void protectCachedData();


  OGLMaterial &material(){return pmaterial;};

  bool pick(const View &/*v*/,double /*x*/,double /*y*/,double &z)
  {z=0.;return false;};

  static void setUseDisplayLists(bool u){pusedisplaylists=u;};
  static bool useDisplayLists() {return pusedisplaylists;};

protected:


private:
  OGLTubeType ptype;
  long pnphi;
  double plength;
  double pradius;
  double ptiplength;
  double ptipradius;
  static bool pusedisplaylists;
  Vec3 pposition0,pposition1;
  OGLMaterial pmaterial;
  int npreduced;
  OGLDataCacheItem *cachedDataNormal;
  OGLDataCacheItem *cachedDataReduced;

};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLDataCacheTubeItem: public OGLDataCacheItem
{
 public:
  OGLDataCacheTubeItem(OGLTube::OGLTubeType type
		       ,double length
		       ,double radius
		       ,double tiplenght
		       ,double tipradius
		       ,long nphi);
  
  QString printSpecQ()const;
  
  void construct(bool transparent);

  
 private:
  OGLTube::OGLTubeType ptype;
  double plength;
  double pradius;
  double ptiplength;
  double ptipradius;
  long pnphi;
    
};
#ifdef	__cplusplus
extern "C" {
#endif
  
#ifdef	__cplusplus
}
#endif


#endif /* _OGLTUBE_H */

