/* @(#)OGLView.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _OGLVIEW_H
#define _OGLVIEW_H 1

#include <QGLWidget>
#include "MyQRgb.hh"
#include <QHash>
#include "Atom.hh"
#include "MACROS.hh"
#include "OGLMeshPoints.hh"
#include "OGLMeshCells.hh"




class OGLTransparentCollector;
class SpecialSymPoints;
class OGLDataCache;
class DocBase;
class IsoColor;
class STDoc;
class FSDoc;
class OGLObjects;
class Fog;
class Boundary;
class AtomPropPicker;
namespace XFPLO{
  class Axes;
  class Polyhedron;
  class OGLAutoGlyph;
  class OGLVolume;
}
class RegularMesh;
class TriangleMesh;
class View;
class AtomFindMarker;
class OGLSphere;
class DisplayCell;
class Status;
class OGLUserData;


struct cellState;

using namespace XFPLO;

/* ***********************************************************************
 * save data needed for context menus
 * ***********************************************************************
 */
class ContextMenuContext
{
public:
  enum ContextType{None=0,AtomDelete=1,SetViewCenter=2,HideAtom=3};
  ContextMenuContext():ptype(None){}
  ~ContextMenuContext(){}
  //
  ContextMenuContext &setAtomDelete(long atomindex){
    pmenutitle="&Delete Wyckoff position";
    ptype=AtomDelete;
    patomindex=atomindex;
    return *this;}
  ContextMenuContext &setHideAtom(long atomindex){
    pmenutitle="&Hide Wyckoff position";
    ptype=HideAtom;
    patomindex=atomindex;
    return *this;}
  ContextMenuContext &setViewCenter(const Vec3 &currentpoint){
    pmenutitle="Set View &Center";
    ptype=SetViewCenter;
    pcurrentpoint=currentpoint;
    return *this;}
  //
  inline ContextType  type()const{return ptype;}
  inline long atomIndex()const{return patomindex;}
  inline const Vec3 &currentPoint()const{return pcurrentpoint;}
  inline const QString &menuTitle()const{return pmenutitle;}
private:
  ContextType ptype;
  long patomindex;
  Vec3 pcurrentpoint;
  QString pmenutitle;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class OGLView: public QGLWidget
{
  Q_OBJECT;

public:

  enum AntiAliasingMode {None=0,Hardware=1,Software=2};
  enum CameraResetMode {Reset=0x3,Center=0x1,Rescale=0x2,UserFocus=0x4};

  OGLView(QWidget *parent=0);
  ~OGLView();

  void init(DocBase *doc);


  void setWireFrameOnMove(bool w){pwireFrameOnMove=w;};
  bool wireFrameOnMove()const { return pwireFrameOnMove;};

  void setReducedModelOnMove(bool w){preducedModelOnMove=w;};
  bool reducedModelOnMove()const { return preducedModelOnMove;};

  //bool badTransparency()const{return pbadTransparency;};
  void setBadTransparency(bool b){pbadTransparency=b;};

  AntiAliasingMode currentAntiAliasingMode();


  void setLighting();
  void setFog(const Fog *fog){pfog=fog;};
  
  void toggleHiddenAtoms(const STDoc *stdoc,bool on);
    
  void setVisible(const String &id,bool v);

  void drawDisplayCell(const DisplayCell *cell);
  //void drawCell(const Boundary *bd,const DisplayCell *dc);
  //  void drawSpannedBox(const Matrix3x3 &cell,const cellState& cellstate);
  void drawAtoms(const STDoc *stdoc);
  void drawSTMAtom(const STDoc *stdoc);
  void drawBonds(const STDoc *stdoc);
  void drawPolyhedra(const STDoc *stdoc);
  void drawSymmetryOperations(const STDoc *stdoc);
  void drawAxes(const Axes *axes,int fontres);
  void drawMeshPoints(const RegularMesh *mesh
		      ,OGLMeshPoints::OGLMeshPointsType type,bool state);
  void drawMeshCells(const RegularMesh *mesh
		     ,OGLMeshCells::OGLMeshCellsType type,bool state);
  void drawTriangleMesh(const TriangleMesh *triamesh,bool state,double linewidth);
  void drawLegend(const QString &title
		  ,const QString &min,const QString &max
		  ,bool legendstate,bool legendunitstate,
		  const IsoColor &isocolor);
  void drawSpecialPoints(const SpecialSymPoints *specpoints,int fontres);
  void invalidateMesh(const RegularMesh *mesh);
  void invalidateMesh(const TriangleMesh *mesh);

  void colorAtoms(const STDoc *stdoc);
  void reCalcScene();

  void showPotentialSpecPoints(bool on);

  void setAllowedMoveObjects(const String &ObjectType,const String &idprefix);

  void writePNG(const QString &file,long magfac,int quality
		,Status *statusbar);

  inline void setNeedRepaint(bool r){pneedrepaint=r;}
  void setTransNeedReConstruct(bool r);

  OGLObjects *objects()const{return pobjects;}
  void removeObject(const String &id){pobjects->clear(id);};
  void appendObject(OGLObject *o){pobjects->append(o);};

  void resetPanZoom(CameraResetMode mode,bool render/*=true*/,const Vec3 &center=Vec3());

  void doRendering();

  const View &view()const;

  void setAtomPropPicker(AtomPropPicker *ap);

  AtomFindMarker *atomFindMarker(){return patomFindMarker;};
  void setMoveBlockingClick(bool b){pmoveblockingclick=b;};

  void setAllowContexts(const String &context,bool b){
    pcontexts[context()]=b;
  };

  void drawPolyhedron(const Polyhedron *,const String &idprefix
		       ,bool drawspecdot);

  void mute(bool muted){pmuted=muted;};
  void setShowLightHelpers(bool onoff);
  
signals:

  void sizeChanged(const OGLView*);
  void atomClicked(long atindex);
  void objectClicked(const String &,const OGLUserData*);
  void atomInfoToShow(String);
  void textShiftChanged(const String&,const Vec3&);
  void contextActionRequest(const ContextMenuContext&);

protected:
  void initializeGL();
  void paintGL();
  void resizeGL(int width, int height);
  virtual void resizeEvent(QResizeEvent *event); 
  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void wheelEvent(QWheelEvent *event);
  
private slots:
  void onMarkAtomsRequest(bool onoff,bool update);
  void contextMenuCallback();

private:
  void setCamera();

  void applyFog();

  void drawScene(const View &);

  void applyAntiAliasing();

  bool markPickedAtom(long atindex,bool onoff);

  bool setAtomPickerMarkers(bool onoff);

  

  void colorAtom(OGLSphere *sph,QRgb diffusecolor,QRgb emissioncolor,
		 QRgb spotcolor,double shininess);

  void standardMaterialSettings(OGLMaterial &,QRgb base,double opacity);

  void contextMenu(const QList<ContextMenuContext> &contextlist,
		   QMouseEvent *event);

private:
  double H,W;
  //double worldUnitInPixel;
  QPoint lastPos;
  OGLObjects *pobjects;
  ValueArray<ValueArray<GLdouble> > pseedplanes;
  bool pwireFrameOnMove;
  bool preducedModelOnMove;
  bool pdrawReduced;
  bool pnoaa;
  bool pbadtrans;
  bool pblockmove;
  bool pmoveblockingclick;
  bool pneedrepaint;
  bool pmuted;
  bool ptriggerrepaint;
  bool pbufferedrepaint;
  bool pprinting;
  bool pbadTransparency;
  bool pshowpotentialspcepoints;
  DocBase *pdoc;
  AtomPropPicker *atomPropPicker;
  AtomFindMarker *patomFindMarker;

  double winB,winT,winL,winR;
  const Fog *pfog;
  bool phasaccumbuffer;
  bool phasmultisample;
  OGLDataCache *ogldatacache;
  OGLTransparentCollector *ptpCollector;
  OGLObject *pcurrentmoveobject;
  String pallowedmoveobjecttype;
  String pallowedmoveobjectidprefix;
  QHash<QString,bool> pcontexts;
  ROPROPERTY(showlighthelpers,showLightHelpers,bool);
  friend class OGLSphere;
  friend class OGLTube;
  friend class OGLMeshPoints;
  friend class OGLMeshCells;
  friend class OGLTriangleMesh;
  friend class OGLPoly;
  friend class OGLLegend;
  friend class OGLText;
  friend class OGLObjects;
  friend class OGLRotationSymbol;
  friend class XFPLO::OGLVolume;
  friend class XFPLO::OGLAutoGlyph;
};

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _OGLVIEW_H */

