/*************************************************************************
 * @(#)OGLVolume.cpp
 * author: magru@unas
 * date:   02 Jul 2013
 *************************************************************************
 */

#include "MyDebug.hh"
#include "Field.hh"
#include "RepVolume.hh"
#include "OGLMaterial.hh"
#include "OGLVolume.hh"
#include "OGLView.hh"

using namespace XFPLO;



/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLDataCacheVolumeItem::
OGLDataCacheVolumeItem(const RepVolume *rep)
  :OGLDataCacheItem("OGLDataCacheVolumeItem")
  ,prep(rep)
{
  NSF("OGLDataCacheVolumeItem::OGLDataCacheVolumeItem");
  pcull=false;
  
  String s=Sprintf("OGLDataCacheVolumeItem:rep(%p)",prep);
  if(pid) delete [] pid;
  pid=new char [s.length()+1];
  strcpy(pid,s());
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QString OGLDataCacheVolumeItem::printSpecQ()const
{
  NSF("OGLDataCacheVolumeItem::printSpecQ");
  QString s;
  
  s.sprintf("OGLDataCacheVolumeItem:   rep=%p\n",prep);
  s+=OGLDataCacheItem::printSpecQ();
  return s;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheVolumeItem::construct(bool transparent)
{
  NSF("OGLDataCacheVolumeItem::construct");

  const Field *field=prep->field();
  
  
  if(!field) return;

  
  myDebug()<<nsubfunc<<(void*)field;

   
  const BaseArray<double> *ar=field->positions()->array();
  if(!ar) return;
  const BaseArray<long> *co=field->connections()->array();
  if(!co) return;
  const BaseArray<double> *no=field->normals();
  const BaseArray<double> *bacol=field->colors();

  bool shownormals=(prep->showNormals() && (no!=0));

  pndefs=1;
  if(shownormals) pndefs=2;
  pdefs=new struct VertexArrayDef[pndefs];


  pverticessize=ar->size();
  if(!shownormals){
    pvertices=new GLdouble[pverticessize*3];
    if(no) pnormals=new GLdouble[pverticessize*3];
    if(bacol) pcolors=new GLfloat[pverticessize*4];  
  }else{
    pvertices=new GLdouble[pverticessize*3*2];
    if(no) pnormals=new GLdouble[pverticessize*3*2];
    if(bacol) pcolors=new GLfloat[pverticessize*4*2];  
  }
  

  long dembb=field->positions()->embeddingDimension();

  // currently positions dimension (embedding dimension must be 3)
  // can do: 2d cub & simp
  // can do: 1d cub & simp

  if(dembb>3)
    throw Except(
		 Sprintf("OGLVolume:: do not know how to "
			 "represent %ld-dimensional positions",dembb));

  long nvert=0;
  for(int ip=0;ip<pverticessize;ip++){
    
    if(dembb==1){
      pvertices[nvert*3]=ar->point(ip,0);
      pvertices[nvert*3+1]=0.;
      pvertices[nvert*3+2]=0.;
      if(no){
	pnormals[nvert*3]=no->point(ip,0);
	pnormals[nvert*3+1]=0.;
	pnormals[nvert*3+2]=0.;
      }
    }else if(dembb==2){
      pvertices[nvert*3]=ar->point(ip,0);
      pvertices[nvert*3+1]=ar->point(ip,1);
      pvertices[nvert*3+2]=0.;
      if(no){
	pnormals[nvert*3]=no->point(ip,0);
	pnormals[nvert*3+1]=no->point(ip,1);
	pnormals[nvert*3+2]=0.;
      }
    }else if(dembb==3){
      pvertices[nvert*3]=ar->point(ip,0);
      pvertices[nvert*3+1]=ar->point(ip,1);
      pvertices[nvert*3+2]=ar->point(ip,2);
      if(no){
	pnormals[nvert*3]=no->point(ip,0);
	pnormals[nvert*3+1]=no->point(ip,1);
	pnormals[nvert*3+2]=no->point(ip,2);
      }
    }
    if(bacol){
      for(int icol=-1;++icol<4;){
	pcolors[nvert*4+icol]=bacol->point(ip,icol);
      }
    }
    // myDebug()<<"vertex"<<ip
    // 	     <<pvertices[nvert*3+0]<<pvertices[nvert*3+1]<<pvertices[nvert*3+2];
    nvert++;
  }

    
  long nind=0;

  if(field->connections()->connectionType()==Connections::CTYSimplex){
    long shape=co->shape(0);
    myDebug()<<"shape="<<shape;
    long D=shape-1;

    // d  = 3 2 1
    // #c =  d+1
    // #tr=   1 0

    pindicessize=co->size()*(D+1);
    if(!shownormals){
      pindices=new GLuint[pindicessize];
    }else{
      pindices=new GLuint[pindicessize+pverticessize*2];
    }
    myDebug()<<"pindicessize="<<pindicessize;
    myDebug()<<"D="<<D;


    if(D==2)
      pdefs[0].mode=GL_TRIANGLES;
    else if(D==1)
      pdefs[0].mode=GL_LINES;
    else
      throw Except("OGLVolume: Unkown GL connection type");

    

    
    for(int ic=0;ic<co->size();ic++){
      for(int c=0;c<D+1;c++){
	pindices[nind++]=co->point(ic,c);
      }
    }

    myDebug()<<"nind="<<nind;

    pdefs[0].count=nind;
    pdefs[0].start=0;
    pdefs[0].useindex=true;

  //}else if(field->connections()->connectionType()==Connections::CTYCuboid){
  }else{
    throw Except("OGLVolume: Unkown connection type");
  }

  long nvert0=nvert;

  if(shownormals){
    pdefs[1].mode=GL_LINES;
    pdefs[1].start=nind;
    pdefs[1].useindex=true;

    for(int ip=0;ip<pverticessize;ip++){
      if(dembb==1){
	pvertices[nvert*3]=ar->point(ip,0)+no->point(ip,0);
	pvertices[nvert*3+1]=0.;
	pvertices[nvert*3+2]=0.;
	if(no){
	  pnormals[nvert*3]=no->point(ip,0);
	  pnormals[nvert*3+1]=0.;
	  pnormals[nvert*3+2]=0.;
	}
      }else if(dembb==2){
	pvertices[nvert*3]=ar->point(ip,0)+no->point(ip,0);
	pvertices[nvert*3+1]=ar->point(ip,1)+no->point(ip,1);
	pvertices[nvert*3+2]=0.;
	if(no){
	  pnormals[nvert*3]=no->point(ip,0);
	  pnormals[nvert*3+1]=no->point(ip,1);
	  pnormals[nvert*3+2]=0.;
	}
      }else if(dembb==3){
	pvertices[nvert*3]=ar->point(ip,0)+no->point(ip,0);
	pvertices[nvert*3+1]=ar->point(ip,1)+no->point(ip,1);
	pvertices[nvert*3+2]=ar->point(ip,2)+no->point(ip,2);
	if(no){
	  pnormals[nvert*3]=no->point(ip,0);
	  pnormals[nvert*3+1]=no->point(ip,1);
	  pnormals[nvert*3+2]=no->point(ip,2);
	}
      }
      if(bacol){
	for(int icol=-1;++icol<4;){
	  pcolors[nvert*4+icol]=bacol->point(ip,icol);
	}
      }
      nvert++;
    }

    long nind0=nind;
    
    for(int ip=0;ip<pverticessize;ip++){    
      pindices[nind++]=ip;
      pindices[nind++]=ip+nvert0;
    }


    pdefs[1].count=nind-nind0;
    
  }


  
  pdispListName=0;
  
  
  ogl.reportGLError(nsubfunc);

  //myDebug() << nsubfunc  ;
  //myDebug() << printSpecQ();

  OGLDataCacheItem::construct(transparent);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLDataCacheVolumeItem::paint(bool transparent)  const
{
  NSF("OGLDataCacheVolumeItem::paint");
  //myDebug() << nsubfunc;
  if(pcull) {
    glEnable(GL_CULL_FACE);
  }else{
    glDisable(GL_CULL_FACE);
  }

  if(transparent){
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  }

  
  bool datacolor=true;//( pmesh->isoColor().outerFromData()||
  // 		   pmesh->isoColor().innerFromData() );
  
  GLenum faces;
  // if(pmesh->isoColor().outerFromData() && 
  //    pmesh->isoColor().innerFromData()){
    faces=GL_FRONT_AND_BACK;
  // }else if(pmesh->isoColor().outerFromData()){
  //   faces=GL_FRONT;
  // }else if(pmesh->isoColor().innerFromData()){
  //   faces=GL_BACK;
  // }
  
  OGLIssueVertexArray2(pvertices
		      ,pnormals
		      ,(datacolor?pcolors:0)
		      ,pindices
		      ,pndefs,pdefs
		      ,faces
		      );
  
  
  
  if(transparent){
    glDepthMask(GL_TRUE);
    glDisable(GL_BLEND);
  }
  
  glDisable(GL_CULL_FACE);


 ogl.reportGLError(nsubfunc);
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLVolume::OGLVolume(const RepVolume *rep)
  :OGLObject("OGLVolume")
  ,prep(rep)
  ,cachedData(0)
{
  setVisible(rep->isVisible());
  setID(rep->id());
  pmaterialfront.setFace(GL_FRONT);
  pmaterialback.setFace(GL_BACK);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

OGLVolume::~OGLVolume()
{
  NSF("OGLMeshPoints::~OGLMeshPoints");
  //myDebug() << nsubfunc<< "I am dying";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLVolume::protectCachedData()
{
  if(cachedData)
    cachedData->increaseUsed();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLVolume::construct(OGLView *oglview)
{
  NSF("OGLVolume::construct");
  //myDebug() << nsubfunc;

  //todo: here we must have ensured that we never touch prep if invalid
  
  if(prep->isDirty())return;

  OGLDataCacheVolumeItem *data=new OGLDataCacheVolumeItem(prep);
  

  if(data!=(cachedData=oglview->ogldatacache->append(data,false,true)))
    delete data;
  
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OGLVolume::paint(OGLView *
			 ,const OGLMagnifier &,bool reduced
			 ,bool badTransparency
			 )const
{
  NSF("OGLVolume::paint");
  Q_UNUSED(reduced);
  Q_UNUSED(badTransparency);
  //myDebug() << nsubfunc;


  
  pmaterialfront.issue(ptransparent);
  pmaterialback.issue(ptransparent);

  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  //glEnable(GL_COLOR_MATERIAL);
  //glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);

  if(cachedData)
    cachedData->paint(ptransparent);
  else{
    myDebug() << "cachedData is zero ?";
  }
  
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  //glDisable(GL_COLOR_MATERIAL);

  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool OGLVolume::NFPlane(const Vec3 &n,double &near,double &far)
{
  //myDebug() << "OGLVolume::NFPlane";


  const BoundingBox &bbox=prep->boundingBox();
  
  //myDebug() << cell.printS()();
  
  double p0=bbox.origin()*n;
  const Matrix3x3 &cell=bbox.cell();
  
  near=-p0;
  far=-p0;
  
  double c0=cell(0)*n;
  double c1=cell(1)*n;
  double c2=cell(2)*n;
  
  

  
  double d=c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  
  d=c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1+c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c1+c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c2+c0;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  d=c0+c1+c2;
  near=MyMath::min<double>(near,-(p0+d));
  far=MyMath::max<double>(far,-(p0+d));
  
  
  //myDebug() << near<<far;  
  
  return true;
}



    


/*EOF*/ 
