/*************************************************************************
 * @(#)RepBoundingBox.cpp
 * author: magru@unas
 * date:   10 Jun 2013
 *************************************************************************
 */

#include "RepBoundingBox.hh"
#include "Field.hh"
#include "Vlevel.hh"
#include "Polyhedron.hh"
#include "OGLView.hh"
#include "Xf.hh"

using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */

RepBoundingBox::RepBoundingBox(OGLView *oglview)
  :Representation(oglview,"RepBoundingBox"),ppoly(new Polyhedron())
{
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
RepBoundingBox::~RepBoundingBox()
{
  if(ppoly) delete ppoly;ppoly=0;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool RepBoundingBox::cascadingUpdate(const String &prefix)
{
  if(Module::debugmode) printf("%s%s::cascadingUpdate\n",prefix(),id()());
  Module::cascadingUpdate(prefix+"  ");
  bool ret=false;
  showDirt(prefix,Sprintf("%s must react to",id()()));

  if(!(pisdirty||dirtyInput())) return ret;

  {
    const BoundingBox &bbox=boundingBox();
    //  ppoly->fromCell(bbox.origin(),bbox.cell());
    
    ValueArray<Plane> pla;
    
    const Matrix3x3 &box=bbox.cell();
    Matrix3x3 G=bbox.cell().inverseT();
    
    double radius=(norm(box(0))+norm(box(1))+norm(box(2)))/3;
    
    for(int i=0;i<3;i++){
      double gg=G(i)*G(i);
      pla.append(Plane( G(i)/gg, 1));
      pla.append(Plane(-G(i)/gg,-0));
    }
    
    for(int i=0;i<pla.size();i++) {
      double d=(bbox.origin())*pla[i].n()/norm(pla[i].P());
      pla[i]=Plane(pla[i].P(),pla[i].D()+d);
    }
    
    //myDebug()<<"pla="<<pla;
    ppoly->properties().setVisible(true);
    ppoly->properties().setRimRadiusUnits(Xf::SUBoundary);
    ppoly->properties().setRimRadius(0.01*radius);
    ppoly->properties().setRimMaterial(Material(0xffffff00,50.));
    ppoly->properties().setFaceMaterial(Material(0x30ffff00,50.));
    ppoly->properties().setDrawRim(false);
    ppoly->properties().setDrawFaces(true);
    ppoly->makeCorners(pla,1.0e-6,Vlevel(Vlevel::Info));
    
    ppoly->properties().calculate(Xf::LTDirect,1.,Xf::BohrRadius);

    ret=true;
  }
  resetDirt();

  if(Module::debugmode) printf("%sret=%d\n",prefix(),ret);
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const Polyhedron *RepBoundingBox::polyhedron()const
{
  NSF("RepBoundingBox::polyhedron()");
  //exit(0);
  return ppoly;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool RepBoundingBox::hasTransparency()const
{
  return qAlpha(ppoly->properties().faceMaterial().color())<255;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void RepBoundingBox::draw(OGLView *oglview)
{
  //printf("%s::draw\n",id()());

    
  oglview->makeCurrent();
    
  if(!pisvisible) return;

  //find out if we need updates of data
  if(!cascadingUpdate("")) return;
  // if data update was succesfull, and if changed remake the object

  
    
  //myDebug()<<"id  "<<id()<<"is visible";

  oglview->removeObject(id());
  
  oglview->drawPolyhedron(ppoly,id(),false);
  
  //myDebug()<<"rep->hasTransparency()"<<rep->hasTransparency();
  oglview->setTransNeedReConstruct(hasTransparency());
  oglview->setNeedRepaint(true);
    
  
}

/*EOF*/ 
