/*************************************************************************
 * @(#)Representation.cpp
 * author: magru@bono
 * date:   21 Jun 2013
 *************************************************************************
 */

#include "Representation.hh"
#include "RepAutoGlyph.hh"
#include "RepBoundingBox.hh"
#include "OGLView.hh"
#include "Field.hh"
#include "IsoSurface.hh"
#include "AutoColor.hh"
#include "MyDebug.hh"


using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Representation::setInput(const Module *input)
{
  if(!input) return;
  if(input->isA("Field")){
    pfield=dynamic_cast<const Field*>(input);
    pisdirty=true;
  }else if(input->isA("IsoSurface")){
    pfield=dynamic_cast<const IsoSurface*>(input)->outputField();
    pisdirty=true;
    return;
  }else if(input->isA("AutoColor")){
    pfield=dynamic_cast<const AutoColor*>(input)->field();
  }else{
    throw Except("Representation::setInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Representation::removeInput(const Module *input)
{
  //  myDebug()<<pclassname<<"removeInput"<<(void*)input<<input->classname();
  // dynamic_cast no longer possible, since called from destructor
  if(input->isA("Field")){
    if(pfield==input) pfield=0;
  }else if(input->isA("IsoSurface")){
    pfield=0;
  }else if(input->isA("AutoColor")){
    pfield=0;
  }else{
    throw Except("Representation::removeInput: unkown input module.");
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
const BoundingBox &Representation::boundingBox()const{
  if(pfield){
    return pfield->boundingBox();
  }
  return BoundingBox::defaultBox;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Representation::draw(OGLView *oglview,ValueArray<Representation *>reps)
{
  myDebug()<<"==================Representation::draw=================";
  // first find all pobjects that are no longer needed and remove them
  // then only create the ones that are currently visible
  long N=reps.size();
  for(long i=-1;++i<N;){
    //myDebug()<<"draw rep"<<i;
    reps[i]->draw(oglview);
    
  }
  myDebug()<<"=======================================================";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Representation::setVisible(bool v)
{
  pisvisible=v;
  poglview->setVisible(id(),v);
}




/*EOF*/ 
