//avoid reload after changes

/*************************************************************************
 * @(#)BWEdit.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   30 Oct 2010
 *************************************************************************
 */

#include <QToolBar>
#include <QFileDialog>
#include <QDebug>
#include <QProcess>
#include <QMessageBox>
#include "BWEdit.hh"
#include "Exception.hh"
#include "MWBase.hh"
#include "Settings.hh"
#include "Config.hh"
#include "MySignalMapper.hh" 
#include "HelpDlg.hh"
#include "Parser.hh"
#include "Status.hh"
#include "ShortCuts.hh"
#include "BWDoc.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"


#define workdir (doc->currentDir() +'/')

/* ***********************************************************************
 *
 * ***********************************************************************
 */
BWEdit::BWEdit(MWBase *parent) 
  : ControlBase(parent),doc(0)//,sttoolbox(0),propview(0),bounddlg(0)
	      //,bondsdlg(0),lightingdlg(0)
{
  setupUi(this);


  doc=new BWDoc(0);

  setupMenus();

  MASSERT("",doc);
  setupDoc(doc);

  trvCombis->setBase(base());
  connect(trvCombis,SIGNAL(contribsValid(bool))
	  ,pbNewContrib,SLOT(setEnabled(bool)));
    
  base()->setWindowTitle(IDS_APP_NAME + " : "+doc->fileName());

  QString fw=doc->bwfromcoeffs.currentBWFile()();
  leBandWeights->setText(fw);

  cbUseCoeffFile->setVisible(true);
  Settings set("./bwdef.ini","Controls");
  bool onoff=set.getBool("cbUseCoeffFile",false);
  cbUseCoeffFile->setChecked(onoff);

  populateTrvCombis();


  QAction *a=new QAction(this);
  ShortCuts::setShortCut(ShortCuts::deleteItemSC,a);
  pbDelete->addAction(a);

  signalMapper=new MySignalMapper(this,trvCombis,SLOT(onActions(int)),
				  QList<MySignalMapper::Data>()
				  <<MySignalMapper::Data(pbNewAtomic,SIGNAL(clicked()),0)
				  <<MySignalMapper::Data(pbNewMolecular,SIGNAL(clicked()),1)
				  <<MySignalMapper::Data(pbNewContrib,SIGNAL(clicked()),2)
				  <<MySignalMapper::Data(pbDelete,SIGNAL(clicked()),3)
				  <<MySignalMapper::Data(a,SIGNAL(triggered()),3)
				  <<MySignalMapper::Data(tbUp,SIGNAL(clicked()),4)
				  <<MySignalMapper::Data(tbDown,SIGNAL(clicked()),5)
				  <<MySignalMapper::Data(pbCopy,SIGNAL(clicked()),6)
				  );


  connect(trvCombis,SIGNAL(enableBase(bool)),this,SLOT(slotEnableBase(bool)));


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
BWEdit::~BWEdit()
{
  //myDebug() << "BWEdit:: here I die!\n";
  delete doc;
  if(signalMapper) delete signalMapper;
  QList<QAction*> aa=pbDelete->actions();
  qDeleteAll(aa);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BWEdit::isModified()
{
  //myDebug() <<"BWEdit::isModified";
  return false;
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::afterShow(const QString &load)
{
  //myDebug() << "afterShow" << load;
  if(!load.isEmpty()){
    QFileInfo fi=QFileInfo(load);
    QString ft=Master::fileType(load,false);
    if(ft=="directory"){
      doc->setCurrentDir(fi.absoluteFilePath());
      doc->readRelativisticMode();
      if(cbUseCoeffFile->isChecked()){
	QString fc=doc->bwfromcoeffs.currentCoeffFile()();
	QFileInfo fi(fc);
	if(fi.isFile()){
	  leCoefficients->setText(fc);
	  CoeffImport(fc,true);
	  populateTrvCombis();
	}
	
      }
    }else{
      doc->setCurrentDir(fi.absolutePath());
      doc->readRelativisticMode();
      QString f=load;
      if(workdir == doc->absolutePath(f)+'/'){
	f=doc->fileName(f);
      }
      if(ft=="coeff"){
	leCoefficients->setText(f);
	CoeffImport(f,true);
	populateTrvCombis();
      }else if(ft=="bwdef"){
	fileLoad(workdir+fi.fileName());
      }
    }

  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::setupMenus()
{
  QAction *a;
  //QMenu *m;

  QToolBar *mainToolBar=base()->getNewToolbar();
  mainToolBar->setObjectName("mainToolBar");
  mainToolBar->setContextMenuPolicy(Qt::NoContextMenu);
  mainToolBar->setWindowTitle("xfsf tool bar");

  QAction *fileAction=base()->getNewMenuBarItem(Actions::File);
  fileAction->setText("&File");


  a=MWBase::addMenu(fileAction,"&Open","Load Settings",Actions::File_Open
			     ,QIcon(":/Icons/fileopen")
			     ,QKeySequence(Qt::CTRL+Qt::Key_O));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileOpen()));
  mainToolBar->addAction(a); 

  a=MWBase::addMenu(fileAction,"&Save","Save Settings File",Actions::File_Save
			     ,QIcon(":/Icons/filesave")
			     ,QKeySequence(Qt::CTRL+Qt::Key_S));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSave()));
  mainToolBar->addAction(a); 

  reloadAction=a=MWBase::addMenu(fileAction,"&Reload","Reload current file (F5)"
				 ,Actions::File_Reload
		    ,QIcon(":/Icons/redo")
		    ,QKeySequence(Qt::Key_F5));
  connect(a,SIGNAL(triggered()),this,SLOT(slotReload()));
  mainToolBar->addAction(a); 
  a->setEnabled(false);

  a=MWBase::addMenu(fileAction,"Save &As","Save as",Actions::File_SaveAs );
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSaveAs()));


  QAction *closeAction=
    a=MWBase::addMenu(fileAction,"&Close","Close this activity"
		      ,Actions::File_Close
		      ,QIcon(":/Icons/exit")
		    ,QKeySequence(Qt::CTRL+Qt::Key_W));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileClose()));

  mainToolBar->addAction(closeAction);

  QAction *runAction=base()->getNewMenuBarItem(Actions::Run);
  runAction->setText("&Run");
//   a=MWBase::addMenu(runAction,"Run &Fplo","Run interactive FPLO session");
//   connect(a,SIGNAL(triggered()),this,SLOT(slotRunFPLO()));

  a=MWBase::addMenu(runAction,"&Terminal","Run an xterm in the current directory(On MAC xfplo must be started in X11/Terminal for this to work)"
		    ,Actions::Run_Terminal);
  connect(a,SIGNAL(triggered()),this,SLOT(slotRunTerminal()));

  a=MWBase::addMenu(runAction,"&External editor"
		    ,"Edit definitions in external editor"
		    ,Actions::Run_ExternalEditor);
  connect(a,SIGNAL(triggered()),this,SLOT(slotExternalEdit()));


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotFileSave() 
{ 
  saveit(FILESAVE); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotFileSaveAs()
{
  saveit(FILESAVEAS); 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::saveit( SaveMode saveas )
{
  int doit;
  QString fn;

  if(saveas != FILESAVE && saveas != FILESAVEAS  )return;
  
  if(saveas == FILESAVEAS || doc->isNew() ){
    QString f0=doc->absoluteFilePath();
    //    myDebug() <<"f0"<<f0;
//     if(doc->isNew()){
//       f0=workdir+"=.bwdef";
//     }


#ifdef Q_OS_MAC
    fn = QFileDialog::getSaveFileName(this,"Save File",f0);
#else
    fn = QFileDialog::getSaveFileName(this,"Save File",f0
				      , " FPLO files(=.*) ;; All files(*)"
				      ,0,0);
#endif

    if ( (doit=!fn.isEmpty()) ) {
      //todo   doit=overWriteFile(fn);
    }
  }else{
    fn=doc->absoluteFilePath();
    doit=1;
  }
  if ( doit ) {
    try{
      doc->bwfromcoeffs.save(fn);
    }catch(QString msg){
      QMessageBox::warning(this,"Save",msg,QMessageBox::Yes);
      return;
    }
    
    doc->setFileName(fn);
    doc->setCurrentDir(doc->absolutePath());
    doc->readRelativisticMode();
    doc->setOld();
    reloadAction->setEnabled(true);

    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());
//     statusbar->setDocument(doc->absoluteFilePath());
//     statusbar->setMessage(QString("Saved %1")
// 			  .arg(doc->absoluteFilePath()),1000);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotFileOpen()
{
  //x  statusbar->setMessage("Opening file...");
  QString f0=doc->absoluteFilePath();
//   if(doc->isNew()){
//     f0=workdir+"=.xstr";
//   }

  QString fileName = QFileDialog::getOpenFileName(this,"Open File",f0
						  ,"FPLO input (=.*);;All files (*)");

  if (!fileName.isEmpty()) 
    fileLoad(fileName);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::fileLoad(const QString &fileName)
{
  //myDebug() << "BWEdit::fileLoad" << fileName ;

  QString ft=Master::fileType(fileName,false);
  if(ft!="bwdef"){
    QMessageBox::warning(this,"Open","File type is no "
			 "band weight definitions file",QMessageBox::Yes);
    return ;
  }

  //myDebug() << "hier1" ;
  doc->saveCurrentDir();
  QFileInfo fi(fileName);
  bool isfile=fi.isFile();
  doc->restoreCurrentDir();
  if(!isfile){
    myDebug() << "file " << fileName << "is not a file";
    return ;
  }


  //myDebug() << "hier2" ;


  if (!fileName.isEmpty()) {
    
    try{
      doc->bwfromcoeffs.load(fileName);
      doc->setOld();
      reloadAction->setEnabled(true);
    }catch(Exception ex){
      QMessageBox::warning(this,"Open",ex.msg()(),QMessageBox::Yes);
      if(ex.severity()>Exception::warning)
	return;
      else{
	doc->setOld();
	reloadAction->setEnabled(true);
      }
	
      //statusbar->setDocument(workdir);
    }

    doc->setFileName(fileName);
    doc->setCurrentDir(doc->absolutePath());
    doc->readRelativisticMode();

    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());
    


    QString fc=doc->bwfromcoeffs.currentCoeffFile()();
    leCoefficients->setText(fc);
    QString fw=doc->bwfromcoeffs.currentBWFile()();
    leBandWeights->setText(fw);
    CoeffImport(fc,false);
    populateTrvCombis();


    //todo: check consistency

//     QString message="Loaded document: "+fileName;
    
//     statusbar->setDocument(doc->absoluteFilePath());

  
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotReload()
{
  if(!doc->isNew())
    fileLoad(doc->absoluteFilePath());
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotExternalEdit()
{
  ///myDebug () << "slotExternalEdit";
  qint64 pid;
  
  QString f0=doc->absoluteFilePath();


  //  fileSystemWatcher.addPath(f0);
  //  myDebug() << "files:"<<fileSystemWatcher.files();
//   connect(&fileSystemWatcher,SIGNAL(fileChanged(const QString &))
// 	  ,this,SLOT(filemodified(const QString &)));

  Settings set("application");
  QString editor=set.getString("external editor","vi");
  QString term="xterm";
  QStringList args;
  if(set.getBool("external editor in terminal",true)){
    args<<"-e"<<editor<<f0;
    if(!QProcess::startDetached(term,args,doc->currentDir(),&pid)){
      QMessageBox::warning(this,"execute",QString("Could not execute %1!")
			   .arg(term+args.join(" "))
			   ,QMessageBox::Yes);

    }
  }else{
    String s(editor);
    long i=s.find("%f");
    if(i<0){
      s+=" %f";
      i=s.find("%f");
    }
    s=s.subString(0,i-1)+String(f0)+s.subString(i+2);
    StringList sl=s.split(" \t");
    if(sl.size()>0){
      String prog=sl[0];
      QStringList args;
      for(int i=1;i<sl.size();i++) args<<sl[i]();
      //myDebug() << prog()  << args;
      if(!QProcess::startDetached(prog(),args,doc->currentDir(),&pid)){
	QMessageBox::warning(this,"execute",QString("Could not execute %1!")
			     .arg(prog()+args.join(" "))
			     ,QMessageBox::Yes);
      }
    }
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void BWEdit::filemodified(const QString & path)
// {
//   myDebug() << path << "changed";
// }


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotFileClose()
{
  //myDebug() <<"BWEdit::slotFileClose";
  if(queryExit()){
    
    base()->finish();
    //myDebug() <<" STR::slotFileClose after base()->finish";
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool BWEdit::queryExit()
{
  return ControlBase::queryExit();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::help()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","FatbandEditor");
  h->show();

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::CoeffImport(const QString &fileName,bool forceload)
{
  if(!cbUseCoeffFile->isChecked())return;
  
  QString fn=doc->absoluteFilePath(fileName);
  //myDebug()<<"CoeffImport"<<workdir<<fileName<<fn;

  QString ft=Master::fileType(fn,false);
  //myDebug()<<"ft="<<ft;
  
  if(ft!="coeff"){
    QMessageBox::warning(this,"Open",QString("File %1 is no usefull coefficient file")
			 .arg(fn)
			 ,QMessageBox::Yes);
    return ;
  }
  
  if(fn.isEmpty()) return;

  statusbar->setMessage("reading coefficients file ...");
  statusbar->setAllotedPercent(1.0);
  statusbar->setProgress(0.01);

  try{
    //doc->saveCurrentDir();
    connect(doc->bwfromcoeffs.coefficientData(),SIGNAL(progress(int)),
	    statusbar,SLOT(setProgressInt(int)) );

    enableOnCoeffRead(false);

    doc->bwfromcoeffs.readCoeffsFromFile(fn.toAscii().constData(),forceload);
    //doc->restoreCurrentDir();
  }catch(Exception &ex){
    QMessageBox::warning(this,"Loading coefficients",ex.msg()()
			 ,QMessageBox::Ok | QMessageBox::Default);
    return ;
  }

  enableOnCoeffRead(true);
  
  statusbar->setProgress(1.);
  statusbar->setProgress(0.);
  statusbar->setMessage();


  //statusbar->setDocument(doc->absoluteFilePath());

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BWEdit::slotPlotCalc()
{
  if(!doc->bwfromcoeffs.coeffsAreReady()){
    QMessageBox::warning(this,"Open","Load a coefficient file first!\n"
			 "Or put =.bwdef into the fedit bandplot menu and run FPLO."
			 ,QMessageBox::Ok | QMessageBox::Default);
    return;
  }

  QString f0=doc->bwfromcoeffs.currentBWFile()();

  if(f0.isEmpty()) {
    QMessageBox::warning(this,"Calculate","Define a band weights output "
			 "file first!"
			 ,QMessageBox::Ok | QMessageBox::Default);
    
    return;
  }

  enableOnCoeffRead(false);
  statusbar->setMessage("building band weights ...");
  statusbar->setAllotedPercent(1.0);
  statusbar->setProgress(0.01);

  try{


    try{
      doc->saveCurrentDir();
      Parser inparser(1);
      String fname="=.in";
      inparser.parseFile(fname);
      Vec3 qaxis;
      inparser()("quantization_axis").getArray(3,qaxis());
      doc->bwfromcoeffs.setGlobalSpinZaxis(qaxis);
      doc->restoreCurrentDir();
    }catch(Except &ex){
      QMessageBox::warning(this,"Creating bandweights file",ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default);
      goto end;
    }
    connect(doc->bwfromcoeffs.coefficientData(),SIGNAL(progress(int)),
	    statusbar,SLOT(setProgressInt(int)) );
    doc->bwfromcoeffs.makeBandWeights(f0.toAscii().constData());
    
  }catch(Exception &ex){
    QMessageBox::warning(this,"Creating bandweights file",ex.msg()()
			 ,QMessageBox::Ok | QMessageBox::Default);
    goto end;
  }

 end:
  enableOnCoeffRead(true);
  statusbar->setProgress(1.);
  statusbar->setProgress(0.);
  statusbar->setMessage();
  return;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::populateTrvCombis()
{
  //myDebug() <<"BWEdit::populateTrvCombis"<<doc;
  if(!doc)return;

  trvCombis->resetModel(doc->bwfromcoeffs.molOrbDefs(),
			doc->bwfromcoeffs.coeffBasisInfo());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::on_pbBuildWeights_clicked()
{
  //myDebug() << "BWEdit::on_pbBuildWeights_clicked";
  slotPlotCalc();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::on_leBandWeights_textChanged()
{
  //  myDebug() << "BWEdit::on_leBandWeights_textChanged";
  QString fw=leBandWeights->text();
  doc->bwfromcoeffs.setCurrentBWFile(fw);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::on_leCoefficients_textChanged()
{
  //myDebug() << "BWEdit::on_leCoefficients_textChanged";
  QString fw=leCoefficients->text();
  doc->bwfromcoeffs.setCurrentCoeffFile(fw);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BWEdit::on_tbBrowseBWF_clicked()
{
  //  myDebug() << "BWEdit::on_tbBrowseBWF_clicked";

  QString f0=doc->absoluteFilePath(doc->bwfromcoeffs.currentBWFile()());
  QString fw = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"FPLO output (+*);;All files (*)");
  if(fw.isEmpty()) fw=f0;
  if(workdir == doc->absolutePath(fw)+'/'){
    fw=doc->fileName(fw);
  }

  leBandWeights->setText(fw);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void BWEdit::on_tbBrowse_clicked()
{
  //myDebug() << "BWEdit::on_tbBrowse_clicked";


  QString f0=doc->absoluteFilePath(doc->bwfromcoeffs.currentCoeffFile()());

  //myDebug() << "f0" << f0;
  QString fc = 
    QFileDialog::getOpenFileName(this,"Open File",f0

				 ,"FPLO output (+*);;All files (*)");
  
  if(fc.isEmpty()) fc=f0;
  if(workdir == doc->absolutePath(fc)+'/'){
    fc=doc->fileName(fc);
  }
  //myDebug() << "fc" << fc;

  leCoefficients->setText(fc);
  
  CoeffImport(fc,true);
  populateTrvCombis();

  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::on_pbRefresh_clicked()
{
  trvCombis->resetModel();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::slotEnableBase(bool en)
{
  base()->enable_but(en,QString("activities help").split(" "));
  reloadAction->setEnabled(en);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::on_cbUseCoeffFile_toggled(bool onoff)
{

  Settings set("./bwdef.ini","Controls");
  set.set("cbUseCoeffFile",onoff);

  lCoefficients->setVisible(onoff);
  leCoefficients->setVisible(onoff);
  tbBrowse->setVisible(onoff);
  pbBuildWeights->setVisible(onoff);
  QString txt="<span style=\" color:#0000c0;\">Define individual band weights"
    " and put the band weights file into fedit bandplot menu to replace "
    "default FPLO band weights";
  if(onoff){
    lMesg->setText(txt+" or use +coeff file.</span>");
    QString fc=doc->bwfromcoeffs.currentCoeffFile()();
    CoeffImport(fc,false);
    populateTrvCombis();
  }else{
    lMesg->setText(txt+".</span>");
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void BWEdit::enableOnCoeffRead(bool onoff)
{
  pbBuildWeights->setEnabled(onoff);
  pbNewAtomic->setEnabled(onoff);
  pbNewMolecular->setEnabled(onoff);
  pbNewContrib->setEnabled(onoff);
  pbDelete->setEnabled(onoff);
  tbUp->setEnabled(onoff);
  tbDown->setEnabled(onoff);
  pbCopy->setEnabled(onoff);
  trvCombis->setEnabled(onoff);
  pbRefresh->setEnabled(onoff);
  tbBrowse->setEnabled(onoff);
  tbBrowseBWF->setEnabled(onoff);
  leBandWeights->setEnabled(onoff);
  cbUseCoeffFile->setEnabled(onoff);
  
}



/*EOF*/ 
