/*************************************************************************
 * @(#)ControlBase.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Jul 2009
 *************************************************************************
 */
#include <QMessageBox>
#include <QProcess>
#include "ControlBase.hh"
#include "MWBase.hh"
#include "Status.hh"
#include "DocBase.hh"
#include "AfterShowEvent.hh"
#include "AnnotationDlg.hh"
#include "OGLView.hh"
#include "CanvasDlg.hh"
#include "Config.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
ControlBase::ControlBase(MWBase *parent)
  :QWidget(parent),pbase(parent),docbase(0),qlviewwidget(0)
  ,statusbar(0),annotationdlg(0)

{
  MASSERT("",pbase);

  statusbar=qobject_cast<Status *> (pbase->statusBar());
  MASSERT("",statusbar);


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
ControlBase::~ControlBase()
{
  if(!annotationdlg.isNull()) delete annotationdlg;annotationdlg=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ControlBase::queryExit()
{
  //  myDebug() <<"ControlBase::queryExit";
  
  if(!isModified()){
    return true;
  }else{
    int exit=QMessageBox::
      information(this, "Quit..."
		  ,"There might be unsafed changes?"
		  ,QMessageBox::Discard| QMessageBox::Default
		  , QMessageBox::Cancel| QMessageBox::Escape);
    if(exit==QMessageBox::Discard) return true;
  }
  return false;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::setupDocAndView(OGLView *ov,DocBase *d)
{
  if(d){
    docbase=d;
  }

  if(ov){
    qlviewwidget=ov;

    connect(ov,SIGNAL(sizeChanged(const OGLView *)),this
	    ,SLOT(oglSizeChanged(const OGLView *)));
  }
  

  if( d && ov){
    MASSERT("Oops ",statusbar);
    ov->init(d);//todo: this is reverted logic
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::setupDoc(DocBase *d)
{
  if(d){
    docbase=d;
  }


  

  if( d){
    MASSERT("Oops ",statusbar);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::slotCanvasDefine()
{
  if(!docbase) return;
  CanvasDlg *cdlg=new CanvasDlg(this,"CanvasDlg");
  cdlg->setData(docbase->canvas());
  cdlg->setWindowTitle(IDS_APP_NAME + " : Define canvas size" );

  int ret=cdlg->exec();
  if(ret){
    cdlg->getData(docbase->canvas());
    
    statusbar->setCanvasControl(docbase->canvas());  
    
    slotCanvasLockClicked(docbase->canvas().locked());
  }

  delete (cdlg);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::slotCanvasLockClicked(bool locked)
{
  //  myDebug() << "ControlBase::slotCanvasLockClicked" << locked;
  if(!docbase) return;
  if(!qlviewwidget) return;
  //  myDebug() << "hier" << locked;


  docbase->setCanvasLocked(locked);



  if(locked){
    qlviewwidget->setMinimumSize(docbase->canvas().width()
				 ,docbase->canvas().height());
    qlviewwidget->setMaximumSize(docbase->canvas().width()
				 ,docbase->canvas().height());
    qlviewwidget->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  }else{
    qlviewwidget->setMinimumSize(0,0);
    qlviewwidget->setMaximumSize(0xffffff,0xffffff);
    qlviewwidget->setSizePolicy(sizepolicy);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::oglSizeChanged(const OGLView *v)
{
  if(!docbase) return;

  if(!docbase->canvas().locked()){
    docbase->setCanvasSize(CanvasSize(false,v->width(),v->height()));
    statusbar->setCanvasControl(docbase->canvas());  
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long ControlBase::overWriteFile( const QString & file )
{
  int doit=1;
  if(QFileInfo(file).exists()){
    QString text="File "+file+" exists!\nOverwrite?" ;
    doit=(QMessageBox::Yes==
	  QMessageBox::warning(qApp->activeWindow(),"Save file", text,
			       QMessageBox::Yes,QMessageBox::No ));
  }
  return doit;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::help()
{
  myDebug() << "ControlBase::help";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool ControlBase::event(QEvent *event)
{
  if(event->type()==AfterShowEvent::AfterShowEventType){
    if(AfterShowEvent *ae=dynamic_cast<AfterShowEvent*>(event)){
      afterShow(ae->toLoad());
    }
    return true;
  }
  return QWidget::event(event);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void ControlBase::slotRunTerminal()
{
  qint64 pid;
  QString term="xterm";
  bool ret=QProcess::startDetached(term,QStringList(),docbase->currentDir(),&pid);
  if(!ret){
    QMessageBox::warning(this,"execute",QString("Could not execute %1!")
			 .arg(term)
			 ,QMessageBox::Yes);

  }

}
