/* @(#)ControlBase.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef CONTROLBASE_H
#define CONTROLBASE_H

#include <QPointer>
#include <QWidget>
#include <QSizePolicy>
#include <QString>


class MWBase;
class Status;
class DocBase;
class OGLView;
class AnnotationDlg;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class ControlBase: public QWidget
{
  Q_OBJECT
public:
  ControlBase(MWBase *parent=0);
  virtual ~ControlBase();


  MWBase *base(){return pbase;};
  virtual bool queryExit();
  virtual bool isModified()=0;
  DocBase *document() {return docbase;} ;
  virtual void help();
  

  OGLView *oglView(){return qlviewwidget;};
  QPointer<AnnotationDlg> &annotationDlg(){return annotationdlg;};

public slots:

  virtual void slotPlotCalc(bool updatePanZoom=false){Q_UNUSED(updatePanZoom);}
  virtual void updateData(bool updatePanZoom=false){Q_UNUSED(updatePanZoom);}
  virtual void slotRunTerminal();

  
protected:
  virtual void afterShow(const QString &load)=0;
 void setupDocAndView(OGLView *ov,DocBase *d);
 void setupDoc(DocBase *d);
 virtual bool event(QEvent *event);

protected slots:
  void slotCanvasDefine();

  long overWriteFile( const QString & file );
  
  void slotCanvasLockClicked(bool);
  void oglSizeChanged(const  OGLView *); // todo:remove

protected:
  MWBase *pbase;
  DocBase *docbase;
  OGLView *qlviewwidget;
  Status *statusbar;
  QSizePolicy sizepolicy;
  QPointer<AnnotationDlg> annotationdlg;
};

#endif // CONTROLBASE_H
