/*
todo: make output as sqrt(m/n)
todo: save dmats and make selection
todo: use symmetry for axes and when loading in fplo
todo: natural orbital analysis
*/
#include <QStringList>
#include <QMessageBox>
#include <QSignalMapper>
#include <QMenu>
#include <QDir>
#include "DMEdit.hh"
#include "Except.hh"
#include "DMDoc.hh"
#include "Settings.hh"
#include "DMConfig.hh"
#include "AxisEdit.hh"
#include "Valid.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"







/* ***********************************************************************
 *
 * ***********************************************************************
 */

DMEdit::DMEdit(QWidget *parent) 
  : QDialog(parent),protectTableSpinUp(false),protectTableSpinDown(false),
    protectTableSpinMixed(false),starttables(false),
    allfilter("(All)"),disptol(1.0e-10),
    setnullactionUp(0),setnullactionDown(0),setnullactionMixed(0),
    setunitactionUp(0),setunitactionDown(0),setunitactionMixed(0),
    setflipactionUp(0),setflipactionDown(0),
    setfliplzactionUp(0),setfliplzactionDown(0),
    setiactionMixed(0),signalMapper(0),xyzvalid(0),quitAction(0)
{
  setupUi(this);

  xyzvalid=new RealValidator(this,0,"occupation number: ",-1e10,1e10);

  // restore global settings
  Settings set(dmmainsettings,"DMEdit");
  resize(set.getSize("Geometry/Size",QSize(600, 600)));
  move(set.getPoint("Geometry/Pos",QPoint(200, 200)));
  spbxColSize->setValue(set.getInt("Tables/ColumnSize",50));
  spbx0Tol->setValue(set.getInt("Tables/Tol0Exp",5));

  cmbRep->addItem("real part",Real);
  cmbRep->addItem("imaginary part",Imag);
  cmbRep->addItem("complex",Complex);


  
  QAction *a;
  QMenu *m=new QMenu(tbDefaultUp);
  a=setnullactionUp=m->addAction("Set &zero");  
  a=setunitactionUp=m->addAction("Set &Unity");  
  a=setflipactionUp=m->addAction("&Flip spin");
  a=setfliplzactionUp=m->addAction("&Flip Lz");

  tbDefaultUp->setMenu(m);
  tbDefaultUp->setDefaultAction(setnullactionUp);

  m=new QMenu(tbDefaultDown);
  a=setnullactionDown=m->addAction("Set &zero");  
  a=setunitactionDown=m->addAction("Set &Unity");  
  a=setflipactionDown=m->addAction("&Flip spin");  
  a=setfliplzactionDown=m->addAction("&Flip Lz");  

  tbDefaultDown->setMenu(m);
  tbDefaultDown->setDefaultAction(setnullactionDown);

  m=new QMenu(tbDefaultMixed);
  a=setnullactionMixed=m->addAction("Set &zero");  
  a=setunitactionMixed=m->addAction("Set &Unity");  
  a=setiactionMixed=m->addAction("Set &I-Unity");  

  tbDefaultMixed->setMenu(m);
  tbDefaultMixed->setDefaultAction(setnullactionMixed);

  signalMapper = new QSignalMapper(this);
  signalMapper->setMapping(setnullactionUp, Zero);
  signalMapper->setMapping(setunitactionUp, Unity);
  signalMapper->setMapping(setflipactionUp, Flip);
  signalMapper->setMapping(setfliplzactionUp, FlipLz);

  signalMapper->setMapping(setnullactionDown, Zero+Down);
  signalMapper->setMapping(setunitactionDown, Unity+Down);
  signalMapper->setMapping(setflipactionDown, Flip+Down);
  signalMapper->setMapping(setfliplzactionDown, FlipLz+Down);

  signalMapper->setMapping(setnullactionMixed,Zero+Mixed);
  signalMapper->setMapping(setunitactionMixed, Unity+Mixed);
  signalMapper->setMapping(setiactionMixed, IUnity+Mixed);

  connect(setnullactionUp,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setunitactionUp,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setflipactionUp,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setfliplzactionUp,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setnullactionDown,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setunitactionDown,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setflipactionDown,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setfliplzactionDown,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setnullactionMixed,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setunitactionMixed,SIGNAL(triggered()),signalMapper,SLOT(map()));
  connect(setiactionMixed,SIGNAL(triggered()),signalMapper,SLOT(map()));


  connect(signalMapper,SIGNAL(mapped(int)),
	  this,SLOT(onTbDefault(int)));

  a=quitAction=new QAction(this);
  a->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_Q));
  connect(a,SIGNAL(triggered()),this,SLOT(on_pbQuit_clicked()));
  addAction(a);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
DMEdit::~DMEdit()
{
  if(xyzvalid)  delete xyzvalid;
  if(setnullactionUp) delete setnullactionUp;
  if(setnullactionDown) delete setnullactionDown;
  if(setnullactionMixed) delete setnullactionMixed;
  if(setunitactionUp) delete setunitactionUp;
  if(setunitactionDown) delete setunitactionDown;
  if(setunitactionMixed) delete setunitactionMixed;
  if(setiactionMixed) delete setiactionMixed;
  if(setflipactionUp) delete setflipactionUp;
  if(setfliplzactionUp) delete setfliplzactionUp;
  if(setflipactionDown) delete setflipactionDown;
  if(setfliplzactionDown) delete setfliplzactionDown;

  if(quitAction) delete quitAction;
  if(signalMapper) delete signalMapper;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::onTbDefault(int map)
{
  int ms=map;
  SetDefault mode;
  if(ms<Mixed) {
    ms=ms/Down+1;
    mode=(SetDefault)(map%Down);
  }else{
    ms=3;
    mode=(SetDefault)(map%Mixed);
  }
  ms--;

  if(ms==0){
    tbDefaultUp->
      setDefaultAction(qobject_cast<QAction*>(signalMapper->mapping(map)));

    setTableDefault(mode,ms);
  }else if(ms==1){
    tbDefaultDown->
      setDefaultAction(qobject_cast<QAction*>(signalMapper->mapping(map)));
    setTableDefault(mode,ms);
  }else if(ms==2){
    tbDefaultMixed->
      setDefaultAction(qobject_cast<QAction*>(signalMapper->mapping(map)));
    setTableDefault(mode,ms);
  }



  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
int DMEdit::getCurrentOccu()
{
  QListWidgetItem *currentitem=lwDMat->currentItem();
  if(!currentitem) return -1;
  int ioccu=currentitem->data(Qt::UserRole).toInt();
  if(ioccu<0 || ioccu>=dmdoc->occus().size()) return -1;
  return ioccu;
};

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::saveMainSettings()
{
  //myDebug() << mainsettings;

  {
    Settings set(dmmainsettings,"DMEdit");

    set.set("Geometry/Size", size());
    set.set("Geometry/Pos", pos());
    set.set("Tables/ColumnSize",spbxColSize->value());
    set.set("Tables/Tol0Exp",spbx0Tol->value());
  }
  {
    Settings set("./dmatedit.ini","Controls");

    ShowPart rep=getShowPart();
    set.set("Rep", rep);
    set.set("Element", cmbElement->currentText());
    set.set("Site", cmbSite->currentText());
    set.set("Orbital", cmbOrbital->currentText());
    int ylm=cmbYlm->itemData(cmbYlm->currentIndex()).toInt();
    set.set("YlmRep", ylm);

    QListWidgetItem *w=lwDMat->currentItem();
    if(w) set.set("OccuMat", w->text());
    
    

    int col=MyMath::max<long>(twSpinUp->currentColumn(),0);
    int row=MyMath::max<long>(twSpinUp->currentRow(),0);
    set.set("Spinup/Column",dmIndexFromColumn(col,rep));
    set.set("Spinup/ColumnReIm",dmReImFromColumn(col,rep));
    set.set("Spinup/Row", row);
    set.set("Spinup/Focus", twSpinUp->hasFocus());

    col=MyMath::max<long>(twSpinDown->currentColumn(),0);
    row=MyMath::max<long>(twSpinDown->currentRow(),0);
    set.set("Spindown/Column", dmIndexFromColumn(col,rep));
    set.set("Spindown/ColumnReIm", dmReImFromColumn(col,rep));
    set.set("Spindown/Row", row);
    set.set("Spindown/Focus", twSpinDown->hasFocus());
    
    col=MyMath::max<long>(twSpinMixed->currentColumn(),0);
    row=MyMath::max<long>(twSpinMixed->currentRow(),0);
    set.set("Spinmixed/Column", dmIndexFromColumn(col,rep));
    set.set("Spinmixed/ColumnReIm", dmReImFromColumn(col,rep));
    set.set("Spinmixed/Row", row);
    set.set("Spinmixed/Focus", twSpinMixed->hasFocus());
    
    QAction *a=tbDefaultUp->defaultAction();
    SetDefault sd=Zero;
    if(a==setunitactionUp) sd=Unity;
    if(a==setflipactionUp) sd=Flip;
    if(a==setfliplzactionUp) sd=FlipLz;
    set.set("TbDefaultUp/action",sd);

    a=tbDefaultDown->defaultAction();
    sd=Zero;
    if(a==setunitactionDown) sd=Unity;
    if(a==setflipactionDown) sd=Flip;
    if(a==setfliplzactionDown) sd=FlipLz;
    set.set("TbDefaultDown/action",sd);

    a=tbDefaultMixed->defaultAction();
    sd=Zero;
    if(a==setunitactionMixed) sd=Unity;
    if(a==setiactionMixed) sd=IUnity;
    set.set("TbDefaultMixed/action",sd);



  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void DMEdit::setData(DMDoc *d)
{
  //myDebug() << "DMEdit::setData";
  dmdoc=d;
  Q_ASSERT(d != NULL);


  lQuant->setText(Sprintf("Spin z-axis (FPLO): (%s)",
			  dmdoc->globalSpinZaxis().printS("%5g",",")())());
  
  dmdoc->loadAxes();

  setWindowTitle(Sprintf(" Editing '%s'",dmdoc->fileName()())());
  if(dmdoc->mode()=="dmat"){
    lMatrix->setText("Occupation Number Matrix");
  }else{
    lMatrix->setText("Potential Matrix");
  }

  
  populateCmbElement();

  //restore local settings
  Settings set("./dmatedit.ini","Controls");

  ShowPart rep=(ShowPart)set.getInt("Rep",Real);
  long ylmrep=set.getInt("YlmRep",Occu::RealYlm);
  QString el=set.getString("Element","(All)");
  QString site=set.getString("Site","(All)");
  QString o=set.getString("Orbital","(All)");
  QString dm=set.getString("OccuMat",0);
  long colup=set.getInt("Spinup/Column",0);
  long reimup=set.getInt("Spinup/ColumnReIm",0);
  long rowup=set.getInt("Spinup/Row",0);
  bool focusup=set.getBool("Spinup/Focus",0);
  long coldn=set.getInt("Spindown/Column",0);
  long reimdn=set.getInt("Spindown/ColumnReIm",0);
  long rowdn=set.getInt("Spindown/Row",0);
  bool focusdn=set.getBool("Spindown/Focus",0);
  long colmx=set.getInt("Spinmixed/Column",0);
  long reimmx=set.getInt("Spinmixed/ColumnReIm",0);
  long rowmx=set.getInt("Spinmixed/Row",0);
  bool focusmx=set.getBool("Spinmixed/Focus",0);
  SetDefault sdup=(SetDefault)set.getInt("TbDefaultUp/action",Zero);
  SetDefault sddn=(SetDefault)set.getInt("TbDefaultDown/action",Zero);
  SetDefault sdmx=(SetDefault)set.getInt("TbDefaultMixed/action",Zero);

  cmbYlm->addItem("Real Ylm",Occu::RealYlm);
  cmbYlm->addItem("Complex Ylm",Occu::ComplexYlm);
  if(dmdoc->isFREL()){
    cmbYlm->addItem("Spherical spinors",Occu::SphericalSpinor);
  }


  for(int i=0;i<cmbYlm->count();i++){
    if(cmbYlm->itemData(i).toInt() == ylmrep){
      cmbYlm->setCurrentIndex(i);
      break;
    }
  }

  for(int i=0;i<cmbRep->count();i++){
    if(cmbRep->itemData(i).toInt() == rep){
      cmbRep->setCurrentIndex(i);
      break;
    }
  }

  //myDebug()<<"set current Element";
  for(int i=0;i<cmbElement->count();i++){
    if(cmbElement->itemText(i) == el){
      cmbElement->setCurrentIndex(i);
      break;
    }
  }

  //myDebug()<<"set current Site";
  for(int i=0;i<cmbSite->count();i++){
    if(cmbSite->itemText(i) == site){
      cmbSite->setCurrentIndex(i);
      break;
    }
  }
  //myDebug()<<"set current Orbital";
  for(int i=0;i<cmbOrbital->count();i++){
    if(cmbOrbital->itemText(i) == o){
      cmbOrbital->setCurrentIndex(i);
      break;
    }
  }
  
  for(int i=0;i<lwDMat->count();i++){
    if(lwDMat->item(i)->text() == dm){
      lwDMat->setCurrentRow(i);
      break;
    }
  }





  starttables=true;
  populateTables();
    
  {
    QTableWidget  *tw;
    tw=twSpinUp;
    int N=(rep==Complex?2:1);
    reimup=(rep==Complex?reimup:0);
    int maxCol=tw->columnCount()/N;
    colup=MyMath::min<long>(colup,maxCol-1)*N+reimup;
    rowup=MyMath::min<long>(rowup,tw->rowCount()-1);
    tw->setCurrentCell(rowup,colup);

    tw=twSpinDown;
    N=(rep==Complex?2:1);
    reimdn=(rep==Complex?reimdn:0);
    maxCol=tw->columnCount()/N;
    coldn=MyMath::min<long>(coldn,maxCol-1)*N+reimdn;
    rowdn=MyMath::min<long>(rowdn,tw->rowCount()-1);
    tw->setCurrentCell(rowdn,coldn);

    tw=twSpinMixed;
    N=(rep==Complex?2:1);
    reimmx=(rep==Complex?reimmx:0);
    maxCol=tw->columnCount()/N;
    colmx=MyMath::min<long>(colmx,maxCol-1)*N+reimmx;
    rowmx=MyMath::min<long>(rowmx,tw->rowCount()-1);
    tw->setCurrentCell(rowmx,colmx);
  }

  if(focusup){
    twSpinUp->setFocus();
  }else if(focusdn){
    twSpinDown->setFocus();
  }else if(focusmx){
    twSpinMixed->setFocus();
  }else{
    twSpinUp->setFocus();
  }

  tbDefaultUp->
    setDefaultAction(qobject_cast<QAction*>(signalMapper->mapping(sdup)));

  tbDefaultDown->
    setDefaultAction(qobject_cast<QAction*>(signalMapper->
					    mapping(sddn+Down)));

  tbDefaultMixed->
    setDefaultAction(qobject_cast<QAction*>(signalMapper->
					    mapping(sdmx+Mixed)));

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::reject()
{
  //myDebug() << "DMEdit::reject";

  saveMainSettings();
  QDialog::reject();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_pbQuit_clicked()
{
  //myDebug() << "DMEdit::on_pbQuit_clicked";

  saveMainSettings();
  
  QWidget *savew=saveTableFocus(pbQuit);

  if(!dataAcceptable()) {
    if(savew){
      savew->setFocus();
    }
    return ;
  }


  dmdoc->save();
  QDialog::accept();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QWidget *DMEdit::saveTableFocus(QWidget *me)
{
  QWidget *w=focusWidget();
  while(w){
    //myDebug() << w;
    if(w==twSpinUp || w== twSpinDown || w== twSpinMixed){
      break;
    }
    w=qobject_cast<QWidget*>(w->parent());
  }
  QWidget *savew=0;
  if(w){
    savew=w;
    me->setFocus();
  }
  return savew;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void DMEdit::on_pbNext_clicked()
{
  //myDebug() << "DMEdit::on_pbNext_clicked"; 
  int row=lwDMat->currentRow()+1;
  if(row>=lwDMat->count()){
    row=0;
  }
  lwDMat->setCurrentRow(row);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void DMEdit::on_pbPrev_clicked()
{
  //myDebug() << "DMEdit::on_pbPrev_clicked"; 

  int row=lwDMat->currentRow()-1;
  if(row<0){
    row=lwDMat->count()-1;
  }
  lwDMat->setCurrentRow(row);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_pbSetAxis_clicked()
{

  QWidget *savew=saveTableFocus(pbSetAxis);
  
  int ioccu=getCurrentOccu();

  
  if(ioccu<0 || ioccu> dmdoc->occus().size()-1) return;
  
  AxisEdit ae(this);
  ae.setData(dmdoc,ioccu,dmdoc->occus().at(ioccu).currentAxes());
  if(ae.exec()){
    ae.getData();
    dmdoc->saveAxes();
    //myDebug() << "take this";
    //myDebug() << dmdoc->currentAxes();
    //     myDebug() << dmdoc->axes().xaxis.printS()();
    //     myDebug() << dmdoc->axes().zaxis.printS()();
    
    populateTables();
    
  }
  if(savew){
    savew->setFocus();
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_cmbElement_currentIndexChanged(int elindex)
{
  if(elindex==-1) return;
  //myDebug() << "DMEdit::on_cmbElement_currentIndexChanged: " << elindex;

  //   QString el=cmbElement->itemText(elindex);
  //   myDebug() << el;
  
  populateCmbSite();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_cmbSite_currentIndexChanged(int siteindex)
{
  if(siteindex==-1) return;
  //myDebug() << "DMEdit::on_cmbSite_currentIndexChanged: "<< siteindex;

  //  QString si=cmbSite->itemText(siteindex);
  //  myDebug() << si;

  populateCmbOrbital();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_cmbOrbital_currentIndexChanged(int orbindex)
{
  if(orbindex==-1) return;
  //myDebug() << "DMEdit::on_cmbOrbital_currentIndexChanged: "<< orbindex;

  //  QString orb=cmbOrbital->itemText(orbindex);
  //  myDebug() << orb;

  populateLwDMat();  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_lwDMat_currentItemChanged(QListWidgetItem *currentitem
					  ,QListWidgetItem *previousitem)
{
  Q_UNUSED(currentitem);
  Q_UNUSED(previousitem);
  if(!currentitem) return;
  //  myDebug() << "DMEdit::on_lwDMat_currentItemChanged";

  populateTables();  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_cmbRep_currentIndexChanged(int index)
{
  Q_UNUSED(index);
  //myDebug() << "DMEdit::on_cmbRep_currentIndexChanged" ; 

  populateTables();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_cmbYlm_currentIndexChanged(int index)
{
  Q_UNUSED(index);
  //myDebug() << "DMEdit::on_cmbYlm_currentIndexChanged" ; 

  populateTables();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_spbxColSize_valueChanged(int v)
{
  //myDebug() << "DMEdit::on_spbxColSize_valueChanged" << v;
  twSpinUp->horizontalHeader()->setDefaultSectionSize(v);
  twSpinDown->horizontalHeader()->setDefaultSectionSize(v);
  twSpinMixed->horizontalHeader()->setDefaultSectionSize(v);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::on_spbx0Tol_valueChanged(int v)
{
  //myDebug() << "DMEdit::on_spbxColSize_valueChanged" << v;
  disptol=::pow(10.,-v);
  populateTables();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::populateCmbElement()
{
  //myDebug() << "DMEdit::populateCmbElement";
  
  QStringList sle;
  sle.append(allfilter);
  
  for(int i=0;i<dmdoc->occus().size();i++){
    const Occu &oc=dmdoc->occus().at(i);
    //myDebug() << oc.el();
    if(!sle.contains(oc.el()())){
      sle.append(oc.el()());
    }
  }
  
  cmbElement->clear();
  cmbElement->addItems(sle);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::populateCmbSite()
{
  //myDebug() << "DMEdit::populateCmbSite";

  QString el=cmbElement->currentText();

  QString currentSite=cmbSite->currentText();

  QStringList sls;
  sls.append(allfilter);
  for(int i=0;i<dmdoc->occus().size();i++){
    const Occu &oc=dmdoc->occus().at(i);
    if( el==allfilter || QString(oc.el()()) == el  ){
      QString s=QString("%1").arg(oc.site());
      if(!sls.contains(s)){
	sls.append(s);
      }
    }
  }
  //myDebug() << "hier1";
  cmbSite->clear();
  //myDebug() << "hier2";
  cmbSite->addItems(sls);
  //myDebug() << "hier3";

  cmbSite->setCurrentIndex(MyMath::max<long>(cmbSite->findText(currentSite),0));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::populateCmbOrbital()
{
  //myDebug() << "DMEdit::populateCmbOrbital";


  QString currentOrbital=cmbOrbital->currentText();

  QString el=cmbElement->currentText();
  QString si=cmbSite->currentText();
  long site=-1;
  if(si !=allfilter) site=si.toInt();

  QStringList slo;
  slo.append(allfilter);
  for(int i=0;i<dmdoc->occus().size();i++){
    const Occu &oc=dmdoc->occus().at(i);
    if(
       (el==allfilter || QString(oc.el()()) == el ) &&
       (site==-1 || oc.site() == site)
       ){
      QString s=oc.orbital()();
      if(!slo.contains(s)){
	slo.append(s);
      }
    }
  }
  
  cmbOrbital->clear();
  cmbOrbital->addItems(slo);

  cmbOrbital->setCurrentIndex(MyMath::max<long>(cmbOrbital->findText(currentOrbital),0));
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::populateLwDMat()
{
  //myDebug() << "DMEdit::populateLwDMat";
  
  QString el=cmbElement->currentText();
  QString si=cmbSite->currentText();
  long site=-1;
  if(si !=allfilter) site=si.toInt();
  QString o=cmbOrbital->currentText();

  QListWidgetItem *w=lwDMat->currentItem();
  long curdmatindex=0;
  if(w){
    curdmatindex=w->data(Qt::UserRole).toInt();
  }

  lwDMat->clear();
  
  long ind=0;
  
  
  long n=0;
  for(int i=0;i<dmdoc->occus().size();i++){
    const Occu &oc=dmdoc->occus().at(i);
    //myDebug() << oc.site << site;
    //myDebug() << el << site << o;
    if(
       (el==allfilter || QString(oc.el()()) == el ) &&
       (oc.site() == site || site ==-1)&&
       ( o ==allfilter || QString(oc.orbital()()) == o)
       ){
      
      if(i==curdmatindex){
	ind=n;
      }
      
      QListWidgetItem *w=new QListWidgetItem(oc.name()(), lwDMat);
      w->setData(Qt::UserRole,i);
      
      n++;
    }
  }
  lwDMat->setCurrentRow(ind);
  
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::populateTables()
{
  //myDebug() << "DMEdit::populateTables" ; 

  QList<QStringList> OrbitalNames;
  OrbitalNames.append(QStringList());
  OrbitalNames[0]<<"";
  OrbitalNames.append(QStringList());
  OrbitalNames[1]<<"y"<<"z"<<"x";
  OrbitalNames.append(QStringList());
  OrbitalNames[2]<<"xy"<<"yz"<<"z^2"<< "xz"<<"x2-y2";
  OrbitalNames.append(QStringList());
  OrbitalNames[3]<<"y(3x2-y2)"<<"xyz"<<"y(5z2-1)"<< "z(5z2-3)"<<"x(5z2-1)"
		 <<"(x2-y2)z"<<"x(x2-3y2)";


  if(!starttables) return;
  ShowPart rep=getShowPart();
  Occu::YlmRepresentation ylmrep=(Occu::YlmRepresentation)cmbYlm
    ->itemData(MyMath::max<long>(cmbYlm->currentIndex(),0)).toInt();
  //myDebug()  <<"ylmrep"<<ylmrep << cmbYlm->currentIndex() ;
  //myDebug()  <<"rep"<<rep << "previousRep"<<previousRep;
  
  int ioccu=getCurrentOccu();
  if(ioccu<0) return;
  
  Occu &oc=dmdoc->occus()[ioccu];


  long curax=oc.currentAxes();
  Vec3 x=oc.localaxes.at(curax).xaxis;
  Vec3 z=oc.localaxes.at(curax).zaxis;
  
  leX1->setText(Sprintf("%g",x[0])());
  leX2->setText(Sprintf("%g",x[1])());
  leX3->setText(Sprintf("%g",x[2])());

  leZ1->setText(Sprintf("%g",z[0])());
  leZ2->setText(Sprintf("%g",z[1])());
  leZ3->setText(Sprintf("%g",z[2])());


  protectTableSpinUp=true;
  protectTableSpinDown=true;
  protectTableSpinMixed=true;

  
  if(!dmdoc->isFREL()){
    lQuant->hide();
    lSpmix->hide();
    lSpmixExplain->hide();
    tbDefaultMixed->hide();
    twSpinMixed->hide();
    lSpDown->show();
    tbDefaultDown->show();
    twSpinDown->show();
  }else{
    lQuant->show();
    if(ylmrep==Occu::SphericalSpinor){
      lSpmix->hide();
      lSpmixExplain->hide();
      tbDefaultMixed->hide();
      twSpinMixed->hide();
      lSpDown->hide();
      tbDefaultDown->hide();
      twSpinDown->hide();
    }else{
      lSpmix->show();
      lSpmixExplain->show();
      tbDefaultMixed->show();
      twSpinMixed->show();
      lSpDown->show();
      tbDefaultDown->show();
      twSpinDown->show();
    }
  }

  try{
    oc.calcTrans((ylmrep==Occu::SphericalSpinor));
  }catch(Except &ex){
    QMessageBox::warning(0,"Preparing transformation"
			 ,ex.msg(),QMessageBox::Yes );
    return;
  }
  
  
  // set table
  // we rotate the internal data into the local axes

  Occu::Props pr;
  long nspin=3;

  //and then we go into complex/rel/spherspinor rep for displaying
  oc.toRepresentation(ylmrep,dmdoc->globalSpinZaxis(),pr);

  if(ylmrep==Occu::SphericalSpinor) {
    nspin=1;
    lQuant->setText("Spin z-axis: as local axis");
    lLz->setText("L(local-axes)");
    lSz->setText("S(local-axes)");
  }else{
    lQuant->setText(Sprintf("Spin z-axis (FPLO): (%s)",
			    dmdoc->globalSpinZaxis().printS("%5g",",")())());
    if(dmdoc->isFREL()){
      lLz->setText("L(local-axes)");
      lSz->setText("S(spin-axes)");
    }else{
      lLz->setText("L(local-axes)");
      lSz->setText("S(global-axes)");
    }
  }

  

  
  leN->setText(String::doubleToString("%7.3f",pr.N,true)());
  
  //leSz->setText(String::doubleToString("%7.3f",pr.Sz,true)());
  leLz->setText(("("+pr.L.printS("% 7.3f"," , ",true)+")")());
  leSz->setText(("("+pr.S.printS("% 7.3f"," , ",true)+")")());
  //leJz->setText(("("+pr.J.printS("%7.3f"," ",true)+")")());
  

  for(int ms=0;ms<nspin;ms++){
    
    QTableWidget *tw;
    long dim;
    
    dim=oc.degL();
    if(ms==0){
      tw=twSpinUp;
    }else if(ms==1){
      tw=twSpinDown;
    }else{
      tw=twSpinMixed;
    }
    
    long N=1;
    if(rep==Complex){
      N=2;
    }
    int l=oc.l();
    int degl=2*l+1;
    QStringList hl;
    if(ylmrep==Occu::SphericalSpinor){
      for(int kap=l;kap>=-l-1;kap-=degl){
	int degj=2*abs(kap);
	int j=(degj-1);
	for(int jj=0;jj<degj;jj++){
	  hl<<Sprintf("%c%1d/2%+2d/2",String::intToSpdf(l),j,2*jj-j)();
	}
      }
      dim*=2;
    }else{
      for(int jj=0;jj<dim;jj++){
	if(ylmrep==Occu::RealYlm && l<OrbitalNames.size()){
	  hl<< OrbitalNames.at(l)[jj];
	}else{
	  hl<<Sprintf("%c%+2d",String::intToSpdf(l),jj-l)();
	}
	if(rep==Complex) {hl<<"";}
      }
    }
    //myDebug()<<"dim="<<dim;


    int prepN=(previousRep==Complex?2:1);
    int previousColumn=MyMath::max<long>(tw->currentColumn(),0);
    int previousReIm=(rep==Complex?
		      (previousRep == Imag? 1: previousColumn % prepN)
		      :0);
    previousColumn=previousColumn/prepN;

    int previousRow=MyMath::max<long>(tw->currentRow(),0);
      
    
    tw->setRowCount(dim);
    tw->setColumnCount(N*dim); 
    tw->setHorizontalHeaderLabels(hl);
    tw->setVerticalHeaderLabels(hl);
   
    //myDebug() <<dim;
    for(int ii=0;ii<dim;ii++){
      for(int jj=0;jj<dim;jj++){
	Cmplx c;
	c=oc.currentRepMatElement(ms,ii,jj);
	for(int kk=0;kk<N;kk++){
	  double d=0.;
	  if((rep==Real) || ((rep==Complex) && (kk==0))){
	    d=c.real();
	  }else  if((rep==Imag) || ((rep==Complex) && (kk==1))){
	    d=c.imag();
	  }

	  if(fabs(d)<disptol)d=0.;
	  QTableWidgetItem *w=
	    new QTableWidgetItem(String::doubleToRatio(d,1,
                       MyMath::min(1.0-3,disptol))());
	  
	  if(kk==0 && rep != Imag) {
	    w->setBackground ( QBrush(QColor(255,255,255)) );
	  }else {
	    w->setBackground ( QBrush(QColor(255,160,160)) );
	  }
	  if(ii==jj && 
	     ((kk==1 && rep==Complex) || (kk==0 && rep==Imag))){
	    if(ms!=2){
	      w->setBackground ( QBrush(QColor(255,200,200)) );
	      w->setForeground ( QBrush(QColor(100,100,100)) );
	      w->setFlags(Qt::ItemIsEnabled| Qt::ItemIsSelectable);
	    }
	  }
	  tw->setItem(ii,kk+N*jj,w);
	  
	  
	}
      }
    }
    
    tw->horizontalHeader()->setResizeMode(QHeaderView::Fixed);
    tw->resizeRowsToContents();
    //tw->resizeColumnsToContents();
    //myDebug() << "previousColumn" << previousColumn << previousReIm;
    int maxCol=tw->columnCount()/N;
    previousColumn=MyMath::min<long>(previousColumn,maxCol-1)*N+previousReIm;
    previousRow=MyMath::min<long>(previousRow,tw->rowCount()-1);
    tw->setCurrentCell(previousRow,previousColumn);
    

    
  }

  previousRep=rep;

  protectTableSpinUp=false;
  protectTableSpinDown=false;
  protectTableSpinMixed=false;
}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::onCellEdit(int row,int col,QTableWidget *tw
			,bool &protectTable,int ms)
{
  if(protectTable) return;
  //myDebug() << "DMEdit::onCellEdit" << row << col ;
  //myDebug() << "protectTable" << protectTable;
  long saveProtectTable=protectTable;
  
  
  QTableWidgetItem *w=tw->item(row,col);
  if(!w) return;
  QString s=w->text();
  double d;
  try{
    //myDebug() << String(s.toAscii().constData())();
    d=String(s.toAscii().constData()).convertToFortranReal();
  }catch(Except &ex){
    return;
  }

  
  //printf("d=%30.15g\n",d);
  
  int ioccu=getCurrentOccu();
  if(ioccu<0) return;

  
  Occu &oc=dmdoc->occus()[ioccu];

  ShowPart rep=getShowPart();
  
  int i=row;
  int j=dmIndexFromColumn(col,rep);
  int reim=dmReImFromColumn(col,rep);
  
  Cmplx c;
  c=oc.currentRepMatElement(ms,i,j);

  double d1;
  if(reim){
    c=Cmplx(c.real(),d);
    d1=-c.imag();
  }else{
    c=Cmplx(d,c.imag());
    d1=c.real();
  }

  protectTable=true;


  int col1=(rep==Complex? i*2+reim : i);
  
  if(reim && i==j && ms!=2){ d1=0.; c=Cmplx(c.real(),0.);}

  if(ms<2){
    // set hermit conjg if needed
    w=tw->item(j,col1);
    if(w){
      if(fabs(d)<disptol)d1=0.;
      w->setText(String::doubleToRatio(d1,1,MyMath::min(1.0-3,disptol))());
    }
  }
  
  oc.currentRepMatSet(ms,i,j,c);


  Occu::Props pr;
  oc.fromRepresentation(dmdoc->globalSpinZaxis(),pr);
  

  leN->setText(String::doubleToString("%7.3f",pr.N,true)());
  //leSz->setText(String::doubleToString("%7.3f",pr.Sz,true)());
  leLz->setText(("("+pr.L.printS("% 7.3f"," , ",true)+")")());
  leSz->setText(("("+pr.S.printS("% 7.3f"," , ",true)+")")());
  //leJz->setText(("("+pr.J.printS("%7.3f"," ",true)+")")());

  protectTable=saveProtectTable;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void DMEdit::setTableDefault(SetDefault mode,int ms)
{
  int ioccu=getCurrentOccu();
  if(ioccu<0) return;
  Occu &oc=dmdoc->occus()[ioccu];

  if(mode==Flip){
    oc.flipSpin();
  }else if(mode==FlipLz){
    oc.flipLz();
  }else{
    Cmplx c;
    if(mode==Unity) c=1.;
    if(mode==IUnity) c=Cmplx(0.,1.);
    oc.currentRepMatSetToDiag(ms,c);
  }

  Occu::Props pr;
  oc.fromRepresentation(dmdoc->globalSpinZaxis(),pr);
  

  populateTables();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool DMEdit::dataAcceptable()
{
  //myDebug() <<"DMEdit::dataAcceptable";
  QString txt="";
  const char *updn[]={"up","down","mixed"};
  QTableWidget *tw;
  QTableWidgetItem *w;
  
  for(int ms=0;ms<3;ms++){
    tw=twSpinUp;
    if(ms==1) tw=twSpinDown;
    if(ms==2) tw=twSpinMixed;
    
    QString Vtxt="";  
    for(int row=0;row<tw->rowCount();row++){
      long err=0;
      QString m;
      
      for(int col=0;col<tw->columnCount();col++){
	//myDebug() << row<<col;
	w=tw->item(row,col);
	if(w){
	  QString s=w->text();
	  
	  err|=!xyzvalid->check(m,s);
	  //myDebug() << row<<col<<s<<err;
	  IntValidator::stripAllWhiteSpace(s);
	}
      }
      if(err)
	Vtxt+=QString("row %2: ").arg(row+1)+m;
      
    }
    if(Vtxt.length()>0)
      txt+=QString("Spin ")+updn[ms]+" occus\n"+Vtxt;
  }
  
  if(txt.length()){
    QMessageBox::warning ( this,"Save & Quit ... ", txt, 1, 0,0 ) ;
    return false;
  }
  return true;
 
}
