#ifndef DMEDIT_H
#define DMEDIT_H

#include <QDialog>
#include "ui_DMEdit.h"

class QAction;
class QSignalMapper;
class RealValidator;
class DMDoc;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class DMEdit : public QDialog, private Ui::DMEdit
{
  Q_OBJECT
public:
  DMEdit(QWidget *parent = 0);
  ~DMEdit();
  void setData(DMDoc *dmdoc);
			    
public slots:
  virtual void reject();
  
  
private:
  enum ShowPart{Real=1,Imag=2,Complex=3};
  
  enum SetDefault{Zero=0,Unity=1,Down=10,Mixed=20,Flip=2,FlipLz=3,IUnity=4};
							 
private slots:
  void on_pbQuit_clicked();
  void on_pbNext_clicked();
  void on_pbPrev_clicked();
  void on_pbSetAxis_clicked();
  
  void on_cmbElement_currentIndexChanged(int index);
  void on_cmbSite_currentIndexChanged(int index);
  void on_cmbOrbital_currentIndexChanged(int index);
  void on_lwDMat_currentItemChanged(QListWidgetItem *currentitem
				    ,QListWidgetItem *previousitem);
  
  void on_cmbRep_currentIndexChanged(int index);
  void on_cmbYlm_currentIndexChanged(int index);
  
  void on_spbxColSize_valueChanged(int );
  void on_spbx0Tol_valueChanged(int v);
  
  void on_twSpinUp_cellChanged(int row,int col) {
    onCellEdit(row,col,twSpinUp,protectTableSpinUp,0); };
  void on_twSpinDown_cellChanged(int row,int col ){
    onCellEdit(row,col,twSpinDown,protectTableSpinDown,1); };
  void on_twSpinMixed_cellChanged(int row,int col ){
    onCellEdit(row,col,twSpinMixed,protectTableSpinMixed,2); };
  
  void onTbDefault(int);
  
private:
  void populateTables();
  void populateCmbElement();
  void populateCmbSite();
  void populateCmbOrbital();
  void populateLwDMat();
  void saveMainSettings();
  
  bool dataAcceptable();
  
  void onCellEdit(int row,int col,QTableWidget *tw,bool &protectTable,int ms);
  
  void setTableDefault(SetDefault mode,int ms);
  
  int getCurrentOccu();
  
  QWidget *saveTableFocus(QWidget *me);
  
  inline ShowPart getShowPart()const{
    return (ShowPart)cmbRep->itemData(cmbRep->currentIndex()).toInt();
  }
  
  inline int dmIndexFromColumn(long col,ShowPart rep){
    return (rep==Complex?col/2:col); };
  
  inline int dmReImFromColumn(long col,ShowPart rep){
    return (rep==Complex?col%2:(rep==Imag?1:0)); };
  
  
protected:
private:
  DMDoc *dmdoc;
  int previousRep;
  bool protectTableSpinUp;
  bool protectTableSpinDown;
  bool protectTableSpinMixed;
  bool starttables;
  
  QString allfilter;
  
  double disptol;
  QAction *setnullactionUp;
  QAction *setnullactionDown;
  QAction *setnullactionMixed;
  QAction *setunitactionUp;
  QAction *setunitactionDown;
  QAction *setunitactionMixed;
  QAction *setflipactionUp;
  QAction *setflipactionDown;
  QAction *setfliplzactionUp;
  QAction *setfliplzactionDown;
  QAction *setiactionMixed;
  QSignalMapper *signalMapper;
  
  RealValidator *xyzvalid;
  QAction *quitAction;
};

#endif // DMEDIT_H
