// @(#)FSW.cpp
//author: Klaus.Koepernik@ifw-dresden.de
//


#include <QMenu>
#include <QMessageBox>
#include <QToolBar>
#include <QThread>
#include "Worker.hh"
#include <QFileDialog>
#include <QProcess>
#include "Except.hh"
#include "Assorted.hh"
#include "constants.h" 
#include "Xf.hh"
#include "Plane.hh"
#include "MWBase.hh"
#include "FSW.hh"
#include "Status.hh"
#include "Exception.hh"
#include "Light.hh"
#include "LightDlg.hh"
#include "Fog.hh"
#include "Polyhedron.hh"
#include "Spsp.hh"
#include "FogDlg.hh"
#include "FSToolBox.hh"
#include "DisplayCells.hh"
#include "MeshSymDlg.hh"
#include "FSDoc.hh"
#include "OutWin.hh"
#include "AnnotationDlg.hh"
#include "RegularMesh.hh"
#include "FSSweepDialog.hh"
#include "TriangleMesh.hh"
#include "Settings.hh"
#include "Config.hh"
#include "Axes.hh"
#include "PlotDlg.hh"
#include "Fplo.hh"
#include "FploDlg.hh"
#include "Symmetry.hh"
#include "HelpDlg.hh"
#include "Boundary.hh"
#include "SurColDlg.hh"
#include "Interrupt.hh"
#include "SpecialPointDlg.hh"
#include "CutDlg.hh"
#include "SymDlg.hh"
#include "MeshDlg.hh"
#include "BoundDlg.hh"
#include "FermiDlg.hh"
#include "FilesDlg.hh"
#include "InpConDlg.hh"
#include "KexpDlg.hh"
#include "OptDlg.hh"
#include "PrntDlg.hh"
#include "MyDebug.hh"

#define workdir (doc->currentDir() +'/')

#define fstoolbox ((FSToolBox*)base()->toolBox())

/* ***********************************************************************
 *
 * ***********************************************************************
 */

FSW::FSW(MWBase *parent)
  : ControlBase(parent),doc(0),
    plotdlg(0),surcoldlg(0),
    cutdlg(0),fogdlg(0),bounddlg(0),lightdlg(0),symdlg(0),
    fssweepdlg(0),specialpointdlg(0)
{

  setupUi(this);

  

  doc=new FSDoc(oglview);
  
  
  base()->setToolBox(new FSToolBox(this,doc));
  fstoolbox->setObjectName("fstoolbox");
  fstoolbox->setFloating(false);
  fstoolbox->setAllowedAreas(Qt::RightDockWidgetArea);
  base()->addDockWidget(static_cast<Qt::DockWidgetArea>(2), fstoolbox);
  

  setupMenus();
  

  

  //  base()->setWindowTitle(IDS_APP_NAME + " : " + doc->fName());
  

  setupDocAndView(oglview,doc);
  oglview->setFog(doc->fog());
  

  {
    Settings set("application");
    oglview->
      setBadTransparency(
			 set.getBool("graphics/badTransparencyOnMove",true));
    
  }

  MASSERT("",qlviewwidget);


  sizepolicy=QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  sizepolicy.setHorizontalStretch(1);
  sizepolicy.setVerticalStretch(1);
  sizepolicy.setHeightForWidth(oglview->sizePolicy().hasHeightForWidth());



  Settings set("fermisurface");
  fstoolbox->setVisible(set.getBool("maintoolbox/visible",true));

  fstoolbox->updateToolBox();  
  //  if(!doc->emptyl)
  //      fstoolbox->setMuted(true);

  //  updateGeometry();

  inputMenu_aboutToShow();

  MASSERT("",base()->outwin);
  // base()->outwin->show();



  base()->setWindowTitle(IDS_APP_NAME + " : " + doc->fileName());


  connect(statusbar,SIGNAL(canvasLockClicked(bool))
	  ,this,SLOT(slotCanvasLockClicked(bool)));

  statusbar->setCanvasControlEnabled(true);

  statusbar->setCanvasControl(doc->canvas());
  slotCanvasLockClicked(doc->canvas().locked());
  


  {
    Settings set("fermisurface");
    doc->surffname=set.getString("printing/filename",doc->surffname);
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void FSW::setupMenus()
{
  QToolBar *mainToolBar = new QToolBar(base());
  mainToolBar->setObjectName("mainToolBar");
  mainToolBar->setContextMenuPolicy(Qt::NoContextMenu);
  mainToolBar->setWindowTitle("xfsf tool bar");
  base()->addToolBar(Qt::TopToolBarArea, mainToolBar);


  QAction *fileAction=base()->getNewMenuBarItem(Actions::File);
  fileAction->setText("&File");
  
  QAction *a=MWBase::addMenu(fileAction,"&Open","Open fermi surface settings"
			     ,Actions::File_Open
			     ,QIcon(":/Icons/fileopen")
			     ,QKeySequence(Qt::CTRL+Qt::Key_O));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileOpen()));
  mainToolBar->addAction(a); 

  a=MWBase::addMenu(fileAction,"&Save","Save Settings File",Actions::File_Save
			     ,QIcon(":/Icons/filesave")
			     ,QKeySequence(Qt::CTRL+Qt::Key_S));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSave()));
  mainToolBar->addAction(a); 


  a=MWBase::addMenu(fileAction,"Save &As","Save as",Actions::File_SaveAs);
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSaveAs()));
 
  fileAction->menu()->addSeparator();

  
  QMenu *m;
  m = new QMenu(fileAction->menu());
  m->setTitle("&Export");
  a=MWBase::addMenu(m->menuAction(),"&k-points"
		    ,"export k-points to file '=.kp'"
		    ,Actions::File_Export_kPoints);
  connect(a, SIGNAL(triggered()), this, SLOT(slotMeshExport()));
  m->addAction(a);

  fileAction->menu()->addAction(m->menuAction());
  fileAction->menu()->addSeparator();


  m = new QMenu(fileAction->menu());
  m->setTitle("&Print");
  a=MWBase::addMenu(m->menuAction(),"&Surface"
		    ,"Print Surface" ,Actions::File_Print_Surface
		    ,QIcon(":/Icons/print") 
		    ,QKeySequence(Qt::CTRL+Qt::Key_P));
  connect(a, SIGNAL(triggered()), this, SLOT(slotFilePrint()));
  m->addAction(a);
  mainToolBar->addAction(a);
  fileAction->menu()->addAction(m->menuAction());


  fileAction->menu()->addSeparator();

  QAction *closeAction=
  a=MWBase::addMenu(fileAction,"&Close","Close this activity"
		    ,Actions::File_Close
                    ,QIcon(":/Icons/exit")
		    ,QKeySequence(Qt::CTRL+Qt::Key_W));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileClose()));

   

  mainToolBar->addSeparator();

  //////////////////////////
  QAction *inputAction=base()->getNewMenuBarItem(Actions::Input);
  inputAction->setText("&Input");
  
  a=MWBase::addMenu(inputAction,"F&iles"
		    ,"Set files",Actions::Input_FermiDlg);
  connect(a,SIGNAL(triggered()),this,SLOT(slotFilesDlg()));

  a=MWBase::addMenu(inputAction,"&Mesh","Define k-mesh"
		    ,Actions::Input_DefineMesh
                    ,QIcon(":/Icons/mesh")
		    ,QKeySequence(Qt::CTRL+Qt::Key_M));
  mainToolBar->addAction(a);
  connect(a,SIGNAL(triggered()),this,SLOT(slotMeshDefine()));

  a=MWBase::addMenu(inputAction,"&Fermi energy/ bands"
		    ,"Define energy settings",Actions::Input_FermiDlg
                    ,QIcon(":/Icons/erg")
		    ,QKeySequence(Qt::CTRL+Qt::Key_E));
  connect(a,SIGNAL(triggered()),this,SLOT(slotErgDefine()));
  mainToolBar->addAction(a);

  a=MWBase::addMenu(inputAction,"&Hand made symmetry"
		    ,"Define hand made symmetry settings"
		    ,Actions::Input_HandMadeSymmetry
		    );
  connect(a,SIGNAL(triggered()),this,SLOT(slotMeshSymmetryDefine()));
  //mainToolBar->addAction(a);

  a=MWBase::addMenu(inputAction,"&Boundary","Define boundary"
		    ,Actions::Input_BoundaryDlg
                    ,QIcon(":/Icons/bound")
		    ,QKeySequence(Qt::CTRL+Qt::Key_B));
  connect(a,SIGNAL(triggered()),this,SLOT(onBoundDlgTalkBack()));
  mainToolBar->addAction(a);

  symAction=a=MWBase::addMenu(inputAction,"&Symmetry"
		    ,"Symmetry settings for empty lattice test"
			      ,Actions::Input_SymmetryDlg
                    ,QIcon(":/Icons/sym")
		    ,QKeySequence(Qt::CTRL+Qt::Key_Y));
  connect(a,SIGNAL(triggered()),this,SLOT(onSymDlgTalkBack()));
  mainToolBar->addAction(a);
  mainToolBar->addSeparator();


  connect(inputAction->menu()
	  ,SIGNAL(aboutToShow()),this,SLOT(inputMenu_aboutToShow()));



  QAction *plotAction=base()->getNewMenuBarItem(Actions::Plot);
  plotAction->setText("&Plot");
  
  a=MWBase::addMenu(plotAction,"&Plot","Plot settings"
		    ,Actions::Plot_PlotSettings);
  connect(a,SIGNAL(triggered()),this,SLOT(onPlotDlgTalkBack()));

  a=MWBase::addMenu(plotAction,"Co&lors","Color settings"
		    ,Actions::Plot_ColorSettings);
  connect(a,SIGNAL(triggered()),this,SLOT(onColorDlgTalkBack()));

//   a=MWBase::addMenu(plotAction,"Cu&ts","Cut settings");
//   connect(a,SIGNAL(triggered()),this,SLOT(slotCutDlg()));

  a=MWBase::addMenu(plotAction,"&Annotations" ,"Annotation Settings" 
		    ,Actions::Plot_Annotations
		    ,QKeySequence(Qt::CTRL+Qt::Key_A));
  a->setData(AnnotationDlg::AMAxes);
  connect(a,SIGNAL(triggered()),this,SLOT(onAnnotationDlgTalkBack()));

  a=MWBase::addMenu(plotAction,"&Display cells" ,"Display cell settings" 
		    ,Actions::Plot_DisplayCells);
  a->setData(AnnotationDlg::AMDisplayCells);
  connect(a,SIGNAL(triggered()),this,SLOT(onAnnotationDlgTalkBack()));


  a=MWBase::addMenu(plotAction,"&High symmetry points" ,"High symemtry points" 
		    ,Actions::Plot_SpecialPoints
		    ,QKeySequence(Qt::CTRL+Qt::Key_H));
  connect(a,SIGNAL(triggered()),this,SLOT(onSpecialPointDlgTalkBack()));


  a=MWBase::addMenu(plotAction,"Li&ghts","Light settings"
		    ,Actions::Plot_Lighting);
  connect(a,SIGNAL(triggered()),this,SLOT(onLightDlgTalkBack()));

  a=MWBase::addMenu(plotAction,"&Fog","Fog  settings"
		    ,Actions::Plot_Fog);
  connect(a,SIGNAL(triggered()),this,SLOT(onFogDlgTalkBack()));

  calcAction
    =a=MWBase::addMenu(plotAction,"&Calculate","Calculate scene"
		       ,Actions::Plot_Calculate
		     ,QIcon(":/Icons/xfsf")
		     ,QKeySequence(Qt::CTRL+Qt::Key_C));
  connect(a,SIGNAL(triggered()),this,SLOT(slotPlotCalc()));
  fstoolbox->tbCalc->setDefaultAction(a);
  mainToolBar->addAction(a);


  a=MWBase::addMenu(plotAction,"&Recalculate","Force re-calculation"
		    ,Actions::Plot_Recalculate
		     ,QIcon(":/Icons/recalc"));
  connect(a,SIGNAL(triggered()),this,SLOT(slotPlotReCalc()));
  fstoolbox->tbReCalc->setDefaultAction(a);
  mainToolBar->addAction(a);

  
  canvasAction
    =a=MWBase::addMenu(plotAction,"Can&vas","Canvas Settings"
		       ,Actions::Plot_Canvas
		       ,QKeySequence(Qt::CTRL+Qt::Key_V));
  connect(a,SIGNAL(triggered()),this,SLOT(slotCanvasDefine()));


  // QAction *toolAction=base()->getNewMenuBarItem(Actions::Tools);
  // toolAction->setText("&Tools");
  // a=MWBase::addMenu(toolAction,"&Sweep FS","Sweep Fermi surface energy"
  //  		    ,Actions::Tools_FSSweep);
  // connect(a,SIGNAL(triggered()),this,SLOT(onSweepDlgTalkBack()));


  
  QAction *runAction=base()->getNewMenuBarItem(Actions::Run);
  runAction->setText("&Run");
  a=MWBase::addMenu(runAction,"Run &Fplo","Run interactive FPLO session"
		    ,Actions::Run_RunFPLO);
  connect(a,SIGNAL(triggered()),this,SLOT(slotRunFPLO()));

  a=MWBase::addMenu(runAction,"&Terminal","Run an xterm in the current directory(On MAC xfplo must be started in X11/Terminal for this to work)"
		    ,Actions::Run_Terminal);
  connect(a,SIGNAL(triggered()),this,SLOT(slotRunTerminal()));


  QAction *optionsAction=base()->getNewMenuBarItem(Actions::Options);
  optionsAction->setText("&Options");

  a=MWBase::addMenu(optionsAction,"&K-export options"
		    ,"Set options for export of k-mesh."
		    ,Actions::Options_kExport);
  connect(a,SIGNAL(triggered()),this,SLOT(slotKexpDlg()));

  a=MWBase::addMenu(optionsAction,"&Input control","Input control"
		    ,Actions::Options_InputControl
		    ,QKeySequence(Qt::CTRL+Qt::Key_I));
  connect(a,SIGNAL(triggered()),this,SLOT(slotInputControl()));

  a=MWBase::addMenu(optionsAction,"&Miscelaneous","Set misc options"
		    ,Actions::Options_Misc);
  connect(a,SIGNAL(triggered()),this,SLOT(slotOptDlg()));

  optionsAction->menu()->addSeparator();

  emptlAction
    =a=MWBase::addMenu(optionsAction,"&Empty lattice","Empty lattice"
		       ,Actions::Options_EmptyLattice);
  a->setCheckable(true);
  connect(a,SIGNAL(triggered(bool)),this,SLOT(emptlAction_triggered(bool)));

  clipAction
    =a=MWBase::addMenu(optionsAction,"&Clip","Clipping on/off"
		       ,Actions::Options_Clip);
  a->setCheckable(true);
  connect(a,SIGNAL(triggered(bool)),this,SLOT(clipAction_triggered(bool)));


  connect(optionsAction->menu(),SIGNAL(aboutToShow())
	  ,this,SLOT(optionsMenu_aboutToShow()));


  QAction *winAction=base()->getWindowMenuAction();

  showOutAction
    =a=MWBase::addMenu(winAction,"&Output","Show output window"
		       ,Actions::Window_Output);
  a->setCheckable(true);
  connect(a,SIGNAL(triggered(bool)),this,SLOT(slotShowOutwin(bool)));

  connect(winAction->menu(),SIGNAL(aboutToShow())
	  ,this,SLOT(winMenu_aboutToShow()));


  m = new QMenu(winAction->menu());
  m->setTitle("&Toolbars");
  winAction->menu()->addAction(m->menuAction());

  mainToolBar->addSeparator();
  
  mainToolBar->addAction(closeAction);

  a=mainToolBar->toggleViewAction();
  a->setText("&Tool bar");
  m->addAction(a);
  a=fstoolbox->toggleViewAction();
  a->setText("Main &tool box");
  m->addAction(a);



}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::afterShow(const QString &loadp)
{
  //  myDebug() << "FSW::afterShow";
  


  
  if(!loadp.isEmpty()){
    QFileInfo fi(loadp);
    if(fi.isFile()){
      
      doc->setCurrentDir(fi.absolutePath());
      
      fileLoad(workdir+fi.fileName());
	
    }else{
      if(fi.exists()){
	doc->setCurrentDir(fi.absoluteFilePath());
      }else{
	doc->setCurrentDir(fi.absolutePath());
      }

    }
  }
  updateSizes(true);

  //  slotMeshDefine();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::updateSizes(bool fresh)
{
  int w,h;
  bool canvaslocked=doc->canvas().locked();
  if(fresh){
    Settings set("fermisurface");
    w=set.getInt("canvas/width",400);
    h=set.getInt("canvas/height",400);
    if(w<10)w=400;
    if(h<10)h=400;
    doc->canvas().setWidth(w);
    doc->canvas().setHeight(h);
  }
  slotCanvasLockClicked(true);
  oglSizeChanged(oglview);
  statusbar->setCanvasControl(doc->canvas()); 
  if(fresh)
    oglview->resize(w,h); 
  updateGeometry();
  if(fresh){
    qApp->processEvents();
  }
  slotCanvasLockClicked(canvaslocked);
  statusbar->setCanvasControl(doc->canvas());  
  oglSizeChanged(oglview);

}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
FSW::~FSW()
{
  //myDebug() << "FSW:: here I die!\n";
  oglview->mute(true);
  if(doc) delete doc;

  if(!plotdlg.isNull()) delete plotdlg; plotdlg=0;
  if(!surcoldlg.isNull()) delete surcoldlg; surcoldlg=0;
  if(!cutdlg.isNull()) delete cutdlg; cutdlg=0;
  if(!fogdlg.isNull()) delete fogdlg;fogdlg=0;
  if(!bounddlg.isNull()) delete bounddlg;bounddlg=0;
  if(!lightdlg.isNull()) delete lightdlg;lightdlg=0;
  if(!specialpointdlg.isNull()) delete specialpointdlg;specialpointdlg=0;
  if(!symdlg.isNull()) delete symdlg;symdlg=0;
  if(!fssweepdlg.isNull()) delete fssweepdlg; fssweepdlg=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotRunFPLO()
{

  
  QString fn=workdir+"/=.in";

  Fplo *fplo=0;
  try{
    fplo=new Fplo(fn,"out_kp");
  }catch(Except &ex){
  
    QMessageBox::warning(this,"Running fplo"
			 ,QString("Nothing to do here ... %1\n")
			 .arg(ex.msg())
			 ,QMessageBox::Ok | QMessageBox::Default
			 );
    return;
  }  

  fplo->setCheckBandPlot(true);
  fplo->setCheckInitialSSplit(true);

  FploDlg *dlg=new FploDlg(this,fplo);
  dlg->exec();
  delete dlg;
  if(fplo) delete fplo;


  doc->erg.setEfChanged(true);
  doc->allcalced=false;
      

  if(!doc->erg.emptyLattice()){
    int ret=QMessageBox::warning(this,"+band calculated"
				 ,"Do you want to recalculate the"
				 " sceene now?"
				 ,QMessageBox::Yes | QMessageBox::Default
				 ,QMessageBox::No  
				 );
    if(ret==QMessageBox::Yes){
      slotPlotReCalc();
    }
  }  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onFSSweepDlgCurrentValueChanged(double value)
{
  if(!fssweepdlg) return;

  //  myDebug() << "onFSSweepDlgCurrentValueChanged" << value;

  //todo: save original settings and restore on dialog close
  
  doc->erg.setFermiLevel(value,false);
  doc->erg.calcRange(doc->vlevel,true);


  doc->allcalced=false;
  doc->triacalced=false;
  doc->setModified();



  updateData();
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::help()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","Index");
  h->show();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::fileLoad(const QString &fileName)
{

  QString ft=Master::fileType(fileName,false);
  if(ft!="xef"){
    QMessageBox::warning(this,"Open","File type is no "
			 "fermi-surface file",QMessageBox::Yes);
    return ;
  }

  //  myDebug() << fileName ;
  doc->saveCurrentDir();
  QFileInfo fi(fileName);
  bool isfile=fi.isFile();
  doc->restoreCurrentDir();
  if(!isfile){
    myDebug() << "file " << fileName << "is not a file";
    return ;
  }




  if (!fileName.isEmpty()) {
    
    try{
      doc->load(fileName);
    }catch(QString &msg){
      QMessageBox::warning(this,"Open",msg,QMessageBox::Yes);
      inputMenu_aboutToShow();
      statusbar->setDocument(workdir);
      return;
    }

    doc->setFileName(fileName);
    doc->setCurrentDir(doc->absolutePath());

    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());

    QString message="Loaded document: "+fileName;
    
    statusbar->setDocument(doc->absoluteFilePath());

    onFogDlgTalkBack(ProtoDlg::Refresh);
    onBoundDlgTalkBack(ProtoDlg::Refresh);
    onSpecialPointDlgTalkBack(ProtoDlg::Refresh);
    onSymDlgTalkBack(ProtoDlg::Refresh);
    onSweepDlgTalkBack(ProtoDlg::Refresh);
    onPlotDlgTalkBack(ProtoDlg::Refresh);
    onColorDlgTalkBack(ProtoDlg::Refresh);
    onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    onLightDlgTalkBack(ProtoDlg::Refresh);
    onCutDlgTalkBack(ProtoDlg::Refresh);

    //fstoolbox->updateAxesButtons();
    //fstoolbox->updateCellButtons();

    updateSizes(false);
    fstoolbox->updateToolBox();  
    base()->outwin->clear();  

    updateData();

  }
  inputMenu_aboutToShow();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFileOpen()
{
  //x  statusbar->setMessage("Opening file...");
  QString f0=doc->absoluteFilePath();
  if(doc->isNew()){
    f0=workdir+"=.xef";
  }

  QString fileName = QFileDialog::getOpenFileName(this,"Open File",f0
						  ,"FPLO input (=.*);;All files (*)");

  if(!fileName.isEmpty()){
    fileLoad(fileName);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFileSave() 
{ 
  saveit(FILESAVE); 
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFileSaveAs()
{
  saveit(FILESAVEAS); 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFileClose()
{
  //myDebug() <<"FSW::slotFileClose";
  if(queryExit()){
    base()->finish();
    //myDebug() <<" FSW::slotFileClose after base()->finish";
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FSW::queryExit()
{
  {
    Settings set("fermisurface");
    set.set("maintoolbox/visible",fstoolbox->isVisible());
    set.set("canvas/width",doc->canvas().width());
    set.set("canvas/height",doc->canvas().height());
  }
  return ControlBase::queryExit();

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::saveit( SaveMode saveas )
{
  int doit;
  QString fn;

  if(saveas != FILESAVE && saveas != FILESAVEAS  )return;
  if(saveas == FILESAVEAS || doc->isNew() ){
    QString f0=doc->absoluteFilePath();
    if(doc->isNew()){
      f0=workdir+"=.xef";
    }

#ifdef Q_OS_MAC
    fn = QFileDialog::getSaveFileName(this,"Save File",f0);
#else
    fn = QFileDialog::getSaveFileName(this,"Save File",f0
 				      , " FPLO files(=.*) ;; All files(*)"
 				      ,0,0);
#endif
    
    
    
    if ( (doit=!fn.isEmpty()) ) {
      //todo   doit=overWriteFile(fn);
    }
  }else{
    fn=doc->absoluteFilePath();
    doit=1;
  }
  if ( doit ) {
    try{
      doc->save(fn);
    }catch(QString &msg){
      QMessageBox::warning(this,"Save",msg,QMessageBox::Yes);
      return;
    }
    
    doc->setFileName(fn);
    doc->setCurrentDir(doc->absolutePath());
    doc->setOld();
    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());
    statusbar->setDocument(doc->absoluteFilePath());
    statusbar->setMessage(QString("Saved %1")
			  .arg(doc->absoluteFilePath()),1000);
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotMeshExport()
{



  if(!calcKPoints())return ;
  

  
  
  long  doit=1;
  if(doc->erg.emptyLattice()){
    doit=(QMessageBox::Yes==
	  QMessageBox::warning(this,"Export mesh", 
				"You are in emptylattice mode\n"
				"Symmetry may be different from\n"
				"this one in the fplo files !\n"
				"Want to export anyway??",
				QMessageBox::Yes,QMessageBox::No));
    
  }
  QString file=workdir + "=.kp";
  if(doit && overWriteFile(file)){
  
    
    try{
      doc->saveKPoints(workdir);
    }catch(Except &e){
      QMessageBox::warning(this,"Export mesh",e.msg()
			   ,QMessageBox::Yes,QMessageBox::No);
      
    return ;
    }

    if(!doc->erg.emptyLattice()){
      int ret=QMessageBox::warning(this,"Mesh exported"
				   ,"Do you want to save the settings now?"
				   ,QMessageBox::Yes | QMessageBox::Default
				   ,QMessageBox::No  
				   );
      if(ret==QMessageBox::Yes){
	slotFileSave();
      }
      ret=QMessageBox::warning(this,"Mesh exported"
			       ,QString("Do you want to run fplo interactively"
					" now to calculate a new %1?")
			       .arg(doc->erg.bandFile())
				   ,QMessageBox::Yes | QMessageBox::Default
				   ,QMessageBox::No  
				   );
      if(ret==QMessageBox::Yes){
	slotRunFPLO();
      }
    }
  }
  
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFilePrint()
{

  
  QString filename=doc->surffname;
  
  PrntDlg *pdlg=new PrntDlg(this,"PrtDlg");
  pdlg->setData(doc,filename);
  pdlg->setWindowTitle(IDS_APP_NAME + " : Print Fermi Surface" );
  long rett=pdlg->exec();
  if(rett){  	
    pdlg->getData(doc,filename);
  }
  delete (pdlg);
  if(!rett) return;



  long doit;
  if ( (doit=!filename.isEmpty()) ) {
    doit=overWriteFile(filename); 
  }
  if(!doit) return;


  if(!QFileInfo(filename).dir().exists()){
    QMessageBox::warning(this,"Save image", 
			 "The file "+doc->surffname+
			 " cannot be saved (directory invalid)??",
			 QMessageBox::Yes);
      return;
  }
  
  doc->surffname=filename;



  qApp->processEvents();
  qApp->flush();
  

  
  //MBox *mbox=new MBox(this);

  //mbox->show();
  //qApp->processEvents();
  //qApp->flush();
  statusbar->setMessage(QString("Printing to \'")+doc->surffname+"\' ...");

  doc->saveCurrentDir();
  QFileInfo fi(doc->surffname);
  QString ap=fi.absoluteFilePath();
  doc->restoreCurrentDir();

//   QMessageBox::information(this,"Check", doc->surffname+"\n"+ap
// 			   ,QMessageBox::Ok);

  Settings set("fermisurface");
  set.set("printing/filename",fi.fileName());

  oglview->writePNG(ap,doc->magfac,doc->picquality,statusbar);

  //delete mbox;
  statusbar->deleteMessage();

  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool FSW::isModified()
{
  //myDebug() <<"FSW::isModified " << doc->isModified();
  return doc->isModified();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::inputMenu_aboutToShow()
{
  if(!doc) return;
  //  symAction->setEnabled(doc->emptyl);
}

/* ***********************************************************************
 * exits the Application
 * ***********************************************************************
 */
void FSW::emptlAction_triggered(bool onoff)
{

  if(!doc)return;
  doc->erg.setEmptyLattice(onoff);
  doc->resetflags();
  doc->setModified();
  
  onSymDlgTalkBack(ProtoDlg::Refresh);

  inputMenu_aboutToShow();

  //  fstoolbox->setMuted(false);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::optionsMenu_aboutToShow()
{
  emptlAction->setChecked(doc->erg.emptyLattice());
  clipAction->setChecked(doc->bclip1);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::clipAction_triggered( bool onoff )
{
  if(!doc)return;

  doc->bclip1=onoff;
  doc->triacalced=false;
  doc->allcalced=false;
  
  updateData();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::winMenu_aboutToShow()
{
  //  showCutAction->setChecked(cutview->isVisible());
  showOutAction->setChecked(base()->outwin->isVisible());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotShowOutwin(bool onoff)
{
  if( onoff ){
    base()->outwin->show();
    base()->outwin->raise();
  }else{
    base()->outwin->hide();
  }
}














// /* ***********************************************************************
//  *
//  * ***********************************************************************
//  */
// void FSW::refreshSpecialPointDlg()
// {
//   if(!specialpointdlg) return;
//   specialpointdlg->refreshActive(doc->specpoi);
//   specialpointdlg->update();
// }



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onFogDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!fogdlg.isNull()) {
      fogdlg->activateWindow();
      fogdlg->raise();
      return;
    }
  
    fogdlg=new FogDlg(0,base(),"FogDlg");
    connect(fogdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onFogDlgTalkBack(int)));
    

    fogdlg->setData(doc->fog());
    fogdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(fogdlg.isNull()) return;
    fogdlg->setData(doc->fog());
    fogdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(fogdlg.isNull()) return;

    fogdlg->getData(doc->fog());
    if(doc->fog()->changed()) {
      doc->setModified();
      doc->allcalced=false;
    }
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onBoundDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){

    if(!calcSymmetry()) return;

    if(!bounddlg.isNull()) {
      bounddlg->activateWindow();
      bounddlg->raise();
      return;
    }
  
    bounddlg=new BoundDlg(this,base(),"BoundDlg");
    connect(bounddlg,SIGNAL(talkBack(int)),
	    this,SLOT(onBoundDlgTalkBack(int)));
    
    bounddlg->setData(doc->bound(),doc->currentsym1);
    bounddlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(bounddlg.isNull()) return;
    if(!calcSymmetry()) return;

    bounddlg->setData(doc->bound(),doc->currentsym1);
    bounddlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(bounddlg.isNull()) return;

    bool ch=false;
    bounddlg->getData(doc->bound(),ch);
    if(ch) {
      doc->setModified();
      doc->allcalced=false;
    }

    if(doc->erg.useBoundaryCell())
      doc->erg.setMeshCalculated(false);

    if(!doc->allcalced){
      updateData();
    }

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onSymDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!symdlg.isNull()) {
      symdlg->activateWindow();
      symdlg->raise();
      return;
    }
  
    symdlg=new SymDlg(0,base(),"FSWSymDlg");
    connect(symdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onSymDlgTalkBack(int)));
    connect(symdlg,SIGNAL(loadFPLOInFile(SymDlg *))
	    ,this,SLOT(onLoadFPLOInFile(SymDlg *)));
    
    
    if(doc->erg.emptyLattice()){
      symdlg->setData(doc->xefsym,SymDlg::FermiSurfaceEditMode);
    }else{
      try{
	QString file=workdir + "=.in";
	doc->fploinsym->readSymInfo(file);
      }catch(Except &ex){
	QMessageBox::warning(this,"Loading symmetry", ex.msg()
			     ,QMessageBox::Ok | QMessageBox::Default);
	return;
	
      }
      
      symdlg->setData(doc->fploinsym,SymDlg::FermiSurfaceViewMode);
    }
    
    symdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(symdlg.isNull()) return;
    
    if(doc->erg.emptyLattice()){
      symdlg->setData(doc->xefsym,SymDlg::FermiSurfaceEditMode);
    }else{
      try{
	QString file=workdir + "=.in";
	doc->fploinsym->readSymInfo(file);
      }catch(Except &ex){
	QMessageBox::warning(this,"Loading symmetry", ex.msg()
			     ,QMessageBox::Ok | QMessageBox::Default);
	return;
	
      }
      
      symdlg->setData(doc->fploinsym,SymDlg::FermiSurfaceViewMode);
    }
    symdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(!doc->erg.emptyLattice()) return;
    
    if(symdlg.isNull()) return;

    bool ch=false;
    symdlg->getData(doc->xefsym,ch);

    if(ch || !doc->xefsym->calculated()) {
      doc->setModified();
      doc->allcalced=false;
      //refreshBoundDlg();
      if(!doc->allcalced){
	//updateData(true);
	updateData();
      }
      onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    }

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onColorDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!surcoldlg.isNull()) {
      surcoldlg->activateWindow();
      surcoldlg->raise();
      return;
    }
    
    surcoldlg=new SurColDlg(0,base(),"SurColDlg"); //todo: this or 0?
    connect(surcoldlg,SIGNAL(talkBack(int)),
	    this,SLOT(onColorDlgTalkBack(int)));
    
    
    surcoldlg->setData(doc->isocolor,doc->excol
		       ,doc->vmin,doc->vmax
		       ,doc->vmine,doc->vmaxe
		       ,doc->vdef,doc->vdefe
		       ,&doc->vmindef,&doc->vmaxdef
		       ,&doc->vminedef,&doc->vmaxedef
		       ,doc->vunit,doc->ufactor,doc->ufactore
		       ,doc->iweight,doc->legendcontrol);
    surcoldlg->show();
    
    return ;
  }else if(mode==ProtoDlg::Refresh){
    if(surcoldlg.isNull()) return;
    surcoldlg->setData(doc->isocolor,doc->excol
		       ,doc->vmin,doc->vmax
		       ,doc->vmine,doc->vmaxe
		       ,doc->vdef,doc->vdefe
		       ,&doc->vmindef,&doc->vmaxdef
		       ,&doc->vminedef,&doc->vmaxedef
		       ,doc->vunit,doc->ufactor,doc->ufactore
		       ,doc->iweight,doc->legendcontrol);
    
    
    surcoldlg->update();
  }else if(mode==ProtoDlg::Accept){
    
    if(surcoldlg.isNull()) return;
    
    long chn,excolchn;
    surcoldlg->getData(doc->isocolor
		       ,(doc->excol)
		       ,(doc->vmin),(doc->vmax),(doc->vmine),(doc->vmaxe)
		       ,(doc->vdef),(doc->vdefe)
		       ,(doc->vunit),(doc->ufactor),(doc->ufactore)
		       ,doc->iweight,doc->legendcontrol
		       ,chn,excolchn);
    /* document is changed tell it*/

    if(chn) { 
      doc->setModified(); 
      doc->allcalced=false;
    }
    //   if(excolchn) { doc->setModified(); doc->allcalced=false;
    //     doc->erg.setEnergiesLoaded(false);}

    if(!doc->allcalced){
      updateData();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onPlotDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!plotdlg.isNull()) {
      plotdlg->activateWindow();
      plotdlg->raise();
      return;
    }
  
    plotdlg=new PlotDlg(0,base(),"PlotDlg");
    connect(plotdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onPlotDlgTalkBack(int)));
    
    plotdlg->setData(doc->pstate.pcubes
		     ,doc->pstate.pgrid
		     ,doc->pstate.pgridi
		     ,doc->pstate.ptetra
		     );
    
    plotdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(plotdlg.isNull()) return;
    plotdlg->setData(doc->pstate.pcubes
		     ,doc->pstate.pgrid
		     ,doc->pstate.pgridi
		     ,doc->pstate.ptetra
		     );
    plotdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(plotdlg.isNull()) return;
    
    long chn;
    
    plotdlg->getData(doc->pstate.pcubes
		     ,doc->pstate.pgrid
		     ,doc->pstate.pgridi
		     ,doc->pstate.ptetra
		     ,chn);
    
    if(chn) { 
      doc->setModified(); 
      doc->allcalced=false;
    }

    if(!doc->allcalced){
      
      oglview->drawMeshPoints(doc->erg.mesh(),OGLMeshPoints::Full
			      ,doc->pstate.pgrid);
      oglview->drawMeshPoints(doc->erg.mesh(),OGLMeshPoints::Irreducible
			      ,doc->pstate.pgridi);
      oglview->drawMeshCells(doc->erg.mesh(),OGLMeshCells::Microcells
			     ,doc->pstate.pcubes);
      oglview->drawMeshCells(doc->erg.mesh(),OGLMeshCells::Simplices
			     ,doc->pstate.ptetra);
      
      updateData();
    }
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onSweepDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!fssweepdlg.isNull()) {
      fssweepdlg->activateWindow();
      fssweepdlg->raise();
      return;
    }
  
    fssweepdlg=new FSSweepDialog(0,base(),"FSSweepDialog");
    connect(fssweepdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onSweepDlgTalkBack(int)));
    connect(fssweepdlg,SIGNAL(currentValueChanged(double)),
	    this,SLOT(onFSSweepDlgCurrentValueChanged(double)));

    fssweepdlg->saveData(doc->erg);

    fssweepdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(fssweepdlg.isNull()) return;
    fssweepdlg->update();
 }else if(mode==ProtoDlg::End || (mode==ProtoDlg::Accept)){

    if(fssweepdlg.isNull()) return;

    fssweepdlg->restoreData(doc->erg);

    doc->allcalced=false;
    doc->triacalced=false;
    doc->erg.calcRange(doc->vlevel);
    doc->setModified();
    
    updateData();
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onSpecialPointDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!specialpointdlg.isNull()) {
      specialpointdlg->activateWindow();
      specialpointdlg->raise();
      return;
    }
  
    specialpointdlg=new SpecialPointDlg(oglview,0,base(),"SpecialPointDlg");
    connect(specialpointdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onSpecialPointDlgTalkBack(int)));
    

    specialpointdlg->setData(doc->specpoi,doc->defaultspecpoi,doc->currentsym1);
    specialpointdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(specialpointdlg.isNull()) return;
    specialpointdlg->setData(doc->specpoi,doc->defaultspecpoi,doc->currentsym1);
    specialpointdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(specialpointdlg.isNull()) return;

    specialpointdlg->getData(doc->specpoi);
    fstoolbox->updateToolBox();

    if(doc->specpoi->changed()) {
      doc->setModified();
      doc->allcalced=false;
    }
    
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }else{
      oglview->setVisible("special point",doc->specpoi->isVisible());
      oglview->setVisible("special point path",doc->specpoi->showPath()
			  &&doc->specpoi->isVisible());
      oglview->doRendering();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onCutDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!cutdlg.isNull()) {
      cutdlg->activateWindow();
      cutdlg->raise();
      return;
    }
  
    cutdlg=new CutDlg(0,base(),"CutDlg");
    connect(cutdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onCutDlgTalkBack(int)));
    
    cutdlg->setData(doc->icut,doc->ncut0,doc->np,doc->fro,doc->to);

    cutdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(cutdlg.isNull()) return;
    cutdlg->setData(doc->icut,doc->ncut0,doc->np,doc->fro,doc->to);
    cutdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(cutdlg.isNull()) return;

    long chn;
    cutdlg->getData(&doc->icut,doc->ncut0,&doc->np,&doc->fro,&doc->to,&chn);

    /* document is changed tell it*/
    if(chn) {
      doc->setModified();
      doc->refresh();
    }
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onLightDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!lightdlg.isNull()) {
      lightdlg->activateWindow();
      lightdlg->raise();
      return;
    }
  
    lightdlg=new LightDlg(0,base(),"LightDlg");
    connect(lightdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onLightDlgTalkBack(int)));
    

    lightdlg->setData(doc,oglview);
    lightdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(lightdlg.isNull()) return;
    lightdlg->setData(doc,oglview);
    lightdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(lightdlg.isNull()) return;

    lightdlg->getData(doc);
    if(doc->lights()->changed()) {
      doc->setModified();
      doc->allcalced=false;
    }
    if(!doc->allcalced){
      oglview->setLighting();
      oglview->doRendering();
      doc->lights()->setChanged(false);
      doc->allcalced=true;
    }
  }
}








/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onLoadFPLOInFile(SymDlg*symdlg)
{
  if(!doc->erg.emptyLattice()) return;
  //myDebug() << "onLoadFPLOInFile"<<symdlg;
  try{
    QString file=workdir + "=.in";
    doc->fploinsym->readSymInfo(file);
  }catch(Except &ex){
    QMessageBox::warning(this,"Loading symmetry", ex.msg()
			 ,QMessageBox::Ok | QMessageBox::Default);
    return;
    
  }

  *doc->xefsym=*doc->fploinsym;
  symdlg->setData(doc->xefsym,SymDlg::FermiSurfaceEditMode);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotMeshDefine()
{
  MeshDlg *dlg=new MeshDlg(this,base(),"MeshDlg");
  dlg->setData(&doc->erg);
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(&doc->erg);
    if(doc->erg.meshDefChanged()) {
      if(!doc->erg.emptyLattice()){
	int ret=QMessageBox::warning(this,"Mesh changed"
				     ,"Do you want to export the mesh now?"
				     ,QMessageBox::Yes | QMessageBox::Default
				     ,QMessageBox::No  
				     );
	if(ret==QMessageBox::Yes){
	  slotMeshExport();
	}
      }
      doc->allcalced=false;
      doc->setModified();
    }
  }
  delete (dlg);
  return ;


}







/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::onAnnotationDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!annotationdlg.isNull()) {
      annotationdlg->activateWindow();
      annotationdlg->raise();
      return;
    }
    
    AnnotationDlg::AnnotationMode mode=AnnotationDlg::AMAxes;
    
    QAction *a=dynamic_cast<QAction*>(sender());
    if(a){
      mode=(AnnotationDlg::AnnotationMode)a->data().toInt();
    }
    
    annotationdlg=new AnnotationDlg(0,base(),"AnnotationDlg");
    connect(annotationdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onAnnotationDlgTalkBack(int)));
    
    
    annotationdlg->setData(mode,doc,doc->currentsym1,Xf::STCrystal,Xf::LTReciprocial);
    annotationdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(annotationdlg.isNull()) return;
    annotationdlg->refresh(Xf::STCrystal,Xf::LTReciprocial);
    annotationdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(annotationdlg.isNull()) return;

    long ch=0;
    annotationdlg->getData(doc,ch);

    fstoolbox->updateAxesButtons();
    fstoolbox->updateCellButtons();
    if(ch) {doc->setModified();doc->allcalced=false;}

    if(!doc->allcalced){
      updateData();
    }else{
      for(int i=0;i<doc->displayCells()->size();i++){
	DisplayCell *dc=doc->displayCells()->cell(i);
	if(
	   dc->type()!=Xf::DCTMainBoundary&&
	   dc->type()!=Xf::DCTConventional&&
	   dc->type()!=Xf::DCTPrimitive
	   ){
	  oglview->setVisible(dc->id(),dc->boundary()->polyhedron()
			      ->properties().isVisible());
	}
      }
      oglview->doRendering();
      
    }
  

  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */ 

void FSW::slotErgDefine()
{
  //NSF("FSW::slotErgDefine");
  //CPR("");

  if(!ErgImport())   return;



  //CPR("after ErgImport");
  
  FermiDlg *dlg=new FermiDlg(this,base(),"FermiDlg");

  //CPRP(dlg);

  doc->erg.calcRange(doc->vlevel);

  //CPR("after calcefrange");

  dlg->setData(&doc->erg,doc->vlevel);
  //CPR("after setData");
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(&doc->erg);
    if(doc->erg.efChanged()) {
      doc->allcalced=false;
      doc->triacalced=false;

      doc->erg.calcRange(doc->vlevel);
      

      doc->setModified();
    }

    
  }
  delete (dlg);
  return ;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotFilesDlg()
{
  FilesDlg *dlg=new FilesDlg(this,base(),"FilesDlg");
  dlg->setData(&doc->erg,workdir);
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(&doc->erg);
    if(doc->erg.efChanged()) {
      doc->setModified();
      if(!ErgImport()){};

    }
  }
  delete (dlg);
  return;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */ 

void FSW::slotMeshSymmetryDefine()
{
  //NSF("FSW::slotMeshSymmetryDefine");
  //CPR("");

  //if(!ErgImport())   return;



  
  MeshSymDlg *dlg=new MeshSymDlg(this,base(),"MeshSymDlg");




  dlg->setData(&doc->meshsymmetry);
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(&doc->meshsymmetry);
    if(doc->meshsymmetry.userDefinedStuffChanged()) {
      doc->allcalced=false;
      doc->triacalced=false;
      doc->erg.setMeshCalculated(false);
      doc->setModified();
      if(!doc->erg.emptyLattice()){
	int ret=QMessageBox::warning(this,"Mesh changed"
				     ,"Do you want to export the mesh now?"
				     ,QMessageBox::Yes | QMessageBox::Default
				     ,QMessageBox::No  
				     );
	if(ret==QMessageBox::Yes){
	  slotMeshExport();
	}
      }
      
    }

    
  }
  delete (dlg);
  return ;

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotKexpDlg()
{
  long chn;
  KexpDlg *dlg=new KexpDlg(this,base(),"KexpDlg");
  dlg->setData(doc->write_tet_file,doc->calc_only_part_occupied
	       ,doc->part_occu_offset );
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(&(doc->write_tet_file),&(doc->calc_only_part_occupied)
		 ,doc->part_occu_offset,&chn);
    if(chn) {
      doc->setModified();
      doc->refresh();
    }
  }
  delete (dlg);
  
  return ;


}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotOptDlg()
{
  long chn;
  OptDlg *dlg=new OptDlg(this,base(),"OptDlg");
  dlg->setData(doc->vlevel,doc->tol );
  long ret=dlg->exec();
  if(ret){  	
    dlg->getData(doc->vlevel,doc->tol,chn);
    if(chn) {
      doc->setModified();
      doc->refresh();
    }
  }
  delete (dlg);
  
  return ;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotInputControl()
{
  InpConDlg *dlg=new InpConDlg(this,base(),"InpConDlg");

  dlg->setData(doc->icbandrestrict,doc->icbandmin,doc->icbandmax);
  long ret=dlg->exec();
  if(ret){  	
    long chn;
    dlg->getData(doc->icbandrestrict,doc->icbandmin,doc->icbandmax,chn);


    if(chn) {
      doc->erg.setEnergiesLoaded(false);
      doc->setModified();
    }

  }
  delete (dlg);
  return ;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long FSW::calcSymmetry()
{
  NSF("FSW::calcSymmetry");
  //myDebug("%s\n",nsubfunc);



  //  Except ex; long warn=0;
  if(doc->erg.emptyLattice() ){
    //    if( ! doc->xefsym->changed() ) return 1;
    *(doc->currentsym1)=*(doc->xefsym);
  }else{
    /* get symm info from file */


    QString file=workdir + "=.in";

    try{
      doc->fploinsym->readSymInfo(file);
    }catch(Except &ex){
      QMessageBox::warning(this,"Loading symmetry", ex.msg()
			   ,QMessageBox::Ok | QMessageBox::Default);
      return 0;

    }
    if(doc->fploinsym->structureType() != Xf::STCrystal){
      QMessageBox::warning(this,"Loading symmetry", 
			   "Structure type of underlying FPLO\n"
			   "calculation does not belong to a 3D crystal!"
			   ,QMessageBox::Ok | QMessageBox::Default);
      return 0;
    }

    //    if( ! doc->fploinsym->changed() ) return 1;
    *(doc->currentsym1)=*(doc->fploinsym);
  }


  if( doc->currentsym1->calculated() ) return 1;

  statusbar->setMessage("calculate symmetry");

  /* calc symmetry info */

  String out;
  try{
    doc->currentsym1->calcSymFromGroupInput(out,false,
					    doc->currentsym1->changed());
  }catch(Except &ex){
    //myDebug() << ex.msg() << ex.stop();
    if(ex.stop()==0){
      QMessageBox::warning(this,ex.title(),ex.msg()
			   ,QMessageBox::Ok|QMessageBox::Default
			   ,QMessageBox::NoButton);
    }else if(ex.stop()==1){
      QMessageBox::critical(this,ex.title(),ex.msg()
			    ,QMessageBox::Ok|QMessageBox::Default
			    ,QMessageBox::NoButton);
      statusbar->setMessage();
      return 0;
    }else{
      QMessageBox::critical(this,ex.title(),ex.msg()
			    ,QMessageBox::Ok|QMessageBox::Default
			    ,QMessageBox::NoButton);

      qApp->closeAllWindows();	
      qApp->exit(1);
      exit(1);
    }    
  }



  
  /* special symmtry point */
  doc->defaultspecpoi->calcPoints(doc->currentsym1,doc->vlevel);
  

  

  
  /* collect output */
  base()->outwin->setText( "symmetry",out() );

  base()->outwin->setText( "specpoi"
			   ,doc->defaultspecpoi->reportSpecPoints(doc->vlevel) );


  doc->xefsym->setChanged(false);
  doc->fploinsym->setChanged(false);
  doc->bound()->setCalculated(false);
  doc->displayCells()->setCalculated(false);
  doc->axes()->setCalculated(false);


  doc->erg.setMeshCalculated(false);
  doc->erg.setEnergiesLoaded(false);
  doc->triacalced=false;
  doc->allcalced=false;


  statusbar->setMessage();

  return 1;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FSW::calcKPoints()
{
  NSF("FSW::calcKPoints");
  myDebug("%s\n",nsubfunc);


  //myDebug()<<"here1";
  if(! calcSymmetry() ) return 0;
  //myDebug()<<"here2";
  if(doc->erg.meshCalculated() ) return 1;
  //myDebug()<<"here3";

  statusbar->setMessage("Calculate mesh");
  qApp->processEvents();
  qApp->flush();
  /* init output buffer */



  
  try{
    doc->meshsymmetry.setUpFromSymmetry(doc->currentsym1);
    Matrix3x3 cell;
    Vec3 origin;
    //todo: make sure cell is used in boundary
    doc->bound()->getBoxCell(origin,cell,doc->currentsym1);
//     myDebug() << "orig="<<origin;
//     myDebug() << "cell="<<cell;
    doc->erg.setUpMesh(oglview,doc->meshsymmetry,doc->vlevel
		       ,doc->erg.useBoundaryCell(),origin,cell);
  }catch(Exception &ex){
    statusbar->showMessage("");
    QMessageBox::warning(this,"Loading energies", ex.msg()()
			   ,QMessageBox::Ok | QMessageBox::Default);
    if(ex.severity()>=Exception::error){
      return 0;
    };
  }

  



  oglview->drawMeshPoints(doc->erg.mesh(),OGLMeshPoints::Full
			  ,doc->pstate.pgrid);
  oglview->drawMeshPoints(doc->erg.mesh(),OGLMeshPoints::Irreducible
			  ,doc->pstate.pgridi);
  oglview->drawMeshCells(doc->erg.mesh(),OGLMeshCells::Microcells
			 ,doc->pstate.pcubes);
  oglview->drawMeshCells(doc->erg.mesh(),OGLMeshCells::Simplices
			 ,doc->pstate.ptetra);
  

//   cellState cstate={"temp",true,0xffff0000,0.01};
//   oglview->drawSpannedBox(doc->erg.mesh()->cell(),cstate);

  
  doc->triacalced=false;
  doc->allcalced=false;
  

  
  base()->outwin->setText( "k-mesh"
                         , doc->erg.mesh()->reportKPoints(doc->vlevel) );


  statusbar->setMessage();
  return 1;

}




/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FSW::calcDisplayCells()
{
  NSF("FSW::calcDisplayCells");

  //myDebug()<<"ZZZZZZZZZZZZZZZZZZZ"<<doc->displayCells()->changed();
  //myDebug()<<"CCCCCCCCCCCCCCCCAAA"<<doc->displayCells()->calculated();

  if(! calcSymmetry() ) return 0;
  if( doc->displayCells()->calculated()) return 1;
  

  //  myDebug("%s\n",nsubfunc);

  statusbar->setMessage("calculating boundary and display cells ... ");
  qApp->processEvents();
  qApp->flush();

  
  /* calc display cells*/

  
  oglview->removeObject("Display cells ");

  QString out=doc->displayCells()->calculate(doc->bound()->latticeType()
					     ,doc->currentsym1,doc->tol
					     ,doc->vlevel)();
  base()->outwin->setText( "cell" , out );

  

  
  for(int i=0;i<doc->displayCells()->size();i++){
    DisplayCell *dc=doc->displayCells()->cell(i);
    oglview->drawDisplayCell(dc);
    //myDebug()<<"GGGGGGGGGGGG   dc->Id"<<dc->id()<<dc->name();
  }
  
  doc->axes()->setCalculated(false);


 


  doc->triacalced=false;
  doc->allcalced=false;

  statusbar->setMessage();
  
  return 1;
}

/* ***********************************************************************
 * import energy
 * ***********************************************************************
 */

long FSW::ErgImport()
{
  NSF("FSW::ErgImport");
  myDebug("%s\n",nsubfunc);



  if(!calcSymmetry())return 0 ;

  if(!calcKPoints())return 0 ;


  if(doc->erg.energiesLoaded()) return 1;
  
  if(doc->erg.emptyLattice()){
    statusbar->setMessage("calculating  band energies ... ");
  }else{
    statusbar->setMessage("importing  band energies ... ");
  }
  
  qApp->processEvents();
  qApp->flush();
  

  /* read in fermi energy or calc empty lattice */
  if(doc->pstate.pfermi){
    try{
      doc->loadEnergies(workdir,doc->erg.bandFile(),doc->erg.bandWeightsFile());
    }catch (Exception &ex){
      QMessageBox::warning(this,"Loading energies", ex.msg()()
			   ,QMessageBox::Ok | QMessageBox::Default);
      statusbar->setMessage();
      //if(!doc->pstate.pfermi) return 1;
      return 0;
    }
  }else{
    return 1;
  }
  
  doc->erg.mesh()->calculateGradient();
  //doc->erg.mesh()->printData();
  doc->triacalced=false;
  
  
  doc->erg.calcRange(doc->vlevel);
  
  
  doc->complainedWeightSym=true;
  
  
  doc->erg.setEnergiesLoaded(true);
  doc->allcalced=false;
 
  if(!doc->erg.emptyLattice()){
    statusbar->setMessage("band energies imported");
  }
  //  myDebug("%s returning 1\n",nsubfunc);

  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FSW::TriaCalc()
{
  NSF("FSW::TriaCalc");
  myDebug("%s\n",nsubfunc);


  if(!doc->pstate.pfermi) return 1;

  if(!calcSymmetry())return 0 ;
  if(!calcKPoints())return 0 ;
  if(!calcDisplayCells())return 0 ;
  if(!ErgImport())  return 0 ;


  QString text=doc->erg.checkIndices();
  if(!text.isEmpty()){
    QMessageBox::information(this,"Make Fermi surface", text
			     ,QMessageBox::Ok | QMessageBox::Default);
    slotErgDefine();
  }


  if(!doc->triacalced){
    doc->erg.calculateIsoSurfaces(oglview,doc->bound(),doc->bclip1);
    doc->triacalced=true;

    doc->isocolor.setChanged(true);
    doc->triacalced=true;
  }
  
  
  if( ! doc->erg.efChanged() && !doc->erg.rangeChanged()  ) return 1;

  myDebug() <<"analyse Fermi surface";

  statusbar->setMessage("analyse Fermi surface");

  

  

  
  
  try{
    base()->outwin->setText( "fs-analyse"
 			   ,doc->
 			   erg.analyseFermiSurface(doc->currentsym1->vuc()
 						   ,doc->tol
 						   ,doc->currentsym1
 						   ->lengthUnit()
						   ,doc->relativisticMode()
 						   ,doc->vlevel) );
  }catch (QString &msg){
    QMessageBox::warning(this,"Calculating", msg
		       ,QMessageBox::Ok | QMessageBox::Default);
    statusbar->setMessage();
    return 0;
  }
  
  doc->erg.setEfChanged(false);
  doc->erg.setRangeChanged(false);
  doc->allcalced=false;
  
  statusbar->setMessage();
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long FSW::calcAxes()
{

  if(! calcSymmetry() ) return 0;
  if(! calcDisplayCells() ) return 0;

  if( doc->axes()->calculated()  ) return 1;

  doc->axes()->calculateAxes(doc->bound()->latticeType()
			     ,doc->bound()->radius(),doc->currentsym1);


  for(long i=-1;++i<doc->axes()->size();){
    Axes *ax=doc->axes()->axes(i);
    oglview->drawAxes(ax,doc->fontRes());
  }

  doc->allcalced=false;
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::updateData(bool updatePanZoom)
{
  // slotPlotCalc(updatePanZoom);
  // return;
  QThread *thread=new QThread();
  Worker *worker=new Worker(30000/10000,updatePanZoom);
  worker->moveToThread(thread);
  //connect(worker, SIGNAL(error(QString)), this, SLOT(errorString(QString)));
  connect(&interrupt, SIGNAL(interuptRequested()), worker, SLOT(requestInterrupt()),
	  Qt::QueuedConnection);
  connect(thread, SIGNAL(started()), worker, SLOT(process()));
  connect(worker, SIGNAL(done(bool)), this, SLOT(dataUpdated(bool)));
  connect(worker, SIGNAL(finished()), thread, SLOT(quit()));
  connect(worker, SIGNAL(finished()), worker, SLOT(deleteLater()));
  connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
  thread->start();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void FSW::dataUpdated(bool updatePanZoom)
{
  myDebug()<<"dataUpdated, updatePanZoom="<<updatePanZoom;
  slotPlotCalc(updatePanZoom);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotPlotCalc(bool updatePanZoom)
{
  NSF("FSW::slotPlotCalc");
  //myDebug() << nsubfunc<<updatePanZoom;

  oglview->mute(true);

  statusbar->setProgress(0.); 
  statusbar->setProgress(0.01); 

  double p=statusbar->getCurrentProgress();
  if(!calcSymmetry())goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 

  if(interrupt.requested()) goto clean;

  p=statusbar->getCurrentProgress();
  if(!calcKPoints()) goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 

  if(interrupt.requested()) goto clean;


  p=statusbar->getCurrentProgress();
  if(!calcDisplayCells())goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 
  
  if(interrupt.requested()) goto clean;

  if(!calcAxes())goto clean;

  p=statusbar->getCurrentProgress();
  if(!ErgImport())   goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 

  if(interrupt.requested()) goto clean;


  p=statusbar->getCurrentProgress();
  if(!TriaCalc())    goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 

  if(interrupt.requested()) goto clean;

  p=statusbar->getCurrentProgress();

  statusbar->setAllotedPercent(((0.9-p)/(1-p)));


  doc->defaultspecpoi->setVisible(doc->specpoi->isVisible());
  doc->defaultspecpoi->setShowPath(doc->specpoi->showPath());
  doc->defaultspecpoi->setPathColor(doc->specpoi->pathColor());
  doc->defaultspecpoi->setUserDefined(doc->specpoi->userDefined());
  if(doc->specpoi->userDefined()){
    
    if(doc->specpoi->changed()){
      oglview->drawSpecialPoints(doc->specpoi,doc->fontRes());
      doc->specpoi->setChanged(false);
    }
  }else{
    if(doc->defaultspecpoi->changed()){
      oglview->drawSpecialPoints(doc->defaultspecpoi,doc->fontRes());
      doc->defaultspecpoi->setChanged(false);
    }
  }

  if(1){
    long idata=0;
    if(doc->excol){
      idata=doc->iweight;
      //myDebug()<<"idata"<<idata;
      
      if(doc->erg.useSymmetry() && doc->complainedWeightSym){
	doc->complainedWeightSym=false;
	QMessageBox::warning(this,"",
			     "Beware that the bandweights do not have the\n"
			     "point symmetry as the energies.\n"
			     "Using irreducible meshes is faulty"
			     ,QMessageBox::Ok | QMessageBox::Default);

      }
    }
    String min="0";
    String max="1";
    QString title="[m/s]";
    double prufac=1.;
    if(!doc->excol){
      prufac=pow(10.,doc->ufactor);
      prufac/=VUnits::velocityUnitFactor(doc->vunit);
      if(doc->ufactor==0){
	title.sprintf("[%s]",VUnits::velocityUnit(doc->vunit));
      }else{
	title.sprintf("[10^%ld %s]",doc->ufactor
		      ,VUnits::velocityUnit(doc->vunit));
      }
    }else{
      //myDebug() << "erg.labels()"<<doc->erg.labels();

      prufac=pow(10.,doc->ufactore);
      if(doc->ufactore!=0 && doc->legendcontrol.showunits){
	title.sprintf("%s[10^%ld]",String(doc->erg.label(doc->iweight-1))(),
		      doc->ufactore);
      }else{
	title.sprintf("%s",String(doc->erg.label(doc->iweight-1))());
      }
    }
    if(doc->legendcontrol.usetitle){
      title=Sprintf("%-s",doc->legendcontrol.title()).stripBlanks().chop("0")();

    }
    myDebug()<<"TITLE "<<title;
    double vmin,vmax;
    
    if(doc->isocolor.changed()){
      
      double conv=constant_hbar/(constant_me*constant_aBtoAng
				 *constant_angstroem
				 *constant_hatoev
				 );
      
      if(doc->currentsym1->lengthUnit() == Xf::Angstroem ) {
	conv=conv/constant_aBtoAng; 
      }
      if(true){
	printf("hbar  =%g\n",constant_hbar);
	printf("me    =%g\n",constant_me);
	printf("aB    =%g\n",constant_aBtoAng*constant_angstroem);
	printf("Ha/eV =%g\n",constant_hatoev);
	printf("conv: eV aB/hbar   =%g m/s\n",conv);
      }
      conv/=twopi;// reciprocial
      if(doc->excol) { conv=1; }
      double dmin=0,dmax=1;
      int n=0;
      foreach(TriangleMesh *trm,doc->erg.triameshs()){
	if(trm) {
	  double d0,d1;
	  for(int id=0;id<1;id++){
	    long ida;
	    if(id==0)ida=1;//1;
	    if(id==1)ida=2;//2;
	    if(id==2)ida=3;
	    ida=idata;
	    trm->dataRange(ida,d0,d1);
	    myDebug()<<"d0,d1"<<d0*conv<<d1*conv;
	    if(n==0){
	      dmin=d0;dmax=d1;
	    }else{
	      dmin=MyMath::min<double>(dmin,d0);
	      dmax=MyMath::max<double>(dmax,d1);
	    }
	    n++;
	  }
	}
      }
      if(!n){
	dmin=0.;
	dmax=1.;
      }
      myDebug()<<"n,dmin,dmax"<<n<<dmin*conv<<dmax*conv;
      doc->vmindef=dmin*conv;
      doc->vmaxdef=dmax*conv;

      doc->vminedef=dmin*conv;
      doc->vmaxedef=dmax*conv;


      if(doc->excol){
	double ufac=pow(10.,doc->ufactore);
	vmin=(doc->vdefe ?     doc->vminedef   : doc->vmine*ufac );
	vmax=(doc->vdefe ?     doc->vmaxedef   : doc->vmaxe*ufac );
      }else{
	double f=VUnits::velocityUnitFactor(doc->vunit);
	double ufac=pow(10.,doc->ufactor)/f;

	vmin=(doc->vdef ?     doc->vmindef : doc->vmin*ufac );
	vmax=(doc->vdef ?     doc->vmaxdef : doc->vmax*ufac );
      }

      foreach(TriangleMesh *trm,doc->erg.triameshs()){
	if(trm) {
	  oglview->invalidateMesh(trm);
	

	  trm->applyColor(idata,doc->isocolor,vmin/conv,vmax/conv);

	  oglview->drawTriangleMesh(trm,doc->pstate.pfermi,0);
	}
	doc->isocolor.setChanged(false);
      }

    }
    if(doc->excol){
      double ufac=pow(10.,doc->ufactore);
      vmin=(doc->vdefe ?     doc->vminedef   : doc->vmine*ufac );
      vmax=(doc->vdefe ?     doc->vmaxedef   : doc->vmaxe*ufac );
    }else{
      double f=VUnits::velocityUnitFactor(doc->vunit);
      double ufac=pow(10.,doc->ufactor)/f;
      
      vmin=(doc->vdef ?     doc->vmindef : doc->vmin*ufac );
      vmax=(doc->vdef ?     doc->vmaxdef : doc->vmax*ufac );
    }
    myDebug()<<"vmin,vmax"<<vmin<<vmax;
    if(doc->legendcontrol.useminlabel){
      min.sprintf("%-s",doc->legendcontrol.minlabel())
	.stripBlanks().chop("0");
    }else{
      min.sprintf("%-#6.3f",vmin/prufac).stripBlanks().chop("0");
    }
    if(doc->legendcontrol.usemaxlabel){
      max.sprintf("%-s",doc->legendcontrol.maxlabel())
	.stripBlanks().chop("0");
    }else{
      max.sprintf("%-#6.3f",vmax/prufac).stripBlanks().chop("0");
    }
    oglview->drawLegend(title,min(),max(),doc->showLegend()
			,doc->pstate.pvelolegunit,
			doc->isocolor);

  }

  foreach(TriangleMesh *trm,doc->erg.triameshs()){
    if(trm){
      oglview->setVisible(trm->id(),doc->pstate.pfermi);
    }
  }
  oglview->setVisible("Legend title",doc->pstate.pvelolegunit&&
		      doc->showLegend());
  oglview->setVisible("Legend colorband",doc->showLegend());
  oglview->setVisible("Legend text",doc->showLegend());

  oglview->setLighting();

      
  if(interrupt.requested()) goto clean;

  if(updatePanZoom) oglview->resetPanZoom(OGLView::Reset,false);

  doc->refresh();
  statusbar->setAllotedPercent(1.00);

  oglview->mute(false);

  oglview->doRendering();
  
  doc->axes()->setChanged(false);  
  doc->lights()->setChanged(false);

  statusbar->deleteMessage();

  fstoolbox->setMuted(false);
  
  doc->allcalced=true;
  

 clean:
  statusbar->setProgress(0); 
  statusbar->setMessage();
  return;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void FSW::slotPlotReCalc()
{
  doc->resetflags();
  updateData();

}

//EOF
