// @(#)FSW.hh
//author: Klaus.Koepernik@ifw-dresden.de
//
#ifndef FSW_H
#define FSW_H

#include <QPointer>
#include "ProtoDlg.hh"
#include "ControlBase.hh"
#include "ui_FSW.h"

class MWBase;
class QMenu;
class FSDoc;
class FogDlg;
class Status;

class PlotDlg;
class SymDlg;
class SurColDlg;
class CutDlg;
class LightDlg;
class BoundDlg;
class FSSweepDialog;
class SpecialPointDlg;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class FSW : public ControlBase ,private Ui::FSW
{
  Q_OBJECT
public:
  FSW(MWBase *parent = 0);
  ~FSW();

  virtual void help();
  virtual void afterShow(const QString &load);
  virtual bool queryExit();

  void slotPlotCalc(bool updatePanZoom=false);

  virtual void updateData(bool updatePanZoom=false);

protected:
  bool isModified();

 private:

  enum SaveMode{FILESAVE=1,FILESAVEAS=2};




private slots:
  void slotFileSave();
  void slotFileSaveAs();
  void slotFileOpen();
  void slotFileClose();
  void slotFilePrint();
  void slotMeshExport();
  void slotFilesDlg();

  void onLoadFPLOInFile(SymDlg *);
  
  void saveit( SaveMode saveas );
  void fileLoad(const QString &fileName);


  void inputMenu_aboutToShow();
  void emptlAction_triggered(bool onoff);
  void optionsMenu_aboutToShow();
  void clipAction_triggered( bool onoff );
  void winMenu_aboutToShow();
  void slotShowOutwin(bool onoff);

  void slotMeshDefine();
  void slotErgDefine();
  void slotMeshSymmetryDefine();

  void slotInputControl();
  void slotOptDlg();
  void slotKexpDlg();

  void slotRunFPLO();

  long calcSymmetry();
  long calcKPoints();
  long calcDisplayCells(); 
  long calcAxes();
  long ErgImport();
  long TriaCalc();

  void slotPlotReCalc();

  void dataUpdated(bool );








  void onFogDlgTalkBack(int mode=ProtoDlg::Create);
  void onBoundDlgTalkBack(int mode=ProtoDlg::Create);
  void onSymDlgTalkBack(int mode=ProtoDlg::Create);
  void onColorDlgTalkBack(int mode=ProtoDlg::Create);
  void onPlotDlgTalkBack(int mode=ProtoDlg::Create);
  void onLightDlgTalkBack(int mode=ProtoDlg::Create);
  void onAnnotationDlgTalkBack(int mode=ProtoDlg::Create);
  void onSweepDlgTalkBack(int mode=ProtoDlg::Create);
  void onSpecialPointDlgTalkBack(int mode=ProtoDlg::Create);
  void onCutDlgTalkBack(int mode=ProtoDlg::Create);

  void onFSSweepDlgCurrentValueChanged(double value);


private:

  void setupMenus();
  void updateSizes(bool fresh);

private:
  FSDoc *doc;
  friend class FSToolBox;
  QAction *symAction;
  QAction *emptlAction;
  QAction *clipAction;
  QAction *showOutAction;
  QAction *calcAction;
  QAction *canvasAction;

  QPointer<PlotDlg> plotdlg;
  QPointer<SurColDlg> surcoldlg;
  QPointer<CutDlg> cutdlg;
  QPointer<FogDlg> fogdlg;
  QPointer<BoundDlg> bounddlg;
  QPointer<LightDlg> lightdlg;
  QPointer<SymDlg> symdlg;
  QPointer<FSSweepDialog> fssweepdlg;
  QPointer<SpecialPointDlg> specialpointdlg;
};

#endif // FSW_H
