/*************************************************************************
 * @(#)IconCache.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   12 Aug 2009
 *************************************************************************
 */

#include <QColor>
#include <QDebug>
#include <QPainter>
#include "IconCache.hh"
#include "Polyhedron.hh"
#include "MyMath.hh"
#include "BondDefs.hh"
#include "Vec3.hh"


using namespace XFPLO;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
static QColor ff(double x,double la,double expo,int h0,int s0,int v0,int a)
{
  int v,s;

  if(1){
    double v1=la*v0;
    double w1=v1;
    double w2=s0;
    double w=w1+w2;
    w1/=w;
    w2/=w;
    
    
    double y=pow(x,expo);
    
    if(y<w1){
      v=(int)(v1+(256-v1)*y/w1);
      s=s0;
    }else{
      v=MyMath::max<long>(0,MyMath::min<long>(255,(int)((v1+(1-v1)*(y-w1/w)/w2)*256)));
      s=MyMath::max<long>(0,MyMath::min<long>(255,(int)(s0-s0*(y-w1/w)/w2)));
      v=255;
      s=s0-s0*(y-w1)/w2;
    }
   
    //myDebug() <<"x"<<x<<"y"<<y<<"s"<<s<<"v"<<v << w1<<w2;
  }
  return QColor::fromHsv(h0,s,v,a);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void IconCache::makeImage(QImage &qi,QRgb rgb)
{
  int Nx=qi.width();
  int Ny=qi.height();
  int h=QColor(rgb).toHsv().hue();
  int s=QColor(rgb).toHsv().saturation();
  int v=QColor(rgb).toHsv().value();
  
  QColor c;
  
  for(int x=0;x<Nx;x++){
    for(int y=0;y<Ny;y++){
      QRgb q=qi.pixel(x,y);
      if(qAlpha(q)!=0){
	double X=qGray(q)/256.;
	c=ff(X,0.85,4,h,s,v,qAlpha(q));
	qi.setPixel(x,y,c.rgba());
      }
    }
  }
  
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QIcon IconCache::getBondAtomIcon(QRgb atomrgb,bool hasbond,
				 const XFPLO::PolyhedronProperties &polyprops,
				 int imode)
{

  BondDefGroupEnum::CutOffMode mode=(BondDefGroupEnum::CutOffMode) imode;

  QImage bal=getAtomImage(atomrgb);
  int Nxa=bal.width();
  int Nya=bal.height();

  //myDebug()<<"atom"<<Nxa<<Nya;
  // myDebug()<<"bond"<<Nxo<<Nyo;

  QImage im(Nxa*3,Nya,bal.format());
  im.fill(0x00);
  QPainter wp;
  wp.begin(&im);
  if(hasbond) {
    // QImage bon=getBondImage(bondrgb);
    // wp.drawImage(QPointF(0,0),bon);
  
    switch(mode){
    case BondDefGroupEnum::Boundary:
      wp.drawImage(QPointF(0*Nxa,0),QImage(":/Icons/cutmodeb"));
      break;
    case BondDefGroupEnum::AddMissingAtoms:
      wp.drawImage(QPointF(0*Nxa,0),QImage(":/Icons/cutmodem"));
      break;
    case BondDefGroupEnum::RemoveIncomplete:
      wp.drawImage(QPointF(0*Nxa,0),QImage(":/Icons/cutmodei"));
      break;
    case BondDefGroupEnum::RemoveLoner:
      wp.drawImage(QPointF(0*Nxa,0),QImage(":/Icons/cutmodel"));
      break;
    }
  

    if(polyprops.isVisible()){
      if(polyprops.drawFaces()){
	wp.drawImage(QPointF(1*Nxa,0),getPolyImage(polyprops.faceMaterial().color()));
      }
      if(polyprops.drawRim()){
	QRgb rgb=polyprops.rimMaterial().color();
	if((rgb&0xffffff)==0x00){
	  rgb=0x010101;
	}
	wp.drawImage(QPointF(1*Nxa,0),getPolyRimImage(rgb));
      }
    }
  }
  wp.drawImage(QPointF(2*Nxa,0),bal);
  wp.end();
  

  return QIcon(QPixmap::fromImage(im));
};


/*EOF*/ 
