/* @(#)IconCache.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */

#ifndef _ICONCACHE_H
#define _ICONCACHE_H 1

#include <QImage>
#include <QPixmap>
#include <QIcon>
#include <QHash>
#include "SReal.hh"
#include "MyDebug.hh"

namespace XFPLO{
  class PolyhedronProperties;
}



class IconCache
{
public:
  IconCache(){};

  void reset(){atomimages.clear();bondimages.clear();
    polyimages.clear();
    polyrimimages.clear();
  }

  const QImage &getAtomImage(QRgb rgb){
    //myDebug()<<"getAtomImage:rgb="<<rgb;
    if( !atomimages.contains(rgb) ) {  
      //myDebug()<<"getAtomImage: does not contain";
      atomimages.insert(rgb,QImage(":/Icons/ball16"));
      makeImage(atomimages[rgb],rgb);
    }
    return atomimages[rgb];
  };


  QIcon getAtomIcon(QRgb rgb){
    return QIcon(QPixmap::fromImage(getAtomImage(rgb)));
  };
  QIcon getBondAtomIcon(QRgb atomrgb,bool hasbond,
			const XFPLO::PolyhedronProperties &polyprops,
			int mode);

  const QImage &getBondImage(QRgb rgb){
    if( !bondimages.contains(rgb) ) {  
      bondimages.insert(rgb,QImage(":/Icons/bond16"));
      makeImage(bondimages[rgb],rgb);
    }
    return bondimages[rgb];
  }

  const QImage &getPolyImage(QRgb rgb){
    if( !polyimages.contains(rgb) ) {  
      polyimages.insert(rgb,QImage(":/Icons/poly16"));
      makeImage(polyimages[rgb],rgb);
    }
    return polyimages[rgb];
  }

  QIcon getBondIcon(QRgb rgb){
    return QIcon(QPixmap::fromImage(getBondImage(rgb)));
  }

private:
  const QImage &getPolyRimImage(QRgb rgb){
    if( !polyrimimages.contains(rgb) ) {  
      polyrimimages.insert(rgb,QImage(":/Icons/polyrim16"));
      makeImage(polyrimimages[rgb],rgb);
    }
    return polyrimimages[rgb];
  }


private:

  static void makeImage(QImage &qi,QRgb rgb);
  
  QHash<QRgb,QImage> atomimages;
  QHash<QRgb,QImage> bondimages;
  QHash<QRgb,QImage> polyimages;
  QHash<QRgb,QImage> polyrimimages;

};

extern IconCache iconcache;


#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif


#endif /* _ICONCACHE_H */

