#include <QToolBar>
#include <QSignalMapper>
#include <QMenuBar>
#include <QStyleFactory>
#include <QMessageBox>
#include "MWBase.hh"
#include "Master.hh"
#include "FSW.hh"
#include "STR.hh"
#include "AfterShowEvent.hh"
#include "Settings.hh"
#include "BWEdit.hh"
#include "Status.hh"
#include "Actions.hh"
#include "OutWin.hh"
#include "Config.hh"
#include "BrainDialog.hh"
#include "SettingsDlg.hh"
#include "ToolBox.hh"
#include "DocBase.hh"
#include "MyDebug.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */
MWBase::MWBase(Master *m,const Activity &into,bool isfirst)
  : QMainWindow(0),outwin(0),pactivity(into)
  ,master(m),pcontrol(0),statusbar(0),dummy(0)
  ,pinitialmenu(isfirst),signalMapper(0),ptoolbox(0)
{
  
  MASSERT("Master is zero",m);

  setContextMenuPolicy(Qt::NoContextMenu);  

  master->registerActivity(this);

  if(!menuBar()){
    setMenuBar(new QMenuBar(this));
  }
  cmdMenu=menuBar()->addMenu("&Activity");
  cmdMenu->menuAction()->setData(Actions::data(Actions::Activities));
  settingsMenu=menuBar()->addMenu("&Settings");
  settingsMenu->menuAction()->setData(Actions::data(Actions::Settings));
  windowMenu=menuBar()->addMenu("&Window");
  windowMenu->menuAction()->setData(Actions::data(Actions::Window));
  helpMenu=menuBar()->addMenu("&Help");
  helpMenu->menuAction()->setData(Actions::data(Actions::Help));
  pmenulist.append(cmdMenu);
  pmenulist.append(settingsMenu);
  pmenulist.append(windowMenu);
  pmenulist.append(helpMenu);
  
  QAction *a;

  makeCmdMenu();

  a=helpMenu->addAction("&Help");
  a->setShortcut(QKeySequence(Qt::Key_F1));

  connect(a,SIGNAL(triggered()),this,SLOT(slotHelp()));
  a=helpMenu->addAction("About &Qt");
  connect(a,SIGNAL(triggered()),this,SLOT(slotHelpAboutQt()));
  a=helpMenu->addAction(QIcon(":/Icons/logo32"),"&About");
  connect(a,SIGNAL(triggered()),this,SLOT(slotHelpAbout()));


  a=addMenu(settingsMenu->menuAction(),"&Preferences","Preferences"
	    ,Actions::Preferences);
  connect(a,SIGNAL(triggered()),this,SLOT(slotPreferences()));


  QMenu *me = new QMenu(windowMenu);
  me->setTitle("&Style");
  windowMenu->addAction(me->menuAction());
  QAction  *ac;
  foreach(QString st,QStyleFactory::keys()){
    ac=addMenu(me->menuAction(),st
	       ,QString("set %1 style").arg(st)
	       ,Actions::Window_Style_Style);
    ac->setData(Actions::data(Actions::Window_Style_Style,st));
    me->addAction(ac);
  }
  connect(me, SIGNAL(triggered(QAction*))
	  , this, SLOT(slotWindowStyleAction(QAction *)));


  this->resize(200,0);


  if(statusbar) delete statusbar;
  statusbar = new Status(this);
  statusbar->setObjectName("statusbar");
  setStatusBar(statusbar);

  dummy=new QWidget(this);
  setCentralWidget(dummy);

  // create outputwindow instance
  outwin = new OutWin(0);

  
  
  Settings set(QString("mainwindow ")+pactivity.activityTypeName());
  resize(set.getSize("geometry/size",QSize(400, 400)));
  if(master->numOfActivitiesOfType(pactivity.type())<=1){
    move(set.getPoint("geometry/pos",QPoint(100,100)));
  }


  setWindowIcon(QIcon(":/Icons/logo32"));

  
  if(!pactivity.isA(Activity::None)){
    morph(pactivity);
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MWBase::~MWBase()
{
  //  myDebug() << "MWBase:: here I die!\n";

  Settings set(QString("mainwindow ")+pactivity.activityTypeName());
  set.set("geometry/size",size());
  set.set("geometry/pos",pos());

  if(ptoolbox) delete ptoolbox;ptoolbox=0;


  if(pcontrol) delete pcontrol; pcontrol=0;
  if(statusbar) delete statusbar;
  if(outwin) {
    outwin->close();
    delete outwin;
  }
  delete dummy;

  if(signalMapper) delete signalMapper; signalMapper=0;
  close();
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::morph(const Activity &into)
{
  //myDebug() << "MWBase::morph 1\n" << pcontrol;
  if(pcontrol){
    myDebug() << "Cannot morph, once specialized\n";
    exit(2);
  }

  if(dummy) delete dummy; dummy=0;

  pactivity=into;
  
  myDebug()<<"morp pactivity="<<pactivity;
  QString load=".";
  if(!pactivity.cdir().isEmpty())
    load=pactivity.cdir()+"/"+pactivity.file();
 
  Settings set(QString("mainwindow ")+into.activityTypeName());
  resize(set.getSize("geometry/size",QSize(400, 400)));
  if(into.isA(Activity::FermiSurface)){
    pcontrol=new FSW(this);
  }else if(into.isA(Activity::Structure)){
    pcontrol=new STR(this);
  }else if(into.isA(Activity::StructureFPLOIn)){
    pcontrol=new STR(this);
    if(!pactivity.cdir().isEmpty())
      load=pactivity.cdir()+"/=.in";
    else
      load="./=.in";
  }else if(into.isA(Activity::BandWeightCreate)){
    pcontrol=new BWEdit(this);
  }else{
    myDebug() << "MWBase::morph: unknown morphing request \n";
    exit(2);
  }

  
  if(master->showOutWin()) outwin->show();
  setCentralWidget(pcontrol);
  if(!pactivity.cdir().isEmpty())
    pcontrol->document()->setCurrentDir(pactivity.cdir());
  pcontrol->show();
  

  pinitialmenu=false;
  makeCmdMenu();

  if(pcontrol){
    QEvent *event=new AfterShowEvent(AfterShowEvent::AfterShowEventType,load);
    QCoreApplication::postEvent(pcontrol,event);
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::slotHelpAbout()
{
  QMessageBox::about(this,windowTitle(), IDS_APP_ABOUT );
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::slotHelpAboutQt()
{
  QMessageBox::aboutQt (this,this->windowTitle());
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::slotHelp()
{
  if(pcontrol)
    pcontrol->help();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::slotPreferences()
{
  SettingsDlg *dlg=new SettingsDlg(this,this,"SettingsDlg");
  dlg->exec();
  delete (dlg);
  if(ptoolbox)
    ptoolbox->updateToolBox();  
  if(pcontrol){
    if(pcontrol->oglView()){
      Settings set("application");
      pcontrol->oglView()->
	setBadTransparency(
			   set.getBool("graphics/badTransparencyOnMove",true));
      pcontrol->oglView()->doRendering();
    }
  }

  return ;

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::slotWindowStyleAction(QAction *a)
{
  //myDebug() << a->data();
  QString data=Actions::extract(a->data(),1);
  //myDebug() << data;
  qApp->setStyle( data );

  Settings("application").set("style",data);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QAction *MWBase::getNewMenuBarItem(Actions::ActionData ad)
{
  QMenu *m=new QMenu(menuBar());
  m->menuAction()->setData(Actions::data(ad));
  pmenulist.append(m);
  return menuBar()->insertMenu(settingsMenu->menuAction(),m);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
QToolBar *MWBase::getNewToolbar()
{
  QToolBar *t=new QToolBar(this);
  ptoolbarlist.append(t);
  addToolBar(Qt::TopToolBarArea, t);
  return t;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
QAction *MWBase::addMenu(const QAction *parent ,const QString &text
			 ,const QString &tip
			 ,Actions::ActionData ad
			 ,QKeySequence seq)
{
  QAction *a=parent->menu()->addAction(text);
  a->setData(Actions::data(ad));
  a->setIconText(QString(text).remove("&"));
#ifndef QT_NO_TOOLTIP
  a->setToolTip(tip);
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_STATUSTIP
  a->setStatusTip(tip);
#endif // QT_NO_STATUSTIP
  if(!seq.isEmpty())
    a->setShortcut(seq);
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
QAction *MWBase::addMenu(const QAction *parent ,const QString &text
			 ,const QString &tip,Actions::ActionData ad
			 ,const QIcon &icon
			 ,QKeySequence seq)
{
  QAction *a=MWBase::addMenu(parent,text,tip,ad,seq);
  a->setIcon(icon);  
  return a;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool MWBase::requireQuit()
{
  if(!pcontrol)
    return true;
  else
    return pcontrol->queryExit();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::finish()
{
  master->unRegisterActivity(this);
  //myDebug() <<" MWBase::finish after unregistered";
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::makeCmdMenu()
{

  signalMapper = new QSignalMapper(this);
  

  cmdMenu->clear();

  QAction *a=addMenu(cmdMenu->menuAction(),"&Fermi surface"
		     ,"New fermi surface module"
		     ,Actions::Activities_FermiSurface );
  signalMapper->setMapping(a,Activity::FermiSurface);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));



  a=addMenu(cmdMenu->menuAction(),"&Structure","New structure module"
	    ,Actions::Activities_Structure);
  signalMapper->setMapping(a,Activity::Structure);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));


  a=addMenu(cmdMenu->menuAction(),"Structure =.&in"
	    ,"New structure module from FPLO =.in"
	    ,Actions::Activities_StructureFromIn );
  signalMapper->setMapping(a,Activity::StructureFPLOIn);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));


  a=addMenu(cmdMenu->menuAction(),"Fat&Bands","New fatband module" 
	    ,Actions::Activities_FatBand);
  signalMapper->setMapping(a,Activity::BandWeightCreate);
  connect(a,SIGNAL(triggered()),signalMapper,SLOT(map()));


  a=addMenu(cmdMenu->menuAction(),"&Quit all"
	    ,"Close all activities"
	    ,Actions::Activities_Quit
	    ,QKeySequence(Qt::CTRL+Qt::Key_Q));
  connect(a,SIGNAL(triggered()),master,SLOT(requireQuit()));



  connect(signalMapper,SIGNAL(mapped(int)),this,SLOT(onActivityMenu(int)));

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
MWBase *MWBase::callActivityMenuEntry(const Activity &into)
{
  //myDebug() <<"MWBase::callActivityMenuEntry: into="<<into;
  if(pinitialmenu){
    morph(into);
    show();
    return this;
  }else{
    MWBase *base=new MWBase(master,into);
    if(master->showOutWin()) base->outwin->show();
    base->show();
    return base;
  }

  //  myDebug() << "onActivityMenu end";
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::onActivityMenu(int mode)
{

  QString cd="";
  if(pcontrol && pcontrol->document()){
    cd=pcontrol->document()->currentDir();
  }
  //myDebug()<< "MWBase::onActivityMenu" <<  mode<<cd;
  callActivityMenuEntry(Activity((Activity::ActivityType)mode,cd,""));
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void MWBase::enable_but(bool en,const QStringList &menusToSpare)
{
  if(centralWidget()){
    centralWidget()->setEnabled(en);
  }
  menuBar()->setEnabled(true);

  foreach(QMenu *m,pmenulist){
    if(menusToSpare.contains("activities") && m==cmdMenu)
      m->setEnabled(true);
    else if(menusToSpare.contains("help") && m==helpMenu)
      m->setEnabled(true);
    else
      m->setEnabled(en);
  }

  foreach(QToolBar *t,ptoolbarlist){
    t->setEnabled(en);
  }
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool iterateActions(QWidget *w
		    ,bool en,const QList<int> &actionsToSpare)
{
  bool spare=false;

  foreach(QAction *a,w->actions()){
    int act=Actions::extract(a->data(),0).toInt();
    //myDebug() << a->text() << act;
    if(en){
      // enable invariably
      a->setEnabled(true);
      if(a->menu()){
	iterateActions(a->menu(),en,QList<int>());
      }
    }else{
      // disable if not spared
      if(actionsToSpare.contains(act)){
	spare=spare|true;
	a->setEnabled(true);
      }else{
	// disable all
	bool sp=false;
	if(a->menu()){
	  //myDebug() <<a->text()<<"has menu";
	  sp=iterateActions(a->menu(),en,actionsToSpare);
	  //myDebug() <<a->text()<<"sp="<<sp;
	}
	if(sp){
	  spare|=sp;
	  a->setEnabled(spare);
	}else
	  a->setEnabled(false);
      }
    }

  }// foreach
  
  return spare;
}

void MWBase::setEnableAction_but(bool en
				 ,const QList<int> &actionsToSpare)
{
  //myDebug() << "MWBase::setEnableAction_but";
  
  iterateActions(menuBar(),en,actionsToSpare);

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
OGLView *MWBase::oglView()
{
  return pcontrol->oglView();
}


//EOF
