#ifndef MWBASE_H
#define MWBASE_H

#include <QMainWindow>
#include <QMenu>
#include "Activity.hh"
#include "Actions.hh"
#include "Master.hh"

class QMenuBar;
class QToolbar;

class Status;
class QSignalMapper;
class Master;
class OutWin;
class ControlBase;
class ToolBox;
class OGLView;


/* ***********************************************************************
 *
 * ***********************************************************************
 */
class MWBase : public QMainWindow
{
Q_OBJECT
public:
  MWBase(Master *m,const Activity &into,bool isfirst=false);
  ~MWBase();

  bool requireQuit();

  ToolBox * toolBox(){return ptoolbox;};
  void setToolBox(ToolBox *t){ptoolbox=t;};
  ControlBase *control(){return pcontrol;};
  OGLView *oglView();


  void enable_but(bool en,const QStringList &menusToSpare);
  void setEnableAction_but(bool en,const QList<int> &actionsToSpare);

  QToolBar *getNewToolbar();
  QAction *getNewMenuBarItem(Actions::ActionData ad);
  static QAction *addMenu(const QAction *parent ,const QString &text
			  ,const QString &tip,Actions::ActionData ad
			  ,QKeySequence seq=QKeySequence());
  static QAction *addMenu(const QAction *parent ,const QString &text
			  ,const QString &tip,Actions::ActionData ad
			  ,const QIcon &icon
			  ,QKeySequence seq=QKeySequence());
  QAction *getWindowMenuAction(){return windowMenu->menuAction();};
  QAction *getSettingsMenuAction(){return settingsMenu->menuAction();};
  void finish();

  inline bool isA(Activity::ActivityType at)const{ 
    return pactivity.isA(at); };

  MWBase *callActivityMenuEntry(const Activity &into);
    

private slots:
  void slotHelpAbout();
  void slotHelpAboutQt();
  void slotHelp();  
  void slotWindowStyleAction(QAction *);
  void onActivityMenu(int mode);
  void slotPreferences();
  


private:
  void morph(const Activity &into);
  void makeCmdMenu();


public:
  OutWin *outwin;

private:
  Activity pactivity;
  QList<QMenu *>pmenulist;
  QList<QToolBar *>ptoolbarlist;
  QMenu *cmdMenu;
  QMenu *windowMenu;
  QMenu *settingsMenu;
  QMenu *helpMenu;
  Master *master;
  ControlBase *pcontrol;
  Status *statusbar;
  QWidget *dummy;
  bool pinitialmenu;
  QSignalMapper *signalMapper;
  ToolBox *ptoolbox;

  
};

#endif // MWBASE_H
