#include <stdlib.h>
#include <QApplication>
#include "DMEdit.hh"
#include "DMDoc.hh"
#include "Cmplx.hh"
#include "SqMatrix.hh"
#include "MyDebug.hh"
#include "qt5compat.hh"


char lang[7] ="LANG=C";
char lcall[9]="LC_ALL=C";

int main(int argc, char *argv[])
{
  // hack to force english numeric settings and stuff
  putenv(lang);
  putenv(lcall);

  if(0){
    {
      Cmplx c1;
      myDebug() << Sprintf("(%g,%g)",c1.real(),c1.imag())();
    }
    {
      Cmplx c1(5.3);
      myDebug() << Sprintf("(%g,%g)",c1.real(),c1.imag())();
    }
    
    {
      Cmplx c1(15.87,-0.23);
      myDebug() << Sprintf("(%g,%g)",c1.real(),c1.imag())();
      Cmplx c2(c1);
      myDebug() << Sprintf("(%g,%g)",c2.real(),c2.imag())();
      Cmplx c3=c2;
      myDebug() << Sprintf("(%g,%g)",c3.real(),c3.imag())();
    }
    
    {
//       SqMatrix<double> rm;
//       rm.resize(2);
      myDebug() << "create cm";
      SqMatrix<Cmplx> cm;

      myDebug() << "resize cm";
      cm.resize(3);
      //       myDebug()<< rm.size();
      myDebug()<< "cm size "<<cm.size();

      myDebug() << "call makeYlmUMatrix and assign";
      cm=makeYlmUMatrix(2); 
      myDebug()<< "cm size "<<cm.size();
      cm.printSInfo("cm");
      
      printf("cm\n");
      for(int i=0;i<cm.size();i++){
	for(int j=0;j<cm.size();j++){
	  printf("(%10.5f,%10.5f) ", cm(i,j).real(), cm(i,j).imag());
	}
	printf("\n");
      }
      

      SqMatrix<Cmplx> im;
      im.resize(3);
      
      for(int i=0;i<im.size();i++){
	for(int j=0;j<im.size();j++){
	  //im(i,j)=i*10+j;
	  //im(i,j)=i*10.+j;
	  //im(i,j)=Cmplx(0,i+j/10.);
	  im(i,j)=Cmplx(i+j/10.);
	}
      }
      printf("im\n");
      printSqMatrix(im);

      SqMatrix<Cmplx> jm;
      jm.resize(3);
      
      for(int i=0;i<jm.size();i++){
	for(int j=0;j<jm.size();j++){
	  //jm(i,j)=i+j;
	  //jm(i,j)=(i+j)*1.;
	  jm(i,j)=Cmplx(0,(i+j)/10.);
	  //jm(i,j)=Cmplx((i+j)/10.);
	}
      }
      printf("jm\n");
      printSqMatrix(jm);

      SqMatrix<Cmplx> km=im*jm;

      printf("km\n");
      printSqMatrix(km.hermiteConjugated());

      Cmplx a(1,2);
      Cmplx b(3,4);
      Cmplx c=a*b;
      myDebug() << c.real() << c.imag();


      SqMatrix<Cmplx> m;
      m.resize(2);
      
      for(int i=0;i<m.size();i++){
	for(int j=0;j<m.size();j++){
	  m(i,j)=Cmplx(i+j*2+5,(i+j)/10.);
	  //m(i,j)=(i+j)*1.;
	  //m(i,j)=Cmplx(0,(i+j)/10.);
	  //m(i,j)=Cmplx((i+j)/10.);
	}
      }
      printf("m\n");
      printSqMatrix(m);
      printf("m^+\n");
      printSqMatrix(m.hermiteConjugated());

      
    }
    
    exit(0);
  }

  
  QApplication a(argc, argv);

  QStringList arg=a.arguments();
  int i=arg.size()-1;
  QString mode="dmat";
  while(i>=0){
    if(arg[i]=="-d"){
      mode="dmat";
    }else if(arg[i]=="-v"){
      mode="vmat";
    }else if(arg[i][0]=='-'){
      printf("unknown option %s\n",arg[i].toAscii().constData());
      exit(1);
    }
    i--;
  }

  DMDoc dmdoc(mode);
  if(!dmdoc.load()){
    exit(1);
  };
  DMEdit w;
  w.setData(&dmdoc);
  w.show();
  return a.exec();
}
