/*************************************************************************
 * @(#)OutWin.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   24 May 2009
 *************************************************************************
 */

#include <QList>
#include <QFile>
#include <QCheckBox>
#include <QTextStream>
#include <QFileDialog>
#include "OutWin.hh"
#include "Config.hh"


/* ***********************************************************************
 *
 * ***********************************************************************
 */

OutWin::OutWin(QWidget *parent)
  :QMainWindow(parent),fname("output.txt")
{
  setupUi(this);

  setWindowTitle(IDS_APP_NAME + " : " + "output");


  page->setReadOnly ( true ) ;

  connect(cbAll,SIGNAL(clicked()),this,SLOT(setControlEnabled()));
  connect(cbAll,SIGNAL(clicked()),this,SLOT(stateChanged()));

  connect(fontComboBox,SIGNAL(currentFontChanged(QFont))
	  ,this, SLOT(setCurrentFont(QFont)));

  connect(fileCloseAction,SIGNAL(triggered()),this,SLOT(hide()));
  connect(fileSaveAction,SIGNAL(triggered()),this,SLOT(save()));

  page->setFocus();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::setCurrentFont(QFont f)
{
  page->setFont(f);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::clear()
{
  //myDebug("clear\n");
  chlist.clear();
  list.clear();
  page->clear();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::setText(const QString &title ,const  QString &text)
{ 
  int ind;
  if( (ind=list.indexOf(Chapter(title))) >-1){
    //myDebug("reset text of %s",list[ind].title.toAscii().constData());
    list[ind].text=text;
  }else{
    list.append(Chapter(title,text));
    //myDebug("append text of %s",title.toAscii().constData());
  }
//   for(int i=0;i<list.size();i++){
//     myDebug("title[%d] = %s",i,list[i].title.toAscii().constData());
//   }
//  getchar();
  
  setupCheckBoxes();

  page->clear();
  for(int i=0;i<list.size();i++){
    if( visible(list[i].title) && page )
      page->append(list[i].text); 
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::setupCheckBoxes()
{
  chlist.clear();
  QObjectList l = gbxChapter->children( );
  for (int i = 0; i < l.size(); ++i) {
    QCheckBox *cb=qobject_cast<QCheckBox *>(l[i]);
    if(cb){
      QString s=cb->objectName();
      if( s.startsWith("cbChapter")) {
	//myDebug("%s:\n",cb->objectName().toAscii().constData());
	if(cb->isChecked())
	  chlist.append(cb->text());
	delete cb;
      }
    }
  }

  for(int i=0;i<list.size();i++){
    QCheckBox *cb = new QCheckBox(gbxChapter);
    cb->setObjectName(QString("cbChapter%1").arg(i));
    gbxLayout->insertWidget(i+2,cb);
    cb->setText(list[i].title);
    connect(cb,SIGNAL(clicked()),this,SLOT(stateChanged()));
    if(chlist.contains(list[i].title))
      cb->setChecked(true);
    if(cbAll->isChecked())
      cb->setEnabled(false);
    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::setControlEnabled()
{
  int e=!cbAll->isChecked();
  QObjectList l = gbxChapter->children( );
  for (int i = 0; i < l.size(); ++i) {
    QCheckBox *cb=qobject_cast<QCheckBox *>(l[i]);
    if(cb){
      QString s=cb->objectName();
      if( s.startsWith("cbChapter")) {
	cb->setEnabled(e);
      }
    }
  }
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::stateChanged()
{

  page->clear();
  for(int i=0;i<list.size();i++){
    if( visible(list[i].title) && page )
      page->append(list[i].text); 
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

bool OutWin::visible(const QString &title)
{
  if(cbAll->isChecked()) return true;
  QObjectList l = gbxChapter->children( );
  for (int i = 0; i < l.size(); ++i) {
    QCheckBox *cb=qobject_cast<QCheckBox *>(l[i]);
    if(cb){
      QString s=cb->objectName();
      if( s.startsWith("cbChapter")) {
	//myDebug("%s:\n",cb->objectName().toAscii().constData());
	if(cb->isChecked() && cb->text()==title)
	  return true;
      }
    }
  }

  return false;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void OutWin::save()
{

  QString fn = QFileDialog::getSaveFileName(this,"Save output"
					    ,QDir::currentPath() +'/'+fname
					    , " All files(*)"
					    ,0,0);
  
  
  if( fn.isEmpty()  ) return ;
  

  fname=fn;
  QFile file(fname);
  if (file.open(QFile::WriteOnly | QFile::Truncate)) {
    QTextStream out(&file);
 
    for(int i=0;i<list.size();i++){
      if( visible(list[i].title) && page )
	out << list[i].text ; 
    }
    //    out <<flush;
  }

}

/*EOF*/ 
