/* @(#)STR.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#include <QMessageBox>
#include <QToolBar>
#include <QToolTip>
#include <QThread>//todo: remove
#include "Worker.hh"//todo: remove
#include <QKeyEvent>
#include <QProcess>
#include <QMenu>
#include "MachineTime.hh"
#include <QFileDialog>
#include <QInputDialog>
#include <QFileInfo>
#include "MWBase.hh"
#include "Actions.hh"
#include "Assorted.hh"
#include "MyMath.hh"
#include "IconCache.hh"
#include "Settings.hh"
#include "STR.hh"
#include "Status.hh"
#include "HelpDlg.hh"
#include "Except.hh"
#include "DisplayCells.hh"
#include "Fog.hh"
#include "AtomPropsDlg.hh"
#include "Polyhedron.hh"
#include "Exception.hh"
#include "FogDlg.hh"
#include "Light.hh"
#include "LightDlg.hh"
#include "TestDlg.hh"
#include "CIFImportDlg.hh"
#include "UnfoldDlg.hh"
#include "Boundary.hh"
#include "DataExchange.hh"
#include "Representation.hh"
#include "Cluster.hh"
#include "RegularMesh.hh"
#include "STM.hh"
#include "TriangleMesh.hh"
#include "Symmetry.hh"
#include "OutWin.hh"
#include "Settings.hh"
#include "Axes.hh"
#include "AtomFindMarker.hh"
#include "MapAtomsDlg.hh"
#include "AnnotationDlg.hh"
#include "STToolBox.hh"
#include "PrntDlg.hh"
#include "constants.h"
#include "Propview.hh"
#include "BondsDlg.hh"
#include "FindAtomDlg.hh"
#include "BondDefs.hh"
#include "Config.hh"
#include "STDoc.hh"
#include "BoundDlg.hh"
#include "SymDlg.hh"
#include "AtomSequenceDlg.hh"
#include "Interrupt.hh"
//#include "ShowSymOpsDlg.hh"

#define workdir (doc->currentDir() +'/')


#define TODOBOX(text)				\
  QMessageBox::warning(this,"todo",text				\
		       ,QMessageBox::Ok|QMessageBox::Default	\
		       ,QMessageBox::NoButton);

#define sttoolbox ((STToolBox*)base()->toolBox())

/* ***********************************************************************
 *
 * ***********************************************************************
 */
STR::STR(MWBase *parent) 
  : ControlBase(parent),doc(0),propview(0),
//    showsymopsdlg(0),
    fogdlg(0),
    bounddlg(0),findatomdlg(0),symdlg(0),mapatomsdlg(0),bondsdlg(0),
    testdlg(0),cifimportdlg(0),unfolddlg(0),atompropsdlg(0),lightdlg(0)
{
  setupUi(this);

  doc=new STDoc(oglview);

  base()->setToolBox(new STToolBox(this,doc)) ;
  
  sttoolbox->setObjectName("sttoolbox");
  sttoolbox->setFloating(false);
  sttoolbox->setAllowedAreas(Qt::RightDockWidgetArea);
  base()->addDockWidget(Qt::RightDockWidgetArea, sttoolbox);



  setupMenus();

  MASSERT("",oglview);
  MASSERT("",doc);
  setupDocAndView(oglview,doc);

  oglview->setFog(doc->fog());
 
  {
    Settings set("application");
    oglview->
      setBadTransparency(
			 set.getBool("graphics/badTransparencyOnMove",true));
    
  }
  sizepolicy=QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  sizepolicy.setHorizontalStretch(6);
  sizepolicy.setVerticalStretch(1);

  if(oglview)
    sizepolicy.setHeightForWidth(oglview->sizePolicy().hasHeightForWidth());





  MASSERT("",base()->outwin);

  base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());

  Settings set("structure");
  sttoolbox->setVisible(set.getBool("maintoolbox/visible",true));


  sttoolbox->updateToolBox();  
  

  connect(statusbar,SIGNAL(canvasLockClicked(bool))
	  ,this,SLOT(slotCanvasLockClicked(bool)));

  statusbar->setCanvasControlEnabled(true);

  statusbar->setCanvasControl(doc->canvas());
  slotCanvasLockClicked(doc->canvas().locked());

  connect(oglview,SIGNAL(atomInfoToShow(String)),
 	  this,SLOT(onAtomInfoToShow(String)));

  connect(oglview,SIGNAL(atomClicked(long)),
 	  this,SLOT(onAtomClicked(long)));
  connect(oglview,SIGNAL(contextActionRequest(const ContextMenuContext&)),
 	  this,SLOT(onContextActionRequest(const ContextMenuContext&)));

  
  {
    Settings set("structure");
    doc->strfname=set.getString("printing/filename",doc->strfname);
  }
  oglview->setAllowContexts("recenter",true);
  oglview->setAllowContexts("hideatom",true);

  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::setupMenus()
{
  QAction *a;
  QMenu *m;

  QToolBar *mainToolBar = base()->getNewToolbar();
  mainToolBar->setObjectName("mainToolBar");
  mainToolBar->setContextMenuPolicy(Qt::NoContextMenu);
  mainToolBar->setWindowTitle("xfsf tool bar");
  base()->addToolBar(Qt::TopToolBarArea, mainToolBar);

  QAction *fileAction=base()->getNewMenuBarItem(Actions::File);
  fileAction->setText("&File");

  a=MWBase::addMenu(fileAction,"&Open","Load Settings"
		    ,Actions::File_Open
		    ,QIcon(":/Icons/fileopen")
		    ,QKeySequence(Qt::CTRL+Qt::Key_O));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileOpen()));
  mainToolBar->addAction(a); 

  a=MWBase::addMenu(fileAction,"&Save","Save Settings File"
		    ,Actions::File_Save
		    ,QIcon(":/Icons/filesave")
		    ,QKeySequence(Qt::CTRL+Qt::Key_S));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSave()));
  mainToolBar->addAction(a); 

  a=MWBase::addMenu(fileAction,"Save &As","Save as",Actions::File_SaveAs );
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileSaveAs()));


  fileAction->menu()->addSeparator();

  m = new QMenu(fileAction->menu());
  m->setTitle("&Import");
  a=MWBase::addMenu(m->menuAction(),"FPLO &input file =.in"
		    ,"import symmetry from the main FPLO input file '=.in'"
		    ,Actions::File_Import_In);
  connect(a, SIGNAL(triggered()), this, SLOT(slotFPLOInImport()));
  m->addAction(a);
  a=MWBase::addMenu(m->menuAction(),"FPLO &stm file =.stm"
		    ,"import STM info from '=.stm'"
		    ,Actions::File_Import_STM);
  connect(a, SIGNAL(triggered()), this, SLOT(slotSTMImport()));
  m->addAction(a);
  a=MWBase::addMenu(m->menuAction(),"&CIF file *.cif"
		    ,"import a CIF file"
		    ,Actions::File_Import_CIF);
  connect(a, SIGNAL(triggered()), this, SLOT(onCIFImportDlgTalkBack()));
  m->addAction(a);

  fileAction->menu()->addAction(m->menuAction());


  m = new QMenu(fileAction->menu());
  m->menuAction()->setData(Actions::data(Actions::File_Print));
  m->setTitle("&Print");
  a=MWBase::addMenu(m->menuAction(),"&Structure"
		    ,"Print Structure" 
		    ,Actions::File_Print_Structure
		    ,QIcon(":/Icons/print") 
		    ,QKeySequence(Qt::CTRL+Qt::Key_P));
  connect(a, SIGNAL(triggered()), this, SLOT(slotFilePrint()));
  m->addAction(a);
  mainToolBar->addAction(a);
  mainToolBar->addSeparator();
  fileAction->menu()->addAction(m->menuAction());


  QAction *closeAction=
    a=MWBase::addMenu(fileAction,"&Close","Close this activity"
		      ,Actions::File_Close
                    ,QIcon(":/Icons/exit")
		    ,QKeySequence(Qt::CTRL+Qt::Key_W));
  connect(a,SIGNAL(triggered()),this,SLOT(slotFileClose()));



  QAction *inputAction=base()->getNewMenuBarItem(Actions::Input);
  inputAction->setText("&Input");

  a=MWBase::addMenu(inputAction,"&Boundary","Define boundary"
		    ,Actions::Input_BoundaryDlg
                    ,QIcon(":/Icons/bound")
		    ,QKeySequence(Qt::CTRL+Qt::Key_B));
  connect(a,SIGNAL(triggered()),this,SLOT(onBoundDlgTalkBack()));
  mainToolBar->addAction(a);


  a=MWBase::addMenu(inputAction,"&Symmetry"
		    ,"Symmetry settings"
		    ,Actions::Input_SymmetryDlg
		    ,QIcon(":/Icons/sym")
		    ,QKeySequence(Qt::CTRL+Qt::Key_Y));
  connect(a,SIGNAL(triggered()),this,SLOT(onSymDlgTalkBack()));
  mainToolBar->addAction(a);
  mainToolBar->addSeparator();

  QAction *plotAction=base()->getNewMenuBarItem(Actions::Plot);
  plotAction->setText("&Plot");

  calcAction=
    a=MWBase::addMenu(plotAction,"&Calculate","Calculate scene"
		      ,Actions::Plot_Calculate
		      ,QIcon(":/Icons/xfsf")
		      ,QKeySequence(Qt::CTRL+Qt::Key_C));
  connect(a,SIGNAL(triggered()),this,SLOT(updateData()));
  sttoolbox->tbCalc->setDefaultAction(a);
  
  QAction *recalcAction=
  a=MWBase::addMenu(plotAction,"&Recalculate","Force re-calculation"
		    ,Actions::Plot_Recalculate
		    ,QIcon(":/Icons/recalc"));
  connect(a,SIGNAL(triggered()),this,SLOT(slotPlotReCalc()));
  sttoolbox->tbReCalc->setDefaultAction(a);


  a=MWBase::addMenu(plotAction,"Li&ghts","Light settings"
		    ,Actions::Plot_Lighting);
  connect(a,SIGNAL(triggered()),this,SLOT(onLightDlgTalkBack()));

  a=MWBase::addMenu(plotAction,"&Fog","Fog  settings"
		    ,Actions::Plot_Fog);
  connect(a,SIGNAL(triggered()),this,SLOT(onFogDlgTalkBack()));




  a=MWBase::addMenu(plotAction,"&Bonds/Atoms"
		    ,"Bond Settings",Actions::Plot_Bonds
		    ,QKeySequence(Qt::CTRL+Qt::Key_D));
  connect(a,SIGNAL(triggered()),this,SLOT(onBondsDlgTalkBack()));


  a=MWBase::addMenu(plotAction,"Atom &properties"
		    ,"Atom properties (colors ...)",Actions::Plot_AtomProps
		    );
  connect(a,SIGNAL(triggered()),this,SLOT(onAtomPropsDlgTalkBack()));


  a=MWBase::addMenu(plotAction,"&Annotations" ,"Annotation Settings" 
		    ,Actions::Plot_Annotations
		    ,QKeySequence(Qt::CTRL+Qt::Key_A));
  a->setData(AnnotationDlg::AMAxes);
  connect(a,SIGNAL(triggered()),this,SLOT(onAnnotationDlgTalkBack()));

  a=MWBase::addMenu(plotAction,"&Display cells" ,"Display cell settings" 
		    ,Actions::Plot_DisplayCells);
  a->setData(AnnotationDlg::AMDisplayCells);
  connect(a,SIGNAL(triggered()),this,SLOT(onAnnotationDlgTalkBack()));

  canvasAction
    =a=MWBase::addMenu(plotAction,"Can&vas","Canvas Settings"
		       ,Actions::Plot_Canvas
		       ,QKeySequence(Qt::CTRL+Qt::Key_V));
  connect(a,SIGNAL(triggered()),this,SLOT(slotCanvasDefine()));

  // a=MWBase::addMenu(plotAction,"&Test","Test  settings"
  // 		    ,Actions::Plot_Test);
  // connect(a,SIGNAL(triggered()),this,SLOT(onTestDlgTalkBack()));


  toolAction=base()->getNewMenuBarItem(Actions::Tools);
  toolAction->setText("&Tools");
  atomPickAction
    =a=MWBase::addMenu(toolAction,"&Measure","Measure atom distances/anlges"
		       ,Actions::Tools_Measure
		       ,QKeySequence(Qt::Key_M));
  a->setCheckable(true);
  connect(a,SIGNAL(triggered()),this,SLOT(onAtomPickActionTriggered()));
  
  a=MWBase::addMenu(toolAction,"&Atom sequence (Alt+S)"
		    ,"Show a sequence of atom postiions"
		    ,Actions::Tools_SiteHistory
		    ,QKeySequence(Qt::ALT + Qt::Key_S)
		    );
  connect(a,SIGNAL(triggered()),this,SLOT(onAtomSequence()));

  a=MWBase::addMenu(toolAction,"&Find atom (Ctrl+F)"
		    ,"Find an atom"
		    ,Actions::Tools_FindAtom
		    ,QKeySequence(Qt::CTRL + Qt::Key_F)
		    );
  connect(a,SIGNAL(triggered()),this,SLOT(onFindAtomDlgTalkBack()));

  a=MWBase::addMenu(toolAction,"&Density Mapper"
		    ,"Map density files"
		    ,Actions::Tools_MapDensity
		    );
  connect(a,SIGNAL(triggered()),this,SLOT(onMapAtomsDlgTalkBack()));


  a=MWBase::addMenu(toolAction,"&Unfold","Unfolding editor"
		    ,Actions::Tools_UnfoldingEditor);
  connect(a,SIGNAL(triggered()),this,SLOT(onUnfoldDlgTalkBack()));

  // a hidden action
  a=atomPickResetAction=new QAction("Reset Measuring",this);
  a->setShortcut(QKeySequence(Qt::Key_Escape));
  connect(a,SIGNAL(triggered()),this,SLOT(onAtomPickResetActionTriggered()));
  this->addAction(atomPickResetAction);

  // a=MWBase::addMenu(toolAction,"Show &Symm-ops"
  // 		    ,"Display symmetry operations"
  // 		    ,Actions::Tools_ShowSymOps
  // 		    );
  // a->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_X,Qt::CTRL+Qt::Key_S));
  // connect(a,SIGNAL(triggered()),this,SLOT(onShowSymOpsDlgTalkBack()));


  QAction *runAction=base()->getNewMenuBarItem(Actions::Run);
  runAction->setText("&Run");
//   a=MWBase::addMenu(runAction,"Run &Fplo","Run interactive FPLO session");
//   connect(a,SIGNAL(triggered()),this,SLOT(slotRunFPLO()));

  a=MWBase::addMenu(runAction,"&Terminal","Run an xterm in the current directory (On MAC xfplo must be started in X11/Terminal for this to work)"
		    ,Actions::Run_Terminal);
  connect(a,SIGNAL(triggered()),this,SLOT(slotRunTerminal()));


  QAction *winAction=base()->getWindowMenuAction();

  showOutAction
    =a=MWBase::addMenu(winAction,"&Output","Show output window"
		       ,Actions::Window_Output);
  a->setCheckable(true);
  connect(a,SIGNAL(triggered(bool)),this,SLOT(slotShowOutwin(bool)));

  connect(winAction->menu(),SIGNAL(aboutToShow())
	  ,this,SLOT(winMenu_aboutToShow()));

  connect(toolAction->menu(),SIGNAL(aboutToShow())
	  ,this,SLOT(toolMenu_aboutToShow()));

  m = new QMenu(winAction->menu());
  m->setTitle("&Toolbars");
  winAction->menu()->addAction(m->menuAction());

  a=mainToolBar->toggleViewAction();
  a->setText("&Tool bar");
  m->addAction(a);
  a=sttoolbox->toggleViewAction();
  a->setText("Main &tool box");
  m->addAction(a);

  mainToolBar->addAction(calcAction);
  mainToolBar->addAction(recalcAction);
  mainToolBar->addSeparator();

  mainToolBar->addAction(closeAction);
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFileOpen()
{
  //x  statusbar->setMessage("Opening file...");
  QString f0=doc->absoluteFilePath();
  if(doc->isNew()){
    f0=workdir+"=.xstr";
  }

  QString fileName = QFileDialog::getOpenFileName(this,"Open File",f0
						  ,"FPLO input (=.*);;All files (*)");

  if (!fileName.isEmpty()) 
    fileLoad(fileName);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool STR::fileLoad(const QString &fileName)
{

  QString ft=Master::fileType(fileName,true);
  if(ft!="xstr"){
    QMessageBox::warning(this,"Open","File type is no "
			 "structure file",QMessageBox::Yes);
    return false;
  }

  //  myDebug() << fileName ;
  doc->saveCurrentDir();
  QFileInfo fi(fileName);
  bool isfile=fi.isFile();
  doc->restoreCurrentDir();
  if(!isfile){
    myDebug() << "file " << fileName << "is not a file";
    return false;
  }




  if (!fileName.isEmpty()) {
    
    try{
      doc->load(fileName);
    }catch(QString msg){
      QMessageBox::warning(this,"Open",msg,QMessageBox::Yes);

      statusbar->setDocument(workdir);
      return false;
    }

    doc->setFileName(fileName);
    doc->setCurrentDir(doc->absolutePath());

    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());

    QString message="Loaded document: "+fileName;
    
    statusbar->setDocument(doc->absoluteFilePath());

    onFogDlgTalkBack(ProtoDlg::Refresh);
    onBondsDlgTalkBack(ProtoDlg::Refresh);
    onLightDlgTalkBack(ProtoDlg::Refresh);
    onTestDlgTalkBack(ProtoDlg::Refresh);
    onCIFImportDlgTalkBack(ProtoDlg::Refresh);
    onUnfoldDlgTalkBack(ProtoDlg::Refresh);
    onAtomPropsDlgTalkBack(ProtoDlg::Refresh);
    //    onShowSymOpsDlgTalkBack(ProtoDlg::Refresh);
    onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    onBoundDlgTalkBack(ProtoDlg::Refresh);
    onSymDlgTalkBack(ProtoDlg::Refresh);
    onFindAtomDlgTalkBack(ProtoDlg::Refresh);
    //onMapAtomsDlgTalkBack(ProtoDlg::Refresh);
    updateSizes(false);
    sttoolbox->updateToolBox();  
    base()->outwin->clear();  

//     myDebug() << "todo remove default bonds on load";
//     if(calcSymmetry()){
//       doc->bonddefs->setDefault(doc->symmetry);
//       doc->cluster->setCalculated(false);
//     }

    updateData();

  }
  //inputMenu_aboutToShow();

  if(dataExchange.listensFor("structure loaded")){
    dataExchange.sendData(this,"structure loaded",0);
  }

  return true;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFileSave() 
{ 
  saveit(FILESAVE); 
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFileSaveAs()
{
  saveit(FILESAVEAS); 
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::saveit( SaveMode saveas )
{
  int doit;
  QString fn;

  if(saveas != FILESAVE && saveas != FILESAVEAS  )return;
  if(saveas == FILESAVEAS || doc->isNew() ){
    QString f0=doc->absoluteFilePath();
    if(doc->isNew()){
      f0=workdir+"=.xstr";
    }


#ifdef Q_OS_MAC
    fn = QFileDialog::getSaveFileName(this,"Save File",f0);
#else
    fn = QFileDialog::getSaveFileName(this,"Save File",f0
				      , " FPLO files(=.*) ;; All files(*)"
				      ,0,0);
#endif

    if ( (doit=!fn.isEmpty()) ) {
      //todo   doit=overWriteFile(fn);
    }
  }else{
    fn=doc->absoluteFilePath();
    doit=1;
  }
  if ( doit ) {
    try{
      doc->save(fn);
    }catch(QString msg){
      QMessageBox::warning(this,"Save",msg,QMessageBox::Yes);
      return;
    }
    
    doc->setFileName(fn);
    doc->setCurrentDir(doc->absolutePath());
    doc->setOld();
    base()->setWindowTitle(IDS_APP_NAME + " : " +doc->fileName());
    statusbar->setDocument(doc->absoluteFilePath());
    statusbar->setMessage(QString("Saved %1")
			  .arg(doc->absoluteFilePath()),1000);
  }

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::afterShow(const QString &loadp)
{
  //myDebug() << "STR::afterShow" <<loadp;

  //doc->canvas().print();

  
  
  if(!loadp.isEmpty()){
    QFileInfo fi=QFileInfo(loadp);
    QString ft=Master::fileType(loadp,true);
    if(ft=="directory"){
      doc->setCurrentDir(fi.absoluteFilePath());
    }else{
      doc->setCurrentDir(fi.absolutePath());
      if(ft=="in" || ft=="sym"){
	//myDebug() << "STR::afterShow" <<ft;
	FPLOInImport(workdir+fi.fileName());
	updateSizes(true);
	return;
      }else if(ft=="cif"){
	onCIFImportDlgTalkBack(ProtoDlg::Create,loadp);
      }else if(ft=="xstr"){
	if(fileLoad(workdir+fi.fileName())){
	  return;
	}
      }
    }
    
  }


  updateSizes(true);
  if(calcSymmetry()){
    doc->bonddefs->setDefaultFromSymmetry(doc->symmetry);
    doc->cluster->setCalculated(false);
  }
  updateData(true);
   
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::winMenu_aboutToShow()
{
  showOutAction->setChecked(base()->outwin->isVisible());
}


void STR::toolMenu_aboutToShow()
{

  if(propview)
    atomPickAction->setChecked(propview->isVisible());
  else
    atomPickAction->setChecked(false);
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomClicked(long atindex)
{
  NSF("STR::onAtomClicked(long atindex)");
  //myDebug() << nsubfunc;

  if(atindex<0) return;
  //  printf("atomindex=%ld\n",atindex);
  
  const Atom &at=doc->cluster->atom(atindex);
  
  
  if(dataExchange.listensFor("atom data")){
    // This is for communication between different 
    // mwbase-derived main-windows.
    // Eample: strA opens mapatomsdlg and user clickes on strB
    // to fill in the data in the mapatomsdlg. Then mapatomsdlg 
    // was opened andis known by strA, but strB is an unknown
    // entity. dataExchange enables communication.
    dataExchange.sendData(this,"atom data",&at);
  }
  
  
  if(!bondsdlg.isNull()){
    oglview->setMoveBlockingClick(true);
    bondsdlg->activateWindow();
    bondsdlg->raise();
    bondsdlg->selectSite(at.site);
  }

  if(!symdlg.isNull()){
    oglview->setMoveBlockingClick(true);
    //symdlg->activateWindow();
    //symdlg->raise();
    symdlg->selectWyck(at.sort);
  }

  if(!atompropsdlg.isNull()){
    oglview->setMoveBlockingClick(true);
    atompropsdlg->selectElement(at.el);
  }

  if(!findatomdlg.isNull()){
    oglview->setMoveBlockingClick(true);
    findatomdlg->updateData(at);
  }

  if(!mapatomsdlg.isNull()){
    //myDebug() << "site"<<at.site;
    mapatomsdlg->selectSite(at.site);
  }

}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomInfoToShow(String info)
{

  //myDebug()<<info;
  if(info.startsWith("atom")){
    long atindex=QString(info()).remove("atom ").toInt();
    if(atindex<0) {statusbar->deleteMessage(); return;}
    const Atom &at=doc->cluster->atom(atindex);

    String pref;
    double unit=1;
    Xf::LengthUnits slu=statusbar->preferedLengthUnit();
    if(slu==Xf::BohrRadius){
      pref="aB";
      if(at.lengthunit==Xf::Angstroem) unit=1/constant_aBtoAng;
    }else{
      pref="Ang";
      if(at.lengthunit==Xf::BohrRadius) unit=constant_aBtoAng;
    }


    QString ss=QString()
      .sprintf("%ld: %s(S%ld,W%ld,T%ld) (%s) %s",at.index+1
 	       ,at.el() ,at.site+1 ,at.sort+1,at.type
 	       ,(at.tauD*unit).printS("%- 8g",",")(),pref());
    statusbar->setMessage(ss);
  }else if(info.startsWith("stm atom")){
    //long atindex=info.remove("stm atom ").toInt();

    String pref;
    double unit=1;
    Xf::LengthUnits slu=statusbar->preferedLengthUnit();
    if(slu==Xf::BohrRadius){
      pref="aB";
      if(doc->stm->lengthUnit()==Xf::Angstroem) unit=1/constant_aBtoAng;
    }else{
      pref="Ang";
      if(doc->stm->lengthUnit()==Xf::BohrRadius) unit=constant_aBtoAng;
    }


    QString ss=QString()
      .sprintf("STM atom %s (%s) %s"
 	       ,doc->stm->element()() 
 	       ,(doc->stm->tauD()*unit).printS("%- 8g",",")(),pref());
    statusbar->setMessage(ss);
  }else{
    statusbar->deleteMessage();
  }
  return;
  

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomPickActionTriggered()
{
  //  myDebug() << "STR::onAtomPickActionTriggered";

  
  if(!propview){
    propview =new PropView(0,doc,oglview);
    propview->setObjectName("propview");
    propview->installEventFilter(this);

  }

 
 
 
  propview->toggleVisiblity();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomPickResetActionTriggered()
{
  //myDebug() << "STR::onAtomPickResetActionTriggered";
  if(propview){
    propview->fullReset();
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onContextActionRequest(const ContextMenuContext&comuc)
{
  //myDebug()<<comuc.menuTitle();
  if(comuc.type()==ContextMenuContext::AtomDelete){
    long atomindex=comuc.atomIndex();
    //myDebug()<<atomindex;
    if(atomindex<0) return;
    const Atom &at=doc->cluster->atom(atomindex);
    //myDebug()<<at.sort;
    if(!symdlg.isNull()){
      symdlg->deleteWyckoffPosition(at.sort);
    }
  }else if(comuc.type()==ContextMenuContext::HideAtom){
    long atomindex=comuc.atomIndex();
    if(atomindex<0) return;
    const Atom &at=doc->cluster->atom(atomindex);
    if(!symdlg.isNull()){
      symdlg->hideWyckoffPosition(at.sort);
    }else{
      if(Settings("structure").getBool("Atoms/ShowHidden",false)){
	QToolTip::showText(oglview->mapToGlobal(QPoint(0,0)),
			   "Currently the \"show hidden atoms\""
			   "\nbutton is activeated.",
			   oglview);
      }

      doc->symmetry->wyckoffPosition(at.sort).setVisible(false);
      doc->symmetry->setCalculated(false);
      updateData(false);

    }
  }else if(comuc.type()==ContextMenuContext::SetViewCenter){
    //myDebug()<<comuc.currentPoint();
    oglview->resetPanZoom(OGLView::UserFocus,true,comuc.currentPoint());
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotShowOutwin(bool onoff)
{
  if( onoff ){
    base()->outwin->show();
    base()->outwin->raise();
  }else{
    base()->outwin->hide();
  }
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
STR::~STR()
{
  oglview->mute(true);
  myDebug() << "STR:: here I die!\n";
  
  if(atomPickAction) delete atomPickAction;atomPickAction=0;
  if(propview) delete propview;propview=0;
  if(doc) delete doc;doc=0;

  //  if(!showsymopsdlg.isNull()) delete showsymopsdlg;showsymopsdlg=0;
  if(!fogdlg.isNull()) delete fogdlg;fogdlg=0;
  if(!bounddlg.isNull()) delete bounddlg;bounddlg=0;
  if(!findatomdlg.isNull()) delete findatomdlg;findatomdlg=0;
  if(!symdlg.isNull()) delete symdlg;symdlg=0;
  if(!mapatomsdlg.isNull()) delete mapatomsdlg; mapatomsdlg=0;
  if(!bondsdlg.isNull()) delete bondsdlg;bondsdlg=0;
  if(!lightdlg.isNull()) delete lightdlg;lightdlg=0;
  if(!testdlg.isNull()) delete testdlg;testdlg=0;
  if(!cifimportdlg.isNull()) delete cifimportdlg;cifimportdlg=0;
  if(!unfolddlg.isNull()) delete unfolddlg;unfolddlg=0;
  if(!atompropsdlg.isNull()) delete atompropsdlg;atompropsdlg=0;

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::help()
{
  HelpDlg *h=new HelpDlg(this,this,"HelpDlg","Index");
  h->show();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool STR::isModified()
{
  MASSERT("doc not created ",doc);
  //myDebug() <<"STR::isModified";
  return doc->isModified();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFileClose()
{
  //myDebug() <<"STR::slotFileClose";
  if(queryExit()){
    
    base()->finish();
    //myDebug() <<" STR::slotFileClose after base()->finish";
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool STR::queryExit()
{
  if(!mapatomsdlg.isNull()){
    if(!mapatomsdlg->dataAcceptable()) return false;
  }

  {
    Settings set("structure");
    set.set("maintoolbox/visible",sttoolbox->isVisible());
    set.set("canvas/width",doc->canvas().width());
    set.set("canvas/height",doc->canvas().height());
  }
  return ControlBase::queryExit();
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::FPLOInImport(const QString &fileName)
{
  QString ft=Master::fileType(fileName,true);
  if(ft!="in" && ft!= "sym"){
    QMessageBox::warning(this,"Open","File type is no usefull input file"
			 ,QMessageBox::Yes);
    return ;
  }

  if(fileName.isEmpty()) return;
  
  QFileInfo fi(fileName);
  //  doc->setCurrentDir(fi.absolutePath());

  try{
    doc->symmetry->readSymInfo(fileName);
  }catch(Except &ex){
    QMessageBox::warning(this,"Loading symmetry", ex.msg()
			 ,QMessageBox::Ok | QMessageBox::Default);
    return ;
  }
  if(calcSymmetry()){
    doc->bonddefs->setDefaultFromSymmetry(doc->symmetry);
  }

  
  updateData(true);



  statusbar->setDocument(doc->absoluteFilePath());
  //myDebug() << " symchanged=" <<doc->symmetry->changed();
  
  if(dataExchange.listensFor("structure loaded")){
    dataExchange.sendData(this,"structure loaded",0);
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::STMImport(const QString &fileName)
{

  if(fileName.isEmpty()) return;
  
  QFileInfo fi(fileName);
  //  doc->setCurrentDir(fi.absolutePath());

  try{
    doc->stm->readSTMFile(fileName);
  }catch(Exception &ex){
    QMessageBox::warning(this,"Loading STM file", ex.msg()()
			 ,QMessageBox::Ok | QMessageBox::Default);
    return ;
  }
  
  
  updateData(false);



  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::updateSizes(bool fresh)
{
  int w,h;
  bool canvaslocked=doc->canvas().locked();
  if(fresh){
    Settings set("structure");
    w=set.getInt("canvas/width",400);
    h=set.getInt("canvas/height",400);
    if(w<10)w=400;
    if(h<10)h=400;
    doc->canvas().setWidth(w);
    doc->canvas().setHeight(h);
  }
  slotCanvasLockClicked(true);
  oglSizeChanged(oglview);
  statusbar->setCanvasControl(doc->canvas()); 
  if(fresh)
    oglview->resize(w,h); 
  updateGeometry();
  if(fresh){
    qApp->processEvents();
  }else{
    //    updateData();
  }
  slotCanvasLockClicked(canvaslocked);
  statusbar->setCanvasControl(doc->canvas());  
  oglSizeChanged(oglview);

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFilePrint()
{

  
  QString filename=doc->strfname;
  
  PrntDlg *pdlg=new PrntDlg(this,"PrtDlg");
  pdlg->setData(doc,filename);
  pdlg->setWindowTitle(IDS_APP_NAME + " : Print Structure" );
  long rett=pdlg->exec();
  if(rett){  	
    pdlg->getData(doc,filename);
  }
  delete (pdlg);
  if(!rett) return;



  long doit;
  if ( (doit=!filename.isEmpty()) ) {
    doit=overWriteFile(filename); 
  }
  if(!doit) return;


  if(!QFileInfo(filename).dir().exists()){
    QMessageBox::warning(this,"Save image", 
			 "The file "+doc->strfname+
			 " cannot be saved (directory invalid)??",
			 QMessageBox::Yes);
      return;
  }
  
  doc->strfname=filename;



  qApp->processEvents();
  qApp->flush();
  

  
  //MBox *mbox=new MBox(this);

  //mbox->show();
  //qApp->processEvents();
  //qApp->flush();
  statusbar->setMessage(QString("Printing to \'")+doc->strfname+"\' ...");

  doc->saveCurrentDir();
  QFileInfo fi(doc->strfname);
  QString ap=fi.absoluteFilePath();
  doc->restoreCurrentDir();

//   QMessageBox::information(this,"Check", doc->strfname+"\n"+ap
// 			   ,QMessageBox::Ok);

  Settings set("structure");
  set.set("printing/filename",fi.fileName());

  qApp->processEvents();
  qApp->flush();

  oglview->writePNG(ap,doc->magfac,doc->picquality,statusbar);

  //delete mbox;
  statusbar->deleteMessage();

  


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotFPLOInImport()
{

  QString f0=workdir+"=.in";
  
  QString fileName = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"FPLO input (=.*);;All files (*)");

  if (!fileName.isEmpty()) {
    FPLOInImport(fileName);
  
    sttoolbox->updateAxesButtons();
    sttoolbox->updateCellButtons();
    onSymDlgTalkBack(ProtoDlg::Refresh);
    onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    onBondsDlgTalkBack(ProtoDlg::Refresh);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotSTMImport()
{

  QString f0=workdir+"=.stm";
  
  QString fileName = 
    QFileDialog::getOpenFileName(this,"Open File",f0
				 ,"FPLO input (=.*);;All files (*)");

  if (!fileName.isEmpty()) {
    STMImport(fileName);
  
  }
  
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAnnotationDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!annotationdlg.isNull()) {
      annotationdlg->activateWindow();
      annotationdlg->raise();
      return;
    }
    
    AnnotationDlg::AnnotationMode mode=AnnotationDlg::AMAxes;
    
    QAction *a=dynamic_cast<QAction*>(sender());
    if(a){
      mode=(AnnotationDlg::AnnotationMode)a->data().toInt();
    }
    
    annotationdlg=new AnnotationDlg(0,base(),"AnnotationDlg");
    connect(annotationdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onAnnotationDlgTalkBack(int)));
    

    annotationdlg->setData(mode,doc,doc->symmetry
			   ,doc->symmetry->dataStructureType()
			   ,Xf::LTDirect);
    annotationdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(annotationdlg.isNull()) return;
    if(!calcSymmetry()) return;
    annotationdlg->refresh(doc->symmetry->dataStructureType(),Xf::LTDirect);
    annotationdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(annotationdlg.isNull()) return;

    long ch=0;
    annotationdlg->getData(doc,ch);

    sttoolbox->updateAxesButtons();
    sttoolbox->updateCellButtons();
    if(ch) {doc->setModified();doc->allcalced=false;}

    if(!doc->allcalced){
      updateData();
    }else{
      for(int i=0;i<doc->displayCells()->size();i++){
	DisplayCell *dc=doc->displayCells()->cell(i);
	if(
	   dc->type()!=Xf::DCTMainBoundary&&
	   dc->type()!=Xf::DCTConventional&&
	   dc->type()!=Xf::DCTPrimitive
	   ){
	  oglview->setVisible(dc->id(),dc->boundary()->polyhedron()
			      ->properties().isVisible());
	}
      }
      oglview->doRendering();
      
    }
  
    
  }
}







/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomPropsDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!atompropsdlg.isNull()) {
      atompropsdlg->activateWindow();
      atompropsdlg->raise();
      return;
    }
  
    atompropsdlg=new AtomPropsDlg(0,base(),"AtomPropsDlg");
    connect(atompropsdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onAtomPropsDlgTalkBack(int)));
    

    atompropsdlg->setData(doc,globalproperties);
    atompropsdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(atompropsdlg.isNull()) return;
    atompropsdlg->setData(doc,globalproperties);
    atompropsdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(atompropsdlg.isNull()) return;

    atompropsdlg->getData(doc,globalproperties);

    if(doc->symmetry->changed()) {
      if(!symdlg.isNull()){
	symdlg->updateRequest("atomproperties");
      }
      doc->setModified();
      doc->allcalced=false;
    }
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}










/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onFogDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!fogdlg.isNull()) {
      fogdlg->activateWindow();
      fogdlg->raise();
      return;
    }
  
    fogdlg=new FogDlg(0,base(),"FogDlg");
    connect(fogdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onFogDlgTalkBack(int)));
    

    fogdlg->setData(doc->fog());
    fogdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(fogdlg.isNull()) return;
    fogdlg->setData(doc->fog());
    fogdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(fogdlg.isNull()) return;

    fogdlg->getData(doc->fog());
    if(doc->fog()->changed()) {
      doc->setModified();
      doc->allcalced=false;
    }
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onBoundDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){

    if(!calcSymmetry()) return;

    if(!bounddlg.isNull()) {
      bounddlg->activateWindow();
      bounddlg->raise();
      return;
    }
  
    bounddlg=new BoundDlg(0,base(),"BoundDlg");
    connect(bounddlg,SIGNAL(talkBack(int)),
	    this,SLOT(onBoundDlgTalkBack(int)));
    

    bounddlg->setData(doc->bound(),doc->symmetry);
    bounddlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(bounddlg.isNull()) return;
    if(!calcSymmetry()) return;
    bounddlg->setData(doc->bound(),doc->symmetry);
    bounddlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(bounddlg.isNull()) return;

    bool ch=false;
    bounddlg->getData(doc->bound(),ch);
    if(ch) {
      doc->setModified();
      doc->allcalced=false;
    }

    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onFindAtomDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){

    //TODOBOX("figure out what needs to be calculated before findatom")

    if(!calcSymmetry()) return;

    if(!findatomdlg.isNull()) {
      findatomdlg->activateWindow();
      findatomdlg->raise();
      return;
    }
    //if(!findatomdlg.isNull()) findatomdlg->setFocus();

    findatomdlg=new FindAtomDlg(0,base(),"FindAtomDlg",oglview->atomFindMarker());
    connect(findatomdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onFindAtomDlgTalkBack(int)));
    
    if(!mapatomsdlg.isNull())
      findatomdlg->setEnabled(false);
    else
      oglview->atomFindMarker()->clear();

  
    findatomdlg->setData();
    findatomdlg->show();
    
    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(findatomdlg.isNull()) return;
    findatomdlg->setData();
    findatomdlg->update();
 }else if(mode==ProtoDlg::End){
    if(mapatomsdlg.isNull())
      oglview->atomFindMarker()->clear();
 }else if(mode==ProtoDlg::Accept){
    
    if(findatomdlg.isNull()) return;

    long number=0;
    Xf::FindAtomModes mode=Xf::FindSite;
    findatomdlg->getData(mode,number);


    if(oglview->atomFindMarker()->mark(doc->cluster,mode,number))
      findatomdlg->setMessage("");
    else
      findatomdlg->setMessage("Not found");

    
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onSymDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!symdlg.isNull()) {
      symdlg->activateWindow();
      symdlg->raise();
      return;
    }
    
    symdlg=new SymDlg(0,base(),"STRSymDlg");
    connect(symdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onSymDlgTalkBack(int)));
    
    
    symdlg->setData(doc->symmetry,SymDlg::StructureMode);
    symdlg->show();
    oglview->setAllowContexts("deletewyck",true);
    
    return ;
  }else if(mode==ProtoDlg::Refresh){
    if(symdlg.isNull()) return;
    symdlg->setData(doc->symmetry,SymDlg::StructureMode);
    symdlg->update();
  }else if(mode==ProtoDlg::End){
    oglview->setAllowContexts("deletewyck",false);
  }else if(mode==ProtoDlg::Accept){
    
    if(symdlg.isNull()) return;
    
    bool ch=false;
    symdlg->getData(doc->symmetry,ch);

    if(ch||!doc->symmetry->calculated()) {
      doc->setModified();
      doc->allcalced=false;
      //refreshBoundDlg();
      if(!doc->allcalced){
	//updateData(true);
	updateData();
      }
      onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    }

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onMapAtomsDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!mapatomsdlg.isNull()) {
      mapatomsdlg->activateWindow();
      mapatomsdlg->raise();
      return;
    }
    
    mapatomsdlg=new MapAtomsDlg(0,base(),"MapAtomsDlg");
    connect(mapatomsdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onMapAtomsDlgTalkBack(int)));
    
    if(!findatomdlg.isNull())
      findatomdlg->setEnabled(false);
    mapatomsdlg->setData(oglview->atomFindMarker()->properties());
    
    
    QList<int> spare;
    spare
      <<Actions::Activities
      <<Actions::File_Save
      <<Actions::File_SaveAs
      <<Actions::File_Print
      <<Actions::File_Close
      <<Actions::Plot
      <<Actions::Input_BoundaryDlg
      <<Actions::Tools_Measure
      <<Actions::Tools_FindAtom
      <<Actions::Run
      <<Actions::Window
      <<Actions::Help
      ;
    base()->setEnableAction_but(false,spare);
    mapatomsdlg->setEnabled(true);
    
    
    mapatomsdlg->show();
    
    return ;
  }else if(mode==ProtoDlg::Refresh){
  }else if(mode==ProtoDlg::End){
    if(mapatomsdlg.isNull()) return;
    
    base()->setEnableAction_but(true,QList<int>());
    if(!findatomdlg.isNull())
      findatomdlg->setEnabled(true);
    
    
    
  }else if(mode==ProtoDlg::Accept){
    
  }
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onBondsDlgTalkBack(int mode)
{

  if(mode==ProtoDlg::Create){
    if(!calcSymmetry()) return;
    if(!bondsdlg.isNull()) {
      bondsdlg->activateWindow();
      bondsdlg->raise();
      return;
    }
  
    bondsdlg=new BondsDlg(0,base(),"BondsDlg");
    connect(bondsdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onBondsDlgTalkBack(int)));
    

    bondsdlg->setData(doc);
    bondsdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(bondsdlg.isNull()) return;
    if(!calcSymmetry()) return;
    bondsdlg->setData(doc);
    bondsdlg->update();
 }else if(mode==ProtoDlg::Accept){
    if(bondsdlg.isNull()) return;
    if(!calcSymmetry()) return;
    long ch=0;
    bondsdlg->getData(doc,ch);

    if(ch) {doc->setModified();doc->allcalced=false;}

    if(!doc->allcalced){
      doc->cluster->setCalculated(false);
      updateData();
    }
  }
}








/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onTestDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!testdlg.isNull()) {
      testdlg->activateWindow();
      testdlg->raise();
      return;
    }
  
    testdlg=new TestDlg(0,base(),"TestDlg");
    connect(testdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onTestDlgTalkBack(int)));
    

    testdlg->setData(doc);
    testdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(testdlg.isNull()) return;
    testdlg->setData(doc);
    testdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(testdlg.isNull()) return;

    testdlg->getData(doc);
    // if(doc->test()->changed()) {
    doc->setModified();
    doc->allcalced=false;
    //}
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onLightDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!lightdlg.isNull()) {
      lightdlg->activateWindow();
      lightdlg->raise();
      return;
    }
  
    lightdlg=new LightDlg(0,base(),"LightDlg");
    connect(lightdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onLightDlgTalkBack(int)));
    

    lightdlg->setData(doc,oglview);
    lightdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(lightdlg.isNull()) return;
    lightdlg->setData(doc,oglview);
    lightdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(lightdlg.isNull()) return;

    lightdlg->getData(doc);
    if(doc->lights()->changed()) {
      doc->setModified();
      doc->allcalced=false;
    }
    if(!doc->allcalced){
      //updateData(true);
      oglview->setLighting();
      oglview->doRendering();
      doc->lights()->setChanged(false);
      doc->allcalced=true;

    }
  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onCIFImportDlgTalkBack(int mode,const QString filename)
{
  if(mode==ProtoDlg::Create){
    if(!cifimportdlg.isNull()) {
      cifimportdlg->activateWindow();
      cifimportdlg->raise();
      return;
    }
  
    cifimportdlg=new CIFImportDlg(0,base(),"CIFImportDlg");
    connect(cifimportdlg,SIGNAL(talkBack(int)),
	    this,SLOT(onCIFImportDlgTalkBack(int)));
    

    cifimportdlg->setData(doc,filename);
    cifimportdlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(cifimportdlg.isNull()) return;
    cifimportdlg->setData(doc);
    cifimportdlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(cifimportdlg.isNull()) return;
    bool ok;
    cifimportdlg->getData(ok);

    if(ok && !doc->symmetry->calculated()) {
      doc->setModified();
      doc->allcalced=false;
      //refreshBoundDlg();
      if(!doc->allcalced){
	//updateData(true);
	updateData();
      }
      onAnnotationDlgTalkBack(ProtoDlg::Refresh);
    }
    

  }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onUnfoldDlgTalkBack(int mode)
{
  if(mode==ProtoDlg::Create){
    if(!unfolddlg.isNull()) {
      unfolddlg->activateWindow();
      unfolddlg->raise();
      return;
    }
  
    unfolddlg=new UnfoldDlg(0,base(),"UnfoldDlg");
    connect(unfolddlg,SIGNAL(talkBack(int)),
	    this,SLOT(onUnfoldDlgTalkBack(int)));
    

    unfolddlg->setData(doc->symmetry);
    unfolddlg->show();

    return ;
 }else if(mode==ProtoDlg::Refresh){
    if(unfolddlg.isNull()) return;
    unfolddlg->setData(doc->symmetry);
    unfolddlg->update();
 }else if(mode==ProtoDlg::Accept){
    
    if(unfolddlg.isNull()) return;

    //todo    unfolddlg->getData(doc->fog());
//todo     if(doc->unfold()->changed()) {
//todo       doc->setModified();
//todo       doc->allcalced=false;
//todo     }
    if(!doc->allcalced){
      //updateData(true);
      updateData();
    }
  }
}








/* ***********************************************************************
 *
 * ***********************************************************************
 */
// void STR::onShowSymOpsDlgTalkBack(int mode)
// {
//   //myDebug()<<"mode"<<mode;
//   if(mode==ProtoDlg::Create){
    
//     //  myDebug()<<"STR::slotSymOpShowDlg";
//     if(!showsymopsdlg.isNull()) {
//       showsymopsdlg->activateWindow();
//       showsymopsdlg->raise();
//       return;
//     }
//     showsymopsdlg=new ShowSymOpsDlg(0,base(),"ShowSymOpsDlg");
//     connect(showsymopsdlg,SIGNAL(talkBack(int)),
// 	    this,SLOT(onShowSymOpsDlgTalkBack(int)));
    
    
//     showsymopsdlg->setData(doc->symmetry);
//     showsymopsdlg->show();
//   }else  if(mode==ProtoDlg::Refresh){
//     if(showsymopsdlg.isNull()) return;
//     showsymopsdlg->setData(doc->symmetry);
//     showsymopsdlg->update();
//   }else if(mode==ProtoDlg::Accept){
//     doc->setModified();
//     doc->allcalced=false;
//     if(!doc->allcalced){
//       //updateData(true);
//       updateData();
//     }
//   }
// }

/* ***********************************************************************
 *
 * ***********************************************************************
 */

long STR::calcSymmetry()
{
  NSF("STR::calcSymmetry"); Q_UNUSED(nsubfunc);
  //myDebug() << nsubfunc;


  statusbar->setMessage("calculate symmetry");


  
  if(doc->symmetry->calculated() ) return 1;

  String out;

  try{
    doc->symmetry->calcSymFromGroupInput(out,true,false);
  }catch(Except &ex){
    //myDebug() << ex.msg() << ex.stop();
    if(ex.stop()==0){
      QMessageBox::warning(this,ex.title(),ex.msg()
			   ,QMessageBox::Ok|QMessageBox::Default
			   ,QMessageBox::NoButton);
    }else if(ex.stop()==1){
      QMessageBox::critical(this,ex.title(),ex.msg()
			    ,QMessageBox::Ok|QMessageBox::Default
			    ,QMessageBox::NoButton);
      statusbar->setMessage();
      return 0;
    }else{
      QMessageBox::critical(this,ex.title(),ex.msg()
			    ,QMessageBox::Ok|QMessageBox::Default
			    ,QMessageBox::NoButton);

      qApp->closeAllWindows();	
      qApp->exit(1);
      exit(1);
    }    
  }

  //printf("%s\n",out());

  if(propview) propview->fullReset();

  doc->midcalced=false;
  

  base()->outwin->setText( "&Symmetry",out() );


  doc->bonddefs->cleanBonds(doc->symmetry);

  doc->bound()->setCalculated(false);
  doc->displayCells()->setCalculated(false);
  

  doc->axes()->setCalculated(false);

//  doc->kpcalced=0;
//  doc->ergcalced=0;
  doc->allcalced=false;

  iconcache.reset();

  sttoolbox->updateAxesButtons();
  sttoolbox->updateCellButtons();

  statusbar->setName(doc->symmetry->formula()());

  statusbar->setMessage();

  onBondsDlgTalkBack(ProtoDlg::Refresh);

  try{
    //    doc->symmetry->calcSymmetryOperations();
  }catch (Except &ex){
    QMessageBox::warning(this,"determine symmetry operations",ex.msg()
			 ,QMessageBox::Ok|QMessageBox::Default
			 ,QMessageBox::NoButton);
  }

  //  onShowSymOpsDlgTalkBack(ProtoDlg::Refresh);
    


  return 1;
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long STR::calcSymmetryOperations()
{
  //  return 1;

  NSF("STR::calcSTM");
  if(! calcSymmetry() ) return 0;
  statusbar->setMessage("calculating Symmetry information ... ");
  qApp->processEvents();
  qApp->flush();
  
  
  oglview->drawSymmetryOperations(doc);
  

  doc->allcalced=false;
  statusbar->setMessage();
  return 1;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

long STR::calcSTM()
{
  NSF("STR::calcSTM");
  if(! calcSymmetry() ) return 0;

  if(doc->stm->calculated() )return 1;

  statusbar->setMessage("calculating STM info ... ");
  qApp->processEvents();
  qApp->flush();


  QString out;
  try{
    out=doc->stm->calculate(doc->symmetry);
  }catch(Exception &ex){
    QMessageBox::warning(this,"calculate stm",ex.msg()()	
			 ,QMessageBox::Ok|QMessageBox::Default	
			 ,QMessageBox::NoButton);
  }

  base()->outwin->setText( "ST&M" ,  out );

  oglview->drawSTMAtom(doc);


  doc->allcalced=false;
  statusbar->setMessage();
  return 1;


}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
long STR::calcDisplayCells()
{
  NSF("STR::calcDisplayCells");

  //myDebug()<<"ZZZZZZZZZZZZZZZZZZZ"<<doc->displayCells()->changed();
  //myDebug()<<"CCCCCCCCCCCCCCCCAAA"<<doc->displayCells()->calculated();

  if(! calcSymmetry() ) return 0;
  if( doc->displayCells()->calculated()) return 1;
  

  //myDebug("%s\n",nsubfunc);

  statusbar->setMessage("calculating boundary/display cells ... ");
  qApp->processEvents();
  qApp->flush();

  /* calc display cells*/
  
  oglview->removeObject("Display cells ");

  QString out=doc->displayCells()->
    calculate(doc->bound()->latticeType()
	      ,doc->symmetry,doc->tol,doc->vlevel)();
  
  base()->outwin->setText( "&cell" , out );

  doc->cluster->setCalculated(false);
  doc->axes()->setCalculated(false);

  
  //bool iscrystal=(doc->symmetry->structureType() == Xf::STCrystal);

  
  for(int i=0;i<doc->displayCells()->size();i++){
    DisplayCell *dc=doc->displayCells()->cell(i);
    //if(dc->type()==Xf::DCTMainBoundary || iscrystal)
    oglview->drawDisplayCell(dc);
  }
  

  //todo start:remove
  //todo: toggling visibility makes a full recalc of cache since
  //      cache gets invalidates ?? or is it because of other things
  if(doc->mesh) {
    oglview->invalidateMesh(doc->mesh);
    delete doc->mesh; doc->mesh=0;
  }
//   doc->mesh=new RegularMesh("RegularMesh 1");
//   const Matrix3x3 &cell=doc->symmetry->cell();
//   doc->mesh->setOrigin((0./12-0.5)*(cell(0)+cell(1)+cell(2)));
//   doc->mesh->setCell(doc->symmetry->cell());
//   doc->mesh->setSubdivisions(2,2,2);
//   doc->mesh->calcPoints(0,false);
//   doc->mesh->calcMicroCells();
//   doc->mesh->calculateGradient();

//   doc->triamesh=new TriangleMesh("TriangleMesh 1");
//   doc->mesh->createIsoSurface(doc->triamesh,doc->bound(),0,0.8,0,false);
//   double dmin,dmax;
//   doc->triamesh->applyColor(idata,IsoColor(),dmin,dmax);

  //  doc->mesh->printPoints();
  
//   oglview->drawMeshPoints(doc->mesh,OGLMeshPoints::Full,true);
//   oglview->drawMeshCells(doc->mesh,OGLMeshCells::Microcells,true);
//   oglview->drawMeshCells(doc->mesh,OGLMeshCells::Simplices,true);
//   oglview->drawTriangleMesh(doc->triamesh,true);
  
  //todo end:remove
  
  
  

  //doc->specpoidirty=true;
  doc->allcalced=false;

  statusbar->setMessage();
  
  return 1;
}



/* ***********************************************************************
 *
 * ***********************************************************************
 */
long STR::calcCluster()
{
  NSF("STR::calcCluster");
  //  myDebug("%s\n",nsubfunc);

  MachineTime time;

  if(! calcSymmetry() ) return 0;
  if(!calcDisplayCells())return 0 ;


  if(doc->cluster->calculated() )return 1;

  statusbar->setMessage("calculating  cluster ... ");

  
  
  
  String out=doc->cluster->create(doc->symmetry,doc->bound()
				   ,doc->bonddefs,statusbar);
  time.print(String(nsubfunc)+" cluster->create");

  if(propview)propview->fullReset();


  base()->outwin->setText( "c&luster" ,  out() );
  time.print(String(nsubfunc)+" base()->outwin->setText");
  

  oglview->drawAtoms(doc);
  time.print(String(nsubfunc)+" drawAtoms");
  oglview->drawBonds(doc);
  time.print(String(nsubfunc)+" drawBonds");
  oglview->drawPolyhedra(doc);
  time.print(String(nsubfunc)+" drawPolyhedra");

  doc->allcalced=false;

  statusbar->setMessage();

  onFindAtomDlgTalkBack(ProtoDlg::Refresh);
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
long STR::calcAxes()
{

  if(! calcSymmetry() ) return 0;
  if(!calcDisplayCells())return 0 ;


  if( doc->axes()->calculated() ) return 1;

  doc->axes()->calculateAxes(doc->bound()->latticeType()
			     ,doc->bound()->radius(),doc->symmetry);
  

  for(long i=-1;++i<doc->axes()->size();){
    Axes *ax=doc->axes()->axes(i);
    oglview->drawAxes(ax,doc->fontRes());
  }


  doc->allcalced=false;
  
  return 1;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::updateData(bool updatePanZoom)
{
  slotPlotCalc(updatePanZoom);
  return;
  QThread *thread=new QThread();
  Worker *worker=new Worker(30000/10000,updatePanZoom);
  worker->moveToThread(thread);
  //connect(worker, SIGNAL(error(QString)), this, SLOT(errorString(QString)));
  connect(&interrupt, SIGNAL(interuptRequested()), worker, SLOT(requestInterrupt()),
	  Qt::QueuedConnection);
  connect(thread, SIGNAL(started()), worker, SLOT(process()));
  connect(worker, SIGNAL(done(bool)), this, SLOT(dataUpdated(bool)));
  connect(worker, SIGNAL(finished()), thread, SLOT(quit()));
  connect(worker, SIGNAL(finished()), worker, SLOT(deleteLater()));
  connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
  thread->start();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void STR::dataUpdated(bool updatePanZoom)
{
  myDebug()<<"dataUpdated, updatePanZoom="<<updatePanZoom;
  slotPlotCalc(updatePanZoom);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotPlotCalc(bool updatePanZoom)
{
  NSF("STR::slotPlotCalc");
  myDebug() << nsubfunc<<updatePanZoom;
  


  oglview->mute(true);

  statusbar->setProgress(0.); 
  statusbar->setProgress(0.01); 


  double p=statusbar->getCurrentProgress();
  if(!calcSymmetry())goto clean;
  //myDebug() << "hier10";

  statusbar->setProgress(p+0.03*(1-p)); 
  //myDebug() << "hier11";

 
  if(interrupt.requested()) goto clean;

  
  //myDebug() << "hier2";
  

  p=statusbar->getCurrentProgress();
  if(!calcDisplayCells())goto clean;
  statusbar->setProgress(p+0.03*(1-p)); 


  //myDebug() << "hier3";


  if(!calcSymmetryOperations()) goto clean;


  
  if(interrupt.requested()) goto clean;

  

  if(!calcAxes())goto clean;
  
  //myDebug() << "hier4";

  if(interrupt.requested()) goto clean;
  

  p=statusbar->getCurrentProgress();
  statusbar->setAllotedPercent(0.10);
  if(!calcCluster())goto clean;
  statusbar->setProgress(p+0.2*(1-p)); 

  //myDebug() << "hier5";
  

  if(interrupt.requested()) goto clean;
  

  if(!calcSTM())goto clean;

  

  

  p=statusbar->getCurrentProgress();
  statusbar->setAllotedPercent(((0.9-p)/(1-p)));

  //myDebug() << "hier1";
  oglview->setLighting();
  oglview->colorAtoms(doc);

  Representation::draw(oglview,doc->prepresentations);

  oglview->reCalcScene();
  
  if(interrupt.requested()) goto clean;

  if(updatePanZoom) oglview->resetPanZoom(OGLView::Reset,false);

  statusbar->setAllotedPercent(1.00);




  oglview->mute(false);
  
  oglview->doRendering();


  doc->axes()->setChanged(false);  
  doc->lights()->setChanged(false);

  statusbar->deleteMessage();
  
  sttoolbox->setMuted(false);
  
  doc->allcalced=true;
  
  
 clean:
  statusbar->setProgress(0); 
  statusbar->setMessage();

  //statusbar->setFullGo(true);

  oglview->atomFindMarker()->refresh();
  
  return;
   
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::slotPlotReCalc()
{
  doc->resetflags();
  updateData();

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomSequence()
{
  AtomSequenceDlg *dlg=new AtomSequenceDlg(this,base(),"AtomSequenceDlg");

  connect(dlg,SIGNAL(atomSequenceChanged())
	  ,this,SLOT(onAtomSequenceChanged()));

  dlg->setData(doc->symmetry);

  long ret=dlg->exec();
  if(ret){  	
//     long ch=0;
//     dlg->getData(doc->symmetry,ch);
//     //    myDebug() << "after chn" << ch;
//     if(ch) {
//       doc->setModified();
//       doc->allcalced=false;
//       refreshBoundDlg();
//     }
  }
  delete (dlg);

  return ;


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void STR::onAtomSequenceChanged()
{
  updateData();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool STR::eventFilter(QObject *object, QEvent *event)
{
  //NSF("bool STR::eventFilter(QObject *object, QEvent *event)");
  if (object == propview && event->type() == QEvent::KeyPress) {
    QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
    //myDebug() <<nsubfunc<< keyEvent->key();
    if (keyEvent->key() == Qt::Key_M) {
      // Special tab handling
      onAtomPickActionTriggered();
      return true;
    }else if(keyEvent->key() == Qt::Key_Escape) {
      // Special tab handling
      onAtomPickResetActionTriggered();
      return true;
    }else
      return false;
  }
  return false;
}

//EOF
