/* @(#)STR.hh
 * author: Klaus.Koepernik@ifw-dresden.de
 */
#ifndef STRWIDGETCLASS_H
#define STRWIDGETCLASS_H

#include <QPointer>
#include "ui_STR.h"
#include "ControlBase.hh"
#include "ProtoDlg.hh"

class String;
class QAction;
class STDoc;
class FogDlg;
class TestDlg;
class CIFImportDlg;
class UnfoldDlg;
class AtomPropsDlg;
class PropView;
class BoundDlg;
class BondsDlg;
class FindAtomDlg;
class MapAtomsDlg;
class SymDlg;
class LightDlg;
//class ShowSymOpsDlg;
class ContextMenuContext;

/* ***********************************************************************
 *
 * ***********************************************************************
 */
class STR : public ControlBase ,private Ui::STR
{
  Q_OBJECT
public:
  STR(MWBase *parent = 0);
  ~STR();


  virtual void help();
  virtual void afterShow(const QString &load);
  virtual bool queryExit();
  virtual bool eventFilter(QObject *object, QEvent *event);

  virtual void slotPlotCalc(bool updatePanZoom=false);

  virtual void updateData(bool updatePanZoom=false);


private:
  enum SaveMode{FILESAVE=1,FILESAVEAS=2};
private slots:
  void dataUpdated(bool );
  void slotFPLOInImport();
  void slotSTMImport();
  void slotFileClose();
  void slotPlotReCalc();
  long calcSymmetry();
  long calcAxes();
  long calcCluster();
  long calcSTM();
  long calcDisplayCells(); 
  long calcSymmetryOperations();

  void winMenu_aboutToShow();
  void toolMenu_aboutToShow();
  void slotShowOutwin(bool onoff);


  //  void onShowSymOpsDlgTalkBack(int mode=ProtoDlg::Create);
  void onAtomPropsDlgTalkBack(int mode=ProtoDlg::Create);
  void onFogDlgTalkBack(int mode=ProtoDlg::Create);
  void onBoundDlgTalkBack(int mode=ProtoDlg::Create);
  void onFindAtomDlgTalkBack(int mode=ProtoDlg::Create);
  void onSymDlgTalkBack(int mode=ProtoDlg::Create);
  void onMapAtomsDlgTalkBack(int mode=ProtoDlg::Create);
  void onBondsDlgTalkBack(int mode=ProtoDlg::Create);
  void onAnnotationDlgTalkBack(int mode=ProtoDlg::Create);
  void onLightDlgTalkBack(int mode=ProtoDlg::Create);
  void onCIFImportDlgTalkBack(int mode=ProtoDlg::Create,
			      const QString filename="");
  void onUnfoldDlgTalkBack(int mode=ProtoDlg::Create);
  void onTestDlgTalkBack(int mode=ProtoDlg::Create);


  void slotFileOpen();
  void slotFileSave();
  void slotFileSaveAs();
  void slotFilePrint();
  bool fileLoad(const QString &fileName);
  void saveit(SaveMode saveas);

  void onAtomInfoToShow(String);
  void onAtomPickActionTriggered();
  void onAtomPickResetActionTriggered();
  void onAtomClicked(long);
  void onContextActionRequest(const ContextMenuContext&);

  void onAtomSequence();
  void onAtomSequenceChanged();


protected:
  bool isModified();
private:
  void setupMenus();
  void FPLOInImport(const QString &);
  void STMImport(const QString &);
  void CIFImport(const QString &);
  void updateSizes(bool fresh);

private:
  STDoc *doc;
  PropView *propview;
  QAction *canvasAction;
  QAction *showOutAction;
  QAction *calcAction;
  QAction *atomPickAction;
  QAction *atomPickResetAction;
  QAction *toolAction;
  
  //  QPointer<ShowSymOpsDlg> showsymopsdlg;
  QPointer<FogDlg> fogdlg;
  QPointer<BoundDlg> bounddlg;
  QPointer<FindAtomDlg> findatomdlg;
  QPointer<SymDlg> symdlg;
  QPointer<MapAtomsDlg> mapatomsdlg;
  QPointer<BondsDlg> bondsdlg;
  QPointer<TestDlg> testdlg;
  QPointer<CIFImportDlg> cifimportdlg;
  QPointer<UnfoldDlg> unfolddlg;
  QPointer<AtomPropsDlg> atompropsdlg;
  QPointer<LightDlg> lightdlg;
  
  friend class STToolBox;
  friend class ToolBox;
};


#endif // STRWIDGETCLASS_H
