/*************************************************************************
 * @(#)Status.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   01 Jul 2009
 *************************************************************************
 */


#include <QProgressBar>
#include <QApplication>
#include <QTimer>
#include <QToolButton>
#include <QLabel>
#include "Status.hh"
#include "ElidLabel.hh"
#include "MyMath.hh"
#include "Settings.hh"
#include "CanvasSize.hh"
#include "Interrupt.hh"
#include "StatusConfigDlg.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
Status::Status(QWidget * parent )
  :QStatusBar(parent),progbar(0),tbInterupt(0),msglabel(0),sizeindicator(0)
  ,labeltimer(0),interval(500)
  ,tbCanvasLock(0),tbConfig(0)
  ,pdocumentname(""),pcanvascontrol(false),palloted(-1.)
  ,pfullgo(false),plengthunit(Xf::BohrRadius)
{
  progbar=new QProgressBar(this);
  progbar->hide();

  {
    Settings set("Status/FermiSurface");
    plengthunit=(Xf::LengthUnits)set.getInt("Config/lenghtunit",Xf::BohrRadius);
  }

    
  if(1){
    tbCanvasLock=new QToolButton(this);
    tbCanvasLock->setFocusPolicy(Qt::NoFocus);
#ifndef QT_NO_TOOLTIP
    tbCanvasLock->setToolTip("Lock canvas size");
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_STATUSTIP
    tbCanvasLock->setStatusTip("Lock canvas size");
#endif // QT_NO_STATUSTIP
    QIcon icon;
    icon.addPixmap(QPixmap(":/Icons/unlock"), QIcon::Normal, QIcon::Off);
    icon.addPixmap(QPixmap(":/Icons/lock"), QIcon::Normal, QIcon::On);
    tbCanvasLock->setIcon(icon);
    tbCanvasLock->setCheckable(true);
    tbCanvasLock->setText("");
    addPermanentWidget(tbCanvasLock,0);
    
    
    tbConfig=new QToolButton(this);
    tbConfig->setFocusPolicy(Qt::NoFocus);
#ifndef QT_NO_TOOLTIP
    tbConfig->setToolTip("Status configuration");
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_STATUSTIP
    tbConfig->setStatusTip("Status configuration");
#endif // QT_NO_STATUSTIP
    tbConfig->setIcon(QIcon(":/Icons/config"));
    tbConfig->setText("");
    addPermanentWidget(tbConfig,0);
    
    
  }

  if(1){
    sizeindicator=new QLabel(this);
    addPermanentWidget(sizeindicator,0);
    sizeindicator->setFrameShape(QFrame::Panel);
    sizeindicator->setFrameShadow(QFrame::Sunken);
    sizeindicator->setLineWidth(1);
  }

  if(1){
    msglabel=new QLabel(this);
    msglabel->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Preferred);
    addPermanentWidget(msglabel,2);
    msglabel->setFrameShape(QFrame::NoFrame);
//   msglabel->setFrameShadow(QFrame::Sunken);
//   mgslabel->setLineWidth(1);
  }
  
  if(1){
    tbInterupt=new QToolButton(this);
    //tbInterupt->hide();
    tbInterupt->setFocusPolicy(Qt::NoFocus);
#ifndef QT_NO_TOOLTIP
    tbInterupt->setToolTip("Interupt calculation/printing... (ESC)");
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_STATUSTIP
    tbInterupt->setStatusTip("Interupt calculation/printing... (ESC)");
#endif // QT_NO_STATUSTIP
    QIcon icon;
    icon.addPixmap(QPixmap(":/Icons/stop"));
    tbInterupt->setIcon(icon);
    tbInterupt->setText("");
    tbInterupt->setShortcut(Qt::Key_Escape);
    addPermanentWidget(tbInterupt,0);
  }

  addPermanentWidget(progbar,1);

  labeltimer=new QTimer(this);

  connect(this,SIGNAL(messageChanged ( const QString & ))
	  ,this,SLOT(beware(const QString &)));

  connect(tbInterupt,SIGNAL(clicked())
	  ,this,SLOT(onTbInteruptClicked()));


  connect(labeltimer, SIGNAL(timeout()), this, SLOT(labelTimer()));


  connect(tbCanvasLock, SIGNAL(clicked(bool))
	  , this, SLOT(tbCanvasLockClicked(bool)));

  connect(tbConfig, SIGNAL(clicked(bool))
	  , this, SLOT(tbConfigClicked(bool)));

  if(!pcanvascontrol){
    tbCanvasLock->hide();
    tbConfig->hide();
    sizeindicator->hide();
  }


}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::tbCanvasLockClicked(bool onoff)
{
  //myDebug() << "tbCanvasLockClicked"<<onoff;
  emit canvasLockClicked(onoff);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::tbConfigClicked(bool )
{
  StatusConfigDlg *dlg=new StatusConfigDlg(this);
  dlg->setData(plengthunit);
  if(dlg->exec()==QDialog::Accepted){
    dlg->getData(plengthunit);
    Settings set("Status/FermiSurface");
    set.set("Config/lenghtunit",plengthunit);

  }
  delete dlg;
}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setCanvasControlEnabled(bool e)
{
  pcanvascontrol=e;
  tbConfig->show();
  tbCanvasLock->show();
  sizeindicator->show();
    
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setCanvasControl(const CanvasSize &c)
{
  if(!pcanvascontrol) return;


  sizeindicator->setText(QString("%1 x %2").arg(c.width()).arg(c.height()));

  tbCanvasLock->setChecked(c.locked());
    
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::beware(const QString &msg)
{
  //  myDebug() << "Status::beware" <<msg;
  if(msg.isEmpty()){
    if(msglabel){
//       QFontMetrics fm=msglabel->fontMetrics();
//       QString s=fm.elidedText (,Qt::ElideLeft,msglabel->width());
      msglabel->setText(pname+pdocumentname);
    }
  }else{
    if(msglabel) msglabel->setText(pname+msg);
  }
  //qApp->processEvents();
  // qApp->flush();

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */
Status::~Status()
{
  if(progbar) delete progbar; progbar=0;
  if(tbInterupt) delete tbInterupt; tbInterupt=0;
  if(labeltimer) delete labeltimer; labeltimer=0;
  if(msglabel) delete msglabel; msglabel=0;
  if(sizeindicator) delete sizeindicator; sizeindicator=0;
  if(tbCanvasLock) delete tbCanvasLock; tbCanvasLock=0;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setDocument(const QString &name)
{
  pdocumentname=name;
  if(msglabel) msglabel->setText(pname+name);
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setProgress(double percent)
{
  NSF("Status::setProgress");
  if(percent<=0.001){
    pcurrentprogold=0;
    pcurrentprog=0;
    progbar->setValue(0); 
    progbar->hide();
    interrupt.request(false);
    //tbInterupt->hide();
    return;
  }else{
    //    myDebug("%s: precent %ld\n",nsubfunc,percent);

    progbar->show();
    tbInterupt->show();

    if(pcurrentprogold+0.07<=percent){
      //myDebug() << "percent" << pcurrentprogold << percent;
      pcurrentprogold=pcurrentprog;
      progbar->setValue(dRound(100*percent)); 
//       if(pfullgo){
// 	myDebug() << nsubfunc << "processEvents";
// 	qApp->processEvents();
// 	qApp->flush();
//       }
    }
    pcurrentprog=percent;

    //    sleep(1);
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
double Status::getCurrentProgress()
{
  if(!progbar) return 0;
  if(progbar->isHidden()){
    return 0;
  }else{
    return pcurrentprog;
  }
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setMessage(const QString &msg,int msec)
{
  NSF("Status::setMessage");
  //  printf("%s: mesg(%s)\n",nsubfunc,msg.toAscii().constData());
  if(msg.isEmpty()){
    labeltimer->start(1000);
  }else{
    labeltimer->stop();
    showMessage(msg,msec);
  }
//   if(pfullgo){
//     myDebug() << nsubfunc << "processEvents";
//     //myDebug() << "isVisible" << isVisible();
//     qApp->processEvents();
//     qApp->flush();
//   }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::labelTimer()
{
  //NSF("Status::labelTimer");
  //  myDebug("%s: duration %g dt=%g\n",nsubfunc,duration/1000, dt);

  labeltimer->stop();
  clearMessage();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::onTbInteruptClicked( )
{
  //myDebug("onTbInteruptClicked");
  interrupt.request();
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void Status::setBusyProgress(bool onoff)
{
  if(onoff){
    progbar->setMinimum(0);
    progbar->setMaximum(0);
    progbar->setValue(0);
    progbar->show();

  }else{
    progbar->setMinimum(0);
    progbar->setMaximum(100);
    progbar->hide();
  }
}

/*EOF*/ 
