//todo: molecules
//todo: if plane lattice is perp to two cell vectors, use those to 
//shorten instead: do we need the conventional cell for this?
// what if rotoreflections fall onto C
/*************************************************************************
 * @(#)SpaceGroupOperation.cpp
 * author: Klaus.Koepernik@ifw-dresden.de
 * date:   03 Oct 2011
 *************************************************************************
 */

#include "SpaceGroupOperation.hh"
#include "Except.hh"
#include "MyMath.hh"
#include "IVec3.hh"
#include "MyDebug.hh"

/* ***********************************************************************
 *
 * ***********************************************************************
 */
SpaceGroupOperationBase * SpaceGroupOperationBase
::generateOperation(const Matrix3x3 &opmat,const Vec3 &optau,
		    const Matrix3x3 &cell,bool prot)
{
/*
  determine a symbol for the operation
  the op is supposed to rotate a vector given as coordinates
  like r.opmat, which is the transposed of the normal definition used elsewhere
  Let the vector of cartesian unit vectors be
  E^T=(\vec{e}_1,\vec{e}_2,\vec{e}_3)
  Then the operation on the coordinate system is defined as
  \hat(D) E^T =E^T R
  with the rotation matrix R.
  
  A vector \vec{r} =E^T.r, r^T=(r1,r2,r3) is rotated as.
  
  \hat(D) \vec{r}=\hat(D) E^T.r ==E^T.R. r
  
  Now, here we assume that opmat is given as R^T !!!
*/

//  NSF("detOpSymbol");
  double tolsymdet=1.0e-4;
//     Integer(t_int) :: i,n2
//     Real(t_real)   :: vv(3),V(3,3),S(3,3),we(3),d
//     !!
  if(prot){
    printf("opmat=\n%s\n",opmat.printS("%12.5f")());
  }
  // 
  // 
  // determine the determinant, proper   op ->  1
  //                            improper op -> -1
  //
  
  long pdet=lRound(opmat.determinant());
  double trace=opmat.trace()*pdet;
  if(prot) printf("det=%ld\n",pdet);
  if(prot) printf("trace=%g\n",trace);
  
  //determine the eigensystem of S=(R^T-det)*(R-det) (no longer done that way)
  
  Matrix3x3 S=opmat*pdet;
  Vec3 paxis;
  double pangle;
  long pn=0; // singnals non-rational angle
  long pm=0;
  bool isC2=false;

  if(fabs(trace-3.)<tolsymdet) {
    pn=1;
    pm=1;
  }else{
    if(fabs(trace+1)<tolsymdet){
      pangle=180;
      isC2=true;
      pn=2;
      pm=1;
      if(prot) printf("C2 case=%g\n",pangle);
      Vec3 n1=Vec3(1,0,0);
      Vec3 n2=Vec3(0,1,0);
      Vec3 n3=Vec3(0,0,1);
      n1=n1+n1*S;
      n2=n2+n2*S;
      n3=n3+n3*S;
      double d1=norm(n1);
      double d2=norm(n2);
      double d3=norm(n3);
      if(d1>d2){
	if(d1>d3){
	  paxis=n1;
	}else{
	  paxis=n3;
	}
      }else{
	if(d2>d3){
	  paxis=n2;
	}else{
	  paxis=n3;
	}
      }
    }else{
      Matrix3x3 a=((S-S.transposed())*0.5);
      if(prot)printf("assym \n%s\n",a.printS("%12.5f")());
      paxis[0]=-a(1,2);
      paxis[1]=a(0,2);
      paxis[2]=-a(0,1);
    }
    paxis.normalize();
    // determine the symmetry axis in a sign convention
    if(paxis[2]<0.0) paxis*=-1;
    if(fabs(paxis[2])<tolsymdet){
      if(paxis[1]<0.0)paxis*=-1;
      if(fabs(paxis[1])<tolsymdet){
	if(paxis[0]<0.0)paxis*=-1;
      }
    }
    // det rotation sense
    if(!isC2){
      Vec3 v1=Vec3(1,0,0).cross(paxis);
      Vec3 v2=Vec3(0,1,0).cross(paxis);
      Vec3 v3=Vec3(0,0,1).cross(paxis);
      double d1=norm(v1);
      double d2=norm(v2);
      double d3=norm(v3);
      Vec3 x=v1;
      if(d1>d2){
	if(d1<d3){
	  x=v3;
	}
      }else{
	if(d2>d3){
	  x=v2;
	}else{
	  x=v3;
	}
      }
      x.normalize();
      Vec3 gx=x*S;
      Vec3 z=x.cross(gx);
      if(prot) printf("z = %s\n",paxis.printS("%15.8f")());
      if(prot) printf("x = %s\n",x.printS("%15.8f")());
      if(prot) printf("gx= %s\n",gx.printS("%15.8f")());
      if(prot) printf("x x gx= %s\n",z.printS("%15.8f")());
      double s=norm(z);
      double c=(x*gx);

      pangle=atan2(s,c)/Pi*180.;
      if(prot) printf("angle1=%g\n",pangle);
      if((paxis*z)<0.){
	pangle=360-pangle;
      }
    }
    if(prot) printf("angle=%g\n",pangle);

    // make simple integers out of angle if possible
    
    String::makeRatio(pangle/360.,pm,pn,100,1.0e-12);

    if(pn == 0){
      // not resolved
      throw Except(Sprintf("cannot resolve axis order from angle(%g)\n",
			   pangle));
    }

    if(pn == 1){
      if(prot) printf("E/I\n");
      // integer
    }else{
      //ratio
      if(prot) printf("known =%ld/%ld\n",pm,pn); 
      if(pdet<0 && !isC2){
	// adjust the rotoreflexions according to the convention
	// being here, we have E/I C(n,m)
	//
	// I C(n,m)=s_h C(2,1) C(n,m)=s_h C(N,M) , N=2n, M=2m+n
	//        if M odd 
	//         =S(N,M)
	//        extract common factor 2: N=N/2, M=M/2 until N or M odd
	//        then if the remaining M even put M=M+N
	pm=2*pm+pn;
	pn=2*pn;
	if(prot) printf("raw n=%ld\n",pn);
	if(prot) printf("raw m=%ld\n",pm);
	while( (pn%2)==0 && (pm%2)==0){
	  pn/=2;
	  pm/=2;
	}
	if(prot) printf("one odd n=%ld\n",pn);
	if(prot) printf("one odd m=%ld\n",pm);
	if((pm%2)==0){
	  pm+=pn;
	}
	if(prot) printf("m odd n=%ld\n",pn);
	if(prot) printf("m odd n=%ld\n",pm);
	long n2=pn;
	if((pn%2)!=0)n2=2*pn;
	while(pm>n2){
	  pm-=n2;
	}
      }
    }
      
  }
  if(prot) printf("axis= %s\n",paxis.printS("%15.8f")());
  if(prot) printf("n=%ld\n",pn);
  if(prot) printf("m=%ld\n",pm);

  if(pdet<0){
    // improper
    if(pn==1){
      if(prot) printf("is Inversion\n");
      return new SpaceGroupInversion(opmat,optau,cell);
    }else if(pn==2 && pm==1){
      return new SpaceGroupMirror(opmat,optau,paxis,cell);
    }else{
      long pRIm=pn+2*pm;
      long pRIn=2*pn;
      long d=MyMath::gcd(pRIm,pRIn);
      pRIn/=d;
      pRIm=((pRIm/d)%pRIn);
      return new SpaceGroupRotoInversion(opmat,optau,paxis,pRIn,pRIm,cell);
    }
  }else{
    // proper
    if(pn==1){
      return new SpaceGroupIdentity();
    }else{
      return new SpaceGroupRotation(opmat,optau,paxis,pn,pm,cell);
    }
  }

}


/* ***********************************************************************
 *
 * print axis of operation
 * note the axis is normalized such that the last non-zero component is
 * positive: so x-y is expected as (-sq2o2,sq2o2,0)
 *
 * ***********************************************************************
 */
String SpaceGroupOperationBase::printAxis(const Vec3 &axis,bool isC2)
{
  //  NSF("prnt_axis");
  double tol=1.0e-10;
  double known_axis[][3]={
    {1,0,0},
    {0,1,0},
    {0,0,1},
//     {0.707106781187,0.707106781187,0},
//     {-0.707106781187,0.707106781187,0},
//     {0.707106781187,0,0.707106781187},
//     {-0.707106781187,0,0.707106781187},
//     {0,0.707106781187,0.707106781187},
//     {0,-0.707106781187,0.707106781187},
    {-0.57735026919,0.57735026919,0.57735026919},
    {0.57735026919,-0.57735026919,0.57735026919},
    {-0.57735026919,-0.57735026919,0.57735026919},
    { 0.57735026919, 0.57735026919,0.57735026919}
  };
  const char *known_axis_names[]={
    "x",
    "y",
    "z",
//     "xy",
//     "-xy",
//     "xz",
//     "-xz",
//     "yz",
//     "-yz",
    "-111",
    "1-11",
    "-1-11",
    "111"
  };
  long num_known_axis=sizeof(known_axis_names)/sizeof(char*);

  
  for(int i=0;i<num_known_axis;i++){
    if(norm(axis-known_axis[i])<tol){
      return known_axis_names[i];
    }
  }
  String plane="  ";
  double x=axis[0];
  double y=axis[1];
  if( fabs(axis[2])<tol){
    plane="xy";
    x=axis[0];
    y=axis[1];
  }else if( fabs(axis[1])<tol){
    plane="xz";
    x=axis[0];
    y=axis[2];
  }else if( fabs(axis[0])<tol){
    plane="yz";
    x=axis[1];
    y=axis[2];
  }
  if(plane != "  "){
    double d=atan2(y,x)*180./Pi;
    if(isC2 && d>90.) d-=180.;
    long id=lRound(d);
    if(fabs(id-d)<tol){
      if(id==45 && isC2){
	return Sprintf("%s",plane(),id);
      }else if(id==-45 && isC2){
	return Sprintf("%c-%c",plane[0],plane[1]);
      }else{
	return Sprintf("%s[%ld]",plane(),id);
      }
    }else{
      return Sprintf("%s[%f]",plane(),d);
    }
  }
  return Sprintf("%f,%f,%f",axis[0],axis[1],axis[2]);
}





/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::RAlongAxis(const Vec3 &axis,long n,
				     const Matrix3x3 &A
				     ,const Matrix3x3 &G,
				     IVec3 &NR,double tol)
{
  // get R with |R|->Min, which is  along symmetry axis
  // return lattice indices of such a vector
  
  NSF("R_along_axis");    

  long i=-1;
  double da=0.;
  for(int k=0;k<3;k++){
    da=fabs(axis*A(k));
    if(da>tol) {i=k;break;}
  }
  if(i<0)
    throw Except(Sprintf("%s: All basis vectors orthogonal to axis?\n",
			 nsubfunc));
  
  da=n*da;
  for(int k=0;k<3;k++){
    double d=da*(axis*G(k));
    if(fabs(d-dRound(d))>tol)
      throw Except(Sprintf("%s: not a symmetry axis is?\n",nsubfunc));
    
    NR[k]=lRound(d);
    if(fabs(NR[k]-dRound(d))>tol)
      throw Except(Sprintf("%s: Integer overflow !\n",nsubfunc));
    
  }
  i=labs(NR.gcd());
  if(i==0)
    throw Except(Sprintf("%s: GCD error !\n",nsubfunc));
  
  for(int k=0;k<3;k++){
    NR[k]/=i;
  }
  
  Vec3 R=A*NR;
  da=(axis*R)/norm(R);
  if(fabs(da-1.)>tol)
    throw Except(Sprintf("%s: R not parallel to axis !\n",nsubfunc));
    

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::latticePlane(const IVec3 &MG,IMatrix3x3 &RN,
				       IMatrix3x3 &GN,bool prot)
{
  // given a reciprocial lattice vector
  // calculate the basis vectors  of the plane
  // (G|R)=0
  // and one vector in plane 
  // (G | R_||  )=1
  // these vectors form a possible unit cell of the lattice
  // RN[2] is the one with the smallest projection onto the direction
  // of G
  // MG are the components of G in term of the lattice vectors A
  // return indices of 
  NSF("lattice_plane");    



  long l2=-1; for(int i=0;i<3;i++){ if(MG[i] != 0){ l2=i; break; } }
  if(l2<0) 
    throw Except(Sprintf("%s: Reciprocial vector is zero.\n",nsubfunc));
  
  long l0=(l2+1) %3;
  long l1=(l0+1) %3;
  if( MG[l1] ==0 ){ long i=l0; l0=l1; l1=i; }
  
  long M0=MG[l0]; long M1=MG[l1]; long M2=MG[l2];
  long D=MyMath::gcd(M0,M1);
  long D0=labs(MyMath::gcd(D,M2));
  M0/=D0; M1/=D0; M2/=D0; D/=D0; 
  D=MyMath::gcd(M0,M1);
  
  if(  labs(MyMath::gcd(D,M2)) !=1)
    throw Except(Sprintf("%s: Reciprocial has common devisor?\n",nsubfunc)); 
  
  long a,b;
  if(M0==0){
    M0=0; M1=1; a=0; b=1;
  }else{
    M0=M0/D;
    M1=M1/D;
    a=0; while ( (1-M0*a)%M1 !=0 ) a++;
    b=(1-M0 *a) / M1;
  }
  long la=0; while ( (1-D*la)%M2 !=0 ) la++;
  long mu=(1-D *la) / M2;
  
  if( a*M0 +b*M1 != 1 || D*la +M2*mu != 1 )
    throw Except(Sprintf("%s: Modulus devision failed?\n",nsubfunc));
  
  RN(l0,0)=M1;
  RN(l1,0)=-M0;
  RN(l2,0)=0;
  
  
  RN(l0,1)=M2 *a;
  RN(l1,1)=M2 * b;
  RN(l2,1)=-D;

  RN(l0,2)=la * a;
  RN(l1,2)=la * b;
  RN(l2,2)=mu;

  if(prot) {
    myDebug() << "l012="<<l0<<l1<<l2;
    myDebug() << "RN[0]"<<M1<<-M0<<0;
    myDebug() << "RN[1]"<<M2*a<<M2*b<<-D;
    myDebug() << "RN[2]"<<la*a<<la*b<<mu;
  
    myDebug()<<"MG"<<MG;
    ///myDebug()<<"RN\n"<<RN;
  }
  long idet;
  GN=RN.inverseTtV(idet);
  

  //  if(prot) myDebug()<<"GN\n"<<GN;

  GN.scaleColumn(0,lSign(1,RN.column(0)*GN.column(0)));
  GN.scaleColumn(1,lSign(1,RN.column(1)*GN.column(1)));
  GN.scaleColumn(2,lSign(1,RN.column(2)*GN.column(2)));

  //  if(prot) myDebug()<<"GN\n"<<GN;
  
  // column wise RN[c]*GN[c]=RN.transposed()*GN
  IMatrix3x3 un=RN.transposed()*GN;

  //  if(prot) myDebug()<<"un"<<un;
  
  if( un(2,2)!=1 )
    throw Except(Sprintf("%s: Did not find the smallest cell? (V=%ld)\n"
			 ,nsubfunc,un(2,2)));
  
  if(prot){
    myDebug()<<"D0"<<D0;
    myDebug()<<"GN[2]*D0"<<GN.column(2)*D0;
    myDebug()<<"GN[2]*D0-MG"<<GN.column(2)*D0-MG;
  }

  
  if(mhnorm(GN.column(2)*D0-MG)!= 0 )
    throw Except(Sprintf("%s: GC(%ld,%ld,%ld)*D0(%ld) != MG(%ld,%ld,%ld) ?\n",
			 nsubfunc,
			 GN(0,2),GN(1,2),GN(2,2),D0,
			 MG[0],MG[1],MG[2]
			 ));

  if( RN.column(0)*MG!=0 )
    throw Except(Sprintf("%s: RA not orthogonal to G?\n",nsubfunc));
  
  if( RN.column(1)*MG!=0 )
    throw Except(Sprintf("%s: RB not orthogonal to G?\n",nsubfunc));
  
  
  if( RN.column(2)*MG!=D0 )
    throw Except(Sprintf("%s: RC not in first plane?\n",nsubfunc));
  
  D=labs(RN.column(0).gcd());
  if( D!=1 )
    throw Except(Sprintf("%s: RA not in shortest possible?\n",nsubfunc));

  D=labs(RN.column(1).gcd());
  if( D!=1 )
    throw Except(Sprintf("%s: RB not in shortest possible?\n",nsubfunc));

  D=labs(RN.column(2).gcd());
  if( D!=1 )
    throw Except(Sprintf("%s: RC not in shortest possible?\n",nsubfunc));
  
  
  
  if( !(un==IMatrix3x3::unitMatrix(1)))
    throw Except(Sprintf("%s: Ri*Gj !=1?\n",nsubfunc));
  
  
  
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::minl12(const Matrix3x3 &A,IMatrix3x3 &RN,
				     long i1,long i2,double &smin,
				     bool prot)
{
  
  if(prot) printf("\nminl12 i1,i2=%ld %ld\n",i1,i2);

  double S11=(A*RN.column(i1))*(A*RN.column(i1));
  double S22=(A*RN.column(i2))*(A*RN.column(i2));
  double S12=(A*RN.column(i1))*(A*RN.column(i2));
  //
  //
  // Snew= (a1+i20*a2)*S*(a1+i20*a2)+S22
  //     = trace(S)+(2*S12+S22*i20)*i20
  //
  //
  //
  long n2=lRound(-2.0*S12/S22);
  //
    
  if(prot)printf("n2=%ld\n",n2);
      
  //
  smin=S11+S22;
  long i20=n2;
  for(long ii2=n2-2;ii2<=n2+2;ii2++){
    double s=S11+S22+(2.0*S12+S22*ii2)*ii2;
      if(smin>s){
	i20=ii2;
	smin=s;
      }
  }
  if(prot){
    printf("S11=%g\n",S11);
    printf("S22=%g\n",S22);
    printf("S12=%g\n",S12);
    
      
    printf("i2=%ld\n",i20);
  }
  //
  //for(int i=0;i<3;i++){
  RN.setColumn(i1,RN.column(i1)+RN.column(i2)*i20);
    //}
  
  if(prot){
    Matrix3x3 a=A*RN;
    Matrix3x3 S=a.transposed()*a;
    printf("\nAA=\n%s\n",a.transposed().printS("%12.5g")());
    printf("\nS=\n%s\n",S.printS("%12.5g")());
    printf("smin=%g %g\n",smin,trace(S));
    
    printf("\nminl12 END\n");
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::cellReduce2D(const Matrix3x3 &A,IMatrix3x3 &RN,
				       double tol,bool prot)
{
  double s1;



  double S11=(A*RN.column(0))*(A*RN.column(0));
  double S22=(A*RN.column(1))*(A*RN.column(1));

  double smin=S11+S22;
  //
  if(prot){
    printf("smin0=%12.5g\n",smin);
    printf("cellReduce2D a1: %s: %12.5g\n",(A*RN.column(0)).printS("%12.5g")(),sqrt(S11));
    printf("cellReduce2D a2: %s: %12.5g\n",(A*RN.column(1)).printS("%12.5g")(),sqrt(S22));
    printf("cellReduce2D a3: %s: %12.5g\n",(A*RN.column(2)).printS("%12.5g")(),sqrt(S22));
  }
  //
  while(true){
    if(prot){
      printf("sminx=%12.5g\n",smin);
      printf("cellReduce2D a1: %s: %12.5g\n",(A*RN.column(0)).printS("%12.5g")(),sqrt(S11));
      printf("cellReduce2D a2: %s: %12.5g\n",(A*RN.column(1)).printS("%12.5g")(),sqrt(S22));
      printf("cellReduce2D a3: %s: %12.5g\n",(A*RN.column(2)).printS("%12.5g")(),sqrt(S22));
    }
    S11=(A*RN.column(0))*(A*RN.column(0));
    S22=(A*RN.column(1))*(A*RN.column(1));
    smin=S11+S22;

    if(S11>=S22){
      minl12(A,RN,0,1,s1,prot);
      if(prot) printf("s1=%12.5g\n",s1);
    }else if(S22>=S11){
      minl12(A,RN,1,0,s1,prot);
      if(prot) printf("s2=%12.5g\n",s1);
    }
    if(prot) printf("smin=%12.5g\n",smin);
    //     
    if(fabs(s1-smin)<tol) break;
  }
  

  S11=(A*RN.column(0))*(A*RN.column(0));
  S22=(A*RN.column(1))*(A*RN.column(1));



  if(prot){
    printf("smin0=%12.5g\n",smin);
    printf("cellReduce2D a1: %s: %12.5g\n",(A*RN.column(0)).printS("%12.5g")(),sqrt(S11));
    printf("cellReduce2D a2: %s: %12.5g\n",(A*RN.column(1)).printS("%12.5g")(),sqrt(S22));
    printf("cellReduce2D a3: %s: %12.5g\n",(A*RN.column(2)).printS("%12.5g")(),sqrt(S22));
  }
  
  
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::minimizeVectorIn2D(const Matrix3x3 &A,
						 const IMatrix3x3 &RN,
						 long &it0,long &it1,long n,
						 bool prot)
{
  //n*V=A*RN[0]*it0+A*RN[1]*it1
  //find smallest possible V via changing it0,it1 in steps of n
  
  if(prot) printf("minimizeVectorIn2D: it0=%ld it1=%ld\n",it0,it1);

  // |V'|= |(A*RN[0]*(it0+n0)+A*RN[1]*(it1+n1))|
  
  Vec3 b1=A*RN.column(0);
  Vec3 b2=A*RN.column(1);
  
  double S11=b1*b1;
  double S22=b2*b2;

  bool changed=true;

  while(changed){
    changed=false;

    Vec3 V0=(b1*it0+b2*it1)/n;
    if(prot) printf("minimizeVectorIn2D: V0=%s\n",V0.printS()());

    double dV0=0;
    long n0=lRound(-(V0*b1)/S11);
    long i0=0;
    for(int i=n0-1;i<=n0+1;i++){
      double dV=(2*(V0*b1)+S11*i)*i;
      if(dV<dV0){
	i0=i;
	dV0=dV;
	changed=true;
      }
    }
    it0+=i0*n;
    
    V0=(b1*it0+b2*it1)/n;
    
    dV0=0;
    n0=lRound(-(V0*b2)/S22);
    i0=0;
    for(int i=n0-1;i<=n0+1;i++){
      double dV=(2*(V0*b2)+S22*i)*i;
      if(dV<dV0){
	i0=i;
	dV0=dV;
	changed=true;
      }
    }
    it1+=i0*n;
  }


  if(prot) printf("minimizeVectorIn2D: it0=%ld it1=%ld\n",it0,it1);
}
/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::subLatticeForAxis(const Vec3 &axis,long n,
						const Matrix3x3 &A,
						const Matrix3x3 &G,
						OperationLatticeHelper &olh,
						bool prot)
{
  // given an n-fold axis, and a lattice basis A (G=A^{-T}), find the shortes 
  // lattice vector  along the axis and perpendicular to the axis
  // the perp axes span the smallest possible in-plane lattice cell
  // return a Matrix3x3 where the last column is the axis-parallel and 
  // 0th, 1st column are the in plane vectors.
  // 
  NSF("SpaceGroupOperationBase::subLatticeForAxis");
  

  double tol=1.0e-6;
  RAlongAxis(axis,n,A,G,olh.RNMIN,tol);
  RAlongAxis(axis,n,G,A,olh.GNMIN,tol);
  olh.PMIN_GC=labs(olh.RNMIN*olh.GNMIN);

  if(prot){
    printf("axis=%s\n",axis.printS()());
    printf("RNMIN=%ld %ld %ld=%g %g %g\n",
	   olh.RNMIN[0],olh.RNMIN[1],olh.RNMIN[2],
	   (A*olh.RNMIN)[0],(A*olh.RNMIN)[1],(A*olh.RNMIN)[2]);
    printf("GNMIN=%ld %ld %ld=%g %g %g\n",
	   olh.GNMIN[0],olh.GNMIN[1],olh.GNMIN[2]
	   ,(G*olh.GNMIN)[0],(G*olh.GNMIN)[1],(G*olh.GNMIN)[2]);
    printf("PMIN_GC=%ld\n",olh.PMIN_GC);
  }
  /* PMIN_GC must be a prime factor of n */
  long i=n/olh.PMIN_GC;
  
  if( i * olh.PMIN_GC !=n )
    throw Except(Sprintf("%s: n (RC|G2)/(Pmin|G2) not integer ??\n ",
			 nsubfunc));

  
  
  //   printf(" 2 (RC|G2)/(PNMIN|G2)=%ld\n",2/symop->PMIN_GC); 
  // create perp unit cell
  SpaceGroupOperationBase::latticePlane(olh.GNMIN,olh.RN,olh.GN,prot);

  {
    // make sure RN[1,2] forms smallest possible planar basis in the sense
    // that accute cells are avoided
    
    bool prot1=false;
    if(prot1){
      printf("subLatticeForAxis: A*RN[0]=%g %g %g\n"
	     ,(A*olh.RN.column(0))[0],(A*olh.RN.column(0))[1],(A*olh.RN.column(0))[2]);
      printf("subLatticeForAxis: A*RN[1]=%g %g %g\n"
	     ,(A*olh.RN.column(1))[0],(A*olh.RN.column(1))[1],(A*olh.RN.column(1))[2]);
      printf("subLatticeForAxis: G*GN[0]=%g %g %g\n"
	     ,(G*olh.GN.column(0))[0],(G*olh.GN.column(0))[1],(G*olh.GN.column(0))[2]);
      printf("subLatticeForAxis: G*GN[1]=%g %g %g\n"
	     ,(G*olh.GN.column(1))[0],(G*olh.GN.column(1))[1],(G*olh.GN.column(1))[2]);
      printf("subLatticeForAxis: RN1*GN1=%ld\n",olh.RN.column(0)*olh.GN.column(0));
      printf("subLatticeForAxis: RN1*GN2=%ld\n",olh.RN.column(0)*olh.GN.column(1));
      printf("subLatticeForAxis: RN2*GN2=%ld\n",olh.RN.column(1)*olh.GN.column(1));
      printf("subLatticeForAxis: RN2*GN1=%ld\n",olh.RN.column(1)*olh.GN.column(0));
    }


    cellReduce2D(A,olh.RN,1.0e-6,prot1);

    olh.GN.setColumn(0,olh.RN.column(1).cross(olh.RN.column(2)));
    olh.GN.setColumn(1,olh.RN.column(2).cross(olh.RN.column(0)));

    olh.GN.scaleColumn(0,lSign(1,olh.GN.column(0)*olh.RN.column(0)));
    olh.GN.scaleColumn(1,lSign(1,olh.GN.column(1)*olh.RN.column(1)));


    if(prot1){
      printf("subLatticeForAxis: A*RN[0]=%g %g %g\n"
	     ,(A*olh.RN.column(0))[0],(A*olh.RN.column(0))[1],(A*olh.RN.column(0))[2]);
      printf("subLatticeForAxis: A*RN[1]=%g %g %g\n"
	     ,(A*olh.RN.column(1))[0],(A*olh.RN.column(1))[1],(A*olh.RN.column(1))[2]);
      printf("subLatticeForAxis: G*GN[0]=%g %g %g\n"
	     ,(G*olh.GN.column(0))[0],(G*olh.GN.column(0))[1],(G*olh.GN.column(0))[2]);
      printf("subLatticeForAxis: G*GN[1]=%g %g %g\n"
	     ,(G*olh.GN.column(1))[0],(G*olh.GN.column(1))[1],(G*olh.GN.column(1))[2]);
      printf("subLatticeForAxis: RN1*GN1=%ld\n",olh.RN.column(0)*olh.GN.column(0));
      printf("subLatticeForAxis: RN1*GN2=%ld\n",olh.RN.column(0)*olh.GN.column(1));
      printf("subLatticeForAxis: RN2*GN2=%ld\n",olh.RN.column(1)*olh.GN.column(1));
      printf("subLatticeForAxis: RN2*GN1=%ld\n",olh.RN.column(1)*olh.GN.column(0));
    }

  }



  if( mhnorm(olh.GNMIN-olh.GN.column(2))>0) {
    printf("Gmin=%ld %ld %ld\n"
	   "G3  =%ld %ld %ld\n"
	   ,olh.GNMIN[0],olh.GNMIN[1],olh.GNMIN[2]
	   ,olh.GN(0,2),olh.GN(1,2),olh.GN(2,2)
	   );
    throw Except(Sprintf("%s: n Gmin!=G3 ??\n ",nsubfunc));
  }

}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperationBase::prepareSetUpOrigin(const Vec3 &axis,long order,
						 const Matrix3x3 &cell,
						 OperationLatticeHelper &olh)
{
  bool prot=true;
  
  Matrix3x3 A=cell; // each column is a vector
  Matrix3x3 G=A.inverseT();

  Vec3 tauG=(poptau*G);

  
  {
    /* 
       rp^T=r^T*popmat in expansion of the lattice vectors r=cell*rc:
       (cell*rpc)^T=(cell*rc)^T*popmat
       rpc^T*cell^T=(rc^T*cell^T)*popmat
       cell^(-T)=rcell
       rpc^T=rc^T*(cell^T*popmat*rcell)
    */
    Matrix3x3 tmp=G.transposed()*popmat.transposed()*cell;
    olh.iopmat=round(tmp);
    if(Fnorm(olh.iopmat-tmp)>1.0e-6){
      myDebug()<<"cell.transposed()*popmat*G"
	      <<G.transposed()*popmat.transposed()*cell;
      //      myDebug()<<"Iopmat                    "<<olh.iopmat;
      throw Except("SpaceGroupOperationBase::prepareSetUpOrigin: "
		   "iopmat is not integer?");
    }
  }  
  subLatticeForAxis(axis,order,A,G,olh,prot);

  
  if(prot) {
    printf("R0  =%s\n",(A*olh.RN.column(0)).printS()());
    printf("R1  =%s\n",(A*olh.RN.column(1)).printS()());
    printf("R2  =%s\n",(A*olh.RN.column(2)).printS()());
    printf("PMIN=%s\n",(A*olh.RNMIN).printS()());
    
    printf("G0  =%s\n",(G*olh.GN.column(0)).printS()());
    printf("G1  =%s\n",(G*olh.GN.column(1)).printS()());
    printf("G2  =%s\n",(G*olh.GN.column(2)).printS()());
    
    IMatrix3x3 u=olh.RN.transposed()*olh.GN;
    
    printf("1=%s\n",u.printS()());
  }
  
  // The first lattice vector in axis direction is PMIN=A*RNMIN.
  // The perpendicular planes are spanned by R0=A*RN[0] and R1=A*RN[1].
  // The first neighbor plane is going throug R2=A*RN[2].
  // These three Ri span a minimal unit cell. The corresopnding reciprocal
  // cell is Gi=G*GN[i] with the property GN[i]*RN[j]=delta_ij.
  // GMIN=G[2] parallel PMIN being the shortest reciprocal vector along the 
  // axis.
  // The bravais lattice properties as related to the axis is given by
  // PMIN*GMIN=|PMIN|*|GMIN|=PMIN_GC, where PMIN_GC is a prime divisor
  // of the axis order. (Not all divisors are allowed, though).
  // Of course, R2*GMIN=RN[2]*GN[2]=1.
  //
  // n=2: PMIN_GC=1,2
  // n=3: PMIN_GC=1,3
  // n=4: PMIN_GC=1,2
  // n=6: PMIN_GC=1
  //
  // The axis normal can be represented as n=GMIN/|GMIN|=PMIN/|PMIN|.
  //
  //
  // tau can be projected onto axis and plane using this basis
  // tau=A*tauG
  // tau_par=tauGmin*PMIN

  olh.tauGmin=tauG*olh.GN.column(2)/olh.PMIN_GC;

  // tau_perp=(tau-tau_par)
  //         =(tauG*A-tauGmin*A*RNMIN)*Gi*Ri
  //         =(tauG*A-tauGmin*A*RNMIN)*G*GN[i]*(A*RN[i])
  //         =(tauGi)*(A*RN[i])

  olh.tauG0=(tauG-olh.tauGmin*olh.RNMIN)*olh.GN.column(0);
  olh.tauG1=(tauG-olh.tauGmin*olh.RNMIN)*olh.GN.column(1);
  
  // R2 can also be decomposed into this basis
  // R2_par=R2*GMIN/|GMIN|*PMIN/|PMIN|
  //       =RN[2]*GN[2]/PMIN_GC*PMIN
  //       =1/PMIN_GC*PMIN
  // R2_perp=(R2-R2_par)*Gi*Ri,  i=0,1
  //        = - R2_par*Gi*Ri
  //        = - 1/PMIN_GC*PMIN*Gi*Ri
  //        = - 1/PMIN_GC*(RNMIN*GN[i])*Ri
  //        = - NPi/PMIN_GC*Ri
  
  olh.NP0=olh.RNMIN*olh.GN.column(0);
  olh.NP1=olh.RNMIN*olh.GN.column(1);
  if(prot)printf("NP0=%ld NP1=%ld\n",olh.NP0,olh.NP1);

}


/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpaceGroupInversion::setUpOrigin(const Matrix3x3 &cell)
{
  NSF("SpaceGroupInversion::setUpOrigin");
  //
  bool prot=true;
  
  Matrix3x3 A=cell; // each column is a vector

  
  if(prot)printf("--------------------------------------\n"
		 "Op=%s\n"
		 "--------------------------------------\n",psymbol());


  // O=porigin+R/2
  porigin=poptau*0.5;
  pshiftperiods[0]=A(0)*0.5;
  pshiftperiods[1]=A(1)*0.5;
  pshiftperiods[2]=A(2)*0.5;

  
  pissymmorphic=true;
  

  
  if(prot){
    printf("%s",printS()());
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SpaceGroupInversion::printS()const
{
  String s=Sprintf("\n sym   : %s"
		   "@ %s \n",
		   psymbol(),porigin.printS("%10.5f").stripBlanks()()
		   );
  for(int i=0;i<3;i++){
    s+=Sprintf(" periodic with R%d=%s\n",i+1,
	       pshiftperiods[i].printS("%12.5f")());
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpaceGroupMirror::setUpOrigin(const Matrix3x3 &cell)
{
  NSF("SpaceGroupMirror::setUpOrigin");
  //
  bool prot=true;
  
  if(prot)printf("--------------------------------------\n"
		 "Op=%s\n"
		 "--------------------------------------\n",psymbol());
  
  OperationLatticeHelper olh;
  prepareSetUpOrigin(paxis,2,cell,olh);  

  
  porigins.clear();
  ptaus.clear();
  pissymmorphic=false;

  
  
  if(prot)printf("p=%ld\n",olh.PMIN_GC);
  pshiftperiod=cell*olh.RNMIN*0.5;
  olh.tauGmin-=dRound(olh.tauGmin);
  long it0=dRound(2*olh.tauG0);
  long it1=dRound(2*olh.tauG1);
  if(fabs(it0-2*olh.tauG0)>1.0e-6)
    throw Except("mirror: essential shift invalid?!\n");
  if(fabs(it1-2*olh.tauG1)>1.0e-6)
    throw Except("mirror: essential shift invalid?!\n");
  it0=(it0%2?1:0);
  it1=(it1%2?1:0);
  porigins.append(olh.tauGmin*pshiftperiod);
  if(olh.PMIN_GC==1){
    // only one origin
    ptaus.append((cell*olh.RN.column(0)*it0+cell*olh.RN.column(1)*it1)*0.5);
    pissymmorphic|=(norm(ptaus.last())<1.0e-5);
  }else if(olh.PMIN_GC==2){
    if(prot){
      printf("mirror case p=2\n");
//       printf("mirror case p=2: GN0=%g %g %g\n"
// 	     ,(G*olh.GN[0])[0],(G*olh.GN[0])[1],(G*olh.GN[0])[2]);
//       printf("mirror case p=2: GN1=%g %g %g\n"
// 	     ,(G*olh.GN[1])[0],(G*olh.GN[1])[1],(G*olh.GN[1])[2]);
      printf("mirror case p=2: PMin*G[0,1]=[%ld,%ld]\n"
	     ,olh.NP0%2,olh.NP1%2);
    }
    if((olh.NP0%2!=0) || (olh.NP1%2!=0)){
      // two origins
      if(prot)printf("mirror case p=2: two solutions\n");
      // 	  printf("A*RN[0]=%g %g %g\n"
      // 		 ,(A*RN[0])[0],(A*RN[0])[1],(A*RN[0])[2]);
      // 	  printf("A*RN[1]=%g %g %g\n"
      // 	     ,(A*RN[1])[0],(A*RN[1])[1],(A*RN[1])[2]);
      
      //printf("it12=%ld %ld\n",it0,it1);
      minimizeVectorIn2D(cell,olh.RN,it0,it1,2,prot);
      //printf("it12=%ld %ld\n",it0,it1);

      ptaus.append((cell*(olh.RN.column(0)*it0+olh.RN.column(1)*it1))*0.5);
      pissymmorphic|=(norm(ptaus.last())<1.0e-5);

      it0=((it0+olh.NP0)%2?1:0);
      it1=((it1+olh.NP1)%2?1:0);
      //printf("it12=%ld %ld\n",it0,it1);
      minimizeVectorIn2D(cell,olh.RN,it0,it1,2,prot);
      //printf("it12=%ld %ld\n",it0,it1);
      porigins.append((olh.tauGmin+0.5)*pshiftperiod);
      ptaus.append((cell*((olh.RN.column(0)*it0)+(olh.RN.column(1)*it1)))*0.5);
      pissymmorphic|=(norm(ptaus.last())<1.0e-5);
    }else{
      throw Except("mirror case p=2: one solution\n");
    }
  }else{
    throw Except("invalid prime divisor for mirror plane\n");
  }
  
  
  

  if(porigins.size()>1){
    if(norm(ptaus[0])>norm(ptaus[1])){
      Vec3 d=porigins[0];porigins[0]=porigins[1];porigins[1]=d;
      d=ptaus[0];ptaus[0]=ptaus[1];ptaus[1]=d;
      
    }
  }
  
  
  if(prot){
    printf("%s",printS()());
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SpaceGroupMirror::printS()const
{
  String s;
  for(int i=0;i<porigins.size();i++){
    s+=Sprintf("\n %s: {%s"
	       "|%s} @ %s\n",
	       (pissymmorphic?"sym   ":"nonsym"),psymbol(),
	       ptaus.at(i).printS("%10.5f").stripBlanks()(),
	       porigins.at(i).printS("%10.5f").stripBlanks()());
    
  }
  s+=Sprintf(" perdiodic with %s\n",
	     pshiftperiod.printS("%10.5f").stripBlanks()());
  
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpaceGroupRotation::setUpOrigin(const Matrix3x3 &cell)
{
  NSF("SpaceGroupRotation::setUpOrigin");
  //
  bool prot=true;
  

  
  if(prot)printf("--------------------------------------\n"
		 "Op=%s\n"
		 "--------------------------------------\n",psymbol());

  
  prepareSetUpOrigin(paxis,pn,cell,polh);  

  long labeta;
  if(pn==2){
    labeta=9;
  }else if(pn==3){
    labeta=8;
  }else if(pn==4){
    labeta=6;
  }else if(pn==6){
    labeta=0;
  }else{
    throw Except(Sprintf("%s: unknown rotation order",nsubfunc));
  }

  if(fabs((12-labeta)/12.-0.5/(1.-cos(2*Pi*pm/pn)))>1.0e-6)
    throw Except(Sprintf("%s: something wrong C%ld: 12*labeta(%ld)!=%g",
			 nsubfunc,pn,labeta,12.*(1-0.5/(1-cos(2*Pi*pm/pn)))));

  // 12 times the matrix which makes the perp periods out of the perp R
  IMatrix3x3 IDelta12=IMatrix3x3::unitMatrix(labeta)+polh.iopmat*(12-labeta);


  pissymmorphic=false;
  porigins.clear();
  pmtaus.clear();
  


  long km=lRound(pn*polh.tauGmin);
  while(km<0){km+=pn;} km=km%pn;
  long d=MyMath::gcd(km,pn);
  km/=d;
  long kn=pn/d;
  
  


  
  long idet;
  IMatrix3x3 IDelta12inv=IDelta12.inversetV(idet);
  //  myDebug()<<"IDelta12     ="<<IDelta12;
  myDebug()<<"det(IDelta12)="<<idet;
  //  myDebug()<<"IDelta12inv  ="<<IDelta12inv;
  //myDebug()<<IDelta12*(IMatrix3x3::unitMatrix(1)-polh.iopmat);
  //  myDebug()<<"IDelta12*IDelta12inv"<<IDelta12*IDelta12inv;

  pdelta[0]=IDelta12*polh.RN.column(0);
  pdelta[1]=IDelta12*polh.RN.column(1);

  myDebug()<<"Delta"<<pdelta[0]<<pdelta[1];
  myDebug()<<"RN"<<polh.RN.column(0)<<polh.RN.column(1);

  IMatrix3x3 mi=polh.GN.transposed()*IDelta12inv*polh.RN;
  long gcdmi=MyMath::gcd(MyMath::gcd(mi(0,0),mi(1,0)),
			 MyMath::gcd(mi(0,1),mi(1,1)));
  //  myDebug()<<"mi"<<mi;
  if((12*gcdmi)%idet!=0){
    myDebug()<<"12*gcdmi^3"<<12*gcdmi<<idet;
    throw Except("IDelta12inv: something is wrong with the integer assumption");
  }
  mi*=12;// now it is dividableby idet
  mi/=idet;
  mi(2,0)=0;
  mi(2,1)=0;
  mi.setColumn(2,IVec3(0,0,1));
  //  myDebug()<<"mi"<<mi;
  //pdelta*mi=polh.RN;
  IMatrix3x3 mdelta;
  mdelta.setColumn(0,pdelta[0]);
  mdelta.setColumn(1,pdelta[1]);
  // now pdelta*mi=12*RN
  if(mhnorm(polh.RN.column(0)*12L-(mdelta*mi).column(0))
     +mhnorm(polh.RN.column(1)*12L-(mdelta*mi).column(1))!=0){
    throw Except("inverse delta-RN relation failed");
  }
  //  myDebug()<<"mdelta"<<mdelta;
  myDebug()<<polh.RN.column(0)<<(mdelta*mi).column(0);
  myDebug()<<polh.RN.column(1)<<(mdelta*mi).column(1);

//   long D1=labs(MyMath::gcd(mi[0][0],mi[1][0]));
//   long D2=labs(MyMath::gcd(mi[0][1],mi[1][1])*mi.determinant());


  //todo backfold delta
  
  
  //origin from tau_perp
  Vec3 os=pdelta[0]*polh.tauG0+pdelta[1]*polh.tauG1;
  
  porigins.append(cell*os/12.);
  pmtaus.append((pn/kn)*km);
  pissymmorphic|=(km==0);  

  
  if(polh.PMIN_GC!=1){
    // add R2 to tau and to origin
    
    // R2_par=PMIN/p
    // R2_par=l/n*PMIN
    // l=n/p
    // hence R2 changes tau0 by (n/p)/n
    
    km*=(pn/kn);
    kn=pn;
    long km0=km;
    long changeunit=pn/polh.PMIN_GC;
    
    while(1){
      km*=(pn/kn);
      kn=pn;
      km+=changeunit;
      while(km<0){km+=pn;} km=km%pn;
      if(km==km0) break;
      long d=MyMath::gcd(km,pn);
      km/=d;
      kn=pn/d;
      
      
      // R2_perp must be added to the origin
      
      double t0=-1.*polh.NP0/polh.PMIN_GC;
      double t1=-1.*polh.NP1/polh.PMIN_GC;
      
      os+=pdelta[0]*t0+pdelta[1]*t1;
      
      porigins.append(cell*os/12.);
      pmtaus.append((pn/kn)*km);
      pissymmorphic|=(km==0);  
    }
    
  }
  

  
  

  for(int i=0;i<porigins.size();i++){
    for(int j=i+1;j<porigins.size();j++){
      if(abs(pmtaus[j])<abs(pmtaus[i])){
	Vec3 d=porigins[i];porigins[i]=porigins[j];porigins[j]=d;
	long l=pmtaus[i];pmtaus[i]=pmtaus[j];pmtaus[j]=l;
      }
    }
  }
  

  
  

  
  if(prot){
    printf("%s",printS()());
  }
  //todo  printf("unfolded: D1=%ld,D2=%ld\n",D1,D2);
//   for(long i0=0;i0<D1;i0++){
//     for(long i1=0;i1<D2;i1++){
//       printf(" shift=%s",(pdelta[0]*i0+pdelta[1]*i1).printS()());
//     }
//   }
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupRotation::expand(const Matrix3x3 &/*cell*/)
{
//   double tol=1.0e-4;
//   Matrix3x3 G=cell.inverseT();
//   Matrix3x3 g=Matrix3x3(pshiftperiods[0],pshiftperiods[1],paxis).inverseT();
//   //pshiftperiods[0]=cell*pdelta[0]/12.;
//   //pshiftperiods[1]=cell*pdelta[1]/12.;

//   for(int i=0;i<porigins.size();i++){
//     myDebug()<<"expanding Rotation"<<psymbol,pmtaus[i];
//     double dNmin[2]={0,0};
//     double dNmax[2]={0,0};
//     long Nmin[2];
//     long Nmax[2];
//     for(int j=0;j<2;j++){
//       Vec3 v=g(j)*cell;
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[0]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[0]);
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[1]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[1]);
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[0]+v[1]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[0]+v[1]);
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[2]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[2]);
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[2]+v[0]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[2]+v[0]);
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[2]+v[1]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[2]+v[1]); 
//       dNmin[j]=MyMath::min<double>(dNmin[j],v[2]+v[0]+v[1]);
//       dNmax[j]=MyMath::max<double>(dNmax[j],v[2]+v[0]+v[1]);
//       Nmin[j]=floor(dNmin[j])-1;
//       Nmax[j]=ceil(dNmax[j])+1;
//       myDebug()<<Nmin[j]<<Nmax[j];
//    }
//     for(int i0=Nmin[0];i0<=Nmax[0];i0++){
//       for(int i1=Nmin[1];i1<=Nmax[1];i1++){
// 	Vec3 v=(pshiftperiods[0]*i0+pshiftperiods[1]*i1)*G;
//   //pshiftperiods[0]=cell*pdelta[0]/12.;
//   //pshiftperiods[1]=cell*pdelta[1]/12.;

// 	if( (-tol<v[0] && v[0]<1.+tol) &&
// 	    (-tol<v[1] && v[1]<1.+tol) &&
// 	    (-tol<v[2] && v[2]<1.+tol) ){
// 	  myDebug()<<v.printS("%10.5f");
// 	}
//       }
//     }
//   }
  
    
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SpaceGroupRotation::printS()const
{
  String s;
  for(int i=0;i<porigins.size();i++){
    s+=Sprintf("\n %s: {%s"
	       "|%ld/%ld} @ %s\n",
	       (pissymmorphic?"sym   ":"nonsym"),psymbol(),
	       pmtaus.at(i),pn,
	       porigins.at(i).printS("%10.5f").stripBlanks()());
    
  }
  for(int i=0;i<2;i++){
    IVec3 d=pdelta[i];
    long D=labs(d.gcd());
    d/=D;
    long D12=labs(MyMath::gcd(D,12));
    d*=(D/D12);
    long q=12/D12;
    s+=Sprintf(" periodic with=A*[%ld/%ld,%ld/%ld,%ld/%ld]\n",
	       d[0],q,d[1],q,d[2],q);
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */

void SpaceGroupRotoInversion::setUpOrigin(const Matrix3x3 &cell)
{
  NSF("SpaceGroupRotoInversion::setUpOrigin");
  //
  bool prot=true;
  

  if(prot)printf("--------------------------------------\n"
		 "Op=%s\n"
		 "--------------------------------------\n",psymbol());

  // convert to mirrorrotations for sorting out the stuff
  //  S_n^m=I*C2*C_n^m=I* (n+2m)/2n
  long ppm=pn+2*pm;
  long ppn=2*pn;
  long d=MyMath::gcd(ppn,ppm);
  ppn/=d;
  ppm=(ppm/d)%ppn;

  double csbeta=cos(2*Pi*ppm/ppn);

  OperationLatticeHelper olh;
  if((ppn%2)==0){
    // n even
    prepareSetUpOrigin(paxis,ppn/2,cell,olh);  
  }else{
    // n odd
    prepareSetUpOrigin(paxis,ppn,cell,olh);  
  }

  

  
  porigins.clear();
  pissymmorphic=true;
  
  



  ppmin=(cell*olh.RNMIN);
  
  Vec3 r=(cell*olh.RN.column(0));
  Vec3 Cr=r*popmat;
  Vec3 delta=((1.-2.*csbeta)*r+Cr)*0.5/(1.-csbeta);
  pshiftperiods[0]=delta;
  
  r=(cell*olh.RN.column(1));
  Cr=r*popmat;
  delta=((1.-2.*csbeta)*r+Cr)*0.5/(1.-csbeta);
  pshiftperiods[1]=delta;
  
  
  
  double oparrel=olh.tauGmin*0.5;
  
  // tau_perp
  r=(cell*olh.RN.column(0)*olh.tauG0+cell*olh.RN.column(1)*olh.tauG1);
  Cr=r*popmat;
  Vec3 os=((1.-2.*csbeta)*r+Cr)*0.5/(1.-csbeta);
  porigins.append(os+ppmin*oparrel);
  
  
  
  if(olh.PMIN_GC!=1){
    // add R2 to tau and to origin
    r=-((cell*olh.RN.column(0))*olh.NP0+(cell*olh.RN.column(1))*olh.NP1)/olh.PMIN_GC;
    Cr=r*popmat;
    Vec3 dtau=((1.-2.*csbeta)*r+Cr)*0.5/(1.-csbeta);
    double dtaupar=0.5/olh.PMIN_GC;
    for(long l=1;l<olh.PMIN_GC;l++){
      os+=dtau;
      oparrel+=dtaupar;
      porigins.append(os+ppmin*oparrel);
    }
  }
  

    
  
  

  
  
  if(prot){
    printf("%s",printS()());
  }
  
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
String SpaceGroupRotoInversion::printS()const
{
  String s;
  for(int i=0;i<porigins.size();i++){
    s+=Sprintf("\n %s: %s"
	       "  @ %s; %10.5f\n",
	       (pissymmorphic?"sym   ":"nonsym"),psymbol(),
	       porigins.at(i).printS("%10.5f").stripBlanks()(),
	       (porigins.at(i)*ppmin)/(ppmin*ppmin));
    
  }
  for(int i=0;i<2;i++){
    s+=Sprintf(" periodic with=%s\n",
	       pshiftperiods[i].printS("%12.5f").stripBlanks()());
  }
  return s;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
bool SpaceGroupOperations::
hasType(SpaceGroupOperationBase::SpaceGroupOperationType t)
  const
{
  bool ret=false;
  for(int ig=0;ig<pspacegroupoperations.size();ig++){
    ret|=(pspacegroupoperations.at(ig)->type()==t);
  }
  return ret;
}

/* ***********************************************************************
 *
 * ***********************************************************************
 */
void SpaceGroupOperations::unfold()
{
  printf("SpaceGroupOperations::unfold()\n");
  for(int ig=0;ig<pspacegroupoperations.size();ig++){
    SpaceGroupOperationBase *spop=pspacegroupoperations[ig];
    printf("%s\n",spop->printS()());
    spop->expand(pcell);
  }
  
}
/*EOF*/ 
