#! /usr/bin/perl
#
#  downgrad fplo input files via brute force, this may result in data loss
#
########################################################################


use strict;

########################################################################
# all input files, whose version number will just be changed
# no data downgrading
my @FILES=(  "=.bp", "=.xef"  , "=.dirac" , "=.devel" );
# special treatment for basis files, depending on version number
my $INFILE="=.in";
my $SYMFILE="=.sym";
my $BASISFILE="=.basis";
my $DENSFILE="=.dens";
########################################################################




########################################################################
# temporary file
#my $tmpfile="/tmp/tfdwn$$";
my $tmpfile="tfdwn$$";
my $tmpfile2="tfdwn2.$$";

########################################################################




########################################################################
# greetings
print <<EOF
########################################################################

		!!!!    WARNING    !!!!

	Downgrading input files may result in data loss!

	Do you want to continue ?
EOF
;
print "[yes/no]:" ; my $yn=<STDIN>; $yn=~s/\s+$//g;
if ( $yn  ne  "yes" ){ &EXIT("Abort"); }
print "\n\n";
########################################################################




########################################################################
# request for new version number
print  "Downgrade to which main version? [n.nn] :" ; my $newversion=<STDIN>;  $newversion=~s/\s+$//g;
print "\n\n";
########################################################################



my $BACKUPDIR="fdowngrad_backup";

my $n=1;
while ( -d $BACKUPDIR.$n ){ $n++;}

$BACKUPDIR=$BACKUPDIR.$n;
mkdir $BACKUPDIR;


########################################################################
# special treatment for downgrading =.dens
#
{
  my $file=$DENSFILE;
  if (-f $file){

    system ("cp $file $BACKUPDIR/$file");

    (my $fileversion,my $subver,my $by) = &get_file_version($file);

    
    next if(! &file_newer_than_downgrad($file,$fileversion,$newversion));


    print "file '$file' from version $fileversion to version $newversion\n";

    my $critver9=9.061;
    
    if(  ($newversion<3.60 && ($fileversion>=3.599)) || 
         ($newversion<$critver9 && ($fileversion>=$critver9 )) ){


      my $vr="";
      $by=~s/\s+//g;
      $by=~s/.*\((.*)\).*/$1/g;
      if ( $by =~ /\d+\.\d\d-\d+/){
	$vr=$by;
      }
      
      #print "ver=$fileversion\n";
      #print "subver=$subver\n";
      #print "by=$by\n";
      #print "vr=$vr\n";
      
      
      print <<EOF
########################################################################

		!!!!    WARNING    !!!!

	Downgrading the file $file needs an fplo executable
        whose version equals the version of the inputfiles ($fileversion).
        I found $file was modified by version/release \'$vr\'.

        I try to execute fplo$vr and the corresponding fedit to downgrad
        $file.

        Press <ENTER> or specify a complete version-release number
        of an fedit/fplo pair with version equal to $fileversion,
        which is accesible in the current PATH. Do not include 
        fplo or fedit, just give the
        version-release suffix of the installed executable.

EOF
;
      my $fplo;
      print "Press <ENTER> or type a version-release number (like i.e. $vr) :" ; my $yn=<STDIN>; $yn=~s/\s+$//g;
      if (  $yn ne ""   ){ $vr=$yn; };
      print "\n\n";
      if($fileversion>=$critver9){
	  print "using =.densconvert option\n";
	  open(FH,">=.densconvert");
	  my $vvv=$newversion;
	  $vvv=~/(\d+)\..*/;
	  print FH "$1\n";
	  close(FH);
	  system("fplo$vr");
	  system("rm =.densconvert");
      }else{
	  print "using EXPORT_V3_DENSFILE option\n";
	  system<<"EOF"
cat <<END | fedit$vr -p fplo$vr -pipe 2> ./+log 1>/dev/null
\@-\@
!EXPORT_V3_DENSFILE!+
\@x\@
\@q\@
END
fplo$vr
cat <<END | fedit$vr -p fplo$vr -pipe 2> ./+log 1>/dev/null
\@-\@
!EXPORT_V3_DENSFILE!-
\@x\@
\@q\@
END

EOF
;
        }
    }
    system ("cp $file $tmpfile");
    &set_version_and_finalize($file,$newversion);

    
  }
}
#
########################################################################

########################################################################
# downgrading without special treatment
foreach my $file (@FILES){
  if (-f $file){

    system ("cp $file $BACKUPDIR/$file");
    system ("cp $file $tmpfile");
  
    (my $fileversion,my $subver,my $by) =&get_file_version($file);

    next if(! &file_newer_than_downgrad($file,$fileversion,$newversion));

    print "file '$file' from version $fileversion to version $newversion\n";

    &set_version_and_finalize($file,$newversion);

  }
  
}
########################################################################

########################################################################
#special treatment for downgrading =.in
{
  my $file=$INFILE;


  if (-f $file){
    
    system ("cp $file $BACKUPDIR/$file");
    system ("cp $file $tmpfile");
    
    (my $fileversion,my $subver,my $by) =&get_file_version($file);
    
    next if(! &file_newer_than_downgrad($file,$fileversion,$newversion));
    
    print "file '$file' from version $fileversion to version $newversion\n";
    
    
    if ($newversion<9.02 && ($fileversion>=9.01)) {
      eval `fpiotest -mode REMOVE -var grids.energywindows -f $tmpfile -value $newversion > $tmpfile2`;
      if (($?==256)) { 
	system ("mv $tmpfile2 $tmpfile");
      }
      eval `fpiotest -mode REMOVE -var grids.kresolved -f $tmpfile -value $newversion > $tmpfile2`;
      if (($?==256)) { 
	system ("mv $tmpfile2 $tmpfile");
      }
      eval `fpiotest -mode REMOVE -var grids.scfwavefnctwindow -f $tmpfile -value $newversion > $tmpfile2`;
      if (($?==256)) { 
	system ("mv $tmpfile2 $tmpfile");
      }

    }

    if ($newversion<7.03 && ($fileversion>=7.03)) {

      system (" sed -e \"s/CALC_PLASMON_FREQ/NOT_USED/g\" $tmpfile >$tmpfile2");
      system "mv $tmpfile2 $tmpfile";
    }

    if ($newversion<6.00 && ($fileversion>=6.00)) {
      
      my $maxl=&get_scalar_variable($file,"max_L");
      my $ratioU=&get_scalar_variable($file,"iterationvector_ratio.U");

      
      #print "max_L=$maxl\n";


      &set_scalar_variable($file,"max_L",12);


      system (" sed -e \"s/iterationvector_ratio/iterationvector_ratio_dummy/g\" $tmpfile >$tmpfile2");
      system "mv $tmpfile2 $tmpfile";

      system <<EOF
cat $tmpfile > $tmpfile2



cat <<EOFF >> $tmpfile2

   struct {int nxi;int neta;} cryfld_integrals
    ={20,26};

   struct {int nit;int degree;int nvg;logical sym;} exc_calc
    ={150,4,13,t};


   struct {int nr;} three_center_integrals
    ={40};

   struct {int nxi;int neta;} two_center_integrals
    ={26,56};

   struct {real exponent;} confining_potential
    ={4.0};

   struct {int nit;int degree;int nvg;logical sym;logical rot;} vxc_calc
    ={80,4,13,t,t};

   struct {int nit;int nvg;logical sym;logical rot;} ewp_calc
    ={80,13,t,t};

    struct {int nr;int neta;} vatcr_calc
    ={50,50};

    struct {real x0;real U;} iterationvector_ratio
    ={1.0,1.0};


section Vatom{

    struct {int type;char[*] description;} vatom_type
    ={3,"FPLO-3 (convoluted)"};


    struct {real r0;real rcut;} vatom_convolution
    ={1.5,1.0};

};

   struct {int mmin;int mmax;real tol;logical dv_wins;} basismapping
    ={2,2,1.0e-12,t};


section shape{

    struct {real rcut;int ncut;logical doc;int nprint;int nprint0;real tol;} voronoi_control
    ={100.,200,f,0,0,1.0e-8};


    struct {logical test;real tol;int nmesh;
        struct {int site;real vector[3];real distance;} directions[*];
    } shape_test
    ={f,1.e-13,200
        ,{
            {1,{1.,0.,0.},7.}
            ,{1,{1.,1.,0.},7.}
            ,{1,{1.,1.,1.},7.}
        }
    };

    int shape_stiffness=2;
    logical build_shape=t;
    char[*] voronoi_file="+voronoi";

    struct {int type;char[*] description;} shape_profile
    ={1,"Cosine"};

};



EOFF

mv $tmpfile2 $tmpfile
EOF
;



      &set_scalar_variable($file,"iterationvector_ratio.U",$ratioU);

      
      print "\n\n !!! Downgrading to this version means lots of information loss !!! \n\n\n";

    }

    if ($newversion<4.01 && ($fileversion>=4.01)) {
      
      my $lbound=&get_scalar_variable($file,"bandplot_control.emin");
      my $ubound=&get_scalar_variable($file,"bandplot_control.emax");

      
      #print "emin=$lbound\n";
      #print "emax=$ubound\n";


      my $hatoev=27.2116;

      
      &set_scalar_variable($file,"bandplot_control.emin",$lbound/$hatoev);
      &set_scalar_variable($file,"bandplot_control.emax",$ubound/$hatoev);


      my $ldos=&get_vector_variable($file,"ldossites");
      if(($#$ldos+1)>0){
	open(FH1,">=.ldos");
	print FH1 ($#$ldos+1)."\n";
	foreach my $l (@$ldos){
	  print FH1 "$l\n";
	}
	close(FH1);
      }


    }
    if ($newversion<10.00 && ($fileversion>=10.00)) {
	my $ti=&get_scalar_variable($file,"topological_insulator.active");
	if(uc($ti) eq "T"){
	    open(FH1,">=.ti");
	    printf( FH1 "#\n%s\n",$ti);
	    close(FH1);
	}
	eval `fpiotest -mode REMOVE -var topological_insulator -f $tmpfile -value $newversion > $tmpfile2`;
	if (($?==256)) { 
	    system ("mv $tmpfile2 $tmpfile");
	}

	my $ex=&get_scalar_variable($file,"ex_field_strength");
	if($ex<1.0){
	    open(FH1,">=.xreduce");
	    printf( FH1 "%s\n",eval($ex));
	    close(FH1);
	}
	eval `fpiotest -mode REMOVE -var ex_field_strength -f $tmpfile -value $newversion > $tmpfile2`;
	if (($?==256)) { 
	    system ("mv $tmpfile2 $tmpfile");
	}

	my $mo=&get_scalar_variable($file,"charges.chargemode.type");
	my $io=&get_scalar_variable($file,"charges.ionicity");
	if($mo==3){
	    if(abs($io)>1.0e-6){
		open(FH1,">=.mol_charge");
		printf( FH1 "%s\n",eval($io));
		close(FH1);
	    }
	}elsif($mo==2){
	    if(abs($io)>1.0e-6){
		open(FH1,">=.charging");
		printf( FH1 "\njellium\n\n%s\n",-eval($io));
		close(FH1);
	    }
	}

	my $makevca=&get_scalar_variable($file,"charges.make_vca");
	if(uc($makevca) eq "T"){
	    my $sorts=&get_vector_variable($file,"charges.vca.sort");
	    my $Zs=&get_vector_variable($file,"charges.vca.Z");
	    if(($#$sorts+1)>0){
		open(FH1,">=.atcharge");
		print FH1 ($#$sorts+1)."\n";
		for(my $i=0;$i<=$#$sorts;$i++){
		    print FH1 "$sorts->[$i] ".eval($Zs->[$i])."\n";
		}
		close(FH1);
	    }
	}

	eval `fpiotest -mode REMOVE -var charges -f $tmpfile -value $newversion > $tmpfile2`;
	if (($?==256)) { 
	    system ("mv $tmpfile2 $tmpfile");
	}
    }


    &set_version_and_finalize($file,$newversion);
  }
  
}
########################################################################


########################################################################
#special treatment for downgrading =.sym
{
  my $file=$SYMFILE;


  if (-f $file){
    
    system ("cp $file $BACKUPDIR/$file");
    system ("cp $file $tmpfile");
    
    (my $fileversion,my $subver,my $by) =&get_file_version($file);
    
    next if(! &file_newer_than_downgrad($file,$fileversion,$newversion));
    
    print "file '$file' from version $fileversion to version $newversion\n";
    
    if ($newversion<6.00 && ($fileversion>=6.00)) {
      
      my $maxl=&get_scalar_variable($file,"max_L");

      
      #print "max_L=$maxl\n";



      
      &set_scalar_variable($file,"max_L",12);


    }

    &set_version_and_finalize($file,$newversion);
  }
  
}
########################################################################

########################################################################
# special treatment for downgrading =.basis
{
  my $file=$BASISFILE;
  if (-f $file){

    system ("cp $file $BACKUPDIR/$file");
    system ("cp $file $tmpfile");
    
    (my $fileversion,my $subver,my $by) =&get_file_version($file);


    next if(! &file_newer_than_downgrad($file,$fileversion,$newversion));
  
    print "file '$file' from version $fileversion to version $newversion\n";
  
    if ($newversion<3.01 && ($fileversion>=3.01)) {
      &DowngradeBasisGe3_01LtXXX($file,$subver,0);
      system ("mv $tmpfile $file");
    } else {
      &set_version_and_finalize($file,$newversion);
    }

  }


}
########################################################################





########################################################################
# END of main code
print "\n\nIMPORTANT: Original files copied to backup directory '$BACKUPDIR'\n\n";
print "\n\n !!! Call $newversion fedit and use symmetry-update to regularize files !!! \n\n\n";
&EXIT("\nReady!");
########################################################################





########################################################################
sub EXIT(){
  my $txt=shift;
  printf "$txt\n";
  unlink $tmpfile;
  unlink $tmpfile2;
  exit;
}
########################################################################

########################################################################
sub DIE(){
  my $txt=shift;
  #printf "$txt\n";
  unlink $tmpfile;
  unlink $tmpfile2;
  die "$txt";
}
########################################################################




########################################################################
sub DowngradeBasisGe3_01LtXXX(){
  my $file=shift;
  my $subver=shift;
  my $prot=shift;
  my $rank;
  my @size;
  my @sizearray;
  my @value;

  eval `fpiotest -mode VALUE -var basis.atom -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  my @atoms=@value;



  eval `fpiotest -mode RANK -var basis.sorts -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  my @sortsize=@size;
  eval `fpiotest -mode SIZEARRAY -var basis.sorts -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  my @sortsizearray=@sizearray;
  eval `fpiotest -mode VALUE -var basis.sorts -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  my @sorts=@value;



  my $semstateinfo=&extractValenceStates($file,"semicorestates",$prot);
  my $valstateinfo=&extractValenceStates($file,"valencestates",$prot);
  my $polstateinfo=&extractValenceStates($file,"polarizationstates",$prot);
  my $corestateinfo=&extractCoreStates($file,0);

  my @value;
  eval `fpiotest -mode VALUE -var nsort -f $file -format perl` ;
  &EXIT("fpiotest error in file '$file'") if(($?!=256));
  my $nsort=@value[0];


  my $date=`date`; $date=~s/\s+$//g;


  #####################
  # output
  #####################

  open(FO,">$tmpfile");

print FO <<EOF
########################################################################
# this file is part of the FPLO bandstructure package
# it was created automatically
# please dont delete this file nor
# edit it directly, unless you really know what you are doing
# Use FEDIT to change its content
#
########################################################################
section header{

    struct {char[*] mainversion;char[*] subversion;} version
    ={"$newversion","$subver"};


    struct {char[*] hostname;char[*] date;char[*] by;} last_changes
    ={"","$date","fdowngrad"};

};


section structure{
    int nsort=$nsort;
};


section basis{
EOF
;



  print FO "struct {char[2] atom;struct {char[2] state;} corestates[*];struct {char[2] state;real compression;} valencestates[*];} basis[nsort]\n";
  
  print FO "={";

  for(my $issa=1;$issa<=$nsort;$issa++){
  classloop: for(my $icl=0;$icl<$sortsize[1];$icl++){
      my $atom=$atoms[$icl];
      for(my $incl=0;$incl<$sortsizearray[$icl];$incl++){
	my $ind=$icl*$sortsize[0]+$incl;
	if($issa == $sorts[$ind]){
	  print FO "{\"$atom\",{";
	  # core states
	  my $comma='';
	  my $stateinfo=$corestateinfo;
	  for(my $ist=0;$ist<$stateinfo->{statessizearray}[$icl];$ist++){
	    my $ind=$icl*$stateinfo->{statessize}[0]+$ist;
	    print FO "$comma";
	    print FO "{\"$stateinfo->{states}[$ind]\"}\n";
	    $comma=','
	  }
	  #
	  print FO "},{";
	  # other states
	  my $comma='';
	  for my $stateinfo ( $semstateinfo,$valstateinfo,$polstateinfo) {
	    for(my $inscl=0;$inscl<$stateinfo->{sizearray}[$icl];$inscl++){
	      my $ind1=($icl*$stateinfo->{statessize}[1]+$inscl);
	      for(my $ist=0;$ist<$stateinfo->{statessizearray}[$ind1];$ist++){
		my $ind=$ind1*$stateinfo->{statessize}[0]+$ist;
		print FO "$comma";
		print FO "{\"$stateinfo->{states}[$ind]\",$stateinfo->{compression}[$ind1]}\n";
		$comma=','
	      }
	    }
	  }
	  print FO "}";
	  print FO "}";
	  if($issa<$nsort){ print FO ","; }
	  last classloop;
	}
      }# in classes
    }# all classes
  }# all sorts

  print FO "};";



print FO <<EOF
};
EOF
;


  close(FO);

}
########################################################################




########################################################################
#
# extract relevant information for the various valence state types
#
########################################################################
sub extractValenceStates{
  my $file=shift;
  my $type=shift;
  my $prot=shift;
  my $stateinfo={};
  my $rank;
  my @size;
  my @sizearray;
  my @value;
  eval `fpiotest -mode SIZEARRAY -var basis.$type -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{sizearray}=[@sizearray];
  eval `fpiotest -mode RANK -var basis.$type.states -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{statessize}=[@size];
  eval `fpiotest -mode SIZEARRAY -var basis.$type.states -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{statessizearray}=[@sizearray];
  eval `fpiotest -mode VALUE -var basis.$type.states -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{states}=[@value];
  eval `fpiotest -mode VALUE -var basis.$type.compression -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{compression}=[@value];

  if($prot){
    foreach my $k (keys %$stateinfo){
      print "$type: $k => ". join(',',@{$stateinfo->{$k}})."\n";
    }
  }
  return $stateinfo;
}

########################################################################
#
# extract relevant information for the core state type
#
########################################################################


sub extractCoreStates{
  my $file=shift;
  my $prot=shift;
  my $stateinfo={};
  my $rank;
  my @size;
  my @sizearray;
  my @value;
  eval `fpiotest -mode RANK -var basis.corestates.state -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{statessize}=[@size];
  eval `fpiotest -mode SIZEARRAY -var basis.corestates.state -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{statessizearray}=[@sizearray];
  eval `fpiotest -mode VALUE -var basis.corestates.state -f $file -format perl` ; die "fpiotest error" if(($?!=256));
  $stateinfo->{states}=[@value];

  if($prot){
    foreach my $k (keys %$stateinfo){
      print "corestates: $k => ". join(',',@{$stateinfo->{$k}})."\n";
    }
  }
  return $stateinfo;
}
########################################################################



########################################################################
sub set_version_and_finalize{
  my $file=shift;
  my $newversion=shift;
  
  eval `fpiotest -mode SET_SCALAR -var version.mainversion -f $tmpfile -value $newversion > $tmpfile2`;
  if (($?!=256)) { &EXIT("fpiotest error in file '$file'"); }
  system "mv $tmpfile2 $tmpfile";

  eval `fpiotest -mode SET_SCALAR -var last_changes.by -f $tmpfile -value \"fdowngrad\" > $tmpfile2`;
  if (($?!=256)) { &EXIT("fpiotest error in file '$file'"); }
  system "mv $tmpfile2 $tmpfile";


  system ("mv $tmpfile $file");
  
}

########################################################################
sub get_file_version{
  my $file=shift;
  my @value;
  eval `fpiotest -mode VALUE -var version.mainversion -f $file -format perl` ;
  &EXIT("fpiotest error in file '$file'") if(($?!=256));
  my $v=@value[0];
  
  eval `fpiotest -mode VALUE -var version.subversion -f $file -format perl`;
  &EXIT("fpiotest error in file '$file'") if(($?!=256));
  my $s=@value[0];

  eval `fpiotest -mode VALUE -var last_changes.by -f =.in -format perl`;
  &EXIT("fpiotest error in file '=.in'") if(($?!=256));
  my $b=@value[0];

  return ($v,$s,$b);
}

########################################################################
sub file_newer_than_downgrad{
  my $file=shift;
  my $fileversion=shift;
  my $newversion=shift;
  my $ret=1;
  if ($fileversion<$newversion) {
    print "file '$file' has version $fileversion < $newversion. For upgrading use fedit (fplo) itself!\n";
    print "\twill skip '$file'\n";
    $ret=0;
  }
  return $ret;
}

########################################################################
sub get_scalar_variable{
  my $file=shift;
  my $name=shift;
  
  my @value;
  eval `fpiotest -mode VALUE -var $name -f $file -format perl` ;
  &EXIT("fpiotest error in file '$file'") if(($?!=256));
  
  return $value[0];
}

########################################################################
sub set_scalar_variable{
  my $file=shift;
  my $name=shift;
  my $value=shift;

#  print "value=[$value]\n";
  system <<EOF
   fpiotest -mode SET_SCALAR -var $name -f $tmpfile -value "$value" > $tmpfile2
EOF
;
  &EXIT("fpiotest error in file '$file'") if(($?!=256));
  system "mv $tmpfile2 $tmpfile";
}

########################################################################
sub get_vector_variable{
  my $file=shift;
  my $name=shift;

  my @value;

  eval `fpiotest -mode VALUE -var $name -f $file -format perl` ;
  &DIE("fpiotest error in file '$file'") if(($?!=256));

 #print "\@value=".(join ( ':',@value))."\n";
 return \@value;
}
