#! /bin/sh 
#
#
# Please dont change the content of this file !!
# 
#
########################################################################

VERSION=14.00
RELEASE=49
APP_NAME=`basename $0`

########################################################################
abort()
{
    cat<<EOF
$1
Abort
EOF
    exit
}
########################################################################
# this is to allow general existence tests for old shells    
fexist()
{
    for f in `ls -a`; do [ x"$f" = x"$1" ] && return 1 ; done
    return 0;
}

########################################################################
greeting(){
    cat <<EOF
------------------------------------------------------------------------
		 Installation of FPLO $1-$2

The FPLO$1-$2 code will be installed on your system now.
EOF



    if [ ! -f $ROOT/FPLO$VERSION-$RELEASE.tar.gz ] ; then

	abort "
    Cannot find the archive file  FPLO$VERSION-$RELEASE.tar.gz in 
    the current directory ($ROOT).
    It is part of the FPLO distribution. May be you renamed it, or
    you did not copy \"$APP_NAME\" and \"FPLO$VERSION-$RELEASE.tar.gz\" 
    into the current directory?"
    
    fi

    echo "Do you want to continue? [y/n]:" ; read  yn
    if [ x$yn != xy ] ; then
	abort "" 
    fi
    echo "--------------------------------------------------"
    cat <<EOF

The code will be installed in a different way compared to FPLO2.
This will help you keeping old versions running.
The new code tree will be as follows: A directory FPLO will be created 
in the current directory ($ROOT). 
Under FPLO there will be directories for the various versions and a bin
directory containing the executables of all versions together with the
auxillary scripts/programs.

FPLO--
     |--bin
     |--FPLO2
     |--FPLO3.00-5
     |--...

EOF
}

########################################################################
# check for non-directory file-name FPLO
# rename it into FPLO_saved
#
########################################################################
check_for_nondir_FPLO()
{

    fexist FPLO
    ret=$?
    if [ $ret -eq 1  -a ! -d FPLO ] ; then
	cat <<EOF
    There is a file/link ... called FPLO in the current path ($ROOT).
    We need this pathname for the installation, therefore we propose
    to let us rename the current FPLO into FPLO_saved.

EOF
	echo "Do you aggree? [y/n]:" ; read   yn
	echo "--------------------------------------------------"
	if [ x"$yn" != xy ] ; then
	abort "
    You do not aggree.
    In this case you need to install FPLO$VERSION-$RELEASE in another directory."
	fi
	echo "--------------------------------------------------"
	fexist FPLO_saved
	ret=$?
 	if [ $ret -eq 1 ] ; then
	    abort "
    FPLO_saved exists allready.
    Please rearrange the path names by hand and restart this script, 
    or install it somewhere else."
	fi
	
	mv FPLO FPLO_saved

	fexist FPLO
	ret=$?
	if [ $ret -eq 1 -a ! -d FPLO ] ; then
	    abort "
    It seems that we could not rename FPLO into FPLO_saved.
    May be permissions are not set."
	fi
	echo "renamed FPLO into FPLO_saved."
	
    fi
}

########################################################################
#
#
#
#
########################################################################
check_for_dir_FPLO()
{
    if [ -d FPLO ] ; then
	# directory exists
	cat<<EOF
    There is a directory called FPLO in the current path ($ROOT).
Checking for content ... 
EOF
	chmod u+xr FPLO
	if [ -f FPLO/00dont_remove ] ; then
	    # seems to be an FPLO-code directory
	    echo "FPLO exists and is assumed to contain a previous installation"
	    echo "(directory FPLO is used starting from version FPLO3.00-5)!"

    	    echo  "Is this correct? Shall we continue? [y/n]:" ; read yn
    	    if [ x"$yn" != xy ] ; then
		abort ""
	    fi
	    echo "--------------------------------------------------"
	else

	    cat<<EOF
    There is a directory called FPLO in the current path ($ROOT),
    which seems NOT to be an FPLO-code directory of a previous installation.
    (Missing the file "FPLO/00dont_remove").
    We need this pathname for the installation, therefore we propose
    to let us rename the current FPLO into FPLO_saved.
EOF
	    echo "Do you aggree with it? [y/n]:" ; read yn
	    if [ x"$yn" != xy ] ; then
		abort "
    You do not aggree.
    In this case you need to install FPLO$VERSION-$RELEASE in another directory."
	    fi
	    echo "--------------------------------------------------"
	    fexist FPLO_saved
	    ret=$?
	    if [ $ret -eq 1 ] ; then
		abort "
    FPLO_saved exists allready.
    Please rearrange the path names by hand and restart this script, 
    or install it somewhere else."
	    fi

	    mv FPLO FPLO_saved
	    if [ -d FPLO  ] ; then
		abort "
    It seems that we could not rename FPLO into FPLO_saved.
    May be permissions are not set."
	    fi
	    echo "renamed FPLO into FPLO_saved."

	fi
    fi  
}

########################################################################
#
#
#
########################################################################
create_FPLO_tree ()
{
    if [ ! -d FPLO ] ; then
	mkdir FPLO
    
	if [ ! -d FPLO ] ; then
	    abort "
    It seems that we could not create $ROOT/FPLO.
    May be permissions are not set."
	fi
	echo "Created directory $ROOT/FPLO"
    fi
    chmod u+xwr FPLO



    
    if [ ! -d FPLO/bin ] ; then
	mkdir FPLO/bin

	if [ ! -d FPLO/bin ] ; then
	    abort "
    It seems that we could not create $ROOT/FPLO/bin.
    May be permissions are not set."
	fi
	echo "Created directory $ROOT/FPLO/bin"
    fi
    chmod u+xwr FPLO/bin

    fexist FPLO/00dont_remove && chmod u+w FPLO/00dont_remove

    cat <<EOF>FPLO/00dont_remove
# Please dont remove this file, it is needed for the installation
# of forthcoming FPLO versions
VERSION=$VERSION
RELEASE=$RELEASE
EOF
    chmod -w FPLO/00dont_remove


}
########################################################################
#
#
#
#
########################################################################
unpack_FPLO()
{
    
    cd $ROOT
    if [ -d FPLO/FPLO$VERSION-$RELEASE ] ; then
	abort "
    Directory FPLO/FPLO$VERSION-$RELEASE does allready exist.
    Will abort. If you want to reinstall this version, rename 
    or delete FPLO/FPLO$VERSION-$RELEASE first and rerun \"$APP_NAME\"
    "
    fi

    gzip -cd FPLO$VERSION-$RELEASE.tar.gz| tar -xvf -
    echo ""
    echo "    Extracted new version."
    echo ""
    cp -f FPLO/FPLO$VERSION-$RELEASE/bin/* FPLO/bin
    echo ""
    echo "    Copied auxillary scripts to FPLO/bin."
    echo ""
}
########################################################################
#
#
#
########################################################################
link_version_two()
{
    
    cd $ROOT/FPLO/bin
    ln -s ../FPLO2/bin/fplo2 ./fplo2
    ln -s ../FPLO2/bin/fedit2 ./fedit2
    ln -s ../FPLO2/bin/dirac2 ./dirac2
    ln -s ../FPLO2/bin/bandplot2 ./bandplot2
    echo ""
    echo "    Created (dead) symbolic links to old FPLO2 executables."
    echo ""
    cd $ROOT
}
########################################################################

ROOT=`pwd`




greeting $VERSION $RELEASE



# search for FPLO instalation
cat<<EOF

Now checking, if a directory called FPLO does exist...
EOF



check_for_nondir_FPLO 

check_for_dir_FPLO 

create_FPLO_tree 

unpack_FPLO

link_version_two



cat<<EOF
------------------------------------------------------------------------
------------------------------------------------------------------------
Succesfully finished step 1 of installation of FPLO$VERSION-$RELEASE!
Now you need to follow the installation instructions in the 
file "FPLO/FPLO$VERSION-$RELEASE/install/README", which is part of the
distribution.
------------------------------------------------------------------------
------------------------------------------------------------------------
EOF

########################################################################
# EOF
########################################################################
