!!----DEBUG options-------------------|
!!DIRACSOLVER:odeint:ODE2             |
!!DIRACSOLVER:odeint:ODE4             |
!!DIRACSOLVER:odeint:IODE             |
!!DIRACSOLVER:solver2:ODE2            |
!!DIRACSOLVER:solver2:ODE4            |
!!DIRACSOLVER:solver4:                |
!!DIRACSOLVER:rel_ode_expansion:      |
!!DIRACSOLVER:kh_ode_expansion:       |
!!DIRACSOLVER:mag_diagno:             |
!!DIRACSOLVER:analyse_wf:             |
!!DIRACSOLVER:dwfdwdirac:ODE2         |
!!DIRACSOLVER:dwfdwdirac:ODE4         |
!!DIRACSOLVER:odematch:ODE            |
!!DIRACSOLVER:odematch:IODE           |
!!DIRACSOLVER:prepclassturn:write_pot |
!!------------------------------------|
!!
!!TODO: odematch for cases where some n_i is exactly zero ()
Module diracsolver
  Use PrecisionDefs
  Use memory
  Use Constants
  Use File_iounits
  Use errors
  Use Bandmatrix
  Use tmesh
  Use mesh
  Use diracdefs
  Use Matrixinvers
  Use Integrate1d
  Use trelativistic
  Use finuc
  !!
  Implicit None
  !!
  Private
  !!
  Public:: wfdirac,init_diracsolver,clean_diracsolver,T_DIRACSOLVER&
       &,init_t_diracsolv_control,T_DIRACSOLV_CONTROL
  !!
  Character(*),Parameter::nmodule="DIRACSOLVER"
  !!***********************************************************************
  !!
  !! Comment: relativistic confining potential is (1+beta)/2 W
  !! this is analytically the only possible way to confine
  !!
  !!     order of predictor and corrector (max 7)
  Integer(t_int), Private,Parameter :: pred_order=7_t_int

  !!fitting newton method
  Integer(t_int),Private, Parameter :: fit_init=0_t_int,fit_loop=1_t_int
  !!
  !! spin coupling indicees
  Integer(t_int),Parameter          :: kap_kap    =1_t_int
  Integer(t_int),Parameter          :: kapq_kap   =2_t_int
  Integer(t_int),Parameter          :: kapq_kapq  =3_t_int
  Integer(t_int),Parameter          :: mkap_mkap  =4_t_int
  Integer(t_int),Parameter          :: mkapq_mkapq=5_t_int
  !!
  Integer(t_int),Parameter :: OUT= 1_t_int
  Integer(t_int),Parameter :: IN =-1_t_int
  Integer(t_int) :: fillins=0_t_int
  !!
  !!***********************************************************************
  !! make data tracking simpler
  Type T_DIRACSOLV_CONTROL
     !! use gam_factors in solver or not
     Logical(t_log) :: gam_factor
     !! mode of relativistic approximation
     Integer(t_int) :: relmode
     !! wether we need the b-field or not
     Logical(t_log) :: relmag
     !! finite nucleus model
     Integer(t_int) :: nuc_type
     !! gamma exponent
     Real(t_real)   :: gamma
     !! main angular momentum
     Integer(t_int) :: la
     !! main qn
     Integer(t_int) :: nmain
     !! main kappa
     Integer(t_int) :: kap
     !! 2*magnetic Jz qn.
     Integer(t_int) :: mu
     !! nuclear charge
     Real(t_real)   :: ZZ
     !! speed of light 
     Real(t_real)   :: c,c2
     !!
     !! confining potential:  fx0*(r/rx0)^n +  fx2*(r/rx2)^n2
     !!
     Real(t_real)   :: fx0,rx0,exn0,fx2,rx2,exn2
     !!
     !! a manipulated vtr, which has local extrema removed, for the purpose of
     !! finding the region of the classical turning point
     !! (the input potential need not be monotonic)
     Real(t_real),Pointer   :: p_vc(:)
     Real(t_real),Pointer   :: p_vtrcorrected(:)
     !! expansion of potential including finite nucleus addon
     Integer(t_int) :: Nv
     Real(t_real),Pointer :: p_Vcoeff(:)
     Real(t_real),Pointer :: p_Bcoeff(:)
     !!
  End Type T_DIRACSOLV_CONTROL
  !!***********************************************************************
  !! bookkeeping for odeorder=4 to avoid multiple calculations of the same data
  Type T_DIRACSOLVER
     !! the maximum mesh point, below which the solution is calculated
     !! above nmax the solution is set to zero
     !! this avoids underflow
     Integer(t_int) :: nmax
     !! the matching point
     Integer(t_int) :: nfit
     !! the kappa value for which it was calculated before
     Integer(t_int) :: kap
     !! the main qn for which it was calculated before
     Integer(t_int) :: nmain
     !! the eigen energies of the solutions, which serve as basis states
     !! for the Ritz energy approximation
     Real(t_real)   :: E12(2)
     !! the basis states for the Ritz  approximation
     Real(t_real),Pointer   :: p_wf(:,:,:) !! nr,2,2
  End Type T_DIRACSOLVER
  !!
  !!***********************************************************************
  !! odeshoot data
  Integer(t_int),Private :: ldi,ldj,ldk
  Integer(t_int),Private,Parameter :: maxcorder=7_t_int
  Real(t_real),Private ::  pred_hcof(maxcorder)
  Real(t_real),Private ::  pred_cof(0:maxcorder,maxcorder,maxcorder)

  Data pred_hcof /2._t_real,12._t_real,24._t_real,720._t_real,1440._t_real&
       &,60480._t_real,120960._t_real/
  Data (((pred_cof(ldi,ldj,ldk),ldi=0,maxcorder)&
       &,ldj=1,maxcorder)&
       &,ldk=1,2)/&
       ! {{{

       !!     1
       & 1._t_real,1._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       !!     2
       & ,5._t_real,8._t_real,-1._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,-1._t_real,8._t_real,5._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real/
       !!     3
  Data (((pred_cof(ldi,ldj,ldk),ldi=0,maxcorder)&
       &,ldj=1,maxcorder)&
       &,ldk=3,4)/&

       &  9._t_real,19._t_real,-5._t_real,1._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,-1._t_real,13._t_real,13._t_real,-1._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,1._t_real,-5._t_real,19._t_real,9._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       !!     4
       & ,251._t_real,646._t_real,-264._t_real,106._t_real,-19._t_real&
       &,0._t_real,0._t_real,0._t_real&
       & ,-19._t_real,346._t_real,456._t_real,-74._t_real,11._t_real&
       &,0._t_real,0._t_real,0._t_real&
       & ,11._t_real,-74._t_real,456._t_real,346._t_real,-19._t_real&
       &,0._t_real,0._t_real,0._t_real&
       & ,-19._t_real,106._t_real,-264._t_real,646._t_real,251._t_real&
       &,0._t_real,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real/
       !!     5
  Data (((pred_cof(ldi,ldj,ldk),ldi=0,maxcorder)&
       &,ldj=1,maxcorder)&
       &,ldk=5,6)/&
       &  475._t_real,1427._t_real,-798._t_real,482._t_real,-173._t_real&
       &,27._t_real,0._t_real,0._t_real&
       & ,-27._t_real,637._t_real,1022._t_real,-258._t_real,77._t_real&
       &,-11._t_real,0._t_real,0._t_real&
       & ,11._t_real,-93._t_real,802._t_real,802._t_real,-93._t_real&
       &,11._t_real,0._t_real,0._t_real&
       & ,-11._t_real,77._t_real,-258._t_real,1022._t_real,637._t_real&
       &,-27._t_real,0._t_real,0._t_real&
       & ,27._t_real,-173._t_real,482._t_real,-798._t_real,1427._t_real&
       &,475._t_real,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real&
       !!     6
       & ,19087._t_real,65112._t_real,-46461._t_real,37504._t_real&
       &,-20211._t_real,6312._t_real,-863._t_real&
       & ,0._t_real&
       & ,-863._t_real,25128._t_real,46989._t_real,-16256._t_real&
       &,7299._t_real,-2088._t_real,271._t_real,0._t_real&
       & ,271._t_real,-2760._t_real,30819._t_real,37504._t_real&
       &,-6771._t_real,1608._t_real,-191._t_real,0._t_real&
       & ,-191._t_real,1608._t_real,-6771._t_real,37504._t_real&
       &,30819._t_real,-2760._t_real,271._t_real,0._t_real&
       & ,271._t_real,-2088._t_real,7299._t_real,-16256._t_real&
       &,46989._t_real,25128._t_real,-863._t_real,0._t_real&
       & ,-863._t_real,6312._t_real,-20211._t_real,37504._t_real&
       &,-46461._t_real,65112._t_real,19087._t_real&
       & ,0._t_real&
       & ,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real,0._t_real&
       &,0._t_real,0._t_real/
  !!
  Data (((pred_cof(ldi,ldj,ldk),ldi=0,maxcorder)&
       &,ldj=1,maxcorder)&
       &,ldk=7,maxcorder)/&

       !!     7
       &  +36799._t_real,+139849._t_real,-121797._t_real,+123133._t_real&
       &,-88547._t_real,+41499._t_real,&
       & -11351._t_real,+1375._t_real&
       & ,-1375._t_real,+47799._t_real,+101349._t_real,-44797._t_real&
       &,+26883._t_real,-11547._t_real,&
       & +2999._t_real,-351._t_real&
       & ,+351._t_real,-4183._t_real,+57627._t_real,+81693._t_real&
       &,-20227._t_real,+7227._t_real,-1719._t_real&
       & ,+191._t_real&
       & ,-191._t_real,+1879._t_real,-9531._t_real,+68323._t_real&
       &,+68323._t_real,-9531._t_real,+1879._t_real,&
       & -191._t_real&
       & ,+191._t_real,-1719._t_real,+7227._t_real,-20227._t_real&
       &,+81693._t_real,+57627._t_real,-4183._t_real&
       & ,+351._t_real&
       & ,-351._t_real,+2999._t_real,-11547._t_real,+26883._t_real&
       &,-44797._t_real,+101349._t_real&
       & ,+47799._t_real,-1375._t_real&
       & ,+1375._t_real,-11351._t_real,+41499._t_real,-88547._t_real&
       &,+123133._t_real,-121797._t_real,&
       & +139849._t_real,+36799._t_real&

       ! }}}
       & /
  !!***********************************************************************
  !!***********************************************************************

Contains
  !***********************************************************************
  ! calculate the wavefunction for given quantum numbers for 4 cases
  !
  ! potential input:
  ! non    relativistic                : vtr=potential, b not used
  ! scalar relativistic                : vtr=potential, b not used
  ! full   relativistic non  polarized : vtr=potential, b=0 required
  ! full   relativistic spin polarized : vtr=potential, b=magnetic field B_z
  !
  ! the mesh is r_i=g(x_i), i=1,nr, x_i=i*hx
  ! with the jacobian jr=dr/dx and  
  !!
  !! the option for calculating the derivative with respect to fx0 is
  !! not working currently
  !!
  Subroutine wfdirac(mesh,wf,wkf,dwfdw,dedw,dedwq&
       &,fx0,rx0,exn0,fx2,rx2,exn2,nmain&
       &,la,ZZ,vtr,btr,mu,relativistic,kap,etab,etabq,E,Eq,vtab1,vtab2&
       &,odeorder,tol,restart,sortJ,dsdata,nuc_type,gamma,gam_factor&
       &,gradwf)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLVER),Intent(inout) :: dsdata
    !Logical(t_log),Intent(in) ::prot
    !! use confining potential
    Type(T_Mesh),Intent(in) :: mesh
    !! sort such that nominal kappa state has larger j_kappa weight
    Logical(t_log),Intent(in)    :: sortJ
    !! relativistic mode
    Integer(t_int),Intent(in) :: relativistic
    !!
    Integer(t_int),Intent(in)    :: nuc_type
    !! main quantum number
    Integer(t_int),Intent(in) :: nmain
    !! order of ODE
    Integer(t_int),Intent(in) :: odeorder
    !! angular momentum quantum number, non relativistic
    Integer(t_int),Intent(in) :: la
    !! spin orbit quantum number, relativistic
    Integer(t_int),Intent(in) :: kap
    !! magnetic quantum number
    Integer(t_int),Intent(in) :: mu
    ! management of density iteration restart if shit happens
    Integer(t_int),Intent(inout):: restart
    !!
    !! potential times r
    Real(t_real),Intent(in)   :: vtr(mesh% nr)  
    !! magnetic field
    Real(t_real),Intent(in)   :: btr(mesh% nr)  
    !! wave functions
    Real(t_real),Intent(out)  :: wf(mesh% nr,4,2)
    !! gradient functions
    Real(t_real),Intent(out)  :: gradwf(mesh% nr,2,4)
    !! kinetic operator on wave function (only odeorder 2)
    Real(t_real),Intent(out)  :: wkf(mesh% nr,4)
    !! d(wf)/d(w), IODE (inhomogenous ODE) has to be solved
    Real(t_real),Intent(out)  :: dwfdw(mesh% nr,4,2)
    !! energy eigenvalue,derivative d(E)/d(w)
    Real(t_real),Intent(out)  :: dedw,dedwq
    !!
    !! nucleus charge
    Real(t_real),Intent(in)   :: ZZ
    Real(t_real),Intent(in)   :: fx0,rx0,exn0,fx2,rx2,exn2
    !! tabulated values of energies and fitting parameters, for the
    !! solutions (etab,vtab for magnetic relativistic)
    Real(t_real),Intent(inout):: etab,etabq,vtab1(2),vtab2(2)
    !! the energies of the solutions (may differ from etab in cae of
    !! magnetic relativistic)
    Real(t_real),Intent(out)  :: E,Eq
    !! tolerance of atomic calculations
    Real(t_real),Intent(in)   :: tol
    !! leading order for relativistic s1/2 p1/2 functions
    !! common prefactor r^gamma is assumed for ALL orbitals
    !! if gam_factor is true
    !! in NREL a prefactor r is assumed, which makes the first component
    !! the true radial function and NOT r*R(r)
    !! in this way, we get the explicit values for the s function at the
    !! nucleus and the gradient functions, without numerical derivatives
    Real(t_real),Intent(out)     :: gamma
    Logical(t_log),Intent(in)    :: gam_factor
    !!LOCAL
    Character(*),Parameter:: nsubfunc="wfdirac"
    !!
    !! number of nodes required (selects main quantum number of the solution)
    Integer(t_int) :: nodenr
    !! max mesh point where the solution is allowed to be non zero (to avoid
    !! underflow)
    Integer(t_int) :: nmax,nmax1
    !! point, where to match the outward and inward shoot
    Integer(t_int) :: nfit,nfit1
    !!
    !! dummies
    Integer(t_int) :: j,ir,nr
    Real(t_real)   :: E12(2)!,d
    !! upper/lower bound of Energy (heuristic)
    Real(t_real)   :: emax,emin,vm1
    !! protocol output
    Logical(t_log) :: deb,deb2
    !!
    Integer :: icheck
    !!
    Real(t_real),Allocatable  :: dy(:,:,:) !! aux array for odeiter
    Real(t_real),Allocatable  :: tmp(:) !! aux array for integration
    Real(t_real),Allocatable  :: tmp2(:) !! aux array for integration
    Real(t_real),Allocatable  :: dfs(:) !! aux array for integration
    !!
    Real(t_real),Allocatable  :: y(:,:,:) !! aux array for storage
    Real(t_real),Allocatable  :: ydw(:,:,:) !! aux array for storage
    !!
    Type(T_DIRACSOLV_CONTROL) :: dsc
    !!
    !!
    Integer(t_int) :: deg,l
    Real(t_real)   :: rr,rrx,rrx2,vv,vvg,xx,r0,rr2,pg2,a0,Egc,Efv,g,f,r2Mc,LL
    Real(t_real)   :: sL,gammal
    Integer(t_int),Parameter :: Ng=2
    Real(t_real)   :: Gcoeff(0:Ng)!,dGcoeff(0:Ng)
    !***********************************************************************
    !!
    dedw=0.0_t_real
    dedwq=0.0_t_real
    dwfdw=0.0_t_real
    !!
    Call init_t_diracsolv_control(dsc)
    !!
    dsc% relmode=relativistic
    dsc% kap=kap
    dsc% mu=mu
    dsc% gam_factor=gam_factor
    dsc% ZZ=ZZ
    dsc% nuc_type=nuc_type
    dsc% fx0=fx0
    dsc% rx0=rx0
    dsc% exn0=exn0
    dsc% fx2=fx2
    dsc% rx2=rx2
    dsc% exn2=exn2
    !!
    dsc% c=speed_of_light
    dsc% c2=dsc% c**2
    dsc% la=la
    dsc% nmain=nmain
    dsc% relmag=.False.
    If(dsc% relmode == FULL_RELATIVISTIC ) Then
       dsc% relmag=.True. !! the field decides
    Endif
    !!
    !! determine gamma
    If(gam_factor)Then
       If(dsc% relmode == NON_RELATIVISTIC)Then
          gamma=1.0_t_real          
          gammal=1.0_t_real
       Else If(dsc% relmode == SCALAR_RELATIVISTIC_KH) Then
          vm1=nuc_zeta(0.0_t_real,dsc% nuc_type)+1.0_t_real
          gamma=Sqrt(1.0_t_real-(dsc% ZZ*vm1/dsc% c)**2)
          gammal=Sqrt(dsc% la*(dsc% la+1)+1.0_t_real-(dsc% ZZ*vm1/dsc% c)**2)
          If(Abs(vm1)<1.0e-6_t_real)Then
             gamma=1.0_t_real
             gammal=dsc% la
          Endif
       Else !! FREL
          vm1=nuc_zeta(0.0_t_real,dsc% nuc_type)+1.0_t_real
          gamma=Sqrt(1.0_t_real-(dsc% ZZ*vm1/dsc% c)**2)
          gammal=gamma
       Endif
    Else
       gamma=0.0_t_real
       gammal=0.0_t_real
    Endif
    !!
    dsc% gamma=gamma
    !!
    !!
    Call prep_pot_expansion(mesh,vtr,btr,dsc)
    !!
    nr=mesh% nr
    !!
    gradwf(:,:,:)=0.0_t_real
    !!--------------------------------------------------------------------
    !!
    !! allocate aux array for classturn
    !!
    !!
    Allocate(dy(nr,4,2),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for dy !'")
    Endif
    Allocate(tmp(nr),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for tmp !'")
    Endif
    Allocate(tmp2(nr),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for tmp2 !'")
    Endif
    Allocate(dfs(nr),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for dfs !'")
    Endif
    Allocate(y(nr,4,2),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for y !'")
       Stop !! avoid unitialized warning
    Endif
    Allocate(ydw(nr,4,2),STAT=icheck)
    If(icheck/=0)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Could not allocate memory for ydw !'")
    Endif

    !
    Call prepclassturn(mesh,vtr,dsc)
    !!

    !!
    !!         maximal/minimal allowed energy
    !!--------------------------------------------------------------------
    !! search interval for energy [Emin,Emax]
    Call erg_estimate(mesh,vtr,dsc,Emin,Emax)
    !!
    !!--------------------------------------------------------------------
    nmax=nr 
    !! required node number and thus main quantum number
    nodenr=dsc% nmain-dsc% la-1_t_int
    If(nodenr<0_t_int) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg="'Node number<0!'")
    Endif
    If(odeorder==4 .And. dsc% relmode /= FULL_RELATIVISTIC) Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'odeorder ==4 and not full relativistic ?'")
    Endif

    If (odeorder == 2) Then
       deb=debugfun(nmodule,"odeint",'ODE2')
       deb2=debugfun(nmodule,"solver2",'ODE2')
       Call solver2(nodenr,kap,odeorder,mesh,vtr,btr&
            &,wf,ydw,dy,etab&
            &,vtab1,emin,emax,nmax,nfit,tol ,deb2,deb,restart,tmp,dsc)
           !!
       E=etab

       !!
       !!
       deg=2*dsc% la+1
       r0=nuc_max_rad(dsc% nuc_type,dsc% ZZ)
       !!
       Call wave_coef_nrel(dsc% la,r0,E,dsc% ZZ,dsc% p_Vcoeff ,dsc% Nv,Gcoeff,Ng)

       !! 
       Do ir=1,nr
          !!
          rr=mesh% p_rmesh(ir)
          rr2=mesh% p_rmesh(2)
          xx=rr/r0
          vv=vtr(ir)-dsc% ZZ*nuc_zeta(xx,dsc% nuc_type)
          rrx= fx0*(rr/rx0)**exn0*rr
          rrx2=fx2*(rr/rx2)**exn2*rr
          !!
          vvg=(vv+rrx+rrx2)
          Egc =(E*rr-vvg)/dsc% c
          Efv=E*rr-vv
          r2Mc =(Efv +rr*2.0_t_real*dsc% c2)/dsc% c
          !! NREL: r*\psi^{kin}
          !! REL:  r*\psi^{kin} / r^{gamma_0 -1}
          wkf(ir,LARGE_COMP1)=Egc*dsc% c *wf(ir,LARGE_COMP1,1)
          wkf(ir,SMALL_COMP1)=Efv        *wf(ir,SMALL_COMP1,1)
          wkf(ir,SMALL_COMP2)=Efv        *wf(ir,SMALL_COMP2,1)
          If(.Not. dsc% gam_factor) Goto 99
          If(dsc% relmode == NON_RELATIVISTIC ) Then
             !! first gradient function (d-l/r)\psi
             gradwf(ir,1,LARGE_COMP1)=wf(ir,SMALL_COMP1,1)*rr
             !! first gradient function (d+(l+1)/r)\psi
             If(dsc% la /= 0 ) Then
                If(ir==1)Then
                   !!
                   If(dsc% la == 1) Then
                      !! zero value from expansion
                      pg2=horner(rr2/r0,Gcoeff,Ng)
                      a0=wf(2,LARGE_COMP1,1)/(rr2*pg2)*Gcoeff(0)
                      !!
                      gradwf(ir,2,LARGE_COMP1)=(wf(ir,SMALL_COMP1,1)+deg*a0)*rr
                   Else
                      gradwf(ir,2,LARGE_COMP1)=0.0_t_real
                   Endif
                Else
                   gradwf(ir,2,LARGE_COMP1)=(wf(ir,SMALL_COMP1,1)&
                        &+wf(ir,LARGE_COMP1,1)*deg/rr)*rr
                Endif
             Endif
          Else If(&
               &dsc% relmode == SCALAR_RELATIVISTIC .Or.&
               &dsc% relmode == FULL_RELATIVISTIC &
               &) Then
             !! REL non-mag
             !! first gradient function (d-l/r)\psi /r^{gamma_0-2}
             l=(Abs(2*kap+1)-1)/2
             g=wf(ir,LARGE_COMP1,1)
             f=wf(ir,SMALL_COMP1,1)/dsc% c
             gradwf(ir,1,LARGE_COMP1)= -(kap+l+1)   *g +r2Mc *f
             gradwf(ir,1,SMALL_COMP1)= -Egc *g +(kap-l-1) *f
             If(l /= 0 ) Then
                !! second gradient function (d+(l+1)/r)\psi /r^{gamma_0-2}
                gradwf(ir,2,LARGE_COMP1)= -(kap-l)   *g +r2Mc *f
                gradwf(ir,2,SMALL_COMP1)= -Egc *g +(kap+l) *f
             Endif
          Else If( dsc% relmode == SCALAR_RELATIVISTIC_KH) Then
             !! REL non-mag
             !! first gradient function (d-l/r)\psi /r^{gamma_0-2}
             l=dsc% la
             sL=dsc% la*(dsc% la+1.0_t_real)
             LL=sL/r2Mc-Egc
             sL=Sqrt(sL)
             g=wf(ir,LARGE_COMP1,1)
             f=wf(ir,SMALL_COMP1,1)/dsc% c
             gradwf(ir,1,LARGE_COMP1)= -(l) *g +r2Mc  *f
             gradwf(ir,1,SMALL_COMP1)=  LL  *g -(l+2) *f
             tmp(ir)=sL/r2Mc*g*rr/rr**(gammal-dsc% gamma)
             gradwf(ir,1,LARGE_COMP2)=  sL/r2Mc*g*(gammal-l-2.0_t_real)

             If(dsc% nuc_type /= FINUC_NONE .And. ir==1 ) Then
                gradwf(1,1,SMALL_COMP1)=-f
                gradwf(1,1,LARGE_COMP2)=sL*f*(gammal-l-2.0_t_real)
             Endif


             If(l /= 0 ) Then
                !! second gradient function (d+(l+1)/r)\psi /r^{gamma_0-2}
                gradwf(ir,2,LARGE_COMP1)=  (l+1)  *g +r2Mc  *f
                gradwf(ir,2,SMALL_COMP1)=    LL   *g +(l-1) *f
                tmp2(ir)=sL/r2Mc*g*rr/rr**(gammal-dsc% gamma)
                gradwf(ir,2,LARGE_COMP2)=  sL/r2Mc*g*(gammal+l-1.0_t_real)

                If(dsc% nuc_type /= FINUC_NONE .And. ir==1 ) Then
                   gradwf(1,2,SMALL_COMP1)=2*f
                   gradwf(1,2,LARGE_COMP2)=sL*f*(gammal+l-1.0_t_real)
                Endif
             Endif
          Else
             Stop 'grad functions'
          Endif
99        Continue          
       Enddo !! ir
       !!
       If( dsc% relmode == SCALAR_RELATIVISTIC_KH) Then
          tmp(1)=0.0_t_real
          Call dfspline1(3_t_int,mesh% nr,mesh% p_rmesh,tmp,dfs)
          If(.Not.( (dsc% nuc_type /= FINUC_NONE  ) .And. dsc% la==1) )Then
             dfs(:)=dfs(:)*mesh% p_rmesh(:)**(gammal-dsc% gamma)
          Endif
          gradwf(:,1,LARGE_COMP2)=gradwf(:,1,LARGE_COMP2)+dfs(:)
          If(.Not.(dsc% nuc_type /= FINUC_NONE .And. dsc% la==1 )) Then
             gradwf(1,1,LARGE_COMP2)=0.0_t_real
          Endif
          !!
          l=dsc% la
          If(l /= 0 ) Then          
             tmp2(1)=0.0_t_real
             Call dfspline1(3_t_int,mesh% nr,mesh% p_rmesh,tmp2,dfs)
             If(.Not.( (dsc% nuc_type /= FINUC_NONE  ) .And. dsc% la==1) )Then
                dfs(:)=dfs(:)*mesh% p_rmesh(:)**(gammal-dsc% gamma)
             Endif
             gradwf(:,2,LARGE_COMP2)=gradwf(:,2,LARGE_COMP2)+dfs(:)
             If(.Not.(dsc% nuc_type /= FINUC_NONE .And. dsc% la==1 )) Then
                gradwf(1,2,LARGE_COMP2)=0.0_t_real
             Endif
          Endif


       Endif

       !! W derivative of wave function
       !!
!!$       deb2=debugfun(nmodule,"dwfdwdirac",'ODE2')
!!$       Call dwfdwdirac(mesh,nmax,nfit,btr,vtr,kap&
!!$            &,etab,dedw,wf,dy,dwfdw,deb2&
!!$            &,odeorder,tol,tmp,dsc)
       !!
       !!
    Else
       !!
          !!-------------------------------------------------------------------
          !! make guess of energy for odeorder=4
          !! this is a Ritz variation using some odeorder=2 solutions
          !! as basis of the magnetic Hamiltonian
          !! as basis we may choose different sets
          !!
          If(kap/=dsdata% kap .Or. dsc% nmain /= dsdata% nmain) Then
             !!
             dsdata% kap=kap
             dsdata% nmain=dsc% nmain
             !!
             !! nonpolarized states as basis for estimate
             !!
             !! ignore the magnetic field
             dsc% relmag=.False.
             
             deb=debugfun(nmodule,"odeint",'ODE2')
             deb2=debugfun(nmodule,"solver2",'ODE4')
             Call solver2(nodenr,kap,2_t_int,mesh,vtr&
                  &,btr,y,ydw,dy&
                  &,etab,vtab1,emin,emax &
                  &,nmax,nfit,tol,deb2,deb,restart,tmp,dsc)
             dsdata% p_wf(1:nr,1:2,1)=y(1:nr,1:2,1)
             dsdata% E12(1)=etab
             !!
             deb=debugfun(nmodule,"odeint",'ODE2')
             deb2=debugfun(nmodule,"solver2",'ODE2')
             Call solver2(nodenr,-kap-1,2_t_int,mesh,vtr&
                  &,btr,y,ydw,dy&
                  &,etabq,vtab2,emin,emax ,nmax1,nfit1&
                  &,tol,deb2,deb,restart,tmp,dsc)
             dsdata% p_wf(1:nr,1:2,2)=y(1:nr,1:2,1)
             dsdata% E12(2)=etabq
             !!
             nmax=Max(nmax,nmax1)
             nfit=Nint((nfit+nfit1)*0.5_t_real,t_int)
             !!
             dsdata% nmax=nmax
             dsdata% nfit=nfit
             !! put flag back
             dsc% relmag=.True.
             !!
          Else
             nmax=dsdata% nmax
             nfit=dsdata% nfit
          Endif
          !!
          !!
          !!
          deb=debugfun(nmodule,"mag_diagno",'')
          Call mag_diagno(kap,-kap-1,dsdata% E12(1),dsdata% E12(2)&
               &,mesh,btr,dsdata% p_wf,tmp,E12,deb,sortJ,dsc)
          !!TODO: make fourfold basis as long as not indefinite
          !!      manage to use the 2j=2l+1,mu=+/-j  states in the true field
          !!      and the 2j=2l-1 states in the scalar magnetic field
          !!      avoid multiple solver calls for higher l


          !!-------------------------------------------------------------------
          !!        CALCULATE RADIAL WAVEFUNCTIONS
          !!-------------------------------------------------------------------
          !!
          !!
          !! the first solution
          emin=E12(1)-Abs(E12(2)-E12(1))*0.25_t_real
          emax=E12(1)+Abs(E12(2)-E12(1))*0.25_t_real
          !!
          E=E12(1)
          !!
          !!
          !!Print *,'solver4'
          deb=debugfun(nmodule,"odeint",'ODE4')
          deb2=debugfun(nmodule,"solver4",'')
          Call solver4(nodenr,kap,odeorder,mesh,vtr,btr&
               & ,y,ydw,dy,E,vtab1&
               &,emin,emax,nmax,nfit ,tol,deb2,deb,restart,tmp,dsc)
          !!
          !!
          deb=debugfun(nmodule,"analyse_wf",'')
          Call analyse_wf(mesh,y,tmp,kap,deb,dsc)
          !!
          !!
          !!Print *,'solverdwf'
!!$          deb2=debugfun(nmodule,"dwfdwdirac",'ODE4')
!!$          Call dwfdwdirac(mesh,nmax,nfit,btr,vtr,kap&
!!$               &,E,dedw,y,dy,ydw&
!!$               &,deb2,odeorder,tol,tmp,dsc)

          !!
          !! store wf as sum over partial waves, still scaled small components 
          !!
          Do j=1,4
             wf(1:nr,j,1)=(y(1:nr,j,1)+y(1:nr,j,2))
          Enddo
!!$          If(dsc% compr)Then
!!$             Do j=1,4
!!$                dwfdw(1:nr,j,1)=(ydw(1:nr,j,1)+ydw(1:nr,j,2))
!!$             Enddo
!!$          Endif
          !!
          !! the second solution
          emin=E12(2)-Abs(E12(2)-E12(1))*0.25_t_real
          emax=E12(2)+Abs(E12(2)-E12(1))*0.25_t_real
          !!
          Eq=E12(2)
          !!
          !!Print *,'solver4'
          deb=debugfun(nmodule,"odeint",'ODE4')
          deb2=debugfun(nmodule,"solver4",'')
          Call solver4(nodenr,-kap-1,odeorder,mesh,vtr&
               &,btr,y,ydw,dy,Eq&
               &,vtab2,emin,emax,nmax ,nfit,tol,deb2,deb,restart,tmp,dsc)

          !!
          !!
          deb=debugfun(nmodule,"analyse_wf",'')
          Call analyse_wf(mesh,y,tmp,-kap-1,deb,dsc)
          !!
          !!
          !!Print *,'solverdwf'
!!$          deb2=debugfun(nmodule,"dwfdwdirac",'ODE4')
!!$          Call dwfdwdirac(mesh,nmax,nfit,btr,vtr,-kap-1&
!!$               &,Eq,dedwq,y,dy&
!!$               &,ydw,deb2,odeorder,tol,tmp,dsc)
          !!
          !! store wf as sum over partial waves, still scaled small components 
          !!
          Do j=1,4
             wf(1:nr,j,2)=(y(1:nr,j,1)+y(1:nr,j,2))
          Enddo
!!$          If(dsc% compr)Then
!!$             Do j=1,4
!!$                dwfdw(1:nr,j,2)=(ydw(1:nr,j,1)+ydw(1:nr,j,2))
!!$             Enddo
!!$          Endif
          !!
          !!--------------------------------------------
          !!  CHECK ORTHOGONALITY FOR odeorder=4
          !!--------------------------------------------
          !!
          Call  check_ortho_wf(mesh,wf,tmp,odeorder,dsc)
          !!
       


       !!
    Endif ! odeorder=2
    !!
    
    !!



    !!
    !!--------------------------------------------------------------------
    !! scale back small component
    !!
    If(dsc% relmode /= NON_RELATIVISTIC)Then
       wf(1:nr,SMALL_COMP1,1:2)=wf(1:nr,SMALL_COMP1,1:2)/dsc% c
       wkf(1:nr,SMALL_COMP1)=wkf(1:nr,SMALL_COMP1)/dsc% c
       If(dsc% relmode == SCALAR_RELATIVISTIC_KH )Then
          wf(1:nr,LARGE_COMP2,1)=wf(1:nr,SMALL_COMP2,1)/dsc% c
          wkf(1:nr,LARGE_COMP2)=wkf(1:nr,SMALL_COMP2)/dsc% c
       Endif
       wf(1:nr,SMALL_COMP2,1:2)=wf(1:nr,SMALL_COMP2,1:2)/dsc% c
!!$       dwfdw(1:nr,SMALL_COMP1,1:2)=dwfdw(1:nr,SMALL_COMP1,1:2)/dsc% c
!!$       dwfdw(1:nr,SMALL_COMP2,1:2)=dwfdw(1:nr,SMALL_COMP2,1:2)/dsc% c
    End If
    !!
    !!
    If(Allocated(ydw))Deallocate(ydw)
    If(Allocated(y))Deallocate(y)
    If(Allocated(tmp))Deallocate(tmp)
    If(Allocated(tmp2))Deallocate(tmp2)
    If(Allocated(dfs))Deallocate(dfs)
    If(Allocated(dy))Deallocate(dy)
    !!
    Call clean_t_diracsolv_control(dsc)
    !!
    Return
    !!
  End Subroutine wfdirac
  !***********************************************************************
  !! the actual solver for the 4-dimensional Dirac equation
  !! the small component is scaled with the speed of light to get  better
  !! numerics
  Subroutine  solver4(nodenr,kap,odeorder,mesh,vtr,btr&
       &,wf,dwfdw,dy,EE,BIn,emin&
       &,emax,nmax,nfit,wftol,prot,odeintprot,restart,tmp,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    !! protocol output
    Logical(t_log) :: prot,odeintprot
    !! number of nodes required (selects main quantum number of the solution)
    Integer(t_int),Intent(in)    :: nodenr
    !! current kappa qn. (relativistic)
    Integer(t_int),Intent(in)    :: kap
    !! management of density iteration restart if shit happens
    Integer(t_int),Intent(inout) :: restart
    !! order of the ODE: normaly  2 , but 4 in case of relativistic magnetic
    !! equation with |2 mu|<2l+1
    Integer(t_int),Intent(in)    :: odeorder
    !! max mesh point where the solution is allowed to be non zero (to avoid
    !! underflow), may be input in some cases
    Integer(t_int),Intent(in)    :: nmax
    !! point, where to match the outward and inward shoot, may be input in
    !! some cases
    Integer(t_int),Intent(in)    :: nfit
    !! potential (contains the magnetic field for scalar- and non
    !!-relativistic case)
    Real(t_real),Intent(in)      :: vtr(mesh% nr)
    !! magnetic field (must be zero for scalar- and non-relativistic case)
    Real(t_real),Intent(in)      :: btr(mesh% nr)
    !! energy interval to confine search to
    Real(t_real),Intent(in)      :: emin,emax
    !! tolerance of wave function fit
    Real(t_real),Intent(in)      :: wftol
    !! on input initial guess of energy, on output energy=eigenvalue
    Real(t_real),Intent(inout)   :: EE
    !! on input guess of the matching parameter, on output resulting matching
    !! parameter
    Real(t_real),Intent(inout)   :: BIn(2)
    !! the wave function (solution)
    Real(t_real),Intent(out)     :: wf(mesh% nr,4,2)
    !! derivative d(wf)/d(w), only dummy here
    Real(t_real),Intent(in)      :: dwfdw(mesh% nr,4,2)
    !! dummy array for the bookkeeping of the shoots
    Real(t_real),Intent(out)     :: dy(mesh% nr,4,2)
    !! dummy array for integration
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !!
    !!
    !!LOCAL
    Character(*),Parameter:: nsubfunc="solver4"
    !!
    !! do not calculate the derivative d(wf)/d(w) here
    Logical(t_log),Parameter :: dodw=.False.
    !!
    !! total number of iterations
    Integer(t_int),Parameter :: totaliter=80_t_int
    !! maximum number of back loops
    Integer(t_int),Parameter :: totalback=30_t_int
    !! total number of starting iterations for searching initial energy
    !Integer(t_int),Parameter :: starttotaliter=20_t_int
    !! maximum number of newton loops
    Integer(t_int),Parameter :: maxnewiter=40_t_int
    !!
    !! not yet converged
    Logical(t_log) :: noconv
    !!
    !! number of nodes for energies EE
    Integer(t_int) :: node
    !! various counter
    Integer(t_int) :: iback,titer,iter
    !! control the re-adjustment of the matching point
    !Integer(t_int) :: istart
    !!
    !! local tolerance for convergence (subject to adjustment in case of
    !! panic)
    Real(t_real)   :: tol
    !! best solution found so far
    Real(t_real)   :: bestEE,bestdev
    !! upper/lower bound for energy search interval ( set up by checking
    !! number the node number)
    !Real(t_real)   :: Eup,Edn
    !! mismatch of the outward and inward shoot at the matching point
    Real(t_real)   :: dif
    !! bookkeeping for the newton method: vec=energy,fvec=mismatch (dif)
    Real(t_real)   :: vec,vecn,fvec,fvecn
    !! dummies
    Real(t_real)   :: dedwdummy,norm
    !!
    dedwdummy=0.0_t_real
    !!
    If(odeorder/=4_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Odeorder ==2 : must use solver2!'")
    Endif
    !!
    !!--------------------------------------------------------------------
    If(prot) Write(outfile,1100)nsubfunc
    If(prot) Write(outfile,1101)nsubfunc,Trim(relat_name(dsc% relmode))
    If(prot) Write(outfile,"(a,': inital EE=',g30.16e2)")nsubfunc,EE
    If(prot) Write(outfile,"(a,': nodenr=',i4)")nsubfunc,nodenr
    If(prot) Write(outfile,"(a,': kap   =',i4)")nsubfunc,kap
1100 Format(a,': -START--------------------------------------------------')
1101 Format(a,':',1x,a)
    !!
    !!---------------------------------------------------------------------
    !!
    tol=wftol
    If(dsc% c2>=139.0_t_real**2) tol=Max(1.e-12_t_real,wftol)
    !!
    !!
    bestEE=EE
    bestdev=1.e30_t_real
    !!
    !! set up start value for the outward shoot
    !!
    wf(2_t_int,LARGE_COMP1,1_t_int)=1.0_t_real
    wf(2_t_int,LARGE_COMP2,2_t_int)=1.0_t_real
    !!
    !Eup=emax
    !Edn=emin
    !!
    !!
    iback=0_t_int
    !istart=0_t_int
    titer=0_t_int
    iter=0_t_int
    !!
    !!
    !!
11  Continue
    If(prot) Write(outfile,"(a,': start iteration ',i4)")nsubfunc,titer
    
    !! initial newton step
   
    Call odeint(mesh,nmax,vtr,btr,kap&
         &   ,EE,dedwdummy,BIn,wf,dy,dwfdw&
         &   ,odeorder,dodw,nfit,dif,odeintprot,tmp,dsc)


    node=countnodes(wf,mesh% nr,nmax,odeorder)
    !!TODO: what happens, if node/=nodenr

    
    vec=EE
    fvec=dif
    !!EE=EE*1.0001_t_real
    EE=EE+Min(Abs(emax-emin)*1.0e-4_t_real,1.0e-4_t_real)
    vecn=EE
    
    !!Call fitsol(EE,dif,vec,vecn,fvec,fvecn,fit_init)


    !! loop over iteration of energy to fit the solution
    !!
    !! start of EE-iteration -------------------------------
    !!
    iter=0_t_int
    noconv=.True.
    newtonloop: Do While(titer<totaliter.And.noconv.And.iback<totalback)
       iter=iter+1_t_int
       titer=titer+1_t_int
       !       if it is jumping around, just kill it be restarting
       If(iter>maxnewiter) Then
          stop 'iter>maxnewiter'
       Endif
       If(iter>1_t_int) Then
          node=countnodes(wf,mesh% nr,nmax,odeorder)
          !!TODO: node/=nodenr
       Endif

       !! newton step dif at new point
       Call odeint(mesh,nmax,vtr,btr,kap&
            &     ,EE,dedwdummy,BIn,wf,dy&
            &     ,dwfdw,odeorder,dodw,nfit,dif,odeintprot,tmp&
            &,dsc)
       

       If(prot)Write(outfile,234)nsubfunc,EE,dif,nsubfunc,mesh% nr,nmax,nfit

       If(Abs(dif)<bestdev)Then
          ! best sampling
          bestEE=EE
          bestdev=Abs(dif)
       Endif

       Call fitsol(EE,dif,vec,vecn,fvec,fvecn,fit_loop)


       !
       noconv=.Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol))
       If(prot)Write(outfile,235)nsubfunc,vec,vecn,nsubfunc,vecn-vec
    Enddo newtonloop


    If(noconv.Or..Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol)))Then
       restart=restart+1_t_int
       If(tol<1.e-7_t_real)Then
          tol=tol*10.0_t_real
          titer=0_t_int
          iback=0_t_int
          If(prot) Write(outfile,"(a,': decrease accuracy to '"&
               &     //",g30.16e2)") nsubfunc,tol
          Goto 11 

       Endif
    Endif
    If(noconv.Or..Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol)))Then
       Write(errbuf,1001) titer,fvec
1001   Format(''' Sorry, no solutions found after ',i4,' iterations '',/&
            &,'' Cost function ',1(g30.15e3,3x),''',/')

       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg=errbuf)
    Endif
    EE=bestEE
    dif=bestdev
    If(prot) Write(outfile,236)nsubfunc,EE,nsubfunc,vecn-vec,nsubfunc,dif
    
    
    
    Call odeint(mesh,nmax,vtr,btr,kap&
         &   ,EE,dedwdummy,BIn,wf,dy,dwfdw&
         &   ,odeorder,dodw,nfit,dif,odeintprot,tmp,dsc)
    
    node=countnodes(wf,mesh% nr,nmax,odeorder)
    If(node/=nodenr)Then
       Write(errbuf,1000)node,nodenr
1000   Format("'final node number incorrect, sorry! found ",i3," need ",i3,"'")
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    !! finally normalize
    !!
    !!
    Call normwf(mesh,nmax,wf,norm,odeorder,tmp,dsc,EE,vtr)
    !!
    !!
    If(prot) Write(outfile,"(a,':-END----------'"&
         &   //",'-------------------------------')")nsubfunc
    !!
    !!
    Return
    !!
    !!
235 Format(a,': Eold=',g30.16e2,' Enew=',g30.16e2&
         & ,/,a,': Delta=',g30.16e2)
236 Format(a,': E =',g30.16e2&
         & ,/,a,': DeltaE =',g30.16e2&
         & ,/,a,': DeltaWF=',g30.16e2)
234 Format(a,': E=',g30.16e2,' dif=',g30.16e2&
         &   ,/,a,': nr=',i4,' nmax=',i4,' nfit=',i4)
    !!
  End Subroutine solver4
  !***********************************************************************
  !! the actual solver for the 2-dimensional Dirac and for the
  !! Schroedinger equation
  !! the small component is scaled with the speed of light to get  better
  !! numerics
  Subroutine  solver2(nodenr,kap,odeorder,mesh,vtr,btr&
       &,wf,dwfdw,dy,EE,BIn,emin&
       &,emax,nmax,nfit,wftol,prot,odeintprot,restart,tmp,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    !! protocol output
    Logical(t_log) :: prot,odeintprot
    !! number of nodes required (selects main quantum number of the solution)
    Integer(t_int),Intent(in)    :: nodenr
    !! kappa qn. (relativistic)
    Integer(t_int),Intent(in)    :: kap
    !! management of density iteration restart if shit happens
    Integer(t_int),Intent(inout) :: restart
    !! order of the ODE: normaly  2 , but 4 in case of relativistic magnetic
    !! equation with |2 mu|<2l+1
    Integer(t_int),Intent(in)    :: odeorder
    !! max mesh point where the solution is allowed to be non zero (to avoid
    !! underflow), may be input in some cases
    Integer(t_int),Intent(inout) :: nmax
    !! point, where to match the outward and inward shoot, may be input in
    !! some cases
    Integer(t_int),Intent(inout) :: nfit
    !! potential (contains the magnetic field for scalar- and non
    !!-relativistic case)
    Real(t_real),Intent(in),Target :: vtr(mesh% nr)
    !! magnetic field (must be zero for scalar- and non-relativistic case)
    Real(t_real),Intent(in)      :: btr(mesh% nr)
    !! energy interval to confine search to
    Real(t_real),Intent(in)      :: emin,emax
    !! tolerance of wave function fit
    Real(t_real),Intent(in)      :: wftol
    !! on input initial guess of energy, on output energy=eigenvalue
    Real(t_real),Intent(inout)   :: EE
    !! on input guess of the matching parameter, on output resulting matching
    !! parameter
    Real(t_real),Intent(inout)   :: BIn(2)
    !! the wave function (solution)
    Real(t_real),Intent(out)     :: wf(mesh% nr,4,2)
    !! derivative d(wf)/d(w), only dummy here
    Real(t_real),Intent(in)      :: dwfdw(mesh% nr,4,2)
    !! dummy array for the bookkeeping of the shoots
    Real(t_real),Intent(out)     :: dy(mesh% nr,4,2)
    !! dummy array for integration
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !!
    !!LOCAL
    Character(*),Parameter:: nsubfunc="solver2"
    !!
    !! do not calculate the derivative d(wf)/d(w) here
    Logical(t_log),Parameter :: dodw=.False.
    !!
    !! total number of iterations
    Integer(t_int),Parameter :: totaliter=80_t_int
    !! maximum number of back loops
    Integer(t_int),Parameter :: totalback=30_t_int
    !! total number of starting iterations for searching initial energy
    Integer(t_int),Parameter :: starttotaliter=20_t_int
    !! maximum number of newton loops
    Integer(t_int),Parameter :: maxnewiter=40_t_int
    !!
    !! energy was shifted in shifte(...)
    Logical(t_log) :: shift
    !! not yet converged
    Logical(t_log) :: noconv
    !! last fallback is active
    Integer(t_int) :: lasttry
    !!
    !! number of nodes for energies EE,Eup,Edn
    Integer(t_int) :: node,nodeup,nodedn
    !! max mesh point where the solution is allowed to be non zero (to avoid
    !! underflow)
    Integer(t_int) :: nmax1
    !! point, where to match the outward and inward shoot
    Integer(t_int) :: nfit1
    !! various counter
    Integer(t_int) :: iback,titer,iter
    !! control the re-adjustment of the matching point
    Integer(t_int) :: istart
    !!
    !! local tolerance for convergence (subject to adjustment in case of
    !! panic)
    Real(t_real)   :: tol
    !! best solution found so far
    Real(t_real)   :: bestEE,bestdev
    !! upper/lower bound for energy search interval ( set up by checking
    !! number the node number)
    Real(t_real)   :: Eup,Edn
    !! portion of the mesh, which may contain the fitting mesh point
    !! (is subject to adjustment in case of panic)
    Real(t_real)   :: pmin
    !! mismatch of the outward and inward shoot at the matching point
    Real(t_real)   :: dif
    !! bookkeeping for the newton method: vec=energy,fvec=mismatch (dif)
    Real(t_real)   :: vec,vecn,fvec,fvecn
    !! if nfit!=nfit1 for a long time we release the tight restrictions on this
    !! change: nfittol keeps track of this
    Integer(t_int) :: nfittol,additer
    !! dummies
    Integer(t_int) :: ir,i0
    Real(t_real)   :: dedwdummy,norm,r,r2m,sL,xx,vv,r0,d
    Real(t_real),Pointer :: cvtr(:)
    !!
    nfittol=2
    dedwdummy=0.0_t_real
    If(odeorder/=2_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Odeorder ==4 : must use solver4!'")
    Endif
    !!
    !!--------------------------------------------------------------------
    If(prot) Write(outfile,1100)nsubfunc
    If(prot) Write(outfile,1101)nsubfunc,Trim(relat_name(dsc% relmode))
    If(prot) Write(outfile,"(a,': inital EE=',g30.16e2)")nsubfunc,EE
    If(prot) Write(outfile,"(a,': nodenr=',i4)")nsubfunc,nodenr
    If(prot) Write(outfile,"(a,': kap   =',i4)")nsubfunc,kap
    If(prot) Write(outfile,"(a,': emin  =',f20.7)")nsubfunc,emin
    If(prot) Write(outfile,"(a,': emax  =',f20.7)")nsubfunc,emax
1100 Format(a,': -START--------------------------------------------------')
1101 Format(a,':',1x,a)
    !!
    !!---------------------------------------------------------------------
    !!
    tol=wftol
    If(dsc% c2>=139.0_t_real**2) tol=Max(1.e-12_t_real,wftol)
    pmin=0.90_t_real
    !!
    !!
    bestEE=EE
    bestdev=1.e30_t_real
    !!
    !! set up start value for the outward shoot
    !!
    wf(2_t_int,LARGE_COMP1,1_t_int)=1.0_t_real
    wf(2_t_int,LARGE_COMP2,2_t_int)=1.0_t_real
    !!
    Eup=emax
    Edn=emin
    nodeup=1000_t_int
    nodedn=-1_t_int
    !!
    !!
    iback=0_t_int
    istart=0_t_int
    titer=0_t_int
    iter=0_t_int
    !!
    lasttry=0
    cvtr=>vtr
    !!
    !!
11  Continue
    If(prot) Write(outfile,"(a,': start iteration ',i4)")nsubfunc,titer
    !!     loop over initial energy, to get right number of nodes
    !!
    If(prot) Then
       Write(outfile,674) nsubfunc,EE,emax,emin
    Endif
674 Format(a,': EE=',f20.7,' emax=',f20.7,' emin=',f20.7)
    !!
    If(EE>emax) EE=(emax-emin)*0.90_t_real+emin
    If(EE<emin) EE=(emax-emin)*0.10_t_real+emin
    shift=.True.
    iter=0_t_int

    nmax=nzeros(mesh% nr,mesh% p_rmesh,cvtr,EE,1060.0_t_real,dsc)
    nfit=classturn(nmax,dsc% p_vc,EE,pmin,.False.)
    nfit=Min(nfit,nmax-pred_order)
    additer=0
    shiftloop: Do While(iter<starttotaliter+additer.And.shift)
       iter=iter+1_t_int

       nmax=nzeros(mesh% nr,mesh% p_rmesh,cvtr,EE,1060.0_t_real,dsc)
       If(prot) Then
          Write(outfile,675) nsubfunc,iter,pmin,nmax,EE
       Endif
675    Format(a,': iter=',i5,' pmin=',f20.7,' nmax=',i5,' EE=',f20.7)
       
       nfit=classturn(nmax,dsc% p_vc,EE,pmin,.False.)
       nfit=Min(nfit,nmax-pred_order)

       If(prot) Then
          Write(outfile,676) nsubfunc,nfit,nmax
       Endif
676    Format(a,': nfit=',i5,' nmax=',i5)
       
       Call odeint(mesh,nmax,cvtr,btr,kap&
            &     ,EE,dedwdummy,BIn,wf,dy&
            &     ,dwfdw,odeorder,dodw,nfit,dif,odeintprot,tmp&
            &     ,dsc)
       

       node=countnodes(wf,mesh% nr,nmax,odeorder)

       If(node>nodenr)Then
          If(nodeup>node)Then
             nodeup=node
          Endif
          If(EE<Eup) Eup=EE+0.1_t_real*Abs(EE)
       Endif
       If(node<nodenr)Then
          If(nodedn<node)Then
             nodedn=node
          Endif
          If(EE>Edn) Edn=EE-0.1_t_real*Abs(EE)
       Endif
       
       If(iter>starttotaliter*0.75)Then
          additer=starttotaliter
          d=Eup
          Eup=Eup+(d-Edn)*0.5
          Edn=Edn-(d-Edn)*0.5
          If(prot) Write(outfile,"(a,': correct Edn,(E,)Eup=',i5,3(f30.15))")nsubfunc,iter,Edn,EE,Eup
       Endif
       
       If(iter==1_t_int .And. node == nodenr )Exit shiftloop
       If(prot) Then
          Write(outfile,678) nsubfunc,ee,eup,edn,node,nodenr,nfit,shift
       Endif

       shift=shifte(EE,eup,edn,node,nodenr,mesh% nr,nfit,wf,.False.,odeorder)
       If(prot) Then
          Write(outfile,679) nsubfunc,ee,eup,edn,node,nodenr,nfit,shift
       Endif

678    Format(a,' before shifte: EE=',f20.7,' Eup=',f20.7,' Edn=',f20.7&
            &,/,t10,' node=',i5,' nodenr=',i5,' nfit=',i5,' shift',l3)
679    Format(a,'after shifte : EE=',f20.7,' Eup=',f20.7,' Edn=',f20.7&
            &,/,t10,' node=',i5,' nodenr=',i5,' nfit=',i5,' shift',l3)
       
    Enddo shiftloop
    
    
    !     next integration
    
    nmax1=nzeros(mesh% nr,mesh% p_rmesh,cvtr,EE,1060.0_t_real,dsc)
    nfit1=classturn(nmax,dsc% p_vc,EE,pmin,.False.)
    !Print *,'nfit1=',nfit1
    nfit1=Min(nfit1,nmax-pred_order)

    If((Abs(nfit1-nfit)>20_t_int.Or.Abs(nmax-nmax1)>20_t_int).And. additer==0)Then
       Eup=emax
       Edn=emin
       goto 11
    Endif




    !! initial newton step

    
    Call odeint(mesh,nmax,cvtr,btr,kap&
         &   ,EE,dedwdummy,BIn,wf,dy,dwfdw&
         &   ,odeorder,dodw,nfit,dif,odeintprot,tmp,dsc)


    node=countnodes(wf,mesh% nr,nmax,odeorder)

    Call fitsol(EE,dif,vec,vecn,fvec,fvecn,fit_init)


    !     loop over iteration of energy to fit the solution
    !
    !     start of EE-iteration -------------------------------
    !
    iter=0_t_int
    noconv=.True.
    newtonloop: Do While(titer<totaliter.And.noconv.And.iback<totalback)
       If(prot) Write(outfile,"(a,': newtonloop')")
       iter=iter+1_t_int
       titer=titer+1_t_int
       !       if it is jumping around, just kill it be restarting
       If(iter>maxnewiter) Then
          iback=iback+1_t_int
          Goto 11
       Endif
       If(iter>1_t_int) Then
          node=countnodes(wf,mesh% nr,nmax,odeorder)
          If(shifte(EE,eup,edn,node,nodenr,mesh% nr,nfit,wf,.True.,odeorder))Then
             istart = istart + 1_t_int
             If(istart<200_t_int)  Then
                Goto 11
             Endif
          Endif
       Endif

       !! newton step dif at new point
       
       Call odeint(mesh,nmax,cvtr,btr,kap&
            &     ,EE,dedwdummy,BIn,wf,dy&
            &     ,dwfdw,odeorder,dodw,nfit,dif,odeintprot,tmp&
            &     ,dsc)
       

       If(prot)Write(outfile,234)nsubfunc,EE,dif,nsubfunc,mesh% nr,nmax,nfit

       If(Abs(dif)<bestdev)Then
          ! best sampling
          bestEE=EE
          bestdev=Abs(dif)
       Endif

       Call fitsol(EE,dif,vec,vecn,fvec,fvecn,fit_loop)


       !
       noconv=.Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol))
       If(prot)Write(outfile,235)nsubfunc,vec,vecn,nsubfunc,vecn-vec
    Enddo newtonloop


    nfit1=classturn(nmax,dsc% p_vc,EE,pmin,.False.)
    nfit1=Min(nfit1,nmax-pred_order)

    If(prot)Then
       Write(outfile, 203)nsubfunc,nfit,mesh% p_rmesh(nfit)
       Write(outfile, 201)nsubfunc,nfit1,mesh% p_rmesh(nfit1)
    Endif

    If(prot) Write(outfile,"(a,': nfittol=',i4)") nsubfunc,nfittol
    If(titer>60)nfittol=nfittol+2
    If(Abs(nfit-nfit1)>nfittol) Then
       If(prot) Write(outfile, 202)nsubfunc,nfit,nfit1
       Goto 11
    Endif

    If(lasttry==1)Then
       lasttry=lasttry+1
       cvtr=>vtr
       titer=0_t_int
       iback=0_t_int
       tol=Max(1.e-12_t_real,wftol)
       bestEE=EE
       bestdev=1.e30_t_real
       !! switch back to real potential and see if it gets through
       If(prot) Write(outfile,"(a,': reverse kick off last fallback method')")&
            &nsubfunc
       Goto 11
    Endif


    If(noconv.Or..Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol)))Then
       restart=restart+1_t_int
       If(pmin<0.989_t_real)Then
          pmin=Min(0.99_t_real,(1.0_t_real+pmin)*0.5_t_real)
          titer=0_t_int
          iback=0_t_int
          If(prot) Write(outfile,"(a,': adjust turningpoint to '"&
               &     //",g30.16e2)") nsubfunc,pmin
          Goto 11
       Else If(tol<1.e-7_t_real)Then
          tol=tol*10.0_t_real
          titer=0_t_int
          iback=0_t_int
          If(prot) Write(outfile,"(a,': decrease accuracy to '"&
               &     //",g30.16e2)") nsubfunc,tol
          Goto 11
       Endif
    Endif
    If(noconv.Or..Not.((Abs(dif)<tol).Or.numequ(vec,vecn,tol)))Then

       If(lasttry==0)Then
          lasttry=lasttry+1
          cvtr=>dsc% p_vtrcorrected
          titer=0_t_int
          iback=0_t_int
          tol=Max(1.e-12_t_real,wftol)
          !! everything failed, switch to modified potential, with
          !! possible unimportant minima remove.
          bestEE=EE
          bestdev=1.e30_t_real
          If(prot) Write(outfile,"(a,': kick off last fallback method')")&
               &nsubfunc
          goto 11
       Endif
       
       Write(errbuf,1001) titer,fvec
1001   Format(''' Sorry, no solutions found after ',i4,' iterations '',/&
            &,'' Cost function ',1(g30.15e3,3x),''',/')

       Call errmsg(nmodule,nsubfunc,E_warning&
            &,msg=errbuf)
    Endif
    EE=bestEE
    dif=bestdev
    If(prot) Write(outfile,236)nsubfunc,EE,nsubfunc,vecn-vec,nsubfunc,dif





    Call odeint(mesh,nmax,vtr,btr,kap&
         &   ,EE,dedwdummy,BIn,wf,dy,dwfdw&
         &   ,odeorder,dodw,nfit,dif,odeintprot,tmp,dsc)
    
    node=countnodes(wf,mesh% nr,nmax,odeorder)
    If(node/=nodenr)Then
       Write(errbuf,1000)node,nodenr
1000   Format("'final node number incorrect, sorry! found ",i3," need ",i3,"'")
!       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
    Endif
    !!
    !! finally normalize
    !!
    !!
    Call normwf(mesh,nmax,wf,norm,odeorder,tmp,dsc,EE,vtr)
    !!
    If(dsc% relmode == SCALAR_RELATIVISTIC_KH)Then
       sL=Sqrt(Real(dsc% la*(dsc% la+1),t_real))
       r0=nuc_max_rad(dsc% nuc_type,dsc% ZZ)
       i0=1
       If(dsc% nuc_type /= FINUC_NONE ) i0=2
       Do ir=i0,mesh% nr
          r=mesh% p_rmesh(ir)
          xx=r/r0
          vv=vtr(ir)-dsc% ZZ*nuc_zeta(xx,dsc% nuc_type)
          r2m=2.0_t_real*r+(r*EE-vv)/dsc% c2
          wf(ir,SMALL_COMP2,1)=sL*wf(ir,LARGE_COMP1,1)/r2m
       Enddo
       If(dsc% nuc_type /= FINUC_NONE )then
          If(dsc% la == 1)Then
             wf(1,SMALL_COMP2,1)=wf(1,SMALL_COMP1,1)/sq2o2
          Else
             wf(1,SMALL_COMP2,1)=0.0_t_real
          Endif
       Endif
    Else
       wf(:,LARGE_COMP2,1:2)=0.0_t_real
       wf(:,SMALL_COMP2,1:2)=0.0_t_real
    Endif
    !!
    If(prot) Write(outfile,"(a,':-END----------'"&
         &   //",'-------------------------------')")nsubfunc
    !!
    !!
    Return
    !!
    !!
201 Format (a,': new fitting point=r(',i4,')=',f10.4)
203 Format (a,': old fitting point=r(',i4,')=',f10.4)
202 Format (a,': old fitting point ',i4&
         & ,' deviates to much from new ',i4,' do it again')
235 Format(a,': Eold=',g30.16e2,' Enew=',g30.16e2&
         & ,/,a,': Delta=',g30.16e2)
236 Format(a,': E =',g30.16e2&
         & ,/,a,': DeltaE =',g30.16e2&
         & ,/,a,': DeltaWF=',g30.16e2)
234 Format(a,': E=',g30.16e2,' dif=',g30.16e2&
         &   ,/,a,': nr=',i4,' nmax=',i4,' nfit=',i4)
    !!
  End Subroutine solver2
  !***********************************************************************
  ! shifts energy by a certain factor to make a new guess
  !
  ! if node is between match and match-1,
  ! increase of the energy shifts node towards zero
  ! if y(n+1)/y(n) ge 1 we have max/min or still are not at the tail
  !
  !
  Logical Function shifte(E,emax,emin,node,nodenr,nr,nfit,wf,inloop,odeorder)
    !
    Implicit None
    !!IO
    !     in initial loop: shift to the off falling tail in case when
    !     nodes are OK
    Logical(t_log),Intent(in)  :: inloop
    Integer(t_int),Intent(in) :: odeorder
    !! number of nodes
    Integer(t_int),Intent(in) :: node
    !! required nodenr
    Integer(t_int),Intent(in) :: nodenr
    !! nr mesh points,matching point
    Integer(t_int),Intent(in) :: nr,nfit
    !! nsol solutions
    Real(t_real),Intent(in)  :: wf(nr,2)
    !! energy
    Real(t_real),Intent(inout) :: E
    !! upper border of energy
    Real(t_real),Intent(in)  :: emax
    !! lower border of energy
    Real(t_real),Intent(in)  :: emin
    !!LOCAL
    Character(*),Parameter:: nsubfunc="shifte"
    Real(t_real) :: a,d
    !
    If(odeorder>2_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Odeorder > 2, not implemented!'")
    Endif


    a=E-emax
    shifte=.False.
    If(node==nodenr) Then

       If(inloop) Return
       d=wf(nfit,LARGE_COMP1)/wf(nfit-1_t_int,LARGE_COMP1)
       If(d<=0.0_t_real.Or.d>=1.0_t_real)Then
          E=a*0.7_t_real+emax
          shifte=.True.
          Return
       Endif
    Else
       If(node<nodenr) Then
          E=E*0.75_t_real+emax*0.25_t_real
       Else
          E=(emin-E)*.32_t_real+E
       Endif
       shifte=.True.
       Return
    Endif
    !
    !
  End Function shifte
  !
  !***********************************************************************

  !***********************************************************************
  ! this gives the maximum mesh point allowed for numeric reasons
  ! the wave function is proportional to exp(-r sqrt(2(v-e)))
  ! this shall be a stable number, thus we claim
  ! r*sqrt(2(v-e))< a, with a being the maximum number allowed
  ! this gives r**2 (v-e)< a**2 /2 = amax
  ! by taking exp(-a)=1e-15 we get a=34.53, amax=600
  ! by taking exp(-a)=1e-20 we get a=46.05, amax=1060
  ! (inspired by a program of Hubert Ebert)
  !
  Integer(t_int) Function nzeros(nr,rmesh,vtr,E,amax,dsc)
    !
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    !! number of mesh points
    Integer(t_int),Intent(in) ::  nr
    !! mesh points
    Real(t_real),Intent(in) :: rmesh(nr)
    !! potential*r
    Real(t_real),Intent(in) :: vtr(nr)
    Real(t_real),Intent(in) :: E
    !! cutoff condition  600.0_t_real or 1060._t_real
    Real(t_real),Intent(in) :: amax

    !!LOCAL
    !Character(*),Parameter:: nsubfunc="nzeros"
    Integer(t_int) ::  i,nzero
    Real(t_real)  ::  cc,cc2!,c1
    Logical(t_log) :: go
    !
    nzero=nr
    outer:Do i=2_t_int,(nr-1_t_int)
       If( Abs(dsc% fx0) > 1.0e-6_t_real .Or. Abs(dsc% fx2) > 1.0e-6_t_real )Then
          !c1=(exn0*0.5_t_real+1.0_t_real)
          cc =(dsc% exn0*0.5_t_real+1.0_t_real)**2_t_int
          cc2=(dsc% exn2*0.5_t_real+1.0_t_real)**2_t_int
          !!
          go=(   (vtr(i)/rmesh(i))&
               &+dsc% fx0*(rmesh(i)/dsc% rx0)**dsc% exn0/cc&
               &+dsc% fx2*(rmesh(i)/dsc% rx2)**dsc% exn2/cc2&
               &-E&
               &)*rmesh(i)**2_t_int>amax
       Else
          go=((vtr(i)/rmesh(i))-E)*rmesh(i)**2_t_int>amax
       Endif
       If(go) Then
          If(Mod(i,2_t_int)==0_t_int) Then
             nzero=i+1_t_int
          Else
             nzero=i
          Endif
          Exit  outer
       Endif
    Enddo outer

    !
    nzeros=nzero
    !
  End Function nzeros

  !***********************************************************************

  !***********************************************************************
  !
  Integer(t_int) Function countnodes(wf,nr,nmax,odeorder)
    !
    Implicit None
    !!IO
    Integer(t_int),Intent(in)    :: odeorder 
    Integer(t_int),Intent(in) :: nr,nmax
    Real(t_real),Intent(in)  :: wf(nr,4,2)
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="countnodes"
    Logical(t_log) :: start
    Integer(t_int) :: ir,n,n1,n0,icomp
    Real(t_real) :: d ,tail,d2
    !
    tail=1.e-6_t_real

    !!
    icomp=LARGE_COMP1
    d=0.0_t_real
    Do ir=1,nmax
       d=d+Abs(Sum(wf(ir,LARGE_COMP1,1:odeorder/2)))
    Enddo
    d=d/Real(nmax,t_real)
    !!
    If(odeorder==4_t_int)Then
       d2=0.0_t_real
       Do ir=1,nmax
          d2=d2+Abs(Sum(wf(ir,LARGE_COMP2,1:odeorder/2)))
       Enddo
       d2=d2/Real(nmax,t_real)
       If(d2>d)Then
          d=d2
          icomp=LARGE_COMP2
       Endif
    Endif
    !
    !!Print *,'icomp=',icomp

    n=0_t_int
    n0=1_t_int
    If(Sum(wf(1_t_int,icomp,1:odeorder/2))==0.0_t_real) n0=2_t_int
    n1=nmax-2_t_int
    start=.False.

    ir=n1
    Do While(ir>=n0)
       If(.Not.start.And.Abs(Sum(wf(ir,icomp,1:odeorder/2)))>tail*d)Then
          start=.True.
       Endif
       If(start)Then
          If(Sum(wf(ir,icomp,1:odeorder/2))&
               &*Sum(wf(ir+1_t_int,icomp,1:odeorder/2))<0.0_t_real)Then
             n=n+1_t_int
          Elseif(Sum(wf(ir,icomp,1:odeorder/2))&
               &*Sum(wf(ir+1_t_int,icomp,1:odeorder/2))==0.0_t_real)Then
             n=n+1_t_int
             ir=ir-1_t_int
          Endif
       Endif
       ir=ir-1_t_int
    Enddo
    !

    countnodes=n
    !
  End Function countnodes
  !
  !***********************************************************************
  !! prepare for search of "classical" turning point
  !! many site extrema are replaced by a straight line
  Subroutine  prepclassturn(mesh,vtr,dsc)
    !!
    Implicit none
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(inout) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: vtr(mesh% nr)
    !!LOCAL
    Character(*),Parameter:: nsubfunc="prepclassturn"
    Logical(t_log) :: fillin
    Integer(t_int) :: ir,ifmin,i1,i2,imax,imin
    Character(100)   :: str
    Real(t_real)   :: d,dmax,dmin
    Logical(t_log) :: expectmax,deb
    !!
    deb=debugfun(nmodule,nsubfunc,'write_pot')
    !!
    Call palloc('dsc% p_vc',dsc% p_vc,1_t_int,mesh% nr,nmodule,nsubfunc)
    Call palloc('dsc% p_vtrcorrected',dsc% p_vtrcorrected,1_t_int,mesh% nr&
         &,nmodule,nsubfunc)
    !!
    dsc% p_vc(1)=0.0_t_real
    Do ir=2,mesh% nr
       dsc% p_vc(ir)=((vtr(ir)/mesh% p_rmesh(ir)) &
            &+dsc% fx0*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0&
            &+dsc% fx2*(mesh% p_rmesh(ir)/dsc% rx2)**dsc% exn2&
            &+ 0.50_t_real*dsc% la*(dsc% la+1_t_int)/mesh% p_rmesh(ir)**2_t_int )
    Enddo
    !!
    !! skip first part of mesh, 1.e-3 should be left of possible first minimum
    !! for all practical cases
    ifmin=2
    Do ir=2,mesh% nr
       If(mesh% p_rmesh(ir)>1.e-3_t_real)Then
          ifmin=ir
          Exit
       Endif
    Enddo
    If(deb) Print *,nsubfunc,' ifmin0=',ifmin,mesh% p_rmesh(ifmin)
    !!
    !! search first minimum for la>0
    !!
    If(dsc% la>0_t_int)Then
       i1=ifmin
       d=dsc% p_vc(i1)
       Do ir=i1+1,mesh% nr
          If(dsc% p_vc(ir)>d)Then
             ifmin=ir-1
             If(deb) Print *,nsubfunc,' la=',dsc% la,'ifmin=',ifmin,mesh% p_rmesh(ifmin)
             Exit
          Endif
          d=dsc% p_vc(ir)
       Enddo
    Else
       If(deb) Print *,nsubfunc,' la=',dsc% la,'ifmin=',ifmin
    Endif
    !!
    !! search maxima and minima right of first minimum
    expectmax=.True.
    imin=-1
    imax=-1
    !!Open(51,file='+trash',status='unknown')
    !!Write(51,"(2i8)")-1,ifmin
    dmin=0.0_t_real
    dmax=0.0_t_real
    d=dsc% p_vc(ifmin)
    Do ir=ifmin+1,mesh% nr
       If(expectmax)Then
          If(dsc% p_vc(ir)<d)Then
             If(imax<0)Then
                imax=ir-1
                dmax=dsc% p_vc(imax)
             Else
                If(dsc% p_vc(ir-1)>dmax)Then
                   imax=ir-1
                   dmax=dsc% p_vc(imax)
                Endif
             Endif
             If(deb) Print *,nsubfunc,' max=',ir-1,mesh% p_rmesh(ir-1)
             !!Write(51,"(2i8)")1,ir-1
             expectmax=.False.
          Endif
          d=dsc% p_vc(ir)
       Else
          If(dsc% p_vc(ir)>d)Then
             If(imin<0)Then
                imin=ir-1
                dmin=dsc% p_vc(imin)
             Else
                If(dsc% p_vc(ir-1)<dmin)Then
                   imin=ir-1
                   dmin=dsc% p_vc(imin)
                Endif

             Endif
             If(deb) Print *,nsubfunc,' min=',ir-1,mesh% p_rmesh(ir-1)
             !!Write(51,"(2i8)")0,ir-1
             expectmax=.True.
          Endif
          d=dsc% p_vc(ir)
       Endif
    Enddo
    !!
    If(imin<0 .And. imax>0) Then
       imin=mesh% nr
       If(deb) Print *,nsubfunc,' imin<0'
    Endif
    If(deb) Print *,nsubfunc,' MAX=',imax!,mesh% p_rmesh(imax)
    If(imax>0)Then
       If(deb) Print *,nsubfunc,' MAX=',imax,mesh% p_rmesh(imax)
       !!Write(51,"(2i8)")2,imax
    Endif
    If(deb) Print *,nsubfunc,' MIN=',imin!,mesh% p_rmesh(imin)
    If(imin>0)Then
       If(deb) Print *,nsubfunc,' MIN=',imin,mesh% p_rmesh(imin)
       !!Write(51,"(2i8)")-2,imin
    Endif

    !!Close(51)
    !!
    i1=-1
    i2=-1
    If(imax>0)Then
       If(imin>0)Then
          If(dsc% p_vc(ifmin)>dsc% p_vc(imin))Then
             !! dont know what to do in this case
          Else
             d=dsc% p_vc(imin)
             i1=ifmin
             Do ir=ifmin,mesh% nr
                If(dsc% p_vc(ir)>=d)Then
                   i1=ir
                   Exit
                Endif
             Enddo
          Endif
       Else
          !! no other minimum
          If(dsc% p_vc(ifmin)>dsc% p_vc(mesh% nr))Then
             i1=ifmin
          Else
             d=dsc% p_vc(mesh% nr)
             i1=ifmin
             Do ir=ifmin,mesh% nr
                If(dsc% p_vc(ir)>=d)Then
                   i1=ir
                   Exit
                Endif
             Enddo
          Endif
       Endif!!imin=-1

       d=dsc% p_vc(imax)
       i2=mesh% nr
       Do ir=mesh% nr,imax,-1
          If(dsc% p_vc(ir)<=d)Then
             i2=ir
             Exit
          Endif
       Enddo

    Endif
    If(deb) Then
       Print *,nsubfunc,' i1=',i1
       Print *,nsubfunc,' i2=',i2
       Print *,nsubfunc,' imin=',imin
       Print *,nsubfunc,' imax=',imax
    Endif
    !! fill in linear part ro remove the minima,maxima, if possible and meaningful
    fillin=.False.
    If(i1 > 0_t_int .And. i2 > 0_t_int .And.  i2>i1 )Then
       If(deb) Print *,nsubfunc,' fillin'
       fillin=.True.
       dmin=dsc% p_vc(imin)
       dmax=dsc% p_vc(imax)
       Do ir=i1,i2
          dsc% p_vc(ir)=dmin*(mesh% p_rmesh(ir)-mesh% p_rmesh(i2))&
               &/(mesh% p_rmesh(i1)-mesh% p_rmesh(i2))&
               &+dmax*(mesh% p_rmesh(ir)-mesh% p_rmesh(i1))&
               &/(mesh% p_rmesh(i2)-mesh% p_rmesh(i1))
       Enddo

    Endif
    !!
    !!
    !! create a vtr with the correction for a fallback
    dsc% p_vtrcorrected(1)=0.0_t_real
    Do ir=2,mesh% nr
       d=(dsc% fx0*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0&
            &+dsc% fx2*(mesh% p_rmesh(ir)/dsc% rx2)**dsc% exn2&
            &+ 0.50_t_real*dsc% la*(dsc% la+1_t_int)/mesh% p_rmesh(ir)**2_t_int )
       dsc% p_vtrcorrected(ir)=(dsc% p_vc(ir)-d)*mesh% p_rmesh(ir)
    Enddo
    !!
    !!
    If(  deb) Then
       If(fillin)Then
          Write(outfile,"('write +prepclassturn.',i5.5)")fillins
          Write(str,"('+prepclassturn.',i5.5)")fillins
          fillins=fillins+1
          Open(13,file=Trim(str))
          Do ir=1,mesh% nr
             d=((vtr(ir)/mesh% p_rmesh(ir)) &
                  &+dsc% fx0*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0&
                  &+dsc% fx2*(mesh% p_rmesh(ir)/dsc% rx2)**dsc% exn2&
                  &+ 0.50_t_real*dsc% la*(dsc% la+1_t_int)/mesh% p_rmesh(ir)**2_t_int )
             Write(13,10) mesh% p_rmesh(ir),dsc% p_vc(ir)*mesh% p_rmesh(ir)&
                  &,d*mesh% p_rmesh(ir),vtr(ir),dsc% p_vtrcorrected(ir)
          Enddo
          Close(13)
       Endif
10     Format(5(e30.15e3,2x))
    Endif
    !!
  End Subroutine prepclassturn
  !***********************************************************************
  ! calculate classical turning point, there all nodes should be
  ! included in the outward integration
  !
  !
  Integer(t_int) Function classturn(nr,vc,E,pmin,prot)
    !
    Implicit None
    !!IO
    Logical(t_log),Intent(in)  :: prot
    !! number of mesh points
    Integer(t_int),Intent(in)  :: nr
    !!! mesh points
    !Real(t_real),Intent(in)  :: rmesh(nr)
    !! potential *r
    Real(t_real),Intent(in)  :: vc(nr)
    !! energy
    Real(t_real),Intent(in)  :: E
    Real(t_real),Intent(in)  :: pmin
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="classturn"
    Logical(t_log) :: found
    Integer(t_int) :: n0,i,imin,n1,nmin,nmax

    Real(t_real)  :: d,dmin
    !
    !
    n0=nr
    i=nr
    dmin=vc(i)
    imin=nr
    found=.False.
    Do i=nr-1_t_int,2_t_int,-1_t_int

       d=vc(i)
       If(d<dmin)Then
          dmin=d
          imin=i
       Endif

       If(.Not.found.And.(d-E<0.0_t_real)) Then
          n0=i
          !!Print *,'found n0=',n0,rmesh(n0)
          found=.True.
       Endif
    Enddo


    nmin=Max(2_t_int,Int(nr*0.1_t_real,t_int))
    nmax=Min(Int(nr*0.99_t_real,t_int),nr-5_t_int)
    If(n0>nmax.Or.n0<nmin) Then
       n1=Int(nr*pmin,t_int)
       If(imin>=nmin.And.imin<=nmax)Then
          n1=imin
          !!Print *,'n1=imin =',n1,rmesh(n1)
       Endif
       If(prot)Then
          Write(outfile, 1000) n0,n1
       Endif
       n0=n1
       ! old comment, may be irrelevant
       !       K.Koepernik 6.Jan.2000
       !       I found for 4f-states it is better to take n0 close to nzero
       !       this is done outside if n0=nr
    Endif
    !
    classturn=n0
    !!Print *,'classturn=',classturn,rmesh(classturn)
    !
1000 Format('NCLASSTURN: matching point lies at border of the ',&
         & 'mesh, nmatch: ',i4,' will try ',i4)
    !
  End Function classturn
  !
  !***********************************************************************

  !***********************************************************************
  ! this routine fits the solutions by changing the energy
  ! via a Newton Raphson method
  ! this is nsol=2 version
  !
  !
  !
  Subroutine fitsol(E,dif,vec,vecn,fvec,fvecn,typ)
    !
    Implicit None
    !!IO
    !! typ of entry 0:initial, 1:in the loop
    Integer(t_int),Intent(in) :: typ
    !! the energy (first parameter)
    Real(t_real),Intent(inout) :: E
    !! the fit coefficient of outward and inward solution
    Real(t_real),Intent(in)  :: dif
    !! the solution vector of the previous solution
    Real(t_real),Intent(inout) :: vec
    !! the cost function vector of the previous solution
    !! fvec=f(out,i)-B*f(in,i) ,  i =1,nsol
    Real(t_real),Intent(inout) :: fvec
    !! the solution vector of the new solution
    Real(t_real),Intent(inout) :: vecn
    !! the cost function vector of the new solution
    Real(t_real),Intent(inout) :: fvecn
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="fitsol"
    Real(t_real):: det
    !

    If (typ==fit_init) Then
       vec=E
       fvec=dif
       E=E*1.0001_t_real
       vecn=E
       Return
    Else

       fvecn=dif
       If(Abs(vecn-vec)<1.e-20_t_real) Then
          det=1.0_t_real
       Else
          det=(fvecn-fvec)/(vecn-vec)
       Endif
       vec=vecn
       If(det==0.0_t_real)Then
          det=1.0_t_real
       Endif
       vecn=vec-fvecn/det
       fvec=fvecn
       E=vecn
       Return
       !
    Endif                     ! typ=fit_loop
    !
  End Subroutine fitsol
  !
  !***********************************************************************

  !***********************************************************************
  !
#ifdef my_compile_unused
  Subroutine dwfdwdirac(mesh,nmax,nfit,btr,vtr,kap&
       & ,E,dedw,wf0,dy,dwfdw&
       & ,prot,odeorder,wftol,tmp,dsc)

    !
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    Logical(t_log),Intent(in)  :: prot
    Integer(t_int),Intent(in)  :: kap,nmax,nfit,odeorder
    !! dummy array for integration
    Real(t_real),Intent(out)   :: tmp(mesh% nr)
    Real(t_real),Intent(in)    :: vtr(mesh% nr)
    Real(t_real),Intent(in)    :: btr(mesh% nr)
    Real(t_real),Intent(in)    :: wftol
    !! ODE solution
    Real(t_real),Intent(in)    :: wf0(mesh% nr,4,2)
    !! aux array
    Real(t_real),Intent(out)   :: dy(mesh% nr,4,2)
    !! IODE solution d(wf)/d(w)
    Real(t_real),Intent(out)   :: dwfdw(mesh% nr,4,2),E,dedw
    !!LOCAL
    Character(*),Parameter:: nsubfunc="dwfdwdirac"
    Logical(t_log) :: noconv,trial,deb
    Logical(t_log),Parameter :: dodw=.True.

    Integer(t_int) :: titer,totaliter,rest,ir
    Real(t_real)   :: dfit,dedw0,BIn(2),tol,ortho
    !     auxillary vars
    Real(t_real)   :: vec,vecn,fvec,fvecn,bestdev,bestdedw

    !!***********************************************************************
    !
    !
    tol=Max(wftol,1.e-10_t_real)
    
!!    If(.Not.dsc% compr)Return
    !!
    If(dsc% gam_factor) Stop 'mismatch: gam_factor and dedw !'
    !!
    deb=debugfun(nmodule,"odeint",'IODE')

    !
    dwfdw(2_t_int,LARGE_COMP1,1_t_int)=1.0_t_real
    dwfdw(2_t_int,LARGE_COMP2,2_t_int)=1.0_t_real
    BIn(1)=1.0_t_real
    BIn(2)=1.0_t_real
    trial=.True.
    !
    !! dE/dw=<phi|W0*r^exn0|phi>
    If(dsc% relmode == NON_RELATIVISTIC)Then
       tmp(1_t_int:nmax)=wf0(1_t_int:nmax,LARGE_COMP1,1_t_int)**2_t_int&
            &*(mesh% p_rmesh(1_t_int:nmax)/dsc% rx0)**dsc% exn0&
            &*mesh% p_jmesh(1_t_int:nmax)
    Else
       If(odeorder==2)Then
          tmp(1_t_int:nmax)=(wf0(1_t_int:nmax,LARGE_COMP1,1_t_int)**2_t_int)&
               &*(mesh% p_rmesh(1_t_int:nmax)/dsc% rx0)**dsc% exn0&
               &*mesh% p_jmesh(1_t_int:nmax)
       Else
          Do ir=1,nmax
             tmp(ir)=(&
                  & Sum(wf0(ir,LARGE_COMP1,1:2))**2_t_int&
                  &+Sum(wf0(ir,LARGE_COMP2,1:2))**2_t_int&
                  &)&
                  &*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0*mesh% p_jmesh(ir)
          End Do
       Endif
    End If


    Call lag4int(nmax,tmp,mesh% hx,dedw,lagint_out)


    !! Print *,'dedw0=',dedw
    !! Print *,'pred_order=',pred_order
    !! Print *,'wi nfit=0',nfit,dedw
    
    Call odeint(mesh,nmax,vtr,btr,kap&
         & ,E,dedw,BIn,dwfdw,dy,wf0&
         & ,odeorder,dodw,nfit,dfit,deb,tmp,dsc)


    !
    If(trial)Then

       !
       !       calibrate dedw to minimize errors
       !
       bestdedw=dedw
       bestdev=1.e30_t_real

       !
       If(prot)&
            &Write(outfile,"(a,': DEDW improvment start: ',g30.16e2)")&
            &nsubfunc,dedw
       dedw0=dedw

       
       Call odeint(mesh,nmax,vtr,btr,kap&
            &   ,E,dedw,BIn,dwfdw,dy,wf0&
            &   ,odeorder,dodw,nfit,dfit,deb,tmp,dsc)

       If(prot) Write(outfile,234)nsubfunc,dedw,dfit,nsubfunc,mesh% nr,nfit

       vec=dedw
       fvec=dfit
       dedw=dedw*(1.0_t_real+1.0e-6_t_real)
       !!dedw=dedw+Min(Abs(emax-emin)*1.0e-4_t_real,1.0e-4_t_real)
       vecn=dedw
    
       !!Call fitsol(dedw,dfit,vec,vecn,fvec,fvecn,fit_init)

       !
       !       loop over iteration of denergy/dw to fit the solution
       !
       !
       rest=1_t_int
11     Continue

       totaliter=10_t_int
       titer=0_t_int
       noconv=.True.
       Do While(titer<totaliter.And.noconv)
          titer=titer+1_t_int

          Call odeint(mesh,nmax,vtr,btr,kap&
               &     ,E,dedw,BIn,dwfdw,dy,wf0&
               &     ,odeorder,dodw,nfit,dfit,deb,tmp,dsc)

          Call fitsol(dedw,dfit,vec,vecn,fvec,fvecn,fit_loop)
          !         best sampling
          If(Abs(dfit)<bestdev)Then
             bestdedw=dedw
             bestdev=Abs(dfit)
          Endif


          If(prot) Write(outfile,234)nsubfunc,dedw,dfit,nsubfunc,mesh% nr,nfit
234       Format(a,': DEDW=',g30.16e2,' dif=',g30.16e2&
               &     ,/,a,': nr=',i4,' nfit=',i4)

          noconv=(Abs(dfit)>tol)
          noconv=noconv.Or.(Abs(vecn-vec)>tol)

          If(numequ(vec,vecn,tol))noconv=.False.
          If(prot) Write(outfile,235)nsubfunc,vec,vecn,nsubfunc,vecn-vec
235       Format(a,': DEDWold=',g30.16e2,' DEDWnew=',g30.16e2&
               &     ,/,a,': Delta=',g30.16e2)

       Enddo                   ! while totaliter

       If(noconv.Or.Abs(dfit)>tol)Then
          If(tol<1.e-7_t_real)Then
             tol=tol*10.0_t_real
             titer=0_t_int
             If(prot) Write(outfile,"(a,': decrease accuracy to '"//&
                  &",g30.16e2)") nsubfunc,tol
             Goto 11
          Endif
       Endif
       If(rest==1_t_int)Then
          rest=2_t_int
          titer=0_t_int
          Goto 11
       Endif
       If(noconv.Or.Abs(dfit)>tol)Then
          Write(outfile,1001)nsubfunc,titer,fvec
       Endif
       !! Print *,'dedw=',dedw
       dedw=bestdedw
       dfit=bestdev
       !! Print *,'dedw=',dedw,bestdev
       If(prot) Write(outfile,236)nsubfunc,dedw&
            &,nsubfunc,vecn-vec&
            &,nsubfunc,dfit&
            &,nsubfunc,dedw-dedw0
236    Format(a,': DEDW =',g30.16e2&
            &   ,/,a,': DeltaDEDW =',g30.16e2&
            &   ,/,a,': DeltaDWFDW=',g30.16e2 &
            &   ,/,a,': improvement=',g30.16e2 )


       If(Abs(dedw-dedw0)>1.0e-6_t_real .and. outlevel>=L_MANY)Then
          Write(outfile,236)nsubfunc,dedw&
               &,nsubfunc,vecn-vec&
               &,nsubfunc,dfit&
               &,nsubfunc,dedw-dedw0
          Write(outfile,237)nsubfunc,nsubfunc
237       Format(a,': improvement to large: increase number '&
               & ,/,a,': of points in atomic mesh, if not yet >=1000!!!')
       Endif

       Call odeint(mesh,nmax,vtr,btr,kap&
            &   ,E,dedw,BIn,dwfdw,dy,wf0&
            &   ,odeorder,dodw,nfit,dfit,deb,tmp,dsc)

    Endif ! trial


    Call orthowfdwf(mesh,nmax,dwfdw,wf0,ortho,odeorder,dsc)
    If(prot) Write(outfile,"(a,': ORTHO=',g30.15e3)")nsubfunc,ortho

    !
1001 Format(a,': Sorry, no solutions found after ',i4&
          & ,' iterations ',/,'Cost function ',1(g30.15e3,3x))
    !

  End Subroutine dwfdwdirac
#endif
  !
  !***********************************************************************
  Logical Function numequ(x0,x1,tol)

    Implicit None
    !!IO
    Real(t_real),Intent(in):: x0,x1,tol
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="numequ"
    numequ=(x0-x1==0.0_t_real).Or.(Abs(x0-x1)<tol)
  End Function numequ



  !***********************************************************************
  !
  ! out and inward integration of the set of linear differential
  ! equations (ODE) with simplified  predictor corrector  and with
  ! relaxation method
  !
  !
  Subroutine odeint(mesh,nmax,vtr,btr,kap&
       & ,E,dedw,BIn,wf,dy,wf0,odeorder&
       & ,dodw,nfit,dif,prot,tmp,dsc)
    !
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    !! spin orbit quantum number
    Integer(t_int),Intent(in) :: kap
    !! order of ODE,IODE
    Integer(t_int),Intent(in) :: odeorder
    !! maximum mesh points to calculate at (underflow protection)
    Integer(t_int),Intent(in) :: nmax
    !! matching point
    Integer(t_int),Intent(in) :: nfit 
    !!
    !! verbose ?
    Logical(t_log),Intent(in) :: prot
    !! make d(wf)/d(w)
    Logical(t_log),Intent(in) :: dodw
    !! d(E)/d(w)
    Real(t_real),Intent(in)   :: dedw
    !! potential times r
    Real(t_real),Intent(in)   :: vtr(mesh% nr)
    !!! for variation of conf. pot. (inhomogenity of IODE)
    Real(t_real),Intent(in)   :: wf0(mesh% nr,4,2)
    !! magnetic field times r
    Real(t_real),Intent(in)   :: btr(mesh% nr)
    !! eigenvalue
    Real(t_real),Intent(inout) :: E
    !! factor for inward solution
    Real(t_real),Intent(inout) :: BIn(2)
    !! array of solutions and first derivative
    Real(t_real),Intent(inout) :: wf(mesh% nr,4,2)
    Real(t_real),Intent(out)   :: dy(mesh% nr,4,2)
    !! error of fitting
    Real(t_real),Intent(out)   :: dif
    !! dummy array for integration 
    Real(t_real),Intent(out)   :: tmp(mesh% nr)
    !!LOCAL
    Character(*),Parameter:: nsubfunc="odeint"
    Logical(t_log) :: deb
    Integer(t_int) :: ir,n1,n2,n3,n4,j&
         !     typ of integration, OUT=1 or IN=-1
         & ,typ
    !! which partial wave
    Integer(t_int) :: ipart
    Real(t_real)   :: norm,ortho,Bout(2),mj,BIn0(2),dwf(2,4)
    !! values at matching points
    Real(t_real)   :: pio(4,2,-1:1),AA(4,4),CC(4)
    !! spin factors for relativistic spin polarized
    Real(t_real)   :: cm(5)
    !!
    !!
    !***********************************************************************
    !!
    !!
    If(nfit<pred_order.Or.nfit>nmax-pred_order)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Nfit out of range!'")

    Endif
    !!
    If((dsc% relmode /= NON_RELATIVISTIC).And.&
         &(dsc% relmode /= SCALAR_RELATIVISTIC_KH)&
         &.And.kap==0_t_int)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Kappa is zero ??!'")
    Endif
    !!

    !!
    !! preparation
    !!
    !!
    mj=dsc% mu*0.5_t_real
    cm(kap_kap)     =-(mj/(kap+0.5_t_real))
    cm(kapq_kap)   =-Sqrt(1.0_t_real-cm(kap_kap)**2_t_int)
    cm(kapq_kapq)  =-cm(kap_kap)
    cm(mkap_mkap)  =-(mj/(-kap+0.5_t_real))
    cm(mkapq_mkapq)=-(mj/(kap+1.5_t_real))
    !!
    !!
    BIn0(1:2)=BIn(1:2)
    ipartloop: Do ipart=1,(odeorder/2_t_int)
       !!
       BOut(ipart)=wf(2_t_int,LARGE_COMP1+(ipart-1)*2,ipart)
       !!
       If(prot)Then
          Write(outfile,1)nsubfunc,ipart
          Write(outfile,2)nsubfunc,ipart,BIn0(ipart)
          Write(outfile,3)nsubfunc,ipart,Bout(ipart)
       Endif
       !!
       !!
       If(BIn(ipart)==0.0_t_real)BIn(ipart)=1.0e-30_t_real
       !!
       outinloop: Do typ=OUT,IN,-2_t_int
          !! initialize
          If(typ==IN)Then
             n1=nmax
             n2=nmax
             n3=nmax-pred_order
             n4=nfit
             !Print *,'=============== IN ================='
          Else
             n1=2_t_int
             n2=pred_order
             n3=pred_order
             n4=nfit
             If( dsc% gam_factor )Then
                If(order0(dsc,kap)) Then
                   n1=1
                   n2=1
                   n3=1+pred_order
                   n4=nfit
                Else
                   n1=2_t_int
                   n2=2
                   n3=2+pred_order
!                   n2=pred_order
!                   n3=pred_order
                   n4=nfit
                Endif
             Endif
             !Print *,'=============== OUT ================='

          Endif
          !!
          !! ode solve
          !!
          !! start values
          !!
          !!
          !
          !dy(1,1:odeorder,ipart)=0.0_t_real
          If( typ == OUT .And. &
               &(.Not. ( dsc% gam_factor .And. &
               &order0(dsc,kap))) )Then
             wf(1,1:4,1:2)=0.0_t_real  
          Endif
          Do ir=n1,n2,typ
             Call fstart(mesh% nr,mesh% p_rmesh,ir,cm,E,dedw,wf,wf0,dwf&
                  &,BOut(ipart),BIn(ipart),kap,odeorder&
                  &,ipart,dodw,typ,dsc)
             !!
             !Print *,'wf1  =',ir,wf(ir,1:2,1)
             !Print *,'dwfPo=',ir,dwf(1:2,ipart)

             If(ir/=1)&
             &Call odekernel(mesh% p_rmesh(ir),mesh% p_jmesh(ir)&
                  &   ,vtr(ir),btr(ir),cm&
                  &   ,kap,E,dedw&
                  &   ,Sum(wf0(ir,LARGE_COMP1,1:odeorder/2))&
                  &   ,Sum(wf0(ir,SMALL_COMP1,1:odeorder/2))&
                  &   ,Sum(wf0(ir,LARGE_COMP2,1:odeorder/2))&
                  &   ,Sum(wf0(ir,SMALL_COMP2,1:odeorder/2))&
                  &   ,AA,CC,odeorder,dodw,dsc)
             Do j=1_t_int,odeorder
                If(ir==1)Then
                   If(dsc% gam_factor )Then
                      dy(ir,j,ipart)=dwf(j,ipart)*mesh% p_jmesh(ir)
                   Else
                      dy(ir,j,ipart)=0.0_t_real
                   Endif
                Else
                   dy(ir,j,ipart)=Sum(AA(j,1_t_int:odeorder)&
                        &*wf(ir,1_t_int:odeorder,ipart))+CC(j)
                Endif
             Enddo
             !Print *,'odwf1=',dy(ir,1:2,1)/mesh% p_jmesh(ir)
             !!
          Enddo
          !!
!          If(n3<n2)Then
          If(n3 /= n2)Then
             Call odeshootBC(mesh,n1,vtr,btr&
                  &       ,cm,kap,E,dedw,wf,dy,wf0&
                  &       ,odeorder,ipart,dodw,typ,dsc)
             If(typ==OUT)Then
                If(.Not.(dsc% gam_factor ) ) Then
                   !             print *,'typ=',typ,' n3=',n3
                   Call terminate(TERM_CRASH,Trim(nsubfunc)//': typ==1 BC ?')
                Endif
             Endif
          Endif
          !!
          !! inner points
          !!
          Call odeshoot5(mesh,vtr,btr&
               &       ,cm,kap,E,dedw,wf,dy,wf0&
               &       ,odeorder,ipart,dodw,n3+typ,n4,typ,dsc)
          !!
          !! store matching values at nfit
          pio(1_t_int:odeorder,ipart,typ)=wf(nfit,1_t_int:odeorder,ipart)
          !!
       Enddo outinloop
       !!
    End Do ipartloop
    !!
    !! set undefined wf at larger mesh points to zero
    Do ipart=1,odeorder/2_t_int
       wf(nmax+1_t_int:mesh% nr,1_t_int:odeorder,ipart)=0.0_t_real
    Enddo
    !!
    !! fitting of components
    !!
    !!
    
    If(.Not.dodw)Then
       !!
       deb=debugfun(nmodule,"odematch",'ODE')
       Call odematch(dodw,odeorder,pio,mesh% nr,nmax,nfit,wf,BIn,wf0,dif,deb)
       !!
       !! assure normalization via solution process (it stabilizes)
       !!
       Call normwf(mesh,nmax,wf,norm,odeorder,tmp,dsc,E,vtr)
       !!
       If(prot)Then
          Write(outfile,4)nsubfunc,norm
       Endif
       !!
       dif=dif/norm
       !!
       Do ipart=1,odeorder/2
          BIn(ipart)=wf(nmax,SMALL_COMP1+(ipart-1)*2,ipart)
       Enddo
       !!
       
    Else
       !!
       deb=debugfun(nmodule,"odematch",'IODE')
       Call odematch(dodw,odeorder,pio,mesh% nr,nmax,nfit,wf,BIn,wf0,dif,deb)
       !!
       !!
       Call orthowfdwf(mesh,nmax,wf,wf0,ortho,odeorder,dsc)
       !!
       If(prot)Then
          Write(outfile,7)nsubfunc,ortho
       Endif
       !!
!!$       If(odeorder==4)Then
!!$          Do ipart=1,odeorder/2
!!$             Print *,'bin=',bin(ipart),' corr='&
!!$                  &,ortho*wf0(nmax,SMALL_COMP1+(ipart-1)*2,ipart)*2.0_t_real
!!$             BIn(ipart)=BIn(ipart)&
!!$                  &-ortho*wf0(nmax,SMALL_COMP1+(ipart-1)*2,ipart)*2.0_t_real
!!$          Enddo
!!$       Endif
       !!
    Endif
    !!
    !!
    !!
    If(prot)Then
       Do ipart=1,odeorder/2
          Write(outfile,5)nsubfunc,ipart&
               &,Abs((BIn(ipart)-BIn0(ipart))/BIn0(ipart))
          !!
          Write(outfile,6)nsubfunc,ipart&
               &,Abs((wf(2,LARGE_COMP1+(ipart-1)*2,ipart)-BOut(ipart))&
               &/BOut(ipart))
       Enddo
    Endif
    !!
    If(.False.)Then
       If(odeorder==4)Then
          Open(99,file='+wf1')
          Do ir=1,mesh% nr
             Write(99,999)mesh% p_rmesh(ir),wf(ir,1:4,1)
          Enddo
          Close(99)
          Open(99,file='+wf2')
          Do ir=1,mesh% nr
             Write(99,999)mesh% p_rmesh(ir),wf(ir,1:4,2)
          Enddo
          Close(99)
          Open(99,file='+wf')
          Do ir=1,mesh% nr
             Write(99,999)mesh% p_rmesh(ir),(wf(ir,j,1)+wf(ir,j,2),j=1,4)
          Enddo
          Close(99)
       Endif
    Endif
    
    !!Print *,'dif=',dif
    !!
    Return
    !!
1   Format(a,': ipart=',i1)
2   Format(a,': Bin0  (',i1,')=',e20.8e3)
3   Format(a,': Bout0 (',i1,')=',e20.8e3)
4   Format(a,': Norm     =',e20.8e3)
5   Format(a,': d Bin (',i1,')=',e20.8e3)
6   Format(a,': d Bout(',i1,')=',e20.8e3)
7   Format(a,': ortho    =',e20.8e3)
999 Format(5(e30.15e3,2x))       
    !!
  End Subroutine Odeint
  !!***********************************************************************
  !! wether the corresponding function has O(1) or not
  Logical(t_log) Function order0(dsc,kap)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Integer(t_int),Intent(in)    :: kap
    !!LOCAL
    If(dsc% relmode == NON_RELATIVISTIC) Then
       order0=(dsc% la==0)
    Else If(dsc% relmode == SCALAR_RELATIVISTIC_KH) Then
       order0=(dsc% la==0 .Or. (dsc% nuc_type /=  FINUC_NONE .And. dsc% la==1))
    Else
       order0=(Abs(kap)==1)
    Endif
    !!
    Return
    !!
  End Function order0
  !!***********************************************************************
  Subroutine orthowfdwf(mesh,nmax,dwfdw,wf,ortho&
       &,odeorder,dsc)
    !
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    Integer(t_int),Intent(in)  :: odeorder
    Integer(t_int),Intent(in)  :: nmax
    !! ODE solution
    Real(t_real),Intent(in)      :: wf(mesh% nr,4,2)
    !! d(wf)/d(w)
    Real(t_real),Intent(inout)   :: dwfdw(mesh% nr,4,2)
    !! mixing of wf into dwfdw
    Real(t_real),Intent(out)     :: ortho

    !!LOCAL
    !Character(*),Parameter:: nsubfunc="orthowfdwf"
    Integer(t_int) :: ir
    Real(t_real) :: tmp(mesh% nr)

    !!
    !! orthogonalize
    !!
    !     <y|phi>
    !
    If(dsc% relmode == NON_RELATIVISTIC)Then
       tmp(1_t_int:nmax)=wf(1_t_int:nmax,LARGE_COMP1,1_t_int)&
            &*dwfdw(1_t_int:nmax,LARGE_COMP1,1_t_int)&
            &*mesh% p_jmesh(1_t_int:nmax)
    Else
       If(odeorder==2)Then
          tmp(1_t_int:nmax)=(&
               &       wf(1_t_int:nmax,LARGE_COMP1,1_t_int)&
               &   *dwfdw(1_t_int:nmax,LARGE_COMP1,1_t_int) &
               & +     wf(1_t_int:nmax,SMALL_COMP1,1_t_int)&
               &   *dwfdw(1_t_int:nmax,SMALL_COMP1,1_t_int)/dsc% c2&
               &)*mesh% p_jmesh(1_t_int:nmax)
       Else
          Do ir=1,nmax
             tmp(ir)=(&
                  &       Sum(wf(ir,LARGE_COMP1,1:2))&
                  &   *Sum(dwfdw(ir,LARGE_COMP1,1:2)) &
                  & +     Sum(wf(ir,SMALL_COMP1,1:2))&
                  &   *Sum(dwfdw(ir,SMALL_COMP1,1:2))/dsc% c2&
                  & +     Sum(wf(ir,LARGE_COMP2,1:2))&
                  &   *Sum(dwfdw(ir,LARGE_COMP2,1:2)) &
                  & +     Sum(wf(ir,SMALL_COMP2,1:2))&
                  &   *Sum(dwfdw(ir,SMALL_COMP2,1:2))/dsc% c2&
                  &)*mesh% p_jmesh(ir)
          Enddo

       Endif
    End If
    !!
    Call lag4int(nmax,tmp,mesh% hx,ortho,lagint_out)
    !!
    !!
    !!
    dwfdw(1:nmax,1:odeorder,1:odeorder/2)=&
         &dwfdw(1:nmax,1:odeorder,1:odeorder/2)&
         &-ortho*wf(1:nmax,1:odeorder,1:odeorder/2)
    !!
  End Subroutine orthowfdwf
  !***********************************************************************
  !! The wavefunction wf consists of up to four componenets and up to two
  !! partial waves. The sum of the partial waves gives the total function
  !! The small components are scaled with the speed of light
  !! On output the functions are scaled such that the real (summed and
  !! unsacled) wavefunction would be normalized to unity
  Subroutine normwf(mesh,nmax,wf,norm,odeorder,tmp,dsc,E,vtr)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    Integer(t_int),Intent(in) :: odeorder
    Integer(t_int),Intent(in) :: nmax
    Real(t_real),Intent(in)   :: vtr(mesh% nr),E
    Real(t_real),Intent(out)  :: norm
    Real(t_real),Intent(out)  :: tmp(mesh% nr)
    !! the wave function, last index refers to the two partial waves
    !! where the first has asymptotics \gamma_{kap}
    !! and the second  has asymptotics \gamma_{\bar{kap}}
    Real(t_real),Intent(inout):: wf(mesh% nr,4,2)
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="normwf"
    Integer(t_int) :: ir
    Real(t_real)   :: L
    Real(t_real)   :: texp(1:mesh% nr)
    Real(t_real)   :: r2m,r,vv,xx,r0
    !!
    If( dsc% gam_factor)Then
       texp(1:nmax)=mesh% p_rmesh(1:nmax)**(2*dsc% gamma)
    Else
       texp(1:nmax)=1.0_t_real
    Endif
    !!
    !!
    If(dsc% relmode == NON_RELATIVISTIC)Then
       tmp(1_t_int:nmax)=(&
            & wf(1_t_int:nmax,LARGE_COMP1,1)**2_t_int&
            &)*mesh% p_jmesh(1_t_int:nmax)*texp
    Else If (dsc% relmode == SCALAR_RELATIVISTIC_KH)Then
       L=Real(dsc% la*(dsc% la+1),t_real)
       r0=nuc_max_rad(dsc% nuc_type,dsc% ZZ)
       tmp(1)=0.0_t_real
       Do ir=2,nmax
          r=mesh% p_rmesh(ir)
          xx=r/r0
          vv=vtr(ir)-dsc% ZZ*nuc_zeta(xx,dsc% nuc_type)
          r2m=2.0_t_real*r+(r*e-vv)/dsc% c2
          tmp(ir)=(&
               & wf(ir,LARGE_COMP1,1)**2&
               &+wf(ir,SMALL_COMP1,1)**2/dsc% c2&
               &+L*(wf(ir,LARGE_COMP1,1)/r2m)**2/dsc% c2&
               &)*mesh% p_jmesh(ir)*texp(ir)
       Enddo
    Else
       If(odeorder==2)Then
          tmp(1_t_int:nmax)=(&
               & wf(1_t_int:nmax,LARGE_COMP1,1)**2_t_int&
               &+wf(1_t_int:nmax,SMALL_COMP1,1)**2_t_int/dsc% c2&
               &)*mesh% p_jmesh(1_t_int:nmax)*texp
       Else
          !!
          Do ir=1,nmax
             tmp(ir)=(&
                  & Sum(wf(ir,LARGE_COMP1,1:2))**2_t_int    &
                  &+Sum(wf(ir,SMALL_COMP1,1:2))**2_t_int/dsc% c2 &
                  &+Sum(wf(ir,LARGE_COMP2,1:2))**2_t_int    &
                  &+Sum(wf(ir,SMALL_COMP2,1:2))**2_t_int/dsc% c2 &
                  &)*mesh% p_jmesh(ir)*texp(ir)
          Enddo
       Endif
    Endif
    !!
    Call lag4int(nmax,tmp,mesh% hx,norm,lagint_out)
    !!
    norm=Sqrt(norm)
    !Print "('normwfr=',g30.15e3)",norm
    !!
    wf(1_t_int:nmax,1_t_int:odeorder,1:odeorder/2)=&
         & wf(1_t_int:nmax,1_t_int:odeorder,1:odeorder/2)/norm
    !!
  End Subroutine normwf
  !!***********************************************************************
  !!
  !! calculate the kernel of the set of differential equations
  !!
  !!
  Subroutine odekernel(rr,jr,vvr,bbr,cm &
       & ,kap,E,dedw,wf01,wf02,wf03,wf04 &
       & ,AA,CC,odeorder,dodw,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    !! d(wf)/d(w)
    Logical(t_log),Intent(in)  :: dodw
    Integer(t_int),Intent(in) :: kap,odeorder
    Real(t_real),Intent(in) :: rr,jr,E,dedw,vvr,bbr,wf01,wf02,wf03,wf04,cm(5)
    Real(t_real),Intent(out) ::  AA(4,4),CC(4)
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="odekernel"
    Real(t_real)  :: jac,Ep,Em,ll2,xxx0,rrx0,rr2,bb,vv,rrx2,xx,r0,zetaN,MV2
    !!
    r0=nuc_max_rad(dsc% nuc_type,dsc% ZZ)
    xx=rr/r0
    zetaN=nuc_zeta(xx,dsc% nuc_type)
    !!
    bb=0.0_t_real
    vv=(vvr-dsc% ZZ*zetaN)/rr
    !!
    jac=jr
    !!
    rr2=rr*rr
    
    
    
    xxx0=         (rr/dsc% rx0)**dsc% exn0
    rrx0=dsc% fx0*(rr/dsc% rx0)**dsc% exn0
    rrx2=dsc% fx2*(rr/dsc% rx2)**dsc% exn2

    If(dsc% relmode == NON_RELATIVISTIC) Then
       ll2=Real(dsc% la*(dsc% la+1_t_int),t_real)*0.5_t_real

       CC(1_t_int)=0.0_t_real
       CC(2_t_int)=0.0_t_real

       If(dodw)Then
          CC(2_t_int)=2.0_t_real*(xxx0-dedw)*wf01*jac
       Endif

       If(dsc% gam_factor)Then
          !!
          AA(1_t_int,1_t_int)=dsc% la*jac/rr
          AA(1_t_int,2_t_int)=jac
          AA(2_t_int,1_t_int)=2.0_t_real*(vv+rrx0+rrx2-E)*jac
          AA(2_t_int,2_t_int)=-(dsc% la+2)*jac/rr
       Else
          AA(1_t_int,1_t_int)=0.0_t_real
          AA(1_t_int,2_t_int)=jac
          AA(2_t_int,1_t_int)=2.0_t_real*(vv+rrx0+rrx2+ll2/rr2-E)*jac
          AA(2_t_int,2_t_int)=0.0_t_real
       Endif
       Return
    Else If(dsc% relmode == SCALAR_RELATIVISTIC_KH) Then

       MV2=2.0_t_real+(E-vv)/dsc% C2

       ll2=Real(dsc% la*(dsc% la+1_t_int),t_real)/rr2/MV2

       CC(1_t_int)=0.0_t_real
       CC(2_t_int)=0.0_t_real

       If(dodw)Then
          stop 'todo dodw'
       Endif

       If(dsc% gam_factor)Then
          !!
          AA(1_t_int,1_t_int)=(1.0_t_real-dsc% gamma)/rr*jac
          AA(1_t_int,2_t_int)=MV2*jac
          AA(2_t_int,1_t_int)=(vv+rrx0+rrx2+ll2-E)*jac
          AA(2_t_int,2_t_int)=-(1.0_t_real+dsc% gamma)/rr*jac
       Else
          AA(1_t_int,1_t_int)=1.0_t_real/rr*jac
          AA(1_t_int,2_t_int)=MV2*jac
          AA(2_t_int,1_t_int)=(vv+rrx0+rrx2+ll2-E)*jac
          AA(2_t_int,2_t_int)=-1.0_t_real/rr*jac
       Endif
       Return
    Else
       
       If(dsc% relmag) Then
          bb=bbr/rr
       Endif
       Ep=E+dsc% C2*2.0_t_real-vv
       Em=E-vv-rrx0-rrx2
       !! relat nonpolarized and l=0 and 2|mu|=2l+1, scalar relativistic
       orderif: If(odeorder==2_t_int) Then

          CC(1_t_int)=0.0_t_real
          CC(2_t_int)=0.0_t_real

          If(dodw)Then
             CC(1_t_int)=(dedw)*wf02/dsc% C2*jac
             CC(2_t_int)=(xxx0-dedw)*wf01*jac
          Endif


          If( dsc% gam_factor)Then
             !! 
             AA(1_t_int,1_t_int)=-(Real(kap,t_real)+dsc% gamma)/rr*jac
             AA(1_t_int,2_t_int)=(Ep+bb*cm(mkap_mkap))/dsc% C2*jac
             AA(2_t_int,1_t_int)=-(Em-bb*cm(kap_kap))*jac
             AA(2_t_int,2_t_int)=(Real(kap,t_real)-dsc% gamma)/rr*jac
          Else
             AA(1_t_int,1_t_int)=-Real(kap,t_real)/rr*jac
             AA(1_t_int,2_t_int)=(Ep+bb*cm(mkap_mkap))/dsc% C2*jac
             AA(2_t_int,1_t_int)=-(Em-bb*cm(kap_kap))*jac
             AA(2_t_int,2_t_int)=Real(kap,t_real)/rr*jac
          Endif
          Return

       Else  ! orderif
          !! relat polarized


          CC(1_t_int:4_t_int)=0.0_t_real

          If(dodw)Then
             CC(1_t_int)=(dedw)*wf02/dsc% C2*jac
             CC(2_t_int)=(xxx0-dedw)*wf01*jac
             CC(3_t_int)=(dedw)*wf04/dsc% C2*jac
             CC(4_t_int)=(xxx0-dedw)*wf03*jac
          Endif


          If( dsc% gam_factor)Then
             AA(1_t_int,1_t_int)=-(Real(kap,t_real)+dsc% gamma)/rr*jac
             AA(1_t_int,2_t_int)=(Ep+bb*cm(mkap_mkap))/dsc% C2*jac
             AA(1_t_int,3_t_int)=0.0_t_real
             AA(1_t_int,4_t_int)=0.0_t_real
             
             AA(2_t_int,1_t_int)=-(Em-bb*cm(kap_kap))*jac
             AA(2_t_int,2_t_int)=(Real(kap,t_real)-dsc% gamma)/rr*jac
             AA(2_t_int,3_t_int)=bb*cm(kapq_kap)*jac
             AA(2_t_int,4_t_int)=0.0_t_real
             
             AA(3_t_int,1_t_int)=0.0_t_real
             AA(3_t_int,2_t_int)=0.0_t_real
             AA(3_t_int,3_t_int)=-(Real(-kap-1,t_real)+dsc% gamma)/rr*jac
             AA(3_t_int,4_t_int)=(Ep+bb*cm(mkapq_mkapq))/dsc% C2*jac
             
             AA(4_t_int,1_t_int)=bb*cm(kapq_kap)*jac
             AA(4_t_int,2_t_int)=0.0_t_real
             AA(4_t_int,3_t_int)=-(Em-bb*cm(kapq_kapq))*jac
             AA(4_t_int,4_t_int)=(Real(-kap-1,t_real)-dsc% gamma)/rr*jac
          Else
             AA(1_t_int,1_t_int)=-Real(kap,t_real)/rr*jac
             AA(1_t_int,2_t_int)=(Ep+bb*cm(mkap_mkap))/dsc% C2*jac
             AA(1_t_int,3_t_int)=0.0_t_real
             AA(1_t_int,4_t_int)=0.0_t_real
             
             AA(2_t_int,1_t_int)=-(Em-bb*cm(kap_kap))*jac
             AA(2_t_int,2_t_int)=Real(kap,t_real)/rr*jac
             AA(2_t_int,3_t_int)=bb*cm(kapq_kap)*jac
             AA(2_t_int,4_t_int)=0.0_t_real
             
             AA(3_t_int,1_t_int)=0.0_t_real
             AA(3_t_int,2_t_int)=0.0_t_real
             AA(3_t_int,3_t_int)=-Real(-kap-1,t_real)/rr*jac
             AA(3_t_int,4_t_int)=(Ep+bb*cm(mkapq_mkapq))/dsc% C2*jac
             
             AA(4_t_int,1_t_int)=bb*cm(kapq_kap)*jac
             AA(4_t_int,2_t_int)=0.0_t_real
             AA(4_t_int,3_t_int)=-(Em-bb*cm(kapq_kapq))*jac
             AA(4_t_int,4_t_int)=Real(-kap-1,t_real)/rr*jac
          Endif
          Return

       Endif orderif
    Endif ! relativistic
    !
    !
  End Subroutine odekernel

  !!***********************************************************************
  !
  ! calculate the starting solution for the set of ODE's via a
  ! series expansion
  !
  !
  !
  Subroutine fstart(nr,rmesh,ir,cm,E,dedw,wf,wf0,dwf,BOut  &
       & ,BIn,kap,odeorder,ipart,dodw,typ,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Integer(t_int),Intent(in) :: typ,nr,ir,ipart,odeorder,kap
    Logical(t_log),Intent(in) :: dodw
    Real(t_real),Intent(in)   :: rmesh(nr),BOut,BIn
    Real(t_real),Intent(in)   :: cm(5),E,dedw
    Real(t_real),Intent(in)   :: wf0(nr,4,2)
    Real(t_real),Intent(out)  :: wf(nr,4,2)
    Real(t_real),Intent(out)  :: dwf(4,2)
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="fstart"
    Integer(t_int) :: j,la
    Real(t_real) :: Po(4,2),dPo(4,2),dPo1(4,2),Pod(4,2)
    Real(t_real)   :: lp,p2,pg,pg1,w1,dd,s1,r0
    Integer(t_int),Parameter :: Ng=2
    Real(t_real)   :: Gcoeff(0:Ng),dGcoeff(0:Ng)
    Real(t_real)   :: rr,rr2,ZZ
    Real(t_real)   :: xx,xx2
    !!
    la=dsc% la
    ZZ=dsc% ZZ
    r0=nuc_max_rad(dsc% nuc_type,ZZ)
    !Write(outfile,"('r0=',e30.15)")r0
    !!
    rr=rmesh(ir)
    rr2=rmesh(2)
    xx=rr/r0
    xx2=rr2/r0

    !!
    
    !!
    If (typ==1_t_int) Then! outward
       !!
       !!
       !!
       !!
       !!
       !!
       If (dsc% relmode == NON_RELATIVISTIC) Then
          lp=Real(la+1_t_int,t_real)
          !!
          !!
          Call wave_coef_nrel(la,r0,E,ZZ,dsc% p_Vcoeff ,dsc% Nv,Gcoeff,Ng)
          !!
          !!
          If(dsc% gam_factor)Then
             !! at second point
             pg=horner(xx2,Gcoeff,Ng)
             p2=xx2**la*pg
             !! at current point
             pg=horner(xx,Gcoeff,Ng,pg1)
             Po(1,1)=xx**la*pg/p2
             Po(2,1)=xx**la*pg1/r0/p2 
             !! next works only for xx==0 and la==0
             Pod(1,1)=Gcoeff(1)/p2/r0
             Pod(2,1)=2*Gcoeff(2)/r0**2/p2 
             !!TODO: dedw must be adjusted
          Else
             !! at second point
             pg=horner(xx2,Gcoeff,Ng)
             p2=xx2**lp*pg
             !! at current point
             pg=horner(xx,Gcoeff,Ng,pg1)
             Po(1,1)=xx**lp*pg/p2
             Po(2,1)=xx**la*(lp*pg+xx*pg1)/r0/p2
          Endif
          !!
          If(.Not.dodw)Then
             w1=BOut
             If(w1==0.0_t_real)w1=1.0e-30_t_real
             Do j=1,odeorder
                wf(ir,j,ipart)=Po(j,ipart)*w1
                dwf(j,ipart)  =Pod(j,ipart)*w1
             Enddo
          Else
             !!now wf is dwf/dw and wf0 is wf
             
             Call wave_coef_dW_nrel(la,r0,E,dedw,ZZ,dsc% p_Vcoeff,dsc% Nv&
                  &,Gcoeff,dGcoeff,Ng)
             !!
             pg=horner(xx2,dGcoeff,Ng)
             dPo1(1,1)=xx2**lp*pg/p2
             !!
             pg=horner(xx,dGcoeff,Ng,pg1)
             dPo(1,1) =xx**lp*pg/p2
             dPo(2,1) =xx**la*(lp*pg+xx*pg1)/r0/p2
             !! wf   =Nwf*p
             !! dwfdw=dNwf*p+Nwf*dp
             w1=BOut
             dd=(w1-dPo1(1,ipart)*wf0(2,LARGE_COMP1,ipart))
             Do j=1,odeorder
                wf(ir,j,ipart)=Po(j,ipart)*dd&
                     &+dPo(j,ipart)*wf0(2,LARGE_COMP1,ipart)
             Enddo

          Endif
          Return
       Else If (dsc% relmode == SCALAR_RELATIVISTIC_KH)Then
          !! Koelling Harmon
          Call kh_ode_expansion(xx,xx2,E,la,Po,r0,dsc,Pod)
          !!
          !!
          If(.Not.dodw)Then
             w1=BOut
             If(w1==0.0_t_real)w1=1.0e-30_t_real
             Do j=1,2
                wf(ir,j,ipart)=Po(j,ipart)*w1
                dwf(j,ipart)  =Pod(j,ipart)*w1
             Enddo
          Else
             !!now wf is dwf/dw and wf0 is wf
             Stop 'todo: fstart KH dwf'
          Endif
          Return
       Else !! relativistic
          Call rel_ode_expansion(odeorder,xx,xx2,E,dedw,kap,cm&
               &,Po,dPo,dPo1,r0,dsc,Pod)
          !!
          !!If(ir==1)&
          !!     & Print *,'Po(',1+(ipart-1)*2,')=',Po(1+(ipart-1)*2,ipart)
          !!
          If(.Not.dodw)Then
             w1=BOut
             If(w1==0.0_t_real)w1=1.0e-30_t_real
             Do j=1,odeorder
                wf(ir,j,ipart)=Po(j,ipart)*w1
                dwf(j,ipart)=Pod(j,ipart)*w1
             Enddo
          Else
             If(odeorder==2_t_int) Then
                !! relat nonpolarized and polararized l=0 2|M|=2l+1
                w1=BOut
                dd=(w1-dPo1(1,ipart)*wf0(2,LARGE_COMP1,ipart))
                Do j=1,odeorder
                   wf(ir,j,ipart)=Po(j,ipart)*dd&
                        &+dPo(j,ipart)*wf0(2,LARGE_COMP1,ipart)
                Enddo
                Return
             Else !! odeorder
                !! polarized
                !!
                s1=    (&
                     & dPo1(LARGE_COMP1+2*ipart-2,1)*wf0(2,LARGE_COMP1,1)&
                     &+dPo1(LARGE_COMP1+2*ipart-2,2)*wf0(2,LARGE_COMP2,2)&
                     &)
                w1=BOut
                !!
                !!dd=(w1-s1)*2.0_t_real
                dd=(w1*2.0_t_real-s1)
                !!Print *,'ipart=',ipart,'dd=',dd,wf0(2,1+2*ipart-2,ipart)
                !!
                Do j=1,odeorder
                   wf(ir,j,ipart)=Po(j,ipart)*dd&
                        !! take the full inhomogenity
                        &+(dPo(j,1)*wf0(2,LARGE_COMP1,1)&
                        &+ dPo(j,2)*wf0(2,LARGE_COMP2,2))
                         
                Enddo
                Return
             Endif ! odeorder==2
          Endif !! dodw
          Return
       Endif ! nonrelat
    Else ! inward
       If(dsc% relmode == NON_RELATIVISTIC) Then
          wf(ir,LARGE_COMP1,ipart)=0.0_t_real
          wf(ir,SMALL_COMP1,ipart)=BIn
          Return

       Else If(dsc% relmode == SCALAR_RELATIVISTIC_KH) Then
          wf(ir,LARGE_COMP1,ipart)=0.0_t_real
          wf(ir,SMALL_COMP1,ipart)=BIn
          Return
       Else          ! relat nonpolarized
          If(odeorder==2_t_int)Then
             wf(ir,LARGE_COMP1,ipart)=0.0_t_real
             wf(ir,SMALL_COMP1,ipart)=BIn
             Return
          Else
             If(ipart==1)Then
                wf(ir,LARGE_COMP1,ipart)=0.0_t_real
                wf(ir,SMALL_COMP1,ipart)=BIn
                wf(ir,LARGE_COMP2,ipart)=0.0_t_real
                wf(ir,SMALL_COMP2,ipart)=0.0_t_real
             Else
                wf(ir,LARGE_COMP1,ipart)=0.0_t_real
                wf(ir,SMALL_COMP1,ipart)=0.0_t_real
                wf(ir,LARGE_COMP2,ipart)=0.0_t_real
                wf(ir,SMALL_COMP2,ipart)=BIn
             Endif
             Return
          Endif             ! odeorder
       Endif          ! relativistic
    Endif       ! outward or inward
    !
  End Subroutine fstart
  !!***********************************************************************
  Subroutine  kh_ode_expansion(xx,xx2,E,la,Po,r0,dsc,Pod)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Real(t_real),Intent(in)      :: r0
    Integer(t_int),Intent(in)    :: la
    !! current mesh point /r0
    Real(t_real),Intent(in)      :: xx
    !! second  mesh point/r0
    Real(t_real),Intent(in)      :: xx2
    !! energy
    Real(t_real),Intent(in)      :: E
    !! the expansion functions, normalized such that
    !! the large component at the first mesh point is unity
    !! Po: expansion of the homogenous equation (ODE)
    Real(t_real),Intent(out)     :: Po(4,2),Pod(4,2)
    !!LOCAL
    !Character(*),Parameter::nsubfunc="kh_ode_expansion"
    Integer(t_int),Parameter :: Ng=20
    Integer(t_int) :: j
    Real(t_real)   :: pg,p2,gamsol,xx2gam,xxgam,pg1
    Real(t_real)   :: GcoeffR(0:Ng,3)
    Logical(t_log) :: prot
    !!
    !!
    !!
    prot=debugfun(nmodule,'kh_ode_expansion','')
    !!
    !!
    Call wave_coef_dW_KH(la,r0,E,dsc% ZZ,dsc% p_Vcoeff,dsc% Nv&
         &,GcoeffR,Ng,gamsol)
    !!
    If(prot)Then
       Do j=0,Ng
          Write(outfile,"('gcoeff=',i5,4(' ',e30.15))")j,GcoeffR(j,1:3)
       Enddo
       Print *
    Endif
!    Print *,'xx=',xx
    !!
    xx2gam=xx2**gamsol
    xxgam=xx**gamsol
    !!
    !!
    If( dsc% gam_factor)Then
       If(Abs(la)==0)Then
          xx2gam=1.0_t_real
          xxgam=1.0_t_real
       Else
          xx2gam=xx2**(gamsol-dsc% gamma)
          xxgam=xx**(gamsol-dsc% gamma)
          !Print *,'delta gamma=',(gamsol-dsc% gamma)
       Endif
    Endif
    !! normalize first large component
    pg=horner(xx2,GcoeffR(:,LARGE_COMP1),Ng)
    p2 =xx2gam*pg
    !!
    Do j=1,2
       pg=horner(xx,GcoeffR(:,j),Ng)
       Po(j,1)=xxgam*pg/p2

       pg=horner(xx,GcoeffR(:,j),Ng,pg1)
       Pod(j,1)=pg1/p2/r0 !! This is not complete, works only for la==0 and xx=0
       !!                    or la==1 and finuc and xx=0
    Enddo
    !!
    !!
    !!
    !!
    Return
    !!
  End Subroutine kh_ode_expansion
  !!***********************************************************************
  Subroutine rel_ode_expansion(odeorder,xx,xx2,E,dedw,kap,cm&
       &,Po,dPo,dPo1,r0,dsc,Pod&
       &)
    !!
    Implicit none  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Real(t_real),Intent(in)      :: r0
    Integer(t_int),Intent(in)    :: odeorder
    !! $[\kappa]$ : the kappa value of the solution
    Integer(t_int),Intent(in)    :: kap
    !! current mesh point /r0
    Real(t_real),Intent(in)      :: xx
    !! second  mesh point/r0
    Real(t_real),Intent(in)      :: xx2
    !! energy
    Real(t_real),Intent(in)      :: E
    !! d(E)/d(w)
    Real(t_real),Intent(in)      :: dedw
    !! magnetic oupling coefficients
    Real(t_real),Intent(in)      :: cm(5)
    !! the expansion functions of the partial waves, normalized such that
    !! the ipart-th large component of the ipart-th partial wave 
    !! at the first mesh point is unity
    !! Po: expansion of the homogenous equation (ODE)
    !! dPo: expansion of the inhomogenous equation (IODE)
    !! dPo1(j,ipart): value of the IODE expansion function for the ipart-th
    !!               partial wave
    Real(t_real),Intent(out)     :: Po(4,2),dPo(4,2),dPo1(4,2),Pod(4,2)
    !!LOCAL
    Integer(t_int),Parameter :: Ng=20
    Integer(t_int) :: j,ipart
    Real(t_real)   :: pg,p2,gamsol,xx2gam,xxgam,pg1
    Real(t_real)   :: GcoeffR(0:Ng,4),dGcoeffR(0:Ng,4)
    Logical(t_log) :: prot
    !!
    ipart=1
    !!
    !!
    prot=debugfun(nmodule,'rel_ode_expansion','')
    !!
    !!
    Call wave_coef_dW_rel4(ipart,kap,cm,r0,E,dedw,dsc% ZZ&
         &,dsc% p_Vcoeff,dsc% p_Bcoeff,dsc% Nv&
         &,GcoeffR,dGcoeffR,Ng,gamsol)
    !!
    If(prot)Then
       Do j=0,Ng
          Write(outfile,"('gcoeff=',i5,4(' ',e30.15))")j,GcoeffR(j,1:4)
       Enddo
       Print *
    Endif
!    Print *,'xx=',xx
    !!
    xx2gam=xx2**gamsol
    xxgam=xx**gamsol
    !!
    !!
    If( dsc% gam_factor)Then
       If(Abs(kap)==1)Then
          xx2gam=1.0_t_real
          xxgam=1.0_t_real
       Else
          xx2gam=xx2**(gamsol-dsc% gamma)
          xxgam=xx**(gamsol-dsc% gamma)
       Endif
    Endif
    !! normalize first large component
    pg=horner(xx2,GcoeffR(:,LARGE_COMP1),Ng)
    p2 =xx2gam*pg
    !!
    Do j=1,4
       pg=horner(xx,GcoeffR(:,j),Ng)
       Po(j,ipart)=xxgam*pg/p2

       pg=horner(xx,GcoeffR(:,j),Ng,pg1)
       Pod(j,ipart)=pg1/p2/r0 !! This is not complete, works only for |kap|=1 and xx=0
       !!TODO: adjust dedw for gam_factor
       pg=horner(xx2,dGcoeffR(:,j),Ng)
       dPo1(j,ipart)=xx2gam*pg/p2
       !!
       pg=horner(xx,dGcoeffR(:,j),Ng)
       dPo(j,ipart)=xxgam*pg/p2
    Enddo
    !!
    !!---------------------------------------------------------------------
    !!
    If(odeorder==4)Then
       ipart=2
       !!
       !!
       Call wave_coef_dW_rel4(ipart,kap,cm,r0,E,dedw,dsc% ZZ&
            &,dsc% p_Vcoeff,dsc% p_Bcoeff,dsc% Nv&
            &,GcoeffR,dGcoeffR,Ng,gamsol)
       !!
       xx2gam=xx2**gamsol
       xxgam=xx**gamsol
       !!
       If( dsc% gam_factor)Then
          If(Abs(-kap-1)==1)Then
             xx2gam=1.0_t_real
             xxgam=1.0_t_real
          Else
             xx2gam=xx2**(gamsol-dsc% gamma)
             xxgam=xx**(gamsol-dsc% gamma)
          Endif
       Endif
       !! normalize second large component
       !!
       pg=horner(xx2,GcoeffR(:,LARGE_COMP2),Ng)
       p2 =xx2gam*pg
       !!
       Do j=1,4
          pg=horner(xx,GcoeffR(:,j),Ng)
          Po(j,ipart)=xxgam*pg/p2
          !!
          pg=horner(xx,GcoeffR(:,j),Ng,pg1)
          Pod(j,ipart)=pg1/p2/r0 !! This is not complete, works only for |kap|=1 and xx=0
          !!TODO: adjust dedw for gam_factor
          pg=horner(xx2,dGcoeffR(:,j),Ng)
          dPo1(j,ipart)=xx2gam*pg/p2
          !!
          pg=horner(xx,dGcoeffR(:,j),Ng)
          dPo(j,ipart)=xxgam*pg/p2
       Enddo
    Endif!! odeorder==4
    !!
    Return
    !!
  End Subroutine rel_ode_expansion
  !!***********************************************************************
  !! make collective predictor-corrector step to get
  !! the solution for the points n1+typ*predictor,...,n1 at once
  !! from a single known starting value at point n1
  !! this is similar to a relaxation method and better then a shooting with
  !! increasing order
  Subroutine odeshootBC(mesh,n1,vtr,btr &
       & ,cm,kap,E,dedw &
       & ,wf,dy,wf0,odeorder,ipart,dodw,typ,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    !! make d(wf)/d(w) ?
    Logical(t_log),Intent(in)  :: dodw
    !! start mesh point with known starting values
    Integer(t_int),Intent(in)  :: n1
    !! angular momentum, kappa qns
    Integer(t_int),Intent(in)  :: kap
    !! order of system
    Integer(t_int),Intent(in)  :: odeorder
    !! which solution
    Integer(t_int),Intent(in)  :: ipart
    !! outward=1,inward=-1
    Integer(t_int),Intent(in)  :: typ
    !!
    !! energy,dE/dW
    Real(t_real),Intent(in)    :: E,dedw
    !! spin factors
    Real(t_real),Intent(in)    :: cm(5)
    !! potential,B-field
    Real(t_real),Intent(in)    :: vtr(mesh% nr),btr(mesh% nr)
    !! the wave function (inhomogenity for d(wf)/d(w)-ODE)
    Real(t_real),Intent(in)    :: wf0(mesh% nr,4,2)
    !! book keeping: d(wf)/dr
    Real(t_real),Intent(inout) :: dy(mesh% nr,4,2)
    !! solution (=dwfdw, if IODE)
    Real(t_real),Intent(inout) :: wf(mesh% nr,4,2)

    !!LOCAL
    Character(*),Parameter::nsubfunc="odeshootBC"
    Logical(t_log) :: indef
    Integer(t_int) :: j,k,i,ie,iv,oie,oiv,ir
    Real(t_real)  :: dd,fac
    !! local aux arrays
    Real(t_real) :: &
         & CC(4_t_int,maxcorder) &
         & ,AA(4_t_int,4_t_int,maxcorder) &
         & ,MM(maxcorder*4_t_int,maxcorder*4_t_int),MB(maxcorder*4_t_int)

    !!
!    Print *,'typ=',typ
    !!
    If(pred_order>maxcorder)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'Pred_order exceeds maxcorder!'")
    Endif
    !!
    !!
    Do i=1_t_int,pred_order
       ir=n1+i*typ
       Call odekernel(mesh% p_rmesh(ir),mesh% p_jmesh(ir)&
            &   ,vtr(ir),btr(ir),cm&
            &   ,kap,E,dedw&
            &   ,Sum(wf0(ir,LARGE_COMP1,1:odeorder/2))&
            &   ,Sum(wf0(ir,SMALL_COMP1,1:odeorder/2))&
            &   ,Sum(wf0(ir,LARGE_COMP2,1:odeorder/2))&
            &   ,Sum(wf0(ir,SMALL_COMP2,1:odeorder/2))&
            &   ,AA(1_t_int,1_t_int,i),CC(1_t_int,i),odeorder,dodw&
            &,dsc)
    Enddo
    !! A(i)=A(r_{N-i})=A^N_i
    !! C(i)=C(r_{N-i})=C^N_i
    !!
    fac=typ*mesh% hx/pred_hcof(pred_order)
    !!
    MM=0.0_t_real
    Do ie=0_t_int,pred_order-1_t_int
       oie=ie*odeorder
       Do iv=0_t_int,pred_order-1_t_int
          oiv=iv*odeorder
          Do j=1_t_int,odeorder
             Do k=1_t_int,odeorder
                !! A(M-iv)=A^N_{M-iv}
                MM(oie+j,oiv+k)=AA(j,k,pred_order-iv)*fac&
                     &           *pred_cof(iv,ie+1_t_int,pred_order)
             Enddo
             !!C(M-iv)=C^N_{M-iv}
             !! C_{M-iv} c_{iv}^{ie}
             MB(oie+j)=fac*CC(j,pred_order-iv)*&
                  &pred_cof(iv,ie+1_t_int,pred_order)
          Enddo
       Enddo
       Do j=1_t_int,odeorder
          !! \delta_{ie,iv}
          MM(oie+j,oie+j)=MM(oie+j,oie+j)-1.0_t_real
          !! \delta_{ie+1,iv}
          If(ie<pred_order-1)&
               &       MM(oie+j,oie+odeorder+j)=MM(oie+j,oie+odeorder+j)&
               &+1.0_t_real
       Enddo
       !!dy_{N} c_{M}^{ie}
       Do j=1_t_int,odeorder
          MB(oie+j)=fac*dy(n1,j,ipart)*pred_cof(pred_order,ie+1_t_int&
               &,pred_order)
       Enddo
    Enddo
    !! wf_{N} \delta_{ie,M-1}
    Do j=1_t_int,odeorder
       MB(odeorder*(pred_order-1_t_int)+j)  = &
            &MB(odeorder*(pred_order-1_t_int)+j)&
            &     +wf(n1,j,ipart)
    Enddo

    !! now we have the equation MM*Y+MB=0
    Call gaussj(MM,odeorder*pred_order,maxcorder*4_t_int,MB,0_t_int,1_t_int&
         &,indef)
    If(indef)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'indefinite matrix, sorry!'")
    Endif
    Do i=1_t_int,odeorder*pred_order
       dd=0.0_t_real
       Do j=1_t_int,odeorder*pred_order
          dd=dd-MM(i,j)*MB(j)
       Enddo
       !! wf(n1-M+iv,j,ipart)=MM^{-1}*MB
       wf(n1+typ*(pred_order-((i-1_t_int)/odeorder))&
            &,Mod(i-1_t_int,odeorder)+1_t_int,ipart)&
            &=dd!!MB(i)
    Enddo

    !! store d(wf)/d(r)
    Do i=1_t_int,pred_order
       ir=n1+i*typ
       Do j=1_t_int,odeorder
          dy(ir,j,ipart)=Sum(AA(j,1_t_int:odeorder,i)&
               &*wf(ir,1_t_int:odeorder,ipart))+CC(j,i)
       Enddo
    Enddo



    Return
  End Subroutine odeshootBC
  !!***********************************************************************
  !! one shoot of integration with 5-th order polynomial
  Subroutine odeshoot5(mesh,vtr,btr &
       & ,cm,kap,E,dedw &
       & ,wf,dy,wf0,odeorder,ipart,dodw,n1,n2,typ,dsc)
    !!
    Implicit None
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)    :: mesh
    !! make d(wf)/d(w)
    Logical(t_log),Intent(in)  :: dodw
    !! angular momentum
    Integer(t_int),Intent(in) :: kap 
    !! order of system
    Integer(t_int),Intent(in) :: odeorder 
    !! which partial wave
    Integer(t_int),Intent(in) :: ipart 
    !! outward=1,inward=-1
    Integer(t_int),Intent(in) :: typ
    !! start, end index
    Integer(t_int),Intent(in) :: n1,n2 
    !! energy,dE/dWW
    Real(t_real),Intent(in)   :: E,dedw 
    !! spin coupling factors
    Real(t_real),Intent(in)   :: cm(5) 
    !! potential,B-field 
    Real(t_real),Intent(in)   :: vtr(mesh% nr),btr(mesh% nr) 
    Real(t_real),Intent(in)   :: wf0(mesh% nr,4,2)
    !! d(wf)/dr
    Real(t_real),Intent(inout) :: dy(mesh% nr,4,2)
    !! solution
    Real(t_real),Intent(inout) :: wf(mesh% nr,4,2)
    !!
    !!LOCAL
    !Character(*),Parameter::nsubfunc="odeshoot5"
    !Logical(t_log) :: indef
    Integer(t_int) :: j,ir
    Real(t_real)   :: dd,fac,f,d12,d34,det,b,b2,B1M22,B3M44,B1M22b,B3M44b
    Real(t_real)   :: AA(4,4),CC(4),MB(4)
    !!
    !!
    !!
    fac=typ*mesh% hx/1440.0_t_real
    f=1.0_t_real/(fac*475.0_t_real)
    !!
    orderif: If(odeorder==2)Then
       irloop2: Do ir=n1,n2,typ

          Call odekernel(mesh% p_rmesh(ir),mesh% p_jmesh(ir)&
               &   ,vtr(ir),btr(ir),cm&
               &   ,kap,E,dedw&
               &   ,Sum(wf0(ir,LARGE_COMP1,1:odeorder/2))&
               &   ,Sum(wf0(ir,SMALL_COMP1,1:odeorder/2))&
               &   ,Sum(wf0(ir,LARGE_COMP2,1:odeorder/2))&
               &   ,Sum(wf0(ir,SMALL_COMP2,1:odeorder/2))&
               &   ,AA,CC,2_t_int,dodw,dsc)


          dd=f/&
               &(f**2-f*(AA(1,1)+AA(2,2))+(AA(1,1)*AA(2,2)-AA(1,2)*AA(2,1)))

          Do j=1_t_int,2
             MB(j)=(                      wf(ir-typ,j,ipart)+&
                  &(+ 475.0_t_real*   CC(j)&
                  & +1427.0_t_real*dy(ir-  typ,j,ipart)&
                  & - 798.0_t_real*dy(ir-2*typ,j,ipart)&
                  & + 482.0_t_real*dy(ir-3*typ,j,ipart)&
                  & - 173.0_t_real*dy(ir-4*typ,j,ipart)&
                  & +  27.0_t_real*dy(ir-5*typ,j,ipart))*fac)*dd
          Enddo

          !!
          !!
          wf(ir,LARGE_COMP1,ipart)=(f-AA(2,2))*MB(1)+   AA(1,2) *MB(2)
          wf(ir,SMALL_COMP1,ipart)=   AA(2,1) *MB(1)+(f-AA(1,1))*MB(2)
          !!
          !! store new d(wf)/dr
          dy(ir,LARGE_COMP1,ipart)=&
               &AA(1,1)*wf(ir,1,ipart)+AA(1,2)*wf(ir,2,ipart)+CC(1)
          dy(ir,SMALL_COMP1,ipart)=&
               &AA(2,1)*wf(ir,1,ipart)+AA(2,2)*wf(ir,2,ipart)+CC(2)
          !!
       Enddo irloop2
    else
       !! odeorder=4
       !!
       irloop4: Do ir=n1,n2,typ


          Call odekernel(mesh% p_rmesh(ir),mesh% p_jmesh(ir)&
               &   ,vtr(ir),btr(ir),cm&
               &   ,kap,E,dedw&
               &   ,Sum(wf0(ir,LARGE_COMP1,1:odeorder/2))&
               &   ,Sum(wf0(ir,SMALL_COMP1,1:odeorder/2))&
               &   ,Sum(wf0(ir,LARGE_COMP2,1:odeorder/2))&
               &   ,Sum(wf0(ir,SMALL_COMP2,1:odeorder/2))&
               &   ,AA,CC,4_t_int,dodw,dsc)


          Do j=1_t_int,4
             MB(j)=                      wf(ir-typ,j,ipart)+&
                  &(+ 475.0_t_real*   CC(j)&
                  & +1427.0_t_real*dy(ir-  typ,j,ipart)&
                  & - 798.0_t_real*dy(ir-2*typ,j,ipart)&
                  & + 482.0_t_real*dy(ir-3*typ,j,ipart)&
                  & - 173.0_t_real*dy(ir-4*typ,j,ipart)&
                  & +  27.0_t_real*dy(ir-5*typ,j,ipart))*fac
          Enddo
          !!
          !! the paricular form of AA is used to make 
          !! a hand inversion:
          !!
          b=-AA(2,3)
          b2=b**2
          d12=(f-AA(1,1))*(f-AA(2,2))-AA(1,2)*AA(2,1)
          d34=(f-AA(3,3))*(f-AA(4,4))-AA(3,4)*AA(4,3)
          det=f/(d12*d34-AA(1,2)*AA(3,4)*b2)
          
          B1M22=MB(1)*(f-AA(2,2))
          B3M44=MB(3)*(f-AA(4,4))
          B1M22b=(B1M22+MB(2)*AA(1,2))*b-MB(4)*d12
          B3M44b=-(MB(4)*AA(3,4)+B3M44)*b+MB(2)*d34
          
          wf(ir,1,ipart)=(AA(1,2)*B3M44b+B1M22*d34)*det
          wf(ir,2,ipart)=((f-AA(1,1))*B3M44b&
               &+MB(1)*(AA(2,1)*d34+AA(3,4)*b2))*det
          wf(ir,3,ipart)=(-AA(3,4)*B1M22b+B3M44*d12)*det
          wf(ir,4,ipart)=(-(f-AA(3,3))*B1M22b&
               &+MB(3)*(AA(1,2)*b2+AA(4,3)*d12))*det


          !! store new d(wf)/dr
          Do j=1_t_int,4
             dy(ir,j,ipart)=Sum(AA(j,1_t_int:4)&
                  &*wf(ir,1_t_int:4,ipart))+CC(j)
          Enddo
       Enddo irloop4
    End If orderif
    !!
    Return
    !!
  End Subroutine odeshoot5
  !!***********************************************************************
  !! This subroutine estimates the energy of the states
  !! $|[\kappa]\mu\rangle$ and $|[\bar{\kappa}]\mu\rangle$
  !! in the XC-field $\Delta B_z$ by a Ritz variation using a basis of pure
  !! kappa and kappaq states
  !! The two basis states must have the same $l$ value of the large
  !! component, i. e. $\kappa = -\bar{\kappa}-1$. Otherwise the calculation
  !! is stopped. The states may either result from a nonmagnetic
  !! or a magnetic calculation in a different field. All quantities
  !! except the estimated new energies $En$ and $Enq$
  !! must be supplied as input.
  !!
  Subroutine mag_diagno(kap,kapq,E,Eq,mesh,btr,wf,tmp,E12,prot,sortJ,dsc)
    !!
    Implicit None
    !!
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    Logical(t_log),Intent(in)    :: prot
    !! sort such that nominal kappa state has larger j_kappa weight
    Logical(t_log),Intent(in)    :: sortJ
    !! $\kappa$ quantum number
    Integer(t_int),Intent(in)    :: kap
    !! $\bar{\kappa}$ quantum number
    Integer(t_int),Intent(in)    :: kapq
    !!energy of $|\kappa\mu\rangle$ without $\Delta B_z$
    Real(t_real),Intent(in)      :: E
    !! energy of $|\kappa_2\mu\rangle$ without $\Delta B_z$
    Real(t_real),Intent(in)      :: Eq
    !! magnetic field times r $r*B(r)$
    Real(t_real),Intent(in)      :: btr(mesh% nr)
    !! wave functions of the basis states
    Real(t_real),Intent(in)      :: wf(mesh% nr,2,2)
    !! dummy array used for radial integration
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !! Estimate of the new energy of $|[\kappa_{1,2}]\mu\rangle$
    Real(t_real),Intent(out)     :: E12(2)
    !!
    !!LOCAL
    Character(*),Parameter::nsubfunc="mag_diagno"
    !!
    Integer(t_int) :: l
    Real(t_real)   :: M(2,2),C(2,2),d
!!$    Integer(t_int) :: j
!!$    Real(t_real)   :: dj(2)
    !!
    !!--------------------------------------------------------------------
    !!             PREPARATIONS
    !!---------------------------------------------------------------------
    !!
    If (kap /= (-kapq-1)) Then
       Write(errbuf,5000)kap,kapq
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
    Endif
    !!
    !!---------------------------------------------------------------------
    !!            MATRIX ELEMENTS
    !!---------------------------------------------------------------------

    M(1,1)=magmat(kap,kap,mesh,wf(1,1,1),wf(1,1,1),btr,tmp,dsc)+E
    M(2,2)=magmat(kapq,kapq,mesh,wf(1,1,2),wf(1,1,2),btr,tmp,dsc)+Eq
    M(1,2)=magmat(kap,kapq,mesh,wf(1,1,1),wf(1,1,2),btr,tmp,dsc)
    M(2,1)=magmat(kapq,kap,mesh,wf(1,1,2),wf(1,1,1),btr,tmp,dsc)
    !!
    !! calculate new eigenvalues by diagonalization
    Call diagno2(M,C,E12)
    !!
    !! order energies such that E12(1) belongs to the solution
    !! where the kap-part has heigher weight
    !!  
!!$    Do l=1,2
!!$       j=2*Abs(kap)-1
!!$       dj(l)=C(1,l)**2*j*(j+1)*0.25_t_real
!!$       j=2*Abs(kapq)-1
!!$       dj(l)=dj(l)+C(2,l)**2*j*(j+1)*0.25_t_real
!!$    Enddo
!!$    Do l=1,2
!!$       !!
!!$       d=-0.5_t_real+Sqrt(0.25_t_real+dj(l))
!!$       !!
!!$       Write(outfile,110)l,E12(l),d*2.0_t_real
!!$110    Format('E   (',i1,')=',e20.8e3,' <J^2>=',f10.5)
!!$       Write(outfile,11)l,C(1,l)**2,C(2,l)**2
!!$       If(l==1) Then
!!$          Write(outfile,12)l,kap
!!$       Else 
!!$          Write(outfile,12)l,kapq
!!$       End If
!!$    Enddo

    !! check weights for kap parts in the two solutions
    !! order such that solution one has the larger kap weight
    If(sortJ.And.C(1,1)**2<C(1,2)**2)Then
       d=E12(1)
       E12(1)=E12(2)
       E12(2)=d
       Do l=1,2
          d=C(l,1)
          C(l,1)=C(l,2)
          C(l,2)=d
       Enddo
    Endif
    !!
    If(prot)Then
       Write(outfile,13)kap
       Do l=1,2
          Write(outfile,10)l,E12(l)
          Write(outfile,11)l,C(1,l)**2,C(2,l)**2
          If(l==1) Then
             Write(outfile,12)l,kap
          Else 
             Write(outfile,12)l,kapq
          End If
       Enddo
10     Format('E   (',i1,')=',e20.8e3)
11     Format('C   (',i1,')=',2(e20.8e3,2x))
12     Format('kap (',i1,')=',i2)
13     Format('kap=',i2)
    Endif
    !!
    Return
    !!
5000 Format(''' kap=',i3,', kapq=',i3,', but kap must equal -kapq-1 !!''')
    !!
  End Subroutine mag_diagno
  !!***********************************************************************
  !! This function calculates the matrix element
  !! $\langle \Psi_{\kappa_1\mu}|\beta\Sigma_z B| \Psi_{\kappa_2\mu} \rangle$.
  !! The functions must be of nonmagnetic symmetry
  !! The small component of functions is assumed to be scaled with
  !! speed of light.
  !!
  Real(t_real) Function magmat(kap1,kap2,mesh,wf1,wf2,btr,tmp,dsc)
    !!
    Implicit none
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    !! $\kappa_1$ quantum number
    Integer(t_int),Intent(in)    :: kap1
    !! $\kappa_2$ quantum number
    Integer(t_int),Intent(in)    :: kap2
    !! dummy array used for radial integration
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !! magnetic field times r $r*B(r)$
    Real(t_real),Intent(in)      :: btr(mesh% nr)
    !! wave functions of the basis states
    Real(t_real),Intent(in)      :: wf1(mesh% nr,4)
    Real(t_real),Intent(in)      :: wf2(mesh% nr,4)
    !!LOCAL
    Logical(t_log) :: exists
    Integer(t_int) :: ir
    Real(t_real)   :: h,alph,val,c2
    Real(t_real)   :: texp(1:mesh% nr)
    !!
    c2=dsc% c2
    If( dsc% gam_factor)Then
       texp(1:mesh% nr)=mesh% p_rmesh(1:mesh% nr)**(2*dsc% gamma)
    Else
       texp(1:mesh% nr)=1.0_t_real
    Endif
    !!
    h=0.0_t_real
    !! large component
    alph=c_mukk(dsc% mu,kap1,kap2,exists)
    If (exists) Then
       tmp(1)=0.0_t_real
       Do ir=2,mesh% nr
          tmp(ir)=wf1(ir,LARGE_COMP1)*wf2(ir,LARGE_COMP1)*btr(ir)&
               &/mesh% p_rmesh(ir)*mesh% p_jmesh(ir)*texp(ir)
       Enddo
       Call lag4int(mesh% nr,tmp,mesh% hx,val,lagint_out)
       h=h+val*alph
    Endif
    !! small component
    alph=c_mukk(dsc% mu,-kap1,-kap2,exists)
    If (exists) Then
       tmp(1)=0.0_t_real
       Do ir=2,mesh% nr
          tmp(ir)=wf1(ir,SMALL_COMP1)*wf2(ir,SMALL_COMP1)*btr(ir)/c2&
               &/mesh% p_rmesh(ir)*mesh% p_jmesh(ir)*texp(ir)
       Enddo
       Call lag4int(mesh% nr,tmp,mesh% hx,val,lagint_out)
       h=h-val*alph
    Endif
    !!
    magmat=h
    !!
    Return
    !!
  End Function magmat
  !!***********************************************************************
  !!***********************************************************************
  !! calculate 4 dimensional analogue of reciprocial vectors
  !! do not normalize as usual since used in context where the
  !! volume becomes zero
  !! v_l=\eps_{ijkl}v1_i v2_j a_k
  !!
#ifdef my_compile_unused
  Subroutine reciprocal4(v1,v2,v3a,va,v3b,vb)
    !!
    Implicit None  
    !!IO
    Real(t_real),Intent(in)      :: v1(4),v2(4),v3a(4),v3b(4)
    Real(t_real),Intent(out)     :: va(4),vb(4)
    !!LOCAL
    Real(t_real)   :: m1324,m1422,m1223,m1124,m1123,m1122
    !!
    m1324=v1(3)*v2(4)-v1(4)*v2(3)
    m1422=v1(4)*v2(2)-v1(2)*v2(4)
    m1223=v1(2)*v2(3)-v1(3)*v2(2)
    m1124=v1(1)*v2(4)-v1(4)*v2(1)
    m1123=v1(3)*v2(1)-v1(1)*v2(3)
    m1122=v1(1)*v2(2)-v1(2)*v2(1)
    !!
    va(1)=(             v3a(2)*m1324+v3a(3)*m1422+v3a(4)*m1223)
    va(2)=(-v3a(1)*m1324            +v3a(3)*m1124+v3a(4)*m1123)
    va(3)=(-v3a(1)*m1422-v3a(2)*m1124            +v3a(4)*m1122)
    va(4)=(-v3a(1)*m1223-v3a(2)*m1123-v3a(3)*m1122            )
    !!
    vb(1)=-(             v3b(2)*m1324+v3b(3)*m1422+v3b(4)*m1223)
    vb(2)=-(-v3b(1)*m1324            +v3b(3)*m1124+v3b(4)*m1123)
    vb(3)=-(-v3b(1)*m1422-v3b(2)*m1124            +v3b(4)*m1122)
    vb(4)=-(-v3b(1)*m1223-v3b(2)*m1123-v3b(3)*m1122            )
    !!
    Return
  End Subroutine reciprocal4
#endif
  !!***********************************************************************
  !! diagonalize a symmetric real 2x2 matrix 
  !! (symmetricity is forced by averaging)
  !! acending ordered eigenvalues on output 
  Subroutine diagno2(M,C,E)
    !!
    Implicit none  
    !!IO
    !! matrix
    Real(t_real),Intent(in)      :: M(2,2)
    !! eigenvectors 
    Real(t_real),Intent(out)     :: C(2,2)
    !! eigenvalues
    Real(t_real),Intent(out)     :: E(2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="diagno2"
    Logical(t_log) :: error
    Integer(t_int) :: l
    Real(t_real)   :: b,tr2,s,H(2,2)
    !!
    b=(M(1,2)+M(2,1))*0.5_t_real
    !!
    If(b==0.0_t_real)Then
       If(M(1,1)<=M(2,2))Then
          E(1)=M(1,1)
          E(2)=M(2,2)
          C(1,1)=1.0_t_real
          C(2,1)=0.0_t_real
          C(1,2)=0.0_t_real
          C(2,2)=1.0_t_real
       Else
          E(1)=M(2,2)
          E(2)=M(1,1)
          C(1,1)=0.0_t_real
          C(2,1)=1.0_t_real
          C(1,2)=1.0_t_real
          C(2,2)=0.0_t_real
       Endif
    Else 
       !! b/= 0
       tr2=(M(1,1)+M(2,2))*0.5_t_real
       !!
       s=(M(1,1)-M(2,2))**2*0.25_t_real+b**2
       !!
       E(1)=tr2-Sqrt(s)
       E(2)=tr2+Sqrt(s)
       Do l=1,2
          If((Abs(M(1,1)-E(l)) < Abs(M(2,2)-E(l))) )Then
             s=Sqrt((M(2,2)-E(l))**2+b**2)
             C(1,l)=(M(2,2)-E(l))/s
             C(2,l)=-b/s
          Else
             s=Sqrt((M(1,1)-E(l))**2+b**2)
             C(1,l)=-b/s
             C(2,l)=(M(1,1)-E(l))/s
          Endif
       Enddo
    Endif
    !!
    !!
    !!output
    !!
    error=.False.
    !! orthonormality test
    H=Matmul(Transpose(C),C)
    H(1,1)=H(1,1)-1.0_t_real
    H(2,2)=H(2,2)-1.0_t_real
    If(Any(Abs(H(:,:))>1.e-12_t_real))Then
       error=.True.
       Write(errbuf,"('''Eigenvectors not orthonormal?''')")
    Endif
    !!
    !!
    !!solution test
    s=Max(Abs(E(1)),Abs(E(2)))
    If(s==0)s=1.0_t_real
    H=Matmul(M,C)
    H(:,1)=H(:,1)-C(:,1)*E(1)
    H(:,2)=H(:,2)-C(:,2)*E(2)
    If(Any(Abs(H(:,:)/s)>1.e-12_t_real))Then
       error=.True.
       Write(errbuf,"('''Eigensystem not solved?''')")
    Endif
    !!
    If(error)Then
       !!
       Write(outfile,"('M=')")
       Do l=1,2
          Write(outfile,"(2(e20.8e3,3x))")M(l,1:2)
       Enddo
       !!
       Write(outfile,"('C=')")
       Do l=1,2
          Write(outfile,"(2(e20.8e3,3x))")C(l,1:2)
       Enddo
       !!
       Write(outfile,"('E=')")
       Write(outfile,"(1(e20.8e3,3x))")E(1:2)
       !!
       Write(outfile,"('MC-CE=')")
       Do l=1,2
          Write(outfile,"(2(e20.8e3,3x))") H(l,1:2)
       Enddo
       !!
       H=Matmul(Transpose(C),C)
       Write(outfile,"('C^T*C=')")
       Do l=1,2
          Write(outfile,"(2(e20.8e3,3x))") H(l,1:2)
       Enddo
       !!
       !!
       Call errmsg(nmodule,nsubfunc,E_fatalerror,msg=errbuf)
       !!
    Endif
    !!
    Return
    !!
  End Subroutine diagno2
  !!***********************************************************************
  !! Here the outward and inward solutions are matched in a way
  !! that the large components and one small component allways match
  !! perfect (within the numerical accuracy), while the match of the
  !! small components is as good as possible
  !!
  Subroutine  odematch(dodw,odeorder,pio,nr,nmax,nfit,wf,BIn,wf0,dif,prot)
    !!
    Implicit None  
    !!IO
    Logical(t_log),Intent(in) :: prot
    !! d(wf)/d(w) mode?
    Logical(t_log),Intent(in) :: dodw
    !! order of ODE
    Integer(t_int),Intent(in) :: odeorder
    !! the number of mesh points
    Integer(t_int),Intent(in) :: nr
    !! maximum mesh point where ODE is solved at
    Integer(t_int),Intent(in) :: nmax
    !! matching point
    Integer(t_int),Intent(in) :: nfit
    !! the values of the outward and inward solutions at the
    !! matching point (4 components, two partial waves, OUT and IN)
    !! must be a separate array since matching point values of outward
    !! solution are overwritten by inward solution
    Real(t_real),Intent(in)   :: pio(4,2,-1:1)
    !! the error function
    Real(t_real),Intent(out)  :: dif
    !! The wave function
    Real(t_real),Intent(out)  :: wf(nr,4,2)
    !! the ODE solution in IODE mode
    Real(t_real),Intent(in)   :: wf0(nr,4,2)
    !! starting values for inward shoot (for an IODE problem it must be here)
    Real(t_real),Intent(inout):: BIn(2)
    !!LOCAL
    Character(*),Parameter::nsubfunc="odematch"
    !!
    Logical(t_log) :: indef
    Integer(t_int) :: j,k,ipart
    Real(t_real)   :: vn(4,4),s,AL(4)
    Real(t_real)   :: M(3,3),b(3),vb(4),d
    !!
    !!
    If(prot)Then
       Do ipart=1,odeorder/2
          Do j=1,odeorder
             Write(outfile,1)nsubfunc,ipart,pio(j,ipart,OUT),ipart,pio(j,ipart,IN)
          Enddo
       Enddo
    Endif
    !!
    If(dodw)Then
       
       odeorderifdw: If(odeorder==2)Then
          ipart=1_t_int
          AL(3)=(pio(LARGE_COMP1,ipart,OUT)-pio(LARGE_COMP1,ipart,IN))&
               &/wf0(nfit,LARGE_COMP1,ipart)

          If(prot)Then
             Write(outfile,2,advance='no')nsubfunc
             Write(outfile,3)AL(3:3)
             Write(outfile,4,advance='no')nsubfunc
             Write(outfile,3) Sqrt(Sum(AL(3:3)**2))
          Endif
          
          wf(nfit:nmax,1_t_int:odeorder,ipart)=&
               &wf(nfit:nmax,1_t_int:odeorder,ipart)&
               &+wf0(nfit:nmax,1_t_int:odeorder,ipart)*AL(3)
          dif=pio(SMALL_COMP1,ipart,OUT)-pio(SMALL_COMP1,ipart,IN)&
               &-wf0(nfit,SMALL_COMP1,ipart)*AL(3)

          If(prot)Then
             Write(outfile,5,advance='no')nsubfunc
             Write(outfile,3)dif
          Endif

          
          BIn(ipart)=wf(nmax,SMALL_COMP1+(ipart-1)*2,ipart)
          

          Return
       Else
          !! odeorder==4
          Do j=1,4
             vn(j,1)=-(pio(j,2,OUT)-pio(j,1,OUT))
             vn(j,2)=(pio(j,2,IN )-pio(j,1,IN ))
             vn(j,3)=Sum(wf0(nfit,j,1:2))
             vb(j)=pio(j,1,OUT)-pio(j,1,IN)
          Enddo
          !! (la,mu,B)
          M(1,1:3)=vn(1,1:3)
          M(2:3,1:3)=vn(3:4,1:3)
          b(1)=vb(1)
          b(2:3)=vb(3:4)
          
          Call gaussj(M,3_t_int,3_t_int,b,1_t_int,1_t_int,indef)
          
          If(indef)Then
             Call errmsg(nmodule,nsubfunc,E_fatalerror&
                  &,msg="'Matrix singular case 1!?'")
          Else
             !!
             AL(1:3)=b(1:3)
             !!
             dif=(Sum(vn(2,1:3)*AL(1:3))-vb(2))
             !!
          Endif
          
          
          If(prot)Then
             !!
             Write(outfile,2,advance='no')nsubfunc
             Write(outfile,3)AL(1:3)
             Write(outfile,4,advance='no')nsubfunc
             Write(outfile,3) Sqrt(Sum(AL(1:3)**2))
             !!
             Do j=1,4
                Write(outfile,10)nsubfunc,j,Sum(vn(j,1:3)*AL(1:3))-vb(j)
             Enddo
             Write(outfile,5,advance='no')nsubfunc
             Write(outfile,3)dif
          Endif
          !!
          !!outwar: (1-la)psi1 +la psi2
          !!inward: (1-la)psi1 +la psi2 + D Psi0
          !!

          
          !! match-scale outward partial waves
          wf(1:nfit-1,1_t_int:odeorder,1)=&
               &wf(1:nfit-1,1_t_int:odeorder,1)*(1.0_t_real-AL(1))
          wf(1:nfit-1,1_t_int:odeorder,2)=&
               &wf(1:nfit-1,1_t_int:odeorder,2)*AL(1)

          !! need separate start values, to achieve AL(3)->0

          d=0.5_t_real
          BIn(1)=(&
               & (  wf(nmax,SMALL_COMP1,1) )*(1.0_t_real-AL(2))&
               &+( wf0(nmax,SMALL_COMP1,1) )*Al(3)&
               &)/d
          d=0.5_t_real
          BIn(2)=(&
               & (  wf(nmax,SMALL_COMP2,2) )*AL(2)&
               &+( wf0(nmax,SMALL_COMP2,2) )*Al(3)&
               &)/d


          !! match-scale inward partial waves
          wf(nfit:nr,1_t_int:odeorder,1)=&
               &(wf(nfit:nr,1_t_int:odeorder,1) )*(1.0_t_real-AL(2))&
               &+( wf0(nfit:nr,1_t_int:odeorder,1) )*Al(3)
          wf(nfit:nr,1_t_int:odeorder,2)=&
               &(wf(nfit:nr,1_t_int:odeorder,2) )*AL(2)&
               &+( wf0(nfit:nr,1_t_int:odeorder,2) )*Al(3)
          
          

          Return
       Endif odeorderifdw
       Return
    Endif !! dodw
    !!
    odeorderif: If(odeorder==2)Then
       !! The same algorithm as for odeorder=4, but hand made.
       !! The error function is NOT a quadratic form
       AL(1)=pio(LARGE_COMP1,1,IN)
       AL(2)=pio(LARGE_COMP1,1,OUT)
       s=Sqrt(AL(1)**2+AL(2)**2)
       If(s/=0.0_t_real) AL(1:2)=AL(1:2)/s
       !! choice of phase factor
       If(Al(1)<0.0_t_real)AL(1:2)=-AL(1:2)
          
       !! protect against zero scaling due to numerical roundoff in
       !! extreme cases, at convergence this may not happen
       If(AL(1)==0.0_t_real)Then
          AL(1)=1.e-30_t_real
       Endif
       If(AL(2)==0.0_t_real)Then
          AL(2)=1.e-30_t_real
       Endif

       If(prot)Then
          Write(outfile,2,advance='no')nsubfunc
          Write(outfile,3)AL(1:2)
          Write(outfile,4,advance='no')nsubfunc
          Write(outfile,3) Sqrt(Sum(AL(1:2)**2))
       Endif

       dif=pio(SMALL_COMP1,1,OUT)*AL(1)-pio(SMALL_COMP1,1,IN)*AL(2)
       If(prot)Then
          Write(outfile,5,advance='no')nsubfunc
          Write(outfile,3)dif
       Endif
       !! match-scale outward partial waves
       wf(1:nfit-1,1_t_int:odeorder,1)=&
            &wf(1:nfit-1,1_t_int:odeorder,1)*AL(1)
       !! match-scale inward partial waves
       wf(nfit:nr,1_t_int:odeorder,1)=&
            &wf(nfit:nr,1_t_int:odeorder,1)*AL(2)
    Else
       !!odeorder ==4

       !! make exact fit of large components and one small componenet
       !! and best fit of small components
       Do j=1,4 !! for all components
          vn(j,1)=+pio(j,1,OUT)
          vn(j,2)=+pio(j,2,OUT)
          vn(j,3)=-pio(j,1,IN)
          vn(j,4)=-pio(j,2,IN)
       Enddo
       !!
       !!
       If(prot)Then
          Do j=1,4
             Write(outfile,11)nsubfunc,j,vn(j,1:4)
          Enddo
       Endif
       !!
       !!
       Do k=2,4
          Do j=1,1
             M(j,k-1)=vn(j,k)
             b(j)=vn(j,1)
          Enddo
          Do j=3,4
             M(j-1,k-1)=vn(j,k)
             b(j-1)=vn(j,1)
          Enddo
       Enddo
       !!
       !!
       Call gaussj(M,3_t_int,3_t_int,b,1_t_int,1_t_int,indef)
       !!
       If(indef)Then
          Call errmsg(nmodule,nsubfunc,E_fatalerror&
               &,msg="'Matrix singular case 2!?'")
       Endif
       !!

       AL(2:4)=-b(1:3)
       !!
       AL(1)=1.0_t_real
       !!
       dif=(Sum(vn(2,:)*AL(:)))
       !!
       
       !!
       If(prot)Then
          !!
          Write(outfile,2,advance='no')nsubfunc
          Write(outfile,3)AL(1:4)
          Write(outfile,4,advance='no')nsubfunc
          Write(outfile,3) Sqrt(Sum(AL(1:4)**2))
          !!
          Do j=1,4
             Write(outfile,10)nsubfunc,j,Sum(vn(j,:)*AL(:))
          Enddo
          Write(outfile,5,advance='no')nsubfunc
          Write(outfile,3)dif
       Endif
       
       
       !!
       Do ipart=1,2
          !! match-scale outward partial waves
          wf(1:nfit-1,1_t_int:odeorder,ipart)=&
               &wf(1:nfit-1,1_t_int:odeorder,ipart)*AL(ipart)
          !! match-scale inward partial waves
          wf(nfit:nr,1_t_int:odeorder,ipart)=&
               &wf(nfit:nr,1_t_int:odeorder,ipart)*AL(2+ipart)
       Enddo
       !!
    Endif odeorderif
    !!
    Return
    !!
1   Format(a,': Psi_O',i1,'=',e20.8e3,'  PSI_I',i1,'=',e20.8e3)
2   Format(a,': A=')
3   Format(4(e20.8e3,3x))
4   Format(a,': |A|=')
5   Format(a,': dif=')
!!6   Format(a,': (',i1,'|v',i1,')=',e20.8e3)
!!7   Format(a,': S=')
!!8   Format(a,': ',4(e20.8e3,3x))
!!9   Format(a,': E=')
10  Format(a,': (',i1,'|A)=',e20.8e3)
11  Format(a,': v',i1,'=',4(e20.8e3,2x))


    !!
  End Subroutine odematch
  !!***********************************************************************
  Subroutine  analyse_wf(mesh,wf,tmp,kap,prot,dsc)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    Logical(t_log),Intent(in)    :: prot
    Integer(t_int),Intent(in)    :: kap
    Real(t_real),Intent(in)      :: wf(mesh% nr,4,2)
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !!LOCAL
    Integer(t_int) :: ir,ipart
    Real(t_real)   :: norm,c2
    Real(t_real)   :: texp(1:mesh% nr)
    !!
    If(.Not.prot) Return
    !!
    c2=dsc% c2
    If( dsc% gam_factor)Then
       texp(1:mesh% nr)=mesh% p_rmesh(1:mesh% nr)**(2*dsc% gamma)
    Else
       texp(1:mesh% nr)=1.0_t_real
    Endif
    !! J^2 expectation value
    !!
    Do ir=1,mesh% nr
       tmp(ir)=(&
            & Sum(wf(ir,LARGE_COMP1,1:2))**2_t_int    &
            &+Sum(wf(ir,SMALL_COMP1,1:2))**2_t_int/c2 &
            &)*mesh% p_jmesh(ir)*texp(ir)
    Enddo
    Call lag4int(mesh% nr,tmp,mesh% hx,norm,lagint_out)
    !!
    Print *,'j=',2*Abs(kap)-1,'/2:',norm
    !!
    !! partial wave contribs
    !!
    Do ipart=1,2
       Do ir=1,mesh% nr
          tmp(ir)=(&
               & wf(ir,LARGE_COMP1,ipart)**2_t_int    &
               &+wf(ir,SMALL_COMP1,ipart)**2_t_int/c2 &
               &)*mesh% p_jmesh(ir)*texp(ir)
       Enddo
       Call lag4int(mesh% nr,tmp,mesh% hx,norm,lagint_out)
       !!
       Print *,'j=',2*Abs(kap)-1,'/2, '&
            &,2*Abs(kap*(3-2*ipart)-(ipart-1))-1&
            &,'/2:',norm
    Enddo

    !!--------------------------------------------------------------------
    Do ir=1,mesh% nr
       tmp(ir)=(&
            & Sum(wf(ir,LARGE_COMP2,1:2))**2_t_int    &
            &+Sum(wf(ir,SMALL_COMP2,1:2))**2_t_int/c2 &
            &)*mesh% p_jmesh(ir)*texp(ir)
    Enddo
    Call lag4int(mesh% nr,tmp,mesh% hx,norm,lagint_out)
    !!
    Print *,'j=',2*Abs(-kap-1)-1,'/2:',norm
    !!
    !! partial wave contribs
    !!
    Do ipart=1,2
       Do ir=1,mesh% nr
          tmp(ir)=(&
               & wf(ir,LARGE_COMP2,ipart)**2_t_int    &
               &+wf(ir,SMALL_COMP2,ipart)**2_t_int/c2 &
               &)*mesh% p_jmesh(ir)*texp(ir)
       Enddo
       Call lag4int(mesh% nr,tmp,mesh% hx,norm,lagint_out)
       !!
       Print *,'j=',2*Abs(-kap-1)-1,'/2, '&
            &,2*Abs((kap)*(3-2*ipart)-(ipart-1))-1&
            &,'/2:',norm
    Enddo
    !!
    Return
    !!
  End Subroutine analyse_wf
  !!***********************************************************************
  !! here wf(:,:,i) are the two solutions i=kap,-kap-1
  !! (no longer partial waves)
  !! small components still scaled
  Subroutine  check_ortho_wf(mesh,wf,tmp,odeorder,dsc)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in)      :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    Integer(t_int),Intent(in)    :: odeorder
    !! speed of light**2
    Real(t_real),Intent(in)      :: wf(mesh% nr,4,2)
    Real(t_real),Intent(out)     :: tmp(mesh% nr)
    !!LOCAL
    Character(*),Parameter::nsubfunc="check_ortho_wf"
    Integer(t_int) :: ir
    Real(t_real)   :: norm,c2
    Real(t_real),Parameter   :: TINY=1.0e-8_t_real
    Real(t_real),Parameter   :: TOO_LARGE=1.0e-4_t_real
    Real(t_real)   :: texp(1:mesh% nr)
    !!
    If(odeorder==2) Return
    !!
    c2=dsc% c2
    !!
    If( dsc% gam_factor)Then
       texp(1:mesh% nr)=mesh% p_rmesh(1:mesh% nr)**(2*dsc% gamma)
    Else
       texp(1:mesh% nr)=1.0_t_real
    Endif
    !!
    Do ir=1,mesh% nr
       tmp(ir)=(&
            & (wf(ir,LARGE_COMP1,1)*wf(ir,LARGE_COMP2,2))       &
            &+(wf(ir,LARGE_COMP2,1)*wf(ir,LARGE_COMP1,2))       &
            &+(wf(ir,SMALL_COMP1,1)*wf(ir,SMALL_COMP2,2))/c2    &
            &+(wf(ir,SMALL_COMP2,1)*wf(ir,SMALL_COMP1,2))/c2    &
            &)*mesh% p_jmesh(ir)*texp(ir)
    Enddo
    Call lag4int(mesh% nr,tmp,mesh% hx,norm,lagint_out)
    !!
    !!Print *,'(kap|kapq)=',norm
    !!
    If (Abs(norm) > TINY) Then
       Write(errbuf,140) norm
       Call errmsg(nmodule,nsubfunc,E_warning,msg=errbuf)
    Endif
    If (Abs(norm)> TOO_LARGE) Then
       Write(errbuf,130)norm
       Call errmsg(nmodule,nsubfunc,E_error,msg=errbuf)
    Endif
    !!
    Return
    !!
130 Format('''non orthogonal solutions: <kap|kapq>=',e14.4e2,' != 0 ''')
140 Format(''' <kap|kapq>=',e14.4e2,' != 0 ''')
    !!
  End Subroutine check_ortho_wf
  !!***********************************************************************
  Subroutine erg_estimate(mesh,vtr,dsc,Emin,Emax)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(in) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    Real(t_real),Intent(in)      :: vtr(:)
    Real(t_real),Intent(out)     :: Emin,Emax
    !!LOCAL
    Integer(t_int) :: la,nmain,ir
    Real(t_real)   :: ZZ,ll2,d
    !!
    la=dsc% la
    nmain=dsc% nmain
    ZZ=dsc% ZZ
    !!
    ll2=la*(la+1)*0.5_t_real
    !!
    If(la==0) Then
       Emin=-(ZZ/nmain)**2
       If(ZZ < 4.5_t_real)Then
          !! make sure the higher states in Q-potential are possible
          Emin=-(ZZ/nmain)**2*10
          If(ZZ<1.0e-6_t_real)Then
             Emin=-(1.0_t_real/nmain)**2*10
          Endif
       Endif
       !!If(Abs(ZZ)<1.e-4_t_real)Emin=-(10.0_t_real**2_t_int/(dsc% nmain**2_t_int))
    Else
       ir=2
       Emin=vtr(ir)/mesh% p_rmesh(ir)&
            &+ll2/mesh% p_rmesh(ir)**2&
            &+dsc% fx0*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0&
            &+dsc% fx2*(mesh% p_rmesh(ir)/dsc% rx2)**dsc% exn2
       !!
       Do ir=3_t_int,mesh% nr
          d=vtr(ir)/mesh% p_rmesh(ir)&
               &+ll2/mesh% p_rmesh(ir)**2&
               &+dsc% fx0*(mesh% p_rmesh(ir)/dsc% rx0)**dsc% exn0&
               &+dsc% fx2*(mesh% p_rmesh(ir)/dsc% rx2)**dsc% exn2
          If(d<=Emin ) Then
             Emin=d
          Endif
       Enddo
    Endif
    !!
    If( ( Abs(dsc% fx0) > 1.0e-6_t_real ) .Or. &
         &( Abs(dsc% fx2) > 1.0e-6_t_real ) )Then
       emax=10.0_t_real
    Else
       emax=0.0_t_real
    Endif
    !!
    Return
    !!
  End Subroutine erg_estimate
  !!***********************************************************************
  Subroutine prep_pot_expansion(mesh,vtr,btr,dsc)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(inout) :: dsc
    Type(T_Mesh),Intent(in)      :: mesh
    
    Real(t_real),Intent(in)      :: vtr(mesh% nr),btr(mesh% nr)
    !!LOCAL
    Character(*),Parameter:: nsubfunc="prep_pot_expansion"
    Real(t_real)   :: ZZ,r0,rr2,rr3,xx2,xx3,v0,v1,b0,b1
    !!
    ZZ=dsc% ZZ
    r0=nuc_max_rad(dsc% nuc_type,ZZ)
    !!
    If(.Not. mesh% with_zero)Then
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'need a mesh includign the zero!'")
    Endif
    If(Abs(ZZ+vtr(1))>1.0e-12_t_real)Then
       Write(outfile,"('ZZ=',f30.15,' vtr(1)=',f30.15)") ZZ,-vtr(1)
       Call errmsg(nmodule,nsubfunc,E_fatalerror&
            &,msg="'ZZ /= vtr(1)!'")
    Endif
    !!
    rr2=mesh% p_rmesh(2)
    rr3=mesh% p_rmesh(3)
    xx2=rr2/r0
    xx3=rr3/r0
    !!
    !!       vtr(r)=v(r)*r=(-ZZ+r*p(r)),  p(r)=v0+v1*r
    !!
    v0= ( -(vtr(3)-vtr(2))/(xx3-xx2) +(vtr(2)+ZZ)/xx2 &
         &+(vtr(3)+ZZ)/xx3)/ZZ
    !!
    v1=((-ZZ*(xx3-xx2)-vtr(2)*xx3+vtr(3)*xx2) &
         &/(xx2*xx3*(xx3-xx2)))/ZZ
    
    If(Abs(ZZ)<1.0e-6_t_real)Then
       v0=0.0_t_real
       v1=0.0_t_real
    Endif
    !Print *,'V0=',v0
    !Print *,'V1=',v1
    
    !!       btr(r)=b(r)*r= b0*r+b1*r^2
    b0=0.0_t_real
    b1=0.0_t_real
    If(dsc% relmag)Then
       b0= (  -(btr(3)-btr(2))/(xx3-xx2)+(btr(2))/xx2&
            &+(btr(3))/xx3)/ZZ
       !!
       b1=((-btr(2)*xx3+btr(3)*xx2)/(xx2*xx3*(xx3-xx2)))/ZZ
    Endif
    
    !Print *,'b0=',b0
    !Print *,'b1=',b1
    !!
    dsc% Nv=8
    Call palloc('dsc% p_Vcoeff',dsc% p_Vcoeff,0_t_int,dsc% Nv,nmodule,nsubfunc)
    Call palloc('dsc% p_Bcoeff',dsc% p_Bcoeff,0_t_int,dsc% Nv,nmodule,nsubfunc)

    Call nuc_pot_coefs(dsc% nuc_type,dsc% p_Vcoeff,dsc% Nv)
    !!
    dsc% p_Bcoeff(:)=0.0_t_real
    !!
    dsc% p_Vcoeff(1)=dsc% p_Vcoeff(1)-v0
    dsc% p_Vcoeff(2)=dsc% p_Vcoeff(2)-v1
    dsc% p_Bcoeff(1)=dsc% p_Bcoeff(1)-b0
    dsc% p_Bcoeff(2)=dsc% p_Bcoeff(2)-b1
    !!
    Return
    !!
  End Subroutine prep_pot_expansion
  !!***********************************************************************
  !! initialize the diracsolver data 
  Subroutine  init_diracsolver(ds,nr)
    !!
    Implicit None  
    !!IO
    Integer(t_int),Intent(in)       :: nr
    Type(T_DIRACSOLVER),Intent(out) :: ds
    !!LOCAL
    Character(*),Parameter:: nsubfunc="init_diracsolver"
    !!
    ds% kap=0
    ds% nmain=-1
    ds% nmax=0
    ds% nfit=0
    !!
    Nullify(ds% p_wf)
    !!
    Call palloc('ds % p_wf',ds % p_wf&
         &,1_t_int,nr&
         &,1_t_int,2_t_int&
         &,1_t_int,2_t_int&
         &,nmodule,nsubfunc)
    !!
    Return
    !!
  End Subroutine init_diracsolver
  !!***********************************************************************
  !! initialize the diracsolver data 
  Subroutine  clean_diracsolver(ds)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLVER),Intent(inout) :: ds
    !!LOCAL
    !Character(*),Parameter:: nsubfunc="clean_diracsolver"
    !!
    ds% kap=0
    ds% nmain=-1
    ds% nmax=0
    ds% nfit=0
    !!
    Call pfree(ds% p_wf)
    !!
    Nullify(ds% p_wf)
    !!
    Return
    !!
  End Subroutine clean_diracsolver
  !!***********************************************************************
  Subroutine  init_t_diracsolv_control(dsc)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(out) :: dsc
    !!LOCAL
    !!
    dsc% gam_factor=.False.
    dsc% gamma=0.0_t_real
    Nullify(dsc% p_vc)
    Nullify(dsc% p_vtrcorrected)
    Nullify(dsc% p_Vcoeff)
    Nullify(dsc% p_Bcoeff)
    !!
    Return
    !!
  End Subroutine init_t_diracsolv_control
  !!***********************************************************************
  Subroutine  clean_t_diracsolv_control(dsc)
    !!
    Implicit None  
    !!IO
    Type(T_DIRACSOLV_CONTROL),Intent(inout) :: dsc
    !!LOCAL
    !!
    Call pfree(dsc% p_vc)
    Call pfree(dsc% p_vtrcorrected)
    Call pfree(dsc% p_Vcoeff)
    Call pfree(dsc% p_Bcoeff)
    !!
    Return
    !!
  End Subroutine clean_t_diracsolv_control
  !!***********************************************************************

End Module diracsolver
